/*!
*  filename: ej.mobile.all.js
*  version : 12.3
*  Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/


window.ej = window.Syncfusion = window.Syncfusion || {};


(function ($, ej, undefined) {
    'use strict';

    ej.version = "13.3.0.8";

    ej.consts = {
        NamespaceJoin: '-'
    };
    ej.TextAlign = {
        Center: 'center',
        Justify: 'justify',
        Left: 'left',
        Right: 'right'
    };
    ej.Orientation = { Horizontal: "horizontal", Vertical: "vertical" };

    ej.serverTimezoneOffset = 0;

    ej.persistStateVersion = null;

    if (!Object.prototype.hasOwnProperty) {
        Object.prototype.hasOwnProperty = function (obj, prop) {
            return obj[prop] !== undefined;
        };
    }

    //to support toISOString() in IE8
    if (!Date.prototype.toISOString) {
        (function () {
            function pad(number) {
                var r = String(number);
                if (r.length === 1) {
                    r = '0' + r;
                }
                return r;
            }
            Date.prototype.toISOString = function () {
                return this.getUTCFullYear()
                    + '-' + pad(this.getUTCMonth() + 1)
                    + '-' + pad(this.getUTCDate())
                    + 'T' + pad(this.getUTCHours())
                    + ':' + pad(this.getUTCMinutes())
                    + ':' + pad(this.getUTCSeconds())
                    + '.' + String((this.getUTCMilliseconds() / 1000).toFixed(3)).slice(2, 5)
                    + 'Z';
            };
        }());
    }

    String.format = function () {
        var source = arguments[0];
        for (var i = 0; i < arguments.length - 1; i++)
            source = source.replace(new RegExp("\\{" + i + "\\}", "gm"), arguments[i + 1]);

        source = source.replace(/\{[0-9]\}/g, "");
        return source;
    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };
    // Function to create new class
    ej.defineClass = function (className, constructor, proto, replace) {
        /// <summary>Creates the javascript class with given namespace & class name & constructor etc</summary>
        /// <param name="className" type="String">class name prefixed with namespace</param>
        /// <param name="constructor" type="Function">constructor function</param>
        /// <param name="proto" type="Object">prototype for the class</param>
        /// <param name="replace" type="Boolean">[Optional]Replace existing class if exists</param>
        /// <returns type="Function">returns the class function</returns>
        if (!className || !proto) return undefined;

        var parts = className.split(".");

        // Object creation
        var obj = window, i = 0;
        for (; i < parts.length - 1; i++) {

            if (ej.isNullOrUndefined(obj[parts[i]]))
                obj[parts[i]] = {};

            obj = obj[parts[i]];
        }

        if (replace || ej.isNullOrUndefined(obj[parts[i]])) {

            //constructor
            constructor = typeof constructor === "function" ? constructor : function () {
            };

            obj[parts[i]] = constructor;

            // prototype
            obj[parts[i]].prototype = proto;
        }

        return obj[parts[i]];
    };

    ej.util = {
        getNameSpace: function (className) {
            /// <summary>Internal function, this will create namespace for plugins using class name</summary>
            /// <param name="className" type="String"></param>
            /// <returns type="String"></returns>
            var splits = className.toLowerCase().split(".");
            splits[0] === "ej" && (splits[0] = "e");

            return splits.join(ej.consts.NamespaceJoin);
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (ej.util.isNullOrUndefined(value)) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i, t, length = splits.length;

            for (i = 0; i < length; i++) {
                t = splits[i];
                if (i + 1 == length)
                    from[t] = value;
                else if (ej.isNullOrUndefined(from[t]))
                    from[t] = {};

                from = from[t];
            }

            return start;
        },

        isNullOrUndefined: function (value) {
            /// <summary>Util to check null or undefined</summary>
            /// <param name="value" type="Object"></param>
            /// <returns type="Boolean"></returns>
            return value === undefined || value === null;
        },
        print: function (element, printWin) {
            var $div = ej.buildTag('div')
            var elementClone = element.clone();
            $div.append(elementClone);
            if (!printWin)
                var printWin = window.open('', 'print', "height=452,width=1024,tabbar=no");
            printWin.document.write('<!DOCTYPE html>');
            var links = $('head').find('link').add("style");
            if (ej.browserInfo().name === "msie") {
                var a = ""
                links.each(function (index, obj) {
                    if (obj.tagName == "LINK")
                        $(obj).attr('href', obj.href);
                    a += obj.outerHTML;
                });
                printWin.document.write('<html><head></head><body>' + a + $div[0].innerHTML + '</body></html>');
            }
            else {
                var a = ""
                printWin.document.write('<html><head>')
                links.each(function (index, obj) {
                    if (obj.tagName == "LINK")
                        $(obj).attr('href', obj.href);
                    a += obj.outerHTML;
                });
                printWin.document.writeln(a + '</head><body>')
                printWin.document.writeln($div[0].innerHTML + '</body></html>')
            }
            printWin.document.close();
            printWin.focus();
            setTimeout(function () {
                if (!ej.isNullOrUndefined(printWin.window)) {
                    printWin.print();
                    setTimeout(function () { printWin.close() }, 1000);
                }
            }, 1000);
        },
        ieClearRemover: function (element) {
            var searchBoxHeight = $(element).height();
            element.style.paddingTop = parseFloat(searchBoxHeight / 2) + "px";
            element.style.paddingBottom = parseFloat(searchBoxHeight / 2) + "px";
            element.style.height = "1px";
            element.style.lineHeight = "1px";
        },
        //To send ajax request
        sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.successHandler,
                error: ajaxOptions.errorHandler,
                beforeSend: ajaxOptions.beforeSendHandler,
                complete: ajaxOptions.completeHandler
            });
        },

        buildTag: function (tag, innerHtml, styles, attrs) {
            /// <summary>Helper to build jQuery element</summary>
            /// <param name="tag" type="String">tagName#id.cssClass</param>
            /// <param name="innerHtml" type="String"></param>
            /// <param name="styles" type="Object">A set of key/value pairs that configure styles</param>
            /// <param name="attrs" type="Object">A set of key/value pairs that configure attributes</param>
            /// <returns type="jQuery"></returns>
            var tagName = /^[a-z]*[0-9a-z]+/ig.exec(tag)[0];

            var id = /#([_a-z]+[-_0-9a-z]+)/ig.exec(tag);
            id = id ? id[id.length - 1] : undefined;

            var className = /\.([a-z]+[-_0-9a-z ]+)/ig.exec(tag);
            className = className ? className[className.length - 1] : undefined;

            return $(document.createElement(tagName))
                .attr(id ? { "id": id } : {})
                .addClass(className || "")
                .css(styles || {})
                .attr(attrs || {})
                .html(innerHtml || "");
        },
        _preventDefaultException: function (el, exceptions) {
            if (el) {
                for (var i in exceptions) {
                    if (exceptions[i].test(el[i])) {
                        return true;
                    }
                }
            }

            return false;
        },

        //Gets the maximum z-index in the document
        getMaxZindex: function () {
            var maxZ = 1;
            maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                if ($(e).css('position') == 'absolute' || $(e).css('position') == 'fixed')
                    return parseInt($(e).css('z-index')) || 1;
            })
            );
            if (maxZ == undefined || maxZ == null)
                maxZ = 1;
            return maxZ;
        },

        //To prevent default actions for the element
        blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },

        //To get dimensions of the element when its hidden
        getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            if ($hidden) {
                var prop = { visibility: 'hidden', display: 'block' };
                var tmp = [];
                $hidden.each(function () {
                    var temp = {}, name;
                    for (name in prop) {
                        temp[name] = this.style[name];
                        this.style[name] = prop[name];
                    }
                    tmp.push(temp);
                });
                value = /(outer)/g.test(method) ?
                $(element)[method](true) :
               $(element)[method]();

                $hidden.each(function (i) {
                    var temp = tmp[i], name;
                    for (name in prop) {
                        this.style[name] = temp[name];
                    }
                });
            }
            return value;
        },
        //Get triggers when transition End
        transitionEndEvent: function () {
            var transitionEnd = {
                '': 'transitionend',
                'webkit': 'webkitTransitionEnd',
                'Moz': 'transitionend',
                'O': 'otransitionend',
                'ms': 'MSTransitionEnd'
            };

            return transitionEnd[ej.userAgent()];
        },
        //Get triggers when transition End
        animationEndEvent: function () {
            var animationEnd = {
                '': 'animationend',
                'webkit': 'webkitAnimationEnd',
                'Moz': 'animationend',
                'O': 'webkitAnimationEnd',
                'ms': 'animationend'
            };

            return animationEnd[ej.userAgent()];
        },
        //To return the start event to bind for element
        startEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchstart" : "mousedown";
        },
        //To return end event to bind for element
        endEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchend" : "mouseup"
        },
        //To return move event to bind for element
        moveEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? ($.support.hasPointer && !ej.isMobile()) ? "ejtouchmove" : "touchmove" : "mousemove";
        },
        //To return cancel event to bind for element
        cancelEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchcancel" : "mousecancel";
        },
        //To return tap event to bind for element
        tapEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "tap" : "click";
        },
        //To return tap hold event to bind for element
        tapHoldEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "taphold" : "click";
        },
        //To check whether its Device
        isDevice: function () {
            if (ej.getBooleanVal($('head'), 'data-ej-forceset', false))
                return ej.getBooleanVal($('head'), 'data-ej-device', this._device());
            else
                return this._device();
        },
        //To check whether its portrait or landscape mode
        isPortrait: function () {
            var elem = document.documentElement;
            return (elem) && ((elem.clientWidth / elem.clientHeight) < 1.1);
        },
        //To check whether its in lower resolution
        isLowerResolution: function () {
            return ((window.innerWidth <= 640 && ej.isPortrait() && ej.isDevice()) || (window.innerWidth <= 800 && !ej.isDevice()) || (window.innerWidth <= 800 && !ej.isPortrait() && ej.isWindows() && ej.isDevice()) || ej.isMobile());
        },
        //To check whether its iOS web view
        isIOSWebView: function () {
            return (/(iPhone|iPod|iPad).*AppleWebKit(?!.*Safari)/i.test(navigator.userAgent));
        },
        //To check whether its Android web view
        isAndroidWebView: function () {
            return (!(typeof (Android) === "undefined"));
        },
        //To check whether its windows web view
        isWindowsWebView: function () {
            return location.href.indexOf("x-wmapp") != -1;
        },
        _device: function () {
            return (/Android|BlackBerry|iPhone|iPad|iPod|IEMobile|kindle|windows\sce|palm|smartphone|iemobile|mobile|pad|xoom|sch-i800|playbook/i.test(navigator.userAgent.toLowerCase()));
        },
        //To check whether its Mobile
        isMobile: function () {
            return ((/iphone|ipod|android|blackberry|opera|mini|windows\sce|palm|smartphone|iemobile/i.test(navigator.userAgent.toLowerCase()) && /mobile/i.test(navigator.userAgent.toLowerCase()))) || (ej.getBooleanVal($('head'), 'data-ej-mobile', false) === true);
        },
        //To check whether its Tablet
        isTablet: function () {
            return (/ipad|xoom|sch-i800|playbook|tablet|kindle/i.test(navigator.userAgent.toLowerCase())) || (ej.getBooleanVal($('head'), 'data-ej-tablet', false) === true) || (!ej.isMobile() && ej.isDevice());
        },
        //To check whether its Touch Device
        isTouchDevice: function () {
            return (('ontouchstart' in window || (window.navigator.msPointerEnabled && ej.isMobile())) && this.isDevice());
        },
        //To get the outerHTML string for object
        getClearString: function (string) {
            return $.trim(string.replace(/\s+/g, " ").replace(/(\r\n|\n|\r)/gm, "").replace(new RegExp("\>[\n\t ]+\<", "g"), "><"));
        },
        //Get the attribute value with boolean type of element
        getBooleanVal: function (ele, val, option) {
            /// <summary>Util to get the property from data attributes</summary>
            /// <param name="ele" type="Object"></param>
            /// <param name="val" type="String"></param>
            /// <param name="option" type="GenericType"></param>
            /// <returns type="GenericType"></returns>
            var value = $(ele).attr(val);
            if (value != null)
                return value.toLowerCase() == "true";
            else
                return option;
        },
        //Gets the Skew class based on the element current position
        _getSkewClass: function (item, pageX, pageY) {
            var itemwidth = item.width();
            var itemheight = item.height();
            var leftOffset = item.offset().left;
            var rightOffset = item.offset().left + itemwidth;
            var topOffset = item.offset().top;
            var bottomOffset = item.offset().top + itemheight;
            var widthoffset = itemwidth * 0.3;
            var heightoffset = itemheight * 0.3;
            if (pageX < leftOffset + widthoffset && pageY < topOffset + heightoffset)
                return "e-m-skew-topleft";
            if (pageX > rightOffset - widthoffset && pageY < topOffset + heightoffset)
                return "e-m-skew-topright";
            if (pageX > rightOffset - widthoffset && pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottomright";
            if (pageX < leftOffset + widthoffset && pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottomleft";
            if (pageX > leftOffset + widthoffset && pageY < topOffset + heightoffset && pageX < rightOffset - widthoffset)
                return "e-m-skew-top";
            if (pageX < leftOffset + widthoffset)
                return "e-m-skew-left";
            if (pageX > rightOffset - widthoffset)
                return "e-m-skew-right";
            if (pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottom";
            return "e-m-skew-center";
        },
        //Removes the added Skew class on the element
        _removeSkewClass: function (element) {
            $(element).removeClass("e-m-skew-top e-m-skew-bottom e-m-skew-left e-m-skew-right e-m-skew-topleft e-m-skew-topright e-m-skew-bottomleft e-m-skew-bottomright e-m-skew-center e-skew-top e-skew-bottom e-skew-left e-skew-right e-skew-topleft e-skew-topright e-skew-bottomleft e-skew-bottomright e-skew-center");
        },
        //Object.keys  method to support all the browser including IE8.
        _getObjectKeys: function (obj) {
            var i, keys = [];
            obj = Object.prototype.toString.call(obj) === Object.prototype.toString() ? obj : {};
            if (!Object.keys) {
                for (i in obj) {
                    if (obj.hasOwnProperty(i))
                        keys.push(i);
                }
                return keys;
            }
            if (Object.keys)
                return Object.keys(obj);
        },
        _touchStartPoints: function (evt, object) {
            if (evt) {
                var point = evt.touches ? evt.touches[0] : evt;
                object._distX = 0;
                object._distY = 0;
                object._moved = false;
                object._pointX = point.pageX;
                object._pointY = point.pageY;
            }
        },
        _isTouchMoved: function (evt, object) {
            if (evt) {
                var point = evt.touches ? evt.touches[0] : evt,
                deltaX = point.pageX - object._pointX,
                deltaY = point.pageY - object._pointY,
                timestamp = Date.now(),
                newX, newY,
                absDistX, absDistY;
                object._pointX = point.pageX;
                object._pointY = point.pageY;
                object._distX += deltaX;
                object._distY += deltaY;
                absDistX = Math.abs(object._distX);
                absDistY = Math.abs(object._distY);
                return !(absDistX < 5 && absDistY < 5);
            }
        },
        //To bind events for element
        listenEvents: function (selectors, eventTypes, handlers, remove, pluginObj, disableMouse) {
            for (var i = 0; i < selectors.length; i++) {
                ej.listenTouchEvent(selectors[i], eventTypes[i], handlers[i], remove, pluginObj, disableMouse);
            }
        },
        //To bind touch events for element
        listenTouchEvent: function (selector, eventType, handler, remove, pluginObj, disableMouse) {
            var event = remove ? "removeEventListener" : "addEventListener";
            var jqueryEvent = remove ? "off" : "on";
            var elements = $(selector);
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                switch (eventType) {
                    case "touchstart":
                        ej._bindEvent(element, event, eventType, handler, "mousedown", "MSPointerDown", "pointerdown", disableMouse);
                        break;
                    case "touchmove":
                        ej._bindEvent(element, event, eventType, handler, "mousemove", "MSPointerMove", "pointermove", disableMouse);
                        break;
                    case "touchend":
                        ej._bindEvent(element, event, eventType, handler, "mouseup", "MSPointerUp", "pointerup", disableMouse);
                        break;
                    case "touchcancel":
                        ej._bindEvent(element, event, eventType, handler, "mousecancel", "MSPointerCancel", "pointercancel", disableMouse);
                        break;
                    case "tap": case "taphold": case "ejtouchmove": case "click":
                        $(element)[jqueryEvent](eventType, handler);
                        break;
                    default:
                        if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)
                            pluginObj["_on"]($(element), eventType, handler);
                        else
                            element[event](eventType, handler, true);
                        break;
                }
            }
        },
        //To bind events for element
        _bindEvent: function (element, event, eventType, handler, mouseEvent, pointerEvent, ie11pointerEvent, disableMouse) {
            if ($.support.hasPointer)
                element[event](window.navigator.pointerEnabled ? ie11pointerEvent : pointerEvent, handler, true);
            else
                element[event](eventType, handler, true);
        },
        _browser: function () {
            return (/webkit/i).test(navigator.appVersion) ? 'webkit' : (/firefox/i).test(navigator.userAgent) ? 'Moz' : (/trident/i).test(navigator.userAgent) ? 'ms' : 'opera' in window ? 'O' : '';
        },
        styles: document.createElement('div').style,
        /**
       * To get the userAgent Name     
       * @example             
       * &lt;script&gt;
       *       ej.userAgent();//return user agent name
       * &lt;/script&gt         
       * @memberof AppView
       * @instance
       */
        userAgent: function () {
            var agents = 'webkitT,t,MozT,msT,OT'.split(','),
            t,
            i = 0,
            l = agents.length;

            for (; i < l; i++) {
                t = agents[i] + 'ransform';
                if (t in ej.styles) {
                    return agents[i].substr(0, agents[i].length - 1);
                }
            }

            return false;
        },
        addPrefix: function (style) {
            if (ej.userAgent() === '') return style;

            style = style.charAt(0).toUpperCase() + style.substr(1);
            return ej.userAgent() + style;
        },
        //To Prevent Default Exception

        //To destroy the mobile widgets
        destroyWidgets: function (element) {
            var dataEl = $(element).find("[data-role *= ejm]");
            dataEl.each(function (index, element) {
                var $element = $(element);
                var plugin = $element.data("ejWidgets");
                if (plugin)
                    $element[plugin]("destroy");
            });
        },
        //Get the attribute value of element
        getAttrVal: function (ele, val, option) {
            /// <summary>Util to get the property from data attributes</summary>
            /// <param name="ele" type="Object"></param>
            /// <param name="val" type="String"></param>
            /// <param name="option" type="GenericType"></param>
            /// <returns type="GenericType"></returns>
            var value = $(ele).attr(val);
            if (value != null)
                return value;
            else
                return option;
        },

        // Get the offset value of element
        getOffset: function (ele) {
            var pos = {};
            var offsetObj = ele.offset() || { left: 0, top: 0 };
            $.extend(true, pos, offsetObj);
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },

        // Z-index calculation for the element
        getZindexPartial: function (element, popupEle) {
            if (!ej.isNullOrUndefined(element) && element.length > 0) {
                var parents = element.parents(), bodyEle;
                bodyEle = $('body').children();
                if (!ej.isNullOrUndefined(element) && element.length > 0)
                    bodyEle.splice(bodyEle.index(popupEle), 1);
                $(bodyEle).each(function (i, ele) { parents.push(ele); });

                var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                    if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
                }));
                if (!maxZ || maxZ < 10000) maxZ = 10000;
                else maxZ += 1;
                return maxZ;
            }
        },

        isValidAttr: function (element, attribute) {
            var element = $(element)[0];
            if (typeof element[attribute] != "undefined")
                return true;
            else {
                var _isValid = false;
                $.each(element, function (key) {
                    if (key.toLowerCase() == attribute.toLowerCase()) {
                        _isValid = true;
                        return false;
                    }
                });
            }
            return _isValid;
        }

    };

    $.extend(ej, ej.util);

    // base class for all ej widgets. It will automatically inhertied
    ej.widgetBase = {
        droppables: { 'default': [] },
        resizables: { 'default': [] },

        _renderEjTemplate: function (selector, data, index, prop) {
            var type = null;
            if (typeof selector === "object" || selector.startsWith("#") || selector.startsWith("."))
                type = $(selector).attr("type");
            if (type) {
                type = type.toLowerCase();
                if (ej.template[type])
                    return ej.template[type](this, selector, data, index, prop);
            }
            return ej.template.render(this, selector, data, index, prop);
        },

        destroy: function () {

            if (this._trigger("destroy"))
                return;

            if (this.model.enablePersistence) {
                this.persistState();
                $(window).off("unload", this._persistHandler);
            }

            try {
                this._destroy();
            } catch (e) { }

            var arr = this.element.data("ejWidgets") || [];
            for (var i = 0; i < arr.length; i++) {
                if (arr[i] == this.pluginName) {
                    arr.splice(i, 1);
                }
            }
            if (!arr.length)
                this.element.removeData("ejWidgets");

            while (this._events) {
                var item = this._events.pop(), args = [];

                if (!item)
                    break;

                for (var i = 0; i < item[1].length; i++)
                    if (!$.isPlainObject(item[1][i]))
                        args.push(item[1][i]);

                $.fn.off.apply(item[0], args);
            }

            this._events = null;

            this.element
                .removeClass(ej.util.getNameSpace(this.sfType))
                .removeClass("e-js")
                .removeData(this.pluginName);

            this.element = null;
            this.model = null;
        },

        _on: function (element) {
            if (!this._events)
                this._events = [];
            var args = [].splice.call(arguments, 1, arguments.length - 1);

            var handler = {}, i = args.length;
            while (handler && typeof handler !== "function") {
                handler = args[--i];
            }

            args[i] = ej.proxy(args[i], this);

            this._events.push([element, args, handler, args[i]]);

            $.fn.on.apply(element, args);

            return this;
        },

        _off: function (element, eventName, selector, handlerObject) {
            var e = this._events, temp;
            if (!e || !e.length)
                return this;
            if (typeof selector == "function") {
                temp = handlerObject;
                handlerObject = selector;
                selector = temp;
            }
            var t = (eventName.match(/\S+/g) || [""]);
            for (var i = 0; i < e.length; i++) {
                var arg = e[i],
                r = arg[0].length && (!handlerObject || arg[2] === handlerObject) && (arg[1][0] === eventName || t[0]) && (!selector || arg[1][1] === selector) && $.inArray(element[0], arg[0]) > -1;
                if (r) {
                    $.fn.off.apply(element, handlerObject ? [eventName, selector, arg[3]] : [eventName, selector]);
                    e.splice(i, 1);
                    break;
                }
            }

            return this;
        },

        // Client side events wire-up / trigger helper.
        _trigger: function (eventName, eventProp) {
            var fn = null, returnValue, args, clientProp = {};
            $.extend(clientProp, eventProp)

            if (eventName in this.model)
                fn = this.model[eventName];

            if (fn) {
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }

                if ($.isFunction(fn)) {

                    args = ej.event(eventName, this.model, eventProp);

                    var scopeFn = this.model["_applyScope"];

                    returnValue = fn.call(this, args);

                    scopeFn && scopeFn.call();

                    // sending changes back - deep copy option should not be enabled for this $.extend 
                    if (eventProp) $.extend(eventProp, args);

                    if (args.cancel || !ej.isNullOrUndefined(returnValue))
                        return returnValue === false || args.cancel;
                }
            }

            var isPropDefined = Boolean(eventProp);
            eventProp = eventProp || {};
            eventProp.originalEventType = eventName;
            eventProp.type = this.pluginName + eventName;

            args = $.Event(eventProp.type, ej.event(eventProp.type, this.model, eventProp));

            this.element && this.element.trigger(args);

            // sending changes back - deep copy option should not be enabled for this $.extend 
            if (isPropDefined) $.extend(eventProp, args);

            if (ej.isOnWebForms && args.cancel == false && this.model.serverEvents && this.model.serverEvents.length)
                ej.raiseWebFormsServerEvents(eventName, eventProp, clientProp);

            return args.cancel;
        },

        setModel: function (options, forceSet) {
            // check for whether to apply values are not. if _setModel function is defined in child,
            //  this will call that function and validate it using return value

            if (this._trigger("modelChange", { "changes": options }))
                return;

            for (var prop in options) {
                if (!forceSet) {
                    if (this.model[prop] === options[prop]) {
                        delete options[prop];
                        continue;
                    }
                    if ($.isPlainObject(options[prop])) {
                        iterateAndRemoveProps(this.model[prop], options[prop]);
                        if ($.isEmptyObject(options[prop])) {
                            delete options[prop];
                            continue;
                        }
                    }
                }

                if (this.dataTypes) {
                    var returnValue = this._isValidModelValue(prop, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + prop + " - " + returnValue;
                }
                if (this.model.notifyOnEachPropertyChanges && this.model[prop] !== options[prop]) {
                    var arg = {
                        oldValue: this.model[prop],
                        newValue: options[prop]
                    };

                    options[prop] = this._trigger(prop + "Change", arg) ? this.model[prop] : arg.newValue;
                }
            }
            if ($.isEmptyObject(options))
                return;

            if (this._setFirst) {
                var ds = options.dataSource;
                if (ds) delete options.dataSource;

                $.extend(true, this.model, options);
                if (ds) {
                    this.model.dataSource = (ds instanceof Array) ? ds.slice() : ds;
                    options["dataSource"] = this.model.dataSource;
                }
                !this._setModel || this._setModel(options);

            } else if (!this._setModel || this._setModel(options) !== false) {
                $.extend(true, this.model, options);
            }
            if ("enablePersistence" in options) {
                this._setState(options.enablePersistence);
            }
        },
        option: function (prop, value, forceSet) {
            if (!prop)
                return this.model;

            if ($.isPlainObject(prop))
                return this.setModel(prop, forceSet);

            if (typeof prop === "string") {
                prop = prop.replace(/^model\./, "");
                var oldValue = ej.getObject(prop, this.model);

                if (value === undefined && !forceSet)
                    return oldValue;

                if (prop === "enablePersistence")
                    return this._setState(value);

                if (forceSet && value === ej.extensions.modelGUID) {
                    return this._setModel(ej.createObject(prop, ej.getObject(prop, this.model), {}));
                }

                if (forceSet || ej.getObject(prop, this.model) !== value)
                    return this.setModel(ej.createObject(prop, value, {}), forceSet);
            }
            return undefined;
        },

        _isValidModelValue: function (prop, types, options) {
            var value = types[prop], option = options[prop], returnValue;

            if (!value)
                return true;

            if (typeof value === "string") {
                if (value == "enum") {
                    options[prop] = option ? option.toString().toLowerCase() : option;
                    value = "string";
                }

                if (value === "array") {
                    if (Object.prototype.toString.call(option) === '[object Array]')
                        return true;
                }
                else if (value === "data") {
                    return true;
                }
                else if (value === "parent") {
                    return true;
                }
                else if (typeof option === value)
                    return true;

                return "Expected type - " + value;
            }

            if (option instanceof Array) {
                for (var i = 0; i < option.length; i++) {
                    returnValue = this._isValidModelValue(prop, types, option[i]);
                    if (returnValue !== true) {
                        return " [" + i + "] - " + returnValue;
                    }
                }
                return true;
            }

            for (var innerProp in option) {
                returnValue = this._isValidModelValue(innerProp, value, option);
                if (returnValue !== true)
                    return innerProp + " : " + returnValue;
            }

            return true;
        },

        _returnFn: function (obj, propName) {
            if (propName.indexOf('.') != -1) {
                this._returnFn(obj[propName.split('.')[0]], propName.split('.').slice(1).join('.'));
            }
            else
                obj[propName] = obj[propName].call(obj.propName);
        },

        _removeCircularRef: function (obj) {
            var seen = [];
            function detect(obj, key, parent) {
                if (typeof obj != 'object') { return; }
                if (!Array.prototype.indexOf) {
                    Array.prototype.indexOf = function (val) {
                        return jQuery.inArray(val, this);
                    };
                }
                if (seen.indexOf(obj) >= 0) {
                    delete parent[key];
                    return;
                }
                seen.push(obj);
                for (var k in obj) { //dive on the object's children
                    if (obj.hasOwnProperty(k)) { detect(obj[k], k, obj); }
                }
                seen.pop();
                return;
            }
            detect(obj, 'obj', null);
            return obj;
        },

        stringify: function (model, removeCircular) {
            var observables = this.observables;
            for (var k = 0; k < observables.length; k++) {
                var val = ej.getObject(observables[k], model);
                if (!ej.isNullOrUndefined(val) && typeof (val) === "function")
                    this._returnFn(model, observables[k]);
            }
            if (removeCircular) model = this._removeCircularRef(model);
            return JSON.stringify(model);
        },

        _setState: function (val) {
            if (val === true) {
                this._persistHandler = ej.proxy(this.persistState, this);
                $(window).on("unload", this._persistHandler);
            } else {
                this.deleteState();
                $(window).off("unload", this._persistHandler);
            }
        },

        _removeProp: function (obj, propName) {
            if (!ej.isNullOrUndefined(obj)) {
                if (propName.indexOf('.') != -1) {
                    this._removeProp(obj[propName.split('.')[0]], propName.split('.').slice(1).join('.'));
                }
                else
                    delete obj[propName];
            }
        },

        persistState: function () {
            var model;

            if (this._ignoreOnPersist) {
                model = copyObject({}, this.model);
                for (var i = 0; i < this._ignoreOnPersist.length; i++) {
                    this._removeProp(model, this._ignoreOnPersist[i]);
                }
                model.ignoreOnPersist = this._ignoreOnPersist;
            } else if (this._addToPersist) {
                model = {};
                for (var i = 0; i < this._addToPersist.length; i++) {
                    ej.createObject(this._addToPersist[i], ej.getObject(this._addToPersist[i], this.model), model);
                }
                model.addToPersist = this._addToPersist;
            } else {
                model = copyObject({}, this.model);
            }

            if (this._persistState) {
                model.customPersists = {};
                this._persistState(model.customPersists);
            }

            if (window.localStorage) {
                if (!ej.isNullOrUndefined(ej.persistStateVersion) && window.localStorage.getItem("persistKey") == null)
                    window.localStorage.setItem("persistKey", ej.persistStateVersion);
                window.localStorage.setItem("$ej$" + this.pluginName + this._id, JSON.stringify(model));
            }
            else if (document.cookie) {
                if (!ej.isNullOrUndefined(ej.persistStateVersion) && ej.cookie.get("persistKey") == null)
                    ej.cookie.set("persistKey", ej.persistStateVersion);
                ej.cookie.set("$ej$" + this.pluginName + this._id, model);
            }
        },

        deleteState: function () {
            if (window.localStorage)
                window.localStorage.removeItem("$ej$" + this.pluginName + this._id);
            else if (document.cookie)
                ej.cookie.set("$ej$" + this.pluginName + this._id, model, new Date());
        },

        restoreState: function (silent) {
            var value = null;
            if (window.localStorage)
                value = window.localStorage.getItem("$ej$" + this.pluginName + this._id);
            else if (document.cookie)
                value = ej.cookie.get("$ej$" + this.pluginName + this._id);

            if (value) {
                var model = JSON.parse(value);

                if (this._restoreState) {
                    this._restoreState(model.customPersists);
                    delete model.customPersists;
                }

                if (ej.isNullOrUndefined(model) === false)
                    if (!ej.isNullOrUndefined(model.ignoreOnPersist)) {
                        this._ignoreOnPersist = model.ignoreOnPersist;
                        delete model.ignoreOnPersist;
                    } else if (!ej.isNullOrUndefined(model.addToPersist)) {
                        this._addToPersist = model.addToPersist;
                        delete model.addToPersist;
                    }
            }
            if (!ej.isNullOrUndefined(model) && !ej.isNullOrUndefined(this._ignoreOnPersist)) {
                for (var i in this._ignoreOnPersist) {
                    if (this._ignoreOnPersist[i].indexOf('.') !== -1)
                        ej.createObject(this._ignoreOnPersist[i], ej.getObject(this._ignoreOnPersist[i], this.model), model);
                    else
                        model[this._ignoreOnPersist[i]] = this.model[this._ignoreOnPersist[i]];
                }
                this.model = model;
            }
            else
                this.model = $.extend(true, this.model, model);

            if (!silent && value && this._setModel)
                this._setModel(this.model);
        },

        //to prevent persistence
        ignoreOnPersist: function (properties) {
            var collection = [];
            if (typeof (properties) == "object")
                collection = properties;
            else if (typeof (properties) == 'string')
                collection.push(properties);
            if (this._addToPersist === undefined) {
                this._ignoreOnPersist = this._ignoreOnPersist || [];
                for (var i = 0; i < collection.length; i++) {
                    this._ignoreOnPersist.push(collection[i]);
                }
            } else {
                for (var i = 0; i < collection.length; i++) {
                    var index = this._addToPersist.indexOf(collection[i]);
                    this._addToPersist.splice(index, 1);
                }
            }
        },

        //to maintain persistence
        addToPersist: function (properties) {
            var collection = [];
            if (typeof (properties) == "object")
                collection = properties;
            else if (typeof (properties) == 'string')
                collection.push(properties);
            if (this._addToPersist === undefined) {
                this._ignoreOnPersist = this._ignoreOnPersist || [];
                for (var i = 0; i < collection.length; i++) {
                    var index = this._ignoreOnPersist.indexOf(collection[i]);
                    this._ignoreOnPersist.splice(index, 1);
                }
            } else {
                for (var i = 0; i < collection.length; i++) {
                    if ($.inArray(collection[i], this._addToPersist) === -1)
                        this._addToPersist.push(collection[i]);
                }
            }
        },

        // Get formatted text 
        formatting: function (formatstring, str, locale) {
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            locale = ej.preferredCulture(locale) ? locale : "en-US";
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (typeof (str) == "string" && $.isNumeric(str))
                str = Number(str);
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + ej.format(str, formatVal[2], locale) + RearHtml;
                    else
                        str = ej.format(str, formatVal[2], locale);
                } else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            } else if (s.startsWith("{") && !s.startsWith("{0:")) {
                var fVal = s.split(""), str = (str || "") + "", strSplt = str.split(""), formats = /[0aA\*CN<>\?]/gm;
                for (var f = 0, f, val = 0; f < fVal.length; f++)
                    fVal[f] = formats.test(fVal[f]) ? "{" + val++ + "}" : fVal[f];
                return String.format.apply(String, [fVal.join("")].concat(strSplt)).replace('{', '').replace('}', '');
            } else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            } else {
                return this.data.Value;
            }
        },
    };

    ej.WidgetBase = function () {
    }

    var iterateAndRemoveProps = function (source, target) {
        for (var prop in source) {
            if (source[prop] === target[prop])
                delete target[prop];
            if ($.isPlainObject(target[prop]) && $.isPlainObject(source[prop]))
                iterateAndRemoveProps(source[prop], target[prop]);
        }
    }

    ej.widget = function (pluginName, className, proto) {
        /// <summary>Widget helper for developers, this set have predefined function to jQuery plug-ins</summary>
        /// <param name="pluginName" type="String">the plugin name that will be added in jquery.fn</param>
        /// <param name="className" type="String">the class name for your plugin, this will help create default cssClas</param>
        /// <param name="proto" type="Object">prototype for of the plug-in</param>

        if (typeof pluginName === "object") {
            proto = className;
            for (var prop in pluginName) {
                var name = pluginName[prop];

                if (name instanceof Array) {
                    proto._rootCSS = name[1];
                    name = name[0];
                }

                ej.widget(prop, name, proto);

                if (pluginName[prop] instanceof Array)
                    proto._rootCSS = "";
            }

            return;
        }

        var nameSpace = proto._rootCSS || ej.getNameSpace(className);

        proto = ej.defineClass(className, function (element, options) {

            this.sfType = className;
            this.pluginName = pluginName;
            this.instance = pInstance;

            if (ej.isNullOrUndefined(this._setFirst))
                this._setFirst = true;

            this["ob.values"] = {};

            $.extend(this, ej.widgetBase);

            if (this.dataTypes) {
                for (var property in options) {
                    var returnValue = this._isValidModelValue(property, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + property + " - " + returnValue;
                }
            }

            var arr = (element.data("ejWidgets") || []);
            arr.push(pluginName);
            element.data("ejWidgets", arr);

            for (var i = 0; ej.widget.observables && this.observables && i < this.observables.length; i++) {
                var t = ej.getObject(this.observables[i], options);
                if (t) ej.createObject(this.observables[i], ej.widget.observables.register(t, this.observables[i], this, element), options);
            }

            this.element = element.jquery ? element : $(element);
            this.model = copyObject(true, {}, proto.prototype.defaults, options);
            this.model.keyConfigs = copyObject(this.keyConfigs);

            this.element.addClass(nameSpace + " e-js").data(pluginName, this);

            this._id = element[0].id;

            if (!this.element.attr("tabIndex"))
                this.element.attr("tabIndex", "");

            if (this.model.enablePersistence) {
                if (window.localStorage && !ej.isNullOrUndefined(ej.persistStateVersion) && window.localStorage.getItem("persistKey") != ej.persistStateVersion) {
                    for (var i in window.localStorage) {
                        if (i.indexOf("$ej$") != -1)
                            window.localStorage.removeItem(i); //removing the previously stored plugin item from local storage				
                    }
                }
                else if (document.cookie && !ej.isNullOrUndefined(ej.persistStateVersion) && ej.cookie.get("persistKey") != ej.persistStateVersion) {
                    var splits = document.cookie.split(/; */);
                    for (var k in splits) {
                        if (k.indexOf("$ej$") != -1)
                            ej.cookie.set(k.split("=")[0], model, new Date()); //removing the previously stored plugin item from local storage		
                    }
                }
                this._persistHandler = ej.proxy(this.persistState, this);
                $(window).on("unload", this._persistHandler);
                this.restoreState(true);
            }

            this._init(options);

            if (typeof this.model.keyConfigs === "object" && !(this.model.keyConfigs instanceof Array)) {
                var requiresEvt = false;
                if (this.model.keyConfigs.focus)
                    this.element.attr("accesskey", this.model.keyConfigs.focus);

                for (var keyProps in this.model.keyConfigs) {
                    if (keyProps !== "focus") {
                        requiresEvt = true;
                        break;
                    }
                }

                if (requiresEvt && this._keyPressed) {
                    var el = element, evt = "keydown";

                    if (this.keySettings) {
                        el = this.keySettings.getElement ? this.keySettings.getElement() || el : el;
                        evt = this.keySettings.event || evt;
                    }

                    this._on(el, evt, function (e) {
                        if (!this.model.keyConfigs) return;

                        var action = keyFn.getActionFromCode(this.model.keyConfigs, e.which, e.ctrlKey, e.shiftKey, e.altKey);
                        var arg = {
                            code: e.which,
                            ctrl: e.ctrlKey,
                            alt: e.altKey,
                            shift: e.shiftKey
                        };
                        if (!action) return;

                        if (this._keyPressed(action, e.target, arg, e) === false)
                            e.preventDefault();
                    });
                }
            }
            this._trigger("create");
        }, proto);

        $.fn[pluginName] = function (options) {
            var opt = options, args;
            for (var i = 0; i < this.length; i++) {

                var $this = $(this[i]),
                    pluginObj = $this.data(pluginName),
                    isAlreadyExists = pluginObj && $this.hasClass(nameSpace),
                    obj = null;

                if (this.length > 0 && $.isPlainObject(opt))
                    options = ej.copyObject({}, opt);

                // ----- plug-in creation/init
                if (!isAlreadyExists) {
                    if (proto.prototype._requiresID === true && !$(this[i]).attr("id")) {
                        $this.attr("id", getUid("ejControl_"));
                    }
                    if (!options || typeof options === "object") {
                        if (proto.prototype.defaults && !ej.isNullOrUndefined(ej.setCulture) && "locale" in proto.prototype.defaults && pluginName != "ejChart") {
                            if (options && !("locale" in options)) options.locale = ej.setCulture().name;
                            else if (ej.isNullOrUndefined(options)) {
                                options = {}; options.locale = ej.setCulture().name;
                            }
                        }
                        new proto($this, options);
                    }
                    else {
                        throwError(pluginName + ": methods/properties can be accessed only after plugin creation");
                    }
                    continue;
                }

                if (!options) continue;

                args = [].slice.call(arguments, 1);

                if (this.length > 0 && args[0] && opt === "option" && $.isPlainObject(args[0])) {
                    args[0] = ej.copyObject({}, args[0]);
                }

                // --- Function/property set/access
                if ($.isPlainObject(options)) {
                    // setModel using JSON object
                    pluginObj.setModel(options);
                }

                    // function/property name starts with "_" is private so ignore it.
                else if (options.indexOf('_') !== 0
                    && !ej.isNullOrUndefined(obj = ej.getObject(options, pluginObj))
                    || options.indexOf("model.") === 0) {

                    if (!obj || !$.isFunction(obj)) {

                        // if property is accessed, then break the jquery chain
                        if (arguments.length == 1)
                            return obj;

                        //setModel using string input
                        pluginObj.option(options, arguments[1]);

                        continue;
                    }

                    var value = obj.apply(pluginObj, args);

                    // If function call returns any value, then break the jquery chain
                    if (value !== undefined)
                        return value;

                } else {
                    throwError(className + ": function/property - " + options + " does not exist");
                }
            }
            if (pluginName.indexOf("ejm") != -1)
                ej.widget.registerInstance($this, pluginName, className, proto.prototype);
            // maintaining jquery chain
            return this;
        };

        ej.widget.register(pluginName, className, proto.prototype);
    };

    $.extend(ej.widget, (function () {
        var _widgets = {}, _registeredInstances = [],

        register = function (pluginName, className, prototype) {
            if (!ej.isNullOrUndefined(_widgets[pluginName]))
                throwError("ej.widget : The widget named " + pluginName + " is trying to register twice.");

            _widgets[pluginName] = { name: pluginName, className: className, proto: prototype };

            ej.widget.extensions && ej.widget.extensions.registerWidget(pluginName);
        },
        registerInstance = function (element, pluginName, className, prototype) {
            _registeredInstances.push({ element: element, pluginName: pluginName, className: className, proto: prototype });
        }

        return {
            register: register,
            registerInstance: registerInstance,
            registeredWidgets: _widgets,
            registeredInstances: _registeredInstances
        };

    })());

    ej.widget.destroyAll = function (elements) {
        if (!elements || !elements.length) return;

        for (var i = 0; i < elements.length; i++) {
            var data = elements.eq(i).data(), wds = data["ejWidgets"];
            if (wds && wds.length) {
                for (var j = 0; j < wds.length; j++) {
                    if (data[wds[j]] && data[wds[j]].destroy)
                        data[wds[j]].destroy();
                }
            }
        }
    };

    ej.cookie = {
        get: function (name) {
            var value = RegExp(name + "=([^;]+)").exec(document.cookie);

            if (value && value.length > 1)
                return value[1];

            return undefined;
        },
        set: function (name, value, expiryDate) {
            if (typeof value === "object")
                value = JSON.stringify(value);

            value = escape(value) + ((expiryDate == null) ? "" : "; expires=" + expiryDate.toUTCString());
            document.cookie = name + "=" + value;
        }
    };

    var keyFn = {
        getActionFromCode: function (keyConfigs, keyCode, isCtrl, isShift, isAlt) {
            isCtrl = isCtrl || false;
            isShift = isShift || false;
            isAlt = isAlt || false;

            for (var keys in keyConfigs) {
                if (keys === "focus") continue;

                var key = keyFn.getKeyObject(keyConfigs[keys]);
                for (var i = 0; i < key.length; i++) {
                    if (keyCode === key[i].code && isCtrl == key[i].isCtrl && isShift == key[i].isShift && isAlt == key[i].isAlt)
                        return keys;
                }
            }
            return null;
        },
        getKeyObject: function (key) {
            var res = {
                isCtrl: false,
                isShift: false,
                isAlt: false
            };
            var tempRes = $.extend(true, {}, res);
            var $key = key.split(","), $res = [];
            for (var i = 0; i < $key.length; i++) {
                var rslt = null;
                if ($key[i].indexOf("+") != -1) {
                    var k = $key[i].split("+");
                    for (var j = 0; j < k.length; j++) {
                        rslt = keyFn.getResult($.trim(k[j]), res);
                    }
                }
                else {
                    rslt = keyFn.getResult($.trim($key[i]), $.extend(true, {}, tempRes));
                }
                $res.push(rslt);
            }
            return $res;
        },
        getResult: function (key, res) {
            if (key === "ctrl")
                res.isCtrl = true;
            else if (key === "shift")
                res.isShift = true;
            else if (key === "alt")
                res.isAlt = true;
            else res.code = parseInt(key, 10);
            return res;
        }
    };

    ej.getScrollableParents = function (element) {
        return $(element).parentsUntil("html").filter(function () {
            return $(this).css("overflow") != "visible";
        }).add($(window));
    }
    ej.browserInfo = function () {
        var browser = {}, clientInfo = [],
        browserClients = {
            opera: /(opera|opr)(?:.*version|)[ \/]([\w.]+)/i, edge: /(edge)(?:.*version|)[ \/]([\w.]+)/i, webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie|trident) ([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase() == "opr" ? "opera" : clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    browser.culture = {};
                    browser.culture.name = browser.culture.language = navigator.language || navigator.userLanguage;
                    if (typeof (ej.globalize) != 'undefined') {
                        var oldCulture = ej.preferredCulture().name;
                        var culture = (navigator.language || navigator.userLanguage) ? ej.preferredCulture(navigator.language || navigator.userLanguage) : ej.preferredCulture("en-US");
                        for (var i = 0; (navigator.languages) && i < navigator.languages.length; i++) {
                            culture = ej.preferredCulture(navigator.languages[i]);
                            if (culture.language == navigator.languages[i])
                                break;
                        }
                        ej.preferredCulture(oldCulture);
                        $.extend(true, browser.culture, culture);
                    }
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        browser.pointerEnabled = window.navigator.pointerEnabled;
        return browser;
    };
    ej.eventType = {
        mouseDown: "mousedown touchstart",
        mouseMove: "mousemove touchmove",
        mouseUp: "mouseup touchend",
        mouseLeave: "mouseleave touchcancel",
        click: "click touchend"
    };

    ej.event = function (type, data, eventProp) {

        var e = $.extend(eventProp || {},
            {
                "type": type,
                "model": data,
                "cancel": false
            });

        return e;
    };

    ej.proxy = function (fn, context, arg) {
        if (!fn || typeof fn !== "function")
            return null;

        if ('on' in fn && context)
            return arg ? fn.on(context, arg) : fn.on(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            return fn.apply(context || this, args);
        };
    };

    ej.hasStyle = function (prop) {
        var style = document.documentElement.style;

        if (prop in style) return true;

        var prefixs = ['ms', 'Moz', 'Webkit', 'O', 'Khtml'];

        prop = prop[0].toUpperCase() + prop.slice(1);

        for (var i = 0; i < prefixs.length; i++) {
            if (prefixs[i] + prop in style)
                return true;
        }

        return false;
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };
    var copyObject = ej.copyObject = function (isDeepCopy, target) {
        var start = 2, current, source;
        if (typeof isDeepCopy !== "boolean") {
            start = 1;
        }
        var objects = [].slice.call(arguments, start);
        if (start === 1) {
            target = isDeepCopy;
            isDeepCopy = undefined;
        }

        for (var i = 0; i < objects.length; i++) {
            for (var prop in objects[i]) {
                current = target[prop], source = objects[i][prop];

                if (source === undefined || current === source || objects[i] === source || target === source)
                    continue;
                if (source instanceof Array) {
                    if (i === 0 && isDeepCopy) {
                        target[prop] = new Array();
                        for (var j = 0; j < source.length; j++) {
                            copyObject(true, target[prop], source);
                        }
                    }
                    else
                        target[prop] = source.slice();
                }
                else if (ej.isPlainObject(source)) {
                    target[prop] = current || {};
                    if (isDeepCopy)
                        copyObject(isDeepCopy, target[prop], source);
                    else
                        copyObject(target[prop], source);
                } else
                    target[prop] = source;
            }
        }
        return target;
    };
    var pInstance = function () {
        return this;
    }

    var _uid = 0;
    var getUid = function (prefix) {
        return prefix + _uid++;
    }

    ej.template = {};

    ej.template.render = ej.template["text/x-jsrender"] = function (self, selector, data, index, prop) {
        if (selector.slice(0, 1) !== "#")
            selector = ["<div>", selector, "</div>"].join("");
        var property = { prop: prop, index: index };
        return $(selector).render(data, property);
    }

    ej.isPlainObject = function (obj) {
        if (!obj) return false;
        if (ej.DataManager !== undefined && obj instanceof ej.DataManager) return false;
        if (typeof obj !== "object" || obj.nodeType || jQuery.isWindow(obj)) return false;
        try {
            if (obj.constructor &&
                !obj.constructor.prototype.hasOwnProperty("isPrototypeOf")) {
                return false;
            }
        } catch (e) {
            return false;
        }

        var key, ownLast = ej.support.isOwnLast;
        for (key in obj) {
            if (ownLast) break;
        }

        return key === undefined || obj.hasOwnProperty(key);
    };
    var getValueFn = false;
    ej.util.valueFunction = function (prop) {
        return function (value, getObservable) {
            var val = ej.getObject(prop, this.model);

            if (getValueFn === false)
                getValueFn = ej.getObject("observables.getValue", ej.widget);

            if (value === undefined) {
                if (!ej.isNullOrUndefined(getValueFn)) {
                    return getValueFn(val, getObservable);
                }
                return typeof val === "function" ? val.call(this) : val;
            }

            if (typeof val === "function") {
                this["ob.values"][prop] = value;
                val.call(this, value);
            }
            else
                ej.createObject(prop, value, this.model);
        }
    };
    ej.util.getVal = function (val) {
        if (typeof val === "function")
            return val();
        return val;
    };
    ej.support = {
        isOwnLast: function () {
            var fn = function () { this.a = 1; };
            fn.prototype.b = 1;

            for (var p in new fn()) {
                return p === "b";
            }
        }(),
        outerHTML: function () {
            return document.createElement("div").outerHTML !== undefined;
        }()
    };

    var throwError = ej.throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };

    ej.getRandomValue = function (min, max) {
        if (min === undefined || max === undefined)
            return ej.throwError("Min and Max values are required for generating a random number");

        var rand;
        if ("crypto" in window && "getRandomValues" in crypto) {
            var arr = new Uint16Array(1);
            window.crypto.getRandomValues(arr);
            rand = arr[0] % (max - min) + min;
        }
        else rand = Math.random() * (max - min) + min;
        return rand | 0;
    }

    ej.extensions = {};
    ej.extensions.modelGUID = "{0B1051BA-1CCB-42C2-A3B5-635389B92A50}";
})(window.jQuery, window.Syncfusion);
(function () {
    $.fn.addEleAttrs = function (json) {
        var $this = $(this);
        $.each(json, function (i, attr) {
            if (attr && attr.specified) {
                $this.attr(attr.name, attr.value);
            }
        });

    };
    $.fn.removeEleAttrs = function (regex) {
        return this.each(function () {
            var $this = $(this),
                names = [],
                attrs = $(this.attributes).clone();
            $.each(attrs, function (i, attr) {
                if (attr && attr.specified && regex.test(attr.name)) {
                    $this.removeAttr(attr.name);
                }
            });
        });
    };
    $.fn.attrNotStartsWith = function (regex) {
        var proxy = this;
        var attributes = [], attrs;
        this.each(function () {
            attrs = $(this.attributes).clone();
        });
        for (i = 0; i < attrs.length; i++) {
            if (attrs[i] && attrs[i].specified && regex.test(attrs[i].name)) {
                continue
            }
            else
                attributes.push(attrs[i])
        }
        return attributes;

    }
    $.fn.removeEleEmptyAttrs = function () {
        return this.each(function () {
            var $this = $(this),
                names = [],
                attrs = $(this.attributes).clone();
            $.each(attrs, function (i, attr) {
                if (attr && attr.specified && attr.value === "") {
                    $this.removeAttr(attr.name);
                }
            });
        });
    };
    $.extend($.support, {
        has3d: ej.addPrefix('perspective') in ej.styles,
        hasTouch: 'ontouchstart' in window,
        hasPointer: navigator.msPointerEnabled,
        hasTransform: ej.userAgent() !== false,
        pushstate: "pushState" in history &&
        "replaceState" in history,
        hasTransition: ej.addPrefix('transition') in ej.styles
    });
    //Ensuring elements having attribute starts with 'ejm-' 
    $.extend($.expr[':'], {
        attrNotStartsWith: function (element, index, match) {
            var i, attrs = element.attributes;
            for (i = 0; i < attrs.length; i++) {
                if (attrs[i].nodeName.indexOf(match[3]) === 0) {
                    return false;
                }
            }
            return true;
        }
    });
    //addBack() is supported from Jquery >1.8 and andSelf() supports later version< 1.8. support for both the method is provided by extending the JQuery function.
    var oldSelf = $.fn.andSelf || $.fn.addBack;
    $.fn.andSelf = $.fn.addBack = function () {
        return oldSelf.apply(this, arguments);
    };
})();;
;
ej.mobUtil = {

    /*------Public Properties------*/
    cssUA: ej.userAgent ? '-' + ej.userAgent().toString().toLowerCase() + '-' : '',
    transform: ej.addPrefix('transform'),
    transition: ej.addPrefix('transition'),
    transitionProperty: ej.addPrefix('transitionProperty'),
    transformStyle: ej.addPrefix('transformStyle'),
    transitionDuration: ej.addPrefix('transitionDuration'),
    transformOrigin: ej.addPrefix('transformOrigin'),
    transitionTimingFunction: ej.addPrefix('transitionTimingFunction'),
    transitionDelay: ej.addPrefix('transitionDelay'),
    ease: {
        quadratic: {
            style: 'cubic-bezier(0.25, 0.46, 0.45, 0.94)',
            fn: function (k) {
                return k * (2 - k);
            }
        },
        circular: {
            style: 'cubic-bezier(0.1, 0.57, 0.1, 1)',
            fn: function (k) {
                return Math.sqrt(1 - (--k * k));
            }
        },
        back: {
            style: 'cubic-bezier(0.175, 0.885, 0.32, 1.275)',
            fn: function (k) {
                var b = 4;
                return (k = k - 1) * k * ((b + 1) * k + b) + 1;
            }
        },
        bounce: {
            style: '',
            fn: function (k) {
                if ((k /= 1) < (1 / 2.75)) {
                    return 7.5625 * k * k;
                } else if (k < (2 / 2.75)) {
                    return 7.5625 * (k -= (1.5 / 2.75)) * k + 0.75;
                } else if (k < (2.5 / 2.75)) {
                    return 7.5625 * (k -= (2.25 / 2.75)) * k + 0.9375;
                } else {
                    return 7.5625 * (k -= (2.625 / 2.75)) * k + 0.984375;
                }
            }
        },
        elastic: {
            style: '',
            fn: function (k) {
                var f = 0.22,
					e = 0.4;

                if (k === 0) { return 0; }
                if (k == 1) { return 1; }

                return (e * Math.pow(2, -10 * k) * Math.sin((k - f / 4) * (2 * Math.PI) / f) + 1);
            }
        }
    },

    /*------Public Methods------*/

    //Angular dynamic compiler
    angular: {
        defaultAppName: $('[ng-app]').attr('ng-app'),
        //Compile Code
        compile: function ($targetElement, appName) {
            var $injector = angular.element(document.querySelector('[ng-app],[data-ng-app]') || document).injector();
            $injector.invoke(["$compile", "$rootScope", function ($compile, $rootScope) {
                //Get the scope of the target, use the rootScope if it does not exists
                var $localScope = angular.element($targetElement.closest(".ng-scope") || $targetElement).scope();
                if ($localScope || $rootScope) {
                    $compile($targetElement)($localScope || $rootScope);
                }
                if (!($localScope || $rootScope).$$phase)
                    ($localScope || $rootScope).$digest();
            }]);
        }
    },
    angularMobileSettings: {
        enableAnimation: true,
        animationTime: 300
    },
    isAppNullOrUndefined: function () {
        return (typeof App == 'undefined');
    },
    device: {
        //To check whether its iOS
        isIOS: function () {
            return ((/(ipad|iphone|ipod touch)/i).test(navigator.userAgent.toLowerCase()) && !this.isWindows());
        },
        //To check whether its iOS7
        isIOS7: function () {
            return ((/(ipad|iphone|ipod touch);.*os 7_\d|(ipad|iphone|ipod touch);.*os 8_\d/i).test(navigator.userAgent.toLowerCase()) && !this.isWindows());
        },
        //To check whether its Android
        isAndroid: function () {
            return ((/android/i.test(navigator.userAgent.toLowerCase())) && !this.isWindows());
        },
        //To check whether its Windows
        isWindows: function () {
            return (/trident|windows phone/i.test(navigator.userAgent.toLowerCase()));
        },
        //To check whether its Flat
        isFlat: function () {
            return (ej.getBooleanVal($('head'), 'data-ej-flat', false) === true);
        }
    },

    //To check whether its iOS
    isIOS: function () {
        if (!ej.getBooleanVal($('head'), 'data-ej-windows') && !ej.getBooleanVal($('head'), 'data-ej-android') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._ios();
    },
    //To check whether its iOS7
    isIOS7: function () {
        if (!ej.getBooleanVal($('head'), 'data-ej-windows') && !ej.getBooleanVal($('head'), 'data-ej-android') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._ios7();
    },
    //To check whether its Android
    isAndroid: function () {
        if (!ej.getBooleanVal($('head'), 'data-ej-windows') && !ej.getBooleanVal($('head'), 'data-ej-ios') && !ej.getBooleanVal($('head'), 'data-ej-ios7') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._android();
    },
    setTransition: function () {
        return ej.isAndroid() ? "pop" : ej.isWindows() ? (ej.isMobile() ? "slide" : "slide") : "slide";
    },
    //To check whether its lower version Android device
    isLowerAndroid: function () {
        return /Android /.test(window.navigator.appVersion) && !(/Chrome\/\d/.test(window.navigator.appVersion));
    },
    //To get the version of Android device
    getAndroidVersion: function () {
        if (this._android() && navigator.userAgent.toLowerCase().match(/android\s+([\d\.]+)/))
            return parseFloat(navigator.userAgent.toLowerCase().match(/android\s+([\d\.]+)/)[1]);
        else
            return false;
    },
    //To check whether its Windows
    isWindows: function () {
        if (!ej.getBooleanVal($('head'), 'data-ej-android') && !ej.getBooleanVal($('head'), 'data-ej-ios') && !ej.getBooleanVal($('head'), 'data-ej-ios7') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._windows();
    },
    //To check whether its Flat
    isFlat: function () {
        return (ej.getBooleanVal($('head'), 'data-ej-flat', false) === true);
    },
    //To check whether element has theme
    hasTheme: function (element) {
        return element.hasClass('e-m-dark') || element.hasClass('e-m-light') || element.hasClass('e-m-default');
    },
    //Sets the Theme for the element
    setTheme: function (object) {
        if (object.model.renderMode != "windows" && !ej.isMobile() && object.model.theme == "default")
            object.model.theme = "auto";
        var comTheme = ej.getAttrVal($('head'), "data-ej-theme", "auto");
        var renderDefault = ej.getBooleanVal($('head'), "data-ej-windows-renderdefault", object.model.windows ? object.model.windows.renderDefault : false);
        if (object.model.windows)
            object.model.windows.renderDefault = renderDefault;
        object.model.theme = object.model.theme == "auto" ? comTheme == "auto" ? ((object.model.renderMode == "ios7" || ((object.model.renderMode == "android" || object.model.renderMode == "windows") && !ej.isMobile())) ? "light" : "dark") : comTheme : object.model.theme;
    },
    //Get theme that applied for app
    getTheme: function () {
        objTheme = { model: { theme: "auto" } };
        this.setTheme(objTheme);
        return objTheme.model.theme;
    },
    //To check whether element has rendermode
    hasRenderMode: function (element) {
        return element.hasClass('e-m-ios7') || element.hasClass('e-m-android') || element.hasClass('e-m-windows') || element.hasClass('e-m-flat');
    },
    //Sets the Render Mode for the element
    setRenderMode: function (element) {
        element.model.renderMode = ej.getAttrVal(element.element, 'data-ej-rendermode', element.model.renderMode);
        if (element.model.renderMode == "auto") {
            if (ej.isAndroid())
                element.model.renderMode = "android";
            else if (ej.isIOS())
                element.model.renderMode = "ios7";
            else if (ej.isWindows())
                element.model.renderMode = "windows";
            else if (ej.isFlat())
                element.model.renderMode = "flat";
            else
                element.model.renderMode = "ios7";
        }
    },
    //Gets the Render Mode for the element
    getRenderMode: function () {
        if (this.isAndroid())
            return "android";
        else if (this.isIOS())
            return "ios7";
        else if (this.isWindows())
            return "windows";
        else if (this.isFlat())
            return "flat";
        else
            return "ios7";
    },
    //To get CurrentPage
    getCurrentPage: function () {
        return ej.isAppNullOrUndefined() ? $("body") : App._renderingPage ? App._renderingPage : App.activePage;
    },
    //To detect the browser
    browser: function () {
        return (/webkit/i).test(navigator.appVersion) ? 'webkit' : (/firefox/i).test(navigator.userAgent) ? 'Moz' : (/trident/i).test(navigator.userAgent) ? 'ms' : 'opera' in window ? 'O' : '';
    },

    //To round the value
    round: function (value, div, up) {
        return div * (up ? Math.ceil(value / div) : Math.floor(value / div));
    },
    //To log base the value
    logBase: function (val, base) {
        return Math.log(val) / Math.log(base);
    },
    //To make the rectangle with absolute value
    correctRect: function (x1, y1, x2, y2) {
        return { X: Math.min(x1, x2), Y: Math.min(y1, y2), Width: Math.abs(x2 - x1), Height: Math.abs(y2 - y1) };
    },
    //To check the text length
    measureText: function (text, maxwidth, font) {
        var textObj = document.createElement('DIV');
        textObj.innerHTML = text;
        if (font != null)
            textObj.style.font = font;
        textObj.style.backgroundColor = 'white';
        textObj.style.position = 'absolute';
        textObj.style.top = -100
        textObj.style.left = 0;
        if (maxwidth)
            textObj.style.maxwidth = maxwidth + "px";

        document.body.appendChild(textObj);

        var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
        $(textObj).remove();
        return bounds;
    },
    //To get the current time
    getTime: Date.now || new Date().getTime(),
    //To get the Font String
    getFontString: function (fontObj) {
        if (fontObj == null)
            fontObj = {};
        if (!fontObj.FontFamily)
            fontObj.FontFamily = "Arial";
        if (!fontObj.FontStyle)
            fontObj.FontStyle = 0;
        if (!fontObj.Size)
            fontObj.Size = "12px";

        return ej.GetFontStyle(fontObj.FontStyle) + " " + fontObj.Size + " " + fontObj.FontFamily;
    },
    //To get the Font Style
    getFontStyle: function (style) {
        switch (style) {
            case 0:
                return "Regular";
            case 1:
                return "Bold";
            case 2: return "Italic";
            case 4: return "Underline";
            case 8: return "StrikeOut";
        }
    },
    //To convert RGB to Hexadecimal value
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        var hex = [r.toString(16), g.toString(16), b.toString(16)];
        $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
        return hex.join("").toUpperCase();
    },

    adjustFixedElement: function (ele) {
            ele.removeClass("e-m-adjheader e-m-adjfooter");
            var topnavLen = ele.siblings(".e-m-navbar-top").length,
                bottomnavLen = ele.siblings(".e-m-navbar-bottom").length;
            //Audjust Fixed position for Header
            if (topnavLen) ele.addClass("e-m-adjheader-" + topnavLen);
            //Audjust Fixed position for Footer
            if (bottomnavLen) ele.addClass("e-m-adjfooter-" + bottomnavLen);
    },

    //To set caret position in the textbox
    setCaretToPos: function (input, pos1, pos2) {
        if (input.setSelectionRange && input.type != "number") {
            input.focus();
            input.setSelectionRange(pos1, pos2);
        }
        else if (input.createTextRange) {
            var range = input.createTextRange();
            range.collapse(true);
            range.moveStart('character', pos1);
            range.moveEnd('character', pos2);
            range.select();
        }
    },
    isCssCalc: function () {
        $('body').append('<div id="css3-calc" style=" width: 10px; width: calc(10px + 10px);display: none;"></div>');
        var css3_calc = $('#css3-calc').width();
        $('#css3-calc').remove();
        return css3_calc == 20 ? true : false;
    },  

    getLocation: function (url) {
        var uri = url ? this.route.splitUrl(url) : location,
                hash = this.route.splitUrl(url || location.href).hash;
        hash = hash === "#" ? "" : hash;
        return uri.protocol + "//" + uri.host + uri.pathname + uri.search + hash;
    },

    route: {
        urlSplitReg: /^\s*(((([^:\/#\?]+:)?(?:(\/\/)((?:(([^:@\/#\?]+)(?:\:([^:@\/#\?]+))?)@)?(([^:\/#\?\]\[]+|\[[^\/\]@#?]+\])(?:\:([0-9]+))?))?)?)?((\/?(?:[^\/\?#]+\/+)*)([^\?#]*)))?(\?[^#]+)?)(#.*)?/,
        splitUrl: function (url) {
            if ($.type(url) === "object") {
                return url;
            }
            var matches = this.urlSplitReg.exec(url || "") || [];
            return {
                href: matches[0] || "",
                protocol: matches[4] || "",
                host: matches[10] || "",
                pathname: matches[13] || "",
                search: matches[16] || "",
                hash: matches[17] || ""
            };
        },
    },

    resize: function () {
        $(window).bind("onorientationchange" in window ? "orientationchange" : "resize", function (e) {
            if (ej.getRenderMode() == "android" || ej.getRenderMode() == "ios7")
                $(document.activeElement).blur();

            if (ej._currentResolution != ej.isLowerResolution()) {
                ej._currentResolution = ej.isLowerResolution();
                $.extend(true, e, { resolutionChanged: true });
            }
            else
                $.extend(true, e, { resolutionChanged: false });
            for (var i = 0; i < ej.widget.registeredInstances.length; i++) {
                if ($(ej.widget.registeredInstances[i].element).is(":visible") && $.isFunction(ej.widget.registeredInstances[i].proto.resize))
                    $(ej.widget.registeredInstances[i].element).data(ej.widget.registeredInstances[i].pluginName).resize(e);
            }
            if (e.type == "orientationchange") {
                window.setTimeout(function () {
                    if (ej.getRenderMode() == "ios7")
                        window.scrollTo(0, 0);
                });
            }
        });
    },

    initPage: function () {
        if ((/(ipad|iphone|ipod touch);.*os 7_\d/i).test(navigator.userAgent.toLowerCase())) {
            $(document).on('blur', 'input, select, textarea', function () {
                window.setTimeout(function () {
                    if (document.activeElement.nodeName && document.activeElement.nodeName.toLowerCase() != "input")
                        window.scrollTo(0, 0);
                });
            });
        }
        $(window).bind('touchmove', function (e) {
            if ($(e.target).closest('.e-m-scroll-native').length == 0 && ej.isTouchDevice())
                ej.blockDefaultActions(e);
        });
    },


    /*------Private Methods------*/
    //To do momentum for the desired element
    _momentum: function (dist, time, maxDistUpper, maxDistLower, size) {
        var deceleration = 0.0006,
			speed = Math.abs(dist) / time,
			newDist = (speed * speed) / (2 * deceleration),
			newTime = 0, outsideDist = 0;

        // Proportinally reduce speed if we are outside of the boundaries
        if (dist > 0 && newDist > maxDistUpper) {
            outsideDist = size / (6 / (newDist / speed * deceleration));
            maxDistUpper = maxDistUpper + outsideDist;
            speed = speed * maxDistUpper / newDist;
            newDist = maxDistUpper;
        } else if (dist < 0 && newDist > maxDistLower) {
            outsideDist = size / (6 / (newDist / speed * deceleration));
            maxDistLower = maxDistLower + outsideDist;
            speed = speed * maxDistLower / newDist;
            newDist = maxDistLower;
        }

        newDist = newDist * (dist < 0 ? -1 : 1);
        newTime = speed / deceleration;

        return { dist: newDist, time: Math.round(newTime) };
    },
    //To do native momentum for the desired element
    _nativeMomentum: function (current, start, time, lowerMargin, wrapperSize, deceleration, division) {
        var distance = current - start,
			speed = Math.abs(distance) / time,
			dest,
			duration;

        deceleration = deceleration == null ? 0.0006 : deceleration;
        dest = current + (speed * speed) / (2 * (deceleration / division)) * (distance < 0 ? -1 : 1);
        duration = speed / deceleration;

        if (dest < lowerMargin) {
            dest = wrapperSize ? lowerMargin - (wrapperSize / 2.5 * (speed / 8)) : lowerMargin;
            distance = Math.abs(dest - current);
            duration = distance / speed;
        } else if (dest > 0) {
            dest = wrapperSize ? wrapperSize / 2.5 * (speed / 8) : 0;
            distance = Math.abs(current) + dest;
            duration = distance / speed;
        }

        return {
            dest: Math.round(dest),
            duration: duration
        };
    },
    //Sets the hidden style for element 
    _setHidden: function (element) {
        var proxy = this;
        this.hidden.each(function (i) {
            var temp = proxy.tmp[i], name;
            for (name in prop) {
                this.style[name] = temp[name];
            }
        });
    },
    //Removes the hidden style for element 
    _removeHidden: function (element) {
        var value;
        var proxy = this;
        this.hidden = $(element).parents().andSelf().filter(':hidden');
        prop = { visibility: 'hidden', display: 'block' };
        this.tmp = [];
        this.hidden.each(function () {
            var temp = {}, name;
            for (name in prop) {
                temp[name] = this.style[name];
                this.style[name] = prop[name];
            }
            proxy.tmp.push(temp);
        });

    },

    _transitionTime: function (time, target) {
        time += 'ms';
        target.style[ej.transitionDuration] = time;
    },
    _slideOutWithoutDuration: function (target, reverse) {
        ej._transitionTime(0, target);
        target.style[ej.transform] = reverse ? "translateX(-100%)" : "translateX(100%)";
    },
    _slideInWithoutDuration: function (target, reverse) {
        ej._transitionTime(0, target);
        target.style[ej.transform] = reverse ? "translateX(100%)" : "translateX(-100%)";
    },
    _slideOutWithDuration: function (target, reverse) {
        ej._transitionTime(300, target);
        target.style[ej.transform] = reverse ? "translateX(-100%)" : "translateX(100%)";
    },
    _slideInWithDuration: function (target, reverse) {
        ej._transitionTime(100, target);
        target.style[ej.transform] = reverse ? "translateX(100%)" : "translateX(-100%)";
    },
    _slide: function (target) {
        ej._transitionTime(100, target);
        target.style[ej.transform] = "translateX(0)";
    },
    _device: function () {
        return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
    },
    _ios: function () {
        return ((/(ipad|iphone|ipod touch)/i).test(navigator.userAgent.toLowerCase()) && !this._windows()) || (ej.getBooleanVal($('head'), 'data-ej-ios7', false) === true);
    },
    _ios7: function () {
        return ((/(ipad|iphone|ipod touch);.*os 7_\d|(ipad|iphone|ipod touch);.*os 8_\d/i).test(navigator.userAgent.toLowerCase()) && !this._windows()) || (ej.getBooleanVal($('head'), 'data-ej-ios7', false) === true);
    },
    _android: function () {
        return ((/android/i.test(navigator.userAgent.toLowerCase())) && !this._windows()) || (ej.getBooleanVal($('head'), 'data-ej-android', false) === true);
    },
    _windows: function () {
        return (/trident|windows phone|edge/i.test(navigator.userAgent.toLowerCase())) || (ej.getBooleanVal($('head'), 'data-ej-windows', false) === true);
    },
    _getFontString: function (element, font) {
        return element.model.fontStyle[font.FontStyle] + " " + ((font.Size == null) ? "11px" : font.Size) + " " + font.FontFamily;
    },
    _setScroller: function (proxy) {
        proxy._wrpWd = proxy.element[0].clientWidth;
        proxy._scrollerWidth = proxy.element[0].scrollWidth;
        proxy._maxScrollX = proxy._wrpWd - proxy._scrollerWidth;
    },
    _inputTouchStart: function (evt, proxy) {
        var point = evt.touches ? evt.touches[0] : evt,
			pos;
        proxy._moved = false;
        proxy._distX = 0;
        proxy._x = proxy._x == undefined ? proxy.element[0].scrollLeft : proxy._x;
        proxy._cloned = false;
        proxy._startX = 0;
        proxy._pointX = point.pageX;
        proxy._startTime = ej.getTime();
    },
    _inputTouchMove: function (evt, proxy) {
        var point = evt.touches ? evt.touches[0] : evt,
			deltaX = point.pageX - proxy._pointX;
        if (deltaX > 0)
            proxy._reverse = true;
        if (proxy.element[0].scrollLeft == 0 && deltaX > 0 || (deltaX > 0 && proxy.element[0].scrollLeft < deltaX) || (deltaX < 0 && proxy.element[0].scrollLeftMax <= proxy.element[0].scrollLeft))
            return;
        if (deltaX > 0 && proxy.element[0].scrollLeft + deltaX > proxy.element[0].scrollLeftMax) {
            if (proxy._x < 0)
                proxy._x = -proxy._x;
            deltaX = -deltaX;
        }
        var newX = proxy._x + deltaX,
            timestamp = ej.getTime();
        proxy._distX += deltaX;
        ej._translateText(proxy.element, newX);
    },
    _inputTouchEnd: function (evt, proxy) {
        var point = evt.touches ? evt.changedTouches[0] : evt,
			deltaX = point.pageX - proxy._pointX;
        var newX = deltaX,
            timestamp = ej.getTime();
        proxy._distX += deltaX;
        ej._translateText(proxy.element, proxy.element[0].scrollLeft);
        proxy._x = -proxy.element[0].scrollLeft;
    },
    _translateText: function (ele, scrollX) {
        ele[0].scrollLeft = Math.abs(scrollX);
    },
    _setGradientColor: function (element, gradient, options) {
        var self = element;
        if (options.Name || typeof (options) === "string")
            gradient.addColorStop(0, ej._getColor(options));
        else
            $.each(options, function (index, colorElement) {
                gradient.addColorStop(colorElement.ColorStop != NaN ? colorElement.ColorStop : 0, typeof (colorElement.Color) === "string" ? colorElement.Color : ej._getColor(colorElement.Color));
            });
    },
    //To push value into array in particular index
    _pushValue: function (stored, val, index) {
        var tempStored = [];
        for (i = 0; i < index; i++) {
            tempStored[i] = stored[i];
        }
        tempStored[index] = val;
        var index = tempStored.length;
        for (i = index; i <= stored.length; i++) {
            tempStored[i] = stored[i - 1];
        }
        stored = tempStored;
        return stored;
    }
};
$.extend(ej, ej.mobUtil);
ej.mobile = {};
ej.mobile.enableRippleEffect = true;
ej.mobile.WaitingPopup = {
    _init: function (options) {
        this.model = {
            text: ("Loading..."),
            target: $("body"),
            showOnInit: false
        };
        $.extend(true, this.model, options);
        this.createWaitingPopup();
    },
    createWaitingPopup: function () {
        if (!this.waitingPopupDiv) {
            var devName = ej.isMobile() ? 'mobile' : 'tablet';
            this.waitingPopupDiv = ej.buildTag('div.e-m-waitingpopup e-m-' + ej.getRenderMode() + ' e-m-' + devName, "<div class='e-m-image' style='top: " + (!ej.isCssCalc() ? window.innerHeight / 2 - 16 : '') + "px'></div>");
            this.waitingPopupDiv.find(".e-m-image").text(this.model.text);
            this.model.target.append(this.waitingPopupDiv);
            if (!this.model.showOnInit)
                this.hide();
        }
    },

    setText: function (text) {
        this.model.text = text;
        this.waitingPopupDiv.find(".e-m-image").text(text);
    },
    changeTarget: function (target) {
        this.model.target = target;
        this.model.target.append(this.waitingPopupDiv);
    },
    show: function () {
        this.model.showOnInit = true;
        this.waitingPopupDiv.addClass("e-m-show").removeClass("e-m-hide");
    },
    hide: function () {
        this.model.showOnInit = false;
        this.waitingPopupDiv.removeClass("e-m-show").addClass("e-m-hide");
    }
};

$(function () {
    ej.initPage();
    ej._currentResolution = ej.isLowerResolution();
    ej.mobile.WaitingPopup._init();
    if (ej.mobile.enableRippleEffect && !ej.isNullOrUndefined(window["ejAnimation"]))
        $("body").ejAnimation("rippleEffect");
    ej.resize();    
});
ej.mobile.RenderMode = {    
    Auto: "auto",    
    IOS7: "ios7",    
    Android: "android",    
    Windows: "windows"
};

ej.mobile.Theme = {    
    Auto: "auto",    
    Dark: "dark",    
    Light: "light"
};
;
window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, doc, undefined) {
    'use strict';
	
    ej.DataManager = function (dataSource, query, adaptor) {
          if (!(this instanceof ej.DataManager))
            return new ej.DataManager(dataSource, query, adaptor);

        if (!dataSource)
            dataSource = [];
        adaptor = adaptor || dataSource.adaptor;

        if (typeof (adaptor) === "string") 
            adaptor = new ej[adaptor]();
        var data = [], self = this;

        if (dataSource instanceof Array) {
            // JSON array
            data = {
                json: dataSource,
                offline: true
            };

        } else if (typeof dataSource === "object") {
            if ($.isPlainObject(dataSource)) {
                if (!dataSource.json)
                    dataSource.json = [];
                if (dataSource.table)
                    dataSource.json = this._getJsonFromElement(dataSource.table, dataSource.headerOption);
                data = {
                    url: dataSource.url,
                    insertUrl: dataSource.insertUrl,
                    removeUrl: dataSource.removeUrl,
                    updateUrl: dataSource.updateUrl,
                    crudUrl: dataSource.crudUrl,
                    batchUrl: dataSource.batchUrl,
                    json: dataSource.json,
                    headers: dataSource.headers,
                    accept: dataSource.accept,
                    data: dataSource.data,
					async : dataSource.async,
                    timeTillExpiration: dataSource.timeTillExpiration,
                    cachingPageSize: dataSource.cachingPageSize,
                    enableCaching: dataSource.enableCaching,
                    requestType: dataSource.requestType,
                    key: dataSource.key,
                    crossDomain: dataSource.crossDomain,
                    jsonp: dataSource.jsonp,
                    dataType: dataSource.dataType,
                    offline: dataSource.offline !== undefined ? dataSource.offline : dataSource.adaptor == "remoteSaveAdaptor" || dataSource.adaptor instanceof ej.remoteSaveAdaptor ? false : dataSource.url ? false : true,
                    requiresFormat: dataSource.requiresFormat
                };
            } else if (dataSource.jquery || isHtmlElement(dataSource)) {
                data = {
                    json: this._getJsonFromElement(dataSource),
                    offline: true,
                    table: dataSource
                };
            }
        } else if (typeof dataSource === "string") {
            data = {
                url: dataSource,
                offline: false,
                dataType: "json",
                json: []
            };
        }

        if (data.requiresFormat === undefined && !ej.support.cors)
            data.requiresFormat = isNull(data.crossDomain) ? true : data.crossDomain;
        if (data.dataType === undefined)
            data.dataType = "json";
        this.dataSource = data;
        this.defaultQuery = query;

        if (data.url && data.offline && !data.json.length) {
            this.isDataAvailable = false;
            this.adaptor = adaptor || new ej.ODataAdaptor();
            this.dataSource.offline = false;
            this.ready = this.executeQuery(query || ej.Query()).done(function (e) {
                self.dataSource.offline = true;
                self.isDataAvailable = true;
                data.json = e.result;
                self.adaptor = new ej.JsonAdaptor();
            });
        }
        else
            this.adaptor = data.offline ? new ej.JsonAdaptor() : new ej.ODataAdaptor();
        if (!data.jsonp && this.adaptor instanceof ej.ODataAdaptor)
            data.jsonp = "callback";
        this.adaptor = adaptor || this.adaptor;
        if (data.enableCaching)
            this.adaptor = new ej.CacheAdaptor(this.adaptor, data.timeTillExpiration, data.cachingPageSize);
        return this;
    };

    ej.DataManager.prototype = {
        setDefaultQuery: function (query) {
            this.defaultQuery = query;
        },
	
        executeQuery: function (query, done, fail, always) {
            if (typeof query === "function") {
                always = fail;
                fail = done;
                done = query;
                query = null;
            }

            if (!query)
                query = this.defaultQuery;

            if (!(query instanceof ej.Query))
                throwError("DataManager - executeQuery() : A query is required to execute");

            var deffered = $.Deferred();

            deffered.then(done, fail, always);
            var args = { query: query };

            if (!this.dataSource.offline && this.dataSource.url != undefined) {
				 var result = this.adaptor.processQuery(this, query);
                if (!ej.isNullOrUndefined(result.url))
                    this._makeRequest(result, deffered, args, query);
                else {
                    nextTick(function () {
                        args = this._getDeferedArgs(query, result, args);
                        deffered.resolveWith(this, [args]);;
                    }, this);
                }
            } else {
				if(!ej.isNullOrUndefined(this.dataSource.async) && this.dataSource.async == false)
					this._localQueryProcess(query, args, deffered);
				else{
					nextTick(function () {
						this._localQueryProcess(query, args, deffered);
					}, this);
				}
            }
            return deffered.promise();
        },
		_localQueryProcess: function(query, args, deffered){
			var res = this.executeLocal(query);
			args = this._getDeferedArgs(query, res, args);
			deffered.resolveWith(this, [args]);
		},
        _getDeferedArgs: function (query, result, args) {
            if (query._requiresCount) {
                args.result = result.result;
                args.count = result.count;
            } else
                args.result = result;
            args.getTableModel = getTableModel(query._fromTable, args.result, this);
            args.getKnockoutModel = getKnockoutModel(args.result);
            return args;
        },
	
        executeLocal: function (query) {
            if (!this.defaultQuery && !(query instanceof ej.Query))
                throwError("DataManager - executeLocal() : A query is required to execute");

            if (!this.dataSource.json)
                throwError("DataManager - executeLocal() : Json data is required to execute");

            query = query || this.defaultQuery;

            var result = this.adaptor.processQuery(this, query);

            if (query._subQuery) {
                var from = query._subQuery._fromTable, lookup = query._subQuery._lookup,
                    res = query._requiresCount ? result.result : result;

                if (lookup && lookup instanceof Array) {
                    buildHierarchy(query._subQuery._fKey, from, res, lookup, query._subQuery._key);
                }

                for (var j = 0; j < res.length; j++) {
                    if (res[j][from] instanceof Array) {
                        res[j] = $.extend({}, res[j]);
                        res[j][from] = this.adaptor.processResponse(query._subQuery.using(ej.DataManager(res[j][from].slice(0))).executeLocal(), this, query);
                    }
                }
            }

            return this.adaptor.processResponse(result, this, query);
        },

        _makeRequest: function (url, deffered, args, query) {
            var isSelector = !!query._subQuerySelector;

            var fnFail = $proxy(function (e) {
                args.error = e;
                deffered.rejectWith(this, [args]);
            }, this);

            var process = $proxy(function (data, count, xhr, request, actual, aggregates, virtualSelectRecords) {
                if (isSelector) return;

                args.xhr = xhr;
                args.count = parseInt(count, 10);
                args.result = data;
                args.request = request;
                args.aggregates = aggregates;
                args.getTableModel = getTableModel(query._fromTable, data, this);
                args.getKnockoutModel = getKnockoutModel(data);
                args.actual = actual;
                args.virtualSelectRecords = virtualSelectRecords;
                deffered.resolveWith(this, [args]);

            }, this);

            var fnQueryChild = $proxy(function (data, selector) {
                var subDeffer = $.Deferred(),
                    childArgs = { parent: args };

                query._subQuery._isChild = true;

                var subUrl = this.adaptor.processQuery(this, query._subQuery, data ? this.adaptor.processResponse(data) : selector);

                var childReq = this._makeRequest(subUrl, subDeffer, childArgs, query._subQuery);

                if(!isSelector)
                    subDeffer.then(function (subData) {
                        if (data) {
                            buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, data, subData, query._subQuery._key);
                            process(data);
                        }
                    }, fnFail);

                return childReq;
            }, this);

            var fnSuccess = proxy(function (data, status, xhr, request) {
                if (xhr.getResponseHeader("Content-Type").indexOf("xml") == -1 && ej.dateParse)
                    data = ej.parseJSON(data);
                var result = this.adaptor.processResponse(data, this, query, xhr, request), count = 0, aggregates = null;
                var virtualSelectRecords = data.virtualSelectRecords;
                if (query._requiresCount) {
                    count = result.count;
                    aggregates = result.aggregates;
                    result = result.result;
                }

                if (!query._subQuery) {
                    process(result, count, xhr, request, data, aggregates, virtualSelectRecords);
                    return;
                }

                if (!isSelector)
                    fnQueryChild(result);

            }, this);

            var req = $.extend({
                type: "GET",
                dataType: this.dataSource.dataType,
                crossDomain: this.dataSource.crossDomain,
                jsonp: this.dataSource.jsonp,
                cache: true,
                beforeSend: $proxy(this._beforeSend, this),
                processData: false,
                success: fnSuccess,
                error: fnFail
            }, url);

            if ("async" in this.dataSource)
                req.async = this.dataSource.async;

            req = $.ajax(req);

            if (isSelector) {
                var res = query._subQuerySelector.call(this, { query: query._subQuery, parent: query });

                if (res && res.length) {
                    req = $.when(req, fnQueryChild(null, res));

                    req.then(proxy(function (pData, cData, requests) {
                        var pResult = this.adaptor.processResponse(pData[0], this, query, pData[2], requests[0]), count = 0;
                        if (query._requiresCount) {
                            count = pResult.count;
                            pResult = pResult.result;
                        }
                        var cResult = this.adaptor.processResponse(cData[0], this, query._subQuery, cData[2], requests[1]), count = 0;
                        if (query._subQuery._requiresCount) {
                            count = cResult.count;
                            cResult = cResult.result;
                        }

                        buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, pResult, cResult, query._subQuery._key);
                        isSelector = false;
                        process(pResult, count, pData[2]);

                    }, this), fnFail);
                } else {
                    isSelector = false;
                }
            }

            return req;
        },

        _beforeSend: function (request, settings) {
            this.adaptor.beforeSend(this, request, settings);

            var headers = this.dataSource.headers, props;
            for (var i = 0; headers && i < headers.length; i++) {
                props = [];
                for (var prop in headers[i]) {
                    props.push(prop);
                    request.setRequestHeader(prop, headers[i][prop]);
                }
            }
        },
	
        saveChanges: function (changes, key, tableName, query) {

            if (tableName instanceof ej.Query) {
                query = tableName;
                tableName = null;
            }

            var args = {
                url: tableName,
                key: key || this.dataSource.key
            };

            var req = this.adaptor.batchRequest(this, changes, args, query);

            if (this.dataSource.offline) {
                return req;
            }

            var deff = $.Deferred();
            $.ajax($.extend({
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (data, status, xhr, request) {
                    deff.resolveWith(this, [this.adaptor.processResponse(data, this, null, xhr, request, changes)]);
                }, this),
                error: function (e) {
                    deff.rejectWith(this, [{ error: e }]);
                }
            }, req));

            return deff.promise();
        },
	
        insert: function (data, tableName, query) {           
            data = p.replacer(data);

            if (tableName instanceof ej.Query) {
                query = tableName;
                tableName = null;
            }

            var res = this.adaptor.insert(this, data, tableName, query);

            if (this.dataSource.offline) {
                return res;
            }            

            var deffer = $.Deferred();

            $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                processData: false,
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    try {
                        if (ej.isNullOrUndefined(record))
                            record = [];
                        else
                            p.parseJson(record);
                    }
                    catch (e) {
                        record = [];
                    }
                    record = this.adaptor.processResponse(p.parseJson(record), this, null, xhr, request);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
            }, res));

            return deffer.promise();
        },
	
        remove: function (keyField, value, tableName, query) {
            if (typeof value === "object")
                value = value[keyField];

            if (tableName instanceof ej.Query) {
                query = tableName;
                tableName = null;
            }

            var res = this.adaptor.remove(this, keyField, value, tableName, query);

            if (this.dataSource.offline)
                return res;          

            var deffer = $.Deferred();
            $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    try {
                        if (ej.isNullOrUndefined(record))
                            record = [];
                        else
                            p.parseJson(record);
                    }
                    catch (e) {
                        record = [];
                    }
                    record = this.adaptor.processResponse(p.parseJson(record), this, null, xhr, request);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
            }, res));
            return deffer.promise();
        },
	
        update: function (keyField, value, tableName, query) {
            value = p.replacer(value);

            if (tableName instanceof ej.Query) {
                query = tableName;
                tableName = null;
            }

            var res = this.adaptor.update(this, keyField, value, tableName, query);

            if (this.dataSource.offline) {
                return res;
            }           

            var deffer = $.Deferred();

           $.ajax($.extend({
                contentType: "application/json; charset=utf-8",
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    try {
                        if (ej.isNullOrUndefined(record))
                            record = [];
                        else
                            p.parseJson(record);
                    }
                    catch (e) {
                        record = [];
                    }
                    record = this.adaptor.processResponse(p.parseJson(record), this, null, xhr, request);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
           }, res));

           return deffer.promise();
        },

        _getJsonFromElement: function (ds) {
            if (typeof (ds) == "string")
                ds = $($(ds).html());

            ds = ds.jquery ? ds[0] : ds;

            var tagName = ds.tagName.toLowerCase();

            if (tagName !== "table")
                throwError("ej.DataManager : Unsupported htmlElement : " + tagName);

            return ej.parseTable(ds);
        }
    };

    var buildHierarchy = function (fKey, from, source, lookup, pKey) {
        var i, grp = {}, t;
        if (lookup.result) lookup = lookup.result;

        if (lookup.GROUPGUID)
            throwError("ej.DataManager: Do not have support Grouping in hierarchy");

        for (i = 0; i < lookup.length; i++) {
            var fKeyData = ej.getObject(fKey, lookup[i]);
            t = grp[fKeyData] || (grp[fKeyData] = []);

            t.push(lookup[i]);
        }

        for (i = 0; i < source.length; i++) {
            source[i][from] = grp[ej.getObject(pKey || fKey, source[i])];
        }
    };

    var oData = {
        accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
        multipartAccept: "multipart/mixed",
        batch: "$batch",
        changeSet: "--changeset_",
        batchPre: "batch_",
        contentId: "Content-Id: ",
        batchContent: "Content-Type: multipart/mixed; boundary=",
        changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
        batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
    };
    var p = {
        parseJson: function (jsonText) {
            var type = typeof jsonText;
            if (type === "string") {
                jsonText = JSON.parse(jsonText, p.jsonReviver);
            } else if (jsonText instanceof Array) {
                p.iterateAndReviveArray(jsonText);
            } else if (type === "object")
                p.iterateAndReviveJson(jsonText);
            return jsonText;
        },
        iterateAndReviveArray: function (array) {
            for (var i = 0; i < array.length; i++) {
                if (typeof array[i] === "object")
                    p.iterateAndReviveJson(array[i]);
                else if (typeof array[i] === "string" && !/^[\s]*\[|^[\s]*\{|\"/g.test(array[i]))
                    array[i] = p.jsonReviver("",array[i]);
                else
                    array[i] = p.parseJson(array[i]);
            }
        },
        iterateAndReviveJson: function (json) {
            var value;

            for (var prop in json) {
                if (prop.startsWith("__"))
                    continue;

                value = json[prop];
                if (typeof value === "object") {
                    if (value instanceof Array)
                        p.iterateAndReviveArray(value);
                    else
                        p.iterateAndReviveJson(value);
                } else
                    json[prop] = p.jsonReviver(prop, value);
            }
        },
        jsonReviver: function (field, value) {
            var s = value;
            if (typeof value === "string") {
                var ms = /^\/Date\(([+-]?[0-9]+)([+-][0-9]{4})?\)\/$/.exec(value);
                if (ms)
                    return p.replacer(new Date(parseInt(ms[1])));
                else if (/^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*){1})([zZ]|([+\-])(\d\d):?(\d\d))?$/.test(value)) {
                    value = p.replacer(new Date(value));
                    if (isNaN(value)) {
                        var a = s.split(/[^0-9]/);
                        value = p.replacer(new Date(a[0], a[1] - 1, a[2], a[3], a[4], a[5]));
                    }
                }
            }

            return value;
        },
        isJson: function (jsonData) {
            if(typeof jsonData[0]== "string")
                return jsonData;
            return ej.parseJSON(jsonData);
        },
        isGuid: function (value) {
            var regex = /[A-Fa-f0-9]{8}(?:-[A-Fa-f0-9]{4}){3}-[A-Fa-f0-9]{12}/i;
            var match = regex.exec(value);
            return match != null;
        },
        replacer: function (value) {

            if (ej.isPlainObject(value))
                return p.jsonReplacer(value);

            if (value instanceof Array)
                return p.arrayReplacer(value);

            if (value instanceof Date)
                return p.jsonReplacer({ val: value }).val;

            return value;
        },
        jsonReplacer: function (val) {
            var value;
            for (var prop in val) {
                value = val[prop];

                if (!(value instanceof Date))
                    continue;
                val[prop] = new Date(+value + (ej.serverTimezoneOffset * 60 * 60 * 1000));
            }

            return val;
        },
        arrayReplacer: function (val) {

            for (var i = 0; i < val.length; i++) {            
                if (ej.isPlainObject(val[i]))
                    val[i] = p.jsonReplacer(val[i]);
                else if (val[i] instanceof Date)
                    val[i] = p.jsonReplacer({ date: val[i] }).date;
            }

            return val;
        }
    };

    ej.isJSON = p.isJson;
    ej.parseJSON = p.parseJson;
    ej.dateParse = true;
    ej.isGUID = p.isGuid;
    ej.Query = function (from) {
        if (!(this instanceof ej.Query))
            return new ej.Query(from);

        this.queries = [];
        this._key = "";
        this._fKey = "";

        if (typeof from === "string")
            this._fromTable = from || "";
        else if (from && from instanceof Array)
            this._lookup = from;

        this._expands = [];
        this._sortedColumns = [];
        this._groupedColumns = [];
        this._subQuery = null;
        this._isChild = false;
        this._params = [];
        return this;
    };

    ej.Query.prototype = {
        key: function (field) {
            if (typeof field === "string")
                this._key = field;

            return this;
        },
	
        using: function (dataManager) {
            if (dataManager instanceof ej.DataManager) {
                this.dataManagar = dataManager;
                return this;
            }

            return throwError("Query - using() : 'using' function should be called with parameter of instance ej.DataManager");
        },
	
        execute: function (dataManager, done, fail, always) {
            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeQuery(this, done, fail, always);

            return throwError("Query - execute() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },
	
        executeLocal: function (dataManager) {
            // this does not support for URL binding
            

            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeLocal(this);

            return throwError("Query - executeLocal() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },
	
        clone: function () {
            var cl = new ej.Query();
            cl.queries = this.queries.slice(0);
            cl._key = this._key;
            cl._isChild = this._isChild;
            cl.dataManagar = this.dataManager;
            cl._fromTable = this._fromTable;
            cl._params = this._params.slice(0);
            cl._expands = this._expands.slice(0);
            cl._sortedColumns = this._sortedColumns.slice(0);
            cl._groupedColumns = this._groupedColumns.slice(0);
            cl._subQuerySelector = this._subQuerySelector;
            cl._subQuery = this._subQuery;
            cl._fKey = this._fKey;
            cl._requiresCount = this._requiresCount;
            return cl;
        },
	
        from: function (tableName) {
            if (typeof tableName === "string")
                this._fromTable = tableName;

            return this;
        },
	
        addParams: function (key, value) {
            if (typeof value !== "function" && !ej.isPlainObject(value))
                this._params.push({ key: key, value: value });
            else if (typeof value === "function")
                this._params.push({ key: key, fn: value });

            return this;
        },
	
        expand: function (tables) {
            if (typeof tables === "string")
                this._expands = [].slice.call(arguments, 0);
            else
                this._expands = tables.slice(0);

            return this;
        },
	
        where: function (fieldName, operator, value, ignoreCase) {
            operator = (operator || ej.FilterOperators.equal).toLowerCase();
            var predicate = null;

            if (typeof fieldName === "string")
                predicate = new ej.Predicate(fieldName, operator, value, ignoreCase);
            else if (fieldName instanceof ej.Predicate)
                predicate = fieldName;
            else
                throwError("Query - where : Invalid arguments");

            this.queries.push({
                fn: "onWhere",
                e: predicate
            });
            return this;
        },
	
        search: function (searchKey, fieldNames, operator, ignoreCase) {
            if (!fieldNames || typeof fieldNames === "boolean") {
                fieldNames = [];
                ignoreCase = fieldNames;
            } else if (typeof fieldNames === "string")
                fieldNames = [fieldNames];

            if (typeof operator === "boolean") {
                ignoreCase = operator;
                operator = null;
            }
            operator = operator || ej.FilterOperators.contains;
            if (operator.length < 3)
                operator = ej.data.operatorSymbols[operator];

            var comparer = ej.data.fnOperators[operator] || ej.data.fnOperators.processSymbols(operator);

            this.queries.push({
                fn: "onSearch",
                e: {
                    fieldNames: fieldNames,
                    operator: operator,
                    searchKey: searchKey,
                    ignoreCase: ignoreCase,
                    comparer: comparer
                }
            });
            return this;
        },
		
        sortBy: function (fieldName, comparer, isFromGroup) {
            var order = ej.sortOrder.Ascending, sorts, t;

            if (typeof fieldName === "string" && fieldName.toLowerCase().endsWith(" desc")) {
                fieldName = fieldName.replace(/ desc$/i, '');
                comparer = ej.sortOrder.Descending;
            }
            if (fieldName instanceof Array) {
                for(var i=0;i<fieldName.length;i++)
                   this.sortBy(fieldName[i],comparer,isFromGroup);
                return this;
            }
            if (typeof comparer === "boolean")
                comparer = !comparer ? ej.sortOrder.Ascending : ej.sortOrder.Descending;
            else if (typeof comparer === "function")
                order = "custom";

            if (!comparer || typeof comparer === "string") {
                order = comparer ? comparer.toLowerCase() : ej.sortOrder.Ascending;
                comparer = ej.pvt.fnSort(comparer);
            }
            if (isFromGroup) {
                sorts = filterQueries(this.queries, "onSortBy");

                for (var i = 0; i < sorts.length; i++) {
                    t = sorts[i].e.fieldName;
                    if (typeof t === "string") {
                        if (t === fieldName) return this;
                    } else if (t instanceof Array) {
                        for (var j = 0; j < t.length; j++)
                            if (t[j] === fieldName || fieldName.toLowerCase() === t[j] + " desc")
                                return this;
                    }
                }
            }

            this.queries.push({
                fn: "onSortBy",
                e: {
                    fieldName: fieldName,
                    comparer: comparer,
                    direction: order
                }
            });

            return this;
        },
		
        sortByDesc: function (fieldName) {
            return this.sortBy(fieldName, ej.sortOrder.Descending);
        },
		
        group: function (fieldName,fn) {
            this.sortBy(fieldName, null, true);

            this.queries.push({
                fn: "onGroup",
                e: {
                    fieldName: fieldName,
                    fn: fn
                }
            });
            return this;
        },
	
        page: function (pageIndex, pageSize) {
            this.queries.push({
                fn: "onPage",
                e: {
                    pageIndex: pageIndex,
                    pageSize: pageSize
                }
            });
            return this;
        },
	
        range: function (start, end) {
            if (typeof start !== "number" || typeof end !== "number")
                throwError("Query() - range : Arguments type should be a number");

            this.queries.push({
                fn: "onRange",
                e: {
                    start: start,
                    end: end
                }
            });
            return this;
        },
	

        take: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Take : Argument type should be a number");

            this.queries.push({
                fn: "onTake",
                e: {
                    nos: nos
                }
            });
            return this;
        },
	
        skip: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Skip : Argument type should be a number");

            this.queries.push({
                fn: "onSkip",
                e: { nos: nos }
            });
            return this;
        },
	
        select: function (fieldNames) {
            if (typeof fieldNames === "string")
                fieldNames = [].slice.call(arguments, 0);

            if (!(fieldNames instanceof Array)) {
                throwError("Query() - Select : Argument type should be String or Array");
            }

            this.queries.push({
                fn: "onSelect",
                e: { fieldNames: fieldNames }
            });
            return this;
        },
	
        hierarchy: function (query, selectorFn) {
            if (!query || !(query instanceof ej.Query))
                throwError("Query() - hierarchy : query must be instance of ej.Query");

            if (typeof selectorFn === "function")
                this._subQuerySelector = selectorFn;

            this._subQuery = query;
            return this;
        },
	
        foreignKey: function (key) {
            if (typeof key === "string")
                this._fKey = key;

            return this;
        },
	
        requiresCount: function () {
            this._requiresCount = true;

            return this;
        },
        //type - sum, avg, min, max
        aggregate: function (type, field) {
            this.queries.push({
                fn: "onAggregates",
                e: { field: field, type: type }
            });
        }
    };

    ej.Adaptor = function (ds) {
        this.dataSource = ds;
        this.pvt = {};
		this.init.apply(this, [].slice.call(arguments, 1));
    };

    ej.Adaptor.prototype = {
        options: {
            from: "table",
            requestType: "json",
            sortBy: "sorted",
            select: "select",
            skip: "skip",
            group: "group",
            take: "take",
            search: "search",
            count: "requiresCounts",
            where: "where",
            aggregates: "aggregates"
        },
        init: function () {
        },
        extend: function (overrides) {
            var fn = function (ds) {
                this.dataSource = ds;

                if (this.options)
                    this.options = $.extend({}, this.options);
				this.init.apply(this, [].slice.call(arguments, 0));

                this.pvt = {};
            };
            fn.prototype = new this.type();
            fn.prototype.type = fn;

            var base = fn.prototype.base = {};
            for (var p in overrides) {
                if (fn.prototype[p])
                    base[p] = fn.prototype[p];
            }
            $.extend(true, fn.prototype, overrides);
            return fn;
        },
        processQuery: function (dm, query) {
            // this needs to be overridden
        },
        processResponse: function (data, ds, query, xhr) {
            if (data.d)
               return data.d;
            return data;
        },
        convertToQueryString: function (req, query, dm) {
            return $.param(req);
        },
        type: ej.Adaptor
    };

    ej.UrlAdaptor = new ej.Adaptor().extend({
        processQuery: function (dm, query, hierarchyFilters) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
                aggregates = filterQueries(query.queries, "onAggregates"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params,
                url = dm.dataSource.url, tmp, skip, take = null,
                op = this.options;

            var r = {
                sorted: [],
                grouped: [],
                filters: [],
                searches: [],
                aggregates: []
            };

            // calc Paging & Range
            if (singles["onPage"]) {
                tmp = singles["onPage"];
                skip = getValue(tmp.pageIndex, query);
                take = getValue(tmp.pageSize, query);
				skip = (skip - 1) * take;
            } else if (singles["onRange"]) {
                tmp = singles["onRange"];
                skip = tmp.start;
                take = tmp.end - tmp.start;
            }

            // Sorting
            for (var i = 0; i < sorted.length; i++) {
                tmp = getValue(sorted[i].e.fieldName, query);

                r.sorted.push(callAdaptorFunc(this, "onEachSort", { name: tmp, direction: sorted[i].e.direction }, query));
            }

            // hierarchy
            if (hierarchyFilters) {
                tmp = this.getFiltersFrom(hierarchyFilters, query);
                if (tmp)
                    r.filters.push(callAdaptorFunc(this, "onEachWhere", tmp.toJSON(), query));
            }

            // Filters
            for (var i = 0; i < filters.length; i++) {
                r.filters.push(callAdaptorFunc(this, "onEachWhere", filters[i].e.toJSON(), query));

                for (var prop in r.filters[i]) {
                    if (isNull(r[prop]))
                        delete r[prop];
                }
            }

            // Searches
            for (var i = 0; i < searchs.length; i++) {
                tmp = searchs[i].e;
                r.searches.push(callAdaptorFunc(this, "onEachSearch", {
                    fields: tmp.fieldNames,
                    operator: tmp.operator,
                    key: tmp.searchKey,
                    ignoreCase: tmp.ignoreCase
                }, query));
            }

            // Grouping
            for (var i = 0; i < grouped.length; i++) {
                r.grouped.push(getValue(grouped[i].e.fieldName, query));
            }

            // aggregates
            for (var i = 0; i < aggregates.length; i++) {
                tmp = aggregates[i].e; 
                r.aggregates.push({ type: tmp.type, field: getValue(tmp.field, query) });
            }

            var req = {};
            req[op.from] = query._fromTable;
            if (op.expand) req[op.expand] = query._expands;
            req[op.select] = singles["onSelect"] ? callAdaptorFunc(this, "onSelect", getValue(singles["onSelect"].fieldNames, query), query) : "";
            req[op.count] = query._requiresCount ? callAdaptorFunc(this, "onCount", query._requiresCount, query) : "";
            req[op.search] = r.searches.length ? callAdaptorFunc(this, "onSearch", r.searches, query) : "";
            req[op.skip] = singles["onSkip"] ? callAdaptorFunc(this, "onSkip", getValue(singles["onSkip"].nos, query), query) : "";
            req[op.take] = singles["onTake"] ? callAdaptorFunc(this, "onTake", getValue(singles["onTake"].nos, query), query) : "";
            req[op.where] = r.filters.length || r.searches.length ? callAdaptorFunc(this, "onWhere", r.filters, query) : "";
            req[op.sortBy] = r.sorted.length ? callAdaptorFunc(this, "onSortBy", r.sorted, query) : "";
            req[op.group] = r.grouped.length ? callAdaptorFunc(this, "onGroup", r.grouped, query) : "";
            req[op.aggregates] = r.aggregates.length ? callAdaptorFunc(this, "onAggregates", r.aggregates, query) : "";
			req["param"] = [];
			
            // Params
			callAdaptorFunc(this, "addParams", { dm: dm, query: query, params: params, reqParams: req });

            // cleanup
            for (var prop in req) {
                if (isNull(req[prop]) || req[prop] === "" || req[prop].length === 0 || prop === "params")
                    delete req[prop];
            }

            if (!(op.skip in req && op.take in req) && take !== null) {
                req[op.skip] = callAdaptorFunc(this, "onSkip", skip, query);
                req[op.take] = callAdaptorFunc(this, "onTake", take, query);
            }
            var p = this.pvt;
            this.pvt = {};

            if (this.options.requestType === "json") {
                return {
                    data: JSON.stringify(req),
                    url: url,
                    ejPvtData: p,
                    type: "POST",
                    contentType: "application/json; charset=utf-8"
                }
            }
            tmp = this.convertToQueryString(req, query, dm);
            tmp =  (dm.dataSource.url.indexOf("?")!== -1 ? "&" : "/") + tmp;
            return {
                type: "GET",
                url: tmp.length ? url.replace(/\/*$/, tmp) : url,
                ejPvtData: p
            };
        },
        convertToQueryString: function (req, query, dm) {
            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1)
                return $.param(req);
            return "?" + $.param(req);
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request.ejPvtData || {};
			var groupDs= data.groupDs;
			if (xhr && xhr.getResponseHeader("Content-Type") && xhr.getResponseHeader("Content-Type").indexOf("xml") != -1 && data.nodeType == 9)
                return query._requiresCount ? { result: [], count: 0 } : [];
            var d = JSON.parse(request.data);
            if (d && d.action === "batch" && data.added) {
                changes.added = data.added;
                return changes;
            }
            if (data.d)
                data = data.d;

            if (pvt && pvt.aggregates && pvt.aggregates.length) {
                var agg = pvt.aggregates, args = {}, fn, res = {};
                if ('count' in data) args.count = data.count;
                if (data["result"]) args.result = data.result;
                if (data["aggregate"]) data = data.aggregate;
                for (var i = 0; i < agg.length; i++) {
                    fn = ej.aggregates[agg[i].type];
                    if (fn)
                        res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
                }
                args["aggregates"] = res;
                data = args;
            }

            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups, args = {};
                if ('count' in data) args.count = data.count;
                if (data["aggregates"]) args.aggregates = data.aggregates;
                if (data["result"]) data = data.result;
                for (var i = 0; i < groups.length; i++){
                    var level = null;
                    var format = getColFormat(groups[i], query.queries);
                    if (!ej.isNullOrUndefined(groupDs))
                        groupDs = ej.group(groupDs, groups[i], null, format);
                    data = ej.group(data, groups[i], pvt.aggregates, format, level, groupDs);
                }
                if (args.count != undefined)
                    args.result = data;
                else
                    args = data;
                return args;
            }
            return data;
        },
        onGroup: function (e) {
            this.pvt.groups = e;
        },
        onAggregates: function (e) {
            this.pvt.aggregates = e;
        },
        batchRequest: function (dm, changes, e, query) {
            var res = {
                changed: changes.changed,
                added: changes.added,
                deleted: changes.deleted,
                action: "batch",
                table: e.url,
                key: e.key
            };
            if (query)
                this.addParams({ dm: dm, query: query, params: query._params, reqParams: res });

            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.removeUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify(res)
            };
        },
        beforeSend: function (dm, request) {
        },
        insert: function (dm, data, tableName, query) {
            var res = {
                value: data,
                table: tableName,
                action: "insert"
            };
            if (query)
                this.addParams({ dm: dm, query: query, params: query._params, reqParams: res });

            return {
                url: dm.dataSource.insertUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify(res)
            };
        },
        remove: function (dm, keyField, value, tableName, query) {
            var res = {
                key: value,
                keyColumn: keyField,
                table: tableName,
                action: "remove"
            };
            if (query)
                this.addParams({ dm: dm, query: query, params: query._params, reqParams: res });

            return {
                type: "POST",
                url: dm.dataSource.removeUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify(res)
            };
        },
        update: function (dm, keyField, value, tableName, query) {
            var res = {
                value: value,
                action: "update",
                keyColumn: keyField,
                key: value[keyField],
                table: tableName
            };
            if (query)
                this.addParams({ dm: dm, query: query, params: query._params, reqParams: res });

            return {
                type: "POST",
                url: dm.dataSource.updateUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify(res)
            };
        },
        getFiltersFrom: function (data, query) {
            if (!(data instanceof Array) || !data.length)
                throwError("ej.SubQuery: Array of key values required");
            var key = query._fKey, value, prop = key, pKey = query._key, predicats = [],
                isValues = typeof data[0] !== "object";

            if (typeof data[0] !== "object") prop = null;

            for (var i = 0; i < data.length; i++) {
                value = !isValues ? ej.pvt.getObject(pKey || prop, data[i]) : data[i];
                predicats.push(new ej.Predicate(key, "==", value));
            }

            return ej.Predicate.or(predicats);
        },
        addParams: function (options) {
            var dm = options.dm, query = options.query, params = options.params, req = options.reqParams; req["params"] = {};
            for (var i = 0, tmp; tmp = params[i]; i++) {
                if (req[tmp.key]) throwError("ej.Query: Custom Param is conflicting other request arguments");
                req[tmp.key] = tmp.value;
                if (tmp.fn)
                    req[tmp.key] = tmp.fn.call(query, tmp.key, query, dm);                
                req["params"][tmp.key] = req[tmp.key];
            }
        }
    });
    ej.WebMethodAdaptor = new ej.UrlAdaptor().extend({
        processQuery: function (dm, query, hierarchyFilters) {
            var obj = ej.UrlAdaptor.prototype.processQuery(dm, query, hierarchyFilters);
            var data = ej.parseJSON(obj.data), result = {};

            result["value"] = data;

            //Params             
            callAdaptorFunc(this, "addParams", { dm: dm, query: query, params: query._params, reqParams: result });

            return {
                data: JSON.stringify(result),
                url: obj.url,
                ejPvtData: obj.ejPvtData,
                type: "POST",
                contentType: "application/json; charset=utf-8"
            }
        },
        addParams: function (options) {
            var dm = options.dm, query = options.query, params = options.params, req = options.reqParams; req["params"] = {};
            for (var i = 0, tmp; tmp = params[i]; i++) {
                if (req[tmp.key]) throwError("ej.Query: Custom Param is conflicting other request arguments");
                var webkey = tmp.key, webvalue = tmp.value;
                if (tmp.fn)
                    webvalue = tmp.fn.call(query, tmp.key, query, dm);
                req[webkey] = webvalue;
                req["params"][webkey] = req[webkey];
            }
        }
    });
    ej.CacheAdaptor = new ej.UrlAdaptor().extend({
        init: function (adaptor, timeStamp, pageSize) {
            if (!ej.isNullOrUndefined(adaptor)) {
                this.cacheAdaptor = adaptor;
            }
            this.pageSize = pageSize;
            this.guidId = ej.getGuid("cacheAdaptor");
            var obj = { keys: [], results: [] };
            if (window.localStorage)
                window.localStorage.setItem(this.guidId, JSON.stringify(obj));
            var guid = this.guidId;
            if (!ej.isNullOrUndefined(timeStamp)) {
                setInterval(function () {
                    var data = ej.parseJSON(window.localStorage.getItem(guid));
                    var forDel = [];
                    for (var i = 0; i < data.results.length; i++) {
                        data.results[i].timeStamp = new Date() - new Date(data.results[i].timeStamp)
                        if (new Date() - new Date(data.results[i].timeStamp) > timeStamp)
                            forDel.push(i);
                    }
                    var d = forDel;
                    for (var i = 0; i < forDel.length; i++) {
                        data.results.splice(forDel[i], 1);
                        data.keys.splice(forDel[i], 1);
                    }
                    window.localStorage.removeItem(guid);
                    window.localStorage.setItem(guid, JSON.stringify(data));
                }, timeStamp);
            }
        },
        generateKey: function (url, query) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
				pageQuery = filterQueries(query.queries, "onPage"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params;
            var key = url;
            if (singles["onPage"])
              key += singles["onPage"].pageIndex;
              sorted.forEach(function (obj) {
                   key += obj.e.direction + obj.e.fieldName;
              });
                grouped.forEach(function (obj) {
                    key += obj.e.fieldName;
                });
                searchs.forEach(function (obj) {
                    key += obj.e.searchKey;
                });
            
            for (var filter = 0; filter < filters.length; filter++) {
                var currentFilter = filters[filter];
                if (currentFilter.e.isComplex) {
                    var newQuery = query.clone();
                    newQuery.queries = [];
                    for (var i = 0; i < currentFilter.e.predicates.length; i++) {
                        newQuery.queries.push({ fn: "onWhere", e: currentFilter.e.predicates[i], filter: query.queries.filter });
                    }
                    key += currentFilter.e.condition + this.generateKey(url, newQuery);
                }
                else
                    key += currentFilter.e.field + currentFilter.e.operator + currentFilter.e.value
            }
            return key;
        },
        processQuery: function (dm, query, hierarchyFilters) {
            var key = this.generateKey(dm.dataSource.url, query);
            var cachedItems;
            if (window.localStorage)
                cachedItems = ej.parseJSON(window.localStorage.getItem(this.guidId));
            var data = cachedItems ? cachedItems.results[cachedItems.keys.indexOf(key)] : null;
            if (data != null && !this._crudAction && !this._insertAction) {
                return data;
            }
            this._crudAction = null; this._insertAction = null;
            return this.cacheAdaptor.processQuery.apply(this.cacheAdaptor, [].slice.call(arguments, 0))
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            if (this._insertAction || (request && this.cacheAdaptor.options.batch && request.url.endsWith(this.cacheAdaptor.options.batch) && request.type.toLowerCase() === "post")) {
                return this.cacheAdaptor.processResponse(data, ds, query, xhr, request, changes);
            }
            var data = this.cacheAdaptor.processResponse.apply(this, [].slice.call(arguments, 0));
            var key = this.generateKey(ds.dataSource.url, query)
            var obj = {};
            if (window.localStorage)
                obj = ej.parseJSON(window.localStorage.getItem(this.guidId));
            var index = $.inArray(key, obj.keys);
            if (index != -1) {
                obj.results.splice(index, 1);
                obj.keys.splice(index, 1);
            }
            obj.results[obj.keys.push(key) - 1] = { keys: key, result: data.result, timeStamp: new Date(), count: data.count }
            while (obj.results.length > this.pageSize) {
                obj.results.splice(0, 1);
                obj.keys.splice(0, 1);
            }
            window.localStorage.setItem(this.guidId, JSON.stringify(obj));
            return data;
        },
        update: function (dm, keyField, value, tableName) {
            this._crudAction = true;
            return this.cacheAdaptor.update(dm, keyField, value, tableName);
        },
        insert: function (dm, data, tableName) {
            this._insertAction = true;
            return this.cacheAdaptor.insert(dm, data, tableName);
        },
        remove: function (dm, keyField, value, tableName) {
            this._crudAction = true;
            return this.cacheAdaptor.remove(dm, keyField, value, tableName);
        },
        batchRequest: function (dm, changes, e) {
            return this.cacheAdaptor.batchRequest(dm, changes, e);
        }
    });
    var filterQueries = function (queries, name) {
        return queries.filter(function (q) {
            return q.fn === name;
        }) || [];
    };
    var filterQueryLists = function (queries, singles) {
        var filtered = queries.filter(function (q) {
            return singles.indexOf(q.fn) !== -1;
        }), res = {};
        for (var i = 0; i < filtered.length; i++) {
            if (!res[filtered[i].fn])
                res[filtered[i].fn] = filtered[i].e;
        }
        return res;
    };
    var callAdaptorFunc = function (obj, fnName, param, param1) {
        if (obj[fnName]) {
            var res = obj[fnName](param, param1);
            if (!isNull(res)) param = res;
        }
        return param;
    };

    ej.ODataAdaptor = new ej.UrlAdaptor().extend({
        options: {
            requestType: "get",
            accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
            multipartAccept: "multipart/mixed",
            sortBy: "$orderby",
            select: "$select",
            skip: "$skip",
            take: "$top",
            count: "$inlinecount",
            where: "$filter",
            expand: "$expand",
            batch: "$batch",
            changeSet: "--changeset_",
            batchPre: "batch_",
            contentId: "Content-Id: ",
            batchContent: "Content-Type: multipart/mixed; boundary=",
            changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
            batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
        },
        onEachWhere: function (filter, requiresCast) {
            return filter.isComplex ? this.onComplexPredicate(filter, requiresCast) : this.onPredicate(filter, requiresCast);
        },
        onPredicate: function (pred, query, requiresCast) {
            var returnValue = "",
                operator,guid,
                val = pred.value,
                type = typeof val,
                field = this._p(pred.field);

            if (val instanceof Date)
                val = "datetime'" + p.replacer(val).toJSON() + "'";

            if (type === "string") {
				if(val.indexOf("'") != -1)
                    val = val.replace(new RegExp(/'/g),"''");
                val = "'" + val + "'";

                if (requiresCast) {
                    field = "cast(" + field + ", 'Edm.String')";
                }
                if (ej.isGUID(val))
                    guid = 'guid';
                if (pred.ignoreCase) {
                    !guid ? field = "tolower(" + field + ")" : field;
                    val = val.toLowerCase();
                }
            }

            operator = ej.data.odBiOperator[pred.operator];
            if (operator) {
                returnValue += field;
                returnValue += operator;
                if (guid)
                    returnValue += guid;
                return returnValue + val;
            }

            operator = ej.data.odUniOperator[pred.operator];
            if (!operator || type !== "string") return "";

            if (operator === "substringof") {
                var t = val;
                val = field;
                field = t;
            }

            returnValue += operator + "(";
            returnValue += field + ",";
            if (guid) returnValue += guid;
            returnValue += val + ")";

            return returnValue;
        },
        onComplexPredicate: function (pred, requiresCast) {
            var res = [];
            for (var i = 0; i < pred.predicates.length; i++) {
                res.push("(" + this.onEachWhere(pred.predicates[i], requiresCast) + ")");
            }
            return res.join(" " + pred.condition + " ");
        },
        onWhere: function (filters) {
            if (this.pvt.searches)
                filters.push(this.onEachWhere(this.pvt.searches, null, true));

            return filters.join(" and ");
        },
        onEachSearch: function (e) {
            if (e.fields.length === 0)
                throwError("Query() - Search : oData search requires list of field names to search");

            var filter = this.pvt.searches || [];
            for (var i = 0; i < e.fields.length; i++) {
                filter.push(new ej.Predicate(e.fields[i], e.operator, e.key, e.ignoreCase));
            }
            this.pvt.searches = filter;
        },
        onSearch: function (e) {
            this.pvt.searches = ej.Predicate.or(this.pvt.searches);
            return "";
        },
        onEachSort: function (e) {
            var res = [];
            if (e.name instanceof Array) {
                for (var i = 0; i < e.name.length; i++)
                    res.push(this._p(e.name[i]));
            } else {
                res.push(this._p(e.name) + (e.direction === "descending" ? " desc" : ""));
            }
            return res.join(",");
        },
        onSortBy: function (e) {
            return e.reverse().join(",");
        },
        onGroup: function (e) {
            this.pvt.groups = e;
            return "";
        },
        onSelect: function (e) {
            for (var i = 0; i < e.length; i++)
                e[i] = this._p(e[i]);

            return e.join(',');
        },
        onAggregates: function(e){
            this.pvt.aggregates = e;
            return "";
        },
        onCount: function (e) {
            return e === true ? "allpages" : "";
        },
        beforeSend: function (dm, request, settings) {
            if (settings.url.endsWith(this.options.batch) && settings.type.toLowerCase() === "post") {
                request.setRequestHeader("Accept", oData.multipartAccept);
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.overrideMimeType("text/plain; charset=x-user-defined");
            }

            if (!dm.dataSource.crossDomain) {
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.setRequestHeader("MaxDataServiceVersion", "2.0");
            }
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            if (!ej.isNullOrUndefined(data.d)) {
                var dataCopy = (query && query._requiresCount) ? data.d.results : data.d;
                if (!ej.isNullOrUndefined(dataCopy))
                    for (var i = 0; i < dataCopy.length; i++) {
                        !ej.isNullOrUndefined(dataCopy[i].__metadata) && delete dataCopy[i].__metadata;
                    }
            }
            var pvt = request && request.ejPvtData;
            if (xhr && xhr.getResponseHeader("Content-Type") && xhr.getResponseHeader("Content-Type").indexOf("xml") != -1 && data.nodeType == 9)
                return query._requiresCount ? { result: [], count: 0 } : [];
            if (request && this.options.batch && request.url.endsWith(this.options.batch) && request.type.toLowerCase() === "post") {
                var guid = xhr.getResponseHeader("Content-Type"), cIdx, jsonObj;
                guid = guid.substring(guid.indexOf("=batchresponse") + 1);
                data = data.split(guid);
                if (data.length < 2) return;

                data = data[1];
                var exVal = /(?:\bContent-Type.+boundary=)(changesetresponse.+)/i.exec(data);
                data.replace(exVal[0], "");

                var changeGuid = exVal[1];
                data = data.split(changeGuid);

                for (var i = data.length; i > -1; i--) {
                    if (!/\bContent-ID:/i.test(data[i]) || !/\bHTTP.+201/.test(data[i]))
                        continue;

                    cIdx = parseInt(/\bContent-ID: (\d+)/i.exec(data[i])[1]);

                    if (changes.added[cIdx]) {
                        jsonObj = p.parseJson(/^\{.+\}/m.exec(data[i])[0]);
                        $.extend(changes.added[cIdx], this.processResponse(jsonObj));
                    }
                }
                return changes;
            }
            var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null, aggregateResult = {};
            version = (version && parseInt(version, 10)) || 2;

            if (query && query._requiresCount) {
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
                if (data.d) data = data.d;
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
            }

            if (version === 3 && data.value) data = data.value;
            if (data.d) data = data.d;
            if (version < 3 && data.results) data = data.results;

            if (pvt && pvt.aggregates && pvt.aggregates.length) {
                var agg = pvt.aggregates, args = {}, fn, res = {};
                for (var i = 0; i < agg.length; i++) {
                    fn = ej.aggregates[agg[i].type];
                    if (fn)
                        res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
                }
                aggregateResult = res;
            }
            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups;
                for (var i = 0; i < groups.length; i++) {
                    var format = getColFormat(groups[i], query.queries)
                    data = ej.group(data, groups[i], pvt.aggregates, format);
                }
            }
            return isNull(count) ? data : { result: data, count: count, aggregates: aggregateResult };
        },
        convertToQueryString: function (req, query, dm) {
            var res = [], tableName = req.table || "";
            delete req.table;

            if (dm.dataSource.requiresFormat)
                req["$format"] = "json";

            for (var prop in req)
                res.push(prop + "=" + req[prop]);

            res = res.join("&");

            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1 && !tableName)
                return res;

            return res.length ? tableName + "?" + res : tableName || "";
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : ''),
                data: JSON.stringify(data)
            }
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value + ')'
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value[keyField] + ')',
                data: JSON.stringify(value),
                accept: this.options.accept
            };
        },
        batchRequest: function (dm, changes, e) {
            var initialGuid = e.guid = ej.getGuid(oData.batchPre);
            var url = dm.dataSource.url.replace(/\/*$/, '/' + this.options.batch);
            var args = {
                url: e.url,
                key: e.key,
                cid: 1,
                cSet: ej.getGuid(oData.changeSet)
            };
            var req = "--" + initialGuid + "\n";

            req += "Content-Type: multipart/mixed; boundary=" + args.cSet.replace("--", "") + "\n";

            this.pvt.changeSet = 0;

            req += this.generateInsertRequest(changes.added, args);
            req += this.generateUpdateRequest(changes.changed, args);
            req += this.generateDeleteRequest(changes.deleted, args);

            req += args.cSet + "--\n";
            req += "--" + initialGuid + "--";

            return {
                type: "POST",
                url: url,
                contentType: "multipart/mixed; charset=UTF-8;boundary=" + initialGuid,
                data: req
            };
        },
        generateDeleteRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "DELETE ";
                req += e.url + "(" + arr[i][e.key] + ") HTTP/1.1\n";
                req += "If-Match : * \n"
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + this.pvt.changeSet++ + "\n";
                req += oData.batchChangeSetContentType + "\n";
            }

            return req + "\n";
        },
        generateInsertRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "POST ";
                req += e.url + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + this.pvt.changeSet++ + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n";
            }

            return req;
        },
        generateUpdateRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "PUT ";
                req += e.url + "(" + arr[i][e.key] + ")" + " HTTP/1.1\n";
                req += "If-Match : * \n"
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + this.pvt.changeSet++ + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n\n";
            }

            return req;
        },
        _p: function (prop) {
            return prop.replace(/\./g, "/");
        }
    });
    ej.ODataV4Adaptor = new ej.ODataAdaptor().extend({
        options: {
            requestType: "get",
            accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
            multipartAccept: "multipart/mixed",
            sortBy: "$orderby",
            select: "$select",
            skip: "$skip",
            take: "$top",
            count: "$count",
            search: "$search",
            where: "$filter",
            expand: "$expand",
            batch: "$batch",
            changeSet: "--changeset_",
            batchPre: "batch_",
            contentId: "Content-Id: ",
            batchContent: "Content-Type: multipart/mixed; boundary=",
            changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
            batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
        },
        onCount: function (e) {
            return e === true ? "true" : "";
        },
        onPredicate: function (pred, query, requiresCast) {
            var returnValue = "",
                val = pred.value,
                isDate = val instanceof Date;               
            ej.data.odUniOperator["contains"] = "contains";
            returnValue = ej.ODataAdaptor.prototype.onPredicate.call(this, pred, query, requiresCast);
            ej.data.odUniOperator["contains"] = "substringof";
                if (isDate)
                    returnValue = returnValue.replace(/datetime'(.*)'$/, "$1");

            return returnValue;
        },
        onEachSearch: function (e) {
			 var search = this.pvt.search || [];
			 search.push(e.key);
			 this.pvt.search = search;
		},
		onSearch: function (e) {
			 return this.pvt.search.join(" OR ");
		},
        beforeSend: function (dm, request, settings) {
 
        },
        processQuery: function (ds, query) {
            var digitsWithSlashesExp = /\/[\d*\/]*/g;
            var poppedExpand = "";
            for (var i = query._expands.length - 1; i > 0; i--) {
                if (poppedExpand.indexOf(query._expands[i]) >= 0) { // If current expand is child of previous
                    query._expands.pop(); // Just remove it because its in the expand already
                }
                else {
                    if (digitsWithSlashesExp.test(query._expands[i])) { //If expanded to subentities
                        poppedExpand = query._expands.pop();
                        var r = poppedExpand.replace(digitsWithSlashesExp, "($expand="); //Rewrite into odata v4 expand
                        for (var j = 0; j < poppedExpand.split(digitsWithSlashesExp).length - 1; j++) {
                            r = r + ")"; // Add closing brackets
                        }
                        query._expands.unshift(r); // Add to the front of the array
                        i++;
                    }
                }
            }
            return ej.ODataAdaptor.prototype.processQuery.apply(this, [ds, query]);
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request && request.ejPvtData;
            if (xhr && xhr.getResponseHeader("Content-Type") && xhr.getResponseHeader("Content-Type").indexOf("xml") != -1 && data.nodeType == 9)
                return query._requiresCount ? { result: [], count: 0 } : [];
            if (request && this.options.batch && request.url.endsWith(this.options.batch) && request.type.toLowerCase() === "post") {
                var guid = xhr.getResponseHeader("Content-Type"), cIdx, jsonObj;
                guid = guid.substring(guid.indexOf("=batchresponse") + 1);
                data = data.split(guid);
                if (data.length < 2) return;

                data = data[1];
                var exVal = /(?:\bContent-Type.+boundary=)(changesetresponse.+)/i.exec(data);
                data.replace(exVal[0], "");

                var changeGuid = exVal[1];
                data = data.split(changeGuid);

                for (var i = data.length; i > -1; i--) {
                   if (!/\bContent-ID:/i.test(data[i]) || !/\bHTTP.+201/.test(data[i]))
                        continue;

                    cIdx = parseInt(/\bContent-ID: (\d+)/i.exec(data[i])[1]);

                    if (changes.added[cIdx]) {
                        jsonObj = p.parseJson(/^\{.+\}/m.exec(data[i])[0]);
                        $.extend(changes.added[cIdx], this.processResponse(jsonObj));
                    }
                }
                return changes;
           }
            var count = null, aggregateResult = {};
            if (query && query._requiresCount)
                if ('@odata.count' in data) count = data['@odata.count'];

            data = ej.isNullOrUndefined(data.value) ? data : data.value;
           if (pvt && pvt.aggregates && pvt.aggregates.length) {
               var agg = pvt.aggregates, args = {}, fn, res = {};
               for (var i = 0; i < agg.length; i++) {
                   fn = ej.aggregates[agg[i].type];
                   if (fn)
                       res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
               }
               aggregateResult = res;
           }
            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups;
                for (var i = 0; i < groups.length; i++) {
                    var format = getColFormat(groups[i], query.queries);
                    data = ej.group(data, groups[i], pvt.aggregates, format);
                }
            }
            return isNull(count) ? data : { result: data, count: count, aggregates: aggregateResult };
        },
    });
    ej.JsonAdaptor = new ej.Adaptor().extend({
        processQuery: function (ds, query) {
            var result = ds.dataSource.json.slice(0), count = result.length, cntFlg = true, ret, key, agg = {};

            for (var i = 0; i < query.queries.length; i++) {
                key = query.queries[i];
                ret = this[key.fn].call(this, result, key.e, query);
                if (key.fn == "onAggregates")
                    agg[key.e.field + " - " + key.e.type] = ret;
                else
                result = ret !== undefined ? ret : result;

                if (key.fn === "onPage" || key.fn === "onSkip" || key.fn === "onTake" || key.fn === "onRange") cntFlg = false;

                if (cntFlg) count = result.length;
            }

            if (query._requiresCount) {
                result = {
                    result: result,
                    count: count,
                    aggregates: agg
                };
            }

            return result;
        },
        batchRequest: function (dm, changes, e) {
            var i;
            for (i = 0; i < changes.added.length; i++)
                this.insert(dm, changes.added[i]);
            for (i = 0; i < changes.changed.length; i++)
                this.update(dm, e.key, changes.changed[i]);
            for (i = 0; i < changes.deleted.length; i++)
                this.remove(dm, e.key, changes.deleted[i]);
            return changes;
        },
        onWhere: function (ds, e) {
            if (!ds) return ds;

            return ds.filter(function (obj) {
                return e.validate(obj);
            });
        },
        onAggregates: function(ds, e){
            var fn = ej.aggregates[e.type];
            if (!ds || !fn || ds.length == 0) return null;
            return fn(ds, e.field);
        },
        onSearch: function (ds, e) {
            if (!ds || !ds.length) return ds;

            if (e.fieldNames.length === 0) {
                ej.pvt.getFieldList(ds[0], e.fieldNames);
            }

            return ds.filter(function (obj) {
                for (var j = 0; j < e.fieldNames.length; j++) {
                    if (e.comparer.call(obj, ej.pvt.getObject(e.fieldNames[j], obj), e.searchKey, e.ignoreCase))
                        return true;
                }
                return false;
            });
        },
        onSortBy: function (ds, e, query) {
            if (!ds) return ds;
            var fnCompare, field = getValue(e.fieldName, query);
            if (!field)
                return ds.sort(e.comparer);

            if (field instanceof Array) {
                field = field.slice(0);

                for (var i = field.length - 1; i >= 0; i--) {
                    if (!field[i]) continue;

                    fnCompare = e.comparer;

                    if (field[i].endsWith(" desc")) {
                        fnCompare = ej.pvt.fnSort(ej.sortOrder.Descending);
                        field[i] = field[i].replace(" desc", "");
                    }

                    ds = stableSort(ds, field[i], fnCompare, []);
                }
                return ds;
            }
            return stableSort(ds, field, e.comparer, query ? query.queries : []);
        },
        onGroup: function (ds, e, query) {
            if (!ds) return ds;
            var aggQuery = filterQueries(query.queries, "onAggregates"), agg = [];
            if (aggQuery.length) {
                var tmp;
                for (var i = 0; i < aggQuery.length; i++) {
                    tmp = aggQuery[i].e;
                    agg.push({ type: tmp.type, field: getValue(tmp.field, query) });
                }
            }
            var format = getColFormat(e.fieldName, query.queries);
            return ej.group(ds, getValue(e.fieldName, query), agg, format);
        },
        onPage: function (ds, e, query) {
            var size = getValue(e.pageSize, query),
                start = (getValue(e.pageIndex, query) - 1) * size, end = start + size;

            if (!ds) return ds;

            return ds.slice(start, end);
        },
        onRange: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.start), getValue(e.end));
        },
        onTake: function (ds, e) {
            if (!ds) return ds;

            return ds.slice(0, getValue(e.nos));
        },
        onSkip: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.nos));
        },
        onSelect: function (ds, e) {
            if (!ds) return ds;
            return ej.select(ds, getValue(e.fieldNames));
        },
        insert: function (dm, data) {
            return dm.dataSource.json.push(data);
        },
        remove: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i;
            if (typeof value === "object")
                value = ej.getObject(keyField, value);
            for (i = 0; i < ds.length; i++) {
                if (ej.getObject(keyField, ds[i]) === value) break;
            }

            return i !== ds.length ? ds.splice(i, 1) : null;
        },
        update: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i, key = ej.getObject(keyField, value);

            for (i = 0; i < ds.length; i++) {
                if (ej.getObject(keyField, ds[i]) === key) break;
            }

            return i < ds.length ? $.extend(ds[i], value) : null;
        }
    });
    ej.ForeignKeyAdaptor = function (data, type) {
        var foreignObj = new ej[type || "JsonAdaptor"]().extend({
            init: function () {
                this.foreignData = [];
                this.key = [];
                this.adaptorType = type;
                this.value = [];
                this.fValue = [];
                this.keyField = [];
                var dataObj = data;
                for (var i = 0; i < dataObj.length; i++) {
                    this.foreignData[i] = dataObj[i].dataSource;
                    this.key[i] = dataObj[i].foreignKeyField;
                    this.fValue[i] = ej.isNullOrUndefined(dataObj[i].field)? dataObj[i].foreignKeyValue : dataObj[i].field + "_" + dataObj[i].foreignKeyValue;
                    this.value[i] = dataObj[i].foreignKeyValue;
                    this.keyField[i] = dataObj[i].field || dataObj[i].foreignKeyField;
                    this.initial = true;
                }
            },
            processQuery: function (ds, query) {
                var data = ds.dataSource.json;
                if (this.initial) {
                    for (var i = 0; i < data.length; i++) {
                        var proxy = this;
                        for (var j = 0; j < this.foreignData.length; j++) {
                            this.foreignData[j].filter(function (col) { //filtering the foreignKey dataSource
                                if (ej.getObject(proxy.key[j], col) == ej.getObject(proxy.keyField[j], data[i]))
                                    data[i][proxy.fValue[j]] = ej.getObject(proxy.value[j], col);
                            });
                        }
                    }
                    this.initial = false;
                }
                return this.base.processQuery.apply(this, [ds, query]);
            },
            setValue: function (value) {
                for (var i = 0; i < this.foreignData.length; i++) {
                    var proxy = this;
                    var keyValue = value[this.fValue[i]];
                    if (typeof keyValue == "string" && !isNaN(keyValue))
                        keyValue = ej.parseFloat(keyValue);
                    var data = $.grep(proxy.foreignData[i], function (e) {
                        return e[proxy.value[i]] == keyValue;
                    })[0];
                    if (ej.isNullOrUndefined(data)) {
                        data = $.grep(proxy.foreignData[i], function (e) {
                            return e[proxy.key[i]] == keyValue;
                        })[0];
                        if (ej.getObject(this.value[i], data) != undefined)
                            ej.createObject(proxy.value[i], ej.getObject(this.value[i], data), value);
                    }
                    if (ej.getObject(this.value[i], data) != undefined)
                        ej.createObject(this.keyField[i], ej.getObject(this.key[i], data), value);
                }
            },
            insert: function (dm, data, tableName) {
                this.setValue(data);
                return {
                    url: dm.dataSource.insertUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                    data: JSON.stringify({
                        value: data,
                        table: tableName,
                        action: "insert"
                    })
                };
            },
            update: function (dm, keyField, value, tableName) {
                this.setValue(value);
                ej.JsonAdaptor.prototype.update(dm, keyField, value, tableName);
                return {
                    type: "POST",
                    url: dm.dataSource.updateUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                    data: JSON.stringify({
                        value: value,
                        action: "update",
                        keyColumn: keyField,
                        key: value[keyField],
                        table: tableName
                    })
                };
            }
        });
        $.extend(this, new foreignObj());
        return this;
    }
    ej.remoteSaveAdaptor = new ej.JsonAdaptor().extend({
        beforeSend: ej.UrlAdaptor.prototype.beforeSend,
        insert: ej.UrlAdaptor.prototype.insert,
        update: ej.UrlAdaptor.prototype.update,
        remove: ej.UrlAdaptor.prototype.remove,
        addParams: ej.UrlAdaptor.prototype.addParams,
        batchRequest: function (dm, changes, e, query) { 
			var res = {
                changed: changes.changed,
                added: changes.added,
                deleted: changes.deleted,
                action: "batch",
                table: e.url,
                key: e.key
            };
            if (query)
                this.addParams({ dm: dm, query: query, params: query._params, reqParams: res });
            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify(res)
            };
        },
         processResponse: function (data, ds, query, xhr, request, changes) {
			var i;
            for (i = 0; i < changes.added.length; i++)
                ej.JsonAdaptor.prototype.insert(dm, changes.added[i]);
            for (i = 0; i < changes.changed.length; i++)
                ej.JsonAdaptor.prototype.update(dm, e.key, changes.changed[i]);
            for (i = 0; i < changes.deleted.length; i++)
                ej.JsonAdaptor.prototype.remove(dm, e.key, changes.deleted[i]);
            return this.base.processResponse.apply(this, [data, ds, query, xhr, request, changes]);
        }
    });
    ej.WebApiAdaptor = new ej.ODataAdaptor().extend({
        insert: function (dm, data, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.url,
                data: JSON.stringify(data)
            };
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url + "/" + value,
                data: JSON.stringify(value)
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url,
                data: JSON.stringify(value)
            };
        },
		batchRequest: function (dm, changes, e) {
            var initialGuid = e.guid = ej.getGuid(oData.batchPre);
            var req = [];

		    //insertion 
		
			$.each(changes.added, function (i, d) {
			    req.push('--' + initialGuid);
			    req.push('Content-Type: application/http; msgtype=request', '');
			    req.push('POST' + ' ' + dm.dataSource.insertUrl + ' HTTP/1.1');
			    req.push('Content-Type: ' + 'application/json; charset=utf-8');
			    req.push('Host: ' + location.host);
			    req.push('', d ? JSON.stringify(d) : '');
			});
			
			//updation
			$.each(changes.changed, function (i, d) {
			    req.push('--' + initialGuid);
			    req.push('Content-Type: application/http; msgtype=request', '');
			    req.push('PUT' + ' ' + dm.dataSource.updateUrl + ' HTTP/1.1');
			    req.push('Content-Type: ' + 'application/json; charset=utf-8');
			    req.push('Host: ' + location.host);
			    req.push('', d ? JSON.stringify(d) : '');
			});
			
			//deletion
			$.each(changes.deleted, function (i, d) {
			    req.push('--' + initialGuid);
                req.push('Content-Type: application/http; msgtype=request', '');
                req.push('DELETE' + ' ' + dm.dataSource.removeUrl +"/"+ d[e.key] + ' HTTP/1.1');
                req.push('Content-Type: ' + 'application/json; charset=utf-8');
                req.push('Host: ' + location.host);
                req.push('', d ? JSON.stringify(d) : '');		
			});
			req.push('--' + initialGuid + '--', '');
            return {
				type: 'POST',
				url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: req.join('\r\n'),
                contentType: 'multipart/mixed; boundary="' + initialGuid + '"',
            };
        },
        processResponse: function (data, ds, query, xhr, request, changes) {

            var pvt = request && request.ejPvtData;
            if (request && request.type.toLowerCase() != "post") {
                var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null, aggregateResult = {};
                version = (version && parseInt(version, 10)) || 2;

                if (query && query._requiresCount) {
                     if (!isNull(data.Count)) count = data.Count;
                }

                if (version < 3 && data.Items) data = data.Items;

                if (pvt && pvt.aggregates && pvt.aggregates.length) {
                    var agg = pvt.aggregates, args = {}, fn, res = {};
                    for (var i = 0; i < agg.length; i++) {
                        fn = ej.aggregates[agg[i].type];
                        if (fn)
                            res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
                    }
                    aggregateResult = res;
                }
                if (pvt && pvt.groups && pvt.groups.length) {
                    var groups = pvt.groups;
                    for (var i = 0; i < groups.length; i++) {
                        var format = getColFormat(groups[i], query.queries);
                        data = ej.group(data, groups[i], pvt.aggregates, format);
                    }
                }
                return isNull(count) ? data : { result: data, count: count, aggregates: aggregateResult };
            }
        }
    });
    var getValue = function (value, inst) {
        if (typeof value === "function")
            return value.call(inst || {});
        return value;
    }

    ej.TableModel = function (name, jsonArray, dataManager, modelComputed) {
        if (!instance(this, ej.TableModel))
            return new ej.TableModel(jsonArray);

        if (!instance(jsonArray, Array))
            throwError("ej.TableModel - Json Array is required");

        var rows = [], model, dirtyFn = $proxy(setDirty, this);

        for (var i = 0; i < jsonArray.length; i++) {
            model = new ej.Model(jsonArray[i], this);
            model.state = "unchanged";
            model.on("stateChange", dirtyFn);
            if (modelComputed)
                model.computes(modelComputed);
            rows.push(model);
        }

        this.name = name || "table1";

        this.rows = ej.NotifierArray(rows);
        this._deleted = [];

        this._events = $({});

        this.dataManager = dataManager;

        this._isDirty = false;

        return this;
    };

    ej.TableModel.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },

        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },

        setDataManager: function (dataManager) {
            this.dataManagar = dataManager;
        },

        saveChanges: function () {
            if (!this.dataManager || !instance(this.dataManager, ej.DataManager))
                throwError("ej.TableModel - saveChanges : Set the dataManager using setDataManager function");

            if (!this.isDirty())
                return;

            var promise = this.dataManager.saveChanges(this.getChanges(), this.key, this.name);

            promise.done($proxy(function (changes) {
                var rows = this.toArray();
                for (var i = 0; i < rows.length; i++) {
                    if (rows.state === "added") {
                        rows.set(this.key, changes.added.filter(function (e) {
                            return e[this.key] === rows.get(this.key);
                        })[0][this.key]);
                    }
                    rows[i].markCommit();
                }

                this._events.triggerHandler({ type: "save", table: this });

            }, this));

            promise.fail($proxy(function (e) {
                this.rejectChanges();
                this._events.triggerHandler({ type: "reject", table: this, error: e });
            }, this));

            this._isDirty = false;
        },

        rejectChanges: function () {
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++)
                rows[i].revert(true);

            this._isDirty = false;
            this._events.triggerHandler({ type: "reject", table: this });
        },

        insert: function (json) {
            var model = new ej.Model(json);
            model._isDirty = this._isDirty = true;

            this.rows.push(model);

            this._events.triggerHandler({ type: "insert", model: model, table: this });
        },

        update: function (value) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var row = value, model, key = this.key, keyValue = row[key];

            model = this.rows.array.filter(function (obj) {
                return obj.get(key) === keyValue;
            });

            model = model[0];

            for (var col in row) {
                model.set(col, row[col]);
            }

            this._isDirty = true;

            this._events.triggerHandler({ type: "update", model: model, table: this });
        },

        remove: function (key) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var field = this.key;

            var index = -1, model;

            if (key && typeof key === "object") {
                key = key[field] !== undefined ? key[field] : key.get(field);
            }

            for (var i = 0; i < this.rows.length() ; i++) {
                if (this.rows.array[i].get(field) === key) {
                    index = i;
                    break;
                }
            }

            if (index > -1) {
                model = this.rows.removeAt(index);
                model.markDelete();

                this._deleted.push({ model: model, position: index });

                this._isDirty = true;
                this._events.triggerHandler({ type: "remove", model: model, table: this });
            }
        },

        isDirty: function () {
            return this._isDirty;
        },

        getChanges: function () {

            var changes = {
                added: [],
                changed: []
            };
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++) {
                if (changes[rows[i].state])
                    changes[rows[i].state].push(rows[i].json);
            }

            changes.deleted = ej.select(this._deleted, ["model"]);

            return changes;
        },

        toArray: function () {
            return this.rows.toArray();
        },

        setDirty: function (dirty, model) {
            if (this._isDirty === !!dirty) return;

            this._isDirty = !!dirty;

            this._events.triggerHandler({ type: "dirty", table: this, model: model });
        },
        get: function (index) {
            return this.rows.array[index];
        },
        length: function () {
            return this.rows.array.length;
        },

        bindTo: function (element) {
            var marker = tDiv, template = $(element.html()), rows = this.toArray(), cur;
            if ($.inArray(element.prop("tagName").toLowerCase(), ["table", "tbody"]))
                marker = tTR;

            marker.insertBefore(element);
            element.detach().empty();

            for (var i = 0; i < rows.length; i++) {
                cur = template.clone();
                rows[i].bindTo(cur);
                element.append(cur);
            }

            element.insertAfter(marker);
            marker.remove();
        }
    };

    var tDiv = doc ? $(document.createElement("div")) : {},
        tTR = doc ? $(document.createElement("tr")) : {};

    ej.Model = function (json, table, name) {
        if (typeof table === "string") {
            name = table;
            table = null;
        }
        this.$id = getUid("m");

        this.json = json;
        this.table = table instanceof ej.TableModel ? table : null;
        this.name = name || (this.table && this.table.name);
        this.dataManager = (table instanceof ej.DataManager) ? table : table.dataManagar;
        this.actual = {};
        this._events = $({});
        this.isDirty = false;
        this.state = "added";
        this._props = [];
        this._computeEls = {};
        this._fields = {};
        this._attrEls = {};
        this._updates = {};
        this.computed = {};
    };

    ej.Model.prototype = {
        computes: function (value) {
            $.extend(this.computed, value);
        },
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        set: function (field, value) {
            var obj = this.json, actual = field, prev;
            field = field.split('.');

            for (var i = 0; i < field.length - 1; i++) {
                field = field[0];
                obj = obj[field[0]];
            }

            this.isDirty = true;
            this.changeState("changed", { from: "set" });

            prev = obj[field];

            if (this.actual[field] === undefined && !(field in this.actual))
                this.actual[field] = value; // Complex property ?

            obj[field] = value;

            this._updateValues(field, value);
            this._events.triggerHandler({ type: actual, current: value, previous: prev, model: this });
        },
        get: function (field) {
            return ej.pvt.getObject(field, this.json);
        },
        revert: function (suspendEvent) {
            for (var prop in this.actual) {
                this.json[prop] = this.actual[prop];
            }

            this.isDirty = false;

            if (suspendEvent)
                this.state = "unchanged";
            else
                this.changeState("unchanged", { from: "revert" });
        },
        save: function (dm, key) {
            dm = dm || this.dataManagar;
            key = key || dm.dataSource.key;
            if (!dm) throwError("ej.Model - DataManager is required to commit the changes");
            if (this.state === "added") {
                return dm.insert(this.json, this.name).done(ej.proxy(function (e) {
                    $.extend(this.json, e.record);
                }, this));
            }
            else if (this.state === "changed") {
                return dm.update(key, this.json, this.name);
            }
            else if (this.state === "deleted") {
                return dm.remove(key, this.json, this.name);
            }
        },
        markCommit: function () {
            this.isDirty = false;
            this.changeState("unchanged", { from: "commit" });
        },
        markDelete: function () {
            this.changeState("deleted", { from: "delete" });
        },
        changeState: function (state, args) {
            if (this.state === state) return;

            if (this.state === "added") {
                if (state === "deleted")
                    state = "unchanged";
                else return;
            }

            var prev = state;
            args = args || {};

            this.state = state;
            this._events.triggerHandler($.extend({ type: "stateChange", current: state, previous: prev, model: this }, args));
        },
        properties: function () {
            if (this._props.length)
                return this._props;

            for (var pr in this.json) {
                this._props.push(pr);
                this._updates[pr] = { read: [], input: [] };
            }

            return this._props;
        },
        bindTo: function (element) {
            var el = $(element), ctl, field,
                elements = el.find("[ej-observe], [ej-computed], [ej-prop]"), len = elements.length;

            el.data("ejModel", this);
            var unbindData = { fields: [], props: [], computes: [] };
            for (var i = 0; i < len; i++) {
                ctl = elements.eq(i);

                field = ctl.attr("ej-prop");
                if (field) {
                    this._processAttrib(field, ctl, unbindData);
                }
                field = ctl.attr("ej-observe");
                if (field && this._props.indexOf(field) !== -1) {
                    this._processField(ctl, field, unbindData);
                    continue;
                }

                field = ctl.attr("ej-computed");
                if (field) {
                    this._processComputed(field, ctl, unbindData);
                    continue;
                }
            }
            el.data("ejModelBinding" + this.$id, unbindData);
        },
        unbind: function (element) {
            var tmp, data = {
                props: this._attrEls,
                computes: this._computeEls
            }, isCustom = false;

            if (element) {
                data = $(element).removeData("ejModel").data("ejModelBinding" + this.$id) || data;
                isCustom = true;
            }

            for (var p in this.computed) {
                tmp = data.computes[p], p = this.computed[p];
                if (tmp && p.deps) {
                    this.off(p.deps.join(' '), tmp.handle);
                    if (isCustom)
                        delete this._computeEls[p];
                }
            }
            if (!isCustom)
                this._computeEls = {};

            for (var p in data.props) {
                tmp = data.props[p];
                if (tmp) {
                    this.off(tmp.deps.join(' '), tmp.handle);
                    delete data.props[p];
                    if (isCustom)
                        delete this._attrEls[p];
                }
            }
            if (!isCustom)
                this._attrEls = {};

            if (data.fields && data.fields.length) {
                var len = data.fields.length, ctl, idx, ty;
                for (var i = 0; i < len; i++) {
                    ctl = data.fields[i];
                    $(ctl).off("change", null, this._changeHandler);

                    ty = this.formElements.indexOf(ctl.tagName.toLowerCase()) !== -1 ? "input" : "read";
                    idx = this._updates[ty].indexOf(ctl);
                    if (idx !== -1)
                        this._updates[ty].splice(idx, 1);
                }
            }
        },
        _processComputed: function (value, element, data) {
            if (!value) return;

            var val, deps, safeVal = safeStr(value),
            type = this.formElements.indexOf(element[0].tagName.toLowerCase()) !== -1 ? "val" : "html";

            if (!this.computed[value] || !this.computed[safeVal]) {
                this.computed[safeVal] = {
                    value: new Function("var e = this; return " + value),
                    deps: this._generateDeps(value)
                }
                value = safeVal;
            }

            val = this.computed[value];
            if (!val.get) {
                val.get = function () {
                    val.value.call(this.json);
                }
            }

            deps = val.deps;
            val = val.value;

            this._updateDeps(deps);
            this._updateElement(element, type, val);

            val = { el: element, handle: $proxy(this._computeHandle, this, { value: value, type: type }) };
            this._computeEls[value] = val;
            data.computes[value] = val;

            this.on(deps.join(' '), val.handle);
        },
        _computeHandle: function (e) {
            var el = this._computeEls[e.value];
            if (el && this.computed[e.value])
                this._updateElement(el.el, e.type, this.computed[e.value].value);
        },
        _updateElement: function (el, type, val) {
            el[type](val.call($.extend({}, this.json, this.computed)));
        },
        _updateDeps: function (deps) {
            for (var i = 0; i < deps.length; i++) {
                if (!(deps[i] in this.json) && deps[i] in this.computed)
                    ej.merge(deps, this.computed[deps[i]].deps);
            }
        },
        _generateDeps: function (value) {
            var splits = value.replace(/(^e\.)|( e\.)/g, '#%^*##ej.#').split("#%^*#"),
                field, deps = [];

            for (var i = 0; i < splits.length; i++) {
                if (splits[i].startsWith("#ej.#")) {
                    field = splits[i].replace("#ej.#", "").split(' ')[0];
                    if (field && this._props.indexOf(field) !== -1)
                        deps.push(field);
                }
            }

            return deps;
        },
        _processAttrib: function (value, el, data) {
            var prop, val, res = {};
            value = value.replace(/^ +| +$/g, "").split(";");
            for (var i = 0; i < value.length; i++) {
                value[i] = value[i].split(":");
                if (value[i].length < 2) continue;

                prop = value[i][0].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
                res[prop] = value[i][1].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
            }
            value = res;
            var deps = [];
            for (prop in value)
                deps.push(value[prop]);

            this._updateDeps(deps);
            this._updateProps(el, value);

            res = getUid("emak");
            val = { el: el, handle: $proxy(this._attrHandle, this, res), value: value, deps: deps };
            el.prop("ejmodelattrkey", res);

            data.props[res] = val;
            this._attrEls[res] = val;

            this.on(deps.join(' '), val.handle);
        },
        _attrHandle: function (res) {
            var el = this._attrEls[res];
            if (el)
                this._updateProps(el.el, el.value);
        },
        _updateProps: function (element, value) {
            var json = this.json, t, c = this.computed;
            for (var prop in value) {
                t = value[prop];
                if (t in json)
                    t = json[t];
                else if (t in c) {
                    t = c[t];
                    if (t) {
                        t = t.value.call($.extend({}, this.json, c));
                    }
                }

                if (!isNull(t)) {
                    element.prop(prop, t);
                }
            }
        },
        _updateValues: function (prop, value) {
            var arr = this._updates[prop];

            if (!arr || (!arr.read && !arr.input)) return;

            this._ensureItems(arr.read, "html", value);
            this._ensureItems(arr.input, "val", value);
        },
        _ensureItems: function (a, type, value) {
            if (!a) return;

            for (var i = a.length - 1; i > -1; i--) {
                if (!a[i].offsetParent) {
                    a.splice(i, 1);
                    continue;
                }
                $(a[i])[type](value);
            }
        },
        _changeHandler: function (e) {
            e.data.self.set(e.data.prop, $(this).val());
        },
        _processField: function (ctl, field, data) {
            var e = { self: this, prop: field }, val = this.get(field);

            data.fields.push(ctl[0]);

            if (this.formElements.indexOf(ctl[0].tagName.toLowerCase()) === -1) {
                ctl.html(val);
                return this._updates[field].read.push(ctl[0]);
            }

            ctl.val(val)
                    .off("change", null, this._changeHandler)
                    .on("change", null, e, this._changeHandler);

            return this._updates[field].input.push(ctl[0]);
        },
        formElements: ["input", "select", "textarea"]
    };

    var safeReg = /[^\w]+/g;
    var safeStr = function (value) {
        return value.replace(safeReg, "_");
    };
    var setDirty = function (e) {
        this.setDirty(true, e.model);
    };

    ej.Predicate = function (field, operator, value, ignoreCase) {
        if (!(this instanceof ej.Predicate))
            return new ej.Predicate(field, operator, value, ignoreCase);

        if (typeof field === "string") {
            this.field = field;
            this.operator = operator;
            this.value = value;
            this.ignoreCase = ignoreCase;
            this.isComplex = false;

            this._comparer = ej.data.fnOperators.processOperator(this.operator);

        } else if (field instanceof ej.Predicate && value instanceof ej.Predicate || value instanceof Array) {
            this.isComplex = true;
            this.condition = operator.toLowerCase();
            this.predicates = [field];
            if (value instanceof Array)
                [].push.apply(this.predicates, value);
            else
                this.predicates.push(value);
        }
        return this;
    };

    ej.Predicate.and = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "and");
    };

    ej.Predicate.or = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "or");
    };

    ej.Predicate.fromJSON = function (json) {
        if (instance(json, Array)) {
            var res = [];
            for (var i = 0, len = json.length; i < len; i++)
                res.push(pvtPredicate._fromJSON(json[i]));
            return res;
        }

        return pvtPredicate._fromJSON(json);
    };

    // Private fn
    var pvtPredicate = {
        _combinePredicates: function (predicates, operator) {
            if (!predicates.length) return undefined;
            if (predicates.length === 1) {
                if (!instance(predicates[0], Array))
                    return predicates[0];
                predicates = predicates[0];
            }
            return new ej.Predicate(predicates[0], operator, predicates.slice(1));
        },

        _combine: function (pred, field, operator, value, condition, ignoreCase) {
            if (field instanceof ej.Predicate)
                return ej.Predicate[condition](pred, field);

            if (typeof field === "string")
                return ej.Predicate[condition](pred, new ej.Predicate(field, operator, value, ignoreCase));

            return throwError("Predicate - " + condition + " : invalid arguments");
        },

        _fromJSON: function (json) {

            if (!json || instance(json, ej.Predicate))
                return json;

            var preds = json.predicates || [], len = preds.length, predicates = [], result;

            for (var i = 0; i < len; i++)
                predicates.push(pvtPredicate._fromJSON(preds[i]));                     

            if(!json.isComplex)
                result = new ej.Predicate(json.field, json.operator, ej.parseJSON({ val: json.value }).val, json.ignoreCase);
            else
                result = new ej.Predicate(predicates[0], json.condition, predicates.slice(1));

            return result;
        }
    };

    ej.Predicate.prototype = {
        and: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "and", ignoreCase);
        },
        or: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "or", ignoreCase);
        },
        validate: function (record) {
            var p = this.predicates, isAnd, ret;

            if (!this.isComplex) {
                return this._comparer.call(this, ej.pvt.getObject(this.field, record), this.value, this.ignoreCase);
            }

            isAnd = this.condition === "and";

            for (var i = 0; i < p.length; i++) {
                ret = p[i].validate(record);
                if (isAnd) {
                    if (!ret) return false;
                } else {
                    if (ret) return true;
                }
            }

            return isAnd;
        },
        toJSON: function () {
            var predicates, p;
            if (this.isComplex) {
                predicates = [], p = this.predicates;
                for (var i = 0; i < p.length; i++)
                    predicates.push(p[i].toJSON());
            }
            return {
                isComplex: this.isComplex,
                field: this.field,
                operator: this.operator,
                value: this.value,
                ignoreCase: this.ignoreCase,
                condition: this.condition,
                predicates: predicates
            }
        }
    };

    ej.dataUtil = {
        swap: function (array, x, y) {
            if (x == y) return;

            var tmp = array[x];
            array[x] = array[y];
            array[y] = tmp;
        },

        mergeSort: function (jsonArray, fieldName, comparer) {
            if (!comparer || typeof comparer === "string")
                comparer = ej.pvt.fnSort(comparer, true);

            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.mergeSort(jsonArray, fieldName, comparer);
        },

        max: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnDescending);
        },

        min: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnAscending);
        },

        distinct: function (json, fieldName, requiresCompleteRecord) {
            var result = [], val, tmp = {};
            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (!(val in tmp)) {
                    result.push(!requiresCompleteRecord ? val : json[i]);
                    tmp[val] = 1;
                }
            }
            return result;
        },

        sum: function (json, fieldName) {
            var result = 0, val, castRequired = typeof getVal(json, fieldName, 0) !== "number";

            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (!isNaN(val) && val !== null) {
                    if (castRequired)
                       val = +val;
                   result += val;
                }
            }
            return result;
        },

        avg: function (json, fieldName) {
            return ej.sum(json, fieldName) / json.length;
        },

        select: function (jsonArray, fields) {
            var newData = [];

            for (var i = 0; i < jsonArray.length; i++) {
                newData.push(ej.pvt.extractFields(jsonArray[i], fields));
            }

            return newData;
        },

        group: function (jsonArray, field, agg, format,/* internal */ level,groupDs) {
            level = level || 1;

            if (jsonArray.GROUPGUID == ej.pvt.consts.GROUPGUID) {
                for (var j = 0; j < jsonArray.length; j++) {
                    if(!ej.isNullOrUndefined(groupDs)){
                        var indx = -1;
                        var temp = $.grep(groupDs,function(e){return e.key==jsonArray[j].key});
                        indx = groupDs.indexOf(temp[0]);
                        jsonArray[j].items = ej.group(jsonArray[j].items, field, agg, format, jsonArray.level + 1, groupDs[indx].items);
                        jsonArray[j].count = groupDs[indx].count;
                    }
                    else{
                        jsonArray[j].items = ej.group(jsonArray[j].items, field, agg, format, jsonArray.level + 1);
                        jsonArray[j].count = jsonArray[j].items.length;
                    }  
                }

                jsonArray.childLevels += 1;
                return jsonArray;
            }

            var grouped = {}, groupedArray = [];

            groupedArray.GROUPGUID = ej.pvt.consts.GROUPGUID;
            groupedArray.level = level;
            groupedArray.childLevels = 0;
            groupedArray.records = jsonArray;

            for (var i = 0; i < jsonArray.length; i++) {
                var val = getVal(jsonArray, field, i);
                if (!ej.isNullOrUndefined(format)) val = format(val, field);

                if (!grouped[val]) {
                    grouped[val] = {
                        key: val,
                        count: 0,
                        items: [],
                        aggregates: {},
                        field: field
                    };
                    groupedArray.push(grouped[val]);
					if(!ej.isNullOrUndefined(groupDs)) {
                        var tempObj = $.grep(groupDs,function(e){return e.key==grouped[val].key});
                       grouped[val].count = tempObj[0].count
                    }
                }

                grouped[val].count = !ej.isNullOrUndefined(groupDs) ? grouped[val].count :  grouped[val].count += 1;
                grouped[val].items.push(jsonArray[i]);
            }
            if (agg && agg.length) {

                for (var i = 0; i < groupedArray.length; i++) {
                    var res = {}, fn;
                    for (var j = 0; j < agg.length; j++) {

                        fn = ej.aggregates[agg[j].type];
                        if(!ej.isNullOrUndefined(groupDs)) {
                            var temp = $.grep(groupDs,function(e){return e.key==groupedArray[i].key});
                            if(fn)
                                res[agg[j].field + " - " + agg[j].type] = fn(temp[0].items, agg[j].field);
                        }
                        else{
                            if (fn)
                                res[agg[j].field + " - " + agg[j].type] = fn(groupedArray[i].items, agg[j].field);
                        }

                    }
                    groupedArray[i]["aggregates"] = res;
                }
            }
            return groupedArray;
        },

        parseTable: function (table, headerOption, headerRowIndex) {
            var tr = table.rows, headerRow, headerTds = [], data = [], i;

            if (!tr.length) return [];

            headerRowIndex = headerRowIndex || 0;

            switch ((headerOption || "").toLowerCase()) {
                case ej.headerOption.tHead:
                    headerRow = table.tHead.rows[headerRowIndex];
                    break;
                case ej.headerOption.row:
                default:
                    headerRow = table.rows[headerRowIndex];
                    break;
            }

            var hTd = headerRow.cells;

            for (i = 0; i < hTd.length; i++)
                headerTds.push($.trim(hTd[i].innerHTML));

            for (i = headerRowIndex + 1; i < tr.length; i++) {
                var json = {}, td = tr[i].cells;
                for (var j = 0; j < td.length; j++) {
                    var temp = td[j].innerHTML;
                    if (typeof temp == "string" && $.isNumeric(temp))
                       json[headerTds[j]] = Number(temp);
				    else
                       json[headerTds[j]] = temp;
                }
                data.push(json);
            }
            return data;
        }
    };

    ej.headerOption = {
        tHead: "thead",
        row: "row"
    };

    ej.aggregates = {
        sum: function (ds, field) {
            return ej.sum(ds, field);
        },
        average: function (ds, field) {
            return ej.avg(ds, field);
        },
        minimum: function (ds, field) {
            return ej.getObject(field, ej.min(ds, field));
        },
        maximum: function (ds, field) {
            return  ej.getObject(field, ej.max(ds, field));
        },
        truecount: function (ds, field){
            var predicate = ej.Predicate(field, "equal", true);
            return ej.DataManager(ds).executeLocal(ej.Query().where(predicate)).length;
        },
        falsecount: function (ds, field) {
            var predicate = ej.Predicate(field, "equal", false);
            return ej.DataManager(ds).executeLocal(ej.Query().where(predicate)).length;
        },
        count: function (ds, field) {
            return ds.length;
        }

    };
    ej.pvt = {
        filterQueries: filterQueries,
        mergeSort: function (jsonArray, fieldName, comparer) {
            if (jsonArray.length <= 1)
                return jsonArray;

            // else list size is > 1, so split the list into two sublists
            var middle = parseInt(jsonArray.length / 2, 10);

            var left = jsonArray.slice(0, middle),
                right = jsonArray.slice(middle);

            left = ej.pvt.mergeSort(left, fieldName, comparer);
            right = ej.pvt.mergeSort(right, fieldName, comparer);

            return ej.pvt.merge(left, right, fieldName, comparer);
        },

        getItemFromComparer: function (array, field, comparer) {
            var keyVal, current, key, i = 0,castRequired = typeof getVal(array, field, 0) == "string";
            if (array.length)
            while (ej.isNullOrUndefined(keyVal) && i < array.length) {
                keyVal = getVal(array, field, i);
                key = array[i++];
            }
            for (; i < array.length; i++) {
                current = getVal(array, field, i);
                if (ej.isNullOrUndefined(current))
                    continue;
                if (castRequired) {
                    keyVal = +keyVal;
                    current = +current;
                }
                if (comparer(keyVal, current) > 0) {
                    keyVal = current;
                    key = array[i];
                }
            }
            return key;
        },

        quickSelect: function (array, fieldName, left, right, k, comparer) {
            if (left == right)
                return array[left];

            var pivotNewIndex = ej.pvt.partition(array, fieldName, left, right, comparer);

            var pivotDist = pivotNewIndex - left + 1;

            if (pivotDist == k)
                return array[pivotNewIndex];

            else if (k < pivotDist)
                return ej.pvt.quickSelect(array, fieldName, left, pivotNewIndex - 1, k, comparer);
            else
                return ej.pvt.quickSelect(array, fieldName, pivotNewIndex + 1, right, k - pivotDist, comparer);
        },

        extractFields: function (obj, fields) {
            var newObj = {};

            if (fields.length == 1)
                return ej.pvt.getObject(fields[0], obj);

            for (var i = 0; i < fields.length; i++) {
                newObj[fields[i].replace('.', ej.pvt.consts.complexPropertyMerge)] = ej.pvt.getObject(fields[i], obj);
            }

            return newObj;
        },

        partition: function (array, field, left, right, comparer) {

            var pivotIndex = parseInt((left + right) / 2, 10),
                pivot = getVal(array, field, pivotIndex);

            ej.swap(array, pivotIndex, right);

            pivotIndex = left;

            for (var i = left; i < right; i++) {
                if (comparer(getVal(array, field, i), pivot)) {
                    ej.swap(array, i, pivotIndex);
                    pivotIndex++;
                }
            }

            ej.swap(array, pivotIndex, right);

            return pivotIndex;
        },

        fnSort: function (order) {
            order = order ? order.toLowerCase() : ej.sortOrder.Ascending;

            if (order == ej.sortOrder.Ascending)
                return ej.pvt.fnAscending;

            return ej.pvt.fnDescending;
        },

        fnGetComparer: function (field, fn) {
            return function (x, y) {
                return fn(ej.pvt.getObject(field, x), ej.pvt.getObject(field, y));
            }
        },

        fnAscending: function (x, y) {
            if (y === null || y === undefined)
                return -1;

            if (typeof x === "string")
                return x.localeCompare(y);

            if (x === null || x === undefined)
                return 1;

            return x - y;
        },

        fnDescending: function (x, y) {
            if (y === null || y === undefined)
                return 1;

            if (typeof x === "string")
                return x.localeCompare(y) * -1;

            if (x === null || x === undefined)
                return -1;

            return y - x;
        },

        merge: function (left, right, fieldName, comparer) {
            var result = [], current;

            while (left.length > 0 || right.length > 0) {
                if (left.length > 0 && right.length > 0) {
                    if (comparer)
                        current = comparer(getVal(left, fieldName, 0), getVal(right, fieldName, 0)) <= 0 ? left : right;
                    else
                        current = left[0][fieldName] < left[0][fieldName] ? left : right;
                } else {
                    current = left.length > 0 ? left : right;
                }

                result.push(current.shift());
            }

            return result;
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;
            if (!nameSpace) return from;

            if (nameSpace.indexOf('.') === -1) return from[nameSpace];

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (value == null) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i;

            for (i = 0; i < splits.length; i++) {

                if (i + 1 == splits.length)
                    from[splits[i]] = value === undefined ? {} : value;
                else if (from[splits[i]] == null)
                    from[splits[i]] = {};

                from = from[splits[i]];
            }

            return start;
        },

        getFieldList: function (obj, fields, prefix) {
            if (prefix === undefined)
                prefix = "";

            if (fields === undefined || fields === null)
                return ej.pvt.getFieldList(obj, [], prefix);

            for (var prop in obj) {
                if (typeof obj[prop] === "object" && !(obj[prop] instanceof Array))
                    ej.pvt.getFieldList(obj[prop], fields, prefix + prop + ".");
                else
                    fields.push(prefix + prop);
            }

            return fields;
        }
    };

    ej.FilterOperators = {
        lessThan: "lessthan",
        greaterThan: "greaterthan",
        lessThanOrEqual: "lessthanorequal",
        greaterThanOrEqual: "greaterthanorequal",
        equal: "equal",
        contains: "contains",
        startsWith: "startswith",
        endsWith: "endswith",
        notEqual: "notequal"
    };

    ej.data = {};

    ej.data.operatorSymbols = {
        "<": "lessthan",
        ">": "greaterthan",
        "<=": "lessthanorequal",
        ">=": "greaterthanorequal",
        "==": "equal",
        "!=": "notequal",
        "*=": "contains",
        "$=": "endswith",
        "^=": "startswith"
    };

    ej.data.odBiOperator = {
        "<": " lt ",
        ">": " gt ",
        "<=": " le ",
        ">=": " ge ",
        "==": " eq ",
        "!=": " ne ",
        "lessthan": " lt ",
        "lessthanorequal": " le ",
        "greaterthan": " gt ",
        "greaterthanorequal": " ge ",
        "equal": " eq ",
        "notequal": " ne "
    };

    ej.data.odUniOperator = {
        "$=": "endswith",
        "^=": "startswith",
        "*=": "substringof",
        "endswith": "endswith",
        "startswith": "startswith",
        "contains": "substringof"
    };

    ej.data.fnOperators = {
        equal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) == toLowerCase(expected);

            return actual == expected;
        },
        notequal: function (actual, expected, ignoreCase) {
            return !ej.data.fnOperators.equal(actual, expected, ignoreCase);
        },
        lessthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) < toLowerCase(expected);

            return actual < expected;
        },
        greaterthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) > toLowerCase(expected);

            return actual > expected;
        },
        lessthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) <= toLowerCase(expected);

            return actual <= expected;
        },
        greaterthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) >= toLowerCase(expected);

            return actual >= expected;
        },
        contains: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return !isNull(actual) && !isNull(expected) && toLowerCase(actual).indexOf(toLowerCase(expected)) != -1;

            return !isNull(actual) && !isNull(expected) && actual.toString().indexOf(expected) != -1;
        },
        notnull: function (actual) {
            return actual !== null;
        },
        isnull: function (actual) {
            return actual === null;
        },
        startswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).startsWith(toLowerCase(expected));

            return actual && expected && actual.startsWith(expected);
        },
        endswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).endsWith(toLowerCase(expected));

            return actual && expected && actual.endsWith(expected);
        },

        processSymbols: function (operator) {
            var fnName = ej.data.operatorSymbols[operator];
            if (fnName) {
                var fn = ej.data.fnOperators[fnName];
                if (fn) return fn;
            }

            return throwError("Query - Process Operator : Invalid operator");
        },

        processOperator: function (operator) {
            var fn = ej.data.fnOperators[operator];
            if (fn) return fn;
            return ej.data.fnOperators.processSymbols(operator);
        }
    };

    ej.NotifierArray = function (array) {
        if (!instance(this, ej.NotifierArray))
            return new ej.NotifierArray(array);

        this.array = array;

        this._events = $({});
        this._isDirty = false;

        return this;
    };

    ej.NotifierArray.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        push: function (item) {
            var ret;

            if (instance(item, Array))
                ret = [].push.apply(this.array, item);
            else
                ret = this.array.push(item);

            this._raise("add", { item: item, index: this.length() - 1 });

            return ret;
        },
        pop: function () {
            var ret = this.array.pop();

            this._raise("remove", { item: ret, index: this.length() - 1 });

            return ret;
        },
        addAt: function (index, item) {
            this.array.splice(index, 0, item);

            this._raise("add", { item: item, index: index });

            return item;
        },
        removeAt: function (index) {
            var ret = this.array.splice(index, 1)[0];

            this._raise("remove", { item: ret, index: index });

            return ret;
        },
        remove: function (item) {
            var index = this.array.indexOf(item);

            if (index > -1) {
                this.array.splice(index, 1);
                this._raise("remove", { item: item, index: index });
            }

            return index;
        },
        length: function () {
            return this.array.length;
        },
        _raise: function (e, args) {
            this._events.triggerHandler($.extend({ type: e }, args));
            this._events.triggerHandler({ type: "all", name: e, args: args });
        },
        toArray: function () {
            return this.array;
        }
    };

    $.extend(ej, ej.dataUtil);

    // For IE8
    Array.prototype.forEach = Array.prototype.forEach || function (fn, scope) {
        for (var i = 0, len = this.length; i < len; ++i) {
            fn.call(scope, this[i], i, this);
        }
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    Array.prototype.filter = Array.prototype.filter || function (fn) {
        if (typeof fn != "function")
            throw new TypeError();

        var res = [];
        var thisp = arguments[1] || this;
        for (var i = 0; i < this.length; i++) {
            var val = this[i]; // in case fun mutates this
            if (fn.call(thisp, val, i, this))
                res.push(val);
        }

        return res;
    };

    String.prototype.endsWith = String.prototype.endsWith || function (key) {
        return this.slice(-key.length) === key;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };

    if (!ej.support) ej.support = {};
    ej.support.stableSort = function () {
        var res = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16].sort(function () { return 0; });
		for(var i = 0; i < 17; i++){
		    if(i !== res[i]) return false;
		}
        return true;
    }();
    ej.support.cors = $.support.cors;

    if (!$.support.cors && window.XDomainRequest) {
        var httpRegEx = /^https?:\/\//i;
        var getOrPostRegEx = /^get|post$/i;
        var sameSchemeRegEx = new RegExp('^' + location.protocol, 'i');
        var xmlRegEx = /\/xml/i;

        // ajaxTransport exists in jQuery 1.5+
        $.ajaxTransport('text html xml json', function (options, userOptions, jqXHR) {
            // XDomainRequests must be: asynchronous, GET or POST methods, HTTP or HTTPS protocol, and same scheme as calling page
            if (options.crossDomain && options.async && getOrPostRegEx.test(options.type) && httpRegEx.test(userOptions.url) && sameSchemeRegEx.test(userOptions.url)) {
                var xdr = null;
                var userType = (userOptions.dataType || '').toLowerCase();
                return {
                    send: function (headers, complete) {
                        xdr = new XDomainRequest();
                        if (/^\d+$/.test(userOptions.timeout)) {
                            xdr.timeout = userOptions.timeout;
                        }
                        xdr.ontimeout = function () {
                            complete(500, 'timeout');
                        };
                        xdr.onload = function () {
                            var allResponseHeaders = 'Content-Length: ' + xdr.responseText.length + '\r\nContent-Type: ' + xdr.contentType;
                            var status = {
                                code: 200,
                                message: 'success'
                            };
                            var responses = {
                                text: xdr.responseText
                            };

                            try {
                                if (userType === 'json') {
                                    try {
                                        responses.json = JSON.parse(xdr.responseText);
                                    } catch (e) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        //throw 'Invalid JSON: ' + xdr.responseText;
                                    }
                                } else if ((userType === 'xml') || ((userType !== 'text') && xmlRegEx.test(xdr.contentType))) {
                                    var doc = new ActiveXObject('Microsoft.XMLDOM');
                                    doc.async = false;
                                    try {
                                        doc.loadXML(xdr.responseText);
                                    } catch (e) {
                                        doc = undefined;
                                    }
                                    if (!doc || !doc.documentElement || doc.getElementsByTagName('parsererror').length) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        throw 'Invalid XML: ' + xdr.responseText;
                                    }
                                    responses.xml = doc;
                                }
                            } catch (parseMessage) {
                                throw parseMessage;
                            } finally {
                                complete(status.code, status.message, responses, allResponseHeaders);
                            }
                        };
                        xdr.onerror = function () {
                            complete(500, 'error', {
                                text: xdr.responseText
                            });
                        };
						if(navigator.userAgent.indexOf("MSIE 9.0") != -1)
							xdr.onprogress = function() {};
                        xdr.open(options.type, options.url);
                        xdr.send(userOptions.data);
                        //xdr.send();
                    },
                    abort: function () {
                        if (xdr) {
                            xdr.abort();
                        }
                    }
                };
            }
        });
    }

    $.support.cors = true;

    ej.sortOrder = {
        Ascending: "ascending",
        Descending: "descending"
    };

    // privates
    ej.pvt.consts = {
        GROUPGUID: "{271bbba0-1ee7}",
        complexPropertyMerge: "_"
    };

    // private utils
    var nextTick = function (fn, context) {
        if (context) fn = $proxy(fn, context);
        (window.setImmediate || window.setTimeout)(fn, 0);
    };

    ej.support.enableLocalizedSort = false;

    var stableSort = function (ds, field, comparer, queries) {
        if (ej.support.stableSort) {
            if(!ej.support.enableLocalizedSort && typeof ej.pvt.getObject(field, ds[0] || {}) == "string" 
                && (comparer === ej.pvt.fnAscending || comparer === ej.pvt.fnDescending)
                && queries.filter(function(e){return e.fn === "onSortBy";}).length === 1)
                return fastSort(ds, field, comparer === ej.pvt.fnDescending);
            return ds.sort(ej.pvt.fnGetComparer(field, comparer));
        }
        return ej.mergeSort(ds, field, comparer);
    };
    var getColFormat = function (field, query) {
        var grpQuery = $.grep(query, function (args) { return args.fn == "onGroup" });
        for (var grp = 0; grp < grpQuery.length; grp++) {
            if (ej.getObject("fieldName", grpQuery[grp].e) == field) {
                return ej.getObject("fn", grpQuery[grp].e);
            }
        }
    };
    var fastSort = function(ds, field, isDesc){
        var old = Object.prototype.toString;
        Object.prototype.toString = (field.indexOf('.') === -1) ? function(){
            return this[field];
        }:function(){
            return ej.pvt.getObject(field, this);
        };
        ds = ds.sort();
        Object.prototype.toString = old;
        if(isDesc)
            ds.reverse();
    }

    var toLowerCase = function (val) {
        return val ? val.toLowerCase ? val.toLowerCase() : val.toString().toLowerCase() : (val === 0 || val === false) ? val.toString() : "";
    };

    var getVal = function (array, field, index) {
        return field ? ej.pvt.getObject(field, array[index]) : array[index];
    };

    var isHtmlElement = function (e) {
        return typeof HTMLElement === "object" ? e instanceof HTMLElement :
            e && e.nodeType === 1 && typeof e === "object" && typeof e.nodeName === "string";
    };

    var instance = function (obj, element) {
        return obj instanceof element;
    };

    var getTableModel = function (name, result, dm, computed) {
        return function (tName) {
            if (typeof tName === "object") {
                computed = tName;
                tName = null;
            }
            return new ej.TableModel(tName || name, result, dm, computed);
        };
    };

    var getKnockoutModel = function (result) {
        return function (computedObservables, ko) {
            ko = ko || window.ko;

            if (!ko) throwError("Knockout is undefined");

            var model, koModels = [], prop, ob;
            for (var i = 0; i < result.length; i++) {
                model = {};
                for (prop in result[i]) {
                    if (!prop.startsWith("_"))
                        model[prop] = ko.observable(result[i][prop]);
                }
                for (prop in computedObservables) {
                    ob = computedObservables[prop];

                    if ($.isPlainObject(ob)) {
                        if (!ob.owner) ob.owner = model;
                        ob = ko.computed(ob);
                    } else
                        ob = ko.computed(ob, model);

                    model[prop] = ob;
                }
                koModels.push(model);
            }

            return ko.observableArray(koModels);
        };
    };

    var uidIndex = 0;
    var getUid = function (prefix) {
        uidIndex += 1;
        return prefix + uidIndex;
    };

    ej.getGuid = function (prefix) {
        var hexs = '0123456789abcdef', rand;
        return (prefix || "") + '00000000-0000-4000-0000-000000000000'.replace(/0/g, function (val, i) {
            if ("crypto" in window && "getRandomValues" in crypto) {
                var arr = new Uint8Array(1)
                window.crypto.getRandomValues(arr);
                rand = arr[0] % 16|0
            }
            else rand = Math.random() * 16 | 0;
            return hexs[i === 19 ? rand & 0x3 | 0x8 : rand];
        });
    };

    var proxy = function (fn, context) {
        return function () {
            var args = [].slice.call(arguments, 0);
            args.push(this);

            return fn.apply(context || this, args);
        };
    };

    var $proxy = function (fn, context, arg) {
        if ('bind' in fn)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            return fn.apply(context || this, args);
        };
    };

    ej.merge = function (first, second) {
        if (!first || !second) return;

        Array.prototype.push.apply(first, second);
    };

    var isNull = function (val) {
        return val === undefined || val === null;
    };

    var throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };

})(window.jQuery, window.Syncfusion, window.document);;

(function ($, undefined) {

    var $document = $(document);
    // add new event shortcuts
    $.each(("touchstart touchmove touchend " +
		"tap doubletap taphold " +
		"swipe swipeleft swiperight " + "pinch pinchin pinchout pinchstop " +
		"scrollstart scrollstop").split(" "), function (i, name) {

		    $.fn[name] = function (fn) {
		        return fn ? this.on(name, fn) : this.trigger(name);
		    };

		    // jQuery < 1.8
		    if ($.attrFn) {
		        $.attrFn[name] = true;
		    }
		});

    var isPointer = browserInfo().isMSPointerEnabled,
    isIE11Pointer = browserInfo().pointerEnabled,
    supportTouch = 'ontouchstart' in window,
	scrollEvent = "scroll",
    isDesktop = (typeof window.orientation === "undefined"),
    isIosDevice = navigator.userAgent.match(/iPhone|iPad|iPod/i),
	touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : (supportTouch ? "touchstart" : "mousedown"),
	touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : isIosDevice ? ("touchend") : (supportTouch ? "touchend" : "mouseup"),
	touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : (supportTouch ? "touchmove" : "mousemove"),
    touchCancelEvent = isPointer ? (isIE11Pointer ? "pointercancel" : "MSPointerCancel") : (supportTouch ? "touchcancel" : "mouseleave"),
    mouseStartEvent = isPointer || !isDesktop ? touchStartEvent : isIosDevice ? "touchstart" : "touchstart mousedown",
    mouseStopEvent = isPointer || !isDesktop ? touchStopEvent : "touchend mouseup",
    mouseMoveEvent = isPointer || !isDesktop ? touchMoveEvent : "touchmove mousemove",
    browser = browserInfo(),
    isIE9 = ((browser.name == 'msie') && (browser.version == '9.0')) ? true : false;

    function browserInfo() {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        browser.pointerEnabled = window.navigator.pointerEnabled;
        return browser;
    }
    function initiateCustomEvent(obj, eventType, e) {
        var originalType = e.type;
        e.type = eventType;
        $.event.dispatch.call(obj, e);
        e.type = originalType;
    }
    function copyObject(e, origEvent) {
        if (origEvent) {
            for (prop in origEvent) {
                if (!(prop in e)) {
                    e[prop] = origEvent[prop];
                }
            }
        }
    }
    $.event.special.ejtouchmove = {
        setup: function () {
            var thisObj = this,
				$this = $(thisObj);
            $this.on(touchStartEvent, startMoveHandler);
            $document.on(touchStopEvent, clearTouchMoveHandlers);
            function clearTouchMoveHandlers() {
               // $this.off(touchMoveEvent, moveHandler)
            }
            var coords = {};
            function startMoveHandler(e) {
                if (!(e.which && e.which !== 1)) {
                    var origTarget = e.target,
					origEvent = e.originalEvent;
                    if (isPointer)
                        coords = { x: origEvent.x, y: origEvent.y };
                    $this.on(touchMoveEvent, moveHandler);
                }
            }
            function moveHandler(e) {
                if (!(e.which && e.which !== 1)) {
                    var origTarget = e.target,
					origEvent = e.originalEvent;
                    copyObject(e, e.originalEvent);
                    if (!isPointer || !coords || (Math.abs(coords.x - origEvent.x) > 10 || Math.abs(coords.y - origEvent.y) > 10 && isPointer))
                        initiateCustomEvent(thisObj, "ejtouchmove", e);
                }
            }
        }
    };
    function touchObj(e) {
        return e.originalEvent.touches ?
					e.originalEvent.touches[0] : isPointer ? e.originalEvent : e;
    }
    // handles swipeup and swipedown
    $.event.special.swipeupdown = {
        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);
            $this.on(touchStartEvent, function (e) {
                var _startevent = e;
                var data = touchObj(e),
                            startPoint = {
                                time: (new Date).getTime(),
                                coords: [data.pageX, data.pageY],
                                origin: $(e.target)
                            },
                            stopPoint;
                function moveHandler(e) {
                    e.preventDefault();
                    if (!startPoint) return;
                    var data = touchObj(e);
                    stopPoint = {
                        time: (new Date).getTime(),
                        coords: [data.pageX, data.pageY]
                    };
                    if (Math.abs(startPoint.coords[1] - stopPoint.coords[1]) > 10) e.preventDefault();
                }

                $this
                            .on(touchMoveEvent, moveHandler)
                            .one(touchStopEvent, function (e) {
                                $this.off(touchMoveEvent, moveHandler);
                                if (startPoint && stopPoint) {
                                    if (stopPoint.time - startPoint.time < 1000 &&
                                    Math.abs(startPoint.coords[1] - stopPoint.coords[1]) > 30 &&
                                    Math.abs(startPoint.coords[0] - stopPoint.coords[0]) < 75) {
                                        var _addDetails = { time: stopPoint.time - startPoint.time, _isSwipe: true, _isDelta: true, stopPoint: stopPoint };
                                        var _options = _getOptions(e, _addDetails, _startevent);

                                        startPoint.origin
                                        .trigger($.extend(true, { type: "swipeupdown" }, _options))
                                        .trigger($.extend(true, { type: startPoint.coords[1] > stopPoint.coords[1] ? "swipeup" : "swipedown" }, _options));
                                    }
                                }
                                startPoint = stopPoint = undefined;
                            });
            });
        }
    };
    $.event.special.scrollstart = {
        isEnabled: true,
        setup: function () {
            var thisObj = this, $this = $(thisObj), scrolling, timer;
            function trigger(e, scrollState) {
                scrolling = scrollState;
                initiateCustomEvent(thisObj, scrolling ? "scrollstart" : "scrollstop", e);
            }
            $this.on(scrollEvent, function (e) {
                if (!$.event.special.scrollstart.isEnabled) return;
                if (!scrolling) trigger(e, true);
                clearTimeout(timer);
                timer = setTimeout(function () {
                    trigger(e, false);
                }, 250);
            });
        }
    };

    // also handles doubletap, taphold
    $.event.special.tap = {
        doubleTapThreshold: 500,
        tapholdThreshold: 650,
        canDoubleTap: function (d) {
            return ((getTimeSpan() - d.doubleTapStartTime) <= $.event.special.tap.doubleTapThreshold);
        },
        setup: function () {
            var thisObj = this, $this = $(thisObj), d = $this.data();
            var mouseDownTarget;            
            checkMsieTouch($this);
            d.isDoubleTapWait = false;
            d.stopProcess = false;
            d.preTouchend = null;
            d.preTouchstart = null;

            $this.on(mouseStartEvent, function (event) {
                if (event.type == "mousedown" || event.type == "pointerdown" || "MSPointerDown")                   
                    mouseDownTarget = event.target;
                               
                d = $this.data();
                d.startTime = getTimeSpan();
                if (!d.isDoubleTapWait) d.doubleTapStartTime = d.startTime;
                if (event.type == "touchstart") d.preTouchstart = d.startTime;
                // checked mousedown event arrives within 300'ms after the touchend completes
                if (event.type == "mousedown" && (d.startTime - d.preTouchend < 300 || d.startTime - d.preTouchstart < 30))
                    d.stopProcess = true;
                else d.stopProcess = false;

                var origTarget = event.currentTarget,
					origEvent = event.originalEvent,
					timer;

                function clearTapHandlers() {
                    clearTimeout(timer);

                    $this.off(mouseStopEvent, clickHandler);
                    if (isIE9) $document.off(mouseStopEvent, clickHandler);
                    $this.off(touchCancelEvent, clearTapHandlers);
                    $this.off(mouseMoveEvent, touchMoveAction);
                    $this.off('dragstart', dragAction);
                }
                function touchMoveAction(e) {
                    var touchmoveThreshold = 10;
                    var coor1 = (e.originalEvent.changedTouches ? e.originalEvent.changedTouches[0] : e.originalEvent),
                        coor2 = (event.originalEvent.changedTouches ? event.originalEvent.changedTouches[0] : event.originalEvent);
                    if (!((coor1.pageX - coor2.pageX < touchmoveThreshold && coor1.pageX - coor2.pageX > -(touchmoveThreshold)) &&
                        (coor1.pageY - coor2.pageY < touchmoveThreshold && coor1.pageY - coor2.pageY > -(touchmoveThreshold)))) {
                        if (e.type == "mousemove" || (e.type == "pointermove" && e.originalEvent.pointerType == "mouse") ||
                            e.type == "MSPointerMove" && e.originalEvent.pointerType == 4) {
                            clearTimeout(timer);
                            $this.off(touchCancelEvent, clearTapHandlers);
                            $this.off(mouseMoveEvent, touchMoveAction);
                        }
                        else
                          clearTapHandlers();                                            
                    }                    
                }
                function clickHandler(e) {
                    if (e.type == "touchend") d.preTouchend = getTimeSpan();
                    clearTapHandlers();

                    // Modify the tap event target
                    if (mouseDownTarget != e.target && (e.type == "mouseup" || event.type == "pointerup" || "MSPointerUp")) {
                        var mouseupTarget = e.target;                        
                        // Mousedown element is the parent of the mouseup element.                       
                        if (jQuery.contains(mouseDownTarget, mouseupTarget))
                            // Mousedown element is target.
                            updateTargetEle(e, mouseDownTarget);
                       
                        // Mousedown element is the sibiling of the mouseup element.                            
                        else if (!(jQuery.contains(mouseupTarget, mouseDownTarget))) {
                            var ele = $(mouseDownTarget).parents().has($(mouseupTarget)).first()[0];
							if(!ej.isNullOrUndefined(ele))
                                updateTargetEle(e, ele);                         
                        }
                    }

                    // ONLY trigger a 'tap' event if the startPoint target is
                    // the same as the stopPoint target.
                    if (origTarget === e.currentTarget) {
                        initiateCustomEvent(thisObj, "tap", $.extend(_getBaseOptions(e), {
                            time: getTimeSpan() - d.startTime
                        }));

                        if (d.isDoubleTapWait && $.event.special.tap.canDoubleTap(d)) {
                            d.isDoubleTapWait = false;

                            initiateCustomEvent(thisObj, "doubletap", $.extend(_getBaseOptions(e), {
                                time: getTimeSpan() - d.doubleTapStartTime
                            }));
                        }
                        else {
                            if (d.isDoubleTapWait) {
                                d.isDoubleTapWait = false;
                                d.doubleTapStartTime = d.startTime;
                            }
                            if ($.event.special.tap.canDoubleTap(d)) {
                                d.isDoubleTapWait = true;
                            }
                        }
                    }
                }
                function updateTargetEle(e,target) {
                    e.target = target;
                    e.toElement = target;                  
                }
                function dragAction(e) {
                    // Remove the tap handler while moving the element inside the tap.
                    clearTapHandlers();
                }
                if (!(event.which && event.which !== 1) && !d.stopProcess) {                   
                    $this.on(mouseStopEvent, clickHandler);
                    if (isIE9) $document.on(mouseStopEvent, clickHandler);
                    $this.on(touchCancelEvent, clearTapHandlers);
                    $this.on(mouseMoveEvent, touchMoveAction);
                    $this.on('dragstart', dragAction);
                    var eventCopy = {};  
                    for (var i in origEvent) {
                        eventCopy[i] = origEvent[i];    // For IE8 taphold issue, copy the original event in local variable 
                    }
                    timer = setTimeout(function () {
                        if (d.isDoubleTapWait) d.isDoubleTapWait = false;
                        initiateCustomEvent(thisObj, "taphold", $.extend(_getBaseOptions(event), {
                            options: eventCopy,
                            time: getTimeSpan() - d.startTime
                        }));

                    }, $.event.special.tap.tapholdThreshold);
                }
                else if (d.stopProcess) d.stopProcess = false;
            });
        }
    };    
    $.event.special.swipe = {
        scrollSupression: 10,
        duration: 1000,
        horizontalDistance: 30,
        verticalDistance: 75,
        pointers: window.navigator.msPointerEnabled,
        startPoint: function (e) {
            var data = touchObj(e);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY],
                origin: $(e.target)
            };
        },
        stopPoint: function (e) {
            var data = touchObj(e);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY]
            };
        },
        handleSwipe: function (startPoint, stopPoint, e, _startevent) {

            if (stopPoint.time - startPoint.time < $.event.special.swipe.duration &&
				Math.abs(startPoint.coords[0] - stopPoint.coords[0]) > $.event.special.swipe.horizontalDistance &&
				Math.abs(startPoint.coords[1] - stopPoint.coords[1]) < $.event.special.swipe.verticalDistance) {

                var _addDetails = { time: stopPoint.time - startPoint.time, _isSwipe: true, _isDelta: true, stopPoint: stopPoint };
                var _options = _getOptions(e, _addDetails, _startevent);

                startPoint.origin.trigger($.extend(true, { type: "swipe" }, _options))
					.trigger($.extend(true,
                    { type: startPoint.coords[0] > stopPoint.coords[0] ? "swipeleft" : "swiperight" }, _options));
            }
        },

        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);

            $this.on(touchStartEvent, function (e) {
                var startPoint = $.event.special.swipe.startPoint(e),
					stopPoint;
                var _startevent = e;
                $(e.target).data('_dataTouchStart', { event: e, _now: new Date().getTime() });

                function moveHandler(e) {
                    if (!startPoint) return;
                    stopPoint = $.event.special.swipe.stopPoint(e);
                    if (Math.abs(startPoint.coords[0] - stopPoint.coords[0]) > $.event.special.swipe.scrollSupression) e.preventDefault();
                }

                $this.on(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (e) {
					    $this.off(touchMoveEvent, moveHandler);
					    if (startPoint && stopPoint) {
					        $.event.special.swipe.handleSwipe(startPoint, stopPoint, e, _startevent);
					    }
					    startPoint = stopPoint = undefined;
					});
            });
        }
    };
    // also handles pinchin, pinchout
    $.event.special.pinch = {
        distance: function (e) {
            if (e.originalEvent.touches.length < 2) return null;
            return $.event.special.pinch._getdistance(e.originalEvent.touches[0], e.originalEvent.touches[1]);
        },
        _getdistance: function (coor1, coor2) {
            return Math.sqrt((coor1.pageX - coor2.pageX) * (coor1.pageX - coor2.pageX) +
                (coor1.pageY - coor2.pageY) * (coor1.pageY - coor2.pageY));
        },
        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);
            $this.on(touchStartEvent, function (e) {
                var _startevent = e;
                if (e.originalEvent.touches && e.originalEvent.touches.length >= 2) {
                    var startPoint = $.event.special.pinch.distance(e), stopPoint, minDistance = 5;

                    var _pinchDistance = startPoint, moveEvent;
                    var _options = _getOptions(e, { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent);

                    $(e.target).trigger($.extend(true, { type: "pinch" }, _options));

                    function moveHandler(e) {
                        moveEvent = e;
                        stopPoint = $.event.special.pinch.distance(e) || null;
                        if (startPoint && stopPoint && Math.abs(startPoint - stopPoint) > minDistance) {

                            $(e.target).trigger($.extend(true,
                                { type: startPoint > stopPoint ? "pinchin" : "pinchout" }, _getOptions(e,
                                { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent)));

                            startPoint = stopPoint;
                        }
                    }

                    $this.on(touchMoveEvent, moveHandler)
                        .one(touchStopEvent, function () {
                            $this.off(touchMoveEvent, moveHandler);
                            $(e.target).trigger($.extend(true, { type: "pinchstop" }, _getOptions(moveEvent,
                                { _isPinch: true, _pinchDistance: stopPoint }, _startevent)));
                            startPoint = stopPoint = undefined;
                        });
                }
            });
        }
    };
    //  handles touchdrag, touchdrag
    $.event.special.touchdrag = {

        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);

            $this.on(touchStartEvent, function (e) {

                var startPoint = touchObj(e),
					stopPoint;
                var _startevent = e;
                $(e.target).data('_dataTouchStart', { event: e, _now: new Date().getTime() });

                function moveHandler(e) {
                    if (!startPoint) {
                        return;
                    }
                    stopPoint = touchObj(e);

                    if ($.event.special.pinch._getdistance(startPoint, stopPoint) > 5)
                        $(e.target).trigger($.extend(true, { type: "touchdrag" },
                            _getOptions(e, { _isdrag: true, stopPoint: stopPoint, _isDelta: true }, _startevent)));
                }

                $this.on(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (e) {
					    $this.off(touchMoveEvent, moveHandler);
					    startPoint = stopPoint = undefined;
					});
            });
        }
    };
    function _getBaseOptions(e) {
        var _pointer = isPointer ? e.originalEvent.pointerType : (!e.originalEvent.touches ? "mouse" : "touch")
        var pointerType = (isPointer && isIE11Pointer == undefined) ? (_pointer == 4 ? "mouse" : "touch") : _pointer;
        e["pointerType"] = pointerType;
        if (e.type != "mousedown" && e.type != "mouseup") {
            copyObject(e, e.originalEvent);
        }
        return e;
    }
    function _getOptions(e, _details, _startevent) {
        var _distance, _time, _scale, _iDelta = {}, _startXY, _endXY;
        if (_startevent) {
            var coor;
            if (!_startevent.originalEvent.touches) coor = [_startevent.originalEvent, e.originalEvent];
            else coor = [_startevent.originalEvent.touches[0], e.originalEvent.changedTouches[0]];

            if (_details._isSwipe || _details._isdrag) {
                _distance = $.event.special.pinch._getdistance(coor[0], coor[1]);
                _time = _details.time;
                _endXY = { pageX: _details.stopPoint.pageX, pageY: _details.stopPoint.pageY };
            }
            else if (_details._isPinch) {
                _distance = $.event.special.pinch.distance(e);
                _time = e.timeStamp - _startevent.timeStamp;
                _scale = _details._pinchDistance;
            }
            if (_details._isDelta) {
                _iDelta._dTime = e.timeStamp - _startevent.timeStamp;
                _iDelta._x = coor[1].pageX - coor[0].pageX;
                _iDelta._y = coor[1].pageY - coor[0].pageY;
            }
        }

        return {
            options: e,
            delta: {
                time: _iDelta._dTime || null,
                X: _iDelta._x || null,
                Y: _iDelta._y || null
            },
            distance: _distance,
            scale: _details._isPinch ? _scale : null,
            time: _time,
            velocity: {
                XY: _distance / _iDelta._dTime || null,
                X: _iDelta._x / _iDelta._dTime || null,
                Y: _iDelta._y / _iDelta._dTime || null
            },
            currentPosition: { pageX: _endXY ? _endXY.pageX : null, pageY: _endXY ? _endXY.pageY : null }
        };
    }

    function getTimeSpan() {
        var now = new Date();
        return now.getTime();
    }
    function checkMsieTouch($this) {
        if (isPointer)
            $this.css("-ms-touch-action", "pinch-zoom").css("touch-action", "pinch-zoom");
    }

    $.each({
        scrollstop: "scrollstart",
        doubletap: "tap",
        taphold: "tap",
        swipeleft: "swipe",
        swiperight: "swipe",
        swipedown: "swipeupdown",
        swipeup: "swipeupdown",
        pinchin: "pinch",
        pinchout: "pinch",
        pinchstop: "pinch"
    }, function (event, sourceEvent) {

        $.event.special[event] = {
            setup: function () {
                $(this).on(sourceEvent, $.noop);
            }
        };
    });

})(jQuery);;
(function($, undefined){
    
ej.globalize = {};
ej.cultures = {};

ej.cultures['default'] = ej.cultures['en-US'] = $.extend(true, {
    name: 'en-US',
    englishName: "English",
    nativeName: "English",
    language: 'en',
    isRTL: false,
    numberFormat: {
        pattern: ["-n"],
        decimals: 2,
        ',': ",",
        '.': ".",
        groupSizes: [3],
        '+': "+",
        '-': "-",
        percent: {
            pattern: ["-n %", "n %"],
            decimals: 2,
            groupSizes: [3],
            ',': ",",
            '.': ".",
            symbol: '%'
        },
        currency: {
            pattern: ["($n)", "$n"],
            decimals: 2,
            groupSizes: [3],
            ',': ",",
            '.': ".",
            symbol: '$'
        }
    },
    calendars: {
    	standard: {
	        '/': '/',
	        ':': ':',
	        firstDay: 0,
	        days: {
	            names: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
	            namesAbbr: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
	            namesShort: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]
	        },
	        months: {
	            names: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December", ""],
	            namesAbbr: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", ""]
	        },
	        AM: ['AM', 'am', 'AM'],
	        PM: ['PM', 'pm', 'PM'],
            twoDigitYearMax: 2029,
	        patterns: {
                d: "M/d/yyyy",
                D: "dddd, MMMM dd, yyyy",
                t: "h:mm tt",
                T: "h:mm:ss tt",
                f: "dddd, MMMM dd, yyyy h:mm tt",
                F: "dddd, MMMM dd, yyyy h:mm:ss tt",
                M: "MMMM dd",
                Y: "yyyy MMMM",
                S: "yyyy\u0027-\u0027MM\u0027-\u0027dd\u0027T\u0027HH\u0027:\u0027mm\u0027:\u0027ss"

	        }
    	}
    }
}, ej.cultures['en-US']);

ej.cultures['en-US'].calendar = ej.cultures['en-US'].calendar || ej.cultures['en-US'].calendars.standard; 



// *************************************** Numbers ***************************************
var regexTrim = /^\s+|\s+$/g,
    regexInfinity = /^[+-]?infinity$/i,
    regexHex = /^0x[a-f0-9]+$/i,
    regexParseFloat = /^[+-]?\d*\.?\d*(e[+-]?\d+)?$/;

function patternStartsWith(value, pattern) {
    return value.indexOf( pattern ) === 0;
}

function patternEndsWith(value, pattern) {
    return value.substr( value.length - pattern.length ) === pattern;
}

function trim(value) {
    return (value+"").replace( regexTrim, "" );
}

function truncate(value){
    if(isNaN(value))
        return NaN;
    
    return Math[value < 0 ? "ceil" : "floor"](value);
}

function padWithZero(str, count, left) {
    for (var l = str.length; l < count; l++) {
        str = (left ? ('0' + str) : (str + '0'));
    }
    return str;
}

function parseNumberWithNegativePattern(value, nf, negativePattern) {
    var neg = nf["-"],
        pos = nf["+"],
        ret;
    switch (negativePattern) {
        case "n -":
            neg = ' ' + neg;
            pos = ' ' + pos;
            // fall through
        case "n-":
            if ( patternEndsWith( value, neg ) ) {
                ret = [ '-', value.substr( 0, value.length - neg.length ) ];
            }
            else if ( patternEndsWith( value, pos ) ) {
                ret = [ '+', value.substr( 0, value.length - pos.length ) ];
            }
            break;
        case "- n":
            neg += ' ';
            pos += ' ';
            // fall through
        case "-n":
            if ( patternStartsWith( value, neg ) ) {
                ret = [ '-', value.substr( neg.length ) ];
            }
            else if ( patternStartsWith(value, pos) ) {
                ret = [ '+', value.substr( pos.length ) ];
            }
            break;
        case "(n)":
            if ( patternStartsWith( value, '(' ) && patternEndsWith( value, ')' ) ) {
                ret = [ '-', value.substr( 1, value.length - 2 ) ];
            }
            break;
    }
    return ret || [ '', value ];
}

function getFullNumber(number, precision, formatInfo) {
    var groupSizes = formatInfo.groupSizes || [3],
        curSize = groupSizes[0],
        curGroupIndex = 1,
        factor = Math.pow(10, precision),
        rounded = Math.round(number * factor) / factor;
    if (!isFinite(rounded)) {
        rounded = number;
    }
    number = rounded;

    var numberString = number + "",
        right = "",
        split = numberString.split(/e/i),
        exponent = split.length > 1 ? parseInt(split[1], 10) : 0;
    numberString = split[0];
    split = numberString.split(".");
    numberString = split[0];
    right = split.length > 1 ? split[1] : "";

    var l;
    if (exponent > 0) {
        right = padWithZero(right, exponent, false);
        numberString += right.slice(0, exponent);
        right = right.substr(exponent);
    } else if (exponent < 0) {
        exponent = -exponent;
        numberString = padWithZero(numberString, exponent + 1);
        right = numberString.slice(-exponent, numberString.length) + right;
        numberString = numberString.slice(0, -exponent);
    }

    var dot = formatInfo['.'] || '.';
    if (precision > 0) {
        right = dot +
            ((right.length > precision) ? right.slice(0, precision) : padWithZero(right, precision));
    } else {
        right = "";
    }

    var stringIndex = numberString.length - 1,
        sep = formatInfo[","] || ',',
        ret = "";

    while (stringIndex >= 0) {
        if (curSize === 0 || curSize > stringIndex) {
            return numberString.slice(0, stringIndex + 1) + (ret.length ? (sep + ret + right) : right);
        }
        ret = numberString.slice(stringIndex - curSize + 1, stringIndex + 1) + (ret.length ? (sep + ret) : "");

        stringIndex -= curSize;

        if (curGroupIndex < groupSizes.length) {
            curSize = groupSizes[curGroupIndex];
            curGroupIndex++;
        }
    }
    return numberString.slice(0, stringIndex + 1) + sep + ret + right;
}

function formatNumberToCulture(value, format, culture) {
    if (!format || format === 'i') {
        return culture.name.length ? value.toLocaleString() : value.toString();
    }
    format = format || "D";

    var nf = culture.numberFormat,
        number = Math.abs(value),
        precision = -1,
        pattern;

    if (format.length > 1) precision = parseInt(format.slice(1), 10);

    var current = format.charAt(0).toUpperCase(),
        formatInfo;

    switch (current) {
        case 'D':
            pattern = 'n';
            number = truncate(number);
            if (precision !== -1) {
                number = padWithZero("" + number, precision, true);
            }
            if (value < 0) number = -number;
            break;
        case 'N':
            formatInfo = nf;
            formatInfo.pattern = formatInfo.pattern || ['-n'];
            // fall through
        case 'C':
            formatInfo = formatInfo || nf.currency;
            formatInfo.pattern = formatInfo.pattern || ['-$n', '$n'];
            // fall through
        case 'P':
            formatInfo = formatInfo || nf.percent;
            formatInfo.pattern = formatInfo.pattern || ['-n %', 'n %'];
            pattern = value < 0 ? (formatInfo.pattern[0] || "-n") : (formatInfo.pattern[1] || "n");
            if (precision === -1) precision = formatInfo.decimals;
            number = getFullNumber(number * (current === "P" ? 100 : 1), precision, formatInfo);
            break;
        default:
            throw "Bad number format specifier: " + current;
    }

    return matchNumberToPattern(number, pattern, nf);
}



function matchNumberToPattern(number, pattern, nf){
    var patternParts = /n|\$|-|%/g,
        ret = "";
    for (;;) {
        var index = patternParts.lastIndex,
            ar = patternParts.exec(pattern);

        ret += pattern.slice(index, ar ? ar.index : pattern.length);

        if (!ar) {
            break;
        }

        switch (ar[0]) {
            case "n":
                ret += number;
                break;
            case "$":
                ret += nf.currency.symbol || "$";
                break;
            case "-":
                // don't make 0 negative
                if (/[1-9]/.test(number)) {
                    ret += nf["-"] || "-";
                }
                break;
            case "%":
                ret += nf.percent.symbol || "%";
                break;
        }
    }

    return ret;
}

// *************************************** Dates ***************************************

var dateFormat = {
    DAY_OF_WEEK_THREE_LETTER : "ddd",
    DAY_OF_WEEK_FULL_NAME : "dddd",
    DAY_OF_MONTH_SINGLE_DIGIT : "d",
    DAY_OF_MONTH_DOUBLE_DIGIT : "dd",
    MONTH_THREE_LETTER : "MMM",
    MONTH_FULL_NAME : "MMMM",
    MONTH_SINGLE_DIGIT : "M",
    MONTH_DOUBLE_DIGIT : "MM",
    YEAR_SINGLE_DIGIT : "y",
    YEAR_DOUBLE_DIGIT : "yy",
    YEAR_FULL : "yyyy",
    HOURS_SINGLE_DIGIT_12_HOUR_CLOCK : "h",
    HOURS_DOUBLE_DIGIT_12_HOUR_CLOCK : "hh",
    HOURS_SINGLE_DIGIT_24_HOUR_CLOCK : "H",
    HOURS_DOUBLE_DIGIT_24_HOUR_CLOCK : "HH",
    MINUTES_SINGLE_DIGIT : "m",
    MINUTES_DOUBLE_DIGIT : "mm",
    SECONDS_SINGLE_DIGIT : "s",
    SECONDS_DOUBLE_DIGIT : "ss",
    MERIDIAN_INDICATOR_SINGLE : "t",
    MERIDIAN_INDICATOR_FULL : "tt",
    DECISECONDS : "f",
    CENTISECONDS: "ff",
    MILLISECONDS : "fff",
    TIME_ZONE_OFFSET_SINGLE_DIGIT : "z",
    TIME_ZONE_OFFSET_DOUBLE_DIGIT : "zz",
    TIME_ZONE_OFFSET_FULL : "zzz",
    DATE_SEPARATOR : "/"
};

function valueOutOfRange(value, low, high) {
    return value < low || value > high;
}

function expandYear(cal, year) {
    // expands 2-digit year into 4 digits.
    var now = new Date();
    if ( year < 100 ) {
        var twoDigitYearMax = cal.twoDigitYearMax;
        twoDigitYearMax = typeof twoDigitYearMax === 'string' ? new Date().getFullYear() % 100 + parseInt( twoDigitYearMax, 10 ) : twoDigitYearMax;
        var curr = now.getFullYear();
        year += curr - ( curr % 100 );
        if ( year > twoDigitYearMax ) {
            year -= 100;
        }
    }
    return year;
}

function arrayIndexOf( array, item ) {
    if ( array.indexOf ) {
        return array.indexOf( item );
    }
    for ( var i = 0, length = array.length; i < length; i++ ) {
        if ( array[ i ] === item ) return i;
    }
    return -1;
}

function toUpper(value) {
    // 'he-IL' has non-breaking space in weekday names.
    return value.split( "\u00A0" ).join(' ').toUpperCase();
}

function toUpperArray(arr) {
    var results = [];
    for ( var i = 0, l = arr.length; i < l; i++ ) {
        results[i] = toUpper(arr[i]);
    }
    return results;
}

function getIndexOfDay(cal, value, abbr) {
    var ret,
        days = cal.days,
        upperDays = cal._upperDays;
    if ( !upperDays ) {
        cal._upperDays = upperDays = [
            toUpperArray( days.names ),
            toUpperArray( days.namesAbbr ),
            toUpperArray( days.namesShort )
        ];
    }
    value = toUpper( value );
    if ( abbr ) {
        ret = arrayIndexOf( upperDays[ 1 ], value );
        if ( ret === -1 ) {
            ret = arrayIndexOf( upperDays[ 2 ], value );
        }
    }
    else {
        ret = arrayIndexOf( upperDays[ 0 ], value );
    }
    return ret;
}

function getIndexOfMonth(cal, value, abbr) {
    var months = cal.months,
        monthsGen = cal.monthsGenitive || cal.months,
        upperMonths = cal._upperMonths,
        upperMonthsGen = cal._upperMonthsGen;
    if ( !upperMonths ) {
        cal._upperMonths = upperMonths = [
            toUpperArray( months.names ),
            toUpperArray( months.namesAbbr )
        ];
        cal._upperMonthsGen = upperMonthsGen = [
            toUpperArray( monthsGen.names ),
            toUpperArray( monthsGen.namesAbbr )
        ];
    }
    value = toUpper( value );
    var i = arrayIndexOf( abbr ? upperMonths[ 1 ] : upperMonths[ 0 ], value );
    if ( i < 0 ) {
        i = arrayIndexOf( abbr ? upperMonthsGen[ 1 ] : upperMonthsGen[ 0 ], value );
    }
    return i;
}

function appendMatchStringCount(preMatch, strings) {
    var quoteCount = 0,
        escaped = false;
    for ( var i = 0, il = preMatch.length; i < il; i++ ) {
        var c = preMatch.charAt( i );
        if(c == '\''){
            escaped ? strings.push( "'" ) : quoteCount++;
            escaped = false;
        } else if( c == '\\'){
            if (escaped) strings.push( "\\" );
            escaped = !escaped;
        } else {
            strings.push( c );
            escaped = false;
        }
    }
    return quoteCount;
}


function parseDayByInt(value, format, culture, cal) {
    if (!value) {
        return null;
    }
    var index = 0, valueX = 0, day = null;
    format = format.split("");
    length = format.length;
    var countDays = function (match) {
        var i = 0;
        while (format[index] === match) {
            i++;
            index++;
        }
        if (i > 0) {
            index -= 1;
        }
        return i;
    },
    getNumber = function (size) {
        var rg = new RegExp('^\\d{1,' + size + '}'),
            match = value.substr(valueX, size).match(rg);

        if (match) {
            match = match[0];
            valueX += match.length;
            return parseInt(match, 10);
        }
        return null;
    },
    getName = function (names, lower) {
        var i = 0,
            length = names.length,
            name, nameLength,
            subValue;

        for (; i < length; i++) {
            name = names[i];
            nameLength = name.length;
            subValue = value.substr(valueX, nameLength);

            if (lower) {
                subValue = subValue.toLowerCase();
            }

            if (subValue == name) {
                valueX += nameLength;
                return i + 1;
            }
        }
        return null;
    },
     lowerArray = function (data) {
         var index = 0,
             length = data.length,
             array = [];

         for (; index < length; index++) {
             array[index] = (data[index] + "").toLowerCase();
         }

         return array;
     },
     lowerInfo = function (localInfo) {
         var newLocalInfo = {}, property;

         for (property in localInfo) {
             newLocalInfo[property] = lowerArray(localInfo[property]);
         }

         return newLocalInfo;
     };
    for (; index < length; index++) {
        ch = format[index];
        if (ch === "d") {
            count = countDays("d");
            if (!cal._lowerDays) {
                cal._lowerDays = lowerInfo(cal.days);
            }
            day = count < 3 ? getNumber(2) : getName(cal._lowerDays[count == 3 ? "namesAbbr" : "names"], true)
        }
    }
    return day;
}


function getFullDateFormat(cal, format) {
    // expands unspecified or single character date formats into the full pattern.
    format = format || "F";
    var pattern,
        patterns = cal.patterns,
        len = format.length;
    if ( len === 1 ) {
        pattern = patterns[ format ];
        if ( !pattern ) {
            throw "Invalid date format string '" + format + "'.";
        }
        format = pattern;
    }
    else if ( len === 2  && format.charAt(0) === "%" ) {
        // %X escape format -- intended as a custom format string that is only one character, not a built-in format.
        format = format.charAt( 1 );
    }
    return format;
}

ej.globalize._getDateParseRegExp = function (cal, format) {
    // converts a format string into a regular expression with groups that
    // can be used to extract date fields from a date string.
    // check for a cached parse regex.
    var re = cal._parseRegExp;
    if ( !re ) {
        cal._parseRegExp = re = {};
    }
    else {
        var reFormat = re[ format ];
        if ( reFormat ) {
            return reFormat;
        }
    }

    // expand single digit formats, then escape regular expression characters.
    var expFormat = getFullDateFormat( cal, format ).replace( /([\^\$\.\*\+\?\|\[\]\(\)\{\}])/g, "\\\\$1" ),
        regexp = ["^"],
        groups = [],
        index = 0,
        quoteCount = 0,
        tokenRegExp = /\/|dddd|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|y|hh|h|HH|H|mm|m|ss|s|tt|t|fff|ff|f|zzz|zz|z|gg|g/g,
        match;

    // iterate through each date token found.
    while ( (match = tokenRegExp.exec( expFormat )) !== null ) {
        var preMatch = expFormat.slice( index, match.index );
        index = tokenRegExp.lastIndex;

        // don't replace any matches that occur inside a string literal.
        quoteCount += appendMatchStringCount( preMatch, regexp );
        if ( quoteCount % 2 ) {
            regexp.push( match[ 0 ] );
            continue;
        }

        // add a regex group for the token.
        var m = match[ 0 ],
            len = m.length,
            add;
            
        switch ( m ) {
            case dateFormat.DAY_OF_WEEK_THREE_LETTER: case dateFormat.DAY_OF_WEEK_FULL_NAME:
            case dateFormat.MONTH_FULL_NAME: case dateFormat.MONTH_THREE_LETTER:
                add = "(\\D+)";
                break;
            case dateFormat.MERIDIAN_INDICATOR_FULL: case dateFormat.MERIDIAN_INDICATOR_SINGLE:
                add = "(\\D*)";
                break;
            case dateFormat.YEAR_FULL:
            case dateFormat.MILLISECONDS:
            case dateFormat.CENTISECONDS:
            case dateFormat.DECISECONDS:
                add = "(\\d{" + len + "})";
                break;
            case dateFormat.DAY_OF_MONTH_DOUBLE_DIGIT: case dateFormat.DAY_OF_MONTH_SINGLE_DIGIT:
            case dateFormat.MONTH_DOUBLE_DIGIT: case dateFormat.MONTH_SINGLE_DIGIT:
            case dateFormat.YEAR_DOUBLE_DIGIT: case dateFormat.YEAR_SINGLE_DIGIT:
            case dateFormat.HOURS_DOUBLE_DIGIT_24_HOUR_CLOCK: case dateFormat.HOURS_SINGLE_DIGIT_24_HOUR_CLOCK:
            case dateFormat.HOURS_DOUBLE_DIGIT_12_HOUR_CLOCK: case dateFormat.HOURS_SINGLE_DIGIT_12_HOUR_CLOCK:
            case dateFormat.MINUTES_DOUBLE_DIGIT: case dateFormat.MINUTES_SINGLE_DIGIT:
            case dateFormat.SECONDS_DOUBLE_DIGIT: case dateFormat.SECONDS_SINGLE_DIGIT:
                add = "(\\d\\d?)";
                break;
            case dateFormat.TIME_ZONE_OFFSET_FULL:
                add = "([+-]?\\d\\d?:\\d{2})";
                break;
            case dateFormat.TIME_ZONE_OFFSET_DOUBLE_DIGIT: case dateFormat.TIME_ZONE_OFFSET_SINGLE_DIGIT:
                add = "([+-]?\\d\\d?)";
                break;
            case dateFormat.DATE_SEPARATOR:
                add = "(\\" + cal["/"] + ")";
                break;
            default:
                throw "Invalid date format pattern '" + m + "'.";
                break;
        }
        if ( add ) {
            regexp.push( add );
        }
        groups.push( match[ 0 ] );
    }
    appendMatchStringCount( expFormat.slice( index ), regexp );
    regexp.push( "$" );

    // allow whitespace to differ when matching formats.
    var regexpStr = regexp.join( '' ).replace( /\s+/g, "\\s+" ),
        parseRegExp = {'regExp': regexpStr, 'groups': groups};

    // cache the regex for this format.
    return re[ format ] = parseRegExp;
}

function getParsedDate(value, format, culture) {
    // try to parse the date string by matching against the format string
    // while using the specified culture for date field names.
    value = trim( value );
    format = trim(format);
    var cal = culture.calendar,
        // convert date formats into regular expressions with groupings.
        parseInfo = ej.globalize._getDateParseRegExp(cal, format),
        match = new RegExp(parseInfo.regExp).exec(value);
    if (match === null) {
        return null;
    }
    // found a date format that matches the input.
    var groups = parseInfo.groups,
        year = null, month = null, date = null, weekDay = null,
        hour = 0, hourOffset, min = 0, sec = 0, msec = 0, tzMinOffset = null,
        pmHour = false;
    // iterate the format groups to extract and set the date fields.
    for ( var j = 0, jl = groups.length; j < jl; j++ ) {
        var matchGroup = match[ j + 1 ];
        if ( matchGroup ) {
            var current = groups[ j ],
                clength = current.length,
                matchInt = parseInt( matchGroup, 10 );
            
            switch ( current ) {
                case dateFormat.DAY_OF_MONTH_DOUBLE_DIGIT: case dateFormat.DAY_OF_MONTH_SINGLE_DIGIT:
                    date = matchInt;
                    if ( valueOutOfRange( date, 1, 31 ) ) return null;
                    break;
                case dateFormat.MONTH_THREE_LETTER:
                case dateFormat.MONTH_FULL_NAME:
                    month = getIndexOfMonth( cal, matchGroup, clength === 3 );
                    if ( valueOutOfRange( month, 0, 11 ) ) return null;
                    break;
                case dateFormat.MONTH_SINGLE_DIGIT: case dateFormat.MONTH_DOUBLE_DIGIT:
                    month = matchInt - 1;
                    if ( valueOutOfRange( month, 0, 11 ) ) return null;
                    break;
                case dateFormat.YEAR_SINGLE_DIGIT: case dateFormat.YEAR_DOUBLE_DIGIT:
                case dateFormat.YEAR_FULL:
                    year = clength < 4 ? expandYear( cal, matchInt ) : matchInt;
                    if ( valueOutOfRange( year, 0, 9999 ) ) return null;
                    break;
                case dateFormat.HOURS_SINGLE_DIGIT_12_HOUR_CLOCK: case dateFormat.HOURS_DOUBLE_DIGIT_12_HOUR_CLOCK:
                    hour = matchInt;
                    if ( hour === 12 ) hour = 0;
                    if ( valueOutOfRange( hour, 0, 11 ) ) return null;
                    break;
                case dateFormat.HOURS_SINGLE_DIGIT_24_HOUR_CLOCK: case dateFormat.HOURS_DOUBLE_DIGIT_24_HOUR_CLOCK:
                    hour = matchInt;
                    if ( valueOutOfRange( hour, 0, 23 ) ) return null;
                    break;
                case dateFormat.MINUTES_SINGLE_DIGIT: case dateFormat.MINUTES_DOUBLE_DIGIT:
                    min = matchInt;
                    if ( valueOutOfRange( min, 0, 59 ) ) return null;
                    break;
                case dateFormat.SECONDS_SINGLE_DIGIT: case dateFormat.SECONDS_DOUBLE_DIGIT:
                    sec = matchInt;
                    if ( valueOutOfRange( sec, 0, 59 ) ) return null;
                    break;
                case dateFormat.MERIDIAN_INDICATOR_FULL: case dateFormat.MERIDIAN_INDICATOR_SINGLE:
                    pmHour = cal.PM && ( matchGroup === cal.PM[0] || matchGroup === cal.PM[1] || matchGroup === cal.PM[2] );
                    if ( !pmHour && ( !cal.AM || (matchGroup !== cal.AM[0] && matchGroup !== cal.AM[1] && matchGroup !== cal.AM[2]) ) ) return null;
                    break;
                case dateFormat.DECISECONDS:
                case dateFormat.CENTISECONDS:
                case dateFormat.MILLISECONDS:
                    msec = matchInt * Math.pow( 10, 3-clength );
                    if ( valueOutOfRange( msec, 0, 999 ) ) return null;
                    break;
                case dateFormat.DAY_OF_WEEK_THREE_LETTER:
                    date = parseDayByInt(value, format, culture, cal);
                    break;
                case dateFormat.DAY_OF_WEEK_FULL_NAME:
                     getIndexOfDay( cal, matchGroup, clength === 3 );
                    if ( valueOutOfRange( weekDay, 0, 6 ) ) return null;
                    break;
                case dateFormat.TIME_ZONE_OFFSET_FULL:
                    var offsets = matchGroup.split( /:/ );
                    if ( offsets.length !== 2 ) return null;

                    hourOffset = parseInt( offsets[ 0 ], 10 );
                    if ( valueOutOfRange( hourOffset, -12, 13 ) ) return null;
                    
                    var minOffset = parseInt( offsets[ 1 ], 10 );
                    if ( valueOutOfRange( minOffset, 0, 59 ) ) return null;
                    
                    tzMinOffset = (hourOffset * 60) + (patternStartsWith( matchGroup, '-' ) ? -minOffset : minOffset);
                    break;
                case dateFormat.TIME_ZONE_OFFSET_SINGLE_DIGIT: case dateFormat.TIME_ZONE_OFFSET_DOUBLE_DIGIT:
                    // Time zone offset in +/- hours.
                    hourOffset = matchInt;
                    if ( valueOutOfRange( hourOffset, -12, 13 ) ) return null;
                    tzMinOffset = hourOffset * 60;
                    break;
            }
        }
    }
    var result = new Date(), defaultYear, convert = cal.convert;
    defaultYear = convert ? convert.fromGregorian( result )[ 0 ] : result.getFullYear();
    if ( year === null ) {
        year = defaultYear;
    }
    
    // set default day and month to 1 and January, so if unspecified, these are the defaults
    // instead of the current day/month.
    if ( month === null ) {
        month = 0;
    }
    if ( date === null ) {
        date = 1;
    }
    // now have year, month, and date, but in the culture's calendar.
    if ( convert ) {
        result = convert.toGregorian( year, month, date );
        if ( result === null ) return null;
    }
    else {
        // have to set year, month and date together to avoid overflow based on current date.
        result.setFullYear( year, month, date );
        // check to see if date overflowed for specified month (only checked 1-31 above).
        if ( result.getDate() !== date ) return null;
        // invalid day of week.
        if ( weekDay !== null && result.getDay() !== weekDay ) {
            return null;
        }
    }
    // if pm designator token was found make sure the hours fit the 24-hour clock.
    if ( pmHour && hour < 12 ) {
        hour += 12;
    }
    result.setHours( hour, min, sec, msec );
    if ( tzMinOffset !== null ) {
        var adjustedMin = result.getMinutes() - ( tzMinOffset + result.getTimezoneOffset() );
        result.setHours( result.getHours() + parseInt( adjustedMin / 60, 10 ), adjustedMin % 60 );
    }
    return result;
}


function formatDateToCulture(value, format, culture) {
    var cal = culture.calendar,
        convert = cal.convert;
    if ( !format || !format.length || format === 'i' ) {
        var ret;
        if ( culture && culture.name.length ) {
            if ( convert ) {
                // non-gregorian calendar, so we cannot use built-in toLocaleString()
                ret = formatDateToCulture( value, cal.patterns.F, culture );
            }
            else {
                ret = value.toLocaleString();
            }
        }
        else {
            ret = value.toString();
        }
        return ret;
    }

    var sortable = format === "s";
        format = getFullDateFormat(cal, format);


    // Start with an empty string
    ret = [];
    var hour,
        zeros = ['0','00','000'],
        foundDay,
        checkedDay,
        dayPartRegExp = /([^d]|^)(d|dd)([^d]|$)/g,
        quoteCount = 0,
        tokenRegExp = /\/|dddd|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|y|hh|h|HH|H|mm|m|ss|s|tt|t|fff|ff|f|zzz|zz|z|gg|g/g,
        converted;

    function padWithZeros(num, c) {
        var r, s = num+'';
        if ( c > 1 && s.length < c ) {
            r = ( zeros[ c - 2 ] + s);
            return r.substr( r.length - c, c );
        }
        else {
            r = s;
        }
        return r;
    }

    function hasDay() {
        if ( foundDay || checkedDay ) {
            return foundDay;
        }
        foundDay = dayPartRegExp.test( format );
        checkedDay = true;
        return foundDay;
    }

    if ( !sortable && convert ) {
        converted = convert.fromGregorian( value );
    }

    for (;;) {
        // Save the current index
        var index = tokenRegExp.lastIndex,
            // Look for the next pattern
            ar = tokenRegExp.exec( format );

        // Append the text before the pattern (or the end of the string if not found)
        var preMatch = format.slice( index, ar ? ar.index : format.length );
        quoteCount += appendMatchStringCount( preMatch, ret );

        if ( !ar ) {
            break;
        }

        // do not replace any matches that occur inside a string literal.
        if ( quoteCount % 2 ) {
            ret.push( ar[ 0 ] );
            continue;
        }

        var current = ar[ 0 ],
            clength = current.length;


        switch ( current ) {
            case dateFormat.DAY_OF_WEEK_THREE_LETTER:
            case dateFormat.DAY_OF_WEEK_FULL_NAME:
                var names = (clength === 3) ? cal.days.namesAbbr : cal.days.names;
                ret.push( names[ value.getDay() ] );
                break;
            case dateFormat.DAY_OF_MONTH_SINGLE_DIGIT:
            case dateFormat.DAY_OF_MONTH_DOUBLE_DIGIT:
                foundDay = true;
                ret.push( padWithZeros( (converted ? converted[2] : value.getDate()), clength ) );
                break;
            case dateFormat.MONTH_THREE_LETTER:
            case dateFormat.MONTH_FULL_NAME:
                var part = converted ? converted[1] : value.getMonth();
                ret.push( (cal.monthsGenitive && hasDay())
                    ? cal.monthsGenitive[ clength === 3 ? "namesAbbr" : "names" ][ part ]
                    : cal.months[ clength === 3 ? "namesAbbr" : "names" ][ part ] );
                break;
            case dateFormat.MONTH_SINGLE_DIGIT:
            case dateFormat.MONTH_DOUBLE_DIGIT:
                ret.push( padWithZeros((converted ? converted[1] : value.getMonth()) + 1, clength ) );
                break;
            case dateFormat.YEAR_SINGLE_DIGIT:
            case dateFormat.YEAR_DOUBLE_DIGIT:
            case dateFormat.YEAR_FULL:
                part = converted ? converted[ 0 ] : value.getFullYear();
                if ( clength < 4 ) {
                    part = part % 100;
                }
                ret.push( padWithZeros( part, clength ) );
                break;
            case dateFormat.HOURS_SINGLE_DIGIT_12_HOUR_CLOCK:
            case dateFormat.HOURS_DOUBLE_DIGIT_12_HOUR_CLOCK:
                hour = value.getHours() % 12;
                if ( hour === 0 ) hour = 12;
                ret.push( padWithZeros( hour, clength ) );
                break;
            case dateFormat.HOURS_SINGLE_DIGIT_24_HOUR_CLOCK:
            case dateFormat.HOURS_DOUBLE_DIGIT_24_HOUR_CLOCK:
                ret.push( padWithZeros( value.getHours(), clength ) );
                break;
            case dateFormat.MINUTES_SINGLE_DIGIT:
            case dateFormat.MINUTES_DOUBLE_DIGIT:
                ret.push( padWithZeros( value.getMinutes(), clength ) );
                break;
            case dateFormat.SECONDS_SINGLE_DIGIT:
            case dateFormat.SECONDS_DOUBLE_DIGIT:
                ret.push( padWithZeros(value .getSeconds(), clength ) );
                break;
            case dateFormat.MERIDIAN_INDICATOR_SINGLE:
            case dateFormat.MERIDIAN_INDICATOR_FULL:
                part = value.getHours() < 12 ? (cal.AM ? cal.AM[0] : " ") : (cal.PM ? cal.PM[0] : " ");
                ret.push( clength === 1 ? part.charAt( 0 ) : part );
                break;
            case dateFormat.DECISECONDS:
            case dateFormat.CENTISECONDS:
            case dateFormat.MILLISECONDS:
                ret.push( padWithZeros( value.getMilliseconds(), 3 ).substr( 0, clength ) );
                break;
            case dateFormat.TIME_ZONE_OFFSET_SINGLE_DIGIT:
            case dateFormat.TIME_ZONE_OFFSET_DOUBLE_DIGIT:
                hour = value.getTimezoneOffset() / 60;
                ret.push( (hour <= 0 ? '+' : '-') + padWithZeros( Math.floor( Math.abs( hour ) ), clength ) );
                break;
            case dateFormat.TIME_ZONE_OFFSET_FULL:
                hour = value.getTimezoneOffset() / 60;
                ret.push( (hour <= 0 ? '+' : '-') + padWithZeros( Math.floor( Math.abs( hour ) ), 2 ) +
                    ":" + padWithZeros( Math.abs( value.getTimezoneOffset() % 60 ), 2 ) );
                break;
            case dateFormat.DATE_SEPARATOR:
                ret.push( cal["/"] || "/" );
                break;
            default:
                throw "Invalid date format pattern '" + current + "'.";
                break;
        }
    }
    return ret.join( '' );
}

//add new culture into ej 
ej.globalize.addCulture = function (name, culture) {
    ej.cultures[name] = $.extend(true, $.extend(true, {}, ej.cultures['default'], culture), ej.cultures[name]);
	ej.cultures[name].calendar = ej.cultures[name].calendars.standard;
}

//return the specified culture or default if not found
ej.globalize.preferredCulture = function (culture) {
    culture = (typeof culture != "undefined" && typeof culture === typeof this.cultureObject) ? culture.name : culture;
    this.cultureObject = ej.globalize.findCulture(culture);
    return this.cultureObject;
}
ej.globalize.setCulture = function (culture) {
	if (ej.isNullOrUndefined(this.globalCultureObject)) this.globalCultureObject = ej.globalize.findCulture(culture);
	culture = (typeof culture != "undefined" && typeof culture === typeof this.globalCultureObject) ? culture.name : culture;
    if (culture) this.globalCultureObject = ej.globalize.findCulture(culture);
    ej.cultures.current = this.globalCultureObject;
    return this.globalCultureObject;
}
ej.globalize.culture=function(name){
    ej.cultures.current = ej.globalize.findCulture(name);
}

//return the specified culture or current else default if not found
ej.globalize.findCulture = function (culture) {
    var cultureObject;
    if (culture) {

        if ($.isPlainObject(culture) && culture.numberFormat) {
            cultureObject = culture;
        }
        if (typeof culture === "string") {
            var cultures = ej.cultures;
            if (cultures[culture]) {
                return cultures[culture];
            }
            else {
                if (culture.indexOf("-") > -1) {
                    var cultureShortName = culture.split("-")[0];
                    if (cultures[cultureShortName]) {
                        return cultures[cultureShortName];
                    }
                }
                else {
                    var cultureArray = $.map(cultures, function (el) { return el });
                    for (var i = 0; i < cultureArray.length; i++) {
                        var shortName = cultureArray[i].name.split("-")[0];
                        if (shortName === culture) {
                            return cultureArray[i];
                        }
                    };
                }
            }
            return ej.cultures["default"];
        }
    }
    else {
        cultureObject = ej.cultures.current || ej.cultures["default"];
    }

    return cultureObject;
}
//formatting date and number based on given format
ej.globalize.format = function (value, format, culture) {
    var cultureObject =  ej.globalize.findCulture(culture);
    if (typeof(value) === 'number') {
        value = formatNumberToCulture(value, format, cultureObject);
    } else if(value instanceof Date){
    	value = formatDateToCulture(value, format, cultureObject);
    }

    return value;
}

//parsing integer takes string as input and return as number
ej.globalize.parseInt = function(value, radix, culture) {
    return Math.floor( this.parseFloat( value, radix, culture ) );
}

//returns the ISO date string from date object
ej.globalize.getISODate = function(value) {
    if(value instanceof Date) return value.toISOString();
}

//parsing floationg poing number takes string as input and return as number
ej.globalize.parseFloat = function(value, radix, culture) {
    // make radix optional
    if (typeof radix === "string") {
        culture = radix;
        radix = 10;
    }

    culture = ej.globalize.findCulture(culture);
    var ret = NaN,
        nf = culture.numberFormat,
        npattern = culture.numberFormat.pattern[0];
    value = value.replace(/ /g, '');
    if (value.indexOf(culture.numberFormat.currency.symbol) > -1) {
        // remove currency symbol
        value = value.replace(culture.numberFormat.currency.symbol || "$", "");
        // replace decimal seperator
        value = value.replace(culture.numberFormat.currency["."] || ".", culture.numberFormat["."] || ".");
        // pattern of the currency
        npattern = trim(culture.numberFormat.currency.pattern[0].replace("$", ""));
    } else if (value.indexOf(culture.numberFormat.percent.symbol) > -1) {
        // remove percentage symbol
        value = value.replace(culture.numberFormat.percent.symbol || "%", "");
        // replace decimal seperator
        value = value.replace(culture.numberFormat.percent["."] || ".", culture.numberFormat["."] || ".");
        // pattern of the percent
        npattern = trim(culture.numberFormat.percent.pattern[0].replace("%", ""));
    }

    // trim leading and trailing whitespace
    value = trim( value );

    // allow infinity or hexidecimal
    if (regexInfinity.test(value)) {
        ret = parseFloat(value, radix);
    }
    else if (!radix && regexHex.test(value)) {
        ret = parseInt(value, 16);
    }
    else {
        var signInfo = parseNumberWithNegativePattern( value, nf, npattern ),
            sign = signInfo[0],
            num = signInfo[1];
        // determine sign and number
        if ( sign === "" && nf.pattern[0] !== "-n" ) {
            signInfo = parseNumberWithNegativePattern( value, nf, "-n" );
            sign = signInfo[0];
            num = signInfo[1];
        }
        sign = sign || "+";
        // determine exponent and number
        var exponent,
            intAndFraction,
            exponentPos = num.indexOf( 'e' );
        if ( exponentPos < 0 ) exponentPos = num.indexOf( 'E' );
        if ( exponentPos < 0 ) {
            intAndFraction = num;
            exponent = null;
        }
        else {
            intAndFraction = num.substr( 0, exponentPos );
            exponent = num.substr( exponentPos + 1 );
        }
        // determine decimal position
        var integer,
            fraction,
            decSep = nf['.'] || '.',
            decimalPos = intAndFraction.indexOf( decSep );
        if ( decimalPos < 0 ) {
            integer = intAndFraction;
            fraction = null;
        }
        else {
            integer = intAndFraction.substr( 0, decimalPos );
            fraction = intAndFraction.substr( decimalPos + decSep.length );
        }
        // handle groups (e.g. 1,000,000)
        var groupSep = nf[","] || ",";
        integer = integer.split(groupSep).join('');
        var altGroupSep = groupSep.replace(/\u00A0/g, " ");
        if ( groupSep !== altGroupSep ) {
            integer = integer.split(altGroupSep).join('');
        }
        // build a natively parsable number string
        var p = sign + integer;
        if ( fraction !== null ) {
            p += '.' + fraction;
        }
        if ( exponent !== null ) {
            // exponent itself may have a number patternd
            var expSignInfo = parseNumberWithNegativePattern( exponent, nf, npattern );
            p += 'e' + (expSignInfo[0] || "+") + expSignInfo[1];
        }
        if ( regexParseFloat.test( p ) ) {
            ret = parseFloat( p );
        }
    }
    return ret;
}

//parsing date takes string as input and return as date object
ej.globalize.parseDate = function(value, formats, culture) {
    culture = ej.globalize.findCulture(culture);

    var date, prop, patterns;
    if ( formats ) {
        if ( typeof formats === "string" ) {
            formats = [ formats ];
        }
        if ( formats.length ) {
            for ( var i = 0, l = formats.length; i < l; i++ ) {
                var format = formats[ i ];
                if ( format ) {
                    date = getParsedDate( value, format, culture );
                    if ( date ) break;
                }
            }
        }
    }
    else {
        patterns = culture.calendar.patterns;
        for ( prop in patterns ) {
            date = getParsedDate( value, patterns[prop], culture );
            if ( date ) break;
        }
    }
    return date || null;
}

function getControlObject(obj, stringArray){
    return stringArray.length ? getControlObject(obj[stringArray[0]], stringArray.slice(1)) : obj;
}

//return localized constants as object for the given widget control and culture
ej.globalize.getLocalizedConstants = function(controlName, culture){
    var returnObject,
        controlNameArray = controlName.replace("ej.", "").split(".");
    
    returnObject = getControlObject(ej, controlNameArray);

    return ( $.extend(true, {}, returnObject.Locale['default'], returnObject.Locale[culture ? culture : this.cultureObject.name]) ) ;
}

$.extend(ej, ej.globalize);

}(jQuery));;
var eAnimation = eAnimation || {};
ejAnimation = {};

$.fn.ejAnimation = function (effectName, duration, easing) {
    var _deferred = $.Deferred();
    new eAnimation(this, effectName, duration, easing).done(function (ele) {
        return _deferred.resolve(ele);
    });
    return _deferred.promise();
}
eAnimation = function (ele, effectName, duration, easing) {
    this._deferred = $.Deferred();
    this._initAnimation(ele, effectName, duration, easing)
    return this._deferred.promise(this);
}
eAnimation.prototype = {
    _initAnimation: function (ele, effectName, duration, easing) {
        var ele = $(ele),
            $proxy = this;
        duration = duration ? duration : 400;
        easing = easing ? easing : ejAnimation.Easing.Ease;
        ele.off(ej.animationEndEvent());
        if (effectName == "rippleEffect") {
            this._createRippleSpan(ele);
            ej.listenTouchEvent(ele.addClass("e-ripple-parent"),ej.startEvent(), $.proxy(this._rippleAnimation, this));
        } else if (effectName == "stop" || effectName == "stopAnimation") {
            this.stopAnimation(ele);
        } else {
            ele.addClass("in-animation").on(ej.animationEndEvent(), function (evt) {
                ele.css("animation", "").removeClass("in-animation");
                ele.off(ej.animationEndEvent());
                return $proxy._deferred.resolve(ele);
            });
            ele.css((ej.userAgent() ? "-" + ej.userAgent() + "-" : "") + "animation", effectName + " " + duration + "ms " + easing);
        }
    },
    stopAnimation: function (ele) {
        ele.css((ej.userAgent() ? "-" + ej.userAgent() + "-" : "") + "animation", "").removeClass("in-animation");
        this._deferred.resolve(ele);
    },
    _rippleAnimation: function (e) {
       var point = e.changedTouches ? e.changedTouches[0] : e;
        var target = e.target,
            ele = $(e.currentTarget),
            $proxy = this,
            rect = target.getBoundingClientRect();
        ele.on(ej.animationEndEvent(), function (evt) {
            $(ripple).css("animation", "");
            return $proxy._deferred.resolve(this);
            ele.off(ej.animationEndEvent());
        });
        var effectEle = $(e.target).closest(".e-ripple");
        if (!effectEle.length) {
            effectEle = e.target;
        } else {
            effectEle = effectEle[0];
            rect = effectEle.getBoundingClientRect();
        }
        if ($(effectEle).hasClass("e-ripple")) {
            ripple = this._createRippleSpan(ele);
            $(ripple).appendTo(effectEle);
            ripple.style.height = ripple.style.width = Math.max(rect.width, rect.height) + 'px';
            var top = point.pageY - rect.top - ripple.offsetHeight / 2 - document.body.scrollTop;
            var left = point.pageX - rect.left - ripple.offsetWidth / 2 - document.body.scrollLeft;
            ripple.style.top = top + 'px';
            ripple.style.left = left + 'px';
            $(ripple).css("animation", "ripple .5s ease-out");
        }
    },
    _createRippleSpan: function (ele) {
        ripple = ele.find('.ripple')[0];
        if (!ripple) {
            ripple = document.createElement('span');
            ripple.className = 'ripple';
            ripple.style.height = ripple.style.width = "100%";
            ele[0].appendChild(ripple);
        }
        return ripple;
    }
};

ejAnimation.Effect = {
    SlideLeftIn: "slideLeftIn",
    SlideLeftOut: "slideLeftOut",
    SlideRightIn: "slideRightIn",
    SlideRightOut: "slideRightOut",
    SlideTopIn: "slideTopIn",
    SlideTopOut: "slideTopOut",
    SlideBottomIn: "slideBottomIn",
    SlideBottomOut: "slideBottomOut",
    SlideLeft: "slideLeft",
    SlideRight: "slideRight",
    SlideUp: "slideUp",
    SlideDown: "slideDown",
    FadeIn: "fadeIn",
    FadeOut: "fadeOut",
    FadeZoomIn: "fadeZoomIn",
    FadeZoomOut: "fadeZoomOut",
    ZoomIn: "zoomIn",
    ZoomOut: "zoomOut",
    FlipRightDownIn: "flipRightDownIn",
    FlipRightDownOut: "flipRightDownOut",
    FlipRightUpIn: "flipRightUpIn",
    FlipRightUpOut: "flipRightUpOut",
    FlipLeftDownIn: "flipLeftDownIn",
    FlipLeftDownOut: "flipLeftDownOut",
    FlipLeftUpIn: "flipLeftUpIn",
    FlipLeftUpOut: "flipLeftUpOut",
    FlipYRightIn: "flipYRightIn",
    FlipYRightOut: "flipYRightOut",
    FlipYLeftIn: "flipYLeftIn",
    FlipYLeftOut: "flipYLeftOut",
    FlipXUpIn: "flipXUpIn",
    FlipXUpOut: "flipXUpOut",
    FlipXDownIn: "flipXDownIn",
    FlipXDownOut: "flipXDownOut",
    RippleEffect: "rippleEffect"
}

ejAnimation.Easing = {
    Ease: "cubic-bezier(0.250, 0.100, 0.250, 1.000)",
    Linear: "cubic-bezier(0.250, 0.250, 0.750, 0.750)",
    Easein: "cubic-bezier(0.420, 0.000, 1.000, 1.000)",
    EaseOut: "cubic-bezier(0.000, 0.000, 0.580, 1.000)",
    EaseInOut: "cubic-bezier(0.420, 0.000, 0.580, 1.000)",
    Bounce: "cubic-bezier(0.215, 0.610, 0.355, 1.000)",
    ElasticInOut: "cubic-bezier(0.5,-0.58,0.38,1.81)",
    ElasticIn: "cubic-bezier(0.17,0.67,0.59,1.81)",
    ElasticOut: "cubic-bezier(0.7,-0.75,0.99,1.01)",

};
(function () {
    ejPageAnimation = {
        slideLeft: ["slideLeftOut", "slideRightIn", "slideRightOut", "slideLeftIn"],
        slideRight: ["slideRightOut", "slideLeftIn", "slideLeftOut", "slideRightIn"],
        slideUp: ["slideTopOut", "slideBottomIn", "slideBottomOut", "slideTopIn"],
        slideDown: ["slideBottomOut", "slideTopIn", "slideTopOut", "slideBottomIn"],
        flipIn: ["flipYLeftOut", "flipYRightIn", "flipYRightOut", "flipYLeftIn"],
        flipOut: ["flipYRightOut", "flipYLeftIn", "flipYLeftOut", "flipYRightIn"],
        pop: ["fadeZoomOut", "fadeZoomIn", "fadeZoomOut", "fadeZoomIn"]
    }
    App = {
        window: $(window),
        document: $(document),
        html: $("html"),
        head: $("head"),
        container: $("body"),
        model: {},
        _isBack: false,
        _initialHeap: true,
        currentStateId: null,
        checkStateId: null,
        activeSubPage: null,
        activePage: null,
        renderEJMControlsByDef: true,
        applyMobileStyles:true,
        defaults: {
            enableBrowserHistory: true,
            forceHash: false,
            enableAjax: "auto",
            ajaxOptions: {
                async: true,
                crossDomain: true,
                data: null,
                dataType: "html",
                contentType: "text/plain",
                method: "GET",
            },
            hashValue: null,
            enableCache: false,
            enableAnimation: true,
            animateOptions: {
                transitionType: "slideLeft",
                transitionDuration: "400",
                easing: "ease"

            },
            toPageClass:"",
            isReverseAnimation: false,
            customOption: null,
            viewTransfered: null,
            viewBeforeTransfer: null,
            ajaxSuccess: null,
            ajaxFailure: null,
            refreshed: null,
            onBack: null,
            onHistoryBack: null
        },

        _isUrl: function (url) {
            var regex = /[\.|\/|\#]/g;
            var result = regex.test(url);
            return result;
        },
        _getModel: function (target) {
            return target.data("historyModel");
        },
        _getEventArguments: function (target) {
            var data = {
                heap: target.data("heap") ? target.data("heap") : {},
                historyState: history.state ? {
                    target: history.state.target,
                    templateUrl: history.state.targetUrl,
                    templateContent: history.state.targetContent,
                    userModel: history.state.userModel,
                    uid: history.state.uid
                } : {},
                renderingPage: App._renderingPage
            };
            return data;
        },
        transferPage: function (fromPage, toPage, options) {
            ej.mobile.WaitingPopup.show()
            var fromPage = $("#" + fromPage);
            fromPage.addClass("page-container").data({
                historyModel: $.extend(false, App.defaults, options ? options : {})
            });
            var curModel = App._getModel(fromPage);
            var enableAjax = App._isUrl(toPage);            
            if (fromPage.children().first().attr("data-url")) {
                currentPageUrl = fromPage.children().first().attr("id");
                newpageurl = toPage;
                var pageIds = [];
                for (var i = 0; i < fromPage.data("heap").length; i++)
                    pageIds.push(fromPage.data("heap")[i].href);
                var heapIndex = pageIds.indexOf(newpageurl);
                if (heapIndex > -1) {
                    var duplicateHeap = fromPage.data("heap")[heapIndex];
                    fromPage.data("heap").splice(heapIndex, 1);
                    fromPage.data("heap").push(duplicateHeap);
                    App._loadExistingView(fromPage, toPage, curModel);
                    if (curModel.enableBrowserHistory) {
                        this._trimmedContent = $.trim(fromPage.children().html());
                        if (App._initialHeap) {
                            App._initialTargetId = $("#" + fromPage[0].id);
                            App._initialHeap = false;
                        }
                        if (duplicateHeap.href == toPage && heapIndex == fromPage.data("heap").length - 1) {
                            history.replaceState({
                                target: fromPage[0].id,
                                targetUrl: toPage,
                                targetContent: curModel.enableCache ? this._trimmedContent : "",
                                userModel: options,
                                uid: App.currentStateId
                            }, toPage, curModel.forceHash ? "#" + (curModel.hashValue ? curModel.hashValue : toPage) : App._isUrl(toPage) ? toPage : "#" + toPage);
                        } else {
                            history.pushState({
                                target: fromPage[0].id,
                                targetUrl: toPage,
                                targetContent: curModel.enableCache ? this._trimmedContent : "",
                                userModel: options,
                                uid: App.currentStateId
                            }, toPage, curModel.forceHash ? "#" + (curModel.hashValue ? curModel.hashValue : toPage) : App._isUrl(toPage) ? toPage : "#" + toPage);
                        }
                    }
                } else {
                    App._getContent(fromPage, toPage, curModel, true);
                }

            } else {
                var pageContainer = $("<div class='e-m-hidden' style='z-index:1'></div>");
                pageContainer.attr({
                    "id": "page_" + Math.floor((Math.random() * 1000) + 1) + "_container"
                });
                fromPage.wrapInner(pageContainer);
                App._pushHistory(fromPage, toPage, fromPage.html(), options);
                App._getContent(fromPage, toPage, curModel, true);
            }
            App.currentStateId = history.state ? history.state.uid + 1 : 1;
        },
        _updateContent: function (fromPage, toPage, options, templateContent) {
            var curModel = App._getModel(fromPage);
            var templateId = App._isUrl(toPage) ? "template_" + Math.floor((Math.random() * 1000) + 1) : toPage;
            App._trimmedContent = $.trim(templateContent);
            fromPage.css({
                "overflow": "hidden"
            });
            var templateContainer = $("<div id=" + templateId + "_container class='e-m-hidden' data-url=" + toPage + "></div>");
            templateContainer.html(App._trimmedContent);
            var page = $("<div></div>");
            page.append(templateContainer.clone());
            App._pushHistory(fromPage, toPage, page.html(), options);
            page.remove();
            App._renderingPage = templateContainer;
            fromPage.append(templateContainer);
            App._animatePages(fromPage, curModel.isReverseAnimation);
            if (curModel.enableBrowserHistory) {
                if (App._initialHeap) {
                    App._initialTargetId = $("#" + fromPage[0].id);
                    App._initialHeap = false;
                }
                history.pushState({
                    target: fromPage[0].id,
                    targetUrl: toPage,
                    targetContent: curModel.enableCache ? App._trimmedContent : "",
                    userModel: options,
                    uid: App.currentStateId
                }, toPage, curModel.forceHash ? "#" + (curModel.hashValue ? curModel.hashValue : toPage) : App._isUrl(toPage) ? toPage : "#" + toPage);
            }
            App._currentTarget = fromPage;
        },
        _animatePages: function (target, isReverse) {
            var curModel = App._getModel(target);
            if (typeof curModel.animateOptions.transitionType === "string")
                curModel.animateOptions.transitionType = ejPageAnimation[curModel.animateOptions.transitionType];
            if (typeof curModel.animateOptions.easing === "string")
                curModel.animateOptions.easing = ejAnimation.Easing[curModel.animateOptions.easing];            
            App._page1 = target.children().first().addClass("page");
            App._page2 = target.children().last().addClass("page " + curModel.toPageClass);            
            if (curModel.viewBeforeTransfer)
                App._trigger("viewBeforeTransfer", App._getEventArguments(target), target);
            var proxy = App;
            ej.widget.init(App._page2);
            if (!curModel.enableAnimation) {
                target.children().not(":last").remove();
                App._page2.removeClass("e-m-hidden page").css("z-index", "");
                ej.mobile.WaitingPopup.hide();
                if (curModel.viewTransfered)
                    App._trigger("viewTransfered", App._getEventArguments(target), target);
            } else {
                target.addClass("e-m-intransition");
                App._page1.ejAnimation("stop");
                App._page2.ejAnimation("stop");
                if (isReverse) {
                    proxy._page1.removeClass("e-m-hidden").ejAnimation(curModel.animateOptions.transitionType[2], curModel.animateOptions.transitionDuration, curModel.animateOptions.easing);
                    proxy._page2.removeClass("e-m-hidden").ejAnimation(curModel.animateOptions.transitionType[3], curModel.animateOptions.transitionDuration, curModel.animateOptions.easing).done($.proxy(function (e) {
                        e.siblings().remove();
                        ej.mobile.WaitingPopup.hide();
                        target.removeClass("e-m-intransition");
                        this._page2.css("z-index", "").removeClass("page");
                        if (curModel.viewTransfered)
                            App._trigger("viewTransfered", App._getEventArguments(target), target);
                    }, proxy));
                } else {
                    proxy._page1.removeClass("e-m-hidden").ejAnimation(curModel.animateOptions.transitionType[0], curModel.animateOptions.transitionDuration, curModel.animateOptions.easing);
                    proxy._page2.removeClass("e-m-hidden").ejAnimation(curModel.animateOptions.transitionType[1], curModel.animateOptions.transitionDuration, curModel.animateOptions.easing).done($.proxy(function (e) {
                        e.siblings().remove();
                        ej.mobile.WaitingPopup.hide();
                        target.removeClass("e-m-intransition");
                        this._page2.css("z-index", "").removeClass("page");
                        if (curModel.viewTransfered)
                            App._trigger("viewTransfered", App._getEventArguments(target), target);
                    }, proxy));
                }
            }

            App._isBack = false;
        },
        _pushHistory: function (target, template, templateContent, options) {
            var curModel = App._getModel(target);
            var heap = target.data("heap");
            var heapLength = heap ? heap.length : 0;
            currentPageUrl = curModel.enableAjax ? template : target.children().first().attr("id");
            if (heapLength == 0) {
                target.data("heap", [{
                    target: target,
                    href: target.children().first().attr("id"),
                    content: templateContent,
                    model: options
                }]);
            } else {
                target.data("heap").push({
                    target: target,
                    href: template,
                    content: curModel.enableCache ? templateContent : "",
                    model: options
                });
            }
        },
        _loadExistingView: function (target, template, options) {
            var curModel = App._getModel(target);
            var heap = target.data("heap");
            var heapLength = heap.length;
            heapLastContent = heap[heapLength - 1].content;
            heapLastUrl = heap[heapLength - 1].href;
            heapModel = heap[heapLength - 1].model;
            if (heapLastContent != "") {
                App._renderingPage = $(heapLastContent);
                target.append(App._renderingPage);
                App._animatePages(target, curModel.isReverseAnimation);
            } else
                App._getContent(target, heapLastUrl, heapModel, false);
            if (options.refreshed ? options.refreshed : curModel.refreshed) {
                App._trigger("refreshed", App._getEventArguments(target), target);
            }
        },
        _getContent: function (target, href, userModel, isInitialLoad) {
            var curModel = App._getModel(target);
            if (App._isUrl(href)) {
                proxy = App;
                $.ajax({
                    url: href,
                    dataType: userModel.dataType ? userModel.dataType : curModel.ajaxOptions.dataType,
                    crossDomain: userModel.crossDomain ? userModel.crossDomain : curModel.ajaxOptions.crossDomain,
                    contentType: userModel.contentType ? userModel.contentType : curModel.ajaxOptions.contentType,
                    async: false,
                    data: userModel.data ? userModel.data : curModel.ajaxOptions.data,
                    method: userModel.method ? userModel.method : curModel.ajaxOptions.method,
                    success: function (result) {
                        if (curModel.ajaxSuccess)
                            App._trigger("ajaxSuccess", App._getEventArguments(target), target);
                        var regex = /<body[^>]*>((.|[\n\r])*)<\/body>/im;
                        templateContent = regex.exec(result) ? regex.exec(result)[1] : result;
                        if (!isInitialLoad) {
                            var templateId = proxy._isUrl(href) ? "template_" + Math.floor((Math.random() * 1000) + 1) : href;
                            var templateContainer = $("<div id=" + templateId + "_container class='e-m-hidden' data-url=" + href + "></div>");
                            templateContainer.append($.trim(templateContent));
                            App._renderingPage = templateContainer;
                            target.append(templateContainer);
                            proxy._animatePages(target, App._isBack);
                        } else {
                            proxy._updateContent(target, href, userModel, templateContent);
                        }
                    },
                    error: function (result) {
                        ej.mobile.WaitingPopup.hide();
                        if (curModel.ajaxFailure)
                            App._trigger("ajaxFailure", App._getEventArguments(target), target);
                    }
                });
            } else {
                var templateElement = $("#" + href);
                templateContent = templateElement.html();
                if (!isInitialLoad) {
                    var templateId = App._isUrl(href) ? "template_" + Math.floor((Math.random() * 1000) + 1) : href;
                    var templateContainer = $("<div id=" + templateId + "_container class='e-m-hidden' data-url=" + href + "></div>");
                    templateContainer.append($.trim(templateContent));
                    App._renderingPage = templateContainer;
                    target.append(templateContainer);
                    App._animatePages(target, curModel.isReverseAnimation);
                } else
                    App._updateContent(target, href, userModel, templateContent);
            }
        },

        renderEJMControls: function (element) {
            var target = element ? element : App.activePage;
            if (App.renderEJMControlsByDef) {
                target.find("input[type='button']:not([data-role]):attrNotStartsWith('ejm-'),input[type='submit']:not([data-role]):attrNotStartsWith('ejm-'),input[type='reset']:not([data-role]):attrNotStartsWith('ejm-')").attr("data-role", "ejmbutton");
                target.find("a:not([data-role]):attrNotStartsWith('ejm-')").attr("data-role", "ejmactionlink");
                target.find("input:not([data-role]):not([type]):attrNotStartsWith('ejm-'),input[type='text']:not([data-role]):attrNotStartsWith('ejm-'),input[type='tel']:not([data-role]):attrNotStartsWith('ejm-'),input[type='email']:not([data-role]):attrNotStartsWith('ejm-'),input[type='password']:not([data-role]):attrNotStartsWith('ejm-')").attr("data-role", "ejmtextbox");
                target.find("input[type='checkbox']:not([data-role]):attrNotStartsWith('ejm-')").filter(function () { if (!$(this).closest("[data-role='ejmgroupbutton']").length) return this; }).attr("data-role", "ejmcheckbox");
                target.find("input[type='radio']:not([data-role]):attrNotStartsWith('ejm-')").filter(function () { if (!$(this).closest("[data-role='ejmgroupbutton']").length) return this; }).attr("data-role", "ejmradiobutton");
                target.find("input[type='date']:not([data-role]):attrNotStartsWith('ejm-')").attr("data-role", "ejmdatepicker");
                target.find("input[type='time']:not([data-role]):attrNotStartsWith('ejm-')").attr("data-role", "ejmtimepicker");
            }
        },

        back: function (target) {
            if (typeof target == "undefined" ) {
                var newPageContainer = $("<div class='e-m-hidden'></div>");
                if (history.state) {
                    if (ej.isNullOrUndefined(history.state.target)) return;
                    App._currentTarget = target = $("#" + history.state.target);
                    var curModel = App._getModel(target);
                    App._currentTarget.data({
                        historyModel: $.extend(false, App.defaults, history.state.userModel ? history.state.userModel : {})
                    });                    
                    if (history.state.targetContent == "")
                        App._getContent(target, history.state.targetUrl, curModel, false);
                    else {
                        var templateId = App._isUrl(history.state.targetUrl) ? "template_" + Math.floor((Math.random() * 1000) + 1) + "_container" : history.state.targetUrl;
                        newPageContainer.attr({
                            "id": templateId,
                            "page-url": history.state.targetUrl
                        });
                        newPageContainer.html(history.state.targetContent);
                        App._renderingPage = newPageContainer;
                        target.append(newPageContainer);
                        App._animatePages(App._currentTarget, curModel.isReverseAnimation);
                    }
                    if (App.model.onHistoryBack)
                        App._trigger("onHistoryBack", App._getEventArguments(target), target);
                    App.checkStateId = history.state && history.state.uid;
                } else {
                    var curModel = App._getModel(App._initialTargetId);                    
                    newPageContainer.attr("id", App._initialTargetId.data("heap")[0].href + "_container");
                    newPageContainer.html(App._initialTargetId.data("heap")[0].content);
                    App._renderingPage = newPageContainer;
                    App._initialTargetId.append(newPageContainer);
                    App._animatePages(App._initialTargetId, curModel.isReverseAnimation);
                    if (App.model.onHistoryBack)
                        App._trigger("onHistoryBack", App._getEventArguments(App._initialTargetId), target);
                }
            } else {
                App._isBack = true;
                var heap = $("#" + target).data("heap");
                if (heap && heap.length) {
                    var targetElement = $("#" + target);
                    var lastHeap = heap.pop();
                    if (heap.length) {
                        var elementToBeLoad = heap[heap.length - 1].content;
                        var urlToBeLoad = heap[heap.length - 1].href;
                        var modelToBeLoad = heap[heap.length - 1].model;
                        if (elementToBeLoad != "" || heap.length <= 1) {
                            App._renderingPage = elementToBeLoad;
                            targetElement.append(elementToBeLoad);
                            App._animatePages($("#" + target), App._isBack);
                        } else
                            App._getContent(targetElement, urlToBeLoad, modelToBeLoad, false);
                    }
                    if (App.model.onBack)
                        App._trigger("onBack", App._getEventArguments($("#" + target)), $("#" + target));
                } else
                    return false;
            }
        },
        _trigger: function (eventName, eventProp, target) {
            var curModel = App._getModel(target);
            if (!curModel) return;
            var fn = null,
                returnValue, args, clientProp = {};
            eventProp = $.extend({
                target: target
            }, eventProp);
            $.extend(clientProp, eventProp)

            if (eventName in curModel)
                fn = curModel[eventName];

            if (fn) {
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }

                if ($.isFunction(fn)) {
                    args = ej.event(eventName, curModel, eventProp);

                    returnValue = fn.call(App, args);

                    // sending changes back - deep copy option should not be enabled for App $.extend 
                    if (eventProp) $.extend(eventProp, args);

                    if (args.cancel || !ej.isNullOrUndefined(returnValue))
                        return returnValue === false || args.cancel;
                }
            }
            var isPropDefined = Boolean(eventProp);
            eventProp = eventProp || {};
            eventProp.originalEventType = eventName;
            eventProp.type = eventName;

            args = $.Event(eventProp.type, ej.event(eventProp.type, curModel, eventProp));
            // sending changes back - deep copy option should not be enabled for App $.extend 
            if (isPropDefined) $.extend(eventProp, args);

            return args.cancel;
        }
    }
})();

$(function () {
    if (App.applyMobileStyles && $("body").find("[data-role*='ejm']").length > 0)
        $("body").addClass("e-m-user-select e-m-viewport e-m-" + ej.getRenderMode());
    App._currentPage = App.activeSubPage = App.activePage = $("body");
    App.window.on("popstate", function (args) {
        App._isBack = true;
        if (App.checkStateId && args.originalEvent.state) {
            if (args.originalEvent.state.uid >= App.checkStateId)
                App._isBack = false;
        }
        App.back();
    });
});;
(function ($, ej, undefined) {
    ej.mobile.AjaxLoader = {
        _renderItems: function (item, appendTo, index) {
            var item = $(item);
            this._control = this._rootCSS.split("e-m-")[1],
            item.attr("id", this.elementId + "-" + this._control + "-item" + index).addClass("e-m-" + this._control + "-item");
            var isAjax = !(ej.isNullOrUndefined(this._items[index].enableAjax)) && this._items[index].enableAjax ? this._items[index].enableAjax : this.model.enableAjax;
            if (!this._storedContent[index] && !isAjax) {
                if (this._items[index].href) {
                    var templateContent = ej.getCurrentPage().find(this._items[index].href);
                    this._storedContent[index] = templateContent[0] && templateContent[0].nodeName && templateContent[0].nodeName.toLowerCase() == "script" ? ej.getClearString(templateContent[0].innerHTML) : templateContent;
                }
                else {
                    this._storedContent[index] = this._items[index].template ? this._items[index].template : item.html();
                }
            }
            if (!isAjax) {
                var itemContent = ej.buildTag("div.e-m-" + this._control + "-content e-m-" + this._control + "-static-content#" + this.elementId + "-" + this._control + "-item" + index + "-content", this._storedContent[index]).appendTo(appendTo);                
            }
            if (isAjax && this.model.showAjaxPopup)
                ej.mobile.WaitingPopup.show();
            if (isAjax && this.model.prefetchAjaxContent)
                this._loadAjaxContent(item, appendTo, index);
        },

        _loadAjaxContent: function (item, appendTo, index) {
            var currentItem = $(item), proxy = this, local = this;
            var urlVal = this._items[index].href;
            if (!$.support.pushstate || ej.isWindowsWebView())
                urlVal = this._makeUrlAbsolute(urlVal);
            var BeforeLoad = { item: item, index: index, text: currentItem.text(), url: this.model.ajaxSettings.url ? this.model.ajaxSettings.url : urlVal };
            if (this.model.ajaxBeforeLoad)
                this._trigger("ajaxBeforeLoad", BeforeLoad);
            var ajaxSettings = {
                cache: this.model.ajaxSettings.cache,
                async: this.model.ajaxSettings.async,
                type: this.model.ajaxSettings.type,
                contentType: this.model.ajaxSettings.contentType,
                url: ej.isWindowsWebView() ? urlVal : this.model.ajaxSettings.url ? this.model.ajaxSettings.url : urlVal,
                dataType: this.model.ajaxSettings.dataType,
                data: this.model.ajaxSettings.data,
                "successHandler": function (data) {
                    //Ajax post success event handler 
                    var successData = { item: item, index: index, text: currentItem.text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    var currContent = ej.getCurrentPage().find("#" + proxy.elementId + "-" + local._control + "-item" + index + "-content");
                    if (!proxy.model.enableCache || currContent.length == 0) {
                        var content = ej.buildTag("div.e-m-" + local._control + "-content e-m-" + local._control + "-ajax-content#" + proxy.elementId + "-" + local._control + "-item" + index + "-content", (/<\/?body[^>]*>/gmi).test(data) ? data.split(/<\/?body[^>]*>/gmi)[1] : data || "").appendTo(appendTo);
                        proxy._content = this._control == "acc" ? content.children() : content;
                        if ((!ej.isAppNullOrUndefined() && App.angularAppName) || ej.angular.defaultAppName)
                            ej.angular.compile($(content));
                    }
                    ej.widget.init(proxy._content);
                    if (proxy.model.ajaxSuccess)
                        proxy._trigger("ajaxSuccess", successData);
                    if (proxy._ajaxSuccessHandler)
                        proxy._ajaxSuccessHandler(successData);
                },
                "errorHandler": function (xhr, textStatus, errorThrown) {
                    var errorData = { "xhr": xhr, "textStatus": textStatus, "errorThrown": errorThrown, item: item, index: currentItem.index(), text: currentItem.text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxError)
                        proxy._trigger("ajaxError", errorData);
                },
                "completeHandler": function (data) {
                    if (proxy.model.showAjaxPopup)
                        ej.mobile.WaitingPopup.hide();
                    var CompleteData = { content: proxy._content, item: item, index: currentItem.index(), text: currentItem.text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxComplete)
                        proxy._trigger("ajaxComplete", CompleteData);
                }
            };
            ej.sendAjaxRequest(ajaxSettings);
        },
        _showHideItems: function (targetItem, appendTo, index) {
            var wrapper = ej.getCurrentPage();
            var isAjax = !(ej.isNullOrUndefined(this._items[index].enableAjax)) ? this._items[index].enableAjax : this.model.enableAjax;
            var contentExists = wrapper.find("#" + targetItem.id + "-content").length;
            if (isAjax && contentExists == 0 && !this.model.prefetchAjaxContent)
                this._loadAjaxContent(targetItem, appendTo, index);
            $(this["_" + this._control + "Items"]).removeClass("e-m-state-active");
            $(targetItem).addClass("e-m-state-active");
            wrapper.find(".e-m-" + this._control + "-content").addClass("e-m-" + this._control + "-state-hide");
            wrapper.find("#" + targetItem.id + "-content").removeClass("e-m-" + this._control + "-state-hide");
            if (!this.model.prefetchAjaxContent && !this.model.enableCache)
                wrapper.find(".e-m-" + this._control + "-ajax-content.e-m-" + this._control + "-state-hide").remove();
            if (this.model.select)
                this._trigger("select", { index: index, item: $(targetItem), text: $(targetItem).text(), isInteraction: this._isInteraction ? true : false });
        },
        ajaxReload: function () {
            var activeItem = this.element.find(".e-m-" + this._control + "-item.e-m-state-active"), activeContent = this.element.find(".e-m-" + this._control + "-content:not(.e-m-" + this._control + "-state-hide)"), activeContentWrapper = activeContent.parent(), index = activeItem[0].id.split("item")[1];
            activeContent.remove();
            this._loadAjaxContent(activeItem, activeContentWrapper, index);
        },
        _makeUrlAbsolute: function (relUrl) {
            var link = $("<a href ='" + relUrl + "'></a>")[0];
            $("body").append(link);
            var pathname;
            if (link.pathname.indexOf('/') != 0)
                pathname = '/' + link.pathname;
            else
                pathname = link.pathname;
            var browser = ej.browserInfo();
            var isIE9 = (browser.name == "msie" && browser.version == "9.0");
            var newLink = (link.protocol + "//" + ((isIE9 && link.port == "80") ? (link.host).replace(":80", "") : link.host) + pathname + link.search + link.hash);
            $(link).remove();
            return newLink;
        }
    }
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Accordion elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejmAccordion", "ej.mobile.Accordion", {
        _setFirst: true,
        _requiresID: true,
        _rootCSS: "e-m-acc",
        validTags: ["ul"],
        _tags: [{
            tag: "items",
            attr: ["text", "href", "enableAjax"],
            content: "template"
        }],

        defaults: {
            renderMode: "auto",
            cssClass: "",
            selectedItems: [0],
            disabledItems: [],
            enableMultipleOpen: false,
            expandAll: false,
            collapseAll: false,
            heightAdjustMode: "content",
            enableRippleEffect: ej.isAndroid() ? true : false,
            showHeaderIcon: true,
            headerIcon: {
                normal: "e-m-icon-down",
                active: "e-m-icon-up"
            },
            spinnerText: null,
            enablePersistence: false,
            enableAjax: false,
            enableCache: false,
            prefetchAjaxContent: false,
            showAjaxPopup: false,
            items: [],
            itemTouchStart: null,
            itemTouchEnd: null,
            select: null,
            expand: null,
            collapse: null,
            ajaxBeforeLoad: null,
            ajaxSuccess: null,
            ajaxError: null,
            ajaxComplete: null,
            ajaxSettings: {
                type: 'GET',
                cache: false,
                async: true,
                dataType: "html",
                contentType: "html",
                url: "",
                data: {}
            },
            locale: "en-US"
        },

        dataTypes: {
            renderMode: "enum",
            cssClass: "string",
            selectedItems: "array",
            disabledItems: "array",
            enableMultipleOpen: "boolean",
            expandAll: "boolean",
            enableRippleEffect: "boolean",
            collapseAll: "boolean",
            heightAdjustMode: "enum",
            showHeaderIcon: "boolean",
            headerIcon: {
                normal: "string",
                active: "string"
            },
            spinnerText: "string",
            enablePersistence: "boolean",
            enableAjax: "boolean",
            enableCache: "boolean",
            prefetchAjaxContent: "boolean",
            showAjaxPopup: "boolean",
            locale: "string"
        },
        observableArray: ["selectedItems"],
        selectedItems: ej.util.valueFunction("selectedItems"),

        _init: function () {
            this._getLocalizedLabels();
            this.model.spinnerText = !ej.isNullOrUndefined(this.model.spinnerText) ? this.model.spinnerText : this._localizedLabels["spinnerText"];
            this._renderControl();
            this._wireEvents(false);
        },

        _renderControl: function () {
            this._orgEle = $(this.element).clone();
            ej.setRenderMode(this);
            this.element.addClass("e-m-user-select e-m-clearall e-m-" + this.model.renderMode + " " + this.model.cssClass);
            this.elementId = this.element[0].id;
            this._headerIcon = jQuery.extend({}, this.model.headerIcon);
            this.model.prefetchAjaxContent = (this.model.expandAll && this.model.enableAjax) ? true : this.model.prefetchAjaxContent;
            this.model.enableMultipleOpen = this.model.expandAll ? true : this.model.enableMultipleOpen;
            if (typeof this.model.items === "string") { this.model.items = eval(this.model.items) };
            this._storedContent = [], this._accItems = [], this._items = [], this._isAngular = this.model.items.length > 0 ? true : false;
            if (!this._isAngular) {
                this._accItems = this.element.find("li");
                for (var i = 0, itemLength = this._accItems.length; i < itemLength; i++)
                    this._items.push(this._insertItem($(this._accItems[i])));
            }
            else this._items = this.model.items;
            for (var li = 0, itemLength = this._items.length; li < itemLength; li++) {
                if (this._isAngular)
                    this._accItems.push(ej.buildTag('li').appendTo(this.element)[0]);
                var content = ej.buildTag("li#" + this.elementId + "-acc-item" + li + "-content-wrapper.e-m-acc-content-wrapper e-m-acc-state-hide");
                $(this._accItems[li]).after(content);
                this._renderItems(this._accItems[li], content, li);
                $(this._accItems[li]).text(this._items[li].text ? this._items[li].text : $(this._accItems[li]).attr("data-ej-text")).addClass((li == 0 ? "e-m-acc-item-first " : li == itemLength - 1 ? "e-m-acc-item-last " : "") + (this.model.showHeaderIcon ? this.model.headerIcon.normal : ""));
            }
            this._setEnableRippleEffect();
            this._postRendering();
        },

        _setEnableRippleEffect: function () {
            $(this._accItems)[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
        },
        _insertItem: function (item) {
            var itemVal = {};
            itemVal.text = (ej.getAttrVal(item, 'data-ej-text') == undefined) ? $(item).html() : ej.getAttrVal(item, 'data-ej-text');
            itemVal.href = ej.getAttrVal(item, 'data-ej-href', undefined);
            itemVal.enableAjax = ej.getBooleanVal(item, 'data-ej-enableAjax', undefined);
            itemVal.template = $(item).html();
            return itemVal;
        },
        _postRendering: function () {
            this._setHeightAdjustMode();
            this.model.expandAll ? this._setExpandAll(true) : this.model.collapseAll ? this._setCollapseAll(true) : this._loadItem(this.selectedItems());
            if (this.model.disabledItems.length)
                this._enableDisable(this.model.disabledItems, true);
        },
        _ajaxSuccessHandler: function (args) {
            if ($(args.item).text() == this.model.spinnerText) {
                $(args.item).text(this._items[args.index].text).next(".e-m-acc-content-wrapper").slideDown(300, "linear");
                if (this.model.expand && !this.model.enableMultipleOpen) {
                    var proxy = this;
                    var data = { item: args.item, index: args.index };
                    setTimeout(function () {
                        proxy._trigger('expand', data);
                    }, 300);
                }
            }
        },
        _loadItem: function (items, itemClick) {
            var proxy = this;
            if (this.model.enableMultipleOpen) {
                for (var i = 0, itemLength = items.length; i < itemLength; i++) {
                    var data = { item: targetItem, index: items[i] };
                    var targetItem = $(this._accItems[items[i]]), targetWrapper = this.element.find("#" + this._accItems[items[i]].id + "-content-wrapper"), contentExists = targetWrapper.find(".e-m-acc-content").length > 0;
                    !(targetItem.hasClass("e-m-state-active")) ? targetItem.addClass("e-m-state-active " + (this.model.showHeaderIcon ? this.model.headerIcon.active : "")).removeClass(this.model.headerIcon.normal) : itemClick ? targetItem.removeClass("e-m-state-active " + this.model.headerIcon.active).addClass(this.model.headerIcon.normal) : "";
                    targetItem.hasClass("e-m-state-active") ? itemClick ? this.selectedItems().push(parseInt(items[i])) : "" : this.selectedItems().splice(items[i], 1);
                    if ((!(ej.isNullOrUndefined(this._items[items[i]].enableAjax)) ? this._items[items[i]].enableAjax : this.model.enableAjax) && !this.model.prefetchAjaxContent && !contentExists) {
                        targetItem.text(this.model.spinnerText);
                        this._loadAjaxContent(targetItem, targetWrapper, items[i]);
                    }
                    if (targetWrapper.hasClass("e-m-acc-state-hide")) {
                        if (!this.model.enableAjax || (this.model.prefetchAjaxContent || contentExists))
                            $(targetWrapper).slideDown(300, "linear");
                        if (this.model.expand) {
                            setTimeout(function () {
                                proxy._trigger('expand', data);
                            }, 300);
                        }
                        targetWrapper.removeClass("e-m-acc-state-hide");
                    }
                    else if (itemClick) {
                        targetWrapper.slideUp(300, "linear").addClass("e-m-acc-state-hide");
                        if (this.model.collapse) {
                            setTimeout(function () {
                                proxy._trigger('collapse', data);
                            }, 300);
                        }
                    }
                }
                if (!this.model.enableCache && !this.model.prefetchAjaxContent)
                    this.element.find(".e-m-acc-content-wrapper.e-m-acc-state-hide > .e-m-acc-ajax-content").remove();
            }
            else {
                this.selectedItems(items);
                if (itemClick && !this.model.collapseAll)
                    var collapseData = { item: this.element.find(".e-m-acc-item.e-m-state-active"), index: this.element.find(".e-m-acc-item.e-m-state-active")[0].id.split("item")[1] };
                this.element.find(".e-m-acc-item.e-m-state-active").removeClass(this.model.headerIcon.active).addClass((this.model.showHeaderIcon ? this.model.headerIcon.normal : ""));
                var targetWrapper = this.element.find("#" + this._accItems[items[0]].id + "-content-wrapper"), contentExists = targetWrapper.find(".e-m-acc-content").length > 0;
                if ((!(ej.isNullOrUndefined(this._items[items[0]].enableAjax)) ? this._items[items[0]].enableAjax : this.model.enableAjax) && !this.model.prefetchAjaxContent && !contentExists)
                    $(this._accItems[items[0]]).text(this.model.spinnerText);
                if (!this.model.enableAjax || (this.model.prefetchAjaxContent || contentExists)) {
                    $(targetWrapper).slideDown(300, "linear");
                    var data = { item: this._accItems[items[0]], index: items[0] };
                    if (this.model.expand) {
                        setTimeout(function () {
                            proxy._trigger('expand', data);
                        }, 300);
                    }
                }
                targetWrapper.removeClass("e-m-acc-state-hide");
                this._showHideItems(this._accItems[items[0]], targetWrapper, items[0]);
                if (this.model.collapse && itemClick) {
                    setTimeout(function () {
                        proxy._trigger('collapse', collapseData);
                    }, 300);
                }
            }
            this.element.find(".e-m-acc-item.e-m-state-active").removeClass(this.model.headerIcon.normal).addClass((this.model.showHeaderIcon ? this.model.headerIcon.active : ""));
            this.element.find(".e-m-acc-item:not(.e-m-state-active)").next(".e-m-acc-content-wrapper").slideUp(itemClick ? 300 : 0, "linear").addClass("e-m-acc-state-hide");
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
        },
        _setLocale: function () {
            this._getLocalizedLabels();
            this.model.spinnerText = this._localizedLabels["spinnerText"];
        },
        _createDelegates: function () {
            this._itemTouchStartHndlr = $.proxy(this._itemTouchStart, this);
            this._itemTouchEndHndlr = $.proxy(this._itemTouchEnd, this);
            this._touchMoveHndlr = $.proxy(this._onTouchMove, this);
            this._resizeHndlr = $.proxy(this._resize, this);
        },
        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([this._accItems], [ej.startEvent()], [this._itemTouchStartHndlr], remove);
            ej.listenTouchEvent($(window), "onorientationchange" in window ? "orientationchange" : "resize", this._resizeHndlr, remove);
        },
        _itemTouchStart: function (evt) {
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            this._clickX = point.clientX;
            this._isMouseMove = false;
            var data = { item: evt.target, index: evt.target.id.split("item")[1] };
            this._target = evt.target;
            ej.listenEvents([this._accItems, this._accItems], [ej.endEvent(), ej.moveEvent()], [this._itemTouchEndHndlr, this._touchMoveHndlr], false);
            if (this.model.itemTouchStart)
                this._trigger('itemTouchStart', data);
        },
        _onTouchMove: function (evt) {
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            if (this._clickX != point.clientX) this._isMouseMove = true;
        },
        _itemTouchEnd: function (evt) {
            ej.listenEvents([this._accItems, this._accItems], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHndlr, this._itemTouchEndHndlr], true);
            if (evt.target === this._target && !this._isMouseMove) {
                var data = { item: evt.target, index: evt.target.id.split("item")[1] };
                if (!(!this.model.enableMultipleOpen && $(evt.target).hasClass("e-m-state-active")) && !$(evt.target).hasClass("e-m-state-disabled"))
                    this._loadItem([parseInt(evt.target.id.split("item")[1])], true);
                this.element.find(".e-m-state-active").length == this._items.length ? this.model.expandAll = true : this.element.find(".e-m-state-active").length == 0 ? this.model.collapseAll = true : this.model.collapseAll = false, this.model.expandAll = false;
                if (this.model.itemTouchEnd)
                    this._trigger('itemTouchEnd', data);
            }
        },
        _resize: function () {
            this._setHeightAdjustMode();
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-m-acc");
            this._renderControl();
            this._wireEvents();
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },
        _setExpandAll: function (options) {
            if (options) {
                (this.model.enableAjax && !this.model.prefetchAjaxContent) ? this._refresh() : this._addRemoveClass(true);
                if (this.model.collapseAll) this.model.collapseAll = false;
            }
        },
        _setCollapseAll: function (options) {
            if (options) {
                this._addRemoveClass(false);
                if (this.model.expandAll) this.model.expandAll = false;
            }
        },
        _addRemoveClass: function (option, index) {
            $(index >= 0 ? this._accItems[index] : this._accItems)[(option ? "add" : "remove") + "Class"]("e-m-state-active " + this.model.headerIcon.active)[(option ? "remove" : "add") + "Class"](this.model.headerIcon.normal);
            this.element.find(index >= 0 ? "#" + this._accItems[index].id + "-content-wrapper" : ".e-m-acc-content-wrapper")["slide" + (option ? "Down" : "Up")](0);
            this.element.find(index >= 0 ? "#" + this._accItems[index].id + "-content-wrapper" : ".e-m-acc-content-wrapper")[(option ? "remove" : "add") + "Class"]("e-m-acc-state-hide");
        },
        _setSelectedItems: function (options) {
            if (options.length)
                this.model.enableMultipleOpen ? this._selectDeselect(this.selectedItems(), true) : !($(this._accItems[options[0]]).hasClass("e-m-state-active") || $(this._accItems[options[0]]).hasClass("e-m-state-disabled")) ? this._loadItem(options) : "";
        },
        _setSpinnerText: function () {  //empty method given to avoid control refresh
        },
        _selectDeselect: function (items, select) {
            for (var i = 0, itemLength = items.length; i < itemLength; i++) {
                if (this.element.find("#" + this._accItems[items[i]].id + "-content").length == 0)
                    this._loadAjaxContent(this._accItems[items[i]], this.element.find("#" + this._accItems[items[i]].id + "-content-wrapper"), items[i]);
                this._addRemoveClass(select, items[i]);
            }
        },
        _setDisabledItems: function () {
            this._enableDisable(this.model.disabledItems, true);
        },
        _enableDisable: function (items, disable) {
            addremove = disable ? "add" : "remove";
            for (var i = 0, itemLength = items.length; i < itemLength; i++) {
                $(this._accItems[items[i]])[addremove + "Class"]("e-m-state-disabled");
                this.element.find("#" + this._accItems[items[i]].id + "-content-wrapper")[[addremove + "Class"]]("e-m-state-disabled");
            }
        },
        _setHeightAdjustMode: function () {
            var maxHeight = this.model.heightAdjustMode == "fill" ? this._adjustFillHeight() : (this.model.heightAdjustMode == "fixed") ? this._adjustFixedHeight() : "auto";
            $(this.element.find(".e-m-acc-content-wrapper")).height(maxHeight);
        },
        _adjustFixedHeight: function () {
            if (!this.model.enableAjax) {
                return maxHeight = Math.max.apply(null, $(".e-m-acc-content-wrapper").map(function () {
                    return $(this).height();
                }).get());
            }
        },
        _adjustFillHeight: function () {
            return ej.getDimension(this.element.parent(), "height") - (ej.getDimension(this._accItems[0], "outerHeight") * this._accItems.length);
        },
        _setHeaderIcon: function () {
            this.element.find(".e-m-acc-item.e-m-state-active").removeClass(this._headerIcon.active).addClass(this.model.headerIcon.active);
            this.element.find(".e-m-acc-item:not(.e-m-state-active)").removeClass(this._headerIcon.normal).addClass(this.model.headerIcon.normal);
            this._headerIcon = jQuery.extend({}, this.model.headerIcon);
        },
        //Public Methods
        selectItems: function (items) {
            this.model.enableMultipleOpen ? this._selectDeselect(items, true) : this._loadItem(items);
        },
        deselectItems: function (items) {
            if (this.model.enableMultipleOpen) this._selectDeselect(items, false);
        },
        enableItems: function (items) {
            this._enableDisable(items, false);
        },
        disableItems: function (items) {
            this._enableDisable(items, true);
        },
        addItem: function (item, index) {
            this.model.items = this._items;
            (index >= 0) ? this.model.items.splice(index, 0, this._insertItem($(item))) : this.model.items.push(this._insertItem($(item)));
            this.element.empty();
            this._renderControl();
            this._wireEvents();
            this._items = this.model.items;
        },
        removeItem: function (index) {
            this.model.items = this._items;
            if (index >= 0) {
                this.model.items.splice(index, 1);
                this.element.empty();
                this._renderControl();
                this._wireEvents();
            }
            this._items = this.model.items;
        },
        getItemsCount: function () {
            return this._items.length;
        },
    });
    $.extend(true, ej.mobile.Accordion.prototype, ej.mobile.AjaxLoader);

        ej.mobile.Accordion.HeightAdjustMode = {
        Fixed: "fixed",
        Content: "content",
        Fill: "fill"
    };

    ej.mobile.Accordion.Locale = ej.mobile.Accordion.Locale || {};
    ej.mobile.Accordion.Locale["default"] = ej.mobile.Accordion.Locale["en-US"] = {
        spinnerText: "Loading..."
    };
})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html AutoComplete elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmAutocomplete", "ej.mobile.Autocomplete", {
        _setFirst: true,
        _rootCSS: "e-m-ac",
        defaults: {
            renderMode: "auto",
            cssClass: "",
            watermarkText: null,
            minCharacter: 1,
            enableMultiSelect: false,
            caseSensitiveSearch: false,
            enableDistinct: false,
            enablePersistence: false,
            enableAutoFill: false,
            allowSorting: true,
            templateId: "",
            popupHeight: "164px",
            popupWidth: "auto",
            itemsCount: 0,
            sortOrder: "ascending",
            delimiterChar: ",",
            emptyResultText: null,
            showEmptyResultText: true,
            filterType: "startswith",
            mode: "default",
            value: "",
            enabled: true,
            dataSource: [],
            fields: {
                text: null,
                key: null,
                image: null
            },
            locale: "en-US",
            touchEnd: null,
            keyPress: null,
            select: null,
            change: null,
            focusIn: null,
            focusOut: null
        },
        dataTypes: {
            renderMode: "enum",
            filterType: "enum",
            sortOrder: "enum",
            minCharacter: "number",
            dataSource: "data",
            enableMultiSelect: "boolean",
            caseSensitiveSearch: "boolean",
            enableDistinct: "boolean",
            enablePersistence: "boolean",
            enableAutoFill: "boolean",
            allowSorting: "boolean",
            showEmptyResultText: "boolean",
            enabled: "boolean",
            itemsCount: "number",
            locale: "string"
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),
        _init: function () {
            this._orgEle = this.element.clone();
            ej.setRenderMode(this);
            this._getLocalizedLabels();
            this.model.emptyResultText = !ej.isNullOrUndefined(this.model.emptyResultText) ? this.model.emptyResultText : this._localizedLabels["emptyResultText"];
            this.model.watermarkText = !ej.isNullOrUndefined(this.model.watermarkText) ? this.model.watermarkText : this._localizedLabels["watermarkText"];
            this._initialize();
            this._renderControl();
            this._wireEvents();
        },
        _initialize: function () {
            this._id = this.element.attr('id');
            this._noResultEle = ej.buildTag('li.e-m-lv-item e-m-emptyresult', ej.buildTag('a.e-m-lv-content',
                this.model.emptyResultText, {}, { 'data-ej-appajax': false }));
            this._selectedData = [];
            this._hiddenVal = [];
            this.model.dataSource = eval(this.model.dataSource);
            if (this.model.value != "") {
                this._selectedData.push(this.model.value);
                this.element.val(this.model.value);
            }
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale)
        },
        _renderControl: function () {
            this.element.addClass(this.model.cssClass);
            this._wrapper = ej.buildTag("div#" + this._id + "_wrapper.e-m-ac-wrapper").addClass("e-m-" + this.model.renderMode);
            this._wrapper = this.element.wrap(this._wrapper).parent();
            this._setWatermarkText(this.model.watermarkText);
            this._controlState(this.model.enabled);
            this._hiddenElement = ej.buildTag("input#" + this._id + "_hidden", {}, { "display": "none" }, { "name": this._id });
            this._accList = ej.buildTag('ul#' + this._id + 'list.e-m-ac-list').appendTo(this._wrapper.append(this._hiddenElement));
            this._listWrapper = ej.buildTag("div.e-m-list-wrapper");
            if (this.model.mode == ej.mobile.Autocomplete.Mode.Search) {
                this._search = ej.buildTag("div.e-m-icon-search");
                this._clear = ej.buildTag("div.e-m-icon-close");
                this._search.appendTo(this._wrapper.addClass('e-m-ac-search'));
                this._clear.appendTo(this._wrapper).hide();
            }
            this._listWrapper.appendTo(this._wrapper).append(this._accList).hide();
        },
        _createDelegates: function () {
            this._focus = $.proxy(this._focusHandler, this);
            this._blur = $.proxy(this._blurHandler, this);
            this._keyPress = $.proxy(this._keyPressHandler, this);
            this._docTouchStart = $.proxy(this._docTouchStartHandler, this);
            this._docTouchEnd = $.proxy(this._docTouchEndHandler, this);
            this._clearClick = $.proxy(this._clearClickHandler, this);
        },
        _wireEvents: function (remove) {
            var doc = $(document);
            this._createDelegates();
            ej.listenEvents([this.element, this.element, this.element, doc, doc], ["focus", "blur", "keyup", ej.startEvent(), ej.endEvent()],
                [this._focus, this._blur, this._keyPress, this._docTouchStart, this._docTouchEnd], remove);
            this.model.mode == ej.mobile.Autocomplete.Mode.Search && ej.listenTouchEvent(this._clear, ej.endEvent(), this._clearClick, remove);
        },
        _focusHandler: function (evt) {
            this.element.addClass("e-m-focus");
            var char = this.element.val().substring(this.element.val().length - 2, this.element.val().length);
            char != this.model.delimiterChar + " " && this.model.enableMultiSelect && this.element.val().length &&
            this.element.val(this.element.val() + this.model.delimiterChar + " ");
            this.model.focusIn && this._trigger("focusIn", { event: evt, isInteraction: true });
        },
        _blurHandler: function (evt) {
            this.element.removeClass("e-m-focus");
            this.model.focusOut && this._trigger("focusOut", { event: evt, isInteraction: true });
        },
        _keyPressHandler: function (evt) {
            var delimitterIndex = this.element.val().lastIndexOf(this.model.delimiterChar),
                searchKey = (delimitterIndex < 0) ? this.element.val() : this.element.val().substring(delimitterIndex + 2);
            if (searchKey.length >= this.model.minCharacter) {
                dataQuery = ej.Query().where(this.model.fields.text, this.model.filterType, searchKey, !this.model.caseSensitiveSearch);
                this._addQuery(dataQuery);
                this._renderSuggestionsList(dataQuery);
                this.model.enableAutoFill && evt.keyCode != 8 && this.model.filterType == ej.mobile.Autocomplete.FilterType.StartsWith
                && !this._selectedData.length && this._autoFill();
                if (!(this.model.dataSource instanceof ej.DataManager)) {
                    this._listWrapper.show();
                    !this._accList.children().length ? this.model.showEmptyResultText &&
                    this._accList.append(this._noResultEle) : this._checkItems();
                }
            }
            !this.element.val() && this.clearText();
            this.model.keyPress && this._trigger("keyPress", { event: evt, isInteraction: true });
        },
        _listTouchHandler: function (evt) {
            if (!$(evt.event.target).closest(this._noResultEle).length) {
                var text = this.model.templateId != "" ? evt.data.text : evt.text, key = evt.item.attr("data-value"),
                    val = key ? key : text;
                if (this.model.enableMultiSelect) {
                    if (!evt.isChecked) {
                        this._selectedData = jQuery.grep(this._selectedData, function (value) { return value != text; });
                        this._hiddenVal = jQuery.grep(this._hiddenVal, function (value) { return value != val; });
                    }
                    else if ($.inArray(text, this._selectedData) == -1) {
                        this._selectedData.push(text);
                        this._hiddenVal.push(val);
                    }
                    this.element.val(this._setDelimiterChar(this._selectedData));
                    this._hiddenElement.attr("value", this._hiddenVal.toString());
                    this.value(this._setDelimiterChar(this._selectedData));
                }
                else {
                    this.element.val(text);
                    this.value(text);
                    this._hiddenElement.attr("value", val);
                    this._selectedData = [];
                    this._hiddenVal = [];
                    this._selectedData.push(text);
                    this._listWrapper.hide();
                }
                this.element.blur();
                this.model.select && this._trigger("select", evt);
                this.model.touchEnd && this._trigger("touchEnd", evt);
                this.model.change && this._trigger("change", evt);
            }
            this._accList.find('.e-m-emptyresult').removeClass('e-m-lv-selected e-m-lv-checked');
        },
        _onComplete: function (evt) {
            this.model.enableDistinct && this._distinctList();
            this.element.val() && this._listWrapper.show();
            !this._accList.children().length ? this.model.showEmptyResultText && this._accList.append(this._noResultEle) : this._checkItems();
            this._renderScroll();
        },
        _clearClickHandler: function (evt) {
            this.clearText();
        },
        _docTouchStartHandler: function (evt) {
            if (!this._isTargetDropDown(evt.target))
                this._hide = true;
        },
        _docTouchEndHandler: function (evt) {
            if (!this._isTargetDropDown(evt.target) && this._hide) {
                this._listWrapper.hide();
                this.element.removeClass("e-m-focus");
                this.model.enableMultiSelect && this.element.val(this.element.val().replace(eval("/" + this.model.delimiterChar + "\\s*$/"), ''));
                this._hide = false;
            }
        },
        _isTargetDropDown: function (target) {
            return $(target).closest(this._wrapper).length;
        },
        _addQuery: function (query) {
            if (this.model.allowSorting) {
                var order = (this.model.sortOrder == ej.mobile.Autocomplete.SortOrder.Descending) ? true : false;
                query.sortBy(this.model.fields.text ? this.model.fields.text : "", order);
            }
            this.model.itemsCount > 0 && query.take(this.model.itemsCount);
        },
        _renderSuggestionsList: function (dataQuery) {
            var listModel = {
                renderMode: this.model.renderMode, dataSource: this.model.dataSource, query: dataQuery, templateId: this.model.templateId,
                touchEnd: $.proxy(this._listTouchHandler, this), enableChecklist: this.model.enableMultiSelect,
                fields: { text: this.model.fields.text, image: this.model.fields.image, value: this.model.fields.key },
                actionSuccess: $.proxy(this._onComplete, this)
            };
            this._accList.ejmListView(listModel);
            this._listWrapper.hide();
            this.model.enableDistinct && this._distinctList();
            this._renderScroll();
            this.model.mode == ej.mobile.Autocomplete.Mode.Search && this._clear.show();
        },
        _renderScroll: function () {
            this._scrollDestroy();
            var lHeight = this._listWrapper.height(), wHeight = window.innerHeight, popupHeight = parseInt(this.model.popupHeight),
                tarHeight = lHeight > popupHeight ? popupHeight : lHeight, tarOffSet = this.element.offset().top + this.element.height();
            popupHeight < lHeight && this._listWrapper.ejmScrollPanel({
                renderMode: this.model.renderMode, targetHeight: popupHeight
            });
            (tarOffSet + tarHeight) > wHeight && this._listWrapper.css('top', -tarHeight);
            this.model.popupWidth != "auto" && this._listWrapper.css('width', this.model.popupWidth);
        },
        _scrollDestroy: function () {
            var scrollEle = this._wrapper.find('.e-m-scroll');
            if (scrollEle.length) {
                scrollEle.ejmScrollPanel("destroy");
                this._listWrapper.addClass('e-m-list-wrapper');
            }
        },
        _autoFill: function () {
            var data = this._accList.ejmListView("getTextByIndex", 0);
            data.length && typeof data === "string" ? this._setInputText(data) : this.model.showEmptyResultText &&
            this._accList.append(this._noResultEle);
        },
        _distinctList: function () {
            var list = this._accList.data("ejmListView"), data = ej.dataUtil.distinct(list.model.dataSource, this.model.fields.text, true);
            this._accList.ejmListView({ dataSource: data });
            this._listWrapper.hide();
        },
        _setDelimiterChar: function (textObj) {
            var text = textObj.toString();
            return text.replace(/,/g, this.model.delimiterChar + " ");
        },
        _getDelimiterIndex: function () {
            return this.element.val().lastIndexOf(this.model.delimiterChar);
        },
        _setInputText: function (data) {
            var delimiterIndex = this._getDelimiterIndex() + 1;
            var currentVal = this.element.val();
            var x = this.element[0].selectionStart;
            this.element.val(currentVal.substring(0, delimiterIndex) + data);
            this.element[0].selectionStart = x;
            var y = this.element[0].selectionEnd;
            this.element[0].selectionEnd = y;
        },
        _setWatermarkText: function (text) {
            text ? this.element.attr("placeholder", text) : this.element.removeAttr("placeholder");
        },
        _controlState: function (enabled) {
            this._wrapper[!enabled ? "addClass" : "removeClass"]("e-m-state-disabled");
        },
        _checkItems: function () {
            if (this.model.enableMultiSelect) {
                var listInstance = this._accList.data("ejmListView");
                var checked = [];
                for (i = 0; i < this._selectedData.length; i++) {
                    for (j = 0; j < listInstance.model.dataSource.length; j++) {
                        var text = this.model.fields.text ? listInstance.model.dataSource[j][this.model.fields.text]
                            : listInstance.model.dataSource[j];
                        this._selectedData[i] == text && checked.push(j);
                    }
                }
                listInstance.checkItemsByIndices(checked);
            }
        },
        enable: function () {
            this.element.removeClass("e-m-state-disabled");
        },
        disable: function () {
            this.element.addClass("e-m-state-disabled");
        },
        getValue: function () {
            return this.element.val();
        },
        clearText: function () {
            this.element.val('');
            this.value('');
            this._selectedData = [];
            this._hiddenVal = [];
            this.model.mode == ej.mobile.Autocomplete.Mode.Search && this._clear.hide();
            this._listWrapper.hide();
        },
        getSelectedItems: function () {
            return this._selectedData;
        },
        _setModel: function (options) {
            var refresh;
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var option in options) {
                switch (option) {
                    case "renderMode": this._setRenderMode(options.renderMode); break;
                    case "delimiterChar": this.model.enableMultiSelect && this._setDelimiterChar(this._text); break;
                    case "enabled": this._controlState(this.model.enabled); break;
                    case "watermarkText": this._setWatermarkText(this.model.watermarkText); break;
                    case "value": this.value(this.model.value); break;
                    case "emptyResultText": this._setEmptyRestultText(); break;
                    case "locale": this._setLocale(options.locale); break;
                    default: refresh = true; break;
                }
            }
            refresh && this._refresh();
            refresh = false;
        },
        _setEmptyRestultText: function () {
            this._noResultEle.find("a.e-m-lv-content").html(this.model.emptyResultText);
        },
        _setLocale: function (option) {
            this._getLocalizedLabels();
            this.model.emptyResultText = this._localizedLabels["emptyResultText"];
            this.model.watermarkText = this._localizedLabels["watermarkText"];
            this._setWatermarkText();
            this._setEmptyRestultText();
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass(this._rootCSS);
            this._renderControl();
            this._wireEvents();
        },
        _clearElement: function () {
            this.element.removeAttr("class").removeAttr("style");
            this._orgEle.insertBefore(this._wrapper);
            this._wrapper.remove();
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        }
    });

    ej.mobile.Autocomplete.FilterType = {
        StartsWith: "startswith",
        Contains: "contains"
    };
    ej.mobile.Autocomplete.Mode = {
        Search: "search",
        Default: "default"
    };
    ej.mobile.Autocomplete.SortOrder = {
        Ascending: "ascending",
        Descending: "descending"
    };
    ej.mobile.Autocomplete.Locale = ej.mobile.Autocomplete.Locale || {};
    ej.mobile.Autocomplete.Locale["default"] = ej.mobile.Autocomplete.Locale["en-US"] = {
        watermarkText: null,
        emptyResultText: "No suggestions"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget({ "ejmButton": "ej.mobile.Button", "ejmActionlink": "ej.mobile.Actionlink" },
        {
            _setFirst: true,
            _rootCSS: "e-m-btn",
            defaults: {
                touchStart: null,
                touchEnd: null,
                enabled: true,
                cssClass: "",
                renderMode: "auto",
                enableRippleEffect: ej.isAndroid() ? true : false,
                text: null,
                href: "",
                imageClass: null,
                imagePosition: "left",
                contentType: "text",
                enablePersistence: false,
                showBorder: true,
                style: "normal",
                color: {
                    "border": "",
                    "fill": "",
                    "text": ""
                },
                locale: 'en-US'
            },

            dataTypes: {
                enabled: "boolean",
                renderMode: "enum",
                imagePoisition: "enum",
                contentType: "enum",
                enablePersistence: "boolean",
                enableRippleEffect: "boolean"
            },

            _init: function () {
                this._getLocalizedLabels();
                this.model.text = !ej.isNullOrUndefined(this.model.text) ? this.model.text : this._localizedLabels["text"];
                this._renderControl();
                this._createDelegates();
                this._wireEvents();
            },

            _renderControl: function () {
                ej.setRenderMode(this);
                this._isLinkButton = this.element[0].nodeName == "A";
                if (this._isLinkButton && this.model.href)
                    this.element.attr("href", this.model.href);
                this.element.addClass("e-m-" + this.model.renderMode + " e-m-btn-" + this.model.style + " e-m-btn-image e-m-state-default e-m-user-select e-m-border" + (this._isLinkButton ? " e-m-actionlink " : " ") + this.model.cssClass);
                if (!this.element.attr("name") && this.element[0].id)
                    this.element.attr("name", this.element[0].id);
                this._setImagePosition();
                this.model.text = this.element.html() != "" ? this.element.html() : !ej.isNullOrUndefined(this.element.attr("value")) ? this.element.attr("value") : this.model.text;
                this._setText(this.model.text);
                if (this.model.style == "back")
                    this.element.addClass("e-m-icon-back");
                this._controlStatus(this.model.enabled);
                if (!this.model.showBorder)
                    this._setShowBorder(false);
                this._setColor();
                    this._setEnableRippleEffect();
            },
            _setEnableRippleEffect: function () {
                this.element[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
            },

            _getLocalizedLabels: function () {
                this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
            },

            _setColor: function () {
                this.element.css({ "color": this.model.color.text, "border-color": this.model.color.border, "background-color": this.model.color.fill });
            },

            _createDelegates: function () {
                this._touchStartHandler = $.proxy(this._touchStart, this);
                this._touchEndHandler = $.proxy(this._touchEnd, this);
                this._touchMoveHandler = $.proxy(this._touchMove, this);
            },

            _wireEvents: function (remove) {
                ej.listenTouchEvent(this.element, ej.startEvent(), this._touchStartHandler, remove);
            },

            _touchStart: function (evt) {
                this._target = evt.currentTarget;
                this.element.removeClass("e-m-state-default").addClass("e-m-state-active");
                if (this.model.touchStart)
                    this._trigger("touchStart", data = { text: this.model.text, currentEvent: evt });
                ej.listenEvents([window, this.element], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler], false);
            },

            _touchMove: function (evt) {
                this.element.removeClass("e-m-state-active").addClass("e-m-state-default");
                ej.listenTouchEvent(window, ej.moveEvent(), this._touchMoveHandler, true);
            },

            _touchEnd: function (evt) {
                if (this._target == this.element[0]) {
                    this._target = [];
                    this.element.removeClass("e-m-state-active").addClass("e-m-state-default");
                    if (this.element.hasClass("e-m-btn-large"))
                        evt.preventDefault();
                    if (this.model.style == "back" && !this.model.touchEnd)
                        history.back();
                    else if (this.model.touchEnd)
                        this._trigger("touchEnd", data = { text: this.model.text, currentEvent: evt });
                }
            },

            _controlStatus: function (enabled) {
                this.element[enabled ? "removeClass" : "addClass"]("e-m-state-disabled");
                this._wireEvents(!enabled);
            },

            _setImagePosition: function () {
                if (this.model.imageClass && this.model.contentType != "text") {
                    this._imgClass = this.model.imageClass;
                    this.element.addClass("e-m-btn-image " + this.model.imageClass);
                    if (this.model.contentType == "both")
                        this.element.removeClass("e-m-image-left e-m-image-right").addClass("e-m-image-" + this.model.imagePosition);
                    if (this.model.contentType == "image") {
                        this.element.removeClass("e-m-image-" + this.model.imagePosition).addClass("e-m-btn-imageonly").attr("value", "").html("");
                        this.model.text = "";
                    }
                }
                if (this.model.contentType == "text")
                    this.element.removeClass(this.model.imageClass + " e-m-image-" + this.model.imagePosition);
            },

            _setLocale: function () {
                this._getLocalizedLabels();
                this.model.text = this._localizedLabels["text"];
                this._setText(this.model.text);
            },

            _setModel: function (options) {
                for (var prop in options) {
                    setprop = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                    if (this[setprop])
                        this[setprop](options[prop])
                }
            },

            _setRenderMode: function () {
                this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
            },

            _setStyle: function () {
                this.element.removeClass("e-m-btn-normal e-m-btn-back e-m-btn-header e-m-btn-large").addClass("e-m-btn-" + this.model.style + " " + (this.model.style == "back" ? "e-m-icon-back" : ""));
                this._setText(this.model.text);
            },
            _setShowBorder: function (border) {
                border ? this.element.removeClass("e-m-noborder").addClass("e-m-border") : this.element.addClass("e-m-noborder").removeClass("e-m-border");
            },

            _setEnabled: function () {
                this._controlStatus(this.model.enabled);
            },

            _setText: function () {
                this.element[0].nodeName == "INPUT" ? this.element.attr("value", this.model.text) : this.element.html(this.model.text);
            },

            _setContentType: function () {
                this._setImagePosition();
            },

            _setImageClass: function () {
                this.element.removeClass(this._imgClass);
                this._setImagePosition();
            },

            _clearElement: function () {
                this.element.removeAttr("class");
                this.element.addClass("e-m-btn");
            },

            _destroy: function () {
                this._wireEvents(true);
                this._clearElement();
            },

            /*---------------Public Methods---------------*/
            enable: function () {
                this.model.enabled = true;
                this._controlStatus(true);
            },

            disable: function () {
                this.model.enabled = false;
                this._controlStatus(false);
            }
        });


    ej.mobile.Button.ImagePosition = {
        Left: "left",
        Right: "right"
    };

    ej.mobile.Button.ContentType = {
        Text: "text",
        Image: "image",
        Both: "both"
    };

    ej.mobile.Button.Style = {
        Normal: "normal",
        Back: "back",
        Header: "header",
        Large: "large"
    };

    ej.mobile.Button.Locale = ej.mobile.Button.Locale || {};
    ej.mobile.Actionlink.Locale = ej.mobile.Actionlink.Locale || {};
    ej.mobile.Button.Locale["default"] = ej.mobile.Button.Locale["en-US"] = {
        text: "Button"
    };

    ej.mobile.Actionlink.Locale["default"] = ej.mobile.Actionlink.Locale["en-US"] = {
        text: "Actionlink"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Datepicker elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejmDatePicker", "ej.mobile.DatePicker", {
        _setFirst: true,
        _requiresID: true,
        _rootCSS: "e-m-datepicker",
        defaults: {
            dateFormat: null,
            renderMode: "auto",
            minDate: null,
            maxDate: null,
            locale: "en-US",
            value: null,
            enabled: true,
            enablePersistence: false,
            cssClass: "",
            select: null,
            load: null,
            focusIn: null,
            focusOut: null,
            open: null,
            close: null,
            change: null,
        },
        dataTypes: {
            renderMode: "enum",
            enabled: "boolean",
            enablePersistence: "boolean"
        },
        _init: function () {
            this._createDelegates();
            this._renderControl();
            this._wireEvents();
        },
        _datepickerInitialize: function () {
            this._winrt = this.model.renderMode == "windows" && !ej.isLowerResolution();
            this._menuTargetId = this._id;
            this._setLocale(this.model.locale);
            this.model.dateFormat = this.model.dateFormat == null ? Date.format : this.model.dateFormat;
            this.model.minDate = this.model.minDate ? this.model.minDate : "01/01/2000";
            this.model.maxDate = this.model.maxDate ? this.model.maxDate : "12/31/2030";
            if (!this.model.value) {
                date = new Date();
                this.model.value = (date.getMonth() + 1) + '/' + date.getDate() + '/' + date.getFullYear();
            }
            this._currentDate = new Date(this.model.minDate) > new Date(this.model.value) ? this.model.minDate : new Date(this.model.maxDate) < new Date(this.model.value) ? this.model.maxDate : this.model.value;
            this._prevResolution = ej.isLowerResolution();
        },
        _setLocale: function (culture) {
            this.culture = ej.preferredCulture(culture);
            this._localizedLabels = this._getLocalizedLabels();
            if (this.culture) {
                Date.dayNames = this.culture.calendar.days.names;
                Date.abbrDayNames = this.culture.calendar.days.namesAbbr;
                Date.monthNames = this.culture.calendar.months.names;
                Date.abbrMonthNames = this.culture.calendar.months.namesAbbr;
                Date.format = this.culture.calendar.patterns.d;
            }
        },
        _renderControl: function () {
            ej.setRenderMode(this);
            this._datepickerInitialize();
            this._datepickerwrapper = ej.buildTag("div.e-m-datepickerwrapper" + " " + this.model.cssClass + " " + "e-m-" + this.model.renderMode + " e-m-user-select#" + this._id + "_Wrapper", "", {}, {});
            this.element.attr({ "name": this._id }).wrap(this._datepickerwrapper);
            this._datepickerwrapper = this.element.parent();
            this.element.attr({ "readonly": true });
            this._outerdiv = ej.buildTag("div.e-m-" + this.model.renderMode + " e-m-dp e-m-user-select #" + this._id + "_dp", "", {}, {});
            this._innerdiv = ej.buildTag("div.e-m-dpinner", "", {}, {});
            this._wrapdiv = ej.buildTag("div.e-m-dpwrap", "", {}, {});
            this._visibletexthtml = this._winrt ? "" : "<div class='col-fill'></div>";
            this._months = ej.buildTag("div.e-m-dp e-m-dpmonth e-m-left", "<div class='e-m-datewrapper'><div id=" + this._id + "_dpmonth><div class='e-m-dpouter'>" + this._visibletexthtml + "</div></div></div><div class='e-m-dp-overlay'></div></div>", {}, {});
            this._days = ej.buildTag("div.e-m-dp e-m-dpdate e-m-center", "<div class='e-m-datewrapper'><div id=" + this._id + "_dpdate><div class='e-m-dpouter'>" + this._visibletexthtml + "</div></div></div><div class='e-m-dp-overlay'></div></div>", {}, {});
            this._years = ej.buildTag("div.e-m-dp e-m-dpyear e-m-right", "<div class='e-m-datewrapper'><div id=" + this._id + "_dpyear><div class='e-m-dpouter'>" + this._visibletexthtml + "</div></div></div><div class='e-m-dp-overlay'></div></div>", {}, {});
            this["_" + this.model.renderMode + "Rendering"]();
            this._scrollInitialize();
            this._domElements();
            this._minDateInitialize();
            this._maxDateInitialize();
            if (this.model.load)
                this._trigger("load", this._argsValue());
            this._controlStatus(this.model.enabled);
            this._initscroll = false;
            this._setcurrent = false;
            this._setformat = false;
            this.element.addClass("e-m-user-select");
            this._setCurrentPreviousDate();
        },
        _setCurrentPreviousDate: function () {
            this._updateValue(this.model.dateFormat);
            this._previousDate = $(this.element).val();
        },
        _scrollPanel: function (e) {
            this["_" + e] = ej.buildTag("div#" + this._id + "dp" + e + "_scrollpanel");
            this._outerdiv.find("#" + this._id + "_dp" + e).append(this["_" + e]);
            this["_" + e].ejmScrollPanel({
                enableMouseWheel: false, enableNativeScrolling: false, enableTransition: false, enableTransform: false, showScrollbars: false, target: this._id + "_dp" + e, targetHeight: this._targetHeight, renderMode: this.model.renderMode,
                scrollStart: $.proxy(this._onScrollStart, this), scrollStop: $.proxy(this._onScrollStop, this), enableBounce: false, enableDisplacement: true, displacementValue: this._displacement, displacementTime: ej.isWindows() ? 800 : 350, isRelative: true
            });
        },
        _windowsTargetHeight: function () {
            return this._closestAppview.innerHeight() - (ej.getDimension(this._toolbar, "outerHeight") + ej.getDimension($(this._outerdiv).find(".e-m-dp-header"), "outerHeight"))
        },
        _scrollInitialize: function () {
            this._displacement = this.model.renderMode == "ios7" ? 40 : (this.model.renderMode == "android" ? 45 : 90); //40, 45, 90 - height of ios7, android, windows tiles
            this._targetHeight = this.model.renderMode == "ios7" ? 200 : (this.model.renderMode == "android" ? 135 : !this._winrt ? this._windowsTargetHeight() : 0); //200, 135 - scrollheights of ios7, android
            if (!this._winrt) {
                this._scrollPanel("date");
                this._scrollPanel("month");
                this._scrollPanel("year");
            }
        },
        _minDateInitialize: function () {
            if (!this._winrt) {
                this._minDate = new Date(this.model.minDate);
                var d = this._minDate.getDate(), m = this._minDate.getMonth();
                this._minMonths = this._monthtext.slice(0, m);
                this._minDates = this._datetext.slice(0, d - 1);
            }
        },
        _maxDateInitialize: function () {
            if (!this._winrt) {
                this._maxDate = new Date(this.model.maxDate);
                var d = this._maxDate.getDate(), m = this._maxDate.getMonth();
                this._maxMonths = this._monthtext.slice(m + 1, 12);
                this._maxDates = this._datetext.slice(d, 31);
            }
        },
        _renderTemplate: function () {
            var template = {};
            template["default"] = "<div class='e-m-text {{:class}}'>{{:value}}</div>";
            template["monthtemplate"] = "<div class='e-m-text'>{{:secondvalue}}</div>";
            template["windows"] = "<div class='e-m-text {{:class}}'><div class='e-m-text-inner'>{{:value}}</div><div class='e-m-text-val e-m-dp-dayweek'>{{:secondvalue}}</div></div>";
            template["winrt"] = "<option value='{{:value}}'>{{:value}}</option>";
            template["winrtmonths"] = "<option value='{{:value}}'>{{:secondvalue}}</option>";
            $.templates(template);
        },
        _renderDateContent: function () {
            this._renderTemplate();
            this._monthsRendering();
            this._daysRendering();
            this._yearsRendering();
            this._innerdiv.append(this._wrapdiv);
            this._outerdiv.append(this._innerdiv);
        },
        _ios7Rendering: function () {
            this._renderDateContent();
            this._innerdiv.append("<div class='e-m-ios7-overlay'></div>");
            this._menu = ej.buildTag('div#' + this._id + '_menu.e-m-dp-menu');
            this._menu.append(this._outerdiv);
            ej.getCurrentPage().append(this._menu);
            this._menu.ejmMenu({
                allowScrolling: false, type: (ej.isLowerResolution() ? "actionsheet" : "popover"), target: this._menuTargetId, height: 205,
                renderMode: this.model.renderMode, show: $.proxy(this._setInitialDate, this), showTitle: false, cancelButton: { show: false }, hide: $.proxy(this._menuHide, this)
            });
        },
        _androidRendering: function () {
            this._renderDateContent();
            this._dialog = ej.buildTag("div.e-m-dp-dialog#" + this._id + "_dialog");
            this._dialog.ejmDialog({ allowScrolling: false, title: this._localizedLabels.headerText, mode: "confirm", leftButtonCaption: this._localizedLabels.confirmText, rightButtonCaption: this._localizedLabels.cancelText, renderMode: this.model.renderMode, enableAnimation: false });
            this._dialog.find(".e-m-dlg-content").append(this._outerdiv);
        },
        _windowsRendering: function () {
            if (this.model.renderMode == "windows" && !ej.isLowerResolution()) {
                this._renderDateContent();
                this._dialog = ej.buildTag("div.e-m-dp-dialog#" + this._id + "_dialog");
                this._dialog.ejmDialog({
                    enableModal: true, allowScrolling: false, width: "100%", enableAnimation: false, title: this._localizedLabels.headerText, leftButtonCaption: this._localizedLabels.confirmText, rightButtonCaption: this._localizedLabels.cancelText, mode: "confirm", renderMode: this.model.renderMode
                });
                this._dialog.find(".e-m-dlg-content").append(this._outerdiv);
            }
            else {
                this._outerdiv.append("<div class='e-m-dp-header' style='visibility:hidden;'>" + this._localizedLabels.headerText + "</div>");
                this._renderDateContent();
                this._toolbar = ej.buildTag("div#" + this._id + "_toolbar.e-m-dp-tb", "<ul><li data-ej-iconname='check'></li><li data-ej-iconname='close'></li></ul>");
                this._toolbar.ejmNavigationBar({ mode: "toolbar", position: "bottom", renderMode: this.model.renderMode, touchEnd: $.proxy(this._winToolbarItem, this) });
                this._closestAppview = this._outerdiv.closest(".appview").length ? this._outerdiv.closest(".appview") : ej.getCurrentPage();
                ej.getCurrentPage().append(this._outerdiv).append(this._toolbar);
                this._outerdiv.find(".e-m-dpouter").find(".col-fill").css("height", ((this._windowsTargetHeight() - 90) / 2)); //height of a tile is 90px
                this._outerdiv.css("top", -this._closestAppview.innerHeight() + "px");
                this._toolbar.css({ "visibility": "hidden" });
            }
        },
        _flatRendering: function () {
            this._windowsRendering();
        },
        _daysRendering: function () {
            var daysdata = [];
            this._daysofmonth = this._getDaysInMonth(new Date(this.model.value));
            for (var i = 1; i <= 31; i++) {
                var dayofweek = Date.dayNames[(new Date(new Date(this.model.value).getMonth() + 1 + '/' + (i) + "/" + new Date(this.model.value).getFullYear()).getDay())];
                if (i <= this._daysofmonth)
                    daysdata.push({ "value": i, "secondvalue": dayofweek, "class": "" });
                else
                    daysdata.push({ "value": i, "secondvalue": dayofweek, "class": "e-m-state-disabled" });
            }
            this._columnRendering("days", daysdata);
        },
        _winrtContent: function (column, data) {
            this._select = ej.buildTag("select");
            data = $.render[column == "months" ? "winrtmonths" : "winrt"](data);
            this["_" + column].find(".e-m-dpouter").html(this._select.append(data));
            this._wrapdiv.append(this["_" + column]);
        },
        _monthsRendering: function () {
            var monthsdata = [];
            var monthnames = this.model.renderMode == "android" ? "abbrMonthNames" : "monthNames";
            for (var i = 0; i < 12; i++)
                monthsdata.push({ "secondvalue": Date[monthnames][i], "value": (i + 1) });
            this._columnRendering("months", monthsdata);
        },
        _yearsRendering: function () {
            var yeardata = [];
            for (var i = new Date(this.model.minDate).getFullYear() ; i <= new Date(this.model.maxDate).getFullYear() ; i++)
                yeardata.push({ "value": i });
            this._columnRendering("years", yeardata);
        },
        _columnRendering: function (column, data) {
            if (this._winrt)
                this._winrtContent(column, data);
            else {
                var templateval = this.model.renderMode == "windows" || this.model.renderMode == "flat" ? "windows" : column == "years" ? "default" : (column == "months" ? "monthtemplate" : "default");
                data = $.render[templateval](data) + this._visibletexthtml;
                this["_" + column].find(".e-m-dpouter").html(this["_" + column].find(".e-m-dpouter").html() + data);
                this._wrapdiv.append(this["_" + column]);
            }
        },
        _createDelegates: function () {
            this._winScrlHndlr = $.proxy(this._outerScroll, this);
            this._focusHndlr = $.proxy(this._showDatePicker, this);
            this._blurHndlr = $.proxy(this._blurEventHandler, this);
            this._andBtnClkHndlr = $.proxy(this._androidDialogButtonClick, this);
            this._winrtBtnClkHndlr = $.proxy(this._winrtDialogButtonClick, this);
            this._OrientationChangeHndlr = $.proxy(this._OrientationChange, this);
            this._resizeHndlr = $.proxy(this._resize, this);
        },
        _wireEvents: function (remove) {
            if (this.model.renderMode == "android")
                ej.listenTouchEvent($(this._dialog).find(".e-m-dlg-btn"), ej.tapEvent(), this._andBtnClkHndlr, remove);
            else if (this._winrt)
                ej.listenTouchEvent($(this._dialog).find(".e-m-dlg-btn"), ej.tapEvent(), this._winrtBtnClkHndlr, remove);
            else if (this.model.renderMode == "windows" || this.model.renderMode == "flat")
                ej.listenTouchEvent($(this._outerdiv).find(".e-m-dpwrap .e-m-dp"), ej.startEvent(), this._winScrlHndlr, remove);
            ej.listenEvents([$(this.element), $(this.element)], ["focus", "blur"], [this._focusHndlr, this._blurHndlr], remove);
            ej.listenTouchEvent($(window), "onorientationchange" in window ? "orientationchange" : "resize", this._resizeHndlr, remove);
        },
        _resize: function (e) {
            this._currentResolution = ej.isLowerResolution();
            if ((this.model.renderMode == "ios7" || this.model.renderMode == "windows") && this._prevResolution != this._currentResolution)
                this._refresh();
            else
                this._prevResolution = this._currentResolution;
            if ((this.model.renderMode == "windows" && ej.isLowerResolution()) || this.model.renderMode == "flat") {
                this._outerdiv.find(".e-m-dpouter").find(".col-fill").css("height", ((this._windowsTargetHeight() - 90) / 2)); //height of a tile is 90px
                this._date.ejmScrollPanel({ "targetHeight": this._windowsTargetHeight() });
                this._month.ejmScrollPanel({ "targetHeight": this._windowsTargetHeight() });
                this._year.ejmScrollPanel({ "targetHeight": this._windowsTargetHeight() });
                this._setValue(this._currentDate);
                if (this._outerdiv.offset().top != 0)
                    this._outerdiv.css("top", -this._closestAppview.innerHeight());
            }
        },
        _scrollCoordinates: function () {
            var day = {};
            day.Scrolldate = Math.round(this._date.ejmScrollPanel("getScrollPosition").y / this._displacement);
            day.Scrollmonth = Math.round(this._month.ejmScrollPanel("getScrollPosition").y / this._displacement);
            day.Scrollyear = Math.round(this._year.ejmScrollPanel("getScrollPosition").y / this._displacement);
            day.date = -(day.Scrolldate) + 1;
            day.month = -(day.Scrollmonth) + 1;
            day.year = -(day.Scrollyear) + new Date(this.model.minDate).getFullYear();
            return day;
        },
        _domElements: function () {
            this._windate = this._days.find("#" + this._id + "_dpdate");
            this._winmonth = this._months.find("#" + this._id + "_dpmonth");
            this._winyear = this._years.find("#" + this._id + "_dpyear");
            this._datetext = this._windate.find(".e-m-text");
            this._monthtext = this._winmonth.find(".e-m-text");
            this._yeartext = this._winyear.find(".e-m-text");
        },
        _activeText: function (date, month, year, target) {
            if ((target && target == "dpdate") || target == null) {
                $(this._datetext).removeClass("e-m-text-active");
                $(this._datetext[date - 1]).addClass("e-m-text-active");
            }
            if (target && target == "dpmonth" || target == null) {
                $(this._monthtext).removeClass("e-m-text-active");
                $(this._monthtext[month - 1]).addClass("e-m-text-active");
            }
            if (target && target == "dpyear" || target == null) {
                $(this._yeartext).removeClass("e-m-text-active");
                $(this._yeartext[(year - new Date(this.model.minDate).getFullYear())]).addClass("e-m-text-active");
            }
        },
        _setDateInput: function () {
            if (this.model.renderMode != "ios7")
                this._updateValue(this.model.dateFormat);
            this._dateSelect();
        },
        _winrtInitialDate: function (date) {
            var date = new Date(this._currentDate);
            this._windate.find("select option[value='" + date.getDate() + "']").attr("selected", true);
            this._winmonth.find("select option[value='" + (date.getMonth() + 1) + "']").attr("selected", true);
            this._winyear.find("select option[value='" + date.getFullYear() + "']").attr("selected", true);
        },
        _setInitialDate: function () {
            if (this._winrt)
                this._winrtInitialDate(this._currentDate);
            else {
                if (this.model.renderMode == "ios7")
                    this._initscroll = true;
                this._setValue(this._currentDate);
                var proxy = this;
                setTimeout(function () {
                    proxy._setProperDate(100);
                }, 200);
                if (this.model.renderMode == "ios7")
                    this._initscroll = false;
            }
        },
        _setValue: function (date) {
            var date = new Date(date);
            this._resetScrollPos();
            this._time = !ej.isIOS7() ? 0 : (!this._initscroll && !this._setcurrent) ? 200 : 0;
            this._date.ejmScrollPanel("scrollTo", 0, -this._displacement * (date.getDate() - 1), this._time);
            this._month.ejmScrollPanel("scrollTo", 0, -this._displacement * (date.getMonth()), this._time);
            this._year.ejmScrollPanel("scrollTo", 0, -this._displacement * (date.getFullYear() - new Date(this.model.minDate).getFullYear()), this._time);
        },
        _resetScrollPos: function () {
            this._date.ejmScrollPanel("scrollTo", 0, 0, 0);
            this._month.ejmScrollPanel("scrollTo", 0, 0, 0);
            this._year.ejmScrollPanel("scrollTo", 0, 0, 0);
        },
        _formatter: function (date, format) {
            var newFormat = this._checkFormat(format);
            return ej.format(date, newFormat, this.model.locale);
        },
        _parseDate: function (date) {
            if (!this._setformat)
                this._oldFormat = this.model.dateFormat;
            var newFormat = this._checkFormat(this._oldFormat);
            return ej.parseDate(date, newFormat, this.model.locale);
        },
        _checkFormat: function (format) {
            var proxy = this;
            var dateFormatRegExp = this._regExp();
            return format.replace(dateFormatRegExp, function (match) {
                match = match === "/" ? proxy.culture.calendars.standard['/'] !== "/" ? "'/'" : match : match;
                return match;
            });
        },
        _changeFormat: function (dateformat) {
            var proxy = this;
            var dateFormatRegExp = this._regExp();
            return dateformat.replace(dateFormatRegExp, function (match) {
                match = match === "/" ? proxy.culture.calendars.standard['/'] !== "/" ? "'/'" : match : match == 'd' ? new Date(proxy._currentDate).getDate() : match == 'M' ? new Date(proxy._currentDate).getMonth() + 1 : ej.format(new Date(proxy._currentDate), match);
                return match;
            });
        },
        _regExp: function () {
            return /dddd|\/|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|HH|H|hh|h|mm|m|fff|ff|f|tt|ss|s|zzz|zz|z|gg|g|"[^"]*"|'[^']*'|[/]/g;
        },
        _checkMinMax: function (day) {
            this._outerdiv.find(".e-m-state-disabled").removeClass("e-m-state-disabled");
            if (day.year == this._minDate.getFullYear() || day.year == this._maxDate.getFullYear()) {
                var m = day.year == this._minDate.getFullYear() ? "min" : "max";
                this["_" + m + "Months"].addClass("e-m-state-disabled");
                if (m == "min" ? day.month < this._minDate.getMonth() + 1 : day.month > this._maxDate.getMonth() + 1) {
                    day.month = this["_" + m + "Date"].getMonth() + 1;
                    this._month.ejmScrollPanel("scrollTo", 0, -this._displacement * (this["_" + m + "Date"].getMonth()), this._time);
                }
                else if (m == "min" ? day.month == this._minDate.getMonth() + 1 : day.month == this._maxDate.getMonth() + 1) {
                    this["_" + m + "Dates"].addClass("e-m-state-disabled");
                    if (m == "min" ? day.date < this._minDate.getDate() : day.date > this._maxDate.getDate()) {
                        day.date = this["_" + m + "Date"].getDate();
                        this._date.ejmScrollPanel("scrollTo", 0, -this._displacement * (this["_" + m + "Date"].getDate() - 1), this._time);
                    }
                }
            }
            return day;
        },
        _setProperDate: function (duration, target) {
            var day = this._scrollCoordinates();
            day = this._checkMinMax(day);
            var proxy = this;
            if (target == "dpmonth" || target == "dpyear")
                this._daysofmonth = this._getDaysInMonth(new Date(day.month + "/1/" + day.year));
            for (var i = 27; i <= 30; i++) {
                if (i + 1 > this._daysofmonth)
                    $(this._datetext[i]).addClass("e-m-state-disabled");
            }
            if (this._daysofmonth < day.date) {
                day.date = this._daysofmonth;
                this._date.ejmScrollPanel("scrollTo", 0, -this._displacement * (this._daysofmonth - 1), this._time);
            }
            if (this.model.renderMode == "ios7" && !this._initscroll) {
                setTimeout(function () {
                    $(proxy.element).val(proxy._formatter(new Date(day.month + "/" + day.date + "/" + day.year), proxy.model.dateFormat));
                }, duration);
            }
            this._currentDate = day.month + "/" + day.date + "/" + day.year;
            this._activeText(day.date, day.month, day.year, target);
            this.model.value = this._currentDate;
            if (this.model.change)
                this._trigger("change", this._argsValue());
        },
        _argsValue: function () {
            return { value: this._currentDate };
        },
        _onScrollStop: function (e) {
            var target = e.model.target.replace(this._id + "_", '');
            this._setProperDate(this.model.renderMode == "windows" ? 700 : 200, target);
            if (this["_" + this.model.renderMode + "ScrollStop"])
                this["_" + this.model.renderMode + "ScrollStop"](e.model.target, target, e);
        },
        _windowsScrollStop: function (e, target, scrollModel) {
            var day = this._scrollCoordinates();
            for (var i = 0; i < this._getDaysInMonth(new Date(day.month + "/" + day.date + "/" + day.year)) ; i++) {
                var dayofweek = Date.dayNames[(new Date(day.month + '/' + (i + 1) + "/" + day.year)).getDay()];
                $(this._datetext[i]).find(".e-m-dp-dayweek").html(dayofweek);
            }
            var startTarget = this._id + "_" + this._target;
            this._hideWrapper(target);
        },
        _flatScrollStop: function (e, target, scrollModel) {
            this._windowsScrollStop(e, target, scrollModel);
        },
        _onScrollStart: function (e) {
            this._target = e.model.target.replace(this._id + "_", '');
            if (this["_" + this.model.renderMode + "ScrollStart"])
                this["_" + this.model.renderMode + "ScrollStart"](e.model.target, this._target);
        },
        _windowsScrollStart: function (e, target) {
            $("#" + e + "").find(".e-m-text.e-m-text-active").removeClass("e-m-text-active");
            this._hideWrapper(target);
        },
        _flatScrollStart: function (e, target) {
            this._windowsScrollStart(e, target);
        },
        _hideWrapper: function (target) {
            if (target && target == "dpdate") {
                if (!this._year.ejmScrollPanel("instance")._moved)
                    this._winyear.closest(".e-m-datewrapper").removeClass("e-m-scrollstart");
                if (!this._month.ejmScrollPanel("instance")._moved)
                    this._winmonth.closest(".e-m-datewrapper").removeClass("e-m-scrollstart");
            }
            if (target && target == "dpmonth") {
                if (!this._year.ejmScrollPanel("instance")._moved)
                    this._winyear.closest(".e-m-datewrapper").removeClass("e-m-scrollstart");
                if (!this._date.ejmScrollPanel("instance")._moved)
                    this._windate.closest(".e-m-datewrapper").removeClass("e-m-scrollstart");
            }
            if (target && target == "dpyear") {
                if (!this._month.ejmScrollPanel("instance")._moved)
                    this._winmonth.closest(".e-m-datewrapper").removeClass("e-m-scrollstart");
                if (!this._date.ejmScrollPanel("instance")._moved)
                    this._windate.closest(".e-m-datewrapper").removeClass("e-m-scrollstart");
            }
        },
        _blurEventHandler: function () {
            if (this.model.focusOut) {
                this._trigger("focusOut", this._argsValue());
            }
            if (this.model.renderMode == "ios7") {
                if (this.model.select) {
                    if (this._previousDate != $(this.element).val())
                        this._trigger("select", this._argsValue());
                }
                if (this.model.close)
                    this._trigger("close", this._argsValue());
            }
        },
        _dateSelect: function (e) {
            if (this._previousDate != $(this.element).val()) {
                if (this.model.select)
                    this._trigger("select", this._argsValue());
            }
            this._previousDate = $(this.element).val();
            this._hideDatePicker();
        },
        _winToolbarItem: function (e) {
            if (e.iconname == "check")
                this._setDateInput();
            else if (e.iconname == "close")
                this._hideDatePicker();
        },
        _winrtDialogButtonClick: function (e) {
            if ($(e.target).text() == this._localizedLabels.confirmText) {
                this._currentDate = this._formatter(new Date(this._winmonth.find("select option:selected").val() + "/" + this._windate.find("select option:selected").val() + "/" + this._winyear.find("select option:selected").val()), this._setformat ? this._oldFormat : this.model.dateFormat);
                this._setDateInput();
            }
            else
                this._showHidePicker(true);
        },
        _androidDialogButtonClick: function (e) {
            if ($(e.target).text() == this._localizedLabels.confirmText)
                this._setDateInput();
            else
                this._showHidePicker(true);
        },
        _outerScroll: function (e) {
            this._hideWrapper($(e.target).parents(".e-m-scrollpanel")[0].id.replace(this._id + "_", ''));
            $(e.target).closest(".e-m-dp").find(".e-m-datewrapper").addClass("e-m-scrollstart");
        },
        _showHidePicker: function (remove) {
            var proxy = this;
            var e = { currentTarget: this._datepickerwrapper };
            var visibility = remove ? "hidden" : "visible";
            var zindex = ej.getMaxZindex();
            this.element.blur();
            if (this._winrt)
                this._dialog.ejmDialog(remove ? "close" : "open");
            else if (this.model.renderMode == "android")
                this._dialog.ejmDialog(remove ? "close" : "open");
            else if (this.model.renderMode == "windows" || this.model.renderMode == "flat") {
                this._outerdiv.css({ "top": remove ? -this._closestAppview.innerHeight() : "0px", "visibility ": visibility });
                $(this._outerdiv).find(".e-m-dp-header").css({ "visibility": visibility });
                this._toolbar.css({ "visibility": visibility });
            }
        },
        _showDatePicker: function (e) {
            this._setInitialDate();
            if (this.model.focusIn)
                this._trigger("focusIn", this._argsValue());
            this._showHidePicker();
            this._outerdiv.find(".e-m-datewrapper").removeClass("e-m-scrollstart");
            if (this.model.open)
                this._trigger("open", this._argsValue());
        },
        _hideDatePicker: function (e) {
            this._showHidePicker(true);
            if (this.model.close)
                this._trigger("close", this._argsValue());
        },
        _getDaysInMonth: function (date) {
            return [31, (this._isLeapYear(date.getFullYear()) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][date.getMonth()];
        },
        _isLeapYear: function (year) {
            return (year % 4 == 0 && year % 100 != 0) || year % 400 == 0;
        },
        _refreshScroller: function (e) {
            this._date.ejmScrollPanel("refresh");
            this._month.ejmScrollPanel("refresh");
            this._year.ejmScrollPanel("refresh");
        },
        _setModel: function (options) {
            var refresh;
            for (var prop in options) {
                if (prop == "renderMode" || prop == "minDate" || prop == "maxDate" || prop == "locale")
                    refresh = true;
                else {
                    setprop = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                    if (this[setprop])
                        this[setprop](options[prop])
                }
            }
            if (refresh)
                this._refresh();
        },
        _setEnabled: function (enabled) {
            this._controlStatus(enabled);
        },
        _setDateFormat: function (dateformat) {
            this._setformat = true;
            this._updateValue(dateformat);
        },
        _updateValue: function (dateformat) {
            var newVal = this._changeFormat(dateformat);
            $(this.element).val(newVal);
        },
        _controlStatus: function (enabled) {
            this._datepickerwrapper[!enabled ? "addClass" : "removeClass"]("e-m-state-disabled");
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-m-datepicker");
            this._renderControl();
            this._wireEvents();
        },
        _clearElement: function () {
            this.element.removeAttr("class").removeAttr("style");
            this.element.insertAfter(this._datepickerwrapper);
            this._datepickerwrapper.remove();
            if (this._dialog) {
                this._dialog.ejmDialog("instance")._destroy();
                this._dialog.remove();
                this._dialog = null;
            }
            else if (this._toolbar) {
                this._toolbar.ejmNavigationBar("instance")._destroy();
                this._toolbar.remove();
                this._toolbar = null;
            }
            else if (this._menu) {
                this._menu.ejmMenu("instance")._destroy();
                this._menu.remove();
                this._menu = null;
            }
            this.element = $(this.element);
            this._outerdiv.remove();
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
        _getLocalizedLabels: function () {
            return ej.getLocalizedConstants(this.sfType, this.model.locale)
        },

        /*---------------Public Methods---------------*/
        show: function (e) {
            var e = { currentTarget: this._datepickerwrapper };
            if (this.model.renderMode == "ios7")
                this._menu.ejmMenu("show", e);
            else
                this._showHidePicker();
        },
        hide: function (e) {
            var e = { currentTarget: this._datepickerwrapper };
            if (this.model.renderMode == "ios7")
                this._menu.ejmMenu("hide", e);
            else
                this._showHidePicker(true);
        },
        enable: function (e) {
            this.model.enabled = true;
            this._controlStatus(this.model.enabled);
            this._showHidePicker(true);
        },

        disable: function (e) {
            this.model.enabled = false;
            this._controlStatus(this.model.enabled);
            this._showHidePicker(true);
        },
        getValue: function () {
            return $(this.element).val();
        },
        setCurrentDate: function (date) {
            this._setcurrent = true;
            this._currentDate = date;
            this._daysofmonth = this._getDaysInMonth(new Date(date));
            this._setCurrentPreviousDate();
            this._setcurrent = false;
        }
        /*---------------Public Methods End---------------*/
    });
    ej.mobile.DatePicker.Locale = ej.mobile.DatePicker.Locale || {};

    ej.mobile.DatePicker.Locale["default"] = ej.mobile.DatePicker.Locale["en-US"] = {
        confirmText: "Done",
        cancelText: "Cancel",
        headerText: "CHOOSE DATE",
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Dialog elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmDialog", "ej.mobile.Dialog", {
        _setFirst: true,
        _rootCSS: "e-m-dialog",
        defaults: {
            renderMode: "auto",
            mode: "alert",
            enableAutoOpen: false,
            title: null,
            closeOndocumentTap: false,
            cssClass: "",
            enableModal: true,
            showButtons: true,
            showHeader: true,
            leftButtonCaption: null,
            rightButtonCaption: null,
            allowScrolling: false,
            enableNativeScrolling: (ej.isAndroid() || ej.isIOS()) ? false : ej.isDevice() ? true : false,
            templateId: null,
            height: null,
            width: null,
            enablePersistence: false,
            enableAnimation: true,
            locale: "en-US",
            open: null,
            close: null,
            beforeClose: null,
            buttonTap: null
        },
        dataTypes: {
            renderMode: "enum",
            mode: "enum",
            enableAutoOpen: "boolean",
            title: "string",
            cssClass: "string",
            enableModal: "boolean",
            showButtons: "boolean",
            showHeader: "boolean",
            leftButtonCaption: "string",
            rightButtonCaption: "string",
            allowScrolling: "boolean",
            templateId: "string",
            height: "string" || "number",
            width: "string" || "number",
            locale: "string",
            enablePersistence: "boolean",
            enableAnimation: "boolean",
        },
        _init: function () {
            ej.setRenderMode(this);
            this._orgEle = this.element.clone();
            this._getLocalizedLabels();
            this.model.title = !ej.isNullOrUndefined(this.model.title) ? this.model.title : this._localizedLabels["title"];
            this.model.leftButtonCaption = !ej.isNullOrUndefined(this.model.leftButtonCaption) ? this.model.leftButtonCaption : this._localizedLabels["leftButtonCaption"];
            this.model.rightButtonCaption = !ej.isNullOrUndefined(this.model.rightButtonCaption) ? this.model.rightButtonCaption : this._localizedLabels["rightButtonCaption"];
            this._renderControl();
            this._createDelegates();
            this._wireEvents();
            if (this.model.enableAutoOpen)
                this.open();
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale)
        },
        _renderControl: function () {
            this.element.addClass("e-m-" + this.model.renderMode + " e-m-dialog e-m-user-select " + this.model.cssClass + " " + "e-m-" + this.model.mode);
            if (this.model.enableAnimation) this.element.addClass("e-m-dlg-animate");
            if (!ej.isNullOrUndefined(this.model.templateId)) this._setTemplate();
            else this._html = this.element.html();
            this.element.empty();
            this._renderDialog();
            this._dialogInit();
            this.element.addClass("e-m-" + (this.model.enableAutoOpen ? "show" : "hide"));
            if (this.model.enableModal) this._setModal();
        },
        _setTemplate: function () {
            if (ej.getCurrentPage().find("#" + this.model.templateId).length) {
                var templateContent = ej.getCurrentPage().find("#" + this.model.templateId);
                this._html = templateContent[0].nodeName.toLowerCase() == "script" ? ej.getClearString(templateContent[0].innerHTML) : templateContent;
            }
            ej.destroyWidgets(this._html);
        },
        _renderButton: function () {
            this._leftbtn = ej.buildTag("button").addClass("e-m-" + this.model.mode + "btn e-m-dlg-btn e-m-dlg-alertbtn e-m-state-default").html(this.model.leftButtonCaption);
            this._dlgbtnwrapper = ej.buildTag("div.e-m-dlgbtnwrapper");
            this._innerDiv.append(this._dlgbtnwrapper);
            if (this.model.mode == "alert") this._dlgbtnwrapper.append(this._leftbtn);
            else if (this.model.mode == "confirm") {
                this._leftBtn = this._leftbtn.removeClass("e-m-" + this.model.mode + "btn").addClass("e-m-dlg-btn e-m-dlg-leftbtn").html(this.model.leftButtonCaption);
                this._rightBtn = ej.buildTag("button").addClass("e-m-dlg-btn e-m-dlg-rightbtn e-m-state-default").html(this.model.rightButtonCaption);
                this._dlgbtnwrapper.append(this._leftBtn).append(this._rightBtn);
            }
        },
        _renderDialog: function () {
            var height = !ej.isNullOrUndefined(this.model.height) && parseInt(this.model.height) ? this.model.height : "";
            var width = !ej.isNullOrUndefined(this.model.width) && parseInt(this.model.width) ? this.model.width : "";
            this.element.addClass("e-m-dialog e-m-abs");
            this._innerDiv = ej.buildTag("div", "", { "width": width }, { "class": "e-m-dlg-container e-m-abs " });
            this.element.append(this._innerDiv);
            this._contentContainer = ej.buildTag("div", this._html, { "height": height }, { "class": "e-m-dlg-content e-m-rel" });
            if (this.model.allowScrolling) this._contentContainer.attr({ "data-role": "ejmscrollpanel", "data-ej-showscrollbars": true, "data-ej-enablenativescrolling": this.model.enableNativeScrolling, "data-ej-isrelative": true })
            this._innerDiv.append(this._contentContainer);
            if (!(this.model.mode == "custom")) {
                ej.buildTag("div", this.model.title, {}, { "class": "e-m-dlg-hdr " + (!this.model.showHeader ? "e-m-hide" : "") }).insertBefore(this._contentContainer);
                this._header = this._innerDiv.find(".e-m-dlg-hdr");
                if (this.model.showButtons) this._renderButton();
            }
        },
        _dialogInit: function () {
            var target = this.element.parents("form").length > 0 ? this.element.parents("form") : ej.getCurrentPage();
            this.element.appendTo(target);
            ej.widget.init(this._innerDiv);
            if ((!ej.isAppNullOrUndefined()) && ej.angular.defaultAppName)
                ej.angular.compile(this._innerDiv);
        },
        _setModal: function () {
            var dialogZIndex = this.element.css("zIndex");
            this._overlay = ej.buildTag("div#" + ".e-m-overlay e-m-abs", "", { "zIndex": dialogZIndex - 1 });
            this.element.append(this._overlay);
        },
        _setTemplateId: function () {
            if (ej.getCurrentPage().find("#" + this.model.templateId).length) {
                this._setTemplate()
                this._contentContainer.empty().html(this._html);
                this._dialogInit()
            }
        },
        _setCloseOndocumentTap: function () {
            if (this.model.closeOndocumentTap) this._documentClickHandler = $.proxy(this._docClick, this);
            ej.listenEvents([$(document)], [ej.endEvent()], [this._documentClickHandler], !this.model.closeOndocumentTap);
        },
        _setLocale: function (option) {
            this._getLocalizedLabels();
            this.model.title = this._localizedLabels["title"];
            this.model.leftButtonCaption = this._localizedLabels["leftButtonCaption"];
            this.model.rightButtonCaption = this._localizedLabels["rightButtonCaption"];
            this._setTitle();
            this._setLeftButtonCaption();
            this._setRightButtonCaption();
        },
        _createDelegates: function () {
            this._touchEndHandler = $.proxy(this._buttonTap, this);
            this._touchStartHandler = $.proxy(this._touchStart, this);
            if (this.model.closeOndocumentTap) this._documentClickHandler = $.proxy(this._docClick, this);
        },
        _wireEvents: function (remove) {
            var evt = "onorientationchange" in window ? "orientationchange" : "resize";
            ej.listenEvents([window, this.element.find(".e-m-dlg-btn"), this.element.find(".e-m-dlg-btn"), $(document)], [evt, ej.startEvent(), ej.endEvent(), ej.endEvent()], [this._resizeHandler, this._touchStartHandler, this._touchEndHandler, this._documentClickHandler], remove);
        },
        _docClick: function (e) {
            if (($(e.target).hasClass("e-m-overlay") || $(e.target).hasClass("e-m-dialog")) && this.element.hasClass("e-m-show"))
                this.close();
        },
        _buttonTap: function (evt) {
            $(evt.target).removeClass("e-m-state-active").addClass("e-m-state-default");
            if (this.model.buttonTap) this._trigger("buttonTap", { text: $(evt.target).text(), title: this.model.title, currentEvent: evt });
        },
        _touchStart: function (evt) {
            $(evt.target).removeClass("e-m-state-default").addClass("e-m-state-active");
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel]();
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        _setRenderMode: function () {
            this.element.removeClass("e-m-android e-m-ios7 e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
        },

        _setMode: function () {
            this._refresh();
        },

        _setTitle: function () {
            if (this._header.length) this._header.html(this.model.title);
        },

        _setLeftButtonCaption: function () {
            this.element.find(".e-m-dlg-btn.e-m-dlg-" + (this.model.mode == "confirm" ? "leftbtn" : "alertbtn")).html(this.model.leftButtonCaption);
        },

        _setRightButtonCaption: function () {
            this.element.find(".e-m-dlg-btn.e-m-dlg-rightbtn").html(this.model.rightButtonCaption);
        },

        _setEnableModal: function () {
            if (this.model.enableModal)
                this.element.find(".e-m-overlay").length ? "" : this._setModal()
            else
                this.element.find(".e-m-overlay").remove();
        },

        _setShowButtons: function () {
            this._refresh();
        },

        _setShowHeader: function () {
            this.model.showHeader ? this._header.removeClass("e-m-hide") : this._header.addClass("e-m-hide")
        },

        /*** Instead of re-rendering dialog control, empty method triggerred ***/
        _setButtonTap: function () {
        },

        _setBeforeClose: function () {
        },

        _setClose: function () {
        },

        _setOpen: function () {
        },

        _destroy: function () {
            this._wireEvents(true);
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },

        _refresh: function () {
            this._destroy();
            this._renderControl();
            this._wireEvents(false);
        },

        _animate: function (add, remove) {
            var proxy = this;
            this._innerDiv.removeClass(add + " " + remove);
            this._innerDiv.addClass(add);
            if (add == "e-m-dlg-hideanimate") {
                delay = this.model.renderMode == "android" ? 250 : this.model.renderMode == "windows" ? 200 : 0;
                setTimeout(function () {
                    proxy.element.addClass("e-m-hide");
                }, delay);
            }
        },
        open: function () {
            if (this.model.allowScrolling) this._contentContainer.ejmScrollPanel("scrollTo", 0, 0);
            this.element.addClass("e-m-show").removeClass("e-m-hide");
            if (this.model.enableAnimation)
                this._animate("e-m-dlg-showanimate", "e-m-dlg-hideanimate");
            if (this.model.open)
                this._trigger("open", { "title": this.model.title });
        },
        close: function () {
            var data = { "title": this.model.title };
            if (this.model.beforeClose)
                this._trigger("beforeClose", data);
            if (this.model.enableAnimation)
                this._animate("e-m-dlg-hideanimate", "e-m-dlg-showanimate");
            else
                this.element.addClass("e-m-hide");
            this.element.removeClass("e-m-show");
            if (this.model.close)
                this._trigger("close", data);
        },

        disableButton: function (args) {
            ej.isNullOrUndefined(args) ? this.element.find(".e-m-dlg-btn").addClass("e-m-state-disabled") : this.element.find(".e-m-dlg-btn.e-m-" + args + "btn").addClass("e-m-state-disabled")
        },

        enableButton: function (args) {
            ej.isNullOrUndefined(args) ? this.element.find(".e-m-dlg-btn").removeClass("e-m-state-disabled") : this.element.find(".e-m-dlg-btn.e-m-" + args + "btn").removeClass("e-m-state-disabled")
        }
    });

    ej.mobile.Dialog.Mode = {
        Alert: "alert",
        Confirm: "confirm",
        Custom: "custom"
    };
    ej.mobile.Dialog.Locale = ej.mobile.Dialog.Locale || {};
    ej.mobile.Dialog.Locale["default"] = ej.mobile.Dialog.Locale["en-US"] = {
        title: "Title",
        leftButtonCaption: "Cancel",
        rightButtonCaption: "Continue"
    };
})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html DropDown elements
* @copyright Copyright Syncfusion Inc. 2001 - 2016. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmDropDownList", "ej.mobile.DropDownList", {
        _setFirst: true,
        _rootCSS: "e-m-dropdownlist",
        validTags: ["input"],
        defaults: {
            renderMode: "auto",
            cssClass: "",
            targetId: null,
            templateId: "",
            popupHeight: "164px",
            popupWidth: "auto",
            dataSource: [],
            fields: {
                text: null,
                value: null,
                groupBy: null,
                image: null,
                checkBy: null,
            },
            enableMultiSelect: false,
            enablePersistence: false,
            readOnly: false,
            enabled: true,
            query: null,
            watermarkText: null,
            locale: "en-US",
            selectedItemIndex: -1,
            delimiterChar: ",",
            itemsCount: 0,
            allowVirtualScrolling: false,
            focusIn: null,
            focusOut: null,
            select: null,
            change: null,
            actionSuccess: null,
            actionFailure: null,
            actionComplete: null,
        },
        dataTypes: {
            renderMode: "enum",
            dataSource: "data",
            itemsCount: "number",
            enableMultiSelect: "boolean",
            enablePersistence: "boolean",
            readOnly: "boolean",
            enabled: "boolean",
            allowVirtualScrolling: "boolean",
            locale: "string",
        },
        observables: ["selectedItemIndex"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        _init: function () {
            ej.setRenderMode(this);
            this._getLocalizedLabels();
            this._initialization();
            this._renderControl();
            !this.model.readOnly && this._wireEvents();
            this.model.watermarkText = !ej.isNullOrUndefined(this.model.watermarkText) ? this.model.watermarkText : this._localizedLabels["watermarkText"];
        },
        _renderControl: function () {
            this._renderList();
        },
        _wireEvents: function (remove) {
			if(!remove){
            this._touchStart = $.proxy(this._touchStartHandler, this);
            this._docTouchStart = $.proxy(this._docTouchStartHandler, this);
            this._docTouchEnd = $.proxy(this._docTouchEndHandler, this);
			}
            var doc = $(document);
            ej.listenEvents([this._eleWarpper, this._dropDownIcon, doc, doc], ["focus", ej.startEvent(), ej.startEvent(), ej.endEvent()], [this._touchStart, this._touchStart, this._docTouchStart, this._docTouchEnd], remove);
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
        },
        _initialization: function () {
            this._index = 0;
            var id = this.element.attr("id"),
                selectEle = this.element.is('select'),
                ulEle = selectEle && this._renderSelectOption(id);
            this._orgEle = this.element.clone();
            this._text = [];
            this._hiddenVal = [];
            this._targetEle = this.model.targetId ? $("#" + this.model.targetId) : (selectEle ? ulEle : ej.buildTag("ul"));
            this._eleWarpper = ej.buildTag("div#" + id + "_wrapper.e-m-dropdown-wrapper " + this.model.cssClass + " " + " e-m-" + this.model.renderMode + " e-m-user-select");
            this._hiddenElement = ej.buildTag("input#" + this._id + "_hidden", {}, { "display": "none" }, { "name": this._id });
            this.element.addClass("e-m-user-select " + "e-m-" + this.model.renderMode).attr({ "name": id, "readonly": true }).wrap(this._eleWarpper);
            this._eleWarpper = this.element.parent();
            this._dropDownIcon = ej.buildTag("div.e-m-icon-down").appendTo(this._eleWarpper.append(this._hiddenElement));
            this._setWatermarkText(this.model.watermarkText = !ej.isNullOrUndefined(this.model.watermarkText) ? this.model.watermarkText : this._localizedLabels["watermarkText"]);
            this._controlState(this.model.enabled);
        },
        _renderList: function () {
            if (ej.isNullOrUndefined(this._targetWrapper))
                this._targetWrapper = ej.buildTag("div.e-m-target-wrapper");
            this._targetWrapper.append(this._targetEle.ejmListView({
                renderMode: this.model.renderMode, dataSource: this.model.dataSource,
                query: this.model.query, enableChecklist: this.model.enableMultiSelect, itemsCount: this.model.itemsCount,
                fields: {
                    text: this.model.fields.text, value: this.model.fields.value, groupBy: this.model.fields.groupBy,
                    image: this.model.fields.image, checkBy: this.model.fields.checkBy
                }, templateId: this.model.templateId, touchEnd: $.proxy(this._listItemClickHandler, this),
                actionSuccess: $.proxy(this._onComplete, this), actionFailure: this.model.actionFailure, actionComplete: this.model.actionComplete
            })).appendTo(this._eleWarpper);
            !(this.model.dataSource instanceof ej.DataManager) && this._renderScroll();
            this.hidePopup();
            this._setSelectedItemIndex(this.selectedItemIndex(), true);
        },
        _renderScroll: function (evt) {
            var scrollEnd = this.model.allowVirtualScrolling ? $.proxy(this._virtualScroll, this) : null,
                lHeight = this._targetWrapper.height(), wHeight = window.innerHeight, popupHeight = parseInt(this.model.popupHeight),
                tarHeight = lHeight > popupHeight ? popupHeight : lHeight, tarOffSet = this.element.offset().top + this.element.height();
            popupHeight < lHeight && this._targetWrapper.ejmScrollPanel({
                renderMode: this.model.renderMode, targetHeight: popupHeight, scrollEnd: scrollEnd
            });
            (tarOffSet + tarHeight) > wHeight && this._targetWrapper.css('top', -tarHeight);
            this.model.popupWidth != "auto" && this._targetWrapper.css('width', this.model.popupWidth);
        },
        _onComplete: function (evt) {
            this._renderScroll(evt);
            this.model.actionSuccess && this._trigger("actionSuccess");
        },
        _touchStartHandler: function (evt) {
            if (!this.model.readOnly) {
                if (this._targetWrapper.css("display") == "none") {
                    this.element.addClass("e-m-focus");
                    this.model.focusIn && this._trigger("focusIn");
                    this.showPopup();
                }
                else {
                    this.element.removeClass("e-m-focus");
                    this.model.focusOut && this._trigger("focusOut");
                    this.hidePopup();
                }
                this.element.blur();
            }
        },
        _virtualScroll: function (evt) {
            if (evt.position == "bottom") {
                var listIns = this._targetEle.data('ejmListView'),
                    scollIns = this._targetWrapper.data('ejmScrollPanel');
                listIns.append(this.model.dataSource);
                scollIns.refresh();
            }
        },
        _docTouchStartHandler: function (evt) {
            if (!this._isTargetDropDown(evt.target))
                this._hide = true;
        },
        _docTouchEndHandler: function (evt) {
            if (!this._isTargetDropDown(evt.target) && this._hide) {
                this.element.removeClass("e-m-focus");
                this.model.focusOut && this._trigger("focusOut");
                this.hidePopup();
                this._hide = false;
            }
        },
        _isTargetDropDown: function (target) {
            return $(target).closest(this._eleWarpper).hasClass("e-m-dropdown-wrapper");
        },
        _listItemClickHandler: function (evt) {
            var text = this.model.templateId != "" ? evt.data[this.model.fields.text] : evt.text;
            this._updateSelectedItem(evt.item, text, evt.item.attr("data-value"), evt.isChecked);
            this.element.blur();
        },
        _renderSelectOption: function (id) {
            var inputEle = ej.buildTag("input#" + id + ".e-m-dropdownlist", null, null, { 'data-role': 'ejmdropdownlist' }),
                ulEle = ej.buildTag("ul");
            $(this.element.children()).each(function (i, e) {
                var liEle = ej.buildTag("li", null, null, { 'data-ej-text': e.text });
                ulEle.append(liEle);
            });
            this.element.hide();
            this.element.before(inputEle.after(ulEle));
            this.element = inputEle;
            return ulEle;
        },
        _setWatermarkText: function (text) {
            text ? this.element.attr("placeholder", text) : this.element.removeAttr("placeholder");
        },
        _controlState: function (enabled) {
            this._eleWarpper[!enabled ? "addClass" : "removeClass"]("e-m-state-disabled");
        },
        _setSelectedItemIndex: function (index, option) {
            var listObj = this._targetEle.data("ejmListView");
            if (this.selectedItemIndex() >= 0) {
                var text = listObj.getTextByIndex(this.selectedItemIndex()),
                    item = listObj.getItemByIndex(this.selectedItemIndex());
                if (option)
                    this.model.enableMultiSelect ? listObj.checkItemsByIndex(this.selectedItemIndex()) : listObj.selectItemByIndex(this.selectedItemIndex());
                else
                    this.model.enableMultiSelect ? listObj.uncheckItemsByIndex(this.selectedItemIndex()) : listObj.deselectItem();
                this._updateSelectedItem(item, text, item.attr("data-value"), option ? true : false);
            }
            else if (this.model.fields.checkBy) {
                index = this._targetWrapper.find(".e-m-lv-checked").length;
                for (i = 0; i < index; i++) {
                    text = $(this._targetWrapper.find(".e-m-lv-checked")[i]).find(".e-m-lv-content").text();
                    this._updateSelectedItem([], text, null, option ? true : false);
                }
            }
        },
        _updateSelectedItem: function (item, text, keyVal, isChecked) {
            var val = keyVal ? keyVal : text;
            if (this.model.enableMultiSelect) {
                if (!isChecked) {
                    this._text = jQuery.grep(this._text, function (value) { return value != text; });
                    this._hiddenVal = jQuery.grep(this._hiddenVal, function (value) { return value != val; });
                }
                else if ($.inArray(text, this._text) == -1) {
                    this._text.push(text);
                    this._hiddenVal.push(val);
                }
                this.model.select && this._trigger("select", this._getArgs(item, text, keyVal, isChecked));
                this.element.val(this._setDelimiterChar(this._text));
                this._hiddenElement.attr("value", this._hiddenVal.toString());
                this.model.change && this._trigger("change", this._getArgs(item, text, keyVal, isChecked));
            }
            else {
                this._hiddenVal = [];
                this._hiddenVal.push(val);
                this.model.select && this._trigger("select", this._getArgs(item, text, keyVal, isChecked));
                if (this.element.val() != text) {
                    this.selectedItemIndex(this._targetEle.data("ejmListView").getIndexByText(text));
                    this.element.val(text);
                    this._hiddenElement.attr("value", val);
                    this.model.change && this._trigger("change", this._getArgs(item, text, keyVal, isChecked));
                }
                this.hidePopup();
            }
        },
        _getArgs: function (item, text, keyVal, isChecked) {
            return data = { selectedItem: item, selectedText: text, selectedValue: keyVal, isChecked: isChecked };
        },
        _setDelimiterChar: function (textObj) {
            var text = textObj.toString();
            return text.replace(/,/g, this.model.delimiterChar + " ");
        },
        showPopup: function () {
            this._targetWrapper.show();
        },
        hidePopup: function () {
            this._targetWrapper.hide();
        },
        getValue: function () {
            return this.element.val();
        },
        selectItemByIndex: function (index) {
            this.model.selectedItemIndex = index;
            this._setSelectedItemIndex(this.selectedItemIndex(), true);
        },
        selectItemByIndices: function (indices) {
            if (this.model.enableMultiSelect) {
                var proxy = this;
                $(indices).each(function (i, e) {
                    proxy.model.selectedItemIndex = e;
                    proxy._setSelectedItemIndex(proxy.selectedItemIndex(), true);
                });
            }
        },
        unselectItemByIndex: function (index) {
            this.model.selectedItemIndex = index;
            this._setSelectedItemIndex(this.selectedItemIndex(), false);
        },
        unselectItemByIndices: function (indices) {
            if (this.model.enableMultiSelect) {
                var proxy = this;
                $(indices).each(function (i, e) {
                    proxy.model.selectedItemIndex = e;
                    proxy._setSelectedItemIndex(proxy.selectedItemIndex(), false);
                });
            }
        },
        getSelectedItemValue: function () {
            return this.model.enableMultiSelect ? this._hiddenVal : this._hiddenVal[0];
        },
        _setModel: function (options) {
            var refresh;
            for (var option in options) {
                switch (option) {
                    case "renderMode": {
                        this._targetWrapper.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
                        this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
                        this._targetEle.ejmListView({ renderMode: this.model.renderMode });
                        this._targetWrapper.ejmScrollPanel({ renderMode: this.model.renderMode });
                        break;
                    }
                    case "delimiterChar": this.model.enableMultiSelect && this._setDelimiterChar(this._text); break;
                    case "enabled": this._controlState(this.model.enabled); break;
                    case "watermarkText": this._setWatermarkText(this.model.watermarkText); break;
                    case "selectedItemIndex": this._setSelectedItemIndex(this.selectedItemIndex(), true); break;
                    case "locale": this._setLocale(); break;
                    default: refresh = true; break;
                }
            }
            refresh && this._refresh();
            refresh = false;
        },
        _setLocale: function () {
            this._getLocalizedLabels();
            this.model.watermarkText = this._localizedLabels["watermarkText"];
            this._setWatermarkText(this.model.watermarkText);
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass(this._rootCSS);            
			this._initialization();
            this._renderControl();
            this._wireEvents();
        },
        _clearElement: function () {
            this.element.removeAttr("class").removeAttr("style");
            this.element.empty().html(this._orgEle.html());
			this._targetEle.remove();
            this.element.unwrap();
            this._eleWarpper.remove();
            this._hiddenElement.remove();
            this._dropDownIcon.remove();
            this._targetWrapper.remove();
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
    });
    ej.mobile.DropDownList.Locale = ej.mobile.DropDownList.Locale || {};
    ej.mobile.DropDownList.Locale["default"] = ej.mobile.DropDownList.Locale["en-US"] = {
        watermarkText: null
    };
})(jQuery, Syncfusion);


;
/**
* @fileOverview Plugin to style the Html Editor elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget({ "ejmTextBox": "ej.mobile.TextBox", "ejmTextArea": "ej.mobile.TextArea", "ejmPassword": "ej.mobile.Password", "ejmNumeric": "ej.mobile.Numeric", "ejmMaskEdit": "ej.mobile.MaskEdit" }, {
        _setFirst: true,
        _rootCSS: "e-m-editor",
        _requiresID: true,
        validTags: ["input", "textarea"],

        defaults: {
            renderMode: "auto",
            cssClass: '',
            enablePersistence: false,
            borderStyle: "box",
            watermarkText: "",
            value: '',
            readOnly: false,
            type: 'text',
            enabled: true,
            maxLength: '',
            autoFocus: false,
            height: '',
            width: '',
            step: 1,
            minValue: -(Number.MAX_VALUE),
            maxValue: Number.MAX_VALUE,
            format: '',
            locale: 'en-US',
            touchStart: null,
            touchEnd: null,
            keyUp: null,
            keyDown: null
        },

        dataTypes: {
            renderMode: "enum",
            enablePersistence: 'boolean',
            borderStyle: 'enum',
            readOnly: 'boolean',
            enabled: 'boolean',
            autoFocus: 'boolean',
            step: 'number',
            minValue: 'number',
            maxValue: 'number',
            locale: "string"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),

        _init: function () {
            this._cloneElement = this.element.clone();
            this.culture = ej.preferredCulture(this.model.locale);
            this._getLocalizedLabels();
            this.model.watermarkText = !ej.isNullOrUndefined(this.model.watermarkText) && this.model.watermarkText != "" ? this.model.watermarkText : this._localizedLabels["watermarkText"];
            this._renderControl();
            this._wireEvents();
        },
        _getControlName: function () {
            return this.pluginName.replace("ejm", "");
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
        },
        _renderControl: function () {
            ej.setRenderMode(this);
            var obj = this.model;
            this.model.type = this.element.attr('type') ? this.element.attr('type') : obj.type;
            if ((this.element.attr("name") == "" || ej.isNullOrUndefined(this.element.attr("name"))))
                this.element.attr("name", this._id);
            this._noClearBtn = ['MaskEdit', 'Numeric', 'Password', 'TextArea'];
            this["_set" + this._getControlName() + "Properties"]();
        },

        _setTextBoxProperties: function () {
            this._setCommonWrapper(this.model.type);
            this._setCommonProperties();
        },

        _setPasswordProperties: function () {
            this._setCommonWrapper('password');
            this._setCommonProperties();
        },

        _setTextAreaProperties: function () {
            var obj = this.model;
            this.element.addClass("e-m-" + obj.renderMode + " e-m-editor " + obj.cssClass + " e-m-input-wrapper e-m-textbox");
            this._setCommonProperties();
            if (obj.height) this._setHeight(obj.height);
        },

        _setNumericProperties: function () {
            this._setCommonWrapper('number');
            this._numbersRegex = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '.', 'Delete', 'Backspace', 'ArrowRight', 'Tab', 'ArrowLeft', 'Home', 'End', 'e'];
            this._keyPressCalled = false;
            this._setCommonProperties();
            this.element.css('padding-right', 64);
        },

        _setMaskEditProperties: function () {
            var obj = this.model;
            this.element.addClass("e-m-" + obj.renderMode + " e-m-input-wrapper e-m-mask " + obj.cssClass);
            this._alphaNumeric = /^[a-zA-Z0-9]*$/;
            this._alphabetic = /^[a-zA-Z]/;
            this._numeric = /^[0-9]/;
            this._allowedKeys = ['ArrowRight', 'Tab', 'ArrowLeft', 'Shift', 'Enter', 'Home', 'End', 'Ctrl'];
            this._restrictedKeys = ['Backspace', 'ArrowUp', 'ArrowDown', 'Delete', 'VolumeMute', 'VolumeUp', 'VolumeDown', 'PageDown', 'PageUp', 'F1', 'F2', 'F3', 'F4', 'F5', 'F6', 'F7', 'F8', 'F9', 'F10', 'F11', 'F12'];
            this._islastindex = false;
            this._keyDownCalled = false;
            this._setCommonProperties();
        },

        _setCommonWrapper: function (type) {
            this.model.type = type;
            this.element.attr('type', type);
            var obj = this.model;
            this._inputWrapper = ej.buildTag("div.e-m-" + obj.renderMode + " e-m-editor " + obj.cssClass + " e-m-input-wrapper");
            this.element.after(this._inputWrapper).addClass('e-m-textbox');
            this._inputWrapper.append(this.element);
            if (obj.type != 'number') {
                this._rightIcon = ej.buildTag("span");
                this._inputWrapper.append(this._rightIcon);
            }
            else {
                this._upBtn = ej.buildTag("span.e-m-icon-up up-btn");
                this._downBtn = ej.buildTag("span.e-m-icon-down down-btn");
                this._inputWrapper.append(this._upBtn).append(this._downBtn);
            }
        },

        _setCssClass: function () {
            this._inputWrapper.addClass(this.model.cssClass);
        },

        _setCommonProperties: function () {
            var obj = this.model;
            this._setBorderStyle();
            if (obj.width != '') this._setWidth(obj.width);
            if (obj.type == 'number') this._setValue(this.value() == '' ? this._validateNumericInput(0) : this._validateNumericInput(this.value()));
            else {
                var attr = $(this.element).attr('value');
                if (typeof attr !== typeof undefined) this.value(ej.parseFloat(attr));
                if (this.value()) this._setValue(this.value());
            }
            if (obj.watermarkText != '') this._setWatermarkText(obj.watermarkText);
            if (obj.maxLength != '') this._setMaxLength(obj.maxLength);
            if (obj.autoFocus) this._setAutoFocus(true);
            if (obj.readOnly) this._setReadOnly(true);
            if (!obj.enabled) this._setEnabled(false);
        },

        _setMinValue: function (value) {
            if (!this.element.prop('disabled') && this.model.type == 'number') {
                this.element.attr('min', value);
                if (value > this.element.val())
                    this.element.val(value);
            }
        },

        _setMaxValue: function (value) {
            if (!this.element.prop('disabled') && this.model.type == 'number') {
                this.element.attr('max', value);
                if (value < this.element.val())
                    this.element.val(value);
            }
        },

        _setStep: function (step) {
            if (this.model.type == 'number' && !this.element.prop('disabled') && step && !isNaN(step)) this.element.attr('step', step);
        },

        _setHeight: function (height) {
            if (this._getControlName() == 'TextArea' && !this.element.prop('disabled') && height) this.element.height(height);
        },

        _setReadOnly: function (status) {
            if (!this.element.prop('disabled')) this.element.attr('readonly', status);
            if (this.model.type == 'number') {
                this._upBtn[(!status ? "removeClass" : "addClass")]("e-m-disabled");
                this._downBtn[(!status ? "removeClass" : "addClass")]("e-m-disabled");
            }
            else if (this._getControlName() != 'MaskEdit' && this._getControlName() != 'TextArea') this._rightIcon[(status ? "removeClass" : "addClass")]("e-m-disabled");
        },

        _setAutoFocus: function (focusstate) {
            if (!this.element.prop('disabled') && focusstate) {
                this.element.focus();
                this.element.prop('autofocus', 'autofocus');
                if (this._getControlName() != 'MaskEdit' && this._getControlName() != 'TextArea') this.element.parent().addClass('focus');
            }
        },

        _setMaxLength: function (maxlength) {
            if (!this.element.prop('disabled') && maxlength) this.element.attr('maxlength', maxlength);
        },

        _setEnabled: function (status) {
            this.element.attr('disabled', !status);
            if (this._getControlName() == 'TextArea' || this._getControlName() == 'MaskEdit') this.element[(status ? "removeClass" : "addClass")]("e-m-state-disabled");
            else this.element.parent()[(status ? "removeClass" : "addClass")]("e-m-state-disabled");
        },

        _setWidth: function (width) {
            if (!this.element.prop('disabled')) {
                this.element.width(width);
                if (this._getControlName() != 'MaskEdit' || this._getControlName() != 'TextArea') this.element.parent().width(width);
            }
        },

        _setWatermarkText: function (watermarktext) {
            if (!this.element.prop('disabled')) this.element.attr("placeholder", this.model.watermarkText);
        },

        _setValue: function (value) {
            if (!this.element.prop('disabled')) {
                this.element.val(value);
                this.value(value);
            }
        },

        _setBorderStyle: function () {
            if (!this.element.prop('disabled')) {
                if (this._getControlName() != 'MaskEdit' && this._getControlName() != 'TextArea')
                    this.element.parent().removeClass("e-m-line e-m-box e-m-none").addClass("e-m-" + this.model.borderStyle);
                else
                    this.element.removeClass("e-m-line e-m-box e-m-none").addClass("e-m-" + this.model.borderStyle);
            }
        },

        _setRenderMode: function (mode) {
            if (!this.element.prop('disabled')) this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + mode);
        },

        _setLocale: function () {
            this._getLocalizedLabels();
            this.model.watermarkText = this._localizedLabels["watermarkText"];
            this._setWatermarkText();
        },

        _createDelegate: function () {
            this._keyDownProxy = $.proxy(this._keyDown, this);
            this._keyUpProxy = $.proxy(this._keyUp, this);
            this._touchStartProxy = $.proxy(this._touchStart, this);
            this._touchEndProxy = $.proxy(this._touchEnd, this);
            this._focus = $.proxy(this._focus, this);
            this._blur = $.proxy(this._blur, this);
            this._keyPressProxy = $.proxy(this._keyPress, this);
            this._cutProxy = $.proxy(this._cut, this);
            this._pasteProxy = $.proxy(this._paste, this);
            this._docClickProxy = $.proxy(this._docClick, this);
            this._showPasswordProxy = $.proxy(this._showPassword, this);
            this._hidePasswordProxy = $.proxy(this._hidePassword, this);
            this._clearInputProxy = $.proxy(this._clearInput, this);
            this._stepProxy = $.proxy(this._step, this);
            this._stepCloseProxy = $.proxy(this._stepClose, this);
        },

        _wireEvents: function (remove) {
            var obj = this.model;
            var element = this.element;
            var controlname = this._getControlName();
            this._createDelegate();
            if (controlname == 'MaskEdit' || controlname == 'Numeric')
                ej.listenTouchEvent(element, "keypress", this._keyPressProxy, remove);
            if (obj.type == 'number')
                ej.listenEvents([this._upBtn, this._downBtn, this._upBtn, this._downBtn, this._upBtn, this._downBtn, this._upBtn, this._downBtn], [ej.startEvent(), ej.startEvent(), ej.endEvent(), ej.endEvent(), ej.cancelEvent(), ej.cancelEvent(), 'mouseout', 'mouseout'], [this._stepProxy, this._stepProxy, this._stepCloseProxy, this._stepCloseProxy, this._stepCloseProxy, this._stepCloseProxy, this._stepCloseProxy, this._stepCloseProxy], remove);
            if (obj.type == 'password') {
                ej.listenEvents([this._rightIcon, this._rightIcon], [ej.startEvent(), ej.endEvent()], [this._showPasswordProxy, this._hidePasswordProxy], remove);
            }
            if (this._noClearBtn.indexOf(this._getControlName()) == -1) {
                ej.listenTouchEvent(this._rightIcon, "tap", this._clearInputProxy, remove);
            }
            ej.listenEvents([element, element, element, element, element, element, element, element, $(document)], [ej.startEvent(), ej.endEvent(), "keydown", "keyup", "focus", "blur", "cut", "paste", 'click'], [this._touchStartProxy, this._touchEndProxy, this._keyDownProxy, this._keyUpProxy, this._focus, this._blur, this._cutProxy, this._pasteProxy, this._docClickProxy], remove);
        },

        _docClick: function (e) {
            if (!$(e.target).hasClass('e-m-editor') && !$(e.target).hasClass('right-eye-btn e-m-icon-eye') && !$(e.target).hasClass('e-m-icon-clear')) {
                this._removecleariconclass();
            }
        },

        _cut: function (e) {
            if (this._getControlName() == 'MaskEdit' || this.model.type == 'number') e.preventDefault();
        },

        _paste: function (e) {
            if (this._getControlName() == 'MaskEdit' || this.model.type == 'number') e.preventDefault();
        },

        _blur: function (e) {
            if (this.model.type == 'password' || this.model.type == 'number' || this.model.type == 'text' || this.model.type == 'tel' || this.model.type == 'email')
                this.element.parent().removeClass('focus');
            if (this.model.type == 'number') this._setValue(this._validateNumericInput(this.element.val()));
            this._removecleariconclass();
        },

        _removecleariconclass: function(){
            if (this.value()) {
                var righticon = this._getControlName() == 'Password' ? 'eye' : (this._noClearBtn.indexOf(this._getControlName()) == -1 ? 'clear' : '');
                if (righticon) this._rightIcon.removeClass('right-' + righticon + '-btn e-m-icon-' + righticon);
            }
        },

        _step: function (e) {
            if (this.model.enabled && this.model.type == 'number') {
                var self = this;
                var stepmode = $(e.target).hasClass('e-m-icon-down') ? 'down' : 'up';
                this._updateValue(stepmode);
                this._timeout = setInterval(function () { self._updateValue(stepmode); }, 200);
            }
        },

        _updateValue: function (updown) {
            var val = this.element.val() == '' ? this._validateNumericInput(0) : (updown == 'down' ? parseFloat(this.element.val()) - this.model.step : parseFloat(this.element.val()) + this.model.step);
            if (this.model.step.toString().indexOf('.') != -1) val = val.toFixed(this._trimfloat(this.model.step.toString()));
            this._setValue(this._validateNumericInput(val));
        },

        _stepClose: function (e) {
            clearTimeout(this._timeout);
        },

        _focus: function (e) {
            if (this._getControlName() != 'MaskEdit' && this._getControlName() != 'TextArea') this.element.parent().addClass('focus');
            if (this.value()) {
                var righticon = this._getControlName() == 'Password' ? 'eye' : (this._noClearBtn.indexOf(this._getControlName()) == -1 ? 'clear' : '');
                if (righticon) this._rightIcon.addClass('right-' + righticon + '-btn e-m-icon-' + righticon);
            }
        },

        _showPassword: function (e) {
            this.element.attr('type', 'text');
        },

        _hidePassword: function (e) {
            this.element.attr('type', 'password');
        },

        _clearInput: function (e) {
            if (this._rightIcon.hasClass('e-m-icon-clear')) {
                this._rightIcon.removeClass('right-clear-btn e-m-icon-clear');
                this._setValue('');
            }
        },

        _touchStart: function (e) {
            if (this.model.touchStart && !this.model.readOnly) this._trigger("touchStart", { value: this.element.val() });
        },

        _touchEnd: function (e) {
            var data = { value: this.element.val() };
            if (this.model.touchEnd && !this.model.readOnly) this._trigger("touchEnd", data);
        },

        _setCharAt: function (str, index, chr) {
            if (index > str.length - 1) return str;
            return str.substr(0, index) + chr + str.substr(index + 1);
        },

        _validateMaskInput: function (key) {
            if ((this._charType == 'alphabetic' || this._charType == 'numeric' || key == 'Backspace' || key == 'Delete') && this._charType != '') {
                if (this['_' + this._charType].test(key) || key == 'Delete' || key == 'Backspace') {
                    if (this._replace) {
                        var value = (key == 'Backspace' || key == 'Delete') ? '_' : (this._restrictedKeys.indexOf(key) == -1) ? key : '';
                        if (value == '')
                            return false;
                        this.value(this._setCharAt(this.value(), (key == 'Backspace') ? this._charIndex - 1 : this._charIndex, value));
                        this.element.val(this.value());
                        this._setCursor(this.element[0], (key == 'Backspace') ? this._charIndex - 1 : this._charIndex + 1);
                    }
                    else {
                        if (this._restrictedKeys.indexOf(key) == -1) {
                            this.value(this.value() + key);
                            this.element.val(this.value());
                        }
                    }
                }
                else if (!this['_' + this._charType].test(key)) {
                    this.element.val(this.value());
                    this._setCursor(this.element[0], this._charIndex);
                }
                else this._setCursor(this.element[0], this._charIndex);
            }
        },

        _getFormatCharType: function (index) {
            this._formatChar = this.model.format.toString().charAt(index);
            var formatCharType = !(this._alphaNumeric.test(this._formatChar)) ? 'symbol' : this._numeric.test(this._formatChar) ? 'numeric' : this._alphabetic.test(this._formatChar) ? 'alphabetic' : '';
            return formatCharType;
        },

        _setMaskSpecialChar: function (char, index) {
            this.value(this.value() + this._formatChar);
            this.element.val(this.value());
        },

        _setCursor: function (node, pos) {
            node = (typeof node == "string" || node instanceof String) ? document.getElementById(node) : node;
            if (!node) {
                return false;
            } else if (node.createTextRange) {
                var textRange = node.createTextRange();
                textRange.collapse(true);
                textRange.moveEnd('character', pos);
                textRange.moveStart('character', pos);
                textRange.select();
                return true;
            } else if (node.setSelectionRange) {
                node.setSelectionRange(pos, pos);
                return true;
            }
            return false;
        },

        _handleMaskValue: function (e, key) {
            if (this._allowedKeys.indexOf(key) == -1) {
                this._charIndex = this.element[0].selectionStart;
                if (this.element[0].selectionEnd - this._charIndex == 1 && this._charIndex == 0 && e.keyCode == 8)
                    this._charIndex = 1;
                if (this._charIndex == this.model.format.length)
                    this._islastindex = true;
                else
                    this._islastindex = false;

                if (this.value().charAt(this._charIndex) || key == 'Backspace')
                    this._replace = true;
                else
                    this._replace = false;
                this._charType = this._getFormatCharType((key == 'Backspace') ? this._charIndex - 1 : this._charIndex);
                if (!this._islastindex || key == 'Backspace') {
                    if (this._charType == 'symbol') {
                        var i = this._charIndex;
                        do {
                            if (!this._replace) {
                                this._setMaskSpecialChar(this.value() + this._formatChar, i);
                            }
                            else {
                                this._setCursor(this.element[0], (key == 'Backspace') ? i : i + 1);
                                this._charIndex = this.element[0].selectionStart;
                            }
                            (key == 'Backspace') ? i-- : i++;
                            this._charType = this._getFormatCharType(i);
                        } while (this._charType == 'symbol');
                    }
                    this._validateMaskInput(key);
                }
                e.preventDefault();
            }
        },

        _trimfloat: function (numstr) {
            return (numstr.split('.')[1] || []).length;
        },

        _validateNumericInput: function (value) {
            value = parseFloat(value);
            var minval = this.model.minValue;
            var maxval = this.model.maxValue;
            if (isNaN(value)) value = 0;
            return (value > minval && value < maxval) ? value : (value == minval || value < minval) ? minval : maxval;
        },

        _keyDown: function (e) {
            this._keyDownCalled = false;
            if (this._getControlName() == 'MaskEdit' && (e.keyCode == 8 || e.keyCode == 46)) {
                var key = e.keyCode == 8 ? 'Backspace' : 'Delete';
                this._handleMaskValue(e, key);
                this._keyDownCalled = true;
            }
            if (this.model.keyDown && !this.model.readOnly)
                this._trigger("keyDown", { value: this.element.val(), keyCode: e.keyCode });
        },

        _keyPress: function (e) {
            var key = e.key ? e.key : String.fromCharCode(e.keyCode || e.charCode);
            if (this._getControlName() == 'MaskEdit') {
                if (key == 'BackSpace' || key == 'Delete') {
                    if (!this._keyDownCalled) this._handleMaskValue(e, key);
                }
                else
                    this._handleMaskValue(e, key);
            }
            else if (this.model.type == 'number' && this._numbersRegex.indexOf(key) == -1) e.preventDefault();
        },

        _keyUp: function (e) {
            var obj = this.model;
            if (!obj.readOnly) {
                if (this._getControlName() == 'MaskEdit' && (e.keyCode == 229 || e.keyCode == 0)) {
                    var key = String.fromCharCode(e.which);
                    if (e.keyCode == 229 || e.keyCode == 0) {
                        key = String.fromCharCode(this.element.val().charCodeAt(this.element[0].selectionStart - 1));
                        this.element[0].selectionStart = this.element[0].selectionStart - 1;
                    }
                    this._handleMaskValue(e, key);
                    this.element.val(this.value());
                    e.preventDefault();
                }
                else if (this._getControlName() != 'MaskEdit' && obj.type != 'number' && this._getControlName() != 'TextArea') {
                    this.value(this.element.val());
                    var righticon = this._getControlName() == 'Password' ? 'eye' : (this._noClearBtn.indexOf(this._getControlName()) == -1 ? 'clear' : '');
                    if (righticon) this._rightIcon[this.value() ? "addClass" : "removeClass"]('right-' + righticon + '-btn e-m-icon-' + righticon);
                    if (this.value()) this.element.css('padding-right', 32);
                }
                else {
                    this.value(this.element.val());
                    e.preventDefault();
                }
            }
            else {
                e.preventDefault();
            }
            if (this.model.keyUp && !this.model.readOnly)
                this._trigger("keyUp", { value: this.element.val(), keyCode: e.keyCode });
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        _clearElement: function () {
            if (!ej.isNullOrUndefined(this._inputWrapper)) {
                this._inputWrapper.before(this._cloneElement);
                this._inputWrapper.remove();
                this._inputWrapper = null;
            }
            else
                this.element.replaceWith(this._cloneElement);
            this.element = this._cloneElement;
            this.element.removeAttr("class");
        },

        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },

        _refresh: function () {
            this._destroy();
            this.element.addClass("e-m-editor");
            this._init();
        }
    });

    ej.mobile.TextBox.BorderStyle = {
        Box: "box",
        Line: "line",
        None: "none"
    };

    ej.mobile.TextBox.Type = {
        Number: "number",
        Text: "text",
        Password: "password",
        Tel: "tel",
        Email: "email"
    };
    var LocaleObj = {
        watermarkText: ""
    };
    ej.mobile.TextBox.Locale = ej.mobile.TextBox.Locale || {};
    ej.mobile.TextArea.Locale = ej.mobile.TextArea.Locale || {};
    ej.mobile.Password.Locale = ej.mobile.Password.Locale || {};
    ej.mobile.Numeric.Locale = ej.mobile.Numeric.Locale || {};
    ej.mobile.MaskEdit.Locale = ej.mobile.MaskEdit.Locale || {};

    ej.mobile.TextBox.Locale["default"] = ej.mobile.TextBox.Locale["en-US"] =
    ej.mobile.TextArea.Locale["default"] = ej.mobile.TextArea.Locale["en-US"] =
    ej.mobile.Password.Locale["default"] = ej.mobile.Password.Locale["en-US"] =
    ej.mobile.Numeric.Locale["default"] = ej.mobile.Numeric.Locale["en-US"] =
    ej.mobile.MaskEdit.Locale["default"] = ej.mobile.MaskEdit.Locale["en-US"] = LocaleObj;
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html groupbutton elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    // ejmGroupButton is the plugin name 
    // "ej.mobile.GroupButton" is "namespace.className" will hold functions and properties

    ej.widget("ejmGroupButton", "ej.mobile.GroupButton", {
        _setFirst: true,
        _rootCSS: "e-m-grpbtn",
        _tags: [{
            tag: "items",
            attr: ["text", "type", "name", "imageClass", "imageUrl"]
        }],
        defaults: {
            selectedItemIndex: 0,
            renderMode: "auto",
            enablePersistence: false,
            cssClass: "",
            items: [],
            touchStart: null,
            touchEnd: null,
            select: null
        },
        dataTypes: {
            renderMode: "enum",
            enablePersistence: "boolean"
        },
        observables: ["selectedItemIndex"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),

        _init: function () {
            this._orgEle = this.element.clone();
            this._renderControl();
            this._createDelegates();
            this._wireEvents();
        },

        _renderControl: function () {
            ej.setRenderMode(this);
            var element = this.element;
            element.addClass("e-m-grpbtn e-m-" + this.model.renderMode + " " + this.model.cssClass);
            if (typeof this.model.items === "string") { this.model.items = eval(this.model.items) };
            var isAngular = this.model.items.length > 0 ? true : false;
            for (var i = 0, btnelement = element.children() ; i < (isAngular ? this.model.items.length : btnelement.length) ; i++) {
                if (isAngular)
                    var label = (!ej.isNullOrUndefined(this.model.items[i].type) || !ej.isNullOrUndefined(this.model.items[i].type)) ? ej.buildTag('label', ('<input ' + (ej.isNullOrUndefined(this.model.items[i].type) ? "" : 'type="' + this.model.items[i].type + '"') + (ej.isNullOrUndefined(this.model.items[i].name) ? "" : 'name="' + this.model.items[i].name + '"') + ' />' + this.model.items[i].text)) : ej.buildTag("button", this.model.items[i].text).appendTo(element);
                else
                    this.model.items.push(this._insertItem(btnelement[i]));
            }
            var proxy = this;
            $.each(element.children(), function (index, val) {
                var $img = $(this);
                imageClass = proxy.model.items[index].imageClass;
                imagePath = proxy.model.items[index].imageUrl;
                if (imageClass || imagePath) {
                    var image = ej.buildTag("div.grpimage " + imageClass).appendTo($img)
                    if (imagePath) {
                        image.css({ "background-image": "url('" + imagePath + "')" });
                        $img.text().trim() == "" && $img.addClass("e-m-imageonly");
                    }
                }
                $img.addClass("e-m-btn");
            });
            this._selectItem(this.selectedItemIndex());
        },
        _insertItem: function (element) {
            var item = {};
            for (var j = 0, attrLength = this._tags[0].attr.length; j < attrLength; j++) {
                var attr = this._tags[0].attr[j];
                item[attr] = ej.getAttrVal(element, 'data-ej-' + attr.toLowerCase(), undefined);
            }
            return item;
        },
        _createDelegates: function () {
            this._touchStartHandler = $.proxy(this._touchStartHandler, this);
            this._touchEndHandler = $.proxy(this._touchEndHandler, this);
            this._touchMoveHandler = $.proxy(this._onTouchMove, this);
        },

        _wireEvents: function (remove) {
            var selector = this.element.find(".e-m-btn");
            ej.listenEvents([selector], [ej.startEvent()], [this._touchStartHandler], remove);
        },

        _touchStartHandler: function (evt) {
            if (!$(this.element.children()[this.model.selectedItemIndex]).children().is(":checked")) $(this.element.children()[this.model.selectedItemIndex]).removeClass("e-m-active");
            this._prevItem = this.model.selectedItemIndex;
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            this._clickX = point.clientX;
            this._isMouseMove = false;
            this._target = evt.target;
            this.currentText = $(evt.target).text();
            this.data = { text: this.currentText.trim() };
            ej.listenEvents([this._target, this._target], [ej.endEvent(), ej.moveEvent()], [this._touchEndHandler, this._touchMoveHandler], false);
            if (this.model.touchStart) this._trigger("touchStart", this.data);
        },
        _onTouchMove: function (evt) {
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            if (this._clickX != point.clientX) this._isMouseMove = true;
        },

        _touchEndHandler: function (evt) {
            ej.listenEvents([$(evt.currentTarget), $(evt.currentTarget)], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler], true);
            if (evt.target === this._target && !this._isMouseMove) {
                var element = this.element;
                $(evt.target).addClass('e-m-active');
                var parent = $(evt.currentTarget);
                this.model.selectedItemIndex = parent.index();
                if ((parent.find('input').length == 0) || (parent.find("input[type='groupbutton']").length == 0)) {
                    if (element.children().find("input").attr("type") != "checkbox") element.find('.e-m-active').removeClass('e-m-active').find("input[type='radio']").attr("checked", false);
                    parent.addClass('e-m-active').find("input[type='radio']").prop("checked", true);
                }
                else
                    parent.find("input").is(":checked") ? parent.removeClass('e-m-active') : parent.addClass('e-m-active');

                if (element.children().find("input").attr("type") == "checkbox") {
                    if ($(element.children()[this._prevItem]).children().is(":checked")) $(element.children()[this._prevItem]).addClass("e-m-active");
                    if ($(element.children()[this.model.selectedItemIndex]).children().is(":checked")) $(element.children()[this.model.selectedItemIndex]).removeClass("e-m-active");

                }
                this.selectedItemIndex($(evt.currentTarget).index());
                if (this.model.touchEnd) this._trigger("touchEnd", this.data);
            }
            this.data = { text: this.currentText.trim() };
            if (this.model.select) this._trigger("select", this.data);
        },

        _selectItem: function (index) {
            var element = this.element;
            if (index >= 0) {
                this.selectedItemIndex(index);
                var select = $(element.find("input")[this.selectedItemIndex()]);
                if (element.find(".e-m-active").children().attr("type") == "checkbox") {
                    if ($(element.children().find("input")[index]).is(":checked")) {
                        $(element.children()[index]).removeClass("e-m-active");
                        select.prop("checked", false);
                    }
                    else {
                        $(element.children()[index]).addClass("e-m-active");
                        select.prop("checked", true);
                    }
                } else {
                    element.find('.e-m-active').removeClass("e-m-active");
                    $(element.find('.e-m-btn')[this.selectedItemIndex()]).addClass("e-m-active");
                    select.prop("checked", true);
                }
                this.currentText = $(element.find('.e-m-btn')[this.selectedItemIndex()]).text();
                this.data = { text: this.currentText.trim() };
                if (this.model.select) this._trigger("select", this.data);
            }
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                var tempProp = (typeof (options[prop]) == "function") ? this[prop]() : options[prop];
                if (this[setModel]) this[setModel](tempProp);
                else
                    refresh = true;
            }
            if (refresh) this._refresh();
        },

        _setRenderMode: function () {
            this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode + "");
        },

        _setSelectedItemIndex: function (index) {
            this._selectItem(index);
        },

        _refresh: function () {
            this._destroy();
            this.element.addClass("ejm-groupbutton");
            this._renderControl();
            this._wireEvents(false);
        },

        _clearElement: function () {
            this.element.removeAttr("class")
            this.element.html(this._orgEle.html());
        },

        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        }

    });

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {

    ej.widget({ "ejmRadioButton": "ej.mobile.RadioButton", "ejmCheckBox": "ej.mobile.CheckBox" }, {
        _setFirst: true,
        _rootCSS: "e-m-editor",
        validTags: ["input"],

        defaults: {
            renderMode: "auto",
            cssClass: '',
            enablePersistence: false,
            enabled: true,
            checked: false,
            touchStart: null,
            enableRippleEffect: ej.isAndroid() ? true : false,
            touchEnd: null,
            beforeChange: null,
            change: null
        },

        dataTypes: {
            renderMode: "enum",
            enablePersistence: 'boolean',
            enabled: 'boolean',
            enableRippleEffect: "boolean",
            checked: 'boolean'
        },

        _init: function () {
            this._cloneElement = this.element.clone();
            this._renderControl();
            this._wireEvents();
        },

        _getControlName: function () {
            return this.pluginName.replace("ejm", "");
        },

        _renderControl: function () {
            ej.setRenderMode(this);
            var obj = this.model;
            this._radioCheckCoreProperties(this._getControlName().toLowerCase());
        },


        _radioCheckCoreProperties: function (type) {
            this.model.type = (type == "checkbox" ? "checkbox" : "radio");
            if (!this.element.attr('type'))
                this.element.attr('type', this.model.type);
            this.element.css("display", "none");
            this._wrapper = this.element.parent();
            this._elementName = this.element.attr("name")
            this._inputWrapper = ej.buildTag("div.e-m-" + this.model.renderMode).addClass("e-m-input-" + type + " " + this.model.cssClass + " " + (this._elementName ? this._elementName : ""));
            this.element.addClass(this.model.cssClass);
            this._inputWrapper.insertBefore(this.element);
            this._setEnableRippleEffect();
            this.element.appendTo(this._inputWrapper);
            this._label = $("label[for='" + this._id + "']");
            this._setCommonProperties();
        },
        _setEnableRippleEffect: function () {
            this._inputWrapper[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
        },
        _setCommonProperties: function () {
            var obj = this.model;
            if (obj.checked) this._setChecked(true);
            if (!obj.enabled) this._setEnabled(false);
        },

        _setChecked: function (state) {
            if (!this.element.prop('disabled')) {
                this.element.prop('checked', state);
                this._inputWrapper[(state ? "addClass" : "removeClass")]("e-m-state-active");
            }
        },

        _setEnabled: function (status) {
            this.element.attr('disabled', !status);
            this._inputWrapper[(status ? "removeClass" : "addClass")]("e-m-state-disabled");
        },

        _setRenderMode: function (mode) {
            if (!this.element.prop('disabled'))
                this._inputWrapper.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
        },

        _createDelegate: function () {
            this._touchStartProxy = $.proxy(this._touchStart, this);
            this._touchEndProxy = $.proxy(this._touchEnd, this);
        },

        _wireEvents: function (remove) {
            var obj = this.model;
            var element = this.element;
            var controlname = this._getControlName();
            this._createDelegate();
            ej.listenEvents([this._inputWrapper, this._inputWrapper, this._label, this._label], [ej.startEvent(), ej.endEvent(), ej.startEvent(), ej.endEvent()], [this._touchStartProxy, this._touchEndProxy, this._touchStartProxy, this._touchEndProxy], remove);
        },

        _touchStart: function (e) {
            if (this.model.enabled) {
                this._inputWrapper.addClass("e-m-state-glow")
                var data = { isChecked: this.model.checked, value: this.element.val(), controlType: this.model.type, element: this.element, events: e }
                if (this.model.touchStart)
                    this._trigger("touchStart", data);
                if (this.model.beforeChange)
                    this._trigger("beforeChange", data);
            }
        },

        _touchEnd: function (e) {
            if (this.model.enabled) {
                if (this.model.type == "checkbox") {
                    if (this.element.attr("checked") && this.element.attr("checked").length) {
                        this.element.attr("checked", false);
                        this.model.checked = false;
                        this._inputWrapper.removeClass("e-m-state-active");
                    }
                    else {
                        this.element.attr("checked", true);
                        this.model.checked = true;
                        this._inputWrapper.addClass("e-m-state-active");
                    }
                }
                if (this.model.type == "radio") {
                    var radioButton = $(".e-m-input-radiobutton." + this._elementName);
                    $.each(radioButton, function (i, ele) {
                        $(ele).removeClass("e-m-state-active");
                        $(ele).children().attr("checked", false);
                    })
                    this.element.attr("checked", true);
                    this.model.checked = true;
                    this.element.parent().addClass("e-m-state-active");
                }
                var data = { isChecked: this.model.checked, value: this.element.val(), controlType: this.model.type, element: this.element, events: e }
                this._inputWrapper.removeClass("e-m-state-glow");
                if (this.model.touchEnd)
                    this._trigger("touchEnd", data);
                if (this.model.change)
                    this._trigger("change", data);
            }
        },

        enable: function () {
            this.model.enabled = true;
            this._inputWrapper.removeClass("e-m-state-disabled");
        },

        disable: function () {
            this.model.enabled = false;
            this._inputWrapper.addClass("e-m-state-disabled");
        },

        isChecked: function () {
            return this.model.checked;
        },

        _setModel: function (options) {
            for (var prop in options) {
                this['_set' + prop.charAt(0).toUpperCase() + prop.slice(1)](options[prop]);
            }
        },
        _clearElement: function () {
            this._cloneElement.insertBefore(this.element.parent());
            this.element.parent().remove();
            this.element.remove();
            this._wireEvents(true);
            return this;
        },

        _destroy: function () {
            this._clearElement();
        },

        _refresh: function () {
            this._clearElement()._init();
            this._wireEvents();
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ListView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmListView", "ej.mobile.ListView", {
        _setFirst: true,
        _rootCSS: "e-m-lv",
        validTags: ["ul"],
        _tags: [{
            tag: "items",
            attr: ["text", "value", "href", "imageUrl", "enabled", "badge.value", "badge.maxValue", "allowSelection"],
            content: "template"
        }],
        defaults: {
            renderMode: "auto",
            cssClass: "",
            dataSource: [],
            templateId: "",
            enabled: true,
            enableChecklist: false,
            persistSelection: false,
            enablePersistence: false,
            enableRippleEffect: ej.isAndroid() ? true : false,
            allowSelection: true,
            selectedIndex: null,
            checkedIndices: [],
            query: null,
            deleteMode: "none",
            fields: {
                text: null,
                value: null,
                image: null,
                groupBy: null,
                checkBy: null,
                enabled: null,
                href: null,
                allowSelection: null,
                badge: {
                    value: null,
                    maxValue: 100
                }
            },
            windows: {
                preventSkew: false
            },
            itemsCount: 0,
            items: [],
            touchStart: null,
            select: null,
            touchEnd: null,
            beforeDelete: null,
            afterDelete: null,
            actionSuccess: null,
            actionFailure: null,
            actionComplete: null
        },
        dataTypes: {
            renderMode: "enum",
            dataSource: "data",
            items: "data",
            enabled: "boolean",
            enableRippleEffect: "boolean",
            selectedIndex: ["string", "number"],
            checkedIndices: "array",
            enableChecklist: "boolean",
            persistSelection: "boolean",
            allowSelection: "boolean",
            windows: { preventSkew: "boolean" },
            templateId: "string",
            itemsCount: "number"
        },
        observables: ["selectedIndex"],
        selectedIndex: ej.util.valueFunction("selectedIndex"),
        _init: function () {
            ej.setRenderMode(this);
            this._dataObj = this.model.dataSource;
            this._index = 0;
            this._initialize();
        },
        _initialize: function () {
            this._listItems = [];
            this._orgEle = this.element.clone();
            if (typeof this.model.dataSource === "string") this.model.dataSource = eval(this.model.dataSource);
            Object.keys(this.model.dataSource).length ? this._dataProcess(this.model.dataSource, null) : this._render();
            this._browser = ej.browser().toLowerCase();
            this._doc = $(document);
            this._relDis = 0;
            this._move = false;
        },
        _dataProcess: function (data, action) {
            var proxy = this, query;
            query = action == "append" ? this.model.query ? this.model.query : "ej.Query()" + ".range(" + this._index + "," + (this._index + this.model.itemsCount) + ")"
                    : (this.model.query && this.model.itemsCount ? this.model.query + ".take(" + this.model.itemsCount + ")" :
                    (!this.model.query && this.model.itemsCount ? "ej.Query().take(" + this.model.itemsCount + ")" : (this.model.query ? this.model.query : "ej.Query()")));
            if (data instanceof ej.DataManager)
                data.executeQuery(eval(query)).done(function (e) {
                    action == "refresh" && proxy.element.children().remove();
                    proxy.model.dataSource = (action == "append") ? proxy.model.dataSource.concat(e.result) : e.result;
                    proxy._dataSource = e.result;
                    proxy._render();
                    proxy._trigger("actionSuccess", e);
                    if (proxy.model.itemsCount)
                        proxy._index += proxy.model.itemsCount;
                }).fail(function (e) {
                    proxy._trigger("actionFailure", e);
                }).always(function (e) {
                    proxy._trigger("actionComplete", e);
                });
            else {
                this._dataSource = ej.DataManager(data).executeLocal(eval(query));
                action == "refresh" && this.element.children().remove();
                this.model.dataSource = (action == "append") ? this.model.dataSource.concat(this._dataSource) : this._dataSource;
                this._render();
                if (this.model.itemsCount)
                    this._index += this.model.itemsCount;
            }
        },
        _render: function () {
            this._renderControl();
            this._setValues();
            this._wireEvents();
        },
        _setValues: function () {
            if (!this.model.enabled) this.disable();
            else {
                this.model.selectedIndex != null && this.selectItemByIndex(this.selectedIndex());
                this.model.checkedIndices.length && this.checkItemsByIndices(this.model.checkedIndices);
            }
        },
        _renderItems: function (proxy) {
            if (this.model.items.length < 1) {
                var elements = this.element.find("li");
                for (i = 0; i < elements.length; i++) {
                    this.model.items.push({
                        text: ej.getAttrVal(elements[i], 'data-ej-text', null),
                        value: ej.getAttrVal(elements[i], 'data-ej-value', null),
                        href: ej.getAttrVal(elements[i], 'data-ej-href', null),
                        imageUrl: ej.getAttrVal(elements[i], 'data-ej-imageurl', null),
                        enabled: ej.getAttrVal(elements[i], 'data-ej-enabled', true),
                        allowSelection: ej.getAttrVal(elements[i], 'data-ej-allowselection', true),
                        badge: {
                            value: ej.getAttrVal(elements[i], 'data-ej-badge-value', null),
                            maxValue: ej.getAttrVal(elements[i], 'data-ej-badge-maxvalue', null)
                        }
                    });
                }
            }
            else {
                for (i = 0; i < this.model.items.length; i++)
                    if (this.model.items[i].template)
                        this.element.append(ej.buildTag('li', this.model.items[i].template));
            }
        },
        _renderControl: function () {
            this.element.addClass('e-m-' + this.model.renderMode + ' e-m-lv' + ' e-m-user-select' + (!this.model.allowSelection ? ' e-m-lv-preventselection' : '') + " " + this.model.cssClass);
            if (this.model.templateId.length)
                this._renderTemplateList();
            else if (this._dataObj.length == 0) {
                this._renderItems();
                this._renderList();
            }
            else if (this.model.dataSource.length)
                this.model.fields.groupBy ? this._dataSourceGroupList() : this._dataSourceList(this._dataSource);
        },
        _wireEvents: function (remove) {
            this._touchStart = $.proxy(this._touchStartHandler, this);
            this._touchEnd = $.proxy(this._touchEndHandler, this);
            this._touchMove = $.proxy(this._touchMoveHandler, this);
            ej.listenEvents([this.element], [ej.startEvent()], [this._touchStart], remove);
        },
        _renderList: function () {
            this._listItems = this.element.find('li').addClass("e-m-lv-item");
            this._setEnableRippleEffect();
            this._listHeaders = this.element.find(">span").addClass("e-m-lv-group");
            for (var i = 0; i < this._listItems.length; i++) {
                var currentItem = $(this._listItems[i]).attr({ "index": i, "data-value": this.model.items[i].value, "data-allowselection": this.model.items[i].allowSelection }).addClass((this.model.items[i].enabled == "false") ? "e-m-state-disabled" : "");
                var ele = ej.buildTag("a.e-m-lv-content", this.model.items[i].text ? this.model.items[i].text : currentItem.html(), {}, { 'data-ej-appajax': false, "href": this.model.items[i].href });
                currentItem.empty().append(ele);
                this._slideDeleteButton(currentItem, ele);
                if (this.model.items[i].imageUrl) ele.css("background-image", "url(" + this.model.items[i].imageUrl + ")").addClass("e-m-lv-image");
                this._setBadge(currentItem);
            }
        },
        _setEnableRippleEffect: function () {
            this._listItems[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
        },
        _slideDeleteButton: function (currentItem, ele) {
            if (this.model.deleteMode == ej.mobile.ListView.DeleteMode.SlideButton) {
                var container = ej.buildTag("div.e-m-lv-slideitem"), deleteBtn = ej.buildTag("span.e-m-lv-delete", "Delete");
                currentItem.append(container.append(deleteBtn));
                ele.addClass("e-m-lv-swipeout");
                this._swipeDis = this.element.find(".e-m-lv-slideitem").width();
            }
        },
        _renderTemplateList: function () {
            var template = $("#" + this.model.templateId);
            template.html("<li class='e-m-lv-item'>" + template.html() + "</li>");
            this._dataSource.length && this.element.html(template.render(this._dataSource));
            ej.widget.init(this.element);
        },
        _dataSourceList: function (dataObj) {
            var proxy = this;
            $(dataObj).each(function (i, e) {
                var ele = ej.buildTag("a.e-m-lv-content", e[proxy.model.fields.text], {}, { 'data-ej-appajax': false, "href": e[proxy.model.fields.href] });
                var item = { badge: { value: !ej.isNullOrUndefined(e[proxy.model.fields.badge.value]) ? e[proxy.model.fields.badge.value] : null, maxValue: !ej.isNullOrUndefined(e[proxy.model.fields.badge.value]) ? e[proxy.model.fields.badge.maxValue] : null } };
                var li = ej.buildTag("li.e-m-lv-item" + (e[proxy.model.fields.checkBy] ? " e-m-lv-checked" : "") + (e[proxy.model.fields.enabled] ? " e-m-state-disabled" : ""), ele, {}, { "data-ej-badge-value": !ej.isNullOrUndefined(e[proxy.model.fields.badge.value]) ? e[proxy.model.fields.badge.value] : null, "data-ej-badge-maxvalue": !ej.isNullOrUndefined(e[proxy.model.fields.badge.maxValue]) ? e[proxy.model.fields.badge.maxValue] : null });
                if (!ej.isNullOrUndefined(e[proxy.model.fields.image]))
                    ele.css("background-image", "url(" + e[proxy.model.fields.image] + ")").addClass("e-m-lv-image");
                proxy.model.items.push(item);
                proxy._listItems.push(li);
                proxy._slideDeleteButton(li, ele);
                proxy.element.append(li.attr({ "index": i, "data-value": e[proxy.model.fields.value], "data-allowselection": e[proxy.model.fields.allowSelection] }));
                proxy._setBadge(li);
            });
        },
        _dataSourceGroupList: function () {
            var proxy = this, groupDataObj = ej.DataManager(proxy._dataSource).executeLocal(ej.Query().group(proxy.model.fields.groupBy));
            $(groupDataObj).each(function (i, e) {
                proxy.element.append(ej.buildTag("span.e-m-lv-group", e.key));
                proxy._dataSourceList(e.items);
            });
        },
        _touchStartHandler: function (evt) {
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            this._startX = evt.clientX;
            this._startY = evt.clientY;
            this._timeStart = evt.timeStamp || Date.now();
            var curTarget = $(evt.target);
            this._curentEle = curTarget.closest('li');
            this._removeSelect();
            if (!curTarget.hasClass("e-m-lv-delete")) {
                if (!this._curentEle.hasClass("swipeend")) {
                    if (this.model.allowSelection && this._curentEle.attr("data-allowselection") != "false")
                        this._curentEle.addClass("e-m-lv-active e-m-lv-selected");
                    if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                        this._curentEle.addClass(ej.isMobile() ? ej._getSkewClass($(this._curentEle), evt.pageX, evt.pageY) : "e-m-skew-center");
                    this.element.find('a,span').css("-" + this._browser + "-transform", "");
                    this.element.find(".e-m-lv-slideitem,.e-m-lv-delete").removeClass("e-m-lv-swipe");
                    this.element.find("li").removeClass("swipeend");
                }
            }
            ej.listenEvents([this._doc, this.element], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], false);
            this.model.touchStart && this._trigger("touchStart", this._getItemObject(this._curentEle, evt));
        },
        _touchMoveHandler: function (evt) {
            this._isMoved = (evt.clientX != this._startX) ? true : false;
            if (this._isMoved)
                evt.preventDefault();
            var target = $(evt.target);
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            var relDis = evt.clientX - this._startX, relDisY = evt.clientY - this._startY;
            if (Math.abs(relDisY) >= 10) {
                this._move = true;
                this._removeActive();
                ej._removeSkewClass(this._curentEle);
            }
            this.model.deleteMode != ej.mobile.ListView.DeleteMode.Default && this._removeActive();
            if (target.closest(this.element).length != 0) {
                if (this.model.deleteMode == ej.mobile.ListView.DeleteMode.SlideButton && (relDis < 0 || this._curentEle.hasClass("swipeend"))) {
                    this._relDis = this._curentEle.hasClass("swipeend") ? -this._swipeDis + relDis : relDis;
                    if (this._relDis <= 0) {
                        this._curentEle.find('a,span').css("-" + this._browser + "-transform", "translate3d(" + this._relDis + "px,0px,0px) scale(1)").css("-" + this._browser + "-transition", "none");
                        this._curentEle.find(".e-m-lv-slideitem,.e-m-lv-delete").addClass("e-m-lv-swipe");
                    }
                }
                if (this.model.deleteMode == ej.mobile.ListView.DeleteMode.Swipe) {
                    this._relDis = relDis;
                    this._curentEle.find('a').css("-" + this._browser + "-transform", "translate3d(" + relDis + "px,0px,0px) scale(1)").css("-" + this._browser + "-transition", "none");
                }
            }
            else
                this._touchEndHandler(evt);
        },
        _touchEndHandler: function (evt) {
            var curTarget = $(evt.target), duration = (evt.timeStamp || Date.now()) - this._timeStart;
            !this.model.persistSelection && this._curentEle.removeClass("e-m-lv-active");
            if (this.model.deleteMode == ej.mobile.ListView.DeleteMode.Swipe) {
                this._curentEle.find('a').css("-" + this._browser + "-transform", "translate3d(0px,0px,0px) scale(1)").css("-" + this._browser + "-transition", "").addClass("e-m-lv-swipeout");
                if (this._curentEle.width() / 2 <= Math.abs(this._relDis) || (duration >= 120 && duration <= 200 && Math.abs(this._relDis) >= 10))
                    this._deleteItem(this._curentEle, evt);
            }
            if (this.model.deleteMode == ej.mobile.ListView.DeleteMode.SlideButton) {
                if (this._relDis != 0) {
                    var dis = (-this._swipeDis / 2 <= this._relDis) ? 0 : -this._swipeDis;
                    dis != 0 ? this._curentEle.addClass("swipeend") : this._curentEle.removeClass("swipeend");
                    this._curentEle.find('a,span').css("-" + this._browser + "-transform", "translate3d(" + dis + "px,0px,0px) scale(1)").css("-" + this._browser + "-transition", "").addClass("e-m-lv-swipeout");
                }
                else if (curTarget.hasClass("e-m-lv-delete"))
                    this._deleteItem(this._curentEle, evt);
                this._relDis = 0;
            }
            if (!this._move) {
                this.model.enableChecklist && this._curentEle.toggleClass("e-m-lv-checked");
                this.model.renderMode == "windows" && !this.model.windows.preventSkew && ej._removeSkewClass(this._curentEle);
                this._setModelValues();
                this.model.touchEnd && this._trigger("touchEnd", this._getItemObject(this._curentEle, evt));
                this.model.select && this._trigger("select", this._getItemObject(this._curentEle, evt));
            }
            ej.listenEvents([this._doc, this.element], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
            this._move = false;
        },
        _deleteItem: function (curentEle, evt) {
            var index = curentEle.index();
            this.model.beforeDelete && this._trigger("beforeDelete", this._getItemObject(curentEle, evt));
            this.model.dataSource.length && this.model.dataSource.splice(index, 1);
            curentEle.remove();
            this.model.afterDelete && this._trigger("afterDelete", this._getItemObject(curentEle, evt));
        },
        _removeSelect: function () {
            this.element.find(".e-m-lv-active,.e-m-lv-selected").removeClass("e-m-lv-active e-m-lv-selected");
        },
        _removeActive: function () {
            this.element.find(".e-m-lv-active").removeClass("e-m-lv-active");
        },
        _setModelValues: function () {
            var proxy = this;
            this.model.selectedIndex = parseInt(this.element.find(".e-m-lv-selected").attr("index"));
            this.model.checkedIndices = [];
            this.element.find(".e-m-lv-checked").each(function (e) {
                proxy.model.checkedIndices.push(parseInt($(this).attr('index')));
            });
        },
        _setBadge: function (li, value) {
            var currentBageValue = value || (this.model.items[li.attr('index')].badge ? this.model.items[li.attr('index')].badge.value : null);
            maxBadgeValue = this.model.items[li.attr('index')].badge ? this.model.items[li.attr('index')].badge.maxValue : this.model.fields.bage.maxValue;
            if (!ej.isNullOrUndefined(currentBageValue)) {
                var badgeValue = (parseInt(currentBageValue) > parseInt(maxBadgeValue)) ? maxBadgeValue + "+" : currentBageValue;
                li.find('.e-m-lv-content').attr('badgeValue', badgeValue).addClass('e-m-lv-badge');
            }
        },
        _getItemObject: function (item, evt) {
            var index = this.element.find('li').index(item);
            return {
                item: item,
                index: index,
                text: item.find('a').html(),
                badge: item.find('a').attr("badgeValue") || null,
                isSelected: item.hasClass("e-m-lv-selected"),
                isChecked: item.hasClass("e-m-lv-checked"),
                data: this.model.dataSource[index] || null,
                event: evt,
                isInteraction: evt ? true : false
            };
        },
        /* Public Methods */
        addItem: function (itemVal, index) {
            var proxy = this;
            if (!this.model.dataSource.length) {
                ItemObj = $.isArray(itemVal) ? itemVal : $.makeArray(itemVal);
                $(ItemObj).each(function (i, e) {
                    var ele = ej.buildTag("a.e-m-lv-content");
                    var li = ej.buildTag("li.e-m-lv-item");
                    index ? $(proxy.element.find("li")[index]).before(li.append(ele.html(e))) : $(proxy.element.find("li")[proxy.element.find("li").length - 1]).after(li.append(ele.html(e)));
                });
            }
            else {
                $(itemVal).each(function (i, e) {
                    proxy.model.dataSource.splice(index ? index : proxy.model.dataSource.length, 0, e);
                });
                this._refresh();
            }
        },
        refresh: function (data) {
            this._dataProcess(data, "refresh");
        },
        append: function (newData) {
            this._dataProcess(newData, "append");
        },
        enable: function () {
            this.element.removeClass("e-m-state-disabled");
            this.model.enabled = true;
        },
        disable: function () {
            this.element.addClass("e-m-state-disabled");
            this.model.enabled = false;
        },
        enableItemByIndex: function (index) {
            if (!this.model.enabled) return;
            $(this.element.find('li')[index]).removeClass("e-m-state-disabled");
        },
        disableItemByIndex: function (index) {
            if (!this.model.enabled) return;
            $(this.element.find('li')[index]).addClass("e-m-state-disabled");
        },
        enableItemsByIndices: function (indices) {
            if (!this.model.enabled) return;
            var proxy = this;
            $(indices).each(function (e, index) {
                proxy.enableItemByIndex(index);
            });
        },
        disableItemsByIndices: function (indices) {
            if (!this.model.enabled) return;
            var proxy = this;
            $(indices).each(function (e, index) {
                proxy.disableItemByIndex(index);
            });
        },
        enableAll: function () {
            if (!this.model.enabled) return;
            $(this.element.find('li').removeClass("e-m-state-disabled"));
        },
        disableAll: function () {
            if (!this.model.enabled) return;
            $(this.element.find('li').addClass("e-m-state-disabled"));
        },
        selectItemByIndex: function (index) {
            if (!this.model.enabled) return;
            if (index >= 0) {
                this._curentEle = $(this.element.find('li').removeClass("e-m-lv-selected e-m-lv-active")[this.model.selectedIndex = index]);
                this._curentEle.addClass("e-m-lv-selected " + ((this.model.persistSelection) ? "e-m-lv-active" : ""));
                this.model.select && this._trigger("select", this._getItemObject(this._curentEle));
            }
        },
        deselectItem: function () {
            if (!this.model.enabled) return;
            $(this.element.find('li').removeClass("e-m-lv-selected e-m-lv-active"));
            this.model.selectedIndex = null;
        },
        checkItemsByIndex: function (index) {
            if (!this.model.enableChecklist || !this.model.enabled) return;
            $(this.element.find('li')[index]).addClass("e-m-lv-checked");
            this._setModelValues();
        },
        checkItemsByIndices: function (indices) {
            if (!this.model.enableChecklist || !this.model.enabled) return;
            var proxy = this;
            $(indices).each(function (e, index) {
                $(proxy.element.find('li')[index]).addClass("e-m-lv-checked");
            });
            this._setModelValues();
        },
        uncheckItemsByIndex: function (index) {
            if (!this.model.enableChecklist || !this.model.enabled) return;
            $(this.element.find('li')[index]).removeClass("e-m-lv-checked");
            this._setModelValues();
        },
        uncheckItemsByIndices: function (indices) {
            if (!this.model.enableChecklist || !this.model.enabled) return;
            var proxy = this;
            $(indices).each(function (e, index) {
                $(proxy.element.find('li')[index]).removeClass("e-m-lv-checked");
            });
            this._setModelValues();
        },
        checkAll: function () {
            if (!this.model.enableChecklist || !this.model.enabled) return;
            $(this.element.find('li').addClass("e-m-lv-checked"));
            this._setModelValues();
        },
        uncheckAll: function () {
            if (!this.model.enableChecklist || !this.model.enabled) return;
            $(this.element.find('li').removeClass("e-m-lv-checked"));
            this._setModelValues();
        },
        getItemByIndex: function (index) {
            if (!this.model.enabled) return;
            return $(this.element.find('li')[index]);
        },
        getItemByText: function (text) {
            if (!this.model.enabled) return;
            var index;
            $(this.element.find('li')).each(function (i, element) {
                if ($(element).find('a').html() == text) index = i;
            });
            return this.getItemByIndex(index);
        },
        getTextByIndex: function (index) {
            if (!this.model.enabled) return;
            return $(this.element.find('li')[index]).find('a').html();
        },
        getIndexByText: function (text) {
            if (!this.model.enabled) return;
            var index;
            $(this.element.find('li')).each(function (i, element) {
                if ($(element).find('a').html() == text) index = i;
            });
            return index;
        },
        deleteItemByIndex: function (index) {
            if (!this.model.enabled) return;
            var item = $(this.element.find('li')[index]).remove().find('a').html();
            if (this.model.dataSource.length)
                item = this.model.dataSource.splice(index, 1);
            return item;
        },
        deleteItemByText: function (text) {
            if (!this.model.enabled) return;
            return this.deleteItemByIndex(this.getIndexByText(text));
        },
        setBadge: function (index, value) {
            if (!this.model.enabled) return;
            this._setBadge(this.getItemByIndex(index), value);
        },
        /* Public Methods */
        _setModel: function (options) {
            var option, refresh = false;
            for (option in options) {
                if (!this.model.enabled && option != "enabled") return false;
                switch (option) {
                    case "renderMode":
                        this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
                    case "selectedIndex":
                        this.selectItemByIndex(options[option]); break;
                    case "checkedIndices":
                        this.checkItemsByIndices(options[option]); break;
                    case "enabled":
                        options[option] ? this.enable() : this.disable(); break;
                    case "enableRippleEffect":
                        this._setEnableRippleEffect(); break;
                    case "allowSelection":
                        this.element[(options[option] ? "addClass" : "removeClass")]("e-m-lv-preventselection");
                }
                if (options.dataSource || options.fields || options.query) {
                    if (!options.dataSource)
                        this.model.dataSource = this._dataObj;
                    this.deselectItem();
                    this.uncheckAll();
                    refresh = true;
                }
            }
            refresh && this._refresh();
            refresh = false;
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass(this._rootCSS);
            this._initialize();
        },
        _clearElement: function () {
            this.element.removeAttr("class").removeAttr("style");
            this.element.empty().html(this._orgEle.html());
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
    });

    ej.mobile.ListView.DeleteMode = {
        Default: "none",
        Swipe: "swipe",
        SlideButton: "slideButton"
    };
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html Menu elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    //mMenu is the plugin name 
    //"m.Menu" is "namespace.className" will hold functions and properties

    ej.widget("ejmMenu", "ej.mobile.Menu", {
        _setFirst: true,
        _rootCSS: "e-m-menu",
        _tags: [{
            tag: "items",
            attr: ["text", "touchStart", "touchEnd", "href", "color"],
            content: "textTemplate"
        }],

        defaults: {
            renderMode: "auto",
            height: null,
            width: null,
            templateId: null,
            dataSource: [],
            fields: {
                text: "",
                href: null,
                color: ""
            },
            query: ej.Query(),
            allowScrolling: true,
            enablePersistence: false,
            text: "",
            color: "",
            target: null,
            showOn: "tap",
            //Cancel Button properties works only Actionsheet type
            cancelButton: {
                text: null,
                show: true,
                color: ""
            },
            //Title properties works only Actionsheet and PopOver types
            showTitle: true,
            title: null,
            //showArrow property works only PopOver type
            showArrow: true,
            type: "actionsheet",
            locale: "en-US",
            load: null,
            loadComplete: null,
            touchStart: null,
            touchEnd: null,
            hide: null,
            show: null,
            cssClass: "",
            enableRippleEffect: ej.isAndroid() ? true : false,
            items: [],
            href: null,
            actionSuccess: null,
            actionFailure: null,
            actionComplete: null
        },

        dataTypes: {
            dataSource: "data",
            query: "data",
            enableRippleEffect: "boolean",
            renderMode: "enum",
            locale: "string",
        },
        observableArray: ["dataSource"],
        dataSource: ej.util.valueFunction("dataSource"),

        _init: function () {
            ej.getCurrentPage().append(this.element);
            this._getLocalizedLabels();
            this.model.title = !ej.isNullOrUndefined(this.model.title) ? this.model.title : this._localizedLabels["title"];
            this.model.cancelButton.text = !ej.isNullOrUndefined(this.model.cancelButton.text) ? this.model.cancelButton.text : this._localizedLabels["cancelButtonText"];
            this._template = ej.getCurrentPage().find("#" + this.model.templateId);
            if (Object.keys(this.model.dataSource).length > 1)
                this._initDataSource();
            else
                this._render();
        },

        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
        },

        _load: function () {
            ej.setRenderMode(this);
            this._touchStart = this.model.touchStart;
            this._touchEnd = this.model.touchEnd;
            this._orgEle = this.element.clone();
            if (this._orgEle.find("ul").length || Object.keys(this.model.dataSource).length > 1)
                this.model.items = [];
            this.model.id = this.element[0].id;
            if (Object.keys(this.model.dataSource).length > 1)
                this._dataSourceList(this._dataSource);
            if (this.model.load)
                this._trigger("load");
        },

        _renderItems: function (proxy) {
            if (proxy.model.items.length < 1) {
                var elements = this.element.find("li");
                for (i = 0; i < elements.length; i++) {
                    var item = {};
                    var element = elements[i];
                    item.text = ej.getAttrVal(element, 'data-ej-text', "")
                    item.href = ej.getAttrVal(element, 'data-ej-href', null);
                    item.color = ej.getAttrVal(element, 'data-ej-color', "");
                    item.templateId = ej.getAttrVal(element, 'data-ej-templateid', undefined);
                    item.touchStart = ej.getAttrVal(element, 'data-ej-touchstart', undefined);
                    item.touchEnd = ej.getAttrVal(element, 'data-ej-touchend', undefined);
                    this.model.items.push(item);
                }
            }
            else {
                var ul = ej.buildTag('ul.e-m-menu-hdr e-m-clearall');
                var elements = this.model.items;
                for (i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    element.text = element.text ? element.text : "";
                    element.href = element.href ? element.href : null;
                    element.color = element.color ? element.color : "";
                    element.templateId = element.templateId ? element.templateId : undefined;
                    element.touchStart = element.touchStart ? element.touchStart : undefined;
                    element.touchEnd = element.touchEnd ? element.touchEnd : undefined;
                    var li = ej.buildTag('li', elements[i].textTemplate);
                    this.element.append(ul.append(li));
                }
            }
            this._setEnableRippleEffect();
        },

        _setEnableRippleEffect: function () {
            this.element.find("li")[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
                this.element.find("span.e-m-menu-btn")[(this.model.enableRippleEffect && this.model.cancelButton.show ? "addClass" : "removeClass")]("e-ripple");
        },

        _render: function () {
            this._renderControl();
            this._wireEvents();
        },

        _initDataSource: function () {
            ej.mobile.WaitingPopup.show();
            var proxy = this;
            if (this.dataSource() instanceof ej.DataManager)
                this.dataSource().executeQuery(proxy.model.query).done(function (e) {
                    proxy._dataSource = e.result;
                    proxy._render();
                    proxy._trigger("actionSuccess", e);
                    ej.mobile.WaitingPopup.hide();
                }).fail(function (e) {
                    proxy._trigger("actionFailure", e);
                }).always(function (e) {
                    proxy._trigger("actionComplete", e);
                });
            else {
                this._dataSource = ej.DataManager(this.dataSource()).executeLocal(this.model.query);
                this._render();
                ej.mobile.WaitingPopup.hide();
            }
        },

        _renderControl: function () {
            this._load();
            this.element.addClass("e-m-" + this.model.renderMode + " " + this.model.cssClass);
            var proxy = this;
            this._renderItems(proxy);
            this._menuUl = this.element.find("ul").addClass("e-m-menulist");
            if (this.model.templateId && (Object.keys(this.model.dataSource).length < 1)) {
                ej.getCurrentPage().find("#" + this.model.templateId).remove();
                this.element.append(ej.buildTag("div.e-m-menutemplate").append(this._template.html()));
                this._menuProperties();
            }
            else {
                this._menuLiItems = this.element.find("li").addClass("e-m-state-default e-m-user-select e-m-menuitem");
                this._menuLiItems.first().addClass("e-m-firstitem");
                this._menuLiItems.last().addClass("e-m-lastitem");
                this._menuProperties();
                this._menuLiItems.each(function (index, element) {
                    proxy._renderMenuItems(element, index);
                });
                this._liHeight = ej.getDimension(this._menuLiItems, "height");
            }
            ej.widget.init(this.element);
            this.hide();
            this._setHeightWidth();
            if (this.model.allowScrolling)
                this._renderScrollPanel();
            if (this.model.loadComplete)
                this._trigger("loadComplete");
        },

        _getTarget: function () {
            return ((typeof (this.model.target) == "string") ? (((ej.getCurrentPage().find("#" + this.model.target)).length > 0) ? (ej.getCurrentPage().find("#" + this.model.target)) : $("body").find("#" + this.model.target)) : this.model.target);
        },

        _setHeightWidth: function () {
            if (this.model.width)
                this.element.width(this.model.width);
            this._titleHeight = (Math.abs(this._titleDiv ? ej.getDimension(this._titleDiv, "outerHeight") : 0));
            this._menuHeight = (this._liHeight * 5) + (this._titleHeight) + this._menuArrowHeight;
            if (this.model.height)
                this._menuUl.height(this.model.height - (this._titleHeight + this.element.find(".e-m-menu-cancelbtn").outerHeight() + (this.model.type == "actionsheet" ? 10 : 0)));
            else if (this.model.allowScrolling) {
                if (this.model.type == "actionsheet") {
                    if (window.innerHeight < ((this._liHeight * 5) + (this._titleDiv ? this._titleDiv.outerHeight() : 0) + this.element.find(".e-m-menu-cancelbtn").outerHeight() + 25))
                        this._menuUl.css("max-height", window.innerHeight - ((this._titleDiv ? this._titleDiv.outerHeight() : 0) + this.element.find(".e-m-menu-cancelbtn").outerHeight() + 20));
                    else
                        this._menuUl.css("max-height", this._liHeight * 5);
                }
                else {
                    var top = this._getTarget().offset().top;
                    var hgt = top + this._getTarget().outerHeight();
                    this.element.css({ "top": "auto", "bottom": "auto" });
                    if (((this._menuHeight) + (this.element.offset().top + 10)) > window.innerHeight)
                        this._menuUl.css("max-height", (this._liHeight * 5) - ((this._menuHeight) - (window.innerHeight - (this.element.offset().top))));
                    else if (!this.model.height && this.model.allowScrolling && ej.isNullOrUndefined(this.model.templateId) && (window.innerHeight >= (this._menuHeight)))
                        this._menuUl.css("max-height", this._liHeight * 5);
                }
            }
        },

        _renderScrollPanel: function () {
            var scrollWrapper = ej.buildTag("div.e-m-menuscrollwrapper");
            if (ej.isNullOrUndefined(this.model.templateId))
                this._menuUl.wrap(scrollWrapper);
            else
                this.element.find(".e-m-menutemplate").wrap(scrollWrapper);
            this.element.find(".e-m-menuscrollwrapper").ejmScrollPanel({ isRelative: true });
        },

        _dataSourceList: function (dataObj) {
            if (ej.isNullOrUndefined(this.model.templateId)) {
                var proxy = this;
                this._datasrcUl = ej.buildTag("ul");
                this.element.append(this._datasrcUl);
                $(dataObj).each(function (i, e) {
                    var li = ej.buildTag("li", e[proxy.model.fields.text], { color: e[proxy.model.fields.color] }, {}).attr("data-ej-href", e[proxy.model.fields.href]).appendTo(proxy.element);
                    proxy._datasrcUl.append(li);
                });
            }
            else {
                var template = ej.getCurrentPage().find("#" + this.model.templateId);
                if (this._dataSource.length) {
                    this._temp = ej.buildTag("div.e-m-menutemplate");
                    this.element.append(this._temp);
                    this._temp.html(template.render(this._dataSource));
                }
            }
        },

        _menuProperties: function () {
            this.element.addClass("e-m-menu-" + this.model.type + (this.model.type == "actionsheet" ? " e-m-abs" : " e-m-fixed"));
            if (this.model.type == "popover")
                this.element.addClass("e-m-corner-all").prepend(this.model.showArrow ? ej.buildTag("div.e-m-icon-arrow e-m-top") : "").append(this.model.showArrow ? ej.buildTag("div.e-m-icon-arrow e-m-bottom") : "");
            if (this.model.showTitle && !(this.model.type == "popup")) {
                this._titleDiv = ej.buildTag("span.e-m-menu-title", this.model.title);
                this.model.type == "actionsheet" ? this.element.prepend(this._titleDiv) : (this.model.showArrow ? this._titleDiv.insertAfter(this.element.find('.e-m-icon-arrow.e-m-top')) : this.element.prepend(this._titleDiv));
            }
            this._menuArrowHeight = this.element.find(".e-m-icon-arrow").height();
            if (this.model.cancelButton.show && this.model.type == "actionsheet") 
                this.element.append(ej.buildTag("span.e-m-menu-btn").addClass("e-m-menu-cancelbtn").text(this.model.cancelButton.text).css("color", this.model.cancelButton.color));
            this._createOverlayDiv();
        },

        _renderMenuItems: function (element, index) {
            if (!$(element).hasClass("e-m-menu-title")) {
                var text = (ej.getAttrVal(element, 'data-ej-text') == ("" || undefined)) ? ((this.model.items[index].text == "") ? $(this._menuLiItems[index]).html() : this.model.items[index].text) : (ej.getAttrVal(element, 'data-ej-text'));
                ctrl = ej.buildTag("a.e-m-menu-btn#" + this.model.id + index, text, {}, { href: ej.getAttrVal(element, 'data-ej-href') || (this.model.items[index].href) }).css("color", ej.getAttrVal(element, 'data-ej-color') || this.model.items[index].color);
                $(element).empty().append(ctrl.css("color", element.style.color == "" ? this.model.items[index].color : element.style.color));
            }
        },

        //Popover and popup positioning
        _normalPosition: function (target) {
            if (target.length) {
                var width = this.element.width();
                this.element.find('.e-m-icon-arrow.e-m-top').removeClass("e-m-state-hide").addClass("e-m-state-block");
                this.element.find('.e-m-icon-arrow.e-m-bottom').removeClass("e-m-state-block").addClass("e-m-state-hide");
                var left = target.offset().left + target.outerWidth() / 2 - (width / 2);
                var win_width = window.innerWidth;
                this._windowHeight = window.innerHeight;
                this._targetHeight = this.element.outerHeight();
                if (left < 0)
                    left = 5;
                else if (win_width - left < width)
                    left = win_width - width;
                if (this.model.type == "popover")
                    this.element.css({ "left": left + "px" });
                else
                    this.element.css({ "left": win_width / 2 - (this.element.width() / 2) + "px" });
                this.element.find('.e-m-icon-arrow').css("left", target.offset().left + ((target.outerWidth() - this.element.find('.e-m-icon-arrow').width()) / 2) - left);
                var top = target.offset().top;
                var hgt = top + target.outerHeight();
                this.element.css({ "top": hgt + 10, "bottom": "auto" });
                if (!this.model.showArrow && this.model.type == "popover")
                    this.element.css("top", this.element.offset().top - (target.height() + 10));
                if (window.innerHeight < this.element.offset().top)
                    this.element.css({ "top": "auto" });
                if (this.model.type == "popover") {
                    if (!this.model.height && ((window.innerHeight - (top + 10)) < this._targetHeight))
                        this._menuUl.css("max-height", ((this._liHeight * 5) - ((this._titleDiv ? this._titleDiv.outerHeight() : 0) + this._menuArrowHeight + 10)));
                    if (top > (this._targetHeight + 10) && ((window.innerHeight - top) < this._targetHeight)) {
                        this.element.css({ "bottom": this._windowHeight - top + 10, "top": "auto" });
                        this.element.find('.e-m-icon-arrow.e-m-bottom').removeClass("e-m-state-hide").addClass("e-m-state-block");
                        this.element.find('.e-m-icon-arrow.e-m-top').removeClass("e-m-state-block").addClass("e-m-state-hide");
                    }
                }
                if (!this.model.showArrow && this.model.type == "popover")
                    this.element.css("bottom", (this._windowHeight - top + 10) - (target.height() + 10));
                if (this.model.type == "popup") {
                    this.element.css({ "top": (this._windowHeight - this._targetHeight) / 2 });
                    this.element.css({ "left": (this._windowWidth - this.element.outerWidth()) / 2 });
                }
            }
        },

        _createOverlayDiv: function () {
            this._overlay = ej.buildTag("div#" + this.model.id + "_overlay", "", "", { "class": "e-m-overlay e-m-menu-overlay" + " e-m-menu-" + this.model.type });
            this.element.after(this._overlay);
        },

        _cancelbtnTouchEnd: function (evt) {
            this.element.find(".e-m-menu-cancelbtn").removeClass("e-m-state-active");
            var currentMenu = evt.currentTarget;
            var data = { item: $(currentMenu), text: $(currentMenu).text() };
            this._trigger("touchEnd", data);
            this.hide();
        },

        _cancelbtnTouchStart: function (evt) {
            this.element.find(".e-m-menu-cancelbtn").addClass("e-m-state-active");
            var currentMenu = evt.currentTarget;
            var data = { item: $(currentMenu), text: $(currentMenu).text() };
            this._trigger("touchStart", data);
        },

        _docTouchStartHandler: function (e) {
            this._currentTarget = (this.model.target && (e.currentTarget.id != this.model.target || e.target.id != this.model.target));
            if (!$(e.target).closest('.e-m-menu').length && this._currentTarget) {
                this._hide = true;
                this.hide();
            }
        },
        _docTouchEndHandler: function (e) {
            if (this._hide && !$(e.target).closest('.e-m-menu').length && this._currentTarget)
                this._hide = false;
        },

        _onTouchHandler: function (evt, evtName) {
            if (ej.isNullOrUndefined(this.model.templateId)) {
                this._currentItem[evtName == "touchStart" ? "addClass" : "removeClass"]("e-m-state-active");
                var index = $(evt.currentTarget).index();
                this.model[evtName] = this.model.items[index][evtName] ? this.model.items[index][evtName] : this["_" + evtName];
                if (this.model[evtName])
                    this._trigger(evtName, { item: this._currentItem, text: this._currentItem.text() });
            }
        },

        _onTouchStartHandler: function (evt) {
            this._currentItem = $(evt.currentTarget);
            this._onTouchHandler(evt, "touchStart");
        },

        _onTouchEndHandler: function (evt) {
            this._onTouchHandler(evt, "touchEnd");
        },

        _onTapEventHandler: function (evt) {
            if (this.model.showOn == "taphold" && (evt.currentTarget == this._getTarget()[0])) {
                ej.listenEvents([$(document), $(document)], [ej.startEvent(), ej.endEvent], [this._docTouchStartDelegate, this._docTouchEndDelegate], true);
                this.show();
                ej.listenEvents([$(document), $(document)], [ej.startEvent(), ej.endEvent], [this._docTouchStartDelegate, this._docTouchEndDelegate], false);
            }
        },

        _onTouchMoveHandler: function (evt) {
            this._currentItem = $(evt.currentTarget);
            this._currentItem.removeClass("e-m-state-active");
        },

        _orientation: function (evt) {
            var target = this._getTarget();
            var proxy = this;
            setTimeout(function () {
                proxy._setHeightWidth();
                if (proxy.model.type != "actionsheet")
                    proxy._normalPosition(target);
            }, ej.isAndroid() ? 100 : 0);
        },


        _createDelegates: function () {
            this._showDelegate = $.proxy(this.show, this);
            this._cancelbtnTouchEndDelegate = $.proxy(this._cancelbtnTouchEnd, this);
            this._cancelbtnTouchStartDelegate = $.proxy(this._cancelbtnTouchStart, this);
            this._docTouchStartDelegate = $.proxy(this._docTouchStartHandler, this);
            this._docTouchEndDelegate = $.proxy(this._docTouchEndHandler, this);
            this._touchStartDelegate = $.proxy(this._onTouchStartHandler, this);
            this._touchEndDelegate = $.proxy(this._onTouchEndHandler, this);
            this._tapDelegate = $.proxy(this._onTapEventHandler, this);
            this._touchMoveDelegate = $.proxy(this._onTouchMoveHandler, this);
            this._resizeDelegate = $.proxy(this._orientation, this);
            this._orientationDelegate = $.proxy(this._orientation, this);
        },

        _wireEvents: function (remove) {
            var eventType = remove ? "unbind" : "bind";
            this._createDelegates();
            this._li = this.element.find("li");
            ej.listenEvents([this._li, this._li, this._li, window], [ej.startEvent(), ej.endEvent(), ej.moveEvent(), 'orientationchange'], [this._touchStartDelegate, this._touchEndDelegate, this._touchMoveDelegate, this._orientationDelegate], remove);
            this._tapEvent = (this.model.showOn == "taphold") ? ej.tapHoldEvent() : ej.tapEvent();
            if (this.model.target) {
                if (this.model.showOn == "taphold")
                    ej.listenTouchEvent(this._getTarget(), this._tapEvent, this._tapDelegate, remove);
                else
                    ej.listenEvents([this._getTarget(), $(document), $(document)], [this._tapEvent, ej.startEvent(), ej.endEvent()], [this._showDelegate, this._docTouchStartDelegate, this._docTouchEndDelegate], remove);
            }
            if (this.model.cancelButton.show)
                ej.listenEvents([this.element.find('.e-m-menu-cancelbtn'), this.element.find('.e-m-menu-cancelbtn')], [ej.startEvent(), ej.endEvent()], [this._cancelbtnTouchStartDelegate, this._cancelbtnTouchEndDelegate], remove);
            if (!ej.isTouchDevice())
                $(window)[eventType]("resize", this._resizeDelegate);
        },

        _refresh: function () {
            this._destroy();
            this.element.addClass("e-m-menu");
            this._render();
        },

        _clearElement: function () {
            this.element.removeAttr("class").removeAttr("style");
            this._overlay.remove();
            this.element.html(this._orgEle.html());
        },

        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },

        _setCancelButton: function () {
            this.element.find(".e-m-menu-cancelbtn").css("color", this.model.cancelButton.color).text(this.model.cancelButton.text);
        },
        _setShowCancelButton: function (state) {
            this.element.find(".e-m-menu-cancelbtn").css("display", state ? "block" : "none");
        },

        _setType: function () {
            this._refresh();
        },

        _setAllowScrolling: function () {
            this._refresh();
        },

        _setTitle: function () {
            this._titleDiv.text(this.model.title);
        },

        _setRendermode: function () {
            this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
        },

        _setLocale: function () {
            this._getLocalizedLabels();
            this.model.title = this._localizedLabels["title"];
            this.model.cancelButton.text = this._localizedLabels["cancelButtonText"];
            this._setTitle();
            this._setCancelButtonText();
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },

        /*---------------Public Methods---------------*/
        show: function (e) {
            this.element.addClass("e-m-menushow");
            if (this.model.type == "popover" || this.model.type == "popup") {
                var target = $(e.currentTarget);
                this.element.show().css({ "opacity": "1" });
                this._setHeightWidth();
                this._normalPosition(target);
            }
            if (this.model.type == "actionsheet")
                this.element.css({ "bottom": 0 });
            this._overlay.removeClass("e-m-overlay-hide").css("pointer-events", "auto");
            if (this.model.show)
                this._trigger("show");
        },

        hide: function () {
            if (this.model.type == "actionsheet")
                this.element.css({ "bottom": (-ej.getDimension(this.element, "outerHeight") - this.model.height) + "px" });
            else
                this.element.css("display", "none");
            this._overlay.addClass("e-m-overlay-hide").css("pointer-events", "none");
            if (this.model.hide && this.element.hasClass("e-m-menushow"))
                this._trigger("hide");
            this.element.removeClass("e-m-menushow");
        },
        _enableDisable: function (val) {
            this.element[val ? "removeClass" : "addClass"]('e-m-state-disabled');
        },
        _enableDisableItem: function (index, val) {
            if (index)
                $(this.element.find('li.e-m-menuitem')[index - 1])[val ? "removeClass" : "addClass"]('e-m-state-disabled');
        },
        enableItem: function (index) {
            this._enableDisableItem(index, true);
        },
        disableItem: function (index) {
            this._enableDisableItem(index, false);
        },
        enable: function () {
            this._enableDisable(true);
        },

        disable: function () {
            this._enableDisable(false);
        },

        disableCancelButton: function () {
            this.element.find(".e-m-menu-cancelbtn").addClass("e-m-state-disabled");
        },

        enableCancelButton: function () {
            this.element.find(".e-m-menu-cancelbtn").removeClass("e-m-state-disabled");
        },

        addItem: function (menu, index) {
            if ($(this._orgEle.children()).length)
                (index) ?
                    menu.insertBefore($(this._orgEle.children().children()[index - 1])) :
                    this._orgEle.children().append(menu);
            else
                this.model.items.push(menu);
            this._refresh();
        },

        removeItem: function (index) {
            if ($(this._orgEle.children()).length)
                (index) ?
                    $(this._orgEle.children().children()[index - 1]).remove() :
                    $(this._orgEle.children().children()).last().remove();
            else 
                (index) ? this.model.items.splice((index - 1), 1) : this.model.items.pop();
            this._refresh();
        }
    });

    ej.mobile.Menu.ShowOn = {
        Tap: "tap",
        TapHold: "taphold"
    };

    ej.mobile.Menu.Type = {
        Popover: "popover",
        Popup: "popup",
        ActionSheet: "actionsheet"
    };
    ej.mobile.Menu.Locale = ej.mobile.Menu.Locale || {};
    ej.mobile.Menu.Locale["default"] = ej.mobile.Menu.Locale["en-US"] = {
        title: "Title",
        cancelButtonText: "Cancel"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html NavigationBar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget({ "ejmNavigationBar": "ej.mobile.NavigationBar", "ejmHeader": "ej.mobile.Header", "ejmFooter": "ej.mobile.Footer", "ejmToolbar": "ej.mobile.Toolbar" }, {
        _setFirst: true,
        _rootCSS: "e-m-navbar",
        _tags: [{
            tag: "items",
            attr: ["iconName", "badgeValue"],
            content: "template"
        }],
        defaults: {
            title: null,
            titleAlignment: "left",
            isRelative: false,
            renderMode: "auto",
            templateID: null,
            enablePersistence: false,
            cssClass: "",
            mode: "header",
            position: "auto",
            iconAlignment: "auto",
            locale: "en-US",
            badge: {
                "maxValue": 99
            },
            android: {
                "position": "auto"
            },
            ios7: {
                "position": "auto"
            },
            flat: {
                "position": "auto"
            },
            windows: {
                "position": "auto"
            },
            touchStart: null,
            touchEnd: null,
            ellipsisTouchStart: null,
            ellipsisTouchEnd: null,
            enableRippleEffect: ej.isAndroid() ? true : false,
            items: []
        },
        dataTypes: {
            title: "string",
            titleAlignment: "enum",
            items: "data",
            isRelative: "boolean",
            renderMode: "enum",
            templateID: "string",
            cssClass: "string",
            enableRippleEffect: "boolean",
            mode: "enum",
            position: "enum",
            iconAlignment: "enum",
            android: {
                "position": "enum"
            },
            ios7: {
                "position": "enum"
            },
            flat: {
                "position": "enum"
            },
            windows: {
                "position": "enum"
            },
            badge: {
                "maxValue": "number"
            },
            enablePersistence: "boolean"
        },
        _init: function () {
            ej.setRenderMode(this);
            this._getLocalizedLabels();
            this.model.title = !ej.isNullOrUndefined(this.model.title) ? this.model.title : this._localizedLabels["title"];
            this._renderControl();
            this._wireEvents(false);
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
        },
        _renderControl: function () {
            this._orgEle = $(this.element).clone();
            this._angularContent = this.element.children();
            this._position = this.model[this.model.renderMode].position != "auto" ? this.model[this.model.renderMode].position : this.model.position != "auto" ? this.model.position : this.model.mode == "header" ? "top" : this.model.renderMode == "android" ? "top" : "bottom";
            this.model.iconAlignment = (!ej.isNullOrUndefined(this.model.iconAlignment) && this.model.iconAlignment != "auto") ? this.model.iconAlignment : this.model.renderMode == "windows" ? "group" : "split";
            this.element.addClass("e-m-user-select e-m-" + this.model.renderMode + " " + this.model.cssClass + " e-m-navbar-" + this._position + " e-m-navbar-" + this.model.mode + " e-m-" + (this.model.isRelative ? "rel" : "abs") + (this.model.templateID ? " e-m-navbar-template" : ""));
            this.model.templateID ? this._renderNavigationBarTemplate() : this._renderDefaultNavigationBar();
        },
        _renderNavigationBarTemplate: function () {
            var templateContent = ej.getCurrentPage().find("#" + this.model.templateID);
            if (templateContent.length) {
                this.element.empty();
                ej.destroyWidgets(templateContent);
                this._template = templateContent[0].nodeName && templateContent[0].nodeName.toLowerCase() == "script" ? ej.getClearString(templateContent[0].innerHTML) : templateContent;
                this.element.append(this._template);
                ej.widget.init(this.element);
                this._compileAngularElement(this.element.children());
            }
        },
        _renderDefaultNavigationBar: function () {
            this._compileAngularElement(this._angularContent);
            if (this.model.mode == "header" || (this.model.mode == "toolbar" && this._position == "top")) {
                this._titleSpan = ej.buildTag("span.e-m-navbar-text", this.model.title, null, null).appendTo(this.element);
                this._titleSpan.addClass("e-m-title-" + this.model.titleAlignment).css({ "margin-left": this.element.find(".e-m-navbar-left").length ? (ej.getDimension(this.element.find(".e-m-navbar-left"), "outerWidth") + 32) : 16, "margin-right": (this.model.mode == "header") ? this.element.find(".e-m-navbar-right").length ? (ej.getDimension(this.element.find(".e-m-navbar-right"), "outerWidth") + 32) : 16 : "" });
            }
            if (this.model.mode == "toolbar")
                this._toolbarRendering();
        },
        _compileAngularElement: function (element) {
            if ((!ej.isAppNullOrUndefined() && App.angularAppName) || ej.angular.defaultAppName)
                ej.angular.compile($(element));
        },
        _toolbarRendering: function () {
            var proxy = this, isAngular = this.model.items.length ? true : false;
            this._items = [];
            if (!isAngular) {
                this._navIcons = this.element.find("li");
                for (var i = 0, itemLength = this._navIcons.length; i < itemLength; i++)
                    this._items.push(this._insertItemValue($(this._navIcons[i])));
            }
            else this._items = this.model.items;
            this._navContainer = isAngular ? ej.buildTag("ul.e-m-clearall").appendTo(this.element) : this.element.find('ul').addClass("e-m-clearall");
            $.each(this._items, function (index) {
                var liitem = (isAngular ? (ej.buildTag("li").attr("data-ej-badgevalue", (!ej.isNullOrUndefined(this.badgeValue) && this.badgeValue > 0) ? this.badgeValue : null)) : $(proxy._navIcons[index])).attr("id", "nav-item" + index).addClass("e-m-navbar-icon e-m-icon-" + this.iconName).appendTo(proxy._navContainer);
                if (parseInt(liitem.attr("data-ej-badgevalue")) > 0) {
                    var val = parseInt(this.badgeValue);
                    liitem.addClass("e-m-nav-badge").attr("badgeValue", val <= proxy.model.badge.maxValue ? val : proxy.model.badge.maxValue.toString() + "+");
                }
            });
            this._allIconsLength = this._navContainer.find("li").length;
            this._maxIconsLength = this._position == "top" ? (this._allIconsLength > 3 ? 2 : this._allIconsLength) : this._allIconsLength > 5 ? 5 : this._allIconsLength;
            this._ellipsis = null;
            if (this._allIconsLength > this._maxIconsLength) {
                this._ellipsis = ej.buildTag("span.e-m-navbar-ellipsis e-m-icon-overflow").appendTo(this.element);
                this._overflowContainer = this._navContainer.clone();
                this.element.append(this._overflowContainer);
                this._navContainer.find("li").slice(this._maxIconsLength, this._allIconsLength).remove();
                this._overflowContainer.addClass("e-m-overflow-container").find("li").slice(0, this._maxIconsLength).remove();
                this._overflowIcons = this._overflowContainer.find("li").addClass("e-m-overflow-icon");
            }
            this._navbarIcons = this._navContainer.addClass("e-m-navbar-container" + (this._position == "bottom" ? " e-m-" + this.model.iconAlignment + "-icons" + (!ej.isNullOrUndefined(this._ellipsis) ? " e-m-more" : "") : "")).find("li");
            if (this._position == "top") {
                this._navContainer.addClass((this._allIconsLength > this._maxIconsLength ? "e-m-margin-right" : ""));
                this._titleSpan.addClass("e-m-margin-right-" + (this._allIconsLength > this._maxIconsLength ? 3 : this._allIconsLength));
            }
            this._iconWidth();
            this._setEnableRippleEffect();
        },

        _setEnableRippleEffect: function () {
            this.element.find("li")[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
        },
        _insertItemValue: function (item) {
            var itemVal = {};
            itemVal.iconName = ej.getAttrVal(item, 'data-ej-iconname', undefined);
            itemVal["badgeValue"] = ej.getAttrVal(item, 'data-ej-badgevalue', undefined);
            return itemVal;
        },
        _iconWidth: function () {
            if (this.model.iconAlignment != "group") {
                if (this.model.mode == "toolbar" && this._position == "bottom") {
                    if (!ej.isNullOrUndefined(this._ellipsis))
                        this._navContainer.width(window.innerWidth - 56);
                    else this._navContainer.width("100%");
                }
            }
            else this._navContainer.width("auto");

        },
        _createDelegates: function () {
            this._ellipsisTouchStartHndlr = $.proxy(this._ellipsisTouchStart, this);
            this._ellipsisTouchEndHndlr = $.proxy(this._ellipsisTouchEnd, this);
            this._touchStartHndlr = $.proxy(this._touchStart, this);
            this._touchEndHndlr = $.proxy(this._touchEnd, this);
            this._resizeHndlr = $.proxy(this._resize, this);
        },
        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([$(this._ellipsis), $(this._ellipsis), this._navbarIcons, this._navbarIcons], [ej.startEvent(), ej.endEvent(), ej.startEvent(), ej.endEvent()], [this._ellipsisTouchStartHndlr, this._ellipsisTouchEndHndlr, this._touchStartHndlr, this._touchEndHndlr], remove);
            ej.listenTouchEvent($(document), ej.tapEvent(), this._docClickHndlr, remove);
            ej.listenTouchEvent($(window), "onorientationchange" in window ? "orientationchange" : "resize", this._resizeHndlr, remove);
        },
        _touchStart: function (args) {
            this._touchEvents(args, true, 'touchStart');
        },
        _touchEnd: function (args) {
            this._touchEvents(args, true, 'touchEnd');
        },
        _ellipsisTouchStart: function (args) {
            this._touchEvents(args, false, 'ellipsisTouchStart');
        },
        _ellipsisTouchEnd: function (args) {
            this._touchEvents(args, false, 'ellipsisTouchEnd');
        },
        _touchEvents: function (args, isIconEvent, event) {
            var data = isIconEvent ? { element: $(args.target), iconname: args.target.getAttribute("data-ej-iconname"), index: args.target.id.split("item")[1] } : { element: $(args.target) };
            var add = args.type == ej.startEvent() ? "add" : "remove";
            $(args.target)[add + "Class"]("e-m-state-active");
            if (this.model[event])
                this._trigger(event, data);
        },
        _resize: function (args) {
            var proxy = this;
            setTimeout(function () {
                proxy._iconWidth();
            }, (ej.isAndroid() ? 200 : 0));
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        _setRenderMode: function (renderMode) {
            this.element.removeClass("e-m-android e-m-ios7 e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
        },
        _setIsRelative: function (relative) {
            this.element.removeClass("e-m-abs e-m-rel").addClass("e-m-" + (relative ? "rel" : "abs"));
        },
        _setTitle: function () {
            this._titleSpan.text(this.model.title);
        },
        _setTemplateID: function () {
            this._renderNavigationBarTemplate();
        },
        _setTitleAlignment: function () {
            this._titleSpan.removeClass("e-m-title-left e-m-title-center e-m-title-right").addClass("e-m-title-" + this.model.titleAlignment);
        },
        _setIconAlignment: function () {
            this._navContainer.removeClass("e-m-split-icons e-m-group-icons").addClass("e-m-" + this.model.iconAlignment + "-icons");
            this._iconWidth();
        },
        _setLocale: function () {
            this._getLocalizedLabels();
            this.model.title = this._localizedLabels["title"];
            this._setTitle(this.model.title)
        },
        _refresh: function () {
            this._clearElement();
            this.element.addClass("e-m-navbar");
            this._renderControl();
        },
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },
        _destroy: function () {
            this._clearElement();
        },
        _addRemoveRefresh: function () {
            this.element.empty();
            this._renderControl();
            this._wireEvents();
            this._items = this.model.items;
        },

        /*---------------Public Methods---------------*/
        removeItem: function (index) {
            this.model.items = this._items;
            if (index >= 0)
                this.model.items.splice(index, 1);
            this._addRemoveRefresh();
        },
        addItem: function (item, index) {
            this.model.items = this._items;
            (index >= 0) ? this.model.items.splice(index, 0, this._insertItemValue($(item))) : this.model.items.push(this._insertItemValue($(item)));
            this._addRemoveRefresh();
        },
        disableItem: function (index) {
            if (index >= 0)
                $(this._navbarIcons[index]).addClass("e-m-state-disabled");
        },
        enableItem: function (index) {
            if (index >= 0)
                $(this._navbarIcons[index]).removeClass("e-m-state-disabled");
        },
        hideItem: function (index) {
            if (index >= 0)
                $(this._navbarIcons[index]).addClass("e-m-hide");
        },
        showItem: function (index) {
            if (index >= 0)
                $(this._navbarIcons[index]).removeClass("e-m-hide");
        },
        getTitle: function () {
            return this._titleSpan.text();
        },
        hide: function () {
            this.element.addClass("e-m-hide");
        },
        show: function () {
            this.element.removeClass("e-m-hide");
        }
        /*---------------Public Methods End---------------*/

    });

    ej.mobile.NavigationBar.Position = {
        Top: "top",
        Bottom: "bottom"
    };

    ej.mobile.NavigationBar.Mode = {
        Header: "header",
        Toolbar: "toolbar"
    };

    ej.mobile.NavigationBar.TitleAlignment = {
        Left: "left",
        Center: "center",
        Right: "right"
    };

    ej.mobile.NavigationBar.IconAlignment = {
        Split: "split",
        Group: "group"
    };
    var navBarLocaleObj = {
        title: null
    }
    ej.mobile.NavigationBar.Locale = ej.mobile.NavigationBar.Locale || {};
    ej.mobile.Header.Locale = ej.mobile.Header.Locale || {};
    ej.mobile.Footer.Locale = ej.mobile.Footer.Locale || {};
    ej.mobile.Toolbar.Locale = ej.mobile.Toolbar.Locale || {};
    ej.mobile.NavigationBar.Locale["default"] = ej.mobile.NavigationBar.Locale["en-US"] =
    ej.mobile.Header.Locale["default"] = ej.mobile.NavigationBar.Locale["en-US"] =
    ej.mobile.Footer.Locale["default"] = ej.mobile.NavigationBar.Locale["en-US"] =
    ej.mobile.Toolbar.Locale["default"] = ej.mobile.NavigationBar.Locale["en-US"] =
    navBarLocaleObj;

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ProgressBar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejmProgress", "ej.mobile.Progress", {
        _setFirst: true,
        _rootCSS: "e-m-progress",

        defaults: {
            renderMode: "auto",
            value: 0,
            percentage: 0,
            incrementStep: 0,
            minValue: 0,
            maxValue: 100,
            width: null,
            height: null,
            orientation: "horizontal",
            create: null,
            change: null,
            start: null,
            complete: null,
            text: "",
            enabled: true,
            enableCustomText: false,
            theme: "auto",
            enablePersistence: false
        },
        dataTypes: {
            renderMode: "enum",
            orientation: "enum",
            enabled: "boolean",
            enableCustomText: "boolean",
            theme: "enum",
            enablePersistence: "boolean"
        },

        // constructor function
        _init: function () {
            if (!App.activePage)
                App.createAppView();
            this._getLocalizedLabels();
            this.model.text = !ej.isNullOrUndefined(this.model.text) ? this.model.text : this._localizedLabels["text"];
            this._cloneElement = this.element.clone();
            this._createDelegate();
            this._wireEvents(false);
            this._renderControl();
            this._progressbarInit();
            this._pageLoad();
        },
        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale);
        },
        _renderControl: function () {

            ej.setRenderMode(this);
            ej.setTheme(this);
            var canvas = ej.buildTag("canvas");
            this.element.append(canvas);
        },
        _createDelegate: function () {
            this._onOrientationChangedDelegate = $.proxy(this._onOrientationChangedHandler, this);
            this._onWindowsResizeDelegate = $.proxy(this._onWindowResizeHandler, this);
        },
        _wireEvents: function (remove) {
            if (!remove)
                ej.listenEvents([window], ["onorientationchange" in window ? "orientationchange" : "resize"], [this._onWindowsResizeDelegate], remove);
        },
        _onWindowResizeHandler: function () {
            var proxy = this;
            window.setTimeout(function () {
                proxy._draw();
            }, 250);
        },
        _widthAndHeightProperties: function () {
            this._tempHeight = this.model.orientation == "horizontal" ? (this.model.height ? this.model.height : 3) : (this.model.width ? this.model.width : this.element.parent().height());
            this._tempWidth = this.model.orientation == "horizontal" ? (this.model.width ? this.model.width : this.element.parent().width()) : (this.model.height ? this.model.height : 3);
        },
        _progressbarInit: function () {

            var span;
            this._initialX = 0;
            this._initialY = 0;
            this._mProgress = this.element;
            this._increment = 0;
            this._result = 0;
            this._context = null;
            this._text = this.model.textStyle;
            this._externalText = null;
            this._totalWidth = 0;
            this._totalHeight = 0;
            this._radius = 0;
            this._currentValue = 1;
        },
        _pageLoad: function () {

            var elem = this.element.find("canvas")[0];
            var parent = $(elem).parent();
            if (typeof window.G_vmlCanvasManager != "undefined")
                elem = window.G_vmlCanvasManager.initElement(elem);
            if (!elem || !elem.getContext)
                return;
            var elem = this.element.find("canvas")[0];
            this._context = elem.getContext('2d');
            if (!this._context)
                return;
            this.model.value = this.model.value + this.model.incrementStep;
            this._validate();
            var data = { element: this.element, value: this.model.value };
            this._result = this._draw();
            var data = { value: this._setValue, percentage: this.model.percentage };
            if (this.model.minValue == this.model.value)
                this._trigger("start", data);
        },
        _validate: function () {
            if (this._setValue == this.model.maxValue) {
                var data = { value: this._setValue, text: this.model.text, percentage: this.model.percentage };
                if (this.model.complete)
                    this._trigger("complete", data);
            }
            this._setValue = this._setValue < this.model.minValue ? this.model.minValue : (this._setValue > this.model.maxValue ? this.model.maxValue : this._setValue);
            if (this._step > this._max)
                this._step = 1;
        },
        _draw: function () {
            this._widthAndHeightProperties();
            if (this._context != null) {
                if (!this.model.enabled) {
                    $(this.element).addClass("e-m-state-disabled");
                    this._wireEvents(true);
                }
                if (this.model.value <= 0 && this.model.percentage != 0)
                    this.model.value = Math.round((this.model.percentage * this.model.maxValue) / 100);
                else if (this.model.value != 0 && this.model.percentage <= 0)
                    this.model.percentage = Math.round((this.model.value / this.model.maxValue) * 100);
                else if (this.model.value != 0 && this.model.percentage != 0)
                    this.model.percentage = Math.round((this.model.value / this.model.maxValue) * 100);
                if ((this.model.minValue <= this.model.value) && (this.model.maxValue >= this.model.value)) {
                    this._cutOffValue = this.model.value - this.model.minValue;
                    if (this.model.percentage >= 100)
                        this.model.percentage = 100;
                    else if (this.model.percentage <= 0)
                        this.model.percentage = 0;
                    if (this.model.percentage != undefined)
                        this._setValue = this.model.percentage;
                    else
                        this._setValue = Math.round((100 / (this.model.maxValue - this.model.minValue)) * this._cutOffValue);
                }
                else if ((this.model.minValue > this.model.value) && (this.model.maxValue > this.model.value))
                    this._setValue = 0;
                else if ((this.model.minValue < this.model.value) && (this.model.maxValue < this.model.value))
                    this._setValue = 100;
                this._setOrientationProperties(this._tempWidth, this._tempHeight, this._tempHeight + 50, this._tempWidth + 50);
                this._context.translate(0.5, 0.5);
                // Clear everything before drawing
                this._context.clearRect(this._initialX - 5, this._initialY - 5, this._totalWidth + 15, this._totalHeight + 15);
                var value;
                value = (this.model.renderMode == "windows" || this.model.renderMode == "flat") ? 10 : 25;
                this._initialY = (this.model.orientation == "horizontal") ? value : 0;
                this._initialX = (this.model.orientation == "horizontal") ? 0 : value;
                if (this.model.renderMode == "ios7") {
                    if (this.model.orientation == "horizontal")
                        this._totalHeight = (this._tempHeight == "") ? 3 : this._tempHeight;
                    else
                        this._totalWidth = (this._tempWidth == "") ? 3 : this._tempWidth;
                    this.totalRadius = (this.model.orientation == "horizontal") ? Math.round(this._totalHeight / 2) : Math.round(this._totalWidth / 2);
                    this._fillRadius = this.totalRadius;
                }
                else {
                    if (this.model.orientation == "horizontal")
                        this._totalHeight = (this._tempHeight == "") ? 2 : this._tempHeight;
                    else
                        this._totalWidth = (this._tempWidth == "") ? 2 : this._tempWidth;
                    this.totalRadius = 0; //default radius value for other modes
                    this._fillRadius = 0;
                }
                (this.model.orientation == "horizontal") ?
                this._roundRectOuter(this._context, this._initialX, this._initialY, this._totalWidth - 15, this._totalHeight, this.totalRadius) :
                this._roundRectOuter(this._context, this._initialX, this._initialY, this._totalWidth, this._totalHeight - 15, this.totalRadius);
                (this.model.orientation == "horizontal") ?
                this._roundRectFill(this._context, this._initialX, this._initialY, this._increment, this._totalHeight, this._fillRadius) :
                this._roundRectFill(this._context, this._initialX, this._initialY, this._increment, this._totalHeight - 15, this._fillRadius);
                if (this._increment >= this._totalWidth)
                    window.clearInterval(this._result);
            }
            this._validate();
        },
        _setOrientationProperties: function (horLength, verLength, canvasHeight, canvasWidth) {
            this._totalWidth = horLength;
            this._totalHeight = verLength;
            this._radius = (this.model.orientation == "horizontal" ? this._totalHeight : this._totalWidth) / 2;
            this._increment = (this._setValue) * ((this.model.orientation == "horizontal") ? (this._totalWidth - 15) / 100 : (this._totalHeight - 15) / 100);
            this._context.canvas.width = (this.model.orientation == "horizontal") ? horLength : canvasWidth;
            this._context.canvas.height = (this.model.orientation == "horizontal") ? canvasHeight : verLength;
        },

        _setLocale: function () {
            this._getLocalizedLabels();
            this._setText(this._localizedLabels["text"])
        },

        _roundRect: function (ctx, x, y, width, height, radius) {
            ctx.save();
            ctx.beginPath();
            ctx.strokeStyle = this._strokeStyle;
            ctx.fillStyle = this._fillStyle;
            ctx.lineWidth = 1;
            ctx.moveTo(x + radius, y);
            ctx.lineTo(x + width - radius, y);
            ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
            ctx.lineTo(x + width, y + height - radius);
            ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
            ctx.lineTo(x + radius, y + height);
            ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
            ctx.lineTo(x, y + radius);
            ctx.quadraticCurveTo(x, y, x + radius, y);
            ctx.fill();
            ctx.stroke();
            ctx.closePath();
            ctx.restore();
        },
        _roundRectOuter: function (ctx, x, y, width, height, radius) {
            if (this.model.renderMode == "ios7") {
                var gradientColor = (this.model.theme == 'dark') ? [{ "ColorStop": 1, "Color": "#363636" }] : [{ "ColorStop": 1, "Color": "#b5b5b6" }];
                grd = (this.model.orientation == "horizontal") ? ctx.createLinearGradient(x, y, x, y + height) : ctx.createLinearGradient(x, y, x + width, y);
                ej._setGradientColor(this, grd, gradientColor);
                this._strokeStyle = grd;
                this._fillStyle = grd;
            }
            else if (this.model.renderMode == "android")
                this._setRectOuterProperties("#3A3A3A", "#A9A9A9");
            else if (this.model.renderMode == "windows")
                this._setRectOuterProperties("#373737", "#c5c5c5");
            else if (this.model.renderMode == "flat")
                this._setRectOuterProperties("#b3b3b3", "#b3b3b3");
            (this.model.orientation == "horizontal") ? this._roundRect(ctx, x, y, width + radius, height, radius) : this._roundRect(ctx, x + 5, y, width, height + radius, radius);
        },
        _setRectOuterProperties: function (dark, light) {
            if (this.model.theme == "dark") {
                this._strokeStyle = dark;
                this._fillStyle = dark;
            }
            else {
                this._strokeStyle = light;
                this._fillStyle = light;
            }

        },
        _roundRectFill: function (ctx, x, y, width, height, radius) {//for positioning the progressbar
            if (this.model.renderMode == "ios7") {
                var gradientColor = [{ "ColorStop": 0, "Color": "#017aff" }];
                grd = (this.model.orientation == "horizontal") ? ctx.createLinearGradient(x, y, x, y + height) : ctx.createLinearGradient(x, y, x + width, y);
                ej._setGradientColor(this, grd, gradientColor);
                this._strokeStyle = grd;
                this._fillStyle = grd;
            }
            else if (this.model.renderMode == "android")
                this._strokeStyle = this._fillStyle = "#33B5E5";
            else if (this.model.renderMode == "windows")
                this._strokeStyle = this._fillStyle = "#2ca1dd";
            else if (this.model.renderMode == "flat")
                this._strokeStyle = this._fillStyle = "#f48b22";
            if (width > 0)
                (this.model.orientation == "horizontal") ? this._roundRect(ctx, x, y, width + radius, height, radius) : this._roundRect(ctx, x + 5, height - width + y, this._totalWidth, width + radius, radius);
            ctx.save();
            if (this.model.orientation == "horizontal")
                this._setTextProperties();
            else {
                this._setTextProperties();
                var ang = 270;
                ctx.save();
                ctx.translate(this._totalWidth / 2, height / 2);
                ctx.rotate(ang * 2 * 3.1416 / 360);
                if (this.model.renderMode == "ios7")
                    ctx.textAlign = "center";
                else if (this.model.renderMode == "android")
                    ctx.textAlign = "right";
            }
            if (this.model.enableCustomText) {
                if (this.model.text == "" || this.model.text == undefined)
                    this.setCustomText(this._text);
                else
                    this.setCustomText(this.model.text);
            }
            else
                this.setCustomText(this._text);
            var data = { value: this._setValue, text: this.model.text, percentage: this.model.percentage };
            if (this.model.change && this.model.value > 0)
                this._trigger("change", data);
            ctx.restore();
        },
        _setTextProperties: function () {
            if (this.model.renderMode == "ios7")
                this._text = "Downloading " + this._setValue + " of 100%";
            else if (this.model.renderMode == "android")
                this._text = this._setValue + "%";
            else if (this.model.renderMode == "windows" || this.model.renderMode == "flat")
                this._text = this._setValue;
        },
        _setCustomtextProperties: function (context, dark, light, font) {
            context.fillStyle = this.model.theme == "dark" ? dark : light;
            context.font = font;
        },

        _setRenderMode: function (mode) {
            if (this.model.enabled) {
                this.model.renderMode = mode;
                if (mode == "auto")
                    ej.setRenderMode(this);
                this._draw();
            }
        },

        _setTheme: function (theme) {
            if (this.model.enabled) {
                this.model.theme = theme;
                this._draw();
            }
        },

        _value: function (value) {
            if (this.model.enabled) {
                this._setValue = value;
                this.model.percentage = Math.round((this.model.value * this.model.maxValue) / 100);
                this._draw();
            }
        },

        _setPercentage: function (percentage) {
            if (this.model.enabled) {
                this._value = percentage;
                this._draw();
            }
        },

        _disable: function () {
            this.model.enabled = false;
            this.element.addClass('e-m-state-disabled');
            this._wireEvents(true);
        },

        _enable: function () {
            this.model.enabled = true;
            this.element.removeClass('e-m-state-disabled');
            this._wireEvents(false);
        },

        _refresh: function () {
            this._clearElements();
            this._wireEvents(false);
            this._renderControl();
            this._progressbarInit();
            this._pageLoad();
        },

        _setWidth: function (width) {
            this._tempWidth = this.model.width = width;
            this._draw();
        },

        _setHeight: function (height) {
            this._tempHeight = this.model.height = height;
            this._draw();
        },

        _setText: function (text) {
            if (this.model.enabled) {
                this.model.text = text;
                this._draw();
            }
        },
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "renderMode": this._setRenderMode(options.renderMode); break;
                    case "theme": (this.model.enabled) ? this._setTheme(options.theme) : ""; break;
                    case "width": (this.model.enabled) ? this._setWidth(options.width) : ""; break;
                    case "height": (this.model.enabled) ? this._setHeight(options.height) : ""; break;
                    case "orientation": (this.model.enabled) ? this._draw() : ""; break;
                    case "value": (this.model.enabled) ? this._value(options.value) : ""; break;
                    case "percentage": (this.model.enabled) ? this._setPercentage(options.percentage) : ""; break;
                    case "minValue": (this.model.enabled) ? this._draw() : ""; break;
                    case "maxValue": (this.model.enabled) ? this._draw() : ""; break;
                    case "enabled": (options.enabled) ? this._enable() : this._disable(); break;
                    case "text": (this.model.enabled) ? this._setText(options.text) : ""; break;
                }
            }
        },
        _clearElements: function () {
            this._wireEvents(true);
            this.element.removeAttr("class");
            this.element.removeClass('e-m-progress');
            this.element.html("");
        },
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this._cloneElement.insertBefore(this.element).removeClass('e-m-progress');
            this.element.remove();
        },

        /*---------------Public Methods---------------*/

        getValue: function () {
            return this._setValue;
        },

        getPercentage: function () {
            /// <summary>Returns the Progress bar value in percentage.</summary>
            return this.model.percentage;
        },

        setCustomText: function (customText) {
            var context = this._context, height = this._totalHeight, radius = this.totalRadius, width = this._totalWidth, x, y;
            this.model.text = customText;
            var ang = 270;
            if (this.model.renderMode == "ios7") {
                x = (this.model.orientation == "horizontal") ? 0 : 8;
                y = (this.model.orientation == "horizontal") ? 8 : 0;
            }
            else if (this.model.renderMode == "android")
                y = 15;
            else if (this.model.renderMode == "windows" || this.model.renderMode == "flat") {
                x = 0;
                y = 30;
            }
            context.save();
            if (this.model.renderMode == "ios7")
                this._setCustomtextProperties(context, "#fefefe", "#000000", "14px Helvetica");
            else if (this.model.renderMode == "android")
                this._setCustomtextProperties(context, "#fefefe", "#333333", "17px Roboto");
            else if (this.model.renderMode == "windows")
                this._setCustomtextProperties(context, "#ffffff", "#000000", "17px Segoe UI");
            else if (this.model.renderMode == "flat")
                this._setCustomtextProperties(context, "#333333", "#333333", "17px Segoe UI");
            var textWidth = context.measureText(this.model.text).width;
            var def_wid = context.measureText(this._setValue + "%").width;
            if (this.model.orientation == "horizontal") {
                var textX = x + width / 2 - textWidth / 2;
                if (this.model.renderMode == "ios7")
                    context.fillText(this.model.text, textX, y + 5);
                else if (this.model.renderMode == "android") {
                    if (this.model.text != "" && this.model.text != undefined)
                        context.fillText(this.model.text, width - textWidth - 15, y);
                    else
                        context.fillText(this.model.text, width - def_wid - 15, y);

                }
                else if (this.model.renderMode == "windows" || this.model.renderMode == "flat")
                    context.fillText(this.model.text, x, height + y);
            }
            else {
                var textX = y + 15;
                if (this.model.renderMode == "ios7")
                    context.fillText(this.model.text, y, textX);
                else if (this.model.renderMode == "android") {
                    if (this.model.text != "" && this.model.text != undefined)
                        context.fillText(this.model.text, (height - 15) / 2, 15);
                    else
                        context.fillText(this.model.text, (height - 15) / 2, y);
                    context.textAlign = "right";
                }
                else if (this.model.renderMode == "windows" || this.model.renderMode == "flat")
                    context.fillText(this.model.text, -(height - 15) / 2, y + width);
            }
            context.restore();
        },
        changeEventHandler: function (proxy, value) {
            var data = { element: this.element, value: this._setValue, percentage: this._value };
            if (this.model.change)
                this._trigger("change", data);
        }
        /*---------------Public Methods End---------------*/
    });


    ej.mobile.Progress.Orientation = {
        Horizontal: "horizontal",
        Vertical: "vertical"
    };
    ej.mobile.Progress.Locale = ej.mobile.Progress.Locale || {};

    ej.mobile.Progress.Locale["default"] = ej.mobile.Progress.Locale["en-US"] = {
        text: ""
    }
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Radial Menu elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejRadialMenuBase", "ej.RadialMenuBase", {
        _tags: [{
            tag: "items",
            attr: ["imageName", "imagePath", "imageUrl", "windows.text", "flat.text", "text", "click", "touch", "enabled",
                "badge.enabled", "badge.value", "type", "sliderSettings.ticks", "sliderSettings.strokeWidth", "sliderSettings.labelSpace", [{
                    tag: "items", attr: ["imageName", "imagePath", "imageUrl", "windows.text", "flat.text", "text", "click",
                     "touch", "enabled"]
                }]]
        }],
        defaults: {
            radius: 150,
            cssClass: "",
            enableAnimation: true,
            open: null,
            close: null,
            items: []
        },
        dataTypes: {
            radius: "number",
            enableAnimation: "boolean",
            items: "array"
        },
        _initialization: function () {
            this.element.addClass(this.model.cssClass);
            this._svgLink = "http://www.w3.org/2000/svg";
            if (this.model.items.length < 1) {
                this._radialItems = $(this.element.find(">ul").children());
                for (var i = 0; i < this._radialItems.length ; i++) {
                    this.model.items.push(this._itemsObjectCollection($(this._radialItems[i])));
                    if ($(this._radialItems[i]).find("ul").children().length > 0) {
                        this._nestedRadialItems = $(this._radialItems[i]).find("ul").children();
                        this.model.items[i].items = [];
                        for (var j = 0; j < this._nestedRadialItems.length; j++) {
                            this.model.items[i].items.push(this._itemsObjectCollection($(this._nestedRadialItems[j])));
                        }
                    }
                }
            }
            this._itemCount = this.model.items.length;
            $(this.element.children()).remove();
            this._startXY = this.model.radius;
            this._diameter = 2 * this.model.radius;
        },
        _itemsObjectCollection: function (element) {
            var item = {}, innerItem = {};
            $(this._tags[0].attr).each(function (i, e) {
                if (typeof (e) == "string") {
                    var attr = e.split('.');
                    if (attr.length == 1)
                        item[e] = element.attr("data-ej-" + e);
                    else {
                        var prop1 = attr[0], prop2 = attr[1];
                        if (!item[prop1])
                            innerItem = {};
                        innerItem[prop2] = element.attr("data-ej-" + e.replace(".", "-"));
                        item[prop1] = innerItem;
                    }
                }
            });
            return item;
        },
        _createSVGElement: function () {
            var svgObj = document.createElementNS(this._svgLink, 'svg');
            if (this.model.radius != null) {
                svgObj.setAttribute("width", this._diameter);
                svgObj.setAttribute("height", this._diameter);
            }
            return $(svgObj);
        },
        _createGroupElement: function (id) {
            var group = document.createElementNS(this._svgLink, "g");
            if (id)
                group.setAttribute("id", id);
            return $(group);
        },
        _createImageElement: function (width, height, currentEle, x, y) {
            if (!ej.isNullOrUndefined(currentEle["imageUrl"]) || !ej.isNullOrUndefined(currentEle["imagePath"])) {
                imgLoc = currentEle["imageUrl"] ? currentEle["imageUrl"] : (this.model.renderMode.toLowerCase() != "flat" ? currentEle["imagePath"] + "/"
                    + this.model.renderMode.toLowerCase() + "/" + currentEle["imageName"] : currentEle["imagePath"] + "/" + this.model.renderMode.toLowerCase() + "/" + currentEle["imageName"]),
                image = document.createElementNS(this._svgLink, "image");
                image.setAttribute("width", width);
                image.setAttribute("height", height);
                image.setAttributeNS('http://www.w3.org/1999/xlink', 'href', imgLoc);
                image.setAttribute("x", x);
                image.setAttribute("y", y);
                return image;
            }
        },
        _createTextElement: function (x, y, style, content) {
            var text = document.createElementNS(this._svgLink, "text");
            text.setAttribute("x", x);
            text.setAttribute("y", y);
            text.setAttribute("class", style);
            text.textContent = content;
            return text;
        },
        _createPathElement: function (index, direction, style) {
            var path = document.createElementNS(this._svgLink, "path");
            path.setAttribute("index", index);
            path.setAttribute("d", direction);
            path.setAttribute("class", style);
            return path;
        },
        _createPolygonElement: function (index, transform) {
            var polygon = document.createElementNS(this._svgLink, "polygon");
            polygon.setAttribute("points", "10,10 0,10 5,5");
            polygon.setAttribute("index", index);
            polygon.setAttribute("fill", "#FFFFFF");
            polygon.setAttribute("transform", transform);
            return polygon;
        },
        _createCircleElement: function (x, y, r, style) {
            var circle = document.createElementNS(this._svgLink, "circle");
            circle.setAttribute("cx", x);
            circle.setAttribute("cy", y);
            circle.setAttribute("r", r);
            circle.setAttribute("class", style);
            return circle;
        },
        _calculateRadialArc: function (menuCount, size, mRadius, Arcdradius, polyRadius) {
            var itemCurrentPos = 0,
                startAngle = 2 * Math.PI * itemCurrentPos,
                endAngle = 2 * Math.PI * (itemCurrentPos + (10 / (menuCount * 10))),
                radius = 0.5 * size * Math.min(this._diameter, this._diameter),
                menuItemRadius = mRadius * size * Math.min(this._diameter, this._diameter),
                dradius = Arcdradius * radius,
                polygonRadius = polyRadius * radius,
                degPoint = 360 / menuCount,
                degree = degPoint / 2;
            var x1 = [], y1 = [], x2 = [], y2 = [], midx = [], midy = [], dStartX = [], dStartY = [], dEndX = [], dEndY = [], dmidx = [], dmidy = [], deg = [], longArc = [];
            for (var i = 0; i < menuCount; i++) {
                var pathStartAngle = -.5 * Math.PI;
                startAngle = startAngle + pathStartAngle;
                endAngle = endAngle + pathStartAngle - 0.000001;
                longArc[i] = endAngle - startAngle < Math.PI ? 0 : 1;
                var midAngle = (startAngle + endAngle) / 2;
                x1[i] = this._startXY + radius * Math.cos(startAngle);
                y1[i] = this._startXY + radius * Math.sin(startAngle);
                x2[i] = this._startXY + radius * Math.cos(endAngle);
                y2[i] = this._startXY + radius * Math.sin(endAngle);
                midx[i] = this._startXY + menuItemRadius * Math.cos(midAngle);
                midy[i] = this._startXY + menuItemRadius * Math.sin(midAngle);
                dStartX[i] = this._startXY + dradius * Math.cos(startAngle);
                dStartY[i] = this._startXY + dradius * Math.sin(startAngle);
                dEndX[i] = this._startXY + dradius * Math.cos(endAngle);
                dEndY[i] = this._startXY + dradius * Math.sin(endAngle);
                dmidx[i] = this._startXY + polygonRadius * Math.cos(midAngle);
                dmidy[i] = this._startXY + polygonRadius * Math.sin(midAngle);
                deg[i] = degree;
                itemCurrentPos += 10 / (menuCount * 10);
                startAngle = 2 * Math.PI * itemCurrentPos;
                endAngle = 2 * Math.PI * (itemCurrentPos + (10 / (menuCount * 10)));
                degree += degPoint;
            }
            return ({
                x1: x1, y1: y1, x2: x2, y2: y2, midx: midx, midy: midy, dStartX: dStartX, dStartY: dStartY, dEndX: dEndX, dEndY: dEndY, dmidx: dmidx,
                dmidy: dmidy, deg: deg, longArc: longArc, radius: radius, dradius: dradius
            });
        },
        _renderRadialMenu: function () {
            this._radialSVG = this._createSVGElement();
            this._radialChildSVG = this._createSVGElement();
            this._childArcGroup = this._createGroupElement("childarcgroup");
            this._polygonGroup = this._createGroupElement("polygongroup");
            this._polygonGroup.attr("transform", "translate(-10,-10)");
            this._radialArcGroup = this._createGroupElement("outerarcpathgroup");
            this._menuItemsArcGroup = this._createGroupElement("arcpathgroup");
            this._menuItemsGroup = this._createGroupElement("menuitemsgroup");
            this._menuItemsGroup.attr("transform", "translate(-10,-5)");
            this._menuItemsArcPaths = this._createGroupElement("menuitemsarcpaths");
            this._menuItemsPaths = this._createGroupElement("itemspathgroup");
            this._radialSVGDiv = ej.buildTag("div#" + this._id + "_svgdiv." + this._prefix + "abs").css("height", this._diameter);
            this._radialCircleSVGDiv = ej.buildTag("div#" + this._id + "_circlesvgdiv." + this._prefix + "abs " + this._prefix + "displaynone");
            this._radialChildSVGDiv = ej.buildTag("div#" + this._id + "_childsvgdiv." + this._prefix + "abs " + this._prefix + "displaynone");
            var pointsVal = this._calculateRadialArc(this._itemCount, 1, 0.28, 0.8, 0.95),
                pointsArcVal = this._calculateRadialArc(this._itemCount, 0.79, null, 0.97, null),
                pointsItemVal = this._calculateRadialArc(this._itemCount, 0.8, null, 0.3, null),
                pointsOuterArcVal = this._calculateRadialArc(1, 1, 0.28, 0.8, null),
                outerArcDirection = this._pathDirection(pointsOuterArcVal.x1[0], pointsOuterArcVal.y1[0], pointsOuterArcVal.radius, pointsOuterArcVal.longArc[0], pointsOuterArcVal.x2[0], pointsOuterArcVal.y2[0],
                                    pointsOuterArcVal.dEndX[0], pointsOuterArcVal.dEndY[0], pointsOuterArcVal.dradius, pointsOuterArcVal.dStartX[0], pointsOuterArcVal.dStartY[0]),
                imgDim = 20;
            for (var i = 0; i < this._itemCount; i++) {
                var direction = this._pathDirection(pointsVal.x1[i], pointsVal.y1[i], pointsVal.radius, pointsVal.longArc[i], pointsVal.x2[i], pointsVal.y2[i],
                    pointsVal.dEndX[i], pointsVal.dEndY[i], pointsVal.dradius, pointsVal.dStartX[i], pointsVal.dStartY[i]),
                    arcDirection = this._pathDirection(pointsArcVal.x1[i], pointsArcVal.y1[i], pointsArcVal.radius, pointsArcVal.longArc[i], pointsArcVal.x2[i],
                    pointsArcVal.y2[i], pointsArcVal.dEndX[i], pointsArcVal.dEndY[i], pointsArcVal.dradius, pointsArcVal.dStartX[i], pointsArcVal.dStartY[i]),
                    itemDirection = this._pathDirection(pointsItemVal.x1[i], pointsItemVal.y1[i], pointsItemVal.radius, pointsItemVal.longArc[i], pointsItemVal.x2[i],
                    pointsItemVal.y2[i], pointsItemVal.dEndX[i], pointsItemVal.dEndY[i], pointsItemVal.dradius, pointsItemVal.dStartX[i], pointsItemVal.dStartY[i]),
                    currentEle = this.model.items[i],
                    imgTextGroup = this._createGroupElement(),
                    itemEnable = this._enableItem(currentEle, imgTextGroup);
                this._menuItemsArcPaths.append(this._createPathElement(i, arcDirection, "" + this._prefix + "arcbgcolor"));
                this._menuItemsPaths.append(this._createPathElement(i, itemDirection, itemEnable.itemPathClass));
                this._menuItemsArcGroup.append(this._createPathElement(i, direction, itemEnable.arcPathClass));
                imgTextGroup.attr({ "cursor": "context-menu", "index": i });
                imgTextGroup.append(this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i], pointsVal.midy[i] - imgDim / 2));
                imgTextGroup.append(this._createTextElement(pointsVal.midx[i] + 10, pointsVal.midy[i] + 25, itemEnable.textClass, itemEnable.dataText));
                if (currentEle.badge && (currentEle.badge.enabled == "true" || currentEle.badge.enabled == true))
                    imgTextGroup.append(this._createCircleElement(pointsVal.midx[i] + 25, pointsVal.midy[i] - 10, 10, "" + this._prefix + "badgecircle"))
                                .append(this._createTextElement(pointsVal.midx[i] + 25, pointsVal.midy[i] - 7, "" + this._prefix + "badgetext", currentEle.badge.value));
                this._menuItemsGroup.append(imgTextGroup);
                if (currentEle["items"] || (currentEle.type && currentEle.type.toLowerCase() == (this.model.renderMode ? ej.mobile : ej).RadialMenu.ItemType.Slider)) {
                    var transform = pointsVal.deg[i] > 270 ? "translate(" + (pointsVal.dmidx[i] + 5) + "," + (pointsVal.dmidy[i] + 15) + ")" + "rotate(" + pointsVal.deg[i] + ")"
                        : "translate(" + (pointsVal.dmidx[i] + 10) + "," + (pointsVal.dmidy[i] + 10) + ")" + "rotate(" + pointsVal.deg[i] + ")";
                    this._childArcGroup.append(this._createPathElement(i, direction, "" + this._prefix + "childdefault"));
                    this._polygonGroup.append(this._createPolygonElement(i, transform));
                }
            }
            var circle = this._createCircleElement(this._startXY, this._startXY, this._startXY / 3.5, "" + this._prefix + "circlebgcolor");
            this._radialArcGroup.append(this._createPathElement(1, outerArcDirection, "" + this._prefix + "outerdefault"));
            this._radialChildSVG.append(this._menuItemsPaths).append(this._menuItemsArcGroup).append(this._childArcGroup)
                                .append(this._polygonGroup).append(this._menuItemsArcPaths).append(this._menuItemsGroup).append(circle);
            this._radialContainer.append(this._radialSVGDiv.append(this._radialCircleSVGDiv.append(this._radialSVG.append(this._radialArcGroup)))
                                 .append(this._radialChildSVGDiv.append(this._radialChildSVG)));
        },
        _ejMenuBaseItemsRemove: function () {
            this._menuChildSVGDiv = ej.buildTag("div#" + this._id + "_menuchildsvgdiv." + this._prefix + "abs");
            if (this.model.enableAnimation) {
                this._radialChildSVGDiv.removeClass("" + this._prefix + "radialshow").addClass("" + this._prefix + "scalehide");
                this._radialChildAnimate();
                this._menuChildSVGDiv.addClass("" + this._prefix + "scaleshow");
            }
            else
                this._radialChildSVGDiv.addClass("" + this._prefix + "displaynone");
            this._radial.removeClass(this.model.imageClass).addClass("" + this._prefix + "backarrow " + this.model.backImageClass + "");
        },
        _renderRadialChildMenu: function (currentItem) {
            this._menuChildSVG = this._createSVGElement();
            this._childItemsGroup = this._createGroupElement("childitemsgroup");
            this._childItemsGroup.attr("transform", "translate(-10,-10)");
            this._childItemArcGroup = this._createGroupElement();
            this._childItemsArcPaths = this._createGroupElement("childitemsarcpaths");
            this._childItemsPaths = this._createGroupElement();
            var childCount = currentItem.items.length,
                pointsVal = this._calculateRadialArc(childCount, 1, 0.28, 0.8, null),
                pointsArcVal = this._calculateRadialArc(childCount, 0.79, null, 0.97, null),
                pointsItemVal = this._calculateRadialArc(childCount, 0.8, null, 0.3, null),
                imgDim = 20;
            for (var i = 0; i < childCount; i++) {
                var direction = this._pathDirection(pointsVal.x1[i], pointsVal.y1[i], pointsVal.radius, pointsVal.longArc[i], pointsVal.x2[i], pointsVal.y2[i],
                    pointsVal.dEndX[i], pointsVal.dEndY[i], pointsVal.dradius, pointsVal.dStartX[i], pointsVal.dStartY[i]),
                    arcDirection = this._pathDirection(pointsArcVal.x1[i], pointsArcVal.y1[i], pointsArcVal.radius, pointsArcVal.longArc[i], pointsArcVal.x2[i],
                    pointsArcVal.y2[i], pointsArcVal.dEndX[i], pointsArcVal.dEndY[i], pointsArcVal.dradius, pointsArcVal.dStartX[i], pointsArcVal.dStartY[i]),
                    itemDirection = this._pathDirection(pointsItemVal.x1[i], pointsItemVal.y1[i], pointsItemVal.radius, pointsItemVal.longArc[i], pointsItemVal.x2[i],
                    pointsItemVal.y2[i], pointsItemVal.dEndX[i], pointsItemVal.dEndY[i], pointsItemVal.dradius, pointsItemVal.dStartX[i], pointsItemVal.dStartY[i]),
                    imgTextGroup = this._createGroupElement(),
                    currentEle = currentItem.items[i],
                    itemEnable = this._enableItem(currentEle, imgTextGroup);
                this._childItemArcGroup.append(this._createPathElement(i, direction, itemEnable.arcPathClass));
                this._childItemsArcPaths.append(this._createPathElement(i, arcDirection, "" + this._prefix + "arcbgcolor"));
                this._childItemsPaths.append(this._createPathElement(i, itemDirection, itemEnable.itemPathClass));
                imgTextGroup.attr({ "cursor": "context-menu", "index": i });
                imgTextGroup.append(this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i], pointsVal.midy[i] - imgDim / 2));
                imgTextGroup.append(this._createTextElement(pointsVal.midx[i] + 10, pointsVal.midy[i] + 25, itemEnable.textClass, itemEnable.dataText));
                this._childItemsGroup.append(imgTextGroup);
            }
            var circle = this._createCircleElement(this._startXY, this._startXY, this._startXY / 3.5, "" + this._prefix + "circlebgcolor");
            this._radialSVGDiv.append(this._menuChildSVGDiv.append(this._menuChildSVG.append(this._childItemsPaths)
                              .append(this._childItemArcGroup).append(this._childItemsArcPaths).append(this._childItemsGroup).append(circle)));
        },
        _radialSliderHandler: function (e) {
            this.updateBadgeValue(this._index, e.value);
            this._data = { value: e.value, oldValue: e.oldValue, index: this._index };
            this._eventsTrigger(this._data, this.model.renderMode ? "touch" : "click");
        },
        _isRSText: function (eleClass) {
            return ((eleClass == this._prefix + "ticks-text" || eleClass == this._prefix + "dynamic-text") ? true : false);
        },
        _pathDirection: function (x1, y1, radius, lArc, x2, y2, dEndX, dEndY, dRadius, dStartX, dStartY) {
            return "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + lArc + " " + "1" + " " + x2 + " " + y2 + " " + "L" +
                " " + dEndX + " " + dEndY + " " + "A" + " " + dRadius + " " + dRadius + " " + "1" + " " + lArc + " " + "0" + " " + dStartX + " " + dStartY + " " + "z";
        },
        _enableItem: function (currentObj, ele) {
            var itemPathClass = this._prefix + "itembgcolor", arcPathClass = this._prefix + "default", textClass = "" + this._prefix + "textcolor",
                dataText = this.model.renderMode ? currentObj[this.model.renderMode].text : currentObj.text;
            if (currentObj.enabled == "false" || currentObj.enabled == false) {
                ele.attr("class", this._prefix + "itemdisabled");
                return {
                    itemPathClass: this._prefix + "itembgcolor " + this._prefix + "pathdisabled", arcPathClass: this._prefix + "default " + this._prefix + "pathdisabled",
                    dataText: dataText, textClass: textClass
                };
            }
            else
                return { itemPathClass: itemPathClass, arcPathClass: arcPathClass, dataText: dataText, textClass: textClass };
        },
        _windowsMenuShow: function () {
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "displaynone " + this._prefix + "radialhide").addClass(this._prefix + "radialshow");
                this._radialChildSVGDiv.removeClass(this._prefix + "displaynone " + this._prefix + "radialhide").addClass(this._prefix + "radialshow");
            }
            else {
                this._radialCircleSVGDiv.removeClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.removeClass(this._prefix + "displaynone");
            }
            this._openCloseTrigger("open");
        },
        _windowsMenuHide: function (proxy) {
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "radialshow").addClass(this._prefix + "radialhide");
                this._radialChildSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "scaleshow").addClass(this._prefix + "radialhide");
                setTimeout(function () {
                    proxy._radialCircleSVGDiv.addClass(proxy._prefix + "displaynone");
                    proxy._radialChildSVGDiv.addClass(proxy._prefix + "displaynone");
                }, 160);
            }
            else {
                this._radialCircleSVGDiv.addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.addClass(this._prefix + "displaynone");
            }
            this._openCloseTrigger("close");
        },
        _windowsInnerMenuHide: function (proxy) {
            this._radial.removeClass(this._prefix + "backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
            if (this.model.enableAnimation) {
                this._menuChildSVGDiv.removeClass(this._prefix + "scaleshow").addClass(this._prefix + "scalehide");
                setTimeout(function () { proxy._menuChildSVGDiv.remove(); }, 150);
                setTimeout(function () { proxy._radialChildSVGDiv.removeClass(proxy._prefix + "scalehide " + proxy._prefix + "displaynone").addClass(proxy._prefix + "scaleshow"); }, 200);
            }
            else {
                this._menuChildSVGDiv.remove();
                this._radialChildSVGDiv.removeClass(this._prefix + "displaynone");
            }
        },
        _eventsTrigger: function (data, eventName) {
            var event = this.model[eventName];
            this.model[eventName] = this._childTarget && this.model.items[data.index].items[data.childIndex][eventName] ? this.model.items[data.index].items[data.childIndex][eventName]
                : (!this._childTarget && this.model.items[data.index][eventName] ? this.model.items[data.index][eventName] : this.model[eventName]);
            if (this.model[eventName])
                this._trigger(eventName, data);
            this.model[eventName] = event;
        },
        _openCloseTrigger: function (eventName) {
            if (this.model[eventName])
                this._trigger(eventName);
        },
        _menuEnabledDisabled: function (itemIndices, menuClass) {
            var proxy = this,
                itemIndex;
            if ($.isArray(itemIndices)) {
                $.each(itemIndices, function (i, itemIndex) {
                    if (!$.isArray(itemIndex))
                        proxy._menuItems(proxy, itemIndex, menuClass);
                    else {
                        $.each(itemIndex, function (i, index) {
                            if (!$.isArray(index)) {
                                if (!proxy.model.renderMode || (proxy.model.renderMode && (proxy.model.renderMode == "windows" || proxy.model.renderMode == "flat")))
                                    proxy._menuItems(proxy, index, menuClass);
                                itemIndex = index;
                            }
                            else {
                                $.each(index, function (i, childIndex) {
                                    proxy.model.items[itemIndex].items[childIndex].enabled = (menuClass == "disabled") ? false : true;
                                });
                            }
                        });
                    }
                });
            }
        },
        _ejMenuItem: function (proxy, index, menuClass) {
            var currentGroupEle = $(this._menuItemsGroup.find("g")[index]),
                currentItemPathEle = $(this._menuItemsPaths.find("path")[index]),
                currentArcPathEle = $(this._radialChildSVGDiv.find("#arcpathgroup path")[index]);
            if (menuClass == "disabled") {
                currentItemPathEle.attr("class", "" + this._prefix + "itembgcolor " + this._prefix + "pathdisabled");
                currentArcPathEle.attr("class", "" + this._prefix + "default " + this._prefix + "pathdisabled");
                currentGroupEle.attr("class", "" + this._prefix + "itemdisabled");
				if(!ej.isNullOrUndefined(this._radialChildSVGDiv) && $(this._radialChildSVGDiv.find("#polygongroup polygon[index='"+index+"']")).length > 0) {
					$(this._radialChildSVGDiv.find("#polygongroup polygon[index='"+index+"']")).attr("class",this._prefix + "itemdisabled");
					$(this._radialChildSVGDiv.find("#childarcgroup path[index='"+index+"']")).attr("class",this._prefix + "itemdisabled");
				} 
            }
            else {
                currentItemPathEle.attr("class", "" + this._prefix + "itembgcolor");
                currentArcPathEle.attr("class", "" + this._prefix + "default");
                currentGroupEle.removeAttr("class");
				if(!ej.isNullOrUndefined(this._radialChildSVGDiv) && $(this._radialChildSVGDiv.find("#polygongroup polygon[index='"+index+"']")).length > 0){
					$(this._radialChildSVGDiv.find("#polygongroup polygon[index='"+index+"']")).removeAttr("class");
				}  $(this._radialChildSVGDiv.find("#childarcgroup path[index='"+index+"']")).attr("class",this._prefix + "childdefault");
            }
        },
        _ejMenuHide: function (proxy) {
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "radialshow").addClass(this._prefix + "radialhide");
                this._radialChildSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "scaleshow").addClass(this._prefix + "radialhide");
                setTimeout(function () {
                    proxy._radialCircleSVGDiv.addClass(proxy._prefix + "displaynone").removeClass(proxy._prefix + "radialshow " + proxy._prefix + "radialhide " + proxy._prefix + "scaleshow " + proxy._prefix + "scalehide");
                    proxy._radialChildSVGDiv.addClass(proxy._prefix + "displaynone").removeClass(proxy._prefix + "radialshow " + proxy._prefix + "radialhide " + proxy._prefix + "scaleshow " + proxy._prefix + "scalehide");
                }, 160);
            }
            else {
                this._radialCircleSVGDiv.addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.addClass(this._prefix + "displaynone");
            }
            this._openCloseTrigger("close");
        },
        _ejInnerMenuHide: function (proxy) {
            this._radial.removeClass(this._prefix + "backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "radialhide " + this._prefix + "scaleshow " + this._prefix + "scalehide").addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "radialhide " + this._prefix + "scaleshow " + this._prefix + "scalehide").addClass(this._prefix + "displaynone");
                this._menuChildSVGDiv.removeClass(this._prefix + "scaleshow").addClass(this._prefix + "radialhide");
                setTimeout(function () { proxy._menuChildSVGDiv.removeClass(proxy._prefix + "radialhide").remove(); }, 150);
            }
            else {
                this._radialCircleSVGDiv.addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.addClass(this._prefix + "displaynone");
                this._menuChildSVGDiv.remove();
            }
            this._openCloseTrigger("close");
        },
        _currentBadgeEle: function (index) {
            return this.model.renderMode == "android" ? $(this._imageBadgeGroup.find("g")[index]) : $(this._menuItemsGroup.find("g")[index]);
        },
        _badge: function (index) {
            return this._currentBadgeEle(index).find("." + this._prefix + "badgetext,." + this._prefix + "badgecircle");
        },
        show: function () {
            this.element.removeClass(this._prefix + "displaynone");
        },
        hide: function () {
            this.element.addClass(this._prefix + "displaynone");
        },
        showMenu: function () {
            this._showMenu();
        },
        hideMenu: function () {
            this._hideMenu();
        },
        enableItemByIndex: function (itemIndex) {
            this._menuItems(this, itemIndex, "enabled");
        },
        enableItemsByIndices: function (itemIndices) {
            this._menuEnabledDisabled(itemIndices, "enabled");
        },
        disableItemByIndex: function (itemIndex) {
            this._menuItems(this, itemIndex, "disabled");
        },
        disableItemsByIndices: function (itemIndices) {
            this._menuEnabledDisabled(itemIndices, "disabled");
        },
        updateBadgeValue: function (index, value) {
            this.model.items[index].badge.value = value;
            this._currentBadgeEle(index).children("text." + this._prefix + "badgetext").html(value);
        },
        showBadge: function (index) {
            this._badge(index).show();
        },
        hideBadge: function (index) {
            this._badge(index).hide();
        },
        _setModel: function (options) {
            if (options.items) this.model.items = options.items;
            this._refresh();
        },
        _refresh: function () {
            this._destroy();
            this._init();
        },
        _destroy: function () {
            ej.listenEvents([document, document], [ej.endEvent(), ej.startEvent()], [this._targetClick, this._targetDown], true);
            this.element.removeAttr("class style").children().remove();
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Radial Menu elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejmRadialMenu", "ej.mobile.RadialMenu", {
        _setFirst: true,
        _rootCSS: "e-m-radialmenu",
        defaults: {
            renderMode: "auto",
            imageClass: "e-m-radialimage",
            backImageClass: "e-m-backimage",
            touch: null,
            //Deprecated property
            select:null,
            position: "rightcenter"
        },
        dataTypes: {
            renderMode: "enum",
            position: "enum"
        },
        _init: function () {
            this._initialization();
            this._setValues();
            this._renderRadial();
            this._wireEvents();
            this._id = this.element[0].id;
        },
        _setValues: function () {
            if (!(ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)) {
                ej.setRenderMode(this);
                this._degDis = 0;
                this._totDis = 0;
                this._prefix = "e-m-";
                this._pos = this.model.position;
            }
        },
        _renderRadial: function () {
            this._radial = ej.buildTag("div.e-m-radial e-m-abs " + this.model.imageClass + "").css({ "left": this._startXY - 25 + "px", "top": this._startXY - 25 + "px" });
            this._radialContainer = ej.buildTag("div.e-m-rel").css({ "height": this._diameter, "width": this._diameter });
            this.element.addClass("e-m-radialmenu e-m-overlow e-m-abs e-m-user-select e-m-" + this.model.renderMode + " e-m-radial" + this.model.position + "")
                        .append(this._radialContainer.append(this._radial));
            if (this._pos.charAt(0).toLowerCase() == "r")
                this._radialContainer.addClass("e-m-radialright").css("right", -this._startXY + "px");
            else
                this._radialContainer.css("left", -this._startXY + "px");
            this._posCenter();
            this["_" + this.model.renderMode + "RadialMenu"]();
        },
        _wireEvents: function () {
            this._touchStart = $.proxy(this._touchStartHandler, this);
            this._touchMove = $.proxy(this._touchMoveHandler, this);
            this._touchCanel = $.proxy(this._touchCancelHandler, this);
            this._touchEnd = $.proxy(this._touchEndHandler, this);
            this._radialTouch = $.proxy(this._showMenu, this);
            ej.listenEvents([this._radial, this._radialSVGDiv], [ej.endEvent(), ej.startEvent()], [this._radialTouch, this._touchStart], false);
        },
        _touchCancelHandler: function (e) {
            ele = e.toElement || e.relatedTarget;
            if (!$(ele).hasClass("e-m-radial") && $(ele).attr("class") != ("e-m-radialcircle") && $(ele).attr("class") != "e-m-radialdefault" && !this._radialSVGDiv.find(ele).length) {
                ej.listenEvents([this._radialSVGDiv], ["mouseout", ], [this._touchCanel], true);
                ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
                ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
                this._move = false;
            }
        },
        _ios7RadialMenu: function () {
            this._radialSVG = this._createSVGElement();
            this._menuItemsGroup = this._createGroupElement("menuitemsgroup");
            var childCircleGroup = this._createGroupElement();
            this._radialSVGDiv = ej.buildTag("div#" + this._id + "_svgdiv.e-m-abs").css("height", this._diameter);
            this._radialMenuSVGDiv = ej.buildTag("div#" + this._id + "_menusvgdiv.e-m-abs e-m-displaynone");
            var pointsVal = this._calculateRadialArc(this._itemCount, 1, 0.35, 0.2, 0.95),
                imgDim = 32;
            for (var i = 0; i < this._itemCount; i++) {
                var circleImgGroup = this._createGroupElement(),
                    currentEle = this.model.items[i],
                    imgEle = this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i] - imgDim / 2, pointsVal.midy[i] - imgDim / 2),
                    itemEnable = this._ejmEnableItem(currentEle, imgEle);
                this._menuItemsGroup.append(circleImgGroup
                                   .append(this._createCircleElement(pointsVal.midx[i], pointsVal.midy[i], 22, !itemEnable ? "e-m-radialdefault e-m-pathdisabled" : "e-m-radialdefault"))
                                   .append(imgEle));
                if (currentEle.badge && (currentEle.badge.enabled == "true" || currentEle.badge.enabled == true))
                    circleImgGroup.append(this._createCircleElement(pointsVal.midx[i] + 30, pointsVal.midy[i] - 5, 10, !itemEnable ? "e-m-badgecircle e-m-itemdisabled" : "e-m-badgecircle"))
                                  .append(this._createTextElement(pointsVal.midx[i] + 30, pointsVal.midy[i] - 2, !itemEnable ? "e-m-badgetext e-m-itemdisabled" : "e-m-badgetext", currentEle.badge.value));
                if (currentEle.items || (currentEle.type && currentEle.type.toLowerCase() == ej.mobile.RadialMenu.ItemType.Slider))
                    childCircleGroup.append(this._createCircleElement(pointsVal.midx[i], pointsVal.midy[i], 25, "e-m-chaildcircle"));
            }
            var radialCircle = this._createCircleElement(this._startXY, this._startXY, this._startXY, "e-m-radialcircle");
            this._radialContainer.append(this._radialSVGDiv.append(this._radialMenuSVGDiv.append(this._radialSVG.append(radialCircle)
                                 .append(this._menuItemsGroup).append(childCircleGroup))));
        },
        _ios7ChildRadialMenu: function (child, childCount, index) {
            this._radialChildSVG = this._createSVGElement();
            this._menuChildItemsGroup = this._createGroupElement("menuchilditemsgroup");
            var pointsVal = this._calculateRadialArc(childCount, 1, 0.35, 0.2, 0.95),
                imgDim = 32;
            for (var i = 0; i < childCount; i++) {
                var circleImgGroup = this._createGroupElement(),
                    currentEle = this.model.items[index].items[i],
                    imgEle = this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i] - imgDim / 2, pointsVal.midy[i] - imgDim / 2),
                    itemEnable = this._ejmEnableItem(currentEle, imgEle);
                this._menuChildItemsGroup.append(circleImgGroup
                                         .append(this._createCircleElement(pointsVal.midx[i], pointsVal.midy[i], 22, !itemEnable ? "e-m-radialdefault e-m-pathdisabled" : "e-m-radialdefault"))
                                         .append(imgEle));
            }
            var radialCircle = this._createCircleElement(this._startXY, this._startXY, this._startXY, "e-m-radialcircle");
            this._radialSVGDiv.append(this._menuChildSVGDiv.append(this._radialChildSVG.append(radialCircle).append(this._menuChildItemsGroup)));
        },
        _androidRadialMenu: function () {
            this._radialSVG = this._createSVGElement();
            this._menuItemsGroup = this._createGroupElement("menuitemsgroup");
            this._androidArcGroup = this._createGroupElement("androidarcgroup");
            this._imageBadgeGroup = this._createGroupElement("imagebadgegroup");
            this._imageBadgeGroup.attr("transform", "translate(-12,-12)");
            this._radialSVGDiv = ej.buildTag("div#" + this._id + "_svgdiv.e-m-abs").css("height", this._diameter);
            this._radialMenuSVGDiv = ej.buildTag("div#" + this._id + "_menusvgdiv.e-m-abs e-m-displaynone");
            var pointsVal = this._calculateRadialArc(this._itemCount, 1, 0.41, 0.65, null),
                pointsArcVal = this._calculateRadialArc(this._itemCount, 0.63, null, 0.95, null),
                childArc = this._createGroupElement(),
                imgDim = 25;
            for (var i = 0; i < this._itemCount; i++) {
                var direction = this._pathDirection(pointsVal.x1[i], pointsVal.y1[i], pointsVal.radius, pointsVal.longArc[i], pointsVal.x2[i], pointsVal.y2[i],
                    pointsVal.dEndX[i], pointsVal.dEndY[i], pointsVal.dradius, pointsVal.dStartX[i], pointsVal.dStartY[i]),
                    arcDirection = this._pathDirection(pointsArcVal.x1[i], pointsArcVal.y1[i], pointsArcVal.radius, pointsArcVal.longArc[i], pointsArcVal.x2[i],
                    pointsArcVal.y2[i], pointsArcVal.dEndX[i], pointsArcVal.dEndY[i], pointsArcVal.dradius, pointsArcVal.dStartX[i], pointsArcVal.dStartY[i]),
                    currentEle = this.model.items[i],
                    arcPath = this._createPathElement(i, arcDirection, "e-m-radialarcdefault"),
                    itemsContainer = this._createGroupElement("itemscontainer"),
                    itemEnable = this._ejmEnableItem(currentEle, itemsContainer);
                this._imageBadgeGroup.append(itemsContainer.append(this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i], pointsVal.midy[i])));
                if (currentEle.badge && (currentEle.badge.enabled == "true" || currentEle.badge.enabled == true))
                    itemsContainer.append(this._createCircleElement(pointsVal.midx[i] + 25, pointsVal.midy[i], 10, "e-m-badgecircle"))
                                  .append(this._createTextElement(pointsVal.midx[i] + 25, pointsVal.midy[i] + 3, "e-m-badgetext", currentEle.badge.value));
                this._androidArcGroup.append(this._createPathElement(i, direction, !itemEnable ? "e-m-radialdefault e-m-pathdisabled" : "e-m-radialdefault"));
                childArc.append(arcPath);
                if (currentEle.items || (currentEle.type && currentEle.type.toLowerCase() == ej.mobile.RadialMenu.ItemType.Slider))
                    arcPath.setAttribute("class", "e-m-radialarcactive");
            }
            this._radialContainer.append(this._radialSVGDiv.append(this._radialMenuSVGDiv.append(this._radialSVG.append(this._menuItemsGroup
                                 .append(this._androidArcGroup).append(childArc).append(this._imageBadgeGroup)))));
        },
        _androidChildRadialMenu: function (child, childCount, index) {
            this._radialChildSVG = this._createSVGElement();
            this._menuChildItemsGroup = this._createGroupElement("menuchilditemsgroup");
            this._androidChildArcGroup = this._createGroupElement("androidchildarcgroup");
            this._childImageBadgeGroup = this._createGroupElement("childimagebadgegroup");
            this._childImageBadgeGroup.attr("transform", "translate(-12,-12)");
            var pointsVal = this._calculateRadialArc(childCount, 1, 0.41, 0.65, null),
                pointsArcVal = this._calculateRadialArc(childCount, 0.63, null, 0.95, null),
                childArc = this._createGroupElement(),
                imgDim = 25;
            for (var i = 0; i < childCount; i++) {
                var direction = this._pathDirection(pointsVal.x1[i], pointsVal.y1[i], pointsVal.radius, pointsVal.longArc[i], pointsVal.x2[i], pointsVal.y2[i],
                    pointsVal.dEndX[i], pointsVal.dEndY[i], pointsVal.dradius, pointsVal.dStartX[i], pointsVal.dStartY[i]),
                    arcDirection = this._pathDirection(pointsArcVal.x1[i], pointsArcVal.y1[i], pointsArcVal.radius, pointsArcVal.longArc[i], pointsArcVal.x2[i],
                    pointsArcVal.y2[i], pointsArcVal.dEndX[i], pointsArcVal.dEndY[i], pointsArcVal.dradius, pointsArcVal.dStartX[i], pointsArcVal.dStartY[i]),
                    currentEle = this.model.items[index].items[i],
                    itemsContainer = this._createGroupElement("childitemscontainer"),
                    itemEnable = this._ejmEnableItem(currentEle, itemsContainer);
                this._childImageBadgeGroup.append(itemsContainer.append(this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i], pointsVal.midy[i])));
                this._androidChildArcGroup.append(this._createPathElement(i, direction, !itemEnable ? "e-m-radialdefault e-m-pathdisabled" : "e-m-radialdefault"));
                childArc.append(this._createPathElement(i, arcDirection, "e-m-radialarcdefault"));
            }
            this._radialSVGDiv.append(this._menuChildSVGDiv.append(this._radialChildSVG.append(this._menuChildItemsGroup.append(this._androidChildArcGroup)
                              .append(this._childImageBadgeGroup).append(childArc))));
        },
        _windowsRadialMenu: function () {
            this._renderRadialMenu();
        },
        _flatRadialMenu: function () {
            this._renderRadialMenu();
        },
        _ejmEnableItem: function (currentObj, ele) {
            if (currentObj.enabled == "false" || currentObj.enabled == false) {
                $(ele).attr("class", "e-m-itemdisabled");
                return false;
            }
            return true;
        },
        _ejmMenuBaseItemsRemove: function () {
            var proxy = this;
            this._degDis = 0;
            this._totDis = 0;
            this._menuChildSVGDiv = ej.buildTag("div#" + this._id + "_menuchildsvgdiv.e-m-abs");
            if (this.model.enableAnimation) {
                setTimeout(function () { proxy._radialMenuSVGDiv.removeClass("e-m-scalehide e-m-scaleshow").addClass("e-m-displaynone"); }, 150);
                this._menuChildSVGDiv.addClass("e-m-scaleshow");
            }
            else
                this._radialMenuSVGDiv.addClass("e-m-displaynone");
            this._radial.removeClass(this.model.imageClass).addClass("e-m-backarrow " + this.model.backImageClass + "");
        },
        _touchStartHandler: function (evt) {
            evt.preventDefault();
            ej._touchStartPoints(evt, this);
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            this._startY = evt.clientY;
            this["_" + this.model.renderMode + "TouchStart"](evt);
            ej.listenEvents([this._radialSVGDiv], ["mouseout"], [this._touchCanel], false);
            ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], false);
            ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, true);
        },
        _ios7TouchStart: function (evt) {
            var targetEle = $(evt.target), eleClass = targetEle.attr("class"), tagName = evt.target.tagName;
            if (eleClass == "e-m-radialdefault" || this._isBadge(eleClass, tagName)) {
                this._currentTarget = targetEle.parent();
                this._currentTarget.find("circle.e-m-radialdefault").attr("class", "e-m-radialactive");
            }
        },
        _androidTouchStart: function (evt) {
            var targetEle = $(evt.target), eleClass = targetEle.attr("class"), tagName = evt.target.tagName,
                index = !this._isBadge(eleClass, tagName) ? targetEle.index() : targetEle.parent().index();
            if (eleClass == "e-m-radialdefault" || this._isBadge(eleClass, tagName)) {
                var ele = this._childTarget ? this._androidChildArcGroup : this._androidArcGroup;
                this._currentTarget = $(ele.children()[index]);
                this._currentTarget.attr("class", "e-m-radialactive");
            }
        },
        _windowsTouchStart: function (evt) {
            this._windowsTouchStartAction(evt);
        },
        _flatTouchStart: function (evt) {
            this._windowsTouchStartAction(evt);
        },
        _windowsTouchStartAction: function (evt) {
            var targetEle = $(evt.target), eleClass = targetEle.attr("class"), tagName = evt.target.tagName, index, text;
            if (eleClass == "e-m-itembgcolor" || eleClass == "e-m-default") {
                index = targetEle.index();
                text = !this._childTarget ? $(this._menuItemsGroup.children()[targetEle.index()]).text() : $(this._childItemsGroup.children()[targetEle.index()]).text();
                this._event = true;
            }
            else if ((tagName == "text" && !this._isRSText(eleClass)) || tagName == "image") {
                index = targetEle.parent().index();
                text = targetEle.parent().text();
                this._event = true;
            }
            else if (eleClass == "e-m-childdefault" || tagName == "polygon") {
                index = parseInt(targetEle.attr("index"));
                this._windowsChild = this._childArcGroup.find('path[index=' + index + ']');
                this._windowsChild.attr("class", "e-m-active");
            }
            if (this._event) {
                this._currentTarget = !this._childTarget ? $(this._menuItemsArcPaths.children()[index]) : $(this._childItemsArcPaths.children()[index]);
                this._currentTarget.attr("class", "e-m-childdefault");
                this._event = false;
            }
        },
        _touchMoveHandler: function (evt) {
            evt.preventDefault();
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            this._relativeDis = evt.clientY - this._startY;
            if (this._degDis != 0)
                this._relativeDis += this._totDis;
            this._degDis = this._pos.charAt(0).toLowerCase() == "r" ? -(this._relativeDis % 360) : this._relativeDis % 360;
            if (ej._isTouchMoved(evt, this)) {
                this["_" + this.model.renderMode + "TouchMove"](evt);
                this._move = true;
                this._currentTarget = null;
            }
        },
        _windowsTouchMoveAction: function (evt) {
            if (this._windowsChild)
                this._windowsChild.attr("class", "e-m-childdefault");
            if (this._currentTarget)
                this._currentTarget.attr("class", "e-m-arcbgcolor");
            if (!this._childTarget) {
                this._radialSVG.css({ "transform-origin": "center", "transform": "rotate(" + this._degDis + "deg)" });
                this._radialChildSVG.css({ "transform-origin": "center", "transform": "rotate(" + this._degDis + "deg)" });
                this._iconRotation(this._menuItemsGroup);
            }
            else {
                this._menuChildSVG.css({ "transform-origin": "center", "transform": "rotate(" + this._degDis + "deg)" });
                this._iconRotation(this._childItemsGroup);
            }
        },
        _touchMoveAction: function (childEle, ele) {
            if (this._childTarget) {
                this._radialChildSVG.css({ "transform-origin": "center", "transform": "rotate(" + this._degDis + "deg)" });
                this._iconRotation(childEle);
            }
            else {
                this._radialSVG.css({ "transform-origin": "center", "transform": "rotate(" + this._degDis + "deg)" });
                this._iconRotation(ele);
            }
        },
        _ios7TouchMove: function () {
            this._touchMoveAction(this._menuChildItemsGroup, this._menuItemsGroup);
            if (this._currentTarget)
                this._currentTarget.find("circle.e-m-radialactive").attr("class", "e-m-radialdefault");
        },
        _androidTouchMove: function () {
            this._touchMoveAction(this._childImageBadgeGroup, this._imageBadgeGroup);
            if (this._currentTarget)
                this._currentTarget.attr("class", "e-m-radialdefault");
        },
        _windowsTouchMove: function (evt) {
            this._windowsTouchMoveAction(evt);
        },
        _flatTouchMove: function () {
            this._windowsTouchMoveAction();
        },
        _touchEndHandler: function (evt) {
            this._totDis = this._relativeDis;
            this["_" + this.model.renderMode + "TouchEnd"](evt);
            ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
            ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
            this._move = false;
        },
        _ios7TouchEnd: function (evt) {
            this._ios7TouchEndAction(evt);
        },
        _androidTouchEnd: function (evt) {
            this._ios7TouchEndAction(evt);
        },
        _windowsTouchEnd: function (evt) {
            this._windowsTouchEndAction(evt);
        },
        _flatTouchEnd: function (evt) {
            this._windowsTouchEndAction(evt);
        },
        _ios7TouchEndAction: function (evt) {
            if (this._currentTarget) {
                var index = this._currentTarget.index();
                this.model.renderMode == "ios7" ? this._currentTarget.find("circle.e-m-radialactive").attr("class", "e-m-radialdefault")
                : this._currentTarget.attr("class", "e-m-radialdefault");
                var currentItem = this.model.items[index];
                if (!this._childTarget && currentItem.items) {
                    this._ejmMenuBaseItemsRemove();
                    this["_" + this.model.renderMode + "ChildRadialMenu"](currentItem.items, currentItem.items.length, index);
                    this._childTarget = true;
                    this._index = index;
                }
                else if (!this._childTarget && currentItem.type && currentItem.type.toLowerCase() == ej.mobile.RadialMenu.ItemType.Slider) {
                    this._ejmMenuBaseItemsRemove();
                    var container = ej.buildTag("div"), sliderEle = ej.buildTag("div.e-m-rm-slider"), radialChildSVG = this._createSVGElement(),
                        radialCircle = this._createCircleElement(this._startXY, this._startXY, this._startXY, "e-m-radialcircle");
                    this._renderRadialSlider(currentItem, sliderEle);
                    this._radialSVGDiv.append(this._menuChildSVGDiv.append(container.append(radialChildSVG.append(radialCircle)).append(sliderEle.removeAttr("style"))));
                    if (this.model.renderMode == "android")
                        radialChildSVG.append(this._createCircleElement(this._startXY, this._startXY, this._startXY - 60, "e-m-radialcircle"));
                    this._index = index;
                }              
                    var data = this._childTarget ? { index: this._index, childIndex: index } : { index: index, childIndex: null };
                        this._eventsTrigger(data, "select");
                        this._eventsTrigger(data, "touch");              
            }
            this._currentTarget = null;
        },
        _windowsTouchEndAction: function (evt) {
            var targetEle = $(evt.target);
            if (targetEle.attr("class") == "e-m-active" || evt.target.tagName == "polygon") {
                this._index = parseInt(targetEle.attr("index"));
                this._ejMenuBaseItemsRemove();
                var currentItem = this.model.items[this._index];
                $(this._childArcGroup.find('path[index=' + this._index + ']')).attr("class", "e-m-childdefault");
                if (currentItem.type && currentItem.type.toLowerCase() == ej.mobile.RadialMenu.ItemType.Slider) {
                    this._renderRadialSlider(currentItem, this._menuChildSVGDiv);
                    this._radialSVGDiv.append(this._menuChildSVGDiv.removeAttr("style").addClass("e-m-rm-slider"));
                }
                else {
                    this._renderRadialChildMenu(currentItem);
					data ={ index: this._index, childIndex: null };
                    this._eventsTrigger(data, "select");
                    this._eventsTrigger(data, "touch");
                    this._childTarget = true;
                }
            }
            if (this._currentTarget) {
                var index = this._currentTarget.index(),
                    data = this._childTarget ? { index: this._index, childIndex: index } : { index: index, childIndex: null };
                    this._eventsTrigger(data, "select");
                    this._eventsTrigger(data, "touch");
                this._currentTarget.attr("class", "e-m-arcbgcolor");
            }
            this._currentTarget = null;
        },
        _isBadge: function (eleClass, tagName) {
            return (eleClass == "e-m-badgetext" || eleClass == "e-m-badgecircle" || tagName == "image" ? true : false);
        },
        _renderRadialSlider: function (currentItem, sliderEle) {
            var ticks = currentItem.sliderSettings.ticks ? eval(currentItem.sliderSettings.ticks) : [0, 2, 4, 6, 8, 10, 12],
                value = currentItem.badge.value ? parseInt(currentItem.badge.value) : 4,
                radius = this.model.renderMode == "android" ? this.model.radius - 22 : (this.model.renderMode == "ios7" ? this.model.radius - 35 : this.model.radius - 30),
                stroke = currentItem.sliderSettings.strokeWidth ? parseInt(currentItem.sliderSettings.strokeWidth) : ((this.model.renderMode == "windows" || this.model.renderMode == "flat") ? 10 : 2),
                labelSpace = currentItem.sliderSettings.labelSpace ? parseInt(currentItem.sliderSettings.labelSpace) : (this.model.renderMode == "android" ? 22 : 25);
            sliderEle.ejmRadialSlider({
                enableAnimation: false, radius: radius, value: value, ticks: ticks, strokeWidth: stroke, labelSpace: labelSpace,
                position: this.model.position, renderMode: this.model.renderMode, change: $.proxy(this._radialSliderHandler, this)
            });
        },
        resize: function (evt) {
            this._posCenter();
        },
        _posCenter: function () {
            var proxy = this;
            window.setTimeout(function () {
                if (proxy._pos == "rightcenter" || proxy._pos == "leftcenter")
                    proxy.element.css({ "top": (window.innerHeight / 2) - (proxy.model.radius) + "px" });
            }, ej.isAndroid() ? 200 : 0);
        },
        _showMenu: function () {
            this["_" + this.model.renderMode + "RadialTouch"]();
        },
        showRadialMenu: function () {
            this._showMenu();
        },
        _ios7RadialTouch: function () {
            this._ios7RadialAction();
        },
        _androidRadialTouch: function () {
            this._ios7RadialAction();
        },
        _windowsRadialTouch: function () {
            this._windowsRadialAction();
        },
        _flatRadialTouch: function () {
            this._windowsRadialAction();
        },
        _ios7RadialAction: function () {
            var proxy = this;
            if (this._radial.hasClass("e-m-backarrow")) {
                this._radial.removeClass("e-m-backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
                if (this.model.enableAnimation) {
                    this._menuChildSVGDiv.removeClass("e-m-scaleshow").addClass("e-m-scalehide");
                    setTimeout(function () { proxy._menuChildSVGDiv.remove(); }, 150);
                    setTimeout(function () { proxy._radialMenuSVGDiv.removeClass("e-m-scalehide e-m-displaynone").addClass("e-m-scaleshow"); }, 200);
                }
                else {
                    this._menuChildSVGDiv.remove();
                    this._radialMenuSVGDiv.removeClass("e-m-displaynone");
                }
                ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
                ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
            }
            else {
                if (this._radialMenuSVGDiv.hasClass("e-m-displaynone")) {
                    this.model.enableAnimation ? this._radialMenuSVGDiv.removeClass("e-m-displaynone e-m-radialhide").addClass("e-m-radialshow") : this._radialMenuSVGDiv.removeClass("e-m-displaynone");
                    ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
                    this._openCloseTrigger("open");
                }
                else {
                    if (this.model.enableAnimation) {
                        this._radialMenuSVGDiv.removeClass("e-m-radialshow e-m-scaleshow").addClass("e-m-radialhide");
                        setTimeout(function () { proxy._radialMenuSVGDiv.addClass("e-m-displaynone"); }, 160);
                    }
                    else
                        this._radialMenuSVGDiv.addClass("e-m-displaynone");
                    this._openCloseTrigger("close");
                    ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
                }
            }
            this._degDis = 0;
            this._totDis = 0;
            this._childTarget = false;
            $(this._menuItemsGroup).each(function (e) {
                $(this).find("image,circle,text").removeAttr("transform");
            });
            this._radialSVG.css("transform", "rotate(0deg)");
        },
        _windowsRadialAction: function () {
            if (this._radial.hasClass("e-m-backarrow")) {
                this._windowsInnerMenuHide(this);
                ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
                ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
            }
            else {
                if (this._radialCircleSVGDiv.hasClass("e-m-displaynone")) {
                    this._windowsMenuShow();
                    ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
                }
                else {
                    this._windowsMenuHide(this);
                    ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
                }
            }
            this._degDis = 0;
            this._totDis = 0;
            this._childTarget = false;
            $(this._menuItemsGroup).each(function (e) {
                $(this).find("g").removeAttr("transform");
            });
            this._radialSVG.css("transform", "rotate(0deg)");
            this._radialChildSVG.css("transform", "rotate(0deg)");
        },
        _radialChildAnimate: function () {
            this._degDis = 0;
            this._totDis = 0;
            var proxy = this;
            setTimeout(function () {
                proxy._radialChildSVGDiv.removeClass("e-m-scalehide e-m-scaleshow").addClass("e-m-displaynone");
            }, 150);
        },
        _iconRotation: function (element) {
            var proxy = this;
            $.each(element.children(), function (i, ele) {
                var currentEle = $(ele), currentImgEle = currentEle.find("image"), currentCircleEle = currentEle.find("circle"),
                    x = parseFloat(currentImgEle.attr("x")), y = parseFloat(currentImgEle.attr("y")), disX = currentImgEle.attr("width") / 2, disY = currentImgEle.attr("height") / 2,
                    cX = parseFloat(currentCircleEle.attr("cx")), cY = parseFloat(currentCircleEle.attr("cy")),
                    imageRotateX = proxy.model.renderMode == "ios7" ? cX : x + disX, imageRotateY = proxy.model.renderMode == "ios7" ? cY : y + disY,
                    imgRotate = "rotate(" + (-proxy._degDis) + " " + imageRotateX + " " + imageRotateY + ")",
                    circleRotate = "rotate(" + (-proxy._degDis) + " " + (x + disX) + " " + (y + disY) + ")";
                if (proxy.model.renderMode == "windows" || proxy.model.renderMode == "flat")
                    currentEle.attr({ "transform": imgRotate });
                else {
                    currentImgEle.attr({ "transform": imgRotate });
                    currentEle.find("text,circle").attr({ "transform": circleRotate });
                }
            });
        },
        _menuItems: function (proxy, index, menuClass) {
            if (proxy.model.renderMode == "ios7") {
                var currentGroupEle = $(proxy._menuItemsGroup.find("g")[index]),
                    imgEle = currentGroupEle.find("image"), itemCircleEle = currentGroupEle.find("circle.e-m-radialdefault"),
                    badgeCircleEle = currentGroupEle.find("circle.e-m-badgecircle"), textEle = currentGroupEle.find("text");
                if (menuClass == "disabled") {
                    imgEle.attr("class", "e-m-itemdisabled");
                    itemCircleEle.attr("class", "e-m-radialdefault e-m-pathdisabled");
                    badgeCircleEle.attr("class", "e-m-badgecircle e-m-itemdisabled");
                    textEle.attr("class", "e-m-badgetext e-m-itemdisabled");
                }
                else {
                    imgEle.removeAttr("class");
                    itemCircleEle.attr("class", "e-m-radialdefault");
                    badgeCircleEle.attr("class", "e-m-badgecircle");
                    textEle.attr("class", "e-m-badgetext");
                }
            }
            else if (proxy.model.renderMode == "android") {
                var currentGroupEle = $(proxy._imageBadgeGroup.find("g")[index]),
                    currentPathEle = $(proxy._androidArcGroup.find("path")[index]);
                if (menuClass == "disabled") {
                    currentGroupEle.attr("class", "e-m-itemdisabled");
                    currentPathEle.attr("class", "e-m-radialdefault e-m-pathdisabled");
                }
                else {
                    currentGroupEle.removeAttr("class");
                    currentPathEle.attr("class", "e-m-radialdefault");
                }
            }
            else
                this._ejMenuItem(proxy, index, menuClass);
        },
        _hideMenu: function () {
            var proxy = this;
            if (this.model.renderMode == "ios7" || this.model.renderMode == "android") {
                if (this._radial.hasClass("e-m-backarrow")) {
                    this._radial.removeClass("e-m-backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
                    this._childTarget = false;
                    if (this.model.enableAnimation) {
                        this._radialMenuSVGDiv.removeClass("e-m-radialshow e-m-radialhide e-m-scaleshow e-m-scalehide").addClass("e-m-displaynone");
                        this._menuChildSVGDiv.removeClass("e-m-scaleshow").addClass("e-m-radialhide");
                        setTimeout(function () { proxy._menuChildSVGDiv.removeClass("e-m-radialhide").remove(); }, 150);
                    }
                    else {
                        this._radialMenuSVGDiv.addClass("e-m-displaynone");
                        this._menuChildSVGDiv.remove();
                    }
                    ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
                }
                else {
                    if (this.model.enableAnimation) {
                        this._radialMenuSVGDiv.removeClass("e-m-radialshow e-m-scaleshow").addClass("e-m-radialhide");
                        setTimeout(function () {
                            proxy._radialMenuSVGDiv.addClass("e-m-displaynone").removeClass("e-m-radialshow e-m-radialhide e-m-scaleshow e-m-scalehide");
                        }, 160);
                    }
                    else
                        this._radialMenuSVGDiv.addClass("e-m-displaynone");
                }
                this._openCloseTrigger("close");
                ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
            }
            else {
                if (this._radial.hasClass("e-m-backarrow")) {
                    this._childTarget = false;
                    this._ejInnerMenuHide(this);
                    ej.listenTouchEvent(this._radialSVGDiv, ej.startEvent(), this._touchStart, false);
                }
                else
                    this._ejMenuHide(this);
                ej.listenEvents([this._radialSVGDiv, this._radialSVGDiv], [ej.moveEvent(), ej.endEvent()], [this._touchMove, this._touchEnd], true);
            }
        },
        documentClick: function (evt) {
            if ($($(evt.target).closest("div.e-m-rel")).length == 0 && !this._move) {
                this.hideMenu();
                this._move = false;
            }
        }
    });
    ej.mobile.RadialMenu.Position = {
        RightCenter: "rightcenter",
        RightTop: "righttop",
        RightBottom: "rightbottom",
        LeftCenter: "leftcenter",
        LeftTop: "lefttop",
        LeftBottom: "leftbottom"
    };
    ej.mobile.RadialMenu.ItemType = {
        Default: "default",
        Slider: "slider"
    };
    $.extend(true, ej.mobile.RadialMenu.prototype, ej.RadialMenuBase.prototype);
})(jQuery, Syncfusion);;
// JavaScript source code
/**
* @fileOverview Plugin to style the Html RadialSlider elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejRadialSliderBase", "ej.RadialSliderBase", {
        defaults: {
            radius: 200,
            ticks: [0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
            enableRoundOff: true,
            value: 10,
            autoOpen: true,
            enableAnimation: true,
            cssClass: null,
            labelSpace: 30,
            stop: null,
            slide: null,
            start: null,
            change: null,
            create: null,
            destroy: null
        },
        dataTypes: {
            radius: "number",
            enableRoundOff: "boolean",
            enableAnimation: "boolean",
            cssClass: "string"
        },
        observables: ["value"],
        observableArray: ["ticks"],
        value: ej.util.valueFunction("value"),
        ticks: ej.util.valueFunction("ticks"),
        _outerTextCalculation: function (length, startAngle, endAngle) {
            var radius = this._isMobile() ? (this.model.radius - this.model.labelSpace) : (this.model.radius + this.model.labelSpace);
            this._point = ((endAngle - startAngle) / (length - 1));
            endAngle = startAngle + ((endAngle - startAngle) / (length - 1));
            var endValueCal = endAngle,
                startValueCal = endValueCal;
            this._textPoints = [];
            for (var j = 0; j < length; j++) {
                var line = {};
                var angleCondition = startAngle;
                startAngle = startAngle * Math.PI / 180.0;
                endAngle = endAngle * Math.PI / 180.0;
                line.X2 = (this._startXY) + (radius) * Math.cos(startAngle);
                line.Y2 = (this._startXY) + (radius) * Math.sin(startAngle);
                if (angleCondition <= 270 && 90 <= angleCondition)
                    line.textAlignment = "middle";
                else
                    line.textAlignment = "start";
                startAngle = endValueCal;
                endAngle = endValueCal + this._point;
                endValueCal += this._point;
                this._textPoints.push(line);
            }
            var line = {};
            var angleCondition = this._startValueAngle;
            this._startValueAngle = this._startValueAngle * Math.PI / 180.0;
            line.X2 = (this._startXY) + (radius) * Math.cos(this._startValueAngle);
            line.Y2 = (this._startXY) + (radius) * Math.sin(this._startValueAngle);
            if (angleCondition <= 270 && 90 <= angleCondition)
                line.textAlignment = "middle";
            else
                line.textAlignment = "start";
            this._textPoints.push(line);
            return this._textPoints;
        },

        _polarToCartesian: function (cX, cY, radius, angle, isDynamic) {
            var angleRadians = (angle) * Math.PI / 180.0;
            return {
                x: cX + (radius * Math.cos(angleRadians)),
                y: cY + (radius * Math.sin(isDynamic ? -angleRadians : angleRadians))
            };
        },
        _tapHandlerEvent: function (e) {
            var baseRect = $("#" + this._prefix + this._elementID + "-radial-slider-svg").offset();
            var y2 = e.clientY, x2 = e.clientX, y1 =( baseRect.top + (this._radialWidth / 2)) - $(window).scrollTop(), x1 = baseRect.left + (this._radialWidth / 2);
            this._dynamicAngleCalculation(x1, x2, y1, y2);
            var space = this._isMobile() && this.model.renderMode == "ios7" ? 6.5 : this._isMobile() && this.model.renderMode == "windows" ? 2.5 : 5;
            if ((!this._isMobile() && (this._angle >= (360 - this._endAngle) && this._angle <= (360 - this._startAngle))) || (this._isMobile() && (this._angle >= (360 - (this._endAngle - space)) && this._angle <= (360 - (this._startAngle + space)))) || (this._isMobile() && this.model.position.charAt(0).toLowerCase() == "l" && (this._angle > (270 + space) || this._angle < (90 - space)))) {
                this._lineAngleCalculation(true);
                this._previousAngle = this._angle;
                if (!this._isMobile()) {
                    $(this._overLine).remove();
                    this._pathBeforeAddlength = this._tickCount + 1;
                    this._pathAfterAddLength = this._directionLine.toString().replace(/[^M]/g, "").length;
                    if (this._pathBeforeAddlength < this._pathAfterAddLength) {
                        var deleteCount = this._isTapSelected ? 2 : 1;
                        this._directionLine.remove(this._tickCount, deleteCount, deleteCount);
                    }
                    else
                        this._directionLine.remove(this._tickCount, 1);
                    this._dynamicLineCalculation(this._angle, false, true, true, this._tickCount, this._tickCount + 1, true);
                    $(this._pathLineElement).attr("d", this._directionLine.toString());
                }
                if ($(this._textGroupElement).find('[id=' + this._prefix + this._elementID + "-dynamic-text" + ']').length > 0)
                    this._textGroupElement.find('[id=' + this._prefix + this._elementID + "-dynamic-text" + ']').remove();
                var selectPart = this._selectPart();
                var select = selectPart.select;
                var selectValue;
                if (this._isTicksControl()) {
                    selectValue = this._ticksCalculation();
                    if (this._isMobile() && this.model.position.charAt(0).toLowerCase() == "l")
                        this.line.textAlignment = "end";
                    else if (this._isMobile() && this.model.position.charAt(0).toLowerCase() == "r")
                        this.line.textAlignment = "start";
                    else if (this._isMobile() && (this.model.position.charAt(0).toLowerCase() == "b" || this.model.position.charAt(0).toLowerCase() == "t"))
                        this.line.textAlignment = "middle";
                    if (this._isMobile() && (this.ticks().indexOf(selectValue) == 0) || (this.ticks().indexOf(selectValue) == this._tickCount - 1))
                        this.line.textAlignment = "middle";
                    if (this.ticks().indexOf(selectValue) < 0) {
                        this._outerTextElement = this._createSVGElements("text", { "stroke-width": 0.5, "x": this.line.X2, "y": this.line.Y2, "class": this._prefix + "dynamic-text", "id": this._prefix + this._elementID + "-dynamic-text", "textContent": selectValue, "text-anchor": this.line.textAlignment });
                        this._textGroupElement.append(this._outerTextElement);
                        var dynamicText = document.getElementById(this._prefix + this._elementID + "-dynamic-text").getBoundingClientRect();
                        var x1 = this._textPoints[select.toFixed()].X2,
                            y1 = this._textPoints[select.toFixed()].Y2,
                            x2 = this.line.X2,
                            y2 = this.line.Y2;
                        var distance = Math.sqrt(Math.pow((x2 - x1), 2) + Math.pow((y2 - y1), 2));
                        if (distance < (dynamicText.width * 72 / 96)) {
                            this.line = {};
                            var value = (selectValue.toString().length)
                            var pixToDegree = (dynamicText.width * 72 / 96) - (this._isMobile() ? (value * 2) : (value * 4));
                            var degPoint = (parseFloat(select.toFixed(3).substr(0, 1)) == select.toFixed()) ? this._degPoint[select.toFixed()] + pixToDegree : this._degPoint[select.toFixed()] - pixToDegree;
                            var angle = (360 - degPoint) * (Math.PI / 180.0)
                            var radius = this._isMobile() ? (this.model.radius - (this.model.labelSpace)) : (this.model.radius + (this.model.labelSpace));
                            this.line.X2 = (this._startXY) + radius * Math.cos(angle);
                            this.line.Y2 = (this._startXY) + radius * Math.sin(-angle);
                            $("#" + this._prefix + this._elementID + "-dynamic-text").attr({ "x": this.line.X2, "y": this.line.Y2 });
                        }
                    }
                }
                else
                    selectValue = this.ticks()[select.toFixed()];
                this._trigger("change", { value: selectValue, oldValue: this.value() });
                if (this._needleStop && this.model.stop)
                    this._trigger("stop", { value: selectValue });
                this.value(selectValue);
                this._needleStop = true;
                this._needleMove = false;
                if (this._isMobile())
                    ((this.model.renderMode == "windows" || this.model.renderMode == "flat") ? this._dynamicWindowsRadial() : this._dynamicIOSandAndroidRadial());
            }
        },
        _selectPart: function () {
            var dynamicAngle = this._dynamicAngle, startAngle = this._startAngle, endAngle = this._endAngle;
            if (this._isMobile()) {
                var space = this.model.renderMode == "ios7" ? 5 : this.model.renderMode == "windows" ? 2.5 : 5;
                startAngle = this._startAngle + space, endAngle = this._endAngle - space;
                if (this.model.position.charAt(0).toLowerCase() == "l") {
                    if (this._dynamicAngle < 180)
                        dynamicAngle = this._dynamicAngle + 360;
                }
            }
            var select = (dynamicAngle - startAngle) / ((endAngle - startAngle) / (this.ticks().length - 1));
            var firstValue = this.ticks()[parseInt(select)];
            var secondValue = this.ticks()[parseInt(select) + 1];
            var difference = (secondValue - firstValue);
            return { select: select, firstValue: firstValue, space: space, difference: difference, dynamicAngle: dynamicAngle, };
        },
        _lineAngleCalculation: function (isMove) {
            var selectPart = this._selectPart();
            this._degPoint.splice(this.ticks().length, this.ticks().length + 1);
            this._degPoint.push(this._degPoint[this._degPoint.length - 1] + this._point);
            var innerValue = (((this._degPoint[parseInt(selectPart.select) + 1] - this._degPoint[parseInt(selectPart.select)]) - (this._degPoint[parseInt(selectPart.select) + 1] - (selectPart.dynamicAngle))) / (this._point / selectPart.difference));
            var startDegree = parseInt(selectPart.select) != 0 ? this._degPoint[parseInt(selectPart.select)] : this._degPoint[parseInt(selectPart.select)];
            var range;
            if (this.model.enableRoundOff && isMove) {
                var smallValue = parseFloat(innerValue.toFixed(2));
                innerValue = parseInt(innerValue.toFixed());
                range = (selectPart.difference == 0.5 && smallValue >= 0.25) ? this._point * 1 : (this._point / selectPart.difference) * innerValue;
            }
            else
                range = (this._point / selectPart.difference) * parseFloat(innerValue.toFixed(2));
            this._angle = (360 - this._degPoint[parseInt(selectPart.select)]) - Math.abs(range);
        },

        _isTicksControl: function () {
            var selectPart = this._selectPart();
            this._degPoint.splice(this.ticks().length, this.ticks().length + 1);
            this._degPoint.push(this._degPoint[this._degPoint.length - 1] + this._point);
            var innerValue = (((this._degPoint[parseInt(selectPart.select) + 1] - this._degPoint[parseInt(selectPart.select)]) - (this._degPoint[parseInt(selectPart.select) + 1] - (this._dynamicAngle))) / (this._point / selectPart.difference));
            var isSecSame = this.model.enableRoundOff ? parseInt(innerValue.toFixed()) == 0 || parseInt(innerValue.toFixed()) == (selectPart.difference) ? false : true : $.inArray(this._angle, this._degPoint) > -1 ? false : true;
            return isSecSame;
        },

        _ticksCalculation: function () {
            var selectPart = this._selectPart();
            var select = selectPart.select, dynamicAngle = selectPart.dynamicAngle, difference = selectPart.difference, firstValue = selectPart.firstValue, space = selectPart.space;
            var firstValue = this.ticks()[parseInt(select)];
            this.line = {};
            var angle = (this._angle) * (Math.PI / 180.0)
            var radius = this._isMobile() ? (this.model.radius - this.model.labelSpace) : (this.model.radius + this.model.labelSpace);
            this.line.X2 = (this._startXY) + radius * Math.cos(angle);
            this.line.Y2 = (this._startXY) + radius * Math.sin(-angle);
            if (this._angle <= 270 && 90 <= this._angle)
                this.line.textAlignment = "middle";
            else
                this.line.textAlignment = "start";
            var startDegree = parseInt(select) != 0 ? this._degPoint[parseInt(select)] : this._degPoint[parseInt(select)];
            var range = (dynamicAngle - startDegree) / (this._point / difference);
            var setValue;
            if (this.model.enableRoundOff) {
                var controlValue = parseFloat(range.toFixed(1).substr(1, 3));
                range = parseInt(range.toFixed());
                setValue = firstValue + Math.abs(range);
            }
            else {
                setValue = firstValue + Math.abs(parseFloat(range.toFixed(2)));
                setValue = parseFloat(setValue.toFixed(2));
            }
            return setValue;
        },
        _dynamicAngleCalculation: function (x1, x2, y1, y2) {
            var theta = Math.atan2((y2 - y1), (x2 - x1));
            this._angle = (360 - ((theta * 180) / Math.PI)) % 360;
            this._dynamicAngle = (360 + ((theta * 180) / Math.PI)) % 360;
        },
        _createSVGElements: function (element, attr) {
            var svgObj = document.createElementNS(this._svgLink, element);
            $.each(attr, function (attr, value) {
                if (attr == 'xlink:href')
                    svgObj.setAttributeNS('http://www.w3.org/1999/xlink', 'xlink:href', value);
                if (attr != "textContent")
                    svgObj.setAttribute(attr, value);
                else
                    svgObj.textContent = value;
            })
            return svgObj;
        },
        show: function () {
            if (this.model.enableAnimation) {
                this.element.removeClass(this._prefix + "slider-hide").addClass(this._prefix + "slider-show");
                this._radialSVG.attr("class", "").attr("class", (this._prefix + "radialslider-svg-show " + this._prefix + "rs-svg"));
            }
            this.element.css("display", "block");
            this.model.autoOpen = true;
        },
        hide: function () {
            var proxy = this;
            if (this.model.enableAnimation) {
                this.element.removeClass(this._prefix + "slider-show").addClass(this._prefix + "slider-hide");
                this._radialSVG.attr("class", "").attr("class", this._prefix + "radialslider-svg-hide " + this._prefix + "rs-svg");
               (this.model.autoOpen) ?
                setTimeout(function () {
                    proxy.element.css("display", "none");
                }, this._isMobile ? 150 : 400)
                : proxy.element.css("display", "none");
            }
            else
                proxy.element.css("display", "none");
            this.model.autoOpen = false;
        },
        _setModel: function (options) {
            if (!ej.isNullOrUndefined(options["inline"]) && !options["inline"]) this.model.radius += 50;
            if (options.ticks) this.model.ticks = options.ticks;
            if (options.enableRoundOff) this.model.value(Math.round(ej.util.getVal(this.model.value)));
            this._refresh();
        },
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
        _isMobile: function () {
            return (this._prefix == "e-m-" ? true : false);
        },
    });
})(jQuery, Syncfusion);;
// JavaScript source code
/**
* @fileOverview Plugin to style the Html Tile elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    // ejmTile is the plugin name 
    // "ej.mobile.Tile" is "namespace.className" will hold functions and properties
    ej.widget("ejmRadialSlider", "ej.mobile.RadialSlider", {
        _setFirst: true,
        _rootCSS: "e-m-radialslider",
        defaults: {
            renderMode: "auto",
            theme: "auto",
            position: "rightcenter",
            strokeWidth: ej.isWindows() || ej.isFlat() ? 12 : 5
        },
        dataTypes:{
            ticks:"array"
        },
        _init: function () {
            this._docClick = false;
            this._orgEle = this.element.clone();
            this._renderEJMControl();
        },
        _renderEJMControl: function () {
            ej.setRenderMode(this);
            ej.setTheme(this);
            this._prefix = "e-m-";
            this._initialization();
            this.element.addClass("e-m-radialslider e-m-overlow e-m-abs e-m-user-select e-m-" + this.model.renderMode + " e-m-" + this.model.theme + " e-m-radial" + this.model.position + "")
            this._radialSVG = $(this._createSVGElements("svg", { "id": this._prefix + this._elementID + "-radial-slider-svg", "class": this._prefix + "rs-svg", "width": this._diameter, "height": this._diameter }));
            this._circleGroupElement = $(this._createSVGElements("g", { "id": "circlegroup" }));
            this._radialCircleCalculation(50, 40, this._tickCount, true, true, "path");
            this._textGroupElement = $(this._createSVGElements("g", { "id": "outerTextCircle", "transform": "translate(0, 4)" }));
            var space = this.model.renderMode == "ios7" ? 6.5 : 5;
            this._outerTextDirection = this._outerTextCalculation(this._tickCount, (this._startAngle + space), (this._endAngle - space));
            for (var i = 0; i < this._tickCount + 1; i++) {
                var k = (i == 0) && (this._startAngle == 0) && (this._endAngle == 360) ? (this._tickCount - 1) : i;
                var isCornerTicks = ((i == 0 || i == this._tickCount - 1) || ((this.ticks()[i] == this.value() && this.ticks()[0] == this.value()) || (this.ticks()[i] == this.value() && this.ticks()[this._tickCount - 1] == this.value())));
                if (this.model.position.charAt(0).toLowerCase() == "l")
                    this._outerTextDirection[i].textAlignment = "end";
                if (isCornerTicks)
                    this._outerTextDirection[i].textAlignment = "middle";
                if (this.model.position.charAt(0).toLowerCase() == "r" && !isCornerTicks)
                    this._outerTextDirection[i].textAlignment = "start";
                if (this.model.position.charAt(0).toLowerCase() == "b" || this.model.position.charAt(0).toLowerCase() == "t")
                    this._outerTextDirection[i].textAlignment = "middle";
                this._outerTextElement;
                if (i == this._tickCount) {
                    if ((this.ticks()[0] == this.value()))
                        continue;
                    else
                        this._outerTextElement = this._createSVGElements("text", { "stroke-width": 0.5, "x": this._outerTextDirection[i].X2, "y": this._outerTextDirection[i].Y2, "id": this._prefix + this._elementID + "-dynamic-text", "class": this._prefix + "dynamic-text", "textContent": this.value(), "text-anchor": this._outerTextDirection[i].textAlignment });
                }
                else
                    this._outerTextElement = this._createSVGElements("text", { "stroke-width": 0.5, "x": this._outerTextDirection[i].X2, "y": this._outerTextDirection[i].Y2, "class": this._prefix + "ticks-text", "textContent": this.ticks()[k], "text-anchor": this._outerTextDirection[i].textAlignment });
                this._textGroupElement.append(this._outerTextElement);
            }
            this._radialSliderWrapper.append(this._radialSVG.append(this._textGroupElement).append(this._circleGroupElement));
            if (this.model.autoOpen)
                this.element.css("display", "block");
            else
                this.element.css("display", "none");
            this._wireEvents(false);
        },
        _initialization: function () {
            this.element.addClass(this.model.cssClass);
            this._svgLink = "http://www.w3.org/2000/svg";
            this._startXY = this.model.radius;
            this._diameter = 2 * this.model.radius;
            this.model.radius = this._startXY;
            this._elementID = this.element.attr("id");
            this._tickCount = this.ticks().length;
            this._radialWidth = this._diameter;
            //Angle initialization
            if (this.model.position.charAt(0).toLowerCase() == "r") {
                this._startAngle = 90;
                this._endAngle = 270;
            }
            if (this.model.position.charAt(0).toLowerCase() == "l") {
                this._startAngle = 270;
                this._endAngle = 450;
            }
            if (this.model.position.charAt(0).toLowerCase() == "t") {
                this._startAngle = 0;
                this._endAngle = 180;
            }
            if (this.model.position.charAt(0).toLowerCase() == "b") {
                this._startAngle = 180;
                this._endAngle = 360;
            }
            this._radialSliderWrapper = ej.buildTag("div", {}, {}, { "class": this._prefix + "radail-slider-wrapper" });
            this.element.append(this._radialSliderWrapper);
            this._positionRadial();
        },
        resize: function (e) {
            var proxy = this;
            window.setTimeout(function () {
                proxy._positionRadial();
            }, ej.isAndroid() ? 200 : 0);
        },
        _documentClick: function (evt) {
            if ($($(evt.target).closest("svg.e-m-rs-svg")).length == 0 && this._docClick) {
                this.hide();
                this._docClick = false;
            }
            if (this.element.hasClass("e-m-slider-show"))
                this._docClick = true;
        },
        _positionRadial: function () {
            if (this.model.position.charAt(0).toLowerCase() == "r")
                this.element.addClass("e-m-radialright").css("right", -this._startXY + "px");
            if (this.model.position == "rightcenter" || this.model.position == "leftcenter")
                this.element.css({ "top": (window.innerHeight / 2) - (this.model.radius) + "px" });
            if (this.model.position.charAt(0).toLowerCase() == "b") {
                this.element.css({ "bottom": -(this.model.radius) + "px" });
                if (this.model.position == "bottomcenter")
                    this.element.css({ "left": (window.innerWidth / 2) - (this.model.radius) + "px" });
                if (this.model.position == "bottomright")
                    this.element.css({ "right": 10 + "px" });
                if (this.model.position == "bottomleft")
                    this.element.css({ "left": 10 + "px" });
            }
            if (this.model.position.charAt(0).toLowerCase() == "t") {
                this.element.css({ "top": -(this.model.radius) + "px" });
                if (this.model.position == "topcenter")
                    this.element.css({ "left": (window.innerWidth / 2) - (this.model.radius) + "px" });
                else if (this.model.position == "topleft")
                    this.element.css({ "left": 10 + "px" });
                else if (this.model.position == "topright")
                    this.element.css("right", 10 + "px");
            }
            if (this.model.position.charAt(0).toLowerCase() == "l")
                this.element.css({ "left": -(this.model.radius) + "px" });
        },

        _pathDirection: function (x1, y1, radius, lArc, x2, y2, dEndX, dEndY, dRadius, dStartX, dStartY, isDynamic) {
            lArc = isDynamic ? 0 : lArc;
            return ["M", x1, y1, "A", radius, radius, "0", lArc, "1", x2, y2, "L", dEndX, dEndY, "A", dRadius, dRadius, "1", lArc, "0", dStartX, dStartY, "z"].join(" ");
        },

        _radialCircleCalculation: function (dis1, dis2, length, isOuterLine, isInnerLine, element) {
            var space = this.model.renderMode == "ios7" ? 6.5 : this.model.renderMode == "windows" ? 2.5 : 5;
            var startAngle = this._startAngle + space, endAngle = startAngle + (((this._endAngle - space) - (this._startAngle + space)) / (length - 1));
            this._point = (((this._endAngle - space) - (this._startAngle + space)) / (length - 1));
            var endValueCal = endAngle, startValueCal = endValueCal;
            this._degPoint = [];
            this._degPoint.push(startAngle);
            for (var j = 0; j < length; j++) {
                var line = {};
                startAngle = startAngle * Math.PI / 180.0;
                endAngle = endAngle * Math.PI / 180.0;
                this._degPoint.push(endValueCal);
                startAngle = endValueCal;
                endAngle = endValueCal + this._point;
                endValueCal += this._point;
            }
            for (var i = 0; i < length; i++) {
                var k = i + 1;
                var lastValue = i == (this.ticks().length - 1) ? this.ticks()[this.ticks().length - 1] + 0.5 : this.ticks()[k];
                var firstValue = i == 0 ? this.ticks()[i] - 0.5 : this.ticks()[i];
                var ticksSecond = firstValue <= 0 && firstValue >= this.value() ? true : false;
                var ticksFirst = lastValue <= 0 && lastValue <= this.value() ? true : false;
                var condition = ticksSecond && ticksFirst ? true : (firstValue <= this.value() && this.value() < lastValue);
                if (condition) {
                    if (this.ticks()[i] != this.value()) {
                        var difference = this._point / (this.ticks()[k] - this.ticks()[i]);
                        var angleDifference = this.ticks()[k] - this.value();
                        this._startValueAngle = this._degPoint[k] - (difference * angleDifference);
                        this._radialSliderCircle();
                    }
                    else {
                        this._startValueAngle = this._degPoint[i];
                        this._radialSliderCircle();
                    }
                }
            }
        },
        _radialSliderCircle: function () {
            var direction = this._calculateCircleDirection(this._startAngle, (this.model.renderMode == "windows" || this.model.renderMode == "flat") ? this._startValueAngle - 2.5 : this._startValueAngle);
            this._fillCircleElement = this._createSVGElements("path", { "d": direction, "class": this._prefix + "rs-fill-circle" });
            this._circleGroupElement.append(this._fillCircleElement);
            var direction = this._calculateCircleDirection(((this.model.renderMode == "windows" || this.model.renderMode == "flat") ? this._startValueAngle + 2.5 : this._startValueAngle), this._endAngle);
            this._radialCircleDefault = this._createSVGElements("path", { "d": direction, "class": this._prefix + "rs-circle-default" });
            this._circleGroupElement.append(this._radialCircleDefault);
            if ((this.model.renderMode == "windows" || this.model.renderMode == "flat")) {
                var direction = this._calculateCircleDirection(this._startValueAngle - 2.5, this._startValueAngle + 2.5);
                this._markerElement = this._createSVGElements("path", { "d": direction, "class": this._prefix + "rs-marker", "stroke-width": ej.isFlat() ? (this.model.strokeWidth - 5) : "none" });
                this._circleGroupElement.append(this._markerElement);
            }
            else {
                var radius = (0.5 * Math.min(this._diameter, this._diameter));
                radius = radius - ((radius - this._arcRadius) / 2);
                if (this.model.renderMode == "android") {
                    var startPoint = this._polarToCartesian(this._startXY, this._startXY, radius, this._startValueAngle);
                    this._smallCircle = this._createSVGElements("circle", { "class": "e-m-rs-marker", "cx": (startPoint.x), "cy": startPoint.y, "r": 6 });
                    this._outerSmallCircel = this._createSVGElements("circle", { "class": "e-m-marker-large", "cx": (startPoint.x), "cy": startPoint.y, "r": 15 });
                    this._circleGroupElement.append(this._outerSmallCircel).append(this._smallCircle);
                }
                else {
                    this._iosMarkerElement = ej.buildTag("div.e-m-rs-marker");
                    var startPoint = this._polarToCartesian(this._startXY, this._startXY, radius, this._startValueAngle);
                    this._radialSliderWrapper.append(this._iosMarkerElement);
                    this._iosMarkerElement.css({ "left": startPoint.x - 15, "top": startPoint.y - 15 });
                }
            }
        },

        _refresh: function () {
            this._destroy();
            this.element.removeAttr("style");
            this.element.addClass("e-m-radialslider e-js");
            this._renderEJMControl();
        },
        //events part
        _createDelegates: function () {
            this._documentDelegate = $.proxy(this._documentClick, this);
            this._touchStartDelegate = $.proxy(this._touchStartHandler, this);
            this._markerMoveDelegate = $.proxy(this._markerMoveHandler, this);
            this._touchEndDelegate = $.proxy(this._touchEndHandler, this);
        },
        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([this._radialSVG], [ej.startEvent()], [this._touchStartDelegate], remove);
            ej.listenEvents([$(document)], ["click"], [this._documentDelegate], remove);
            if (this.model.renderMode == "ios7")
                ej.listenEvents([this._iosMarkerElement], [ej.startEvent()], [this._touchStartDelegate], remove);
        },

        _touchEndHandler: function (e) {
            e = e.touches ? e.changedTouches[0] : e;
            ej.listenEvents([this._radialSVG], [ej.moveEvent()], [this._markerMoveDelegate], true);
            if (this.model.renderMode == "ios7")
                ej.listenEvents([this._iosMarkerElement], [ej.moveEvent()], [this._markerMoveDelegate], true);
            this._tapHandlerEvent(e);
            this._docClick = false;
        },
        _touchStartHandler: function (e) {
            ej.blockDefaultActions(e);
            if (ej.isTouchDevice())
                e = e.touches ? e.touches[0] : e;
            if (this.model.start)
                this._trigger("start", { value: this.value() });
            ej.listenEvents([this._radialSVG, this._radialSVG], [ej.moveEvent(), ej.endEvent()], [this._markerMoveDelegate, this._touchEndDelegate], false);
            if (this.model.renderMode == "ios7")
                ej.listenEvents([this._iosMarkerElement, this._iosMarkerElement], [ej.moveEvent(), ej.endEvent()], [this._markerMoveDelegate, this._touchEndDelegate], false);
        },
        _markerMoveHandler: function (e) {
            ej.blockDefaultActions(e);
            if (ej.isTouchDevice())
                e = e.touches ? e.touches[0] : e;
            var baseRect = $("#" + this._prefix + this._elementID + "-radial-slider-svg").offset();
            var y2 = e.clientY, x2 = e.clientX, y1 = baseRect.top + (this._radialWidth / 2), x1 = baseRect.left + (this._radialWidth / 2);
            this._dynamicAngleCalculation(x1, x2, y1, y2);
            var space = this.model.renderMode == "ios7" ? 6.5 : this.model.renderMode == "windows" ? 2.5 : 5;
            if ((this.model.position.charAt(0).toLowerCase() == "l" && (this._angle > (270 + space) || this._angle < (90 - space))) || (this.model.position.charAt(0).toLowerCase() != "l" && (this._angle > ((360 - (this._endAngle - space))) && this._angle < ((360 - (this._startAngle + space)))))) {
                ((this.model.renderMode == "windows" || this.model.renderMode == "flat") ? this._dynamicWindowsRadial() : this._dynamicIOSandAndroidRadial());
                if (this.model.slide) {
                    var selectValue = this._ticksCalculation();
                    this._trigger("slide", { value: selectValue, selectedValue: this.value() });
                }
            }
        },

        _calculateCircleDirection: function (startAngle, endAngle, isDynamic, isFill, isDefault) {
            var radius = 0.5 * Math.min(this._diameter, this._diameter),
            dradius = radius - this.model.strokeWidth;
            this._arcRadius = dradius;
            var x1, y1, x2, y2, midx, dStartX, dStartY, dEndX, dEndY, longArc;
            startAngle = startAngle * Math.PI / 180.0;
            endAngle = endAngle * Math.PI / 180.0;
            longArc = endAngle - startAngle < Math.PI ? 0 : 1;
            var midAngle = (startAngle + endAngle) / 2;
            x1 = this._startXY + radius * Math.cos(startAngle);
            y1 = this._startXY + radius * Math.sin(isDynamic ? -startAngle : startAngle);
            x2 = this._startXY + radius * Math.cos(endAngle);
            y2 = this._startXY + radius * Math.sin(isDynamic ? -endAngle : endAngle);
            dStartX = this._startXY + dradius * Math.cos(startAngle);
            dStartY = this._startXY + dradius * Math.sin(isDynamic ? -startAngle : startAngle);
            dEndX = this._startXY + dradius * Math.cos(endAngle);
            dEndY = this._startXY + dradius * Math.sin(isDynamic ? -endAngle : endAngle);
            leftPosition = false;
            if (this.model.position.charAt(0).toLowerCase() == "l"  || this.model.position.charAt(0).toLowerCase() == "b")
                leftPosition = true;
            return this._pathDirection(x1, y1, radius, longArc, x2, y2, dEndX, dEndY, dradius, dStartX, dStartY, leftPosition);
        },

        _dynamicWindowsRadial: function () {
            var direction = this._calculateCircleDirection(360 - this._startAngle, this._angle + 2.5, true, true);
            $(this._fillCircleElement).attr("d", direction);
            var direction = this._calculateCircleDirection(this._angle + 2.5, this._angle - 2.5, true, false);
            $(this._markerElement).attr("d", direction);
            var direction = this._calculateCircleDirection(this._angle - 2.5, (360 - this._endAngle), true, false, true);
            $(this._radialCircleDefault).attr("d", direction);
        },

        _dynamicIOSandAndroidRadial: function () {
            var direction = this._calculateCircleDirection(this._angle, (360 - this._endAngle), true, false, true);
            $(this._radialCircleDefault).attr("d", direction);
            var direction = this._calculateCircleDirection((360 - this._startAngle), this._angle, true, true);
            $(this._fillCircleElement).attr("d", direction);
            var radius = (0.5 * Math.min(this._diameter, this._diameter));
            radius = radius - ((radius - this._arcRadius) / 2);
            if (this.model.renderMode == "android") {
                var startPoint = this._polarToCartesian(this._startXY, this._startXY, radius, this._angle, true);
                $(this._smallCircle).attr({ "cx": (startPoint.x), "cy": startPoint.y });
                $(this._outerSmallCircel).attr({ "cx": (startPoint.x), "cy": startPoint.y });
            }
            else {
                var startPoint = this._polarToCartesian(this._startXY, this._startXY, radius, this._angle, true);
                this._iosMarkerElement.css({ "left": startPoint.x - 15, "top": startPoint.y - 15 });
            }
        },

    })
    ej.mobile.RadialSlider.Position = {
        RightCenter: "rightcenter",
        RightTop: "righttop",
        RightBottom: "rightbottom",
        LeftCenter: "leftcenter",
        LeftTop: "lefttop",
        LeftBottom: "leftbottom",
        TopLeft: "topleft",
        TopRight: "topright",
        TopCenter: "topcenter",
        BottomLeft: "bottomleft",
        BottomRight: "bottomright",
        BottomCenter: "bottomcenter"
    };

    $.extend(true, ej.mobile.RadialSlider.prototype, ej.RadialSliderBase.prototype);

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html rating elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    // ejmRating is the plugin name 
    // "ej.mobile.Rating" is "namespace.className" will hold functions and properties

    ej.widget("ejmRating", "ej.mobile.Rating", {
        _setFirst: true,
        _rootCSS: "e-m-rating",

        defaults: {
            maxValue: 5,
            minValue: 0,
            value: 1,
            incrementStep: 1,
            precision: "full",
            enabled: true,
            renderMode: "auto",
            shape: "star",
            shapeWidth: 25,
            shapeHeight: 25,
            spaceBetweenShapes: 15.0,
            tap: null,
            change: null,
            touchMove: null,
            orientation: 'horizontal',
            readOnly: false,
            enablePersistence: false,
            backgroundColor: null,
            selectionColor: null,
            hoverColor: null,
            borderColor: null
        },
        dataTypes: {
            enabled: "boolean",
            renderMode: "enum",
            precision: "enum",
            shape: "enum",
            orientation: "enum",
            readOnly: "boolean",
            enablePersistence: "boolean"
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),

        _init: function () {
            //Get each rating properties
            if (!App.activePage)
                App.createAppView();
            this._cloneElement = this.element.clone();
            this._renderControl();
            this._wireEvents(false);
            this._isLoaded = true;
        },

        _initObjects: function () {
            this.element.css({ "display": "inline-block" });
            this._canvasElement = this.element.find("canvas")[0];
            this._renderRatingControl();
            this._hiddenField = this.element.find("input")[0];
        },

        _renderControl: function () {
            ej.setRenderMode(this);
            this.element.addClass('e-m-' + this.model.renderMode);
            if (!this.model.enabled)
                this.element.addClass('e-m-state-disabled');
            this._renderColorCodes();
            this["_" + this.model.precision + "PrecisionProperties"]();
            this._canvasElement = ej.buildTag('canvas');
            this._canvasElement.css("-ms-touch-action", "none");
            this.element.append(this._canvasElement);
            this._hiddenField = ej.buildTag("input ", {}, {}, { "type": "hidden", "value": this.model.minValue + this.value() });
            this.element.append(this._hiddenField);
            this._initObjects();
        },

        _fullPrecisionProperties: function () {
            var checkValue = Math.round(this.value());
            this.value(checkValue - (checkValue % this.model.incrementStep));
            this._currentIndex = Number(this.value() / this.model.incrementStep);
        },

        _exactPrecisionProperties: function () {
            var checkValue = Math.ceil(this.value());
            this.value(Math.round(this.value(), 1));
            this._pricisionProperties(checkValue);
        },

        _halfPrecisionProperties: function () {
            var checkValue = Math.ceil(this.value());
            this.value(this.value() - (this.value() % (Number(this.model.incrementStep) / 2)));
            this._pricisionProperties(checkValue);
        },

        _pricisionProperties: function (checkValue) {
            if (this.value() % this.model.incrementStep != 0) {
                checkValue = checkValue + (checkValue % this.model.incrementStep);
                currentItemIndex = Number(checkValue / this.model.incrementStep);
            }
            else {
                checkValue = this.value();
                currentItemIndex = Number(checkValue / this.model.incrementStep);
            }
        },

        _hexTorgb: function (result) {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(result);
            result[1] = parseInt(result[1], 16);
            result[2] = parseInt(result[2], 16);
            result[3] = parseInt(result[3], 16);
            color = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": result[1], "G": result[2], "B": result[3], "A": 0, "Name": "64b7b7b7" } }] });
            return color;
        },
        _renderColorCodes: function () {
            var bgcolor, shapebgcolor, shapeselectcolor, shapehovercolor, glowcolor;
            if (this.model.renderMode == "ios7") {
                    bgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 13, "G": 122, "B": 255, "A": 0, "Name": "Transparent" } }] });
                    shapebgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 238, "G": 238, "B": 238, "A": 100, "Name": "64ff9d09" } }] });
                    shapeselectcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": 0, "Color": { "R": 13, "G": 122, "B": 255, "A": 100, "Name": "64c1dff7" } }] });
                    shapehovercolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": 0, "Color": { "R": 13, "G": 122, "B": 255, "A": 100, "Name": "64c1dff7" } }] });
            }
            else if (this.model.renderMode == "android") {
                    bgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 255, "G": 255, "B": 255, "A": 100, "Name": "6433b5e5" } }] });
                    shapebgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 188, "G": 188, "B": 188, "A": 40, "Name": "64b7b7b7" } }] });
                    shapeselectcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 54, "G": 190, "B": 240, "A": 100, "Name": "6433b5e5" } }] });
                    shapehovercolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 54, "G": 190, "B": 240, "A": 100, "Name": "6433b5e5" } }] });
                    glowcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 74, "G": 188, "B": 230, "A": 100, "Name": "644abce6" } }] });
            }
            else if (this.model.renderMode == "windows") {
                    bgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 255, "G": 255, "B": 255, "A": 100, "Name": "6433b5e5" } }] });
                    shapebgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 209, "G": 207, "B": 207, "A": 100, "Name": "64b7b7b7" } }] });
                    shapeselectcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 70, "G": 70, "B": 70, "A": 100, "Name": "6433b5e5" } }] });
                    shapehovercolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 70, "G": 70, "B": 70, "A": 100, "Name": "6433b5e5" } }] });

            }
            else if (this.model.renderMode == "flat") {
                bgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 51, "G": 181, "B": 229, "A": 100, "Name": "6433b5e5" } }] });
                shapebgcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 242, "G": 242, "B": 242, "A": 100, "Name": "64b7b7b7" } }] });
                shapeselectcolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 243, "G": 133, "B": 41, "A": 100, "Name": "6433b5e5" } }] });
                shapehovercolor = JSON.stringify({ "ColorInfo": [{ "ColorStop": NaN, "Color": { "R": 243, "G": 133, "B": 41, "A": 100, "Name": "6433b5e5" } }] });
            }
            if (typeof (this.model.backgroundColor) == "string")
                shapebgcolor = this._hexTorgb(this.model.backgroundColor);
            if (typeof (this.model.selectionColor) == "string")
                shapeselectcolor = this._hexTorgb(this.model.selectionColor);
            if (typeof (this.model.hoverColor) == "string")
                shapehovercolor = this._hexTorgb(this.model.hoverColor);
            bgcolor = bgcolor.replace('null', NaN);
            shapebgcolor = shapebgcolor.replace('null', NaN);
            shapeselectcolor = shapeselectcolor.replace('null', NaN);
            shapehovercolor = shapehovercolor.replace('null', NaN);
            this._backgroundColor = eval("(" + bgcolor + ")");
            this._shapeBackgroundColor = eval("(" + shapebgcolor + ")");
            this._shapeSelectedColor = eval("(" + shapeselectcolor + ")");
            this._shapeHoverColor = eval("(" + shapehovercolor + ")");
            if (this.model.renderMode == "android") {
                glowcolor = glowcolor.replace('null', NaN);
                this.model.glowColor = eval("(" + glowcolor + ")");
            }
        },

        _createDelegates: function () {
            this._canvasMDDelegate = $.proxy(this._canvasMouseDownHandler, this);
            this._canvasMMDelegate = $.proxy(this._canvasMouseMoveHandler, this);
            this._canvasMUDelegate = $.proxy(this._canvasMouseUpHandler, this);
            this._documentEndDelegate = $.proxy(this._documentEndHandler, this);
        },

        _wireEvents: function (remove) {
            if (this.model.enabled) {
                this._createDelegates();
                if (!this.model.readOnly)
                    ej.listenEvents([this._canvasElement, this._canvasElement, document], [ej.startEvent(), ej.endEvent(), ej.endEvent()], [this._canvasMDDelegate, this._canvasMUDelegate, this._documentEndDelegate], remove);
            }
        },

        _canvasMouseDownHandler: function (evt) {
            ej.blockDefaultActions(evt);
            this._move = false;
            this._canvasAction = "mouseDown";
            ej.listenTouchEvent(this._canvasElement, ej.moveEvent(), this._canvasMMDelegate, false);
            var data = { value: this.value() };
            if (this.model.tap)
                this._trigger("tap", data);
        },

        _canvasMouseMoveHandler: function (evt) {
            ej.blockDefaultActions(evt);
            this._move = true;
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            this._canvasAction = "mouseMove";
            this._getCursorPosition(evt);
            this._mouseMove(evt);
        },

        _mouseMove: function (evt) {
            this._getCursorPosition(evt);
            this._mousepos = (this.model.orientation == "vertical") ? this._mousePosition.y : this._mousePosition.x;
            var _value;
            var shapesCount = 0, st = 0;
            this["_" + this.model.orientation + "MouseMoveOrUpProperties"]();
            var roundValue = this._getRoundOffValue(this._value);
            this._currentMouseMove = roundValue;
            this._canvasContext.canvas.title = this.model.minValue + parseFloat(roundValue);
            this._canvasContext.canvas.alt = this.model.minValue + parseFloat(roundValue);
            this._renderRatingControl();
            var data = { value: this.value() };
            if (this.model.touchMove)
                this._trigger("touchMove", data);
        },

        _verticalMouseMoveOrUpProperties: function () {
            shapesCount = Math.floor(this._mousepos - 1) / (this.model.shapeHeight + this.model.spaceBetweenShapes);
            this._value = ((this._mousepos - (shapesCount * this.model.spaceBetweenShapes)) * this.model.incrementStep / this.model.shapeHeight).toFixed(1);
        },

        _horizontalMouseMoveOrUpProperties: function () {
            shapesCount = Math.floor(this._mousepos - 1) / (this.model.shapeWidth + this.model.spaceBetweenShapes);
            this._value = ((this._mousepos - (shapesCount * this.model.spaceBetweenShapes)) * this.model.incrementStep / this.model.shapeWidth).toFixed(1);
        },

        _canvasMouseUpHandler: function (evt) {
            ej.blockDefaultActions(evt);
            if (ej.isTouchDevice())
                evt = evt.changedTouches ? evt.changedTouches[0] : evt;
            ej.listenTouchEvent(this._canvasElement, ej.moveEvent(), this._canvasMMDelegate, true);
            this._canvasAction = "mouseUp";
            this._getCursorPosition(evt);
            this._mouseUp(evt);
        },

        _documentEndHandler: function (evt) {
            if ($(evt.target).closest('.e-m-rating').length == 0) {
                if (ej.isTouchDevice())
                    evt = evt.touches ? evt.touches[0] : evt;
                ej.listenTouchEvent(this._canvasElement, ej.moveEvent(), this._canvasMMDelegate, true);
            }
        },

        _mouseUp: function (evt) {
            if (this.model.precision == "exact" && this._move) {
                if (this.model.change)
                    this._trigger("change", data);
                return;
            }
            this._move = false;
            var noofshapes = this._shapesCount, st = 0;
            this._mousepos = (this.model.orientation == "vertical") ? this._mousePosition.y : this._mousePosition.x;
            this._getCursorPosition(evt);
            var _value = 0;
            var shapesCount = 0;
            var width = this.model.shapeWidth, height = this.model.shapeHeight;
            this["_" + this.model.orientation + "MouseMoveOrUpProperties"]();
            var roundValue = this._getRoundOffValue(this._value);
            this.value(roundValue);
            this._currentMouseMove = roundValue;
            this._canvasContext.canvas.title = this.model.minValue + parseFloat(roundValue);
            this._canvasContext.canvas.alt = this.model.minValue + parseFloat(roundValue);
            $(this._hiddenField).val(this._canvasContext.canvas.title);
            this._renderRatingControl();
            var data = { value: this.value() };
            if (this.model.change)
                this._trigger("change", data);
        },

        _renderRatingControl: function () {
            var elem = this._canvasElement;
            if (typeof window.G_vmlCanvasManager != "undefined")
                elem = window.G_vmlCanvasManager.initElement(elem);
            if (!elem || !elem.getContext)
                return;
            if (this._canvasElement.getContext) {
                this._canvasContext = this._canvasElement.getContext('2d');
                var context = this._canvasContext;
                this._shapesCount = Math.floor((this.model.maxValue - this.model.minValue) / this.model.incrementStep);
                this["_" + this.model.orientation + "RenderRatingProperties"]();
                this._canvasWidth = this.tempCanvasWidth;
                this._canvasHeight = this.tempCanvasHeight;
                context.canvas.width = this._canvasWidth;
                context.canvas.height = this._canvasHeight;
                var gradient = context.createLinearGradient(0, 0, this._canvasWidth, this._canvasHeight);
                if (this._backgroundColor.ColorInfo[0].ColorStop.toString() != "NaN") {
                    $.each(this._backgroundColor.ColorInfo, function (context, index, color) {
                        gradient.addColorStop(color.ColorStop.toString() == "NaN" ? 0 : color.ColorStop, "#" + (color.Color.Name ? ej.hexFromRGB(color.Color) : color.Color));
                    });
                }
                if (this.model.orientation == "horizontal")
                    context.translate(7, 15);
                else
                    context.translate(15, 7);
                context.fillStyle = gradient;
                context.fillRect(0, 0, context.canvas.width, context.canvas.height);
                this._renderRatingShapes(context);

            }
        },

        _verticalRenderRatingProperties: function () {
            this._renderRatingProperties(this.model.shapeWidth, this.model.shapeHeight, this._canvasWidth, this._canvasHeight);
        },

        _horizontalRenderRatingProperties: function () {
            this._renderRatingProperties(this.model.shapeHeight, this.model.shapeWidth, this._canvasHeight, this._canvasWidth);
        },

        _renderRatingProperties: function (sHorLen, sVerLen, cHorLen, cVerLen) {
            cHorLen = sHorLen + 3 + 30;
            cVerLen = this._shapesCount * sVerLen;
            cVerLen += ((this._shapesCount) * this.model.spaceBetweenShapes);
            this.tempCanvasWidth = (this.model.orientation == "horizontal") ? cVerLen : cHorLen;
            this.tempCanvasHeight = (this.model.orientation == "horizontal") ? cHorLen : cVerLen;
        },

        _renderRatingShapes: function (context) {
            var noOfShapes = this._shapesCount, noOfRatingShapes;
            var currValue = this.value();
            noOfRatingShapes = noOfShapes;
            st = 0;
            var fullValue = Math.floor(currValue / this.model.incrementStep);
            var partialValue = currValue % this.model.incrementStep;
            currValue = this.value();
            for (st = st; st < noOfRatingShapes; st++) {
                context.save();
                this._drawRatingShape(context, st);
                if (st < fullValue) {
                    this._notClipped = true;
                    this._setClippingRegion(context, st, this.model.shapeWidth, true);
                }
                else if (partialValue != 0) {
                    this._notClipped = true;
                    var value = partialValue * this.model.shapeWidth / this.model.incrementStep;
                    this._setClippingRegion(context, st, value, true);
                    partialValue = 0;
                }
                else
                    this._setClippingRegion(context, st, this.model.shapeWidth, false);
                context.restore();
            }
        },

        _drawRatingShape: function (context, currentItemIndex) {
            this._drawShapeProperties(currentItemIndex);
            this._drawShape(context, this.x1, this.y1, this.width, this.height);
        },

        _drawShapeProperties: function (currentItemIndex) {
            this.width = this.model.shapeWidth, this.height = this.model.shapeHeight;
            this.x1, this.y1;
            if (this.model.orientation == "vertical") {
                this.x1 = 1;
                this.y1 = currentItemIndex * this.height + currentItemIndex * this.model.spaceBetweenShapes;
            }
            else {
                this.x1 = currentItemIndex * this.width + (currentItemIndex) * this.model.spaceBetweenShapes;
                this.y1 = 0;
            }
        },

        _drawShape: function (context, x1, y1, width, height) {
            context.beginPath();
            switch (this.model.shape) {
                case "heart": this._drawHeartenShape(context, x1, y1, width, height); break;
                case "pentagon": this._drawPentagonShape(context, x1, y1, width, height); break;
                case "diamond": this._drawDiamondShape(context, x1, y1, width, height); break;
                case "triangle": this._drawTriangleShape(context, x1, y1, width, height); break;
                case "square": this._drawSquareShape(context, x1, y1, width, height); break;
                case "circle": this._drawCircleShape(context, x1, y1, width, height); break;
                default: this._drawStarShape(context, x1, y1, width, height); break;
            }
            context.closePath();
            if (this.model.borderColor) {
                context.strokeStyle = this.model.borderColor;
                context.stroke();
            }
            else
                if (this.model.renderMode != "windows") {
                    context.strokeStyle = (this.model.renderMode == "ios7") ? this._generateGradient(context, this._backgroundColor, x1, y1) : "";
                    context.stroke();
                }
            context.save();
            context.fillStyle = this._generateGradient(context, this._shapeBackgroundColor, x1, y1);
            context.fill();
            context.restore();
        },

        _drawStarShape: function (context, x1, y1, width, height) {
            context.moveTo(x1 + (width / 6), y1 + height); // 1
            context.lineTo(x1 + width, y1 + (height / 3)); // 2
            context.lineTo(x1, y1 + (height / 3)); // 3
            context.lineTo(x1 + width - (width / 6), y1 + height); // 4
            context.lineTo(x1 + width / 2, y1); // 5
            context.lineWidth = (this.model.renderMode == "android") ? 2 : 1;
        },

        _drawCircleShape: function (context, x1, y1, width, height) {
            if (this._draw) {
                context.arc(x1 + width / 2, y1 + height / 2, height / 9, 0, Math.PI * 2, true);
                this._draw = false;
            }
            else
                context.arc(x1 + width / 2, y1 + height / 2, height / 2, 0, Math.PI * 2, true);
        },

        _drawSquareShape: function (context, x1, y1, width, height) {
            context.moveTo(x1, y1);
            context.lineTo(x1 + width, y1);
            context.lineTo(x1 + width, y1 + height);
            context.lineTo(x1, y1 + height);
        },

        _drawTriangleShape: function (context, x1, y1, width, height) {
            context.moveTo(x1 + width / 2, y1);
            context.lineTo(x1 + width, y1 + height);
            context.lineTo(x1, y1 + height);
        },

        _drawDiamondShape: function (context, x1, y1, width, height) {
            context.moveTo(x1 + width / 2, y1);
            context.lineTo(x1 + width, y1 + height / 2);
            context.lineTo(x1 + width / 2, y1 + height);
            context.lineTo(x1, y1 + height / 2);
        },

        _drawPentagonShape: function (context, x1, y1, width, height) {
            context.moveTo(x1 + width / 2, y1);
            context.lineTo(x1 + width, y1 + height / 2);
            context.lineTo(x1 + width - width / 4, y1 + height);
            context.lineTo(x1 + width / 4, y1 + height);
            context.lineTo(x1, y1 + height / 2);
        },

        _drawHeartenShape: function (context, x1, y1, width, height) {
            x1 += width / 2;
            y1 += height / 4;
            height -= (height * .25);
            context.beginPath();
            context.moveTo(x1, y1);
            context.bezierCurveTo(x1, y1 - (height * .4), x1 - width / 2, y1 - (height * .4), x1 - width / 2, y1);
            context.bezierCurveTo(x1 - width / 2, y1 + height / 2, x1, y1 + height / 2, x1, y1 + height);
            context.bezierCurveTo(x1, y1 + height / 2, x1 + width / 2, y1 + height / 2, x1 + width / 2, y1);
            context.bezierCurveTo(x1 + width / 2, y1 - (height * .4), x1, y1 - (height * .4), x1, y1);
        },

        _setClippingRegion: function (context, currentItemIndex, clipValue, isClip) {
            context.save();
            this._drawShapeProperties(currentItemIndex);
            context.clip();
            context.beginPath();
            if (this._canvasAction == "mouseMove") {
                var posX, posY;
                if (this._mouseMoveValue != null) {
                    posX = this._mouseMoveValue * this.width + Math.floor(this._mouseMoveValue) * this.model.spaceBetweenShapes - 7;
                    posY = this._mouseMoveValue * this.height + Math.floor(this._mouseMoveValue) * this.model.spaceBetweenShapes - 7;
                }
                else {
                    posX = this._mousePosition.x - 7;
                    posY = this._mousePosition.y - 7;
                }
                context.fillStyle = this._generateGradient(context, this._shapeHoverColor, this.x1, this.y1);
                context.fill();
                (this.model.orientation == "vertical") ? context.fillRect(0, 0, this.width, posY) : context.fillRect(0, 0, posX, this.height);
            }
            else {
                if (isClip) {
                    context.fillStyle = this._generateGradient(context, this._shapeSelectedColor, this.x1, this.y1);
                    context.fill();
                    isClip = false;
                }
                (this.model.orientation == "vertical") ? context.fillRect(this.x1, this.y1, this.width, clipValue) : context.fillRect(this.x1, this.y1, clipValue, this.height);
            }
            context.restore();
        },

        _generateGradient: function (context, gradientObj, x1, y1) {
            var gradient = (this.model.renderMode == "ios7") ? context.createLinearGradient(x1 + this.model.shapeWidth / 2, y1, x1 + this.model.shapeWidth / 2, y1 + this.model.shapeHeight) : context.createLinearGradient(x1 + this.model.shapeWidth / 2, y1 + this.model.shapeHeight, x1 + this.model.shapeWidth / 2, y1);
            $.each(gradientObj.ColorInfo, function (index, color) {
                gradient.addColorStop(color.ColorStop.toString() == "NaN" ? 0 : color.ColorStop, "#" + (color.Color.Name ? ej.hexFromRGB(color.Color) : color.Color));
            });
            return gradient;
        },

        _clearRatingControl: function (context) {
            context.clearRect(0, 0, this._canvasWidth, this._canvasHeight);
        },

        _getCursorPosition: function (e) {
            var xPos, yPos;
            if (e.pageX || e.pageY) {
                xPos = e.pageX;
                yPos = e.pageY;
            }
            else {
                xPos = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
                yPos = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
            }
            xPos -= $(this._canvasElement).offset().left;
            yPos -= $(this._canvasElement).offset().top;
            this._mousePosition = { x: xPos, y: yPos };
            return this._mousePosition;
        },

        _getRoundOffValue: function (currValue) {
            var returnValue;
            if (this.model.precision == "full") {
                returnValue = Math.round(currValue);
                returnValue = returnValue - (returnValue % this.model.incrementStep);
            }
            else if (this.model.precision == "exact")
                returnValue = currValue;
            else {
                currValue = Math.round(currValue / .5) * .5;
                currValue = currValue - (currValue % (this.model.incrementStep / 2));
                returnValue = currValue;
            }
            return returnValue;
        },


        _setRenderMode: function (mode) {
            this.model.renderMode = mode;
            this._refresh();
        },

        _changeShape: function (shape) {
            this.model.shape = shape;
            this._renderRatingControl();
        },

        _refresh: function () {
            this._clearElement();
            this.element.addClass("e-m-rating");
            this._renderControl();
            this._wireEvents(false);
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                switch (prop) {
                    case "renderMode": this._setRenderMode(options.renderMode); break;
                    case "shape": this._changeShape(options.shape); break;
                    case "shapeWidth": this._refresh(); break;
                    case "shapeHeight": this._refresh(); break;
                    case "incrementStep": this._refresh(); break;
                    case "orientation": this._refresh(); break;
                    case "precision": this._refresh(); break;
                    case "value": this._refresh(); break;
                    case "maxValue": this._refresh(); break;
                    case "enabled": this._refresh(); break;
                    case "readOnly": this._refresh(); break;
                    case "backgroundColor": this._refresh(); break;
                    case "selectionColor": this._refresh(); break;
                    case "hoverColor": this._refresh(); break;
                }
            }
        },
        _clearElement: function () {
            this._wireEvents(true);
            this.element.removeAttr("class");
            this.element.removeAttr("style");
            this.element.html("");
        },
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this._cloneElement.insertBefore(this.element).removeClass('e-m-rating');
            this.element.remove();
        },

        /*---------------Public Methods---------------*/

        disable: function () {
            this.model.enabled = false;
            this._refresh();
        },

        enable: function () {
            this.model.enabled = true;
            this._refresh();
        },

        show: function () {
            if (!this.model.enabled) return false;
            this.element.css("display", "block");
        },

        hide: function () {
            if (!this.model.enabled) return false;
            this.element.css("display", "none");
        },

        getValue: function () {
            return this.value();
        },

        reset: function () {
            this.value(0);
            this._refresh();
        },

        setValue: function (value) {
            this.value(value);
            this._refresh();
        }
        /*---------------Public Methods End---------------*/

    });

    ej.mobile.Rating.Precision = {
        Full: "full",
        Exact: "exact",
        Half: "half"
    };


    ej.mobile.Rating.Shape = {
        Star: "star",
        Circle: "circle",
        Diamond: "diamond",
        Heart: "heart",
        Pentagon: "pentagon",
        Square: "square",
        Triangle: "triangle"
    };

    ej.mobile.Rating.Orientation = {
        Horizontal: "horizontal",
        Vertical: "vertical"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Rotator elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejmRotator is the plugin name 
    // "ej.mobile.Rotator" is "namespace.className" will hold functions and properties

    ej.widget("ejmRotator", "ej.mobile.Rotator", {
        _requiresID: true,
        _setFirst: true,
        _rootCSS: "e-m-rotator",
        _tags: [{
            tag: "items",
            attr: ["imageUrl", "text"]
        }],

        defaults: {
            targetId: null,
            renderMode: "auto",
            targetHeight: "auto",
            targetWidth: "auto",
            currentItemIndex: 0,
            showPager: true,
            cssClass: "",
            orientation: "horizontal",
            enablePersistence: false,
            pagerPosition: "bottom",
            items: [],
            swipeLeft: null,
            swipeRight: null,
            swipeUp: null,
            swipeDown: null,
            change: null,
            pagerSelect: null,
            dataSource: null,
        },

        dataTypes: {
            renderMode: "enum",
            showPager: "boolean",
            dataSource: "data",
            enablePersistence: "boolean"
        },
        observables: ["currentItemIndex"],
        currentItemIndex: ej.util.valueFunction("currentItemIndex"),
        _init: function () {
            this._renderControl();
            this._createDelegate();
            this._wireEvents();
        },
        _renderControl: function () {
            this._setValues();
            this.element.addClass("e-m-" + this.model.renderMode + " " + this.model.cssClass + " e-m-rotator e-m-user-select").append(this._wrapper);
            this.model.dataSource ? this._renderDatabind() : this._renderDefault();
        },
        _setValues: function () {
            ej.setRenderMode(this);
            if (eval(this.model.items).length != 0) {
                this.model.items = eval(this.model.items)
                var itemsLen=this.model.items.length
                this._wrapper = ej.buildTag("div#" + this._id + "_container");
                for (i = 0; i <itemsLen ; i++)
                    this._wrapper.append(ej.buildTag("div"));
            }
            else {
                this._wrapper = this.model.targetId ? (ej.getCurrentPage()).find("#" + this.model.targetId) : $(this.element.children()[0]);
                var childEle = this._wrapper.children();
                var tempImgAttr = ej.getAttrVal($(childEle[0]), 'data-ej-imageUrl');
                var tempTextAttr = ej.getAttrVal($(childEle[0]), 'data-ej-text');
                if (tempImgAttr || tempTextAttr) {
                    for (i = 0; i < childEle.length; i++) {
                        var item = {};
                        item.imageUrl = ej.getAttrVal($(childEle[i]), 'data-ej-imageUrl');
                        if (tempTextAttr)
                            item.text = ej.getAttrVal($(childEle[i]), 'data-ej-text');
                        this.model.items.push(item);
                    }
                }
            }
            this._divCount = this._wrapper.children().length;
            this._orgEle = this._wrapper;
            this._autoHeightWidthValues();
            this._browser = ej.browser().toLowerCase();
            this._startValues();
        },
        _autoHeightWidthValues: function () {
            this._mHeight = (this.model.targetHeight == "auto" ? (ej.getDimension(this.element.parent(), "height")) : parseInt(this.model.targetHeight));
            this._mWidth = this.model.targetWidth == "auto" ? ej.getDimension(this.element.parent(), "width") : parseInt(this.model.targetWidth);
        },
        _startValues: function () {
            this._moveEle = (this._divCount > this.currentItemIndex() && this.currentItemIndex() >= 0) ? -(this.currentItemIndex() * ((this.model.orientation == "horizontal") ? this._mWidth : this._mHeight)) : 0;
            this._divEle = this._moveEle ? this.currentItemIndex() + 1 : 1;
            this._timeStart = 0;
        },
        _transform: function (dist, time) {
            var translate = "-" + this._browser + "-transform",
                translateVal = this.model.orientation == "horizontal" ? "translate3d(" + dist + "px,0px,0px) scale(1)" : "translate3d(0px," + dist + "px,0px) scale(1)",
                trans = "-" + this._browser + "-transition-property",
                transVal = "transform",
                transDur = "-" + this._browser + "-transition-duration",
                transDurVal = time + "ms";
            this._wrapper.css(translate, translateVal).css(trans, transVal).css(transDur, transDurVal);
        },
        _renderDatabind: function () {
            var jsRender = ej.buildTag("script#" + this._id + "_Template", {}, {}, { "type": "text/x-jsrender" });
            var tempEle = this._wrapper.find("div");
            tempEle.remove();
            var newTemp = ej.buildTag("div").append(tempEle);
            jsRender[0].text = newTemp[0].innerHTML;
            ej.getCurrentPage().append(jsRender);
            if (this.model.dataSource)
                this.renderDatasource(this.model.dataSource);
        },
        _renderDefault: function () {
            if (this.model.items.length != 0) {
                this._imgTag = (this.model.items[0].imageUrl);
                this._textTag = this.model.items[0].text;
          
            }   
            if(this._imgTag!=undefined || this._textTag!=undefined)
            this._tagCreate();
            this._scrollItem();
            this._transform(this._moveEle, 350);
            this._renderPager();
        },
        _scrollItem: function () {
            this.element.css({ "width": "" + this._mWidth + "px", "height": "" + (this._mHeight) + "px" });
            this._wrapper.addClass("e-m-sv-container e-m-" + this.model.orientation + "").css({ "height": "" + this._mHeight + "", "width": "" + this._mWidth + "" });
            this._wrapper.children().addClass("e-m-item");
        },
        _tagCreate: function () {
            var items = this._wrapper.find('div'), divImage, para;
            for (i = 0; i < this._divCount; i++) {
                if (this._textTag != undefined) {
                    para = ej.buildTag("p").html(this.model.items[i].text).addClass("e-m-rotator-p");
                    $(items[i]).append(para);
                }
                if (this._imgTag != undefined) {
                    divImage = ej.buildTag("div").css({
                        "background-image": "url(" + this.model.items[i].imageUrl + ")",
                    }).addClass("e-m-rotator-image");
                    $(items[i]).append(divImage);
                }
            }
        },
        _renderPager: function () {
            this._span = ej.buildTag("div.e-m-abs  e-m-pager e-m-" + this.model.renderMode + "");
            this._wrapper.after(this._span);
            for (i = 0; i < this._divCount; i++)
                this._span.append(ej.buildTag("span.e-m-sv-pager"));
            this._span.find(":nth-child(" + this._divEle + ")").addClass("e-m-active");
            this._pagerPos();
            this._setShowPager();
        },
        _pagerPos: function () {
            this.model.orientation == "horizontal" ? (this.model.pagerPosition = "top" ? this.model.pagerPosition == "top" ? this._span.addClass("e-m-sv-pagertop") : this._span.addClass("e-m-sv-pagerbottom") : this._span.addClass("e-m-sv-pagerbottom")) : this.model.pagerPosition == "right" ? this._span.addClass("e-m-sv-pagerright") : this._span.addClass("e-m-sv-pagerleft")
            this.model.orientation == "horizontal" ? (this._span.addClass("e-m-rotator-spanleft")) : this._span.css("bottom", +((this._mHeight) - $(".e-m-pager").height()) / 2 + "px");
        },
        _createDelegate: function()
        {
            this._onMouseDownDelg = $.proxy(this._onMouseDownHandler, this);
            this._onMouseMoveDelg = $.proxy(this._onMouseMoveHandler, this);
            this._onMouseUpDelg = $.proxy(this._onMouseUpHandler, this);
            this._onTouchEndDelegate = $.proxy(this._onTouchEndHandler, this);
        },
        _wireEvents: function (remove) {
            ej.listenTouchEvent(this._wrapper, ej.startEvent(), this._onMouseDownDelg, remove);
            ej.listenEvents([window], [ "onorientationchange" in window ? "orientationchange" : "resize"], [$.proxy(this._ResizeHandler, this)], remove);
            this._pagerEvents(remove);
        },
        _ResizeHandler: function (evt) {
            if (this.model.targetHeight == "auto" || this.model.targetWidth == "auto") {
                var proxy = this;
                window.setTimeout(function () {
                    evt.preventDefault();
                    proxy._orientationAndResize();
                }, ej.isAndroid() ? 200 : 0);
            }
        },
        _pagerEvents: function (remove) {
            this._spanEle = this.element.find("div.e-m-abs").children();
            ej.listenEvents([this._spanEle, this._spanEle], [ej.startEvent(), ej.endEvent()], [this._onTouchStartDelegate, this._onTouchEndDelegate], remove);
        },
        _onTouchEndHandler: function (evt) {
            this._index = $(evt.currentTarget).index();
            this.currentItemIndex(this._index);
            this._moveEle = -(this._index * (this.model.orientation == "horizontal" ? this._mWidth : this._mHeight));
            this._transform(this._moveEle, 350);
            this._span.find(":nth-child(" + this._divEle + ")").removeClass("e-m-active");
            this._divEle = this._index + 1;
            this._span.find(":nth-child(" + this._divEle + ")").addClass("e-m-active");
            var data = { targetElement: this._wrapper, element: this._divEle };
            if (this.model.pagerSelect)
                this._trigger("pagerSelect", data);
        },
        _orientationAndResize: function () {
            this._autoHeightWidthValues();
            var val = this.model.orientation == "horizontal" ? this._mWidth : this._mHeight;
            this._moveEle = -((this._divEle * val) - val);
            this._transform(this._moveEle, 0);
            this._scrollItem();
            this._spanTrans = (this._divEle - 1) * this._andSpanVal;
            this._pagerPos();
        },
        _onMouseDownHandler: function (evt) {
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            this._startX = evt.clientX;
            this._startY = evt.clientY;
            this._timeStart = evt.timeStamp || Date.now();
            ej.listenEvents([this._wrapper, this._wrapper], [ej.moveEvent(), ej.endEvent()], [this._onMouseMoveDelg, this._onMouseUpDelg], false);
        },
        _onMouseMoveHandler: function (evt) {
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            this._relativeX = evt.clientX - this._startX;
            this._relativeY = evt.clientY - this._startY;
            if (this.model.orientation == "horizontal") {
                this._start = this._startX;
                this._relative = this._relativeX;
                this._distValue = this._mWidth;
                this._swipeLeftUp = "swipeLeft";
                this._swipeRightDown = "swipeRight";
            }
            else {
                this._start = this._startY;
                this._relative = this._relativeY;
                this._distValue = this._mHeight;
                this._swipeLeftUp = "swipeUp";
                this._swipeRightDown = "swipeDown";
            }
           if (this._start != 0 && Math.abs(this._relative)) {
                if ((this._relative < 0 && this._divEle < this._divCount) || (this._relative > 0 && this._divEle > 1)) {}
                else {
                    (Math.abs(this._relative) < this._distValue / 2)  
                }
                    var move = this._moveEle + this._relative;
                    this._transform(move, 0);
            }
            ej.listenTouchEvent(this._wrapper, "mouseleave", this._onMouseUpDelg, false);
        },
       _onMouseUpHandler: function (evt) {
            if (this._timeStart != 0) {
                this._data = { targetElement: this._wrapper, element: this._divEle };
                var duration = (evt.timeStamp || Date.now()) - this._timeStart,
                momentumX = { dist: 0, time: 0 },
                reverse = this._relative > 0 ? true : false,
                moveDist = this._relative;
                if (duration < 300) {
                    momentumX = this._relative ? this._momentum(this._relative, duration, (reverse ? this._relative : -this._relative), this._distValue + this._relative, this._distValue) : momentumX;
                    moveDist = this._relative + momentumX.dist;
                }
                if (moveDist < 0 && Math.abs(moveDist) > this._distValue / 2 && this._divEle < this._divCount) {
                    this._moveEle += -this._distValue;
                    this._movePager(true);
                    if (this.model.swipeLeft || this.model.swipeUp)
                        this._trigger(this._swipeLeftUp, this._data);
                }
                else if (moveDist > 0 && Math.abs(moveDist) > this._distValue / 2 && this._divEle > 1) {
                    this._moveEle -= -this._distValue;
                    this._movePager(false);
                    if (this.model.swipeRight || this.model.swipeDown)
                        this._trigger(this._swipeRightDown, this._data);
                }
                this._transform(this._moveEle, 350);
                this._timeStart = 0;
                this._startX = 0;
                this._relativeX = 0;
                this._relative = 0;
                this._start = 0;
                ej.listenEvents([this._wrapper, this._wrapper], [ej.moveEvent(), ej.endEvent()], [this._onMouseMoveDelg, this._onMouseUpDelg], true);
            }
            this.currentItemIndex(this._divEle - 1);
        },
        _momentum: function (dist, time, maxDistUpper, maxDistLower, size) {
           var deceleration = 0.0006,
            speed = Math.abs(dist) / time,
            newDist = (speed * speed) / (2 * deceleration),
            newTime = 0, outsideDist = 0;
            if (dist > 0 && newDist > maxDistUpper) {
                outsideDist = size / (6 / (newDist / speed * deceleration));
                maxDistUpper = maxDistUpper + outsideDist;
                speed = speed * maxDistUpper / newDist;
                newDist = maxDistUpper;
            } else if (dist < 0 && newDist > maxDistLower) {
                outsideDist = size / (6 / (newDist / speed * deceleration));
                maxDistLower = maxDistLower + outsideDist;
                speed = speed * maxDistLower / newDist;
                newDist = maxDistLower;
            }
            newDist = newDist * (dist < 0 ? -1 : 1);
            newTime = speed / deceleration;
            return { dist: newDist, time: Math.round(newTime) };
        },
        _movePager: function (page) {
            this._span.find(":nth-child(" + this._divEle + ")") .removeClass("e-m-active");
            this._divEle = page ? (this._divEle + 1) : (this._divEle - 1);
            this._span.find(":nth-child(" + this._divEle + ")").addClass("e-m-active");
            this._spanTrans = (this._divEle - 1) * this._andSpanVal;
            this._data = { targetElement: this._wrapper, element: this._divEle };
            if (this.model.change)
                this._trigger("change", this._data);
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                if (prop != "swipeLeft" || prop != "swipeRight")
                    var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                this[setModel] ? this[setModel]() : refresh = true;
                
            }
            if (refresh)
                this._refresh();
        },
        _setRenderMode: function () {
            this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode + "");
            this._span.remove();
            this._renderPager();
            this._pagerEvents(false);
        },
        _setShowPager: function () {
            this.model.showPager ? this._span.css("display", "block") : this._span.css("display", "none");
        },
        _setOrientation: function () {
            this.model.orientation == "horizontal" ? this._span.addClass("e-m-sv-pagerbottom") : this._span.addClass("e-m-sv-pagerright");
            this.currentItemIndex(this._divEle - 1);
            this._refresh();
        },
        _setCurrentItemIndex: function () {
            var span=this._span.find(":nth-child(" + this._divEle + ")");
            span.removeClass("e-m-active");
            this._startValues();
            this._transform(this._moveEle, 1000);
            this._spanTrans = (this._divEle - 1) * this._andSpanVal;
            span.addClass("e-m-active");
        },
        _refresh: function () {
            this._destroy();
            this._init();
        },
        _destroy: function () {
            this._wireEvents(true);
               this._wrapper.children().find(".e-m-rotator-p,.e-m-rotator-image").remove().removeClass("e-m-item");
            this.element.removeAttr("class style").children().removeAttr("class style").remove();
            this._span.remove();
            ej.getCurrentPage().append(this._orgEle);
        },
        /*---------------Public Methods---------------*/
        renderDatasource: function (data) {
            var template = $.templates("#" + this._id + "_Template");
            var htmlOutput = template.render(data);
            this._wrapper.html(htmlOutput);
            this._divCount = this._wrapper.children().length;
            this._scrollItem();
            this._transform(this._moveEle, 350);
            this._renderPager();
            this._wireEvents();
        }
        /*---------------Public Methods End---------------*/
    });
})(jQuery, Syncfusion);;
function Scrollbar(scroller, options) {
    this._wrapper = typeof options.el == 'string' ? document.querySelector(options.el) : options.el;
    this._sb = this._wrapper.children[0];
    this._sbStyle = this._sb.style;
    this.scroller = scroller;

    this.model = {
        listenX: true,
        listenY: true,
        enableInteraction: false,
        enableResize: true,
        enableShrink: false,
        enableFade: false,
        speedRatioX: 0,
        disableTouch: false,
        speedRatioY: 0,
        ignoreBoundaries: false
    };

    for (var i in options) {
        this.model[i] = options[i];
    }

    this._sizeRatioX = 1;
    this._sizeRatioY = 1;
    this._maxPosX = 0;
    this._maxPosY = 0;
    if (this.model.enableFade) {
        this._sbStyle[ej.transform] = this.scroller.model.translateZ;
        this._sbStyle[ej.transitionDuration] = ej.isLowerAndroid() ? '0.001s' : '0ms';
        this._sbStyle.opacity = '0';
    }
    this._createDelegates();
    this._wireEvents();
}

Scrollbar.prototype = {
    _createDelegates: function () {
        this._touchStartDelegate = $.proxy(this._touchStart, this);
        this._touchMoveDelegate = $.proxy(this._touchMove, this);
        this._touchEndDelegate = $.proxy(this._touchEnd, this);
    },
    _wireEvents: function (remove) {
        target = this.scroller.model.bindToWrapper ? this.scroller._wrapper : window;
        if ($.support.hasTouch && !this.model.disableTouch)
            ej.listenEvents([this.scroller._wrapper, target, target], [ej.startEvent(), ej.cancelEvent(), ej.endEvent()], [this._scrollStartDelegate, this.scrollEndDelegate, this.scrollEndDelegate], remove);
        if (this.model.enableInteraction) {
            if (!this.model.disableTouch)
                ej.listenEvents([this._sb, window, window], [ej.startEvent(), ej.cancelEvent(), ej.endEvent()], [this._touchStartDelegate, this._touchEndDelegate, this._touchEndDelegate], remove);
            if (!this.model.disablePointer)
                ej.listenEvents([this._sb, window], [ej.startEvent(), ej.endEvent()], [this._touchStartDelegate, this._touchEndDelegate], remove);
            if (!this.model.disableMouse)
                ej.listenEvents([this._sb, window], [ej.startEvent(), ej.endEvent()], [this._touchStartDelegate, this._touchEndDelegate], remove);
        }

    },
    _touchStart: function (e) {
        var point = e.touches ? e.touches[0] : e;
        ej.blockDefaultActions(e);
        this.transitionTime();
        this._startedScroll = true;
        this._moved = false;
        this._lastPointX = point.pageX;
        this._lastPointY = point.pageY;
        this._startTime = ej.getTime();
        if (!this.model.disableTouch || !this.model.disableMouse)
            ej.listenTouchEvent(window, ej.moveEvent(), this._touchMoveDelegate);
        if (!this.model.disablePointer)
            ej.listenTouchEvent(window, ej.moveEvent(), this._touchMoveDelegate);        
    },

    _touchMove: function (e) {
        ej.blockDefaultActions(e);
        var point = e.touches ? e.touches[0] : e,
			deltaX, deltaY,
			newX, newY,
			timestamp = ej.getTime();
        if (!this._moved)
            this.scroller._triggerEvent('scrollStart',this.scroller._dataValue());
        this._moved = true;
        deltaX = point.pageX - this._lastPointX;
        this._lastPointX = point.pageX;
        deltaY = point.pageY - this._lastPointY;
        this._lastPointY = point.pageY;
        newX = this._x + deltaX;
        newY = this._y + deltaY;
        this._pos(newX, newY);
    },

    _touchEnd: function (e) {
        if (!this._startedScroll)
            return;
        this._startedScroll = false;
        ej.blockDefaultActions(e);
        ej.listenEvents([window, window, window], [ej.moveEvent(), ej.moveEvent(), ej.moveEvent()], [this._touchMoveDelegate, this._touchMoveDelegate, this._touchMoveDelegate], true);
        if (this.scroller.model.snap) {
            var snap = this.scroller._nearestSnap(this.scroller._x, this.scroller._y);

            var time = this.model.snapSpeed || Math.max(
					Math.max(
						Math.min(Math.abs(this.scroller._x - snap.x), 1000),
						Math.min(Math.abs(this.scroller._y - snap.y), 1000)
					), 300);

            if (this.scroller._x != snap.x || this.scroller._y != snap.y) {
                this.scroller._directionX = 0;
                this.scroller._directionY = 0;
                this.scroller._currentPage = snap;
                this.scroller.scrollTo(snap.x, snap.y, time, this.scroller.model.bounceEasing);
            }
        }
        if (this._moved)
            this.scroller._triggerEvent('scrollStop',this.scroller._dataValue());
    },

    _pos: function (x, y) {
        if (x < 0) {
            x = 0;
        } else if (x > this._maxPosX) {
            x = this._maxPosX;
        }
        if (y < 0) {
            y = 0;
        } else if (y > this._maxPosY) {
            y = this._maxPosY;
        }
        x = this.model.listenX ? Math.round(x / this._sizeRatioX) : this.scroller._x;
        y = this.model.listenY ? Math.round(y / this._sizeRatioY) : this.scroller._y;
        this.scroller.scrollTo(x, y);
    },

    /*---------------Public Methods---------------*/
    enableFade: function (val, hold) {
        if ((hold && !this._visible) || !this.model.enableFade )
            return;

        clearTimeout(this._fadeTimeout);
        this._fadeTimeout = null;

        var time = val ? 250 : 500,
            proxy = this,
			delay = val ? 0 : 300;

        val = val ? '1' : '0';

        this._sbStyle[ej.transitionDuration] = time + 'ms';

        this._fadeTimeout = setTimeout((function (val) {
            proxy._sbStyle.opacity = val;
            proxy._visible = +val;
        }).bind(this, val), delay);
    },

    destroy: function () {
        if (this.model.enableInteraction) {
            this._wireEvents(true);
        }
    },
    transitionTime: function (time) {
        time = time || 0;
        this._sbStyle[ej.transitionDuration] = time + 'ms';
        if (!time && ej.isLowerAndroid()) {
            this._sbStyle[ej.transitionDuration] = '0.001s';
        }
    },

    transitionTimingFunction: function (easing) {
        this._sbStyle[ej.transitionTimingFunction] = easing;
    },

    refresh: function () {
        this.transitionTime(0);
        if (this.model.listenX && !this.model.listenY) {
            this._sbStyle.display = this.scroller._hasHScroll ? 'block' : 'none';
        } else if (this.model.listenY && !this.model.listenX) {
            this._sbStyle.display = this.scroller._hasVScroll ? 'block' : 'none';
        } else {
            this._sbStyle.display = this.scroller._hasHScroll || this.scroller._hasVScroll ? 'block' : 'none';
        }
        if (this.model.listenX) {
            this._wrpWd = this._wrapper.clientWidth;
            if (this.model.enableResize) {
                this._sbWidth = Math.max(Math.round(this._wrpWd * this._wrpWd / (this.scroller._scrollerWidth || this._wrpWd || 1)), 8);
                this._sbStyle.width = this._sbWidth + 'px';
            } else {
                this._sbWidth = this._sb.clientWidth;
            }
            this._maxPosX = this._wrpWd - this._sbWidth;
            if (this.model.enableShrink == 'clip') {
                this._minBoundaryX = -this._sbWidth + 8;
                this._maxBoundaryX = this._wrpWd - 8;
            } else {
                this._minBoundaryX = 0;
                this._maxBoundaryX = this._maxPosX;
            }
            this._sizeRatioX = this.model.speedRatioX || (this.scroller._maxScrollX && (this._maxPosX / this.scroller._maxScrollX));
        }
        if (this.model.listenY) {
            this._wrpHt = this._wrapper.clientHeight;
            if (this.model.enableResize) {
                this._sbHeight = Math.max(Math.round(this._wrpHt * this._wrpHt / (this.scroller._scrollerHeight || this._wrpHt || 1)), 8);
                this._sbStyle.height = this._sbHeight + 'px';
            } else {
                this._sbHeight = this._sb.clientHeight;
            }

            this._maxPosY = this._wrpHt - this._sbHeight;
            if (this.model.enableShrink == 'clip') {
                this._minBoundaryY = -this._sbHeight + 8;
                this._maxBoundaryY = this._wrpHt - 8;
            } else {
                this._minBoundaryY = 0;
                this._maxBoundaryY = this._maxPosY;
            }
            this._sizeRatioY = this.model.speedRatioY || (this.scroller._maxScrollY && (this._maxPosY / this.scroller._maxScrollY));
        }
        this.setPosition();
    },

    setPosition: function () {
        var x = this.model.listenX && Math.round(this._sizeRatioX * this.scroller._x) || 0, proxy = this,
        y = this.model.listenY && Math.round(this._sizeRatioY * this.scroller._y) || 0;
        if (!this.model.ignoreBoundaries) {
            if (x < this._minBoundaryX) {
                if (this.model.enableShrink == 'scale') {
                    this._width = Math.max(this._sbWidth + x, 8);
                    this._sbStyle.width = this._width + 'px';
                }
                x = this._minBoundaryX;
            } else if (x > this._maxBoundaryX) {
                if (this.model.enableShrink == 'scale') {
                    this._width = Math.max(this._sbWidth - (x - this._maxPosX), 8);
                    this._sbStyle.width = this._width + 'px';
                    x = this._maxPosX + this._sbWidth - this.width;
                } else {
                    x = this._maxBoundaryX;
                }
            }
            else if (this.model.enableShrink == 'scale' && this._width != this._sbWidth) {
                this._width = this._sbWidth;
                this._sbStyle.width = this._width + 'px';
            }
            if (y < this._minBoundaryY) {
                if (this.model.enableShrink == 'scale') {
                    this._height = Math.max(this._sbHeight + y * 3, 8);
                    this._sbStyle.height = this._height + 'px';
                }
                y = this._minBoundaryY;
            } else if (y > this._maxBoundaryY) {
                if (this.model.enableShrink == 'scale') {
                    this._height = Math.max(this._sbHeight - (y - this._maxPosY) * 3, 8);
                    this._sbStyle.height = this._height + 'px';
                    y = this._maxPosY + this._sbHeight - this._height;
                } else {
                    y = this._maxBoundaryY;
                }
            } else if (this.model.enableShrink == 'scale' && this._height != this._sbHeight) {
                this._height = this._sbHeight;
                this._sbStyle.height = this._height + 'px';
            }
        }
        if ($(this._wrapper).hasClass("e-m-ver"))
            x = 0;
        else
            y = 0;
        this._x = x;
        this._y = y;
        if (this.scroller.model.enableTransform) {
            this._sbStyle[ej.transform] = 'translate(' + x + 'px,' + y + 'px)' + this.scroller.model.translateZ;

        } else {
            this._sbStyle.left = x + 'px';
            this._sbStyle.top = y + 'px';
        }
    }
    /*---------------Public Methods End---------------*/
};;
/**
 * @fileOverview Plugin to style the Html Scrollpanel elements
 * @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
 *  Use of this code is subject to the terms of our license.
 *  A copy of the current license can be obtained at any time by e-mailing
 *  licensing@syncfusion.com. Any infringement will be prosecuted under
 *  applicable laws. 
 * @version 12.1 
 * @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
 */

(function ($, ej, undefined) {
    // mScrollPanel is the plugin name 
    // "m.ScrollPanel" is "namespace.className" will hold functions and properties

    nextFrame = (function () {
        return window.requestAnimationFrame ||
            window.webkitRequestAnimationFrame ||
            window.mozRequestAnimationFrame ||
            window.oRequestAnimationFrame ||
            window.msRequestAnimationFrame ||
            function (callback) {
                return setTimeout(callback, 1);
            };
    })();
    cancelFrame = (function () {
        return window.cancelRequestAnimationFrame ||
            window.webkitCancelAnimationFrame ||
            window.webkitCancelRequestAnimationFrame ||
            window.mozCancelRequestAnimationFrame ||
            window.oCancelRequestAnimationFrame ||
            window.msCancelRequestAnimationFrame ||
            clearTimeout;
    })();
    ej.widget("ejmScrollPanel", "ej.mobile.ScrollPanel", {
        _setFirst: true,
        _rootCSS: "e-m-scroll",
        angular: {
            terminal: false
        },
        defaults: {
            renderMode: 'auto',            
            enableResize: true,
            targetHeight: null,
            targetWidth: null,
            scrollHeight: "auto",
            scrollWidth: "auto",
            enableFade: true,
            enableShrink: ej.isWindows() ? false : true,
            allowPullToRefresh: false,
            cssClass: null,
            pullToRefreshSettings: {
                thresholdDistance: 75,
                textOnPull: "Pull to Refresh...",
                textOnRelease: "Release to Refresh...",
                textOnRefresh: "Refreshing..."
            },
            isRelative: false,
            wheelSpeed: 16,
            enableInteraction: true,
            enabled: true,
            enableHrScroll: (ej.isWindows() && !ej.isMobile()) ? true : false,
            enableVrScroll: true,
            eventPassthrough: null,
            translateZ: ej.browserInfo()["name"] == "msie" ? '' : $.support.has3d ? ' translateZ(0)' : '',
            zoomMin: 1,
            zoomMax: 6,
            startZoom: 1,
            startX: 0,
            startY: 0,
            enableDisplacement: false,
            displacementValue: 94,
            displacementTime: 800,
            deceleration: null,
            disableTouch: false,
            directionLockThreshold: 5,
            scrollEndThresholdDistance: 5,
            momentum: true,
            enableBounce: ej.isAndroid() ? false : true,
            bounceTime: 450,
            bounceEasing: '',
            preventDefault: true,
            preventDefaultException: {
                tagName: /^(INPUT|TEXTAREA|BUTTON|SELECT)$/
            },
            enableTransition: true,
            enableTransform: true,
            showScrollbars: true,
            enableMouseWheel: true,
            enableKeys: true,
            enableZoom: false,
            enableNativeScrolling: (ej.isAndroid() || ej.isIOS()) ? false : ej.isDevice() ? true : false,
            invertWheel: false,
            scrollStart: null,
            scroll: null,
            scrollStop: null,
            scrollEnd: null,
            zoomStart: null,
            zoomEnd: null,
            pull: null,
            target: null
        },
        dataTypes: {},

        _init: function () {
            ej.setRenderMode(this);
            this._createDelegates();
            if (this.model.enableDisplacement && ej.isWindows())
                this.model.deceleration = 0.00006;
            this._eventType = {
                touchstart: 1,
                touchmove: 1,
                touchend: 1,
                ejtouchmove: 3,
                mousedown: 2,
                mousemove: 2,
                mouseup: 2,
                MSPointerDown: 3,
                MSPointerMove: 3,
                MSPointerUp: 3,
                pointerdown: 3,
                pointermove: 3,
                pointerup: 3
            };
            this._scrollEvents = {
                scrollStop: [this.model.scrollStop],
                scroll: [this.model.scroll],
                scrollStart: [this.model.scrollStart],
                zoomStart: [this.model.zoomStart],
                zoomEnd: [this.model.zoomEnd]
            };
            this._scale = Math.min(Math.max(this.model.startZoom, this.model.zoomMin), this.model.zoomMax);
            if (this.model.target)
                this._wrapper = typeof this.model.target == 'string' ? $("#" + this.model.target)[0] : this.model.target;
            else
                this._wrapper = this.element[0];
            this._$wrapper = $(this._wrapper);
            this._$wrapper.wrapInner("<div>")
            if (this._wrapper) this._scroller = this._wrapper.children[0];
            if (!this._scroller) return;
            this._aniTime = null;
            $(this._scroller).addClass("e-m-scrollcontent e-m-user-select").css({
                height: this.model.scrollHeight,
                width: this.model.scrollWidth
            });

            if (this.model.target) this._$wrapper.append(this.element);
            this._$wrapper.addClass("e-m-scrollpanel " + (this.model.isRelative ? "e-m-rel" : "e-m-abs") + (this.model.cssClass ? " " + this.model.cssClass : "") + " e-m-" + this.model.renderMode);
            this.element.addClass("e-m-scroll");
            if (this.model.enableNativeScrolling) {
                this.refresh();
                this._$wrapper.addClass('e-m-scroll-native e-m-scroll-wrapper');
                if (this.model.renderMode == "ios7" || this.model.renderMode == "android")
                    $(this._scroller).addClass('e-m-scroll-content');
                this._scroller[ej.transform] = 'translate(' + 0 + 'px,' + 0 + 'px)';
                if (((!this._hasVScroll && this._maxScrollY == 0) || (!this._hasHScroll && this._maxScrollX == 0)) && this.model.renderMode == "ios7")
                    this._$wrapper.addClass('e-m-scroll-box');
                if (!this.model.showScrollbars)
                    this._$wrapper.addClass('e-m-scroll-hide');
                if (!this.model.enableZoom)
                    this._$wrapper.addClass('e-m-zoom');
            } else {
                var proxy = this;
                this._scrollerStyle = this._scroller.style;
                this.model.enableTransition = $.support.hasTransition && this.model.enableTransition;
                this.model.enableTransform = $.support.hasTransform && this.model.enableTransform;
                this.model.eventPassthrough = this.model.eventPassthrough === true ? 'vertical' : this.model.eventPassthrough;
                this.model.preventDefault = !this.model.eventPassthrough && this.model.preventDefault;
                this.model.enableVrScroll = this.model.eventPassthrough == 'vertical' ? false : this.model.enableVrScroll;
                this.model.enableHrScroll = this.model.eventPassthrough == 'horizontal' ? false : this.model.enableHrScroll;
                this.model.directionLockThreshold = this.model.eventPassthrough ? 0 : this.model.directionLockThreshold;
                this.model.invertWheel = this.model.invertWheel ? -1 : 1;
                this._x = this._y = this._directionX = this._directionY = this._startX = this._startY = 0;
                this._createRefresher();
                if (this.model.enableZoom)
                    this._scrollerStyle[ej.transformOrigin] = '0 0';
                if (this.model.showScrollbars)
                    this._initScrollbars();
                if (this.model.enableMouseWheel)
                    this._initWheel();
                if (this.model.enableKeys)
                    this._initKeys();
                this.refresh();
                this.scrollTo(this.model.startX, this.model.startY);
            }
            this._wireEvents();
        },
        _createRefresher: function () {
            if (this.model.allowPullToRefresh) {
                var refreshEle = ej.buildTag("div.e-m-scroll-refresher", '<span class="e-m-scroll-refreshcontent"><span class="e-m-left"><span class="e-m-icon e-m-pull"></span></span><span class="e-m-right e-m-text">' + this.model.pullToRefreshSettings.textOnPull + '</span></span>')
                $(this._scroller).prepend(refreshEle);
                this._refresherElement = $(this._scroller).find(".e-m-scroll-refresher");
                this._refresherIcon = this._refresherElement.find(".e-m-icon");
                this._refresherText = this._refresherElement.find(".e-m-text");
            }
        },

        _dataValue: function () {
            return data = {
                x: this.model.enableNativeScrolling ? this._nx : this._x,
                y: this.model.enableNativeScrolling ? this._ny : this._y,
                object: this
            };
        },

        _transitionEnd: function (e) {
            if (e.target != this._scroller || !this._isInTransition) return;
            this._transitionTime();
            if (!this.resetPosition(this.model.bounceTime)) {
                this._moved = this._isInTransition = false;
                this._triggerEvent('scrollStop', this._dataValue());
                if (this._y - this.model.scrollEndThresholdDistance <= this._maxScrollY)
                    this._trigger('scrollEnd', $.extend(this._dataValue(), {
                        position: "bottom"
                    }));
                if (this._y >= -this.model.scrollEndThresholdDistance)
                    this._trigger('scrollEnd', $.extend(this._dataValue(), {
                        position: "top"
                    }));
            }

        },

        _touchStart: function (e) {
            if ($(e.target).parents(".e-m-scrollpanel").length > 1 && $(e.target).closest(".e-m-scrollpanel")[0] != this._wrapper) return;
            if (this._lockRefresh) return;
            var point = e.touches ? e.touches[0] : e,
                pos;
            if (this.model.enableNativeScrolling) {
                this._moved = false;
                if (this.model.renderMode == "ios7" || this.model.renderMode == "android") {
                    startY = point.pageY;
                    startTopScroll = this._wrapper.scrollTop;
                    if (startTopScroll <= 0)
                        this._wrapper.scrollTop = 1;
                    if (startTopScroll + this._wrapper.offsetHeight >= this._wrapper.scrollHeight)
                        this._wrapper.scrollTop = this._wrapper.scrollHeight - this._wrapper.offsetHeight - 1;
                }
                return;
            }

            if ((this._eventType[e.type] != 1 && e.button !== 0) || (!this.model.enabled || (this._startedScroll && this._eventType[e.type] !== this._startedScroll)))
                return;

            ej.listenEvents([window, window, window], [ej.moveEvent(), ej.cancelEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler, this._touchEndHandler],
                false, this, this.model.disableMouse);


            if (this.model.preventDefault && !ej.isDevice() && !ej._preventDefaultException(e.target, this.model.preventDefaultException)) {
                e.preventDefault();
            }
            if (this.model.preventDefault && this._moved) {
                ej.blockDefaultActions(e);
            }


            this._startedScroll = this._eventType[e.type];
            this._distX = this._distY = this._directionX = this._directionY = this._lockedDir = 0;
            this._transitionTime();
            this._startTime = this._getTime();
            if (this.model.enableTransition && this._isInTransition) {
                this._isInTransition = false;
                pos = this.getComputedPosition();
                this._translate(Math.round(pos.x), Math.round(pos.y));
                this._triggerEvent('scrollStop', this._dataValue());
            } else if (!this.model.enableTransition && this._animating) {
                cancelFrame(this._aniTime);
                this._steps = [];
                this._animating = false;
                this._translate(this._wrapper.scrollLeft, this._wrapper.scrollTop);
                this._triggerEvent('scrollStop', this._dataValue());
            }
            this._moved = false;
            this._startX = this._x;
            this._startY = this._y;
            this._absStartX = this._x;
            this._absStartY = this._y;
            this._pointX = point.pageX;
            this._pointY = point.pageY;
            if (this.model.enableZoom && e.touches && e.touches.length > 1)
                this._zoomStart(e);
            if (this._oldScrollX != this._scroller.scrollWidth || this._oldScrollY != this._scroller.scrollHeight)
                this._calculateXY()._refreshScrollbar();

        },

        _scrollStart: function (e) {
            this._moved = false;
            this._nx = this._$wrapper.scrollLeft();
            this._ny = this._$wrapper.scrollTop();
            this._trigger('scrollStart', this._dataValue());
        },

        _touchMove: function (e) {
            if (!this.model.enabled || this._eventType[e.type] !== this._startedScroll) return;
            if (this.model.preventDefault) e.preventDefault();
            if (this.model.enableZoom && e.touches && e.touches[1]) {
                this._zoom(e);
                return;
            }
            var point = e.touches ? e.touches[0] : e,
                deltaX = point.pageX - this._pointX,
                deltaY = point.pageY - this._pointY,
                timestamp = this._getTime(),
                newX, newY, absDistX, absDistY;
            this._pointX = point.pageX;
            this._pointY = point.pageY;
            this._distX += deltaX;
            this._distY += deltaY;
            absDistX = Math.abs(this._distX);
            absDistY = Math.abs(this._distY);
            if (timestamp - this._endTime > 300 && (absDistX < 10 && absDistY < 10)) return;
            if (!this._lockedDir) {
                if (absDistX > absDistY + this.model.directionLockThreshold)
                    this._lockedDir = 'horizontal';
                else if (absDistY >= absDistX + this.model.directionLockThreshold)
                    this._lockedDir = 'vertical';
                else
                    this._lockedDir = 'none';
            }
            if (this._lockedDir == 'horizontal') {
                if (this.model.eventPassthrough == 'vertical')
                    e.preventDefault();
                else if (this.model.eventPassthrough == 'horizontal') {
                    this._startedScroll = false;
                    return;
                }
                deltaY = 0;
            } else if (this._lockedDir == 'vertical') {
                if (this.model.eventPassthrough == 'horizontal')
                    e.preventDefault();
                else if (this.model.eventPassthrough == 'vertical') {
                    this._startedScroll = false;
                    return;
                }
                deltaX = 0;
            }

            deltaX = (this.model.enableHrScroll || this._hasHScroll) ? deltaX : 0;
            deltaY = (this.model.enableVrScroll || this._hasVScroll) ? deltaY : 0;
            newX = this._x + deltaX;
            newY = this._y + deltaY;
            if (newX > 0 || newX < this._maxScrollX) {
                newX = this.model.enableBounce ? this._x + deltaX / 3 : newX > 0 ? 0 : this._maxScrollX;
            }
            if (newY > 0 || newY < this._maxScrollY) {
                newY = this.model.enableBounce ? this._y + deltaY / 3 : newY > 0 ? 0 : this._maxScrollY;
            }
            this._directionX = deltaX > 0 ? -1 : deltaX < 0 ? 1 : 0;
            this._directionY = deltaY > 0 ? -1 : deltaY < 0 ? 1 : 0;
            if (this.model.allowPullToRefresh && !this._lockRefresh) {
                this._refresherIcon[(newY >= this.model.pullToRefreshSettings.thresholdDistance ? "addClass" : "removeClass")]("e-m-rotate");
                this._refresherText.text(newY >= this.model.pullToRefreshSettings.thresholdDistance ? this.model.pullToRefreshSettings.textOnRelease : this.model.pullToRefreshSettings.textOnPull);
            }
            this._translate(newX, newY);
            if (timestamp - this._startTime > 300) {
                this._startTime = timestamp;
                this._startX = this._x;
                this._startY = this._y;
            }
            if (!this._moved)
                this._triggerEvent('scrollStart', this._dataValue());
            else
                this._triggerEvent('scroll', this._dataValue());
            this._moved = ((this._lockedDir != 'none') ? true : false);
        },

        _scroll: function (e) {
            this._moved = true;
            this._nx = this._$wrapper.scrollLeft();
            this._ny = this._$wrapper.scrollTop();
            this._trigger('scroll', this._dataValue());
            proxy = this;
            clearTimeout(this._wheelEndTimeout);
            this._wheelEndTimeout = setTimeout(function () {
                proxy._trigger('scrollStop', proxy._dataValue());
                if ((Math.abs(proxy._maxScrollY) - proxy.model.scrollEndThresholdDistance) <= proxy._ny)
                    proxy._trigger('scrollEnd', $.extend(proxy._dataValue(), {
                        position: "bottom"
                    }));
                if (proxy._ny <= proxy.model.scrollEndThresholdDistance)
                    proxy._trigger('scrollEnd', $.extend(proxy._dataValue(), {
                        position: "top"
                    }));
            }, 200);

        },

        _touchEnd: function (e) {
            ej.listenEvents([window, window, window], [ej.moveEvent(), ej.cancelEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler, this._touchEndHandler],
                true, this, this.model.disableMouse);
            if (!this.model.enabled || this._eventType[e.type] !== this._startedScroll) return;
            if (this.model.preventDefault && !ej.isDevice() && !ej._preventDefaultException(e.target, this.model.preventDefaultException))
                e.preventDefault();
            if (this.model.enableNativeScrolling && this._moved) {
                ej.blockDefaultActions(e);
                this._moved = false;
                return;
            }
            if (this._scaled) {
                this._zoomEnd(e);
                return;
            }
            var point = e.changedTouches ? e.changedTouches[0] : e,
                momentumX,
                momentumY,
                duration = this._getTime() - this._startTime,
                newX = Math.round(this._x),
                newY = Math.round(this._y),
                distanceX = Math.abs(newX - this._startX),
                distanceY = Math.abs(newY - this._startY),
                time = 0,
                easing = '';
            this._isInTransition = this._startedScroll = 0;
            this._endTime = this._getTime();
            if (this.model.allowPullToRefresh) {
                this._lockRefresh = this._refresherIcon.hasClass("e-m-rotate");
                if (this._lockRefresh) {
                    this._refresherText.text(this.model.pullToRefreshSettings.textOnRefresh);
                    this._refresherIcon.removeClass("e-m-pull").addClass("e-m-refresh");
                }
            }

            if (this.resetPosition(this.model.bounceTime)) {
                if (this._lockRefresh)
                    this._trigger('pull', {
                        Obj: this.element.data("ejmScrollPanel")
                    });
                return;
            }
            if (this.model.momentum && duration < 500 && this._moved) {
                momentumX = this._hasHScroll ? ej._nativeMomentum(this._x, this._startX, duration, this._maxScrollX, this.model.enableBounce ? this._wrpWd : 0, this.model.deceleration, ej.isWindows() ? 10 : 1) : {
                    dest: newX,
                    duration: 0
                };
                momentumY = this._hasVScroll ? ej._nativeMomentum(this._y, this._startY, duration, this._maxScrollY, this.model.enableBounce ? this._wrpHt : 0, this.model.deceleration, ej.isWindows() ? 10 : 1) : {
                    dest: newY,
                    duration: 0
                };
                newX = momentumX.dest;
                newY = momentumY.dest;
                time = Math.max(momentumX.duration, momentumY.duration);
                if((newX !=0 && newX != this._maxScrollX) ||  (newY !=0 && newY != this._maxScrollY))
                    this._isInTransition = 1;
            }

            if (this.model.displacementValue && this.model.enableDisplacement) {
                newY = this.model.displacementValue * Math.round(newY / this.model.displacementValue);
                newX = this.model.displacementValue * Math.round(newX / this.model.displacementValue);

                if (time < this.model.displacementTime)
                    time = this.model.diplacementTime;
            }
            if (newY < this._maxScrollY && Math.abs(this._y))
                newY = this._maxScrollY;
            if (newX < this._maxScrollX)
                newX = this._maxScrollX;
            if (this._moved && !this._isInTransition)
                this._trigger('scrollStop', this._dataValue());
            if (newX != this._x || newY != this._y) {
                if (newX > 0 || newX < this._maxScrollX || newY > 0 || newY < this._maxScrollY) {
                    easing = ej.ease.quadratic;
                }
                this.scrollTo(newX, newY, time, easing);
                return;
            }
            this._moved = false;
        },

        _scrollStop: function (e) {
            this._nx = this._$wrapper.scrollLeft();
            this._ny = this._$wrapper.scrollTop();
            this._trigger('scrollStop', this._dataValue());
        },

        _zoomStart: function (e) {
            var c1 = Math.abs(e.touches[0].pageX - e.touches[1].pageX),
                c2 = Math.abs(e.touches[0].pageY - e.touches[1].pageY);

            this._touchesDistanceStart = Math.sqrt(c1 * c1 + c2 * c2);
            this._startScale = this._scale;

            this._originX = Math.abs(e.touches[0].pageX + e.touches[1].pageX) / 2 + this._wrapperOffset.left - this._x;
            this._originY = Math.abs(e.touches[0].pageY + e.touches[1].pageY) / 2 + this._wrapperOffset.top - this._y;
            if (this.model.allowPullToRefresh) this._refresherElement.css("display", "none");
            this._triggerEvent('zoomStart', this._dataValue());
        },

        _zoom: function (e) {

            if (!this.model.enabled || this._eventType[e.type] !== this._startedScroll) return;

            if (this.model.preventDefault) e.preventDefault();


            var c1 = Math.abs(e.touches[0].pageX - e.touches[1].pageX),
                c2 = Math.abs(e.touches[0].pageY - e.touches[1].pageY),
                distance = Math.sqrt(c1 * c1 + c2 * c2),
                scale = 1 / this._touchesDistanceStart * distance * this._startScale,
                lastScale,
                x, y;

            this._scaled = true;

            if (scale < this.model.zoomMin)
                scale = 0.5 * this.model.zoomMin * Math.pow(2.0, scale / this.model.zoomMin);
            else if (scale > this.model.zoomMax)
                scale = 2.0 * this.model.zoomMax * Math.pow(0.5, this.model.zoomMax / scale);


            lastScale = scale / this._startScale;
            x = this._originX - this._originX * lastScale + this._startX;
            y = this._originY - this._originY * lastScale + this._startY;

            this._scale = scale;

            this.scrollTo(x, y, 0);
        },

        _zoomEnd: function (e) {
            if (!this.model.enabled || this._eventType[e.type] !== this._startedScroll) return;

            if (this.model.preventDefault) e.preventDefault();


            var newX, newY, lastScale;

            this._isInTransition = this._startedScroll = 0;

            if (this._scale > this.model.zoomMax)
                this._scale = this.model.zoomMax;
            else if (this._scale < this.model.zoomMin)
                this._scale = this.model.zoomMin;

            // Update boundaries
            this.refresh();

            lastScale = this._scale / this._startScale;

            newX = this._originX - this._originX * lastScale + this._startX;
            newY = this._originY - this._originY * lastScale + this._startY;

            if (newX > 0)
                newX = 0;
            else if (newX < this._maxScrollX)
                newX = this._maxScrollX;

            if (newY > 0)
                newY = 0;
            else if (newY < this._maxScrollY)
                newY = this._maxScrollY;

            if (this._x != newX || this._y != newY)
                this.scrollTo(newX, newY, this.model.bounceTime);

            this._scaled = false;
            if (this.model.allowPullToRefresh) this._refresherElement.css("display", "");
            this._triggerEvent('zoomEnd', this._dataValue());
        },

        _resize: function () {
            if (!this._$wrapper.is(":visible")) return;
            var proxy = this;
            clearTimeout(this._resizeTimeout);
            this._resizeTimeout = setTimeout(function () {
                proxy.refresh();
            }, ej.isAndroid() ? 250 : 0);
        },

        _scrollable: function () {
            return $(this._wrapper).map(function () {
                var elem = this,
                    isWin = !elem.nodeName || $.inArray(elem.nodeName.toLowerCase(), ['iframe', '#document', 'html', 'body']) != -1;

                if (!isWin) return elem;

                var doc = (elem.contentWindow || elem).document || elem.ownerDocument || elem;

                return /webkit/i.test(navigator.userAgent) || doc.compatMode == 'BackCompat' ? doc.body : doc.documentElement;
            });
        },

        _both: function (val) {
            return $.isFunction(val) || typeof val == 'object' ? val : {
                top: val,
                left: val
            };
        },

        _scrollTo: function (target, duration, settings) {
            var proxy = this;
            if (typeof duration == 'object') {
                settings = duration;
                duration = 0;
            }
            if (typeof settings == 'function')
                settings = {
                    onAfter: settings
                };

            if (target == 'max')
                target = 9e9;

            settings = $.extend({}, {
                axis: 'xy',
                duration: 0,
                limit: true
            }, settings);
            duration = duration || settings.duration;
            settings.queue = settings.queue && settings.axis.length > 1;

            if (settings.queue)
                duration /= 2;
            settings.offset = this._both(settings.offset);
            settings.over = this._both(settings.over);

            return this._scrollable().each(function () {
                if (target == null) return;

                var elem = this,
                    $elem = $(elem),
                    targ = target,
                    toff, attr = {},
                    win = $elem.is('html,body');

                switch (typeof targ) {
                case 'number':
                case 'string':
                    if (/^([+-]=?)?\d+(\.\d+)?(px|%)?$/.test(targ)) {
                        targ = this._both(targ);
                        break;
                    }
                    targ = $(targ, this);
                    if (!targ.length) return;
                case 'object':
                    if (targ.is || targ.style)
                        toff = (targ = $(targ)).offset();
                }

                var offset = $.isFunction(settings.offset) && settings.offset(elem, targ) || settings.offset;

                $.each(settings.axis.split(''), function (i, axis) {
                    var Pos = axis == 'x' ? 'Left' : 'Top',
                        pos = Pos.toLowerCase(),
                        key = 'scroll' + Pos,
                        old = elem[key],
                        max = proxy._scrollToMax(elem, axis);

                    if (toff) {
                        attr[key] = toff[pos] + (win ? 0 : old - $elem.offset()[pos]);

                        if (settings.margin) {
                            attr[key] -= parseInt(targ.css('margin' + Pos)) || 0;
                            attr[key] -= parseInt(targ.css('border' + Pos + 'Width')) || 0;
                        }

                        attr[key] += offset[pos] || 0;

                        if (settings.over[pos])
                            attr[key] += targ[axis == 'x' ? 'width' : 'height']() * settings.over[pos];
                    } else {
                        var val = targ[pos];
                        attr[key] = val.slice && val.slice(-1) == '%' ?
                            parseFloat(val) / 100 * max : val;
                    }

                    if (settings.limit && /^\d+$/.test(attr[key]))
                        attr[key] = attr[key] <= 0 ? 0 : Math.min(attr[key], max);

                    if (!i && settings.queue) {
                        if (old != attr[key])
                            animate(settings.onAfterFirst);
                        delete attr[key];
                    }
                });

                animate(settings.onAfter);

                function animate(callback) {
                    $elem.animate(attr, duration, settings.easing, callback && function () {
                        callback.call(this, targ, settings);
                    });
                };

            }).end();
        },
        _scrollToMax: function (elem, axis) {
            var Dim = axis == 'x' ? 'Width' : 'Height',
                scroll = 'scroll' + Dim;

            if (!$(elem).is('html,body')) return elem[scroll] - $(elem)[Dim.toLowerCase()]();

            var size = 'client' + Dim,
                html = elem.ownerDocument.documentElement,
                body = elem.ownerDocument.body;

            return Math.max(html[scroll], body[scroll]) - Math.min(html[size], body[size]);
        },
        _transitionTime: function (time) {

            if (ej.getAndroidVersion() && ej.getAndroidVersion() <= 4) return;

            time = time || 0;
            this._scrollerStyle[ej.transitionDuration] = time + 'ms';
            if (this._scrollbars)
                for (var i = this._scrollbars.length; i--;)
                    this._scrollbars[i].transitionTime(time);
        },

        _transitionTimingFunction: function (easing) {
            this._scrollerStyle[ej.transitionTimingFunction] = easing;
            if (this._scrollbars)
                for (var i = this._scrollbars.length; i--;)
                    this._scrollbars[i].transitionTimingFunction(easing);
        },

        _translate: function (x, y) {
            if (this.model.enableTransform)
                this._scrollerStyle[ej.transform] = 'translate(' + x + 'px,' + y + 'px) scale(' + this._scale + ')';
            else {
                $(this._wrapper).scrollTop(Math.abs(y));
                $(this._wrapper).scrollLeft(Math.abs(x));
            }
            this._x = x;
            this._y = this.model.enableTransform ? y : -Math.abs(y);
            if (this._scrollbars) {
                for (var i = this._scrollbars.length; i--;) {
                    this._scrollbars[i].setPosition();
                }
            }

        },

        _createDelegates: function () {
            this._resizeHandler = $.proxy(this._resize, this);
            this._touchStartHandler = $.proxy(this._touchStart, this);
            this._touchMoveHandler = $.proxy(this._touchMove, this);
            this._touchEndHandler = $.proxy(this._touchEnd, this);
            this._transitionEndHandler = $.proxy(this._transitionEnd, this);
            this._scrollStartHandler = $.proxy(this._scrollStart, this);
            this._scrollHandler = $.proxy(this._scroll, this);
            this._scrollStopHandler = $.proxy(this._scrollStop, this);
            this._wheelDelegate = $.proxy(this._wheel, this);
        },

        _isWinrt: function () {
            return (!this.model.enableTransform && ej.isWindows() && !ej.isMobile())
        },

        _wireEvents: function (remove) {
            var bind = remove ? "off" : "on";
            var evt = "onorientationchange" in window ? "orientationchange" : "resize";
            ej.listenTouchEvent(window, evt, this._resizeHandler, remove);
            if (!this.model.disableTouch)
                ej.listenTouchEvent(this._scroller, ej.startEvent(), this._touchStartHandler, remove, this, this.model.disableMouse);
            if (this.model.enableNativeScrolling) {
                $(this._wrapper).bind("scroll", this._scrollHandler);
            }
            ej.listenTouchEvent(this._scroller, ej.transitionEndEvent(), this._transitionEndHandler, remove);
        },

        _initScrollbars: function () {
            var interactive = this.model.enableInteraction,
                showScrollbars = [],
                scrollbar;
            this._scrollbars = [];

            if (this.model.showScrollbars) {
                // Vertical scrollbar
                if (this.model.enableVrScroll) {
                    scrollbar = {
                        el: this._createDefaultScrollbar('vertical', interactive, this.model.showScrollbars),
                        enableInteraction: interactive,
                        enableShrink: (this.model.enableShrink ? "scale" : false),
                        enableFade: this.model.enableFade,
                        enableResize: this.model.enableResize,
                        listenX: false
                    };
                    this._wrapper.appendChild(scrollbar.el);
                    showScrollbars.push(scrollbar);
                    this.element.append(this._sbw);
                }

                // Horizontal scrollbar
                if (this.model.enableHrScroll) {
                    scrollbar = {
                        el: this._createDefaultScrollbar('horizontal', interactive, this.model.showScrollbars),
                        enableInteraction: interactive,
                        enableShrink: (this.model.enableShrink ? "scale" : false),
                        enableFade: this.model.enableFade,
                        enableResize: this.model.enableResize,
                        listenY: false
                    };

                    this._wrapper.appendChild(scrollbar.el);
                    showScrollbars.push(scrollbar);
                    this.element.append(this._sbw);
                }
            }

            if (this.model.showScrollbars) {
                showScrollbars = showScrollbars.concat();
            }

            for (var i = showScrollbars.length; i--;) {
                this._scrollbars[i] = new Scrollbar(this, showScrollbars[i]);
            }
            if (this.model.enableFade) {
                this.on('scrollStop', function (args) {
                    if (args.model.showScrollbars) {
                        for (var i = args.object._scrollbars.length; i--;) {
                            args.object._scrollbars[i].enableFade();
                        }
                    }
                });

                this.on('scrollCancel', function (args) {
                    if (args.model.showScrollbars) {
                        for (var i = args.object._scrollbars.length; i--;) {
                            args.object._scrollbars[i].enableFade();
                        }
                    }
                });

                this.on('scrollStart', function (args) {
                    if (args.model.showScrollbars) {
                        for (var i = args.object._scrollbars.length; i--;) {
                            args.object._scrollbars[i].enableFade(1);
                        }
                    }
                });
            }
            this.on('refresh', function (args) {
                if (args.model.showScrollbars) {
                    for (var i = args.object._scrollbars.length; i--;) {
                        args.object._scrollbars[i].refresh();
                    }
                }
            });

            this.on('destroy', function (args) {
                if (args.model.showScrollbars) {
                    for (var i = args.object._scrollbars.length; i--;) {
                        args.object._scrollbars[i].destroy();
                    }
                }

                delete this._scrollbars;
            });
        },

        _refreshScrollbar: function () {
            if (this.model.showScrollbars && this._scrollbars) {
                for (var i = this._scrollbars.length; i--;) {
                    this._scrollbars[i].refresh();
                }
            }
        },

        _triggerEvent: function (type, data) {
            if (!this._scrollEvents[type]) {
                return;
            }

            var i = 0,
                l = this._scrollEvents[type].length;

            if (!l) {
                return;
            }
            for (; i < l; i++) {
                this._triggerInternalEvent(this._scrollEvents[type][i], type, data, this);
            }
        },
        _triggerInternalEvent: function (type, event, data, element) {
            if (type) {
                var fn = type;
                if (typeof fn === "string")
                    fn = ej.util.getObject(fn, window);
                event = $.Event(event);
                event.type = type;
                args = ej.event(event, this.model, data);
                return !($.isFunction(fn) && fn.call(this, args) === false || event.isDefaultPrevented());
            }
        },
        _initWheel: function () {
            var proxy = this;
            this._bindWheelEvents();
            this.on('destroy', function () {
                proxy._bindWheelEvents(remove);
            });
        },
        _bindWheelEvents: function (remove) {
            ej.listenEvents([this._wrapper, this._wrapper], ["mousewheel", "DOMMouseScroll"], [this._wheelDelegate, this._wheelDelegate], remove);
        },
        _wheel: function (e) {
            if (!this.model.enabled) return;

            e.preventDefault();

            var wheelDeltaX, wheelDeltaY,
                newX, newY,
                proxy = this;

            if (this.wheelTimeout === undefined) {
                this._triggerEvent('scrollStart', this._dataValue());
            };
            clearTimeout(this.wheelTimeout);
            this.wheelTimeout = setTimeout(function () {
                proxy._triggerEvent('scrollStop', proxy._dataValue());
            }, 400);
            if ('deltaX' in e) {
                wheelDeltaX = -e.deltaX;
                wheelDeltaY = -e.deltaY;
            }
            if ('wheelDeltaX' in e) {
                wheelDeltaX = e.wheelDeltaX / 120;
                wheelDeltaY = e.wheelDeltaY / 120;
            } else if ('wheelDelta' in e) {
                wheelDeltaX = wheelDeltaY = e.wheelDelta / 120;
            } else if ('detail' in e) {
                wheelDeltaX = wheelDeltaY = -e.detail / 3;
            } else {
                if (wheelDeltaX == undefined)
                    return;
            }

            wheelDeltaX *= this.model.wheelSpeed;
            wheelDeltaY *= this.model.wheelSpeed;

            if (!this._hasVScroll) {
                wheelDeltaX = wheelDeltaY;
                wheelDeltaY = 0;
            }

            newX = this._x + Math.round(this._hasHScroll ? wheelDeltaX * this.model.invertWheel : 0);
            newY = this._y + Math.round(this._hasVScroll ? wheelDeltaY * this.model.invertWheel : 0);
            if (newX > 0) {
                newX = 0;
            } else if (newX < this._maxScrollX) {
                newX = this._maxScrollX;
            }
            if (newY > 0) {
                newY = 0;
            } else if (newY < this._maxScrollY) {
                newY = this._maxScrollY;
            }

            clearTimeout(this._wheelEndTimeout);
            this._wheelEndTimeout = setTimeout(function () {
                if (proxy._y - proxy.model.scrollEndThresholdDistance <= proxy._maxScrollY)
                    proxy._trigger('scrollEnd', $.extend(proxy._dataValue(), {
                        position: "bottom"
                    }));
                if (proxy._y >= -proxy.model.scrollEndThresholdDistance)
                    proxy._trigger('scrollEnd', $.extend(proxy._dataValue(), {
                        position: "top"
                    }));
            }, 400);

            this.scrollTo(newX, newY, 0);
        },
        _initKeys: function (e) {
            var keys = {
                pageUp: 33,
                pageDown: 34,
                end: 35,
                home: 36,
                left: 37,
                up: 38,
                right: 39,
                down: 40
            };
            var i;
            if (typeof this.model.enableKeys == 'object') {
                for (i in this.model.enableKeys) {
                    if (typeof this.model.enableKeys[i] == 'string') {
                        this.mdoel.enableKeys[i] = this.model.enableKeys[i].toUpperCase().charCodeAt(0);
                    }
                }
            } else {
                this.model.enableKeys = {};
            }
            for (i in keys) {
                this.model.enableKeys[i] = this.model.enableKeys[i] || keys[i];
            }
            this._keyDownDelegate = $.proxy(this._keyPress, this);
            ej.listenTouchEvent(window, "keydown", this._keyDownDelegate);
        },
        _startAni: function () {
            var proxy = this,
                startX = this._startX,
                startY = this._startY,
                startTime = Date.now(),
                step, easeOut,
                animate;

            var proxy = this;
            if (this._animating) return;

            if (!this._steps.length) {
                this.resetPosition(400);
                return;
            }

            step = this._steps.shift();

            if (step.x == startX && step.y == startY) step.time = 0;

            this._animating = true;
            this._moved = true;

            animate = function () {
                var now = Date.now(),
                    newX, newY;
                if (now >= startTime + step.time) {
                    proxy._translate(step.x, step.y);
                    proxy._animating = false;
                    proxy._moved = false;
                    if (proxy.model.scrollStop) proxy._triggerEvent('scrollStop', proxy._dataValue());
                    proxy._startAni();
                    return;
                }

                now = (now - startTime) / step.time - 1;
                easeOut = Math.sqrt(1 - now * now);
                newX = (step.x - startX) * easeOut + startX;
                newY = (step.y - startY) * easeOut + startY;
                proxy._translate(newX, newY);
                if (proxy._animating) proxy._aniTime = nextFrame(animate);
            };

            animate();
        },
        _keyPress: function (e) {
            if (!this.model.enabled) {
                return;
            }
            var newX = this._x,
                newY = this._y,
                now = this._getTime(),
                prevTime = this._keyTime || 0,
                acceleration = 0.250,
                pos;
            if (this.model.enableTransition && this._isInTransition) {
                pos = this.getComputedPosition();

                this._translate(Math.round(pos.x), Math.round(pos.y));
                this._isInTransition = false;
            }
            this._keyAcc = now - prevTime < 200 ? Math.min(this._keyAcc + acceleration, 50) : 0;
            switch (e.keyCode) {
            case this.model.enableKeys.pageUp:
                if (this._hasHScroll && !this._hasVScroll) {
                    newX += this._wrpWd;
                } else {
                    newY += this._wrpHt;
                }
                break;
            case this.model.enableKeys.pageDown:
                if (this._hasHScroll && !this._hasVScroll) {
                    newX -= this._wrpWd;
                } else {
                    newY -= this._wrpHt;
                }
                break;
            case this.model.enableKeys.end:
                newX = this._maxScrollX;
                newY = this._maxScrollY;
                break;
            case this.model.enableKeys.home:
                newX = newY = 0;
                break;
            case this.model.enableKeys.left:
                newX += 5 + this._keyAcc >> 0;
                break;
            case this.model.enableKeys.up:
                newY += 5 + this._keyAcc >> 0;
                break;
            case this.model.enableKeys.right:
                newX -= 5 + this._keyAcc >> 0;
                break;
            case this.model.enableKeys.down:
                newY -= 5 + this._keyAcc >> 0;
                break;
            default:
                return;
            }

            if (newX > 0) {
                newX = this.keyAcceleration = 0;
            } else if (newX < this._maxScrollX) {
                newX = this._maxScrollX;
                this._keyAcc = 0;
            }

            if (newY > 0) {
                newY = this._keyAcc = 0;
            } else if (newY < this._maxScrollY) {
                newY = this._maxScrollY;
                this._keyAcc = 0;
            }

            this.scrollTo(newX, newY, 0);

            this._keyTime = now;
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                switch (prop) {
                case "enabled":
                case "invertWheel":
                    break;
                case "targetHeight":
                case "targetWidth":
                    this.refresh();
                    break;
                case "enableFade":
                    for (var i = this._scrollbars.length; i--;) {
                        this._scrollbars[i].model.enableFade = false;
                        this._sbw.children().css("opacity", '');
                    }
                    break;
                default:
                    refresh = true;
                }

            }
            if (refresh)
                this._refresh();

        },

        _refresh: function () {
            this.scrollTo(0, 0, 0);
            this._destroy();
            this._init();
            this._wireEvents();
            $(this._scroller).offset({
                top: this._$wrapper.offset().top
            });

        },

        _clearElement: function () {
            if (this._sbw)
                this._sbw.remove();
            this.element.removeAttr("class");
            var target = typeof this.model.target == 'string' ? $("#" + this.model.target)[0] : (this.model.target == null ? this.element[0] : this.model.target);
            $(target).removeAttr("style");
            $(this._scroller).children().unwrap();
            $(target).removeClass("e-m-scrollpanel e-m-scroll-native e-m-scroll-wrapper e-m-scroll-box e-m-scroll-hide e-m-zoom e-m-abs e-m-adjheader-1");
            if (this.model.enableTransform)
                $($(target).children()[0]).css(ej.transform, "");
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this._wireEvents(true);
            if (this._keyDownDelegate)
                ej.listenTouchEvent(window, "keydown", this._keyDownDelegate, true);
            this._bindWheelEvents(true);
            this._clearElement();
        },

        _getTime: function () {
            return Date.now();
        },


        resetPosition: function (time) {
            if (!this.model.enableNativeScrolling && this.model.enableTransform) {
                var x = this._x,
                    y = this._y;

                time = time || 0;

                if (!this._hasHScroll || this._x > 0) {
                    x = 0;
                } else if (this._x < this._maxScrollX) {
                    x = this._maxScrollX;
                }

                if (!this._hasVScroll || this._y > 0) {
                    y = 0;
                } else if (this._y < this._maxScrollY) {
                    y = this._maxScrollY;
                }

                if (this._lockRefresh && y == 0) {
                    y = this._refresherElement.height();
                }

                if (x == this._x && y == this._y) {
                    return false;
                }

                this.scrollTo(x, y, time, this.model.bounceEasing);

                return true;
            }
        },

        scrollToElement: function (el, time, offsetX, offsetY, easing) {
            el = el.nodeType ? el : this._scroller.querySelector(el);

            if (!el) return;


            var posLeft = el.offsetLeft;
            var posTop = el.offsetTop;

            // if offsetX/Y are true we center the element to the screen
            if (offsetX === true) {
                offsetX = Math.round(el.offsetWidth / 2 - this._wrapper.offsetWidth / 2);
            }
            if (offsetY === true) {
                offsetY = Math.round(el.offsetHeight / 2 - this._wrapper.offsetHeight / 2);
            }

            posLeft = -posLeft;
            posTop = -posTop;
            posLeft -= offsetX || 0;
            posTop -= offsetY || 0;

            posLeft = posLeft > 0 ? 0 : posLeft < this._maxScrollX ? this._maxScrollX : posLeft;
            posTop = posTop > 0 ? 0 : posTop < this._maxScrollY ? this._maxScrollY : posTop;

            time = time === undefined || time === null || time === 'auto' ? Math.max(Math.abs(this._x - posLeft), Math.abs(this._y - posTop)) : time;

            this.scrollTo(posLeft, posTop, time, easing);
        },

        on: function (type, fn) {
            if (!this._scrollEvents[type])
                this._scrollEvents[type] = [];

            this._scrollEvents[type].push(fn);
        },

        _createDefaultScrollbar: function (direction, interactive, type) {
            this._sbw = ej.buildTag('div.e-m-sbw e-m-' + this.model.renderMode),
                sb = ej.buildTag('div.e-m-sb');
            if (direction == 'horizontal' && type)
                this._sbw.addClass('e-m-hr');
            else if (direction == 'vertical' && type)
                this._sbw.addClass('e-m-ver');
            if (!interactive) {
                this._sbw[0].style.pointerEvents = 'none';
            }
            this._sbw.append(sb);
            return this._sbw[0];
        },

        _calculateXY: function () {
            this._wrpWd = this._wrapper.clientWidth;
            this._wrpHt = this._wrapper.clientHeight;
            this._scrollerWidth = this._scroller.scrollWidth * this._scale;
            this._scrollerHeight = this._scroller.scrollHeight * this._scale;
            this._maxScrollX = this._wrpWd - this._scrollerWidth;
            this._maxScrollY = this._wrpHt - this._scrollerHeight;
            this._oldScrollX = this._scroller.scrollWidth;
            this._oldScrollY = this._scroller.scrollHeight;
            this._hasHScroll = this.model.enableHrScroll && this._maxScrollX < 0;
            this._hasVScroll = this.model.enableVrScroll && this._maxScrollY < 0;

            if (!this._hasHScroll) {
                this._maxScrollX = 0;
                this._scrollerWidth = this._wrpWd;
            }
            if (!this._hasVScroll) {
                this._maxScrollY = 0;
                this._scrollerHeight = this._wrpHt;
            }
            return this;
        },

        /*---------------Public Methods---------------*/

        zoom: function (scale, x, y, time) {

            if (scale < this.model.zoomMin)
                scale = this.model.zoomMin;
            else if (scale > this.model.zoomMax)
                scale = this.model.zoomMax;

            if (scale == this._scale) return;


            var relScale = scale / this._scale;

            x = x === undefined ? this._wrpWd / 2 : x;
            y = y === undefined ? this._wrpHt / 2 : y;
            time = time === undefined ? 300 : time;

            x = x + this._wrapperOffset.left - this._x;
            y = y + this._wrapperOffset.top - this._y;

            x = x - x * relScale + this._x;
            y = y - y * relScale + this._y;
            this._scale = scale;
            this.refresh();
            if (x > 0) {
                x = 0;
            } else if (x < this._maxScrollX) {
                x = this._maxScrollX;
            }
            if (y > 0) {
                y = 0;
            } else if (y < this._maxScrollY) {
                y = this._maxScrollY;
            }

            this.scrollTo(x, y, time);
        },

        disable: function () {
            this.model.enabled = false;
        },

        enable: function () {
            this.model.enabled = true;
        },

        closeRefresher: function () {
            this._refresherText.text(this.model.pullToRefreshSettings.textOnPull);
            this._refresherIcon.removeClass("e-m-refresh e-m-rotate").addClass("e-m-pull")
            this._lockRefresh = this._refresherIcon.hasClass("e-m-rotate");
            this.scrollTo(0, 0);
        },

        refresh: function () {
            if (!this._scroller) return;

            var rf = this._wrapper.offsetHeight;

            ej.adjustFixedElement(this._$wrapper);

            if (this.model.targetHeight)
                this._$wrapper.height(this.model.targetHeight);
            if (this.model.targetWidth)
                this._$wrapper.width(this.model.targetWidth);

            this._calculateXY();
            this._endTime = this._directionX = this._directionY = 0;
            this._wrapperOffset = $(this._wrapper).offset();
            if (!this.model.showScrollbars)
                this._$wrapper.addClass("e-m-hidescrollbar");
            if (this.model.enableNativeScrolling)
                this._hasHScroll && this._hasVScroll ? this._$wrapper.css("overflow", "auto") : this._hasVScroll ? this._$wrapper.css("overflow-y", "auto") : this._hasHScroll ? this._$wrapper.css("overflow-x", "auto") : (this.model.renderMode == "windows" && !ej.isMobile()) ? this._$wrapper.css("overflow", "scroll") : this._$wrapper.css("overflow", "auto");
            if (this.model.enableNativeScrolling && ej.isWindows()) {
                if (this._hasHScroll && !this._hasVScroll)
                    this._$wrapper.addClass("e-m-horizontal");
                else if (this._hasVScroll && !this._hasHScroll)
                    this._$wrapper.addClass("e-m-vertical");
            }
            this.resetPosition();
            this._triggerEvent('refresh', this._dataValue());
        },

        scrollBy: function (x, y, time, easing) {
            x = this._x + x;
            y = this._y + y;
            time = time || 0;
            this.scrollTo(x, y, time, easing);
        },

        scrollTo: function (x, y, time, relative, easing) {
            var proxy = this;
            if (!this.model.enableNativeScrolling) {
                if (this._oldScrollX != this._scroller.scrollWidth || this._oldScrollY != this._scroller.scrollHeight)
                    this._calculateXY()._refreshScrollbar();
                easing = easing || ej.ease.circular;
                if (this.model.enableTransition && easing.style) {
                    this._isInTransition = this.model.enableTransition && time > 0;
                    this._transitionTimingFunction(easing.style);
                    this._transitionTime(time);
                    this._translate(x, y);
                } else {
                    if (this._x == x && this._y == y)
                        return;
                    var step = x,
                        i, l;
                    this.stop();
                    if (!step.length) step = [{
                        x: x,
                        y: y,
                        time: time,
                        relative: relative
                    }];
                    for (i = 0, l = step.length; i < l; i++) {
                        if (step[i].relative) {
                            step[i].x = this._x - step[i].x;
                            step[i].y = this._y - step[i].y;
                        }
                        this._steps.push({
                            x: step[i].x,
                            y: step[i].y,
                            time: step[i].time || 0
                        });
                    }
                    this._startAni();
                }
            } else {
                this._nx = x, this._ny = y;
                this._scrollTo({
                    top: y,
                    left: x
                }, time, {
                    queue: true,
                    easing: 0,
                    onAfter: function () {
                        proxy._nx = $(proxy._wrapper).scrollLeft();
                        proxy._ny = $(proxy._wrapper).scrollTop();
                    }
                });
            }

        },

        getComputedPosition: function () {
            var matrix = window.getComputedStyle(this._scroller, null),
                x, y;

            if (this.model.enableTransform) {
                matrix = matrix[ej.transform].split(')')[0].split(', ');
                x = +(matrix[12] || matrix[4]);
                y = +(matrix[13] || matrix[5]);
            } else {
                x = +matrix.left.replace(/[^-\d.]/g, '');
                y = +matrix.top.replace(/[^-\d.]/g, '');
            }

            return {
                x: x,
                y: y
            };
        },


        stop: function () {
            cancelFrame(this._aniTime);
            this._steps = [];
            this._moved = false;
            this._animating = false;
        },

        getScrollPosition: function () {
            return {
                "x": this.model.enableNativeScrolling ? this._nx : this._x,
                "y": this.model.enableNativeScrolling ? this._ny : this._y
            };
        }

        /*---------------Public Methods End---------------*/

    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html slider elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    // ejSlider is the plugin name 
    // "ej.slider" is "namespace.className" will hold functions and properties

    ej.widget("ejmSlider", "ej.mobile.Slider", {
        _setFirst: true,
        _rootCSS: "e-m-slider",

        defaults: {
            theme: "auto",
            orientation: "horizontal",
            maxValue: 100,
            minValue: 0,
            enableRange: false,
            values: [20, 80],
            enableAnimation: false,
            animationSpeed: 400,
            readOnly: false,
            incrementStep: 1,
            value: 0,
            enabled: true,
            enablePersistence: false,
            touchStart: null,
            touchEnd: null,
            change: null,
            slide: null,
            load: null,
            windows: {
                "renderDefault": false
            },
            ios7: {
                "thumbStyle": "normal"
            },
            renderMode: "auto"
        },
        dataTypes: {
            theme: "enum",
            orientation: "enum",
            enableRange: "boolean",
            thumbStyle: "enum",
            enableAnimation: "boolean",
            enabled: "boolean",
            renderMode: "enum",
            values: "array",
            enablePersistence: "boolean"
        },
        observables: ["value"],
        observableArray: ["values"],
        value: ej.util.valueFunction("value"),
        values: ej.util.valueFunction("values"),
        // constructor function
        _init: function () {
            if (!App.activePage)
                App.createAppView();
            this._cloneElement = this.element.clone();
            this._load();
            ej.setRenderMode(this);
            ej.setTheme(this);
            this._renderControl();
            this._initObjects();
            if (!this.model.readOnly)
                this._wireEvents(false);
        },
        _renderControl: function () {
            var obj = this.model;
            if (!this.model.enabled)
                this.element.addClass("e-m-state-disabled");
            if (this.model.renderMode == "windows" && this.model.windows.renderDefault)
                this.model.theme = "default";
            this._sliderHandle = ej.buildTag("div .e-m-slider-outer");
            this._selectedRegion = ej.buildTag("div .e-m-slider-inner").appendTo(this._sliderHandle);
            if (this.model.renderMode == "ios7" && this.model.ios7.thumbStyle == "small")
                this._dragHandle = ej.buildTag("div .e-m-slider-handleout e-m-small-handle e-m-slider-left ").appendTo(this._sliderHandle);
            else
                this._dragHandle = ej.buildTag("div .e-m-slider-handleout e-m-slider-left ").appendTo(this._sliderHandle);
            var handleIn = ej.buildTag("div .e-m-slider-handlein").appendTo(this._dragHandle);
            if (obj.enableRange) {
                if (this.model.renderMode == "ios7" && this.model.ios7.thumbStyle == "small")
                    var handleRange = ej.buildTag("div .e-m-slider-handleout e-m-small-handle e-m-slider-right");
                else
                    var handleRange = ej.buildTag("div .e-m-slider-handleout e-m-slider-right");
                var rangeHandleIn = ej.buildTag("div .e-m-slider-handlein").appendTo(handleRange);
                handleRange.appendTo(this._sliderHandle);
            }
            this.element.addClass("e-m-" + obj.orientation + " e-m-" + obj.theme).addClass("e-m-slider e-m-" + obj.renderMode).append(this._sliderHandle);
            this.element.append(ej.buildTag("input ", {}, {}, { "type": "hidden", "name": "sliderValue" }));
            this._dragHandle = this.element.find(".e-m-slider-handleout");
            this._inputValue = this.element.find("input");
            this._wrapperDiv = this.element;
            this._currentValue = this.value();
            this["_" + obj.orientation + "RenderMode"]();
        },
        _horizontalRenderMode: function () {
            this._setOrientationProperties("width", "left");
        },
        _verticalRenderMode: function () {
            this._setOrientationProperties("height", "bottom");
        },
        _setOrientationProperties: function (dimensionProp, posProp) {
            var obj = this.model;
            if (obj.enableRange) {
                if (this.values()[0] <= this.values()[1]) {
                    this.values()[0] = (this.values()[0] < obj.minValue && this.values()[0] <= obj.maxValue) ? obj.minValue : this.values()[0];
                    this.values()[1] = (this.values()[1] >= obj.minValue && this.values()[1] > obj.maxValue) ? obj.maxValue : this.values()[1];
                }
                else
                    this.values()[1] = this.values()[0];
                this._selectedRegion.css(posProp, parseFloat(((this.values()[0] - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) + "%").css(dimensionProp,
                    parseFloat(((this.values()[1] - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) - parseFloat(((this.values()[0] - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) + "%");
                $(this._dragHandle[0]).css(posProp, parseFloat(((this.values()[0] - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) + "%");
                $(this._dragHandle[1]).css(posProp, parseFloat(((this.values()[1] - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) + "%");
            }
            else {
                this.value((this.value() < obj.minValue) ? obj.minValue : this.value());
                this.value((this.value() > obj.maxValue) ? obj.maxValue : this.value());
                this._selectedRegion.css(dimensionProp, parseFloat(((this.value() - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) + "%");
                this._dragHandle.css(posProp, parseFloat(((this.value() - obj.minValue) / (obj.maxValue - obj.minValue)) * 100) + "%");
            }
        },
        _initObjects: function () {
            this._createDelegates();
        },
        _createDelegates: function () {
            this._mouseDownHandler = $.proxy(this._onMouseDownHandler, this);
            this._mouseMoveHandler = $.proxy(this._onMouseMove, this);
            this._mouseUpHandler = $.proxy(this._onMouseUp, this);
        },
        _onMouseDownHandler: function (evt) {
            ej.blockDefaultActions(evt);
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            if (!this.model.enabled)
                return false;
            jQuery.fx.off = false;
            this.sliderClick = evt;
            if (!this.model.enableRange)
                this._dragHandle.addClass("e-m-state-active");
            ej.listenEvents([document, document], [ej.moveEvent(), ej.endEvent()], [this._mouseMoveHandler, this._mouseUpHandler], false);
            var index;
            var position = { x: evt.pageX, y: evt.pageY };
            var normalValue = this._getNormalValue(position);
            this._currentValue = normalValue;
            var self = this;
            var distance = this.model.maxValue - this.model.minValue + 1;
            $(this._dragHandle).each(function (curIndex) {
                var curDistance = Math.abs(normalValue - self._getNewValues(curIndex));
                if (distance >= curDistance) {
                    if (self._getNewValues(0) != self._getNewValues(1)) {
                        distance = curDistance;
                        self._closestHandle = $(this);
                        index = curIndex;
                    }
                }
            });
            $(this.model.enableRange ? this._closestHandle : this._dragHandle).removeClass("e-m-state-hover").addClass("e-m-state-active");
            this._start(evt, index);
            if (this.model.enableRange) {
                this._setZindex();
                if (self._getNewValues(0) == self._getNewValues(1)) {
                    self._closestHandle = $(this);
                    index = this._handlingIndex;
                }
                this._setRangeValueProperties(evt, index);
            }
        },
        _setZindex: function () {
            this._dragHandle.css("z-index", 0);
            this._closestHandle.css("z-index", 5);
        },
        _setRangeValueProperties: function (evt, index) {
            var tempValues = [];
            var otherValues = [];
            var highestValue, lowestValue;
            var tempValue = this.values()[index];
            this._closestHandle = $(this._dragHandle[index]);
            this._handlingIndex = index;
            for (var curValue = 0; curValue < this.values().length; curValue++) {
                if (curValue != index)
                    otherValues.push(this.values()[curValue]);
                tempValues.push(this.values()[curValue]);
            }
            for (var curValue = 0; curValue < otherValues.length; curValue++) {
                if (tempValue >= otherValues[curValue]) {
                    this._closestLeftValue = otherValues[curValue];
                    break;
                }
                else if (tempValue <= otherValues[curValue]) {
                    this._closestRightValue = otherValues[curValue];
                    break;
                }
            }
            lowestValue = tempValues[0];
            highestValue = tempValues[tempValues.length - 1];
            if (lowestValue != highestValue) {
                if (tempValue <= lowestValue)
                    this._closestLeftValue = this.model.minValue;
                else if (tempValue >= highestValue)
                    this._closestRightValue = this.model.maxValue;
            }
            else if (lowestValue == highestValue) {
                if (this._closestHandle.hasClass("e-m-slider-left"))
                    this._closestLeftValue = this.model.minValue;
                else if (this._closestHandle.hasClass("e-m-slider-right"))
                    this._closestRightValue = this.model.maxValue;
            }
            if (evt.preventDefault)
                evt.preventDefault();
            return true;
        },
        _onMouseMove: function (evt, bAnimate) {
            ej.blockDefaultActions(evt);
            if (ej.isTouchDevice())
                evt = evt.touches ? evt.touches[0] : evt;
            if (!this.model.enabled)
                return false;
            jQuery.fx.off = true;
            this.element.css("cursor", "pointer");
            var position = { x: evt.pageX, y: evt.pageY };
            var normalValue = this._getNormalValue(position);
            this._currentValue = normalValue;
            if (this.model.enableRange)
                this._slide(evt, normalValue, this._handlingIndex);
            else
                this._slide(evt, normalValue);
            $(this.model.enableRange ? this._closestHandle : this._dragHandle).addClass("e-m-state-active");
            if (evt.preventDefault)
                evt.preventDefault();
        },
        _onMouseUp: function (evt) {
            ej.blockDefaultActions(evt);
            if (ej.isTouchDevice())
                evt = evt.changedTouches ? evt.changedTouches[0] : evt;
            if (!this.model.enabled)
                return false;
            jQuery.fx.off = false;
            ej.listenEvents([document, document], [ej.moveEvent(), ej.endEvent()], [this._mouseMoveHandler, this._mouseUpHandler], true);
            if (this._mouseMoveStarted) {
                this._mouseMoveStarted = false;
                if (evt.target == this.sliderClick.target)
                    $.data(evt.target, this.model.sliderId + '.preventClickEvent', true);
            }
            this.element.css("cursor", "default");
            var position = { x: evt.pageX, y: evt.pageY };
            var normalValue = this._getNormalValue(position);
            if (this.model.enableRange)
                this._slide(evt, normalValue, this._handlingIndex);
            else
                this._slide(evt, normalValue);
            this._stop(evt, this._handlingIndex);
            this._change(evt, this._handlingIndex);
            $(this.model.enableRange ? this._closestHandle : this._dragHandle).removeClass("e-m-state-hover").removeClass("e-m-state-active");
            return false;
        },
        _getNewValues: function (index) {
            if (!this.model.enableRange)
                return this.value();
            else
                return this._trimValue(this.values()[index]);
        },
        _getCurrentValue: function (newValue, index) {
            if (this.model.enableRange) {
                this.values()[index] = this._trimValue(newValue);
                this.value(this.values()[index]);
            }
            else {
                this.value(this._trimValue(newValue));
                this._currentValue = newValue;
            }
            this._refreshValue();
            return this._trimValue(this.value());
        },
        _refreshValue: function () {
            var animateOrCss = this.model.enableAnimation ? "animate" : "css";
            var values;
            if (this.model.enableRange) {
                values = (this.values()[0] + "," + this.values()[1]);
                this._inputValue.val(values);
            }
            else
                this._inputValue.val(this._currentValue);
            correctedPercent = (this._dragHandle.width() / 2) / this._dragHandle.parent().width() * 100;
            if ((this.model.enableRange))
                this._rangeRefreshProperties();
            else {
                var percentValue = (this.model.minValue != this.model.maxValue) ? (this.value() - this.model.minValue) / (this.model.maxValue - this.model.minValue) * 100 : 0;
                var handlePercent = percentValue - ((this._dragHandle.width() / $(this._dragHandle.parent()).width()) * 100);
                if (handlePercent < 0)
                    handlePercent = percentValue;
                if (this.model.orientation == "horizontal")
                    this._rangeRefreshOrientationProperties({ 'width': percentValue + "%" }, { 'left': percentValue + "%" }, this.model.animationSpeed, animateOrCss);
                else
                    this._rangeRefreshOrientationProperties({ 'height': percentValue + "%" }, { 'bottom': percentValue + "%" }, this.model.animationSpeed, animateOrCss);
            }
        },
        _rangeRefreshProperties: function () {
            var left, width, bottom, left;
            var self = this;
            var animateOrCss = this.model.enableAnimation ? "animate" : "css";
            var animateValue = this.model.animationSpeed;
            var maxPercentValue = (this.values()[1] - this.model.minValue) / (this.model.maxValue - this.model.minValue) * 100;
            var minPercentValue = (this.values()[0] - this.model.minValue) / (this.model.maxValue - this.model.minValue) * 100;
            if (this.model.orientation == "horizontal") {
                this._selectedRegion[animateOrCss]({ "left": (minPercentValue) + "%" }, animateValue)
                [animateOrCss]({ "width": (maxPercentValue - minPercentValue) + "%" }, animateValue);
            }
            else {
                this._selectedRegion[animateOrCss]({ "bottom": (minPercentValue) + "%" }, animateValue)
                    [animateOrCss]({ "height": (maxPercentValue - minPercentValue) + "%" }, animateValue);
            }
            this._dragHandle.each(function (index, element) {
                percentValue = (self.values()[index] - self.model.minValue) / (self.model.maxValue - self.model.minValue) * 100;
                handlePercent = percentValue - (($(this).width() / $($(this).parent()).width()) * 100);
                if (handlePercent < 0)
                    handlePercent = percentValue;
                if (self.model.orientation == "horizontal")
                    $(this)[animateOrCss]({ "left": percentValue + "%" }, animateValue);
                else
                    $(this)[animateOrCss]({ "bottom": percentValue + "%" }, animateValue);
                lastPercentValue = percentValue;
            });
        },
        _rangeRefreshOrientationProperties: function (dimObj, posObj, animateValue, animateOrCss) {
            this._selectedRegion[animateOrCss](dimObj, animateValue);
            this._dragHandle[animateOrCss](posObj, animateValue);
        },

        _getNormalValue: function (position) {
            var currentLOB, currentLOBPercent, totalValue, currentValue; //currentLOB = current left or bottom
            if (this.model.orientation == "vertical") {
                currentLOB = position.y - this._sliderHandle.offset().top;
                currentLOBPercent = (currentLOB / this._sliderHandle[0].offsetHeight);
            }
            else {
                currentLOB = position.x - this._sliderHandle.offset().left;
                currentLOBPercent = (currentLOB / this._sliderHandle[0].offsetWidth);
            }

            if (this.model.orientation == "vertical")
                currentLOBPercent = 1 - currentLOBPercent;
            totalValue = this.model.maxValue - this.model.minValue;
            currentValue = this.model.minValue + currentLOBPercent * totalValue;
            return this._trimValue(currentValue);
        },
        _trimValue: function (value) {
            if (!this.model.enableRange) {
                if (value < this.model.minValue)
                    value = this.model.minValue;
                else if (value > this.model.maxValue)
                    value = this.model.maxValue;
            }
            var step = (this.model.incrementStep > 0) ? this.model.incrementStep : 1;
            var stepModValue = (value - this.model.minValue) % step;
            var correctedValue = value - stepModValue;
            if (Math.abs(stepModValue) * 2 >= step)
                correctedValue += (stepModValue > 0) ? step : (-step);
            return parseFloat(correctedValue.toFixed(5));
        },
        _slide: function (evt, newValue, index) {
            if (this.model.enableRange) {
                if (newValue != this._getCurrentValue()) {
                    this.value(newValue);
                    if (newValue < this._closestLeftValue)
                        newValue = this._closestLeftValue;
                    else if (newValue > this._closestRightValue)
                        newValue = this._closestRightValue;
                    if (newValue != this._getNewValues(index)) {
                        var newValues = this._getCurrentValue(newValue, index);
                        newValues[index] = newValue;
                        var data = { value: this.value(), values: this.values() };
                        this.values([this.values()[0], this.values()[1]]);
                        this._triggerSlide(data, newValue, this._getNewValues(index), index);
                    }
                }
            }
            else {

                if (newValue != this.value()) {
                    this._getCurrentValue(newValue);
                    var data = { value: this.value(), values: this.values() };
                    this._triggerSlide(data, newValue, this.value());
                }
            }

        },
        _triggerSlide: function (data, newValue, prevValue, index) {
            if (this.model.slide)
                this._trigger("slide", data);
            this._getCurrentValue(newValue, index);
            this.prevValue = this._getNewValues(index);
        },
        //client side events
        _load: function () {
            var data = (this.model.enableRange) ? { values: this._getNewValues() } : { value: this.value() };
            if (this.model.load)
                this._trigger("load", data);
        },

        _start: function (evt, index) {
            var data = { value: this.value(), values: this.values() };
            if (this.model.enableRange) {
                data._getCurrentValue = this._getNewValues(index);
                data.values = this._getNewValues();
            }
            else {
                data._getCurrentValue = this.value();
                data.value = this.value();

            }
            if (this.model.touchStart)
                this._trigger("touchStart", data);
        },
        _stop: function (evt, index) {
            var data = { value: this.value(), values: this.values() };
            if (this.model.enableRange) {
                data._getCurrentValue = this._getNewValues(index);
                data.values = this._getNewValues();
            }
            else {
                data._getCurrentValue = this.value();
                data.value = this.value();
            }
            if (this.model.touchEnd)
                this._trigger("touchEnd", data);
        },
        _change: function (evt, index) {
            var data = { value: this.value(), values: this.values() };
            if (this.model.enableRange) {
                data._getCurrentValue = this._getNewValues(index);
                data.values = this._getNewValues();
            }
            else {
                data._getCurrentValue = this.value();
                data.value = this.value();

            }
            if (this.model.change)
                this._trigger("change", data);
        },
        _wireEvents: function (remove) {
            ej.listenEvents([this._sliderHandle, this.element], [ej.startEvent(), ej.startEvent()], [this._mouseDownHandler, this._mouseDownHandler], remove);
        },

        _setRenderMode: function (mode) {
            if (this.model.enabled) {
                this.model.renderMode = mode;
                if (mode == "auto")
                    ej.setRenderMode(this);
                this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
            }
        },

        _setTheme: function (theme) {
            if (this.model.enabled) {
                this.model.theme = theme;
                this.element.removeClass("e-m-dark e-m-light").addClass("e-m-" + theme);
            }
        },

        _validateValue: function (value) {
            if (this.model.enabled) {
                if (!this.model.enableRange)
                    this["_" + this.model.orientation + "RenderMode"]();
            }
        },

        _validateRangeValue: function (rangeValue) {
            this["_" + this.model.orientation + "RenderMode"]();
            if (this.values()[0] == this.values()[1]) {
                (this.model.maxValue == this.values()[1]) ? this._setRangeValueProperties("", 0) : this._setRangeValueProperties("", 1);
            }
        },

        _disable: function () {
            this.model.enabled = false;
            this.element.addClass("e-m-state-disabled");
            this._wireEvents(true);
        },

        _enable: function () {
            this.model.enabled = true;
            this.element.removeClass("e-m-state-disabled");
            this._wireEvents();
        },

        _enableAnimation: function (isAnimate) {
            if (this.model.enabled)
                this.model.enableAnimation = isAnimate;
        },

        _refresh: function () {
            if (this.model.enabled) {
                this._clearElements();
                this._renderControl();
                this._wireEvents(false);
            }
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                switch (prop) {
                    case "renderMode": this._setRenderMode(options.renderMode); break;
                    case "theme": this._setTheme(options.theme); break;
                    case "orientation": this._refresh(); break;
                    case "value": this._validateValue(options.value); break;
                    case "values": if (this.model.enableRange && this.model.enabled)
                        this._validateRangeValue(options.values);
                        break;
                    case "enabled": (options.enabled) ? this._enable() : this._disable(); break;
                    case "enableRange": this._refresh(); break;
                    case "minValue": this.model.minValue = options.minValue; break;
                    case "maxValue": this.model.maxValue = options.maxValue; break;
                    case "enableAnimation": this._enableAnimation(options.enableAnimation); break;
                    case "readOnly": (options.readOnly) ? this._wireEvents(true) : this._wireEvents(false); break;
                }
            }
        },
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this._cloneElement.insertBefore(this.element);
            this.element.remove();
        },
        _clearElements: function () {
            this._wireEvents(true);
            this.element.removeAttr("class");
            this.element.removeAttr("style");
            this.element.html("");
        },

        /*---------------Public Methods---------------*/
        getValue: function () {
            if (this.model.enableRange)
                return (this.values()[0] + "," + this.values()[1]);
            else
                return this._trimValue(this.value());
        },
        //client side methods
        dispose: function () {
            return this._destroy();
        }
        /*---------------Public Methods End---------------*/

    });

    ej.mobile.Slider.Orientation = {
        Horizontal: "horizontal",
        Vertical: "vertical"
    };

    ej.mobile.Slider.ThumbStyle = {
        Normal: "normal",
        Small: "small"
    };
})(jQuery, Syncfusion);;
/**
 * @fileOverview Plugin to style the Html SplitPane elements
 * @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
 *  Use of this code is subject to the terms of our license.
 *  A copy of the current license can be obtained at any time by e-mailing
 *  licensing@syncfusion.com. Any infringement will be prosecuted under
 *  applicable laws. 
 * @version 12.1 
 * @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
 */
(function ($, ej, undefined) {
    ej.widget("ejmSplitPane", "ej.mobile.SplitPane", {
        _setFirst: true,
        _rootCSS: "e-m-splitpane",
        _ignoreOnPersist: ["create","destroy","beforeOpen","beforeClose","close","open","swipe"],
        angular: {
            terminal: false
        },
        defaults: {
            renderMode: "auto",
            cssClass: "",
            enablePersistence: false,
            height: "auto",
            width: "auto",
            isRelative: false,
            edgeThreshold: 50,
            enableSwipe: true,
            stopEventPropagation: ej.isWindows(),
            leftPane: {
                templateId: null,
                showOnTablet: true,
                animationType: "overlay",
                width: "280px"
            },
            rightPane: {
                templateId: null,
                showOnTablet: false,
                animationType: "overlay",
                width: "280px"
            },
            contentPane: {
                templateId: null
            },
            swipe: null,
            open: null,
            close: null,
            beforeClose: null,
            beforeOpen: null,
            create: null,
            destroy: null

        },

        dataTypes: {
            renderMode: "enum",
            enablePersistence: "boolean",
            isRelative: "boolean",
            edgeThreshold: "number",
            stopEventPropagation: "boolean",
            leftPane: "object",
            rightPane: "object",
            contentPane: "object"
        },

        _lModel: {
            overlayOpacity: 0.3,
            swipeThreshold: 15,
            defaultTransitionTime: 400,
            fastTransitionTime: 200
        },

        _class: {
            opened: "e-m-opened",
            hide: "e-m-hide",
            show: "e-m-show",
            hidden:"e-m-hidden",
        },

        _init: function () {
            this._isNavPaneOpened=false;
            this._orgEleStyle = $(this.element).attr("style");
            this._orgEleClass = $(this.element).attr("class");
            this._browser = ej.browser().toLowerCase();
            this._cssClass = this.model.cssClass;
            ej.setRenderMode(this);
            this._initPane();
            this._renderControl();
            this._createDelegates();
            this._wireEvents();
        },

        _initPane: function () {
            //Initialize the pane Objects
            this._panel = {
                left: (this.model.leftPane.templateId ? this._getTemplate(this.model.leftPane.templateId, "_leftpane") : (this.element.find("[data-ej-pane='left']").length ? this.element.find("[data-ej-pane='left']") : this.element.find("[e-pane='left']"))),
                content: (this.model.contentPane.templateId ? this._getTemplate(this.model.contentPane.templateId, "_contentpane") : (this.element.find("[data-ej-pane='content']").length ? this.element.find("[data-ej-pane='content']") : this.element.find("[e-pane='content']"))),
                right: (this.model.rightPane.templateId ? this._getTemplate(this.model.rightPane.templateId, "_rightpane") : (this.element.find("[data-ej-pane='right']").length ? this.element.find("[data-ej-pane='right']") : this.element.find("[e-pane='right']")))
            };
            //add class to all panes
            this._panel.left.addClass("e-m-type-" + this.model.leftPane.animationType + " e-m-sp-left e-m-pane e-m-hidden e-m-abs");
            this._panel.right.addClass("e-m-type-" + this.model.rightPane.animationType + " e-m-sp-right e-m-pane e-m-hidden e-m-abs");
            this._panel.content.addClass("e-m-sp-content e-m-pane e-m-abs");
            //creating overlay
            this._elementOverlay = ej.buildTag("div#" + this._id + "_Overlay", {}, {}, {
                "class": "e-m-overlay e-m-splitpane-overlay e-m-hide e-m-abs"
            });
            this._elementOverlay.appendTo(this._panel.content);

            return this;
        },

        _getTemplate: function (eleId, pane) {
            var returnEle = null;
            if ($("#" + eleId)[0].nodeName == "SCRIPT") {
                returnEle = ej.buildTag("div#" + this._id + pane);
                returnEle.html($("#" + eleId).text()).appendTo(this.element);
                ej.widget.init(returnEle);
                if (ej.angular.defaultAppName && angular)
                    ej.angular.compile(returnEle);
            } else {
                returnEle = $("#" + eleId);
                if ($("#" + eleId).parent() != this.element)
                    returnEle.appendTo(this.element);
            }
            return returnEle;
        },

        _renderControl: function () {
            this._direction = ["left", "right"];
            this.element.addClass("e-m-splitpane e-m-" + this.model.renderMode + " e-m-" + (ej.isMobile() ? "mobile " : "tablet ") + this.model.cssClass)
                .addClass(ej.isLowerResolution() ? "e-m-lower" : "e-m-higher")
                .addClass(this.model.isRelative ? "e-m-rel" : "e-m-abs");
            this._setPaneSize();
        },

        _createDelegates: function () {
            // Swipe Event Delegates
            this._swipeStartHandler = $.proxy(this._swipeStart, this);
            this._swipeEndHandler = $.proxy(this._swipeEnd, this);
            this._swipeMoveHandler = $.proxy(this._swipeMove, this);

            this._overlayTapStartDelegate = $.proxy(this._overlayTapStart, this);
            this._overlayTapEndDelegate = $.proxy(this._overlayTapEnd, this);
            this._resizeDelegate = $.proxy(this._resize, this);
        },

        _wireEvents: function (remove) {
            var eventType = !remove && this.model.enableSwipe ? "bind" : "unbind";
            this._bindSwipeEvent(remove);
            ej.listenTouchEvent(this._elementOverlay, ej.startEvent(), this._overlayTapStartDelegate, remove);
            if (ej.isTouchDevice() && this._isOrientationSupported())
                ej.listenTouchEvent(window, "orientationchange", this._resizeDelegate, remove, this);
            else
                ej.listenTouchEvent(window, "resize", this._resizeDelegate, remove, this);
        },
        _bindSwipeEvent: function (remove) {
            if (this.model.enableSwipe)
                this._on(this._panel.content, "mousedown touchstart MSPointerDown pointerdown", this._swipeStartHandler);
        },

        _unbindSwipeEvent: function () {
            if (this.model.enableSwipe){
                this._off(this._panel.content, "mousedown touchstart MSPointerDown pointerdown", this._swipeStartHandler);
                this._off(t.element, "mousemove touchmove MSPointerMove pointermove", this._swipeMoveHandler);
                this._off(t.element, "mouseup touchend MSPointerUp pointerup", this._swipeEndHandler);
                this._off(t.element, "mousecancel touchcancel MSPointerCancel pointercancel", this._swipeEndHandler);
            }
        },

        _overlayTapStart: function (e) {
            ej.listenTouchEvent(this._elementOverlay, ej.endEvent(), this._overlayTapEndDelegate);

        },

        _overlayTapEnd: function (e) {
            if (!this._cancelClose)
                this._closeOpenedPane(this._lModel.defaultTransitionTime);
            ej.listenTouchEvent(this._elementOverlay, ej.endEvent(), this._overlayTapEndDelegate, true);

        },

        _swipeStart: function (e) {
            if(this._isNavPaneOpened==true) return;
            var point = e.originalEvent.changedTouches ? e.originalEvent.changedTouches[0] : e;
            var t = this;
            t._startX = point.clientX;
            t._cancelClose = null;
            t._startTime = new Date();
            t._ap = (point.clientX < t.element.offset().left + t.model.edgeThreshold) ? "left" :
                t._ap = (point.clientX < t.element.offset().left + t.model.edgeThreshold) ? "left" :
                ((point.clientX > t.element.width() - t.model.edgeThreshold) ? "right" : "");
            if (t._isPaneOpen())
                t._ap = t._isPaneOpen();
            if (t._ap && t._panel[t._ap].length) {
                t._tp = t._getTransitionPane(t._ap);
                if ((ej.isLowerResolution() * (t.model.rightPane.showOnTablet))) t._panel.right.hide();
                if ((ej.isLowerResolution() * (t.model.leftPane.showOnTablet))) t._panel.left.hide();
                t._panel[t._ap].show();
                if (t.model.stopEventPropagation && ej.browserInfo().name != "msie") ej.blockDefaultActions(e);
            } else
                return;
            this._on(t.element, "mousemove touchmove MSPointerMove pointermove", this._swipeMoveHandler);
            this._on(t.element, "mouseup touchend MSPointerUp pointerup", this._swipeEndHandler);
            this._on(t.element, "mousecancel touchcancel MSPointerCancel pointercancel", this._swipeEndHandler);
        },

        _swipeMove: function (e) {
            if (this._isNavPaneOpened==true || !this._ap) return;
            var point = e.originalEvent.changedTouches ? e.originalEvent.changedTouches[0] : e;
            var t = this;
            if (t.model.stopEventPropagation && ej.browserInfo().name == "msie") ej.blockDefaultActions(e);
            t._swipeDirection = (t._startX > point.clientX) ? "right" : ((t._startX == point.clientX) ? "" : "left");
            var distanceX = point.clientX - t._startX,
                width = t._panel[t._ap].outerWidth(),
                _sign = (t._ap == "left" ? +1 : -1);
            if (t._panel[t._swipeDirection] && t._panel[t._swipeDirection].length && !t._isPaneOpen()) t._panel[t._swipeDirection].removeClass(t._class.hidden).addClass("e-m-opening");
            if (t._tp && !t._isPaneOpen() && t._ap) {
                distanceX = width <= Math.abs(distanceX) ? _sign * width : distanceX;
                t._transform(distanceX, 0, t._tp)._elementOverlay.removeClass(t._class.hide);
                if (t.model[t._ap + "Pane"].animationType != "overlay")
                    t._elementOverlay.css("opacity", 0);
                else
                    t._elementOverlay.css("opacity", Math.abs(distanceX / t._panel[t._ap].width()) * t._lModel.overlayOpacity);
            } else if (t._tp && t._ap && (t._isPaneOpen() != t._swipeDirection) && t._swipeDirection!="") {
                distanceX = width <= Math.abs(distanceX) ? 0 : (_sign * distanceX) + width;
                t._transform(_sign * distanceX, 0, t._tp);
                t._cancelClose = true;
                if (t.model[t._ap + "Pane"].animationType != "reveal")
                    t._elementOverlay.css("opacity", Math.abs(distanceX / t._panel[t._ap].width()) * t._lModel.overlayOpacity);
            }
            t._movedX = distanceX;
        },

        _swipeEnd: function (e) {
            if(this._isNavPaneOpened==true || this._ap==null || this._ap=="") return;
            var point = e.originalEvent.changedTouches ? e.originalEvent.changedTouches[0] : e;
            var t = this;
            var mX = Math.abs(t._movedX),
                mTime = new Date() - t._startTime,
                fastSwipe = false;
            if (Math.abs(mTime) < 100) fastSwipe = true;
            if (t._panel[t._swipeDirection] && t._panel[t._swipeDirection].length && !t._isPaneOpen()) t._panel[t._swipeDirection].removeClass("e-m-opening");
            mTime = t._getMomentumTime(t._movedX, Math.abs(mTime), t._panel[t._ap].outerWidth());
            mTime = mTime > 500 ? 500 : mTime;
            if ((t._cancelClose && t._swipeDirection) || (mX && !t._isPaneOpen() && t._tp && t._ap)) {
                var width = t._panel[t._ap].outerWidth();
                var distanceX = ((t._ap == "left") ? width : -width);
                var avgDist = (t._panel[t._ap].outerWidth() / 100) * (t._lModel.swipeThreshold);
                if (((avgDist < mX) && !t._cancelClose) || ((avgDist > (width - mX)) && t._cancelClose)) {
                    t._transform(distanceX, (!t._isPaneOpen() && !t._cancelClose ? mTime : t._lModel.defaultTransitionTime), t._tp);
                    opacity = t.model[t._ap + "Pane"].animationType != "overlay" ? 0 :
                        Math.abs(distanceX / t._panel[t._ap].width()) * t._lModel.overlayOpacity;
                    t._elementOverlay.css("opacity", opacity);
                    t._panel[t._ap].addClass(t._class.opened);

                } else {
                    if (!t._cancelClose) {
                        t._transform(0, t._lModel.fastTransitionTime, t._tp)._elementOverlay.addClass(t._class.hide);
                        t._panel[t._ap].addClass(t._class.hidden);
                    } else
                        t._closeOpenedPane((mTime < 100 ? 100 : mTime));

                }
            } else
                t._closeOpenedPane(t._lModel.defaultTransitionTime);

            this._trigger("swipe", {
                direction: t._swipeDirection,
                event: e
            });
            t._lastClientX = t._swipeDirection = t._ap = t._tp = t._movedX = t._startTime = null;
            this._on(t.element, "mousemove touchmove MSPointerMove pointermove", this._swipeMoveHandler);
            this._on(t.element, "mouseup touchend MSPointerUp pointerup", this._swipeEndHandler);
            this._on(t.element, "mousecancel touchcancel MSPointerCancel pointercancel", this._swipeEndHandler);
        },

        _getMomentumTime: function (distance, time, newDistance) {
            return Math.abs(time - Math.abs(newDistance / distance * time));
        },

        _getTransitionPane: function (pane) {
            return ((this.model[pane + "Pane"].animationType == "reveal") ? this._panel.content : this._panel[pane]);
        },

        _isPaneOpen: function () {
            var returnVal = "";
            if (this.element.find("." + this._class.opened).length)
                returnVal = (this.element.children("." + this._class.opened).hasClass("e-m-sp-left") ? "left" : "right");
            return returnVal;
        },

        _isOrientationSupported: function () {
            return ("orientation" in window && "onorientationchange" in window);
        },

        _transform: function (dist, time, element, addClass) {
            if (dist == "none") return this;
            var translate = "-" + this._browser + "-transform",
                translateVal = "translateX(" + dist + "px) translateZ(0px)";
            var trans = "-" + this._browser + "-transition-property",
                transVal = "transform",
                transDur = "-" + this._browser + "-transition-duration",
                transDurVal = time + "ms";
            element.css(trans, transVal).css(transDur, transDurVal).css(translate, translateVal).css("transition-duration", transDurVal);
            if (addClass) element.addClass(addClass);
            return this;
        },

        _closeOpenedPane: function (time) {
            time = time == undefined ? 0 : time;
            if (this._isPaneOpen()) {
                var _pane = this._isPaneOpen();
                var transPane = this._getTransitionPane(_pane),
                    _proxy = this;
                this._elementOverlay.addClass(this._class.hide);
                this._trigger("beforeClose", this._getArgData(_pane));
                transPane.bind(ej.transitionEndEvent(), function () {
                    _proxy._trigger("close", _proxy._getArgData(_pane));
                    if(_proxy._panel.left.length)_proxy._panel.left.removeClass(_proxy._class.hidden);
                    if(_proxy._panel.right.length)_proxy._panel.right.removeClass(_proxy._class.hidden);
                    transPane.unbind(ej.transitionEndEvent());
                });
                this._transform(0, time, transPane).element.children("." + this._class.opened).removeClass(this._class.opened);
            }
            this._isNavPaneOpened=false;
            return this;
        },

        _resize: function () {
            this._closeOpenedPane(0)._setPaneSize();
            this.element.removeClass("e-m-lower e-m-higher").addClass(ej.isLowerResolution() ? "e-m-lower" : "e-m-higher");
        },

        _setPaneSize: function () {
            this._panel.content.css({
                "right": "",
                "left": ""
            });
            if (this._panel.left.length) {
                this._panel.left.css({
                    "width": this.model.leftPane.width,
                    "left": ""
                });

                if (!ej.isLowerResolution() * this.model.leftPane.showOnTablet) {                    
                    this._panel.left.addClass("e-m-exposed").removeClass(this._class.hidden);
                    this._panel.content.css("left", this._panel.left.outerWidth());
                } else {
                    var lwidth=((this.model.leftPane.width !="auto")? "-" + this.model.leftPane.width : -(ej.getDimension(this._panel.left, "width")));
                    if (this.model.leftPane.animationType == "overlay") this._panel.left.css("left", lwidth).addClass(this._class.hidden);
                    this._panel.left.removeClass("e-m-exposed");
                }
            }

            if (this._panel.right.length) {
                this._panel.right.css({
                    "width": this.model.rightPane.width,
                    "right": ""
                });
                if (!ej.isLowerResolution() * this.model.rightPane.showOnTablet) {
                    this._panel.right.addClass("e-m-exposed").removeClass(this._class.hidden);
                    this._panel.content.css("right", this._panel.right.outerWidth());
                } else {
                    var rwidth=((this.model.rightPane.width !="auto")? "-" + this.model.rightPane.width : -(ej.getDimension(this._panel.right, "width")));
                    if (this.model.rightPane.animationType == "overlay") this._panel.right.css("right", rwidth).addClass(this._class.hidden);
                    this._panel.right.removeClass("e-m-exposed");
                }
            }
            //setting splitpane height and width
            this.element.css({
                height: this.model.height,
                width: this.model.width
            });
            return this;
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                switch (prop) {
                case "renderMode":
                    this._setRenderMode(options.renderMode);
                    break;
                case "cssClass":
                    this.element.removeClass(this._cssClass).addClass(options.cssClass);
                    this._cssClass = options.cssClass;
                    break;
                case "isRelative":
                    this.element.removeClass("e-m-rel e-m-abs").addClass(this.model.isRelative ? "e-m-rel" : "e-m-abs");
                    break;
                case "height":
                case "width":
                    this.element.css({
                        height: this.model.height,
                        width: this.model.width
                    });
                    break;
                case "leftPane":
                case "rightPane":
                case "contentPane":
                    this["_set" + prop.charAt(0).toUpperCase() + prop.slice(1)](options[prop]);
                    break;
                case "edgeThreshold":
                    break;
                }
                if (refresh)
                    this._refresh();
            }
        },

        _setRenderMode: function (mode) {
            if (mode == "auto")
                ej.setRenderMode(this);
            this.element.removeClass("e-m-ios7 e-m-android e-m-windows e-m-flat").addClass("e-m-" + this.model.renderMode);
            this._setPaneSize();
        },

        _setLeftPane: function (options) {
            for (var prop in options) {
                switch (prop) {
                case "animationType":
                    this._panel.left.removeClass("e-m-type-overlay");
                case "width":
                case "showOnTablet":
                    this._setPaneSize();
                    break;
                case "templateId":
                    this._setTemplate(options[prop], "left");
                    break;
                }
            }
        },

        _setRightPane: function (options) {
            for (var prop in options) {
                switch (prop) {
                case "animationType":
                    this._panel.right.removeClass("e-m-type-overlay");
                case "width":
                case "showOnTablet":
                    this._setPaneSize();
                    break;
                case "templateId":
                    this._setTemplate(options[prop], "right");
                    break;
                }
            }
        },

        _setTemplate: function (id, pane) {
            this._closeOpenedPane();
            $("#" + id).removeAttr("class style");
            if (this._panel[pane].length) this._panel[pane].removeAttr("class style").addClass(this._class.hidden);
            this._initPane()._setPaneSize();
        },

        _setContentPane: function (options) {
            for (var prop in options) {
                switch (prop) {
                case "templateId":
                    this._bindSwipeEvent(true);
                    this._setTemplate(options[prop], "content");
                    this._bindSwipeEvent();
                    break;
                }
            }
        },

        _contentTransition: function (e) {
            if ($(e.target).hasClass("new"))
                $(e.target).contents().unwrap();
            if ($(e.target).hasClass("old"))
                $(e.target).remove();
        },

        _getArgData: function (panel) {
            return {
                panel: panel,
                element: this._panel[panel],
                content: this._panel.content
            };
        },

        _isCanOpen: function (pane) {
            return (this.model[pane + "Pane"].showOnTablet == true && ej.isLowerResolution() == false) ? false : true;
        },

        _refresh: function () {
            this._clearElements();
            this._renderControl();
            if (this.model.renderTemplate && ej.angular.defaultAppName) ej.angular.compile(this._layouts);
            this._wireEvents();
        },

        _clearElements: function () {
            this.element.removeAttr("class style");
            if (this._panel.left.length) this._panel.left.removeAttr("class style");
            if (this._panel.content.length) this._panel.content.removeAttr("class style");
            if (this._panel.right.length) this._panel.right.removeAttr("class style");
            if (this._elementOverlay) this._elementOverlay.remove();
            this.element.attr({
                "style": this._orgEleStyle, "class": this._orgEleClass
            }).removeClass(this._rootCSS);
        },

        _destroy: function () {
            this._wireEvents(true);
            this._unbindSwipeEvent();
            this._clearElements();
        },        

        //Public Function
        
        openLeftPane: function (time) {
            if (!this._isPaneOpen() && this._panel.left.length && this._isCanOpen("left")) {
                var transPane = this._getTransitionPane("left"),
                    _proxy = this;
                this._panel.left.removeClass(this._class.hidden);                
                this._trigger("beforeOpen", this._getArgData("left"));
                transPane.bind(ej.transitionEndEvent(), function () {
                    _proxy._trigger("open", _proxy._getArgData("left"));
                    transPane.unbind(ej.transitionEndEvent());
                });                
                this._transform(this._panel.left.outerWidth(), (time ? time : this._lModel.defaultTransitionTime), transPane);
                opacity = this.model.leftPane.animationType != "overlay" ? 0 : this._lModel.overlayOpacity;
                this._elementOverlay.css("opacity", opacity).removeClass(this._class.hide);
                this._panel.left.addClass(this._class.opened);
                this._isNavPaneOpened=true;
                return this;
            }
        },

        openRightPane: function (time) {
            if (!this._isPaneOpen() && this._panel.right.length && this._isCanOpen("right")) {
                var transPane = this._getTransitionPane("right"),
                    _proxy = this;
                this._panel.right.removeClass(this._class.hidden);                
                this._trigger("beforeOpen", this._getArgData("right"));
                transPane.bind(ej.transitionEndEvent(), function () {
                    _proxy._trigger("open", _proxy._getArgData("right"));
                    transPane.unbind(ej.transitionEndEvent());
                });
                this._transform(-this._panel.right.outerWidth(), (time ? time : this._lModel.defaultTransitionTime), transPane);
                opacity = this.model.rightPane.animationType != "overlay" ? 0 : this._lModel.overlayOpacity;
                this._elementOverlay.css("opacity", opacity).removeClass(this._class.hide);
                this._panel.right.addClass(this._class.opened);
                return this;
            }
        },

        closePane: function (time) {
            this._closeOpenedPane((time ? time : this._lModel.defaultTransitionTime));
        }
        
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Tab elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmTab", "ej.mobile.Tab", {
        _setFirst: true,
        _rootCSS: "e-m-tab",
        validTags: ["ul"],
        _tags: [{
            tag: "items",
            attr: ["icon", "text", "href", "enableAjax", "imageClass", "badge.value", "badge.maxValue"],
            content: "template"
        }],
        defaults: { 
            renderMode: "auto",
            enableAjax: false,
            enableCache: false,
            showAjaxPopup: false,
            selectedItemIndex: 0,
            enablePersistence: false,
            badge: {
                maxValue: "100"
            },
            itemStyle: "bothblock",
            allowSwiping: true,
            position: "bottom",
            prefetchAjaxContent: false,
            cssClass: "",
            items: [],
            contentId: null,
            ajaxSettings: {
                type: 'GET',
                cache: false,
                async: true,
                dataType: "html",
                contentType: "html",
                url: "",
                data: {}
            },
            enableRippleEffect: ej.isAndroid() ? true : false,
            load: null,
            loadComplete: null,
            touchStart: null,
            touchEnd: null,
            ajaxBeforeLoad: null,
            ajaxSuccess: null,
            ajaxError: null,
            ajaxComplete: null,
            select: null

        },
        observables: ["selectedItemIndex"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        dataTypes: {
            renderMode: "enum",
            enablePersistence: "boolean",
            items: "data",
            enableRippleEffect: "boolean"
        },
        _init: function () {
            this._orgEle = this.element.clone();
            this._storedContent = [];
            this._items = [];
            this._renderControl();
            this._wireEvents(false);
        },
        _renderControl: function () {
            ej.setRenderMode(this);
            this.element.addClass("e-m-user-select e-m-" + this.model.renderMode + " e-m-tab-" + this.model.position + " e-m-tab-content-" + this.model.itemStyle + " " + this.model.cssClass);
            this.elementId = this.element[0].id;
            this._tabContainer = $("#" + this.model.contentId).addClass("e-m-" + this.model.renderMode + " e-m-tab-" + this.model.position + " e-m-tab-content-wrapper");
            this.model.items = typeof this.model.items == "string" ? eval(this.model.items) : this.model.items;
            if (this.model.itemStyle == "bothblock")
                this._tabContainer.addClass("e-m-tab-content-bothblock");
            if (this.model.load)
                this._trigger("load");
            if (!this.model.items.length) {
                this._tabItems = this.element.find("li");
                for (var i = 0, itemLength = this._tabItems.length; i < itemLength; i++) {
                    this._items.push(this._insertItemValue($(this._tabItems[i])));
                    this._renderTabItems(this._tabItems[i], i);
                }
            }
            else {
                for (var i = 0, itemLength = this.model.items.length; i < itemLength; i++) {
                    var liItem = ej.buildTag('li', this.model.items[i].template);
                    this.element.append(liItem);
                    this._renderTabItems(liItem, i);
                }
                this._tabItems = this.element.find("li");
            }
            if (!this.model.prefetchAjaxContent && this.selectedItemIndex() >= 0)
                this._showHideItems(this._tabItems[this.selectedItemIndex()], this._tabContainer, this.selectedItemIndex());
            if (this.model.loadComplete)
                this._trigger("loadComplete");
                this._setEnableRippleEffect();
        },

        _setEnableRippleEffect: function () {
            this.element.find("li")[(this.model.enableRippleEffect ? "addClass" : "removeClass")]("e-ripple");
        },
        _insertItemValue: function (element) {
            var item = {};
            for (var j = 0, attrLength = this._tags[0].attr.length + 1; j < attrLength; j++) {
                var attr = this._tags[0].attr[j] == undefined ? "template" : this._tags[0].attr[j];
                item[attr] = (attr == "enableAjax") ? ej.getBooleanVal(element, 'data-ej-' + attr.replace(".", "-").toLowerCase(), undefined) : (attr == "template") ? element.html() : ej.getAttrVal(element, 'data-ej-' + attr.replace(".", "-").toLowerCase(), undefined);
            }
            return item;
        },
        _renderTabItems: function (element, index) {
            this._items = this.model.items.length > 0 ? this.model.items : this._items;
            var element = $(element);
            element.attr("id", this.elementId + "-tab-item" + index);
            if (!ej.isNullOrUndefined(this._items[index].icon))
                element.addClass("e-m-icon-" + this._items[index].icon.toLowerCase());
            if ((this._items[index]["badge.value"] || this.model.badge.value) > 0) {
                var val = this._items[index]["badge.value"] ? this._items[index]["badge.value"] : null,
                 maxVal = this._items[index]["badge.maxValue"] ? this._items[index]["badge.maxValue"] : this.model.badge.maxValue;
                element.addClass("e-m-tab-badge").attr("badgeValue", this._setBadgeValue(parseInt(val), parseInt(maxVal)));
            }
            this._renderItems(element, this._tabContainer, index);
            if (!ej.isNullOrUndefined(this._items[index].text))
                element.text(this._items[index].text)
        },
        _ajaxSuccessHandler: function () {
            if (this.model.prefetchAjaxContent)
                this._showHideItems(this._tabItems[this.selectedItemIndex()], this._tabContainer, this.selectedItemIndex());
        },
        _setBadgeValue: function (val, maxVal) {
            return (val <= maxVal ? val : maxVal + "+").toString();
        },
        _createDelegates: function () {
            this._touchStartDelegate = $.proxy(this._onTouchStartHandler, this);
            this._touchEndDelegate = $.proxy(this._onTouchEndHandler, this);
            this._touchMoveDelegate = $.proxy(this._onTouchMoveHandler, this);
            this._contentTouchStartDelegate = $.proxy(this._onContentTouchStartHandler, this);
            this._contentTouchMoveDelegate = $.proxy(this._onContentTouchMoveHandler, this);
            this._contentTouchEndDelegate = $.proxy(this._onContentTouchEndHandler, this);
        },
        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([this._tabItems], [ej.startEvent()], [this._touchStartDelegate], remove);
            if (this.model.allowSwiping)
                ej.listenEvents([this._tabContainer], [ej.startEvent()], [this._contentTouchStartDelegate], remove)
        },
        _eventArgs: function (currentTab) {
            return { item: $(currentTab), text: $(currentTab).text(), index: $(currentTab).index() };
        },
        _onTouchStartHandler: function (evt) {
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            this._tabClickX = point.clientX;
            this._currentTab = evt.target;
            this._targetItemId = evt.target.id;
            this._tabItems = this.element.find(".e-m-tab-item");
            this._index = parseInt(this._targetItemId.split(this.elementId + "-tab-item")[1]);
            if (this.model.touchStart) this._trigger("touchStart", this._eventArgs(this._currentTab));
            this._tabMouseMove = false;
            ej.listenEvents([this._tabItems, this._tabItems], [ej.moveEvent(), ej.endEvent()], [this._touchMoveDelegate, this._touchEndDelegate], false);
        },

        _onTouchMoveHandler: function (evt) {
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            if (this._tabClickX != point.clientX) this._tabMouseMove = true;
        },
        _onTouchEndHandler: function (evt) {
            this._isInteraction = true;
            ej.listenEvents([this._tabItems, this._tabItems], [ej.moveEvent(), ej.endEvent()], [this._touchMoveDelegate, this._touchEndDelegate], true);
            if (this._tabMouseMove) return;
            this.selectedItemIndex(this._index);
            this._showHideItems(this._currentTab, this._tabContainer, this._index);
            if (this.model.touchEnd)
                this._trigger("touchEnd", this._eventArgs(this._currentTab));
            this._isInteraction = false;
        },


        _onContentTouchStartHandler: function (evt) {
            var point = evt.touches ? evt.touches[0] : evt;
            this._pointX = point.pageX;
            this._pointY = point.pageY;
            ej.listenEvents([this._tabContainer], [ej.moveEvent()], [this._contentTouchMoveDelegate], false);
        },
        _onContentTouchMoveHandler: function (evt) {
            ej.listenEvents([this._tabContainer], [ej.moveEvent()], [this._contentTouchMoveDelegate], true);
            ej.listenEvents([this._tabContainer], [ej.endEvent()], [this._contentTouchEndDelegate], false);
        },
        _onContentTouchEndHandler: function (evt) {
            var point = evt.touches ? (evt.touches[0] ? evt.touches[0] : (evt.changedTouches ? evt.changedTouches[0] : evt)) : evt;
            ej.listenEvents([this._tabContainer, this._tabContainer], [ej.moveEvent(), ej.endEvent()], [this._contentTouchMoveDelegate, this._contentTouchEndDelegate], true);
            this._endPointX = point.pageX;
            this._endPointY = point.pageY;
            relativeX = Math.abs(this._endPointX - this._pointX);
            relativeY = Math.abs(this._endPointY - this._pointY);
            relativeXY = Math.abs(relativeX - relativeY);
            if (relativeX > 30 && relativeXY > 30 && relativeX > relativeY) {
                direction = (this._endPointX - this._pointX) > 30 ? "swiperight" : "swipeleft";
                var index = direction == "swiperight" ? this.selectedItemIndex() - 1 : this.selectedItemIndex() + 1;
                if (index >= 0 && index < this._items.length) {
                    var element = this.element.find("#" + this.elementId + "-tab-item" + index);
                    this.selectedItemIndex(index);
                    this._showHideItems(element[0], this._tabContainer, index);
                }
            }
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-m-tab");
            this._renderControl();
            this._wireEvents();
        },
        _clearElement: function () {
            this.element.removeAttr("class").find("*").removeAttr("class id");
            $.each(this._storedContent, function (index, element) {
                ej.getCurrentPage().append(element);
            });
            this._tabContainer.removeAttr("class").find("*").remove();
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
        //api setmodel function
        _setContentId: function (value) {
            $.each(this._storedContent, function (index, element) {
                ej.getCurrentPage().append(element);
            });
            if (this.model.allowSwiping)
                ej.listenEvents([this._tabContainer], [ej.startEvent()], [this._contentTouchStartDelegate], true);
            this._tabContainer.removeAttr("class").find("*").remove();
            this._tabContainer = $("#" + value).addClass("e-m-" + this.model.renderMode + " e-m-tab-" + this.model.position + " e-m-tab-content-wrapper");
            for (var i = 0, itemLength = this._tabItems.length; i < itemLength; i++) {
                this._renderItems(this._tabItems[i], this._tabContainer, i);
            }
            if (!this.model.prefetchAjaxContent && this.selectedItemIndex() >= 0)
                this._showHideItems(this._tabItems[this.selectedItemIndex()], this._tabContainer, this.selectedItemIndex());
            if (this.model.allowSwiping)
                ej.listenEvents([this._tabContainer], [ej.startEvent()], [this._contentTouchStartDelegate], false);
        },
        _addRemovClass: function (classes, value) {
            this.element.removeClass(classes).addClass("e-m-" + value);
            this._tabContainer.removeClass(classes).addClass("e-m-" + value);
        }, 
        _setRenderMode: function (value) {
            this._addRemovClass("e-m-ios7 e-m-android e-m-windows e-m-flat", value);
        },
        _setPosition: function (value) {
            this._addRemovClass("e-m-tab-top e-m-tab-bottom", value);
        },
        _setItemStyle: function (value) {
            this.element.removeClass("e-m-tab-content-bothblock e-m-tab-content-bothinline e-m-tab-content-image e-m-tab-content-text").addClass("e-m-" + value);
        },
        _setSelectedItemIndex: function (value) {
            if (this.selectedItemIndex() >= 0 && this.selectedItemIndex() != "")
                this._showHideItems(this._tabItems[parseInt(this.selectedItemIndex())], this._tabContainer, parseInt(this.selectedItemIndex()));
        },
        _setCssClass: function (value) {
            this.element.addClass(value)
        },
        //public method
        showBadge: function (index) {
            var element = this.element.find("#" + this.elementId + "-tab-item" + index);
            if (index >= 0) {
                element.addClass("e-m-tab-badge");
            }
        },
        hideBadge: function (index) {
            var element = this.element.find("#" + this.elementId + "-tab-item" + index);
            if (index >= 0) {
                element.removeClass("e-m-tab-badge");
            }
        },
        setBadgeValue: function (index, val) {
            var element = this.element.find("#" + this.elementId + "-tab-item" + index);
            if (index >= 0 && val != undefined) {
                var maxVal = this._items[index]["badge.maxValue"] ? this._items[index]["badge.maxValue"] : this.model.badge.maxValue;
                element.addClass("e-m-tab-badge").attr("badgeValue", this._setBadgeValue(parseInt(val), parseInt(maxVal)));
                this._items[index]["badge.value"] = val;
            }
        },
        setActiveItem: function (index) {
            if (index >= 0) {
                this.selectedItemIndex(index);
                this._showHideItems(this._tabItems[index], this._tabContainer, index)
            }
        },
        enableItem: function (index) {
            if (index >= 0) {
                var element = this.element.find("#" + this.elementId + "-tab-item" + index);
                element.removeClass('e-m-state-disabled');
                this.enableContent(index);
            }
        },
        disableItem: function (index) {
            if (index >= 0) {
                var element = this.element.find("#" + this.elementId + "-tab-item" + index);
                element.addClass('e-m-state-disabled');
                this.disableContent(index);
            }
        },
        enableContent: function (index) {
            var ele = $(this._tabContainer.find("#" + this.elementId + "-tab-item" + index + "-content"));
            if (index >= 0)
                ele.removeClass('e-m-state-disabled');
            else
                this._tabContainer.removeClass('e-m-state-disabled');
        },
        disableContent: function (index) {
            var ele = $(this._tabContainer.find("#" + this.elementId + "-tab-item" + index + "-content"));
            if (index >= 0)
                ele.addClass('e-m-state-disabled');
            else
                this._tabContainer.addClass('e-m-state-disabled');
        },
        addItem: function (tab, index) {
            if (index >= 0) {
                this._items.splice(index, 0, this._insertItemValue($(tab)));
                this.element.find("#" + this.elementId + "-tab-item" + index).before($(tab).addClass('e-m-tab-item'));
            }
            else {
                this._items.push(this._insertItemValue($(tab).addClass('e-m-tab-item')));
                this.element.find("#" + this.elementId + "-tab-item" + this._items.length).after($(tab))
            }
            $.each(this._storedContent, function (index, element) {
                ej.getCurrentPage().append(element);
            });
            this._storedContent = [];
            this._tabContainer.find("*").remove();
            for (var i = 0, itemLength = this.element.find('.e-m-tab-item').length; i < itemLength; i++) {
                var li = $(this.element.find('.e-m-tab-item')[i]);
                this._renderTabItems(li, i);
            }
            if (!this.model.prefetchAjaxContent && this.selectedItemIndex() >= 0)
                this._showHideItems(this._tabItems[this.selectedItemIndex()], this._tabContainer, this.selectedItemIndex());
            this._tabItems = this.element.find("li");
            this._wireEvents(false);
        },
        removeItem: function (index) {
            if (index >= 0) {
                this.element.find("#" + this.elementId + "-tab-item" + index).remove();
                this._tabContainer.find("#" + this.elementId + "-tab-item" + index + "-content").remove();
                this._items.splice(index, 1);
                for (var i = 0, itemLength = this.element.find('.e-m-tab-item').length; i < itemLength; i++) {
                    $(this.element.find('.e-m-tab-item')[i]).attr("id", this.elementId + "-tab-item" + i);
                    $(this._tabContainer.find(".e-m-tab-content")[i]).attr("id", this.elementId + "-tab-item" + i + "-content");
                }
            }
        },
        getItemsCount: function () {
            return this.element.find('.e-m-tab-item').length;
        },
        getActiveItem: function () {
            return this.element.find('li.e-m-state-active');
        },
        getActiveItemText: function () {
            return this.element.find('li.e-m-state-active').text();
        },
        setContent: function (url, index) {
            if (index >= 0)
                this._items[index].href = url;
            this._refresh();
        }

    });
    ej.mobile.Tab.ItemStyle = {
        BothBlock: "bothblock",
        Image: "image",
        Text: "text",
        BothInline: "bothinline"
    };
    ej.mobile.Tab.Position = {
        Top: "top",
        Bottom: "bottom"
    };
    $.extend(true, ej.mobile.Tab.prototype, ej.mobile.AjaxLoader);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TileView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejTileBase", "ej.TileBase", {
        defaults: {
            badge: {
                enabled: false,
                value: 1,
                text: null,
                maxValue: 100,
                minValue: 1,
                position: "bottomright"
            },

            caption: {
                enabled: true,
                text: "text",
                icon: null,
                position: "innerbottom",
                alignment: "normal"
            },

            enablePersistence: false,

            imageClass: null,

            imagePosition: "center",

            imageTemplateId: null,

            imageUrl: null,

            liveTile: {

                text: null,

                enabled: false,

                imageClass: null,

                imageTemplateId: null,

                imageUrl: null,

                type: "flip",

                updateInterval: 2000
            },

            cssClass: "",

            tileSize: "small",

            width: null,

            height: null,

            showRoundedCorner: false,

            backgroundColor: null,

            allowSelection: false,

           locale: "en-US",

            //deprecated
            showText: true,

            text: "text",

            textAlignment: "normal",

        },
        dataTypes: {
            imagePosition: "enum",
            liveTile: {
                text: "array",
                type: "enum",
                enabled: "boolean",
                updateInterval: "number",
                imageUrl: "array",
                imageClass: "array",
                imageTemplateId: "array"
            },
            alignment: "enum",
            tileSize: "enum",
            locale: "string"
        },

        observables: ["badge.value", "badge.enabled", "badge.text", "badge.position", "text", "caption.text"],
        _badgeValue: ej.util.valueFunction("badge.value"),
        _badgeEnabled: ej.util.valueFunction("badge.enabled"),
        _badgeText: ej.util.valueFunction("badge.text"),
        _badgePosition: ej.util.valueFunction("badge.position"),
        _captionText: ej.util.valueFunction("caption.text"),
        text: ej.util.valueFunction("text"),

        _setDeprecatedProperties: function () {
            this.model["caption"].enabled = (this.defaults["caption"].enabled == this.model["caption"].enabled && this.defaults.showText == this.model.showText) ? this.defaults["caption"].enabled : this.model["caption"].enabled != this.defaults["caption"].enabled ? this.model["caption"].enabled : this.model.showText;
            this._captionText((this.defaults["caption"].text == this._captionText() && this.defaults.text == this.text()) ? this.defaults["caption"].text : this._captionText() != this.defaults["caption"].text ? this._captionText() : this.text());
            this.model["caption"].alignment = (this.defaults["caption"].alignment == this.model["caption"].alignment && this.defaults.textAlignment == this.model.textAlignment) ? this.defaults["caption"].alignment : this.model["caption"].alignment != this.defaults["caption"].alignment ? this.model["caption"].alignment : this.model.textAlignment;
        },
        _tileRender: function () {
            this._setDeprecatedProperties();
            this._imagePosition = this.model.imagePosition;
            this._cssClass = this.model.cssClass;
            this.element.addClass(this.model.cssClass + " " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.tileSize);
            if (this.model.showRoundedCorner)
                this.element.addClass(this._prefix + "tile-round-corner");
            if (this._isLiveTile())
                this._liveTile();
            else {
                this._image = ej.buildTag("div").addClass(this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "image-parent");
                this._innerImage = ej.buildTag("span").addClass(this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
                if (this.model.imageTemplateId) {
                    var imageTemp = this._isMobile() ? ej.getCurrentPage().find('#' + this.model.imageTemplateId) : $('#' + this.model.imageTemplateId);
                    this._innerImage.addClass(this._prefix + "tile-template").append(imageTemp);
                }
                else if (this.model.imageClass) 
                    this._innerImage.addClass(this.model.imageClass);
                else if (this.model.imageUrl) {
                    var url = this._isMobile() && this.model.imagePath ? (this._getAbsolutePath(this.model.imagePath) + "/" + this.model.renderMode.toLowerCase() + "/" + this.model.imageUrl) : this._getAbsolutePath(this.model.imageUrl);
                    this._innerImage.css({ "background-image": "url('" + url + "')"});
                    if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && this.model.imagePosition == "fill")
                        this._innerImage.css({ "filter": "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + url + "',sizingMethod='scale')", "-ms-filter": "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + url + "',sizingMethod='scale')" });
                }
                this._image.append(this._innerImage);
                this.element.append(this._image);
                this._setBackgroundColor(this.model.backgroundColor);
                if (this.model.imageTemplateId && this._prefix == "e-m-" && ej.angular.defaultAppName)
                    ej.angular.compile(this._image);
            }
            if (this._isCaptionEnable()) {
                if (this.model["liveTile"].text && this._isLiveTile()) {
                    var liveTileImage = this.element.find("." + this._prefix + "image-parent");
                    for (var i = 0, tileLength = liveTileImage.length; i < tileLength; i++) {
                        this._setCaptionClass($(liveTileImage[i]));
                        $(liveTileImage[i]).addClass(this._prefix + "tile-caption-text ").attr("text", this.model["liveTile"].text[i]);
                    }
                }
                else {
                    this._setCaptionClass(this.element);
                    if (this.model["caption"].icon) {
                        this._captionIcon = this.model["caption"].icon;
                        this.element.addClass(this._prefix + "tile-caption-icon " + this._prefix + "icon-" + this._captionIcon);
                    }
                    else {
                        this.element.addClass(this._prefix + "tile-caption-text ").attr("text", this._captionText());
                    }

                }
            }
            if (this._badgeEnabled())
                this._setBadgeEnabled();
            this._selectElement = ej.buildTag("span." + this._prefix + "tile-overlay");
            this.element.append(this._selectElement);
            if(!this.model.height) this.model.height = this.model["caption"].position == "outer" ? this.element.height() - 50 : this.element.height();
            if(!this.model.width) this.model.width = this.element.width();
            if (this._isCustomizeSize())
                this._setCustomizSize();
        },

        _liveTile: function () {
            this.element.addClass(this._prefix + "livetile-enable"); var image;
            if (this.model["liveTile"].imageTemplateId) {
                for (var i = 0; i < this.model["liveTile"].imageTemplateId.length; i++) {
                    var imageTempEle = ej.buildTag("div").addClass(this._prefix + "tile-template " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "image-parent " + this._prefix + "tile-" + this.model["liveTile"].type);
                    var template = this._isMobile() ? ej.getCurrentPage().find('#' + this.model["liveTile"].imageTemplateId[i]) : $('#' + this.model["liveTile"].imageTemplateId[i]);
                    this.element.append(imageTempEle.append(template));
                }
            }
            else if (this.model["liveTile"].imageClass) {
                for (var i = 0; i < this.model["liveTile"].imageClass.length; i++) {
                    image = ej.buildTag("span").addClass(this.model["liveTile"].imageClass[i] + " " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
                    var imageClassEle = ej.buildTag("div").addClass( this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model["liveTile"].type + " " + this._prefix + "image-parent");
                    image.appendTo(imageClassEle);
                    this.element.append(imageClassEle);
                }
            }
            else {
                for (var i = 0; i < this.model["liveTile"].imageUrl.length; i++) {
                    var url = this._isMobile() && this.model.imagePath ? this._getAbsolutePath(this.model.imagePath) + "/windows/" + this.model["liveTile"].imageUrl[i] : this._getAbsolutePath(this.model["liveTile"].imageUrl[i]);
                    image = ej.buildTag("span").addClass(this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition).css({ "background-image": "url('" + url + "')" });
                    var imageUrlEle = ej.buildTag("div." + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model["liveTile"].type + " " + this._prefix + "image-parent");
                    image.appendTo(imageUrlEle);
                    this.element.append(imageUrlEle);
                }
            }
            this._setBackgroundColor(this.model.backgroundColor);
            $(this.element.children()[0]).addClass(this._prefix + "tile-" + this.model["liveTile"].type + 'back').removeClass(this._prefix + "tile-" + this.model["liveTile"].type);
            if (this.model.imageTemplateId && this._prefix == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                ej.angular.compile(this.element);
        },
        _setCustomizSize: function () {
            var minWidth = (this._isLiveTileMode() || this.model.renderMode == "flat") && (parseInt(this.model.width) <= 70) ? "70" :
                (this._isMobile() && this.model.renderMode == "android") && this._isBelowSmallSize() ? "85" :
                (this._isMobile() && this.model.renderMode == "ios7") && this._isBelowSmallSize() ? "74" :
                 this.model.width;
            var minHeight = (this._isLiveTileMode() || this.model.renderMode == "flat") && (parseInt(this.model.height) <= 70) ? "70" :
                (this._isMobile() && (this.model.renderMode == "android" || this.model.renderMode == "ios7")) && this._isBelowSmallSize() ? "70" :
                (this._isMobile() && this.model.renderMode == "ios7") && this._isBelowSmallSize() ? "70" : this.model.height;
            var extraSpace = (this._isLiveTileMode() || this.model.renderMode == "flat") && !this._isBelowSmallSize() ? 50 :
                 (this._isLiveTileMode() || this.model.renderMode == "flat") && this._isBelowSmallSize() ? 0 :
                (this._isMobile() && this.model.renderMode == "android") && this._isBelowSmallSize() ? 30 : 30;
            var newHeight = (this.model["caption"].enabled && this.model["caption"].position == "outer" && this.model.height) ? minHeight ? parseInt(minHeight) + extraSpace : parseInt(minHeight) + extraSpace : this.model.height;
            this.element.css({ "width": this.model.width ? this.model.width : "70", "height": this.model.height ? newHeight : this.element.height() + extraSpace });
            this.element.find("." + this._prefix + "image-parent").css({"width": this.element.css("width"), "height": this.model.height ? minHeight : this.element.height() - extraSpace });
            if (!this._isBelowSmallSize() && this.model["caption"].enabled && this.model["caption"].position == "outer") {
                this.element.find(".e-m-tile-image").css({ "width": this.model.width ? minWidth : "", "height": this.model.height ? minHeight : "" });
                this._selectElement.css({ "width": this.model.width ? minWidth : "", "height": this.model.height ? minHeight : "" });
            }
            if (this._isBelowSmallSize()) {
                this.element.addClass(this._prefix + "tile-small");
                if (!(this._isMobile() && (this.model.renderMode == "android" || this.model.renderMode == "ios7") && this.model["caption"].position == "outer"))
                    this.element.removeAttr("text");
            }
        },
        _isLiveTile: function () {
            return (this._isLiveTileMode() && this.model["liveTile"].enabled);
        },

        _isLiveTileMode: function () {
            return (!this._isMobile() || this.model.renderMode == "windows");
        },

        _setCaptionClass: function (element) {
            element.addClass(this._prefix + "tile-caption " + this._prefix + "caption-" + this.model["caption"].position + " " + this._prefix + "caption-align-" + this.model["caption"].alignment);
        },

        _isCaptionEnable: function () {
            return (this.model["caption"].enabled && !(this.model.tileSize == "small" || (this._isBelowSmallSize() && this.model.tileSize == "small")) || ((this._isMobile() && (this.model.renderMode == "ios7" || this.model.renderMode == "android") && this.model["caption"].position == "outer") || this._isMobile() && (this.model.renderMode == "android" || this.model.renderMode == "ios7") && this.model["caption"].position == "outer" && (this._isCustomizeSize() && this._isBelowSmallSize())));
        },

        _isBelowSmallSize: function () {
            var width = this.model.width ? parseInt(this.model.width) : this.model.width;
            var height = this.model.height ? parseInt(this.model.height) : this.model.height;
            var minWidthandHeight = (!this._isMobile() || (this._isMobile() && (this.model.renderMode == "windows" || this.model.renderMode == "flat"))) ? 70 : (this._isMobile() && this.model.renderMode == "android") ? 85 : 74;
            return ((width <= minWidthandHeight && width != null) || (height <= minWidthandHeight && height != null));
        },

        _isCustomizeSize: function () {
            return this.model.height || this.model.width;
        },

        _createDelegates: function () {
            this._onStartDelegate = $.proxy(this._isMobile() ? this._onTouchStartHandler : this._onMouseDownHandler, this);
            this._onEndDelegate = $.proxy(this._isMobile() ? this._onTouchEndHandler : this._onMouseUpHandler, this);
            this._onMoveDelegate = $.proxy(this._isMobile() ? this._onTouchMoveHandler : this._onMouseMoveHandler, this);
            this._onDocClickDelegate = $.proxy(this._onDocClickHandler, this);
            this._startAnimationDelegate = $.proxy(this["_" + this.model["liveTile"].type + "Tile"], this);
        },

        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([this.element, $(document)], [ej.startEvent(), ej.endEvent()], [this._onStartDelegate, this._onDocClickDelegate], remove, this);
            if (this._isLiveTile())
                remove ? this._stopTileAnimation() : this._startTileAnimation();
        },

        _onDocClickHandler: function (evt) {
            this._removeActiveClass();
        },

        _removeActiveClass: function () {
            this._isLiveTileMode() ? ej._removeSkewClass(this.element) : $(this.element).removeClass(this._prefix + "state-active");
        },

        _startTileAnimation: function () {
            this._intervalCounter = setInterval(this._startAnimationDelegate, this.model["liveTile"].updateInterval);
        },

        _stopTileAnimation: function () {
            clearInterval(this._intervalCounter);
        },

        _flipTile: function () {
            var front = this.element.find("." + this._prefix + "tile-flipback");
            var back = this.element.find("." + this._prefix + "tile-flip").first();
            front.addClass(this._prefix + "tile-flip").removeClass(this._prefix + "tile-flipback");
            back.addClass(this._prefix + "tile-flipback").removeClass(this._prefix + "tile-flip");
        },

        _slideTile: function () {
            this._animateEffect();
        },

        _carouselTile: function () {
            this._animateEffect();
        },

        _animateEffect: function () {
            var type = this.model["liveTile"].type.toLowerCase();
            var items = this.element.find("." + this._prefix + "image-parent");
            var currentitem = this.element.find("." + this._prefix + "tile-" + type + "back");
            var nextitem = currentitem.next()[0] == null ? items.first() : currentitem.next();
            var previtem = currentitem.prev()[0] == null ? items.last() : currentitem.prev();
            if ($(currentitem.next()[0]).hasClass(this._prefix + "tile-overlay") || $(currentitem.prev()[0]).hasClass(this._prefix + "tile-overlay")) {
                var currentItem = $(currentitem.next()[0]).next();
                var previousItem = $(currentitem.next()[0]).prev();
                nextitem = currentItem.next()[0] == null ? items.first() : currentItem.next();
                previtem = previousItem.prev()[0] == null ? items.last() : previousItem.prev();
            }
            previtem.addClass(this._prefix + "tile-" + type).removeClass(this._prefix + "tile-" + type + "up");
            currentitem.addClass(this._prefix + "tile-" + type + "up").removeClass(this._prefix + "tile-" + type + "back");
            nextitem.removeClass(this._prefix + "tile-" + type).addClass(this._prefix + "tile-" + type + "back");
        },

        _getAbsolutePath: function (path) {
            return this._isMobile() ? this._makeUrlAbsolute(path).toString() : path;
        },

        _makeUrlAbsolute: function (relUrl) {
            var link = $("<a href ='" + relUrl + "'></a>")[0];
            $("body").append(link);
            var pathname;
            if (link.pathname.indexOf('/') != 0)
                pathname = '/' + link.pathname;
            else
                pathname = link.pathname;
            var browser = ej.browserInfo();
            var isIE9 = (browser.name == "msie" && browser.version == "9.0");
            var newLink = (link.protocol + "//" + ((isIE9 && link.port == "80") ? (link.host).replace(":80", "") : link.host) + pathname + link.search + link.hash);
            $(link).remove();
            return newLink;
        },

        _isMobile: function () {
            return this._prefix == "e-m-" ? true : false;
        },
        /*Public Method*/
        updateTemplate: function (id, index) {
            if (this._isLiveTile()) {
                $(this.element.children()[index]).empty();
                var tempLive = (this._isMobile() ? ej.getCurrentPage().find('#' + id) : $('#' + id));
                $(this.element.children()[index]).append(tempLive);
            }
            else {
                var template = this._isMobile() ? ej.getCurrentPage().find('#' + id) : $('#' + id);
                this._image.addClass(this._prefix + "tile-template").empty().append(template);
            }
        },

        _setModel: function (options) {
            var refresh = false;
            for (var property in options) {
                var setModel = "_set" + property.charAt(0).toUpperCase() + property.slice(1);
                if (this[setModel] || property == "locale") {
                    switch (property) {
                        case "text":
                            this[setModel](ej.util.getVal(options[property]));
                            options[property] = this.text(ej.util.getVal(options[property]));
							if (ej.isNullOrUndefined(this._options)) this._options = {};
							this._options["caption"] = this.model.caption;
                            break;
						case "locale":
							if (ej.Tile.Locale[options[property]]) {
								this.model.locale = options[property];
								this._setCulture();
								this._setCaptionText(this._captionText());
							}
                            break;
                        default:
                            this[setModel](options[property]);
                    }
                }
                else
                    refresh = true
            }
            if (refresh)
                this._refresh();
        },
        /*Setmodel method*/
        _setCaption: function (options) {
            var refresh = false;
            for (var property in options) {
                var setModel = "_setCaption" + property.charAt(0).toUpperCase() + property.slice(1);
                if (this[setModel]) {
                    switch (property) {
                        case "text":
                            this[setModel](ej.util.getVal(options[property]));
                            options[property] = this._captionText(ej.util.getVal(options[property]));
                            break;
                        default:
                            this[setModel](options[property]);
                    }
                }
                else
                    refresh = true
            }
            if (refresh)
                this._refresh();
        },

        _setText: function (value) {
			this._captionText(value);
            this._setCaptionText(value)
        },
        _setCaptionText: function (value) {
            (this._isCaptionEnable() ? this.element.attr("text", value) : this.element.removeAttr("text"));
        },
        _setCaptionEnabled: function (value) {
            if (this._isCaptionEnable())
                this.element.attr("text", this._captionText())
            else
                this.element.removeAttr("text")
        },
        _setCaptionIcon: function (value) {
            if (this._isCaptionEnable()) {
                this.element.removeClass(this._prefix + "caption-text " + this._prefix + "icon-" + this._captionIcon).addClass(this._prefix + "caption-icon " + this._prefix + "icon-" + value).removeAttr("text");
                this._captionIcon = value;
            }
        },
        _setCaptionAlignment: function (value) {
            if (this._isCaptionEnable())
                this.element.removeClass(this._prefix + "caption-align-normal " + this._prefix + "caption-align-left " + this._prefix + "caption-align-center " + this._prefix + "caption-align-right").addClass(this._prefix + "caption-align-" + value)
        },
        _setShowRoundedCorner: function (value) {
            (value ? this.element.addClass(this._prefix + "tile-round-corner") : this.element.removeClass(this._prefix + "tile-round-corner"));
        },
        _setCssClass: function (value) {
            this.element.removeClass(this._cssClass).addClass(value);
            this._cssClass = value;
        },
        _setBackgroundColor: function (value) {
            ((!this._isMobile() || (this._isMobile() && this.model.renderMode == "windows")) || (this.model["caption"].position == "outer") ? this.element.find("." + this._prefix + "tile-image").css("background-color", value) : this.element.css("background-color", value));
        },
        _setTileSize: function (value) {
            if (!this._isCaptionEnable())
                this.element.removeAttr("text");
            else if (this._isCaptionEnable())
                this.element.attr("text", this._captionText());
            this.element.css({ "width": "", "height": "" });
            this.element.find("." + this._prefix + "image-parent").css({ "width": "", "height": "" });
            this.element.removeClass(this._prefix + "tile-small " + this._prefix + "tile-medium " + this._prefix + "tile-wide " + this._prefix + "tile-large").addClass(this._prefix + "tile-" + value);
            this.model.height = this.element.height();
            this.model.width = this.element.width();
        },
        _setImagePosition: function (value) {
            this.element.removeClass(this._prefix + "tile-image" + this._imagePosition).addClass(this._prefix + "tile-image" + value);
            $(this.element.find("." + this._prefix + "tile-image")).removeClass(this._prefix + "tile-image" + this._imagePosition).addClass(this._prefix + "tile-image" + value);
            this._imagePosition = this.model.imagePosition;
        },
        _setBadge: function (options) {
            for (var property in options) {
                var setModel = "_setBadge" + property.charAt(0).toUpperCase() + property.slice(1);
                if (this[setModel]) {
                    switch (property) {
                        case "value":
                        case "enabled":
                        case "text":
                            this[setModel](ej.util.getVal(options[property]));
                            if (typeof options[property] == "function")
                                options[property](this["_badge" + property.charAt(0).toUpperCase() + property.slice(1)]());
                            else
                                options[property] = this["_badge" + property.charAt(0).toUpperCase() + property.slice(1)]();
                            break;
                        default:
                            this[setModel](options[property]);
                    }
                }
            }
        },

        _setBadgeValue: function (value) {
            if (this._badgeEnabled()) {
                value = ej.isNullOrUndefined(value) ? this._badgeValue() : value;
                var badgeValue;
                if (this._badgeText())
                    badgeValue = this._badgeText();
                else {
                    if (value <= this.model["badge"].maxValue && value >= this.model["badge"].minValue) {
                        badgeValue = value.toString();
                        this._badgeValue(value);
                    }
                    else if (value > this.model["badge"].maxValue) {
                        badgeValue = this.model["badge"].maxValue.toString() + "+";
                        this._badgeValue(this.model["badge"].maxValue);
                    }
                    else {
                        badgeValue = this.model["badge"].minValue.toString();
                        this._badgeValue(this.model["badge"].minValue);
                    }
                }
                this.element.addClass(this._prefix + "tile-badge " + this._prefix + "badge-position-" + this._badgePosition());
                this.element.addClass(this._prefix + "tile-badge-value").attr("badgeValue", badgeValue);
            } else 
                this.element.removeClass(this._prefix + "tile-badge " + this._prefix + "badge-position-" + this._badgePosition() + " " + this._prefix + "tile-badge-value");
        },

        _setBadgePosition: function (value) {
            this._badgePosition(value)
            if (this._badgeEnabled())
                this.element.removeClass(this._prefix + "badge-position-bottomright " + this._prefix + "badge-position-topright").addClass(this._prefix + "badge-position-" + this._badgePosition());
        },

        _setBadgeEnabled: function (enabled) {
            this._badgeEnabled(enabled);
            this._setBadgeValue();
        },

        _setBadgeMaxValue: function (value) {
            this.model["badge"].maxValue = value;
            this._setBadgeValue();
        },

        _setBadgeMinValue: function (value) {
            this.model["badge"].minValue = value;
            if (this._badgeEnabled())
                this._setBadgeValue();
        },

        _setBadgeText: function (value) {
            this._badgeText(value);
            if (this._badgeEnabled())
                this._setBadgeValue();
        },

        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },

        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Tile elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmTile", "ej.mobile.Tile", {
        _setFirst: true,
        _rootCSS: "e-m-tile",
        defaults: {
            imagePath: null,
            renderMode: "auto",
            touchStart: null,
            touchEnd: null
        },
        dataTypes: {
            renderMode: "enum",
        },
        //Constructor function
        _init: function () {
            this._orgEle = this.element.clone();
            this._renderEJMControl();
        },
        //To render the Tile control
        _renderEJMControl: function () {
            ej.setRenderMode(this);
            this._prefix = "e-m-";
            this.element.addClass("e-m-" + this.model.renderMode);
            this._tileRender();
            this._wireEvents();
        },
        //Section for handling the touchstart event
        _onTouchStartHandler: function (evt) {
            var item = this.element, eventobj;
            if (this.model.renderMode == "windows")
                eventobj = evt.touches ? evt.touches[0] : evt;
            if (ej.isWindows())
                ej._touchStartPoints(evt, this);
            item.addClass(this.model.renderMode == "windows" ? ej._getSkewClass(item, eventobj.pageX, eventobj.pageY) : this._prefix + "state-active");
            if (this.model.touchStart)
                this._trigger("touchStart", { text: item.attr("text"), index: $(evt.target).index() });
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, false);
            ej.listenTouchEvent(this.element, ej.endEvent(), this._onEndDelegate, false);
            this._isMoved = false;
        },
        //Section for handling the touchmove event
        _onTouchMoveHandler: function (evt) {
            if (!ej.isWindows() || (ej.isWindows() && ej._isTouchMoved(evt, this))) {
                this._isMoved = true;
                this._removeActiveClass();
            }
        },
        // Section for handling the touchstart event
        _onTouchEndHandler: function (evt) {
            if (this.model.allowSelection) {
                if (!this._selected) {
                    this._selected = true;
                    this._selectElement.addClass("e-m-tile-selected");
                }
                else if (this._selected) {
                    this._selected = false;
                    this._selectElement.removeClass("e-m-tile-selected");
                }
            }
            if (!this._isMoved && this.model.touchEnd)
                this._trigger("touchEnd", { text: this.element.attr("text"), index: $(evt.target).index(), select: this._selected });
            this._removeActiveClass();
            this._isMoved = false;
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, true);
            ej.listenTouchEvent(this.element, ej.endEvent(), this._onEndDelegate, true);
        },
        //To refresh the control
        _refresh: function () {
            this._destroy();
            this.element.removeAttr("style");
            this.element.addClass(this._prefix + "tile e-js");
            this._renderEJMControl();
        }
    });

    ej.mobile.Tile.ImagePosition = {
        Center: "center",
        TopCenter: "topcenter",
        BottomCenter: "bottomcenter",
        RightCenter: "rightcenter",
        LeftCenter: "leftcenter",
        TopLeft: "topleft",
        TopRight: "topright",
        BottomRight: "bottomright",
        BottomLeft: "bottomleft",
        Fill: "fill"
    };

    ej.mobile.Tile.CaptionAlignment = {
        Normal: "normal",
        Left: "left",
        Right: "right",
        Center: "center"
    };

    ej.mobile.Tile.CaptionPosition = {
        InnerTop: "innertop",
        InnerBottom: "innerbottom",
        Outer: "outer"
    };

    ej.mobile.Tile.TileSize = {
        Medium: "medium",
        Small: "small",
        Large: "large",
        Wide: "wide"
    };

    ej.mobile.Tile.LiveTileType = {
        Flip: "flip",
        Slide: "slide",
        Carousel: "carousel"
    };
    ej.mobile.Tile.BadgePosition = {
        TopRight: "topright",
        BottomRight: "bottomright"
    };
    $.extend(true, ej.mobile.Tile.prototype, ej.TileBase.prototype);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Timepicker elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejmTimePicker", "ej.mobile.TimePicker", {
        _setFirst: true,
        _requiresID: true,
        _rootCSS: "e-m-timepicker",
        defaults: {
            renderMode: "auto",
            hourFormat: "twentyfour",
            value: null,
            timeFormat: null,
            locale: "en-US",
            enabled: true,
            enablePersistence: false,
            cssClass: "",
            select: null,
            load: null,
            focusIn: null,
            focusOut: null,
            open: null,
            close: null,
            change: null
        },
        dataTypes: {
            renderMode: "enum",
            hourFormat: "enum",
            value: "string",
            timeFormat: "enum",
            locale: "string",
            enabled: "boolean",
            enablePersistence: "boolean",
            cssClass: "string"
        },
        _init: function () {
            this._renderControl();
            this._createDelegates();
            this._wireEvents();
        },
        _timepickerInitialize: function () {
            this.model.timeFormat = this.model.timeFormat == null ? (this.model.hourFormat == "twelve" ? "hh:mm tt" : "HH:mm") : this.model.timeFormat;
            this.model.value = new Date(ej.format(new Date(), "MM/dd/yyyy") + " " + (typeof this.model.value == "string" ? this.model.value : ej.format(new Date(), this.model.timeFormat)));
            this._currentTime = this.model.value;
            this._twelvedata = [{ "value": "01" }, { "value": "02" }, { "value": "03" }, { "value": "04" }, { "value": "05" }, { "value": "06" }, { "value": "07" }, { "value": "08" }, { "value": "09" }, { "value": "10" }, { "value": "11" }, { "value": "12" }];
            this._twentyfourdata = [{ "value": "00" }, { "value": "01" }, { "value": "02" }, { "value": "03" }, { "value": "04" }, { "value": "05" }, { "value": "06" }, { "value": "07" }, { "value": "08" }, { "value": "09" }, { "value": "10" }, { "value": "11" }, { "value": "12" }, { "value": "13" }, { "value": "14" }, { "value": "15" }, { "value": "16" }, { "value": "17" }, { "value": "18" }, { "value": "19" }, { "value": "20" }, { "value": "21" }, { "value": "22" }, { "value": "23" }];
            this._minsdata = [{ "value": "00" }, { "value": "01" }, { "value": "02" }, { "value": "03" }, { "value": "04" }, { "value": "05" }, { "value": "06" }, { "value": "07" }, { "value": "08" }, { "value": "09" }, { "value": "10" }, { "value": "11" }, { "value": "12" }, { "value": "13" }, { "value": "14" }, { "value": "15" }, { "value": "16" }, { "value": "17" }, { "value": "18" }, { "value": "19" },
                                { "value": "20" }, { "value": "21" }, { "value": "22" }, { "value": "23" }, { "value": "24" }, { "value": "25" }, { "value": "26" }, { "value": "27" }, { "value": "28" }, { "value": "29" }, { "value": "30" }, { "value": "31" }, { "value": "32" }, { "value": "33" }, { "value": "34" }, { "value": "35" }, { "value": "36" }, { "value": "37" }, { "value": "38" }, { "value": "39" },
                                { "value": "40" }, { "value": "41" }, { "value": "42" }, { "value": "43" }, { "value": "44" }, { "value": "45" }, { "value": "46" }, { "value": "47" }, { "value": "48" }, { "value": "49" }, { "value": "50" }, { "value": "51" }, { "value": "52" }, { "value": "53" }, { "value": "54" }, { "value": "55" }, { "value": "56" }, { "value": "57" }, { "value": "58" }, { "value": "59" }];
            this._meridiansdata = [{ "value": this._localizedLabels.AM }, { "value": this._localizedLabels.PM }];
        },
        _renderControl: function () {
            this._orgEle = $(this.element).clone();
            ej.setRenderMode(this);
            this._localizedLabels = this._getLocalizedLabels();
            this._timepickerInitialize();
            this.element.addClass(this.model.cssClass + " e-m-" + this.model.renderMode + " e-m-user-select").attr({ "name": this._id, "readonly": true });
            this._outerdiv = ej.buildTag("div.e-m-" + this.model.renderMode + " e-m-tp e-m-user-select #" + this._id + "_tp", "<div class='e-m-tp-header'><span class='e-m-tp-time'></span>" + (this.model.hourFormat == "twelve" ? "<div class='e-m-tp-mer'><span class='mer am'>" + this._localizedLabels.AM + "</span><span class='mer pm'>" + this._localizedLabels.PM + "</span></div>" : "") + "</div>");
            this._innerdiv = ej.buildTag("div.e-m-tpinner");
            this._wrapdiv = ej.buildTag("div.e-m-tpwrap");
            this._visibletexthtml = "<div class='e-m-col-fill'></div>";
            this._hours = ej.buildTag("div.e-m-tp e-m-tphours e-m-left", "<div class='e-m-timewrapper'><div id=" + this._id + "_tphour><div class='e-m-tpouter'>" + this._visibletexthtml + "</div></div></div></div>");
            this._mins = ej.buildTag("div.e-m-tp e-m-tpmins e-m-center", "<div class='e-m-timewrapper'><div id=" + this._id + "_tpmin><div class='e-m-tpouter'>" + this._visibletexthtml + "</div></div></div></div>");
            this._meridians = ej.buildTag("div.e-m-tp e-m-tpmeridians e-m-right", "<div class='e-m-timewrapper'><div id=" + this._id + "_tpmeridian><div class='e-m-tpouter'>" + this._visibletexthtml + "</div></div></div></div>");
            this._dialogRendering();
            this._scrollInitialize();
            this._setCurrentPreviousTime();
            this._hourstext = this._outerdiv.find("#" + this._id + "_tphour .e-m-text");
            this._minstext = this._outerdiv.find("#" + this._id + "_tpmin .e-m-text");
            this._meridianstext = this._outerdiv.find("#" + this._id + "_tpmeridian .e-m-text");
            if (this.model.load)
                this._trigger("load", { value: this._currentTime });
            this._controlStatus(this.model.enabled);
        },
        _setCurrentPreviousTime: function () {
            var time = ej.format(new Date(this._currentTime), this.model.timeFormat, this.model.locale);
            $(this.element).val(time);
            this._updateHeader();
            this._previousTime = time;
        },
        _updateHeader: function () {
            this._outerdiv.find(".e-m-tp-header .e-m-tp-time").text(ej.format(new Date(this._currentTime), this.model.timeFormat, this.model.locale).slice(0, 5));
            if (this.model.hourFormat == "twelve") {
                this._outerdiv.find(".e-m-state-active").removeClass("e-m-state-active");
                this._outerdiv.find(".e-m-tp-header .e-m-tp-mer ." + ej.format(new Date(this._currentTime), "tt", this.model.locale).toLowerCase()).addClass("e-m-state-active");
            }
        },
        _scrollInitialize: function () {
            this._displacement = 45;    //height of a time element is 45px
            this._targetHeight = (window.innerHeight < 260) ? 45 : (window.innerHeight < 350) ? 135 : 225;   //height of a time element is 45
            this._minHeight = (window.innerHeight < 260) ? "low" : (window.innerHeight < 350) ? "medium" : "high";
            this._outerdiv.find(".e-m-col-fill").addClass("e-m-" + this._minHeight);
            this._scrollPanel("hour");
            this._scrollPanel("min");
            this._scrollPanel("meridian");
        },
        _scrollPanel: function (e) {
            this["_" + e] = ej.buildTag("div#" + this._id + "tp" + e + "_scrollpanel");
            this._outerdiv.find("#" + this._id + "_tp" + e).append(this["_" + e]);
            this["_" + e].ejmScrollPanel({
                enableMouseWheel: false, enableNativeScrolling: false, enableTransition: false, enableTransform: false, showScrollbars: false, target: this._id + "_tp" + e, targetHeight: this._targetHeight, renderMode: this.model.renderMode,
                scrollStart: $.proxy(this._onScrollStart, this), scrollStop: $.proxy(this._onScrollStop, this), enableBounce: false, enableDisplacement: true, displacementValue: this._displacement, displacementTime: 500,
                isRelative: true
            });
        },
        _dialogRendering: function () {
            this._renderTimeContent();
            this._dialog = ej.buildTag("div.e-m-tp-dialog e-m-" + this.model.hourFormat + "#" + this._id + "_dialog");
            this._dialog.ejmDialog({ allowScrolling: false, mode: "confirm", showHeader: false, leftButtonCaption: this._localizedLabels.cancelText, rightButtonCaption: this._localizedLabels.confirmText, renderMode: this.model.renderMode, enableAnimation: false });
            this._dialog.find(".e-m-dlg-content").append(this._outerdiv);
            this._dialog.find(".e-m-dlgbtnwrapper").appendTo(this._outerdiv);
        },
        _renderTimeContent: function () {
            if (this.model.hourFormat == "twentyfour")
                this._meridians.addClass("e-m-display-none");
            this._renderTemplate();
            this._columnRendering("hours");
            this._columnRendering("mins");
            this._columnRendering("meridians");
            this._innerdiv.append(this._wrapdiv);
            this._outerdiv.append(this._innerdiv);
        },
        _renderTemplate: function () {
            var template = {};
            template["default"] = "<div class='e-m-text'>{{:value}}</div>";
            $.templates(template);
        },
        _columnRendering: function (colname) {
            var dataval = (colname == "hours") ? this["_" + this.model.hourFormat + "data"] : this["_" + colname + "data"];
            var htmldata = $.render["default"](dataval) + this._visibletexthtml;
            this["_" + colname].find(".e-m-tpouter").html(this["_" + colname].find(".e-m-tpouter").html() + htmldata);
            this._wrapdiv.append(this["_" + colname]);
        },
        _createDelegates: function () {
            this._focusHndlr = $.proxy(this._showTimePicker, this);
            this._blurHndlr = $.proxy(this._blurEventHandler, this);
            this._dlgBtnClkHndlr = $.proxy(this._dialogButtonClick, this);
            this._resizeHndlr = $.proxy(this._resize, this);
        },
        _wireEvents: function (remove) {
            ej.listenTouchEvent($(this._dialog).find(".e-m-dlg-btn"), ej.tapEvent(), this._dlgBtnClkHndlr, remove);
            ej.listenEvents([$(this.element), $(this.element)], ["focus", "blur"], [this._focusHndlr, this._blurHndlr], remove);
            ej.listenTouchEvent($(window), "onorientationchange" in window ? "orientationchange" : "resize", this._resizeHndlr, remove);
        },
        _showTimePicker: function (e) {
            if (this.model.focusIn)
                this._trigger("focusIn", { value: this._currentTime });
            this._showHidePicker();
            this._setInitialTime();
            this._outerdiv.find(".e-m-timewrapper").removeClass("e-m-scrollstart");
            if (this.model.open)
                this._trigger("open", { value: this._currentTime });
        },
        _setInitialTime: function () {
            this._setValue(this._currentTime);
            this._setProperTime();
        },
        _setValue: function (date) {
            var date = new Date(date);
            this._resetScrollPos();
            this._hour.ejmScrollPanel("scrollTo", 0, -this._displacement * (this.model.hourFormat == "twelve" ? (date.getHours() % 12 == 0 ? 11 : date.getHours() % 12 - 1) : date.getHours()), 0);
            this._min.ejmScrollPanel("scrollTo", 0, -this._displacement * (date.getMinutes()), 0);
            this._meridian.ejmScrollPanel("scrollTo", 0, -this._displacement * (date.getHours() > 11 ? 1 : 0), 0);
        },
        _resetScrollPos: function () {
            this._hour.ejmScrollPanel("scrollTo", 0, 0, 0);
            this._min.ejmScrollPanel("scrollTo", 0, 0, 0);
            this._meridian.ejmScrollPanel("scrollTo", 0, 0, 0);
        },
        _blurEventHandler: function () {
            if (this.model.focusOut)
                this._trigger("focusOut", { value: this._currentTime });
        },
        _dialogButtonClick: function (e) {
            if ($(e.target).text() == this._localizedLabels.confirmText) {
                $(this.element).val(ej.format(this._currentTime, this.model.timeFormat));
                if (this._previousTime != $(this.element).val()) {
                    if (this.model.select)
                        this._trigger("select", { value: this._currentTime });
                }
                this._previousTime = $(this.element).val();
            }
            else if ($(e.target).text() == this._localizedLabels.cancelText) {
                this._currentTime = new Date(ej.format(new Date(), "MM/dd/yyyy") + " " + this._previousTime);
                this._setInitialTime();
            }
            this._showHidePicker(true);
            if (this.model.close)
                this._trigger("close", { value: this._currentTime });
        },
        _showHidePicker: function (remove) {
            var proxy = this;
            var e = { currentTarget: this.element };
            var visibility = remove ? "hidden" : "visible";
            this.element.blur();
            this._dialog.ejmDialog(remove ? "close" : "open");
        },
        _resize: function (e) {
            var proxy = this;
            setTimeout(function () {
                var currentHeight = (window.innerHeight < 350) ? (window.innerHeight < 260) ? "low" : "medium" : "high";
                if (proxy._minHeight != currentHeight) {
                    proxy._targetHeight = (currentHeight == "low") ? 45 : (currentHeight == "medium") ? 135 : 225;   //height of a time element is 45
                    proxy._minHeight = currentHeight;
                    proxy._hour.ejmScrollPanel({ targetHeight: proxy._targetHeight });
                    proxy._min.ejmScrollPanel({ targetHeight: proxy._targetHeight });
                    proxy._meridian.ejmScrollPanel({ targetHeight: proxy._targetHeight });
                    proxy._outerdiv.find(".e-m-col-fill").removeClass("e-m-short e-m-low e-m-high").addClass("e-m-" + currentHeight);
                    proxy._setInitialTime();
                }
            }, 200);
        },
        _onScrollStart: function (e) {
            $("#" + e.model.target).find(".e-m-text-active").removeClass('e-m-text-active');
        },
        _onScrollStop: function (e) {            
                var target = e.model.target.replace(this._id + "_tp", '');
                this._setProperTime(target);            
        },
        _setProperTime: function (target) {
            var time = this._scrollCoordinates();
            var hours = this.model.hourFormat == "twelve" ? (time.meridian > 0 ? (time.hours == 12 ? 12 : time.hours == 0 ? 13 : time.hours == 11 ? 12 : time.hours + 13) : (time.hours == 0 ? 1 : time.hours == 12 ? 11 : time.hours == 11 ? 0 : time.hours + 1)) : time.hours;
            this._currentTime = new Date(new Date().setHours(this.model.hourFormat == "twelve" ? hours : time.hours, time.mins));
            this._activeText(this.model.hourFormat == 'twelve' ? time.hours == 12 ? time.hours : time.hours + 1 : time.hours + 1, time.mins + 1, time.meridian, target);
            this._updateHeader();
            if (this.model.change)
                this._trigger("change", { value: this._currentTime });
        },
        _scrollCoordinates: function () {
            var time = {};
            time.Scrollhours = Math.round(this._hour.ejmScrollPanel("getScrollPosition").y / this._displacement);
            time.Scrollmins = Math.round(this._min.ejmScrollPanel("getScrollPosition").y / this._displacement);
            time.Scrollmeridian = Math.round(this._meridian.ejmScrollPanel("getScrollPosition").y / this._displacement);
            time.hours = -(time.Scrollhours);
            time.mins = -(time.Scrollmins);
            time.meridian = -(time.Scrollmeridian);
            return time;
        },
        _activeText: function (hours, mins, meridians, target) {
            if (target) {
                $(this["_" + target + "stext"]).removeClass("e-m-text-active");
                $(this["_" + target + "stext"][eval(target + "s") - (target == "meridian" ? 0 : 1)]).addClass("e-m-text-active");
            }
            else {
                $(this._hourstext[hours - 1]).addClass("e-m-text-active");
                $(this._minstext[mins - 1]).addClass("e-m-text-active");
                $(this._meridianstext[meridians]).addClass("e-m-text-active");
            }
        },
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        _setTimeFormat: function (timeformat) {
            $(this.element).val(ej.format(this._currentTime, this.model.timeFormat));
        },
        _setEnabled: function (enabled) {
            this._controlStatus(enabled);
        },
        _controlStatus: function (enabled) {
            this.element[!enabled ? "addClass" : "removeClass"]("e-m-state-disabled");
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-m-timepicker");
            this._renderControl();
            this._wireEvents();
        },
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
        _getLocalizedLabels: function () {
            return ej.getLocalizedConstants(this.sfType, this.model.locale);
        },

        /*---------------Public Methods---------------*/
        show: function (e) {
            this._showHidePicker();
        },
        hide: function (e) {
            this._showHidePicker(true);
        },
        enable: function (e) {
            this.model.enabled = true;
            this._controlStatus(this.model.enabled);
            this._showHidePicker(true);
        },
        disable: function (e) {
            this.model.enabled = false;
            this._controlStatus(this.model.enabled);
            this._showHidePicker(true);
        },
        getValue: function () {
            return $(this.element).val();
        },
        setCurrentTime: function (time) {
            this._currentTime = new Date(ej.format(new Date(), "MM/dd/yyyy") + " " + time);
            this._setCurrentPreviousTime();
        }
        /*---------------Public Methods End---------------*/
    });

    ej.mobile.TimePicker.HourFormat = {
        TwentyFour: "twentyfour",
        Twelve: "twelve"
    };
    ej.mobile.TimePicker.Locale = ej.mobile.TimePicker.Locale || {};
    ej.mobile.TimePicker.Locale["default"] = ej.mobile.TimePicker.Locale["en-US"] = {
        confirmText: "OK",
        cancelText: "CANCEL",
        AM: "AM",
        PM: "PM",
    };
})(jQuery, Syncfusion);;

(function ($, ej, undefined) {

    ej.widget("ejmToggleButton", "ej.mobile.ToggleButton", {
        _setFirst: true,
        _rootCSS: "e-m-tbutton",
        defaults: {
            cssClass: "",
            toggleState: true,
            renderMode: "auto",
            touchStart: null,
            touchEnd: null,
            change: null,
            enabled: true,
            enablePersistence: false
        },
        dataTypes: {
            toggleState: "boolean",
            enabled: "boolean",
            enablePersistence: "boolean"
        },

        _init: function () {
            this._renderControl();
            this._createDelegates();
            this._wireEvents(false);
            (this.model.enabled) ? this.enable() : this.disable();
        },

        _renderControl: function () {
        ej.setRenderMode(this);
        var modelobj = this.model;
        var element = this.element;
        element.addClass('e-m-tbutton' + " " + modelobj.cssClass + " " + 'e-m-' + modelobj.renderMode);
        this._input = ej.buildTag("input", {}, {}, { "name": $(element).attr('id'), "type": "checkbox", "data-role": "none" });
        var input = this._input;
        this._slider = ej.buildTag("div.e-m-tslider");
        var slider = this._slider;
        if (modelobj.toggleState) {
            input.attr('checked', 'checked').attr("value", true);
            slider.addClass("e-m-state-active");
            element.addClass("e-m-state-active");
            this._onState();
        }
        else  
        {
            input.attr("value", false)
            this._offState();
        }
        element.append(input).append(slider);
    }, 

        _createDelegates: function () {
            this._touchStartHandler = $.proxy(this._touchStart, this);
            this._touchEndHandler = $.proxy(this._touchEnd, this);
            this._touchMoveHandler = $.proxy(this._touchMove, this);
        },

        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenTouchEvent(this.element, ej.startEvent(), this._touchStartHandler, remove);
        },

        _touchStart: function (evt) {
            ej.blockDefaultActions(evt);
            var point = evt.touches ? evt.touches[0] : evt;
            this._startPoint = point.pageY;
            var slider = this._slider;
            var element = this.element;
            this.maxMove = element.width() - slider.width();
            var modelobj = this.model;
            this._flag = true;
            this._startstate = modelobj.toggleState;
            this._startTime = evt.timeStamp || Date.now();
            this._startPos = evt.pageX;
            if (ej.isTouchDevice()) evt =point;
            if(modelobj.renderMode == "windows"||modelobj.renderMode == "flat")  this.maxMove = this.maxMove - 5;
            slider.addClass("e-m-touchactive");
            this._position = { x: evt.pageX, y: evt.pageY };
            this.startValue = this._getNormalValue(this._position);
            this._handleX = slider.offset().left - element.offset().left;
            ej.listenEvents([document, document], [ej.endEvent(), ej.moveEvent()], [this._touchEndHandler, this._touchMoveHandler], false);
            var data = { state: modelobj.toggleState };
            if (modelobj.touchStart) this._trigger("touchStart", data);
        },

        _touchMove: function (evt) {
            ej.blockDefaultActions(evt);
            this._endPoint = (evt.touches ? evt.touches[0] : evt).pageY;
            if (this._startPoint != this._endPoint) {
                var modelobj = this.model;
                this._flag = false;
                if (ej.isTouchDevice()) evt = evt.touches ? evt.touches[0] : evt;
                this._position = { x: evt.pageX, y: evt.pageY };
                this._curValue = (this._getNormalValue(this._position)) - this.startValue;
                var curValue = this._curValue;
                var transformValue =curValue + this._handleX;
                this._endTime = evt.timeStamp || Date.now();
                this._endPos = evt.pageX;
                if (modelobj.renderMode == "flat" || modelobj.renderMode == "windows") {
                    if (transformValue >= 0 && transformValue <= this.maxMove) {
                         this._translateContent(this._slider, transformValue+2);
                    }
                }
                else (transformValue < this.maxMove / 2) ? this._offState() : this._onState();
            }
        },

        _touchEnd: function (evt) {
            ej.blockDefaultActions(evt);
            var slider = this._slider;
            var element = this.element;
            slider.removeClass("e-m-touchactive");
            var modelobj = this.model;
            ej.listenEvents([document, document], [ej.endEvent(), ej.moveEvent()], [this._touchEndHandler, this._touchMoveHandler], true);
            if (ej.isTouchDevice()) evt = evt.changedTouches ? evt.changedTouches[0] : evt;
            var endX = slider.offset().left - element.offset().left;
            this._delTime = this._endTime - this._startTime;
            this._delPos = (this._startstate) ? -(this._endPos - this._startPos) : (this._endPos - this._startPos);
            if ((modelobj.renderMode === 'ios7' || modelobj.renderMode === 'windows') && this._delTime < 300 && this._delPos > 10) (this._startstate) ? this._offState() : this._onState();
            else {
                if (!this._flag) ((endX < this.maxMove / 2) ? this._offState() : this._onState());
                else ((slider.hasClass("e-m-state-active")) ? this._offState() : this._onState());
            }
            if (this._input.attr("checked")) {
                slider.addClass("e-m-state-active");
                element.addClass("e-m-state-active");
            }
            else {
                slider.removeClass("e-m-state-active");
                element.removeClass("e-m-state-active");
            }
            var data = { state: modelobj.toggleState };
            if (modelobj.touchEnd) this._trigger("touchEnd", data);
            if (modelobj.change) {
                if (this._startstate != modelobj.toggleState)
                    this._trigger("change", data);
            }
        },

        _getNormalValue: function (position) {
            var currentLOB, currentLOBPercent, totalValue, currentValue; //currentLOT = current left or bottom
            currentLOB = position.x - this.element.offset().left;
            currentLOBPercent = (currentLOB / this.element[0].offsetWidth);
            totalValue = this.element.width();
            currentValue = currentLOBPercent * totalValue;
            return this._trimValue(currentValue);
        },

        _trimValue: function (value) {
            var step = 1;
            var stepModValue = (value) % step;
            var correctedValue = value - stepModValue;
            if (Math.abs(stepModValue) * 2 >= step) correctedValue += (stepModValue > 0) ? step : (-step);
            return parseFloat(correctedValue.toFixed(5));
        },

        _offState: function () {
            var modelobj = this.model;
            var slider = this._slider;
            modelobj.toggleState = false;
            this._input.removeAttr('checked', 'checked').attr("value", false);
            this._translateContent(slider, 0);
            slider.removeClass("e-m-state-active");
            this.element.removeClass("e-m-state-active");
        },

        _onState: function () {
            var modelobj = this.model;
            var slider = this._slider;
            modelobj.toggleState = true;
            this._input.attr('checked', 'checked').attr("value", true);
            var transValue = (modelobj.renderMode == "flat" || modelobj.renderMode == "windows") ? 43 : 49;
            this._translateContent(slider, transValue);
            slider.addClass("e-m-state-active");
            this.element.addClass("e-m-state-active");
            if (modelobj.renderMode == "ios7" || modelobj.renderMode == "android") this._translateContent(slider, 22);
        },

        _translateContent: function (element, transvalue) {
            element[0].style[ej.transform] = "translateX(" + transvalue + "px)";
        },

        _setModel: function (options) {
            var refresh;
            for (var key in options)
                switch (key) {
                    case "renderMode": (options.renderMode == "auto") ? ej.setRenderMode(this) : this.model.renderMode = options.renderMode;
                        refresh = true;
                        break;
                    case "toggleState":(this.model.toggleState) ? this._onState() : this._offState(); break;
                    case "enabled": options.enabled ? this.enable() : this.disable(); break;
                }
            if (refresh) this._refresh();
        },

        _refresh: function () {
            this._clearElement();
            this._renderControl();
            this._wireEvents();
        },

        _clearElement: function () {
            this._wireEvents(true);
            this.element.removeAttr("class");
            this.element.removeAttr("style");
            this.element.html("");
        },

        _destroy: function () {
            this._clearElement();
        },

        /*---------------Public Methods---------------*/

        disable: function () {
            this.element.addClass("e-m-state-disabled");
            this._wireEvents(true);
            this.model.enabled = false;
        },

        enable: function () {
            this.element.removeClass("e-m-state-disabled");
            this._wireEvents(false);
            this.model.enabled = true;
        }
        /*---------------Public Methods End---------------*/

    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Grid elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    // ejmGrid is the plugin name 
    // "ej.mobile.Grid" is "namespace.className" will hold functions and properties    

    ej.widget("ejmGrid", "ej.mobile.Grid", {
        _setFirst: true,
        _rootCSS: "e-m-grid-core",
        validTags: ["div"],
        _tags: [{
            tag: "columns",
            attr: ["field", "headerText", "allowFiltering", "allowSorting", "width", "textAlign", "format", "type"]
        }],
        _ignoreOnPersist: ["query", "dataSource", "allowScrolling", "cssClass"],

        defaults: {
            allowPaging: false,
            allowSorting: false,
            allowFiltering: false,
            allowScrolling: false,
            allowSelection: true,
            dataSource: null,
            caption: "",
            cssClass: "",
            locale: "en-US",
            enablePersistence: false,
            selectedRowIndex: -1,
            rowSelecting: null,
            rowSelected: null,
            actionBegin: null,
            actionComplete: null,
            actionSuccess: null,
            actionFailure: null,
            load: null,
            queryCellInfo: null,
            rowDataBound: null,
            modelChange: null,
            pageSettings: {
                pageSize: 5,
                currentPage: 1,
                display: "normal",
                type: "scrollable",
                totalRecordsCount: null
            },
            scrollSettings: {
                enableColumnScrolling: false,
                height: "auto",
                enableRowScrolling: true,
                enableNativeScrolling: (ej.isAndroid() || ej.isIOS()) ? false : ej.isDevice() ? true : false
            },
            sortSettings: {
                allowMultiSorting: false,
                sortedColumns: []
            },
            filterSettings: {
                filterBarMode: "onenter",
                interval: 1500,
                filteredColumns: []
            },
            showCaption: false,
            allowColumnSelector: false,
            transition: ej.isAndroid() ? "pop" : ej.isWindows() ? "slide" : "slide",
            renderMode: "auto",
            query: ej.Query(),
            columns: []
        },
        dataTypes: {
            allowPaging: "boolean",
            allowSorting: "boolean",
            allowFiltering: "boolean",
            allowSelection: "boolean",
            allowScrolling: "boolean",
            locale: "string",
            sortSettings: {
                allowMultiSorting: "boolean",
                sortedColumns: "array"
            },
            filterSettings: {
                filteredColumns: "array",
            },
            dataSource: "data",
            query: "data",
            columns: "array",
            pageSettings: {
                pageSize: "number",
                display: "enum",
                type: "enum",
            },
            scrollSettings: {
                enableRowScrolling: "boolean",
                enableColumnScrolling: "boolean",
            },
            renderMode: "enum",
            allowColumnSelector: "boolean",
            showCaption: "boolean",
            transition: "string"
        },

        observables: ["dataSource", "selectedRowIndex", "pageSettings.currentPage"],
        dataSource: ej.util.valueFunction("dataSource"),
        selectedRowIndex: ej.util.valueFunction("selectedRowIndex"),
        currentPage: ej.util.valueFunction("pageSettings.currentPage"),

        _columns: function (index, property, value, old) {
            $('.e-m-grid-hbar th .e-m-title')[index.columns].innerHTML = value;
        },
        _init: function () {
            this._getLocalizedLabels();
            this._trigger("load");
            this._page = ej.getCurrentPage();
            this._hiddenColumns = [];
            this._visibleColumns = [];
            this._dataColumns = [];
            this._rowSelect = true;
            this._colInit = false;
            this._touchMove = false;
            this._controlInit = true;
            if (this.model.renderMode == "auto")
                ej.setRenderMode(this);
            if (this.model.allowSorting) {
                this._curSort = "clearsorting";
                this._sortedColumns = [];
                this._sortDirection = [];
                this._handleInitialSorting();
            }
            this._id = this.element[0].id;
            this._render();
            this._createDelegates();
            this._wireEvents(false);
        },

        _handleInitialSorting: function () {
            var proxy = this;
            var length = this.model.sortSettings.sortedColumns.length;
            $.each(this.model.sortSettings.sortedColumns, function (index, value) {
                if (proxy.model.sortSettings.allowMultiSorting || index == length - 1) {
                    proxy._sortedColumns.push(value.columnName);
                    proxy._sortDirection.push(value.sortDirection);
                }

            });
        },

        _getLocalizedLabels: function () {
            this._localizedLabels = ej.getLocalizedConstants(this.sfType, this.model.locale)
        },

        _setLocale: function () {
            this._getLocalizedLabels();            
        },

        _render: function () {
            this._renderClientgrid();
            this._content = this.element.find(".e-m-grid-content");
            if (this.model.allowPaging)
                this._createPagerObject();
            if (this.model.allowFiltering) {
                this._createFilterObject();
                this._filters = this._filteringMgr.model.filterConditions = this.model.filterSettings.filteredColumns;
            }
            if (this.model.allowColumnSelector)
                this._createColumnListContainer();
            this._load();
        },

        _createColumnListContainer: function () {
            this.element.wrap(ej.buildTag("div.e-m-grid-parent-container"));
            this._parentElement = this.element.parent();
            var container = ej.buildTag("div.e-m-grid-colnavcontainer e-m-" + this.model.renderMode + " subpage e-m-state-hide e-m-grid-core#" + this._id + "colnav", {}, {}, { "data-url": this._id + "colnav" });
            var header = ej.buildTag("div.e-m-grid-colnavhdr", {}, {}, {
                "data-role": "ejmnavigationbar", "data-ej-title": this._localizedLabels["columnSelectorText"],
                "data-ej-rendermode": this.model.renderMode, "data-ej-position": "normal"
            });
            container.append(header);
            this._subPagelistDiv = ej.buildTag("div", {}, {}, {});
            var ul = ej.buildTag("ul");
            $.each(this.model.columns, function (indx, col) {
                var li = ej.buildTag("li", {}, {}, { "data-ej-text": col.headerText });
                ul.append(li);
            });
            this._subPagelistDiv.append(ul);
            container.append(this._subPagelistDiv);
            var doneBtn, cancelBtn;
            if (this.model.renderMode == "android" || this.model.renderMode == "windows") {
                doneBtn = ej.buildTag("input", {}, {}, {
                    "data-role": "ejmbutton",
                    "data-ej-rendermode": this.model.renderMode, "data-ej-text": this._localizedLabels["columnSelectorDone"], "type": "button"
                });
                cancelBtn = ej.buildTag("input", {}, {}, {
                    "data-role": "ejmbutton",
                    "data-ej-rendermode": this.model.renderMode, "data-ej-text": this._localizedLabels["columnSelectorCancel"], "type": "button"
                });
                container.append(doneBtn).append(cancelBtn);
            }
            this._warningDialog = ej.buildTag("div#" + this._id + "_warningDlg", {}, {}, {
                "data-role": "ejmdialog",
                "data-ej-rendermode": this.model.renderMode, "data-ej-title": this._localizedLabels["columnSelectorWarning"],
                "data-ej-mode": "alert", "data-ej-enablemodal": true, "data-ej-enableautoopen": false
            }).append(ej.buildTag("div").html(this._localizedLabels["HideColumnAlert"]));
            container.append(this._warningDialog);
            if (ej.isMobile())
                $("body").append(container);
            else
                this._parentElement.append(container);
            this._subPagelistDiv.ejmListView({
                "touchEnd": ej.proxy(this._onColumnSelectHandler, this),
                "renderMode": this.model.renderMode,
                "enableChecklist": true
            });
            ej.widget.init(container);
            if (this.model.renderMode == "android" || this.model.renderMode == "windows") {
                doneBtn.ejmButton({ "touchEnd": ej.proxy(this._columnSelectionDoneHandler, this) });
                cancelBtn.ejmButton({ "touchEnd": ej.proxy(this._columnSelectionCancelHandler, this) });
            }
            else
                header.ejmNavigationBar({ "rightButtonTap": ej.proxy(this._columnSelectionDoneHandler, this), "leftButtonTap": ej.proxy(this._columnSelectionCancelHandler, this) });
            this._warningDialog.ejmDialog({ "buttonTap": ej.proxy(this._dialogButtonClick, this) });

        },

        _dialogButtonClick: function (e) {
            this._warningDialog.ejmDialog("close");
        },

        _columnSelectionCancelHandler: function (e) {
            if (this._colHideList != this._prevColHideList) {
                this._subPagelistDiv.ejmListView("uncheckAll");
                if (this._prevColHideList) {
                    for (var i = 0; i < this._prevColHideList.length; i++)
                        this._subPagelistDiv.ejmListView("checkItemsByIndex", this._dataColumns.indexOf(this._prevColHideList[i]));
                }
                this._colHideList = this._prevColHideList;
            }
            history.back();
        },

        _columnSelectionDoneHandler: function (e) {
            if (this._colHideList && this._colHideList.length == this.model.columns.length) {
                this._warningDialog.ejmDialog("open");
            }
            else {
                if (this._colHideList) {
                    if (this._colHideList && this._hiddenColumns != this._colHideList)
                        this.hideColumns(this._colHideList);
                    var proxy = this;
                    this._colShowList = [];
                    $.each(this.model.columns, function (indx, col) {
                        if (proxy._colHideList.indexOf(col.headerText) == -1)
                            proxy._colShowList.push(col.headerText);
                    });
                    if (this._colShowList && this._visibleColumns.toString() != this._colShowList.toString())
                        this.showColumns(this._colShowList);
                }
                else
                    this.showColumns(this._prevColHideList);
                history.back();
            }
            this._prevColHideList = this._colHideList;
        },

        _onColumnSelectHandler: function (args) {
            this._colHideList = args.checkedItemsText;
        },

        _createPagerObject: function () {
            var pSettings = this.model.pageSettings;
            pSettings["id"] = this._id;
            var element = $("#" + this._id + "_pager");
            element.ejmGridPager(pSettings);
            this._pager = element.ejmGridPager("instance");
            this._pager.model.currentPage = parseInt(this.currentPage());
        },

        _createFilterObject: function () {
            var element = this.element.find(".e-m-grid-header");
            var fSettings = this.model.filterSettings;
            fSettings["id"] = this._id;
            element.ejmGridFilter(fSettings);
            this._filteringMgr = element.ejmGridFilter("instance");
        },


        _renderClientgrid: function () {
            this.element.addClass("e-m-grid-core e-m-core subpage");
            if (this.model.cssClass != "" && this.model.cssClass != undefined) {
                this.element.addClass(this.model.cssClass);
                this._cssClass = this.model.cssClass;
            }
            this._mainContainer = ej.buildTag("div.e-m-grid-container#" + this._id + "container");
            if (this.model.allowFiltering)
                this._mainContainer.addClass("e-m-filter");
            if (this.model.allowFiltering)
                this._renderFilterBar();
            if (this.model.showCaption || this.model.allowColumnSelector)
                this._renderCaption();
            if (this.model.allowColumnSelector) {
                this._colNavigator = ej.buildTag("div.e-m-grid-colnav e-m-fontimage");
                this.element.find(".e-m-caption-row").append(this._colNavigator);
            }
            this.element.append(this._mainContainer);
            if (this.model.allowScrolling && this.model.scrollSettings.enableColumnScrolling) {
                this._innerContainer = ej.buildTag("div.e-m-grid-innercontainer")
                .appendTo(this._mainContainer);
            }
            this._renderHeader();
            this._renderContent();

            this._setRenderMode();
        },

        _setRenderMode: function () {
            var modes = "e-m-android e-m-ios7 e-m-windows e-m-flat";
            this.element.removeClass(modes).addClass("e-m-" + this.model.renderMode);
            $("#" + this._id + "_pager").removeClass(modes).addClass("e-m-" + this.model.renderMode);
        },

        _renderContent: function () {
            var content = ej.buildTag("div#" + this._id + "_content.e-m-grid-content");
            this._renderInnerContent(content);
            if (this._innerContainer)
                this._innerContainer.append(content);
            else
                this._mainContainer.append(content);
            if (this.model.allowPaging)
                this._renderPager();
        },

        _renderPager: function () {
            var pgr = ej.buildTag("div#" + this._id + "_pager.e-m-grid-core e-m-grid-pager e-m-pager-" + this.model.pageSettings.display + " e-m-pagertype-" + this.model.pageSettings.type + " e-m-" + this.model.renderMode);
            var wrapper = ej.buildTag("div.e-m-grid-pwrapper e-m-state-default");
            var ipgr = ej.buildTag("div.e-m-grid-inrpgr e-m-state-default");
            if (this.model.pageSettings.type == "normal") {
                ej.buildTag("div.e-m-grid-first  e-m-fontimage").append(ej.buildTag("div.e-m-grid-hfirst  e-m-fontimage")).appendTo(ipgr);
                ej.buildTag("div.e-m-grid-prev  e-m-fontimage").append(ej.buildTag("div.e-m-grid-hprev  e-m-fontimage")).appendTo(ipgr);
            }
            ej.buildTag("div#" + this._id + "_pgrtxt.e-m-grid-ptxt").html("1").appendTo(ipgr);
            if (this.model.pageSettings.type == "normal") {
                ej.buildTag("div.e-m-grid-next  e-m-fontimage").append(ej.buildTag("div.e-m-grid-hnext  e-m-fontimage")).appendTo(ipgr);
                ej.buildTag("div.e-m-grid-last  e-m-fontimage").append(ej.buildTag("div.e-m-grid-hlast  e-m-fontimage")).appendTo(ipgr);
            }
            ipgr.appendTo(wrapper);
            wrapper.appendTo(pgr);
            if (this.model.pageSettings.display == ej.mobile.Grid.PagerDisplay.Fixed)
                pgr.appendTo(ej.getCurrentPage());
            else
                pgr.appendTo(this.element);
        },

        _renderInnerContent: function (content) {
            var tab = ej.buildTag("table.e-m-grid-table", {}, {}, { "cellspacing": "0.25px", "data-role": this.element[0].id + "_Table" });
            this._renderColGrp(tab, false);
            var tbody = ej.buildTag("tbody").append(ej.buildTag("tr"));
            tbody.append(ej.buildTag("td.e-m-grid-emptycell", {}, {}, { "colspan": this.model.columns.length }).html(this._localizedLabels["spinnerText"]));
            tab.append(tbody);
            content.append(tab);
            this._setGridContentTable(tab[0]);
        },

        _renderCaption: function () {
            var caption = ej.buildTag("div.e-m-state-default e-m-caption-row").append(ej.buildTag("span").html(this.model.caption + ": " + this.model.dataSource.length + " " + this._localizedLabels["captionText"]));
            this.element.prepend(caption);
        },

        _renderFilterBar: function () {
            var fbar = ej.buildTag("div.e-m-grid-fbardiv");
            var search = ej.buildTag("span.e-m-icon-grid-search  e-m-fontimage");
            var ispan = ej.buildTag("div.e-m-grid-finput");
            var clr = ej.buildTag("span.e-m-icon-grid-clear  e-m-fontimage");
            var finput = ej.buildTag("input.e-m-text-input", {}, {}, { "data-role": "none" });
            if (ej.isDevice() && ej.isAndroid()) finput.addClass("e-m-placeholder-css");
            ej.buildTag("form").append(finput).append(search).append(clr).appendTo(ispan);
            fbar.append(ispan);//.appendTo(ihdr);
            (this.model.renderMode == "windows" && !ej.isMobile()) ? clr.css("display", "none") : "";
            this.element.append(fbar);
        },

        _renderHeader: function () {
            var hdr = ej.buildTag("div.e-m-state-default e-m-grid-header");
            var ihdr = ej.buildTag("div.e-m-grid-inrhdr");
            var hset = ej.buildTag("div.e-m-grid-hdrsettings");
            var htable = ej.buildTag("table.e-m-grid-table", {}, {}, { "cellspacing": "0.25px" });
            this._renderColGrp(htable, true);
            this._renderTHead(htable);
            hset.append(htable);
            ihdr.append(hset);
            hdr.append(ihdr);
            if (this._innerContainer)
                this._innerContainer.append(hdr);
            else
                this._mainContainer.append(hdr);
        },

        _renderColGrp: function (table, hiddenColInit) {
            var proxy = this;
            var colgrp = ej.buildTag("colgroup");
            this._colWidth = 0;
            this._setHorScrWidth = true;
            this._hiddenColumns = [];
            this._visibleColumns = [];
            $.each(proxy.model.columns, function (indx, val) {
                var col = ej.buildTag("col");
                if (val.width && val.width > 0) {
                    col.css("width", val.width + "px");
                    proxy._colWidth = proxy._colWidth + val.width;
                }
                else
                    proxy._setHorScrWidth = false;
                colgrp.append(col);
                if (!val.headerText)
                    val.headerText = val.field;
                if (val.visible == false) {
                    hiddenColInit ? proxy._hiddenColumns.push(val.headerText) : {};
                    col.addClass("e-hide");
                }
                else {
                    hiddenColInit ? proxy._visibleColumns.push(val.headerText) : {};
                    proxy.model.columns[indx]["visible"] = true;
                }
                proxy._dataColumns.push(val.headerText);
            });
            setTimeout(function () {
                if (proxy.element.width() > proxy._colWidth && proxy._setHorScrWidth && proxy._colWidth > 0 && !ej.isMobile() && proxy.model.allowScrolling && proxy.model.scrollSettings.enableColumnScrolling)
                    proxy.element.width(proxy._colWidth);
            }, 100);
            table.append(colgrp);
        },

        _renderTHead: function (htable) {
            var proxy = this;
            var thead = ej.buildTag("thead");
            var tr = ej.buildTag("tr.e-m-grid-hbar");
            if (this.model.columns.length > 0 && typeof (this.model.columns[0]) == "string") {
                var columns = $.extend({}, this.model.columns);
                this.model.columns = [];
                $.each(columns, function (indx, column) {
                    proxy.model.columns.push({ "field": column, "headerText": column });
                });
            }
            $.each(proxy.model.columns, function (indx, column) {
                var th = ej.buildTag("th.e-m-grid-hdrcell e-m-state-default");
                column.visible == false && th.addClass("e-hide");
                if (column.textAlign != undefined)
                    th.css("text-align", column.textAlign);
                if (indx == proxy.model.columns.length - 1)
                    th.addClass("e-m-grid-lastth");
                var div = ej.buildTag("div.e-m-grid-col-elem e-m-grid-coldesc e-m-grid-hcell");
                $.each(proxy.model.sortSettings.sortedColumns, function (i, sortCol) {
                    if (sortCol.columnName == column.field)
                        div.addClass("e-m-grid-" + sortCol.sortDirection);
                });
                var hcellClasses = "e-m-grid-hcelldiv";
                if (proxy.model.allowSorting && (column.allowSorting == undefined || column.allowSorting))
                    hcellClasses = hcellClasses + " e-m-sort";
                if (proxy.model.allowFiltering)
                    hcellClasses = hcellClasses + " e-m-filter";
                var idiv = ej.buildTag("div." + hcellClasses);
                var textSpan = ej.buildTag("span.e-m-title").html(column.headerText);
                idiv.append(textSpan);
                if (proxy.model.allowSorting && (column.allowSorting == undefined || column.allowSorting)) {
                    var sdiv = ej.buildTag("span.e-m-icon-grid-sort e-m-fontimage");
                    textSpan.append(sdiv);
                }
                div.append(idiv);
                if (proxy.model.allowFiltering && (column.allowFiltering == undefined || column.allowFiltering)) {
                    var fdiv = ej.buildTag("span.e-m-icon-grid-fltr e-m-fontimage");
                    div.append(fdiv);
                }
                th.append(div);
                tr.append(th);
            });
            thead.append(tr);
            htable.append(thead);
            var tbody = ej.buildTag("tbody", {}, { "display": "none" }).append(ej.buildTag("tr").append(ej.buildTag("td")));
            htable.append(tbody);
        },

        _renderColElements: function (action) {
            return ej.buildTag("div.e-m-grid-col-child e-m-grid-col-" + action).append(ej.buildTag("span.e-m-grid-" + action + "  e-m-fontimage"));
        },
        /* --------------------end region grid rendering-------------------------------------------*/

        /*----------------------- Region internal functions---------------------------------------------*/

        _load: function () {
            if (this.model.allowSelection && !this._selectionManager) {
                this.element.ejmGridSelectionManager(this.model.pageSettings);
                this._selectionManager = this.element.ejmGridSelectionManager("instance");
            }
            if (this._checkWaitingPopup())
                this.element.ejWaitingPopup({ autoDisplay: false });
            this.refreshContent("refresh");

        },

        _addInitialTemplate: function () {
            this._colInit = true;
            if (this.model.pageSettings.totalRecordsCount) {
                this._initColumns(this.model.currentViewData[0] != undefined ? this.model.currentViewData[0] : this.model.currentViewData.value);
                this.element.find(".e-m-grid-header").remove();
                this._renderHeader();
                var hdrElement = this.element.find(".e-m-grid-header");
                hdrElement.insertBefore(this.element.find(".e-m-grid-content"));

            }
            var id = this.element[0].id;
            var template = this._page.find("#" + id + "_JSONTemplate");
            $.views.helpers({ _gridFormatting: this._formatting });
            var TBODY = ej.buildTag("tbody");
            var trTemplate = ej.buildTag("tr");
            $(TBODY).attr('id', id + "_JSONTemplate").css('display', 'none');
            $.each(this.model.columns, function (index, column) {
                var tdCell = ej.buildTag("td.e-m-grid-rowcell");
                column.visible == false && tdCell.addClass("e-hide");
                tdCell.html("{{:" + "#data['" + column.field + "']" + "}}");
                if (column.textAlign != undefined)
                    tdCell.css("text-align", column.textAlign);
                if (column.format != undefined && column.format != "" || (column.type && column.type.toLowerCase() == "datetime" || column.type && column.type.toLowerCase() == "date")) {
                    if (column.format == "" || !column.format)
                        column.format = "{0:" + ej.preferredCulture().calendar.patterns.d + "}";
                    tdCell.html("{{:~_gridFormatting('" + column.format + "'," + "#data['" + column.field + "']" + "," + "#data['" + column.field + "']" + ")}}");
                }
                if (!ej.isNullOrUndefined(column["cssClass"])) {
                    tdCell.addClass(column["cssClass"]);
                }
                $(trTemplate).append(tdCell);
            });
            $(TBODY).append(trTemplate);
            $(this._page).append(TBODY);
        },

        _handleQueryCell: function (data) {
            if (data && data.length != 0) {
                this._gridRows = this._getGridContentTable().rows;
                if (this.model.queryCellInfo != null || this.model.rowDataBound != null) {
                    for (var row = 0; row < this._gridRows.length; row++)
                        this._rowEventtrigger(this._gridRows[row], data[row]);
                }
            }
        },

        _rowEventtrigger: function (row, data) {
            this._trigger("rowDataBound", { element: row, data: data });
            var tdCells = row.cells;
            for (cellIndex = 0; cellIndex < tdCells.length; cellIndex++) {
                var args = { element: tdCells[cellIndex], data: data, text: tdCells[cellIndex].innerHTML };
                this._trigger("queryCellInfo", args);
            }

        },
        _checkWaitingPopup: function () {
            return $.isFunction($.fn.ejWaitingPopup);
        },
        _renderScrollPager: function () {
            this._controlInit = false;
            this._pgrTxtContainer = $("#" + this._id + "_pgrtxt");
            this._updateScrollPager();
            this._pageScroller = this._createScroller({ enableHrScroll: true, enableVrScroll: false, showScrollbars: false, target: this._id + "_pgrtxt", rendermode: this.model.renderMode });
        },
        _updateScrollPager: function () {
            var template = {}, pageNumbers = [];
            if (!$.templates["pager"]) {
                template["pager"] = "{{if pageNumber < 100 }}<div class='e-m-page-num'>{{:pageNumber}}</div>{{else}}<div class='e-m-page-num e-m-page-large'>{{:pageNumber}}</div>{{/if}}";
                $.templates(template);
            }
            for (i = 1 ; i <= this._getPageCount() ; i++)
                pageNumbers.push({ "pageNumber": i });
            if (this._pgrTxtContainer.find(".e-m-pscroll").length > 0) {
                var div = $.render["pager"](pageNumbers);
                this._pgrTxtContainer.find(".e-m-pscroll").html(div);
            }
            else {
                var div = "<div class='e-m-pscroll'>" + $.render["pager"](pageNumbers) + "</div>";
                this._pgrTxtContainer.html(div);
            }
            $(this._pgrTxtContainer.find(".e-m-page-num")[parseInt(this.currentPage()) - 1]).addClass("e-m-state-active");
            if (this._pageScroller)
                this._pageScroller.ejmScrollPanel('refresh');
        },

        _findPredicate: function () {
            var predicate, prevColumn;
            $.each(this._filters, function (indx, obj) {
                var ignoreCase = false;
                if (typeof (obj.value) == "string")
                    ignoreCase = !obj.isCaseSensitive;
                if (prevColumn && prevColumn != obj.columnName)
                    obj.predicate = "and";
                if (obj.value != "") {
                    if (!predicate)
                        predicate = ej.Predicate(obj.columnName, obj.operator, obj.value, ignoreCase);
                    else
                        predicate = predicate[obj.predicate](obj.columnName, obj.operator, obj.value, ignoreCase);
                }
                prevColumn = obj.columnName;
            });
            return predicate;
        },

        _createQuery: function (requestType) {
            var predicate, sortedColumnInfo = [], query = [];
            query = this.model.query.clone().requiresCount();
            if (this.model.allowFiltering) {
                predicate = this._findPredicate();
                if (predicate) {
                    if (requestType == "filtering") {
                        this.currentPage(1);
                        if (this._pager)
                            this._pager.model.currentPage = 1;
                    }
                    query = query.where(predicate);
                }
            }
            if (this._sortedColumns && this.model.allowSorting) {
                var proxy = this;
                $.each(this._sortedColumns, function (index, value) {
                    if (proxy._sortDirection[index] == "descending")
                        value = value + " desc";
                    if (proxy._sortDirection[index] != "clearsorting")
                        sortedColumnInfo.push(value);
                });
                if (sortedColumnInfo.length > 0)
                    query.sortBy(sortedColumnInfo);
            }
            if (this.model.allowPaging) {
                if (this._getPageCount() < parseInt(this.currentPage()) && this.model.pageSettings.totalRecordsCount)
                    this.currentPage(1);
                query.page(parseInt(this.currentPage()), this.model.pageSettings.pageSize);
            }
            return query;
        },

        _getPageCount: function () {
            return Math.ceil(this.model.pageSettings.totalRecordsCount / this._pager.model.pageSize);
        },

        _afterDataProcess: function (requestType, data) {
            if (this.model.allowFiltering && this.model.allowPaging) {
                if (this.model.pageSettings.totalRecordsCount <= this._pager.model.pageSize)
                    this._pager.model.currentPage = 1;
                else {
                    var pages = this._getPageCount();
                    this._pager.model.currentPage = this._pager.model.currentPage < pages ? this._pager.model.currentPage : pages;
                }
                this.currentPage(this._pager.model.currentPage);
            }
            this._commonSuccessAction(this.model.pageSettings.totalRecordsCount, requestType);
            this._jsonSuccessAction(data, requestType);
            if (this.model.allowScrolling) {
                if (this.model.scrollSettings.height == "auto") {
                    var container = this.element.find(".e-m-grid-content");
                    var height = window.innerHeight - container.offset().top - this.element.nextAll().outerHeight();
                    if (!this.model.allowPaging)
                        height = height;
                    else
                        height = height - this.element.find(".e-m-grid-pager").height();
                    if (height > container.height())
                        height = container.height();
                }
                if (this.model.scrollSettings.enableRowScrolling && !this._verScroller)
                    this._verScroller = this._createScroller({ showScrollbars: false, targetHeight: this.model.scrollSettings.height == "auto" ? height : this.model.scrollSettings.height, rendermode: this.model.renderMode, target: this._id + "_content" });
                if (this.model.scrollSettings.enableColumnScrolling && !this._horScroller) {
                    var scrObj = { enableHrScroll: true, enableVrScroll: false, showScrollbars: false, target: this._id + "container", rendermode: this.model.renderMode };
                    var proxy = this;
                    setTimeout(function () {
                        if (proxy._setHorScrWidth || proxy.element.width() < proxy._colWidth)
                            scrObj["scrollWidth"] = proxy._colWidth;
                        proxy._horScroller = proxy._createScroller(scrObj);
                    }, 0);
                }
                if (this._controlInit)
                    this._resize();
            }
        },
        _createScroller: function (sObject) {
            var scroller = ej.buildTag("div");
            scroller.appendTo(this._page);
            sObject["enableNativeScrolling"] = this.model.scrollSettings.enableNativeScrolling;
			sObject["isRelative"] = true;
            scroller.ejmScrollPanel(sObject);
            return scroller;
        },
        _sendSortingRequest: function (columnName, sortDirection) {
            var currentIndx = 0;
            var avail = false;
            sortDirection = sortDirection.toLowerCase();
            if (columnName != null && columnName != "") {
                if (this.model.sortSettings.allowMultiSorting) {
                    for (i = 0; i < this._sortedColumns.length; i++) {
                        if (this._sortedColumns[i] == columnName) {
                            avail = true;
                            this._sortDirection[i] = sortDirection;
                        }
                    }
                    if (avail == false) {
                        this._sortedColumns.push(columnName);
                        this._sortDirection.push(sortDirection);
                    }
                }
                else {
                    this._sortedColumns[0] = columnName;
                    this._sortDirection[0] = sortDirection;
                }
            }
            this.refreshContent("sorting");

        },

        _setDateFilters: function (filterObject) {
            var prevDate = new Date(filterObject.value.setDate(filterObject.value.getDate() - 1));
            var nextDate = new Date(filterObject.value.setDate(filterObject.value.getDate() + 2));
            var prevObj = $.extend({}, filterObject);
            var nextObj = $.extend({}, filterObject);
            prevObj.value = prevDate;
            prevObj.operator = ">";
            prevObj.predicate = "and";
            nextObj.value = nextDate;
            nextObj.operator = "<";
            nextObj.predicate = "and";
            this._filters.push(prevObj);
            this._filters.push(nextObj);
        },

        _sendFilteringRequest: function (operators, values, predicate) {
            var firstLoop = false;
            if (operators != null) {
                for (var index = 0; index < operators.length; index++) {
                    if (values[index] != null) {
                        var filterObject = {
                            columnName: this._filteringMgr._currentFilterColumn.field,
                            operator: operators[index],
                            value: values[index],
                            predicate: predicate,
                            isCaseSensitive: this._filteringMgr._currentFilterColumn.isCaseSensitive == undefined ? false : this._filteringMgr._currentFilterColumn.isCaseSensitive
                        };
                        if (this._filters.length == 0) {
                            if ((this._filteringMgr._currentFilterColumn.type.toLowerCase() == "datetime" || this._filteringMgr._currentFilterColumn.type.toLowerCase() == "date") && operators == "equal")
                                this._setDateFilters(filterObject);
                            else
                                this._filters.push(filterObject);
                        }
                        else {
                            var proxy = this;
                            if (!firstLoop) {
                                var clearObj = new Array();
                                $.each(this._filters, function (key, value) {
                                    if (value.columnName == filterObject.columnName) {
                                        clearObj.push(value);
                                    }
                                });
                                $.each(clearObj, function () {
                                    proxy._filters.splice($.inArray(this, proxy._filters), 1);
                                });
                            }
                            if ((this._filteringMgr._currentFilterColumn.type.toLowerCase() == "datetime" || this._filteringMgr._currentFilterColumn.type.toLowerCase() == "date") && values[0] != "" && operators == "equal")
                                this._setDateFilters(filterObject);
                            else
                                this._filters.push(filterObject);
                        }
                    }
                    firstLoop = true;
                }
            }
            this._filteringMgr.model.filterConditions = this._filters;
            this.refreshContent("filtering");
        },

        _commonSuccessAction: function (totalRecordsCount, requestType) {
            if (requestType == "refresh" || requestType == "filtering") {
                this.model.pageSettings.totalRecordsCount = totalRecordsCount;
                var caption = this.model.caption == "" ? "" : this.model.caption + " :";
                this.element.find('.e-m-caption-row span').html(caption + totalRecordsCount + " " + this._localizedLabels["captionText"]);
            }
            if (this._pager != null) {
                if (this._pager.model.currentPage == 0 || this.model.pageSettings.totalRecordsCount <= this.model.pageSettings.pageSize)
                    this._pager._currentPageNo = 1;
                this._pager.model.totalRecordsCount = totalRecordsCount;
                this._pager._findLastPageNo();
                this._pager._refreshPager();
            }
            if (requestType == "filtering") {
                this._mainContainer.css("margin-top", -this.element.find(".e-m-grid-fbardiv").height() - 1 + "px");
            }
            else if (requestType == "sorting") {
                var hdrcell = $(this.element.find(".e-m-grid-hcell"));
                if (this.model.sortSettings.allowMultiSorting) {
                    $(hdrcell[this._currentColIdx])
                        .removeClass("e-m-grid-ascending e-m-grid-descending e-m-clearsorting")
                        .addClass("e-m-grid-" + this._currentSortDirection);
                }
                else {
                    hdrcell.removeClass("e-m-grid-ascending e-m-grid-descending e-m-clearsorting");
                    $(hdrcell[this._currentColIdx])
                       .addClass("e-m-grid-" + this._currentSortDirection);
                }

            }
        },

        _jsonSuccessAction: function (JSONdata, requestType) {
            var proxy = this;
            var records = JSONdata;
            var tbody = null;
            this._curTable = this.element.find(".e-m-grid-content table");
            if (requestType == "paging") {
                //hide pager
                var proxy = this;
                this._pager._refreshPager();
                this._selectionManager._parent = this._curTable;
            }
            tbody = $(this._curTable).find("tbody:first")[0];
            $(tbody).empty();
            if (records && records.length > 0) {
                $(tbody).html($("#" + this.element[0].id + "_JSONTemplate").render(records));
            }
            else {
                var tr = document.createElement('tr');
                var td = document.createElement('td');
                $(td).addClass("e-m-grid-emptycell").attr("colspan", "7").html(this._localizedLabels["emptyResult"]);
                $(tr).append(td);
                $(tbody).append(tr);
            }

            if (this._hiddenColumns.length > 0)
                this.hideColumns(this._colHideList);
            this._handleQueryCell(records);
            this._onRequestSuccess(requestType);
        },

        _formatting: function (formatstring, str, Member) {
            if (typeof (str) == "string" && str.indexOf("Date(") != -1) {
                str = new Date(parseInt(str.replace(/[^0-9 +]/g, '')));
                this.data[Member] = str;
            }
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + ej.format(str, formatVal[2], ej.preferredCulture().name) + RearHtml;
                    else
                        str = ej.format(str, formatVal[2], ej.preferredCulture().name);
                }
                else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            }
            else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            }
            else {
                return this.data.Value;
            }
        },

        _onRequestSuccess: function (requestType) {
            if (this.model.allowSorting && requestType == 'sorting') {
                if (!this.model.sortSettings.allowMultiSorting && this._prevSortedCell)
                    $(this._prevSortedCell).removeClass(this._prevSortDirection.toLowerCase());
                var proxy = this;
                proxy.model.sortSettings.sortedColumns = [];
                $.each(this._sortedColumns, function (index, value) {
                    proxy.model.sortSettings.sortedColumns.push({ columnName: value, sortDirection: proxy._sortDirection[index] });
                });
            }
            var args = { requestType: requestType };
            this._trigger("actionSuccess", args);
            this._prevSortDirection = this._currentSortDirection;
            this._prevSortedCell = this._currentClickedCell;
        },

        _colNavigationHandler: function (e) {
            if (ej.isMobile())
                App.transferPage(ej.getCurrentPage, "#" + this._id + "colnav", { transition: this.model.transition, parentSubPage: true });
            else
                App.transferPage(this.element, "#" + this._id + "colnav", { transition: this.model.transition, parentSubPage: true });
        },

        _touchStartHandler: function (e) {
            var target = this._findColTarget($(e.target));
            if (ej.isWindows())
                ej._touchStartPoints(e, this);
            if (target.hasClass("e-m-grid-col-elem") || (target.hasClass("e-m-icon-grid-clear"))
                 || (this.model.allowColumnSelector && target.hasClass("e-m-grid-colnav"))) {
                $(target).addClass("e-m-state-active");
            }
            ej.listenEvents([this.element], [ej.moveEvent()], [this._touchMoveDelegate], false);
        },

        _touchMoveHandler: function (e) {
            $(e.target.parentElement).hasClass("e-m-state-active") ? $(e.target.parentElement).removeClass("e-m-state-active").addClass("e-m-state-default") : $(e.target).hasClass("e-m-icon-grid-clear e-m-fontimage e-m-state-active") ? $(e.target).removeClass("e-m-state-active") : null;
            if (!ej.isWindows() || (ej.isWindows() && ej._isTouchMoved(e, this)))
                this._touchMove = true;
        },

        _findColTarget: function (target) {
            if (target.hasClass("e-m-grid-hcell") || target.hasClass("e-m-grid-hcelldiv")
                || target.hasClass("e-m-icon-grid-sort") || target.hasClass("e-m-icon-grid-fltr") || target.hasClass("e-m-title"))
                target = target.parent();
            if (target.hasClass("e-m-grid-hcelldiv") || target.hasClass("e-m-title"))
                target = target.parent();
            if (target.hasClass("e-m-grid-hcelldiv"))
                target = target.parent();
            return target;
        },

        _touchEndHandler: function (e) {
            this.element.find(".e-m-grid-col-elem").removeClass("e-m-state-active");
            if (!this._touchMove) {
                if (this.model.allowSelection && this._selectionManager != null)
                    this._selectionManager._selectHandler(e);

                if (this.model.allowColumnSelector && $(e.target).hasClass("e-m-grid-colnav")) {
                    this._colNavigationHandler(e);
                    $(e.target).removeClass("e-m-state-active");
                }
                if ($(e.target).hasClass("e-m-icon-grid-clear"))
                    $(e.target).removeClass("e-m-state-active");
                var target = e.target;
                if (!$(target).hasClass("e-m-text-input")) {
                    e.preventDefault();
                    e.returnValue = false;
                }
                var parent = $(target).parents('table:first');
                if (this._currentColIdx)
                    this._currentClickedCell = $(this.element.find('.e-m-grid-hbar').first().find('th')[this._currentColIdx]).find('div:first').first();
                if (target != null && target.parentNode != null) {
                    if (this._processSortDirection(e, target) && this._currentSortColumn != null) {
                        if (!this.model.sortSettings.allowMultiSorting) {
                            this._sortedColumns = [];
                            this._sortDirection = [];
                        }
                        this._sendSortingRequest(this._currentSortColumn, this._currentSortDirection);
                    }
                    if (this.model.allowFiltering && $(target).hasClass('e-m-icon-grid-fltr')) {
                        this._currentColIdx = $(target).parents('th.e-m-grid-hdrcell')[0].cellIndex;
                        this._filteringMgr._currentFilterColumn = this.model.columns[this._currentColIdx];
                        this._filteringMgr._displayFilterbar();
                    }
                }
                if (!$(e.target).hasClass("e-m-text-input") && !$(e.target).hasClass("e-m-icon-grid-clear") && !$(e.target).hasClass("e-m-icon-grid-fltr") && this.model.allowFiltering) {
                    this._filteringMgr._fDiv.css("visibility", "hidden");
                    $(this.element).find(".e-m-grid-container").css("margin-top", -this._filteringMgr._fDiv.height() - 1 + "px");
                    this._filteringMgr._fBarVisible = false;
                    this._filteringMgr._fInput.blur();
                }
            }
            this._touchMove = false;
            ej.listenEvents([this.element], [ej.moveEvent()], [this._touchMoveDelegate], true);
        },

        _processSortDirection: function (e, target) {
            if (this.model.allowSorting && ($(target).hasClass("e-m-grid-hcelldiv") || $(target).hasClass("e-m-icon-grid-sort") || $(target).hasClass("e-m-title"))) {
                var parNode = $(e.target).parents('th.e-m-grid-hdrcell')[0];
                if ($(parNode).find(".e-m-sort").length > 0) {
                    this._currentColIdx = parNode.cellIndex;
                    var headerDiv = $(parNode).find('div.e-m-grid-col-elem')[0];
                    if (this._currentColIdx >= 0) {
                        this._currentSortColumn = this.getColumnMemberByIndex(this._currentColIdx);
                        if (this._currentSortColumn == null)
                            return false;
                        this._currentSortDirection = $(headerDiv).hasClass("e-m-grid-ascending") ? "descending" :
                                                     ($(headerDiv).hasClass("e-m-grid-descending") ? "clearsorting" : "ascending");
                    }
                    return true;
                }
                else
                    return false;
            } else {
                return false;
            }
        },

        /* ---------------------region Event handling----------------------------------------------*/

        _wireEvents: function (remove) {
            ej.listenEvents([this.element, this.element, this.element], [ej.startEvent(), ej.endEvent()],
                [this._touchStartDelegate, this._touchEndDelegate], remove);
            if (this._filteringMgr && this.model.allowFiltering)
                this._filteringMgr._wireEvents(remove);
            if (this._pager && this.model.allowPaging)
                this._pager._wireEvents(remove);
            var evt = "onorientationchange" in window ? "orientationchange" : "resize";
            ej.listenEvents([window], [evt], [this._orientationChangeDelegate], remove);
        },

        _createDelegates: function () {
            this._touchStartDelegate = $.proxy(this._touchStartHandler, this);
            this._touchEndDelegate = $.proxy(this._touchEndHandler, this);
            this._touchMoveDelegate = $.proxy(this._touchMoveHandler, this);
            this._orientationChangeDelegate = $.proxy(this._orientationChangeHandler, this);
        },
        /* --------------------- end region Event handling----------------------------------------------*/

        _getGridContentTable: function () {
            return this._gridContentTable;
        },

        _orientationChangeHandler: function () {
            if (this.model.scrollSettings.height == "auto") {
                var proxy = this;
                if (!ej.isAndroid())
                    this._resize();
                else {
                    setTimeout(function () {
                        proxy._resize();
                    }, 400);
                }
            }
        },

        _resize: function () {
            var height;
            var container = this.element.find(".e-m-grid-content");
            var winHeihgt = window.innerHeight;
            height = this.element.parent().height() > winHeihgt ? winHeihgt : this.element.parent().height();
            height -= this.element.find('.e-m-grid-header').outerHeight() + this.element.nextAll().outerHeight();
            if (!this.model.allowPaging)
                height = height;
            else {
                height = height - this.element.find(".e-m-grid-pager").height();
            }
            var scrollObj = this._verScroller.ejmScrollPanel("instance");
            scrollObj.model.targetHeight = height;
            scrollObj.refresh();
        },

        _setGridContentTable: function (value) {
            this._gridContentTable = value;
        },

        _hideHeaderColumn: function (hiddenColumns, field) {
            var $table = this.element.find(".e-m-grid-header .e-m-grid-table");
            var $head = $table.find("thead");
            var $headerCell = $head.find(".e-m-grid-hdrcell");
            var $col = $table.find("colgroup").find("col");
            for (var i = 0; i < hiddenColumns.length; i++) {
                var column = this.getColumnByHeaderText(hiddenColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var thIndex = $headerCell.eq(index).addClass("e-hide").index();
                if ($col.length > this.model.columns.length)
                    $col = $col.slice($col.length - this.model.columns.length);
                $col.eq(index).addClass("e-hide");
                $(this._curTable).find("col").eq(index).addClass("e-hide");
            }

        },

        _showHeaderColumn: function (showColumns, field) {
            var $table = this.element.find(".e-m-grid-header .e-m-grid-table");
            var $head = $table.find("thead");
            var $headerCell = $head.find(".e-m-grid-hdrcell");
            var $col = $table.find("colgroup").find("col");
            for (var i = 0; i < showColumns.length; i++) {
                var column = this.getColumnByHeaderText(showColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var thIndex = $headerCell.eq(index).removeClass("e-hide").index();
                $col.eq(index).removeClass("e-hide");
                $(this._curTable).find("col").eq(index).removeClass("e-hide");
            }
        },

        _showHideColumns: function (action, col, colCollection1, colCollection2) {
            var i, count = 0, args = {}, index, column;
            if ($.isArray(col)) {
                for (i = 0; i < col.length; i++) {
                    index = $.inArray(col[i], colCollection2);
                    if (index != -1) {
                        colCollection1.push(col[i]);
                        colCollection2.splice(index, 1);
                    }
                }
            } else {
                index = $.inArray(col, colCollection2);
                if (index != -1) {
                    colCollection1.push(col);
                    colCollection2.splice(index, 1);
                }
            }
            for (i = 0; i < this.model.columns.length; i++) {
                if ($.inArray(this.model.columns[i].headerText, colCollection1) != -1) {
                    var visible = false, classAction = "addClass";
                    if (action == "show") {
                        visible = true;
                        classAction = "removeClass";
                    }
                    this.model.columns[i].visible = visible;
                    $(this._curTable).find("tr td:nth-child(" + (i + 1) + ")")[classAction]("e-hide");
                    count++;
                }
                if (colCollection1.length == count)
                    break;
            }
            var func = action == "hide" ? "_hideHeaderColumn" : "_showHeaderColumn";
            this[func](colCollection1);
        },
        _initColumns: function (object) {
            while (object.items != undefined)
                object = object.items[0];
            if (this.model.columns.length == 0 && object) {
                for (var field in object) {
                    if (object.hasOwnProperty(field) && (typeof (object[field]) != "object" || object[field] instanceof Date)) {
                        var value = object[field];
                        this.model.columns.push({
                            field: field,
                            type: value != null ? (value.getDay ? "date" : typeof (value)) : null
                        });
                    }
                }
            }
            else {
                for (var index = 0; index < this.model.columns.length; index++) {
                    if (ej.isNullOrUndefined(this.model.columns[index].type)) {
                        var $field = object[this.model.columns[index].field];
                        this.model.columns[index].type = $field != null ? ($field.getDay ? "date" : typeof ($field)) : null;
                    }
                }
            }
        },

        _refresh: function () {
            this._destroy();
            this.element.addClass("m-grid");
            this._render();
            this._wireEvents(false);
        },

        _destroy: function () {
            this._wireEvents(true);
            if (this.model.allowColumnSelector) {
                this.element.unwrap();
                $(ej.getCurrentPage().find("#" + this._id + "colnav")).remove();
            }
            if ($(ej.getCurrentPage().find("#" + this._id + "_validationDlg")).length)
                $(ej.getCurrentPage().find("#" + this._id + "_validationDlg")).ejmDialog("destroy").remove();
            if ($(ej.getCurrentPage().find("#" + this._id + "_warningDlg")).length)
                $(ej.getCurrentPage().find("#" + this._id + "_warningDlg")).ejmDialog("destroy").remove();
            if ($(ej.getCurrentPage().find("#" + this._id + "_JSONTemplate")).length) {
                $(ej.getCurrentPage().find("#" + this._id + "_JSONTemplate")).remove();
                this._colInit = false;
            }
            this._verScroller = undefined;
            this._horScroller = undefined;
            this._pageScroller = undefined;
            this.element.empty().removeAttr("class");
            $("#" + this._id + "_pager").remove();
        },
		_setFormat: function () {
            var column = [];
            for (var i = 0 ; i < this.model.columns.length ; i++) {
                if (this.model.columns[i].type == "date" || this.model.columns[i].type == "datetime") {
                    column.push(this.model.columns[i]);
                }
            }
            if (column.length > 0 && !ej.isNullOrUndefined(this.model.currentViewData)) {
                for (var i = 0, len = this.model.currentViewData.length; i < len ; i++) {
                    for (var j = 0 ; j < column.length ; j++) {
                        var data = ej.getObject(column[j].field, this.model.currentViewData[i]);
                        if (/^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*)?)([zZ]|([+\-])(\d\d):?(\d\d))?$/.test(data))
                            ej.createObject(column[j].field, new Date(data), this.model.currentViewData[i]);
                    }
                }
            }
        },

        _processDataSource: function (query, dataManager, requestType) {
            var proxy = this;
            if (this._checkWaitingPopup())
                this.element.ejWaitingPopup("show");
            var process = dataManager.executeQuery(query);
            process.done(function (e) {
                query.queries = [];
                proxy.model.pageSettings.totalRecordsCount = e.count;
                proxy.model.currentViewData = e.result;
				proxy._setFormat();
                if (!proxy._colInit)
                    proxy._addInitialTemplate();
                if (proxy.model.pageSettings.type == "scrollable" && proxy.model.allowPaging && proxy._controlInit)
                    proxy._renderScrollPager();
                if (!proxy.model.totalRecorsCount && proxy.model.allowPaging)
                    proxy._pager._findLastPageNo();
                proxy._afterDataProcess(requestType, e.result);
                if (proxy._verScroller)
                    proxy._verScroller.ejmScrollPanel("refresh");
                if (proxy.model.allowPaging && proxy._pager._lastPNo > 1)
                    proxy._pageScroller.ejmScrollPanel("scrollToElement", proxy._pager.element.find(".e-m-page-num")[parseInt(proxy.currentPage()) - 1], 100, true, true);
                if (proxy._checkWaitingPopup())
                    proxy.element.ejWaitingPopup("hide");
                if (proxy.selectedRowIndex() != -1 && proxy.model.allowSelection && proxy._rowSelect) {
                    proxy._selectionManager._selectRow(proxy.selectedRowIndex());
                    proxy._rowSelect = false;
                }
            });
            process.always(function (e) {
                var args = { requestType: requestType };
                proxy._trigger("actionComplete", args);
            });
            process.fail(function (e) {
                if (proxy._checkWaitingPopup())
                    proxy.element.ejWaitingPopup("hide");
                var args = { requestType: requestType };
                proxy._trigger("actionFailure", args);
            });
        },
        /* Public Functions*/

        refreshContent: function (requestType) {
            if (!requestType)
                requestType = "refresh";
            var args = { requestType: requestType };
            this._trigger("actionBegin", args);
            var proxy = this, dataManager, query;
            if (proxy.model.pageSettings.type == "scrollable" && proxy.model.allowPaging && proxy._pgrTxtContainer) {
                this._pgrTxtContainer.find(".e-m-page-num").removeClass("e-m-state-active");
                $(this._pgrTxtContainer.find(".e-m-page-num")[parseInt(proxy.currentPage()) - 1]).addClass("e-m-state-active");
            }
            if (proxy.selectedRowIndex() != -1)
                proxy._rowSelect = true;
            if (this.dataSource() instanceof ej.DataManager) {
                dataManager = this.dataSource();
                if (dataManager.dataSource.table != null)
                    dataManager.dataSource.table.css("display", "none");
                query = this._createQuery(requestType), columnNames = [];
                $.each(this.model.columns, function (index, object) {
                    columnNames.push(object.field);
                });
                query = query.select(columnNames);
                if (this.dataSource().ready != undefined) {
                    var proxy = this;
                    this.dataSource().ready.done(function (args) {
                        dataManager = ej.DataManager(args.result);
                        proxy._processDataSource(query, dataManager, requestType);
                    });
                }
                else
                    this._processDataSource(query, dataManager, requestType);

            }
            else {
                dataManager = ej.DataManager(this.dataSource());
                query = this._createQuery(requestType);
                this._processDataSource(query, dataManager, requestType);
            }

        },

        scrollRefresh: function () {
            if (this._verScroller)
                this._verScroller.ejmScrollPanel('refresh');
            if (this._horScroller)
                this._horScroller.ejmScrollPanel('refresh');
            if (this._pageScroller)
                this._pageScroller.ejmScrollPanel('refresh');
        },

        getColumnByHeaderText: function (headerText) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["headerText"] == headerText)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },

        getColumnByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },

        getColumnByIndex: function (index) {
            if (index < this.model.columns.length)
                return this.model.columns[index];
            return null;
        },

        getColumnIndexByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column;
        },

        getColumnMemberByIndex: function (colIdx) {
            if (colIdx < 0)
                return null;
            else if (this.model.columns.length > colIdx)
                return this.model.columns[colIdx].field;
            else
                return null;
        },

        getColumnFieldNames: function () {
            var columnNames = [];
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"])
                    columnNames.push(this.model.columns[column]["field"]);
            }
            return columnNames;
        },

        hideColumns: function (col) {
            this._showHideColumns("hide", col, this._hiddenColumns, this._visibleColumns);
        },

        showColumns: function (col) {
            this._showHideColumns("show", col, this._visibleColumns, this._hiddenColumns);
        },

        /* End Public function */

        _setModel: function (options) {
            var refresh = false;
            var updateDB = false;
            var refreshPage = false;
            for (var prop in options) {
                if (typeof (options[prop]) == "object" && prop != "dataSource" && prop != "columns")
                    $.each(options[prop], function (propName) {
                        prop = propName;
                    });
                switch (prop) {
                    case "locale":
                        this._setLocale(options.locale);
                        refresh = true;
                        break;
                    case "renderMode":
                        this.model.renderMode = options.renderMode;
                        this._setRenderMode();
                        break;
                    case "pageSize":
                        this._pager.model.pageSize = this.model.pageSettings.pageSize;
                        refreshPage = true;
                        break;
                    case "currentPage":
                        if (this.model.allowPaging) {
                            if (ej.isNullOrUndefined(options[prop]) || this._pager.model.currentPage != parseInt(this.currentPage())) {
                                this._pager.model.currentPage = parseInt(this.currentPage());
                            }
                            refreshPage = true;
                        }
                        break;
                    case "display":
                    case "type":
                        if (this.model.allowPaging) {
                            $.extend(this.model.pageSettings, options[prop]);
                            this._pager.model.display = this.model.pageSettings.display;
                            this._pager.model.type = this.model.pageSettings.type;
                            refresh = true;
                        }
                        break;
                    case "dataSource":
                        updateDB = true;
                        break;
                    case "allowPaging":
                    case "allowFiltering":
                    case "allowSorting":
                    case "allowScrolling":
                    case "allowColumnSelector":
                    case "height":
                    case "columns":
                        refresh = true;
                        break;
                    case "sortedColumns":
                        if (this.model.allowSorting) {
                            this._sortedColumns = [];
                            this._sortDirection = [];
                            this._handleInitialSorting();
                            this.refreshContent("sorting");
                        }
                        break;
                    case "filteredColumns":
                        if (this.model.allowFiltering) {
                            this._filters = options.filterSettings.filteredColumns;
                            this.refreshContent("filtering");
                        }
                        break;
                    case "showCaption":
                        if (this.model.showCaption)
                            this._renderCaption();
                        else
                            this.element.find(".e-m-caption-row").remove();
                        break;
                    case "caption":
                        this.element.find(".e-m-caption-row span").html(this.model.caption + ": " + this.dataSource().length + " " + this._localizedLabels["captionText"]);
                        break;
                    case "cssClass":
                        this.element.addClass(this.model.cssClass).removeClass(this._cssClass);
                        this._cssClass = this.model.cssClass;
                        break;
                    case "selectedRowIndex":
                        if (this.model.allowSelection) {
                            this._selectionManager._selectRow(this.selectedRowIndex());
                            this._rowSelect = false;
                        }
                        break;
                }

            }
            if (refresh) {
                this._controlInit = true;
                if (this._verScroller) {
                    this._verScroller.remove();
                    this._verScroller = null;
                }
                if (this._horScroller) {
                    this._horScroller.remove();
                    this._horScroller = null;
                }
                if (this._pageScroller) {
                    this._pageScroller.remove();
                    this._pageScroller = null;
                }
                this._refresh();
            }
            else if (updateDB)
                this.refreshContent("refresh");
            else if (refreshPage)
                this.refreshContent("paging");
            //this._trigger("modelChange", { "changes": options });
        },
        /* End Set Model */

    });

})(jQuery, Syncfusion);




(function ($, ej, undefined) {
    // ejmGridFilter is the plugin name 
    // "ej.mobile.GridFilter" is "namespace.className" will hold functions and properties

    ej.widget("ejmGridFilter", "ej.mobile.GridFilter", {
        _rootCSS: "e-m-filter",

        _init: function () {
            this.page = ej.getCurrentPage();
            this._fDiv = this.page.find("#" + this.model.id + ' .e-m-grid-fbardiv');
            this._fInput = $(this._fDiv).find('input');
            this._currentFilterColumn = null;
            this.predicate = "or";
            this._filterValue = "";
            this._oldFilterValue = "";
            this._operatorValue = new Array();
            this._filterValueOldLenth = 0;
            this._filterValueCurrentLenth = 0;
            this._currentFilterbarValue = null;
            this._alreadyFilterProcessed = false;
            this._oldFilterColumn = null;
            this._skipFilterProcess = false;
            this._tokens = new Array("%", "<", "<=", ">", ">=", "!", "=", "*");
            this._tokenNames = new Array("startswith", "lessthan", "lessthanorequal", "greaterthan", "greaterthanorequal", "notequal", "equal", "contains");
            this._tokenType = null;
            this._fltrValCollection = new Array();
            this._colFltrValues = [];
        },

        _wireEvents: function (remove) {
            var event = remove ? "off" : "on";
            var clear = $(this._fDiv).find('.e-m-icon-grid-clear');
            var form = $(this._fDiv).find('form');
            ej.listenEvents([clear, form], [ej.endEvent(), "submit"], [$.proxy(this._clearHandler, this), $.proxy(this._filterBarValueHandler, this)], remove);
            if (this.model.filterBarMode == "immediate")
                ej.listenEvents([this._fInput], ["keyup"], [$.proxy(this._filterBarValueHandler, this)], remove);
        },

        _clearHandler: function (e) {
            this._fInput.val("");
            this._fInput.focus();
        },

        _displayFilterbar: function () {
            if (!this._fBarVisible) {
                this._fInput.val("");
                if (this._colFltrValues[this._currentFilterColumn.field] != null)
                    this._fInput.val(this._colFltrValues[this._currentFilterColumn.field]);
                else if (this.model.filterConditions) {
                    var predicate, value = "", proxy = this;
                    $.each(this.model.filterConditions, function (index, val) {
                        if (val.columnName == proxy._currentFilterColumn.field) {
                            if (predicate)
                                value = value + " " + (predicate == "or" ? "||" : "&&") + " ";
                            if (val.operator == "startswith" || val.operator == "endswith")
                                value = value + (val.operator == "startswith" ? val.value + "%" : "%" + val.value);
                            else {
                                var index = $.inArray(val.operator, proxy._tokenNames);
                                index = index == -1 ? 1 : index;
                                value = value + proxy._tokens[index] + val.value;
                            }
                            predicate = val.predicate;
                        }
                    });

                }
                var proxy = this;
                var value = proxy._fInput.val();
                this._fInput.attr("placeholder", this._currentFilterColumn.headerText);
                this._fDiv.css("visibility", "visible");
                $("#" + this.model.id).find(".e-m-grid-container").css("margin-top", "0px");
                this._fBarVisible = true;
            }
            else {
                $("#" + this.model.id).find(".e-m-grid-container").css("margin-top", -this._fDiv.height() - 1 + "px");
                this._fBarVisible = false;
                this._fInput.blur();
            }
        },

        _filterBarValueHandler: function (e) {
            e.preventDefault();
            if (!(e.type == "submit" && this.model.filterBarMode == "immediate")) {
                var keycode = e.keyCode;
                if (keycode != 9) {
                    if (this._currentFilterColumn == null)
                        return;
                    if (this._currentFilterColumn != this._oldFilterColumn)
                        this._filterValueOldLenth = 0;

                    this._currentFilterbarValue = $(this.page.find("#" + this.model.id + " input")).val();
                    this._colFltrValues[this._currentFilterColumn.field] = this._currentFilterbarValue;
                    this._filterValueCurrentLenth = this._currentFilterbarValue.length;
                    if (((this._filterValueCurrentLenth == 0 && this._filterValueOldLenth == 0) || this._currentFilterbarValue == this._oldFilterValue) && this._currentFilterColumn == this._oldFilterColumn)
                        return false;
                    this._skipFilterProcess = this._checkForSkipInput();
                    if (!this._skipFilterProcess) {
                        this._checkFilterProcess();
                    }
                    else {
                        this._skipFilterProcess = false;
                        return false;
                    }
                }
            }
            return false;
        },

        _createValidationDialog: function () {
            this._validationDlg = ej.buildTag("div")
                .append(ej.buildTag("div.e-m-filter-validate").html(this._gridObj._localizedLabels["filterValidation"]));
            this._validationDlg.attr("id", this.model.id + "_validationDlg");
            this.element.append(this._validationDlg);
            this._validationDlg.ejmDialog({ leftButtonCaption: this._gridObj._localizedLabels["filterOk"], renderMode: this._gridObj.model.renderMode, title: this._gridObj._localizedLabels["filterWarning"], mode: "alert", enableAutoOpen: false, enableModal: true });
            this._validationDlg.ejmDialog({ "buttonTap": ej.proxy(this._OkBtnHandler, this) });
        },

        _OkBtnHandler: function (e) {
            this._validationDlg.ejmDialog("close");
        },

        _checkFilterProcess: function () {
            if (this.model.filterBarMode == "immediate") {
                if (!this._alreadyFilterProcessed) {
                    this._alreadyFilterProcessed = true;
                    this._startTimer();
                } else {
                    this._stopTimer();
                    this._startTimer();
                }
            }
            else
                this._processFilter();
        },

        _startTimer: function () {
            var proxy = this;
            this._timer = window.setTimeout(
                function () {
                    proxy._processFilter();
                },
                proxy.model.interval);
        },

        _stopTimer: function () {
            this._oldFilterColumn = this._currentFilterColumn;
            this._filterValueOldLenth = this._filterValueCurrentLenth;
            if (this._timer != null)
                window.clearTimeout(this._timer);
        },

        _processFilter: function () {
            this._findPredicate();
            this._fInput.blur();
            if (!this._gridObj)
                this._gridObj = $("#" + this.model.id).ejmGrid("instance");
            if (!this._skipFilterProcess && this._fltrValCollection.length > 0) {
                this._gridObj._sendFilteringRequest(this._operatorValue, this._fltrValCollection, this.predicate);
                this._fBarVisible = (this._fBarVisible) ? false : true;
            }
            else {
                if (!this._validationDlg)
                    this._createValidationDialog();
                this._validationDlg.find(".e-m-filter-validate").html(this._gridObj._localizedLabels["filterTypeValidation"] + " \ " + this._currentFilterColumn.type + "\ ");
                this._validationDlg.ejmDialog("open");
            }
            this._oldFilterValue = this._filterValue;
            this._filterValue = "";
            //clearing the array collection
            this._fltrValCollection = new Array();
            this._operatorValue = new Array();
            this._tokenType = null;
        },

        _findPredicate: function () {
            var _value = this._currentFilterbarValue.replace(/ and /i, " AND ").replace(/ or /i, " OR ").replace(/ && /i, " AND ").replace(" || ", " OR ");
            var _predicateFinder = _value.split(' ');
            if (_predicateFinder.length != 0) {
                if (_predicateFinder[1] == "AND") {
                    this._skipFilterProcess = false;
                    this.predicate = "and";
                    var valuesArray = _value.split(" AND ");
                    this._validator(valuesArray);
                }
                else if (_predicateFinder[1] == "OR") {
                    this._skipFilterProcess = false;
                    this.predicate = "or";
                    var valuesArray = _value.split(" OR ");
                    this._validator(valuesArray);
                }
                else
                    this._validateFilterValue(this._currentFilterbarValue.trim());
            }
            else
                this._validateFilterValue(this._currentFilterbarValue.trim());
        },

        _validator: function (valuesArray) {
            var context = this;
            $.each(valuesArray, function (indx, val) {
                //to check whether the expressions has some values
                if (val != "") {
                    context._validateFilterValue(val, context._currentFilterColumn.type);
                    context._filterValue = "";
                }
                else {
                    context._skipFilterProcess = true;
                    //clearing the array collection
                    context.filterValueCollection = new Array();
                    context._operatorValue = new Array();
                    context.TokenType = null;
                }
            });
        },

        _checkNumericDataType: function () {
            if (this._currentFilterColumn.type && (this._currentFilterColumn.type.toLowerCase() == "number" || this._currentFilterColumn.type.toLowerCase() == "double" || this._currentFilterColumn.type.toLowerCase() == "decimal" || this._currentFilterColumn.type.toLowerCase() == "datetime" || this._currentFilterColumn.type.toLowerCase() == "date")) {
                return true;
            }
        },

        _checkForSkipInput: function () {
            var IsSkip = false;
            var skipInput = new Array(">", "<", "=", "!", " ");
            var context = this;
            if (this._checkNumericDataType()) {
                $.each(skipInput, function (index, value) {
                    if (value == context._currentFilterbarValue)
                        IsSkip = true;
                });
            }
            if (this._currentFilterColumn.type && this._currentFilterColumn.type.toLowerCase() == "string") {
                var stringSkipInput = new Array(">", "<", "=", "!");
                for (var str = 0; str < this._currentFilterbarValue.length ; str++) {
                    if (jQuery.inArray(this._currentFilterbarValue[str], stringSkipInput) != -1)
                        IsSkip = true;
                }
            }
            return IsSkip;
        },

        _validateFilterValue: function (_value) {
            this._findFilterValue(_value);
            if ((this._tokenType == null || this._tokenType == undefined) && (this._currentFilterColumn.type && this._currentFilterColumn.type.toLowerCase() == "string"))
                this._operatorValue.push(ej.FilterOperators.startsWith);
            else if ((this._tokenType == null || this._tokenType == undefined) && (this._checkNumericDataType()))
                this._operatorValue.push(ej.FilterOperators.equal);
            if (this._currentFilterColumn.type && (this._currentFilterColumn.type.toLowerCase() == "datetime" || this._currentFilterColumn.type.toLowerCase() == "date")) {
                var _cArr;
                if (this._currentFilterColumn.format == "")
                    _cArr = ej.preferredCulture().calendar.patterns.d; //System Date format
                else
                    _cArr = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                if (this._filterValue != "")
                    this._filterValue = ej.parseDate(this._filterValue, _cArr);

            }

            //to validate for spaces, <,<=,etc in between the values
            if (this._checkNumericDataType()) {
                var numericSkip = new Array(">", "<", "=", "!", " ");
                for (var i = 0; i < this._filterValue.length; i++) {
                    char = this._filterValue.charAt(i);
                    if (jQuery.inArray(char, numericSkip) != -1)
                        this._skipFilterProcess = true;
                }
                if (this._currentFilterColumn.type && this._currentFilterColumn.type.toLowerCase() != "datetime" && this._currentFilterColumn.type.toLowerCase() != "date" && this._filterValue != "")
                    this._filterValue = parseFloat(this._filterValue);
            }
            if ((this.model.filterConditions.length > 0 && this._filterValue == "") || (this.model.filterConditions.length >= 0 && this._filterValue != ""))
                this._fltrValCollection.push(this._filterValue);
            //if operator not mentioned in second expression, set it to '=' operator
            if (this._operatorValue.length != this._fltrValCollection.length)
                this._operatorValue.push(ej.FilterOperators.equal);
        },

        _findOperators: function (isEmptyText, _value) {
            switch (_value) {
                case "%":
                    if (isEmptyText) {
                        this._tokenType = "startsWith";
                        this._operatorValue.push(ej.FilterOperators.startsWith);
                    }
                    else {
                        this._tokenType = "endsWith";
                        this._operatorValue.push(ej.FilterOperators.endsWith);
                    }
                    break;
                case "*":
                    this._tokenType = "contains";
                    this._operatorValue.push(ej.FilterOperators.contains);
                    break;
                case "<":
                    this._tokenType = "lessThan";
                    this._operatorValue.push(ej.FilterOperators.lessThan);
                    break;
                case "<=":
                    this._tokenType = "lessThanOrEqual";
                    this._operatorValue.push(ej.FilterOperators.lessThanOrEqual);
                    break;
                case "=":
                    this._tokenType = "equal";
                    this._operatorValue.push(ej.FilterOperators.equal);
                    break;
                case "!":
                    this._tokenType = "notEqual";
                    this._operatorValue.push(ej.FilterOperators.notEqual);
                    break;
                case ">=":
                    this._tokenType = "greaterThanOrEqual";
                    this._operatorValue.push(ej.FilterOperators.greaterThanOrEqual);
                    break;
                case ">":
                    this._tokenType = "greaterThan";
                    this._operatorValue.push(ej.FilterOperators.greaterThan);
                    break;
            }
        },

        _checkNumeric: function (value) {
            var ValidChars = "0123456789.<>=!";
            var isNumeric = true;
            if (this._currentFilterColumn.type && (this._currentFilterColumn.type.toLowerCase() == "datetime" || this._currentFilterColumn.type.toLowerCase() == "date"))
                ValidChars = "0123456789.AMP<>=!/: ";
            for (var i = 0; i < value.length; i++) {
                char = value.charAt(i);
                if (jQuery.inArray(char, ValidChars) == -1) {
                    isNumeric = false;
                }
            }
            return isNumeric;
        },

        _findFilterValue: function (_value) {
            if (_value != "") {
                if (this._currentFilterColumn.type && this._currentFilterColumn.type.toLowerCase() == "string")
                    this._getToken(_value);
                else if (this._currentFilterColumn.type && this._currentFilterColumn.type.toLowerCase() == "boolean" && _value.length == 1) {
                    var boolSkipInPut = "01";
                    if (jQuery.inArray(_value, boolSkipInPut) != -1) {
                        if (_value == "1")
                            this._filterValue = "True";
                        else if (_value == "0")
                            this._filterValue = "False";
                        this._operatorValue.push(ej.FilterOperators.equal);
                    }
                    else
                        this._skipFilterProcess = true;
                }
                else if (this._checkNumericDataType()) {
                    var hasNumeric = this._checkNumeric(_value);
                    if (hasNumeric)
                        this._getToken(_value);
                    else
                        this._skipFilterProcess = true;
                }
                else
                    this._skipFilterProcess = true;
            }
        },

        _getToken: function (filterStr) {
            if (filterStr == null || filterStr == undefined)
                this._filterValue = "";
            var firstBlock = filterStr.split('');
            this._getTokens(firstBlock, firstBlock[0]);
        },

        _getTokens: function (remainingChars, filterFirstChar) {
            if (this._isToken(filterFirstChar)) {
                this._findOperators(this._filterValue != "", filterFirstChar);

                if (this._tokenType == "lessThan" || this._tokenType == "greaterThan") {
                    if (remainingChars.length > 1) {
                        remainingChars.splice(0, 1);
                        var nextBlock = remainingChars;
                        var nextChar = nextBlock[0];
                        if (this._isToken(nextChar)) {
                            var tokenType = this._tokenType;
                            this._findOperators(false, nextChar);
                            if (this._tokenType == "equal") {
                                if (tokenType == "lessThan") {
                                    this._tokenType = "lessThanOrEqual";
                                    var operatorlength = this._operatorValue.length;
                                    for (var i = this._operatorValue.length; i > operatorlength - 2; i--)
                                        this._operatorValue.splice(i - 1, 1);
                                    this._operatorValue.push(ej.FilterOperators.lessThanOrEqual);
                                }
                                else if (tokenType == "greaterThan") {
                                    this._tokenType = "greaterThanOrEqual";
                                    var operatorlength = this._operatorValue.length;
                                    for (var i = this._operatorValue.length; i > operatorlength - 2; i--)
                                        this._operatorValue.splice(i - 1, 1);
                                    this._operatorValue.push(ej.FilterOperators.greaterThanOrEqual);
                                }
                            }
                            this._skipAndTokenizeBlock(nextBlock);
                        }
                        else {
                            this._getFilterValue(nextBlock, nextChar);
                        }
                    }

                }
                else {
                    if (remainingChars.length > 1) {
                        remainingChars.splice(0, 1);
                        var nextBlock = remainingChars;
                        var nextChar = nextBlock[0];
                        this._getFilterValue(nextBlock, nextChar);
                    }
                }
            }
            else {
                this._getFilterValue(remainingChars, filterFirstChar);
            }

        },

        _getFilterValue: function (nextBlock, nextChar) {
            this._filterValue += nextChar;
            this._skipAndTokenizeBlock(nextBlock);
        },
        _skipAndTokenizeBlock: function (remainingChars) {
            if (remainingChars.length > 1) {
                remainingChars.splice(0, 1);
                var nextBlock = remainingChars;
                var nextChar = nextBlock[0];
                this._getTokens(nextBlock, nextChar);
            }
        },

        _isToken: function (filterchar) {
            if (jQuery.inArray(filterchar, this._tokens) == -1)
                return false;
            else
                return true;
        },
    });

})(jQuery, Syncfusion);


(function ($, ej, undefined) {
    // ejmGridPager is the plugin name 
    // "ej.mobile.GridPager" is "namespace.className" will hold functions and properties

    ej.widget("ejmGridPager", "ej.mobile.GridPager", {
        _rootCSS: "e-m-grid-core",
        _init: function () {
            this._page = ej.getCurrentPage();
            this._findLastPageNo();
            this._pElement = this._page.find("#" + this.model.id + "_pager");
            this._pLast = this._pElement.find(".e-m-grid-last");
            this._pFirst = this._pElement.find(".e-m-grid-first");
            this._pPrev = this._pElement.find(".e-m-grid-prev");
            this._pNext = this._pElement.find(".e-m-grid-next");
            this._pTxt = this._pElement.find(".e-m-grid-ptxt");
        },
        /* ---------------------region event binding----------------------------------------------*/

        _wireEvents: function (remove) {
            var elements = this._pElement.find(".e-m-grid-inrpgr >div");
            ej.listenEvents([elements, elements], [ej.endEvent(), ej.startEvent()],
                [$.proxy(this._pageHandler, this), $.proxy(this._tStartHandler, this)], remove);
            if (this.model.type == ej.mobile.Grid.PagerType.Scrollable) {
                ej.listenEvents([this._pElement.find(".e-m-grid-ptxt")], [ej.endEvent()],
               [$.proxy(this._scrollPageHandler, this)], remove);
            }
        },

        /*----------------------- End Region event binding---------------------------------------------*/

        _findLastPageNo: function () {
            if (this.model.totalRecordsCount % this.model.pageSize > 0)
                this._lastPNo = parseInt(this.model.totalRecordsCount / this.model.pageSize) + 1;
            else
                this._lastPNo = parseInt(this.model.totalRecordsCount / this.model.pageSize);
        },

        _isPageBtn: function (target) {
            var canhlight = (target.hasClass("e-m-grid-next") || target.hasClass("e-m-grid-prev") || target.hasClass("e-m-grid-last") || target.hasClass("e-m-grid-first")) ? true : false;
            var isdisabled = (target.hasClass("e-m-grid-dnext") || target.hasClass("e-m-grid-dprev") || target.hasClass("e-m-grid-dlast") || target.hasClass("e-m-grid-dfirst")) ? true : false;
            return canhlight && !isdisabled;
        },

        _tStartHandler: function (e) {
            var target = $(e.target);
            var className = "";
            if (target.hasClass("e-m-grid-next") && !target.hasClass("e-m-grid-dnext"))
                className = "hnext";
            else if (target.hasClass("e-m-grid-prev") && !target.hasClass("e-m-grid-dprev"))
                className = "hprev";
            else if (target.hasClass("e-m-grid-last") && !target.hasClass("e-m-grid-dlast"))
                className = "hlast";
            else if (target.hasClass("e-m-grid-first") && !target.hasClass("e-m-grid-dfirst"))
                className = "hfirst";
            var element = this._pElement.find(".e-m-grid-" + className);
            if (element.length > 0)
                element.css("display", "block").addClass("e-m-state-active");
        },

        _pageHandler: function (e) {
            if (!this._tMove)
                this._sendPageRequest($(e.target));
        },

        _scrollPageHandler: function (e) {
            this._gridObj = !this._gridObj ? $("#" + this.model.id).ejmGrid("instance") : this._gridObj;
            if (!this._gridObj._touchMove) {
                if ($(e.target).hasClass("e-m-page-num")) {
                    var page = parseInt(this.model.currentPage);
                    this.model.currentPage = parseInt($(e.target).html());
                    if (page != this.model.currentPage) {
                        this._gridObj.currentPage(this.model.currentPage);
                        this._gridObj.refreshContent("paging");
                    }
                }
            }
        },

        _sendPageRequest: function (target) {
            var page = this.model.currentPage, className = "";
            if ((target.hasClass("e-m-grid-hnext") || target.hasClass("e-m-grid-next")) && !target.hasClass("e-m-grid-dnext")) {
                this.model.currentPage = this.model.currentPage + 1;
                className = "hnext";
            }
            else if ((target.hasClass("e-m-grid-hprev") || target.hasClass("e-m-grid-prev")) && !target.hasClass("e-m-grid-dprev")) {
                this.model.currentPage = this.model.currentPage - 1;
                className = "hprev";
            }
            else if ((target.hasClass("e-m-grid-hlast") || target.hasClass("e-m-grid-last"))) {
                this.model.currentPage = this._lastPNo;
                className = "hlast";
            }
            else if ((target.hasClass("e-m-grid-hfirst") || target.hasClass("e-m-grid-first"))) {
                this.model.currentPage = 1;
                className = "hfirst";
            }
            var element = this._pElement.find(".e-m-grid-" + className);
            if (element.length > 0)
                element.css("display", "none").removeClass("e-m-state-active");
            if (!this._gridObj)
                this._gridObj = $("#" + this.model.id).ejmGrid("instance");
            if (page != this.model.currentPage) {
                this._gridObj.currentPage(this.model.currentPage);
                this._gridObj.refreshContent("paging");
            }
            this._tMove = false;
        },

        _refreshPager: function () {
            if (this.model.type == ej.mobile.Grid.PagerType.Normal) {
                this._pLast.removeClass("e-m-grid-dlast");
                this._pNext.removeClass("e-m-grid-dnext");
                this._pFirst.removeClass("e-m-grid-dfirst");
                this._pPrev.removeClass("e-m-grid-dprev");
                if (this.model.currentPage == this._lastPNo) {
                    this._pLast.addClass("e-m-grid-dlast");
                    this._pNext.addClass("e-m-grid-dnext");
                }
                if (this.model.currentPage == 1) {
                    this._pFirst.addClass("e-m-grid-dfirst");
                    this._pPrev.addClass("e-m-grid-dprev");
                }
                this._pTxt.html(this.model.currentPage);
            }
            else if (this.model.type == ej.mobile.Grid.PagerType.Scrollable) {
                if (!this._gridObj)
                    this._gridObj = $("#" + this.model.id).ejmGrid("instance");
                this._gridObj._updateScrollPager();
            }
        },

    });

})(jQuery, Syncfusion);

(function ($, ej, undefined) {
    // ejmGridSelectionManager is the plugin name 
    // "ej.mobile.GridSelectionManager" is "namespace.className" will hold functions and properties

    ej.widget("ejmGridSelectionManager", "ej.mobile.GridSelectionManager", {
        _rootCSS: "e-m-grid-core",

        _init: function () {
            this._record = {};
        },

        _selectRow: function (index, target) {
            var targetRow = this.element.find(".e-m-grid-content tr")[index];
            this._cleanUp();
            this._selectedRowIndex = targetRow;
            if (targetRow) {
                this._setCurrentRecord(targetRow.rowIndex);
                this._gridObj.selectedRowIndex(index);
                var args = { rowIndex: targetRow.rowIndex, row: targetRow, cell: target, data: this._record };
                this._gridObj._trigger("rowSelecting", args);
                $(targetRow).find("td.e-m-grid-rowcell").addClass("e-m-state-active");
                this._gridObj._trigger("rowSelected", args);
            }
        },

        _setCurrentRecord: function (index) {
            if (!this._gridObj)
                this._gridObj = this.element.ejmGrid("instance");
            this._record = this._gridObj.model.currentViewData[index];
        },

        _getRow: function (target) {
            return $(target).parents('tr:first')[0];
        },

        _getRowCell: function (target) {
            if (target.tagName == 'TD' && $(target).hasClass('e-m-grid-rowcell')) {
                target = $(target);
            }
            else if ($(target).parents('td.e-m-grid-rowcell:first').length > 0) {
                target = $(target).parents('td.e-m-grid-rowcell:first');
            }
            else {
                target = $($(target).parents('tr:first')[0]).find('.e-m-grid-rowcell');
            }
            return target;
        },

        _getAllRowCells: function (target) {
            return $(target.parentNode).children('td:[class*=e-m-grid-rowcell]');
        },

        _deSelectRow: function () {
            if (this._selectedRowIndex != null) {
                this._setCurrentRecord(this._selectedRowIndex);
                var args = { row: this._selectedRowIndex, record: this._record };
                //single cell deselection 
                $(this._selectedCell).removeClass("e-m-grid-current-cell");
                //e-m-grid-rowcell deselection 
                $(this._selectedRowIndex).find("td.e-m-grid-rowcell").removeClass("e-m-state-active");
            }
        },

        _cleanUp: function () {
            if (this._selectedRowIndex)
                this._deSelectRow();
        },

        /*----------------------- End Region internal functions---------------------------------------------*/
        _selectHandler: function (e) {
            var target = this._getRowCell(e.target);
            var targetRow = this._getRow(e.target);
            var parent = this.element.find(".e-m-grid-content table:first")[0];
            // Used to select Only One Cell When MultiSelection Property is true
            if (target == null || target.length < 1 || !$(target).hasClass('e-m-grid-rowcell') || $(target).parents("table:first")[0] != parent) {
                return;
            }
            this._selectRow(targetRow.rowIndex, target[0]);
        },
        /* ---------------------EndRegion handlers----------------------------------------------*/

    });

    ej.mobile.Grid.PagerDisplay = {
        Normal: "normal",
        Fixed: "fixed"
    };

    ej.mobile.Grid.PagerType = {
        Normal: "normal",
        Scrollable: "scrollable"
    };
    ej.mobile.Grid.Actions = {
        Paging: "paging",
        Sorting: "sorting",
        Filtering: "filtering",
        Refresh: "refresh"
    };

    ej.mobile.Grid.FilterBarMode = {
        Immediate: "immediate",
        OnEnter: "onenter"
    };
    ej.mobile.Grid.Locale = ej.mobile.Grid.Locale || {};
    ej.mobile.Grid.Locale["default"] = ej.mobile.Grid.Locale["en-US"] = {
        emptyResult: "No records to display",
        filterValidation: "Enter valid filter data",
        filterTypeValidation: "Enter valid filter data. The current filter column is of type",
        captionText: "Items",
        spinnerText: "loading...",
        HideColumnAlert: "Atleast one column must be displayed in grid",
        columnSelectorText: "Hide Column",
        columnSelectorDone: "OK",
        columnSelectorCancel: "Cancel",
        columnSelectorWarning: "Warning",
        filterOk: "Ok",
        filterWarning: "Warning"
    }
})(jQuery, Syncfusion);;

ej.Ej3DRender = function () {

   
};

(function ($) {
    
    ej.Ej3DRender.Polygons = [];

    
    ej.Ej3DRender.prototype = {
        matrix3D: function () {
            
        },

        vector3D: function () {
             
        },
       
        BSPTreeBuilder: function () {

        },

        Graphics3D: function () {

        },

        polygon3D: function () {

        }
    }

    var Ej3DRender = new ej.Ej3DRender();
    
    Ej3DRender.vector3D.prototype = {
        //Members
        x: "",
        y: "",
        z: "",
        Epsilon:0.00001,
        //all coordinates indicate 0
        //  empty: new Vector3D(0, 0, 0),

        //isValid: !isNaN(this.x) && !isNaN(this.y) && !isNaN(this.z),
        isValid: function (point) {
            return !isNaN(point.x) && !isNaN(point.y) && !isNaN(point.z)
        },
        //Constructor
        vector3D: function (points, vz) {
            this.x = points.x;
            this.y = points.y;
            this.z = vz;
        },

        vector3D: function (vx, vy, vz) {
            this.x = vx;
            this.y = vy;
            this.z = vz;
            return {x:vx, y:vy, z:vz}
        },

        //operations
        vector3DMinus: function (v1, v2) {
            return this.vector3D(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
        },

        vector3DPlus: function (v1, v2) {
            return this.vector3D(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
        },

        vector3DMultiply: function (v1, v2) {
            var _x = v1.y * v2.z - v2.y * v1.z;
            var _y = v1.z * v2.x - v2.z * v1.x;
            var _z = v1.x * v2.y - v2.x * v1.y;

            return this.vector3D(_x, _y, _z);
        },

        vector3dAND: function (v1, v2) {
            return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
        },

        vector3DStarMultiply: function (v1, val) {
            var _x = v1.x * val;
            var _y = v1.y * val;
            var _z = v1.z * val;

            return this.vector3D(_x, _y, _z);
        },

        //Methods
        getLength: function (vector) {
            var sqt = this.vector3dAND(vector, vector);
            return Math.sqrt(sqt);
        },

       

        normalize: function () {
            var l = this.getLength();
            x /= l;
            y /= l;
            z /= l;
        },
        getNormal:function(v1, v2, v3)
        {
            var v4 = this.vector3DMinus(v1, v2);
            var v5 = this.vector3DMinus(v3, v2);
            var n = this.vector3DMultiply(v4, v5);
            var l = this.getLength(n);

           if (l < this.Epsilon)
           {
              l = 0;
            }
           return { x: n.x/ l, y: n.y / l, z: n.z / l }
        }
    }

   Ej3DRender.matrix3D.prototype = {
       //Memebr
        MATRIX_SIZE: 4,
       //Constructor
        matrix3D: function (size) {
            var mData = [];
           for (var i = 0; i < size; i++) {
               mData[i] = this.createArray(size);
           }
           return mData;
        },

       //method
        isAffine: function (mData) {

            return (mData[0][3] == 0) && (mData[1][3] == 0)
                    && (mData[2][3] == 0) && (mData[3][3] == 1);
        },
       createArray: function (initialSize) {
           var a = [];
           for (index = 0; index < initialSize; ++index) {
               a[index] = 0;
           }
           return a;
       },

       getIdentity: function () {

           var mData=this.matrix3D(this.MATRIX_SIZE);

           for (var i = 0; i < this.MATRIX_SIZE; i++) {
               mData[i][i] = 1.0;
           }
          
           return mData;
       },

      getInvertal:function(matrix)
      {
         var m = this.getIdentity();

         for (var i = 0; i < this.MATRIX_SIZE; i++)
         {
           for (var j = 0; j < this.MATRIX_SIZE; j++)
           {
               m[i][j] = this.getMinor(matrix, i, j);
            }
         }

         m = this.transposed(m);
         m = this.getMatrixMultiple((1 / this.getDeterminant(matrix)) , m);

      return m;
      },
      getMatrixMultiple: function (factor, matrix) {
          for (var i = 0; i < matrix.length; i++) {
              for (var j = 0; j < matrix[i].length; j++) {
                  matrix[i][j] = matrix[i][j] * factor;
              }
          }
          return matrix;
      },
      getMatrixVectorMutiple:function(m1, point)
      {
            var x = m1[0][0] * point.x + m1[1][0] * point.y + m1[2][0] * point.z + m1[3][0];
            var y = m1[0][1] * point.x + m1[1][1] * point.y + m1[2][1] * point.z + m1[3][1];
            var z = m1[0][2] * point.x + m1[1][2] * point.y + m1[2][2] * point.z + m1[3][2];

      if (!this.isAffine(m1))
      {
         var c = 1 / (m1[0][3] * point.x + m1[1][3] * point.y + m1[2][3] * point.z + m1[3][3]);
         x *= c;
         y *= c;
         z *= c;
      }

       return {x:x, y:y, z:z};
      },
      getMatrixVectorAnd:function(m1, v1)
      {
            var x = m1[0][0] * v1.x + m1[1][0] * v1.y + m1[2][0] * v1.z;
            var y = m1[0][1] * v1.x + m1[1][1] * v1.y + m1[2][1] * v1.z;
            var z = m1[0][2] * v1.x + m1[1][2] * v1.y + m1[2][2] * v1.z;

            return Ej3DRender.vector3D.prototype.vector3D(x, y, z);
         },
      getMatrixAdd:function(m1, m2) 
      {
          var m = this.matrix3D(4);

       for (var i = 0; i < 4; i++)
        {
          for (var j = 0; j < 4; j++)
          {
             m[i][j] = m1[i][j] + m2[i][j];
          }
        }

       return m;
     },
      getMatrixMultiplication:function(m1, m2)
      {
          var res = this.getIdentity();

         for (var i = 0; i < this.MATRIX_SIZE; i++)
         {
             for (var j = 0; j < this.MATRIX_SIZE; j++)
             {
             var v = 0;

             for (var k = 0; k < this.MATRIX_SIZE; k++)
             {
                 v += m1[k][j] * m2[i][k];
                 var t = v;
             }

             res[i][j] = v;
          }
       }

       return res;
      },
      getMatrixEqual:function(m1,m2)
       {
            var res = true;

           for (var i = 0; i < m1.length; i++)
          {
              for (var j = 0; j < m1.length; j++)
              {
                 if (m1[i][j] != m2[i][j])
                 {
                     res = false;
                 }
             }
         }

      return res;
      },
      getMatrixNotEqual:function(m1, m2)
      {
            var res = true; 

         for (var i = 0; i < m1.length; i++)
        {
             for (var j = 0; j < m1.length; j++)
          {
             if (m1[i][j] != m2[i][j])
             {
                res = false;
             }
          }
       }

        return !res;
    },


      getMinor:function(dd, columnIndex, rowIndex)
      {
          return (((columnIndex + rowIndex) % 2 == 0) ? 1 : -1) * this.getDeterminant(this.getMMtr(dd, columnIndex, rowIndex));

      },

     getMMtr:function(dd, columnIndex, rowIndex)
     {
        var count = dd.length - 1;
        var d = this.createArray(count);

        for (var i = 0; i < count; i++)
        {
            var m = (i >= columnIndex) ? i + 1 : i;
            d[i] = this.createArray(count);

            for (var j = 0; j < count; j++)
            {
                var n = (j >= rowIndex) ? j + 1 : j;

                d[i][j] = dd[m][n];
            }
        }
         return d;
     },
    getDeterminant:function(dd)
    {
        var count = dd.length;
        var res = 0;

        if (count < 2)
        {
            res = dd[0][0];
        }
        else
        {
            var k = 1;

            for (var i = 0; i < count; i++)
            {
                var dm = this.getMMtr(dd, i, 0);

                res += k * dd[i][0] * this.getDeterminant(dm);
                k = (k > 0) ? -1 : 1;
            }
        }

        return res;
    },


       transform: function (x, y, z) {
           var res = this.getIdentity();

           res[3][0] = x;
           res[3][1] = y;
           res[3][2] = z;

           return res;
       },


       turn: function (angle) {
           var res = this.getIdentity();

           res[0][0] = Math.cos(angle);
           res[2][0] = -Math.sin(angle);
           res[0][2] = Math.sin(angle);
           res[2][2] = Math.cos(angle);

           return res;
       },

       tilt: function (angle) {
           var res = this.getIdentity();
          
           res[1][1] = Math.cos(angle);
           res[2][1] = Math.sin(angle);
           res[1][2] = -(Math.sin(angle));
           res[2][2] = Math.cos(angle);
          
           return res;          
       },
       transposed:function(matrix3D)
       {
           var m = this.getIdentity();

           for (var i = 0; i < this.MATRIX_SIZE; i++)
            {
                for (var j = 0; j < this.MATRIX_SIZE; j++)
                {
                    m[i][j] = matrix3D[j][i];
                }
           }
          return m;
       },
      shear:function(xy, xz, yx, yz, zx, zy)
        {
          var res = this.getIdentity();

           res[1, 0] = xy;
           res[2, 0] = xz;
           res[0, 1] = yx;
           res[2, 1] = yz;
           res[0, 2] = zx;
           res[1, 2] = zy;

           return res;
        }


   }
   Ej3DRender.Graphics3D.prototype = {
      addVisual:function(polygon)
         {
          if ((polygon == null) || (polygonobj.test(polygon)))
            {
                return -1;
            }
         polygon.Graphics3D = this;
         return bsptreeobj.add(polygon);
      },

       getVisualCount:function()
           {
           return ej.Ej3DRender.Polygons.length();
       },
       remove:function(polygon)
        {
           ej.Ej3DRender.Polygons.Remove(polygon);
       },
 
     clearVisual:function()
       {
         ej.Ej3DRender.Polygons=null;
       },

      getVisual:function()
          {
                return ej.Ej3DRender.Polygons;
         },
      prepareView:function(perspectiveAngle, depth, rotation, tilt, size, sender)
      {
          if (arguments.length==0) {
              bsptreeobj.build();
          }
          else {
              if (ej.Ej3DRender.transform == null)
                  ej.Ej3DRender.transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
              else
                  ej.Ej3DRender.transform.mViewport = size;
              if(!ej.Ej3DRender.tree)
                   ej.Ej3DRender.tree=[];
              ej.Ej3DRender.transform.Rotation = rotation;
              ej.Ej3DRender.transform.Tilt = tilt;
              ej.Ej3DRender.transform.Depth = depth;
              ej.Ej3DRender.transform.PerspectiveAngle = perspectiveAngle;
              ej.EjSvgRender.chartTransform3D.transform(ej.Ej3DRender.transform);
              ej.Ej3DRender.tree[sender._id] = bsptreeobj.build();
             
          } 
      },
      view: function (panel, sender, rotation, tilt, size, perspectiveAngle, depth)
      {
          var MaxValue = 32767;
          if (arguments.length == 2) {
              if (panel == null) return;
              var eye = vector.vector3D(0, 0, MaxValue);
              this.drawBspNode3D(ej.Ej3DRender.tree[sender._id], eye, panel, sender);
          }
          else {
              if (panel == null) return;
            
              if (ej.Ej3DRender.transform == null)
                  ej.Ej3DRender.transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
              else
                  ej.Ej3DRender.transform.mViewport = size;
              ej.Ej3DRender.transform.Rotation = rotation;
              ej.Ej3DRender.transform.Tilt = tilt;
              ej.Ej3DRender.transform.Depth = depth;
              ej.Ej3DRender.transform.PerspectiveAngle = perspectiveAngle;
              ej.EjSvgRender.chartTransform3D.transform(ej.Ej3DRender.transform);
              var eye = vector.vector3D(0, 0, MaxValue);
              this.drawBspNode3D(ej.Ej3DRender.tree[sender._id], eye, panel, sender);
          }
      },

      draw3DElement: function (tr, sender) {
          if (tr.Plane.element){
              if (tr.Plane.element.tag == "text" || tr.Plane.element.tag == "dataLabel")
                  polygonobj.drawText(tr.Plane, sender);
              else if (tr.Plane.element.tag == "polyline")
                  polygonobj.drawPolyLine(tr.Plane, sender);
              else if(tr.Plane.element.tag == "template")
                  polygonobj.drawTemplate(tr.Plane, sender);
              else
                  polygonobj.drawLine(tr.Plane, sender);
            }
              
          else
            polygonobj.draw(tr.Plane, sender);          
      },

    drawBspNode3D:function(tr, eye, panel, sender)
    {
        if (tr == null || ej.Ej3DRender.transform == null) return;
     while (true)
     {
         var r = vector.vector3dAND(polygonobj.getNormal(ej.EjSvgRender.chartTransform3D.result(ej.Ej3DRender.transform), tr.Plane.VectorPoints), eye);
        if (r > tr.Plane.D)
        {
            if (tr.Front != null)
            {
                this.drawBspNode3D(tr.Front, eye, panel, sender);
            }

            this.draw3DElement(tr, sender);

            if (tr.Back != null)
            {
                tr = tr.Back;
                continue;
            }
        }
        else
        {
            if (tr.Back != null)
            {
                this.drawBspNode3D(tr.Back, eye, panel, sender);
            }

            this.draw3DElement(tr, sender);

            if (tr.Front != null)
            {
                tr = tr.Front;
                continue;
            }
        }
        break;
    }

    }

 }


   Ej3DRender.BSPTreeBuilder.prototype = {
       //Members
       EPSILON: 0.0005,
       Polygon:[],
       
       //Methods
       add:function(poly)
       {
           ej.Ej3DRender.Polygons.push(poly);
           return ej.Ej3DRender.Polygons.length - 1;
       },

       remove:function(index)
       {
           ej.Ej3DRender.Polygons.splice(index,1);
       },

       clear:function()
       {
           ej.Ej3DRender.Polygons=[];
       },

       count:function()
       {
           return ej.Ej3DRender.Polygons.length;
       },

       getNext:function(i, count)
        {
            if (i >= count)
            {
             return i - count;
            }
            if (i < 0)
              {
                  return i + count;
              }

             return i;
       },
       getNodeCount:function(el)
        {
            return (el == null) ? 0 : 1 + this.getNodeCount(el.Back) + this.getNodeCount(el.Front);
        },
       vector3DIndexClassification:function(point, ind, res){
           return{
                 index:ind,
                 result: res,
                 vector:point,

                isCuttingBackPoint:false,
                cuttingBackPairIndex:null,
                alreadyCuttedBack:false,

                isCuttingFrontPoint:false,
                cuttingFrontPairIndex:null,
                alreadyCuttedFront:false
         }
       },
       classifyPoint: function(pt, pln)
         {
            var res = "OnPlane";
            var sv = -pln.D - Ej3DRender.vector3D.prototype.vector3dAND(pt , pln.normal);

           if (sv > this.EPSILON)
            {
              res ="OnBack";
            }
          else if (sv < -this.EPSILON)
            {
              res = "OnFront";
            }

            return res;
        },
      
     classifyPolygon:function(pln, plg)
        {
            var res = "Unknown";
            var points = plg.Points;

           if (points == null)
               return res;
               var onBack = 0;
               var onFront = 0;
               var onPlane = 0;
               var normal = pln.normal;// root node normailized value perpendicular direction
               var d = pln.D; // constant of the plan or depth

          for (var i = 0, len = points.length; i < len; i++)
           {
              var r = -d - Ej3DRender.vector3D.prototype.vector3dAND(points[i], normal); // Comparision of Plane point depth with the other nodes
 
              if (r > this.EPSILON)
                {
                  onBack++;
                }
              else if (r < -this.EPSILON)
                {
                   onFront++;
                }
             else
               {
                  onPlane++;
               }

             if ((onBack > 0) && (onFront > 0))
               {
                 break;
               }
          }
         if (onPlane == points.length)
            {
               res = "OnPlane";
            }
          else if (onFront + onPlane == points.length)
           {
              res = "ToRight";
           }
          else if (onBack + onPlane == points.length)
           {
             res = "ToLeft";
           }
          else
          {
                res = "Unknown";
          }
      return res;
     },
   splitPolygon:function(poly, part)
     {
            var backP = [];
            var frontP = [];

    // this code looks for points which lie on the part plane and divide polygon into two parts
    if (poly.Points != null)
    {
        var polyPoints = [];
        var backPartPoints = [];
        var frontPartPoints = [];

        var outpts;
        var inpts;

        var count = poly.Points.length;
        for (var i = 0; i < count;i++)
        {
            var ptB = poly.Points[i];
            var ptC = poly.Points[this.getNext(i + 1, count)];
            var sideB = this.classifyPoint(ptB, part);
            var sideC = this.classifyPoint(ptC, part);

            var vwiwcB = this.vector3DIndexClassification(ptB, polyPoints.length, sideB);
            polyPoints.push(vwiwcB);

            if ((sideB != sideC) && (sideB != "OnPlane") &&
                (sideC !="OnPlane"))
            {
                var v = vector.vector3DMinus(ptB, ptC);
                var dir = vector.vector3DMinus(vector.vector3DStarMultiply(part.normal, (-part.D)), ptC);

                var sv = vector.vector3dAND(dir, part.normal);
                var sect = sv / vector.vector3dAND(part.normal, v);
                var ptP = vector.vector3DPlus(ptC, vector.vector3DStarMultiply(v, sect));
                var vwiwc = this.vector3DIndexClassification(ptP, polyPoints.length, "OnPlane");

                polyPoints.push(vwiwc);
                backPartPoints.push(vwiwc);
                frontPartPoints.push(vwiwc);
            }
            else
                if (sideB == "OnPlane")
                {
                    var ptA = poly.Points[this.getNext(i - 1, count)];
                    var sideA = this.classifyPoint(ptA, part);
                    if ((sideA == sideC)) continue;
                    if ((sideA != "OnPlane") && (sideC != "OnPlane"))
                    {
                        backPartPoints.push(vwiwcB);
                        frontPartPoints.push(vwiwcB);
                    }
                    else
                        if (sideA =="OnPlane")
                        {
                            switch (sideC)
                            {
                                case "OnBack":
                                    backPartPoints.push(vwiwcB);
                                    break;
                                case "OnFront":
                                    frontPartPoints.push(vwiwcB);
                                    break;
                            }
                        }
                        else
                            if (sideC =="OnPlane")
                            {
                                switch (sideA)
                                {
                                    case "OnBack":
                                        backPartPoints.push(vwiwcB);
                                        break;
                                    case "OnFront":
                                        frontPartPoints.push(vwiwcB);
                                        break;
                                }
                            }
                }
        }

        if ((frontPartPoints.length != 0) || (backPartPoints.length != 0))
        {
            for (var i = 0; i < backPartPoints.length - 1; i += 2)
            {
                var vwiwc1 = backPartPoints[i];
                var vwiwc2 = backPartPoints[i + 1];
                vwiwc1.CuttingBackPoint = true;
                vwiwc2.CuttingBackPoint = true;
                vwiwc1.CuttingBackPairIndex = vwiwc2.index;
                vwiwc2.CuttingBackPairIndex = vwiwc1.index;
            }
            for (var i = 0; i < frontPartPoints.length - 1; i += 2)
            {
                var vwiwc1 = frontPartPoints[i];
                var vwiwc2 = frontPartPoints[i + 1];
                vwiwc1.CuttingFrontPoint = true;
                vwiwc2.CuttingFrontPoint = true;
                vwiwc1.CuttingFrontPairIndex = vwiwc2.index;
                vwiwc2.CuttingFrontPairIndex = vwiwc1.index;
            }


            for (var i = 0; i < backPartPoints.length - 1; i++)
            {
                var vwiwc1 = backPartPoints[i];
                if (vwiwc1.alreadyCuttedBack) continue;
                var outpts=this.cutOutBackPolygon(polyPoints, vwiwc1);

                if (outpts.length > 2) {
                    var polygon1 = polygonobj.polygon3D(outpts, poly);
                    backP.push($.extend({}, polygon1));
                }
            }

            for (var i = 0; i < frontPartPoints.length - 1; i++)
            {
                var vwiwc2 = frontPartPoints[i];
                if (vwiwc2.alreadyCuttedFront) continue;
                inpts=this.cutOutFrontPolygon(polyPoints, vwiwc2);
                if (inpts.length > 2) {
                    var polygon2 = polygonobj.polygon3D(inpts, poly);
                    frontP.push($.extend({}, polygon2));
                }
            }
        } 
    }
     else
     {
        backP.push(poly);
        frontP.push(poly); 
     }

    return{BackP:backP, FrontP:frontP}
   },
  cutOutFrontPolygon:function(polyPoints, vwiwc)
   {
      points = [];

    var curVW = vwiwc;

    while (true)
    {
        curVW.alreadyCuttedFront = true;
        points.push(curVW.vector);

        var curVWPair = polyPoints[curVW.CuttingFrontPairIndex];

        if (curVW.CuttingFrontPoint)
        {
            if (!curVWPair.alreadyCuttedFront)
            {
                curVW = curVWPair;
            }
            else
            {
                var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
                var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

                if ((curVWPrev.result =="OnFront") && !curVWPrev.alreadyCuttedFront)
                {
                    curVW = curVWPrev;
                }
                else
                    if ((curVWNext.result == "OnFront") && !curVWNext.alreadyCuttedFront)
                    {
                        curVW = curVWNext;
                    }
                    else
                    {
                        return points;
                    }
            }
        }
        else
        {
            var curPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
            var curNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

            if ((curPrev.result != "OnBack") && !curPrev.alreadyCuttedFront)
            {
                curVW = curPrev;
            }
            else
                if ((curNext.result != "OnBack") && !curNext.alreadyCuttedFront)
                {
                    curVW = curNext;
                }
                else
                {
                    return points;
                }
        }
    }
    return points;
 },

   cutOutBackPolygon:function(polyPoints, vwiwc)
   {
       var points = [];
       var curVW = vwiwc;

    while (true)
    {
        curVW.alreadyCuttedBack = true;
        points.push(curVW.vector);

        var curVWPair = polyPoints[curVW.CuttingBackPairIndex];

        if (curVW.CuttingBackPoint)
        {
            if (!curVWPair.alreadyCuttedBack)
            {
                curVW = curVWPair;
            }
            else
            {
                var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
                var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

                if ((curVWPrev.result == "OnBack") && !curVWPrev.alreadyCuttedBack)
                {
                    curVW = curVWPrev;
                }
                else
                    if ((curVWNext.result == "OnBack") && !curVWNext.alreadyCuttedBack)
                    {
                        curVW = curVWNext;
                    }
                    else
                    {
                        return points;
                    }
            }
        }
        else
        {
            var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
            var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

            if ((curVWPrev.result != "OnFront") && !curVWPrev.alreadyCuttedBack)
            {
                curVW = curVWPrev;
            }
            else
                if ((curVWNext.result != "OnFront") && !curVWNext.alreadyCuttedBack)
                {
                    curVW = curVWNext;
                }
                else
                {
                    return points;
                }
        }
    }
     return points
  },
     build:function()
       {
           if(!arguments[0]){
             return (this.build(ej.Ej3DRender.Polygons));               
           }
           else{
               var arlist=arguments[0];
               if (arlist.length < 1) return null;
               var bspNode = {Back:null, Front:null, Plane:null}
               var plane = arlist[0];
               bspNode.Plane = plane;
               var arleft = [];
               var arright = [];

               for (var i = 1, len = arlist.length; i < len; i++)
               {
                   var pln = arlist[i];

                   if (pln == plane) continue;
                   var r = this.classifyPolygon(plane, pln);

                   switch (r)
                   {
                       case "OnPlane":
                       case "ToRight":
                           arright.push(pln);
                           break;

                       case "ToLeft":
                           arleft.push(pln);
                           break;

                       case "Unknown":
                           //if (pln is Line3D || pln is UIElement3D)
                           if(pln.element && (pln.element.tag == "line" || pln.element.tag == "text"))
                           {
                               arleft.push(pln);
                           }
                         else
                         {
                            
                               var result = this.splitPolygon(pln, plane);                              
                               for (var k = 0; k < result.BackP.length; k++) {
                                   result.BackP[k].Name = result.BackP[k].Name + "back";
                                   arleft.push(result.BackP[k]);
                               }
                               for (var j = 0; j < result.FrontP.length; j++) {
                                   result.FrontP[j].Name = result.FrontP[j].Name + "front";
                                   arright.push(result.FrontP[j]);
                               }
                        }
                   break;
               }
           }

           if (arleft.length > 0)
           {
               bspNode.Back = this.build(arleft);
           }

           if (arright.length > 0)
           {
               bspNode.Front = this.build(arright);
           }

           return bspNode;
         }
    }

   }

   Ej3DRender.polygon3D.prototype = {
       // Member    
       epsilon: 0.00001,
       normal: { x: 0, y: 0, z: 0 },
       a: (this.normal) ? this.normal.x : 0,
       b: (this.normal) ? this.normal.y : 0,
       c: (this.normal) ? this.normal.z : 0,
       vector: new (new ej.Ej3DRender()).vector3D(),

       // Constructor
       polygon3D: function (points, tag, index, stroke, strokeThickness, opacity, fill, name) {

           if (arguments.length == 3) {
               this.calcNormal(arguments[0], arguments[1], arguments[2])
           }
           else if (arguments.length == 2) {
               points = arguments[0];
               this.calcNormal(points[0], points[1], points[2]);
               this.vectorPoints = points;
               this.calcNormal(this.vectorPoints);
               var polygon = arguments[1];
               polygon.Normal = this.normal;
               polygon.normal = this.normal;
               polygon.Points = points;
               polygon.VectorPoints = this.vectorPoints;
               polygon.IsSplitted = true;
               polygon.d = this.d;
               polygon.D = this.d;
               return polygon;
                
           }
           else {
               this.calcNormal(points[0], points[1], points[2]);
               this.vectorPoints = points;
               this.calcNormal(this.vectorPoints);
               var element = { Tag: 'path', Parent: arguments[arguments.length - 1] }
               var polygon= {
                   Normal: this.normal,
                   normal: this.normal,
                   Points: points,
                   VectorPoints: this.vectorPoints,
                   Index: index,
                   Tag: tag,
                   Name: (name) ? name : null,
                   StrokeThickness: strokeThickness,
                   Opacity:opacity,
                   Fill: fill,
                   d: this.d,
                   D: this.d
               }
               if (arguments.length != 1) 
                   polygon.Element = element;

               return polygon;
               
           }
       },

       //Methods
       createLine: function (line, x1, y1, x2, y2, depth) {
           var strokeThickness = line.width;
           var vectorColl = [];
           vectorColl[0] = vector.vector3D(x1, y1, depth);
           vectorColl[1] = vector.vector3D(x1 + strokeThickness, y2 + strokeThickness, depth);
           vectorColl[2] = vector.vector3D(x2, y2, depth);
           return this.line3D(line, vectorColl);
       },
       createPolyline: function (points, element) {
           if (points.length == 2) {
               var prePoint = points[1];
               points.push(vector.vector3D(prePoint.x, prePoint.y, prePoint.z));
           }
            return this.polyLine3D(element, points);
       },
       polyLine3D: function (element, points) {
           var plane = this.polygon3D(points);
           plane.element = element;
           return plane;
       },

       line3D:function(element, points)
       {
           var plane = this.polygon3D(points);
           plane.element = element;
           return plane;
       },
      text3D:function(element,points)
      {
          var plane = this.polygon3D(points);
          plane.element = element;
          return plane;
      },
      createCylinder: function (v1, v2, tag, index, type, stroke, fill, strokeThickness, opacity, inverse, name, parent) {
          var i = 0, ox, oy, oz, pathCount = 24, theta = 360 / pathCount, DtoR = Math.PI / 180, CenterZ = (parseFloat)((v1.z + v2.z) / 2), res = [], oPts = [], tVtxs = [], bVtxs = [], radiusb = (v2.y - v1.y) < (v2.z - v1.z) ? (parseFloat)((v2.y - v1.y) / 2) : (parseFloat)((v2.z - v1.z) / 2), radiusc = (v2.x - v1.x) < (v2.z - v1.z) ? (parseFloat)((v2.x - v1.x) / 2) : (parseFloat)((v2.z - v1.z) / 2), CenterX = (parseFloat)((v1.x + v2.x) / 2), CenterY = (parseFloat)((v1.y + v2.y) / 2);
          switch (type) {
              case "bar":
              case "stackingbar":
              case "stackingbar100":                 
                  pathCount++;
                  while (pathCount--) {
                      oy = (parseFloat)(CenterY + radiusb * Math.cos((i * theta) * DtoR));
                      oz = (parseFloat)(CenterZ + radiusb * Math.sin((i * theta) * DtoR));
                      oPts[i] = { Y: oy, Z: oz };
                      tVtxs.push(vector.vector3D(v1.x, oPts[i].Y, oPts[i].Z));
                      bVtxs.push(vector.vector3D(v2.x, oPts[i].Y, oPts[i].Z));
                      if (i > 0) {
                          vts = new Array(vector.vector3D(v1.x, oPts[i - 1].Y, oPts[i - 1].Z),
                               vector.vector3D(v2.x, oPts[i - 1].Y, oPts[i - 1].Z),
                                vector.vector3D(v2.x, oPts[i].Y, oPts[i].Z),
                                vector.vector3D(v1.x, oPts[i].Y, oPts[i].Z))
                          res[i + 1] = this.polygon3D(vts, tag, index, fill, 0, opacity, fill, "_"+(i + 1).toString()+"_" + name, parent);
                          graphics.addVisual(res[i + 1]);
                      }
                      i++;
                  }
                  break;
              case "column":
              case "stackingcolumn":
              case "stackingcolumn100":
                  pathCount++;             
                  while (pathCount--) {
                      ox = (parseFloat)(CenterX + radiusc * Math.cos((i * theta) * DtoR));
                      oz = (parseFloat)(CenterZ + radiusc * Math.sin((i * theta) * DtoR));
                      oPts[i] = { X: ox, Z: oz };
                      tVtxs.push(vector.vector3D(oPts[i].X, v1.y, oPts[i].Z));
                      bVtxs.push(vector.vector3D(oPts[i].X, v2.y, oPts[i].Z));
                      if (i > 0) {
                          vts = new Array(vector.vector3D(oPts[i-1].X, v1.y, oPts[i-1].Z),
                                vector.vector3D(oPts[i-1].X, v2.y, oPts[i-1].Z),
                                 vector.vector3D(oPts[i].X, v2.y, oPts[i].Z),
                                 vector.vector3D(oPts[i].X, v1.y, oPts[i].Z))
                          res[i+1] = this.polygon3D(vts, tag, index, fill, 0, opacity, fill, "_"+(i+1).toString()+"_" + name, parent);
                          graphics.addVisual(res[i+1]);
                      }
                      i++;
                  }
                  break;
              default:
                  break;
          }
          res[0]=this.polygon3D(bVtxs, tag, index, stroke, 0, opacity, fill, "_0_" + name, parent);          
          res[1]=this.polygon3D(tVtxs, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + name, parent);        
          graphics.addVisual(res[0]);
          graphics.addVisual(res[1]);        
          return res;
      },

       createBox: function (v1, v2, tag, index, graphics3D, stroke, fill, strokeThickness, opacity, inverse, name, parent) {
           var res = [];

           var p1 = new Array(vector.vector3D(v1.x, v1.y, v1.z),
                vector.vector3D(v2.x, v1.y, v1.z),
                vector.vector3D(v2.x, v2.y, v1.z),
                vector.vector3D(v1.x, v2.y, v1.z))

           var p2 = new Array(
               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v1.x, v2.y, v2.z)
            )

           var p3 = new Array(

               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v1.z),
               vector.vector3D(v1.x, v1.y, v1.z)
          )

           var p4 = new Array(
               vector.vector3D(v1.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v1.z),
               vector.vector3D(v1.x, v2.y, v1.z)
           )

           var p5 = new Array(
               vector.vector3D(v1.x, v1.y, v1.z),
               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v1.x, v2.y, v2.z),
               vector.vector3D(v1.x, v2.y, v1.z)
           )

           var p6 = new Array(
               vector.vector3D(v2.x, v1.y, v1.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v1.z)
           )
           if (arguments[10]) {

               res[0] = this.polygon3D(p1, tag, index, stroke, strokeThickness, opacity, fill, "_0_"+ name, parent);
               res[1] = this.polygon3D(p2, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + name, parent);
               res[2] = this.polygon3D(p3, tag, index, stroke, strokeThickness, opacity, fill, "_2_" + name, parent);
               res[3] = this.polygon3D(p4, tag, index, stroke, strokeThickness, opacity, fill, "_3_" + name, parent);
               res[4] = this.polygon3D(p5, tag, index, stroke, strokeThickness, opacity, fill, "_4_" + name, parent);
               res[5] = this.polygon3D(p6, tag, index, stroke, strokeThickness, opacity, fill, "_5_" + name, parent);
           }
           else {
               var parent= arguments[arguments.length-1];
               res[0] = this.polygon3D(p1, tag, index, stroke, strokeThickness, opacity, fill, "_0_" + index, parent);
               res[1] = this.polygon3D(p2, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + index, parent);
               res[2] = this.polygon3D(p3, tag, index, stroke, strokeThickness, opacity, fill, "_2_" + index, parent);
               res[3] = this.polygon3D(p4, tag, index, stroke, strokeThickness, opacity, fill, "_3_" + index, parent);
               res[4] = this.polygon3D(p5, tag, index, stroke, strokeThickness, opacity, fill, "_4_" + index, parent);
               res[5] = this.polygon3D(p6, tag, index, stroke, strokeThickness, opacity, fill, "_5_" + index, parent);
           }

           if (inverse) {
               graphics.addVisual(res[0]);
               graphics.addVisual(res[1]);
               graphics.addVisual(res[2]);
               graphics.addVisual(res[3]);
               graphics.addVisual(res[4]);
               graphics.addVisual(res[5]);
           }
           else {
               graphics.addVisual(res[5]);
               graphics.addVisual(res[4]);
               graphics.addVisual(res[0]);
               graphics.addVisual(res[1]);
               graphics.addVisual(res[2]);
               graphics.addVisual(res[3]);
              
           }
           return res;
       },


       calcNormal: function () {
           if (arguments.length >= 3) {
               // Relative information of the points
               var v1 = arguments[0];
               var v2 = arguments[1];
               var v3 = arguments[2];
               var v4 = vector.vector3DMinus(v1, v2);
               var v5 = vector.vector3DMinus(v3, v2);
               var n = vector.vector3DMultiply(v4, v5);

               var l = vector.getLength(n);//Get length of the vector

               if (l < this.epsilon) {
                   l = 1;
               }

               this.normal= vector.vector3D(n.x / l, n.y / l, n.z / l); //Calculate normalization of the vector
               this.d = -(this.normal.x * v1.x + this.normal.y * v1.y + this.normal.z * v1.z);// Normalized values * 1st coordinates Coordinates - Depth of the plan

               if (arguments[3]) {
                   arguments[3].normal = this.normal;
                   arguments.d = this.d;
               }
           }
           else {
               var Points = arguments[0];
               this.calcNormal(Points[0], Points[1], Points[2], arguments[1]);

               for (var i = 3; (i < Points.length) && (this.test()) ; i++) {
                   this.calcNormal(Points[i], Points[0], Points[i / 2]);
               }
           }
       },

       test: function () {
           return !vector.isValid(this.normal);
       },

       transform: function (matrix, plan) {
           if (plan.Points != null) {
               for (var i = 0; i < plan.Points.length; i++) {

                   plan.VectorPoints[i] = plan.Points[i] = matrixobj.getMatrixVectorMutiple(matrix, plan.Points[i]);
               }

               this.calcNormal(plan.VectorPoints, plan);
           }
           else {
               var v = matrix * (plan.normal * -plan.d);
               plan.normal = matrixobj.getMatrixVectorAnd(matrix, plan.normal);
               vector.normalize(normal);
               plan.d = -(plan.normal & v);
           }
       },

       getPoint: function (x, y, normal, d) {
           if (typeof x == double) {
               var z = -(normal.x * x + normal.y * y + d) / normal.z;

               return vector.vector3D(x, y, z);
           }
           else {
               var position = x;
               var ray = y;
               var dir = vector.vector3DMinus(vector.vector3DStarMultiply(normal, (-d)), position);

               var sv = vector.vector3dAND(dir, normal);
               var sect = sv / vector.vector3dAND(normal, ray);

               return vector.vector3DPlus(position, (vector.vector3DStarMultiply(ray * sect)));
           }
       },

       getNormal: function (transform, VectorPoints) {
           var norm;

           if (VectorPoints != null) {
               norm = vector.getNormal(matrixobj.getMatrixVectorMutiple(transform, VectorPoints[0]),
                  matrixobj.getMatrixVectorMutiple(transform, VectorPoints[1]), matrixobj.getMatrixVectorMutiple(transform, VectorPoints[2]))


               for (var i = 3; (i < VectorPoints.length) && !vector.isValid(norm) && VectorPoints[i / 2]; i++) {
                   var v1 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[i]);
                   var v2 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[0]);
                   var v3 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[i / 2]);

                   norm = vector.getNormal(v1, v2, v3);
               }
           }
           else {
               norm = matrixobj.getMatrixVectorAnd(transform, normal);
               vector.normalize();
           }

           return norm;

       },

     createTextElement:function(position,  element, xLen, yLen)
      {
        
       var vectorColl = [];
       var x = position.x;
       var y = position.y;
 
       var desiredWidth = element.Width;
       var desiredHeight = element.Height;

          vectorColl[0] = this.vector.vector3D(x, y, position.z);
          vectorColl[1] = this.vector.vector3D(x + desiredWidth, y + desiredHeight + yLen, position.z);
          vectorColl[2] = this.vector.vector3D(x + desiredWidth + xLen, y + desiredHeight + yLen, position.z);
         return this.text3D(element, vectorColl);
     },

     redraw: function () { },

     drawPolyLine: function (panel, sender) {
         var transform = ej.Ej3DRender.transform;
         var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
         var StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
         pathDirection.append("M" + " " + (StartPoint.x) + " " + (StartPoint.y) + " ");
         for(var i=0; i< panel.VectorPoints.length; i++)
         {
            lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
            pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");
                     
         }        
          
         direction = pathDirection.toString();

        
         var optionsLine = {
             'id': panel.element.id,
             'stroke-dasharray': panel.element.dashArray,
             'stroke-width': panel.element.width,
             'stroke': panel.element.stroke,
             'd': direction
         };

         sender.svgRenderer.drawPath(optionsLine, panel.element.child);
     },

       drawLine: function (panel, sender) {
           
           var transform = ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition1 = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           var actual3DPosition2 = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[2], transform);

           var optionsLine = {
               'id': panel.element.id,
               x1: actual3DPosition1.x ,
               y1: actual3DPosition1.y,
               x2: actual3DPosition2.x,
               y2:actual3DPosition2.y,
               'stroke-dasharray': panel.element.dashArray,
               'stroke-width': panel.element.width,
               'stroke': panel.element.stroke,
               'opacity': panel.element.opacity
           };

           sender.svgRenderer.drawLine(optionsLine, panel.element.child);
 

       },
       drawTemplate: function (panel, sender) {
           var element = panel.element;
           var transform = ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           ej.EjSeriesRender.prototype.drawLabelTemplate(element.Label.series, element.Label.series.points[element.Label.pointIndex], element.Label.pointIndex, { X: actual3DPosition.x, Y: actual3DPosition.y }, sender)
       },
       drawText: function (panel, sender) {
           
           var element = panel.element;
           var transform =ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           var x = actual3DPosition.x;
           var y = actual3DPosition.y;
 
           if (element.tag == "text") {
               var options = {
                   'id': element.id,
                   'x': x,
                   'y': y,
                   'fill': element.font.color,
                   'font-size': element.font.size,
                   'font-family': element.font.fontFamily,
                   'font-style': element.font.fontStyle,
                   'font-weight': element.font.fontWeight,
                   'opacity': element.font.opacity,
                   'text-anchor': element.TextAnchor,            
                   "cursor":"default",
                   'transform': element.Angle ? 'rotate(' + element.Angle + ',' + (x) + ',' + y + ')' : null
               };           
              
               sender.svgRenderer.drawText(options, panel.element.Label.Text, panel.element.child, panel.element.Label);			 
           }
           else {
               var series = element.series;           
 
               var labelFormat = (series.yAxis && series.yAxis.labelFormat) ? series.yAxis.labelFormat : "";
               var pointText = (element.point.text) ? element.point.text : element.point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
               var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);
               var margin = series.marker.dataLabel.margin;
               var width = textOffset.width + margin.left + margin.right;
               var height = textOffset.height + margin.top + margin.bottom;
               var location = { X: actual3DPosition.x, Y: actual3DPosition.y };

                  
                      if (series.marker.dataLabel.shape)
                          var elementShape = series.marker.dataLabel.shape;
                      else if (series.marker.dataLabel.shape)
                          elementShape = marker.dataLabel.shape;
                      else
                          elementShape = "None";

                      $.each(sender.model.symbolShape, function (name) {
                          if (elementShape.toLowerCase() == name.toLowerCase())
                              symbolName = name;
                      });

                   var xXalue = location.X - (margin.left) / 2 + (margin.right) / 2;
                   var yValue = location.Y - (margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2;

                   var seriesIndex = $.inArray(series, sender.model.series);
                   ej.EjSeriesRender.prototype.dataLabelSymbol(seriesIndex, series, element.pointIndex, xXalue, yValue, width, height, symbolName, sender);
            
           }

       },

       draw: function (panel,sender) {            
           if (panel.VectorPoints == null || panel.VectorPoints.length <= 0) return;
           var transform = ej.Ej3DRender.transform;
           var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
           var color = panel.Fill;
		   var format = sender.svgRenderer.checkColorFormat(color);
		   if (!format)
		      color=sender.colorNameToHex(color);
           var figure={Segments:null, StartPoint:null};
           if (transform != null)
           {
               figure.StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
               pathDirection.append("M" + " " + (figure.StartPoint.x) + " " + (figure.StartPoint.y) + " ");
               for(var i=0; i< panel.VectorPoints.length; i++)
                {
                       lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
                       pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");
                     
                }
           }
          
           direction = pathDirection.toString();
          
            var name = "Light";
            var lightCoefZ =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(0, 0, 1))) - 1));
            var lightCoefY =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(0, 1, 0))) - 1));
            var lightCoefX =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(1, 0, 0))) - 1));
           if (lightCoefZ == lightCoefX)
           {               
              name = "ZLight";
              color = this.applyZLight(color, sender);  
           }
          else if (((lightCoefY == lightCoefZ) || (lightCoefZ != 0 && lightCoefY < lightCoefZ)))
          {
              name = "XLight";
              color= this.applyXLight(color, sender);
           }
          else if (lightCoefZ < 0)
          {
              name = "ZLight";
              color = this.applyZLight(color, sender);
          }
          else
          {
              name = "Light";
              color = color;
          }
          if (sender.model.AreaType == "none")
               panel.StrokeThickness = 0;
           var optionsMinorGrid = {
               'id': sender.svgObject.id +"_"+ panel.Name,
               'name':name, 
               'fill': color,
               'stroke': panel.Stroke,
               'stroke-width': panel.StrokeThickness,
               'opacity': panel.Opacity,
               'd': direction
           };
           sender.svgRenderer.drawPath(optionsMinorGrid, panel.Element.Parent);
           if (sender.model.previousID && sender.model.touchCross) 
               $("#" + optionsMinorGrid.id).insertAfter($("#" + sender.model.previousID));
              
           sender.model.previousID = optionsMinorGrid.id;
       },

       applyXLight: function (color, sender)
       { 
           var RGB = sender.svgRenderer.hexToRGB(color);
           RGB.R = (parseInt)(RGB.R * 0.7);
           RGB.G = (parseInt)(RGB.G * 0.7);
           RGB.B = (parseInt)(RGB.B * 0.7);
           return sender.svgRenderer.hexFromRGB(RGB);
       },

       applyZLight:function(color, sender)
         {
           var RGB = sender.svgRenderer.hexToRGB(color);
           RGB.R = (parseInt)(RGB.R * 0.9);
           RGB.G = (parseInt)(RGB.G * 0.9);
           RGB.B = (parseInt)(RGB.B * 0.9);
           return sender.svgRenderer.hexFromRGB(RGB);
        },

       update: function (updatedVectors, panel, sender) {
           if (panel.VectorPoints == null || panel.VectorPoints.length <= 0) return;
           panel.VectorPoints = updatedVectors;
           var transform = ej.Ej3DRender.transform;
           var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
           var color = panel.Fill;
           var figure = { Segments: null, StartPoint: null };
           if (transform != null) {
               figure.StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
               pathDirection.append("M" + " " + (figure.StartPoint.x) + " " + (figure.StartPoint.y) + " ");
               for (var i = 0; i < panel.VectorPoints.length; i++) {
                   lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
                   pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");

               }
           }

           direction = pathDirection.toString();    

           if ($(sender.chartObj.chart3D).find("#" + sender.chartObj.svgObject.id + "_" + panel.Name).length > 0) {
               var element = $(sender.chartObj.chart3D).find("#" + sender.chartObj.svgObject.id + "_" + panel.Name)[0];
               sender.chartObj.svgRenderer._setAttr($(element), { 'd': direction });
              
           }
         
       }


   }
   var Ej3DRender = new ej.Ej3DRender();
   var vector = new Ej3DRender.vector3D();
   var matrixobj = new Ej3DRender.matrix3D();
   var bsptreeobj = new Ej3DRender.BSPTreeBuilder();
   var polygonobj = new Ej3DRender.polygon3D();
   var graphics = new Ej3DRender.Graphics3D();

  
})(jQuery);;
ej.Ej3DAxisRenderer = function () {
};

ej.Ej3DSeriesRender = function () {
};


ej.Ej3DChart = function () {
};

ej.series3DTypes = {};

(function ($) {

    ej.Ej3DAxisRenderer.prototype = {

        _drawAxes: function (index, axis, params) {
            if (axis.majorGridLines.visible)
                this._drawGridLines3D(axis, params);
            if (axis.visible && axis.majorTickLines.visible)
                this._renderTicks3D(axis, axis.majorTickLines.size, axis.majorTickLines.width, params);

            if (axis.visible) {
                this._drawAxisLabel(axis, params);
				 this._drawMultiLevelLabel(index, axis, params);
                this._drawAxisTitle(axis);
            }
        },

        _drawAxisTitle: function (axis) {
           if(axis.title.visible)
           {
			   var font = axis.title.font;
			   this.gAxisTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + "axisTitle_" + axis.name});
               var opposedPosition = axis._opposed;
			   var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height()},
			    padding = 10,
				transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
				transform.mViewport = size;
				transform.Rotation = 0;
				transform.Tilt = 0;
				transform.Depth = 100;
				transform.PerspectiveAngle = 90;
				ej.EjSvgRender.chartTransform3D.transform(transform);
				var result = ej.EjSvgRender.chartTransform3D.result,
				matrix3D = ej.Ej3DRender.prototype.matrix3D.prototype,
				orientation = axis.orientation.toLowerCase(),
				multiLevelLabelsVisible = false;
				
			for (var i=0; i< this.model._axes.length; i++){
				currentAxis = this.model._axes[i];
				if (currentAxis.orientation.toLowerCase() == orientation){
					for (var j=0; j< currentAxis.multiLevelLabels.length; j++){
						if (currentAxis.multiLevelLabels[j].visible){
							multiLevelLabelsVisible = true;
							break;
						}
					}
				}
			}
			
            if (orientation == "horizontal") {
                var xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height;
                var titlesize = (ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font).height / 2);
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: axis.title.text, location: { x: (axis.width) / 2, y: (xtitleLocation + titlesize + axis.majorTickLines.size + axis.axisLine.width + this.model.wallSize + axis._multiLevelLabelHeight ) }, axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);

				font = axis.title.font;
                var x1 = commonEventArgs.data.location.x + axis.x;
                var y1 = (opposedPosition) ? (axis.y - commonEventArgs.data.location.y) : (commonEventArgs.data.location.y + axis.y);

                element = { Width: titlesize.width, Height: titlesize.height, Label: commonEventArgs.data, TextAnchor: 'middle', tag: 'text', font: font, id: this.svgObject.id + "axisTitle", child: this.chart3D };
				if (!multiLevelLabelsVisible)
					this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
				else  {
				
						 var options = {
                'id': this.svgObject.id + "axisTitle",
                'x': x1,
                'y': y1,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': "middle"
            };
					    var newOptions = ej.EjSvgRender.chartTransform3D.toScreen({x: x1, y: y1, z:0}, transform, result, matrix3D);
						options.x = newOptions.x; options.y = newOptions.y;
						this.svgRenderer.drawText(options, commonEventArgs.data.Text, this.gAxisTitleEle);
						this.svgRenderer.append(this.gAxisTitleEle, this.svgObject);
				}
            }
            else {
                var titleSize = ej.EjSvgRender.utils._measureText(axis.title.text, (axis.height), axis.title.font);
                var x1 = (opposedPosition) ? axis.x + ((4 * this.model.elementSpacing) + axis._LableMaxWidth.width + axis.majorTickLines.size + axis.axisLine.width + this.model.wallSize + axis._multiLevelLabelHeight) : axis.x - ((3 * this.model.elementSpacing) + axis._LableMaxWidth.width + axis.majorTickLines.size + axis.axisLine.width + this.model.wallSize + axis._multiLevelLabelHeight);

                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: axis.title.text, location: { x: titleSize.width / 2, y: 0 }, axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);

				font = axis.title.font;
                var y1 = commonEventArgs.data.location.y + (axis.y + axis.height) + (((axis.height) / 2) * -1);

                var labelRotation = (opposedPosition) ? 90 : -90;
                var value = opposedPosition ? 25 : -25;
                element = { Width: titleSize.width, Height: titleSize.height, Angle: -90 , Label: commonEventArgs.data, TextAnchor: 'middle', tag: 'text', font: font, id: this.svgObject.id + '_YAxisTitle', child: this.chart3D };
				if (!multiLevelLabelsVisible)
					this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
				else  {
				
						 var options = {
                'id': id,
                'x': x1,
                'y': y1,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': "middle",
				'transform': 'rotate(' + -90 + ',' + (x1) + ',' + (opposedPosition ? y1 + padding : y1 - padding) + ')',
            };
					    var newOptions = ej.EjSvgRender.chartTransform3D.toScreen({x: x1, y: y1, z:0}, transform, result, matrix3D);
						options.x = newOptions.x + value; options.y = newOptions.y;
						this.svgRenderer.drawText(options, commonEventArgs.data.Text, this.gAxisTitleEle);
						this.svgRenderer.append(this.gAxisTitleEle, this.svgObject);
				}
            }
           }
        },

		_drawMultiLevelLabel: function (axisIndex, axis, params){    // to render multilevel labels
				// declaration
			this.gMultiLevelEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_axisMultiLevelLabels' + '_' + axisIndex});
			var i, j, x, y, rows, gap, labelSize, grpLabel, font, center, centerX, startX, startY, endX, textOptions, textOverflow, depth = 0,
				textCollection, borderOptions, border, style, alignment, multiLevelLabelsLength = axis.multiLevelLabels.length, model = this.model,
				elementSpacing = model.elementSpacing, padding = 10, x1 = 0, y1 = 0, element, anchor, topValue, bottomValue, leftValue,
				rightValue, lineTop, bottom, right, left, areaBounds = model.m_AreaBounds, textSize, visibleRange = axis.visibleRange, actual3DPosition1,
				wallSize = model.wallSize, opposedPosition = axis._opposed, validCross = params.axes[axis.name]._validCross, actual3DPosition2, 
				orientation = axis.orientation.toLowerCase(), y = validCross ? axis.y : areaBounds.Y + (!opposedPosition && areaBounds.Height), alignment,
				x = validCross ? axis.x : areaBounds.X + (opposedPosition && areaBounds.Width), options, newOptions, vectorColl = [], l, borderWidth;
				
				var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height() };
				var transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
				transform.mViewport = size;
				transform.Rotation = 0;
				transform.Tilt = 0;
				transform.Depth = 100;
				transform.PerspectiveAngle = model.perspectiveAngle;
				ej.EjSvgRender.chartTransform3D.transform(transform);
				var result = ej.EjSvgRender.chartTransform3D.result,
				matrix3D = ej.Ej3DRender.prototype.matrix3D.prototype;
			
			for (var i = 0; i < multiLevelLabelsLength; i++){
				grpLabel = axis.multiLevelLabels[i];
				id = this.svgObject.id + "_" + axis.name + "MultiLevelLabels_" + axisIndex + "_" + i;
				if (grpLabel.text != "" && grpLabel.visible){
				level = grpLabel._level;
				anchor = "middle";
				font = grpLabel.font;
				borderWidth = grpLabel.border.width;
				maximumTextWidth = grpLabel.maximumTextWidth;
				alignment = grpLabel.textAlignment.toLowerCase();
				actualText =  text = grpLabel.text;
				textOverflow = grpLabel.textOverflow.toLowerCase();
				textSize = ej.EjSvgRender.utils._measureText(grpLabel.text, areaBounds.Width, grpLabel.font);
				center = (grpLabel.end - grpLabel.start) / 2;
				centerX = (grpLabel.start + center - visibleRange.min) / (visibleRange.delta);
				startX = (grpLabel.start - visibleRange.min) / (visibleRange.delta);
				endX = (grpLabel.end - visibleRange.min) / (visibleRange.delta);
				if (axis.isInversed){
					centerX = isNaN(centerX) ? 0 : 1 - centerX;
					startX = isNaN(startX) ? 0 : 1 - startX;
					endX = isNaN(endX) ? 0 : 1 - endX;
				}
                if (orientation == "horizontal") {    // start, end and gap calculation
                    x1 = (Math.round(axis.width * centerX)) + areaBounds.X + ((axis.plotOffset));
					y1 = (opposedPosition) ? (y - wallSize - (axis.majorTickLines.size) - elementSpacing - padding/2 - axis._LableMaxWidth.height) : y + wallSize + (axis.majorTickLines.size) + elementSpacing + axis._LableMaxWidth.height;
					startX = (Math.round(axis.width * startX)) + areaBounds.X + ((axis.plotOffset));
					endX = (Math.round(axis.width * endX)) + areaBounds.X + ((axis.plotOffset));
					gap = maximumTextWidth ? maximumTextWidth : endX - startX - padding;
				} else {
                    y1 = Math.round(axis.plotOffset + axis.y + (axis.height * (1 - centerX)));
                    x1 = (opposedPosition) ? (axis.x + wallSize + axis.majorTickLines.size + axis.axisLine.width + elementSpacing/2 + axis._LableMaxWidth.width + axis.multiLevelLabelHeight[level]/2) : (x - wallSize - axis.majorTickLines.size - elementSpacing / 2 - axis._LableMaxWidth.width - axis.multiLevelLabelHeight[level]/2);
                    startX = Math.round(axis.plotOffset + axis.y + (axis.height * (1 - startX)));
					endX = Math.round(axis.plotOffset + axis.y + (axis.height * (1 - endX)));
					gap = maximumTextWidth ? maximumTextWidth: axis.multiLevelLabelHeight[level] + padding;
                }
				y1 = opposedPosition ?  y1 - axis.prevHeight[level] : y1 + axis.prevHeight[level];
			     
				if (alignment == "far") {
					x1 = x1 + gap/2 - borderWidth / 2;
					anchor = "end";
				} else if (alignment == "near") {
					anchor = "start";
					x1 = x1 - (gap / 2) + (borderWidth / 2);
				}
				
				// to trigger event
				var data =  ej.EjAxisRenderer.prototype._triggerMultiLevelLabelsRendering(actualText, x1, y1, textOverflow, font, grpLabel.border, this);
				text = actualText = data.text;
				x1 = data.location.x;
				y1 = data.location.y;
				textOverflow = data.textOverflow.toLowerCase();
				font = data.font;
				font.color = font.color ? font.color: axis.multiLevelLabelsFontColor;
				border = data.border;
				style = data.border.type.toLowerCase();
				borderWidth = data.border.width;
			 
			 options = {
                'id': id,
                'x': x1,
                'y': y1,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': anchor
            };
			
				// calculation for wrap and wrapbyword
				if (textOverflow != "none")
					text = ej.EjAxisRenderer.prototype.textOverflowMultiLevelLabels(axis, gap, text, textOverflow, font, textSize, {x: x1, y: y1}, null, this);
				if (textOverflow == "wrap" || textOverflow == "wrapandtrim"){
					newY = y1;
					for (j = 0; j < text.length ; j++) {
					textSize = ej.EjSvgRender.utils._measureText(text[j], null, font);
					if (j!=0) newY = opposedPosition ? newY - textSize.height: newY + textSize.height;
					element = { Width: textSize.width, Height: textSize.height, Label: {"Text" : text[j]}, TextAnchor: anchor, tag: 'text', font: font, id: id + "_" + j, child: this.chart3D };
					options.id = options.id + "_" + j;
					var newOptions = ej.EjSvgRender.chartTransform3D.toScreen({x: x1, y: newY + padding/2, z:0}, transform, result, matrix3D);
				    options.x = newOptions.x; options.y = newOptions.y;
					this.svgRenderer.drawText(options, text[j], this.gMultiLevelEle);
				  }
				}
				x1= orientation == "horizontal" ? x1 : (opposedPosition ? x1 + padding : x1 - padding);
				
				//rendering multi level labels text
				if (textOverflow != "wrap" && textOverflow != "wrapandtrim"){
					element = { Width: textSize.width, Height: textSize.height, Label: {"Text" : text}, TextAnchor: anchor, tag: 'text', font: font, id: id, child: this.chart3D };
					var newOptions = ej.EjSvgRender.chartTransform3D.toScreen({x: x1, y: y1+padding/2, z:0}, transform, result, matrix3D);
					options.x = newOptions.x; options.y = newOptions.y;
					this.svgRenderer.drawText(options, text, this.gMultiLevelEle);
				}
				// storing region for multi level labels click event
				if (orientation == "horizontal")
					var region = {bounds: {x: startX, y: y1 - padding, height: axis.multiLevelLabelHeight[level], width: gap }, axisIndex: axisIndex, multiLevelLabel: grpLabel};
				else 
					var region = {bounds: {x: x1- width/2 - padding/2, y: endX, height: startX - endX, width: width }, axisIndex: axisIndex, multiLevelLabel: grpLabel};
				this.model.multiLevelLabelRegions.push(region);
				
				// to render border
				if (style != "none" && borderWidth > 0){
					id = this.svgObject.id + "_" + axis.name + "MultiLevelLabelsBorder_" + axisIndex + "_" + i;
					var height = y1-padding + axis.multiLevelLabelHeight[level] + padding/2;
					var borderColor = grpLabel.border.color ? grpLabel.border.color : axis.multiLevelLabelsColor
					if (orientation == "horizontal"){
						topValue = {x1: startX, y1: y1 - padding, x2: endX, y2: y1 - padding };
						bottomValue = {x1: startX, y1: height, x2: endX, y2: height };
						leftValue = {x1: startX, y1: y1-padding, x2: startX, y2: height };
						rightValue = {x1: endX, y1: y1-padding, x2: endX, y2: height };
					} else {
						width = axis.multiLevelLabelHeight[level] ;
						topValue = {x1: x1- width/2 - padding/2, y1: endX, x2: x1+ width/2 + padding/2, y2: endX };
						bottomValue = {x1: x1- width/2 - padding/2, y1: startX, x2: x1+ width/2 + padding/2, y2: startX };
						leftValue = {x1: x1- width/2 - padding/2, y1: endX, x2: x1- width/2 - padding/2, y2: startX };
						rightValue = {x1: x1+ width/2 + padding/2, y1: endX, x2: x1+ width/2 + padding/2, y2: startX };
					}
						for (l= 0; l< 4; l++){
							switch (l){
								case 0:
									newId = id + "_top";
									value = topValue;
									break;
								case 1:
									newId = id + "_bottom";
									value = bottomValue;
									break;
								case 2:
									newId = id + "_left";
									value = leftValue;
									break;
								case 3:
									newId = id + "_right";
									value = rightValue;
									break;
							}
							vectorColl[0] = ej.Ej3DRender.prototype.vector3D.prototype.vector3D(value.x1, value.y1, depth);
							vectorColl[1] = ej.Ej3DRender.prototype.vector3D.prototype.vector3D(value.x2, value.y2, depth);
							actual3DPosition1 = ej.EjSvgRender.chartTransform3D.toScreen(vectorColl[0], transform, result, matrix3D);
							actual3DPosition2 = ej.EjSvgRender.chartTransform3D.toScreen(vectorColl[1], transform, result, matrix3D);
							options = {
								'id': newId,
								'x1': actual3DPosition1.x,
								'y1': actual3DPosition1.y,
								'x2': actual3DPosition2.x,
								'y2': actual3DPosition2.y,
								'stroke': borderColor,
								'stroke-width': borderWidth
							};
							this.svgRenderer.drawLine(options, this.gMultiLevelEle);
						}
					   this.svgRenderer.append(this.gMultiLevelEle, this.svgObject);
				}
			  }
			}
		},
		
        _drawAxisLabel: function (axis, params) {
            var labelsCount = axis.visibleLabels.length,
				areaBounds = this.model.m_AreaBounds,
                opposedPosition = axis._opposed,
				validCross = params.axes[axis.name]._validCross,
				y = validCross ? axis.y : areaBounds.Y + (!opposedPosition && areaBounds.Height),
				x = validCross ? axis.x : areaBounds.X + (opposedPosition && areaBounds.Width);
           
            for (var i = 0; i < labelsCount; i++) {
				if (!ej.util.isNullOrUndefined(axis.visibleLabels[i].Text)) {
                var x1 = 0, y1 = 0, x2 = 0, y2 = 0;
                var textAnchor;
                
                var textSize = ej.EjSvgRender.utils._measureText(axis.visibleLabels[i].Text, this.model.m_AreaBounds.Width, axis.font);
				var value = (axis.visibleLabels[i].Value - axis.visibleRange.min) / (axis.visibleRange.delta);
                value = (axis.isInversed) ? 1 - value : value;
                value = isNaN(value) ? 0 : value;

                if (axis.orientation.toLowerCase() == "horizontal") {
                    x2 = x1 = (Math.round(axis.width * value)) + this.model.m_AreaBounds.X + ((axis.plotOffset));
                    y1 = (opposedPosition) ? (y - this.model.wallSize - (axis.majorTickLines.size) - (this.model.elementSpacing/2)) : y + this.model.wallSize + (axis.majorTickLines.size) + this.model.elementSpacing;
                    textAnchor="middle"
                }
                else {
                    y1 = y2 = Math.round(axis.plotOffset + axis.y + (textSize.height / 4) + (axis.height * (1 - value)));
                    x1 = (opposedPosition) ? (axis.x + axis.majorTickLines.size + axis.axisLine.width + this.model.elementSpacing/2 ) : (x - this.model.wallSize - axis.majorTickLines.size - this.model.elementSpacing / 2);
                    textAnchor = (opposedPosition) ?"start" : "end"
                }
 
             
                element = { Width: textSize.width, Height: textSize.height, Label: axis.visibleLabels[i], TextAnchor: textAnchor, tag: 'text', font: axis.font, id: this.svgObject.id + axis.orientation + i, child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
				}
            }

        },

        _renderTicks3D: function (axis, size, width, params) {
            var labelsCount = axis.visibleLabels.length,
             parent,
             gEle,
             m_AreaBounds = this.model.m_AreaBounds,
			 y = axis.y;

            for (var i = 0; i < labelsCount; i++)
            {
                var x1 = 0, x2 = 0, y1=0, y2=0;
            
       
                var value = (axis.visibleLabels[i].Value - axis.visibleRange.min) / (axis.visibleRange.delta);
                value = (axis.isInversed) ? 1 - value : value;
                value = isNaN(value) ? 0 : value;
            
                if (axis.orientation.toLowerCase() == "horizontal")
                {
                    x2 = x1 = (Math.round(axis.width * value)) + m_AreaBounds.X + ((axis.plotOffset));
                }
                else
                {
                    y1 = y2 = Math.round(axis.plotOffset + ((axis.height) * (1 - value))+axis.y);
                }

                var position = this._calculatePosition3D(axis, axis.tickLinesPosition, size, width, x1, y1, x2, y2, params);


                var line = { width: axis.majorTickLines.width, stroke: axis.majorTickLines.color, child: this.chart3D,  tag: 'line' };

                line.id = this.svgObject.id + axis.name + "_majorTickLines_" + i;

                this.graphics.addVisual(this.polygon.createLine(line, position.X1, position.Y1, position.X2, position.Y2, 0));

                if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i < labelsCount - 1) {
                    minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                    for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                        value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k + 1)));

                        value = isNaN(value) ? 0 : value;

                        if (axis.orientation.toLowerCase() == "horizontal") {
                            x1 = x2 = Math.round(axis.plotOffset + (m_AreaBounds.Width * value) + m_AreaBounds.X);
                        }
                        else {
                            y1 = y2 = Math.round(axis.plotOffset + ((m_AreaBounds.Height) * (1 - value))) + axis.y;
                        }

                        var position = this._calculatePosition3D(axis, axis.tickLinesPosition, size, width, x1, y1, x2, y2, params);


                        var line = { width: axis.minorTickLines.width, stroke: axis.minorTickLines.color, child: this.chart3D, tag: 'line' };

                        line.id = this.svgObject.id + axis.name + "_minorTickLines_" + i + k;

                        this.graphics.addVisual(this.polygon.createLine(line, position.X1, position.Y1, position.X2, position.Y2, 0));
                    }
                }
            }
        },

        _calculatePosition3D: function (axis, ticksPosition, tickSize, width, x1, y1, x2, y2, params) {
            var orientation = axis.orientation;
            var isOpposed = axis._opposed,
				ticksPosition = "outside",
				areaBounds = this.model.m_AreaBounds,
				validCross = params.axes[axis.name]._validCross,
				y = validCross ? axis.y : areaBounds.Y + (!isOpposed && areaBounds.Height),
				x = validCross ? axis.x : areaBounds.X + (isOpposed && areaBounds.Width);
            if (axis.orientation.toLowerCase() == "horizontal") {
                switch (ticksPosition) {
                    case "inside":
                        y1 = isOpposed ? width : 0;
                        y2 = isOpposed ? y1 + tickSize : tickSize;
                        break;
                    case "outside":
                        y1 = 0;
                        y2 = isOpposed ? tickSize : y1 + tickSize;
                        break;
                }                
                var screenPositionTop = ((isOpposed) ? y - this.model.wallSize - (tickSize) : y + this.model.wallSize - (tickSize / 2));
                var screenPositionLeft = axis.x;
                y1 += screenPositionTop;
                y2 += screenPositionTop;

                x1 = x2 = x1;
            }
            else {
                switch (ticksPosition) {
                    case "inside":
                        x1 = 0;
                        x2 = isOpposed ? tickSize : x1 + tickSize;
                        break;
                    case "outside":
                        x1 = 0;
                        x2 = isOpposed ? x1 + tickSize : tickSize;
                        break;
                }
                var screenPositionLeft = (isOpposed) ? x + this.model.wallSize  : (x - this.model.wallSize - tickSize);
                var screenPositionTop = this.model.m_AreaBounds.Y;
                x1 += screenPositionLeft;
                x2 += screenPositionLeft;

                y1 = y2 = y1;
            }
            return { X1: x1, Y1: y1, X2: x2, Y2: y2 };
        },

    

        _drawGridLines3D:function(axis, params)
        {          
            if (axis == null)
                return;
        
            var labelsCount = axis.visibleLabels.length,
                minorTicks, opposedPosition = axis._opposed,
                orientation = axis.orientation, validCross = params.axes[axis.name]._validCross,
                x1, x2, y1, y2;
            if (orientation.toLowerCase() == "horizontal")
            {
                 
                var i;
                for (i = 0; i < labelsCount; i++)
                {                
                    var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value);
                    value = isNaN(value) ? 0 : value;
					 x2 = x1 = (Math.round(axis.width * value)) + this.model.m_AreaBounds.X+axis.plotOffset;
                    y1 = this.model.m_AreaBounds.Y;
                    y2 = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
              
                
                    var depth = this.model.depth > 2 ? this.model.depth - 2 : 1;
                    var bottom = validCross ? axis.y : this.model.m_AreaBounds.Y + (!opposedPosition && this.model.m_AreaBounds.Height);

                    var line = { opacity: axis.majorGridLines.opacity, width: axis.majorGridLines.width, stroke: axis.majorGridLines.color, child: this.chart3D,  tag: 'line' };
                    line.id = this.svgObject.id + axis.name + "_gridlines_" + i;
                    this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));
                    var parallelLine = $.extend({}, line);
                    parallelLine.id = this.svgObject.id + axis.name + "_parallelGridlines_" + i;
                  
                    var line3D =   this.polygon.createLine(parallelLine, x2, 0, x2, -depth, bottom);
                    //To fold the gridline alone the wall(bottom)
                    this.polygon.transform(this.matrixobj.tilt((parseFloat)(Math.PI / 2)), line3D);
                        
                    this.graphics.addVisual(line3D);
                    
                    if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i<labelsCount-1) {
                        minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                        for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                            value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k+1)));
                            value = isNaN(value) ? 0 : value;
                            x2 = x1 = (Math.round(this.model.m_AreaBounds.Width * value) + this.model.m_AreaBounds.X);
                            y1 = this.model.m_AreaBounds.Y;
                            y2 = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;

                            var line = { opacity: axis.minorGridLines.opacity, width: axis.minorGridLines.width, stroke: axis.minorGridLines.color, child: this.chart3D, tag: 'line' };
                            line.id = this.svgObject.id + axis.name + "_minorgridlines_" + i + k;
                            this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));
                            var parallelLine = $.extend({}, line);
                            parallelLine.id = this.svgObject.id + axis.name + "_parallelMinorGridlines_" + i + k;

                            var line3D = this.polygon.createLine(parallelLine, x2, 0, x2, -depth, bottom);
                            //To fold the gridline alone the wall(bottom)
                            this.polygon.transform(this.matrixobj.tilt((parseFloat)(Math.PI / 2)), line3D);

                            this.graphics.addVisual(line3D);
                        }
                    }
                
                    index++;
                }
            }
            else
            {
                var i;
       
              
                for (i = 0; i < labelsCount; i++)
                {
                    //if (i < linesCount)
                    //{
                    var line;
					var value = (axis.visibleLabels[i].Value - axis.visibleRange.min) / (axis.visibleRange.delta); 
                    value = (axis.isInversed) ? 1 - value : value;
                    value = isNaN(value) ? 0 : value;
                    x1 = this.model.m_AreaBounds.X;
                    y1 = Math.round((axis.height) * (1 - value)) + 0.5;
                    y1 += axis.y;
                    x2 = x1 + this.model.m_AreaBounds.Width;
                    y2 = y1;

                    var depth = this.model.depth > 2 ? this.model.depth - 2 : 1;

                    var line = { opacity: axis.majorGridLines.opacity, width: axis.majorGridLines.width, stroke: axis.majorGridLines.color, axisName: axis.name, child: this.chart3D, tag: 'line' };
                 
                    line.id = this.svgObject.id + axis.name + "_gridlines_" + i;

                    this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));

                    var depthD = validCross ? axis.x : this.model.m_AreaBounds.X + (opposedPosition && this.model.m_AreaBounds.Width + 1);

                    var sideLine = $.extend({}, line);
                    sideLine.id = this.svgObject.id + axis.name + "_parallelGridlines_" + i;
              
                    var line3D = this.polygon.createLine(sideLine, -depth, y2, 0, y2, depthD);
                    //To fold the gridline alone the wall(right of vertical)
                    this.polygon.transform(this.matrixobj.turn((parseFloat)(-Math.PI / 2)), line3D);
                    this.graphics.addVisual(line3D);

                    if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i < labelsCount - 1) {
                        minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                        for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                            var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k + 1)));
                            value = isNaN(value) ? 0 : value;
                            x1 = this.model.m_AreaBounds.X;
                            y1 = Math.round((axis.height) * (1 - value)) + 0.5;
                            y1 += axis.y;
                            x2 = x1 + this.model.m_AreaBounds.Width;
                            y2 = y1;
                            var line = { opacity: axis.minorGridLines.opacity, width: axis.minorGridLines.width, stroke: axis.minorGridLines.color, axisName: axis.name, child: this.chart3D, tag: 'line' };

                            line.id = this.svgObject.id + axis.name + "_minorgridlines_" + i + k;

                            this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));                          

                            var sideLine = $.extend({}, line);
                            sideLine.id = this.svgObject.id + axis.name + "_parallelMinorGridlines_" + i + k;

                            var line3D = this.polygon.createLine(sideLine, -depth, y2, 0, y2, depthD);
                            //To fold the gridline alone the wall(right of vertical)
                            this.polygon.transform(this.matrixobj.turn((parseFloat)(-Math.PI / 2)), line3D);
                            this.graphics.addVisual(line3D);
                        }
                    }

                    ////}
                    index++;
                }

            }
        }

    },

    

    ej.Ej3DSeriesRender.prototype={
        _getSegmentDepth:function(series)
        {
            var actualDepth = this.chartObj.model.depth;
            var start, end;

            if (this.chartObj.model._sideBySideSeriesPlacement)
            {
                var space = actualDepth / 4;
                start = space;
                end = space * 3;
            }
            else
            {
                var index = series.position-1;
                var count = series.all;
                var space = actualDepth / ((count * 2) + count + 1);
                start = space + (space * index * 3);
                end = start + space * 2;
            }
            return {Start:start, End:end, Delta:end-start};
        },

        createSegment: function (center, start, end, height, r, i, y, insideRadius, pointindex) {
         
            return {
                StartValue : start,
                EndValue : end,
                depth : height,
                radius : r,      
                index : i,
                YData : y,
                Center : center,
                inSideRadius : insideRadius,
                ActualEndValue : end,
                ActualStartValue: start,
                pointIndex: pointindex
            }

            return segment;

        },

        calculateSize:function(sender, series){
            var legend = sender.chartObj.model.legend;
   
            var legXSpace = 0;
            var legYSpace = 0,
                title = sender.chartObj.model.title ,
                subTitle =  sender.chartObj.model.title.subTitle ,
                titleLocation = sender.chartObj.model._titleLocation,
                subTitleLocation = sender.chartObj.model._subTitleLocation,
                titleTextOverflow = sender.chartObj.model.title.textOverflow,
                subTitleTextOverflow = sender.chartObj.model.title.subTitle.textOverflow,
                titleEnable=(title.text && title.visible && title.enableTrim && (titleTextOverflow == 'wrap' || titleTextOverflow == 'wrapandtrim'))?true:false;
                subTitleEnable=(subTitle.text && subTitle.visible && subTitle.enableTrim && (subTitleTextOverflow == 'wrap' || subTitleTextOverflow == 'wrapandtrim'))?true:false;
            if (legend.visible && legend.position.toLowerCase() != "custom") {
                if (legend.position.toLowerCase() == "right" || legend.position.toLowerCase() == "left")
                    legXSpace = ((legend.position.toLowerCase() == "right") ? sender.chartObj.model.margin.right : sender.chartObj.model.margin.left) + sender.chartObj.model.LegendViewerBounds.Width;
                else
                    legYSpace = ((legend.position.toLowerCase() == "top") ? sender.chartObj.model.margin.top : sender.chartObj.model.margin.bottom) + sender.chartObj.model.LegendViewerBounds.Height;

            }
            series.actualWidth = $(sender.chartObj.svgObject).width() - legXSpace;
            var centerx = series.actualWidth * 0.5 + ((legend.position.toLowerCase() === "left") ? legXSpace : 0);
            if (titleEnable || subTitleEnable){
                var yOffset = titleLocation.size.height + (subTitleEnable ? subTitleLocation.size.height : 0) + legYSpace;
                series.actualHeight = $(sender.chartObj.svgObject).height() - yOffset;
                var centery = series.actualHeight * 0.5 + ((legend.position.toLowerCase() === "top") ? yOffset : titleLocation.size.height + (subTitleEnable ? subTitleLocation.size.height : 0));
            }
            else {
                var yOffset = ((title.text && title.visible) ? titleLocation.Y : 0) + legYSpace;
                series.actualHeight = $(sender.chartObj.svgObject).height() - yOffset;
                var centery = series.actualHeight * 0.5 + ((legend.position.toLowerCase() === "top") ? yOffset : ((title.text && title.visible) ? (titleLocation.Y ) : 0));
            }
            return { centerX: centerx, centerY: centery };
        },

        createPoints: function (series, sender) {
            series.segments = [];
            var size = this.calculateSize(sender, series),
                all = 0,
                visiblepoints = this._calculateVisiblePoints(series).visiblePoints,
                count = series._visiblePoints.length;
            for (j = 0; j < count; j++)
                all += visiblepoints[j].YValues[0];
            var coef = 360 / all,
                seriesIndex = $.inArray(series, sender.chartObj.model._visibleSeries),
                seriesLength = this.chartObj.model._visibleSeries.length,
                InsideRadius = sender.chartObj.model.innerRadius[seriesIndex],
                YValues = sender.chartObj._getYValues(series._visiblePoints),
                pieHeight = sender.chartObj.model.depth, center,
                arcStartAngle = 0, arcEndAngle = 0,
                current = 0, pointindex, val, rect, offset, segment,
                segindex = 0,
                radius = sender.chartObj.model.circularRadius[seriesIndex];

            for (var i = 0; i < count; i++) {
                if (series._visiblePoints[i].visible || series._visiblePoints[i].gapMode) {
                    pointindex = $.inArray(series._visiblePoints[i], series._visiblePoints);
                val = Math.abs(YValues[i]);
                arcEndAngle = Math.abs(val) * ((Math.PI * 2) / all);
                rect = {};
                rect.x = 0;
                rect.y = 0;

                if (val != 0) {
                    if (series.explodeIndex == series._visiblePoints[i].actualIndex || series.explodeAll) {
                        offset = {X:0, Y:0};
                        offset.X = Math.cos(2 * Math.PI * (current + val / 2) / all),
                        offset.Y = Math.sin(2 * Math.PI * (current + val / 2) / all);
                        rect.x = 0.01 * radius * offset.X * series.explodeOffset;
                        rect.y = 0.01 * radius * offset.Y * series.explodeOffset;
                    }

                    center = sender.chartObj.vector.vector3D(rect.x +  size.centerX, rect.y + size.centerY, 0);
                    {
                            segment = this.createSegment(center, (parseFloat)(coef * current), (parseFloat)(coef * val), pieHeight, radius, i, val, InsideRadius, pointindex, series);
                            if (series._visiblePoints[i].gapMode)
                                segment.visible = false;
                            else
                                segment.visible = true;
                            series.segments.push(segment);
                    }
                }

                if (series.marker.dataLabel.visible)
                    this._addPieDataLabel(segindex, YValues[i], arcStartAngle, arcStartAngle + arcEndAngle, i, radius, this._isChartRotated(sender) ? sender.chartObj.model.depth + 5 : 0, center, series._visiblePoints[i]);

                segindex++;
                arcStartAngle += arcEndAngle;
                current += val;

                }
            }
            return series.segments;
        },

        _addPieDataLabel: function (x, y, startAngle, endAngle, i, radius, startDepth, center, point) {
            var angle = (startAngle + endAngle) / 2;
            point.symbolLocation = { x: 0, y: 0, radius: 0, angle: 0, StartDepth: 0 };
            point.symbolLocation.x = x;
            point.symbolLocation.y = y;
            point.symbolLocation.radius = radius;
            point.symbolLocation.angle = angle;
            point.symbolLocation.center = center;
            point.startDepth = startDepth;
        },

        _isChartRotated:function(sender)
           {
            var actualTiltView = Math.abs(sender.chartObj.model.tilt % 360);
            var actualRotateView = Math.abs(sender.chartObj.model.rotattion % 360);
            if ((actualTiltView > 90 && actualTiltView < 270) ^ (actualRotateView > 90 && actualRotateView < 270))
              {
                return true;
              }
                  return false;
          },

        createSector: function (seg, sender, style, seriesIndex) {
            var count = (parseInt)(Math.ceil(seg.ActualEndValue / 6));
            var DtoR = Math.PI / 180;
            var inc = 0;
            var Points = [];
            var depth = sender.chartObj.model.depth;
            if (count < 1) return null;
            var res = [];
            var f = seg.ActualEndValue / count;

            var oPts = [];
            var iPts = [];

            for (var i = 0; i < count + 1; i++) {
                var ox = (parseFloat)(seg.Center.x + seg.radius * Math.cos((seg.ActualStartValue + i * f) * DtoR));
                var oy = (parseFloat)(seg.Center.y + seg.radius * Math.sin((seg.ActualStartValue + i * f) * DtoR));

                oPts[i] = { X: ox, Y: oy };

                var ix = (parseFloat)(seg.Center.x + seg.inSideRadius * Math.cos((seg.ActualStartValue + i * f) * DtoR));
                var iy = (parseFloat)(seg.Center.y + seg.inSideRadius * Math.sin((seg.ActualStartValue + i * f) * DtoR));

                iPts[i] = { X: ix, Y: iy };
                Points.push({ X: ox, Y: oy });
            }

            var oPlgs = [];

            for (var i = 0; i < count; i++) {
                vts = new Array(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, 0),
                       sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, depth),
                       sender.chartObj.vector.vector3D(oPts[i + 1].X, oPts[i + 1].Y, depth),
                       sender.chartObj.vector.vector3D(oPts[i + 1].X, oPts[i + 1].Y, 0))


                oPlgs[i] = sender.chartObj.polygon.polygon3D(vts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString()  + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D);
                inc++;
            }

            res[1] = oPlgs;

            if (seg.inSideRadius > 0) {
                var iPlgs = [];

                for (var i = 0; i < count; i++) {
                    vts = new Array(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, 0),
                      sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[i + 1].X, iPts[i + 1].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[i + 1].X, iPts[i + 1].Y, 0))


                    iPlgs[i] = sender.chartObj.polygon.polygon3D(vts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D);
                    inc++;
                }

                res[3] = iPlgs;
            }

            var tVtxs = [];
            var bVtxs = [];

            for (var i = 0; i < count + 1; i++) {
                tVtxs.push(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, 0));
                bVtxs.push(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, depth));
            }

            if (seg.inSideRadius > 0) {
                for (var i = count; i > -1; i--) {
                    tVtxs.push(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, 0));
                    bVtxs.push(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, depth));
                }
            }
            else {
                tVtxs.push(seg.Center);
                bVtxs.push(sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth));
            }
            res[0] = [];
            res[0].push(sender.chartObj.polygon.polygon3D(tVtxs, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D));
            inc++;
            res[0].push(sender.chartObj.polygon.polygon3D(bVtxs, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D));
            inc++;



            if (seg.inSideRadius > 0) {
                var rvts = new Array(

                      sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, 0),
                      sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[0].X, iPts[0].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[0].X, iPts[0].Y, 0)
                 )

                var lvts = new Array
                 (
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(iPts[count].X, iPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(iPts[count].X, iPts[count].Y, 0)
                 )
                res[2] = [];

                res[2].push(sender.chartObj.polygon.polygon3D(rvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;
                res[2].push(sender.chartObj.polygon.polygon3D(lvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;             

            }
            else {
                var rvts = new Array(
                     sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, 0)
                )

                var lvts = new Array(
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, 0)
                )
                res[2] = [];
                res[2].push(sender.chartObj.polygon.polygon3D(rvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;
                res[2].push(sender.chartObj.polygon.polygon3D(lvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + seriesIndex + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;


            }


            return res;
        },

        createPolygons: function (series, sender, returnPoly) {
            var segments, poligons = [], seg, style,
                pointindex, segmentsLength, plgs,
                seriesIndex = $.inArray(series, sender.chartObj.model._visibleSeries);
            if (!series.segments || !returnPoly)
                segments = this.createPoints(series, sender);
            else
                segments = series.segments;
            segmentsLength = segments.length;
            for (var i = 0; i < segmentsLength; i++) {
                seg = segments[i];
                if (seg.visible) {
                    pointindex = segments[i].pointIndex;
                    style = sender.chartObj.setStyle(sender, series, seriesIndex, pointindex);
                    if (typeof (sender.chartObj.model.pointColors[pointindex]) == "object")
                        style.interior = sender.chartObj.model.pointColors[pointindex][0].color;
                    plgs = this.createSector(seg, sender, style, seriesIndex);
                    if (plgs != null)
                        for (var ai = 0; ai < plgs.length; ai++) {
                            if (!poligons[ai])
                                poligons[ai] = [];
                            if (plgs[ai] != null) {
                                for (var pi = 0; pi < plgs[ai].length; pi++) {
                                    poligons[ai].push(plgs[ai][pi]);

                                }
                            }
                        }
                    }


            }
            if (returnPoly) {
                return poligons;               
            }

            for (var ai = 0; ai < poligons.length; ai++) {
                for (var k = 0; k < poligons[ai].length; k++) {
                    sender.chartObj.graphics.addVisual(poligons[ai][k]);
                }
            }
        },

        draw3DDataLabel: function (series, pointIndex, point, sender) {

            var connectorHeight = series.marker.dataLabel.connectorLine.height;
            var tag;
            if (this.chartObj.model.AreaType == "none") {
                location = { x: 0, y: 0 };
                var pointText = (point.text) ? point.text : point.y;
                var seriesIndex = $.inArray(series, this.chartObj.model.series);
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { text: pointText, location: { x: pointX, y: pointY }, series: series, pointIndex: pointIndex ,seriesIndex:seriesIndex};
                this.chartObj._trigger("displayTextRendering", commonEventArgs);
                commonEventArgs.data.Text = commonEventArgs.data.text;
                var size = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);

                if (series.type.toLowerCase() == "doughnut"  || series.type.toLowerCase() == "pie")
                {
                   
                    var width = Math.min(series.actualWidth, series.actualHeight) / 2;
                    var center = point.symbolLocation.center;

                    var dradius = point.symbolLocation.radius * series._coefficient;
                    if (ej.util.isNullOrUndefined(connectorHeight))
                         connectorHeight = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, series.marker.dataLabel.font).height;
                    if (series != null &&series.labelPosition != "inside")
                        radius = point.symbolLocation.radius +  connectorHeight;
                    else if (series != null )
                        radius = dradius + (point.symbolLocation.radius-dradius) / 2;
                    pointX=location.X = center.x + radius * Math.cos(point.symbolLocation.angle);
                    pointY=location.Y = center.y + radius * Math.sin(point.symbolLocation.angle);
                }
            }
            else {
                var labelFormat = series.yAxis.labelFormat ? series.yAxis.labelFormat : "";
                var seriesIndex = $.inArray(series, this.chartObj.model.series);
                var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
                var pointHeight = 0;
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { text: pointText, location: { x: pointX, y: pointY }, series: series, pointIndex: pointIndex };
                this.chartObj._trigger("displayTextRendering", commonEventArgs);
                commonEventArgs.data.Text = commonEventArgs.data.text;
                var size = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);

                var location = ej.EjSvgRender.utils.Transform3DToVisible(series, point.symbolLocation.x, point.symbolLocation.y, this.chartObj);
                var connectorHeight = (series.marker.dataLabel.connectorLine.height) ? series.marker.dataLabel.connectorLine.height : 0;
                var angle = (6.28 * (1 - (90 / 360.0)));
                var pointY = location.Y;
                var pointX = location.X;
                if (!this.chartObj.model.requireInvertedAxes) {
                    if (point.y > 0)
                        pointY = location.Y + (Math.sin((angle)) * connectorHeight);
                    else
                        pointY = location.Y + (Math.sin((-angle)) * connectorHeight);
                }
                else {
                    if (point.x > 0)
                        pointX = location.X - (Math.sin((angle)) * connectorHeight);
                    else
                        pointX = location.X + (Math.sin((angle)) * connectorHeight);
                      pointHeight = - size.height / 3;
                }
            }

         
          
            var depthInfo = this._getSegmentDepth(series);

            

            if (series.marker.dataLabel.shape.toLowerCase() != "none" && !point.marker.dataLabel.template && commonEventArgs.data.Text != '') {
              
                var element = { tag: 'dataLabel', series: series, point: point, pointIndex: pointIndex, id: this.chartObj.svgObject.id + seriesIndex + '_DataLabel' + pointIndex, child: this.chartObj.chart3D };

                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(pointX, pointY, (point.symbolLocation.z) ? point.symbolLocation.z : 0), element, 0, -size.height));
            }

            tag = (!point.marker.dataLabel.template) ? "text" : "template";
             
            var element = { Width: size.width, Height: size.height, Label: commonEventArgs.data, TextAnchor: "middle", tag: tag, font: series.marker.dataLabel.font, Angle: series.marker.dataLabel.angle, id: this.chartObj.svgObject.id + '_SeriesText' + pointIndex + seriesIndex, child: this.chartObj.chart3D };
           

            if (this.chartObj.model.AreaType != "none") {
                if (series.marker.dataLabel.connectorLine.height && series.marker.dataLabel.connectorLine.height > 0) {
                    var drawPoints = [];
                    drawPoints.push({ x: location.X, y: location.Y + pointHeight, z: point.symbolLocation.z });

                    drawPoints.push({ x: pointX, y: pointY + pointHeight, z: point.symbolLocation.z });
                    this.drawLineSegment(drawPoints, pointIndex, series);
                }
                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(pointX, pointY, point.symbolLocation.z), element, 0, -size.height));
            }
            else {
                  if (series.labelPosition.toLowerCase() != "inside")
                    this.updateConnectorLine(point, pointIndex, series, location, connectorHeight)
                var finalSize = { x: series.actualWidth, y: series.actualHeight };
                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(location.X, location.Y, -1), element, 0, -size.height));
            }        
        },
        updateConnectorLine: function (point, pointIndex, series, location, connectorHeight) {
            var drawPoints = [];
            var symbolLocation = point.symbolLocation;
            var x = symbolLocation.center.x + Math.cos(symbolLocation.angle) * symbolLocation.radius;
            var y = symbolLocation.center.y + Math.sin(symbolLocation.angle) * symbolLocation.radius;
            drawPoints.push({ x: x, y: y });
            var labelRadiusFromOrigin = symbolLocation.radius + connectorHeight;
            x = symbolLocation.center.x + (Math.cos((symbolLocation.angle)) * (labelRadiusFromOrigin));
            y = symbolLocation.center.y + (Math.sin((symbolLocation.angle)) * (labelRadiusFromOrigin));
           
            drawPoints.push({ x: x, y: y });
            //var hipen = height / 5;
            //x += (x > symbolLocation.center.x) ? hipen : -hipen;
            //drawPoints.push({ x: x, y: y });
            if (series.marker.dataLabel.connectorLine.type =="bezier")
                drawPoints = this.getBezierApproximation(drawPoints, 256, series);
            this.drawLineSegment(drawPoints, pointIndex, series)
        },
        /// <returns></returns>
        getBezierApproximation: function (controlPoints, outputSegmentCount, series)
        {
            var points = [];
             for (var i = 0; i <= outputSegmentCount; i++)
             {
               var t =i / outputSegmentCount;
               points.push(this.GetBezierPoint(t, controlPoints, 0, controlPoints.length, series));
             }
            return points;
         },
        drawLineSegment: function (drawpoints, pointIndex, series) {
            var vectorPoints = [];
            for (var i = 0; i < drawpoints.length; i++) {
                vectorPoints.push(this.chartObj.vector.vector3D(drawpoints[i].x, drawpoints[i].y, drawpoints[i].z ? drawpoints[i].z:0));
            }
            var seriesIndex=$.inArray(series, this.chartObj.model._visibleSeries);
            var color = (this.chartObj.model.AreaType=="none") ? this.chartObj.model.pointColors[pointIndex] : this.chartObj.model.seriesColors[seriesIndex];
            var stroke = (series.marker.dataLabel.connectorLine.stroke)?series.marker.dataLabel.connectorLine.stroke:color;
            var line = { width: series.marker.dataLabel.connectorLine.width, stroke: stroke, child: this.chartObj.chart3D, tag: 'polyline', id:this.chartObj.svgObject.id + "_" + seriesIndex + "_" + pointIndex };
          
            this.chartObj.graphics.addVisual(this.chartObj.polygon.createPolyline(vectorPoints, line));
        }

    },

     ej.ejCircularSeries = {

         draw: function (series, sender, type) {
             var MARGINS_RATIO = 0.03;
             
             seriesLength = sender.chartObj.model._visibleSeries.length,
             seriesIndex = $.inArray(series, sender.chartObj.model._visibleSeries);
             sender._calculateVisiblePoints(series);
              sender.calculateSize(sender, series);
             
             if (type == "pie") {
                 series._size = series.pieCoefficient;
                 series._coefficient = seriesIndex == 0 ? 0 : series._size ;
             }
             else {
                 series._size = series.doughnutSize;
                 series._coefficient =  series.doughnutCoefficient;
             }
            if (sender.chartObj.model.circularRadius.length > 1) {
                 for (i = seriesIndex; i < sender.chartObj.model.circularRadius.length; i++) {
                     if (!ej.util.isNullOrUndefined(sender.chartObj.model.circularRadius[i])) {
                         sender.chartObj.model.circularRadius[seriesIndex] = sender.chartObj.model.circularRadius[i] * series._size;
                         break;
                     }
                 }
             }
             else
                 sender.chartObj.model.circularRadius[seriesIndex] = (((1 - MARGINS_RATIO) * Math.min(series.actualWidth / 2, series.actualHeight / 2)) * series._size);
             sender.chartObj.model.innerRadius[seriesIndex] = series._coefficient * sender.chartObj.model.circularRadius[seriesIndex];
             sender.createPolygons(series, sender);
         },
         doAnimation: function (series, sender) {
             var seriesRendering = this;
             series.count = 0;
             $.each(series.segments, function (pointIndex, point) {

                 var radius = point.radius;
                 var insideRadius = point.inSideRadius;

                 series.animate = true;
                 $(sender.chartObj.element).each(function () { point.radius = 0, point.inSideRadius=0 }).animate(
                     { Radius: radius, InsideRadius: insideRadius },

                     {
                         duration: 1000, queue: false, step: function (now, fx) {


                             if (fx.prop.toString() === "Radius") {
                                 point.radius = now;
                             }
                             else {
                                 point.inSideRadius = now;
                             }


                             var poligons = sender.createPolygons(series, sender, true);

                             for (var ai = 0; ai < poligons.length; ai++) {
                                 for (var k = 0; k < poligons[ai].length; k++) {                                     
                                     sender.chartObj.polygon.update(poligons[ai][k].VectorPoints, poligons[ai][k], sender)
                                 }
                             }
                            
                         },
                         complete: function () {
                             sender.chartObj.model.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             sender.chartObj._trigger("animationComplete", commonEventArgs);

                         }
                     });
             });
         }
     },

      ej.series3DTypes.doughnut = ej.ejCircularSeries;
      ej.series3DTypes.pie = ej.ejCircularSeries;

    

    ej.ej3DStackingColumn = {

        draw: function (series, sender, type, params) {
            var visiblePoints = sender._isVisiblePoints(series);
            this.createSegments(series, sender, params);
            for (var i = 0; i < visiblePoints.length; i++) {
                if (visiblePoints[i].visible) {
                    series._visiblePoints[i].plans = null;
                    ej.ej3DColumnSeries.update(series, series._visiblePoints[i], i, sender)
                }
            }
        },

        createSegments: function (series, sender, params) {
            var xValues = sender.chartObj._getXValues(series._visiblePoints);
          
            var median,
			 cons=0.2;
            if (xValues == null) return;
            
            var sbsInfo = sender.getSideBySideInfo(series, params);
            var depthInfo = sender._getSegmentDepth(series);           
            median = sbsInfo.Delta / 2;
            var visiblePoints = sender._isVisiblePoints(series);
            for (var i = 0; i < visiblePoints.length; i++) {

                var x1 = xValues[i] + sbsInfo.Start;
                var x2 = xValues[i] + sbsInfo.End;
                var y2 = series.stackedValue.StartValues[i];
                var y1 = series.stackedValue.EndValues[i];

                ej.ej3DColumnSeries._setData(x1, y1, x2, y2, depthInfo.Start, depthInfo.End, sender, visiblePoints[i]);

                if (!series.marker.dataLabel.visible) continue;
                visiblePoints[i].symbolLocation = { x: 0, y: 0, z: 0 };
                switch (series.marker.dataLabel.textPosition) {
                    case "top":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.Delta / 2);
                        break;
                    case "bottom":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y2 - cons;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.End - depthInfo.Start) / 2;
                        break;
                    default:
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1 + (y2 - y1) / 2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start;
                        break;
                }

            }

        },

        doAnimation: function (series, sender) {            

                var seriesRender = this;

                $.each(series._visiblePoints, function (pointIndex, point) {

                    var topValue = point.Top;
                    var bottomValue = point.Bottom;

                    $(point).each(function () { point.Top = 0, point.Bottom = 0}).animate(
                    { Top: topValue, Bottom: bottomValue },

                    {
                        duration: 1200, queue: false, step: function (now, fx) {

                            if (fx.prop.toString() === "Top") {
                                point.Top = now;
                            }
                            if (fx.prop.toString() === "Bottom") {
                                point.Bottom = now;
                            }

                            ej.ej3DColumnSeries.update(series, point, pointIndex, sender);

                        },
                        
                         complete: function () {                  
                       
                                sender.chartObj.model.AnimationComplete = true;
                                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                                commonEventArgs.data = { series: series };
                                sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                            }

                    });

                });

           }

        
        
    },

    ej.series3DTypes.stackingcolumn = ej.ej3DStackingColumn;

    ej.series3DTypes.stackingbar = ej.ej3DStackingColumn;
     
	ej.series3DTypes.stackingbar100 = ej.ej3DStackingColumn;
	 
	ej.series3DTypes.stackingcolumn100 = ej.ej3DStackingColumn;
   
    ej.ej3DColumnSeries = {
        draw: function (series, sender, type, params) {
            var visiblePoints = sender._isVisiblePoints(series);
            this.createSegments(series, sender, params);
            for (var i = 0; i < visiblePoints.length; i++) {
                if (visiblePoints[i].visible) {
                    series._visiblePoints[i].plans = null;
                    this.update(series, series._visiblePoints[i], i, sender);
                }
            }
        },

        update: function (series, point, pointIndex, sender) {
            var plans;
            var valueType = series.xAxis._valueType.toLowerCase();
            var seriesIndex = $.inArray(series, sender.chartObj.model._visibleSeries);    
            var xBase = (valueType == "logarithmic") ? null : 1;
            var xIsLogarithmic = (valueType == "logarithmic") ? true :false;
            var left = xIsLogarithmic ? Math.log(point.Left, xBase) : point.Left;
            var right = xIsLogarithmic ? Math.log(point.Right, xBase) : point.Right;
            var bottom = series.yAxis.visibleRange.min;
            var top = series.yAxis.visibleRange.max;
            var xStart = series.xAxis.visibleRange.min;
            var xEnd = series.xAxis.visibleRange.max;
            if ((!(left >= xStart) || !(left <= xEnd)) && (!(right >= xStart) || !(right <= xEnd))) return;
           
            var topValue;
            if(point.Top<0)
                topValue = point.Top > bottom ? point.Top : bottom;
            else
                topValue = (series.yAxis.valueType && series.yAxis.valueType.toLowerCase() == "logarithmic") ?  point.Top : point.Top < top ? point.Top : top;
            var tlpoint = ej.EjSvgRender.utils.Transform3DToVisible(series, point.Left > xStart ? point.Left : xStart, topValue, sender.chartObj);
            var rbpoint = ej.EjSvgRender.utils.Transform3DToVisible(series, xEnd > point.Right ? point.Right : xEnd, bottom > point.Bottom ? bottom : point.Bottom, sender.chartObj);
            
            var tlfVector = sender.chartObj.vector.vector3D(Math.min(tlpoint.X, rbpoint.X), Math.min(tlpoint.Y, rbpoint.Y), point.StartDepth);
            var brbVector = sender.chartObj.vector.vector3D(Math.max(tlpoint.X, rbpoint.X), Math.max(tlpoint.Y, rbpoint.Y), point.EndDepth);
           
            var styleOptions = sender.chartObj.setStyle(sender, series, seriesIndex, pointIndex);
            if (styleOptions.interior.indexOf("url") >= 0)
                styleOptions.interior = sender.chartObj.model.seriesColors[seriesIndex][0].color;
            var name = "Region" + "_Series_" + seriesIndex + "_Point_" + pointIndex
            if (series.columnFacet == "cylinder")

                sender.chartObj.polygon.createCylinder(tlfVector, brbVector, this, pointIndex, series.type,
                    styleOptions.borderColor, styleOptions.interior, styleOptions.borderWidth, styleOptions.opacity, sender.chartObj.model.requireInvertedAxes, name, sender.chartObj.chart3D);
            else if (series.columnFacet == "rectangle")
    			sender.chartObj.polygon.createBox(tlfVector, brbVector, this, pointIndex, series.type,
                    styleOptions.borderColor, styleOptions.interior, styleOptions.borderWidth, styleOptions.opacity, sender.chartObj.model.requireInvertedAxes, name, sender.chartObj.chart3D);
           
        },

        _setData:function()
        {  
            var values=arguments;
            point = arguments[arguments.length - 1];
            sender = arguments[arguments.length - 2];
        
            point.Left = values[0];
            point.Bottom = values[3];
            point.Top = values[1];
            point.Right = values[2];
            point.StartDepth = values[4];
            point.EndDepth = values[5];
            point.XRange = sender.getDoubleRange(point.Left, point.Right);
            if (!isNaN(point.Top) && !isNaN(point.Bottom))
                point.YRange = sender.getDoubleRange(point.Top, point.Bottom);
        },

        createSegments:function(series, sender, params)
        {
            var xValues = sender.chartObj._getXValues(series._visiblePoints),
                YValues = sender.chartObj._getYValues(series._visiblePoints);            
            if (xValues == null) return;
      
            var sbsInfo = sender.getSideBySideInfo(series, params),
                depthInfo = sender._getSegmentDepth(series);
                crossValue = sender.chartObj._getXCrossValue(series, series.xAxis, params),
                median = sbsInfo.Delta / 2,
            visiblePoints = series._visiblePoints,
			 cons = 0.2;
            for (var i = 0; i < visiblePoints.length; i++)
            {                 
                var x1 = xValues[i] + sbsInfo.Start,
                    x2 = xValues[i] + sbsInfo.End,
                    y1 = YValues[i],
                    y2 = crossValue;
                
                this._setData(x1, y1, x2, y2, depthInfo.Start, depthInfo.End, sender, visiblePoints[i]);

                XData = xValues[i];
                YData = YValues[i];
                //Item = ActualData[i];
              
                if (!series.marker.dataLabel.visible) continue;
                visiblePoints[i].symbolLocation = {x:0, y:0, z:0};
                switch (series.marker.dataLabel.textPosition) {
                    case "top":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.Delta / 2);
                        break;
                    case "bottom":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y2 - cons;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.End - depthInfo.Start) / 2;
                        break;
                    default:
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = Math.abs(y2 - y1) / 2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start;
                        break;
                }
        
            }

    
        },

        doAnimation: function (series, sender) {

            var seriesRender = this;

            $.each(series._visiblePoints, function (pointIndex, point) {

                var topValue = point.Top;
                $(point).each(function () { point.Top = 0 }).animate(
                { Top: topValue },

                {
                    duration: 1200, queue: false, step: function (now, fx) {

                        if (fx.prop.toString() === "Top") {
                            point.Top = now;
                        }                      

                        seriesRender.update(series, point, pointIndex, sender);

                    },
                    
                     complete: function () {                        
                            sender.chartObj.model.AnimationComplete = true;
                            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                            commonEventArgs.data = { series: series };
                            sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                        }

                });

            })

        }
     
    }

    ej.series3DTypes.column = ej.ej3DColumnSeries;

    //bar series
    ej.series3DTypes.bar = ej.ej3DColumnSeries;

    ej.Ej3DChart.prototype = {
        renderSeries: function (sender, series, params) {            

            var series;
            ej.Ej3DSeriesRender.prototype.chartObj = sender;
         
            $.extend(ej.Ej3DSeriesRender.prototype, ej.EjSeriesRender.prototype);
            
            var seriesRendering = new ej.Ej3DSeriesRender();
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = {"series":series};
            seriesRendering.chartObj._trigger("seriesRendering", commonEventArgs);
                var type = series.type.toLowerCase();
                var options = ej.series3DTypes[type];                
                if (ej.util.isNullOrUndefined(options)) return;
                options.draw(series, seriesRendering,type, params);
                if (series.marker.dataLabel.visible) {
                    $.each(series._visiblePoints, function (pointIndex, point) {
                        point.marker = $.extend(true, {}, series.marker, point.marker);
                       if (point.visible && (ej.util.isNullOrUndefined(point.marker) || (point.marker.dataLabel && point.marker.dataLabel.visible)))
                            seriesRendering.draw3DDataLabel(series, pointIndex, point, sender);
                    });
                }


               
 
        },
        update3DWall: function (sender, params) {
            if (sender.model.AreaType == "cartesianaxes") {
                $.extend(ej.Ej3DRender.prototype, this);
                this.updateBackWall(sender);                
                for (var i = 0; i < sender.model._axes.length; i++) {
                    var axis = sender.model._axes[i],
                        opposedPosition = axis._opposed;
                    if (axis.orientation.toLowerCase() == "vertical") {
                        if (!opposedPosition)
                            this.updateLeftWall(sender, axis, params);
                        else
                            this.updateRightWall(sender, axis, params);
                    }
                    else {
                        if (!opposedPosition)
                            this.updateBottomWall(sender, axis, params);
                        else
                            this.updateTopWall(sender, axis, params);
                    }
                }
            }                  
        },

      updateTopWall:function(sender, axis, params)
        {
        
            var offset = 0;
            var  areaBounds= sender.model.m_AreaBounds, y = params.axes[axis.name]._validCross ? axis.y : areaBounds.Y;			
            if (sender.model.wallSize < y)
                offset = y - sender.model.wallSize;
           else
                offset = -(sender.model.wallSize - y);
            var tlfVector = sender.vector.vector3D(areaBounds.X + areaBounds.Width, -sender.model.depth, y - 0.1);
            var brbVector = sender.vector.vector3D(areaBounds.X, - 0.1, offset);
            topSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, this, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "TopWallBrush", sender.chart3D);
  
          for (var i = 0; i < topSideWallPlans.length; i++)
              sender.polygon.transform(sender.matrixobj.tilt((parseFloat)(Math.PI / 2)), topSideWallPlans[i]);
    
                  
    },

       updateRightWall:function(sender, axis, params)
        {
		   var x = params.axes[axis.name]._validCross ? axis.x : sender.model.m_AreaBounds.X+sender.model.m_AreaBounds.Width;
           var rightRect = { left: -(sender.model.depth), top: sender.model.m_AreaBounds.Y, bottom: sender.model.m_AreaBounds.Height+sender.model.m_AreaBounds.Y, right: 0 };
           var tlfVector = sender.vector.vector3D(rightRect.left, rightRect.top, x + 1.5);
           var brbVector = sender.vector.vector3D(rightRect.right, rightRect.bottom, x + sender.model.wallSize);
           rightSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, sender, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "RightWallBrush", sender.chart3D);
           for (var i = 0; i < rightSideWallPlans.length; i++)
             sender.polygon.transform(sender.matrixobj.turn((parseFloat)(-Math.PI / 2)), rightSideWallPlans[i]);           
        },
        updateBackWall: function (sender) {
            var areaBounds = sender.model.m_AreaBounds;           
            var tlfVector = sender.vector.vector3D(areaBounds.X, areaBounds.Y, sender.model.depth == 0 ? 1.5 : sender.model.depth + sender.model.wallSize);
            var brbVector = sender.vector.vector3D((areaBounds.X + areaBounds.Width), areaBounds.Y + areaBounds.Height, sender.model.depth == 0 ? 1.5 : sender.model.depth);

            sender.polygon.createBox(tlfVector, brbVector, sender, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "BackWallBrush", sender.chart3D);
        },
        updateLeftWall: function (sender, axis, params) {
            var leftRect = { left: -(sender.model.depth), top: sender.model.m_AreaBounds.Y, bottom: sender.model.m_AreaBounds.Height+sender.model.m_AreaBounds.Y, right: 0 },
				offset = params.axes[axis.name]._validCross ? axis.x : sender.model.m_AreaBounds.X;
            var tlfVector = sender.vector.vector3D(leftRect.left, leftRect.top, offset - 0.1);
            var brbVector = sender.vector.vector3D(leftRect.right, leftRect.bottom, offset - sender.model.wallSize);

            leftSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, this, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "LeftWallBrush", sender.chart3D);
            for (var i = 0; i < leftSideWallPlans.length; i++)
                sender.polygon.transform(sender.matrixobj.turn((parseFloat)(-Math.PI / 2)), leftSideWallPlans[i]);
        },
        updateBottomWall: function (sender, axis, params) {
            var areaBounds = sender.model.m_AreaBounds;
			var y = params.axes[axis.name]._validCross ? axis.y : areaBounds.Y + areaBounds.Height;
            var tlfVector = sender.vector.vector3D((areaBounds.X + areaBounds.Width), -(sender.model.depth), sender.model.wallSize + y);
            var brbVector = sender.vector.vector3D(areaBounds.X, -0.1, y + 1);

            bottomSideWallPlans = sender.polygon.createBox(brbVector, tlfVector, this, 0, "Graphics3D", "#D3D3D3", "#D3D3D3", 0, 0.15, false, "BottomWallBrush", sender.chart3D);
            for (var i = 0; i < bottomSideWallPlans.length; i++)
                sender.polygon.transform(sender.matrixobj.tilt((parseFloat)(Math.PI / 2)), bottomSideWallPlans[i]);
        }
    }

})(jQuery);
ej.EjVmlRender = function (element) {
   var doc = document;
    this.vmlNamespace = "urn:schemas-microsoft-com:vml";
    this.vmStyle = "#default#VML";
    doc.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
    this.isdocmode8 = doc.documentMode === 8;
    if (!doc.getElementById("vml_chart")) {
        var cssText
        if (this.isdocmode8) {
            cssText =
                    'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' +
                    '{ behavior:url(#default#VML); display: inline-block; } ';
        }
        else {
            cssText =
               'v\\:*' +
               '{ behavior:url(#default#VML); display: inline-block; } ';
        }
        var vmlStyleSheet = doc.createStyleSheet();
        vmlStyleSheet.owningElement.id = 'vml_chart';
        vmlStyleSheet.cssText = cssText;
    }
    this._rootId = jQuery(element).attr("id");
    this.svgObj = doc.createElement("div");
    this.svgObj.style.position = 'relative';
    this.svgObj.setAttribute('id', this._rootId + '_vml');   
    this.changeOptions = {'id':'id',
        'fill': 'fillcolor', 'stroke': 'strokecolor', 'stroke-width': 'strokeweight', 'd': 'path', 'font-size': 'font-size', 'font-family': 'font-family', 'font-style': 'font-style',
        'font-weight': 'font-weight', 'points':'points'

        };
};
(function($) {
ej.EjVmlRender.prototype = {
  
    drawPath: function (options, element) {
        var darray= options.d.split(" ");
        if($.inArray('A', darray) != -1)
        {
            var w = parseFloat(darray[4]);
            var h = parseFloat(darray[5]);
            options.d=this.drawArc(w, h, options);
        }
        if ($("#" + options.id).length > 0) {
            options.d = this.changePathValue(options.d);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            options.d = this.changePathValue(options.d);
            var shape = document.createElement("v:shape");
            var $shapeEle = $(shape);
            this.applyVMLStyle($shapeEle, options);
            $shapeEle.appendTo(element);
        }
    },
    createLegendSvg:function(element)
    {
        var doc = document;        
        this._rootId = jQuery(element).attr("id");
        this.legendsvgObj = doc.createElement("div");
        this.legendsvgObj.style.position = 'relative';
        this.legendsvgObj.setAttribute('id',"legend_"+ this._rootId + '_vml');       
        return this.legendsvgObj;
    },

    drawPolyline: function (options, element) {
        var darray = options.points.split(" ");
        
        if ($("#" + options.id).length > 0) {
            options.points = this.changePathValue(options.points);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            options.points = this.changePathValue(options.points);
            var shape = document.createElement("v:polyline");
            var $shapeEle = $(shape);
            this.applyVMLStyle($shapeEle, options);
            $shapeEle.appendTo(element);
        }
    },
    setFillAttribute: function (element, options) {
        var fill = document.createElement("v:fill");
        var opacity = options["fill-opacity"] ? options["fill-opacity"] : options["opacity"];
        opacity = (!ej.util.isNullOrUndefined(opacity)) ? opacity : (options["opacity"]) ? options["opacity"] : 1;
        var fillcolor = options["fill"];
        if (fillcolor != "none" && !ej.util.isNullOrUndefined(fillcolor)) {
          
            if (!(fillcolor).match('gradient'))
                $(element).attr("fillcolor", fillcolor);
            else {
                var colorName = ((fillcolor).replace('gradient', ""));
                fill.setAttribute('type', "gradient");
                fill.setAttribute("colors", colorName);
            }
        }
        else if(fillcolor=='none'){
            fill.setAttribute('type',"gradient");
            fill.setAttribute('color',options.color);
            fill.setAttribute('color2',options.color2);
            fill.setAttribute('colors',options.colors);
            fill.setAttribute('angle', options.angle);
            opacity = options.fill_opacity;
        }
        else 
            opacity = 0.1;
        
        
        fill.setAttribute('opacity', opacity);
        $(fill).appendTo(element);
    },
    setStrokeAttribute: function (element, options) {
        var stroke = options['stroke'];
        if (stroke == "transparent") 
            options['opacity'] = 0.1;
        if (options.name) {
            $(element).attr('name', options.name);
        }
         
         
        var opacity = options["opacity"];
        if (opacity) {
            var fill = document.createElement("v:stroke");

           

            fill.setAttribute('opacity', opacity);
            $(fill).appendTo(element);
        }
    },
    changePathValue: function (options) {
        if (!options["d"]) {
            var pathArray = options.split(" ");
            for (var i = 0; i < pathArray.length; i++) {
                var matches = pathArray[i].match(/\d+/g);
                if (matches != null) {
                    pathArray[i] = Math.round(parseFloat(pathArray[i]));
                }
            }
            options = pathArray.join(' ');
            return options;
        }
    },


    drawArc: function ( w, h, options) {
    var x = 0, y = 0;
    var startAngle = options.start,
    endAngle =options.end,
    endAngle = (Number(Math.PI.toFixed(2)) == Number((endAngle + startAngle).toFixed(2))) ? endAngle - 0.01 : endAngle,
    radius = options.r || w || h,
    cosStart = Math.cos(startAngle),
    sinStart = Math.sin(startAngle),
    cosEnd = Math.cos(endAngle),
    sinEnd = Math.sin(endAngle),
    innerRadius = options.innerR,
    circleCorrection = 0.08 / radius, 
    innerCorrection = (innerRadius && 0.1 / innerRadius) || 0,
    path;

    if (endAngle - startAngle === 0) {
            return '';

    } else if (2 * Math.PI - endAngle + startAngle < circleCorrection) {
            cosEnd = -circleCorrection;
    } else if (endAngle - startAngle < innerCorrection) {
            cosEnd = Math.cos(startAngle + innerCorrection);
        }

        path = [
    'wa', // To draw clockwise arc
    x - radius, 
    y - radius, 
    x + radius, 
    y + radius, 
    x + radius * cosStart, 
    y + radius * sinStart, 
    x + radius * cosEnd, 
    y + radius * sinEnd  
        ];

        if (options.open && !innerRadius) {
            path.push(
        'e',
        M,
        x, 
        y
    );
        }

        path.push(
            // To draw anti clockwise arc
    'at', 
    x - innerRadius, 
    y - innerRadius, 
    x + innerRadius, 
    y + innerRadius, 
    x + innerRadius * cosEnd, 
    y + innerRadius * sinEnd, 
    x + innerRadius * cosStart, 
    y + innerRadius * sinStart, 
    'x', 
    'e' 
);
        path=path.join(" ");
        return path;

  },
    changeVMLStyle: function ($element, options) {
        var chart = this;
		var strokeColor = options.stroke;
        $element.css("width", options.width).css("visibility", options.visibility).css("height", options.height).css("position", "absolute").css('left', options.x).css('top', options.y);
        $.each(options, function (index, val) {
            if (chart.changeOptions[index] != null || chart.changeOptions[index] != undefined) {
                if (index == 'd' || index=='points') {
                    val = chart.changePathValue(options[index]).toLowerCase();
                }
                if (val == 'transparent' && !options.fill_opacity) {
                    options[index] = val = 'white';
                     if(index=="fill" && (!options.hasOwnProperty('opacity') ||  !options['opacity'])) 
                       options['opacity'] = 0.1;                  
                }
                if (options.fill_opacity) {
                    options['opacity'] = options.fill_opacity;
                }
                if ($element[0].tagName === "SPAN") {
                    if (index == "fill") $element.css('color', val);
                    else {
                        if (index == "id") $element.attr(chart.changeOptions[index], val);
                        else
                            $element.css(chart.changeOptions[index], val);
                    }
                }else {

                    if (chart.isdocmode8) { // IE8 setAttribute bug
                        $element.get(0)[chart.changeOptions[index]] = val;
                    } else {
                        $element.get(0).setAttribute(chart.changeOptions[index], val);
                    }      
                }
            }
        });
        if (options.hasOwnProperty('fill-opacity') || options.hasOwnProperty('opacity') || options.hasOwnProperty('fill'))
            if (!($element[0].tagName === "SPAN")) {
                chart.setFillAttribute($element, options);
               if (options.hasOwnProperty('stroke')) {
                    options.stroke = strokeColor;
                   if (options.fill_opacity) {
                       options['opacity'] = options.fill_opacity;
                   }
                    chart.setStrokeAttribute($element, options);
                }
            }
    },
    applyVMLStyle: function ($element, options) {
        
        if ($element[0].tagName !== "SPAN") {
            if (ej.util.isNullOrUndefined(options.width)) options.width = "1000px";
            if (ej.util.isNullOrUndefined(options.height)) options.height = "1000px";
        }
        if (ej.util.isNullOrUndefined(options.x)) options.x = "0px";
        if (ej.util.isNullOrUndefined(options.y)) options.y = "0px";

        this.changeVMLStyle($element, options);
    },
    drawLine: function (options, element) {
        if ($("#" + options.id).length > 0) {
            var from = (options.x1) + ',' + (options.y1);
            var to = (options.x2) + ',' + (options.y2);
            $(options).attr("from", from).attr("to", to);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            var line = document.createElement("v:line");
            var $lineEle = $(line);
            var from = (options.x1) + ',' + (options.y1);
            var to = (options.x2) + ',' + (options.y2);
            $lineEle.attr("from", from).attr("to", to);
            this.applyVMLStyle($lineEle, options);
            $lineEle.appendTo(element);
        }
     },
    //drawPolygon: function (options, element) {
     
    //},
     drawCircle: function (options, element) {
         if ($("#" + options.id).length > 0) {
             options.x = options.cx - options.r;
             options.y = options.cy - options.r;
             options.width = options.height = (2 * options.r);
             this.applyVMLStyle($("#" + options.id), options);
         }
         else {
             var circle = document.createElement("v:oval");
             var $circleEle = $(circle);
             options.x = options.cx - options.r;
             options.y = options.cy - options.r;
             options.width = options.height = (2 * options.r);
             this.applyVMLStyle($circleEle, options);
             if ($("#" + options.id).length == 0)
             $circleEle.appendTo(element);
         }
     },
    
   drawEllipse: function (options, element) {
       if ($("#" + options.id).length > 0) {
           options.x = options.cx - options.rx;
           options.y = options.cy - options.ry;
           options.width = (2 * options.rx);
           options.height = (2 * options.ry);
           this.applyVMLStyle($("#" + options.id), options);
       }
       else {
           var ellipse = document.createElement("v:oval");
           var $ellipseEle = $(ellipse);
           options.x = options.cx - options.rx;
           options.y = options.cy - options.ry;
           options.width = (2 * options.rx);
           options.height = (2 * options.ry);
           this.applyVMLStyle($ellipseEle, options);
           $ellipseEle.appendTo(element);
       }
    },
    drawRect: function (options, element) {
        if ($("#" + options.id).length > 0) {
            this.applyVMLStyle($("#" + options.id), options);
        } else {
            var rect = document.createElement("v:rect");
            var $rectEle = $(rect);
            this.applyVMLStyle($rectEle, options);
            $rectEle.appendTo(element);
        }
    },
    drawCylinder: function (options, element, seriesOption) {
        var x = options.x, y = options.y, w = options.width, h = options.height, id = options.id, angle, path, rx, ry, cx2, cx1, cy1, X1, Y1, LX1, LY1, LX2, LY2, cy2, sa, ea, sa2, ea2, X, Y, cx, cy, LX, LY;
        var gradientColor = options.fill;
        var format = ej.EjSvgRender.prototype.checkColorFormat(gradientColor);
        if (!format)
        var gradientColor = ej.datavisualization.Chart.prototype.colorNameToHex(options.fill);
        var opacity = options.opacity, obj = { svgRenderer: ej.EjSvgRender.prototype }, colorz = ej.Ej3DRender.prototype.polygon3D.prototype.applyZLight(gradientColor, obj), colorx = ej.Ej3DRender.prototype.polygon3D.prototype.applyXLight(gradientColor, obj);
        var angleConvertor = 65535; // VML measures angles in degrees/65535
        if (seriesOption.isColumn == true) {
            options.rx =rx= w / 2;
            options.ry =ry= rx / 4;
            cx2 = cx1 = x + rx;
            Y1 = cy1 = (y - ry);
            LY2 = Y = cy2 = y + h - ry;
            LX1 = cx2 + rx;
            X1 = LX2 = X = cx2 - rx;
            sa1 = -Math.round(angleConvertor * 180);
            ea1 = Math.round(angleConvertor *180);
            sa2 = Math.round(angleConvertor * 360);
            ea2 = -Math.round(angleConvertor * 180);
            gradientAngle="90";
            if (seriesOption.stacking == true) {
                if (!seriesOption.isLastSeries) {
                    cy1 = y + ry;
                }
            }
            LY1 = cy1;
        }
        else {
            options.ry = ry = h / 2;
            options.rx = rx = ry / 4;
            LX1 = X1 = cx1 = x + rx;
            X = cx2 = x + w + rx;
            sa1 = Math.round(angleConvertor * 90); ea1 = Math.round(angleConvertor * 180);
            sa2 = Math.round(angleConvertor * 270);
            ea2 = -Math.round(angleConvertor * 180);
            cy2 = cy1 = y + ry;
            Y = LY1 = cy1 + ry;
            Y1 = LY2 = cy2 - ry;
            gradientAngle="0";
            if (seriesOption.stacking == true) {
                if (!seriesOption.isLastSeries) {
                    cx2 = x+w-rx;
                }
            }
            LX2 = cx2;
        }
        delete options.x;
        delete options.y;
        options.width = 1000;
        options.x = 0;
        options.y = 0;
        delete options.stacking;
        options.height = 1000;
        delete options.isColumn;
        options.fill = colorz;
        options.fill_opacity = opacity;
        options.stroke = colorz;
        options['stroke-width'] = 0;
        for (i = 1; i <= 4; i++) {
           
            if (i % 2 == 0) {
                cx = cx1;
                cy = cy1; LX = LX2; LY = LY2; sa = sa2; ea = ea2;

            }
            else {
                cx = cx2; cy = cy2; LX = LX1; LY = LY1; sa = sa1; ea = ea1;
            }
            if (i < 3) {
                sa= angleConvertor;
                ea = 23592600;
            }
            if (i < 4) {
                path = [
  'M',
Math.round(X),
Math.round(Y)
                ];
            }
            path.push(

 'ae',
Math.round(cx),
Math.round(cy),
Math.round(rx),
Math.round(ry),
sa,
ea
);
            if (i > 2) {
                path.push(

   'l',
   Math.round(LX),
   Math.round(LY)
);
                options.fill = "none";
                options.color = gradientColor;
                options.color2 = gradientColor;
                options.colors = ("30% " + colorx + "," + "70% " + colorx);
                options.angle = gradientAngle;
            }
            if (i != 3) {
                path.push(
        'e'
    );
                path = path.join(" ");
                options.d = path;
                options.id = id + "_" + "Region_" + (i - 1).toString();
                if (i == 4)
                options.id = id + "_" + "Region_" + (i - 2).toString();
                var shape = document.createElement("v:shape");
                var $shape = $(shape);
                this.applyVMLStyle($shape, options);
                $shape.appendTo(element);
            }
            X = X1; Y = Y1;
        }    
    },
    _getAttrVal: function (ele, val, option) {
          val = (val == "x") ? "left" : val;
        val = (val == "y") ? "top" : val;
        var value1 = $(ele).attr(val);
        var value = value1 ? value1 : ($(ele).css(val));
        
        if (value != null)
            return value;
        else
            return option;
    },
    _setAttr: function (element, attribute) { 
        if (attribute.transform) {
		  var attrTrans=attribute.transform;
            var trans = attrTrans.slice(attrTrans.indexOf('(') + 1, attrTrans.indexOf(')')).split(',');
            attribute.x = trans[0];
            attribute.y = trans[1];
        }
        var $ele = $(element);
        if($ele.length>0)
            this.changeVMLStyle($ele, attribute);
    },
    createGradientElement: function (name, colors) {
        var colorName;
       
        if (Object.prototype.toString.call(colors) == '[object Array]') {
            colorName = "gradient" + " ";
            var appendString = "";
            for (var j = 0, i=colors.length-1 ; i>=0, j < colors.length ; j++,i--){
                colorName += appendString + colors[j]["colorStop"] + " " + colors[i]["color"];
                appendString = ",";
            }
        }
        else {
            colorName = colors;
        }
        return colorName; 
    },

    setSpanAttr: function (options, label, element) {
	    var fontSize = (options["font-size"] > 0) ? (options["font-size"] + "px") : options["font-size"];
        var font = { size: fontSize, fontStyle: options["font-style"], fontFamily: options["font-family"] };
        var bounds = ej.EjSvgRender.utils._measureText(label, null, font);
        
            $(element).css("white-space", "nowrap");
        
        if (options["text-anchor"] === "middle")
            options.x = Math.round(options.x - bounds.width / 2);
        else if (options["text-anchor"] === "end")
            options.x = Math.round(options.x - bounds.width);
        options.y = Math.round(options.y -( font.size.replace("px","")));
        this.applyVMLStyle(element, options);
        
       
    },
    drawText: function (options, label, groupEle,font) {
        var textDoc = document;
        var $Ele = $("#" + options.id);
        if ($Ele.length > 0) {
            this._textAttrReplace(options, label, font, $Ele);
        }
        else if (options.id.indexOf("YAxisTitle") > 0 && !this.isdocmode8) {
            options.on = true;
            var line = document.createElement("v:line");
            line.style.behavior = "url(#default#VML)";
            line.style.display = "inline-block";
            var bounds = ej.EjSvgRender.utils._measureText(label, null, null);
             
            line.setAttribute("to", (options.x - 1).toString() + "," + (options.y - bounds.width / 2).toString());
            line.setAttribute("from", (options.x).toString() + "," + options.y.toString());
           
            var path =  document.createElement("v:path");
            path.setAttribute("textpathok", true);
            var textpath =  document.createElement("v:textpath");
            textpath.setAttribute("on", true);
            textpath.setAttribute("string", label);
       
          textpath.style["font-style"] = options["font-style"];
          textpath.style["font-size"] = options["font-size"];
          textpath.style["font-family"] = options["font-family"];         
          line.appendChild(path);
          line.appendChild(textpath);           
          $(line).appendTo(groupEle);
        }
        else {
            var span = textDoc.createElement("span");
            var $spanEle = $(span);
            if (jQuery.type(label) == "array") {
                this.setSpanAttr(options, label, $spanEle);
                for (var i = 0; i < label.length; i++) {
                    var textspan = textDoc.createElement("span");
                    $(textspan).html(label[i]);                
                    $(textspan).appendTo(span);
                    var brtag = textDoc.createElement("br");
                    $(brtag).appendTo(span);
                    
                }
            } else {
                this.setSpanAttr(options, label, $spanEle);
                $spanEle.html(label);
            }         
            $spanEle.appendTo(groupEle);          
        }
    },
    //text element attributes values are replaced 
    _textAttrReplace: function (options, label, font, $Ele) {
        if (jQuery.type(label) == "array") {
            var elements = $("#" + options.id).children("span");
            this.setSpanAttr(options, label, $Ele);
            if (elements.length > 0 && elements.length == label.length) {
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    $(element).text(label[i]);
                }
            }
            else {
                $("#" + options.id).remove();
                this.drawText(options, label, this.gTransToolEle, font);

            }
        } else {
            this.setSpanAttr(options, label, $Ele);
            $Ele.text(label);
        }

    },

   drawImage: function (options, element) {
        var imgElement = document.createElement("img");
        var $imgElement = $(imgElement);
          $imgElement.css({ 'position': 'absolute' });
          $.each(options, function (index, val) {
              if (index == "href") index = "src";
                $imgElement.attr(index, val);
        });
        $imgElement.appendTo(element);
       
    },

    createDefs: function () {
        
    },

    //createClipPath: function (options) {
        
    //},
    //createForeignObject: function (options) {
      
    //},
    createGroup: function (options) {
        var group = document.createElement("div");
        if (!ej.util.isNullOrUndefined(options.transform)) {
            var trans = options.transform.slice(options.transform.indexOf('(') + 1, options.transform.indexOf(')')).split(',');
            $(group).css({ 'position': 'absolute', 'left': trans[0] + 'px', 'top': trans[1] + 'px' });
        }
		 if (options.cursor) 
            $(group).css({ 'cursor': options.cursor });
        
        $.each(options, function(index, val) {
            if (index !== 'transform' && index !="clip-path")
                $(group).attr(index, val);
        });
      
        return group;
    },

    createText: function (options, label) {
        var text = document.createElement("span");
        $(text).attr(options);
        if (!ej.util.isNullOrUndefined(label))
            text.textContent = label;
        return text;
        
        
    },   
   
    //Draw clip path for each series to avoid series overlap in multiple axes zooming
    drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
        var clipOptions;
        var element = $(gSeriesGroupEle);
        element.removeAttr("clip-path");
        var trans = this._getTransform(options.xAxis, options.yAxis, invertedAxis);
        clipOptions = {
            'id': gSeriesGroupEle.id + '_ClipRect',
            'x': 0,
            'y': 0,
            'width': (trans.width),
            'height': (trans.height),
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'transparent'
        };
        this.drawClipPath(clipOptions, gSeriesGroupEle);
        element.attr('clip-path', 'url(#' + clipOptions.id + ')');

    },
   
    getAttrVal: function (ele, val, option) {
        var value = $(ele).attr(val);
        if (value != null)
            return value;
        else
            return option;
    },
   
    round: function (value, div, up) {

        return div * (up ? Math.ceil(value / div) : Math.floor(value / div));
    },
    
   
   
    
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        if (color.A) {
            var returncolor = "rgba(" + r.toString() + "," + g.toString() + "," + b.toString() + "," + color.A + ")";
            return returncolor;
        }
        else {
            var r = color.R;
            var g = color.G;
            var b = color.B;
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return "#" + hex.join("").toUpperCase();
        }
    },
    hexToRGB: function (hex) {
        var rgbRegex = /^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/.test(hex);
        var result;
        if (rgbRegex == true) {
            result = /^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/.exec(hex);
            return result ? {
                R: parseInt(result[1]),
                G: parseInt(result[2]),
                B: parseInt(result[3]),
                A: result[4],
            } : null;
        }
        else {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return result ? {
                R: parseInt(result[1], 16),
                G: parseInt(result[2], 16),
                B: parseInt(result[3], 16)
            } : null;
        }
    },
    checkColorFormat: function (color) {
        return /(rgba?\((?:\d{1,3}[,\)]){3}(?:\d+\.\d+\))?)|(^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$)/gmi.test(color);
    },
    createDelegate: function (context, handler) {
        return function (e) {
            handler.apply(context, [e, this]);
        };
    },

    drawClipPath: function (options, element) {
        if (options.id.indexOf("ChartAreaClipRect") == -1) {
            var left = Math.round(options.x)+'px';
            var top = Math.round(options.y)+'px';
            var right = Math.round(options.width) + Math.round(options.x) + 'px';
            var bottom = Math.round(options.y)+ Math.round(options.height) + 'px';
            var cssClip = 'rect(' + top + ' ' + right + ' ' + bottom + ' ' +left + ')';
            $(element).css("position", "absolute").css('clip',cssClip);
        }
    },

    drawCircularClipPath: function (options, element) {
        var radius = Math.round(options.r);
        var x = Math.round(options.cx);
        var y = Math.round(options.cy);
        if (options.id.indexOf("_ClipRect") == -1) {
            var left = (x - radius) + 'px';
            var top = (y - radius) + 'px';
            var right = (x + radius) + 'px';
            var bottom = (y + radius) + 'px';
            var cssClip = 'rect(' + top + ' ' + right + ' ' + bottom + ' ' + left + ')';
            $(element).css("position", "absolute").css('clip', cssClip);
        }
    },
   
    append: function (childEle, parentEle) {
        $(childEle).appendTo(parentEle);
    }
};
})(jQuery);;
ej.EjCanvasRender = function (element) {
    this.svgObj = document.createElement('canvas');
    this._rootId = jQuery(element).attr("id");
    this.svgObj.setAttribute('id', this._rootId + '_canvas');
    this.ctx = this.svgObj.getContext("2d");
    
};
(function ($) {
    ej.EjCanvasRender.prototype = {

        drawPath: function (options, element, canvasTranslate) {
            var hasStackingInnerRadius = options.hasStackingInnerRadius ? options.hasStackingInnerRadius : false;
            var path = options.d;
            var dataSplit = path.split(" ");
            var borderWidth = options["stroke-width"];
            if (!options.lgndCtx) {
                var canvasCtx = this.ctx;
                this.ctx = element != undefined ? element : this.ctx;
                this.ctx.save();
                this.ctx.beginPath();
                if (canvasTranslate) this.ctx.translate(canvasTranslate[0], canvasTranslate[1]);
                this.ctx.globalAlpha = options.opacity ? options.opacity : options["fill-opacity"];
                var flag = true;
                this.ctx.lineWidth = borderWidth;
				var dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
                if (dashArray) this.ctx.setLineDash(dashArray);
                this.ctx.strokeStyle = options.stroke;
                for (var i = 0; i < dataSplit.length; i = i + 3) {
                    var x1 = parseFloat(dataSplit[i + 1]);
                    var y1 = parseFloat(dataSplit[i + 2]);
                    switch (dataSplit[i]) {
                        case "M":
                            if ((!options.innerR|| hasStackingInnerRadius) && !options.cx)
                                this.ctx.moveTo(x1, y1);
                            break;
                        case "L":
                            if (!options.innerR || hasStackingInnerRadius)
                                this.ctx.lineTo(x1, y1);
                            break;
                        case "C":
                            this.ctx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                            i = i + 4;
                            break;
                        case "A":
                            if (!options.innerR) {
							    if (options.cx) {
                                    this.ctx.arc(options.cx, options.cy, options.radius, 0, 2 * Math.PI, options.counterClockWise);
                               }
								else{
                                    this.ctx.moveTo(options.x, options.y);
                                    this.ctx.arc(options.x, options.y, options.radius, options.start, options.end, options.counterClockWise);
                                    this.ctx.lineTo(options.x, options.y);
								}
                            }
                            else if (flag) {
                                this.ctx.arc(options.x, options.y, options.radius, options.start, options.end, options.counterClockWise);
                                this.ctx.arc(options.x, options.y, options.innerR, options.end, options.start, !options.counterClockWise);
                                flag = false; 
                            }
                            i = i + 5;
                            break;
						case "Q":
							this.ctx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i+1]), parseFloat(dataSplit[i + 2]), parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]));
							i = i + 2;
							break;
					    case "z":
                            this.ctx.closePath();
                            break;
                    }
                }
                if (options.fill != "none" && options.fill != undefined) {
                    this.ctx.fillStyle = options.fill;
                    this.ctx.fill();
                }
                if (borderWidth > 0)
                    this.ctx.stroke();
                this.ctx.restore();
                this.ctx = canvasCtx;
            } else {
                var legendCtx = this.lgndCtx;
                this.lgndCtx = element != undefined ? element : this.lgndCtx;
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                if (canvasTranslate) this.lgndCtx.translate(canvasTranslate[0], canvasTranslate[1]);
                this.lgndCtx.globalAlpha = options.opacity ? options.opacity : options["fill-opacity"];

                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                for (var i = 0; i < dataSplit.length; i = i + 3) {
                    var x1 = parseFloat(dataSplit[i + 1]);
                    var y1 = parseFloat(dataSplit[i + 2]);
                    switch (dataSplit[i]) {
                        case "M":
                            this.lgndCtx.moveTo(x1, y1);
                            break;
                        case "L":
                            this.lgndCtx.lineTo(x1, y1);
                            break;
                        case "C":
                            this.lgndCtx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                            i = i + 4;
                            break;
                        case "A":
                            this.lgndCtx.arc(parseFloat(dataSplit[i - 5]), parseFloat(dataSplit[i - 4]), x1, 0, 2 * Math.PI, false);
                            i = i + 5;
                            break;
							case "a":
                            this.lgndCtx.beginPath();
                            var centerX = parseFloat(dataSplit[i - 2]) + x1;
                            var centerY = dataSplit[i - 1];
                            this.lgndCtx.arc(centerX, centerY, x1, 0, 2 * Math.PI, false);
                            i = i + 5;
                            break;
                        case "Z":
                            this.lgndCtx.fillStyle = options.fill;
                            this.lgndCtx.fill();
                            i = i - 2;
                            break;
					    case "Q":
                            this.lgndCtx.quadraticCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]));
                            i = i + 2;
                            break;
                    }
                }
                if (options.fill != "none" && options.fill != undefined) {
                    this.lgndCtx.fillStyle = options.fill;
                    this.lgndCtx.fill();
                }
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
                this.lgndCtx = legendCtx;
            }
        },
        
       
        drawRect: function (options, element) {
            var canvasCtx = this.ctx, cornerRadius = options.rx;
            this.ctx = element ? (element.canvas ? element : this.ctx ) : this.ctx;
            this.ctx.save();
            this.ctx.beginPath();
            this.ctx.globalAlpha = options.opacity;
            this.ctx.lineWidth = options["stroke-width"];
            dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
            if (dashArray) this.ctx.setLineDash(dashArray);
            this.ctx.strokeStyle = options.stroke;
            this.ctx.rect(options.x, options.y, options.width, options.height);
            if (cornerRadius != null && cornerRadius >= 0) {
                this.drawCornerRadius(options);
            }
            else {
                if (options.fill == "none") options.fill = "transparent";
                this.ctx.fillStyle = options.fill;
                this.ctx.fillRect(options.x, options.y, options.width, options.height);
                this.ctx.stroke();
             }
            if(options.id.indexOf("Series") >= 0)
            this.ctx.clip();
	        this.ctx.restore();
            this.ctx = canvasCtx;
        },
        drawCylinder: function (options, element, seriesOption) {
            var canvasCtx = this.ctx;
            this.ctx = element ? (element.canvas ? element : this.ctx) : this.ctx;
            this.ctx.save();
            var grad = ej.datavisualization.Chart.prototype.colorNameToHex(options.fill);
            var obj = { svgRenderer: ej.EjSvgRender.prototype };
            'use strict';
            var i, xPos, yPos, pi = Math.PI, twoPi = 2 * pi + 0.1, rx, ry, cx1, cx2, cy1, cy2, x1, x2, y1, y2, cx, cy, xl, yl, step, ini, gx1, gy2, gx2, gy1,rxt,ryt;
            var x = options.x, y = options.y, w = options.width, h = options.height, a=twoPi;
            this.ctx.fillStyle = ej.Ej3DRender.prototype.polygon3D.prototype.applyZLight(grad, obj)
            this.ctx.lineWidth = 0;
            this.ctx.strokeStyle = ej.Ej3DRender.prototype.polygon3D.prototype.applyZLight(grad, obj);
            this.ctx.globalAlpha = options.opacity;
            if (seriesOption.isColumn == true) {
                gx1 = x;
                gx2 = w + x;
                gy1=gy2 = 0;
                rx = w / 2;
                ry = rx / 4;
                cx2 = cx1 = x + rx;
                y2 = cy1 = y - ry;
                x2 = x;
                    x1 = x + w;
                cy2 = y1 = y + h - ry;
                ini = 0;
                step = pi;
                rxt = -rx;
                ryt = ry;
                if (seriesOption.stacking = true) {
                    if (!seriesOption.isLastSeries) {
                         y2 = cy1 = y + ry;
                        
                    }
                }

            }
            else {
                gx1=gx2 = 0;
                gy2 = h + y;
                gy1 = y;
                ry = h / 2;
                rx = ry / 4;
                x2 = cx1 = x + rx;
                x1= cx2 = x + w + rx;
                y1 = y+h;
                y2 = y;
                cy2 = cy1 = y + ry;
                ini = pi / 2;
                step = pi * 1.5; 
                if (seriesOption.stacking = true) {
                    if (!seriesOption.isLastSeries) {
                        x1 = cx2 = x + w - rx;
                    }
                }
                ry = -ry;
                rx = -rx;
                rxt = rx;
                ryt = -ry;
            }
            var color=ej.Ej3DRender.prototype.polygon3D.prototype.applyXLight(grad, obj);
            var grd = this.ctx.createLinearGradient(gx1, gy1, gx2, gy2);
            grd.addColorStop(0, grad);
            grd.addColorStop(0.3, color);
            grd.addColorStop(0.7,color);
            grd.addColorStop(1, grad);
            
            for (j = 1; j <= 4; j++) {
                var i = 0;
                    j < 4 ? this.ctx.beginPath() : "";
                    if (j % 2 == 0) {
                        cx = cx2; cy = cy2; xl = x2; yl = y2;
                    }
                    else {
                        cx = cx1; cy = cy1; xl = x1; yl = y1;
                    }
                    if (j == 4)
                    {
                        rx = rxt;
                        ry = ryt;
                        this.ctx.fillStyle = grd;
                    }
                    if (j > 2) {
                        i = ini;
                        a = step;
                    }
                    for (; i <= a; i += 0.1) {
                        xPos = cx - (rx * Math.cos(i));
                        yPos = cy + (ry * Math.sin(i));

                        if (i === 0) {
                            this.ctx.moveTo(xPos, yPos);
                        } else {
                            this.ctx.lineTo(xPos, yPos);
                        }
                    }
                    
                    if (j > 2) {
                        
                            this.ctx.lineTo(xl,yl);
                        }
                    if (j != 3) {
                       
                            this.ctx.stroke();
                            this.ctx.fill();
                        }
                }
          
            if (options.id.indexOf("Series") >= 0)
                this.ctx.clip();
            this.ctx.restore();
            this.ctx = canvasCtx;
        },
        drawCornerRadius: function (options) {
            var cornerRadius = options.rx, x = options.x, y = options.y, width = options.width, height = options.height;
            if (options.fill == "none") options.fill = "transparent";
            this.ctx.fillStyle = options.fill;
            if (width < 2 * cornerRadius) cornerRadius = width / 2;
            if (height < 2 * cornerRadius) cornerRadius = height / 2;
            this.ctx.beginPath();
            this.ctx.moveTo(x + width - cornerRadius, y);
            this.ctx.arcTo(x + width, y, x + width, y + height, cornerRadius);
            this.ctx.arcTo(x + width, y + height, x, y + height, cornerRadius);
            this.ctx.arcTo(x, y + height, x, y, cornerRadius);
            this.ctx.arcTo(x, y, x + width, y, cornerRadius);
            this.ctx.closePath();
            this.ctx.fill();
            this.ctx.stroke();
        },

        createGradientElement: function (name, colors) {
            var colorName;
            var ctx = this.ctx;
            if (Object.prototype.toString.call(colors) == '[object Array]') {
                var my_gradient = ctx.createLinearGradient(0, 0, 0, this.svgObj.height);
             
                for (var i = 0; i <= colors.length - 1 ; i++) {
                    var color = colors[i].color;
                    var colorStop = (colors[i].colorStop).slice(0, -1);
                    colorStop = parseInt(colorStop) / 100;
                    my_gradient.addColorStop(colorStop, color);
                    
                }
                colorName = my_gradient;
            }
            else {
                colorName = colors;
            }
            return colorName;
        },
        drawLine: function (options) {
            this.ctx.save();
            this.ctx.beginPath();
			this.ctx.lineWidth = options["stroke-width"];
            this.ctx.strokeStyle = options.stroke;
            this.ctx.moveTo(options.x1, options.y1);
            this.ctx.lineTo(options.x2, options.y2);
            this.ctx.stroke();
            this.ctx.restore();
        },
        drawText: function (options, label) {
            if (!ej.util.isNullOrUndefined(options["font-weight"]) && options["font-weight"].toLowerCase() == "regular")
                options["font-weight"] = "normal";
            var font = (options["font-style"].toLowerCase() + " " + options["font-weight"] + " " + options["font-size"] + " " + options["font-family"]);
            // text-anchor
            var anchor = options["text-anchor"];
            var opacity = options["opacity"] !== undefined ? options["opacity"] : 1;
            if (anchor == "middle")
                anchor = "center";
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.fillStyle = options.fill;                
                this.ctx.font = font;                
                this.ctx.textAlign = anchor;
                this.ctx.globalAlpha = opacity;
                if (options.baseline)
                    this.ctx.textBaseline = options.baseline;
                // dominant-baseline
                //if (options["dominant-baseline"] == "middle")
                    //this.ctx.textBaseline = "center";
                if (!options.labelRotation) {
                    if (typeof label == "object" && options.isTrackball) {
                        var len = label.length;
                        for (var i = 0; i < len; i++) {
                            this.ctx.fillText(label[i], options.x, options.y + (Number(parseInt(options["font-size"])) + options.padding) * i);
                        }
                    }
                    else {
                        this.ctx.fillText(label, options.x, options.y);
                    }
                } else {
                    var txtlngth = 0, rotation = options.labelRotation;
                    if ((rotation) && options.id.indexOf("XLabel") != -1) {
                        var rotate = 'rotate(' + options.labelRotation.toString() + 'deg)';
                        $(options).attr('rotateAngle', rotate);
                        var textElement = this.createText(options, label);
                        $(document.body).append(textElement);
                        var box = textElement.getBoundingClientRect();
                        $(textElement).remove();
                        txtlngth = box.height;
                        if (rotation < 0) {
                            rotation = 360 + rotation;
                        }
                        var str = options.transform.split(',');
                        if (options.labelPosition == "outside")
                            this.ctx.translate(parseFloat(str[1]), parseFloat(str[2]) + (txtlngth / 2));
                        else
                            this.ctx.translate(parseFloat(str[1]), parseFloat(str[2]) - (txtlngth / 2));
                        this.ctx.textAlign = "center";
                        this.ctx.rotate(rotation * (Math.PI / 180));
                        this.ctx.fillText(label, 0, 0);
                    }
                    else {
                        this.ctx.translate(options.x, options.y);
                        this.ctx.rotate(options.labelRotation * Math.PI / 180);
                        this.ctx.fillText(label, 0, 0);
                    }
                }
                this.ctx.restore();
            } else {
                this.lgndCtx.save();
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.font = font;
                this.lgndCtx.textAlign = anchor;
                this.lgndCtx.globalAlpha = opacity;
                // dominant-baseline
                if (options["dominant-baseline"] == "middle")
                    this.lgndCtx.textBaseline = "center";
                if (!options.labelRotation) {
                    this.lgndCtx.fillText(label, options.x, options.y);
                } else {
                    this.lgndCtx.translate(options.x, options.y);
                    this.lgndCtx.rotate(options.labelRotation * Math.PI / 180);
                    this.lgndCtx.fillText(label, 0, 0);
                }
                this.lgndCtx.restore();
            }
        },
        drawZoomRect: function(options, chartObj) {

            var chartPos = { left: 0, top: 0 };
            var zoomRect = $("<div id=" + options.id + "></div>");
            var areaBounds = chartObj.model.m_AreaBounds;
			
			// for drawing zoom area within the chartArea
            if (options.x < areaBounds.X || options.x + chartPos.left + options.width > chartPos.left + $(this.svgObj).width() - 20) {
                width = this.prevWidth;
                x = this.prevX;
            } else {
                width = this.prevWidth = options.width;
                x = this.prevX = options.x + chartPos.left;
            }
            
            if (options.y < areaBounds.Y || options.y + chartPos.top + options.height > chartPos.top + areaBounds.Y + areaBounds.Height) {
                height = this.prevHeight;
                y = this.prevY;
            } else {
                height = this.prevHeight = options.height;
                y = this.prevY = options.y + chartPos.top;
            }
            
            $(zoomRect).css({
                    "width": width,
                    "height": height,
                    "top": y,
                    "left": x,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-color": options.stroke,
                    "border-width": options["stroke-width"]
                });
                document.getElementById("chartContainer_" + this._rootId).appendChild(zoomRect[0]);
        },
        zoomButton: function (options, currentItem, chart) {
			var svgObj = chart.svgObj ? chart.svgObj : chart.svgObject;
			var id = chart._rootId ? chart._rootId : chart._id;
            if (currentItem == "zoom")
                var currentItemId = "_Zoom";
            else if (currentItem == "zoomIn")
                currentItemId = "_ZoomIn";
            else
                currentItemId = "_ZoomOut";

            var chartPos = { left: 0, top: 0 };
            if ($("#" + options.zoomId).length <= 0) {
                var zoomButton = $("<div id=" + svgObj.id + currentItemId +'Btn' + "></div>");
                $(zoomButton).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-radius": "0.4em",
                    "border-color": options.fill,
                    "box-sizing": "content-box",
					 "z-index" : 2000
                });                
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.zoomId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);
               
                if (currentItem == "zoom") {
                    var direction = "M26.101,22.893l-6.605-6.174c1.414-2.781,0.897-6.267-1.496-8.504c-2.901-2.711-7.448-2.56-10.161,0.341    c-2.712,2.9-2.56,7.45,0.341,10.163c2.426,2.266,6,2.523,8.694,0.853l6.579,6.151L26.101,22.893z M10.186,16.571    c-1.715-1.604-1.805-4.293-0.203-6.009c1.605-1.715,4.295-1.805,6.009-0.201c1.715,1.603,1.805,4.293,0.202,6.007    C14.59,18.084,11.901,18.175,10.186,16.571";
                    var transform = "translate(-3,-3)";
                } else if (currentItem == "zoomIn") {
                    direction = "M9.0983096,4.2999878L9.0983096,9.3999634 3.9983783,9.3999634 3.9983783,12.699951 9.0983096,12.699951 9.0983096,17.799988 12.398249,17.799988 12.398249,12.699951 17.49818,12.699951 17.49818,9.5 12.398249,9.5 12.398249,4.2999878z M10.998276,0C14.298215,0 17.49818,1.3999634 19.69813,4.1999512 22.79809,8.0999756 22.79809,13.5 19.998144,17.399963L28.597992,28.299988 23.898081,32 15.398205,21.199951C10.898271,23.099976 5.5983606,21.899963 2.3983956,17.899963 -1.4015366,13.099976 -0.60156059,6.0999756 4.1983567,2.3999634 6.1983276,0.79998779 8.5983163,0 10.998276,0z";
                    transform = "translate(4,4) scale(0.6)";
                } else {
                    direction = "M3.9983433,9.5L3.9983433,12.799988 17.598165,12.799988 17.598165,9.5z M10.998234,0C14.298169,0 17.498131,1.3999634 19.698108,4.1999512 22.798034,8.0999756 22.798034,13.5 19.998092,17.399963L28.597994,28.299988 23.898054,32 15.398188,21.199951C10.898259,23.099976 5.5983546,21.899963 2.398393,17.899963 -1.4015351,13.099976 -0.60155994,6.0999756 4.1983522,2.3999634 6.1983207,0.79998779 8.5983074,0 10.998234,0z";
                    transform = "translate(4,4) scale(0.6)";
                }

                var attrOptions = {
                    'id': currentItemId + 'Path',
                    'fill': options.iconColor,
                    'transform': transform,
                    'd': direction
                };

                var path = document.createElementNS(svgLink, "path");
                $(path).attr(attrOptions).appendTo(svgShape);
                zoomButton[0].appendChild(svgShape);
                document.getElementById("chartContainer_" + id).appendChild(zoomButton[0]);
            } else {
                var zoomButton = document.getElementById(svgObj.id + currentItemId + 'Btn');
                $(zoomButton).css({
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left
                });
            }
        },
        panButton: function (options, chart) {
			var svgObj = chart.svgObj ? chart.svgObj : chart.svgObject;
			var id = chart._rootId ? chart._rootId : chart._id;
            var chartPos = { left: 0, top: 0 };
            if ($("#" + options.panId).length <= 0) {
                var panButton = $("<div id=" + svgObj.id + '_PanBtn' + "></div>");
                $(panButton).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-radius": "0.4em",
                    "border-color": options.fill,
                    "box-sizing": "content-box",
					 "z-index" : 2000
                });                
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.panId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);

                var attrOptions = {
                    'id': 'panPath',
                    'fill': options.iconColor,
                    'transform': 'translate(-3,-3)'
                };
                attrOptions.points = "26.105,16 21.053,12.211 21.053,14.737 17.263,14.737 17.263,10.947 19.834,10.947 16.044,5.895 12.255,10.947 14.737,10.947 14.737,14.737 10.947,14.737 10.947,12.211 5.895,16 10.947,19.789 10.947,17.263 14.737,17.263 14.737,21.053 12.255,21.053 16.044,26.105 19.834,21.053 17.263,21.053 17.263,17.263 21.053,17.263 21.053,19.789";

                var path = document.createElementNS(svgLink, "polygon");
                $(path).attr(attrOptions).appendTo(svgShape);
                panButton[0].appendChild(svgShape);
                document.getElementById("chartContainer_" + id).appendChild(panButton[0]);     
            } else {
                var panButton = document.getElementById(svgObj.id + '_PanBtn');
                $(panButton).css({
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left
                });
            }
        },
        resetZoom: function (options, chart) {
			var svgObj = chart.svgObj ? chart.svgObj : chart.svgObject;
			var id = chart._rootId ? chart._rootId : chart._id;
           var chartPos = { left: 0, top: 0 };
            if ($("#" + options.resetZoomId).length <= 0) {
                var resetZoom = $("<div id=" + svgObj.id + '_ResetZoom' + "></div>");
                $(resetZoom).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "border-radius": "0.4em",
                    "position": "absolute",
                    "border-color": options.fill,
                    "box-sizing":"content-box",
					 "z-index" : 2000
                });                
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.resetZoomId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);

                var points1 = {
                    'id': 'p1',
                    'fill': options.iconColor,
                    'transform': 'translate(-3,-3)'
                };
                points1.points = "11.895,18.398 8.061,22.23 5.796,19.967 5.796,26.283 12.112,26.283 9.848,24.018 13.682,20.186";

                var path1 = document.createElementNS(svgLink, "polygon");
                $(path1).attr(points1).appendTo(svgShape);

                var points2 = {
                    'id': 'p2',
                    'fill': options.iconColor,
                    'transform': 'translate(-3,-3)'
                };
                points2.points = "19.691,6.072 21.955,8.337 18.121,12.172 19.908,13.959 23.742,10.123 26.007,12.389 26.007,6.072";

                var path2 = document.createElementNS(svgLink, "polygon");
                $(path2).attr(points2).appendTo(svgShape);

                var points3 = {
                    'id': 'p3',
                    'fill': options.iconColor,
                    'transform': 'translate(-3,-3)'
                };
                points3.points = "11.895,13.958 13.682,12.172 9.848,8.337 12.112,6.072 5.796,6.072 5.796,12.389 8.061,10.123";

                var path3 = document.createElementNS(svgLink, "polygon");
                $(path3).attr(points3).appendTo(svgShape);

                var points4 = {
                    'id': 'p4',
                    'fill': options.iconColor,
                    'transform': 'translate(-3,-3)'
                };
                points4.points = "19.908,18.396 18.121,20.186 21.955,24.018 19.691,26.283 26.007,26.283 26.007,19.967 23.741,22.23";

                var path4 = document.createElementNS(svgLink, "polygon");
                $(path4).attr(points4).appendTo(svgShape);

                resetZoom[0].appendChild(svgShape);
                document.getElementById("chartContainer_" + id).appendChild(resetZoom[0]);
            } else {
                var resetZoom = document.getElementById(svgObj.id + '_ResetZoom');
                $(resetZoom).css({
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left
                });
            }
        },
        drawCrosshairLine: function (options, element) {
            var line = $("<div id=" + options.id + "></div>");
            $(line).css({
                "width": options.width,
                "height": options.height,
                "left": options.left,
                "top": options.top,
                "border-style": options.style,
                "opacity": options.opacity,
                "visibility": "visible",
                "border-color": options.stroke,
                "border-width": options["stroke-width"],
                "position": options.position                 
            });
            $(element).append(line[0]);
        },
        drawCrosshairlabel: function (rectOptions, textOptions, text) {
		     var chartOffset = $("#"+ this._rootId).offset(),
                chartPos = { left: 0, top: 0 };
            if ($("#" + rectOptions.id).length > 0) {
                var label = document.getElementById(rectOptions.id);                
                $(label).css({
                    "width": rectOptions.width - 5,
                    "height": rectOptions.height - 5,
                    "left": textOptions.x + chartPos.left - 5,
                    "top": textOptions.y + chartPos.top - 15,
                    "visibility": "visible",
                    "display": rectOptions.display
                });
                document.getElementById(rectOptions.id).textContent = text;
            } else {
                var label = $("<div id=" + rectOptions.id + " style='position: absolute; z-index: 13000;'></div>");
                $(label).css({
                    "color": textOptions.fill,
                    "font-family": textOptions["font-family"],
                    "font-size": textOptions["font-size"],
                    "font-weight": textOptions["font-weight"],
                    "font-style": textOptions["font-style"],
                    "background-color": rectOptions.fill,
                    "border-style": "solid",
                    "border-color": rectOptions.stroke,
                    "border-width": rectOptions["stroke-width"],
                    "opacity": rectOptions["fill-opacity"],
                    "text-align" : "center"
                });
                
                document.getElementById(this._rootId).appendChild(label[0]);
            }
        },
        createCrosshairCanvas: function () {
          
            var chartCanvas = document.getElementById(this._rootId + "_canvas"),
             chartOffset = $("#" + this._rootId).offset(), secCanvas,
             chartPos = { left: 0, top: 0 };
           
            if (!$("#secondCanvas").length) {
                 secCanvas = document.createElement('canvas');
                 $(document).find("#" + this.svgObj.id + '_CrosshairGroup').append(secCanvas);
                $(secCanvas).attr({
                    'id': 'secondCanvas',
                    'width': chartCanvas.width,
                    'height': chartCanvas.height,
                    'top': chartPos.top,
                    'left': chartPos.left,
                });
                $(secCanvas).css({
                    'width': chartCanvas.width,
                    'height': chartCanvas.height,
                    'top': chartPos.top,
                    'left': chartPos.left                   
                });
            }
            else {
                 secCanvas = document.getElementById("secondCanvas");
            }
            return secCanvas;
        },      
        drawTrackToolTip: function (rectOptions, textOptions, text, padding) {
            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            if ($("#" + rectOptions.id).length > 0) {
                var label = document.getElementById(rectOptions.id);               
                $(label).css({
                    "left": rectOptions.x + chartPos.left,
                    "top": rectOptions.y + chartPos.top,
                    "visibility": "visible",
                    "border-box": "content-box",
					"background-color": rectOptions.fill,
					"color": textOptions.fill, 
					"border-color": rectOptions.stroke,
					"border-width": rectOptions["stroke-width"]
                });
                 if (jQuery.type(text) == "array") {
                    var j = 0;
                    var arrayText = "";
                    for (var i = 0; i < text.length; i++) {
                        arrayText = arrayText + text[i] + '</br>';
                    }
                    $("#" + rectOptions.id).html(arrayText);
                } else
                    $("#" + rectOptions.id).html(text);
            } else {
                var label = $("<div id=" + rectOptions.id + " style='position: absolute; z-index: 13000;'></div>");
                $(label).css({
                    "color": textOptions.fill,
                    "font-family": textOptions["font-family"],
                    "font-size": textOptions["font-size"],
                    "font-weight": textOptions["font-weight"],
                    "font-style": textOptions["font-style"],
                    "width": rectOptions.width,
                    "height": rectOptions.height - padding - 2 * rectOptions["stroke-width"],
                    "background-color": rectOptions.fill,
                    "border-style": "solid",
                    "text-align": "center",
                    "padding" : "0px",
                    "padding-top": (padding/2) + 'px',
                    "padding-bottom": (padding/2) + 'px',
                    "border-color": rectOptions.stroke,
                    "border-width": rectOptions["stroke-width"],
                    "opacity": rectOptions["fill-opacity"]
                });
                document.getElementById(this._rootId).appendChild(label[0]);
            }
        },
        trackSymbol: function (options, evt, symbolName, trackSymbol, chartObj) {
            var chartAreaRect = chartObj.model.m_AreaBounds, chartOffset = $("#" + this._rootId).offset(), parent;
        if (evt.startX >= chartAreaRect.X && evt.startX <= chartAreaRect.X + chartAreaRect.Width && evt.startY >= chartAreaRect.Y && evt.startY <= chartAreaRect.Y + chartAreaRect.Height) {
            var seriesIndex = options.SeriesIndex, radius;
            var pointIndex = options.PointIndex;
            if (chartObj.model.series[seriesIndex].type.toLowerCase() == "bubble") {
                var point = chartObj.model.series[seriesIndex]._visiblePoints[pointIndex];
                radius = (point.radius * 2) + 5;//exploding the bubble
                options.Style.BorderColor = options.Style.Color;
            }
            
            if (chartObj.model.series[seriesIndex].type.toLowerCase() == "scatter" && ej.util.isNullOrUndefined(trackSymbol)) {
                var circleOptions = {
                    "cx": evt.startX -2,
                    "cy": evt.startY -2,
                    "r": options.ShapeSize.width/2,
                    "fill" : options.Style.Color,
                    "opacity": options.Style.Opacity,
                    "stroke" : options.Style.BorderColor
                };
                this.drawCircle(circleOptions, null);
            }
            
        else {
                if ($("#" + "canvas_trackSymbol").length > 0) {
                    var parentDiv = $("#" + "canvas_trackSymbol");
                    $(parentDiv).css({ "visibility": "visible", "display": "block" });
                }
                else
                    parentDiv = $("<div id=" + "canvas_trackSymbol" + "></div>");

                var chartPos = { left: evt.startX, top: evt.startY };
               
                var symbol = $("<div id=" + options.ID + "></div>");
                var left = (chartPos.left - (radius || options.ShapeSize.width));
                var top = (chartPos.top - (radius || options.ShapeSize.height));
                
                    $(symbol).css({
                       
                        "left":  left ,
                        "top": top,
                        "box-sizing": 'content-box',
                        "position": "absolute",
                        "visibility": "visible",                        
                        "opacity": options.Style.Opacity,
                        "z-index" : 200
                    });
                    var style = options.ShapeSize;
                    options.symbol = symbol;
                    var shapeAttr = {};
                    switch (symbolName.toLowerCase()) {
                        case "circle":
                            $(symbol).css({
                                //radius for bubble exploding
                                "width": radius || options.ShapeSize.width + 'px', 
                                "height": radius || options.ShapeSize.height + 'px',
                                "left": radius ? (left + radius / 2) : (left + options.ShapeSize.width / 2 - options.Style.BorderWidth) + 'px',
                                "top": radius ? (top + radius / 2) : (top + options.ShapeSize.height / 2 - options.Style.BorderWidth) + 'px',
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "border-radius": "100%"
                            });
                            break;
                        case "rectangle":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height,
                                "left": left + options.ShapeSize.width / 2 - options.Style.BorderWidth / 2,
                                "top": top + options.Style.BorderWidth / 2,
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth
                            });
                            break;
                        case "diamond":

                            var direction  = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " 
                                             + "L" + " " + (evt.startX) + " " + (evt.startY + (-options.ShapeSize.height / 2)) + " "
                                             + "L" + " " + (evt.startX + (options.ShapeSize.width / 2)) + " " + (evt.startY) + " "
                                             + "L" + " " + (evt.startX) + " " + (evt.startY + ( options.ShapeSize.height/2)) + " " 
                                             + "L" + " " + (evt.startX + (-options.ShapeSize.width / 2)) + " " + (evt.startY) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);
                          
                            break;
                        case "triangle":
                                                     
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY + (-style.height / 2))
                                + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);
                           
                            break;
                        case "invertedtriangle":
                                                     
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + "z";
                            var x = evt.startX;
                            var y = evt.startY;
                            var rotate = 'rotate(180,' + x + ',' + y + ')';

                            shapeAttr.direction = direction;
                            shapeAttr.rotate = rotate;
                            this.drawSvgShape(options, shapeAttr);
                            
                            break;
                       
                        case "cross":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (style.width / 2))
                                + " " + (evt.startY) + " " + "M" + " " + (evt.startX) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " +
                                (evt.startY + (-style.height / 2)) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "star":
                           
                            var direction = "M" + " " + (evt.startX + (style.width / 3)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " +
                                (evt.startY + (style.height / 6)) + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 6)) + " " + "L" + " " + (evt.startX + (-style.width / 3)) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 3)) + " " + (evt.startY + (-style.height / 2)) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "hexagon":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (-style.width / 4)) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 4)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (style.width / 4)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 4)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "wedge":
                           
                            var direction = "M" + " " + (evt.startX -style.width) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + style.width) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (3 * (style.width / 4))) + " " + (evt.startY) + " " + "L" + " " +
                                (evt.startX + (style.width)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX - style.width) + " " + (evt.startY) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "trapezoid":
                                                       
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " +
                                (evt.startY + (-style.height / 4)) + " " + "L" + " " + (evt.startX + (-style.width / 2) + (style.width)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 2) + (style.width)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 4)) +
                                " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "uparrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX)) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "downarrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX)) + " " + (evt.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "leftarrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " +
                                ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + ((evt.startX -
                                (style.width / 2))) + " " + (evt.startY) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " +
                                (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4)) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "rightarrow":

                            var direction = "M" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY + (style.height / 4)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "pentagon":
                           
                            var eq = 72;
                            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
                            var sb = ej.EjSvgRender.utils._getStringBuilder();
                            for (var i = 0; i <= 5; i++) {
                                var deg = i * eq;
                                var rad = (Math.PI / 180) * deg;
                                var x1 = radius * Math.cos(rad);
                                var y1 = radius * Math.sin(rad);
                                if (i == 0)
                                    sb.append("M" + " " + (evt.startX + x1) + " " + (evt.startY + y1) + " ");
                                else
                                    sb.append("L" + " " + (evt.startX + x1) + " " + (evt.startY + y1) + " ");
                            }
                            sb.append("z");
                            var direction = sb.toString();
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "ellipse":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height / 2,
                                "border-style": "solid",
                                "left": left + options.ShapeSize.height / 2 - options.Style.BorderWidth / 2,
                                "top": (evt.startY + chartPos.top - options.ShapeSize.height + 4),
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "-moz-border-radius": "50%",
                                "-webkit-border-radius": "50%",
                                "border-radius": "50%",
                                "border-box": "content-box"
                            });
                            break;
                        case "horizline":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": 0,
                                "border-style": "solid",
                                "left": left + options.ShapeSize.width / 2,
                                "top": (evt.startY + chartPos.top - 1),
                                "border-color": options.Style.BorderColor,
                                "border-width": '2px',
                                "border-box": "content-box"
                            });
                            break;
                        case "vertline":
                            $(symbol).css({
                                "width": 0,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "left": (evt.startX + chartPos.left - 1),
                                "top": top + options.ShapeSize.height / 2 - 2,
                                "border-color": options.Style.BorderColor,
                                "border-width": '2px',
                                "border-box": "content-box"
                            });
                            break;
                        default:
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "border-radius": "50px"
                            });
                            break;
                    }
                    parentDiv[0].appendChild(symbol[0]);
                    parent = document.getElementById(this.svgObj.id + '_CrosshairGroup') || document.getElementById("chartContainer_" + this._rootId);
                    parent.appendChild(parentDiv[0]);
            }
		  }
        },
        //draw SVG Marker shape on mouse over the points
        drawSvgShape: function (options, attr) {
            var chartCanvas = document.getElementById(this._rootId + "_canvas");
            var chartPos = { left: 0, top: 0 };
            var svgLink = "http://www.w3.org/2000/svg";
            var svgShape = document.createElementNS(svgLink, "svg");
            svgShape.setAttribute('id', 'mySVG');
            svgShape.setAttribute('width', $(chartCanvas).width());
            svgShape.setAttribute('height', $(chartCanvas).height());
            $(options.symbol).css({
                "left":  chartPos.left + 'px',
                "top": chartPos.top + 'px'
            });
            var attrOptions = {
                'id': 'svgPath',
                'fill-opacity': options.Style.Opacity,
                'stroke-width': options.Style.BorderWidth,
                'fill': options.Style.Color,
                'transform' : attr.rotate,
                'stroke': options.Style.BorderColor
            };
            attrOptions.d = attr.direction;

            var path = document.createElementNS(svgLink, "path");
            $(path).attr(attrOptions).appendTo(svgShape);
            options.symbol[0].appendChild(svgShape);
        },
        drawCircle: function (options, element) {
            if (!options.lgndCtx) {
                var canvasCtx = this.ctx, dashArray;
                this.ctx = element != undefined ? element : this.ctx;
                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.arc(options.cx, options.cy, options.r, 0, 2 * Math.PI);
                this.ctx.fillStyle = options.fill;
                this.ctx.globalAlpha = options.opacity;
                this.ctx.fill();
                this.ctx.lineWidth = options["stroke-width"];
                dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
                if (dashArray) this.ctx.setLineDash(dashArray);
                this.ctx.strokeStyle = options.stroke;
                this.ctx.stroke();
                this.ctx.restore();
                this.ctx = canvasCtx;
            } else {
                var legendCtx = this.lgndCtx;
                this.lgndCtx = element != undefined ? element : this.lgndCtx;
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                this.lgndCtx.arc(options.cx, options.cy, options.r, 0, 2 * Math.PI);
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.globalAlpha = options.opacity;
                this.lgndCtx.fill();
                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
                this.lgndCtx = legendCtx;
            }
        },    
        
        drawPolyline: function (options, element) {
            
            this.ctx.save();
            this.ctx.beginPath();
            var points = options.points.split(" ");
            for (var i = 0; i < points.length - 1; i++) {
                var point = points[i].split(",");
                var x = point[0];
                var y = point[1];
                if (i == 0) 
                    this.ctx.moveTo(x,y);
                 else
                    this.ctx.lineTo(x,y);
            }
            this.ctx.lineWidth = options["stroke-width"];
			 this.ctx.strokeStyle = options.stroke;
            this.ctx.stroke();
            this.ctx.restore();            
        },
        drawPolygon: function (options, element) {
           
        },
        setFillAttribute: function (element, options) {
           
        },
        setStrokeAttribute: function (element, options) {
            
        },
        changePathValue: function (options) {
            
        },


        drawArc: function (w, h, options) {
            var x = 0, y = 0;
  
        },
        changeVMLStyle: function($element, options) {
           
        },
        applyVMLStyle: function ($element, options) {

        },
       
        drawEllipse: function (options, element) {
            var canvasCtx = this.ctx;
            this.ctx = element != undefined ? element : this.ctx;
            var circumference = Math.max(options.rx, options.ry);
            var scaleX = options.rx / circumference;
            var scaleY = options.ry / circumference;
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.translate(options.cx, options.cy);
                this.ctx.save();
                this.ctx.scale(scaleX, scaleY);
                this.ctx.arc(0, 0, circumference, 0, 2 * Math.PI, false);
                this.ctx.fillStyle = options.fill;
                this.ctx.fill();
                this.ctx.restore();
                this.ctx.lineWidth = options["stroke-width"];
                this.ctx.strokeStyle = options.stroke;
                this.ctx.stroke();
                this.ctx.restore();
                this.ctx = canvasCtx;
            } else {
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                this.lgndCtx.translate(options.rx, options.ry);
                this.lgndCtx.scale(scaleX, scaleY);
                this.lgndCtx.arc(options.cx - options.rx, options.cy * 2 - options.rx, options.rx, 0, 2 * Math.PI, false);
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.fill();
                this.lgndCtx.restore();
                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
            }
        },
      
        _getAttrVal: function (ele, val, option) {
            var value = $(ele).attr(val);
            if (value != null)
                return value;
            else
                return option;
        },
        _setAttr: function (element, attribute) {
          
        },
       
        setSpanAttr: function (options, label, element) {
           
        },
       
        //text element attributes values are replaced 
        _textAttrReplace: function (options, label, font, $Ele) {
           
        },

        drawImage: function (options, element) {
             this.ctx.save();
             var renderer = this;
             var imageObj = new Image();
             imageObj.src = options.href;
            imageObj.onload = function () {
                  renderer.ctx.drawImage(imageObj, options.x, options.y, options.width, options.height);
            }
             this.ctx.restore();
        },

        createDefs: function () {

        },

        createGroup: function (options) {
           
        },

        createText: function (options, label) {
            var text = document.createElement("div");
            var diff= 1.1; // difference between svg and div element rotated text height
            $(text).css({
                "transform": options.rotateAngle,
                "font-family": parseFloat(options["font-family"]),
                "font-size": parseFloat(options["font-size"]) * diff,
                "position": "absolute"}); 
            if (!ej.util.isNullOrUndefined(label))
                text.textContent = label;
            return text;
        },


        //Draw clip path for each series to avoid series overlap in multiple axes zooming
        drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
            var clipOptions = options;
           
        },

        getAttrVal: function (ele, val, option) {
       
        },

        hexFromRGB: function (color) {
          
        },

        createDelegate: function (context, handler) {
           
        },

        drawClipPath: function (options, element) {
            
        },

        drawCircularClipPath: function (options, element) {

        },

        append: function (childEle, parentEle) {
            $(childEle).appendTo(parentEle);
			},
			
		createLegendCanvas: function (element) {            
            this.legendsvgObj = document.createElement('canvas');
            this._rootId = jQuery(element).attr("id");
            this.legendsvgObj.setAttribute('id', "legend_" + this._rootId + '_canvas');
            this.lgndCtx = this.legendsvgObj.getContext("2d");
            return this.legendsvgObj;
        }
		
    };
})(jQuery);


;
ej.EjSvgRender = function (element) {

    this.svgLink = "http://www.w3.org/2000/svg";
    this.svgObj = document.createElementNS(this.svgLink, "svg");
    this._rootId = jQuery(element).attr("id");
    var id = this._rootId + '_svg';   
    if ($(document).find("#" + id).length > 0){       
        var count=0
        do {
            count++;
        } while ($(document).find("#" + this._rootId + '_svg' + count).length > 0);
        id = this._rootId + '_svg' + count;
    }    
    this.svgObj.setAttribute('id', id);

};
(function ($) {
ej.EjSvgRender.prototype = {
    drawPath: function (options, element) {
        if ($("#" + options.id).length > 0) { 
            $("#" + options.id).attr(options);
        }
        else {
            var path = document.createElementNS(this.svgLink, "path");
            $(path).attr(options).appendTo(element);
        }

    },
    createLegendSvg:function(element)
    {
        this.svgLink = "http://www.w3.org/2000/svg";
        this.legendsvgObj = document.createElementNS(this.svgLink, "svg");
        this._rootId = jQuery(element).attr("id");
        this.legendsvgObj.setAttribute('id', "legend_" + this._rootId + '_svg');
        return this.legendsvgObj;
    },
    drawPolyline: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var polyline = document.createElementNS(this.svgLink, "polyline");
            $(polyline).attr(options).appendTo(element);
        }

    },

    drawLine: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var path = document.createElementNS(this.svgLink, "line");
            $(path).attr(options);
            $(path).appendTo(element);
        }
    },
    drawPolygon: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var polygon = document.createElementNS(this.svgLink, "polygon");
            $(polygon).attr(options);
            $(polygon).appendTo(element);
        }
    },
    drawCircle: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var circle = document.createElementNS(this.svgLink, "circle");
            $(circle).attr(options).appendTo(element);
        }
    },
    drawEllipse: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var ellipse = document.createElementNS(this.svgLink, "ellipse");
            $(ellipse).attr(options).appendTo(element);
        }
    },
    
    drawRect: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var rect = document.createElementNS(this.svgLink, "rect");
                $(rect).attr(options).appendTo(element);
            }    
    },
    drawCylinder: function (options, element, seriesOption) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var x = options.x, y = options.y, id = options.id;
            var gradientColor = options.fill;
            var fillColor = gradientColor;
            var format = this.checkColorFormat(gradientColor);
            if (!format)
                gradientColor = ej.datavisualization.Chart.prototype.colorNameToHex(gradientColor);
            var AEx,AEy, LX, LY, X, Y,X2, Y2,i=2,X1,Y1,GX=0,GY=0;
            var obj = { svgRenderer: this };
            if (seriesOption.isColumn == true) {
                var rx = options.width / 2;
                var length = options.height;
                var ry=rx/4;
                X = X1 = x;
                Y = (y - ry);
                Y1 = Y;
                AEx = 2 * rx;
                AEy = 0;
                LX = 0;
                LY = length;
                X2 = X;
                Y2 = Y + length;
                GX = 100;
                if (seriesOption.stacking = true) {
                    if (!seriesOption.isLastSeries) {
                        Y =Y1= y + ry;
                        LY = length - rx / 2;
                    }
                }
            }
            else{
                var ry = options.height / 2;
                var length = options.width;
                var rx= ry/4;
                Y=Y1=y;
                X=X1=(x+rx);
                AEx = 0;
                AEy = 2 * ry;
                LX = length;
                LY = 0;
                X2 = X + length ;
                Y2 = Y;
                GY = 100;
                if (seriesOption.stacking = true) {
                    if (!seriesOption.isLastSeries) {
                        X2 = (X + length - rx*2);
                        LX =length-rx*2;
                    }
                }
            }
                delete options.x;
                delete options.y;
                delete options.width;
                delete options.height;
                delete options.isColumn;
                //options.stroke = "black";
                while(i--){
                    direction = "M" + X.toString() + "," + Y.toString();
                    direction += "a" + rx.toString() + "," + ry.toString() + " 0 1,0 " + AEx.toString() + "," + AEy.toString();
                    direction += "a" + rx.toString() + "," + ry.toString() + " 0 1,0 " + (-1*AEx).toString() + "," + (-1*AEy).toString();
                    options.d = direction;
                    options.id = id + "_" + "Region_" + i;
                    options.fill = ej.Ej3DRender.prototype.polygon3D.prototype.applyZLight(gradientColor, obj);
                    this.drawPath(options, element);
                    X=X2;
                    Y=Y2;
                }
                direction = "M" + X1.toString() + "," + Y1.toString();
                    direction += "a" + rx.toString() + "," + ry.toString() + " 0 1,0 " + AEx.toString() + "," + AEy.toString();
                    direction += "l" + LX.toString()+ " " + LY.toString();
                    direction += "a" + rx.toString() + "," + ry.toString() + " 0 1,1 " + (-1*AEx).toString() + "," + (-1*AEy).toString() + " z";
              
            options.d = direction;
            options.id = id + "_" + "Region_2";
            if (fillColor.indexOf("url") == -1) {
                var gradientId = id.substring(0, id.indexOf('P'));
                if ($("#" + gradientId).length == 0) {

                    var optiong = { 'id': gradientId, x1: "0%", y1: "0%", x2: GX.toString() + "%", y2: GY.toString() + "%" };
                    var gradientele = [];
                    gradientele[0] = { colorStop: "0%", color: gradientColor };
                    gradientele[1] = { colorStop: "30%", color: ej.Ej3DRender.prototype.polygon3D.prototype.applyXLight(gradientColor, obj) };
                    gradientele[2] = { colorStop: "70%", color: ej.Ej3DRender.prototype.polygon3D.prototype.applyXLight(gradientColor, obj) };
                    gradientele[3] = { colorStop: "100%", color: gradientColor };
                    this.drawGradient(optiong, gradientele, element);
                }
                options.fill = "url(#" + gradientId + ")";
            }
            this.drawPath(options, element);
           
        }
    },
    
    createGradientElement: function (name, colors, x1, y1, x2, y2, element) {
        var colorName;
        if (Object.prototype.toString.call(colors) == '[object Array]') {
            var options = {
                'id': this.svgObj.id + '_' + name + 'Gradient',
                'x1': x1 + '%',
                'y1': y1 + '%',
                'x2': x2 + '%',
                'y2': y2 + '%'
            };
            var cName = '#' + this.svgObj.id + '_' + name + 'Gradient';
                this.drawGradient(options, colors, element);
            colorName = 'url(#' + this.svgObj.id + '_' + name + 'Gradient)';
        }
        else {
            colorName = colors;
        }
        return colorName;
    },

    drawGradient: function (options, gradientEle, element) {

        var defs = this.createDefs();
        var linerGradient = document.createElementNS(this.svgLink, "linearGradient");
	
        $(linerGradient).attr(options);
        for (var i = 0; i < gradientEle.length; i++) {
            var stop = document.createElementNS(this.svgLink, "stop");
            $(stop).attr({
                'offset': gradientEle[i].colorStop,
                'stop-color': gradientEle[i].color,
                'stop-opacity': 1
            });
            $(stop).appendTo(linerGradient);
        }

        $(linerGradient).appendTo(defs);
        $(defs).appendTo(element);
    },

    drawText: function (options, label, groupEle,font) {
        
        if ($("#" + options.id).length > 0) 
            this._textAttrReplace(options, label,font);           
       else{
            var text = document.createElementNS(this.svgLink, "text");
            var $text = $(text);


            if (jQuery.type(label) == "array") {
                var j = 0;
                $text.attr(options);
                for (var i = 0; i < label.length; i++) {
                    var textspan = document.createElementNS(this.svgLink, "tspan");
                    textspan.textContent = label[i];
                    $(textspan).attr({ "x": options.x, "dy": j });
                    $(textspan).appendTo(text);
                    var bounds = ej.EjSvgRender.utils._measureText(label[i],null,font);
                    j = bounds.height+2;
                }
            } else {
                text.textContent = label;
                $text.attr(options);
            }
            $text.appendTo(groupEle);

 
        }
    },
    //text element attributes values are replaced 
    _textAttrReplace: function (options, label, font) {
        $("#" + options.id).attr(options);

        if (jQuery.type(label) == "array") {
            var elements = $("#" + options.id).children("tspan");
            var j = 0;
            if (elements.length > 0 && elements.length == label.length) {
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    $(element).attr({ "x": options.x, "dy": j });
                    element.textContent = label[i];
                    var bounds = ej.EjSvgRender.utils._measureText(label[i], null, font);
                    j = bounds.height + 2;
                }
            }
            else {
                $("#" + options.id).remove();
                this.drawText(options, label, this.gTransToolEle,font);

            }
        } else {
            $("#" + options.id).text(label);
        }

    },

    drawImage: function (options, element) {

        var img = document.createElementNS(this.svgLink, 'image');
        img.setAttributeNS(null, 'height', options.height);
        img.setAttributeNS(null, 'width', options.width);
        img.setAttributeNS('http://www.w3.org/1999/xlink', 'href', options.href);
        img.setAttributeNS(null, 'x', options.x);
        img.setAttributeNS(null, 'y', options.y);
        img.setAttributeNS(null, 'id', options.id);
        img.setAttributeNS(null, 'visibility', options.visibility);
        if (!ej.util.isNullOrUndefined(options.clippath) || !ej.util.isNullOrUndefined(options.preserveAspectRatio)) {
            img.setAttributeNS(null, 'clip-path', options.clippath);
            img.setAttributeNS(null, "preserveAspectRatio", options.preserveAspectRatio);
        }
        $(img).appendTo(element);

    },

    createDefs: function () {
        var defs = document.createElementNS(this.svgLink, "defs");
        return defs;
    },

    createClipPath: function (options) {
        var clipPath = document.createElementNS(this.svgLink, "clipPath");
        $(clipPath).attr(options);
        return clipPath;
    },
    createForeignObject: function (options) {
        var object = document.createElementNS(this.svgLink, "foreignObject");
        $(object).attr(options);
        return object;
    },
    createGroup: function (options) {
        var group = document.createElementNS(this.svgLink, "g");
        $(group).attr(options);
        return group;
    },
    createPattern: function (options,element) {
        var pattern = document.createElementNS(this.svgLink, element);
        for (var name in options) {
            if (options.hasOwnProperty(name)) {
                pattern.setAttribute(name, options[name]);
            }
        }
        return pattern;
    },
    createText: function (options, label) {
        var text = document.createElementNS(this.svgLink, "text");
        $(text).attr(options);
        if (!ej.util.isNullOrUndefined(label))
            text.textContent = label;
        return text;
    },

    createPath: function (options) {
        var path = document.createElementNS(this.svgLink, "path");
        $(path).attr(options);
        return path;
    },

    createCircle: function (options) {
        var circle = document.createElementNS(this.svgLink, "circle");
        $(circle).attr(options);
        return circle;
    },

    createLine: function (options) {
        var line = document.createElementNS(this.svgLink, "line");
        $(line).attr(options);
        return line;
    },

   
    _getAttrVal: function (ele, val, option) {
        var value = $(ele).attr(val);
        if (value != null)
            return value;
        else
            return option;
    },
 
    
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        if (color.A) {
            var returncolor = "rgba(" + r.toString() + "," + g.toString() + "," + b.toString() + "," + color.A + ")";
            return returncolor;
        }
        else {
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return "#" + (hex.join("").toUpperCase());
        }
    },

    checkColorFormat: function (color) {
        return /(rgba?\((?:\d{1,3}[,\)]){3}(?:\d+\.\d+\))?)|(^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$)/gmi.test(color);
    },

    hexToRGB: function (hex) {
        var rgbRegex = /^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/.test(hex);
        var result;
        if (rgbRegex == true) {
            result = /^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/.exec(hex);
            return result ? {
                R: parseInt(result[1]),
                G: parseInt(result[2]),
                B: parseInt(result[3]),
                A: result[4],
            } : null;
        }
        else {
            result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return result ? {
                R: parseInt(result[1], 16),
                G: parseInt(result[2], 16),
                B: parseInt(result[3], 16)
            } : null;
        }
    },


    createDelegate: function (context, handler) {
        return function (e) {
            handler.apply(context, [e, this]);
        };
    },

    drawClipPath: function (options, element) {
		//To avoid creating and appending the same element more than once
		if($(element).find('#' + options.id).length > 0)
		{
			$(element).find('#' + options.id).attr(options);
		}
		else
		{
			var defs = this.createDefs();
			var clipPath = this.createClipPath({ 'id': options.id });
			this.drawRect(options, clipPath);
			this.append(clipPath, defs);
			this.append(defs, element);
		}
    },

    drawCircularClipPath: function (options, element) {
        var defs = this.createDefs();
        var clipPath = this.createClipPath({ 'id': options.id });
        this.drawCircle(options, clipPath);
        this.append(clipPath, defs);
        this.append(defs, element);
    },
   
   append: function (childEle, parentEle) {
        $(childEle).appendTo(parentEle);
   },
   _setAttr: function (element, attribute) {
       $(element).attr(attribute);
   }
};

ej.EjSvgRender.commonChartEventArgs = {
    cancel: false,
    data: null
};
ej.EjSvgRender.utils = {

   

    _decimalPlaces: function (num) {
        var match = ('' + num).match(/(?:\.(\d+))?(?:[eE]([+-]?\d+))?$/);
        if (!match) { return 0; }
        return Math.max(
       0,
        // Number of digits right of decimal point.
       (match[1] ? match[1].length : 0)
        // Adjust for scientific notation.
       - (match[2] ? +match[2] : 0));
    },
    _getLabelContent: function (pos, axis, locale) {
        switch (axis._categoryValueType) {
            case 'number':
                var customFormat = (ej.util.isNullOrUndefined(axis.labelFormat)) ? null : axis.labelFormat.match('{value}');
                return (ej.util.isNullOrUndefined(axis.labelFormat)) ? axis.labels[Math.floor(pos)] : (customFormat != null) ? axis.labelFormat.replace('{value}', axis.labels[Math.floor(pos)]) : (ej.format(axis.labels[Math.floor(pos)], axis.labelFormat, locale));
            case 'date':
                return (ej.format(new Date(axis.labels[Math.floor(pos)]), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? "dd/MM/yyyy" : axis.labelFormat), locale));
            case 'string':
                return axis.labels[Math.floor(pos)];
            default:
                return "";
        }
    },
    
    //Calculation for label size in template
    _getSeriesTemplateSize: function (point, pointIndex, series, isLeft, sender) {
        var point;
        var pointIndex;
        var areaType = sender.model.AreaType;
         
        var chartContainer = sender._id;
        var seriesIndex = $.inArray(series, sender.model._visibleSeries);

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
            var cloneNode = $("#" + series.marker.dataLabel.template).clone();
            $(cloneNode).attr("id", series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");
           
            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            var display = (areaType == "cartesianaxes" || !series.enableAnimation || (series.type.toLowerCase() == "pyramid" || series.type.toLowerCase() == "funnel")) ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
            $(templateContainer).appendTo('#' + chartContainer);
            point.size = { height: $cloneNode.height(), width: $cloneNode.width() };
         
            if (isLeft) {
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxHeight) || series.LeftLabelMaxHeight < point.size.height) {
                    series.LeftLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxWidth) || series.LeftLabelMaxWidth < point.size.width) {
                    series.LeftLabelMaxWidth = point.size.width;
                }
            }
            else {
                if (ej.util.isNullOrUndefined(series.RightLabelMaxHeight) || series.RightLabelMaxHeight < point.size.height) {
                    series.RightLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.RightLabelMaxWidth) || series.RightLabelMaxWidth < point.size.width) {
                    series.RightLabelMaxWidth = point.size.width;
                }
            }
        
    },
    getMinPointsDelta: function (axis, chartObj, start) {
        var chart = chartObj;
        var m_minPointsDelta = Number.MAX_VALUE;
        $.each(chart.model._visibleSeries, function (index, series) {
            if (series.visibility.toLowerCase() == 'visible' && axis.name == series._xAxisName) {
                var xValues = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
                if ((xValues.length == 1) && (!chart.currentSeries || chart.currentSeries._yAxisName.toLowerCase() == series._yAxisName.toLowerCase())) {
                    var minValue = ej.util.isNullOrUndefined(start) ? series.xAxis.visibleRange.min : start;
                    var delta = xValues[0].xValue - minValue;
                    if (delta != 0)
                        m_minPointsDelta = Math.min(m_minPointsDelta, delta);
                }
                else {
                    $.each(xValues, function (pointIndex, point) {
                        if (pointIndex > 0 && point.xValue) {
                            var deltaValue = point.xValue - xValues[pointIndex - 1].xValue;
                            if (deltaValue != 0) {
                                m_minPointsDelta = Math.min(m_minPointsDelta, deltaValue);
                            }
                        }
                    });
                }
            }
        });

        if (m_minPointsDelta == Number.MAX_VALUE) {
            m_minPointsDelta = 1;
        }
        return m_minPointsDelta;
    },
    //Calculation for label size
    _getSeriesMaxLabel: function (series) {
        var maxTxtDim = { width: 0, height: 0 };
        var width = $(this.svgObject).width();
        var labels=[];
        if (series.labels.length > 0) {
            for (var j = 0; j < series.labels.length; j++) {
                var dim = this._measureText(series.visibleLabels[j], width, series.marker.dataLabel.font);
                if (maxTxtDim.width < dim.width) {
                    maxTxtDim.width = dim.width;
                }
                if (maxTxtDim.height < dim.height) {
                    maxTxtDim.height = dim.height;
                }
            }
            series.LabelMaxHeight = maxTxtDim.height;
            series.LabelMaxWidth = maxTxtDim.width;
        }
        else {
            labels.push(series.rightsidePoints);
            labels.push(series.leftsidePoints);

            for (var k = 0; k < labels.length; k++) {
                for (var j = 0; j < labels[k].length; j++) {
                    var text = (labels[k][j].text) ? labels[k][j].text : labels[k][j].y;
                    var dim = this._measureText(text, width, series.marker.dataLabel.font);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                    }
                    if (maxTxtDim.height < dim.height) {
                        maxTxtDim.height = dim.height;
                    }
                }
                if (k == 0) {
                    series.RightLabelMaxHeight = maxTxtDim.height;
                    series.RightLabelMaxWidth = maxTxtDim.width;
                }
                else {
                    series.LeftLabelMaxHeight = maxTxtDim.height;
                    series.LeftLabelMaxWidth = maxTxtDim.width;
                }
             }
        }
       

    },
    _getMaxLabelWidth: function (axis, sender) {
        this.chartObj = sender;
        var maxTxtDim = { width: 0, height: 0, maxHeight:0, maxWidth:0 };
        var rotateLabel = '';
        var currentRow = 1;
        var vmlrendering = sender.svgRenderer.vmlNamespace;
        var range = axis.visibleRange;
        var intersectAction = (axis.labelIntersectAction && !sender.model.enable3D) ? axis.labelIntersectAction.toLowerCase() : "";
        var labelPlacement = axis.labelPlacement;
        var opposedPosition = axis.opposedPosition;
        var orientation = axis.orientation.toLowerCase();
        var isHorizontal = sender.model.requireInvertedAxes ? (orientation == 'vertical') : orientation == 'horizontal';
        var roundingPlaces = axis.roundingPlaces;
        var labelRotation = axis.labelRotation;
        var orientation = axis.orientation;
        var maxHeight = 0;
        var prevLabels = [];
        var visibleLabels = axis.visibleLabels;
        var visibleLabelsLength =visibleLabels.length;
        if (axis.visible) {
                for (var j = 0; j < visibleLabelsLength ; j++) {
                    axis.visibleLabels[j].y = 0;
                    var dim = this._measureText(axis.visibleLabels[j].Text, $(this.svgObject).width(), axis.font, axis.labelRotation);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                        rotateLabel = axis.visibleLabels[j].Text;
                    }
                    if (maxTxtDim.height < dim.height)
                        maxTxtDim.height = dim.height;
                }
			
			if (axis.enableTrim){   // for enable trim
			  var derivedGap = axis.maximumLabelWidth;
			  maxTxtDim.width = maxTxtDim.width > derivedGap ? derivedGap : maxTxtDim.width;
			}
			
            // initialize rows to 1
            maxTxtDim.rows = 1;
            maxTxtDim.maxWidth = maxTxtDim.width;
            maxTxtDim.maxHeight = maxTxtDim.height;
        
            if (!vmlrendering && (labelRotation ||intersectAction)) {
                var intersectRotation = 0;
                if (intersectAction == 'rotate45')
                    intersectRotation = 45;
                else if (intersectAction == 'rotate90')
                    intersectRotation = 90;
                labelRotation = labelRotation!=null || orientation== "vertical" ? labelRotation : intersectRotation;
                axis.rotationValue = labelRotation;
                if (labelRotation) {                
                    rotateLabel = (!ej.isNullOrUndefined(rotateLabel)) ? rotateLabel : '';
					labeltextWidth = this._measureText(rotateLabel, $(this.svgObject).width(), axis.font, axis.labelRotation);
					if (axis.enableTrim && labeltextWidth.width > derivedGap && rotateLabel != "") { // to find trimmed text
					  var t, textWidth, text = rotateLabel;
                      for (t = 1; t < text.toString().length; t++) {
                        text = text.toString().substring(0, t) + '... ';
                        textWidth = this._measureText(text, $(sender.svgObject).width(), axis.font);
                        if (textWidth.width >= derivedGap) {
                            text = text.toString().substring(0, t - 1) + '... ';
                            rotateLabel = text;
                            break;
                         }
                      }
                    }
					
                    maxSize = this.rotatedLabel(axis, sender, labelRotation, rotateLabel);
                    if (axis.labelIntersectAction && axis.labelIntersectAction.toLowerCase() == 'multiplerows')
                        maxHeight = maxTxtDim.height + maxSize.height;
                    maxTxtDim.height = maxSize.height;
					maxTxtDim.width = maxSize.width;
                }
            }

                if (axis.labelIntersectAction) {
                    if (intersectAction == 'wrap' || intersectAction == 'wrapbyword') {
                        // pointX calculation to find the gap between the ticks
                        var pointX, tempInterval;
                        labelPlacement = (!(labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : labelPlacement;
                        if (labelPlacement.toLowerCase() == "betweenticks") {          
                            tempInterval = 1 + (axis.labels.length > 1 ? -0.5 : 0);
                            tempInterval = (!(roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(range.interval) == 0 ?
                                1 : ej.EjSvgRender.utils._decimalPlaces(range.interval)))) : parseFloat(tempInterval.toFixed(roundingPlaces));
                            pointX = Math.ceil(((tempInterval - range.min) / (range.max - range.min)) * (axis.length));
                        }
                        else {
                            pointX = Math.ceil(axis.length / visibleLabels.length);
                        }
                        var row = Math.round(maxTxtDim.width / pointX);
                        maxTxtDim.rows = labelRotation ? row - currentRow : row + currentRow;
                        maxTxtDim.height = (maxTxtDim.height + (row) * maxTxtDim.height);
                    }
                    else if (intersectAction == 'multiplerows') {
                        var spaceValue = sender._getLegendSpace();
                        var chartBorderWidth = sender.model.border.width;
                      
                        var verticalaxis = sender.model._axes[1];
                        var realWidth = $(sender.svgObject).width() - sender.model.margin.left - sender.model.margin.right;
                        var axisTitleHeight = (axis.title.text == "" || !(axis.visible)) ? 0 : (this._measureText(axis.title.text, realWidth, axis.title.font).height + (2 * sender.model.elementSpacing));
                        var vAxesWidth = sender.model.elementSpacing + axisTitleHeight + axis.majorTickLines.size + axis.axisLine.width;
                        var yLabels = sender._getYValues(sender.model._visibleSeries[0].points);
                        var largest = Math.max.apply(Math, yLabels);
                        var vaxis = sender.model._axes[1];
                        var dim = this._measureText(largest, $(this.svgObject).width(), vaxis.font, vaxis.labelRotation);
 
                        var rightSpacing = spaceValue.rightLegendWidth + vAxesWidth + dim.width + sender.model.margin.right + sender.model.margin.left + (2 * chartBorderWidth);
                        var boundsWidth = $(sender.svgObject).width() - (rightSpacing);
                        if (isHorizontal) {
                            var addedMaxHeight = 0;
                            //loop to get current label
                            for (j = 0; j < visibleLabels.length; j++) {
                                //declaration
                                var currentLabel = visibleLabels[j];
                                var flag = false, isMultiRows = false;
                                maxHeight = Math.max(maxHeight, addedMaxHeight)
                                addedMaxHeight = currentLabel.y = maxTxtDim.height;
                                var text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                var textWidth = text.width;
                                var textHeight = text.height;
                                var currentPoint = Math.abs(Math.floor(((currentLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                // loop to get previous labels                         
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var prevLabel = prevLabels[k][l];
                                        var prevPoint = Math.abs(Math.floor(((prevLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                        var preTextWidth = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).width;
                                        var value = prevPoint + preTextWidth / 2;
                                        if (value > currentPoint - textWidth / 2 && !ej.util.isNullOrUndefined(currentLabel.y)) {
                                            addedMaxHeight = currentLabel.y + textHeight;
                                            currentLabel.y += textHeight;
                                            if (k + 1 == prevLabels.length)
                                            flag = true;
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }
                                    }
                                  
                                 }
                                currentLabel.y = addedMaxHeight;
                                row = (addedMaxHeight / textHeight) - 1;
                                if( prevLabels[row]==undefined)
                                prevLabels[row] = [];
                                prevLabels[row].push(currentLabel);
                               
                            }
                            maxHeight = (prevLabels.length * textHeight);
                         //   maxHeight = opposedPosition ? maxHeight + textHeight : maxHeight;
                            maxTxtDim.height = maxTxtDim.height > maxHeight ? maxTxtDim.height : maxHeight;
                            //calculating rows
                            if (opposedPosition)
                                maxTxtDim.rows = Math.round((maxHeight + textHeight) / textHeight);
                            else
                                maxTxtDim.rows = Math.round(maxHeight / textHeight);
                            maxTxtDim.rows = labelRotation ? maxTxtDim.rows - 1 : maxTxtDim.rows ;
                            if (maxTxtDim.rows < 1) maxTxtDim.rows = 1;
                        }
                        if (!isHorizontal) {
                            var addedMaxWidth = 0;
                            //loop to get current label
                            for (j = 0; j < axis.visibleLabels.length; j++) {
                                currentLabel = axis.visibleLabels[j];
                                text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                textWidth = text.width;
                                textHeight = text.height;
                                currentPoint = Math.abs(Math.floor(((currentLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                for (i = 0; i < j; i++) {
                                    // loop to get previous labels
                                    prevLabel = axis.visibleLabels[i];
                                    prevPoint = Math.abs(Math.floor(((prevLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                    var prevTextHeight = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).height;
                                    value = prevPoint + prevTextHeight / 2;
                                    if (value > currentPoint - textHeight / 2 && axis.visibleLabels[i].y == currentLabel.y) {
                                        addedMaxWidth = currentLabel.y + textWidth;
                                        currentLabel.y = addedMaxWidth;
                                        isMultiRows = true;
                                    }
                                }
                               var maxheight = addedMaxWidth;
                            }
                            // label width + multiple rows width + gap between the rows added only on text render in multiple rows
                            maxTxtDim.width = maxTxtDim.width + maxheight + (isMultiRows ? 5 : 0);
                        }
                    }
               }
        }
        return maxTxtDim;
    },
	
        rotatedLabel: function (axis, sender, value, rotatedLabel) {
            // to get height of rotated labels
            var rotatedOptions = {
                'font-size': axis.font.size,
                'transform': 'rotate(' + value + ',0,0)',
                'font-family': axis.font.fontFamily,
                'font-style': axis.font.fontStyle,
                'rotateAngle' : 'rotate('+ value +'deg)',
                'text-anchor': 'middle'
            };
            var text = sender.svgRenderer.createText(rotatedOptions, rotatedLabel);
            var height = Math.ceil((this._measureBounds(text, sender).height));
			var width = Math.ceil((this._measureBounds(text, sender).width));
            return {height: height, width: width};
        },
		
    _getTransform: function (xAxis, yAxis, invertedAxis) {
        var x , y , width, height;
        if (invertedAxis) {
            x = yAxis.x;
            y = xAxis.y;
            width = yAxis.width;
            height = xAxis.height;
        } else {
            {
                x = xAxis.x;
                y = yAxis.y;
                width = xAxis.width;
                height = yAxis.height;
            }
        }
        return { x: x, y: y, width: width, height: height };
    },
    //calculate path for roundedCorner of the series column,bar,stacking column,stacking bar,range column
    _calculateroundedCorner: function (cornerRadius, options, isArrow, orientation, tipPosition, tracker) {
        var x1 = options.x, y1 = options.y, padding = 5, canvasPadding = (this.chartObj.model.enableCanvasRendering && !tracker) ? padding / 4 : 0,
		width = options.width, height = options.height,topLeft,
        	bottomLeft, topRight, bottomRight, d;
        		
        		if (typeof(cornerRadius) != "object" || ej.util.isNullOrUndefined(cornerRadius))
        			topLeft = bottomLeft = topRight = bottomRight = cornerRadius;
        		else {
        		    topLeft = cornerRadius.topLeft;
        		    bottomLeft = cornerRadius.bottomLeft;
        		    topRight = cornerRadius.topRight;
        		    bottomRight = cornerRadius.bottomRight;
        		}
        		topLeft = ej.util.isNullOrUndefined(options.rx) ? topLeft : options.rx;
        		bottomLeft =  ej.util.isNullOrUndefined(options.rx) ? bottomLeft : options.rx;
        		topRight = ej.util.isNullOrUndefined(options.ry) ? topRight : options.ry;
        		bottomRight = ej.util.isNullOrUndefined(options.ry) ? bottomRight : options.ry;

		d = "M" + " " + x1 + " " + (topLeft + y1) + " Q " + x1 + " " + y1 + " " + (x1 + topLeft) + " " + y1 + " ";
        if (isArrow && orientation == "top")            
            d += "L " + (x1 + width / 2 - padding / 2 + canvasPadding) + " " + (y1) + " L " + (x1 + width / 2 + canvasPadding - tipPosition) + " " + (y1 - padding) + " L " + (x1 + width / 2 + padding / 2 + canvasPadding) + " " + (y1) + " ";
        d += "L" + " " + (x1 + width - topRight) + " " + y1 + " Q " + (x1 + width) + " " + y1 + " " + (x1 + width) + " " + (y1 + topRight) + " ";
        if (isArrow && orientation == "right")
            d += "L " + (x1 + width) + " " + (y1 + height / 2 - padding / 2 + canvasPadding) + " L " + (x1 + width + padding) + " " + (y1 + height / 2 + canvasPadding - tipPosition) + " L " + (x1 + width) + " " + (y1 + height / 2 + padding / 2 + canvasPadding) + " ";
        d += "L " + (x1 + width) + " " + (y1 + height - bottomRight) + " Q " + (x1 + width) + " " + (y1 + height) + " " + (x1 + width - bottomRight) + " " + (y1 + height) + " ";
        if (isArrow && orientation == "bottom")
            d += "L " + (x1 + width / 2 - padding / 2 + canvasPadding) + " " + (y1 + height) + " L " + (x1 + width / 2 + canvasPadding - tipPosition) + " " + (y1 + height + padding) + " L " + (x1 + width / 2 + padding / 2 + canvasPadding) + " " + (y1 + height) + " ";
        d += "L " + (x1 + bottomLeft) + " " + (y1 + height) + " Q " + x1 + " " + (y1 + height) + " " + x1 + " " + (y1 + height - bottomLeft) + " ";
        if (isArrow && orientation == "left")
            d += "L " + (x1) + " " + (y1 + height / 2 - padding / 2 + canvasPadding) + " L " + (x1 - padding) + " " + (y1 + height / 2 + canvasPadding - tipPosition) + " L " + (x1) + " " + (y1 + height / 2 + padding / 2 + canvasPadding) + " ";
        d += "L" + " " + x1 + " " + (topLeft + y1) + " " + "z";
        return d;
       
    },
    browserInfo: function () {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        browser.pointerEnabled = window.navigator.pointerEnabled;
        return browser;
    },
    _measureText: function (text, maxwidth, font) {
        var element = $(document).find("#measureTex");
		$("#measureTex").css('display', 'block'); // fixed for scroll issue in sample browser
        if (element.length==0) {
            var textObj = document.createElement('text');
            $(textObj).attr({ 'id': 'measureTex'});
            document.body.appendChild(textObj);
        }
        else
        {
            var textObj = element[0];
        }
       
        var style = null, size = null, family = null, weight = null;
        if (typeof (text) == "string" && ( text.indexOf("<") > -1 || text.indexOf(">") > -1)) {
                var textArray = text.split(" ");
                for (var i = 0; i < textArray.length; i++) {
                    if (textArray[i].indexOf("<br/>") == -1)
                       textArray[i]=textArray[i].replace(/[<>]/g, '&');                
                     }
                    text=textArray.join(' ');           
         }
        textObj.innerHTML = text;
        if (font != undefined && font.size == undefined) {
            var fontarray = font;
            fontarray=fontarray.split(" ");
            style = fontarray[0];
            size = fontarray[1];
            family = fontarray[2];
            weight = fontarray[3];
        }
       
        if (font != null) {
            textObj.style.fontSize = (font.size > 0) ? (font.size + "px") : font.size? font.size : size ;
            if (textObj.style.fontStyle) 
     			textObj.style.fontStyle = (font.fontStyle)? font.fontStyle : style;
            textObj.style.fontFamily = font.fontFamily ? font.fontFamily : family;
            if(window.navigator.userAgent.indexOf('MSIE 8.0')==-1)
                textObj.style.fontWeight = font.fontWeight ? font.fontWeight : weight;
        }
        textObj.style.backgroundColor = 'white';
        textObj.style.position = 'absolute';
        textObj.style.top = -100;
        textObj.style.left = 0;
        textObj.style.visibility = 'hidden';
        textObj.style.whiteSpace = 'nowrap';
        if (maxwidth)
            textObj.style.maxwidth = maxwidth + "px";
         
    
        var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
        $("#measureTex").css('display', 'none'); // fixed for scroll issue in sample browser
        return bounds;
    },
    //to trim legend text
    _trimText: function (legendtext, maxwidth, font) {
        var textWidth, text,textLength;
        text = legendtext.toString();
        textLength=text.length;
        for (var t = 1; t < textLength; t++) {
            text = legendtext.substring(0, t) + '...';
            textWidth = ej.EjSvgRender.utils._measureText(text, null, font).width;
            if (textWidth >= maxwidth) {
                text = text.substring(0, t - 1) + '... ';
                return text;
            }
        }
        return legendtext;
    },
    _measureBounds: function (element, sender) {
        if (sender.model.enableCanvasRendering) 
             $(document.body).append(element);
         else {
            sender.svgRenderer.append(element, sender.svgObject);
            sender.svgRenderer.append(sender.svgObject, sender.element);
        }
        var box = element.getBoundingClientRect();
        var bounds = { left: box.left, right: box.right, top: box.top, bottom: box.bottom, width: (box.right - box.left), height: (box.bottom - box.top) };
        $(element).remove();
        return bounds;
    },
    //Draw clip path for each series to avoid series overlap in multiple axes zooming
    _drawAxesBoundsClipPath: function (gSeriesGroupEle, options, sender) {
        var clipOptions;
        var element = $(gSeriesGroupEle);
        var trans = this._getTransform(options.xAxis, options.yAxis, sender.model.requireInvertedAxes);
        var width = (sender.model.AreaType == "polaraxes") ? $(sender.svgObject).width() : trans.width;
        var height = (sender.model.AreaType == "polaraxes") ? $(sender.svgObject).height() : trans.height;

        var hPlotOffset = sender.model.requireInvertedAxes ? options.yAxis.plotOffset : options.xAxis.plotOffset;
        var vPlotOffset = sender.model.requireInvertedAxes ? options.xAxis.plotOffset : options.yAxis.plotOffset;
        if (sender.model.AreaType == 'polaraxes') {
            clipOptions = {
                'id': gSeriesGroupEle ? gSeriesGroupEle.id + '_ClipRect' : '',
                'cx': sender.model.centerX,
                'cy': sender.model.centerY,
                'r': sender.model.Radius,
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'transparent'
            };
            sender.svgRenderer.drawCircularClipPath(clipOptions, gSeriesGroupEle);
        }
        else {
            clipOptions = {
                'id': gSeriesGroupEle ? gSeriesGroupEle.id + '_ClipRect' : '',
                'x': (0 - hPlotOffset),
                'y': (0 - vPlotOffset),
                'width': (width + 2 * hPlotOffset),
                'height': (height + 2 * vPlotOffset),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'transparent'
            };
            sender.svgRenderer.drawClipPath(clipOptions, gSeriesGroupEle);
        }
                
        element.attr('clip-path', 'url(#' + clipOptions.id + ')');

    },
    _getStringBuilder: function () {

        var data = [];
        var counter = 0;

        return {
            // adds string s to the stringbuilder

            append: function (s) {
                data[counter++] = s;
                return this;
            },

            // removes j elements starting at i, or 1 if j is omitted

            remove: function (i, j) {
                data.splice(i, j || 1);
                return this;
            },

            // inserts string s at i

            insert: function (i, s) {
                data.splice(i, 0, s);
                return this;
            },

            // builds the string

            toString: function (s) { return data.join(s || ""); }
        };


    },
    _addRegion: function (chart, bounds, series, point, pointIndex) {
	    var type = series.type;
        var seriesIndex = $.inArray(series, chart.model._visibleSeries);
        if (seriesIndex >= 0) {
	        var regionItem = { SeriesIndex: seriesIndex, Region: { PointIndex: pointIndex, Bounds: bounds }, type: type };
	        chart.model.chartRegions.push(regionItem);
	    }
    },

    AddRegion: function (chart, bounds, isStripLine) {
       
        if (isStripLine) {
            var regionItem = { isStripLine: isStripLine, Region: { Bounds: bounds } };
            chart.model.chartRegions.push(regionItem);
        }
    },


    _getSvgXY: function (x, y, series, sender) {
        var svgX, svgY;
        if (!(sender.model.requireInvertedAxes)) {
              svgX = x + series.xAxis.x;
              svgY = y + series.yAxis.y;
        } else {
              svgX = x + series.yAxis.x;
              svgY = y + series.xAxis.y;
        }
        return { X: svgX, Y: svgY };
    },
    _getPoint: function (point, series) {
	  
        var x = point.xValue,
         y = series.type == "boxandwhisker" ? point.YValues : point.YValues[0],
         xLength = series._isTransposed ? series.xAxis.height : series.xAxis.width;
        yLength = series._isTransposed ? series.yAxis.width : series.yAxis.height;
        point.location = {};
       
        if (series._hiloTypes) {
            var low = point.YValues[1], low = (series.yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((low == 0 ? 1 : low), series.xAxis.logBase) : low;
            low =  this._getPointXY(low, series.yAxis.visibleRange, series.yAxis.isInversed);        
            point.location.low = (series._isTransposed ?low :(1-low)) * (yLength);                        
        }

        var xvalue = (series.xAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((x == 0 ? 1 : x), series.xAxis.logBase) : x;
        var yvalue = (series.yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((y == 0 ? 1 : y), series.xAxis.logBase) : y;
     
       
        xvalue = (this._getPointXY(xvalue, series.xAxis.visibleRange, series.xAxis.isInversed));
        yvalue = this._getPointXY(yvalue, series.yAxis.visibleRange, series.yAxis.isInversed);
        point.location.X = series._isTransposed ? yvalue * (yLength) : xvalue * (xLength);
        point.location.Y = series._isTransposed ? (1 - xvalue) * (xLength) : (1 - yvalue) * (yLength);
        
        return point.location;
        
        
           
       
    },
    _getPointXY: function (value, Range, isInversed) {
         
        var result = 0;
        result = (value - Range.min) / (Range.delta);
     
        return (isInversed) ? (1 - result) : result;   
        
    },
    
    _dateTimeLabelFormat: function (intervalType) {
        var format = 'd';
        if (intervalType.toLowerCase() == 'years')
            format = 'MMM, yyyy';
        else if (intervalType.toLowerCase() == 'months')
            format = 'dd, MMM';
        else if (intervalType.toLowerCase() == 'days')
            format = 'MM/dd/yyyy';
        else if (intervalType.toLowerCase() == 'hours')
            format = 'dd, hh:mm';
        else if (intervalType.toLowerCase() == 'seconds' || intervalType.toLowerCase() == 'minutes')
            format = 'hh:mm:ss';
        else if (intervalType.toLowerCase() == 'milliseconds')
            format = 'hh:mm:ss:tt';
        return format;
    },
    _getFontString: function (fontObj) {
        if (fontObj == null)
            fontObj = {};
        if (!fontObj.FontFamily)
            fontObj.FontFamily = "Arial";
        if (!fontObj.FontStyle)
            fontObj.FontStyle = "Normal";
        if (!fontObj.Size)
            fontObj.Size = "12px";

        return fontObj.FontStyle + " " + fontObj.Size + " " + fontObj.FontFamily;
    },

    _valueToVector: function (axis, value) {
        return this._coefficientToVector(this._valueToPolarCoefficient(axis, value));
    },

    TransformToVisible: function (currentseries, x, y, sender) {
        x = (currentseries.xAxis._valueType == "logarithmic") && x > 0 ? Math.log(x, currentseries.xAxis.logBase) : x;
        y = (currentseries.xAxis._valueType == "logarithmic") && y > 0 ? Math.log(y, currentseries.yAxis.logBase) : y;
        var radius = sender.model.Radius * this._valueToCoefficient(currentseries.yAxis, y);
        //var radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToPolarCoefficient(currentseries.yAxis, y);
        var point = this._valueToVector(currentseries.xAxis, x);
        return { X: sender.model.centerX + radius * point.X, Y: sender.model.centerY + radius * point.Y };
    },

    Transform3DToVisible:function(currentseries, x, y, sender)
    {
      if (currentseries.xAxis != null && currentseries.yAxis != null)
       {
          var valueType = currentseries.xAxis._valueType.toLowerCase();
          var xIsLogarithmic = (valueType == "logarithmic") ? true :false;
          x = x = xIsLogarithmic && x > 0 ? math.log(x, xlogarithmicBase) : x;
          y = y;

       if (sender.model.requireInvertedAxes)
        {
           var left = sender.model.m_AreaBounds.X;
           var top = sender.model.m_AreaBounds.Y;
            var pointX = left + currentseries.yAxis.width * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, y, sender);
            var pointY = top + currentseries.xAxis.height * (1 - ej.EjSvgRender.utils._valueToCoefficient(currentseries.xAxis, x, sender))
           return { X: pointX, Y: pointY }          
        }
        else
        {
            var left = currentseries.xAxis.x;
            var top = currentseries.yAxis.y;
            var x = left + Math.round(currentseries.xAxis.width * ej.EjSvgRender.utils._valueToCoefficient(currentseries.xAxis, x, sender));
            var y= top + Math.round(currentseries.yAxis.height * (1 - ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis,y, sender)));
            return {X:x, Y:y}
        }
    }
                
    return new Point(0, 0);
  },

    _valueToPolarCoefficient:function(axis, value)
         {
          var start = axis.visibleRange.min;
          var delta;
          var length;
          if (axis._valueType != "category") {
              delta = (axis.visibleRange.max - axis.visibleRange.interval) - axis.visibleRange.min;
              length = axis.visibleLabels.length - 1;
          }
          else {
              delta = axis.visibleRange.delta;
              length = axis.visibleLabels.length;
          }

            result = (value - start) / delta;
            result *= 1 - 1 / (length);

            return axis.isInversed ?  result : 1 - result;
        },

    _coefficientToVector: function (coefficient)
       {
            var angle = Math.PI * (1.5 - 2 * coefficient);

            return  {X:Math.cos(angle), Y: Math.sin(angle)};
      },


    _valueToCoefficient: function (axis, value, sender) {
        if (sender && sender.model.AreaType=='polaraxes') {
            var yvalue = value;
        }
        else
        var yvalue = (axis._valueType && axis._valueType.toLowerCase() == "logarithmic") ?
          ej.EjSvgRender.utils._logBase((value == 0 ? 1 : value), axis.logBase) : value;

        yvalue = (yvalue - axis.visibleRange.min) / (axis.visibleRange.delta);
        
        return (axis.isInversed) ? 1 - yvalue : yvalue;
    },
    _getBoundingClientRect: function (element, sender,series,invertedAxes) {
        var box = element.getBoundingClientRect();
        var position = $("#" + (sender.svgObject.id))[0].getBoundingClientRect();
        var xSeries, ySeries;
        if (invertedAxes) {
            xSeries = this._getTransform(series.xAxis, series.yAxis, true).x;
            ySeries = this._getTransform(series.xAxis, series.yAxis, true).y;
        } else {
            xSeries = this._getTransform(series.xAxis, series.yAxis, false).x;
            ySeries = this._getTransform(series.xAxis, series.yAxis, false).y;
        }
        var x = box.left - (xSeries + position.left);
        var y = box.top - (ySeries + position.top);
        return { x: x, y: y, width: (box.right - box.left), height: (box.bottom - box.top) };
    },
    _minMax: function (value, min, max) {
        return value > max ? max : (value < min ? min : value);
    },
    _inside: function (value, range) {
        if (value === "")
            return false;
        return (value <= range.max) && (value >= range.min);
    },
    _logBase: function (val, base) {
        return Math.log(val) / Math.log(base);
    },
    _correctRect: function (x1, y1, x2, y2) {
        return { X: Math.min(x1, x2), Y: Math.min(y1, y2), Width: Math.abs(x2 - x1), Height: Math.abs(y2 - y1) };
    },
    _getValuebyPoint: function (x, y, series) {

        var xSize = (this.chartObj.model.requireInvertedAxes) ? series.xAxis.height : series.xAxis.width;
        var ySize = (this.chartObj.model.requireInvertedAxes) ? series.yAxis.width : series.yAxis.height;
      
        var xValue = (series.xAxis.isInversed) ? (1 - (x / xSize)) : (x / xSize);
        var yValue = (series.yAxis.isInversed) ? (1 - (y / ySize)) : (y / ySize);

        xValue = xValue * (series.xAxis.visibleRange.delta) + series.xAxis.visibleRange.min;

        yValue = yValue * (series.yAxis.visibleRange.delta) + series.yAxis.visibleRange.min;

        xValue = (series.xAxis._valueType == "logarithmic") ? Math.pow(series.xAxis.logBase, xValue) : xValue;

        yValue = (series.yAxis._valueType == "logarithmic") ? Math.pow(series.yAxis.logBase, yValue) : yValue;

        return { PointX: xValue, PointY: yValue };

    }
   
};
ej.EjSvgRender.chartTransform3D = {
      ToRadial: Math.PI / 180,

      transform3D: function (size) {
          if (!this.vector){
          this.vector=new (new ej.Ej3DRender()).vector3D();
          this.matrixobj = new (new ej.Ej3DRender()).matrix3D();
          this.bsptreeobj = new (new ej.Ej3DRender()).BSPTreeBuilder();
          this.polygon = new (new ej.Ej3DRender()).polygon3D();
          }
        return {
            mViewport: size,
            Rotation: 0,
            Tilt: 0,
            Depth: 0,
            PerspectiveAngle: 0,
            needUpdate:true,
            centeredMatrix: this.matrixobj.getIdentity(),
            Perspective: this.matrixobj.getIdentity(),
            resultMatrix: this.matrixobj.getIdentity(),
            viewMatrix: this.matrixobj.getIdentity(),
            Depth: 0           
          };
     },

    transform: function (trans) {
        this.setCenter(this.vector.vector3D(trans.mViewport.Width / 2, trans.mViewport.Height / 2, trans.Depth / 2), trans);
        this.setViewMatrix(this.matrixobj.transform(0, 0, trans.Depth), trans)
        this.setViewMatrix(this.matrixobj.getMatrixMultiplication(trans.viewMatrix, this.matrixobj.turn(-this.ToRadial * trans.Rotation)), trans);
        this.setViewMatrix(this.matrixobj.getMatrixMultiplication(trans.viewMatrix, this.matrixobj.tilt(-this.ToRadial * trans.Tilt)), trans);
        this.updatePerspective(trans.PerspectiveAngle, trans);
        trans.needUpdate = true;
        
    },

   updatePerspective:function(angle, trans)
    {
       var width = (((trans.mViewport.Width + trans.mViewport.Height) * Math.tan(this.degreeToRadianConverter((180 - Math.abs(angle % 181)) / 2.0))) + (trans.Depth * 2) / 2);
       trans.Perspective[0][0] = width;
       trans.Perspective[1][1] = width;
       trans.Perspective[2][3] = 1;
       trans.Perspective[3][3] = width;
     
   },

   degreeToRadianConverter:function(degree)
   {
           return degree * Math.PI / 180;
   },
   toScreen:function(vector3D, trans, result, chartObj)
      {
	   var result = result ? result: this.result;
	   if (!chartObj)
		    vector3D = this.matrixobj.getMatrixVectorMutiple(result(trans), vector3D);
		else{
			this.matrixobj = chartObj;
			 vector3D = chartObj.getMatrixVectorMutiple(result(trans, chartObj), vector3D);
		}
       return { x: vector3D.x, y: vector3D.y };
   },

   setViewMatrix: function (value, trans) {
       if (trans.viewMatrix == value) return;
       trans.viewMatrix = value;
       trans.needUpdate = true;
      
   },

   setCenteredMatrix: function (value, trans) {
       if (trans.viewMatrix == value) return;
       trans.centeredMatrix = value;
       trans.needUpdate = true;
       
   },

   result:function(trans, matrixobj) 
     {
	   if (!this.matrixobj) this.matrixobj = matrixobj;
       if (!trans.needUpdate) return trans.resultMatrix;
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(this.matrixobj.getInvertal(trans.centeredMatrix),
                       trans.Perspective);
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(trans.resultMatrix, trans.viewMatrix);
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(trans.resultMatrix, trans.centeredMatrix);
       trans.needUpdate = false;

       return trans.resultMatrix;
    },


   setCenter:function(center, trans)
    {
        trans.centeredMatrix = this.matrixobj.transform(-center.x, -center.y, -center.z);
        trans.needUpdate = true;
   },
   toPlane:function(point, plane, trans)
     {
     var vec1 = this.vector.vector3D(point.x, point.y, 0);
     var vec2 = this.vector.vector3DPlus(vec1, vector.vector3D(0, 0, 1));

     vec1 = this.vector.vector3DMultiply(trans.centeredMatrix , vec1);
     vec2 = this.vector.vector3DMultiply(trans.centeredMatrix , vec2);

     vec1 = this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.Perspective) , vec1);
     vec2 =  this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.Perspective) , vec2);

     vec1 = this.polygon.getPoint(vec1, this.vector.vector3DMinus(vec2 - vec1));

     vec1 = this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.viewMatrix), vec1);
     vec1 =  this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.centeredMatrix), vec1);

     return vec1;
    }

};
ej.EjSvgRender.chartSymbol =
{
    _drawSeriesType: function (location, symbolStyle, sender) {
        var seriesType = sender.model.series[symbolStyle.SeriesIndex].type.toLowerCase();
        var trendlinetype = sender.legendItem.drawType;
        seriesType = ej.util.isNullOrUndefined(trendlinetype) ? seriesType : trendlinetype;
        switch (seriesType) {
            case ej.datavisualization.Chart.Type.Line: {
                var line;
                if(ej.util.isNullOrUndefined(trendlinetype))
                     line = this._drawLine(location, symbolStyle, sender, sender.gLegendItemEle);
                else
                    line = this._drawStraightLine(location, symbolStyle, sender, sender.gLegendItemEle);
                return line;
            }
                
            case ej.datavisualization.Chart.Type.StepLine:
                return this._drawStepLine(location, symbolStyle, sender, sender.gLegendItemEle);
                 
            case ej.datavisualization.Chart.Type.StackingArea:
            case ej.datavisualization.Chart.Type.StackingArea100:
            case ej.datavisualization.Chart.Type.Area:
            case ej.datavisualization.Chart.Type.RangeArea:
                return this._drawArea(location, symbolStyle, sender, sender.gLegendItemEle);
                 
            case ej.datavisualization.Chart.Type.StepArea:
                return this._drawStepArea(location, symbolStyle, sender, sender.gLegendItemEle);
                 
            case ej.datavisualization.Chart.Type.Bar:
            case ej.datavisualization.Chart.Type.StackingBar100:
            case ej.datavisualization.Chart.Type.StackingBar:
                return this._drawBar(location, symbolStyle, sender, sender.gLegendItemEle);
                
            case ej.datavisualization.Chart.Type.Pie:
                return this._drawPie(location, symbolStyle, sender, sender.gLegendItemEle);
                
            case ej.datavisualization.Chart.Type.Doughnut:
                return this._drawDoughnut(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Hilo:
                return this._drawHilo(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.HiloOpenClose:
                return this._drawHiloOpenClose(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Candle:
                return this._drawCandle(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Pyramid:
                return this._drawPyramid(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Funnel:
                return this._drawFunnel(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Spline:
                return this._drawSpline(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.SplineArea:
                return this._drawSplineArea(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.RangeColumn:
                return this._drawRangeColumn(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Bubble:
            case ej.datavisualization.Chart.Type.Scatter:
                return this._drawCircle(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Column:
            case ej.datavisualization.Chart.Type.StackingColumn:
            case ej.datavisualization.Chart.Type.StackingColumn100:
			case ej.datavisualization.Chart.Type.Waterfall:
                return this._drawColumn(location, symbolStyle, sender, sender.gLegendItemEle);
                
            default:
                return this._drawRectangle(location, symbolStyle, sender, sender.gLegendItemEle);
                 
        }

    },

    _drawCircle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer, svgObj = sender.svgObject, style = symbolStyle.ShapeSize,
        radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2,
        symbolstyleStyle=symbolStyle.Style;
     		
        var options = {
            'id': symbolStyle.ID,
            'cx': location.startX,
            'cy': location.startY,
            'r': radius,
            'fill': symbolstyleStyle.Color,
            'stroke-width': symbolstyleStyle.BorderWidth,
            'stroke': symbolstyleStyle.BorderColor,
            'opacity': symbolstyleStyle.Opacity,
            'visibility': symbolstyleStyle.Visibility,
            'lgndCtx': symbolStyle.context
        };

        svgRender.drawCircle(options, element);

        return (location.startX - radius);
    },

    _drawLeftArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility':symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },


    _drawRightArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY + (style.height / 4)) + " z";       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility':symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);


    },


    _drawUpArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);


    },

    _drawDownArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);

    },

    _drawCross: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " "+ (location.startX + (style.width / 2)) + " " + (location.startY) + " " +
        "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawHorizLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY);
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },
    _drawVertLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);

    },

    _drawTriangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawInvertedTriangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
         path = "M" + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2)) + " "
             + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " "
             + "L" + " " + (location.startX - (style.width / 2)) + " " + (location.startY - (style.height / 2)) + " "
             + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2)) + " z";
        var x = location.startX;
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawHexagon: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawWedge: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX - style.width) + " " + (location.startY) + " " + "L" + " " + (location.startX + style.width) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (3 * (style.width / 4))) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX - style.width) + " " + (location.startY) + " z";
        var options = {
		    'stroke-linecap': 'miter',
            'stroke-miterlimit':style.width / 4,
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawPentagon: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var eq = 72;
        var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        for (var i = 0; i <= 5; i++) {
            var deg = i * eq;
            var rad = (Math.PI / 180) * deg;
            var x1 = radius * Math.cos(rad);
            var y1 = radius * Math.sin(rad);
            if (i == 0)
                sb.append("M" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");

            else
                sb.append("L" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");


        }
        sb.append("z");
        var path = sb.toString();
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawStar: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var style = symbolStyle.ShapeSize;
        var svgObj = sender.svgObject;

        var path;
        path = "M" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (-style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawRectangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2))+ " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);
    },

    _drawTrapezoid: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawDiamond: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },
    _drawEllipse: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var x = location.startX;
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'cx': x,
            'cy': y,
            'rx': style.width,
            'ry': style.height / 2
        };


        svgRender.drawEllipse(options, element);


    },
    _drawImage: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var x = location.startX + (-style.width / 2);
        var y = location.startY + (-style.width / 2);
        var width = style.width;
        var height = style.height;
        var options = { 'id': svgObj.id + '_image' + symbolStyle.PointIndex, 'height': height, 'width': width, 'href': symbolStyle.Imageurl,
            'x': x, 'y': y, 'visibility': 'visible', 'lgndCtx': symbolStyle.context
        };

        svgRender.drawImage(options, element);


    },
	
	 _drawStraightLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;

        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " +
                "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth * 2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        svgRender.drawPath(options, element);
        return (location.startX + (-style.width / 2) + (-elementspace / 4));
    },
	
    _drawLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       
        if (sender.model.enableCanvasRendering === true) {
            path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "M" + " " + (location.startX + Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10));
            var options = {
                'id': symbolStyle.ID,
                'fill': symbolStyle.Style.Color,
                'stroke-width': symbolStyle.Style.BorderWidth * 2,
                'stroke': symbolStyle.Style.Color,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path,
                'lgndCtx': true
            };
            var circlepath = "M" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "a " + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + 2 * (Math.floor(style.width / 3)) + " " + 0 + " " + "a" + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + (-2 * (Math.floor(style.width / 3))) + " " + 0;
            var circleoptions = {
                'id': symbolStyle.ID,
                'fill': "transparent",
                'stroke-width': symbolStyle.Style.BorderWidth * 2,
                'stroke': symbolStyle.Style.Color,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': circlepath,
                'lgndCtx': true
            };

            svgRender.drawPath(options, element);
            svgRender.drawPath(circleoptions, element);
        }
        else {
            path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "M" + " " + (location.startX + Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " + "M" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "a " + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + 2 * (Math.floor(style.width / 3)) + " " + 0 + " " + "a" + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + (-2 * (Math.floor(style.width / 3))) + " " + 0;
            var options = {
                'id': symbolStyle.ID,
                'fill': "transparent",
                'stroke-width': symbolStyle.Style.BorderWidth * 2,
                'stroke': symbolStyle.Style.Color,
                'opacity': symbolStyle.Style.Opacity,
                'visibility': symbolStyle.Style.Visibility,
                'd': path,
                'lgndCtx': true
            };
            svgRender.drawPath(options, element);
        }
        return (location.startX + (-style.width / 2) + (-elementspace / 4));

    },
    _drawColumn: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
         path = "M" + " " + (location.startX - 3 * (style.width / 5)) + " " + (location.startY - (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (-style.width / 10)) + " " + (location.startY - (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (-style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX - 3 * (style.height / 5)) + " " + (location.startY + (style.height / 2)) + " " + "Z" + " " + "M" + " " + (location.startX + (-style.width / 10) - (style.width / 20)) + " " + (location.startY - (style.height / 4) - (elementspace / 2)) + " " + "L" + " " + (location.startX + (style.width / 10) + (style.width / 20)) + " " + (location.startY - (style.height / 4) - (elementspace / 2)) + " " + "L" + " " + (location.startX + (style.width / 10) + (style.width / 20)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 10) - (style.width / 20)) + " " + (location.startY + (style.height / 2)) + " " + "Z" + " " + "M" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + 3 * (style.width / 5)) + " " + (location.startY) + " " + "L" + " " + (location.startX + 3 * (style.width / 5)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "Z";
      
         var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
             'lgndCtx': true
        };

        svgRender.drawPath(options, element);

         return (location.startX - 3 * (style.width / 5));

    },
    _drawRangeColumn: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
        path = "M" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY - (style.height / 2) - (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "Z";
       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 5));

    },

    _drawBar: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY - 3 * (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY - 3 * (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "Z" + " " + "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY - (style.height / 5) + (elementspace / 20)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY - (style.height / 5) + (elementspace / 20)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10) + (elementspace / 20)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10) + (elementspace / 20)) + " " + "Z" + " " + "M" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height/5) + (elementspace / 10)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height/5) + (elementspace / 10)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/10)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/10)) + " " + "Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));

    },
   
    _drawStepLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
       
        path = "M" + " " + (location.startX + (-style.width / 2) - (elementspace/4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + "" + (location.startX + (style.width/2) + (elementspace/4)) + " " + (location.startY + (style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) - (elementspace / 4));

    },
    _drawSpline: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;        
        path = "M" + " " + (location.startX - (style.width / 2)) + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + location.startX + " " + (location.startY - style.height) + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "M" + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2));		
        var options = {
            'id': symbolStyle.ID,
             'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX - (style.width / 2));

    },
    _drawSplineArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;       
        path = "M" + " " + (location.startX - (style.width / 2)) + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + location.startX + " " + (location.startY - style.height) + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " +  "Z" +" " + "M" + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2)) +" " +" Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX - (style.width / 2));

    },
    _drawArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX - (style.width / 2) - (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4) + (-elementspace / 8)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4) + (elementspace / 8)) + " " + (location.startY + (-style.height / 2) + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.height / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX - (style.width / 2) - (elementspace / 4));

    },
    _drawStepArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;

        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX - (style.width / 4)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + (location.startX - (style.width / 4)) + " " + (location.startY - (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY - (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startX) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "Z";       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));

    },
    _drawPyramid: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (-elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/8)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));


    },
    _drawFunnel: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 5)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 5)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 5));

    },
    _drawCandle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
       
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "Z" + " " + "M" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "M" + " " + (location.startX) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 4) + (elementspace / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2));

    },
    _drawHilo: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)+(elementspace/4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)+(-elementspace/4));        
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (style.width / 2));

    },
    _drawHiloOpenClose: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var cartesian = polarToCartesian(18, 12, 4.0710678118654755, 270);
        var elementspace = symbolStyle.ElementSpace;
        var path;        
        
        path = "M" + " " + (location.startX) + " " + (location.startY - 3 * (style.height / 10)) + " " + "L" + " " + (location.startX - (style.width / 2) - (elementspace / 4)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "M" + " " + (location.startX) + " " + (location.startY - (style.height / 2) - (elementspace / 4)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "M" + " " + (location.startX) + " " + (location.startY + 3 * (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + 3 * (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return ((location.startX) - (style.width / 2));

    },

    _drawDoughnut: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var radius = Math.sqrt(style.height * style.width) / 2;
        radius = radius + (elementspace / 5);
        var cartesianlarge = polarToCartesian(location.startX, location.startY, radius, 270);
        var cartesiansmall = polarToCartesian(location.startX + (style.width / 10), location.startY, radius, 270);
        var elementspace = symbolStyle.ElementSpace;
        path = "M" + " " + location.startX + " " + location.startY + " " + "L" + " " + (location.startX + radius) + " " + (location.startY) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 1 + " " + 1 + " " + cartesianlarge[0] + " " + cartesianlarge[1] + " " + "Z" + " " + "M" + " " + (location.startX + (style.width / 10)) + " " + (location.startY - (style.height / 10)) + " " + "L" + (location.startX + (radius)) + " " + (location.startY - style.height / 10) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 0 + " " + 0 + " " + cartesiansmall[0] + " " + cartesiansmall[1] + " " + "Z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        var circle = {
            'id': symbolStyle.ID, 'cx': location.startX, 'cy': location.startY, 'r': (radius/2),'lgndCtx': true,
            'fill': "white", 'stroke-width': symbolStyle.Style.BorderWidth, 'stroke': symbolStyle.Style.BorderColor, 'opacity': symbolStyle.Style.Opacity, 'visibility': symbolStyle.Style.Visibility
        };

        svgRender.drawPath(options, element);
        svgRender.drawCircle(circle, element);

        return (location.startX-radius);
    },

    _drawPie: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var radius = Math.sqrt(style.height * style.width) / 2;
        radius = radius + (elementspace / 5);
        var cartesianlarge = polarToCartesian(location.startX, location.startY, radius, 270);
        var cartesiansmall = polarToCartesian(location.startX + (style.width / 10), location.startY, radius, 270);
        var elementspace = symbolStyle.ElementSpace;
        path = "M" + " " + location.startX + " " + location.startY + " " + "L" + " " + (location.startX + radius) + " " + (location.startY) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 1 + " " + 1 + " " + cartesianlarge[0] + " " + cartesianlarge[1] + " " + "Z" + " " + "M" + " " + (location.startX + (style.width / 10)) + " " + (location.startY - (style.height / 10)) + " " + "L" + (location.startX + (radius)) + " " + (location.startY - style.height / 10) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 0 + " " + 0 + " " + cartesiansmall[0] + " " + cartesiansmall[1] + " " + "Z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);
        return (location.startX - radius);
    }

};
function polarToCartesian(startX, startY, radius, angleInDegrees) {
    var angleInRadians = angleInDegrees * Math.PI / 180.0;
    var x = startX + radius * Math.cos(angleInRadians);
    var y = startY + radius * Math.sin(angleInRadians);
    return [x, y];
}
ej.EjSvgRender.seriesPalette = {
    defaultMetro: ["#E94649", "#F6B53F", "#6FAAB0", "#C4C24A", "#FB954F", "#005277", "#8BC652", "#69D2E7", "#E27F2D", "#6A4B82"],
    defaultHighContrast: ["#F93A00", "#44E2D6", "#DDD10D", "#0AA368", "#0556CB", "#AB40B2", "#5F930A", "#D12E41", "#E0670E", "#008FFF"],
    defaultOffice: ["#005277", "#8BC652", "#6A4B82", "#E94649", "#6FAAB0", "#F7B74F", "#C4C24A", "#EF863F", "#69D2E7", "#FFD13E"],
    defaultMaterial: ["#663AB6","#EB3F79","#F8AB1D","#B82E3D","#049CB1","#F2424F","#C2C924","#3DA046","#074D67","#02A8F4"],
    defaultGradient:
        {
            borderColors: ["#F34649", "#F6D321", "#6EB9B0", "#CBC26A", "#FBAF4F", "#E2CDB1", "#FFC0B7", "#68E1E6", "#E1A62D", "#9C6EBF"],
            seriesColors: [[{ color: "#F34649", colorStop: "0%" }, { color: "#B74143", colorStop: "100%"}],
     [{ color: "#F6D321", colorStop: "0%" }, { color: "#F6AE26", colorStop: "100%"}],
     [{ color: "#6EB9B0", colorStop: "0%" }, { color: "#3F77BD", colorStop: "100%"}],
     [{ color: "#CBC26A", colorStop: "0%" }, { color: "#9AAD21", colorStop: "100%"}],
     [{ color: "#FBAF4F", colorStop: "0%" }, { color: "#F07542", colorStop: "100%"}],
     [{ color: "#E2CDB1", colorStop: "0%" }, { color: "#AAA089", colorStop: "100%"}],
     [{ color: "#8BC652", colorStop: "0%" }, { color: "#6F9E41", colorStop: "100%"}],
     [{ color: "#68E1E6", colorStop: "0%" }, { color: "#3D9CBE", colorStop: "100%"}],
     [{ color: "#E1A62D", colorStop: "0%" }, { color: "#B66824", colorStop: "100%"}],
     [{ color: "#9C6EBF", colorStop: "0%"}], [{ color: "#593F6D", colorStop: "100%"}]]
        },
    blueMetro: ["#005378", "#006691", "#007EB5", "#0D97D4", "#00AEFF", "#14B9FF", "#54CCFF", "#87DBFF", "#ADE5FF", "#C5EDFF"],
    blueGradient:
     {
         seriesColors: [[{ color: "#005277", colorStop: "0%" }, { color: "#00304F", colorStop: "100%"}],
            [{ color: "#006590", colorStop: "0%" }, { color: "#004068", colorStop: "100%"}],
            [{ color: "#007DB4", colorStop: "0%" }, { color: "#00558B", colorStop: "100%"}],
            [{ color: "#0D97D4", colorStop: "0%" }, { color: "#057FC7", colorStop: "100%"}],
            [{ color: "#00ADFE", colorStop: "0%" }, { color: "#008BE9", colorStop: "100%"}],
            [{ color: "#14B8FE", colorStop: "0%" }, { color: "#0798EB", colorStop: "100%"}],
            [{ color: "#53CBFF", colorStop: "0%" }, { color: "#35AFEB", colorStop: "100%"}],
            [{ color: "#86DAFF", colorStop: "0%" }, { color: "#64C0EC", colorStop: "100%"}],
            [{ color: "#ACE5FF", colorStop: "0%" }, { color: "#8DCEED", colorStop: "100%"}],
            [{ color: "#C4ECFF", colorStop: "0%"}], [{ color: "#A3D1E6", colorStop: "100%"}]],
         borderColors: ["#005277", "#006590", "#007DB4", "#0D97D4", "#00ADFE", "#14B8FE", "#53CBFF", "#86DAFF", "#ACE5FF", "#C4ECFF"]
     },
    greenMetro: ["#496612", "#597B15", "#709A1B", "#87B62A", "#9AD926", "#A6DC37", "#BCE654", "#C8E780", "#D5EFA5", "#E2F3BE"],
    greenGradient:
     {

         seriesColors: [[{ color: "#5C7F16", colorStop: "0%" }, { color: "#384C08", colorStop: "100%"}],
            [{ color: "#6A9319", colorStop: "0%" }, { color: "#486009", colorStop: "100%"}],
            [{ color: "#739D1C", colorStop: "0%" }, { color: "#57760B", colorStop: "100%"}],
            [{ color: "#90B546", colorStop: "0%" }, { color: "#6E9215", colorStop: "100%"}],
            [{ color: "#9AD826", colorStop: "0%" }, { color: "#75A010", colorStop: "100%"}],
            [{ color: "#A5DB36", colorStop: "0%" }, { color: "#8EB91D", colorStop: "100%"}],
            [{ color: "#BBE554", colorStop: "0%" }, { color: "#A4C849", colorStop: "100%"}],
            [{ color: "#C8E780", colorStop: "0%" }, { color: "#B4D072", colorStop: "100%"}],
            [{ color: "#D4EEA5", colorStop: "0%" }, { color: "#BFD593", colorStop: "100%"}],
            [{ color: "#E1F2BD", colorStop: "0%"}], [{ color: "#C8D7A8", colorStop: "100%"}]],
         borderColors: ["#5C7F16", "#6A9319", "#739D1C", "#90B546", "#9AD826", "#A5DB36", "#BBE554", "#C8E780", "#D4EEA5", "#E1F2BD"]
     },

    sandleMetro: ["#6C450C", "#82520D", "#A36812", "#C07F1F", "#E69719", "#E89A2B", "#EEB564", "#F3CB93", "#F7DEB4", "#F9E6CA"],
    sandleGradient:
     {


         seriesColors: [[{ color: "#7F602F", colorStop: "0%" }, { color: "#512D04", colorStop: "100%"}],
            [{ color: "#986827", colorStop: "0%" }, { color: "#673803", colorStop: "100%"}],
            [{ color: "#A16C1F", colorStop: "0%" }, { color: "#8A4B05", colorStop: "100%"}],
            [{ color: "#BF812A", colorStop: "0%" }, { color: "#AD630D", colorStop: "100%"}],
            [{ color: "#E49519", colorStop: "0%" }, { color: "#B86607", colorStop: "100%"}],
            [{ color: "#E7992A", colorStop: "0%" }, { color: "#D7780D", colorStop: "100%"}],
            [{ color: "#EDB463", colorStop: "0%" }, { color: "#D98F31", colorStop: "100%"}],
            [{ color: "#F2CA92", colorStop: "0%" }, { color: "#DAAC6F", colorStop: "100%"}],
            [{ color: "#F6DDB3", colorStop: "0%" }, { color: "#DABE8F", colorStop: "100%"}],
            [{ color: "#F8E5C9", colorStop: "0%"}], [{ color: "#DDBE92", colorStop: "100%"}]],
         borderColors: ["#7F602F", "#986827", "#A16C1F", "#BF812A", "#E49519", "#E7992A", "#EDB463", "#F2CA92", "#F6DDB3", "#F8E5C9"]

     }
};

ej.EjSvgRender.themes = {
    flatlight:
    {
        highlightColor : 'black',

        background: 'transparent',
        legend:
        {
            font: { color: "#282828"},
            title:
            {
               font: { color: "#282828"}
             }
        },
        title:
        {
            font: { color: '#565656'},
            subTitle:
                {
                    font: { color: '#565656'}
                }
        },
        primaryXAxis:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            labelBorder:
            {
               color: "#8E8E8E"
            },
			multiLevelLabelsColor: "#8E8E8E",
			multiLevelLabelsFontColor: "#282828",
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 3, ry: 3, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        secondaryX:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            labelBorder:
            {
                color: "#8E8E8E"
            },
			multiLevelLabelsColor: "#8E8E8E",
			multiLevelLabelsFontColor: "#282828",
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 3, ry: 3, border: { color: '#3D3D3D' ,width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
        },

        primaryYAxis:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            labelBorder:{
                color: "#8E8E8E"
            },
			multiLevelLabelsColor: "#8E8E8E",
			multiLevelLabelsFontColor: "#282828",
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 3, ry: 3, border: { color: '#3D3D3D', borderWidth: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        secondaryY:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            labelBorder:{
                color: "#8E8E8E"
            },
			multiLevelLabelsColor: "#8E8E8E",
			multiLevelLabelsFontColor: "#282828",
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 3, ry: 3, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#565656' } }
},
  errorBar:{
        fill:"#000000",
		cap:
		{
			fill:"#000000"
		}
  },
  connectorLine: { color: '#565656' }
	},
        crosshair:
        {
            line:
            {
                width: 1,
                color: 'Black'
            }
        },
        
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
        colors: ej.EjSvgRender.seriesPalette.defaultMetro

    },

    flatdark:
        {
            highlightColor : 'white',

            background: '#111111',

            legend:
            {
                font: { color: "#C9C9C9"},
                title:
                {
                  font: { color: "#C9C9C9"}
                }
            },
            title:
            {
                font: { color: '#C9C9C9'},
                subTitle:
                {
                    font: { color: '#C9C9C9'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 }, 
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {   rx: 0, ry: 0, fill: '#B5B5B5', border:{color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border:{color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                      color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#C9C9C9' } }
},
errorBar:{
           fill:"#ffffff",
		   cap:
		   {
			fill:"#ffffff"
		   }
},
connectorLine: { color: '#C9C9C9' }
        },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },
            
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
            colors: ej.EjSvgRender.seriesPalette.defaultMetro

        },


    gradientlight:
        {
            highlightColor : 'black',

            background: 'transparent',
            legend:
            {
                font: { color: "#282828"},
                title:
                {
                    font: { color: "#282828"}
                }
            },
            title:
            {
                font: { color: '#565656'},
                subTitle:
                {
                    font: { color: '#565656'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 labelBorder:{
                     color: "#8E8E8E"
                 },
				 multiLevelLabelsColor: "#8E8E8E",
				 multiLevelLabelsFontColor: "#282828",
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 labelBorder:{
                      color: "#8E8E8E"
                 },
				 multiLevelLabelsColor: "#8E8E8E",
				 multiLevelLabelsFontColor: "#282828",
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 labelBorder:{
                      color: "#8E8E8E"
                 },
				 multiLevelLabelsColor: "#8E8E8E",
				 multiLevelLabelsFontColor: "#282828",
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 labelBorder:{
                      color: "#8E8E8E"
                 },
				 multiLevelLabelsColor: "#8E8E8E",
				 multiLevelLabelsFontColor: "#282828",
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#565656' } }
},
    errorBar:{
        fill:"#000000",
		cap:
		{
			fill:"#000000"
		}
    },
    connectorLine: { color: '#565656' }
	},
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'Black'
                 }
             },
            
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

        },

    gradientdark:
        {
            highlightColor : 'white',

            background: '#111111',
            legend:
            {
                font: { color: "#C9C9C9"},
                title:
               {
                   font: { color: "#C9C9C9"}
               }
            },
            title:
            {
                font: { color: '#C9C9C9'},
                subTitle:
                {
                    font: { color: '#C9C9C9'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                      color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder: {
                     color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
				 multiLevelLabelsColor: "#AAAAAA",
				 multiLevelLabelsFontColor: "#C9C9C9",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#C9C9C9' } }
      },
     errorBar:{
              fill:"#ffffff",
		      cap:
		      {
			     fill:"#ffffff"
		      }
     },
     connectorLine: { color: "#C9C9C9" }
        },
       
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },
             
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

        },

    highcontrast01:
        {
            highlightColor : 'white',

            background: '#111111',
            legend:
            {
                font: { color: '#ffffff' },
                title:
               {
                   font: { color: '#ffffff' }
               }
            },
            title:
            {
                font: { color: '#ffffff' },
                subTitle:
                {
                    font: { color: '#ffffff' }
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
	             multiLevelLabelsColor: "#AAAAAA",
                 multiLevelLabelsFontColor: "#ffffff",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#ffffff' },
                 title:
                 {

                     font: { color: '#ffffff' }
                 },
                 crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#ffffff' },
                 title:
                 {

                     font: { color: '#ffffff' }
                 },
                 crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 labelBorder:{
                     color: "#AAAAAA"
                 },
                 multiLevelLabelsColor: "#AAAAAA",
                 multiLevelLabelsFontColor: "#ffffff",
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#ffffff' },
                 title:
                 {

                     font: { color: '#ffffff' }
                 },
                 crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#ffffff' },
                 title:
                 {

                     font: { color: '#ffffff' }
                 },
                 crosshairLabel: { rx: 3, ry: 3, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            commonSeriesOptions:
                  {
                      marker: {
                          dataLabel: { font: { color: '#ffffff' } }
                      },
                      errorBar: {
                          fill: "#ffffff",
                          cap:
                          {
                              fill: "#ffffff"
                          }
                      },
                      connectorLine: { color: "#C9C9C9" }
                  },

            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },

            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultHighContrast,
            colors: ej.EjSvgRender.seriesPalette.defaultHighContrast

        },
    material:
   {
       highlightColor : 'black',

       background: 'transparent',
       legend:
       {
           font: { color: "#333333" },
           title:
           {
               font: { color: "#333333" }
           }
       },
       title:
       {
           font: { color: '#333333' },
           subTitle:
               {
                   font: { color: '#333333' }
               }
       },
       primaryXAxis:
       {
           majorGridLines:
           {
               color: "#DFDFDF"
           },
           majorTickLines:
           {
               color: "#8E8E8E"
           },
           minorGridLines:
           {
               color: "#DFDFDF"
           },
           minorTickLines:
           {
               color: "#8E8E8E"
           },
          labelBorder:
           {
               color: "#8E8E8E"
           },
           multiLevelLabelsColor: "#8E8E8E",
           multiLevelLabelsFontColor: "#333333",
           axisLine: { color: '#8E8E8E' },
           font: { color: '#333333' },
           title:
           {
               font: { color: '#333333' }
           },
           crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
       },
       secondaryX:
       {
           majorGridLines:
           {
               color: "#DFDFDF"
           },
           majorTickLines:
           {
               color: "#8E8E8E"
           },
           minorGridLines:
           {
               color: "#DFDFDF"
           },
           minorTickLines:
           {
               color: "#8E8E8E"
           },
           axisLine: { color: '#8E8E8E' },
           font: { color: '#333333' },
           title:
           {
               font: { color: '#333333' }
           },
           crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
       },

       primaryYAxis:
       {
           majorGridLines:
           {
               color: "#DFDFDF"
           },
           majorTickLines:
           {
               color: "#8E8E8E"
           },
           minorGridLines:
           {
               color: "#DFDFDF"
           },
           minorTickLines:
           {
               color: "#8E8E8E"
           },
           labelBorder:
           {
               color: "#8E8E8E"
           },
           multiLevelLabelsColor: "#8E8E8E",
           multiLevelLabelsFontColor: "#333333",
           axisLine: { color: '#8E8E8E' },
           font: { color: '#333333' },
           title:
           {
               font: { color: '#333333' }
           },
           crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', borderWidth: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
       },
       secondaryY:
       {
           majorGridLines:
           {
               color: "#DFDFDF"
           },
           majorTickLines:
           {
               color: "#8E8E8E"
           },
           minorGridLines:
           {
               color: "#DFDFDF"
           },
           minorTickLines:
           {
               color: "#8E8E8E"
           },
           axisLine: { color: '#8E8E8E' },
           font: { color: '#333333' },
           title:
           {
               font: { color: '#333333' }
           },
           crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
       },
       commonSeriesOptions:
             {
                 marker: {
                     dataLabel: { font: { color: '#333333' } }
                 },
                 errorBar: {
                     fill: "#000000",
                     cap:
                     {
                         fill: "#000000"
                     }
                 },
                 connectorLine: { color: '#565656' }
             },
       crosshair:
       {
           line:
           {
               width: 1,
               color: 'Black'
           }
       },

       seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMaterial,
       colors: ej.EjSvgRender.seriesPalette.defaultMaterial

   },
    office365:
  {
      highlightColor : 'black',

      background: 'transparent',
      legend:
      {
          font: { color: "#333333" },
          title:
          {
              font: { color: "#333333" }
          }
      },
      title:
      {
          font: { color: '#333333' },
          subTitle:
              {
                  font: { color: '#333333' }
              }
      },
      primaryXAxis:
      {
          majorGridLines:
          {
              color: "#DFDFDF"
          },
          majorTickLines:
          {
              color: "#8E8E8E"
          },
          minorGridLines:
          {
              color: "#DFDFDF"
          },
          minorTickLines:
          {
              color: "#8E8E8E"
          },
         labelBorder:
           {
               color: "#8E8E8E"
           },
           multiLevelLabelsColor: "#8E8E8E",
           multiLevelLabelsFontColor: "#333333",
          axisLine: { color: '#8E8E8E' },
          font: { color: '#333333' },
          title:
          {
              font: { color: '#333333' }
          },
          crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
      },
      secondaryX:
      {
          majorGridLines:
          {
              color: "#DFDFDF"
          },
          majorTickLines:
          {
              color: "#8E8E8E"
          },
          minorGridLines:
          {
              color: "#DFDFDF"
          },
          minorTickLines:
          {
              color: "#8E8E8E"
          },
          axisLine: { color: '#8E8E8E' },
          font: { color: '#333333' },
          title:
          {
              font: { color: '#333333' }
          },
          crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
      },

      primaryYAxis:
      {
          majorGridLines:
          {
              color: "#DFDFDF"
          },
          majorTickLines:
          {
              color: "#8E8E8E"
          },
          minorGridLines:
          {
              color: "#DFDFDF"
          },
          minorTickLines:
          {
              color: "#8E8E8E"
          },
          labelBorder:
           {
               color: "#8E8E8E"
           },
           multiLevelLabelsColor: "#8E8E8E",
           multiLevelLabelsFontColor: "#333333",
          axisLine: { color: '#8E8E8E' },
          font: { color: '#333333' },
          title:
          {
              font: { color: '#333333' }
          },
          crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', borderWidth: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
      },
      secondaryY:
      {
          majorGridLines:
          {
              color: "#DFDFDF"
          },
          majorTickLines:
          {
              color: "#8E8E8E"
          },
          minorGridLines:
          {
              color: "#DFDFDF"
          },
          minorTickLines:
          {
              color: "#8E8E8E"
          },
          axisLine: { color: '#8E8E8E' },
          font: { color: '#333333' },
          title:
          {
              font: { color: '#333333' }
          },
          crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
      },
      commonSeriesOptions:
            {
                marker: {
                    dataLabel: { font: { color: '#333333' } }
                },
                errorBar: {
                    fill: "#000000",
                    cap:
                    {
                        fill: "#000000"
                    }
                },
                connectorLine: { color: '#565656' }
            },
      crosshair:
      {
          line:
          {
              width: 1,
              color: 'Black'
          }
      },

      seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultOffice,
      colors: ej.EjSvgRender.seriesPalette.defaultOffice

  }
};
$.extend(ej.EjSvgRender.themes, {

    "azure":
    {

        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
        colors: ej.EjSvgRender.seriesPalette.blueMetro
    },

    "azuredark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
        colors: ej.EjSvgRender.seriesPalette.blueMetro
    },
    "gradient-azure":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
    },

    "gradient-azuredark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
    },

    "lime":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
        colors: ej.EjSvgRender.seriesPalette.greenMetro
    },

    "limedark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
        colors: ej.EjSvgRender.seriesPalette.greenMetro
    },
    "gradient-lime":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
    },

    "gradient-limedark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
    },
    "saffron":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
        colors: ej.EjSvgRender.seriesPalette.sandleMetro
    },

    "saffrondark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
        colors: ej.EjSvgRender.seriesPalette.sandleMetro
    },
    "gradient-saffron":
{
    seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
    colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
},

    "gradient-saffrondark":
{
    seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
    colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
}
});
})(jQuery);;
ej.axisTypes = {};

ej.EjAxisRenderer = function () {
};
var _sideBySeriesPadding = function (chartObj, start, end, axis) {
    var visibleSeries = ej.DataManager(chartObj.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
    var isPadding = false;
    var data, startPadding, endPadding;
    var visibleSeriesLength = visibleSeries.length;
	var axisOrientation = axis.orientation.toLowerCase();
    for (var i = 0; i < visibleSeriesLength; i++) {
        var type = visibleSeries[i].type.toLowerCase();
        if (((type.indexOf("column") != -1 || type.indexOf("bar") != -1 || type.indexOf("waterfall") != -1 || type == "candle" || type.indexOf("hilo") != -1 || type.indexOf("box") != -1)))
        {
            isPadding = true;
            break;
        }

    }
    if (chartObj.model.AreaType != "polaraxes" && isPadding) {
        axis.m_minPointsDelta = undefined;
        var padding = ej.EjSvgRender.utils.getMinPointsDelta(axis, chartObj, start) * 0.5;
        start = start - padding;
        end = (end + padding > Number.MAX_VALUE) ? Number.MAX_VALUE : end + padding;
        axis.padding = padding;
    }
    else
		axis.padding=0;
    return data = { startPadding: start, endPadding: end };
};
ej.EjStripline = function (chartobj) {
    this.chart = chartobj;
};
(function ($) {
    ej.EjAxisRenderer.prototype = {

        //Calculate min/max value for series
        _seriesMinMax: function (chartObj, axis, seriesCollection, type, params) {
     		var orientation = axis.orientation.toLowerCase(), seriesLength = seriesCollection.length;
     		params.seriesCollection = ej.util.isNullOrUndefined(params.seriesCollection) ? {} : params.seriesCollection;
            var count;
            if ((seriesLength > 0 && chartObj.model._hasSeriesPoints && seriesCollection[0].points.length > 0) || this.chartObj.model.indicators[0]._points.length>0) {

                var maxX, minY, minX, maxY;
                var maxMinXValue = { maxX: null, minX: null };
                var maxMinYValue = { maxY: null, minY: null };
                var xMax = maxMinXValue.maxX;
                var xMin = maxMinXValue.minX;
                var yMax = maxMinYValue.maxY;
                var yMin = maxMinYValue.minY;
                var seriesVisibility,pointsLength,visibleLength,seriesType;

                for (var i = 0; i < seriesLength; i++) {
                    seriesVisibility = seriesCollection[i].visibility.toLowerCase();
                    pointsLength = seriesCollection[i].points.length;
                    count = 0;
                    if (seriesVisibility === 'visible') {
                        seriesCollection[i]._visiblePoints = [];
                        for (var j = 0; j < pointsLength; j++) {
                            var point = seriesCollection[i].points[j];
                             if ((point.x != null && point.x != undefined) && ((typeof point.y == "object" || !isNaN(point.y)) && point.y != null && point.y != undefined)
							 || point.showIntermediateSum || point.showTotalSum || point.isEmpty == false) {
                                seriesCollection[i]._visiblePoints[count] = point;
								seriesCollection[i]._visiblePoints[count].actualIndex = j;
                                count++;
                            }
                        }
                    }
                    axis.maxPointLength = pointsLength;
                }

                for (var i = 0; i < seriesLength; i++) {
                    visibleLength = seriesCollection[i]._visiblePoints.length;
					var series = seriesCollection[i];
                    if (visibleLength > 0) {
                        seriesVisibility = seriesCollection[i].visibility.toLowerCase();
                        pointsLength = seriesCollection[i].points.length;
                        if (seriesVisibility === 'visible' && pointsLength > 0) {
                            if ((orientation == "horizontal" && !(chartObj.model.requireInvertedAxes)) || (orientation == "vertical" && chartObj.model.requireInvertedAxes) ) {
                                minX = maxX = seriesCollection[i]._visiblePoints[0].xValue;
                                for (var j = 0; j < visibleLength; j++) {
                                    var currentPoint = seriesCollection[i]._visiblePoints[j].xValue;
                                    if (minX > currentPoint)
                                        minX = currentPoint;
                                    if (maxX < currentPoint)
                                        maxX = currentPoint;
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;

                                if ((xMin === null || xMin === undefined) && visibleLength > 0) {
                                    xMax = seriesCollection[i].maxX;
                                    xMin = seriesCollection[i].minX;
                                }
                                if (xMin > seriesCollection[i].minX) {
                                    xMin = seriesCollection[i].minX;
                                }
                                if (xMax < seriesCollection[i].maxX) {
                                    xMax = seriesCollection[i].maxX;
                                }
                            }
                            else {
								var isSpline = (series.type.toLowerCase().indexOf('spline')!=-1), controlPoint, naturalSpline, getBezierControlPoints,
									controlPointsCount = 0, visiblePoints = seriesCollection[i]._visiblePoints;
								if(isSpline){
									var seriesKey = series._name = series.name || 'series' + i;									
									params.seriesCollection[seriesKey] = {};
									visiblePoints = ej.ejSplineSeries.prototype._isVisiblePoints(series),
									naturalSpline = params.seriesCollection[seriesKey].naturalSpline = ej.ejSplineSeries.prototype.naturalSpline(visiblePoints);
									params.seriesCollection[seriesKey].controlPoints = [];
									getBezierControlPoints = ej.ejSplineSeries.prototype.getBezierControlPoints;									
								}
                                maxY = minY = visiblePoints[0].YValues[0];
								 var minval,maxval;
                                for (var j = 0; j < visiblePoints.length; j++) {
                                    var currentPoint = visiblePoints[j];
                                    if (seriesCollection[i]._hiloTypes) {
                                        for (var y = 0; y < 2; y++) {
                                            if (minY > currentPoint.YValues[y])
                                                minY = currentPoint.YValues[y];
                                            if (maxY < currentPoint.YValues[y])
                                                maxY = currentPoint.YValues[y];
                                        }
                                    }
									else if(series.type.toLowerCase() == "boxandwhisker")
									{
									     minval =  visiblePoints[j].YValues[0][0];
										 maxval = visiblePoints[j].YValues[0][0];
										for (var yindex = 0; yindex < currentPoint.YValues[0].length; yindex++) 
										{
											if ( minval > currentPoint.YValues[0][yindex])
                                                minval = currentPoint.YValues[0][yindex];
											if (maxval < currentPoint.YValues[0][yindex])
                                                maxval = currentPoint.YValues[0][yindex];
										}
									    minY = minval > minY  ? minY : minval;
										maxY = maxval < maxY ? maxY : maxval;
									}
									else if(isSpline){
										if(j!=0 && currentPoint.visible && visiblePoints[j-1].visible){											
											params.seriesCollection[seriesKey].controlPoints.push(getBezierControlPoints(visiblePoints[j-1], currentPoint, naturalSpline[j-1], naturalSpline[j], 0)); 
											controlPoint = params.seriesCollection[seriesKey].controlPoints[controlPointsCount++];
											minY = Math.min(minY, currentPoint.YValues[0], controlPoint.controlPoint1.YValues[0], controlPoint.controlPoint2.YValues[0]);
											maxY = Math.max(maxY, currentPoint.YValues[0], controlPoint.controlPoint1.YValues[0], controlPoint.controlPoint2.YValues[0]);
										}
										else{
											minY = Math.min(minY, currentPoint.YValues[0] || null);
											maxY = Math.max(maxY, currentPoint.YValues[0] || null);										
										}
									}
									else {
                                        if (minY > currentPoint.YValues[0])
                                            minY = currentPoint.YValues[0];
                                        if (maxY < currentPoint.YValues[0])
                                            maxY = currentPoint.YValues[0];
                                    }
                                }
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;

                                if ((yMin === null || yMin === undefined) && visibleLength > 0) {
                                    yMax = seriesCollection[i].maxY;
                                    yMin = seriesCollection[i].minY;
                                }
                                if (yMin > seriesCollection[i].minY) {
                                    yMin = seriesCollection[i].minY;
                                }
                                if (yMax < seriesCollection[i].maxY) {
                                    yMax = seriesCollection[i].maxY;
                                }

                                // Finding Min and Max for Column and Bar series
                                seriesType = seriesCollection[i].type
                                if ((seriesType == 'column' || seriesType == 'bar') && axis.startFromZero)
                                    yMin = (yMin < 0) ? yMin : 0;
                            }
                        }
                    }
                }
                if ((orientation == "horizontal" && !(chartObj.model.requireInvertedAxes)) || (orientation == "vertical" && chartObj.model.requireInvertedAxes)) {

                    // Find min/max for indicator series
                    if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                        var value = this.chartObj.model.indicatorRange[axis.name];
                        if (yMin > value.min || yMin == null)
                            yMin = value.min;
                        if (yMax < value.max || yMax == null)
                            yMax = value.max;                       
                    }

                    if (xMax == xMin)
                        xMax += 1;

                    if (type == "double") {
                        var data = _sideBySeriesPadding(chartObj, xMin, xMax, axis);
                        xMin = data.startPadding;
                        xMax = data.endPadding;
                        deltaX = xMax - xMin;
                        axis.range = this._getDoubleAutoRange(axis, xMin, xMax, deltaX);    
                    }                        
                    else
                        axis.range = this._getLogAutoRange(axis, xMin, xMax, xMax - xMin);

                }
                else {

                    // Find min/max for stacked series
                    if (this.chartObj.model.stackedValue[axis.name]) {
                        var value = this.chartObj.model.stackedValue[axis.name],
                            isStacked = true;
                        if (yMin > value.min)                            
                            yMin = value.min;
                        if (yMax < value.max)
                            yMax = value.max;
                    }

                    // Find min/max for indicator series
                    if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                        var value = this.chartObj.model.indicatorRange[axis.name];                       
                        if (yMin > value.min || yMin == null)
                            yMin = value.min;
                        if (yMax < value.max || yMax == null)
                            yMax = value.max;                        
                    }

                    if (yMax == yMin) {                   // max == min
                        if (yMax < 0)                                  // value less than 0
                            yMax = 0;
                        else if (yMax <= 1 && yMax > 0)   // value between 0 and 1
                            yMin = 0;
                        else {                                                      // value greater than 1
                            yMin = 0;
                            yMax += 1;
                        }
                    }

                    if ((seriesType == 'column' || seriesType == 'bar') && !axis.startFromZero) {
                        var maxLimit = (axis.range && axis.range.max) || yMax;
                        var interval = (axis.range && axis.range.interval) || this.calculateNumericNiceInterval(maxLimit - yMin, axis, null, maxLimit);
                        yMax += interval;
                        if ((yMin - interval < 0 && yMin > 0) || isStacked) {
                            yMin = 0;                           
                        }
                        else
                            yMin -= interval;
                    }

                    var deltaY = yMax - yMin;
                    isStacked = false;

                    if (type == "double")
                        axis.range = this._getDoubleAutoRange(axis, yMin, yMax, deltaY)
                    else
                        axis.range = this._getLogAutoRange(axis, yMin, yMax, deltaY)

                }
            }
            else {
                if (!axis.setRange){
                    axis.range = { min: 0, max: 5, interval: 1, Delta: 4 };
				}
				else {
                    min=( axis.range.min== null || axis.range.min == undefined )?0:axis.range.min;
                    max = (axis.range.max == null || axis.range.max == undefined) ? 5 : axis.range.max;
			        delta=max-min;
			        axis.range=this._getDoubleAutoRange(axis,min,max,delta)
				}
            }
			 var commonAxisEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAxisEventArgs.data = { range: axis.range, axis: axis };
            this.chartObj._trigger("axesRangeCalculate", commonAxisEventArgs);
            var range = commonAxisEventArgs.data.range;
            axis.range = { min: range.min, max: range.max, interval: range.interval, delta: (range.max - range.min) };

        },

        //Calculate ranges for Trendline series
        _calculateTrendLineRange: function (chart, axis, seriesCollection) {            
            var series, trendlines, trendline, count, serLength, trendLength, trendlineType;
            var serLength = seriesCollection.length;
            for (var m = 0; m < serLength; m++) {
                series = seriesCollection[m];
                trendlines = series.trendlines;
                if (chart.model.AreaType == "cartesianaxes" && (axis.name == series._xAxisName || axis.name == series._yAxisName)) {
                    trendLength = trendlines.length;
                    for (var i = 0; i < trendLength; i++) {
                        trendline = trendlines[i];
                        if (trendline.visibility.toLowerCase() === 'visible') {                            
                            
                            if (axis.name == series._xAxisName) {
                                trendlineType = trendline.type.toLowerCase();
                                options = new ej.trendlineTypes[trendlineType]();
                                options.calculateTrendLineSegment(series, trendline, axis, chart);
                            } else
                                if (axis.name == series._yAxisName && !axis.setRange && trendline.points && trendline.points.length > 1) {
                                    count = trendline.points.length - 1;
                                    axis.range.min = axis.range.min > trendline.minY ? trendline.minY : axis.range.min;
                                    axis.range.max = axis.range.max < trendline.maxY ? trendline.maxY : axis.range.max;
                                }
                        }
                    }
                }
            }
            return false;
        },

        AlignRangeStart: function (sDate, intervalSize, intervalType) {
            var sResult = new Date(sDate);
            if (intervalType.toLowerCase() == "days") {
                var day = Math.floor(Math.floor((sDate.getDate()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), day, 0, 0, 0);
            } else if (intervalType.toLowerCase() == "hours") {
                var hour = Math.floor(Math.floor((sDate.getHours()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), hour, 0, 0);
            } else if (intervalType.toLowerCase() == "milliseconds") {
                var milliseconds = Math.floor(Math.floor((sDate.getMilliseconds()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), sDate.getSeconds(), milliseconds);
            } else if (intervalType.toLowerCase() == "seconds") {
                var seconds = Math.floor(Math.floor((sDate.getSeconds()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), seconds, 0);
            } else if (intervalType.toLowerCase() == "minutes") {
                var minutes = Math.floor(Math.floor((sDate.getMinutes()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), minutes, 0, 0);
            } else if (intervalType.toLowerCase() == "months") {
                var month = Math.floor(Math.floor((sDate.getMonth()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), month, sDate.getDate(), 0, 0, 0);
            } else if (intervalType.toLowerCase() == "years") {
                var year = Math.floor(Math.floor(sDate.getFullYear() / intervalSize) * intervalSize);
                sResult = new Date(year, sDate.getMonth(), sDate.getDate(), 0, 0, 0);
            }
            return sResult;
        },
        _increaseDateTimeInterval: function (date, interval, intervalType) {
            var result = new Date(date);
            if (intervalType.toLowerCase() == "days") {
                result.setDate(date.getDate() + interval);
            } else if (intervalType.toLowerCase() == "hours") {
                result.setHours(date.getHours() + interval);
            } else if (intervalType.toLowerCase() == "milliseconds") {
                result.setMilliseconds(date.getMilliseconds() + interval);
            } else if (intervalType.toLowerCase() == "seconds") {
                result.setSeconds(date.getSeconds() + interval);
            } else if (intervalType.toLowerCase() == "minutes") {
                result.setMinutes(date.getMinutes() + interval);
            } else if (intervalType.toLowerCase() == "months") {
                result.setMonth(date.getMonth() + interval);
            } else if (intervalType.toLowerCase() == "years") {
                result.setYear(date.getFullYear() + interval);
            }

            return result;
        },

        _calculatePadding: function (axis) {

            var start = (typeof axis.range.min == "string" && !isNaN(Date.parse(axis.range.min))) ? Date.parse(axis.range.min) : (axis.range.min).getTime();
            var end = (typeof axis.range.max == "string" && !isNaN(Date.parse(axis.range.max))) ? Date.parse(axis.range.max) : (axis.range.max).getTime();
            var interval = ((this._increaseDateTimeInterval(new Date(start), axis.range.interval, axis._intervalType)).getTime()) - start;
            var rangePadding =axis.rangePadding.toLowerCase();
            if (!this.chartObj.zoomed && !axis.setRange) {
                start = new Date(start);
                end = new Date(end);
                var intervalType = axis._intervalType.toLowerCase();
                if (axis.rangePadding.toLowerCase() == 'none') {
                start =  start.getTime();
                end = end.getTime();
            } else if (rangePadding == 'additional' || rangePadding == 'round') {
                switch (intervalType) {
                    case 'years':
                        var startYear = start.getFullYear();
                        var endYear = end.getFullYear();
                        if (axis.rangePadding.toLowerCase() == 'additional') {
                            start = (new Date(startYear - axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                            end = (new Date(endYear + axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                        } else {
                            start = new Date(startYear, 0, 0, 0, 0, 0).getTime();
                            end=new Date(endYear, 11, 30, 23, 59, 59).getTime();
                        }
                        break;
                    case 'months':                        
                        var month = start.getMonth();
                        var endMonth = end.getMonth();
                        if (rangePadding == 'round') {
                            start = (new Date(start.getFullYear(), month, 0, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), endMonth, new Date(end.getFullYear(), end.getMonth(), 0).getDate(), 23, 59, 59)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), month + (-axis.range.interval), 1, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), endMonth + (axis.range.interval), endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                        }
                        break;
                    case 'days':
                        var day = start.getDate();
                        var endDay = end.getDate();
                        if (rangePadding == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), day, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), endDay, 23, 59, 59)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), day + (-axis.range.interval), 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), endDay + (axis.range.interval), 0, 0, 0)).getTime();
                        }
                        break;
                    case 'hours':
                        var hour = (start.getHours() / axis.range.interval) * axis.range.interval;
                        var endHour = end.getHours() + (start.getHours() - hour);
                        if (rangePadding == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour, 59, 59)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour + (-axis.range.interval), 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour + (axis.range.interval), 0, 0)).getTime();
                        }
                        break;
                    case 'minutes':
                        var minute = (start.getMinutes() / axis.range.interval) * axis.range.interval;
                        var endMinute = end.getMinutes() + (start.getMinutes() - minute);
                        if (rangePadding == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHour(), minute, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), endMinute, 59)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), minute + (-axis.range.interval), 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), minute + (axis.range.interval), 0)).getTime();
                        }
                        break;
                    case 'seconds':
                        var second = (start.getSeconds() / axis.range.interval) * axis.range.interval;
                        var endSecond = end.getSeconds() + (start.getSeconds() - second);
                        if (rangePadding == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), end.getMinutes(), endSecond, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second + (-axis.range.interval), 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), endSecond + (axis.range.interval), 0)).getTime();
                        }
                        break;
                    case 'milliseconds':
                        var milliSecond = (start.getMilliseconds() / axis.range.interval) * axis.range.interval;
                        var endMilliSecond = end.getMilliseconds() + (start.getMilliseconds() - milliSecond);
                        if (rangePadding == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond + (-axis.range.interval))).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond + (axis.range.interval))).getTime();
                        }
                        break;
                }

            }

        }
        axis.actualRange.interval = interval;
        axis.actualRange.min = start;
        axis.actualRange.max = end;
        axis._interval = axis.range.interval;
        axis.actualRange.delta = (axis.actualRange.max - axis.actualRange.min);
        axis.range.min = new Date(start);
        axis.range.max = new Date(end);
        this._calculateVisibleRange(axis);

    },
    _calculateVisibleRange: function (axis, sender, isScroll) {
        this.chartObj = sender ? sender : this.chartObj;
        axis.visibleRange = (!isScroll) ? $.extend(true, {}, axis.actualRange) : axis.visibleRange;
        if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType != "polaraxes" && !this.chartObj.model.disableZoom) {

            axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
            axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
            var baseRange = axis.actualRange;
            if (axis.isInversed) {
                var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }

            var startDate = new Date(start);
            var endDate = new Date(end);
			 if (start == end)
            {
                startDate = new Date(Date.parse(startDate) - 2592000000);
                endDate = new Date(Date.parse(endDate) + 2592000000);
            }
            //Use below code to find min,max and interval for visible range
			 var min = Math.min(startDate.getTime(), endDate.getTime());
			 var max = Math.max(startDate.getTime(), endDate.getTime());
			 var intervalX = axis.enableAutoIntervalOnZooming ? this.calculateDateTimeNiceInterval(axis, startDate, endDate) : { 'interval': axis._interval, 'intervalType': axis._intervalType };
			 var interval = ((this._increaseDateTimeInterval(startDate, intervalX.interval, intervalX.intervalType)).getTime()) - min;

			 if (isScroll) {
			     return {min:min, max:max, interval:interval};
			 }
			 else {
			     axis._intervalType = intervalX.intervalType;
			     axis.visibleRange.min = min;
			     axis.visibleRange.max = max;
			     axis.visibleRange.interval = interval;
			     axis.visibleRange.delta = Math.abs(axis.visibleRange.max - axis.visibleRange.min);
			     axis._interval = intervalX.interval;
			 }
        }

    },
    _calculateAxisLabels: function (axis) {
        var position,text;
		var minRange = axis.enableAutoIntervalOnZooming ? axis.visibleRange.min : axis.actualRange.min;
        if (this.chartObj.zoomed || axis.zoomed || !axis.setRange)
            position = (this.AlignRangeStart(new Date(minRange), axis._interval, axis._intervalType)).getTime();

        else
            position = axis.visibleRange.min;
        while (position <= axis.visibleRange.max) {
            if (position >= axis.visibleRange.min && position <= axis.visibleRange.max) {
                text=(ej.format(new Date(position), ((!(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat), this.chartObj.model.locale));
				 var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
				 commonAxesEventArgs.data = { label: { Text: text, Value: position }, axis: axis };
                this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                axis.visibleLabels.push({ Value: position, Text: commonAxesEventArgs.data.label.Text });
            }
            position = (this._increaseDateTimeInterval(new Date(position), axis._interval, axis._intervalType)).getTime();
            
            axis.visibleLabels[axis.visibleLabels.length - 1] = axis.visibleLabels[axis.visibleLabels.length-1];
        }
        axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
    },
    calculateDateTimeNiceInterval: function (axis, startDate, endDate) { 
        var oneDay = 24 * 60 * 60 * 1000;
        //var axisInterval ;
        var totalDays = (Math.abs((startDate.getTime() - endDate.getTime()) / (oneDay)));
        var interval = this.calculateNumericNiceInterval(totalDays / 365, axis, "years");

        if (interval >= 1) {
            return { interval: interval, intervalType: "Years" };
        }

        interval = this.calculateNumericNiceInterval(totalDays / 30, axis, "months");

        if (interval >= 1) {
            return { interval: interval, intervalType: "Months" };
        }

        interval = this.calculateNumericNiceInterval(totalDays, axis, "days");

        if (interval >= 1) {


            return { interval: interval, intervalType: "Days" };
        }

        var totalHours = totalDays * 24;

        interval = this.calculateNumericNiceInterval(totalHours, axis, "hours");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Hours" };
        }

        var totalMinutes = totalDays * 24 * 60;

        interval = this.calculateNumericNiceInterval(totalMinutes, axis, "minutes");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Minutes" };
        }

        var totalSeconds = totalDays * 24 * 60 * 60;

        interval = this.calculateNumericNiceInterval(totalSeconds, axis, "seconds");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Seconds" };
        }

        var totalMilliseconds = totalDays * 24 * 60 * 60 * 1000;

        interval = this.calculateNumericNiceInterval(totalMilliseconds, axis, "milliseconds");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Milliseconds" };
        }

    },
    calculateNumericNiceInterval: function (delta, axis, setInterval, max) {
        var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
        var niceInterval = delta / desiredIntervalsCount;
        if (axis._valueType=='double'){
            if (axis.desiredIntervals!= null)
                return niceInterval;
        }
        var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10))),
            intervalDivs = [10, 5, 2, 1],
            maxLimit = max || (axis.range && axis.range.max);


        for (var i = 0; i < intervalDivs.length; i++) {
            var currentInterval = minInterval * intervalDivs[i];
            if (desiredIntervalsCount < (delta / currentInterval)) {
                if (axis._valueType == "datetime") {
                    return niceInterval=(!ej.util.isNullOrUndefined(axis.intervalType) && axis.intervalType.toLowerCase() == setInterval) ? Math.ceil(niceInterval):niceInterval;
                }
				 if (!ej.util.isNullOrUndefined(axis.range) && !ej.util.isNullOrUndefined(axis.range.max))
				     return (niceInterval > maxLimit ? currentInterval : niceInterval);
				 else
					return niceInterval;
            }

            niceInterval = currentInterval;
        }

        return niceInterval;
    },
    GetActualDesiredIntervalsCount: function (axis, size) {
        if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
            var desiredIntervalsCount = (axis.orientation.toLowerCase() == "horizontal" ? 0.533 : 1) * axis.maximumLabels;
            desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
        
                return desiredIntervalsCount;
            } else {
                return axis.desiredIntervals;
            }
        },
        _calculateRange: function (axis, seriesCollection) {
            if (!this.chartObj.zoomed) {
                var defaultDate = new Date();
                if (seriesCollection.length > 0 && this.chartObj.model._hasSeriesPoints && seriesCollection[0].points.length > 0 || this.chartObj.model.indicators[0]._points.length > 0) {
                    var maxX, minY;
                    var maxY, minX;
                    var maxMinXValue = { maxX: null, minX: null };
                    var maxMinYValue = { maxY: null, minY: null };
                    var pointLength;
                    for (var i = 0; i < seriesCollection.length; i++) {
                        pointLength = seriesCollection[i].points.length;
                        if (pointLength > 0) {
                            if (seriesCollection[i].visibility.toLowerCase() === 'visible' && seriesCollection[i].points.length > 0) {
                                minX = maxX = seriesCollection[i].points[0].xValue;
                                maxY = minY = seriesCollection[i].points[0].YValues[0];                                
                                for (var j = 0; j < pointLength; j++) {
                                    if (minX != null && minX != undefined) {
                                        if (minX > seriesCollection[i].points[j].xValue)
                                            minX = seriesCollection[i].points[j].xValue;
                                        if (maxX < seriesCollection[i].points[j].xValue)
                                            maxX = seriesCollection[i].points[j].xValue;
                                    }
                                    else {
                                        minX = maxX = seriesCollection[i].points[j+1].xValue;
                                    }
                                    if (minY > seriesCollection[i].points[j].YValues[0])
                                        minY = seriesCollection[i].points[j].YValues[0];
                                    if (maxY < seriesCollection[i].points[j].YValues[0])
                                        maxY = seriesCollection[i].points[j].YValues[0];
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;
                                if (!axis.maxPointLength || axis.maxPointLength < pointLength)
                                    axis.maxPointLength = pointLength;
                            }
                        }
                    
                    if ((maxMinXValue.minX = (maxMinXValue.minX == null  && typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX) > seriesCollection[i].minX) {
                            maxMinXValue.minX = (typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX;
                        }
                        if ((maxMinXValue.maxX = (maxMinXValue.maxX == null && typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX) < seriesCollection[i].maxX) {
                            maxMinXValue.maxX = (typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX;
                        }
                        if (maxMinYValue.minY > seriesCollection[i].minY) {
                            maxMinYValue.minY = seriesCollection[i].minY;
                        }
                        if (maxMinYValue.maxY < seriesCollection[i].maxY) {
                            maxMinYValue.maxY = seriesCollection[i].maxY;
                        }
               
                }
				// Find min/max for indicator series

                  if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                    var value = this.chartObj.model.indicatorRange[axis.name];
                    if (maxMinXValue.minX > new Date(value.min) || maxMinXValue.minX == null)
                        maxMinXValue.minX = new Date(value.min);
                    if (maxMinXValue.maxX < new Date(value.max) || maxMinXValue.maxX == null)
                        maxMinXValue.maxX = new Date(value.max);
                 }
                maxMinXValue.minX = (maxMinXValue.minX == null) ? defaultDate : maxMinXValue.minX;
                maxMinXValue.maxX = (maxMinXValue.maxX == null) ? new Date(new Date().setMonth(defaultDate.getMonth() + 5)) : maxMinXValue.maxX;
                if (maxMinXValue.minX == maxMinXValue.maxX) {
                    maxMinXValue.minX = new Date(Date.parse(maxMinXValue.minX) - 2592000000);
                    maxMinXValue.maxX = new Date(Date.parse(maxMinXValue.maxX) + 2592000000);
                }

                var data = _sideBySeriesPadding(this.chartObj, Date.parse(maxMinXValue.minX), Date.parse(maxMinXValue.maxX), axis);
                maxMinXValue.minX = new Date(data.startPadding);
                maxMinXValue.maxX = new Date(data.endPadding);

                

                var intervalX = this.calculateDateTimeNiceInterval(axis, maxMinXValue.minX, maxMinXValue.maxX);                
                axis._intervalType = (axis.intervalType) ? axis.intervalType : intervalX.intervalType;
                if (axis.intervalType == "auto")
                    axis._intervalType = axis.intervalType = intervalX.intervalType;
                if (axis.setAxisInterval == null || axis.setAxisInterval==undefined) {
                    axis.setAxisInterval = (axis.range && axis.range.interval) ? true : false;
                }
				
                //condition chechked when changing interval dynamically
                if (axis._setInterval && !this.chartObj._chartResize && axis.range && axis._setInterval != axis.range.interval)
                    axis._setInterval = null;

                if (!axis.setRange){
                    axis.range = { min: maxMinXValue.minX, max: maxMinXValue.maxX, interval: intervalX.interval };
                } else {
                        if (!axis.range.max)
                            axis.range.max = maxMinXValue.maxX;
                        if (!axis.range.min)
                            axis.range.min = maxMinXValue.minX;
                        if (typeof axis.range.min == "string" && axis.range.min.indexOf("/Date(") != -1)
                            axis.range.min = new Date(parseInt(axis.range.min.substr(6)));
                        if (typeof axis.range.max == "string" && axis.range.max.indexOf("/Date(") != -1)
                            axis.range.max = new Date(parseInt(axis.range.max.substr(6)));
                        if ( new Date(axis.range.min).getTime() ==  new Date(axis.range.max).getTime()) {                            
                            axis.range.max = new Date(Date.parse(axis.range.max) + 86400000);
                        }
                        intervalX = this.calculateDateTimeNiceInterval(axis, new Date(axis.range.min), new Date(axis.range.max));
                        axis._intervalType = (axis.intervalType) ? axis.intervalType : intervalX.intervalType;
                        if (!axis.setAxisInterval) {
                            axis.range.interval = intervalX.interval; // set auto interval value
                            axis._setInterval = intervalX.interval;
                        }
                        axis.actual_Range = $.extend(true, {}, axis.range);
                        axis.setRange = true;
                    var interval = axis._setInterval? intervalX.interval : (axis.range.interval || axis.actual_Range.interval);
                    axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: interval };
                  }
                } else {
                    if (!axis.setRange) {
                        axis.range = { min: defaultDate, max: new Date(new Date().setMonth(defaultDate.getMonth() + 5)), interval: 1 };
                        axis._intervalType = 'Months';
                    }
					else {
                        axis.range.min=( axis.range.min === null || axis.range.min === undefined) ?defaultDate:axis.range.min;
                        axis.range.max=(axis.range.max === null || axis.range.max=== undefined)?new Date(new Date().setMonth(defaultDate.getMonth() + 5)):axis.range.max;
                        axis.range.interval = (axis.range.interval === null || axis.range.interval === undefined) ? 1 : axis.range.interval;
			            axis._intervalType='Months';
                    }

                }
				 var commonAxisEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxisEventArgs.data = { range: axis.range, axis: axis };
                this.chartObj._trigger("axesRangeCalculate", commonAxisEventArgs);
                var range = commonAxisEventArgs.data.range;
                axis.range = { min: range.min, max: range.max, interval: range.interval, delta: (range.max - range.min) };
            }
        },
        _calculateRanges: function (chartObj, axis, seriesCollection) {

            this.chartObj = chartObj;

            this._calculateRange(axis, seriesCollection);

            if (chartObj.model._drawTrendline)
                this._calculateTrendLineRange(chartObj, axis, seriesCollection);

            this._calculatePadding(axis);

        this._calculateAxisLabels(axis);
    }
};

function ejExtendClass(parent, members) {
    var object = function () {
    };
    object.prototype = new parent();
    $.extend(object.prototype, members);
    return object;
}

//DateTime calculation
var ejDateTimeValue = ejExtendClass(ej.EjAxisRenderer);
ej.axisTypes.datetime = ejDateTimeValue;

var ejDoubleValue = ejExtendClass(ej.EjAxisRenderer, {
    
  
    _calculateRange: function (chartObj, axis, seriesCollection, params) {
        if (!chartObj.zoomed) {
            this._seriesMinMax(chartObj,axis, seriesCollection, "double", params);
        }
    },
    
    _getDoubleAutoRange: function (axis, min, max) {
        var delta;
        var interval;
        var intervalstr;
        if (!axis.setRange) {
            axis.range = { min: min, max: max};

        } else {
           
            if (ej.util.isNullOrUndefined(axis.range.max))
                       axis.range.max = max;          
            if (ej.util.isNullOrUndefined(axis.range.min))
                       axis.range.min = min;
     
            axis.actual_Range = $.extend(true, {}, axis.range);
            axis.setRange = true;
        }
        delta = axis.range.max - axis.range.min;
        interval = this.calculateNumericNiceInterval(delta, axis);
        if (ej.EjSvgRender.utils._decimalPlaces(interval) > 20) {
            intervalstr = interval.toString();
            interval = parseFloat((intervalstr.substring(0, intervalstr.indexOf(".")) + intervalstr.substring(intervalstr.indexOf("."), 22)));
        }
        if ((ej.util.isNullOrUndefined(axis.range.interval))||(axis.range.interval<0))
            axis.range.interval = interval;
        axis.range.delta = delta;

        return axis.range;
    },
    _calculatePadding: function (chartObj, axis, baseRange) {
        var start = baseRange.min;
        var end = baseRange.max;
        var rangePadding = axis.rangePadding.toLowerCase();
        var interval= baseRange.interval;
        if ((!axis.setRange) && (!chartObj.zoomed)) {
               
            if (rangePadding == 'normal') {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = baseRange.min + (baseRange.min / 20);

                    remaining =  interval + (minimum %  interval);

                    if ((0.365 *  interval) >= remaining) {
                        minimum -= interval;
                    }

                    if (minimum %  interval < 0) {
                        minimum = (minimum -  interval) - (minimum % interval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * end)
                        ? 0
                        : (start - (end - start) / 2);
                    if (minimum %  interval > 0) {
                        minimum -= (minimum %  interval);
                    }
                }
                if (end > 0)
                    var maximum = ((end + (end - start) / 20) > Number.MAX_VALUE) ? Number.MAX_VALUE : (end + (end - start) / 20);
                else
                    var maximum = (end - (end - start) / 20);

                remaining = interval - (maximum % interval);

                if ((0.365 *  interval) >= remaining) {
                    maximum = (maximum + interval > Number.MAX_VALUE) ? Number.MAX_VALUE : maximum + interval;
                }

                if (maximum %  interval > 0) {
                    maximum = ((maximum + interval) - (maximum % interval) > Number.MAX_VALUE) ? Number.MAX_VALUE : (maximum + interval) - (maximum % interval);
                }
                 if (minimum <= 0) {
                    interval = this.calculateNumericNiceInterval(maximum - minimum, axis);
                    maximum = (Math.ceil(maximum / interval) * interval > Number.MAX_VALUE) ? Number.MAX_VALUE : Math.ceil(maximum / interval) * interval;
                }
                start = (!(axis.roundingPlaces)) ? parseFloat(minimum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(minimum.toFixed(axis.roundingPlaces));
                end = (!(axis.roundingPlaces)) ? parseFloat(maximum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(maximum.toFixed(axis.roundingPlaces));

            } else if (rangePadding == 'additional' || rangePadding == 'round') {
                start = Math.floor(start / interval) * interval;
                end = (Math.ceil(end / interval) * interval > Number.MAX_VALUE) ? Number.MAX_VALUE : Math.ceil(end / interval) * interval;
                if (rangePadding == 'additional') {
                    start -= interval;
                    end = end + interval >= Number.MAX_VALUE ? Number.MAX_VALUE : end + interval;
                }
            }


        }
        axis.actualRange.min = start;
        axis.actualRange.max = end;
        axis.actualRange.interval = interval;
        axis.range.min = start;
        axis.range.max = end;
        axis.range.interval =  interval;
        axis.actualRange.delta = end - start >= Number.MAX_VALUE ? end : end - start;
        this._calculateVisibleRange(axis);
    },
    _calculateVisibleRange: function (axis, sender, isScroll) {
        this.chartObj = sender ? sender : this.chartObj;
        axis.visibleRange = (!isScroll) ? $.extend(true, {}, axis.actualRange) : axis.visibleRange;
        if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType != "polaraxes" && !this.chartObj.model.disableZoom) {
            var baseRange = axis.actualRange;

            axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
            axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
            if (axis.isInversed) {
                var start=axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }
          
            var delta =Math.abs(end-start);
            var interval = axis.enableAutoIntervalOnZooming ? this.calculateNumericNiceInterval(delta, axis) : axis.actualRange.interval;          
            if (isScroll) {
                return { min: Math.min(start, end), max: Math.max(start, end), delta: delta, interval: interval };
            }
            else {                
                axis.visibleRange.min = Math.min(start, end);
                axis.visibleRange.max = Math.max(start, end);
                axis.visibleRange.delta = delta;
                axis.visibleRange.interval = interval;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }
        }

    },
    _calculateAxisLabels: function (chartObj, currentAxis) {

        var tempInterval,
            customFormat,
             round = currentAxis.roundingPlaces,
             tempPlace, labelValue,
            commonAxesEventArgs,
            labelText, locale = this.chartObj.model.locale,
            labelFormat = currentAxis.labelFormat, labelPrecision, labelPrecisionHighest = 20, labelPrecisionDefault = 6;
        if (chartObj.zoomed || currentAxis.zoomed || currentAxis.padding)
            tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
		else
            tempInterval = currentAxis.visibleRange.min;
        for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {          

            if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {
                tempPlace = (tempInterval.toString().split(0).length - 1 > 10) ? 10 : 20;
                tempInterval = parseFloat(tempInterval.toFixed((!round && round > tempPlace) ? round : tempPlace));
				//By default axis labels are rounded based on interval. These rounded values are not used to draw labels, so labels are misplaced from ticks
				labelValue = (!round) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval)))) : tempInterval;
                labelText = (!(round)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(round));
                
                customFormat = currentAxis.labelFormat;
                if(!(customFormat))
                    labelText = !currentAxis.isStacked100 ? (!locale ? labelText : labelText.toLocaleString(locale)): labelText + "%";
                else if(customFormat.match('{value}') != null )
                    labelText = customFormat.replace('{value}', labelText);
                else if ((customFormat.indexOf('e') == 0 || customFormat.indexOf('E') == 0) && (customFormat.indexOf("ej.format") == -1)) {
                    labelPrecision = customFormat.match(/(\d+)/g);
                    labelPrecision = labelPrecision == null ? labelPrecisionDefault : labelPrecision > labelPrecisionHighest ? labelPrecisionHighest : labelPrecision;
                    labelText = labelText.toExponential(labelPrecision);
                }
                else {
                    if (customFormat.indexOf("ej.format") != -1)
                        customFormat = customFormat.substring(customFormat.indexOf("(") + 1, customFormat.indexOf(")"));
                    labelText = ej.format(labelText, customFormat, locale);
                }

                // customize label by event
                commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.data = { label: { Text: labelText, Value: tempInterval }, axis: currentAxis };
                chartObj._trigger("axesLabelRendering", commonAxesEventArgs); 
                currentAxis.visibleLabels.push({ Value: labelValue, Text: commonAxesEventArgs.data.label.Text });
       
            }
        }

        currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

    },

    _calculateRanges: function (chartObj, axis, seriesCollection, params) {
        this.chartObj = chartObj;

        this._calculateRange(chartObj, axis, seriesCollection, params);

        if (chartObj.model._drawTrendline)
            this._calculateTrendLineRange(chartObj, axis, seriesCollection);

        this._calculatePadding(chartObj, axis, axis.range);

        if(axis.maximumLabels > 0)  // to calculate only if labels have to be displayed
        this._calculateAxisLabels(chartObj, axis);

    }
});
ej.axisTypes.double = ejDoubleValue;
//log axis
var ejLogValue = ejExtendClass(ej.EjAxisRenderer, {
    
    // Get value for log min and max
   _getRoundValue:function (x,div,up)
    {
        return (up ? Math.ceil(x / div) : Math.floor(x / div)) * div;
   },
   
   // Caculate log nice interval
  calculateLogNiceInterval: function (delta, axis) {
       var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
       var niceInterval = delta;
       var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
       var intervalDivs = [10, 5, 2, 1];

       for (var i = 0; i < intervalDivs.length; i++) {
           var currentInterval = minInterval * intervalDivs[i];
           if (desiredIntervalsCount < (delta / currentInterval)) {
               return niceInterval;
           }

           niceInterval = currentInterval;
       }

       return niceInterval;
  },
   
    // Get log value for auto range
    _getLogAutoRange: function (axis, min, max, delta) {
        
        var logStart = ej.EjSvgRender.utils._logBase(min, axis.logBase);
        logStart = $.isNumeric(logStart) ? logStart : min;
        var logEnd = ej.EjSvgRender.utils._logBase(max, axis.logBase);
        logEnd = $.isNumeric(logEnd) ? logEnd : max;
     
        var mulS = this._getRoundValue(logStart, 1, false);
        var mulE = this._getRoundValue(logEnd, 1, true);
        if (mulS == 0 && mulE == 0) {
            mulS = ej.EjSvgRender.utils._logBase((max - max / 2), axis.logBase);
            mulE = ej.EjSvgRender.utils._logBase((max + max / 2), axis.logBase);
        }
        delta = mulE - mulS;
        var interval = this.calculateLogNiceInterval(delta, axis);

        axis.actual_Range = { min: mulS, max: mulE, interval: interval, Delta: delta };

        mulS = Math.pow(axis.logBase, Math.floor(logStart));
        mulE = Math.pow(axis.logBase, Math.ceil(logEnd));

        delta = mulE - mulS;        
      
        if (!axis.setRange) {        
          
            return axis.range = { min: mulS, max: mulE, interval: interval, Delta: delta };

        } else {
            if (ej.util.isNullOrUndefined(axis.range.interval))
                axis.range.interval = interval;
            if (ej.util.isNullOrUndefined(axis.range.max))
                axis.range.max = mulE;
            if (ej.util.isNullOrUndefined(axis.range.min))
                axis.range.min = mulS;

            axis.actual_Range = $.extend(true, {}, axis.range);

            if (!ej.util.isNullOrUndefined(axis.actual_Range.max)) {
                axis.actual_Range.max = ej.EjSvgRender.utils._logBase(axis.actual_Range.max, axis.logBase);
                axis.actual_Range.max = this._getRoundValue(axis.actual_Range.max, 1, true);
            }               
            if (!ej.util.isNullOrUndefined(axis.range.min)) {
                axis.actual_Range.min = ej.EjSvgRender.utils._logBase(axis.actual_Range.min, axis.logBase);
                axis.actual_Range.min = this._getRoundValue(axis.actual_Range.min, 1, false);
            }            
            
            axis.setRange = true;
          
            return axis.range;
        }
    },
   
      //Calcualte auto log range from series
        _calculateLogRange: function (chartObj, axis, seriesCollection, params) {
            if (!chartObj.zoomed) {
                this._seriesMinMax(chartObj,axis, seriesCollection, "logarithmic", params);
            }
        },
   
       //Assign base range values to actual range
        _calculateLogPadding: function (chartObj, axis, baseRange) {
            // No padding support for log axis
            var start = baseRange.min;
            var end = baseRange.max;
            var interval = baseRange.interval;
            axis.actualRange.min = start;
            axis.actualRange.max = end;
            axis.actualRange.interval = interval;         
            axis.actualRange.delta = end - start;
            this._calculateVisibleRange(axis);
        },
   
        // Calculate visible range for zooming
        _calculateVisibleRange: function (axis, sender, isScroll) {
            this.chartObj = sender ? sender : this.chartObj;
            axis.visibleRange = (!isScroll) ? $.extend(true, {}, axis.actualRange) : axis.visibleRange;
            if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType != "polaraxes" && !this.chartObj.model.disableZoom) {
			     axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
                axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                var interval = axis.enableAutoIntervalOnZooming ? (this.calculateLogNiceInterval(delta, axis)) : axis.visibleRange.interval;
                var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ?
                    (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1
                    : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
                interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
                if (isScroll)
                    return { min: Math.min(start, end), max: Math.max(start, end), interval: interval };
                else{
                    axis.visibleRange.interval = interval;
                    axis.visibleRange.min = Math.min(start, end);
                    axis.visibleRange.max = Math.max(start, end);
                    axis.visibleRange.delta = delta;
                    axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
                }
            }

        },
   
        //Method implementation for Generate Labels in ChartAxis
        _calculateAxisLabels: function (chartObj, currentAxis) {

            var tempInterval, round = currentAxis.roundingPlaces;
            if (chartObj.zoomed || currentAxis.zoomed)
                tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
            else
                tempInterval = currentAxis.visibleRange.min;
            for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {
                

                if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {      
                    var customFormat = (!(currentAxis.labelFormat)) ? null : currentAxis.labelFormat.match('{value}');
                    var tempIntervaltext = Math.pow(currentAxis.logBase, tempInterval);
                    tempPlace = (tempIntervaltext.toString().split(0).length - 1 > 10) ? 10 : 20;
                    tempIntervaltext = parseFloat(tempIntervaltext.toFixed((!round && round > tempPlace) ? round : tempPlace));
                     
                    var labelText = (!(currentAxis.labelFormat)) ? tempIntervaltext : (customFormat != null) ?
                             currentAxis.labelFormat.replace('{value}', tempIntervaltext) :
                              (ej.format(tempIntervaltext, currentAxis.labelFormat, chartObj.model.locale));
					 // customize label by event
                    var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonAxesEventArgs.data = { label: { Text: labelText, Value: tempInterval }, axis: currentAxis };
                    chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                    currentAxis.visibleLabels.push({ Value:tempInterval, Text: commonAxesEventArgs.data.label.Text });
                }
            }

            currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

        },


        //Category axis calculation
        _calculateRanges: function (chartObj, axis, seriesCollection, params) {
            this.chartObj = chartObj;
            this._calculateLogRange(chartObj,axis, seriesCollection, params);
            if (chartObj.model._drawTrendline)
                this._calculateTrendLineRange(chartObj, axis, seriesCollection);
            this._calculateLogPadding(chartObj, axis, axis.actual_Range);
            this._calculateAxisLabels(chartObj,axis);
        }

        
    });
    ej.axisTypes.logarithmic = ejLogValue;
    
//Category Axes calculation
    var ejCategoryValue = ejExtendClass(ej.EjAxisRenderer, {
        //axis range calculation and assign indexed value to x value of points
        _calculateCategoryRange: function (axis, seriesCollection) {
            var legendCollapsed = (this.chartObj.model.legendCollapsed == undefined) ? !this.chartObj.zoomed : this.chartObj.model.legendCollapsed;
            if (!this.chartObj.model.excludeDataUpdate || legendCollapsed) {
            var maxXValue = { maxX: null, minX: 0 },
                labels = [],
                pointX,
                intervalX,
                deltaX,
                index,
                isIndex = axis.isIndexed,
                locale = this.chartObj.model.locale;
            if (seriesCollection.length > 0 && this.chartObj.model._hasSeriesPoints && seriesCollection[0].points.length > 0  || this.chartObj.model.indicators[0]._points.length>0) {
                for (var m = 0; m < seriesCollection.length; m++) {
                    pointsLength = seriesCollection[m].points.length;
                    for (var n = 0; n < pointsLength; n++) {
                        if (seriesCollection[m]._xAxisValueType == "date")
                            pointX = ej.format(new Date(seriesCollection[m].points[n].x), (ej.util.isNullOrUndefined(axis.labelFormat) ? "dd/MM/yyyy" : axis.labelFormat), locale);
                        else
                            pointX = seriesCollection[m].points[n].x;
                        if (!isIndex) {
                            index = $.inArray(pointX, labels);
                            if (index < 0)
                                labels.push(pointX);
                        }
                        else {
                            if (m == 0)
                                labels[n] = pointX;
                            else
                                labels[n] = labels[n] + ", " + pointX;
                        }

                    }
                    if (!axis.maxPointLength || axis.maxPointLength < pointsLength)
                        axis.maxPointLength = pointsLength;
                }
                for (var l = 0; l < seriesCollection.length; l++) {
                    seriesCollection[l]._pointCollection = [];
                    for (var k = 0; k < seriesCollection[l].points.length; k++) {
                        seriesCollection[l]._pointCollection.push(seriesCollection[l].points[k].x);
                        if (seriesCollection[l]._xAxisValueType == "date")
                            pointX = ej.format(new Date(seriesCollection[l].points[k].x), (ej.util.isNullOrUndefined(axis.labelFormat) ? "dd/MM/yyyy" : axis.labelFormat), locale);
                        else
                            pointX = seriesCollection[l].points[k].x;
                        if (!isIndex)
                        seriesCollection[l].points[k].xValue = $.inArray(pointX, labels);
                        else
                            seriesCollection[l].points[k].xValue = k;
                       
                    }
                    seriesCollection[l].minX = 0;
                    seriesCollection[l].maxX = (seriesCollection[l]._xAxisValueType == "date") ? seriesCollection[l]._pointCollection.length - 1 : labels.length - 1;
                    // To find maximum x value of axis from series collection
                    if (l == 0) {
                        maxXValue.maxX = seriesCollection[l].maxX;
                        axis.labels = (seriesCollection[l]._xAxisValueType == "date") ? seriesCollection[l]._pointCollection : labels;
                        axis._categoryValueType = seriesCollection[l]._xAxisValueType;
                    }
                    if (maxXValue.maxX < (seriesCollection[l].maxX)) {
                        maxXValue.maxX = seriesCollection[l].maxX;
                        axis.labels = [];
                        axis.labels = labels;
                    }
                }
                if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                          var value = this.chartObj.model.indicatorRange[axis.name];
                          if (maxXValue.minX > value.min || maxXValue.minX == null)
                              maxXValue.minX = value.min;
                          if (maxXValue.maxX < value.max || maxXValue.maxX == null)
                              maxXValue.maxX = value.max;
                }
                if (maxXValue.minX == maxXValue.maxX && axis.labelPlacement && axis.labelPlacement.toLowerCase() == "onticks") {
                    maxXValue.maxX += 1;
                }
                deltaX = maxXValue.maxX - maxXValue.minX;
                
                if (axis.setRange && axis.range.interval) {
                    intervalX = Math.ceil(axis.range.interval);
                }
                else {
                    intervalX = this._calculateActualInterval(deltaX, axis);
                }

                axis.range = { min: maxXValue.minX, max: maxXValue.maxX, interval: intervalX, Delta: deltaX };
            }
            else {
                var labelLength = axis.labels.length;
                if (labelLength > 0) {
                    deltaX = labelLength - 1;
                    intervalX = this._calculateActualInterval(deltaX, axis); 
                    axis.range = { min: 0, max: labelLength - 1, interval: intervalX, Delta: deltaX };
                }
                else if (!axis.range) // set default range when no series points added to category axes
                    axis.range = { min: 0, max: 6, interval: 1, Delta: 6 };
            }       
            }

        },

    //Calculates actual interval
    _calculateActualInterval: function (delta, axis) {
        if (axis.categoryInterval == null)
            return Math.max(1, Math.floor(delta / this.GetActualDesiredIntervalsCount(axis, axis.length)));
        else
            return axis.categoryInterval;
    },

    //Apply padding based on labelPlacement
    _applyRangePadding: function (axis) {
        if (!this.chartObj.zoomed && !this.chartObj._scrollBarEnabled) {
            // ticks based on labelplacement and polaraxes
            var ticks = 0;
            if(this.chartObj.model.AreaType !== 'polaraxes')
            ticks = ((axis.labelPlacement === null || axis.labelPlacement === undefined || axis.labelPlacement === "")) ? -0.5 : (axis.labelPlacement.toLowerCase() == "betweenticks") ? (-0.5) : 0;
			 if (ticks < 0) {
                axis.range.min = axis.range.min + ticks;
                axis.range.max = axis.range.max - ticks;
                axis.range.delta = axis.range.max - axis.range.min;
                axis.actualRange = axis.range;
            } else {
                var data = _sideBySeriesPadding(this.chartObj, axis.range.min, axis.range.max, axis);
                axis.range.min = data.startPadding;
                axis.range.max = data.endPadding;
                axis.range.delta = axis.range.max - axis.range.min;
                axis.actualRange = axis.range;
			}
        }
        this._calculateVisibleRange(axis);
    },

    //Calculates the visible range  
    _calculateVisibleRange: function (axis, sender, isScroll) {
        this.chartObj = sender ? sender : this.chartObj;
        axis.visibleRange = (!isScroll) ? $.extend(true, {}, axis.actualRange) : axis.visibleRange;
        if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType != "polaraxes" && !this.chartObj.model.disableZoom) {
		     axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
            axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
            var baseRange = axis.actualRange;
            if (axis.isInversed) {
                var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + (axis.zoomPosition * axis.actualRange.delta);
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }
            var delta = Math.abs(end - start);
            var interval = this._calculateActualInterval(delta, axis);
            var min = Math.min(start, end);
            var max = Math.max(start, end);
            if (isScroll) {
                return {min:min, max:max, interval:interval};
            }
            else {
                var delta = Math.abs(end - start);
                axis.visibleRange.interval = interval;
                axis.visibleRange.min = min;
                axis.visibleRange.max = max;
                axis.visibleRange.delta = delta;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }
        }

    },

    //Generate category label for axis
    _calculateAxisLabels: function (axis) {
        var interval = axis.visibleRange.interval;
        var position = axis.visibleRange.min - (axis.visibleRange.min % axis.range.interval);
        for (; position <= axis.visibleRange.max; position += interval) {
            if (ej.EjSvgRender.utils._inside(position, axis.visibleRange)) {
                var pos = Math.round(position);
                axis.visibleLabels.push({ Value: pos, Text: ej.EjSvgRender.utils._getLabelContent(pos, axis, this.chartObj.model.locale) });

                // customize label by event
                var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.data = { label: axis.visibleLabels[axis.visibleLabels.length - 1], axis: axis };
                this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
            }
        }
        axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
    },

    //Category axis calculation
    _calculateRanges: function (chartObj, axis, seriesCollection) {
        this.chartObj = chartObj;

        this._calculateCategoryRange(axis, seriesCollection);
        if (chartObj.model._drawTrendline)
            this._calculateTrendLineRange(chartObj, axis, seriesCollection);
        this._applyRangePadding(axis);

        this._calculateAxisLabels(axis);
    }
});
ej.axisTypes.category = ejCategoryValue;

    // for Datetime category axis calculation

    var ejDateTimeCategory = ejExtendClass(ej.EjAxisRenderer, {
        _calculateDateTimeCategoryRange: function (axis, seriesCollection) {
            ///Variables declaration part here ///
            var xRange = { minX: 0, MaxX: null },xLabels = [],xValue=[], xPoint,index,index1, deltaX,intervalX,
                         prevDateVal, points, xVerfiedPoints = [], dateDouble, xValues= [],
                         xDateVal, xVal, previous, current, pointsLength, point, m, pt, ser;
            for (ser = 0; ser < seriesCollection.length; ser++) {
                seriesCollection[ser]._pointCollection = [], points = [],seriesCollection[ser]._points=[], seriesCollection[ser]._xPoints = [], seriesCollection[ser]._yPoints=[],
                pointsLength = seriesCollection[ser].points.length;
                for (pt = 0; pt < pointsLength; pt++) {
                    dateDouble = !isNaN(new Date(seriesCollection[ser].points[pt].x).getTime()) ? new Date(seriesCollection[ser].points[pt].x).getTime() : null;
                    points.push(dateDouble);
                    seriesCollection[ser]._xPoints.push(dateDouble);
                    seriesCollection[ser]._yPoints.push(seriesCollection[ser].points[pt].y);
					if (dateDouble != null)
                        xValues.push(dateDouble);
                }
                points.sort(function (a, b) { return a - b; }); // date time values sorted
                if(ej.isNullOrUndefined(axis.intervalType)){
                    intervalX = this.calculateDateTimeNiceInterval(axis, new Date(Math.min.apply(null, xValues)), new Date(Math.max.apply(null, xValues)));
                    axis.intervalType = intervalX.intervalType.toLowerCase();
                }
                for (point = 0; point < points.length; point++) {
                    xPoint = new Date(points[point]);
                    seriesCollection[ser]._points.push(xPoint.getTime());
                    var ySortPos = $.inArray(points[point], seriesCollection[ser]._xPoints);
                    seriesCollection[ser].points[point].y = seriesCollection[ser].points[point].YValues[0] = seriesCollection[ser]._yPoints[ySortPos];
                    seriesCollection[ser]._pointCollection.push(new Date(seriesCollection[ser].points[point].x));
                    if (ser == 0)
                        seriesCollection[ser].points[point].xValue = point;
                    else {
                        for (var i = 0; i <= ser - 1; i++) {
                            index1 = $.inArray(new Date(seriesCollection[ser].points[point].x).getTime(), seriesCollection[i]._points);
                            if (index1 >= 0) {
                                seriesCollection[ser].points[point].xValue = index1;
                            }
                            else if(i = ser) {
                                seriesCollection[ser].points[point].xValue = (axis.maxPointLength - 1) + (point + 1);
                                break;
                            }
                        }                           
                    }
                    index = (ser == 0)? -1 : $.inArray(xPoint.getTime(), xVerfiedPoints);
                    if (index < 0) {
                        switch (axis.intervalType) {
                            case "auto": // for auto calculate
                                xLabels.push(xPoint.getTime());
                                break;
                            case "years":  // for years calculate
                                xVal = xPoint.getFullYear();                              
                                break;
                            case "months":  // for months calculate
                                current = xPoint.getFullYear();
                                xDateVal = xPoint.getMonth();
                                if (point <= 0) xVal = point + 1;
                                else {
                                    previous = new Date(points[point - 1]).getFullYear();
                                    xVal += (current != previous) ? ((12 - prevDateVal) + (xDateVal)) : (xDateVal - prevDateVal);
                                }
                                prevDateVal = xDateVal;
                                break;
                            case "days":  // for days calculate
                                current = xPoint.getMonth();
                                xDateVal = xPoint.getDate();
                                if (point <= 0) xVal = point + 1;
                                else {
                                    previous = new Date(points[point - 1]).getMonth();
                                    xVal += (current != previous) ? (((new Date(xPoint.getFullYear(), previous, 0).getDate() - prevDateVal) + (xDateVal))) : (xDateVal - prevDateVal);
                                    if(current == previous)
                                    {
                                        var diff = this._diffDate(xPoint, new Date(points[point - 1]),1);
                                        var days = (new Date(xPoint.getFullYear(), previous, 0).getDate());
                                        xVal = (diff > days) ? (xValue[xValue.length - 1]) + 1 : xVal;
                                    }
                                }
                                prevDateVal = xDateVal;
                                break;
                            case "hours":  // for hours calculate
                                current = xPoint.getDate();
                                xDateVal = xPoint.getHours();
                                if (point <= 0) xVal = point + 1;
                                else {
                                    previous = new Date(points[point - 1]).getDate();
                                    xVal += (current != previous) ? ((24 - prevDateVal) + (xDateVal)) : (xDateVal - prevDateVal);
                                    if (current == previous) {
                                        var diff = this._diffDate(xPoint, new Date(points[point - 1]),24);
                                        xVal = (diff> 24) ? (xValue[xValue.length - 1]) + 1 : xVal;
                                    }
                                }
                                prevDateVal = xDateVal;
                                break;
                            case "minutes": // for minutes calculate
                                current = xPoint.getHours();
                                xDateVal = xPoint.getMinutes();
                                if (point <= 0) xVal = point + 1;
                                else {
                                    previous = new Date(points[point - 1]).getHours();
                                    xVal += (current != previous) ? ((60 - prevDateVal) + (xDateVal)) : (xDateVal - prevDateVal);
                                    if (current == previous) {
                                        var diff = this._diffDate(xPoint, new Date(points[point - 1]),(24 * 60));
                                        xVal = (diff > 60) ? (xValue[xValue.length - 1]) + 1 : xVal;
                                    }
                                }
                                prevDateVal = xDateVal;
                                break;
                            case "seconds": // for seconds calculate
                                current = xPoint.getMinutes();
                                xDateVal = xPoint.getSeconds();
                                if (point <= 0) xVal = point + 1;
                                else {
                                    previous = new Date(points[point - 1]).getMinutes();
                                    xVal += (current != previous) ? ((60 - prevDateVal) + (xDateVal)) : (xDateVal - prevDateVal);
                                    if (current == previous) {
                                        var diff = this._diffDate(xPoint, new Date(points[point - 1]),( 24 * 60 * 60));
                                        xVal = (diff > 60) ? (xValue[xValue.length - 1]) + 1 : xVal;
                                    }
                                }
                                prevDateVal = xDateVal;
                                break;
                            case "milliseconds": // for milliseconds calculate
                                current = xPoint.getSeconds();
                                xDateVal = xPoint.getMilliseconds();
                                if (point <= 0) xVal = point + 1;
                                else {
                                    previous = new Date(points[point - 1]).getSeconds();
                                    xVal += (current != previous) ? ((1000 - prevDateVal) + (xDateVal)) : (xDateVal - prevDateVal);
                                    if (current == previous) {
                                        var diff = this._diffDate(xPoint, new Date(points[point - 1]), (24 * 60 * 60 * 1000));
                                        xVal = (diff > 1000) ? (xValue[xValue.length - 1]) + 1 : xVal;
                                    }
                                }
                                prevDateVal = xDateVal;
                                break;
                        }
                        if (axis.intervalType != "auto")
                            xLabels.push(xPoint.getTime());  
                        xVal = (ser == 0) ? xVal : xValue[xValue.length - 1] + xVal;
                        xValue.push(xVal);
                        xVerfiedPoints.push(new Date(points[point]).getTime());                      
                    }
               }           
                   if (!axis.maxPointLength || axis.maxPointLength < pointsLength) axis.maxPointLength = pointsLength;
            }
            for (m = 0; m < seriesCollection.length; m++) { // set the min and max range for axis
                if (!axis.setRange) axis.range = { min: null, max: null, interval: null };
				if(!ej.isNullOrUndefined(axis.m_minPointsDelta)) axis.m_minPointsDelta = undefined;
                seriesCollection[m].minX = (ej.isNullOrUndefined(axis.range.min)) ? 0 : (axis.range.min < xRange.minX) ? 0 : axis.range.min;
                seriesCollection[m].maxX = (ej.isNullOrUndefined(axis.range.max)) ? xLabels.length - 1 : (axis.range.max > xLabels.length - 1) ? xLabels.length-1 : axis.range.max;
                    if (m == 0) {
                        xRange.MaxX = seriesCollection[m].maxX;
                        xRange.minX = seriesCollection[m].minX;
                        axis.labels = xLabels;
                        axis.dateLabels = xValue;
                        axis._categoryValueType = "date";
                    }
                }
            if (xRange.minX == xRange.MaxX && axis.labelPlacement.toLowerCase() == "onticks") xRange.MaxX += 1;
                deltaX = xRange.MaxX - xRange.minX;
                intervalX = ((axis.range.interval == null || axis.range.interval == undefined)) ? 1 : (axis.range.interval < 1) ? 1 : Math.ceil(axis.range.interval);               
                axis.range = { min: xRange.minX, max: xRange.MaxX, interval: intervalX, Delta: deltaX };
        },

        _diffDate:function(currentPoint,previousPoint,mul)
        {
            var oneDay, diff;
             oneDay = 24 * 60 * 60 * 1000;
            diff = (Math.abs((currentPoint.getTime() - previousPoint.getTime()))/ (oneDay));
            return (Math.round(diff*mul));
        },

        _calculateAxisLabels:function(axis) // calculate visible labels for axis
        {
            var interval = axis.visibleRange.interval, currentLabels = [], count = 0,
                           prevLabelVal, pos,arr, currentLabel, expectLabel, index, i, isCurrentLabel, isNextLabel, commonAxesEventArgs;
            var position = axis.visibleRange.min - (axis.visibleRange.min % axis.range.interval);
            for (; position <= axis.visibleRange.max; position += interval) {
                if (ej.EjSvgRender.utils._inside(position, axis.visibleRange)) {
                     pos = Math.round(position);
                     arr = axis.dateLabels;
                     if (axis.intervalType != "auto") {
                         if (count > 0 && interval > 1) {
                             if (pos > arr.length - 1)
                                 pos = arr.length - 1;
                             currentLabel = arr[count];
                             expectLabel = prevLabelVal + interval;
                             isCurrentLabel = $.inArray(currentLabel, arr);
                             isNextLabel = $.inArray(expectLabel, arr);
                             if (isCurrentLabel > 0 && (Math.abs(currentLabel - prevLabelVal) == interval) && currentLabel > prevLabelVal) {
                                 pos = isCurrentLabel;
                                 prevLabelVal = arr[pos];
                             }
                             else {
                                 for (i = 0; i < i + 1; i++) {
                                     expectLabel += (i == 0) ? i : 1;
                                     isNextLabel = $.inArray(expectLabel, arr);
                                     if (isNextLabel > 0) {
                                         pos = isNextLabel;
                                         prevLabelVal = arr[pos];
                                         break;
                                     }
                                 }
                             }
                         }
                     }
                    count++;
                    if(!ej.util.isNullOrUndefined(arr)){
                        if (((arr[arr.length - 1] - arr[pos]) >= interval) && (axis.visibleRange.max < (position + interval))) axis.visibleRange.max += ((position + interval) - axis.visibleRange.max);
                        index = $.inArray(arr[pos], currentLabels);
                     }
                    if((index < 0 || axis.intervalType == "auto") && (pos < axis.labels.length)) {
                        if (axis.intervalType != "auto") {
                            prevLabelVal = arr[pos];
                            currentLabels.push(prevLabelVal);
                        }                    
                        axis.visibleLabels.push({ Value: pos, Text: ej.EjSvgRender.utils._getLabelContent(pos, axis, this.chartObj.model.locale) });
                        commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                        commonAxesEventArgs.data = { label: axis.visibleLabels[axis.visibleLabels.length - 1], axis: axis };
                        this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                        axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;                    
                    }
                }
            }
            axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
        },

        //Calculates actual interval
        _calculateActualInterval: function (delta, axis) {
            if (axis.categoryInterval == null)
                return Math.max(1, Math.floor(delta / this.GetActualDesiredIntervalsCount(axis, axis.length)));
            else
                return axis.categoryInterval;
        },

        //Apply padding based on labelPlacement
        _applyRangePadding: function (axis) {
            if (!this.chartObj.zoomed && !this.chartObj._scrollBarEnabled) {
                // ticks based on labelplacement and polaraxes
                var ticks = 0;
                if (this.chartObj.model.AreaType !== 'polaraxes')
                    ticks = ((axis.labelPlacement === null || axis.labelPlacement === undefined)) ? -0.5 : (axis.labelPlacement.toLowerCase() == "betweenticks") ? (-0.5) : 0;
                if (ticks < 0) {
                    axis.range.min = axis.range.min + ticks;
                    axis.range.max = axis.range.max - ticks;
                    axis.range.delta = axis.range.max - axis.range.min;
                    axis.actualRange = axis.range;
                } else {
                    var data = _sideBySeriesPadding(this.chartObj, axis.range.min, axis.range.max, axis);
                    axis.range.min = data.startPadding;
                    axis.range.max = data.endPadding;
                    axis.range.delta = axis.range.max - axis.range.min;
                    axis.actualRange = axis.range;
                }
            }
            this._calculateVisibleRange(axis);
        },

        //Calculates the visible range  
        _calculateVisibleRange: function (axis, sender, isScroll) {
            this.chartObj = sender ? sender : this.chartObj;
            axis.visibleRange = (!isScroll) ? $.extend(true, {}, axis.actualRange) : axis.visibleRange;
            if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType != "polaraxes" && !this.chartObj.model.disableZoom) {
                axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
                axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + (axis.zoomPosition * axis.actualRange.delta);
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                var interval = this._calculateActualInterval(delta, axis);
                var min = Math.min(start, end);
                var max = Math.max(start, end);
                if (isScroll) {
                    return { min: min, max: max, interval: interval };
                }
                else {
                    var delta = Math.abs(end - start);
                    axis.visibleRange.interval = interval;
                    axis.visibleRange.min = min;
                    axis.visibleRange.max = max;
                    axis.visibleRange.delta = delta;
                    axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
                }
            }

        },

        _calculateRanges: function (chartObj, axis, seriesCollection) {
            this.chartObj = chartObj;
            this._calculateDateTimeCategoryRange(axis, seriesCollection);   
            this._applyRangePadding(axis);
            this._calculateAxisLabels(axis);          
        }
    });

    ej.axisTypes.datetimecategory = ejDateTimeCategory;

ej.EjAxisRenderer.prototype = {
    _drawGridLines: function (axisIndex, axis, params) {
        if (this.model.AreaType == "cartesianaxes") {
            if (axis.orientation.toLowerCase() == "horizontal")
                this._drawXAxisGridLine(axisIndex, axis, params);

            else
                this._drawYAxisGridLine(axisIndex, axis, params);
        }
        else {
            if (axis.orientation.toLowerCase() == "horizontal" && axisIndex == 0) {
                this._drawPolarGridLine(axis);
                this._drawPolarLabels(axis);

            }
            else if (axisIndex == 1)
                this._drawPolarCircle(axis);
        }
    },
    _drawAxes: function (axisIndex, axis) {

        if (this.model.AreaType == "cartesianaxes") {
            if (axis.visible && axis.orientation.toLowerCase() == "horizontal") {
                this._drawXAxisLabels(axisIndex, axis);
                this._drawXTitle(axisIndex, axis);
            }

            if (axis.visible && axis.orientation.toLowerCase() == "vertical") {
                this._drawYAxisLabels(axisIndex, axis);
                this._drawYTitle(axisIndex, axis);
            }
        }
    },
    _getSharpPath:function (width) {
        var value = ((width % 2) == 0) ? 0 : 0.5;
        return value;
    },
    _drawAxisLine: function (axis) {

        // Yaxis MajorGridlines, Ticklines and Labels.
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLine' });
        var sbYMajorTick = ej.EjSvgRender.utils._getStringBuilder();
        var x1 = this.model.centerX;
        var y1 = this.model.centerY;
        var x2 = this.model.centerX;
        var y2 = this.model.centerY - this.model.Radius;
        var axisLine = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2;
        if (axis.visible && axis.axisLine.visible) {
            var options = {
                'id': this.svgObject.id + '_YAxisLines',
                'fill': 'none',
                'stroke-width': axis.axisLine.width,
                'stroke': axis.axisLine.color,
                'opacity': axis.axisLine.opacity,
                'stroke-dasharray': axis.axisLine.dashArray,
                'd': axisLine
            };
            this.svgRenderer.drawPath(options, gEle);

            //Drawing Major Grid Lines 
            $(gEle).appendTo(this.gPolarAxisEle);
        }
       

        if (axis.visible && axis.majorTickLines.visible) {
            gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisMajorTicks' });
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    var label = axis.visibleLabels[j];
                    var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);
                    var line = {};
                    line.X1 = this.model.centerX;
                    line.Y1 = this.model.centerY - radius;
                    line.X2 = line.X1 - axis.majorTickLines.size;
                    line.Y2 = line.Y1;
                    sbYMajorTick.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
                }
               
                  yMajorTickDir = sbYMajorTick.toString();
                    var options = {
                        'id': this.svgObject.id + '_YAxisMajorTicks',
                        'fill': 'none',
                        'stroke-width': axis.majorTickLines.width,
                        'stroke': axis.majorTickLines.color,
                        'd': yMajorTickDir
                    };
            //Drawing Major Tickline Lines
                    this.svgRenderer.drawPath(options, gEle);
                    $(gEle).appendTo(this.gPolarAxisEle);
                
        }
        if (axis.visible) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' }),
                font = axis.font,
                bounds,
                regionX,
                labelsLength = axis.visibleLabels.length;
            for (var j = 0; j < labelsLength; j++) {
                var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[j].Value, this);
                var labelText = axis.visibleLabels[j].Text;

                var size = ej.EjSvgRender.utils._measureText(labelText, null, axis.font);

                var X = this.model.centerX - axis.majorTickLines.size - (this.model.elementSpacing / 3);
                var Y = this.model.centerY - radius + (size.height / 5);


                var options = {
                    'id': this.svgObject.id + '_' + axis.name + '_YLabel_' + j,
                    'x': X,
                    'y': Y,
                    'fill': font.color,
                    'font-size': font.size,
                    'font-family': font.fontFamily,
                    'font-style': font.fontStyle,
                    'font-weight': font.fontWeight,
                    'opacity': font.opacity,
                    'text-anchor': 'end'
                };

                this.svgRenderer.drawText(options, labelText, gEle);
                regionX = options.x - size.width;
                bounds = { x: regionX, y: options.y, width: size.width, height: size.height };
                axis.visibleLabels[j].region = { bounds: bounds, labelText: labelText };
            }
            //Drawing YAxis Labels
            $(gEle).appendTo(this.gPolarAxisEle);
        }
       

    },
    _drawPolarLabels: function (axis) {
	     if (!axis.visible)
			return 0;
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels'  }),
            aroundRadius = this.model.Radius + axis.majorTickLines.size,
            font = axis.font,
            length = axis.visibleLabels.length,
            firstLabelX,
            bounds,
            regionX,
            lastLabelX;
        for (var j = 0; j < length; j++) {
            var coef = ej.EjSvgRender.utils._valueToPolarCoefficient(axis, axis.visibleLabels[j].Value);

            var vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
		 if (!isNaN(vector.X) && !isNaN(vector.Y)) {
            var labelText = axis.visibleLabels[j].Text;
            var x = this.model.centerX + aroundRadius * vector.X;
            var y = this.model.centerY + aroundRadius * vector.Y;
            var textAnchor = (x < this.model.centerX) ? 'end' : ((x > this.model.centerX) ? 'start' : 'middle');

            var size= ej.EjSvgRender.utils._measureText(labelText, this.model.m_AreaBounds.Height, axis.font);
            // calculation for positioning the label along the arc
            if (coef == 0.25)
            {
                x -= this.model.elementSpacing / 2;
                y += (size.height) / 4;
            }
           else if (coef == 0.5)
            {
               y += (size.height) ;
            }
           else if (coef == 0.75)
           {
               x += this.model.elementSpacing / 2;
               y += (size.height) / 4;
           }
           else if (coef == 1 || coef==0) {
               y -= (size.height) / 2;
           }
               
           else if (0.25 < coef && coef < 0.5) {
               x -= this.model.elementSpacing / 2;
               y += (size.height) / 2;
           }
           else if (0.5 < coef && coef <= 0.75) {
               x += this.model.elementSpacing / 2;
               y += (size.height) / 2;
           }
           else if (0 < coef && coef < 0.25)
             {
                   x -= this.model.elementSpacing / 2;
           }
           else
               x += this.model.elementSpacing / 2;

            if (j == 0) {
                firstLabelX = x;
            }
            if (j == length - 1 && axis._valueType !="category" ) {
                lastLabelX = ej.EjSvgRender.utils._measureText(labelText, null, font).width;
                lastLabelX += x;
                if (lastLabelX > firstLabelX)
                    labelText = ""; //Hide the last overlapping labels
            }
            var options = {
                'id': this.svgObject.id + '_' + axis.name + '_XLabel_' + j,
                'x': x,
                'y': y,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': textAnchor
            };

            this.svgRenderer.drawText(options, labelText, gEle);
			 if (textAnchor == "middle") 
               regionX = options.x - size.width / 2;
           
            else if (textAnchor == "end") 
               regionX = options.x - size.width;
            else
                regionX = options.x;
				
            bounds = { x: regionX, y: options.y, width: size.width, height: size.height };
            axis.visibleLabels[j].region = { bounds: bounds, labelText: labelText };
          }
        }
		
        $(gEle).appendTo(this.gXaxisEle);

    },
    _drawPolarCircle: function (axis) {
        if (axis.majorGridLines.visible) {
            var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' });
            if (this.model.isPolar) {
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    var label = axis.visibleLabels[j];
                    var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);

                    var options = {
                        'id': this.svgObject.id + '_YAxisGridLines_' + j,
                        'cx': this.model.centerX,
                        'cy': this.model.centerY,
                        'r': radius,
                        'fill': 'transparent',
                        'stroke-width': axis.majorGridLines.width,
                        'stroke': axis.majorGridLines.color,
                         'opacity': (this.vmlRendering) ? 0.3: axis.majorGridLines.opacity
                    };

                    //Drawing Major Grid Lines for polar
                    this.svgRenderer.drawCircle(options, gEle);
                   
                }
            }
            else {
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    
                        var label = axis.visibleLabels[j];
                        var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);
                        // calculation for spider web
                        for (var i = 0; i < this.model._axes[0].visibleLabels.length; i++) {
                            var xAxis = this.model._axes[0];
                            var vector = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[i].Value);
						    if (!isNaN(vector.X) && !isNaN(vector.Y)){
                              var vector2;
                              if ((i + 1) < xAxis.visibleLabels.length) {
                                vector2 = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[i + 1].Value);
                              }
                              else {
                                vector2 = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[0].Value);
                              }
                              var connectPoint = { X: this.model.centerX + radius * vector.X, Y: this.model.centerY + radius * vector.Y };
                              var endPoint = { X: this.model.centerX + radius * vector2.X, Y: this.model.centerY + radius * vector2.Y };
                              sbYMajorGrid.append("M" + " " + connectPoint.X + " " + connectPoint.Y + " " + "L" + " " + endPoint.X + " " + endPoint.Y + " ");
						    }
                        }
                }
                if (axis.majorGridLines.visible) {
                    yMajorGridDir = sbYMajorGrid.toString();
                    var options = {
                        'id': this.svgObject.id + '_YAxisGridLines',
                        'fill': 'none',
                        'stroke-width': axis.majorGridLines.width,
                        'stroke': axis.majorGridLines.color,
                        'opacity': axis.majorGridLines.opacity,
                        'stroke-dasharray': axis.majorGridLines.dashArray,
                        'd': yMajorGridDir
                    };
                    //Drawing Major Grid Lines for radar
                    this.svgRenderer.drawPath(options, gEle);
                }
            }
        }
        $(gEle).appendTo(this.gYaxisEle);

    },
    _drawPolarGridLine: function (axis) {

        var chartModel = this.model,
            legend = chartModel.legend,
            legendTitleHeight = ej.EjSvgRender.utils._measureText(legend.title.text, null, legend.title.font).height,
            legendPosition = legend.position.toLowerCase(),
            legXSpace = 0,
            legYSpace = 0,
            sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder(),
            sbXMajorTick = ej.EjSvgRender.utils._getStringBuilder(),
            borderSize = legend.border.width,
            yOffset, actualWidth, actualHeight,
            isRadar = false, vector, line,
            length = axis.visibleLabels.length,
            chartBorder = chartModel.border.width,
            title=chartModel.title,
            subTitle=chartModel.title.subTitle,
            titleEnable=title.enableTrim && (title.textOverflow == 'wrap' || title.textOverflow == 'wrapandtrim')? true:false,
            subTitleEnable=subTitle.text != "" && subTitle.visible && subTitle.enableTrim && (subTitle.textOverflow == 'wrap' || subTitle.textOverflow == 'wrapandtrim')?true:false; 
        if (legend.visible && legendPosition != "custom") {
            if (legendPosition == "right" || legendPosition == "left")
                legXSpace = ((legendPosition == "right") ? chartModel.margin.right : chartModel.margin.left) + chartModel.LegendActualBounds.Width + (borderSize + chartBorder) * 2;
            else
                legYSpace = ((legendPosition == "top") ? chartModel.margin.top : chartModel.margin.bottom) + chartModel.LegendActualBounds.Height + legendTitleHeight + (borderSize + chartBorder) * 2;

        }
        // calculating the radius of the chart with avaible size
        yOffset = ((chartModel.title.text && chartModel.title.visible) ? chartModel._titleLocation.Y + (titleEnable ? chartModel._titleLocation.size.height : 0) + (subTitleEnable ? chartModel._subTitleLocation.size.height : 0) : 0) + legYSpace;

        actualWidth = chartModel.svgWidth - legXSpace;
        actualHeight = chartModel.svgHeight - yOffset;

        chartModel.centerX = actualWidth * 0.5 + ((legendPosition === "left") ? legXSpace : 0);
        chartModel.centerY = actualHeight * 0.5 + ((legendPosition === "top") ? yOffset : ((title.text && title.visible) ? (chartModel._titleLocation.Y + (titleEnable ? chartModel._titleLocation.size.height : 0) +
            (subTitleEnable ? chartModel._subTitleLocation.size.height : 0)) : 0));

        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' });
        chartModel.finalSize = { width: actualWidth, height: actualHeight };

        chartModel.Radius = Math.min(chartModel.finalSize.width, chartModel.finalSize.height) / 2 - (2 * chartModel.elementSpacing) - axis.majorTickLines.size - axis._LableMaxWidth.maxHeight;
       
        var pos = 0;

        if (axis.majorGridLines.visible) {

            for (var j = 0; j < length; j++) {

                vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
                line = {};
                line.X1 = chartModel.centerX;

                line.Y1 = chartModel.centerY;
                line.X2 = chartModel.centerX + chartModel.Radius * vector.X;
                line.Y2 = chartModel.centerY + chartModel.Radius * vector.Y;
                sbXMajorGrid.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
            }

        }
        if (axis.visible && axis.majorTickLines.visible) {
            var radius = chartModel.Radius;
            for (var j = 0; j < length; j++) {
                vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
                line = {};
                line.X1 = chartModel.centerX + radius * vector.X;
                line.Y1 = chartModel.centerY + radius * vector.Y;
                line.X2 = line.X1 + axis.majorTickLines.size * vector.X;
                line.Y2 = line.Y1 + axis.majorTickLines.size * vector.Y;
                var direction = "M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ";
                if (direction.indexOf("NaN") == -1)
                    sbXMajorTick.append(direction);
            }
        }
    

   
        if (axis.majorGridLines.visible) {
            xMajorGridDir = sbXMajorGrid.toString();
            var options = {
                'id': this.svgObject.id + '_XAxisGridLines',
                'fill': 'none',
                'stroke-width': axis.majorGridLines.width,
                'stroke': axis.majorGridLines.color,
                'opacity': axis.majorGridLines.opacity,
                'stroke-dasharray': axis.majorGridLines.dashArray,
                'd': xMajorGridDir
            };
            //Drawing XAxis Major Grid Lines
			if (xMajorGridDir.indexOf("NaN")== -1) {
              this.svgRenderer.drawPath(options, gEle);
              $(gEle).appendTo(this.gXaxisEle);
			}
        }

          if (axis.visible && axis.majorTickLines.visible) {
                xMajorTickDir = sbXMajorTick.toString();
                var options = {
                    'id': this.svgObject.id + '_XAxisMajorTicks',
                    'fill': 'none',
                    'stroke-width': axis.majorTickLines.width,
                    'stroke': axis.majorTickLines.color,
                    'd': xMajorTickDir
                };
                //Drawing XAxis Major Ticks Lines
                this.svgRenderer.drawPath(options, gEle);
                $(gEle).appendTo(this.gXaxisEle);
            }       
    },
    _drawXAxisTickLine:function(axisIndex,xAxis,gEle,params)
    {
        var xMajorTicksDir, xMinorTicksDir;
        var sbXMinorTicks = ej.EjSvgRender.utils._getStringBuilder();       
        var sbXMajorTicks = ej.EjSvgRender.utils._getStringBuilder();       
        var minorPointX;
        var x1 = Math.floor(xAxis.x);
        var x2 = Math.floor(xAxis.x + xAxis.width);
        var opposedPosition = xAxis._opposed;
        var labelValue, xPointValue = [];
        var isScroll = xAxis._isScroll && !(xAxis.scrollbarSettings.pointsLength != null && xAxis.scrollbarSettings.pointsLength < xAxis.maxPointLength);
        var labelBorder = xAxis.labelBorder;
        isCrossesAt= params.axes[xAxis.name]._validCross;
        if (axisIndex != 0 && !isCrossesAt && ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') || (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside')) && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && !isCrossesAt && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            var y1 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
            var y2 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
        }

        else
        {
            var y1 = y2 = (xAxis._y)?(!xAxis.showNextToAxisLine) ? xAxis._y : xAxis.y:Math.floor( xAxis.y + xAxis.majorTickLines.size - 5 ) ;
        }
        if (!gEle)
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTicklines' + '_' + axisIndex });
        xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : xAxis.labelPlacement;
        var ticksbwtLabel = ((xAxis.labels.length > 0) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = xAxis.visibleLabels.length;
        //For between ticks and gridlines
        if ((xAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = collectionLength > 0 ? xAxis.visibleLabels.length + 1 : collectionLength;
        }
        for (var i = 0; i < collectionLength; i++) {

            labelValue = (!xAxis.visibleLabels[i]) ? xAxis.visibleLabels[i - 1].Value + xAxis.range.interval : xAxis.visibleLabels[i].Value;
 
            var tempInterval = ((xAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;

            tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : tempInterval;

            if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                    continue;
                if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                    var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                    var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                    var yTickPosition;
                    if (xAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = xAxis.visibleRange.max;
                        var logmin = xAxis.visibleRange.min;

                        var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                        var logtickend = Math.pow(xAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                        for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                                //Calculate Minor Ticks
                                if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                    var xTickPosition = Math.floor(minorPointX + x1);
                                    if ((xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition != 'inside') || (xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition == 'inside'))
                                        yTickPosition = (opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                    else
                                        yTickPosition = (opposedPosition) ? Math.floor(y1 + xAxis.minorTickLines.size) : y1 - Math.floor(xAxis.minorTickLines.size);
                                    sbXMinorTicks.append("M" + " " + (xTickPosition + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (xTickPosition + ticksVal) + " " + (y1 + ticksVal) + " ");
                                }
                            }
                        }
                    }
                    else {
                        for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                            minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                            if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));

                                //Calculate Minor Ticks
                                if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                    if ((xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition != 'inside') || (xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition == 'inside'))
                                        yTickPosition = (opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                    else
                                        yTickPosition = (opposedPosition) ? Math.floor(y1 + xAxis.minorTickLines.size) : Math.floor(y1 - xAxis.minorTickLines.size);
                                    var points = Math.floor(minorPointX + x1);
                                    sbXMinorTicks.append("M" + " " + (points + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (points + ticksVal) + " " + (y1 + ticksVal) + " ");
                                }
                            }
                        }
                    }

                }

                //calculate major ticks
                if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible || labelBorder.width > 0) {
                    var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                    var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                    //Calculate Major Ticks and Grid lines
                    var pointX = xPointValue[i] = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                    //Calculate Major Ticks
                    if (xAxis.majorTickLines.visible) {
                     
                        var lineStart = xAxis.majorTickLines.size + xAxis.axisLine.width / 2;
                        if ((xAxis.tickLinesPosition == 'inside' && xAxis.labelPosition == 'inside') || (xAxis.tickLinesPosition == 'inside' && xAxis.labelPosition != 'inside'))
                            var yPosition = (opposedPosition) ? y1 + lineStart : y1 - lineStart;
                        else
                            var yPosition = ((opposedPosition) ? y1 - lineStart : lineStart + y1) - ((isScroll) ? opposedPosition ? this.model.scrollerSize : -this.model.scrollerSize : 0);
                        sbXMajorTicks.append("M" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(yPosition) + mtVal) + " " + "L" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(y1 + ((opposedPosition) ? (-xAxis.axisLine.width / 2) : (xAxis.axisLine.width / 2))) + mtVal) + " ");
                    }
                }
            }
        }
        // draw xAxis label border
        if (labelBorder.width > 0) {
            var xLabelGrid = ej.EjSvgRender.utils._getStringBuilder(), scrollSize = isScroll ? this.model.scrollerSize: 0,
            lineStart = xAxis.axisLine.width, yPosition, i, gap, labelPosition = xAxis.labelPosition.toLowerCase(), boundsX = this.model.m_AreaBounds.X,
			boundsWidth = this.model.m_AreaBounds.Width, borderSize = labelBorder.width, length = xAxis._LableMaxWidth.height + xAxis.axisLine.width / 2 + 2,
			collectionLength = xAxis.visibleLabels.length;
			
			if (xAxis.tickLinesPosition == labelPosition) length += xAxis.majorTickLines.size;	
            if (labelPosition == 'inside'){
                yPosition = (opposedPosition) ? y1 + lineStart : y1 + lineStart;
				endY = opposedPosition ? y1 + length : y1 - length; 
			}
            else {
                yPosition = (opposedPosition) ? y1 - lineStart : lineStart + y1 + ((xAxis.zoomFactor < 1 && this.model.zooming.enableScrollbar
                    && axisIndex == 0) ? this.model.scrollerSize : 0);
				endY = opposedPosition ? y1 - length - scrollSize : y1 + length + scrollSize;
			}
					
            for (i = 0; i < collectionLength; i++) {
              gap = (xPointValue[xPointValue.length - 1] - xPointValue[xPointValue.length - 2]) / 2;
              if (!ej.util.isNullOrUndefined(xPointValue[i]) && xPointValue.length >= 1 && !isNaN(gap)) {
                if (xAxis._valueType == "category" && xAxis.labelPlacement.toLowerCase() == "betweenticks") {
                    startX = xPointValue[i] + x1;
                    endX = xPointValue[i] + (gap * 2) + x1;
                } else {
                    startX = xPointValue[i] - gap + x1;
                    endX = xPointValue[i] + gap + x1;    
                }
                endX = Math.floor(endX);
                endY = Math.floor(endY);
                startX = Math.floor(startX);
                yPosition = Math.floor(yPosition);
                if (startX < xAxis.x)
                    xLabelGrid.append("M" + " " + Math.floor(x1) + " " + endY + " " +
                      "L" + " " + endX + " " + endY + " ");
                else if (endX > xAxis.width + xAxis.x){
                    xLabelGrid.append("M" + " " + startX + " " + yPosition + " " + "L" + " " + startX + " " + endY + " " +
                      "L" + " " + Math.floor(xAxis.width + x1) + " " + endY + " ");
					  if (endX - borderSize  <= xAxis.width + xAxis.x)
						   xLabelGrid.append("M " + Math.floor(xAxis.width + x1) + " " + endY + " L " + (xAxis.x + xAxis.width) + " " + yPosition);
				}
                else {
                    if (i == 0)
                        xLabelGrid.append("M " + xAxis.x + " " + endY + " L " + startX + " " + endY + " ");
                    xLabelGrid.append("M" + " " + startX + " " + yPosition + " " + "L" + " " + startX + " " + endY + " " +
                       "L" + " " + endX + " " + endY + " ");
                    if (i == collectionLength - 1)
                        xLabelGrid.append("M" + " " + endX + " " + yPosition + " " + "L" + " " + endX + " " + endY + " " +
                            "M " + endX + " " + endY + " L " + (xAxis.x + xAxis.width) + " " + endY);
                }
              }
            }
            var labelGridOptions = {
                'id': this.svgObject.id + '_XAxisLabelBorder_' + axisIndex,
                'fill': 'transparent',
                'stroke-width': labelBorder.width,
                'stroke': labelBorder.color,
                'd': xLabelGrid.toString()
            };
            this.svgRenderer.drawPath(labelGridOptions, gEle);
            $(gEle).appendTo(this.gXaxisEle);
        }

        //draw minor ticks
        if (xAxis.minorTickLines.visible && xAxis.visible && xAxis.minorTicksPerInterval > 0) {
            xMinorTicksDir = sbXMinorTicks.toString();
            var optionsMinorTicks = {
                'id': this.svgObject.id + '_XAxisMinorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.minorTickLines.width,
                'stroke': xAxis.minorTickLines.color,
                'd': xMinorTicksDir
            };
            if(!gEle)
                this.svgRenderer.drawPath(optionsMinorTicks, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMinorTicks, gEle);

        }
        if (xAxis.majorTickLines.visible && xAxis.visible) {
            xMajorTicksDir = sbXMajorTicks.toString();
            var optionsMajorTicks = {
                'id': this.svgObject.id + '_XAxisMajorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.majorTickLines.width,
                'stroke': xAxis.majorTickLines.color,
                'd': xMajorTicksDir
            };
            if (!gEle) {
                
                this.svgRenderer.drawPath(optionsMajorTicks, gTickEle);
            }
            else
                this.svgRenderer.drawPath(optionsMajorTicks, gEle);
        }
        if(xAxis.tickLinesPosition=='inside')
            $(gTickEle).appendTo(this.gXaxisEle);

    },
    _drawXAxisGridLine: function (axisIndex, xAxis, params) {
        var xMajorGridDir, xMinorGridDir,
			sbXMinorGrid = ej.EjSvgRender.utils._getStringBuilder(),        
			sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder(),
			minorPointX,
			xPoint = [], padding = this.model.elementSpacing + 5,
			labelValue,
			opposedPosition = xAxis._opposed,
			x1 = Math.floor(xAxis.x),
			x2 = Math.floor(xAxis.x + xAxis.width),
			gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' + '_' + axisIndex }),
			gAgb = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisAlternateGridBand' + '_' + axisIndex });
        var isCrossesAt = params.axes[xAxis.name]._validCross;
        if (axisIndex != 0 &&!isCrossesAt && (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = y2 = (!opposedPosition) ? (xAxis.showNextToAxisLine) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) - padding : Math.floor(xAxis.y + xAxis._LableMaxWidth.height)  : Math.floor(xAxis.y - xAxis._LableMaxWidth.height) + padding;
        }
        else if (axisIndex != 0 &&!isCrossesAt && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            var y1 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
            var y2 = (!opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
        }
        else
        {
            var y1 = Math.floor(xAxis.y);
            var y2 = Math.floor(xAxis.y);
        }
       
        if (xAxis.visible && xAxis.axisLine.visible) {
            var offset = xAxis.axisLine.offset > 0 ? xAxis.axisLine.offset : 0;
            var val=this._getSharpPath(xAxis.axisLine.width);
            var optionsLine = {
                'id': this.svgObject.id + '_XAxisLine_' + axisIndex,
                x1: (x1 - xAxis.plotOffset)+ val + offset ,
                y1: y1+val,
                x2: (x2 + xAxis.plotOffset) -offset+val,
                y2: y2+val,
                'stroke-dasharray': xAxis.axisLine.dashArray,
                'stroke-width': xAxis.axisLine.width,
                'stroke': xAxis.axisLine.color,
                'opacity': xAxis.axisLine.opacity || 1
            };

            params.axes[xAxis.name]._lineOption = [optionsLine, this.gXaxisEle];
        }
		else
			params.axes[xAxis.name]._lineOption = [null, this.gXaxisEle];
        xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : xAxis.labelPlacement;
        var ticksbwtLabel = ((xAxis.labels.length > 0) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = xAxis.visibleLabels.length;
		//For between ticks and gridlines
        if ((xAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = collectionLength > 0 ? xAxis.visibleLabels.length + 1 : collectionLength;
        }
        for (var i = 0; i < collectionLength; i++) {
			 labelValue = (!xAxis.visibleLabels[i]) ? xAxis.visibleLabels[i - 1].Value + xAxis.range.interval : xAxis.visibleLabels[i].Value;

            var tempInterval = ((xAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;
            
            tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : tempInterval;

            if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                    continue;
            
                //Calculate Minor Ticks and Grid lines
                if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                    var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                    var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                    var yTickPosition;
                    if (xAxis._valueType == "logarithmic"){
                    minorTicks = tempInterval;
                    var logmax = xAxis.visibleRange.max;
                    var logmin = xAxis.visibleRange.min;

                    var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                    var logtickend = Math.pow(xAxis.logBase, minorTicks);
                    var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                    var logtickPos = logtickstart + logtickInterval;
                    minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                    for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                        if (minorTicks < logmax && minorTicks > logmin) {

                            minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                           

                            //Calculate Minor Gridlines
                            if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible) {
                                var tickPosition = Math.floor(minorPointX + x1) ;
                                sbXMinorGrid.append("M" + " " + (tickPosition + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (tickPosition + linesVal) + " " + ( Math.floor(opposedPosition ?  this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height: this.model.m_AreaBounds.Y) + linesVal) + " ");
                            }

                        }
                        logtickPos += logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                    }
                    } else {
                        for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                            minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                            if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));


                                //Calculate Minor Gridlines
                                if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible)
                                    sbXMinorGrid.append("M" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + ( Math.floor(opposedPosition ?  this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height: this.model.m_AreaBounds.Y) + linesVal) + " ");
                            }
                        }
                    }
            }

                if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                    var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                    var pointX = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                    var mX1 = (Math.floor(pointX + x1));
                    if (xAxis.zoomed)
                        xPoint[0] = xAxis.x;
                    if (i == 0) {
                        if (mX1 == Math.floor(this.model.m_AreaBounds.X)) {
                            xPoint.push(mX1 + mlVal);
                            continue;
                            
                        }
                    }
                    if ((i == (collectionLength - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                         if (mX1 == Math.floor(this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width))
                             continue;
                         if (xAxis.zoomed)
                             xPoint[collectionLength] = mX1 + mlVal;
                       
                    }
                     
                    //Calculate Major Gridlines
                    if (xAxis.majorGridLines.visible)
                        sbXMajorGrid.append("M" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + mlVal) + " " + "L" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + mlVal) + " ");
                    if ((xAxis._valueType.toLowerCase() != "category") && i == collectionLength - 1)
                        continue;
                    xPoint.push((mX1 + mlVal));
                }

            }

        }

       
        if (xAxis.alternateGridBand.odd.fill != "transparent" || xAxis.alternateGridBand.even.fill != "transparent") {
            var height = this.model.m_AreaBounds.Height, optionsAlternateeven;

            if(xPoint.length === 0){
                optionsAlternateeven = {
                    'id': this.svgObject.id + '_XAxis' + axisIndex + '_Alternateeven' + 0,
                    'x': xAxis.x,
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': xAxis.width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.even.fill,
                    'opacity': xAxis.alternateGridBand.even.opacity,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'stroke': "transparent"
                };
                this.svgRenderer.drawRect(optionsAlternateeven, gAgb);
            }
            
            for (var i = 1; i < xPoint.length; i += 2) {
                var width = xPoint[i + 1] - xPoint[i];
                width = (width || xAxis.labelPlacement.toLowerCase() == 'onticks') ? width : (xAxis.width+xAxis.x) - xPoint[i];
                if (i == xPoint.length - 1 && xAxis.zoomed)
                    width = this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - xPoint[xPoint.length - 1] - xAxis.plotOffset;
                var optionsAlternateodd = {
                    'id': this.svgObject.id + '_XAxis' + axisIndex + '_Alternateodd' + [i],
                    'x': xPoint[i],
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.odd.fill,
                    'opacity': xAxis.alternateGridBand.odd.opacity,
                    'stroke': "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'

                };
                if(width > 0)
                	this.svgRenderer.drawRect(optionsAlternateodd, gAgb);
            }

            for (var i = 0; i < xPoint.length; i += 2) {
                var width = xPoint[i + 1] - xPoint[i];
                width = (width || xAxis.labelPlacement.toLowerCase() == 'onticks') ? width : (xAxis.width + xAxis.x) - xPoint[i];
                if (i == 0 && xAxis.zoomed)
                    width = xPoint[1] - xAxis.x;
                
                if (i == xPoint.length - 1 && xAxis.zoomed)
                    width = this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - xPoint[xPoint.length - 1]-xAxis.plotOffset;
                    
                optionsAlternateeven = {
                    'id': this.svgObject.id + '_XAxis' + axisIndex + '_Alternateeven' + [i],
                    'x': xPoint[i],
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.even.fill,
                    'opacity': xAxis.alternateGridBand.even.opacity,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'stroke': "transparent"
                };
                if(width > 0)
                    this.svgRenderer.drawRect(optionsAlternateeven, gAgb);
            }
            $(gAgb).appendTo(gEle);
        }
        if (xAxis.minorGridLines.visible && xAxis.minorTicksPerInterval > 0) {
            xMinorGridDir = sbXMinorGrid.toString();
            //Drawing Minor Grid Lines
            var optionsMinorGrid = {
                'id': this.svgObject.id + '_XAxisMinorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.minorGridLines.width,
                'stroke': xAxis.minorGridLines.color,
                'stroke-dasharray': xAxis.minorGridLines.dashArray,
                'd': xMinorGridDir
            };
            this.svgRenderer.drawPath(optionsMinorGrid, gEle);
        }       
        xMajorGridDir = sbXMajorGrid.toString();
        if (xAxis.majorGridLines.visible) {
            var options = {
                'id': this.svgObject.id + '_XAxisMajorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.majorGridLines.width,
                'stroke': xAxis.majorGridLines.color,
                'opacity': xAxis.majorGridLines.opacity,
                'stroke-dasharray': xAxis.majorGridLines.dashArray,
                'd': xMajorGridDir
            };

            //Drawing Major Grid Lines
            if(options.d != '')
            this.svgRenderer.drawPath(options, gEle);

        }
        $(gEle).appendTo(this.svgObject);
         
        
    },

    _drawYAxisTickLine:function(axisIndex,yAxis,gEle,isValidCross)
    {
        var yMajorTicksDir = "", yMinorTicksDir = "";
        var pointY = 0;
        var isScroll = yAxis._isScroll && !(yAxis.scrollbarSettings.pointsLength != null && yAxis.scrollbarSettings.pointsLength < yAxis.maxPointLength);
        var opposedPosition = yAxis._opposed;
        var labelValue, yPointValue = [];
		var labelBorder = yAxis.labelBorder;
      
        if (axisIndex != 1  && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')) && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!opposedPosition && !isValidCross) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : yAxis.showNextToAxisLine ? yAxis.x :Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
            var x2 = (!opposedPosition && !isValidCross ) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) :  yAxis.showNextToAxisLine ? yAxis.x : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
        else if (axisIndex != 1 && !isValidCross && (yAxis.labelPosition != 'inside' && yAxis.tickLinesPosition == 'inside') && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = Math.floor(yAxis.x+yAxis.majorTickLines.size);
            var x2 = Math.floor(yAxis.x+yAxis.majorTickLines.size);
        }
        else            
        {
            var x1 = x2 = (!yAxis.showNextToAxisLine) ? yAxis._x : !yAxis.showNextToAxisLine && yAxis.tickLinesPosition == "inside" ? yAxis.x + yAxis.majorTickLines.size : yAxis.x;
        }   

        var y1 = Math.floor(yAxis.y + yAxis.height);
        var y2 = Math.floor(yAxis.y);       
    

        var sbYMinorTicks = ej.EjSvgRender.utils._getStringBuilder();   
        var sbYMajorTicks = ej.EjSvgRender.utils._getStringBuilder();      
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTicklines' + '_' + axisIndex });
      
		 yAxis.labelPlacement = (!(yAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : yAxis.labelPlacement;
        var ticksbwtLabel = ((yAxis.labels.length > 0) && (yAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = yAxis.visibleLabels.length;
        //For between ticks and gridlines
        if ((yAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = collectionLength > 0 ? yAxis.visibleLabels.length + 1 : collectionLength;
        }
        for (var i = 0; i < collectionLength; i++)
        {
            labelValue = (!yAxis.visibleLabels[i]) ? yAxis.visibleLabels[i - 1].Value + yAxis.range.interval : yAxis.visibleLabels[i].Value;
            var tempInterval = ((yAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;
            tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : tempInterval;

            if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange))
            {
 
                if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                    continue;

                // Calculate Minor Ticks and Grid lines      
                if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                    var minorPointY;
                    var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                    if (yAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = yAxis.visibleRange.max;
                        var logmin = yAxis.visibleRange.min;

                        var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                        var logtickend = Math.pow(yAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                                //Calculate Minor Ticks
                                if ((yAxis.minorTickLines) && yAxis.minorTickLines.visible) {

                                    if (yAxis.minorTickLines.visible) {
                                        if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                                            ytickPosition = (opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                        else
                                            var ytickPosition = (opposedPosition) ? -yAxis.minorTickLines.size : yAxis.minorTickLines.size;
                                        sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                    }
                                }                              
                            }
                            logtickPos += logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        }
                    }
                    else {
                        for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                            minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                            if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                                // Calculate Minor Ticks   
                                if (yAxis.minorTickLines.visible) {
                                    if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                                        var ytickPosition = (opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                    else
                                        var ytickPosition = (opposedPosition) ? -yAxis.minorTickLines.size : yAxis.minorTickLines.size;
                                    sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                }
                               
                            }
                        }
                    }
                }
                // Calculate Major Ticks and Grid lines
                if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible || labelBorder.width > 0) {
                    var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
					pointY = yPointValue[i] = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                    // Calculate Major Ticks
                    if (yAxis.majorTickLines.visible)
                    {
                        if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                            var xPosition = ((opposedPosition) ? yAxis.majorTickLines.size + yAxis.axisLine.width / 2 : -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2)) + ((isScroll) ? opposedPosition ? this.model.scrollerSize : opposedPosition - this.model.scrollerSize : 0);
                        else if ((yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition == 'inside'))
                            var xPosition = (opposedPosition) ? -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2) : (yAxis.majorTickLines.size + yAxis.axisLine.width / 2);

                        sbYMajorTicks.append("M" + " " + (Math.floor(xPosition + x1) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " " + "L" + " " + (Math.floor(x1 + ((opposedPosition) ? (yAxis.axisLine.width / 2) : (yAxis.axisLine.width / 2))) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " ");

                    }


                   
                }
            }
        }

        // draw yAxis label border
        if (labelBorder.width > 0) {
            var yLabelGrid = ej.EjSvgRender.utils._getStringBuilder(), xPosition = yAxis.axisLine.width / 2, i, 
			startY, endY, endX, gap, scrollSize = isScroll ? this.model.scrollerSize: 0, length = yAxis._LableMaxWidth.maxWidth + 10 ;
                
				if (yAxis.tickLinesPosition == yAxis.labelPosition) length += yAxis.majorTickLines.size;
				endX = Math.floor(x1 - length);
				if (yAxis.labelPosition == "outside")
                        endX = opposedPosition ? Math.floor(x1 + length) + scrollSize : Math.floor(x1 - length) - scrollSize;
                    else
                        endX = opposedPosition ? Math.floor(x1 - length) : Math.floor(x1 + length);
						
                for (var i = 0; i < collectionLength; i++) {
                  gap = (yPointValue[yPointValue.length - 1] - yPointValue[yPointValue.length - 2]) / 2;
                  if (!ej.util.isNullOrUndefined(yPointValue[i]) && yPointValue.length > 1 && !isNaN(gap)) { 
					if (yAxis._valueType == "category" && yAxis.labelPlacement.toLowerCase() == "betweenticks") {
						startY = Math.floor((yPointValue[i] * -1) + y1);
						endY = Math.floor((yPointValue[i] * -1) - (gap*2) + y1);
					} else{
						startY = Math.floor((yPointValue[i] * -1) + gap + y1);
						endY = Math.floor((yPointValue[i] * -1) - gap + y1);
					}
                    if (startY > y1)
                        yLabelGrid.append("M" + " " + endX + " " + y1 + " " + "L" + " " + endX + " " + endY + " ");
                    else if (endY < this.model.m_AreaBounds.Y)
                        yLabelGrid.append("M" + " " + Math.floor(xPosition + x1) + " " + startY + " " + "L" + " " + endX + " " + startY + " "
                         + "L" + " " + endX + " " + this.model.m_AreaBounds.Y + " ");
                    else {
                        yLabelGrid.append("M" + " " + Math.floor(xPosition + x1) + " " + startY + " " + "L" + " " + endX + " " + startY + " "
                        + "L" + " " + endX + " " + endY + " ");
                        if (i == collectionLength - 1) 
                            yLabelGrid.append("M" + " " + Math.floor(xPosition + x1) + " " + endY + " " + "L" + " " + endX + " " + endY + " ");
                        }
                    }
                }
                var labelGridOptions = {
                    'id': this.svgObject.id + '_YAxisLabelBorder_' + axisIndex,
                    'fill': 'none',
                    'stroke-width': labelBorder.width,
                    'stroke': labelBorder.color,
                    'd': yLabelGrid.toString()
                };
                this.svgRenderer.drawPath(labelGridOptions, gEle);
                $(gEle).appendTo(this.gYaxisEle);
            }

        //Drawning Minor Ticks
        if (yAxis.minorTickLines.visible && yAxis.visible && yAxis.minorTicksPerInterval > 0) {
            yMinorTicksDir = sbYMinorTicks.toString();
            var optionsMinotTick = {
                'id': this.svgObject.id + '_YAxisMinorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.minorTickLines.width,
                'stroke': yAxis.minorTickLines.color,
                'd': yMinorTicksDir
            };
			this.svgRenderer.drawPath(optionsMinotTick, gEle);
        }
       

        if (yAxis.majorTickLines.visible && yAxis.visible) {
            yMajorTicksDir = sbYMajorTicks.toString();
            //Drawning Major Ticks
            var optionsMajorTick = {
                'id': this.svgObject.id + '_YAxisMajorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.majorTickLines.width,
                'stroke': yAxis.majorTickLines.color,
                'd': yMajorTicksDir
            };
			this.svgRenderer.drawPath(optionsMajorTick, gEle);
        }
    },

    _drawYAxisGridLine: function (axisIndex, yAxis, params) {

        var yMajorGridDir = "", yMajorTicksDir = "", yMinorTicksDir = "", yMinorGridDir = "";
        var pointY = 0;
        var labelValue;
        var y1 = Math.floor(yAxis.y + yAxis.height);
        var y2 = Math.floor(yAxis.y);
        var opposedPosition = yAxis._opposed;
        var yPoint = [];
        var axisName = (yAxis.name).replace(/[^a-zA-Z0-9]/g, "");
        var isCrossesAt = params.axes[axisName]._validCross;
        if (axisIndex != 1 && !isCrossesAt &&((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')) && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
            var x2 = (!opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
    else if(axisIndex != 1 && !isCrossesAt&& (yAxis.labelPosition != 'inside' && yAxis.tickLinesPosition == 'inside') && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) 
        {
   
            var x1 = (!opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
            var x2 = (!opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }
        else {
            var x1 = Math.floor(yAxis.x);
            var x2 = Math.floor(yAxis.x);
        }             
        var sbYMinorGrid = ej.EjSvgRender.utils._getStringBuilder();        
        var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' + '_' + axisIndex });
        var gAgb = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisAlternateGridBand' + '_' + axisIndex });

        if (yAxis.visible && yAxis.axisLine.visible) {
            var offset = yAxis.axisLine.offset > 0 ? yAxis.axisLine.offset : 0;
            var val = this._getSharpPath(yAxis.axisLine.width);
            
            var optionsLine = {
                'id': this.svgObject.id +axisName + '_YAxisLine_' + axisIndex,
                x1: x1+val,
                y1: (y2 - yAxis.plotOffset) + offset+val,
                x2: x2+val,
                y2: (y1 + yAxis.plotOffset) - offset+val,
                'stroke-width': yAxis.axisLine.width,
                'stroke-dasharray': yAxis.axisLine.dashArray,
                'stroke': yAxis.axisLine.color,
                'opacity': yAxis.axisLine.opacity||1
            };
            params.axes[yAxis.name]._lineOption = [optionsLine, this.gYaxisEle];
        }
		else
			params.axes[yAxis.name]._lineOption = [null, this.gYaxisEle];
     
        if (yAxis.axisBottomLine) {
            if (yAxis.axisBottomLine.visible) {
                var optionsBottomLine = {
                    'id': this.svgObject.id +axisName+ '_YAxisBottomLine_' + axisIndex,
                    x1: this.model.m_AreaBounds.X,
                    y1: y1,
                    x2: this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width,
                    y2: y1,
                    'stroke-width': yAxis.axisBottomLine.width,
                    'stroke': yAxis.axisBottomLine.color
                };
                this.svgRenderer.drawLine(optionsBottomLine, gEle);
            }
        }
        yAxis.labelPlacement = (!(yAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : yAxis.labelPlacement;
        var ticksbwtLabel = ((yAxis.labels.length > 0) && (yAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = yAxis.visibleLabels.length;
        for (var i = 0; i < collectionLength; i++) {
            labelValue = yAxis.visibleLabels[i].Value;
            var tempInterval = ((yAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;
            tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : tempInterval;

            if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange)) {

                if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                    continue;

                // Calculate Minor Ticks and Grid lines      
                if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                    var minorPointY;
                    var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                    if (yAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = yAxis.visibleRange.max;
                        var logmin = yAxis.visibleRange.min;

                        var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                        var logtickend = Math.pow(yAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                            
                              
                                //Calculate Minor Gridlines
                                if (yAxis.minorGridLines.visible)
                                    sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");

                            }
                            logtickPos += logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos,yAxis.logBase);
                        }
                    } else {
                        for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                            minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                            if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                  minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                              
                                // Calculate Minor Grid lines   
                                if (yAxis.minorGridLines.visible)
                                    sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");
                            }
                        }
                    }
                } 
                // Calculate Grid lines
                if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
                   
                    // Calculate Major Grid lines
                    if (yAxis.majorGridLines.visible) {
                        pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                        var mX1 = (Math.floor(this.model.m_AreaBounds.X) + mlVal);
                        var mY1 = (Math.floor((pointY * -1) + y1) );
                        var mX2 = (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + mlVal);
                        if (i == 0) {
                            if (mY1==Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height))
                                continue;
                            if (yAxis.zoomed)
                                yPoint[0] = mY1 + mlVal;
                            
                        }
                        if ((i == (yAxis.visibleLabels.length - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                           
                            if (mY1 == (Math.floor(this.model.m_AreaBounds.Y))) {
                                yPoint.push(mY1 + mlVal);
                                continue;
                            }

                        }
                       
                        

                        sbYMajorGrid.append("M" + " " + mX1 + " " + (mY1 + mlVal) + " " + "L" + " " + mX2 + " " + (mY1 + mlVal) + " ");
                        if (i == 0) 
                            continue;
                        
                        yPoint.push(mY1 + mlVal);
                        
                    }
                }
            }
        }
        
        if (yAxis.alternateGridBand.odd.fill != "transparent" || yAxis.alternateGridBand.even.fill != "transparent") {
           
            var width = this.model.m_AreaBounds.Width;
            if (yAxis.zoomed) {
                yPoint[yPoint.length] = yAxis.y;
                
            }
            for (var i = 1; i < yPoint.length; i += 2) {
                var height = yPoint[0] - yPoint[1];
                if (i == yPoint.length - 1 && yAxis.zoomed) {
                    var zoomHeight = yPoint[yPoint.length - 2] - yAxis.y;
                    height = zoomHeight;
                }
                var yoptionsAlternateodd = {
                    'id': this.svgObject.id + '_YAxisAlternateodd' + [i],
                    'x': Math.floor(this.model.m_AreaBounds.X),
                    'y': yPoint[i],
                    'width': width,
                    'height': height,
                    'fill': yAxis.alternateGridBand.odd.fill,
                    'opacity': yAxis.alternateGridBand.odd.opacity,
                    'stroke' : "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                };
                this.svgRenderer.drawRect(yoptionsAlternateodd, gAgb);
            }
            
            for (var i = 0; i < yPoint.length; i += 2) {
                var height = yPoint[0] - yPoint[1];
                if (i == yPoint.length - 1 && yAxis.zoomed) {
                    var zoomHeight = yPoint[yPoint.length - 2] - yAxis.y;
                    height = zoomHeight;
                }
                if (i == 0 && yAxis.zoomed) {
                    var zoomHeight = yAxis.y + yAxis.height - yPoint[0] - yAxis.plotOffset;
                    height = zoomHeight;
                }
                var yoptionsAlternateeven = {
                    'id': this.svgObject.id + '_YAxisAlternateeven' + [i],
                    'x': Math.floor(this.model.m_AreaBounds.X),
                    'y': yPoint[i],
                    'width': width,
                    'height': height,
                    'fill': yAxis.alternateGridBand.even.fill,
                    'opacity': yAxis.alternateGridBand.even.opacity,
                    'stroke': "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                };
                this.svgRenderer.drawRect(yoptionsAlternateeven, gAgb);
            }
            $(gAgb).appendTo(gEle);
        }

     
        if (yAxis.minorGridLines.visible && yAxis.minorTicksPerInterval > 0) {
            //Drawning Minor Grid Lines
            yMinorGridDir = sbYMinorGrid.toString();
            var optionsMinorGrid = {
                'id': this.svgObject.id + '_YAxisMinorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.minorGridLines.width,
                'stroke': yAxis.minorGridLines.color,
                'stroke-dasharray': yAxis.minorGridLines.dashArray,
                'd': yMinorGridDir
            };
            this.svgRenderer.drawPath(optionsMinorGrid, gEle);

        }

        //Drawing Major Grid Lines
        yMajorGridDir = sbYMajorGrid.toString();
        if (yAxis.majorGridLines.visible) {
            var options = {
                'id': this.svgObject.id + '_YAxisMajorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.majorGridLines.width,
                'opacity': yAxis.majorGridLines.opacity,
                'stroke-dasharray': yAxis.majorGridLines.dashArray,
                'stroke': yAxis.majorGridLines.color,
                'd': yMajorGridDir
            };

            if(options.d != '')
                this.svgRenderer.drawPath(options, gEle);

        }
        $(gEle).appendTo(this.svgObject);
    },
	
	textOverflowMultiLevelLabels: function (axis, gap, text, textOverflow, font, labelSize, textOptions, gMultiLevelEle, chartObj) {
		var yVal = textOptions.y, count = 0;
		 if (textOverflow == "wrap" || textOverflow == "wrapandtrim") {
                collection = chartObj._wrap(axis, gap, text, textOverflow, font, labelSize);
                textCollection = collection["text"];
                rows = textCollection.length;
                for (j = 0; j < rows ; j++) {
					textOptions.y += labelSize.height;
					if (!chartObj.model.enable3D) {
						textOptions.id += "_" + j;
						chartObj.svgRenderer.drawText(textOptions, textCollection[j], gMultiLevelEle);
					}
                    if (textOverflow == "wrapandtrim" && textCollection[j].indexOf("...") > -1) {
						if (chartObj.model.enable3D && count == 0) 
							textOptions.y -= labelSize.height;
                        unTrimmedText = collection["unTrimmedText"];
                        labelSize = ej.EjSvgRender.utils._measureText(textCollection[j], null, font);
                        bounds = { X: textOptions.x - labelSize.width / 2, Y: textOptions.y, Width: labelSize.width, Height: labelSize.height };
                        chartObj.model.axisMultiLevelLabelRegions.push({ trimText: textCollection[j], labelText: unTrimmedText[count], Bounds: bounds });
                        count++;
                    }
                }
				return textCollection;
            } else {
                text = (textOverflow == "trim") ? ej.EjSvgRender.utils._trimText(actualText, gap, font) : actualText;
                // to store region
				if (!chartObj.model.enable3D) 
					yVal = textOptions.y +labelSize.height;
				if (axis.orientation.toLowerCase() == "vertical")
					yVal = textOptions.y -labelSize.height/4;
                labelSize = ej.EjSvgRender.utils._measureText(text, null, font);
                bounds = { X: textOptions.x - labelSize.width/2, Y: yVal, Width: labelSize.width, Height: labelSize.height };
                chartObj.model.axisMultiLevelLabelRegions.push({ trimText: text, labelText: actualText, Bounds: bounds });
				return text;
            }
	},
	
	_triggerMultiLevelLabelsRendering: function (text, x, y, textOverflow, font, border, chart){
		var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);          // trigger event
        commonEventArgs.data = {
                text: text, location: { x: x, y: y },
                textOverflow: textOverflow, font: font, border: border
            };
            chart._trigger("multiLevelLabelRendering", commonEventArgs);
			return commonEventArgs.data;
	},
	
    _drawXAxisMultiLevelLabels: function (axisIndex, xAxis) {          // to render x axis multi level labels
        // declaration 
        var i, j, x, y, rows, gap, labelSize, grpLabel, font, startX, startY, endX, textOptions, textOverflow,
            textCollection, borderOptions, border, style, alignment, svgId = this.svgObject.id, range = xAxis.visibleRange,
            gMultiLevelEle = this.svgRenderer.createGroup({ 'id': svgId + '_XAxisMultiLevelLabels' + '_' + axisIndex, 'clip-path': 'url(#' + this.svgObject.id +
			'_XAxis_Clippath_' + axisIndex + ")" }), areaBounds = this.model.m_AreaBounds, opposedPosition = xAxis.opposedPosition, borderWidth,
            labelPosition = xAxis.labelPosition.toLowerCase(), labelBorder = xAxis.labelBorder, padding = 10,
            scrollerSize = xAxis._isScroll ? this.model.scrollerSize :0;
            
        // create clip path
        startY = xAxis.majorTickLines.size + xAxis._LableMaxWidth.height + (labelBorder.width);
        clipX = areaBounds.X;
        clipY = xAxis.y + startY + scrollerSize;
        clipWidth = areaBounds.Width;
        clipHeight = xAxis._multiLevelLabelHeight + padding - 2;

        if (opposedPosition) {
            if (labelPosition == "outside")
                clipY = xAxis.y - startY - xAxis._multiLevelLabelHeight - padding / 2 - scrollerSize;
            else
                clipY = xAxis.y + startY - padding/2;
        } else {
            if (labelPosition == "inside")
                clipY = xAxis.y - startY - xAxis._multiLevelLabelHeight;
        }

        // to clip multi level labels
        var clipRectOptions = {
            'id': this.svgObject.id + '_XAxis_Clippath_' + axisIndex,
            'x': clipX,
            'y': clipY,
            'width': clipWidth,
            'height': clipHeight,
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'Gray'
        };
        this.svgRenderer.drawClipPath(clipRectOptions, gMultiLevelEle);
        
        for (i = 0; i < xAxis.multiLevelLabels.length; i++) {
          // assignment
          grpLabel = xAxis.multiLevelLabels[i];
          if (grpLabel.visible && ((grpLabel.start >= range.min && grpLabel.start <= range.max)
                || (grpLabel.end >= range.min && grpLabel.end <= range.max) ||
                    (grpLabel.start < range.min && grpLabel.end > range.max))) {
            font = grpLabel.font;
            border = grpLabel.border;
            borderWidth = border.width;
            level = grpLabel._level;
            textOverflow = grpLabel.textOverflow.toLowerCase();
            style = border.type.toLowerCase();
            alignment = grpLabel.textAlignment.toLowerCase();
            startX = Math.ceil(ej.EjSvgRender.utils._getPointXY(grpLabel.start, range, xAxis.isInversed) * (xAxis.width));
            endX = Math.ceil(ej.EjSvgRender.utils._getPointXY(grpLabel.end, range, xAxis.isInversed) * (xAxis.width));
			if (xAxis.isInversed){
				startX = startX - endX;
				endX = startX + endX;
				startX = endX - startX;
			}
            actualText = grpLabel.text;
            labelSize = ej.EjSvgRender.utils._measureText(actualText, null, font);
            gap = ej.util.isNullOrUndefined(grpLabel.maximumTextWidth) ? endX - startX - borderWidth - padding : grpLabel.maximumTextWidth - padding;
            rows = 1;
            count = 0;
            x = startX + xAxis.x + padding/2;
            y = startY + xAxis.y;
           
            // to position text
            if (opposedPosition) {
                if (labelPosition == "outside")
                    y = xAxis.y - startY - xAxis.multiLevelLabelHeight[level] - padding - xAxis.prevHeight[level] - scrollerSize;
                else
                    y = xAxis.y + startY - padding/2 + xAxis.prevHeight[level];
            } else {
                if (labelPosition == "inside")
                    y = xAxis.y - startY - xAxis.multiLevelLabelHeight[level] - xAxis.prevHeight[level];
                else
                    y = startY + xAxis.y + xAxis.prevHeight[level] + scrollerSize;
            }

            // to clip canvas
            if (this.model.enableCanvasRendering) {
                this.svgRenderer.ctx.save();
                this.svgRenderer.ctx.beginPath();
                this.svgRenderer.ctx.rect(clipX, clipY, clipWidth, clipHeight);
                this.svgRenderer.ctx.clip();
            }

            // text alignment calculation
            if (alignment == "center") {
                x += gap / 2;
                anchor = "middle";
            } else if (alignment == "far") {
                x = x + gap - borderWidth / 2;
                anchor = "end";
            } else {
                anchor = "start";
                x += borderWidth / 2;
            }
            
            var data = this._triggerMultiLevelLabelsRendering(actualText, x, y, textOverflow, font, grpLabel.border, this);
			actualText = data.text;
			x = data.location.x;
			y = data.location.y;
			textOverflow = data.textOverflow.toLowerCase();
			font = data.font;
			border = data.border;
			style = data.border.type.toLowerCase();
			borderWidth = data.border.width;
			
            // options required to render text
            textOptions = {
                'id': svgId + '_XAxisMultiLevelLabels_Text_' + axisIndex + '_' + i,
                'x': x,
                'y': style == "curlybrace" ? y + 10 : y,
                'fill': font.color ? font.color : xAxis.multiLevelLabelsFontColor,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': anchor
            };
			text = actualText;
			// calculation for wrap and wrapbyword
			if (textOverflow != "none")
				text = this.textOverflowMultiLevelLabels(xAxis, gap, text, textOverflow, font, labelSize, textOptions, gMultiLevelEle, this);
			
			if (textOverflow != "wrap" && textOverflow != "wrapandtrim"){
				textOptions.y += labelSize.height;
				if (labelSize.height < xAxis.multiLevelLabelHeight[level])
					textOptions.y = textOptions.y + xAxis.multiLevelLabelHeight[level] / 2 - labelSize.height/2;
				this.svgRenderer.drawText(textOptions, text, gMultiLevelEle);   // to draw text
			}

            // border calculation
            if (opposedPosition) {
                if(labelPosition == "inside")
                    y = startY + xAxis.y + xAxis.prevHeight[level] - padding/2;
                else
                    y = xAxis.y - startY - xAxis.multiLevelLabelHeight[level] - padding + 2 - xAxis.prevHeight[level] - scrollerSize;
            } else {
                if (labelPosition == "inside")
                    y = xAxis.y - startY - xAxis.multiLevelLabelHeight[level] - 3 - xAxis.prevHeight[level];
                else
                    y = startY + xAxis.y + xAxis.prevHeight[level] + scrollerSize;
            }
            x = startX + xAxis.x;
            y += borderWidth;
            height = xAxis.multiLevelLabelHeight[level] + padding/2;
            width = gap + padding + borderWidth;
			
			// storing region for multi level labels click event
			var region = {bounds: {x: x, y: y, height: height, width: width }, axisIndex: axisIndex, multiLevelLabel: grpLabel};
			this.model.multiLevelLabelRegions.push(region);
			
            path = "";
            braceGap = ((width + borderWidth) - labelSize.width)/2 - 2;
            if (style == "rectangle") {
                borderOptions = {
                    'id': svgId + '_XAxisMultiLevelLabels_Rect_' + axisIndex + '_' + i,
                    'x': x,
                    'y': y,
                    'width': width,
                    'height': height,
                    'fill': "transparent",
                    'stroke': border.color ? border.color : xAxis.multiLevelLabelsColor,
                    'stroke-width': borderWidth,
                };
                this.svgRenderer.drawRect(borderOptions, gMultiLevelEle);
            } else {
                switch (style) {
                    case "withouttopandbottomborder":
                        path = "M " + x + " " + y + " L " + x + " " + (y + height) + 
                            " M " + (x + width) + " " + y + " L " + (x + width) + " " + (y + height);
						break;
					case "withouttopborder":
						if ((!opposedPosition && labelPosition == "outside") || (opposedPosition && labelPosition == "inside"))
							labelHeight = height;
						else
							labelHeight = 0;
						path = "M " + x + " " + (y + labelHeight) + " L " + (x + width) + " " + (y + labelHeight) + " M " + x + " " + y + " L " + x + " " + 
								(y + height) + " M " + (x + width) + " " + y + " L " + (x + width) + " " + (y + height);
                        break;
                    case "brace":
						if (alignment == "near"){
							var value = textOptions.x;
							value2 = textOptions.x + labelSize.width + 2;
						}
						else if (alignment == "center"){
							var value = textOptions.x - labelSize.width / 2 - 2 <= areaBounds.X + startX ? areaBounds.X + startX + padding : 
								textOptions.x - labelSize.width / 2 - 2;
							var value2 = textOptions.x + labelSize.width / 2 + 2 > areaBounds.X + endX ? areaBounds.X + endX - padding :
								textOptions.x + labelSize.width / 2 + 2
						} else{
							var value = textOptions.x - labelSize.width - 2;
							value2 = textOptions.x;
						}
                        if ((!opposedPosition && labelPosition == "outside") || (opposedPosition && labelPosition == "inside"))
                            path = "M " + x + " " + y + " L " + x + " " + (y + height / 2) +
                       " M " + x + " " + (y + height / 2) + " L " + (value - 2) + " " + (y + height / 2) +
                       " M " + (value2) + " " + (y + height / 2) + " L " + (x + width) + " " + (y + height / 2 ) +
                       " M " + (x + width) + " " + (y + height / 2) + " L " + (x + width) + " " + (y);
                        else
                            path = "M " + (x) + " " + (y + height) + " L " + (x) + " " + (y + height / 2) +
                                " M " + x + " " + (y + height / 2) + " L " + (value) + " " + (y + height / 2) +
                                " M " + (value2) + " " + (y + height / 2) + " L " + (x + width) + " " + (y + height / 2) + 
                                 " M " + (x + width) + " " + (y + height) + " L " + (x + width) + " " + (y + height/2);
                        break;
					case "curlybrace":        // using constant values to render a curely brace
						y = xAxis.y + startY + xAxis.prevHeight[level]; 
								if (width< 30){
									path = "M " + (x+width/2 -5) + " " + (y+10) + " L " + (x+width/2) + " " + (y+15) + " L " + (x+width/2 +5) + " " + (y+10);
								}
								else if (!opposedPosition){
									if (alignment == "center") {
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x+5) + " " + (y+10)  + " " + (x+10) + " " + (y+10) + 
										" L " + (x+ width/2 -5) + " " + (y+ 10) + " L " + (x+ width/2) + " " + (y+15)  +
										" L " + (x + width/2 +5) + " " + (y + 10) + " L " + (x + width-10)  + " " + (y +10) + 
										" C " + (x+width-10) + " " + (y + 10) + " " + (x+width) + " " + (y +5) + " " + (x+width) + " " + (y) ;
									} else if (alignment == "near"){
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x+5) + " " + (y+10)  + " " + (x+10) + " " + (y+10) + 
										" L " + (x+ 15) + " " + (y+15)  +
										" L " + (x +15 +5) + " " + (y + 10) + " L " + (x + width-10)  + " " + (y +10) + 
										" C " + (x+width-10) + " " + (y + 10) + " " + (x+width) + " " + (y +5) + " " + (x+width) + " " + (y) ;
									} else {
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x+5) + " " + (y+10)  + " " + (x+10) + " " + (y+10) + 
										" L " + (x+ width-20) + " " + (y+ 10) + " L " + (x+ width-15) + " " + (y+15)  +
										" L " + (x + width -10) + " " + (y + 10) + " L " + (x + width-10)  + " " + (y +10) + 
										" C " + (x+width-10) + " " + (y + 10) + " " + (x+width) + " " + (y +5) + " " + (x+width) + " " + (y);
									}
								}
								 else {
									path = "M " + x + " " + y + " C " + x + " " + y + " " + (x+5) + " " + (y+10)  + " " + (x+10) + " " + (y+10) + 
										" L " + (x+ width/2 -5) + " " + (y+ 10) + " L " + (x+ width/2) + " " + (y+15)  +
										" L " + (x + width/2 +5) + " " + (y + 10) + " L " + (x + width-10)  + " " + (y +10) + 
										" C " + (x+width-10) + " " + (y + 10) + " " + (x+width) + " " + (y +5) + " " + (x+width) + " " + (y) ;
								 }
						break;
                }
                if (path) {
                    borderOptions = {
                        'id': svgId + '_XAxisMultiLevelLabels_Rect_' + axisIndex + '_' + i,
                        'd': path,
                        'stroke': border.color ? border.color : xAxis.multiLevelLabelsColor,
                        'stroke-width': border.width,
						'fill':'none'
                    }
                    this.svgRenderer.drawPath(borderOptions, gMultiLevelEle);
                }
            }
            if (this.model.enableCanvasRendering)
                this.svgRenderer.ctx.restore();
				
            $(gMultiLevelEle).appendTo(this.gXaxisEle);
         }
        }
    },

    _drawYAxisMultiLevelLabels: function (axisIndex, yAxis) {          // to render y axis multi level labels
        // declaration 
        var i, j, x, y, rows, gap, labelSize, grpLabel, font, startX, startY, endX, textOptions, textOverflow, padding = 5,
            textCollection, borderOptions, border, style, alignment, svgId = this.svgObject.id, range = yAxis.visibleRange,
            gMultiLevelEle = this.svgRenderer.createGroup({ 'id': svgId + '_YAxisMultiLevelLabels' + '_' + axisIndex, 'clip-path': 'url(#' + this.svgObject.id +
			'_YAxis_Clippath_' + axisIndex + ")" }), anchor = "middle", areaBounds = this.model.m_AreaBounds, opposedPosition = yAxis.opposedPosition,
            labelPosition = yAxis.labelPosition.toLowerCase(), labelBorder = yAxis.labelBorder, multiLevelLabelWidth = yAxis._multiLevelLabelHeight,
            startX = yAxis.majorTickLines.size + yAxis._LableMaxWidth.width + (labelBorder.width) + 2 * padding,
              scrollerSize = yAxis._isScroll ? this.model.scrollerSize : 0;

	    // to clip the multi level labels
        clipY = yAxis.y;
        clipWidth = multiLevelLabelWidth + padding;
        clipHeight = yAxis.height + padding;
        if (opposedPosition) {
            if(labelPosition == "outside")
                clipX = yAxis.x + startX - padding + scrollerSize;
            else
                clipX = yAxis.x - multiLevelLabelWidth - startX  + padding;
        }
        else{
            if (labelPosition == "outside")
                clipX = yAxis.x - multiLevelLabelWidth - startX - scrollerSize;
            else
                clipX = yAxis.x + startX - padding;
        }
        var clipRectOptions = {
            'id': this.svgObject.id + '_YAxis_Clippath_' + axisIndex,
            'x': clipX,
            'y': yAxis.y,
            'width': clipWidth,
            'height': clipHeight,
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'Gray'
        };
        this.svgRenderer.drawClipPath(clipRectOptions, gMultiLevelEle);

        for (i = 0; i < yAxis.multiLevelLabels.length; i++) {
            // assignment
            grpLabel = $.extend(true, {}, this.model.multiLevelLabelsDefault, yAxis.multiLevelLabels[i]);
            if (grpLabel.visible && ((grpLabel.start >= range.min && grpLabel.start <= range.max)
                || (grpLabel.end >= range.min && grpLabel.end <= range.max) ||
                    (grpLabel.start < range.min && grpLabel.end > range.max))) {
                font = grpLabel.font;
                border = grpLabel.border;
                level = grpLabel._level;
				maximumTextWidth = grpLabel.maximumTextWidth;
                textOverflow = grpLabel.textOverflow.toLowerCase();
                style = border.type.toLowerCase();
                alignment = grpLabel.textAlignment.toLowerCase();
                startY = Math.ceil(ej.EjSvgRender.utils._getPointXY(grpLabel.start, range, yAxis.isInversed) * (yAxis.height));
                endY = Math.ceil(ej.EjSvgRender.utils._getPointXY(grpLabel.end, range, yAxis.isInversed) * (yAxis.height));
				if (yAxis.isInversed){
					startY = startY - endY;
					endY = startY + endY;
					startY = endY - startY;
				}
                actualText = grpLabel.text;
                labelSize = ej.EjSvgRender.utils._measureText(actualText, null, font);
                gap = endY - startY - border.width;
                space = ej.util.isNullOrUndefined(maximumTextWidth) ? yAxis.multiLevelLabelHeight[level] + padding : maximumTextWidth - padding;
                rows = 1; count = 0; textX = 0;
                x = yAxis.x - startX - yAxis.prevHeight[level] - yAxis.multiLevelLabelHeight[level]/2 + border.width - padding/2;
                y = yAxis.height + yAxis.y - startY - (gap/2);

                // for text position
                   if ((opposedPosition)) {
                    if (labelPosition == "outside")
                        x = yAxis.x + startX + yAxis.multiLevelLabelHeight[level] / 2 + yAxis.prevHeight[level] + border.width + scrollerSize;
                    else
                        x = yAxis.x - startX - yAxis.multiLevelLabelHeight[level] / 2 - yAxis.prevHeight[level] + border.width + padding / 2;
                }
                else {
                    if (labelPosition == "inside")
                        x = yAxis.x + startX + yAxis.multiLevelLabelHeight[level] / 2 + yAxis.prevHeight[level] - padding / 2;
                    else
                        x = x + textX - scrollerSize;
                }
                if (this.model.enableCanvasRendering) {
                    this.svgRenderer.ctx.save();
                    this.svgRenderer.ctx.beginPath();
                    this.svgRenderer.ctx.rect(clipX, clipY, clipWidth, clipHeight);
                    this.svgRenderer.ctx.clip();
                }

                // text alignment calculation
				if (alignment == "far") 
                    y =  y + gap/2 - labelSize.height/2 ;
                else if (alignment == "near")
                    y = y - gap/2 + labelSize.height ;
                else
					y = y + labelSize.height/4;
				data = this._triggerMultiLevelLabelsRendering(actualText, x, y, textOverflow, font, grpLabel.border, this);
				actualText = data.text;
				x = data.location.x;
				y = data.location.y;
				textOverflow = data.textOverflow;
				font = data.font;
				border = data.border;
				style = border.type.toLowerCase();
				borderWidth = border.width;
				
                // options required to render text
                textOptions = {
                    'id': svgId + '_YAxisMultiLevelLabels_Text_' + axisIndex + '_' + i,
                    'x': style == "curlybrace" ? (opposedPosition ? x + textX + 10 :  x + textX - 10) : x + textX ,
                    'y': y,
                    'fill': font.color ? font.color : yAxis.multiLevelLabelsFontColor,
                    'font-size': font.size,
                    'font-family': font.fontFamily,
                    'font-style': font.fontStyle,
                    'font-weight': font.fontWeight,
                    'opacity': font.opacity,
                    'text-anchor': anchor
                };

				text = actualText;
				// calculation for wrap and wrapandtrim
				if (textOverflow != "none")
					text = this.textOverflowMultiLevelLabels(yAxis, space, text, textOverflow, font, labelSize, textOptions, gMultiLevelEle, this);
				if (textOverflow != "wrap" && textOverflow != "wrapandtrim")
					this.svgRenderer.drawText(textOptions, text, gMultiLevelEle);   // to draw text
                x = x - padding / 2 - yAxis.multiLevelLabelHeight[level] / 2;
                y = yAxis.y + yAxis.height - endY + border.width;
                height = endY - startY;
                width = yAxis.multiLevelLabelHeight[level] + padding;
                path = "";
                braceGap = ((width + border.width) - multiLevelLabelWidth) / 2 - 2;

				// storing region for multi level labels click event
				var region = {bounds: {x: x, y: y, height: height, width: width }, axisIndex: axisIndex, multiLevelLabel: grpLabel};
				this.model.multiLevelLabelRegions.push(region);
			
                if (style == "rectangle") {
                    borderOptions = {
                        'id': svgId + '_YAxisMultiLevelLabels_Rect_' + axisIndex + '_' + i,
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': "transparent",
                        'stroke': border.color ? border.color : yAxis.multiLevelLabelsColor,
                        'stroke-width': border.width,
                    };
                    this.svgRenderer.drawRect(borderOptions, gMultiLevelEle);
                } else {
                    switch (style) {
                        case "withouttopandbottomborder":
                            path = "M" + " " + x + " " + (y + height) + " " + "L" + " " + (x + width) + " " + (y + height) + " " +
                                "M" + " " + x + " " + y + " " + "L" + " " + (x + width) + " " + y;
                            break;
						case "withouttopborder":
							if ((!opposedPosition && labelPosition == "outside") || (opposedPosition && labelPosition == "inside"))
								labelWidth = 0;
							else
								labelWidth = width;
							path = "M" + " " + x + " " + (y + height) + " " + "L" + " " + (x + width) + " " + (y + height) + " " +
                                "M" + " " + x + " " + y + " " + "L" + " " + (x + width) + " " + y + 
								" M" + " " + (x + labelWidth) + " " + (y + height) + " " + "L" + " " + (x + labelWidth) + " " + (y);
                            break;
                        case "brace":
						value1 = textOptions.y  - labelSize.height/2 - 4;
						value2 = textOptions.y + labelSize.height/4 + 2;
                            if ((!opposedPosition && labelPosition == "outside") || (opposedPosition && labelPosition == "inside"))
                                path = "M " + (x + width) + " " + y + " L " + (x + width / 2) + " " + y +
                                " M " + (x + width / 2) + " " + y + " L " + (x + width / 2) + " " + value1 +
                                " M " + (x + width / 2) + " " + value2 + " L " + (x + width / 2) + " " + (y + height) +
                                " M " + (x + width / 2) + " " + (y + height) + " L " + (x + width) + " " + (y + height);
                            else
                                path = "M " + (x) + " " + y + " L " + (x + width / 2) + " " + y +
                               " M " + (x + width / 2) + " " + y + " L " + (x + width / 2) + " " + value1 +
                               " M " + (x + width / 2) + " " + value2 + " L " + (x + width / 2) + " " + (y + height) +
                               " M " + (x + width / 2) + " " + (y + height) + " L " + (x) + " " + (y + height);
                            break;
						case "curlybrace":               // using constant values to render a curely brace
								x = yAxis.x - startX - yAxis.prevHeight[level];
						        newHeight = height - 10;
								if (height < 30){
									if (opposedPosition){
										x = yAxis.x + startX + yAxis.prevHeight[level];
									newHeight = height - 10;
										path = 
										" M " + (x+10) + " " + (y+ newHeight/2) + " " + " L " + (x+15) + " " + (y+ newHeight/2 + 5) +
										" L " + (x+10) + " " + (y+newHeight/2 + 10) ;
										}
									else
										path = "M " + (x-10) + " " + (y+height/2 -5) + " L " + (x-15) + " " + (y+height/2 ) + " L " + (x-10) + " " + (y+height/2 +5);
								} 
								else if (!opposedPosition){
									if (alignment == "center") {
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x-10) + " " + y  + " " + (x-10) + " " + (y+10) + 
										" L " + (x-10) + " " + (y+ newHeight/2) + " L " + (x-15) + " " + (y+ newHeight/2 + 5) +
										" L " + (x-10) + " " + (y+newHeight/2 + 10) + " L " + (x-10)  + " " + (y + newHeight) + 
										" C " + (x-10) + " " + (y + newHeight) + " " + (x-5) + " " + (y + newHeight+ 10) + " " + x + " " + (y + newHeight+ 10) ;
									} else if (alignment == "far"){
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x-10) + " " + y  + " " + (x-10) + " " + (y+10) + 
										" L " + (x-10) + " " + (y+ newHeight - 10) + " " + " L " + (x-15) + " " + (y+ newHeight - 10+ 5) +
										" L " + (x-10) + " " + (y+newHeight) + " L " + (x-10)  + " " + (y + newHeight) + 
										" C" + (x-10) + " " + (y + newHeight) + " " + (x -10) + " " + (y + newHeight+ 10) + " " + x + " " + (y + newHeight+ 10) ;
									} else {
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x-10) + " " + y  + " " + (x-10) + " " + (y+10) + 
										" L " + (x-15) + " " + (y +10 + 5) +
										" L " + (x-10) + " " + (y+10 + 10) + " L " + (x-10)  + " " + (y + newHeight) + 
										" C" + (x-10) + " " + (y + newHeight) + " " + (x-5) + " " + (y + newHeight+ 10) + " " + x + " " + (y + newHeight+ 10) ;
									}
								} else {
									x = yAxis.x + startX + yAxis.prevHeight[level];
									newHeight = height - 10;
										path = "M " + x + " " + y + " C " + x + " " + y + " " + (x+10) + " " + y  + " " + (x+10) + " " + (y+10) + 
										" L " + (x+10) + " " + (y+ newHeight/2) + " " + " L " + (x+15) + " " + (y+ newHeight/2 + 5) +
										" L " + (x+10) + " " + (y+newHeight/2 + 10) + " L " + (x+10)  + " " + (y + newHeight) + 
										" C" + (x+10) + " " + (y + newHeight) + " " + (x+5) + " " + (y + newHeight+ 10) + " " + x + " " + (y + newHeight+ 10) ;
								}
                    }
                    if (path) {
                        borderOptions = {
                            'id': svgId + '_YAxisMultiLevelLabels_Rect_' + axisIndex + '_' + i,
                            'd': path,
                            'stroke': border.color ? border.color : yAxis.multiLevelLabelsColor,
                            'stroke-width': border.width,
							'fill': "none"
                        }
                        this.svgRenderer.drawPath(borderOptions, gMultiLevelEle);
                    }
                }
                if (this.model.enableCanvasRendering)
                    this.svgRenderer.ctx.restore();
                $(gMultiLevelEle).appendTo(this.gYaxisEle);
            }
        }
    },

    _drawXAxisLabels: function (axisIndex, xAxis) {
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' }),
            gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' }),
            prevLabels = [],
            bounds, areaBounds = this.model.m_AreaBounds,
            labelSize,
            axisName = (xAxis.name).replace(/[^a-zA-Z0-9]/g, ""),
        	isScroll = xAxis._isScroll && !(xAxis.scrollbarSettings.pointsLength != null && xAxis.scrollbarSettings.pointsLength < xAxis.maxPointLength);
        var opposedPosition = xAxis._opposed, valueType = xAxis._valueType, pixel = 3, xAxisAignment = xAxis.alignment.toLowerCase(), x;
        if (axisIndex != 0 && ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside')||(xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside')) && (!(xAxis.y == (areaBounds.Height + areaBounds.Y) || (xAxis.y == areaBounds.Y)))) {

            xAxis.y = (!opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (areaBounds.Height + areaBounds.Y) || (xAxis.y == areaBounds.Y)))) {
            xAxis.y = (!opposedPosition) ? Math.floor(xAxis.y+xAxis.majorTickLines.size) : Math.floor(xAxis.y-xAxis.majorTickLines.size);
        }
        else{
			xAxis._y = Math.floor(xAxis.y);//to store the current positons of the labels,before the property showNextToAxisLine as false.
			xAxis.y = (!xAxis.showNextToAxisLine) ? opposedPosition ? Math.floor(areaBounds.Y) : Math.floor(areaBounds.Height + areaBounds.Y) : Math.floor(xAxis.y);
		}
       
        // loop to draw labels
        for (var i = 0; i < xAxis.visibleLabels.length; i++) {
            var label = xAxis.visibleLabels[i];
            var labelText = label.displayText = ej.util.isNullOrUndefined(label.displayText) ? label.Text : label.displayText;
            if (ej.EjSvgRender.utils._inside(label.Value, xAxis.visibleRange)) {
                //Declaration
                var textanchor = null, newGapVal1, newGapVal2, nextPointX, nextLabel, nextLabelValue;
                var textcollection = [];
                    var gap = xAxis.width / xAxis.visibleLabels.length;       //space between ticks
					 var trimGap = xAxis.maximumLabelWidth // this holds maximumLabelWidth value      
                    var range = xAxis.visibleRange;
                    var areaBoundWidth = areaBounds.Width;
					if(!ej.isNullOrUndefined(xAxis.valueType)){
                    if (i<xAxis.visibleLabels.length-1 && xAxis.valueType.toLowerCase() == "datetimecategory" && xAxis.labelIntersectAction == "wrap" && xAxis.labelPlacement.toLowerCase() == "betweenticks")
                    {
                        nextLabel = xAxis.visibleLabels[i + 1];
                        label.Value -= .5;
                        nextLabelValue = nextLabel.Value - .5;
                    }
					}
                    var pointX = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(label.Value, range, xAxis.isInversed) * (xAxis.width)));
					if(!ej.isNullOrUndefined(xAxis.valueType)){
                    if (i < xAxis.visibleLabels.length - 1 && xAxis.valueType.toLowerCase() == "datetimecategory" && xAxis.labelIntersectAction == "wrap" && xAxis.labelPlacement.toLowerCase() == "betweenticks") {
                        nextPointX = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(nextLabelValue, range, xAxis.isInversed) * (xAxis.width)));
                        newGapVal1 = (i == 0) ? (xAxis.x + pointX) + (xAxis.majorTickLines.width - 0.5) : (xAxis.x + pointX) + (xAxis.majorTickLines.width + 0.5);
                        newGapVal2 = (xAxis.x + nextPointX) + (xAxis.majorTickLines.width + 0.5);
                        gap = newGapVal2 - newGapVal1;
                    }
                    else if (xAxis.valueType.toLowerCase() == "datetimecategory" && xAxis.labelIntersectAction == "wrap" && xAxis.labelPlacement.toLowerCase() == "betweenticks") {
                        gap = (xAxis.width + xAxis.x) - newGapVal2;
                        newGapVal1 = newGapVal2;
                    }
                    else if (xAxis.valueType.toLowerCase() == "datetimecategory" && xAxis.labelIntersectAction == "wrap" && xAxis.labelPlacement.toLowerCase() == "onticks") {
                        gap = xAxis.width / xAxis.labels.length;
                    }
					}					
                    xAxis._gap = newGapVal1;
                    var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font);      
                    var textWidth = textSize.width;
                    var textHeight = textSize.height;
                    var labelMaxWidth = xAxis._LableMaxWidth;
                    var labelMaxHeight = labelMaxWidth.height;
                    var labelMaxRow = labelMaxWidth.rows;
                    var vmlrendering = this.svgRenderer.vmlNamespace;                    
                    var lineWidth = xAxis.axisLine.width;
                    var font = xAxis.font;
					var labelIntersectAction = xAxis.labelIntersectAction;
                    var labelPlacement = xAxis.labelPlacement.toLowerCase();
					var labelRotation = xAxis.rotationValue;
                    var count = 0;
                    var flag = false;
                    this.edgeLabel = false; // flag to find the supprot for edge labels
                    var chartAreaWidth =areaBounds.Width;
                    var chartAreaX = areaBounds.X;
                    var intersectAction = xAxis.labelIntersectAction.toLowerCase();
                    var svgWidth = $(this.svgObject).width(), maxLineWidth;
                    //To Perform trim 
                    if (intersectAction == "trim" || xAxis.enableTrim) {
                        if (labelPlacement == "onticks" && i != xAxis.visibleLabels.length -1) {
                            var nextLabel = xAxis.visibleLabels[i+1];
                            var nextPoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(nextLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                            var nextTextWidth = ej.EjSvgRender.utils._measureText(nextLabel.Text, areaBoundWidth, xAxis.font).width;
                            var value = nextPoint - nextTextWidth / 2;
                            var count = 0, current = false;
                            while (value < pointX + textWidth / 2 &&  count<2) {             // intersect
                                count++; 
                                if (nextTextWidth > textWidth) {                            // getting max label
                                    otherLabel = trimLabel = nextLabel;
                                    current = false;
                                }
                                else {
                                    trimLabel = label;
                                    current = true;
                                }
                                var trimLabelText = trimLabel.displayText = trimLabel.Text.toString();
                                for (var t = 1; t < trimLabelText.length; t++) {
                                    trimLabelText = trimLabel.displayText.substring(0, t) + '... ';
                                    newTextWidth = ej.EjSvgRender.utils._measureText(trimLabelText, areaBoundWidth, font).width;
                                    if (!current) {                                                  // to trim next label
                                        value = nextPoint - newTextWidth / 2;
                                        if (value < pointX + textWidth / 2) {
                                            if (t - 1 != 0) 
                                                trimLabelText = trimLabelText.toString().substring(0, t - 1) + '... ';
                                            nextTextWidth = newTextWidth;
                                            trimLabel.displayText = trimLabelText;
                                            break;
                                        }
                                    }
                                    else {                                                           // to trim current label
                                        value = nextPoint - nextTextWidth / 2;
                                        if (value < pointX + newTextWidth / 2) {
                                            if (t - 1 != 0) 
                                                trimLabelText = trimLabelText.toString().substring(0, t - 1) + '... ';
                                            textWidth = newTextWidth;
                                            labelText = trimLabel.displayText = trimLabelText;
                                            break;
                                        }
                                    }
                                }
                            }
                        } else if (labelPlacement == "betweenticks") {
                            var derivedGap = (xAxis.enableTrim) ? trimGap : gap;
                        if (textWidth > derivedGap && labelText != "") { // check textwidth is greater than gap
                            for (var t = 1; t < labelText.toString().length; t++) {
                                labelText = label.displayText.toString().substring(0, t) + '... ';
                                textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;                                
                                if (textWidth >= derivedGap) {
                                    labelText = labelText.toString().substring(0, t - 1) + '... ';
                                    label.displayText = labelText;
                                    if (i == xAxis.visibleLabels.length - 1) {
                                        if (((pointX + xAxis.x) + (textWidth / 2) > (svgWidth)) && derivedGap==gap ) {
                                            derivedGap -= (textWidth / 2);
                                            t = 1;
                                            continue;
                                        }
                                    }
                                    break;
                                }
                            }
                            label.displayText = labelText; // trimed text is assigned to displayText
                        }
                      }
                    }
                 var textSize = ej.EjSvgRender.utils._measureText(label.displayText, areaBoundWidth, font);                    
                 var anchor = textSize.width / 2;
                 if (xAxis.majorGridLines.width > xAxis.majorTickLines.width)
                     maxLineWidth = xAxis.majorGridLines.width
                 else  maxLineWidth = xAxis.majorTickLines.width;
                 if (!ej.util.isNullOrUndefined(label.Text)) {
                     if (xAxisAignment == "near")
                         x = pointX + xAxis.x + maxLineWidth + ((valueType == "category" && labelPlacement != "onticks") ? -(xAxis.width / xAxis.actualRange.delta) / 2 + pixel : -textSize.width - pixel);
                     else if (xAxisAignment == "far")
                         x = (pointX + xAxis.x) + ((valueType == "category" && labelPlacement != "onticks") ? +((xAxis.width / xAxis.actualRange.delta) / 2) - textSize.width : +maxLineWidth + pixel);
                     else     
                         x = pointX + xAxis.x - anchor;
                    if ((xAxis.labelPosition == "inside" && xAxis.tickLinesPosition == 'inside') || ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') && xAxis.labelIntersectAction == 'multiplerows'))
                    {                        
                        y = (opposedPosition) ? (xAxis.y + xAxis.majorTickLines.size + textHeight / 2 + (lineWidth / 2)
                       + (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y - xAxis.majorTickLines.size - (lineWidth / 2)
                      - (labelMaxHeight / (2 * labelMaxRow)) + 5);
                    }                        
                    else if (xAxis.labelPosition == 'inside'&& xAxis.tickLinesPosition!='inside')
                    {                        
                          y = (opposedPosition) ? (xAxis.y + textHeight / 2 + (lineWidth / 2)
                       + (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y - (lineWidth / 2)
                      - (labelMaxHeight / (2 * labelMaxRow)) + 5);                    
                    }
                    else if ((xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition != 'inside') || ((xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && xAxis.labelIntersectAction == 'multiplerows')) {
                        var y = (opposedPosition) ? (xAxis.y - xAxis.majorTickLines.size + textHeight / 2 - (lineWidth / 2)
                            - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + xAxis.majorTickLines.size + (lineWidth / 2)
                            + (labelMaxHeight / (2 * labelMaxRow)) + 5);
                        y = y + ((isScroll ? (opposedPosition ? -this.model.scrollerSize: this.model.scrollerSize): 0 ));
                    }
                    else if (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside')
                    {                      
                            var y = (opposedPosition) ? (xAxis.y + textHeight / 2 - (lineWidth / 2)
                             - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + (lineWidth / 2)
                             + (labelMaxHeight / (2 * labelMaxRow)) + 5);                       
                            y = y + ((isScroll ? (opposedPosition ? -this.model.scrollerSize : this.model.scrollerSize) : 0));
                    }
                   
                    
                    if (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y  : y - xAxis.majorTickLines.size;
                    }
                    else if (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y - xAxis.majorTickLines.size : y - xAxis.majorTickLines.size;
                    }
                    else if (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y : y - xAxis.majorTickLines.size;
                    }

                        var options = {
                            'id': this.svgObject.id + '_' + axisName + '_XLabel_' + i,
                            'x': x,
                            'y': y,
                            'fill': font.color,
                            'font-size': font.size,
                            'font-family': font.fontFamily,
                            'font-style': font.fontStyle,
                            'font-weight': font.fontWeight,
                            'opacity': font.opacity
                        };
              
                    // To set position of edge labels
                        if (xAxis.edgeLabelPlacement) {
                            var position = xAxis.edgeLabelPlacement.toLowerCase();
                            var lastLabel = xAxis.visibleLabels.length - 1;
                            switch (position) {
                                case "none":
                                    break;
                                case "shift":
                                    {
                                        if (i == 0) {  // Shift first label
                                            if (options.x < xAxis.x) {
                                                var diff = xAxis.x - options.x;
                                                options.x = x = options.x + diff;
                                                this.diff = diff;
                                                this.edgeLabel = true;
                                            }
                                        } else if (i == lastLabel)  // Shift last label
                                            if (options.x + textSize.width > xAxis.x + xAxis.width) {
                                                diff = (options.x + textSize.width) - (xAxis.x + xAxis.width);
                                                options.x = x = options.x - diff;
                                                this.diff = diff;
                                                this.edgeLabel = true;
                                            }
                                        break;
                                    }
                                  case "hide": // to hide the edge labels
                                    if ((i == 0 && options.x < xAxis.x) || (i == lastLabel && options.x + textWidth >  chartAreaWidth + chartAreaX)) {
                                        labelText = "";
                                    }
                            }
                        }
                    //To perform rotation             
                         if (labelRotation != 0 && !vmlrendering)
                             this.labelRotation(xAxis, x + anchor, y, options, label, labelRotation);                                                     

                    //To perform labelIntersect Actions none , wrap, hide ,multiplerows
                         if (xAxis.labelIntersectAction || xAxis.enableTrim) {
                            this.diff = ej.util.isNullOrUndefined(this.diff) ? 0 : this.diff;
                            var intersectAction = xAxis.labelIntersectAction.toLowerCase();
                            if (intersectAction == 'none' && !xAxis.enableTrim) { 
                               
                                this.svgRenderer.drawText(options, labelText, gTickEle);
								 //create region to chart axis labels
                                xAxis.visibleLabels[i].region = this.calculateRegion(labelText, font, options, label);
                                this.model.xAxisLabelRegions.push(xAxis.visibleLabels[i].region)
                                if(xAxis.labelPosition=='inside')
                                    $(gTickEle).appendTo(this.gXaxisTickEle);
                                else
                                    $(gTickEle).appendTo(this.gXaxisEle);
                                continue;
                            } 
                            else if (intersectAction == 'wrap' || intersectAction == 'wrapbyword') {
                                
                                var line = 0,
                                    textcoll = this.rowscalculation(labelText.toString(), areaBoundWidth, xAxis, gap, options),
                                    count = textcoll.length,
                                    displayText = '';

                                if (xAxis.labelPosition != 'inside')
                                    options.y = opposedPosition ? options.y + textHeight : options.y - textHeight;
                                else
                                    options.y = opposedPosition ? options.y - textHeight : options.y - textHeight * (count);

                                for (k = 0; k < textcoll.length; k++)
                                {
                                    if (xAxis.labelPosition != 'inside')
                                    {
                                        options.y = opposedPosition ? options.y - (textHeight) : options.y + (textHeight);
                                        this.svgRenderer.drawText(options, textcoll[k], gTickEle);
                                    }
                                    else
                                    {                                        
                                        options.y = opposedPosition ? options.y + (textHeight) : options.y + (textHeight);
                                        this.svgRenderer.drawText(options, textcoll[k], gTickEle);
                                        count -= 1;
                                    }

                                    displayText = displayText + textcoll[k];
                                }
                                
                                labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
                                var textheight = labelSize.height * count;                        
                                bounds = { x: options.x, y: options.y, width: labelSize.width, height: (count) * textheight };
                                xAxis.visibleLabels[i].region = { bounds: bounds, labelText: label.Text, displayText: displayText };
                                xAxis.visibleLabels[i].displayText = displayText;
                                this.model.xAxisLabelRegions.push(xAxis.visibleLabels[i].region)
                                continue;
                            } 
                            else if (intersectAction == 'hide') {
                                var xAxisLabelLength = xAxis.visibleLabels.length - 1;
                                for (var j = 0; j < i; j++) {
                                    // loop to get previous label
                                    var preLabel = xAxis.visibleLabels[j];
                                    var temp = 0;
                                    areaBoundWidth = areaBounds.Width;
                                   
                                    var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                    var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                    var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                    value = (j == 0) ? value + this.diff : value;
                                    var width = (preTextWidth == 0) ? preTextWidth : textWidth;
                                    if ((i != xAxisLabelLength)|| (i == xAxisLabelLength && textWidth <= preTextWidth))
                                        width = width / 2;
                                    if (intersectAction == 'hide' && ((xAxis.isInversed) ? (value < pointX + width) : (value > pointX - width)))
                                        label.displayText = labelText = '';
                                }
                            }
                            else if(intersectAction == 'multiplerows'){ // to perform multipleRows on intersect
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    var xAxisLabelLength = xAxis.visibleLabels.length - 1;
                                    if (i != xAxisLabelLength)
                                        textWidth = textWidth / 2;
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var preLabel = prevLabels[k][l];
                                        var temp = 0;
                                        areaBoundWidth =areaBounds.Width;

                                        var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                        var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                        var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                        if (this.edgeLabel || prevLabels.length==1) {
                                            value = value + this.diff;
                                        }
                                        if ((xAxis.isInversed) ? (value < pointX + textWidth) : (value > pointX - textWidth)) {
                                            if (opposedPosition) {
                                                if (xAxis.labelPosition != 'inside')
                                                    temp = options.y - textHeight;
                                                else
                                                    temp = options.y + textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                            } else {
                                                if (xAxis.labelPosition != 'inside')
                                                    temp = options.y + textHeight;
                                                else
                                                    temp = options.y - textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                              
                                            }
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }                           
                                    }
                                }
                                if (temp == undefined) {
                                    if (prevLabels[0] == undefined)
                                        prevLabels[0] = [];
                                    prevLabels[0].push(label);
                                }
                                else {                                  
                                    if (prevLabels[count] == undefined)
                                        prevLabels[count] = [];
                                    prevLabels[count].push(label);
                                }
                         }         
                        }                  
                    if ((!this.edgeLabel) && !this.zoomed && !xAxis.zoomed && !(labelRotation) && !(labelIntersectAction == "rotate90") && !(labelIntersectAction == "rotate45") && (((pointX + xAxis.x) + (textWidth/2)) > ($(this.svgObject).width()))) {
                        $(options).attr('x', ($(this.svgObject).width() - 2));
                        $(options).attr('text-anchor', 'end');
                    }
					xAxis.visibleLabels[i].y = options.y;
					
					if(this.model.enableCanvasRendering && xAxis.labelPosition == "inside" && (xAxis.labelIntersectAction == "rotate45" || xAxis.labelIntersectAction == "rotate90")){
					options.y = options.y - textWidth;
					}
					
                    this.svgRenderer.drawText(options, labelText, gTickEle);
					 //create region to chart axis labels
                    
					 xAxis.visibleLabels[i].region = this.calculateRegion(labelText, font,options,label);
                    this.model.xAxisLabelRegions.push( xAxis.visibleLabels[i].region)
                }
            }
        }
        $(gTickEle).appendTo(this.gXaxisEle);

    },
	calculateRegion: function (labelText, font, options, label) {
        var labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
        var bounds = { x: options.x, y: options.y, width: labelSize.width, height: labelSize.height };
        var xAxisLabelRegions = { bounds: bounds, trimText: labelText, labelText: label.Text };
        return xAxisLabelRegions;
    },
    rowscalculation: function (labelText, areaBoundWidth,xAxis,gap,options)
    {
        var measureText = ej.EjSvgRender.utils._measureText,
            font = xAxis.font,
            textSize = measureText(labelText, areaBoundWidth, font),
            textWidth = textSize.width,
            anchor = textWidth / 2,
            text = labelText.toString(),
            textLength = text.length,
            textcollection = [],
            i = 0,
            intersectAction = xAxis.labelIntersectAction,
            labelCollection = labelText.split(' '),
            labelCollectionLength = labelCollection.length;

        if (this.edgeLabel) gap = (i == 0) ? gap - this.diff : gap;

        if (textWidth > gap) {
            options.x = options.x + anchor;
            line = 0;
            if (this.edgeLabel) options.x = (i == 0) ? options.x - this.diff : options.x + this.diff;
        }

        if (intersectAction == 'wrap') {
            if (textWidth > gap) {
                for (var w = 1; w <= text.length; w++) {
                    labelText = text.substring(0, w);
                    textWidth = measureText(labelText, areaBoundWidth, font).width;
                    if (textWidth > gap) {
                        line = line + 1; // To find the no of rows splitted
                        labelText = text.substring(0, w - 1);
                        textcollection[i] = labelText;
                        text = text.slice(w - 1, textLength);
                        currentTextCollextion = text.split(' ');
                        text = labelCollection.indexOf(currentTextCollextion[0]) > -1 ? text : '-' + text;
                        var newTextWidth = measureText(labelText, areaBoundWidth, font).width;
                        if (line == 1) options.x = options.x - newTextWidth / 2;
                        i++;
                        w = 0;
                    }
                }
            }
            options.x = (xAxis.valueType == "datetimeCategory" && xAxis.labelPlacement.toLowerCase() == "betweenticks") ? xAxis._gap : options.x;
            textcollection[i] = labelText;
        }
        else {
            var max = 0, word, currentWidth, nextWidth;

            for (var i = 0; i < labelCollectionLength; i++) {
                word = labelCollection[i];
                currentWidth = measureText(word, areaBoundWidth, font).width;
                if (currentWidth < gap && textWidth > gap) {
                    while (i < labelCollectionLength) {
                        currentWidth = measureText(word, areaBoundWidth, font).width;
                        nextWidth = (labelCollection[i + 1]) ? measureText(labelCollection[i + 1], areaBoundWidth, font).width : 0;
                        if ((currentWidth + nextWidth) <= gap && nextWidth > 0) {
                            word = word.concat(' ' + labelCollection[i + 1]);
                            i++;
                        }
                        else {
                            // find the maximum width of the lines
                            max = Math.max(max, currentWidth);
                            break;
                        }
                    }
                    textcollection.push(word);
                }
                else {
                    if (textWidth > gap) {
                        word = ej.EjSvgRender.utils._trimText(word, gap, font);
                        newTextWidth = measureText(word, areaBoundWidth, font).width;
                        max = Math.max(max , newTextWidth)
                        textcollection.push(word);
                    } else {
                        textcollection.push(labelText);
                        break;
                    }
                }
            }
            options.x = options.x - max / 2;
        }
        
        return textcollection;
    },

    labelRotation: function (axis, x, y, options, label, degree, i) {
            var opposedPosition = axis._opposed;
	        // To rotate axis labels
	        var labelText = (label.displayText) ? label.displayText : label.Text;
            var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
            var rotate = 'rotate(' + angle + ',' + (x) + ',' + y + ')';
            $(options).attr('transform', rotate);
            $(options).attr('labelRotation', angle);
            $(options).attr('labelPosition', axis.labelPosition);
            var textElement = this.svgRenderer.createText(options, labelText);
            var diffHeight = Math.ceil(ej.EjSvgRender.utils._measureBounds(textElement, this).height - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height);
            diffHeight = axis._LableMaxWidth.height - diffHeight - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height;
            
        if(axis.labelPosition!='inside')
            var yLocation = (opposedPosition) ? (diffHeight / 2) : (-diffHeight / 2);
        else
            var yLocation = (opposedPosition) ? (-diffHeight / 2) : (diffHeight / 2);

        rotate = 'rotate(' + angle + ',' + (x) + ',' + (y + yLocation) + ')';
        if (this.edgeLabel && degree == 90)               
            y = (i == 0) ? options.y + this.diff : options.y - this.diff;
       
	        $(options).attr({
                'transform' :rotate,
                'y': (y + yLocation)
            });
        },
		
    _drawYAxisLabels: function (axisIndex, yAxis) {
        // method to draw y axis labels
        //if(yAxis.labelPosition=='inside')
        var axisName = (yAxis.name).replace(/[^a-zA-Z0-9]/g, ""),
            gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' + '_' + axisIndex, 'cursor': 'default' }),
            labelSize,
            yAxisLabelRegions, opposedPosition = yAxis._opposed,
            bounds,
            regionX, areaBounds = this.model.m_AreaBounds,
            requireInvertedAxes = this.model.requireInvertedAxes,
         	isScroll = yAxis._isScroll && !(yAxis.scrollbarSettings.pointsLength != null && yAxis.scrollbarSettings.pointsLength < yAxis.maxPointLength),
            yAxisAlignment = yAxis.alignment.toLowerCase(), pixel = 3, y;
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside'))&& !((yAxis.x == (areaBounds.Width + areaBounds.X)) || (yAxis.x == (areaBounds.X))))
        {
            yAxis.x = (!opposedPosition) ? (yAxis.showNextToAxisLine) ? Math.floor(yAxis.x) : Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
        else if (axisIndex != 1 && (!(yAxis.x == (areaBounds.Width + areaBounds.X)) || (yAxis.x == (areaBounds.X))) && (yAxis.labelPosition!='inside' && yAxis.tickLinesPosition=='inside'))
        {
            yAxis._x = Math.floor(yAxis.x);
            yAxis.x = (!opposedPosition) ? !yAxis.showNextToAxisLine ? Math.floor(areaBounds.X) : Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }
        else
        {
			yAxis._x = Math.floor(yAxis.x);//to store the current positons of the labels,before the property showNextToAxisLine as false.
			yAxis.x = (!yAxis.showNextToAxisLine) ? opposedPosition ? Math.floor(areaBounds.X + areaBounds.Width) :  Math.floor(areaBounds.X): Math.floor(yAxis.x);
        }   
      
        for (var i = 0; i < yAxis.visibleLabels.length; i++) {
            var label = yAxis.visibleLabels[i];
			var visibleRange = yAxis.visibleRange;
			if (ej.EjSvgRender.utils._inside(label.Value, visibleRange)) {
			    var pointY = Math.abs(ej.EjSvgRender.utils._getPointXY(label.Value, visibleRange, yAxis.isInversed) * (yAxis.height));
			    var text = ej.EjSvgRender.utils._measureText(label.Text, areaBounds.Height, yAxis.font);
			    var textHeight = text.height;			    
			    var font = yAxis.font;
			    var range = yAxis.visibleRange;
			    var lineWidth = yAxis.axisLine.width;
			    var labelMaxWidth = yAxis._LableMaxWidth;
			    var labelMaxHeight = labelMaxWidth.height;
			    var labelMaxRow = labelMaxWidth.rows;
               var areaBoundWidth = areaBounds.Width;
               var gap = yAxis.maximumLabelWidth;  // width for 4 digits
               var labelText = label.displayText = label.Text;
			    var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, yAxis.font);
			    var textWidth = textSize.width;
			    var font = yAxis.font;
			    var degree = yAxis.labelRotation,maxLineWidth;

			    if(yAxis.labelPosition=='inside' && yAxis.tickLinesPosition=='inside')
			    {
			        var x = (opposedPosition) ? (yAxis.x - (yAxis.majorTickLines.size) - (lineWidth / 2) - 5)
               : (yAxis.x + (lineWidth / 2) + 5 + yAxis.majorTickLines.size);
			        var textanchor = (opposedPosition) ? 'end' : 'start';			       
			    }
			    else if (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')
			    {			        
			        var x = (opposedPosition) ? (yAxis.x - (lineWidth / 2)  - 5)
                : (yAxis.x + (lineWidth / 2) + 5);
			        var textanchor = (opposedPosition) ? 'end' : 'start';			       
			    }
			    else if (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside')
			    {

			        var x = (opposedPosition) ? ((lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5);
			        var textanchor = (opposedPosition) ? 'start' : 'end';
			        x = x - ((isScroll ? (opposedPosition ? -this.model.scrollerSize : this.model.scrollerSize) : 0));
			    }
			    else
			    {
			        var x = (opposedPosition) ? ((yAxis.majorTickLines.size) + (lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5 - yAxis.majorTickLines.size);
			        var textanchor = (opposedPosition) ? 'start' : 'end';			    
			        x = x - ((isScroll ? (opposedPosition ? -this.model.scrollerSize : this.model.scrollerSize) : 0));
			    }
			    if (yAxis.majorGridLines.width > yAxis.majorTickLines.width)
			        maxLineWidth = yAxis.majorGridLines.width;
			    else maxLineWidth = yAxis.majorTickLines.width;
			    if (yAxisAlignment == "far")
			        y = ((-pointY) + (yAxis.y + yAxis.height)) - maxLineWidth - pixel;
			    else if (yAxisAlignment == "near")
			        y = ((-pointY) + (yAxis.y + yAxis.height) + textHeight / 2) + maxLineWidth + pixel;
                else
                    y = ((-pointY) + (yAxis.y + yAxis.height) + textHeight/4);
                    var options = {
                        'id': this.svgObject.id + '_' + axisName + '_YLabel_' + i,
                        'x': x,
                        'y': y,
                        'fill': font.color,
                        'font-size': font.size,
                        'font-family': font.fontFamily,
                        'font-style': font.fontStyle,
                        'font-weight': font.fontWeight,
                        'opacity': font.opacity,
                        'text-anchor': !ej.util.isNullOrUndefined(degree) && degree != 0 ? "middle" : textanchor
                    };
					
                // To set position of edge labels in yAxis
                    if (yAxis.edgeLabelPlacement) {
                        var position = yAxis.edgeLabelPlacement.toLowerCase();
                        var lastLabel = yAxis.visibleLabels.length - 1;

                        switch (position) {
                            case "none":
                                break;
                            case "shift":
                                {
                                    if (i == 0) {  // Shift first label
                                        if (options.y > yAxis.y + yAxis.height) {
                                            var diff = options.y - (yAxis.y + yAxis.height);
                                            options.y = y = options.y - diff;
                                            this.diff = diff;
                                            this.edgeLabel = true;
                                        }
                                    } else if (i == lastLabel) // Shift last label
                                        if (options.y - textHeight/2 < yAxis.y) {
                                            diff = yAxis.y - (options.y - textHeight / 2);
                                            options.y = y = options.y + diff;
                                            this.diff = diff;
                                            this.edgeLabel = true;
                                        }
                                    break;
                                }
                            case "hide": //to hide the edge labels
                                if ((i == 0 && options.y > yAxis.y + yAxis.height) || (i == lastLabel && options.y - textHeight / 2 < areaBounds.Y)) {
                                    labelText = label.Text = "";
                                }
                        }
                    }

			      //To perform rotation         
                        if (!ej.util.isNullOrUndefined(degree) && degree != 0 && !this.svgRenderer.vmlNamespace){
						    if (yAxis.labelPosition!='inside')
                               x = (opposedPosition) ? (x + textWidth/2) : (x - textWidth/2);
                            else
                               x = (opposedPosition) ? (x - textWidth/2) : (x + textWidth/2);
							   
						    var labelText = (label.displayText) ? label.displayText : label.Text;
                            var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
                            var rotate = 'rotate(' + angle + ',' + (x) + ',' + y + ')';
                            $(options).attr('transform', rotate);
			                $(options).attr('labelRotation', angle);
                            var textElement = this.svgRenderer.createText(options, labelText);
                            var diffHeight = Math.ceil(ej.EjSvgRender.utils._measureBounds(textElement, this).width - 
							                           ej.EjSvgRender.utils._measureText(labelText, null, yAxis.font).width);
            
                            if (yAxis.labelPosition!='inside')
                                var yLocation = (opposedPosition) ? (diffHeight / 2) : (-diffHeight / 2);
                            else
                                var yLocation = (opposedPosition) ? (-diffHeight / 2) : (diffHeight / 2);

                           rotate = 'rotate(' + angle + ',' + (x + yLocation) + ',' + (y) + ')';
       
	                        $(options).attr({
                                'transform' :rotate,
                                'x': (x + yLocation)
                            });
						}
				 //Label intersection
                    if (yAxis.labelIntersectAction || yAxis.enableTrim) {
                        if (intersectAction == 'none' && !(yAxis.enableTrim)) {
                            this.svgRenderer.drawText(options, label.Text, gEle);
						     labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
                            if (yAxis.opposedPosition)
                                regionX = options.x;
                            else
                                regionX = options.x - labelSize.width;
                            // yAxis labels region stored 
                            bounds = { x: regionX, y: options.y, width: labelSize.width, height: labelSize.height };
                            yAxisLabelRegions = { bounds: bounds, trimText: labelText, labelText: label.Text };
                            yAxis.visibleLabels[i].region = { bounds: bounds, trimText: labelText, labelText: label.Text };
                            this.model.yAxisLabelRegions.push(yAxisLabelRegions)
                            continue;
                        }
                        else if (yAxis.enableTrim) {
                            if (textWidth > gap && labelText != "") { // check textwidth is greater than gap
                                for (var t = 1; t < labelText.toString().length; t++) {
                                    labelText = label.displayText.toString().substring(0, t - 1) + '...';
                                    textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;
                                    if (textWidth >= gap) {
                                        labelText = labelText.toString().substring(0, t - 1) + '...';                                      
                                        break;
                                    }
                                }
                                label.displayText = labelText;
                            }
                        }
                        var intersectAction = yAxis.labelIntersectAction.toLowerCase();
                        for (var j = 0; j < i; j++) {                           
                            var prevLabel = yAxis.visibleLabels[j];
                            var temp = 0;
                            var prevPoint = Math.abs((ej.EjSvgRender.utils._getPointXY(prevLabel.Value, range, yAxis.isInversed) * (yAxis.height)));
                                var prevTextHeight = ej.EjSvgRender.utils._measureText(prevLabel.Text, areaBounds.Height, font).height;
                                var value = ((prevPoint * -1) + (yAxis.y + yAxis.height) + prevTextHeight/4);
                                value = value + ((yAxis.isInversed) ? prevTextHeight / 2 : -prevTextHeight / 2);
                            if (this.edgeLabel)
                                value = (j == 0) ? value - prevTextHeight / 4 : value;
                                if (intersectAction == 'hide' && prevLabel.Text.toString()!="" && ((yAxis.isInversed) ? value > options.y - textHeight / 4 : value < options.y + textHeight / 4))
                                    labelText=label.Text = '';
                                else if (intersectAction == 'multiplerows' && ((yAxis.isInversed) ? value > options.y - textHeight / 4 : value < options.y + textHeight / 4) && prevLabel.x == options.x) {
                                    if (opposedPosition) {
                                        if(yAxis.labelPosition !="inside")
                                            temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5;  // gap between the rows
                                        else
                                            temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5;                                            
                                        options.x = temp;
                                    }
                                    else {
                                        if (yAxis.labelPosition != "inside")
                                            temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5; // gap between the rows
                                        else
                                            temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5;
                                        options.x = temp;
                                     }
                                }
                            }
                        }

                    yAxis.visibleLabels[i].x = options.x;	
                    this.svgRenderer.drawText(options, labelText, gEle);
                      //create region to chart axis labels
                    labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
                   
                   if (opposedPosition)
                       regionX = options.x;
                    else
                       regionX = options.x - labelSize.width;
                   
          			    // yAxis labels region stored 
                       bounds = { x: regionX, y: options.y, width: labelSize.width, height: labelSize.height };
                       yAxisLabelRegions = { bounds: bounds, trimText: labelText, labelText: label.Text };
					    yAxis.visibleLabels[i].region = { bounds: bounds, trimText: labelText, labelText: label.Text };
                       this.model.yAxisLabelRegions.push(yAxisLabelRegions)
				

            }
        }
		$(gEle).appendTo(this.gYaxisEle);
    },
	//To Trim axis title
	 trimText: function (text, maxLength, ellipsis) {
        maxLength--;
        text = text.substring(0, maxLength - ellipsis.length);
        return text + ellipsis;
    },
	
    _drawXTitle: function (axisIndex, axis) {

        if (axis.title.text != "" && axis.title.text && axis.title.visible) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex,'cursor': 'default'}),
            xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height,
            axisTitleSize = ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font),
            titlesize = axisTitleSize.height / 2,
            opposedPosition = axis._opposed,
			titleText = axis.title.text,
			_LableMaxWidth = axis._LableMaxWidth, 
			textAnchor="middle",
			elementSpacing = this.model.elementSpacing,
            isEnableTrim = axis.title.enableTrim,
            isScroll = (axis._isScroll || (this.model.zooming.enableScrollbar && axis.scrollbarSettings.visible && (axis.zoomFactor < 1 || axis.zoomPosition > 0))),
			axisTitleWidth=axis.title.maximumTitleWidth,
			axisTitlePosition = axis.title.position.toLowerCase(),
		    axisTitleAlignment = axis.title.alignment.toLowerCase(),y,
			tickLinesPosition  = axis.tickLinesPosition.toLowerCase(),
			labelPosition = axis.labelPosition.toLowerCase(),
			m_AreaBounds = this.model.m_AreaBounds,
			axistitleoffset = axis.title.offset,locX,newY,
			scrollerSize = this.model.scrollerSize;
            if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                var maxTitleWidth = (!axisTitleWidth) ? axis.width - (elementSpacing * 2) : axisTitleWidth;
			    while (axisTitleSize.width > maxTitleWidth) {
			        titleText = this.trimText(titleText, titleText.length, "...");
			        axisTitleSize = ej.EjSvgRender.utils._measureText(titleText, (axis.width), axis.title.font);
			        if (titleText == "...") {
			            titleText = "";
			            break;
			        }
			    }
			}			  
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);

            var y = elementSpacing + _LableMaxWidth.height + titlesize + axis._multiLevelLabelHeight + axis.majorTickLines.size + axis.axisLine.width;

           if ((tickLinesPosition == 'inside' && labelPosition == 'inside') && (axisIndex == 0 || (axis.y == (m_AreaBounds.Height + m_AreaBounds.Y)) || (axis.y == (m_AreaBounds.Y))))
                 y = (axisTitlePosition == "inside") ? elementSpacing + _LableMaxWidth.height + titlesize + axis._multiLevelLabelHeight + axis.majorTickLines.size + axis.axisLine.width : elementSpacing + titlesize + axis.axisLine.width;
            else if ((axisIndex == 0 || (axis.y == (m_AreaBounds.Width + m_AreaBounds.X)) || (axis.y == (m_AreaBounds.X))) && (tickLinesPosition != 'inside' && labelPosition == 'inside'))
                 y = (axisTitlePosition == "inside") ? elementSpacing + _LableMaxWidth.height+titlesize + axis._multiLevelLabelHeight + axis.axisLine.width : elementSpacing +axis.majorTickLines.size+ axis.axisLine.width;
            else if ((axisIndex == 0 || (axis.y == (m_AreaBounds.Width + m_AreaBounds.X)) || (axis.y == (m_AreaBounds.X))) && (tickLinesPosition == 'inside' && labelPosition != 'inside'))
                y = (axisTitlePosition == "inside") ? elementSpacing + titlesize + axis.majorTickLines.size+ axis.axisLine.width :  elementSpacing + _LableMaxWidth.height + titlesize + axis._multiLevelLabelHeight + axis.axisLine.width;
            else if ((tickLinesPosition != 'inside' && labelPosition != 'inside') || (axisIndex != 1))
                 y = (axisTitlePosition == "inside") ?  elementSpacing  : elementSpacing + _LableMaxWidth.height + titlesize + axis.majorTickLines.size + axis.axisLine.width + axis._multiLevelLabelHeight;
            else if ((tickLinesPosition != 'inside' && labelPosition == 'inside' && !axis.showNextToAxisLine))
                y = (axisTitlePosition == "inside") ? elementSpacing + _LableMaxWidth.height + titlesize + axis._multiLevelLabelHeight + axis.axisLine.width :elementSpacing + titlesize + axis.majorTickLines.size + axis.axisLine.width + elementSpacing + elementSpacing;
            if (axisTitleAlignment == "far") {
                locX = axis.x + axis.width + axistitleoffset;
				textAnchor="end";
            }
            else if (axisTitleAlignment == "near") {
				locX = axis.x + axistitleoffset;
				textAnchor="start";
            }
			else
				locX = (axis.x + axis.width / 2) + axistitleoffset ;
				
            commonEventArgs.data = { title: axis.title.text, location: { x: locX, y: y }, axes: axis };
            this._trigger("axesTitleRendering", commonEventArgs);
			
			if (axisTitlePosition == "inside")
				  newY = ((opposedPosition) ? (tickLinesPosition == "inside" && labelPosition == "outside" ? axis.y + commonEventArgs.data.location.y - (isScroll ? scrollerSize : 0) - axis.majorTickLines.size 
												: (axis.y + commonEventArgs.data.location.y - (isScroll ? scrollerSize : 0)))
											: (tickLinesPosition=="outside" && labelPosition=="inside" ? (isScroll ? scrollerSize : 0) + (axis.y - commonEventArgs.data.location.y)  + axis.majorTickLines.size 
												:(isScroll ? scrollerSize : 0) + (axis.y - commonEventArgs.data.location.y))) + axisTitleSize.height / 3.5;
			else
				 newY = ((opposedPosition) ? (tickLinesPosition == "inside" && labelPosition == "outside" ? axis.y - commonEventArgs.data.location.y - (isScroll ? scrollerSize : 0) - axis.majorTickLines.size 
												: (axis.y - commonEventArgs.data.location.y - (isScroll ? scrollerSize : 0)))
											: (tickLinesPosition == "outside" && labelPosition == "inside" ? (isScroll ? scrollerSize : 0) + (commonEventArgs.data.location.y + axis.y) - axis.majorTickLines.size 
												: (isScroll ? scrollerSize : 0) + (commonEventArgs.data.location.y + axis.y))) + axisTitleSize.height / 3.5;
				 
            var options = {
                'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex,
                'x': commonEventArgs.data.location.x,
                'y': newY,
                'fill': axis.title.font.color,
                'font-size': axis.title.font.size,
                'font-family': axis.title.font.fontFamily,
                'font-style': axis.title.font.fontStyle,
                'font-weight': axis.title.font.fontWeight,
                'opacity': axis.title.font.opacity,
                'text-anchor': textAnchor
            };
            if (this.model.enableCanvasRendering) {
                var matched = jQuery.uaMatch(navigator.userAgent);
                var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
                var browser = matched.browser.toLowerCase();
                 if (browser != "msie" && !isIE11)
                     options.y = !opposedPosition ? options.y + (titlesize / 2) + (browser == "mozilla" ? 1 : -2) : options.y + (titlesize / 2) + 1; // text dominant baseline for canvas
            }
            if ($(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).length > 0)
                $(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).attr(options);
            else if (!isEnableTrim)
                this.svgRenderer.drawText(options, commonEventArgs.data.title, gEle);
		    else if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
				var x = (textAnchor == "middle") ? options.x - (axisTitleSize.width) / 2 : (textAnchor == "end" ? options.x - axisTitleSize.width : options.x);
                var bounds = { X: x, Y: options.y, Width: axisTitleSize.width, Height: axisTitleSize.height };
                var xAxisTitleRegion = { Bounds: bounds, trimText: titleText, labelText: axis.title.text };
                this.model.xAxisTitleRegion.push(xAxisTitleRegion)
                this.svgRenderer.drawText(options, titleText, gEle);
            }
			
			if (!this.model.enableCanvasRendering){
				var borderY = (textAnchor == "start" ? 0 : (textAnchor== "end" ? axisTitleSize.width : axisTitleSize.width/2));
				var borderOptions = {
					'id': this.svgObject.id + '_XAxisTitleBorder' + '_' + axisIndex,
					'x': commonEventArgs.data.location.x - borderY - elementSpacing,
					'y': newY - axisTitleSize.height + (axisTitleSize.height/4),
					'width': axisTitleSize.width + (2 * elementSpacing),
					'height': axisTitleSize.height,
					'fill': "transparent",
					'class': "e-xaxistitleborder"
				};
				this.svgRenderer.drawRect(borderOptions, gEle);
			}
            $(gEle).appendTo(this.gXaxisEle);
			}
        },
    _drawYTitle: function (axisIndex, yAxis) {

        if (yAxis.title.text != "" && yAxis.title.text && yAxis.title.visible) {
            var opposedPosition = yAxis._opposed,
			elementSpacing = this.model.elementSpacing,
			textAnchor = "middle",
			isTransposed = this.model.series.isTransposed,
			majorTickLinesize = yAxis.majorTickLines.size,
			axisTitlePosition = yAxis.title.position.toLowerCase(),
			_LableMaxWidth = yAxis._LableMaxWidth,
			m_AreaBounds = this.model.m_AreaBounds,x,
			axisLine = yAxis.axisLine,
			tickLinesPosition = yAxis.tickLinesPosition.toLowerCase(),
			labelPosition = yAxis.labelPosition.toLowerCase(),
			 titleText = yAxis.title.text,
            isEnableTrim = yAxis.title.enableTrim,
            requireInvertedAxes = this.model.requireInvertedAxes,y,
            isScroll = yAxis._isScroll,
            axisTitleWidth = yAxis.title.maximumTitleWidth,
		    axistitleoffset= yAxis.title.offset,
		    axisTitleAlignment = yAxis.title.alignment.toLowerCase(),
            gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTitleGroup' + '_' + axisIndex,'cursor': 'default' });//Group id and text element id should not be same
            var titleSize = ej.EjSvgRender.utils._measureText(yAxis.title.text, (yAxis.height), yAxis.title.font);
             if ((tickLinesPosition == 'inside' && labelPosition == 'inside') && (axisIndex == 1 || (yAxis.x == (m_AreaBounds.Width + m_AreaBounds.X)) || (yAxis.x == (m_AreaBounds.X)))) {
                if (axisTitlePosition == "inside")
                    x = (opposedPosition) ? (yAxis.x - ((3 * elementSpacing) + _LableMaxWidth.width +yAxis._multiLevelLabelHeight+ majorTickLinesize + axisLine.width)) : yAxis.x + ((3 * elementSpacing) + _LableMaxWidth.width + yAxis._multiLevelLabelHeight + majorTickLinesize + axisLine.width);
                else
                    x = (opposedPosition) ? yAxis.x + ((3 * elementSpacing) + axisLine.width - 5) : yAxis.x - ((3 * elementSpacing) + axisLine.width);
            } else if ((axisIndex == 1 || (yAxis.x == (m_AreaBounds.Width + m_AreaBounds.X)) || (yAxis.x == (m_AreaBounds.X))) && (tickLinesPosition != 'inside' && labelPosition == 'inside')) {
                if (axisTitlePosition == "inside")
                    x = (opposedPosition) ? (yAxis.x - ((3 * elementSpacing) + _LableMaxWidth.width + axis._multiLevelLabelHeight + axisLine.width)) : yAxis.x + ((3 * elementSpacing) + _LableMaxWidth.width + yAxis._multiLevelLabelHeight + axisLine.width);
                else
                    x = (opposedPosition) ? yAxis.x + ((3 * elementSpacing) + majorTickLinesize + axisLine.width) : yAxis.x - ((3 * elementSpacing) + majorTickLinesize + axisLine.width);
            } else if ((axisIndex == 1 || (yAxis.x == (m_AreaBounds.Width + m_AreaBounds.X)) || (yAxis.x == (m_AreaBounds.X))) && (tickLinesPosition == 'inside' && labelPosition != 'inside')) {
                if (axisTitlePosition == "inside")
                    x = (opposedPosition) ? (yAxis.x - ((3 * elementSpacing) + majorTickLinesize + axisLine.width)) : yAxis.x + ((3 * elementSpacing) + majorTickLinesize + axisLine.width) +
                        (!yAxis.showNextToAxisLine ? -(elementSpacing) : labelPosition == "inside" ? -yAxis._multiLevelLabelHeight : yAxis._multiLevelLabelHeight);
               
                else
                    x = (opposedPosition) ? yAxis.x + ((3 * elementSpacing) + yAxis._multiLevelLabelHeight + _LableMaxWidth.width + axisLine.width) : yAxis.x - ((3 * elementSpacing) + _LableMaxWidth.width + axisLine.width + yAxis._multiLevelLabelHeight);
            } else if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (axisIndex != 1)) {
                if (axisTitlePosition == "inside")
                    x = (opposedPosition) ? (yAxis.x - (elementSpacing + axisLine.width + majorTickLinesize)) : yAxis.x + (elementSpacing + (!yAxis.showNextToAxisLine ? _LableMaxWidth.width : 0) + axisLine.width + majorTickLinesize);
                else
                    x = (opposedPosition) ? yAxis.x + ((3 * elementSpacing) + yAxis._multiLevelLabelHeight + _LableMaxWidth.width + majorTickLinesize + axisLine.width) : yAxis.x - ((3 * elementSpacing) + _LableMaxWidth.width + yAxis._multiLevelLabelHeight + majorTickLinesize + axisLine.width);
            }
            var axisTitleSize = ej.EjSvgRender.utils._measureText(titleText, (yAxis.height), yAxis.title.font);
            if (isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                var maxTitleWidth = (!axisTitleWidth)? yAxis.height - (elementSpacing * 2) : axisTitleWidth;                
                while (axisTitleSize.width > maxTitleWidth) {
                    titleText = this.trimText(titleText, titleText.length, "...");
                    axisTitleSize = ej.EjSvgRender.utils._measureText(titleText, (yAxis.height), yAxis.title.font);
                    if (titleText == "..."){
                        titleText = "";
                        break;
                    }
                        
                }
            }
            x = (yAxis.opposedPosition) ? x : x - (isScroll ? this.model.scrollerSize : 0)
            if (document.documentMode === 8)
                x = (!opposedPosition) ? (x - axisTitleSize.width / 2 + elementSpacing) : (x + axisTitleSize.width / 2 - elementSpacing);
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { title: yAxis.title.text, location: { x: titleSize.width / 2, y: 0 }, axes: yAxis };
            this._trigger("axesTitleRendering", commonEventArgs);
			y = commonEventArgs.data.location.y + (yAxis.y + yAxis.height) + (((yAxis.height) / 2) * -1);
			
			if (axisTitleAlignment == "near") {
				y = (yAxis.y + yAxis.height) - axistitleoffset ;
			    textAnchor = opposedPosition||isTransposed ? "end":"start"; 
            }
            else if (axisTitleAlignment == "far") {
				 y = yAxis.y  - axistitleoffset ;
				 textAnchor = opposedPosition||isTransposed  ? "start":"end";
            }
			else if (axisTitleAlignment == "center")
				 y = yAxis.y + (yAxis.height/2) - axistitleoffset ;
	
			var labelRotation = (opposedPosition) ? 90 : -90;

            var options = {
                'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex,
                'x': x,
                'y': y,
                'fill': yAxis.title.font.color,
				'labelRotation': labelRotation,
                'transform': 'rotate(' + labelRotation + ',' + (x) + ',' + y + ')',
                'font-size': yAxis.title.font.size,
                'font-family': yAxis.title.font.fontFamily,
                'font-style': yAxis.title.font.fontStyle,
                'font-weight': yAxis.title.font.fontWeight,
                'opacity': yAxis.title.font.opacity,
                'text-anchor': textAnchor,
                'dominant-baseline': 'middle'
            };
             if (this.model.enableCanvasRendering) {
                var matched = jQuery.uaMatch(navigator.userAgent);
                var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
                var browser = matched.browser.toLowerCase();
                 if (browser != "msie" && !isIE11)
                    options.x = !opposedPosition ? options.x + (titleSize.height / 4) + (browser == "mozilla" ? 1 : -2) : options.x - (titleSize.height / 4) + 1; // text dominant baseline for canvas
            }
            if ($(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).length > 0)
                $(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).attr(options);
            else if (!isEnableTrim)
                this.svgRenderer.drawText(options, commonEventArgs.data.title, gEle);
			else if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                 if (!this.svgRenderer.vmlNamespace)
                    var bounds = { X: options.x - (axisTitleSize.height / 2), Y: options.y + (axisTitleSize.width / 2), Width: (axisTitleSize.height)/2, Height: axisTitleSize.width };
                 else
                    var bounds = { X: options.x - (axisTitleSize.width)/2, Y: options.y + (axisTitleSize.height / 2), Width: (axisTitleSize.width), Height: axisTitleSize.height };
                var yAxisTitleRegion = { Bounds: bounds, trimText: titleText, labelText: yAxis.title.text };
                this.model.yAxisTitleRegion.push(yAxisTitleRegion)
                this.svgRenderer.drawText(options, titleText, gEle);
            }	
			
			if (!this.model.enableCanvasRendering){
				var borderY = (textAnchor == "start" ? (axisTitleSize.width) : (textAnchor== "end" ? 0 : axisTitleSize.width/2));
				var borderOptions = {
					'id': this.svgObject.id + '_YAxisTitleBorder' + '_' + axisIndex,
					'x': options.x - axisTitleSize.height + (axisTitleSize.height/2),
					'y': options.y - borderY - elementSpacing,
					'width': axisTitleSize.height,
					'height': axisTitleSize.width + (2 * elementSpacing),
					'fill': "transparent",
					'class': "e-yaxistitleborder"
				};
				this.svgRenderer.drawRect(borderOptions, gEle);
			}
            $(gEle).appendTo(this.gYaxisEle);
			}
		}
    },
ej.EjStripline.prototype = {
    _drawStripline: function (axis, stripLine,axisIndex, stripLineIndex) {

        var start = (stripLine.start < axis.range.min) ? axis.range.min : stripLine.start;
        var end = (stripLine.end > axis.range.max) ? axis.range.max : stripLine.end;
        var x = 0, height = 0, angle = 0;
        var y = 0, pointYstart = 0, yHeight = 0;
        var width = 0, widthValue = 0, textX = 0, textY = 0;
        if (stripLine.visible) {
		     this.model._stripeline = true;
            var striplineid = (stripLine.zIndex == 'over') ? this.gStriplineOver : this.gStriplineBehind;
            striplineid = ej.util.isNullOrUndefined(striplineid) ? "" : striplineid;
            var isVisible = true;
            if (axis.orientation.toLowerCase() == "horizontal") {
                var pointstart = 0;

                if (stripLine.startFromAxis) {

                    var point = (ej.util.isNullOrUndefined(stripLine.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripLine.offset;
                    pointstart = (((point - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);

                    widthValue = (((point + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width));

                } else {
                    if (ej.util.isNullOrUndefined(start) && (ej.util.isNullOrUndefined(end) || ej.util.isNullOrUndefined(stripLine.width))) {
                        isVisible = false;
                    } else {
                        start = (jQuery.type(start) == "date") ? (start).getTime() : start;
                        if (!ej.util.isNullOrUndefined(end) && (jQuery.type(end) == "date")) {
                            end = (end).getTime();
                        }
                        pointstart = (ej.EjSvgRender.utils._getPointXY(start, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                        var pointend = (ej.EjSvgRender.utils._getPointXY(end, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                        var widthMax = (ej.EjSvgRender.utils._getPointXY((start + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width)), axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x;

                        widthValue = (axis.isInversed ? (pointend < widthMax) : (pointend > widthMax)) ? Math.abs(pointstart - pointend) : Math.abs(pointstart - widthMax);
                        
                    }
                }
                width = widthValue;
                height = this.chart.model.m_AreaBounds.Height;
                x = (axis.isInversed) ? (pointstart - widthValue) : pointstart;
                y = this.chart.model.m_AreaBounds.Y;
                angle = -90;
                var textWidth = ej.EjSvgRender.utils._measureText(stripLine.text, null, stripLine.font).height;
                if (stripLine.textAlignment == 'middletop') {
                    textX = x+textWidth;
                    textY = Math.abs(y + height / 2);
                } else if (stripLine.textAlignment == 'middlecenter') {
                    textX = (Math.floor(x + width / 2) + (textWidth / 3));
                    textY = Math.abs(y + height / 2);
                } else if (stripLine.textAlignment == 'middlebottom') {
                    textX = x + width - (textWidth);
                    textY = Math.abs(y + height / 2);
                }


            } else {
                if (stripLine.startFromAxis) {
                    var pointY = (ej.util.isNullOrUndefined(stripLine.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripLine.offset;
                    pointYstart = (axis.y + axis.height) - (((pointY - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                    widthValue = (((pointY + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                    yHeight = widthValue;
                } else {

                    if (ej.util.isNullOrUndefined(start) && (ej.util.isNullOrUndefined(end) || ej.util.isNullOrUndefined(stripLine.width))) {
                        isVisible = false;
                    } else {
                        start = (jQuery.type(start) == "date") ? (start).getTime() : start;
                        if (!ej.util.isNullOrUndefined(end) && (jQuery.type(end) == "date")) {
                            end = (end).getTime();
                        }
                        pointYstart = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY(start, axis.visibleRange, axis.isInversed) * (axis.height));
                        var pointYend = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((ej.util.isNullOrUndefined(end) ? 0 : end), axis.visibleRange, axis.isInversed) * (axis.height));
                        var widthYValue = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((start + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width)), axis.visibleRange, axis.isInversed) * (axis.height));
                        yHeight = (axis.isInversed ? (pointYend > widthYValue) : (pointYend < widthYValue)) ? Math.abs(pointYstart - pointYend) : Math.abs(pointYstart - widthYValue);
                    }
                }
                height = yHeight;
                x = this.chart.model.m_AreaBounds.X;
                width = this.chart.model.m_AreaBounds.Width;
                y = pointYstart + ((axis.isInversed) ? 0 : -yHeight);
                var textHeight = ej.EjSvgRender.utils._measureText(stripLine.text, width, stripLine.font).height;

                if (stripLine.textAlignment == 'middletop') {
                    textX = x + width / 2;
                    textY = y + textHeight;
                } else if (stripLine.textAlignment == 'middlecenter') {
                    textX = x + width / 2;
                    textY = (Math.abs(y + (height / 2)) + (textHeight / 3));
                } else if (stripLine.textAlignment == 'middlebottom') {
                    textX = x + width / 2;
                    textY = (y + height) - textHeight;

                }


            }
            if (isVisible) {
                if (!ej.util.isNullOrUndefined(stripLine.imageUrl)) {
                    var imgOptions = {
                        'height': height,
                        'width': width,
                        'href': stripLine.imageUrl,
                        'x': x,
                        'y': y,
                        'id': striplineid.id + '_backImage',
                        'visibility': 'visible',
                        'preserveAspectRatio': 'none'
                    };

                    this.svgRenderer.drawImage(imgOptions, striplineid);
                } else {
                    var options = {
                        'id': striplineid.id + '_striplineRect_'+ axisIndex+ '_'+ stripLineIndex,
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': stripLine.color,
                        'opacity': stripLine.opacity,
                        'stroke-width': stripLine.borderWidth,
                        'stroke':(stripLine.borderWidth==0)?"transparent": stripLine.borderColor
                    };
                    var textOptions = {
                        'id': striplineid.id + '_striplineRectText_'+ axisIndex+ '_'+ stripLineIndex,
                        'x': textX,
                        'y': textY,
                        'fill': stripLine.font.color,
                        'transform': 'rotate(' + angle + ',' + (textX) + ',' + textY + ')',
                        'labelRotation': angle,
                        'font-size': stripLine.font.size,
                        'font-family': stripLine.font.fontFamily,
                        'font-style': stripLine.font.fontStyle,
                        'font-weight': stripLine.font.fontWeight,
						'opacity': stripLine.font.opacity,
                        'text-anchor': 'middle'
                    };

                    var bounds = { X: x, Y: y, Width: width, Height: height };
                    var isStripline = stripLine.zIndex == "over" ? true : false;
                    ej.EjSvgRender.utils.AddRegion(this.chart, bounds, isStripline)

                    if (this.chart.model.enableCanvasRendering) {
                        var ctx = this.chart.svgObject.getContext("2d");
                        var clipBounds = axis.orientation.toLowerCase() == 'horizontal' ? { X: axis.x, Y: y, Width: axis.width, Height: this.chart.model.m_AreaBounds.Height }
                        : { X: x, Y: axis.y, Width: this.chart.model.m_AreaBounds.Width, Height: axis.height };
                        ctx.save();
                        ctx.beginPath();
                        ctx.rect(clipBounds.X, clipBounds.Y, clipBounds.Width, clipBounds.Height);
                        ctx.clip();
                        this.svgRenderer.drawRect(options, striplineid);
                        this.svgRenderer.drawText(textOptions, stripLine.text, striplineid);
                        ctx.closePath();
                        ctx.restore();
                    }
                    else {
                        this.svgRenderer.drawRect(options, striplineid);
                        this.svgRenderer.drawText(textOptions, stripLine.text, striplineid);
                    }
                }
            }

        }
    }
};
})(jQuery);
ej.EjLegendRender = function (chartobj) {

    this.chartObj = chartobj;
};


(function ($) {
ej.EjLegendRender.prototype =
{
		sumofArray: function(array) {
            var sum = 0,
            length = array.length;
            for (var i = 0; i < length; i++)
                sum += array[i];
            return sum;
		},

        legendRectSpace: function(svgBounds,divBounds) {
            var width = svgBounds.Width - 10;
            var height = svgBounds.Height - 10;
            var isVisible = false;
            if (Math.ceil(divBounds.Width) >= width && Math.ceil(divBounds.Height) >= height)
                isVisible = true;
            return isVisible;
        },

		drawLegendTitle: function () {
            //draw legend title

            var chart = this.chartObj,
                chartModel = chart.model,
                legend = chartModel.legend,
                legendBounds = chartModel.LegendBounds,
                elementSpacing = chartModel.elementSpacing,
                legendTitle = legend.title,
                measureText = ej.EjSvgRender.utils._measureText,
                font = legendTitle.font,
                legendBoundsWidth = legendBounds.Width,
                titleSize = measureText(legendTitle.text, null, font),
                titleWidth = titleSize.width,
                titleHeight = titleSize.height,
                textAlignment = legendTitle.textAlignment.toLowerCase(),
                startX = 0,
                startY = elementSpacing + (titleHeight / 2);

            switch (textAlignment) {
                case 'far':
                    startX = legendBoundsWidth - titleWidth - startX;
                    break;
                case 'center':
                    startX = legendBoundsWidth / 2 - (titleWidth) / 2;
                    break;
            }

            if (startX < 0) {
                startX = 0;
                legendBoundsWidth = titleWidth;
            }
            if (legendBoundsWidth < titleWidth + startX)
                legendBoundsWidth = titleWidth + startX;

            var options = {
                'id': chart.svgObject.id + '_LegendTitleText',
                'x': startX,
                'y': startY,
                'fill': font.color,
                'font-size': font.size,
                'font-style': font.fontStyle,
                'font-family': font.fontFamily,
                'font-weight': font.fontWeight,
                'text-anchor': 'start',
                'lgndCtx': true
            };

            chart.svgRenderer.drawText(options, legendTitle.text, this.chartObj.gLegendEle);

        },
    
        drawLegendItem: function (legendItem, x, y) {
            var chart = this.chartObj,
                chartModel = chart.model,
                legend = chartModel.legend,
                textOverflow = legend.textOverflow.toLowerCase(),
                textMaxWidth = legend.textWidth,
                legendBounds = chartModel.LegendBounds,
                elementSpacing = chartModel.elementSpacing,
                legendBorder = legend.border.width,
                measureText = ej.EjSvgRender.utils._measureText,
                svgRender = ej.EjSvgRender,
                chartSvgRender = chart.svgRenderer,
                legendFont = legendItem.Font,
                symbolShape = chartModel.symbolShape,
                isCanvas = chartModel.enableCanvasRendering,
                shapeSize = legend.itemStyle,
                textHeight = measureText(legendItem.Text, legendBounds.Width, legendFont).height,
                location = {
                    startX: x + shapeSize.width / 2,
                    startY: (y + (textHeight > shapeSize.height ? textHeight : shapeSize.height) / 2)
                },
                pointIndex = legendItem.PointIndex,
                seriesIndex = legendItem.SeriesIndex,
                index = pointIndex ? pointIndex : seriesIndex,
                series = chartModel.series[seriesIndex],
                svgObject = chart.svgObject,
                legendStyle = legendItem.LegendStyle,
                matched = jQuery.uaMatch(navigator.userAgent),
                isIE11 = !!navigator.userAgent.match(/Trident\/7\./),
                color,itemInfo, data,style,
                seriesType = series.type.toLowerCase(),
                symbolName, symbol,textinfo,legendTextRegion,textLength;

            for (name in symbolShape) {
                if (legendItem.Shape.toLowerCase() == name.toLowerCase()) {
                    if (name.toLowerCase() == "seriestype")
                        symbolName = "SeriesType";
                    else
                        symbolName = name;
                    break;
                }
            }

            if (symbolName == "SeriesType") {
                if (seriesType == "radar" || seriesType == "polar") {
                    symbol = series.drawType;
                    symbolName = symbol.capitalizeFirstString();
                }
                if (seriesType == "scatter") {
                    symbol = series.marker.shape;
                    symbolName = symbol.capitalizeFirstString();
                }
                if (legendItem.drawType)// // TrendLine drawType assigned to legend shape
                    symbolName = "SeriesType";
                if (chart.vmlRendering && (seriesType == "pie" || seriesType == "doughnut" || seriesType == "pieofpie"))
					symbolName = "Circle";          // VML legend shape for pie & doughnut
            }

            if (symbolName == "None") {
                if (seriesType == "pie" || seriesType == "doughnut" || seriesType == "pieofpie")
                    symbolName = "Circle";
                else
                    symbolName = "Rectangle";
            }

            index = (ej.util.isNullOrUndefined(pointIndex)) ? seriesIndex : pointIndex;
            chart.gLegendItemEle = chartSvgRender.createGroup({ 'id': svgObject.id + '_Legend' + index, 'cursor': 'pointer' });
            legendItem.CommonEventArgs.data.gLegendItemEle = chart.gLegendItemEle;
            data = $.extend(true, {}, legendItem.CommonEventArgs.data);
            data.model = chart.model;
            style = data.style;
            if (!legendItem.CommonEventArgs.cancel) {
                startLocation = svgRender.chartSymbol["_draw" + symbolName](location,style, data, chart.gLegendItemEle);
                if (legendStyle.Color && legendStyle.Color.toString().toLowerCase() !== 'gray' && isCanvas) // for canvas gradient color
                    color = legendFont.color;
                else if (legendStyle.Color && legendStyle.Color.toLowerCase() !== 'gray') // for svg
                    color = legendFont.color;
                else
                    color = 'gray';

                itemInfo = legendItem;
                textLength = itemInfo.Text.length;
                if (legend.shape == "seriestype" || legend.shape == "seriesType")
                    x += elementSpacing / 2;
                var options = {
                    'id': this.chartObj.svgObject.id + '_LegendItemText' + index,
                    'x': shapeSize.width + x + elementSpacing / 2,
                    'y': location.startY,
                    'fill': color,
                    'font-size': legendFont.size,
                    'font-style': legendFont.fontStyle,
                    'font-family': legendFont.fontFamily,
                    'font-weight': legendFont.fontWeight,
                    'text-anchor': 'start',
                    'lgndCtx': true
                };

                options.y = (location.startY + (textHeight / 4));
                
                if (textOverflow == "wrap" || textOverflow == "wrapandtrim") {
                    options.y = options.y - textHeight;
                    for (k = 0; k < textLength; k++) {
                        options.y = options.y + (textHeight);
                        chartSvgRender.drawText(options, itemInfo.Text[k], chart.gLegendItemEle);
                        if (textOverflow == "wrapandtrim") {
                            textinfo = { x: options.x + legendBounds.X, y: options.y + legendBounds.Y, height: textHeight * (textLength), width: textMaxWidth };
                            legendTextRegion = { bounds: textinfo, trimText: itemInfo.Text, labelText: itemInfo.displayText };
                            chartModel.legendTextRegion.push(legendTextRegion);
                        }
                    }
                }
                else {
					if (textOverflow == "trim") {
                        textinfo = { x: options.x + legendBounds.X, y: options.y + legendBounds.Y, height: textHeight + elementSpacing, width: textMaxWidth };
                        legendTextRegion = { bounds: textinfo, trimText: itemInfo.Text, labelText: itemInfo.displayText };
                        chartModel.legendTextRegion.push(legendTextRegion);
                    }
                    chartSvgRender.drawText(options, itemInfo.Text, chart.gLegendItemEle);
                }
                chartSvgRender.append(chart.gLegendItemEle, chart.gLegendEle);
                legendbound = {
                    X: legendBounds.X + legendBorder,
                    Y: legendBounds.Y + legendBorder
                };
                
                x = (startLocation) ? startLocation : x;
                var itembound = { 
                    X: (x), Y: (y), _Width: legendItem.Bounds._Width, Width: legendItem.Bounds.Width, Height: legendItem.Bounds.Height
                };
                var bounds = { LegendBound: legendbound, ItemBound: itembound };
                var legendRegion = {
                    LegendItem: legendItem,
                    Location: location, SymbolShape: symbolName, Style: legendItem.CommonEventArgs.data.style, Bounds: bounds
                };
                chartModel.legendRegion.push(legendRegion);
            }
        },

        drawLegend: function (params) {
            var chart = this.chartObj,
                chartModel = chart.model,
                legend = chartModel.legend,
                legendViewerBounds = chartModel.LegendViewerBounds,
                legendBounds = chartModel.LegendBounds,
                actualBounds = chartModel.LegendActualBounds,
                AreaType = chartModel.AreaType,
                legendPosition = legend.position.toLowerCase(),
                alignment = legend.alignment.toLowerCase(),
                elementSpacing = chartModel.elementSpacing,
                svgObjectWidth = chartModel.svgWidth,
                svgObjectHeight = chartModel.svgHeight,
                title = chartModel.title,
                titleFontHeight = title.font.size,
                subTitle = title.subTitle,
                subTitleFontHeight = title.subTitle.font.size,
                legendTitle = legend.title,
                legendBorder = legend.border.width,
                measureText = ej.EjSvgRender.utils._measureText,
                margin = chartModel.margin,
                borderSize = chartModel.border.width,
                legendCollection = chartModel.legendCollection,
                collectionLength = legendCollection.length,
                svgWidth = svgObjectWidth - ((borderSize * 2)),
                svgHeight = svgObjectHeight - ((borderSize * 2)),
                itemPadding = legend.itemPadding > 0 ? legend.itemPadding : 0,
                padding = 20,
                hPadding = 10,
                vPadding = 10,
                modelsubTitleHeight = subTitle.text == ""|| !subTitle.visible || !title.visible ? 0 : measureText(subTitle.text, svgWidth - margin.left - margin.right, subTitle.font).height + elementSpacing,
                titleLocation = chartModel._titleLocation ? chartModel._titleLocation.Y : 0 + modelsubTitleHeight,
                legendTitleBounds = measureText(legendTitle.text, null, legendTitle.font),
                legendTitleHeight = legendTitleBounds.height,
                legendTitleWidth = legendTitleBounds.width,
                rowDefinitions = chartModel._rowDefinitions,
                maxWidth = 0, startX, startY,
                columnDefinitions = chartModel._columnDefinitions,
                vSizeFar = this.sumofArray(rowDefinitions.farSizes),
                vSizeNear = this.sumofArray(rowDefinitions.nearSizes),
                hSizeFar = this.sumofArray(columnDefinitions.farSizes),
                hSizeNear = this.sumofArray(columnDefinitions.nearSizes),
                isScrolling = false,
                areaBounds = chartModel.m_AreaBounds,
                isScroll = legend.enableScrollbar,
                legendContainer = $(chart.legendContainer),
                leftScroll = $("#ScrollerParent_" + 'vertical' + "_" + chart._id).length > 0 ? 18 : 0,
                bottomScroll = $("#ScrollerParent_" + 'horizontal' + "_" + chart._id).length > 0 ? 18 : 0,
                legendSvgContainer = $(chart.legendSvgContainer);
            var textBorderConstant = 1.2,
                titleBorderSpacing = 10,
                subTitleBorderSpacing = 10,
                title = chartModel.title,
                subTitle = chartModel.title.subTitle,
                titleTextHeight = title.visible ? parseInt(titleFontHeight) * textBorderConstant : 0,
                subTitleTextHeight = subTitle.visible ? parseInt(subTitleFontHeight) * textBorderConstant : 0,
                titleTextHeight = (chartModel.titleWrapTextCollection && chartModel.titleWrapTextCollection.length > 1) ? chartModel._titleLocation._height * chartModel.titleWrapTextCollection.length - 1 : titleTextHeight,
                subTitleTextHeight = (chartModel.subTitleWrapTextCollection && chartModel.subTitleWrapTextCollection.length > 1) ? chartModel._subTitleLocation.size.height : subTitleTextHeight;
                if ((chartModel.legend.border) && (!subTitle.text)) {
                    subTitleTextHeight = 0;
                    subTitleBorderSpacing = 5;
                }
            
            if (legendPosition == 'right' || legendPosition == 'left') {
                hPadding = 10;
            } else {
                vPadding = isScroll ? svgHeight > 200 ? 10 : svgHeight > 100 ? 5 : 0 : 10;
            }
            legendViewerBounds.Height += legendTitleHeight,
            legendBounds.Height += legendTitleHeight;            
            if (legend.visible) {                
                // draw legend.
                if (legendPosition != 'custom') {
                    if (AreaType == "cartesianaxes") {
                        switch (legendPosition) {
                            case "bottom":
                                legendBounds.Y = areaBounds.Y + areaBounds.Height + hSizeNear - (elementSpacing / 2) + bottomScroll;
                                break;
                            case "top":
                                legendBounds.Y = titleLocation == 0 ? borderSize + elementSpacing : (titleTextHeight + titleBorderSpacing + subTitleTextHeight + subTitleBorderSpacing + elementSpacing);
                                break;
                            case "right":
                                legendBounds.X = areaBounds.X + areaBounds.Width + vSizeFar + elementSpacing * 2;
                                break;
                            case "left":
                                legendBounds.X = areaBounds.X - vSizeNear - actualBounds.Width - (elementSpacing / 2) - (legendBorder * 2) - leftScroll;
                                break;
                        }
                    } else {
                        switch (legendPosition) {
                            case "bottom":
                                legendBounds.Y = svgHeight - (actualBounds.Height + (legendBorder)) - (elementSpacing * 2);
                                break;
                            case "top":
                                legendBounds.Y = titleLocation == 0 ? borderSize + elementSpacing : (titleTextHeight + titleBorderSpacing + subTitleTextHeight + subTitleBorderSpacing + elementSpacing);
                                break;
                            case "right":
                                legendBounds.X = svgWidth - actualBounds.Width - (elementSpacing * 2);
                                break;
                            case "left":
                                legendBounds.X = borderSize + (elementSpacing * 2);
                                break;
                        }
                    }                    
                    //Avoid axis and legend overlapping during crossing
                    if (!chartModel.enable3D && AreaType == "cartesianaxes" && params._crossAxisOverlap!=null) {                        						
                        for (var i = 0, len = chartModel._axes.length; i < len; i++) {
                            var axis = chartModel._axes[i], axisBounds = params.axes[axis.name]._bounds;
                                var hor = axis.orientation.toLowerCase() === 'horizontal';
                                if (legendPosition=="top" && hor && axis._opposed && (axis.y - axisBounds < legendBounds.Y + legendBounds.Height))
                                    legendBounds.Y -= legendBounds.Y + legendBounds.Height - axis.y + axisBounds;
                                else if (legendPosition=="bottom" && hor && !axis._opposed && (axis.y + axisBounds > legendBounds.Y))                                    
                                    legendBounds.Y += axis.y + axisBounds - legendBounds.Y;
                                else if (legendPosition=="left" && !hor && axis._opposed && (axis.x - axisBounds < legendBounds.X + legendBounds.Width))
                                    legendBounds.X += axis.x + axisBounds - legendBounds.X;                                    
                                else if (legendPosition=="right" && !hor && !axis._opposed && (axis.x + axisBounds > legendBounds.X))
                                    legendBounds.X -= legendBounds.X + legendBounds.Width - axis.x + axisBounds;                            
                        }
                    }
                    if (legendPosition == 'left' || legendPosition == 'right') {
                        if (isScroll) {
                            switch (alignment) {
                                case "center":
                                    legendBounds.Y = areaBounds.Height / 2 - (actualBounds.Height / 2) + areaBounds.Y - legendBorder;
                                    break;
                                case "near":
                                    legendBounds.Y = areaBounds.Y;
                                    break;
                                case "far":
                                    legendBounds.Y = areaBounds.Y + areaBounds.Height - actualBounds.Height - (legendBorder * 2);
                                    break;
                            }
                        } else {
                            switch (alignment) {
                                case "center":
                                    legendBounds.Y = (svgHeight / 2) - ((actualBounds.Height + legendBorder * 2) / 2) + (elementSpacing / 2);
                                    break;
                                case "near":
                                    legendBounds.Y = borderSize + (elementSpacing * 2);
                                    break;
                                case "far":
                                    legendBounds.Y = svgHeight - (actualBounds.Height + (legendBorder)) - (elementSpacing * 2);
                                    break;
                            }
                        }

                    } else {
                        switch (alignment) {
                            case "center":
                                legendBounds.X = (svgWidth / 2) - ((actualBounds.Width + legendBorder * 2) / 2) + (elementSpacing / 2);
                                break;
                            case "near":
                                legendBounds.X = borderSize + (elementSpacing * 2);
                                break;
                            case "far":
                                legendBounds.X = svgWidth - (actualBounds.Width + (legendBorder)) - (elementSpacing * 2);
                                break;
                        }
                    }
                } else {
                    legendBounds.Y = (legend.location.y < svgHeight) ? legend.location.y : 0;
                    legendBounds.X = (legend.location.x < svgWidth) ? legend.location.x : 0;
                }
                if (isScroll) {
                    if (legendPosition == 'right' || legendPosition == 'left') {
                        if (legendBounds.Y < areaBounds.Y || actualBounds.Height + (legendBorder * 2) >= areaBounds.Height) {
                            var legendHeight = (legendBorder * 2);
                            for (var k = 0; k < collectionLength; k++) {
                                legendHeight += legendCollection[k].Bounds.Height + itemPadding;
                                if (legendHeight > areaBounds.Height) {
                                    actualBounds.Height = areaBounds.Height - (legendBorder * 2);
                                    legendBounds.Y = areaBounds.Y;
                                    break;
                                }
                            }
                        }
                    }
                    else {
                        if ((legendBorder * 2 + actualBounds.Width) >= svgWidth) {
                            var legendWidth = (legendBorder * 2);
                            for (var k = 0; k < collectionLength; k++) {
                                legendWidth += legendCollection[k].Bounds.Width + itemPadding;
                                if (legendWidth > svgWidth) {
                                    actualBounds.Width = svgWidth - (legendBorder * 2) - actualBounds.vScrollSize - (borderSize * 2);
                                    legendBounds.X = (svgWidth / 2) - ((actualBounds.Width + legendBorder * 2) / 2) + (elementSpacing / 2);
                                    break;
                                }

                            }
                        }
                    }
                }
                    
                //Set height and width to legend Canvas element
                chart.legendSvgContainer.height = legendBounds.Height;
                chart.legendSvgContainer.width = Math.max(legendTitleWidth, legendBounds.Width);
                currentX = startX = hPadding;
                currentY = startY = vPadding;
                if (legendTitle.text != "" && legendTitle.text) {
                    this.drawLegendTitle();
                    currentY = startY = vPadding + legendTitleHeight;
                }
                for (var k = 0; k < collectionLength; k++) {
                    if ((legend.rowCount < legend.columnCount || legend.rowCount == legend.columnCount )&& (legendPosition == 'top'|| legendPosition == 'bottom'|| legendPosition == 'custom' )) {
                        if ((currentX + legendCollection[k].Bounds.Width) > legendBounds.Width + startX) {
                            currentX = hPadding;
                            currentY += legendCollection[k].Bounds.Height + itemPadding;
                        }
                        this.drawLegendItem(legendCollection[k], currentX, (currentY));
                        currentX += legendCollection[k].Bounds.Width + itemPadding;
                    }
                    else {
                        if (((currentY + legendCollection[k].Bounds.Height + itemPadding) + legendTitleHeight + borderSize > legendBounds.Height + startY)) {
                            currentY = startY;
                            currentX += maxWidth + (itemPadding);
                        }
                        this.drawLegendItem(legendCollection[k], (currentX), (currentY));
                        currentY += legendCollection[k].Bounds.Height + itemPadding;
                        maxWidth = Math.max(maxWidth, legendCollection[k].Bounds.Width);
                    }
                }
                var legendVal  = chart.legendContainer[0].offsetLeft;
                var chartOffsetVal = $(chart.element).offset();
                var offsetVal = (chart.vmlRendering) ? (legendVal <= 0) ? legendBounds.X : (legendVal  - chartOffsetVal.left) : 
                                ($(chart.svgObject).offset().left - chartOffsetVal.left);
                var legnTx = (chart.vmlRendering) ? offsetVal : legendBounds.X + (offsetVal <= 0 ? 0 : offsetVal);
                legendContainer.attr('style','');
				if (legend._ejScroller)
				    $('#' + legendContainer[0].id).ejScroller("instance").destroy();
                legendContainer.show();
                legendContainer.css({ "position": "absolute", "background": legend.background, "left": legnTx, "top": legendBounds.Y, "width": actualBounds.Width, "height": actualBounds.Height, "border-width": legend.border.width, "border-color": legend.border.color, "border-style": "solid" });
				legendContainer.addClass("e-legendborder");
                legendSvgContainer.css({ "height": legendBounds.Height, "width": legendBounds.Width });
                legendBounds.Height = legendBounds.Height - (padding - (vPadding * 2));
                var isVisible = this.legendRectSpace(legendBounds, actualBounds);
                if (isScroll) {
                    if (legend._ejScroller) {
                        if (!isVisible && (legendBounds.Width > Math.ceil(actualBounds.Width) || legendBounds.Height > Math.ceil(actualBounds.Height))) {
                            $('#' + legendContainer[0].id).ejScroller({ scrollTop: chart.scrolltop, scrollLeft: chart.scrollleft,height: actualBounds.Height, width: actualBounds.Width });
                            isScrolling = true;
                            if (legendBounds.Height > Math.ceil(actualBounds.Height)) {
                                if (actualBounds.vScrollSize == 0) {
                                    vScrollSize = legendContainer[0].offsetWidth + (legendBorder * 5) - legendContainer[0].clientWidth;
                                    legendContainer.css({ "width": actualBounds.Width + vScrollSize });
                                }
                            }
                        } else {
                            legend._ejScroller = false;
                        }
                    }
                    else {
                        if (!isVisible && legendBounds.Width > Math.ceil(actualBounds.Width) && legendBounds.Height > Math.ceil(actualBounds.Height)) {
                            legendContainer.css({ "overflow": "scroll" });
                            isScrolling = true;
                        } else {
                            if (!isVisible && legendBounds.Height > Math.ceil(actualBounds.Height) && actualBounds.vScrollSize >= 0) {
                                legendContainer[0].style.overflowY = "scroll";
                                legendContainer[0].style.overflowX = "hidden";
                                if (actualBounds.vScrollSize == 0) {
                                    vScrollSize = legendContainer[0].offsetWidth - (legendBorder * 2) - legendContainer[0].clientWidth;
                                    legendContainer.css({ "width": actualBounds.Width + vScrollSize });
                                }
                                isScrolling = true;
                            }
                            if (!isVisible && legendBounds.Width > Math.ceil(actualBounds.Width) && actualBounds.hScrollSize >= 0) {
                                legendContainer[0].style.overflowX = "scroll";
                                legendContainer[0].style.overflowY = "hidden";
                                if (actualBounds.hScrollSize == 0) {
                                    hScrollSize = legendContainer[0].offsetHeight - (legendBorder * 2) - legendContainer[0].clientHeight;
                                    legendContainer.css({ "height": actualBounds.Height + hScrollSize });
                                }
                                isScrolling = true;
                            }
                        }
                    }
                }
                
                chart.svgRenderer.append(chart.gLegendEle, chart.legendSvgContainer);
                if (!isScrolling) {
                    legendContainer.css({ "width": actualBounds.Width - actualBounds.vScrollSize, "height": actualBounds.Height - actualBounds.hScrollSize });
                }
				if (!legend._ejScroller) {
					if (chart.scrolltop != undefined)
						legendContainer.scrollTop(chart.scrolltop);
					if (chart.scrollleft != undefined)
						legendContainer.scrollLeft(chart.scrollleft);
				}                

            }
            else
                legendContainer.hide();
        }
    };
})(jQuery);;

ej.seriesTypes = {};

ej.EjSeriesRender = function () {

};
ej.indicatorTypes = {};

ej.EjIndicatorRender = function () {

};
ej.trendlineTypes = {};

ej.EjTrendLineRenderer = function () {

};
(function ($) {
    ej.EjTrendLineRenderer.prototype = {
        draw: function (trendline, series, sender) {
            this.chartObj = sender;
            var trendIndex = $.inArray(trendline, series.trendlines);
			trendline._isTransposed = sender.model.requireInvertedAxes;
            trendline.xAxis = series.xAxis;
            trendline.yAxis = series.yAxis;
            var type = trendline.type.toLowerCase();
            if (type == "linear" || type == "movingaverage")
                this.calculateLineDirection(trendline, series, trendIndex);
            else
                this.calculateSplineDirection(trendline, series, trendIndex);

            return false;
        },        
        calculateSplineDirection: function (trendline, series, trendIndex) {
            //var series = trendline;
            var spDirection = "";
            var splinesb = ej.EjSvgRender.utils._getStringBuilder();
            var startingPoint = true;
            var yIndex = 0;
            var visiblePoints = trendline.points;
            var ySpline = ej.EjSeriesRender.prototype.naturalSpline(visiblePoints);
            var firstPoint = null;
            var secondPoint = null;
            var firstIndex = -1;
            var canvasX = 0;
            var canvasY = 0;
            var point, pointIndex;
            var length = visiblePoints.length;
            for (var i = 0; i < length; i++) {
                pointIndex = i;
                secondPoint = visiblePoints[i];
                    if (firstPoint != null) {
                        var controlPoint1 = null;
                        var controlPoint2 = null;
                        var data = ej.EjSeriesRender.prototype.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                        controlPoint1 = data["controlPoint1"];
                        controlPoint2 = data["controlPoint2"];
                        var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, trendline);
                        var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, trendline);
                        var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, trendline);
                        var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, trendline);
                        var chartObj = this.chartObj;
                        if (startingPoint) {
                            splinesb.append("M" + " " + (pt1.X) + " " + (pt1.Y) + " " + "C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                            startingPoint = false;
                        }
                        else
                            splinesb.append("C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                    }
                    firstPoint = secondPoint;
                    firstIndex = pointIndex;
            }
            spDirection = splinesb.toString();
            this._drawLinePath(trendline, series, spDirection, trendIndex);

            return false;
        },
        calculateLineDirection: function (trendline, series, trendIndex) {
            var canvasX = 0;
            var canvasY = 0;
            var point;           
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();

            var firstPoint = false;
            var secondPoint;
            var length = trendline.points.length;
            for (var i = 0; i < length; i++) {
                secondPoint = trendline.points[i];
                    point = ej.EjSvgRender.utils._getPoint(secondPoint, trendline);
                    if (firstPoint)
                        sb.append("L" + " " + (point.X) + " " + ((point.Y)) + " ");
                    else {
                        sb.append("M" + " " + (point.X) + " " + ((point.Y)) + " ");
                        sb.append("L" + " " + (point.X) + " " + ((point.Y)) + " ");
                        firstPoint = true;
                    }
            }
            lDirection = sb.toString();
            this._drawLinePath(trendline, series, lDirection, trendIndex);
            return false;
        },
        _drawLinePath: function (trendline, series, lDirection, trendIndex) {
            var translate = [];
            translate[0] = trendline.xAxis.x;
            translate[1] = trendline.yAxis.y;

            if (lDirection != "") {
                var options = {
                    'id': this.chartObj.svgObject.id + "_" + trendline.seriesIndex + "_" + trendIndex + "_" + trendline.name.replace(/[^a-zA-Z ]/g, ""),
                    'fill': 'none',
                    'stroke-dasharray': trendline.dashArray,
                    'stroke-width': trendline.width,
                    'stroke': trendline.fill,
                    'opacity': trendline.opacity,
                    'd': lDirection
                };
                trendline.id = options.id;
                this.chartObj.svgRenderer.drawPath(options, series.gTrendGroupEle, translate);
            }
            this.chartObj.svgRenderer.append(series.gTrendGroupEle, this.chartObj.gTrendlinesGroupEle[trendline.seriesIndex]);

            return false;
        },

        calculateTrendLineSegment: function (series, trendline, axis, sender) {
            var chartmodel = sender.model;
            var axis, points = $.extend([], series.points);
            var trendlineType = trendline.type.toLowerCase();
            trendline.backwardForecast = trendline.backwardForecast < 0 ? 0 : trendline.backwardForecast;
            trendline.forwardForecast = trendline.forwardForecast < 0 ? 0 : trendline.forwardForecast;
            points = points.sort(function(a,b) {return (a.xValue> b.xValue) ? 1 : ((b.xValue > a.xValue) ? -1 : 0);});
            switch (trendlineType) {
                case "linear":
                    this.updateTrendSource(points, trendline);
                    this.calculateLinearTrendline(series, trendline, axis);
                    break;
                case "exponential":
                    this.updateExponentialTrendSource(points, trendline);
                    this.calculateExponentialTrendline(series, trendline, axis);
                    break;
                case "logarithmic":
                    this.updateLogarithmicTrendSource(points, trendline);
                    this.calculateLogarithmicTrendline(series, trendline, axis);
                    break;
                case "power":
                    this.updatePowerTrendSource(points, trendline);
                    this.calculatePowerTrendline(series, trendline, axis);
                    break;
                case "polynomial":
				    var length = series.points.length;
				    trendline.polynomialOrder = length <= trendline.polynomialOrder ? length : trendline.polynomialOrder;
                    trendline.polynomialOrder = trendline.polynomialOrder < 2 ? 2 : trendline.polynomialOrder;
                    trendline.polynomialOrder = trendline.polynomialOrder > 6 ? 6 : trendline.polynomialOrder;
                    if (length > 1) {
                        this.updateTrendSource(points, trendline);
                        this.calculatePolynomialTrendline(series, trendline, axis);
                    }
                    break;
                case "movingaverage":
                    this.updateMovingAverageTrendSource(points, trendline);
                    this.calculateMovingAverageTrendline(series, trendline, axis);
                    break;
            }
            if (series.points.length > 1)
                this.calculateTrendLineRange(trendline, axis);
            return false;
        },
        updateTrendSource: function (points, trendline) {
            var len = points.length, point, xValue;

            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                point = points[i];
                if (!point.isEmpty) {
                    trendline.xPoints.push(point.xValue);
                    xValue = typeof (point.xValue) === 'number' ? point.xValue : Date.parse(point.xValue);
                    trendline.xValues.push(xValue);
                    trendline.yValues.push(point.y);
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculateLinearTrendline: function (series, trendline, axis) {            
            
            var count = trendline.xValues.length;
            var x1, x2;
            trendline.points = [];

            if (count > 1) {
                if (axis._valueType.toLowerCase() == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    x2 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                } else {
                     x1 = (trendline.xPoints[0] - trendline.backwardForecast);
                     x2 = (trendline.xPoints[count - 1] + trendline.forwardForecast );
                }

                //trendline linear segment  y = ax + b;
                var y1 = trendline.slope * x1 + trendline._intercept;
                var y2 = trendline.slope * x2 + trendline._intercept;
                trendline.points.push({ x: x1, xValue: x1, y: y1, YValues: [], visible: true });
                trendline.points.push({ x: x2, xValue: x2, y: y2, YValues: [], visible: true });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
            }
            return false;
        },
       
        //// Calculate Trendline range
        calculateTrendLineRange: function (trendline, axis) {

            var length = trendline.points.length;
            var y;
            trendline.minY = trendline.points[0].y;
            trendline.maxY = trendline.points[length-1].y;
            if (!axis.setRange) {
                if (trendline.backwardForecast > 0 && axis._valueType != "category" && length > 1) {
                    axis.range.min = axis.range.min > trendline.points[0].x ? trendline.points[0].x : axis.range.min;
                    axis.visibleRange.min = axis.range.min;
                }
                if (trendline.forwardForecast > 0 && length > 1) {
                    axis.range.max = axis.range.max < trendline.points[length - 1].x ? trendline.points[length - 1].x : axis.range.max;
                    axis.visibleRange.max = axis.range.max;
                }
            }
            for (var i = 0; i < length; i++) {
                y = trendline.points[i].y;
                trendline.minY = Math.min(trendline.minY, y);
                trendline.maxY = Math.max(trendline.maxY, y);
            }
            return false;
        },

        /// Calculate Sum of x and y values
        calculateSumXAndYValue: function (trendline) {
            var points = trendline.points,
            N = trendline.xPoints.length,
            type = trendline.type.toLowerCase(),
            Slope, Intercept, SX = 0, SY = 0,
            SXX = 0, SXY = 0, SYY = 0, i,
            SumProduct = 0, X = trendline.xValues, Y = trendline.yValues;

            for (i = 0; i < N; i++) {
                SX = SX + X[i];
                SY = SY + Y[i];
                SumProduct = SumProduct + (X[i] * Y[i]);
                SXY = SXY + X[i] * Y[i];
                SXX = SXX + X[i] * X[i];
                SYY = SYY + Y[i] * Y[i];
            }
            if (!ej.util.isNullOrUndefined(trendline.intercept) && (type == "linear" || type == "exponential" || type == "polynomial")) { //Set intercept value externally
                switch (type) {
                    case "polynomial":
                        Slope = (((SumProduct) - (Math.log(trendline.intercept) * SX)) / SXX) * 3;
                        break;
                    case "linear":
                        Slope = ((SumProduct) - (trendline.intercept * SX)) / SXX;
                        break;
                    case "exponential":
                        Slope = ((SumProduct) - (Math.log(trendline.intercept) * SX)) / SXX;
                        break;

                }
                trendline._intercept = trendline.intercept;
            }
            else {
                Slope = ((N * SXY) - (SX * SY)) / ((N * SXX) - (SX * SX));
                if (type == "exponential" || type == "power")
                    Intercept = Math.exp((SY - (Slope * SX)) / N);
                else
                    Intercept = (SY - (Slope * SX)) / N;

                trendline._intercept = Intercept;
            }
            trendline.slope = Slope;
            return false;
        },

        ////Calculate forward backward Forecast points for dateTime axis
        _increaseDateTimeInterval: function (date, interval, intervalType) {
        var result = new Date(date);
        if (intervalType.toLowerCase() == "days") {
            result.setDate(date.getDate() + interval);
        } else if (intervalType.toLowerCase() == "hours") {
            result.setHours(date.getHours() + interval);
        } else if (intervalType.toLowerCase() == "milliseconds") {
            result.setMilliseconds(date.getMilliseconds() + interval);
        } else if (intervalType.toLowerCase() == "seconds") {
            result.setSeconds(date.getSeconds() + interval);
        } else if (intervalType.toLowerCase() == "minutes") {
            result.setMinutes(date.getMinutes() + interval);
        } else if (intervalType.toLowerCase() == "months") {
            result.setMonth(date.getMonth() + interval);
        } else if (intervalType.toLowerCase() == "years") {
            result.setYear(date.getFullYear() + interval);
        }
        return result;
        }

    },

    ej.ejLinear = ejExtendClass(ej.EjTrendLineRenderer);
    ej.trendlineTypes.linear = ej.ejLinear;

    ej.Exponential = ejExtendClass(ej.EjTrendLineRenderer, {

        updateExponentialTrendSource: function (points, trendline) {
            var len = points.length, point, xValue;
            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                point = points[i];
                if (!point.isEmpty) {
                    trendline.xPoints.push(point.xValue);
                    xValue = typeof (point.xValue) === 'number' ? point.xValue : Date.parse(point.xValue);
                    trendline.xValues.push(xValue);
                    trendline.yValues.push(Math.log(point.y));
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculateExponentialTrendline: function (series, trendline, axis) {
            var count = trendline.xValues.length;
            var x1, x2, x3;
            var X1, X2, X3;
            
            var midPoint = Math.round((count / 2));
            trendline.points = [];
            
            if (count > 1) {
                if (axis._valueType.toLowerCase() == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    X1 = x1.getTime();
                    x2 = (trendline.xPoints[midPoint - 1]);
                    X2 = x2.getTime();
                    x3 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                    X3 = x3.getTime();
                } else {
                    x1 = X1 = (trendline.xPoints[0] - trendline.backwardForecast);
                    x2 = X2 = (trendline.xPoints[midPoint-1]);
                    x3 = X3 =(trendline.xPoints[count - 1] + trendline.forwardForecast);
                }

                //trendline exponential segment  (Intercept * Math.Exp(Slope * xValue));
                var y1 = (trendline._intercept * Math.exp(trendline.slope * x1));
                var y2 = (trendline._intercept * Math.exp(trendline.slope * x2));
                var y3 = (trendline._intercept * Math.exp(trendline.slope * x3));
                trendline.points.push({ x: x1, xValue: X1, y: y1, YValues: [], visible: true });
                trendline.points.push({ x: x2, xValue: X2, y: y2, YValues: [], visible: true });
                trendline.points.push({ x: x3, xValue: X3, y: y3, YValues: [], visible: true });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
                trendline.points[2].YValues[0] = trendline.points[2].y;
            }
            return false;
        }
    });
    ej.trendlineTypes.exponential = ej.Exponential;

    ej.Logarithmic = ejExtendClass(ej.EjTrendLineRenderer, {

        updateLogarithmicTrendSource: function (points, trendline) {
            var len = points.length, point, xValue;
            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                point = points[i];
                if (!point.isEmpty) {
                    trendline.xPoints.push(point.xValue);
                    xValue = typeof (point.xValue) === 'number' ? point.xValue : Date.parse(point.xValue);
                    trendline.xValues.push(Math.log(xValue));
                    trendline.yValues.push(point.y);
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculateLogarithmicTrendline: function (series, trendline, axis) {
            var count = trendline.xValues.length;
            var x1, x2, x3;
            var X1, X2, X3;
            var midPoint = Math.round((count / 2));
            trendline.points = [];

            if (count > 1) {
                if (axis._valueType.toLowerCase() == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    X1 = x1.getTime();
                    x2 = (trendline.xPoints[midPoint - 1]);
                    X2 = x2.getTime();
                    x3 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                    X3 = x3.getTime();
                } else {
                    x1 = X1 = (trendline.xPoints[0] - trendline.backwardForecast);
                    x2 = X2 = (trendline.xPoints[midPoint-1]);
                    x3 = X3 = (trendline.xPoints[count - 1] + trendline.forwardForecast);
                }

                //trendline logarithmic segment  Intercept + Slope * Math.Log(xValue);
                var y1 = trendline._intercept + (trendline.slope * Math.log(x1));
                var y2 = trendline._intercept + (trendline.slope * Math.log(x2));
                var y3 = trendline._intercept + (trendline.slope * Math.log(x3));
                trendline.points.push({ x: x1, xValue: X1, y: y1, YValues: [], visible: true });
                trendline.points.push({ x: x2, xValue: X2, y: y2, YValues: [], visible: true });
                trendline.points.push({ x: x3, xValue: X3, y: y3, YValues: [], visible: true });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
                trendline.points[2].YValues[0] = trendline.points[2].y;
            }
            return false;
        }
    });
    ej.trendlineTypes.logarithmic = ej.Logarithmic;

    ej.Power = ejExtendClass(ej.EjTrendLineRenderer, {

        updatePowerTrendSource: function (points, trendline) {
            var len = points.length, point, xValue;
            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                point = points[i];
                if (!point.isEmpty) {
                    trendline.xPoints.push(point.xValue);
                    xValue = typeof (point.xValue) === 'number' ? point.xValue : Date.parse(point.xValue);
                    trendline.xValues.push(Math.log(xValue));
                    trendline.yValues.push(Math.log(point.y));
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculatePowerTrendline: function (series, trendline, axis) {
            var count = trendline.xValues.length;
            var x1, x2, x3;
            var X1, X2, X3;
            var midPoint = Math.round((count / 2));
            trendline.points = [];

            if (count > 1) {
                if (axis._valueType.toLowerCase() == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    X1 = x1.getTime();
                    x2 = (trendline.xPoints[midPoint - 1]);
                    X2 = x2.getTime();
                    x3 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                    X3 = x3.getTime();
                } else {
                    var p1 = (trendline.xPoints[0] - trendline.backwardForecast);
                    x1 = X1 = p1 > -1 ? p1 : 0;
                    x2 = X2 = (trendline.xPoints[midPoint - 1]);
                    x3 = X3 = (trendline.xPoints[count - 1] + trendline.forwardForecast);
                }

                //trendline power segment  (Intercept * Math.Pow(xValue, Slope))
                var y1 = trendline._intercept * Math.pow(x1, trendline.slope);
                var y2 = trendline._intercept * Math.pow(x2, trendline.slope);
                var y3 = trendline._intercept * Math.pow(x3, trendline.slope);
                trendline.points.push({ x: x1, xValue: X1, y: y1, YValues: [], visible: true });
                trendline.points.push({ x: x2, xValue: X2, y: y2, YValues: [], visible: true });
                trendline.points.push({ x: x3, xValue: X3, y: y3, YValues: [], visible: true });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
                trendline.points[2].YValues[0] = trendline.points[2].y;
            }
            return false;
        }
    });
    ej.trendlineTypes.power = ej.Power;

    ej.Polynomial = ejExtendClass(ej.EjTrendLineRenderer, {

        //// Calculate Polynomial Trendline with order
        calculatePolynomialTrendline: function (series, trendline, axis) {
            var power = trendline.polynomialOrder;

            // Calculate sum of y datapoints 1 X power matrix
            trendline.polynomialSlopes = new Array(power + 1);
            var xLength = trendline.xValues.length;
            for (var index1 = 0; index1 < xLength; index1++)
            {
                var num2 = trendline.xValues[index1];
                var yval = trendline.yValues[index1];
                
                for (var index2 = 0; index2 <= power; ++index2) {
                    if (!trendline.polynomialSlopes[index2]) trendline.polynomialSlopes[index2] = 0;
                    trendline.polynomialSlopes[index2] += Math.pow(num2, index2) * yval;
                }
            }

            // Calculate sum matrix of x datapoints
            var numArray = new Array(1 + 2 * power);
            var matrixOfA = new Array(power + 1);
            for (var i = 0; i < (power + 1); i++)
                matrixOfA[i] = new Array(3);
            var num1 = 0;
            for (var index1 = 0; index1 < xLength; ++index1) {
                var num2 = 1.0;
                var d = trendline.xValues[index1];
                for (var index2 = 0; index2 < numArray.length; ++index2) {
                    if (!numArray[index2]) numArray[index2] = 0;
                    numArray[index2] += num2;
                    num2 *= d;
                    ++num1;
                }   
            }

            for (var index1 = 0; index1 <= power; ++index1) {
                for (var index2 = 0; index2 <= power; ++index2)
                    matrixOfA[index1][index2] = numArray[index1 + index2];
            }

            //Calculation Gauss jordan eliminiation value of a and b matrix
            if (!this.gaussJordanEliminiation(matrixOfA, trendline.polynomialSlopes))
                trendline.polynomialSlopes = null;

            //Create segments methods
            this.createPolynomialSegments(series, axis, trendline);
            return false;
        },
        createPolynomialSegments: function (series, axis, trendline) {
            var polynomialSlopes = trendline.polynomialSlopes;
            if (polynomialSlopes != null) {
                var count = trendline.xValues.length, midPoint = Math.round((count / 2)), x1 = 1, X, length = polynomialSlopes.length;
                var valueType = axis._valueType.toLowerCase();

                trendline.points = [];
                for (var i = 1; i <= length; i++) {                   
                    var x, y;
                    if (i == 1) {
                        if (valueType == "datetime") {
                            x = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                            X = x.getTime();
                        }
                        else
                            x = X = (trendline.xPoints[0] - trendline.backwardForecast);
                        var y = this.getPolynomialYValue(polynomialSlopes, X);
                    }
                    else if (i == polynomialSlopes.length) {
                        if (valueType == "datetime") {
                            x = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                            X = x.getTime();
                        }
                        else
                            x = X = (trendline.xPoints[count - 1] + trendline.forwardForecast);

                        var y = this.getPolynomialYValue(polynomialSlopes, X);
                    } else {
                        x1 += (count + trendline.forwardForecast) / polynomialSlopes.length;
                        if (valueType == "category") {
                            x = X = trendline.xPoints[0] + (x1 - 1);

                        }
                        else {
                            x = X = trendline.xPoints[parseInt(x1) - 1];
                            if (valueType == "datetime" && count > x1)
                                X = x.getTime();
                        }                        
                        var y = this.getPolynomialYValue(polynomialSlopes, X);
                    }
                    if (i == 1 || i == polynomialSlopes.length || valueType == "category" || count > x1) {
                        trendline.points.push({ x: x, xValue: X, y: y, YValues: [], visible: true });
                        trendline.points[trendline.points.length - 1].YValues[0] = trendline.points[trendline.points.length - 1].y;
                    }
                }
            }
            return false;
        },

        getPolynomialYValue: function(slopes, x){
            var sum=0;
            for (var i = 0; i < slopes.length; i++) {
                sum += slopes[i] * Math.pow(x, i);
            }
            return sum;
        },        

        gaussJordanEliminiation: function (matrixOfA, polynomialSlopes) {
            var length = matrixOfA.length;
            var numArray1 = new Array(length);
            var numArray2 = new Array(length);
            var numArray3 = new Array(length);
            for (var index = 0; index < length; ++index)
                numArray3[index] = 0;
            for (var index1 = 0; index1 < length; ++index1)
            {
                var num1 = 0.0;
                var index2 = 0;
                var index3 = 0;
                for (var index4 = 0; index4 < length; ++index4)
                {
                    if (numArray3[index4] != 1)
                    {
                        for (var index5 = 0; index5 < length; ++index5)
                        {
                            if (numArray3[index5] == 0 && Math.abs(matrixOfA[index4][index5]) >= num1)
                            {
                                num1 = Math.abs(matrixOfA[index4][index5]);
                                index2 = index4;
                                index3 = index5;
                            }
                        }
                    }
                }
                ++numArray3[index3];
                if (index2 != index3)
                {
                    for (var index4 = 0; index4 < length; ++index4)
                    {
                        var num2 = matrixOfA[index2, index4];
                        matrixOfA[index2][index4] = matrixOfA[index3][index4];
                        matrixOfA[index3][index4] = num2;
                    }
                    var num3 = polynomialSlopes[index2];
                    polynomialSlopes[index2] = polynomialSlopes[index3];
                    polynomialSlopes[index3] = num3;
                }
                numArray2[index1] = index2;
                numArray1[index1] = index3;
                if (matrixOfA[index3][index3] == 0.0)
                    return false;
                var num4 = 1.0 / matrixOfA[index3][index3];
                matrixOfA[index3][index3] = 1.0;
               
                for (var index4 = 0; index4 < length; ++index4)
                    matrixOfA[index3][index4] *= num4;
                
                polynomialSlopes[index3] *= num4;
                
                for (var index4 = 0; index4 < length; ++index4)
                {
                    if (index4 != index3)
                    {
                        var num2 = matrixOfA[index4][index3];
                        matrixOfA[index4][index3] = 0.0;
                        for (var index5 = 0; index5 < length; ++index5)
                            matrixOfA[index4][index5] -= matrixOfA[index3][index5] * num2;
                        polynomialSlopes[index4] -= polynomialSlopes[index3] * num2;
                    }
                }
            }
            for (var index1 = length - 1; index1 >= 0; --index1)
            {
                if (numArray2[index1] != numArray1[index1])
                {
                    for (var index2 = 0; index2 < length; ++index2)
                    {
                        var num = matrixOfA[index2, numArray2[index1]];
                        matrixOfA[index2][numArray2[index1]] = matrixOfA[index2][numArray1[index1]];
                        matrixOfA[index2][numArray1[index1]] = num;
                    }
                }
            }
            return true;
        }
    });
    ej.trendlineTypes.polynomial = ej.Polynomial;

    ej.MovingAverage = ejExtendClass(ej.EjTrendLineRenderer, {

        updateMovingAverageTrendSource: function (points, trendline) {
            var len = points.length, i, yVal;
            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (i = 0; i < len; i++) {
                trendline.xPoints.push(points[i].xValue);
                trendline.xValues.push(i + 1);
                yVal = !points[i].isEmpty ? points[i].y : null;
                trendline.yValues.push(yVal);
            }
            return false;
        },
        calculateMovingAverageTrendline: function (series, trendline, axis) {
            var pointsLength = trendline.xPoints.length,
            period = trendline.period >= pointsLength ? pointsLength - 1 : trendline.period,
			x1, i, j, yVal, count, nullCount, xVal, length;
            period = period < 2 ? 2 : period;
            trendline.points = [];

            for (i = 0; i < pointsLength-1; i++) {
                yVal = count = nullCount = 0;
                for (j = i; count < period ; j++) {
                    count++;
                    if (ej.util.isNullOrUndefined(trendline.yValues[j])) nullCount++;
                    yVal += trendline.yValues[j];
                }
                yVal = period - nullCount <= 0 ? null : yVal / (period - nullCount);
                if (!ej.util.isNullOrUndefined(yVal) && !isNaN(yVal)) {
				    xVal = trendline.xPoints[period - 1 + i];
                    trendline.points.push({ 
					        x: xVal, xValue: xVal, 
					        y: yVal, YValues: [], visible: true 
					});
                    length = trendline.points.length;
                    trendline.points[length - 1].YValues[0] = trendline.points[length - 1].y;
                }
            }
        }
    });
    ej.trendlineTypes.movingaverage = ej.MovingAverage;

    ej.EjIndicatorRender.prototype = {

        draw: function (indicator, sender) {

            this.chartObj = sender;
            var translate = null;

            if (this.chartObj.model.AreaType == "cartesianaxes") {
                var transX = indicator.xAxis.x;
                var transY = indicator.yAxis.y;
                translate = 'translate(' + transX + ',' + transY + ')';
            }

            var indicatorIndex = $.inArray(indicator, this.chartObj.model.indicators);
            var indicatorOptions = { 'id': this.chartObj.svgObject.id + '_indicatorGroup' + '_' + indicatorIndex, 'transform': translate };
            this.gIndicatorGroupEle = this.chartObj.svgRenderer.createGroup(indicatorOptions);

            
            if (indicator._points.length > 0) {
                for (var i = 0; i < indicator.segment.length; i++) {
                    var segment = indicator.segment[i];
                    var style = { fill: segment.fill, width: segment.width, opacity: indicator.opacity };
                        segment.isIndicator = true;
                        segment.xAxis = indicator.xAxis;
                        segment.yAxis = indicator.yAxis;
                         if (segment.type == "line")
                            this.calculateDirection(segment, style, indicatorIndex);
                        else
                            this.drawColumn(segment, style, indicator);
                    }
                }

        },

        _drawLinePath: function (indicator, style, lDirection, indicatorIndex) {
            
          
          
           
            if (lDirection != "") {
                var options = {
				    'name': 'line',
                    'id': this.chartObj.svgObject.id + "_" + indicatorIndex + "_" + indicator.name,
                    'fill': 'none',
                    'stroke-dasharray': indicator.dashArray,
                    'stroke-width': style.width,
                    'stroke': style.fill,
                    'stroke-linecap': indicator.lineCap,
                    'stroke-linejoin': indicator.lineJoin,
                    'opacity': style.opacity,
                    'd': lDirection
                };

                this.chartObj.svgRenderer.drawPath(options, this.gIndicatorGroupEle);
            }

            this.chartObj.svgRenderer.append(this.gIndicatorGroupEle, this.chartObj.gIndicatorEle);
        },

        calculateDirection: function (currentseries, style, indicatorIndex) {
		    var canvasX = 0;
            var canvasY = 0;
		    var point;
            if (this.chartObj.model.enableCanvasRendering) {
                canvasX = currentseries.xAxis.x;
                canvasY = currentseries.yAxis.y;
            }
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();

            var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(currentseries);
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();

            // Dashed Line issue in Indicator is fixed.
            var firstPoint = false;
            var secondPoint;
            
            for (var i = 0; i < visiblePoints.length; i++) {
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                     point = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (firstPoint) {                                             
                        sb.append("L" + " " + (point.X + canvasX) + " " + ((point.Y + canvasY)) + " ");                    
                        }
                    else {                       
                        sb.append("M" + " " + (point.X + canvasX) + " " + ((point.Y + canvasY)) + " ");
                         firstPoint = true;
                       }         

                }
                else {
                    firstPoint = false;
                }

            }

            lDirection = sb.toString();

            this._drawLinePath(currentseries, style, lDirection, indicatorIndex);
        },

        drawColumn: function (option, style, indicator) {
            var series = option;
            var seriesIndex = $.inArray(indicator, this.chartObj.model.indicators);
            series.index = seriesIndex;
            var origin = Math.max(option.yAxis.visibleRange.min, 0);

            var sidebysideinfo = { Start: -0.35, End: 0.35 };
            var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(series);

            var cSer = this;
			 var chart = cSer.chartObj;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];

                var y1 = point.YValues[0];
                var y2 = origin;
                if (point.visible) {
                    //calculate sides
                    var data = ej.EjSeriesRender.prototype.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;
                    var rect = ej.EjSeriesRender.prototype.getRectangle(x1, y1, x2, y2, series, chart);
					
                    //drawing part
                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);
                    if (rect.Width < 0.0001)// condition checked for IE width issue
                        rect.Width=0.0001;
                    var bounds;
                    if ((xr == 0 || yr == 0) && rect.Width > 0) {
                        options = {
                            'id': this.chartObj.svgObject.id + "_" + seriesIndex + "_" + series.name,
                            'x': rect.X + cSer.chartObj.canvasX,
                            'y': rect.Y + cSer.chartObj.canvasY,
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': (!visiblePoints[i].fill ) ? style.fill : visiblePoints[i].fill,
                            'stroke-width': series.border.width,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': series.histogram.opacity,
                            'stroke': (!visiblePoints[i].fill ) ? series.border.color : visiblePoints[i].fill
                        };

                        this.chartObj.svgRenderer.drawRect(options, this.gIndicatorGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                        this._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gIndicatorGroupEle, this.chartObj.gIndicatorEle);
        },
        _addRegion: function (chart, bounds, series, point, pointIndex) {
            var type = series.type;
            if (series.index >= 0) {
                var regionItem = { SeriesIndex: series.index, Region: { PointIndex: pointIndex, Bounds: bounds }, type: type, isIndicator: true };
                chart.model.chartRegions.push(regionItem);
            }
        },

        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
              indicator.segment = [];
              indicator.xRange = {};
              indicator.yRange = {};
			  indicator.period = parseInt(indicator.period);   // to convert float value to integer
              if (indicator.period < indicator._points.length && indicator.period > 0) {
              //Upperband 
              var upperSeries = $.extend(true, {}, indicator);
              upperSeries.name = "upperseries";
              upperSeries.fill = indicator.upperLine.fill;
              upperSeries.width = indicator.upperLine.width;
              upperSeries.type = "line";

              for (var k = 0; k < indicator._points.length; k++) {
                  indicator._points[k].YValues.length = 1;
                  indicator._points[k].YValues[0] = indicator._points[k].y = 70;
              }
              
              upperSeries.points = $.extend(true, [], indicator._points);
              var xValues = sender._getXValues(upperSeries.points);
              var yValues = sender._getYValues(upperSeries.points);
              
              indicator.xRange.min = Math.min.apply(Math, xValues);
              indicator.xRange.max = Math.max.apply(Math, xValues);
              indicator.yRange.min = Math.min.apply(Math, yValues);
              indicator.yRange.max = Math.max.apply(Math, yValues);

              indicator.segment.push(upperSeries);

              //Lowerband
              var lowerSeries = $.extend(true, {}, indicator);
              lowerSeries.name = "lowerSeries";
              lowerSeries.width = indicator.lowerLine.width;
              lowerSeries.fill = indicator.lowerLine.fill;
              lowerSeries.type = "line";
           

              for (var n = 0; n < indicator._points.length; n++) {
                  indicator._points[n].YValues[0] = indicator._points[n].y = 30;
              }

               lowerSeries.points = $.extend(true, [], indicator._points);

              yValues = sender._getYValues(lowerSeries.points);

              indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

              indicator.segment.push(lowerSeries);
              
              // signalLine
              var trendSeries = $.extend(true, {}, indicator); 
              trendSeries.name = "signalSeries";
              trendSeries.width = indicator.width;
              trendSeries.fill = indicator.fill;
              trendSeries.type = "line";
              
              var c = 0, c1 = 0;
              var pmf = 0;
              var nmf = 0;
              var len = indicator.period;
              c1 = indicator._points[0].close;

              for (var i = 1; i <=len; ++i) {
                  c = indicator._points[i].close;
                  if (c > c1)
                      pmf += c - c1;
                  else if (c < c1)
                      nmf += c1 - c;
                  c1 = c;
                  indicator._points[i-1].YValues[0] = indicator._points[i-1].y = null;
              }
              pmf = pmf / len;
              nmf = nmf / len;
              
              indicator._points[len].YValues[0] = indicator._points[len].y = 100 - 100 / (1 + pmf / nmf);

              for (var j = len + 1; j < indicator._points.length; j++) {
                  c = indicator._points[j].close;
                  if (c > c1) {
                      pmf = (pmf * (len - 1) + (c - c1)) / len;
                      nmf = (nmf * (len - 1)) / len;
                  }
                  else if (c < c1) {
                      nmf = (nmf * (len - 1) + (c1 - c)) / len;
                      pmf = (pmf * (len - 1)) / len;
                  }
                  c1 = c;
                  indicator._points[j].YValues[0] = indicator._points[j].y = 100 - (100 / (1 + pmf / nmf));
              }
              trendSeries.points = $.extend(true, [], indicator._points);
              var trendPoints = [];
              for (var i = len, j = 0; i < trendSeries.points.length; i++, j++){
                  trendPoints[j] = trendSeries.points[i].y;
				  }

              indicator.yRange.min = Math.min(Math.min.apply(Math, trendPoints), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, trendPoints), indicator.yRange.max);

              indicator.segment.push(trendSeries);
              this.calculateIndicatorRange(sender, indicator);
           }
        },

        calculateIndicatorRange: function(sender, indicator){
            //Indicator X-Axis Calculation
            if (sender.model.indicatorRange[indicator.xAxisName] == undefined) {
                sender.model.indicatorRange[indicator.xAxisName] = {};
                sender.model.indicatorRange[indicator.xAxisName] = { min: indicator.xRange.min, max: indicator.xRange.max };
            } else {
                if (sender.model.indicatorRange[indicator.xAxisName].min > indicator.xRange.min)
                    sender.model.indicatorRange[indicator.xAxisName].min = indicator.xRange.min;
                if (sender.model.indicatorRange[indicator.xAxisName].max < indicator.xRange.max)
                    sender.model.indicatorRange[indicator.xAxisName].max = indicator.xRange.max;
            }
            //Indicator Y-Axis Calculation
            if (sender.model.indicatorRange[indicator.yAxisName] === undefined) {
                sender.model.indicatorRange[indicator.yAxisName] = {};
                sender.model.indicatorRange[indicator.yAxisName] = { min: indicator.yRange.min, max: indicator.yRange.max };
            } else {
                if (sender.model.indicatorRange[indicator.yAxisName].min > indicator.yRange.min)
                    sender.model.indicatorRange[indicator.yAxisName].min = indicator.yRange.min;
                if (sender.model.indicatorRange[indicator.yAxisName].max < indicator.yRange.max)
                    sender.model.indicatorRange[indicator.yAxisName].max = indicator.yRange.max;
            }
        }
    },
    ej.ejRSI = ejExtendClass(ej.EjIndicatorRender);

    ej.indicatorTypes.rsi = ej.ejRSI;
	ej.ejMACD = ejExtendClass(ej.EjIndicatorRender, {


        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};

            var pointsLength = indicator._points.length;
            var longPeriod = indicator.longPeriod;
            var shortPeriod = indicator.shortPeriod;
            var trigger = indicator.trigger;
            var diff = longPeriod-shortPeriod;
            var shortEMA = [];
            var longEMA = [];
            var macdPoints = [];
            var signalEMA = [];
            var histogramPoints = [];
            var length = longPeriod + trigger;
            if (length <= indicator._points.length && ((length-2) >= 0) && (shortPeriod <= longPeriod) && (shortPeriod > 0)) {
                shortEMA = this.calculateEMA(indicator._points, shortPeriod, true);
                longEMA = this.calculateEMA(indicator._points, longPeriod, true);

                //Calculte MACD Line points: MACD Line: (shortPeriod EMA - longPeriod EMA)  
                for (i = 0; i < longEMA.length; i++)
                    macdPoints.push((shortEMA[i + diff] - longEMA[i]));

                //MACD Line 
                var macdSeries = $.extend(true, {}, indicator);
                macdSeries.name = "macdSeries";
                macdSeries.fill = indicator.macdLine.fill;
                macdSeries.width = indicator.macdLine.width;
                macdSeries.type = "line";
                macdSeries.xAxis = indicator.xAxis;
                macdSeries.yAxis = indicator.yAxis;

                var l = longPeriod - 1;
                for (var t = 0, i = 0; t < pointsLength; t++) {
                    if (t < l)
                        indicator._points[t].YValues[0] = indicator._points[t].y = null;
                    else {
                        indicator._points[t].YValues[0] = indicator._points[t].y = macdPoints[i];
                        i += 1;
                    }
                }
                macdSeries.points = $.extend(true, [], indicator._points);
                //////////////////////////////////////////////

                //Signal Line: Trigger value(9-day default) EMA of MACD Line 
                signalEMA = this.calculateEMA(macdPoints, trigger, false);
                var trendSeries = $.extend(true, {}, indicator);
                trendSeries.name = "signalSeries";
                trendSeries.fill = indicator.fill;
                trendSeries.width = indicator.width;
                trendSeries.type = "line";
                trendSeries.xAxis = indicator.xAxis;
                trendSeries.yAxis = indicator.yAxis;

                var l = longPeriod + trigger - 2;
                for (var t = 0, i = 0; t < pointsLength; t++) {
                    if (t < l)
                        indicator._points[t].YValues[0] = indicator._points[t].y = null;
                    else {
                        indicator._points[t].YValues[0] = indicator._points[t].y = signalEMA[i];
                        i += 1;
                    }
                }
                trendSeries.points = $.extend(true, [], indicator._points);
                /////////////////////////////////////////

                //MACD Histogram: MACD Line - Signal Line
                var histogramSeries = $.extend(true, {}, indicator);
                histogramSeries.border = {};
                histogramSeries.name = "histogramSeries";
                histogramSeries.fill = indicator.histogram.fill;
                histogramSeries.opacity = indicator.histogram.opacity;
                histogramSeries.fill = indicator.histogram.fill;
                histogramSeries.border = indicator.histogram.border;
                histogramSeries.type = "column";
                histogramSeries.xAxis = indicator.xAxis;
                histogramSeries.yAxis = indicator.yAxis;

                for (var j = 0, k = l; k < pointsLength; j++, k++)
                    histogramPoints.push((macdPoints[j + (trigger - 1)]) - (signalEMA[j]));
                for (var j = 0, k = l; k < pointsLength; j++, k++) {
                    indicator._points[k].YValues[0] = indicator._points[k].y = histogramPoints[j];
                }
                histogramSeries.points = $.extend(true, [], indicator._points);
                xValues = sender._getXValues(macdSeries.points);
                indicator.xRange.min = Math.min.apply(Math, xValues);
                indicator.xRange.max = Math.max.apply(Math, xValues);

                switch (indicator.macdType) {
                    case "line":
                        {
                            //Push MACD Line
                            indicator.yRange.min = Math.min.apply(Math, macdPoints);
                            indicator.yRange.max = Math.max.apply(Math, macdPoints);
                            indicator.segment.push(macdSeries);

                            //Push Signal Line
                            var min = Math.min.apply(Math, signalEMA), max = Math.max.apply(Math, signalEMA);
                            indicator.yRange.min = indicator.yRange.min < min ? indicator.yRange.min : min;
                            indicator.yRange.max = indicator.yRange.max > max ? indicator.yRange.max : max;
                            indicator.segment.push(trendSeries);
                        }
                        break;
                    case "histogram":
                        {
                            //Push histogram Series
                            var min1 = Math.min.apply(Math, histogramPoints), max1 = Math.max.apply(Math, histogramPoints);
                            indicator.yRange.min = indicator.yRange.min < min1 ? indicator.yRange.min : min1;
                            indicator.yRange.max = indicator.yRange.max > max1 ? indicator.yRange.max : max1;
                            indicator.segment.push(histogramSeries);
                        }
                        break;
                    case "both":
                        {
                            //Push histogram Series
                            var min1 = Math.min.apply(Math, histogramPoints), max1 = Math.max.apply(Math, histogramPoints);
                            indicator.yRange.min = indicator.yRange.min < min1 ? indicator.yRange.min : min1;
                            indicator.yRange.max = indicator.yRange.max > max1 ? indicator.yRange.max : max1;
                            indicator.segment.push(histogramSeries);

                            //Push MACD Line 
                            indicator.yRange.min = Math.min.apply(Math, macdPoints);
                            indicator.yRange.max = Math.max.apply(Math, macdPoints);
                            indicator.segment.push(macdSeries);

                            //Push Signal Line
                            var min = Math.min.apply(Math, signalEMA), max = Math.max.apply(Math, signalEMA);
                            indicator.yRange.min = indicator.yRange.min < min ? indicator.yRange.min : min;
                            indicator.yRange.max = indicator.yRange.max > max ? indicator.yRange.max : max;
                            indicator.segment.push(trendSeries);
                        }
                        break;
                }
                this.calculateIndicatorRange(sender, indicator);
            }
        },
        calculateEMA: function (points, period, flag) {
            var sma = 0;
            var initialEMA = 0;
            var pointsLength = points.length;
            var getPoints = [];
            var getEma = [];
            var emaPercent = (2 / (period + 1));
            if (flag) { // calculating EMA for Indicators Points
                for (var i = 0; i < period; i++) {
                    getPoints.push(points[i].close);
                    sma = sma + getPoints[i];
                }

                initialEMA = (sma / period);
                getEma.push(initialEMA);
                var emaAvg = initialEMA;
                for (j = period; j < pointsLength; j++) {
                    emaAvg = (points[j].close - emaAvg) * emaPercent + emaAvg;
                    getEma.push(emaAvg);
                }
            } else { //Calculating EMA for Signal Points
                for (var i = 0; i < period; i++) {
                    getPoints.push(points[i]);
                    sma = sma + getPoints[i];
                }

                initialEMA = (sma / period);
                getEma.push(initialEMA);
                var emaAvg = initialEMA;
                for (j = period; j < pointsLength; j++) {
                    emaAvg = (points[j] - emaAvg) * emaPercent + emaAvg;
                    getEma.push(emaAvg);
                }
            }
            return getEma;
        }

    });

    ej.indicatorTypes.macd = ej.ejMACD;
	ej.ejSMA = ejExtendClass(ej.EjIndicatorRender, {

      
        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};


         
            // signalLine
            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

			//variable declaration
            var closevalues = [];
            var smavalues = [];
			var trendPoints = [];
            var sum;
            var length = indicator._points.length;
            var period = indicator.period;
			
			if (period <= 0 || length < period) return;
			
			//Push the Close values
            for (var i = 0; i < length; i++) {
               
                closevalues.push(indicator._points[i].close);

            }

            var close_length =closevalues.length;
			//calculation for SMA values
			
	     
            while (close_length >= period) {
                sum = 0;
                for (var j = 0; j < period; j++) {
                    sum = sum + closevalues[j];

                }
                sum = sum / period;
                smavalues.push(sum);
                closevalues.splice(0, 1);
                close_length = closevalues.length;
            }

	 
           //set the SMA values to Y-Axis
            for (var k = 0; k < length; k++) {
                if (k < period - 1) {
                    indicator._points[k].YValues[0] = indicator._points[k].y = null;

                }
                else {
                    indicator._points[k].YValues[0] = indicator._points[k].y =smavalues[k -(period - 1) ];
                }

            }

            
            trendSeries.points = $.extend(true, [], indicator._points);
            xValues = sender._getXValues(trendSeries.points);
            
            //Assign the Y-Axis values into trendPoints
            for (var i = period-1 , j = 0; i < trendSeries.points.length; i++, j++)
             {
                trendPoints[j] = trendSeries.points[i].y;
             }
                
			//X-Axis and Y-Axis range calculation
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, trendPoints);
            indicator.yRange.max = Math.max.apply(Math, trendPoints);
            
			//push trendSeries
            indicator.segment.push(trendSeries);
        	this.calculateIndicatorRange(sender, indicator);
            
          
        }

    });

    ej.indicatorTypes.sma = ej.ejSMA;
	
	 ej.ejEMA = ejExtendClass(ej.EjIndicatorRender, {


        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};



            // signalLine
            
            trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.type = "line";
            trendSeries.xAxis = indicator.xAxis;
            trendSeries.yAxis = indicator.yAxis;


            var getCloseValues = [];       
            var ema = [];
            var pointsLength = indicator._points.length;       
            var period = indicator.period;
            var sma = 0;
            if(pointsLength < period || period <=0) return ;
            // K=Smoothing Factor
            var k = (2 / (period + 1));

            //Get a close Values and push the previous Ema 
            for (var i = 0; i < period; i++) {
                getCloseValues.push(indicator._points[i].close);
                sma = sma + getCloseValues[i];            
            }
            ema.push(sma / period);

            //Find Remaining EMA by Adding smoothing factor K to the close values
            for (j = period; j < pointsLength; j++)
                  {              
                   ema.push((indicator._points[j].close - ema[j - period]) * k + ema[j - period]);                  
                   }        
            
            // Set Ema to the Indicator Points
           
            for (var t = 0; t < pointsLength; t++) {
                if (t < (period - 1)) {
                    indicator._points[t].YValues[0] = indicator._points[t].y = null;
                }
                else if (t >= (period - 1))
                {
                    indicator._points[t].YValues[0] = indicator._points[t].y = ema[t - (period - 1)];
                }
            }
             
            // Send the indicator Points for draw series.
            trendSeries.points = $.extend(true, [], indicator._points);
            xValues = sender._getXValues(trendSeries.points);
            var trendPoints = [];
            for (var i = period, j = 0; i < trendSeries.points.length; i++, j++)
                trendPoints[j] = trendSeries.points[i].y;

            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, trendPoints);
            indicator.yRange.max = Math.max.apply(Math, trendPoints);

            indicator.segment.push(trendSeries);
        	this.calculateIndicatorRange(sender, indicator);
            
          
        }

    });

    ej.indicatorTypes.ema = ej.ejEMA;


    ej.ejSTOCHASTIC = ejExtendClass(ej.EjIndicatorRender, {

        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
			var KPeriod = indicator.kPeriod;
            var DPeriod = indicator.dPeriod;
            var Period = indicator.period;
            if (Period < 1 || DPeriod < 1 || KPeriod < 1) return;
            if (Period > indicator._points.length || DPeriod > indicator._points.length || KPeriod > indicator._points.length) return;
			// Upper Line Calculation
            var upperSeries = $.extend(true, {}, indicator);
            upperSeries.name = "upperseries";
            upperSeries.type = "line";
            upperSeries.fill = indicator.upperLine.fill;
            upperSeries.width = indicator.upperLine.width;

            for (var k = 0; k < indicator._points.length; k++) {
                indicator._points[k].YValues.length = 1;
                indicator._points[k].YValues[0] = indicator._points[k].y = 80;
            }

            upperSeries.points = $.extend(true, [], indicator._points);
            var xValues = sender._getXValues(upperSeries.points);
            var yValues = sender._getYValues(upperSeries.points);

            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, yValues);
            indicator.yRange.max = Math.max.apply(Math, yValues);

            indicator.segment.push(upperSeries);
            // End for Upper Line Calculation

            // Lower Line Calculation
            var lowerSeries = $.extend(true, {}, indicator);
            lowerSeries.name = "lowerSeries";
            lowerSeries.width = indicator.lowerLine.width;
            lowerSeries.fill = indicator.lowerLine.fill;
            lowerSeries.type = "line";


            for (var n = 0; n < indicator._points.length; n++) {
                indicator._points[n].YValues[0] = indicator._points[n].y = 20;
            }

            lowerSeries.points = $.extend(true, [], indicator._points);

            xValues = sender._getXValues(lowerSeries.points);
            yValues = sender._getYValues(lowerSeries.points);

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

            indicator.segment.push(lowerSeries);
            // End for Lower Line Calculation

            // Signal and Period Line Calculation

            var periodSeries = $.extend(true, {}, indicator);
            periodSeries.name = "periodSeries";
            periodSeries.width = indicator.periodLine.width;
            periodSeries.fill = indicator.periodLine.fill;
            periodSeries.type = "line";

            var HighValues = [];
            var LowValues = [];
            var CloseValues = [];
            var check = [];
            for (var i = 0; i < indicator._points.length; i++) {
                LowValues[i] = indicator._points[i].low;
                HighValues[i] = indicator._points[i].high;
                CloseValues[i] = indicator._points[i].close;
            }


            AddPoints(Period, KPeriod, indicator._points);
            check = indicator._points;
            SMA(Period, KPeriod, indicator._points);
            check = indicator._points;
            periodSeries.points = $.extend(true, [], indicator._points);

            xValues = sender._getXValues(periodSeries.points);
            yValues = sender._getYValues(periodSeries.points);

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

            indicator.segment.push(periodSeries);


            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

            SMA((Period+KPeriod-1), DPeriod, indicator._points);
            trendSeries.points = $.extend(true, [], indicator._points);

            indicator.segment.push(trendSeries);

            ///SMA Calculation(Signal Line and Period Line)

            function SMA(Period1, Period2, series) {
                if (indicator._points.length >= Period1 + Period2) {
                    SMACal(Period1, Period2, series);
                }
            }

            function SMACal(period1, period2, series) {
                var count = period1 + (period2-1);
                var temp = [];
                var values = [];
                var sum;
                for (var i = 0; i < indicator._points.length; i++) {
                    var val = check[i].y;
                    temp.push(val);
                }
                var length = temp.length;

                while (length >= count) {
                    sum = 0;
                    for (var j = period1-1; j < (period1 + period2-1); j++) {
                        sum = sum + temp[j];

                    }
                    sum = sum / period2;
                    values.push(sum.toFixed(2));
                    temp.splice(0, 1);
                    length = temp.length;
                }
                var len = count - 1;
                for (var k = 0; k < indicator._points.length; k++) {
                    if (k < len) {
                        series[k].YValues[0] = series[k].y = null;

                    }
                    else {
                        series[k].YValues[0] = series[k].y = Number((values[k - len]));
                    }

                }

            }

            // End for SMA Calculation

            // Period Line Calculation

            function AddPoints(Period1, Period2, series) {
                if (indicator._points.length > Period1) {
                    ComputeStochastics(Period1, Period2, series);
                }
            }
            function ComputeStochastics(len1, len2, series) {
                var len = len1 + len2;
                var mins = [];
                var maxs = [];
                var max;
                var min;
                var top = 0;
                var bottom = 0;
                for (var i = 0; i < len1 - 1; ++i) {
                    maxs.push(0);
                    mins.push(0);
                }

                for (var i = len1 - 1; i < indicator._points.length; ++i) {
                    min = Number.MAX_VALUE;
                    max = Number.MIN_VALUE;
                    for (var j = 0; j < len1; ++j) {
                        min = Math.min(min, LowValues[i - j]);
                        max = Math.max(max, HighValues[i - j]);
                    }
                    maxs.push(max);
                    mins.push(min);
                }

                for (var i = 0 ; i < len - 1; ++i) {
                    series[i].YValues.length = 1;
                    series[i].YValues[0] = series[i].y = null;
                }

                for (var i = len1 - 1; i < indicator._points.length; ++i) {
                    top = 0;
                    bottom = 0;
                    top += CloseValues[i] - mins[i];
                    bottom += maxs[i] - mins[i];
                    series[i].YValues.length = 1;
                    series[i].YValues[0] = series[i].y = (((top / bottom) * 100));
                }
            }

            // End for Period Line Calculation

            // Indicator Axis Range Calculation
            this.calculateIndicatorRange(sender, indicator);
        }


    });
    ej.indicatorTypes.stochastic = ej.ejSTOCHASTIC;
ej.ejbollingerBand = ejExtendClass(ej.EjIndicatorRender, {
        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
            var sum = 0.0;
            var deviationSum = 0.0;
            var limit = indicator._points.length;
            var multiplier =indicator.standardDeviations;
            var movingavg =indicator.period;
            var length = Math.round(movingavg);
            var smaPoints = [];
            var deviations = [];
            var bbiPoints = [];
            //Calculating simple moving average
         if ((length < limit) && (length > 0))
{
            for (var i = 0; i < length; i++) {
                sum += indicator._points[i].close;

            }
            var sma = sum / movingavg;
            //Calculating smapoints using iteration
            for (var i = 0; i < limit; ++i) {
                if (i >= length - 1 && i < limit) {
                    if (i - movingavg >= 0) {
                        var diff = indicator._points[i].close - indicator._points[i - length].close;

                        sum = sum + diff;
                        sma = sum / (movingavg);
                        smaPoints[i] = sma;
                        deviations[i] = Math.pow(indicator._points[i].close - sma, 2);
                        deviationSum += deviations[i] - deviations[i - length];


                    }
                    else {

                        smaPoints[i] = sma;
                        deviations[i] = Math.pow(indicator._points[i].close - sma, 2);
                        deviationSum += deviations[i];
                    }


                    //calculation for standarddeviation for each period

                    var range = Math.sqrt(deviationSum / (movingavg));

                    //lower Band points for bollinger band
                    var lb = smaPoints[i] - (multiplier * range);
                    //upper Band points for bollinger band
                    var ub = smaPoints[i] + (multiplier * range);
                    //Binding the lowerband,upperband and middleband values to bollingerband array if the i value equal to period
                    if (i + 1 == length) {
                        for (var j = 0; j < length - 1; j++)
                            bbiPoints[j] = { "X": indicator._points[j].X, "mb": smaPoints[i], "lb": lb, "ub": ub, visible: true };



                    }
                    //binding the lb,ub,mb values to bollinger band array if i greather than period value.
                    bbiPoints[i] = { "X": indicator._points[i].X, "mb": smaPoints[i], "lb": lb, "ub": ub, visible: true };


                }


                else {
                    if (i < movingavg - 1) {

                        smaPoints[i] = sma;
                        deviations[i] = Math.pow(indicator._points[i].close - sma, 2);
                        deviationSum += deviations[i];
                    }


                }

            }


            var upperSeries = $.extend(true, {}, indicator);
            upperSeries.name = "upperseries";
            upperSeries.fill = indicator.upperLine.fill;
            upperSeries.width = indicator.upperLine.width;
            upperSeries.type = "line";

            //binding bollingerband array values to upper series and assigning null values 
            for (var k = 0; k < indicator._points.length; k++) {
                if (k >= (length - 1)) {
                    indicator._points[k].YValues.length = 1;
                    indicator._points[k].YValues[0] = indicator._points[k].y = bbiPoints[k].ub;
                }
                else {
                    indicator._points[k].YValues[0] = indicator._points[k].y = null;
                }
            }

            upperSeries.points = $.extend(true, [], indicator._points);
            var xValues = sender._getXValues(upperSeries.points);
            var values = [];
            //storing the y values from the period value
            for (var i = length; i < upperSeries.points.length; i++)
                values.push(upperSeries.points[i].y);
            var yValues = values;
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, yValues);
            indicator.yRange.max = Math.max.apply(Math, yValues);

            indicator.segment.push(upperSeries);


            var lowerSeries = $.extend(true, {}, indicator);
            lowerSeries.name = "lowerSeries";
            lowerSeries.width = indicator.lowerLine.width;
            lowerSeries.fill = indicator.lowerLine.fill;
            lowerSeries.type = "line";


            for (var n = 0; n < indicator._points.length; n++) {
                if (n >= (length - 1)) {
                    indicator._points[n].YValues.length = 1;
                    indicator._points[n].YValues[0] = indicator._points[n].y = bbiPoints[n].lb;
                }
                else {
                    indicator._points[n].YValues[0] = indicator._points[n].y = null;
                }
            }


            lowerSeries.points = $.extend(true, [], indicator._points);

            xValues = sender._getXValues(lowerSeries.points);

            var lowvalues = [];
            for (var i = length; i < lowerSeries.points.length; i++)
                lowvalues.push(lowerSeries.points[i].y);
            var yValues = lowvalues;

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

            indicator.segment.push(lowerSeries);



            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

            for (var m = 0; m < indicator._points.length; m++) {
                if (m >= (length - 1)) {
                    indicator._points[m].YValues.length = 1;
                    indicator._points[m].YValues[0] = indicator._points[m].y = bbiPoints[m].mb;
                }
                else {
                    indicator._points[m].YValues[0] = indicator._points[m].y = null;
                }
            }
            trendSeries.points = $.extend(true, [], indicator._points);
            xValues = sender._getXValues(trendSeries.points);
            var trendvalues = [];
            for (var i = length; i < trendSeries.points.length; i++)
                trendvalues.push(trendSeries.points[i].y);
            var yValues = trendvalues;

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);
            indicator.segment.push(trendSeries);
            //calculating indicator range
            this.calculateIndicatorRange(sender, indicator);
           }
        }
     
        });
    ej.indicatorTypes.bollingerband = ej.ejbollingerBand;
  ej.ejATR = ejExtendClass(ej.EjIndicatorRender,{       
        calculateSegment:function(indicator,sender)
        {
            //calculation for atr
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
            var atrPoints = [];
            var high_Low, high_ClosePrevious, low_ClosePrevious;
            high_Low = high_ClosePrevious = low_ClosePrevious = 0.0;
            var trueRange;
            var sumTruRange = 0.0;
            var firstFlag = false;
            var atr;
            var period =  indicator.period;
            if ((period <= 0) || (period > indicator._points.length))
                return;
            for (var i = 0; i < period; i++) {

                high_Low = indicator._points[i].high - indicator._points[i].low;
                if (i > 0) {
                    high_ClosePrevious = Math.abs(indicator._points[i].high - indicator._points[i - 1].close);
                    low_ClosePrevious = Math.abs(indicator._points[i].low - indicator._points[i - 1].close);
                 }
              
                trueRange = Math.max(high_Low, high_ClosePrevious, low_ClosePrevious);
               
                sumTruRange += trueRange;
                atr = (sumTruRange / period);
          
                    atrPoints[i] = { "X": indicator._points[i].X, "YValues": atr };

            }
       
             for (var i = period; i < indicator._points.length; ++i) {
                high_Low = indicator._points[i].high - indicator._points[i].low;
                high_ClosePrevious = Math.abs(indicator._points[i].high - indicator._points[i - 1].close);
                low_ClosePrevious = Math.abs(indicator._points[i].low - indicator._points[i-1].close);
                trueRange = Math.max(high_Low, high_ClosePrevious, low_ClosePrevious);
               
               atr = (atrPoints[i-1].YValues * (period - 1) + trueRange) / period;
             
                atrPoints[i] = {
                    "X": indicator._points[i].X, "YValues": atr
                };

               }
          


             var trendSeries = $.extend(true, {}, indicator);


            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

         

            for (var k = 0; k < indicator._points.length; k++) {
                if (k >= (period - 1)) {
                    indicator._points[k].YValues.length = 1;
                    indicator._points[k].YValues[0] = indicator._points[k].y = atrPoints[k].YValues;
                }
                else {
                    indicator._points[k].YValues[0] = indicator._points[k].y = null;
                }
            }

           
       
            trendSeries.points = $.extend(true, [], indicator._points);
           
            var xValues = sender._getXValues(trendSeries.points);
           
            var values = [];
            for (var i = period; i < trendSeries.points.length; i++)
             values.push(trendSeries.points[i].y);
            
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, values);
            indicator.yRange.max = Math.max.apply(Math, values);

           indicator.segment.push(trendSeries);
        

            this.calculateIndicatorRange(sender, indicator);


            
        }

    });
    ej.indicatorTypes.atr = ej.ejATR;

 ej.ejaccumulationDistribution = ejExtendClass(ej.EjIndicatorRender, {
        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
            var adlPoints = [];
            var sum = 0;
          //calculating accumulation distribution line points 
            for (var i = 0; i < indicator._points.length;i++) {
                var close = indicator._points[i].close;
                
                    var value=(((close - indicator._points[i].low) - (indicator._points[i].high - close)) / (indicator._points[i].high - indicator._points[i].low));
                    sum += (isNaN(value) ? 0 : value) * indicator._points[i].volume;
                    adlPoints[i] = sum;
                
            }
            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendseries";
            trendSeries.fill = indicator.fill;
            trendSeries.width = indicator.width;
            trendSeries.type = "line";

            //Binding the accumulation distribution array values to indicator y values
            for (var k = 0; k < indicator._points.length; k++) {

                   indicator._points[k].YValues[0] = indicator._points[k].y = adlPoints[k];
                  }

            trendSeries.points = $.extend(true, [], indicator._points);
            var xValues = sender._getXValues(trendSeries.points);
           var yValues = sender._getYValues(trendSeries.points);
         
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, yValues);
            indicator.yRange.max = Math.max.apply(Math, yValues);

            indicator.segment.push(trendSeries);
            //calculating indicator range
            this.calculateIndicatorRange(sender, indicator);


        }

    });
    ej.indicatorTypes.accumulationdistribution = ej.ejaccumulationDistribution;
ej.ejTMA = ejExtendClass(ej.EjIndicatorRender, {
     calculateSegment: function (indicator, sender) {
         this.chartObj = sender;
         indicator.segment = [];
         indicator.xRange = {};
         indicator.yRange = {};

         // signalLine
         var trendSeries = $.extend(true, {}, indicator);
         trendSeries.name = "trendSeries";
         trendSeries.width = indicator.width;
         trendSeries.fill = indicator.fill;
         trendSeries.type = "line";

         //variable declaration
         var closevalues = [];
         var close_values = [];
         var smavalues = [];
         var tmavalues = [];
         var smaPoints = [];
         var trendPoints = [];
         var sum=0;
         var sum1;
         var length = indicator._points.length;
         var period = indicator.period;
        if (period < 1) return;
        
         if (period < length)
         {
                //Push the values to closevalues array
             for (var i = 0; i < length; i++) {
                 closevalues.push(indicator._points[i].close);
             }
             //find the closevalues length
             var close_length = closevalues.length;
         
             //calculation for SMA values

             while (close_length >= period) {
                 sum = 0;
                 for (var j = 0; j < period; j++) {
                     sum = sum + closevalues[j];

                 }
                 sum = sum / period;
                 smavalues.push( parseFloat(sum.toFixed(2)));
                 closevalues.splice(0, 1);
                 close_length = closevalues.length;
             }
             //calculating sma values depend on period
             for (var k = 0; k < length; k++) {
                 if (k < period - 1) {
                     sum = 0;
                     for (var j = 0; j < k+1; j++) {
                         sum = sum + indicator._points[j].close;

                     }
                     sum = sum / (k+1);
                     smaPoints[k] = parseFloat(sum.toFixed(2));
                 }
                 else {
                     smaPoints[k]= smavalues[k - (period - 1)];
                 }

             }
             //find the smavalues count
             var sma_count = smaPoints.length;
             //calculating TMA values
             while (sma_count >=period) {
                 sum1 = 0;
                 for (var j = 0; j < period; j++) {
                     sum1 = sum1 + smaPoints[j];

                 }
                 sum1 = sum1 / period;
                 tmavalues.push(sum1);
                 smaPoints.splice(0, 1);
                 sma_count = smaPoints.length;
                 }
            //set the TMA values to Y-Axis
             for (var k = 0; k < length; k++) {
                 if (k < period - 1) {
                     indicator._points[k].YValues[0] = indicator._points[k].y = null;

                 }
                 else {
                      indicator._points[k].YValues[0] = indicator._points[k].y = tmavalues[k - (period - 1)];
                    
                 }

             }
             trendSeries.points = $.extend(true, [], indicator._points);
             xValues = sender._getXValues(trendSeries.points);

             //Assign the Y-Axis values into trendPoints
             for (var i = period - 1, j = 0; i < trendSeries.points.length; i++, j++) {
                 trendPoints[j] = trendSeries.points[i].y;
             }

             //X-Axis and Y-Axis range calculation
             indicator.xRange.min = Math.min.apply(Math, xValues);
             indicator.xRange.max = Math.max.apply(Math, xValues);
             indicator.yRange.min = Math.min.apply(Math, trendPoints);
             indicator.yRange.max = Math.max.apply(Math, trendPoints);

             //push trendSeries
             indicator.segment.push(trendSeries);
             this.calculateIndicatorRange(sender, indicator);

            
         }
     }
 });
 ej.indicatorTypes.tma = ej.ejTMA;
	
	ej.ejMomentum = ejExtendClass(ej.EjIndicatorRender, {
	       calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
              indicator.segment = [];
              indicator.xRange = {};
              indicator.yRange = {};

              //Upperband 
              var upperSeries = $.extend(true, {}, indicator);
              upperSeries.name = "upperseries";
              upperSeries.fill = indicator.upperLine.fill;
              upperSeries.width = indicator.upperLine.width;
              upperSeries.type = "line";

			  var period = indicator.period;
			  var pointsLength = indicator._points.length;
			  
          if(period > 0 && period < pointsLength -1){  // to check period value is greater than 0 and it is less than pointslength
              for (var k = 0; k < pointsLength; k++) {
                  indicator._points[k].YValues.length = 1;
                  indicator._points[k].YValues[0] = indicator._points[k].y = 100;
              }
              
              upperSeries.points = $.extend(true, [], indicator._points);
              var xValues = sender._getXValues(upperSeries.points);
              var yValues = sender._getYValues(upperSeries.points);
              
              indicator.xRange.min = Math.min.apply(Math, xValues);
              indicator.xRange.max = Math.max.apply(Math, xValues);
              indicator.yRange.min = Math.min.apply(Math, yValues);
              indicator.yRange.max = Math.max.apply(Math, yValues);

              indicator.segment.push(upperSeries);
              
              // signalLine
              var trendSeries = $.extend(true, {}, indicator); 
              trendSeries.name = "trendSeries";
              trendSeries.width = indicator.width;
              trendSeries.fill = indicator.fill;
              trendSeries.type = "line";
              
               var c = []; 
        var len = indicator.period;      // period value
            
        for (var i = 0; i < indicator._points.length; ++i)
        {
		   var points = indicator._points;
            if (!(i < len))                                       // calculate signal line points
                indicator._points[i].YValues[0] =indicator._points[i].y= (points[i].close / (points[i - len].close) *100);
        }

        for (var k = 0; k < len && k<indicator._points.length; k++) {
            indicator._points[k].YValues[0] = indicator._points[k].y = null;
        }
			  
              trendSeries.points = $.extend(true, [], indicator._points);
              xValues = sender._getXValues(trendSeries.points);
              var trendPoints = [];
              for (var i = len, j = 0; i < trendSeries.points.length; i++, j++){
                  trendPoints[j] = trendSeries.points[i].y;
			  }

              indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
              indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
              indicator.yRange.min = Math.min(Math.min.apply(Math, trendPoints), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, trendPoints), indicator.yRange.max);

              indicator.segment.push(trendSeries);
              this.calculateIndicatorRange(sender, indicator);
           }
        }
	});
	
	ej.indicatorTypes.momentum = ej.ejMomentum;
	
    ej.EjSeriesRender.prototype = {

	    getOrigin: function (chart, options, params) {
	        if (params.axes[options.xAxis.name]._validCross) {
	            var crossAxis = chart.chartObj._getCrossAxis(chart.chartObj.model._axes, true, options.xAxis.crossesInAxis);
	            if ((chart.chartObj.model.requireInvertedAxes ? options.xAxis.name : options.yAxis.name === crossAxis.name))
	                return options.xAxis._crossValue;
	        }
	        return Math.max(options.yAxis.visibleRange.min, 0);
	    },

    setLineSeriesStyle: function (currentSeries) {

        var seriesIndex = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
        var serColor = this.chartObj.model.seriesColors[seriesIndex];
       
        
        var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentSeries) + seriesIndex, serColor, 0, 0, 0, $(this.chartObj.svgObject).height(), this.chartObj.gSeriesEle);
        var seriesBorder = this.chartObj.model.seriesBorderColors[seriesIndex];
        return { SeriesInterior: seriesInterior, SeriesBorder: seriesBorder };

    },
    getSeriesName: function(series) {
        var serName;
        if (series.name)
            serName = series.name.replace(/\s/g, '');
        else
            serName = "series";
        
        return serName;
    },
    
    drawAreaPath: function (series, style, direction, canvasTranslate) {
	    var requireInvertedAxes = this.chartObj.model.requireInvertedAxes,
        seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries),
        translate = null, areaType = this.chartObj.model.AreaType, seriesBorder = series.border;
        if (areaType == "cartesianaxes")
        {
            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
            translate = 'translate(' + trans.x + ',' + trans.y + ')'
        }
        var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate};

        this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
        if (direction != "") {
            var options = {
                'id': this.chartObj.svgObject.id + "_Series" + seriesIndex,
                'fill': style.SeriesInterior,
                'fill-opacity': series.opacity,
                'stroke-width': seriesBorder.width,
                'stroke': seriesBorder.color,
                'stroke-dasharray': seriesBorder.dashArray,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'd': direction
            };
            if (this.chartObj.dragPoint) {
                options.id = this.chartObj.svgObject.id + "_PreviewSeries" + seriesIndex;
                options['fill-opacity'] = 0.6;
                if (this.chartObj.model.enableCanvasRendering) this.chartObj.svgRenderer.drawPath(options, this.chartObj.canvasElement, canvasTranslate);
                else this.chartObj.svgRenderer.drawPath(options, this.chartObj.gPreviewSeriesGroupEle, canvasTranslate);
            }
            else
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle, canvasTranslate);
        } 
    },

    setAreaSeriesStyle: function (series) {

        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var maxY = Math.max.apply(0,$.map(series.points, function(v) {return v.YValues[0];
        }));
        var minY = Math.min.apply(0, $.map(series.points, function(v) {
            return v.YValues[0];
        }));
        var minX = Math.min.apply(0, $.map(series.points, function (v) {
            return v.xValue;
        }));

        var point1 = {xValue: minX, YValues : []};
        point1.YValues[0] = minY;
        var point2 = {xValue: minX, YValues : []};
        point2.YValues[0] = maxY;
        var point1Xy = ej.EjSvgRender.utils._getPoint(point1, series);
        var point2Xy = ej.EjSvgRender.utils._getPoint(point2, series);
        var serColor = this.chartObj.model.seriesColors[seriesIndex];

        var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(series) + seriesIndex, serColor, point1Xy.X, point1Xy.Y, point2Xy.X, point2Xy.Y, this.chartObj.gSeriesEle);

        return { SeriesInterior: seriesInterior };

    },





    naturalSpline: function (points) {
        var count = points.length;
        //constant
        var yIndex = 0, a = 6;
        var ySpline = [], ySplineDuplicate = [];

        // assigning the first and last value as zero
        ySpline[0] = ySplineDuplicate[0] = 0;
        ySpline[points.length - 1] = 0;

        for (var i = 1; i < count - 1; i++) {
            var d1 = points[i].xValue - points[i - 1].xValue;
            var d2 = points[i + 1].xValue - points[i - 1].xValue;
            var d3 = points[i + 1].xValue - points[i].xValue;
            var dy1 = points[i + 1].YValues[yIndex] - points[i].YValues[yIndex] || null;
            var dy2 = points[i].YValues[yIndex] - points[i - 1].YValues[yIndex];

            if (d1 == 0 || d2 == 0 || d3 == 0) {
                ySpline[i] = 0;
                ySplineDuplicate[i] = 0;
            }
            else {
                var p = 1 / (d1 * ySpline[i - 1] + 2 * d2);

                ySpline[i] = -p * d3;
                ySplineDuplicate[i] = p * (a * (dy1 / d3 - dy2 / d1) - d1 * ySplineDuplicate[i - 1]);
            }
        }

        for (var k = count - 2; k >= 0; k--)
            ySpline[k] = ySpline[k] * ySpline[k + 1] + ySplineDuplicate[k];


        return ySpline;

    },
    getBezierControlPoints: function (point1, point2, ySpline1, ySpline2, yIndex) {
        var controlPoint1;
        var controlPoint2;
        var one3 = 1 / 3.0;
        var deltaX2 = (point2.xValue - point1.xValue);

        deltaX2 = deltaX2 * deltaX2;

        var dx1 = 2 * parseFloat(point1.xValue) + parseFloat(point2.xValue);
        var dx2 = parseFloat(point1.xValue) + 2 * parseFloat(point2.xValue);

        var dy1 = 2 * point1.YValues[yIndex] + point2.YValues[yIndex];
        var dy2 = point1.YValues[yIndex] + 2 * point2.YValues[yIndex];

        var y1 = one3 * (dy1 - one3 * deltaX2 * (ySpline1 + 0.5 * ySpline2));
        var y2 = one3 * (dy2 - one3 * deltaX2 * (0.5 * ySpline1 + ySpline2));

        controlPoint1 = { xValue: dx1 * one3, YValues: [y1] };
        controlPoint2 = { xValue: dx2 * one3, YValues: [y2] };

        return { controlPoint1: controlPoint1, controlPoint2: controlPoint2 };


    },

    improveChartPerformance: function(chartSeries) {
        var tempPoints = [],
            xVisibleRange = chartSeries.xAxis.visibleRange,
            yVisibleRange = chartSeries.yAxis.visibleRange,
            seriesPoints = chartSeries.pointCollection && chartSeries.dragPoint? chartSeries.pointCollection : chartSeries.points,
            AreaBounds = this.chartObj.model.m_AreaBounds,
            emptyPoints,
            xDelta = xVisibleRange.max - xVisibleRange.min,
            yDelta = yVisibleRange.max - yVisibleRange.min,
            xTolerance = Math.abs((xDelta * 1) / AreaBounds.Width),
            yTolerance = Math.abs((yDelta * 1) / AreaBounds.Height),
            prevXValue = (seriesPoints[0] && seriesPoints[0].X > xTolerance) ? 0 : xTolerance,
            prevYValue = (seriesPoints[0] && seriesPoints[0].y > yTolerance) ? 0 : yTolerance,
            xVal = 0, yVal = 0, isEmpty, currentPoint,
            currentPointsLenght = seriesPoints.length,
            count = 0;

        for (var k = 0; k < currentPointsLenght; k++) {
            currentPoint = seriesPoints[k];
            if (currentPoint.fill)
                chartSeries.pointFill = true;
            isEmpty = (currentPoint.isEmpty === null || currentPoint.isEmpty === undefined) ? false : currentPoint.isEmpty;
            currentPoint.visible = (currentPoint.visible === null || currentPoint.visible === undefined) ? true : currentPoint.visible;
            if (currentPoint.xValue === null || currentPoint.xValue === undefined) {
                currentPoint.isEmpty = true;
                currentPoint.visible = false;
                currentPoint.xValue = xVisibleRange.min;
            } else {
                currentPoint.isEmpty = isEmpty;
                if (jQuery.type(currentPoint.xValue) == "date")
                    currentPoint.xValue = currentPoint.xValue.getTime();
            }
            if ((currentPoint.YValues[0] == null) || (currentPoint.YValues[0] == 'undefined') || (currentPoint.YValues[0] === "")) {
                currentPoint.isEmpty = true;
                currentPoint.visible = false;
                currentPoint.YValues[0] = yVisibleRange.min;
            } else {
                currentPoint.isEmpty = currentPoint.isEmpty;
                currentPoint.visible = (currentPoint.isEmpty) ? !currentPoint.isEmpty : currentPoint.visible;
            }

            xVal = currentPoint.xValue;
            yVal = currentPoint.YValues[0];
            if (Math.abs(prevXValue - xVal) >= xTolerance || Math.abs(prevYValue - yVal) >= yTolerance || chartSeries.enableSmartLabels) {

                tempPoints[count]=currentPoint;
                prevXValue = xVal;
                prevYValue = yVal;
                count++;
            }
        }

        emptyPoints = this._calculateEmptyPoints(chartSeries, tempPoints);
        chartSeries._visiblePoints = emptyPoints;
        return emptyPoints;

    },
    _isVisiblePoints: function(chartSeries) {
        var points = chartSeries.pointCollection && chartSeries.dragPoint? chartSeries.pointCollection : chartSeries.points,
            emptyPoints,
            isEmpty,
            checkNull = ej.util.isNullOrUndefined,
            length = points.length;
        for (var k = 0; k < length; k++) {
            isEmpty = (checkNull(points[k].isEmpty)) ? false : points[k].isEmpty;
            points[k].visible = (checkNull(points[k].visible)) ? true : points[k].visible;
            if (checkNull(points[k].xValue)) {
                points[k].isEmpty = true;
                points[k].visible = false;
                points[k].xValue = chartSeries.xAxis ? chartSeries.xAxis.visibleRange.min : null;
            } else
                points[k].isEmpty = isEmpty;

           if (( typeof points[k].YValues[0] != "object" && isNaN(points[k].YValues[0]) || points[k].YValues[0] == null || points[k].YValues[0] == 'undefined') ||
		   (!chartSeries.isIndicator && (chartSeries._hiloTypes && ( isNaN(points[k].YValues[1]) || points[k].YValues[1] == undefined || points[k].YValues[1] == null)))) {
                points[k].isEmpty = true;
                points[k].visible = false;
            } else {
                points[k].isEmpty = points[k].isEmpty;
                points[k].visible = (points[k].isEmpty) ? !points[k].isEmpty : points[k].visible;
            }
            if (!checkNull(points[k].x)) {
                if (jQuery.type(points[k].xValue) == "date")
                    points[k].xValue = (points[k].xValue).getTime();
            }
        }
        if (chartSeries.emptyPointSettings)
            emptyPoints = this._calculateEmptyPoints(chartSeries, points);
        else
            emptyPoints = points;

        if (chartSeries.type.toLowerCase() == "waterfall")
            this.chartObj.calculateWaterfallSeriesPoints(emptyPoints);
		
        chartSeries._visiblePoints = emptyPoints;

        return emptyPoints;

    },

    _calculateEmptyPoints: function (series, points,chartObj) {
        var currentPoints;
        var emptyPoints = series.emptyPointSettings,
            style = emptyPoints.style,
            fill = style.color,
            checkNull = ej.util.isNullOrUndefined,
            borderColor = style.border.color,
            borderWidth = style.border.width,
            mode = emptyPoints.displayMode.toLowerCase(),
            emptyPointsVisible = emptyPoints.visible,
            length = points.length,
            point,
            xPoints =[],
            visiblePoints = [];
            currentPoints = $.extend(true, {}, points, currentPoints);
            seriesType = series.type.toLowerCase(),
            currentPoints.length = points.length;
            mode = ( seriesType == "boxandwhisker" )? "gap" : mode;
            if (emptyPointsVisible) {
             if (mode != 'gap') {                
                for (var i = 0; i < length; i++) {
                    currentPoints[i].actualIndex = checkNull(chartObj) ? i : (chartObj.model._isPieOfPie ? currentPoints[i].actualIndex : i);
                    point = currentPoints[i];
                    if ((point.isEmpty) && (!checkNull(point.x) || series._xAxisValueType == "string")) {
                        if (fill != '')
                            point.fill = fill;
                        if (borderColor != '') {
                            point.border = point.border ? point.border : {};
                            point.border.color = borderColor;
                            point.border.width = borderWidth;
                        }
                        switch (mode) {
                            case 'average':
                                if (series._hiloTypes) {
                                    $.each(point.YValues, function (index, value) {
                                        value = ((currentPoints[i - 1] ? currentPoints[i - 1].YValues[index] : 0) + (currentPoints[i + 1] ? (currentPoints[i + 1].isEmpty ? 0 : currentPoints[i + 1].YValues[index] ) : 0)) / 2
                                            point.YValues[index] = value;
                                            ej.EjSeriesRender.prototype._sethlocPoint(index, point, value);
                                    })
                                } else {
                                    value = ((currentPoints[i - 1] ? currentPoints[i - 1].y : 0) + (currentPoints[i + 1] ? ( currentPoints[i + 1].isEmpty ? 0 : currentPoints[i + 1].y ) : 0)) / 2;
                                    point.YValues[0] = value;
                                    point.y = value;
                                }
                                break;
                            case 'zero':
                                if (series._hiloTypes) {
                                    $.each(point.YValues, function (index, value) {
                                            point.y = 0;
                                            point.YValues[index] = 0;
                                            ej.EjSeriesRender.prototype._sethlocPoint(index, point, 0);
                                    })
                                } else {
                                    point.YValues[0] = 0;
                                    point.y = 0;
                                }
                                break;
                        }
                        point.isEmpty = false;
                        point.visible = true;
                        visiblePoints.push(point);
                    } else {
                        if (!checkNull(currentPoints[i].x) || series._xAxisValueType == "string")
                            visiblePoints.push(point);
                    }   
                }
            } else {
                 for (var i = 0; i < length; i++) {
                     currentPoints[i].actualIndex = checkNull(chartObj) ? i : (chartObj.model._isPieOfPie ? currentPoints[i].actualIndex : i);                  
                    visiblePoints[i]= (currentPoints[i]);
                }
            }
        } else {
            for (var i = 0; i < length; i++) {
                currentPoints[i].actualIndex = i;
                if (currentPoints[i].visible)
                    visiblePoints.push(currentPoints[i]);
            }
        }

        return visiblePoints;

    },
   
    _sethlocPoint: function (index,point,value) {
        switch (index) {
            case 0:
                point.high = value;
                break;
            case 1:
                point.low = value;
                break;
            case 2:
                point.open = value;
                break;
            case 3:
                point.close = value;
                break;
        }
    },

    _calculatePolarAxesSegment: function (currentseries) {

        var points = currentseries._visiblePoints;
        var segment = ej.EjSvgRender.utils._getStringBuilder();
        var SeriesIndex=$.inArray(currentseries, this.chartObj.model._visibleSeries)
        var size;
        var interval;
        var centerX = this.chartObj.model.centerX;
        var centerY = this.chartObj.model.centerY;
        // calculating side by side info.
        var sideInfo = this._getSeriesPosition(currentseries);

        size = $.extend(true, {}, currentseries.marker.size, size);
        if (size.width <= 10) {
            size.width = size.height = (1.5 * this.chartObj.model.elementSpacing);
        }
        if (currentseries.drawType.toLowerCase() == 'column') {
           
            var innerRadius = 0;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + SeriesIndex };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var interval = currentseries.xAxis.visibleRange.interval;
            currentseries.xAxis.labelPlacement = !(currentseries.xAxis.labelPlacement) ? "betweenticks" : currentseries.xAxis.labelPlacement;
            var ticks=0;
            if (currentseries.xAxis._valueType != "category" || (currentseries.xAxis._valueType == "category" && currentseries.xAxis.labelPlacement=="onticks"))
              ticks= interval / 2;
            var inversed = (currentseries.xAxis.isInversed) ? 0 : 1;
            interval = interval / sideInfo.all;
            var nextPoint = interval;
            var visiblePoints = currentseries._visiblePoints;
            var length = (currentseries.xAxis._valueType != "category") ? visiblePoints.length - 1 : visiblePoints.length;
            // calculation for start and end angle for each positions.
            for (var l = 0; l < length; l++) {
                var itemCurrentXPos = temp = (visiblePoints[l].xValue - currentseries.xAxis.visibleRange.min) + ((interval) * sideInfo.pos - ticks);
                itemCurrentXPos = (itemCurrentXPos) / this.chartObj.model.sumofYValues;
                point = visiblePoints[l];
               
                var nextPoint = (currentseries.xAxis.visibleRange.interval) / sideInfo.all;
                point.startAngle = 2 * Math.PI * itemCurrentXPos;
                point.endAngle = 2 * Math.PI * (itemCurrentXPos + ((nextPoint) / this.chartObj.model.sumofYValues));
            }
            if (!visiblePoints[visiblePoints.length - 1].startAngle) {
                visiblePoints[visiblePoints.length - 1].startAngle = visiblePoints[0].startAngle;
                visiblePoints[visiblePoints.length - 1].endAngle = visiblePoints[0].endAngle;
            }
        }
        var gap = false;
        for (var i = 0; i < points.length; i++) {
            var point1;
            var point2;
            if (points[i].visible) {

            // drawType 'Line' calculation
            if (currentseries.drawType.toLowerCase() == 'line') {
                point1 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i].xValue, points[i].y, this.chartObj);

                if (i < points.length - 1 && points[i + 1].visible) {
                    point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i + 1].xValue, points[i + 1].y, this.chartObj);
                    segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                }
                    // when isClosed is true
                else if(i == points.length - 1){
                    if (currentseries.isClosed) {
                        point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[0].xValue, points[0].y, this.chartObj);
                        segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                    }
                }            
            }

            else if (currentseries.drawType.toLowerCase() == 'column') {
                var interval = currentseries.xAxis.visibleRange.interval;
                var inversed= (currentseries.xAxis.isInversed) ? 0 : 1;
                var interval = interval / sideInfo.all;
                // radius for each point based on the y values.
                var radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, points[i].y);
                
                var chartStartAngle = -.5 * Math.PI;
                var startAngle = points[i].startAngle + chartStartAngle;
                var endAngle = points[i].endAngle + chartStartAngle - 0.000001;

                var x1,y1, x2, y2;
           
                // calculating arc when the Stacking mode is enabled
               
                if (currentseries.isStacking) {
				     var startValue = currentseries.stackedValue.StartValues[i];
                    var endValue = currentseries.stackedValue.EndValues[i];
                    radius = ((startValue==endValue) ? 0 : (this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, endValue)));

                    x1 = centerX + radius * Math.cos(startAngle);
                    y1 = centerY + radius * Math.sin(startAngle);

                    x2 = centerX + radius * Math.cos(endAngle);
                    y2 = centerY + radius * Math.sin(endAngle);

                    innerRadius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, (startValue == 0 && currentseries.yAxis.visibleRange.min != 0) ? currentseries.yAxis.visibleRange.min : startValue);

                    var dStartX = centerX + innerRadius * Math.cos(startAngle);
                    var dStartY = centerY + innerRadius * Math.sin(startAngle);

                    var dEndX = centerX + innerRadius * Math.cos(endAngle);
                    var dEndY = centerY + innerRadius * Math.sin(endAngle);
                     if (this.chartObj.model.isPolar) 
                            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + "0" + " " + inversed + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + innerRadius + " " + innerRadius + " " + "1" + " " + "0" + " " + "0" + " " + dStartX + " " + dStartY + " " + "z";
                       else
                            direction = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "L" + " " + dStartX + " " + dStartY + " " + "z";
                    }
                        // calculating arc for normal mode.
                    else {

                        x1 = centerX + radius * Math.cos(startAngle);
                        y1 = centerY + radius * Math.sin(startAngle);

                        x2 = centerX + radius * Math.cos(endAngle);
                        y2 = centerY + radius * Math.sin(endAngle);
                      if (this.chartObj.model.isPolar)
                            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + "0" + " " + inversed + " " + x2 + " " + y2 + " " + "L" + " " + centerX + " " + centerY + " " + "z";
                     else
                          direction = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2 + " " + "L" + " " + centerX + " " + centerY + " " + "z";
                    }                   
        
                   // gradient creation
                var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentseries) + "_" + SeriesIndex + i, this.chartObj.model.seriesColors[SeriesIndex], 0, 0, 0, 100, this.chartObj.gSeriesEle),
                    seriesBorder = currentseries.border;
                options = {
                     'id': this.chartObj.svgObject.id + "_Series" + SeriesIndex + "_Point" + i,
                     'fill': seriesInterior,
                     'stroke': seriesBorder.color,
                     'stroke-dasharray': seriesBorder.dashArray,
                     'stroke-width': seriesBorder.width,
                    'seriesIndex': SeriesIndex,
                    'opacity':currentseries.opacity,
                    'pointIndex': i,
                    'd': direction,
                    'start': points[i].startAngle - 1.57,
                    'end': points[i].endAngle - 1.57,
                    'radius': radius,
                    'innerR': innerRadius
                };
                if (this.chartObj.model.isPolar) {
                    options.x = centerX;
                    options.y = centerY;
					bounds = { PointIndex: i, StartAngle: points[i].startAngle, EndAngle: points[i].endAngle, CenterX: centerX, CenterY: centerY, Radius: radius, DRadius: innerRadius };
                }
				if (currentseries.type=="radar") {
                    var line1 = { x: centerX, y: centerY };
                    var line2 = { x: x1, y: y1 };
                    var line3 = { x: x2, y: y2 };
                    bounds = { PointIndex: i, Line1: line1, Line2: line2, Line3: line3 };
                    var hasStackingInnerRadius = innerRadius ? true : false;
                    options.hasStackingInnerRadius = hasStackingInnerRadius;
                }
                point = currentseries._visiblePoints[i];
                if ((!ej.util.isNullOrUndefined(currentseries.marker) && currentseries.marker.visible) || (!ej.util.isNullOrUndefined(currentseries.marker) && currentseries.marker.dataLabel.visible))
                    point.symbolLocation = { X: ((x1 + x2) / 2), Y: ((y1 + y2) / 2) };
                            
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

            }

                // drawType 'Area' calculation
            else {
                point1 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i].xValue, points[i].y, this.chartObj);
                if (i == 0 || gap) {
                    segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " ");
                }
                    // when isClosed is enabled
                else if (i == points.length - 1) {
                    if(!currentseries.isClosed){
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (centerX) + " " + (centerY)  );
                    }
                    else {
                        point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[0].xValue, points[0].y, this.chartObj);
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "z");
                    }
                }
                    
                else {
                    if (points[i + 1].visible)
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " ");
                    else
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (centerX) + " " + (centerY) + " " + "z");
                }
                if (points[i + 1] && points[i + 1].visible)
                    gap = false;
                else
                    gap = true;
            }
            if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() != 'column') {
                var bounds = { X: point1.X - size.width / 2, Y: point1.Y - size.height / 2, Width: size.width, Height: size.height };
                ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, points[i], i);
            }
			if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() == 'column' && this.chartObj.model.enableCanvasRendering)
                ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, null, null); 
            }
        }

        if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() == 'column' && !this.chartObj.model.enableCanvasRendering)
            ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, null, null);
        return segment.toString();
    },
    _calculateVisiblePoints: function (currentseries,chartObj) {
        var visiblePoints = [],
            legendPoints = [],
            emptyPointsLength,
            checkNull = ej.util.isNullOrUndefined,
            points = checkNull(chartObj) ? currentseries.points : (chartObj.model._isPieOfPie ? currentseries.pieCollections[currentseries.collectionIndex] : currentseries.points),currentPoints,
            length = points.length,
            point;

        // calculate visible points
        for (var i = 0; i < length; i++) {
            // for legend visibility
            point = points[i];
            if (checkNull(point._visibility))
                point._visibility = 'visible';
            point.actualIndex =  checkNull(chartObj)? i:(chartObj.model._isPieOfPie ? point.actualIndex :i);
			point.visible = (point.isEmpty) ? false : point.visible;
			if (checkNull(point.visible))
                point.visible = true;
            if (point.y < 0) {
                point.YValues[0] = Math.abs(point.YValues[0])
            }
            if (checkNull(point.x) && currentseries._xAxisValueType != "string") {
                point.visible = false;
                point.isEmpty = true;
            }
            if (checkNull(point.y)) {
                point.visible = false;
                point.isEmpty = true;
            }
        }
        currentPoints = $.extend(true, {}, points, currentPoints);
        currentPoints.length = length;
        if (currentseries.emptyPointSettings.visible){
            emptyPoints = this._calculateEmptyPoints(currentseries, currentPoints,chartObj);
            emptyPointsLength = emptyPoints.length;
            for (var j = 0; j < emptyPointsLength; j++) {
                if (emptyPoints[j].visible) {
                    emptyPoints[j].visible = emptyPoints[j]._visibility == 'hidden' ? false : true;
                    legendPoints.push(emptyPoints[j]);
                    if (emptyPoints[j].visible)
                        visiblePoints.push(emptyPoints[j]);
                } else {
                    //calculating gap mode values
                    emptyPoints[j].y = ((emptyPoints[j - 1] ? emptyPoints[j - 1].y : 0) + (emptyPoints[j + 1] ? emptyPoints[j + 1].y : 0)) / 2;
                    if (isNaN(emptyPoints[j].y))
                        emptyPoints[j].y = ((emptyPoints[j - 2] ? emptyPoints[j - 2].y : 0) + (emptyPoints[j + 2] ? emptyPoints[j + 2].y : 0)) / 2;
                   
				   emptyPoints[j].YValues[0] = emptyPoints[j].y;
                    if (!checkNull(emptyPoints[j].x) || currentseries._xAxisValueType == "string") {
                        emptyPoints[j].gapMode = true;
                        visiblePoints.push(emptyPoints[j]);
                    }   
                }
            }
        }            
        else {
            for (var j = 0; j < length; j++) {
				legendPoints.push(currentPoints[j]);
                if (currentPoints[j].visible) {
                    currentPoints[j].visible = currentPoints[j]._visibility == 'hidden' ? false : true;
                    if (currentPoints[j].visible)
                        visiblePoints.push(currentPoints[j]);
                }
            }
        }  
        if (currentseries.visibility == "hidden")
        {
            for(var i=0;i<legendPoints.length;i++)
            {
                legendPoints[i].visible = false;
            }
        }
        currentseries._visiblePoints = visiblePoints;
        currentseries.visiblePoints = legendPoints;

        return {
            'visiblePoints': visiblePoints,
            'legendPoints': legendPoints
        };
    },
    getMinMaxValue: function (point1, point2, degree) {
        var minX = Math.min(point1.x, point2.x);
        var minY = Math.min(point1.y, point2.y);
        var maxX = Math.max(point1.x, point2.x);
        var maxY = Math.max(point1.y, point2.y);
        var pointValue;
        switch (degree) {
            case 0:
            case 360:
                pointValue = maxY;
                break;
            case 90:
                pointValue = minX;
                break;
            case 180:
                pointValue = minY;
                break;
            case 270:
                pointValue = maxX;
                break;
        }
        return pointValue;
    },
    pieDoughnutCenter:function(currentseries)
    {
               var startAngle = currentseries.startAngle;
        var endAngle = currentseries.endAngle;
		 this.chartObj.model.centerCount ++;
		   var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
        var regions = [-630,-540,-450,-360,-270,-180,-90,0, 90, 180, 270,360,450,540,630];
        var region = [];
         var oldCenterX = this.chartObj.model.circleCenterX[seriesIndex];
        var oldCenterY = this.chartObj.model.circleCenterY[seriesIndex];

        if (currentseries.startAngle < currentseries.endAngle) {
            for (i = 0; i < regions.length;i++) {
                if (regions[i] > startAngle && regions[i] < endAngle)
                    region[region.length] = (regions[i] % 360) < 0 ? (regions[i] % 360) + 360 : (regions[i] % 360);
            }
            }
        else {
            for (i = 0; i < regions.length; i++) {
                if (regions[i] < startAngle && regions[i] > endAngle)
                    region[region.length] = (regions[i] % 360) < 0 ? (regions[i] % 360) + 360 : (regions[i] % 360);
            }
        }
		 if (this.chartObj.model.centerCount == 1) {

                var radius = this.chartObj.model.circularRadius[seriesIndex];
                var startRadian = 2 * Math.PI * (startAngle - 90) / 360;
                var endRadian = 2 * Math.PI * (endAngle - 90) / 360;
                var centerPoint = { x: this.chartObj.model.circleCenterX[seriesIndex], y: this.chartObj.model.circleCenterY[seriesIndex] };
                var startPoint = { x: this.chartObj.model.circleCenterX[seriesIndex] + radius * Math.cos(startRadian), y: this.chartObj.model.circleCenterY[seriesIndex] + radius * Math.sin(startRadian) }
                var endPoint = { x: this.chartObj.model.circleCenterX[seriesIndex] + radius * Math.cos(endRadian), y: this.chartObj.model.circleCenterY[seriesIndex] + radius * Math.sin(endRadian) }
                switch (region.length) {
                    case 0:
                        var longX = Math.abs(centerPoint.x - startPoint.x) > Math.abs(centerPoint.x - endPoint.x) ? startPoint.x : endPoint.x;
                        var longY = Math.abs(centerPoint.y - startPoint.y) > Math.abs(centerPoint.y - endPoint.y) ? startPoint.y : endPoint.y;
                        var midPoint = { x: (centerPoint.x + longX) / 2, y: (centerPoint.y + longY) / 2 }
                        this.chartObj.model.circleCenterX[seriesIndex] = this.chartObj.model.circleCenterX[seriesIndex] + (this.chartObj.model.circleCenterX[seriesIndex] - midPoint.x);
                        this.chartObj.model.circleCenterY[seriesIndex] = this.chartObj.model.circleCenterY[seriesIndex] + (this.chartObj.model.circleCenterY[seriesIndex] - midPoint.y);
                        break;
                    case 1:
                        var maxRadian = 2 * Math.PI * (region[0] - 90) / 360;
                        var maxPoint = { x: this.chartObj.model.circleCenterX[seriesIndex] + radius * Math.cos(maxRadian), y: this.chartObj.model.circleCenterY[seriesIndex] + radius * Math.sin(maxRadian) }
                        switch (region[0]) {
                            case 0:
                            case 360:
                                point1 = { x: startPoint.x, y: maxPoint.y };
                                point2 = { x: endPoint.x, y: centerPoint.y };
                                break;
                            case 90:
                                point1 = { x: centerPoint.x, y: endPoint.y };
                                point2 = { x: maxPoint.x, y: startPoint.y };
                                break;
                            case 180:
                                point1 = { x: endPoint.x, y: centerPoint.y };
                                point2 = { x: startPoint.x, y: maxPoint.y };
                                break;
                            case 270:
                                point1 = { x: maxPoint.x, y: startPoint.y };
                                point2 = { x: centerPoint.x, y: endPoint.y };
                                break;
                        }
                        var midPoint = { x: (point1.x + point2.x) / 2, y: (point1.y + point2.y) / 2 }
                        this.chartObj.model.circleCenterX[seriesIndex] = this.chartObj.model.circleCenterX[seriesIndex] + ((this.chartObj.model.circleCenterX[seriesIndex] - midPoint.x) >= radius ? 0 : (this.chartObj.model.circleCenterX[seriesIndex] - midPoint.x));
                        this.chartObj.model.circleCenterY[seriesIndex] = this.chartObj.model.circleCenterY[seriesIndex] + ((this.chartObj.model.circleCenterY[seriesIndex] - midPoint.y) >= radius ? 0 : (this.chartObj.model.circleCenterY[seriesIndex] - midPoint.y));
                        break;
                    case 2:
                        var minRadian = 2 * Math.PI * (region[0] - 90) / 360;
                        var maxRadian = 2 * Math.PI * (region[1] - 90) / 360;
                        var maxPoint = { x: this.chartObj.model.circleCenterX[seriesIndex] + radius * Math.cos(maxRadian), y: this.chartObj.model.circleCenterY[seriesIndex] + radius * Math.sin(maxRadian) }
                        var minPoint = { x: this.chartObj.model.circleCenterX[seriesIndex] + radius * Math.cos(minRadian), y: this.chartObj.model.circleCenterY[seriesIndex] + radius * Math.sin(minRadian) }
                        var point1;
                        var point2
                        if (region[0] == 90 && region[1] == 180 || region[0] == 270 && region[1] == 0)
                            point1 = { x: minPoint.x, y: maxPoint.y }
                        else
                            point1 = { x: maxPoint.x, y: minPoint.y }
                        if (region[0] == 90 || region[0] == 270)
                            point2 = { x: this.getMinMaxValue(startPoint, endPoint, region[0]), y: this.getMinMaxValue(startPoint, endPoint, region[1]) }
                        else
                            point2 = { x: this.getMinMaxValue(startPoint, endPoint, region[1]), y: this.getMinMaxValue(startPoint, endPoint, region[0]) }
                        var midPoint = { x: Math.abs(point1.x - point2.x) / 2 >= radius ? 0 : (point1.x + point2.x) / 2, y: Math.abs(point1.y - point2.y) / 2 >= radius ? 0 : (point1.y + point2.y) / 2 }
                        this.chartObj.model.circleCenterX[seriesIndex] = this.chartObj.model.circleCenterX[seriesIndex] + (midPoint.x == 0 ? 0 : (this.chartObj.model.circleCenterX[seriesIndex] - midPoint.x) >= radius ? 0 : (this.chartObj.model.circleCenterX[seriesIndex] - midPoint.x));
                        this.chartObj.model.circleCenterY[seriesIndex] = this.chartObj.model.circleCenterY[seriesIndex] + (midPoint.y == 0 ? 0 : (this.chartObj.model.circleCenterY[seriesIndex] - midPoint.y) >= radius ? 0 : (this.chartObj.model.circleCenterY[seriesIndex] - midPoint.y));
                        break;
                }
            }
            for (var i = 0; i < currentseries.points.length; i++) {
                if (seriesIndex < this.chartObj.model._visibleSeries.length && this.chartObj.model.centerCount > 1) {
                    if (this.chartObj.model._visibleSeries[seriesIndex + 1].startAngle == startAngle && this.chartObj.model._visibleSeries[seriesIndex + 1].endAngle == endAngle) {
                        this.chartObj.model.circleCenterX[seriesIndex] = this.chartObj.model.circleCenterX[seriesIndex + 1];
                        this.chartObj.model.circleCenterY[seriesIndex] = this.chartObj.model.circleCenterY[seriesIndex + 1];
                    }
                }
                this.chartObj.model.startX[i] = this.chartObj.model.startX[i] - (oldCenterX - this.chartObj.model.circleCenterX[seriesIndex]);
                this.chartObj.model.startY[i] = this.chartObj.model.startY[i] - (oldCenterY - this.chartObj.model.circleCenterY[seriesIndex]);


            }

    },
    _drawEmptyPieOfPie: function(chartObj){
        var startAngle = -.5 * Math.PI,
             endAngle = 360 * (Math.PI / 180) + (-.5 * Math.PI) - 0.000001,
             startX = chartObj.model.circleCenterX[1],
             startY = chartObj.model.circleCenterY[1],
             radius = chartObj.model.circularRadius[1];
        var direction = "M" + " " + (startX + radius * Math.cos(startAngle)) + " " + (startY + radius * Math.sin(startAngle)) + " " + "A" + " " + radius + " " + radius + " " + "0" + " " +
            1 + " " + 1 + " " + (startX + radius * Math.cos(endAngle)) + " " + (startY + radius * Math.sin(endAngle)) + " " + "z";
        var theme = chartObj.model.theme.toLowerCase();
		 var pathOptions = {
            'id': chartObj.svgObject.id + '_Series' + 0 + "_EmptyCircle",
            'fill': 'none',
            'stroke-width': 1.5,
            'stroke': theme.indexOf("dark") >= 0 || theme.indexOf("contrast") >= 0 ? " #FFFFFF" : "#000000",
            'stroke-dasharray': '',
            'd': direction,
            'opacity': 1,
            'radius': radius,
            'start': 0 - 1.57,
            'end': (360 * (Math.PI / 180)) - 1.57,
            'cx': startX,
            'cy': startY
        };
        return pathOptions;
    },
    calculatingSliceAngle: function (currentseries, chartObj) {
        var chart = this.chartObj ? this.chartObj : chartObj,
            chartModel = chart.model,
            legend = chartModel.legend,
			 seriesIndex = $.inArray(currentseries, chartModel._visibleSeries),
            legendPosition = legend.position.toLowerCase(),
            legendActualBounds = chartModel.LegendActualBounds,
            visiblePoints = currentseries._visiblePoints,
			visiblePointslength = visiblePoints.length,
            legXSpace = 0, totalDegree,
            legYSpace = 0,
            legendBorder = legend.border.width,
            border = chartModel.border.width,
            point, margin = chartModel.margin,
            elementSpacing = chartModel.elementSpacing,
            itemCurrentXPos = 0,
            subTitle=chartModel.title.subTitle.text?chartModel.title.subTitle:'';
        chartModel.arcData = [];
        currentseries.rightsidePoints = [];
        currentseries.leftsidePoints = [];
        currentseries.labels = [];
        chartModel.sumofYValues = 0;
        chartModel.midPoint = 0;
        type = currentseries.type.toLowerCase();
		gapWidthValue = currentseries.gapWidth;


        for (j = 0; j < visiblePoints.length; j++) {
            chartModel.sumofYValues += visiblePoints[j].YValues[0];
        }
        if (currentseries.endAngle != null && type!="pieofpie")
            totalDegree = (currentseries.endAngle - currentseries.startAngle);
        else
            totalDegree = 360;
        totalDegree = (totalDegree != 360 && totalDegree != -360) ? totalDegree % 360 : totalDegree;
        if (currentseries.startAngle && type!="pieofpie") {
            currentseries.startAngle = currentseries.startAngle % 360;
            currentseries.endAngle = currentseries.startAngle + totalDegree;
            itemCurrentXPos = (currentseries.startAngle) ? ((chartModel.sumofYValues / 360) * currentseries.startAngle) : 0;
            itemCurrentXPos = itemCurrentXPos / chartModel.sumofYValues;
            chartModel.itemCurrentXPos = itemCurrentXPos;
        }
        else
            currentseries.endAngle = totalDegree;
        if (legend.visible && legendPosition != "custom") {
            if (legendPosition == "right" || legendPosition == "left")
                legXSpace += legendActualBounds.Width + (legendBorder * 2) + elementSpacing;
            else
                legYSpace += legendActualBounds.Height + (legendBorder * 2) + elementSpacing;

        }

        var yOffset = ((chartModel.title.text && chartModel.title.visible) ? chartModel._titleLocation.size.height + chartModel.elementSpacing + (subTitle == '' ? 0 : (subTitle.text && subTitle.visible && subTitle.enableTrim && (subTitle.textOverflow == 'wrap' || subTitle.textOverflow == 'wrapandtrim') ? chartModel._subTitleLocation.size.height : 0)) : 0);
     
          var constantWidth = 0.1* Math.min($(chart.svgObject).width(), $(chart.svgObject).height());
        var actualWidth = $(chart.svgObject).width() - (legXSpace + margin.left + margin.right + border * 2) - (type == "pieofpie" ? gapWidthValue : 0);
        var actualHeight = $(chart.svgObject).height() - (legYSpace + yOffset + margin.top + margin.bottom + border * 2);
        seriesIndex = chart.model._isPieOfPie ? currentseries.collectionIndex : seriesIndex;
      
        if (type == "pieofpie") {
            if (actualWidth < constantWidth) {
                var widthDifference = constantWidth - actualWidth;
                actualWidth = constantWidth;
            }
            gapWidthValue = widthDifference ? gapWidthValue - widthDifference : $(chart.svgObject).width() >= gapWidthValue ? gapWidthValue : 0;
        }
        var value = type == "pieofpie" ? (seriesIndex == 0 ? 0.25 : 0.25 * 3) : 0.5;
        chartModel.circleCenterX[seriesIndex] = (actualWidth * value) + (seriesIndex == 1 && type == "pieofpie" ? gapWidthValue : 0) + margin.left + ((legend.visible && legendPosition === "left") ? legXSpace : 0);
            chartModel.circleCenterY[seriesIndex] = (actualHeight * 0.5) + margin.top + yOffset + ((legend.visible && legendPosition === "top") ? legYSpace : 0);
    
        if (!ej.util.isNullOrUndefined(chart.model.pieGapWidth) && chart.model.pieGapWidth != 0) {
            if (seriesIndex == 0)
                chartModel.circleCenterX[seriesIndex] = chartModel.circleCenterX[seriesIndex] + chart.model.pieGapWidth / 2;
            else
                chartModel.circleCenterX[seriesIndex] = chartModel.circleCenterX[seriesIndex] - chart.model.pieGapWidth / 2;
        }
        var endAngle;
        currentseries.startAngle = (type == "pieofpie") ? 0 : currentseries.startAngle;
        totalDegree = currentseries.endAngle - currentseries.startAngle;
        var pieFactor = totalDegree / 180;
        var circleMidAngle = (totalDegree / 4) * (Math.PI / 180);
        for (var l = 0; l < visiblePointslength; l++) {
            point = currentseries._visiblePoints[l];
            if (point.visible || point.gapMode) {
                var pointIndex = type=="pieofpie"? point.actualIndex : l;
                if (l == 0 && seriesIndex == 0 && type == "pieofpie") {
                    var startAngle = 2 * Math.PI * itemCurrentXPos;
                    var endAngle = pieFactor * Math.PI * (point.YValues[0] / chartModel.sumofYValues) + startAngle;
                    endAngle = (isNaN(endAngle)) ? startAngle : endAngle;
                    midAngle = (endAngle + startAngle) / 2;
                    point.start = point.startAngle = circleMidAngle - midAngle;
                    point.end = point.endAngle = circleMidAngle + midAngle;
                    point.midAngle = circleMidAngle;
                }
                else {
                    if (l == 0)
                        point.startAngle = 2 * Math.PI * itemCurrentXPos;
                    else
                        point.startAngle = (type == "pieofpie") ? currentseries.pieCollections[seriesIndex][l - 1].endAngle : endAngle;
                    endAngle = point.endAngle = pieFactor * Math.PI * (point.YValues[0] / chartModel.sumofYValues) + point.startAngle;
                    endAngle = point.endAngle = (isNaN(endAngle)) ? point.startAngle : endAngle;
                    point.start = point.startAngle;
                    point.end = point.endAngle;
            point.midAngle = (point.endAngle + point.startAngle) / 2;
                }
            point.pointIndex = pointIndex;
            var chartStartingAngle = -.5 * Math.PI;

                point.radian = (point.midAngle) % (2 * Math.PI)

                if (currentseries.labelPosition.toLowerCase() == "outsideextended" && type != "pieofpie") {

                    if (point.radian < Math.PI)
                        currentseries.rightsidePoints.push(point);
                    else
                        currentseries.leftsidePoints.push(point);
                }
                itemCurrentXPos += point.YValues[0] / chartModel.sumofYValues;

                var midAngle = point.midAngle + chartStartingAngle;

                if ((point.actualIndex == currentseries.explodeIndex || currentseries.explodeAll) && !chart.vmlRendering) {

                    chartModel.startX[pointIndex] = chartModel.circleCenterX[seriesIndex] + Math.cos(midAngle) * currentseries.explodeOffset;
                    chartModel.startY[pointIndex] = chartModel.circleCenterY[seriesIndex]+ Math.sin(midAngle) * currentseries.explodeOffset;
                }
                else {
                    chartModel.startX[pointIndex] = chartModel.circleCenterX[seriesIndex];
                    chartModel.startY[pointIndex] = chartModel.circleCenterY[seriesIndex];
                }
            }

        }

        if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
            currentseries.rightsidePoints = ej.DataManager(currentseries.rightsidePoints, ej.Query().sortBy("radian")).executeLocal();
            currentseries.leftsidePoints = ej.DataManager(currentseries.leftsidePoints, ej.Query().sortBy("radian")).executeLocal();
        }
        if (currentseries.marker.dataLabel.template) {
            if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
                for (var i = 0; i < currentseries.rightsidePoints.length; i++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.rightsidePoints[i], currentseries.rightsidePoints[i].actualIndex, currentseries, false, chart);
                }
                for (var k = 0; k < currentseries.leftsidePoints.length; k++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.leftsidePoints[k], currentseries.leftsidePoints[k].actualIndex, currentseries, true, chart);
                }

            }
            else {
                for (var j = 0; j < currentseries._visiblePoints.length; j++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries._visiblePoints[j], currentseries._visiblePoints[j].actualIndex, currentseries, true, chart);
                }
            }
            
        }
        else {
            ej.EjSvgRender.utils._getSeriesMaxLabel(currentseries);
        }

      
        currentseries.finalSize= { width: actualWidth, height: actualHeight };
        return currentseries.finalSize;
    },
	 _drawHiloPath: function (series, style, interior, direction, i, pointbounds) {

	     var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);

        var options = {
            'name': series.type,
            'id': this.chartObj.svgObject.id + "_Series" + seriesIndex + "_Point" + i,
            'fill': "none",
            'stroke-dasharray': series.dashArray,
            'stroke-width': style.borderWidth,
            'stroke': interior,
            'stroke-linecap': series.lineCap,
            'stroke-linejoin': series.lineJoin,
            'opacity': series.opacity,
            'd': direction
        };
        this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        var valWidth, valHeight;
        var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.chartObj.model.requireInvertedAxes);
        x = (Math.min(pointbounds.point2.X, pointbounds.point1.X) +trans.x) - (style.borderWidth);
        y = (Math.min(pointbounds.point2.Y, pointbounds.point1.Y) +trans.y);
        if (pointbounds.point1.Y != pointbounds.point2.Y) {
            valWidth = style.borderWidth + (style.borderWidth / 2);
              valHeight = Math.abs(pointbounds.point1.Y - pointbounds.point2.Y);
        }
	     else {
            valWidth = Math.abs(pointbounds.point2.X - pointbounds.point1.X);
            valHeight = style.borderWidth + (style.borderWidth / 2); 
        }
        var bounds = { X: x, Y: y, Width: valWidth, Height: valHeight };
        ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
      
	 },
	 isIntersec: function (other, currBounds) {
	     if (currBounds.Bottom < other.Top || currBounds.Right < other.Left
               || currBounds.Top > other.Bottom || currBounds.Left > other.Right) {
	         return false;
	     }
	      return true;
	 },
	 IntersectWith: function (sender, point, currBounds, labelPosition, seriesIndex) {
	     var chartObj = this,intersect = false,
	      smartIndex;
	     if (ej.util.isNullOrUndefined(sender.model.rightsidebounds[seriesIndex]) || ej.util.isNullOrUndefined(sender.model.leftsidebounds[seriesIndex])) {
	         sender.model.leftsidebounds[seriesIndex] = [];
	         sender.model.rightsidebounds[seriesIndex] = [];
	         sender.model.leftsidebounds[seriesIndex].points = [];
	         sender.model.rightsidebounds[seriesIndex].points = [];
	     }
	     if (labelPosition == "outsideExtended") {
                for (i = seriesIndex; i < sender.model.bounds.length ; i++) {
                    if (sender.model._visibleSeries[i].labelPosition.toLowerCase() == "outsideextended") {
                        if ((!ej.util.isNullOrUndefined(sender.model.bounds[i].points["left"])) && (sender.model.bounds[i].points["left"].length > 0)) {
                            var pointsLength = sender.model.bounds[i].points["left"].length;
                            for (j = pointsLength - 1; j >= 0 && !(ej.util.isNullOrUndefined(sender.model.bounds[i].points["left"][j])) ; j--) {
                                sender.model.leftsidebounds[i].points[j] = sender.model.bounds[i].points["left"][j];

                            }
                        }
                        if ((!ej.util.isNullOrUndefined(sender.model.bounds[i].points["right"])) && (sender.model.bounds[i].points["right"].length > 0)) {
                            var pointsLength = sender.model.bounds[i].points["right"].length;
                            for (k = 0; k < pointsLength && !(ej.util.isNullOrUndefined(sender.model.bounds[i].points["right"][k])) ; k++) {
                                sender.model.rightsidebounds[i].points[k] = sender.model.bounds[i].points["right"][k];
                            }
                        }

                    }
                }
            }
            else {

                if (sender.model.bounds[seriesIndex].points.length > 0 || seriesIndex >= 0) {
                    for (i = seriesIndex; i < sender.model.bounds.length && (!ej.util.isNullOrUndefined(sender.model.bounds[i])) ; i++) {
                        if (sender.model._visibleSeries[i].labelPosition.toLowerCase() != "outsideextended") {
                            var pointsLength = sender.model.bounds[i].points.length;
                            for (j = 0; j < pointsLength && !(ej.util.isNullOrUndefined(sender.model.bounds[i].points[j])) ; j++) {
                                if (sender.model.bounds[i].points[j].X < sender.model.circleCenterX[seriesIndex]) {
                                    if (($.inArray(sender.model.bounds[i].points[j], sender.model.leftsidebounds[i].points)) == -1)
                                        sender.model.leftsidebounds[i].points[j] = sender.model.bounds[i].points[j];
                                }
                                else if (($.inArray(sender.model.bounds[i].points[j], sender.model.rightsidebounds[i].points)) == -1)
                                    sender.model.rightsidebounds[i].points[j] = (sender.model.bounds[i].points[j]);

                            }
                        }
                    }
                }

            }
            for (i = seriesIndex; i < sender.model.bounds.length && (!ej.util.isNullOrUndefined(sender.model.bounds[i])) ; i++) {
                if (sender.model.leftsidebounds[i].points.length > 0) {
                    bounds = sender.model.leftsidebounds[i].points;
                    if (bounds.length > 0) {
                        for (var j = bounds.length - 1  ; j >= 0 ; j--) {
                            var other = bounds[j];
                          
                                if (!(ej.util.isNullOrUndefined(other) && other != currBounds)) {
                                if (other.SeriesIndex == currBounds.SeriesIndex && other.PointIndex == currBounds.PointIndex) {
                                    if (point.smartLabelPosition == "outside")
                                        continue;
                                    else
                                        break;
                                }
                                else if (this.isIntersec(other, currBounds))
                                    return true;
                            }
                        }
                    }

                }
            }
            for (i = seriesIndex; i < sender.model.bounds.length && (!ej.util.isNullOrUndefined(sender.model.bounds[i])) ; i++) {
                bounds = sender.model.rightsidebounds[i].points;
                if (bounds.length > 0) {
                    for (var j = bounds.length - 1 ; j >= 0; j--) {
                        var other = bounds[j];
                        if (!(ej.util.isNullOrUndefined(other) && other != currBounds)) {
                                if (other.SeriesIndex == currBounds.SeriesIndex && other.PointIndex == currBounds.PointIndex) {
                                    if (point.smartLabelPosition == "outside")
                                        continue;
                                    else
                                        break;
                                }
                                else if (this.isIntersec(other, currBounds))
                                    return true;
                            }
                    }

                }
            }
	     return intersect;
	     
	 },
   
   
	 _calculateArcData: function (sender, pointIndex, point, series, seriesIndex, pieSeriesIndex) {
	     var seriesIndex = sender.model._isPieOfPie ? pieSeriesIndex : seriesIndex;
	     var visiblePoints = series._visiblePoints;
	     var index = point.actualIndex;
	     var chartObj = sender;
	     var chartStartAngle = -.5 * Math.PI;
	     var currBounds, startAngle, endAngle;
         // fix for doughnut width is inconsistent when series having single point-EJMVC-6272
	     if (series.type.toLowerCase() == "doughnut" && series._visiblePoints.length == 1 && Math.abs(series.endAngle - series.startAngle) == 360 
		 && series.startAngle % 90 != 0) {
	         startAngle = parseFloat((point.startAngle + chartStartAngle).toFixed(3));
	         endAngle = parseFloat((point.endAngle + chartStartAngle).toFixed(3)) - 0.000001;
	     }
	     else
	     {
	         startAngle = point.startAngle + chartStartAngle;
	         endAngle = point.endAngle + chartStartAngle - 0.000001;
	     }
	     var totalDegree = series.endAngle - series.startAngle;
	     var longArc = endAngle - startAngle < Math.PI ? 0 : 1;
	     var midAngle = (startAngle + endAngle) / 2;
		  if (ej.util.isNullOrUndefined(point.currentMidAngle)) {
	         point.currentMidAngle = (startAngle + endAngle) / 2;
	     }
	     var direction;
	     var startX;
	     var startY
	     var clockWise=totalDegree > 0?1:0;
	     var longArc = clockWise ? endAngle - startAngle < Math.PI ? 0 : 1 : endAngle - startAngle > -1 * Math.PI ? 0 : 1;
	     var radius = chartObj.model.circularRadius[seriesIndex] ;
		 var innerRadius = chartObj.model.innerRadius[seriesIndex] ;
	     if ((index == series.explodeIndex || series.explodeAll) && !chartObj.vmlRendering) {
           
	         startX = chartObj.model.circleCenterX[seriesIndex] + Math.cos(midAngle) * series.explodeOffset;
	         startY = chartObj.model.circleCenterY[seriesIndex] + Math.sin(midAngle) * series.explodeOffset;
	     }
	     else {
	         startX = chartObj.model.circleCenterX[seriesIndex];
	         startY = chartObj.model.circleCenterY[seriesIndex];
	     }


	     var x1 = startX + radius* Math.cos(startAngle);
	     var y1 = startY + radius * Math.sin(startAngle);

	     var x2 = startX + radius * Math.cos(endAngle);
	     var y2 = startY + radius * Math.sin(endAngle);
	    
	     if (series.type.toLowerCase() == "doughnut") {
	         var dStartX = startX + innerRadius * Math.cos(startAngle);
	         var dStartY = startY + innerRadius * Math.sin(startAngle);

	         var dEndX = startX + innerRadius * Math.cos(endAngle);
	         var dEndY = startY + innerRadius * Math.sin(endAngle);
	         var dClockWise = clockWise ? 0 : 1;
	         if ((startAngle < 0) && Math.round(point.endAngle - point.startAngle) == 6)
	         {
	             dEndX = dEndX - 0.01;
	             x2 = x2 - 0.01;
	         }
	         direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " +innerRadius+ " " + innerRadius + " " + "1" + " " + longArc + " " + dClockWise + " " + dStartX + " " + dStartY + " " + "z";
	     }

	     else{
	         if ((point.endAngle-point.startAngle).toFixed(4) == (2*Math.PI).toFixed(4)){
				 var centerx = startX;
	             var centery = startY;
	             direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2;
            }
			else
	             direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + startX + " " + startY + " " + "z";
	     }

        // display label for pie and doughnut chart
	     

        return { Direction: direction, centerX: centerx, centerY: centery };

    },

    getXCordinate: function (x, radius, angle) {
        var x1 = x + radius * (Math.cos(angle));
        return x1;
    },

    getYCordinate: function (y, radius, angle) {
        var y1 = y + radius * (Math.sin(angle));
        return y1;
    },
 getDoubleRange: function (start, end) {
        var mstart;
        var mend;
        if (start > end) {
            mstart = end;
            mend = start;
        }
        else {
            mstart = start;
            mend = end;
        }

        var mdelta = mend - mstart;
        var mmedian = (mstart + mend) / 2;
        var misEmpty = isNaN(mstart) || isNaN(mend);

        return { Start: mstart, End: mend, Delta: mdelta, Median: mmedian, IsEmpty: misEmpty };

    },
    isDependentSeries: function (series) {
        var type = series.type.toLowerCase();
        var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
       if((!requireInvertedAxes) || (requireInvertedAxes && series.isTransposed))
           return ((type.indexOf("bar") != -1 && !series._isTransposed) || (type.indexOf("column") != -1) || (type.indexOf("waterfall") != -1) || (type.indexOf("hilo") != -1) || (type.indexOf("candle") != -1) || (type.indexOf("boxandwhisker") != -1) ) ? true : false;
        else
            return (type.indexOf("bar") != -1) ? true : false;
    },

    getPointXYOrgin: function (x, y, series, sender) {
	     var requireInvertedAxes = sender.model.requireInvertedAxes;
		 var xvalue;
		 var yvalue;
        if ((series.isTransposed && requireInvertedAxes) || series._isTransposed) {
            xvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x)) * (series.xAxis.height);
            yvalue = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.width);
            return { X: yvalue, Y: xvalue };
        }
        else {
             xvalue = ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x) * (series.xAxis.width);
             yvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.height);
            return { X: xvalue, Y: yvalue };
        }
    },
    getRectangle: function (x1, y1, x2, y2, series, sender) {
        var pt1 = this.getPointXYOrgin(x1, y1, series, sender);
        var pt2 = this.getPointXYOrgin(x2, y2, series, sender);
        return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
    },
	
	calculateMean: function (values,series,pointIndex){
	
		var q1Arr,q3Arr,q1Arrodd,q1Arreven,q2Arr ,q3Arrodd,q3Arreven,q1half,q3half,midhalf,lowerquartile,upperquartile;
		var wholenumlower,remainderlower,intpartlower,decimalpartlower,wholenumupper,remainderupper,intpartupper,decimalpartupper,
		intpartlowerposition,intpartupperposition,averageValue = 0,boxAverageValue = 0 ,boxWhiskerValues =[],boxWhiskerCount=0,boxes=[],boxArrCount=0;
		if(values.length > 1)
		values.sort( function(a,b) {return a - b;} );
		
		//showMean calculation
		for(var q=0;q<values.length;q++)
		{
			averageValue = averageValue + values[q];
		}
		boxAverageValue = averageValue/values.length ;
		boxAverageValue = parseInt(boxAverageValue.toFixed(2));
		boxes[boxArrCount] =boxAverageValue;
		boxArrCount++;
	
		if(series.boxPlotMode.toLowerCase() == "normal")
		{
			//Lower quartile calculation for normal mode
			q1Arr = (values.length % 2 == 0) ? values.slice(0, (values.length / 2) ) : values.slice(0, Math.floor(values.length / 2) );
			q1half = Math.floor(q1Arr.length/2);			
			lowerquartile = q1Arr.length % 2 == 0 ? (q1Arr[q1half-1] + q1Arr[q1half]) / 2 :  q1Arr[q1half];
			lowerquartile = (values.length <= 3) ? values[0] : lowerquartile;
			boxes[boxArrCount] = lowerquartile;
			boxArrCount++;
			//Upper quartile calculation for normal mode 	 
			q3Arr = (values.length % 2 == 0) ? values.slice( (values.length/2), values.length) : values.slice(Math.ceil(values.length / 2), values.length);
			q3half = Math.floor(q3Arr.length/2);
			upperquartile = q3Arr.length % 2 == 0 ? (q3Arr[q3half-1] + q3Arr[q3half]) / 2 :  q3Arr[q3half];
			upperquartile = (values.length <= 3) ? values[values.length - 1] : upperquartile;
			boxes[boxArrCount] = upperquartile;
			boxArrCount++;
		}
		else if(series.boxPlotMode.toLowerCase() == "exclusive" || series.boxPlotMode.toLowerCase() == "inclusive")
		{
		
			//Lower quartile calculation for exclusive meadian and inclusive median 
			wholenumlower = series.boxPlotMode .toLowerCase() == "exclusive" ? (values.length + 1)/4 : (values.length - 1)/4 ;
			intpartlower = Math.floor(wholenumlower);
			decimalpartlower = wholenumlower - intpartlower ;
			intpartlowerposition = series.boxPlotMode.toLowerCase() == "exclusive" ? intpartlower - 1 : intpartlower ;
			q1Arr = (1 - decimalpartlower) * values[intpartlowerposition] + decimalpartlower * values[intpartlowerposition + 1];
			q1Arr = (values.length <= 3) ? values[0] : q1Arr;
			lowerquartile = q1Arr;
			boxes[boxArrCount] = lowerquartile;
			boxArrCount++;
								
			//Upper quartile calculation for exclusive meadian and inclusive median 
			wholenumupper = series.boxPlotMode.toLowerCase() == "exclusive" ? (3 * (values.length + 1))/4 : (3 * (values.length - 1))/4 ;
			intpartupper = Math.floor(wholenumupper);
			decimalpartupper = wholenumupper - intpartupper;
			intpartupperposition = series.boxPlotMode.toLowerCase() == "exclusive" ? intpartupper - 1 : intpartupper;
			q3Arr = (1 - decimalpartupper) * values[intpartupperposition] + decimalpartupper * values[intpartupperposition + 1];
			q3Arr = (values.length <= 3) ? values[values.length - 1] : q3Arr;
			upperquartile = q3Arr;
			boxes[boxArrCount] = upperquartile;
			boxArrCount++;
		}
				
				
	    //calculate median
		midhalf = Math.floor(values.length/2);
		q2Arr = values.length % 2 == 0 ? (values[midhalf-1] + values[midhalf]) / 2 :  values[midhalf];
		boxes[boxArrCount] = q2Arr;
		boxArrCount++;
		
		//calculate interquartile range
		var iqr = upperquartile - lowerquartile;
		var upperrange = upperquartile + 1.5 * (iqr);
		var lowerrange = lowerquartile - 1.5 * (iqr);
		var maximum = values[values.length-1];
		var minimum = values[0];		
		var outliers =[],maxNumCount=0,index=0;
		
		
		for(var i=0;i<values.length;i++)
		{
		 if(upperrange < values[i])
			{
				maxNumCount++;
			    outliers[index]=values[i];
				index++;
				maximum = values[i-maxNumCount];
				
			}
			if(lowerrange > values[i])
			{
				outliers[index] = values[i];
				index++;
				minimum = values[i+1];
			}
		}
		boxes[boxArrCount] = minimum;
		boxArrCount++;
		boxes[boxArrCount] = maximum;
		boxArrCount++;
				
		var boxPlotValues = {Minimum : minimum, LowerQuartile : lowerquartile , midvalue : q2Arr , UpperQuartile : upperquartile , Maximum : maximum,boxAverage : boxAverageValue,outliers:outliers};
		series._visiblePoints[pointIndex].boxPlotValues = boxPlotValues;
		series._visiblePoints[pointIndex].boxPlotValues.quartileValues = boxes;
		
		
		
		
	  return {Minimum : minimum, LowerQuartile : lowerquartile , midvalue : q2Arr , UpperQuartile : upperquartile , Maximum : maximum,boxAverage : boxAverageValue,outliers:outliers};
},
	calculateBoxAndWhiskerPath : function(options,rect,options1,list2,series,point,pointIndex)	{
	
	    series._visiblePoints[pointIndex].boxPlotLocation = [];
	    series._visiblePoints[pointIndex].dataLabelLocation = [];
		var boxCount=0;			
		var cSer = this,offset=5;
		var isTransposed = series._isTransposed;
		var invertedAxis = this.chartObj.model.requireInvertedAxes,whiskerpath,boxAvg,boxAvgValue,fenceValue,outlierpoint;
		
		
		for(var p=0;p<series._visiblePoints[pointIndex].boxPlotValues.quartileValues.length;p++)
		{
			boxAvg = {"xValue": point.xValue, "YValues":series._visiblePoints[pointIndex].boxPlotValues.quartileValues[p]  };		
			boxAvgValue = ej.EjSvgRender.utils._getPoint(boxAvg, series);
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount] = boxAvgValue;
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount].xValue  = boxAvg.xValue ;
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount].YValues = boxAvg.YValues;
			if(isTransposed)
			    series._visiblePoints[pointIndex].dataLabelLocation[boxCount] = { X: boxAvgValue.X, Y: rect.Y + rect.Height / 2 + offset}
			else
			    series._visiblePoints[pointIndex].dataLabelLocation[boxCount] = { X: rect.X + rect.Width / 2 ,Y: boxAvgValue.Y};
			boxCount++;
		
		}
							
		for(var k=0;k<list2.outliers.length;k++)
		{
			fenceValue = {"xValue": point.xValue, "YValues": list2.outliers[k] };						
			outlierpoint =  ej.EjSvgRender.utils._getPoint(fenceValue, series);												
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount] = outlierpoint ;//to store x and y positions of upper outlier
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount].xValue = fenceValue.xValue;
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount].YValues = fenceValue.YValues;								
			series._visiblePoints[pointIndex].boxPlotLocation[boxCount].outlier = true;
			if(isTransposed)
				outlierpoint.Y = rect.Y + rect.Height / 2;
			else
				outlierpoint.X = rect.X + rect.Width / 2;
			if (isTransposed)
			    series._visiblePoints[pointIndex].dataLabelLocation[boxCount] = { X:outlierpoint.X - series.outlierSettings.size.height ,Y: rect.Y + rect.Height / 2 + series.outlierSettings.size.width   ,outlier:true}
			else
			    series._visiblePoints[pointIndex].dataLabelLocation[boxCount] = { X: rect.X + rect.Width / 2, Y: outlierpoint.Y ,outlier:true};
			boxCount++;	
		}	
		whiskerpath = series._visiblePoints[pointIndex].boxPlotLocation;
		
		if(series._isTransposed)
		{
		    var plotlowerwhisker = "M" + " " + (cSer.chartObj.canvasX + whiskerpath[4].X)   + " " + (cSer.chartObj.canvasY + rect.Y) + " L " + (whiskerpath[4].X + cSer.chartObj.canvasX)  + " " + (rect.Height + rect.Y + cSer.chartObj.canvasY) + " M " + (cSer.chartObj.canvasX + whiskerpath[4].X)  + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height/2)) + " L " + (cSer.chartObj.canvasX + whiskerpath[1].X) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height/2))  + " " + " z"; 			
		    var plotupperwhisker = "M" + " " +  (cSer.chartObj.canvasX + whiskerpath[5].X) + " " + (cSer.chartObj.canvasY + rect.Y) + " L " + (cSer.chartObj.canvasX + whiskerpath[5].X) + " " + (rect.Height + rect.Y + cSer.chartObj.canvasY) + " M " + (cSer.chartObj.canvasX + whiskerpath[5].X) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height/2)) + " L " + (cSer.chartObj.canvasX + whiskerpath[2].X) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height/2))  + " " + " z"; 
 		
		}
		else
		{
		    var plotlowerwhisker = "M" + " " + (cSer.chartObj.canvasX + rect.X) + " " + (cSer.chartObj.canvasY + whiskerpath[4].Y) + " L " + (cSer.chartObj.canvasX + rect.X + rect.Width) + " " + (cSer.chartObj.canvasY + whiskerpath[4].Y) + " M " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2)) + " " + (cSer.chartObj.canvasY + whiskerpath[4].Y) + " L " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2)) + " " + (cSer.chartObj.canvasY + whiskerpath[1].Y) + " " + " z";
		    var plotupperwhisker = "M" + " " + (cSer.chartObj.canvasX + rect.X) + " " + (cSer.chartObj.canvasY + whiskerpath[5].Y) + " L " + (cSer.chartObj.canvasX + rect.X + rect.Width) + " " + (cSer.chartObj.canvasY + whiskerpath[5].Y) + " M " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2)) + " " + (cSer.chartObj.canvasY + whiskerpath[5].Y) + " L " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2)) + " " + (cSer.chartObj.canvasY + whiskerpath[2].Y) + " " + " z";
		}
					
		var meanPath = !series._isTransposed ? "M" + " " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2) - 5) + " " + (cSer.chartObj.canvasY + whiskerpath[0].Y - 5) + " L " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2) + 5) + " " + (cSer.chartObj.canvasY + whiskerpath[0].Y + 5) + " M " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2) - 5) + " " + (cSer.chartObj.canvasY + whiskerpath[0].Y + 5) + " L " + (cSer.chartObj.canvasX + rect.X + (rect.Width / 2) + 5) + " " + (cSer.chartObj.canvasY + whiskerpath[0].Y - 5) + " " + "z" :
			"M" + " " + (cSer.chartObj.canvasX + whiskerpath[0].X - 5) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height / 2) - 5) + " L " + (cSer.chartObj.canvasX + whiskerpath[0].X + 5) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height / 2) + 5) + " M " + (cSer.chartObj.canvasX + whiskerpath[0].X + 5) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height / 2) - 5) + " L " + (cSer.chartObj.canvasX + whiskerpath[0].X - 5) + " " + (cSer.chartObj.canvasY + rect.Y + (rect.Height / 2) + 5) + " " + "z";

							
		return { upperWhisker : plotupperwhisker , lowerWhisker : plotlowerwhisker , mean : meanPath };
},		
    calculateSides: function (point, sidebysideinfo) {
        var x1 = point.xValue + sidebysideinfo.Start;
        var x2 = point.xValue + sidebysideinfo.End;
        return { x1: x1, x2: x2 };
    },
    _getSeriesPosition: function (currentSeries) {
     // calculation for position of series in radar and polar.
        if (ej.util.isNullOrUndefined(currentSeries.position) || this.chartObj.model.legendCollapsed) {
            var stacked;
            var stackingposition
            var all=0;
            var seriesCollection=[];
            for (var i = 0; i < this.chartObj.model._visibleSeries.length; i++) {
                var series=this.chartObj.model._visibleSeries[i];
                if (series.visibility.toLowerCase() === 'visible' && series.drawType.toLowerCase() == 'column') {
                    seriesCollection.push(series);
                }
            }
            for (var j = 0; j < seriesCollection.length; j++) {
                var series = seriesCollection[j];
			    if (series._xAxisName == this.chartObj.model._axes[0].name && series._yAxisName == this.chartObj.model._axes[1].name) {
                    if (series.isStacking) {
                        if (!stacked) {
                            stackingposition = all;
                            all++;
                            stacked = true;
                        }
                        series.position = stackingposition;
                    }
                    else {
                        series.position = all;
                        all++;
                      }
				}	  
            }
            for (var k = 0; k < seriesCollection.length; k++) 
                seriesCollection[k].all = all;
            
        }
        return { all: currentSeries.all, pos: currentSeries.position };
    },
     getSideBySidePositions: function (currentSeries, params) {

         var chart = this, 
            stackingGroup = [],
            model = chart.chartObj.model,
            vAxeseries,
            i = j = k = 0,
            visibleSeriesLength=model._visibleSeries.length;
		 //Spacing not calculated during resize event because position value in series is not null or undefined		 
         if (ej.util.isNullOrUndefined(currentSeries.position) || model.legendCollapsed || !params.spacingCalculated) {
             var hAxes = (model.requireInvertedAxes) ? model.vAxes : model.hAxes;
             var vAxes = (model.requireInvertedAxes) ? model.hAxes : model.vAxes;

             $.each(hAxes, function (index, hAxis) {
                 var all = 0, seriesCollection = [];
				 vAxeseries=[];
                 for (j = 0; j <visibleSeriesLength; j++) {
                     var series = model._visibleSeries[j];
                     if (series.visibility.toLowerCase() === 'visible') {
                         if (chart.isDependentSeries(series)) {
                             if (hAxis.name == series._xAxisName) {
                                 seriesCollection.push(series);
                             }
                         }
                     }
                 }
				  var seriesCollectionLength=seriesCollection.length;                 
                 for (j = 0; j < seriesCollectionLength; j++) {
                     var index = (model.requireInvertedAxes) ? seriesCollection[j].yAxis.columnIndex : seriesCollection[j].yAxis.rowIndex;
                     if (!vAxeseries[index]) {
                         vAxeseries[index] = [];
                         seriesCollection[j].yAxis.position = null;
                     }
                     vAxeseries[index].push(seriesCollection[j]);
                 }

                 for (i = 0; i < vAxeseries.length; i++) {
                     if (!ej.util.isNullOrUndefined(vAxeseries[i])) {
                         vAxeseries[i].all = 0;
                         for (j = 0; j < vAxeseries[i].length; j++) {
                             series = vAxeseries[i][j];
                             if (series.type.toLowerCase().indexOf("stacking") != -1) {
                                 if (series.stackingGroup) {
                                     if (ej.util.isNullOrUndefined(stackingGroup[series.stackingGroup])) {
                                         vAxeseries[i].all++;
                                         series.position = vAxeseries[i].all;
                                         stackingGroup[series.stackingGroup] = vAxeseries[i].all;
                                     }
                                     else
                                         series.position = stackingGroup[series.stackingGroup];
                                 }
                                 else {
                                     if (ej.util.isNullOrUndefined(series.yAxis.position) || model.legendCollapsed) {
                                         vAxeseries[i].all++;
                                         series.position = vAxeseries[i].all;
                                         series.yAxis.position = vAxeseries[i].all;
										 model.legendCollapsed=false;
                                     }
                                     else
                                         series.position = series.yAxis.position;
                                 }
                             }
                             else {
                                 vAxeseries[i].all++;
                                 series.position = vAxeseries[i].all;
                             }
                         }
                     }
                 }
                 var columnWidth = seriesCollectionLength > 0 && seriesCollection[0].columnWidth;
                 for (k = 0; k < seriesCollectionLength; k++) {
                     var index = (model.requireInvertedAxes) ? seriesCollection[k].yAxis.columnIndex : seriesCollection[k].yAxis.rowIndex;
                     seriesCollection[k].all = vAxeseries[index].all;
                     params.needSpace = params.needSpace || columnWidth != seriesCollection[k].columnWidth
                 }

             });
             if (params.needSpace && this.chartObj.model._sideBySideSeriesPlacement)
                 this._getColumnSpacing(vAxeseries, params);
			 params.spacingCalculated = true;
         }
         return { all: currentSeries.all, pos: currentSeries.position };
     },
	
     getSideBySideInfo: function (series, params) {
         if (ej.util.isNullOrUndefined(series.xAxis.m_minPointsDelta))
		    {
			 this.chartObj.currentSeries = series;
             series.xAxis.m_minPointsDelta = ej.EjSvgRender.utils.getMinPointsDelta(series.xAxis, this.chartObj);   
            }			 
         var spacing = series.columnSpacing,
             columnWidth = series.columnWidth,
             data = this.getSideBySidePositions(series, params),
             pos = data.pos,
             all = data.all,
             needSpace = params.needSpace && this.chartObj.model._sideBySideSeriesPlacement,
             width = series.xAxis.m_minPointsDelta * (needSpace ? 1 : columnWidth),
             loc = needSpace ? params[series.name || series._name].loc : (pos - 1) / all - 0.5,
             range = this.getDoubleRange(loc, loc + (needSpace ? columnWidth : 1) / all);        
        if (!this.chartObj.model._sideBySideSeriesPlacement)
            return this.getDoubleRange(-width / 2, width / 2);


         // multiplying with width  and scaling
         if (!range.IsEmpty) {
             range = this.getDoubleRange(range.Start * width, range.End * width);
             var radius = spacing * range.Delta;
             range = this.getDoubleRange(range.Start + radius / 2, range.End - radius / 2);
         }
         return range;
    },

     _getColumnSpacing: function (axesSeries, params) {
         var i = j = 0, length, len, series, name;
         for (len = axesSeries.length; i < len; i++) {
             var width = 0, stackingSeries = [], group = {};
             for (length = axesSeries[i].length; j < length; j++)
                 if ((series = axesSeries[i][j]).type.toLowerCase().indexOf("stacking") == -1)
                     width += series.columnWidth / series.all;
                 else
                     group[name = series.stackingGroup] = { width: Math.max(series.columnWidth, group[name] ? group[name].width : 0), all: series.all }
             for (var key in group)
                 width += group[key].width / group[key].all;

             var start = (1 - width) / 2 - .5;
             for (j = 0, length = axesSeries[i].length; j < length; j++) {
                 if ((series = axesSeries[i][j]).type.toLowerCase().indexOf("stacking") == -1) {
                     params[series.name || (series._name="series"+i+j)] = { loc: start };
                     start += series.columnWidth / series.all;
                 }
                 else {
                     if (group[name = series.stackingGroup].loc == null) {
                         params[series.name || (series._name="series"+i+j)] = { loc: start + (group[name].width - series.columnWidth) / 2 / series.all };
                         group[name].loc = params[series.name || series._name].loc + series.columnWidth / 2 / series.all;
                         start += group[name].width / series.all;
                     }
                     else
                         params[series.name || series._name] = { loc: group[name].loc - series.columnWidth / 2 / series.all };
                 }
             }
         }
     },

    draw: function (chart, options) {

        this.chartObj = chart;
        
        //DrawLineGraph
        var lDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
		options.animationType = "path";
        var currentseries = options;
        var style = this.setLineSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);

        if (!currentseries.pointFill) {

            var firstPoint = null;
            var secondPoint;
            var firstIndex = -1;
            for (var i = 0; i < visiblePoints.length; i++) {
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                    if (firstPoint != null) {
                        if (visiblePoints.length > firstIndex + 1) {
                            var nextpoint = visiblePoints[firstIndex + 1];
                            var point1 = ej.EjSvgRender.utils._getPoint(firstPoint, currentseries);
                            var point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentseries);
                            var chartObj = this.chartObj;
                            sb.append("M" + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " + "L" + " " + (point2.X + chartObj.canvasX) + " " + ((point2.Y + chartObj.canvasY)) + " ");

                        }

                    }
                    firstPoint = secondPoint;
                    firstIndex = i;

                }
                else {
                    firstPoint = null;
                }

            }

            lDirection = sb.toString();

            this._drawLinePath(currentseries, style, lDirection);
        }

        else {
            var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries),
            previousStyle, styleOptions, currentPoint,
            nextPoint, point1, point2,
            requireInvertedAxes = this.chartObj.model.requireInvertedAxes,
            count=0,
            chartObj = this.chartObj;

            var visiblePoints = currentseries._visiblePoints,
           length = currentseries._visiblePoints.length - 1;

            var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);

            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

            for (var i = 0; i < length; i++) {
                currentPoint = visiblePoints[i];             
                nextpoint = visiblePoints[i + 1];               
                
                 if (!previousStyle) 
                     previousStyle = this.chartObj.setStyle(this, currentseries, seriesIndex, i);

                 if (nextpoint.visible && currentPoint.visible) {
                     point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentseries);

                     point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentseries);

                     sb.append("M" + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " + "L" + " " + (point2.X + chartObj.canvasX) + " " + ((point2.Y + chartObj.canvasY)) + " ");
                  }
                   
                    styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, i + 1);

                    if (styleOptions.interior == previousStyle.interior && (i != length - 1) && nextpoint.visible)
                        continue;                 
                    
                  
                    var options = {
                        'name': currentseries.type,
                        'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + count,
                        'stroke-dasharray': previousStyle.dashArray,
                        'fill': "none",
                        'stroke-width': previousStyle.width,
                        'stroke-linecap': currentseries.lineCap,
                        'stroke-linejoin': currentseries.lineJoin,
                        'stroke': previousStyle.interior,
                        'opacity': previousStyle.opacity,
                        'd': sb.toString()
                    };

                    this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

                    count++;

                    sb = ej.EjSvgRender.utils._getStringBuilder();

                    previousStyle = styleOptions;

                 
                
            }
        }
        if (this.chartObj.dragPoint)
            this.chartObj.svgRenderer.append(this.chartObj.gPreviewSeriesGroupEle, this.chartObj.gSeriesEle);
        else
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle,this.chartObj.gSeriesEle); 
    },
	
	doCircularAnimation: function (chartObj, series, seriesType, seriesIndex) {
	  var type = series.type.toLowerCase();
	  if (type == "pie" || type == "doughnut" || type == "pieofpie"){
         var seriesRender = this, startAngle, endAngle,
         angle = (series.points[0]) ? series.points[0].startAngle : 0;
         var collectionIndex = chartObj.model._isPieOfPie ? series.collectionIndex : 0;
         visiblePoints = series.emptyPointSettings.visible? series.visiblePoints :series._visiblePoints;
         $.each(visiblePoints, function (pointIndex, point) {            
             startAngle = point.startAngle;
             endAngle = point.endAngle;
             chartObj.model._radius = chartObj.model._isPieOfPie? chartObj.model.circularRadius[collectionIndex]:chartObj.model.circularRadius[seriesIndex];
             point.startAngle = 0;
             point.endAngle =point.startAngle;
             chartObj.model.Radius = 0;             
             var piePath = $(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex)[collectionIndex].childNodes[pointIndex];
			 if (type == "pie" || type=="pieofpie"){
               $(piePath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(
                 { StartAngle: startAngle, EndAngle: endAngle, pieRadius: chartObj.model._radius },
                 {
                     duration: 1200, queue: false, step: function (now, fx) {
                         if (fx.prop.toString() === "StartAngle") {
                             point.startAngle = now;
                             point.endAngle = point.startAngle;
                         }
                         else if (fx.prop.toString() == "pieRadius" && chartObj.model) 
                             chartObj.model.Radius = now;
                         else  
                             point.endAngle = now;
                         if (chartObj.model) {
                             var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series, seriesIndex, collectionIndex);
                             chartObj.svgRenderer._setAttr($(piePath), { "d": result.Direction });
                             chartObj.model.Radius = chartObj.model._radius;
                         }
                     },
                     complete: function () {
                         seriesRender.circularAnimationComplete(pointIndex, chartObj, series);
                     }
                 });
			   } 
			   else{ 
			      $(piePath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(
                 { StartAngle: startAngle, EndAngle: endAngle },
                 {
                     duration: 1200, queue: false, step: function (now, fx) {
                         if (fx.prop.toString() === "StartAngle") {
                             point.startAngle = now;
                             point.endAngle = point.startAngle;
                         }
                         else  
                             point.endAngle = now;
                         if (chartObj.model) {
                             var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series, seriesIndex);
                             chartObj.svgRenderer._setAttr($(piePath), { "d": result.Direction });
                             chartObj.model.Radius = chartObj.model._radius;
                         }
                     },
                     complete: function () {
                         seriesRender.circularAnimationComplete(pointIndex, chartObj, series);
                     }
                 });
			   }
           });
	    }
     },
	 circularAnimationComplete: function(pointIndex, chartObj, series){
	     if(!ej.util.isNullOrUndefined(chartObj.model)){
		 var commonEventArgs, i,seriesIndex= $.inArray(series,chartObj.model._visibleSeries), elements;
		     if (chartObj.gSeriesTextEle[seriesIndex])
		         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle[seriesIndex].id), { "visibility": "visible" });
		     if (chartObj.gSymbolGroupEle[seriesIndex])
		         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle[seriesIndex].id), { "visibility": "visible" });
		     if (chartObj.gDataLabelEle[seriesIndex])
		         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gDataLabelEle[seriesIndex].id), { "visibility": "visible" });
		     if (series.marker.dataLabel.template) {
		         elements = $(chartObj.element[0].childNodes[0].childNodes);
		         for (i = 0; i < elements.length; i++)
		             $(elements[i]).css('display', 'block');
		     }
		     if (chartObj.gConnectorEle[seriesIndex])
		         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gConnectorEle[seriesIndex].id), { "visibility": "visible" });
		     if (chartObj.gConnectorLinesGroup)
		         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gConnectorLinesGroup.id), { "visibility": "visible" });
		     chartObj.model.AnimationComplete = true;
		     series.AnimationComplete = true;
		     commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
		     commonEventArgs.data = { series: series };
		     chartObj._trigger("animationComplete", commonEventArgs);
             }
	 },
	 
    _doLineSymbol: function(element, sec, val, series, invertedAxes, chartobj) {
	  if (element.tagName != "defs") {
        var beginTime = parseInt(val * sec),
        chartObj = (this.chartObj == undefined) ? chartobj : this.chartObj,
        box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes),
        centerX = box.x + (box.width / 2),
        centerY = box.y + (box.height / 2),
        scaleVal, $ele = $(element),
        rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
        rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " " ;
        chartObj.svgRenderer._setAttr($ele, { "transform": "scale(0)" });
        $ele.delay(beginTime).animate(
            {
                scales: 1
            },
            {
                duration: 200,
                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + rotate });

                }
            }
        );
	  }
    },
    animateSymbol: function(element,delayInterval,series,invertedAxes,chartobj) {
	  if (element.tagName != "defs") {
        var chartObj = (this.chartObj == undefined) ? chartobj : this.chartObj,
        box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes),
        centerX = box.x + (box.width / 2),
        centerY = box.y + (box.height / 2),
        scaleVal, $ele = $(element),
        rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
        rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " ";
        $ele.delay(delayInterval).each(function () {this.scale = 0.5; }).animate(
            {
                scales: 1
            },
            {
                duration: 500,
                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + (rotate) });
                }
            }
        );
	  }
    },
    animateCylinder: function (chartObj, series,invertedAxes, clipRect) {
        if (series.isTransposed && invertedAxes) {
            var width = clipRect[0].width.animVal.value
            if (chartObj.model.series[0].type.indexOf("bar") > -1) {
                chartObj.svgRenderer._setAttr($(clipRect), { "x": width });
                  $(clipRect).animate({
                      x: 0,
                  }, 2000, function () {
                  });
            }
            else {
                var height = clipRect[0].height.animVal.value;
                chartObj.svgRenderer._setAttr($(clipRect), { "y": 0 });
                $(clipRect).animate(
                    {
                        y: parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "y")),
                        height: parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "height"))
                    },
                    {
                        duration: 2000,
                        step: function (now) {
                            chartObj.svgRenderer._setAttr($(clipRect), { "y": height - now });
                            chartObj.svgRenderer._setAttr($(clipRect), { "height": now });
                        }
                    });
            }
        }
        else {
            if (chartObj.model.series[0].type.indexOf("bar") > -1) {
                $(clipRect).animate(
                  {
                      width: parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "width"))
                  },
                  {
                      duration: 2000,
                      step: function (now) {
                          chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                      }
                  });
            }
            else {
                var height = clipRect[0].height.animVal.value;
                chartObj.svgRenderer._setAttr($(clipRect), { "y": height });
                $(clipRect).animate({

                    y: 0,
                }, 2000, function () {
                });
            }
        }
    },
    animateRect: function (element,series,invertedAxes, chartObj) {       
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes),
        centerX, centerY,scale1, scale2, scaleVal,
        $element = $(element);
        if ((series.isTransposed && invertedAxes) || invertedAxes) {
            if (chartObj.svgRenderer._getAttrVal($element, "plot") === "negative") {
                centerX = box.x + box.width;
                centerY = box.y + box.height;
            } else {               
                centerX = box.x;
                centerY = box.y;
			}
            scale1 = ") scale(";
            scale2 = ",1) translate(";
        }
        else {
            if (chartObj.svgRenderer._getAttrVal($element, "plot") === "negative") {
                centerX = box.x;
                centerY = box.y;
			}
			else {
                 centerX = (box.x +box.width);
                 centerY = (box.y + box.height);
             }
            scale1 = ") scale(1,";
            scale2 = ") translate(";   
        }
        $element.animate(
            {
                scales: 1
            },
            {
                duration: 1000,
                complete: function () {
                    if (series.type.toLowerCase() == "waterfall")
                        $("#" + chartObj.svgObject.id + '_SeriesGroup' + '_waterfallLine_' + series.index).attr("visibility", "visible");
                },
                step: function (now) {
                    scaleVal = now; 
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + scale1 + scaleVal + scale2 + (-centerX) + " " + (-centerY) + ")" });
                }
            });
    },
    animateStackingRect: function (element, series, invertedAxes, chartObj) {
       var centerX, centerY, scale1, scale2, $element = $(element), boxX, boxY, scaleVal,
       box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        if ((series.isTransposed && invertedAxes) || invertedAxes) {
            boxX = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.width);
            centerX = boxX;
            centerY = box.y;
			scale1 = ") scale(";
            scale2 = ",1) translate(";
        }
        else {
            boxY = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.height);
            centerX = box.x;
            centerY = boxY;
            scale1 = ") scale(1,";
            scale2 = ") translate(";
        }
        $element.animate(
            {
                scales: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + scale1 + scaleVal + scale2 + (-centerX) + " " + (-centerY) + ")" });
                }
            });
    },
    _doLineAnimation: function (chartObj, clipRect) {
		if (chartObj.model.requireInvertedAxes) {
            var height = parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "height"));
			//Animation for transposed series is working from top to bottom instead of bottom to top
			//In some scenario, series is not appearing because clip rect location gets changed
            //chartObj.svgRenderer._setAttr($(clipRect), { "y": height });
            $(clipRect).animate(
                {
                    //y: parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "y")),
                    clipRectHeight:parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "height")) 
                },
                {
                    duration: 2000,
                    step: function (now) {
                        chartObj.svgRenderer._setAttr($(clipRect), { "y": height - now });
                        chartObj.svgRenderer._setAttr($(clipRect), { "height": now});
                    }
                });
        }
		else
		{
          $(clipRect).animate(
            { 
                clipRectWidth: parseFloat(chartObj.svgRenderer._getAttrVal(clipRect, "width"))
			},
            {
                duration: 2000,
                step: function (now) {
                    chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                }
            });
		} 
    },
    
    _drawLinePath: function (series, style, direction) {
        var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
		var trans;
        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var translate = null;
        if (this.chartObj.model.AreaType == "cartesianaxes") {
            trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
            translate = 'translate(' + trans.x + ',' + trans.y + ')'
        }
        
        var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate };
        this.chartObj.gSeriesGroupEle =  this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
        if (direction != "" && direction.indexOf("NaN") == -1) {
            var options = {
                'name': series.type,
                'id': this.chartObj.svgObject.id + "_Series" + seriesIndex,
                'fill': 'none',
                'stroke-dasharray': series.dashArray,
                'stroke-width': series.width,
                'stroke': style.SeriesInterior,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'opacity': series.opacity,
                'd': direction
            };
            if (this.chartObj.dragPoint) {               
                    options.id = this.chartObj.svgObject.id + "_PreviewSeries" + seriesIndex;
                    options.opacity = 0.6;
                    if (this.chartObj.model.enableCanvasRendering) this.chartObj.svgRenderer.drawPath(options, this.chartObj.canvasElement, translate);
                    else this.chartObj.svgRenderer.drawPath(options, this.chartObj.gPreviewSeriesGroupEle);               
            }
            else
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        }


    },
   
   
    
    

    drawDataLabel: function (series, index, point, x, y, chartRegionIndex, params,outlier) {
	       // method for data label symbols - cartesianaxes
        point.marker = point.marker ? point.marker : {};
        var labelFormat = series.yAxis.labelFormat, labelPrecision, labelPrecisionDefault = 6, labelPrecisionHighest = 20,
            valueType = series.yAxis.valueType, pointText,
            boxPlotLabels = series._visiblePoints[index].boxPlotValues;
            value = (series.type.toLowerCase() == "waterfall" && (point.showIntermediateSum || point.showTotalSum)) ? point.waterfallSum : point.y;
            if (point.text)
                pointText = point.text;
            else if (labelFormat) {
                if (labelFormat.indexOf("{value}") > -1)
                    pointText = labelFormat.replace("{value}", value);
                else if (labelFormat.indexOf('e') == 0 || labelFormat.indexOf('E') == 0) {
                    labelPrecision = labelFormat.match(/(\d+)/g);
                    labelPrecision = labelPrecision == null ? labelPrecisionDefault : labelPrecision > labelPrecisionHighest ? labelPrecisionHighest : labelPrecision;
                    pointText = value.toExponential(labelPrecision);
                }
                else pointText = ej.format(value, labelFormat, this.chartObj.model.locale);
            }
            else pointText = value;
        var chart = this.chartObj,
			chartModel = chart.model,
			seriesIndex = $.inArray(series, chartModel._visibleSeries),
			textAlign, element,
			textBaseline, symbolName,
			marker = $.extend(true, {}, series.marker, point.marker),
			dataLabel = marker.dataLabel,
			dataLabelFont = dataLabel.font,
			textPosition = dataLabel.textPosition.toLowerCase(),
			type = series.type.toLowerCase(),
			size = ej.util.isNullOrUndefined(dataLabelFont.size) ? "11px" : dataLabelFont.size,
			fontStyle = ej.util.isNullOrUndefined(dataLabelFont.fontStyle) ? "Normal" : dataLabelFont.fontStyle,
			fontFamily = ej.util.isNullOrUndefined(dataLabelFont.fontFamily) ? "Segoe UI" : dataLabelFont.fontFamily,
			labelfont = { size: size, fontStyle: fontStyle, fontFamily: fontFamily },
			textOffset = ej.EjSvgRender.utils._measureText(pointText, null, labelfont),
			position = this.textPosition(series, seriesIndex, point, textOffset, type, x, y, chartRegionIndex, index, params),
			commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { text: pointText, location: { x: position.x + chart.canvasX, y: position.y + chart.canvasY }, seriesIndex: seriesIndex, pointIndex: index };
        chart._trigger("displayTextRendering", commonEventArgs);
        var dataLabelOffset = (marker.dataLabel.offset.y==undefined) ? marker.dataLabel.offset : marker.dataLabel.offset.y;
        var xval = marker.dataLabel.offset.x;
		var textOffset = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, labelfont);
        if (textPosition == 'bottom')
              commonEventArgs.data.location.y = commonEventArgs.data.location.y + dataLabelOffset;
        else
              commonEventArgs.data.location.y = commonEventArgs.data.location.y - dataLabelOffset;
        if (xval)                           
              commonEventArgs.data.location.x = commonEventArgs.data.location.x + xval;
                          
        if (!commonEventArgs.cancel)
        {
            var options = {
                'id': chart.svgObject.id + '_SeriesText' + index + seriesIndex,
                'x': commonEventArgs.data.location.x,
                'y': commonEventArgs.data.location.y,
                'fill': dataLabelFont.color,
                'font-size': dataLabelFont.size,
                'font-family': dataLabelFont.fontFamily,
                'font-style': dataLabelFont.fontStyle,
                'font-weight': dataLabelFont.fontWeight,
                'opacity': dataLabelFont.opacity,
                'type':series.type,
                'text-anchor': 'middle'
            };
            var margin = dataLabel.margin,
                width = textOffset.width + margin.left + margin.right,
                height = textOffset.height + margin.top + margin.bottom,
                location = { X: commonEventArgs.data.location.x, Y: commonEventArgs.data.location.y };

            if (ej.util.isNullOrUndefined(dataLabel.template)) {
				if (dataLabel.shape)
                    element = dataLabel.shape;
                else
                    element = "None";
                $.each(chartModel.symbolShape, function (name) {
                    if (element.toLowerCase() == name.toLowerCase())
                        symbolName = name;
                });
							
				var xXalue = commonEventArgs.data.location.x - (margin.left) / 2 + (margin.right) / 2,
                    yValue = commonEventArgs.data.location.y - (margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2,
                    degree = dataLabel.angle,
                    angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
							
 
							

                //For accurate placement of rotated data labels in canvas
				if (chartModel.enableCanvasRendering && (degree % 360 != 0)) {
				    options.y -= textOffset.height / 4;
				    options.baseline = 'middle';
				}

				if (series._enableSmartLabels) {
				    point.dataLabel = { textX: options.x, textY: options.y, x: xXalue, y: yValue, width: width, height: height }
				    point.margin = {top: margin.top, bottom: margin.bottom}
				}

				if (point.y == point.low && !(point.y === undefined && point.low === undefined)) //undefined condition checked for waterfall series sum value display
							    {
							        point.xPosLow = xXalue;
							        point.yPosLow = yValue;
							        point.widthLow = (symbolName == "None") ? textOffset.width : width;
							        point.heightLow = (symbolName == "None") ? textOffset.height : height;
							        point.textOptionsLow = options;
							        point.textOptionsLow.angle = angle;
							        point.drawTextLow = commonEventArgs.data.text;
							    }
				else if( type.toLowerCase() == "boxandwhisker" && (boxPlotLabels.midvalue != point.y))
				{
					var textOptionsBox = {
					'id': chart.svgObject.id + '_SeriesText' + index + seriesIndex,
					'x':ej.util.isNullOrUndefined(outlier)? commonEventArgs.data.location.x:xXalue,
					'y': ej.util.isNullOrUndefined(outlier)? commonEventArgs.data.location.y:yValue,
					'fill': dataLabelFont.color,
					'font-size': dataLabelFont.size,
					'font-family': dataLabelFont.fontFamily,
					'font-style': dataLabelFont.fontStyle,
					'font-weight': dataLabelFont.fontWeight,
					'opacity': dataLabelFont.opacity,
					'type':series.type,
					'text-anchor': 'middle',
					};
					var boxLabelCount = point.textOptionsBoxValues.length;
                   
					point.textOptionsBoxValues[boxLabelCount] = textOptionsBox;
					point.textOptionsBoxValues[boxLabelCount].angle = angle;
					point.textOptionsBoxValues[boxLabelCount].drawText = commonEventArgs.data.text;
					point.textOptionsBoxValues[boxLabelCount].index = index;
					point.textOptionsBoxValues[boxLabelCount].xPos = ej.util.isNullOrUndefined(outlier)?xXalue:xXalue- (margin.left) / 2 + (margin.right) / 2;
					point.textOptionsBoxValues[boxLabelCount].yPos = ej.util.isNullOrUndefined(outlier) ? yValue :yValue -(margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2;
					point.textOptionsBoxValues[boxLabelCount].seriesIndex = seriesIndex;
					point.textOptionsBoxValues[boxLabelCount].width = (symbolName == "None") ? textOffset.width : width;
					point.textOptionsBoxValues[boxLabelCount].height = (symbolName == "None") ? textOffset.height : height;
					point.textOptionsBoxValues[boxLabelCount].symbolName = symbolName;
                   
				}
							
							    else {
							        // storing the values in point
							    point.textOptions = options;
							    point.textOptions.angle = angle;
							    point.drawText = commonEventArgs.data.text;
							    point.index = index;
							    point.xPos = xXalue;
							    point.yPos = yValue;
							    point.seriesIndex = seriesIndex;
							    point.width = (symbolName == "None") ? textOffset.width : width;
							    point.height = (symbolName == "None") ? textOffset.height : height;
							    if (angle == 90 || angle == -90) //Swapping datalabel text height and width value
							        point.width = [point.height, point.height = point.width][0]; 

							    point.symbolName = symbolName;
							   }
						}
            else {
					location.X -= chart.canvasX;
					location.Y -= chart.canvasY;
					this.drawLabelTemplate(series, point, index, location, chart);
                }                             
        }
    },

    //smart labels for pie and doughnut

    updateSmartLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender, seriesIndex) {
        var renderingPoints = [],
            chartObj = sender,
            chartModel = chartObj.model,
            legend = chartModel.legend,
            legendActualBounds = chartModel.LegendActualBounds,
            legendPosition = legend.position.toLowerCase(),
            seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()](),
            connectorDirection = "",
			bezierPoints="",
            connectorX = bounds.connector.connectorX,
            connectorY = bounds.connector.connectorY,
            midX = bounds.midPoint.midX,
            midY = bounds.midPoint.midY,
            dMidX = bounds.doughnutMidPoint.dMidX,
            dMidY = bounds.doughnutMidPoint.dMidY,
            startX = startPoint.startX,
            startY = startPoint.startY,
            marker = $.extend(true, {}, currentseries.marker, point.marker),
            textGap = (marker.dataLabel.template) ? 0 : 4,
            outerX,
            radius,
            isIntersectedLabel,
            angle = midAngle,
            smartAngle = (currentseries.startAngle > currentseries.endAngle) ? -0.01 : 0.01,
            connectorType = marker.dataLabel.connectorLine.type.toLowerCase(),
            previousLeft, type = currentseries.type.toLowerCase(),
            radius = chartModel._isPieOfPie ? chartModel.circularRadius[currentseries.collectionIndex] : chartModel.circularRadius[seriesIndex],
            labelPosition = type == "pieofpie" ? "inside" :currentseries.labelPosition.toLowerCase();
        if (ej.util.isNullOrUndefined(chartModel.bounds[seriesIndex])){
            chartModel.bounds[seriesIndex] = [];
            chartModel.bounds[seriesIndex].points = [];
            if (labelPosition == 'outsideextended') {
                chartModel.bounds[seriesIndex].points["left"] = [];
                chartModel.bounds[seriesIndex].points["right"] = [];
            }
        }

        //calculation for inside labels
        if (labelPosition == 'inside' && ej.util.isNullOrUndefined(point.smartLabelPosition)) {
            positionX = (midX + startX) / 2;
            positionY = (midY + startY) / 2;

            positionX = (midX + positionX) / 2;
            positionY = (midY + positionY) / 2;

            if (point.isIntersected && chartModel.bounds[seriesIndex].points[pointIndex]) {
                currBounds = chartModel.bounds[seriesIndex].points[pointIndex];
            }
            else {
                if (midX > startX)
                    currBounds = { X: positionX, Y: positionY, Left: positionX, Top: positionY, Right: positionX + size.width, Bottom: positionY + size.height,SeriesIndex:seriesIndex,PointIndex:pointIndex };
                else
                    currBounds = { X: positionX, Y: positionY, Left: positionX - size.width, Top: positionY, Right: positionX, Bottom: positionY + size.height,SeriesIndex:seriesIndex,PointIndex:pointIndex };
             }

            do {
                isIntersectedLabel = false;
                if (!this.IntersectWith(chartObj, point, currBounds, "inside", seriesIndex)) continue;
                previousLeft = currBounds.Left;
				
				//convert radian to degree and break the loop after complete the 360 degree 
                var degree = angle * 180 / Math.PI;
                // chart start the angle from -90. So it will be skip in 270 degree
                if (degree >= 270) continue; 
				
                angle += smartAngle;
                point.isIntersected = true;
                currBounds.Angle = angle;
                if (midX > startX) {
                    currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Right = currBounds.X + size.width;
                }
                else {
                    currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Left = currBounds.Right - size.width;
                }
                currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);
                currBounds.Bottom = currBounds.Y + size.height;
                isIntersectedLabel = true;
            } while (isIntersectedLabel)
            if (midX > startX && currBounds.X < startX) {
                currBounds.Right = previousLeft;
                currBounds.Left = previousLeft - size.width;
            }
            chartModel.bounds[seriesIndex].points[pointIndex] = currBounds;
       

            if (point.isIntersected && chartModel.bounds[seriesIndex].points[pointIndex].Angle) {

                positionX = seriesType.getXCordinate(startX, (radius) + textOffset, chartModel.bounds[seriesIndex].points[pointIndex].Angle);
                positionY = seriesType.getYCordinate(startY, (radius) + textOffset, chartModel.bounds[seriesIndex].points[pointIndex].Angle);

                var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
                var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);
                renderingPoints.push({ X: midX, Y: midY });
                renderingPoints.push({ X: connectorMidX, Y: connectorMidY });
                renderingPoints.push({ X: positionX, Y: positionY });
                if (point.isIntersected && connectorType == 'bezier' && !chartObj.vmlRendering) {
                    bezierPoints = seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj, seriesIndex);
                }
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + " " + "M" + " " + connectorMidX + " " + connectorMidY + " " + "L" + " " + positionX + " " + positionY;

            }
        }

       //calculation for outside labels
        else if (labelPosition == 'outside' || point.smartLabelPosition == "outside") {
           
            if (midX >= startX) 
                currBounds = { X: connectorX, Y: connectorY, Left: connectorX, Top: connectorY, Right: connectorX + size.width, Bottom: connectorY + size.height, SeriesIndex: seriesIndex, PointIndex: pointIndex };
           else
                currBounds = { X: connectorX, Y: connectorY, Left: connectorX - size.width, Top: connectorY, Right: connectorX, Bottom: connectorY + size.height, SeriesIndex: seriesIndex, PointIndex: pointIndex };
            
            do {
                isIntersectedLabel = false;
               if (!this.IntersectWith(chartObj, point, currBounds, "outside", seriesIndex)) continue;
				 previousLeft = currBounds.Left;
                angle += smartAngle;
                point.isIntersected = true;
                if (midX > startX) {
                    currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Right = currBounds.X + size.width;
                }
                else {
                    currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                   currBounds.Left = currBounds.Right - size.width;
                }

                currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);              
                currBounds.Bottom = currBounds.Y + size.height;
                isIntersectedLabel = true;
            } while (isIntersectedLabel)
			 if (midX > startX && currBounds.X < startX) {
                currBounds.Right = previousLeft;
                currBounds.Left = previousLeft - size.width;
			 }
         	 chartModel.bounds[seriesIndex].points[pointIndex] = currBounds;
            var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
            var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);

            renderingPoints.push({ X: midX, Y: midY });
            renderingPoints.push({ X: connectorMidX, Y: connectorMidY });
            renderingPoints.push({ X: currBounds.X, Y: currBounds.Y });
            positionX = currBounds.X;
            positionY = currBounds.Y;
            if (textOffset > 0) {
                if (midX < startX) {
                    outerX = currBounds.X - chartModel.elementSpacing;
                    positionX = outerX - textGap;
                }
                else {
                    outerX = currBounds.X + chartModel.elementSpacing;
                    positionX = outerX + textGap;
                }
                renderingPoints.push({ X: outerX, Y: currBounds.Y });
            }
            if (connectorType == 'bezier' && !chartObj.vmlRendering) {
                bezierPoints = seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj, seriesIndex);
            }
            else {
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + " "+ "M" + " " + connectorMidX + " " + connectorMidY + " " + "L" + " " + currBounds.X + " " + currBounds.Y + " " + "M" + " " + currBounds.X + " " + currBounds.Y +  " " + "L" + " " + outerX + " " + currBounds.Y;
              }
                      
        }


         //calculation for outsideExtended labels
        else {
            var labelX = midX;
            var labelY = midY;

            midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
            midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));

            var connectorHeight = radius / 4;

            renderingPoints.push({ X: midX, Y: midY });
            midX = midX + (Math.cos((midAngle)) * connectorHeight);
            midY = midY + (Math.sin((midAngle)) * connectorHeight);

            renderingPoints.push({ X: midX, Y: midY });

            var legendWidth = (legend.visible && legendPosition == "left") ? (chartModel.margin.left + legendActualBounds.Width) : 0;
            var centerX = chartModel.circleCenterX[seriesIndex];
            var labelEdge;

            if (midX < startX) {

                midX = ((centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                 ? centerX - radius - (connectorHeight * 2) + legendWidth
                                 : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth + legendWidth) - size.width / 2;

                labelEdge = renderingPoints[1].X - currentseries.LeftLabelMaxWidth;

                var height = (centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0) ? connectorHeight
                             : (currentseries.LeftLabelMaxWidth > labelX) ? 0
                             : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;

                connectorLineEdge = +size.width / 2;

            }
            else {
                var legendRight = (legend.visible && legendPosition == "right") ? (chartModel.margin.right + legendActualBounds.Width) : 0;
                midX = ((centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width() - legendRight)
               ? centerX + radius + (connectorHeight * 2) - legendWidth
               : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth - legendRight) + size.width / 2;


                labelEdge = ($(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth) - renderingPoints[1].X;

                var height = (centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width()) ? connectorHeight
                            : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? 0
                            : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;
                connectorLineEdge = -size.width / 2;
            }

            var distanceFromOrigin = (Math.sqrt(Math.pow(labelX - midX, 2) + Math.pow(labelY - midY, 2))) / 10;

            var isLeft = (midX < startX) ? true : false;
            midX = isLeft ? midX + distanceFromOrigin : midX - distanceFromOrigin;
            var index = isLeft ? 1 : -1;

            var bounds = isLeft ? chartModel.bounds[seriesIndex].points["left"] : chartModel.bounds[seriesIndex].points["right"];
            var currBounds = { X: midX, Y: midY, Left: midX, Top: midY, Right: midX + size.width, Bottom: midY + size.height, index: pointIndex, SeriesIndex: seriesIndex, PointIndex: pointIndex };
            var clockwise = currentseries.endAngle > currentseries.startAngle ? true : false;
            if (this.IntersectWith(chartObj, point, currBounds, "outsideExtended", seriesIndex)) {
                renderingPoints.push(midX < startX
                       ? { X: midX + height + connectorLineEdge, Y: midY }
                       : { X: midX - height + connectorLineEdge, Y: midY });
                if (!clockwise && currentseries.startAngle != null) {
                    if (!isLeft)
                        midY = bounds[bounds.length - 1].Y + size.height + 2;
                    else
                        midY = bounds[bounds.length - 1].Y - size.height - 2;
                }
                else {
                    midY = bounds[bounds.length - 1].Y + size.height + 2;
                }
                currBounds.Y = midY;
                currBounds.Top = midY;
                currBounds.Bottom = midY + size.height;
            }
            renderingPoints.push({ X: midX + connectorLineEdge, Y: midY });
            if (centerX < renderingPoints[0].X && (renderingPoints[1].X > renderingPoints[2].X))
                renderingPoints[2].X = renderingPoints[1].X;
            if (centerX > renderingPoints[0].X && (renderingPoints[1].X < renderingPoints[2].X))
                renderingPoints[2].X = renderingPoints[1].X;
            if (!isLeft) {
                chartModel.bounds[seriesIndex].points["right"].push(currBounds);
                if (renderingPoints[renderingPoints.length - 1].X < renderingPoints[renderingPoints.length - 2].X)
                    renderingPoints[renderingPoints.length - 2].X = renderingPoints[renderingPoints.length - 1].X;
               
            }
            else {
                chartModel.bounds[seriesIndex].points["left"].push(currBounds);
                if (renderingPoints[renderingPoints.length - 1].X > renderingPoints[renderingPoints.length - 2].X)
                    renderingPoints[renderingPoints.length - 2].X = renderingPoints[renderingPoints.length - 1].X;
            }


            connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
            if (connectorType == 'bezier' && !chartObj.vmlRendering) {
                bezierPoints=seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj,seriesIndex);
            }
            else {
                for (k = 0; k < renderingPoints.length; k++) {
                    if (k == renderingPoints.length - 1)
                        connectorDirection.append(" ");
                    else
                        connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                }
                connectorDirection = connectorDirection.toString();
            }
            positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);
            positionY = renderingPoints[renderingPoints.length - 1].Y;
        }
        return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection, isInterSected:point.isIntersected,bezierPath:bezierPoints };
    },

    updateLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender, seriesIndex){

        var renderingPoints=[];
        var chartObj=sender; 
        var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
        var connectorDirection = "";

        var connectorX = bounds.connector.connectorX;
        var connectorY = bounds.connector.connectorY;
        var midX = bounds.midPoint.midX;
        var midY = bounds.midPoint.midY;
        var dMidX = bounds.doughnutMidPoint.dMidX;
        var dMidY = bounds.doughnutMidPoint.dMidY;
        var startX = startPoint.startX;
        var startY = startPoint.startY;
        var marker = $.extend(true, {}, currentseries.marker, point.marker);
        var textGap = (marker.dataLabel.template) ? 0 :4;
       
        var outerX;
        var radius = chartObj.model.circularRadius[seriesIndex];
        var labelPosition = type == "pieofpie" ? "inside" : currentseries.labelPosition.toLowerCase();

        //calculation for outside labels

        renderingPoints.push({ X: midX, Y: midY });
        renderingPoints.push({ X: connectorX, Y: connectorY });
        positionX = connectorX;
        positionY = connectorY;

        if (labelPosition == 'outside') {
            if (textOffset > 0) {
                if (connectorX < startX) {
                    outerX = connectorX - chartObj.model.elementSpacing;
                    positionX = outerX - textGap;
                }
                else {
                    outerX = connectorX + chartObj.model.elementSpacing;
                    positionX = outerX + textGap;
                }
                renderingPoints.push({ X: outerX, Y: connectorY });
            } 
           
            if (marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj,seriesIndex);
            }
            else {
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorX + " " + connectorY + " " + "M" + " " + connectorX + " " + connectorY +" "+ "L" + " " + outerX + " " + connectorY;
            }
          
        }


            //calculation for outsideextended labels
        else if (labelPosition == 'outsideextended') {

            labelX=midX;
            labelY=midY;

            midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
            midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));


            var renderingPoints = [];

            var connectorHeight = radius / 4;

            renderingPoints.push({ X: midX, Y: midY });
            connectorX = midX + (Math.cos((midAngle)) * connectorHeight);
            connectorY = midY + (Math.sin((midAngle)) * connectorHeight);

            renderingPoints.push({ X: connectorX, Y: connectorY });

            var legendWidth = (chartObj.model.legend.visible && (chartObj.model.legend.position.toLowerCase() == "left" || chartObj.model.legend.position.toLowerCase() == "right")) ? (chartObj.model.elementSpacing) : 0;

            if (midX < startX) {
                connectorX = (chartObj.model.centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                     ? chartObj.model.centerX - radius - (connectorHeight * 2) + legendWidth
                                     : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth;
            }
            else {
                connectorX = (chartObj.model.centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width())
                     ? chartObj.model.centerX + radius + (connectorHeight * 2) - legendWidth
                     : ((chartObj.model.centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth;
                      
            }

            renderingPoints.push({ X: connectorX, Y: connectorY });

            connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
            if (marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj, seriesIndex);
            }
            else {
                for (k = 0; k < renderingPoints.length; k++) {
                    if (k == renderingPoints.length - 1)
                        connectorDirection.append(" ");
                    else
                        connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                }
                connectorDirection = connectorDirection.toString();
            }
            positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);;
            positionY = renderingPoints[renderingPoints.length - 1].Y;
        }

       //calculation for inside labels

        else {
            if (currentseries.type.toLowerCase() == "doughnut") {
                positionX = (midX + dMidX) / 2;
                positionY = (midY + dMidY) / 2;
            }
            else {
                positionX = (midX + startX) / 2;
                positionY = (midY + startY) / 2;

                positionX = (midX + positionX) / 2;
                positionY = (midY + positionY) / 2;
            }
        }

        return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection, points:renderingPoints[renderingPoints.length-1]};
    },
    // method to update the position of the data labels in pyramid chart
        updateSmartLabel: function (chartObj, currentseries, pointIndex, point) {
            // declaration
        var datalabelTemplateVisiblity = true,
            connectorDirection = "",
            labelPosition = currentseries.labelPosition.toLowerCase(),
            marker = $.extend(true, {}, currentseries.marker, point.marker),
            dataLabel = marker.dataLabel,
            areaBounds = chartObj.model.m_AreaBounds,
			svgHeight = chartObj.model.svgHeight,
            type = currentseries.type.toLowerCase(),
            pyrX, pyrY, ele, xPos, yPos;
            point._labelPlacement = "";
            if (labelPosition == "inside" || labelPosition == "outsideextended")
                this.calculateInsideConnectorLines(pointIndex, point, currentseries, chartObj);
            else
                this.compareDataLabels(point, pointIndex, chartObj.model.outsideDataRegionPoints, currentseries, chartObj);

            pyrX = (chartObj.model.enableCanvasRendering && !dataLabel.template) ? chartObj.pyrX : 0;
            pyrY = (chartObj.model.enableCanvasRendering && !dataLabel.template) ? chartObj.pyrY : 0;
            if (type == "pyramid") {
                if ((chartObj.model.actualHeight< point.yPos - pyrY) || (point.xPos - pyrX > areaBounds.Width && point._labelPlacement == "insideoverlap") || (chartObj.model.legend.position == "right" && dataLabel.template && (point.xPos + point.width) > areaBounds.Width + chartObj.model.LegendActualBounds.Width)) {
                    datalabelTemplateVisiblity = false;
                    point.drawTextacc = "";
                }
            }
            else {
                if ((point.yPos - pyrY < 0) || (point.xPos - pyrX > areaBounds.Width && point._labelPlacement == "insideoverlap") || (chartObj.model.legend.position == "right" && dataLabel.template && (point.xPos + point.width) > areaBounds.Width + chartObj.model.LegendActualBounds.Width)) {
                    datalabelTemplateVisiblity = false;
                    point.drawTextacc = "";

                }
            }
            if ((labelPosition == "outside" || point._labelPlacement == "insideoverlap") && datalabelTemplateVisiblity)
                connectorDirection = this.drawConnectorLines(pointIndex, point, currentseries, chartObj);

            if (dataLabel.template) {
                ele = $("#" + point.id);
                if (!datalabelTemplateVisiblity) {
                    ele.css("display", "none");
                }
                else {
                    xPos = (point._labelPlacement == "insidenooverlap") ? point.textOptionsacc.x + chartObj.pyrX - point.width / 2 : point.textOptionsacc.x + chartObj.pyrX;
                    yPos = (point._labelPlacement == "insidenooverlap") ? point.textOptionsacc.y + chartObj.pyrY : point.textOptionsacc.y + chartObj.pyrY - chartObj.model.elementSpacing;
                    ele.css("left", xPos).css("top", yPos);
                    ele.css("height", point.height).css("overflow", "hidden");
                }
            }


            return { xPos: point.xPos, yPos: point.yPos, textOptionsacc: point.textOptionsacc, connectorDirection: connectorDirection.connectorDirection,bezierPath:connectorDirection.bezierdir, drawTextacc: point.drawTextacc };
        },
        // method to compare the data labels in chart for intersection
        compareDataLabels: function (point, pointIndex, outsideDataRegionPoints, currentseries, chartObj) {

            var marker = $.extend(true, {}, currentseries.marker, point.marker),
                dataLabel = marker.dataLabel,
                prevLabel, collide, regionPoint, degree, angle, rotate,
                elementSpacing = chartObj.model.elementSpacing,
                margin = marker.dataLabel.margin,
				visiblePointslength = currentseries._visiblePoints.length,
				labelPosition = currentseries.labelPosition.toLowerCase(),
                pyrX = (chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0,
                pyrY = (chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0;
            for (var j = 0, length = outsideDataRegionPoints.length; j < length ; j++) {
                prevLabel = outsideDataRegionPoints[j];
                collide = this.isCollide(prevLabel, point, currentseries);
                if (collide.state) {
                    if (labelPosition == "inside" || labelPosition == "outsideextended") {
                        point.textOptionsacc.y = point.yPos = point.textOptionsacc.y + collide.height;
                        positionPoint = (dataLabel.template) ? { x: point.xPos, y: point.yPos } : { x: point.xPos - pyrX, y: point.yPos - pyrY };
                        for (var k = 0, visibleLength = visiblePointslength ; k < visibleLength ; k++) {
                            regionPoint = (chartObj.isPointInPolygon(chartObj.model.chartRegions[0].Region[k].Polygon, positionPoint));
                            if (regionPoint) {
                                point.textOptionsacc.x = point.xPos = point.textOptionsacc.x + point.width;
                                positionPoint = (dataLabel.template) ? { x: point.xPos, y: point.yPos } : { x: point.xPos - pyrX, y: point.yPos - pyrY };
                            }
                        }
                    }
                    else {
                        point._labelPlacement = "outside";
                        point.textOptionsacc.y = point.yPos = (currentseries.type.toLowerCase() == "pyramid") ? point.yPos + collide.height + elementSpacing / 2 : point.yPos + collide.height - elementSpacing / 2;
                        point.textOptionsacc.y = point.textOptionsacc.y + point.height / 4 - elementSpacing / 2 + margin.top / 2 - margin.bottom / 2;
                    }
                }
            }
            degree = dataLabel.angle;
            angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
            rotate = 'rotate(' + angle + ',' + (point.textOptionsacc.x) + ',' + (point.textOptionsacc.y) + ')';
            $(point.textOptionsacc).attr('transform', rotate);
            $(point.textOptionsacc).attr('labelRotation', degree);//for canvas label rotation
            $(point.textOptionsacc).attr({
                'transform': rotate
            });
            if (labelPosition == "inside" || labelPosition == "outsideextended") {
                point.yPos = point.yPos + elementSpacing / 4;
                point.textOptionsacc.y = point.textOptionsacc.y + point.height / 4 + margin.top / 2 - margin.bottom / 2;
            }
            outsideDataRegionPoints[outsideDataRegionPoints.length] = point;
        },

        //Inside region checking and smartlabels
        calculateInsideConnectorLines: function (pointIndex, point, currentseries, chartObj) {

            var polygon = point.Polygon,
                marker = $.extend(true, {}, currentseries.marker, point.marker),
                dataLabel = marker.dataLabel,
                font = dataLabel.font, connectorHeight,
                pyrX = (chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0,
                pyrY = (chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0,
                //inside connector line height
                connectorLength = 70,
                datalabelWidth = ej.EjSvgRender.utils._measureText(point.drawTextacc, null, font),
                positionPoint = (dataLabel.template) ? { x: point.xPos, y: point.yPos } : { x: point.xPos + (datalabelWidth.width / 2) - pyrX, y: point.yPos - (point.height / 2) - pyrY },
                regionPoint = chartObj.isPointInPolygon(polygon, positionPoint);
            if (!regionPoint) {
                point._labelPlacement = "insideoverlap";
                point.textOptionsacc['text-anchor'] = 'start';
                if (point.startX + connectorLength < chartObj.model.m_AreaBounds.Width)
                    connectorHeight = connectorLength;
                else
                    connectorHeight = chartObj.model.m_AreaBounds.Width - (point.startX);

                point.textOptionsacc.x = point.xPos = point.startX + connectorHeight;
                point.textOptionsacc.y = point.yPos = point.yLocation;
                if (chartObj.model.enableCanvasRendering && !dataLabel.template) {
                    point.textOptionsacc.x += chartObj.pyrX;
                    point.textOptionsacc.y += chartObj.pyrY;
                    point.xPos += chartObj.pyrX;
                    point.yPos += chartObj.pyrY;
                }
                this.compareDataLabels(point, pointIndex, chartObj.model.outsideDataRegionPoints, currentseries, chartObj);

            }

            else {
                point._labelPlacement = "insidenooverlap";
            }
        },
        //smartLabels connector Lines
        drawConnectorLines: function (index, currentPoint, currentseries, chartObj) {
            var bezierdir = "";
            var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()](),
                startx = currentseries._visiblePoints[index].startX,
                starty = currentseries._visiblePoints[index].startY,
                connectorDirection = "",
				seriesIndex = $.inArray(currentseries, chartObj.model._visibleSeries),
                marker = $.extend(true, {}, currentseries.marker, currentPoint.marker),
                dataLabel = marker.dataLabel,
                endy = currentPoint.textOptionsacc.y - chartObj.model.elementSpacing / 2,
                pyrX = (chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0,
                pyrY = (chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0,
                renderingPoints = [{ X: startx + pyrX, Y: starty + pyrY },//bezier curve control points
                                   { X: currentPoint.textOptionsacc.x, Y: endy }];
            if (dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering)
                bezierdir =  seriesType.drawBezierSegment(renderingPoints, currentseries, index, chartObj, seriesIndex);
            else {
                connectorDirection = "M" + " " + (startx) + " " + (starty) + " " + "L" + " " + (currentPoint.textOptionsacc.x) + " " + (endy);
                if (chartObj.model.enableCanvasRendering && !dataLabel.template)
                    connectorDirection = "M" + " " + (startx) + " " + (starty) + " " + "L" + " " + (currentPoint.textOptionsacc.x - pyrX) + " " + (endy - pyrY);
            }
            return { connectorDirection: connectorDirection, bezierdir: bezierdir };
        },

        //method to detect collision between data labels
        isCollide: function (prevlabel, currentlabel, currentseries) {
            var width = 0,
                height = 0,
                type = currentseries.type.toLowerCase(),
                state;
            state = !(
           ((prevlabel.yPos + prevlabel.height) < (currentlabel.yPos)) ||
           (prevlabel.yPos > (currentlabel.yPos + currentlabel.height)) ||
           ((prevlabel.xPos + prevlabel.width / 2) < currentlabel.xPos - currentlabel.width / 2) ||
           ((prevlabel.xPos - prevlabel.width) > (currentlabel.xPos + currentlabel.width)));

            if (state && type == "pyramid") {
                height = ((prevlabel.yPos + prevlabel.height) - (currentlabel.yPos));
                width = (prevlabel.xPos + prevlabel.width) - currentlabel.xPos;
            }
            else if (state && type == "funnel") {
                height = ((prevlabel.yPos) - (currentlabel.yPos + currentlabel.height));
                width = (prevlabel.xPos + prevlabel.width) - currentlabel.xPos;
            }
            return { state: state, height: height, width: width };
        },
        // trimming the text when it exceeds chart area
        trimText: function (text, maxLength, ellipsis) {
            maxLength--;
            var length = maxLength - ellipsis.length;
            trimmedText = text.substr(0, length);
            return trimmedText + ellipsis;
        },

        //calculate a length of a text for datalabel trimming
        trimfunction: function (datalabelText, remainingWidth, datalabelLength, elipsis, axisWidth, font) {
            var datalabelText, datalabelLength;
            while (remainingWidth < datalabelLength) {
                datalabelText = this.trimText(datalabelText, datalabelText.length, elipsis);
                datalabelLength = ej.EjSvgRender.utils._measureText(datalabelText, axisWidth, font).width;
                if (datalabelText == elipsis) {
                    datalabelText = "";
                    break;
                }
            }
            return datalabelText;
        },

        //To calculate start and end point of a text inside a pie and doughnut region
        calculatePosition: function (chartObj, textOptions, datalabelLength, startX, startY) {
            var rightEndX, rightEndY, leftEndX, leftEndY;            
            rightEndX = (textOptions.x + (datalabelLength / 2)) - startX;
            rightEndY = (textOptions.y) - (startY);
            leftEndX = (textOptions.x - (datalabelLength / 2)) - startX;
            leftEndY = textOptions.y - startY;
            return { rightEndX: rightEndX, rightEndY: rightEndY, leftEndX: leftEndX, leftEndY: leftEndY };
                        
        },

        //To caluculate textoptions
        textOption: function (currentseries, chartObj, textsize, pyrX, pyrY, pointIndex, textAnchor, positionX, positionY, seriesIndex,visiblePoint) {
            if (chartObj.model._isPieOfPie) 
                var marker = $.extend(true, {}, currentseries.marker, visiblePoint.marker);            
            else
                var marker = $.extend(true, {}, currentseries.marker, currentseries._visiblePoints[pointIndex].marker);
            if (!marker.dataLabel.template) {
                $("#" + chartObj.svgObject.id +'series' + seriesIndex+ '_PointText' + pointIndex).remove();
                var textOptions = {
                    'id': chartObj.svgObject.id + 'series' + seriesIndex + '_PointText' + pointIndex,
                    'x': positionX + pyrX,
                    'y': positionY + textsize.height / 4 + pyrY,
                    'fill': marker.dataLabel.font.color,
                    'font-size': marker.dataLabel.font.size,
                    'font-family': marker.dataLabel.font.fontFamily,
                    'font-style': marker.dataLabel.font.fontStyle,
                    'font-weight': marker.dataLabel.font.fontWeight,
                    'opacity': marker.dataLabel.font.opacity,
                    'text-anchor': textAnchor,
                    'cursor': 'default',
                    'degree':marker.dataLabel.angle
                };
                //To rotate datalabels
                var degree = marker.dataLabel.angle;
                var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
                var rotate = 'rotate(' + angle + ',' + (positionX) + ',' + (positionY) + ')';
                $(textOptions).attr('transform', rotate);
                $(textOptions).attr('labelRotation', degree);//for canvas label rotation
                $(textOptions).attr({
                    'transform': rotate
                });
            }
            return textOptions;
        },

        //To trim  datalabel when it exceed a particular pie and doughnut region
        labelTrim: function (chartObj, textOptions, datalabelLength, datalabelText, startX, startY, point, font,seriesIndex, pieSeriesIndex) {
            var chartStartAngle = -.5 * Math.PI;
            var chartEndAngle = .5 * Math.PI;
            var position = this.calculatePosition(chartObj, textOptions, datalabelLength, startX, startY);
            var startAngle = chartObj.model.series[seriesIndex].startAngle;
            var endAngle = chartObj.model.series[seriesIndex].endAngle;
            var regionData = (chartObj.model.chartRegions.length <chartObj.model._visibleSeries.length) ? chartObj.model.chartRegions[0].Region : chartObj.model.chartRegions[seriesIndex].Region;
            var pointStartAngle, pointEndAngle;
            var distanceFromCenter;
            var textinRegion;
            var trimPosition, datalabelWidth;
            var ellipsis = "...";
            var firstStartAngle = [];
            var totalDegree = endAngle - startAngle;
            var seriesIndex = chartObj.model._isPieOfPie ? pieSeriesIndex : seriesIndex;
            if (totalDegree < 0) {
                endAngle = endAngle / 360;
                var arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
            }
            else
                var arcAngle = startAngle ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;            
            startAngle = (Math.atan2(position.rightEndY, position.rightEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
            if (startAngle < 0) startAngle = 2 * Math.PI + startAngle;
            endAngle = (Math.atan2(position.leftEndY, position.leftEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
            if (endAngle < 0) endAngle = 2 * Math.PI + endAngle;            
            pointStartAngle = parseFloat(point.start.toFixed(14));
            pointEndAngle = parseFloat(point.end.toFixed(14));
            if (totalDegree > 0) {
                if (ej.util.isNullOrUndefined(chartObj.firstStartAngle[0])) {
                    chartObj.firstStartAngle.push(pointStartAngle);
                }
                pointStartAngle = (chartObj.firstStartAngle[0] < 0) ? (chartObj.model._isPieOfPie ? pointStartAngle : 2 * Math.PI + pointStartAngle) : pointStartAngle;
                pointEndAngle = (chartObj.firstStartAngle[0] < 0) ? (chartObj.model._isPieOfPie ? pointEndAngle : 2 * Math.PI + pointEndAngle) : pointEndAngle;             
            }
            else {
                  pointStartAngle = pointStartAngle < 0 ? (chartObj.model._isPieOfPie ? pointStartAngle : 2 * Math.PI + pointStartAngle): pointStartAngle;
                  pointEndAngle = pointEndAngle < 0 ? (chartObj.model._isPieOfPie ? pointEndAngle : 2 * Math.PI + pointEndAngle) : pointEndAngle;
            }
            pointStartAngle -= arcAngle;
            pointEndAngle -= arcAngle;
			// to check start and end angle when greater than 360 in pieofpie
            if (pointStartAngle > 2 * Math.PI && pointEndAngle > 2 * Math.PI) {
                startAngle = startAngle + 2 * Math.PI;
                endAngle = endAngle + 2 * Math.PI;
            }
            else if (pointStartAngle < 2 * Math.PI && pointEndAngle > 2 * Math.PI) {
                if (startAngle < Math.PI / 2)
                    startAngle = startAngle + 2 * Math.PI;
                if (endAngle < Math.PI / 2)
                    endAngle = endAngle + 2 * Math.PI;
            }
            if (startAngle < 0 && (endAngle > 0 || endAngle == null)) {
                pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                pointEndAngle = pointEndAngle <= 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
            }
            if (chartObj.model.circleCenterX[seriesIndex] < textOptions.x) {
                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position.rightEndX), 2) + Math.pow(Math.abs(position.rightEndY), 2));
            }
            else
                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position.leftEndX), 2) + Math.pow(Math.abs(position.leftEndY), 2));            
            if (totalDegree < 0) {               
                pointEndAngle = [pointStartAngle, pointStartAngle = pointEndAngle][0];
            }            
            if ((endAngle >= pointStartAngle && endAngle <= pointEndAngle) && (startAngle >= pointStartAngle && startAngle <= pointEndAngle) && (distanceFromCenter <= chartObj.model.circularRadius[seriesIndex] && distanceFromCenter > 0)) {
                textinRegion = true;
            }
            else {
                textinRegion = false;
                while (!textinRegion) {
                   datalabelText = this.trimText(datalabelText, datalabelText.length, ellipsis);
                   if (datalabelText == ellipsis) {
                       datalabelText = "";
                       break;
                   }  
                   datalabeltWidth = ej.EjSvgRender.utils._measureText(datalabelText, null, font);
                   trimPosition = this.calculatePosition(chartObj, textOptions, datalabeltWidth.width, startX, startY);
                   startangle = (Math.atan2(trimPosition.rightEndY, trimPosition.rightEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                   if (startangle < 0) startangle = 2 * Math.PI + startangle;
                   var endAngle = (Math.atan2(trimPosition.leftEndY, trimPosition.leftEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                   if (endAngle < 0) endAngle = 2 * Math.PI + endAngle;
                   if (chartObj.model.circleCenterX[seriesIndex]< textOptions.x) {
                      distanceFromCenter = Math.sqrt(Math.pow(Math.abs(trimPosition.rightEndX), 2) + Math.pow(Math.abs(trimPosition.rightEndY), 2));
                   }
                   else
                      distanceFromCenter = Math.sqrt(Math.pow(Math.abs(trimPosition.leftEndX), 2) + Math.pow(Math.abs(trimPosition.leftEndY), 2));
                    // to check when startangle and endangle is greater than 360 degree for pieofpie
                   if ((pointStartAngle > 2 * Math.PI && pointEndAngle > 2 * Math.PI) && chartObj.model._isPieOfPie) {
                         var  stAngle = pointStartAngle - 2 * Math.PI;
                           pointStartAngle = stAngle;
                         var  enAngle = pointEndAngle - 2 * Math.PI;
                           pointEndAngle = enAngle;
                   }
                   else if ((pointStartAngle !=0 && pointStartAngle < 2 * Math.PI && pointEndAngle >= 2 * Math.PI) && chartObj.model._isPieOfPie) {
                      var stAngle = pointStartAngle - 2 * Math.PI;
                       pointStartAngle = stAngle;
                   }
                   if ((endAngle >= pointStartAngle && endAngle <= pointEndAngle) &&
                       (startangle >= pointStartAngle && startangle <= pointEndAngle) &&
                       (distanceFromCenter <= chartObj.model.circularRadius[seriesIndex] && distanceFromCenter > 0)) {
                      textinRegion = true;
                   }
                   
                 }
            }
                        
         return datalabelText;
        },
    
    
        drawDataLabelAcc: function (sender, currentseries, pointIndex, point, seriesIndex, pieSeriesIndex) {
            // method for data label symbols - accumulation series	
            var pointMarker = point.marker;
            if (point.visible && ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && currentseries.marker.dataLabel.visible)) {
                var chartObj = sender,
                    type = currentseries.type.toLowerCase(),
                    seriesType = new ej.seriesTypes[type](),
                    isNull = ej.util.isNullOrUndefined,
                    _labelPosition = type == "pieofpie" ? "inside" : currentseries.labelPosition.toLowerCase(),
                    chartModel = chartObj.model,
					chartTitle = chartModel.title,
                    measureText = ej.EjSvgRender.utils._measureText,
                    elementSpacing = chartModel.elementSpacing,
                    seriesMarker = currentseries.marker,
                    dataLabel = seriesMarker.dataLabel,
                    margin = dataLabel.margin,
                    dataLabelFont = dataLabel.font,
                    connectorLine = dataLabel.connectorLine,
                    connectorType = connectorLine.type.toLowerCase(),
                    legend = chartModel.legend,
                    legendPosition = legend.position.toLowerCase(),
                    legendActualBounds = chartModel.LegendActualBounds,
                    text = isNull(point.text) ? point.y : point.text,
                    enableWrap=dataLabel.enableWrap,
                    wrapConnectorpoints,
                    labelText = text,
                    srIndex = chartModel._isPieOfPie ? pieSeriesIndex : seriesIndex;
              
                var connectorDirection, labelfont,
                    commonEventArgs, textsize,
                    positionX = 0, textWidth, textHeight,
                    positionY = 0,
                    pointColor;

                if ((type == "pyramid" || type == "funnel") && _labelPosition == 'outsideextended')
                    _labelPosition = 'inside';

                commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { text: text, series: currentseries, pointIndex: pointIndex };
                chartObj._trigger("displayTextRendering", commonEventArgs);

                labelfont = { size: dataLabelFont.size, fontStyle: dataLabelFont.fontStyle, fontFamily: dataLabelFont.fontFamily };
                textsize = measureText(commonEventArgs.data.text, null, labelfont);
                textWidth = textsize.width + margin.left + margin.right;
                textHeight = textsize.height + margin.top + margin.bottom;

                if (type == "pyramid" || type == "funnel") {
                    var measureTitle = (chartModel.title.text) ? measureText(chartModel.title.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.font) : 0;
                    var mesureSubTitle = (chartObj.model.title.subTitle.text) ? ej.EjSvgRender.utils._measureText(chartObj.model.title.subTitle.text, $(this.svgObject).width() - chartObj.model.margin.left - chartObj.model.margin.right, chartObj.model.title.subTitle.font) : 0;
                    var subtitleHeight = (chartObj.model.title.subTitle.text === '') ? 0 : mesureSubTitle.height;
                    var pyrX = elementSpacing + chartModel.margin.left + (legendPosition === "left") ? (legendActualBounds.Width) : 0;
                    var pyrY = ((legendPosition === "top") ? (legendActualBounds.Height) : 0) + ((chartModel.title.text && chartModel.title.visible) ? (chartModel._titleLocation.Y + measureTitle.height + subtitleHeight) : (chartModel.margin.top + elementSpacing));
                    textWidth = textWidth + 10;
                    var Location = { X: point.xLocation + chartObj.pyrX, Y: point.yLocation + chartObj.pyrY };
                    positionX = point.xLocation;
                    positionY = point.yLocation;
                    var marker = point.marker ? point.marker : seriesMarker;
                    marker = $.extend(true, {}, seriesMarker, marker);
                    connectorDirection = point.connectorLine;
                    symbolPos = { positionX: positionX, positionY: positionY };
                }

                if (type != "pyramid" && type != "funnel") {
                    var chartStartingAngle = -.5 * Math.PI,
                        midAngle = point.midAngle + chartStartingAngle,
                        midPoint, doughnutMidPoint, connector, bounds,
                        startPoint,
                        measureTitle = (chartModel.title.text) ? measureText(chartModel.title.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.font) : 0,
                        mesureSubTitle = (chartObj.model.title.subTitle.text) ? ej.EjSvgRender.utils._measureText(chartObj.model.title.subTitle.text, $(this.svgObject).width() - chartObj.model.margin.left - chartObj.model.margin.right, chartObj.model.title.subTitle.font) : 0,
                        subtitleHeight = (chartObj.model.title.subTitle.text === '') ? 0 : mesureSubTitle.height,
                        textOffset, startX, startY, midX, midY, dMidX, dMidY, connectorX, connectorY,
                        renderingPoints = [],
                        radius = chartModel.circularRadius[srIndex];
                    if (isNull(connectorLine.height))
                        textOffset = measureText(commonEventArgs.data.text, null, dataLabelFont).height;
                    else
                        textOffset = connectorLine.height;
                    var actualIndex = chartObj.model._isPieOfPie ? point.actualIndex : currentseries._visiblePoints[pointIndex].actualIndex;
                    if ((actualIndex == currentseries.explodeIndex || currentseries.explodeAll) && !chartObj.vmlRendering) {
                        startX = chartModel.circleCenterX[srIndex] + Math.cos(midAngle) * currentseries.explodeOffset;
                        startY = chartModel.circleCenterY[srIndex] + Math.sin(midAngle) * currentseries.explodeOffset;
                    }
                    else {
                        startX = chartModel.circleCenterX[srIndex];
                        startY = chartModel.circleCenterY[srIndex];
                    }
                    midX = seriesType.getXCordinate(startX, (radius), midAngle);
                    midX = point.y != 0 ? midX : parseFloat(midX.toFixed(12));
                    midY = seriesType.getYCordinate(startY, (radius), midAngle);

                    dMidX = seriesType.getXCordinate(startX, (chartModel.innerRadius[seriesIndex]), midAngle);
                    dMidY = seriesType.getYCordinate(startY, (chartModel.innerRadius[seriesIndex]), midAngle);

                    connectorX = this.getXCordinate(startX, (radius) + textOffset, midAngle);
					connectorX = point.y != 0 ? connectorX : connectorX.toFixed(12);
                    connectorY = this.getYCordinate(startY, (radius) + textOffset, midAngle);

                    if (dataLabel.template)
                        size = point.size;
                    else {
                        var marker = point.marker ? point.marker : seriesMarker,
                            marker = $.extend(true, {}, seriesMarker, marker);
                        shape = marker.dataLabel.shape;
                        size = measureText(commonEventArgs.data.text, $(chartObj.svgObject).width(), dataLabelFont);
                        if (shape.toLowerCase() != "none") {
                            size.height = size.height + margin.top + margin.bottom;
                            size.width = size.width + margin.left + margin.right;
                        }
                    }

                    midPoint = { midX: midX, midY: midY };
                    doughnutMidPoint = { dMidX: dMidX, dMidY: dMidY };
                    connector = { connectorX: connectorX, connectorY: connectorY };
                    bounds = { midPoint: midPoint, doughnutMidPoint: doughnutMidPoint, connector: connector };
                    startPoint = { startX: startX, startY: startY };
                   
                    if (!currentseries._enableSmartLabels) {
                        position = seriesType.updateLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender, seriesIndex);
                        positionX = position.positionX;
                        positionY = position.positionY;
                        connectorDirection = position.connectorDirection;
                    }
                        // smart labels

                    else {
                        position = seriesType.updateSmartLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender, seriesIndex);
                        positionX = position.positionX;
                        positionY = position.positionY;
                        symbolPos = { positionX: positionX, positionY: positionY, isInterSected: position.isInterSected };
                        connectorDirection = chartObj.model._isPieOfPie? "" : position.connectorDirection;
                    }
                }
                
                var textAnchor = ((_labelPosition != 'inside' && _labelPosition != 'insidenooverlap') || ((type == "pyramid" || type == "funnel") && _labelPosition == 'outside')) ? (positionX < chartModel.circleCenterX[srIndex] ? 'end' : 'start') : 'middle',
                    width = (_labelPosition == 'inside' || _labelPosition == 'insidenooverlap') ? 0 : (positionX < chartModel.circleCenterX[srIndex]) ? -textsize.width : textsize.width,
                    pyrX = 0, pyrY = 0, textOptions;

                if ((type == "pie" || type == "doughnut" || type == "pieofpie") && _labelPosition == "inside")
                    textAnchor = (connectorDirection != "") ? (positionX < chartModel.circleCenterX[srIndex] ? 'end' : 'start') : 'middle';
                if ((type == "pyramid" || type == "funnel") && _labelPosition == "outside")
                    textAnchor = 'start';

                var subtitleHeight = (chartObj.model.title.subTitle.text === '') ? 0 : mesureSubTitle.height;
                if ((this.chartObj && chartModel.enableCanvasRendering) && (type == "pyramid" || type == "funnel")) {
                    pyrX = ((legendPosition === "left") ? (legendActualBounds.Width) : 0) + elementSpacing + chartModel.margin.left;
                    pyrY = ((legendPosition === "top") ? (legendActualBounds.Height) : 0) + ((chartTitle.text) ? (chartModel._titleLocation.Y + measureTitle.height + subtitleHeight) : (chartModel.margin.top + elementSpacing));
                }
               
                pointIndex = chartObj.model._isPieOfPie? point.actualIndex: pointIndex;
                textOptions = this.textOption(currentseries, chartObj, textsize, pyrX, pyrY, pointIndex, textAnchor, positionX, positionY, seriesIndex, point);
                if (dataLabel.template) {
                    Location = isNull(Location) ? { X: positionX, Y: positionY } : Location;
                    var pointindex = point.actualIndex;
                    seriesType.drawLabelTemplate(currentseries, point, pointindex, Location, sender);
                    if (connectorType == "bezier" && (position.bezierPath) != "")
                        sender.svgRenderer.drawPolyline(position.bezierPath, sender.gConnectorEle[seriesIndex]);
                }
                else {
                    $.each(chartModel.symbolShape, function (name) {
                        marker = point.marker ? point.marker : seriesMarker;
                        marker = $.extend(true, {}, seriesMarker, marker);
                        shape = marker.dataLabel.shape;
                        if (shape.toLowerCase() == name.toLowerCase())
                            symbolName = name;
                    });

                    positionX = pyrX + positionX - (margin.left) / 2 + (margin.right) / 2 + width / 2;
                    positionY = pyrY + positionY - (margin.top) / 2 + (margin.bottom) / 2;
                    //SmartLabels value storing for pyramid and funnel
                    if (type == "pyramid" || type == "funnel") {
                        point.xPos = symbolPos.positionX + pyrX;
                        point.yPos = symbolPos.positionY + pyrY;
                        point.width = (symbolName == "None") ? textsize.width : textWidth;
                        point.height = (symbolName == "None") ? textsize.height : textHeight;
                        point.symbolName = (dataLabel.template) ? "none" : symbolName;
                        point.textOptionsacc = textOptions;
                        point.drawTextacc = commonEventArgs.data.text;
                    }
                    //For accurate placement of rotated data labels in canvas
                    if (chartModel.enableCanvasRendering && (textOptions.degree % 360 != 0)) {
                        textOptions.y -= (textsize.height / 4);
                        textOptions.baseline = 'middle';
                    } if (currentseries._enableSmartLabels) {
                        var axisWidth = chartModel.m_AreaBounds.Width + chartModel.m_AreaBounds.X + chartModel.margin.left,
                            labelPosition = _labelPosition,
                            enableCanvasRendering = chartModel.enableCanvasRendering,
                            datalabelText = (typeof commonEventArgs.data.text == "string") ? commonEventArgs.data.text : commonEventArgs.data.text.toString(),
                            datalabelLength = measureText(datalabelText, datalabelText.length, dataLabelFont).width,
                            legendBounds,
                            remainingWidth,
                            legendWidth = 0, legendHeight = 0;
                        if (legend.visible) {
                            if (legendPosition == "left" || legendPosition == "right")
                                legendBounds = legendActualBounds;
                        }
                        else
                            legendBounds = { Width: 0 };

                        if (type == "pie" || type == "doughnut" || type == "pieofpie") {

                            if (labelPosition == "outside" || labelPosition == "outsideextended" || connectorDirection != "") {
                                if (chartModel.circleCenterX[srIndex] < textOptions.x)
                                    remainingWidth = (axisWidth - textOptions.x);
                                else
                                    remainingWidth = legendPosition == "left" ? (textOptions.x - chartModel.margin.left - (chartModel.elementSpacing * 2) - legendBounds.Width) : (textOptions.x - chartModel.margin.left);
                                datalabelText = this.trimfunction(datalabelText, remainingWidth, datalabelLength, "...", axisWidth, dataLabelFont);

                                if (legend.visible && (legendPosition == 'top' || legendPosition == 'bottom')) {
                                    datalabelText = this._overlapLegend(textOptions, datalabelText, sender,seriesIndex);
                                }
                            }
                            if (labelPosition == "inside" && (connectorDirection == "")) {
                                datalabelText = this.labelTrim(chartObj, textOptions, datalabelLength, datalabelText, startX, startY, point, dataLabelFont, seriesIndex,pieSeriesIndex);
                               
                                if (datalabelText == "" && !chartObj.model._isPieOfPie) {
                                    point.smartLabelPosition = "outside";
                                    position = seriesType.updateSmartLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender, seriesIndex);
                                    symbolPos = { positionX: position.positionX, positionY: position.positionY, isInterSected: position.isInterSected };
                                    datalabelText = commonEventArgs.data.text;
                                    connectorDirection = position.connectorDirection;
                                    textAnchor = (chartModel.circleCenterX[srIndex] < position.positionX) ? "start" : "end";
                                    textOptions = this.textOption(currentseries, chartObj, textsize, pyrX, pyrY, pointIndex, textAnchor, position.positionX, position.positionY, seriesIndex, point);
                                    if (chartModel.circleCenterX[srIndex] < position.positionX)
                                        remainingWidth = legendPosition == "right" ? (axisWidth - textOptions.x  - legendBounds) : (axisWidth - textOptions.x );
                                    else
                                        remainingWidth = legendPosition == "left" ? (textOptions.x  - legendBounds) : (textOptions.x );
                                    datalabelText = this.trimfunction(datalabelText, remainingWidth, datalabelLength, "...", axisWidth, dataLabelFont);
                                    delete point.smartLabelPosition;
                                }
                            }
                        }
                        else if ((type == "pyramid" || type == "funnel")) {
                            position = seriesType.updateSmartLabel(sender, currentseries, pointIndex, point);
                            positionX = position.xPos;
                            positionY = position.yPos;
                            connectorDirection = position.connectorDirection;
                            textOptions = position.textOptionsacc;
                            datalabelText = point.drawTextacc;

                            if ((connectorDirection != "" || connectorType == "bezier") && datalabelText != "") {
                                datalabelText = datalabelText.toString();
                                if (legendPosition == "right")
                                    remainingWidth = (enableCanvasRendering) ? (axisWidth - textOptions.x) : (axisWidth - textOptions.x - chartObj.pyrX);
                                else if (legendPosition == "left")
                                    remainingWidth = enableCanvasRendering ? (axisWidth - textOptions.x) : (axisWidth - textOptions.x - chartObj.pyrX);
                                else if (legendPosition == "top")
                                    remainingWidth = (enableCanvasRendering) ? (axisWidth - textOptions.x) : axisWidth - textOptions.x - chartObj.pyrX;
                                else
                                    remainingWidth = (enableCanvasRendering) ? (axisWidth - textOptions.x) : axisWidth - textOptions.x - chartObj.pyrX;
                                if (remainingWidth < datalabelLength)
                                    datalabelText = this.trimfunction(datalabelText, remainingWidth, datalabelLength, "...", axisWidth, dataLabelFont);
                            }
                        }

                        var datalabelShape = measureText(datalabelText, null, dataLabelFont);
                        if (type == "pie" || type == "doughnut" || type == "pieofpie")
                            var width = ((_labelPosition == 'inside' || _labelPosition == 'insidenooverlap') && !symbolPos.isInterSected && connectorDirection == "") ? 0 : (positionX < chartModel.circleCenterX[srIndex]) ? -datalabelShape.width : datalabelShape.width;
                        else if (type == "pyramid" || type == "funnel")
                            var width = (point._labelPlacement == "insidenooverlap") ? 0 : datalabelShape.width;
                        if (type == "pie" || type == "doughnut" || type == "pieofpie") {
                            positionX = pyrX + symbolPos.positionX - (margin.left) / 2 + (margin.right) / 2 + (width / 2);
                            positionY = pyrY + symbolPos.positionY - (margin.top) / 2 + (margin.bottom) / 2;
                        }
                        else {
                            positionX = positionX - (margin.left) / 2 + (margin.right) / 2 + (width / 2);
                            positionY = positionY - (margin.top) / 2 + (margin.bottom) / 2;
                        }
                        
                        var dataLabelEle = $('#' + chartObj.svgObject.id + '_dataLabel_series' + seriesIndex + "_point_" + pointIndex);
						var conEle = $('#' + chartObj.svgObject.id + "_connectorLine_series"+ seriesIndex+"_point_" + pointIndex);
						if(datalabelText!= ""){
						dataLabelEle.show();
						conEle.show();
						if (!enableWrap) {
						    seriesType.dataLabelSymbol(seriesIndex, currentseries, pointIndex, positionX, positionY, textWidth, textHeight, symbolName, chartObj);
						}
							if (connectorType == "bezier" && (position.bezierPath) != "" && !(ej.util.isNullOrUndefined(position.bezierPath)))
									sender.svgRenderer.drawPolyline(position.bezierPath, sender.gConnectorEle[seriesIndex]);
						}
						else{
							dataLabelEle.hide();
							conEle.hide();
							connectorDirection = "";
						}
						if (!enableWrap)
						    chartObj.svgRenderer.drawText(textOptions, datalabelText, chartObj.gSeriesTextEle[seriesIndex]);
                        var datalabelSize = measureText(datalabelText, datalabelText.length, dataLabelFont);
                        var untrimmedSize = measureText(commonEventArgs.data.text, null, dataLabelFont);
                        var bounds;
                        var minX, minY, maxX, maxY;
                        if (type == "pie" || type == "doughnut" || type=="pieofpie") {
                            if (labelPosition == "inside" && position.connectorDirection == "") {
                                minX = textOptions.x - datalabelShape.width / 2;
                                maxX = textOptions.x + datalabelShape.width / 2;
                                minY = textOptions.y - datalabelShape.height / 2;
                                maxY = textOptions.y + datalabelShape.height / 2;
                            }
                            else {
                                if (chartModel.circleCenterX[srIndex] < textOptions.x) {
                                    minX = textOptions.x;
                                    maxX = textOptions.x + datalabelShape.width;
                                }
                                else {
                                    minX = textOptions.x - datalabelShape.width;
                                    maxX = textOptions.x;
                                }
                                minY = textOptions.y - datalabelShape.height / 2;
                                maxY = textOptions.y + datalabelShape.height / 3;
                            }
                            bounds = { centerX: chartModel.circleCenterX[srIndex], minX: minX, minY: minY, maxX: maxX, maxY: maxY, width: datalabelSize.width, height: datalabelSize.height, labelPosition: labelPosition };
                        }

                        if ((type == "funnel" || type == "pyramid")) {
                            legendWidth = legendPosition == "left" ? legendWidth : 0;
                            var legendHeight = 0;
                            if (connectorDirection != "" || connectorType == "bezier") {
                                minX = textOptions.x - legendWidth;
                                maxX = textOptions.x + datalabelShape.width - legendWidth;
                                minY = textOptions.y - datalabelShape.height;
                                maxY = textOptions.y;
                            }
                            if (!enableCanvasRendering)
                                bounds = { minX: minX + chartObj.pyrX + legendWidth, maxX: maxX + chartObj.pyrX + legendWidth, minY: minY + chartObj.pyrY + legendHeight, maxY: maxY + chartObj.pyrY + legendHeight, width: datalabelSize.width, height: datalabelSize.height, labelPosition: labelPosition };
                            else
                                bounds = { minX: minX + legendWidth, maxX: maxX + legendWidth, minY: minY + legendHeight, maxY: maxY + legendHeight, width: datalabelSize.width, height: datalabelSize.height, labelPosition: labelPosition };
                        }
                        accDataLabelRegion = { bounds: bounds, trimmedText: datalabelText, text: commonEventArgs.data.text, type: type, font: dataLabelFont };
                        chartObj.accDataLabelRegion.type = type;
                        chartObj.accDataLabelRegion.connectorDirection = connectorDirection;
                        if (ej.util.isNullOrUndefined(chartObj.accDataLabelRegion[srIndex])) {
                            chartObj.accDataLabelRegion[srIndex] = [];
                        }
                       
                        if (chartObj.accDataLabelRegion[srIndex].length < currentseries._visiblePoints.length)
                            chartObj.accDataLabelRegion[srIndex].push(accDataLabelRegion);
                        if (((!isNull(pointIndex) && currentseries.explode) && (!enableCanvasRendering)))
                            chartObj.accDataLabelRegion[srIndex].splice(pointIndex, 1, accDataLabelRegion);
                        if (pieSeriesIndex == 1 && chartObj.accDataLabelRegion[srIndex].length == currentseries._visiblePoints.length) {
                            chartObj.accDataLabelRegion[seriesIndex] = $.merge($.merge([], chartObj.accDataLabelRegion[0]), chartObj.accDataLabelRegion[1]);
                        }
                    }
                   
                    else if(!enableWrap) {
                        seriesType.dataLabelSymbol(seriesIndex, currentseries, pointIndex, positionX, positionY, textWidth, textHeight, symbolName, chartObj);
                        chartObj.svgRenderer.drawText(textOptions, commonEventArgs.data.text, chartObj.gSeriesTextEle[seriesIndex]);
                    }
                   
                    if (enableWrap) {
                        var labelTextSize = measureText(labelText, null, labelfont), maxWidth, collection, padding = 5;
                        collection = this.updateWrappedText(textOptions, labelText, labelTextSize, point, type, startX, startY, radius, chartModel.innerRadius[seriesIndex], _labelPosition, dataLabel, (connectorType == "bezier") ? (currentseries._enableSmartLabels ? (!isNull(position.bezierPath)? position.bezierPath.points:"") : position.points) : connectorDirection);
                        datalabelText = collection.text;
                        textOptions.x = parseFloat(collection.x); textOptions.y = parseFloat(collection.y);
                        if (datalabelText != "") {
                            var x = (textOptions['text-anchor'] == 'start') ? parseFloat(textOptions.x + collection.width / 2) : (isNull(connectorDirection) || connectorDirection == "") ? parseFloat(textOptions.x) : parseFloat(textOptions.x - collection.width / 2);
                            seriesType.dataLabelSymbol(seriesIndex, currentseries, pointIndex, x, textOptions.y + collection.height / 2 - labelTextSize.height / 2 - padding, collection.width, collection.height, symbolName, chartObj);
                        }
                        if (!isNull(connectorDirection) && connectorDirection != "") {
                            var pyrX, pyrY;
                            if (type == "funnel" || type == "pyramid") {
                                pyrX = ((chartModel.enableCanvasRendering) ? this.chartObj.pyrX : 0);
                                pyrY = ((chartModel.enableCanvasRendering) ? this.chartObj.pyrY : 0);
                            }
                            var connectorSegments = connectorDirection.split(" "), connectorLength = (_labelPosition == "outsideextended" && (type == "pie" || type == "doughnut")) ? connectorSegments.length - 2 : connectorSegments.length;
                            connectorSegments[connectorLength - 1] = textOptions.y - pyrY + (collection.height / 2 - labelTextSize.height);//canvas connector line issue fix
                            connectorSegments[connectorLength - 2] = textOptions.x - pyrX;//canvas connector line issue fix
                            connectorDirection = connectorSegments.toString().replace(/,/g, " ");
                        }
                        connectorDirection = (collection.exceed == false) ? "" : connectorDirection;

                        if ((chartModel.enableCanvasRendering || this.chartObj.vmlRendering) && (typeof datalabelText == "object")) {
                            for (var txt = 0, tempY = textOptions.y, tempX = textOptions.x; txt < datalabelText.length; txt++) {
                                textOptions.y = parseFloat(tempY) + ((txt) * textsize.height);
                                textOptions.x = tempX;
                                chartObj.svgRenderer.drawText(textOptions, datalabelText[txt], chartObj.gSeriesTextEle[seriesIndex]);
                            }
                        }
                        else {
                            chartObj.svgRenderer.drawText(textOptions, datalabelText, chartObj.gSeriesTextEle[seriesIndex]);
                        }
                    }
                }
                if ((type == "pyramid" || type == "funnel") && dataLabel.template && currentseries._enableSmartLabels) {
                    position = seriesType.updateSmartLabel(sender, currentseries, pointIndex, point);
                    positionX = position.xPos;
                    positionY = position.yPos;
                    connectorDirection = position.connectorDirection;

                }
                pointColor = jQuery.type(chartModel.pointColors[point.actualIndex]) == "array" ? chartModel.pointColors[point.actualIndex][0].color : chartModel.pointColors[point.actualIndex];
                // To set the point color when set the color in empty point
                pointColor = point.fill ? jQuery.type(point.fill) == "array" ? point.fill[0].color : point.fill : pointColor;

                var translate = [];
                translate[0] = pyrX;
                translate[1] = pyrY;
                if ((connectorType != 'bezier' || chartObj.vmlRendering || (type == "pyramid" || type == "funnel")) && (connectorDirection != '')) {
                    if (connectorDirection) {
                        var connectorOptions = {
                            'id': chartObj.svgObject.id + "_connectorLine_series"+seriesIndex+"_point_" + pointIndex,
                            'stroke': (connectorLine.color) ? connectorLine.color : pointColor,
                            'stroke-width': connectorLine.width,
                            'd': connectorDirection
                        };
                        chartObj.svgRenderer.drawPath(connectorOptions, chartObj.gConnectorEle[seriesIndex], translate);
                    }
                }
            }
        },

	_overlapLegend: function (textOptions, datalabelText, chart,seriesIndex) {

	    var svgHeight = chart.svgHeight,
            svgWidth = chart.svgWidth,
	        legendBounds = chart.model.LegendActualBounds,
	        elementSpacing = chart.model.elementSpacing,
            margin = chart.model.series[seriesIndex].marker.dataLabel.margin,
            shape = chart.model.series[seriesIndex].marker.dataLabel.shape,
            top = shape != 'none' ? margin.top : 0,
            bottom = shape != 'none' ? margin.bottom : 0,
	        textHeight = ej.EjSvgRender.utils._measureText((datalabelText, null, textOptions)).height,
	        dataLabelEndPos = textOptions.y + textHeight + bottom,
	        dataLabelTopPos = textOptions.y - top,
	        legendPosition = chart.model.legend.position.toLowerCase(),
            title = chart.model.title,
            subTitle = title.subTitle,
            margin = chart.model.margin,
            borderSize = chart.model.border.width,
            modelsubTitleHeight = subTitle.text == "" ? 0 : ej.EjSvgRender.utils._measureText(subTitle.text, svgWidth - margin.left - margin.right, subTitle.font).height + elementSpacing,
            titleLocation = chart.model._titleLocation ? chart.model._titleLocation.Y : 0 + modelsubTitleHeight,
	        text = datalabelText, legendPosY;
	        
        
	    if (legendPosition == 'bottom') {
	        legendPosY = svgHeight - chart.model.LegendActualBounds.Height - elementSpacing;
            text = dataLabelEndPos > legendPosY ? '' : text;
	    } else {
	        legendPosY = titleLocation == 0 ? borderSize + elementSpacing : titleLocation + elementSpacing;
            text = dataLabelTopPos < (legendPosY + legendBounds.Height) ? '' : text;
        }
	    if (dataLabelEndPos > svgHeight)
	        text = '';
	    return text;

	},

	updateWrappedText: function (textOptions, labelText, textSize, currentPoint, seriesType, centerX, centerY, radius, innerRadius, labelPlacement, dataLabel,connector) {
	    var chartObj = this.chartObj,
            point = currentPoint,
            previousBoundary = {},
            textWidth = textSize.width,
            textHeight = textSize.height,
            wrapTrim,
            text, arrayText,
            textPosX = textOptions.x,
            textPosY = textOptions.y,
            segmentHeight,
            segmentWidth,
            midX = textPosX,
            exceed = true,
            areaSpace = chartObj.model.m_AreaBounds.X,
	        midY = textPosY,
            connectorPoints,
            txtLength,
            textX, textY;
	    if (!ej.util.isNullOrUndefined(connector) && connector != "") {
	        if (typeof connector == "object") {
	            textX = connector.X;
	            textY = connector.Y;
	        }
	        else {
	            connectorPoints = connector.split(" "),
                txtLength = (labelPlacement == "outsideextended") ? (connectorPoints.length - 2) : (dataLabel.connectorLine.type.toLowerCase() == "bezier") ? connectorPoints.length-2 : connectorPoints.length;
	            if (connectorPoints[txtLength - 1].indexOf(",") > -1) {
	                var tempXY = connectorPoints[txtLength].split(",");
	                textX = tempXY[0];
	                textY = tempXY[1];
	            }
	            else {
	                textX = connectorPoints[txtLength - 2];
	                textY = connectorPoints[txtLength - 1];
	            }
	        }
	    }
	    if (seriesType == 'pie' || seriesType == 'doughnut') {
	        if (labelPlacement == "inside") {
	            var innerRadius = (ej.util.isNullOrUndefined(innerRadius)) ? 0 : innerRadius,
                    chartStartingAngle = -.5 * Math.PI,
                    startAngle = point.startAngle + chartStartingAngle,
                    endAngle = point.endAngle + chartStartingAngle,
                    midAngleRadian = point.midAngle + chartStartingAngle,
                    midAngle = ((180 / Math.PI) * midAngleRadian) % 360,
                    startANG = ((180 / Math.PI) * startAngle) % 360,
                    endANG = ((180 / Math.PI) * endAngle) % 360,
                    radiusTextPosPercentage = (seriesType == "doughnut") ? 0.75 : 0.5;
	            midX = this.getXCordinate(centerX, (radius * radiusTextPosPercentage), midAngleRadian),
                midY = this.getYCordinate(centerY, (radius * radiusTextPosPercentage), midAngleRadian),
                startPointX = this.getXCordinate(centerX, (radius), startAngle),
                startPointY = this.getYCordinate(centerY, (radius), startAngle),
                endPointX = this.getXCordinate(centerX, (radius), endAngle),
                endPointY = this.getYCordinate(centerY, (radius), endAngle),
                midPointX = this.getXCordinate(centerX, (radius), midAngleRadian),
                midPointY = this.getYCordinate(centerY, (radius), midAngleRadian),
                innerCenterX = this.getXCordinate(centerX, innerRadius, midAngleRadian),
                innerCenterY = this.getYCordinate(centerY, innerRadius, midAngleRadian),
	            segmentBounds = this.getSegementMinMax([startPointX, endPointX, (seriesType == "doughnut") ? innerCenterX : centerX, midPointX], [startPointY, endPointY, (seriesType == "doughnut") ? innerCenterY : centerY, midPointY]);
	            segmentBounds.width = (segmentBounds.height > radius) ? segmentBounds.height : segmentBounds.width;
	            segmentWidth = segmentBounds.width * 0.75;
	        }
	        else if (((labelPlacement == "outside" || labelPlacement == "outsideextended") && (seriesType == 'pie' || seriesType == 'doughnut')) && connector != "") {
	            var areabounds = chartObj.model.m_AreaBounds,
                    areaStartX = areaSpace,
	                areaEndX = chartObj.svgWidth - areaSpace;	            
	            segmentWidth = (textX <= centerX) ? (textX - areaStartX) : (areaEndX - textX);
	            midX = textX;
                midY = textY;
	        }
	    }
	    else {
	        var svgWidth = chartObj.svgWidth;
	        if (labelPlacement == "outside") {
	            segmentWidth = svgWidth - (textX);
	        }
	        else {
	            segmentWidth = point.width;
	            var insidenooverap=chartObj.isPointInPolygon(point.Polygon,{x:midX+textWidth,y:midY});
	            if (!insidenooverap) {
	                segmentWidth = ((seriesType == "funnel") ? (point.Polygon[3].x - point.Polygon[4].x) : (point.Polygon[1].x - point.Polygon[0].x));
	                segmentHeight = point.Polygon[3].y - point.Polygon[0].y;
	                var changedHeight = Math.round(textWidth / segmentWidth) * textHeight, padding = 5,
                    exceed = ((changedHeight >= (segmentHeight - (2 * padding))) || (segmentWidth <= 50)) ? true : false;
	                if (!exceed) {
	                    textOptions["text-anchor"]="middle";
	                    midX = ((seriesType == "funnel") ? point.Polygon[4].x : point.Polygon[0].x)+(segmentWidth/2) + ((chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0);
	                    midY = ((point.Polygon[0].y + point.Polygon[3].y) / 2) - (changedHeight / 2) + ((chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0);
	                    point._labelPlacement = "insidenooverlap";
	                }
	                else {
	                    segmentWidth = svgWidth - midX - areaSpace;
	                }
	            }
	        }	        
	    }
	    if (labelPlacement == "inside" || (connector != ""|| !ej.util.isNullOrUndefined(connector))) {
	        maxWidth = (dataLabel.maximumLabelWidth != null) ?  dataLabel.maximumLabelWidth : parseFloat(segmentWidth);
	        text = labelText;
	        wrapText = chartObj._rowsCalculation({ text: text.toString(), font: dataLabel.font }, maxWidth, wrapTrim);
	        text = wrapText.textCollection;
	        arrayText = $.extend([], text);
	        for (var i = 0, temp; i < arrayText.length - 1; i++) {
	            arrayText[i + 1] = arrayText[i].concat("<br/>", arrayText[i + 1]);
	        }
	        arrayText = arrayText[arrayText.length - 1];
	    }
	    else {
	        text = "";
	    }
	    var changedText = ej.EjSvgRender.utils._measureText(arrayText, segmentWidth, dataLabel.font);
	    changedText.height += (textHeight / 2);// size differing canvas and svg space between new line text more 
	    if ((seriesType == "pie" || seriesType == "doughnut") && (labelPlacement == "inside")) {
	        midY = midY - (changedText.height / 2) + (textHeight / 2);
	        text = (changedText.height > (segmentBounds.height)) ? "" : text;
	        text = (changedText.width > (segmentBounds.width)) ? "" : text;
	        if (text == "") {
	            changedText.width = 0; changedText.height = 0;
	        }
	    }
	        previousBoundary = (this.currentBoundary == 'undefined') ? "undefined" : this.currentBoundary;
	        this.currentBoundary = ({ X: midX - (changedText.width / 2), Y: parseFloat(midY), Height: changedText.height, Width: changedText.width });
	        if (!!previousBoundary && (point._labelPlacement != "insidenooverlap")) {
	            var overlap = this.checkOverlapping(chartObj,this.currentBoundary, previousBoundary, seriesType, text, firstSegment, midAngle, labelPlacement, {startAngle:startAngle,endAngle:endAngle,radius:radius,centerX:centerX,centerY:centerY});
	            
	            if (overlap[1] && (overlap[2] != "")) {
	                previousBoundary = overlap[0];
	                midX = previousBoundary.X + (previousBoundary.Width / 2);
	                midY = previousBoundary.Y;
	                text = overlap[2];
	                changedText.height = overlap[0].Height;
	                changedText.width = overlap[0].Width;
	            }
	            else {
	                text = overlap[2];
	            }
	        }
	        else{
	            firstSegment = this.currentBoundary;
                
	        }
	        exceed = (text == "") ? false : exceed;
	        this.currentBoundary = (text == "") ? previousBoundary : this.currentBoundary;
	        return { text: text, x: midX, y: midY, exceed: exceed, height: changedText.height, width: changedText.width };
	},
	checkOverlapping: function (chartObj,current, previous, seriesType,text,firstSegment,midAngle,labelPlacement,segmentBounds) {
	    var gap = 10, overlap = false, textOverlap,xOverlap,yOverlap;
	    textOverlap = this.getOverlapArea(current,previous);
	    overlap = textOverlap.overlap;
	    xOverlap = textOverlap.xOverlap;
	    yOverlap = textOverlap.yOverlap;
	        if (overlap) {
	            if (seriesType == "pie" || seriesType == "doughnut") {
	                if (labelPlacement.toLowerCase() == "inside") {
	                    var minDiff = (xOverlap >= yOverlap) ? "Y" : "X",
                            flag;
	                    if (minDiff == "X") {
	                        flag = (midAngle > 0 && midAngle < 180) ? -1 : 1;
	                        current.X += flag * (xOverlap + gap);
	                    }
	                    else {
	                        flag = (midAngle > 90 && midAngle < 270) ? -1 : 1
	                        current.Y += (flag) * (yOverlap + gap);
	                    }
	                        
	                }
	                else {
	                    current.Y -= (yOverlap + gap);
	                }
	            }
	            else if(seriesType=="funnel") {
	                    current.Y = (previous.Y - current.Height - gap);
	            }
	            else {
	                    current.Y = (previous.Y + previous.Height + gap);
	            }
	        }
	        if ((seriesType == "pie" || seriesType == "doughnut" || seriesType == "pieofpie") && labelPlacement.toLowerCase() == "inside") {
	            textOverlap = this.getOverlapArea(current, firstSegment);
	            text = (textOverlap.overlap == true) ? "" : text;
	            overlap = (textOverlap.overlap == true) ? true : overlap;
	            var labelexceeding = this.checkSegmentExceeding(current, segmentBounds);
	            text = (!labelexceeding) ? "" : text;
	        }
	        else if(seriesType=="funnel"){
	            if ((current.Y + current.Height) >= previous.Y) {
	                text = "";
	            }
	        }
	        else if(seriesType=="pyramid"){
	            if ((current.Y + current.Height) <= (previous.Y+previous.Height)) {
	                text = "";
	            }
	        }
	        var areaBounds = chartObj.model.m_AreaBounds;
	    return [{ X: current.X, Y: current.Y, Height: current.Height, Width: current.Width }, overlap,text];
	},
	getOverlapArea: function (firstTextSegment, lastTextSegment) {
	    var xOverlap = Math.max(0, Math.min((firstTextSegment.X + firstTextSegment.Width), (lastTextSegment.X + lastTextSegment.Width)) - Math.max(firstTextSegment.X,lastTextSegment.X)),
            yOverlap = Math.max(0, Math.min((firstTextSegment.Y + firstTextSegment.Height), (lastTextSegment.Y + lastTextSegment.Height)) - Math.max(firstTextSegment.Y,lastTextSegment.Y)),
            overlapArea = xOverlap * yOverlap;
	    overlap = (overlapArea > 0) ? true : false;
	    return { overlap: overlap, xOverlap: xOverlap, yOverlap: yOverlap };
	},
	getSegementMinMax:function(xArray,yArray){
	    var minX, maxX, minY, maxY, width, height;
	    xArray.sort(function (a, b) { return a - b; });
	    yArray.sort(function (a, b) { return a - b; });
	    minX = xArray[0]; maxX = xArray[3]; minY = yArray[0]; maxY = yArray[3];
	    width = maxX - minX;
	    height = maxY - minY;
	    return { minX: minX, maxX: maxX, minY: minY, maxY: maxY, width: width, height: height };
	},
	checkSegmentExceeding:function(textBounds,limits){
	    var inside=false,midPointX = textBounds.X + (textBounds.Width / 2), midPointY = textBounds.Y + (textBounds.Height / 2);
	    var dx = midPointX - limits.centerX, dy = midPointY - limits.centerY, angle = Math.atan2(dy, dx), pointRadius = Math.abs(Math.sqrt(dx * dx + dy * dy));
	    if (pointRadius <= limits.radius) {
	        var PI = 2 * Math.PI;
	        angle = (angle < 0) ? (PI + angle) : angle;
	        limits.startAngle = (limits.startAngle < 0) ? (PI + limits.startAngle) : limits.startAngle;
	        limits.endAngle = (limits.endAngle < 0) ? (PI + limits.endAngle) : limits.endAngle;
	        if (angle <= limits.endAngle && angle >= limits.startAngle) {
	            inside = true;
	        }
	        else if ((angle <= PI && angle >= limits.startAngle && limits.endAngle < limits.startAngle) || (angle > 0 && angle <= limits.endAngle && limits.startAngle > limits.endAngle)) {
	            inside = true;
	        }
	    }
	    return inside;
	},
    drawBezierSegment: function (controlpoints, currentseries, pointIndex, sender, seriesIndex) {
	    //currentseries.points[pointIndex].marker = currentseries.points[pointIndex].marker ? currentseries.points[pointIndex].marker : {};
	    //var marker = $.extend(true, {}, currentseries.marker, currentseries.points[pointIndex].marker);
	    var polyLine = ej.EjSvgRender.utils._getStringBuilder();
	    var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	    for (var i = 0; i <= 16; i++) {
	        var t = i / 16;
	        var points = seriesType.GetBezierPoint(t, controlpoints, 0, controlpoints.length, currentseries);
	        polyLine.append(points.X + "," + points.Y + " ");
	    }
	    var connectorOptions = {
	        'id': sender.svgObject.id + "_bezierLine_series"+ seriesIndex + "_point" + pointIndex,
	        'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : sender.model.pointColors[pointIndex],
	        'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
            'fill':'none',
	        'points': polyLine.toString()
	    };
		if (!currentseries._enableSmartLabels)
	       sender.svgRenderer.drawPolyline(connectorOptions, sender.gConnectorEle);
		else
	       return connectorOptions;
	},
	GetBezierPoint:function(t, controlPoints, index, count, currentseries)
	{
	    var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	    if (count == 1)
	        return controlPoints[index];
	    var p0 = seriesType.GetBezierPoint(t, controlPoints, index, count - 1, currentseries);
	    var p1 = seriesType.GetBezierPoint(t, controlPoints, index + 1, count - 1, currentseries);
	    var x = (p0.X) ? p0.X : p0.x;
	    var y = (p0.Y) ? p0.Y : p0.y;
	    var x1 = (p1.X) ? p1.X : p1.x;
	    var y1 = (p1.Y) ? p1.Y : p1.y;
	    var X = (1 - t) * x+ t * x1;
	    var Y = (1 - t) * y + t * y1;
	    if (p0.x)
	        return { x: X, y: Y }
	    else
	        return { X: X, Y: Y };;
	    


       // return new Point((1 - t) * p0.X + t * p1.X, (1 - t) * p0.Y + t * p1.Y);
    },
	
    drawLabelTemplate: function (series, point, pointIndex, location, sender) {
        // method for data label template
        if(sender)
            this.chartObj = sender;
        var areaBoundsX = this.chartObj.model.m_AreaBounds.X;
        var areaBoundsY = this.chartObj.model.m_AreaBounds.Y;
        var areaBoundsWidth = this.chartObj.model.m_AreaBounds.Width;
        var areaBoundsHeight = this.chartObj.model.m_AreaBounds.Height;
	    var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var areaType = this.chartObj.model.AreaType;
        var currentSeries = this.chartObj.model.series[seriesIndex];
		var xPosition = 0, yPosition = 0;
        var chartContainer = this.chartObj._id;
        var marker = $.extend(true, {}, series.marker, point.marker);
        var style = { 'interior': marker.dataLabel.fill, 'opacity': marker.dataLabel.opacity, 'borderColor': marker.dataLabel.border.color, 'borderWidth': marker.dataLabel.border.width };
        var color, width, height;
		var type = this.chartObj.model.chartRegions[pointIndex] ? this.chartObj.model.chartRegions[pointIndex].type : "";
		if (areaType != "none" || this.chartObj.model.enable3D) {

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
			if ($("#" + marker.dataLabel.template).length == 0) // To check the specified div is in DOM
                return;
            else
            var cloneNode = $("#" + marker.dataLabel.template).clone();
            $(cloneNode).attr("id", marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");

            var pointX = point.x; 
            point.dataLabeltemplate = true;	
            point.id = marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer;			
            if (series.xAxis._valueType.toLowerCase() == "datetime")
                point.x = (ej.format(new Date(point.xValue), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis._intervalType) : series.xAxis.labelFormat), this.chartObj.model.locale));
            if (series.xAxis._valueType.toLowerCase() == "category")
                point.x = ej.EjSvgRender.utils._getLabelContent(pointIndex, series.xAxis, this.chartObj.model.locale);

            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            point.x = pointX;
            var display = (areaType == "cartesianaxes" || !series.enableAnimation || (series.type.toLowerCase() == "pyramid" || series.type.toLowerCase() == "funnel")) ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
			//template_group_container Div appended in chartcontainer_container Div 
            $(templateContainer).appendTo('#' + 'chartContainer_' + chartContainer);
            var areaBound = this.chartObj.model.m_AreaBounds;
            if (areaType == "cartesianaxes" && !this.chartObj.model.enable3D) {
                xPosition = areaBound.X + (this.chartObj.model.requireInvertedAxes ? currentSeries.yAxis.plotOffset : currentSeries.xAxis.plotOffset);
                yPosition = currentSeries.yAxis.y;
            }
            width = $cloneNode.width() / 2;
            height = $cloneNode.height();
            if (ej.util.isNullOrUndefined(style) && style.interior)
                color = style.interior;
            else
                color = (areaType == "none") ? this.chartObj.model.pointColors[pointIndex] : this.chartObj.model.seriesColors[seriesIndex];
        }
        else {
            if (ej.util.isNullOrUndefined(style) && style.interior)
                color = style.interior;
            else
                color = this.chartObj.model.pointColors[pointIndex];
             width = this.chartObj.model.circleCenterX[seriesIndex]> location.X ? point.size.width : 0;
            $cloneNode = $('#template_group_' + chartContainer).find("#" + marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            point.id = marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer;
            height = (!this.chartObj.model.enable3D) ? point.size.height / 2 : 0;
        }
        
        if (currentSeries.type == 'hilo' || currentSeries.type == 'hiloopenclose' || currentSeries.type == 'candle')
            color = currentSeries._visiblePoints[pointIndex]._hiloFill;
            color = jQuery.type(color) == "array" ? color[0].color : color;
			
           
        if (currentSeries.type.indexOf("bar") != -1) {
            var left = location.X + xPosition - width;
            var top = location.Y + ((!this.chartObj.model.enable3D) ? areaBound.Y : 0) - height + currentSeries.xAxis.plotOffset;
            var right = location.X + xPosition + width;
        } else {
        var left =location.X + xPosition - width;
        var top = yPosition + location.Y - height;
        var right = location.X + xPosition + width;
       }
       var backgroundColor = $cloneNode[0].style.backgroundColor;
       $cloneNode.css("left", left).css("top", top).css("background-color", (backgroundColor != '' ? backgroundColor : color)).css("display", "block").css("cursor", "default");
	    
		if(this.chartObj.model.AreaType !="cartesianaxes"){
          point.xPos = left - this.chartObj.pyrX;
          point.yPos = top - this.chartObj.pyrY;
		  point.textOptionsacc = { x: left - this.chartObj.pyrX, y: top - this.chartObj.pyrY };
		}
		else{
		    if (point.y == point.low) {
		        point.xPosLow = left;
		        point.yPosLow = top;
		        point.textOptionsLow = { x: left, y: top };
		        point.dataLabelLow = { textX: left, textY: top, x: left, y: top };
		    }
		    else {
		        point.xPos = left;
		        point.yPos = top;
		        point.seriesIndex = seriesIndex;
		        point.textOptions = { x: left, y: top };
		        point.dataLabel = { textX: left, textY: top, x: left, y: top };
		        point.margin = { top: 0, bottom: 0, left: 0, right: 0 };
		    }
		}
        point.height = $cloneNode.outerHeight();
        point.width = $cloneNode.outerWidth();
        var topHeight = leftWidth = rightWidth = 0;
        var bottomHeight = 0;
        if (areaType == "cartesianaxes") {
            if ((left > xPosition + areaBound.Width || top + height > areaBound.Y + areaBound.Height) || ((left < xPosition || top < yPosition || left > xPosition + areaBound.Width || top > yPosition + areaBound.Height || right > areaBound.Width + areaBound.X)))
            {
                width = $cloneNode.outerWidth();
                if (left < areaBoundsX) {                                                  
                    leftWidth = areaBoundsX - left;
                }
                if (areaBoundsHeight + areaBoundsY < top + height) {                      
                    var excessHeight = (top + height) - (areaBoundsHeight + areaBoundsY);
                    bottomHeight = height - excessHeight;
                }
                if (top < areaBoundsY) {                                                 
                    topHeight = areaBoundsY - top;
                }
                if (areaBoundsWidth + areaBoundsX < left + width) {                      
                    var excessWidth = (left + width) - (areaBoundsWidth + areaBoundsX);
                    rightWidth = width - excessWidth;
                }
                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                document.getElementById($cloneNode[0].id).style.clip = "rect(" + topHeight + "px," +  
                         rightWidth + "px," +
                         bottomHeight + "px," +
                         leftWidth + "px)";
            }
        }
    },
	
	animateLabelTemplate : function(options) {
	// method to animate data label template
        var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
        var length = options._visiblePoints.length;
        var secondsPerPoint = 1000;
        
        for (var i = 0; i < length; i++) {            
            var ele = options.marker.dataLabel.template + '_' + seriesIndex + '_' + i + '_' + this.chartObj._id;
            ele = $('#' + ele);
            ele.css("display", "none");
            ele.delay(secondsPerPoint).fadeIn(300);
			}
    },
	
	textPosition: function (series, seriesIndex, point, textOffset, type, x, y, chartRegionIndex, index, params) {
	    var marker = $.extend(true, {}, series.marker, point.marker),
            dataLabel = marker.dataLabel,
            horizontalTextAlignment = dataLabel.horizontalTextAlignment.toLowerCase(),
            verticalTextAlignment = dataLabel.verticalTextAlignment.toLowerCase(),
            textPosition = dataLabel.textPosition.toLowerCase(),
            visiblePointLength = series._visiblePoints.length,
            seriesPointIndex = chartRegionIndex,
            lineHeight = 10,
			padding = 3,
            margin = dataLabel.margin,
			isYInversed = series.yAxis.isInversed,
			xOrigin = this.chartObj._getXCrossValue(series, series.xAxis, params);	    
	    var seriesType = (type.indexOf("bar") != -1 || (type.indexOf("column") != -1 && type!="rangecolumn")) ? (this.chartObj.model.requireInvertedAxes) ? "bar" : "column" : type;
	    if (seriesType == 'column' || seriesType == 'stackingcolumn' || seriesType == 'stackingcolumn100') {
			var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds,
				originChanged = params.axes[series.xAxis.name]._validCross||false;
                pointHeight = bounds.Height,
                pointWidth = bounds.Width;

			if(!originChanged){				
				if (textPosition == "bottom") {
					if ((point.y > xOrigin && !series.yAxis.isInversed) || (point.y < xOrigin && series.yAxis.isInversed))
						y = y + pointHeight - margin.bottom;
					else 
						y = textOffset.height + margin.bottom;
				} else if (textPosition == "middle") {
					if ((point.y > xOrigin && !series.yAxis.isInversed) || (point.y < xOrigin && series.yAxis.isInversed))
						y += pointHeight / 2 + (textOffset.height) / 4;
					else  
						y -= pointHeight / 2;
				}
				else {
					if ((point.y > xOrigin && series.yAxis.isInversed) || (point.y < xOrigin && !series.yAxis.isInversed))
                   y += lineHeight;
               else
                   y -= lineHeight;
				}				
           }
			else{
				var origin = Math.abs((isYInversed ? series.yAxis.visibleRange.min : series.yAxis.visibleRange.max) - xOrigin) / series.yAxis.visibleRange.delta * series.yAxis.height;
				if (textPosition == "bottom") {
					if (y <= origin)
						y = y + pointHeight - margin.bottom;
					else 
						y = y - pointHeight + textOffset.height - margin.bottom;
				} else if (textPosition == "middle") {
					if (y < origin)
						y += pointHeight / 2 + (textOffset.height) / 4;
					else  
						y -= pointHeight / 2;
				}
				else {
					if ((y < origin && isYInversed) || (y < origin && !isYInversed))
						y -= (lineHeight + (2 * textOffset.height) / 2);
					else
						y += (lineHeight + (2 * textOffset.height) / 2);
				}			
			}

           if (horizontalTextAlignment && horizontalTextAlignment == "far")
               x += pointWidth / 2;
            else if (horizontalTextAlignment && horizontalTextAlignment == "near")
               x -= pointWidth / 2;

           if (verticalTextAlignment && verticalTextAlignment == "near") {
               if (textPosition == "bottom")
                   y += lineHeight;
               else
                   y += lineHeight - padding;
           }
           else if (verticalTextAlignment && verticalTextAlignment == "far") {
               if (textPosition == "bottom")
                   y -= lineHeight;
               else
                   y -= lineHeight;
           }
        }

	    else if (seriesType.indexOf("bar") != -1) {
	        var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds,
                pointWidth = bounds.Width,
			    originChanged = params.axes[series.xAxis.name]._validCross||false,
	            pointHeight = bounds.Height;
			y = y+ textOffset.height / 4;
			if(!originChanged)
			{				
				if (textPosition == "bottom") {
		  	    if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
		  	        x = margin.left + textOffset.width / 4;
		  	    else
		  	        x += (pointWidth - textOffset.height - margin.right);
		  	} else if (textPosition == "middle") {
			      if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
			          x -= pointWidth / 2 + textOffset.width / 2;
			      else
			          x += pointWidth / 2;
		  	} else {
					if ((point.y >= 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
		  	        x  += textOffset.width/4 + lineHeight ;
                else
		  	        x -= textOffset.width/4 + lineHeight;
           }
			}
			else
			{
				var yOrigin = series.xAxis._crossValue||0,
				origin = Math.abs(isYInversed ? series.yAxis.visibleRange.max : series.yAxis.visibleRange.min - yOrigin) / series.yAxis.visibleRange.delta * series.yAxis.width;
				if (textPosition == "bottom") {
				    if ((x < origin && !isYInversed)||(x >= origin && isYInversed))
						x = x + pointWidth - textOffset.width;
					else
						x = x - pointWidth + margin.left;
				} else if (textPosition == "middle") {
				    if ((x < origin && !isYInversed) || (x >= origin && isYInversed))
				        x = x + pointWidth / 2 - textOffset.width / 2;
				    else
				        x = x - pointWidth / 2 - textOffset.width / 2;
				}
				else {
				    if ((x < origin && !isYInversed) || (x >= origin && isYInversed))
				        x -= (lineHeight + (2 * textOffset.width) / 2);
				    else
				        x += (lineHeight + (2 * textOffset.width) / 2);
				}
			}

		  	if (horizontalTextAlignment && horizontalTextAlignment == "far") {
		  	    if (textPosition == "bottom")
		  	        x += textOffset.width - textOffset.width / 4;
		  	    else
		  	        x += textOffset.width/4 + lineHeight;
		  	}
		  	else if (horizontalTextAlignment && horizontalTextAlignment == "near") {
		  	    if (textPosition == "bottom")
		  	        x -= textOffset.width / 4;
		  	    else
		  	        x -= lineHeight - textOffset.width/4 - padding ;
		  	}
		  	else // for center
		  	    x += textOffset.width / 4;
				
            if (verticalTextAlignment && verticalTextAlignment == "near")
                y += pointHeight / 2;
            else if (verticalTextAlignment && verticalTextAlignment == "far")
                y -= pointHeight / 2;
	    }
	    else if (seriesType == "rangecolumn" ) {
	         var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds;
	         var height = 0;
	         var width = 0;
	         if (point.high != point.y) {
	            height = bounds.Height;
	     	 	width = bounds.Width;
	          }
	       	 if (this.chartObj.model.requireInvertedAxes) {
	             if (textPosition == "top")
                     x = x - width + (lineHeight + (2 * textOffset.width) / 2);
                 else if (textPosition == "bottom")
                     x = x - width - (2 * textOffset.width + lineHeight);
	             else if (textPosition == "middle")
                     x = x - width;
             }
	         else{
                 if(textPosition == "top")
                     y += height - lineHeight;
	        
                 else if(textPosition == "bottom")
                     y = y + height + (2 * textOffset.height + lineHeight);
	             else if (textPosition == "middle")
                     y = y + height;
	         }
	         if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                  x += bounds.Width / 2;
	         else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                  x -= bounds.Width / 2;	    
            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                  y += lineHeight - padding;
	         else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                  y -= lineHeight;
	    
        }
		
		else if (seriesType == "boxandwhisker") {
	         var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds;
	         var height = 0;
	         var width = 0;
	         if (this.chartObj.model.requireInvertedAxes) {
	             if (textPosition == "top")
	                 x = x - width + (lineHeight + (2 * textOffset.width) / 2);
	             else if (textPosition == "bottom")
	                 x = x - width - (2 * textOffset.width + lineHeight);
	             else if (textPosition == "middle")
	                 x = x - width;
	         }
	         else {
	             if (textPosition == "top")
	                 y += height - padding;
	             else if (textPosition == "bottom")
	               //  y = y + height + (2 * textOffset.height + lineHeight);
	                 y += height + lineHeight + (textOffset.width / 2);
	             else if (textPosition == "middle")
	                 y += height + lineHeight;
	         }
	         if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
				  x +=  2 * textOffset.width ;
	         else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                  x -= 2 * textOffset.width  ;	
				  
            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                  y = (textPosition == "top" ) ? y + lineHeight : y - lineHeight;
	         else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                  y = (textPosition == "top" ) ? y - lineHeight : y + lineHeight;
	    
        }
		
	    else if (type == "waterfall") {
	        var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds,
                pointHeight = bounds.Height,
                pointWidth = bounds.Width;

	        var isNegativePoint = point.waterfallSum < 0 || (!point.showIntermediateSum && !point.showTotalSum && point.y < 0);
	       
	        switch (textPosition) {
	            case "bottom":
	                if (!series.isTransposed) {
	                    if (!series.yAxis.isInversed)
	                        y = y + pointHeight - margin.bottom - (isNegativePoint ? pointHeight : 0);
	                    else
	                        y = y - textOffset.height + margin.bottom + (isNegativePoint ? pointHeight : 0);
	                } else {
	                    if (!series.yAxis.isInversed)
	                        x = x - pointWidth + (lineHeight + (2 * textOffset.height) / 2) + (isNegativePoint ? pointWidth : 0);
	                    else
	                        x = x + pointWidth - (lineHeight + (2 * textOffset.height) / 2) - (isNegativePoint ? pointWidth : 0);
	                }
	                break;

	            case "middle":
	                if (!series.isTransposed) {
	                    if (!series.yAxis.isInversed)
	                        y = y + (pointHeight / 2 + (textOffset.height) / 4) - (isNegativePoint ? pointHeight : 0);
	                    else
	                        y = y - (pointHeight / 2) + (isNegativePoint ? pointHeight : 0);
	                } else {
	                    if (!series.yAxis.isInversed)
	                        x = x - (pointWidth / 2 + (textOffset.width) / 4) + (isNegativePoint ? pointWidth : 0);
	                    else
	                        x = x + (pointWidth / 2) - (isNegativePoint ? pointWidth : 0);
	                }
	                break;

	            default:
	                if (!series.isTransposed) {
	                    if (!series.yAxis.isInversed)
	                        y = y - (lineHeight + (2 * textOffset.height) / 2) - (isNegativePoint ? pointHeight : 0);
	                    else
	                        y = y + (lineHeight + (2 * textOffset.height) / 2) + (isNegativePoint ? pointHeight : 0);
	                } else {
	                    if (!series.yAxis.isInversed)
	                        x = x + (lineHeight + (2 * textOffset.height) / 2) + (isNegativePoint ? pointWidth : 0);
	                    else
	                        x = x - (lineHeight + (2 * textOffset.height) / 2) - (isNegativePoint ? pointWidth : 0);
	                }
	        }

	        if (horizontalTextAlignment && horizontalTextAlignment == "far")
	            x += pointWidth / 2;
	        else if (horizontalTextAlignment && horizontalTextAlignment == "near")
	            x -= pointWidth / 2;

	        if (verticalTextAlignment && verticalTextAlignment == "near") {
	            if (textPosition == "bottom")
	                y += textOffset.height / 3;
	            else
	                y += (textOffset.height + lineHeight);
	        }
	        else if (verticalTextAlignment && verticalTextAlignment == "far") {
	            if (textPosition == "bottom")
	                y -= textOffset.height / 3;
	            else
	                y -= (lineHeight + (textOffset.height) / 2);
	        }

	    }
        else if (type == "polar" || type == "radar") {
                y += textOffset.height / 4;
            if (textPosition == "bottom")
                y += (textOffset.height);
            else if (textPosition == "top")
                y -=  (textOffset.height);

            if (horizontalTextAlignment && horizontalTextAlignment == "far")
                x += textOffset.width;
            else if (horizontalTextAlignment && horizontalTextAlignment == "near")
                x -= (textOffset.width);

            if (verticalTextAlignment && verticalTextAlignment == "near")
                y += (textOffset.height)/2;
            else if (verticalTextAlignment && verticalTextAlignment == "far")
                y -= ((textOffset.height)/2);
        }
        else {
            if ((seriesType == "rangearea") && (point.high != point.y)) {
                if (this.chartObj.model.series.length == 1) {
                    var length = this.chartObj.model.chartRegions[seriesIndex][index].region.length;
                    y = this.chartObj.model.chartRegions[seriesIndex][index].region[length - 1].Y;
                    x = this.chartObj.model.chartRegions[seriesIndex][index].region[length - 1].X;
                }
                else {
                    var precount = 0;
                    for (var a = 0; a < seriesIndex; a++) {
                        var pretype = this.chartObj.model.series[a].type;
                        if (pretype != "rangearea")
                            precount = precount + this.chartObj.model.series[a].points.length;
                        else
                            precount = precount + 1;

                    }
                    var length = this.chartObj.model.chartRegions[precount][index].region.length;
                    y = this.chartObj.model.chartRegions[precount][index].region[length - 1].Y;
                    x = this.chartObj.model.chartRegions[precount][index].region[length - 1].X;
                }

                
            }    
            if (this.chartObj.model.requireInvertedAxes) {
                if (textPosition == "top")
                    x += lineHeight;
                else if (textPosition == "bottom")
                    x -= (lineHeight + (2 * textOffset.width) / 2);
            }
            else {
                if (textPosition == "bottom")
                    y += (2 * textOffset.height + lineHeight);
                else if (textPosition == "top")
                    y -= lineHeight;
            }
            if (horizontalTextAlignment && horizontalTextAlignment == "far")
                x += textOffset.width + lineHeight;
            else if (horizontalTextAlignment && horizontalTextAlignment == "near")
                x -= ((textOffset.width) + lineHeight);

            if (verticalTextAlignment && verticalTextAlignment == "near")
                y += lineHeight - padding;
            else if (verticalTextAlignment && verticalTextAlignment == "far")
                y -= lineHeight;
        }
        var location = { x: x, y: y };
        return location;
    },
	



    changeCrossHairSymbol: function(element, track, ptIndex, serIndex, series) {
        if (!ej.util.isNullOrUndefined(track)) {
            var trackMarker = {};
            element = $.extend(true, {}, series.marker, element);
            trackMarker = $.extend(true, {}, this.chartObj.model.crosshair.marker, trackMarker);
            var point = series.points[ptIndex], seriesType = series.type.toLowerCase();
            if (ej.util.isNullOrUndefined(trackMarker.shape)) {
                trackMarker.shape = element.shape;
                if (seriesType == 'bubble') {
                    trackMarker.border.color = ((point.border) && point.border.color) ? point.border.color : series.border.color;
                    trackMarker.border.width = ((point.border) && point.border.width) ? point.border.width : series.border.width;
                }
            }
            
            if ((!this.chartObj.model.crosshair.visible || ej.util.isNullOrUndefined(trackMarker.fill)) && series.type.toLowerCase() != "bubble" && series.type.toLowerCase() != "scatter") {
                if(trackMarker.shape.toLowerCase()=="cross" ||trackMarker.shape.toLowerCase()=="horizline"||trackMarker.shape.toLowerCase()=="vertline" )   
                    trackMarker.fill = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];         
                else
                    trackMarker.fill = element.border.color;
            }
            if (series.type.toLowerCase() == "bubble" && !this.chartObj.model.enableCanvasRendering) {
                var box = $(this.chartObj.gSeriesEle).find("#" + this.chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex)[0].getBoundingClientRect();
                trackMarker.size.width = (box.right - box.left) / 2 + 15;
                trackMarker.size.height = (box.bottom - box.top) / 2 + 15;
            } else {
                if (trackMarker.size.width <= element.size.width)
                    trackMarker.size.width = element.size.width + 2;
                if (trackMarker.size.height <= element.size.height)
                    trackMarker.size.height = element.size.height + 2;
               
                if (ej.util.isNullOrUndefined(trackMarker.border.color) || !this.chartObj.model.crosshair.visible){              
                 if (!ej.util.isNullOrUndefined(element.fill))
                    trackMarker.border.color = jQuery.type(element.fill) == "array" ? element.fill._gradientStop[0].color : element.fill;
                 else if (seriesType == 'scatter') {
                     trackMarker.border.color = ((point.border) && point.border.color) ? point.border.color : series.border.color;
                     trackMarker.border.width = ((point.border) && point.border.width) ? point.border.width : series.border.width;
                 }
                else
                  trackMarker.border.color = (point.fill) ? point.fill : (jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex]);
                }
            }
        }
        series._trackMarker = trackMarker;
        return trackMarker;


    },
	
    dataLabelSymbol: function (seriesIndex, series, pointIndex, x, y, width, height, symbolName, sender, group) {
        var chartObj = sender;
        if (symbolName == 'None')
            return;
        var point = chartObj.model._isPieOfPie ? ej.ejChart._getPieOfPiePoint(pointIndex, series) : series._visiblePoints[pointIndex];
            var location = { startX: x, startY: y };
            if(!sender.model.enable3D)
                var symbolEle = group ? group : chartObj.gDataLabelEle[seriesIndex];
				symbolEle = chartObj.model.enableCanvasRendering ? null : symbolEle;
            var element = $.extend(true, {}, series.marker, point.marker);
            var gradientName = "symbol";
            var style = { 'interior': element.dataLabel.fill, 'opacity': element.dataLabel.opacity, 'borderColor': element.dataLabel.border.color, 'borderWidth': element.dataLabel.border.width };
            var options = {
                'style': style,
                'gradientName': gradientName,
                'symbolEle': symbolEle,
                'width': width,
                'height': height,
                'point': point,
                'location': location,
                'seriesIndex': seriesIndex,
                'pointIndex': pointIndex,
                'element': element,
                'symbolName': symbolName,
                'id': chartObj.svgObject.id + '_dataLabel_series'+ seriesIndex + "_" + pointIndex
            };
            this.drawSymbolStyle(options, chartObj);
    },

    drawSymbolStyle: function (options, sender) {
        var chartObj = sender;
        var borderColor, borderWidth, opacity;
        var style = options.style, gradientName = options.gradientName, symbolEle = options.symbolEle,
            width = options.width, dashArray = options.dashArray;
        var height = options.height, point = options.point, location = options.location,
            seriesIndex = options.seriesIndex, visibility = options.visibility;
        var pointIndex = options.pointIndex, element = options.element, symbolName = options.symbolName;
        var trackSymbol = options.trackSymbol;
        var isCanvas = chartObj.model.enableCanvasRendering;
        var isPolar = chartObj.model.AreaType == 'polaraxes';
        var series = chartObj.model.series[seriesIndex];

        if (style != null) {
            if (style.borderColor != "")
                borderColor = style.borderColor ? style.borderColor: "transparent";

            opacity = style.opacity;
            borderWidth = style.borderWidth < 0 ? 0 : style.borderWidth;
        }
        var colors = null;
        if (style.interior) {
            colors = ej.util.isNullOrUndefined(style.interior._gradientStop) ? style.interior : style.interior._gradientStop;
        }
        else {       
            if (point.fill)
                colors = jQuery.type(point.fill) == "array" ? point.fill[0].color : point.fill;
            else {
                var pointColors = chartObj.model.pointColors;
                colors = (chartObj.model.AreaType != "none") ? ((series.type.toLowerCase() == "waterfall" && series.positiveFill && (point.y > 0 || point.waterfallSum > 0)) ? series.positiveFill :
                    chartObj.model.seriesColors[seriesIndex]) : jQuery.type(pointColors[pointIndex]) == "array" ? pointColors[pointIndex][0].color : pointColors[pointIndex];
            }
        }
       chartObj.symbolColorName = chartObj.svgRenderer.createGradientElement(gradientName + seriesIndex, colors, 0, 0, 0, $(chartObj.svgObject).height(), symbolEle);

       var symbolStyle = {
           dashArray: options.dashArray,
           ShapeSize: {
               width: width,
               height: height
           },
           Style: {
               BorderColor: borderColor,
               BorderWidth: borderWidth,
               Opacity: opacity,
               Visibility: visibility,
               Color: chartObj.symbolColorName
           },
           PointIndex: pointIndex,
           SeriesIndex: seriesIndex,
           Imageurl: element.imageUrl,
           Image: element.image,
           ID: options.id
       };

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { location: location, style: symbolStyle };
        chartObj._trigger("symbolRendering", commonEventArgs);
        if (!commonEventArgs.cancel) {
            if (isCanvas){
				//Added for Marker location while setting plotoffset value to axis
                  location.startY = (!chartObj.model.requireInvertedAxes) ? (location.startY + (series.yAxis.y ? (series.yAxis.y - chartObj.canvasY) : 0)) : location.startY;
                  location.startX = (!chartObj.model.requireInvertedAxes) ? (location.startX + (series.xAxis.x ? (series.xAxis.x - chartObj.canvasX) : 0)) : location.startX;
              
                commonEventArgs.data.location.startY = location.startY;
                if (ej.util.isNullOrUndefined(trackSymbol)) { // to draw marker/datalabel
				    if (chartObj.model.AreaType == "cartesianaxes") {
					    symbolEle = chartObj.svgRenderer.ctx;
					    chartObj.svgRenderer.ctx.save();
				        chartObj.svgRenderer.ctx.beginPath();
                        if (!chartObj.model.requireInvertedAxes)
                            chartObj.svgRenderer.ctx.rect(series.xAxis.x, series.yAxis.y, series.xAxis.width, series.yAxis.height);
					    else
                            chartObj.svgRenderer.ctx.rect(series.yAxis.x, series.xAxis.y, series.yAxis.width, series.xAxis.height);
                        chartObj.svgRenderer.ctx.clip();
					    ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, symbolEle);
					    chartObj.svgRenderer.ctx.restore();
				    }
				    else{
				        if (isCanvas && isPolar) {
				            chartObj.svgRenderer.ctx.save();
				            chartObj.svgRenderer.ctx.beginPath();
				            chartObj.svgRenderer.ctx.arc(chartObj.model.centerX, chartObj.model.centerY, chartObj.model.Radius, 0, 2 * Math.PI, false);
				            chartObj.svgRenderer.ctx.clip();
				        }
				        ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, symbolEle);
				        if (isCanvas && isPolar)
				            chartObj.svgRenderer.ctx.restore();
                    }
                   
                }
                else {
                        var ptIndex = commonEventArgs.data.style.PointIndex;
                        var serIndex = commonEventArgs.data.style.SeriesIndex;
                        if (!chartObj.model.series[serIndex].highlightSettings.enable) {
                            if ($('#' +chartObj._id+'_selection_'+serIndex+'_'+ptIndex+'_canvas').length == 0)
                                chartObj.svgRenderer.trackSymbol(commonEventArgs.data.style, commonEventArgs.data.location, symbolName, trackSymbol, chartObj);
                        }
                    }
                }
                else {
                    var ptIndex = commonEventArgs.data.style.PointIndex;
                    var serIndex = commonEventArgs.data.style.SeriesIndex;
                    var type = series.type;
                    var className;
                    if (series.highlightSettings.enable && trackSymbol && !chartObj.model.crosshair.visible) {
                        var style = $.extend(true, commonEventArgs.data.style);
                        var highlight =series.highlightSettings;
                        style.Style.Color = (highlight.pattern.toLowerCase() == "none" || highlight.pattern == "") ? (highlight.color != "" ? highlight.color : style.Style.Color) : "url(#" + highlight.pattern.toLowerCase() + '_Highlight_2D_' + serIndex + ")";
                        style.Style.Opacity = highlight.opacity;
                        style.Style.BorderColor = highlight.border.color;
                        style.Style.BorderWidth = highlight.border.width;
                        // checked condition for is already selected or not
                        if(type == 'bubble')
                            className = $('#' + this.chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex).attr('class');
                        else
                            className = $('#' + this.chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex+'_symbol').attr('class');
                        if (className != "SelectionStyleseries" + serIndex)
                            ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, style, chartObj, (chartObj.model.enable3D) ? sender.chart3D : symbolEle);
                    }
                    else {
                        if (type == 'scatter')
                            className = $('#' + chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex + '_symbol').attr('class');
                        else
                            className = $('#' + chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex).attr('class');
                        if (className != "SelectionStyleseries" + serIndex)
                            ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, (chartObj.model.enable3D) ? sender.chart3D : symbolEle);
                    }
                }
            }

        if (!sender.chart3D && type!="pieofpie" && chartObj.model.AreaType == "none" && $(symbolEle).children().not("defs").length > chartObj.model._visibleSeries[seriesIndex]._visiblePoints.length) {
            $(symbolEle.childNodes[options.pointIndex]).replaceWith($(symbolEle.childNodes[$(symbolEle.childNodes).length - 1]));
        }

    },
	
    drawSymbol: function (seriesIndex,series, pointIndex, x, y,chart,trackSymbol,trackcount) {

        if (ej.util.isNullOrUndefined(this.chartObj)) this.chartObj = chart;


        var point = series._visiblePoints[pointIndex], seriesType = series.type.toLowerCase(),
        index = seriesType == "boxandwhisker" ? trackcount : pointIndex,
        dashArray = (seriesType == 'bubble' || seriesType == 'scatter') ? series.border.dashArray : "";
             var symbolName = "None";
             var tracker = this.changeCrossHairSymbol(point.marker ? point.marker : series.marker, trackSymbol, pointIndex, seriesIndex, series);
             if(seriesType.toLowerCase() == 'boxandwhisker')
                var element = tracker ? tracker : $.extend(true, {}, series.outlierSettings, point.outlierSettings,series.border,point.border);				
             else
                var element = tracker ? tracker : $.extend(true, {}, series.marker, point.marker);

             $.each(this.chartObj.model.symbolShape, function(name) {
                 if (element.shape.toLowerCase() == name.toLowerCase())
                     symbolName = name;
             });
                if (symbolName == "None")
                    return;
                var location = this.chartObj.model.AreaType == "cartesianaxes" ? { startX: x + this.chartObj.canvasX, startY: y + this.chartObj.canvasY } : { startX: x, startY: y };
				var borderColor,borderWidth,style;
                if (seriesType == 'scatter') {
                    borderColor = ((point.border) && point.border.color) ? point.border.color : series.border.color;
                    borderWidth = ((point.border) && point.border.width) ? point.border.width : series.border.width;
                    style = { 'interior': element.fill, 'opacity': element.opacity, 'borderColor': borderColor, 'borderWidth': borderWidth };
                }
				else if(seriesType == "boxandwhisker")
				{
                     borderColor = ((point.border) && point.border.color) ? (point.border.color == "transparent" ? this.chartObj._saturationColor(series.fill,-0.6) : point.border.color ) : (series.border.color == "transparent" ? this.chartObj._saturationColor(series.fill,-0.6) : series.border.color ) ;
                     borderWidth = ((point.border) && point.border.width) ? point.border.width : series.border.width;
                     style = { 'interior': series.fill, 'opacity': series.opacity, 'borderColor': borderColor, 'borderWidth': borderWidth };
				}
                else {
                    var style = { 'interior': element.fill, 'opacity': element.opacity, 'borderColor': element.border.color, 'borderWidth': element.border.width };
                }
                var symbolEle = (trackSymbol) ? this.chartObj.gTrackerEle : this.chartObj.gSymbolGroupEle[seriesIndex];
                var id = (trackSymbol) ? (this.chartObj.svgObject.id + '_trackSymbol_' + seriesIndex + "_" + index) : (this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + index + '_symbol');
                var gradientName = trackSymbol ? "TrackSymbol" : "symbol";
                var width = element.size.width;
                var height = element.size.height;
                var options = {
                    'style': style,
                    'gradientName': gradientName,
                    'symbolEle': symbolEle,
                    'width': width,
                    'height': height,
                    'point': point,
                    'location': location,
                    'seriesIndex': seriesIndex,
                    'pointIndex': pointIndex,
                    'element': element,
                    'visibility': (point.visible) ? 'visible' : 'hidden',
                    'symbolName': symbolName,
                    'trackSymbol' : trackSymbol,
                    'id': id,
                    'dashArray':dashArray
                };
                this.drawSymbolStyle(options, this.chartObj, trackSymbol);

                if (series.type.toLowerCase() == "scatter") {
                    var areaBoundsX = series.xAxis.x;
                    var areaBoundsY = series.yAxis.y;
                    if (this.chartObj.model.enableCanvasRendering) {
                        areaBoundsX = 0;
                        areaBoundsY = 0;
                    }
                    var bounds = { X: areaBoundsX + location.startX - (height / 2), Y: areaBoundsY + location.startY - (width / 2), Height: height, Width: width };
                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, pointIndex);
                }
    },

    chartAreaType: "cartesianAxes",
    requireInvertedAxes: false,
    stackingSeries: false,
    hiloTypes:false
    
};


function ejExtendClass(parent, members) {
    var object = function () { };
    object.prototype = new parent();
    $.extend(object.prototype, members);
    return object;
}

ej.ejLineSeries = ejExtendClass(ej.EjSeriesRender);

ej.seriesTypes.line = ej.ejLineSeries;


ej.ejStepLineSeries = ejExtendClass(ej.EjSeriesRender, {

    draw: function (chart, options) {
        var chartObj = this.chartObj = chart;
        var lDirection;
		options.animationType = "path";
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setLineSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);
        var firstPoint = null;
        var startPath = "M";
        var secondPoint,nextpoint,point1,point2,point3;
        var firstIndex = -1;
        for (var i = 0; i < visiblePoints.length; i++) {
            secondPoint = visiblePoints[i];
            if (secondPoint.visible) {
                if (firstPoint != null) {
                    if (visiblePoints.length > firstIndex + 1) {
                        nextpoint = { xValue: visiblePoints[firstIndex + 1].xValue, YValues: [firstPoint.YValues[0]] };
                        point1 = ej.EjSvgRender.utils._getPoint(firstPoint, currentseries);
                        point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentseries);
                        point3 = ej.EjSvgRender.utils._getPoint(visiblePoints[firstIndex + 1], currentseries);
                        chartObj = this.chartObj;
                        sb.append(startPath + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " +
						"L" + " " + (point2.X + chartObj.canvasX) + " " + ((point2.Y + chartObj.canvasY)) + " " +
						"L" + " " + (point3.X + chartObj.canvasX) + " " + ((point3.Y + chartObj.canvasY)) + " ");
                    }
                    startPath = "L";
                }
                firstPoint = secondPoint;
                firstIndex = i;
            }
            else {
                firstPoint = null;
                startPath = "M";
            }
        }
        lDirection = sb.toString();

        this._drawLinePath(currentseries, style, lDirection);

        if(this.chartObj.dragPoint) 
            this.chartObj.svgRenderer.append(this.chartObj.gPreviewSeriesGroupEle, this.chartObj.gSeriesEle);
        else
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle); 
    }

});

ej.ejStepAreaSeries = ejExtendClass(ej.EjSeriesRender, {

    draw: function (chart, options, params) {
        this.chartObj = chart;
		options.animationType = "path";
		var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
        var aDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setAreaSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);
        var xOffset = 0;
        var firstPoint = null;
        var secondPoint;
        if (currentseries.xAxis._valueType.toLowerCase() == "category" && currentseries.xAxis.labelPlacement.toLowerCase() != "onticks")
            xOffset = 0.5;
        var origin = ej.EjSeriesRender.prototype.getOrigin(this, currentseries, params);
        var startPoint = null;
        var start = true;
        var chartObj = this.chartObj;
        for (var i = 0; i <= visiblePoints.length; i++) {

            if (i < visiblePoints.length) {
                if (visiblePoints[i].visible) {

                    secondPoint = { xValue: visiblePoints[i].xValue - xOffset, YValues: [visiblePoints[i].y] };
                    point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (!startPoint) {
                        startPoint = { xValue: visiblePoints[i].xValue - xOffset, YValues: [origin] }
                        var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentseries);
                        sb.append("M" + " " + (startLoc.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                    }
                    if (firstPoint != null) {
                        var step = { xValue: secondPoint.xValue, YValues: [firstPoint.YValues[0]] };
                        var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                        if (start) {
                            sb.append("L" + " " + (point2.X + chartObj.canvasX) + " " + (point2.Y + chartObj.canvasY) + " ");
                            start = false;
                        }
                        sb.append("L" + " " + (stepPoint.X + chartObj.canvasX) + " " + ((stepPoint.Y + chartObj.canvasY)) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " ");
                        if ((xOffset == 0 && i == visiblePoints.length - 1) || (i < (visiblePoints.length - 1) && !visiblePoints[i + 1].visible)) {
                            if (xOffset > 0) {
                                secondPoint = { xValue: visiblePoints[i].xValue + xOffset, YValues: [visiblePoints[i].y] };
                                var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                                 if (requireInvertedAxes)
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.Y + chartObj.canvasX) + " " + (startLoc.X + chartObj.canvasY) + " ");
                                 else
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                            }
                            else
                               if (requireInvertedAxes)
                                   sb.append("L" + " " + (startLoc.X + chartObj.canvasY) + " " + (point1.Y + chartObj.canvasX)+  " ");
                                else
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX)+ " " + (startLoc.Y + chartObj.canvasY) + " ");
                            startPoint = null;
                            firstPoint = null;
                            start = true;
                        }
                    }
                    if (startPoint) {
                        point2 = point1;
                        firstPoint = secondPoint;
                    }
                }
                else {
                    startPoint = null;
                    firstPoint = null;
                    start = true;
                }

              }
             else {
                    if  (visiblePoints[i-1].visible && xOffset > 0) {
                                secondPoint = { xValue: visiblePoints[i - 1].xValue + xOffset, YValues: [visiblePoints[i - 1].y] };
                                var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                               if (requireInvertedAxes)
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.Y + chartObj.canvasX) + " " + (startLoc.X + chartObj.canvasY) + " ");
                                else
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                       }                           
                }
            
         }
        aDirection = sb.toString();

        this.drawAreaPath(currentseries, style, aDirection);
        if (this.chartObj.dragPoint)
            this.chartObj.svgRenderer.append(this.chartObj.gPreviewSeriesGroupEle, this.chartObj.gSeriesEle);
        else
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

    }
});
ej.seriesTypes.steparea = ej.ejStepAreaSeries;


ej.seriesTypes.stepline = ej.ejStepLineSeries;


 ej.ejColumnSeries = ejExtendClass(ej.EjSeriesRender, {
   
  
      
     draw: function (chart,options, params)
      { 
         this.chartObj=chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
		 options.animationType = "rect";
		 if (options.columnFacet == "cylinder")
		     options.animationType = "cylinder";
         var series = options;
		 var cornerRadius = series.cornerRadius;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);         
         var origin = ej.EjSeriesRender.prototype.getOrigin(this, series, params);
        
         var sidebysideinfo = this.getSideBySideInfo(series, params);
		 if(series.dragSettings.enable) series.sidebysideInfo = sidebysideinfo;    
         var visiblePoints = this._isVisiblePoints(series);
              
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var pointMarker;
         
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex=i;
             var point = visiblePoints[i];
             pointMarker = visiblePoints[i].marker;
           
             var y1 = point.YValues[0];
             var y2 = origin;
             if (point.visible) {
             //calculate sides
             var data = cSer.calculateSides(point, sidebysideinfo);
             var x1 = data.x1;
             var x2 = data.x2;
             
			 var styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

			 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
			 
             //drawing part
             var xr = Math.min(0, rect.Width);
             var yr = Math.min(0, rect.Height);

             var  bounds;        
             if ((xr == 0 || yr == 0) && rect.Width>0) {                      
                 options = {
                     'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                     'x': rect.X + cSer.chartObj.canvasX,
                     'y': rect.Y + cSer.chartObj.canvasY,
                     'width': rect.Width,
                     'height': rect.Height,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'plot': y1 < 0 ? "negative" : "positive",
                     'opacity': styleOptions.opacity,
                     'stroke': styleOptions.borderColor,
                     'stroke-dasharray': styleOptions.dashArray,
                 };
                 var cylinderSeriesOption = {
                     'isColumn': true,
                     'stacking': false,
                     'isLastSeries': true,
                 };
                 if(series.columnFacet=="cylinder")               
                         cSer.chartObj.svgRenderer.drawCylinder(options, cSer.gSeriesGroupEle, cylinderSeriesOption);
				//calculate path for rounded corner
				 else if ((typeof(cornerRadius) != "object" && cornerRadius >0) || cornerRadius.topLeft >0 || cornerRadius.bottomLeft >0 || cornerRadius.topRight >0 
				 || cornerRadius.bottomRight >0)
				 {
				     var roundrect = ej.EjSvgRender.utils._calculateroundedCorner(cornerRadius, options);
				     options.d = roundrect;
					 series.animationType = "path";
				 	 cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
				 }
                 else if (series.columnFacet == "rectangle")
                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);
                         var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
            
                  if (requireInvertedAxes)
                      point.symbolLocation = { X: ((y1 < (series.xAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.xAxis._crossValue || 0))) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                  else
                      point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < (series.xAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.xAxis._crossValue ||0))) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
   
   isRegion:true

 });
 ej.seriesTypes.column = ej.ejColumnSeries;

 ej.ejStackingColumnSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options, params) {
         this.chartObj = chart;
		 options.animationType = "rect";
         if (options.columnFacet == "cylinder")
             options.animationType = "cylinder";
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
		 var cornerRadius = series.cornerRadius;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         //var origin = Math.max(options.yAxis.visibleRange.min, 0);
        
         var sidebysideinfo = this.getSideBySideInfo(series, params);
         var visiblePoints = this._isVisiblePoints(series);

         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = series._visiblePoints[i];

             var y2 = series.stackedValue.StartValues[i];
             var y1= series.stackedValue.EndValues[i];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
             
                 //drawing part
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 var bounds;
                 var isstacking100 = series.type == 'stackingcolumn100' ? true : false;
                 if ((xr == 0 || yr == 0) && rect.Width > 0) {
                     options = {
                     'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                     'x': rect.X + cSer.chartObj.canvasX,
                     'y': rect.Y + cSer.chartObj.canvasY,
                     'width': rect.Width,
                     'height': rect.Height,
                     'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                         'plot': y1 < 0 ? "negative" : "positive",
                         'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor,
                         'stroke-dasharray': styleOptions.dashArray,
                     };
                     var cylinderSeriesOption = {
                         'isColumn': true,
                         'stacking': isstacking100,
                         'isLastSeries': series.stackedValue.stackedSeries,
                     };
                     if (series.columnFacet == "cylinder")
                         cSer.chartObj.svgRenderer.drawCylinder(options, cSer.gSeriesGroupEle, cylinderSeriesOption);
					 else if ((typeof(cornerRadius) != "object" && cornerRadius >0) || cornerRadius.topLeft >0 || cornerRadius.bottomLeft >0 
					          || cornerRadius.topRight >0 || cornerRadius.bottomRight >0)           //calculate path for rounded corner
					{
					     var roundrect = ej.EjSvgRender.utils._calculateroundedCorner(cornerRadius, options);
						options.d = roundrect;
						series.animationType = "path";	
						cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
				    }
                     else if (series.columnFacet == "rectangle")
                         cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                
                 if (requireInvertedAxes)
                     point.symbolLocation = { X: ((y1 < (series.xAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.xAxis._crossValue || 0))) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                 else
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < (series.xAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.xAxis._crossValue || 0))) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        

     },

     stackingSeries: true,
	  isRegion:true

 });
 ej.seriesTypes.stackingcolumn = ej.ejStackingColumnSeries;
 
 ej.seriesTypes.stackingcolumn100 = ej.ejStackingColumnSeries;

 ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

 ej.ejRangeColumnSeries = ejExtendClass(ej.EjSeriesRender, {
   
     draw: function (chart, options, params) {
         this.chartObj = chart;
		 options.animationType = "rect";
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
		 var cornerRadius = series.cornerRadius;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
      
         var sidebysideinfo = this.getSideBySideInfo(series, params);
         var visiblePoints = this._isVisiblePoints(series);

         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];

             var y1 = point.YValues[0];
             var y2 = point.YValues[1];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                  
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);
				  
                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
                 var width=cSer.chartObj.model.m_AreaBounds.Width;
                 //drawing part
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 var bounds;
                 if (xr == 0 || yr == 0) {
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                         'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor,
                         'stroke-dasharray': styleOptions.dashArray,
                     };
					 
					 if ((typeof(cornerRadius) != "object" && cornerRadius >0) || cornerRadius.topLeft >0 || cornerRadius.bottomLeft >0 
					    || cornerRadius.topRight >0 || cornerRadius.bottomRight >0)           //calculate path for rounded corner
					 {
					     var roundrect = ej.EjSvgRender.utils._calculateroundedCorner(cornerRadius, options);
						options.d = roundrect;
						series.animationType = "path";
						cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
					 }
					 else
						cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle, trans.y);

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle, trans.y);

                     //Add region for each rect
                     var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                 //Add rect location details for symbol                
                 if (series.type.toLowerCase() == "waterfall")
                     y1 = point.waterfallSum ? point.waterfallSum : point.y;              
                 if (requireInvertedAxes)
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                 else
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

      

     },
	 
     hiloTypes: true,
	  isRegion:true

 });
 ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;
 
 
  ej.ejBoxandwhiskerSeries = ejExtendClass(ej.EjSeriesRender, {
 
 draw: function (chart,options, params)
      { 
         this.chartObj=chart;
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes,previousStyle;
         options.animationType = "rect"; 
         var series = options,
         seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries),
         origin = ej.EjSeriesRender.prototype.getOrigin(this, series, params),
         sidebysideinfo = this.getSideBySideInfo(series, params),
         visiblePoints = this._isVisiblePoints(series),
         trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes),
         pointMarker,trackCount=0,
         serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
            var pointIndex=i,list1,list2,count =0,y1,y2,data,x1,x2,styleOptions,rect,xr,yr,seriesIndex,sb,newPoint,boxplotmidpoint,
            bounds,options1,whiskerpath,newPoint1,outlierpoint,svgXyOutlier,boundsOutlier,svgXy,
            point = visiblePoints[i];
            pointMarker = visiblePoints[i].marker;
            list1 = point.YValues[0];
            list2 = cSer.calculateMean(list1,series,pointIndex);
            y1=list2.UpperQuartile;
            y2=list2.LowerQuartile;
            if (point.visible) {
                //calculate sides
                data = cSer.calculateSides(point, sidebysideinfo);
                x1 = data.x1;
                x2 = data.x2;
                styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex); 
                rect=  cSer.getRectangle(x1, y1, x2, y2, series, chart);
                //drawing part
                xr = Math.min(0, rect.Width);
                yr = Math.min(0, rect.Height);
                seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
                previousStyle = this.chartObj.setStyle(this, series, seriesIndex, i);
                sb = ej.EjSvgRender.utils._getStringBuilder();
                newPoint = {"xValue": point.xValue, "YValues": list2.midvalue };
                boxplotmidpoint =  ej.EjSvgRender.utils._getPoint(newPoint, series);
				var boxStrokeColor = styleOptions.borderColor == "transparent" ? this.chartObj._saturationColor(styleOptions.interior,-0.6):styleOptions.borderColor;

             bounds;        
             if ((xr == 0 || yr == 0) && rect.Width>0) {
                 options = {
                     'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                     'x': rect.X + cSer.chartObj.canvasX,
                     'y': rect.Y + cSer.chartObj.canvasY,
                     'width': rect.Width,
                     'height': rect.Height,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'plot': y1 < 0 ? "negative" : "positive",
                     'opacity': styleOptions.opacity,
                     'stroke': boxStrokeColor,
                     'stroke-dasharray': styleOptions.dashArray,
                 };
				 
				 options1 = {
                     'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex + '_median',
                     x1: series._isTransposed ? boxplotmidpoint.X + cSer.chartObj.canvasX : rect.X +  cSer.chartObj.canvasX ,
                     y1: series._isTransposed ? rect.Y + cSer.chartObj.canvasY : boxplotmidpoint.Y + cSer.chartObj.canvasY,
                     x2: series._isTransposed ? boxplotmidpoint.X + cSer.chartObj.canvasX : rect.X + rect.Width + cSer.chartObj.canvasX  ,                  
                     y2: series._isTransposed ? rect.Y + rect.Height + cSer.chartObj.canvasY : boxplotmidpoint.Y + cSer.chartObj.canvasY ,
                     'stroke-width': styleOptions.borderWidth,
                     'stroke': boxStrokeColor,
				};
				
                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);
					 					 					 
					 //draw mean line for boxandwhisker
					 cSer.chartObj.svgRenderer.drawLine(options1, cSer.gSeriesGroupEle);
					 //draw lowerwhisker path calculation
					 whiskerpath = cSer.calculateBoxAndWhiskerPath(options,rect,options1,list2,series,point,pointIndex);					
					 options.d = whiskerpath.lowerWhisker;
  				     options.id = options.id + '_whisker';
					 cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
					 //draw upperwhisker path calculation
					 options.d = whiskerpath.upperWhisker;
					 cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
					 
					 if(series.showMedian == true)
					 {
						options.d = whiskerpath.mean;
						cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
					 }
					 
										 
					 //draw outliers
					for(var k=0;k<list2.outliers.length;k++)
					{
					    newPoint1 = {"xValue": point.xValue, "YValues": list2.outliers[k] };
					    outlierpoint = ej.EjSvgRender.utils._getPoint(newPoint1, series);
					   

					    outlierpoint.X = series._isTransposed ?  outlierpoint.X :(rect.X + (rect.Width / 2)) ;
					    outlierpoint.Y = series._isTransposed ? rect.Y + rect.Height / 2 : outlierpoint.Y;
						
						this.drawSymbol(seriesIndex, series, pointIndex, outlierpoint.X, outlierpoint.Y,null,null,trackCount);							
						trackCount++;
						//to store the outlier regions
						svgXyOutlier = ej.EjSvgRender.utils._getSvgXY(outlierpoint.X, outlierpoint.Y, series, cSer.chartObj);
						boundsOutlier = { X: svgXyOutlier.X - (series.outlierSettings.size.width/2 )   , Y: svgXyOutlier.Y - (series.outlierSettings.size.height/2) ,Width: series.outlierSettings.size.width ,Height: series.outlierSettings.size.height};
						ej.EjSvgRender.utils._addRegion(cSer.chartObj,boundsOutlier, series, point, pointIndex);
					
					}
                    svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                    bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                    ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
   
   isRegion:true
 
 
 
 });
 ej.seriesTypes.boxandwhisker = ej.ejBoxandwhiskerSeries;

 ej.ejWaterfallSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options, params) {

         ej.ejRangeColumnSeries.prototype.draw.call(this, chart, options, params);
         options.animationType = "rect";
         var chartRegions = this.chartObj.model.chartRegions;
         var seriesRegions = [];
         var areaBounds = this.chartObj.model.m_AreaBounds;
         var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
         options.index = seriesIndex;

         for (var j = 0, i = 0; j < chartRegions.length; j++) {
             if (seriesIndex == chartRegions[j].SeriesIndex) {
                 seriesRegions[i] = chartRegions[j];
                 i += 1;
             }
         }
         var serOptions1 = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_waterfallLine_' + seriesIndex };

         this.connectorLineGroup = this.chartObj.svgRenderer.createGroup(serOptions1);

         var sb, point1, point2, y, y1, t1, t2, lineOptions;
         for (var k = 0; k < seriesRegions.length - 1; k++) {
             sb = ej.EjSvgRender.utils._getStringBuilder();
             point1 = seriesRegions[k].Region.Bounds;
             point2 = seriesRegions[k + 1].Region.Bounds;

             if (!options.isTransposed) {
                 y1 = Math.ceil(point1.Y), h1 = Math.ceil(point1.Height), y2 = Math.ceil(point2.Y), h2 = Math.ceil(point2.Height);
                 t1 = Math.ceil(point1.Y + point1.Height);
                 y = ((y1 == y2) ? point1.Y : (t1 == y2 ? point2.Y : (point2.Y + point2.Height)));
                 sb.append("M " + (point1.X) + " " + (y) + " L " + (point2.X + point2.Width )+ " " + y);
             } else {
                 x1 = Math.ceil(point1.X), w1 = Math.ceil(point1.Width), x2 = Math.ceil(point2.X), w2 = Math.ceil(point2.Width);
                 t1 = Math.ceil(point1.X + point1.Width);
                 t2 = Math.ceil(point2.X + point2.Width);
                 x = ((t1 == t2) ? (point1.X + point1.Width) : (x1 == x2 ? point1.X : (x1 == t2 ? point1.X: (point1.X + point1.Width))));
                 sb.append("M " + (x) + " " + (point1.Y + point1.Height) + " L " + x + " " + (point2.Y));
             }

             
             lDirection = sb.toString();
             if (lDirection != "" && point1.Height >= 0) {
                 lineOptions = {
                     'id': this.chartObj.svgObject.id + "_waterFall_" + seriesIndex + "_connectorLine_" + k,
                     'fill': 'none',
                     'stroke-dasharray': options.connectorLine.dashArray,
                     'stroke-width': options.connectorLine.width,
                     'stroke': options.connectorLine.color,
                     'opacity': options.connectorLine.opacity,
                     'd': lDirection
                 };
                 this.chartObj.svgRenderer.drawPath(lineOptions, this.connectorLineGroup);
             }
         }
         this.chartObj.svgRenderer.append(this.connectorLineGroup, this.chartObj.gSeriesEle);

         if (options.enableAnimation && !options._animatedSeries)
             this.chartObj.svgRenderer._setAttr(this.connectorLineGroup, { "visibility": "hidden" });
     },

     hiloTypes: false,
     isRegion: true
 });
 ej.seriesTypes.waterfall = ej.ejWaterfallSeries;

 ej.ejStackingBarSeries = ejExtendClass(ej.EjSeriesRender, {
     
     draw: function (chart, options, params) {
         this.chartObj = chart;
		 options.animationType = "rect";
         if (options.columnFacet == "cylinder")
             options.animationType = "cylinder";
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
		 var cornerRadius = series.cornerRadius;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         //var origin = Math.max(options.yAxis.visibleRange.min, 0);
         var sidebysideinfo = this.getSideBySideInfo(series, params);
         var visiblePoints = this._isVisiblePoints(series);
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];
             var y1 = series.stackedValue.EndValues[i];
             var y2 =series.stackedValue.StartValues[i];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);
                 var isstacking100 = series.type == 'stackingbar100' ? true : false;
                 if (xr == 0 || yr == 0) {
                     var bounds;
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
						 'opacity': styleOptions.opacity,
						 'stroke': styleOptions.borderColor,
						 'stroke-dasharray': styleOptions.dashArray,
                     };
                     var cylinderSeriesOption = {
                         'isColumn': false,
                         'stacking': isstacking100,
                         'isLastSeries': series.stackedValue.stackedSeries,
                     };
                     if (series.columnFacet == "cylinder")
                         cSer.chartObj.svgRenderer.drawCylinder(options, cSer.gSeriesGroupEle, cylinderSeriesOption);
					 else if ((typeof(cornerRadius) != "object" && cornerRadius >0) || cornerRadius.topLeft >0 || cornerRadius.bottomLeft >0 
					        || cornerRadius.topRight >0 || cornerRadius.bottomRight >0)           //calculate path for rounded corner
					{
					     var roundrect = ej.EjSvgRender.utils._calculateroundedCorner(cornerRadius, options);
						options.d = roundrect;
						series.animationType = "path";
						cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
					}
                     else if (series.columnFacet == "rectangle")
                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }

               
                 if (!requireInvertedAxes)
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < (series.yAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.yAxis._crossValue || 0))) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 else
                     point.symbolLocation = { X: ((y1 < (series.yAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.yAxis._crossValue || 0))) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
               
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
 

     },
	 
     stackingSeries: true,
     requireInvertedAxes: true,
	  isRegion:true

 });
 ej.seriesTypes.stackingbar = ej.ejStackingBarSeries;

 ej.seriesTypes.stackingbar100 = ej.ejStackingBarSeries;

 ej.ejBarSeries = ejExtendClass(ej.EjSeriesRender, {
    
     draw: function (chart, options, params) {
         this.chartObj = chart;
		 options.animationType = "rect";
         if (options.columnFacet == "cylinder")
             options.animationType = "cylinder";
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
		 var cornerRadius = series.cornerRadius;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         var origin = ej.EjSeriesRender.prototype.getOrigin(this, series, params);
         var style = $.extend(true, {}, this.chartObj.model.seriesStyle, series.style);
         var sidebysideinfo = this.getSideBySideInfo(series, params);
         if (series.dragSettings.enable) series.sidebysideInfo = sidebysideinfo;
         var visiblePoints = this._isVisiblePoints(series);
         var colors;
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
         
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];           
             var y1 = point.YValues[0];
             var y2 = origin;
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
                
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                if (xr == 0 || yr == 0) {
                     var bounds;          
                         options = {
                             'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + pointIndex,
                             'x': rect.X + cSer.chartObj.canvasX,
                             'y': rect.Y + cSer.chartObj.canvasY,
                             'width': rect.Width,
                             'height': rect.Height,
                             'fill': styleOptions.interior,
                             'stroke-width': styleOptions.borderWidth,
                             'plot': y1 < 0 ? "negative" : "positive",
                             'opacity': styleOptions.opacity,
                             'stroke': styleOptions.borderColor,
                             'stroke-dasharray': styleOptions.dashArray,
                         };
                         var cylinderSeriesOption = {
                             'isColumn': false,
                             'stacking': false,
                             'isLastSeries': true,
                         };
                         if (series.columnFacet == "cylinder")
                             cSer.chartObj.svgRenderer.drawCylinder(options, cSer.gSeriesGroupEle, cylinderSeriesOption);
						 else if ((typeof(cornerRadius) != "object" && cornerRadius >0) || cornerRadius.topLeft >0 || cornerRadius.bottomLeft >0 
					             || cornerRadius.topRight >0 || cornerRadius.bottomRight >0)           //calculate path for rounded corner
						 {
						     var roundrect = ej.EjSvgRender.utils._calculateroundedCorner(cornerRadius, options);
							options.d = roundrect;
							series.animationType = "path";
							cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
						 }
                         else if (series.columnFacet == "rectangle")
                         cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                         var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }


                 
                  if (!requireInvertedAxes)
                      point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < (series.xAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.xAxis._crossValue || 0))) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                else
                    point.symbolLocation = { X: ((y1 < (series.xAxis._crossValue || 0) && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > (series.xAxis._crossValue || 0))) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
	 
     requireInvertedAxes: true,
	  isRegion:true
 });
 ej.seriesTypes.bar = ej.ejBarSeries;


 ej.ejStackingAreaSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {
         this.chartObj = chart;
		 options.animationType = "path";
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);

         



         var visiblePoints = this._isVisiblePoints(currentSeries);
     
         var origin = Math.max(options.yAxis.visibleRange.min, currentSeries.stackedValue.StartValues[0]);
         var canvasX = this.chartObj.canvasX;
         var canvasY = this.chartObj.canvasY;

         var startPoint = { xValue: visiblePoints[0].xValue, YValues: [origin] };
         var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries), startValue = 0, endLength = currentSeries.stackedValue.EndValues.length;
         areasb.append("M" + " " + (startLoc.X + canvasX) + " " + (startLoc.Y + canvasY) + " ");
         
         //var index = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
         
         for (j = 0; j <= endLength; j++) {
             if (j != endLength && visiblePoints[j].visible) {                   
                    currentPoint = { xValue: visiblePoints[j].xValue, YValues: [currentSeries.stackedValue.EndValues[j]] };                       
                    if (visiblePoints[j].visible) {
                         point1 = visiblePoints[j].location = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);                          
                         areasb.append("L" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " ");
                         visiblePoints[j].YValues[0] = currentPoint.YValues[0];
                     }                    
                
             }
             else {
                 origin = currentSeries.stackedValue.StartValues[j + 1];
                 for (var i = j - 1; i >= startValue; i--) {
                     currentPoint = { xValue: visiblePoints[i].xValue, YValues: [currentSeries.stackedValue.StartValues[i]] };
                     point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                     areasb.append("L" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " ");
                 }
                 if (visiblePoints[j + 1] && visiblePoints[j + 1].visible) {
                     var startPoint = { xValue: visiblePoints[j + 1].xValue, YValues: [currentSeries.stackedValue.StartValues[j + 1]] };
                     var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                     areasb.append("M" + " " + (startLoc.X + canvasX) + " " + (startLoc.Y + canvasY) + " ");                    
                 }
                  startValue = j + 1;
             }
         }            
              
      
                

         aDirection = areasb.toString();

         this.drawAreaPath(currentSeries, style, aDirection);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

          


     },
	 
     stackingSeries:true

 });
 ej.seriesTypes.stackingarea = ej.ejStackingAreaSeries;

 ej.seriesTypes.stackingarea100 = ej.ejStackingAreaSeries;

 ej.ejAreaSeries = ejExtendClass(ej.EjSeriesRender, {

  
     draw: function (chart,options, params) {
         this.chartObj = chart;
         var aDirection;
		 options.animationType = "path";
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);
         var chartObj = this.chartObj;
         if (currentSeries.sorting)
         	currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("xValue")).executeLocal();
      
         var visiblePoints = this.chartObj.dragPoint? currentSeries.pointCollection : this._isVisiblePoints(currentSeries);
         var origin = ej.EjSeriesRender.prototype.getOrigin(this, currentSeries, params);
         
         var startPoint;
         var translate = [];
         translate[0] = chartObj.canvasX;
         translate[1] = chartObj.canvasY;
         if(visiblePoints.length>0) { 
         for (var i = 0; i < visiblePoints.length; i++) {
             if (visiblePoints[i].visible) {
                 if (visiblePoints.length > i + 1) {
                     if (!startPoint) {
                         startPoint = { xValue: visiblePoints[i].xValue, YValues: [origin] }
                         var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("M" + " " + (point1.X) + " " + ((point1.Y)) + " ");
                     }


                     var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);

                     areasb.append("L" + " " + (point1.X) + " " + ((point1.Y)) + " ");

                     if (!visiblePoints[i + 1].visible) {
                         point = { xValue: visiblePoints[i].xValue, YValues: [origin] }
                         var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                         var point3 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("L" + " " + (point2.X) + " " + ((point2.Y)) + " " + "L" + " " + (point3.X) + " " + ((point3.Y)) + " ");
                         startPoint = null;
                     }
                 }
                 else {
                     if (visiblePoints[i - 1] && visiblePoints[i - 1].visible) {
                         var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                         areasb.append("L" + " " + (point1.X) + " " + ((point1.Y)) + " ");
                     }
                 }
             }         
         }

         var endPoint = { xValue: visiblePoints[visiblePoints.length - 1].xValue, YValues: [origin] };
         var endLoc = ej.EjSvgRender.utils._getPoint(endPoint, currentSeries);
         if (visiblePoints.length > 1)
             areasb.append("L" + " " + (endLoc.X) + " " + ((endLoc.Y)) + " ");
         
         aDirection = areasb.toString();

         this.drawAreaPath(currentSeries, style, aDirection, translate);
         }
         if (this.chartObj.dragPoint)
             this.chartObj.svgRenderer.append(this.chartObj.gPreviewSeriesGroupEle, this.chartObj.gSeriesEle);
         else
            this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     
       

     }

 });

 ej.seriesTypes.area = ej.ejAreaSeries;

 ej.ejRangeAreaSeries = ejExtendClass(ej.EjSeriesRender, {


     draw: function (chart, options) {
         this.chartObj = chart;
         var aDirection;
		 options.animationType = "path";
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);
         var seriesIndex = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("xValue")).executeLocal();

         var visiblePoints = this._isVisiblePoints(currentSeries);        
         var translate = [];
         translate[0] = this.chartObj.canvasX;
         translate[1] = this.chartObj.canvasY;
         var startPoint;
         var internalRegion=[]
         for (var i = 0; i < visiblePoints.length; i++) {
             internalRegion[i] = {Region:{PointIndex:i}, SeriesIndex:seriesIndex, region:[]};
             if (visiblePoints[i].visible) {     
                     if (!startPoint) {
                         var origin = visiblePoints[i].low;
                         startPoint = { xValue: visiblePoints[i].xValue, YValues: [origin] }
                         var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("M" + " " + (point1.X) + " " + ((point1.Y)) + " ");
                     }
                 var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                 areasb.append("L" + " " + (point1.X) + " " + ((point1.Y)) + " ");
                 internalRegion[i].region.push({ X: point1.X, Y: point1.Y });
                 if (i != 0)
                     internalRegion[i - 1].region.push({ X: point1.X, Y: point1.Y });
                 
                 if( (i +1 ) < visiblePoints.length &&  !visiblePoints[i+1].visible)
                 {
                     for (var j = i; j >=0; j--)
                     {
                         if (visiblePoints[j].visible) {
                             origin = visiblePoints[j].low;
                             var point = { xValue: visiblePoints[j].xValue, YValues: [origin] };
                             var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                             areasb.append("L" + " " + (point2.X) + " " + ((point2.Y)) + " ");
                             internalRegion[j].region.push({ X: point2.X, Y: point2.Y });
                             if(j!=0)
                                 internalRegion[j - 1].region.push({ X: point2.X, Y: point2.Y });
                         }
                         else
                             break;
                     }
                     startPoint = null;
                 }

              
             }
         }
         for (var i = visiblePoints.length-1; i >= 0; i--)
         {
             if (visiblePoints[i].visible)
             {
                 origin = visiblePoints[i].low;
                 var point = { xValue: visiblePoints[i].xValue, YValues: [origin] };
                 var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                 areasb.append("L" + " " + (point2.X) + " " + ((point2.Y)) + " ");
                 internalRegion[i].region.push({ X: point2.X, Y: point2.Y });
                 if(i!=0)
                     internalRegion[i - 1].region.push({ X: point2.X, Y: point2.Y });

             }
             else if (visiblePoints.length>i && visiblePoints[i].visible)
             {
                 var point2 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                 areasb.append("L" + " " + (point2.X) + " " + ((point2.Y)) + " ");
                 break;
             }                       
             else if (i != (visiblePoints.length - 1))
			  {
                 var count=0;
                 for(k=i-1;k>=0;k--)
                 {
                     if (visiblePoints[k].visible)
                        count++;
                     else if (!visiblePoints[k].visible)
                        break;
                 }
                 i = i - count;
             }
         }     
         aDirection = areasb.toString();
         this.chartObj.model.chartRegions.push(internalRegion);

         this.drawAreaPath(currentSeries, style, aDirection, translate);
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },

     hiloTypes:true

 });

 ej.seriesTypes.rangearea = ej.ejRangeAreaSeries;

 ej.ejSplineSeries = ejExtendClass(ej.EjSeriesRender, {
    
     draw: function (chart,options, params) {
		 if(!params.seriesCollection)
		 {			
			params.seriesCollection = {  };
			options._name = ej.util.isNullOrUndefined(options.name) ? "" : options.name.replace(/ /g,'');
         }
		 if(!params.seriesCollection[options._name])
			params.seriesCollection[options._name] = {};
         this.chartObj = chart;
		 options.animationType = "path";
         var series = options,
			spDirection = "",
            splinesb = ej.EjSvgRender.utils._getStringBuilder(),
            style = this.setLineSeriesStyle(series),
            yIndex = 0,
            visiblePoints = this.chartObj.dragPoint? series.pointCollection : this._isVisiblePoints(series),
            ySpline = params.seriesCollection[series._name].naturalSpline || this.naturalSpline(visiblePoints),
            firstPoint = null,
            secondPoint = null,
            firstIndex = -1, controlPointsCount = 0,
			controlPoints = params.seriesCollection[series._name].controlPoints || [];
			
		 //Removed spline sorting behavior based on the 'X' points.
         //series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
         
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             secondPoint = visiblePoints[i];             
             if (secondPoint.visible) {
                 if (firstPoint != null) {
                     var controlPoint1 = null;
                     var controlPoint2 = null;
                     var data = controlPoints[controlPointsCount++] || this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                     controlPoint1 = data["controlPoint1"];
                     controlPoint2 = data["controlPoint2"];
                     var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                     var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                     var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                     var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                     var chartObj = this.chartObj;
                     splinesb.append("M" + " " + (pt1.X + chartObj.canvasX) + " " + (pt1.Y + chartObj.canvasY) + " " + "C" + " " + (bpt1.X + chartObj.canvasX) + " " + (bpt1.Y + chartObj.canvasY) + " " + (bpt2.X + chartObj.canvasX) + " " + (bpt2.Y + chartObj.canvasY) + " " + (pt2.X + chartObj.canvasX) + " " + (pt2.Y + chartObj.canvasY) + " ");

                 }
                 firstPoint = secondPoint;
                 firstIndex = pointIndex;
             } else {
                 firstPoint = null;
             }
         }
         
        spDirection = splinesb.toString();
        this._drawLinePath(series, style, spDirection);
        if (this.chartObj.dragPoint)
            this.chartObj.svgRenderer.append(this.chartObj.gPreviewSeriesGroupEle, this.chartObj.gSeriesEle);
         else
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     }

 });

 ej.seriesTypes.spline = ej.ejSplineSeries;

 ej.ejSplineAreaSeries = ejExtendClass(ej.EjSeriesRender, {


     draw: function (chart, options, params) {
		 if(!params.seriesCollection)
		 {
			params.seriesCollection = {  };
			options._name = ej.util.isNullOrUndefined(options.name) ? "" : options.name.replace(/ /g,'');
         }
		 if(!params.seriesCollection[options._name])
			params.seriesCollection[options._name] = {};
         this.chartObj = chart;
		 options.animationType = "path";
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var spDirection = "";
         var splinesb = ej.EjSvgRender.utils._getStringBuilder();
         var style = this.setAreaSeriesStyle(series);
         var yIndex = 0;
         //Removed splineArea sorting behavior based on the 'X' points.
         if (series.sorting)
         	series.points = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this.chartObj.dragPoint? series.pointCollection : this._isVisiblePoints(series);
         var ySpline = params.seriesCollection[series._name].naturalSpline || this.naturalSpline(visiblePoints);
         var firstPoint = null;
         var secondPoint = null;
         var firstIndex = -1;
		
         var origin = ej.EjSeriesRender.prototype.getOrigin(this, series, params);
         var startPoint = null;
         var start = true;
         var chartObj = this.chartObj;
         var translate = [], controlPointsCount = 0, controlPoints = params.seriesCollection[series._name].controlPoints || [];
         translate[0] = chartObj.canvasX;
         translate[1] = chartObj.canvasY;
         var count = 0;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             secondPoint = visiblePoints[i];
             if (secondPoint.visible) {
                 if (!startPoint) {
                     startPoint = { xValue: visiblePoints[i].xValue, YValues: [origin] };
                     var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, series);
                    }
                 if (firstPoint != null) {
                     var controlPoint1 = null;
                     var controlPoint2 = null;
                     var data = controlPoints[controlPointsCount++] || this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                     controlPoint1 = data["controlPoint1"];
                     controlPoint2 = data["controlPoint2"];
                     var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                     var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                     var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                     var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                     if (start) {
                         if (requireInvertedAxes)
                             splinesb.append("M" + " " + (startLoc.X) + " " + (pt1.Y) + " " + "L" + " " + (pt1.X) + " " + ((pt1.Y)) + " ");
                         else
                             splinesb.append("M" + " " + (pt1.X) + " " + (startLoc.Y) + " " + "L" + " " + (pt1.X) + " " + ((pt1.Y)) + " ");
                         start = false;
                     }
                     splinesb.append("C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                     if (pointIndex == visiblePoints.length - 1 || (pointIndex < visiblePoints.length - 1 && !visiblePoints[i + 1].visible)) {
                        if (requireInvertedAxes)
                            splinesb.append("L" + " " + (startLoc.X) + " " + (pt2.Y) + " ");
                         else
                            splinesb.append("L" + " " + (pt2.X) + " " + (startLoc.Y) + " ");
                         startPoint = null;
                         start = true;
                     }
                 }
                 firstPoint = secondPoint;
                 firstIndex = pointIndex;
             } else {
                 firstPoint = null;
             }
         }

         spDirection = splinesb.toString();

         this.drawAreaPath(series, style, spDirection, translate);
         if (this.chartObj.dragPoint)
             this.chartObj.svgRenderer.append(this.chartObj.gPreviewSeriesGroupEle, this.chartObj.gSeriesEle);
         else
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     }
 });

 ej.seriesTypes.splinearea = ej.ejSplineAreaSeries;



 ej.ejScatterSeries = ejExtendClass(ej.EjSeriesRender, {

        
    
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;
		 options.animationType = "scatter";
		 var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
        var visiblePoints = this._isVisiblePoints(series);
		 var visiblePointsLength=visiblePoints.length;
		 var point,point1;
        for (var i = 0; i < visiblePointsLength; i++)
        {
            point = visiblePoints[i];
            var pointIndex = i;
            point1 = ej.EjSvgRender.utils._getPoint(point, options);
            this.drawSymbol(seriesIndex, options, pointIndex, point1.X, point1.Y);
        }

     }
 });

 ej.seriesTypes.scatter = ej.ejScatterSeries;

 
 ej.ejBubbleSeries = ejExtendClass(ej.EjSeriesRender, {

     createBubbleGroup: function(series) {

         var seriesIndex = $.inArray(series, this.chartObj.model.series);
		 var areaBoundsX = this.chartObj.model.m_AreaBounds.X;
		 var areaBoundsY = this.chartObj.model.m_AreaBounds.Y;
	     var isTransposed =  series._isTransposed;
         var transX = isTransposed? areaBoundsX:series.xAxis.x;

         var transY = isTransposed ? areaBoundsY : series.yAxis.y;
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

     },
     draw: function (chart,options) {
         this.chartObj = chart;
		 options.animationType = "bubble";
         var series = options;    
		 var cornerRadius = series.cornerRadius;
         series.points = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries); 
         var visiblePoints = this._isVisiblePoints(series);
         var sizeValueIndex = 1;
		 //bubble responsive
         var bubbleOptions = options.bubbleOptions;
		 var mode = bubbleOptions.radiusMode.toLowerCase();
         var temp = 100, minRadius, maxRadius;
         var areaBounds = chart.model.m_AreaBounds;
		 var visibleSeries = chart.model._visibleSeries, maximumSize = 0;
		 var value = Math.max(areaBounds.Height, areaBounds.Width);
		 var segmentRadius, radius, percentChange;
		 
		if (mode == "minmax"){  // minmax mode radius calculation
			percentChange = value/temp;
			minRadius = bubbleOptions.minRadius * percentChange;
	        maxRadius = bubbleOptions.maxRadius * percentChange;
			radius = maxRadius - minRadius;
			maximumSize = Math.max.apply(0,$.map(visiblePoints,function (v) { return v.YValues[sizeValueIndex]; }));
		} else {    // auto mode radius calculation
			maxValue = (value/5) /2;
			minRadius = maxRadius = 1;
			radius = maxValue * maxRadius;		
			for (m= 0; m < visibleSeries.length; m++){
				if (visibleSeries[m].type.toLowerCase() == "bubble" && visibleSeries[m].visibility == "visible" 
					&& visibleSeries[m].bubbleOptions.radiusMode.toLowerCase() == "auto"){
					points = visibleSeries[m]._visiblePoints;
					for (n= 0; n < points.length; n++)
						maximumSize = maximumSize < points[n].YValues[sizeValueIndex] ? points[n].YValues[sizeValueIndex] : maximumSize;
				}
			}
		}
         this.createBubbleGroup(series);
         for (var i = 0; i < visiblePoints.length; i++) {
             var point = visiblePoints[i];
             var pointIndex= i;
             if (point.visible) {
				 if (series.bubbleOptions.radiusMode.toLowerCase() == "minmax")
					segmentRadius = minRadius + radius * Math.abs(point.YValues[sizeValueIndex] / maximumSize);
				 else
					segmentRadius = radius * Math.abs(point.YValues[sizeValueIndex] / maximumSize);
					
                 var location = ej.EjSvgRender.utils._getPoint(point, series);
                 
				 var styleOptions = this.chartObj.setStyle(this, series, seriesIndex, pointIndex);
				 
                 var options = {
                     'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                     'cx': location.X + this.chartObj.canvasX,
                     'cy': (location.Y) + this.chartObj.canvasY,
                     'r': segmentRadius,
                         'fill': styleOptions.interior,
						 'cornerRadius': cornerRadius,
                         'stroke-width': styleOptions.borderWidth,
                     'stroke-dasharray': styleOptions.dashArray,
                     'opacity': styleOptions.opacity,
                     'stroke': styleOptions.borderColor
                     };
					 if ((typeof(cornerRadius) != "object" && cornerRadius >0) || cornerRadius.topLeft >0 || cornerRadius.bottomLeft >0 
					    || cornerRadius.topRight >0 || cornerRadius.bottomRight >0)           //calculate path for rounded corner
					{
						var roundrect=ej.EjSvgRender.utils.calculateroundedCorner(options);
						options.d = roundrect;
						cSer.chartObj.svgRenderer.drawPath(options,cSer.gSeriesGroupEle);
					}
					else

                 point.radius = segmentRadius; // radius is stored to draw tracksymbol for bubble in canvas
                 
                 this.chartObj.svgRenderer.drawCircle(options, this.gSeriesGroupEle);
				 
			 

                 var cx = location.X;
                 var cy = location.Y;
                 var r = segmentRadius;


                 var valwidth, x, y, valheight;
                 x = ((cx - r) + (series.isTransposed ? this.chartObj.model.m_AreaBounds.X :series.xAxis.x));
                 y = ((cy - r) + (series.isTransposed ? this.chartObj.model.m_AreaBounds.Y : series.yAxis.y));

                 valheight = 2 * r;
                 valwidth = 2 * r;

                 var bounds = { X: x, Y: y, Width: valwidth, Height: valheight };
                 ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
             }
         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     }
 });

ej.seriesTypes.bubble = ej.ejBubbleSeries;
 ej.ejhiloSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options, params) {

         this.chartObj = chart;
		 options.animationType = "hilo";
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         //DrawHiloGraph      
        
         var currentseries = options;
          currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries, params);
         var point = null;
         
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
       

         for (var i = 0; i < visiblePoints.length; i++) {
             point = visiblePoints[i];
			 var pointIndex= i;
             if (point.visible) {
                
                 var lDirection = "";
                    if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]))
                     continue;
					 
                    var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex);

                    var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex, seriesIndex);
              
				 
                 var lowvalue = {}, highvalue = {};
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                 } else {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                 }
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue   , currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 lDirection = "M" + " " + (point1.X + this.chartObj.canvasX) + " " + ((point1.Y + this.chartObj.canvasY)) + " " + "L" + " " + (point2.X + this.chartObj.canvasX) + " " + ((point2.Y + this.chartObj.canvasY)) + " ";
                 var pointbounds = { point1:point1, point2:point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
             }
         }
        
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },
     
     hiloTypes: true
 });

 ej.seriesTypes.hilo = ej.ejhiloSeries;
 
 //Draw HiloOpenClose
 ej.ejhiloopencloseSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options, params) {

         this.chartObj = chart;
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         //DrawHiloGraph      
         options.animationType = "hilo";
         var currentseries = options;
         currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries, params);
         var point = null;
		 var paletteColor = this.chartObj.model.palette;
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         
         var interior;

     


         for (var i = 0; i < visiblePoints.length; i++) {
		    var pointIndex = i;
             point = visiblePoints[i];
             if (point.visible) {

                 var lDirection = "";
                 var drawOpenWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "open") ? true : false);
                 var drawCloseWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "close") ? true : false);

                
                 var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                  
                    
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     
                 } else {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                    
                 }
				 currentseries.fill = currentseries.isFill ? currentseries.fill : null;
                 if (point.YValues[2] < point.YValues[3]) 
                     currentseries.bearFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? ((currentseries.fill) ? currentseries.fill : ((paletteColor && paletteColor.length > 0 )? paletteColor[seriesIndex] : "#339933")) : currentseries.bearFillColor;
                 
                 else 
                     currentseries.bullFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? ((currentseries.fill) ? currentseries.fill : ((paletteColor && paletteColor.length > 0) ? paletteColor[seriesIndex] : "#E51400")) : currentseries.bullFillColor;
                    
				 
                 var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                 var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex,seriesIndex, interior);
                 

				 if (point.fill) {
                     interior = jQuery.type(point.interior) == "array" ? point.fill[0] : point.fill;
                 }
                 else {
                      point._hiloFill = interior;
                 }
				 var canvasX = this.chartObj.canvasX;
				 var canvasY = this.chartObj.canvasY;
                 
                 //Draw open points
                 if (drawOpenWing) {

                     var startOpenValue = { xValue: openvalue.xValue + sidebysideinfo.Median, YValues: openvalue.YValues };
                     var pto1 = ej.EjSvgRender.utils._getPoint(startOpenValue, currentseries);
                     var endOpenValue = { xValue: openvalue.xValue + sidebysideinfo.Start, YValues: openvalue.YValues };
                     var pto2 = ej.EjSvgRender.utils._getPoint(endOpenValue, currentseries);
                      lDirection = "M" + " " + (pto1.X + canvasX) + " " + ((pto1.Y + canvasY)) + " " + "L" + " " + (pto2.X + canvasX) + " " + ((pto2.Y + canvasY)) + " ";
                     var openbounds = { point1: pto1, point2: pto2 };
                     this._drawHiloPath(currentseries, styleOptions,pathInterior,  lDirection, i, openbounds);
                     lDirection = "";

                 }
                 //Draw close points
                 if (drawCloseWing) {
                     var startCloseValue = { xValue: closevalue.xValue + sidebysideinfo.Median, YValues: closevalue.YValues };
                     var ptc1 = ej.EjSvgRender.utils._getPoint(startCloseValue, currentseries);
                     var endCloseValue = { xValue: closevalue.xValue + sidebysideinfo.End, YValues: closevalue.YValues };
                     var ptc2 = ej.EjSvgRender.utils._getPoint(endCloseValue, currentseries);
                     lDirection = "M" + " " + (ptc1.X + canvasX) + " " + ((ptc1.Y + canvasY)) + " " + "L" + " " + (ptc2.X + canvasX) + " " + ((ptc2.Y + canvasY)) + " ";
                     var pointbounds = { point1: ptc1, point2: ptc2 };
                     this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                     lDirection = "";

                 }
                 // Draw High low points
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 lDirection = "M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ";
                 var bounds = { point1: point1, point2: point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, bounds);
             }

         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     
     },

     hiloTypes: true

 });

 ej.seriesTypes.hiloopenclose = ej.ejhiloopencloseSeries;
 
 //Draw candle
ej.ejCandleSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options, params) {

         this.chartObj = chart;
		 options.animationType = "hilo";
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         //DrawCandleGraph      

         var currentseries = options;
         currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries, params);
         var point = null;
		 var paletteColor = this.chartObj.model.palette;
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var interior;

         for (var i = 0; i < visiblePoints.length; i++) {
             point = visiblePoints[i];
			  var pointIndex= i;
             if (point.visible) {

                 var lDirection = "";
               
                 var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                 
                
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     
                 } else {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     
                 }      
                 if (point.YValues[2] < point.YValues[3]) {
                     
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     currentseries.bearFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? ( (paletteColor && paletteColor.length > 0 ) ? paletteColor[seriesIndex] : "#339933") : currentseries.bearFillColor;
                     
                 } else {
                    
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     currentseries.bullFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? ( (paletteColor && paletteColor.length > 0) ? paletteColor[seriesIndex]:"#E51400") : currentseries.bullFillColor;
                    
                 }
                 
                 var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                 var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex,seriesIndex, interior);
				 
				 if (point.style && point.style.interior) {
                     interior = jQuery.type(point.style.interior) == "array" ? point.style.interior[0] : point.style.interior;
                 }
               
                 else {
                     point._hiloFill = pathInterior;
                 }

                 //Draw High low points
				 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
				 var point2 = ej.EjSvgRender.utils._getPoint({ xValue: openvalue.xValue + sidebysideinfo.Median, YValues: [openvalue.YValues[0]] }, currentseries);
				 var point3 = ej.EjSvgRender.utils._getPoint({ xValue: closevalue.xValue + sidebysideinfo.Median, YValues: [closevalue.YValues[0]] }, currentseries);
                 var point4 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 var canvasX = this.chartObj.canvasX;
                 var canvasY = this.chartObj.canvasY;
                 lDirection = "M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " " + "M" + " " + (point3.X + canvasX) + " " + ((point3.Y + canvasY)) + " " + "L" + " " + (point4.X + canvasX) + " " + ((point4.Y + canvasY)) + " ";
                 var pointbounds = { point1: point1, point2: point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                 //Draw open and close points
              
                 var startRect = { xValue: closevalue.xValue + sidebysideinfo.Start, YValues: closevalue.YValues };
                     var orginRect = ej.EjSvgRender.utils._getPoint(startRect, currentseries);

                     var rectSize = { xValue: openvalue.xValue + sidebysideinfo.End, YValues: openvalue.YValues },
                      sizeRect = ej.EjSvgRender.utils._getPoint(rectSize, currentseries),
                      rect = ej.EjSvgRender.utils._correctRect(orginRect.X, orginRect.Y, sizeRect.X, sizeRect.Y);
                  
                     var candleOptions = {
                        'id': this.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + i,
                        'x': rect.X + this.chartObj.canvasX,
                        'y': rect.Y + this.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                        'stroke-dasharray': styleOptions.dashArray,
                         'stroke': styleOptions.borderColor
                     };

                    this.chartObj.svgRenderer.drawRect(candleOptions, this.gSeriesGroupEle);

                    var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, rect.Y, currentseries, this.chartObj);
                    var bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, point, i);
                                    
             }
             
         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
        

     },
     
     hiloTypes: true
 });
 ej.seriesTypes.candle = ej.ejCandleSeries;

 ej.ejPieOfPieSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {
         this.chartObj = chart;
         var series = new ej.seriesTypes["pie"]();
         series.draw(chart, options);
     },
     chartAreaType: "None"
 });
 ej.seriesTypes.pieofpie = ej.ejPieOfPieSeries;

 ej.ejPieSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var currentseries = options;
         var internalRegion = [];
         var seriesType = options.type.toLowerCase();
         var pieSeriesIndex = options.collectionIndex;
         var coefficient = pieSeriesIndex == 0 ? currentseries.pieCoefficient : currentseries.pieOfPieCoefficient;
         var visiblePoints = this._calculateVisiblePoints(options,this.chartObj).visiblePoints;
        
         var size = this.calculatingSliceAngle(currentseries);                  
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var seriesLength = this.chartObj.model._visibleSeries.length;

         var currentSeries = this.chartObj.model._visibleSeries[seriesIndex];
        		currentSeries._pieCoefficient =  currentSeries.pieCoefficient;
        		if(this.chartObj.model.circularRadius.length>1){
        		    for (i = seriesIndex;!ej.util.isNullOrUndefined(currentSeries.zOrder) ? i >= 0 : i < this.chartObj.model.circularRadius.length; !ej.util.isNullOrUndefined(currentSeries.zOrder) ? i-- : i++) {
        		        if (!ej.util.isNullOrUndefined(this.chartObj.model.circularRadius[i])) {
        		            this.chartObj.model.circularRadius[seriesIndex] = this.chartObj.model.circularRadius[i] * currentSeries._pieCoefficient;
        		            break;
        		        }
        		    }
        		}
        		else {
        		    if (seriesType == "pieofpie" && currentSeries.splitMode != "") 
        		        this.chartObj.model.circularRadius[pieSeriesIndex] = 0.25 * coefficient * Math.min(size.width, size.height);       		       
        		    else
        		        this.chartObj.model.circularRadius[seriesIndex] = 0.5 * currentSeries._pieCoefficient * Math.min(size.width, size.height);
        		}
        		
        var min= Math.min(size.width, size.height);
        if (min < 0)
        {
            return min;
        }
        var totalDegree=Math.abs(currentseries.endAngle-currentseries.startAngle);
        if (totalDegree < 270) 
            this.pieDoughnutCenter(currentseries);
         var text;
        
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         var visibility = (currentseries.enableAnimation && !currentseries._animatedSeries) ? 'hidden' : 'visible';

        
         var txtOptions = { 'id': this.chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gSeriesTextEle) this.chartObj.gSeriesTextEle = [];
         this.chartObj.gSeriesTextEle[seriesIndex] = pieSeriesIndex == 1 ? this.chartObj.gSeriesTextEle[0] : this.chartObj.svgRenderer.createGroup(txtOptions);
         var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gSymbolGroupEle) this.chartObj.gSymbolGroupEle = [];
         this.chartObj.gSymbolGroupEle[seriesIndex] = this.chartObj.svgRenderer.createGroup(symbolOptions);

         var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gConnectorEle) this.chartObj.gConnectorEle = [];
         this.chartObj.gConnectorEle[seriesIndex] = this.chartObj.svgRenderer.createGroup(connectorOptions);
         var dataLabelOptions = { 'id': this.chartObj.svgObject.id + '_DataLabel_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gDataLabelEle) this.chartObj.gDataLabelEle = [];
         this.chartObj.gDataLabelEle[seriesIndex] = pieSeriesIndex == 1 ? this.chartObj.gDataLabelEle[0] : this.chartObj.svgRenderer.createGroup(dataLabelOptions);
             
                 
         for (var j = 0; j < visiblePoints.length; j++) {
             pointIndex = this.chartObj.model._isPieOfPie ? visiblePoints[j].actualIndex : j;
             if (isNaN(currentseries._visiblePoints[j].startAngle)) continue;
             var point = currentseries._visiblePoints[j];
                 if (point.visible) {
                     var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries, seriesIndex, pieSeriesIndex);
                 var sliceXY = result.Direction.split(" "); 
             var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex,  j);
			 if (currentseries.startAngle < currentseries.endAngle) // for canvas rendering with modified start and end angle
                     var counterClockWise = false;
                 else
                     counterClockWise = true;
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                 'stroke-dasharray': styleOptions.dashArray,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'd': result.Direction,
                     'opacity': styleOptions.opacity,
                     'radius': seriesType =="pieofpie"?this.chartObj.model.circularRadius[pieSeriesIndex]: this.chartObj.model.circularRadius[seriesIndex],
                     'start': point.startAngle - 1.57,
                     'end': point.endAngle - 1.57,
                     'pointIndex': pointIndex,
					 'counterClockWise': counterClockWise,
                     'innerR': 0,
					  'cx': result.centerX,
                     'cy':result.centerY,
                     'x': sliceXY[12],
                     'y': sliceXY[13]
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

                 region = { PointIndex: pointIndex, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] ,
                     PieSeriesIndex: pieSeriesIndex,SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries)};
                 internalRegion.push(region);

             }
         }
         var seriesIndex = this.chartObj.model._isPieOfPie ? pieSeriesIndex : seriesIndex;
        
         if (this.chartObj.model._isPieOfPie && pieSeriesIndex == 1
             && currentSeries.pieCollections[0].length > 0 && currentSeries.pieCollections[1].length == 0)
         {
             var emptyCircleOptions = this._drawEmptyPieOfPie(this.chartObj);             
             this.chartObj.svgRenderer.drawPath(emptyCircleOptions, this.gSeriesGroupEle);
         }
        
         var seriesData = { Radius: this.chartObj.model.circularRadius[seriesIndex], CenterX: this.chartObj.model.circleCenterX[seriesIndex], CenterY: this.chartObj.model.circleCenterY[seriesIndex] };
         var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },
 
     chartAreaType : "None"

 });

 ej.seriesTypes.pie = ej.ejPieSeries;

 ej.ejDoughnutSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {
         this.chartObj = chart;
         var currentseries = options;
         var internalRegion = [];

         var visiblePoints = this._calculateVisiblePoints(options).visiblePoints;
         var size = this.calculatingSliceAngle(currentseries);
		 var visiblePointslength = visiblePoints.length;        
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var seriesLength = this.chartObj.model._visibleSeries.length;
         var seriesLength = this.chartObj.model._visibleSeries.length;
         var currentSeries = this.chartObj.model._visibleSeries[seriesIndex];
             	currentSeries._doughnutSize = currentSeries.doughnutSize;
             	currentSeries._doughnutCoefficient =  currentSeries.doughnutCoefficient;
             	if (this.chartObj.model.circularRadius.length > 1) {
             	    for (i = seriesIndex; !ej.util.isNullOrUndefined(currentSeries.zOrder) ? i >= 0 : i < this.chartObj.model.circularRadius.length; !ej.util.isNullOrUndefined(currentSeries.zOrder) ? i-- : i++) {
             	        if (!ej.util.isNullOrUndefined(this.chartObj.model.circularRadius[i])) {
             	            this.chartObj.model.circularRadius[seriesIndex] = this.chartObj.model.circularRadius[i] * currentSeries._doughnutSize;
             	            break;
             	        }
             	    }
             	}
             	else
             	    this.chartObj.model.circularRadius[seriesIndex] = 0.5 * currentSeries._doughnutSize * Math.min(size.width, size.height);
         var min = Math.min(size.width, size.height);
         if (min < 0) {
             return min;
         }
         var totalDegree = Math.abs(currentseries.endAngle - currentseries.startAngle);
         if (totalDegree < 270)//for centering the chart
             this.pieDoughnutCenter(currentseries);
         this.chartObj.model.innerRadius[seriesIndex] = currentSeries._doughnutCoefficient * this.chartObj.model.circularRadius[seriesIndex];


         var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         var visibility = (currentseries.enableAnimation && !currentseries._animatedSeries) ? 'hidden' : 'visible';

          
         var txtOptions = { 'id': this.chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gSeriesTextEle) this.chartObj.gSeriesTextEle = [];
         this.chartObj.gSeriesTextEle[seriesIndex] = this.chartObj.svgRenderer.createGroup(txtOptions);

         var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gSymbolGroupEle) this.chartObj.gSymbolGroupEle = [];
         this.chartObj.gSymbolGroupEle[seriesIndex] = this.chartObj.svgRenderer.createGroup(symbolOptions);

         var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gConnectorEle) this.chartObj.gConnectorEle = [];
         this.chartObj.gConnectorEle[seriesIndex] = this.chartObj.svgRenderer.createGroup(connectorOptions);

         var dataLabelOptions = { 'id': this.chartObj.svgObject.id + '_DataLabel_' + seriesIndex, 'visibility': visibility };
         if (!this.chartObj.gDataLabelEle) this.chartObj.gDataLabelEle = [];
         this.chartObj.gDataLabelEle[seriesIndex] = this.chartObj.svgRenderer.createGroup(dataLabelOptions);
		 if ((currentseries._doughnutSize > 0) && (currentSeries._doughnutCoefficient >= 0) && (currentseries._doughnutSize <= 1) && (currentSeries._doughnutCoefficient <= 1)) {
           
             for (var j = 0; j < visiblePointslength; j++) {
             var point = currentseries._visiblePoints[j];
             var pointIndex = j;
             if (point.visible) {
                 var explodeX = 0;
                 var explodeY = 0;
                 var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries,seriesIndex);
				 //for canvas slice explode
                 if ((point.actualIndex == currentseries.explodeIndex || currentseries.explodeAll) && !this.chartObj.vmlRendering) {
                     var chartStartAngle = -.5 * Math.PI; 
                     var startAngle = point.startAngle + chartStartAngle;
                     var endAngle = point.endAngle + chartStartAngle - 0.000001;
                     var midAngle = (startAngle + endAngle) / 2;
                     explodeX = this.chartObj.model.circleCenterX[seriesIndex] + Math.cos(midAngle) * currentseries.explodeOffset;
                     explodeY = this.chartObj.model.circleCenterY[seriesIndex] + Math.sin(midAngle) * currentseries.explodeOffset;
                 }

             var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex);

                 if (currentseries.startAngle < currentseries.endAngle) // for canvas rendering with modified start and end angle
                     var counterClockWise = false;
                 else
                     counterClockWise = true;
                 
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                 'stroke-dasharray': styleOptions.dashArray,
                     'stroke-linecap': currentseries.lineCap,
                     'opacity': styleOptions.opacity,
                     'stroke-linejoin': currentseries.lineJoin,
                     'd': result.Direction,
                     'start': point.startAngle - 1.57,
                     'end': point.endAngle - 1.57,
                     'pointIndex': pointIndex,
                     'radius': this.chartObj.model.circularRadius[seriesIndex],
                     'innerR': this.chartObj.model.innerRadius[seriesIndex],
                     'counterClockWise': counterClockWise,
                     'x': explodeX !=0 ? explodeX : this.chartObj.model.circleCenterX[seriesIndex],
                     'y': explodeY !=0 ? explodeY : this.chartObj.model.circleCenterY[seriesIndex]
             };
            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);


                 region = { PointIndex: j, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] ,  SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries)};
                 internalRegion.push(region);

             }
         }
}
         var seriesData = { Radius: this.chartObj.model.circularRadius[seriesIndex], DRadius: this.chartObj.model.innerRadius[seriesIndex], CenterX: this.chartObj.model.circleCenterX[seriesIndex], CenterY: this.chartObj.model.circleCenterY[seriesIndex] };
         var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     },

     chartAreaType:"None"
 });

 ej.seriesTypes.doughnut = ej.ejDoughnutSeries;

 ej.ejPyramidSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         var chartModel = chart.model,
             point,
             seriesRender = this,
             legendActualBounds = chartModel.LegendActualBounds,
             currentseries = options,
             visiblePoints = this._calculateVisiblePoints(currentseries).visiblePoints,
             visiblePointslength = visiblePoints.length,
             internalRegion = [],
             translate = [],
             legend = chartModel.legend,
			 dataLabel = currentseries.marker.dataLabel,
             legendPosition = legend.position.toLowerCase(),
             legendWidth = legend.border.width,
             textSize = 0,
             text,
             title = chartModel.title,
             subTitle = chartModel.title.subTitle,
             borderSize = chartModel.border.width,
             titleVisibility = title.text != '' && title.visible && title.enableTrim ? true : false,
             subTitleVisibility = subTitle.text != '' && subTitle.visible && subTitle.enableTrim ? true : false,
             measureTitle = (chartModel.title.text && chartModel.title.visible) ? ej.EjSvgRender.utils._measureText(chartModel.title.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.font) : 0,
             measureSubTitle = (chartModel.title.subTitle.text && chartModel.title.subTitle.visible) ? ej.EjSvgRender.utils._measureText(chartModel.title.subTitle.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.subTitle.font) : 0,
             measureTitleHeight = (chart.model.title.text == '' || !chart.model.title.visible) ? 0 : ((titleVisibility && title.textOverflow == 'wrap') ? measureTitle.height * chart.model.titleWrapTextCollection.length : (titleVisibility && title.textOverflow == 'wrapandtrim') ?
             measureTitle.height * chart.model.titleWrapTextCollection.length : measureTitle.height),
             measureSubTitleHeight = (chart.model.title.subTitle.text == '' || !chart.model.title.subTitle.visible) ? 0 : ((subTitleVisibility && subTitle.textOverflow == 'wrap') ? measureSubTitle.height * chart.model.subTitleWrapTextCollection.length : (subTitleVisibility && subTitle.textOverflow == 'wrapandtrim') ?
             measureSubTitle.height * chart.model.subTitleWrapTextCollection.length : measureSubTitle.height),
             xOffset = (chartModel.margin.left + chartModel.elementSpacing + (borderSize * 2) + chartModel.elementSpacing + chartModel.margin.right + ((legendPosition === "right" || legendPosition === "left") ? legendActualBounds.Width + (2 * legendWidth) : 0)),
             yOffset = (((chartModel.title.text && chartModel.title.visible) ? (chartModel._titleLocation.Y + measureTitleHeight + measureSubTitleHeight) : chartModel.elementSpacing) + (borderSize * 2) + chartModel.margin.top + chartModel.elementSpacing + chartModel.elementSpacing + ((legendPosition === "top" || legendPosition === "bottom") ? legendActualBounds.Height + (2 * legendWidth) + chartModel.elementSpacing : 0));
		if (dataLabel.visible && dataLabel.shape != 'none' && currentseries.labelPosition == 'outside')
			xOffset = xOffset + (dataLabel.margin.left + dataLabel.margin.right);
         chartModel.chartRegions = [];
         chartModel.sumofYValues = 0;

         if (currentseries.labelPosition.toLowerCase() === 'outside') {
             for (var i = 0; i < visiblePointslength; i++) {
                 var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                 if (textSize < textWidth)
                     textSize = textWidth;
             }
             chartModel.textSize = textSize;
         }
         if (currentseries._enableSmartLabels) {
             chartModel.actualWidth = ($(chart.svgObject).width() - xOffset) * 0.8; //calculate a pyramid width when smartlabel is enabled
             chartModel.resWidth = ($(chart.svgObject).width() - xOffset - chartModel.actualWidth) / 2;
         }
         else {
             chartModel.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
             chartModel.resWidth = 0;
         }
         var seriesIndex = $.inArray(currentseries, chartModel._visibleSeries);
         chartModel.actualHeight = $(chart.svgObject).height() - yOffset;
         var subtitleHeight = (chart.model.title.subTitle.text == '') ? 0 : measureSubTitleHeight;
         var pyrX = chart.pyrX = ((legendPosition === "left") ? (legendActualBounds.Width + (2 * legendWidth)) : 0) + chartModel.elementSpacing + chartModel.margin.left;
         var pyrY = chart.pyrY = ((legendPosition === "top") ? (legendActualBounds.Height + (2 * legendWidth)) : 0) + ((title.text && title.visible) ? (chartModel._titleLocation.Y + measureTitleHeight + subtitleHeight) : (chartModel.margin.top + chartModel.elementSpacing));
         translate[0] = pyrX;
         translate[1] = pyrY;
         for (var j = 0; j < visiblePointslength; j++) {
             chartModel.sumofYValues += visiblePoints[j].YValues[0];
         }
         var dataLabelOptions = { 'id': chart.svgObject.id + '_DataLabel_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gDataLabelEle = [];
            chart.gDataLabelEle[seriesIndex] = chart.svgRenderer.createGroup(dataLabelOptions);


         var numberToFixed = ej.util.isNullOrUndefined(chartModel.roundingPlaces) ? 2 : chartModel.roundingPlaces;

         var serOptions = { 'id': chart.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         this.gSeriesGroupEle = chart.svgRenderer.createGroup(serOptions);


         var txtOptions = { 'id': chart.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gSeriesTextEle = [];
            chart.gSeriesTextEle[seriesIndex] = chart.svgRenderer.createGroup(txtOptions);

           var symbolOptions = { 'id': chart.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gSymbolGroupEle = [];
            chart.gSymbolGroupEle[seriesIndex] = chart.svgRenderer.createGroup(symbolOptions);

            var connectorOptions = { 'id': chart.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gConnectorEle = [];
            chart.gConnectorEle[seriesIndex] = chart.svgRenderer.createGroup(connectorOptions);


         if (currentseries.pyramidMode.toLowerCase() == "linear")
             this.calculateLinearSegments(currentseries);
         else
             this.calculateSurfaceSegments(currentseries);

         for (j = 0; j < visiblePointslength; j++) {
             var pointIndex = j;
             point = currentseries._visiblePoints[j];
             if (point.visible) {
                 var result = this._getPyramidData(currentseries, pointIndex);

                 var styleOptions = chart.setStyle(this, currentseries, 0, pointIndex);

             options = {
                     'id': chart.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                 'stroke-dasharray': styleOptions.dashArray,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'opacity': styleOptions.opacity,
                     'pointIndex': pointIndex,
                     'd': result.Direction
                 };
                 chart.svgRenderer.drawPath(options, this.gSeriesGroupEle, translate);

                 if (currentseries.marker.dataLabel.template) {
                     ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, chart);
                 }

                 point.xLocation = result.PositionX;
                 point.yLocation = result.PositionY;
                 point.connectorLine = result.Connector;
                 point.startX = result.startX;
                 point.startY = result.startY;
                 point.Polygon = result.Polygon;
                 region = { PointIndex: pointIndex, Line1: result.Line1, Line2: result.Line2, Line3: result.Line3, Line4: result.Line4, Polygon: result.Polygon };
                 internalRegion.push(region);
             }
         }
         var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, chartModel._visibleSeries) };
         chartModel.chartRegions.push(region);

         if (this.gSeriesGroupEle) {
             $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
                 evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
                 seriesRender.chartObj.chartInteractiveBehavior(evt);
             });

             chart.svgRenderer.append(this.gSeriesGroupEle, chart.gSeriesEle);
         }

        

     },

     _getPyramidData: function (series, index) {
        
         var model = this.chartObj.model;
         var pointMarker = series._visiblePoints[index].marker;
         var actualIndex = series._visiblePoints[index].actualIndex;
         var offset = ((series.explodeIndex == actualIndex) || (series.explodeAll) ? series.explodeOffset : 0);
         var resWidth = model.resWidth;
         var top = model.pyramidData[index].CurrY;
         var bottom = model.pyramidData[index].CurrY + model.pyramidData[index].Height;
         var topRadius = 0.5 * (1 - model.pyramidData[index].CurrY);
         var bottomRadius = 0.5 * (1 - bottom);
         var point = series.points[index];

         var line1 = { x: resWidth + offset + topRadius * model.actualWidth, y: top * model.actualHeight };
         var line2 = { x: resWidth + offset + (1 - topRadius) * model.actualWidth, y: top * model.actualHeight };
         var line3 = { x: resWidth + offset + (1 - bottomRadius) * model.actualWidth, y: bottom * model.actualHeight };
         var line4 = { x: resWidth + offset + bottomRadius * model.actualWidth, y: bottom * model.actualHeight };

         var direction = "M" + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "z";
         var polygon = [
             { x: line1.x, y: line1.y },
             { x: line2.x, y: line2.y },
             { x: line3.x, y: line3.y },
             { x: line4.x, y: line4.y }
            ];
         if ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && series.marker.dataLabel.visible) {
             var positionX;
             var positionY;
             var startX = (line2.x + line3.x) / 2;
             var startY = (line2.y + line3.y) / 2;
             if (series.labelPosition.toLowerCase() === 'outside' ) {

                 positionX = (line2.x + line3.x) / 2;
                 positionY = (line2.y + line3.y) / 2;

                 var pyrX = ((this.chartObj.model.legend.position.toLowerCase() == "left" || this.chartObj.model.legend.position.toLowerCase() == "right") ? (this.chartObj.model.LegendActualBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;

                 if (ej.util.isNullOrUndefined(series.marker.dataLabel.connectorLine.height)) {
                     if (series._enableSmartLabels)
                         var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + resWidth + (2 * this.chartObj.model.elementSpacing));
                     else
                         var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + this.chartObj.model.textSize + this.chartObj.model.elementSpacing);
                 }
                 else
                     textOffset = series.marker.dataLabel.connectorLine.height;
                                                
                 var connectorX = this.getXCordinate(positionX, textOffset, 0);
                 var connectorY = this.getYCordinate(positionY, textOffset, 0);
                
                 var connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                 positionX = connectorX;
                 positionY = connectorY ;

                 
             }
             else {
                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line3.y + line1.y) / 2;
             }
         }
         return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection, Line1: line1, Line2: line2, Line3: line3, Line4: line4, startX: startX, startY: startY, Polygon: polygon };
     },

     calculateLinearSegments: function (series) {
         var currY = 0;
         var model = this.chartObj.model;
         model.pyramidData = [];
         var gapRatio = series.gapRatio >= 0 ? (series.gapRatio <= 1 ? series.gapRatio : 1) : 0;
         var coef = 1 / (model.sumofYValues * (1 + gapRatio / (1 - gapRatio)));
         var count = series._visiblePoints.length;
         for (var i = 0; i < count; i++) {
             if (series._visiblePoints[i].visible || series._visiblePoints[i].gapMode) {
                 var height = coef * series._visiblePoints[i].YValues[0];
                 model.pyramidData[i] = { CurrY: currY, Height: height };
                 currY += (gapRatio / (count - 1)) + height;
             }
         }
     },

     calculateSurfaceSegments: function (series) {
         var count = series._visiblePoints.length;
         var model = this.chartObj.model;
         model.pyramidData = [];
         var currY = 0;
         var y = [];
         var height = [];
         var gapRatio = series.gapRatio >= 0 ? (series.gapRatio <= 1 ? series.gapRatio : 1) : 0;
         var gapHeight = gapRatio / (count - 1);
         var preSum = this.getSurfaceHeight(0, model.sumofYValues);
         for (i = 0; i < count; i++) {
             if (series._visiblePoints[i].visible || series._visiblePoints[i].gapMode) {
                 y[i] = currY;
                 height[i] = this.getSurfaceHeight(currY, Math.abs(series._visiblePoints[i].YValues[0]));
                 currY += height[i] + gapHeight * preSum;
             }
         }
         var coef = 1 / (currY - gapHeight * preSum);
         for (var i = 0; i < count; i++) {
             if (series._visiblePoints[i].visible || series._visiblePoints[i].gapMode) {
                 var currHeight = coef * y[i];
                 model.pyramidData[i] = { CurrY: currHeight, Height: coef * height[i] };
             }
         }
     },

     getSurfaceHeight: function (y, surface) {
         var result = this.solveQuadraticEquation(1, 2 * y, -surface);
         if (result) {
             return result;
         }
         return 0;
     },
     solveQuadraticEquation: function (a, b, c) {
         var root1;
         var root2;

         if (a != 0) {
             var d = b * b - 4 * a * c;

             if (d >= 0) {
                 var sd = Math.sqrt(d);

                 root1 = (-b - sd) / (2 * a);
                 root2 = (-b + sd) / (2 * a);
                 return Math.max(root1, root2);

             }
         }
         else if (b != 0) {
             root1 = -c / b;
             root2 = -c / b;
             return Math.max(root1, root2);
         }

         return false;
     },

     chartAreaType: "None"
 });

 ej.seriesTypes.pyramid = ej.ejPyramidSeries;
   
 ej.ejFunnelSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {        
         this.chartObj = chart;
         var chartModel = chart.model,
             legend = chartModel.legend,
             legendPosition = legend.position.toLowerCase(),
             legendActualBounds = chartModel.LegendActualBounds,
             point,
             seriesRender = this,
             currentseries = options,
             visiblePoints = this._calculateVisiblePoints(currentseries).visiblePoints,
             visiblePointslength = visiblePoints.length,
             internalRegion = [],
			 dataLabel = currentseries.marker,
             legendWidth = legend.border.width,
             translate = [],
             textSize = 0,
             text,
             title = (chart.model.title.text == '') ? '' : chartModel.title,
             subTitle = (chart.model.title.subTitle.text == '') ? '' : chartModel.title.subTitle,
             borderSize = chartModel.border.width,
             titleVisibility = title.text != '' && title.visible && title.enableTrim ? true : false,
             subTitleVisibility = subTitle.text != '' && subTitle.visible && subTitle.enableTrim ? true : false,
             measureTitle = (chartModel.title.text&&chartModel.title.visible) ? ej.EjSvgRender.utils._measureText(chartModel.title.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.font) : 0,
             measureSubTitle = (chartModel.title.subTitle.text&&chartModel.title.subTitle.visible) ? ej.EjSvgRender.utils._measureText(chartModel.title.subTitle.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.subTitle.font) : 0,
             measureTitleHeight = (chart.model.title.text == '' || !chart.model.title.visible) ? 0 : ((titleVisibility && title.textOverflow == 'wrap') ? measureTitle.height * chart.model.titleWrapTextCollection.length : (titleVisibility && title.textOverflow == 'wrapandtrim') ?
             measureTitle.height * chart.model.titleWrapTextCollection.length : measureTitle.height),
             measureSubTitleHeight = (chart.model.title.subTitle.text == '' || !chart.model.title.subTitle.visible) ? 0 : ((subTitleVisibility && subTitle.textOverflow == 'wrap') ? measureSubTitle.height * chart.model.subTitleWrapTextCollection.length : (subTitleVisibility && subTitle.textOverflow == 'wrapandtrim') ?
             measureSubTitle.height * chart.model.subTitleWrapTextCollection.length : measureSubTitle.height),
             xOffset = (chartModel.margin.left + chartModel.elementSpacing + chartModel.elementSpacing + chartModel.margin.right + (borderSize * 2) + ((legendPosition === "right" || legendPosition === "left") ? legendActualBounds.Width + (2 * legendWidth) : 0)),
             yOffset = (((chartModel.title.text&&chartModel.title.visible) ? (chartModel._titleLocation.Y + measureTitleHeight+measureSubTitleHeight) : chartModel.elementSpacing) + chartModel.margin.top + (borderSize * 2) + chartModel.elementSpacing + chartModel.elementSpacing + ((legendPosition === "top" || legendPosition === "bottom") ? legendActualBounds.Height + (2 * legendWidth) + chartModel.elementSpacing : 0));
		if (dataLabel.visible && dataLabel.shape != 'none' && currentseries.labelPosition == 'outside')
			xOffset = xOffset + (dataLabel.margin.left + dataLabel.margin.right);
         chartModel.chartRegions = [];
         chartModel.sumofYValues = 0;
         if (currentseries.labelPosition.toLowerCase() === 'outside') {
             for (var i = 0; i < visiblePointslength; i++) {
                 var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                 if (textSize < textWidth)
                     textSize = textWidth;
             }
             chartModel.textSize = textSize;
         }
         if (currentseries._enableSmartLabels) {
             chartModel.actualWidth = ($(chart.svgObject).width() - xOffset) * 0.8;  //calculate a width of a funnel chart when smartlabel is enabled
             chartModel.resWidth = ($(chart.svgObject).width() - xOffset - chartModel.actualWidth) / 2;
         }
         else {
             chartModel.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
             chartModel.resWidth = 0;
         }
         var subtitleHeight = (chart.model.title.subTitle.text == '') ? 0 : measureSubTitleHeight;
         chartModel.actualHeight = $(chart.svgObject).height() - yOffset;
         var pyrX = chart.pyrX = ((legendPosition === "left") ? (legendActualBounds.Width + (2 * legendWidth)) : 0) + chartModel.elementSpacing + chartModel.margin.left;
         var pyrY = chart.pyrY = ((legendPosition === "top") ? (legendActualBounds.Height + (2 * legendWidth)) : 0) + ((chartModel.title.text&&chartModel.title.visible) ? (chartModel._titleLocation.Y + measureTitleHeight + subtitleHeight) : (chartModel.margin.top + chartModel.elementSpacing));
         translate[0] = pyrX;
         translate[1] = pyrY;
         for (var j = 0; j < visiblePointslength; j++) {
             chartModel.sumofYValues += visiblePoints[j].YValues[0];
         }


         var numberToFixed = ej.util.isNullOrUndefined(chartModel.roundingPlaces) ? 2 : chartModel.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, chartModel._visibleSeries);
         var serOptions = { 'id': chart.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         this.gSeriesGroupEle = chart.svgRenderer.createGroup(serOptions);
         var dataLabelOptions = { 'id': chart.svgObject.id + '_DataLabel_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gDataLabelEle = [];
            chart.gDataLabelEle[seriesIndex] = chart.svgRenderer.createGroup(dataLabelOptions);


            var txtOptions = { 'id': chart.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gSeriesTextEle = [];
            chart.gSeriesTextEle[seriesIndex] = chart.svgRenderer.createGroup(txtOptions);

            var symbolOptions = { 'id': chart.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gSymbolGroupEle = [];
            chart.gSymbolGroupEle[seriesIndex] = chart.svgRenderer.createGroup(symbolOptions);

            var connectorOptions = { 'id': chart.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
            chart.gConnectorEle = [];
            chart.gConnectorEle[seriesIndex] = chart.svgRenderer.createGroup(connectorOptions);

         var currY = 0;
         var model = chartModel;
         model.funnelData = [];
         var gapRatio = currentseries.gapRatio >= 0 ? (currentseries.gapRatio <= 1 ? currentseries.gapRatio : 1) : 0;
         var coef = 1 / (model.sumofYValues * (1 + gapRatio / (1 - gapRatio)));
          var spacing = gapRatio / visiblePointslength;
         for (var i = visiblePointslength - 1; i >= 0; i--) {
             if (currentseries._visiblePoints[i].visible || currentseries._visiblePoints[i].gapMode) {
                 var height = coef * currentseries._visiblePoints[i].YValues[0];
                 model.funnelData[i] = { CurrY: currY, Height: height };
                 currY += height + spacing;
             }
         }
         for (j = 0; j < visiblePointslength  ; j++) {
             var pointIndex = j;
             point = currentseries._visiblePoints[j];
             if (point.visible) {
                 var result = this._getFunnelData(currentseries, pointIndex, chart);

                 var styleOptions = chart.setStyle(this, currentseries, 0, pointIndex);

                 options = {
                     'id': chart.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'stroke': styleOptions.borderColor,
                     'stroke-dasharray': styleOptions.dashArray,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'opacity': styleOptions.opacity,
                     'pointIndex': pointIndex,
                     'd': result.Direction
                 };
                 chart.svgRenderer.drawPath(options, this.gSeriesGroupEle, translate);

                 if (currentseries.marker.dataLabel.template) {
                     ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, chart);
                 }

                 point.xLocation = result.PositionX;
                 point.yLocation = result.PositionY;
                 point.connectorLine = result.Connector;
                 point.startX = result.startX;
                 point.startY = result.startY;
                 point.Polygon = result.Polygon;
                 region = { PointIndex: pointIndex, Line1: result.Line1, Line2: result.Line2, Line3: result.Line3, Line4: result.Line4, Line5: result.Line5, Line6: result.Line6, Polygon: result.Polygon };
                 internalRegion.push(region);
             }
         }
         var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, chartModel._visibleSeries) };
         chartModel.chartRegions.push(region);

         if(this.gSeriesGroupEle)
         $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
             evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
             seriesRender.chartObj.chartInteractiveBehavior(evt);
         });

         chart.svgRenderer.append(this.gSeriesGroupEle, chart.gSeriesEle);


     },

     _getFunnelData: function (series, index, chart) {

         var model = this.chartObj.model,
             point = series._visiblePoints[index],
             pointMarker = point.marker,
             seriesMarer = series.marker,
             funnelWidth = series.funnelWidth,
             funnelHeight = series.funnelHeight,
             actualHeight = model.actualHeight,
             actualWidth = model.actualWidth,
             elementSpacing = model.elementSpacing,
             pyrX, legendPosition = model.legend.position.toLowerCase(),
             actualIndex = point.actualIndex,
             direction,
             resWidth = model.resWidth, polygon,
             textOffset, connectorDirection,
             offset = ((series.explodeIndex == actualIndex) || (series.explodeAll) ? series.explodeOffset : 0),
             topRadius, minRadius, bottomRadius, endTop,
             endMin, endBottom, top, bottomY,
             bottom, connectorX, connectorY,
             positionX, positionY, startX, startY,
             line1, line2, line3, line4, line5, line6,
             lineWidth, minimumHeight, minimumWidth;
         if (funnelHeight.indexOf("%") != -1)
             minimumHeight = actualHeight * (parseInt(funnelHeight) / 100);
         else
             minimumHeight = parseInt(funnelHeight);
         if (funnelWidth.indexOf("%") != -1)
             minimumWidth = actualWidth * (parseInt(funnelWidth) / 100);
         else
             minimumWidth = parseInt(funnelWidth);

         top = model.funnelData[index].CurrY * actualHeight;
         bottom = top + model.funnelData[index].Height * actualHeight;
         lineWidth = minimumWidth + (actualWidth - minimumWidth) * ((actualHeight - minimumHeight - top) / (actualHeight - minimumHeight));
         topRadius = (actualWidth / 2) - lineWidth / 2;
         endTop = topRadius + lineWidth;
         if (bottom > actualHeight - minimumHeight || actualHeight == minimumHeight)
             lineWidth= minimumWidth;
         else
             lineWidth = minimumWidth + (actualWidth - minimumWidth) * ((actualHeight - minimumHeight - bottom) / (actualHeight - minimumHeight));
         bottomRadius = (actualWidth / 2) - (lineWidth / 2);
         endBottom = bottomRadius + lineWidth;
         if (top >= actualHeight - minimumHeight) {
             topRadius = bottomRadius = minRadius = (actualWidth / 2) - minimumWidth / 2;
             endTop = endBottom = endMin = (actualWidth / 2) + minimumWidth / 2;

         } else if (bottom > (actualHeight - minimumHeight)) {
             minRadius = bottomRadius = (actualWidth / 2) - lineWidth / 2;
             endMin = endBottom = minRadius + lineWidth;
             bottomY = actualHeight - minimumHeight;
         }

         line1 = { x: resWidth + offset + topRadius, y: top };
         line2 = { x: resWidth + offset + endTop, y: top };
         line4 = { x: resWidth + offset + endBottom, y: bottom };
         line5 = { x: resWidth + offset + bottomRadius, y: bottom };
         line3 = { x: resWidth + offset + endBottom, y: bottom };
         line6 = { x: resWidth + offset + bottomRadius, y: bottom };
         if (bottomY) {
             line3 = { x: resWidth + offset + endMin, y: bottomY };
             line6 = { x: resWidth + offset + minRadius, y: bottomY };
         }
         direction = "M" + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "L" + " " + (line5.x) + " " + (line5.y) + " " + "L" + " " + (line6.x) + " " + (line6.y) + " " + "z";
         polygon = [
           { x: line1.x, y: line1.y },
           { x: line2.x, y: line2.y },
           { x: line3.x, y: line3.y },
           { x: line4.x, y: line4.y },
           { x: line5.x, y: line5.y },
           { x: line6.x, y: line6.y }
            ];
         if ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && seriesMarer.dataLabel.visible) {
             positionX;
             positionY;
             startX = (line2.x + line3.x) / 2;
             startY = (line2.y + line3.y) / 2;   
             if (series.labelPosition.toLowerCase() === 'outside' ) {

                 positionX = (line2.x + line3.x) / 2;
                 positionY = (line2.y + line3.y) / 2;

                 pyrX = ((legendPosition == "left" || legendPosition == "right") ? (model.LegendActualBounds.Width) : 0) + elementSpacing + model.margin.left;

                 if (ej.util.isNullOrUndefined(seriesMarer.dataLabel.connectorLine.height)) {
                     if (series._enableSmartLabels)
                         textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + resWidth + (2 * elementSpacing));
                     else
                         textOffset = $(this.chartObj.svgObject).width() - (positionX + model.textSize + elementSpacing);
                 }
                 else
                     textOffset = seriesMarer.dataLabel.connectorLine.height;

                 connectorX = this.getXCordinate(positionX, textOffset, 0);
                 connectorY = this.getYCordinate(positionY, textOffset, 0);

                 connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                 positionX = connectorX;
                 positionY = connectorY;


             }
             else {
                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line1.y + line4.y) / 2;
             }
         }
         return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection, Line1: line1, Line2: line2, Line3: line3, Line4: line4, Line5: line5, Line6: line6, startX: startX, startY: startY, Polygon: polygon };
     },

     
     chartAreaType: "None"
 });

 ej.seriesTypes.funnel = ej.ejFunnelSeries;
    /*Funnel series*/

 ej.ejPolarSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._isVisiblePoints(currentseries);
         var internalRegion = [];
        

         var style = this.setLineSeriesStyle(currentseries);
        
         if (currentseries.drawType.toLowerCase() == "column") {
             var range=currentseries.xAxis.visibleRange;
			 range.interval = 1;
             var length = (currentseries.xAxis._valueType == "category") ? range.max : range.max - 1;
             this.chartObj.model.sumofYValues = 0;
             var count = range.interval;
             var min = range.min;
             do{
                 this.chartObj.model.sumofYValues += range.interval;
                 min += range.interval;
             } while (min <= length)
           //  this.chartObj.model.sumofYValues = visiblePoints.length;
         }

         var direction = this._calculatePolarAxesSegment(currentseries);

         if (currentseries.drawType.toLowerCase() == "area")
             this.drawAreaPath(currentseries, style, direction);
         else if (currentseries.drawType.toLowerCase() == "line")
             this._drawLinePath(currentseries, style, direction);
         
         
         
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

       
     },
 
     chartAreaType: "PolarAxes"

 });

 ej.seriesTypes.polar = ej.ejPolarSeries;
 ej.ejRadarSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._isVisiblePoints(currentseries);
         var internalRegion = [];

         var style = this.setLineSeriesStyle(currentseries);

         if (currentseries.drawType.toLowerCase() == "column") {
             var range = currentseries.xAxis.visibleRange;
			 range.interval = 1;
             var length = (currentseries.xAxis._valueType == "category") ? range.max : range.max - 1;
             this.chartObj.model.sumofYValues = 0;
             var count = range.interval;
             var min = range.min;
             do {
                 this.chartObj.model.sumofYValues += range.interval;
                 min += range.interval;
             } while (min <= length)
             //  this.chartObj.model.sumofYValues = visiblePoints.length;
         }

         var direction = this._calculatePolarAxesSegment(currentseries);

          if (currentseries.drawType.toLowerCase() == "area")
             this.drawAreaPath(currentseries, style, direction);
         else if (currentseries.drawType.toLowerCase() == "line")
             this._drawLinePath(currentseries, style, direction);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

         

     },
     doAnimation: function (series, chart) {     // for polar/radar animation
         var chartObj = chart.chartObj;
         series.regionAdded = true;
         chartObj.model._radius = chartObj.model.Radius;
         chartObj.model.Radius = 0;
         var seriesRender = chart;
         var seriesIndex = $.inArray(series, chartObj.model._visibleSeries);
         var Path = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[0];
         $(Path).animate(
             { Radius: chartObj.model._radius },

             {
                 duration: 1200, queue: false, step: function (now, fx) {
                     if(!ej.util.isNullOrUndefined(chartObj.model)){
                     chartObj.model.Radius = now;

                     var direction = seriesRender._calculatePolarAxesSegment(series);
                     if (series.drawType.toLowerCase() != 'column')
                         chartObj.svgRenderer._setAttr($(Path), { "d": direction });
                     chartObj.model.Radius = chartObj.model._radius;
                     }
                     else
                         $(Path).stop(true,true);
                 },
                 complete: function () {
                     if(!ej.util.isNullOrUndefined(chartObj.model)){
                     series.regionAdded = false;                    
                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex), { "visibility": "visible" });
                   
                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex), { "visibility": "visible" });
					chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_DataLabel' + '_' + seriesIndex), { "visibility": "visible" });
                     chartObj.model.AnimationComplete = true;
                     var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                     commonEventArgs.data = { series: series };
                     chartObj._trigger("animationComplete", commonEventArgs);
                     }
                 }

             });
     },
     chartAreaType: "PolarAxes"

 });

 ej.seriesTypes.radar = ej.ejRadarSeries;
})(jQuery);;
ej.ejChart = {};
(function($) {
String.prototype.parseTemplate = function () {
    var str = this;
    $.each(arguments[0], function (dataIndex, dataValue) {
        if (dataValue.count > 0) {
            $.each(dataValue, function (datachildIndex, datachildValue) {
                var val = new RegExp('#' + dataIndex + '.' + datachildIndex + '#', 'gm')
                str = str.replace(val.source, datachildValue);
            });
        } else {
            str = str.replace(new RegExp('\\#' + dataIndex + '\\#', 'gm'), dataValue);
        }

    });
	while (str.indexOf('ej.') >= 0) {
        substr = str.substring(str.indexOf('ej.'), str.indexOf(")") + 1);
        str = str.replace(substr, eval(substr));
    }
    return str;
};

String.prototype.capitalizeFirstString = (function () {
    return this.charAt(0).toUpperCase() + this.slice(1);
});

var Gradient = function (colors) {
    this._gradientStop = [];
    for (var j = 0; j < colors.length; j++) {
        this._gradientStop.push(colors[j]);
    }
};



 ej.ejChart = {

    _renderSfChart: function () {

        var svgSupport = (window.SVGSVGElement) ? true : false;
        var isCanvas = this.model.enableCanvasRendering;
        $("#chartContainer_" + this._id).remove();
        if (svgSupport) {
            this.renderer = new ej.EjSvgRender(this.element);
            if (isCanvas) {
                this.svgRenderer = new ej.EjCanvasRender(this.element);
                this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
            } else {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
            }
            this._appendChartElements();
            if (!isCanvas)
                this.legendContainer.find("svg").attr("class", "e-designerhide");
            this._createChart();
            $(this.svgObject).appendTo(this.element);
        }
        else {
            isCanvas = false;
            this.svgRenderer = new ej.EjVmlRender(this.element);
            this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new div element for legend
            this.renderer = new ej.EjVmlRender(this.element);
            this._appendChartElements();
            this._createChart();
            $(this.svgObject).appendTo(this.chartContainer);
        }
        return true;
    },

    _appendChartElements: function () {
        this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
        this.scrollerContainer = $("<div></div>").attr('id', "legend_Scroller" + this._id).css("height", "0px");
        this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
        this.axisScroll = $("<div></div>").attr('id', "axisScrollbar_" + this._id).css("height", "0px");
        this.svgRenderer.append(this.legendSvgContainer, this.scrollerContainer);
        $(this.scrollerContainer).appendTo(this.legendContainer);
        $(this.axisScroll).appendTo(this.chartContainer);
        $(this.legendContainer).appendTo(this.chartContainer);
        $(this.chartContainer).appendTo(this.element);
    },

 setSvgSize: function (sender) {
            var chartObj = sender;
            var containerHeight = $(chartObj.element).height();
            var height = 450; //set default height if size not specified to chart or container
            var width =(ej.isTouchDevice()) ? 250: 600;
            var containerWidth = $(chartObj.element).width();
            var legendcontainerHeight = $(chartObj.chartContainer).height();
            if (chartObj.model.size.width) {
                var chartWidth = chartObj.model.size.width;
                if (typeof (chartWidth)== "string" && chartWidth.indexOf("%") != -1)
                    width = (containerWidth / 100) * parseInt(chartWidth);
                else
                    width = parseInt(chartWidth);
            }
            else if (containerWidth > 0)
                width = containerWidth;

            $(chartObj.svgObject).width(width);

            if (chartObj.model.size.height) {
                var chartHeight = chartObj.model.size.height;
                if (typeof (chartHeight) == "string" && chartHeight.indexOf("%") != -1)
                    height = (!this.vmlRendering) ? (containerHeight / 100) * parseInt(chartHeight) : height;
                else
                    height = parseInt(chartHeight);
            }
            else if (containerHeight > 0)
                height = containerHeight;
            else
                $(chartObj.svgObject).css("display", "block"); //Added css to chart due to 5px difference for container and svg (http://jsfiddle.net/Me5Zd/)       
            $(chartObj.svgObject).height(height);
			
			$("#"+chartObj._id).css("overflow", "hidden");//Added css to remove scrollbar on zooming the chart
			chartObj.svgObject.width = width;   // assigning width for canvas
			chartObj.svgObject.height = height;  // assigning height for 
},
    setModelProperties: function (excludeDataUpdate) {

        //set size for the svgObject
         if (!excludeDataUpdate)
            this.setSvgSize(this);

        this.svgWidth = $(this.svgObject).width();
        this.svgHeight = $(this.svgObject).height();

        this.chartCross = {
            visible: (excludeDataUpdate) ? this.chartCross.visible : this.model.crosshair.visible,
            mArea: this.model.crosshair.type
        };

        // extend series with commonSeriesOptions
        this.serAnimation = [];
        if (this.model.series && !excludeDataUpdate) {
            var trendlines, len, currentPoint, currentSeries, nullValue = false, trendLineType;
            this.model._drawTrendline = false;
            for (var j = 0; j < this.model.series.length; j++) {
                this.model.series[j] = ej.copyObject({}, this.model.commonSeriesOptions, this.model.series[j]);
				 currentSeries = this.model.series[j];
                this.serAnimation.push((currentSeries.enableAnimation === null || currentSeries.enableAnimation === undefined) ? this.model.commonSeriesOptions.enableAnimation : currentSeries.enableAnimation);
                    trendlines = currentSeries.trendlines;
                    len = trendlines.length;
                    for (var i = 0; i < len; i++) {
                        trendlines[i] = ej.copyObject({}, this.model.trendlineDefaults, trendlines[i]);
						trendlines[i]._visibility = trendlines[i].visibility;
                        if (trendlines[i].visibility.toLowerCase() == "visible"){
						    this.model._drawTrendline = true;
						    trendLineType = trendlines[i].type.toLowerCase();
							if (!ej.util.isNullOrUndefined(currentSeries.points)){
								for (var l= 0; l< currentSeries.points.length; l++){
									currentPoint = currentSeries.points[l];
									if (ej.util.isNullOrUndefined(currentPoint.y) || currentPoint.y == 0){
										nullValue = true; 
										break;
									}
								}
							}
						    trendlines[i].isNull  = nullValue && (trendLineType == "power" || trendLineType == "exponential") ? true: false;
						}
            }
          }
        }
        if (this.model.indicators) {
            for (var k = 0; k < this.model.indicators.length; k++) {
                this.model.indicators[k] = ej.copyObject({}, this.model.indicatorDefaults, this.model.indicators[k]);
            }
        }

    },

    _createChart: function () {
        this.svgObject = this.svgRenderer.svgObj;
        var commonlaodEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        this._trigger("load", commonlaodEventArgs);
        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
        var model = this.model;
  
        if (model.series[0] && model.series[0]._isdesigntime) {
            model.series[0] = {};
            model.series[0].points = [];
            var randomPoints = [];
            for (var i = 1; i <= 10; i++) {
                randomPoints.push({ x: i, y: (Math.floor(Math.random() * (10 - 40)) + 40) });

            }
            model.series[0].points = randomPoints;
        }
		this.vmlRendering = (this.svgRenderer.vmlNamespace) ? true : false;
        if ($(this.element).is(":visible") || $(this.element).css("visibility") != "hidden" || $(this.element).css('display') != 'none') {
            this.bindEvents();
            this.bindTo();
        }
    },
    setTheme: function (jsonObj, themeName) {
        var name = themeName.toLowerCase();
        if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") == -1) && (name.indexOf("dark") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["flatlight"], jsonObj[name]);
        else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark")) >= 0 && (name.indexOf("gradient") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["flatdark"], jsonObj[name]);
        else if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") >= 0) && (name.indexOf("dark") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["gradientlight"], jsonObj[name]);
        else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark") >= 0) && (name.indexOf("gradient") >= 0))
            jsonObj[name] = $.extend(true, {}, jsonObj["gradientdark"], jsonObj[name]);
        else if ((name.indexOf("high") >= 0))
            jsonObj[name] = $.extend(true, {}, jsonObj["highcontrast01"], jsonObj[name]);
        else if ((name.indexOf("material") >= 0) || (name.indexOf("office") >= 0))
            jsonObj[name] = $.extend(true, {}, jsonObj["material"], jsonObj[name]);
        else if (!(jsonObj[name])) name = "flatlight";

        if (this.model._themeChanged)
            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        else
            this.model = ej.copyObject({}, jsonObj[name], this.model);


    },
    _drawBackInterior: function () {
        if (this.model.backInterior && this.model.backInterior._gradientStop) {
            var options = {
                'id': this.svgObject.id + '_backGradient',
                'x1': '0%',
                'y1': '0%',
                'x2': '0%',
                'y2': $(this.svgObject).height() + '%'
            };
            this.svgRenderer.drawGradient(options, this.model.backInterior._gradientStop);
        }
    },
    


    _drawBackImage: function () {
        var options = {
            'height': $(this.svgObject).height(),
            'width': $(this.svgObject).width(),
            'href': this.model.backGroundImageUrl,
            'x': 0,
            'y': 0,
            'id': this.svgObject.id + '_backImage',
            'visibility': 'visible',
            'preserveAspectRatio': 'none'
        };

        this.svgRenderer.drawImage(options, this.svgObject);

    },
    _getDataType: function (val) {
        var type = typeof (val);
        if (type === "number" || type === "string")
            return type;
        else if (jQuery.type(val) == "date")
            return "date";
        else if (val == null) {
            return null;
        }
        else
            return false;

    },
    addedXYValues: function (excludeDataUpdate) {
        excludeDataUpdate = (excludeDataUpdate) ? excludeDataUpdate : false;
        var type = this.model.series[0].type.toLowerCase(),
            pointLength;
        if (ej.seriesTypes[type].prototype.chartAreaType == "cartesianAxes") {
            this.model.AreaType = "cartesianaxes";
        }
        else if (ej.seriesTypes[type].prototype.chartAreaType == "None")
            this.model.AreaType = "none";
        else {
            if (type == 'polar')
                this.model.isPolar = true;
            else
                this.model.isPolar = false;

            this.model.AreaType = "polaraxes";
        }
        if (!excludeDataUpdate) {
            for (var i = 0; i < this.model.series.length; i++) {
                if (this.model.series[i].visibility == "visible") {
                    var setXValueType = false, isDatetime = false, pointX, currentSeries;

                    if (!this.model.series[i].dataSource || this.model.series[i].dataSource.length == 0) {
                        this.model.series[i] = $.extend(true, {}, this.model.commonSeriesOptions, this.model.series[i]);
                        currentSeries = this.model.series[i];
                        if (this.model.primaryXAxis.valueType == "datetime")
                            isDatetime = true;
                        pointLength = currentSeries.points.length;
                        for (k = 0; k < pointLength; k++) {
                            currentSeries.points[k].YValues = [];
                            if (!isNaN(parseFloat(currentSeries.points[k].x)) && (/^-?[0-9]*\.?[0-9]+$/.test(currentSeries.points[k].x) == true) && (this.model.primaryXAxis.valueType != "category") && (this.model.AreaType != "none")) {
                                if (!isDatetime) {
                                    currentSeries.points[k].xValue = parseFloat(currentSeries.points[k].x);
                                    currentSeries.points[k].x = parseFloat(currentSeries.points[k].x);
                                }
                                else {
                                    currentSeries.points[k].xValue = new Date(currentSeries.points[k].x);
                                    currentSeries.points[k].x = new Date(currentSeries.points[k].x);
                                }
                            }
                            else
                                currentSeries.points[k].xValue = currentSeries.points[k].x;
                            if (!setXValueType) {
                                pointX = currentSeries.points[k].x;
                                if (!ej.util.isNullOrUndefined(pointX)) {
                                    currentSeries._xAxisValueType = this._getDataType(pointX);
                                    setXValueType = true;
                                }
                            }
                            if (!(currentSeries._hiloTypes)) {
                                currentSeries.points[k].YValues[0] = currentSeries.points[k].y;

                                if (!ej.util.isNullOrUndefined(currentSeries.points[k].size))
                                    currentSeries.points[k].YValues[1] = currentSeries.points[k].size;

                            }
                            else {
                                if (!ej.util.isNullOrUndefined(currentSeries.points[k].high)) {
                                    currentSeries.points[k].YValues[0] = currentSeries.points[k].high;
                                    currentSeries.points[k].y = currentSeries.points[k].high;
                                }
                                if (!ej.util.isNullOrUndefined(currentSeries.points[k].low))
                                    currentSeries.points[k].YValues[1] = currentSeries.points[k].low;
                                if (!ej.util.isNullOrUndefined(currentSeries.points[k].open))
                                    currentSeries.points[k].YValues[2] = currentSeries.points[k].open;
                                if (!ej.util.isNullOrUndefined(currentSeries.points[k].close))
                                    currentSeries.points[k].YValues[3] = currentSeries.points[k].close;
                            }
                        }
                    }
                    else {
                        currentSeries = this.model.series[i];
                        currentSeries._xAxisValueType = null;
                        pointLength = currentSeries.points.length;
                        for (var j = 0, length = pointLength; j < length; j++) {
                            pointX = currentSeries.points[j].x;
                            if ((pointX != null && pointX != undefined) && (currentSeries._xAxisValueType === null || currentSeries._xAxisValueType === undefined)) {
                                currentSeries._xAxisValueType = this._getDataType(pointX);
                            }
                            currentSeries.points[j].xValue = currentSeries.points[j].x;
                        }
                    }
                    //Calculate waterfall series points and remove on series changed
                    if (currentSeries.type.toLowerCase() == "waterfall")
                        this.calculateWaterfallSeriesPoints(currentSeries.points);
                    else if (currentSeries._previousType && currentSeries._previousType.toLowerCase() == "waterfall") {
                        var visiblePoints = currentSeries.points;
                        for (var j = 0; j < visiblePoints.length; j++) {
                            visiblePoints[j].waterfallSum = undefined;
                            visiblePoints[j].textOptions = null;
                        }
                    }
                }
            }
           this._setInternalValues();
         }
    },
    calculateWaterfallSeriesPoints: function (visiblePoints) {
        
        var intermediateStartValue = 0;
        var totalSum = 0;
        var previousSum = 0;

        for (var j = 0; j < visiblePoints.length; j++) {
            if (!visiblePoints[j].isEmpty) {
                totalSum += ((visiblePoints[j].showIntermediateSum || visiblePoints[j].showTotalSum) ? 0 : visiblePoints[j].y);

                if (visiblePoints[j].showIntermediateSum) {
                    visiblePoints[j].YValues[1] = intermediateStartValue;
                    visiblePoints[j].YValues[0] = totalSum;
                    visiblePoints[j].waterfallSum = totalSum - intermediateStartValue;
                    intermediateStartValue = totalSum;
                } else if (visiblePoints[j].showTotalSum) {
                    visiblePoints[j].YValues[1] = 0;
                    visiblePoints[j].YValues[0] = totalSum;
                    visiblePoints[j].waterfallSum = totalSum;
                }
                else {
                    visiblePoints[j].YValues[1] = previousSum;
                    visiblePoints[j].YValues[0] = previousSum + visiblePoints[j].y;
                    previousSum = totalSum;
                }
            } else {
                visiblePoints[j].YValues[1] = 0;
                visiblePoints[j].YValues[0] = 0;
            }
        }
    },
    _setInternalValues: function () {
        
        for (var k = 0; k < this.model.series.length; k++) {
            var series = this.model.series[k];
            if (this.model.AreaType == "cartesianaxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "cartesianAxes") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }
            else if (this.model.AreaType == "none" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "None") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                var type = this.model.series[k].type.toLowerCase();
                if (type == 'funnel' || type == 'pyramid') {
                    this.model._visibleSeries.push(series);
                    break;
                }
                else
                {
                    this.model._visibleSeries.push(series);
                }
            }
            else if (this.model.AreaType == "polaraxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "PolarAxes") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }

            if (series._yAxisName || series._xAxisName) {
                if (this.model._axes) {
                    for (var t = 0; t < this.model._axes.length; t++) {
                        if (series._yAxisName && (series._yAxisName == this.model._axes[t].name)) {
                            this.model._axes[t].orientation = (this.model._axes[t].orientation) ? this.model._axes[t].orientation : (this.model.requireInvertedAxes) ? "horizontal" : "vertical";
                        }
                        if (series._xAxisName && (series._xAxisName == this.model._axes[t].name)) {
                            this.model._axes[t].orientation = (this.model._axes[t].orientation) ? this.model._axes[t].orientation : (this.model.requireInvertedAxes) ? "vertical" : "horizontal";
                        }
                    }
                }
            }
        }
    },
    setHiloStyle: function(series, pointIndex, seriesIndex, interiorColor){
        var point = series.points[pointIndex];
                  
        if (point.fill)
              interior = point.fill;
         else
              interior = series.fill ? series.fill : interiorColor;
				
            // setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];
          
            // applying gradient color		  
         
            seriesInterior = jQuery.type(colors) == "array" ? colors[0].color : colors;

            return seriesInterior;
                       
    },
	setStyle: function (chart, series, seriesIndex, pointIndex, interiorColor) {
    
	    var point = series._visiblePoints[pointIndex], interior, borderColor, borderWidth, dashArray, 
            opacity, seriesFill, pointBorder = point.border, seriesBorder = series.border;
       // var pointStyle = point.style ? point.style : " ";
      
		if (series._hiloTypes)
		   seriesFill = series.isFill ? series.fill : null; 
		   
        // setting interior
        if (chart.chartObj.model.AreaType == "cartesianaxes") {
            if (point.fill)
                interior = point.fill;
            else {
                if (series.type.toLowerCase() == "waterfall") {
                    if (((point.showIntermediateSum || point.showTotalSum) && (point.waterfallSum > 0)) || (point.y > 0) && !point.showIntermediateSum && !point.showTotalSum)
                        interior = series.positiveFill ? series.positiveFill : interiorColor;
                } else
                    interior = seriesFill ? seriesFill : interiorColor;
            }
				
		// setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];
          
        // applying gradient color		  
			if (chart.chartObj.model.requireInvertedAxes) 
                var seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 100, 0, chart.gSeriesGroupEle);
            else                                       
                seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 0, 100, chart.gSeriesGroupEle);
         } 
		 
		// interior for pie type series
        else {
            var pointColors = this.model.pointColors;
            interior = point.fill ? point.fill : jQuery.type(pointColors[pointIndex]) == "array" ? pointColors[pointIndex][0].color : pointColors[pointIndex];
            var pointInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + pointIndex, interior, 0, 0, 0, ($(chart.svgObject).height() || parseFloat(this.model.size.height)), chart.chartObj.gSeriesEle);
        }
        
        // setting borderColor
        borderColor = ((pointBorder) && pointBorder.color) ? pointBorder.color : seriesBorder.color;
        borderWidth = ((pointBorder) && pointBorder.width) ? pointBorder.width : seriesBorder.width;
        opacity = point.opacity ? point.opacity : series.opacity;
        width = (point.width) ? point.width : series.width;
        dashArray = ((pointBorder) && pointBorder.dashArray) ? pointBorder.dashArray : seriesBorder.dashArray;

        var options = {
            'interior': seriesInterior ? seriesInterior: pointInterior,
            'borderColor': borderColor,
            'borderWidth': borderWidth,
            'width': width,
            'opacity': opacity,
            'dashArray': dashArray
        };
        return options;
    },
	
    colorNameToHex: function (colour) {
        var color = colour;
        var colours = {
            "aliceblue": "#f0f8ff", "antiquewhite": "#faebd7", "aqua": "#00ffff", "aquamarine": "#7fffd4", "azure": "#f0ffff",
            "beige": "#f5f5dc", "bisque": "#ffe4c4", "black": "#000000", "blanchedalmond": "#ffebcd", "blue": "#0000ff", "blueviolet": "#8a2be2", "brown": "#a52a2a", "burlywood": "#deb887",
            "cadetblue": "#5f9ea0", "chartreuse": "#7fff00", "chocolate": "#d2691e", "coral": "#ff7f50", "cornflowerblue": "#6495ed", "cornsilk": "#fff8dc", "crimson": "#dc143c", "cyan": "#00ffff",
            "darkblue": "#00008b", "darkcyan": "#008b8b", "darkgoldenrod": "#b8860b", "darkgray": "#a9a9a9", "darkgreen": "#006400", "darkkhaki": "#bdb76b", "darkmagenta": "#8b008b", "darkolivegreen": "#556b2f",
            "darkorange": "#ff8c00", "darkorchid": "#9932cc", "darkred": "#8b0000", "darksalmon": "#e9967a", "darkseagreen": "#8fbc8f", "darkslateblue": "#483d8b", "darkslategray": "#2f4f4f", "darkturquoise": "#00ced1",
            "darkviolet": "#9400d3", "deeppink": "#ff1493", "deepskyblue": "#00bfff", "dimgray": "#696969", "dodgerblue": "#1e90ff",
            "firebrick": "#b22222", "floralwhite": "#fffaf0", "forestgreen": "#228b22", "fuchsia": "#ff00ff",
            "gainsboro": "#dcdcdc", "ghostwhite": "#f8f8ff", "gold": "#ffd700", "goldenrod": "#daa520", "gray": "#808080", "green": "#008000", "greenyellow": "#adff2f",
            "honeydew": "#f0fff0", "hotpink": "#ff69b4",
            "indianred ": "#cd5c5c", "indigo ": "#4b0082", "ivory": "#fffff0", "khaki": "#f0e68c",
            "lavender": "#e6e6fa", "lavenderblush": "#fff0f5", "lawngreen": "#7cfc00", "lemonchiffon": "#fffacd", "lightblue": "#add8e6", "lightcoral": "#f08080", "lightcyan": "#e0ffff", "lightgoldenrodyellow": "#fafad2",
            "lightgrey": "#d3d3d3", "lightgreen": "#90ee90", "lightpink": "#ffb6c1", "lightsalmon": "#ffa07a", "lightseagreen": "#20b2aa", "lightskyblue": "#87cefa", "lightslategray": "#778899", "lightsteelblue": "#b0c4de",
            "lightyellow": "#ffffe0", "lime": "#00ff00", "limegreen": "#32cd32", "linen": "#faf0e6",
            "magenta": "#ff00ff", "maroon": "#800000", "mediumaquamarine": "#66cdaa", "mediumblue": "#0000cd", "mediumorchid": "#ba55d3", "mediumpurple": "#9370d8", "mediumseagreen": "#3cb371", "mediumslateblue": "#7b68ee",
            "mediumspringgreen": "#00fa9a", "mediumturquoise": "#48d1cc", "mediumvioletred": "#c71585", "midnightblue": "#191970", "mintcream": "#f5fffa", "mistyrose": "#ffe4e1", "moccasin": "#ffe4b5",
            "navajowhite": "#ffdead", "navy": "#000080",
            "oldlace": "#fdf5e6", "olive": "#808000", "olivedrab": "#6b8e23", "orange": "#ffa500", "orangered": "#ff4500", "orchid": "#da70d6",
            "palegoldenrod": "#eee8aa", "palegreen": "#98fb98", "paleturquoise": "#afeeee", "palevioletred": "#d87093", "papayawhip": "#ffefd5", "peachpuff": "#ffdab9", "peru": "#cd853f", "pink": "#ffc0cb", "plum": "#dda0dd", "powderblue": "#b0e0e6", "purple": "#800080",
            "red": "#ff0000", "rosybrown": "#bc8f8f", "royalblue": "#4169e1",
            "saddlebrown": "#8b4513", "salmon": "#fa8072", "sandybrown": "#f4a460", "seagreen": "#2e8b57", "seashell": "#fff5ee", "sienna": "#a0522d", "silver": "#c0c0c0", "skyblue": "#87ceeb", "slateblue": "#6a5acd", "slategray": "#708090", "snow": "#fffafa", "springgreen": "#00ff7f", "steelblue": "#4682b4",
            "tan": "#d2b48c", "teal": "#008080", "thistle": "#d8bfd8", "tomato": "#ff6347", "turquoise": "#40e0d0",
            "violet": "#ee82ee",
            "wheat": "#f5deb3", "white": "#ffffff", "whitesmoke": "#f5f5f5",
            "yellow": "#ffff00", "yellowgreen": "#9acd32"
        };

        if (Object.prototype.toString.call(color) == '[object Array]')
            return color;

        if (typeof colours[color.toLowerCase()] != 'undefined')
            return colours[color.toLowerCase()];

        return color;
    },
	
	_getLocalizedLabels: function(){
        return ej.getLocalizedConstants(this.sfType, this.model.locale);
    },
	
	// set culture for chart
     _setCulture: function (culture) {
         this.culture = ej.preferredCulture(culture);
    },
	
	
     draw: function (excludeDataUpdate) {
        this.removeMultiRect();
        var params = {};
		params.axes = {};		
		this._drawBackInterior();
        this.model.chartRegions = [];         
		this.model.outsideDataRegionPoints = [];
		this.model.regionCount = null;
		this.model.circularRadius = [];
       this.model.innerRadius = [];
       this.model.bounds = [];
       this.accDataLabelRegion = [];
       this.model.rightsidebounds = [];
		this.model.leftsidebounds = [];
        this.model.excludeDataUpdate = excludeDataUpdate;
		this.model.yAxisLabelRegions = [];
		this.model.xAxisLabelRegions = [];
		this.model.axisMultiLevelLabelRegions = [];
		this.model.multiLevelLabelRegions = [];
		this.model.minhightwidth = false;
		this.model._isPieOfPie = false;
        if (!excludeDataUpdate)
            this.model._visibleSeries = [];
        this.model.stackedValue = {};
        this.model._locale = this.model.locale || "en-US";      
        this._setCulture(this.model._locale);
		 this._localizedLabels = this._getLocalizedLabels();
		var seriesLength = this.model.series.length;
		 var series, seriesType;
        if (ej.util.isNullOrUndefined(this.model.sideBySideSeriesPlacement))  // for setting sideBySideSeriesPlacement property values
            this.model._sideBySideSeriesPlacement = (this.model.enable3D) ? false : true;
        else
            this.model._sideBySideSeriesPlacement = this.model.sideBySideSeriesPlacement;
        for (var i = 0; i < seriesLength; i++)
        {
            series = this.model.series[i];
            seriesType = series.type.toLowerCase();
            series._isTransposed = (seriesType.indexOf("bar") == -1) ? series.isTransposed : !series.isTransposed;
        }
        if (seriesLength > 0)
            this.addedXYValues(excludeDataUpdate);
        this.model.AreaType = this.model.AreaType || "cartesianaxes";
        
		this._isEjScroller();        

        var commonpreRenderEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonpreRenderEventArgs.data = {};
        this._trigger("preRender", commonpreRenderEventArgs);

        this.gLegendEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Legend' });

        this.gTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartTitle','cursor':'default' });
        this.gSubTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartsubTitle','cursor':'default' });
        this._createAxisLabelAndRange();
		for(var i=0; i<this.model._axes.length; i++)
			params.axes[this.model._axes[i].name] = {};
        this._initializeSeriesColors();

        this._calculateLegendBounds();

        this.legendRender = new ej.EjLegendRender(this);

        var commonLegendEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonLegendEventArgs.data = { legendBounds: this.model.LegendBounds };
        this._trigger("legendBoundsCalculate", commonLegendEventArgs);

         // Assigning indicator points
        for (var j = 0; j < this.model.indicators.length; j++) {
            var indicator = this.model.indicators[j];
			indicator._points = [];
            indicator.xAxisName = (!(indicator.xAxisName)) ? this.model._axes[0].name : indicator.xAxisName;
            indicator.yAxisName = (!(indicator.yAxisName)) ? this.model._axes[1].name : indicator.yAxisName;
            indicator.isIndicator = true;
            if (indicator.dataSource) {
                indicator._hiloTypes = true;
                this._processJsonData(indicator.dataSource, indicator);
				//Indicator uses _points but ProcessJsonData only populate points collection
                indicator._points = indicator.points;
            } else if (indicator.points.length > 0) {
                for (var i = 0; i < indicator.points.length; i++) {
				     indicator._points = indicator.points;
                    indicator._points[i].xValue = indicator.points[i].x;
                    indicator._points[i].YValues = [];
                    indicator._points[i].YValues[0] = indicator.points[i].close;
                }
            }
            else if (indicator.seriesName != "") {
                for (var index = 0; index < this.model._visibleSeries.length; index++) {
                    var series = this.model._visibleSeries[index];
                    if (indicator.seriesName == series.name)
                        indicator._points = $.extend(true, [], series.points);
                }
            }
        }

         // calculate category points
        for (k = 0; k < this.model._axes.length; k++) {
            if (this.model._axes[k].valueType && this.model._axes[k].valueType.toLowerCase() == "category") {
                for (i = 0; i < this.model.indicators.length; i++) {
                    indicator = this.model.indicators[i];
                    if (this.model._axes[k].name == indicator.xAxisName) {
                        for (j = 0; j < indicator._points.length; j++)
                            indicator._points[j].xValue = j;
                    }
                }
            }
        }

         // calculate ranges for indicator
        this.model.indicatorRange = [];
        for (index = 0; index < this.model.indicators.length; index++) {
            indicator = this.model.indicators[index];
            if (indicator._points.length > 0) {
                type = indicator.type.toLowerCase();
              options = new ej.indicatorTypes[type]();
              options.calculateSegment(indicator, this);
         }
     }


         for (var m = 0; m < this.model._visibleSeries.length; m++) {
            var options = this.model._visibleSeries[m];
            for (i = 0; i < options.points.length; i++) {
                if (typeof options.points[i].xValue == "string" && options.points[i].xValue.indexOf("/Date(") != -1)
                    options.points[i].xValue = new Date(parseInt(options.points[i].xValue.substr(6)));
            }
        }
        this.model.hAxes = [];
        this.model.vAxes = [];
        var chartobj = this;
        var isStriplineOver = false, isStriplineBehind = false;
        this._arrangeAxis();
        var scrollbarSettings;
        this.model.crosshairLabelVisibility = false;
        for (var k = 0; k < this.model._axes.length; k++) {            
            var axis = this.model._axes[k];
            this.model.crosshairLabelVisibility = axis.crosshairLabel.visible || this.model.crosshairLabelVisibility;
            var seriesCollection = [];
            var seriesLength = this.model.series.length;
            var orientation = this.model._axes[k].orientation.toLowerCase();
            var padding = this.model._axes[k].rangePadding.toLowerCase();
            if (padding == "auto") {
                if (orientation == "vertical") {
                    axis.rangePadding = (!this.model.requireInvertedAxes) ? "normal" : "none";
                }
                if (orientation == "horizontal") {
                    axis.rangePadding = (this.model.requireInvertedAxes) ? "normal" : "none";
                }
                for (var j = 0; j < seriesLength; j++) {
                    var type = this.model.series[j].type.toLowerCase();
                    if (type.indexOf("100") != -1) {
                        if (axis.name == this.model.series[j]._yAxisName) {
                            axis.rangePadding = "round"; break;
                        }
                    }
                }
            }          
            for (var slCount = 0; slCount < axis.stripLine.length; slCount++) {
                axis.stripLine[slCount] = $.extend(true, {}, this.model.stripLineDefault, axis.stripLine[slCount]);
                if (axis.stripLine[slCount].zIndex.toLowerCase() == 'over') {
                    isStriplineOver = true;
                } else {
                    isStriplineBehind = true;
                }
            }
            var axisOrientation = axis.orientation.toLowerCase();
            var zOrder = this.model._visibleSeries.length - 1;
            for (j = 0; j < this.model._visibleSeries.length; ++j) {
                zOrder -= 1;
                                      
                    var series = this.model._visibleSeries[j];
                    this.model._hasSeriesPoints = series.points.length > 0 || this.model._hasSeriesPoints;
                    if((!series.xAxisName)){
                    series._xAxisName = chartobj.model._axes[0].name;
					 }
					if(!series.yAxisName){
                    series._yAxisName = chartobj.model._axes[1].name;
                    }
					 series._xAxisName=(!(series._xAxisName))?(!(series.xAxisName)?chartobj.model._axes[0].name:series.xAxisName):series._xAxisName;
                    series._yAxisName = (!(series._yAxisName)) ? (!(series.yAxisName)?chartobj.model._axes[1].name :series.yAxisName):series._yAxisName;
                    var xAxisName = series._xAxisName.toLowerCase();
                    var yAxisName = series._yAxisName.toLowerCase();
                    var name = axis.name.toLowerCase();
                    if (xAxisName || yAxisName) {
                        if (xAxisName == name || yAxisName == name) {
                            if ((axisOrientation == "horizontal" || (series.type.toLowerCase().indexOf("bar") != -1 && axisOrientation == "vertical")) && axis.valueType && axis.valueType.toString() == "datetime") {
                                for (var m = 0; m < series.points.length; m++) {
                                    if (typeof series.points[m].xValue == "string" && !isNaN(Date.parse(series.points[m].xValue)))
                                        series.points[m].xValue = new Date(Date.parse(series.points[m].xValue));
                                }
                            }
                            else {
                                if (series._xAxisValueType == "string" && !(axis.valueType))
                                    axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? 'category' : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'category' : 'double';

                                if (series._xAxisValueType == "date" && !(axis.valueType))
                                    axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? "datetime" : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'datetime' : 'double';
                            }
							if (this.model._visibleSeries[j].visibility.toLowerCase() == 'visible')
                            seriesCollection.push(series);
                        }
                    
                }
            }
            if (axisOrientation == ((!this.model.requireInvertedAxes) ? "vertical" : "horizontal")) {
                var isStacked100 = false;
                var isStacked = false;               
                var seriesLength = seriesCollection.length;
                axis.isStacked100 = (seriesCollection.length>0) ? true :false;
                for (var i = 0; i < seriesLength; i++) {
                    var type = seriesCollection[i].type.toLowerCase();
                    if (ej.seriesTypes[type].prototype.stackingSeries || (this.model.AreaType == 'polaraxes' && seriesCollection[i].drawType.toLowerCase() == 'column' && seriesCollection[i].isStacking)) {
                        if (type.indexOf("100") != -1 && (!isStacked100)) {
                            this._calculateStackingCumulativeValues(seriesCollection, axis, params);
                            isStacked100 = true;
                        } else if (type.indexOf("100") == -1 && (!isStacked)) {
                            this._calculateStackingValues(seriesCollection, axis, params);
                            isStacked = true;
                            axis.isStacked100 = false;
                        }                     
                    }
                    else {
                        axis.isStacked100 = false;
                    }
                }
            }
            scrollbarSettings = axis.scrollbarSettings;
            if ((scrollbarSettings.visible) && (scrollbarSettings.pointsLength != null || scrollbarSettings.range.min != null || scrollbarSettings.range.max != null)) {
                this.model.isLazyZooming = true;
            }
            axis._valueType = (!(axis._valueType)) ? (!(axis.valueType) ? "double" : axis.valueType) : axis._valueType;
            var axisRange = new ej.axisTypes[axis._valueType.toLowerCase()]();
            for (var ser = 0; ser < seriesCollection.length; ser++) {
                for (var trend = 0; trend < seriesCollection[ser].trendlines.length; trend++) {
                    if (seriesCollection[ser].trendlines[trend].visibility == "visible") {
                        this.model._drawTrendline = true;
                        break;
                    }
                }
            }
            axisRange._calculateRanges(chartobj, axis, seriesCollection, params);             
        }
        chartobj._chartResize = false; 
		
        var commonAxesInitEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonAxesInitEventArgs.data = { axes: this.model._axes };
        this._trigger("axesLabelsInitialize", commonAxesInitEventArgs);

        this.model.isLazyZooming ? 0 : this._updateScroll();
         var prevHeight,
             prevWidth;
         if (this.svgHeight != this.model.svgHeight || this.svgWidth != this.model.svgWidth) {
            // beforeResize event
            prevHeight=ej.util.isNullOrUndefined(this.prevHeight)?this.svgHeight:this.prevHeight;
            prevWidth = ej.util.isNullOrUndefined(this.prevWidth) ? this.svgWidth : this.prevWidth;
            commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { currentHeight: prevHeight, currentWidth: prevWidth, newHeight: this.model.svgHeight, newWidth: this.model.svgWidth };
            this._trigger("beforeResize", commonEventArgs);
            if (!commonEventArgs.cancel) {
                if (this.model.enableCanvasRendering) {
                    this.svgRenderer.svgObj.width = commonEventArgs.data.newWidth;
                    this.svgRenderer.svgObj.height = commonEventArgs.data.newHeight;
                }
                this.model.svgWidth = commonEventArgs.data.newWidth;
                this.model.svgHeight = commonEventArgs.data.newHeight;
                $(this.svgObject).width(commonEventArgs.data.newWidth);
                $(this.svgObject).height(commonEventArgs.data.newHeight);
            }
            else
            { 
                if (this.model.enableCanvasRendering)
                {
                    this.svgRenderer.svgObj.width = commonEventArgs.data.currentWidth;
                    this.svgRenderer.svgObj.height = commonEventArgs.data.currentHeight;
                }
                this.model.svgWidth = commonEventArgs.data.currentWidth;
                this.model.svgHeight = commonEventArgs.data.currentHeight;
                $(this.svgObject).width(commonEventArgs.data.currentWidth);
                $(this.svgObject).height(commonEventArgs.data.currentHeight);
            }
        }
        this._calculateAreaBounds(params);

        if (this.model.enable3D) {

            var Ej3DRender = new ej.Ej3DRender();
            this.vector = new Ej3DRender.vector3D();
            this.matrixobj = new Ej3DRender.matrix3D();
            this.bsptreeobj = new Ej3DRender.BSPTreeBuilder();
            this.polygon = new Ej3DRender.polygon3D();
            this.graphics = new Ej3DRender.Graphics3D();

            this.chart3D = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Chart3D' });
            ej.Ej3DRender.Polygons = [];
        }


        if (this.model.AreaType != "none") {        

            this._calculateAxisSize(params);

            //AxisCrossing                
            for (var i = 0, len = this.model._axes.length; i < len; i++)
                if (this._validateCrossing(this.model._axes[i]))
                    this._axisCrossing(this.model._axes[i], false, params);
            if (params._crossAxisOverlap)
			{
                for (var i = 0, len = this.model._axes.length; i < len; i++)
                    if (params.axes[this.model._axes[i].name]._validCross)
                        this._axisCrossing(this.model._axes[i], true, params);
                params._crossAxisOverlap = false;
			}

            var commonAreaEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAreaEventArgs.data = { areaBounds: this.model.m_AreaBounds };
            this._trigger("chartAreaBoundsCalculate", commonAreaEventArgs);

            this._drawChartAreaRect();

            if (this.model.title.text != "" && this.model.title.text && this.model.title.visible)
                this._drawTitle();
            
            this.gXaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxis' });

            this.gYaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxis' });
     
            this.gPolarAxisEle = this.svgRenderer.createGroup( { 'id': this.svgObject.id + '_AxisLine' });
            if (this.model.enable3D) {

                $.extend(ej.Ej3DAxisRenderer.prototype, this);
                this.chartaxis = new ej.Ej3DAxisRenderer(this);
            }
            else {
                $.extend(ej.EjAxisRenderer.prototype, this);
                this.chartaxis = new ej.EjAxisRenderer(this);
            }

            var chartaxis = this.chartaxis;
			this.model.xAxisTitleRegion = [];
            this.model.yAxisTitleRegion = [];
                               
			if (this.model.enable3D) {
			    for (var l = 0; l < this.model._axes.length; l++) {
			        chartaxis._drawAxes(l, this.model._axes[l], params);
			    }
			    ej.Ej3DChart.prototype.update3DWall(this, params);
			}
			else {
			    for (var l = 0; l < this.model._axes.length; l++) {
			        chartaxis._drawGridLines(l, this.model._axes[l], params);
			    }
			}
          

            //Based on the column/bar series, modified primaryAxis append to the SVG Object
            if (!this.model.requireInvertedAxes) { 
                this.svgRenderer.append(this.gXaxisEle, this.svgObject);
                this.svgRenderer.append(this.gYaxisEle, this.svgObject);
            } else {
                this.svgRenderer.append(this.gYaxisEle, this.svgObject);
                this.svgRenderer.append(this.gXaxisEle, this.svgObject);
            }

            var x = (this.model.AreaType == 'polaraxes') ? 0: this.model.m_AreaBounds.X;
            var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;

            var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
            var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

            var clipRectOptions = {
                'id': this.svgObject.id + '_ChartAreaClipRect',
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'Gray'
            };

            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
			
			this._setZoomProperties();
        }

        else {
            this._drawChartAreaRect();

            clipRectOptions = {
                'id': this.svgObject.id + '_ChartAreaClipRect',
                'x': 0,
                'y': 0,
                'width': $(this.svgObject).width(),
                'height': $(this.svgObject).height(),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'Gray'
            };

            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);

            if (this.model.title.text != "" && this.model.title.text && this.model.title.visible)
                this._drawTitle();
        }

        if (isStriplineBehind && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
            this.gStriplineBehind = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineBehind', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
            this._renderStripline('behind');
        }
		if (this.model.initSeriesRender) {
            this.seriesRender(params);
            this.indicatorRender();
        }
         
		var annotationsLength = this.model.annotations.length; 
        $('#annotation_group_' + this._id).empty();
		for (var a = 0; a < annotationsLength; a++) {
            var currentAnnotation = this.model.annotations[a];
            if (currentAnnotation.visible) {
                this.annotationRender(currentAnnotation, a);
              }
        }
		
		if (this.model.AreaType == "polaraxes" && this.model._axes[1].visible && !this.model.enable3D) {
            this.chartaxis._drawAxisLine(this.model._axes[1]);
        }
		
		if(this.model.AreaType==="cartesianaxes"){
			if (!this.model.requireInvertedAxes) {
				this.svgRenderer.append(this.gXaxisEle, this.svgObject);
				this.svgRenderer.append(this.gYaxisEle, this.svgObject);
			} else {
				this.svgRenderer.append(this.gYaxisEle, this.svgObject);
				this.svgRenderer.append(this.gXaxisEle, this.svgObject);
			}

			this.gXaxisTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTick' });

			this.gYaxisTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTick' });
        }      

        $.extend(ej.EjAxisRenderer.prototype, this);
        this.chartaxis = new ej.EjAxisRenderer(this);
        var chartaxis = this.chartaxis;
       
        if (isStriplineOver && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
            this.gStriplineOver = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineOver', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
            this._renderStripline('over');
        }

        if(!this.model.enable3D)
        //draw axes, labels and ticks
        for (var l = 0, len = this.model._axes.length; l < len; l++) {
            var axis = this.model._axes[l], lineOption = params.axes[axis.name]._lineOption, isValidCross = params.axes[axis.name]._validCross;
            if (this.model.AreaType.toLowerCase() == "cartesianaxes") {                
                chartaxis._drawAxes(l, axis);
                if (lineOption[0])
                    this.svgRenderer.drawLine(lineOption[0], lineOption[1]);
                if (axis.orientation.toLowerCase() === "horizontal") {
                    chartaxis._drawXAxisTickLine(l, axis, this.model.enableCanvasRendering ? null : params.axes[axis.name]._lineOption[1],params);
                    chartaxis._drawXAxisMultiLevelLabels(l, axis);
                }
                else {
                    chartaxis._drawYAxisTickLine(l, axis, this.model.enableCanvasRendering ? null : lineOption[1],isValidCross);
                    chartaxis._drawYAxisMultiLevelLabels(l, axis);
                }
                axis._crossValue = null;
            }
        }
         if (!this.resetZooming && (this.model._chartAreaZoom || this.panning))
             this._enableZoomingButtons();
         this.resetZooming = false;

         if (this.svgHeight != this.model.svgHeight || this.svgWidth != this.model.svgWidth) {
                      
             this.prevHeight = this.model.svgHeight;
             this.prevWidth = this.model.svgWidth; 
             // afterResize event 
             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
             commonEventArgs.data = { height: this.prevHeight, width: this.prevWidth, prevHeight: prevHeight, prevWidth: prevWidth, originalHeight: this.svgHeight, originalWidth: this.svgWidth };
             this._trigger("afterResize", commonEventArgs);
         }

         //Checking Selection setting enable 
         var seriesCollection = this.model._visibleSeries, selectionFound;
         selectionFound = $.grep(seriesCollection, function (series) { return series.selectionSettings.enable == true; });
         if (selectionFound.length == 0) {
             this.model.selectionEnable = false;
             this._removeSelection();
         } else
             this.model.selectionEnable = true;

         var selectedData = this.remove_duplicates(this.model.selectedDataPointIndexes);
         //Maintaning the selection state and triggering seleceted data
         if (this.model.selectionEnable) {
             this._removeSelection();
             this.model._isStateChaged = true;
             var data,
                 mode,
             length = selectedData.length;                      
             for (var i = 0; i < length; i++) {
                 data = selectedData[i];
                 series = this.model._visibleSeries[data.seriesIndex];
                 if (series) {
                     mode = series.selectionSettings.mode;
                     if((mode != 'series' && series.points[data.pointIndex]) ||mode == 'series')
                        this.segmentSelection(data.event, data.legendData, data.seriesIndex, data.pointIndex, data.data);
                 }
             }
         }
		this.chartUpdating = false;
        var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonLoadedEventArgs.data = { model: this.model };
        this._trigger("loaded", commonLoadedEventArgs);


     },
     remove_duplicates: function (arr) {
         var arr1 = [],
             arr2 = [],
             length = arr.length,
             mode,
	     	 currentSeries,
             series = this.model._visibleSeries;
         for (var i = 0; i < length; i++) {
             currentSeries = series[arr[i].seriesIndex];
             mode = currentSeries ? currentSeries.selectionSettings.mode : "";
             if (mode == "series") {
                 if (!(arr1.indexOf(arr[i].seriesIndex) > -1))
                     arr2.push(arr[i]);
                 arr1.push(arr[i].seriesIndex);
             }
             else if (mode == "cluster") {
                 if (!(arr1.indexOf(arr[i].pointIndex) > -1))
                     arr2.push(arr[i]);
                 arr1.push(arr[i].pointIndex);
             } else {
                 if (!(arr[i] in arr1))
                     arr2.push(arr[i]);
                 arr1.push(arr[i]);
             }
         }

         return arr2;

     },
	
	// set value to private variable for two way binding
	_setZoomProperties: function () {

         var axis, _zf, _zp, orientation;
	    for (var l = 0; l < this.model._axes.length; l++) {
	        axis = this.model._axes[l];
	        orientation = axis.orientation.toLowerCase();
	        _zf = axis.zoomFactor.toFixed(3);
	        _zp = axis.zoomPosition.toFixed(3);
	        if (orientation == "horizontal" && (_zf != this._xZoomFactor() || _zp != this._xZoomPosition())) {
	            this._xZoomFactor(axis.zoomFactor.toFixed(3));
	            this._xZoomPosition(axis.zoomPosition.toFixed(3));
	        }
	        if (orientation == "vertical" && (_zf != this._yZoomFactor() || _zp != this._yZoomPosition())) {
	            this._yZoomFactor(axis.zoomFactor.toFixed(3));
	            this._yZoomPosition(axis.zoomPosition.toFixed(3));
	        }
	    }			
	},

	  annotationRender: function (currentAnnotation, a) {                           // method to render annotation
            //declaration and initialization
            var chartContainer = this._id;
            var flag = false;
            var currentSeries = this.model.series[0];
            var seriesType = currentSeries.type.toLowerCase();
            var areaBoundsX = this.model.m_AreaBounds.X;
            var areaBoundsY = this.model.m_AreaBounds.Y;
            var areaBoundsWidth = this.model.m_AreaBounds.Width;
            var areaBoundsHeight = this.model.m_AreaBounds.Height;
            var marginLeft = this.model.margin.left;
            var marginTop = this.model.margin.top;
          
            var seriesIndex = $.inArray(currentSeries, this.model._visibleSeries);
            var radius = this.model.circularRadius[seriesIndex];
			  var centerX = this.model.circleCenterX[seriesIndex];
            var centerY = this.model.circleCenterY[seriesIndex];
			var currentDocument = $(document);

            currentAnnotation = $.extend(true, {}, this.model.annotationsDefault, currentAnnotation);   // extending defaults

            // creating annotation group
            if ($('#annotation_group_' + chartContainer).length != 0)
                var annotationContainer = $('#annotation_group_' + chartContainer);
            else
                annotationContainer = $("<div></div>").attr('id', "annotation_group_" + chartContainer);
            annotationContainer.css({ "position": "absolute", "left": "0px", "top": "0px" });

            var content = currentAnnotation.content;
            element = $("#" + content);
            if (element.get(0).tagName == "SCRIPT")                // for jsrender
                var element = $($("#" + content)[0].innerHTML);
            var cloneNode = element.clone();                        // clone
            var id = 'annotation_' + chartContainer + "_" + content + "_" + a;
            $(cloneNode).attr("id", id);
            var $cloneNode = $(cloneNode);
            $cloneNode.css({
                'position': "absolute",
                'z-index': 2000
            });

            // initialization
            var region = currentAnnotation.region.toLowerCase();
            var coordinateUnit = currentAnnotation.coordinateUnit.toLowerCase();
            var annotationX = (typeof currentAnnotation.x=="string"||typeof currentAnnotation.x=="object") ? Date.parse(currentAnnotation.x) : currentAnnotation.x;
            var annotationY = currentAnnotation.y;
            var horizontalAlignment = currentAnnotation.horizontalAlignment.toLowerCase();
            var verticalAlignment = currentAnnotation.verticalAlignment.toLowerCase();

            if (coordinateUnit == "pixels") {                          //pixels
                if (region == "series") {                             // chart area / series
                    if ((areaBoundsWidth + areaBoundsX + marginLeft > annotationX + areaBoundsX) &&
                        areaBoundsHeight + areaBoundsY + marginTop > annotationY + areaBoundsY) {
                        var left = annotationX + areaBoundsX;
                        var top = annotationY + areaBoundsY;
                    }
                }
                else if (region == "chart") {                         // chart
                    if (($(this.svgObject).width() > annotationX) &&
                         $(this.svgObject).height() > annotationY) {
                        left = annotationX;
                        top = annotationY;
                    }
                }

                $cloneNode.css({
                    "left": left,
                    "top": top
                });
                flag = true;                                // to denote that the annotation should be appended in DOM
            } else if (coordinateUnit == "points") {        //points
                point = {
                    'x': annotationX,
                    'y': annotationY
                };

                if (this.model.AreaType == "polaraxes") {   // polar
                    xAxis = this.model._axes[0];
                    yAxis = this.model._axes[1];
                    if ((annotationX >= xAxis.visibleRange.min && annotationX <= xAxis.visibleRange.max)
                        && (annotationY >= yAxis.visibleRange.min && annotationY <= yAxis.visibleRange.max)) {
                        var location = this.TransformToVisibleAnnotation(xAxis, yAxis, annotationX, annotationY, this);
                        $cloneNode.css({
                            "left": location.X,
                            "top": location.Y
                        });
                        flag = true;
                    }
                } else if (seriesType == "pyramid" || seriesType == "funnel") { // pyramid and funnel
                    var currentPoint = currentSeries.points[annotationX];
                    $cloneNode.css({
                        "left": currentPoint.xLocation + this.pyrX,
                        "top": currentPoint.yLocation + this.pyrY
                    });
                    flag = true;
                } else if (seriesType == "pie" || seriesType == "doughnut" || seriesType == "pieofpie") { // pie, pieofpie and doughnut
                    currentPoint = currentSeries._visiblePoints[annotationX];
                    var pointIndex = annotationX;
                    var startX, startY;
                    var series = new ej.seriesTypes[seriesType]();
                    var midAngle = currentPoint ? currentPoint.currentMidAngle : 0;
                    if ((pointIndex == currentSeries.explodeIndex || currentSeries.explodeAll) && !chartObj.vmlRendering) {
                        startX = centerX + Math.cos(midAngle) * currentSeries.explodeOffset;
                        startY = centerY + Math.sin(midAngle) * currentSeries.explodeOffset;
                    } else {
                        startX = centerX;
                        startY = centerY;
                    }
                    var midX = series.getXCordinate(startX, (radius), midAngle);
                    var midY = series.getYCordinate(startY, (radius), midAngle);

                    positionX = (midX + startX) / 2;
                    positionY = (midY + startY) / 2;

                    positionX = (midX + positionX) / 2;
                    positionY = (midY + positionY) / 2;

                    $cloneNode.css({
                        "left": positionX,
                        "top": positionY
                    });
                    flag = true;
                } else {                                               // cartesianaxes
                    var xAxisName = currentAnnotation.xAxisName;
                    var yAxisName = currentAnnotation.yAxisName;
                    var axesLength = this.model._axes.length;
                    for (var i = 0; i < axesLength; i++) {
                        if (this.model._axes[i].name == xAxisName)
                            var xAxis = this.model._axes[i];
                        if (this.model._axes[i].name == yAxisName)
                            var yAxis = this.model._axes[i];
                    }
                    if (!xAxis) xAxis = this.model._axes[0];           // setting primary axis by default
                    if (!yAxis) yAxis = this.model._axes[1];

                        if (!this.model.requireInvertedAxes) {
						    location = this._getAnnotationPoint(point, xAxis, yAxis);
                            $cloneNode.css({
                                "left": location.X + areaBoundsX,
                                "top": location.Y + yAxis.y
                            });
                        }
                        else {
                            point.x = point.y + (point.y = point.x, 0);// swapping x, y => a = b + (b=a, 0)
                            location = this._getAnnotationPoint(point, yAxis, xAxis);
                            $cloneNode.css({
                                 "left": location.X + areaBoundsX,
                                 "top": location.Y + xAxis.y
                            });
						}
                        flag = true;
                }
            } else {                                                                      //other alignment
                $cloneNode.css("display", "block").appendTo($(annotationContainer));       //append
                $(annotationContainer).appendTo('#' + 'chartContainer_' + chartContainer);
                width = $cloneNode.outerWidth();
                height = $cloneNode.outerHeight();
                if (region == "chart") {
                    switch (horizontalAlignment) {
                        case "middle":
                            left = $(this.svgObject).width() / 2 - width / 2;
                            break;
                        case "left":
                            left = 0;
                            break;
                        case "right":
                            left = $(this.svgObject).width() - width;
                    }
                    switch (verticalAlignment) {
                        case "middle":
                            top = $(this.svgObject).height() / 2 - height / 2;
                            break;
                        case "top":
                            top = 0;
                            break;
                        case "bottom":
                            top = $(this.svgObject).height() - height;
                    }
                } else {
                    if (seriesType == "pie" || seriesType == "doughnut" || seriesType == "pieofpie") {             // for pie, pieofpie and doughnut
                        left = centerX;
                        top = centerY;
                        switch (horizontalAlignment) {
                            case "middle":
                                left = centerX - width / 2;
                                break;
                            case "left":
                                left = centerX - radius;
                                break;
                            case "right":
                                left = centerX + radius - width;
                        }

                        switch (verticalAlignment) {
                            case "middle":
                                top = centerY - height / 2;
                                break;
                            case "top":
                                top = centerY - radius;
                                break;
                            case "bottom":
                                top = centerY + radius;
                        }
                    } else {
                        switch (horizontalAlignment) {
                            case "middle":
                                left = areaBoundsX + (areaBoundsWidth / 2) - width / 2;
                                break;
                            case "left":
                                left = areaBoundsX;
                                break;
                            case "right":
                                left = areaBoundsX + areaBoundsWidth - width;
                        }

                        switch (verticalAlignment) {
                            case "middle":
                                top = areaBoundsY + (areaBoundsHeight / 2) - height / 2;
                                break;
                            case "top":
                                top = areaBoundsY;
                                break;
                            case "bottom":
                                top = areaBoundsY + areaBoundsHeight - height;
                        }
                    }
                }
                if (seriesType == "pyramid" || seriesType == "funnel") {         // to remove margin value
                    left -= marginLeft;
                }
                $cloneNode.css({
                    "left": left,
                    "top": top
                });
            }

            left = parseFloat($cloneNode.css("left"));
            top = parseFloat($cloneNode.css("top"));                              // get left and top value
            if (coordinateUnit != "none" && flag) {
                $cloneNode.css("display", "block").appendTo($(annotationContainer));   //append
                $(annotationContainer).appendTo('#' + 'chartContainer_' + chartContainer);
				width = $cloneNode.outerWidth();
                height = $cloneNode.outerHeight();
                switch (horizontalAlignment) {
                    case "middle":
                        left -= width / 2;
                        break;
                    case "left":
                        left -= width;
                        break;
                }
                switch (verticalAlignment) {
                    case "middle":
                        top -= height / 2;
                        break;
                    case "top":
                        top -= height;
                        break;
                }
            }
            var margin = currentAnnotation.margin;                      // margin values
            left = left + margin.left - margin.right;
            top = top + margin.top - margin.bottom;

            commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);          // trigger event
            commonEventArgs.data = {
                content: $cloneNode.html(), location: { x: left, y: top },
                opacity: currentAnnotation.opacity, angle: currentAnnotation.angle
            };
            this._trigger("annotationRendering", commonEventArgs);

            $cloneNode.html(commonEventArgs.data.content);
            $cloneNode.css({
                "left": commonEventArgs.data.location.x,
                "top": commonEventArgs.data.location.y,
                "opacity": commonEventArgs.data.opacity                  //opacity
            });

            var angle = commonEventArgs.data.angle;                     // angle
            if (angle) {
                $cloneNode.css({
                    '-ms-transform': 'rotate(' + angle + 'deg)',        //IE 10 and 9
                    '-webkit-transform': 'rotate(' + angle + 'deg)',    // chrome
                    '-moz-transform': 'rotate(' + angle + 'deg)',       // firefox
                    '-o-transform': 'rotate(' + angle + 'deg)',         // opera
                    '-transform': 'rotate(' + angle + 'deg)'            // general
                });
            }

            var eleId = $cloneNode.attr("id");                          // id of annotation
            var topHeight = bottomHeight = leftWidth = rightWidth = 0;
            var chartWidth = $("#" + this._id).width();
            var chartHeight = $("#" + this._id).height();
            if (flag && (angle == 0) && (region == "series" || coordinateUnit == "points")) {
                if (areaBoundsWidth + areaBoundsX < left + width) {                       // clip right width
                    var excessWidth = (left + width) - (areaBoundsWidth + areaBoundsX);
                    rightWidth = width - excessWidth;
                }
                if (left < areaBoundsX) {                                                  // clip left width
                    leftWidth = areaBoundsX - left;
                }
                if (areaBoundsHeight + areaBoundsY < top + height) {                      // clip bottom height
                    excessHeight = (top + height) - (areaBoundsHeight + areaBoundsY);
                    bottomHeight = height - excessHeight;
                }
                if (top < areaBoundsY) {                                                 // clip top height
                    topHeight = areaBoundsY - top;
                }
                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                document.getElementById(eleId).style.clip = "rect(" + topHeight.toString() + "px," +  // clipping annotation
                     rightWidth.toString() + "px," +
                     bottomHeight.toString() + "px," +
                     leftWidth + "px)";
            }
            else if (flag && (angle == 0) && region == "chart") {
                if (chartWidth < left + width) {                              // clip right width
                    excessWidth = (left + width) - chartWidth;
                    var rightWidth = width - excessWidth;
                }
                if (left < 0) {                                              // clip left width
                    var leftWidth = width + left;
                    leftWidth = (leftWidth == 0) ? width : leftWidth;
                }
                if (chartHeight < top + height) {                            // clip bottom height
                    var excessHeight = (top + height) - chartHeight;
                    var bottomHeight = height - excessHeight;
                }
                if (top < 0) {                                               // clip top height
                    topHeight = height + top;
                    topHeight = (topHeight == 0) ? height : topHeight;
                }
                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                document.getElementById(eleId).style.clip = "rect(" +
                    topHeight.toString() + "px," +
                    rightWidth.toString() + "px," +
                    bottomHeight.toString() + "px," +
                    leftWidth + "px)";
            }
			else if (flag && (angle == 90) && (region == "series" || coordinateUnit == "points")) {
               
                var clientRect = document.getElementById(eleId).getBoundingClientRect();
                left = clientRect.left - $(this.svgObject).offset().left + currentDocument.scrollLeft();
                top = clientRect.top - $(this.svgObject).offset().top + currentDocument.scrollTop();
                
                if (areaBoundsHeight + areaBoundsY < top + width) {                      // clip right width
                    excessWidth = (top + width) - (areaBoundsHeight + areaBoundsY);
                    rightWidth = width - excessWidth;
                }

                if (left < areaBoundsX) {                                                  // clip bottom height

                    excessHeight = areaBoundsX - left;
                    bottomHeight = height - excessHeight;
                }

                if (top < areaBoundsY) {                                                 // clip left width
                    leftWidth = areaBoundsY - top;
                }

                if (areaBoundsWidth + areaBoundsX < left + height) {                       // clip top height
                    excessHeight = (left + height) - (areaBoundsWidth + areaBoundsX);
                    topHeight = excessHeight;
                }

                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
               
                document.getElementById(eleId).style.clip = "rect(" + topHeight.toString() + "px," +         // clipping annotation
                     rightWidth.toString() + "px," +
                     bottomHeight.toString() + "px," +
                     leftWidth + "px)";
            }
            else if (flag && (angle == 90) && region == "chart") {
                var clientRect = document.getElementById(eleId).getBoundingClientRect();
                left = clientRect.left - $(this.svgObject).offset().left ;
                top = clientRect.top - $(this.svgObject).offset().top ;

                if (chartHeight < top + width) {                              // clip right width
                    excessWidth = (top + width) - chartHeight;
                    rightWidth = width - excessWidth;
                }
                if (left < 0) {                                              // clip bottom height
                    excessHeight = 0 - left;
                    bottomHeight = height - excessHeight;
                    bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                }
                if (top < 0) {                                               // clip left width
                    leftWidth = 0 - top;
                    leftWidth = (leftWidth == 0) ? width : leftWidth ;
                }
                if (chartWidth < left + height) {                            // clip top height
                    excessHeight = (left + height) - chartWidth;
                    topHeight = excessHeight;
                }
               
                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                document.getElementById(eleId).style.clip = "rect(" +
                    topHeight.toString() + "px," +
                    rightWidth.toString() + "px," +
                    bottomHeight.toString() + "px," +
                    leftWidth + "px)";
            }
        },                                                                 // ---------------------------- end of annotation 

        TransformToVisibleAnnotation: function (xAxis, yAxis, x, y, sender) {       // method to get the position of points in polar/ radar
            x = (xAxis._valueType == "logarithmic") && x > 0 ? Math.log(x, xAxis.logBase) : x;
            y = (xAxis._valueType == "logarithmic") && y > 0 ? Math.log(y, yAxis.logBase) : y;
            var radius = sender.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(yAxis, y, this);
            var point = ej.EjSvgRender.utils._valueToVector(xAxis, x);
            return { X: sender.model.centerX + radius * point.X, Y: sender.model.centerY + radius * point.Y };
        },

        _getAnnotationPoint: function (point, xAxis, yAxis) {                     // method to get the position of points (cartesianaxes)
            var requireInvertedAxes = this.model.requireInvertedAxes;
			 var x = point.x;
            var y = point.y;

            var xvalue = (xAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((x == 0 ? 1 : x), xAxis.logBase) : x;
            var yvalue = (yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((y == 0 ? 1 : y), xAxis.logBase) : y;
            xvalue = ej.EjSvgRender.utils._getPointXY(xvalue, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width);
            yvalue = (1 - ej.EjSvgRender.utils._getPointXY(yvalue, yAxis.visibleRange, yAxis.isInversed)) * (yAxis.height);

            point.location = { X: xvalue, Y: yvalue };
            return point.location;
        },
		
     indicatorRender: function() {
         if ($(this.svgObject).find("#" + this.svgObject.id + "_IndicatorCollection").length > 0) {
             $(this.svgObject).find("#" + this.svgObject.id + "_IndicatorCollection").remove();
         }

         var indOptions = { 'id': this.svgObject.id + '_IndicatorCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };

         this.gIndicatorEle = this.svgRenderer.createGroup(indOptions);

         for (var i = 0; i < this.model.indicators.length && this.model.indicators[i].visible && this.model.indicators[i]._points.length>0; i++) {
             var indicator = this.model.indicators[i];
             var options;
             for (var j = 0; j < this.model._axes.length; j++) {
                 var axis = this.model._axes[j];

                 if (axis.name && axis.orientation.toLowerCase() == 'horizontal' && axis.name.toLowerCase() == indicator.xAxisName.toLowerCase()) {
                     indicator.xAxis = axis;
                 } else if (axis.name && axis.orientation.toLowerCase() == 'vertical' && axis.name.toLowerCase() == indicator.yAxisName.toLowerCase()) {
                     indicator.yAxis = axis;
                 }
             }
             var type = indicator.type.toLowerCase();
             options = new ej.indicatorTypes[type]();
             //Clip the canvas Chart indicator series out of chartArea Bounds
             if (this.model.enableCanvasRendering) {
                 this.svgRenderer.ctx.save();
                 this.svgRenderer.ctx.beginPath();
                 this.svgRenderer.ctx.rect(this.model.m_AreaBounds.X, indicator.yAxis.y, indicator.xAxis.width, indicator.yAxis.height);
                 this.svgRenderer.ctx.clip();
                 options.draw(indicator, this);
                 this.svgRenderer.ctx.restore();
             } else
                 options.draw(indicator, this);
             var element = options.gIndicatorGroupEle;
             if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, indicator, this);

             if (!this.vmlRendering && !this.model.enableCanvasRendering) {
                 if (indicator.enableAnimation && (!indicator._animatedSeries || indicator.type != indicator._previousType)) {
                     this.svgRenderer.append(this.gIndicatorEle, this.svgObject);
                     var clipRect = $(this.gIndicatorEle).find("#" + this.svgObject.id + '_indicatorGroup' + '_' + i + "_ClipRect").children();					 
					 ej.EjSeriesRender.prototype._doLineAnimation(this, clipRect);    // for indicator animation
                     this.model.AnimationComplete = true;
                     indicator._animatedSeries = true;
                     indicator._previousType = indicator.type;

                 }
             }
	   	  this.svgRenderer.append(this.gIndicatorEle, this.svgObject); 
         }
		 
     },
	 
	 doAnimation: function (series) {
	   // Declaration
	   var chartObj = this,	 
	       model = chartObj.model,
	       requireInvertedAxes = model.requireInvertedAxes,
		   type = series.type.toLowerCase(),
	       seriesIndex = $.inArray(series, model._visibleSeries),
	       gSeriesGroupId = this.svgObject.id + "_SeriesGroup_" + seriesIndex,
		   markerElements = $(chartObj.gSymbolGroupEle)[seriesIndex].childNodes,
		   markerLength = markerElements.length,
		   dataLabels = $(chartObj.gDataLabelEle)[seriesIndex].childNodes,
		   dataLabelLength = dataLabels.length,
		   connectorLines = $(chartObj.gConnectorEle)[seriesIndex].childNodes,
		   connectorLineLength = connectorLines.length,
		   errorBarElements = $(chartObj.gErrorBarGroupEle)[seriesIndex] == undefined ? 
		                      undefined :$(chartObj.gErrorBarGroupEle)[seriesIndex].childNodes,
		   errorBarEleLength = ej.util.isNullOrUndefined(errorBarElements) ? 0 : errorBarElements.length,
		   animationType = series.animationType, clipRect, errorBarInterval,boxWhiskerInterval, lineEle, pathEle,
		   markerInterval, dataLabelInterval, index, elements, elementsLength, clipRect,
           ubound = 20, lbound = 0, randomValue, delayInterval, marker, i, j,
		   seriesRender = ej.EjSeriesRender.prototype;
		   chartObj.gSeriesGroupEle = $("#" + gSeriesGroupId)[0];
		   
		   switch (animationType){
			  case "rect":
			        elements = $(this.gSeriesEle).find("#" + gSeriesGroupId).children("rect");
		            elementsLength = elements.length;
					if(type=="boxandwhisker"){
						 lineEle = $(this.gSeriesEle).find("#" + gSeriesGroupId).children("line");
						 pathEle = $(this.gSeriesEle).find("#" + gSeriesGroupId).children("path");
						 boxWhiskerInterval=2000/pathEle.length;
						for (var k = 0; k < lineEle.length; k++) {
							seriesRender._doLineSymbol(lineEle[k],boxWhiskerInterval,k, series, requireInvertedAxes, chartObj);
							seriesRender._doLineSymbol(elements[k],boxWhiskerInterval,k, series, requireInvertedAxes, chartObj);
						 }
						 for (var j = 0; j < pathEle.length; j++)
						 {
						     seriesRender._doLineSymbol(pathEle[j], boxWhiskerInterval, j, series, requireInvertedAxes, chartObj);
						 }
					} 
					else{
                    for (i = 0; i < elementsLength; i++) {
			          if (type.indexOf("stacking") > -1) 
                         seriesRender.animateStackingRect(elements[i], series, requireInvertedAxes, chartObj);
			          else
				         seriesRender.animateRect(elements[i], series, requireInvertedAxes, chartObj);
		             }
					}
                    markerInterval = dataLabelInterval = errorBarInterval = 500;
					break;					
              case "path":
			         clipRect = $(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children();
                     seriesRender._doLineAnimation(chartObj, clipRect);
                     markerInterval = 2000 / markerLength;
		             dataLabelInterval = 2000 / dataLabelLength;
		             errorBarInterval = 2000 / errorBarEleLength;
					 break;					 
              case "scatter":
			         for (i = 0; i < markerLength; i++) {
                        randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                        delayInterval = parseInt(randomValue * 50);
                        seriesRender.animateSymbol(markerElements[i], delayInterval, series, requireInvertedAxes, chartObj);  
	                 } 
					 break;				 
              case "bubble":
			          elements = $(this.gSeriesGroupEle).children().not("defs");
                      delayInterval = 0;
                      for (i = 0; i < elements.length; i++) 
                           seriesRender.animateSymbol(elements[i], delayInterval, series, requireInvertedAxes, chartObj);
		              for (i = 0; i < markerLength; i++) 
                           seriesRender.animateSymbol(markerElements[i], delayInterval, series, requireInvertedAxes, chartObj);   
		              for (i = 0; i < dataLabelLength; i++) 
                           seriesRender.animateSymbol(dataLabels[i], delayInterval, series, requireInvertedAxes, chartObj);
		              for (i = 0; i < connectorLineLength; i++)
                           seriesRender.animateSymbol(connectorLines[i], delayInterval, series, requireInvertedAxes, chartObj);
					   break;           
			   case "hilo":		
			          elements = $(this.gSeriesGroupEle).children().not("defs");
		              elementsLength = elements.length;
                      time = 2000 / elementsLength;
                      for (var i = 0; i < elementsLength; i++)
                      seriesRender._doLineSymbol(elements[i], time, i, series, requireInvertedAxes, chartObj);
		              dataLabelInterval = 2000/ dataLabelLength;
                      break;					  
		       case "cylinder":
		           clipRect = $(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children();
		           seriesRender.animateCylinder(chartObj, series, requireInvertedAxes, clipRect);
		           markerInterval = dataLabelInterval = errorBarInterval = 500;
		           break;
		   }
	  
	      if (animationType != "bubble") {
	        for (j = 0; j < markerLength; j++) {              // for marker
	           index = animationType == "rect" || animationType == "stackingRect" ? 2 : j;
               seriesRender._doLineSymbol(markerElements[j], markerInterval, index, series, requireInvertedAxes, chartObj);
	         }
		
	        for (j = 0; j< dataLabelLength; j++) {            // for data label
		       index = animationType == "rect" || animationType == "stackingRect"  ? 2 : j;
		       seriesRender._doLineSymbol(dataLabels[j], dataLabelInterval, index, series, requireInvertedAxes, chartObj);
		     }
		
	        time = 2000/ connectorLineLength;
		    for (j = 0; j< connectorLineLength; j++) {        // for connector lines
		       index = animationType == "rect" || animationType == "stackingRect"  ? 2 : j;
		       seriesRender._doLineSymbol(connectorLines[j], time, index, series, requireInvertedAxes, chartObj);
		     }
	      }
		
		  if (series.errorBar.visibility == "visible"){     
		     for (j = 0; j< errorBarEleLength; j++) {      // for error bar
		       index = animationType == "rect" || animationType == "stackingRect"  ? 2 : j;
		       seriesRender._doLineSymbol(errorBarElements[j], errorBarInterval, index, series, requireInvertedAxes, chartObj);
		     }	
		  }
   },
   _animationElements: function (seriesOptions, seriesType, seriesIndex) {
            var chartObj = this;
            seriesOptions._previousType = ej.util.isNullOrUndefined(seriesOptions._previousType) ? seriesOptions.type : seriesOptions._previousType;
            if (seriesOptions.enableAnimation && !this.vmlRendering && !this.model.enableCanvasRendering && (!seriesOptions._animatedSeries || seriesOptions.type != seriesOptions._previousType)) {
                ej.EjSeriesRender.prototype.doCircularAnimation(this, seriesOptions, seriesType, seriesIndex);
                seriesOptions._previousType = seriesOptions.type;
                seriesOptions._animatedSeries = chartObj.isSplitted ? false : true;
            }
            chartObj.isSplitted = false;
            this.drawAccDisplayText(seriesType, seriesOptions, seriesIndex);
            //Show the datalabel text element on redraw when enable animation of series
            if (seriesOptions.enableAnimation && seriesOptions._visiblePoints.length > 0 && seriesOptions.AnimationComplete && !this.vmlRendering && !this.model.enableCanvasRendering) {
                chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle[seriesIndex].id), { "visibility": "visible" });
                if (chartObj.gSymbolGroupEle)
                    chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle[seriesIndex].id), { "visibility": "visible" });
                var elements;
                if (seriesOptions.marker.dataLabel.template) {
                    elements = $(chartObj.element[0].childNodes[0].childNodes);
                    for (var i = 0; i < elements.length; i++)
                        $(elements[i]).css('display', 'block');
                }
                if (chartObj.gConnectorEle)
                    chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gConnectorEle[seriesIndex].id), { "visibility": "visible" });
                if (chartObj.gDataLabelEle)
                    chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gDataLabelEle[seriesIndex].id), { "visibility": "visible" });
            }



        },
		  _appendConnectorElements: function (i) {
            var visibleSeries = ej.DataManager(this.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
            var currentSeries = visibleSeries[i];
            var type = currentSeries.type.toLowerCase();
            if (currentSeries.visibility == "visible" && !this.model.enable3D && currentSeries.connectorLine && (currentSeries.connectorLines || this.model.AreaType == "none")) {
                this.svgRenderer.append(this.gConnectorEle[i], this.gSeriesEle);
                currentSeries.connectorLines = false;
            }
        },
        _appendDataLabelElement: function (i) {
            var currentSeries, type, index;
            currentSeries = this.model._visibleSeries[i];
            index = !ej.util.isNullOrUndefined(currentSeries.zOrder) ? currentSeries.zOrder : index;
            if (currentSeries.visibility == "visible" && !this.model.enable3D) {
                this.svgRenderer.append(this.gSymbolGroupEle[i], this.gSeriesEle);
                if (!this.model.enable3D && !this.model.enableCanvasRendering) {
                    if (this.gDataLabelEle[i].childNodes.length > 0)
                        this.svgRenderer.append(this.gDataLabelEle[i], this.gSeriesEle);
                    if (this.gSeriesTextEle[i].childNodes.length > 0)
                        this.svgRenderer.append(this.gSeriesTextEle[i], this.gSeriesEle);
                }
            }
            if (!this.vmlRendering && currentSeries.visibility == "visible" && !this.model.enableCanvasRendering && !this.model.enable3D) {
                this.svgRenderer.append(this.gSeriesEle, this.svgObject);
                this.svgRenderer.append(this.svgObject, this.element);
            }

        },
        _accSeriesDraw: function (index, seriesOptions, type) {
            var chartObj = this;
            var visibleSeries = ej.DataManager(this.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
            var seriesIndex = $.inArray(visibleSeries[index], this.model._visibleSeries);
            this.model._visibleSeries[seriesIndex] = visibleSeries[index] = $.extend(true, seriesOptions, visibleSeries[index]);
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { series: seriesOptions };
            this._trigger("seriesRendering", commonEventArgs);
            var seriesType = new ej.seriesTypes[type.toLowerCase()]();
            var minhightwidth = seriesType.draw(chartObj, seriesOptions);
            if (!minhightwidth) {
			this.model.minhightwidth = true;
                chartObj._animationElements(seriesOptions, seriesType, seriesIndex);
            }


        },
     // to split the series for pie of pie rendering
        _splitSeries: function (visibleSeries) {
            //declaration of splitseries variables            
            var pieCollections = [];
                pieCollections[0] = $.extend(true, [], visibleSeries._visiblePoints);
                pieCollections[1] = [];               
            var series = visibleSeries,
                splitValue = parseInt(series.splitValue),
                splitMode = series.splitMode.toLowerCase(),
                length = pieCollections[0].length,
                count = 0, j = 0, yValues = [], i = 0, totalValue = 0, yvalue;
            // to calculate sum of y values
            while (i != length) {
               totalValue += pieCollections[0][i].y;
               i++;
            }
            // to split points based on splitmode    
             if (splitMode == "position") {
                 for (var i = length - 1; i >= 0 && j != splitValue; i--) {
                    count = count + pieCollections[0][i].y;
                    pieCollections[1][j] = pieCollections[0][i];
                    j++;
                    pieCollections[0].splice(i, 1);
                 }
                 pieCollections[1].reverse();
             }
             else if (splitMode == "value" || splitMode == "percentage") {
                 for (var k = 0; k < pieCollections[0].length; k++) {
                    yvalue = splitMode == "value" ? pieCollections[0][k].y : Math.round((pieCollections[0][k].y / totalValue) * 100);                    
                    if (yvalue < splitValue) {
                        count = count + pieCollections[0][k].y;
                        pieCollections[1][j] = pieCollections[0][k];
                        j++;
                        pieCollections[0].splice(k, 1);
                        k = -1;
                    }
                   }                  
                }
            else if (splitMode == "indexes") {
                var splitArray = JSON.parse("[" + series.splitValue + "]");              
                // to remove duplicate indices
                splitArray = splitArray.filter(function (item, d, inputArray) {
                    return inputArray.indexOf(item) == d;
                });
                splitArray.sort();
                for (var k = 0; k < pieCollections[0].length; k++) {
                    for (var s = 0; s < splitArray.length; s++) {
                        if (k == splitArray[s]) {
                            count = count + pieCollections[0][k].y;
                            pieCollections[1][j] = pieCollections[0][k];
                            j++;
                        }
                    }
                }
                for (var i = 0; i < splitArray.length; i++) {
                    var index = splitArray[i] - i;
                    pieCollections[0].splice(index, 1);
                }
            }

            // to append the added point in first pie collection
                    if (pieCollections[1].length > 0) {
                        yValues[0] = count,
                               valueType = series._xAxisValueType,
                            pieCollections[0].unshift({
                                _visibility: "visible",
                                visible: true,
                                text:"Others",
                                // dataPoint: undefined,
                                x: valueType == "string" ? "Others" : count,
                                xValue: valueType == "string" ? series.points.length : count,
                                y: count,
                                YValues: yValues,
                                fill: this.model.colors[(series.points.length + 1) % this.model.colors.length],
                                isFill: false,
                                actualIndex: series.points.length
                            });
                    }                 
                    this.isSplitted = true;
                    series.pieCollections = pieCollections;
        },
        _calculateGapWidth:function(currentSeries){
            var circle1end = 0, circle2start = 0, coefficient, centerX, gapWidth;
            var constantWidth = 0.1 * Math.min($(this.svgObject).width(), $(this.svgObject).width());
            ej.EjSeriesRender.prototype._calculateVisiblePoints(currentSeries).visiblePoints;
            this._splitSeries(currentSeries);  // to split the series points for pieofpie seriestype
            this.chartObj = this;
            // gapwidth calculation for pieofpie series type
            for (var j = 0; j < 2; j++) {
                coefficient = (j == 0) ? currentSeries.pieCoefficient : currentSeries.pieOfPieCoefficient;
                currentSeries._visiblePoints = currentSeries.pieCollections[j];
                currentSeries.collectionIndex = j;
                gapWidth = currentSeries.gapWidth;
                var size = ej.EjSeriesRender.prototype.calculatingSliceAngle(currentSeries, this.chartObj);
                var circleradius = size.width < constantWidth ? 0.25 * coefficient * constantWidth : 0.25 * coefficient * Math.min(size.width, size.height);               
                centerX = this.model.circleCenterX[j];
                if (j == 0)
                    circle1end = centerX + circleradius;
                else
                    circle2start = centerX - circleradius;
            }
            var pieGapWidth = (circle2start - circle1end) - gapWidth;
            this.model.pieGapWidth = (pieGapWidth > 0) ? pieGapWidth : 0;
            // to draw connector lines for pieofpie series type
            if (currentSeries.pieCollections[0].length > 0 && currentSeries.pieCollections[1].length > 0 && currentSeries.connectorLine.width > 0)
                this.drawPieofPieConnectors(size, currentSeries); 
            return currentSeries;
        },
    seriesRender: function (params) {
	        this.model.allPoints = [];
           this.model.markerRegion = [];
           this.model.circleCenterX = [];
            this.model.circleCenterY = [];
           this.model.startX = [];
           this.model.startY = [];
           this.model.centerCount = 0;
		    this.gTrendlinesGroupEle = null;
		   if(!params){
				params ={ _crossAxisOverlap : false, axes:{}};
				for(var i=0; i<this.model._axes.length; i++)
					params.axes[this.model._axes[i].name] = {};
		   }
        var chartObj = this;
        if ($(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").length > 0) {
            $(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").remove();
        }
        var serOptions = { 'id': this.svgObject.id + '_SeriesCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };
        var enable3D = this.model.enable3D;
        var areaType = this.model.AreaType;
        var enableCanvasRendering = this.model.enableCanvasRendering;
		 var axesLength = this.model._axes.length;
        this.gSeriesEle = this.svgRenderer.createGroup(serOptions);
        var visibleSeries = ej.DataManager(this.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
		 var visibleSeriesLength=visibleSeries.length;
        for (var i = 0; i < visibleSeriesLength; i++) {
            if (visibleSeries[i].visibility == "visible") {
			    if(ej.util.isNullOrUndefined(this.model.series[i].enableSmartLabels))  // for setting enablesmartlabels property values
                     this.model.series[i]._enableSmartLabels = (this.model.AreaType) == "none" ? true : false;
			     else 
			         this.model.series[i]._enableSmartLabels = this.model.series[i].enableSmartLabels; 
            if (!ej.util.isNullOrUndefined(visibleSeries[i].xAxis)) {
                visibleSeries[i].xAxis = null;
                visibleSeries[i].yAxis = null;
            }
            for (var j = 0; j < axesLength; j++) {
                var axis = this.model._axes[j];
                var index = i;
				 var axisOrientation = axis.orientation.toLowerCase();
                if (axis.name && axisOrientation == 'horizontal' && axis.name.toLowerCase() == visibleSeries[i]._xAxisName.toLowerCase()) {
                    visibleSeries[i].xAxis = axis;
                }
                else if (axis.name && axisOrientation == 'vertical' && axis.name.toLowerCase() == visibleSeries[i]._yAxisName.toLowerCase()) {
                    visibleSeries[i].yAxis = axis;
                }
                if (this.model.requireInvertedAxes) {
                    if (axis.name && axisOrientation == 'vertical' && axis.name.toLowerCase() == visibleSeries[i]._xAxisName.toLowerCase()) {
                        visibleSeries[i].xAxis = axis;
                    }
                    else if (axis.name && axisOrientation == 'horizontal' && axis.name.toLowerCase() == visibleSeries[i]._yAxisName.toLowerCase()) {
                        visibleSeries[i].yAxis = axis;
                    }
                }

            }
        }
        }
		
        var index, seriesOptions, seriesOptionsVisibility, type, commonEventArgs,
         seriesIndex, seriesType, minhightwidth;
	if (visibleSeriesLength > 0 ) {
        var currentSeries = visibleSeries[0];
        var seriesType = currentSeries.type.toLowerCase(),
        visibility = currentSeries.visibility.toLowerCase();
        if (seriesType == "pieofpie" && visibility == 'visible' && !enable3D) {
            this.model._isPieOfPie = true;
            currentSeries = this._calculateGapWidth(currentSeries); // method to split point, gapwidth calculation and to draw pieofpie connectors
            // pieofpie rendering with splitted points
            for (var k = 0; k < 2; k++) {
                currentSeries.collectionIndex = k;
                currentSeries._visiblePoints = currentSeries.pieCollections[k];
                this._accSeriesDraw(0, currentSeries, seriesType);
            }
            currentSeries._visiblePoints = $.merge(currentSeries.pieCollections[0], currentSeries.pieCollections[1]);
            this.model.pieGapWidth = null;
            currentSeries.collectionIndex = 0;
        }	
        else {
            //pie doughnut rendering from higher order to lower order
            for (var l = visibleSeriesLength; l > 0 && areaType == "none"; l--) {
                index = l - 1;
                seriesOptions = visibleSeries[l - 1];
                seriesOptionsVisibility = seriesOptions.visibility.toLowerCase();
                type = seriesOptions.type.toLowerCase();
                if (seriesOptionsVisibility === 'visible' && !enable3D && (type == "pie" || type == "doughnut")) {

                    this._accSeriesDraw(index, seriesOptions, type);


                }
                else {
                    if ((areaType == "none") && seriesOptionsVisibility === 'visible' && !enableCanvasRendering)
                        ej.Ej3DChart.prototype.renderSeries(this, seriesOptions);
                }
            }
        }
	}
        for (var m = 0; m < visibleSeriesLength; m++) {
            var index = m;
            var seriesOptions = visibleSeries[m];
            var  seriesOptionsVisibility=seriesOptions.visibility.toLowerCase();
            var type = seriesOptions.type.toLowerCase();
            if (areaType == "none" &&  seriesOptionsVisibility === 'visible' && !enable3D && (index === 0) && (type == "pyramid" || type == "funnel")) {
                this._accSeriesDraw(index, seriesOptions, type);


            }
            else {			 
                if (seriesOptionsVisibility === 'visible' && !enable3D && areaType != "none") {
                    this.renderSeries(seriesOptions, params);
                    if (areaType == "cartesianaxes" && this.model.series[m].type != "boxandwhisker" )
                        this._renderTrendline(chartObj, seriesOptions);
                }
                else {
                    if (!enable3D && areaType != "none")
                        this.svgRenderer.append(this.svgObject, this.element);
                    else {
                        if ((areaType == "cartesianaxes" ) && seriesOptionsVisibility === 'visible' && !enableCanvasRendering)
                       ej.Ej3DChart.prototype.renderSeries(this, seriesOptions, params);
					 }
			      }
			  }
        }
		
		if (this.model.AreaType != "none"){
		 // smart labels for cartesian axis
		  var i, currentSeries, currentPoint, points, currentSeriesType, isCanvas = this.model.enableCanvasRendering,
		  areaBounds = this.model.m_AreaBounds, currentPointXPos, currentPointYPos;
		  for (i = 0; i < visibleSeriesLength; i++) {
	        currentSeries = this.model.series[i];
	        points = currentSeries._visiblePoints;
	        if (currentSeries._enableSmartLabels && this.model.AreaType == "cartesianaxes") {
			   currentSeriesType = currentSeries.type.toLowerCase();
	            if (currentSeriesType == "column" || currentSeriesType == "stackingcolumn" ||
                    currentSeriesType == "bar" || currentSeriesType == "stackingbar" || currentSeriesType == "waterfall"
					|| currentSeriesType == "stackingbar100" || currentSeriesType == "stackingcolumn100" || currentSeriesType == "rangecolumn")
	                this.cartesianColumnSmartLabels(currentSeries, points, i)
	            else if (currentSeriesType != "boxandwhisker") {
	                this.cartesianSmartLabels(currentSeries, points, i);
				}
				    for (var j=0; j< points.length; j++){
					    currentPoint = points[j]; // condition to hide partially visible labels
						currentPoint.hide = false;
						currentPointXPos = isCanvas ? currentPoint.xPos- this.canvasX : currentPoint.xPos;
						currentPointYPos = isCanvas ? currentPoint.yPos- this.canvasY : currentPoint.yPos;
					    if ((currentPointXPos - currentPoint.width/2 < 0) || (currentPointXPos + currentPoint.width/2 > areaBounds.Width)||
						     (currentPointYPos + currentPoint.height/2 > areaBounds.Height)||(currentPointYPos - currentPoint.height/2 < 0))
							 currentPoint.hide = true;
		            }
			}
		 }
		
        // to draw connector lines for data label
		  var h, currentPoint, currentPointVisibility, visiblePointsLength;
		  var areaBoundsX = chartObj.model.m_AreaBounds.X;
		  for (i = 0; i < visibleSeriesLength; i++) {
		      currentSeries = visibleSeries[i];
			  if (currentSeries.visibility == "visible"){
			   visiblePointsLength = currentSeries._visiblePoints.length;
		       for (h = 0; h < visiblePointsLength; h++) {
			       currentPoint = currentSeries._visiblePoints[h];
				  currentPointVisibility = currentPoint.marker && currentPoint.marker.dataLabel && currentPoint.marker.dataLabel.visible;
				  if (currentSeries.marker.dataLabel.visible || currentPointVisibility){
		           if ((currentPoint.connectorFlag || currentPoint.newConnectorFlag) && !currentPoint.hide && currentPoint.drawText != "" && currentSeries.visibility == "visible" && !ej.util.isNullOrUndefined(currentPoint.textOptions)) {
					   currentSeries.connectorLine = true; 
		               if (chartObj.model.enableCanvasRendering) {
		                   chartObj.svgRenderer.ctx.save();
                        if (!chartObj.model.requireInvertedAxes)
                            chartObj.svgRenderer.ctx.rect(areaBoundsX, currentSeries.yAxis.y, currentSeries.xAxis.width, currentSeries.yAxis.height);
                        else
                            chartObj.svgRenderer.ctx.rect(areaBoundsX + currentSeries.yAxis.plotOffset, currentSeries.xAxis.y, currentSeries.yAxis.width, currentSeries.xAxis.height);
                        chartObj.svgRenderer.ctx.clip();
                        this.drawConnectorLines(i, h, currentPoint);
                        chartObj.svgRenderer.ctx.restore();
                    } else if (currentSeries._enableSmartLabels)
                        this.drawConnectorLines(i, h, currentPoint);
                }
			  }
            }
		  }
        }

        // to draw data label symbol and text
		  var currentPointLabel;
		  var areaBoundsX = chartObj.model.m_AreaBounds.X;
		  for (i = 0; i < visibleSeriesLength; i++) {
		      currentSeries = visibleSeries[i];
		      var type=currentSeries.type.toLowerCase();
		      if (currentSeries.visibility == "visible" && !this.model.enable3D) {
				  visiblePointsLength = currentSeries._visiblePoints.length;
		          for (h = 0; h < visiblePointsLength; h++) {
		              currentPoint = currentSeries._visiblePoints[h];
					  currentPointLabel = currentPoint.marker && currentPoint.marker.dataLabel && currentPoint.marker.dataLabel.visible;
		              if ((currentSeries.marker.dataLabel.visible || currentPointLabel) && !currentPoint.hide && !currentPoint.dataLabeltemplate && currentSeries.visibility == "visible"  && !ej.util.isNullOrUndefined(currentPoint.textOptions) && !currentPoint.hide) {
                        currentPoint.textOptions.transform = 'rotate(' + currentPoint.textOptions.angle + ',' + (currentPoint.xPos) + ',' + (currentPoint.yPos) + ')';
                        currentPoint.textOptions.labelRotation = currentPoint.textOptions.angle;  //for canvas label rotation
                        if (chartObj.model.enableCanvasRendering) {    // for canvas
                            chartObj.svgRenderer.ctx.save();
                        if (!chartObj.model.requireInvertedAxes)
                            chartObj.svgRenderer.ctx.rect(areaBoundsX, currentSeries.yAxis.y, currentSeries.xAxis.width, currentSeries.yAxis.height);
                        else
                            chartObj.svgRenderer.ctx.rect(areaBoundsX + currentSeries.yAxis.plotOffset, currentSeries.xAxis.y, currentSeries.yAxis.width, currentSeries.xAxis.height);
                        chartObj.svgRenderer.ctx.save();
                        chartObj.svgRenderer.ctx.clip();
                        
                        if (type.toLowerCase() == "boxandwhisker") {
						    if(currentSeries.showMedian == true && currentPoint.textOptionsBoxValues != ""){
								ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.textOptionsBoxValues[0].xPos, currentPoint.textOptionsBoxValues[0].yPos, currentPoint.textOptionsBoxValues[0].width, currentPoint.textOptionsBoxValues[0].height, currentPoint.textOptionsBoxValues[0].symbolName, this)
		                        this.svgRenderer.drawText(currentPoint.textOptionsBoxValues[0], currentPoint.textOptionsBoxValues[0].drawText, this.gSeriesTextEle[i]);									 
							}
                            for (var box = 1; box < currentPoint.textOptionsBoxValues.length; box++) {
                                ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.textOptionsBoxValues[box].xPos, currentPoint.textOptionsBoxValues[box].yPos, currentPoint.textOptionsBoxValues[box].width, currentPoint.textOptionsBoxValues[box].height, currentPoint.textOptionsBoxValues[box].symbolName, this)
                                this.svgRenderer.drawText(currentPoint.textOptionsBoxValues[box], currentPoint.textOptionsBoxValues[box].drawText, this.gSeriesTextEle[i]);
                            }
                        }
                        else {
                            ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPos, currentPoint.yPos, currentPoint.width, currentPoint.height, currentPoint.symbolName, this)
                            this.svgRenderer.drawText(currentPoint.textOptions, currentPoint.drawText, this.gSeriesTextEle);
                        }
                        chartObj.svgRenderer.ctx.restore();
                    } else {
                        ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPos, currentPoint.yPos, currentPoint.width, currentPoint.height, currentPoint.symbolName, this, this.gDataLabelEle[i])
                        this.svgRenderer.drawText(currentPoint.textOptions, currentPoint.drawText, this.gSeriesTextEle[i]);
                    }
		                  if (type == "rangecolumn" || type == "rangearea") {
		                      
		                      currentPoint.textOptionsLow.transform = 'rotate(' + currentPoint.textOptionsLow.angle + ',' + (currentPoint.xPosLow) + ',' + (currentPoint.yPosLow) + ')';
		                      currentPoint.textOptionsLow.labelRotation = currentPoint.textOptionsLow.angle;  //for canvas label rotation
		                      if (chartObj.model.enableCanvasRendering) {    // for canvas
		                          chartObj.svgRenderer.ctx.save();
		                          if (!chartObj.model.requireInvertedAxes)
		                              chartObj.svgRenderer.ctx.rect(areaBoundsX, currentSeries.yAxis.y, currentSeries.xAxis.width, currentSeries.yAxis.height);
		                          else
		                              chartObj.svgRenderer.ctx.rect(areaBoundsX + currentSeries.yAxis.plotOffset, currentSeries.xAxis.y, currentSeries.yAxis.width, currentSeries.xAxis.height);
		                          chartObj.svgRenderer.ctx.clip();
		                          
		                          ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPosLow, currentPoint.yPosLow, currentPoint.widthLow, currentPoint.heightLow, currentPoint.symbolName, this)
		                          
		                          this.svgRenderer.drawText(currentPoint.textOptionsLow, currentPoint.drawTextLow, this.gSeriesTextEle);
		                          chartObj.svgRenderer.ctx.restore();
		                      }
		                      else {
		                          
		                          ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPosLow, currentPoint.yPosLow, currentPoint.widthLow, currentPoint.heightLow, currentPoint.symbolName, this, this.gDataLabelEle[i])
		                          this.svgRenderer.drawText(currentPoint.textOptionsLow, currentPoint.drawTextLow, this.gSeriesTextEle[i]);

		                      }
		                  }
		                  else if (type.toLowerCase() == "boxandwhisker") {
								if(currentSeries.showMedian == true && currentPoint.textOptionsBoxValues != ""){
									ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.textOptionsBoxValues[0].xPos, currentPoint.textOptionsBoxValues[0].yPos, currentPoint.textOptionsBoxValues[0].width, currentPoint.textOptionsBoxValues[0].height, currentPoint.textOptionsBoxValues[0].symbolName, this)
		                            this.svgRenderer.drawText(currentPoint.textOptionsBoxValues[0], currentPoint.textOptionsBoxValues[0].drawText, this.gSeriesTextEle[i]);									 
								 }						  
								for (var box = 1; box < currentPoint.textOptionsBoxValues.length; box++) {							      
									 ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.textOptionsBoxValues[box].xPos, currentPoint.textOptionsBoxValues[box].yPos, currentPoint.textOptionsBoxValues[box].width, currentPoint.textOptionsBoxValues[box].height, currentPoint.textOptionsBoxValues[box].symbolName, this)
		                             this.svgRenderer.drawText(currentPoint.textOptionsBoxValues[box], currentPoint.textOptionsBoxValues[box].drawText, this.gSeriesTextEle[i]);								  								  
								}
							}
                }
            }
		  }
        }

        // to append connector lines
		   if (this.model.AreaType != "none") {
                    // to append connector lines
                    var i, currentSeries;
                    for (i = 0; i < visibleSeriesLength; i++) {
                        this._appendConnectorElements(i);

                    }

                    // to append marker, data label symbol and text
                    var index;
                    var length = this.model._visibleSeries.length;
                    for (i = 0; i < length; i++) {
                        this._appendDataLabelElement(i);
                    }
                }
            }  //to append conector lines ,marker,datalabel symbol and text  for pie and doughnut 
            else {
                var p, currentSeries;
                for (p = visibleSeriesLength - 1; p >= 0 ; p--) {
                    currentSeries = this.model._visibleSeries[p];
                    type = currentSeries.type.toLowerCase();
                   if ((type == "pie" || type == "doughnut") && this.model.minhightwidth)
                        this._appendConnectorElements(p);
                }
                var index;
                var length = this.model._visibleSeries.length;
			if (length > 0 && type == "pieofpie") 
                    this._appendDataLabelElement(0);
			else{
                for (p = visibleSeriesLength - 1; p >= 0 ; p--) {
                    currentSeries = this.model._visibleSeries[p];
                    type = currentSeries.type.toLowerCase();
                    if ((type == "pie" || type == "doughnut") && this.model.minhightwidth)
                        this._appendDataLabelElement(p);
                }
            }
			}
		var length = this.model._visibleSeries.length;
		var s, series, templateContainer, dataLabel;
		for (s = 0; s < length; s++){
		 options = this.model._visibleSeries[s];
		    type = options.type.toLowerCase();
       options._previousType = ej.util.isNullOrUndefined(options._previousType) ? options.type : options._previousType;
		 if (!this.vmlRendering && options.visibility =="visible" && !this.model.enableCanvasRendering && !this.model.enable3D && this.model.AreaType != "none") {
            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
            this.svgRenderer.append(this.svgObject, this.element);
			 series = new ej.seriesTypes[options.type.toLowerCase()]();
			 series.chartObj = chartObj;
			 series.gSeriesGroupEle = $("#" + this.svgObject.id + "_SeriesGroup_" + s)[0];
            if (options.enableAnimation && (!options._animatedSeries || options._previousType != options.type)&& this.model.AreaType != "none" ) {
                options._animatedSeries = true;
                options._previousType = options.type;
			    templateContainer = $('#template_group_' + this._id); 
                dataLabel=options.marker.dataLabel;				
               if (templateContainer.children().length != 0 && (dataLabel.visible && dataLabel.template))	 	
                   series.animateLabelTemplate(options);
			      if (type == "polar" || type == "radar")
					  ej.ejRadarSeries.prototype.doAnimation(options, series); // to animate polar/ radar
				  else
                      this.doAnimation(options);   // to animate cartesian series
            }
         }
		}
		
		// to append trend line to DOM and perform animation
		  var visibleSeries = this.model._visibleSeries;
		  var length = visibleSeries.length;
		  if (!this.model.enableCanvasRendering && !this.model.enable3D){
		    for (var k =0; k <length; k++){
		          if (visibleSeries[k].visibility.toLowerCase() == "visible" && this.gTrendlinesGroupEle && 
				      this.gTrendlinesGroupEle[k] && this.gTrendlinesGroupEle[k].childElementCount != 0)
		         this.svgRenderer.append(this.gTrendlinesGroupEle[k], this.svgObject);
			}
		  }
		 
		  var trendlines, trendline, trendlineType, options, i, j, trendLineLength;
		  for (i = 0; i < length; i++){
		    currentSeries = this.model._visibleSeries[i];
			trendlines = currentSeries.trendlines;
			trendLineLength = currentSeries.trendlines.length;
			for (j = 0; j < trendLineLength; j++) {
              trendline = trendlines[j];
                   if (trendline.visibility) {
			        trendlineType = trendline.type.toLowerCase();
			        options = new ej.trendlineTypes[trendlineType]();
			        if (trendline.visibility.toLowerCase() === 'visible' && trendline.points && trendline.points.length > 1 &&
                       currentSeries.enableAnimation && !this.vmlRendering && (!currentSeries._animatedTrendline || trendline._previousType != trendline.type)) {
			            trendline._previousType = trendline.type;
						var clipRect = $(this.svgObject).find("#" + this.svgObject.id + '_TrendGroup' + '_' + i + '_' + j + "_ClipRect").children();
			            ej.EjSeriesRender.prototype._doLineAnimation(this, clipRect); // for trendline animation
			            currentSeries._animatedTrendline = (trendLineLength - 1 == i) ? true : false;
			        }
				}
			}
		  }
	
	      var seriesEle, symbolEle;
		  var length = this.model._visibleSeries.length;
		  var svgObjectId = this.svgObject.id;
		  for (var i =0; i <length; i++){
		      var seriesEle = $("#" + svgObjectId + "_SeriesGroup_" + i);
		      var waterfallLineEle = $("#" + svgObjectId + '_SeriesGroup_waterfallLine_' + i);
			 var symbolEle = $("#" + svgObjectId + "_symbolGroup_" + i);
			 if (seriesEle.length >0 && symbolEle.length >0)
			     symbolEle.insertAfter(seriesEle);
			 if (seriesEle.length > 0 && waterfallLineEle.length > 0)
			     waterfallLineEle.insertAfter(seriesEle);
		  }
		  
	
		  this.model.AnimationComplete = true;
          var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
          commonEventArgs.data = {series : options};
          this._trigger("animationComplete", commonEventArgs);
		
		if (enableCanvasRendering) {
            this.svgRenderer.append(this.svgObject, this.element);
            if (this.panning) $("#" + this._id+ "_canvas").css({ "cursor": "pointer" });
        }       
		if (enable3D && !enableCanvasRendering) {
          this.svgRenderer.append(this.chart3D, this.svgObject);
           
          var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height() };
          this.graphics.prepareView(this.model.perspectiveAngle, this.model.depth, this.model.rotation, this.model.tilt, size, this);
          this.graphics.view(this.svgObject, this);
          this.svgRenderer.append(this.svgObject, this.element);
          //for (var i = 0; i < visibleSeries.length; i++) {
          //    if (visibleSeries[i].visibility.toLowerCase() == "visible" && visibleSeries[i].enableAnimation && !(this.model.animated)) {
          //        var seriesRendering = new ej.Ej3DSeriesRender();
          //        var type = visibleSeries[i].type.toLowerCase();
          //        var options = ej.series3DTypes[type];
          //        if (this.model.AreaType == "none" && i > 0)
          //            break;
          //        options.doAnimation(visibleSeries[i], seriesRendering);
          //    }

          //}

        }
      
       if (this.svgRenderer.vmlNamespace)
            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
        if (areaType == 'polaraxes')
            this.svgRenderer.append(this.gPolarAxisEle, this.svgObject);
        if ($(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").length > 0) {
            var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").detach();
            detachEle.appendTo(this.svgObject);
        }

        this._renderScrollbar();      
		
		this.legendRender.drawLegend(params);
		
        chartObj.model.initSeriesRender = true;
    },
     
    _calculateZoomValue: function (startX, endX, axis, scrollObj) {
        var model = this.model,
            delta, zoomPos, zoomFact, start, end,
            scrollRange = axis.scrollbarSettings.range,
            valueType = axis._valueType.toLowerCase();
        if (valueType == "datetime") {
            start = Date.parse(!ej.util.isNullOrUndefined(scrollRange.min) ? new Date(scrollRange.min) : startX);
            end = Date.parse(!ej.util.isNullOrUndefined(scrollRange.max) ? new Date(scrollRange.max) : endX);
        }else{
            start = parseFloat(!ej.util.isNullOrUndefined(scrollRange.min) ? scrollRange.min : startX);
            end = parseFloat(!ej.util.isNullOrUndefined(scrollRange.max) ? scrollRange.max : endX);
        }
        start = Math.min(start, startX);
        end = Math.max(end, endX);
        delta = end - start;
        if (!scrollObj.isRTL) {
            zoomPos = (startX - start) / delta;
            zoomFact = (endX - startX) / delta;
        } else {
            zoomPos = (end - endX) / delta;
            zoomFact = (endX - startX) / delta;
        }
        scrollObj.scrollRange = { min: start, max: end, delta: end - start };
        scrollObj._previousStart = startX;
        scrollObj._previousEnd = endX;            
        scrollObj.zoomPosition = zoomPos;
        scrollObj.zoomFactor = zoomFact;
    },
     
    _renderScrollbar: function () {

        var model = this.model,
            scrollSize = model.scrollerSize,
            scrollObj, index,  
            chartOffsetVal = $('#' + this._id).offset(),
            offsetVal = ($(this.svgObject).offset().left - chartOffsetVal.left),
            scrollerWidth, scrollerHeight, scrollerX,
            scrollerY, scrollerX, axis,
            zoomFac,zoomPos,opposed,
            axesCollections = model._axes,
            axesLength = axesCollections.length,inversed,
            zoomingScroll = model.zooming.enableScrollbar,range,
            areaType = model.AreaType,scrollbarSettings,pointLength,
            scrollEleLength,scrollRange, scrObj,
            orientation;

        model.scrollObj = model.scrollObj ? model.scrollObj : [];

      
        for (var r = 0; r < axesLength; r++) {
            axis = axesCollections[r];
            valueType = axis._valueType;
            scrollbarSettings = axis.scrollbarSettings;
            range = scrollbarSettings.range;
            inversed = axis.isInversed,
            isZoomAble = areaType == "cartesianaxes" && !model.enable3D;
            opposed = axis._opposed;
            pointLength = scrollbarSettings.pointsLength;
            orientation = axis.orientation.toLowerCase();
            zoomFac = axis.zoomFactor;
            scrollEleLength = $("#scrollbar_" + this.axisScroll[0].id + r).length;
            zoomPos = axis.zoomPosition;

            if (isZoomAble && ((axis._isScroll && model.isLazyZooming) || ((zoomFac < 1 || zoomPos > 0) && zoomingScroll))) {

                scrollerX = axis.x + ((orientation == "vertical") ? (-scrollSize + (opposed ? scrollSize : 0)) : 0) + (offsetVal <= 0 ? 0 : offsetVal);
                scrollerY = axis.y + ((orientation == "horizontal") ? (axis.height + (opposed ? -scrollSize : 0)) : 0);
                posY = scrollerY - (orientation == 'horizontal' ? 0 : axis.plotOffset);
                width = axis.plotOffset + (orientation == 'horizontal' ? axis.width : axis.height);
                scrObj = model.scrollObj;
                if (range.min != null || range.max != null)
                    isRedraw = scrObj[r] ? (scrObj[r].actualRange.min != range.min || scrObj[r].actualRange.max != range.max) : false;
                else
                    isRedraw = scrObj[r] ? (scrObj[r].actualRange != scrollbarSettings.pointsLength) : false;
                this.scrollDraw = scrObj[r] ? (scrObj[r].x != scrollerX || scrObj[r].y != scrollerY || scrObj[r].width != width) : this.scrollDraw;
                var options = {
                    'orientation': axis.orientation,
                    'index': r,
                    'width': width,
                    'x': scrollerX,
                    'y': posY,
                    'opposed': opposed,
                    'isRTL': inversed,
                    'parent': this.axisScroll,
                    'enableResize': scrollbarSettings.canResize,
                }
                if (model.isLazyZooming) {
                   
                    if (scrollEleLength == 0 || this.scrollDraw || isRedraw) {
                        $("#scrollbar_" + this.axisScroll[0].id + r).remove();
                        if (range.min != null || range.max != null) {
                            scrollRange = (scrObj[r] && scrObj[r].scrollRange) ? scrObj[r].scrollRange : axis.visibleRange;
                            if (!this.scrollDraw || isRedraw) {
                                scrObj[r] = {
                                    'isVirtual': true,
                                    'valueType': valueType,
                                    'scrollRange': scrollRange
                                };
                                scrObj[r].actualRange = $.extend(scrObj[r].actualRange, scrollbarSettings.range);
                                $.extend(scrObj[r], options);
                                if (valueType == 'datetime') {
                                    scrObj[r].startDateTime = new Date(axis.visibleRange.min);
                                    scrObj[r].endDateTime = new Date(axis.visibleRange.max);
                                } else {
                                    scrObj[r].startValue = axis.visibleRange.min;
                                    scrObj[r].endValue = axis.visibleRange.max;
                                }

                                this._calculateZoomValue(axis.visibleRange.min, axis.visibleRange.max, axis, this.model.scrollObj[r]);
                            } else {
                                scrObj[r].width = width;
                                scrObj[r].x = scrollerX;
                                scrObj[r].y = posY;
                            }
                        } else if (pointLength >= axis.maxPointLength) {
                            zoomFac = axis.maxPointLength / pointLength;
                            zoomPos = (inversed) ? (1 - axis.zoomPosition) : axis.zoomPosition;
                            if (!this.scrollDraw || isRedraw) {
                                scrObj[r] = {
                                    'isVirtual': true,
                                    'zoomPosition': zoomPos,
                                    'zoomFactor': zoomFac,
                                    'scrollRange': {
                                        'min': 0,
                                        'max': pointLength - 1,
                                        'delta': pointLength - 1
                                    },
                                    'valueType': 'double'
                                };
                                scrObj[r].actualRange = scrollbarSettings.pointsLength;
                                $.extend(scrObj[r], options);
                                scrObj[r].startValue = 0;
                                scrObj[r].endValue = axis.maxPointLength;
                            } else {
                                scrObj[r].width = width;
                                scrObj[r].x = scrollerX;
                                scrObj[r].y = posY;
                            }
                          
                        }
                        if (this.model.scrollObj[r]) {
                            this.scrollbarContainer = new ej.EjSvgScrollbarRender(this.axisScroll, this.model.scrollObj[r]);
                            this.scrollbarContainer._initializeScrollbarVariables(this.model.scrollObj[r]);
                            this.scrollbarContainer._renderScrollbar.call(this, this.model.scrollObj[r]);
                        }
                    }
                }
                else if (scrollbarSettings.visible) {

                    //* Started Resizable Scrollbar here *// 
                    scrObj = model.scrollObj;

                    if (scrollEleLength == 0 || this.scrollDraw) {
                        $("#scrollbar_" + this.axisScroll[0].id + r).remove();
                        scrObj[r] = {
                            'zoomPosition': zoomPos,
                            'zoomFactor': zoomFac,
                            'valueType': valueType,
                            'scrollRange': axis.actualRange,
                            'isZooming':true
                        };
                        $.extend(scrObj[r], options);
                        if (scrObj[r].valueType == 'datetime') {
                            scrObj[r].startDateTime = new Date(scrObj[r].scrollRange.min);
                            scrObj[r].endDateTime = new Date(scrObj[r].scrollRange.max);
                        } else {
                            scrObj[r].startValue = scrObj[r].scrollRange.min;
                            scrObj[r].endValue = scrObj[r].scrollRange.max;
                        }
                        this.scrollbarContainer = new ej.EjSvgScrollbarRender(this.axisScroll, this.model.scrollObj[r]);
                        this.scrollbarContainer._initializeScrollbarVariables(this.model.scrollObj[r]);
                        this.scrollbarContainer._renderScrollbar.call(this, this.model.scrollObj[r]);
                    }
                }
                
            } else {
                if (scrollEleLength > 0)
                    $("#scrollbar_" + this.axisScroll[0].id + r).remove();

            }
        }
        this.scrollDraw = false;
    },

     //Draw the trendline series
    _renderTrendline: function (chartObj, seriesOptions) {
		// declaration
        var seriesIndex = $.inArray(seriesOptions, chartObj.model.series),
        transX = chartObj.model.requireInvertedAxes ? seriesOptions.yAxis.x : seriesOptions.xAxis.x,
        transY = chartObj.model.requireInvertedAxes ? seriesOptions.xAxis.y :seriesOptions.yAxis.y,
        translate = 'translate(' + transX + ',' + transY + ')',
        trendlinesOptions = { 'id': chartObj.svgObject.id + '_TrendlinesGroup' + '_' + seriesIndex, 'clip-path': 'url(#' + chartObj.svgObject.id + '_TrendlinesGroup' + '_' + seriesIndex + '_ClipRect)' },
        trendlines = seriesOptions.trendlines, m, element, polynomialSlopes,  slopeLength,
        options, trendline, i, slope, intercept, displayText, text = "",
        length = trendlines.length, trendOptions, trendlineType, commonEventArgs, backwardPoints, forwardPoints;
		
		if (!chartObj.gTrendlinesGroupEle) chartObj.gTrendlinesGroupEle = [];
        chartObj.gTrendlinesGroupEle[seriesIndex] = chartObj.svgRenderer.createGroup(trendlinesOptions);
        for (i = 0; i < length; i++) {
            trendline = trendlines[i];
            trendline.seriesIndex = seriesIndex;
            trendline.isTrendLine = true;
            trendline.trendlineIndex = i;
            if (trendline.visibility.toLowerCase() === 'visible' && !trendline.isNull && trendline.points && trendline.points.length > 1) {
                trendOptions = { 'id': chartObj.svgObject.id + '_TrendGroup' + '_' + seriesIndex + '_' + i, 'transform': translate, 'clip-path': 'url(#' + chartObj.svgObject.id + '_TrendGroup' + '_' + seriesIndex + '_' + i + '_ClipRect)' };
                seriesOptions.gTrendGroupEle = null;
                seriesOptions.gTrendGroupEle = chartObj.svgRenderer.createGroup(trendOptions);
                trendlineType = trendline.type.toLowerCase();
                options = new ej.trendlineTypes[trendlineType]();
                slope = trendline.slope;
                intercept = trendline._intercept;
                switch (trendlineType) {
                    case "linear":
                        text = "y = " + slope.toFixed(4) + "x + " + intercept.toFixed(4);
                        break;
                    case "exponential":
                        text = "y = " + intercept.toFixed(3) + "e" + "^" + slope.toFixed(4) + "x";
                        break;
                    case "logarithmic":
                        text = "y = " + slope.toFixed(3) + "ln(x) + " + intercept.toFixed(4);
                        break;
                    case "power":
                        text = "y = " + intercept.toFixed(3) + "x^" + slope.toFixed(4);
                        break;
                    case "polynomial":
                        polynomialSlopes = trendline.polynomialSlopes;
                        slopeLength = polynomialSlopes.length;
                        text = "y = ";
                        for (m = slopeLength - 1; m >= 0; m--) {
                            displayText = polynomialSlopes[m];
                            text += displayText > 0 ? "+" + displayText : "-" + Math.abs(displayText);
                            if (m != 0)
                                text += "x^" + m + " ";
                        }
                        break;
                }
                trendline.equation = text;   // for equation

                ////Trigger Trendline rendering events
                commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                backwardPoints = trendline.points[0];
                forwardPoints = trendline.points[trendline.points.length - 1];
                commonEventArgs.data = { series: seriesOptions, trendline: trendline, forwardForecastPoints: forwardPoints, backwardForecastPoints: backwardPoints };
                this._trigger("trendlineRendering", commonEventArgs);

                if (chartObj.model.enableCanvasRendering) {
                    this.svgRenderer.ctx.save();
                    this.svgRenderer.ctx.beginPath();
                    this.svgRenderer.ctx.rect(chartObj.model.m_AreaBounds.X, seriesOptions.yAxis.y, seriesOptions.xAxis.width, seriesOptions.yAxis.height);
                    this.svgRenderer.ctx.clip();
                    this.svgRenderer.ctx.beginPath();
                    options.draw(trendline, seriesOptions, chartObj);
                    this.svgRenderer.ctx.restore();
                } else {
                    options.draw(trendline, seriesOptions, chartObj);
                    //Add axes bounds clip path for series
                    element = seriesOptions.gTrendGroupEle;
                    if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, seriesOptions, this);
                }
            }
        }

    },
    _renderStripline: function (zIndex) {
        $.extend(ej.EjStripline.prototype, this);
        this.chartstripline = new ej.EjStripline(this);
        var stripline = this.chartstripline;
		 this.model._stripeline = false;
        for (var sl = 0; sl < this.model._axes.length; sl++) {
            var slAxis = this.model._axes[sl];
            for (var slCount = 0; slCount < slAxis.stripLine.length; slCount++) {
                if (slAxis.stripLine[slCount].zIndex.toLowerCase() == zIndex)
                    stripline._drawStripline(slAxis, slAxis.stripLine[slCount],sl,slCount);
            }
        }
        if (zIndex == 'over' && this.model._stripeline) {

            this.svgRenderer.append(this.gStriplineOver, this.svgObject);
        } else if(this.model._stripeline){
            this.svgRenderer.append(this.gStriplineBehind, this.svgObject);
        }
    },

    renderSeries: function (options, params) {

        var chart = this;
        var axis;
        var index;
        var seriesIndex = $.inArray(options, this.model._visibleSeries);

        for (var i = 0; i < options.points.length; i++) {
            if (typeof options.points[i].x == "string" && options.points[i].x.indexOf("/Date(") != -1)
                options.points[i].x = new Date(parseInt(options.points[i].x.substr(6)));
        }
        options.xAxis = (options.xAxis === null || options.xAxis === undefined) ? this.model._axes[0] : options.xAxis;
        options.yAxis = (options.yAxis === null || options.yAxis === undefined) ? this.model._axes[1] : options.yAxis;
        var type = options.type.toLowerCase();
		var drawtype = options.drawType.toLowerCase();
        var symbolOptions;
        var translate = null;
        var visibility;

        if (this.model.AreaType == "cartesianaxes") {
            var trans = ej.EjSvgRender.utils._getTransform(options.xAxis, options.yAxis, this.model.requireInvertedAxes);
            var translate = 'translate(' + trans.x + ',' + trans.y + ')';
            visibility = 'visible';
        }
        else {
            visibility = (options.enableAnimation && !options._animatedSeries) ? 'hidden' : 'visible'
        }
        var txtOptions = { 'id': this.svgObject.id + '_TextGroup_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        var dataLabelOptions = { 'id': this.svgObject.id + '_DataLabel_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        if (type != "scatter")
            symbolOptions = { 'id': this.svgObject.id + '_symbolGroup_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        else
            symbolOptions = { 'id': this.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate };
        var connectorOptions = { 'id': this.svgObject.id + '_ConnectorGroup_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        if (!this.gSymbolGroupEle) this.gSymbolGroupEle = [];
        this.gSymbolGroupEle[seriesIndex] = this.svgRenderer.createGroup(symbolOptions);
	    if (!this.gSeriesTextEle) this.gSeriesTextEle = [];
        this.gSeriesTextEle[seriesIndex] = this.svgRenderer.createGroup(txtOptions);
		if (!this.gConnectorEle) this.gConnectorEle = [];
        this.gConnectorEle[seriesIndex] = this.svgRenderer.createGroup(connectorOptions);
		if (!this.gDataLabelEle) this.gDataLabelEle = [];
        this.gDataLabelEle[seriesIndex] = this.svgRenderer.createGroup(dataLabelOptions);

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { series: options };
        this._trigger("seriesRendering", commonEventArgs);

        var series = new ej.seriesTypes[type]();
        if (this.model.enableCanvasRendering && this.model.AreaType != "none") {
            this.svgRenderer.ctx.save();
            this.svgRenderer.ctx.beginPath();
            if (!this.model.requireInvertedAxes) { //checked for bar and stackingbar series
                this.canvasX = options.xAxis.x;
                this.canvasY = options.yAxis.y;
                this.svgRenderer.ctx.rect(options.xAxis.x, options.yAxis.y, options.xAxis.width, options.yAxis.height);
            } else {
                var translate = ej.EjSvgRender.utils._getTransform(options.xAxis, options.yAxis, this.model.requireInvertedAxes);
                this.canvasX = translate.x;
                this.canvasY = translate.y;
                this.svgRenderer.ctx.rect(options.yAxis.x, options.xAxis.y, options.yAxis.width, options.xAxis.height);
            }
            this.svgRenderer.ctx.clip();
            series.draw(this, options, params);
			this.svgRenderer.ctx.restore();
        }
        else if (this.model.enableCanvasRendering && this.model.AreaType == "polaraxes") {
            this.svgRenderer.ctx.save();
            this.svgRenderer.ctx.beginPath();
            this.svgRenderer.ctx.arc(this.model.centerX, this.model.centerY, this.model.Radius, 0, 2 * Math.PI, false);
            this.svgRenderer.ctx.clip();
            this.canvasX = this.canvasY = 0;
            series.draw(this, options);
            this.svgRenderer.ctx.restore();
        }
        else {
            this.canvasX = this.canvasY = 0;
            if (this.model.AreaType == "cartesianaxes")
                series.draw(this, options, params);
            else if (this.model.AreaType == "polaraxes" && (options._xAxisName == this.model._axes[0].name && options._yAxisName == this.model._axes[1].name))
                series.draw(this, options);
        }
        var errorbar = this.model.series[seriesIndex].errorBar,   
            areaType=this.model.AreaType;
        if (errorbar.visibility == 'visible' && areaType=='cartesianaxes' && (this.model.series[seriesIndex].type != "boxandwhisker"))
        	this.renderErrorBar(errorbar, options);                           // to render error bar
		
        //Add axes bounds clip path for series
        var element = series.gSeriesGroupEle;
        if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);

        // Draw Symbols       
            if (!(type == "candle" || type == "hilo" || type == "hiloopenclose" || type == "boxandwhisker")) {
                var chartObj = this;
				var point1;
				var areatype = chartObj.model.AreaType.toLowerCase();
				var regiontype = ej.seriesTypes[type].prototype.isRegion;
				if (!(type == "scatter")){
				    $.each(options._visiblePoints, function (pointIndex, point) {
				        if ((point.visible) && (point.x !== "") && ((point.marker && point.marker.visible) || (options.marker.visible))) {
				            //this condition provide for better Scatter performance
				            if ((areatype == "polaraxes" && drawtype == "column") || regiontype)
				                point1 = point.symbolLocation;
				            else
				                point1 = (areatype != "polaraxes")
                                ? ej.EjSvgRender.utils._getPoint(point, options) : ej.EjSvgRender.utils.TransformToVisible(options, point.xValue, point.y, chartObj)
				            if (areatype == "polaraxes") {
				                var labelFormat = options.yAxis.labelFormat ? options.yAxis.labelFormat : "";
				                var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
				                var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, options.marker.dataLabel.font);
				            }
                            if ((areatype == "polaraxes" && (options._xAxisName == chartObj.model._axes[0].name && options._yAxisName == chartObj.model._axes[1].name))|| areatype == "cartesianaxes" )
				                series.drawSymbol(seriesIndex, options, pointIndex, point1.X, point1.Y);
				        }
                });
				}
                if (type != "scatter") {
                    //Add axes bounds clip path for marker
                    if (options.marker.visible) {
                        element = this.gSymbolGroupEle[seriesIndex];
                       ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    }
                    // Add axes bounds clip path for marker text
                    if (options.marker.dataLabel.visible) {
                        element = this.gSeriesTextEle[seriesIndex];
                        ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
						 // for data label shape
						 element = this.gDataLabelEle[seriesIndex];
						 ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                      }
					// Add axes bounds clip path for connector line
					if (!ej.util.isNullOrUndefined(this.gConnectorEle) && this.gConnectorEle.length >0) {
					    element = this.gConnectorEle[seriesIndex];
                        ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
					}
                } else {
                    element = this.gSymbolGroupEle[seriesIndex];
                    ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    this.svgRenderer.append(element, this.gSeriesEle);

                    if (options.tooltip.visible || type == "scatter") {
                        if (!this.model.enableCanvasRendering)
                            var elements = $(this.gSymbolGroupEle).children().not("defs");
                        else
                            elements = this.model.series[seriesIndex].points;
                        if (this.vmlRendering) {
                            this.cloneSeriesEle = $(this.gSeriesEle).clone();
                            this.cloneobj = $(this.element).clone();
                            this.svgclone = $(this.svgObject).clone();
                            $(document.body).append(this.cloneobj);
                            this.svgRenderer.append(this.cloneSeriesEle, this.svgclone);
                            this.svgRenderer.append(this.svgclone, this.cloneobj);
                        }
                        else {
                            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
                            this.svgRenderer.append(this.svgObject, this.element);
                        }

                        if (this.vmlRendering) {
                            $(this.cloneobj).remove();
                            $(this.svgclone).remove();
                        }
                    }
                }
            }

        

        //Draw Series Text

        this.drawDisplayText(series, options, params);

    },
	
	 renderErrorBar:function(errorOptions,seriesOptions)
    {
        // declaration
        var chartobj = this,
        model = chartobj.model,
        currentseries = seriesOptions,
        type,mode,direction,fill,cap,
        capFill,capWidth,capLength,
        visiblePoints = currentseries._visiblePoints,
        len = visiblePoints.length,
        canvasX = chartobj.canvasX, canvasY = chartobj.canvasY,
        seriesIndex = $.inArray(currentseries, model._visibleSeries),
        translate = null,i,
        trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, model.requireInvertedAxes),
        verticalErrorValue,horizontalErrorValue,
        requireInvertedAxes = model.requireInvertedAxes,
        verticalPositive,verticalNegative,
        horizontalPositive,horizontalNegative,
        svgObjectId = chartobj.svgObject.id,
        areaBounds = chartobj.model.m_AreaBounds,
        translate = 'translate(' + trans.x + ',' + trans.y + ')',
        errorBarElement,errorBarGroup,event,errorBarPoint,
        pointIndex,point,yValue,xVal,standY,standX,
        location, verHighvalue, verLowvalue,
        horHighvalue, horLowvalue, mean, verMean, horMean,
        verStandSquareRoot, horStandSquareRoot, centervalueX, centervalueY,
        verStandardDeviationValue, horStandardDeviationValue,
        verCentervalue, horCentervalue,
        verStandardErrorValue, horStandardErrorValue,
        values, values1, verHighCapX1, verHighCapX2,
        verLowCapX1, verLowCapX2, horHighCapY1, horHighCapY2,
        horLowCapY1, horLowCapY2, eDirectionV, cDirectionV,
        eDirectionH, cDirectionH,eDirection,cDirection,
        fixvalue,pervalue,perY,perX,standDeviValue,standError,
        custom,capValues;

        // parent group creation
         errorBarGroup = {
            'id': svgObjectId + '_ErrorBarGroup' + '_' + seriesIndex,
            'transform': translate,
            'clip-path': 'url(#' + svgObjectId + '_ChartAreaClipRect)'
        };

        if (ej.util.isNullOrUndefined(chartobj.gErrorBarGroupEle))
            chartobj.gErrorBarGroupEle = [];
        errorBarElement =  chartobj.gErrorBarGroupEle[seriesIndex] = chartobj.svgRenderer.createGroup(errorBarGroup);

        commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);         
        commonEventArgs.data = {
            errorBar: errorOptions
        };
        this._trigger("errorBarRendering", commonEventArgs);      // trigger event

        event = commonEventArgs.data.errorBar;
        type = event.type;
        mode = event.mode;
        direction = event.direction;
        fill = event.fill;
        cap = event.cap;
        capFill = event.cap.fill;
        capWidth = event.cap.width;
        capLength = event.cap.length;
        verticalErrorValue = event.verticalErrorValue;
        horizontalErrorValue = event.horizontalErrorValue;
        verticalPositive = event.verticalPositiveErrorValue;
        verticalNegative = event.verticalNegativeErrorValue;
        horizontalPositive = event.horizontalPositiveErrorValue;
        horizontalNegative= event.horizontalNegativeErrorValue;
       
         for (i = 0; i < len; i++)
        {
             if (visiblePoints[i].isEmpty == false && visiblePoints[i].visible == true) {
                 // error bar group creation
                 errorBarPoint = {
                     'id': chartobj.svgObject.id + '_ErrorBarGroup' + '_' + seriesIndex + '_Point' + i
                 }
                 errorBarPointGroup = chartobj.svgRenderer.createGroup(errorBarPoint);
                 
                 pointIndex = i;
                 point = visiblePoints[i];
                 yValue = point.YValues[0];
                 xVal = point.xValue;
                 standY = visiblePoints[i].YValues[0];
                 standX = visiblePoints[i].xValue;

                 if (type.indexOf("column") || type.indexOf("bar") > -1)
                     location = point.symbolLocation;
                 else
                     location = point.location;
                 if (ej.util.isNullOrUndefined(location))
                        location = ej.EjSvgRender.utils._getPoint(point, currentseries);
                  centervalueX = location.X;
				  centervalueY = location.Y;
                 //error bar type calculation
                 switch (type) {             
                     case "fixedValue":         
                         fixvalue = point;
                         if (mode == 'vertical'||mode=='both') {          //vertical mode calculation for fixed value point location
                             fixvalue.YValues[0] = yValue + verticalErrorValue;
                              verHighvalue = ej.EjSvgRender.utils._getPoint(fixvalue, currentseries);
                             fixvalue.YValues[0] = yValue - verticalErrorValue;
                             verLowvalue = ej.EjSvgRender.utils._getPoint(fixvalue, currentseries);
                             if (!requireInvertedAxes)
                                 verHighvalue.X = verLowvalue.X = location.X;
                             else
                                 verHighvalue.Y = verLowvalue.Y = location.Y;
                         }
                         if (mode == 'horizontal' || mode == 'both') {
                             fixvalue.YValues[0] = yValue;
                             fixvalue.xValue = xVal;                    //horizontal mode calculation for fixed value point location
                             fixvalue.xValue = xVal + horizontalErrorValue;
                             horHighvalue = ej.EjSvgRender.utils._getPoint(fixvalue, currentseries);
                             fixvalue.xValue = xVal - horizontalErrorValue;
                             horLowvalue = ej.EjSvgRender.utils._getPoint(fixvalue, currentseries);
                         }
                         fixvalue.YValues[0] = yValue;
                         fixvalue.xValue = xVal;
                         break;

                     case "percentage":         
                         pervalue = point;
                         perY = verticalErrorValue / 100 * yValue;
                         perX = horizontalErrorValue / 100 * xVal;
                         if (mode == 'vertical'||mode=='both') {                  //vertical mode calculation for percentage point locaion
                             pervalue.YValues[0] = yValue + perY;
                             verHighvalue = ej.EjSvgRender.utils._getPoint(pervalue, currentseries);
                             pervalue.YValues[0] = yValue - perY;
                             verLowvalue = ej.EjSvgRender.utils._getPoint(pervalue, currentseries);
                             if (!requireInvertedAxes) 
                                 verHighvalue.X =verLowvalue.X= location.X;
                             else 
                                 verHighvalue.Y =verLowvalue.Y = location.Y;
                         }
                         if (mode == 'horizontal' || mode == 'both') {
                             pervalue.YValues[0] = yValue;
                             pervalue.xValue = xVal;                              //horizontal mode calculation for percentage
                             pervalue.xValue = xVal + perX;
                             horHighvalue = ej.EjSvgRender.utils._getPoint(pervalue, currentseries);
                             pervalue.xValue = xVal - perX;
                             horLowvalue = ej.EjSvgRender.utils._getPoint(pervalue, currentseries);
                         }
                         pervalue.YValues[0] = yValue;
                         pervalue.xValue = xVal;
                         break;

                     case "standardDeviation":          
                          standDeviValue = point;
                             mean = chartobj.meanCalculation(mode, visiblePoints);
                             verMean = mean.verMean;
                             horMean = mean.horMean;
                             verStandSquareRoot = mean.verStandSquareRoot;
                             horStandSquareRoot = mean.horStandSquareRoot;
                         if (mode == 'vertical'|| mode=='both') {                 
                             verStandardDeviationValue = verticalErrorValue * verStandSquareRoot;
                             standDeviValue.YValues[0] = verMean;
                             verCentervalue = ej.EjSvgRender.utils._getPoint(standDeviValue, currentseries);
                             yValue = standDeviValue.YValues[0];
                             standDeviValue.YValues[0] = yValue + verStandardDeviationValue;
                             verHighvalue = ej.EjSvgRender.utils._getPoint(standDeviValue, currentseries);
                             standDeviValue.YValues[0] = yValue - verStandardDeviationValue;
                             verLowvalue = ej.EjSvgRender.utils._getPoint(standDeviValue, currentseries);
                             if (!requireInvertedAxes) 
                                 verCentervalue.X = verHighvalue.X = verLowvalue.X = location.X;
                             else
                                 verCentervalue.Y = verHighvalue.Y = verLowvalue.Y = location.Y;
                             verCentervalue.X = verCentervalue.X + canvasX;
							 verCentervalue.Y = verCentervalue.Y + canvasY;
                         }
                         if (mode == 'horizontal' || mode == 'both') {
                             horStandardDeviationValue = horizontalErrorValue * horStandSquareRoot;
                             standDeviValue.xValue = horMean;
                             horCentervalue = ej.EjSvgRender.utils._getPoint(standDeviValue, currentseries);
                             if (!requireInvertedAxes)
                                 horCentervalue.Y = location.Y;
							 else
								 horCentervalue.X = location.X;
                             standDeviValue.YValues[0] = standY;
                             xVal = standDeviValue.xValue;
                             standDeviValue.xValue = xVal + horStandardDeviationValue;
                             horHighvalue = ej.EjSvgRender.utils._getPoint(standDeviValue, currentseries);  
                             standDeviValue.xValue = xVal - horStandardDeviationValue;
                             horLowvalue = ej.EjSvgRender.utils._getPoint(standDeviValue, currentseries);
							 horCentervalue.X = horCentervalue.X + canvasX;
                             horCentervalue.Y = horCentervalue.Y + canvasY;
							 
                         } 
                         standDeviValue.YValues[0] = standY;
                         standDeviValue.xValue = standX;
                        
                         break;
                     case "standardError":           
                         standError = point;
                         mean = chartobj.meanCalculation(mode, visiblePoints);
                         verStandSquareRoot = mean.verStandSquareRoot;
                         horStandSquareRoot = mean.horStandSquareRoot;
                         if (mode == 'vertical'||mode=='both') {                                   //vertical mode standard Error calculation
                             verStandardDeviationValue = verticalErrorValue * verStandSquareRoot;
                             verStandardErrorValue = verStandardDeviationValue / Math.sqrt(len);
                             standError.YValues[0] = yValue + verStandardErrorValue;
                             verHighvalue = ej.EjSvgRender.utils._getPoint(standError, currentseries);
                             standError.YValues[0] = yValue - verStandardErrorValue
                             verLowvalue = ej.EjSvgRender.utils._getPoint(standError, currentseries);
                             if (!requireInvertedAxes)
                                 verHighvalue.X = verLowvalue.X = location.X;
                             else
                                 verHighvalue.Y = verLowvalue.Y = location.Y;
                         }
                         if (mode == 'horizontal'||mode=='both') {                             //horizontal mode standard Error calculation
                             horStandardDeviationValue = horizontalErrorValue *horStandSquareRoot;
                             horStandardErrorValue = horStandardDeviationValue / Math.sqrt(len);
                             standError.YValues[0] = yValue;
                             standError.xValue = xVal;                      //horizontal mode calculation for  point location
                             standError.xValue = xVal + horStandardErrorValue;
                             horHighvalue = ej.EjSvgRender.utils._getPoint(standError, currentseries);
                             standError.xValue = xVal - horStandardErrorValue;
                             horLowvalue = ej.EjSvgRender.utils._getPoint(standError, currentseries);
                         }
                         standError.YValues[0] = yValue;
                         standError.xValue = xVal;
                         i = pointIndex;
                         break;

                     case "custom":
                         custom = point;
                         if (mode == 'vertical'||mode=='both') {                         //calculation for vertical mode point locaton
                             custom.YValues[0] = yValue + verticalPositive;
                             verHighvalue = ej.EjSvgRender.utils._getPoint(custom, currentseries);
                             custom.YValues[0] = yValue - verticalNegative;
                             verLowvalue = ej.EjSvgRender.utils._getPoint(custom, currentseries);
                             if (!requireInvertedAxes) 
                                 verHighvalue.X = verLowvalue.X=location.X;
                             else 
                                 verHighvalue.Y =verLowvalue.Y = location.Y;
                         }
                         if (mode == 'horizontal'||mode=='both') {
                             custom.YValues[0] = yValue;
                             custom.xValue = xVal;                     //calculation for horizontal mode point location
                             custom.xValue = xVal + horizontalPositive;
                             horHighvalue = ej.EjSvgRender.utils._getPoint(custom, currentseries);
                             custom.xValue = xVal - horizontalNegative;
                             horLowvalue = ej.EjSvgRender.utils._getPoint(custom, currentseries);
                         }
                         custom.YValues[0] = yValue;
                         custom.xValue = xVal;
                         break;
                 }
                
                 //to render cap 
                 if (cap.visible == true) {
                     if(mode=='vertical'||mode=='both')
                          values = { verHighvalue: verHighvalue, verLowvalue: verLowvalue }
                     if(mode=='horizontal'||mode=='both')
                          values1={   horHighvalue: horHighvalue,horLowvalue: horLowvalue}
                     capValues = chartobj.renderCap(cap, mode, values,values1, requireInvertedAxes); 
                     verHighCapX1 = capValues.verHighCapX1, verHighCapX2 = capValues.verHighCapX2,
                     verLowCapX1 = capValues.verLowCapX1,verLowCapX2 = capValues.verLowCapX2,
                     horHighCapY1 = capValues.horHighCapY1,horHighCapY2 = capValues.horHighCapY2,
                     horLowCapY1 = capValues.horLowCapY1,horLowCapY2 = capValues.horLowCapY2;
                 }
                 centervalueX = centervalueX + canvasX;
                 centervalueY = centervalueY + canvasY;
                 if (mode == 'vertical' || mode == 'both') {
                     verHighvalue.X = verHighvalue.X + canvasX;
                     verHighvalue.Y = verHighvalue.Y + canvasY;
                     verLowvalue.X = verLowvalue.X + canvasX;
                     verLowvalue.Y = verLowvalue.Y + canvasY;
                 }
                 if (mode == 'horizontal' || mode == 'both') {
                     horHighvalue.X = horHighvalue.X + canvasX;
                     horHighvalue.Y = horHighvalue.Y + canvasY;
                     horLowvalue.X = horLowvalue.X + canvasX;
                     horLowvalue.Y = horLowvalue.Y + canvasY;
                 }
    //drawing error bar and error bar cap  
             //drawing both direction    
             if (direction == 'both') {   
                  if (mode == 'vertical'|| mode =='both') {             //vertical mode for both direction
                       if (type != 'standardDeviation') 
                           eDirectionV = "M" + " " + (centervalueX) + " " + (centervalueY) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "M" + " " + (centervalueX) + " " + (centervalueY) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " ";
                       else if (type == 'standardDeviation') 
                           eDirectionV = "M" + " " + (verCentervalue.X) + " " + (verCentervalue.Y) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "M" + " " + (verCentervalue.X) + " " + (verCentervalue.Y) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " ";
                       if(!requireInvertedAxes)
                           cDirectionV = !cap.visible ? "" : "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighCapX1) + " " + (verHighvalue.Y) + " " + "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighCapX2) + " " + (verHighvalue.Y) + " " + "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowCapX1) + " " + (verLowvalue.Y) + " " + "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowCapX2) + " " + (verLowvalue.Y) + " ";
                       else
                           cDirectionV = !cap.visible ? "" : "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighCapX1) + " " + "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighCapX2) + " " + "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowCapX1) + " " + "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowCapX2) + " ";
                           eDirection = eDirectionV,cDirection = cDirectionV;
                   }
                   if (mode == 'horizontal' || mode == 'both') {
                       if (type != 'standardDeviation')                          //horizontal mode for both direction
                           eDirectionH = "M" + " " + (centervalueX) + " " + (centervalueY ) + " " + "L" + " " + (horHighvalue.X ) + " " + (horHighvalue.Y ) + " " + "M" + " " + (centervalueX ) + " " + (centervalueY ) + " " + "L" + " " + (horLowvalue.X ) + " " + (horLowvalue.Y ) + " ";
                       else if (type == 'standardDeviation')
                           eDirectionH = "M" + " " + (horCentervalue.X) + " " + (horCentervalue.Y ) + " " + "L" + " " + (horHighvalue.X ) + " " + (horHighvalue.Y) + " " + "M" + " " + (horCentervalue.X ) + " " + (horCentervalue.Y ) + " " + "L" + " " + (horLowvalue.X ) + " " + (horLowvalue.Y) + " ";
                       if (!requireInvertedAxes)
                           cDirectionH = !cap.visible ? "" :"M" + " " + (horHighvalue.X) + " " + (horHighvalue.Y ) + " " + "L" + " " + (horHighvalue.X ) + " " + (horHighCapY1) + " " + "M" + " " + (horHighvalue.X ) + " " + (horHighvalue.Y ) + " " + "L" + " " + (horHighvalue.X ) + " " + (horHighCapY2) + " " + "M" + " " + (horLowvalue.X ) + " " + (horLowvalue.Y ) + " " + "L" + " " + (horLowvalue.X ) + " " + (horLowCapY1) + " " + "M" + " " + (horLowvalue.X ) + " " + (horLowvalue.Y ) + " " + "L" + " " + (horLowvalue.X) + " " + (horLowCapY2) + " ";
                       else
                           cDirectionH = !cap.visible ? "" : "M" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " " + "L" + " " + (horHighCapY1) + " " + (horHighvalue.Y) + " " + "M" + " " + (horHighvalue.X ) + " " + (horHighvalue.Y ) + " " + "L" + " " + (horHighCapY2) + " " + (horHighvalue.Y) + " " + "M" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " " + "L" + " " + (horLowCapY1) + " " + (horLowvalue.Y ) + " " + "M" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " " + "L" + " " + (horLowCapY2) + " " + (horLowvalue.Y ) + " ";
                       eDirection = eDirectionH, cDirection = cDirectionH;
                   }
                    if (mode == 'both') {
                        eDirectionH += eDirectionV,cDirectionH += cDirectionV;
                        eDirection = eDirectionH,cDirection = cDirectionH;
                    }
             }
           //drawing plus direction
             else if (direction == 'plus') {  
                    if (mode == 'vertical'|| mode =='both') {        //vertical mode for plus direction
                            if (type != 'standardDeviation') 
                                  eDirectionV = "M" + " " + (centervalueX) + " " + (centervalueY) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " ";
                            else if (type == 'standardDeviation') 
                                eDirectionV = "M" + " " + (verCentervalue.X) + " " + (verCentervalue.Y ) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " ";
                                if (!requireInvertedAxes)
                                    cDirectionV = !cap.visible ? "" : "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighCapX1) + " " + (verHighvalue.Y) + " " + "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighCapX2) + " " + (verHighvalue.Y) + " ";
                                else
                                    cDirectionV = !cap.visible ? "" : "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighCapX1) + " " + "M" + " " + (verHighvalue.X) + " " + (verHighvalue.Y) + " " + "L" + " " + (verHighvalue.X) + " " + (verHighCapX2) + " ";
                            var eDirection = eDirectionV,cDirection = cDirectionV;
                    }
                    if (mode == 'horizontal' || mode == 'both') {
                        if (type != 'standardDeviation')                        //horizontal mode for both direction
                            eDirectionH = "M" + " " + (centervalueX) + " " + (centervalueY) + " " + "L" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " ";
                        else if (type == 'standardDeviation')
                            eDirectionH =  "M" + " " + (horCentervalue.X) + " " + (horCentervalue.Y) + " " + "L" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " " ;
                        if (!requireInvertedAxes)
                            cDirectionH = !cap.visible ? "" : "M" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " " + "L" + " " + (horHighvalue.X) + " " + (horHighCapY1) + " " + "M" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " " + "L" + " " + (horHighvalue.X) + " " + (horHighCapY2) + " " ;
                        else 
                            cDirectionH = !cap.visible ? "" : "M" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " " + "L" + " " + (horHighCapY1) + " " + (horHighvalue.Y) + " " + "M" + " " + (horHighvalue.X) + " " + (horHighvalue.Y) + " " + "L" + " " + (horHighCapY2) + " " + (horHighvalue.Y) + " ";
                        eDirection = eDirectionH,cDirection = cDirectionH;
                    }
                    if (mode == 'both') {
                        eDirectionH += eDirectionV,cDirectionH += cDirectionV;
                        eDirection = eDirectionH,cDirection = cDirectionH;
                    }
                }
          // drawing minus direction
                else
                {
                    if (mode == 'vertical' || mode == 'both') {             //vertical mode for both direction
                        if (type != 'standardDeviation') 
                             eDirectionV =  "M" + " " + (centervalueX) + " " + (centervalueY) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " ";
                        else if (type == 'standardDeviation') 
                            eDirectionV =  "M" + " " + (verCentervalue.X) + " " + (verCentervalue.Y) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " ";
                           if (!requireInvertedAxes)
                                cDirectionV = !cap.visible ? "" : "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowCapX1) + " " + (verLowvalue.Y) + " " + "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowCapX2) + " " + (verLowvalue.Y) + " ";
                            else
                                cDirectionV = !cap.visible ? "" :  "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowCapX1) + " " + "M" + " " + (verLowvalue.X) + " " + (verLowvalue.Y) + " " + "L" + " " + (verLowvalue.X) + " " + (verLowCapX2) + " ";
                        var eDirection = eDirectionV, cDirection = cDirectionV;
                    }
                    if (mode == 'horizontal' || mode == 'both') {
                        if (type != 'standardDeviation')                        //horizontal mode for both direction
                            eDirectionH =  "M" + " " + (centervalueX) + " " + (centervalueY) + " " + "L" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " ";
                        else if (type == 'standardDeviation')
                            eDirectionH =  "M" + " " + (horCentervalue.X) + " " + (horCentervalue.Y) + " " + "L" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " ";
                        if (!requireInvertedAxes)
                            cDirectionH = !cap.visible ? "" : "M" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " " + "L" + " " + (horLowvalue.X) + " " + (horLowCapY1) + " " + "M" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " " + "L" + " " + (horLowvalue.X ) + " " + (horLowCapY2) + " ";
                        else
                            cDirectionH = !cap.visible ? "" : "M" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " " + "L" + " " + (horLowCapY1) + " " + (horLowvalue.Y) + " " + "M" + " " + (horLowvalue.X) + " " + (horLowvalue.Y) + " " + "L" + " " + (horLowCapY2) + " " + (horLowvalue.Y) + " ";
                        eDirection = eDirectionH, cDirection = cDirectionH;
                    }
                    if (mode == 'both') {
                        eDirectionH += eDirectionV,cDirectionH += cDirectionV;
                        eDirection = eDirectionH,cDirection = cDirectionH;
                    }
                }
                                                 
                var errorBarOptions = {             //error bar path creation
                    'id': chartobj.svgObject.id + '_ErrorBar' + '_' + seriesIndex + '_Point' + pointIndex,
                    'd': eDirection,
                    'stroke': fill,
                    'stroke-width': errorOptions.width,
                    'opacity': errorOptions.opacity
                };
                var errorBarCapOptions = {       //error bar cap creation
                    'id': chartobj.svgObject.id + '_ErrorBarCap' + '_' + seriesIndex + '_Point' + pointIndex,
                    'd': cDirection,
                    'stroke':capFill,
                    'stroke-width':capWidth
                }
        //canvas rendering
                if (chartobj.model.enableCanvasRendering) {
                    this.svgRenderer.ctx.rect(areaBounds.X, areaBounds.Y, areaBounds.Width, areaBounds.Height);
                    this.svgRenderer.ctx.clip();
                    chartobj.svgRenderer.drawPath(errorBarOptions, null);              //To draw error bar for canvas
                    chartobj.svgRenderer.drawPath(errorBarCapOptions, null);           //To drae error bar cap for canvas
                }
                else {                                        //svg rendering
                    chartobj.svgRenderer.drawPath(errorBarOptions, errorBarPointGroup);
                    chartobj.svgRenderer.drawPath(errorBarCapOptions, errorBarPointGroup);
                    chartobj.svgRenderer.append(errorBarPointGroup, errorBarElement);
                }
            }
        }
        //append with series group
         chartobj.svgRenderer.append(errorBarElement, chartobj.gSeriesEle);
        //clip-path 
         ej.EjSvgRender.utils._drawAxesBoundsClipPath(errorBarElement,currentseries,chartobj);
    },
     
    meanCalculation:function(mode,visiblePoints)
    {
        var temp = temp1 = 0, total, verTotal, horTotal, verMean, horMean, ded,i,
            eachDev, squareEachDev, squareTotal, verEachDev, horEachDev,
            verSquareEachDev, verSquareTotal, horSquareTotal, horSquareTotal,
            verStandardMean, horStandardMean, verStandSquareRoot,
            horStandSquareRoot, values, len = visiblePoints.length;
        for (i = 0; i < len; i++) {
            if (mode == 'vertical') {             //vertical mode total calculation
                total = temp + visiblePoints[i].y;
                temp = total;
            }
            else if (mode == 'horizontal') {    //horizontal mode total calculation 
                total = temp1 + visiblePoints[i].xValue;
                temp1 = total;
            }
            else {                     //both mode  total calculation
                 verTotal = temp + visiblePoints[i].y;
                 temp = verTotal;
                 horTotal = temp1 + visiblePoints[i].xValue;
                 temp1 = horTotal;
            }
        }
         verMean = temp / len;
         horMean = temp1 / len;
         ded = len - 1;
        for (i = 0; i < len; i++) {
            if (mode == 'vertical') {
                eachDev = visiblePoints[i].y - verMean;
                squareEachDev = Math.pow(eachDev, 2);
                squareTotal = temp + squareEachDev;
                temp = squareTotal;
            }
            else if (mode == 'horizontal') {
                eachDev = visiblePoints[i].xValue - horMean;
                squareEachDev = Math.pow(eachDev, 2);
                squareTotal = temp1 + squareEachDev;
                temp1 = squareTotal;
            }
            else {
                verEachDev = visiblePoints[i].y - verMean;
                verSquareEachDev = Math.pow(verEachDev, 2);
                verSquareTotal = temp + verSquareEachDev;
                temp = verSquareTotal;
                horEachDev = visiblePoints[i].xValue - horMean;
                horSquareEachDiv = Math.pow(horEachDev, 2);
                horSquareTotal = temp1 + horSquareEachDiv;
                temp1 = horSquareTotal;
            }
        }
        verStandardMean = temp / ded;
        verStandSquareRoot = Math.sqrt(verStandardMean);
        horStandardMean = temp1 / ded;
        horStandSquareRoot = Math.sqrt(horStandardMean);
        values = { verStandSquareRoot: verStandSquareRoot, horStandSquareRoot: horStandSquareRoot, verMean: verMean, horMean: horMean }
        return values;
    },
    renderCap:function(cap,mode,values,values1,requireInvertedAxes)
    {
        //declaration
        var canvasX, canvasY, capLen, verHighvalue,
            verLowvalue, horHighvalue, horLowvalue,
            verHighCapX1, verHighCapX2, verLowCapX1,
            verLowCapX2, horHighCapY1, horHighCapY2,
            horLowCapY1, horLowCapY2,capValues;
        canvasX = this.canvasX;
        canvasY = this.canvasY;
        capLen = cap.length / 2;
        if (mode == 'vertical' || mode == 'both') {
            verHighvalue = values.verHighvalue;
            verLowvalue = values.verLowvalue;
        }
        if (mode == 'horizontal' || mode == 'both') {
            horHighvalue = values1.horHighvalue;
            horLowvalue = values1.horLowvalue;
        }
        if (mode == 'vertical' || mode == 'both') {
            if (!requireInvertedAxes) {
                 verHighCapX1 = verHighvalue.X+canvasX - capLen,
                 verHighCapX2 = verHighvalue.X+canvasX + capLen,
                 verLowCapX1 = verLowvalue.X+canvasX- capLen,
                 verLowCapX2 = verLowvalue.X+canvasX + capLen;
            }
            else {
                verHighCapX1 = verHighvalue.Y+canvasY + capLen;
                verHighCapX2 = verHighvalue.Y+canvasY - capLen;
                verLowCapX1 = verLowvalue.Y+canvasY + capLen;
                verLowCapX2 = verLowvalue.Y+canvasY - capLen;
            }
        }
        if (mode == 'horizontal' || mode == 'both') {
            if (!requireInvertedAxes) {
                 horHighCapY1 = horHighvalue.Y+canvasY - capLen,
                 horHighCapY2 = horHighvalue.Y+canvasY + capLen,
                 horLowCapY1 = horLowvalue.Y+canvasY - capLen,
                 horLowCapY2 = horLowvalue.Y+canvasY + capLen;
            }
            else {
                horHighCapY1 = horHighvalue.X+canvasX + capLen;
                horHighCapY2 = horHighvalue.X+canvasX - capLen;
                horLowCapY1 = horLowvalue.X+canvasX + capLen;
                horLowCapY2 = horLowvalue.X+canvasX - capLen;
            }
        }
            capValues = { verHighCapX1: verHighCapX1, verHighCapX2: verHighCapX2, verLowCapX1: verLowCapX1, verLowCapX2: verLowCapX2, horHighCapY1: horHighCapY1, horHighCapY2: horHighCapY2, horLowCapY1: horLowCapY1, horLowCapY2: horLowCapY2 }
        return capValues;
    },
	
	drawDisplayText: function (series, options, params){
	    // method to draw text 
	   
	        var chartObj = this;
			var point1;
			var type = options.type.toLowerCase();
	        var areatype = chartObj.model.AreaType.toLowerCase();
	        var regiontype = ej.seriesTypes[type].prototype.isRegion;
	        var drawtype = options.drawType.toLowerCase();
	        var pointMarker;
			var zoomed = false;
	        var seriesIndex = $.inArray(options, chartObj.model._visibleSeries);
	        if (ej.util.isNullOrUndefined(chartObj.model.regionCount))
                chartObj.model.regionCount = -1;	        
	         
            var isCanvas = chartObj.model.enableCanvasRendering;
            var isPolar = chartObj.model.AreaType.toLowerCase() == 'polaraxes';
            $.each(options._visiblePoints, function (pointIndex, point) {
                var type = chartObj.model.series[seriesIndex].type.toLowerCase();
			     options._visiblePoints[pointIndex].textOptionsBoxValues =[];
                pointMarker = point.marker;
                if ((areatype == "polaraxes" && drawtype == "column") || regiontype)
                    point1 = type.toLowerCase() == "boxandwhisker" ? point.dataLabelLocation : point.symbolLocation;
                else
                    point1 = (areatype != "polaraxes")
                    ? ej.EjSvgRender.utils._getPoint(point, options) : ej.EjSvgRender.utils.TransformToVisible(options, point.xValue, point.y, chartObj)
                if (point.visible && ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && options.marker.dataLabel.visible)) {
                    if (regiontype)
                        chartObj.model.regionCount++;

                    var type = chartObj.model.series[seriesIndex].type.toLowerCase();
					for (i = 0; i < chartObj.model._axes.length ; i++) {
					    if (chartObj.model._axes[i].zoomFactor < 1 || chartObj.model._axes[i].zoomPosition > 0)
					       { zoomed = true;
						   break;
						   }
					    }
                    //we have reset the chartRegions array based on the zooming points for column and stackingColumn series only
                    if ((chartObj.zoomed || options.xAxis._isScroll || zoomed) && (type == "column" || type == "stackingcolumn" || type == "stackingcolumn100")) {
                        for (var i = 0; i < chartObj.model.chartRegions.length; i++) {
                            if (seriesIndex == chartObj.model.chartRegions[i].SeriesIndex && pointIndex == chartObj.model.chartRegions[i].Region.PointIndex) {
                                if (isCanvas && chartObj.model.AreaType == "cartesianaxes") {
                                    chartObj.svgRenderer.ctx.save();
                                    chartObj.svgRenderer.ctx.beginPath();
                                    if (!chartObj.model.requireInvertedAxes) // checked for bar and stackingbar
                                        chartObj.svgRenderer.ctx.rect(options.xAxis.x, options.yAxis.y, options.xAxis.width, options.yAxis.height);
                                    else
                                        chartObj.svgRenderer.ctx.rect(options.yAxis.x, options.xAxis.y, options.yAxis.width, options.xAxis.height);
                                    chartObj.svgRenderer.ctx.clip();
                                    series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, i, params);
                                    chartObj.svgRenderer.ctx.restore();
                                }
                                else
                                    series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, i, params);
                            }
                        }
                    }
                    else {
                        if (isCanvas && chartObj.model.AreaType == "cartesianaxes") {
                            chartObj.svgRenderer.ctx.save();
                            chartObj.svgRenderer.ctx.beginPath();
                            if (!chartObj.model.requireInvertedAxes) // checked for bar and stackingbar
                                chartObj.svgRenderer.ctx.rect(options.xAxis.x, options.yAxis.y, options.xAxis.width, options.yAxis.height);
	                        else
	                            chartObj.svgRenderer.ctx.rect(options.yAxis.x, options.xAxis.y, options.yAxis.width, options.xAxis.height);
	                        chartObj.svgRenderer.ctx.clip();
							//Check regions for column type, bar type and waterfall series
							if(chartObj.model.chartRegions[chartObj.model.regionCount] || (type.indexOf('column')==-1 && type.indexOf('bar')==-1 && type.indexOf('waterfall')==-1)){
								if(type.toLowerCase() == "boxandwhisker")
								{
									for(var box = 0;box<point1.length;box++)
									{
										point.y = point.boxPlotLocation[box].YValues;
										series.drawDataLabel(options, pointIndex, point, point1[box].X,point1[box].Y, chartObj.model.regionCount, params,point1[box].outlier);
									}
								}
								else
									series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount, params);
								if (type == "rangecolumn" || type == "rangearea") {	                           
									point.y = point.low;
                                    series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount, params);
                                    point.y = point.high;
                                }
                            }
                            chartObj.svgRenderer.ctx.restore();
                        }
                        else {
                            if (isCanvas && isPolar) {
                                chartObj.svgRenderer.ctx.save();
                                chartObj.svgRenderer.ctx.beginPath();
                                chartObj.svgRenderer.ctx.arc(chartObj.model.centerX, chartObj.model.centerY, chartObj.model.Radius, 0, 2 * Math.PI, false);
                                chartObj.svgRenderer.ctx.clip();
                                point1.X -= chartObj.canvasX;
                                point1.Y -= chartObj.canvasY;
                            }
                            if ((type == "column" || type == "stackingcolumn" || type == "stackingcolumn100")) {
                                if (chartObj.model.chartRegions[chartObj.model.regionCount])
                                    series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount, params);
                            }
                            else {
									if(type.toLowerCase() == "boxandwhisker")
									{								
										for(var box = 0;box<point1.length;box++)
										{
										    point.y = point.boxPlotLocation[box].YValues;
										    series.drawDataLabel(options, pointIndex, point, point1[box].X, point1[box].Y, chartObj.model.regionCount, params, point1[box].outlier);
										}								
									}
									else
										series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount, params);
								}
                            if (type == "rangecolumn" || type == "rangearea") {
                                point.y = point.low;
                                series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount, params);
                                point.y = point.high;
                            }
                            if (isCanvas && isPolar)
                                chartObj.svgRenderer.ctx.restore();
                        }
                    }
                }
            });
	},
	
	// method to avoid collision between data labels 
	cartesianSmartLabels: function (currentSeries, points, seriesIndex) {
        // declaration
	    var length = points.length;
	    var count = 0;
	    connectorFlag = false;
	    this.processCount = 1;
	    this.posObj = {
	        top: [], rightTop: [],
	        left : [], leftTop : [],
	        right : [], rightTop : [],
	        rightBottom : [], leftBottom : [],
	        bottom: []
	    };
		var markerWidth, markerHeight, i, d, c, currentPoint, location,
		ele, leftValue, topValue;
	    // loop to store marker region
	    for (d = 0; d < this.model.series.length; d++) {
		    markerWidth = this.model.series[d].marker.size.width;
	        markerHeight = this.model.series[d].marker.size.height;
	        if (this.model.series[d].marker.visible) {
	            for (c = 0; c < this.model.series[d]._visiblePoints.length; c++) {
	                currentPoint = this.model.series[d]._visiblePoints[c];
					location = currentPoint.location;
	                this.model.markerRegion[this.model.markerRegion.length] = { seriesIndex: d, xPos: location.X + this.canvasX, yPos: location.Y + this.canvasY, width: markerWidth, height: markerHeight };
	            }
	        }
	    }
		
		var currentPoint, j, padding = 10, diff;
		var areaBounds = this.model.m_AreaBounds;
		var width, height, xPos, yPos;
		for (j=0; j< length; j++){   // to reposition the partially visible data labels
		    currentPoint = points[j];
			currentPoint.hide = false;
			xPos = currentPoint.xPos; yPos = currentPoint.yPos;
			width = currentPoint.width; height = currentPoint.height;
		    if (xPos - width/2 < 0 && xPos > -width/2){
		         diff = xPos - width/2;
		         currentPoint.textOptions.x = currentPoint.xPos = xPos + Math.abs(diff) + 10;
				 currentPoint.newConnectorFlag  = true;
		    }
			if (xPos + width/2 > areaBounds.Width && xPos + width/2 < areaBounds.Width + width/2){
			     diff = xPos - width/2;
		         currentPoint.textOptions.x = currentPoint.xPos = areaBounds.Width - width/2 -10;
				 currentPoint.newConnectorFlag  = true;
			}
		    if (yPos + width/2 > areaBounds.Height && yPos + height/2 < areaBounds.Height + height/2){
			     diff = yPos + height/2 - areaBounds.Height;
		         currentPoint.yPos = yPos - diff - 10 ;
				 currentPoint.textOptions.y = currentPoint.yPos + 5;
				 currentPoint.newConnectorFlag  = true;
			}
			 if (yPos - height/2 < 0 && yPos > -height/2){
		         diff = yPos - height/2;
		         currentPoint.yPos = currentPoint.yPos + Math.abs(diff) + 10;
				 currentPoint.textOptions.y = currentPoint.yPos +5; 
				 currentPoint.newConnectorFlag  = true;
		    }
		}
		
	    // loop to compare the points
	    for (i = 0; i < length; i++) {
	        if (!ej.util.isNullOrUndefined(points[i].yPos))
	          this.model.allPoints[this.model.allPoints.length] = points[i];
		    else
		      continue;
	        connectorFlag = false;
	        this.compareDataLabels(i, points, count, this.processCount); 
	        currentPoint = points[i];
	        currentPoint.connectorFlag = connectorFlag; // flag to draw connector lines
	        if (points[i].dataLabeltemplate) {    // for repositioning data label template
	            ele = $("#" + points[i].id);
	            leftValue = points[i].xPos;
	            topValue = !this.model.enableCanvasRendering ? points[i].yPos : points[i].yPos + this.model.m_AreaBounds.Y;
	            ele.css("left", leftValue).css("top", topValue);
	            if (this.model.AreaType == "cartesianaxes") {   // to avoid template rendering outside area bounds
	                areaBounds = this.model.m_AreaBounds;
	                xPosition = areaBounds.X + (this.model.requireInvertedAxes ? currentSeries.yAxis.plotOffset : currentSeries.xAxis.plotOffset);
	                yPosition = currentSeries.yAxis.y;
	                width = points[i].width;
	                height = points[i].height;
	                if ((leftValue > xPosition + areaBounds.Width || leftValue + width < xPosition || topValue + height < yPosition || topValue + height > areaBounds.Y + areaBounds.Height) ||
                        (this.zoomed && (leftValue < xPosition || topValue < yPosition || leftValue > xPosition + areaBounds.Width || topValue > yPosition + areaBounds.Height)))
	                    ele.remove();
	            }
	        }
	        else {
	            // to store data label position for furture processing
				this.storeDataLabelPositions (currentPoint);
	        }
	    }
	},
	
	storeDataLabelPositions: function (currentPoint){
	  var position = currentPoint.position;
	  var posObj = this.posObj;
	  switch(position){
				case "top":
				    posObj.top[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break;
				case "rightTop":
				    posObj.rightTop[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break;
				case "right":
				    posObj.right[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break;
				case "rightBottom":
				    posObj.rightBottom[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break
				case "bottom":
				    posObj.bottom[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break;
				case "left":
				    posObj.left[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break;
				case "leftBottom":
				    posObj.leftBottom[this.processCount - 1] = currentPoint;
					this.processCount = 1;
					break;
				case "leftTop":
				    posObj.leftTop[this.processCount - 1] = currentPoint;
					break;
				}
	},
	
	// method to avoid collision between data labels  - column series type
	cartesianColumnSmartLabels: function (currentSeries, points, seriesIndex) {
	    var length = points.length;
	    var count = 0;
	    connectorFlag = false;
	    this.processCount = 1;
		var i, currentPoint, ele;
	    // loop to compare the points
	    for (i = 0; i < length; i++) {
	        if(!ej.util.isNullOrUndefined(points[i].yPos) && !isNaN(points[i].yPos))
	            this.model.allPoints[this.model.allPoints.length] = points[i];
	        else
	            continue;
	        if (currentSeries._enableSmartLabels) {
	            connectorFlag = false;
	            this.compareColumnDataLabels(currentSeries, i, points, count, this.processCount);
	        }
	        currentPoint = currentSeries._visiblePoints[i];
	        if (points[i].dataLabeltemplate) {
	            ele = $("#" + points[i].id);
	            ele.css("left", points[i].xPos).css("top", points[i].yPos);
	        }
	    }
	},

	compareColumnDataLabels: function (currentSeries, i, points, count, processCount) {
	    var length = this.model.allPoints.length - 1;
		var prevLabel, currentLabel, padding;
		var type = currentSeries.type.toLowerCase();
	    for (var j = 0; j < length; j++) {
	        prevLabel = this.model.allPoints[j];
	        currentLabel = this.model.allPoints[this.model.allPoints.length - 1];
	        this.currentCollideLabel = prevLabel;
	        padding = 5;
	        collide = this.isCollide(prevLabel, currentLabel);
	        textPosition = currentSeries.marker.dataLabel.textPosition;
	        if (collide.state) {  
			    // for vertical orientation series types
	            if (type == "column" || type == "stackingcolumn" || type == "rangecolumn" || type == "waterfall") {
	                if (textPosition == "top" || textPosition == "middle")
	                    currentLabel.textOptions.y = currentLabel.yPos = currentLabel.yPos + collide.height + padding;
	                else
	                    currentLabel.textOptions.y = currentLabel.yPos = currentLabel.yPos - collide.height - padding;
	                currentLabel.textOptions.y = currentLabel.textOptions.y + padding + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	            } else {
				    // for horizontal orientation series types
	                if (textPosition == "top" || textPosition == "middle")
	                    currentLabel.textOptions.x = currentLabel.xPos = prevLabel.xPos - prevLabel.width/2 -currentLabel.width/2- padding;
	                else
	                    currentLabel.textOptions.x= currentLabel.xPos = prevLabel.xPos + prevLabel.width / 2 + currentLabel.width / 2 + padding;
	            }
	            this.compareColumnDataLabels(currentSeries, i, points, count, this.processCount);  // to call recursive
	        }
	    }
	},

    // method to comapare data labels and place without collision
	compareDataLabels: function (i, points, count, processCount) {
	    var length = this.model.allPoints.length;
		var j, padding, prevLabel, currentLabel;
	    var posObj = this.posObj;
	    for (j = 0; j < length; j++) {
	        prevLabel = this.model.allPoints[j];
	        currentLabel = this.model.allPoints[length - 1];
	        this.currentCollideLabel = prevLabel;
	        padding = 10;
	        collide = this.isCollide(prevLabel, currentLabel, j);
	        if (collide.state || collide.marker) {
	            connectorFlag = true;
	            switch (count) {
	                case 0:                        // top
	                    if (this.processCount > 1) this.resetValues(currentLabel);
	                    if (!collide.marker) {
	                        if (!ej.util.isNullOrUndefined(this.prevLabel)) {
	                            checkCollide = this.isCollide(this.prevLabel, currentLabel);
	                            if (!checkCollide.state) { this.prevLabel = prevLabel; this.processCount = 1; };
	                        } else {
	                            this.prevLabel = prevLabel;
	                            this.processCount = 1;
	                        }
	                    }
	                    else {
	                        this.prevLabel = collide.markerRegion;
	                        this.processCount = 1;
	                    }
	                                            // right
	                    this.resetValues(currentLabel);
	                    if (processCount > 1 && posObj.right[this.processCount - 2]) {
	                        prevLabel = posObj.right[this.processCount - 2];
	                         currentLabel.textOptions.x = currentLabel.xPos = prevLabel.xPos +  (prevLabel.width / 2 + currentLabel.width / 2 + padding);
	                    }
	                    else 
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos + (this.prevLabel.width / 2 + currentLabel.width / 2 + padding);
	                    count += 1;
	                    currentLabel.position = "right";
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;

	                case 1:                       
                              // right bottom
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos + this.prevLabel.width / 2 + currentLabel.width / 2 + padding;
	                    if (processCount > 1 && posObj.rightBottom[this.processCount - 2]) {
	                        prevLabel = posObj.rightBottom[this.processCount - 2];
	                      currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos + this.canvasY + prevLabel.height + padding;
	                    }
	                    else 
	                        currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "rightBottom";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
					

	                case 2:                       

					// bottom
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    if (processCount > 1 && posObj.bottom[this.processCount - 2]) {
	                        prevLabel = posObj.bottom[this.processCount - 2];
	                       currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos + currentLabel.height + padding;
	                    }
	                    else 
	                        currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "bottom";
	                    count += 1;
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 3:                         
 // left bottom
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos - this.prevLabel.width / 2 - currentLabel.width / 2 - padding;
	                    if (processCount > 1 && posObj.leftBottom[this.processCount - 2]) {
	                        prevLabel = posObj.leftBottom[this.processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos + this.canvasY + prevLabel.height + padding;
	                    }
	                       else 
	                        currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "leftBottom";
	                    count += 1;
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 4:                         
// left
	                   this.resetValues(currentLabel);
	                   if (processCount > 1 && posObj.left[this.processCount - 2]) {
	                       prevLabel = posObj.left[this.processCount - 2];
	                        currentLabel.textOptions.x = currentLabel.xPos = (prevLabel.xPos - prevLabel.width / 2 - currentLabel.width / 2 - padding);
	                    }
	                    else
	                        currentLabel.textOptions.x = currentLabel.xPos = (this.prevLabel.xPos - this.prevLabel.width / 2 - currentLabel.width / 2 - padding);
	                    currentLabel.position = "left";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 5:                        
 // left top
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos - this.prevLabel.width / 2 - currentLabel.width / 2 - padding;
	                    if (processCount > 1 && posObj.leftTop[this.processCount - 2]) {
	                        prevLabel = posObj.leftTop[this.processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = (prevLabel.yPos) - (currentLabel.height + padding);
	                    }
	                    else 
	                    currentLabel.textOptions.y = currentLabel.yPos = this.prevLabel.yPos - currentLabel.height - padding;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "leftTop";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 6:                         
//top

                        this.resetValues(currentLabel);
                        if (processCount > 1 && posObj.top[processCount - 2]) {
                            prevLabel = posObj.top[processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = (prevLabel.yPos) - (currentLabel.height + padding);
	                    }
	                    else 
	                    currentLabel.textOptions.y = currentLabel.yPos = (this.prevLabel.yPos) - (this.processCount * (currentLabel.height + padding));
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    count += 1;
	                    currentLabel.position = "top";
	                    this.compareDataLabels(i, points, count, this.processCount);
						break;
	                case 7:                        
// right top
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos + this.prevLabel.width / 2 + currentLabel.width / 2 + padding;
	                    if (processCount > 1 && posObj.rightTop[this.processCount - 2]) {
	                        prevLabel = posObj.rightTop[this.processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos - currentLabel.height - padding;     
	                    }
	                    else 
	                        currentLabel.textOptions.y = currentLabel.yPos = this.prevLabel.yPos - currentLabel.height - padding;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "rightTop";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 8:
	                    count = 0; 
	                    this.processCount += 1;
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	            }
	        }
	    }
	},
     //to draw connector lines for pieofpie series
	drawPieofPieConnectors: function (size, series) {
	    //if (series.connectorLine.width > 0) {
	        var visibility = (series.enableAnimation && !series._animatedSeries) ? 'hidden' : 'visible';
	        if (!this.model.enableCanvasRendering) {
	            if (!this.gConnectorLinesGroup)
	                this.gConnectorLinesGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_connectorLineGroup', 'visibility': visibility });
	            else (this.gConnectorLinesGroup.childNodes.length > 0)
	            	$(this.gConnectorLinesGroup.childNodes).remove();
	        }
	        var radius = [], startX = [], startY = [], loc = [],
	        chartStartAngle = -.5 * Math.PI, coefficient, pieCollections;
	        for (var j = 0; j < 2; j++) {
	            pieCollections = series.pieCollections[j];
	            coefficient = j == 0 ? series.pieCoefficient : series.pieOfPieCoefficient;
	            radius[j] = 0.25 * coefficient * Math.min(size.width, size.height);	           
	            startX[j] = this.model.circleCenterX[j];
	            startX[j] = (j == 0) ? startX[j] + this.model.pieGapWidth / 2 : startX[j] - this.model.pieGapWidth / 2;
	            startY[j] = this.model.circleCenterY[j];	           
	            if (pieCollections[0].start > 0 && pieCollections[0].end < Math.PI && j==0) {
	                var startAngle = pieCollections[0].start + chartStartAngle,
	                    endAngle = pieCollections[0].end + chartStartAngle - 0.000001;
	                var x1 = startX[j] + radius[j] * Math.cos(startAngle),
                        y1 = startY[j] + radius[j] * Math.sin(startAngle),
                        x2 = startX[j] + radius[j] * Math.cos(endAngle),
                        y2 = startY[j] + radius[j] * Math.sin(endAngle);	                   
	                }
	                else {
	                        x1 = startX[j] + radius[j] * Math.cos(chartStartAngle),
                            y1 = startY[j] + radius[j] * Math.sin(chartStartAngle),
                            x2 = x1,
                            y2 = y1 + radius[j] * 2;
	                }
	                loc.push({ X1: x1, Y1: y1, X2: x2, Y2: y2 });
	        }
	            var pieConnectorLines = series.connectorLine;
	            for (var k = 0; k < 2; k++) {
	                var lineOptions = {
	                    'id': this.svgObject.id + '_connectorLine'+k,
	                    'x1': k == 0 ? loc[k].X1: loc[k-1].X2,
	                    'y1': k == 0 ? loc[k].Y1 : loc[k-1].Y2,
	                    'x2': k == 0 ? loc[k + 1].X1 : loc[k].X2,
	                    'y2': k == 0 ? loc[k + 1].Y1 : loc[k].Y2,
	                    'stroke': pieConnectorLines.color,
	                    'stroke-width': pieConnectorLines.width,
	                    'opacity': pieConnectorLines.opacity,
	                    'stroke-dasharray': pieConnectorLines.dashArray
	                };	                
	                this.svgRenderer.drawLine(lineOptions, this.gConnectorLinesGroup);
	            }	                
	            this.svgRenderer.append(this.gConnectorLinesGroup, this.gSeriesEle);
	},
     // method to draw connector lines for data labels in cartesian axis
	drawConnectorLines: function (seriesIndex, index, currentPoint) {
	    var bottom = false;
		var drawConnectorLine = true;
		var xPos = currentPoint.xPos;
		var yPos = currentPoint.yPos;
		var areaBounds = this.model.m_AreaBounds;
	    var location = currentPoint.location ? currentPoint.location : currentPoint.symbolLocation;
	    if (location.Y > currentPoint.textOptions.y)
	        var endY = (currentPoint.textOptions.y);                         // top
	    else {
            bottom = true
	        endY = (currentPoint.textOptions.y - currentPoint.height / 2);  // bottom
	    }
	    if (currentPoint.dataLabeltemplate) {                               // for template
	        endY = currentPoint.yPos - this.model.m_AreaBounds.Y + currentPoint.height ;
	        currentPoint.textOptions.x = currentPoint.textOptions.x - this.model.m_AreaBounds.X + currentPoint.width/4 + this.canvasX;
	    } else if (xPos > areaBounds.Width || (xPos + currentPoint.width) < 0 ||
             		yPos - currentPoint.height > areaBounds.Height || yPos+ currentPoint.height < 0)
		    drawConnectorLine = false;
		if (drawConnectorLine){
	      var connectorDirection = "M" + " " + (location.X + this.canvasX) + " " + (location.Y + this.canvasY) + " " + "L" + " " + (currentPoint.textOptions.x) + " " + (endY);
	      var connectorLineValues = this.model.series[seriesIndex].marker.dataLabel.connectorLine;
	      var stroke = !ej.util.isNullOrUndefined(connectorLineValues.color) ? connectorLineValues.color : this.model.series[seriesIndex].fill;
	      var connectorOptions = {
	        'id': this.svgObject.id +  "_dataLabelConnectorLine_series" + seriesIndex + "_" + index,
	        'stroke': stroke,
	        'stroke-width': connectorLineValues.width,
	        'd': connectorDirection
	      };
	      this.model.series[seriesIndex].connectorLines = true;
	      this.svgRenderer.drawPath(connectorOptions, this.gConnectorEle[seriesIndex]);
		}
	},

    // to reset the data label values back 
	resetValues: function (currentPoint) {
	    currentPoint.textOptions.y = currentPoint.dataLabel.textY;
	    currentPoint.yPos = currentPoint.dataLabel.y;
	    currentPoint.textOptions.x = currentPoint.dataLabel.textX;
	    currentPoint.xPos = currentPoint.dataLabel.x
	},

	// to detect collision
	isCollide: function (a, b, j) {
	    var width = 0;
	    var height = 0;
		var state = false, markerState = false;
		var borderWidth = a.seriesIndex ? this.model.series[a.seriesIndex].marker.dataLabel.border.width : 0;
		borderWidth += b.seriesIndex ? this.model.series[b.seriesIndex].marker.dataLabel.border.width : 0;
		var series = this.model.series;
		var firstSeries = series[a.seriesIndex];
		var secondSeries = series[b.seriesIndex];
		var aXPos = a.xPos , aYPos= a.yPos, bXPos = b.xPos, bYPos = b.yPos;
		a.xPos += firstSeries.xAxis.x, a.yPos += firstSeries.yAxis.y,
        b.xPos += secondSeries.xAxis.x, b.yPos += secondSeries.yAxis.y;		
		
	    if (a != b) {
	        state = !(                    // to compare data labels
            ((a.yPos + a.height) < (b.yPos)) ||
            (a.yPos > (b.yPos + b.height)) ||
            ((a.xPos + a.width / 2 + borderWidth) < b.xPos - b.width / 2) ||
            (a.xPos - a.width/2 > (b.xPos + b.width/2)));
	        if (state) {
	            height = (a.yPos + a.height) - b.yPos;
	            width = (a.xPos + a.width) - b.xPos;
	        }
			a.xPos = aXPos, a.yPos = aYPos;
	    }
	    if (!state) {
	        for (var f = 0; f < this.model.markerRegion.length; f++) {
	            a = this.model.markerRegion[f];
				firstSeries = series[a.seriesIndex];
				aXPos = a.xPos , aYPos = a.yPos;
				a.xPos+= firstSeries.xAxis.x, b.yPos+= firstSeries.yAxis.y;
	            markerState= !(                // to compare data label and marker
                ((a.yPos + a.height) < (b.yPos - b.height/2)) ||
                (a.yPos > (b.yPos + b.height)) ||
                ((a.xPos + a.width) < b.xPos) ||
                (a.xPos > (b.xPos + b.width)));
	            if (markerState)
	                break;
	        }
	    }
		
		a.xPos = aXPos, a.yPos = aYPos;
		b.xPos = bXPos, b.yPos = bYPos;
		// to hide the partially visible data labels
		
	    return { state: state, marker: markerState, markerRegion: a, width: width, height: height };
	},

	drawAccDisplayText: function (series, options, seriesIndex) {
	    // method to draw text 
	    var chartObj = this;
		var pointsLength = options._visiblePoints.length;
		chartObj.firstStartAngle = [];
		type = options.type.toLowerCase();
		if ((type != "pyramid" && type != "funnel" && type != "pieofpie") && (options.leftsidePoints.length > 0 || options.rightsidePoints.length > 0)) {
				var rightsidePointsLength = options.rightsidePoints.length;
	            var leftsidePointsLength = options.leftsidePoints.length;

	            for (var i = 0; i < rightsidePointsLength; i++) {
	                options.rightsidePoints[i].index = i;
	               series.drawDataLabelAcc(chartObj, options, options.rightsidePoints[i].pointIndex, options.rightsidePoints[i], seriesIndex)
	            }
	            for (var j = leftsidePointsLength - 1; j >= 0; j--) {
	                options.leftsidePoints[j].index = j;
	                series.drawDataLabelAcc(chartObj, options, options.leftsidePoints[j].pointIndex, options.leftsidePoints[j],seriesIndex)
	            }
	        }
	        else {
	             for (var i = 0; i < pointsLength; i++) {
	                 options._visiblePoints[i].index = i;
					 var pointIndex = type == "pieofpie" ? options._visiblePoints[i].actualIndex : i;
	                if (isNaN(options._visiblePoints[i].startAngle) && options.type.toLowerCase() == "pie" && options.type.toLowerCase == "pieofpie") continue;
	                series.drawDataLabelAcc(chartObj, options, pointIndex, options._visiblePoints[i],seriesIndex,options.collectionIndex)
	             }
	        }
			 if (options.type.toLowerCase() == "pyramid" || options.type.toLowerCase() == "funnel")
                {
	        this.svgRenderer.append(this.gConnectorEle, this.gSeriesEle);

	        this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);

	        this.svgRenderer.append(this.gDataLabelEle, this.gSeriesEle);

	        this.svgRenderer.append(this.gSeriesTextEle, this.gSeriesEle);


	        this.svgRenderer.append(this.gSeriesEle, this.svgObject);

	        if (!this.vmlRendering)
               this.svgRenderer.append(this.svgObject, this.element)
	           
	            }
	},

    _processOData: function (series) {
        var chart = this,type = series.type.toLowerCase();
        var queryPromise = series.dataSource.executeQuery(series.query);
        queryPromise.done(function (e) {
            chart._processJsonData(e.result, series);
            series.visibility = 'visible';
            $(chart.svgObject).empty();
            $(chart.legendSvgContainer).empty();
            if (chart.model.enableCanvasRendering) {
                chart.svgRenderer.ctx.clearRect(0, 0, chart.svgObject.width, chart.svgObject.height);
                $("#canvas_trackSymbol").remove();
            }

            var redrawChart=true;
            var dec = (type != "pie" || type != "doughnut") ? 0 : chart.model.series.length - 1
            for (var i = dec, cond ; cond = (type != "pie" || type != "doughnut") ? i < chart.model.series.length : i >= 0; (type != "pie" || type != "doughnut") ? i++ : i--) {
                    if (chart.model.series[i].dataSource) {
                        if (chart.model.series[i].visibility == 'hidden') {
                            redrawChart = false;
                            break;
                        }
                    }
                }
            if (redrawChart) {
                chart.draw();
            }
        });
        queryPromise.fail(function (e) {
            series.visibility = 'visible';
            $(chart.svgObject).empty();
            $(chart.legendSvgContainer).empty();
            var redrawChart=true;
            for (var i = 0; i < chart.model.series.length; i++)
                if (chart.model.series[i].dataSource)
                {
                    if (chart.model.series[i].visibility == 'hidden') {
                        redrawChart = false;
                        break;
                    }
                }
            if (redrawChart) {
                chart.draw();
            }
        });
    },
    _processJsonData: function (jsonObj, series) {
	    if(series.points.length > 0)
           series.points.length = 0;
        var xdata = [];
        var dataSource = series.dataSource;
        var xName = series.xName;
        var yNames = series.yName;
        var pointColorMappingName = series.pointColorMappingName;
        var textMappingName = series.marker && series.marker.dataLabel.textMappingName;
        var pointColorMappingNameValue = [];
        var textMappingNameValue = [];
        var type = series.type.toLowerCase();
        var jsonLength = jsonObj.length, xDataLength, yValuesLength;

        for (var j = 0; j < jsonLength; j++) {
            if (typeof jsonObj[j][xName] == "string" && jsonObj[j][xName].indexOf("/Date(") != -1) {
                jsonObj[j][xName] = new Date(parseInt(jsonObj[j][xName].substr(6)));
                xdata[j] = ((jsonObj[j][xName]) ? jsonObj[j][xName] : j);
            }
            else
                xdata[j]= (jsonObj[j][xName]!=null && jsonObj[j][xName] !=undefined) ? jsonObj[j][xName] : null;
            pointColorMappingNameValue[j] =  (jsonObj[j][pointColorMappingName]!=null && jsonObj[j][pointColorMappingName] !=undefined) ? jsonObj[j][pointColorMappingName] : null;
            textMappingNameValue[j] =  (jsonObj[j][textMappingName]!=null && jsonObj[j][textMappingName] !=undefined) ? jsonObj[j][textMappingName] : null;
        }
        var yValues = [];
        var index = 0;
            if (series._hiloTypes) {
                var yVal = [];
                //For high
                var high = series.high;
                for (var s = 0; s < jsonLength; s++) {
                    yVal[s]=(jsonObj[s][high] === null || jsonObj[s][high] === undefined ? jsonObj[s][high] : parseFloat(jsonObj[s][high]));

                }
                yValues[index] = yVal;
                index = index + 1;
                //For low
                yVal = [];
                var low = series.low;
                for (var l = 0; l < jsonLength; l++) {
                    yVal[l]= (jsonObj[l][low] === null || jsonObj[l][low] === undefined ? jsonObj[l][low] : parseFloat(jsonObj[l][low]));
                }
                yValues[index] = yVal;
                index = index + 1;
                if (series.type.toLowerCase() !== 'hilo') {
                    //For open
                    yVal = [];
                    var open = series.open;
                    for (var h = 0; h < jsonLength; h++) {
                        yVal[h]=(jsonObj[h][open] === null || jsonObj[h][open] === undefined ? jsonObj[h][open] : parseFloat(jsonObj[h][open]));
                    }
                    yValues[index] = yVal;
                    index = index + 1;

                    //For close
                    yVal = [];
                    var close = series.close;
                    for (var mI = 0; mI < jsonLength; mI++) {
                        yVal[mI]=((jsonObj[mI][close] === null || jsonObj[mI][close] === undefined) ? jsonObj[mI][close] : parseFloat(jsonObj[mI][close]));
                    }
                    yValues[index] = yVal;
 index = index + 1;
                    //For volume
                    if (series.volume) {
                        yVal = [];
                        var volume = series.volume;
                        for (var vI = 0; vI < jsonLength; vI++) {
                            yVal[vI]=((jsonObj[vI][volume] === null || jsonObj[vI][volume] === undefined) ? jsonObj[vI][volume] : parseFloat(jsonObj[vI][volume]));
                        }
                        yValues[index] = yVal;
                    }
                }
            }
        if (yNames) {
				var yNVal = new Array();
				if(series.type=='boxandwhisker')
				{
					for (var yI = 0; yI < jsonLength; yI++) {
						yNVal[yI]=new Array();
						for(var yII=0;yII<jsonObj[yI][yNames].length ;yII++)
							yNVal[yI][yII]= ((jsonObj[yI][yNames][yII]) ? parseFloat(jsonObj[yI][yNames][yII]) : jsonObj[yI][yNames][yII]);
						}
				}else
				{
					for (var yI = 0; yI < jsonLength; yI++)
						yNVal[yI] = ((jsonObj[yI][yNames]) ? parseFloat(jsonObj[yI][yNames]) : jsonObj[yI][yNames]);
				}
                yValues[index] = yNVal;
                index = index + 1;
            

            if (series.type.toLowerCase() == 'bubble') {
                yVal = [];
                var size = series.size;
                for (var b = 0; b < jsonLength; b++) {
                    yVal[b]= (jsonObj[b][size] === null || jsonObj[b][size] === undefined ? jsonObj[b][size] : parseFloat(jsonObj[b][size]));
                }
                yValues[index] = yVal;
            }
        }
        xDataLength = xdata.length;
        yValuesLength = yValues.length;
        for (var m = 0; m < xDataLength; m++) {
            series.points[m] = { x: xdata[m], xValue: xdata[m], y: yValues[0][m], YValues: [],_visibility:(m == this.model._hiddenPointIndex[m])?'hidden':null, visible: true, fill: pointColorMappingNameValue[m], text: textMappingNameValue[m] };
            if (type == 'bubble') {
                series.points[m].YValues[0] = (yValues[0][m]);
                series.points[m].size = series.points[m].YValues[1] = yValues[1][m];
            }
            else if (series._hiloTypes && yValuesLength > 1) {
                series.points[m].high = series.points[m].YValues[0] = yValues[0][m];
                series.points[m].low = series.points[m].YValues[1] = yValues[1][m];
                if (type != 'hilo' && type != 'rangecolumn' && type != 'waterfall' && yValuesLength > 3) {
                    series.points[m].open = series.points[m].YValues[2] = yValues[2][m];
                    series.points[m].close = series.points[m].YValues[3] = yValues[3][m];
                if (series.volume)
                    series.points[m].volume = series.points[m].YValues[4] = yValues[4][m];
                }
            }
            else {
                series.points[m].YValues[0]=(yValues[0][m]);
            }
        }
    },
    bindTo: function (excludeDataUpdate) {
          $("#" + this.svgObject.id + '_CrosshairVertical').remove();
          $("#" + this.svgObject.id + '_CrosshairHorizontal').remove();
          if ($.finish) {
              $(".ejTooltip" + this._id).finish();
              $(".tooltipDiv" + this._id).finish();
          }
          else {
              $(".ejTooltip" + this._id).stop(true, true);
              $(".tooltipDiv" + this._id).stop(true, true);
          }
		  $(document).find('[id*="_TrackToolTip"]').remove();// fixed for mobile issue
          $('#template_group_' + this._id).remove();

          //Create id to chart container, if the element don't have id
		  if (!this.element[0].id)
		      this._createChartId();

		  this.model.primaryXAxis._valueType = null;
		  this.model.primaryXAxis.position = null;
		  this.model.primaryYAxis._valueType = null;
		  this.model.primaryYAxis.position = null;
		  this.model.requireInvertedAxes = false;
		  this.model._hiddenPointIndex = [];
		  var seriesLength = this.model.series.length,
		  axesLength = this.model.axes.length,
          axis;
         for (var i = 0; i < axesLength; i++)
		  {
             axis = this.model.axes[i];
             axis._valueType = null;
             axis.position = null;
         }
		  if (this.model._axes)
         {
		       axesLength = this.model._axes.length;
		       for (var j = 0; j < axesLength; j++) {
		           axis = this.model._axes[j];
		           axis._valueType = null;
		           axis.position = null;
		           axis.zoomFactor = (axis._pointsLength && axis._pointsLength != axis.scrollbarSettings.pointsLength) ? 1 : axis.zoomFactor;
		       }
         }
		  for (var i = 0; i < seriesLength; i++) {
		      var series = this.model.series[i];
		      series.position = null;
		      if (series.fill && this.model.seriesColors && series.fill != this.model.seriesColors[i])
		          this.model.series[i].isFill = true;      
		  }
		  if (this.model.AreaType && this.model.AreaType == "none") {
		      var series = this.model.series[0];
			  var point;
		      for (var i = 0; series.points && i < series.points.length; i++) {
		          point = series.points[i]
		          point._visibility = point._visibility ? point._visibility : null;
				  if (point._visibility == "hidden" && this.model.AreaType == "none" && this.model.series.length == 1)
                        this.model._hiddenPointIndex[i] = i;
		          if (point.fill && this.model.pointColors && point.fill != this.model.pointColors[i])
		              point.isFill = true;
		          if (!point.isFill)  
		              point.fill = null;		           
		      }
		  }
          if(this.model.enable3D)
            ej.Ej3DRender.transform = null;
          if (this.model.series) {
            var series = this.model.series;
            this.setModelProperties(excludeDataUpdate);
            if (!excludeDataUpdate) {
                for (var i = 0; i < series.length; i++) {
                    if (series[i].visibility == null || series[i].visibility == undefined) {
                        series[i].visibility = 'visible';
                    }
					 if (series[i].type) series[i]._hiloTypes = ej.seriesTypes[series[i].type.toLowerCase()].prototype.hiloTypes;
                    if (!(series[i].points))
                        series[i].points = [];
                    if (series[i].dataSource) {
                        if (series[i].dataSource instanceof ej.DataManager) {
                            series[i].visibility = 'hidden';
						    if (ej.isNullOrUndefined(series[i].query) || !(series[i].query instanceof ej.Query))
                                series[i].query = ej.Query();
                            this._processOData(series[i]);
                        } else if (series[i].dataSource.length > 0) {
                            if (typeof series[i].dataSource === 'string' || series[i].dataSource.toLowerCase)
                                series[i].dataSource = JSON.parse(series[i].dataSource.replace(/'/g, "\""));
                            this._processJsonData(series[i].dataSource, series[i]);

                        }
                    }
                }
            }
            if($(this.svgObject).width()>0)
            this.draw(excludeDataUpdate);
        }



    },

    _createChartId: function () {
        var exsistId = false, count = 0;
        var className = $(this.element)[0].className.split(" ")[0];
        do {
            if (!$("#" + className + "_" + count).length) {
                $(this).attr("_id", className + "_" + count);
                $(this.element).attr('id', className + "_" + count);
                exsistId = true;               
            }
            count++;
        } while (!exsistId);
    },

	calculateHeight : function(chart){              // height calculation
	     var $svgObj = $(chart.svgObject);
	     var chartHeight = chart.model.size.height;
         var containerHeight= $(chart.element).height();
         var height = (containerHeight / 100) * parseInt(chartHeight);
		 $svgObj.height(height);
		 return height;
	},
	
	calculateWidth : function(chart){               // width calculation
	       var $svgObj = $(chart.svgObject);
	       var chartWidth = chart.model.size.width;
           var containerWidth = $(chart.element).width();
           var width = (containerWidth / 100) * parseInt(chartWidth);
		   $svgObj.width(width);
		   return width;
	},

     // store the element class logic perform here
	selectedStyle: function (chart) {
	    var selected= $('#' + chart._id).find('[class*="Selection"]');
	    var selection = [];
	    for (var i = 0; i < selected.length; i++) {
	        selection[i] = [];
	        selection[i].id = selected[i].id;
	        selection[i].className = $('#' + selection[i].id).attr('class');
	    }
	    if ($('[id $= Def ]').length > 0) {
	        selection.pattern = $('[id $= Def ]');
	    }

	    return selection;
	},
	
	chartResize: function () {
        var chart = this;
        var $svgObj = $(chart.svgObject);
        var selection = this.selectedStyle(chart);
	    this.removeMultiRect();
        this._chartResize = true;
        if (this.model.enableCanvasRendering) {
            $('[id*=' + this._id + '_Selection_' + ']').remove();
            $("#canvas_trackSymbol").remove();
            var ctx = chart.svgObject.getContext("2d");
            ctx.clearRect(0, 0, this.svgRenderer.svgObj.width, this.svgRenderer.svgObj.height);
			
			if (typeof chart.model.size.width=="string" && chart.model.size.width.indexOf("%") != -1) {           // to set width 
                var width = chart.calculateWidth(chart);
				this.svgRenderer.svgObj.width = width;
            }
			else
              this.svgRenderer.svgObj.width = $("#" + this._id).width();
            
			if (typeof chart.model.size.height=="string" && chart.model.size.height.indexOf("%") != -1) {          // to set height
                 var height = chart.calculateHeight(chart);
				  this.svgRenderer.svgObj.height = height;
            }
            else
			 this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : parseInt($(chart.element).height());
			

        }
        if (this.resizeTO) clearTimeout(this.resizeTO);
        this.resizeTO = setTimeout(function () {
          if (!ej.util.isNullOrUndefined(chart.model)) {
            if (typeof chart.model.size.width == "string" && chart.model.size.width.indexOf("%") != -1) {
               chart.calculateWidth(chart);
            }
            else
                $svgObj.width($(chart.element).width());
      
            if (typeof chart.model.size.height == "string" && chart.model.size.height.indexOf("%") != -1) {
                 chart.calculateHeight(chart);
            }
            else
                $svgObj.height($(chart.element).height());
            if ($(chart.svgObject).width() > 0) {
                $(chart.svgObject).empty();
	        $(chart.legendSvgContainer).empty();
                $('#template_group_' + chart._id).remove();
				$("#annotation_group_" + chart._id).remove();       //to remove annotation from DOM on resize
                chart.disableAnimation();
                for (var j = 0; j < chart.model.series.length; j++)
                    chart.model.series[j].regionAdded = false;
                chart.draw();
                for (var i = 0; i < selection.length; i++) {
                    $('#' + selection[i].id).attr('class', selection[i].className);
                }
                if(selection.pattern)
                    chart.svgRenderer.append(selection.pattern, chart.svgObject);
                if (chart.zoomed && !chart.vmlRendering)
                    chart._enableZoomingButtons();
                chart.enableAnimation();
            }
		  }
        }, 500);

    },
    isDevice: function () {
        return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        // comment above line temporary. Due to the below code event wont bind for tablet device
        //  return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
    },

     //To check whether the device is Windows
    isWindows: function () {
        if (!ej.getBooleanVal($('head'), 'data-ej-android') && !ej.getBooleanVal($('head'), 'data-ej-ios') && !ej.getBooleanVal($('head'), 'data-ej-ios7') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._windows();
    },

    _windows: function () {
        return (/trident|windows phone/i.test(navigator.userAgent.toLowerCase())) || (ej.getBooleanVal($('head'), 'data-ej-windows', false) === true);
    },

    bindResizeEvents: function () {
        if (this.model._resizeEventRegistered)
            return 0;
        if (ej.isTouchDevice() && this._isOrientationSupported())
            this._on($(window), "orientationchange", this.chartResize);
        else
            this._on($(window), "resize", this.chartResize);
        this.model._resizeEventRegistered = true;
    },
    removeResizeEvents: function () {
        if (this.model._resizeEventRegistered) {
            if (ej.isTouchDevice() && this._isOrientationSupported())
                this._off($(window), "orientationchange", this.chartResize);
            else
				this._off($(window), "resize", this.chartResize);
            this.model._resizeEventRegistered = false;
        }
    },
	 _isOrientationSupported: function () {
            return ("orientation" in window && "onorientationchange" in window);
	 },
    _calculatePinchZoomPosition: function (e) {
        var areaBounds = this.model.m_AreaBounds, zoomPosition, axes = this.model._axes, axis,
            event = (e.originalEvent.touches ? e.originalEvent.touches[0] : e.originalEvent),
            pageX = event.pageX, pageY = event.pageY, panTouch = this.previousPanTouch, pinchPan,
            orientation, offset, j = 0, length = axes.length;

        if (pageX > areaBounds.X && pageX < areaBounds.X + areaBounds.Width && pageY > areaBounds.Y && pageY < areaBounds.Y + areaBounds.Height) {
            if (!panTouch)
                panTouch = this.previousPanTouch = { pageX: pageX, pageY: pageY };
				
			if(ej.isTouchDevice() || this.model.zooming.enableDeferredZoom)
			{
				this.oPreviousCoords = { x: this.previousPanTouch.pageX, y: this.previousPanTouch.pageY };
				return this.chartMouseMove(e); //Deferred panning for slow devices
			}
			
            for (; j < length; j++) {
                axis = axes[j];
                orientation = axis.orientation.toLowerCase() == "horizontal";
                offset = orientation ? ((panTouch.pageX - pageX) / axis.width * axis.zoomFactor) : ((panTouch.pageY - pageY) / axis.height * axis.zoomFactor);
                zoomPosition = axis.zoomPosition;
                axis.zoomPosition = this._ensureValueInMinMax(orientation ? zoomPosition + offset : zoomPosition - offset, 0, 1 - axis.zoomFactor);
                pinchPan = pinchPan || (zoomPosition != axis.zoomPosition);
            }
            this.previousPanTouch = { pageX: pageX, pageY: pageY };	
            if (pinchPan && !this.chartUpdating) {
                this.redraw(true, pinchPan, e.target);
                this._enableZoomingButtons();
            }
        }

    },
    _calculateTouchDistance: function (previous, current) {
        var result = [], j = 0, length = current.length;

        for (; j < length; j++) {
            if (previous[j] == null)
                previous[j] = current[j];
            result[j] = {};
            if (j > 0) {
                result[j].scaleX = ((previous[0].pageX != previous[1].pageX) && ((current[0].pageX - current[1].pageX) / (previous[0].pageX - previous[1].pageX)));
                result[j].scaleY = ((previous[0].pageY != previous[1].pageY) && ((current[0].pageY - current[1].pageY) / (previous[0].pageY - previous[1].pageY)));
                result[j].center = { x: (previous[0].pageX + current[0].pageX) / 2, y: (previous[1].pageY + current[1].pageY) / 2 };
            }
        }
        return result;
    },
    _ensureValueInMinMax: function (valueToCheck, minValue, maxValue) {
        return valueToCheck < minValue ? minValue : (valueToCheck > maxValue ? maxValue : valueToCheck);
    },
    _calculatePinchZoomFactor: function (scale, orientation, target) {
        var model = this.model, axes = model._axes, zoomedOut = true, zoomed, currentScale, origin, cumulativeScale,
            axisScale, axis, factor, zoomFactor, zoomPosition, areaBounds = model.m_AreaBounds,
            prevScale, scaleLimit = 10000, i = 0, length = axes.length;

        orientation = (orientation == 'x' ? 'horizontal' : (orientation == 'y' ? 'vertical' : null));

        if (this.previousScale != null) {
            for (; i < length; i++) {
                if (!orientation || axes[i].orientation === orientation) {
                    axis = axes[i];
                    currentScale = axis.orientation == 'horizontal' ? scale[1].scaleX : scale[1].scaleY;
                    if (!currentScale)
                        break;
                    prevScale = axis.orientation == 'horizontal' ? this.previousScale[1].scaleX : this.previousScale[1].scaleY;
                    zoomFactor = axis.zoomFactor;
                    zoomPosition = axis.zoomPosition;
                    axisScale = this._ensureValueInMinMax(1 / axis.zoomFactor, 1, scaleLimit);
                    cumulativeScale = this._ensureValueInMinMax(axisScale + (axisScale * (currentScale - prevScale) / prevScale), 1, scaleLimit);
                    origin = (axis.orientation == 'horizontal' ? (scale[1].center.x / areaBounds.Width) : (1 - scale[1].center.y / areaBounds.Height));
                    axis.zoomFactor = this._ensureValueInMinMax(1 / cumulativeScale, 1 / scaleLimit, 1);
                    axis.zoomPosition = this._ensureValueInMinMax(zoomPosition + (zoomFactor - axis.zoomFactor) * origin, 0, 1-axis.zoomFactor);
                    zoomed = zoomed || (zoomFactor != axis.zoomFactor);
                    zoomedOut = zoomedOut && (axis.zoomFactor == 1);
                }
            }
            if (zoomed && !zoomedOut && !this.chartUpdating) {
                this.zoomed = true;
                this.redraw(true, true, target);
                this._enableZoomingButtons();
            }
            else if (zoomedOut && this.zoomed && !this.chartUpdating) {
                this.redraw(true, true, target);
                this.zoomed = false;
                this._removeZoomkit();
            }
        }
        this.previousScale = scale;
    },
    _addTouchPointer: function (touches, e, overwrite, j) {
        for (j = touches.length; j--;)
            if (e.pointerId == touches[j].pointerId) {
                if (overwrite) touches[j] = e;
                return 0;
            }
        touches.push(e);
    },
    _removeTouchPointer: function (touches, e, j) {
        for (j = touches && touches.length; j--;)
            if (e.pointerId == touches[j].pointerId) {
                touches.splice(j, 1);
                return 0;
            }
    },
    _copyTouches: function (touches) {
        var target = [], i = 0, length = touches.length;
        for (; i < length; i++)
            target[i] = { pageX: touches[i].pageX, pageY: touches[i].pageY };
        return target;
    },
    _initEventParams: function () {
        return { touches: [], movements: [] };
    },
    _pointerPinchStart: function (e) {
        var event = e.originalEvent, targetId, model = this.model, bounds = model.m_AreaBounds, eventObj, elementOffset = $(this.element).offset(),
            params = this.eventParams || (this.eventParams = this._initEventParams()),
            x = (event.pageX || event.changedTouches[0].pageX) - elementOffset.left, y = (event.pageY || event.changedTouches[0].pageY) - elementOffset.top;
			
		if(this.model.selectionEnable){
			this.cancelEvent(e);
			event.preventDefault();
		}

        if (!model.enable3D && model.AreaType == 'cartesianaxes' && event.pointerType != 'mouse' && event.pointerType != 4 && model.zooming.enable && model.zooming.enablePinching && (event.touches || event.pointerId)) {
            targetId = e.target.id;
            if (targetId.indexOf("ResetZoom") != -1)
                this.resetZoom(e);
            else if (targetId.indexOf("panIcon") != -1)
                this.startPan(e);
            else if (targetId.indexOf("ZoomIcon") != -1)
                this.startZoom(e);
            else if (targetId.indexOf("ZoomOut") != -1 || targetId.indexOf("ZoomIn") != -1)
                this.startZoomInOut(e);
            else if (x > bounds.X && x < bounds.X + bounds.Width && y > bounds.Y && y < bounds.Y + bounds.Height) {
                event.touches ? (params.touches = this._copyTouches(event.touches)) : this._addTouchPointer(params.touches, { pageX: x, pageY: y, pointerId: event.pointerId });
                if (params.touches.length < 2)
                    this.chartMouseDown(e);
            }
            else
                this.chartMouseDown(e);
        }
        else
            this.chartMouseDown(e);
    },
    _removeInteractions: function () {
        $("[id*=_TrackSymbol]").remove();
        this._removeTrackBall();
        this._removeHighlight();
        $("#" + this.svgObject.id + "_TrackToolTip").hide();
        $("#" + this._id + "_tooltip").remove();
    },
    _pinchGestureMove: function (e) {
        var event = e.originalEvent.changedTouches[0], touches = e.originalEvent.touches, model = this.model, elementOffset = $(this.element).offset(),
            areaBounds = model.m_AreaBounds, x = event.pageX - elementOffset.left, y = event.pageY - elementOffset.top;

        this.eventParams = this.eventParams || this._initEventParams(model, event);
        if (!model.enable3D && model.AreaType == 'cartesianaxes' && event.pointerType != 'mouse' && event.pointerType != 4 && model.zooming.enable && model.zooming.enablePinching && touches) {
            if (x > areaBounds.X && x < areaBounds.X + areaBounds.Width && y > areaBounds.Y && y < areaBounds.Y + areaBounds.Height) {
                this._removeInteractions();
                if (touches.length > 1)
                    this._calculatePinchZoomFactor(this._calculateTouchDistance(this.eventParams.touches, touches), model.zooming.type, e.target);
                else
	                (this.doPan || this.panning) ? this._calculatePinchZoomPosition(e) : this.chartMouseMove(e);
            }
            else
                this.chartMouseMove(e);
        } else
            this.chartMouseMove(e);
    },
    _pointerPinchMove: function (e) {
        var areaBounds, touches, j = 0, event = e.originalEvent, elementOffset = $(this.element).offset(), x = event.pageX - elementOffset.left, y = event.pageY - elementOffset.top,
            model = this.model, params = this.eventParams || this._initEventParams(model, event), startTouches = params.touches, previous = this.previousPointerMove,
            length = startTouches.length, touches = params.movements;


        if (previous != null && previous.pageX == event.pagex && previous.pageY == event.pageY && previous.pointerId == e.pointerId)
            return false;
        if (!model.enable3D && model.AreaType == 'cartesianaxes' && event.pointerType != 'mouse' && event.pointerType != 4 && model.zooming.enable && model.zooming.enablePinching) {
            areaBounds = model.m_AreaBounds;
            if (x > areaBounds.X && x < areaBounds.X + areaBounds.Width && y > areaBounds.Y && y < areaBounds.Y + areaBounds.Height) {
                this._addTouchPointer(startTouches, { pageX: x, pageY: y, pointerId: event.pointerId });
                if (startTouches && startTouches.length > 1) {
                    for (; j < length; j++)
                        this._addTouchPointer(touches, startTouches[j], false);
                    this._addTouchPointer(touches, { pageX: x, pageY: y, pointerId: event.pointerId }, true);
                    if (!((touches[0].pageX == startTouches[0].pageX) && (touches[1].pageX == startTouches[1].pageX) && (touches[0].pageY == startTouches[0].pageY) && (touches[1].pageY == touches[1].pageY)))
                        this._calculatePinchZoomFactor(this._calculateTouchDistance(startTouches, touches), this.model.zooming.type, e.target);
                }
                else 
	                (this.doPan || this.panning) ? this._calculatePinchZoomPosition(e) : this.chartMouseMove(e);
            }
            else {
                startTouches.length > 0 && this._removeTouchPointer(startTouches, e);
                touches.length > 0 && this._removeTouchPointer(touches, e);
                this.chartMouseMove(e);
            }
            this.previousPointerMove = { pageX: x, pageY: y, pointerId: event.pointerId };
        }
        else
            this.chartMouseMove(e);
    },
    _pointerPinchEnd: function (e) {
        var model = this.model, event = e.originalEvent, params = this.eventParams || (this.eventParams = this._initEventParams(model, event)),
                mouseLeave = (event.type.indexOf("leave")!=-1 || event.type.indexOf("out")!= -1 || event.type.indexOf("Out")!=-1);
        if (!model.enable3D && model.AreaType == 'cartesianaxes' && event.pointerType != 'mouse' && event.pointerType != 4 && model.zooming.enable && model.zooming.enablePinching && (event.touches || event.pointerId)) {
            if (this.continuePinching != null)
                this.svgObject.removeChild(this.svgObject.firstChild);
            this.continuePinching = this.previousPanTouch = this.previousScale = null;
            if (event.touches) {
                this.eventParams = null;
                this.chartMouseUp(e);
                mouseLeave && this.chartMouseLeave(e);
            }
            else {
                this._removeTouchPointer(params.touches, { pointerId: event.pointerId });
                if (params.movements.length > 0)
                    this._removeTouchPointer(params.movements, { pointerId: event.pointerId });
                if (params.touches.length == 0) {
                    this.chartMouseUp(e);
                    mouseLeave && this.chartMouseLeave(e);
                    this.eventParams = null;
                }
            }
        }
        else {
            this.chartMouseUp(e);
            mouseLeave && this.chartMouseLeave(e); //Trigger mouse leave event
        }
    },
    _appendStyle: function (selector) {
        var css = '#' + selector + '.e-canvas * {touch-action:none; -ms-touch-action:none}',
                  head = document.head || document.body,
                  style = document.createElement('style');

        style.type = 'text/css';
        style.appendChild(document.createTextNode(css));
        head.appendChild(style);
    },   
	 bindEvents: function () {

	     var matched = jQuery.uaMatch(navigator.userAgent),
         browserInfo = ej.EjSvgRender.utils.browserInfo(),
         isPointer = browserInfo.isMSPointerEnabled,
	     isIE11Pointer = browserInfo.pointerEnabled,	    
         touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
         touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
         touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
         touchCancelEvent = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchleave mouseleave";
	     this.model.browserInfo = browserInfo;  

	     if ((window.navigator.maxTouchPoints > 1 && window.PointerEvent) || window.PointerEventsPolyfill) { //For IE11 and IE devices that supports pointer events
	         this._on(this.element, "pointerup pointerleave", this._pointerPinchEnd);	        
	         this._on(this.element, "pointerdown", this._pointerPinchStart);
	         this._on(this.element, "pointermove", this._pointerPinchMove);
	         this._appendStyle(this.element[0].id);
	     }
	     else if (window.TouchEvent)//For chrome, safari and android
	     {	         
	         this._on(this.element, "touchstart", this._pointerPinchStart);
	         this._on(this.element, "touchmove", this._pinchGestureMove);
	         this._on(this.element, "touchend", this._pointerPinchEnd);	         
	         this._on($(this.element), "mousedown", this.chartMouseDown);
	         this._on($(this.element), "mousemove", this.chartMouseMove);
	         this._on(this.element, 'mouseleave', this.chartMouseLeave);
	         this._on($(document), "mouseup", this.chartMouseUp);	         
	         this._appendStyle(this.element[0].id);			
	     }        
	     else if (window.navigator.msMaxTouchPoints && window.navigator.msPointerEnabled) { //For IE10
	         this._on(this.element, "MSPointerUp", this._pointerPinchEnd);
	         this._on(this.element, "mouseleave", this.chartMouseLeave);
	         this._on(this.element, "MSPointerDown", this._pointerPinchStart);
	         this._on(this.element, "MSPointerMove", this._pointerPinchMove);
	         this._appendStyle(this.element[0].id);
	     }
	     else {
	         this._on(this.element, touchMoveEvent, this.chartMouseMove);
	         this._on(this.element, touchCancelEvent, this.chartMouseLeave);
	         this._on($(document), touchStopEvent, this.chartMouseUp);
	         this._on($(this.element), touchStartEvent, this.chartMouseDown);
	     }

	     this._on($(this.element), "click", this.chartMouseClick);
	     this._on($(this.element), "contextmenu", this.chartRightClick);	     
	     this._on($(this.element), "dblclick", this.chartMouseDoubleClick);	     


	     if (this.model.canResize || this.model.isResponsive) {
	         this.bindResizeEvents();
	     }
	     var touchAction = this.model.zooming.enable || this.model.commonSeriesOptions.dragSettings.enable ? 'none' : 'pan-y';

	     $(this.element).css({ 'ms-touch-action': touchAction, 'touch-action': touchAction, 'content-zooming': 'none' }); //disable browser touch action   
	   // $(this.svgObject).css({ 'ms-touch-action': 'none', 'touch-action': 'none', 'content-zooming': 'none' });
	     $(this.element).css({
	     "-webkit-touch-callout": "none", /* iOS Safari */
	    "-webkit-user-select": "none",  /* Chrome/Safari/Opera */ 	     
	     "-moz-user-select": "none",     /* Firefox */
	     "-ms-user-select": "none",     /* Internet Explorer/Edge */
	     "user-select": "none"
	     });
	     var eventName = matched.browser.toLowerCase() == "mozilla" ? (isIE11Pointer ? "mousewheel" : "DOMMouseScroll") : "mousewheel";
	     this._on(this.element, eventName, this.chartMouseWheel);

	 },

    calZoomFactors: function (cumulativeScale, origin, currentZoomFactor, currentZoomPos) {
        var calcZoomFactorVal, calcZoomPosition;
        if (cumulativeScale == 1) {
            calcZoomFactorVal = 1;
            calcZoomPosition = 0;
        } else {
            calcZoomFactorVal = ej.EjSvgRender.utils._minMax(1 / cumulativeScale, 0, 1);
            calcZoomPosition = currentZoomPos + ((currentZoomFactor - calcZoomFactorVal) * origin);
        }
        return { zoomMFactor: calcZoomFactorVal, zoomMPosition: calcZoomPosition };
    },
    doMouseWheelZoom: function (cumulativeScale, origin, axis) {
        if (cumulativeScale >= 1) {
            var calZoomVal = this.calZoomFactors(cumulativeScale, origin, axis.zoomFactor, axis.zoomPosition);
            //calcZoomPos = Math.Round(calcZoomPos, 2);
            if (axis.zoomPosition != calZoomVal.zoomMPosition || axis.zoomFactor != calZoomVal.zoomMFactor) {
                this.zoomed = true;
                axis.zoomPosition = calZoomVal.zoomMPosition;
                axis.zoomFactor = (calZoomVal.zoomMPosition + calZoomVal.zoomMFactor) > 1 ? (1 - calZoomVal.zoomMPosition) : calZoomVal.zoomMFactor;
                return true;
            }
        }

        return false;

    },
    enableAnimation: function () {
        for (var m = 0; m < this.model.series.length; m++) {
            var series = this.model.series[m];
            series.enableAnimation = this.serAnimation[m];
        }
    },
    disableAnimation: function () {
        var chart = this;
        for (var m = 0; m < chart.model.series.length; m++) 
            chart.model.series[m].enableAnimation = false;
        for (var k = 0; k < chart.model.indicators.length; k++) 
            chart.model.indicators[k].enableAnimation = false;
    },
    _removeTrackBall: function () {
        var id = "#" + this.svgObject.id;
        $("#" + this.svgObject.id + "_trackSymbol").remove();
		 $(document).find("#measureTex").remove();
		 if (this.chartCross.visible) {
		     // Visibility is set to hidden
		     var element = $(this.svgObject).find(id + "_AxisCrossToolTip");
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
		     element = $(this.svgObject).find(id + "_CrosshairVertical");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0' });
		     element = $(this.svgObject).find(id + "_CrosshairHorizontal");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0' });
		     
        element = $(this.svgObject).find(id + "_TrackAxisToolTip");
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden'});
        element = $(this.svgObject).find(id + "_Tracker");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0'});
        element = $(this.svgObject).find('[id*="_TrackToolTip"]');
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        $(element).css('visibility', 'hidden');
        element = $(this.svgObject).find('[id*="_trackSymbol_"]');
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        $(document).find('[id*="_TrackToolTipTemplate_"]').attr("visibility", "hidden");
        $(document).find('[id*="_TrackToolTipTemplate_"]').css("display", "none");
        $(id + "_CrosshairVertical").css("display", "none");
        $(id + "_CrosshairHorizontal").css("display", "none");
        $("#" + this._id).find('[id*="canvas_AxisToolTipRect"]').css("visibility", "hidden");
		$("#" + this._id).find('[id*="canvas_Tracker"]').remove();
        $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
		$("#" + this._id).find('[id*="_gTooltip_"]').remove();
		$("#secondCanvas").remove();
		$("#" + this._id).find('[id*="_trackball_grouping_tooltip"]').remove();
		}
		 
    },
    enableTrackBall: function () {
        this.model.crosshair.visible = this.chartCross.visible;
        this.model.crosshair.type = this.chartCross.mArea;
    },
    disableTrackBall: function () {

        this.model.crosshair.visible = false;
        
        this._removeTrackBall();
    },
    _enableZoomingButtons: function () {
		var svgObjectId = this.svgObject.id;
	    if (this.model.AreaType != 'cartesianaxes') {
                $("#" + svgObjectId + "_ResetZoom").remove();
                $("#" + svgObjectId + "_PanBtn").remove();
                $("#" + svgObjectId+ "_ZoomBtn").remove();
                $("#" + svgObjectId + "_ZoomInBtn").remove();
                $("#" + svgObjectId + "_ZoomOutBtn").remove();
        }
        if (this.model.AreaType == 'cartesianaxes') {
            //Remove zoom buttons before adding them
            this._removeZoomkit();
            var toolbar = this.model.zooming.toolbarItems;
            var length = toolbar.length;
			var currentItem, index;
            for (var i= length -1; i>= 0; i--) {
                currentItem = toolbar[i];
                index = length - i;
                switch (currentItem) {
                    case "reset":
                        this.resetZoomButton(index);
                        break;
                    case "pan":
                        this.panButton(index);
                        break;
                    case "zoom":
                    case "zoomIn":
                    case "zoomOut":
                        this.zoomButton(index, currentItem);
                        break;
                    default:
                        length = length - 1;
                        break;
                }
            }
        }
    },
    chartMouseWheel: function (e) {
        $("#" + this.svgObject.id + "_TrackToolTip").hide(); // fixed tooltip mouse wheel issue
        if (this.model.zooming.enableMouseWheel && this.model.AreaType == "cartesianaxes" && !this.model.enable3D && !this.vmlRendering && ej.util.isNullOrUndefined(this.model.isLazyZooming)) {
            var chart = this;       
            var chartZoomed;
            this.zoomed = false;
            var canUpdate = false;
            var id = "#" + chart.svgObject.id;
            var matched = jQuery.uaMatch(navigator.userAgent);
			var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            var wheelDelta = e.originalEvent.wheelDelta;
			var direction = matched.browser.toLowerCase() == "mozilla" ? ((isIE11 ? ((wheelDelta / 120) > 0 ? 1 : -1) : -(e.originalEvent.detail) / 3 > 0 ? 1 : -1)) : ((wheelDelta / 120) > 0 ? 1 : -1);
            for (var i = 0; i < this.model._axes.length; i++) {
                var axis = this.model._axes[i];
                if ((axis.orientation.toLowerCase() == "vertical" && (chart.model.zooming.type.toLowerCase() == "y" || chart.model.zooming.type.trim().toLowerCase() == "x,y")) || (axis.orientation.toLowerCase() == "horizontal" && (chart.model.zooming.type.toLowerCase() == "x" || chart.model.zooming.type.trim().toLowerCase() == "x,y"))) {

                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    var cumulativeScale = Math.max(currentScale + (0.25 * direction), 1);
                    canUpdate = canUpdate | chart.doMouseWheelZoom(cumulativeScale, 0.5, axis);
                }
                if (axis.zoomFactor != 1 && axis.zoomPosition != 0)
                    chartZoomed = true;
            }
            this._updateScroll();
            this.disableAnimation();

            this.disableTrackBall();
            if (canUpdate) {
                this.model.legendCollapsed = false;
                this.zoomed = true;
                chart.redraw(true);
              }
            this.model._chartAreaZoom = true;
            this._enableZoomingButtons();
            this.mouseWheelCoords = { x: e.originalEvent.pageX, y: e.originalEvent.pageY };
            if (!chartZoomed) {
                $(id + '_ResetZoom').remove();
                $(id + '_PanBtn').remove();
                $(id + '_ZoomBtn').remove();
                $(id + '_ZoomInBtn').remove();
                $(id + '_ZoomOutBtn').remove();
                this.enableAnimation();
                this.panning = false;
            }
			var matched = jQuery.uaMatch(navigator.userAgent);
            if (e.preventDefault && matched.browser == 'chrome' && this.model.zooming.enable)
                e.preventDefault();
            e.returnValue = false;
        }

		$(".tooltipDiv" + this._id).remove();
    },
	
	 getAxisMultiLevelLabelsData: function (evt) {
		var mouseMoveCords = this.calMousePosition(evt),
		    grpLabelsLength = this.model.multiLevelLabelRegions.length,
			ele, x, y, height, width, i;
		this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;
		for (i= 0; i< grpLabelsLength; i++){
		     ele = this.model.multiLevelLabelRegions[i];
			 x = ele.bounds.x;
			 y = ele.bounds.y;
			 height = ele.bounds.height;
			 width = ele.bounds.width;
			 if ((this.mousemoveX >= x) && (this.mousemoveX <= x + width) && (this.mousemoveY <= y + height) && (this.mousemoveY >= y) ) {
                    data = { location: { x: this.mousemoveX, y: this.mousemoveY }, axis: this.model._axes[ele.axisIndex], multiLevelLabel: ele.multiLevelLabel };
                    return data;
             }
		}
	 },
	 
	 getAxisLabelData: function (evt) {
           
        var mouseMoveCords = this.calMousePosition(evt),
            targetid = evt.target.id,
            region,
            x,
            y,
            width,
            height,
            data,
            labelsLength,
            axisVisible,
            axesLength = this.model._axes.length;
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;
        if (this.model.AreaType == 'cartesianaxes' || this.model.AreaType == 'polaraxes') {
            if (!this.model.enable3D ) {
                for (var j = 0; j < axesLength; j++) {
                    labelsLength = this.model._axes[j].visibleLabels.length;
                    axisVisible=this.model._axes[j].visible;
                    if (axisVisible)
                    for (var k = 0; k < labelsLength; k++) {

                        region = this.model._axes[j].visibleLabels[k].region;
                        if (!ej.util.isNullOrUndefined(region)) {
                            x = region.bounds.x;
                            y = region.bounds.y;
                            width = region.bounds.width;
                            height = region.bounds.height;
                            x = (this.vmlRendering && (targetid.indexOf("_YLabel_") >= 0)) ? (this.model._axes[j].opposedPosition) ? x : x + width : x;
                            if ((this.mousemoveX >= x) && (this.mousemoveX <= x + width)) {
                                if ((this.vmlRendering) ? ((this.mousemoveY <= y + height) && (this.mousemoveY >= y)) : (this.mousemoveY >= y - height) && (this.mousemoveY <= y)) {
                                    data = { location: { x: this.mousemoveX, y: this.mousemoveY }, index: k, axis: this.model._axes[j], text: region.labelText };
                                    return data;
                                }
                            }
                        }

                    }
                }
            }
            else {
                if (targetid.indexOf("horizontal") >= 0 || targetid.indexOf("vertical") >= 0 ) {
                    for (var j = 0; j < axesLength; j++) {
                        labelsLength = this.model._axes[j].visibleLabels.length;
                        for (var k = 0; k < labelsLength; k++) {

                            if (targetid == this.svgObject.id + this.model._axes[j].orientation + k) {
                                data = { location: { x: this.mousemoveX, y: this.mousemoveY }, index: k, axis: this.model._axes[j], text: this.model._axes[j].visibleLabels[k].Text};
                                return data;

                            }
                        }
                    }
                } 
            }
    }
    },
    getLegendData: function (evt) {
        var mouseMoveCords = this.calMousePosition(evt);
        mouseMoveCords.X -= parseFloat($(this.element).css("padding-left"));
        mouseMoveCords.Y -= parseFloat($(this.element).css("padding-top"));
        var isEjScroll = this.model.legend._ejScroller;
        var scrolltop = isEjScroll ? $("#legend_" + this._id).ejScroller('instance').model.scrollTop : $("#legend_" + this._id).scrollTop();
        var scrollleft = isEjScroll ? $("#legend_" + this._id).ejScroller('instance').model.scrollLeft : $("#legend_" + this._id).scrollLeft();
        this.scrolltop = scrolltop;
        this.scrollleft = scrollleft;
        this.mousemoveX = mouseMoveCords.X+scrollleft;
        this.mousemoveY = mouseMoveCords.Y+scrolltop;
		 var targetId = evt.target.id;

        {
            var currentX = this.mousemoveX;
            var currentY = this.mousemoveY;
            var chartId = this._id;
            var region;
            if ((!ej.util.isNullOrUndefined(targetId) && this.svgRenderer._getAttrVal($(evt.target).parents(':eq(1)'), 'id') == this.svgObject.id + '_Legend') ||(this.model.enableCanvasRendering && targetId == "legend_" + chartId + "_canvas") || (this.vmlRendering)) {
                $.each(this.model.legendRegion, function (index, regionItem) {

                    if ((currentX >= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X) && (currentX <= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X + regionItem.Bounds.ItemBound._Width)) {
                        if ((currentY >= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y - (regionItem.Bounds.ItemBound.Height / 4)) && (currentY <= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y + regionItem.Bounds.ItemBound.Height)) {
                           region = regionItem;
                           $("#legend_" +chartId + "_canvas").css("cursor", "pointer");
                        }else
                            $("#legend_" + chartId + "_canvas").css("cursor", "default");
                    } 
                });
                if (!ej.util.isNullOrUndefined(region)) {
                    return { legendItem: region, series: this.model._visibleSeries[region.LegendItem.SeriesIndex] };
                }
            }

        }
        return false;
    },

    get3DSeriesPoint: function (evt) {
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var x = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.X;
        var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;
        var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
        var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

        if (currentX > x && currentX < (x + width) && currentY > y && currentY < (y + height)) {
            var nodeName = $(evt.target)[0].nodeName;
            if ((nodeName == "path" || nodeName == "shape") && $(evt.target)[0].id.indexOf("Region")>1) {
                index = $(evt.target)[0].id.match(/(\d+)/g);
                pointIndex = parseInt(index[index.length - 1]);
                seriesIndex = parseInt(index[index.length - 2]);
                var regionValue = {};
                regionValue.Region = {};
                regionValue.Region.PointIndex = pointIndex;
                regionValue.SeriesIndex = seriesIndex;
                var pointData = {};
                pointData.pointIndex = pointIndex;
                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                return region;
            }
        }
    },

    GetSeriesPoint: function (evt) {
        var mouseX;
        var mouseY;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var region;
        var chartObj = this;
		 var indicators = chartObj.model.indicators;
        var seriesCollection = chartObj.model._visibleSeries;
        for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
            seriesCollection = seriesCollection.concat(indicators[j].segment);
        }
        if (chartObj.model.enable3D) {
            return this.get3DSeriesPoint(evt);           
        }
        var x = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.X;
        var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;
        var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
        var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

        if ((this.model.AreaType == 'cartesianaxes' || this.model.AreaType == 'polaraxes')) {
          
            if (currentX > x && currentX < (x + width) && currentY > y && currentY < (y + height)) {
                $.each(this.model.chartRegions, function (index, regionItem) {
                    if (!ej.util.isNullOrUndefined(regionItem.SeriesIndex)) {
					var currentSer = seriesCollection[regionItem.SeriesIndex];
                    if (chartObj.model.AreaType == "polaraxes" && currentSer.drawType.toLowerCase() == 'column' && evt.target) {
                        if (chartObj.model.enableCanvasRendering) {
                                    if (currentSer.type.toLowerCase() == "polar") {
                                        var bounds = regionItem.Region.Bounds;
                                        var chartStartAngle = -.5 * Math.PI;
                                        var innerRadius = bounds.innerRadius;
                                        var fromCenterX;
                                        var fromCenterY;
                                        fromCenterX = (currentX) - (bounds.CenterX);
                                        fromCenterY = (currentY) - (bounds.CenterY);
                                        var series = chartObj.model._visibleSeries[regionItem.SeriesIndex];
                                        var startAngle = series.startAngle;
                                        var endAngle = series.endAngle;

                                        var arcAngle = (startAngle) ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;
                                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                                        if (clickAngle < 0 && bounds.PointIndex != 0) clickAngle = 2 * Math.PI + clickAngle;

                                        var pointStartAngle = parseFloat(bounds.StartAngle.toFixed(14));
                                        var pointEndAngle = parseFloat(bounds.EndAngle.toFixed(14));
                                        pointStartAngle -= arcAngle;
                                        pointEndAngle -= arcAngle;

                                        if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                                            index = bounds.PointIndex;
                                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                                            if (distanceFromCenter <= bounds.Radius && distanceFromCenter > innerRadius) {
                                                var pointRegion = { Region: { PointIndex: index }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                                region = pointRegion;
                                            }
                                        }
                                    } else if (currentSer.type.toLowerCase() == "radar") {
                                        var bounds = regionItem.Region.Bounds;
                                        if (bounds.Line1) {
                                            line1 = bounds.Line1;
                                            line2 = bounds.Line2;
                                            line3 = bounds.Line3;
                                            var polygon = [
                                                            { x: line1.x, y: line1.y },
                                                            { x: line2.x, y: line2.y },
                                                            { x: line3.x, y: line3.y },
                                            ];
                                            var currentPoint = { x: currentX, y: currentY };
                                            var point = false;
                                            for (var p = false, i = -1, l = polygon.length, j = l - 1; ++i < l; j = i)
                                                ((polygon[i].y <= currentPoint.y && currentPoint.y < polygon[j].y) || (polygon[j].y <= currentPoint.y && currentPoint.y < polygon[i].y))
                                                && (currentPoint.x < (polygon[j].x - polygon[i].x) * (currentPoint.y - polygon[i].y) / (polygon[j].y - polygon[i].y) + polygon[i].x)
                                                && (p = !p);
                                            point = p;
                                            if (point) {
                                                index = bounds.PointIndex;
                                                var pointRegion = { Region: { PointIndex: index }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                                region = pointRegion;
                                            }
                                        }
                                    }
                                }
                                else {
                                    var nodeName = $(evt.target)[0].nodeName;
                                    if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + regionItem.SeriesIndex) {
                                        index = $(evt.target)[0].id.match(/(\d+)/g);
                                        pointIndex = parseInt(index[index.length - 1]);
                                        regionItem = { Region: { PointIndex: pointIndex }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                        region = regionItem;
                                        return false;
                                    }
                                }
                    }
                    else {
                        if ((((currentX >= regionItem.Region.Bounds.X) && (currentX <= (regionItem.Region.Bounds.X + (regionItem.Region.Bounds.Width))))) && ((currentY > regionItem.Region.Bounds.Y) && (currentY < (regionItem.Region.Bounds.Y) + (regionItem.Region.Bounds.Height)))) {
                            region = regionItem;
                        }
                    }
                    }
					else {
                        regionValue = chartObj.model.chartRegions;
                        var pointData = {};
						 var areaValue;
					if (!regionItem.isStripLine) {
                        var regionLength = regionItem.length;
						 var requireInvertedAxes = chartObj.model.requireInvertedAxes;
                        var axisName = (!requireInvertedAxes) ? chartObj.model.series[regionItem[regionLength - 1].SeriesIndex]._yAxisName : chartObj.model.series[regionItem[regionLength - 1].SeriesIndex]._xAxisName;
                        for (var k = 0; k < chartObj.model._axes.length; k++)
                        {
                            if (axisName == chartObj.model._axes[k].name)
                            {
                               areaValue = chartObj.model._axes[k];
                               break;
                            }
                        }
                        for (var i = 0; i < regionValue.length; i++) {
							for (var j = 0; j < regionValue[i].length; j++) {
                                var polygon = [];
                                line1 = regionValue[i][j].region;
                                if (line1[0]) polygon.push({ x: line1[0].X + areaValue.x, y: line1[0].Y + areaValue.y });
                                if (line1[1]) polygon.push({ x: line1[1].X + areaValue.x, y: line1[1].Y + areaValue.y });
                                if (line1[2]) polygon.push({ x: line1[2].X + areaValue.x, y: line1[2].Y + areaValue.y });
                                if (line1[3]) polygon.push({ x: line1[3].X + areaValue.x, y: line1[3].Y + areaValue.y });

                                var currentPoint = { x: currentX, y: currentY };
                                var point = chartObj.isPointInPolygon(polygon, currentPoint);
                                if (point)
                                    region = regionValue[i][j];
							}
                                                                         
                        }
                    }
                    }
                });
            }
            if (region) {
                return { region: region, location: { x: currentX, y: currentY} };
            }
        }

        else {
            var chartObj = this;
            var distanceFromCenter;
            var regionValue;
            var chartRegions = chartObj.model.chartRegions;
            if (chartRegions.length > 0 && (chartRegions[0].Series.type.toLowerCase() == "pyramid"|| chartRegions[0].Series.type.toLowerCase() == "funnel")) {
                var nodeName = $(evt.target)[0].nodeName;
                if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + chartRegions[0].SeriesIndex) {
                    regionValue = chartObj.model.chartRegions[0];
                    index = $(evt.target)[0].id.match(/(\d+)/g);
                    index = parseInt(index[index.length - 1]);
                    regionValue.Region.PointIndex = index;
                    var pointData = {};
                    pointData.pointIndex = index;
                    region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                    return region;
                }
                else {                    
                    regionValue = chartRegions[0];
                    var pyrX = this.pyrX;
                    var pyrY = this.pyrY;
                    var pointData = {};
                    var polygon;
                        for (var j = 0; j < regionValue.Region.length; j++) {
                            line1 = regionValue.Region[j].Line1;
                            line2 = regionValue.Region[j].Line2;
                            line3 = regionValue.Region[j].Line3;
                            line4 = regionValue.Region[j].Line4;
                            if (chartRegions[0].Series.type.toLowerCase() == "pyramid")
                                polygon = [
                                            { x: line1.x + pyrX, y: line1.y + pyrY },
                                            { x: line2.x + pyrX, y: line2.y + pyrY },
                                            { x: line3.x + pyrX, y: line3.y + pyrY },
                                            { x: line4.x + pyrX, y: line4.y + pyrY }
                                ];
                            else {
                                line5 = regionValue.Region[j].Line5;
                                line6 = regionValue.Region[j].Line6;
                                polygon = [
                                            { x: line1.x + pyrX, y: line1.y + pyrY },
                                            { x: line2.x + pyrX, y: line2.y + pyrY },
                                            { x: line3.x + pyrX, y: line3.y + pyrY },
                                            { x: line4.x + pyrX, y: line4.y + pyrY },
                                            { x: line5.x + pyrX, y: line5.y + pyrY },
                                            { x: line6.x + pyrX, y: line6.y + pyrY }
                                ];
                            }

                            var currentPoint = { x: currentX, y: currentY };
                            var point = this.isPointInPolygon(polygon, currentPoint);
                            if (point) {
                                pointData.pointIndex = regionValue.Region[j].PointIndex;
                                regionValue.Region.PointIndex = pointData.pointIndex;
                                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY } };                                                                
                                return region;
                            }

                        }
                    }                
                }
            
            else {
                // ReSharper disable DuplicatingLocalDeclaration
                $.each(chartObj.model.chartRegions, function (regionIndex, regionValue) {
                    // ReSharper restore DuplicatingLocalDeclaration
                    var chartStartAngle = -.5 * Math.PI;
                    var seriesData = regionValue.SeriesData,
                    regionData = regionValue.Region,
                    innerRadius = (chartObj.model._visibleSeries[regionValue.SeriesIndex].type.toLowerCase() == 'doughnut') ? seriesData.DRadius: 0;
                    var fromCenterX;
                    var fromCenterY;
                    fromCenterX = (currentX) - (seriesData.CenterX);
                    fromCenterY = (currentY) - (seriesData.CenterY);
                    var series = chartObj.model._visibleSeries[regionValue.SeriesIndex];
                    var startAngle = series.startAngle;
                    var endAngle = series.endAngle;
                    startAngle = startAngle < 0 ? startAngle + 360 : startAngle;
                    endAngle = endAngle < 0 ? endAngle + 360 : endAngle;
                    var totalDegree = (series.endAngle - series.startAngle);
                    //Anticlockwise pie and doughnut
                    if (totalDegree < 0) {
                        endAngle = endAngle / 360;
                        var arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);;
                        if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;

                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var pointStartAngle = parseFloat(regionData[i].StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(regionData[i].EndAngle.toFixed(14));
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;
                            pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                            pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                            if (series.startAngle >= 0 && series.endAngle <= 0) {
                                pointStartAngle = pointStartAngle <= 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                                pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            }
                            if (clickAngle <= pointStartAngle && clickAngle >= pointEndAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                    }  //Clockwise pie,pieofpie and doughnut
                    else {
                        var arcAngle = (startAngle) ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle)%(2*Math.PI);
                        if (clickAngle < 0) clickAngle = chartObj.model._isPieOfPie ? clickAngle:2 * Math.PI + clickAngle;
                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var pointStartAngle = parseFloat(regionData[i].StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(regionData[i].EndAngle.toFixed(14));
                            pointStartAngle = pointStartAngle < 0 ? (chartObj.model._isPieOfPie? pointStartAngle:2 * Math.PI + pointStartAngle):pointStartAngle;
                            pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;
                            pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                            pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                            if (series.startAngle < 0 && (series.endAngle > -1||series.endAngle==null)) {
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle <= 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            }
                            if (clickAngle + 2 * Math.PI < pointEndAngle && series.type == "pieofpie")
                                clickAngle = clickAngle + 2 * Math.PI;
                            if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                    }
                    if (pointData.length > 0) {
                        var isExploded = (series.explodeAll || (series.explodeIndex == pointData[0].PointIndex));
                        if (isExploded) {

                            var fromCenterX1 = currentX - pointData[0].StartX;
                            var fromCenterY1 = currentY - pointData[0].StartY;

                            if (pointData.length > 0) {

                                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX1), 2) + Math.pow(Math.abs(fromCenterY1), 2));
                                if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                    regionData.PointIndex = pointData[0].PointIndex;
                                    regionData.SeriesIndex = pointData[0].SeriesIndex;
                                    region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                                }
                            }
                        }
                        else {
                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                            if (distanceFromCenter <= seriesData.Radius&& distanceFromCenter > innerRadius) {
                                regionData.PointIndex = pointData[0].PointIndex;
                                regionData.SeriesIndex = pointData[0].SeriesIndex;
                                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                            }
                        }
                    }

                });

                return region;
            }
        }
    },
    isPointInPolygon: function (polygon, point) {
        for (var p = false, i = -1, l = polygon.length, j = l - 1; ++i < l; j = i)
            ((polygon[i].y <= point.y && point.y < polygon[j].y) || (polygon[j].y <= point.y && point.y < polygon[i].y))
            && (point.x < (polygon[j].x - polygon[i].x) * (point.y - polygon[i].y) / (polygon[j].y - polygon[i].y) + polygon[i].x)
            && (p = !p);
        return p;
    },
    mousePosition: function (evt) {
         if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX>0)
            return { x: evt.pageX, y: evt.pageY };
         else if (evt.originalEvent && !ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX>0)
            return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
         else if (evt.originalEvent && evt.originalEvent.changedTouches != undefined)
         {
             if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                 return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
         }
         else
             return { x: 0, y: 0 };
         
    },


    calMousePosition: function (e) {
        var matched = jQuery.uaMatch(navigator.userAgent);
        var mouseposition = this.mousePosition(e);
        e.pageX = mouseposition.x;
        e.pageY = mouseposition.y;
        var mouseX, mouseY;
        var browser = matched.browser.toLowerCase();
        mouseX = (e.pageX) - $(this.svgObject).offset().left;
        mouseY = (e.pageY) - $(this.svgObject).offset().top;
       

        return { X: mouseX, Y: mouseY };

    },
    
    calTouchPosition: function (e) {
        var matched = jQuery.uaMatch(navigator.userAgent);
        var mouseX, mouseY;

        if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
            var touch = e.originalEvent.touches[0];
            mouseX = (touch.pageX) - $(this.svgObject).parent().offset().left;
            mouseY = (touch.pageY) - $(this.svgObject).parent().offset().top;
            this.leftPadding = $(this.svgObject).offset().left;
            this.grabPadding = 0;
        } else if (matched.browser.toLowerCase() == "msie") {

            mouseX = (e.originalEvent.pageX) - $(this.svgObject).offset().left;
            mouseY = (e.originalEvent.pageY) - $(this.svgObject).offset().top;
            this.leftPadding = $(this.svgObject).offset().left;
        }
        else {
            var touch = e.originalEvent.touches[0];
            mouseX = (touch.pageX) - $(this.svgObject).offset().left;
            mouseY = (touch.pageY) - $(this.svgObject).offset().top;
            this.leftPadding = $(this.svgObject).offset().left;
        }

        return { X: mouseX, Y: mouseY };

    },

    cancelEvent: function (e) {
        e.stopPropagation();
        var matched = jQuery.uaMatch(navigator.userAgent);
        if(matched.browser == 'chrome' && (this.model.zooming.enable||this.model.crosshair.visible) ){
            e.preventDefault();
        }
        if (this.model.enable3D) {
            e.preventDefault();
            e.stopPropagation();
        }
        e.returnValue = false;
        e.cancelBubble = true;
        return false;
    },
    resetZoom: function () {
        var chartobj = this;
        var id = "#" + this.svgObject.id;
        $(".tooltipDiv" + this._id).remove(); //To remove tooltip template
        $(".zoom"+ this._id).remove();
        if (chartobj.zoomed) {
            $.each(this.model._axes, function (index, axis) {
				axis.zoomed = false;
                axis.zoomFactor = 1;
                axis.zoomPosition = 0;
            });
			//Reset primary axis zoomFactor and zoomPosition
			chartobj.model.primaryXAxis.zoomFactor = 1;
            chartobj.model.primaryYAxis.zoomFactor = 1;
            chartobj.model.primaryXAxis.zoomPosition = 0;
            chartobj.model.primaryYAxis.zoomPosition = 0;
			$("#"+chartobj._id + "_canvas").css({ "cursor": "default" });
            $(id + "_ResetZoom").remove();
            $(id + "_PanBtn").remove();
            $(id + "_ZoomBtn").remove();
            $(id + "_ZoomInBtn").remove();
            $(id + "_ZoomOutBtn").remove();            
            chartobj.zoomed = false;
            chartobj.zooming = false;
            chartobj.panning = false;
            chartobj.model._chartAreaZoom = false;
            chartobj.resetZooming = true;
            chartobj.enableTrackBall();
            chartobj.redraw();
            chartobj.enableAnimation();          
            chartobj.svgRenderer._setAttr($(chartobj.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection"), { "cursor": "default" });
        }
    },
    startZoomInOut: function (currentEle) {
	    var currentItem = currentEle.target.id.indexOf("ZoomIn") > -1 ? "zoomIn" : "zoomOut";
        this.drag = false;
        this.zoomed = true;
		var type = this.model.zooming.type.toLowerCase();
		var zoomValue = currentItem == "zoomIn" ? 0.2 : -0.2;
		var zoomedOut = true;
		var axis, previousZoomFactor, previousZoomPosition;
        for (var k = 0; k < this.model._axes.length; k++) {
            axis = this.model._axes[k];
            previousZoomFactor = axis.zoomFactor;
            previousZoomPosition = axis.zoomPosition;
            if (axis.orientation.toLowerCase() == "horizontal") {
                axis.zoomFactor = type != "y" ? previousZoomFactor - zoomValue : axis.zoomFactor;
                axis.zoomPosition = type != "y" ? previousZoomPosition + zoomValue : axis.zoomPosition;
            } else {
                axis.zoomFactor = type != "x" ? previousZoomFactor - zoomValue: axis.zoomFactor;
                axis.zoomPosition = type != "x" ? previousZoomPosition + zoomValue : axis.zoomPosition;
            }
			if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                axis.zoomFactor = previousZoomFactor;
                axis.zoomPosition = previousZoomPosition;
             }
			 if (parseFloat(axis.zoomFactor.toFixed(3)) >= 1) {
                axis.zoomFactor = 1;
                axis.zoomPosition = 0;
             }
			 zoomedOut = zoomedOut && (axis.zoomFactor == 1);
        }
        this._updateScroll();
        this.redraw(true);
        if (zoomedOut) {
            this._removeZoomkit();
            this.resetZooming = true;
            this.zoomed = this.panning = false;
            this.model._chartAreaZoom = false
        }
        else
            this._enableZoomingButtons();
    },
    startZoom: function () {
        var chartObj = this;
        var id = "#" + this.svgObject.id;
        $(".tooltipDiv" + this._id).remove(); //To remove tooltip template
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'true' });
        chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#14B9FF" });
        chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": '#FFFFFF' });
        chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#B7B7B7' });
        chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": iconColor });
        $(id + "_ZoomBtn").css({ "border-color": "#14B9FF", "background-color": "#14B9FF" });
        $(id + "_PanBtn").css({ "border-color": "#B7B7B7", "background-color": "#B7B7B7" });
		$("#"+chartObj._id + "_canvas").css({ "cursor": "default" });
        this._cursorToDefault();
        chartObj.panning = false;
        chartObj.enableTrackBall();
    },
    startPan: function () {
        var chartObj = this;
        var id = "#" + this.svgObject.id;
        $(".tooltipDiv" + this._id).remove(); //To remove tooltip template
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'false' });
        chartObj.svgRenderer._setAttr($(id + '_PanBtn'), { 'selected': 'true' });
        chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#B7B7B7" });
        chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": iconColor });
        chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#14B9FF' });
        chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": '#FFFFFF' });
        $(id + "_ZoomBtn").css({ "border-color": "#B7B7B7", "background-color": "#B7B7B7" });
        $(id + "_PanBtn").css({ "border-color": "#14B9FF", "background-color": "#14B9FF" });
		$("#"+chartObj._id + "_canvas").css({ "cursor": "pointer" });
        this._cursorToPointer();
        chartObj.panning = true;
        chartObj.disableTrackBall();

    },
    zoomButton: function (index, currentItem) {
        if (currentItem == "zoom")
            var currentItemId = "_Zoom";
        else if (currentItem == "zoomIn")
            currentItemId = "_ZoomIn";
        else
            currentItemId = "_ZoomOut";

        var padding = index * 5 + 5, chartOffset = { left: 0, top: 0 };
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (index * 32) - padding); //32 path bouding box for pan,reset, zoom button 
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";

        var pathOptions = {
            'id': this.svgObject.id + currentItemId + 'Path',
            'd': "M32,27c0,2.75-2.25,5-5,5H5c-2.75,0-5-2.25-5-5V5c0-2.75,2.25-5,5-5h22c2.75,0,5,2.25,5,5V27z",
            'fill': this.panning || currentItem == "zoomIn" ||  currentItem == "zoomOut"? '#b7b7b7' : '#14B9FF',
            'zoomId': this._id + currentItemId + "Btn",
            'width': 26,
            'height': 26,
            'left': transX + chartOffset.left,
            'top': transY + chartOffset.top,
			 'iconColor': iconColor
        };

            ej.EjCanvasRender.prototype.zoomButton(pathOptions, currentItem, this);
            if (currentItem == "zoom")
                this._on($("#" + this.svgObject.id + currentItemId + "Btn"), "touchstart click", this.startZoom);
            else
                this._on($("#" + this.svgObject.id + currentItemId + "Btn"), "touchstart click", this.startZoomInOut);
    },
    panButton: function (index) {
        var padding = index * 5 + 5, chartOffset = { left: 0, top: 0 };
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (index * 32) - padding); //32 path bounding box for pan and resetzoom button
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";

        var pathOptions = {
            'id': this.svgObject.id + '_PanPath',
            'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
            'fill': !this.panning ? '#b7b7b7' : '#14B9FF',
            'panId': this._id + '_PanBtn',
            'width': 26,
            'height': 26,
            'left': transX + chartOffset.left,
            'top': transY + chartOffset.top,
			 'iconColor': iconColor
        };
        ej.EjCanvasRender.prototype.panButton(pathOptions, this);
        this._on($("#" + this.svgObject.id + '_PanBtn'), "touchstart click", this.startPan);
    },
    resetZoomButton: function (index) {
        var padding = index * 5 + 5, chartOffset = { left: 0, top: 0 };
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (index*32 + padding)); //32 path bouding box
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var options = {
            'id': this.svgObject.id + '_ResetZoomPath',
            'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
            'fill': '#b7b7b7',
            'resetZoomId': this._id + '_ResetZoom',
            'width': 26,
            'height': 26,
            'left': transX + chartOffset.left,
            'top': transY + chartOffset.top,
			 'iconColor': iconColor
        };
		 ej.EjCanvasRender.prototype.resetZoom(options, this);
        var events = (window.TouchEvent ? "touchstart" : (window.PointerEvent ? "pointerdown" : (window.MSPointerEvent ? "MSPointerDown" : "click"))) + " click";
		 this._on($("#" + this.svgObject.id + '_ResetZoom'), events, this.resetZoom);
    },

    doZoom: function (zoomRect, zoomRectWidth, zoomRectHeight) {
        var chart = this;
        this.model._chartAreaZoom = true;
        this.scrollDraw = true;
        $(".tooltipDiv" + this._id).remove(); //To remove tooltip template
        if (chart.model.enableCanvasRendering) {
            var zoomArea = $("#" + this.svgObject.id + "_ZoomArea")[0].getClientRects()[0];
            var chartArea = $("#" + this.svgObject.id)[0].getClientRects()[0];
            var zoomX = zoomArea.left - chartArea.left - $(document).scrollLeft(); 
            var zoomY = zoomArea.top - chartArea.top - $(document).scrollTop(); 
        } else {
            var zoomX = parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "x"));
            var zoomY = parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "y"));
        }
        $("#" + this.svgObject.id + "_ZoomArea").remove();
        this.drag = false;
        this.zoomed = true;
        for (var k = 0; k < this.model._axes.length; k++) {
            var axis = this.model._axes[k];
            var previousZoomFactor = axis.zoomFactor;
            var previousZoomPosition = axis.zoomPosition;
            if (axis.orientation.toLowerCase() == "horizontal") {
                axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "y"
                    ? previousZoomFactor * (zoomRectWidth / (chart.model.m_AreaBounds.Width)) : axis.zoomFactor;
                axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "y"
                    ? previousZoomPosition + Math.abs((zoomX - chart.model.m_AreaBounds.X) / (chart.model.m_AreaBounds.Width)) * previousZoomFactor : axis.zoomPosition;
                if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                    axis.zoomFactor = previousZoomFactor;
                    axis.zoomPosition = previousZoomPosition;

                }
            } else {
                axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "x"
                    ? previousZoomFactor * zoomRectHeight / chart.model.m_AreaBounds.Height : axis.zoomFactor;
                axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "x"
                    ? previousZoomPosition + (1 - Math.abs((zoomRectHeight + (zoomY - chart.model.m_AreaBounds.Y)) / (chart.model.m_AreaBounds.Height))) * previousZoomFactor : axis.zoomPosition;
                if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                    axis.zoomFactor = previousZoomFactor;
                    axis.zoomPosition = previousZoomPosition;

                }
            }
        }

        //Getting zoomed region data when zoomedData event fired                    
        var series = this.model._visibleSeries;
        var seriesLength = series.length;
        var zoomedData = [];
        var selectedData = [];
        var points = [];
        var type;
        var axisValueType;
        var xVisibleRange;
        var yVisibleRange;
        var pointsLength;
        for (var j = 0; j < seriesLength; j++) {
            if (series[j].visibility == "visible") {
                points = series[j].points;
                pointsLength = points.length;
                type = series[j].type.toLowerCase();
                axisValueType = new ej.axisTypes[series[j].xAxis._valueType.toLowerCase()]();
                axisValueType._calculateVisibleRange(series[j].xAxis, chart);
                axisValueType = new ej.axisTypes[series[j].yAxis._valueType.toLowerCase()]();
                axisValueType._calculateVisibleRange(series[j].yAxis, chart);
                xVisibleRange = series[j].xAxis.visibleRange;
                yVisibleRange = series[j].yAxis.visibleRange;
                for (var k = 0; k < pointsLength; k++) {
                    if ((xVisibleRange.min <= points[k].xValue && xVisibleRange.max >= points[k].xValue) && (yVisibleRange.min <= points[k].y && yVisibleRange.max >= points[k].y)) {
                        if (series[j]._hiloTypes) {
                            if (type == "rangearea" || type == "rangecolumn" || type == "waterfall" || type == "hilo")
                                zoomedData.push({ XValue: points[k].x, High: points[k].high, Low: points[k].low });
                            else
                                zoomedData.push({ XValue: points[k].x, High: points[k].high, Low: points[k].low, Open: points[k].open, Close: points[k].close });
                        }
                        else
                            zoomedData.push({ XValue: points[k].x, YValue: points[k].y });
                    }
                }
            }
            if (zoomedData.length != 0)
                selectedData.push({ selectedData: zoomedData });
            zoomedData = [];
        }

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = {};
        commonEventArgs.data.series = selectedData;
        this._trigger("zoomed", commonEventArgs);

        this.disableAnimation();
        this.model.legendCollapsed = false;
        this.redraw(true);

        this._enableZoomingButtons();
    },
    _cursorToDefault: function () {
        var id = "#" + this.svgObject.id;
        this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "default" });
    },
    _cursorToPointer: function () {
        var id = "#" + this.svgObject.id;
        this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "pointer" });
    },
    //to remove the select region on redraw
    removeMultiRect: function () {
        $(this.parentgEle).empty();
        this.model.selectedDataCollection = [];
    },
    multiSelectDataCalculation: function () {
        var chart = this, selectedData = [], selectedRectIndex = this.selectedRectIndex,
         rectX = this.oldRectX[selectedRectIndex],
         rectY = this.oldRectY[selectedRectIndex],
         rectWidth = this.oldRectWidth[selectedRectIndex],
         rectHeight = this.oldRectHeight[selectedRectIndex],
         selectFactor = 1,
         selectPosition = 0,
         seriesCollection = this.model.series,
         seriesCollectionLength = seriesCollection.length,
         axes = this.multiAxis,
         axesLength = axes.length,
         multiSelectType = this.multiSelectType,
         axis, previousSelectFactor, previousSelectPosition,
         xMin, xMax, yMin, yMax,
         visiblePoints, type, baseRange,
         start, end, isEmpty, xOriginalVisibleRange, yOriginalVisibleRange,
         areaType = this.model.AreaType.toLowerCase(),
         xValue, y, isInversed, stackYValue,
         isTransposed, selectionSettings, series;
        for (var i = 0; i < seriesCollectionLength; i++) {
            series = seriesCollection[i];
            xOriginalVisibleRange = series.xAxis.visibleRange;
            yOriginalVisibleRange = series.yAxis.visibleRange;
            var xAxisName = axes.some((function (val) {
                return val.name.toLowerCase() == series._xAxisName.toLowerCase();
            }));
            var yAxisName = axes.some((function (val) {
                return val.name.toLowerCase() == series._yAxisName.toLowerCase();
            }));

            selectionSettings = series.selectionSettings;
            if ((xAxisName && yAxisName) && series.visibility.toLowerCase() == 'visible') {
                for (var k = 0; k < axesLength; k++) {
                    axis = axes[k];
                    previousSelectFactor = selectFactor;
                    previousSelectPosition = selectPosition;
                    if (axis.orientation.toLowerCase() == "horizontal") {
                        this.selectFactor = selectionSettings.rangeType.toLowerCase() != "y"
                            ? previousSelectFactor * (rectWidth / (this.multiAxis[k].width)) : selectFactor;
                        this.selectPosition = selectionSettings.rangeType.toLowerCase() != "y"
                            ? previousSelectPosition + Math.abs((rectX - this.multiAxis[k].x) / (this.multiAxis[k].width)) * previousSelectFactor : selectPosition;
                        if (parseFloat(selectFactor.toFixed(3)) <= 0.001) {
                            this.selectFactor = previousSelectFactor;
                            this.selectPosition = previousSelectPosition;
                        }
                    }
                    else {
                        this.selectFactor = selectionSettings.rangeType.toLowerCase() != "x"
                            ? previousSelectFactor * rectHeight / this.multiAxis[k].height : selectFactor;
                        this.selectPosition = selectionSettings.rangeType.toLowerCase() != "x"
                            ? previousSelectPosition + (1 - Math.abs((rectHeight + (rectY - this.multiAxis[k].y)) / (this.multiAxis[k].height))) * previousSelectFactor : selectPosition;
                        if (parseFloat(selectFactor.toFixed(3)) <= 0.001) {
                            this.selectFactor = previousSelectFactor;
                            this.selectPosition = previousSelectPosition;
                        }
                    }
                    axis.visibleRange = $.extend(true, {}, axis.actualRange);
                    if (this.selectFactor < 1 || this.selectPosition > 0) {
                        if (axis.valueType == "logarithmic") baseRange = axis.visibleRange;
                        else baseRange = axis.actualRange;
                        this.selectFactor = this.selectFactor > 1 ? 1 : (this.selectFactor < 0 ? 0 : this.selectFactor);
                        this.selectPosition = this.selectPosition < 0 ? 0 : (this.selectPosition > 1 ? 1 : this.selectPosition);
                        if (axis.isInversed) {
                            start = axis.actualRange.max - this.selectPosition * axis.actualRange.delta;
                            end = start - this.selectFactor * axis.actualRange.delta;
                            isInversed = true;
                        }
                        else {
                            start = Math.abs(axis.actualRange.min + this.selectPosition * axis.actualRange.delta);
                            end = start + this.selectFactor * axis.actualRange.delta;
                        }
                        if (start < baseRange.min) {
                            end = end + (baseRange.min - start);
                            start = baseRange.min;
                        }
                        if (end > baseRange.max) {
                            start = start - (end - baseRange.max);
                            end = baseRange.max;
                        }
                        if (axis.valueType == "logarithmic") {
                            start = Math.pow(10, start);
                            end = Math.pow(10, end);
                        }
                        axis.visibleRange.min = Math.min(start, end);
                        axis.visibleRange.max = Math.max(start, end);
                    }
                }
                visiblePoints = series._visiblePoints;
                type = series.type.toLowerCase(), isTransposed = series.isTransposed;
                xMin = series.xAxis.visibleRange.min; xMax = series.xAxis.visibleRange.max;
                yMin = series.yAxis.visibleRange.min; yMax = series.yAxis.visibleRange.max;
                if (ej.util.isNullOrUndefined(selectedData)) selectedData = [];
                switch (multiSelectType) {
                    case 'x':
                        for (var k = 0; k < visiblePoints.length; k++) {
                            visiblePoints[k].seriesIndex = i;
                            isEmpty = visiblePoints[k].isEmpty;
                            xValue = visiblePoints[k].xValue;
                            y = visiblePoints[k].y;
                            if (series.xAxis.isInversed || series.yAxis.isInversed) {
                                if (type.indexOf('bar') >= 0) {
                                    if (type.indexOf('stacking') >= 0) {
                                        stackYValue = series.stackedValue.EndValues[k];
                                        if (!isEmpty && yMax >= stackYValue && yMin <= stackYValue)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                    else {
                                        if (type == 'waterfall' && !isEmpty && yMax >= visiblePoints[k].YValues[0] && yMin <= visiblePoints[k].YValues[0])
                                            selectedData.push(visiblePoints[k]);
                                        else if (!isEmpty && yMax >= y && yMin <= y)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                }
                                else {
                                    if (!isEmpty && xMax >= xValue && xMin <= xValue)
                                        selectedData.push(visiblePoints[k]);
                                }
                            }
                            else {
                                if (isTransposed && (type.indexOf('bar') >= 0)) {
                                    if (!isEmpty && xMin <= xValue && xMax >= xValue)
                                        selectedData.push(visiblePoints[k]);
                                }
                                else if ((type.indexOf('bar') >= 0) || ((series.isTransposed) && (!(type.indexOf('bar') >= 0)))) {
                                    if (type.indexOf('stacking') >= 0) {
                                        stackYValue = series.stackedValue.EndValues[k];
                                        if (!isEmpty && yMin <= stackYValue && yMax >= stackYValue)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                    else {
                                        if (type == 'waterfall' && !isEmpty && yMin <= visiblePoints[k].YValues[0] && yMax >= visiblePoints[k].YValues[0])
                                            selectedData.push(visiblePoints[k]);
                                        else if (!isEmpty && yMin <= y && yMax >= y)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                }
                                else {
                                    if (!isEmpty && xMin <= xValue && xMax >= xValue)
                                        selectedData.push(visiblePoints[k]);
                                }
                            }
                        }
                        break;
                    case 'y':
                        for (var k = 0; k < visiblePoints.length; k++) {
                            visiblePoints[k].seriesIndex = i;
                            isEmpty = visiblePoints[k].isEmpty;
                            xValue = visiblePoints[k].xValue;
                            y = visiblePoints[k].y;
                            if (series.xAxis.isInversed || series.yAxis.isInversed) {
                                if (type.indexOf('bar') >= 0) {
                                    if (!isEmpty && xMax >= xValue && xMin <= xValue)
                                        selectedData.push(visiblePoints[k]);
                                }
                                else {
                                    if (type.indexOf('stacking') >= 0) {
                                        stackYValue = series.stackedValue.EndValues[k];
                                        if (!isEmpty && yMin <= stackYValue && yMax >= stackYValue)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                    else {
                                        if (type == 'waterfall' && !isEmpty && yMax >= visiblePoints[k].YValues[0] && yMin <= visiblePoints[k].YValues[0])
                                            selectedData.push(visiblePoints[k]);
                                        else if (!isEmpty && yMax >= y && yMin <= y)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                }
                            }
                            else {
                                if (isTransposed && (type.indexOf('bar') >= 0)) {
                                    if (type.indexOf('stacking') >= 0) {
                                        stackYValue = series.stackedValue.EndValues[k];
                                        if (!isEmpty && yMin <= stackYValue && yMax >= stackYValue)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                    else {
                                        if (type == 'waterfall' && !isEmpty && yMin <= visiblePoints[k].YValues[0] && yMax >= visiblePoints[k].YValues[0])
                                            selectedData.push(visiblePoints[k]);
                                        else if (!isEmpty && yMin <= y && yMax >= y)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                }
                                else if ((type.indexOf('bar') >= 0) || ((series.isTransposed) && (!(type.indexOf('bar') >= 0)))) {
                                    if (!isEmpty && xMin <= xValue && xMax >= xValue)
                                        selectedData.push(visiblePoints[k]);
                                }
                                else {
                                    if (type.indexOf('stacking') >= 0) {
                                        stackYValue = series.stackedValue.EndValues[k];
                                        if (!isEmpty && yMin <= stackYValue && yMax >= stackYValue)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                    else {
                                        if (type == 'waterfall' && !isEmpty && yMin <= visiblePoints[k].YValues[0] && yMax >= visiblePoints[k].YValues[0])
                                            selectedData.push(visiblePoints[k]);
                                        else if (!isEmpty && yMin <= y && yMax >= y)
                                            selectedData.push(visiblePoints[k]);
                                    }
                                }
                            }
                        }
                        break;
                    case 'xy':
                        for (var k = 0; k < visiblePoints.length; k++) {
                            visiblePoints[k].seriesIndex = i;
                            isEmpty = visiblePoints[k].isEmpty;
                            xValue = visiblePoints[k].xValue;
                            y = visiblePoints[k].y;
                            if (type.indexOf('stacking') >= 0) {
                                stackYValue = series.stackedValue.EndValues[k];
                                if (!isEmpty && xMax >= xValue && xMin <= xValue && yMin <= stackYValue && yMax >= stackYValue)
                                    selectedData.push(visiblePoints[k]);
                            }
                            else {
                                if (type == 'waterfall' && !isEmpty && xMax >= xValue && xMin <= xValue && yMin <= visiblePoints[k].YValues[0] && yMax >= visiblePoints[k].YValues[0])
                                    selectedData.push(visiblePoints[k]);
                                else if (!isEmpty && xMax >= xValue && xMin <= xValue && yMin <= y && yMax >= y)
                                    selectedData.push(visiblePoints[k]);
                            }
                        }
                        break;
                }
            }
            series.xAxis.visibleRange = xOriginalVisibleRange;
            series.yAxis.visibleRange = yOriginalVisibleRange;
        }
        var currentIndex = this.currentIndex;
        if (ej.util.isNullOrUndefined(this.model.selectedDataCollection)) this.model.selectedDataCollection = [];
        if (this.removeRect) this.model.selectedDataCollection.splice(currentIndex, 1);
        else if (this.rectPan || this.resize) {
            this.model.selectedDataCollection[currentIndex] = 0;
            this.model.selectedDataCollection[currentIndex] = selectedData;
        }
        else this.model.selectedDataCollection[this.model.selectedDataCollection.length] = selectedData;
        //trigger rangeSelected event to get the selected region's data
        var commonloadEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonloadEventArgs.data = {};
        commonloadEventArgs.data.selectedData = selectedData;
        commonloadEventArgs.data.selectedDataCollection = this.model.selectedDataCollection;
        this._trigger("rangeSelected", commonloadEventArgs);
    },
    multiSelectMouseUp: function (evt) {
        if (this._isMultiSelect && (this.multiSelectMode == 'range')) {
            var selectRectWidth, selectRectHeight, selectRectX, selectRectY, cx, cy, index,
            containerSvg = this.svgObject.id, selectedRectIndex = this.selectedRectIndex,
            selectRectId = "#" + containerSvg + "_selectRect",
            selectRectWidth = parseInt($(selectRectId + selectedRectIndex).attr("width")),
            selectRectHeight = parseInt($(selectRectId + selectedRectIndex).attr("height")),
            selectRectX = parseInt($(selectRectId + selectedRectIndex).attr("x")),
            selectRectY = parseInt($(selectRectId + selectedRectIndex).attr("y"));
            if ((this.multiSelectDraw || !this.removeRect || !this.resize || !this.rectPan) && (selectRectWidth > 0 && selectRectHeight > 0)) {
                var rightRectOptions = {
                    'id': containerSvg + '_resize_rightRect' + selectedRectIndex,
                    'x': selectRectX + selectRectWidth - 5,
                    'y': selectRectY + 10,
                    'width': 10,
                    'height': selectRectHeight - 20 > 0 ? selectRectHeight - 20 : 0,
                    'fill': "transparent",
                    'stroke': "transparent",
                    'stroke-width': 2,
                    'cursor': 'e-resize'
                };
                this.svgRenderer.drawRect(rightRectOptions, this.gEle);
                var bottomRectOptions = {
                    'id': containerSvg + '_resize_bottomRect' + selectedRectIndex,
                    'x': selectRectX,
                    'y': selectRectY + selectRectHeight - 5,
                    'width': selectRectWidth - 10 > 0 ? selectRectWidth - 10 : 0,
                    'height': 10,
                    'fill': "transparent",
                    'stroke': "transparent",
                    'stroke-width': 2,
                    'cursor': 's-resize'
                };
                this.svgRenderer.drawRect(bottomRectOptions, this.gEle);
                var leftRectOptions = {
                    'id': containerSvg + '_resize_leftRect' + selectedRectIndex,
                    'x': selectRectX - 5,
                    'y': selectRectY,
                    'width': 10,
                    'height': selectRectHeight,
                    'fill': "transparent",
                    'stroke': "transparent",
                    'stroke-width': 2,
                    'cursor': 'w-resize'
                };
                this.svgRenderer.drawRect(leftRectOptions, this.gEle);
                var topRectOptions = {
                    'id': containerSvg + '_resize_topRect' + selectedRectIndex,
                    'x': selectRectX,
                    'y': selectRectY - 5,
                    'width': selectRectWidth - 10 > 0 ? selectRectWidth - 10 : 0,
                    'height': 10,
                    'fill': "transparent",
                    'stroke': "transparent",
                    'stroke-width': 2,
                    'cursor': 'n-resize'
                };
                this.svgRenderer.drawRect(topRectOptions, this.gEle);
                var bottomRightCornerCircleOptions = {
                    'id': containerSvg + '_resize_bottomRightCornerCircle' + selectedRectIndex,
                    'cx': selectRectX + selectRectWidth,
                    'cy': selectRectY + selectRectHeight,
                    'r': 10,
                    'fill': "transparent",
                    'stroke': "transparent",
                    'stroke-width': 2,
                    'cursor': 'se-resize'
                };
                this.svgRenderer.drawCircle(bottomRightCornerCircleOptions, this.gEle);
                var topRightCornerCircleOptions = {
                    'id': containerSvg + '_closeTopRightCornerCircle' + selectedRectIndex,
                    'cx': selectRectX + selectRectWidth,
                    'cy': selectRectY,
                    'r': 10,
                    'fill': "white",
                    'stroke': "#2988d6",
                    'stroke-width': 2,
                    'cursor': 'pointer'
                };
                this.svgRenderer.drawCircle(topRightCornerCircleOptions, this.gEle);
                var closingPathOptions = {
                    'id': containerSvg + '_closePath' + selectedRectIndex,
                    'x1': selectRectX + selectRectWidth - 4,
                    'y1': selectRectY - 4,
                    'x2': selectRectX + selectRectWidth + 4,
                    'y2': selectRectY + 4,
                    'stroke': "#2988d6",
                    'stroke-width': 2,
                    'cursor': 'pointer'
                };
                this.svgRenderer.drawLine(closingPathOptions, this.gEle);
                var closingPathOppositeOptions = {
                    'id': containerSvg + '_closePathOpposite' + selectedRectIndex,
                    'x1': selectRectX + selectRectWidth + 4,
                    'y1': selectRectY - 4,
                    'x2': selectRectX + selectRectWidth - 4,
                    'y2': selectRectY + 4,
                    'stroke': "#2988d6",
                    'stroke-width': 2,
                    'cursor': 'pointer'
                };
                this.svgRenderer.drawLine(closingPathOppositeOptions, this.gEle);
                cx = [16, 10, 4, 10, 4, 4];
                cy = [4, 4, 4, 10, 10, 16];
                index = ["a", "b", "c", "d", "e", "f"];
                for (var t = 0; t < 6; t++) {
                    var gripOptions = {
                        'id': containerSvg + '_gripCircle_' + index[t] + selectedRectIndex,
                        'cx': selectRectX + selectRectWidth - cx[t],
                        'cy': selectRectY + selectRectHeight - cy[t],
                        'r': 0.4,
                        'stroke': "#5B5B5B",
                        'stroke-width': 2,
                    }
                    this.svgRenderer.drawCircle(gripOptions, this.gripCollection);
                }
                if ((this.gripCollection != null))
                    $(this.gripCollection).appendTo(this.gEle);
                $("#" + containerSvg + "_gripCollection" + selectedRectIndex).css({ 'visibility': 'hidden' });
            }
            if (ej.util.isNullOrUndefined(this.oldRectX)) {
                this.oldRectX = []; this.oldRectY = []; this.oldRectWidth = []; this.oldRectHeight = [];
                this.oldReRightRectX = []; this.oldReTopRectY = []; this.oldReBottomRectY = [];
            }
            this.oldRectX[selectedRectIndex] = parseInt($(selectRectId + selectedRectIndex).attr('x'));
            this.oldRectY[selectedRectIndex] = parseInt($(selectRectId + selectedRectIndex).attr('y'));
            this.oldRectWidth[selectedRectIndex] = parseInt($(selectRectId + selectedRectIndex).attr('width'));
            this.oldRectHeight[selectedRectIndex] = parseInt($(selectRectId + selectedRectIndex).attr('height'));
            this.oldReRightRectX[selectedRectIndex] = parseInt($("#" + containerSvg + "_resize_rightRect" + selectedRectIndex).attr('x'));
            this.oldReTopRectY[selectedRectIndex] = parseInt($("#" + containerSvg + "_resize_topRect" + selectedRectIndex).attr('y'));
            this.oldReBottomRectY[selectedRectIndex] = parseInt($("#" + containerSvg + "_resize_bottomRect" + selectedRectIndex).attr('y'));
            this.oldMultiSelectType = this.multiSelectType;
            if (this.resize || this.rectPan) {
                //to remove the grip after resizing at the bottom right corner
                if (this.resize)
                    $("#" + this.svgObject.id + "_gripCollection" + selectedRectIndex).attr({ 'transform': "" });
                $("#" + containerSvg + "_closeTopRightCornerCircle" + selectedRectIndex).css({ 'display': 'inline' });
                $("#" + containerSvg + "_closePath" + selectedRectIndex).css({ 'display': 'inline' });
                $("#" + containerSvg + "_closePathOpposite" + selectedRectIndex).css({ 'display': 'inline' });
                $("#" + containerSvg + '_resize_rightRect' + selectedRectIndex).css({ 'cursor': 'e-resize' });
                $("#" + containerSvg + '_resize_leftRect' + selectedRectIndex).css({ 'cursor': 'w-resize' });
                $("#" + containerSvg + '_resize_bottomRect' + selectedRectIndex).css({ 'cursor': 's-resize' });
                $("#" + containerSvg + '_resize_topRect' + selectedRectIndex).css({ 'cursor': 'n-resize' });
            }
            this.drag = false;
            if (this.removeRect) {
                $("#" + containerSvg + "_selectedRectGroup" + selectedRectIndex).remove();
                if (this.parentgEle.childNodes.length <= 0) $(this.parentgEle).remove();
            }
            if ((this.multiSelectDraw || this.removeRect || this.resize || this.rectPan) && (selectRectWidth > 0 && selectRectHeight > 0)) {
                this.multiSelectDraw = false;
                this.multiSelectDataCalculation(evt);
            }
            if (this.resize) this.resize = false;
            if (this.rectPan) this.rectPan = false;
            this.removeRect = false;
            this.enableTrackBall();
        }
    },
    chartMouseUp: function (evt) {
        if (this.isTouch(evt)) {
            this.model.touchCross = false;
            this.chartMouseUpTouch(evt);          
        }
		if (!this.currentPageX && this.panning) {
            var mouseDownCords = this.calMousePosition(evt);
            this.currentPageX = mouseDownCords.X;
            this.currentPageY = mouseDownCords.Y;
		}
		
		var chart = this;		
		var model = this.model;
		this.mousedownPointX = this.mousedownPointY = null;
		$("[id*=" + "_PreviewSeries" + "]").remove();
		$("[id*=" + "_Marker" + "]").remove();
		if (chart.dragPoint) {		  
		    var i = this.dragIndex.seriesIndex,
		        chartSeries = model._visibleSeries[i];
		    if (!this.model.enableCanvasRendering) {
		        if (this.gPreviewSeriesGroupEle) {
		            $(this.svgObject).find("#" + this.svgObject.id + "_PreviewSeriesGroup_" + i).remove();
		            if (this.gPreviewSeriesGroupEle.childNodes.length > 0)
		                this.gPreviewSeriesGroupEle.removeChild(this.gPreviewSeriesGroupEle.childNodes[0]);
		        }
		    }
		    var commonDragEventArgs, valAxis, srIndex, ptIndex;
		    this.dragPoint = chartSeries.dragPoint =false;
		        var draggedPoints = this._getDraggedPoint(chartSeries, this.mousemoveX, this.mousemoveY),
                   xPoint = draggedPoints.X,
                   yPoint = draggedPoints.Y;
		        srIndex = chartSeries.data.seriesIndex,
                ptIndex = chartSeries.data.pointIndex;
		       
		    var oldValue = { X: this.commonDragEventArgs.data.oldValue.X, Y: this.commonDragEventArgs.data.oldValue.Y },
                    newValue = { X: xPoint, Y: yPoint };
		        commonDragEventArgs = $.extend({}, this.commonDragEventArgs);
		        commonDragEventArgs.data = { series: chartSeries, seriesIndex: srIndex, pointIndex: ptIndex, oldValue: oldValue, newValue: newValue };
		        this._trigger("dragEnd", commonDragEventArgs);
		        chart.commonDragEventArgs = commonDragEventArgs;
		        chart._changeDraggingPoints(chartSeries, ptIndex, commonDragEventArgs.data.newValue.X, commonDragEventArgs.data.newValue.Y);
		        chartSeries.region = null;
		        this.redraw();		// to redraw the chart after changing the points  
        }
        if (this.multiSelectAreaType == 'cartesianaxes' && (!model.zooming.enable) && (!model.enableCanvasRendering))
            this.multiSelectMouseUp(evt);
        var chart = this;
        if (model.zooming.enable && model.AreaType == "cartesianaxes") {
            var zoomRect = $("#" + this.svgObject.id + "_ZoomArea");
            if (zoomRect[0]) {
                if (model.enableCanvasRendering) {
                    var zoomArea = zoomRect[0].getClientRects()[0];
                    var zoomRectWidth = zoomArea.width;
                    var zoomRectHeight = zoomArea.height;
                } else {
                    var $zoomRect = $(zoomRect);
                    var zoomRectWidth = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "width"));
                    var zoomRectHeight = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "height"));
                }
            }
            if (zoomRectWidth > 0 && zoomRectHeight > 0) {
                chart.doZoom(zoomRect, zoomRectWidth, zoomRectHeight);
            }
            else
                this.drag = false;
            if (!chart.panning)
                this.enableTrackBall();
        }
       // Panning is done on mouse up for other devices and when deferredZoom is set to true
        var id = this.svgObject.id;
        var parentId = ej.util.isNullOrUndefined(evt.target.parentNode) ? "" : evt.target.parentNode.id;

        //condition to find the buttons
        if (parentId == id + "_ResetZoom" || parentId == id + '_ZoomBtn' || evt.target.id == id + "_ResetZoom" || evt.target.id == id + '_ZoomBtn') {
            this.panning = false;
            this.model._chartAreaZoom = false;
        }

        //Panning for other devices
        if ((ej.isTouchDevice() ||model.zooming.enableDeferredZoom) && this.panning && this.doPan && model.AreaType == "cartesianaxes") {
            var oDelta;
            oDelta = {
                'x': this.oPreviousCoords.x - this.currentPageX,
                'y': this.oPreviousCoords.y - this.currentPageY
            };

            this.oPreviousCoords = {
                'x': this.currentPageX,
                'y': this.currentPageY
            };
            $.each(model._axes, function (index, axis) {
                var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
            });
            this.model.legendCollapsed = false;
            this.redraw(true, true, evt.target); //to avoid touch event termination in mobiles / deferred pan
            this._cursorToPointer();
            this._enableZoomingButtons();
        }
        
        if (this.doPan)
            this.doPan = false;
        
        this.rotateActivate = false;
    },
    multiSelectMouseDown: function (evt) {
        this.multiSelectAreaType = this.model.AreaType.toLowerCase();
        var model = this.model, chartX = model.m_AreaBounds.X, chartWidth = model.m_AreaBounds.Width,
        chartY = model.m_AreaBounds.Y, chartHeight = model.m_AreaBounds.Height, mouseDownCords = this.calMousePosition(evt);
        //to check whether the click is made within the chart area
        if (this.multiSelectAreaType == 'cartesianaxes' && (!model.zooming.enable) && (!model.enableCanvasRendering)
             && mouseDownCords.X >= chartX && mouseDownCords.X < (chartX + chartWidth) && mouseDownCords.Y < (chartY + chartHeight) && mouseDownCords.Y >= chartY) {
            var seriesCollection = this.model.series,
                seriesCollectionLength = seriesCollection.length,
                id, mouseDownCords,
                containerSvg = this.svgObject.id,
             selectMode = seriesCollection.some((function (val) {
                 return val.selectionSettings.mode.toLowerCase() == 'range';
             }));
            this._isMultiSelect = seriesCollection.some((function (val) {
                return val.selectionSettings.enable == true && val.selectionSettings.mode.toLowerCase() == "range";
            }));
            if (selectMode) this.multiSelectMode = 'range';
            for (var i = 0; i < seriesCollectionLength; i++) {
                if ((seriesCollection[i].selectionSettings.enable == true) && (seriesCollection[i].selectionSettings.mode.toLowerCase() == 'range'))
                    this.multiSelectType = seriesCollection[i].selectionSettings.rangeType.toLowerCase();
            }
            if (this._isMultiSelect && this.multiSelectMode == 'range') {
                this.disableTrackBall();
                mouseDownCords = this.calMousePosition(evt);
                this.mouseDownX = mouseDownCords.X;
                this.mouseDownY = mouseDownCords.Y;
                this.drag = true;
                if (($(this.parentgEle).find("g").length < 1))
                    this.selectedRectIndex = 0;
                else {
                    id = $(this.parentgEle).find("g").last().attr("id");
                    matchStr = containerSvg + "_gripCollection";
                    id = parseInt(id.substr(matchStr.length));
                    this.selectedRectIndex = id + 1;
                }
                if ($(this.parentgEle).find("g").length == 0) {
                    this.parentgEle = this.svgRenderer.createGroup({ 'id': containerSvg + '_rectSelectionGroup' });
                }
                this.gEle = this.svgRenderer.createGroup({ 'id': containerSvg + '_selectedRectGroup' + this.selectedRectIndex });
                if ($(this.gripCollection).find("g").length == 0)
                    this.gripCollection = this.svgRenderer.createGroup({ 'id': containerSvg + '_gripCollection' + this.selectedRectIndex });                
                if (evt.target.id.indexOf("resize") >= 0) {
                    id = evt.target.id.split("resize")[1].match(/\d+/)[0];
                    this.selectedRectIndex = eval(id);
                }
                if (evt.target.id.indexOf("close") >= 0) {
                    id = evt.target.id.split("close")[1].match(/\d+/)[0];
                    this.selectedRectIndex = eval(id);
                }
                if ((evt.target.id.indexOf(this.svgObject.id + "_selectRect") >= 0)) {
                    matchStr = this.svgObject.id + "_selectRect";
                    this.selectedRectIndex = parseInt(evt.target.id.substr(matchStr.length));
                }
                this.currentIndex = $(evt.target.parentNode).index();
                var axes = this.model._axes,
                axesLength = axes.length, axis,
                mouseMoveCords = this.calMousePosition(evt),
                mouseMoveX = mouseMoveCords.X,
                mouseMoveY = mouseMoveCords.Y;
                this.multiAxis = []
                for (var k = 0; k < axesLength; k++) {
                    axis = axes[k];
                    if ((mouseMoveX > axis.Location.X1) && (mouseMoveX < axis.Location.X2))
                        this.multiAxis.push(axis);
                    if ((mouseMoveY < axis.Location.Y1) && (mouseMoveY > axis.Location.Y2))
                        this.multiAxis.push(axis);
                }
            }
        }
    },
 //Triggers while start dragging column or bar points
      _grab: function (chartSeries, chartObj, srIndex, ptIndex,point) {		
        commonDragEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonDragEventArgs.data = {seriesIndex: srIndex, pointIndex: ptIndex, currentPoint: point};
        chartObj._trigger("dragStart", commonDragEventArgs);
        chartSeries.data = commonDragEventArgs.data;
        this.commonDragEventArgs = commonDragEventArgs;
        this.mousedownPointX = this.mousedownPointY = null;
      },
 _enableDragging: function(chartSeries,seriesIndex,e){
	 var chartObj = this, closestXyPoint, data,
         type = chartSeries.type.toLowerCase(),
         areaType = chartObj.model.AreaType,      
         visibility = chartSeries.visibility.toLowerCase();	     
            if ( this.mousedownPointX && this.mousedownPointY && areaType == "cartesianaxes" && !this.model.enable3D &&visibility == 'visible' &&(type=="line"|| type=="spline"||type=="area"||
                type=="stepline"|| type=="steparea" || type=="splinearea" || type=="column" || type=="bar" || type=="bubble" || type=="scatter")) {
                var serY = [],  serX = [];                  
				closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousedownPointX, this.mousedownPointY, e);
                    this.mousemoveX = this.mousedownPointX;
                    this.mousemoveY = this.mousedownPointY;
                    data = this.GetSeriesPoint(e);
                    if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {                       
                        this.dragPoint = chartSeries.dragPoint = true;
                        closestXyPoint.seriesIndex = seriesIndex;
                        chartSeries.pointData = closestXyPoint;
                       	var point = { xValue: closestXyPoint.point.xValue, yValue: closestXyPoint.point.YValues };
                        chartObj._grab(chartSeries, chartObj, seriesIndex, closestXyPoint.index, point);
                        this.dragIndex = { pointIndex: closestXyPoint.index, seriesIndex: seriesIndex };                      
                    }
                    if (!ej.util.isNullOrUndefined(data)) {
                        var region = data.region;
							if (seriesIndex == region.SeriesIndex) {
							    chartSeries.region = data.region;
							    this.dragPoint = chartSeries.dragPoint = true;
							    var xVal = chartSeries._visiblePoints[region.Region.PointIndex].xValue,
                                    yVal = chartSeries._visiblePoints[region.Region.PointIndex].YValues,
                                    point = { xValue: xVal, yValue: yVal };
							    chartObj._grab(chartSeries, chartObj, region.SeriesIndex, region.Region.PointIndex, point);
							    this.dragIndex = { pointIndex: region.Region.PointIndex, seriesIndex: region.SeriesIndex };                     
                            }
					}               
          }                                  
    },
 chartMouseDown: function (e) {
  
        //this.cancelEvent(e);   
      
        var model = this.model,
            matchStr = this._id + '_scrollbar' + '_',
            parentNodeId = (e.target.parentNode && e.target.parentNode.id) ? e.target.parentNode.id : '',
            selectionIndex = parentNodeId.indexOf(matchStr) > -1 ? parseInt(parentNodeId.substr(matchStr.length)) : NaN,
            isZoom = this.isZoomButtonHovered(e.target);
            axes = model._axes,
            chart = this,            
            mouseDownCords = this.calMousePosition(e),
            browserInfo = this.model.browserInfo;
            this.mousedownPointX = mouseDownCords.X;
            this.mousedownPointY = mouseDownCords.Y;
            if ("which" in e)  // Gecko (Firefox), WebKit (Safari/Chrome) & Opera
                this.chartRightClick = e.which == 3;
            else if ("button" in e)  // IE, Opera 
                this.chartRightClick = e.button == 2;
        if (!isNaN(selectionIndex) && !ej.util.isNullOrUndefined(selectionIndex))
            axes[selectionIndex].previousRange = $.extend(true, {}, model._axes[selectionIndex].visibleRange);       
   
        this.mousemoveX = this.mouseDownX = mouseDownCords.X;
        this.mousemoveY = this.mouseDownY = mouseDownCords.Y;                    
        if (this.isTouch(e)) {            
            this.model.event = e;
            this.model.touchCross = true;
            //this.cancelEvent(e);  
            if ($.finish)
                $(this.model.trackerElement).finish();
            else
                $(this.model.trackerElement).stop(true, true);
            this.model.tapNum = this.model.tapNum || 0;          
            if (!isZoom)
                this.doubleTap(e);           
            if (chart.model.crosshair.visible && !isZoom) {                
                var timer = setTimeout(function () {                   
                    var zoom = $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + '_ZoomArea'),
                        padding = chart.model.elementSpacing / 2,
                        pointer = chart.getEvent(chart.model.event), currX = pointer.pageX,
                        currY = pointer.pageY;                    
                    if (chart.model.touchCross && !chart.panning && Math.abs(currX - chart.model.cachedX) < padding && Math.abs(currY - chart.model.cachedY) < padding && (zoom.length == 0 || (parseFloat($(zoom).attr("width")) == 0 || parseFloat($(zoom).attr("height")) == 0))) {
                        chart.chartInteractiveBehavior(e);
                        chart.model.crosshairMoved = true;
                        chart.drag = false;
                    }
                    

                }, 500);

            }
          
        }

        if (this.model.enable3D && this.model.enableRotation)
            this.rotateActivate = true;

        if (!this.svgRenderer.vmlNamespace) {
            this.oPreviousCoords = {};
            if (e.target.id.indexOf("resize") >= 0) {
                id = e.target.id.split("resize")[1].match(/\d+/)[0];
                this.selectedRectIndex = eval(id);
                this.multiSelectMouseDownId = e.target.id;
                this.resize = true;
            }
            if (e.target.id.indexOf("close") >= 0) {
                id = e.target.id.split("close")[1].match(/\d+/)[0];
                this.selectedRectIndex = eval(id);
                this.removeRect = true;
            }
            if ((e.target.id.indexOf(this.svgObject.id + "_selectRect") >= 0)) {
                matchStr = this.svgObject.id + "_selectRect";
                this.selectedRectIndex = parseInt(e.target.id.substr(matchStr.length));
            }
            if (e.target.id == this.svgObject.id + '_selectRect' + this.selectedRectIndex) {
                this.rectPan = true;
                this.PreviousCoords = {
                    'X': e.pageX, 'Y': e.pageY
                }
            }
            if(!this.isTouch(e))
            this.multiSelectMouseDown(e);
            var parent = this.svgRenderer._getAttrVal($(e.target).parent(), "id");
            var isZoomToolkit = parent != this.svgObject.id + "_ZoomBtn" && parent != this.svgObject.id + "_ZoomInBtn" && parent != this.svgObject.id + "_ZoomOutBtn" && parent != this.svgObject.id + "_ResetZoom" && parent != this.svgObject.id + "_PanBtn"; 
            if (isZoomToolkit && this.model.zooming.enable && !this.model.isLazyZooming && !(this.panning) && this.model.AreaType == "cartesianaxes") {
             
                if (this.mouseDownX >= this.model.m_AreaBounds.X && this.mouseDownX < (model.m_AreaBounds.X + model.m_AreaBounds.Width) && this.mouseDownY < (model.m_AreaBounds.Y + model.m_AreaBounds.Height) && this.mouseDownY >= model.m_AreaBounds.Y && !this.isTouch(e)) {
                    if(isNaN(selectionIndex) || ej.util.isNullOrUndefined(selectionIndex))
                        this.drag = true;
                    this.disableTrackBall();
                }


            }
            else if (isZoomToolkit && this.panning || $(e.target)[0].className == "e-hhandle e-box") {
                var mousePanCords = this.calMousePosition(e);
                this.mousePanX = mousePanCords.X;
                this.mousePanY = mousePanCords.Y;                
                if (this.mousePanX >= model.m_AreaBounds.X && this.mousePanX < (model.m_AreaBounds.X + model.m_AreaBounds.Width) && this.mousePanY < (model.m_AreaBounds.Y + model.m_AreaBounds.Height + 18) && this.mousePanY >= model.m_AreaBounds.Y) {
                    this.doPan = true;
                   
                }
            }
            if(this.rotateActivate ||  this.doPan)
                this.oPreviousCoords = {
                    'x': e.pageX,
                    'y': e.pageY
                };
        }
     
    },
    chartMouseLeave: function (evt) {            
              
        this.cancelEvent(evt);

            if (!this.isTouch(evt)) {
                // Changes the visibility to hidden

                $("[id*=" + "_TrackSymbol" + "]").remove();

                this._removeHighlight();

                this._removeTrackBall();

                $("#" + this.svgObject.id + "_TrackToolTip").hide();

                $("#" + this._id + "_tooltip").remove();
                if (($(".tooltipDiv" + this._id).hasClass(this.svgObject.id)))
                    $(".tooltipDiv" + this._id).remove();
            }



            // chartMouseLeave event           
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY }, size: { height: this.model.svgHeight, width: this.model.svgWidth }, id: evt.target.id, pageX: evt.pageX, pageY: evt.pageY };
            this._trigger("chartMouseLeave", commonEventArgs);         
        
    }, 
  

    _doClick: function (evt) {
		this.drag = false;
        if(!this.model.crosshairMoved)
        this._removeHighlight();
        if (!this.model.selectedDataPointIndexes)
            this.model.selectedDataPointIndexes = [];
        this.model.clusterPoints = [];
        this.model._isStateChaged = false;
        var seriesIndex, pointIndex, seriesLenth, chartSeries, data, pointer = this.getEvent(evt),
        currX = pointer.pageX,	   
        currY = pointer.pageY,
        padding = 10,
		targetId = evt.target.id,
	    svgObjectId = this.svgObject.id,
        seriesCollection = this.model._visibleSeries,
		isZoom = this.isZoomButtonHovered(evt.target),
		targetid = evt.target.id;
			 
           //Set current seriesCollection Selection type for selection state maintains            
			seriesLength = seriesCollection.length;           
           var isMultiSelection, mode, commonEventArgs, text, getIndex;
           data = this.GetSeriesPoint(evt);
           if (data) {
               seriesIndex = data.region.SeriesIndex;
               pointIndex = data.region.Region.PointIndex;
           }else {
               //Getting SeriesIndex and PointIndex for Path element type series when marker and dataLabels visible
               if (this.model.selectionEnable) {
                   if (!this.model.enable3D && !(evt.target.id.indexOf('LegendItem') != -1) && this.model.AreaType != 'none') {
                       text = evt.target.id;
                       getIndex = text.match(/(\d+)/g);
                       if (getIndex && (text.indexOf('symbol') != -1 || text.indexOf('Series') != -1) && !(text.indexOf('Text') != -1)) {
                           seriesIndex = parseInt(getIndex[0]);
                           pointIndex = isNaN(parseInt(getIndex[1])) ? 0 : parseInt(getIndex[1]);
                       } else if (getIndex && (text.indexOf('Text') != -1 || text.indexOf('dataLabel') != -1)) {
                           seriesIndex = parseInt(getIndex[0].charAt(0));
                           pointIndex = isNaN(parseInt(getIndex[0].charAt(1))) ? parseInt(getIndex[1].charAt(0)) : parseInt(getIndex[0].charAt(1));
                       }
                   }
               }
           }
		   if (!isZoom || this.model.enableCanvasRendering) {
				for (var i = 0; i < this.model._visibleSeries.length; i++) {
					chartSeries = this.model._visibleSeries[i];
					var type = chartSeries.type.toLowerCase();
					if (this.model.AreaType == "cartesianaxes" && chartSeries.visibility.toLowerCase() == 'visible' && type !== "scatter" && type !== "bubble" && type !== "column" && type.indexOf("bar") == -1 && type !== "stackingcolumn" && type !== "stackingcolumn100" && !chartSeries._hiloTypes) {
						var serY = [];
						var serX = [];
						var location = null;
						var mouseMoveCords = this.calMousePosition(evt);
						this.mousemoveX = mouseMoveCords.X;
						this.mousemoveY = mouseMoveCords.Y;
						var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY, evt);
						if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {
						    seriesIndex = i;
						    pointIndex = closestXyPoint.index;
							var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
							commonPointEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY }, region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
							this._trigger("pointRegionClick", commonPointEventArgs);
						}
					}
				}			  
				if (data && !isZoom) {				 
					commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
					commonEventArgs.data = data;
					this._trigger("pointRegionClick", commonEventArgs);
				}				
			}
			
        if (this.model.AreaType == "none" && this.model.enable3D && $(evt.target)[0].nodeName == "path" && (!this.isTouch(evt) || (Math.abs(currX - this.model.cachedX) < padding && Math.abs(currY - this.model.cachedY) < padding))){
            index = $(evt.target)[0].id.match(/(\d+)/g);
            seriesIndex = parseInt(index[index.length - 2]);
            pointIndex = parseInt(index[index.length - 1]);
            var currentexplodeindex = this.model._visibleSeries[seriesIndex].explodeIndex;
			 var actualIndex = this.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].actualIndex;
            if (currentexplodeindex == actualIndex)
                this.model._visibleSeries[seriesIndex].explodeIndex = null;           
            else
                this.model._visibleSeries[seriesIndex].explodeIndex = actualIndex;
            if (!this.vmlRendering && !this.model.enableCanvasRendering)
                var selection = this.selectedStyle(this);
            $(this.svgObject).empty();
			$(this.legendSvgContainer).empty();
            this.draw();
            if (this.model.AreaType == 'none') {
                this.model._isStateChaged = false;
				var className,elementsLenth,
				styleLength=selection.length,
                pathId = "_Region_Series_" + seriesIndex + "_Point_" + pointIndex,
                gElement = $(this.svgObject).find('[id$=' + pathId + '],[id*=' + pathId + 'back],[id*=' + pathId + 'front]');
				 elementsLength=gElement.length;
                for (var k = 0; k < styleLength; k++) {                    
                    for (var i = 0; i < elementsLength; i++) {
                        if (selection[k].id == gElement[i].id) {
                            className = selection[k].className;
                            if (className.indexOf('SelectionStyle') < 0 && (className.indexOf('Selection' + name + 'Style') < 0)) {
                                $("[id=" + gElement[i].id + "]").attr('class', className);
                            }
                        }
                    }              
                }
                if (selection.pattern)
                    this.svgRenderer.append(selection.pattern, this.svgObject);
            }
            else {
                $('[id*=' + this.svgObject.id + '_LegendItemShape],[id*=_Region_Series_]').each(function () {
                    $(this).attr('class', '');
                });
            }
            }  
        var legenddata = this.getLegendData(evt);
        if (legenddata) {
            $('#template_group_' + this._id).remove();
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = legenddata;
            this._trigger("legendItemClick", commonEventArgs);
            var seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
            // legend selection logic perform here
            if (!this.model.legend.toggleSeriesVisibility && legenddata.series.selectionSettings.enable) {                 
                this.model._isStateChaged = false;
                this.legendSelection(this, legenddata, evt, data);
            }
            if (!commonEventArgs.cancel && this.model.legend.toggleSeriesVisibility) {
				$('[id*=' + this._id + '_Selection_' + ']').remove();
				var length = this.model._visibleSeries.length;
				if (this.model.AreaType != "none" || this.model.AreaType == "none" && type != 'pieofpie' && type != 'pyramid' && type != 'funnel' && length > 1) {
                    var seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
                    var trendlineIndex = legenddata.legendItem.LegendItem.TrendLineIndex;
                    if (ej.util.isNullOrUndefined(trendlineIndex)) {
                    legenddata.series.visibility = legenddata.series.visibility.toLowerCase() === 'visible' ? 'hidden' : 'visible';
                        this.model.series[seriesIndex].visibility = legenddata.series.visibility;
                    if (this._notifyArrayChange)
                            this._notifyArrayChange("series[" + seriesIndex + "]visibility", legenddata.series.visibility);
                    } else {
                        var trendlineVisible = legenddata.series.trendlines[trendlineIndex].visibility.toLowerCase();
                        trendlineVisible = trendlineVisible === 'visible' ? 'hidden' : 'visible';
                        this.model.series[seriesIndex].trendlines[trendlineIndex].visibility = trendlineVisible;
                        if (this._notifyArrayChange)
                            this._notifyArrayChange("series[" + seriesIndex + "].trendlines[" + trendlineIndex + "].visibility", trendlineVisible);
                }
                }
                else {
                    if (this.model._isPieOfPie) {                        
                        var point = this._getPieOfPiePoint(legenddata.legendItem.LegendItem.ActualIndex, legenddata.series,legenddata)._visibility;
                        var pointIndex = legenddata.legendItem.LegendItem.ActualIndex;
                    }
                else{
                    var point = legenddata.series.visiblePoints[legenddata.legendItem.LegendItem.PointIndex]._visibility;
                    var pointIndex = legenddata.series.visiblePoints[legenddata.legendItem.LegendItem.PointIndex].actualIndex;
					}
                    legenddata.series.points[pointIndex]._visibility= point === 'visible' ? 'hidden' : 'visible';
                }
                if (this.model.enableCanvasRendering) {
                    /** Canvas Chart image is cleared when click the legend item to show/hide the series**/
                    var chartRect = document.getElementById(this._id).getClientRects()[0];
                    this.svgRenderer.ctx.clearRect(0, 0, chartRect.width, chartRect.height);
                    //Canvas series marker element removed
					$("#" + this._id).find('[id*="canvas_symbol"]').remove();
                }
                $(this.svgObject).empty();
            	$(this.legendSvgContainer).empty();
                for (var j = 0; j < this.model.series.length; j++)
                    this.model.series[j].regionAdded = false;
                this.model.legendCollapsed = true;                
                this.disableAnimation();
                this.draw();
                this.enableAnimation();
                if (this.model.AreaType != "none" && this.zoomed)
                    this._enableZoomingButtons();
            }
        }

        //axisLabelClick event
        var axisData = this.getAxisLabelData(evt);
        if (axisData) {
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = axisData;
            this._trigger("axisLabelClick", commonEventArgs);

        }
        
		// multi level labels click event
		var labelData = this.getAxisMultiLevelLabelsData(evt);
        if (labelData) {
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = labelData;
            this._trigger("multiLevelLabelClick", commonEventArgs);
        }
		
        //annotationClick event
        if (targetid.indexOf("annotation_") >= 0) {
		     var len = targetid.lastIndexOf("_");
            var str = targetid.substr(len + 1, targetid.length);
            var index = parseInt(str);
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY }, contentData: this.model.annotations[index], pageX: evt.pageX, pageY: evt.pageY };
            this._trigger("annotationClick", commonEventArgs);
  
  
        }

        //Selection started here
        isSelectionSetting = !ej.isNullOrUndefined(seriesIndex) ? this.model._visibleSeries[seriesIndex].selectionSettings.enable : false;
        if (!ej.isNullOrUndefined(seriesIndex) && !legenddata && isSelectionSetting && (!this.isTouch(evt) || (!this.model.crosshairMoved && Math.abs(currX - this.model.cachedX) < padding && Math.abs(currY - this.model.cachedY) < padding))) {
            isMultiSelection = seriesCollection[seriesIndex].selectionSettings.type.toLowerCase() == 'multiple' ? true : false;
            mode = seriesCollection[seriesIndex].selectionSettings.mode;
            if (mode.toLowerCase() != 'range')
            this.segmentSelection(evt, legenddata, seriesIndex, pointIndex, data, seriesCollection);
        }
        this.model.crosshairMoved = false;
		 //chartClick event
        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY },size:{height:this.model.svgHeight,width:this.model.svgWidth}, id: targetid, pageX: evt.pageX, pageY: evt.pageY };
        this._trigger("chartClick", commonEventArgs);
		
    },

   //Getting targetId when selection state changed by redraw,setmodel
   getTargetId:function(seriesIndex,pointIndex, mode){
      var getSelectionId,
      type = this.model._visibleSeries[seriesIndex].type.toLowerCase(),
      isAreaType = (type.indexOf("area") != -1 || type.indexOf("line") != -1 || type.indexOf("scatter")!=-1 ? true:false) ? true : false;
      if (this.model.enable3D)
          $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + pointIndex + "]").each(function () {
                  getSelectionId = this.id;
              });
      else {
         if (mode != 'series' || this.model.AreaType=='none')
             !isAreaType? $("[id$=" + this.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex ).each(function () {
                    getSelectionId = this.id;
                 }): $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex + '_symbol' + "]").each(function () {
                         getSelectionId = this.id;
                     });
         else 
           $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + "]").each(function () {
                   getSelectionId = this.id;
               });
      }
         return getSelectionId;
   },

  findCanvasSelection: function (seriesIndex, pointIndex, mode) {
      var isSelectionFound;
      if (mode == 'point')
          isSelectionFound = $('#' + this._id + '_Selection_series' + seriesIndex + '_point_' + pointIndex + '_canvas').length;
      else if (mode == 'series')
          isSelectionFound = $('#' + this._id + '_Selection_series' + seriesIndex + '_canvas').length;
      else
          isSelectionFound = $('#' + this._id + '_Selection_Cluster' + '_point_' + pointIndex + '_canvas').length;
      return isSelectionFound==1 ? true : false;
  },

  //Selection logic started here
  segmentSelection: function (evt, legendData, seriesIndex, pointIndex, data) {
      var seriesCollection = this.model._visibleSeries,
          series = seriesCollection[seriesIndex];
          
      if (ej.isNullOrUndefined(series)) return 0;
      var selectionSettings = series.selectionSettings,
        isSelectionSettings = selectionSettings.enable,
        isMultiSelection = selectionSettings.type.toLowerCase() == 'multiple' ? true : false,
        mode = selectionSettings.mode,
        isTrackball = false, chart = this,
        targetId, parentNodeId, isSelectionFound, isElement, sIndex, pIndex, data, dataLength, seriesFlag, accPointFlag, clusterFlag, accSeriesClusterFlag, className;
        this.model.isSelected = true;
       //Calculating cluster points
        if (!ej.isNullOrUndefined(pointIndex) && mode == 'cluster') {
            var clusterPoints = [],
            points, pointsLength,
            xvalue = seriesCollection[seriesIndex]._visiblePoints[pointIndex].xValue;
            for (var i = 0, len = seriesCollection.length; i < len ; i++) {
                points = seriesCollection[i].points;
                pointsLength = points.length;
                for (var j = 0; j < pointsLength; j++)
                    if (xvalue == points[j].xValue)
                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
            }
            this.model.clusterPoints = clusterPoints;
        }

        //Get Target and parent node Id's by using seriesIndex and pointIndex
        if (!this.model.enableCanvasRendering) {
            targetId = this.getTargetId(seriesIndex,pointIndex,mode);
            if (targetId) {
                parentNodeId = $('#' + targetId)[0].parentNode.id;
                var parentNode = $('#' + parentNodeId)[0];
                //Checking wether target parent or tracker symbols  present or not
                if (parentNode != null) {
                   if (parentNodeId) {
                           isElement = (parentNodeId.indexOf(this.svgObject.id + "_TextGroup_") >= 0 ||  parentNodeId.indexOf(this.svgObject.id + "_symbolGroup") >= 0) ? true : false;
                           if (parentNodeId.indexOf(this.svgObject.id + "_TrackSymbol_") >= 0) {
                                   isElement = true;
                                   isTrackball = true;
                               }
                       }
                  }
            }
        }        
       
        if (this.model.enableCanvasRendering)
            isSelectionFound = this.findCanvasSelection(seriesIndex, pointIndex, mode);
        else {
            className = $("#" + targetId).attr('class') ? $("#" + targetId).attr('class') : '';
            isSelectionFound = className.indexOf('Selection') != -1 ? true : false;
            if (this.model.enable3D) {
                className = $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + pointIndex + "]").attr('class') ? $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + pointIndex + "]").attr('class') : '';
                isSelectionFound = className.indexOf('Selection') != -1 ? true : false;
            }
        }
      //Selection Removal Logic 
      if ((isSelectionFound || !isMultiSelection) || isMultiSelection) {
          this.removeSelection(seriesIndex, pointIndex, seriesCollection, mode, isMultiSelection, legendData, isSelectionFound);          
          data = this.model.selectedDataPointIndexes;
          dataLength = data.length;         
          if (dataLength > 0 && !this.model._isStateChaged) {
              if (!isMultiSelection) {
                 for (var k = 0; k < dataLength; k++) {
                     sIndex = data[k].seriesIndex;
                     pIndex = data[k].pointIndex;
                     seriesFlag = (mode == 'series' && ((sIndex == seriesIndex && pIndex == pointIndex) || (sIndex != seriesIndex || pIndex != pointIndex)));
                     accPointFlag = ((mode == 'point'|| this.model.AreaType=='none') && (sIndex == seriesIndex && pIndex == pointIndex));
                     clusterFlag = (mode == 'cluster' && (pIndex == pointIndex));
                     if (seriesFlag || accPointFlag || clusterFlag) {
                         data.splice(k, 1); dataLength = data.length; k = -1;
                     }
                 }                    
              } else if (isSelectionFound) {
                      for (var k = 0; k < dataLength; k++) {
                          sIndex = data[k].seriesIndex;
                          pIndex = data[k].pointIndex;
                          seriesFlag = (mode == 'series' && (sIndex == seriesIndex));
                          accPointFlag = ((mode == 'point'||this.model.AreaType=='none') && (sIndex == seriesIndex && pIndex == pointIndex));
                          clusterFlag = (mode == 'cluster' && (pIndex == pointIndex));
                          if (((seriesFlag || clusterFlag) && this.model.AreaType != 'none') || accPointFlag ) {
                              data.splice(k, 1); dataLength = data.length; k = -1;
                          }
                      }
              }
          }
      }
     //Selection Started for 2D,3D,Canvas
      if (!isSelectionFound) {
          this.model.enableCanvasRendering ? this.canvasSelection(this, evt, series, legendData, seriesIndex, pointIndex, data, selectionSettings) : this.selection(chart, evt, series, isTrackball, legendData, seriesIndex, pointIndex, parentNodeId, targetId, data);
          data = this.model.selectedDataPointIndexes;
          dataLength = data.length;
          if (!this.model._isStateChaged) {
              if (dataLength > 0) {
                  if (!isMultiSelection) {
                      for (var k = 0; k < dataLength; k++) {
                          sIndex = data[k].seriesIndex;
                          pIndex = data[k].pointIndex;
                          accSeriesClusterFlag = ((mode == 'series' || mode == 'cluster' || this.model.AreaType == 'none') && (data[k].SeriesIndex != seriesIndex || data[k].PointIndex != pointIndex));
                          pointFlag = (mode == 'point' && (data[k].SeriesIndex == seriesIndex));
                          if (accSeriesClusterFlag || pointFlag) {
                              data.splice(k, 1); dataLength = this.model.selectedDataPointIndexes.length; k = -1;
                          }
                      }
                      data.push({ 'event': evt, 'legendData': legendData, 'seriesIndex': seriesIndex, 'pointIndex': pointIndex, 'data': data });
                  } else {
                      data.push({ 'event': evt, 'legendData': legendData, 'seriesIndex': seriesIndex, 'pointIndex': pointIndex, 'data': data });
                  }
              } else
                  data.push({ 'event': evt, 'legendData': legendData, 'seriesIndex': seriesIndex, 'pointIndex': pointIndex, 'data': data });
          }
      }

      //Getting selected data here
      var selectedData = { selectedData: this.model.selectedDataPointIndexes };
      this._trigger("seriesRegionClick", selectedData);
    },


     //legend Selection logic perform here
  legendSelection: function (chart, legenddata, evt, data) {
      var selectedData = this.model.selectedDataPointIndexes ? this.model.selectedDataPointIndexes : [],
	  length =this.model._visibleSeries.length,
        index = (chart.model.AreaType == 'none' && length == 1) ? legenddata.legendItem.LegendItem.PointIndex : legenddata.legendItem.LegendItem.SeriesIndex;
        seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
        pointIndex = index;
        legendItem = chart.svgObject.id + '_LegendItemShape' + index,
        legendClass = $('#' + legendItem).attr('class') ? $('#' + legendItem).attr('class') : '';                  
        if ($('#' + chart._id + '_Selection_Legend' + index + '_canvas').length == 1 || legendClass.indexOf('Selection')!=-1) {
             $('#' + chart._id + '_Selection_Legend' + index + '_canvas').remove();
             $("[id*=" + chart._id + '_Selection_series' + index + "]").remove();
             $('#' + this._id + '_Selection_series' + seriesIndex + '_point_' + pointIndex + '_canvas').remove();
             $('#' + legendItem).attr('class', '');
             $("[id*=" + chart.svgObject.id + '_Series' + seriesIndex + "]").attr('class', '');
             $("[id*=" + '_Region_Series_' + seriesIndex + "]").attr('class', '');
             for (var k = 0; k < selectedData.length; k++) {
                 if (selectedData[k].seriesIndex == index || (this.model.AreaType == 'none' && selectedData[k].pointIndex == index)) {
                     selectedData.splice(k, 1); break;
                 }
             }
         }             
        else                                    
           chart.segmentSelection(evt, legenddata, seriesIndex, pointIndex, data);       
    },

    chartMouseClick: function (evt) {
	
	this.cancelEvent(evt);
        if (window.navigator.msPointerEnabled) {
            evt = evt.originalEvent;
        }
        this._doClick(evt);
    },
    chartMouseDoubleClick: function (evt) {

      
        if (window.navigator.msPointerEnabled) {
            evt = evt.originalEvent;
        }
        //chartDoubleClick event
        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY },size:{height:this.model.svgHeight,width:this.model.svgWidth}, id: evt.target.id, pageX: evt.pageX, pageY: evt.pageY };
        this._trigger("chartDoubleClick", commonEventArgs);

    },
    drawTrackerSymbol: function (series, seriesIndex, ptIndex, tracker, point, id,trackcount) {

        var type = series.type.toLowerCase(), offsetX, offsetY;
        var chartSeriesObj = new ej.seriesTypes[type]();
		var index = series.type.toLowerCase() =="boxandwhisker" ? trackcount : ptIndex;
        var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
		offsetX = !series.isTransposed ? series.xAxis.plotOffset : series.yAxis.plotOffset;
		offsetY = !series.isTransposed ? series.yAxis.plotOffset : series.xAxis.plotOffset;
		 var clipRectOptions = {
		     'id': this.svgObject.id + '_ClipRectTrack' + '_' + seriesIndex,
            'x': - offsetX,
            'y': - offsetY,
            'width': trans.width + offsetX * 2,
            'height': trans.height + offsetY * 2,
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'Gray'
        };       
        if (this.model.AreaType != "none") {
            if (this.model.AreaType == 'cartesianaxes' && ((point.X + trans.x) <= (trans.x + trans.width)) && ((point.X + trans.x) >= trans.x || point.X == 0) &&
                 ((point.Y + trans.y) >= (trans.y) || point.Y == 0)) {
                if (this.model.crosshair.visible || type == "bubble" || type == "scatter") {
                    if (!series.trackerRemoved ) {
                        var trackId = this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex;
						trackId = (type == "boxandwhisker") ? trackId + index : trackId ;
                        $("#" + this._id).find('[id*=' + trackId + ']').remove();
                        var trackId = "canvas_trackSymbol_" + seriesIndex;
						trackId = (type == "boxandwhisker") ? trackId + index : trackId ;
                        $("#" + this._id).find('[id*=' + trackId + ']').remove();
                       
                    }					
                    var transSymbolOptions = { 'clip-path': 'url(#' + this.svgObject.id + '_ClipRectTrack' + '_' + seriesIndex + ')', 'class': 'Tracker', 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex + '_' + index, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                    this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);                   
                } else
				    this.gTrackerEle = null; // empty the previously created tracker group if no tracker is drawn now
                }
            else {
					
						$('#' + this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex + '_' + index).remove();
						var transSymbolOptions = { 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex + '_' + index };				
                this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);
                if(!this.model.enableCanvasRendering) { 
                    if (this.model.AreaType == 'polaraxes')
                        this.gTrackerEle.setAttribute('clip-path', 'url(#' + this.svgObject.id + '_SeriesGroup_' + seriesIndex + '_ClipRect)');
                    else {
                        this.gTrackerEle.setAttribute('clip-path', 'url(#' + this.svgObject.id + '_ClipRectTrack' + '_' + seriesIndex + ')');
                        this.gTrackerEle.setAttribute('transform', 'translate(' + trans.x + ',' + trans.y + ')');
                    }
               }
            }
            if ($("#" + this.svgObject.id + "_CrosshairGroup").length == 0) {
                var gTrackballOptions = { 'id': this.svgObject.id + '_CrosshairGroup', 'visibility': 'visible' };
                this.gTrackball = this.svgRenderer.createGroup(gTrackballOptions);
                this.svgRenderer.append(this.gTrackball, this.svgObject);
            }
            if (this.model.crosshair.marker.visible && !series.isIndicator && series._visiblePoints[ptIndex].visible) {
				chartSeriesObj.drawSymbol(seriesIndex, series, ptIndex, (point.X), (point.Y), this, tracker,trackcount);
                if (point.low)
                    chartSeriesObj.drawSymbol(seriesIndex, series, ptIndex, (this.model.requireInvertedAxes ? point.low : point.X), (this.model.requireInvertedAxes ? point.Y : point.low), this, tracker);                
            }
            if (this.gTrackerEle && $("#" + this.gTrackerEle.id).length < 1)
                this.svgRenderer.append(this.gTrackerEle, this.gTrackball);
        }
        if ($(this.svgObject).find("#" + this.svgObject.id + '_ClipRectTrack' + '_' + seriesIndex).length == 0 && !this.vmlRendering) {
            this.svgRenderer.drawClipPath(clipRectOptions, $("#" + this.svgObject.id + "_CrosshairGroup"));
        }
            
        
    },
    getClosesPointXY: function (serX, serY, series, x, y, evt) {
        var closestPoint,
            valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes),outlierPosition,
            ptIndex, chartPoint, location, pointIndex, pointVisible,closestX, padding = 10, isTouch=this.isTouch(evt),
            visiblePointsLength = series._visiblePoints ? series._visiblePoints.length : 0, closestY,
            size = series.marker ? $.extend(true, {}, series.marker.size) : { height: 6, width: 6 };
            size.height = (isTouch && size.height < padding * 2) ? size.height + padding : size.height;
            size.width = (isTouch && size.width < padding * 2) ? size.width + padding : size.width;
        if ((y <= (valAxis.y + valAxis.height) && valAxis.y <= y) && (valAxis.x <= x && x <= (valAxis.x + valAxis.width))) {
            for (var i = 0; i < visiblePointsLength; i++) {
                chartPoint = series._visiblePoints[i];
                location = chartPoint.location;
                pointVisible = false;
                pointIndex = i;
                closestX = null;
                closestY = null;
				
				if(series.type == "boxandwhisker" && (!this.model.enable3D) && !(ej.util.isNullOrUndefined(chartPoint.boxPlotLocation)))
				{ 
				
				for(var k=0;k<chartPoint.boxPlotLocation.length;k++)
				{					
					var symbolOptions ={ X:chartPoint.boxPlotLocation[k].X,Y:chartPoint.boxPlotLocation[k].Y
					};
					if(chartPoint.boxPlotLocation[k].outlier == true){
						if (x > ((symbolOptions.X + valAxis.x )- size.width) && x < (symbolOptions.X + valAxis.x + size.width) ) {
							closestX = chartPoint.x;						
						if (ej.util.isNullOrUndefined(closestX))
                            pointVisible = chartPoint.visible;
						if (y > ((symbolOptions.Y + valAxis.y )- size.height)  &&  y < symbolOptions.Y + valAxis.y + size.height + padding ) {					  
						     closestY = chartPoint.boxPlotValues.outliers[k];
							 outlierPosition = {X:symbolOptions.X,Y:symbolOptions.Y};
						}    
						if(!ej.util.isNullOrUndefined(outlierPosition))
						{
							closestPoint = chartPoint;
							ptIndex = i;                   
						}
					 }
				   }
				
				  }								
				}				
                if (location) {
                    if (x > location.X + valAxis.x - (size.width / 2) && x < location.X + valAxis.x + (size.width / 2)) {
                        closestX = chartPoint.x;
                        if (ej.util.isNullOrUndefined(closestX))
                            pointVisible = chartPoint.visible;
                    }
                    if (y > location.Y + valAxis.y - (size.height / 2) && y < location.Y + valAxis.y + (size.height / 2)) {
                        closestY = chartPoint.YValues[0];
                    }
                    if ((!ej.util.isNullOrUndefined(closestX) || pointVisible) && !ej.util.isNullOrUndefined(closestY)) {
                        closestPoint = chartPoint;
                        ptIndex = i;
                    }
                }
            }         
        }
        return { point: closestPoint, index: ptIndex, outlierPosition: outlierPosition };
    },
    getClosestPointX: function (serX, series, x, y) {
        var closestPoint = [];
        var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        if (this.model.requireInvertedAxes) {
            x = Math.abs(y - (valAxis.y + valAxis.height));
            y = Math.abs(x - valAxis.x);
        }
        else {
            x = Math.abs(x - valAxis.x);
            y = Math.abs(y - (valAxis.y + valAxis.height));
        }
        var mousePoint = ej.EjSvgRender.utils._getValuebyPoint(x, y, series);
        var pointX = (this.model.requireInvertedAxes) ? mousePoint.PointX : mousePoint.PointX;
        var ptIndex = [];
        
        $.each(series.points, function (pointIndex, chartPoint) {
            serX.push(chartPoint.xValue);
        });
        var closest = this.getClosest(serX, pointX);
		if (!ej.util.isNullOrUndefined(closest)) {
            if (typeof closest == 'number') {
                $.each(series._visiblePoints, function (pointIndex, cPoint) {
                    if (cPoint.xValue == closest) {
                        closestPoint.push(cPoint);
                        ptIndex.push(pointIndex);

                    }
                });
            } else {
                closest = closest.getTime();
                $.each(series._visiblePoints, function (pointIndex, cPoint) {
                    if (cPoint.xValue.getTime() == closest) {
                        closestPoint.push(cPoint);
                        ptIndex.push(pointIndex);

                    }
                });
            }
        }
        return { point: closestPoint, index: ptIndex };
    },
    getClosest: function (obj, val) {
        var closest = null;
        this._beforeMin = false;
        this._afterMax = false;

        // Work out min and max
        var min = Math.min.apply(null, obj);
        var max = Math.max.apply(null, obj);

        // Only calculate closest if point is within array
        if (val >= min - 0.5 && val <= max + 0.5) {
            var i;
            for (i = 0; i < obj.length; i++) {
                if (closest == null || Math.abs(obj[i] - val) < Math.abs(closest - val)) {
                    closest = obj[i];
                }
            }
            this._closest = null;
        }
        else {
            if (val > max) {
                closest = obj[obj.length - 1];
                this._closest = ej.util.isNullOrUndefined(this._closest) ? closest : (closest > this._closest ? closest : this._closest);
                this._afterMax = true;
            }
            else {
                closest = obj[0];
                this._closest = ej.util.isNullOrUndefined(this._closest) ? closest : (closest < this._closest ? closest : this._closest);
                this._beforeMin = true;

            }
        }
        return closest;
    },
    createTooltip: function (region, evt, series) {
        if(!series)
          series = this.model._visibleSeries[region.SeriesIndex];
        var seriesPoint = (series.type == "pieofpie") ? this._getPieOfPiePoint(region.Region.PointIndex, series) : series._visiblePoints[region.Region.PointIndex];
        var point = ( this.dragPoint)? series.pointCollection[region.Region.PointIndex] : $.extend(true, {}, seriesPoint);
		var tooltipMargin=10;
        if (point.visible) {
		     var format = series.tooltip.format;
			if(series.type == "boxandwhisker")
			{
				var mouseMoveCords = this.calMousePosition(evt);
				this.mousemoveX = mouseMoveCords.X;
				this.mousemoveY = mouseMoveCords.Y;
				var flag = region.Region.Bounds ? true : false;	
				if(flag){
					if(region.Region.Bounds.Width == series.outlierSettings.size.width){
						for(var s =0;s<point.boxPlotLocation.length;s++){
							if(point.boxPlotLocation[s].outlier == true){
								if((point.boxPlotLocation[s].X + (series.outlierSettings.size.width/2) > (this.mousemoveX - series.xAxis.x ))&& (point.boxPlotLocation[s].X - (series.outlierSettings.size.width/2) < (this.mousemoveX - series.xAxis.x) )){
									if((point.boxPlotLocation[s] .Y + (series.outlierSettings.size.height/2) > (this.mousemoveY - series.yAxis.y)) && (point.boxPlotLocation[s] .Y - (series.outlierSettings.size.height/2) < (this.mousemoveY - series.yAxis.y ))){
										var dataPoints = this.getTooltipFormat(point, series, region.SeriesIndex, region.Region.PointIndex, format,point.boxPlotLocation[s]);
										point.x = dataPoints.data.x;
										point.y = dataPoints.text;
									}
								}
							}
						}
					}
					else
					{
						var dataPoints = this.getTooltipFormat(point, series, region.SeriesIndex, region.Region.PointIndex, format,point.boxPlotLocation[0]);
						point.x = dataPoints.data.x;
						point.y = dataPoints.text;
					}
				}
			}
			else
			{
				var dataPoints = this.getTooltipFormat(point, series, region.SeriesIndex, region.Region.PointIndex, format);
				point.x = dataPoints.data.x;
				point.y = dataPoints.data.y;
			}
		    var tooltip = null;
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { currentText: tooltip, seriesIndex: region.SeriesIndex, pointIndex: region.Region.PointIndex, series: series };
            this._trigger("toolTipInitialize", commonEventArgs);
            var tooltipdiv = $('.tooltipDiv' + this._id);
			$("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
            $("#" + this.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden').hide();
            if ($('.tooltipDiv' + this._id).length == 0) {
                this.tooltipFirst = true;
                tooltipdiv = $("<div></div>").attr('class', 'tooltipDiv' + this._id).css({ 'position': 'absolute', 'z-index': '13000', 'display': 'block' });             
                $(document.body).append(tooltipdiv);
                $('.tooltipDiv' + this._id).addClass(this.svgObject.id);
            } else {
                $(".tooltipDiv" + this._id).css("display", "block");
                this.tooltipFirst = false;
            }
            if (series.tooltip.template != null) {
                var cloneNode = $("#" + series.tooltip.template).clone();
                $('.tooltipDiv' + this._id)[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                series.count = 1;
                point.count = 1;
                var seriesColor = this.getSeriesColor(point, region.SeriesIndex, series);
				if (seriesColor)
                    $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
				else
                   $(tooltipdiv).css("background-color", this.model.pointColors[region.Region.PointIndex]);
				var data = { series: series, point: point };
				if (!ej.util.isNullOrUndefined(commonEventArgs.data.currentText))
				    $(tooltipdiv).html($(cloneNode).html(commonEventArgs.data.currentText));
                else
                $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
            } else {
                $(tooltipdiv).html(commonEventArgs.data.ToolTip);
            }
			var areaBounds = this.model.m_AreaBounds;
            var xPos = evt.pageX  + tooltipMargin;
            var yPos = evt.pageY  + tooltipMargin;
            var tooltipWidth = $(tooltipdiv).width();
            var tooltipHeight = $(tooltipdiv).height();
            var position = document.getElementById(this.svgObject.id).getClientRects()[0];
            if (xPos === undefined || xPos === null)
                xPos = evt.pageX  + tooltipMargin;
            if (yPos === undefined || yPos === null)
                yPos = evt.pageY  + tooltipMargin;
            var leftPos = xPos;
			var topPos = yPos;
            //Checking top position whether tooltip display outside of the area bounds
            if ((xPos - $(this.svgObject).offset().left+ tooltipWidth) > (areaBounds.X + areaBounds.Width )) {
                var diff = (xPos - (tooltipWidth + (2* tooltipMargin)));
                leftPos = diff + $(document).scrollLeft();
            }
            if ((yPos - $(this.svgObject).offset().top + tooltipHeight) > areaBounds.Y + areaBounds.Height ) {
                var diff = (yPos + tooltipHeight) - (areaBounds.Y + areaBounds.Height + position.top)
                topPos = yPos - diff + $(document).scrollTop();
            }
			
			$(tooltipdiv).css('left',leftPos);
			$(tooltipdiv).css('top',topPos);
			
			var tooltipOptions = series.tooltip;
            if (tooltipOptions.enableAnimation) {
                $(tooltipdiv).css({
                    'transition-property': 'left,top',
                    '-moz-transition-property': 'left,top', /* Firefox 4 */
                    '-webkit-transition-property': 'left,top', /* Safari and Chrome */
                    '-o-transition-property': 'left,top',
                    'transition-duration': tooltipOptions.duration,
                    '-moz-transition-duration': tooltipOptions.duration, /* Firefox 4 */

                    '-webkit-transition-duration': this.tooltipFirst ? '0s' : tooltipOptions.duration, /* Safari and Chrome */
                    '-o-transition-duration': tooltipOptions.duration /* Opera */
                });
            }
            templateRect = $(tooltipdiv)[0].getBoundingClientRect();
            if(templateRect.top < areaBounds.Y + position.top)
                $(tooltipdiv).css('top', areaBounds.Y + position.top + $(document).scrollTop());
        }
    },

    translate: function (axis, translateX, translateY, currentScale) {
        var offset = axis.orientation.toLowerCase() == "horizontal"
                                              ? translateX / axis.width / currentScale
                                              : translateY / axis.height / currentScale;

        axis.zoomPosition = axis.orientation.toLowerCase() == "horizontal"
            ? ej.EjSvgRender.utils._minMax(axis.zoomPosition + offset, 0, (1 - axis.zoomFactor))
            : ej.EjSvgRender.utils._minMax(axis.zoomPosition - offset, 0, (1 - axis.zoomFactor));
    },
    highlightFill: function (highlight, chart, seriesIndex, pointIndex, legendData) {
        var seriesColors = chart.model.seriesColors;
        var pointColors = chart.model.pointColors;
        var color = highlight.color;
		var length =this.model._visibleSeries.length;
        if (legendData && chart.model.AreaType == 'none') {
            if(length == 1)
            pointIndex = legendData.legendItem.LegendItem.PointIndex;
            else
            seriesIndex = legendData.legendItem.LegendItem.SeriesIndex;
        }
        var pointColor = legendData ? seriesColors[seriesIndex] : pointIndex ? chart.model.series[seriesIndex].points[pointIndex] ? chart.model.series[seriesIndex].points[pointIndex].fill: null : null;
        var fill = (color != "" ? color : chart.model.AreaType == 'none' ? pointColors[pointIndex] : legendData ? seriesColors[seriesIndex] : pointColor ? pointColor : seriesColors[seriesIndex]);
        if (Object.prototype.toString.call(fill) === '[object Array]')
            fill = fill[1].color;
        return fill;
    },
	
	
   
     // Canvas highlight and selection started here
     // Canvas highlight started here
    canvasHighlight: function (chart, evt, series, legendData) {

        var highlight = series.highlightSettings;
        var data = series.data;
        var color = highlight.color;
        var opacity = highlight.opacity;
        var borderColor = highlight.border.color;
        var borderWidth = highlight.border.width;
        var patternName = highlight.pattern.toLowerCase();
        var legendVisible = chart.model.legend.visible;
        var mode = highlight.mode;
        var seriesIndex = series.seriesIndex;
        var pointIndex = series.pointIndex;
        var seriesType = series.type.toLowerCase();
        var containerStyle = document.getElementById(chart._id + '_canvas').getBoundingClientRect();
        var regions = [];
        var points = [];
        var clusterPoints = [];
		var length =this.model._visibleSeries.length;
        var fill = this.highlightFill(highlight, chart, seriesIndex, pointIndex,legendData);
        if (chart.model.AreaType != 'none' || (chart.model.AreaType == 'none' && length > 1 && (mode == 'series' || legendData))) {
            for (var i = 0, len = chart.model.chartRegions.length; i < len ; i++) {
                if ((mode == 'series' || legendData) && (seriesIndex == chart.model.chartRegions[i].SeriesIndex))
                        regions.push(chart.model.chartRegions[i].Region);
                    }
                   
               
        } else {
            for (var i = 0, len = chart.model.chartRegions.length; i < len; i++) {
                if (seriesIndex == chart.model.chartRegions[i].SeriesIndex && pointIndex == chart.model.chartRegions[i].Region.PointIndex)
                    regions.push(chart.model._isPieOfPie ? chart._getPieOfPiePoint(chart.model.chartRegions[i].Region.PointIndex, series) : chart.model.chartRegions[i].Region[pointIndex]);
            }
        }
        if (mode == 'cluster' && chart.model.Areatype != 'none' && typeof pointIndex !== "undefined") {
            var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].xValue;
            for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                    if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].xValue)
                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };

                }
            }
        }
        for (var k = 0; k < clusterPoints.length; k++) {
            clusterseriesIndex = clusterPoints[k].seriesIndex;
            clusterpointIndex = clusterPoints[k].pointIndex;
            for (var l = 0; l < chart.model.chartRegions.length; l++) {
                if (clusterseriesIndex == chart.model.chartRegions[l].SeriesIndex && clusterpointIndex == chart.model.chartRegions[l].Region.PointIndex)
                    if (mode != "cluster")
                    regions.push(chart.model.chartRegions[l].Region);
                    else
                        regions.push(chart.model.chartRegions[l]);
            }
        }
      var index = (chart.model.AreaType == 'none' && length == 1) ? pointIndex : seriesIndex;
        if ((mode == 'series' || legendData) && ($('#' + chart._id + '_Selection_series' + index + '_canvas').length == 0) && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0) {             
            points = series._visiblePoints
            ctx = this.createCanvasElement(chart._id + '_Highlight_series' + seriesIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
            this.createRect(chart, ctx);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("waterfall") != -1 || seriesType.indexOf("bar") != -1) {
                for (i = 0; i < regions.length; i++) {
                    var isUnSelected = $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + i + '_canvas').length == 0 ? true : false;
                    this.canvasSeriesRect(regions[i], opacity, borderColor, borderWidth, color, isUnSelected, series);
                }
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut' || seriesType == "pieofpie")
			{
               regions = (length == 1) ? regions : regions[0];
                this.canvasHighlightCircle(chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions, seriesType, ctx);
				}
            else if (seriesType == 'pyramid' || seriesType == 'funnel')
                this.canvasHighlightPyramid(chart, regions[0], highlight, ctx, opacity, borderColor, borderWidth, color, seriesType);
            else if (seriesType == 'bubble')
                this.canvasBubbleHighlight(highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            else if (seriesType == 'scatter')
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor);
        }
		else if (mode == 'cluster' && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0 && this.model.AreaType != 'none' && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_series' + seriesIndex + '_canvas').length == 0) {
                for (var i = 0, len = this.model._visibleSeries.length; i < len; i++) {
                    var series = this.model._visibleSeries[i];
                    series.seriesIndex = i;
                    points.push(series._visiblePoints[pointIndex]);

                }
            var sbRegion = [], colors = [];// scatter & bubble region calculation
                ctx = this.createCanvasElement(chart._id + '_Highlight_' + '_Cluster' + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
                this.createRect(chart, ctx);
                for (var i = 0, len = this.model._visibleSeries.length; i < len; i++) {
                sbRegion = [];
                    highlight = chart.model._visibleSeries[i].highlightSettings;
                    color = highlight.color;
                    opacity = highlight.opacity;
                    borderColor = highlight.border.color;
                    borderWidth = highlight.border.width;
                    patternName = highlight.pattern.toLowerCase();
                    fill = this.highlightFill(highlight, chart, i, pointIndex, index, legendData);
                    color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
                colors.push(color);
                for (var r = 0, rlen = regions.length; r < rlen; r++) {
                    if (regions[r].SeriesIndex == i) {
                        sbRegion.push(regions[r].Region);
                    }
                }
                if (seriesType.indexOf("column") != -1 || seriesType.indexOf("waterfall") != -1 || seriesType.indexOf("bar") != -1) {
                        var isUnSelected = $('#' + chart._id + '_Selection_' + i + '_point_' + pointIndex + '_canvas').length == 0 ? true : false;
                    for (var r = 0, rlen = sbRegion.length; r < rlen; r++) {
                        this.canvasSeriesRect(sbRegion[r], opacity, borderColor, borderWidth, color, isUnSelected, series);
                    }
                }
                    else if (seriesType == 'bubble') {
                    this.canvasBubbleHighlight(highlight, sbRegion, chart, i, color, borderWidth, opacity, borderColor);
                    }
                    else if (seriesType == 'scatter') {
                    this.canvasScatterHighlight(sbRegion, color, borderWidth, opacity, borderColor);
                    }

                }
            }
		else if ((mode=="cluster" &&this.model.AreaType == 'none') || mode == 'point' && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_series' + seriesIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0) {
            points[0] = this.model._isPieOfPie? this._getPieOfPiePoint(pointIndex,series):series._visiblePoints[pointIndex];
            ctx = this.createCanvasElement(chart._id + '_Highlight_' + seriesIndex + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            this.createRect(chart, ctx);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("waterfall") != -1 || seriesType.indexOf("bar") != -1) {
                regions = []; regions[0] = data.region.Region;
                this.canvasSeriesRect(regions[0], opacity, borderColor, borderWidth, color, true, series);
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut' || seriesType == "pieofpie") {
                regions = []; regions[0] = data.pointData[0];
                this.canvasHighlightCircle(chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions, seriesType, ctx);
            }
            else if (seriesType == 'pyramid' || seriesType == 'funnel') {
                this.canvasHighlightPyramid(chart, regions[0], highlight, ctx, opacity, borderColor, borderWidth, color, seriesType);
            }
            else if (seriesType == 'bubble') {
                regions = []; regions[0] = data.region.Region;
                this.canvasBubbleHighlight(highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            }
            else if (seriesType == 'scatter') {
                regions = []; regions[0] = data.region.Region;
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor);
            }
        }
        // legend highlight (legend symbol)
        if (legendVisible && $('#' + chart._id + '_Selection_Legend' + index + '_canvas').length == 0) {
            if (mode == 'cluster' && !legendData && this.model.AreaType != 'none') {
                var serIndex;
                for (var i = 0, len = clusterPoints.length; i < len; i++) {
                    serIndex = clusterPoints[i].seriesIndex;
                    chartSeries = this.model._visibleSeries[serIndex];                   
                    this.canvasHighlightLegend(chart, chartSeries, highlight, 'Highlight', containerStyle, colors[serIndex]);
                }
            }
            else
                this.canvasHighlightLegend(chart, series, highlight, 'Highlight', containerStyle, color);
            
        }
        if (series.visibility === 'visible' && series.marker.visible == true && this.model.AreaType != 'none')
            this.canvasHighlightMarker(chart, series, points, highlight, ctx, evt, colors);
    },

     // canvas rect operations perform here
    canvasSeriesRect: function (regions, opacity, borderColor, borderWidth, color, isUnSelected, series) {
        var options = {
            'opacity': opacity,
            'stroke': borderColor,
            'stroke-width': borderWidth,
            'fill': color
        }
		var cornerRadius = series.cornerRadius;
        if (regions) {
            options.x = regions.Bounds.X;
            options.y = regions.Bounds.Y;
            options.width = regions.Bounds.Width;
            options.height = regions.Bounds.Height;
            if (isUnSelected)
				if ((typeof (cornerRadius) != "object" && cornerRadius > 0) || cornerRadius.topLeft > 0 || cornerRadius.bottomLeft > 0
					          || cornerRadius.topRight > 0 || cornerRadius.bottomRight > 0) {
                    var roundrect = ej.EjSvgRender.utils._calculateroundedCorner(cornerRadius, options);
                    options.d = roundrect;
                    ej.EjCanvasRender.prototype.drawPath(options, ctx);
                }
                else
					this.drawRect(options, ctx);
        }
    },

     // // canvas selection logic started here
    canvasSelection: function (chart, evt, series, legendData, seriesIndex, pointIndex, data, selectionSettings) {
        series.seriesIndex = seriesIndex;
        series.pointIndex = pointIndex;
        var seriesCollection = this.model._visibleSeries,
        color = selectionSettings.color,
        opacity = selectionSettings.opacity,
        borderColor = selectionSettings.border.color,
        borderWidth = selectionSettings.border.width,
        patternName = selectionSettings.pattern.toLowerCase(),
        legendVisible = chart.model.legend.visible,
        mode = selectionSettings.mode,
        seriesType = series.type.toLowerCase(),
        containerStyle = document.getElementById(chart._id + '_canvas').getBoundingClientRect(),
        regions = [],
        points = [],
        length = this.model._visibleSeries.length,
        chartRegions = chart.model.chartRegions,
        clusterPoints = this.model.clusterPoints,
        seriesData = { seriesIndex: seriesIndex, series: series },
         index = (chart.model.AreaType == 'none' && length == 1) ? pointIndex : seriesIndex,
        fill = this.highlightFill(selectionSettings, chart, seriesIndex, pointIndex, legendData ,data),regionsLength,cRlength;
		regionsLength = chartRegions.length;
           if (chart.model.AreaType != 'none' || (chart.model.AreaType == 'none' && length > 1)) {        
            for (var i = 0; i < regionsLength; i++) {
                if (((mode == 'series' || legendData) && seriesIndex == chartRegions[i].SeriesIndex) || ((mode == 'point' || legendData) && (seriesIndex == chartRegions[i].SeriesIndex && pointIndex == chartRegions[i].Region.PointIndex))) {
                        if (chart.model.AreaType == 'none' && mode == 'point' && !legendData)
                            regions.push(chartRegions[i].Region[pointIndex])
                        else
                            regions.push(chartRegions[i].Region);
                    }           
                }
           } else if (regionsLength > 0) {
               if (this.model._isPieOfPie) {
                   if (legendData) {
                       for (var p = 0; p < series.pieCollections.length; p++) {
                           var piePoints = series.pieCollections[p];
                           for (var q = 0; q < piePoints.length; q++) {
                               if (piePoints[q].actualIndex == legendData.legendItem.LegendItem.PointIndex) {
                                   var pieColIndex = p;
                                   break;
                               }
                           }
                       }
                   }
                   var pieSrIndex = series.data ? series.data.pointData[0].PieSeriesIndex : pieColIndex;
                   var chartRegion = chartRegions[pieSrIndex];
               }
               else
                   var chartRegion = chartRegions[0];
                   cRlength = chartRegion.Region.length;
            for (var i = 0; i < cRlength; i++) {
                if (pointIndex == chartRegion.Region[i].PointIndex)
                    regions.push(chartRegion.Region[i]);
            }
        }
        if ((mode == 'series' || legendData) && (this.model.AreaType!='none' || this.model.AreaType == 'none' && length >1) ) {
            points = series._visiblePoints;
            ctx = this.createCanvasElement(chart._id + '_Selection_series' + index + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, selectionSettings);
            this.createRect(chart, ctx);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("waterfall") != -1 || seriesType.indexOf("bar") != -1) {
                for (i = 0; i < regions.length; i++)
                    this.canvasSeriesRect(regions[i], opacity, borderColor, borderWidth, color, true, series);
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut' || seriesType == "pieofpie")
                this.canvasHighlightCircle(chart, selectionSettings, pointIndex, opacity, borderColor, borderWidth, color, regions[0], seriesType, ctx);
            else if (seriesType == 'bubble')
                this.canvasBubbleHighlight(selectionSettings, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            else if (seriesType == 'scatter')
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor);
        }
	    else if (mode == 'cluster' && this.model.AreaType != 'none' && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0) {
	        var clength = clusterPoints.length, clusterseriesIndex, clusterpointIndex, region, rlength;
	        for (var k = 0; k < clength; k++) {
	                clusterseriesIndex = clusterPoints[k].seriesIndex;
	                clusterpointIndex = clusterPoints[k].pointIndex;
	                region = $.grep(chartRegions, function (x) { return x.SeriesIndex == clusterseriesIndex && x.Region.PointIndex == clusterpointIndex });
	                rlength = region.length;
	                if (rlength > 0) {
	                    regions.push(region[0]);
	                    points.push(seriesCollection[region[0].SeriesIndex]._visiblePoints[pointIndex]);
	                }else
	                    points.push(seriesCollection[k]._visiblePoints[pointIndex]);
	            }
	            this.selectedPoint = pointIndex;
	            var sbRegion=[], colors=[];
	            ctx = this.createCanvasElement(chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
	            this.createRect(chart, ctx);
                for (i = 0, len = seriesCollection.length; i < len; i++) {
	                sbRegion=[];// scatter & bubble region calculation
                    highlight = seriesCollection[i].selectionSettings;
                    color = highlight.color;
                    opacity = highlight.opacity;
                    borderColor = highlight.border.color;
                    borderWidth = highlight.border.width;
                    patternName = highlight.pattern.toLowerCase();
                    fill = this.highlightFill(highlight, chart, i, pointIndex,  legendData);
                    color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
                    colors.push(color);
                    for (var r = 0, rlen = regions.length; r < rlen; r++) {
                        if (regions[r].SeriesIndex == i) {
                            sbRegion.push(regions[r].Region);
                        }
                    }
                    if (seriesType.indexOf("column") != -1 || seriesType.indexOf("waterfall") != -1 || seriesType.indexOf("bar") != -1){
                        for (var sr = 0, srlen = sbRegion.length; sr < srlen; sr++) {
                            this.canvasSeriesRect(sbRegion[sr], opacity, borderColor, borderWidth, color, true, series);
                        }
                    }
                    else if (seriesType == 'bubble') 
                        this.canvasBubbleHighlight(highlight, sbRegion, chart, i, color, borderWidth, opacity, borderColor);
                    else if (seriesType == 'scatter') 
                        this.canvasScatterHighlight(sbRegion, color, borderWidth, opacity, borderColor);
                    if (!legendData && this.model.AreaType != 'none' && legendVisible) {
                        series = seriesCollection[i];
                        series.seriesIndex = i;
                        this.canvasHighlightLegend(chart, series, selectionSettings, 'SelectionCluster', containerStyle, color);
                    }
                }
            }
	    else if ((mode == 'point' && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0) || this.model.AreaType == 'none') {
            this.selectedPoint = pointIndex;
	        
            points[0] = this.model._isPieOfPie? this._getPieOfPiePoint(pointIndex,series):series._visiblePoints[pointIndex];
            ctx = this.createCanvasElement(chart._id + '_Selection_series' + seriesIndex + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            this.createRect(chart,ctx);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, selectionSettings);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("waterfall") != -1 || seriesType.indexOf("bar") != -1)
                this.canvasSeriesRect(regions[0], opacity, borderColor, borderWidth, color, true, series);
            else if (seriesType == 'pie' || seriesType == 'doughnut' || seriesType == "pieofpie")
                this.canvasHighlightCircle(chart, selectionSettings, pointIndex, opacity, borderColor, borderWidth, color, regions, seriesType, ctx);
            else if (seriesType == 'pyramid' || seriesType == 'funnel') 
                this.canvasHighlightPyramid(chart, regions[0], selectionSettings, ctx, opacity, borderColor, borderWidth, color, seriesType);
            else if (seriesType == 'bubble')  
                this.canvasBubbleHighlight(selectionSettings, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            else if (seriesType == 'scatter') 
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor);
        }   
        // legend highlight (legend symbol)
        if (legendVisible && (mode != 'cluster' || legendData))
            this.canvasHighlightLegend(chart, series, selectionSettings, 'Selection', containerStyle, color);
        if (series.marker.visible == true && this.model.AreaType != 'none')
            this.canvasHighlightMarker(chart, series, points, selectionSettings, ctx, evt, colors);
    },

     // clip rect logic calculated here
    createRect: function (chart,ctx) {
        if (chart.model.AreaType != 'none') {
            ctx.lineWidth = 0;
            ctx.strokeStyle = 'transparent';
            ctx.rect(chart.model.m_AreaBounds.X, chart.model.m_AreaBounds.Y, chart.model.m_AreaBounds.Width, chart.model.m_AreaBounds.Height);
            ctx.clip();
            ctx.stroke();
            ctx.save();
        }
    },

     //canvas pyramid and funnel draw
    canvasHighlightPyramid: function (chart, regions, highlight, ctx, opacity, borderColor, borderWidth, color, seriesType) {
        var fill = (highlight.color != "" ? highlight.color : chart.model.pointColors[regions.PointIndex]);
        var measureTitle = (chart.model.title.text) ? ej.EjSvgRender.utils._measureText(chart.model.title.text, $(this.svgObject).width() - chart.model.margin.left - chart.model.margin.right, chart.model.title.font) : 0;
        var pyrX = chart.pyrX = ((chart.model.legend.position.toLowerCase() === "left") ? (chart.model.LegendViewerBounds.Width) : 0) + chart.model.elementSpacing + chart.model.margin.left;
        var pyrY = chart.pyrY = ((chart.model.legend.position.toLowerCase() === "top") ? (chart.model.LegendViewerBounds.Height) : 0) + ((chart.model.title.text) ? (chart.model._titleLocation.Y + measureTitle.height) : (chart.model.margin.top + chart.model.elementSpacing));
        var options = {
            'opacity': opacity,
            'stroke': borderColor,
            'stroke-width': borderWidth,
            'fill': color,
            'type': seriesType,
            'd': "M" + " " + (pyrX + regions.Line1.x) + " " + (pyrY + regions.Line1.y) + " " + "L" + " " + (pyrX + regions.Line2.x) + " " + (pyrY + regions.Line2.y) + " " + "L" + " " + (pyrX + regions.Line3.x) + " " + (pyrY + regions.Line3.y) + " " + "L" + " " + (pyrX + regions.Line4.x) + " " + (pyrY + regions.Line4.y) + " " + "z"
        }
        if (seriesType == 'funnel')
            options.d = "M" + " " + (pyrX + regions.Line1.x) + " " + (pyrY + regions.Line1.y) + " " + "L" + " " + (pyrX + regions.Line2.x) + " " + (pyrY + regions.Line2.y) + " " + "L" + " " + (pyrX + regions.Line3.x) + " " + (pyrY + regions.Line3.y) + " " + "L" + " " + (pyrX + regions.Line4.x) + " " + (pyrY + regions.Line4.y) + " " + "L" + " " + (pyrX + regions.Line5.x) + " " + (pyrY + regions.Line5.y) + " " + "L" + " " + (pyrX + regions.Line6.x) + " " + (pyrY + regions.Line6.y) + " " + "z";
        ej.EjCanvasRender.prototype.drawPath(options, ctx);
    },

     // canvas scatter logic
    canvasScatterHighlight: function (regions, color, borderWidth, opacity, borderColor) {
        var location, options = {}, size = {};
        for (var i = 0; i < regions.length; i++) {
            location = { startX: regions[i].Bounds.X + regions[i].Bounds.Height/2, startY: regions[i].Bounds.Y + regions[i].Bounds.Width/2 };
            size.height = regions[i].Bounds.Height;
            size.width = regions[i].Bounds.Width;
            options = {};
            options.ShapeSize = size;
            options.cx = location.startX;
            options.cy = location.startY;
            options.Style = {};
            options.Style.BorderColor = borderColor;
            options.Style.Color = color;
            options.Style.Opacity = opacity;
            options.Style.BorderWidth = borderWidth;
            ej.EjSvgRender.chartSymbol['_drawCircle'](location, options, this, ctx);
        }
    },

     // canvas bubble logic perform here
    canvasBubbleHighlight: function (highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor) {
        var size = 10, radius, fill, bubbleOptions = {};
        for (var i = 0; i < regions.length; i++) {
            radius = regions[i].Bounds.Height / 2;
            fill = (highlight.color != "" ? highlight.color : chart.model.seriesColors[seriesIndex]);
            bubbleOptions = {
                'cx': regions[i].Bounds.X - this.model.series[seriesIndex].xAxis.x + radius + this.canvasX,
                'cy': regions[i].Bounds.Y - this.model.series[seriesIndex].yAxis.y + radius+ this.canvasY,
                'r': radius,
                'fill': color,
                'stroke-width': borderWidth,
                'opacity': opacity,
                'stroke': borderColor
            };
            ej.EjCanvasRender.prototype.drawCircle(bubbleOptions, ctx);
        }
    },

     // canvas circle draw
    canvasHighlightCircle: function (chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions, seriesType, ctx) {
        for (var i = 0; i < regions.length; i++) {
            var fill = (highlight.color != "" ? highlight.color : (highlight.mode == "series" && chart.model._visibleSeries.length > 1)?chart.model.pointColors[i]:chart.model.pointColors[pointIndex]);
            var options = {
                'opacity': opacity,
                'stroke': borderColor,
                'lineWidth': borderWidth,
                'color': color,
                'x': regions[i].StartX,
                'y': regions[i].StartY,
                'radius': chart.model._isPieOfPie ? chart.model.circularRadius[regions[i].PieSeriesIndex] : chart.model.circularRadius[regions[i].SeriesIndex],
                'innerRadius': chart.model.innerRadius[regions[i].SeriesIndex],
                'startAngle': regions[i].StartAngle,
                'endAngle': regions[i].EndAngle,
                'type': seriesType
            }
            this.highlightSegment(ctx, options);
        }
    },

     // canvas marker logic
    canvasHighlightMarker: function (chart, series, points, settings, ctx, evt,colors) {
        var highlight = settings;
        var patternName = highlight.pattern.toLowerCase();
        var seriesType = series.type.toLowerCase();
        var options = {};
        var fill = (highlight.color != "" ? highlight.color : chart.model.seriesColors[series.seriesIndex]);
        color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
        options.Style = {};
        options.Style.BorderColor = highlight.border.color;
        options.Style.Opacity = highlight.opacity;
        options.Style.BorderWidth = highlight.border.width;
        var symbolLocation, location, markerOptions, symbolName;
        for (var i = 0; i < points.length; i++) {
            symbolLocation = points[i].symbolLocation;
            symbolLocation = (seriesType.indexOf("line") != -1 || seriesType.indexOf("area") != -1) ? points[i].location : points[i].symbolLocation;
            if (symbolLocation) {
                location = this.model.AreaType == "cartesianaxes" ? { startX: symbolLocation.X + this.canvasX, startY: symbolLocation.Y + this.canvasY } : { startX: symbolLocation.X, startY: symbolLocation.Y };
                markerOptions = series.marker;
                symbolName;
                $.each(chart.model.symbolShape, function (name) {
                    if (markerOptions.shape.toLowerCase() == name.toLowerCase())
                        symbolName = name;
                });
                options.ShapeSize = markerOptions.size;
                options.r = Math.sqrt(markerOptions.size.height * markerOptions.size.height + markerOptions.size.width * markerOptions.size.width) / 2;
                options.cx = location.startX;
                options.cy = location.startY;
                options.Style.Color = colors?colors[i]:color;
                ej.EjSvgRender.chartSymbol['_draw' + symbolName](location, options, chart, ctx);
            }
        }

    },

     // canvas selction legend logic perform here
    canvasHighlightLegend: function (chart, series, settings, name, containerStyle, color) {
	var length =this.model._visibleSeries.length;
        var index =( chart.model.AreaType == 'none' && length == 1) ? series.pointIndex : series.seriesIndex,
        id = chart._id + '_' + name + '_Legend' + index + '_canvas';
        if ($('#' + id).length == 0 && index < series.points.length) {
            legendCtx = this.createCanvasElement(chart._id + '_' + name + '_Legend' + index + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            if (chart.model.AreaType != 'none' || (chart.model.AreaType == "none" && length > 1))
                legendRegion = chart.model.legendRegion[series.seriesIndex];
            else
                legendRegion = chart.model.legendRegion[series.pointIndex];
            var legendBounds = { startX: legendRegion.Bounds.LegendBound.X + legendRegion.Location.startX, startY: legendRegion.Bounds.LegendBound.Y + legendRegion.Location.startY },
            svgData = { svgRenderer: chart.svgRenderer, svgObject: chart.svgObject },
            symbolStyle = { 'SeriesIndex': index, 'Style': { 'Color': color, 'Opacity': settings.opacity, 'Visibility': true, 'BorderWidth': 0, BorderColor: color }, 'context': true, 'ShapeSize': legendRegion.Style.ShapeSize }
            if (legendRegion.SymbolShape.toLowerCase() == 'seriestype') {
                chart.legendItem = legendRegion.LegendItem;
                ej.EjSvgRender.chartSymbol["_draw" + legendRegion.SymbolShape](legendBounds, symbolStyle, chart);
            }else                      
              ej.EjSvgRender.chartSymbol["_draw" + legendRegion.SymbolShape](legendBounds, symbolStyle, svgData, legendCtx);
        }
    },

     // canvas create element 
    createCanvasElement: function (id, width, height, style) {
	    var chartCavasOffset, highLightCanvas;
        svgObj = document.createElement('canvas');
        _rootId = jQuery(this.element).attr("id");
        svgObj.setAttribute('id', id);
        svgObj.height = height;
        svgObj.width = width;
        svgObj.style["touch-action"] && (svgObj.style["touch-action"] = "none")
        svgObj.style["-ms-touch-action"] && (svgObj.style["-ms-touch-action"] = "none")
        this.svgRenderer.append(svgObj, this.element);
        var main = document.getElementById(id);
        main.style.left = style.left + $(document).scrollLeft() + 'px';
        main.style.top = style.top + $(document).scrollTop() + 'px';
		main.style.position = "absolute";
		chartCavasOffset = $("#"+this._id+"_canvas").offset();
		highLightCanvas = $(main).offset();
		if(chartCavasOffset.left < highLightCanvas.left) main.style.left = (style.left - Math.abs(chartCavasOffset.left - highLightCanvas.left) + $(document).scrollLeft()) + "px"; 
		if(chartCavasOffset.top < highLightCanvas.top) main.style.top = (style.top - Math.abs(chartCavasOffset.top - highLightCanvas.top) + $(document).scrollTop()) + "px";
        return svgObj.getContext("2d");
    },

     // canvas pie doughnut logic
    highlightSegment: function (ctx, options) {
        ctx.save();
        ctx.beginPath();
        if (options.type == 'pie' || options.type == "pieofpie") {
            ctx.moveTo(options.x, options.y);
            ctx.arc(options.x, options.y, options.radius, options.startAngle - 1.57, options.endAngle - 1.57, false);
        } else {
            ctx.arc(options.x, options.y, options.radius, options.startAngle - 1.57, options.endAngle - 1.57, false);
            ctx.arc(options.x, options.y, options.innerRadius, options.endAngle - 1.57, options.startAngle - 1.57, true);
        }
        ctx.fillStyle = options.color;
        ctx.globalAlpha = options.opacity;
        ctx.fill();
        ctx.lineWidth = options.lineWidth;
        ctx.strokeStyle = options.stroke;
        ctx.clip();
        ctx.stroke();
        ctx.closePath();
        ctx.restore();        
    },

    // Canvas pattern started here
    canvasPattern: function (ctx, chart, evt, fill, settings) {

        var hoverStyle = settings;
        var name = hoverStyle.name;
        var style = hoverStyle.pattern.toLowerCase();
        var color = fill;
        var opacity = hoverStyle.opacity;
        var backgroundColor = "#ffffff";
        var borderColor = hoverStyle.border.color;
        var borderWidth = hoverStyle.border.width;
        var patternStyle = document.createElement('canvas');
        var patternContext = patternStyle.getContext('2d');
        var pathOptions = [], translate = [];
        switch (style) {
            case "chessboard":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'fill': 'white', 'stroke': 'white', 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = { 'x': 0, 'y': 0, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                pathOptions[2] = { 'x': 5, 'y': 5, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                patternStyle.width = 10;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "pacman":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 18, 'height': 18, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 9.081 9.194 L 14.887 6.114 C 14.075000000000001 4.618 12.484 3.0620000000000007 10.596 3.0620000000000007 L 8.835 3.0620000000000007 C 6.138 3.063 3 6.151 3 8.723 L 3 10.402000000000001 C 3 12.974 6.138 16.063000000000002 8.835 16.063000000000002 L 10.596 16.063000000000002 C 12.681000000000001 16.063000000000002 14.431000000000001 14.303000000000003 15.131 12.549000000000003 L 9.081 9.194 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 18;
                patternStyle.height = 18;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "crosshatch":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 8, 'height': 8, 'fill': 'white', 'stroke': fill, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 0 0 L 8 8 Z',
                    'stroke-width': 1,
                    'stroke': color,
                    'name': 'path'
                };
                pathOptions[2] = {
                    'd': 'M 8 0 L 0 8 Z',
                    'stroke-width': 1,
                    'stroke': color,
                    'name': 'path'
                };
                patternStyle.width = 8;
                patternStyle.height = 8;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
             case "dots":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 7, 'height': 7, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'cx': 3,
                    'cy': 3,
                    'r': 2,
                    'stroke-width': 1,
                    'fill': color,
                    'name': 'circle'
                };
                patternStyle.width = 7;
                patternStyle.height = 7;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "diagonalforward":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'fill': 'white', 'stroke': fill, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 3 -3 L 9 3 M 6 6 L 0 0 M 3 9 L -3 3',
                    'stroke-width': 2,
                    'stroke': color,
                    'name': 'path'
                };
                patternStyle.width = 6;
                patternStyle.height = 6;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "diagonalbackward":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'stroke-width': 2, 'fill': 'white', 'stroke': fill, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 3 -3 L -3 3 M 0 6 L 6 0 M 9 3 L 3 9',
                    'stroke-width': 2,
                    'stroke': color,
                    'name': 'path'
                };
                patternStyle.width = 6;
                patternStyle.height = 6;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "grid":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M 1 3.5 L 11 3.5 M 0 3.5 L 11 3.5 M 0 7.5 L 11 7.5 M 0 11.5 L 11 11.5 M 5.5 0 L 5.5 12 M 11.5 0 L 11.5 12 Z',
                    'stroke-width': 1,
                    'stroke': color
                };
                patternStyle.width = 6;
                patternStyle.height = 6;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "turquoise":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 17, 'height': 17, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 2.697 0.5319999999999996 C 3.8695005238907747 0.5319999999999996 4.82 1.4771268931071944 4.82 2.643 C 4.82 3.808873106892805 3.8695005238907747 4.754 2.697 4.754 C 1.5244994761092252 4.754 0.5739999999999998 3.808873106892805 0.5739999999999998 2.643 C 0.5739999999999998 1.4771268931071944 1.5244994761092252 0.5319999999999996 2.697 0.5319999999999996 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M 13.928 0.5319999999999996 C 15.100500523890776 0.5319999999999996 16.051000000000002 1.4771268931071944 16.051000000000002 2.643 C 16.051000000000002 3.808873106892805 15.100500523890776 4.754 13.928 4.754 C 12.755499476109225 4.754 11.805 3.808873106892805 11.805 2.643 C 11.805 1.4771268931071944 12.755499476109225 0.5319999999999996 13.928 0.5319999999999996 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M 8.313 0.5319999999999996 C 9.485500523890776 0.5319999999999996 10.436 1.4771268931071944 10.436 2.643 C 10.436 3.808873106892805 9.485500523890776 4.754 8.313 4.754 C 7.140499476109226 4.754 6.19 3.808873106892805 6.19 2.643 C 6.19 1.4771268931071944 7.140499476109226 0.5319999999999996 8.313 0.5319999999999996 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M 13.928 6.16 C 15.100500523890776 6.16 16.051000000000002 7.105126893107196 16.051000000000002 8.271 C 16.051000000000002 9.436873106892806 15.100500523890776 10.382000000000001 13.928 10.382000000000001 C 12.755499476109225 10.382000000000001 11.805 9.436873106892806 11.805 8.271 C 11.805 7.105126893107196 12.755499476109225 6.16 13.928 6.16 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[5] = { 'name': 'path', 'd': 'M 8.313 6.16 C 9.485500523890776 6.16 10.436 7.105126893107196 10.436 8.271 C 10.436 9.436873106892806 9.485500523890776 10.382000000000001 8.313 10.382000000000001 C 7.140499476109226 10.382000000000001 6.19 9.436873106892806 6.19 8.271 C 6.19 7.105126893107196 7.140499476109226 6.16 8.313 6.16 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[6] = { 'name': 'path', 'd': 'M 13.928 11.787999999999998 C 15.100500523890776 11.787999999999998 16.051000000000002 12.733126893107194 16.051000000000002 13.899 C 16.051000000000002 15.064873106892804 15.100500523890776 16.009999999999998 13.928 16.009999999999998 C 12.755499476109225 16.009999999999998 11.805 15.064873106892804 11.805 13.899 C 11.805 12.733126893107194 12.755499476109225 11.787999999999998 13.928 11.787999999999998 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[7] = { 'name': 'path', 'd': 'M 8.313 11.787999999999998 C 9.485500523890776 11.787999999999998 10.436 12.733126893107194 10.436 13.899 C 10.436 15.064873106892804 9.485500523890776 16.009999999999998 8.313 16.009999999999998 C 7.140499476109226 16.009999999999998 6.19 15.064873106892804 6.19 13.899 C 6.19 12.733126893107194 7.140499476109226 11.787999999999998 8.313 11.787999999999998 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                patternStyle.width = 17;
                patternStyle.height = 17;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "star":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 21, 'height': 21, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M 15.913 18.59 L 10.762 12.842 L 5.613 18.75 L 8.291 11.422 L 0.325 9.91 L 8.154 8.33 L 5.337 0.91 L 10.488 6.658 L 15.637 0.75 L 12.959 8.078 L 20.925 9.59 L 13.096 11.17 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                patternStyle.width = 21;
                patternStyle.height = 21;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "triangle":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M 4.987 0 L 7.48 4.847 L 9.974 9.694 L 4.987 9.694 L 0 9.694 L 2.493 4.847 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                patternStyle.width = 10;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "circle":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 9, 'height': 9, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'circle',
                    'cx': 5.125,
                    'cy': 3.875,
                    'r': 3.625,
                    'stroke-width': 1,
                    'fill': color
                };
                patternStyle.width = 9;
                patternStyle.height = 9;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "tile":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 18, 'height': 18, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0 9 L 0 0 L 9 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M 9 9 L 9 0 L 18 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M 0 18 L 0 9 L 9 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M 9 18 L 9 9 L 18 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 18;
                patternStyle.height = 18;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "horizontaldash":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0 1.5 L 10 1.5 M 0 5.5 L 10 5.5 M 0 9.5 L 10 9.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 12;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "verticaldash":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 1.5 0 L 1.5 10 M 5.5 0 L 5.5 10 M 9.5 0 L 9.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 12;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "rectangle":
                pathOptions[0] = { 'x': 0, 'y': 0, 'name': 'rect', 'width': 12, 'height': 12, 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                pathOptions[2] = { 'name': 'rect', 'x': 7, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                patternStyle.width = 12;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "box":
                pathOptions[0] = { 'x': 0, 'y': 0, 'name': 'rect', 'width': 13, 'height': 13, 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1.5, 'y': 1.5, 'width': 10, 'height': 9, 'fill': color, 'opacity': opacity };
                patternStyle.width = 10;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "horizontalstripe":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 12, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0 0.5 L 10 0.5 M 0 4.5 L 10 4.5 M 0 8.5 L 10 8.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 10;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "verticalstripe":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 10, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0.5 0 L 0.5 10 M 4.5 0 L 4.5 10 M 8.5 0 L 8.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 12;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "bubble":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 20, 'height': 20, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'circle', 'cx': 5.217, 'cy': 11.325, 'r': 3.429, 'stroke-width': 1, 'fill': '#D0A6D1' };
                pathOptions[2] = { 'name': 'circle', 'cx': 13.328, 'cy': 6.24, 'r': 4.884, 'stroke-width': 1, 'fill': color };
                pathOptions[3] = { 'name': 'circle', 'cx': 13.277, 'cy': 14.66, 'r': 3.018, 'stroke-width': 1, 'fill': '#D0A6D1' };
                patternStyle.width = 20;
                patternStyle.height = 20;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
              
           
           
            case "custom":
                patternStyle = document.getElementById(hoverStyle.customPattern);
                break;
        }
        var pattern = ctx.createPattern(patternStyle, 'repeat');
        return pattern;
    },
     // Canvas pattern end
     // Canvas highlight and selection end
    
    //Get selected class for removing lengeds selection in multiselection mode
    foundClasses: function (id) {
        var pointClasses = [];
        $("[id*=" + id + "]").each(function () {
            var pointClass = $(this).attr('class') ? $(this).attr('class') : '';
            if (pointClass.indexOf('Selection') >= 0) {
                pointClasses.push(pointClass);
            }
        });
        return pointClasses;
    },
     // SVG selection started here
    selection: function (chart, evt, series, tracker, legendData, seriesIndex, pointIndex, parentNodeId, targetId, data) {
        var clusterPoints = this.model.clusterPoints,
        clusterPointslength = clusterPoints ? clusterPoints.length : 0,
        seriesCollection = this.model._visibleSeries,
        seriesLenth = seriesCollection.length,
        index = (this.model.AreaType == 'none' && seriesLenth == 1 ) ? pointIndex : seriesIndex,
        highlight = series.selectionSettings,
        isMultiSelection = this.model._visibleSeries[seriesIndex].selectionSettings.type.toLowerCase() == 'multiple' ? true : false,
        name = 'series' + seriesIndex,
        found_class = false,
        pathName = $('#' + targetId).attr('name'),
        patternName = highlight.pattern.toLowerCase(),
        mode = highlight.mode, isElement;
        if (parentNodeId)
             isElement = (parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 ||parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0) ? true : false;
              
        if (this.model.AreaType != 'none' ||( this.model.AreaType == 'none' && mode == "series" && seriesLenth > 1)) {
            $('style').each(function () {
                if ($(this).html().indexOf('.' + chart._id + 'SelectionStyle' + name) > -1) {
                    found_class = true;
                }
            });
        }    
        if (!chart.vmlRendering && this.model.enable3D) {
            if ((mode == 'series' || legendData) && (this.model.AreaType != 'none'|| (this.model.AreaType == 'none' && seriesLenth != 1))) {
                this.highlightSeries(chart, seriesIndex, pointIndex, 'Selection', name, highlight, evt, legendData,data)
                $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'Selection' + name + 'Style' + name + '1');
            }
            else if (mode == 'cluster' && chart.model.AreaType != 'none') 
                this.highlightCluster(chart, seriesIndex,pointIndex, 'Selection', name, highlight, evt, data)
            else {
                if (this.model.AreaType == 'none') {
                    this.highlightPoint(chart, seriesIndex, pointIndex, 'Selection', name + index, highlight, evt, data);
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'Selection' + name + index + 'Style' + name + index + '1');
                }else
                {
                    this.highlightPoint(chart, seriesIndex, pointIndex, 'Selection', name, highlight, evt,data)
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'Selection' + name + 'Style' + name + '1');
                }                                               
            }
        }
        else {
            if ((mode != 'cluster' || legendData) || chart.model.AreaType == 'none') {
                var colorStyle = {};
                colorStyle.name = "2D",
                colorStyle.index =this.model.AreaType=='none'? index : seriesIndex; colorStyle.api = "Selection",
                colorStyle.color = highlight.color != "" ? highlight.color : legendData ? (this.model.AreaType == 'none' ? this.model.pointColors[index] : this.model.seriesColors[index]) :
                    $('#' + targetId).attr('fill') != 'none' ? $('#' + targetId).attr('fill') : $('#' + targetId).attr('stroke');
                if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                    colorStyle.color = colorStyle.color[1].color;
                var fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? highlight.color : this.pattern(chart, evt, colorStyle, highlight),
                opacity = highlight.opacity,
                strokeColor = highlight.border.color,
                strokeWidth = highlight.border.width;
                if (!found_class) {
                    if (this.model.AreaType == 'none' &&(((mode == 'series' || legendData) && seriesLenth == 1)||( mode != 'series' && !legendData))) {
                        $('style').each(function () {
                            if ($(this).html().indexOf('.SelectionStyle' + name + 'Point' + index) > -1) {
                                found_class = true;
                            }
                        });
                        if (!found_class) {
                              this.createStyle(chart, chart._id + 'Selection', name + 'Point' + index, opacity, fill, strokeColor, strokeWidth);
                                this.createStyle(chart, chart._id + 'SelectionLegend', name + 'Point' + index, opacity, fill, 'transparent', 0);
                        }
                    } else if (this.model.AreaType == 'none' && seriesLenth > 1 ||this.model.AreaType != 'none') {
                            this.createStyle(chart, chart._id + 'Selection', name, opacity, fill, strokeColor, strokeWidth);
                            this.createStyle(chart, chart._id + 'SelectionLegend', name, opacity, fill, 'transparent', 0);
                    }
                    
                }
            }        
            if ((mode == 'series' || legendData) && ((chart.model.AreaType == 'none' && seriesLenth > 1)||(chart.model.AreaType != 'none'))) {
                if (!isElement) {
                    var otherElements = $('#' +parentNodeId).childNodes?$('#' +parentNodeId).childNodes:[];
                    for (var i = 0; i < otherElements.length; i++)
                     $('#' + otherElements[i].id).attr('class', chart._id + 'SelectionStyle');
                }
                $("[id*=" + this.svgObject.id + '_Series' + index + "]").each(function () {
                    if (this.parentNode.id == chart.svgObject.id + '_SeriesGroup_' + index || this.parentNode.id == chart.svgObject.id + '_symbolGroup_' + index) {
                         var className = $(this).attr("name") ? chart._id + 'SelectionPathStyle' + name : chart._id + 'SelectionStyle' + name;
                        $(this).attr('class', className);
                    }
                });

              //for legend selection sets here              
              $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', chart._id +'SelectionLegendStyle' + name);

                // removed tracker symbol
                if (tracker) $('#' + parentNodeId).remove();
                 
            }

            else if (mode == "cluster" && (chart.model.AreaType != 'none') && !ej.util.isNullOrUndefined(targetId)) {
                var matchString, clustername, indexVal, text, colorStyle, opacity, strokeColor, fill, strokeWidth;
                matchString = this.svgObject.id + "_Series";
                var clusterPoints = chart.model.clusterPoints;
                index = targetId.match(/(\d+)/g);
                pointIndex = parseInt(index[index.length - 1]);
                if (targetId.indexOf('SeriesText') >= 0) {
                    var datas = this.GetSeriesPoint(evt);
                    pointIndex = datas.region.Region.PointIndex;
                }
                var findClass, seriesSelectionIndex;
                for (k = 0; k < clusterPoints.length; k++) {
                    seriesSelectionIndex = clusterPoints[k].seriesIndex;
                    name = 'ClusterSeries' + k;
                    //creating individual styles for the points in different series
                    colorStyle = {};
                    colorStyle.index = k;
                    colorStyle.name = "2D"; colorStyle.api = "Selection" + name;
                    highlight = seriesCollection[seriesSelectionIndex].selectionSettings;
                    colorStyle.color = highlight.color != "" ? highlight.color : this.model.seriesColors[seriesSelectionIndex];
                    if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                        colorStyle.color = colorStyle.color[1].color;
                    opacity = highlight.opacity;
                    var strokeColor = highlight.border.color;
                    var strokeWidth = highlight.border.width;
                    var fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? colorStyle.color : this.pattern(chart, evt, colorStyle, highlight);                   
                    $('style').each(function () {
                        if ($(this).html().indexOf('.' + chart._id + 'SelectionStyle' + name) > -1) {
                            findClass = true;
                        }
                    });
                    if (!findClass) {
                        this.createStyle(chart, chart._id+'Selection', name, opacity, fill, strokeColor, strokeWidth);
                        this.createStyle(chart, chart._id +'SelectionLegend', name, opacity, fill, 'transparent', 0);
                    }                    
                    var className = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr("name") ? chart._id +'SelectionPathStyle' + name :chart._id + 'SelectionStyle' + name;
                    $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr('class', className);
                    $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex + '_symbol').attr('class', className);
                    if (chart.model._visibleSeries[seriesSelectionIndex].visibility != "hidden")
                        $('#' + this.svgObject.id + '_LegendItemShape' + k).not("[class*='Highlight']").attr('class', chart._id+'SelectionLegendStyle' + name);
					// checked condition for financial series
                    var finName = $('#' + targetId).attr("name");
                    if (finName != undefined && (finName == 'candle' || finName.indexOf('hilo') >= 0)) {
                        $('[id$=' + matchString + clusterPoints[k].seriesIndex + '_' + 'Point' + clusterPoints[k].pointIndex + ']').attr('class', 'HighlightStyle' + name);
                    }
                }
                if (tracker) {
                    $('#' + this.svgObject.id + '_Series' + k + '_Point' + pointIndex).attr('class', chart._id +'SelectionStyle' + name);
                    $('#' + this.svgObject.id + '_Series' + k + '_Point' + pointIndex + '_symbol').attr('class', chart._id+'SelectionStyle' + name);
                    $('#' + parentNodeId).remove();
                }
            }


            else {
                // checked condition for whether target id is area related or not
                if ((!$('#' + targetId).attr("name") && targetId != chart.svgObject.id + '_Series' + index) && chart.model.AreaType != 'none') {
                    $('#' + targetId).attr('class',chart._id + 'SelectionStyle' + name);
                    if (this.model.commonSeriesOptions.columnFacet == 'cylinder'|| series.columnFacet=='cylinder') {
                        for (i = 0; i < 2; i++) {
                            $('#' + targetId.replace(/2$/, i)).attr('class', chart._id + 'SelectionStyle' + name);
                        }
                    }
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', chart._id +'SelectionLegendStyle' + name);
                    $('#' + targetId + '_symbol').attr('class', chart._id +'SelectionStyle' + name);

                    if (tracker) {
                        $('#' + targetId + '_symbol').attr('class', chart._id +'SelectionStyle' + name);
                        $('#' + parentNodeId).remove();
                    }
                } else{             
                     $('#' + targetId).attr('class', chart._id +'SelectionStyle' + name + 'Point' + index);
                     $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class',chart._id + 'SelectionLegendStyle' + name + 'Point' + index);
                    if (tracker) $('#' + parentNodeId).remove();
                }

                // checked condition for financial series
                var finName = $('#' + targetId).attr("name");
                if (finName != undefined && (finName == 'candle' || finName.indexOf('hilo') >= 0)) {
                    $('[id$=' + targetId + ']').attr('class', chart._id +'SelectionStyle' + name);
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', chart._id +'SelectionLegendStyle' + name);
                }
            }
        }
    },

     //Selection Removing Logic
    removeSelection: function (seriesIndex, pointIndex, seriesCollection, mode, isMultiSelection, legendData, isSelected) {
       var seriesLength = seriesCollection.length,
       clusterPoints=this.model.clusterPoints,
       clusterPointslength = clusterPoints ? clusterPoints.length : 0;       
       //Removing selectiond mode series
       if ((mode == 'series' || legendData) && (this.model.AreaType != 'none' || (this.model.AreaType == 'none' && seriesLength > 1))) {
              for (i = 0; i < seriesLength; i++) {
                  selectionSettings = seriesCollection[i].selectionSettings;
                  if ((selectionSettings.mode != 'point' && selectionSettings.type.toLowerCase() == 'single') || (legendData && selectionSettings.type.toLowerCase() == 'single')) {
                      $(this.svgObject).find('[id*=' + this.svgObject.id + '_SeriesGroup_' + i + '],[id*=' + this.svgObject.id + '_symbolGroup_' + i + '],[id*=_Region_Series_' + i + '_]').each(function () {
                          $(this).find("*").attr('class', '');
                          $(this).attr('class', '');
                      });
                      $('#' + this._id + '_Selection_Legend' + i + '_canvas').remove();
                      $('#' + this._id + '_Selection_series' + i + '_canvas').remove();
                      $("[id*=" + '_Region_Series_' + i + "]").attr('class', '');
                      $('#' + this.svgObject.id + '_LegendItemShape' + i).attr('class', '');
                  } else if (isSelected && selectionSettings.type.toLowerCase() == 'multiple') {
                      $('#' + this._id + '_Selection_series' + seriesIndex + '_canvas').remove();
                      $('#' + this._id + '_Selection_Legend' + seriesIndex + '_canvas').remove();
                      $("[id*=" + '_Region_Series_' + seriesIndex + "]").attr('class', '');
                      $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + "]").attr('class', '');
                      $('#' + this.svgObject.id + '_LegendItemShape' + seriesIndex).attr('class', '');
                  }
              }
       }
       //Removing selection mode Cluster
       if (mode == 'cluster' && this.model.AreaType != 'none' && !legendData) {
           if (!isMultiSelection) {
               if (this.model.enableCanvasRendering) {
                   $('[id*=' + this._id + '_Selection_Cluster' + ']').remove();
                   $('[id*=' + this._id + '_SelectionCluster_Legend' + ']').remove();
                   this.selectedPoint = null;
               }else
                 for (var k = 0; k < clusterPointslength; k++) {
                     $("[id*=" + '_Region_Series_' + clusterPoints[k].seriesIndex + '_Point_' + "]").attr('class', '');
                     $("[id*=" + this.svgObject.id + '_Series' + clusterPoints[k].seriesIndex + '_Point' + "]").attr('class', '');
                     $('#' + this.svgObject.id + '_LegendItemShape' + clusterPoints[k].seriesIndex).attr('class', '');
                 }
           }
           else {
               if (!this.model.enableCanvasRendering) {
                   for (var k = 0; k < clusterPointslength; k++) {
                       $("[id*=" + "_Region_Series_" + clusterPoints[k].seriesIndex + "_Point_" + clusterPoints[k].pointIndex + "]").attr('class', '');
                       $("[id*=" + this.svgObject.id + '_Series' + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex + "]").attr('class', '');
                   }
                   pointClasses = this.model.enable3D ? this.foundClasses('_Region_Series_' + seriesIndex) : this.foundClasses(this.svgObject.id + '_Series' + seriesIndex);
                   (pointClasses.length == 0) ? $("[id*=" + this.svgObject.id + '_LegendItemShape' + "]").attr('class', '') : 0;
               } else {
                   $('[id*=' + this._id + '_Selection_Cluster' + '_point_' + pointIndex + ']').remove();
                   $('[id*=' + this._id + '_Selection_Cluster' + ']').length == 0 ? $('[id*=' + this._id + '_SelectionCluster_Legend' + ']').remove() : 0;
               }
          }
       }

      //Removing selection mode point
       if (mode == 'point' && this.model.AreaType != 'none' && !legendData) {
           if (!isMultiSelection) {
               $('[id*=' + this._id + '_Selection_series' + seriesIndex + '_point_' + ']').remove();
               $('[id*=' + this._id + '_Selection_Legend' + seriesIndex + ']').remove();
               $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + "]").attr('class', '');
               $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + '_Point' + "]").attr('class', '');
               $('#' + this.svgObject.id + '_LegendItemShape' + seriesIndex).attr('class', '');
           }
           else {               
               $('#' + this._id + '_Selection_series' + seriesIndex + '_point_' + pointIndex + '_canvas').remove();               
               $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + pointIndex + "]").attr('class', '');
               $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex + "]").attr('class', '');
               $("[id*=" + this._id + '_Selection_series' + seriesIndex + "]").length == 0 ? $('#' + this._id + '_Selection_Legend' + seriesIndex + '_canvas').remove() : 0;
               pointClasses = this.model.enable3D ? this.foundClasses('_Region_Series_' + seriesIndex) : this.foundClasses(this.svgObject.id + '_Series' + seriesIndex);
               (pointClasses.length == 0) ? $('#' + this.svgObject.id + '_LegendItemShape' + seriesIndex).attr('class', '') : 0;
           }
       }

       //Removing accoumulation series types selections
       if ((mode == 'point' || mode == 'cluster')||(mode == "series" && this.model._visibleSeries.length == 1) && this.model.AreaType == 'none'  && !legendData  ) {
           if (!isMultiSelection) {              
               $("[id*=" + this._id + '_Selection_series' + ']').remove();
               $("[id*=" + this._id + '_Selection_Legend' + ']').remove()
               $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + "]").attr('class', '');
               $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + '_Point' + "]").attr('class', '');
              (this.model._visibleSeries.length > 1)?$("[id*=" + this.svgObject.id + '_LegendItemShape' + seriesIndex + "]").attr('class', ''): $("[id*=" + this.svgObject.id + '_LegendItemShape' + "]").attr('class', '');
           } else {
               $('#' + this._id + '_Selection_series' + seriesIndex + '_point_' + pointIndex + '_canvas').remove();
               $("[id*=" + '_Region_Series_' + seriesIndex + '_Point_' + pointIndex + "]").attr('class', '');               
               $("[id*=" + this.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex + "]").attr('class', '');
               (this.model._visibleSeries.length > 1)? $('#' + this.svgObject.id + '_LegendItemShape' + seriesIndex).attr('class', ''): $('#' + this.svgObject.id + '_LegendItemShape' + pointIndex).attr('class', '');
               (this.model._visibleSeries.length > 1) ? $('#' + this._id + '_Selection_Legend' + seriesIndex + '_canvas').remove() : $('#' + this._id + '_Selection_Legend' + pointIndex + '_canvas').remove();
           }
       }        
    },
     // style append operation perform here
    createStyle: function (chart, api,name, opacity, fill, strokeColor, strokeWidth, strokeOpacity) {
        if (chart.vmlRendering) {
            var style = document.createElement('style');
            style.id = api + "Segment";
            opacity = opacity * 100;
            var text = "." +api+"Style" + name + " {filter: alpha(opacity=" + opacity + ");}."+api+"PathStyle" + name + "  {filter: alpha(opacity=" + opacity + ");}";
            style.setAttribute("type", "text/css");
            if (style.styleSheet) {   // for IE
                style.styleSheet.cssText = text;
            } else {                // others
                var textnode = document.createTextNode(text);
                style.appendChild(textnode);
            }
            var header = document.getElementsByTagName('head')[0];
            header.appendChild(style);
        } else {
            $("<style id="+api+"Segment" + name + " type='text/css'> ."+api+"Style" + name + "{ fill:" + fill + ";opacity:" + opacity + ";stroke:" + strokeColor + ";stroke-width:" + strokeWidth + ";stroke-opacity:" + strokeOpacity + "}</style>").appendTo("body");
            $("<style id=" + api + "Path" + name + " type='text/css'> ." + api + "PathStyle" + name + "{ opacity:" + opacity + ";stroke:" + strokeColor + ";stroke-width:" + strokeWidth + ";stroke-opacity:" + strokeOpacity + "}</style>").appendTo("body");
        }
    },

     // point highlight 3D perform here
    highlightPoint: function (chart, seriesIndex, pointIndex,api,name,highlight,evt ) {
        var pathId = "_Region_Series_" + seriesIndex + "_Point_" + pointIndex,
        gElement = $(this.svgObject).find('[id$=' + pathId + '],[id*=' + pathId + 'back],[id*=' + pathId + 'front]');
        for (var i = 0; i < gElement.length; i++) {
            var style3D = [];
            style3D.api = api+name; style3D.seriesName = name;
            style3D.name = $("[id=" + gElement[i].id + "]").attr('name');
            style3D.index = i;
            var color = this.highlightFill(highlight, chart, seriesIndex, pointIndex);
            var className = ($('#' + gElement[i].id).attr('class'));
            className = className ? className : '';
            if (className.indexOf('Selection') < 0 ) {
                $("[id=" + gElement[i].id + "]").attr('class', this.dStyle(style3D, color, chart, evt, highlight));
            }
        }
    },
       //cluster highlight of 3D is perfomed here
    highlightCluster: function (chart, seriesIndex, pointIndex, api, name, highlight, evt) {
        if (chart.model.AreaType != 'none') {
            var name, pathId, gElement, style3D, color, classname, clusterPoints = [];
            var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].x;
            for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                    if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                }
            }
            for (k = 0; k < clusterPoints.length; k++) {
                name = 'series' + k;
                pathId = "_Region_Series_" + clusterPoints[k].seriesIndex + "_Point_" + clusterPoints[k].pointIndex;
                 gElement = $(this.svgObject).find('[id$=' + pathId + '],[id*=' + pathId + 'back],[id*=' + pathId + 'front]');
                 for (var j = 0; j < gElement.length; j++) {
                        style3D = [];
                        style3D.api = api + name; style3D.seriesName = 'series' + k;
                        style3D.name = $("[id=" + gElement[j].id + "]").attr('name');
                        style3D.index = j;
                        seriesIndex = k;
                        highlight = (api == 'Highlight') ? chart.model._visibleSeries[k].highlightSettings : chart.model._visibleSeries[k].selectionSettings;
                        color = this.highlightFill(highlight, chart, seriesIndex, pointIndex);
                        className = ($('#' + gElement[j].id).attr('class'));
                        className = className ? className : '';
                        if ((className.indexOf('Selection' + name + 'Style') < 0) && (className.indexOf('SelectionStyle') < 0)) {
                            $("[id=" + gElement[j].id + "]").attr('class', this.dStyle(style3D, color, chart, evt, highlight));
                        }

                    }
                    if (api == 'Highlight')
                        $('#' + this.svgObject.id + '_LegendItemShape' + k).not("[class*='Selection']").attr('class', 'Highlight' + name + 'Style' + name + '1');
                    else
                        $('#' + this.svgObject.id + '_LegendItemShape' +k).attr('class', 'Selection' + name + 'Style' + name + '1');
                }

       }
    },  
     // point series 3D perform here
    highlightSeries: function (chart, seriesIndex, index, api, name, highlight, evt, legendData){      
        var seriesIndex = seriesIndex,
        pointIndex = index,
	   length =chart.model._visibleSeries.length,
        pathId = (chart.model.AreaType == 'none' && length == 1) ? "_Region_Series_" + seriesIndex + "_Point_" + index : "_Region_Series_" + seriesIndex + "_Point_",
        gElement = $(this.svgObject).find('[id*=' + pathId + ']'), style3D,color,
        className = legendData ? $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class') : evt ? ($('#' + evt.target.id).attr('class')) : '';
        className = className ? className : '';
        if (className.indexOf('SelectionStyle') < 0 || legendData) {
            for (var i = 0; i < gElement.length; i++) {
                style3D = [];
                style3D.api = api + name, style3D.seriesName = name,
                style3D.name = $("[id=" + gElement[i].id + "]").attr('name'),
                style3D.index = i;
                color = this.highlightFill(highlight, chart, seriesIndex, pointIndex, legendData);
				className = $('#' + gElement[i].id).attr('class');
                className = className ? className : '';
                if (className.indexOf('SelectionStyle') < 0&& (className.indexOf('Selection' + name + 'Style') < 0)) {
                    $("[id=" + gElement[i].id + "]").attr('class', this.dStyle(style3D, color, chart, evt, highlight));
                }
            }
        }
    },

     //SVG highlight logic started here
    highlight: function (chart, evt, series, legendData, data, parentNodeId, targetID, tracker) {
        var highlight = series.highlightSettings;
        var seriesIndex = $.inArray(series, chart.model._visibleSeries);
        var name = 'series' + seriesIndex;//series.name.replace(/ /g, ''); // removed space for series name
        var found_class = false;
        var index = seriesIndex;
        var targetId = targetID || evt.target.id;
        var matched = jQuery.uaMatch(navigator.userAgent);         
        if (!tracker && this.isTouch(evt) && matched.browser == 'chrome') {
            myLocation = evt.originalEvent.changedTouches[0];
            targetId = document.elementFromPoint(myLocation.clientX, myLocation.clientY).id;
        }
        parentNodeId = parentNodeId || evt.target.parentNode.id;
        var isElement = (parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0) ? true : false;
        var pathName = $('#' + targetId).attr('name');
        var patternName = highlight.pattern.toLowerCase();
		var length =chart.model._visibleSeries.length;
        if (chart.model.AreaType == 'none' && length == 1  )
        {
            if (!chart.model.enable3D) {
                var matchStr = legendData ? this.svgObject.id + '_LegendItemShape' : this.svgObject.id + "_Series" + seriesIndex + "_Point";
                var id = legendData ? this.svgObject.id + '_LegendItemShape' + legendData.legendItem.LegendItem.PointIndex : targetId;
                pointIndex = id.substr(matchStr.length);
                index = pointIndex;
            } else {
                index = legendData ? legendData.legendItem.LegendItem.PointIndex:data.region.Region.PointIndex;
            }
        }
        $('style').each(function () {
            if ($(this).html().indexOf('.HighlightStyle'+ name) > -1) {
                found_class = true;
            }
        });
        if (chart.model.enable3D && !chart.vmlRendering) {
            if ((highlight.mode == 'series' || legendData) && ((chart.model.AreaType == 'none' && length != 1) || chart.model.AreaType != 'none')) {
                this.highlightSeries(chart, seriesIndex, index, 'Highlight', name , highlight, evt, legendData)
                $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'Highlight' + name  + 'Style' + name  + '1');
            } else if (highlight.mode == 'cluster' && chart.model.AreaType != 'none') {
                this.highlightCluster(chart, seriesIndex, data.pointData.pointIndex, 'Highlight', name, highlight, evt);
                
            } else {
                pointIndex = data ? data.pointData.pointIndex: index ;
                this.highlightPoint(chart, seriesIndex, pointIndex, 'Highlight', name + pointIndex, highlight, evt)
                $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'Highlight' + name + pointIndex + 'Style' + name + pointIndex + '1');
            }
        } else {
            if (highlight.mode || this.model.AreaType=='none') {
                var colorStyle = {};
                colorStyle.name = "2D";
                colorStyle.index =seriesIndex; colorStyle.api = "Highlight"; colorStyle.color = highlight.color != "" ? highlight.color : legendData ? (this.model.AreaType == 'none' ? this.model.pointColors[index] : this.model.seriesColors[index]) : $('#' + targetId).attr('fill');
                if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                    colorStyle.color = colorStyle.color[1].color;
                var fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? highlight.color : this.pattern(chart, evt, colorStyle, highlight);
                var opacity = tracker ? 1 : highlight.opacity;
                var strokeColor = (tracker) ? ((series.type.toLowerCase().indexOf("hilo") == -1) ?  chart.model.highlightColor : fill) : highlight.border.color;
                var strokeWidth = tracker ? 3.5 : highlight.border.width;
                var strokeOpacity = (tracker && series.type.toLowerCase().indexOf("hilo")==-1)  ? 0.5 : 1;
                if (!found_class) {
                    this.createStyle(chart, 'Highlight', name, opacity, fill, strokeColor, strokeWidth, strokeOpacity);
                    this.createStyle(chart, 'HighlightLegend', name, opacity, fill, 'transparent', 0, strokeOpacity);
                }
            }

            // sets the style for the segment or path started here
            var className = legendData ? $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class') : $('#' + targetId).attr('class');
            className = className ? className : '';
            if (className.indexOf("Selection")==-1) {
                // checked mode of the highlight series
                if (((highlight.mode == 'series' && !tracker) || legendData) && ((chart.model.AreaType == 'none' && length != 1) || chart.model.AreaType != 'none')) {
                        $("[id*=" + this.svgObject.id + '_Series' + index + "]").each(function () {
                            if (this.parentNode.id == chart.svgObject.id + '_SeriesGroup_' + index || this.parentNode.id == chart.svgObject.id + '_symbolGroup_' + index) {
                                var selectClass = $(this).attr('class');
                                var className = $(this).attr("name") ? 'HighlightPathStyle' + name : 'HighlightStyle' + name;
                                // this condition is checked for whether already selected and it's symbol
                                if (selectClass != 'SelectionStyle' + name && selectClass != 'SelectionStyleClusterSeries' + seriesIndex && selectClass != 'SelectionPathStyle' + name && parentNodeId.indexOf(chart.svgObject.id + '_symbolGroup_' + index) <= 0) {
                                    $(this).attr('class', className);
                                }
                            }
                        });                    
                    //for legend highlight sets here
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
                }
                else if (highlight.mode == "cluster" && chart.model.AreaType != 'none' && !tracker) {
                    var matchString = this.svgObject.id + "_Series";
                    var clusterPoints = [],seriesHighlightIndex;
                    index = targetId.match(/(\d+)/g);
                    if (index && targetId.toLowerCase().indexOf("point") > 0) {
                        pointIndex = parseInt(index[index.length - 1]);
                        var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].xValue;
                        for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                            for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                                if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].xValue)
                                    clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                            }
                        }
                    }
                    for (k = 0; k < clusterPoints.length; k++) {
                        seriesHighlightIndex = clusterPoints[k].seriesIndex;
                        name = 'series' + seriesHighlightIndex;
                        //creating individual styles for the points in different series
                        colorStyle = {};
                        colorStyle.name = "2D"; colorStyle.api = "Highlight" + name;
                        highlight = chart.model._visibleSeries[seriesHighlightIndex].highlightSettings;
                        colorStyle.color = highlight.color != "" ? highlight.color : this.model.seriesColors[seriesHighlightIndex];
                        if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                            colorStyle.color = colorStyle.color[1].color;
                        opacity = highlight.opacity;
                        strokeColor = highlight.border.color;
                        strokeWidth = highlight.border.width;
                        fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? colorStyle.color : this.pattern(chart, evt, colorStyle, highlight);
                        if (!found_class) {
                            this.createStyle(chart, 'Highlight', name, opacity, fill, strokeColor, strokeWidth);
                            this.createStyle(chart, 'HighlightLegend', name, opacity, fill, 'transparent', 0);
                        }
                        
                        //applying the created style to the selected element 
                        var selectClass = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr('class');
                        var symbol = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex + '_symbol').attr('class');
                        var className = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr("name") ? 'HighlightPathStyle' + name : 'HighlightStyle' + name;
                        // this condition is checked for whether already selected and it's symbol
                        if (selectClass != 'SelectionStyle' + 'series' + k && selectClass != 'SelectionPathStyle' + name && parentNodeId.indexOf(chart.svgObject.id + '_symbolGroup_' + k) <= 0) {
                            $('#' + matchString + clusterPoints[k].seriesIndex + '_' + 'Point' + clusterPoints[k].pointIndex).attr('class', className);
                            $('#' + matchString + clusterPoints[k].seriesIndex + '_' + 'Point' + clusterPoints[k].pointIndex + '_symbol').attr('class', className);
                        }
                        if (chart.model._visibleSeries[seriesHighlightIndex].visibility != "hidden")
                            $('#' + this.svgObject.id + '_LegendItemShape' + seriesHighlightIndex).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
                        // checked condition for financial series
                        var finName = $('#' + targetId).attr("name");
                        if (finName != undefined && (finName == 'candle' || finName.indexOf('hilo') >= 0)) {
                            $('[id$=' + matchString + clusterPoints[k].seriesIndex + '_' + 'Point' + clusterPoints[k].pointIndex + ']').attr('class', 'HighlightStyle' + name);
                        }
                    }

                }

				else {
					// checked condition for whether target id is area related or not
					if (!$('#' + targetId).attr("name") && targetId != chart.svgObject.id + '_Series' + index) {
						selectClass = $('#' + targetId).attr('class') ? $('#' + targetId).attr('class') : '';
						if ((selectClass.indexOf('Selection') != 0 && parentNodeId.indexOf(chart.svgObject.id + '_symbolGroup_' + index) <= 0 && (data||legendData))) {
						    if (isElement) {
						        $('#' + targetId).attr('class', 'HighlightStyle' + name);
						        if (this.model.commonSeriesOptions.columnFacet == 'cylinder' || series.columnFacet == 'cylinder') {
						            for (i = 0; i < 2; i++) {
						                $('#' + targetId.replace(/2$/, i)).attr('class', 'HighlightStyle' + name);
						            }
						        }
						    }
                            pointIndex = data ? data.region.Region.PointIndex:legendData.legendItem.LegendItem.PointIndex;
                            $('#' + targetId + '_symbol').attr('class', 'HighlightStyle' + name);
                            $('#' + this.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex).not("[class*='Selection']").attr('class', 'HighlightStyle' + name);
                            $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
                        }
						if(targetId.indexOf("symbol") > -1)
						    $('#' + targetId).attr('class', 'HighlightStyle' + name);
                    }
					// checked condition for financial series
                    var finName = $('#' + targetId).attr("name");
                    if (finName != undefined && (finName == 'candle' || finName.indexOf('hilo') >= 0)) {
                        $('[id$=' + targetId + ']').attr('class', 'HighlightStyle' + name);
						$('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
					}

				}
			}
        }

    },

     // 3D style class created here
    dStyle: function (style3D, color, chart, evt, settings) {
        var vector = ej.Ej3DRender.prototype.polygon3D.prototype;
        var highlight = settings;
        var name = style3D.api;
        var seriesName = style3D.seriesName;
        var opacity = highlight.opacity;
        color = chart.colorNameToHex(color);
        if (style3D.name == "XLight")
            color = vector.applyXLight(color, chart);
        else if (style3D.name == "ZLight")
            color = vector.applyZLight(color, chart);
        else
            color = color;
        var colorStyle = {};
        colorStyle.name = style3D.name; colorStyle.color = color; colorStyle.api = name;
        color = (highlight.pattern == "none") ? color : this.pattern(chart, evt, colorStyle, highlight);
        if (document.getElementById(name + "Segment" + seriesName)) {
            var style = document.getElementById(name + "Segment" + seriesName);
            style.innerHTML = style.innerHTML + '.' + name + 'Style' + seriesName+ style3D.index + ' { fill:' + color + ";opacity:" + opacity + ' }';
        } else {
            var style = document.createElement('style');
            style.id = name + "Segment" + seriesName;
            style.type = 'text/css';
            style.innerHTML = '.' + name + 'Style' + seriesName + style3D.index + ' { fill:' + color + ";opacity:" + opacity + ' }';
            document.getElementsByTagName('head')[0].appendChild(style);
        }
        return name + "Style" + seriesName + style3D.index;

    },

     // 3D style class end

     // pattern of the SVG started here
    pattern: function (chart, evt, colorStyle, settings) {
        var hoverStyle = settings;
        var name = colorStyle.api;
        var style = hoverStyle.pattern.toLowerCase();
        var color = colorStyle.color;
        var opacity = hoverStyle.opacity != "" ? hoverStyle.opacity : $('#' + evt.target.id).attr('opacity');
        var backgroundColor = "#ffffff";
        if (document.getElementById(name + "Def")) {
            var svg = $("#" + name + "Def")[0];
        } else {
            var svgid = { 'id': name + "Def" }
            var svg = chart.svgRenderer.createPattern(svgid, 'svg');
            chart.svgRenderer.append(svg, chart.svgObject);
        }
        if (chart.model.enable3D) {
            if (colorStyle.name == "XLight")
                backgroundColor = "#595959";
            else if (colorStyle.name == "ZLight")
                backgroundColor = "#737373";
            else
                backgroundColor = "#808080";
        }        
        if (("#" + style + '_' + name + '_' + colorStyle.name + '_' + colorStyle.index))
            $("#" + style + '_' + name + '_' + colorStyle.name + '_' + colorStyle.index).remove();

        var pathOptions = [];
        switch (style) {
            case "chessboard":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'x': 0, 'y': 0, 'width': 10, 'height': 10 }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = { 'x': 0, 'y': 0, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                pathOptions[2] = { 'x': 5, 'y': 5, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "pacman":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '17.917', 'height': '18.384' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 17.917, 'height': 18.384, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M9.081,9.194l5.806-3.08c-0.812-1.496-2.403-3.052-4.291-3.052H8.835C6.138,3.063,3,6.151,3,8.723v1.679   c0,2.572,3.138,5.661,5.835,5.661h1.761c2.085,0,3.835-1.76,4.535-3.514L9.081,9.194z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "crosshatch":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '8', 'height': '8' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 8, 'height': 8, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M0 0L8 8ZM8 0L0 8Z',
                    'stroke-width': 1,
                    'stroke': color,
                    'name': 'path'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "dots":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 7, 'height': 7, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'cx': 3,
                    'cy': 3,
                    'r': 2,
                    'stroke-width': 1,
                    'fill': color,
                    'name': 'circle'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "diagonalforward":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 3 -3 L 9 3 M 6 6 L 0 0 M 3 9 L -3 3',
                    'stroke-width': 2,
                    'stroke': color,
                    'name': 'path'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "diagonalbackward":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name' : 'rect' };
                pathOptions[1]= {
                    'd': 'M 3 -3 L -3 3 M 0 6 L 6 0 M 9 3 L 3 9',
                    'stroke-width': 2,
                    'stroke': color,
                    'name':'path'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "grid":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M1 3.5L11 3.5 M0 3.5L11 3.5 M0 7.5L11 7.5 M0 11.5L11 11.5 M5.5 0L5.5 12 M11.5 0L11.5 12Z',
                    'stroke-width': 1,
                    'stroke': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "turquoise":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '17', 'height': '17' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 17, 'height': 17, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0.5739999999999998,2.643a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M11.805,2.643a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M6.19,2.643a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M11.805,8.217a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[5] = { 'name': 'path', 'd': 'M6.19,8.217a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[6] = { 'name': 'path', 'd': 'M11.805,13.899a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[7] = { 'name': 'path', 'd': 'M6.19,13.899a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "star":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '21', 'height': '21' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 21, 'height': 21, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M15.913,18.59L10.762 12.842 5.613 18.75 8.291 11.422 0.325 9.91 8.154 8.33 5.337 0.91 10.488 6.658 15.637 0.75 12.959 8.078 20.925 9.59 13.096 11.17 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "triangle":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '10', 'height': '10' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M4.987,0L7.48 4.847 9.974 9.694 4.987 9.694 0 9.694 2.493 4.847 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "circle":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '9', 'height': '9' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 9, 'height': 9, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'circle',
                    'cx': 5.125,
                    'cy': 3.875,
                    'r': 3.625,
                    'stroke-width': 1,
                    'fill': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "tile":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '18', 'height': '18' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 18, 'height': 18, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0,9L0 0 9 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M9,9L9 0 18 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M0,18L0 9 9 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M9,18L9 9 18 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "horizontaldash":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '12', 'height': '12' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0,1.5 L10 1.5 M0,5.5 L10 5.5 M0,9.5 L10 9.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "verticaldash":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '12', 'height': '12' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M1.5,0 L1.5 10 M5.5,0 L5.5 10 M9.5,0 L9.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "rectangle":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'x': 0, 'y': 0, 'width': 12, 'height': 12 }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'width': 12, 'height': 12, 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                pathOptions[2] = { 'name': 'rect', 'x': 7, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "box":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'x': 0, 'y': 0, 'width': 10, 'height': 10 }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'width': 13, 'height': 13, 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1.5, 'y': 1.5, 'width': 10, 'height': 9, 'fill': color, 'opacity': opacity };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "horizontalstripe":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '10', 'height': '12' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 12, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0,0.5 L10 0.5 M0,4.5 L10 4.5 M0,8.5 L10 8.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "verticalstripe":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '12', 'height': '10' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 10, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0.5,0 L0.5 10 M4.5,0 L4.5 10 M8.5,0 L8.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "bubble":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '20', 'height': '20' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = {'name':'rect', 'x': 0, 'y': 0, 'width': 20, 'height': 20, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'circle', 'cx': 5.217, 'cy': 11.325, 'r': 3.429, 'stroke-width': 1, 'fill': '#D0A6D1' };
                pathOptions[2] = { 'name': 'circle', 'cx': 13.328, 'cy': 6.24, 'r': 4.884, 'stroke-width': 1, 'fill': color };
                pathOptions[3] = { 'name': 'circle', 'cx': 13.277, 'cy': 14.66, 'r': 3.018, 'stroke-width': 1, 'fill': '#D0A6D1' };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
             case "custom":
                return "url(#" + hoverStyle.customPattern + ")"
                break;
        }

    },
	loadPattern: function (chart, options, pattern, ctx) {
        var path;
        for (var i = 0; i < options.length; i++) {
            if (!ctx) {
                path = chart.svgRenderer.createPattern(options[i], options[i].name);
                pattern.appendChild(path);
            } else {
                if (options[i].name == 'rect'){
					options[i].stroke = 'transparent';
					this.drawRect(options[i], ctx);
				}
                else if (options[i].name == 'path')
                    ej.EjCanvasRender.prototype.drawPath(options[i], ctx);
                else if(options[i].name == 'circle'){
					options[i].stroke = 'transparent';
					ej.EjCanvasRender.prototype.drawCircle(options[i], ctx);
				}
            }
        }
    },
	drawRect: function (options, element) {
        element.save();
        element.beginPath();
        element.globalAlpha = options.opacity;
        element.lineWidth = options["stroke-width"];
        element.strokeStyle = options.stroke;
	    element.rect(options.x, options.y, options.width, options.height);
        if (options.fill == "none") options.fill = "transparent";
        element.fillStyle = options.fill;
        if (options.transform) element.translate(options.transform[0], options.transform[1]);
        element.rotate(options.rotate * Math.PI / 180);        
        element.fillRect(options.x, options.y, options.width, options.height);
        element.clip();
        element.stroke();
        element.restore();
    },
     // pattern of the SVG end
     // SVG highlight and selection end

	chartRightClick: function (evt) {   
	    this.chartRightClick = true;
	    if (this.isTouch(evt) && this.model.crosshair.visible) {
	        event.preventDefault();	        
	        event.stopPropagation();
	        return false;
	    }
	},

	chartMouseUpTouch: function (evt) {
	    var chart = this,
	    matched = jQuery.uaMatch(navigator.userAgent),
	    mouseMoveCords = this.calMousePosition(evt),
        browserInfo = this.model.browserInfo,
	    element;
	    this.mousemoveX = mouseMoveCords.X;
	    this.mousemoveY = mouseMoveCords.Y;	     
	    this.enableTrackBall();	  
        
	    clearTimeout(this.model.doubleTapTimer);
	   
	    if (!this.drag)
	        this.model.doubleTapTimer = setTimeout(function () {	            
	        if ((chart.model.tapNum < 2 || chart.model.crosshair.visible))
	             chart.singleTap(evt);
	        chart.model.tapNum = 0;
	        chart.model.cachedX = 0;
	        chart.model.cachedY = 0;
	        
	    }, 200);	     

	    if (chart.model.tapNum == 2)
	     chart.model.doubleTapped = false;  
	        
	    return true;	 
	},
	getEvent : function(event) {
	    return event.targetTouches && event.targetTouches[0]  ? event.targetTouches[0] : event
	},

   // Method to check the double tap in chart.
	doubleTap: function (e) {

	    var pointer = this.getEvent(e),
	       currX = pointer.pageX,	   
	       currY = pointer.pageY,
           model = this.model,
           padding = 200,
           element,
           bounds = model.m_AreaBounds;	 

	    this.model.cachedX = this.model.cachedX || currX;

	    this.model.cachedY = this.model.cachedY || currY;

	    timestamp = new Date().getTime();  
	   
	    
	    if (this.mouseDownX >= bounds.X && this.mouseDownX < (bounds.X + bounds.Width) && this.mouseDownY < (bounds.Y + bounds.Height) && this.mouseDownY >= bounds.Y && Math.abs(currX - this.model.cachedX) < padding && Math.abs(currY - this.model.cachedY) < padding)
	        this.model.tapNum++;


	    if (this.model.tapNum == 2) {
	        this.model.doubleTapped = true;
	        this.multiSelectMouseDown(e);
	        this.drag = this.model.zooming.enable || this._isMultiSelect;	     
	        element = this.model.trackerElement;
	        if (element) {
	            $(element).finish();
	            element = null;
	        }
	    }  
	 
	},
     // Method invoking userinteraction in single Tap.
	singleTap: function (evt) {	    
	    var chart = this,
            targetid = evt.target.id,
	        seriesCollection =this.model._visibleSeries,
            element,
            pointer = this.getEvent(evt),
         currX = pointer.pageX,
         currY = pointer.pageY,
         swipeThreshold = 10;
	   
	    if ((!this.panning && !this.isZoomButtonHovered(evt.target))) {
	        this.cancelEvent(evt);
	        if (chart.model.trackerElement) {
                if($.finish)
                    $(chart.model.trackerElement).finish();
                else
                    $(chart.model.trackerElement).stop(true, true);
	            chart.model.trackerElement = null;
	        }
	        $(".ejTooltip"+ this._id).remove();
	        //if ($(element).find("#" + chart.svgObject.id + "_CrosshairGroup").length==0) {
	        if ((!chart.model.crosshair.visible || chart.model.AreaType != "cartesianaxes") && Math.abs(currX - chart.model.cachedX) < swipeThreshold && Math.abs(currY - chart.model.cachedY) < swipeThreshold) {
	            this.chartInteractiveBehavior(evt);
	        }

	        else {

	            chart.model.element = $(document).find("#" + this.svgObject.id + "_CrosshairGroup");
	            chart.model.trackerElement = $(chart.model.element).children().not('.Tracker').not("defs");
	            if ($(chart.model.trackerElement).length > 0)
	                $(chart.model.trackerElement).fadeOut(1000, function () {
	                    $(chart.model.element).find('[id*="trackSymbol"]').attr("visibility", "hidden");
	                    chart.model.trackerElement = null;
	                    chart._removeHighlight();
	                });
	            var groupElement = $(document).find('[id*="_trackball_grouping_tooltip"]');
	            if ($(groupElement).length > 0)
	                $(groupElement).fadeOut(1000, function () {
	                    $(groupElement).remove();
	                });
	        }

	        this.axisTooltip(evt, targetid);

	        this.showTitleTooltip(evt, targetid);

	        if (this.model.enableCanvasRendering) {
	            this._textTooltip(evt, this.model.xAxisLabelRegions);
	            this._textTooltip(evt, this.model.yAxisLabelRegions);
	        }

	        for (var i = 0, seriesLength = this.model.series.length; i < seriesLength; i++) {
	            var series = this.model.series[i], dataLabel = series.marker.dataLabel;
	            if (series._enableSmartLabels && !this.model.enable3D && !dataLabel.enableWrap) {
	                var template = dataLabel.template;
	                var font = dataLabel.font;
	                if (this.model.AreaType == "none" && ej.util.isNullOrUndefined(template)) {
	                    this.datalabelTooltip(evt, i, font);
	                }
	            }
	        }

	        if ($(".ejTooltip" + this._id).length > 0)
	            element = $(".ejTooltip" + this._id);
	        if ($(".tooltipDiv" + this._id).length > 0)
	            element = $(".tooltipDiv" + this._id);	        
	            window.clearTimeout(chart.model.timer);
	            chart.model.trackerElement = element;	           
	            chart.model.timer = setTimeout(function () {
	                pointData = chart.model.prevPoint;
	                if (chart.model.trackerElement) {
	                    chart.model.trackerElement.fadeOut(500, function () {
	                        if (pointData) {
	                            if (chart.model.AreaType != "polaraxes") {
	                                var prevLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
	                                chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, prevLocation);
	                            }
	                            $("#" + "canvas" + "_trackSymbol").remove();
	                        }
	                        $("[id*=" + "_TrackSymbol" + "]").remove();
	                    });
	                }
	                else {
	                    if (pointData) {
	                        if (chart.model.AreaType != "polaraxes") {
	                            var prevLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
	                            chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, prevLocation);
	                        }	                      
	                    }
	                    if (!chart.model.crosshair.visible) {
	                        $("#" + "canvas" + "_trackSymbol").remove();
	                        $("[id*=" + "_TrackSymbol" + "]").remove();
	                    }
	                }

	            }, 1200);
	          
	       
	    }
	    if(!evt.originalEvent.pointerType && !this.panning && this.model.selectionEnable)			
	        this._doClick(evt);
	},
	_getPieOfPiePoint:function(index,series,legenddata)
	{	   
	    var visiblePoints = legenddata? series.points : series._visiblePoints;
	    var length = visiblePoints.length;
	    for (var j = 0; j < length; j++)
	    {
	        if (index == visiblePoints[j].actualIndex)
	            var seriesPoint = visiblePoints[j];
	    }
	    return seriesPoint;
	},
    tooltip: function (chart, evt){
        var tooltipShowing = false;
        var chartSeries;
        var seriesIndex;
        var data = this.GetSeriesPoint(evt);
        $("#" + "canvas" + "_trackSymbol").remove();
		if (ej.util.isNullOrUndefined(data) && !this.dragPoint) // To hide the tooltip when it is moved on chartArea other than series
		    $("#" + chart.svgObject.id + "_TrackToolTip").remove();
		
		if (!this.model.crosshair.visible && this.model.AreaType == "cartesianaxes") {
		    var indicators = this.model.indicators,
		    seriesCollection = this.model._visibleSeries,
		    currentIndicator, l, k, trendLines,
		    length = seriesCollection.length;
			var trackcount=0;	
		    for (k = 0; k < length; k++) {
		        trendLines = seriesCollection[k].trendlines;
		        for (l = 0; l < trendLines.length; l++) {
		            if (trendLines[l].visibility.toLowerCase() == "visible" && !trendLines[l].isNull && this.model.series[0].type != "boxandwhisker") {
						trendLines[l]._visiblePoints = trendLines[l].points;
		                seriesCollection = seriesCollection.concat(trendLines[l]);
		            }
		        }
		    }

		    for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
			    currentIndicator = indicators[j]; 
			    if (currentIndicator.visible)
			        seriesCollection = seriesCollection.concat(currentIndicator.segment);
			    }
		    var seriesCollectionLength = seriesCollection.length;
		    for (var i = 0; i < seriesCollectionLength ; i++) {
		        chartSeries = seriesCollection[i];
                seriesIndex = i;
				
				var type = chartSeries.type.toLowerCase();		
				if (chartSeries.visibility.toLowerCase() == 'visible' && type !== "scatter" && type !== "bubble" && type !== "column" && (type.indexOf("bar")==-1) && type !== "stackingcolumn" && type !== "stackingcolumn100" && !chartSeries._hiloTypes) {
                    $("#" + chart.svgObject.id + "_TrackToolTipTemplate_" + seriesIndex).remove();
                    if (!($(".tooltipDiv" + this._id).hasClass(chart.svgObject.id)))
                        $(".tooltipDiv" + this._id).remove();
                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackSymbol" + '_' + seriesIndex).remove();
                    $('#' + chart.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
                    var serY = [];
                    var serX = [];
                    location = null;
                    var closestXyPoint = chart.getClosesPointXY(serX, serY, chartSeries, chart.mousemoveX, chart.mousemoveY, evt);
					if(ej.util.isNullOrUndefined(closestXyPoint.point) && (!this.model.prevPoint|| (this.model.prevPoint.seriesIndex && !this.model.series[this.model.prevPoint.seriesIndex].tooltip.visible)) && !this.dragPoint && ($(".tooltipDiv" + this._id).hasClass(this.svgObject.id))){
						clearTimeout(this.removeTooltip); 
						chart.removeTooltip = setTimeout(function () {   // To hide the tooltip when it is moved on chartArea other than series
							$(".tooltipDiv" + chart._id).remove();
						}, 1000);
					}
                    if (closestXyPoint.point) {
						if(chartSeries.type.toLowerCase() == "boxandwhisker")
						{							
							trackcount++;
							location = closestXyPoint.outlierPosition;							
						}else
							location = ej.EjSvgRender.utils._getPoint(closestXyPoint.point, chartSeries);
                        var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                        commonPointEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveX }, region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
                        chart._trigger("pointRegionMouseMove", commonPointEventArgs);
                    }
                    var pointData = this.model.prevPoint;
                    if (pointData && pointData.seriesIndex == i && !closestXyPoint.point) {
						if(chartSeries.type.toLowerCase() == "boxandwhisker")
						{
							  trackcount++;
							  pointLocation = pointData.outlierPosition;
							
						}
						else
                              var pointLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                        if (!chart.model.enableCanvasRendering) // No need to draw again in canvas chart after explode marker
                            chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, pointLocation);
                        this.model.prevPoint = null;
                    }					
                    if (!chartSeries.isIndicator && !chartSeries.isTrendLine && !chartSeries.selectionSettings.enable && !chartSeries.highlightSettings.enable && (chartSeries.marker.visible|| chartSeries.type.toLowerCase() == "boxandwhisker" ) && closestXyPoint.point && !commonPointEventArgs.cancel) {
                        if (ej.util.isNullOrUndefined(closestXyPoint.point.marker) || ej.util.isNullOrUndefined(closestXyPoint.point.marker.visible) || (closestXyPoint.point.marker && closestXyPoint.point.marker.visible))
							chart.drawTrackerSymbol(chartSeries, seriesIndex, closestXyPoint.index, false, location);						
						else if(chartSeries.type == "boxandwhisker" && !closestXyPoint.point.marker)	
							chart.drawTrackerSymbol(chartSeries, seriesIndex, closestXyPoint.index, false, location);						
                        if (pointData && closestXyPoint.point != pointData.point){
							if(chartSeries.type.toLowerCase() == "boxandwhisker")
							{
								trackcount++;
								prevLocation = pointData.outlierPosition;							
							}
							else						
								var prevLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                            chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, prevLocation, id,trackcount);
						}
                        if (ej.util.isNullOrUndefined(closestXyPoint.point.marker) || ej.util.isNullOrUndefined(closestXyPoint.point.marker.visible) || (closestXyPoint.point.marker && closestXyPoint.point.marker.visible))
                        this.model.prevPoint = { point: closestXyPoint.point, pointIndex: closestXyPoint.index, series: chartSeries, seriesIndex: seriesIndex,outlierPosition:closestXyPoint.outlierPosition };
                    }


                    if (chartSeries.tooltip.visible && closestXyPoint.point && (!data || (i > data.region.SeriesIndex && !(chartSeries._zOrder < this.model._visibleSeries[data.region.SeriesIndex]._zOrder)))) {
                        trans = ej.EjSvgRender.utils._getTransform(chartSeries.xAxis, chartSeries.yAxis, chart.model.requireInvertedAxes);
                        if ((location.X + trans.x) <= (trans.x + trans.width) && ((location.X + trans.x) >= trans.x || location.X == 0) && (Math.abs(location.Y - (trans.y + trans.height)) <= (trans.y + trans.height))
                                     && (Math.abs(location.Y - (trans.y + trans.height)) >= trans.y || location.Y == 0)) {                         
                            if (!commonPointEventArgs.cancel) {
                                if ($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").length == 0) {
                                    var transOptions = { 'id': chart.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                    chart.gTransToolEle = chart.svgRenderer.createGroup(transOptions);
                                }
                                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')', 'visibility': 'visible' });
                                if (ej.util.isNullOrUndefined(chartSeries.tooltip.template)) {
                                    tooltipShowing = true;
                                    $(".tooltipDiv" + this._id).remove();
                                    chart.displayShowTooltip(location, closestXyPoint.point, chartSeries, closestXyPoint.index);
                                }
                                else {
                                    tooltipShowing = true;
                                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                                    var region = { SeriesIndex: seriesIndex, Region: { PointIndex: closestXyPoint.index } };
                                    chart.createTooltip(region, evt, chartSeries);
                                }

                                //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                                // var detachEle = $(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").detach();
                                //detachEle.appendTo(chart.svgObject);
                            }
                           
                        }
                    }
                } 
				else if (ej.util.isNullOrUndefined(data) && !this.dragPoint && ($(".tooltipDiv" + this._id).hasClass(this.svgObject.id))){ 
						clearTimeout(this.removeTooltip); 
						chart.removeTooltip = setTimeout(function () {   // To hide the tooltip when it is moved on chartArea other than series
							$(".tooltipDiv" + chart._id).remove();
						}, 1000);
				}
            }
        }
	   else {
            $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").hide();
			if (ej.util.isNullOrUndefined(data)){
				clearTimeout(this.removeTooltip); 
				chart.removeTooltip = setTimeout(function () {   // To hide the tooltip when it is moved on chartArea other than series
					$(".tooltipDiv" + chart._id).remove();
				}, 1000);
			}
        }
       
		if (!($(".tooltipDiv" + this._id).hasClass(this.svgObject.id)))
		    $(".tooltipDiv" + this._id).remove();
        if (data && ej.util.isNullOrUndefined(data.region.isStripLine)) {
            if (this.model.AreaType == "none" && this.model._visibleSeries[data.region.SeriesIndex].explode) {
                if (this.model._visibleSeries[data.region.SeriesIndex].enableAnimation) {
                    if (this.model.AnimationComplete || this.model.enableCanvasRendering)
                        this.pieExplosion(data, evt);
                }
                else if(!this.model.enable3D)
                    this.pieExplosion(data, evt);
            }
            if (data.region.isIndicator) {
                series = this.model.indicators[data.region.SeriesIndex];
                var seriesPoint = series.segment[0]._visiblePoints[data.region.Region.PointIndex];
                series.fill = series.segment[0].fill;
            } else {
                series = this.model._visibleSeries[data.region.SeriesIndex];               
                var seriesPoint = (series.type == "pieofpie")? this._getPieOfPiePoint(data.region.Region.PointIndex, series):this.model._visibleSeries[data.region.SeriesIndex]._visiblePoints[data.region.Region.PointIndex];
            }

            if (seriesPoint && this.model.AreaType=='polaraxes') {
                for (var i = 0; i < this.model._visibleSeries.length; i++) {
                    $("[id*=" + "_TrackSymbol" + "]").remove();
                    var trackId = "canvas_trackSymbol_" + i;
                    $("#" + this._id).find('[id*=' + trackId + ']').remove();
                }
                var pointlocation = (this.model.AreaType != "polaraxes")
                    ? ej.EjSvgRender.utils._getPoint(seriesPoint, series)
                    : (ej.EjSvgRender.utils.TransformToVisible(series, seriesPoint.xValue, seriesPoint.y, this))
                    var prevPoint = this.model.prevPoint;
                    if (!ej.util.isNullOrUndefined(prevPoint) && prevPoint!= data && series.marker.visible)
                    {
                        var series = this.model._visibleSeries[prevPoint.region.SeriesIndex],
                        prevSeriesPoint = this.model._visibleSeries[prevPoint.region.SeriesIndex]._visiblePoints[prevPoint.region.Region.PointIndex],
                        prevlocation = ej.EjSvgRender.utils.TransformToVisible(series, prevSeriesPoint.xValue, prevSeriesPoint.y, this);
                        this.drawTrackerSymbol(series, prevPoint.region.SeriesIndex, prevPoint.region.Region.PointIndex, null, prevlocation);
                        this.model.prevPoint = null;
                    }
                    if (series.marker.visible && series.drawType != "column") {
                        this.drawTrackerSymbol(series, data.region.SeriesIndex, data.region.Region.PointIndex, true, pointlocation);
                        this.model.prevPoint = data;                        
                    }
            }

            if (series.tooltip.visible && !tooltipShowing) {

                if (series.visibility.toLowerCase() == 'visible') {
                    if (ej.util.isNullOrUndefined(series.tooltip.template)) {
                        var location;
                        if (this.model.AreaType == "cartesianaxes") {

                            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
                            if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                var transToolOptions = { 'id': this.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                this.gTransToolEle = this.svgRenderer.createGroup(transToolOptions);
                            }
                            this.svgRenderer._setAttr($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')' });
                            $('#' + this.svgObject.id + "_TrackToolTip").attr("visibility", "visible");
                            location = { X: (data.location.x - this.model.m_AreaBounds.X), Y: Math.abs(data.location.y - (this.model.m_AreaBounds.Y)) };
                        } else {
                            if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                var transToolTipOptions = { 'id': this.svgObject.id + '_TrackToolTip' };
                                this.gTransToolEle = this.svgRenderer.createGroup(transToolTipOptions);
                            }
                            location = { X: (data.location.x), Y: Math.abs(data.location.y) };
                        }


                        $(".tooltipDiv" + this._id).remove();
                        this.displayShowTooltip(location, seriesPoint, series, data.region.Region.PointIndex);
                            
                        //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                        // if (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter") {
                        //     var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").detach();
                        //    detachEle.appendTo(this.svgObject);
                        // }
                    } else {
                        $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                        this.createTooltip(data.region, evt);
                    }
                }
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = data;
            this._trigger("pointRegionMouseMove", commonEventArgs);
        }
		else if (!ej.util.isNullOrUndefined(this.model.prevPoint)&& this.model.AreaType == "polaraxes") {
            var prevPoint = this.model.prevPoint;
            series = this.model._visibleSeries[prevPoint.region.SeriesIndex];
            seriesPoint = this.model._visibleSeries[prevPoint.region.SeriesIndex]._visiblePoints[prevPoint.region.Region.PointIndex];
            pointlocation = ej.EjSvgRender.utils.TransformToVisible(series, seriesPoint.xValue, seriesPoint.y, this);
            this.drawTrackerSymbol(series, prevPoint.region.SeriesIndex, prevPoint.region.Region.PointIndex, null, pointlocation);
            this.model.prevPoint = null;
        }
        else {
            if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                var type = this.model.series[this.model.explodeValue.SeriesIndex].type.toLowerCase();
                var series = this.model._visibleSeries[this.model.explodeValue.SeriesIndex];
                    var seriesType = new ej.seriesTypes[type]();
                    var targetElement;
                    var id;
                    var _labelPosition = series.labelPosition.toLowerCase();
                    var seriesIndex = this.model.explodeValue.SeriesIndex;
                    var visiblePoint = (this.model._isPieOfPie) ? this._getPieOfPiePoint(this.model.explodeValue.PointIndex, series) : series._visiblePoints[this.model.explodeValue.PointIndex];
                    var result = seriesType._calculateArcData(this, this.model.explodeValue.PointIndex, visiblePoint, series, seriesIndex, this.model.explodeValue.PieSeriesIndex);
                    if (this.model.enableCanvasRendering) {
                        this.model.series[seriesIndex].explodeIndex = null;
                        var chartRect = document.getElementById(this.svgObject.id).getClientRects()[0];
                        this.svgRenderer.ctx.clearRect(chartRect.left, chartRect.top, chartRect.width, chartRect.height);
                        $("#" + this._id).ejChart("redraw");
                    }
                    id = this.svgObject.id + '_SeriesGroup' + '_'+seriesIndex;
                    targetElement = this.model.explodeValue.PieSeriesIndex ==1 ? $(this.gSeriesEle).children('#' + id)[1] : $(this.gSeriesEle).children('#' + id)[0];
                    var elements = $(targetElement).children();
                    for (var i = 0; i < elements.length; i++) {
                        var element = elements[i];
                        var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                        if (parseInt(index) == this.model.explodeValue.PointIndex) {
                            this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                            break;
                        }
                    }
                    seriesType.drawDataLabelAcc(this, series, this.model.explodeValue.PointIndex, visiblePoint, seriesIndex, this.model.explodeValue.PieSeriesIndex);
                    this.model.explodeValue = null;
					this.model.explode= false;
            }
        }
    },

    chartTrackball: function (chart, mouseLocation, evt) {
        var requireInvertedAxes = chart.model.requireInvertedAxes, point, seriesIndex, series, dir, indicators = this.model.indicators, seriesCollection = this.model._visibleSeries,
            showTrackLine = false, seriesIndex, serX, insideBounds, closestPoint, visibleRange, chartPos, pathOptions, leftPos, topPos, chartAreaStartPos, chartAreaEndPos, pointPos,
            crossHairTrans, tToolOptions, prePoint, initialPoint, rectxt, tooltipfont, textarea, tgap = 0, currentLength, tX, tY, chartLocation, element, visiblepts = [], tooltipRanges = [],
            DisplayTooltipOptions = [], enableCanvas = this.model.enableCanvasRendering, totalVisible = 0, maxWidth = 0, heightwidth = 0, chartOffset = $("#" + this._id).offset(),
            textCollection = [], textOptionsCollection = [], groupingTextId = [], trackballDisplayMode = this.model.crosshair.trackballTooltipSettings.mode.toLowerCase(), groupTooltipId = chart._id + "_trackball_grouping_tooltip",
            locale = this.model.locale,boxSeriesCount = 0,trackOutlier = 0;
       if ((!this.model.enable3D) && this.model.crosshair.visible && this.model.crosshair.type.toLowerCase() == "trackball") {

           if ($("#" + this.svgObject.id + "_CrosshairGroup").length == 0) {     // condition to check track axis tooltip is existing in DOM
               var gTrackballOptions = { 'id': chart.svgObject.id + '_CrosshairGroup', 'visibility': 'visible' };
               if (enableCanvas) {
                   gTrackballOptions.position = "absolute";
                   chart.svgRenderer.drawCrosshairLine(gTrackballOptions, '#chartContainer_' + this._id);
               }
               else{
                   chart.gTrackball = chart.svgRenderer.createGroup(gTrackballOptions);
                   chart.svgRenderer.append(chart.gTrackball, chart.svgObject);
               }               
           }
           else {             
               if (chart.model.trackerElement) {
                   if ($.finish)
                       $(chart.model.trackerElement).finish();
                   else
                       $(chart.model.trackerElement).stop(true, true);
                   chart.model.trackerElement = null;                 
               }
           }
           if (!this.model.enableCanvasRendering) {
               // removed highlight style
               $("[class*=" + "HighlightStyle" + "]").attr('class', '');
               $("[class*=" + "HighlightLegendStyle" + "]").attr('class', '');
               $("[class*=" + "HighlightLegendPathStyle" + "]").attr('class', '');
               $("[class*=" + "HighlightPathStyle" + "]").attr('class', '');
               $("[class*=" + "Highlightseries" + "]").attr('class', '');
               $("[id*=" + this._id + "_Highlight_" + "]").remove();// removed canvas highlight           
           }
            //Hiding previous trackball and tooltip when moving from one column to another
           $("#secondCanvas").remove(), $("#" + this._id).find('[id*="_gTooltip_"]').remove(), $("#" + groupTooltipId).children().remove();
            if (enableCanvas) {
                $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
            }
            var seriesLenIndicator = seriesCollection.length;
            for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
                seriesCollection = seriesCollection.concat(indicators[j].segment);
            }
            seriesLength = seriesCollection.length;
            for (var i = 0; i < seriesLength; i++) {
                if (seriesCollection[i].visibility == "visible") {
                    seriesIndex = i;
                     serX = [];
            
                    if (!requireInvertedAxes)
                        insideBounds = chart.mousemoveX + seriesCollection[i].xAxis.plotOffset > seriesCollection[i].xAxis.x && chart.mousemoveX < seriesCollection[i].xAxis.x + seriesCollection[i].xAxis.width + seriesCollection[i].xAxis.plotOffset;
                    else
                        insideBounds = chart.mousemoveY + seriesCollection[i].xAxis.plotOffset > seriesCollection[i].xAxis.y && chart.mousemoveY < seriesCollection[i].xAxis.y + seriesCollection[i].xAxis.height + seriesCollection[i].xAxis.plotOffset;
                    if (enableCanvas)
                        this.canvasX = seriesCollection[i].xAxis.x;
                     closestPoint = chart.getClosestPointX(serX, seriesCollection[i], chart.mousemoveX, chart.mousemoveY);
                     visibleRange = seriesCollection[i].xAxis.visibleRange;   // condition to check the closestPoint is inside the visible range
					 if(closestPoint.point != "" ){
						var closestPointValues = seriesCollection[i].type.toLowerCase() == "boxandwhisker" ? closestPoint.point[0].boxPlotLocation[0].YValues :   closestPoint.point[0].YValues[0];
						if (closestPoint.point != "" && (closestPointValues >= seriesCollection[i].yAxis.visibleRange.min || trackballDisplayMode == "grouping") && (closestPoint.point[0].xValue >= visibleRange.min) && (closestPoint.point[0].xValue <= visibleRange.max)) {
							if (insideBounds) {
								for (var t = 0; t < closestPoint.point.length; t++) {
									if (closestPoint.point[t].visible) {
										if (!seriesCollection[i].isIndicator) {
											totalVisible++;
										}
									}
								}
							}
						}
					}
                } 

           }
            for (var i = 0; i < seriesLength; i++) {
			 //   if(seriesCollection[0].xAxis.name == seriesCollection[i].xAxis.name)
                seriesIndex = i, series = seriesCollection[i]; 
                var trackPoints =[];
                if (series.visibility.toLowerCase() == 'visible') {
                    serX = [];
                    if (!requireInvertedAxes)
                        insideBounds = chart.mousemoveX + seriesCollection[i].xAxis.plotOffset > series.xAxis.x && chart.mousemoveX < series.xAxis.x + series.xAxis.width + seriesCollection[i].xAxis.plotOffset;
                    else
                        insideBounds = chart.mousemoveY + seriesCollection[i].xAxis.plotOffset > series.xAxis.y && chart.mousemoveY < series.xAxis.y + series.xAxis.height + seriesCollection[i].xAxis.plotOffset;
                    if (enableCanvas)
                        this.canvasX = series.xAxis.x;
                    closestPoint = chart.getClosestPointX(serX, series, chart.mousemoveX, chart.mousemoveY);
				    visibleRange = series.xAxis.visibleRange;   // condition to check the closestPoint is inside the visible range
					var point1;
					if(closestPoint.point != ""){
					var closestPointValues = seriesCollection[i].type.toLowerCase() == "boxandwhisker" ? closestPoint.point[0].boxPlotValues.midvalue :  closestPoint.point[0].YValues[0];
					if ((closestPointValues >= seriesCollection[i].yAxis.visibleRange.min || trackballDisplayMode == "grouping") && (closestPoint.point[0].xValue >= visibleRange.min) && (closestPoint.point[0].xValue <= visibleRange.max)) {					
				        if (insideBounds) {	        
				            var closePointLength = closestPoint.point.length;
				            for (var t = 0; t <closePointLength; t++) {
                              if (closestPoint.point[t].visible) {                                 
                                  if (!series.isIndicator) {
								     if(series.type=="boxandwhisker")
									 {
									    boxSeriesCount++;
										var boxvalue =  {"xValue": closestPoint.point[t].xValue, "YValues": closestPoint.point[t].boxPlotLocation[0].YValues };
										point = ej.EjSvgRender.utils._getPoint(boxvalue, series);
										point.YValues = boxvalue.YValues; 
										trackPoints.push(point);									 
										for(var q=0;q<closestPoint.point[t].boxPlotLocation.length;q++)
										{
											if(closestPoint.point[t].boxPlotLocation[q].outlier == true)
											{
												point1 ={X:closestPoint.point[t].boxPlotLocation[q].X,Y:closestPoint.point[t].boxPlotLocation[q].Y,YValues:closestPoint.point[t].boxPlotLocation[q].YValues,outlier:closestPoint.point[t].boxPlotLocation[q].outlier};
												trackPoints.push(point1);									 
											}
										}									 
									 }else
                                      point = ej.EjSvgRender.utils._getPoint(closestPoint.point[t], series);
                                      this.model.financial = seriesLenIndicator == 1 && closePointLength == 1 && series._hiloTypes || false;
                                      if (requireInvertedAxes) {
                                          if (this.model.financial) {
                                              var high = point.X > point.low ? point.X : point.low;
                                              var low = point.X < point.low ? point.X : point.low;
                                              dir = "M" + " " + (series.yAxis.x) + " " + (point.Y + series.xAxis.y) + " " + "L" + " " + (low + series.yAxis.x) + " " + (point.Y + series.xAxis.y) + " " + "M" + " " + (series.yAxis.x + high) + " " + (point.Y + series.xAxis.y) + " " + "L" + " " + (series.yAxis.x + series.yAxis.width) + " " + (point.Y + series.xAxis.y);
                                          }
                                          else
                                              dir = "M" + " " + (series.yAxis.x) + " " + (point.Y + series.xAxis.y) + " " + "L" + " " + (series.yAxis.x + series.yAxis.width) + " " + (point.Y + series.xAxis.y);
										  if(series.type == "boxandwhisker")
										  {
											  for(var p=0;p<trackPoints.length;p++)
											  {												  
													  visiblepts.push({ X: (trackPoints[p].X ), Y: (trackPoints[p].Y ) , YValues:trackPoints[p].YValues ,outlier:trackPoints[p].outlier});												 
											  }
										   }else  
												visiblepts.push({ X: (closestPoint.point[t].location.X + series.yAxis.x), Y: (closestPoint.point[t].location.Y + series.xAxis.y) });
										  }
									else {                                      
                                          if (this.model.financial) {
                                              var high = point.Y < point.low ? point.Y : point.low;
                                              var low = point.Y > point.low ? point.Y : point.low;
                                              dir = "M" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y) + " " + "L" + " " + (point.X + series.xAxis.x) + " " + (high + series.yAxis.y) + " " + "M" + " " + (point.X + series.xAxis.x) + " " + (low + series.yAxis.y) + " " + "L" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y + chart.model.m_AreaBounds.Height);
                                          }
                                          else
                                              dir = "M" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y) + " " + "L" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y + chart.model.m_AreaBounds.Height);
										  if(series.type == "boxandwhisker")
										  {
											   for(var p=0;p<trackPoints.length;p++)
											  {													
														visiblepts.push({ X: (trackPoints[p].X ), Y: (trackPoints[p].Y ) , YValues:trackPoints[p].YValues ,outlier:trackPoints[p].outlier });
											  }
										  }else   
												visiblepts.push({ X: (closestPoint.point[t].location.X + series.xAxis.x), Y: (closestPoint.point[t].location.Y + series.yAxis.y) });
										  }
                                
                                      chartPos = { left: chartOffset.left, top: chartOffset.top };

                                      pathOptions = {
                                    'id': chart.svgObject.id + "_Tracker",
                                    'fill': 'none',
                                    'stroke-width': chart.model.crosshair.line.width,
                                    'clip-path': 'url(#' + chart.svgObject.id + '_ChartAreaClipRect)',
                                    'stroke': chart.model.crosshair.line.color,
                                    'd': dir
                                };
                                      if ($("#" + chart.svgObject.id + "_Tracker").length == 0) {
                                          if (enableCanvas) {
                                              var obj = this.svgRenderer.createCrosshairCanvas();
                                              obj.ctx = obj.getContext('2d');
                                          }
                                          else {
                                              var obj = this.svgRenderer;
                                              var parentElement = chart.gTrackball;
                                          }
                                                                         
                                          chart.svgRenderer.drawPath.call(obj, pathOptions, parentElement);
                                        showTrackLine = true;
                                   
                                }
                                else {
								    showTrackLine = true;
                                    if (!ej.util.isNullOrUndefined(this._closest)) {
                                        if (this._beforeMin) {
                                            if (this._closest >= closestPoint.point[t].xValue)
                                                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker"), { "d": dir, "visibility": "visible" });

                                        }
                                        else if (this._afterMax) {
                                            if (this._closest <= closestPoint.point[t].xValue)
                                                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker"), { "d": dir, "visibility": "visible" });
                                        }
                                    }
                                    else {
                                        chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker"), { "d": dir, "visibility": "visible" });
                                    }
								    $("#" + chart.svgObject.id + "_Tracker").css({"display": "block" });
								    if (enableCanvas) {								      
								        var obj = this.svgRenderer.createCrosshairCanvas();
								            obj.ctx = obj.getContext('2d');
								            chart.svgRenderer.drawPath.call(obj, pathOptions, chart.gTrackball);
								       
                                    }
                                    $("#" + this._id).find('[id*="canvas_Tracker"]').css("visibility", "visible");
                                }

                                      crossHairTrans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
                                      tToolOptions = { 'transform': 'translate(' + crossHairTrans.x + ',' + crossHairTrans.y + ')', 'id': chart.svgObject.id + '_TrackToolTip' + '_' + seriesIndex };
                                chart.gTransToolEle = chart.svgRenderer.createGroup(tToolOptions);
                                      prePoint = this.model.prePoint;

                                //To check the current closestPoint and previous point is same
                                if (ej.util.isNullOrUndefined(prePoint) || prePoint.pointIndex[0] != closestPoint.index[0] || !$('#' + chart.svgObject.id + "_trackSymbol_" + seriesIndex + '_' + closestPoint.index)) {

                                    //Remove the track elements only once on point change
                                    if (ej.util.isNullOrUndefined(initialPoint)) {
                                        if (this.model.crosshair.tooltipTemplate != null) {
                                            $(document).find('[id*="_TrackToolTipTemplate_"]').attr("visibility", "hidden");
                                        }
                                        if (chart.gTransToolEle && chart.gTrackerEle) {
                                            $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]').attr("visibility", "hidden");
                                            $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
                                        }
                                              initialPoint = closestPoint;
                                    }
                                    var type = series.type.toLowerCase(),trackcount = 0;
                                    if (!series._hiloTypes || type.indexOf("rangearea") > -1)
                                        if(trackballDisplayMode != "grouping" || (trackballDisplayMode == "grouping" && type.indexOf("column") < 0 && type.indexOf("bar") < 0))
											if(series.type == "boxandwhisker")
											{
												for(var z=0;z<trackPoints.length;z++)
												{
													chart.drawTrackerSymbol(series, seriesIndex, closestPoint.index[t], true, trackPoints[z],null,trackcount);
													trackcount++;
												}
											}
											else
										       chart.drawTrackerSymbol(series, seriesIndex, closestPoint.index[t], true, point);
                                    series.trackerRemoved = true;
                                     
                                    // Condition to find the crosshair is specified with tooltip template
                                    if (!chart.model.crosshair.tooltipTemplate) {
                                        var currentLength = visiblepts.length - 1,trackOutlier = trackPoints.length;                                             
                                        if (trackballDisplayMode == "grouping" || (!this.model.financial && series._hiloTypes && type!="rangearea")) {
                                            var seriesColor = this.getSeriesColor(closestPoint.point[t], seriesIndex, series),
                                            textOptions, tooltipFormat = series.tooltip.format, currentText,boxgroupingtext,boxtext =[];
											if(series.type == "boxandwhisker"){
												for(var p=0;p<trackOutlier;p++){
													boxgroupingtext = this.getTooltipFormat(closestPoint.point[t], series, seriesIndex, closestPoint.index[t], tooltipFormat,trackPoints[p]);
													boxtext.push(boxgroupingtext);
													currentText = null;
												}
											}
											else
												currentText = this.getTooltipFormat(closestPoint.point[t], series, seriesIndex, closestPoint.index[t], tooltipFormat);
												
											var textColor = series.tooltip.font ? ((series.tooltip.font.color ? series.tooltip.font.color : seriesColor)) : seriesColor,
                                            font = series.tooltip.font ? $.extend(false, series.font, {}, series.tooltip.font) : series.font;
                                            if (typeof (textColor) == "object") {
                                                textColor = seriesColor[1].color;
                                            }                                          
											if(series.type == "boxandwhisker"){
												for(var b=0;b<boxtext.length;b++)
													textCollection.push({ text: boxtext[b].text, seriesIndex: seriesIndex, pointIndex: closestPoint.index[t] });
											}
											else{
												currentText.text = "&nbsp" + currentText.text + "&nbsp";
												textCollection.push({ text: currentText.text, seriesIndex: seriesIndex, pointIndex: closestPoint.index[t] });
											}
                                            if ((series._hiloTypes && type != "rangearea") || type.indexOf("column") > -1 || type.indexOf("bar") > -1) {
                                                data = { region: { Region: { PointIndex: closestPoint.index[t] }, SeriesIndex: seriesIndex } };
                                                this.highlightStart(evt, data, this._id + "_svg_SeriesGroup_" + seriesIndex, this._id + "_svg_Series" + seriesIndex + "_Point" + closestPoint.index[t], true);
                                            }
											if(series.type == "boxandwhisker"){
												for(var b=0;b<boxtext.length;b++){
													groupingTextId.push(chart._id + "_grouping_text_" + seriesIndex + "_" + closestPoint.index[t] + b);
												}
											}
											else
												groupingTextId.push(chart._id + "_grouping_text_" + seriesIndex + "_" + closestPoint.index[t]);												
                                            textOptions = {
                                                "color": textColor,
                                                'font-size': font.size,
                                                'font-family': font.fontFamily,
                                                'font-style': font.fontStyle,
                                                'font-weight': font.fontWeight,
                                            };
											if(series.type == "boxandwhisker"){
												for(var b=0;b<boxtext.length;b++){
													textOptionsCollection.push(textOptions);
												}
											}
											else
												textOptionsCollection.push(textOptions);											
											if(boxSeriesCount == totalVisible && series.type == "boxandwhisker")											
												chart.displayTooltip(true, { point:closestPoint.point[t], series:series, textCollection:textCollection, textOptionsCollection:textOptionsCollection, groupingTextId:groupingTextId});											
                                            else if (totalVisible == textCollection.length && series.type != "boxandwhisker" ) {
                                                chart.displayTooltip(true, { point:closestPoint.point[t], series:series, textCollection:textCollection, textOptionsCollection:textOptionsCollection, groupingTextId:groupingTextId});
                                            }
                                        }
                                        else {										   
												var rectxt = this.getTooltipFormat(closestPoint.point[t], series, seriesIndex, i, series.tooltip.format, i),
												tooltip = series.tooltip, font = tooltip.font ? $.extend(false, series.font, {}, tooltip.font) : series.font, textarea = ej.EjSvgRender.utils._measureText(rectxt.text, null, font), padding, heightwidth = (requireInvertedAxes) ? textarea.height : textarea.width, maxWidth = (maxWidth > heightwidth) ? maxWidth : heightwidth, newLines = Math.ceil(rectxt.text.toString().split("<br/>").length / 2);										
												this.model.tooltipPadding = Number(font.size.toString().replace(/px/i, '')) / 3;
												padding = this.model.tooltipPadding + series.tooltip.border.width * 2;
												if (!requireInvertedAxes) {
													if(series.type == "boxandwhisker")
													{
														for(var p=0;p<trackOutlier;p++)
														{
															tooltipRanges.push({ Start: trackPoints[p].Y - padding * newLines - textarea.height / 2, End: trackPoints[p].Y + padding * newLines + textarea.height / 2 });
															DisplayTooltipOptions.push({ Point: trackPoints[p], ClosestPoint: closestPoint.point[t], Series: series, ClosestPointIndex: closestPoint.index[t], Tgap: tgap, StEnd: tooltipRanges[p], TextArea: textarea, ReqInvertAxis: requireInvertedAxes });
														}	
													}
												else
												{
													tooltipRanges.push({ Start: visiblepts[currentLength].Y - padding * newLines - textarea.height / 2, End: visiblepts[currentLength].Y + padding * newLines + textarea.height / 2 });
													DisplayTooltipOptions.push({ Point: visiblepts[currentLength], ClosestPoint: closestPoint.point[t], Series: series, ClosestPointIndex: closestPoint.index[t], Tgap: tgap, StEnd: tooltipRanges[currentLength], TextArea: textarea, ReqInvertAxis: requireInvertedAxes });
												}
                                            }
                                            else {
                                                tooltipRanges.push({ Start: visiblepts[currentLength].X - padding - (textarea.width / 2), End: visiblepts[currentLength].X + padding + (textarea.width / 2) });
												DisplayTooltipOptions.push({ Point: visiblepts[currentLength], ClosestPoint: closestPoint.point[t], Series: series, ClosestPointIndex: closestPoint.index[t], Tgap: tgap, StEnd: tooltipRanges[currentLength], TextArea: textarea, ReqInvertAxis: requireInvertedAxes });
                                            }																																	
											if(series.type == "boxandwhisker" && boxSeriesCount == totalVisible)
											{
												maxWidth += (2 * padding);
                                                if (requireInvertedAxes) {
                                                    DisplayTooltipOptions = DisplayTooltipOptions.sort(function (a, b) { return a.Point.X - b.Point.X });
                                                }
                                                else {
                                                    DisplayTooltipOptions = DisplayTooltipOptions.sort(function (a, b) { return a.Point.Y - b.Point.Y });
                                                }
                                                this.SmartTooltipPosition(DisplayTooltipOptions);
                                                var displayLength = DisplayTooltipOptions.length;
                                                chart.displayTooltip(false, DisplayTooltipOptions, maxWidth);
                                                for (var i = 0; i < displayLength; i++) {                                                    
                                                    var seriesIndex = $.inArray(DisplayTooltipOptions[i].Series, this.model._visibleSeries);
                                                    if (DisplayTooltipOptions[i].Series._hiloTypes) {
                                                        data = { region: { Region: { PointIndex: DisplayTooltipOptions[i].ClosestPointIndex }, SeriesIndex: seriesIndex } };
                                                        this.highlightStart(evt, data, this._id + "_svg_SeriesGroup_" + seriesIndex, this._id + "_svg_Series" + seriesIndex + "_Point" + DisplayTooltipOptions[i].ClosestPointIndex, true);
                                                    }
                                                }
											}
                                            else if (DisplayTooltipOptions.length == totalVisible && series.type != "boxandwhisker") {											
                                                maxWidth += (2 * padding);
                                                if (requireInvertedAxes) {
                                                    DisplayTooltipOptions = DisplayTooltipOptions.sort(function (a, b) { return a.Point.X - b.Point.X });
                                                }
                                                else {
                                                    DisplayTooltipOptions = DisplayTooltipOptions.sort(function (a, b) { return a.Point.Y - b.Point.Y });
                                                }
                                                this.SmartTooltipPosition(DisplayTooltipOptions);
                                                var displayLength = DisplayTooltipOptions.length;
                                                chart.displayTooltip(false, DisplayTooltipOptions, maxWidth);
                                                for (var i = 0; i < displayLength; i++) {                                                    
                                                    var seriesIndex = $.inArray(DisplayTooltipOptions[i].Series, this.model._visibleSeries);
                                                    if (DisplayTooltipOptions[i].Series._hiloTypes) {
                                                        data = { region: { Region: { PointIndex: DisplayTooltipOptions[i].ClosestPointIndex }, SeriesIndex: seriesIndex } };
                                                        this.highlightStart(evt, data, this._id + "_svg_SeriesGroup_" + seriesIndex, this._id + "_svg_Series" + seriesIndex + "_Point" + DisplayTooltipOptions[i].ClosestPointIndex, true);
                                                    }
                                                }

                                            }
                                        }
                                    }
                                    else {
                                              tX, tY;
                                              chartLocation = document.getElementById(this.svgRenderer.svgObj.id).getClientRects()[0];
                                        chart.displayTemplateTooltip(tX, tY, point, closestPoint.point[t], series, seriesIndex, closestPoint.index[t], chartLocation);
                                    }
                                } else {
                                    // If both are same points, then viibility is set to visible
                                          element = $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]');
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "visible");
                                    element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    
                                    element = $(this.svgObject).find('[id*="_trackSymbol_"]');
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });

                                    $(chart.svgObject).find('[id*="_TrackAxisToolTip"]').attr("visibility", "visible");
                                    
                                    // Sets the tooltip template visibility to visible
                                    if (this.model.crosshair.tooltipTemplate != null) {
                                        element = $(document).find('[id*="_TrackToolTipTemplate_"]');
                                        this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    }
                                }
                                  }
                            } else {
                                
                                $('#' + this.svgObject.id + '_TrackSymbol_' + seriesIndex).remove();
                            }
                         }
						}
						else{						 
				            this._removeTrackBall();
						}
                    } 
				}else {
                        if (!showTrackLine) // //condition checked to display Track line for single series when chart have more than one series. 
                            $(chart.svgObject).find('[id*="_Tracker"]').remove();
                        var elem = $("#" + chart.svgObject.id + "_TrackSymbol_" + i);
                        var trackId = this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex;
                        $("#" + this._id).find('[id*=' + trackId + ']').remove();					 
                        $("#" + chart._id).find('[id*=' + "_canvas_trackSymbol_" + i + ']').css("visibility", "hidden");
                        if (!this.model.enableCanvasRendering) {
                            // removed highlight style
                            $("[class*=" + "HighlightStyle" + "]").attr('class', '');
                            $("[class*=" + "HighlightLegendStyle" + "]").attr('class', '');
                            $("[class*=" + "HighlightLegendPathStyle" + "]").attr('class', '');
                            $("[class*=" + "HighlightPathStyle" + "]").attr('class', '');
                            $("[class*=" + "Highlightseries" + "]").attr('class', '');
                            $("[id*=" + this._id + "_Highlight_" + "]").remove();// removed canvas highlight           
                        }
                    }
                }
                series.trackerRemoved = false;
            }

           
           // To get the previous point values
            if (this.model.closestPoint != null)
            {
                this.model.prePoint = { point: closestPoint.point, pointIndex: closestPoint.index, series: series, seriesIndex: seriesIndex };
            }
            if ($("#" + this.svgObject.id + "_TrackAxisToolTip").length == 0 && this.model.crosshairLabelVisibility) {     // condition to check track axis tooltip is existing in DOM
                var gTrackAxisOptions = { 'id': chart.svgObject.id + '_TrackAxisToolTip', 'visibility': 'visible' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gTrackAxisOptions);
                chart.svgRenderer.append(chart.gTrackAxisEle, chart.gTrackball);
            }
            else {
                $("#" + this.svgObject.id + "_TrackAxisToolTip").css("display", "block");
            }
           
             if (point) {
                 var pointValue = closestPoint.point[0];
				 //Hide unnecessary crosshair labels. SVG and Canvas renders crosshair labels differently
                 if (enableCanvas)
					$(chart.svgObject.parentElement).find('[id*="AxisToolTipRect"]').css({"visibility": "hidden", "display":"none"});
				else
					$(chart.svgObject).find('[id*="_TrackAxisToolTip"]').attr("visibility", "visible").children().attr({"display": "none"});
                $.each(chart.model._axes, function (axisIndex, axis) {					
                    var areaX = axis.x;
                    var opposedPosition = axis._opposed;
                    if (!requireInvertedAxes) {
                        if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                            if ((point.X + areaX) <= (axis.x + axis.width) && axis.x <= (visiblepts[visiblepts.length - 1].X)) {
                                var valueType = axis._valueType.toLowerCase(), xVal, pointLocation;
                                xVal = ((point.X / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                                if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal);
                                if (valueType == "datetime") xVal = (ej.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat), locale));
                                if (valueType == "category" || valueType == "datetimecategory") {
									if (typeof(pointValue.x) !== "object")
										xVal = pointValue.xValue;
                                    xVal = ej.EjSvgRender.utils._getLabelContent(Math.floor(xVal), axis, locale); xVal = xVal ? xVal : "undefined";
                                }
                                pointLocation = { X: (visiblepts[visiblepts.length-1].X), Y: (opposedPosition) ? (axis.y + axis.height - axis.majorTickLines.size) : (axis.y + axis.height + axis.majorTickLines.size + 10) };
                                chart.displayAxisTooltip(pointLocation, xVal, axis, axisIndex, mouseLocation, true);
                            }
                        }
                    }
                    else {
                        if (axis.orientation.toLowerCase() == "vertical" && axis.crosshairLabel.visible) {
                            if ((chart.mousemoveY) <= (axis.y + axis.height) && axis.y <= (chart.mousemoveY)) {
                                var valueType = axis._valueType.toLowerCase(), yVal = pointValue.xValue, ypointLocation=[];
                                if (valueType == "logarithmic") yVal = Math.pow(axis.logBase, yVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                                if (valueType == "datetime") yVal = (ej.format(new Date(Math.floor(yVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat), locale));
                                if (valueType == "category") yVal = ej.EjSvgRender.utils._getLabelContent(yVal, axis, locale);
                                ypointLocation = { X: axis.x, Y: (point.Y + axis.y) };
                                chart.displayAxisTooltip(ypointLocation, yVal, axis, axisIndex, mouseLocation);                                
                            }
                        }
                    }

                });
             }
            
            

        }
    },
    chartCrossHair: function (chart, mouseLocation) {
        var round = ((ej.util.isNullOrUndefined(this.model.primaryYAxis.roundingPlaces)) ? 2 : this.model.primaryYAxis.roundingPlaces),
            enableCanvas = chart.model.enableCanvasRendering, locale = this.model.locale;
        if ((!this.model.enable3D) && this.model.crosshair.visible && this.model.crosshair.type.toLowerCase() == "crosshair") {
            if ($("#" + this.svgObject.id + "_CrosshairGroup").length == 0) {     
                var gTrackballOptions = { 'id': this.svgObject.id + '_CrosshairGroup', 'visibility': 'visible' };
                if (enableCanvas) {
                    gTrackballOptions.position = "absolute";
                    chart.svgRenderer.drawCrosshairLine(gTrackballOptions, '#chartContainer_' + this._id);
                }
                else{
                    this.gCrosshair = this.svgRenderer.createGroup(gTrackballOptions);
                    this.svgRenderer.append(this.gCrosshair, this.svgObject);
                }
            }
            else
                $("#" + this.svgObject.id + "_CrosshairGroup").css("display", "block");
            if (chart.model.trackerElement) {
                if ($.finish)
                    $(this.model.trackerElement).finish();
                else
                    $(this.model.trackerElement).stop(true, true);
                this.model.trackerElement = null;               
            }

            
            var minX, maxX, minY, maxY;
            for (var i = 0; i < this.model._axes.length; i++) {
                var axis = this.model._axes[i];
                if (axis.orientation.toLowerCase() == 'vertical') {
                    if (mouseLocation.y >= axis.y && mouseLocation.y <= axis.y + axis.height)
                        if (axis.x < mouseLocation.x)
                            minX = Math.min(minX || axis.x, axis.x);
                        else
                            maxX = Math.max(maxX || axis.x, axis.x);
                }
                else
                    if (mouseLocation.x >= axis.x && mouseLocation.x <= axis.x + axis.width)
                        if (axis.y < mouseLocation.y)
                            minY = Math.min(minY || axis.y, axis.y);
                        else
                            maxY = Math.max(maxY || axis.y, axis.y);
            }
            var left = minX || this.model.m_AreaBounds.X,
            width = maxX ? maxX - left : this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - left,
            top = minY || this.model.m_AreaBounds.Y,
            height = maxY ? maxY - top : this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height - top;
			
            var d = "M" + " " + (this.mousemoveX) + " " + (top) + " " + "L" + " " + (this.mousemoveX) + " " + (top + height),
            dhor = "M" + " " + (left) + " " + (this.mousemoveY) + " " + "L" + " " + (left + width) + " " + (this.mousemoveY),
            chartOffset = $("#" + this._id).offset(),
            svgObject = chart.svgObject, options, optionsHor,
            chartPos = { left: svgObject.offsetLeft > chartOffset.left ? svgObject.offsetLeft : chartOffset.left,
                   		      top: svgObject.offsetTop > chartOffset.top ? svgObject.offsetTop : chartOffset.top};
            if ($("#" + chart.svgObject.id + "_CrosshairVertical").length == 0) {
                options = {
                    'id': chart.svgObject.id + "_CrosshairVertical",
                    'fill': 'none',
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': d
                };
                if (enableCanvas) {
                    options.width = 0,
                    options.height = height,
                    options.top = top,
                    options.left = left,
                    options.position = "absolute", options.style = "solid", options.opacity = 0.5,                    
                    chart.svgRenderer.drawCrosshairLine(options, '#' + this.svgObject.id + '_CrosshairGroup');
                }
                else
                    chart.svgRenderer.drawPath(options, chart.gCrosshair);
            } else {
                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairVertical"), { "d": d });
                if (enableCanvas) {
                    $("#secondCanvas").remove();
                    $("#" + chart.svgObject.id + "_CrosshairVertical").css("left", this.mousemoveX).css({"top": top, "height" : height, "display" : "block"});
                }
                $("#" + chart.svgObject.id + "_CrosshairVertical").attr("visibility", "visible").css("display", "block");
            }
            
            if ($("#" + chart.svgObject.id + "_CrosshairHorizontal").length == 0) {
                optionsHor = {
                    'id': chart.svgObject.id + "_CrosshairHorizontal",
                    'fill': 'none',
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': dhor
                };
                if (enableCanvas) {
                    optionsHor.top = top + chartPos.top,
                    optionsHor.left = left + chartPos.left,
                    optionsHor.width = width,
                    optionsHor.height = 0,
                    optionsHor.position = "absolute", optionsHor.style = "solid", optionsHor.opacity = 0.5,                    
                    chart.svgRenderer.drawCrosshairLine(optionsHor, '#' + this.svgObject.id + '_CrosshairGroup');
                }
                else
                    chart.svgRenderer.drawPath(optionsHor, chart.gCrosshair);
            } else {
                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairHorizontal"), { "d": dhor });
                $("#" + chart.svgObject.id + "_CrosshairHorizontal").attr("visibility", "visible").css("display", "block");
                if (enableCanvas) {
                    $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("top", this.mousemoveY).css({"left" : left, "width" : width, "display" : "block"});                    
                }
            }
            if ( $('#' + chart.svgObject.id + '_AxisCrossToolTip').length ==0) {
                var gAxisCrossOptions = { 'id': chart.svgObject.id + '_AxisCrossToolTip', 'visibility': 'visible' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gAxisCrossOptions);
                chart.svgRenderer.append(chart.gTrackAxisEle, chart.gCrosshair);
            }
            else {
                this.svgRenderer._setAttr($(chart.gTrackAxisEle), { "visibility": 'visible' });
                $(chart.gTrackAxisEle).css('display', 'block');
            }

            $.each(chart.model._axes, function (axisIndex, axis) {
                var valueType = axis._valueType.toLowerCase();
                if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                    if ((chart.mousemoveX) <= (axis.x + axis.width) && axis.x <= (chart.mousemoveX)) {
                        var labelplacement = (axis._valueType == "category" && axis.labelPlacement.toLowerCase()=='betweenticks') ? 0.5 : 0;
                        var xVal = ((Math.abs(chart.mousemoveX - (axis.isInversed ? axis.x + axis.width : axis.x)) / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min + labelplacement).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "datetime") xVal = (ej.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat), locale));
                        if (valueType == "category" || valueType == "datetimecategory") xVal = ej.EjSvgRender.utils._getLabelContent(xVal, axis, locale);
                        var xPointLocation = { X: (chart.mousemoveX), Y: (axis.y) };
                        chart.displayAxisTooltip(xPointLocation, xVal, axis, axisIndex, mouseLocation);
						 $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).show();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).show();
                    }
					 else
					 {
                        $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).hide();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).hide();
                    }
                } else if (axis.orientation.toLowerCase() == "vertical" && axis.crosshairLabel.visible) {
                    if ((chart.mousemoveY) <= (axis.y + axis.height) && axis.y <= (chart.mousemoveY)) {
                        var yVal = ((Math.abs(1 - (Math.abs(chart.mousemoveY - (axis.isInversed ? axis.y + axis.height : axis.y)) / (axis.width)) / (axis.height)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(round).replace(new RegExp("\\.0{" + round + "}"), ""));
                        if (valueType == "logarithmic") yVal = Math.pow(axis.logBase, yVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "datetime") yVal = (ej.format(new Date(Math.floor(yVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat), locale));
                        if (valueType == "category" || valueType == "datetimecategory") yVal = ej.EjSvgRender.utils._getLabelContent(yVal, axis, locale);
                        var ypointLocation = { X: axis.x, Y: (chart.mousemoveY) };
                        chart.displayAxisTooltip(ypointLocation, yVal, axis, axisIndex, mouseLocation);
						 $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).show();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).show();
                    }
					 else
				     {
                        $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).hide();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).hide();
                    }
                }
            });

        }
    },
    chartTouchMove: function (evt) {
	   this.cancelEvent(evt);
        evt = evt.originalEvent.touches[0];
        this.chartInteractiveBehavior(evt);

    },

    isZoomButtonHovered: function (target) {
        if (target.parentNode != null) {
            var parentId = target.parentNode.id;
            var id = target.id;
            if (id.indexOf('_ZoomInBtn ')==-1 && id.indexOf('ZoomOutBtn')==-1 && id.indexOf('_ZoomBtn') == -1 && id.indexOf('_ResetZoom') == -1 && id.indexOf('_PanBtn') == -1 && parentId && parentId.indexOf('_ZoomInBtn')==-1 && parentId.indexOf('_ZoomOutBtn')==-1&&parentId.indexOf('_ZoomBtn') == -1 && parentId.indexOf('_ResetZoom') == -1 && parentId.indexOf('_PanBtn') == -1)
                return false;
        }
        return true;
    },
 showTooltipOnDrag: function(chart,evt){
        $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'visible');
        $("#" + chart.svgObject.id + "_TrackToolTip").show();
        $(".tooltipDiv").css("display", "block");
    },
    chartInteractiveBehavior: function (evt) {
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;

        var id = "#" + this.svgObject.id;
        if (this.mouseWheelCoords) {
            var changeX = Math.abs(this.mouseWheelCoords.x - evt.pageX);
            var changeY = Math.abs(this.mouseWheelCoords.y - evt.pageY);
            if ((changeX > 0 || changeY > 0) && !(this.panning)) {
                this.enableTrackBall();
            } else {
                this.disableTrackBall();
            }
        }

        var chart = this,
        targetId = evt.target.id;        
        if (chart.model.AreaType == "cartesianaxes"  && this.mousemoveX >= this.model.m_AreaBounds.X && this.mousemoveX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousemoveY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousemoveY >= this.model.m_AreaBounds.Y && this.model.initSeriesRender) {
            var mouseLocation = { x: chart.mousemoveX, y: chart.mousemoveY };
            var targetId = chart.svgRenderer._getAttrVal($(evt.target).parent(), "id");
            if ((targetId != undefined) && (targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ZoomBtn'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ResetZoom'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_PanBtn'), "id"))) {
                this.disableTrackBall();
            }
            else {
                this.enableTrackBall();
            }

            //show tooltip and track ball
            if (!this.model.crosshair.visible && !this.isZoomButtonHovered(evt.target))
                this.tooltip(chart, evt);
            else {
               if (this.dragPoint) 
                    this.showTooltipOnDrag(chart,evt)
              else if (!this.closestXyPoint) {
                $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
                $("#" + chart.svgObject.id + "_TrackToolTip").hide();
                $(".tooltipDiv" + this._id).css("display", "none");
            }
}
            if (!this.model.enable3D) {
			    //crossHair of chart
                this.chartTrackball(chart, mouseLocation, evt);

			    //crossHair for chart area
                this.chartCrossHair(chart, mouseLocation);
			}

        } else {
            if ((chart.model.AreaType == "none" || chart.model.AreaType == 'polaraxes') && !this.isZoomButtonHovered(evt.target))
                this.tooltip(chart, evt);
            else {
			  if (this.dragPoint) 
                    this.showTooltipOnDrag(chart,evt)
              else if (!this.closestXyPoint) {	
                $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                $("#" + chart.svgObject.id + "_TrackToolTip").hide();
                $(".tooltipDiv" + this._id).css("display", "none");
                this._removeTrackBall();
                this._removeHighlight();
            }
}
             //Removing exploded marker symbol when placing mouse outside chart area
                var pointData = this.model.prevPoint;
                if (pointData && pointData.point) {
                    if (this.model.enableCanvasRendering)
                        $("#canvas_trackSymbol").remove();
                    else {
					    if(pointData.series.type == "boxandwhisker")
							var pointLocation = pointData.outlierPosition;
						else						
							var pointLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                        chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, pointLocation);
                    }
                    this.model.prevPoint = null;
                }
        }      
      
    },

    // highlight SVG logic started here
    highlightSvg: function (chart, parentNodeId, isElement, evt, data, targetID, tracker) {
        if (parentNodeId != undefined) {
            var matchStr = this.svgObject.id + "_SeriesGroup_";
            var serIndex = parentNodeId.substr(matchStr.length);
            if (parentNodeId.indexOf(this.svgObject.id + "_symbolGroup_") >= 0) {
                matchStr = this.svgObject.id + "_symbolGroup_";
                serIndex = parentNodeId.substr(matchStr.length);
            }
        }
      
        serIndex = data != undefined ? data.region.SeriesIndex : serIndex
        var series = this.model.series[serIndex];
        if (isElement && series) {           
            if (series.highlightSettings.enable || tracker)
                this.highlight(chart, evt, series, null, data, parentNodeId, targetID, tracker);
        }

    },

     // canvas highlight logic started here
    highlightCanvas: function (chart, evt, data) {
        for (var i = 0; i < this.model._visibleSeries.length; i++) {
            var seriesIndex;
            var chartSeries;
            seriesIndex = i;
            chartSeries = this.model._visibleSeries[i];
            var type = chartSeries.type.toLowerCase();
            if (this.model.AreaType == "cartesianaxes" && chartSeries.visibility.toLowerCase() == 'visible' && type !== "scatter" && type !== "bubble" && type !== "column" && type.indexOf("bar") == -1 && type !== "stackingcolumn" && type !== "stackingcolumn100" && !chartSeries._hiloTypes) {
                var serY = [];
                var serX = [];
                var location = null;
                var mouseMoveCords = this.calMousePosition(evt);
                this.mousemoveX = mouseMoveCords.X;
                this.mousemoveY = mouseMoveCords.Y;
                var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY, evt);
                if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {
                    var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonPointEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY }, region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
                }
            }
        }
        var data = commonPointEventArgs ? commonPointEventArgs.data : data;
        if (data && this.model.series[data.region.SeriesIndex].highlightSettings.enable) {
            var series = $.extend({}, this.model.series[data.region.SeriesIndex]);
            series.pointIndex = data.region.Region.PointIndex;
            series.seriesIndex = data.region.SeriesIndex;
            series.data = data;
            if ($('#' + this._id + '_Selection_series' + series.seriesIndex + '_canvas').length == 0) {              
                // removed canvas highlight
                $("[id*=" + this._id + "_Highlight_" + "]").remove();
                if(!this.findCanvasSelection(series.seriesIndex,series.pointIndex,series.selectionSettings.mode.toLowerCase()))
                this.canvasHighlight(this, evt, series);
            }
        }
    },
    //change cursor symbols after resizing
    resizeCursor: function (containerSvg, mousemoveX, mousemoveY, chartAreaX, chartAreaY, chartAreaWidth, chartAreaHeight) {
        var selectedRectIndex = this.selectedRectIndex;
        if ((mousemoveX <= chartAreaX) || (mousemoveY <= chartAreaY) || (mousemoveX >= (chartAreaWidth + chartAreaX)) || (mousemoveY >= (chartAreaHeight + chartAreaY))) {
            $("#" + containerSvg + '_resize_rightRect' + selectedRectIndex).css({ 'cursor': 'default' });
            $("#" + containerSvg + '_resize_leftRect' + selectedRectIndex).css({ 'cursor': 'default' });
            $("#" + containerSvg + '_resize_bottomRect' + selectedRectIndex).css({ 'cursor': 'default' });
            $("#" + containerSvg + '_resize_topRect' + selectedRectIndex).css({ 'cursor': 'default' });
            $("#" + containerSvg + '_resize_bottomRightCornerCircle' + selectedRectIndex).css({ 'cursor': 'default' });
        }
        else if ((mousemoveX >= chartAreaX) || (mousemoveY >= chartAreaY) || (mousemoveX <= (chartAreaWidth + chartAreaX)) || (mousemoveY <= (chartAreaHeight + chartAreaY))) {
            $("#" + containerSvg + '_resize_rightRect' + selectedRectIndex).css({ 'cursor': 'e-resize' });
            $("#" + containerSvg + '_resize_leftRect' + selectedRectIndex).css({ 'cursor': 'w-resize' });
            $("#" + containerSvg + '_resize_bottomRect' + selectedRectIndex).css({ 'cursor': 's-resize' });
            $("#" + containerSvg + '_resize_topRect' + selectedRectIndex).css({ 'cursor': 'n-resize' });
            $("#" + containerSvg + '_resize_bottomRightCornerCircle' + selectedRectIndex).css({ 'cursor': 'se-resize' });
        }
    },
    multiSelectMouseMove: function (evt) {
        if (this._isMultiSelect && (this.multiSelectMode == 'range') && !this.dragPoint) {
            var mouseMoveCords = this.calMousePosition(evt),
            mouseMoveX = mouseMoveCords.X,
            mouseMoveY = mouseMoveCords.Y,
            gripMouseMoveX, gripMouseMoveY, translate,
            selectionSettingsType = this.multiSelectType,
            width, height, x, y,
            rectX, rectY, rectWidth, rectHeight,
            x1, x2, y1, y2, xPlotOffset, yPlotOffset,
            modifiedX, modifiedY,
            mouseDownX = this.mouseDownX,
            mouseDownY = this.mouseDownY,
            selectedRectIndex = this.selectedRectIndex,
            containerSvg = this.svgObject.id,
            chartArea = '#' + containerSvg + '_ChartArea'
            chartAreaX = parseInt($(chartArea).attr('x')),
            chartAreaY = parseInt($(chartArea).attr('y')),
            chartAreaWidth = parseInt($(chartArea).attr('width')),
            chartAreaHeight = parseInt($(chartArea).attr('height')),
            multiSelectMouseDownId = this.multiSelectMouseDownId,
            selctRectId = '#' + containerSvg + '_selectRect';
            multiAxis = this.multiAxis; 
            for (var q = 0; q < multiAxis.length; q++) {
                if (multiAxis[q].orientation.toLowerCase() == 'horizontal') {
                    x1 = multiAxis[q].Location.X1; x2 = multiAxis[q].Location.X2;
                    xPlotOffset = multiAxis[q].plotOffset;
                }
                else {
                    y1 = multiAxis[q].Location.Y1; y2 = multiAxis[q].Location.Y2;
                    yPlotOffset = multiAxis[q].plotOffset;
                }
            }
            if ((this.drag) && (!this.resize) && (!this.rectPan) && (!this.removeRect)) {
                var currentX = mouseMoveX,
                currentY = mouseMoveY,
                areaBounds = this.model.m_AreaBounds;
                this.multiSelectDraw = true;
                if (mouseMoveX < areaBounds.X) {
                    currentX = areaBounds.X;
                }
                else if (mouseMoveX > (areaBounds.X + areaBounds.Width)) {
                    currentX = areaBounds.X + areaBounds.Width;
                }
                if (mouseMoveY < areaBounds.Y) {
                    currentY = areaBounds.Y;
                } else if (mouseMoveY > (areaBounds.Y + areaBounds.Height)) {
                    currentY = areaBounds.Y + areaBounds.Height;
                }
                else {
                    currentY = mouseMoveY;
                }
                switch (selectionSettingsType) {
                    case 'x':
                        width = Math.abs(currentX - mouseDownX);
                        height = y1 - y2 + yPlotOffset + yPlotOffset;
                        x = currentX > mouseDownX ? mouseDownX : currentX;
                        y = y2 - yPlotOffset;
                        break;
                    case 'y':
                        width = x2 - x1 + xPlotOffset + xPlotOffset;
                        height = Math.abs(currentY - mouseDownY);
                        x = x1 - xPlotOffset;
                        y = currentY > mouseDownY ? mouseDownY : currentY;
                        break;
                    case 'xy':
                        width = Math.abs(currentX - mouseDownX);
                        height = Math.abs(currentY - mouseDownY);
                        x = currentX > mouseDownX ? mouseDownX : currentX;
                        y = currentY > mouseDownY ? mouseDownY : currentY;
                        break;
                }
                if (width > 0 && height > 0) {
                    var rectOptions = {
                        'id': this.svgObject.id + '_selectRect' + this.selectedRectIndex,
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': 'rgba(41,136,214,0.2)',
                        'stroke-width': 2,
                        'stroke': 'rgba(41,136,214,0.5)',
                        'cursor': 'pointer',
                        'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                    };
                if ((x >= x1 + xPlotOffset) && (mouseMoveX <= x2 - xPlotOffset) && (mouseMoveY <= y1 + yPlotOffset) && (y >= y2 - yPlotOffset))
                    this.svgRenderer.drawRect(rectOptions, this.gEle);
                if ((this.gEle != null)) {
                    $(this.gEle).appendTo(this.parentgEle);
                    if (this.selectedRectIndex == 0) {
                        this.model.gCurrentEle = this.gEle;
                        }
                        $(this.parentgEle).appendTo(this.svgObject);
                    }
                }
            }
            rectX = parseInt($(selctRectId + selectedRectIndex).attr('x'));
            rectY = parseInt($(selctRectId + selectedRectIndex).attr('y'));
            rectWidth = parseInt($(selctRectId + selectedRectIndex).attr('width'));
            rectHeight = parseInt($(selctRectId + selectedRectIndex).attr('height'));
            if (!this.drag && (evt.target.id.indexOf("selectRect") >= 0 || evt.target.id.indexOf("resize") >= 0)) {
                if (evt.target.parentNode.id.indexOf("selectedRect") >= 0) {
                    id = evt.target.parentNode.id.split("selectedRect")[1].match(/\d+/)[0];
                    this.gripIndex = eval(id);
                }
                var selectRectId = '#' + containerSvg + '_selectRect',
                selectRectWidth = parseInt($(selectRectId + this.gripIndex).attr("width")),
                selectRectHeight = parseInt($(selectRectId + this.gripIndex).attr("height")),
                selectRectX = parseInt($(selectRectId + this.gripIndex).attr("x")),
                selectRectY = parseInt($(selectRectId + this.gripIndex).attr("y"));
                $("#" + containerSvg + "_gripCollection" + selectedRectIndex).css({ 'visibility': 'hidden' });
                if (selectRectX + selectRectWidth - 16 >= selectRectX || selectRectY + selectRectHeight - 16 >= selectRectY)
                    $("#" + this.svgObject.id + "_gripCollection" + this.gripIndex).attr({ 'transform': "" });
                if (selectRectX + selectRectWidth - 16 < selectRectX || selectRectY + selectRectHeight - 16 < selectRectY) {
                    var translate = 'translate(' + 15 + ',' + 15 + ')';
                    $("#" + this.svgObject.id + "_gripCollection" + this.gripIndex).attr({ 'transform': translate });
                }
                if (this.oldId != evt.target.id)
                    $("#" + containerSvg + "_gripCollection" + this.oldGripIndex).css({ 'visibility': 'hidden' });
                $("#" + containerSvg + "_gripCollection" + this.gripIndex).css({ 'visibility': 'visible' });
                this.oldId = evt.target.id;
                this.oldGripIndex = this.gripIndex;
            }
            if ((evt.target.id.indexOf("selectRect") < 0) && (evt.target.id.indexOf("resize") < 0 ))
                $("#" + containerSvg + "_gripCollection" + this.gripIndex).css({ 'visibility': 'hidden' });
            if (this.resize) {
                $("#" + containerSvg + "_closeTopRightCornerCircle" + selectedRectIndex).hide();
                $("#" + containerSvg + "_closePath" + selectedRectIndex).hide();
                $("#" + containerSvg + "_closePathOpposite" + selectedRectIndex).hide();
                $("#" + containerSvg + "_gripCollection" + selectedRectIndex).css({ 'visibility': 'hidden' });
                var oldRectX = this.oldRectX[selectedRectIndex], oldRectY = this.oldRectY[selectedRectIndex],
                oldRectHeight = this.oldRectHeight[selectedRectIndex], oldRectWidth = this.oldRectWidth[selectedRectIndex],
                oldReBottomRectY = this.oldReBottomRectY[selectedRectIndex],
                oldReRightRectX = this.oldReRightRectX[selectedRectIndex];
                switch (selectionSettingsType) {
                    case 'x':
                        if (multiSelectMouseDownId == containerSvg + '_resize_rightRect' + selectedRectIndex) {
                            rectWidth = mouseMoveX - oldRectX;
                            $("#" + containerSvg + "_resize_rightRect" + selectedRectIndex).attr('x', mouseMoveX - 5);
                            if (mouseMoveX < oldRectX) {
                                rectX = mouseMoveX;
                                rectWidth = oldRectX - mouseMoveX;
                            }
                            else if (mouseMoveX >= oldRectX) rectX = oldRectX;
                        }
                        else if (multiSelectMouseDownId == containerSvg + '_resize_leftRect' + selectedRectIndex) {
                            rectX = mouseMoveX;
                            rectWidth = oldRectWidth + oldRectX - mouseMoveX;
                            $("#" + containerSvg + "_resize_leftRect" + selectedRectIndex).attr('x', mouseMoveX - 5);
                            if (mouseMoveX > (oldReRightRectX + 5)) {
                                rectX = oldReRightRectX + 5;
                                rectWidth = mouseMoveX - oldRectX - oldRectWidth;
                            }
                        }
                        this.resizeCursor(containerSvg, mouseMoveX, mouseMoveY, chartAreaX, chartAreaY, chartAreaWidth, chartAreaHeight);
                        break;
                    case 'y':
                        if (multiSelectMouseDownId == containerSvg + '_resize_topRect' + selectedRectIndex) {
                            rectHeight = oldRectHeight + oldRectY - mouseMoveY;
                            rectY = mouseMoveY;
                            $("#" + containerSvg + "_resize_topRect" + selectedRectIndex).attr('y', mouseMoveY - 5);
                            if (mouseMoveY > (oldReBottomRectY + 5)) {
                                rectY = oldReBottomRectY + 5;
                                rectHeight = mouseMoveY - oldRectY - oldRectHeight;
                            }
                        }
                        else if (multiSelectMouseDownId == (containerSvg + '_resize_bottomRect' + selectedRectIndex)) {
                            rectHeight = mouseMoveY - oldRectY;
                            $("#" + containerSvg + "_resize_bottomRect" + selectedRectIndex).attr('y', mouseMoveY - 5);
                            if (mouseMoveY < oldRectY) {
                                rectY = mouseMoveY;
                                rectHeight = oldRectY - mouseMoveY;
                            }
                            else if (mouseMoveY >= oldRectY) rectY = oldRectY;
                        }
                        this.resizeCursor(containerSvg, mouseMoveX, mouseMoveY, chartAreaX, chartAreaY, chartAreaWidth, chartAreaHeight);
                        break;
                    case 'xy':
                        if (multiSelectMouseDownId == containerSvg + '_resize_rightRect' + selectedRectIndex) {
                            rectWidth = mouseMoveX - oldRectX;
                            $("#" + containerSvg + "_resize_rightRect" + selectedRectIndex).attr('x', mouseMoveX - 5);
                            if (mouseMoveX < oldRectX) {
                                rectX = mouseMoveX;
                                rectWidth = oldRectX - mouseMoveX;
                            }
                            else if (mouseMoveX >= oldRectX)
                                rectX = oldRectX;
                        }
                        else if (multiSelectMouseDownId == containerSvg + '_resize_leftRect' + selectedRectIndex) {
                            rectX = mouseMoveX;
                            rectWidth = oldRectWidth + oldRectX - mouseMoveX;
                            $("#" + containerSvg + "_resize_leftRect" + selectedRectIndex).attr('x', mouseMoveX - 5);
                            if (mouseMoveX > (oldReRightRectX + 5)) {
                                rectX = oldReRightRectX + 5;
                                rectWidth = mouseMoveX - oldRectX - oldRectWidth;
                            }
                        }
                        else if (multiSelectMouseDownId == containerSvg + '_resize_topRect' + selectedRectIndex) {
                            rectHeight = oldRectHeight + oldRectY - mouseMoveY;
                            rectY = mouseMoveY;
                            $("#" + containerSvg + "_resize_topRect" + selectedRectIndex).attr('y', mouseMoveY - 5);
                            if (mouseMoveY > (oldReBottomRectY + 5)) {
                                rectY = oldReBottomRectY + 5;
                                rectHeight = mouseMoveY - oldRectY - oldRectHeight;
                            }
                        }
                        else if (multiSelectMouseDownId == (containerSvg + '_resize_bottomRect' + selectedRectIndex)) {
                            rectHeight = mouseMoveY - oldRectY;
                            $("#" + containerSvg + "_resize_bottomRect" + selectedRectIndex).attr('y', mouseMoveY - 5);
                            if (mouseMoveY < oldRectY) {
                                rectY = mouseMoveY;
                                rectHeight = oldRectY - mouseMoveY;
                            }
                            else if (mouseMoveY >= oldRectY)
                                rectY = oldRectY;
                        }
                        else if (multiSelectMouseDownId == containerSvg + '_resize_bottomRightCornerCircle' + selectedRectIndex) {
                            rectWidth = mouseMoveX - oldRectX;
                            rectHeight = mouseMoveY - oldRectY;
                            $("#" + containerSvg + "_resize_bottomRightCornerCircle" + selectedRectIndex).attr({ 'cx': mouseMoveX, 'cy': mouseMoveY });
                            if ((mouseMoveX < oldRectX) && (mouseMoveY < oldRectY)) {
                                rectX = mouseMoveX;
                                rectY = mouseMoveY;
                                rectWidth = oldRectX - mouseMoveX;
                                rectHeight = oldRectY - mouseMoveY;
                            }
                            else if (mouseMoveX < oldRectX) {
                                rectX = mouseMoveX;
                                rectWidth = oldRectX - mouseMoveX;
                            }
                            else if (mouseMoveY < oldRectY) {
                                rectY = mouseMoveY;
                                rectHeight = oldRectY - mouseMoveY;
                            }
                            else if ((mouseMoveX >= oldRectX) && (mouseMoveY >= oldRectY)) {
                                rectX = oldRectX;
                                rectY = oldRectY;
                            }
                        }
                        this.resizeCursor(containerSvg, mouseMoveX, mouseMoveY, chartAreaX, chartAreaY, chartAreaWidth, chartAreaHeight);
                        break;
                }
                switch (selectionSettingsType) {
                    case 'x':
                        if (rectX >= x1 - xPlotOffset && (rectWidth + rectX - x1 <= x2 - x1 - xPlotOffset))
                            $(selctRectId + selectedRectIndex).attr({ 'x': rectX, 'width': rectWidth });
                        break;
                    case 'y':
                        if (((rectHeight + rectY - y2) <= y1 - y2 + yPlotOffset) && (rectY >= y2 - yPlotOffset))
                            $(selctRectId + selectedRectIndex).attr({ 'y': rectY, 'height': rectHeight });
                        break;
                    case 'xy':
                        if (rectX >= x1 - xPlotOffset && (rectWidth + rectX - x1 <= x2 - x1 - xPlotOffset) && (rectHeight + rectY - y2 <= y1 - y2 + yPlotOffset) && rectY >= y2 - yPlotOffset)
                            $(selctRectId + selectedRectIndex).attr({ 'x': rectX, 'y': rectY, 'width': rectWidth, 'height': rectHeight });;
                        break;
                }
            }
            if ((this.rectPan)) {
                $("#" + containerSvg + "_closeTopRightCornerCircle" + selectedRectIndex).hide();
                $("#" + containerSvg + "_closePath" + selectedRectIndex).hide();
                $("#" + containerSvg + "_closePathOpposite" + selectedRectIndex).hide();
                $("#" + containerSvg + "_gripCollection" + selectedRectIndex).css({ 'visibility': 'hidden' });
                if ((mouseMoveX <= chartAreaX || mouseMoveY <= chartAreaY || (mouseMoveX >= chartAreaX + chartAreaWidth) || (mouseMoveY >= chartAreaY + chartAreaHeight)))
                    this.multiSelectMouseUp(evt);
                modifiedX = this.PreviousCoords.X - evt.pageX;
                modifiedY = this.PreviousCoords.Y - evt.pageY;
                rectX = rectX - modifiedX;
                rectY = rectY - modifiedY;
                this.PreviousCoords.X = evt.pageX;
                this.PreviousCoords.Y = evt.pageY;
                switch (selectionSettingsType) {
                    case 'x':
                        if (rectX >= x1 - xPlotOffset && (rectWidth + rectX - x1 <= x2 - x1 - xPlotOffset))
                            $(selctRectId + selectedRectIndex).attr({ 'x': rectX });
                        break;
                    case 'y':
                        if ((rectHeight + rectY - y2 <= y1 - y2 + yPlotOffset) && rectY >= y2 - yPlotOffset)
                            $(selctRectId + selectedRectIndex).attr({ 'y': rectY });
                        break;
                    case 'xy':
                        if (rectX >= x1 - xPlotOffset && (rectWidth + rectX - x1 <= x2 - x1 - xPlotOffset) && (rectHeight + rectY - y2 <= y1 - y2 + yPlotOffset) && rectY >= y2 - yPlotOffset)
                            $(selctRectId + selectedRectIndex).attr({ 'x': rectX, 'y': rectY });
                        break;
                }
            }
        }
    },
    isTouch: function (evt) {
        var browserInfo = this.model.browserInfo,
            event = evt.originalEvent ? evt.originalEvent : evt;             
        if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
            return true;
        return false;
        
    },
    highlightStart: function (evt, data, parentNodeId, targetID, tracker) {
        if (evt.target.parentNode != null) {
            var parentNodeId = parentNodeId || evt.target.parentNode.id;
            var isElement = (parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_symbolGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_TrackSymbol_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0) ? true : false;
        }
      
        if (this.model.enableCanvasRendering)
            this.highlightCanvas(this, evt, data);
        else
            this.highlightSvg(this, parentNodeId, isElement, evt, data, targetID, tracker)
    },
    
    _changeDraggingPoints: function (chartSeries, index, X, Y) {       
		X = chartSeries.xAxis._valueType == "datetime" ? new Date(X) : X;
        Y = chartSeries.yAxis._valueType == "datetime" ? new Date(Y) : Y;
        dragType = chartSeries.dragSettings.type.toLowerCase();
		if ((dragType == "x" || dragType == "xy") && chartSeries.xAxis._valueType != "category") {
            if (this.dragPoint) {
                chartSeries.pointCollection[index].x = X;
                chartSeries.pointCollection[index].xValue = (chartSeries.xAxis._valueType == "datetime" && typeof X != "object") ? new Date(X) : X;
            }
            else {
                chartSeries._visiblePoints[index].x = chartSeries.points[index].x = X;
                chartSeries._visiblePoints[index].xValue = chartSeries.points[index].xValue = (chartSeries.xAxis._valueType == "datetime" && typeof X == "object") ? Date.parse(X) : X;
            }
            if (!ej.util.isNullOrUndefined(chartSeries.dataSource))
                chartSeries.dataSource[index][chartSeries.xName] = X;
        }
		if (dragType == "y" || dragType == "xy") {
            if (this.dragPoint) {
                chartSeries.pointCollection[index].y = Y
                chartSeries.pointCollection[index].YValues[0] = (chartSeries.yAxis._valueType == "datetime" && typeof Y != "object") ? new Date(Y) : Y;
            }     
            else {
                chartSeries._visiblePoints[index].y = chartSeries.points[index].y = Y;
                chartSeries._visiblePoints[index].YValues[0] = chartSeries.points[index].YValues[0] = (chartSeries.yAxis._valueType == "datetime" && typeof Y == "object") ? Date.parse(Y) : Y;
            }
            if (!ej.util.isNullOrUndefined(chartSeries.dataSource))
                chartSeries.dataSource[index][chartSeries.yName] = Y;
        }
 },

     // Triggers while dragging the point
 _dragDelta: function (newXval,newYval,oldXval,oldYval,srIndex,ptIndex) {
     var chart = this,
        oldValue = { X: oldXval, Y: oldYval },
        newValue = { X: newXval, Y: newYval },
        commonDragEventArgs = $.extend({}, this.commonDragEventArgs);
        commonDragEventArgs.data = { seriesIndex: srIndex, pointIndex: ptIndex, oldValue: oldValue, newValue: newValue };
        this._trigger("dragging", commonDragEventArgs);
        this.commonDragEventArgs = commonDragEventArgs;
 },
 _getDraggedPoint:function(chartSeries,currentX,currentY){
     var mousePoint, PointX, PointY,x,y;
     var valAxis = ej.EjSvgRender.utils._getTransform(chartSeries.xAxis, chartSeries.yAxis, this.model.requireInvertedAxes);
     if (this.model.requireInvertedAxes) {
         x = (valAxis.y + valAxis.height) - currentY;
         y = (currentX - valAxis.x);
     }
     else {
         x = (currentX - valAxis.x);
         y = (valAxis.y + valAxis.height) - currentY;
     }
     mousePoint = ej.EjSvgRender.utils._getValuebyPoint(x, y, chartSeries);
     PointX = parseFloat(mousePoint.PointX.toFixed(2));
     PointY = parseFloat(mousePoint.PointY.toFixed(2));
     return { X: PointX, Y: PointY };
 },

_pointDragandDrop: function (chartSeries, evt, chart) {
    var i = chartSeries.data.seriesIndex,
        pointCollection = [], translate = null,
        chartType = chartSeries.type.toLowerCase();       
    var params = {};
        params.axes = {};    
        for (var k = 0; k < this.model._axes.length; k++)
            params.axes[this.model._axes[k].name] = {};           
       
        
            $("[id*=" + "_Marker" + "]").remove();
            var draggedPoints = this._getDraggedPoint(chartSeries, this.mousemoveX,this.mousemoveY),
                xPoint = draggedPoints.X,
                yPoint = draggedPoints.Y,
                xVal = this.commonDragEventArgs.data.currentPoint ? this.commonDragEventArgs.data.currentPoint.xValue : this.commonDragEventArgs.data.oldValue.X,
                yVal = this.commonDragEventArgs.data.currentPoint ? this.commonDragEventArgs.data.currentPoint.yValue[0] : this.commonDragEventArgs.data.oldValue.Y;
            if (chartSeries.region) {
                var region = chartSeries.region;                
                chart._dragDelta(xPoint, yPoint, xVal,yVal, region.SeriesIndex, region.Region.PointIndex);
            }
            else {
                var closestPoint = chartSeries.pointData;              
                chart._dragDelta(xPoint, yPoint, xVal, yVal,closestPoint.seriesIndex, closestPoint.index);
            }
            if (this.model.enableCanvasRendering) {
                if (this.canvasElement) this.canvasElement.clearRect(0, 0, chart.svgWidth, chart.svgHeight);
                if ($('#' + chart._id + '_PreviewSeries' + i + '_canvas').length == 0) {
                    var containerStyle = document.getElementById(chart._id + '_canvas').getBoundingClientRect();
                    canvasElement = this.createCanvasElement(chart._id + '_PreviewSeries' + i + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
                    this.createRect(chart, canvasElement);
                    this.canvasElement = canvasElement;
                }
            }
            else {
                chartSeries.xAxis = (chartSeries.xAxis === null || chartSeries.xAxis === undefined) ? this.model._axes[0] : chartSeries.xAxis;
                chartSeries.yAxis = (chartSeries.yAxis === null || chartSeries.yAxis === undefined) ? this.model._axes[1] : chartSeries.yAxis;
                var trans = ej.EjSvgRender.utils._getTransform(chartSeries.xAxis, chartSeries.yAxis, this.model.requireInvertedAxes);
                var translate = 'translate(' + trans.x + ',' + trans.y + ')';
                var serOptions = { 'id': this.svgObject.id + '_PreviewSeriesGroup', 'transform': translate, 'cursor': 'default', 'visibility': 'visible' };
                $("#" + this.svgObject.id + "_PreviewSeriesGroup").attr("transform", translate);
                if (!this.gPreviewSeriesGroupEle) {
                    this.chartObj = chart;
                    this.chartObj.gPreviewSeriesGroupEle = this.gPreviewSeriesGroupEle = this.svgRenderer.createGroup(serOptions);
                }              
                if (this.gPreviewSeriesGroupEle.childNodes.length > 0)
                    this.gPreviewSeriesGroupEle.removeChild(this.gPreviewSeriesGroupEle.childNodes[0]);
            }
                pointCollection = $.extend(true, {}, chartSeries._visiblePoints);         
                chartSeries.pointCollection = pointCollection;
                pointCollection.length = chartSeries._visiblePoints.length;
                var dragEventArgs = chart.commonDragEventArgs.data;
                var ptindex = dragEventArgs.pointIndex,
                    srindex = dragEventArgs.seriesIndex;
                chart._changeDraggingPoints(chartSeries, ptindex, dragEventArgs.newValue.X, dragEventArgs.newValue.Y);
               
                var series = new ej.seriesTypes[chartType](),
                    point = chartSeries.pointCollection[ptindex];              
                if (chartType == "bubble") {
                    var location = ej.EjSvgRender.utils._getPoint(point, chartSeries);
                    var options = {
                        'id': this.svgObject.id + '_PreviewSeries' + srindex + '_Point' + ptindex,
                        'cx': location.X + this.canvasX,
                        'cy': (location.Y) + this.canvasY,
                        'r': point.radius,
                        'fill': chartSeries.fill,
                        'stroke-width': 2,
                        'visibility': 'visible',
                        'stroke-dasharray': "",
                        'opacity': 0.6,
                        'stroke': 'transparent'
                    };
                    if (this.model.enableCanvasRendering)
                        ej.EjCanvasRender.prototype.drawCircle(options, canvasElement);
                    else
                        this.svgRenderer.drawCircle(options, this.gPreviewSeriesGroupEle);
                    this.svgRenderer.append(this.gPreviewSeriesGroupEle, this.gSeriesEle);
                }
                else if(chartType=="scatter"){
                    var location = ej.EjSvgRender.utils._getPoint(point, chartSeries);
                    var symbolName = (chartSeries.marker.shape).capitalizeFirstString();
                    var location = { startX: location.X + this.canvasX, startY: location.Y +this.canvasY };
                    var style = {
                        ShapeSize: { width: chartSeries.marker.size.width, height: chartSeries.marker.size.height },
                        ID: this.svgObject.id + '_PreviewSeries' + srindex + '_Point' + ptindex,
                        Style: {Color:chartSeries.fill,
                            BorderWidth: chartSeries.marker.border.width,
                            BorderColor: 'transparent',
                            Opacity: 0.6,
                            Visibility:'visible'
                        }
                    };
                    if (this.model.enableCanvasRendering)
                        ej.EjSvgRender.chartSymbol["_draw" + symbolName](location, style, this, canvasElement);
                    else
                        ej.EjSvgRender.chartSymbol["_draw" + symbolName](location, style, this,this.gPreviewSeriesGroupEle);
                    this.svgRenderer.append(this.gPreviewSeriesGroupEle, this.gSeriesEle);
                }                      
                            
                    else if (chartType == "column" || chartType == "bar") {
                        point.xValue = (typeof point.xValue=="object")?Date.parse(point.xValue):point.xValue;
                        var origin = ej.EjSeriesRender.prototype.getOrigin(chart, chartSeries, params);
                        var sidebysideinfo = chartSeries.sidebysideInfo;
                        var y1 = point.YValues[0];
                        var y2 = origin;
                        //calculate sides
                        var data = ej.EjSeriesRender.prototype.calculateSides(point, sidebysideinfo);
                        var x1 = data.x1;
                        var x2 = data.x2;
                        var rect = ej.EjSeriesRender.prototype.getRectangle(x1, y1, x2, y2, chartSeries, chart);
                        options = {
                            'id': this.svgObject.id + '_PreviewSeries' + srindex + '_Point' + ptindex,
                            'x': rect.X + this.canvasX,
                            'y': rect.Y + this.canvasY,
                            'width': rect.Width,
                            'height': rect.Height,
                            'cornerRadius': chartSeries.cornerRadius,
                            'fill': chartSeries.fill,
                            'stroke-width': 2,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': 0.6,
                            'stroke': 'transparent',
                            'stroke-dasharray': "",
                        };
                        if (this.model.enableCanvasRendering)
                            ej.EjCanvasRender.prototype.drawRect(options, canvasElement);
                        else {
                            this.svgRenderer.drawRect(options, this.gPreviewSeriesGroupEle);
                            this.svgRenderer.append(this.gPreviewSeriesGroupEle, this.gSeriesEle);
                        }
                    }
                    else
						{
						if(chartType=="spline" || chartType=="splinearea") 
							point.xValue = (typeof point.xValue == "object") ? Date.parse(point.xValue) : point.xValue;
                        series.draw(this, chartSeries, params);
						}
                var location = ej.EjSvgRender.utils._getPoint(point, chartSeries);
                if (chartSeries.tooltip.visible) {
                    if (ej.util.isNullOrUndefined(chartSeries.tooltip.template)) {                      
                        $(".tooltipDiv" + this._id).remove();
                        chart.displayShowTooltip(location, point, chartSeries, point.actualIndex);
                    }
                    else {                      
                        $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                        var region = { SeriesIndex: srindex, Region: { PointIndex: point.actualIndex } };
                        chart.createTooltip(region, evt, chartSeries);
                    }
                }           
    
},
     // to show marker symbol for line type charts in dragdrop
_dragPointMarker: function (chartSeries, currentX, currentY, evt, closestXyPoint) {   
             var location = null,            
             srIndex = chartSeries.seriesIndex;
             location = ej.EjSvgRender.utils._getPoint(closestXyPoint.point, chartSeries);
             if (this.model.enableCanvasRendering) {
                 if ($('#' + this.svgObject.id + '_Marker').length == 0) {
                     var containerStyle = document.getElementById(this.svgObject.id).getBoundingClientRect();
                     ctx = this.createCanvasElement(this.svgObject.id + '_Marker', this.svgWidth, this.svgHeight, containerStyle);
                 }
             }
             else {
                 var markerOptions = { 'id': this.svgObject.id + '_markerGroup' + '_' + srIndex };
                 if (!this.gMarkerGroupEle)
                     this.gMarkerGroupEle = this.svgRenderer.createGroup(markerOptions);
             }
             var options = {
                'id': this.svgObject.id + '_Series' + srIndex + '_Point' + closestXyPoint.index + '_Marker',
                'cx': location.X + this.model.m_AreaBounds.X,
                'cy': location.Y + this.model.m_AreaBounds.Y,
                'r': 7,            
                'fill': chartSeries.fill,           
                'stroke-width': 3,
                'opacity': 1,
                'stroke': 'white',
                'visibility':'visible'
            };
            if (this.model.enableCanvasRendering)
                ej.EjCanvasRender.prototype.drawCircle(options, ctx);
            else {
                this.svgRenderer.drawCircle(options, this.gMarkerGroupEle);
                this.svgRenderer.append(this.gMarkerGroupEle, this.gSeriesEle);
            }
    },
     // to show 3 circles for column and bar while hovering rect in dragdrop
    _dragPointGripper:function(data){
        var type = data.region.type.toLowerCase(),
	                 chartSeries = this.model._visibleSeries[data.region.SeriesIndex],
                     point = chartSeries._visiblePoints[data.region.Region.PointIndex];
            transposed = chartSeries.isTransposed,
            inversed = chartSeries.yAxis.isInversed;
        if (type == "column" || type == "bar") {
            if (this.model.enableCanvasRendering) {
                var containerStyle = document.getElementById(this.svgObject.id).getBoundingClientRect();               
                canvasGripper = this.createCanvasElement(this.svgObject.id + '_Marker', this.svgWidth, this.svgHeight, containerStyle);
                document.getElementById(canvasGripper.canvas.id).style.zIndex = 1;
            }
            else {
                var markerOptions = { 'id': this.svgObject.id + '_markerGroup' + '_' + data.region.SeriesIndex };
                if (!this.gMarkerGroupEle)
                    this.gMarkerGroupEle = this.svgRenderer.createGroup(markerOptions);
            }
            var circleLength = 3,
                bounds = data.region.Region.Bounds;
                if ((transposed && type == "bar") || (type == "column" && !transposed)) {
                    value = bounds.Width;
                    boundValue = bounds.X;
                }
                if ((!transposed && type == "bar") || (type == "column" && transposed)) {
                    value = bounds.Height;
                    boundValue = bounds.Y;
                }
                loc = value < 75 ? value - value / 3 : 75 - 25;
                diff = value > 75 ? value - 75 : 0;
                radius = (loc - 6) / 6;
                cxy = diff / 2 + boundValue + loc / 6 + radius / 2;
            for (var k = 1; k <= circleLength; k++) {
                var options = {
                    'id': this.svgObject.id + '_Series' + data.region.SeriesIndex + '_Point' + data.region.Region.PointIndex + '_Marker' + k,
                    'fill': 'white',
                    'opacity': 1,
                    'stroke': 'white',
                    'visibility': 'visible'
                };
                if (k == 1)
                    cxy += radius;
                else
                    cxy += 2 * radius + 3;
                options.r = Math.abs(radius);
                if ((transposed && type == "bar") || (type == "column" && !transposed)) {                                                        
                    options.cx = cxy;
                    options.cy = inversed ? bounds.Y + bounds.Height - radius - 5 : bounds.Y + radius + 5;
                 }                
                if ((!transposed && type == "bar") || (type == "column" && transposed)) {
                   options.cx = inversed? bounds.X + radius + 5: bounds.X + bounds.Width - radius - 5;
                   options.cy = cxy;
                }             
                if (this.model.enableCanvasRendering)
                    ej.EjCanvasRender.prototype.drawCircle(options, canvasGripper);
                else
                    this.svgRenderer.drawCircle(options, this.gMarkerGroupEle);
            }
            this.svgRenderer.append(this.gMarkerGroupEle, this.gSeriesEle);
        }
    },
chartMouseMove: function (evt) {    

    var data = this.GetSeriesPoint(evt),
        svgObject = this.svgObject,
        model = this.model, zooming = model.zooming.enable,
        enableCanvas = model.enableCanvasRendering;
        model.event = evt;    
		clearTimeout(this.removeTooltip); // to clear the set timeout used to remove the tooltip template
        if (this.doPan && evt.target.id.indexOf(this._id + '_scrollbar') > -1)
            this.doPan = false;
        var classList = evt.target.classList,
		    chartOffset = { left: 0, top: 0 };
        if (this.multiSelectAreaType == 'cartesianaxes' && (!zooming) && (!enableCanvas))
            this.multiSelectMouseMove(evt);         
        if (this.rotateActivate && this.oPreviousCoords) {
            var difY = (this.oPreviousCoords.Y - this.mousemoveY);
            var difX = (this.oPreviousCoords.X - this.mousemoveX);          
            if (difX||difY) {
                model.tilt -= difY;
                model.rotation += difX;     
                // store the class name 
                if(this.model.isSelected)
                  var selection = this.selectedStyle(this);
                if(!this.isTouch(evt))
                   $(this.chart3D).empty();
                $('#template_group_' + this._id).empty();                
                var size = { Width: $(svgObject).width(), Height: $(svgObject).height() };
                this.graphics.view(svgObject, this, model.rotation, model.tilt, size, model.perspectiveAngle, model.depth);
                this.svgRenderer.append(this.chart3D, svgObject);               
                // Apply the class
                if (this.model.isSelected)
                for (var i = 0; i < selection.length; i++) {
                    $('#' + selection[i].id).attr('class', selection[i].className);
                }
            }
            this.oPreviousCoords.Y = this.mousemoveY;
            this.oPreviousCoords.X = this.mousemoveX;
        }
		
		 var targetId = evt.target.id;
            svgObjectId = svgObject.id,
            isZoom = this.isZoomButtonHovered(evt.target),
            visibleSeries = model._visibleSeries,
            seriesLength = visibleSeries.length;
	     $("[id*=" + "_Marker" + "]").remove();
	     for (var i = seriesLength - 1; i >= 0; i--) {
	         chartSeries = visibleSeries[i];
	         type = chartSeries.type.toLowerCase();
	         if (chartSeries.dragSettings.enable && !model.enable3D) {	             
	             this._enableDragging(chartSeries, i, evt);
	             if (this.dragPoint)	              
	                 break;
	             mouseMoveCords = this.calMousePosition(evt);
	             this.mousemoveX = mouseMoveCords.X;
	             this.mousemoveY = mouseMoveCords.Y;
	             var regionData = this.GetSeriesPoint(evt);
	             if (!chartSeries.marker.visible && type != "bubble" && type != "scatter" && type != "column" && type != "bar") {
	                 var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY, evt);
	                 if (closestXyPoint.point) {
	                     chartSeries.seriesIndex = i;
	                     this._dragPointMarker(chartSeries, this.mousemoveX, this.mousemoveY, evt, closestXyPoint);
	                     this.closestXyPoint = closestXyPoint;
	                     break;
	                 }
	             }
	             if (regionData && regionData.region.SeriesIndex == i) {
	                 this._dragPointGripper(regionData);
	                 break;
	             }
	         }
	      }
	     
	     var chart = this;	  
	     if (this.dragPoint) {
	         var serX = [], serY = [], type;
			$(document).keyup(function (evt) {
	             if (evt.keyCode == 27) {	                 
	                 $(".tooltipDiv" + this._id).remove();
	                 $("#" + svgObjectId + "_TrackToolTip").remove();
	                 chart.dragPoint = false;
	                 visibleSeries[chart.commonDragEventArgs.data.seriesIndex].pointCollection = null;
	                 chart.chartMouseUp(evt);
	             }
	         });	        
	         var index = this.dragIndex.seriesIndex;
	         chart._pointDragandDrop(visibleSeries[index], evt, chart);
	         }	 
        
        this.calMousePosition(evt)
        $("#" + this._id + "_tooltip").remove();
         
        if (this.dragPoint ||( !this.drag && !isZoom && ((!this.isTouch(evt) && !model.doubleTapped) || model.crosshairMoved))) {
            this.chartInteractiveBehavior(evt);            
        }
        // Checked condition for series highlighting
        if ((!model.crosshair.visible || model.AreaType != "cartesianaxes")) {
            if (!enableCanvas || !data) {
                // removed highlight style
                $("[class*=" + "HighlightStyle" + "]").attr('class', '');
                $("[class*=" + "HighlightLegendStyle" + "]").attr('class', '');
                $("[class*=" + "HighlightLegendPathStyle" + "]").attr('class', '');
                $("[class*=" + "HighlightPathStyle" + "]").attr('class', '');
                $("[class*=" + "Highlightseries" + "]").attr('class', '');
                $("[id*=" + this._id + "_Highlight_" + "]").remove();// removed canvas highlight           
            }
            this.highlightStart(evt, data);
        }
        if (!this.panning) {     
            var id = "#" + svgObjectId;
            if (zooming && model.AreaType == "cartesianaxes" && !model.enable3D) {
                if (this.drag && !this.dragPoint && !this.chartRightClick) {
                    this.disableTrackBall();
                    $('#' + svgObjectId + '_ZoomArea').remove();
                    var width, height, x, y;
                    var currentX = this.mousemoveX,
                        currentY = this.mousemoveY,
                        areaBounds = model.m_AreaBounds;
                    if (this.mousemoveX < areaBounds.X) {
                        currentX = areaBounds.X;
                    } else if (this.mousemoveX > (areaBounds.X + areaBounds.Width)) {
                        currentX = areaBounds.X + areaBounds.Width;
                    } 
                    if (this.mousemoveY < areaBounds.Y) {                     
                        currentY = areaBounds.Y;
                    } else if (this.mousemoveY > (areaBounds.Y + areaBounds.Height)) {
                        currentY = areaBounds.Y + areaBounds.Height;
                    }
                    else {
                        currentY = this.mousemoveY;
                    }
                   var zoomingType = model.zooming.type.toLowerCase();
                   if (zoomingType == "x") {
                        width = Math.abs(currentX - this.mouseDownX);
                        height = areaBounds.Height;
                        x = currentX > this.mouseDownX ? this.mouseDownX : currentX;
                        y = areaBounds.Y;
                   } else if (zoomingType == "y") {
                        width = areaBounds.Width;                      
                        height = Math.abs(currentY - this.mouseDownY);
                        x = areaBounds.X;
                        y = currentY > this.mouseDownY ? this.mouseDownY : currentY;

                    } else {                        
                        width = Math.abs(currentX - this.mouseDownX);
                        height = Math.abs(currentY - this.mouseDownY);
                        x = currentX > this.mouseDownX ? this.mouseDownX : currentX;
                        y = currentY > this.mouseDownY ? this.mouseDownY : currentY;

                    }                 
                    $(svgObject).css({
                        '-moz-user-select': '-moz-none',
                        '-khtml-user-select': 'none',
                        '-webkit-user-select': 'none',
                        '-ms-user-select': 'none',
                        'user-select': 'none'
                    });

                    this.zooming = true;

                    var rectOptions = {
                        'id': svgObjectId + '_ZoomArea',
                        'x': x + chartOffset.left,
                        'y': y + chartOffset.top,
                        'width': width,
                        'height': height,
                        'fill': 'rgba(69,114,167,0.25)',
                        'stroke-width': 1,
                        'stroke': 'rgba(69,114,167,0.25)',
                        'clip-path': 'url(#' + svgObjectId + '_ChartAreaClipRect)'
                    };
                    if (enableCanvas)
                        this.svgRenderer.drawZoomRect(rectOptions, this);
                    else
                        this.svgRenderer.drawRect(rectOptions, svgObject);
                }

            }
        }
		var deferredZoom = model.zooming.enableDeferredZoom;
        // Panning
		if (this.panning && this.doPan && model.AreaType == "cartesianaxes" && !this.dragPoint) {
            this.currentPageX = evt.pageX;
            this.currentPageY = evt.pageY;
            $("#" + this._id + "_canvas").css({ "cursor": "pointer" });
            if (!ej.isTouchDevice() && !deferredZoom) {
                var oDelta;
                oDelta = {
                    'x': this.oPreviousCoords.x - evt.pageX,
                    'y': this.oPreviousCoords.y - evt.pageY
                };

                this.oPreviousCoords = {
                    'x': evt.pageX,
                    'y': evt.pageY
                };
                $.each(model._axes, function(index, axis) {
                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
                });
                this._updateScroll();
                this.redraw(true, null, evt.target, this.isTouch(evt));
                this._cursorToPointer();
                this._enableZoomingButtons();
            }
            // Translate the series for other devices
            else if (ej.isTouchDevice() || deferredZoom) {
                for (var k = 0; k <visibleSeries.length; k++) {
                    var series = model.series[k],
                    transform = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, model.requireInvertedAxes),
                    transX = (evt.pageX - this.oPreviousCoords.x) + transform.x,
                    transY = (evt.pageY - this.oPreviousCoords.y) + transform.y,
                    translate = 'translate(' + transX + ',' + transY + ')';
                   
                    $("#" + svgObjectId + "_SeriesGroup_" + k).attr("transform", translate);
                    $("#" + svgObjectId + "_symbolGroup_" + k).attr("transform", translate);
                    $("#" + svgObjectId + "_TextGroup_" + k).attr("transform", translate);
                    
                }
                for (var index = 0; index < model.indicators.length && model.indicators[index]._points.length>0; index++) {
                    var indicator = model.indicators[index];
                    transform = ej.EjSvgRender.utils._getTransform(indicator.xAxis, indicator.yAxis, model.requireInvertedAxes);
                    transX = (evt.pageX - this.oPreviousCoords.x) + transform.x;
                    transY = (evt.pageY - this.oPreviousCoords.y) + transform.y;
                    translate = 'translate(' + transX + ',' + transY + ')';
                    $("#" + svgObjectId + "_indicatorGroup_" + index).attr("transform", translate);
                }
            }
        }
		
		//Declaration
            var targetid = evt.target.id;
            var id = this._id;
            var parentId = ej.util.isNullOrUndefined(evt.target.parentNode) ? "" : evt.target.parentNode.id;
			
	//condition to find the buttons and display tooltip on hover
		if (zooming) {
            if (parentId == id + '_svg_ResetZoom')
                createBtnTooltip(this._localizedLabels.reset);
            else if (parentId == id + '_svg_PanBtn')
                createBtnTooltip(this._localizedLabels.pan);
            else if (parentId == id + '_svg_ZoomBtn')
                createBtnTooltip(this._localizedLabels.zoom);
            else if (parentId == id + '_svg_ZoomInBtn')
                createBtnTooltip(this._localizedLabels.zoomIn);
            else if (parentId == id + '_svg_ZoomOutBtn')
                createBtnTooltip(this._localizedLabels.zoomOut);
		}
	// method to create tooltip for zooming and panning buttons
        function createBtnTooltip(text) {
            var id = chart._id;
            if ($("#" + id + "_tooltip").length <= 0) {
                var tooltipdiv = $("<div></div>").attr({ 'id': id + "_tooltip", "class": "zoom" + id});
                $(tooltipdiv).html("&nbsp" + text + "&nbsp");
                $(document.body).append(tooltipdiv);
                // adding css prop to the div
                $(tooltipdiv).css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10,
                    "display": "block",
                    "position": "absolute",
                    "z-index": "13000",
                    "cursor": "default",
                    "font-family": "Segoe UI",
                    "color": "#707070",
                    "font-size": "12px",
                    "background-color": "#FFFFFF",
                    "border": "1px solid #707070"
                });
            } else {
                $("#" + id + "_tooltip").css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10
                });
            }

        }
        // axisLabelMouseMove event
        var axisData = this.getAxisLabelData(evt);
        if (axisData) {
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = axisData;
            this._trigger("axisLabelMouseMove", commonEventArgs);
        }
          
        if (!this.isTouch(evt)) {
            this.axisTooltip(evt, targetid);
              if (model.title.enableTrim)
					this.showTitleTooltip(evt, targetid);
            for (var i = 0, seriesLength = model.series.length; i < seriesLength; i++) {
                var series = model.series[i], dataLabel = series.marker.dataLabel;
                if (series._enableSmartLabels && !this.model.enable3D && !dataLabel.enableWrap) {
                    var template = dataLabel.template;
                    var font = dataLabel.font;
                    if (model.AreaType == "none" && ej.util.isNullOrUndefined(template)) {
                        this.datalabelTooltip(evt, i, font);
                    }
                }
            }
        }
      
            
        
        if ($(svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').length > 0) {
            $(svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').appendTo(svgObject);
        }

        var legenddata = this.getLegendData(evt);
        if (legenddata) {
            var commonLegendMoveEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLegendMoveEventArgs.data = legenddata;
            if (legenddata.series.highlightSettings.enable) {
                if (enableCanvas) {
                    var series = $.extend({}, legenddata.series);
                    series.seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
                    series.pointIndex = legenddata.legendItem.LegendItem.PointIndex;
                    var index = this.model.AreaType == 'none' ? series.pointIndex : series.seriesIndex;
                    if ($('#' + chart._id + '_Selection_Legend' + index + '_canvas').length == 0) {
                        //remove canvas legend highlight 
                        $("[id*=" + this._id + "_Highlight_" + "]").remove();
                        this.canvasHighlight(chart, evt, series, legenddata);                
                    }
                }
                else
                    this.highlight(chart, evt, legenddata.series, legenddata);
            }
            this._trigger("legendItemMouseMove", commonLegendMoveEventArgs);
            if (enableCanvas) {
                var mouseMoveCords = this.calMousePosition(evt),
                    mousemoveX = mouseMoveCords.X,
                    mousemoveY = mouseMoveCords.Y,
                    legendBounds = model.LegendBounds;
                    $(this.legendContainer).children().css({ "cursor": "pointer" });
                    if ((mousemoveX >= legendBounds.X) && (mousemoveX <= legendBounds.X + model.LegendViewerBounds.Width)) {
                        if ((mousemoveY <= legendBounds.Y + (model.LegendViewerBounds.Height)) && (mousemoveY >= legendBounds.Y)) {
                             this._textTooltip(evt, model.legendTextRegion);
                        }
                    }
            }
            else
                this._textTooltip(evt, model.legendTextRegion);
        }
        else if (enableCanvas)
            $(this.legendContainer).children().css('cursor', 'default');
        if (enableCanvas) {

            //condition to find the buttons and display tooltip on hover for Canvas
            if (targetid.indexOf("_ResetZoom") != -1 || parentId.indexOf("_ResetZoom") != -1)
                createBtnTooltip(this._localizedLabels.reset);
            else if (targetid.indexOf("_PanBtn") != -1 || parentId.indexOf("_PanBtn") != -1)
                createBtnTooltip(this._localizedLabels.pan);
            else if (targetid.indexOf("_ZoomBtn") != -1 || parentId.indexOf("_ZoomBtn") != -1)
                createBtnTooltip(this._localizedLabels.zoom);
            else if (targetid.indexOf("_ZoomInBtn") != -1 || parentId.indexOf("_ZoomInBtn") != -1)
                createBtnTooltip(this._localizedLabels.zoomIn);
            else if (targetid.indexOf("_ZoomOutBtn") != -1 || parentId.indexOf("_ZoomOutBtn") != -1)
                createBtnTooltip(this._localizedLabels.zoomOut);
            if (!this.isTouch(evt)) {
                this._textTooltip(evt, model.xAxisLabelRegions);
                this._textTooltip(evt, model.yAxisLabelRegions);
            }
        }
        
        var proxy = this;
        if (model.xAxisTitleRegion) {
            var currentX = this.mousemoveX,
                currentY = this.mousemoveY;
            $.each(model.xAxisTitleRegion, function (index, regionItem) {
                if ((currentX >= regionItem.Bounds.X) && (currentX <= regionItem.Bounds.X + regionItem.Bounds.Width)) {
                    if ((currentY >= regionItem.Bounds.Y - (regionItem.Bounds.Height)) && (currentY <= regionItem.Bounds.Y)) {
					    if (regionItem.trimText != regionItem.labelText) {
                        proxy.showAxisTooltip(evt.pageX, evt.pageY, regionItem.labelText);
					   }
                    }
                }
            });
        }
        if (model.yAxisTitleRegion) {
            var currentX = this.mousemoveX,
                currentY = this.mousemoveY;
            $.each(model.yAxisTitleRegion, function (index, regionItem) {
                 if ((currentX >= regionItem.Bounds.X) && (currentX <= regionItem.Bounds.X + regionItem.Bounds.Width)) {
                    if ((currentY >= regionItem.Bounds.Y - (regionItem.Bounds.Height)) && (currentY <= regionItem.Bounds.Y)) {
					    if (regionItem.trimText != regionItem.labelText) {
                        proxy.showAxisTooltip(evt.pageX, evt.pageY, regionItem.labelText);
						}
                    }
                }
            });
        }
        if (model.axisMultiLevelLabelRegions) {
            var currentX = this.mousemoveX,
                currentY = this.mousemoveY;
            $.each(model.axisMultiLevelLabelRegions, function (index, regionItem) {
                if ((currentX >= regionItem.Bounds.X) && (currentX <= regionItem.Bounds.X + regionItem.Bounds.Width)) {
                    if ((currentY >= regionItem.Bounds.Y - (regionItem.Bounds.Height)) && (currentY <= regionItem.Bounds.Y)) {
                        if (regionItem.trimText != regionItem.labelText) {
                            proxy.showAxisTooltip(evt.pageX, evt.pageY, regionItem.labelText);
                        }
                    }
                }
            });
        }

	   //chartMouseMove event
	   var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
	   commonEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY },size:{height:model.svgHeight,width:model.svgWidth}, id: evt.target.id, pageX: evt.pageX, pageY: evt.pageY };
	   this._trigger("chartMouseMove", commonEventArgs);
    },

    axisTooltip: function (evt, targetid) {
        var svgId = this.svgObject.id,
          hAxes = this.model.hAxes,
         vAxes = this.model.vAxes;

        if (targetid.indexOf("_YLabel_") >= 0) {
            var axis;
            // Loop to find the axis
            for (var j = 0; j < vAxes.length; j++) {
                if (targetid.indexOf(svgId + '_' + vAxes[j].name + '_YLabel_') >= 0) {
                    axis = vAxes[j];
                    break;
                }
            }
            if (axis && axis.enableTrim) {
                // Loop to find the labels in the axis
                visibleLabelsLength = axis.visibleLabels.length;
                for (var i = 0; i < visibleLabelsLength && this.model.AreaType == 'cartesianaxes'; i++) {
                    if (targetid == svgId + '_' + vAxes[j].name + '_YLabel_' + i) {
                        var ele = axis.visibleLabels[i];
                        var str = ele.Text == ele.displayText ? "" : ((axis.labelFormat) ? axis.labelFormat.replace("{value}", ele.Text) : ele.Text);
                        if (str == "") break;
                        if (axis.labelPosition.toLowerCase() == 'inside') {
                            this._hideTooltip();
                            this._removeTrackBall();
                        }
                        this.showAxisTooltip(evt.pageX, evt.pageY, str);
                    }
                }
            }
        }

        if (targetid.indexOf("_XLabel_") >= 0) {
            var axis;
            // Loop to find the axis
            for (var j = 0; j < hAxes.length; j++) {
                if (targetid.indexOf(svgId + '_' + hAxes[j].name + '_XLabel_') >= 0) {
                    axis = hAxes[j];
                    break;
                }
            }
            if (axis && (axis.enableTrim || axis.labelIntersectAction.toLowerCase() == "trim" || axis.labelIntersectAction.toLowerCase() == "wrap" || axis.labelIntersectAction.toLowerCase() == "wrapbyword")) {
                // Loop to find the labels in the axis
                visibleLabelsLength = axis.visibleLabels.length;
                for (var i = 0; i < visibleLabelsLength && this.model.AreaType == 'cartesianaxes'; i++) {
                    if (targetid == svgId + '_' + hAxes[j].name + '_XLabel_' + i) {
                        var ele = axis.visibleLabels[i];
                        var str = ele.Text == ele.displayText ? "" : ele.Text;
                        if (str == "") break;
                        if (axis.labelPosition.toLowerCase() == 'inside') {
                            this._hideTooltip();
                            this._removeTrackBall();
                        }
                        this.showAxisTooltip(evt.pageX, evt.pageY, str);
                    }
                }
            }
        }
    },

    showTitleTooltip: function (evt, targetid) {
        var measureTitle = ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font), model=this.model,
         titleLocation = model._titleLocation,
         subTitleLocation = model._subTitleLocation,
         titleEnableTrim = model.title.enableTrim,
         elementSpacing = model.elementSpacing,
         currentX = this.mousemoveX, currentY = this.mousemoveY,
         titleTextOverflow = model.title.textOverflow.toLowerCase(),
         subTitleEnableTrim = model.title.subTitle.enableTrim,
         subTitleTextOverflow = model.title.subTitle.textOverflow.toLowerCase(),
         titleText = model.title.text, subTitleText = model.title.subTitle.text;
        if (titleEnableTrim && this.model.trimTooltip) {
            if (targetid.indexOf("ChartTitleText") >= 0)
                this.showAxisTooltip(evt.pageX, evt.pageY, titleText);
            else if (this.model.enableCanvasRendering && currentX >= titleLocation.X - elementSpacing && currentX <= titleLocation.X - elementSpacing + titleLocation.size.width &&
               currentY >= (titleLocation.Y - (titleLocation._height)) && currentY <= titleLocation.Y + titleLocation.size.height)
                this.showAxisTooltip(evt.pageX, evt.pageY, titleText);
        }

        if (subTitleEnableTrim && model.subTitleTooltip) {
            if (targetid.indexOf("ChartSubTitleText") >= 0)
                this.showAxisTooltip(evt.pageX, evt.pageY, subTitleText);
            else if (model.enableCanvasRendering && currentX >= subTitleLocation.X && currentX <= subTitleLocation.X + subTitleLocation.size.width &&
                   currentY >= (subTitleLocation.Y) && currentY <= (subTitleLocation.Y) + subTitleLocation.size.height)
                this.showAxisTooltip(evt.pageX, evt.pageY, subTitleText);
        }
    },
     
    _updateScroll: function () {

        var scrollObj = this.model.scrollObj,
            scrollLength = scrollObj ? scrollObj.length : 0,
            axes = this.model._axes,
            scroll = this.scrollbarContainer, obj;
        for (var i = 0; i < scrollLength; i++) {
            if (scrollObj[i]) {
                obj = scrollObj[i];
               if (obj.zoomPosition != axes[i].zoomPosition || obj.zoomFactor != axes[i].zoomFactor) {
                    obj.zoomPosition = axes[i].zoomPosition;
                    obj.zoomFactor = axes[i].zoomFactor;
                    scroll._initializeScrollbarVariables(obj);
                    scroll._scrollbarUpdate.call(this, obj);
                    scroll._setScrollPosition.call(this, obj.startX, obj.startX + obj.rectWidth, obj);
                }
            }
        }
    },

    _hideTooltip: function () {
        $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
        $("#" + this.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden').hide();
        $('.tooltipDiv' + this._id).css("display", "none");
    },
    _removeHighlight: function () {
        $("[id*=" + "HighlightSegment" + "]").remove();
        $("[id*=" + "HighlightPath" + "]").remove();
        $("[id*=" + "HighlightLegendSegment" + "]").remove();
        $("[id*=" + "HighlightLegendPath" + "]").remove();
        $("[id*=" + "HighlightDef" + "]").remove();
	     $("[id*=" + "Highlightseries" + "]").remove();
	     $("[id*=" + this._id + "_Highlight_" + "]").remove();
	     if (!this.model.enableCanvasRendering && !this.vmlRendering) {
	         var highlightedElement = this.svgObject.querySelector("[class^='Highlight']");
	         if (highlightedElement !== null && highlightedElement.length == null)
	             highlightedElement.removeAttribute("class");
	         else
	             for (var j = highlightedElement && highlightedElement.length; j--;)
	                 highlightedElement.removeAttribute("class");
	     }
    },
    _removeSelection: function () {
        $("[id*=" + this._id + '_SelectionCluster_Legend' + "]").remove();
        $("[id*=" + this._id + '_Selection_series' + "]").remove();
        $("[id*=" + this._id + '_Selection_Cluster' + "]").remove();
        $("[id*=" +this._id + "SelectionSegment" + "]").remove();
        $("[id*=" +this._id + "SelectionPath" + "]").remove();
        $("[id*=" +this._id + "SelectionLegend" + "]").remove();
        $("[id*=" +this._id + "SelectionDef" + "]").remove();
	     $("[id*=" + this._id +"Selectionseries" + "]").remove();
        $('[id*=' + this._id + '_Selection_' + ']').remove();
    },

     
     //tooltip for trimmed datalabel
    datalabelTooltip:function(evt, i, font){
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;
		var containerId = this._id;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var vmlRendering = this.vmlRendering;
        var space = this.model.elementSpacing;
        var chart = this;
        var minX, minY, maxX, maxY;
        var tooltipdiv, tooltipLength;
        var remaining
		  if(!ej.util.isNullOrUndefined(this.accDataLabelRegion[i]))
        {
      $.each(this.accDataLabelRegion[i], function (index, regionItem) {
            minX = regionItem.bounds.minX;
            minY = regionItem.bounds.minY;
            maxX = regionItem.bounds.maxX;
            maxY = regionItem.bounds.maxY;            
            if (vmlRendering) {
                minX = regionItem.bounds.minX + (regionItem.bounds.width / 2);
                maxX = regionItem.bounds.maxX + (regionItem.bounds.width / 2);
                minY = regionItem.bounds.minY;
                maxY = regionItem.bounds.maxY;
            }
            if (currentX >= minX && currentX <= maxX) {
                if (currentY >= minY && currentY <= maxY) {
                    $("#" + containerId + "_tooltip").remove();
                    tooltipdiv = $("<div></div>").attr({ 'id': containerId + "_tooltip", 'class': 'ejTooltip' + containerId });                   
                    if (regionItem.trimmedText != regionItem.text) {
                     $("#"+chart.svgObject.id+"_TrackToolTip").hide();
                        $(tooltipdiv).html(regionItem.text);
                        $(document.body).append(tooltipdiv);
                        tooltipLength = ej.EjSvgRender.utils._measureText(regionItem.text, regionItem.text.length, regionItem.font).width;
                        $(tooltipdiv).css({
                            "top": evt.pageY + space,
                            "display": "block",
                            "position": "absolute",
                            "z-index": "13000",
                            "cursor": "default",
                            "color": "#000000",
                            "font-size": font.size,
                            "background-color": "#FFFFFF",
                            "border": "1px solid #707070",
                            "white-space": "nowrap",
                            "padding": "5px"
                        });
                        if (regionItem.type == "pyramid" || regionItem.type == "funnel") {
                            remaining = chart.model.m_AreaBounds.Width + chart.model.m_AreaBounds.X - evt.pageX;
                            if (remaining > tooltipLength)
                                $(tooltipdiv).css({ "left": evt.pageX + space });
                            else {
                                left = evt.pageX - tooltipLength+(2*space);
                                $(tooltipdiv).css({ "left": left  });

                            }
                        }
                        if (regionItem.type == "pie" || regionItem.type == "doughnut" || regionItem.type == "pieofpie") {
                            if (minX > regionItem.bounds.centerX) {
                                remaining = chart.svgWidth - chart.model.margin.left - chart.model.margin.right - currentX;
                                if (remaining < tooltipLength) {
                                    left = evt.pageX - tooltipLength;
                                    $(tooltipdiv).css({ "left": left + space });
                                }
                                else
                                    $(tooltipdiv).css({ "left": evt.pageX + space });
                            }
                            else {
                                remaining = currentX;
                                if (remaining < tooltipLength) {
                                    $(tooltipdiv).css({ "left": evt.pageX + space });
                                }
                                else {
                                    left = evt.pageX - tooltipLength;
                                    $(tooltipdiv).css({ "left": left - (2 * space)});
                                }
                            }
                        }                       

                    }
                }
            }
        });
		}
    },
    // Tooltip in svg 
      showAxisTooltip: function (pageX, pageY, str) {
	        var id = this._id;
	        var tooltipdiv = $("<div></div>").attr({ 'id': id + "_tooltip", 'class': 'ejTooltip' + id });
            $(tooltipdiv).html("&nbsp" + str + "&nbsp");
            $(document.body).append(tooltipdiv);
            // adding css prop to the div
            $(tooltipdiv).css({
                "left": pageX + 10,
                "top": pageY + 10,
                "display": "block",
                "position": "absolute",
                "z-index": "13000",
                "cursor": "default",
                "font-family": "Segoe UI",
                "color": "#707070",
                "font-size": "12px",
                "background-color": "#FFFFFF",
                "border": "1px solid #707070"
            });
        },
    _textTooltip: function(evt,region){
		var chart = this,
            mouseMoveCords = this.calMousePosition(evt),
            isEjScroll = this.model.legend._ejScroller,
            scrolltop = isEjScroll ? $("#legend_" + this._id).ejScroller('instance').model.scrollTop : $("#legend_" + this._id).scrollTop(),
            scrollleft = isEjScroll ? $("#legend_" + this._id).ejScroller('instance').model.scrollLeft : $("#legend_" + this._id).scrollLeft(),    
            mousemoveX = mouseMoveCords.X,
            mousemoveY = mouseMoveCords.Y,
            currentX = mousemoveX,
            currentY = mousemoveY,
            containerId = this._id,
            chartModel = chart.model,
            legend = chartModel.legend,
            isScroll = legend.enableScrollbar,
            textOverflow = legend.textOverflow.toLowerCase(),
            displayText, tooltipdiv;
        
            if (isScroll || isEjScroll) {
                currentX = mousemoveX + scrollleft;
                currentY = mousemoveY + scrolltop;
            }
           
        $.each(region, function (index, regionItem) {             
            
            if ((currentX >= regionItem.bounds.x) && (currentX <= regionItem.bounds.x + regionItem.bounds.width)) {
                if ((currentY >= regionItem.bounds.y - (regionItem.bounds.height/3)) && (currentY <= regionItem.bounds.y)) {                    
                   
                    $("#" + containerId + "_tooltip").remove();
                    tooltipdiv = $("<div></div>").attr({ 'id': containerId + "_tooltip", 'class': 'ejTooltip' + containerId });
                    displayText = jQuery.type(regionItem.trimText) == "array" ? regionItem.trimText.join(" ") : regionItem.trimText;
                    if (displayText != regionItem.labelText) {
                            $(tooltipdiv).html("&nbsp" + regionItem.labelText + "&nbsp");
                            $(document.body).append(tooltipdiv);
                            // adding css prop to the div
                            $(tooltipdiv).css({
                                "left": evt.pageX + 10,
                                "top": evt.pageY + 10,
                                "display": "block",
                                "position": "absolute",
                                "z-index": "13000",
                                "cursor": "default",
                                "font-family": "Segoe UI",
                                "color": "#707070",
                                "font-size": "12px",
                                "background-color": "#FFFFFF",
                                "border": "1px solid #707070"
                            });
                    }       
                }
            }
        });
    },
    pieExplosion: function (data) {
	  this.model.explode = true;
        var region = data;
        var id;
        var targetElement;
        var symbolName;
        var series = region.region.Series;
        var seriesIndex = region.region.SeriesIndex;
        var seriesType = new ej.seriesTypes[series.type.toLowerCase()]();
        var explodePoint = series.explodeIndex;
        var _labelPosition = series.labelPosition.toLowerCase();
        currentExplodePoint = { SeriesIndex: region.pointData[0].SeriesIndex, PointIndex: region.pointData[0].PointIndex };
        var pieActualIndex = this.model._isPieOfPie ? region.pointData[0].PointIndex : region.region.Series._visiblePoints[region.pointData[0].PointIndex].actualIndex;
        if (explodePoint != pieActualIndex && !region.region.Series.explodeAll) {           
            series.explodeIndex = pieActualIndex;
            pieSeriesIndex = region.pointData[0].PieSeriesIndex;
            if (!ej.util.isNullOrUndefined(this.model.explodeValue)&& this.model.explodeValue != currentExplodePoint) {
                if (this.model.explodeValue.SeriesIndex != region.region.SeriesIndex)
                    series = this.model._visibleSeries[this.model.explodeValue.SeriesIndex];
                var visiblePoint = this.model._isPieOfPie? this._getPieOfPiePoint(this.model.explodeValue.PointIndex, series):series._visiblePoints[this.model.explodeValue.PointIndex];
                var result = seriesType._calculateArcData(this, this.model.explodeValue.PointIndex, visiblePoint, series, this.model.explodeValue.SeriesIndex,this.model.explodeValue.PieSeriesIndex);
                id = this.svgObject.id + '_SeriesGroup' + '_' + this.model.explodeValue.SeriesIndex;
                
                targetElement = (this.model.explodeValue.PieSeriesIndex == 1) ? $(this.gSeriesEle).children('#' + id)[1] : $(this.gSeriesEle).children('#' + id)[0];
                var elements = $(targetElement).children();
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                    if (parseInt(index) == this.model.explodeValue.PointIndex) {
                        this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                        break;
                    }
                }

                seriesType.drawDataLabelAcc(this, series, this.model.explodeValue.PointIndex, visiblePoint, this.model.explodeValue.SeriesIndex, this.model.explodeValue.PieSeriesIndex)
                
            }
           
            var seriesPoint = this.model._isPieOfPie ? this._getPieOfPiePoint(data.region.Region.PointIndex, series) : series._visiblePoints[region.pointData[0].PointIndex];
            
            result = seriesType._calculateArcData(this, region.pointData[0].PointIndex, seriesPoint, region.region.Series, seriesIndex, pieSeriesIndex);
           
            id = this.svgObject.id + '_SeriesGroup' + '_' + region.region.SeriesIndex;
            targetElement = pieSeriesIndex == 1? $(this.gSeriesEle).children('#' + id)[1]:$(this.gSeriesEle).children('#' + id)[0];
          
          
            var elements = $(targetElement).children();

            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                if (parseInt(index) == region.pointData[0].PointIndex) {
                    this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                    break;
                }
            }
            if (this.model.enableCanvasRendering) {
                var chartRect = document.getElementById(this.svgObject.id).getClientRects()[0];
                this.svgRenderer.ctx.clearRect(chartRect.left, chartRect.top, chartRect.width, chartRect.height);
                $("#" + this._id).ejChart("redraw");
            }
            
            
            seriesType.drawDataLabelAcc(this, series, region.pointData[0].PointIndex, seriesPoint, seriesIndex,pieSeriesIndex)
         
           
            this.model.explodeValue = { SeriesIndex: region.pointData[0].SeriesIndex, PointIndex: region.pointData[0].PointIndex, PieSeriesIndex:region.pointData[0].PieSeriesIndex };
        }
        region.region.Series.explodeIndex = explodePoint;
    },
    displayTemplateTooltip: function (x, y, location, seriesPoint, series, serIndex, ptIndex, chartLoc) {
        var point = $.extend(true, {}, seriesPoint);
        var padding = 5;
        var locale = this.model.locale;
        point.x = point.X;
        if (series.xAxis._valueType.toLowerCase() == "datetime")
            point.x = (ej.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat), locale));
        if (series.xAxis._valueType.toLowerCase() == "category") point.x = ej.EjSvgRender.utils._getLabelContent(ptIndex, series.xAxis, locale);
        // x and y is set to 0, if undefined
        x = ej.util.isNullOrUndefined(x) ? 0 : x;
        y = ej.util.isNullOrUndefined(y) ? 0 : y;
        var templateLocX = this.model.m_AreaBounds.X + chartLoc.left;
        var xLoc = location.X + x + templateLocX + this.model.crosshair.marker.size.width + padding;
        var yLoc = location.Y + y + this.model.m_AreaBounds.Y + chartLoc.top;
        var tooltipdiv;
        if ($("#" + this.svgObject.id + "_TrackToolTipTemplate_" + serIndex).length == 0) {
            tooltipdiv = $("<div class='TrackToolTipTemplate' style='position: absolute; z-index: 13000; display: block;'></div>");            
            $(document.body).append(tooltipdiv);
        }
        else {
            $(document).find('[id*="_TrackToolTipTemplate_"]').css("display", "block");
            tooltipdiv = $("#" + this.svgObject.id + "_TrackToolTipTemplate_" + serIndex)
            tooltipdiv.empty();
        }
        
        this.svgRenderer._setAttr($(tooltipdiv), { "id": this.svgObject.id + "_TrackToolTipTemplate_" + serIndex });
        if (this.model.crosshair.tooltipTemplate != null) {            
            var cloneNode = $("#" + this.model.crosshair.tooltipTemplate).clone();
            $(cloneNode).css("display", "block").appendTo(tooltipdiv);
            series.count = 1;
            point.count = 1;
            var seriesColor = this.getSeriesColor(point, serIndex, series);
            $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
            var data = { series: series, point: point };
            $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
        }
        yLoc -= parseFloat($(tooltipdiv).css('height')) / 2;
        var templateWidth = parseFloat($(tooltipdiv).css('width'));
        if (xLoc + templateWidth > templateLocX + this.model.m_AreaBounds.Width)
            xLoc = location.X + x + templateLocX - this.model.crosshair.marker.size.width - padding - templateWidth;
        $(tooltipdiv).css("left", xLoc);
        $(tooltipdiv).css("top", yLoc);
    },

    displayAxisTooltip: function (location, text, axis, index, mouseLoc, tracker) {

        if (axis._valueType.toLowerCase() == "double") {
            var customFormat = (!(axis.labelFormat)) ? null : axis.labelFormat.match('{value}');
            text = (!(axis.labelFormat)) ? text : (customFormat != null) ? (axis.labelFormat == "${value}") ? axis.labelFormat.replace('{value}', '$' + Number(text)) : axis.labelFormat.replace('{value}', Number(text)) : (ej.format(Number(text), axis.labelFormat, this.model.locale));
        }
        var maxTickSize = 0,
            orientation = axis.orientation.toLowerCase(), position,
            hPadding = 0,
            vPadding = 0,
            opposedPosition = axis._opposed,
            enableCanvas = this.model.enableCanvasRendering,
            maxLocation = opposedPosition ? ((maxTickSize + 10) + (axis.x + axis.width)) : (axis.x + axis.width);
        for (var i = 0; i < this.model._axes.length; i++) {
            if (this.model._axes[i].majorTickLines.size > maxTickSize) {
                maxTickSize = this.model._axes[i].majorTickLines.size;
            }
        }
        
        if (location.X >= 0 && location.X <= maxLocation) {
            var x = (location.X);
            var y = location.Y;
            var padding = 5;
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { axisIndex: index, chartAxis: axis, currentTrackText: text, location: mouseLoc };
            this._trigger("trackAxisToolTip", commonTrackTextArgs);

            var trackAxisText = commonTrackTextArgs.data.currentTrackText;

            var textOffset = ej.EjSvgRender.utils._measureText(trackAxisText, null, axis.crosshairLabel.font);
            if (orientation == 'horizontal') {
                x = x - textOffset.width / 2;
                if (axis.labelPosition == 'inside' || (opposedPosition ? mouseLoc.y < axis.y : mouseLoc.y > axis.y)) {
                    if (opposedPosition == false) {
                        y = axis.y - textOffset.height + maxTickSize - (axis._isScroll ? this.model.scrollerSize : 0);
                        position = "top";
                    }
                    if (opposedPosition == true) {
                        y = axis.y + textOffset.height - maxTickSize;
                        position = "bottom";
                    }
                }
                else{                   
                    if (opposedPosition == true) {
                        y = axis.y - textOffset.height / 2 - padding - maxTickSize;
                        position = "bottom";
                    }
                    if (opposedPosition == false) {
                        y = axis.y + textOffset.height - padding + maxTickSize;
                        position = "top";
                    }
                }
                hPadding = padding;
                y = y + (axis._isScroll ? (axis.opposedPosition ? -this.model.scrollerSize : this.model.scrollerSize) : 0);
            }
            if (!(opposedPosition) && (x + textOffset.width) > (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) {
                x = ((this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) - textOffset.width) + padding + this.model.elementSpacing;
            }
            if (orientation == 'vertical') {
			     y = location.Y + textOffset.height / 4;
			     if (axis.labelPosition == 'inside' || (!opposedPosition && mouseLoc.x < axis.x)) {
                    if (opposedPosition == true) {
                        x = axis.x - textOffset.width +  padding - maxTickSize;
                        position = "left";
                    }
                    if (opposedPosition == false) {
                        // Space between the marker and the trackball
                        x = axis.x + 2 * padding + (axis._isScroll ? this.model.scrollerSize : 0);
                        position = "right";
                    }
                } else {
			         if (opposedPosition == true) {
			             // Space between the marker and the trackball
                        x = axis.x + 3 * padding;
                        position = "left";
                    }
                    if (opposedPosition == false) {
                        x = axis.x - textOffset.width - maxTickSize;
                        position = "right";
                    }
                }                
			     x = x + (axis._isScroll ? (axis.opposedPosition ? this.model.scrollerSize : -this.model.scrollerSize) : 0);
			     vPadding = padding;
            }
           
            var textAxisOptions = {
                'id': this.svgObject.id + '_AxisToolTipText' + '_' + index,
                'x': x - vPadding,
                'y': y + hPadding,
                'fill': axis.crosshairLabel.font.color,
                'font-size': axis.crosshairLabel.font.size,
                'font-family': axis.crosshairLabel.font.fontFamily,
                'font-style': axis.crosshairLabel.font.fontStyle,
                'font-weight': axis.crosshairLabel.font.fontWeight,
                'text-anchor': 'start',
                'opacity': axis.crosshairLabel.font.opacity
              
            };
            var canvasPadding = (enableCanvas) ? padding / 4 : 0,
                 fontSize = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.data.currentTrackText, this.model.m_AreaBounds.Width, axis.crosshairLabel.font),
                 crosshairLabel = axis.crosshairLabel,			    
                 x = (x - padding),
                 y = (y - (fontSize.height)),
                 width = (fontSize.width) + (2 * padding),
                 height = (2 * fontSize.height) - (2 * padding);
            var toolAxisRectOptions = {
                'x': x - vPadding,
                'y': y + hPadding,
                'width': width,
                'height': height,
                'rx': crosshairLabel.rx * ((enableCanvas) ? 2 : 1),
                'ry': crosshairLabel.ry * ((enableCanvas) ? 2 : 1)
            };
            var direction = ej.EjSvgRender.utils._calculateroundedCorner(null, toolAxisRectOptions, true, position, 0, tracker);
            toolAxisRectOptions = {
                'id': this.svgObject.id + '_AxisToolTipRect' + '_' + index,
                'd': direction,
                'fill': crosshairLabel.fill,
                'stroke-width': crosshairLabel.border.width,
                'stroke': crosshairLabel.border.color
            };          
           
             var axes = this.model._axes[index];
            if ((mouseLoc.y < (axes.y +axes.height) && mouseLoc.y > axes.y)||(mouseLoc.x < (axes.x + axes.width) && mouseLoc.x > axes.x)) {
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipRect' + '_' + index), { "display": 'block' });
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipText' + '_' + index), { "display": 'block' });                   
                     toolAxisRectOptions.display = 'block';
                 }
            else {
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipRect' + '_' + index), { "display": 'none' });
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipText' + '_' + index), { "display": 'none' });                    
                     toolAxisRectOptions.display = 'none';
                 }
            if (enableCanvas) {
                var obj = this.svgRenderer.createCrosshairCanvas();
                obj.ctx = obj.getContext('2d');                  
            }
            else
                var obj = this.svgRenderer;
            this.svgRenderer.drawPath.call(obj, toolAxisRectOptions, this.gTrackAxisEle);        
            this.svgRenderer.drawText.call(obj, textAxisOptions, commonTrackTextArgs.data.currentTrackText, this.gTrackAxisEle);           
        }
    },
    maxWdithArrayCollection: function (arrayCollection, font) {
        var maxTextWidth = 0,size, width = 0,height=0,
            measureText = ej.EjSvgRender.utils._measureText;
        for (var txt = 0, txtlength = arrayCollection.length; txt < txtlength; txt++) {
            size = measureText(arrayCollection[txt], 0, font);
            width = size.width;
            height += size.height;
            maxTextWidth = (maxTextWidth == 0) ? width : (maxTextWidth < width) ? width : maxTextWidth;
        }
        return { maxTextWidth: maxTextWidth, totalHeight: height };
    },
    displayTooltip: function (grouping, tooltipCollections, maxWidth) {
        // common variables for both grouping and float
        var crosshair = this.model.crosshair, trackball = crosshair.trackballTooltipSettings, rx = trackball.rx, ry = trackball.ry, trackballBorder = trackball.border,
            m_AreaBounds = this.model.m_AreaBounds;
        if(grouping){
            var series = tooltipCollections.series, groupSize, groupHeight, container = document.getElementById(this._id),
                offset = $(container).offset(),
                rectOptions = series.tooltip, font = rectOptions.font ? $.extend(false, series.font, {}, rectOptions.font) : series.font,
                point = series.type == "boxandwhisker" ? tooltipCollections.point.boxPlotLocation[3] : tooltipCollections.point.location,
                m_width =((series.marker.size.width/2 + series.marker.border.width+trackballBorder.width))+5,// 5 for space between group element and tracker
                yAxisY = ((series._isTransposed) ? (series.xAxis.y + m_width) : series.yAxis.y),
                xAxisX = ((series._isTransposed) ? series.yAxis.x : (series.xAxis.x + m_width)),
                location = { X: xAxisX + point.X, Y: yAxisY + point.Y },
                textCollection=tooltipCollections.textCollection,textOptionsCollection=tooltipCollections.textOptionsCollection,
                groupingTextId = tooltipCollections.groupingTextId,
                options,
                groupLocation = { X: (location.X), Y: location.Y },
                groupElement = document.getElementById(this._id + "_trackball_grouping_tooltip");
            this._trigger("trackToolTip", { data: { currentText: textCollection, location: groupLocation } });
            if (!ej.util.isNullOrUndefined(this.model.trackToolTip) && this.model.trackToolTip != "") {
                for (var tx = 0, len = textCollection.length; tx < len - 1; tx++) {
                    if (textCollection[tx].text == "") {
                        textCollection.splice(tx, 1);
                        textOptionsCollection.splice(tx, 1);
                        groupingTextId.splice(tx, 1);
                    }
                }
            }
            if (ej.util.isNullOrUndefined(groupElement)) {
                    groupElement = document.createElement("div");
                    options = {
                        'left': groupLocation.X,
                        'top': groupLocation.Y,
                        'border': parseInt((trackballBorder.width) ? trackballBorder.width : 1) + "px solid " + ((trackballBorder.color) ? trackballBorder.color : "#000000"),
                        'background': (trackball.fill) ? trackball.fill : "#ffffff",
                        'opacity': trackball.opacity,
                        'position': 'absolute',
                        'height': 'auto',
                        'width': 'auto',
                        'border-radius': rx + "px " + ry + "px",
                        'pointer-events': 'none'
                        };
                    groupElement.setAttribute('id', this._id + "_trackball_grouping_tooltip");
            }
            else {
                options = {
                    'left': groupLocation.X,
                    'top': groupLocation.Y,
                }
            }
                for (var tx = 0, len = textCollection.length; tx < len; tx++) {
                    span = document.createElement("span");
                    span.setAttribute("id", groupingTextId[tx]);
                    span.innerHTML = textCollection[tx].text + "</br>";
                    $(span).css(textOptionsCollection[tx]);
                    groupElement.appendChild(span);
                }
                var arrayCollection = textCollection.map(function (a) { return a.text; });
                groupSize = this.maxWdithArrayCollection(arrayCollection, font);
                maxWidth = groupSize.maxTextWidth;
                groupHeight = groupSize.totalHeight;
                options.top = (series._isTransposed) ? options.top : (options.top - (groupHeight * 0.5));
                options.left = (series._isTransposed) ? (options.left - (maxWidth * 0.5)) : options.left;
                if (textCollection.length > 1) {
                    options.top = (series._isTransposed) ? options.top : (yAxisY + (m_AreaBounds.Height * 0.5 - (0.5 * groupHeight)));
                    options.left = (series._isTransposed) ? (((m_AreaBounds.Width / 2) + m_AreaBounds.X) - (maxWidth * 0.5)) : options.left;
                }
                if (!series._isTransposed) {
                    if (options.left + maxWidth >= (m_AreaBounds.X + m_AreaBounds.Width))
                        options.left -= (maxWidth + (m_width+trackballBorder.width)*2);// extra space on edge group element swifting for already added need to remove 
                }
                else {

                    if ((groupHeight + options.top) > (m_AreaBounds.Y + m_AreaBounds.Height))
                        options.top -= (groupHeight + (m_width*2));// extra space on edge group element swifting
                }
                options.left = (groupLocation.X == location.X) ? options.left : groupLocation.X;
                options.top = (groupLocation.Y == location.Y) ? options.top : groupLocation.Y;
                $(groupElement).css(options);
				$(groupElement).appendTo(this.chartContainer);
                
        }
        else{
            for(var init=0,tlength=tooltipCollections.length;init<tlength;init++){	
				
                var location=tooltipCollections[init].Point,point= tooltipCollections[init].ClosestPoint, series=tooltipCollections[init].Series, pointIndex=tooltipCollections[init].ClosestPointIndex, tgap=tooltipCollections[init].Tgap,
                    seriesIndex = $.inArray(series, this.model._visibleSeries),   
                    Axis = { X1: m_AreaBounds.X, Y1: (m_AreaBounds.Y + m_AreaBounds.Height), X2: (m_AreaBounds.X + m_AreaBounds.Width), Y2: m_AreaBounds.Y },                    
                    axes = this.model._axes, pointXAxis = series._xAxisName, pointYAxis = series._yAxisName;
        
                for (var j = 0, len = axes.length; j < len; j++) {
                    if (pointXAxis == axes[j].name) {
                        if(!series._isTransposed)
                            Axis.X1 = axes[j].Location.X1, Axis.X2 = axes[j].Location.X2;
                        else
                            Axis.Y1 = axes[j].Location.Y1, Axis.Y2 = axes[j].Location.Y2;
                    }
                    else if (pointYAxis == axes[j].name) {
                        if (!series._isTransposed)
                            Axis.Y1 = axes[j].Location.Y1, Axis.Y2 = axes[j].Location.Y2;
                        else
                            Axis.X1 = axes[j].Location.X1, Axis.X2 = axes[j].Location.X2;
                    }
                }
				location.X = (series.type == "boxandwhisker") ? location.X + Axis.X1 : location.X;
                location.Y = (series.type == "boxandwhisker") ? location.Y + Axis.Y2 : location.Y;
                if((location.X>=Axis.X1 && location.X <=Axis.X2 && location.Y>= Axis.Y2 && location.Y<=Axis.Y1)||(grouping) ){
                    var requireInvertedAxes = this.model.requireInvertedAxes, tx, ty,
                        x = location.X + (!series.isIndicator ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) : 0) + padding + 1,
                        y = (location.Y), format = series.tooltip.format, numberToFixed = ((ej.util.isNullOrUndefined(series.yAxis.roundingPlaces)) ? 2 : series.yAxis.roundingPlaces);
						if(tooltipCollections[init].Series.type == "boxandwhisker" && location.outlier)
							var trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, location);
						else
							var trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);
						var seriesName = (series.name) ? (series.name).replace(' ', '') : "series" + seriesIndex, commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs),
                        toolRectOptions, direction;
                        commonTrackTextArgs.data = { Location: { X: x, Y: y }, Series: series, serIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText.text };
                    this._trigger("trackToolTip", commonTrackTextArgs);
                    if (!commonTrackTextArgs.cancel) {
                        x = commonTrackTextArgs.data.Location.X;
                        y = commonTrackTextArgs.data.Location.Y;
                        var padding = this.model.tooltipPadding, tip_pos = tgap, rectOptions = series.tooltip, font = rectOptions.font ? $.extend(false, series.font, {}, rectOptions.font) : series.font, textOffset = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.data.currentText, series.xAxis.width, font), seriesColor = this.getSeriesColor(point, seriesIndex, series), toolTipOptions = this.getTooltipOptions(seriesColor, series), rectBorderColor = toolTipOptions.rectBColor, rectFillColor = toolTipOptions.rectFColor, textColor = toolTipOptions.rectTextColor, rectWidth = textOffset.width + (2 * padding), rectHeight = textOffset.height + (2 * padding), poly, tipOpen = (requireInvertedAxes) ? ((rectWidth * 10) / 100) : ((rectHeight * 30) / 100),
                            axisWidth = (requireInvertedAxes) ? series.yAxis.width : series.xAxis.width, tooltipStartPosition = (requireInvertedAxes) ? (tgap + location.X - rectWidth / 2) : (tgap + location.Y - rectHeight / 2), tooltipEndPosition = (requireInvertedAxes) ? (tgap + location.X + rectWidth / 2) : (tgap + location.Y + rectHeight / 2), boundryStart = (requireInvertedAxes) ? (m_AreaBounds.X) : (m_AreaBounds.Y), boundryEnd = (requireInvertedAxes) ? (m_AreaBounds.X + m_AreaBounds.Width) : (m_AreaBounds.Y + m_AreaBounds.Height), text_pos = [],
                            marker_width = 3 + crosshair.marker.border.width + ((!requireInvertedAxes) ? crosshair.marker.size.width : crosshair.marker.size.height) / 2,
                            swp = (!this.model.financial) ? marker_width + 5 : 0, type = series.type.toLowerCase(),                           
                            position, hsize = type.indexOf("rangearea") == 0 ? this.model.crosshair.marker.size.height / 2 : 0;
                        if ((tooltipStartPosition >= boundryStart && tooltipEndPosition <= (boundryEnd)) || this.model.financial) {
                            if (requireInvertedAxes) {
                                if (location.Y - swp - maxWidth < m_AreaBounds.Y) {                         
                                    toolRectOptions = {
                                        'x': location.X - rectWidth / 2 + tip_pos,
                                        'y': location.Y + swp,
                                        'width': rectWidth,
                                        'height': rectHeight,
                                        'rx': rx,
                                        'ry': ry
                                    };
                                    text_pos.push({ X: location.X - rectWidth / 2 + padding + tip_pos, Y: location.Y + swp + rectHeight / 2 - padding });
                                    position = "top";
                                  
                                } else {
                                    if (this.model.financial) {
                                        x = location.X - rectWidth / 2 + tip_pos; y = location.Y + padding + hsize; tx = x + padding; ty = y + 2*padding;
                                        position = "top";
                                    }
                                    else {
                                        x = location.X - rectWidth / 2 + tip_pos; y = location.Y - swp - rectHeight; tx = location.X - rectWidth / 2 + padding + tip_pos; ty = location.Y - swp - rectHeight / 2 - padding;
                                        position = "bottom";
                                    }
                                    toolRectOptions = {
                                        'x': x,
                                        'y': y,
                                        'width': rectWidth,
                                        'height': rectHeight,
                                        'rx': rx,
                                        'ry': ry
                                    };
                                    text_pos.push({ X: tx, Y: ty });
                                 
                                }
                            }
                            else {
                               
                                if (location.X + swp + maxWidth - m_AreaBounds.X >= axisWidth) {
                                    if (this.model.financial) {
                                        x = location.X - rectWidth / 2; y = location.Y - rectHeight - padding - hsize; tx = x + padding; ty = y + tip_pos + rectHeight;
                                        position = "bottom";
                                    }
                                    else {
                                        x = location.X - swp - rectWidth; y = location.Y - rectHeight / 2 + tip_pos; tx = x + padding; ty = location.Y + tip_pos + rectHeight / 2;
                                        position = "right";
                                    }
                                    toolRectOptions = {
                                        'x': x,
                                        'y': y,
                                        'width': rectWidth,
                                        'height': rectHeight,
                                        'rx': rx,
                                        'ry': ry
                                    };
                           
                                    text_pos.push({ X: tx, Y:ty });
                                    
                                }
                                else {
                                    if (this.model.financial) {                                       
                                        x = location.X - rectWidth / 2; y = location.Y - hsize - rectHeight - 2 * padding; tx = x + padding; ty = y + tip_pos + rectHeight;
                                        position = "bottom";
                                    }
                                    else {
                                        x = location.X + swp; y = location.Y - rectHeight / 2 + tip_pos; tx = x + padding; ty = location.Y + tip_pos + rectHeight / 2;
                                        position = "left";
                                    }
                               
                                    toolRectOptions = {
                                        'x': x,
                                        'y': y,
                                        'width': rectWidth,
                                        'height': rectHeight,
                                        'rx': rx,
                                        'ry': ry
                                    };
                           
                                    text_pos.push({ X: tx, Y: ty });
                         
                                    
                                }
                            }                
                     
                                 
                   
                  
                            var tooltipfill = seriesColor;
                            if (typeof (tooltipfill) == "object") {
                                tooltipfill = seriesColor[1].color;
                            }
                           
                  
            
                            var text = commonTrackTextArgs.data.currentText,fontText = text,fontSize = ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height,len;
                            if (commonTrackTextArgs.data.currentText.indexOf("<br/>") >= 0) {
                                text = commonTrackTextArgs.data.currentText.split("<br/>");
                                fontText = text[0];
                                if (!requireInvertedAxes) {
                                    var isCandle = (type.indexOf("candle") >= 0 || type.indexOf("hiloopenclose") >= 0) || false;
                                    if (location.X + swp + maxWidth - m_AreaBounds.X >= axisWidth) {
                                        if (!this.model.financial) {
                                            tx = location.X - rectWidth - swp + padding;
                                            ty = ((location.Y + tip_pos) - (rectHeight / 2) + fontSize / text.length);
                                        }
                                        else {
                                            tx = location.X - rectWidth / 2 + padding;
                                            ty = ((location.Y + tip_pos) - (rectHeight)) - hsize + ((isCandle) ? 0 : 2*padding);
                                            toolRectOptions.y -= (isCandle) ? padding * 3 : padding;
                                            toolRectOptions.height += (isCandle) ? padding * 2 : padding;
                                        }

                                          
                                        text_pos.push({ X: tx, Y: ty });
                                    }
                                    else {
                                        if (!this.model.financial) {
                                            tx = location.X + swp + padding;
                                            ty = ((location.Y + tip_pos) - (rectHeight / 2) + fontSize / text.length);
                                        }
                                        else {
                                            
                                            tx = location.X - rectWidth/2 + padding;
                                            ty = ((location.Y + tip_pos) - (rectHeight)) - hsize + ((isCandle) ? 0 : padding);
                                            toolRectOptions.y -= (isCandle) ? padding * 2 : padding;
                                            toolRectOptions.height += (isCandle) ? padding * 2 : padding;
                                        }
                                        text_pos.push({ X: tx, Y: ty });
                                    }
                                }
                                else {
                                    if (location.Y - swp - maxWidth < m_AreaBounds.Y) {
                                        text_pos.push({ X: location.X - rectWidth / 2 + padding + tip_pos, Y: location.Y + swp + (fontSize / text.length) });
                                    }
                                    else {
                                        if (!this.model.financial) {
                                            tx = location.X - rectWidth / 2 + padding + tip_pos;
                                            ty = location.Y - swp - rectHeight + (fontSize / text.length);
                                        }
                                        else {
                                            tx = location.X - rectWidth / 2 + padding + tip_pos;
                                            ty = (location.Y + 4 * padding) + hsize;
                                        }
                                        text_pos.push({ X: tx, Y: ty});
                                    }
                                }
                                fontSize = 0;
                            }
                            len = text_pos.length - 1;
                            fontSize = (requireInvertedAxes) ? fontSize : -fontSize;
							var index = series.type == "boxandwhisker" ? init : pointIndex ;
                            var textOptions = {
                                'id': this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + "_" + index,
                                'x': text_pos[len].X,
                                'y': text_pos[len].Y+fontSize/2,
                                'fill': rectOptions.font ? (rectOptions.font.color ? rectOptions.font.color : "#ffffff") : "#ffffff",
                                'font-size': font.size,
                                'font-family': font.fontFamily,
                                'font-style': font.fontStyle,
                                'font-weight': font.fontWeight,
                                'text-anchor': 'start',
								'isTrackball': true,
                                'padding': this.model.tooltipPadding
                            };

                            direction = ej.EjSvgRender.utils._calculateroundedCorner(null, toolRectOptions, true, position, tip_pos);

                            var arrowPath = {
                                'id': this.svgObject.id + '_gTooltip_' + seriesIndex + '_' + index,
                                'd': direction,
                                'fill': ((trackball.fill) ? trackball.fill : tooltipfill),
                                "stroke-width": parseInt(trackball.border.width ? trackball.border.width : 0) + "px",
                                "stroke": trackball.border.color ? trackball.border.color : 'transparent',
                                'fill-opacity': trackball.opacity
                            };
                            
                            if (this.model.enableCanvasRendering) {
                                var obj = this.svgRenderer.createCrosshairCanvas();
                                obj.ctx = obj.getContext('2d');
                            }

                            else
                                var obj = this.svgRenderer;


                            var gTooltip = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_gTooltip_' + seriesIndex + "_" + index, });

                            this.svgRenderer.drawPath.call(obj, arrowPath, gTooltip);
                            this.svgRenderer.drawText.call(obj, textOptions, text, gTooltip, font);
                     
                        }
                    }

                    if ($('#' + this.svgObject.id + "_TrackToolTip_" + seriesIndex).length == 0)
                        this.svgRenderer.append($(gTooltip), this.gTrackball);
                }  
            }
        }
    },
    SmartTooltipPosition: function (SmartTooltipOptions) {
        var InitialRanges = [[], []], exceed, length_SmartTooltips = SmartTooltipOptions.length, visibleLocation = [], totalSapce = (SmartTooltipOptions[0].Series._isTransposed) ? this.model.m_AreaBounds.Width : this.model.m_AreaBounds.Height, tooltipPadding = this.model.tooltipPadding, tooltipWidth = -tooltipPadding;
        for (var a = 0; a < length_SmartTooltips; a++) {
            InitialRanges[0][a] = $.extend({}, SmartTooltipOptions[a].StEnd);
            InitialRanges[1][a] = { yAxis: SmartTooltipOptions[a].Series._yAxisName, xAxis: SmartTooltipOptions[a].Series._xAxisName, isTransposed: SmartTooltipOptions[a].Series._isTransposed };
            if (SmartTooltipOptions[a].ReqInvertAxis) {
                visibleLocation[a] = SmartTooltipOptions[a].Point.X;
            }
            else {
                visibleLocation[a] = SmartTooltipOptions[a].Point.Y;
            }
            tooltipWidth += SmartTooltipOptions[a].StEnd.End - SmartTooltipOptions[a].StEnd.Start + tooltipPadding;
        }
        this.ContinousOverlappingPoints(InitialRanges[0], visibleLocation);
        if (tooltipWidth < totalSapce)
            this.verticalArrangeMents(InitialRanges);
        for (var a = 0; a < length_SmartTooltips; a++) {
            SmartTooltipOptions[a].Tgap = (!this.model.financial) ? InitialRanges[0][a].Start - SmartTooltipOptions[a].StEnd.Start : 0;
        }
    },
    verticalArrangeMents: function (startEndRanges) {
        var pointsRanges = startEndRanges[0],
            axisNames = startEndRanges[1], //axis names contains isTransposed also
            obj = this.model.m_AreaBounds,
            padding = this.model.tooltipPadding,
            isTransposed,
            chartHeight,
            startPos,
            yAxisName,
            axes = this.model._axes,
            temp,//current trackabll tooltip start end
            secTemp,//vertical arrangements
            secWidth,//vertical arrange width
            condition,
            width, length = pointsRanges.length, i = length - 1, j = 0,k=0,axisLength=axes.length;
        for (; i >= 0; i--) {
            isTransposed = axisNames[i].isTransposed;
            yAxisName =axisNames[i].yAxis;
            for (k = 0; k < axisLength; k++) {
                if (yAxisName == axes[k].name) {
                    if (isTransposed) {
                        chartHeight = axes[k].Location.X2;
                        startPos = axes[k].Location.X1;
                    }
                    else {
                        chartHeight = axes[k].Location.Y1;
                        startPos = axes[k].Location.Y2;
                    }
                }
            }
            temp = pointsRanges[i];
            width = temp.End - temp.Start;
            if (chartHeight < temp.End) {
                temp.End = chartHeight - 2;
                temp.Start = temp.End - width;
                for(j=i-1;j>=0;j--)
                {
                    secTemp = pointsRanges[j];
                    secWidth = secTemp.End - secTemp.Start;
                    if (secTemp.End > pointsRanges[j + 1].Start && (pointsRanges[j + 1].Start > startPos && pointsRanges[j + 1].End<chartHeight)) {
                        secTemp.End = pointsRanges[j + 1].Start - padding;
                        secTemp.Start = secTemp.End - secWidth;
                    }
                }
            }
        }
        for (i = 0; i < length; i++) {
            isTransposed = axisNames[i].isTransposed;
            yAxisName = axisNames[i].yAxis;
            for (k=0; k < axisLength; k++) {
                if (yAxisName == axes[k].name) {
                    if (isTransposed) {
                        chartHeight = axes[k].Location.X2;
                        startPos = axes[k].Location.X1;
                    }
                    else {
                        chartHeight = axes[k].Location.Y1;
                        startPos = axes[k].Location.Y2;
                    }
                }
            }
            temp = pointsRanges[i];
            width = temp.End - temp.Start;
            if (temp.Start < startPos) {
                temp.Start = startPos + 1;
                temp.End = temp.Start + width;
                for (j = i + 1; j <= (length - 1) ; j++) {
                    secTemp = pointsRanges[j];
                    secWidth = secTemp.End - secTemp.Start;
                    if (secTemp.Start < pointsRanges[j - 1].End && (pointsRanges[j - 1].Start > startPos && pointsRanges[j - 1].End < chartHeight)) {
                        secTemp.Start = pointsRanges[j - 1].End + padding;
                        secTemp.End = secTemp.Start + secWidth;
                    }
                }
            }
        }
    },
    ContinousOverlappingPoints: function (TooltipRanges, visibleLocation) {
        var padding = this.model.tooltipPadding, temp = 0, Count = 0, Start = 0, StartPoint = 0, endPoint = TooltipRanges.length - 1, Range = (TooltipRanges[0].End - TooltipRanges[0].Start) + padding, halfHeight, midPos, tRange, kRange;
        temp = TooltipRanges[0].Start + Range;
        Start = TooltipRanges[0].Start;
        for (var i = 0; i < endPoint; i++) {
            if (temp >= TooltipRanges[i + 1].Start) {
                Range = TooltipRanges[i + 1].End - TooltipRanges[i + 1].Start + padding;
                temp += Range;
                Count++;
                if (Count - 1 == endPoint - 1 || i == endPoint - 1) {
                     halfHeight = (temp - Start) / 2;
                     midPos = (visibleLocation[StartPoint] + visibleLocation[i + 1]) / 2;
                     tRange = TooltipRanges[StartPoint].End - TooltipRanges[StartPoint].Start;
                    TooltipRanges[StartPoint].Start = midPos - halfHeight;
                    TooltipRanges[StartPoint].End = TooltipRanges[StartPoint].Start + tRange;
                    for (var k = StartPoint; k > 0; k--) {
                        if (TooltipRanges[k].Start <= TooltipRanges[k - 1].End + padding) {
                            kRange = TooltipRanges[k - 1].End - TooltipRanges[k - 1].Start;
                            TooltipRanges[k - 1].Start = TooltipRanges[k].Start - kRange - padding;
                            TooltipRanges[k - 1].End = TooltipRanges[k - 1].Start + kRange;
                        }
                        else {
                            break;
                        }
                    }
                    for (var j = StartPoint + 1; j <= StartPoint + Count; j++) {
                        tRange = TooltipRanges[j].End - TooltipRanges[j].Start;
                        TooltipRanges[j].Start = TooltipRanges[j - 1].End + padding;
                        TooltipRanges[j].End = TooltipRanges[j].Start + tRange;

                    }
                }

            }
            else {
                Count = i > 0 ? Count : 0;
                if (Count > 0) {
                     halfHeight = (temp - Start) / 2;
                     midPos = (visibleLocation[StartPoint] + visibleLocation[i]) / 2;
                     tRange = TooltipRanges[StartPoint].End - TooltipRanges[StartPoint].Start;
                    TooltipRanges[StartPoint].Start = midPos - halfHeight;
                    TooltipRanges[StartPoint].End = TooltipRanges[StartPoint].Start + tRange;
                    for (var k = StartPoint; k > 0; k--) {
                        if (TooltipRanges[k].Start <= TooltipRanges[k - 1].End + padding) {
                            kRange = TooltipRanges[k - 1].End - TooltipRanges[k - 1].Start;
                            TooltipRanges[k - 1].Start = TooltipRanges[k].Start - kRange - padding;
                            TooltipRanges[k - 1].End = TooltipRanges[k - 1].Start + kRange;
                        }
                        else {
                            break;
                        }
                    }
                    for (var j = StartPoint + 1; j <= StartPoint + Count; j++) {
                        tRange = TooltipRanges[j].End - TooltipRanges[j].Start;
                        TooltipRanges[j].Start = TooltipRanges[j - 1].End + padding;
                        TooltipRanges[j].End = TooltipRanges[j].Start + tRange;
                    }
                    Count = 0;
                }
                Range = (TooltipRanges[i + 1].End - TooltipRanges[i + 1].Start + padding);
                temp = TooltipRanges[i + 1].Start + Range;
                Start = TooltipRanges[i + 1].Start;
                StartPoint = i + 1;
            }

        }
    },
    getTooltipOptions: function (seriesColor, series) {
        var rectBorderColor, rectFillColor, rX, rY, textColor;
        if (this.model.theme.indexOf("gradient") >= 0) {
            rectBorderColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
            rectFillColor = 'white';
            textColor = "#333333";
            rX = 5;
            rY = 5;

        } else {
            var type = series.type.toLowerCase();
            if (type.indexOf("column") == -1 && type.indexOf("waterfall") == -1 && type.indexOf("bar") == -1 && type.indexOf("box") == -1  && type != "rangearea" && this.model.AreaType != "none") {
                rectFillColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
                rectBorderColor = 'transparent';
                textColor = "white";
            } else {
                rectBorderColor = "#333333";
                rectFillColor = 'white';
                textColor = "#333333";
            }
            rX = 0;
            rY = 0;
        }
        return { rectBColor: rectBorderColor, rectFColor: rectFillColor, rectTextColor: textColor, rectX: rX, rectY: rY };
    },
    getSeriesColor: function (point, seriesIndex, series) {
        var seriesColor;
        if (series && !series.isIndicator && !series.isTrendLine) {
            if (point._hiloFill) {
                if (!point._hiloFill._gradientStop)
                    seriesColor = point._hiloFill;
                else {
                    seriesColor = point._hiloFill._gradientStop[0].color;
                }
            }
            else if (point.fill) {
                if (!point.fill._gradientStop)
                    seriesColor = point.fill;
                else {
                    seriesColor = point.fill._gradientStop[0].color;
                }
            }
            else {
                if (series.type.toLowerCase() == "waterfall" && series.positiveFill &&
                    ((point.y > 0 && !point.showIntermediateSum && !point.showTotalSum) || point.waterfallSum > 0))
                    seriesColor = series.positiveFill;
                else
                    seriesColor = this.model.seriesColors[seriesIndex];
                
            }
        } else {
            seriesColor = series.fill;
        }
        return seriesColor;
    },
    getFormat: function (series,boxLocation) {

        if (!series.tooltip.format) {
            var type =series.type.toLowerCase();
			var outlierVisible = series.type != "boxandwhisker" ? 'null' : boxLocation.outlier;
            if (type.indexOf("range") > -1 || (type.indexOf("hilo") > -1 && type.indexOf("open") == -1))
                return "#point.x# <br/> High : #point.high# <br/> Low : #point.low#";
            else if (type.indexOf("candle") > -1 || type.indexOf("open") > -1)
                return "#point.x# <br/> High : #point.high# <br/> Low : #point.low# <br/> Open : #point.open# <br/> Close : #point.close#";
            else if (type.indexOf("bubble") >-1)
                return "#point.x# : #point.y# : #point.size#"
			else if(type.indexOf("box") > -1 && outlierVisible )
				return "X : #boxPlotPoints.xValue#  Y : #boxPlotPoints.YValues#";
			else if(type.indexOf("box") > -1 )
				return "Minimum : #boxPlotPoints.Minimum# <br/> Upper Quartile : #boxPlotPoints.UpperQuartile# <br/> Median : #boxPlotPoints.midvalue# <br/> Lower Quartile : #boxPlotPoints.LowerQuartile# <br/> Maximum : #boxPlotPoints.Maximum#  ";			 
			else	
             return "#point.x# : #point.y#"
        }
        return series.tooltip.format;
    },
    getTooltipFormat: function (seriesPoint, series, serIndex, ptIndex, format,boxLocation) {
         var point = $.extend(true, {}, seriesPoint),
            areaType = this.model.AreaType,
            locale = this.model.locale,
            xAxis = series.xAxis,
            yAxis = series.yAxis,
            xAxisValueType = xAxis._valueType.toLowerCase(),
            nullorundefined = ej.util.isNullOrUndefined,
            xAxisLabelFormat = xAxis.labelFormat,
            yAxisLabelFormat = yAxis.labelFormat,
            dateTimeFormat = ej.EjSvgRender.utils._dateTimeLabelFormat,
            decimalPlaces = ej.EjSvgRender.utils._decimalPlaces,
            xGlobalize = false, yGlobalize = false,
            xFormat,yFormat,
           customFormat, data,
           xPrecision, yPrecision, precisionDefault = 6, precisionHighest = 20;
        series.count = 1;
        point.count = 1;
        format = this.getFormat(series,boxLocation);
		while (format.indexOf('ej.format(') >= 0) {
            substr = format.substring(format.indexOf('ej.format('), format.indexOf(")") + 1);
            var calculate = substr.substring(substr.indexOf('(') + 1, substr.indexOf(","));
            if (calculate == "#point.x#") {
                xGlobalize = true,
                xFormat = substr.substring(substr.indexOf(',') + 1, substr.indexOf(")"));
            }
            else if (calculate == "#point.y#") {
                yGlobalize = true,
                yFormat = substr.substring(substr.indexOf(',') + 1, substr.indexOf(")"));
            } else {
                globalizeformat = substr.substring(substr.indexOf(',') + 1, substr.indexOf(")"));
                subStr1 = calculate.substring(calculate.indexOf('#') + 1);
                val = subStr1.substring(subStr1.indexOf('.') + 1, subStr1.indexOf('#'));
                if (point.hasOwnProperty(val) && point[val])
                    point[val] = ej.format(point[val], globalizeformat, locale);
            }
            format = format.replace(substr, calculate)
        }
		if (areaType == "cartesianaxes") {
            if (xAxisValueType.toLowerCase() == "datetime"){
                point.x = ej.format(new Date(point.xValue), ((nullorundefined(xAxisLabelFormat)) ? dateTimeFormat(series.xAxis._intervalType) : series.xAxis.labelFormat), locale);
                point.x = xGlobalize ? (ej.format(new Date(point.xValue), xFormat, locale)) : point.x;
		    }
            else if (xAxisValueType.toLowerCase() == "datetimecategory") {
                series.xAxis.intervalType = (ej.util.isNullOrUndefined(series.xAxis.intervalType) || series.xAxis.intervalType == "auto") ? "years" : series.xAxis.intervalType;
		        point.x = ej.format(new Date(point.x), ((nullorundefined(xAxisLabelFormat)) ? dateTimeFormat(series.xAxis.intervalType) : series.xAxis.labelFormat), locale);
		        point.x = xGlobalize ? (ej.format(new Date(point.x), xFormat, locale)) : point.x;
		    }
            else if (xAxisValueType.toLowerCase() == "category") {
                if (this.model.primaryXAxis.isIndexed)
                    point.x = (point.x) ? point.x : "undefined";
		        else
                point.x = ej.EjSvgRender.utils._getLabelContent(point.xValue, series.xAxis, locale);
            }
		    else
            {
                if ((xGlobalize && (xFormat.indexOf('e') == 0 || xFormat.indexOf('E') == 0)) ) {
                    point.x = this.convertExponential(xPrecision, precisionDefault, precisionHighest, point.x, xFormat);
                }
                else
                point.x = xGlobalize ? ej.format(point.x, xFormat, locale) : point.x;
                customFormat = (!(xAxisLabelFormat)) ? null : xAxisLabelFormat.match('{value}');
                point.x = (!(xAxisLabelFormat)) ? point.x : (customFormat != null) ? (xAxisLabelFormat == "${value}") ? xAxisLabelFormat.replace('{value}', '$' + point.x) : xAxisLabelFormat.replace('{value}', point.x) : (xGlobalize || (xAxisLabelFormat.indexOf('e') == 0 || xAxisLabelFormat.indexOf('E') == 0)) ? point.x : (ej.format(Number(point.x), xAxisLabelFormat, locale));

            }		
        }
        else {
            if (xAxisValueType == "date") {
                point.x = (ej.format(new Date(point.xValue), (dateTimeFormat('days')), locale));
                point.x = xGlobalize ? (ej.format(new Date(point.xValue), xFormat, locale)) : point.x;
            }                
		}
		if ((yGlobalize && (yFormat.indexOf('e') == 0 || yFormat.indexOf('E') == 0))) {
		    point.y = this.convertExponential(yPrecision, precisionDefault, precisionHighest, point.y, yFormat);
		}
		else {
		    point.y = point.waterfallSum ? point.waterfallSum : point.y;
		    point.y = yGlobalize ? ej.format(point.y, yFormat, locale) : point.y;
		}
        customFormat = (!(yAxisLabelFormat)) ? null : yAxisLabelFormat.match('{value}');
        point.y = (!(yAxisLabelFormat)) ? point.y : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + point.y) : yAxisLabelFormat.replace('{value}', point.y) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? point.y : (ej.format(Number(point.y), yAxisLabelFormat, locale));
        if ((areaType == 'none' || areaType == 'polaraxes') && xGlobalize && (xFormat.indexOf('e') == 0 || xFormat.indexOf('E') == 0)) {
            point.x = this.convertExponential(xPrecision, precisionDefault, precisionHighest, point.x, xFormat);
        }
		if(series.type.toLowerCase()=="boxandwhisker"){
		   var boxPlotLabels;
		   boxPlotLabels = series._visiblePoints[ptIndex].boxPlotValues;
			  if(boxLocation.outlier == true) { 
				for(var z=0;z<boxPlotLabels.outliers.length;z++)
				{
					if(boxLocation.YValues == boxPlotLabels.outliers[z] )
						boxLocation.YValues = (!(yAxisLabelFormat)) ? boxLocation.YValues : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + boxLocation.YValues) : yAxisLabelFormat.replace('{value}', boxLocation.YValues) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? boxLocation.YValues : (ej.format(Number(boxLocation.YValues), yAxisLabelFormat, locale));			
				}
			}
			else{			
				if(boxLocation.YValues == boxPlotLabels.Minimum )
					boxPlotLabels.Minimum = (!(yAxisLabelFormat)) ? boxPlotLabels.Minimum : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + boxPlotLabels.Minimum) : yAxisLabelFormat.replace('{value}', boxPlotLabels.Minimum) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? boxPlotLabels.Minimum: (ej.format(Number(boxPlotLabels.Minimum), yAxisLabelFormat, locale));
				else if(boxLocation.YValues == boxPlotLabels.Maximum )	
					boxPlotLabels.Maximum = (!(yAxisLabelFormat)) ? boxPlotLabels.Maximum : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + boxPlotLabels.Maximum) : yAxisLabelFormat.replace('{value}', boxPlotLabels.Maximum) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? boxPlotLabels.Maximum : (ej.format(Number(boxPlotLabels.Maximum), yAxisLabelFormat, locale));
				else if(boxLocation.YValues == boxPlotLabels.UpperQuartile)	
					boxPlotLabels.UpperQuartile = (!(yAxisLabelFormat)) ? boxPlotLabels.UpperQuartile : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + boxPlotLabels.UpperQuartile) : yAxisLabelFormat.replace('{value}', boxPlotLabels.UpperQuartile) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? boxPlotLabels.UpperQuartile : (ej.format(Number(boxPlotLabels.UpperQuartile), yAxisLabelFormat, locale));
				else if(boxLocation.YValues == boxPlotLabels.LowerQuartile)		
					boxPlotLabels.LowerQuartile = (!(yAxisLabelFormat)) ? boxPlotLabels.LowerQuartile : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + boxPlotLabels.LowerQuartile) : yAxisLabelFormat.replace('{value}', boxPlotLabels.LowerQuartile) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? boxPlotLabels.LowerQuartile : (ej.format(Number(boxPlotLabels.LowerQuartile), yAxisLabelFormat, locale));
				else if(boxLocation.YValues == boxPlotLabels.midvalue)		
					boxPlotLabels.midvalue = (!(yAxisLabelFormat)) ? boxPlotLabels.midvalue : (customFormat != null) ? (yAxisLabelFormat == "${value}") ? yAxisLabelFormat.replace('{value}', '$' + boxPlotLabels.midvalue) : yAxisLabelFormat.replace('{value}', boxPlotLabels.midvalue) : (yGlobalize || (yAxisLabelFormat.indexOf('e') == 0 || yAxisLabelFormat.indexOf('E') == 0)) ? boxPlotLabels.midvalue : (ej.format(Number(boxPlotLabels.midvalue), yAxisLabelFormat, locale));						
			}
			boxPlotLabels.count = series.count;
			boxPlotLabels.xValue = point.x;
			boxPlotLabels.YValues = boxLocation.YValues;		
			data = {boxPlotPoints : boxPlotLabels };	   
		 }
		else
			data = {series:series,point:point};
        format = format.parseTemplate(data);
        return {data: point,text:format};
    },
    convertExponential: function (precision, precisionDefault, precisionHighest, point, format) {
        precision = format.match(/(\d+)/g);
        precision = precision == null ? precisionDefault : precision > precisionHighest ? precisionHighest : precision;
        point = point.toExponential(precision);
        return point;
    },
    displayShowTooltip: function (location, point, series, pointIndex) {
	     var requireInvertedAxes = this.model.requireInvertedAxes;
       if (point.visible) {
            // fixed multiple tooltip issue in mobile
           $(".ejTooltip" + this._id).not("#" + this.svgObject.id + "_TrackToolTip").remove();
           if ($.finish)
               $(".ejTooltip" + this._id).finish();
           else
               $(".ejTooltip" + this._id).stop(true, true);
            var measureText, parentZindex, seriesIndex = (!series.isIndicator && !series.isTrendLine) ? $.inArray(series, this.model._visibleSeries) : 0,
            seriesColor,
            textOffset,
            padding = 7, bubbleSpacing = 15,
            x,
            y,
            trackTooltipText,
            format = series.tooltip.format;
            var position = document.getElementById(this.svgObject.id).getClientRects()[0];
            var chartPos = this.model.m_AreaBounds;
            var type = series.type.toLowerCase();
            //Set tooltip position ,text and color
            switch (this.model.AreaType) {
               
                case "cartesianaxes":
                    x = location.X + ((!series.isIndicator && !series.isTrendLine) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? 
                    (type == "bubble" ? (point.radius + bubbleSpacing): series.marker.size.width) : (series._trackMarker.size.width)) : 0) + (type == "column" || type == "bar" || type == "stackingbar" || type == "stackingcolumn" || type == "waterfall" || type == "rangecolumn" ? chartPos.X : (requireInvertedAxes) ? series.yAxis.x : series.xAxis.x);
                    //condition checked for multipleaxis line tooltip position changed
                    y = (location.Y) + (type.indexOf("column") != -1 || type.indexOf("waterfall") != -1 || type.indexOf("bar") != -1 || type.indexOf("rangearea") != -1 || type == "scatter" || type == "bubble" ? 
						(this.dragPoint ? (requireInvertedAxes ? series.xAxis.y : series.yAxis.y) : chartPos.Y) : (requireInvertedAxes) ? series.xAxis.y : series.yAxis.y);
                    if (this.dragPoint) {
                        x = x > series.xAxis.x ? x : series.xAxis.x;
                        y = y < series.yAxis.height + series.yAxis.y ? y : series.yAxis.height + series.yAxis.y;
                    }
                    seriesColor = this.getSeriesColor(point, seriesIndex, series);
                    break;

                case "polaraxes":
                    x = location.X + padding + ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width));
                    y = location.Y; 
                    seriesColor = this.getSeriesColor(point, seriesIndex, series);
                    break;

                case "none":
                    x = location.X + series.marker.size.width + padding;
                    y = (location.Y);
                    seriesColor = this.model.pointColors[pointIndex];
                    break;
            }
			
			
			if(series.type.toLowerCase() == "boxandwhisker")
			{
				for (var p=0; p< series._visiblePoints[pointIndex].boxPlotLocation.length; p++){
					var boxLocation = series._visiblePoints[pointIndex].boxPlotLocation[p];
					var outlierVisible = boxLocation.outlier;
					if(outlierVisible)
					{
						if(boxLocation .X + (series.outlierSettings.size.width/2) > location.X && boxLocation .X - (series.outlierSettings.size.width/2) < location.X ){
							if(boxLocation .Y + (series.outlierSettings.size.height/2) > location.Y && boxLocation .Y - (series.outlierSettings.size.height/2) < location.Y ){						
								trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format,boxLocation);
								textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText.text);																
							}
						}					
					}					
					else
					{
						trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format,boxLocation);
						textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText.text);	
					}						
			  }			  
			}
			else
			{			
				trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format);
				textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText.text);
			}
            var toolTipOptions = this.getTooltipOptions(seriesColor, series);
            var rectBorderColor = toolTipOptions.rectBColor;
            var rectFillColor = toolTipOptions.rectFColor;
            var textColor = toolTipOptions.rectTextColor;
            var rX = toolTipOptions.rectX;
            var rY = toolTipOptions.rectY;
            var rectOptions = series.tooltip;
            //draw tooltip rectangle
            var tooltipdivRect;
            var padding = 5;

            if (document.getElementById(this.svgObject.id + "_TrackToolTip") == null)
                tooltipdivRect = $("<div></div>").attr({ 'id': this.svgObject.id + "_TrackToolTip", 'class': 'ejTooltip' + this._id });               
            else
                tooltipdivRect = $("#" + this.svgObject.id + "_TrackToolTip");
			var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (ej.isTouchDevice() && !isIE11 && !this.vmlRendering) {
                if (window.navigator.msPointerEnabled)
                    this._on(tooltipdivRect, 'MSPointerMove', this.chartTooltipHover);
                else
                    this._on(tooltipdivRect, 'touchmove', this.chartTooltipHover);
            }
            else {
                this._on(tooltipdivRect, 'mousemove', this.chartTooltipHover);
            }
            $("#" + this.svgObject.id + "_TrackToolTip").show();
			if (!parentZindex)
                parentZindex = $('#' + this._id)[0].offsetParent.style.zIndex;
            var rectOptions = {
                'top': y + $(document).scrollTop(),
                'left': x + $(document).scrollLeft(),
                "background-color": (rectOptions.fill) ? rectOptions.fill : rectFillColor,
                "border-style": "solid",
                "position": "absolute",
                "border-color": (rectOptions.border.color) ? rectOptions.border.color : rectBorderColor,
                "border-width": (rectOptions.border.width || rectOptions.border.width==0) ? rectOptions.border.width : 1,
                "opacity": rectOptions.opacity,
				 'z-index': parentZindex + 1000000,
                'border-radius': ((rectOptions.rx) ? rectOptions.rx : rX).toString() + "px " + ((rectOptions.ry) ? rectOptions.ry : rY).toString() + "px",               
                "padding-left": '5px',
				 "padding-right": '5px',
                "padding-top": '2px',
                "padding-bottom": '2px'
            };
            $(tooltipdivRect).css(rectOptions);

            //event for tooltip text
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { seriesIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText.text, isTrendLine: series.isTrendLine, trendlineIndex: series.trendlineIndex };
            this._trigger("toolTipInitialize", commonTrackTextArgs);
 	        if (!commonTrackTextArgs.cancel) {
            //draw tooltip text
            var fontSize = 0;
            var tooltip = series.tooltip;
            var font = (tooltip.font) ? $.extend(false, series.font, {}, tooltip.font) : series.font;
            
            // draw tooltip text styles
            var textOptions = {
                'top': y + $(document).scrollTop(),
                'left': x + $(document).scrollLeft(),
                "color": (tooltip.font && tooltip.font.color) ? font.color : textColor,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                "align-self": "baseline"
            };
            $(tooltipdivRect).css(textOptions);

            // append the tooltip Div (condition for performance without remove div)
            if (document.getElementById(this.svgObject.id + "_TrackToolTip") == null)
                $(document.body).append(tooltipdivRect[0]);

            // Add string in text array for tooltip format
            var text = measureText = commonTrackTextArgs.data.currentText;
            $("#" + this.svgObject.id + "_TrackToolTip").html(text);

            // div size calculation perform here
            var textHeight = 0;
            var textWidth = 0;
            text = text.replace(/<br >/g, "<br/>")
                       .replace(/<br \/>/g, "<br/>")
                       .replace(/<br>/g, "<br/>");
            text = text.split("<br/>");
            for (var i = 0; i < text.length; i++) {
                text[i] = text[i];
                textHeight += ej.EjSvgRender.utils._measureText(text[i], null, font).height;
                var width = ej.EjSvgRender.utils._measureText(text[i], null, font).width;
                if (textWidth < width)
                    textWidth = width;
            }

            fontSize = (this.model.AreaType == "cartesianaxes") ? ej.EjSvgRender.utils._measureText(measureText, series.xAxis.width, font).height : ej.EjSvgRender.utils._measureText(text, null, font).height;
            var maxWidth = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.yAxis.width : series.xAxis.width : $(this.svgObject).width() - (this.model.legend.position.toLowerCase() == "right" ? (this.model.LegendViewerBounds.Width + 2 * this.model.elementSpacing) : 0);
            var maxHeight = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.xAxis.height : series.yAxis.height : $(this.svgObject).height() - (this.model.legend.position.toLowerCase() == "bottom" ? (this.model.LegendViewerBounds.Height + this.model.elementSpacing) : 0);

            //Adjust x-position to display tooltip within chart area 
            var box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();

            $("#" + this.svgObject.id + "_TrackToolTip").css("top", ($(document).scrollTop() + box.top + position.top - ((textHeight + 4) / 2)));
			$("#" + this.svgObject.id + "_TrackToolTip").css("left",($(document).scrollLeft() + box.left + position.left + (box.left - x))); 
            if ((x + (textWidth + padding)) >= maxWidth +  series.xAxis.x ) {
                var areaPos = document.getElementById(this.svgObject.id).getClientRects()[0];
                var diff = x - ((textWidth + padding * 2) + ((!series.isIndicator && !series.isTrendLine) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ?
                series.marker.size.width : (type == "bubble" ? (point.radius + bubbleSpacing) : 0)) : series._trackMarker.size.width) : 0) + (2 * padding));
                $("#" + this.svgObject.id + "_TrackToolTip").css("left", diff + areaPos.left - (padding) + $(document).scrollLeft());
            }
            box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();
            if (box.bottom >= (series.yAxis.y + maxHeight + position.top) || box.bottom >= (series.xAxis.y + maxHeight + position.top)) {
                var diffY = y - (((box.bottom - box.top) + ((!series.isIndicator && !series.isTrendLine) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.height : 0) : series._trackMarker.size.height) : 0))) + padding + position.top;
                $("#" + this.svgObject.id + "_TrackToolTip").css("top", diffY + $(document).scrollTop());
             }
            //adjust rectangle size based on text length  
            box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();
			if (box.left < 0) {
			// checked condition for whether left position display outsude of the chart area
                $("#" + this.svgObject.id + "_TrackToolTip").css("left", x);
            }
			if (box.top < position.top || box.top < 0)
			    $("#" + this.svgObject.id + "_TrackToolTip").css("top", (position.top < 0 ? $(document).scrollTop() : position.top) + 'px');
            $("#" + this.svgObject.id + "_TrackToolTip").show();
        }
        }
    },
	chartTooltipHover: function (evt) {
        var box = $("#" + evt.target.id)[0].getBoundingClientRect();
		var diff = evt.clientX - box.left;
		var padding = 5;
		$("#" + evt.target.id).css("left", box.left + diff + padding);
		box = $("#" + evt.target.id)[0].getBoundingClientRect();
		var areaPos = document.getElementById(this.svgObject.id).getClientRects()[0];
		if (box.right > areaPos.right + padding)
				$("#" + evt.target.id).css("left", evt.clientX - (box.right - box.left) - padding);
    },
    _initializeSeriesColors: function () {
        var chartObj = this;
        var modelColor = (chartObj.model.palette) ? chartObj.model.palette : chartObj.model.colors;
        var count = modelColor.length;
        var areaType = chartObj.model.AreaType;
        chartObj.model.seriesColors = [];
        chartObj.model.seriesBorderColors = [];
        chartObj.model.pointColors = [];
        chartObj.model.pointBorderColors = [];
        var elementCollection, color, borderColor, element, tlines,visibleSeriesLength =chartObj.model._visibleSeries.length,visibleSeries = chartObj.model._visibleSeries
        if (areaType != "none" ) {
            elementCollection = chartObj.model._visibleSeries;
            color = chartObj.model.seriesColors;
            borderColor = chartObj.model.seriesBorderColors;
        }
        else {
            var lgth = 0;
            var longest;
            for (var i = 0; i < visibleSeriesLength; i++) {
                var type = chartObj.model._visibleSeries[i].type.toLowerCase();
                if (chartObj.model._visibleSeries[i].points.length > lgth) {
                    var lgth = chartObj.model._visibleSeries[i].points.length;
                    longest = chartObj.model._visibleSeries[i].points;
                }
                else if (chartObj.model._visibleSeries[i].points.length == lgth && type != "funnel" && type != "pieofpie" && type != "pyramid")
                    longest = chartObj.model._visibleSeries[i].points;
				}
           elementCollection = longest;
            color = chartObj.model.pointColors;
            borderColor = chartObj.model.pointBorderColors;
        }
        if (areaType == "none" && visibleSeriesLength > 1)
        {
            for (var k = 0; k < visibleSeriesLength; k++)
            {
                chartObj.model.seriesColors[k] = modelColor[k % count];
            }
        }
        for (var i = 0; i < elementCollection.length; i++) {
            element = elementCollection[i];
            tlines = element.trendlines;
                element.isFill = (element.isFill == undefined) ? ((element.fill && element.fill != "") ? true : false) : element.isFill;
                if (element.isFill) {
                    color[i] = element.fill;
                } else {
                    color[i] = modelColor[i % count];
                    if (!element._hiloTypes)
                        element.fill = color[i];
                }
                if (areaType != "none" && tlines.length > 0)
                    this._initializeTrendlinesColors(tlines, color[i]);
                if (!ej.util.isNullOrUndefined(element.border) && !ej.util.isNullOrUndefined(element.border.color) && element.border.color != "") {
                    borderColor[i] = element.border.color;
                } else {
                    borderColor[i] = chartObj.model.seriesBorderDefaultColors[i % 10];
                    if (areaType != "none")
                        element.border.color = borderColor[i];
                }
                element.dataPoint = element.points;
            } 
    },

    _initializeTrendlinesColors: function(tlines, color) {
        for (var k in tlines) {
            if (ej.util.isNullOrUndefined(tlines[k].fill) || tlines[k].fill == "")
                tlines[k].fill = color;
        }
    },

    _createAxisLabelAndRange: function () {
        var count = 0,
            model = this.model,
            seriesCollection = model.series,
            seriesLength = seriesCollection.length,
            indicatorSeries,
            series,
            axisScroll,
            scrollRange,
            axis,
            primaryX = model.primaryXAxis,
            primaryY = model.primaryYAxis,
            axesLength;

        for (var i = 0; i < seriesLength; i++) {
            series = seriesCollection[i];
            if (series._isTransposed)
                count++;
        }

        if (seriesLength) {
            if (count == seriesLength)
                model.requireInvertedAxes = true;
            else {
                series = seriesCollection[0];
                if (series._isTransposed)
                    model.requireInvertedAxes = true;
            }
        }        
        if (model.requireInvertedAxes) {
            primaryY.orientation = "horizontal";
            primaryX.orientation = "vertical";
            primaryY.labelPlacement = (!(primaryY.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.OnTicks : primaryY.labelPlacement;
        }
        else {
            primaryY.orientation = "vertical";
            primaryX.orientation = "horizontal";
        }
		 
        primaryY.name = ej.util.isNullOrUndefined(primaryY.name) ? "SecondaryAxis" : primaryY.name;
        primaryX.name = ej.util.isNullOrUndefined(primaryX.name) ? "PrimaryAxis" : primaryX.name;

        model._axes = [];
        model._axes[0] = primaryX;
        model._axes[1] = primaryY;
       
        axesLength = model.axes.length;
        indicatorSeries = model.indicators;

		for (var i = 0; i < axesLength; i++) {
            axis = model.axes[i];
			if (axis.name) {
			//Sets orientation for axes collections
            for (var j = 0; j < seriesLength; j++) {
                var type = seriesCollection[j].type.toLowerCase();
                if (seriesCollection[j].xAxisName == axis.name) {
                    axis.orientation = (type.indexOf("bar") != -1) ? seriesCollection[j].isTransposed ? "horizontal" : "vertical" : seriesCollection[j].isTransposed ? "vertical" : "horizontal";
                        break;
                    }
                else if (seriesCollection[j].yAxisName == axis.name) {
                    axis.orientation = (type.indexOf("bar") != -1) ? seriesCollection[j].isTransposed ? "vertical" : "horizontal" : seriesCollection[j].isTransposed ? "horizontal" : "vertical";
                            break;
                        }
                 }
				  for (var k = 0; k < indicatorSeries.length; k++) {
                    if (indicatorSeries[k].xAxisName == axis.name) {
                        axis.orientation ="horizontal";
                        break;
                    }
                    else if (indicatorSeries[k].yAxisName == axis.name) {
                        axis.orientation ="vertical";
                        break;
                    }
                 }
		    if (!axis.orientation) continue;
            if (axis.orientation.toLowerCase() == "horizontal")
                model.axes[i] = $.extend(true, {}, model.secondaryX, axis);
            else
                model.axes[i] = $.extend(true, {}, model.secondaryY, axis);
            model._axes.push(model.axes[i]);
		  }
        }
        for (var k = 0; k < model._axes.length; k++) {
             var axis = this.model._axes[k],
                axisScrollSetings = axis.scrollbarSettings,
                scrollRange = axisScrollSetings.range,
                orientation = axis.orientation.toLowerCase();
            axis.visibleLabels = [];
            axis.range = (axis.range.min == null && axis.range.max == null && axis.range.interval == null) ? null : axis.range;            
            axis.setRange = (!(axis.range)) ? false : (ej.util.isNullOrUndefined(axis.setRange)) ? true : axis.setRange;
            axis.setRange = ((axis.actual_Range) || (axis.setRange != false)) ? true : false;
            axis.actualRange = (!(axis.actualRange)) ? (axis.range == null) ? {} : $.extend(true, {}, axis.range) : axis.actualRange;
            axis.visibleRange = (!(axis.visibleRange)) ? (axis.range == null) ? {} : axis.range : axis.visibleRange;
            axis.name = !(axis.name) ? k.toString() : axis.name;
            axis._isScroll = ((axisScrollSetings.visible && (axisScrollSetings.pointsLength != null || scrollRange.min != null || scrollRange.max != null)) || ((model.zooming.enableScrollbar && axisScrollSetings.visible) && (axis.zoomFactor < 1 || axis.zoomPosition > 0)))
            axis._pointsLength = axisScrollSetings.pointsLength;
            if (typeof scrollRange.min == "string" && scrollRange.min.indexOf("/Date(") != -1)
                scrollRange.min = new Date(parseInt(scrollRange.min.substr(6)));
            if (typeof scrollRange.max == "string" && scrollRange.max.indexOf("/Date(") != -1)
                scrollRange.max = new Date(parseInt(scrollRange.max.substr(6)));
        }
    },






    _drawTitle: function () {
        var title = this.model.title;
        var margin = this.model.margin;
        var svgWidth = $(this.svgObject).width();
        var measureTitle = ej.EjSvgRender.utils._measureText(title.text, svgWidth - margin.left - margin.right, title.font);
        var elementSpacing = this.model.elementSpacing;
        var modelTitleHeight = (title.text == "" || !title.visible) ? 0 : (measureTitle.height + elementSpacing);
        var margin = margin;
        var areaBounds = this.model.m_AreaBounds;
        var areaType = this.model.AreaType;
        var legendSpace = (this.model.legend.position.toLowerCase() == "left") ? this.model.LegendViewerBounds.Width / 2 : ((this.model.legend.position.toLowerCase() == "right" ? (-this.model.LegendViewerBounds.Width / 2) : 0));
        // Drawing the chart title
        var leftSpace = margin.left + elementSpacing + this.model.border.width;
        var rightSpace = margin.right + elementSpacing + this.model.border.width;
        var collection = this.model.titleWrapTextCollection;

        var titleLocation, textBorderConstant = 1.2,
            titleFontSize = parseInt(title.font.size),
            titleTextHeight = parseInt(title.font.size) * textBorderConstant,
            enableTrim = title.enableTrim, textAnchor,
            maxTitleWidth = title.maximumWidth,
            textAlignment = title.textAlignment.toLowerCase(),
            textOverflow = title.textOverflow.toLowerCase(),
            titleText = title.text,
            titleEnable = enableTrim && (textOverflow == "wrap" || textOverflow == "wrapandtrim") && (collection.length > 1) ? true : false;
            if (maxTitleWidth.toString() == 'auto' || maxTitleWidth.toString() == '') {
                maxTitleWidth = (areaBounds.Width * 0.75);
                this.model.titleMaxWidth = maxTitleWidth;
              }
            else
                 maxTitleWidth = parseInt(maxTitleWidth);
       
        var titleWidthEnable = measureTitle.width > maxTitleWidth ? true : false;
        //title trim
        if (enableTrim && titleWidthEnable && textOverflow == "trim") {
            titleText = ej.EjSvgRender.utils._trimText(titleText, maxTitleWidth, title.font);
            this.model.titleTrim = measureTitle = ej.EjSvgRender.utils._measureText(titleText, svgWidth - margin.left - margin.right, title.font);
            this.model.trimTooltip = true;
        }

        if (this.model.trimTooltip && textOverflow == "wrap")
            this.model.trimTooltip = false;

        var textSize = { width: ((this.model.trimTooltip && titleEnable) || collection.length > 1) ? this.model.titleMaxWidth : measureTitle.width, height: (measureTitle.height * (collection.length > 0 ? collection.length - 1 : 1)) };
        if (title.text != "" && title.text != null) {
            if ((title.border.color == 'transparent') && (title.background == 'transparent')) {
                titleLocation = margin.top + (modelTitleHeight / 2) + (elementSpacing);
            } else {
                titleLocation = margin.top + (titleTextHeight / 2) + (elementSpacing) + (titleFontSize / 4);
            }
            var locX = ((areaType != "cartesianaxes") ? (svgWidth - margin.left - margin.right) / 2 + (margin.left + legendSpace) : (leftSpace + (svgWidth - rightSpace) / 2)) - textSize.width / 2;

            if (this.model.title.textAlignment.toLowerCase() == "near") {
                locX = leftSpace;
            }
            else if (this.model.title.textAlignment.toLowerCase() == "far") {
                locX = (svgWidth - rightSpace - textSize.width);
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { title: titleText, location: { x: locX, y: titleLocation }, size: measureTitle };
            this._trigger("titleRendering", commonEventArgs);
            var options = {
                'id': this.svgObject.id + '_ChartTitleText',
                'x': commonEventArgs.data.location.x,
                'y': commonEventArgs.data.location.y,
                'fill': title.font.color,
                'font-size': title.font.size,
                'font-family': title.font.fontFamily,
                'font-style': title.font.fontStyle,
                'font-weight': title.font.fontWeight,
                'opacity': title.font.opacity,
                'text-anchor': 'start'
            };
            var borderOptions = {
                'id': this.svgObject.id + '_ChartTitleBorder',
                'x': commonEventArgs.data.location.x - elementSpacing,
                'y': commonEventArgs.data.location.y - titleTextHeight + (titleTextHeight / 4),
                'rx': title.border.cornerRadius,
                'ry': title.border.cornerRadius,
                'width': titleEnable && titleWidthEnable ? this.model.titleMaxWidth + (2 * elementSpacing) : measureTitle.width + (2 * elementSpacing),
                'height': titleEnable && titleWidthEnable ? measureTitle.height * collection.length : titleTextHeight,
                'fill': title.background,
                'stroke-width': title.border.width,
                'stroke': title.border.color,
                'opacity': title.border.opacity,
				'class': "e-titleborder"
            };
            this.svgRenderer.drawRect(borderOptions, this.gTitleEle);

            if (titleEnable && titleWidthEnable) {
                for (var k = 0; k < collection.length; k++) {
                    options.id = this.svgObject.id + '_ChartTitleText_' + k;
                    if (k != 0) options.y = options.y + measureTitle.height;
                    this.svgRenderer.drawText(options, collection[k], this.gTitleEle);
                }
            }
            else this.svgRenderer.drawText(options, commonEventArgs.data.title, this.gTitleEle);           

            this.model._titleLocation = { X: commonEventArgs.data.location.x, Y: commonEventArgs.data.location.y, size: textSize, _height: measureTitle.height };

            this.svgRenderer.append(this.gTitleEle, this.svgObject);
            if (title.subTitle.text != "" && title.subTitle.text && title.subTitle.visible)
                this._drawSubTitle();
        }
    },



    _getLegendSize: function (series) {

        var legend = this.model.legend,
            symbolSize = legend.itemStyle,
            textSize = ej.EjSvgRender.utils._measureText(series.Text, null, series.Font),
            padding = 10,
            width = symbolSize.width + padding + textSize.width,
            height = Math.max(symbolSize.height, textSize.height);

        return { Width: width, Height: height };

    },

    _getYValues: function (points) {
        var values = [],
			length = points.length;
        for (var i = 0; i < length; i++)
            values.push(points[i].y);
        return values;
    },
    _getXValues: function (points) {
         var values = [],
			length = points.length;
        for (var i = 0; i < length; i++)
            values.push(points[i].xValue);
        return values;
    },




    _drawSubTitle: function () {

        var title = this.model.title;
        var subTitle = title.subTitle;
        var svgWidth = $(this.svgObject).width();
        var measureTitle = ej.EjSvgRender.utils._measureText(title.text, svgWidth - this.model.margin.left - this.model.margin.right, title.font);
        var measuresubTitle = ej.EjSvgRender.utils._measureText(subTitle.text, svgWidth - this.model.margin.left - this.model.margin.right, subTitle.font);

        var textanchor = "middle";
        var subTitleLocation, textBorderConstant = 1.2, titleBorderSpacing = 10, subTitleBorderSpacing = 10, elementSpacing = this.model.elementSpacing,
           subTitleFontSize = parseInt(subTitle.font.size),
           areaBounds = this.model.m_AreaBounds.Width,
           titleTextHeight = parseInt(title.font.size) * textBorderConstant,
           subTitleTextHeight = parseInt(subTitle.font.size) * textBorderConstant,
           modelsubTitleHeight = (subTitle.text == "" || !subTitle.visible) ? 0 : (measuresubTitle.height + elementSpacing),
           enableTrim = subTitle.enableTrim,
           maxSubTitleWidth = subTitle.maximumWidth,
           maxTitleWidth = title.maximumWidth,
           textOverflow = subTitle.textOverflow.toLowerCase(),
           titleTextAlignment = title.textAlignment.toLowerCase(),
           subTitleTextAlignment = subTitle.textAlignment.toLowerCase(),
           titleText = title.text,
           titleEnableTrim = title.enableTrim,
           titleTextOverflow = title.textOverflow.toLowerCase(),
           subTitleText = subTitle.text, locX,
           regionX,
           titleVisibility = titleEnableTrim && (titleTextOverflow == "wrap" || titleTextOverflow == "wrapandtrim") ? true : false,
           titleWidthVisibility,
           subTitleVisibility = enableTrim && (textOverflow == 'wrap' || textOverflow == 'wrapandtrim') ? true : false;

        maxTitleWidth = (maxTitleWidth.toString() == 'auto' || maxTitleWidth.toString() == '') ? (areaBounds * 0.75) : parseInt(maxTitleWidth);
        maxSubTitleWidth = (maxSubTitleWidth.toString() == 'auto' || maxSubTitleWidth.toString() == '') ? (areaBounds * 0.75) : parseInt(maxSubTitleWidth);
        var measureTitle = ej.EjSvgRender.utils._measureText(title.text, svgWidth - this.model.margin.left - this.model.margin.right, title.font);
        var titleWidthEnable = measureTitle.width > maxTitleWidth ? true : false;

        //subtitle trim
        if (enableTrim && measuresubTitle.width > maxSubTitleWidth && textOverflow == "trim") {
            titleText = titleEnableTrim && titleTextOverflow == 'trim' ? ej.EjSvgRender.utils._trimText(titleText, maxTitleWidth, title.font) : titleText;
            subTitleText = ej.EjSvgRender.utils._trimText(subTitleText, maxSubTitleWidth, subTitle.font);
            measureTitle = ej.EjSvgRender.utils._measureText(titleText, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, title.font);
            measuresubTitle = ej.EjSvgRender.utils._measureText(subTitleText, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, subTitle.font);
            this.model.subTitleTooltip = true;
        }
        titleWidthVisibility = measureTitle.width > maxTitleWidth;
        // Drawing the chart subtitle

        if (subTitleText != "" && subTitleText != null) {
            if (((this.model.title.border.color == 'transparent') && (this.model.title.background == 'transparent')) || ((this.model.title.subTitle.border.color == 'transparent') && (this.model.title.subTitle.background == 'transparent'))) {
                subTitleLocation = (modelsubTitleHeight) / 2 + elementSpacing + this.model._titleLocation.Y + this.model._titleLocation._height * (this.model.titleWrapTextCollection ? (this.model.titleWrapTextCollection.length - 1) : 0 );
            } else {
                subTitleLocation = (subTitleTextHeight / 2) + subTitleBorderSpacing + elementSpacing + ((titleVisibility && titleWidthVisibility)?(measureTitle.height * this.model.titleWrapTextCollection.length) : titleTextHeight) + titleBorderSpacing + (subTitleFontSize / 3);
            }
            var locX = this.model._titleLocation.X + (this.model._titleLocation.size.width / 2);
           
            if (subTitleTextAlignment == "near") {                
                   locX = regionX= this.model._titleLocation.X;
                   textanchor = "start";
            }
            else if (subTitleTextAlignment == "far") {
                locX = this.model._titleLocation.X + this.model._titleLocation.size.width;
                textanchor = "end";
               
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { subTitle: subTitleText, location: { x: locX, y: subTitleLocation }, size: measuresubTitle };
            this._trigger("subTitleRendering", commonEventArgs);
            var subTitleYposition = (((title.border.color == 'transparent') && (title.background == 'transparent')) || ((title.subTitle.border.color == 'transparent') && (title.subTitle.background == 'transparent'))) ? commonEventArgs.data.location.y : commonEventArgs.data.location.y - (elementSpacing / 2);

            var options = {
                'id': this.svgObject.id + '_ChartSubTitleText',

                'x': commonEventArgs.data.location.x,
                'y': subTitleYposition,
                'fill': subTitle.font.color,
                'font-size': subTitle.font.size,
                'font-family': subTitle.font.fontFamily,
                'font-style': subTitle.font.fontStyle,
                'font-weight': subTitle.font.fontWeight,
				'opacity': subTitle.font.opacity,
                'text-anchor': textanchor
            };

            var x, y, subTitleXLocation = commonEventArgs.data.location.x;
            if (textanchor == "start") x = subTitleXLocation - elementSpacing;
            else if (textanchor == "end") 
                x = subTitleVisibility ? subTitleXLocation - elementSpacing - this.model.subTitleMaxWidth : subTitleXLocation - elementSpacing - (measuresubTitle.width);
            else 
                x = subTitleVisibility ? subTitleXLocation - elementSpacing - (this.model.subTitleMaxWidth / 2) : subTitleXLocation - elementSpacing - (measuresubTitle.width / 2);
            y = (((title.border.color == 'transparent') && (title.background == 'transparent')) || ((subTitle.border.color == 'transparent') && (subTitle.background == 'transparent'))) ? commonEventArgs.data.location.y - subTitleTextHeight + (subTitleTextHeight / 4)  : commonEventArgs.data.location.y - subTitleTextHeight + (subTitleTextHeight / 4) - (elementSpacing / 2);
            var borderOptions = {
                'id': this.svgObject.id + '_ChartSubTitleBorder',
                'x': x,
                'y': y,
                'rx': subTitle.border.cornerRadius,
                'ry': subTitle.border.cornerRadius,
                'width': subTitleVisibility ? this.model.subTitleMaxWidth + (2 * elementSpacing) : measuresubTitle.width + (2 * elementSpacing),
                'height': subTitleVisibility ? measuresubTitle.height * this.model.subTitleWrapTextCollection.length : subTitleTextHeight,
                'fill': subTitle.background,
                'stroke-width': subTitle.border.width,
                'stroke': subTitle.border.color,
                'opacity': subTitle.border.opacity,
				'class': "e-subtitleborder"
            };

            this.svgRenderer.drawRect(borderOptions, this.gSubTitleEle);
            
            if (subTitleVisibility && measuresubTitle.width > maxSubTitleWidth) {
                for (var k = 0; k < this.model.subTitleWrapTextCollection.length; k++) {
                    options.id = this.svgObject.id + '_ChartSubTitleText_' + k;
                    if (k != 0) options.y = options.y + measuresubTitle.height;
                    this.svgRenderer.drawText(options, this.model.subTitleWrapTextCollection[k], this.gSubTitleEle);
                }
                
            }
            else this.svgRenderer.drawText(options, commonEventArgs.data.subTitle, this.gSubTitleEle);          

            var subSize = { width: (this.model.subTitleTooltip) ? maxSubTitleWidth : measuresubTitle.width, height: (measuresubTitle.height * (this.model.subTitleWrapTextCollection.length)) };
            if (textanchor == "middle")
                regionX = locX - subSize.width / 2;
            else if (textanchor == "end")
                regionX = locX - subSize.width;
            this.model._subTitleLocation = { X: regionX, Y: (commonEventArgs.data.location.y - measuresubTitle.height/2), size: subSize };

            this.svgRenderer.append(this.gSubTitleEle, this.gTitleEle);
        }
    },


    _calculateStackingValues: function (seriesCollection, axis, params) {

        var chartModel = this.model;
        chartModel.stackedValue[axis.name] = [];
        var stackedValue = chartModel.stackedValue[axis.name];
        stackedValue.min = 0;
        stackedValue.max = 0;
        var stackAxes = (!chartModel.requireInvertedAxes) ? chartModel.hAxes : chartModel.vAxes,
            stackAxesLength = stackAxes.length,visiblePointsLength, yValues,stackingGroup,
            seriesLength = seriesCollection.length, visiblePoints,
            lastPosValue, lastNegValue, values, lastValue, currentValue,
            isNul = ej.util.isNullOrUndefined;

        for (var k = 0; k < stackAxesLength; k++) {
            j = 0;
            var lastPosValue = [], lastNegValue = [], firstPositive = [], firstNegative = [],
            crossing = 0;
            for (var i = 0; i < seriesCollection.length; i++) {
                if (seriesCollection[i]._xAxisName == stackAxes[k].name) {
                    if (seriesCollection[i].type.toLowerCase().indexOf("stacking") != -1 || seriesCollection[i].isStacking) {
                        values = {};
                        values.StartValues = [];
                        values.EndValues = [];
                        stackingGroup = seriesCollection[i].stackingGroup;
                        if (stackingGroup && seriesCollection[i].type.toLowerCase() != "stackingarea") {
                            if (isNul(lastPosValue[stackingGroup])) {
                                lastPosValue[stackingGroup] = [];
                                lastNegValue[stackingGroup] = [];
                                firstPositive[stackingGroup] = [];
                                firstNegative[stackingGroup] = [];
                            }
                        }

                        else {
                            stackingGroup = "";
                            if (isNul(lastPosValue[stackingGroup])) {
                                lastPosValue[stackingGroup] = [];
                                lastNegValue[stackingGroup] = [];
                                firstPositive[stackingGroup] = [];
                                firstNegative[stackingGroup] = [];
                            }
                        }
                        series = seriesCollection[i];
						crossing = this._getXCrossValue(series, stackAxes[k], params);
						if(typeof(crossing) !== 'number')
							crossing = 0;
                        visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(series);
                        yValues = this._getYValues(series._visiblePoints);
                        visiblePointsLength = visiblePoints.length;
                        for (var j = 0; j < visiblePointsLength; j++) {
                            var lastValue = 0, correction=0, currentValue = yValues[j];
							if (isNul(lastPosValue[stackingGroup][series._visiblePoints[j].xValue])) {
                                lastPosValue[stackingGroup][series._visiblePoints[j].xValue] = crossing;
                            }
                            if (isNul(lastNegValue[stackingGroup][series._visiblePoints[j].xValue])) {
                                lastNegValue[stackingGroup][series._visiblePoints[j].xValue] = crossing;
                            }
                            if (values.StartValues.length <= j) {
                                values.StartValues.push(0);
                                values.EndValues.push(0);
                            }
                            if (currentValue >= 0) {
                                firstPositive[stackingGroup][j] = firstPositive[stackingGroup][j] == null;
                                currentValue -= firstPositive[stackingGroup][j] && crossing;
                                lastValue = lastPosValue[stackingGroup][series._visiblePoints[j].xValue];
                                lastPosValue[stackingGroup][series._visiblePoints[j].xValue] += currentValue;
                                correction = lastValue < crossing ? crossing - lastValue : 0;
                                values.StartValues[j] = lastValue + correction;
                                values.EndValues[j] = currentValue + lastValue;
                            }
                            else {
                                firstNegative[stackingGroup][j] = firstNegative[stackingGroup][j] == null;
                                currentValue -= firstNegative[stackingGroup][j] && crossing;
                                lastValue = lastNegValue[stackingGroup][series._visiblePoints[j].xValue];
                                lastNegValue[stackingGroup][series._visiblePoints[j].xValue] += currentValue;
                                values.StartValues[j] = lastValue;
                                values.EndValues[j] = currentValue + lastValue;
                                if (crossing < values.EndValues[j]) {
                                    lastNegValue[stackingGroup][series._visiblePoints[j].xValue] = values.StartValues[j];
                                    lastPosValue[stackingGroup][series._visiblePoints[j].xValue] = values.EndValues[j];
                                    firstPositive[stackingGroup][j] = firstPositive[stackingGroup][j] == null;
                                }
                            }
                            
                            visiblePointIndex = visiblePoints[j].actualIndex;
                            series.points[visiblePointIndex].YValues = [values.EndValues[j]];
                        }
                        values.stackedSeries = true;
                        stackedValue.push(values);
                        if (stackedValue.min > Math.min.apply(0, values.StartValues))
                            stackedValue.min = Math.min.apply(0, values.StartValues);
                        if (stackedValue.max < Math.max.apply(0, values.EndValues))
                            stackedValue.max = Math.max.apply(0, values.EndValues);
                        series.stackedValue = values;
                    }
                }
            }
        }
    },
    _calculateStackingCumulativeValues: function (seriesCollection, axis, params) {
        this.model.stackedValue[axis.name] = [];
        this.model.stackedValue[axis.name].min = 0;
        this.model.stackedValue[axis.name].max = 0;
        var stackAxes = (!this.model.requireInvertedAxes) ? this.model.hAxes : this.model.vAxes;
        var seriesLength = seriesCollection.length;
        var axesLength = stackAxes.length;
        var percent;
        for (var k = 0; k < axesLength ; k++) {
            var posValues = [], negValues = [], frequencies = [], stackingName = " ", firstPositive = [], firstNegative = [], crossing = 0, seriesGroupCount=[];

            //calculate the cumulative frequencies from the collection of series  
            for (var i = 0; i < seriesLength; i++) {
                var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(seriesCollection[i]);
                stackingName = seriesCollection[i].stackingGroup;
                if (seriesCollection[i].type.toLowerCase().indexOf("100") != -1) {
                    if (seriesCollection[i]._xAxisName == stackAxes[k].name) {
                        if (seriesCollection[i].stackingGroup && seriesCollection[i].type.toLowerCase() != " stackingarea100 ") {
                            if (!posValues[stackingName]) {
                                frequencies[stackingName] = [];
                                posValues[stackingName] = [];
                                negValues[stackingName] = [];
                                firstPositive[stackingName] = [];
                                firstNegative[stackingName] = [];
                            }
                        }
                        else {
                            seriesCollection[i].stackingGroup = "";
                            stackingName = seriesCollection[i].stackingGroup;
                            if (!posValues[stackingName]) {
                                frequencies[stackingName] = [];
                                posValues[stackingName] = [];
                                negValues[stackingName] = [];
                                firstPositive[stackingName] = [];
                                firstNegative[stackingName] = [];
                            }
                        }
                        seriesGroupCount[stackingName] = i;
                        var yValues = this._getYValues(seriesCollection[i]._visiblePoints);
                        var yValuesLength = yValues.length;
                        for (var j = 0; j < yValuesLength; j++) {
                            var getYValue = yValues[j];
                            if (!frequencies[stackingName][seriesCollection[i].points[j].xValue]) {
                                frequencies[stackingName][seriesCollection[i].points[j].xValue] = 0;
                            }
                            frequencies[stackingName][seriesCollection[i].points[j].xValue] += Math.abs(getYValue);
                        }
                    }
                }
              
            }

            //calculate the cumulative percentage for each series points
            for (var i = 0; i < seriesLength; i++) {
                stackingName = seriesCollection[i].stackingGroup;
                if (seriesCollection[i].type.toLowerCase().indexOf("100") != -1) {
                    var stackedSeries = false;
                    if (seriesCollection[i]._xAxisName == stackAxes[k].name) {
                        var values = {};
                        values.StartValues = [];
                        values.EndValues = [];
                        series = seriesCollection[i];
                        crossing = this._getXCrossValue(series, stackAxes[k], params);
						if(typeof(crossing) !== 'number')
							crossing = 0;
                        var yValues = this._getYValues(series._visiblePoints);
                        var yValuesLength = yValues.length;
                        for (var j = 0; j < yValuesLength; j++) {
                            var lastValue, correction = 0;
                            var currentValue = yValues[j];

                            //Pecentage calculation for point 
                            percent = (currentValue / frequencies[stackingName][series.points[j].xValue]) * 100
                            currentValue = !isNaN(percent) ? percent :0;
                            series.points[j].percentage = currentValue.toFixed(2);
                            if (!posValues[stackingName][series.points[j].xValue]) {
                                posValues[stackingName][series.points[j].xValue] = crossing;
                            }
                            if (!negValues[stackingName][series.points[j].xValue]) {
                                negValues[stackingName][series.points[j].xValue] = crossing;
                            }
                            if (values.StartValues.length <= j) {
                                values.StartValues.push(0);
                                values.EndValues.push(0);
                            }
                            if (currentValue >= 0) {
                                var posValue = posValues[stackingName][series.points[j].xValue];
                                firstPositive[stackingName][j] = firstPositive[stackingName][j] == null;
                                currentValue -= firstPositive[stackingName][j] && crossing;
                                correction = posValue < crossing ? crossing - posValue : 0;
                                lastValue = posValue;
                                posValues[stackingName][series.points[j].xValue] += currentValue;
                            }
                            else {
                                var negValue = negValues[stackingName][series.points[j].xValue];
                                firstNegative[stackingName][j] = firstNegative[stackingName][j] == null;
                                currentValue -= firstNegative[stackingName][j] && crossing;
                                correction = negValue > crossing ? crossing - negValue : 0;
                                lastValue = negValue;
                                negValues[stackingName][series.points[j].xValue] += currentValue;
                            }
                            values.StartValues[j] = lastValue + correction;
                            values.EndValues[j] = lastValue + currentValue;
                            if (values.EndValues[j] > 100) values.EndValues[j] = 100;
                            series.points[j].YValues = [values.EndValues[j]];
                            stackedSeries =  seriesGroupCount[stackingName] == i ? false || (stackedSeries): true;
                        }
                        values.stackedSeries = stackedSeries;
                        this.model.stackedValue[axis.name].push(values);
                        if (this.model.stackedValue[axis.name].min > Math.min.apply(0, values.StartValues))
                            this.model.stackedValue[axis.name].min = Math.min.apply(0, values.StartValues);
                        if (this.model.stackedValue[axis.name].max < Math.max.apply(0, values.EndValues))
                            this.model.stackedValue[axis.name].max = Math.max.apply(0, values.EndValues);
                        if (this.model.stackedValue[axis.name].min > Math.min.apply(0, values.EndValues))
                            this.model.stackedValue[axis.name].min = -100;

                        series.stackedValue = values;
                    }
                }
            }
        }
    },
    _legendItemBounds: function (itemCount, legendItemWidth, legendItemHeight) {
         var legend = this.model.legend, legVal,
            position = legend.position.toLowerCase(),
            itemPadding = legend.itemPadding,
            legnedHeightIncr, column, legendWidth = 0, legendHeight = 0;
        if (ej.util.isNullOrUndefined(legend.columnCount) && legend.rowCount) {
            legnedHeightIncr = legend.rowCount;
            column = Math.ceil(itemCount / legnedHeightIncr);
            legendWidth = legendItemWidth * column;
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        else if (ej.util.isNullOrUndefined(legend.rowCount) && legend.columnCount) {
            legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
            legVal = legend.columnCount;
            legendWidth = legendItemWidth * legVal;
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        else if ((legend.rowCount) && (legend.columnCount)) {
            if (legend.columnCount < legend.rowCount) {
                legnedHeightIncr = legend.rowCount;
                column = Math.ceil(itemCount / legnedHeightIncr);
                legendWidth = legendItemWidth * column;
                legendHeight = legendItemHeight * legnedHeightIncr;
            }
			else if(legend.columnCount > legend.rowCount){
                  if (position === 'top' || position === 'bottom' || position === 'custom') {
                    legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                    legVal = legend.columnCount;
                    legendWidth = legendItemWidth * legVal;
                    legendHeight = legendItemHeight * legnedHeightIncr;
                } else {
                    legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                    column = Math.ceil(itemCount / legnedHeightIncr);
                    legendWidth = legendItemWidth * column;
                    legendHeight = legendItemHeight * legnedHeightIncr;
                }
			 }
            else {
                if (position === 'top' || position === 'bottom' || position === 'custom') {
                    legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                    legVal = Math.ceil(itemCount / legend.rowCount); 
                    legendWidth = legendItemWidth * legend.columnCount;
                    legendHeight = legendItemHeight *legVal;
                } else {
                    legnedHeightIncr = legend.rowCount;
                    column = Math.ceil(itemCount / legnedHeightIncr);
                    legendWidth = legendItemWidth * column;
                    legendHeight = legendItemHeight * legnedHeightIncr;
                }
                
            }
			
        }
        legendHeight += this.model.elementSpacing;
        return { LegendWidth: legendWidth, LegendHeight: legendHeight };
    },
	
	_isEjScroller: function () {
        var chart = this,
           chartModel = chart.model,
            legendContainer = $(chart.legendContainer),
            legend = chartModel.legend;
        if (ej.util.isNullOrUndefined($('#' + legendContainer[0].id).ejScroller)) 
            legend._ejScroller = false;
        else
            legend._ejScroller = legend.enableScrollbar;
    },

    //legend text wrapping
	_rowsCalculation: function (data, textmaxwidth, textOverflow)
    {
        var chart = this,
            chartModel = chart.model,
            legend = chartModel.legend,
            word, textCollection = [], currentWidth,nextWidth,
            text = data.legendItem ? data.legendItem.Text.toString() : data.text,
            legendTextCollection = text.split(' '),
            textMaxWidth=textmaxwidth,
            font = data.legendItem ? data.legendItem.LegendStyle.Font : data.font,
            legendTextLength = legendTextCollection.length,
            wordMax = 0;
      
        for (var i = 0; i < legendTextLength; i++) {
            word = legendTextCollection[i];
            currentWidth = ej.EjSvgRender.utils._measureText(word, null, font).width;
            if (currentWidth <= textMaxWidth) {
                while (i < legendTextLength) {
                    currentWidth = ej.EjSvgRender.utils._measureText(word, null, font).width;
                    nextWidth = (legendTextCollection[i + 1]) ? ej.EjSvgRender.utils._measureText(legendTextCollection[i + 1], null, font).width : 0;
                    if ((currentWidth + nextWidth) <= textMaxWidth && nextWidth>0) {
                        word = word.concat(' ' + legendTextCollection[i + 1]);
                        i++;
                    }
                    else {
                        wordMax = Math.max(wordMax, currentWidth);
                        break;
                    }
                }
                textCollection.push(word);
            }
            else{
                if (textOverflow == "wrapandtrim") {
                    word = ej.EjSvgRender.utils._trimText(word, textMaxWidth, font);
                    textCollection.push(word);
                    this.model._legendMaxWidth = textMaxWidth;
                    wordMax = Math.max(wordMax, textMaxWidth);
                    if (!data.legendItem) this.model.trimTooltip = true;
                }
                else {
                    textCollection.push(word);
                    if (data.legendItem)
                        this.model._legendMaxWidth = Math.max(this.model._legendMaxWidth, currentWidth);
                    else wordMax = Math.max(wordMax, currentWidth);
                }                
            }
        }        
        this.model._legendMaxHeight = Math.max(this.model._legendMaxHeight, textCollection.length);
        return { textCollection: textCollection, wordMax: wordMax };
    },

    _triggerLegendEvent:function(name,color,index)
    {
        var chart = this,
            chartModel = chart.model,
            legend = chartModel.legend,
			areaType = chartModel.AreaType,
            itemStyle = legend.itemStyle,
            border = itemStyle.border,
            elementSpacing = chartModel.elementSpacing,
            textMaxWidth = legend.textWidth,
            svgRender = ej.EjSvgRender,
            chartSvgRender = chart.svgRenderer,
            seriesLength = chartModel._visibleSeries.length,
            svgObject = chart.svgObject,
            elementSpacing = chartModel.elementSpacing,
            textOverflow = legend.textOverflow.toLowerCase(),
            legendItem, legendFont,textWidth,
            style= {
                BorderColor:border.color,
                BorderWidth: border.width,
                Opacity: legend.opacity,
                Color: color,
                Font: legend.font
            };
            
        commonEventArgs = $.extend({}, svgRender.commonChartEventArgs);
        commonEventArgs.data = {
            svgRenderer: chartSvgRender,
            svgObject: svgObject,
            symbolShape: legend.shape,
            legendItem: {
                Text: name,
                Shape: legend.shape,
                LegendStyle: style,
                SeriesIndex: (areaType == "none" && seriesLength == 1) ? 0 : index,
            },            
            style: {
                ShapeSize: itemStyle,
                ElementSpace: elementSpacing,
                Style: style,
                ID: chart.svgObject.id + '_LegendItemShape' + index,
                SeriesIndex: (areaType == "none" && seriesLength == 1) ? 0 : index,
                context: true
            },
            gLegendItemEle: chart.gLegendItemEle
       };
           
       chart._trigger("legendItemRendering", commonEventArgs);
       legendItem = commonEventArgs.data.legendItem.Text;
       legendFont = commonEventArgs.data.legendItem.LegendStyle.Font;
       textWidth = ej.EjSvgRender.utils._measureText(legendItem, null, legendFont).width;
       if (textOverflow == "trim") {
           if (textWidth > textMaxWidth) {
               legendItem = ej.EjSvgRender.utils._trimText(legendItem, textMaxWidth, legendFont);
               this.model._legendMaxWidth = textMaxWidth;
           }
       }
       else if (textOverflow == "wrap" || textOverflow == "wrapandtrim") {
           legendItem = this._rowsCalculation(commonEventArgs.data, textMaxWidth, this.model.legend.textOverflow.toLowerCase()).textCollection;
       }
       return { commonEventArgs: commonEventArgs, legendItem: legendItem };  
    },

    _calculateLegendBounds: function () {
        // intialize
        this.model.legendCollection = [];
        this.model.legendTextRegion = [];
        this.model.legendRegion = [];
        this.model.svgHeight = $(this.svgObject).height();
        this.model.svgWidth = $(this.svgObject).width();
        this.model._legendMaxWidth = 0;
        this.model._legendMaxHeight = 0;

        var chart = this,
            chartModel = chart.model,
            math = Math,
            max = math.max,
            min = math.min,
            abs = math.abs,
            legend = chartModel.legend,
            padding = 10,
            legendSizeHeight = legend.size.height,
            legendSizeWidth = legend.size.width,
            itemPadding = legend.itemPadding > 0 ? legend.itemPadding : 0,
            position = legend.position.toLowerCase(),
            width = 0, height = 0,
            svgHeight = chartModel.svgHeight,
            svgWidth = chartModel.svgWidth,
            legendItemWidth = 0,
            legendItemHeight = 0,
            legendWidth = 0,
            legendHeight = 0,
            legnedHeightIncr = 1,
            currentLegend,
            index,
            bounds,
            legendviewerHeight = 0, legendviewerWidth = 0,
            legendHeightTemp, legendWidthTemp,
            tempSeries = {},
            legendSeries = [], trendLineLength, trendLines,
            trendLine,
			trendlineType,
            areaType = chartModel.AreaType,
            visibleSeries = chartModel._visibleSeries,
            rowCount = legend.rowCount, point,
            columnCount = legend.columnCount,
            elementSpacing = chartModel.elementSpacing,
            visiblePoints, series,
            length, legendColor, legendName,legendStyleColor,
            titleSize = ej.EjSvgRender.utils._measureText(legend.title.text, null, legend.title.font),
            legendsize, colorGradName,
            vScrollSize = 0, hScrollSize = 0,
            borderSize = chartModel.border.width,
            legendBorder = legend.border.width,type,
            legendInfo, legendStyle, legendFont, shapeWidth,
            textOverflow = legend.textOverflow.toLowerCase(),
            svgObjectHeight = svgHeight - ((elementSpacing * 4) + (borderSize * 2) + (legend.border.width * 2)),
            svgObjectWidth = svgWidth - ((elementSpacing * 4) + (borderSize * 2) + (legendBorder * 2));

            if (visibleSeries && legend.visible && visibleSeries.length != 0) {  
            type = visibleSeries[0].type.toLowerCase();			
            if (areaType == "none" && visibleSeries.length == 1 || type == 'pieofpie' || type == 'pyramid' || type == 'funnel') {
                  
                series = visibleSeries[0];
                visiblePoints = ej.EjSeriesRender.prototype._calculateVisiblePoints(series).legendPoints;
                length = visiblePoints.length;
                for (var j = 0; j < length; j++) {
                    legendColor = legend.fill ? (!(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop) : chartModel.pointColors[visiblePoints[j].actualIndex];
                    point = visiblePoints[j];
                    legendName = !ej.util.isNullOrUndefined(point.x) ? point.x : 'series' + j;
					visibleOnLegend = !(point.visibleOnLegend) ? 'visible' : point.visibleOnLegend;
                    if (!point.isEmpty && visibleOnLegend.toLowerCase() == 'visible' && series.visibleOnLegend.toLowerCase() == "visible") {
                        legendInfo=this._triggerLegendEvent(legendName, legendColor, j);
                        legendStyle = legendInfo.commonEventArgs.data.legendItem.LegendStyle;
                        tempSeries = {
                            Text: legendInfo.legendItem,
                            displayText: legendInfo.commonEventArgs.data.legendItem.Text,
                            Font: legendStyle.Font,
                            SeriesIndex: 0,
                            PointIndex: j,
                            ActualIndex:point.actualIndex,
                            fill: legendStyle.Color,
                            visibility: point.visible ? 'visible' : 'hidden',
                            Shape: legendInfo.commonEventArgs.data.legendItem.Shape,
                            LegendStyle: legendStyle,
                            CommonEventArgs: { cancel: legendInfo.commonEventArgs.cancel, data: legendInfo.commonEventArgs.data }
                        };
                        legendSeries.push(tempSeries);
                    }
                }
            }
            else {
                length = visibleSeries.length;
                for (var j = 0; j < length; j++) {
                    legendColor = legend.fill ? (!(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop) : ej.util.isNullOrUndefined(chartModel.seriesColors[j]) ? chartModel.pointColors[j] : chartModel.seriesColors[j];
                    series = visibleSeries[j];
                    legendName = series.name ? series.name : 'series' + j;
					if (series.visibleOnLegend.toLowerCase() == "visible") { 
                    legendInfo=this._triggerLegendEvent(legendName, legendColor, j);
                    legendStyle = legendInfo.commonEventArgs.data.legendItem.LegendStyle;
                    tempSeries = {
                        Text: legendInfo.legendItem,
                        displayText: legendInfo.commonEventArgs.data.legendItem.Text,
                        Font: legendStyle.Font,
                        SeriesIndex: j,
                        fill: legendStyle.Color,
                        visibility: series.visibility,
                        Shape: legendInfo.commonEventArgs.data.legendItem.Shape,
                        LegendStyle: legendStyle,
                        CommonEventArgs: { cancel: legendInfo.commonEventArgs.cancel, data: legendInfo.commonEventArgs.data }
                    };
                    legendSeries.push(tempSeries);
					}
                    trendLines = series.trendlines;
                    trendLineLength = trendLines.length;
                    for (var i = 0; i < trendLineLength; i++) {
                        trendLine = trendLines[i];
                        legendColor = legend.fill ? (!(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop) : trendLine.fill;
                        legendName = trendLine.name ? trendLine.name : 'series' + j;                                        
                        if (trendLine.visibility != '' && trendLine.visibleOnLegend.toLowerCase() == "visible") {
                            legendInfo = this._triggerLegendEvent(legendName, legendColor, j);
                            legendStyle = legendInfo.commonEventArgs.data.legendItem.LegendStyle;
                            tempSeries = {
                                Text: legendInfo.legendItem,
                                displayText: legendInfo.commonEventArgs.data.legendItem.Text,
                                Font: legendStyle.Font,
                                SeriesIndex: j,
                                trendLineIndex: i,
                                fill: legendStyle.Color,
                                isTrendLine: true,
                                visibility: series.visibility.toLowerCase() == "visible" ? trendLine.visibility : "hidden",
                                Shape: legendInfo.commonEventArgs.data.legendItem.Shape,
                                LegendStyle: legendStyle,
                                CommonEventArgs: { cancel: legendInfo.commonEventArgs.cancel, data: legendInfo.commonEventArgs.data }
                            };
                            legendSeries.push(tempSeries);
                        }
                    }
                }
            }
            
            legendSeriesLength = legendSeries.length;
            for (var j = 0; j < legendSeriesLength; j++) {
                currentLegend = legendSeries[j];
                shapeWidth = currentLegend.CommonEventArgs.data.style.ShapeSize.width;
                legendsize = chart._getLegendSize(currentLegend);
                legendItemWidth = max(this.model._legendMaxWidth > 0 ? (this.model._legendMaxWidth + itemPadding + shapeWidth) : legendsize.Width, legendItemWidth);
                legendItemHeight = max((textOverflow == "wrap" || textOverflow == "wrapandtrim") ? legendsize.Height * this.model._legendMaxHeight : legendsize.Height, legendItemHeight);
            }
            legendHeight = legendItemHeight + elementSpacing * 2;
            legendWidth = legendItemWidth;
           
            if (columnCount || rowCount) {
                legendBounds = chart._legendItemBounds(legendSeries.length, legendItemWidth + (itemPadding), legendItemHeight + (itemPadding));
                legendWidth = legendBounds.LegendWidth;
                legendHeight = legendBounds.LegendHeight;
                if (position === 'top' || position === 'bottom' || position === 'custom')
                    legendHeight = legendHeight - itemPadding + elementSpacing;
                else
                    legendWidth = legendWidth - itemPadding;
            }
            for (var k = 0; k < legendSeriesLength; k++) {
                currentLegend = legendSeries[k];
                index = currentLegend.SeriesIndex;
                series = chartModel.series[index];
                legendsize = chart._getLegendSize(currentLegend);
                if (textOverflow == "wrap"||textOverflow=="wrapandtrim") {
                    legendsize.Width = legendItemWidth;
                    legendsize.Height = legendItemHeight;
                }
                if (!(rowCount) && !(columnCount)) {
                    
                    if ((position == 'top' || position == 'bottom' || position == 'custom')) {
                        width += legendsize.Width + itemPadding;
                        if (width > svgObjectWidth && k !=0) {
                            width -= legendsize.Width + itemPadding;
                            legendWidth = max(legendWidth, width);
                            width = legendsize.Width + itemPadding;
                            legnedHeightIncr++;
                            legendHeight += legendItemHeight + itemPadding;
                        }
                        else
                            legendWidth = max(legendWidth, width);

                        height = max(height, legendItemHeight);
                    }
                    else {
                        height += legendsize.Height + itemPadding;
                        if (height > svgObjectHeight) {
                            height -= legendsize.Height + itemPadding;
                            legendHeight = max(legendHeight, height);
                            height = legendsize.Height + itemPadding;
                            legendWidth += legendItemWidth + itemPadding;
                        }
                        else
                            legendHeight = max(legendHeight, height);

                        width = max(width, legendItemWidth);
                    }
                }
                if (currentLegend.visibility.toLowerCase() == "visible")
                    colorGradName = chart.svgRenderer.createGradientElement('legend' + k, legendSeries[k].fill, 0, 0, 0, svgObjectHeight, chart.gLegendEle);
                else
                    colorGradName = 'gray';
                currentLegend.CommonEventArgs.data.legendItem.LegendStyle.Color = colorGradName;
                bounds = (rowCount || columnCount) ? { Width: legendItemWidth, Height: legendItemHeight } : legendsize;
				 bounds._Width = legendsize.Width;

                if (currentLegend.isTrendLine) {
                    currentLegend.TrendLineIndex = currentLegend.trendLineIndex;
					trendlineType = series.trendlines[currentLegend.trendLineIndex].type.toLowerCase();
                    currentLegend.CommonEventArgs.data.legendItem.drawType = currentLegend.drawType= trendlineType == 'linear' || trendlineType == "movingaverage" ? 'line' : 'spline';
                }
                legendSeries[k].Bounds = bounds;
                chartModel.legendCollection.push(currentLegend);
            }

            //LegendBounds calculation Perform here..
            if (position === 'top' || position === 'bottom' || position === 'custom') {
                legendWidth = titleSize.width > legendWidth - itemPadding ? (titleSize.width + padding * 2 + itemPadding) : legendWidth + padding * 2;
                width += padding ;
                height += padding * 2;
                chartModel.LegendBounds = { Width: max(legendWidth, width) - itemPadding, Height: max(legendHeight, height), Rows: legnedHeightIncr };
            }
            else {
                legendWidth = titleSize.width > legendWidth ? (titleSize.width + padding * 2) : legendWidth + padding * 2;
                width += padding;
                height += padding;
                chartModel.LegendBounds = { Width: max(legendWidth, width), Height: max(legendHeight, height) + padding - itemPadding, Columns: legnedHeightIncr };
            }
            //Calculating legend viewer bounds or calculate user specified legend bounds
            chartModel.LegendViewerBounds = { Width: "0", Height: "0" };

            if (legendSizeHeight == "" || legendSizeHeight == null) {
                if (position == 'left' || position == 'right' || position == 'custom')
                    legendHeightTemp = abs(svgHeight);
                else
                    legendHeightTemp = abs((svgHeight / 100) * parseInt('20%'));
                chartModel.LegendViewerBounds.Height = min(legendHeightTemp, chartModel.LegendBounds.Height);
            }
            else{
				if (legendSizeHeight.indexOf("%") != -1)
                    chartModel.LegendViewerBounds.Height = abs((svgHeight / 100) * parseInt(legendSizeHeight));
                else
                    chartModel.LegendViewerBounds.Height = parseInt(legendSizeHeight);
			}                

            if (legendSizeWidth == "" || legendSizeWidth == null) {
                if (position == 'top' || position == 'bottom' || position == 'custom')
                    legendWidthTemp = abs(svgWidth);
                else
                    legendWidthTemp = abs((svgWidth / 100) * parseInt('20%'));
                chartModel.LegendViewerBounds.Width = min(legendWidthTemp, chartModel.LegendBounds.Width);
            }
            else{
				 if (legendSizeWidth.indexOf("%") != -1)
                    chartModel.LegendViewerBounds.Width = abs((svgWidth / 100) * parseInt(legendSizeWidth));
                else
                    chartModel.LegendViewerBounds.Width = parseInt(legendSizeWidth);
			}                
        }
        else {
            chartModel.LegendBounds = { Width: 0, Height: 0 };
            chartModel.LegendViewerBounds = { Width: 0, Height: 0 };
        }
        legendContainer = $(chart.legendContainer);
        legendSvgContainer = $(chart.legendSvgContainer);
        if (legend.enableScrollbar) {
            chartModel.LegendActualBounds = chartModel.LegendViewerBounds;
            legendContainer.removeAttr("style");
            legendContainer.css({ "visibility":'hidden',"width": chartModel.LegendViewerBounds.Width, "height": chartModel.LegendViewerBounds.Height });
            legendSvgContainer.css({ "height": chartModel.LegendBounds.Height, "width": chartModel.LegendBounds.Width });
			
			if (legend._ejScroller)
                $('#' + legendContainer[0].id).ejScroller({width: chartModel.LegendViewerBounds.Width, height: chartModel.LegendViewerBounds.Height })
            else
                legendContainer.css({ "overflow": "scroll" });
			
            if (chartModel.LegendBounds.Width > chartModel.LegendViewerBounds.Width && chartModel.LegendBounds.Height > chartModel.LegendViewerBounds.Height) {
                if (legend._ejScroller) {
                    hScrollSize = $('.e-hscrollbar').height() || 0;
                    vScrollSize = $('.e-vscrollbar').width() || 0;
                } else {
                    legendContainer.css({ "overflow": "scroll" });
                    hScrollSize = legendContainer[0].offsetHeight - (legendBorder * 2) - legendContainer[0].clientHeight;
                    vScrollSize = legendContainer[0].offsetWidth - (legendBorder * 2) - legendContainer[0].clientWidth;
                }
            } else if (chartModel.LegendBounds.Width > chartModel.LegendViewerBounds.Width) {
				if (legend._ejScroller) {
                    hScrollSize = $('.e-hscrollbar').height() || 0;
                    vScrollSize = 0;
                }
                else {
                    legendContainer[0].style.overflowX = "scroll";
                    legendContainer[0].style.overflowY = "hidden";
                    hScrollSize = legendContainer[0].offsetHeight - (legendBorder * 2) - legendContainer[0].clientHeight;
                    vScrollSize = 0;
                }
            } else if (chartModel.LegendBounds.Height > chartModel.LegendViewerBounds.Height) {
				if (legend._ejScroller) {
                    hScrollSize = 0;
                    vScrollSize = $('.e-vscrollbar').width();
                }
                else {
                    legendContainer[0].style.overflowY = "scroll";
                    legendContainer[0].style.overflowX = "hidden";
                    hScrollSize = 0;
                    vScrollSize = legendContainer[0].offsetWidth - (legendBorder * 2) - legendContainer[0].clientWidth;
                } 
            }
            
        }
        else {
            chartModel.LegendActualBounds = chartModel.LegendBounds;
            if (legendSizeWidth != "" && legendSizeWidth != null)
                chartModel.LegendActualBounds.Width = parseInt(legendSizeWidth);
            if (legendSizeHeight != "" && legendSizeHeight != null)
                chartModel.LegendActualBounds.Height = parseInt(legendSizeHeight);
        }
        chartModel.LegendActualBounds.Height += Math.abs(hScrollSize);
        chartModel.LegendActualBounds.Width += Math.abs(vScrollSize);
        chartModel.LegendActualBounds.hScrollSize = hScrollSize;
        chartModel.LegendActualBounds.vScrollSize = vScrollSize;
    },

    GetPointXYOrgin: function (x, y, orginX, orginY) {

        var xvalue = ((x - orginX) / (this.model.axes.PrimaryXaxis.visibleRange.max - orginX)) * (this.model.m_AreaBounds.Width);
        var yvalue = ((y - orginY) / (this.model.axes.PrimaryYaxis.visibleRange.max - orginY)) * (this.model.m_AreaBounds.Height - this.GetPointXY(this.model.axes.PrimaryXaxis.visibleRange.min, Math.max(this.model.axes.PrimaryYaxis.visibleRange.min, 0)).Y);

        return { X: xvalue, Y: yvalue };
    },

    _wrap: function (axis, gap, labelText, textOverflow, font, textSize) {
        var text = labelText, textWidth = textSize.width, textCollection = [], unTrimmedText = [], textLength = text.length, 
		line = 0, i = 0, labelCollection = labelText.split(' '), labelCollectionLength = labelCollection.length;
        if (textOverflow == 'wrap') {
            if (textWidth > gap) {
                for (var w = 1; w <= text.length; w++) {
                    labelText = text.substring(0, w);
                    textWidth = ej.EjSvgRender.utils._measureText(labelText, null, font).width;
                    if (textWidth > gap) {
                        line = line + 1; // To find the no of rows splitted
                        labelText = text.substring(0, w - 1);
                        textCollection[i] = labelText;
                        text = text.slice(w - 1, textLength);
                        currentTextCollextion = text.split(' ');
                        text = labelCollection.indexOf(currentTextCollextion[0]) > -1 ? text : '-' + text;
                        var newTextWidth = ej.EjSvgRender.utils._measureText(labelText, null, font).width;
                        i++;
                        w = 0;
                    }
                }
            }
            textCollection[i] = labelText;
        }
        else if (textOverflow == "wrapandtrim") {
            var max = 0, word, currentWidth, nextWidth;
            for (var i = 0; i < labelCollectionLength; i++) {
                word = labelCollection[i];
                currentWidth = ej.EjSvgRender.utils._measureText(word, null, font).width;
                if (currentWidth < gap && textWidth > gap) {
                    while (i < labelCollectionLength) {
                        currentWidth = ej.EjSvgRender.utils._measureText(word, null, font).width;
                        nextWidth = (labelCollection[i + 1]) ? ej.EjSvgRender.utils._measureText(labelCollection[i + 1], null, font).width : 0;
                        if ((currentWidth + nextWidth) <= gap && nextWidth > 0) {
                            word = word.concat(' ' + labelCollection[i + 1]);
                            i++;
                        }
                        else {
                            // find the maximum width of the lines
                            max = Math.max(max, currentWidth);
                            break;
                        }
                    }
                    textCollection.push(word);
                }
                else {
                    if (textWidth > gap) {
                        unTrimmedText.push(word);
                        word = ej.EjSvgRender.utils._trimText(word, gap, font);
                        newTextWidth = ej.EjSvgRender.utils._measureText(word, null, font).width;
                        max = Math.max(max, newTextWidth)
                        textCollection.push(word);
                    } else {
                        textCollection.push(labelText);
                        break;
                    }
                }
            }
        }
        return {text: textCollection, unTrimmedText : unTrimmedText};
    },

    // to get the size of multi level labels
    getMultiLevelLabelSize: function (axis) {
		var value = 0;
        axis._multiLevelLabelHeight = 0;
       
            // declaration
            var grpLabelArr = [] ,multiLevelLabelHeight = [], prevHeight = [], grpLabelLength = axis.multiLevelLabels.length,
            range = axis.visibleRange, grpLabel, level, i, startX, endX, labelSize, gap, height, padding = 10, textCollection,
            orientation = axis.orientation.toLowerCase(), braceFlag = [];
			if (orientation == "vertical")
				axisValue = axis.length;
			else 
				axisValue = axis.width ? axis.width : (this.model.primaryYAxis.AxisMaxWidth ? axis.length - this.model.primaryYAxis.AxisMaxWidth: axis.length);
            for (i = 0; i < grpLabelLength; i++) {
                grpLabel = axis.multiLevelLabels[i] = grpLabelArr[i] = $.extend(true, {}, this.model.multiLevelLabelsDefault, axis.multiLevelLabels[i]);
				borderStyle = grpLabel.border.type.toLowerCase();
                level = grpLabel.level;
                if (grpLabel.visible && grpLabel.text != "" && !ej.util.isNullOrUndefined(grpLabel.start) && !ej.util.isNullOrUndefined(grpLabel.end)) {
                    startX = Math.ceil(ej.EjSvgRender.utils._getPointXY(grpLabel.start, range, axis.isInversed) * (axisValue));
                    endX = Math.ceil(ej.EjSvgRender.utils._getPointXY(grpLabel.end, range, axis.isInversed) * (axisValue));
                    labelSize = ej.EjSvgRender.utils._measureText(grpLabel.text, null, grpLabel.font);
                    textOverflow = grpLabel.textOverflow.toLowerCase();
                    gap = ej.util.isNullOrUndefined(grpLabel.maximumTextWidth) ? endX - startX : grpLabel.maximumTextWidth;
                    height = orientation == "vertical" ? (grpLabel.maximumTextWidth ? grpLabel.maximumTextWidth : labelSize.width): labelSize.height;
                    height += 2 * grpLabel.border.width;
                    if (labelSize.width > gap - padding && (textOverflow == "wrap" || textOverflow == "wrapandtrim")) {   // for wrap
                        textCollection = this._wrap(axis, gap - padding, grpLabel.text, textOverflow, grpLabel.font, labelSize);
                        textCollection = textCollection["text"];
                        height = orientation == "vertical" ? gap-padding : (height * (textCollection.length));
                    }
                    if (ej.util.isNullOrUndefined(multiLevelLabelHeight[level]))
                        multiLevelLabelHeight[level] = height;
                    else
                        multiLevelLabelHeight[level] = multiLevelLabelHeight[level] < height ? height : multiLevelLabelHeight[level];
                }
            }
            // to sort based on level
                grpLabelArr.sort(function (a, b) {
                    return parseFloat(a.level) - parseFloat(b.level);
                });
                axis.multiLevelLabels.sort(function (a, b) {
                    return parseFloat(a.level) - parseFloat(b.level);
                });
            // to modify the level
            for (var j = 0; j < grpLabelArr.length; j++){
                if (j == 0) {
                    axis.multiLevelLabels[j]._level = grpLabelArr[j]._level = 0;
                    greaterIndex = 0;
                }
                else if (grpLabelArr[j].level > greaterIndex + 1) {
                    axis.multiLevelLabels[j]._level = grpLabelArr[j]._level = greaterIndex + 1;
                    greaterIndex = grpLabelArr[j]._level;
                } 
                else {
                    axis.multiLevelLabels[j]._level = grpLabelArr[j]._level = grpLabelArr[j].level;
                    greaterIndex = grpLabelArr[j].level > greaterIndex ? grpLabelArr[j].level : greaterIndex;
                }
				if (borderStyle == "curlybrace")
					braceFlag[axis.multiLevelLabels[j]._level] =  true;
            }
            // to find the sum
            for (var i = 0; i < multiLevelLabelHeight.length ; i++) {
                prevHeight[i] = value;
                if (!ej.util.isNullOrUndefined(multiLevelLabelHeight[i])){
                    value = borderStyle == "brace" ? value + multiLevelLabelHeight[i] : value + multiLevelLabelHeight[i] + padding/2;
					if (braceFlag[i]) {
						value += padding;
						multiLevelLabelHeight[i]+= padding;
					}
				}
                else {
                    multiLevelLabelHeight.splice(i, 1);
                    i--;
                }
            }
            axis._multiLevelLabelHeight = value;
            axis.multiLevelLabelHeight = multiLevelLabelHeight;
            axis.prevHeight = prevHeight;
        
        return value;
    },
	
	_saturationColor: function (color, lum) {

	// validate hex string
	color = this.colorNameToHex(color);
	color = String(color).replace(/[^0-9a-f]/gi, '');
	if (color.length < 6) {
		color = color[0]+color[0]+color[1]+color[1]+color[2]+color[2];
	}
	lum = lum || 0;

	// convert to decimal and change luminosity
	var rgb = "#", c, i;
	for (i = 0; i < 3; i++) {
		c = parseInt(color.substr(i*2,2), 16);
		c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
		rgb += ("00"+c).substr(c.length);
	}

	return rgb;
},
	
    _drawChartAreaRect: function () {

        var fillColor = !(this.model.background && this.model.background._gradientStop) ? this.model.background : 'url(#' + this.svgObject.id + '_backGradient)';

        if (this.model.backGroundImageUrl)
            fillColor = 'transparent';
        
        var chartBorder = this.model.border;
        var options = {
            'id': this.svgObject.id + '_SvgRect',
            'x': chartBorder.width / 2,
            'y': chartBorder.width / 2,
            'width': $(this.svgObject).width() - (2 * chartBorder.width),
            'height': $(this.svgObject).height() - (2 * chartBorder.width),
            'fill': fillColor,
            'opacity': chartBorder.opacity,
            'stroke-width': chartBorder.width,
            'stroke': chartBorder.color,
			'class': "e-chartborder"
        };
        this.svgRenderer.drawRect(options, this.svgObject);
        


        if (this.model.backGroundImageUrl)
            this._drawBackImage();

        if (this.model.AreaType == 'cartesianaxes' && !this.model.enable3D) {
            var borderOptions = {
                'id': this.svgObject.id + '_ChartArea',
                'x': this.model.m_AreaBounds.X,
                'y': this.model.m_AreaBounds.Y,
                'width': this.model.m_AreaBounds.Width,
                'height': this.model.m_AreaBounds.Height,
                'fill': this.model.chartArea.background,
                'stroke-width': this.model.chartArea.border.width,
                'opacity': this.model.chartArea.border.opacity,
                'stroke': this.model.chartArea.border.color,
				'class': "e-chartareaborder"
            };
            this.svgRenderer.drawRect(borderOptions, this.svgObject);


        }

    },
    axesIndexCount: function (axis,index) {
        var vRowcount = [];
        for (var k = 0; k < axis.length; k++) {
            var currentAxis = axis[k];
            if (currentAxis.orientation.toLowerCase() == "vertical") {
                if (currentAxis.rowIndex == index)
                    vRowcount.push(currentAxis);
            }
            else {
                if (currentAxis.columnIndex == index)
                    vRowcount.push(currentAxis);
            }
        }
        return vRowcount ;
    },
    axesCount: function (axis) {
        var vRowcount = [], indexValue=[], definitionsLength=0;
        var chartModel = this.model;
        $.each(axis, function (index, currentAxis) {
            if (currentAxis.orientation.toLowerCase() == "vertical") {
                definitionsLength =(chartModel.rowDefinitions)?chartModel.rowDefinitions.length:0;
                if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex });
                else if (currentAxis.rowIndex != vRowcount[vRowcount.length - 1].axis.rowIndex)
                    vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex});
            }
            else {
                definitionsLength =(chartModel.columnDefinitions)? chartModel.columnDefinitions.length:0;
                if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
                else if (currentAxis.columnIndex != vRowcount[vRowcount.length - 1].axis.columnIndex)
                    vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
            }
        });
        var length = vRowcount.length;
        if (definitionsLength > length) {
            length = definitionsLength;
            for (var l = 0; l < definitionsLength; l++)
                indexValue.push(l);
        } else {
            for (var i = 0; i < length; i++)
                indexValue.push(vRowcount[i].index);
        }
        
        return { length: length, indexValue: indexValue };

    },
   
     //Calculate position for each vertical axis
    _calRowSize: function () {
        var start = 0;
        var yaxisOrign = [];
        var totalRealLength = 0;
        var orginY = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
        //Caluculate size for rowdefinitions values
        if (this.model._rowDefinitions[0].rowDefinitions != null) {
            for (var i = 0, rowMax = this.model._rowDefinitions.length; i < rowMax; i++) {
                var rowDef = this.model._rowDefinitions[i].rowDefinitions;
                rowDef.rowHeight = Math.abs(rowDef.rowHeight);
                var realLength = 0;
                if (rowDef.unit.toLowerCase() == "percentage"){
                    var length = this._axisRowDefinitionSize(this.model.rowDefinitions);
                    realLength = Math.floor((this.model.m_AreaBounds.Height) * (rowDef.rowHeight / length));
                }  
                else
                    realLength = rowDef.rowHeight;
                if (i == 0) yaxisOrign.push(orginY);
                   
                      yaxisOrign.push( (orginY - realLength - totalRealLength ));
                    totalRealLength += realLength;
                

            }
            
        }
        //Calculate size based on chart area size
        else {
            var height = Math.floor((this.model.m_AreaBounds.Height) / this.model._rowDefinitions.length);

            for (start = (orginY) ; start >= this.model.m_AreaBounds.Y; start = start - height) {
                yaxisOrign.push(start);
            }
        }
        return yaxisOrign;
    },
    
     //Calculate position for each horizontal axis
    _calColumnSize: function () {
        var xaxisOrign = [];
        var orginX = this.model.m_AreaBounds.X;
        var areaWidth = this.model.m_AreaBounds.Width;
        var totalRealLength = 0;
        var columnDef = this.model._columnDefinitions;
        //Caluculate size for columndefinitions values
        if (columnDef[0].columnDefinitions != null) {
            for (var i = 0, max = columnDef.length; i < max; i++) {
                var rowDef = columnDef[i].columnDefinitions;
				rowDef.columnWidth = Math.abs(rowDef.columnWidth);
                var realLength;
                if (rowDef.unit.toLowerCase() == "percentage"){
				    var length = this._axisColumnDefinitionSize(this.model.columnDefinitions);
				    realLength = Math.floor((areaWidth) * (rowDef.columnWidth / length));
                } 
                else
                    realLength = rowDef.columnWidth;
                if (i == 0) xaxisOrign.push(orginX);

                xaxisOrign.push((orginX + realLength + totalRealLength));
                totalRealLength += realLength;
            }
        }
        //Caluculate size based on chart area size
        else {
            var width = Math.floor((areaWidth) / columnDef.length);
            for (var start = orginX, maxStart = orginX + areaWidth; start <= maxStart; start = start + width) {
                xaxisOrign.push(start);

            }
        }
        return xaxisOrign;
    },
    
     //Calculate position and size for each axis
    _calculateAxisSize: function (params) {

        //For Horizontal
        var isInitAxes = true, isOppInitAxes = true;
        var xaxisOrign = this._calColumnSize();
        var columnDefinition = this.model._columnDefinitions;		
        for (var i = 0, maxLength = columnDefinition.length; i < maxLength; i++) {
            var x1 = xaxisOrign[i];
            var x2 = xaxisOrign[i + 1];
            var nearIndexVal = 0;
            var farIndexVal = 0;;
            var isColFirstAxes = true, isColOppFirstAxes = true;
            var oppY = this.model.m_AreaBounds.Y;
            var norY = oppY + this.model.m_AreaBounds.Height;
            for (var j = 0, maxCol = columnDefinition[i].axis.length; j < maxCol; j++) {
                var currentColAxis = columnDefinition[i].axis[j];
                var realY = 0, axisColumnSpace = 0;
                var opposedPosition = currentColAxis._opposed;
                if (!params.axes[currentColAxis.name]._validCross) {
                    if (!opposedPosition) {
                        if (!(isColFirstAxes && (isInitAxes || nearIndexVal == 0))) {
                            axisColumnSpace = columnDefinition.nearSizes[nearIndexVal];
                            realY = norY = (norY + axisColumnSpace);
                            nearIndexVal++;
                        }
                        else {
                            realY = norY;
                            isColFirstAxes = false;
                        }

                    } else {
                        if (!(isColOppFirstAxes && (isOppInitAxes || farIndexVal == 0))) {
                            axisColumnSpace = columnDefinition.farSizes[farIndexVal];
                            realY = oppY = (oppY - axisColumnSpace);
                            farIndexVal++;
                        }
                        else {
                            realY = oppY;
                            isColOppFirstAxes = false;
                        }
                    }
                }
                if (currentColAxis.columnSpan) {
                    //Skip the axis size calculation for duplicate span axis
                    if (!(columnDefinition[i].index == currentColAxis.columnIndex))
                        continue;
                    var axisSize = (maxLength >= (i + currentColAxis.columnSpan)) ? i + currentColAxis.columnSpan : i+1;
                    x2 = xaxisOrign[axisSize];
                    if (!opposedPosition && nearIndexVal==0)isInitAxes = isColFirstAxes;
                    if (opposedPosition && farIndexVal == 0) isOppInitAxes = isColOppFirstAxes;
                } else {
                    x2 = xaxisOrign[i + 1];
                    isInitAxes = true;
                    isOppInitAxes = true;
                }
                
               currentColAxis.Location = { };
                currentColAxis.Location.X1 = x1 + currentColAxis.plotOffset;
                currentColAxis.Location.Y1 = realY;
                currentColAxis.Location.X2 = x2 - currentColAxis.plotOffset;
                currentColAxis.Location.Y2 = realY;
                currentColAxis.x = currentColAxis.Location.X1;
                currentColAxis.y = currentColAxis.Location.Y1;
                currentColAxis.width = (currentColAxis.Location.X2 - currentColAxis.Location.X1);
                currentColAxis.height = currentColAxis.Location.Y2 - currentColAxis.Location.Y1;

            }
        }
        
        //For Vertical
        isInitAxes = true, isOppInitAxes = true;
        var yaxisOrign = this._calRowSize();
        var rowDefinition = this.model._rowDefinitions;
        for (var i = 0, rowLength = rowDefinition.length; i < rowLength; i++) {
            var y1 = yaxisOrign[i];
            var y2 = yaxisOrign[i + 1];
            var nearIndex = 0;
            var farIndex = 0;
            var isFirstAxes = true, isOppFirstAxes = true;
           var x = this.model.m_AreaBounds.X;
           var norX = x, oppX = x + this.model.m_AreaBounds.Width;
           for (var j = 0, maxRow = rowDefinition[i].axis.length; j < maxRow; j++) {
                var axisSpace = 0;
                var realX = 0;
               
                var currentAxis = rowDefinition[i].axis[j];
                var opposedPosition = currentAxis._opposed;
                if (!params.axes[currentAxis.name]._validCross) {
                    if (!opposedPosition) {
                        if (!(isFirstAxes && (isInitAxes || nearIndex == 0))) {
                            axisSpace = rowDefinition.nearSizes[nearIndex];
                            realX = norX = (norX - axisSpace);
                            nearIndex++;
                        } else {
                            realX = x;
                            isFirstAxes = false;
                        }

                    } else {
                        if (!(isOppFirstAxes && (isOppInitAxes || isOppInitAxes == 0))) {
                            axisSpace = rowDefinition.farSizes[farIndex];
                            realX = oppX = (oppX + axisSpace);
                            farIndex++;
                        } else {
                            realX = oppX;
                            isOppFirstAxes = false;
                        }
                    }
                }
                if (currentAxis.rowSpan) {
                    //Skip the axis size calculation for duplicate span axis
                    if (!(rowDefinition[i].index == currentAxis.rowIndex))
                        continue;
                    var axisRowSize = (rowLength >= (i + currentAxis.rowSpan)) ? i + currentAxis.rowSpan : i + 1;
                    y2 = yaxisOrign[axisRowSize];
                    if (!opposedPosition && nearIndex==0) isInitAxes = isFirstAxes;
                    if (opposedPosition && farIndex==0) isOppInitAxes = isOppFirstAxes;
                } else {
                    y2 = yaxisOrign[i + 1];
                    isInitAxes = true;
                    isOppInitAxes = true;
                }
                currentAxis.Location = { };
                currentAxis.Location.X1 = realX;
                currentAxis.Location.Y1 = y1 - currentAxis.plotOffset;
                currentAxis.Location.X2 = realX;
                currentAxis.Location.Y2 = y2 + currentAxis.plotOffset;
                currentAxis.x = currentAxis.Location.X1;
                currentAxis.y = currentAxis.Location.Y2;
                currentAxis.height = (currentAxis.Location.Y1 - currentAxis.Location.Y2);
                currentAxis.width = currentAxis.axisLine.width;

            }
        }
    },

     //Validates whether provided crossing value is valid or not
    _validateCrossing: function (axis) {
        axis._opposedPosition = false;
        if (axis.crossesAt != null) {
            var crossAxis = this._getCrossAxis(this.model._axes, axis.orientation.toLowerCase() == 'horizontal', axis.crossesInAxis);
            axis._crossValue = this._getCrossValue(axis, crossAxis, crossAxis._valueType);
            if ((crossAxis.isInversed ? !axis.opposedPosition : axis.opposedPosition) ? axis._crossValue <= crossAxis.visibleRange.min : axis._crossValue >= crossAxis.visibleRange.max)
                axis._opposedPosition = true;
            return (crossAxis.visibleRange.min < axis._crossValue && crossAxis.visibleRange.max > axis._crossValue);
        }
        return false;
    },

     //Method to place axis at crossed location
    _axisCrossing: function (axis, locateAxis, params) {
        var hor = axis.orientation.toLowerCase() == 'horizontal';
        var crossAxis = this._getCrossAxis(this.model._axes, hor, axis.crossesInAxis);
        var crossValue = axis._crossValue;
        var delta = Math.abs((crossAxis.isInversed ? crossAxis.visibleRange.max : crossAxis.visibleRange.min) - crossValue);
        this._locateAxis(axis, crossAxis, hor, delta);
        if (!locateAxis)
            params._crossAxisOverlap = this._avoidOverlapping(axis, this.model.m_AreaBounds, hor, axis.opposedPosition, params) || params._crossAxisOverlap;
    },

    _locateAxis: function (axis, crossAxis, hor, delta) {
        var loc = (hor ? crossAxis.height : crossAxis.width) * delta / crossAxis.visibleRange.delta;
        if (hor)
            axis.y = crossAxis.y + crossAxis.height - loc;
        else
            axis.x = crossAxis.x + loc;
    },

    _avoidOverlapping: function (axis, bounds, hor, opposed, params) {
        var diff = 0;
		var axisbounds = axis.showNextToAxisLine ?params.axes[axis.name]._bounds: 0;
        if (hor) {
            if (!opposed && axis.y + axisbounds > bounds.Y + bounds.Height) {
                bounds.Height -= diff = axis.y + axisbounds - bounds.Y - bounds.Height;
                this._recalculateSpace(axis, hor, opposed, diff);
                return true;
            }
            else if (opposed && axis.y - axisbounds < bounds.Y) {
                bounds.Y += diff = axisbounds - (axis.y - bounds.Y);
                bounds.Height -= diff;
                this._recalculateSpace(axis, hor, opposed, diff);
                return true;
            }
        }
        else {
            if (!opposed && axis.x - axisbounds < bounds.X) {
                bounds.X += diff = axisbounds - (axis.x - bounds.X);
                bounds.Width -= diff;
                this._recalculateSpace(axis, hor, opposed, diff);
                return true;
            }
            else if (opposed && axis.x + axisbounds > bounds.X + bounds.Width) {
                bounds.Width -= diff = axis.x + axisbounds - bounds.X - bounds.Width;
                this._recalculateSpace(axis, hor, opposed, diff);
                return true;
            }
            else if (axis.labelPosition == "inside" && axis.x + axisbounds > bounds.X + bounds.Width)
            {
                bounds.Width -= diff = axis.x + axisbounds - bounds.X - bounds.Width;
                this._recalculateSpace(axis, hor, opposed, diff, axis.showNextToAxisLine);
                return true;
            }
        }
    },

    _recalculateSpace: function (axis, isHor, opposed, diff,isCrossing) {
        if (isHor) {
            var def = this.model._rowDefinitions;
            var index = opposed ? def.length - 1 : 0;
            for (var i = 0, len = def[index].axis.length; i < len; i++) {
                def[index].axis[i].height -= diff;
                def[index].axis[i].Location.Y2 -= diff;
                if (opposed)
                    def[index].axis[i].Location.Y1 = def[index].axis[i].y += diff;
            }
            var colIndex = axis.columnIndex;
            def = this.model._columnDefinitions
            for (var i = 0, len = def.length; i < len; i++)
                if (i !== colIndex)
                    for (var j = 0, length = def[i].axis.length; j < length; j++)
                        if (def[i].axis[j]._opposed === opposed)
                            def[i].axis[j].Location.Y1 = def[i].axis[j].Location.Y2 = def[i].axis[j].y += (opposed ? 1 : -1) * diff;
        }
        else {
            var def = this.model._columnDefinitions;
            var index = opposed ? def.length - 1 : 0;
            for (var i = 0, len = def[index].axis.length; i < len; i++) {
                def[index].axis[i].width -= diff;
                if (!isCrossing) {
                    if (opposed)
                        def[index].axis[i].Location.X2 -= diff;
                    else
                        def[index].axis[i].Location.X1 = def[index].axis[i].x += diff;
                }

            }
            var rowIndex = axis.rowIndex;
            def = this.model._rowDefinitions
            for (var i = 0, len = def.length; i < len; i++)
                if (i !== rowIndex)
                    for (var j = 0, length = def[i].axis.length; j < length; j++)
                        if (def[i].axis[j]._opposed === opposed)
                            def[i].axis[j].Location.X1 = def[i].axis[j].Location.X2 = def[i].axis[j].x += (opposed ? -1 : 1) * diff;
        }
    },

    _getCrossValue: function (axis, crossAxis, type) {
        switch (type.toLowerCase()) {
            case 'category':
                return crossAxis.labels.indexOf(axis.crossesAt) != -1 ? crossAxis.labels.indexOf(axis.crossesAt) : axis.crossesAt;
            case 'datetime':
                return isNaN(Date.parse(axis.crossesAt)) ? null : Date.parse(axis.crossesAt);
            case 'logarithmic':
                return Math.log(axis.crossesAt) / Math.log(crossAxis.logBase);
        }
        return axis.crossesAt;
    },

    _getXCrossValue: function (series, axis, params) {
        var crossesInAxis = axis.crossesInAxis, yAxisName = series._yAxisName, validCross = params.axes[axis.name]._validCross || params.axes[axis.name]._validCross == null;
        //Checks whether origin should be changed for current series or not.
        return validCross && axis.crossesAt && ((!crossesInAxis && yAxisName === this.model.primaryYAxis.name) || (crossesInAxis && this._getCrossAxis(this.model._axes, true, crossesInAxis).name === yAxisName)) ? axis.crossesAt : 0;
    },

    _getAxisByName: function (axes, hor, name) {
        if (name) {
            for (var i = 0, count = axes.length; i < count; i++)
                if (axes[i].name === name && (hor ? axes[i].orientation=='vertical' : axes[i].orientation=='horizontal'))
                    return axes[i];
        }
    },

    _getCrossAxis: function (axes, hor, value) {
        return this._getAxisByName(axes, hor, value) || (this.model.requireInvertedAxes ? hor ? this.model.primaryXAxis : this.model.primaryYAxis : hor ? this.model.primaryYAxis : this.model.primaryXAxis);
    },

    _arrangeAxis: function () {
        var chartobj = this;
        var axis;
        var customRow;
        //Generate columnIndex/RowIndex value if it is not specify in sample  
        for (var i = 0; i < this.model._axes.length; i++) {
            axis = this.model._axes[i];
            if (axis.orientation.toLowerCase() == "horizontal") {
                chartobj.model.hAxes.push(axis);
                axis.columnIndex = (ej.util.isNullOrUndefined(axis.columnIndex)) ? 0 : (axis.columnIndex);
                chartobj.model.hAxes[chartobj.model.hAxes.length - 1].columnIndex = axis.columnIndex;
            } else if (axis.orientation.toLowerCase() == "vertical") {

                chartobj.model.vAxes.push(axis);
                axis.rowIndex = (ej.util.isNullOrUndefined(axis.rowIndex)) ? 0 : (axis.rowIndex);
                chartobj.model.vAxes[chartobj.model.vAxes.length - 1].rowIndex = axis.rowIndex;
                customRow = (!ej.util.isNullOrUndefined(chartobj.model.rowDefinitions)) ? chartobj.model.rowDefinitions[axis.rowIndex] : null;
                if (customRow) {
                    axis.axisBottomLine = {};
                    axis.axisBottomLine.visible = true;
                    axis.axisBottomLine.lineWidth = customRow.lineWidth;
                    axis.axisBottomLine.color = customRow.lineColor;
                }
            }
        }

        this.model.hAxes = ej.DataManager(this.model.hAxes, ej.Query().sortBy("columnIndex")).executeLocal();

        this.model.vAxes = ej.DataManager(this.model.vAxes, ej.Query().sortBy("rowIndex")).executeLocal();
        
        //column axis information push into _rowDefinitions
        var rowCount = this.axesCount(chartobj.model.vAxes);
         chartobj.model._rowDefinitions = [];
        if (chartobj.model.vAxes.length>0) {
            for (var j = 0; j < rowCount.length; j++) {
                var rowCollection = chartobj.model.rowDefinitions;
                var rowDefinitions = ((rowCollection)) ? (rowCollection[j] ? rowCollection[j] : rowCollection[0]) : null;
                chartobj.model._rowDefinitions.push({axis: this.axesIndexCount(chartobj.model.vAxes, rowCount.indexValue[j])
                                , index: rowCount.indexValue[j], rowDefinitions: rowDefinitions });
            }
        }

        var rowDefinition = chartobj.model._rowDefinitions;
        //Using filter to get spanning row axis collection
        var spanningRow = this._axisFilter(rowDefinition, "rowSpan");
        //Arrange the entire axis for row span
        this._SpanningAxes(rowDefinition, spanningRow, true);

        //column axis information push into _columnDefinitions
        var columnCount = this.axesCount(chartobj.model.hAxes);
        chartobj.model._columnDefinitions = [];
        if (chartobj.model.hAxes.length > 0) {
            for (var k = 0; k < columnCount.length; k++) {
                var columnCollection = chartobj.model.columnDefinitions;
                var columnDefinitions = (columnCollection) ? (columnCollection[k] ? columnCollection[k] : columnCollection[0]) : null;
                chartobj.model._columnDefinitions.push({
                    axis: this.axesIndexCount(chartobj.model.hAxes, columnCount.indexValue[k])
                    , index: columnCount.indexValue[k], columnDefinitions: columnDefinitions });
            }
        }
        
        var columnDefinition = chartobj.model._columnDefinitions;
        //Using filter to get spanning column axis collection
        var spanningColumn = this._axisFilter(columnDefinition, "columnSpan");
        //Arrange the entire axis for column span
        this._SpanningAxes(columnDefinition, spanningColumn, false);
       
        this._axisSize();
        
        
    },
    
     //Arrange the entire axis for spanning
    _SpanningAxes: function (definition,spanningCollection, isRow) {
        for (var spanIndex = 0, spanMax = spanningCollection.length; spanIndex < spanMax; spanIndex++) {
            var currentAxis = spanningCollection[spanIndex];
             if (currentAxis == null) break;
             var span = isRow ? currentAxis.rowSpan : currentAxis.columnSpan;
             var index = isRow ? currentAxis.rowIndex : currentAxis.columnIndex;
            if(definition[index]!=undefined){
             var axisIndex = definition[index].axis.indexOf(currentAxis);
             for (var k = 1, m = index + 1; k < span && m < definition.length; k++, m++) {
                 if (definition[m].axis.length >= axisIndex) {
                     //Insert duplicate span row to appropriate rowdefinition
                     definition[m].axis.splice(axisIndex, 0, currentAxis);
                 }
             }
             }
         }
     },
    
     //Filter span axis from axes
    _axisFilter: function (definition, fieldName) {
        var axisResult = [];
        definition.filter(function (axes) {
             //Execute query to get spanning axis from axis array
              var axisCollection = ej.DataManager(axes["axis"], ej.Query().where(fieldName, ">", 1)).executeLocal();
              for (var i = 0, max = axisCollection.length; i < max; i++)
                  axisResult.push(axisCollection[i]);
        });
        return axisResult;
    },
    
    _axisSize: function () {
        //The below calcultion to get approximate length of axis
         var spaceValue = this._getLegendSpace();
         var hSpace = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right - (this.model.elementSpacing * 2) - spaceValue.leftLegendWidth - spaceValue.rightLegendWidth;
        var hWidth = Math.floor(hSpace / this.axesCount(this.model.hAxes).length);

        var vSpace = $(this.svgObject).height() - this.model.margin.top - this.model.margin.bottom - (this.model.elementSpacing * 2) - spaceValue.topLegendHeight - spaceValue.bottomLegendHeight - spaceValue.modelTitleHeight-spaceValue.modelsubTitleHeight;
        var vWidth = Math.floor(vSpace / this.axesCount(this.model.vAxes).length);

       

        for (var j = 0; j < this.model._axes.length; j++) {
            axis = this.model._axes[j];
            if (axis.orientation.toLowerCase() == "horizontal") {
                if (this.model.AreaType != "polaraxes") {
                    var customColumn = (!ej.util.isNullOrUndefined(this.model.columnDefinitions)) ? this.model.columnDefinitions[axis.columnIndex] : [];
                    var columnLength = hWidth;
                    if (customColumn) {
                        customColumn.unit = (customColumn.unit == undefined) ? "percentage" : customColumn.unit;
                        customColumn.columnWidth = (customColumn.columnWidth == undefined) ? 100 : customColumn.columnWidth;
                        if (customColumn.unit.toLowerCase() == "percentage") {
                            var length = this._axisColumnDefinitionSize(this.model.columnDefinitions);
                            columnLength = Math.floor((hWidth) * (customColumn.columnWidth / length));
                        }
                        else
                            columnLength = customColumn.columnWidth;

                    }
                    axis.length = axis.columnSpan ? axis.columnSpan * columnLength : columnLength;
                }
                else {
                    if (hSpace > vSpace*2)
                        axis.length = hSpace - vSpace;
                    else
                        axis.length = hSpace;
                }
            } else if (axis.orientation.toLowerCase() == "vertical") {
                if (this.model.AreaType != "polaraxes") {
                    var customRow = (!ej.util.isNullOrUndefined(this.model.rowDefinitions)) ? this.model.rowDefinitions[axis.rowIndex] : [];
                    var realLength = vWidth;
                    if (customRow) {
                        customRow.unit = (customRow.unit == undefined) ? "percentage" : customRow.unit;
                        customRow.rowHeight = (customRow.rowHeight == undefined) ? 100 : customRow.rowHeight;
                        if (customRow.unit.toLowerCase() == "percentage") {
                            var length = this._axisRowDefinitionSize(this.model.rowDefinitions);
                            realLength = Math.floor((vWidth) * (customRow.rowHeight / length));
                        }
                        else
                            realLength = customRow.rowHeight;

                    }
                    axis.length = axis.rowSpan ? (axis.rowSpan * realLength) : realLength;
                }
                else {
                    axis.length = vSpace / 2;
                }
            }
        }
    },
	_axisRowDefinitionSize: function (array) {
        var length = 0;
        if (array) {
            for (var i = 0; i < array.length; i++) {
                length = length + array[i].rowHeight;
            }
        }
        length = length > 100 ? length : 100;
        return length;
    },
    _axisColumnDefinitionSize: function (array) {
        var length = 0;
        if (array) {
            for (var i = 0; i < array.length; i++) {
                length = length + array[i].columnWidth;
            }
        }
        length = length > 100 ? length : 100;
        return length;
    },
   _axisMeasure: function (axis, realWidth, orientation,j) {
       var axisTitleHeight = 0, labelSize = 0, multiLevelLabelSize = 0;
        axis.axisLine.width = axis.axisLine.width < 0 ? 0 : axis.axisLine.width;
		  axis.majorTickLines.size = axis.majorTickLines.size < 0 ? 0 : axis.majorTickLines.size;
		var axisLineWidth = !this.model.enable3D && axis.axisLine.visible && axis.axisLine.width;
		var size = ej.EjSvgRender.utils._measureText(axis.title.text, realWidth, axis.title.font);
		if(orientation == "vertical")
		{
		 labelSize= axis._LableMaxWidth.width;
		 axisTitleHeight = (axis.title.text == "" || !(axis.title.visible) || !(axis.visible)) ? 0 : ((document.documentMode === 8) ? size.width : size.height + (2 * this.model.elementSpacing));
		}
		else
		{
		labelSize=(axis._LableMaxWidth.height);
		axisTitleHeight = (axis.title.text == "" || !(axis.title.visible) || !(axis.visible)) ? 0 : (size.height + (this.model.elementSpacing));
		}
		axisTitleHeight = axis.title.position.toLowerCase() == "inside" ? 0: axisTitleHeight;
		multiLevelLabelSize = this.getMultiLevelLabelSize(axis);
		var width;		
		if ((j == 0) && (axis.tickLinesPosition == 'inside' && axis.labelPosition == 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + multiLevelLabelSize + axisLineWidth + (this.model.elementSpacing);
		else if ((j == 0) && (axis.tickLinesPosition == 'inside' && axis.labelPosition != 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + multiLevelLabelSize + axisLineWidth + (this.model.elementSpacing) + labelSize;
		else if ((j == 0) && (axis.tickLinesPosition != 'inside' && axis.labelPosition == 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + multiLevelLabelSize + axisLineWidth + (this.model.elementSpacing) + axis.majorTickLines.size;
		else
		    axis.AxisMaxWidth = axisTitleHeight + multiLevelLabelSize + axisLineWidth + this.model.elementSpacing + axis.majorTickLines.size + labelSize;
		axis.AxisMaxWidth += axis._isScroll ? 18 : 0;
        return axis.AxisMaxWidth;
    },
     
    _arraySum: function (val) {
        var total = 0;
        for (var i = 0; i < val.length; i++) {
            total +=  parseFloat(val[i]) || 0;
        }
        return total;
    },
    _calSpace: function (realWidth, definitions,orientation, params) {
        var nearSizes = [];
        var farSizes = [];
        for (var i = 0; i < definitions.length; i++) {
            var nearIndex = 0, farIndex = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {
                var currentaxis = definitions[i].axis[j];                
                    bounds = params.axes[currentaxis.name]._bounds = this._axisMeasure(currentaxis, realWidth, orientation, j);
                    validCross = params.axes[currentaxis.name]._validCross = this._validateCrossing(currentaxis);
                    measureValue = validCross ? 0 : bounds;
                currentaxis._opposed = currentaxis.opposedPosition ? !currentaxis._opposedPosition : currentaxis._opposedPosition || currentaxis.opposedPosiiton||false;
                if (validCross && definitions[i].axis[j].showNextToAxisLine)
                    continue;
                if (!currentaxis.showNextToAxisLine)
                { measureValue = (currentaxis.labelPosition=="inside")? bounds-currentaxis._multiLevelLabelHeight : bounds ; }
                if (!currentaxis._opposed) {
                    if (nearSizes.length <= nearIndex) {
                        nearSizes.push(measureValue);
                    } else if (nearSizes[nearIndex] < (measureValue)) {
                        nearSizes[nearIndex] = measureValue;
                    }
                    nearIndex++;
                }
                else {
                    if (farSizes.length <= farIndex) {
                        farSizes.push(measureValue);
                    } else if (farSizes[farIndex] < (measureValue)) {
                        farSizes[farIndex] = measureValue;
                    }
                    farIndex++;
                }                

            }
        }        
        return { nearSizes: nearSizes, farSizes: farSizes };
    },
    
    _getLegendSpace: function () {
        var chart = this.model,
            legend = chart.legend,
            position = legend.position.toLowerCase(),
            bounds = chart.LegendActualBounds,
            measureText = ej.EjSvgRender.utils._measureText,
            title = chart.title,
            subTitle = title.subTitle,
            elementSpacing = chart.elementSpacing,
            margin = chart.margin,
            svgWidth = chart.svgWidth,
            legendBorder = legend.border.width,
            ltheight = 0, space = 0,
            itemPadding = 10,
            leftLegendWidth = 0, rightLegendWidth = 0,
            topLegendHeight = 0, bottomLegendHeight = 0,
           labelCollection = title.text.split(' '),
           text, line = 0, enableTrim = title.enableTrim,
           maxTitleWidth = title.maximumWidth,
           maxTitleWidth = (maxTitleWidth.toString() == 'auto' || maxTitleWidth.toString() == '') ? (svgWidth * 0.75) : parseInt(maxTitleWidth),
           textOverflow = title.textOverflow.toLowerCase(),
           titleText = title.text,
           titleVisible = title.visible, titleCollection, subTitleCollection, data,
           measureTitle = ej.EjSvgRender.utils._measureText(title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, title.font),
           subTitleLabelCollection = subTitle.text.split(' '),
           subTitleEnableTrim = subTitle.enableTrim,
           maxSubTitleWidth = subTitle.maximumWidth,
           maxSubTitleWidth = (maxSubTitleWidth.toString() == 'auto' || maxSubTitleWidth.toString() == '') ? (svgWidth * 0.75) : parseInt(maxSubTitleWidth),
           subTitleTextOverflow = subTitle.textOverflow.toLowerCase(),
           subTitleText = subTitle.text,
           subTitleVisible = subTitle.visible,
           measureSubTitle = ej.EjSvgRender.utils._measureText(subTitle.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, subTitle.font),
           titleVisibility = titleVisible && enableTrim && measureTitle.width > maxTitleWidth ? true : false,
           subTitleVisibility = subTitleVisible && subTitleEnableTrim && measureSubTitle.width > maxSubTitleWidth ? true : false,
           textOverflowVisibility = textOverflow == "wrap" || textOverflow == "wrapandtrim" ? true : false,
           subTitleTextOverflowVisibility = subTitleTextOverflow == "wrap" || subTitleTextOverflow == "wrapandtrim" ? true : false;
           this.model.titleWrapTextCollection = [];
           this.model.subTitleWrapTextCollection = [];
           this.model.trimTooltip = false;
           this.model.subTitleTooltip = false;
        if (legend.visible) {
            space = (bounds.Width + (itemPadding / 2) + elementSpacing + (2 * legendBorder));
            leftLegendWidth = position == 'left' ? space : 0;
            rightLegendWidth = position == 'right' ? space : 0;
            if (legend.title.text)
                ltheight = measureText(legend.title, null, legend.title.font).height;
            topLegendHeight = position == 'top' ? (bounds.Height + ltheight + (2 * legendBorder)) : 0;
            bottomLegendHeight = position == 'bottom' ? (bounds.Height + ltheight + (2 * legendBorder)) : 0;
        }
        if (titleVisibility && textOverflow == "wrap") {
            data = { text: titleText, font: title.font };
            titleCollection = this._rowsCalculation(data, maxTitleWidth, textOverflow);
        }
        else if (titleVisibility && textOverflow == "wrapandtrim") {
            data = { text: titleText, font: title.font };
            titleCollection = this._rowsCalculation(data, maxTitleWidth, textOverflow);
        }
        else if (textOverflow == "wrapandtrim" || textOverflow == "wrap") {
            data = { text: titleText, font: title.font };
            titleCollection = this._rowsCalculation(data, ej.EjSvgRender.utils._measureText(titleText, null, title.font).width, textOverflow);
        }
        this.model.titleWrapTextCollection = titleCollection ? titleCollection.textCollection : "";
        this.model.titleMaxWidth = titleCollection ? titleCollection.wordMax : maxTitleWidth;
        if (subTitleVisibility && subTitleTextOverflow == "wrap" ) {
            data = { text: subTitleText, font: subTitle.font };
            subTitleCollection = this._rowsCalculation(data, maxSubTitleWidth, subTitleTextOverflow);
        }
        else if (subTitleVisibility && subTitleTextOverflow == "wrapandtrim" ) {
            data = { text: subTitleText, font: subTitle.font };
            subTitleCollection = this._rowsCalculation(data, maxSubTitleWidth, subTitleTextOverflow);
            this.model.subTitleTooltip = this.model.trimTooltip;
        }
        else if (subTitleTextOverflow == "wrapandtrim" || subTitleTextOverflow == "wrap") {
            data = { text: subTitleText, font: subTitle.font };
            subTitleCollection = this._rowsCalculation(data, maxSubTitleWidth, subTitleTextOverflow);
        }
        this.model.subTitleWrapTextCollection = subTitleCollection ? subTitleCollection.textCollection : "";
        this.model.subTitleMaxWidth = subTitleCollection ? subTitleCollection.wordMax : maxSubTitleWidth;
        var titleHeight = measureText(title.text, svgWidth - margin.left - margin.right, title.font).height,
             subTitleHeight = measureText(subTitle.text, svgWidth - margin.left - margin.right, subTitle.font).height;
        modelTitleHeight = (title.text == "" || !title.visible) ? 0 : (enableTrim && (textOverflowVisibility) && measureTitle.width > maxTitleWidth) ?
            ((titleHeight * this.model.titleWrapTextCollection.length) + elementSpacing) : (titleHeight + elementSpacing);
        modelsubTitleHeight = (subTitle.text == "" || !subTitle.visible || !title.visible) ? 0 : (subTitleEnableTrim && (subTitleTextOverflowVisibility) && measureSubTitle.width > maxSubTitleWidth) ?
            ((subTitleHeight * this.model.subTitleWrapTextCollection.length) + elementSpacing) : (subTitleHeight + elementSpacing);

        return {
            leftLegendWidth: leftLegendWidth, rightLegendWidth: rightLegendWidth, topLegendHeight: topLegendHeight, bottomLegendHeight: bottomLegendHeight, modelTitleHeight: modelTitleHeight, modelsubTitleHeight: modelsubTitleHeight
        };
    },


    _arraySome: function (axes,orientation) {
        var scroll = false,
            isZoomScroll = this.model.zooming.enableScrollbar,
            count = axes.length,
            scrollbar,
            pointLength;

        for (var i = 0; i < count; i++) {
            scrollbar = axes[i].scrollbarSettings.enableScrollbar;
            pointLength = axes[i].scrollbarSettings.pointsLength;
            scroll = (scroll || (((axes[i]._isScroll && axes[i].maxPointLength < pointLength) || (axes[i].zoomFactor < 1 && scrollbar && isZoomScroll)) && axes[i].orientation.toLowerCase() == orientation));
        }

        return scroll;
    },

    _calculateAreaBounds: function (params) {
        var chartobj = this,
        realWidth = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right,
        // Calculate area bounds X and width
        chartBorderWidth = chartobj.model.border.width,
        axis = this.model._axes[0],
        zomming = this.model.zooming.enableScrollbar,
        type = this.model.zooming.type.toLowerCase();
        chartobj.model.scrollerSize = 18;
        this.model._yScroll = this._arraySome(this.model._axes, 'vertical');
        this.model._xScroll = this._arraySome(this.model._axes, 'horizontal');

        //Calcultion for multiple axes(vertical)
        var sizes = this._calSpace(realWidth, chartobj.model._rowDefinitions,"vertical", params);
        chartobj.model._rowDefinitions.nearSizes = sizes.nearSizes;
        chartobj.model._rowDefinitions.farSizes = sizes.farSizes;
        var vAxesWidth = this._arraySum(sizes.nearSizes);
        var vAxesOppWidth = this._arraySum(sizes.farSizes);

        var spaceValue=this._getLegendSpace();
       
        var x = vAxesWidth + spaceValue.leftLegendWidth + chartobj.model.margin.left + chartBorderWidth;
        var rightSpacing = vAxesOppWidth + spaceValue.rightLegendWidth + chartobj.model.margin.right + chartobj.model.margin.left + (2 * chartBorderWidth);
        var width = $(this.svgObject).width();
        var boundsWidth =  width - (x + rightSpacing);
        if (boundsWidth < 0 && this.model.AreaType == "cartesianaxes") {
            $(this.svgObject).width(width - boundsWidth + 1);
            this._calculateAreaBounds(params);
        }
        else {
        // Calculate area bounds Y and Height   
        
        //Calcultion for multiple axes(Horizontal)
        var columnSizes = this._calSpace(realWidth, chartobj.model._columnDefinitions,"horizontal", params);
        chartobj.model._columnDefinitions.nearSizes = columnSizes.nearSizes;
            chartobj.model._columnDefinitions.farSizes = columnSizes.farSizes;
            var hAxesWidth = this._arraySum(columnSizes.nearSizes);
            var hAxesOppWidth = this._arraySum(columnSizes.farSizes);

            var y = hAxesOppWidth + this.model.margin.top + (this.model.elementSpacing) + spaceValue.modelTitleHeight + (spaceValue.modelsubTitleHeight) + spaceValue.topLegendHeight + chartBorderWidth;
            var bottomSpacing = hAxesWidth + this.model.margin.bottom + spaceValue.bottomLegendHeight + (2 * chartBorderWidth);
            var boundsHeight = Math.abs($(this.svgObject).height() - (y + bottomSpacing));

            this.model.m_AreaBounds = { X: x, Y: y, Width: boundsWidth, Height: boundsHeight };
            this.model.m_Spacing = { Left: x, Top: y, Right: rightSpacing, Bottom: bottomSpacing };

            if (this.model.enableCanvasRendering) {  // for adding bounds in canvas rendering
                this.canvasX = x;
                this.canvasY = y;
                this.canvasWidth = boundsWidth;
                this.canvasHeight = boundsHeight;
            } else 
                this.canvasX = this.canvasY = this.canvasHeight = this.canvasWidth = 0;
        }
      }
};

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};


 
})(jQuery);;
/**
* @fileOverview Plugin to style the Html Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej) {

    ej.widget("ejChart", "ej.datavisualization.Chart", {
 
        element: null,
 
        model: null,

        validTags: ["div"],
 
        defaults: {

            border: {

                color: 'transparent',

                width: 0,

                opacity: 0.3
            },

            chartArea:
            {

                border: {

                    color: 'Gray',

                    width: 0.5,

                    opacity: 0.3
                },

                background: 'transparent'

            },

            primaryXAxis:
            {
	            crossesAt: null,

         		crossesInAxis: null,

         		isIndexed: false,

         		alignment: "center",

                labelPlacement: "",

                scrollbarSettings: {

         		    visible: true,
                    
         		    canResize: true,

         		    range: {

         		        min: null,

                        max: null,
         		    },

                    pointsLength : null

         		},

         		range: {

                    min: null,

                    max: null,

                    interval: null
                },

                labelPosition: 'outside',

                tickLinesPosition: 'outside',

                alternateGridBand:
                        {

                            odd: {

                                fill: "transparent",

                                opacity: 1
                            },

                            even: {

                                fill: "transparent",

                                opacity: 1
                            }
                        },

                enableAutoIntervalOnZooming : true,
                         

                majorGridLines:
                {

                    width: 1,

                    dashArray: "",

                    visible: true,

                    opacity: 1

                },

                majorTickLines:
               {

                   width: 1,

                   size: 5,

                   visible: true
               },

                minorGridLines:
               {

                   width: 1,

                   dashArray: "",

                   visible: true

               },

                minorTickLines:
                {

                    width: 1,

                    size: 5,

                    visible: true
                },

                labelBorder:
               {	
                    width: 0
               },

                multiLevelLabels: [{
                    visible: false,
                    text: "",
                    textAlignment: "center",
                    start: null,
                    end: null,
                    level: 0,
                    maximumTextWidth: null,
                    textOverflow: "trim",
                    font: {
                        fontFamily: 'Segoe UI',
                        fontStyle: 'Normal',
                        size: '12px',
                        fontWeight: 'Regular',
                        opacity: 1
                    },
                    border: {
                        type: 'rectangle',
                        width: 1
                    }
                }],

                minorTicksPerInterval: null,

                columnIndex: null,

                columnSpan: null,

                labelRotation: null,

                valueType: null,

                name: null,

                labelFormat: null,

                desiredIntervals: null,

                intervalType: null,

                roundingPlaces: null,

                logBase: 10,

                plotOffset: 0,
                labels: [],

                stripLine: [
                       {

                           visible: false,

                           startFromAxis: false,

                           text: "",

                           width: 0,

                           textAlignment: "middlecenter",

                           font: {

                               fontFamily: 'Segoe UI',

                               fontStyle: 'Normal',

                               size: '12px',

                               fontWeight: 'Regular',

                               color: 'black',

                               opacity: 1
                           },

                           start: null,

                           end: null,

                           color: 'gray',

                           borderColor: 'black',

                           zIndex: 'over'
                       }
                ],

                title:
               {


                   text: "",

				    visible:true,

                   enableTrim: false,
				   
				   offset:0,
				
				   alignment:'center',
				
				   position:'outside',

                   maximumTitleWidth: null,

                   font:
                   {

                       fontFamily: 'Segoe UI',

                       fontStyle: 'Normal',

                       size: '14px',

                       opacity: 1,

                       fontWeight: 'regular'
                   }
               },
                rangePadding: 'Auto',
                additionalPadding: [1, 1],
                orientation: 'Horizontal',

                maximumLabels: 3,

                opposedPosition: false,

                showNextToAxisLine: true,

                axisLine:
                {

                    visible: true,

                    width: 1,

                    dashArray: "",

                    offset: 0
                },

                labelIntersectAction: "none",

                maximumLabelWidth: 34,

                enableTrim: false,


                edgeLabelPlacement: "none",



                isInversed: false,



                font:
                {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    size: '11px',

                    fontWeight: 'regular',

                    opacity: 1
					
                },

                visible: true,

                crosshairLabel:
                {

                    visible: false
                },

                zoomFactor: 1,

                zoomPosition: 0
            },


            primaryYAxis:
              {
                  crossesAt: null,
                  crossesInAxis: null,
                  showNextToAxisLine: true,

                  alignment: "center",

                  startFromZero: true,

                  range: {

                      min: null,

                      max: null,

                      interval: null
                  },

                  scrollbarSettings: {

                      visible: true,

                      canResize: true,

                      range: {

                          min: null,

                          max: null,
                      },

                      pointsLength: null

                  },

                  labelPlacement: "",

                  labelPosition: 'outside',

                  tickLinesPosition: 'outside',

                  alternateGridBand:
                             {

                                 odd: {

                                     fill: "transparent",

                                     opacity: 1
                                 },

                                 even: {


                                     fill: "transparent",

                                     opacity: 1
                                 }
                             },

                  enableAutoIntervalOnZooming: true,                                

                  majorGridLines:
                 {

                     width: 1,

                     dashArray: "",

                     visible: true,

                     opacity: 1

                 },

                  majorTickLines:
                {

                    width: 1,

                    size: 5,

                    visible: true
                },

                  minorGridLines:
                 {

                     width: 1,

                     dashArray: "",

                     visible: true

                 },

                  minorTickLines:
                {

                    width: 1,

                    size: 5,

                    visible: true
                },

                  labelBorder:
               {	
                    width: 0
               },

                multiLevelLabels: [{
                    visible: false,
                    text: "",
                    textAlignment: "center",
                    start: null,
                    end: null,
                    level: 0,
                    maximumTextWidth: null,
                    textOverflow: "trim",
                    font: {
                        fontFamily: 'Segoe UI',
                        fontStyle: 'Normal',
                        size: '12px',
                        fontWeight: 'Regular',
                        opacity: 1
                    },
                    border: {
                        type: 'rectangle',
                        width: 1
                    }
                }],

                  minorTicksPerInterval: null,

                  rowIndex: null,

                  rowSpan: null,

                  valueType: null,

                  name: null,

                  labelFormat: null,

                  desiredIntervals: null,

                  intervalType: null,

                  roundingPlaces: null,
                  labels: [],

                  title:
                  {

                      text: "",

					   visible:true,

                      enableTrim: false,
					  
					  offset:0,
				
					  alignment:'center',
				
				      position:'outside',
					  
                      maximumTitleWidth: null,

                      font:
                     {

                         fontFamily: 'Segoe UI',

                         fontStyle: 'Normal',

                         opacity: 1,

                         size: '14px',

                         fontWeight: 'regular'
                     }
                  },

                  rangePadding: 'Auto',


                  stripLine: [
                       {

                           visible: false,

                           startFromAxis: false,

                           width: 0,

                           text: "",

                           textAlignment: "middlecenter",

                           font: {

                               fontFamily: 'Segoe UI',

                               fontWeight: 'Regular',

                               fontStyle: 'Normal',

                               size: '12px',

                               color: 'black',

                               opacity: 1
                           },

                           start: null,

                           end: null,

                           color: 'gray',

                           borderColor: 'black',

                           zIndex: 'over'
                       }
                  ],

                  logBase: 10,

                  plotOffset: 0,

                  orientation: 'Vertical',

                  maximumLabels: 3,

                  labelIntersectAction: "none",


                  maximumLabelWidth: 34,

                  enableTrim: false,

                  edgeLabelPlacement: "none",


                  isInversed: false,


                  crosshairLabel:
                 {

                     visible: false
                 },

                  visible: true,

                  opposedPosition: false,

                  font:
               {

                   fontFamily: 'Segoe UI',

                   fontStyle: 'Normal',

                   size: '11px',

                   opacity: 1,

                   fontWeight: 'regular'
				   
               },

                  axisLine:
              {

                  visible: true,

                  width: 1,

                  dashArray: "",

                  offset: 0
              },


                  zoomFactor: 1,

                  zoomPosition: 0
              },
            axes: [],

            secondaryX:
             {
                 crossesAt: null,
                 crossesInAxis: null,
				 showNextToAxisLine: true,
                 alignment: "center",
                 range: {
                     min: null,
                     max: null,
                     interval: null
                 },

                 scrollbarSettings: {

                     visible: true,

                     canResize: true,

                     range: {

                         min: null,

                         max: null,
                     },

                     pointsLength: null

                 },

                 tickLinesPosition: "outside",
                 labelPosition: "outside",

                 majorGridLines:
        {

            width: 1,

            dashArray: "",

            visible: true
        },
                 alternateGridBand:
           {
               odd: {
                   fill: "transparent",
                   opacity: 1
               },
               even: {
                   fill: "transparent",
                   opacity: 1
               }
           },

                 enableAutoIntervalOnZooming: true,                                

                 majorTickLines:
         {

             width: 1,

             size: 5,

             visible: true
         },

                 minorGridLines:
         {

             width: 1,

             dashArray: "",

             visible: true

         },

                 minorTickLines:
        {

            width: 1,

            size: 5,

            visible: true
        },

				   labelBorder:
               {	
                    width: 0
               },

                multiLevelLabels: [{
                    visible: false,
                    text: "",
                    textAlignment: "center",
                    start: null,
                    end: null,
                    level: 0,
                    maximumTextWidth: null,
                    textOverflow: "trim",
                    font: {
                        fontFamily: 'Segoe UI',
                        fontStyle: 'Normal',
                        size: '12px',
                        fontWeight: 'Regular',
                        opacity: 1
                    },
                    border: {
                        type: 'rectangle',
                        width: 1
                    }
                }],

                 minorTicksPerInterval: null,

                 columnIndex: null,

                 columnSpan: null,

                 labelRotation: null,

                 valueType: null,

                 name: null,

                 labelFormat: null,

                 labelPlacement: "",

                 desiredIntervals: null,

                 intervalType: null,

                 roundingPlaces: null,

                 logBase: 10,

                 plotOffset: 0,
                 labels: [],
                 stripLine: [
              {
                  visible: false,
                  startFromAxis: false,
                  text: "",
                  textAlignment: "middlecenter",
                  font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'regular', size: '12px', color: 'black', opacity: 1 },
                  start: null,
                  end: null,
                  color: 'gray',
                  borderColor: 'black',
                  zIndex: 'over'
              }
                 ],

                 title:
         {


             text: "",
             visible:true,
             enableTrim: false,
			 offset:0,
			 alignment:'center',	
			 position:'outside',
             maximumTitleWidth: null,
             font:
        {

            fontFamily: 'Segoe UI',

            fontStyle: 'Normal',

            size: '14px',

            opacity: 1,

            fontWeight: 'regular'
			
        }

         },

                 rangePadding: 'Auto',
                 additionalPadding: [1, 1],

                 orientation: 'Horizontal',

                 maximumLabels: 3,

                 opposedPosition: false,

                 axisLine:
          {

              visible: true,

              width: 1,

              dashArray: "",

              offset: 0
          },

                 labelIntersectAction: "none",

                 edgeLabelPlacement: "none",

                 font:
          {

              fontFamily: 'Segoe UI',

              fontStyle: 'Normal',

              size: '11px',

              fontWeight: 'regular',

              opacity: 1
          },

                 visible: true,

                 crosshairLabel:
           {

               visible: false
           },

                 zoomFactor: 1,

                 zoomPosition: 0
             },

            secondaryY:
                {
                    crossesAt: null,
                    crossesInAxis: null,
					showNextToAxisLine: true,
                    alignment: "center",
                    startFromZero: true,
                    range: {
                        min: null,
                        max: null,
                        interval: null
                    },

                    scrollbarSettings: {

                        visible: true,

                        canResize: true,

                        range: {

                            min: null,

                            max: null,
                        },

                        pointsLength: null

                    },

                    tickLinesPosition: "outside",
                    labelPosition: "outside",

                    majorGridLines:
       {

           width: 1,

           dashArray: "",

           visible: true

       },
                    alternateGridBand:
           {
               odd: {
                   fill: "transparent",
                   opacity: 1
               },
               even: {
                   fill: "transparent",
                   opacity: 1
               }
           },

                    enableAutoIntervalOnZooming: true,                                   

                    majorTickLines:
                        {

                            width: 1,

                            size: 5,

                            visible: true
                        },

                    minorGridLines:
                        {

                            width: 1,

                            dashArray: "",

                            visible: true

                        },

                    minorTickLines:
                        {

                            width: 1,

                            size: 5,

                            visible: true
                        },

                    labelBorder:
               {	
                    width: 0
               },

                multiLevelLabels: [{
                    visible: false,
                    text: "",
                    textAlignment: "center",
                    start: null,
                    end: null,
                    level: 0,
                    maximumTextWidth: null,
                    textOverflow: "trim",
                    font: {
                        fontFamily: 'Segoe UI',
                        fontStyle: 'Normal',
                        size: '12px',
                        fontWeight: 'Regular',
                        opacity: 1
                    },
                    border: {
                        type: 'rectangle',
                        width: 1
                    }
                }],

                    minorTicksPerInterval: null,

                    rowIndex: null,

                    rowSpan: null,

                    valueType: null,

                    name: null,

                    labelFormat: null,

                    desiredIntervals: null,

                    intervalType: null,

                    roundingPlaces: null,
                    labels: [],

                    title:
                    {

                        text: "",
                        visible:true,
                        enableTrim: false,
						offset:0,			
						alignment:'center',
						position:'outside',
                        maximumTitleWidth: null,
                        font:
                        {
                            fontFamily: 'Segoe UI',

                            fontStyle: 'Normal',

                            opacity: 1,

                            size: '14px',

                            fontWeight: 'regular'
                        }
                    },

                    rangePadding: 'Auto',

                    stripLine: [
                         {
                             visible: false,
                             startFromAxis: false,
                             text: "",
                             textAlignment: "middlecenter",
                             font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'regular', size: '12px', color: 'black', opacity: 1 },
                             start: null,
                             end: null,
                             color: 'gray',
                             borderColor: 'black',
                             zIndex: 'over'
                         }
                    ],

                    logBase: 10,

                    plotOffset: 0,

                    orientation: 'Vertical',

                    maximumLabels: 3,

                    labelIntersectAction: "none",

                    labelPlacement: "",

                    edgeLabelPlacement: "none",

                    crosshairLabel:
                    {

                        visible: false
                    },

                    visible: true,

                    opposedPosition: false,

                    font:
                    {
                        fontFamily: 'Segoe UI',

                        fontStyle: 'Normal',

                        size: '11px',

                        opacity: 1,

                        fontWeight: 'regular'
						
                    },

                    axisLine:
                        {

                            visible: true,

                            width: 1,

                            dashArray: "",

                            offset: 0
                        },


                    zoomFactor: 1,

                    zoomPosition: 0
                },
            trendlineDefaults: {
                type: "linear",
                visibility: "",
				visibleOnLegend:"visible",
                name: "Trendline",
                fill: "",
                width: 1,
                opacity: 1,
                dashArray: "",
                forwardForecast: 0,
                backwardForecast: 0,
                polynomialOrder: 2,
                period: 2,
				intercept: null,
                tooltip: {
                    visible: false, format: null, fill: null, border: { width: 1, color: null }, duration: '500ms', enableAnimation: true, opacity: 0.95, font: {fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', opacity: 1}
                }
            },
            indicatorDefaults: {
                visible: true,
                points: [],
                tooltip: { visible: false, format: null, fill: null, border: { width: 1, color: null }, duration: '500ms', enableAnimation: true, opacity: 0.95 },
                seriesName: "",
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 },
                type: "SMA",
                period: 14,
                standardDeviations: 2,
                kPeriod: 3,
                dPeriod: 3,
                periodLine: { fill: "blue", width: 2 },
                lowerLine: { fill: "#008000", width: 2 },
                upperLine: { fill: "#ff0000", width: 2 },
                macdLine: { fill: "#ff9933", width: 2 },
                histogram: { fill: "#ccccff", opacity: 1, border: { color: "#9999ff", width: 1 } },
                fill: "#00008B",
                width: 2,
                xAxisName: "",
                yAxisName: "",
                visibility: "visible",
                macdType: "line",
                shortPeriod: 12,
                longPeriod: 26,
                trigger: 9,
                enableAnimation: false
            },
            annotationsDefault: {
                visible: false,
                content: "",
                coordinateUnit: "none",
                verticalAlignment: "middle",
                horizontalAlignment: "middle",
                region: "chart",
                x: 0,
                y: 0,
                opacity: 1,
                angle: 0,
                xAxisName: "",
                yAxisName: "",
                margin: {
                    left: 0,
                    right: 0,
                    bottom: 0,
                    top: 0
                }
            },
            multiLevelLabelsDefault: {
                  visible: false,
                    text: "",
                    textAlignment: "center",
                    start: null,
                    end: null,
                    level: 0,
                    maximumTextWidth: null,
                    textOverflow: "trim",
                    font: {
                        fontFamily: 'Segoe UI',
                        fontStyle: 'Normal',
                        size: '12px',
                        fontWeight: 'Regular',
                        opacity: 1
                    },
                    border: {
                        type: 'rectangle',
                        width: 1
                    }
            },
            stripLineDefault: {

                visible: false,

                startFromAxis: false,

                width: 0,

                text: "",

                textAlignment: 'middlecenter',

                font:
                {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px',

                    color: 'black',

                    opacity: 1
                },

                start: null,

                end: null,

                color: 'gray',

                borderColor: 'black',

                zIndex: 'over',

                borderWidth: 1
            },

            rowDefinitions: null,

            columnDefinitions: null,

            title:
				{

				    text: "",

				    enableTrim: false,
                    
                    visible: true,

				    maximumWidth: 'auto',

                    textOverflow: "trim",

				    textAlignment: "center",

				    background: 'transparent',

				    border: {

				        color: 'transparent',

				        width: 1,

				        opacity: 0.8,

				        cornerRadius: 0.8

				    },

				    font: {

				        fontFamily: 'Segoe UI',

				        fontStyle: 'Normal',

				        size: '16px',

				        opacity: 1,

				        fontWeight: 'regular'
				    },


				    subTitle:
                           {

                               text: "",

                               enableTrim: false,

                               visible: true,

                               maximumWidth: 'auto',

                               textOverflow: "trim",

                               textAlignment: 'far',

                               background: 'transparent',

                               border: {

                                   color: 'transparent',

                                   width: 1,

                                   opacity: 0.8,

                                   cornerRadius: 0.8

                               },


                               font:
                               {

                                   fontFamily: 'Segoe UI',

                                   fontStyle: 'Normal',

                                   size: '12px',

                                   opacity: 1,

                                   fontWeight: 'regular'
                               }
                           }

				},





            lineCap: { butt: 'butt', round: 'round', square: 'square' },
            lineJoin: { round: 'round', bevel: 'bevel', miter: 'miter' },
            legendAlignment: { near: 'near', center: 'center', far: 'far' },
            legendPosition: { top: 'top', bottom: 'bottom', right: 'right', left: 'left', custom: 'custom' },

            enableAnimation: true,

            legend:
            {

                title:
                      {
                          text: "",

                          textAlignment: 'center',

                          font:
                              {

                                  size: '12px',

                                  fontFamily: "Segoe UI",

                                  fontStyle: 'normal',

                                  fontWeight: 'regular'
                              }
                      },

                border:
                {

                    color: 'transparent',


                    width: 1
                },

                visible: true,

                textOverflow: 'none',

                textWidth:34,

                toggleSeriesVisibility: true,
				
				enableScrollbar: true,

                itemPadding: 10,

                shape: 'None',

                alignment: 'Center',

                position: 'Bottom',

                location:
                {

                    x: 0,

                    y: 0
                },

                itemStyle:
                {
                    height: 9,

                    width: 9,

                    border:
                    {

                        color: 'transparent',

                        width: 1
                    }
                },

                rowCount: null,

                columnCount: null,

                opacity: 1,

                fill: null,

                background: null,

                font:
                {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px'
                },

                size:
                    {

                        height: null,

                        width: null
                    }
            },

            indicators: [
              {
                  points: [],

                  tooltip:
				  {

				      visible: false,

				      format: null,

				      fill: null,

				      border: {

				          width: 1,

				          color: null
				      },

				      duration: '500ms',

				      enableAnimation: true,

				      opacity: 0.95
				  },

                  seriesName: "",

                  type: "SMA",

                  period: 14,

                  standardDeviations: 2,

                  kPeriod: 3,

                  dPeriod: 3,

                  periodLine: {

                      fill: "blue",

                      width: 2
                  },

                  lowerLine: {


                      fill: "#008000",

                      width: 2
                  },

                  upperLine: {

                      fill: "#ff0000",

                      width: 2
                  },

                  macdLine: {

                      fill: "#ff9933",

                      width: 2
                  },

                  histogram: {

                      fill: "#ccccff",

                      opacity: 1,

                      border: {

                          color: "#9999ff",

                          width: 1
                      }
                  },

                  fill: "#00008B",

                  visible: true,

                  font: {

                      fontFamily: 'Segoe UI',

                      fontStyle: 'Normal',

                      fontWeight: 'Regular',

                      size: '12px',

                      color: '#707070',

                      opacity: 1
                  },

                  width: 2,

                  xAxisName: "",

                  yAxisName: "",

                  macdType: "line",

                  shortPeriod: 12,

                  longPeriod: 26,

                  trigger: 9,

                  visibility: "visible",

                  enableAnimation: false
              }
            ],


            backGroundImageUrl: null,

            annotations: [{

                visible: false,

                content: "",

                coordinateUnit: "none",

                verticalAlignment: "middle",

                horizontalAlignment: "middle",

                region: "chart",

                x: 0,

                y: 0,

                opacity: 1,

                angle: 0,

                xAxisName: "",

                yAxisName: "",

                margin: {

                    left: 0,

                    right: 0,

                    bottom: 0,

                    top: 0
                }
            }],

            series: [
                {
                dragSettings: {
                       enable: false,
                       type: 'xy'                      
                   },

                    type: 'column',

					splitMode: 'value',

					splitValue: null,

					gapWidth: 50,

					pieOfPieCoefficient: 0.6,
					
					visibleOnLegend:"visible",

                    enableAnimation: false,

                    isClosed: true,

					 isTransposed:false,

                    isStacking: true,

					columnSpacing: 0,

                    columnWidth: 0.7,

                    columnFacet:'rectangle',
                    drawType: 'line',

                    doughnutCoefficient: 0.4,

                    explodeOffset: 25,

                    pyramidMode: 'linear',
					
					boxPlotMode : 'exclusive',
										
                    funnelWidth: '11.6%',

                    funnelHeight: '32.7%',

                    positiveFill: null,
					
					showMedian:true,
					
					outlierSettings:{
						
						shape : 'circle',
						
						size:
						{
							width:6,
							
							height:6
						
						},
				},
					
					cornerRadius: {
					
						topLeft: 0,
					
						topRight: 0,
					
						bottomLeft: 0,
					
						bottomRight: 0
					},

                    connectorLine: {

                        width: 1,

                        opacity: 1,

                        dashArray: null
                    },
					
					 errorBar: {

                        visibility: "hidden",

                        mode: "vertical",

                        type:"fixedValue",
                        
                        horizontalErrorValue: 1,

                        verticalErrorValue: 3,

                        horizontalPositiveErrorValue: 1,

                        horizontalNegativeErrorValue: 1,

                        verticalPositiveErrorValue: 5,

                        verticalNegativeErrorValue: 5,

                        direction: "both",

                        cap:{
                            
                            visible:true,
                            
                            width: 1,

                            length: 20,

                        },


                        width: 1,

                        opacity: 1,

                    },

                    highlightSettings: {

                        enable: false,

                        mode: 'series',

                        color: "",

                        opacity: 0.6,

                        border: {

                            color: "",

                            width: 2
                        },

                        pattern: 'none',

                        customPattern: ""
                    },

                    selectionSettings: {

                        type:'Single',

                        enable: false,

                        mode: 'series',

                        rangeType: 'xy',

                        color: "",

                        opacity: 0.6,

                        border: {

                            color: "",

                            width: 2
                        },

                        pattern: 'none',

                        customPattern: ""
                    },
					
					emptyPointSettings: {

                        visible: true,

                        style: {

                            color: '',

                            border: {

                                color: '',

                                width: 1

                            }
                        
                        },

                        displayMode: 'gap'
                    },
					
                    labelPosition: 'inside',

                    gapRatio: 0,

                    points: null,

                    pieCoefficient: 0.8,

                    doughnutSize: 0.8,

                    dataSource: null,

                    high: '',

                    low: '',

                    open: '',

                    close: '',

                    bullFillColor: null,

                    bearFillColor: null,

                    query: null,

                    xName: '',

                    yName: '',

                    pointColorMappingName: '',



                    visibility: "visible",
					
                    startAngle: null,

                    endAngle: null,

                    xAxisName: null,

                    yAxisName: null,

                    explodeAll: null,

                    explode: false,

                    explodeIndex: null,

                    enableSmartLabels: null,

                    tooltip: {

                        visible: false,

                        format: null,

                        template: null,


                        fill: null,

                        border: {

                            width: 1,

                            color: null
                        },

                        enableAnimation: true,

                        duration: '500ms',

                        opacity: 0.95
                    },

                    fill: null,

                    opacity: 1,

                    lineCap: 'butt',

                    lineJoin: 'round',

                    dashArray: "",

                    border: {

                        width: 1,

                        color: 'transparent',

                        dashArray:""
                    },
                    width: 2,

                    marker: {

                        shape: 'circle',

                        imageUrl: "",

                        size: {

                            width: 6,

                            height: 6
                        },

                        visible: false,

                        opacity: 1,

                        fill: null,

                        border: {

                            color: 'white',

                            width: 3
                        },

                        dataLabel: {

                            visible: false,

                            angle: 0,

                            textMappingName:'',

                            verticalTextAlignment: 'center',

                            horizontalTextAlignment: 'center',

                            textPosition: 'top',

                            shape: 'none',

                            opacity: 1,

                            fill: null,

                            border: {

                                color: 'white',

                                width: 0.1
                            },

                            maximumLabelWidth: null,

                            enableWrap:false,

                            connectorLine: {

                                width: 0.5,

                                type: 'line',

                                color: null
                            },

                            offset: {

                                x: 0,

                                y:0
                            },

                            font: {

                                fontFamily: 'Segoe UI',

                                fontStyle: 'Normal',

                                fontWeight: 'Regular',

                                size: '11px',

                                opacity: 1
                            },

                            margin: {

                                left: 5,

                                top: 5,

                                bottom: 5,

                                right: 5
                            }

                        }
                    },

					bubbleOptions: {
					
						radiusMode: "minmax",
					
						minRadius: 1,
						
						maxRadius: 3
						
					},
					
                    trendlines: [{

                        visibility: "",
						
						visibleOnLegend: "visible",

                        type: "linear",

                        name: "Trendline",

                        fill: "",

                        width: 1,

                        opacity: 1,

                        dashArray: "",

                        forwardForecast: 0,

                        backwardForecast: 0,

                        polynomialOrder: 2,

                        period: 2,
						
						intercept: null,

                        tooltip:
                 {

                     visible: false,

                     format: null,

                     fill: null,

                     border: {

                         width: 1,

                         color: null
                     },

                     duration: '500ms',

                     enableAnimation: true,

                     opacity: 0.95,

                     font: {

                         fontFamily: 'Segoe UI',

                         fontStyle: 'Normal',

                         fontWeight: 'Regular',

                         size: '12px',

                         opacity: 1
                     }
                 }
                    }],


                    font: {

                        fontFamily: 'Segoe UI',

                        fontStyle: 'Normal',

                        fontWeight: 'Regular',

                        size: '12px',

                        color: '#707070',

                        opacity: 1
                    },
                    _isdesigntime: true

                }
            ],

            commonSeriesOptions: {
                dragSettings: {
                           enable: false,
                           type: 'xy'                      
                       },

                type: 'column',

                splitMode: 'value',

                splitValue: null,

                gapWidth: 50,

                pieOfPieCoefficient: 0.6,
				
				visibleOnLegend: "visible",

                enableAnimation: false,

                isClosed: true,

				 isTransposed:false,

                isStacking: false,

				columnWidth: 0.7,

                columnSpacing: 0,

                columnFacet: 'rectangle',

                drawType: 'line',

                doughnutCoefficient: 0.4,

                explodeOffset: 25,

                pyramidMode: 'linear',
				
				boxPlotMode : 'exclusive',
								
                funnelWidth: '11.6%',

                funnelHeight: '32.7%',

                positiveFill: null,
				
				showMedian:true,
				
				outlierSettings:{
						
						shape : 'circle',
						
						size:
						{
							width:6,
							
							height:6
						
						},
				},
				
				cornerRadius: {
					
					topLeft: 0,
					
					topRight: 0,
					
					bottomLeft: 0,
					
					bottomRight: 0
				},

                connectorLine: {

                    width: 1,

                    opacity: 1,

                    dashArray: null
                },

				 errorBar: {

                        visibility: "hidden",

                        mode: "vertical",

                        type:"fixedValue",
                        
                        horizontalErrorValue: 1,

                        verticalErrorValue: 3,

                        horizontalPositiveErrorValue: 1,

                        horizontalNegativeErrorValue: 1,

                        verticalPositiveErrorValue: 5,

                        verticalNegativeErrorValue: 5,

                        direction: "both",

                        cap:{
                            
                            visible:true,
                            
                            width: 1,

                            length: 20,

                        },


                        width: 1,

                        opacity: 1,

                    },
				
                highlightSettings: {

                    enable: false,

                    mode: 'series',

                    color: "",

                    opacity: 0.6,

                    border: {

                        color: "",

                        width: 2
                    },

                    pattern: 'none',

                    customPattern: ""
                },

                selectionSettings: {

                    type:'Single',

                    enable: false,

                    mode: 'series',

                    rangeType: 'xy',

                    color: "",

                    opacity: 0.6,

                    border: {

                        color: "",

                        width: 2
                    },

                    pattern: 'none',

                    customPattern: ""
                },
				
				emptyPointSettings: {

                        visible: true,

                        style: {

                            color: '',

                            border: {

                                color: '',

                                width: 1

                            }
                        
                        },

                        displayMode: 'gap'
                },

                labelPosition: 'inside',

                gapRatio: 0,

                pieCoefficient: 0.8,

                doughnutSize: 0.8,

                dataSource: null,

                xName: '',

                yName: '',

                pointColorMappingName:'',

                high: '',

                low: '',

                open: '',

                close: '',

                startAngle: null,

                endAngle: null,

                xAxisName: null,

                yAxisName: null,

                explodeAll: false,

                explode: false,

                explodeIndex: null,

                enableSmartLabels: null,

                tooltip: {

                    visible: false,

                    format: null,

                    template: null,

                    fill: null,

                    border: {

                        color: null,

                        width: 1
                    },

                    enableAnimation: true,

                    duration: '500ms',

                    opacity: 0.95
                },

                fill: null,

                opacity: 1,

                lineCap: 'butt',

                lineJoin: 'round',

                dashArray: "",

                border: {

                    color: 'transparent',

                    width: 1,

                    dashArray: ""
                },
                width: 2,

                marker:
                {

                    shape: 'circle',

                    imageUrl: "",

                    size: {

                        width: 6,

                        height: 6
                    },

                    visible: false,

                    opacity: 1,

                    fill: null,

                    border: {

                        color: 'white',

                        width: 3
                    },

                    dataLabel:
                    {

                        visible: false,

                        angle: 0,

                        textMappingName:'',

                        verticalTextAlignment: 'center',

                        horizontalTextAlignment: 'center',

                        textPosition: 'top',

                        shape: 'none',

                        maximumLabelWidth: null,

                        enableWrap:false,

                        opacity: 1,

                        fill: null,

                        border: {

                            color: 'white',

                            width: 0.1
                        },

                        offset: {

                            x: 0,

                            y: 0
                        },

                        connectorLine:
                        {

                            width: 0.5,

                            type: 'line'
                        },

                        font:
                        {

                            fontFamily: 'Segoe UI',

                            fontStyle: 'Normal',

                            fontWeight: 'Regular',

                            size: '11px',

                            opacity: 1
                        },

                        margin:
                        {

                            left: 5,

                            top: 5,

                            bottom: 5,

                            right: 5
                        }
                    }
                },
				
				bubbleOptions: {
				
					radiusMode: "minmax",
					
					minRadius: 1,
						
					maxRadius: 3
						
				},
					
                trendlines: [{

                    visibility: "",
					
					visibleOnLegend: "visible",

                    type: "linear",

                    name: "Trendline",

                    fill: "",

                    width: 1,

                    opacity: 1,

                    dashArray: "",

                    forwardForecast: 0,

                    backwardForecast: 0,

                    polynomialOrder: 2,

                    period: 2,
					
					intercept: null,

                    tooltip:
                 {

                     visible: false,

                     format: null,

                     fill: null,

                     border: {

                         width: 1,

                         color: null
                     },

                     duration: '500ms',

                     enableAnimation: true,

                     opacity: 0.95,

                     font: {

                         fontFamily: 'Segoe UI',

                         fontStyle: 'Normal',

                         fontWeight: 'Regular',

                         size: '12px',

                         opacity: 1
                     }
                 }
                }],

                font: {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px',

                    color: '#707070',

                    opacity: 1
                }
            },

            crosshair:
            {
				line:{
					
					color: 'black',
					
					width: 1
				},
                marker:
                {

                    visible: true,

                    size:
                    {

                        width: 10,

                        height: 10
                    },

                    opacity: 1,

                    border:
                    {

                        width: 3
                    }
                },

                visible: false,

                type: "crosshair",

                trackballTooltipSettings: {

                    fill: null,

                    mode:"float",

                    border: {

                        width: null,

                        color: null,

                    },

                    rx: 3,

                    ry: 3,

                    opacity:1,
                }
            },


            pointStyle:
            {
                lineCap: 'butt', lineJoin: 'round', opacity: 1, interior: null, borderColor: null, borderWidth: 1
            },
            textStyle:
            {
                marker: { textAlignment: 'center', textPosition: 'top' },
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 }
            },
            symbolShape: {
                None: 0,
                LeftArrow: 1,
                RightArrow: 2,
                Circle: 3,
                Cross: 4,
                HorizLine: 5,
                VertLine: 6,
                Diamond: 7,
                Rectangle: 8,
                Triangle: 9,
                InvertedTriangle: 10,
                Hexagon: 11,
                Pentagon: 12,
                Star: 13,
                Ellipse: 14,
                Wedge: 15,
                Trapezoid: 16,
                UpArrow: 17,
                DownArrow: 18,
                Image: 19,
                SeriesType: 20

            },
            initSeriesRender: true,

            theme: "flatlight",

            canResize: false,

            isResponsive: false,

            enable3D: false,

            enableRotation: false,

            sideBySideSeriesPlacement: null,

            perspectiveAngle: 90,

            rotation: 0,

            tilt: 0,

            wallSize: 2,

            depth: 100,

            enableCanvasRendering: false,
			
			selectedDataPointIndexes: [],

			exportSettings: {

                type: "png",

                fileName: "Chart",

                orientation: "portrait",

                angle: 0,

                mode: "client",

                action: "",
				
				multipleExport: false
            },
			
            zooming:
            {

                enable: false,

                enableScrollbar : false,

                type: 'x,y',

                enablePinching: true,

                enableMouseWheel: false,

                enableDeferredZoom: false,

                toolbarItems: ["zoomIn", "zoomOut", "zoom", "pan", "reset"]
            },
            type: 'x,y',

            locale: null,

            xZoomFactor: 1,

            yZoomFactor: 1,

            xZoomPosition: 0,

            yZoomPosition: 0,

            load: "",

            axesLabelRendering: "",

            axesRangeCalculate: "",

            axesTitleRendering: "",

            dragStart: "",
            
            dragging: "",
            
            dragEnd:"",

            chartAreaBoundsCalculate: "",

            legendItemRendering: "",

            legendBoundsCalculate: "",

            preRender: "",

            seriesRendering: "",


            trendlineRendering: "",

            symbolRendering: "",

            titleRendering: "",
            subtitleRendering: "",

            axesLabelsInitialize: "",

            pointRegionClick: "",

            seriesRegionClick: "",

            annotationClick:"",
 
			 axisLabelClick:"",

			 chartClick: "",

			 scrollStart: "",

			 scrollEnd: "",

            scrollChanged: "",

            chartDoubleClick:"",

            pointRegionMouseMove: "",

            legendItemClick: "",

            axisLabelMouseMove: "",

            chartMouseMove: "",

            legendItemMouseMove: "",

            chartMouseLeave: "",

            displayTextRendering: "",

            toolTipInitialize: "",

            trackAxisToolTip: "",

            trackToolTip: "",

            animationComplete: "",

            zoomed:"",

            destroy: "",

            create: "",

            beforeResize: "",

            afterResize: "",

            rangeSelected: "",
			
			multiLevelLabelClick: "",
			
			multiLevelLabelRendering: "",

            margin: { left: 10, right: 10, top: 10, bottom: 10 },
            size: { width: null, height: null },
            elementSpacing: 10,

            dateStart: 25568.791666666668

        },

        dataTypes: {
            commonSeriesOptions: { dataSource: "data" },
            axes: "array",
            series: "array",
            annotations: "array",
            indicators: "array",
            seriesColors: "array",
            palette: "array",
            rowDefinitions: "array",
            columnDefinitions: "array",
            seriesBorderColors: "array",
            pointColors: "array",
            pointBorderColors: "array",
            initSeriesRender: "boolean",
            theme: "enum",
            canResize: "boolean",
            isResponsive:"boolean",
            elementSpacing: "number",
            primaryXAxis: {
                labels: "array",
                multiLevelLabels: "array",
                stripLine: "array",
                orientation: "enum",
                rangePadding: "enum",
                labelPlacement: "enum",
                tickLinesPosition: "enum",
                labelPosition: "enum",
                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number",                 
                showNextToAxisLine: "boolean"
            },
            primaryYAxis: {
                labels: "array",
                multiLevelLabels: "array",
                stripLine: "array",
                orientation: "enum",
                rangePadding: "enum",
                labelPlacement: "enum",
                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"               
            },
            legend: {
                textOverflow:"enum",
                shape: "enum",
                alignment: "enum",
                position: "enum",
                itemPadding: "number"
            },
			exportSettings: {
                type: "string",
                fileName: "string",
                orientation: "enum",
                angle: "number",
                mode: "enum",
                action: "string",
				multipleExport: "boolean"
            },
            zooming: {
                enable: "boolean",
                type: "string",
                enableMouseWheel: "boolean",
                toolbarItems: "array",
                enableScrollbar: "boolean"
            },
            size: {
                width: "string",
                height: "string"
            }

        },

        observables: ["xZoomFactor", "yZoomFactor", "xZoomPosition", "yZoomPosition"],

        _tags: [{
            tag: "series",
            attr: ["xAxisName", "yAxisName", "zOrder", "endAngle", "startAngle", "explodeIndex", "labelPosition", "xName", "yName","pointColorMappingName", "pyramidMode","boxPlotMode","showMedian",
                   "pieCoefficient", "explodeAll", "explodeOffset", "funnelWidth","columnFacet", "funnelHeight", "gapRatio", "isClosed","isTransposed", 
                   "isStacking", "bearFillColor", "bullFillColor", "dataSource", "enableAnimation", "doughnutCoefficient", "doughnutSize",
                   "enableSmartLabels", "drawType", "dashArray", "visibleOnLegend", "columnSpacing", "columnWidth", "drawType", "positiveFill", "explodeIndex", "cornerRadius",
                   "lineCap", "lineJoin", "highlightSettings.enable", "highlightSettings.mode", "highlightSettings.pattern", "highlightSettings.color", "highlightSettings.opacity",
                   "highlightSettings.customPattern", "highlightSettings.border.color", "highlightSettings.border.width", "selectionSettings.enable", "selectionSettings.mode", "selectionSettings.pattern",
                   "selectionSettings.color", "selectionSettings.opacity", "selectionSettings.customPattern", "selectionSettings.type", "selectionSettings.rangeType",
                   "selectionSettings.border.width", "selectionSettings.border.color", "dragSettings.enable", "dragSettings.type",
                   "connectorLine.width", "connectorLine.opacity", "connectorLine.dashArray", "cornerRadius.topLeft", "cornerRadius.topRight", "cornerRadius.bottomLeft", "cornerRadius.bottomRight",
                   "errorBar.visibility", "errorBar.mode", "errorBar.type", "errorBar.horizontalErrorValue", "errorBar.verticalErrorValue", "errorBar.horizontalPositiveErrorValue",
                   "errorBar.horizontalNegativeErrorValue", "errorBar.verticalPositiveErrorValue", "errorBar.verticalNegativeErrorValue", "errorBar.direction",
                   "errorBar.cap.visible", "errorBar.cap.width", "errorBar.cap.length", "errorBar.width", "errorBar.opacity", "emptyPointSettings.visible",
                   "emptyPointSettings.style.color", "emptyPointSettings.style.border.color", "emptyPointSettings.style.border.width", "emptyPointSettings.displayMode",
				   "bubbleOptions.minRadius", "bubbleOptions.maxRadius", "bubbleOptions.radiusMode",
                   "tooltip.visible", "tooltip.format", "tooltip.template", "tooltip.fill", "tooltip.border.width", "tooltip.border.color", "tooltip.enableAnimation", "tooltip.duration",
                   "tooltip.opacity", "tooltip.font.size", "tooltip.font.opacity", "tooltip.font.fontFamily", "tooltip.font.fontStyle", "tooltip.font.fontWeight", "tooltip.font.color", "border.width", "border.color", "border.dashArray", "marker.shape", "marker.imageUrl", "marker.size.width", "marker.size.height",
                   "marker.visible", "marker.opacity", "marker.fill", "marker.border.color", "marker.border.width", "marker.dataLabel.visible", "marker.dataLabel.angle",
                   "marker.dataLabel.template", "marker.dataLabel.textMappingName", "marker.dataLabel.verticalTextAlignment", "marker.dataLabel.horizontalTextAlignment", "marker.dataLabel.textPosition", "marker.dataLabel.shape",
                   "marker.dataLabel.opacity", "marker.dataLabel.fill", "marker.dataLabel.border.color", "marker.dataLabel.border.width", "marker.dataLabel.maximumLabelWidth", "marker.dataLabel.enableWrap",
                   "marker.dataLabel.connectorLine.width", "marker.dataLabel.connectorLine.type", "marker.dataLabel.connectorLine.color", "marker.dataLabel.offset", "marker.dataLabel.offset.x", "marker.dataLabel.offset.y",
                   "marker.dataLabel.font.size", "marker.dataLabel.font.color", "marker.dataLabel.font.opacity", "marker.dataLabel.font.fontFamily", "marker.dataLabel.font.fontStyle", "marker.dataLabel.font.fontWeight",
                   "marker.dataLabel.margin", "marker.dataLabel.margin.left", "marker.dataLabel.margin.top", "marker.dataLabel.margin.bottom", "marker.dataLabel.margin.right",
                   "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "font.color",
                   [
				     {
				         tag: "trendlines", attr: ["visibility", "visibilityOnLegend", "dashArray", "forewardForecast", "backwardForecast", "polynomialOrder",
                         "tooltip.visible", "tooltip.format", "tooltip.fill", "tooltip.border.width", "tooltip.border.color", "tooltip.enableAnimation", "tooltip.duration",
                        "tooltip.opacity", "tooltip.font.size", "tooltip.font.opacity", "tooltip.font.fontFamily", "tooltip.font.fontStyle", "tooltip.font.fontWeight", "tooltip.font.color",
				         ], singular: "trendline"
				     }
                   ],
			       [
				     {
				         tag: "points", attr: ["x", "y", "text", "textMappingName", "isEmpty", "fill", "visible"], singular: "points"
				     }
			       ]

            ],
            singular: "series"
        }, {
            tag: "axes",
            attr: ["columnIndex", "rowIndex", "desiredIntervals", "isIndexed", "labelPlacement", "edgeLabelPlacement", "intervalType", "labelFormat", "labelPosition", "labelRotation", "logBase",
                "labelIntersectAction", "opposedPosition", "plotOffset", "rangePadding", "roundingPlaces", "valueType", "zoomFactor", "zoomPosition", "orientation",
                "crossesAt", "crossesInAxis", "scrollbarSettings.visible", "scrollbarSettings.canResize", "scrollbarSettings.range.min", "scrollbarSettings.range.max", "scrollbarSettings.pointsLength",
                "range.min", "range.max", "range.interval", "tickLinesPosition", "alternateGridBand.odd.fill", "alternateGridBand.odd.opacity", "alternateGridBand.even.fill", "alternateGridBand.even.opacity",
                "enableAutoIntervalOnZooming", "majorGridLines.width", "majorGridLines.dashArray", "majorGridLines.visible", "majorGridLines.opacity", "majorTickLines.width", "majorTickLines.size", "majorTickLines.visible",
                "minorGridLines.width", "minorGridLines.dashArray", "minorGridLines.visible", "minorTickLines.width", "minorTickLines.size", "minorTickLines.visible", "labelBorder.width", "minorTicksPerInterval",
                "columnSpan", "crosshairLabel.visible", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "isInversed", "enableTrim", "maximumLabelWidth",
                "axisLine.visible", "axisLine.width", "axisLine.dashArray", "axisLine.offset", "maximumLabels", "title.text", "title.visible", "title.enableTrim", "title.offset",
                "title.alignment", "title.position", "title.maximumTitleWidth", "title.font.size", "title.font.opacity", "title.font.fontFamily", "title.font.fontStyle", "title.font.fontWeight",
                "additionalPadding", "showNextToAxisLine",
              [
                {
                    tag: "stripLine", attr: ["borderColor", "startFromAxis", "zIndex", "textAlignment", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "font.color"],
                    singular: "stripLine"
                }
              ],
               [
                {
                    tag: "multiLevelLabels", attr: ["textOverflow", "maximumTextWidth", "textAlignment", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "border.type", "border.width"],
                    singular: "multiLevelLabel"
                }
               ],
            ],
            singular: "axis"
        },
              {
                  tag: "primaryXAxis.stripLine",
                  attr: ["borderColor", "startFromAxis", "zIndex", "textAlignment", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "font.color"],
                  singular: "primaryXAxis.stripLine"
              },
            {
                tag: "primaryYAxis.stripLine",
                attr: ["borderColor", "startFromAxis", "zIndex", "textAlignment", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "font.color"],
                singular: "primaryYAxis.stripLine"
            },
             {
                 tag: "primaryXAxis.multiLevelLabels",
                 attr: ["textOverflow", "maximumTextWidth", "textAlignment", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "border.type", "border.width"],
                 singular: "primaryXAxis.multiLevelLabel"
             },
              {
                  tag: "primaryYAxis.multiLevelLabels",
                  attr: ["textOverflow", "maximumTextWidth", "textAlignment", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "border.type", "border.width"],
                  singular: "primaryYAxis.multiLevelLabel"
              },
           {
               tag: "indicators",
               attr: ["seriesName", "xName", "xAxisName", "macdType", "shortPeriod", "longPeriod", "enableAnimation",
                   "yAxisName", "standardDeviations", "kPeriod", "dPeriod", "periodLine.fill", "periodLine.width", "lowerLine.fill",
                   "lowerLine.width", "upperLine.fill", "upperLine.width", "macdLine.fill", "macdLine.width", "histogram.fill", "histogram.opacity",
                   "histogram.border.color", "histogram.border.width", "font.size", "font.opacity", "font.fontFamily", "font.fontStyle", "font.fontWeight", "font.color",
                   "tooltip.visible", "tooltip.format", "tooltip.fill", "tooltip.border.width", "tooltip.border.color", "tooltip.enableAnimation", "tooltip.duration",
                   "tooltip.opacity",
                  [
                    {
                        tag: "points", attr: []
                    }
                  ]
               ],
               singular: "indicator"
           },
          {
              tag: "annotations",
              attr: ["coordinateUnit", "verticalAlignment", "horizontalAlignment", "xAxisName", "yAxisName", "margin.left", "margin.right", "margin.top", "margin.bottom"],
              singular: "annotation"
          },
          {
              tag: "rowDefinitions",
              attr: ["rowHeight", "lineColor", "lineWidth"],
              singular: "rowDefinition"
          },
          {
              tag: "columnDefinitions",
              attr: ["columnWidth", "lineColor", "lineWidth"],
              singular: "columnDefinition"
          },
        ],
        _xZoomFactor: ej.util.valueFunction("xZoomFactor"),
        _yZoomFactor: ej.util.valueFunction("yZoomFactor"),
        _xZoomPosition: ej.util.valueFunction("xZoomPosition"),
        _yZoomPosition: ej.util.valueFunction("yZoomPosition"),


        _init: function () {

            this._renderSfChart();

        },
        _destroy: function () {
            $('#template_group_' + this._id).remove();
            $("#annotation_group_" + this._id).remove();
            $(this.element).removeClass("e-chart e-js").find("#" + this.svgObject.id).remove();
            $(this.element).removeClass("e-chart e-js").find("#legend_" + this.svgObject.id).remove();
        },

        _series: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _series_points: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _primaryXAxis_stripLine: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _primaryYAxis_stripLine: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _axes: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _axes_stripLine: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },

        _ignoreOnExport: ["dataSource"],


        _removeZoomkit: function () {
            if (this.model.zooming.enable) {
                $("[id^=" + this._id + "_][id$=" + "_ResetZoom" + "]").remove();
                $("[id^=" + this._id + "_][id$=" + "_PanBtn" + "]").remove();
                $("[id^=" + this._id + "_][id$=" + "_ZoomBtn" + "]").remove();
                $("[id^=" + this._id + "_][id$=" + "_ZoomInBtn" + "]").remove();
                $("[id^=" + this._id + "_][id$=" + "_ZoomOutBtn" + "]").remove();
            }
        },
		
		//to create a new canvas element to export and print
		getCanvasElement: function(id){
			 var chart,
			 printId = $("#" + id),
			 exportPrintCanvas = document.createElement('canvas'),
			 expCtx = exportPrintCanvas.getContext("2d"), 
			 chartWidth = $("#" + id + "_canvas").width(),
             chartHeight = $("#" +id + "_canvas").height();
			 exportPrintCanvas.setAttribute('width', chartWidth);
             exportPrintCanvas.setAttribute('height', chartHeight);
			 chart = $("#" + id + '_canvas')[0]; 
			 expCtx.drawImage(chart,0,0,chartWidth,chartHeight); 
				
			return { canvasContainer : exportPrintCanvas , canvasArea : expCtx ,width : chartWidth ,height : chartHeight  }; 
			
		},
		// export and print the svg element
		getSVGElement: function(id){
		    var chartWidthSVG = $("#" + id + "_svg").width(),
			chartHeightSVG = $("#" + id + "_svg").height(),
			svgChart = document.getElementById( id + "_svg"),
			containerSVG =  $('<div>').append($(svgChart).clone()).html(),
			legendChart = document.getElementById("legend_" + id + "_svg"),	
			position = $("#legend_" + id).position(),
			element = $("#" + id + "_svg_Legend").attr('transform', "translate(" + position.left + "," + position.top + ")"),
			containerLegendSVG =  $('<div>').append($(legendChart).clone()).html();
		
			return{chartSVG : containerSVG ,legendChartSVG : containerLegendSVG ,width : chartWidthSVG ,height : chartHeightSVG };
		},
		
		print: function () {		    
			//declaration
		    var contentSVG, Id, svgID, canvasElement, chartObj, printId, printCtxAnnot, legend, selectionChart, selectionLegendChart,
			selectionChartSVG, legendChartSVG, annotationChartCanvas, canvasChart = [], writeCode = [], containerselectionChartSVG,
			containerSelectionLegendSVG, legendPos, annotationChartSVG, containerAnnotationChartSVG,
			serIndex = this.model.series.length, scrollbar, TDStyle = "", oImg = "", chartHeight = 0, chartDistance = 50;
			for(var i=0;i<arguments.length;i++) {
			
				printId = $("#" + arguments[i]);
				Id = arguments[i];

				//to check whether the div contains chart element
				if (printId.hasClass("e-datavisualization-chart")) {
				    chartObj = printId.ejChart("instance");
				    contentSVG = "<html> <div style='position:relative; left:0px; top:" + chartHeight + "px'>";
				    scrollbar = "";
				    svgID = chartObj.svgObject.id;
				    
				    chartHeight = $("#" + svgID).height() + chartDistance;

				    if ($("#axisScrollbar_" + Id)[0])
				        scrollbar = $("#axisScrollbar_" + Id)[0].innerHTML;

				    var zoomButton = ["ResetZoom", "PanBtn", "ZoomBtn", "ZoomOutBtn", "ZoomInBtn"], zoomBtnObj = [], zoomToolbarID, btnCount = 5;
				    for (var l = 0; l < btnCount; l++) {
				        zoomToolbarID = "#" + svgID + "_" + zoomButton[l];
				        if ($(zoomToolbarID)[0]) {
				            zoomBtnObj.push($(zoomToolbarID));
				            //Remove zoomButtons on printing
				            $(zoomToolbarID).remove();
				        }
				    }

				    if (chartObj.model.enableCanvasRendering == true) { //to check the chart to render in canvas
				        canvasElement = this.getCanvasElement(Id);
				        
				        //to find the lengend position  
				        legendPos = chartObj.model.LegendBounds;

				        oImg = document.createElement("img");
				        oImg.setAttribute('src', $('#legend_' + Id + '_canvas')[0].toDataURL());
				        $("#legend_Scroller" + Id).prepend(oImg);
				        contentSVG += document.getElementById('chartContainer_' + Id).outerHTML;
				        $(oImg).remove();
				        oImg = "";

				        var mode;
				        //selection chart to print in canvas
				        for (var index = 0; index < serIndex; index++) {
				            mode = chartObj.model.series[index].selectionSettings.mode.toLowerCase();
				            if (chartObj.model.AreaType != "none") {
				                if (mode == "series") {
				                    if ($(printId).find("#" + Id + "_Selection_series" + index + "_canvas").length > 0) {
				                        selectionChart = $("#" + Id + "_Selection_series" + index + "_canvas")[0];
				                        canvasElement.canvasArea.drawImage(selectionChart, 0, 0, canvasElement.width, canvasElement.height);
				                    }
				                    if ($(printId).find("#" + Id + "_Selection_Legend" + index + "_canvas").length > 0) {
				                        selectionLegendChart = $("#" + Id + "_Selection_Legend" + index + "_canvas")[0];
				                        oImg = "<img style='position:absolute;left:0px;' src='" + selectionLegendChart.toDataURL() + "'/>";
				                    }
				                } else if (mode == "point") {
				                    var selectedPoints = chartObj.model.selectedDataPointIndexes;
				                    for (var l = 0; l < selectedPoints.length; l++) {
				                        if (selectedPoints[l].seriesIndex == index) {
				                            if ($(printId).find("#" + Id + "_Selection_series" + index + "_point_" + selectedPoints[l].pointIndex + "_canvas").length > 0) {
				                                selectionChart = $("#" + Id + "_Selection_series" + index + "_point_" + selectedPoints[l].pointIndex + "_canvas")[0];
				                                canvasElement.canvasArea.drawImage(selectionChart, 0, 0, canvasElement.width, canvasElement.height);
				                            }
				                        }
				                    }
				                    if ($(printId).find("#" + Id + "_Selection_Legend" + index + "_canvas").length > 0) {
				                        selectionLegendChart = $("#" + Id + "_Selection_Legend" + index + "_canvas")[0];
				                        oImg = "<img style='position:absolute;left:0px;' src='" + selectionLegendChart.toDataURL() + "'/>";
				                    }
				                } else {
				                    var selectedPoints = chartObj.model.selectedDataPointIndexes;
				                    for (var l = 0; l < selectedPoints.length; l++) {
				                        if (selectedPoints[l].seriesIndex == index) {
				                            if ($(printId).find("#" + Id + "_Selection_Cluster_point_" + selectedPoints[l].pointIndex + "_canvas").length > 0) {
				                                selectionChart = $("#" + Id + "_Selection_Cluster_point_" + selectedPoints[l].pointIndex + "_canvas")[0];
				                                canvasElement.canvasArea.drawImage(selectionChart, 0, 0, canvasElement.width, canvasElement.height);
				                            }
				                        }
				                    }
				                    if ($(printId).find("#" + Id + "_SelectionCluster_Legend" + index + "_canvas").length > 0) {
				                        selectionLegendChart = $("#" + Id + "_SelectionCluster_Legend" + index + "_canvas")[0];
				                        oImg = "<img style='position:absolute;left:0px;' src='" + selectionLegendChart.toDataURL() + "'/>";
				                    }
				                }
				            } else {
				                var selectedPoints = chartObj.model.selectedDataPointIndexes;
				                for (var l = 0; l < selectedPoints.length; l++) {
				                    if (selectedPoints[l].seriesIndex == index) {
				                        if ($('#' + Id + '_Selection_series' + index + "_point_" + selectedPoints[l].pointIndex + "_canvas").length > 0) {
				                            selectionChart = document.getElementById(Id + '_Selection_series' + index + "_point_" + selectedPoints[l].pointIndex + "_canvas");
				                            canvasElement.canvasArea.drawImage(selectionChart, 0, 0, canvasElement.width, canvasElement.height);
				                            if ($('#' +Id + "_Selection_Legend" + selectedPoints[l].pointIndex + "_canvas").length > 0) {
				                                selectionLegendChart = document.getElementById(Id + "_Selection_Legend" + selectedPoints[l].pointIndex + "_canvas");
				                                oImg = "<img style='position:absolute;left:0px;' src='" + selectionLegendChart.toDataURL() + "'/>";
				                            }
				                        }
				                    }
				                }
				            }
				        }
				        canvasElement.canvasArea.save();
				        //to store the chart canvas element
				        canvasChart.push(canvasElement.canvasContainer);
				    }
				        //to check the chart to render in svg
				    else {
				        var printSvgElement = this.getSVGElement(Id);

				        $("#" + Id + "_svg_Legend").removeAttr("transform");
				        contentSVG += $("#chartContainer_" + Id)[0].outerHTML;

				        //selection chart to print in svg
				        if (!chartObj.model.enable3D) {
				            var mode;
				            for (var index = 0; index < serIndex; index++) {
				                mode = chartObj.model.series[index].selectionSettings.mode.toLowerCase();
				                if (chartObj.model.AreaType != "none") {
				                    if (mode != "cluster") {
				                        if ($('#' + Id + 'SelectionSegmentseries' + index).length > 0) {
				                            selectionChartSVG = document.getElementById(Id + "SelectionSegmentseries" + index);
				                            containerselectionChartSVG = $('<div>').append($(selectionChartSVG).clone()).html();
				                            legendChartSVG = document.getElementById(Id + "SelectionLegendSegmentseries" + index);
				                            containerSelectionLegendSVG = $('<div>').append($(legendChartSVG).clone()).html();
				                            contentSVG += containerselectionChartSVG.toString() + containerSelectionLegendSVG.toString();
				                        }
				                    } else {
				                        if ($('#' + Id + 'SelectionSegmentClusterSeries' + index).length > 0) {
				                            selectionChartSVG = document.getElementById(Id + "SelectionSegmentClusterSeries" + index);
				                            containerselectionChartSVG = $('<div>').append($(selectionChartSVG).clone()).html();
				                            legendChartSVG = document.getElementById(Id + "SelectionLegendSegmentClusterSeries" + index);
				                            containerSelectionLegendSVG = $('<div>').append($(legendChartSVG).clone()).html();
				                            contentSVG += containerselectionChartSVG.toString() + containerSelectionLegendSVG.toString();
				                        }
				                    }
				                } else {
				                    var selectedPoints = chartObj.model.selectedDataPointIndexes;
				                    for (var l = 0; l < selectedPoints.length; l++) {
				                        if (selectedPoints[l].seriesIndex == index) {
				                            if ($('#' + Id + 'SelectionSegmentseries' + index + "Point" + selectedPoints[l].pointIndex).length > 0) {
				                                selectionChartSVG = document.getElementById(Id + 'SelectionSegmentseries' + index + "Point" + selectedPoints[l].pointIndex);
				                                containerselectionChartSVG = $('<div>').append($(selectionChartSVG).clone()).html();
				                                legendChartSVG = document.getElementById(Id + "SelectionLegendSegmentseries" + index + "Point" + selectedPoints[l].pointIndex);
				                                containerSelectionLegendSVG = $('<div>').append($(legendChartSVG).clone()).html();
				                                contentSVG += containerselectionChartSVG.toString() + containerSelectionLegendSVG.toString();
				                            }
				                        }
				                    }
				                }
				            }
				        } else {
				            TDStyle = "";
				            for (var index = 0; index < serIndex; index++) {
				                if ($("#Selectionseries" + index + "Segmentseries" + index).length > 0) {
				                    TDStyle += $("#Selectionseries" + index + "Segmentseries" + index)[0].outerHTML;
				                }
				            }
				            contentSVG += TDStyle;
				        }

				        //draw background image for chart
				        if (chartObj.model.backGroundImageUrl != null) {
				            var img = new Image();
				            img.src = chartObj.model.backGroundImageUrl;
				            var canvasObj = document.createElement('canvas');
				            var ctx = canvasObj.getContext("2d");
				            canvasObj.setAttribute('width', printSvgElement.width);
				            canvasObj.setAttribute('height', printSvgElement.height);
				            ctx.drawImage(img, 0, 0, printSvgElement.width, printSvgElement.height);
				            contentSVG += "<img src='" + canvasObj.toDataURL() + "'/>";
				        }
				        contentSVG += '<svg xmlns="http://www.w3.org/2000/svg" style="left:0px; top:0px; position:absolute;" width="' + printSvgElement.width + '" height="' + printSvgElement.height + '" >';
				        contentSVG += printSvgElement.chartSVG.toString() + " </svg>";

				        //Add zooming scrollbar with chart
				        contentSVG += scrollbar + "</div></html>";

				        //to store the chart svg element
				        writeCode.push(contentSVG);
				    }

				    for (var l = 0; l < zoomBtnObj.length; l++) {
				        //push zoomButtons again
				        $("#chartContainer_" + Id).append(zoomBtnObj[l]);
				    }
				}
				//the div does not contains the chart element
				else {
				    contentSVG = "<html>";
				    contentSVG += document.getElementById(arguments[i]).innerHTML;
				    contentSVG += "</html>";
				    //to store the div id elements
				    writeCode.push(contentSVG);
				}
			}
			//to print the chart in a new window
			var win = window.open();
			for (var s = 0; s < writeCode.length; s++)
			    win.document.write(writeCode[s]);
			//to open a print window for canvas element
			if (canvasChart.length > 0) {
			    var img = new Image();
			    for (var y = 0 ; y < canvasChart.length; y++) {
			        contentSVG += "<img src='" + canvasChart[y].toDataURL() + "'/>";
			        img.src = canvasChart[y].toDataURL();
			    }
			    contentSVG += oImg;
			    //Add zooming scrollbar with chart
			    contentSVG += scrollbar + "</div></html>";
			    img.onload = function () {
			        win.document.write(contentSVG);
			        contentSVG = "";
			        win.document.close();
			        win.focus();
			        win.print();
			        win.close();
			    }
			}
			    //to open a print window for svg element
			else if (writeCode.length > 0) {
			    contentSVG = "";
			    win.document.close();
			    win.focus();
			    win.print();
			    win.close();			    
			}
										
		},


        redraw: function (excludeDataUpdate, pinchPanning, target, isTouch) {
            if (this.model.enableCanvasRendering) {
                if (this.model.zooming.enable && this.svgRenderer.ctx)
                    this.svgRenderer.ctx.clearRect(0, 0, $("#" + this._id).width(), $("#" + this._id).height());
                else {
                    $(this.svgObject).remove();
                    var width = this.svgWidth;
                    $("#canvas_trackSymbol").remove();
                    $("#"+this._id+"_canvas_Tracker").remove();
                    $("#secondCanvas").remove();
                    this.svgRenderer = new ej.EjCanvasRender(this.element); // to create canvas container
                    this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                    this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? width : this.model.size.width;
                    this.svgObject = this.svgRenderer.svgObj;
                    var ctx = this.svgRenderer.svgObj.getContext("2d");

					$(this.scrollerContainer).remove();
                    $(this.legendSvgContainer).remove();
                    $(this.legendContainer).remove();
                    this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
					this.scrollerContainer = $("<div></div>").attr('id', "legend_Scroller" + this._id).css("height", "0px");
                    this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                    this.svgRenderer.append(this.legendSvgContainer, this.scrollerContainer);
					$(this.scrollerContainer).appendTo(this.legendContainer);
                    $(this.legendContainer).appendTo(this.chartContainer);
                }
            }
            else if ((this.svgObject.id).indexOf("_canvas") != -1) {
                this._removeZoomkit();
                $(this.svgObject).remove();
                this.svgRenderer = new ej.EjSvgRender(this.element);  // to create svg container
                this.svgObject = this.svgRenderer.svgObj;
                this.canvasX = this.canvasY = 0;

                $(this.scrollerContainer).remove();
                $(this.legendSvgContainer).remove();
                $(this.legendContainer).remove();
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                this.scrollerContainer = $("<div></div>").attr('id', "legend_Scroller" + this._id).css("height", "0px");
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.scrollerContainer);
                $(this.scrollerContainer).appendTo(this.legendContainer);
                $(this.legendContainer).appendTo(this.chartContainer);
            } else {
                if (!this.dragPoint || (!pinchPanning && (!this.panning || !target || this.scrollbarUpdate || (isTouch ||this.panning && this.model.browserInfo.name != "chrome")) || this.model.enable3D)) {
					$(this.svgObject).empty();  // using existing svg container
					$(this.legendSvgContainer).empty();
				}
				else
				{
				    var svg = this.svgObject;
				    this.chartUpdating = true;
				    if (!this.continuePinching && target) {
				        target.id = "";
				        target.setAttribute("opacity", 0);
				        svg.appendChild(target);
				        this.continuePinching = false;
				    }
				    while (svg.childNodes.length > 1)
				        svg.removeChild(svg.firstChild);

				    $(this.legendSvgContainer).empty();
				}
            }
            $("#annotation_group_" + this._id).remove();   // to remove annotation on redraw
            if (!excludeDataUpdate) {

                if (this.model.canResize || this.model.isResponsive)
                    this.bindResizeEvents();
                else
                    this.removeResizeEvents();
            }
            this.bindTo(excludeDataUpdate);
        },

          "export": function (type, action, multipleExport) {       // method to export chart
            var data,
            exporting = this.model.exportSettings,
            type = ej.util.isNullOrUndefined(type) ? exporting.type : type.toLowerCase(),
            action = ej.util.isNullOrUndefined(action) ? exporting.action : action.toLowerCase();

            if (type == "xlsx")
                this.excelExport(type, action, multipleExport);
            else if (type == "svg" || !this.model.enableCanvasRendering)
                data = this.svgExport();
            else
                data = this.imageExport();

            return data;
        },

         excelExport: function (type, action, multipleExport) {           // to export chart as excel
            var modelClone = $.extend(true, {}, this.model),
            exporting = this.model.exportSettings,
            multipleExport = ej.util.isNullOrUndefined(type) ? exporting.multipleExport : multipleExport,
            attr = { action: action, method: 'post' },
            form = ej.buildTag('form', "", null, attr),
            series, chartobj = this;
            modelClone.event = null; // event is not API and having circular structure object type

            if (this._ignoreOnExport) {
                series = modelClone.series;
                for (var j = 0; j < series.length; j++) {
                    delete series[j].dataSource;
                    delete series[j].query;
                    series[j].fill = jQuery.type(series[j].fill) == "array" ? series[j].fill[0].color : series[j].fill;
                }
            }
            if (ej.raiseWebFormsServerEvents){
                this.raiseWebServerEvents(JSON.stringify(modelClone), "excelExporting");
                this.raiseWebServerEvents(JSON.stringify(modelClone));
             }
            else {
                if (multipleExport) {
                    var chartObjectArray = {};
                    $('body').find('.e-datavisualization-chart').each(function (index, object) {
                        var chartObject = $(object).data('ejChart');
                        if (!ej.isNullOrUndefined(chartObject)) {
						    chartObject.model.event=null;
                            var modelClone = JSON.parse(JSON.stringify(chartObject.model));							
                            if (chartobj._ignoreOnExport) {
                                series = modelClone.series;
                                for (var j = 0; j < series.length; j++) {
                                    delete series[j].dataSource;
                                    delete series[j].query;
                                    series[j].fill = jQuery.type(series[j].fill) == "array" ? series[j].fill[0].color : series[j].fill;
                                }
                                chartObjectArray[index] = JSON.stringify(modelClone);
                                var inputAttr = { name: 'ChartModel', type: 'hidden', value: JSON.stringify(modelClone) }
                                var input = ej.buildTag('input', "", null, inputAttr);
                                form.append(input);
                            }
                        }
                    });
                }
                else {
                    var inputAttr = { name: 'ChartModel', type: 'hidden', value: JSON.stringify(modelClone) }
                    var input = ej.buildTag('input', "", null, inputAttr);
                    form.append(input);
                    form.append(this);
                }
                $('body').append(form);
                form.submit();
            }
            return true;
        },

        imageExport: function () {       // to export chart as image
            var modelClone = $.extend(true, {}, this.model);
			modelClone.event = null; // event is not API and having circular structure object type
            var Id = this._id,
            selectionChart = $('[id*=' + Id + '_Selection_' + ']'),
                legend = $('#legend_' + Id + '_canvas')[0],
            exporting = this.model.exportSettings,
            angle = exporting.angle,exportChart,exprtCtx,chartWidth,chartHeight,chart,
            legendPos = this.model.LegendBounds, containerStyle, i, dataURL,
			exportCanvas = this.getCanvasElement(Id);
            exportChart = exportCanvas.canvasContainer;
			exprtCtx = exportCanvas.canvasArea;
			chartWidth = exportCanvas.width;
			chartHeight = exportCanvas.height;
			chart = $("#" + Id + '_canvas')[0]; 
            exportChart.setAttribute('id', 'Export_' + Id);
            if (angle==0){
                legendPos = this.model.LegendBounds;
			if (legendPos.X && (legendPos.Width != 0 && legendPos.Height != 0))
                   exprtCtx.drawImage(legend, legendPos.X, legendPos.Y);
            } else if (angle == 90 || angle == -90) {
                 exportChart.setAttribute('width', chartHeight);
            exportChart.setAttribute('height', chartWidth);
            exprtCtx.save();
            exprtCtx.translate(chartHeight/2, chartWidth/2);
            exprtCtx.rotate(angle*Math.PI/180);
            exprtCtx.drawImage(chart,-chartWidth/2,- chartHeight/2);
            exprtCtx.restore();
            if (legendPos.X){
              exprtCtx.save();
              if (angle==90)
                exprtCtx.translate(chartHeight - legendPos.Y - legendPos.Height/2, chartWidth- legendPos.X - legendPos.Width/2); 
              else
                exprtCtx.translate(legendPos.Y + legendPos.Height/2, legendPos.X + legendPos.Width/2);
              exprtCtx.rotate(angle*Math.PI/180);
              exprtCtx.drawImage(legend, -legendPos.Width/2 , -legendPos.Height/2);
              exprtCtx.restore();
             }
            } else {
                exportChart.setAttribute('width', chartWidth);
                exportChart.setAttribute('height', chartHeight);
                exprtCtx.save();
                exprtCtx.translate(chartWidth, chartHeight);
                exprtCtx.rotate(180*Math.PI/180);
                exprtCtx.drawImage(chart,0,0);
                exprtCtx.restore();
                if (legendPos.X){
                  exprtCtx.save();
                  exprtCtx.translate(legendPos.X + legendPos.Width/2, chartHeight -legendPos.Y - legendPos.Height/2);
                  exprtCtx.rotate(angle*Math.PI/180);
                  exprtCtx.drawImage(legend, -legendPos.Width/2 , -legendPos.Height/2);
                  exprtCtx.restore();
                }
              }

             for (i = 0; i < selectionChart.length; i++) {       // export for selection
                containerStyle = document.getElementById(selectionChart[i].id).getBoundingClientRect();
                exprtCtx.drawImage(selectionChart[i], 0, 0);
            }
            dataURL = exporting.type == "jpg" ? exportChart.toDataURL("image/jpeg") : exportChart.toDataURL();
			if (exporting.mode == "client")                    // for client side
			    return exportChart;
            else if (ej.raiseWebFormsServerEvents)                            // for web forms
                this.raiseWebServerEvents(dataURL);
            else if (exporting.mode == "server") {                       // server side
                var attr, form, chartModel, input1, data, input2;
                attr = { action: exporting.action, method: 'post' };
                form = ej.buildTag('form', "", null, attr);
                chartModel = { name: 'ChartModel', type: 'hidden', value: JSON.stringify(modelClone) };
                input1 = ej.buildTag('input', "", null, chartModel);
				data = { name: 'Data', type: 'hidden', value: dataURL};
				input2 = ej.buildTag('input', "", null, data);
                form.append(input1).append(input2).append(this);
                $('body').append(form);
                form.submit();
            }
        },

        svgExport: function () {                   // to export chart as SVG
            var svgObj = this.svgObject.id,
            modelClone = $.extend(true, {}, this.model);
			modelClone.event = null; // event is not API and having circular structure object type
			var Id = this._id,
			exportChartSVG = this.getSVGElement(Id),
            exporting = this.model.exportSettings, angle = exporting.angle,chartHeight = exportChartSVG.height,
			svg = exportChartSVG.chartSVG,legend = exportChartSVG.legendChartSVG,chartWidth = exportChartSVG.width, 
            content= '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">';
            if (angle ==0)
                content = content + svg.toString() + legend.toString() + "</svg>";
            else if (angle == 90)
                content = content + '<g transform="translate('+ chartHeight + ',0) rotate(90 0 0)">' + svg.toString() + legend.toString() + "</g></svg>";
            else if (angle == -90)
                content = content + '<g transform="translate(0,'+ chartWidth + ') rotate(-90 0 0)">' + svg.toString() + legend.toString() + "</g></svg>";
            else
                content = content + '<g transform="translate('+ chartWidth + ',' + chartHeight + ') rotate(180 0 0)">' + svg.toString() + legend.toString() + "</g></svg>";
            content = encodeURIComponent(content);
            $("#" + svgObj + "_Legend").removeAttr("transform");
			if (exporting.mode == "client")            // for client side
			    return content;
            else if (ej.raiseWebFormsServerEvents)                               // for web forms
                this.raiseWebServerEvents(content);
            else if (exporting.mode == "server") {                         // server side
                var attr, form, inputAttr, input;
                attr = { action: exporting.action, method: 'post' };
                form = ej.buildTag('form', "", null, attr);
                chartModel = { name: 'ChartModel', type: 'hidden', value: JSON.stringify(modelClone) };
                input1 = ej.buildTag('input', "", null, chartModel);
                inputAttr = { name: 'Data', type: 'hidden', value: content };
                input = ej.buildTag('input', "", null, inputAttr);
                form.append(input).append(input1).append(this);
                $('body').append(form);
                form.submit();
            }
        },
        
		 raiseWebServerEvents: function (content, event) {             // to trigger server events in web forms
            var modelClone = $.extend(true, {}, this.model),
            exporting = this.model.exportSettings,
            event = ej.util.isNullOrUndefined(event) ? "exporting" : event;
            args = { model: modelClone, originalEventType: event },
            clientArgs = { Data: content, Format: exporting.type, Orientation: exporting.orientation, FileName: exporting.fileName };
            ej.raiseWebFormsServerEvents(event, args, clientArgs);
        },
		
        animate: function (options) {
            if (!options) { // Animate the entire series and indicators "previous behavior"
                var series = this.model.series;
                for (var i = 0; i < series.length; i++) {
                    series[i]._animatedSeries = false;
                    series[i]._animatedTrendline = false;
                    series[i].AnimationComplete = false; //pie doughnut animation with datalabel
                }
                var indicators = this.model.indicators;
                for (var j = 0; j < indicators.length; j++) {
                    indicators[j]._animatedSeries = false;
                    indicators[j]._animatedTrendline = false;
                }
            } else if (options.constructor === Array) { // Animate array of series
                for (var i = 0; i < options.length; i++) {
                    options[i]._animatedSeries = false;
                    options[i]._animatedTrendline = false;
                    options[i].AnimationComplete = false; //pie doughnut animation with datalabel
                }
            } else {   // Animate the specific series
                options._animatedSeries = false;
                options._animatedTrendline = false;
                options.AnimationComplete = false;  //pie doughnut animation with datalabel
            }
            this.redraw();
        },


        _setModel: function (options) {
            var series, axes;
            for (var prop in options) {
                this.disableAnimation();
                switch (prop) {
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
                        $.extend(true, this.model.primaryXAxis, ej.EjSvgRender.themes[this.model.theme].primaryXAxis);
                        $.extend(true, this.model.primaryYAxis, ej.EjSvgRender.themes[this.model.theme].primaryYAxis);
                        for (var k = 0; k < this.model.axes.length; k++) {
                            if (this.model.axes[k].orientation && this.model.axes[k].orientation.toLowerCase() == "horizontal")
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryX);
                            else
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryY);
                        }
                        for (var l = 0; l < this.model.series.length; l++) {
                            $.extend(true, this.model.series[l], ej.EjSvgRender.themes[this.model.theme].commonSeriesOptions);
                        }
                        break;
                    case "commonSeriesOptions":
                        for (var i = 0; i < this.model.series.length; i++)
						{
							if (options[prop].dataSource)
								this.model.series[i].dataSource = null;
							$.extend(true, this.model.series[i], {}, options[prop]);
						}
                        break;
                    case "series":
                        series = this.model.series;
						 for (var i = 0; i < series.length; i++) {
                            if (options[prop][i].dataSource) {
                                series[i].dataSource = null;
                            }
                         }
                        $.extend(true, series, {}, options[prop]);
                        for (var i = 0, len = series.length; i < len; i++) {
                            if (series[i].enableAnimation)
                                series[i]._animatedSeries = false;
							if(options[prop][i] && options[prop][i].dataSource instanceof ej.DataManager)
								//jQuery extends data manager as object.
								series[i].dataSource = options[prop][i].dataSource;
							if(options[prop][i] && options[prop][i].query instanceof ej.Query)
								//jQuery.extend does not update the existing query with new one.
								series[i].query = options[prop][i].query;
                        }
                        break;
                    case "legend":
                        $.extend(true, this.model.legend, {}, options[prop]);
                        break;
                    case "axes":
                        $.extend(true, this.model.axes, {}, options[prop]);
                        break;
                    case "primaryXAxis":
                        this.model.primaryXAxis.setRange = (options[prop].range) ? true : false;
                        this.model.primaryXAxis.setAxisInterval = (options[prop].range && options[prop].range.interval) ? true : this.model.primaryXAxis.setAxisInterval;
                        this.model.primaryXAxis.actual_Range = (this.model._axes[0].setRange) ? null : this.model._axes[0].actual_Range;
                        $.extend(true, this.model.primaryXAxis, {}, options[prop]);
                        break;
                    case "primaryYAxis":
                        this.model.primaryYAxis.setRange = (options[prop].range) ? true : false;
                        this.model.primaryYAxis.log_Range = options[prop].range ? options[prop].range : this.model.primaryYAxis.log_Range;
                        this.model.primaryYAxis.actual_Range = (this.model._axes[1].setRange) ? null : this.model._axes[1].actual_Range;
                        $.extend(true, this.model.primaryYAxis, {}, options[prop]);
                        break;
                    case "xZoomFactor":
                        this.model.primaryXAxis.zoomFactor = this._xZoomFactor();
                        axes = this.model.axes;
                        for (var i = 0; i < axes.length; i++) {
                            if(axes[i].orientation.toLowerCase() == "horizontal")
                            axes[i].zoomFactor = this._xZoomFactor();
                        }
                        break;
                    case "yZoomFactor":
                        this.model.primaryYAxis.zoomFactor = this._yZoomFactor();
                        axes = this.model.axes;
                        for (var i = 0; i < axes.length; i++) {
                            if (axes[i].orientation.toLowerCase() == "vertical")
                                axes[i].zoomFactor = this._yZoomFactor();
                        }
                        break;
                    case "xZoomPosition":
                        this.model.primaryXAxis.zoomPosition = this._xZoomPosition();
                        axes = this.model.axes;
                        for (var i = 0; i < axes.length; i++) {
                            if (axes[i].orientation.toLowerCase() == "horizontal")
                                axes[i].zoomPosition = this._xZoomPosition();
                        }
                        break;
                    case "yZoomPosition":
                        this.model.primaryYAxis.zoomPosition = this._yZoomPosition();
                        axes = this.model.axes;
                        for (var i = 0; i < axes.length; i++) {
                            if (axes[i].orientation.toLowerCase() == "vertical")
                                axes[i].zoomPosition = this._yZoomPosition();
                        }
                        break;
                    case "drilldown":
                        this.model.series = [];
                        this.model.explodeValue = null;
                        $.extend(true, this.model, {}, options[prop]);
                        break;
                    case "datasource":
                        for (var i = 0; i < this.model.series.length; i++) {
                            this.model.series[i].dataSource = null;
                            this.model.series[i].points = null;
                        }
                        $.extend(true, this.model, {}, options[prop]);
                        break;
                    case "chartArea":
                        $.extend(true, this.model.chartArea, {}, options[prop]);
                        break;

                    default:
                        $.extend(true, this.model, {}, options[prop]);
                        series = this.model.series;
                        for (var i = 0, len = series.length; i < len; i++) {
                            if (series[i].enableAnimation)
                                series[i]._animatedSeries = false;
                        }
                }
            }
            if (this.model.canResize ||this.model.isResponsive)
                this.bindResizeEvents();
            else
                this.removeResizeEvents();
            $("#annotation_group_" + this._id).remove(); // to remove annotation on set model
            if (this.model.enableCanvasRendering) {
                $(this.svgObject).remove();
                $("#canvas_trackSymbol").remove();
                $("#" + this._id + "_canvas_Tracker").remove();
                $("#secondCanvas").remove();
                this.svgRenderer = new ej.EjCanvasRender(this.element); // to create canvas container
                this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? $("#" + this._id).width() : this.model.size.width;
                this.svgObject = this.svgRenderer.svgObj;

                $(this.legendSvgContainer).remove();
                this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            }
            else if ((this.svgObject.id).indexOf("_canvas") != -1) {
                this._removeZoomkit();
                $(this.svgObject).remove();
                this.svgRenderer = new ej.EjSvgRender(this.element);  // to create svg container
                this.svgObject = this.svgRenderer.svgObj;
                this.canvasX = this.canvasY = 0;

                $(this.legendSvgContainer).remove();
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            }
            else if(!this.chartUpdating){ // Condition checked to avoid knockout removing event.target
                $(this.svgObject).empty();              // using existing svg container
                $(this.legendSvgContainer).empty();
            }

            this.bindTo();
        }
    });
	
	ej.datavisualization.Chart.Locale = ej.datavisualization.Chart.Locale || {};
    ej.datavisualization.Chart.Locale['default'] = ej.datavisualization.Chart.Locale['en-US'] = {
        zoomIn: "Zoom In",
        zoomOut: "Zoom Out",
        zoom: "Zoom",
        pan: "Pan",
        reset: "Reset"
    };
    ej.datavisualization.Chart.CrosshairType = {

        Crosshair: 'crosshair',

        TrackBall: 'trackBall'
    };
    ej.datavisualization.Chart.TrackballDisplayMode = {

        Float: 'float',

        Grouping: 'grouping'
    };
	
	ej.datavisualization.Chart.VisibleOnLegend={
		
		Visible:'visible',
		
		Hidden:'hidden'
	};
	
    ej.datavisualization.Chart.ExportingType = {

        PNG: 'png',

        JPG: 'jpg',

        PDF: 'pdf',

        SVG: 'svg',

        DOCX: 'docx',

        XLSX: 'xlsx'
    };

    ej.datavisualization.Chart.ExportingMode = {
       
        Client: 'client',

        Server: 'server'
    };

     ej.datavisualization.Chart.ExportingOrientation = {
       
        Portrait: 'portrait',

        Landscape: 'landscape'
    };

    ej.datavisualization.Chart.ValueType = {

        Double: 'double',

        DateTime: 'datetime',

        Category: 'category',

        Logarithmic: 'logarithmic',

        DateTimeCategory:'datetimecategory'
    };

    ej.datavisualization.Chart.TextOverflow = {

        None: 'none',

        Wrap: 'wrap',

        Trim: 'trim',

        WrapAndTrim:'wrapandtrim'
    };

    ej.datavisualization.Chart.Type = {

        Line: 'line',

        Spline: 'spline',

        Column: 'column',

        Doughnut: 'doughnut',

        Area: 'area',

        SplineArea: 'splinearea',

        StepLine: 'stepline',

        StepArea: 'steparea',

        Pie: 'pie',

        PieOfPie: 'pieofpie',

        Hilo: 'hilo',

        HiloOpenClose: 'hiloopenclose',

        Candle: 'candle',

        Bubble: 'bubble',

        Scatter: 'scatter',

        Bar: 'bar',

        StackingArea: 'stackingarea',

        StackingArea100: 'stackingarea100',

        RangeColumn: 'rangecolumn',

        StackingColumn: 'stackingcolumn',

        StackingColumn100: 'stackingcolumn100',

        StackingBar: 'stackingbar',

        StackingBar100: 'stackingbar100',

        Pyramid: 'pyramid',

        Funnel: 'funnel',

        Polar: 'polar',

        Radar: 'radar',

        RangeArea: 'rangearea',

        Waterfall: 'waterfall',
		
		 BoxAndWhishker:'boxandwhisker'

    };
    ej.datavisualization.Chart.SplitMode = {

        Position: 'position',

        Value: 'value',

        Percentage: 'percentage',

        Indexes: 'indexes'

    };
	ej.datavisualization.Chart.DragType = {
	
		XY:"xy",
		
		X:"x",
		
		Y:"y"
	};
    ej.datavisualization.Chart.LabelPlacement = {

        BetweenTicks: "betweenTicks",

        OnTicks: "onTicks"
    };

    ej.datavisualization.Chart.TrendlinesType = {

        Linear: "linear",

        Exponential: "exponential",

        Logarithmic: "logarithmic",

        Power: "power",

        Polynomial: "polynomial"
    };

    ej.datavisualization.Chart.ErrorBarType = {

        FixedValue: "fixedValue",

        Percentage: "percentage",

        StandardDeviation: "standardDeviation",

        StandardError: "standardError",

        Custom: "custom"
    };

    ej.datavisualization.Chart.ErrorBarMode = {

        Both: "both",

        Vertical: "vertical",

        Horizontal: "horizontal"
    };
    
    ej.datavisualization.Chart.ErrorBarDirection = {

        Both: "both",

        Plus: "plus",

        Minus:"minus"
    };

    ej.datavisualization.Chart.LabelIntersectAction = {

        None: "none",

        Rotate90: "rotate90",

        Rotate45: "rotate45",

        Wrap: "wrap",

        WrapByword : "wrapByWord",

        Trim: "trim",

        Hide: "hide",

        MultipleRows: "multipleRows"
    };

    ej.datavisualization.Chart.EdgeLabelPlacement = {

        None: "none",

        Shift: "shift",

        Hide: "hide"
    };
	
	 ej.datavisualization.Chart.roundedCorner={
	 
		Both:"both",
		
		Start:"start",
		
		End:"end"
	 
	 };

    ej.datavisualization.Chart.Theme = {

        Azure: 'azure',

        FlatLight: 'flatlight',

        Azuredark: 'azuredark',

        Lime: 'lime',

        LimeDark: 'limedark',

        Saffron: 'saffron',

        SaffronDark: 'saffrondark',

        GradientLight: 'gradientlight',

       GradientDark: 'gradientdark',
		
		HighContrast01: 'highcontrast01',
		
		HighContrast02 : 'highcontrast02',
		
		Material : 'material',
		
		Office365: 'office365',
		
		Bootstrap: 'bootstrap'
    };

    ej.datavisualization.Chart.FontStyle = {

        Normal: 'normal',

        Italic: 'italic'
    };

    ej.datavisualization.Chart.FontWeight = {

        Regular: 'regular',

        Bold: 'bold',

        Lighter: 'lighter'
    };

    ej.datavisualization.Chart.IntervalType = {

        Auto : 'auto',

        Days: 'days',

        Hours: 'hours',

        Seconds: 'seconds',

        Milliseconds: 'milliseconds',

        Minutes: 'minutes',

        Months: 'months',

        Years: 'years'
    };

    ej.datavisualization.Chart.RangePadding = {

        Additional: 'additional',

        Normal: 'normal',

        None: 'none',

        Round: 'round',

        Auto: 'auto'
    };

    ej.datavisualization.Chart.TextAlignment = {

        MiddleTop: 'middletop',

        MiddleCenter: 'middlecenter',

        MiddleBottom: 'middlebottom'
    };

    ej.datavisualization.Chart.LabelPosition = {

        Inside: 'inside',

        Outside: 'outside'
    };

    ej.datavisualization.Chart.ZIndex = {

        Over: 'over',

        Behind: 'behind'
    };

    ej.datavisualization.Chart.Unit = {

        percentage: 'percentage',

        pixel: 'pixel'
    };

    ej.datavisualization.Chart.PyramidMode = {

        Linear: 'linear',

        Surface: 'Surface'
    };

    ej.datavisualization.Chart.DrawType = {

        Line: 'line',

        Column: 'column',

        Area: 'area'
    };
    ej.datavisualization.Chart.columnFacet = {

        Rectangle: 'rectangle',

        Cylinder: 'cylinder'
    };

	ej.datavisualization.Chart.EmptyPointMode = {

        Gap: 'gap',

        Zero: 'zero',

        Average: 'average'

    };

    ej.datavisualization.Chart.Pattern = {

        None: 'none',

        Chessboard: 'chessboard',

        Crosshatch: 'crosshatch',

        Dots: 'dots',

        Pacman: 'pacman',

        DiagonalBackward: 'diagonalBackward',

        DiagonalForward: 'diagonalForward',

        Grid: 'grid',

        Turquoise: 'turquoise',

        Star: 'star',

        Triangle: 'triangle',

        Circle: 'circle',

        Tile: 'tile',

        HorizontalDash: 'horizontalDash',

        VerticalDash: 'verticalDash',

        Rectangle: 'rectangle',

        Box: 'box',

        VerticalStripe: 'verticalStripe',

        HorizontalStripe: 'horizontalStripe',

        Bubble: 'bubble',

        Custom: 'custom'
    };

    ej.datavisualization.Chart.Mode = {

        Series: 'series',

        Point: 'point',
		
        Cluster: 'cluster',

        Range: 'range'
    };

    ej.datavisualization.Chart.SelectionType = {

        Single: 'single',

        Multiple:'multiple'
    };
    ej.datavisualization.Chart.Shape = {

        None: 'none',

        LeftArrow: 'leftarrow',

        RightArrow: 'rightarrow',

        Circle: 'circle',

        Cross: 'cross',

        HorizLine: 'horizline',

        VertLine: 'vertLine',

        Diamond: 'diamond',

        Rectangle: 'rectangle',

        Triangle: 'triangle',

        InvertedTriangle: 'invertedtriangle',

        Hexagon: 'hexagon',

        Pentagon: 'pentagon',

        Star: 'star',

        Ellipse: 'ellipse',

        Wedge: 'wedge',

        Trapezoid: 'trapezoid',

        UpArrow: 'uparrow',

        DownArrow: 'downarrow',

        Image: 'image'
    };

    ej.datavisualization.Chart.DrawMode = {

        Both: 'both',

        Open: 'open',

        Close: 'close'
    };

    ej.datavisualization.Chart.LineCap = {

        Butt: 'butt',

        Round: 'round',

        Square: 'square'
    };

    ej.datavisualization.Chart.LineJoin = {

        Round: 'round',

        Bevel: 'bevel',

        Miter: 'miter'
    };

    ej.datavisualization.Chart.Position = {

        Top: 'top',

        Middle: 'middle',

        Bottom: 'bottom'
    };

    ej.datavisualization.Chart.Alignment = {

        Center: 'center',

        Near: 'near',

        Far: 'far'
    };
	
	ej.datavisualization.Chart.TickLinesPosition = {

        Inside: 'inside',

        Outside: 'outside'
    };
	
	ej.datavisualization.Chart.CoordinateUnit = {

        None: 'none',

        Pixels: 'pixels',
		
		Points: 'points'
    };
	
	ej.datavisualization.Chart.HorizontalAlignment = {

        Left: 'left',

        Right: 'right',
		
		Middle: 'middle'
    };
	
	ej.datavisualization.Chart.VerticalAlignment = {

        Top: 'top',

        Bottom: 'bottom',
		
		Middle: 'middle'
    };
	
	ej.datavisualization.Chart.Region = {

        Chart: 'chart',

        Series: 'series'
    };
	
	ej.datavisualization.Chart.ConnectorLineType = {

        Line: 'line',

        Bezier: 'bezier'
    };
	
	ej.datavisualization.Chart.HorizontalTextAlignment = {

        Near: 'near',

        Far: 'far',
		
		Center: 'center'
    };
	
	ej.datavisualization.Chart.VerticalTextAlignment = {

        Near: 'near',

        Far: 'far',
		
		Center: 'center'
    };
	
	ej.datavisualization.Chart.multiLevelLabelsBorderType = {

        None: 'none',

        Rectangle: 'rectangle',
		
		WithoutTopAndBottom: 'withouttopandbottom',
		
		Brace: 'brace',
		
		CurlyBrace: 'curlybrace'
    };
	
	ej.datavisualization.Chart.MACDType = {

        Line: 'line',

        Histogram: 'histogram',
		
		Both: 'both'
    };
	
	ej.datavisualization.Chart.IndicatorsType = {

        RSI: 'rsi',

        Momentum: 'momentum',
		
		Bollingerband: 'bollingerband',
		
		Accumulationdistribution : 'accumulationdistribution',
		
		EMA: 'ema',
		
		SMA: 'sma',
		
		Stochastic: 'stochastic',
		
		ATR: 'atr',
		
		MACD: 'macd',
		
		TMA: 'tma'
    };
	
	ej.datavisualization.Chart.BoxPlotMode = {
	
		  Normal: 'normal',

         Exclusive: 'exclusive',

         Inclusive: 'inclusive',
		
		
    };
	
	ej.datavisualization.Chart.BubbleRadiusMode = {
	
		 MinMax: 'minmax',

         Auto: 'auto'
		
    };
	
    $.extend(ej.datavisualization.Chart.prototype, ej.ejChart);
})(jQuery, Syncfusion);;
ej.EjSvgScrollbarRender = function (element,scrollObj) {
    this.svgSupport = (window.SVGSVGElement) ? true : false;
    var id = jQuery(element).attr("id");
    this.scrollsvgObj = this.scrollsvgObj ? this.scrollsvgObj : [];
    if (this.svgSupport) {
        this.svgLink = "http://www.w3.org/2000/svg";
        this.scrollsvgObj[scrollObj.index] = document.createElementNS(this.svgLink, "svg");
        this.scrollsvgObj[scrollObj.index].setAttribute('id', "scrollbar_" + id + scrollObj.index);
    } else {
        var doc = document;
        this.scrollsvgObj[scrollObj.index] = doc.createElement("div");
        this.scrollsvgObj[scrollObj.index].style.position = 'relative';
        this.scrollsvgObj[scrollObj.index].setAttribute('id', "scrollbar_" + id + scrollObj.index);
    }
};

(function ($) {
    ej.EjSvgScrollbarRender.prototype = {

        _initializeScrollbarVariables: function (scrollObj) {
            scrollObj.offsetLeftX = 0
            scrollObj.rectWidth = scrollObj.offsetRightX = scrollObj.oldWidth = scrollObj.width;
            scrollObj.startX = 0, scrollObj.endX = 0;
            scrollObj.scrollbarLoaded = false;
            
            if (scrollObj.zoomPosition || scrollObj.zoomFactor) {
                scrollObj.offsetLeftX = (scrollObj.zoomPosition * scrollObj.width);
                scrollObj.rectWidth = (scrollObj.zoomFactor) * scrollObj.width;
            }
        },

        _scrollbarUpdate: function (scrollObj) {

            var width = parseFloat(scrollObj.width),
                arrowsize = 15,
                minimumRect = 44,
                panningRect = width - 30;// arrowSize * 2 
            scrollObj.offsetLeftX = !scrollObj.scrollbarLoaded ? scrollObj.offsetLeftX : (scrollObj.offsetLeftX - arrowsize);
            scrollObj.offsetLeftX = scrollObj.offsetLeftX < arrowsize ? 0 : scrollObj.offsetLeftX;
            var offsetLeftX = ((scrollObj.offsetLeftX / scrollObj.oldWidth) * panningRect) + arrowsize;
            var rectWidth = ((scrollObj.rectWidth / scrollObj.oldWidth) * panningRect);
            isMinWidth = rectWidth < minimumRect;
            scrollObj._diff = isMinWidth ? minimumRect - rectWidth : 0;
            rectWidth = rectWidth > minimumRect ? rectWidth : minimumRect;
            scrollObj.oldWidth = panningRect;
            var offsetRightX = offsetLeftX + rectWidth;
            scrollObj.rectWidth = rectWidth;
            scrollObj.offsetLeftX = scrollObj.startX = offsetLeftX;
            scrollObj.offsetRightX = (offsetRightX - arrowsize) <= offsetLeftX ? (offsetRightX + arrowsize) : offsetRightX;
            scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            scrollObj.startX = offsetLeftX < arrowsize ? arrowsize : offsetLeftX;
            if ((offsetLeftX + rectWidth) > (scrollObj.width - arrowsize)) {
                scrollObj.offsetLeftX = scrollObj.startX = scrollObj.width - arrowsize - rectWidth;
                scrollObj.offsetRightX = scrollObj.width - arrowsize;
                scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            }
            scrollObj.scrollbarLoaded = true;

        },

        _renderScrollbar: function (scrollObj) {

            this.scrollsvgObj = this.scrollsvgObj ? this.scrollsvgObj : [];
            $(this.scrollsvgObj[scrollObj.index]).empty();
			if(this.scrollsvgObj[scrollObj.index])
				$(document).find('[id*= '+this.scrollsvgObj[scrollObj.index].id +']').attr('height','0px');
            var id = this._id;
            var arrowSize = 15;
            var scrollHeight = 18;
            var minimumScrollSize = 44;
            this.scrollsvgObj[scrollObj.index] = this.scrollbarContainer.scrollsvgObj[scrollObj.index];
            var height = 17, width = parseFloat(scrollObj.width), padding = 8, centerY = (height/2), panningRect = width - (arrowSize * 2);
            scrollObj.offsetLeftX = !scrollObj.scrollbarLoaded ? scrollObj.offsetLeftX : (scrollObj.offsetLeftX - arrowSize);
            scrollObj.offsetLeftX = scrollObj.offsetLeftX < arrowSize ? 0 : scrollObj.offsetLeftX;
            var offsetLeftX = ((scrollObj.offsetLeftX / scrollObj.oldWidth) * panningRect) + arrowSize;
            var rectWidth = ((scrollObj.rectWidth / scrollObj.oldWidth) * panningRect);
            isMinWidth = rectWidth < minimumScrollSize;
            scrollObj._diff = isMinWidth ? minimumScrollSize - rectWidth : 0;
            rectWidth = rectWidth > minimumScrollSize ? rectWidth : minimumScrollSize;
            scrollObj.oldWidth = panningRect;
            offsetLeftX = ((offsetLeftX + rectWidth) > panningRect) ? (panningRect - (rectWidth - arrowSize)) : offsetLeftX;
            var offsetRightX = offsetLeftX + rectWidth;
            scrollObj.rectWidth = rectWidth;
            scrollObj.offsetLeftX = scrollObj.startX = offsetLeftX;
            scrollObj.offsetRightX = offsetLeftX + rectWidth;
            scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            scrollObj.scrollbarLoaded = true;
            offsetRightX = (offsetRightX - arrowSize) <= offsetLeftX ? (offsetRightX + arrowSize) : offsetRightX;
            var xPadding = this.padding || 0;
            var yPadding = !this.vmlRendering ? 0 : -5;
            if (scrollObj.orientation == 'horizontal')
                this.scrollbar = this.renderer.createGroup({ 'id': id + '_scrollbar' + '_' + scrollObj.index, transform: "translate(" + xPadding + "," + yPadding + ")" });
            else
                this.scrollbar = this.renderer.createGroup({ 'id': id + '_scrollbar' + '_' + scrollObj.index, transform: "translate(" + 0 + "," + scrollObj.width + ") rotate(270)" });
            //Draw a rounded corner path direction
            var makeShape = function (x, y, width, height, r, scrollObj) {
                var
                  spc = " ", // path drawing instruction letters with readable names
                  moveTo = "M",
                  horizLineTo = "h",
                  vertLineTo = "v",
                  arcTo = "a",
                  closePath = "z",
                  ori = scrollObj.orientation,
                  opp = scrollObj.opposed,
                  dStr;
                if ((ori == 'vertical' && !opp) || (opp && ori != 'vertical')) {
                    dStr = // the "d" path for the svg path
                    moveTo + spc + x + spc + r + spc +
                    vertLineTo + spc + (height - r) + spc +
                    horizLineTo + spc + (width) + spc +
                    vertLineTo + spc + (r - height) + spc +
                    arcTo + spc + r + spc + r + spc + 0 + spc + 0 + spc + 0 + spc + (-r) + spc + (-r) + spc +
                    horizLineTo + spc + -(width - 2 * r) + spc +
                    arcTo + spc + r + spc + r + spc + 0 + spc + 0 + spc + 0 + spc + (-4) + spc + (r) + spc +
                    closePath;

                } else {
                    dStr = // the "d" path for the svg path
                    moveTo + spc + x + spc + y + spc +
                    vertLineTo + spc + (height - r) + spc +
                    arcTo + spc + r + spc + r + spc + 0 + spc + 0 + spc + 0 + spc + r + spc + r + spc +
                    horizLineTo + spc + (width - 2 * r) + spc +
                    arcTo + spc + r + spc + r + spc + 0 + spc + 0 + spc + 0 + spc + r + spc + (-r) + spc +
                    vertLineTo + spc + (r - height) + spc +
                    closePath;
                }
                
                return dStr; 
            };
            var direction = makeShape(0, 0, width, scrollHeight, 4, scrollObj);

            //Create a scrollbar background rectangle
            var rectBorder = {
                'id': id + '_scrollbarBackRect_' + scrollObj.index,
                'stroke-width': 1,
                'height': scrollHeight,
                'width': width,
                'stroke-linejoin' :"round",
                'stroke': "#B4B4B4",               
                'fill': "#F7F7F7",
                'class':'e-rangeScroll-backRect'
            };

            if (!this.vmlRendering) {
                rectBorder.d = direction;
                this.renderer.drawPath(rectBorder, this.scrollbar);
            } else
                this.renderer.drawRect(rectBorder, this.scrollbar);

            var rightRectBorder = {
                'id': id + '_scrollbarRightRect_' + scrollObj.index,
                'stroke-width': 1,
                'x':(width - 5),
                'height': scrollHeight,
                'width': 5,
                'stroke-linejoin': "round",
                'stroke': "transparent",
                'fill': "transparent"
            };
            this.renderer.drawRect(rightRectBorder, this.scrollbar);

            var leftRectBorder = {
                'id': id + '_scrollbarLeftRect_' + scrollObj.index,
                'stroke-width': 1,
                'x': 0,
                'height': scrollHeight,
                'width': 5,
                'stroke-linejoin': "round",
                'stroke': "transparent",
                'fill': "transparent"
            };
            this.renderer.drawRect(leftRectBorder, this.scrollbar);

            //Create a left Arrow
            var leftArrow = {
                'id': id + '_scrollbarLeftArrow_' + scrollObj.index,
                'stroke-width': 1,
                'stroke': "#999999",
                'd': "M " + 5 + " " + 9 + " " + "L " + 10 + " " + 14 + " " + "L " + 10 + " " + 3.5 + " Z",
                'fill': "#999999",
                'class': "e-rangeScroll-arrow"
            };
            this.renderer.drawPath(leftArrow, this.scrollbar);          

            ////Create a right Arrow
            var rightArrow = {
                'id': id + '_scrollbarRightArrow_' + scrollObj.index,
                'stroke-width': 1,
                'stroke': "#999999",
                'd': "M " + (width - 5) + " " + 9 + " " + "L " + (width - 10) + " " + 14 + " " + "L " + (width - 10) + " " + 3.5 + " Z",
                'fill': "#999999",
                'class': "e-rangeScroll-arrow"
            };
            this.renderer.drawPath(rightArrow, this.scrollbar);

            var options = {
                'id': id + '_scrollbarSelect_' + scrollObj.index,
                x: offsetLeftX,
                y: 0,
                'width': rectWidth,
                'rx': 4,
                'ry': 4,
                'height': height,
                'stroke': "#999999",
                'stroke-width': 1,
                'fill': "#CECECE",
                'class':'e-rangeScroll-select'
            };
            this.renderer.drawRect(options, this.scrollbar);
            this.centerLine = this.renderer.createGroup({ 'id': id + '_scrollbarCenterLine_' + scrollObj.index, transform: "translate(" + (offsetLeftX + (rectWidth / 2) - 7.5) + ")" });

            //Create a center shape
            var shape = {
                'id': id + '_scrollbarCenterShape_' + scrollObj.index,
                'stroke-width': 1,
                'stroke': "#999999",
                'd': "M " + 0 + " " + 3.5 + " " + "L " + 0 + " " + 13.5 + " Z" + "M " + 5 + " " + 3.5 + " " + "L " + 5 + " " + 13.5 + " Z" + "M " + 10 + " " + 3.5 + " " + "L " + 10 + " " + 13.5 + " Z" + "M " + 15 + " " + 3.5 + " " + "L " + 15 + " " + 13.5 + " Z",
                'fill': "#999999",
                'class': 'e-rangeScroll-centerShape'
            };
            this.renderer.drawPath(shape, this.centerLine);
            this.renderer.append(this.centerLine, this.scrollbar);
            var leftHeaderHideRectOptions = {
                'id': id + '_leftHeaderHideRect_' + scrollObj.index,
                'x': offsetLeftX,
                'y': 0,
                'width': 11,
                'height': 17,
                'fill': 'transparent',
                'opacity': 0,
                'stroke-width': 1,
                'class': 'e-rangeScroll-leftRect'
            }
            this.renderer.drawRect(leftHeaderHideRectOptions, this.scrollbar);
            var leftHeaderOptions = {
                'id': id + '_scrollbarLeftHeader_' + scrollObj.index,
                'cx': offsetLeftX + padding,
                'cy': centerY,
                'r': 3,
                "fill": "#999999",
                'stroke': "#999999",
                'stroke-width': 1,
                'class': 'e-rangeScroll-leftCircle'
            };
            this.renderer.drawCircle(leftHeaderOptions, this.scrollbar);         
            var rightHeaderHideRectOptions = {
                'id': id + '_rightHeaderHideRect_' + scrollObj.index,
                'x': offsetRightX - 11,
                'y': 0,
                'width': 11,
                'height': height,
                'fill': 'transparent',
                'opacity': 0,
                'stroke-width': 1,
                'class': 'e-rangeScroll-rightRect'
            };
            this.renderer.drawRect(rightHeaderHideRectOptions, this.scrollbar);
            var rightHeaderOptions = {
                'id': id + '_scrollbarRightHeader_' + scrollObj.index,
                "fill": "#999999",
                'cx': offsetRightX - padding,
                'cy': centerY,
                'r': 3,
                'stroke': "#999999",
                'stroke-width': 1,
                'class': 'e-rangeScroll-rightCircle'
            };
            this.renderer.drawCircle(rightHeaderOptions, this.scrollbar);
           
            this.renderer.append(this.scrollbar, this.scrollsvgObj[scrollObj.index]);
            if (scrollObj.orientation == 'horizontal') {
                this.scrollsvgObj[scrollObj.index].setAttribute("height", scrollHeight);
                this.scrollsvgObj[scrollObj.index].setAttribute("width", width);
            } else {
                this.scrollsvgObj[scrollObj.index].setAttribute("height", width);
                this.scrollsvgObj[scrollObj.index].setAttribute("width", scrollHeight);
            }
            this.renderer.append(this.scrollsvgObj[scrollObj.index], scrollObj.parent);

            //Apply CSS styles to scrollbar controls          
            var vmlPadding = !this.vmlRendering ? scrollObj.y : scrollObj.y + 5;
            if (this.pluginName == "ejRangeNavigator")
                this.scrollsvgObj[scrollObj.index].setAttribute('style', 'overflow:visible;position:relative;display:block; top:' + (vmlPadding - scrollObj.y) + 'px;' + 'left:' + scrollObj.x + 'px');
            else
                this.scrollsvgObj[scrollObj.index].setAttribute('style', 'overflow:visible;position:absolute;display:block; top:' + vmlPadding + 'px;' + 'left:' + scrollObj.x + 'px');
            $("#" + id + "_scrollbarSelect_" + scrollObj.index).css({ "cursor": "pointer" });
            if (scrollObj.enableResize) {
                $("#" + id + "_leftHeaderHideRect_" + scrollObj.index).css({ "cursor": scrollObj.orientation == 'horizontal' ? "w-resize" : "n-resize" });
                $("#" + id + "_rightHeaderHideRect_" + scrollObj.index).css({ "cursor": scrollObj.orientation == 'horizontal' ? "w-resize" : "n-resize" });
                $("#" + id + "_scrollbarRightHeader_" + scrollObj.index).css({ "cursor": scrollObj.orientation == 'horizontal' ? "w-resize" : "n-resize" });
                $("#" + id + "_scrollbarLeftHeader_" + scrollObj.index).css({ "cursor": scrollObj.orientation == 'horizontal' ? "w-resize" : "n-resize" });
            } else {
                $("#" + id + "_leftHeaderHideRect_" + scrollObj.index).hide();
                $("#" + id + "_rightHeaderHideRect_" + scrollObj.index).hide();
                $("#" + id + '_scrollbarLeftHeader_' + scrollObj.index).hide();
                $("#" + id + '_scrollbarRightHeader_' + scrollObj.index).hide();
            }

            if (this.vmlRendering) {
                $('#' + id + '_leftHeaderHideRect_' + scrollObj.index).css("visibility", 'hidden');
                $('#' + id + '_rightHeaderHideRect_' + scrollObj.index).css("visibility", 'hidden');
            }
            
            this.scrollbarContainer._bindScrollEvents.call(this, scrollObj);
        },

        _bindScrollEvents: function (scrollObj) {
            //Binding Mouse Events to scrollbar
            var rootId = this._id;
            var matched = jQuery.uaMatch(navigator.userAgent);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);           
            bindDesktopEvents.call(this, scrollObj);
            if (window.PointerEvent) { //Added pointer event for IE11

                this._on($('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index), "pointerdown", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index), "pointerdown", this.scrollbarContainer._rightScrollbarDown);

                this._on($('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index), "pointerdown", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index), "pointerdown", this.scrollbarContainer._rightScrollbarDown);
                
                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "pointerdown", this.scrollbarContainer._leftArrowDown);
                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "pointerup", this.scrollbarContainer._leftArrowUp);

                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "pointerdown", this.scrollbarContainer._rightArrowDown);
                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "pointerup", this.scrollbarContainer._rightArrowUp);

                this._on($('#' + rootId + '_scrollbarSelect_' + scrollObj.index), "pointerdown", this.scrollbarContainer._scrollSelectRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "pointerdown", this.scrollbarContainer._scrollBackRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "pointerup", this.scrollbarContainer._scrollBackRectUp);

                this._on($(window), "pointermove", this.scrollbarContainer._scrollbarMove);
                this._on($(window), "pointerup", this.scrollbarContainer._scrollbarUp);
                this._on($(this.scrollsvgObj[scrollObj.index]), "pointermove", this.scrollbarContainer._scrollbarMove);
                this._on($(this.scrollsvgObj[scrollObj.index]), "pointerup", this.scrollbarContainer._scrollbarUp);

                $(this.scrollsvgObj[scrollObj.index]).css('touch-action', 'none');
            }
            else if (window.navigator.msPointerEnabled && !isIE11) {

                this._on($('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._rightScrollbarDown);

                this._on($('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._rightScrollbarDown);
                
                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._leftArrowDown);
                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "MSPointerUp", this.scrollbarContainer._leftArrowUp);

                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._rightArrowDown);
                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "MSPointerUp", this.scrollbarContainer._rightArrowUp);

                this._on($('#' + rootId + '_scrollbarSelect_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._scrollSelectRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "MSPointerDown", this.scrollbarContainer._scrollBackRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "MSPointerUp", this.scrollbarContainer._scrollBackRectUp);

                this._on($(window), "MSPointerMove", this.scrollbarContainer._scrollbarMove);
                this._on($(window), "MSPointerUp", this.scrollbarContainer._scrollbarUp);
                this._on($(this.scrollsvgObj[scrollObj.index]), "MSPointerMove", this.scrollbarContainer._scrollbarMove);
                this._on($(this.scrollsvgObj[scrollObj.index]), "MSPointerUp", this.scrollbarContainer._scrollbarUp);

                $(this.scrollsvgObj[scrollObj.index]).css('-ms-touch-action', 'none');

            } else if (matched.browser.toLowerCase() == "chrome") {
                bindTouchEvents.call(this, scrollObj);
            } else if (this.isDevice()) {
                
                var isSafari = (navigator.userAgent.indexOf('Safari') != -1 && navigator.userAgent.indexOf('Chrome') == -1);
                var isInIframe = (window.location != window.parent.location) ? true : false;
                var userAgent = isInIframe ? window.parent.navigator.userAgent.toLowerCase() : window.navigator.userAgent.toLowerCase();
                var device = /mobile|tablet|android|kindle/i.test(userAgent);
                if ((ej.isMobile() && this.isWindows()) || (!device && ej.isMobile() && isSafari)) { // for safari mobile browser and windows phone                    
                    bindDesktopEvents.call(this);
                    $(this.scrollsvgObj[scrollObj.index]).css('-ms-touch-action', 'none');
                } else
                    bindTouchEvents.call(this, scrollObj);
            }
            var eventName = matched.browser.toLowerCase() == "mozilla" ? (isIE11 ? "mousewheel" : "DOMMouseScroll") : "mousewheel";
            this._on($(this.scrollsvgObj[scrollObj.index]), eventName, this.scrollbarContainer._scrollMouseWheel);

            if (isIE11)
                $(this.scrollsvgObj[scrollObj.index]).css('touch-action', 'none');

            function bindDesktopEvents(scrollObj) {
                this._on($('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index), "mousedown", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index), "mousedown", this.scrollbarContainer._rightScrollbarDown);

                this._on($('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index), "mousedown", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index), "mousedown", this.scrollbarContainer._rightScrollbarDown);

                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "mousedown", this.scrollbarContainer._leftArrowDown);
                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "mouseup", this.scrollbarContainer._leftArrowUp);

                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "mousedown", this.scrollbarContainer._rightArrowDown);
                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "mouseup", this.scrollbarContainer._rightArrowUp);

                this._on($('#' + rootId + '_scrollbarSelect_' + scrollObj.index), "mousedown", this.scrollbarContainer._scrollSelectRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "mousedown", this.scrollbarContainer._scrollBackRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "mouseup", this.scrollbarContainer._scrollBackRectUp);

                this._on($(window), "mousemove", this.scrollbarContainer._scrollbarMove);
                this._on($(window), "mouseup", this.scrollbarContainer._scrollbarUp);
                this._on($(this.svgObject), "mousemove", this.scrollbarContainer._scrollbarMove);
                this._on($(this.scrollsvgObj[scrollObj.index]), "mousemove", this.scrollbarContainer._scrollbarMove);
                this._on($(this.scrollsvgObj[scrollObj.index]), "mouseup", this.scrollbarContainer._scrollbarUp);
            }

            function bindTouchEvents(scrollObj) {
                this._on($('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index), "touchstart", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index), "touchstart", this.scrollbarContainer._rightScrollbarDown);

                this._on($('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index), "touchstart", this.scrollbarContainer._leftScrollbarDown);
                this._on($('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index), "touchstart", this.scrollbarContainer._rightScrollbarDown);

                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "touchstart", this.scrollbarContainer._leftArrowDown);
                this._on($('#' + rootId + '_scrollbarLeftArrow_' + scrollObj.index), "touchend", this.scrollbarContainer._leftArrowUp);

                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "touchstart", this.scrollbarContainer._rightArrowDown);
                this._on($('#' + rootId + '_scrollbarRightArrow_' + scrollObj.index), "touchend", this.scrollbarContainer._rightArrowUp);

                this._on($('#' + rootId + '_scrollbarSelect_' + scrollObj.index), "touchstart", this.scrollbarContainer._scrollSelectRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "touchstart", this.scrollbarContainer._scrollBackRectDown);
                this._on($('#' + rootId + '_scrollbarBackRect_' + scrollObj.index), "touchend", this.scrollbarContainer._scrollBackRectUp);

                this._on($(window), "touchmove", this.scrollbarContainer._scrollbarMove);
                this._on($(window), "touchend", this.scrollbarContainer._scrollbarUp);
                this._on($(this.scrollsvgObj[scrollObj.index]), "touchmove", this.scrollbarContainer._scrollbarMove);
                this._on($(this.scrollsvgObj[scrollObj.index]), "touchend", this.scrollbarContainer._scrollbarUp);

            }
        },

        _setScrollPosition: function (startX, offsetRightX, scrollObj) {
            var padding = 8;
            var rootId = this._id;
            var centerLine = (startX + (scrollObj.rectWidth / 2) - 7.5);
            $('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index).attr("cx", startX + padding);
            $('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index).attr("x", startX);
            
            $('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index).attr("cx", offsetRightX - padding);
            $('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index).attr("x", offsetRightX - 5);
            $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).attr("x", startX);
            $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).attr("width", scrollObj.rectWidth);
            $('#' + rootId + '_scrollbarCenterLine_' + scrollObj.index).attr('transform', 'translate(' + centerLine + ')');
            //Apply styles for VML support
            if (this.pluginName == "ejRangeNavigator") {
                $('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index).css("left", startX);
                $('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index).css("left", startX);
                $('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index).css("left", offsetRightX - padding);
                $('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index).css("left", offsetRightX - 5);
                $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).css("left", startX);
                $('#' + rootId + '_scrollbarCenterLine_' + scrollObj.index).css('left', centerLine);
            }
        },
        _calculateScrollLeftMove: function (moveLength, scrollObj) {

            if (scrollObj._startX - moveLength > 15) {
                scrollObj.startX = scrollObj._startX - moveLength;
                scrollObj.offsetLeftX = scrollObj.startX;
                scrollObj._offsetRightX = scrollObj.startX + scrollObj.rectWidth;
                scrollObj.offsetRightX = scrollObj._offsetRightX;
                scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            } else {
                scrollObj.startX = 15;
                scrollObj._offsetLeftX = scrollObj.offsetLeftX;
                scrollObj.offsetLeftX = scrollObj.startX;
                scrollObj._offsetRightX = scrollObj.startX + scrollObj.rectWidth;
                scrollObj.offsetRightX = scrollObj._offsetRightX;
                scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            }
        },
        _calculateScrollRightMove: function (moveLength, scrollObj) {
            if (scrollObj && (scrollObj._offsetRightX + (Math.abs(moveLength))) < scrollObj.width - 15) {
                scrollObj._startX = scrollObj.startX + Math.abs(moveLength);
                scrollObj.offsetLeftX = scrollObj._startX;
                scrollObj.offsetRightX = scrollObj._offsetRightX + Math.abs(moveLength);
                scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            } else {
                scrollObj._ofsetRightX = scrollObj.offsetRightX;
                scrollObj.offsetRightX = scrollObj.width - 15;
                scrollObj._startX = scrollObj.offsetRightX - scrollObj.rectWidth;
                scrollObj.offsetLeftX = scrollObj._startX;
                scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
            }
        },
        _calculateRange: function (startX, endX, scrollObj) {
            
            var zoomPos, zoomFactor, scrollRange, start, end, padding = 30,startArgs,endArgs;

            //Modify scrollbar center rectangle position on mouse move
            zoomPos = (startX - 15) / (scrollObj.width - padding - scrollObj._diff);
            scrollRange = scrollObj.scrollRange;
            zoomFactor = ((scrollObj.rectWidth - scrollObj._diff) / (scrollObj.width - padding - scrollObj._diff));
            if (!scrollObj.isRTL) {
                start = scrollRange.min + zoomPos * scrollRange.delta;
                end = start + zoomFactor * scrollRange.delta;
            } else {
                end = scrollRange.max - zoomPos * scrollRange.delta;
                start = end - zoomFactor * scrollRange.delta;
            }
            if (scrollObj.valueType == "datetime") {
                startArgs = scrollObj.startDateTime;
                endArgs = scrollObj.endDateTime;
                scrollObj.startDateTime = this.startDateTime = start = new Date(start);
                scrollObj.endDateTime = this.endDateTime = end = new Date(end);
            } else {
                startArgs = parseInt(scrollObj.startValue);
                endArgs = parseInt(scrollObj.endValue);
                scrollObj.startValue = this.startValue = start = Math.ceil(start);
                scrollObj.endValue = this.endValue = end = Math.ceil(end);
            }

            //Bind event on changing scrollbar position 
                scrollObj._scrollChanged = true;
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = {
                zoomPosition: zoomPos, zoomFactor: zoomFactor,
                oldRange: {
                    start: startArgs,
                    end: endArgs
                },
                newRange: {
                    start: start,
                    end: end
                }
            };
            
            scrollObj.zoomPosition = zoomPos;
            scrollObj.zoomFactor = zoomFactor;

            return commonEventArgs;
        },
        _calculateScrollPosition: function (mouseX, scrollObj) {
            var zoomPos, zoomFact, scrollRange, start, end, startArgs, endArgs, width,
                padding = 30, offset = 15, valueType, startRange, minScrollWidth = 36;

            if (this.leftScrollbarClicked && scrollObj.enableResize) {
                mouseX = mouseX >= offset ? mouseX : offset;
                mouseX = (mouseX >= offset && mouseX < scrollObj.offsetRightX - offset) ? mouseX : scrollObj.offsetRightX - offset;
                zoomPos = (mouseX - minScrollWidth) / (scrollObj.width - padding);
                zoomFact = scrollObj.rectWidth / (scrollObj.width - padding);
                
                    valueType = scrollObj.valueType.toLowerCase();
                    scrollRange = scrollObj.scrollRange;
                    if (!scrollObj.isRTL) {
                        start = scrollRange.min + zoomPos * scrollRange.delta;
                        start = start > scrollRange.min ? start : scrollRange.min;
                        if (valueType == "datetime"){
                            startArgs = scrollObj.startDateTime;
                            endArgs = scrollObj.endDateTime;
                            scrollObj.startDateTime = this.startDateTime = start = new Date(start), end = scrollObj.endDateTime;
                        }
                        else {
                            startArgs =  parseInt(scrollObj.startValue);
                            endArgs = parseInt(scrollObj.endValue);
                            scrollObj.startValue = this.startValue = start = Math.ceil(start), end = Math.ceil(scrollObj.endValue);
                        }
                    } else {
                        end = scrollRange.max - zoomPos * scrollRange.delta;
                        end = end < scrollRange.max ? end : scrollRange.max;
                        if (valueType == "datetime") {
                            startArgs = scrollObj.startDateTime;
                            endArgs = scrollObj.endDateTime;
                            scrollObj.endDateTime = this.endDateTime = end = new Date(end), start = scrollObj.startDateTime;
                        }
                        else {
                            startArgs = parseInt(scrollObj.startValue);
                            endArgs = parseInt(scrollObj.endValue);
                            scrollObj.endValue = this.endValue = end = Math.ceil(end), start = Math.ceil(scrollObj.startValue);
                        }
                    }
                //Bind event on changing scrollbar position 
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = {
                    zoomPosition:zoomPos, zoomFactor: zoomFact,
                    oldRange: {
                        start: startArgs,
                        end: endArgs
                    },
                    newRange: {
                        start: start,
                        end: end
                    }
                };
                scrollObj._scrollChanged = true;
                scrollObj.zoomPosition = zoomPos;
                scrollObj.zoomFactor = zoomFact;

                return commonEventArgs;                
            }
            if (this.rightScrollbarClicked && scrollObj.enableResize) {
                scrollRange = scrollObj.scrollRange;
                zoomFact = scrollObj.rectWidth / (scrollObj.width - padding);
                zoomPos = scrollObj.zoomPosition;
                valueType = scrollObj.valueType.toLowerCase();
                if (!scrollObj.isRTL) {
                    startRange = scrollRange.min + zoomPos * scrollRange.delta;
                    end = startRange + zoomFact * scrollRange.delta;
                    if (valueType == "datetime") {
                        startArgs = scrollObj.startDateTime;
                        endArgs = scrollObj.endDateTime;
                        scrollObj.endDateTime = this.endDateTime = end = new Date(end), start = scrollObj.startDateTime;
                    }
                    else {
                        startArgs = parseInt(scrollObj.startValue);
                        endArgs = parseInt(scrollObj.endValue);
                        scrollObj.endValue = this.endValue = end = Math.floor(end), start = Math.ceil(scrollObj.startValue);
                    }
                } else {
                    zoomPos = (mouseX - minScrollWidth) / (scrollObj.width - padding);
                    start = scrollRange.max - zoomPos * scrollRange.delta;
                    if (valueType == "datetime") {
                        startArgs = scrollObj.startDateTime;
                        endArgs = scrollObj.endDateTime;
                        scrollObj.startDateTime = this.startDateTime = start = new Date(start), end = scrollObj.endDateTime;
                    }
                    else {
                        startArgs = parseInt(scrollObj.startValue);
                        endArgs = parseInt(scrollObj.endValue);
                        scrollObj.startValue = this.startValue = start = Math.floor(start), end = Math.ceil(scrollObj.endValue);
                    }
                }

                //Bind event on changing scrollbar position 
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = {
                    zoomPosition: scrollObj.zoomPosition ? scrollObj.zoomPosition : 0, zoomFactor: zoomFact,
                    oldRange: {
                        start: startArgs,
                        end: endArgs
                    },
                    newRange: {
                        start: start,
                        end: end
                    }
                };
                scrollObj._scrollChanged = true;
                scrollObj.zoomFactor = zoomFact;

                return commonEventArgs;
            }
            if (scrollObj && this.scrollRectClicked) {
                var moveLength = this.mouseDownPos - mouseX;
                if (moveLength > 0 && scrollObj.offsetLeftX >= 0) {
                    this.scrollbarContainer._calculateScrollLeftMove.call(this, moveLength, scrollObj);
                    return this.scrollbarContainer._calculateRange.call(this, scrollObj.startX, scrollObj.offsetRightX, scrollObj);
                }
                if (moveLength < 0) {
                    this.scrollbarContainer._calculateScrollRightMove.call(this, moveLength, scrollObj);
                    return this.scrollbarContainer._calculateRange.call(this, scrollObj._startX, scrollObj.offsetRightX, scrollObj);
                }
            }
        },

        _scrollbarMove: function (evt) {
          
            this.moveClientX = !ej.isNullOrUndefined(evt.clientX) ? evt.clientX : (evt.originalEvent.clientX == null ? evt.originalEvent.touches[0].clientX : evt.originalEvent.clientX);
            if (this.moveClientX != this.downClientX) {//update scrollbar only on mouse move
                var zoomPos, axis, start, end, width, padding = 8, minScrollWidth = 34, mouseX, centerLine;
                var rangePadding = this.model.padding || 0;
                var matched = jQuery.uaMatch(navigator.userAgent);
                var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
                if (!scrollObj) return 0;

                this.lastIndex = scrollObj.index;
                if (((!this.vmlRendering && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") || (this.vmlRendering))
                    mouseX = (scrollObj.orientation == 'horizontal' ? this.calMousePosition(evt).X - scrollObj.x : scrollObj.width - (this.calMousePosition(evt).Y - scrollObj.y)) - rangePadding;
                else
                    mouseX = (scrollObj.orientation == 'horizontal' ? this.calTouchPosition(evt).X - scrollObj.x : scrollObj.width - (this.calTouchPosition(evt).Y - scrollObj.y)) - rangePadding;

                this.mouseX = mouseX;
                var rootId = this._id;
                if (this.leftScrollbarClicked && scrollObj.enableResize) {
                  

                    if (scrollObj && (mouseX > minScrollWidth || mouseX > 23) && mouseX < scrollObj.offsetRightX - minScrollWidth) {
                        scrollObj.rectWidth = scrollObj.width - (scrollObj.endX + mouseX - padding);
                        centerLine = ((mouseX - padding) + (scrollObj.rectWidth / 2) - 7.5);
                        $('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index).attr("cx", mouseX);
                        $('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index).attr("x", mouseX - 7.5);
                        $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).attr('x', mouseX - padding);
                        $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).attr("width", scrollObj.rectWidth);
                        $('#' + rootId + '_scrollbarCenterLine_' + scrollObj.index).attr('transform', 'translate(' + centerLine + ')');
                        //Apply styles for VML support
                        if (this.pluginName == "ejRangeNavigator") {
                            $('#' + rootId + '_scrollbarLeftHeader_' + scrollObj.index).css("left", mouseX);
                            $('#' + rootId + '_leftHeaderHideRect_' + scrollObj.index).css("left", mouseX - 7.5);
                            $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).css('left', mouseX);
                            $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).css("width", scrollObj.rectWidth);
                            $('#' + rootId + '_scrollbarCenterLine_' + scrollObj.index).css('left', centerLine);
                        }
                        scrollObj.offsetLeftX = scrollObj.startX = mouseX - 8;
                        this.scrollbarContainer._scrollStartArgs.call(this, scrollObj);
                        this.scrollbarContainer._scrollChangeArgs.call(this, scrollObj);
                        scrollObj._diff = 0;
                    } else if (mouseX > scrollObj.offsetRightX - minScrollWidth)
                        this.rectWidth = 42.89984101748814;
                }
                if (this.rightScrollbarClicked && scrollObj.enableResize) {
                    if (scrollObj && (mouseX <= scrollObj.width - minScrollWidth || mouseX <= scrollObj.width - 23) && mouseX > scrollObj.offsetLeftX + minScrollWidth) {
                        scrollObj.rectWidth = mouseX - scrollObj.offsetLeftX + padding;
                        centerLine = (scrollObj.offsetLeftX + (scrollObj.rectWidth / 2) - 7.5);
                        $('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index).attr("cx", mouseX);
                        $('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index).attr("x", mouseX - 2.5);
                        $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).attr("width", scrollObj.rectWidth);
                        $('#' + rootId + '_scrollbarCenterLine_' + scrollObj.index).attr('transform', 'translate(' + centerLine + ')');
                        //Apply styles for VML support
                        if (this.pluginName == "ejRangeNavigator") {
                            $('#' + rootId + '_scrollbarRightHeader_' + scrollObj.index).css("left", mouseX);
                            $('#' + rootId + '_rightHeaderHideRect_' + scrollObj.index).css("left", mouseX - 2.5);
                            $('#' + rootId + '_scrollbarSelect_' + scrollObj.index).css("width", scrollObj.rectWidth);
                            $('#' + rootId + '_scrollbarCenterLine_' + scrollObj.index).css('left', centerLine);
                        }
                        scrollObj.offsetRightX = mouseX + padding;
                        scrollObj.endX = scrollObj.width - mouseX - padding;
                       // this.scrollbarContainer._calculateScrollPosition.call(this, mouseX, scrollObj);
                        this.scrollbarContainer._scrollStartArgs.call(this, scrollObj);
                        this.scrollbarContainer._scrollChangeArgs.call(this, scrollObj);
                        scrollObj._diff = 0;
                    }

                }
                if (this.scrollRectClicked) {
                    var moveLength = this.mouseDownPos - mouseX;
                    scrollObj._startX =  ej.isNullOrUndefined(scrollObj._startX) ? scrollObj.startX : scrollObj._startX;
                    if (scrollObj && moveLength > 0 && scrollObj.offsetLeftX >= 0) {
                        this.scrollbarContainer._calculateScrollLeftMove.call(this, moveLength, scrollObj);
                        this.scrollbarContainer._setScrollPosition.call(this, scrollObj.startX, scrollObj.offsetRightX, scrollObj);
                        if (scrollObj.offsetLeftX > 15 || scrollObj.offsetLeftX != scrollObj._offsetLeftX) {
                            this.scrollbarContainer._scrollStartArgs.call(this, scrollObj);
                            this.scrollbarContainer._scrollChangeArgs.call(this, scrollObj);
                        }
                    }

                    if (scrollObj && moveLength < 0) {
                        if (ej.isNullOrUndefined(scrollObj._offsetRightX)) {
                            this.scrollbarContainer._scrollSelectRectDown.call(this, evt);
                            this.scrollbarContainer._scrollbarMove.call(this, evt);
                        }
                        this.scrollbarContainer._calculateScrollRightMove.call(this, moveLength, scrollObj);
                        this.scrollbarContainer._setScrollPosition.call(this, scrollObj._startX, (scrollObj._startX + scrollObj.rectWidth), scrollObj);
                        if (scrollObj.offsetRightX < scrollObj.width - 15 || scrollObj.offsetRightX != scrollObj._ofsetRightX) {
                            this.scrollbarContainer._scrollStartArgs.call(this, scrollObj);
                            if (this.model.enableDeferredUpdate === false || scrollObj.isZooming || this.model.scrollChanged != '') {
                                var commonEventArgs = this.scrollbarContainer._calculateRange.call(this, scrollObj._startX, (scrollObj._startX + scrollObj.rectWidth), scrollObj);
                                var oldRange = commonEventArgs.data.oldRange,
                                     newRange = commonEventArgs.data.newRange, oldStart, oldEnd, newStart, newEnd;
                                if (scrollObj.valueType.toLowerCase() == 'datetime') {
                                    oldStart = Date.parse(oldRange.start), newStart = Date.parse(newRange.start),
                                    oldEnd = Date.parse(oldRange.end), newEnd = Date.parse(newRange.end);
                                } else {
                                    oldStart = oldRange.start, newStart = newRange.start,
                                    oldEnd = oldRange.end, newEnd = newRange.end;
                                }
                                if (this.model.scrollChanged != ''  && ((oldStart != newStart) || (oldEnd != newEnd))) {
                                    commonEventArgs = this.scrollbarContainer._args.call(this, scrollObj, commonEventArgs);
                                    this._trigger("scrollChanged", commonEventArgs);
                                }
                                if(!this.model.enableDeferredUpdate)
                                    this.scrollUpdate ? this.scrollUpdate = false : this.scrollbarContainer._appendScrollRange.call(this,scrollObj);
                            }
                        }
                    }
                }
            }
            this.offsetX = mouseX;
        },
        _scrollbarUp: function (evt) {
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            $("[id^=" + this._id + '_scrollbarSelect_' + "]").attr('class', 'e-rangeScroll-select');
            this.lastIndex = null;
            var args = this.scrollbarContainer._calculateScrollPosition.call(this, this.mouseX, scrollObj),
                deferred = this.model.enableDeferredUpdate;
            if (args) {
                var oldRange = args.data.oldRange,
                    newRange = args.data.newRange,oldStart, oldEnd, newStart, newEnd;
                if (scrollObj.valueType.toLowerCase() == 'datetime') {
                    oldStart = Date.parse(oldRange.start), newStart = Date.parse(newRange.start),
                    oldEnd = Date.parse(oldRange.end), newEnd = Date.parse(newRange.end);
                } else {
                    oldStart = oldRange.start, newStart = newRange.start,
                    oldEnd = oldRange.end,newEnd = newRange.end;
                }

                if (((oldStart != newStart) || (oldEnd != newEnd)) || (this.scrollRectClicked && !deferred) || (deferred) || (this.leftScrollbarClicked || this.rightScrollbarClicked)) {
                    //Checked condition for chrome browser mouse click issue.. on mouseClick in chrome move event also triggered
                    var is_chrome = ((navigator.userAgent.toLowerCase().indexOf('chrome') > -1) && (navigator.vendor.toLowerCase().indexOf("google") > -1));
                    if (!is_chrome || (is_chrome && ((this.leftScrollbarClicked || this.rightScrollbarClicked) && this.downClientX != this.moveClientX)
                        || this.scrollRectClicked || this.leftArrowClicked || this.rightArrowClicked)) {
                        //event will no trigger when rect value is not changed on left and right header moved
                        if ((!this.leftScrollbarClicked && !this.rightScrollbarClicked) ||
                            ((this.leftScrollbarClicked || this.rightScrollbarClicked) && Math.ceil(scrollObj.rectWidth) != Math.ceil(this.rectOldWidth) && (Math.abs(scrollObj.rectWidth - scrollObj.rectOldWidth) > 4 || !scrollObj.rectOldWidth))) {
                            if (ej.isNullOrUndefined(scrollObj._previousStart) || ((scrollObj._previousStart != newStart) || (scrollObj._previousEnd != newEnd))) {
                                args = this.scrollbarContainer._args.call(this, scrollObj, args);
                                this._trigger("scrollEnd", args);
                                scrollObj._scrollStarted = false;
                                scrollObj._previousStart = newStart;
                                scrollObj._previousEnd = newEnd;
                                scrollObj.rectOldWidth = scrollObj.rectWidth;
                                this.scrollUpdate ? this.scrollUpdate = false : this.scrollbarContainer._appendScrollRange.call(this, scrollObj);
                            }
                        }
                    }
                }
                scrollObj.release = true;
                scrollObj.clicked = false;
            }
            this._scrollEnd = false;
            this.leftScrollbarClicked = false;
            this.rightScrollbarClicked = false;
            this.scrollRectClicked = false;
            this.leftArrowClicked = false;
            this.rightArrowClicked = false;
            this.scrollbarBackRectClicked = false;
        },
        _leftScrollbarDown: function (evt) {
            evt.preventDefault();
            this.leftScrollbarClicked = true;
            this.downClientX = !ej.isNullOrUndefined(evt.clientX) ? evt.clientX : evt.originalEvent.touches[0].clientX;
        },       

        _rightScrollbarDown: function (evt) {
            evt.preventDefault();
            this.rightScrollbarClicked = true;
            this.downClientX = !ej.isNullOrUndefined(evt.clientX) ? evt.clientX : evt.originalEvent.touches[0].clientX;
        },

        _scrollSelectRectDown: function (evt) {
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            $('#' + this._id + '_scrollbarSelect_' + scrollObj.index).attr('class', 'e-rangeScroll-select e-rangeScroll-select-hover');
            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var rangePadding = this.model.padding || 0;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            if (((!this.vmlRendering && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") || (this.vmlRendering))
                var mouseX = (scrollObj.orientation == 'horizontal' ? this.calMousePosition(evt).X - scrollObj.x : scrollObj.width - (this.calMousePosition(evt).Y - scrollObj.y)) - rangePadding;
            else
                var mouseX = (scrollObj.orientation == 'horizontal' ? this.calTouchPosition(evt).X - scrollObj.x : scrollObj.width - (this.calTouchPosition(evt).Y - scrollObj.y)) - rangePadding;
            this.mouseDownPos = mouseX;
            scrollObj._startX = scrollObj.startX = scrollObj.offsetLeftX;
            scrollObj._offsetRightX = scrollObj.offsetRightX;
            scrollObj.clicked = true;
            scrollObj.release = false;
            this.scrollRectClicked = true;
        },

        //MouseWheel Scrolling event for scrollbarControl
        _scrollMouseWheel: function (e) {
            e.preventDefault();
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, e);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            var wheelDelta = e.originalEvent.wheelDelta;
            var matched = jQuery.uaMatch(navigator.userAgent);
            var direction = matched.browser.toLowerCase() == "mozilla" ? ((isIE11 ? ((wheelDelta / 120) > 0 ? 1 : -1) : -(e.originalEvent.detail) / 3 > 0 ? 1 : -1)) : ((wheelDelta / 120) > 0 ? 1 : -1);
            var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(scrollObj.zoomFactor, 0, 1), 1);
            var cumulativeScale = Math.max(currentScale + (0.25 * direction), 1);
            this.scrollbarContainer.doMouseWheelZoom.call(this, cumulativeScale, 0.5, scrollObj);
            
        },

        _scrollStartArgs: function (scrollObj) {
            scrollObj._scrollStarted = true;
            if (this.model.scrollStart != '' && !this._scrollEnd) {
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                if (!scrollObj.isZooming) {
                    commonEventArgs.data = {
                        startRange: scrollObj.valueType == "datetime" ? scrollObj.startDateTime : scrollObj.startValue,
                        endRange: scrollObj.valueType == "datetime" ? scrollObj.endDateTime : scrollObj.endValue
                    };
                } else {
                    commonEventArgs.data = {
                        axis :this.model._axes[scrollObj.index],
                        currentRange: this.model._axes[scrollObj.index].visibleRange
                    };
                }
                this._trigger("scrollStart", commonEventArgs);
                this._scrollEnd = true;
            }
        },

        _scrollChangeArgs: function (scrollObj) {
            if (this.model.enableDeferredUpdate === false || scrollObj.isZooming || this.model.scrollChanged != '') {
                var commonEventArgs = this.scrollbarContainer._calculateRange.call(this, scrollObj.offsetLeftX, scrollObj.offsetRightX, scrollObj);
                var oldRange = commonEventArgs.data.oldRange,
                    newRange = commonEventArgs.data.newRange, oldStart, oldEnd, newStart, newEnd;
                if (scrollObj.valueType.toLowerCase() == 'datetime') {
                    oldStart = Date.parse(oldRange.start), newStart = Date.parse(newRange.start),
                    oldEnd = Date.parse(oldRange.end), newEnd = Date.parse(newRange.end);
                } else {
                    oldStart = oldRange.start, newStart = newRange.start,
                    oldEnd = oldRange.end, newEnd = newRange.end;
                }
                
                if (this.model.scrollChanged != '' && ((oldStart != newStart) || (oldEnd != newEnd))) {
                    commonEventArgs = this.scrollbarContainer._args.call(this, scrollObj, commonEventArgs);
                    this._trigger("scrollChanged", commonEventArgs);
                }
                if (!this.model.enableDeferredUpdate)
                    this.scrollUpdate ? this.scrollUpdate = false : this.scrollbarContainer._appendScrollRange.call(this, scrollObj);
            }
        },

        _scrollEndArgs: function (startX, scrollObj) {
            var args = this.scrollbarContainer._calculateRange.call(this, startX, scrollObj.offsetRightX, scrollObj);
            if (args && scrollObj._scrollStarted) {
                var oldRange = args.data.oldRange,
                    newRange = args.data.newRange, oldStart, oldEnd, newStart, newEnd;
                if (scrollObj.valueType.toLowerCase() == 'datetime') {
                    oldStart = Date.parse(oldRange.start), newStart = Date.parse(newRange.start),
                    oldEnd = Date.parse(oldRange.end), newEnd = Date.parse(newRange.end);
                } else {
                    oldStart = oldRange.start, newStart = newRange.start,
                    oldEnd = oldRange.end, newEnd = newRange.end;
                }

                if (((oldStart != newStart) || (oldEnd != newEnd)) || (this.leftScrollbarClicked || this.rightScrollbarClicked)) {
                    args = this.scrollbarContainer._args.call(this, scrollObj,args);
                    this._trigger("scrollEnd", args);
                }
                scrollObj._scrollStarted = false;
                this.scrollUpdate ? this.scrollUpdate = false : this.scrollbarContainer._appendScrollRange.call(this, scrollObj);
            }
        },

        _args: function (scrollObj, args) {
            if (scrollObj.isZooming) {
                var axis = this.model._axes[scrollObj.index],
                    data = args.data;
                data.axis = axis;
                data.newRange = axis.visibleRange;
                data.oldRange = axis.previousRange;
            }

            if (scrollObj.isVirtual) {
                args.data.axis = this.model._axes[scrollObj.index];
                delete args.data.zoomFactor;
                delete args.data.zoomPosition;
            }
            return args;
        },

        doMouseWheelZoom: function (cumulativeScale, origin, scrollObj) {
            if (cumulativeScale >= 1) {
                var calZoomVal = this.scrollbarContainer.calZoomFactors(cumulativeScale, origin, scrollObj.zoomFactor, scrollObj.zoomPosition);
                
                if (scrollObj.zoomPosition != calZoomVal.zoomMPosition && calZoomVal.zoomMPosition + scrollObj.zoomFactor <= 1 && scrollObj.zoomPosition >= 0) {
                    if (calZoomVal.zoomMPosition + scrollObj.zoomFactor >= 1) {
                        calZoomVal.zoomMPosition = 1 - scrollObj.zoomFactor
                    }
                    if (calZoomVal.zoomMPosition < 0) {
                        calZoomVal.zoomMPosition = 0;
                    }
                    scrollObj.zoomPosition = calZoomVal.zoomMPosition;
                    scrollObj.startX = ((scrollObj.zoomPosition) * (scrollObj.width - 30 - scrollObj._diff) + 15);
                    scrollObj.startX = (scrollObj.startX + scrollObj.rectWidth + 5) > scrollObj.width ? (scrollObj.width - scrollObj.rectWidth - 15) : scrollObj.startX;
                    scrollObj.offsetLeftX = scrollObj.startX;
                    
                    scrollObj._offsetRightX = scrollObj.startX + scrollObj.rectWidth;
                    if (scrollObj._offsetRightX > scrollObj.width - 15) {
                        scrollObj.offsetRightX = scrollObj.width - 15;
                        scrollObj.startX = scrollObj.offsetRightX - scrollObj.rectWidth;
                        scrollObj.offsetLeftX = scrollObj.startX;
                    } else {
                        scrollObj.offsetRightX = scrollObj._offsetRightX;
                    }
                    scrollObj.endX = scrollObj.width - scrollObj.offsetRightX;
                    this.scrollbarContainer._setScrollPosition.call(this, scrollObj.startX, scrollObj.offsetRightX, scrollObj);
                    this.scrollbarContainer._calculateMouseWheelRange.call(this, scrollObj.startX, scrollObj.offsetRightX, scrollObj);
                }
            }
            return false;
        },
        _calculateMouseWheelRange: function (startX, endX, scrollObj) {

            var scrollRange, start, end, padding = 30,startArgs,endArgs;
            scrollObj.zoomFactor = (scrollObj.rectWidth / (scrollObj.width - padding));
            scrollObj.zoomPosition = (startX - 15) / (scrollObj.width - padding - scrollObj._diff);
            scrollRange = scrollObj.scrollRange;

            if (!scrollObj.isRTL) {
                start = scrollRange.min + scrollObj.zoomPosition * scrollRange.delta;
                end = start + scrollObj.zoomFactor * scrollRange.delta;
            } else {
                end = scrollRange.max - scrollObj.zoomPosition * scrollRange.delta;
                start = end - scrollObj.zoomFactor * scrollRange.delta;
            }

            if (scrollObj.valueType == "datetime") {
                startArgs = scrollObj.startDateTime;
                endArgs = scrollObj.endDateTime;
                scrollObj.startDateTime = this.startDateTime = start = new Date(start);
                scrollObj.endDateTime = this.endDateTime = end = new Date(end);
            } else {
                startArgs = parseInt(scrollObj.startValue);
                endArgs = parseInt(scrollObj.endValue);
                scrollObj.startValue = this.startValue = start = Math.ceil(start);
                scrollObj.endValue = this.endValue = end = Math.ceil(end);
            }

            //Bind event on changing scrollbar position 
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = {
                zoomPosition: scrollObj.zoomPosition, zoomFactor: scrollObj.zoomFactor,
                oldRange: {
                    start: startArgs,
                    end: endArgs
                },
                newRange: {
                    start: start,
                    end: end
                }
            };
            scrollObj._scrollChanged = true;
            scrollObj._scrollStarted = false;
            var oldRange = commonEventArgs.data.oldRange,
                    newRange = commonEventArgs.data.newRange, oldStart, oldEnd, newStart, newEnd;
            if (scrollObj.valueType.toLowerCase() == 'datetime') {
                oldStart = Date.parse(oldRange.start), newStart = Date.parse(newRange.start),
                oldEnd = Date.parse(oldRange.end), newEnd = Date.parse(newRange.end);
            } else {
                oldStart = oldRange.start, newStart = newRange.start,
                oldEnd = oldRange.end, newEnd = newRange.end;
            }
            if (oldStart != newStart || oldEnd != newEnd || scrollObj.isZooming) {
                this.scrollbarContainer._scrollStartArgs.call(this, scrollObj);
                commonEventArgs = this.scrollbarContainer._args.call(this, scrollObj, commonEventArgs);
                this._trigger("scrollEnd", commonEventArgs);
                this.scrollUpdate ? this.scrollUpdate = false : this.scrollbarContainer._appendScrollRange.call(this, scrollObj);
                this._scrollEnd = false;
            }
        },
        calZoomFactors: function (cumulativeScale, origin, currentZoomFactor, currentZoomPos) {
            var calcZoomFactorVal, calcZoomPosition;
            if (cumulativeScale == 1) {
                calcZoomFactorVal = 1;
                calcZoomPosition = 0;
            } else {
                calcZoomFactorVal = ej.EjSvgRender.utils._minMax(1 / cumulativeScale, 0, 1);
                calcZoomPosition = currentZoomPos + ((currentZoomFactor - calcZoomFactorVal) * origin);
            }
            return { zoomMFactor: calcZoomFactorVal, zoomMPosition: calcZoomPosition };
        },
        _leftArrowDown: function (evt) {
            this.leftArrowClicked = true;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            scrollObj._startX = scrollObj.startX = scrollObj.offsetLeftX;
            scrollObj._offsetRightX = scrollObj.offsetRightX;
            scrollObj.leftIncrement = 5;
            var range=this;
            scrollObj.interval = setInterval(function () {
                range.scrollbarContainer.leftArrowRecursive.call(range, scrollObj)
            }, 50);
        },

        _leftArrowUp: function (evt) {
            this.leftArrowClicked = false;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            this.scrollbarContainer.leftArrowRecursive.call(this, scrollObj);
        },
      
        leftArrowRecursive: function (scrollObj) {
            if (this.leftArrowClicked) {
                var rootId = this._id;
                this.scrollbarContainer._calculateScrollLeftMove.call(this, scrollObj.leftIncrement, scrollObj);
                scrollObj._scrollStarted = true;
                if (scrollObj.startX > 15)
                    this.scrollbarContainer._scrollEndArgs.call(this, scrollObj.startX, scrollObj);
                this.scrollbarContainer._setScrollPosition.call(this, scrollObj.startX, scrollObj.offsetRightX, scrollObj);
                scrollObj.leftIncrement += 5;
            } else {
                clearInterval(scrollObj.interval);
                scrollObj.leftIncrement = 0;
            }
        },

        _rightArrowDown: function (evt) {
            this.rightArrowClicked = true;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            scrollObj._startX = scrollObj.startX = scrollObj.offsetLeftX;
            scrollObj._offsetRightX = scrollObj.offsetRightX;
            scrollObj.rightIncrement = -5;
            var range = this;
            scrollObj.interval = setInterval(function () { range.scrollbarContainer.rightArrowRecursive.call(range, scrollObj) }, 50);
        },

        _rightArrowUp: function (evt) {
            this.rightArrowClicked = false;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            this.scrollbarContainer.rightArrowRecursive.call(this, scrollObj);
        },

        rightArrowRecursive: function (scrollObj) {
            if (this.rightArrowClicked) {
                var rootId = this._id;
                this.scrollbarContainer._calculateScrollRightMove.call(this, scrollObj.rightIncrement, scrollObj);
                scrollObj._scrollStarted = true;
                if (scrollObj.offsetRightX < scrollObj.width - 15)
                    this.scrollbarContainer._scrollEndArgs.call(this, scrollObj._startX, scrollObj);
                this.scrollbarContainer._setScrollPosition.call(this, scrollObj._startX, scrollObj.offsetRightX, scrollObj);
                scrollObj.rightIncrement -= 5;
            } else {
                clearInterval(scrollObj.interval);
                scrollObj.rightIncrement = 0;
            }
        },

        _scrollBackRectDown: function (evt) {

            var matched = jQuery.uaMatch(navigator.userAgent);
            var rangePadding = this.model.padding || 0;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            if (((!this.vmlRendering && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") || (this.vmlRendering))
                var mouseX = (scrollObj.orientation == 'horizontal' ? this.calMousePosition(evt).X - scrollObj.x : scrollObj.width - (this.calMousePosition(evt).Y - scrollObj.y)) - rangePadding;
            else
                var mouseX = (scrollObj.orientation == 'horizontal' ? this.calTouchPosition(evt).X - scrollObj.x : scrollObj.width - (this.calTouchPosition(evt).Y - scrollObj.y)) - rangePadding;
            this.mouseDownPos = mouseX;
            this.scrollbarBackRectClicked = true;
            
            var range = this;
            this.scrollbarContainer._scrollStartArgs.call(this, scrollObj);
            var interval = setInterval(function () {
                range.scrollbarContainer.scrollBackRectRecursive.call(range, scrollObj)
                if (!this.scrollbarBackRectClicked)
                    clearInterval(interval);
            }, 50);
        },
        _scrollBackRectUp: function (evt) {
            this.scrollbarBackRectClicked = false;
            var scrollObj = this.scrollbarContainer._doScrollIndex.call(this, evt);
            this.scrollbarContainer.scrollBackRectRecursive.call(this, scrollObj);
        },

        _doScrollIndex: function(evt){
            var matchStr = this._id + '_scrollbar' + '_';
            var parentNodeId = (evt.target.parentNode && evt.target.parentNode.id) ? evt.target.parentNode.id : '';
            var selectionIndex = parentNodeId.indexOf(matchStr) > -1 ? parseInt(parentNodeId.substr(matchStr.length)) : NaN;
            if (!ej.isNullOrUndefined(this.lastIndex) && !isNaN(this.lastIndex)) {
                if (!this.model.scrollObj[this.lastIndex].release && this.model.scrollObj[this.lastIndex].clicked)
                    selectionIndex = this.lastIndex;
            }
            this.lastIndex = ej.isNullOrUndefined(this.lastIndex) ? selectionIndex : this.lastIndex;
            var scrollObj = ej.isNullOrUndefined(this.model.scrollObj[selectionIndex]) ? this.model.scrollObj[this.lastIndex] : this.model.scrollObj[selectionIndex];

            return scrollObj;
        },

        scrollBackRectRecursive: function (scrollObj) {
            if (this.scrollbarBackRectClicked) {
                var moveLength = (10 / 100) * (scrollObj.width - 30);
                scrollObj._startX = scrollObj.startX = scrollObj.offsetLeftX;
                scrollObj._offsetRightX = scrollObj.offsetRightX;
                if (this.mouseDownPos < scrollObj.startX) {
                    moveLength = moveLength < scrollObj.startX ? moveLength : (scrollObj.startX - 16);
                    this.scrollbarContainer._calculateScrollLeftMove.call(this, moveLength, scrollObj);
                    scrollObj._scrollStarted = true;
                    this.scrollbarContainer._scrollEndArgs.call(this, scrollObj.startX, scrollObj);
                    this.scrollbarContainer._setScrollPosition.call(this, scrollObj.startX, scrollObj.offsetRightX, scrollObj);
                } else if (this.mouseDownPos > scrollObj.offsetRightX) {
                    moveLength = (moveLength < scrollObj.endX ? moveLength : (scrollObj.endX - 16)) * -1;
                    this.scrollbarContainer._calculateScrollRightMove.call(this, moveLength, scrollObj);
                    scrollObj._scrollStarted = true;
                    this.scrollbarContainer._scrollEndArgs.call(this, scrollObj._startX, scrollObj);
                    this.scrollbarContainer._setScrollPosition.call(this, scrollObj._startX, scrollObj.offsetRightX, scrollObj);
                } else {
                    scrollObj._scrollStarted = false;
                }
            } else {
                scrollObj._scrollStarted = false;
            }
        },

        //Redraw the RangeNavigator and Chart
        _appendScrollRange: function (scrollObj) {
            this.scrollbarUpdate = true;
            if (this.pluginName == "ejRangeNavigator")
                this.renderNavigator();
            else {
                var index = scrollObj.index,
                    axes = this.model._axes[index];
                if (scrollObj.isZooming && (axes.zoomFactor != scrollObj.zoomFactor || axes.zoomPosition != scrollObj.zoomPosition)) {
                    axes.zoomFactor = scrollObj.zoomFactor;
                    axes.zoomPosition = scrollObj.zoomPosition;
                    this.redraw(true);
                }
            }
            this.scrollbarUpdate = false;
        }

    };
})(jQuery);;
/**
* @fileOverview Plugin for Circular Gauge
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 11.2 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget({ "ejCircularGauge": "ej.datavisualization.CircularGauge"    }, {
        _rootCSS: "e-circulargauge",
        validTags: ["div", "span"],
        _savedPoints: [],
        _labelRadius:0,
        _customLblMaxSize: 0,
      
        defaults:   {
		
			locale: null,
			
			enableGroupSeparator: false,
              
            value: 0,
              
            minimum: 0,
              
            maximum: 100,
              
            radius: 180,
              
            width: 360,   
              
            height: 360,   
              
            frame: {
                  
                frameType: "fullcircle",   
                  
                backgroundImageUrl: null,
                  
                halfCircleFrameStartAngle: 180,
                  
                halfCircleFrameEndAngle: 360
            },
              
            backgroundColor: null,
              
            interiorGradient: null,   
              
            readOnly: true,
              
            enableAnimation: true,
              
            animationSpeed: 500,
              
            theme: "flatlight",
              
            isRadialGradient: false,
              
            enableResize: false,
              
            isResponsive: false,
              
            tooltip: {
                  
                showLabelTooltip: false,
                  
                showCustomLabelTooltip: false,
                  
                templateID: null,
            },
              
            outerCustomLabelPosition: "bottom",
              
            gaugePosition: "center",  
              
            distanceFromCorner: 20,  
              
            rangeZOrder :"rear",            
               
            drawTicks: null,
              
            drawLabels: null,
              
            drawPointers: null,
              
            drawRange: null,
              
            drawCustomLabel: null,
              
            drawIndicators: null,
              
            drawPointerCap: null,
              
            load: null,
              
            renderComplete: null,
              
            mouseClick: null,
              
            mouseClickMove: null,
              
            mouseClickUp: null,

            legendItemRender: null,

            legendItemClick: null,

            rangeMouseMove:null,

            scales: null,

            legend:{

                visible: false,

                toggleVisibility: true,

                border:
                {

                    color: 'transparent',


                    width: 1
                },


                itemPadding: 20,

                shape: 'circle',

                alignment: 'center',

                position: 'bottom',

                itemStyle:
               {
                   height: 9,

                   width: 9,

                   border:
                   {

                       color: 'transparent',

                       width: 1
                   }
               },

                opacity: 1,

                fill: null,


                font:
                {
                    color: null,

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px'
                },

                size:
                    {

                        height: null,

                        width: null
                    }

            },

			 themeProperties: {
                flatlight: {
                    backgroundColor: "#FFFFFF",
                    scales: {
                        pointerCap: {
                            borderColor: "#424242",
                            backgroundColor: "#424242"
                        },
                        backgroundColor: "#777777",
                        border: {
                            color: "#777777"
                        },
                        pointers: {
                            backgroundColor: "#424242",
                            border: {
                                color: "#424242"
                            },
                        },
                        ticks: {
                            color: "#777777"
                        },
                        labels: {
                            color: "#282828"
                        }
                    }
                },

                flatdark: {
                    backgroundColor: "#1F1F1F",
                    scales: {
                        pointerCap: {
                            borderColor: "#686868",
                            backgroundColor: "#686868"
                        },
                        backgroundColor: "#7a7a7a",
                        border: {
                            color: "#7a7a7a"
                        },
                        pointers: {
                            backgroundColor: "#686868",
                            border: {
                                color: "#686868"
                            },
                        },
                        ticks: {
                            color: "#7a7a7a"
                        },
                        labels: {
                            color: "#d3d3d3"
                        }
                    }
                }

            }
        },
          

        _defaultScaleValues: function () {
            return {
                  
                size: 6,
                  
                pointerCap: {
                      
                    radius: 7,
                      
                    borderWidth: 3,
                      
                    interiorGradient: null,   
                      
                    borderColor: null,
                      
                    backgroundColor:null
                },
                  
                showScaleBar: false,
                  
                sweepAngle: 310,
                  
                radius: 170,
                  
                startAngle: 115,
                  
                majorIntervalValue: 10,
                  
                minorIntervalValue: 2,
                  
                maximum: null,
                  
                minimum: null,
                  
                border: {
                      
                    color: null,
                      
                    width:1.5
                },
                  
                backgroundColor: null,
                  
                direction: "clockwise",   
                  
                showPointers: true,
                  
                showRanges: false,
                  
                showTicks: true,
                  
                showLabels: true,
                  
                showIndicators: false,
				  
                opacity:1,
				  
				shadowOffset:0,                  

				customLabels:[{ 
				
					value: null,
					
					color: null,
					
					textAngle: 0,
					
					positionType: "inner",
					
					position: {
					
						x: 0,
						
						y: 0
					},
					
					font: {
						
						size: "11px",
						
						fontFamily: "arial",
						
						fontStyle: "bold"
					}
				}],
                  
                pointers: [{
                      
                    distanceFromScale: 0,
                      
                    showBackNeedle: false,
                      
                    backNeedleLength: 10,
                      
                    length: 150,
                      
                    placement: "near",   
                      
                    width: 7,
                      
                    opacity: 1,
                    
                      
                    value: null,
                      
                    border: {
                          
                        color: null,
                          
                        width: 1.5
                    },
                      
                    backgroundColor: null,
                      
                    gradients: null,   
                      
                    type: "needle",   
                      
                    needleType: "triangle",   
                      
                    markerType: "rectangle",  
                      
                    imageUrl: "",
                      
                    pointerValueText: {
                          
                        showValue: false,
                          
                        distance: 20,
                          
                        font: {
                              
                            size: "11px",
                              
                            fontFamily: "Arial",
                              
                            fontStyle: "Bold"
                        },
                          
                        color: "#8c8c8c",
                          
                        opacity: 1,
                          
                        autoAngle: false,
                          
                        angle: 0,
                    }
                }],
                  
                ranges: [{

                    legendText: null,
                      
                    distanceFromScale: 25,
                      
                    size: 5,
                      
                    placement: "near",   
                      
                    startValue: null,
                      
                    endValue: null,
                      
                    startWidth: null,
                      
                    endWidth: null,
                      
                    gradients: null,   
                    
                      
                    opacity: null,
                      
                    backgroundColor: "#32b3c6",
                      
                    border: {
                          
                        color: "#32b3c6",
                          
                        width:1.5
                    }
                }],
                  
                ticks: [{
                      
                    angle: 0,
                      
                    distanceFromScale: 0,
                      
                    color:null,
                      
                    type: "major",   
                      
                    placement: "near",   
                      
                    height: 16,
                      
                    width: 3
                }, {
                    angle: 0,
                    distanceFromScale: 0,
                    color:null,
                    type: "minor",   
                    placement: "near",   
                    height: 7,
                    width: 1
                }],
                  
                labels: [{
                      
                    angle: 0,
                      
                    autoAngle:false,
                      
                    opacity: null,
                      
                    font: {
                     
                        size: "11px",
                    
                        fontFamily: "Arial",
                    
                        fontStyle: "Bold"
                    },
                      
                    color: null,
                      
                    distanceFromScale: 0,
                      
                    includeFirstValue: true,
                      
                    placement: "near",   
                      
                    type: "major",  
                      
                    unitText: "",
                      
                    unitTextPosition: "back",  

                }],
                  
                indicators: [{
                      
                    height: 15,
                      
                    width: 15,
                      
                    type: "circle",   
                      
                    imageUrl: null,
                      
                    position: {
                          
                        x: 0,
                          
                        y: 0
                    },
                      
                    stateRanges: [{
                          
                        endValue: 0,
                          
                        startValue: 0,
                          
                        backgroundColor: null,
                          
                        borderColor: null,
                          
                        text: "",
                          
                        textColor: null,
                          
                        font: null
                    }]
                }],                  
                  
                subGauges: [{
                      
                    height: 150,
                      
                    width: 150,
                      
                    position: {
                          
                        x: 0,
                          
                        y: 0
                    }
                }],
                //textAngle ,value:"CustomText", position:{X:5,Y:10}, color, font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" },
            };
        },
        dataTypes: {
            scales: "data",
            isResponsive: "boolean",
        },
        observables: ["value", "minimum", "maximum"],
        _tags: [{
            tag: "scales",
            attr: ["showScaleBar", "pointerCap.radius", "pointerCap.borderWidth", "pointerCap.interiorGradient", "pointerCap.borderColor", "pointerCap.backgroundColor", "sweepAngle", "startAngle", "showPointers", "showTicks", "backgroundColor", "scaleRadius", "majorIntervalValue", "minorIntervalValue", "shadowOffset", "showRanges", "showLabels", "showCustomLabels", "showIndicators", "border.width", "border.color",
                [{
                    tag: "pointers", attr: ["distanceFromScale", "showBackNeedle", "backNeedleLength", "backgroundColor", "needleType", "markerType", "border.width", "border.color", "imageUrl", "pointerValueText.showValue", "pointerValueText.distance", "pointerValueText.font.size", "pointerValueText.font.fontFamily", "pointerValueText.font.fontStyle", "pointerValueText.angle", "pointerValueText.autoAngle", "pointerValueText.color", "pointerValueText.opacity"]
                }, {
                    tag: "labels", attr: ["autoAngle", "distanceFromScale", "includeFirstValue", "unitText", "unitTextPosition", "font.size", "font.fontFamily", "font.fontStyle"]
                }, {
                    tag: "ticks", attr: ["distanceFromScale"]
                }, {
                    tag: "ranges", attr: ["legendText", "distanceFromScale", "startValue", "endValue", "startWidth", "endWidth", "backgroundColor", "border.color", "border.width"]
                }, {
                    tag: "indicators", attr: ["imageUrl", "position.x", "position.y",
                        [{
                            tag: "stateRanges", attr: ["endValue", "startValue", "backgroundColor", "borderColor", "textColor"]
                        }]
                    ]
                }, {
                    tag: "subGauges", attr: ["controlID", "position.x", "position.y"]
                }, {
                    tag: "customLabels", attr: ["positionType", "textAngle", "position.x", "position.y", "font.size", "font.fontFamily", "font.fontStyle"]
                }]
            ]
        }],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),
 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "radius": this.model.radius = options[option]; break;
                    case "frame": $.extend(this.model.frame, options[option]); break;
                    case "tooltip": $.extend(this.model.tooltip, options[option]); break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "interiorGradient": this.model.interiorGradient = options[option]; break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "theme": this.model.theme = options[option]; break;
                    case "isRadialGradient": this.model.isRadialGradient = options[option]; break;
                    case "outerCustomLabelPosition": this.model.outerCustomLabelPosition = options[option]; break;
                    case "gaugePosition": this.model.gaugePosition = options[option]; break;
                    case "distanceFromCorner": this.model.distanceFromCorner = options[option]; break;
                    case "rangeZOrder ": this.model.rangeZOrder  = options[option]; break;
                    case "value":
                        (this.value() == "") && this.value(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (var j = 0; this.model.scales[i].pointers[j] != null; j++) {
                                this.model.scales[i].pointers[j].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        (this.minimum() == "") && this.minimum(0);
                        for (var k = 0; this.model.scales[k] != null; k++) {
                            this.model.scales[k].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        (this.maximum() == "") && this.maximum(100);
                        for (var l = 0; this.model.scales[l] != null; l++) {
                            this.model.scales[l].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        $.extend(true, this.model.scales, options[option]);
                        break;
                }
            }
            this._init();
        },

      
        _destroy: function () {
            this._unWireEvents();
            this.element.empty().removeClass("e-widget");
        },


        // constructor function
		  
        _init: function () {
            _gaugeCount = $(".e-circulargauge").length;
            initialGaugeCount = _gaugeCount;
            //this._isHalfCircle = false;
            this._scaleRedrawn = false;
            this._scalesInitialize();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            this._wireEvents();
            this._onWindowResize();
        },
	 
        _onWindowResize: function () {
            if (this.model.enableResize || this.model.isResponsive) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this.resizeCanvas);
                else
                    this._on($(window), "orientationchange", this.resizeCanvas);
            }
        },
		 
        _initialize: function () {
			//Checks whether canvas animation is supported if the device is android
			this._androidAnimation = this.isAndroid() && ej.datavisualization.CircularGauge.animationPolyfill;			
            this._initObject(this);
            this._drawOuterLayer();
            if (this.model.legend.visible && this.model.legend._legendItemHeight > 0 && this.model.legend._legendItemWidth > 0) {
                this._drawLegend();
                this._legendDrawn = true;
            }
            this._drawScales();         
            if (this.model.renderComplete)
                this._onRenderComplete();
        },

        _drawLegend:function(){
            var legendObj = this.model.legend, legendActualBounds, element = this.GaugeEl, padding = 10, legendSpacing = 20,
                position = this.model.legend.position.toLowerCase(), alignment = this.model.legend.alignment.toLowerCase(),
                legendOptions, borderWidth = this.model.legend.border.width, width = this.model.width, height = this.model.height;
            legendActualBounds = this.model.legendActualBounds;
            if(position == "top" || position == "bottom"){
                legendActualBounds.y = (position == "top") ? borderWidth + legendSpacing : (height - legendActualBounds.Height) - legendSpacing;
                if (alignment == "center")
                    legendActualBounds.x = (width / 2) - (legendActualBounds.Width / 2);
                else if (alignment == "near")
                    legendActualBounds.x = borderWidth + legendSpacing;
                else
                    legendActualBounds.x = (width - legendActualBounds.Width) - legendSpacing;
                legendActualBounds.x = ((legendActualBounds.x + legendActualBounds.Width) > width) ? (legendActualBounds.x - Math.abs((legendActualBounds.x + legendActualBounds.Width) - width)) : legendActualBounds.x;
            }
            else{
                legendActualBounds.x = (position == "left") ? borderWidth + legendSpacing : (width - legendActualBounds.Width) - legendSpacing;
                 if (alignment == "center")
                     legendActualBounds.y = (height / 2) - (legendActualBounds.Height / 2);
                 else if (alignment == "near")
                     legendActualBounds.y = borderWidth + legendSpacing;
                 else
                     legendActualBounds.y = (height - legendActualBounds.Height) - legendSpacing;
                 legendActualBounds.y = ((legendActualBounds.y + legendActualBounds.Height) > height) ? (legendActualBounds.y - Math.abs((legendActualBounds.y + legendActualBounds.Height) - height)) : legendActualBounds.y;
            }
            this._legendBounds = legendActualBounds;
            this.contextEl.lineWidth = borderWidth;
            this.contextEl.save();
            this.contextEl.strokeStyle = this.model.legend.border.color;
            this.contextEl.strokeRect(legendActualBounds.x, legendActualBounds.y, legendActualBounds.Width, legendActualBounds.Height);
            this.contextEl.restore();
            this.model._legendCollection = [];
            for (var i = 0; i < this.model.scales.length; i++) {
                var scale = this.model.scales[i];
                for (var j = 0; j < scale.ranges.length; j++) {
                    var range = scale.ranges[j], currentLegend = {}, itemPadding = 10, legendItemSize;
                    range._visible = (ej.util.isNullOrUndefined(range._visible)) ? true : range._visible;
                    if (!(ej.util.isNullOrUndefined(range.legendText))) {
                        legendItemSize = this._getLegendSize(range, itemPadding);
                        legendOptions = {
                            "Bounds": { Height: legendItemSize.Height, Width: legendItemSize.Width },
                            "legendStyle": { font: legendObj.font, opacity: legendObj.opacity, BorderColor: legendObj.itemStyle.border.color, BorderWidth: legendObj.itemStyle.border.width },
                            "displayText": range.legendText,
                            "fill": (ej.util.isNullOrUndefined(legendObj.fill)) ? range.backgroundColor : legendObj.fill,
                            "ScaleIndex": i,
                            "RangeIndex": j,
                            "visible": range._visible
                        };
                        this.model._legendCollection.push(legendOptions);
                    }
                }
            }
            this._drawLegendItem(legendActualBounds);   
        },

        legendMouseClick: function (evt) {
            var parentLeft, parentTop, element, canvasLeft, canvasTop, offsetLeft, offsetTop, i,
                region, legendX, legendWidth, legendY, legendHeight, args, visible;
            var mouseMoveCords = this.calMousePosition(evt);
            if (this.model.legend.toggleVisibility) {
                for (i = 0; i < this.legendRegion.length; i++) {
                    region = this.legendRegion[i];
                    legendX = region.X
                    legendWidth = region.X + region.Width;
                    legendY = region.Y;
                    legendHeight = region.Y + region.Height;
                    if ((mouseMoveCords.X >= legendX && mouseMoveCords.X <= legendWidth) && (mouseMoveCords.Y >= legendY && mouseMoveCords.Y <= legendHeight)) {
                        args = { type: "legendItemClick", cancel: false, data: region, model: this.model };
                        this._trigger("legendItemClick", args);
                        visible = (this.model.scales[region.item.ScaleIndex].ranges[region.item.RangeIndex]._visible) ? false : true;
                        this.setRangeVisible(region.item.ScaleIndex, region.item.RangeIndex, visible);
                    }
                }
            }
                
        },

        calMousePosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent), element, mouseX, mouseY, browser, node, i;
            element = this.GaugeEl[0];
            node = element.querySelector("canvas");
            browser = matched.browser.toLowerCase();
            mouseX = (e.pageX) - $(node).offset().left;
            mouseY = (e.pageY) - $(node).offset().top;
            return { X: mouseX, Y:mouseY };
        },

        rangesMouseMove: function (evt) {
              var gauge = this, args,element, parentLeft, parentTop, canvasLeft, canvasTop, offsetLeft, offsetTop, i, region, legendX,
                        legendWidth, legendY, legendHeight, element, rectBounds, xOffset, yOffset, mouseX, mouseY, pointData,
                        startAngle, endAngle, gaugeStartAngle, j, distanceFromCenter, rangeRegion;
               var mouseMoveCords = this.calMousePosition(evt);
               if (this.model.legend.toggleVisibility && this._legendDrawn) {
                    for (i = 0; i < this.legendRegion.length; i++) {
                       region = this.legendRegion[i];
                       legendX = region.X;
                       legendWidth = region.X + region.Width;
                       legendY = region.Y;
                       legendHeight = region.Y + region.Height;
                       if ((mouseMoveCords.X >= legendX && mouseMoveCords.X <= legendWidth) && (mouseMoveCords.Y >= legendY && mouseMoveCords.Y <= legendHeight)) {
                           $(this.GaugeEl[0]).css("cursor", "pointer");
                           break;
                       }
                       else
                           $(this.GaugeEl[0]).css("cursor", "default");
                }
             }
             mouseX = mouseMoveCords.X;
             mouseY = mouseMoveCords.Y;
             pointData = [];
             startAngle = this.model.scales[0].startAngle;
             endAngle = this.model.scales[0].sweepAngle;
             gaugeStartAngle = startAngle * (Math.PI / 180);
             for (j = 0; j < this._rangeCollectionRegions.length; j++) {
                 var region = this._rangeCollectionRegions[j], fromCenterX, fromCenterY, innerRadius, totalDegree, circularStartAngle, arcAngle, clickAngle, pointStartAngle, pointEndAngle;
                 fromCenterX = mouseX - region.centerRadius.centerX;
                 fromCenterY = mouseY - region.centerRadius.centerY;
                 innerRadius = region.Radius.endRadius ? region.Radius.endRadius : 0;
                 startAngle = startAngle < 0 ? startAngle + 360 : startAngle;
                 endAngle = endAngle < 0 ? endAngle + 360 : endAngle;
                 totalDegree = endAngle - startAngle;
                 circularStartAngle = (-.5 * Math.PI) + gaugeStartAngle;
                 if (totalDegree < 0) {
                     endAngle = endAngle / 360;
                    arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
                    clickAngle = (Math.atan2(fromCenterY, fromCenterX) - circularStartAngle - arcAngle) % (2 * Math.PI);
                     if (clickAngle < 0)
                         clickAngle = 2 * Math.PI + clickAngle;
                     pointStartAngle = Math.PI * (region.Region.startAngle / 180);
                     pointEndAngle = Math.PI * (region.Region.endAngle / 180);

                     if (clickAngle <= pointStartAngle && clickAngle >= pointEndAngle) {
                         distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                         if (distanceFromCenter <= region.Radius.startRadius && distanceFromCenter > region.Radius.endRadius) {
                             args = { type: "rangeMouseMove", cancel: false, data: region, model: this.model };
                             this._trigger("rangeMouseMove", args);
                             break;
                         }
                     }
                 }
                 else {
                     clickAngle = (Math.atan2(fromCenterY, fromCenterX)) % (2 * Math.PI);
                     if (clickAngle < 0)
                         clickAngle = 2 * Math.PI + clickAngle;
                     if (region.Region.endAngle < region.Region.startAngle) {
                         pointStartAngle = (fromCenterX > 0 && fromCenterY > 0) ? 0 : Math.PI * ((region.Region.startAngle) / 180);
                         pointEndAngle = (fromCenterX > 0 && fromCenterY > 0) ? Math.PI * ((region.Region.endAngle) / 180) : Math.PI * ((360 + region.Region.endAngle) / 180);
                     }
                     else {
                         pointStartAngle = Math.PI * ((region.Region.startAngle) / 180);
                         pointEndAngle = Math.PI * ((region.Region.endAngle) / 180);
                     }
                     if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                             distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                             if (distanceFromCenter <= region.Radius.startRadius && distanceFromCenter > region.Radius.endRadius) {
                                 rangeRegion = region;
                                 args = { type: "rangeMouseMove", cancel: false, data: region, model: this.model };
                                 this._trigger("rangeMouseMove", args);
                                 break;
                             }                       
                     }
                 }
             }
             return rangeRegion;
    },

        _drawLegendItem: function (legendBounds) {
            var legendItemCollections = [], legendTextCollection = [], args;
            this.legendRegion = [];
            this.model.legend._legendTextMaxHeight = 0; this.model.legend._legendTextMaxWidth = 0;
            var position = this.model.legend.position.toLowerCase();
            for (var i = 0; i < this.model._legendCollection.length; i++) {
                var shape = this.model.legend.shape.toLowerCase(), verPadding = 10, horPadding = 10, txtPadding = 3, itemX, itemY, textX, textY;
                var legendItem, rangeVisible, textStyle, fill, x, y, width, height, fontColor, font, textSize, shapeHeight, shapeWidth, shapeStyle, itemStyle,
                    shapeLocation, itemLocation, space= 10;
                legendItem = this.model._legendCollection[i];
                if (!(ej.util.isNullOrUndefined(legendItem.displayText))) {
                    args = { type: "legendItemRender", cancel: false, data: legendItem, model: this.model };
                    this._trigger("legendItemRender", args);
                    this.model.scales[legendItem.ScaleIndex].ranges[legendItem.RangeIndex]._visible = (ej.util.isNullOrUndefined(this.model.scales[legendItem.ScaleIndex].ranges[legendItem.RangeIndex]._visible)) ? true : this.model.scales[legendItem.ScaleIndex].ranges[legendItem.RangeIndex]._visible;
                    fill = (this.model.scales[legendItem.ScaleIndex].ranges[legendItem.RangeIndex]._visible) ? legendItem.fill : "#808080";
                    fontColor = (this.model.scales[legendItem.ScaleIndex].ranges[legendItem.RangeIndex]._visible) ? legendItem.legendStyle.font.color : "#808080";
                    font = this._getFontString(this, legendItem.legendStyle.font);
                    textSize = this.calcText(legendItem.displayText, legendBounds.Width, legendItem.legendStyle.font);
                    this.model.legend._legendTextMaxHeight = Math.max(textSize.height, this.model.legend._legendTextMaxHeight);
                    this.model.legend._legendTextMaxWidth = Math.max(textSize.width, this.model.legend._legendTextMaxWidth);
                    shapeHeight = this.model.legend.itemStyle.height;
                    shapeWidth = this.model.legend.itemStyle.width;
                    shapeStyle = (shapeHeight + shapeWidth) / 2;
                    itemStyle = {
                        "angle": 0,
                        "width": shapeStyle,
                        "isFill": true,
                        "isStroke": true,
                        "height": shapeStyle,
                        "lineWidth": this.model.legend.itemStyle.border.width,
                        "opacity": legendItem.legendStyle.opacity,
                        "strokeStyle": legendItem.legendStyle.BorderColor,
                        "fillStyle": fill,
                    };
                    verPadding = (shapeStyle > (textSize.height / 2)) ? verPadding + (shapeStyle / 2) : verPadding + textSize.height / 4;
                    horPadding = (shape == "circle") ? (horPadding + (shapeStyle / 2)) : horPadding;                  
                    if (i == 0) {
                       itemX = (horPadding + legendBounds.x);
                       itemY = (legendBounds.y + verPadding);
                    }
                    else {
                        if (position == "top" || position == "bottom") {
                            var xPos = (legendTextCollection[i - 1].x + legendTextCollection[i - 1].width) + (this.model.legend.itemPadding);
                            if ((xPos + shapeStyle + txtPadding + textSize.width) >= (legendBounds.x + legendBounds.Width)) {
                                itemX = (horPadding + legendBounds.x);
                                itemY = ((this.model.legend._legendTextMaxHeight / 2) > shapeStyle) ? (legendTextCollection[i - 1].y + (legendItemCollections[i - 1].actualBounds.height / 2) + shapeStyle / 2) + space : 
								space + legendItemCollections[i - 1].actualBounds.y + (legendItemCollections[i - 1].actualBounds.height / 2) + shapeStyle / 2;
                            }
                            else {
                                itemX = (shape.toLowerCase() == "circle") ? xPos + shapeStyle / 2 : xPos;
                                itemY = (legendItemCollections[i - 1].actualBounds.y);
                            }
                        }
                        else {
                            yPos = (legendTextCollection[i - 1].height > shapeStyle) ? (legendTextCollection[i - 1].y + this.model.legend.itemPadding) : (legendItemCollections[i - 1].actualBounds.y + shapeStyle) + this.model.legend.itemPadding;
                            var temp = (legendTextCollection[i - 1].height > shapeStyle) ? textSize.height : shapeStyle;
                            if ((yPos + temp - 2) > legendBounds.y + legendBounds.Height + 2) {
                                itemY = (legendBounds.y + verPadding);
                                itemX = (legendItemCollections[i - 1].actualBounds.x + legendItemCollections[i - 1].actualBounds.width) + (this.model.legend._legendTextMaxWidth + txtPadding) + 10;
                            }
                            else {
                                itemX = (legendItemCollections[i - 1].actualBounds.x);
                                itemY = yPos;
                            }
                        }
                    }
                    shapeLocation = { startX: itemX, startY: itemY };
                    shape = shape.charAt(0).toUpperCase() + shape.slice(1);
                    itemLocation = $.extend({}, shapeLocation, true);
                    itemLocation.startX = (shape.toLowerCase() == "circle") ? itemLocation.startX - (shapeStyle / 2) : itemLocation.startX;
                    itemLocation.startY = itemLocation.startY - (shapeStyle / 2);
                    legendItemCollections.push({ actualBounds: { x: shapeLocation.startX, y: shapeLocation.startY, height: shapeStyle, width: shapeStyle }, item: legendItem });
                    textStyle = {
                        "angle": 0,
                        "width": textSize.width,
                        "isFill": true,
                        "isStroke": true,
                        "height": textSize.height,
                        "textValue": legendItem.displayText,
                        "font": font,
                        "fillStyle": fontColor,
                        "opacity": legendItem.legendStyle.opacity,
                        "strokeStyle": legendItem.legendStyle.BorderColor,
                    };
                    textX = (shape.toLowerCase() == "circle") ? (shapeLocation.startX + (shapeWidth / 2) + txtPadding) : (shapeLocation.startX + itemStyle.width + txtPadding);
                    textY = shapeLocation.startY + (textSize.height / 4);
                    legendTextCollection.push({ x: textX, y: textY, height: textSize.height / 2, width: textSize.width });
                    x = itemLocation.startX;
                    width = (Math.abs((x + legendItemCollections[i].actualBounds.width) - x) + txtPadding) + (Math.abs((legendTextCollection[i].x + legendTextCollection[i].width) - legendTextCollection[i].x));
                    if ((shapeStyle) > (legendTextCollection[i].height)) {
                        y = itemLocation.startY;
                        height = shapeStyle;
                    }
                    else {
                        y = legendTextCollection[i].y - legendTextCollection[i].height;
                        height = legendTextCollection[i].height;
                    }   
                    if (shape.toLowerCase() == "circle") {
                        style = this._setPointerDimension(itemStyle, this);
                        this._contextOpenPath(style, this);
                        this._setContextRotation(style, this);
                        this.contextEl.arc(shapeLocation.startX, shapeLocation.startY, shapeStyle / 2, 0, 2 * Math.PI);
                        this._contextClosePath(style, this);
                    }
                    else
                        this["_draw" + shape](shapeLocation, itemStyle, this);
                    this._contextOpenPath(textStyle, this);
                    this.contextEl.font = textStyle.font;
                    this.contextEl.translate(textX, textY);
                    this.contextEl.fillText(textStyle.textValue, 0, 0);
                    this._contextClosePath(textStyle, this);
                    this.legendRegion.push({ X: x, Y: y, Width: width, Height: height, item: legendItem });
                }
            }
        },

        calcText: function (text, maxwidth, font) {
            var element = $(document).find("#measureTex");
            var textObj, style = null, size = null, family = null, weight = null, textArray, i, fontarray, bounds;
            $("#measureTex").css('display', 'block'); 
            if (element.length == 0) {
                textObj = document.createElement('text');
                $(textObj).attr({ 'id': 'measureTex' });
                document.body.appendChild(textObj);
            }
            else {
                textObj = element[0];
            }
            if (typeof (text) == "string" && (text.indexOf("<") > -1 || text.indexOf(">") > -1)) {
                textArray = text.split(" ");
                for (i = 0; i < textArray.length; i++) {
                    if (textArray[i].indexOf("<br/>") == -1)
                        textArray[i] = textArray[i].replace(/[<>]/g, '&');
                }
                text = textArray.join(' ');
            }
            textObj.innerHTML = text;
            if (font != undefined && font.size == undefined) {
                fontarray = font;
                fontarray = fontarray.split(" ");
                style = fontarray[0];
                size = fontarray[1];
                family = fontarray[2];
                weight = fontarray[3];
            }
            if (font != null) {
                textObj.style.fontSize = (font.size > 0) ? (font.size + "px") : font.size ? font.size : size;
                if (textObj.style.fontStyle)
                    textObj.style.fontStyle = (font.fontStyle) ? font.fontStyle : style;
                textObj.style.fontFamily = font.fontFamily ? font.fontFamily : family;
                if (window.navigator.userAgent.indexOf('MSIE 8.0') == -1)
                    textObj.style.fontWeight = font.fontWeight ? font.fontWeight : weight;
            }
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100;
            textObj.style.left = 0;
            textObj.style.visibility = 'hidden';
            textObj.style.whiteSpace = 'nowrap';
            if (maxwidth)
                textObj.style.maxwidth = maxwidth + "px";
            bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            $("#measureTex").css('display', 'none');
            return bounds;
        },

         _calculateLegendBounds:function(){
            var self = this, legendObj;
            var legendObj = self.model.legend, rangeCount = 0, i, add, position = self.model.legend.position.toLowerCase(), legendSize,
            vericalPadding, horizontalPadding, additionalPadding = 20, space = 10, padd,
            borderWidth = self.model.legend.border.width, itemPadding = self.model.legend.itemPadding, overallMax, bounds, currentWidth, legendMax;
            overallMax = self.findMaxHeightWidth();
            if (this._gaugeResizeState || (ej.util.isNullOrUndefined(self.model.legend.size.height) && ej.util.isNullOrUndefined(self.model.legend.size.width))) {               
                legendMax = self.findLegendMax();
                bounds = { Height: legendMax.Height, Width: legendMax.Width };
                self.model.legend.size._height = self.model.legend.size._width = null;
            }
            else if ((parseInt(self.model.legend.size.width) < overallMax.width && !ej.util.isNullOrUndefined(self.model.legend.size.width)) 
			    || (parseInt(self.model.legend.size.height) < overallMax.height && !ej.util.isNullOrUndefined(self.model.legend.size.height))) {
                legendMax = self.findLegendMax();
                bounds = { Height: legendMax.Height, Width: legendMax.Width };
               self.model.legend.size._height = self.model.legend.size._width = null;
            }
        else {
            self.model.legend._legendItemHeight = 0; self.model.legend._columnWidth= 0, self.model.legend._legendItemWidth = 0, self.model.legend._rowCount = 1,
            self.model.legend._columnCount = 1, self.model.legend._maxWidth = 0, self.model.legend._maxHeight = 0, self._columnIncreasing = 0, self._columnDecreasing= 0;
                $.each(self.model.scales, function (index, element) {
                    if (element.showRanges) {
                        self.scaleIndex = index;
                        if (element.ranges != null) {
                            self.rangeEl = element.ranges;
                            self.index = 0;
                            for (i = 0; i < self.model.scales[self.scaleIndex].ranges.length ; i++) {
                                if (!(ej.util.isNullOrUndefined(self.model.scales[self.scaleIndex].ranges[i].legendText))) {
                                    rangeCount++;
                                }
                            }
                            $.each(element.ranges, function (rangeIndex, rangeElement) {
                                self.rangeIndex = rangeIndex;
                                if (!(ej.util.isNullOrUndefined(rangeElement.legendText)) && (rangeElement.legendText.replace(/\s/g,"").length > 0) && self.index < rangeCount) {
                                    legendSize = self._getLegendSize(rangeElement);
                                    if (parseInt(self.model.legend.size.width)) {
                                            horizontalPadding = (self.index == 0) ? (space + borderWidth) : (position == "top" || position == "bottom") ? itemPadding : space;
                                            vericalPadding = (self.index == 0) ? (space + borderWidth) : 0;
                                            legendObj._legendItemWidth =  (position == "top" || position == "bottom") ? (horizontalPadding + legendSize.Width + legendObj._legendItemWidth) : 
											                              (horizontalPadding + overallMax.width + legendObj._legendItemWidth);
                                            legendObj._legendItemHeight = vericalPadding + Math.max(legendSize.Height, legendObj._legendItemHeight);
                                            self._columnIncreasing++;
                                            self._columnDecreasing = Math.max(self._columnDecreasing, self._columnIncreasing);
                                            if (parseInt(self.model.legend.size.width) <= legendObj._legendItemWidth) {
                                                self.model.legend._rowCount++;
                                                self._columnIncreasing = 0;
                                                legendObj._legendItemHeight += (position == "top" || position == "bottom") ? (self.index == rangeCount - 1) ? (legendSize.Height + space * 2) : legendSize.Height + space :
                                                                               (self.index == rangeCount - 1) ? (legendSize.Height + space + itemPadding) : (legendSize.Height + itemPadding);
                                                 legendObj._legendItemWidth = (position == "top" || position == "bottom") ? (self.index == rangeCount - 1) ? parseInt(self.model.legend.size.width) : 
												                               legendSize.Width + space : overallMax.width + space;
                                            } else if (self.index == rangeCount - 1 && self.model.legend._rowCount > 1) {
                                                legendObj._legendItemHeight += space;
                                                self.model.legend.size._width = self.model.legend.size.width;
                                                if (parseInt(self.model.legend.size.width) > (overallMax.width * (self._columnDecreasing - 1)) + ((self._columnDecreasing - 1) * space) + space) {
                                                    legendObj._legendItemWidth = (overallMax.width * (self._columnDecreasing - 1)) + ((self._columnDecreasing - 1) * space) + space;
                                                    self.model.legend.size._width = null;
                                                } 
                                            }
                                    }
                                    else {
                                        if (position == "left" || position == "right") {
                                            horizontalPadding = (self.index == 0) ? (space + borderWidth) : (self.index == rangeCount - 1) ? space : 0;
                                            vericalPadding = (self.index == 0) ? (space + borderWidth) : itemPadding;
                                            legendObj._legendItemWidth = horizontalPadding + Math.max(overallMax.width, legendObj._legendItemWidth);
                                            legendObj._legendItemHeight = (vericalPadding + legendSize.Height + legendObj._legendItemHeight);
											self.model.legend.size._height = parseInt(self.model.legend.size.height);
                                            if (parseInt(self.model.legend.size.height) < legendObj._legendItemHeight) {
                                                self.model.legend._rowCount++;
                                                legendObj._legendItemWidth += (self.index == rangeCount - 1) ? (overallMax.width + space * 2) : overallMax.width + space;
                                                legendObj._legendItemHeight = legendSize.Height + space;
                                            }
                                            else if (rangeCount == 1 && parseInt(self.model.legend.size.height) > legendObj._legendItemHeight){
                                                self.model.legend.size._height = null;
                                                legendObj._legendItemWidth += space;
                                                legendObj._legendItemHeight += space;
                                            }
                                        }
                                        else {
                                            legendMax = self.findLegendMax();
                                            bounds = { Height: legendMax.Height, Width: legendMax.Width };
                                            self.index = rangeCount - 1;
                                        }
                                    }
                                }
                                self.index++;
                            });
                        }
                    }
                });
                bounds = { Height: legendObj._legendItemHeight, Width: legendObj._legendItemWidth };
            }
            return bounds;
        },

       findLegendMax:function(){
            var self = this, legendObj;
            self.model.legend._legendItemHeight = 0; self.model.legend._rowWidth = 0; self.model.legend._legendItemWidth = 0, self.model.legend._rowCount = 1,
            self.model.legend._columnCount = 1, self.model.legend._maxWidth = 0, self.model.legend._maxHeight = 0;
            var legendObj = self.model.legend, rangeCount = 0, i, add, position = self.model.legend.position.toLowerCase(), legendSize,
            vericalPadding, horizontalPadding, additionalPadding = 20, space = 10, padd,
            borderWidth = self.model.legend.border.width, itemPadding = self.model.legend.itemPadding, currentWidth;
            self.model.legend.size._height = self.model.legend.size._width = null;
            $.each(self.model.scales, function (index, element) {
                if (element.showRanges) {
                    self.scaleIndex = index;
                    if (element.ranges != null) {
                        self.rangeEl = element.ranges;
                        self.index = 0;
                        for (i = 0; i < self.model.scales[self.scaleIndex].ranges.length ; i++) {
                            if (!(ej.util.isNullOrUndefined(self.model.scales[self.scaleIndex].ranges[i].legendText))) {
                                rangeCount++;
                            }
                        }
                        $.each(element.ranges, function (rangeIndex, rangeElement) {
                            self.rangeIndex = rangeIndex;
                            if (!(ej.util.isNullOrUndefined(rangeElement.legendText))) {  
                                legendSize = self._getLegendSize(rangeElement);
                                    if (position == "top" || position == "bottom") {
                                        if (rangeCount != 1) {
                                            horizontalPadding = (self.index == 0) ? (space * 2) + (borderWidth) : (self.index == rangeCount - 1) ? (space + itemPadding) : itemPadding;
                                            vericalPadding = (self.index == 0) ? space : (self.index == rangeCount - 1) ? space : 0;
                                        }
                                        else
                                            vericalPadding = horizontalPadding = (space * 2);
                                        legendObj._legendItemHeight = vericalPadding + Math.max(legendSize.Height, legendObj._legendItemHeight);
                                        legendObj._maxHeight = Math.max(legendSize.Height, legendObj._maxHeight);
                                        legendObj._legendItemWidth = (horizontalPadding + legendSize.Width + legendObj._legendItemWidth);
                                        padd = (self.model.legend._rowCount > 1 && self.index == rangeCount - 1) ? space : 0;
                                        if ((self.model.width - (additionalPadding - borderWidth)) <= legendObj._legendItemWidth - padd) {
                                            self.model.legend._rowCount++;
                                            legendObj._rowWidth = Math.max( legendObj._rowWidth,(legendObj._legendItemWidth - itemPadding - legendSize.Width));
                                            legendObj._legendItemWidth = (self.index == rangeCount - 1) ? (legendObj._rowWidth < (self.model.width - (additionalPadding * 2))) ? legendObj._rowWidth : (self.model.width - (additionalPadding * 2))   : 
											                             (self.index == 0) ? additionalPadding + space + (borderWidth) : legendSize.Width + additionalPadding + space + (borderWidth);
                                            legendObj._legendItemHeight += legendObj._maxHeight + (space + borderWidth);
                                        }
                                        else if ((self.index == rangeCount - 1) && self.model.legend._rowCount > 1)
                                            legendObj._legendItemWidth = (legendObj._rowWidth < (self.model.width - (additionalPadding * 2))) ? legendObj._rowWidth : (self.model.width - (additionalPadding * 2));
                                    }
                                    else {
                                        if (rangeCount != 1) {
                                            vericalPadding = (self.index == 0) ? (space + borderWidth) : (self.index == rangeCount - 1) ? (space + itemPadding) : itemPadding;
                                            horizontalPadding = (self.index == 0) ? space : (self.index == rangeCount - 1) ? space * 2 : 0;
                                        }
                                        else
                                            vericalPadding = horizontalPadding = (space * 2);

                                        legendObj._legendItemHeight = legendSize.Height + vericalPadding + legendObj._legendItemHeight;
                                        legendObj._legendItemWidth = horizontalPadding + Math.max(legendSize.Width, legendObj._legendItemWidth);
                                        legendObj._maxWidth = Math.max(legendSize.Width, legendObj._maxWidth);

                                        if ((self.model.height - additionalPadding) < legendObj._legendItemHeight) {
                                            self.model.legend._columnCount++;
                                            add = (self.model.legend._columnCount == 2) ? additionalPadding : 0;
                                            legendObj._legendItemHeight = (self.index == rangeCount - 1) ? (self.model.height - (additionalPadding * 2)) : legendSize.Height + additionalPadding + space + borderWidth + add;
                                            legendObj._legendItemWidth += legendObj._maxWidth + space;
                                        }
                                        else if ((self.index == rangeCount - 1) && self.model.legend._columnCount > 1) {
                                            legendObj._legendItemHeight = self.model.height - (additionalPadding * 2);
                                        }

                                    }
                                    self.index++;
							}
						});
					}
                 }
            });
            return { Height: legendObj._legendItemHeight, Width: legendObj._legendItemWidth };
	   },
	   
        findMaxHeightWidth: function () {
            var self = this, overallHeight = 0, overallWidth = 0, size;
            $.each(self.model.scales, function (index, element) {
                $.each(element.ranges, function (rangeIndex, rangeElement) {
                    size = self._getLegendSize(rangeElement);
                    overallHeight = Math.max(size.Height, overallHeight);
                    overallWidth = Math.max(size.Width, overallWidth);
                });
            });
            return { height: overallHeight, width: overallWidth };
        },

        _getLegendSize: function (range) {
            var eqaualPadding = 3, shapeStyle, textSize, itemHeight, itemWidth, font = this.model.legend.font;
            shapeStyle = (this.model.legend.itemStyle.width + this.model.legend.itemStyle.height) / 2;
            textSize = this.calcText(range.legendText, null, font);
            itemHeight = Math.max(shapeStyle, (textSize.height / 2));
            itemWidth = shapeStyle + eqaualPadding + textSize.width;
            return {Height:itemHeight,Width:itemWidth};
        },

        _scalesInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
		 
        _checkArrayObject: function (element, initialName) {
            var proxy = this;            
            var type;
            $.each(element, function (name, innerElement) {
                type = typeof name;
                if ((type != "string" || (type == "string" && name.indexOf('_') == -1 && name.indexOf('__') == -1)) && typeof innerElement != "function") {
                    if (innerElement instanceof Array) {
                        proxy._checkArrayObject(innerElement, name);
                    }
                    else if (innerElement != null && typeof innerElement == "object" && !innerElement.setter && !innerElement.factory && !innerElement.key) {
                        var allObjects = proxy._defaultScaleValues();
                        proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                    }
                }
            });
            return element;
        },
	 	
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            var type;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];
            type = typeof name;
            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object" && (type != "string" || (type == "string" && name.indexOf('_') == -1 && name.indexOf('__') == -1))) {
                    proxy._LoadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
	 	
        _initObject: function (element) {
            this._savedPoints=[];
            this.element.addClass("e-widget");
            element.GaugeEl = this.element;
            if (element.canvasEl) {
                element.canvasEl.remove();
                element.GaugeEl.empty();
            }
            else
                element.canvasEl = $("<canvas></canvas>");
                var outerLabelCount=0;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (var j = 0; this.model.scales[i].pointers[j] != null; j++) {
                    if (this.model.scales[i].pointers[j].value == null)
                        this.model.scales[i].pointers[j].value = this.value();
                }
                for (var k = 0; this.model.scales[i].customLabels[k] != null; k++) {
                    if (this.model.scales[i].customLabels[k].value != null) {
					  outerLabelCount++;
					  if (element.GaugeEl.find('div').length == 0) {
                        if (this.model.scales[i].customLabels[k] != null && this.model.scales[i].customLabels[k].positionType !=null && this.model.scales[i].customLabels[k].positionType == "outer") {
                            element.outerDiv = ej.buildTag("div");
                            if (element.model.outerCustomLabelPosition == "bottom") {
                                element.GaugeEl.append(element.canvasEl);
                                element.GaugeEl.append(element.outerDiv);
                                element.outerDiv.css('text-align', 'center');
                            }
                            else {
                                if (element.model.outerCustomLabelPosition != "top") {
                                    var table = ej.buildTag("TABLE");
                                    table.css('width', '100%');
                                    var tr = ej.buildTag("TR");
                                    var td1 = ej.buildTag("TD");
                                    var td2 = ej.buildTag("td");
                                    if (element.model.outerCustomLabelPosition == "left") {
                                        td1.append(element.outerDiv);
                                        td2.append(element.canvasEl);
                                    }
                                    else {
                                        td1.append(element.canvasEl);
                                        td2.append(element.outerDiv);
                                    }
                                    tr.append(td1);
                                    tr.append(td2);
                                    table.append(tr);
                                    element.GaugeEl.append(table);
                                    element.outerDiv.css({
                                        'width': this.element.width() - element.model.width
                                    });
                                }
                                else {
                                    element.GaugeEl.append(element.outerDiv);
                                    element.GaugeEl.append(element.canvasEl);
                                    element.outerDiv.css('text-align', 'center');
                                }
                            }
                        }
                        else
                            element.GaugeEl.append(element.canvasEl);
                    }
                  }
				}
                if (outerLabelCount==0)
                    element.GaugeEl.append(element.canvasEl);
            }
            element.canvasEl.attr("role", "presentation");
            if (_gaugeCount == initialGaugeCount)
            {
                initialDivWidth = window.innerWidth;
            }
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var padding = 20, maxRadius, legendActualBounds, position, height, width;
            if (this.model.legend.visible) {
			    if (this.model.theme == "flatdark") this.model.legend.font.color = (ej.util.isNullOrUndefined(this.model.legend.font.color)) ? "#8c8c8c" : this.model.legend.font.color;
                else this.model.legend.font.color = (ej.util.isNullOrUndefined(this.model.legend.font.color)) ? "#282828" : this.model.legend.font.color;
                position = this.model.legend.position.toLowerCase();
                legendActualBounds = this._calculateLegendBounds();
                legendActualBounds.Height = (ej.util.isNullOrUndefined(this.model.legend.size._height)) ? legendActualBounds.Height : parseInt(this.model.legend.size.height);
                legendActualBounds.Width = (ej.util.isNullOrUndefined(this.model.legend.size._width)) ? legendActualBounds.Width : parseInt(this.model.legend.size.width);
                this.model.legendActualBounds = legendActualBounds;
                if (legendActualBounds.Height > 0 && legendActualBounds.Width > 0) {
                    switch (position) {
                        case "top":
                            height = (element.model.height - (legendActualBounds.Height + padding));
                            element.centerY = (legendActualBounds.Height + padding) + height / 2;
                            maxRadius = (height / 2) - padding;
                            break;
                        case "bottom":
                            element.centerY = (element.model.height - (legendActualBounds.Height + padding)) / 2;
                            maxRadius = element.centerY - padding;
                            break;
                        case "left":
                            width = (element.model.width - (legendActualBounds.Width + padding));
                            element.centerX = (legendActualBounds.Width + padding) + (width / 2);
                            maxRadius = (width / 2) - (padding * 2);
                            break;
                        case "right":
                            element.centerX = (element.model.width - (legendActualBounds.Width + padding)) / 2;
                            maxRadius = element.centerX - (padding * 2);
                            break;
                    }

                    for (var i = 0; i < this.model.scales.length; i++) {
                        var scale = this.model.scales[i], totalSize = 0, rangeSize;
                        if (!this._scaleRedrawn)
                            scale._radius = scale.radius;
                        $.each(scale.ranges, function (rangeIndex, rangeElement) { totalSize += rangeElement.size; });
                        rangeSize = totalSize / scale.ranges.length;
                        maxRadius = Math.abs((maxRadius - rangeSize) / (i + 1));
                        maxRadius = scale.radius = (scale._radius < maxRadius) ? scale._radius : maxRadius;
                    }
                }
            }
            else
                $.each(this.model.scales, function (scaleIndex, scaleElement) {
                    scaleElement.radius = (!ej.util.isNullOrUndefined(scaleElement._radius)) ? scaleElement._radius : scaleElement.radius;
                });
            
            if (this._isHalfCircle && this._isHalfCircle()) {
                if ((element.model.frame.halfCircleFrameEndAngle - element.model.frame.halfCircleFrameStartAngle) >= 180) {
                    if (element.model.frame.halfCircleFrameStartAngle == 0) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.width / 2;
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width / 2;
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.width / 2;
                                break;
                        }
                    }
                    else if(element.model.frame.halfCircleFrameStartAngle == 90){
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = element.model.height / 2;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = element.model.height / 2;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "middleright":
                                element.centerY = element.model.height / 2;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 180) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.width / 2;
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width / 2;
                                break;
                            case "middleleft":
                                element.centerY = element.model.height / 2 + element.model.radius / 2;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "middleright":
                                element.centerY = element.model.height / 2 + element.model.radius / 2;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width / 2;
                                break;
                        }
                    }
                }
                else if (element.model.frame.halfCircleFrameStartAngle == 270 && element.model.frame.halfCircleFrameEndAngle == 90) {
                    switch (element.model.gaugePosition) {
                        case "center":
                            element.centerY = element.model.height / 2;
                            element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                            break;
                        case "topleft":
                            element.centerY = element.model.radius + element.model.distanceFromCorner;
                            element.centerX = element.model.distanceFromCorner;
                            break;
                        case "topright":
                            element.centerY = element.model.radius + element.model.distanceFromCorner;
                            element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                            break;
                        case "topcenter":
                            element.centerY = element.model.radius + element.model.distanceFromCorner;
                            element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                            break;
                        case "middleleft":
                            element.centerY = element.model.height / 2;
                            element.centerX = element.model.distanceFromCorner;
                            break;
                        case "middleright":
                            element.centerY = element.model.height / 2;
                            element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                            break;
                        case "bottomleft":
                            element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                            element.centerX = element.model.distanceFromCorner;
                            break;
                        case "bottomright":
                            element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                            element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                            break;
                        case "bottomcenter":
                            element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                            element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                            break;
                    }
                }
                else if ((element.model.frame.halfCircleFrameEndAngle - element.model.frame.halfCircleFrameStartAngle) <= 90) {
                    if (element.model.frame.halfCircleFrameStartAngle == 0) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = (element.model.width / 2) - (element.model.radius/2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 90) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = (element.model.width/2) + (element.model.radius/2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 180) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomleft":
                                element.centerY=element.model.height-element.model.distanceFromCorner;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 270) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.radius / 2) + (element.model.height / 2);
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.radius / 2) + (element.model.height / 2);;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height-element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                        }
                    }
                }
            }
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
        _browserInfo: function () {
            var browser = {}, clientInfo = [],
            browserClients = {
                webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
                opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
            };
            for (var client in browserClients) {
                if (browserClients.hasOwnProperty(client)) {
                    clientInfo = navigator.userAgent.match(browserClients[client]);
                    if (clientInfo) {
                        browser.name = clientInfo[1].toLowerCase();
                        browser.version = clientInfo[2];
                        if (!!navigator.userAgent.match(/Trident\/7\./)) {
                            browser.name = "msie";
                        }
                        break;
                    }
                }
            }
            browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
            browser.pointerEnabled = window.navigator.pointerEnabled;
            return browser;
        },
		 	
        _wireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent),
                browserInfo = this._browserInfo(),
                isPointer = browserInfo.isMSPointerEnabled,
	            isIE11Pointer = browserInfo.pointerEnabled;
            this.startEv = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
            this.endEv = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
            this.moveEv = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
            this.leaveEv = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchleave mouseleave",
            this.scrollEv = matched.browser.toLowerCase() == "mozilla" ? (isIE11Pointer ? "mousewheel" : "DOMMouseScroll") : "mousewheel";
            this.model.browserInfo = browserInfo;
            
            var action= this.model.readOnly? 'pan-y pan-x' : 'none';
            $(this.element).css('touch-action', action);
            
            this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            this.onHoverOCustomLabel = $.proxy(this._onHoverOCustomLabel, this);
            this.onLeaveOCustomLabel = $.proxy(this._onLeaveOCustomLabel, this);
            if (this.model.legend.visible){
                this._on($(this.canvasEl), "click", this.legendMouseClick);
            }
            this._on($(this.canvasEl), "mousemove", this.rangesMouseMove);
            if (this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) {
                $(this.canvasEl).bind(this.moveEv, this.onMouseMoveHandler);
                $(this.canvasEl).bind(this.scrollEv, this.onMouseMoveHandler);
                $(this.canvasEl).bind(this.startEv, this.onMouseDownHandler);
                $(this.canvasEl).bind(this.endEv, this.onLeaveOCustomLabel);
                $(this.canvasEl).bind(this.leaveEv, this.onLeaveOCustomLabel);
            }
            if (!this.model.readOnly) {
                $(this.canvasEl).bind(this.startEv, this.onMouseDownHandler);
            }
            if (this.model.tooltip.showCustomLabelTooltip){
                $('.'+this._id + 'outercustomlbl').bind("mouseenter", this.onHoverOCustomLabel);
                $('.' + this._id + 'outercustomlbl').bind("this.leaveEv", this.onLeaveOCustomLabel);
            }
        },
		 
        _unWireEvents: function () {
            $(this.canvasEl).unbind(this.startEv, this.onMouseDownHandler);
        },
       
        _onHoverOCustomLabel: function(e) {
            if (e.currentTarget.innerHTML!=null || e.currentTarget.innerHTML!="")
                this._showTooltip(e, e.currentTarget.innerHTML);
        },
      
        _onLeaveOCustomLabel: function (evt) {
            if (this.isTouch(evt)) {
                this._performTooltip(evt);
                window.clearTimeout(this.model.timer);
				this.model.timer = setTimeout(function () {
					$('.tooltipDiv').fadeOut(500);
	            }, 1200);
            }
            else {
                this._hideTooltip();
            }
        },
     
        isTouch: function (evt) {
            var event = evt.originalEvent ? evt.originalEvent : evt;
            if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        },

        _showTooltip: function (event, val) {
			var locale = this.model.locale;
            var tooltipText = (locale && this.model.enableGroupSeparator) ? val.toLocaleString(locale) : val.toString();
            var tooltipdiv = $('.tooltipDiv');
            if (tooltipdiv.length == 0) {
                tooltipdiv = $("<div class='tooltipDiv' style='pointer-events:none; position: absolute; z-index: 105; display: block;'></div>");
                $(document.body).append(tooltipdiv);
            }
            if (this.model.tooltip.templateID != "" && this.model.tooltip.templateID != null) {
                var cloneNode = $("#" + this.model.tooltip.templateID).clone();
                $('.tooltipDiv')[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                $(tooltipdiv).css({
                    'pointer-events': 'none',
                    'background-color': this.model.backgroundColor,
                    'border': '1px solid #bbbcbb',
                    'border-radius': '3px',
                    'color': '#565656'
                });
                tooltipdiv.html(tooltipdiv.html().replace('#label#', tooltipText));
            } else {
                $(tooltipdiv).html(tooltipText);
                $(tooltipdiv).css({
                'pointer-events':'none',
                'background-color': 'white',
                'border': '2px solid #bbbcbb',
                'position': 'absolute',
                'padding': '10px 20px',
                'margin-top': '5px',
                'text-align': 'left',
                'font': '12px Segoe UI',
                'font-stretch': 'condensed',
                'display': 'inline-block',
                'border-radius': '3px',
                'color': '#565656',
                'width': 'auto'
                });
            }
            var tooltipMargin = 10;
            var xPos = event.pageX + tooltipMargin;
            var yPos = event.pageY + tooltipMargin;
            xPos = (((xPos + $(tooltipdiv).width())  < this.model.width)) ? (xPos) : (xPos - $(tooltipdiv).width());
            yPos = ((yPos + $(tooltipdiv).height()) < this.model.height) ? (yPos) : (yPos - $(tooltipdiv).height());
            $(tooltipdiv).css("left", xPos);
            $(tooltipdiv).css("top", yPos);
            $('.tooltipDiv').show();
        },
       
        _hideTooltip: function () {
            $('.tooltipDiv').remove();//.fadeOut(0, "linear");
        },
	 
        _onMouseDown: function (e) {
            this._blockDefaultActions(e);
            this._mouseDown = true;
            var padding = this.isTouch(e) ? 10 : 0;
            var touch = e.originalEvent.touches ? e.originalEvent.touches[0] : e;
            var startPoint = { "x": this.centerX, "y": this.centerY };
            var endPoint = { "x": touch.pageX - $(this.canvasEl).offset().left, "y": touch.pageY - $(this.canvasEl).offset().top };
            var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
            var scaleRadius, majorTickHeight, self = this;
            if (!this.model.readOnly) {
                $.each(this.model.scales, function (index, element) {
                    self.scaleIndex = index;
                    if (element.pointers != null) {
                        self.pointerEl = element.pointers;
                        scaleRadius = element.radius;
                        majorTickHeight = element.ticks[0].height;
                        $.each(element.pointers, function (pointerIndex, pointerElement) {
                            if (self._isHalfCircle())
                                element.showBackNeedle = false;
                            var angle = self._getAngle(pointerElement.value);
                            if (angle > 360)
                                angle = angle - 360;
                            var greatervalue = angle + pointerElement.width;
                            var lesservalue = angle - pointerElement.width;
                            var radius = Math.sqrt((endPoint.x - startPoint.x) * (endPoint.x - startPoint.x) + (endPoint.y - startPoint.y) * (endPoint.y - startPoint.y));
                            //condition added for the markerpointer whether the user mousedown is made on marker pointer location or not.
                            var cond = radius < ((pointerElement.placement == "far") ? scaleRadius + pointerElement.width + pointerElement.distanceFromScale : (pointerElement.placement == "center") ? scaleRadius - pointerElement.distanceFromScale : scaleRadius - 15 - majorTickHeight - pointerElement.distanceFromScale) && radius > ((pointerElement.placement == "far") ? scaleRadius + pointerElement.distanceFromScale : (pointerElement.placement == "center") ? scaleRadius - pointerElement.width - pointerElement.distanceFromScale : scaleRadius - pointerElement.width - 15 - majorTickHeight - pointerElement.distanceFromScale);
                            var chkPointerType = (pointerElement.type == "needle") ? radius <= pointerElement.length : cond;
                            if (self._isBetween(lesservalue, greatervalue, indegree, padding) && chkPointerType) {
                                self._onMouseClick(angle, element.value);
                                self.activeElement = pointerElement;
                                $(document).bind(self.moveEv, self.onMouseMoveHandler);
                                $(document).bind(self.endEv, self.onMouseUpHandler);
                            }
                        });
                    }
                });
            }
        },
	 
        _onMouseUp: function () {
            this._mouseDown=false;
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            if (this.activeElement)
                this._onMouseClickUp(this._getAngle(this.activeElement.value), this.activeElement.value);
            this.activeElement = null;
        },
        _mousePosition: function (evt) {
            if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return { x: evt.pageX, y: evt.pageY };
            else if (evt.originalEvent && !ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX > 0)
                return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
            else if (evt.originalEvent && evt.originalEvent.changedTouches != undefined) {
                if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                    return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
            }
            else
                return { x: 0, y: 0 };

        },
        _calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseposition = this._mousePosition(e);
            e.pageX = mouseposition.x;
            e.pageY = mouseposition.y;
        },
        getEvent: function (event) {
            return (event.targetTouches && event.targetTouches[0]) ? event.targetTouches[0] : event
        },
        _onMouseMove: function (e) {
            if (this._mouseDown && !ej.isNullOrUndefined(this.activeElement)) {
                this._blockDefaultActions(e);
                var startPoint = { "x": this.centerX, "y": this.centerY };
                var touch = e.originalEvent.touches ? e.originalEvent.touches[0] : e;
                var endPoint = { "x": touch.pageX - $(this.canvasEl).offset().left, "y": touch.pageY - $(this.canvasEl).offset().top };
                var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
                if (indegree < this.scaleEl[this.scaleIndex].startAngle && !this._isHalfCircle())
                    indegree = indegree + 360;
                this._onMouseClickMove(this._getAngle(this.activeElement.value), this.activeElement.value);
                if (this._getValue(indegree) <= this.scaleEl[this.scaleIndex].maximum) {
                    if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                        this.activeElement.value = this._getValue(indegree);
                    else
                        this.activeElement.value = this.scaleEl[this.scaleIndex].maximum - this._getValue(indegree);
                }
                if (this.contextEl.putImageData)
                    this._reDrawPointer();
                else
                    this._init();
            }
            else
            {
                if ((this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) && (!this.isTouch(e)))
                {

                    this._performTooltip(e);
                    
                }
            }
        },

        _performTooltip: function (e) {
            var hit = false;
            var padding = 10;
            var isTouch = this.isTouch(e);
            for (var i = 0; this._savedPoints[i] != null; i++) {
                if (isTouch) {
                    var mousedownCords = this._calTouchPosition(e),
                        pointer = this.getEvent(e),
                        currX = pointer.pageX,
                        currY = pointer.pageY;
                    var current = { "X": currX - $(this.canvasEl).offset().left, "Y": currY - $(this.canvasEl).offset().top };
                    if (current.X > this._savedPoints[i].startX - padding && current.X < (this._savedPoints[i].startX + this._savedPoints[i].width + padding) && current.Y > this._savedPoints[i].startY - padding && current.Y < (this._savedPoints[i].startY + this._savedPoints[i].height + padding)) {
                        this._showTooltip(e, this._savedPoints[i].value);
                        hit = true;

                    }
                    else if (hit == false) {
                        this._hideTooltip();
                    }
                } else {
                    var current = { "X": e.pageX - $(this.canvasEl).offset().left, "Y": e.pageY - $(this.canvasEl).offset().top };
                    if (current.X > this._savedPoints[i].startX && current.X < (this._savedPoints[i].startX + this._savedPoints[i].width) && current.Y > this._savedPoints[i].startY  && current.Y < (this._savedPoints[i].startY + this._savedPoints[i].height)) {
                        this._showTooltip(e, this._savedPoints[i].value);
                        hit = true;
                    }
                    else if (hit == false)
                        this._hideTooltip();
                }
            }
        },

		 	
        _isHalfCircle: function () {
            if (this.model.frame.frameType == "halfcircle")
                return true;
            else
                return false;
        },
        
        _calFontLength: function (font) {
            var minText = this.model.scales[0].minimum + this.model.scales[0].labels[0].unitText;
            var maxText = this.model.scales[0].maximum + this.model.scales[0].labels[0].unitText;
            var text = maxText.length > minText.length ? maxText : minText;
            var length = $('<span id="test"></span>').css({ font: font, display: 'none', whiteSpace: 'nowrap' }).appendTo($('body')).text(text).width();
            $("#test").remove();
            return length/2;
        },
		 
        _getHalfCircleYPosition: function () {
            return this._getYCordinate(this.centerY, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2);
        },
		 
        _getHalfCircleXPosition: function () {
            return this._getXCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2);
        },
		 
        _getXCordinate: function (x, radius, angle) {
            var x1 = x + radius * (Math.cos(Math.PI * (angle / 180)));
            return x1;
        },
		 	
        _getYCordinate: function (y, radius, angle) {
            var y1 = y + radius * (Math.sin(Math.PI * (angle / 180)));
            return y1;
        },
		 
        _getAngle: function (value) {
            var tempValue, angle;
            if (value >= this.scaleEl[this.scaleIndex].minimum && value <= this.scaleEl[this.scaleIndex].maximum)
                tempValue = this.scaleEl[this.scaleIndex].direction == "clockwise" ? value - this.scaleEl[this.scaleIndex].minimum : this.scaleEl[this.scaleIndex].maximum - value;
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise") {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                }
                else {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                }
            }
            angle = (tempValue * ((this.scaleEl[this.scaleIndex].sweepAngle) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum))) + (this.scaleEl[this.scaleIndex].startAngle);
            return angle;
        },
		 
        _subtractDecimal: function (value, interval) {
            var strValue = value.toString();
            var strInterval = interval.toString();
            var vDecimal;
            var iDecimal;
            if (strValue.indexOf('.') > -1)
                vDecimal = strValue.length - strValue.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (strInterval.indexOf('.') > -1)
                iDecimal = strInterval.length - strInterval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var decimal = vDecimal > iDecimal ? vDecimal : iDecimal;
            var correctValue = (value * Math.pow(10, decimal) - interval * Math.pow(10, decimal)) / Math.pow(10, decimal);
            return correctValue;
        },
		 	
        _getCirucumferenceAngle: function (startPoint, endPoint) {
            if (endPoint.x > startPoint.x) {
                if (endPoint.y > startPoint.y)
                    return this._tangent(startPoint, endPoint); //quadrant2
                else {
                    if (endPoint.y == startPoint.y)
                        return 0;
                    else
                        return 2 * Math.PI + this._tangent(startPoint, endPoint); //quadrant1
                }
            }
            else {
                if (endPoint.x == startPoint.x) {
                    if (endPoint.y == startPoint.y)
                        return 0;   //0 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI / 2;  // 90 degree
                        else
                            return 1.5 * Math.PI; // 270 degree
                    }
                }
                else {
                    if (endPoint.y == startPoint.y)
                        return Math.PI; // 180 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI + this._tangent(startPoint, endPoint);  // quadrant3
                        else
                            return Math.PI + this._tangent(startPoint, endPoint); //quadrant4
                    }
                }
            }
        },
       
        _calcDistanceFactor: function (angle, placement, labelLength) {
            var distanceFactor;
            if ((angle > 240 && angle <= 300) || (angle > 60 && angle <= 120))
                distanceFactor = 0;
            else if ((angle > 330 && angle <= 360) || (angle >= 0 && angle <= 30))
                distanceFactor = -labelLength * 0.5;
            else if ((angle > 30 && angle <= 60) || (angle > 300 && angle <= 330))
                distanceFactor = -labelLength / 2;
            else if ((angle > 150 && angle <= 210))
                distanceFactor = labelLength * 0.5;
            else
                distanceFactor = labelLength / 2;
            return placement == "far" ? -distanceFactor : distanceFactor;
        },
		 
        _tangent: function (startPoint, endPoint) {
            var distance = (endPoint.y - startPoint.y) / (endPoint.x - startPoint.x);
            var inradians = Math.atan(distance);
            //indegrees=180*inradians/Math.PI
            return inradians;
        },
		 
        _getValue: function (angle) {
            if (this.scaleEl[this.scaleIndex].direction == "counterclockwise")
                return ((angle - this.scaleEl[this.scaleIndex].startAngle) / this.scaleEl[this.scaleIndex].sweepAngle) * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum);
            else
                return ((angle - this.scaleEl[this.scaleIndex].startAngle) / this.scaleEl[this.scaleIndex].sweepAngle) * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) + this.scaleEl[this.scaleIndex].minimum;
        },
		 
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.model.scales;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.showScaleBar)
                    self._setScaleCordinates(element);
            });
            if (this.model.rangeZOrder  == "rear") {
                this._setRanges();
                this._setTicks();
            }
            else {
                this._setTicks();
                this._setRanges();
            }
            this._setLabels();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            if (!this.contextEl.putImageData)
                this.model.enableAnimation = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    var myNav = navigator.userAgent.toLowerCase();
					var IEVersion = (myNav.indexOf('msie') != -1) ? parseInt(myNav.split('msie')[1]) : 0;
                    if (this.model.enableAnimation && IEVersion != 9) {
						for (var i=0; i< this.model.scales[0].pointers.length; i++){
							this.pointerValue = [];
							this.currentValue = [];
							this.pointerValue[i] = null;
							this.currentValue[i] = null;
							this.updatePointerOnAnimation = false;
						}
						this.dt = new Date().getTime();
                        this._onAnimate(this);
                    }
                }
            }
            this._setIndicators();
        },
		 
        _setTicks: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (tickIndex, tickElement) {
                            self.tickIndex = tickIndex;
                            self._setTicksCordinates(tickIndex, tickElement);
                        });
                    }
                }
            });
        },
	 
        _setLabels: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (labelIndex, labelElement) {
                            self.labelIndex = labelIndex;
                            self._setLabelCoridnates(labelIndex, labelElement);
                        });
                    }
                }
            });
        },
		 
        _setIndicators: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showIndicators) {
                    self.scaleIndex = index;
                    if (element.indicators != null) {
                        self.indicatorEl = element.indicators;
                        $.each(element.indicators, function (indIndex, indElement) {
                            self.indicatorIndex = indIndex;
                            self._drawIndicator(indIndex, indElement);
                        });
                    }
                }
            });
        },
		 
        _setPointers: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    self.scaleIndex = index;
                    if (element.pointers != null) {
                        self.pointerEl = element.pointers;
                        $.each(element.pointers, function (pointerIndex, pointerElement) {
                            if (self._isHalfCircle())
                                element.showBackNeedle = false;
                            self.pointerIndex = pointerIndex;
                            self._drawPointers(pointerIndex, pointerElement);
                            //  setInterval( function(){ self._drawPointers(pointerIndex,pointerElement);}, 30 ); 
                        });
                    }
                }

            });
        },
		 
        _onAnimate: function (obj) {
		if(!this._androidAnimation && !ej.util.isNullOrUndefined(obj.model)){
		  var circularGaugeObject = obj, self = circularGaugeObject,
			  delay = self.model.animationSpeed / 25, pointerValue,i ,j,
		      pointersLength = obj.model.scales[0].pointers.length, flag = [];
		  for (i= 0; i< pointersLength; i++) {
			flag[i] = true;
			 if (!self.model.scales[0].pointers[i]._value) 
				self.model.scales[0].pointers[i]._value = self.model.scales[0].pointers[i].value;
			 self.pointerValue[i] = ej.util.isNullOrUndefined(self.pointerValue[i]) ? self.model.scales[0].pointers[i]._value : self.pointerValue[i];
            self.currentValue[i] = !self.currentValue[i] ? self.model.scales[0].minimum : self.currentValue[i];
            pointerValue = self.pointerValue[i];
			now = new Date().getTime();
	        delta = now - self.dt;
                if ((delta > delay) && (pointerValue > self.currentValue[i])) {
                    self.currentValue[i] = self.currentValue[i] + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
					self.updatePointerOnAnimation = true;
                    pointerValue > self.currentValue[i] ? self.setPointerValue(0, i, self.currentValue[i]) : self.setPointerValue(0, i, pointerValue);
					self.dt = now - (delta % delay);
                }
                else if (self.currentValue >= pointerValue)
                    flag[i] = false;
		    }
		  for (j=0; j< flag.length; j++){
				animationFlag = flag[j];
				if (animationFlag) break;
			}	
		  if (animationFlag) {
	            requestAnimationFrame(function () {
	            circularGaugeObject._onAnimate(circularGaugeObject);
	            });
	        }
		}
        },
		
		_pointInterval: function (currentValue, pointerValue, delay, self) {
			 this.timer = setTimeout(function pointerchan() {
                if (pointerValue > currentValue) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    pointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, pointerValue);
                }
                else if (currentValue >= pointerValue) {
                    window.clearInterval(self.timer);
                }
				self._pointInterval(currentValue, pointerValue, delay, self);
               }, delay); 
		},
		 
        _setRanges: function () {
            var self = this;
            this._rangeCollectionRegions = [];
            $.each(this.model.scales, function (index, element) {
                if (element.showRanges) {
                    self.scaleIndex = index;
                    if (element.ranges != null) {
                        self.rangeEl = element.ranges;
                        $.each(element.ranges, function (rangeIndex, rangeElement) {
                            self.rangeIndex = rangeIndex;
                            if (rangeElement._visible || !self.model.legend.visible)
                               self._setRangeCordinates(rangeIndex, rangeElement);
                        });
                    }
                }
            });
        },
		 
        _setCustomLabel: function () {
            var self = this, currentCustomLabel;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabels !=null) {
                    self.customLabelEl = element.customLabels;
                    $.each(element.customLabels, function (labelIndex, labelElement) {
                        self.customLabelIndex = labelIndex;
						currentCustomLabel = self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex];
						if (currentCustomLabel.value != null ) {
							if (currentCustomLabel.positionType == "outer")
								self._setOuterCustomLabels(labelIndex, labelElement);
							else
								self._setCustomLabelCordinates(labelIndex, labelElement);
						}
                    });
                }
            });
        },
	 
        _subGauge: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.subGauges != null) {
                    self.subGaugeEl = element.subGauges;
                    $.each(element.subGauges, function (subGaugeIndex, subGaugeElement) {
                        self.subGaugeIndex = index;
                        self._setSubGauge(subGaugeIndex, subGaugeElement);
                    });
                }
            });
        },
        
        _setOuterCustomLabels: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            var div = ej.buildTag("div."+this._id+"outercustomlbl");
            div.text(this.model.scales[this.scaleIndex].customLabels[index].value);
            var position=(this.model.outerCustomLabelPosition=="right" || this.model.outerCustomLabelPosition=="left")?"left":"center";
            this.outerDiv.append(div);
            this.outerDiv.append('</br>');
            if (position == "center")
                div.css({ 'display': 'inline-block', 'margin': '0 auto', 'max-width': this.model.width });
            else
                div.css({ 'display': 'inline-block', 'max-width': this.element.width() - this.model.width > 10 ? this.element.width() - this.model.width : 10 });
            div.css({
                "color": element.color,
                'overflow': 'hidden',
                'text-overflow': 'ellipsis',
                'white-space': 'nowrap',
                'font-size': (element.font != null && element.font.size != null) ? element.font.size : "12px",
                'font-family': (element.font != null && element.font.fontFamily != null) ? element.font.fontFamily : "Arial",
                'font-weight': (element.font != null && element.font.fontStyle != null) ? element.font.fontStyle : "Normal",
                'text-align': position
            });

        },
	 	
        _setScaleCordinates: function (element) {
            this.region = {
                "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY,
                "startAngle": element.startAngle, "endAngle": (element.startAngle + element.sweepAngle),
                "startRadius": this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2, "endRadius": this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2
            };
            this.style = {
                "radius": element.radius - element.size / 2,
                "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color), // this._getColor(element, element.borderColor),
                "lineWidth": element.border.width,
                "size": element.size,
                "isFill": true,
                "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "isStroke": true,
                "shadowOffset": element.shadowOffset,
                "fillStyle": (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor), //this._getColor(element, element.backgroundColor),
                "counterClockwise": element.direction == "clockwise" ? false : true
            };
            if (element.maximum < element.minimum) {
                var tempValue = element.maximum;
                element.maximum = element.minimum;
                element.minimum=tempValue;
            }
			 if (element.maximum == element.minimum)
                element.maximum = element.maximum + 1;
            this.maximum(element.maximum);
            this.minimum(element.minimum);
            if (this._notifyArrayChange) {
                this._notifyArrayChange("scales[" + this.scaleIndex + "]maximum", element.maximum);
                this._notifyArrayChange("scales[" + this.scaleIndex + "]minimum", element.minimum);
            }
            this._drawScaleBar(this.region, this.style);
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 	
        _drawOuterLayer: function () {
            var interiorGradient, startPoint, endPoint;
            if (this._isHalfCircle()) {
                startPoint = { "x": this.centerX, "y": this.centerY };
                endPoint = { "x": this.centerX + this.model.radius, "y": this.centerY / 2 };
            }
            if (!this.model.isRadialGradient) {
                interiorGradient = this.contextEl.createLinearGradient(0, 0, 0, this.canvasEl[0].height);
            }
            else {
                interiorGradient = this.contextEl.createRadialGradient(this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, 0, this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, this.model.radius);
            }
            if (!ej.isNullOrUndefined(this.model.interiorGradient))
                this._setGradientColor(this, interiorGradient, this.model.interiorGradient.colorInfo);

            this.frameOuterLocation = { "centerX": this.centerX, "hcCenterX": this.centerX, "hcCenterY": this.centerY, "centerY": this.centerY, "startAngle": this._isHalfCircle() ? Math.PI * ((this.model.frame.halfCircleFrameStartAngle) / 180) : 0, "endAngle": this._isHalfCircle() ? Math.PI * ((this.model.frame.halfCircleFrameEndAngle) / 180) : 2 * Math.PI };
            if (this._isHalfCircle())
                this.frameInnerLocation = { "centerX": this.centerX, "hcCenterX": this._getXCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2), "hcCenterY": this._getYCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2), "centerY": this.centerY, "startAngle": Math.PI * ((this.model.frame.halfCircleFrameStartAngle) / 180), "endAngle": Math.PI * ((this.model.frame.halfCircleFrameEndAngle) / 180) };
            this.frameInnerStyle = {
                "radius": this.model.radius,
                "isStroke": false, "isFill": true,
                "fillStyle": (this.model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(null, this.model.backgroundColor),
                "counterClockwise": false
            };
            if (this.model.frame.frameType == "fullcircle") {
                this._drawCircleFrame(this.frameOuterLocation, this.frameInnerStyle);
            }
            else if (this.model.frame.frameType == "halfcircle") {
                this._drawHalfCircle(this.frameInnerLocation, this.frameInnerStyle);
            }
            if (this.contextEl.getImageData)
                this.outerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 
        _setTicksCordinates: function (index, element) {
            var startX, startY, lineHeight, minimumMeet, interval, radius, angle;
            if (element.type == "major") {
                interval = this.scaleEl[this.scaleIndex].majorIntervalValue;
                this.majorTickHeight = element.height;
            }
            else {
                interval = this.scaleEl[this.scaleIndex].minorIntervalValue;
            }
            if (element.placement == "far")
                radius = this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
            if (element.placement == "center")
                radius = this.scaleEl[this.scaleIndex].radius - element.height / 2 - this.scaleEl[this.scaleIndex].border.width / 2 - element.distanceFromScale;
            if (element.placement == "near")
                radius = this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - this.scaleEl[this.scaleIndex].border.width / 2 - element.distanceFromScale;
            lineHeight = element.placement == "near" ? -element.height : element.height;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum)
                        minimumMeet = true;
                    angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.region = { "startX": startX, "startY": startY };
                    this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.width, "strokeStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                    if (this.model.drawTicks)
                        this._onDrawTicks(angle, value);
                    this._drawTickMark(this.region, this.style);
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.region = { "startX": startX, "startY": startY };
                this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.width, "strokeStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                //if (this.model.drawTicks)
                //    this._onDrawTicks(angle, value);
                //this._drawTickMark(this.region, this.style);
            }

            if (this.contextEl.getImageData)
                this.tickImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 
        _setLabelCoridnates: function (index, element) {
            var minimumMeet;
            var startX, startY, lineHeight, incrementAngle, interval, radius;
            interval = element.type == "major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
            incrementAngle = element.type == "major" ? (this.majorIntervalAngle = this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval)) : this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval);
            if (element.placement == "far")
                radius = this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + this.majorTickHeight + element.distanceFromScale;
            if (element.placement == "center")
                radius = this.scaleEl[this.scaleIndex].radius - 10 - element.distanceFromScale;
            if (element.placement == "near")
                radius = this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - 10 - this.majorTickHeight - element.distanceFromScale;
            this._labelRadius = radius;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value = this._subtractDecimal(value, interval)) {
                if (value == this.scaleEl[this.scaleIndex].minimum)
                    minimumMeet = true;
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum && !element.includeFirstValue) {
                        continue;
                    }
                    var angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.region = { "startX": startX, "startY": startY };
                    this.style = { "placement": element.placement, "textPositionAngle": angle, "angle": this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": value, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                    if (this.model.drawLabels)
                        this._onDrawLabels(this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, value);
                    this._drawLabel(this.region, this.style, false);
                    if (this.model.tooltip.showLabelTooltip)
                        this._savedPoints.push({ "startX": startX - 10, "startY": startY-5, "width": 20, "height": parseInt(element.font.size.replace(/\D/g, '')), "value": value });
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.region = { "startX": startX, "startY": startY };
                this.style = { "angle": this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": this.scaleEl[this.scaleIndex].minimum, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "lineHeight": lineHeight, "lineWidth": element.width, "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                if (this.model.drawLabels)
                    this._onDrawLabels(angle, value);
                this._drawLabel(this.region, this.style, false);
                if (this.model.tooltip.showLabelTooltip)
                     this._savedPoints.push({ "startX": startX - 10, "startY": startY-5, "width": 20, "height": parseInt(element.font.size.replace(/\D/g, '')), "value": value });
                angle = this._getAngle(value);
                angle = angle > 360 ? angle - 360 : angle;
            }
            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 
        _setRangeCordinates: function (index, element) {
                if (element.startValue < this.scaleEl[this.scaleIndex].maximum&& element.endValue <= this.scaleEl[this.scaleIndex].maximum) {
                    var tempRangeStart = element.startValue >= this.scaleEl[this.scaleIndex].minimum ? element.startValue : this.scaleEl[this.scaleIndex].minimum;
                    var tempRangeEnd = element.endValue > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.endValue;
                var startX, startY, endX, endY, startAngle, radius, endAngle, startRadius, endRadius, grd, controlAngle;
                controlAngle = this._getAngle((tempRangeStart + tempRangeEnd) / 2);
                startAngle = this._getAngle(tempRangeStart);
                endAngle = this._getAngle(tempRangeEnd);
                radius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - this.scaleEl[this.scaleIndex].size / 2 - element.size - (ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].ticks[0]) ? 16 : this.scaleEl[this.scaleIndex].ticks[0].height);
                startRadius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - this.scaleEl[this.scaleIndex].size / 2 - (ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].ticks[0]) ? 16 : this.scaleEl[this.scaleIndex].ticks[0].height);
                endRadius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - element.size - this.scaleEl[this.scaleIndex].size / 2 - (ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].ticks[0]) ? 16 : this.scaleEl[this.scaleIndex].ticks[0].height);
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, startAngle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, startAngle);
                endX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, endAngle);
                endY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, endAngle);
                startAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": startX, "y": startY })) / Math.PI;
                endAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": endX, "y": endY })) / Math.PI;

                if (!ej.isNullOrUndefined(element.gradients) && element.gradients.colorInfo.length > 0) {
                    grd = this.contextEl.createRadialGradient(this.centerX, this.centerY, endRadius, this.centerX, this.centerY, startRadius);
                    this._setGradientColor(this, grd, element.gradients.colorInfo);
                } else
                    grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor); // this._getColor(element, element.backgroundColor);
                this.region = { "startX": startX, "startY": startY, "endX": endX, "endY": endY, "startAngle": startAngle, "endAngle": endAngle };
                this.style = {
                    "placement":element.placement,
                    "radius": element.placement=="near"?this.scaleEl[this.scaleIndex].radius - element.distanceFromScale:this.scaleEl[this.scaleIndex].radius + element.distanceFromScale,
                    "rangeStart": tempRangeStart,
                    "rangeEnd": tempRangeEnd,
                    "startWidth": element.startWidth,
                    "isFill": true,
                    "fillStyle": grd,
                    "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : element.border.color, // element.borderColor,
                    "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                    "counterClockwise": this.scaleEl[this.scaleIndex].direction == "clockwise" ? false : true,
                    "startRadius": startRadius, "endRadius": endRadius, "endWidth": element.endWidth, "lineWidth": element.border.width, "isStroke": true
                };
                if (this.model.drawRange)
                    this._onDrawRange(this.region, this.style);
                this._rangeCollectionRegions.push({ Range:element, Region: this.region, scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, Radius: { startRadius: this.style.startRadius, endRadius: this.style.endRadius }, centerRadius: { centerX: this.centerX, centerY: this.centerY, radius: this.style.radius } });
                this._drawRange(this.region, this.style);
                if (this.contextEl.getImageData)
                this.rangeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
                }
        },
		 
        _setSubGauge: function (index, element) {
            var subGaugeDiv = $('div[id=' + element.controlID + ']');
            if (subGaugeDiv.length > 0 && subGaugeDiv.find('canvas').length) {
                var sourceCanvas = subGaugeDiv.find('canvas')[0].getContext('2d');
                this.contextEl.drawImage(sourceCanvas.canvas, element.position.x, element.position.y, element.width, element.height);
                subGaugeDiv.css('display', 'none');
                if (this.contextEl.getImageData)
                    this.subGaugeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            }
        },
		 
        _setCustomLabelCordinates: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            element.color = element.color ? (element.color) : "#282828";
            this.region = element.position ? ({ "startX": element.position.x, "startY": element.position.y }) : { "startX": 0, "startY": 0 };
            this.style = { "angle": element.textAngle, "textValue": element.value, "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color), "font": this._getFontString(element, element.font) };
            if (this.model.drawCustomLabel)
                this._onDrawCustomLabel(this.region, this.style);
            this._drawLabel(this.region, this.style, true);
            if (this.model.tooltip.showCustomLabelTooltip)
                this._savedPoints.push({ "startX": this.region.startX-30, "startY": this.region.startY-5, "width": 60, "height": !ej.isNullOrUndefined(element.font) ? parseInt(element.font.size.replace(/\D/g, '')) : 10, "value": this.style.textValue });
            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
	 
        _drawIndicator: function (index, element) {
            var self = this, isInStateRange = false;
            this.region = { "centerX": element.position.x - element.width / 2, "textLocation": element.position, "centerY": element.position.y - element.height / 2, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = {
                "radius": (element.height + element.width) / 2,
                "strokeStyle": "#2BA104",
                "cornerRadius": element.type == "roundedrectangle" ? 2 : 0,
                "height": element.height,
                "width": element.width,
                "lineWidth": element.indicatorBorderWidth,
                "fillStyle": "#2BA104",
                "isStroke": true,
                "isFill": true,
                "indicatorText": element.indicatorText,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.model.drawIndicators)
                this._onDrawIndicators(this.style, this.region);
            if (element.type == ej.datavisualization.CircularGauge.IndicatorType.Image) {
                var image = new Image();
                image.onload = function () {
                    self.contextEl.drawImage(this, element.position.x, element.position.y);
                };
                image.src = element.imageUrl;
            }
            else if (element.stateRanges != null) {
                $.each(element.stateRanges, function (stateRangeIndex, srEl) {//srEl = StateRange Element
                    if (self.pointerEl[self.pointerIndex].value >= srEl.startValue && self.pointerEl[self.pointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (!ej.isNullOrUndefined(srEl.text) && srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = (srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : srEl.textColor; // srEl.textColor;
                            self.style.font = element.font;
                        }
                        self.style.strokeStyle = (srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : srEl.borderColor;
                        self.style.fillStyle = (srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : srEl.backgroundColor;
                        self._drawIndicatorFrame(element.type, self.region, self.style);
                    }
                });
            }
            if (!isInStateRange && element.type != ej.datavisualization.CircularGauge.IndicatorType.Image) {
                self._drawIndicatorFrame(element.type, self.region, self.style);
            }
            if (this.contextEl.getImageData && element.type != ej.datavisualization.CircularGauge.IndicatorType.Image)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },

        _drawIndicatorFrame: function(type, location,style) {
            switch (type) {
                case "circle":
                    this._drawCircleFrame(location, style);
                    break;
                case "roundedrectangle":
                case "rectangle":
                    this._drawRectangleFrame(location, style);
                    break;
                case "text":
                    this._drawText(location, style);
                    break;
            }
        },
		 	
        _drawScaleBar: function (location, style) {
            this.contextEl.shadowColor = (style.strokeStyle == "transparent") ? "rgba(0,0,0,0)" : style.strokeStyle;
            if (style.shadowOffset)
                this.contextEl.shadowBlur = style.shadowOffset;
            this._contextOpenPath(style, this);
            if ((location.endAngle - location.startAngle) == 0)
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (0 / 180), false);
            else if ((location.endAngle - location.startAngle) % 360 == 0) {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (360 / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (360 / 180), Math.PI * (0 / 180), true);
            }
            else {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), true);
            }
            this._contextClosePath(style, this);
        },
		 
        _drawTickMark: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.lineTo(0, 0);
            this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this._contextClosePath(style, this);
        },
	 
        _drawLabel: function (location, style, isCustomLabel) {
            var distanceFactor = 0, locale = this.model.locale;
            if (!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel) {
                var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPosition;
               style.textValue = ((style.textValue % 1 != 0) ? + parseFloat(style.textValue.toFixed(3)) : style.textValue); 
			    style.textValue = ((locale && this.model.enableGroupSeparator) ? style.textValue.toLocaleString(locale) : style.textValue);
				if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "back")
					style.textValue += this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
                else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "front")
                    style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
                distanceFactor = this._calcDistanceFactor(style.textPositionAngle, style.placement, this._calFontLength(style.font));
            }
            this._contextOpenPath(style, this);
            this.contextEl.textAlign = "center";
            this.contextEl.textBaseline = "middle";
            this.contextEl.font = style.font;
            this.contextEl.translate(location.startX + distanceFactor, location.startY);
            this.contextEl.lineTo(0, 0);
			if(!isCustomLabel)
			    this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? this.contextEl.rotate((Math.PI * ((style.angle - 270) / 180))) : this.contextEl.rotate((Math.PI * (style.angle / 180)));
			else if (isCustomLabel && style.angle != 0)
			    this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.fillText(style.textValue, 0, 0);
            this._contextClosePath(style, this);
        },
		 
        _drawCircleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, style.counterClockwise);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		 
        _drawHalfCircle: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.lineJoin = "round";
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, false);
            this.contextEl.lineTo(location.centerX, location.centerY);
            this._contextClosePath(style, this);
        },
		 
        _drawRectangleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.centerX, location.centerY - style.height / 2);
            this.contextEl.lineTo(style.cornerRadius, 0);
            this.contextEl.lineTo(style.width - style.cornerRadius, 0);
            this.contextEl.quadraticCurveTo(style.width, 0, style.width, style.cornerRadius);
            this.contextEl.lineTo(style.width, style.height - style.cornerRadius);
            this.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.cornerRadius, style.height);
            this.contextEl.lineTo(style.cornerRadius, style.height);
            this.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.cornerRadius);
            this.contextEl.lineTo(0, style.cornerRadius);
            this.contextEl.quadraticCurveTo(0, 0, style.cornerRadius, 0);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
	 
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = (style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor;
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
		 	
        _drawRange: function (location, style) {
            if(style.startWidth == null && style.endWidth == null){
            this._contextOpenPath(style, this);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
            this._contextClosePath(style, this);
            }
            else {
                var widthDifference = style.startWidth > style.endWidth ? style.startWidth - style.endWidth : style.endWidth - style.startWidth;
                var angleDifference = this.scaleEl[this.scaleIndex].direction == "clockwise" ? location.startAngle < location.endAngle ? location.endAngle - location.startAngle : 360 - (location.startAngle - location.endAngle) : location.endAngle < location.startAngle ? location.startAngle - location.endAngle : location.startAngle + (360 - location.endAngle);
                var degreeDifference = angleDifference / (widthDifference * 2);
                if (style.startWidth < style.endWidth) {
                    if (style.startWidth != 0) {
                        this._contextOpenPath(style, this);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.placement == "near" ? style.radius - style.startWidth : style.radius + style.startWidth, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
                        this._contextClosePath(style, this);
                    }
                    style.radius = style.placement == "near" ? style.radius - style.startWidth : style.radius + style.startWidth;
                    style.endWidth -= style.startWidth;
                    style.startWidth = 0;
                    for (var i = style.startWidth; i < style.endWidth; i += 0.5) {
                        this.contextEl.beginPath();
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius, Math.PI * (location.endAngle / 180) , Math.PI * (location.startAngle / 180) , !style.counterClockwise);
                        this.contextEl.lineWidth = 2;
                        this.contextEl.strokeStyle = style.fillStyle;
                        this.contextEl.stroke();
                        this.contextEl.restore();
                        location.startAngle = this.scaleEl[this.scaleIndex].direction == "clockwise" ? location.startAngle + degreeDifference : location.startAngle - degreeDifference;;
                        style.radius = style.placement == "near" ? style.radius - 0.5 : style.radius + 0.5;
                    }
                }
                else {
                    if (style.endWidth != 0) {
                        this._contextOpenPath(style, this);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.placement == "near" ? style.radius - style.endWidth : style.radius + style.endWidth, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
                        this._contextClosePath(style, this);
                    }
                    style.radius = style.placement == "near" ? style.radius - style.endWidth : style.radius + style.endWidth;
                    style.startWidth -= style.endWidth;
                    style.endWidth = 0;
                    for (i = style.endWidth; i < style.startWidth; i += 0.5) {
                        this.contextEl.beginPath();
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius,Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180),  style.counterClockwise);
                        this.contextEl.lineWidth = 2;
                        this.contextEl.strokeStyle = style.fillStyle;
                        this.contextEl.stroke();
                        this.contextEl.restore();
                        location.endAngle = this.scaleEl[this.scaleIndex].direction == "clockwise" ? location.endAngle - degreeDifference : location.endAngle + degreeDifference;
                        style.radius = style.placement == "near" ? style.radius - 0.5 : style.radius + 0.5;
                    }
                }
            }
        },
		 
        _drawPointers: function (index, element) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var startX, startY, angle, height, grd, locale = this.model.locale;
            angle = this._getAngle(element.value);
            angle = angle > 360 ? angle - 360 : angle;
            if (element.type == "needle") {
                if (element.placement == "far")
                    height = element.length + this.scaleEl[this.scaleIndex].size / 2;
                if (element.placement == "center")
                    height = element.length;
                if (element.placement == "near")
                    height = element.length - this.scaleEl[this.scaleIndex].size / 2;
            }
            else {
                height = element.length;
                if (element.placement == "far") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + element.distanceFromScale, angle);
                }
                if (element.placement == "center") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale, angle);
                }
                if (element.placement == "near") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                }
                if (!(this._isHalfCircle && this._isHalfCircle())) {
                    if (this.model.height > this.model.width)
                        startY += (this.model.height - this.model.width) / 2;
                    else if (this.model.height < this.model.width)
                        startY -= (this.model.width - this.model.height) / 2;
                }
            }
            if (height > this.model.radius)
                height = this.model.radius;

            this.region = { "startX": element.type == "needle" ? this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX : startX, "startY": element.type == "needle" ? this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY : startY };
            if (!ej.isNullOrUndefined(element.gradients) && element.gradients.colorInfo.length > 0) {
                grd = this.contextEl.createLinearGradient(0, 0, 0, element.width);
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            } else
                grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor);
            this.style = {
                "width": element.width,
                "isFill": true,
                "isStroke": true,
                "radius": 0,
                "showBackNeedle": element.showBackNeedle,
                "backNeedleLength": element.backNeedleLength,
                "angle": element.type == "needle" ? angle : element.placement == "far" ? angle : angle + 180,
                "height": height, "lineWidth": element.border.width, "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color),
                "fillStyle": grd,
                "imageUrl": element.imageUrl,
                "type":element.type,
            };
            if (this.model.drawPointers)
                this._onDrawPointers(angle, element.value);
            if (element.type == "needle") {
                this._drawNeedlePointer(this.region, this.style, element);
                this._setPointerCap(element);
            }
            else {
                if (element.markerType == "roundedrectangle")
                    this.style.radius = 5;
                this._drawMarkerType(element.markerType, this.region, this.style);
            }
            if (element.pointerValueText.showValue) {
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._labelRadius + element.pointerValueText.distance, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, this._labelRadius + element.pointerValueText.distance, angle);
                this.region = { "startX": startX, "startY": startY };
                this.style = { "angle": element.pointerValueText.autoAngle ? element.pointerValueText.angle + angle : element.pointerValueText.angle, "textValue": (locale && this.model.enableGroupSeparator) ? element.value.toLocaleString(locale) : element.value, "fillStyle": (element.pointerValueText.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.pointerValueText.color), "font": this._getFontString(element, element.pointerValueText.font), "opacity": element.pointerValueText.opacity };
                this._drawPointerValueText(this.region, this.style);
            }
            if (this.contextEl.getImageData)
                this.pointerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            if (this._notifyArrayChange)
                this._notifyArrayChange("scales[" + this.scaleIndex + "]pointers[" + index + "]value", element.value);
            this.value(element.value);
        },
        _drawPointerValueText: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.textAlign = "center";
            this.contextEl.textBaseline = "middle";
            this.contextEl.font = style.font;
            this.contextEl.translate(location.startX , location.startY);
            this.contextEl.lineTo(0, 0);
            this.scaleEl[this.scaleIndex].pointers[this.pointerIndex].pointerValueText.autoAngle ? this.contextEl.rotate((Math.PI * ((style.angle - 270) / 180))) : this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.fillText(style.textValue, 0, 0);
            this._contextClosePath(style, this);
        },
        
        _drawMarkerType: function (type, location, style) {
            switch (type) {
                case "rectangle":
                    this._drawRectangle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "triangle":
                    this._drawTriangle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "ellipse":
                    this._drawEllipse(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "diamond":
                    this._drawDiamond(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "pentagon":
                    this._drawPentagon(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "circle":
                    this._drawCircle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "slider":
                    this._drawSlider(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "pointer":
                    this._drawPointer(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "wedge":
                    this._drawWedge(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "trapezoid":
                    this._drawTrapezoid(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawRoundedRectangle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "image":
                    this._drawImagePointer(location, style, this);
                    break;
            }
        },

	 
        _drawNeedlePointer: function (location, style, element) {
            if (this.pointerEl[this.pointerIndex].needleType == "image") {
                this._drawImagePointer(location, style, element);
            }
            else{
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * ((style.angle) / 180));
            this.contextEl.lineTo(0, -style.width / 2);
            if (this.pointerEl[this.pointerIndex].needleType == "triangle") {
                this.contextEl.lineTo(style.height, 0);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "rectangle") {
                this.contextEl.lineTo(style.height, -style.width / 2);
                this.contextEl.lineTo(style.height, style.width / 2);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "trapezoid") {
                this.contextEl.lineTo(style.height, -style.width / 4);
                this.contextEl.lineTo(style.height, style.width / 4);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "arrow") {
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 6);
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 2);
                this.contextEl.lineTo(style.height, 0);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 2);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 6);
            }
            this.contextEl.lineTo(0, style.width / 2);
            if (style.showBackNeedle) {
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCap.radius / 2), style.width / 2);
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCap.radius / 2), -style.width / 2);
            }
            this._contextClosePath(style, this);
            }
            this.canvasEl.attr("aria-label", this.model.scales[this.scaleIndex].pointers[this.pointerIndex].value);
        },
        
        _drawImagePointer: function (location, style, element) {
            var self = this;
            var image = new Image();
            var angle = style.angle;
            var xPos = location.startX;
            var yPos = location.startY;
            var elemWidth = style.width;
            var elemHeight = style.height;
            var type = style.type;
            image.onload = function () {
                self.contextEl.save();
                self.contextEl.translate(xPos, yPos);
                self.contextEl.rotate(Math.PI * (angle / 180));
                self.contextEl.drawImage(this, 0, -elemHeight / 2, elemWidth, elemHeight);
                self.contextEl.restore();
            };
            image.src = style.imageUrl;
        },
	 
        _setPointerCap: function (element) {
            var grd;
            //var gradients = [{ "ColorStop": 0.19, "Color": "#F2DFDF" }, { "ColorStop": 0.71, "Color": "#FFFFFF" }, { "ColorStop": 0.77, "Color": "#836B33" }];
            grd = this.contextEl.createRadialGradient(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, 0, this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, this.scaleEl[this.scaleIndex].pointerCap.radius);
            if (!ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].pointerCap.interiorGradient))
                this._setGradientColor(this, grd, this.scaleEl[this.scaleIndex].pointerCap.interiorGradient.colorInfo);
            else
                grd = (this.scaleEl[this.scaleIndex].pointerCap.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.backgroundColor)); // this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.backgroundColor));
            this.region = { "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = { "isStroke": true, "isFill": true, "strokeStyle": (this.scaleEl[this.scaleIndex].pointerCap.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.borderColor)), "radius": this.scaleEl[this.scaleIndex].pointerCap.radius, "lineWidth": this.scaleEl[this.scaleIndex].pointerCap.borderWidth, "fillStyle": grd };
            if (this.model.drawPointerCap)
                this._onDrawPointerCap();
            this._drawCircleFrame(this.region, this.style);
        },

        
        isAndroid: function () {
            return (/android/i.test(navigator.userAgent.toLowerCase()));
        },
        
        redraw: function (value) {
            switch (value) {
                case "scale":
                    this._reDrawScale(); break;
                case "pointer":
                    this._reDrawPointer(); break;
                case "range":
                    this._reDrawRange(); break;
                case "label":
                    this._reDrawLabel(); break;
                case "tickMark":
                    this._reDrawTickMark();
                    break;
                case "subGauges":
                    this._reDrawSubGauge();
                    break;
                case "CustomLabel":
                    this._reDrawCustomLabel();
                    break;
                default:
                    this._init();
            }
        },
       
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                return this._getPointerImage();
        },
      
        _getPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
      
        _getSubGaugeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
      
        _getCustomLabelImage: function () {
            if (this.subGaugeImage)
                return this.subGaugeImage;
            else
                return this._getSubGaugeImage();
        },
       
        _getRangeImage: function () {
            if (this.model.rangeZOrder  == "rear") {
                if (this.scaleImage)
                    return this.scaleImage;
                else
                    return this.outerImage;
            }
            else {
                if (this.tickImage)
                    return this.tickImage;
                else
                    return this._getTickImage();
            }
        },
       
        _getLabelImage: function () {
            if (this.model.rangeZOrder  == "rear") {
                if (this.tickImage)
                    return this.tickImage;
                else
                    return this._getTickImage();
            }
            else {
                if (this.tickImage)
                    return this.rangeImage;
                else
                    return this._getRangeImage();
            }
        },
      
        _getTickImage: function () {
            if (this.model.rangeZOrder  == "rear") {
                if (this.rangeImage)
                    return this.rangeImage;
                else
                    return this._getRangeImage();
            }
            else {
                if (this.scaleImage)
                    return this.scaleImage;
                else
                    return this.outerImage;
            }
        },
     
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].value;
            else
                return null;
        },
       
        setMarkerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        
        getMarkerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale;
            else
                return null;
        },
        
        setPointerLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].length = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
       
        getPointerLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].length;
            else
                return null;
        },
        
        setPointerWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
       
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].width;
            else
                return null;
        },

        
        setBackNeedleLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        
        getBackNeedleLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength;
            else
                return null;
        },
        
        setNeedleStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].needleType = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        
        getNeedleStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].needleType;
            else
                return null;
        },
       
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
		 
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].placement;
            else
                return null;
        },
      
        setPointerNeedleType: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else this._initialize();
            }
        },
          
        getPointerNeedleType: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].type;
            else
                return null;
        },
          
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                if (value == "roundedrectangle" || !ej.isNullOrUndefined(ej.datavisualization.CircularGauge.MarkerType[value.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); })])) {
                    this.scaleEl[scaleIndex].pointers[pointerIndex].markerType = value;
                    if (this.contextEl.putImageData && !this.isAndroid())
                        this._reDrawCustomLabel();
                    else this._initialize();
                }
            }
        },
          
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].markerType;
            else
                return null;
        },
          
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
          
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].value;
            else
                return null;
        },

          
        setSubGaugeLocation: function (scaleIndex, gaugeIndex, value) {
            if (scaleIndex < this.model.scales.length && gaugeIndex < this.model.scales[scaleIndex].subGauges.length && value != null) {
                this.scaleEl[scaleIndex].subGauges[gaugeIndex].position.x = value.x;
                this.scaleEl[scaleIndex].subGauges[gaugeIndex].position.y = value.y;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawSubGauge();
                else
                    this._initialize();
            }
        },
          
        getSubGaugeLocation: function (scaleIndex, gaugeIndex) {
            if (scaleIndex < this.model.scales.length && gaugeIndex < this.model.scales[scaleIndex].subGauges.length)
                return this.scaleEl[scaleIndex].subGauges[gaugeIndex].position;
            else
                return null;
        },
          
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
          
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle;
            else
                return null;
        },
          
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
            else
                return null;
        },
          
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
            else
                return null;
        },
          
        setRangeSize: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },

        setRangeVisible: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex]._visible = value;
                if (this.contextEl.putImageData && !this.isAndroid()) {
                    this._initialize();
                    this._wireEvents();
                }
                else
                    this._initialize();
            }
        },
          
        getRangeSize: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].size;
            else
                return null;
        },
          
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
            else
                return null;
        },
          
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].placement;
            else
                return null;
        },
          
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].border.width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].border.width;
            else
                return null;
        },
          
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
            if(value >= this.scaleEl[scaleIndex].minimum && value <= this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].pointers[pointerIndex].value = value;
			    if (!this.updatePointerOnAnimation && this.model.enableAnimation)
                    this.pointerValue[pointerIndex] = value;
                if (this.contextEl.putImageData && !this._androidAnimation)
                   {
                    this._reDrawPointer();
                    this.updatePointerOnAnimation = false;
                   }  
                else
                    this._initialize();
            }
        },
          
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
            else
                return null;
        },
          
        setLabelDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale;
            else
                return null;
        },
          
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].type;
            else
                return null;
        },
          
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].placement;
            else
                return null;
        },
          
        setTickAngle: function (scaleIndex, tickIndex, angle) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else
                    this._initialize();
            }
        },
          
        getTickAngle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].angle;
            else
                return null;
        },
          
        setTickStyle: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickStyle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].type;
            else
                return null;
        },
          
        setTickPlacement: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickPlacement: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].placement;
            else
                return null;
        },
          
        setTickWidth: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickWidth: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].width;
            else
                return null;
        },
          
        setTickHeight: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].height = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickHeight: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].height;
            else
                return null;
        },
          
        setTickDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale;
            else
                return null;
        },
          
        setStartAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].startAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getStartAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].startAngle;
            else
                return null;
        },
          
        setSweepAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].sweepAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getSweepAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].sweepAngle;
            else
                return null;
        },
          
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                if(value < this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].minimum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minimum;
            else
                return null;
        },
          
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                if(value > this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].maximum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].maximum;
            else
                return null;
        },
          
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].size;
            else
                return null;
        },
          
        setScaleRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].radius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].radius;
            else
                return null;
        },
          
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
            else
                return null;
        },
          
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
            else
                return null;
        },
          
        setPointerCapRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].pointerCap.radius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getPointerCapRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].pointerCap.radius;
            else
                return null;
        },
          
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].border.width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].border.width;
            else
                return null;
        },
          
        setPointerCapBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].pointerCap.borderWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getPointerCapBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].pointerCap.borderWidth;
            else
                return null;
        },
          
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].direction = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].direction;
            else
                return null;
        },
          
        includeFirstValue: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].includeFirstValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else this.initialize();
            }
        },
          
        _reDrawScale: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this.outerImage, 0, 0);
                this._drawScales();

            }
        },
          
        _reDrawPointer: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getPointerImage(), 0, 0);
                this._setPointers();
                this._setIndicators();
            }
        },
          
        _reDrawCustomLabel: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawRange: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                if (this.model.rangeZOrder  == "rear") {
                    this._setRanges();
                    this._setTicks();
                }
                else {
                    this._setRanges();
                }
                this._setLabels();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawLabel: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawTickMark: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                if (this.model.rangeZOrder  == "rear")
                    this._setTicks();
                else {
                    this._setTicks();
                    this._setRanges();
                }
                this._setLabels();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawSubGauge: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getSubGaugeImage(), 0, 0);
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        refreshSubGauge: function () {
            if (this.contextEl.putImageData)
                this._reDrawSubGauge();
            else
                this._initialize();
        },
		  
        refresh: function () {
            this._scaleRedrawn = true;
            this._initialize();
            this._wireEvents();
        },
          
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
                return false;
            }
            else {
                var image = this.canvasEl[0].toDataURL();
                image = image.replace(/^data:[a-z]*;,/, '');
                var image1 = image.split(',');
                var byteString = atob(image1[1]);
                var buffer = new ArrayBuffer(byteString.length);
                var intArray = new Uint8Array(buffer);
                for (var i = 0; i < byteString.length; i++) {
                    intArray[i] = byteString.charCodeAt(i);
                }
                var blob = new Blob([buffer], { type: "image/png" });
                if (ej.browserInfo().name === "msie")
                    window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
                else {
                    var pom = document.createElement('a');
                    var url = URL.createObjectURL(blob);
                    pom.href = url;
                    pom.setAttribute('download', fileName + '.' + fileType);
                    if (document.createEvent) {
                        var e = document.createEvent("MouseEvents");
                        e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                        pom.dispatchEvent(e);
                    }
                    else if (pom.fireEvent) {
                        pom.fireEvent("onclick");
                    }
                }
                return true;
            }
        },
		  
        resizeCanvas: function () {
            if (_gaugeCount!=0)
                _gaugeCount = _gaugeCount - 1;
            else
                _gaugeCount = $(".e-circulargauge").length - 1;
            var state, chk=true;
            if (!ej.isNullOrUndefined(this.GaugeEl.parent().attr("style")))
                state = this.GaugeEl.parent().attr("style").split(";");
            if (!ej.isNullOrUndefined(state)){
            $.each(state, function (key, val) {
                while (val.indexOf("width")!=-1){
                    chk = val.indexOf("px") == -1 ? true : false;
                    break;
                }
            });
            }
            if (chk) {
            var ratio = window.innerWidth / initialDivWidth;
			var oldWidth = this.model.width;
			var oldHeight = this.model.height;
				if ((oldWidth *ratio > window.innerWidth) || (oldHeight *ratio > window.innerHeight))
					ratio = 1;
                this.model.width *= ratio;
                this.model.height *= ratio;
                this.model.radius *= ratio;
				this._gaugeResizeState = true;
                for (var i = 0; this.model.scales[i] != null; i++) {
                    this.model.scales[i].radius *= ratio;
                    this.model.scales[i].pointerCap.radius *= ratio;
					 for (var k = 0 ; k < this.model.scales[i].customLabels.length; k++)
                    {
                            if (!ej.isNullOrUndefined(this.model.scales[i].customLabels[k])) {
                            if (this.model.scales[i].customLabels[k].positionType != "outer")
                                this.model.scales[i].customLabels[k].position.x *= ratio;
                                this.model.scales[i].customLabels[k].position.y *= ratio;
                            this.model.scales[i].customLabels[k].font.size = (parseFloat(this.model.scales[i].customLabels[k].font.size.match(/\d+/)[0]) * ratio < 10) ? "10px" : ((parseFloat(this.model.scales[i].customLabels[k].font.size.match(/\d+/)[0]) * ratio) > this._customLblMaxSize) ? this._customLblMaxSize.toString() + 'px' : ((parseFloat(this.model.scales[i].customLabels[k].font.size.match(/\d+/)[0])) * ratio).toString() + 'px';
                        }
                    }
                    for (var j = 0; this.model.scales[i].labels[j] != null || this.model.scales[i].pointers[j] != null || this.model.scales[i].ranges[j] != null || this.model.scales[i].indicators[j] != null || this.model.scales[i].subGauges[j] != null || this.model.scales[i].ticks[j] != null; j++) {
                        if (!ej.isNullOrUndefined(this.model.scales[i].ticks[j])) this.model.scales[i].ticks[j].height *= ratio;
                        if (!ej.isNullOrUndefined(this.model.scales[i].pointers[j])) {
                            this.model.scales[i].pointers[j].length *= ratio;
                            this.model.scales[i].pointers[j].width *= ratio;
                            this.model.scales[i].pointers[j].backNeedleLength *= ratio;
                        }
                        if (!ej.isNullOrUndefined(this.model.scales[i].ranges[j])) {
                            this.model.scales[i].ranges[j].distanceFromScale *= ratio;
                            this.model.scales[i].ranges[j].size *= ratio;
                        }
                        if (!ej.isNullOrUndefined(this.model.scales[i].indicators[j]) && this.model.scales[i].showIndicators) {
                            this.model.scales[i].indicators[j].height *= ratio;
                            this.model.scales[i].indicators[j].width *= ratio;
                            this.model.scales[i].indicators[j].position.x *= ratio;
							this.model.scales[i].indicators[j].position.y *= ratio;
                        }
                        if (!(ej.isNullOrUndefined(this.model.scales[i].subGauges[j])) && (ej.isNullOrUndefined(this.model.scales[i].subGauges[j].controlID))) {
                            this.model.scales[i].subGauges[j].height *= ratio;
                            this.model.scales[i].subGauges[j].width *= ratio;
                            this.model.scales[i].subGauges[j].position.x *= ratio;
                        }
                    }
                }
                this.refresh();
                if (_gaugeCount == 0) {
                    initialDivWidth = window.innerWidth;
                }
            }
        },

          
		  
        _onDrawTicks: function (tickAngle, pointerValue) {
            var tick = { index: this.tickIndex, element: this.tickEl[this.tickIndex], angle: parseInt(tickAngle) };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, pointerValue: pointerValue, style: this.style, position: this.region,tick:tick };
            this._trigger("drawTicks", data);
        },
		  
        _onDrawLabels: function (labelAngle, pointerValue) {
            var label = { index: this.labelIndex, element: this.labelEl[this.labelIndex], angle: parseInt(labelAngle) };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, pointerValue: pointerValue, style: this.style, position: this.region,label:label };
            this._trigger("drawLabels", data);
        },
		  
        _onDrawPointers: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], angle: parseInt(pointerAngle), pointerValue: pointerValue};
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, pointer: pointer };
            this._trigger("drawPointers", data);
        },
		  
        _onDrawRange: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeEl], context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawRange", data);
        },
		  
        _onDrawCustomLabel: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawCustomLabel", data);
        },
		  
        _onDrawIndicators: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawIndicators", data);
        },
		  
        _onDrawPointerCap: function () {
            var data = { object: this, scaleElement: this.model.scales, position: this.region, style: this.style, context: this.contextEl };
            this._trigger("drawPointerCap", data);
        },
		  
        _onRenderComplete: function () {
            var data = { object: this, scaleElement: this.model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
		  
        _onMouseClick: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.position, pointers: pointer };
            this._trigger("mouseClick", data);
        },
		  
        _onMouseClickMove: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, pointers: pointer };
            this._trigger("mouseClickMove", data);
        },
		  
        _onMouseClickUp: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, pointers: pointer };
            this._trigger("mouseClickUp", data);
        },

          
		  
        _setTheme: function () {
            var selectedTheme =this.model.themeProperties[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
		  
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = this.model.themeProperties;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].backgroundColor) ? selectedTheme.backgroundColor : this.model.backgroundColor);
                for (var i = 0; i < this.model.scales.length; i++) {
                    this.model.scales[i].backgroundColor = (!this.model.scales[i].backgroundColor || this.model.scales[i].backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.scales[i].backgroundColor;
                    this.model.scales[i].border.color = (!this.model.scales[i].border.color || this.model.scales[i].border.color == jsonObj[result[th]].scales.border.color) ? selectedTheme.scales.border.color : this.model.scales[i].border.color;
                    for (var j = 0; j < this.model.scales[i].pointers.length; j++) {
                        this.model.scales[i].pointers[j].backgroundColor = (!this.model.scales[i].pointers[j].backgroundColor || this.model.scales[i].pointers[j].backgroundColor == jsonObj[result[th]].scales.pointers.backgroundColor) ? selectedTheme.scales.pointers.backgroundColor : this.model.scales[i].pointers[j].backgroundColor;
                        this.model.scales[i].pointers[j].border.color = (!this.model.scales[i].pointers[j].border.color || this.model.scales[i].pointers[j].border.color == jsonObj[result[th]].scales.pointers.border.color) ? selectedTheme.scales.pointers.border.color : this.model.scales[i].pointers[j].border.color;
                        this.model.scales[i].pointerCap.backgroundColor = (!this.model.scales[i].pointerCap.backgroundColor || this.model.scales[i].pointerCap.backgroundColor == jsonObj[result[th]].scales.pointerCap.backgroundColor) ? selectedTheme.scales.pointerCap.backgroundColor : this.model.scales[i].pointerCap.backgroundColor;
                        this.model.scales[i].pointerCap.borderColor = (!this.model.scales[i].pointerCap.borderColor || this.model.scales[i].pointerCap.borderColor == jsonObj[result[th]].scales.pointerCap.borderColor) ? selectedTheme.scales.pointerCap.borderColor : this.model.scales[i].pointerCap.borderColor;
                    }
                    for (var k = 0; k < this.model.scales[i].ticks.length; k++) {
                        this.model.scales[i].ticks[k].color = (!this.model.scales[i].ticks[k].color || this.model.scales[i].ticks[k].color == jsonObj[result[th]].scales.ticks.color) ? selectedTheme.scales.ticks.color : this.model.scales[i].ticks[k].color;
                    }
                    for (var l = 0; l < this.model.scales[i].labels.length; l++) {
                        this.model.scales[i].labels[l].color = (!this.model.scales[i].labels[l].color || this.model.scales[i].labels[l].color == jsonObj[result[th]].scales.labels.color) ? selectedTheme.scales.labels.color : this.model.scales[i].labels[l].color;
                    }
                }
            }
        },
		  
        _getFontString: function (element, font) {
            return (font ? (font.fontStyle ? font.fontStyle : "") + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily : "");
        },
		  
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.model.Orientation == "Horizontal" && element.markerPlacement == "near") {
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY);   
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.lineTo(location.startX + (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY);   
            }
            this._contextClosePath(style, element);
        },
		  
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
		  
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },

        _drawLine: function (location, style, element) {
            element.contextEl.beginPath();
            element.contextEl.strokeStyle = style.fillStyle;
            element.contextEl.globalAlpha = style.opacity;
            element.contextEl.lineWidth = style.lineWidth;
            element.contextEl.moveTo(location.startX, location.startY);
            element.contextEl.lineTo((location.startX + style.width), location.startY);
            element.contextEl.closePath();
            element.contextEl.stroke();
        },
		  
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(style.radius, -style.height / 2);
            element.contextEl.lineTo(style.width - style.radius, -style.height / 2);
            element.contextEl.quadraticCurveTo(style.width, -style.height / 2, style.width, (-style.height / 2) + style.radius);
            element.contextEl.lineTo(style.width, (style.height / 2) - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height / 2, style.width - style.radius, style.height / 2);
            element.contextEl.lineTo(style.radius, style.height / 2);
            element.contextEl.quadraticCurveTo(0, style.height / 2, 0, style.height / 2 - style.radius);
            element.contextEl.lineTo(0, -style.height / 2 + style.radius);
            element.contextEl.quadraticCurveTo(0, -style.height / 2, style.radius, -style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            $(image).on('load', function () {
                element.contextEl.drawImage(this, 0, 0, element.model.Width, element.model.Height);
                if (element.model.scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
		  
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		  
        _setPointerDimension: function (style, element) {
            if (!ej.isNullOrUndefined(element.model.Orientation)) {
                if (element.model.Orientation == "Horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
		  
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
		  
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
		  
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
		  
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
		  
        _isBetween: function (first, last, number, padding) {
            return (first < last ? number >= first - padding && number <= last + padding : number >= last - padding && number <= first + padding);
        },
		  
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
		  
        _setGradientColor: function (element, gradients, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradients.addColorStop(0, this._getColor(element, options));
                gradients.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradients.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : self._getColor(element, colorElement.color));
                });
        }
    });
      
  (function() {      // for requestAnimationFrame
    var now = 0, items = ['ms', 'moz', 'webkit', 'o'], i, prevTime, element, callNow;
    for (i = 0; i < items.length && !window.requestAnimationFrame; ++i) 
        window.requestAnimationFrame = window[items[i]+'RequestAnimationFrame'];
    if (!window.requestAnimationFrame) {
		ej.datavisualization.CircularGauge.animationPolyfill = true;
        window.requestAnimationFrame = function(callback, element) {						
            prevTime = new Date().getTime();
            callNow = Math.max(0, 16 - (prevTime - now));
            element = window.setTimeout(function() { callback(prevTime + callNow); }, callNow);
            now = prevTime + callNow;
            return element;
        };
	}
  })();

    ej.datavisualization.CircularGauge.Frame = {
          
        FullCircle: "fullcircle",
          
        HalfCircle: "halfcircle"
    };
      
    ej.datavisualization.CircularGauge.Directions = {
          
        Clockwise: "clockwise",
          
        CounterClockwise: "counterclockwise"
    };
      
    ej.datavisualization.CircularGauge.PointerPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.PointerType = {
          
        Needle: "needle",
          
        Marker: "marker"
    };
      
    ej.datavisualization.CircularGauge.NeedleType = {
          
        Triangle: "triangle",
          
        Rectangle: "rectangle",
          
        Trapezoid: "trapezoid",
          
        Arrow: "arrow",
          
        Image: "image",
    };
      
    ej.datavisualization.CircularGauge.MarkerType = {
          
        Rectangle: "rectangle",
          
        Triangle: "triangle",
          
        Ellipse: "ellipse",
          
        Diamond: "diamond",
          
        Pentagon: "pentagon",
          
        Circle: "circle",
          
        Slider: "slider",
          
        Pointer: "pointer",
          
        Wedge: "wedge",
          
        Trapezoid: "trapezoid",
          
        RoundedRectangle: "roundedrectangle",
          
        Image: "image"
    };
      
    ej.datavisualization.CircularGauge.RangePlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.TickType = {
          
        Major: "major",
          
        Minor: "minor"
    };
      
    ej.datavisualization.CircularGauge.gaugePosition = {
          
        TopLeft: "topleft",
          
        TopRight: "topright",
          
        TopCenter: "topcenter",
          
        MiddleLeft: "middleleft",
          
        MiddleRight: "middleright",
          
        Center: "center",
          
        BottomLeft: "bottomleft",
          
        BottomRight: "bottomright",
          
        BottomCenter: "bottomcenter",
    };
      
    ej.datavisualization.CircularGauge.TickPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.CustomLabelPositionType = {
          
        Inner: "inner",
          
        Outer: "outer"
    };
      
    ej.datavisualization.CircularGauge.OuterCustomLabelPosition = {
          
        Left: "left",
          
        Right: "right",
          
        Top: "top",
          
        Bottom: "bottom",
    };
      
    ej.datavisualization.CircularGauge.LabelPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.LabelType = {
          
        Major: "major",
          
        Minor: "minor"
    };
      
    ej.datavisualization.CircularGauge.UnitTextPlacement = {
          
        Back: "back",
          
        Front: "front"
    };
      
    ej.datavisualization.CircularGauge.RangeZOrderPosition = {
          
        Rear: "rear",
          
        Front: "front"
    };
      
    ej.datavisualization.CircularGauge.IndicatorType = {
          
        Rectangle: "rectangle",
          
        Circle: "circle",
          
        RoundedRectangle: "roundedrectangle",
          
        Text: "text",
          
        Image:"image"
    };
	  
 	ej.datavisualization.CircularGauge.Themes= {
		
		FlatLight: 'flatlight',

		FlatDark: 'flatdark'
 	};

 	ej.datavisualization.CircularGauge.LegendPosition = {

 	    Top: "top",

 	    Bottom: "bottom",

 	    Left: "left",

        Right: "right"
 	};

 	ej.datavisualization.CircularGauge.LegendAlignment = {

 	    Near: "near",

 	    Center: "center",

        Far: "far"
 	};

 	ej.datavisualization.CircularGauge.LegendShape = {

 	    Rectangle: "rectangle",

 	    Triangle: "triangle",

 	    Diamond: "diamond",

 	    Pentagon: "pentagon",

 	    Circle: "circle",

 	    Slider: "slider",

 	    Wedge: "wedge",

 	    Trapezoid: "trapezoid",

        Line: "line"

 	};
	
      

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
	 

    ej.widget({ "ejLinearGauge": "ej.datavisualization.LinearGauge"}, {
       
        element: null,
        _rootCSS: "e-lineargauge",
        _animationFlag: true,
        
        model: null,
        _customLblMaxSize: 0,
        _savedPoints: [],
        validTags: ["div", "span"],

       
        defaults:  {
		
			locale: null,
			
			enableGroupSeparator: false,
			  	
            value: 0,
			  
            minimum: 0,
			  
            maximum: 100,
			  
            width: 150,
			  
            height: 400,
			  
            theme: "flatlight",
			  
            orientation: "Vertical",
			  
            pointerGradient1: null,
			  
            pointerGradient2: null,
			  
            backgroundColor: null,
			  
            borderColor: null,
			  
            labelColor: null,
			  
            tickColor: null,
			  
            readOnly: true,
			  
            enableResize: false,
              
            isResponsive: false,
              
            tooltip: {
                  
                showLabelTooltip: false,
                  
                showCustomLabelTooltip: false,
                  
                    templateID: null,
            },
              
            outerCustomLabelPosition: "bottom",
              
			frame: {
			      
			    backgroundImageUrl: null,
			      
			    outerWidth: 12,
			      
                innerWidth:8
			},
			  
            scales: null,
			  
            enableAnimation: true,
              
            enableMarkerPointerAnimation: true,
			  
            animationSpeed: 500,
			  
            drawTicks: null,
			  
            drawLabels: null,
			  
            drawBarPointers: null,
			  
            drawMarkerPointers: null,
			  
            drawRange: null,
			  
            drawCustomLabel: null,
			  
            drawIndicators: null,
			  
            load: null,
			  
            init: null,
			  
            renderComplete: null,
              
            mouseClick: null,
              
            mouseClickMove: null,
              
            mouseClickUp: null,
			themeProperties:{
			flatlight: {
            scales: {
                backgroundColor: "#FFFFFF",
                border: { color: "#1d1e1e" },
                barPointers: {
                    backgroundColor: "#8abc3b",
                    border: { color: "#8abc3b" }
                },
                markerPointers: {
                    backgroundColor: "#212121",
                    border: { color: "#212121" }
                },
                ticks: {
                    color: "#1d1e1e"
                },
                labels: {
                    labelColor: "#222222"
                }
            }
        },
		  
        flatdark: {
            scales: {
                backgroundColor: "#808080",
                border: { color: "#808080" },
                barPointers: {
                    backgroundColor: "#8abc3b",
                    border: { color: "#8abc3b" }
                },
                markerPointers: {
                    backgroundColor: "#CCCCCC",
                    border: { color: "#CCCCCC" }
                },
                ticks: {
                    color: "#808080"
                },
                labels: {
                    labelColor: "#CCCCCC"
                }
            }
        }
			
			}
        },
				  	
        _defaultScaleValues: function () {
            return {
				  	
                minimum: null,
				  	
                maximum: null,
				  	
                majorIntervalValue: 10,
				  	
                minorIntervalValue: 2,
				  	
                direction: "counterclockwise",
				  	
                backgroundColor: null,
				  	
                border: {
                      
                    color: null,
                      
                    width:1.5
                },
				  	
                opacity: NaN,
				  	
                width: 30,
				  	
                shadowOffset: 0,
				  	
                length: 290,
				  	
                type: "rectangle",
				  	
                position: { 
				  	
				x: 50, 
				  	
				y: 50 },
				  	
                showRanges: false,
				  	
                showIndicators: false,
				  	
                showCustomLabels: false,
				  	
                showLabels: true,
				  	
                showTicks: true,
				  	
                showBarPointers: true,
				  	
                showMarkerPointers: true,
				  	
                ticks: [{
                                          
                    distanceFromScale: {
                          
                        x: 0,
                          
                        y:0
                    },
					  	
                    angle: 0,
					  	
                    color: null,
					  	
                    type: "majorinterval",
					  	
                    placement: "near",
					  	
                    opacity: 0,
					  	
                    height: 10,
					  	
                    width: 3
                },
                {              
                    distanceFromScale: {
                        x: 0,
                        y:0
                    },
                    angle: 0,
                    color: null,
                    type: "minorinterval",
                    placement: "near",
                    opacity: 0,
                    height: 5,
                    width: 2
                }],
                  
                ranges: [{
                      
                    endWidth: 10,
                      
                    placement: "center",
					  
                    startWidth: 10,
					  
                    distanceFromScale: 0,
					  
                    endValue: 60,
					  
                    startValue: 20,
					  
                    gradients: null,
					  
                    backgroundColor: null,
					  
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: null
                }],
					  
                labels: [{
                      
                    distanceFromScale: {
                            
                        x: -10,
                            
                        y:0
                    },
                      
                    angle: 0,
					  
                    font: { 
					  
					size: "11px", 
					  
					fontFamily: "Arial", 
					  
					fontStyle: "bold" },
					  
                    textColor: null,
					  
                    opacity: 0,
					  
                    type: "major",
					  
                    placement: "near",
					  
                    includeFirstValue: true,
					  
					unitText: "",
					  
					unitTextPlacement: "back"  
                }],
				      
                markerPointers: [{
					  
                    type: "triangle",
					  
                    length: 30,
					  
                    placement: "far",
					  
                    gradients: null,
					  
                    distanceFromScale: 0,
					  
                    width: 30,
					  
                    value: null,
					  
                    backgroundColor: null,
					  
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: 1
                }],
					  
                barPointers: [{
				      
                    gradients: null,
					  
                    distanceFromScale: 0,
					  
                    width: 30,
					  
                    value: null,
					  
                    backgroundColor: null,
                      
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: 1
                }],
					  	
                indicators: [{
					  
                    font: { 
					  
					size: "11px", 
					  
					fontFamily: "Arial",
					  					
					fontStyle: "bold" },
					  					
                    height: 30,
					  	
                    type: "rectangle",
					  	
                    width: 30,
						  	
                    position: { 
					  	
					x: 0, 
					  	
					y: 0 },
					  	
                    textLocation: { 
					  	
					x: 0, 
					  	
					y: 0 },
					  
                    stateRanges: [{
					  
                    endValue: 60,
					  
                    startValue: 50,
					  
                    backgroundColor: null,
					  
                    borderColor: null,
					  
                    text: "",
					  
                        textColor: null
                    }],
					  
                    backgroundColor: null,
                                          
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: NaN
                }],
					  
                customLabels: [{
					  
                    font: { 
					  
					size: "11px", 
					  
					fontFamily: "Arial", 
					  
					fontStyle: "bold" },
					  
                    color: null,
					  
                    opacity: 0,
					  
                    value: "",
					  
                    textAngle: 0,
					  
                    position: { 
					  
					x: 0, 
					  
					y: 0 },
                      
                    positionType:"inner"
                }]
            };
        },

        dataTypes: {
            scales: "data",
            isResponsive: "boolean",
        },
        observables: ["value", "minimum", "maximum"],
        _tags: [{
            tag: "scales",
            attr: ["majorIntervalValue", "minorIntervalValue", "backgroundColor", "shadowOffset", "showRanges", "showIndicators", "showCustomLabels", "showLabels", "showTicks", "showBarPointers", "showMarkerPointers", "border.color", "border.width", "position.x", "position.y",
                [{
                    tag: "markerPointers", attr: ["distanceFromScale", "backgroundColor", "border.width", "border.color"]
                }, {
                    tag: "barPointers", attr: ["distanceFromScale", "backgroundColor", "border.width", "border.color"]
                }, {
                    tag: "ranges", attr: ["distanceFromScale", "startValue", "endValue", "startWidth", "endWidth", "backgroundColor", "border.color", "border.width"]
                }, {
                    tag: "ticks", attr: ["distanceFromScale.x", "distanceFromScale.y"]
                }, {
                    tag: "indicators", attr: ["backgroundColor", "textLocation", "font.size", "font.fontFamily", "font.fontStyle", "position.x", "position.y", "textLocation.x", "textLocation.y", "borderColor", "textColor",
                        [{
                            tag: "stateRanges", attr: ["endValue", "startValue", "backgroundColor", "borderColor", "textColor"]
                        }]
                    ]
                }, {
                    tag: "labels", attr: ["distanceFromScale.x", "distanceFromScale.y", "textColor", "includeFirstValue", "unitText", "unitTextPlacement", "font.size", "font.fontFamily", "font.fontStyle"]
                }, {
                    tag: "customLabels", attr: ["textAngle", "font.size", "font.fontFamily", "font.fontStyle", "position.x", "position.y"]
                }]
            ]
        }],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),

          
		  	 
        _init: function () {
            _linearGaugeCount = $(".e-lineargauge").length;
            initialGaugeCount = _linearGaugeCount;
            this._initialize();
            this._trigger("load");
            this._setTheme();
            this._render();
            this.wireEvents();
			this._onWindowResize();
        },
		  	
		_onWindowResize:function()
        {
            if (this.model.enableResize || this.model.isResponsive) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this.resizeCanvas);
                else
                    this._on($(window), "orientationchange", this.resizeCanvas);
            }
        },
		  	 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "theme": this.model.theme = options[option]; this._init(); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "orientation": this.model.orientation = options[option]; break;
                    case "pointerGradient1": this.model.pointerGradient1 = options[option]; break;
                    case "pointerGradient2": this.model.pointerGradient2 = options[option]; break;
                    case "labelColor": this.model.labelColor = options[option]; break;
                    case "tick": $.extend(this.model.tick,options[option]); break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "borderColor": this.model.borderColor = options[option]; break;
                    case "frame": $.extend(this.model.frame, options[option]);
                    case "outerCustomLabelPosition": this.model.outerCustomLabelPosition = options[option]; break;
                    case "tooltip": $.extend(this.model.tooltip, options[option]); break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "value":
                        (this.value() == "") && this.value(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (var j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                                this.model.scales[i].markerPointers[j].value = parseFloat(this.value());
                            }
                        }
                        for (var k = 0; this.model.scales[k] != null; k++) {
                            for (var l = 0; this.model.scales[k].barPointers[l] != null; l++) {
                                this.model.scales[k].barPointers[l].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        (this.minimum() == "") && this.minimum(0);
                        for (var m = 0; this.model.scales[m] != null; m++) {
                            this.model.scales[m].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        (this.maximum() == "") && this.maximum(0);
                        for (var n = 0; this.model.scales[n] != null; n++) {
                            this.model.scales[n].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        this.model.scales = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._render();
            this.wireEvents();
        },
          	       	 
        _destroy: function () {
            this.activeElement = null;
            this.canvasEl = null;
            this.contextEl = null;
            this.unWireEvents();
            this.element.empty().removeClass("e-widget");
        },
		  	
        _initialize: function () {
            this.GaugeEl = this.element;
            this.scaleStartX = [];
            this.scaleStartY = [];
            this.isScaleModified = false;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },
		  
        _render: function () {
            this.initialize();
            this.wireEvents();
        },
		  
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
		  	
         _checkArrayObject: function (element, initialName) {
            var proxy = this;
			var type;
            $.each(element, function (name, innerElement) {
			 type = typeof name;	 
			if((type!="string" ||( type=="string" && name.indexOf('_') == -1 && name.indexOf('__') == -1)) && typeof innerElement !="function"){
                if (innerElement instanceof Array  ) {
                    proxy._checkArrayObject(innerElement, name);
                }			 
                else if (innerElement != null && typeof innerElement == "object" && !innerElement.setter && !innerElement.factory && !innerElement.key) {
                    var allObjects = proxy._defaultScaleValues();
                    proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
				}	
               				
            });
            return element;
        },
	 	
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {		    
            var defaultObj = null;
            var proxy = this;
            var type;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];
               type = typeof name;	
            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object" && (type!="string" ||( type=="string" && name.indexOf('_') == -1 && name.indexOf('__') == -1)) ) {
                    proxy._LoadIndividualDefaultValues(ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
			 
        },
		  
        initialize: function () {
            this._initObject(this);

            if (this.Model.frame.backgroundImageUrl)
                this._drawCustomImage(this, this.Model.frame.backgroundImageUrl);
            else {
                if (this.Model.scales != null)
                    this._drawScales();
            }
        },
		  
        _initObject: function (element) {
            this._savedPoints = [];
            this.element.addClass("e-widget");
            element.GaugeEl = element.element;
            if (element.canvasEl) {
                element.canvasEl.parent().empty();
                element.GaugeEl.empty();
            }
            element.canvasEl = $("<canvas></canvas>");
            var outerLabelCount = 0;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (var j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                    if (this.model.scales[i].markerPointers[j].value == null)
                        this.model.scales[i].markerPointers[j].value = this.value();
                }
                for (var l = 0; this.model.scales[i].barPointers[l] != null; l++) {
                    if (this.model.scales[i].barPointers[l].value == null)
                        this.model.scales[i].barPointers[l].value = this.value();
                }
                for (var k = 0; this.model.scales[i].customLabels[k] != null && this.model.scales[i].showCustomLabels == true; k++) {
                    outerLabelCount++;
                    if (this.model.scales[i].customLabels[k].value != null && element.GaugeEl.find('div').length == 0) {
                        if (this.model.scales[i].customLabels[k] != null && this.model.scales[i].customLabels[k].positionType != null && this.model.scales[i].customLabels[k].positionType == "outer") {
                            element.outerDiv = ej.buildTag("div");
                            if (element.model.outerCustomLabelPosition == "bottom") {
                                element.GaugeEl.append(element.canvasEl);
                                element.GaugeEl.append(element.outerDiv);
                                element.outerDiv.css('text-align', 'center');
                                element.GaugeEl.css({ 'width': element.model.width });
                            }
                            else {
                                if (element.model.outerCustomLabelPosition != "top") {
                                    var table = ej.buildTag("TABLE");
                                    table.css('width', '100%');
                                    var tr = ej.buildTag("TR");
                                    var td1 = ej.buildTag("TD");
                                    var td2 = ej.buildTag("td");
                                    if (element.model.outerCustomLabelPosition == "left") {
                                        td1.append(element.outerDiv);
                                        td2.append(element.canvasEl);
                                    }
                                    else {
                                        td1.append(element.canvasEl);
                                        td2.append(element.outerDiv);
                                    }
                                    tr.append(td1);
                                    tr.append(td2);
                                    table.append(tr);
                                    element.GaugeEl.append(table);
                                    element.outerDiv.css({
                                        'width': this.element.width() - element.model.width
                                    });
                                }
                                else {
                                    element.GaugeEl.append(element.outerDiv);
                                    element.GaugeEl.append(element.canvasEl);
                                    element.GaugeEl.css({ 'width': element.model.width });
                                    element.outerDiv.css('text-align', 'center');
                                }
                            }
                        }
                        else
                            element.GaugeEl.append(element.canvasEl);
                    }
                }
                if (outerLabelCount == 0)
                    element.GaugeEl.append(element.canvasEl);
            }
            element.canvasEl.attr("role", "presentation");
            if (_linearGaugeCount == initialGaugeCount) {
                initialLinearDivWidth = window.innerWidth;
            }
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
		  
        _drawFrameCircle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.arc(location.startX, location.startY, style.circleRadius, 0, 2 * Math.PI, true);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
		  
        _drawFrameRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.lineTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
		  
        _drawFrameThermometer: function (location, style, element) {
            var radius = element.Model.orientation == "Vertical" ? Math.sqrt(style.width * style.width + style.width * style.width) / 2 : Math.sqrt(style.height * style.height + style.height * style.height) / 2;   
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "Vertical") {
                if (element.scaleEl[element.scaleIndex].direction == "counterclockwise") {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + style.height - Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (-45 / 180), Math.PI * (225 / 180), false);
                    element.contextEl.lineTo(location.startX, location.startY + style.calDistance + style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.width / 2, style.width / 2, -Math.PI, 0, false);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.calDistance + style.width / 2);
                }
                else {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (45 / 180), Math.PI * (-225 / 180), true);
                    element.contextEl.lineTo(location.startX, location.startY + style.height - style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.height - style.width / 2, style.width / 2, -Math.PI, 0, true);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.width / 2);
                }
            }
            else {
                if (element.scaleEl[element.scaleIndex].direction == "counterclockwise") {
                    element.contextEl.arc(location.startX + style.width - radius / 4 - Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (135 / 180), Math.PI * (225 / 180), true);
                    element.contextEl.lineTo(location.startX + style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), true);
                    else
                        element.contextEl.lineTo(location.startX + style.height / 2, location.startY + style.height);

                }
                else {
                    element.contextEl.arc(location.startX + radius / 4 + Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (45 / 180), Math.PI * (315 / 180), false);
                    element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width - style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), false);
                    else
                        element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY + style.height);
                }
            }
            this._contextClosePath(style, element);
        },
		  
        _drawFrameRoundedRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.lineTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY, location.startX + style.width, location.startY + style.radius);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY + style.height, location.startX + style.width - style.radius, location.startY + style.height);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            element.contextEl.quadraticCurveTo(location.startX, location.startY + style.height, location.startX, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX, location.startY + style.radius);
            element.contextEl.quadraticCurveTo(location.startX, location.startY, location.startX + style.radius, location.startY);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		  
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
		  
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
		  
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.Model.scales;
            this.contextEl.save();
            this.contextEl.translate(this.Model.frame.outerWidth + this.Model.frame.innerWidth, this.Model.frame.outerWidth + this.Model.frame.innerWidth);
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                self._setScaleCordinates(element, element.type);
            });
            this._setTicks();
            this._setLabels();
            this._setRange();
            this._setCustomLabel();
            this._flagPointer = false;
            this._tempOpacity=this.model.scales[0].barPointers[0].opacity;
            this._setBarPointers();
            this._setMarkerPointers();
            this._setIndicators();
            $.each(this.Model.scales, function (ind, elemt) {
                if (elemt.showBarPointers) {
                    if (elemt.barPointers.length > 1)
                        self.model.enableAnimation = false;
                }
                if (elemt.showMarkerPointers) {
                    if (elemt.markerPointers.length > 1)
                        self.model.enableAnimation = false;
                }
            });
            if (!this.contextEl.putImageData)
                this.model.enableAnimation = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.enableAnimation && this._animationFlag) {
                        this._onAnimate(delay);
                    }
                }
            }
        },
		  
        _setTicks: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (tickIndex, tickElement) {
                            self.tickIndex = tickIndex;
                            self._setTicksCordinates(tickElement, tickIndex);
                        });
                    }
                }
            });
        },
		  
        _setLabels: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (labelIndex, labelElement) {
                            self.labelIndex = labelIndex;
                            self._setLabelCordinates(labelElement, labelIndex);
                        });
                    }
                }
            });
        },
		  
        _setIndicators: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.indicators != null && element.showIndicators) {
                    self.indicatorEl = element.indicators;
                    $.each(element.indicators, function (indicatorIndex, indicatorElement) {
                        self.indicatorIndex = indicatorIndex;
                        self._drawIndicator(indicatorIndex, indicatorElement);
                    });
                }
            });
        },
		  
        _setBarPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showBarPointers) {
                    self.scaleIndex = index;
                    if (element.barPointers != null) {
                        self.barPointerEl = element.barPointers;
                        $.each(element.barPointers, function (barPointerIndex, barPointerElement) {
                            self.barPointerIndex = barPointerIndex;
                            element.opacity = (self.scaleIndex == 0 && self.barPointerIndex == 0 && self.model.enableAnimation == true && (self._flagPointer == false && self._animationFlag == true) && self.model.scales[0].type == "thermometer") ? 0 : self._tempOpacity;
                            self._drawScaleBarPointer(barPointerElement, barPointerIndex);
                            self._flagPointer=true;
                        });
                    }
                }
            });
        },
		  
        _setMarkerPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showMarkerPointers) {
                    self.scaleIndex = index;
                    if (element.markerPointers != null) {
                        self.markerPointerEl = element.markerPointers;
                        $.each(element.markerPointers, function (markerPointerIndex, markerPointerElement) {
                            self.markerPointerIndex = markerPointerIndex;
                            self._drawMarkerPointer(markerPointerElement, markerPointerIndex);
							self.canvasEl.attr("aria-label", self.model.scales[self.scaleIndex].markerPointers[self.markerPointerIndex].value);
                        });
                    }
                }
            });
        },
		  
        _onAnimate: function (delay) {
            var self = this, timer, timer1;
            var currentValue = self.model.scales[0].minimum;
            var barPointerValue = self.model.scales[0].barPointers[0].value;
            var markerPointerValue = self.model.scales[0].markerPointers[0].value;
            timer = setInterval(function pointerchan() {
			  if(self.model){
                if (barPointerValue > currentValue || currentValue == self.model.scales[0].minimum) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    if (self.scaleEl[0].type == "thermometer") {
                        self.model.scales[0].barPointers[0].value = (barPointerValue > currentValue) ? currentValue: barPointerValue;
                        if (self.contextEl.putImageData != "undefined") {
                            self._setBarPointers();
                        }
                        else {
                            barPointerValue > currentValue ? self.setBarPointerValue(0, 0, currentValue) : self.setBarPointerValue(0, 0, barPointerValue);
                        }
                    }
                    else
                        barPointerValue > currentValue ? self.setBarPointerValue(0, 0, currentValue) : self.setBarPointerValue(0, 0, barPointerValue);
                }
                else {
                    self._animationFlag = false;
                    self.setBarPointerValue(0, 0, barPointerValue);
                    window.clearInterval(timer);
                }
			  } else window.clearInterval(timer); 	
            }, delay);
            timer1 = setInterval(function pointerchan() {
            if(self.model && self.model.enableMarkerPointerAnimation){
                if (markerPointerValue > currentValue || currentValue == self.model.scales[0].minimum) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    if (self.scaleEl[0].type == "thermometer") {
                        self.model.scales[0].markerPointers[0].value = (markerPointerValue > currentValue) ? currentValue : markerPointerValue;
                        if (self.contextEl.putImageData != "undefined") {
                            self._setMarkerPointers();
                            }
                            else {
                                markerPointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, markerPointerValue);
                            }
                        }
                        else
                            markerPointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, markerPointerValue);
                    }
                    else {
                        self._animationFlag = false;
                        self.setPointerValue(0, 0, markerPointerValue);
                        window.clearInterval(timer1);
                }} else window.clearInterval(timer1);
            }, delay);
        },
		  
        _setRange: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.ranges != null && element.showRanges) {
                    self.rangeEl = element.ranges;
                    $.each(element.ranges, function (rangeIndex, rangeElement) {
                        self.rangeIndex = rangeIndex;
                        self._drawRange(rangeElement);
                    });
                }
            });
        },
		  
        _setCustomLabel: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabels != null && element.showCustomLabels) {
                    self.customLabelEl = element.customLabels;
                    $.each(element.customLabels, function (cusLblIndex, cusLblElement) {
                        self.customLabelIndex = cusLblIndex;
                        if (self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex] != null && self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex].positionType != null && self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex].positionType == "outer")
                            self._setOuterCustomLabelCordinates(cusLblIndex, cusLblElement);
                        else
                            self._setCustomLabelCordinates(cusLblIndex, cusLblElement);
                    });
                }
            });
        },
          
        _setOuterCustomLabelCordinates: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            var div = ej.buildTag("div." + this._id + "outercustomlbl");
            div.text(this.model.scales[this.scaleIndex].customLabels[index].value);
            var position = (this.model.outerCustomLabelPosition == "right" || this.model.outerCustomLabelPosition == "left") ? "left" : "center";
            this.outerDiv.append(div);
            this.outerDiv.append('</br>');
            if (position == "center")
                div.css({ 'display': 'inline-block', 'margin': '0 auto', 'max-width': this.model.width });
            else
                div.css({ 'display': 'inline-block', 'max-width': this.element.width() - this.model.width > 10 ? this.element.width() - this.model.width : 10 });
            div.css({
                "color": element.color,
                'overflow': 'hidden',
                'text-overflow': 'ellipsis',
                'white-space': 'nowrap',
                'font-size': (element.font != null && element.font.size != null) ? element.font.size : "12px",
                'font-family': (element.font != null && element.font.fontFamily != null) ? element.font.fontFamily : "Arial",
                'font-weight': (element.font != null && element.font.fontStyle != null) ? element.font.fontStyle : "Normal",
                'text-align': position
            });

        },
		  
        _setScaleCordinates: function (element, scaleStyle) {
            var location, style, radius;
            this.opacity = 1;
            this.bottomRadius = Math.sqrt(element.width * element.width + element.width * element.width) / 2;

            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.Model.frame.outerWidth + this.Model.frame.innerWidth),
                width: this.canvasEl[0].width - 2 * (this.Model.frame.outerWidth + this.Model.frame.innerWidth)
            };
            if (this.Model.orientation == "Vertical") {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.width) * ((element.position.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.length) * ((element.position.y) / 100);
            }
            else {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.length) * ((element.position.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.width) * ((element.position.y) / 100);
            }
            radius = scaleStyle == "roundedrectangle" ? 5 : 0;
            location = { "startX": this.scaleStartX[this.scaleIndex], "startY": this.scaleStartY[this.scaleIndex] };
            style = {
                "width": this.Model.orientation == "Vertical" ? element.width : element.length, "isStroke": true,
                "topRounded": true,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((this.Model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.backgroundColor)),
                "lineWidth": element.border.width,
                "radius": radius,
                "height": this.Model.orientation == "Vertical" ? element.length : element.width,
                "isFill": true,
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
            };
	    if (element.maximum < element.minimum) {
                var tempValue = element.maximum;
                element.maximum = element.minimum;
                element.minimum=tempValue;
            }
			 if (element.maximum == element.minimum)
	            element.maximum = element.maximum + 1;
            this.minimum(element.minimum);
            this.maximum(element.maximum);
            if (this._notifyArrayChange) {
                this._notifyArrayChange("scales[" + this.scaleIndex + "]maximum", element.maximum);
                this._notifyArrayChange("scales[" + this.scaleIndex + "]minimum", element.minimum);
            }
            if (element.shadowOffset) {
                this.contextEl.shadowBlur = element.shadowOffset;
                this.contextEl.shadowColor = (style.fillStyle == "transparent") ? "rgba(0,0,0,0)" : style.fillStyle;
            }
            this._drawFrame(scaleStyle, location, style);
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && !this.isScaleModified) {
                this._modifyWidth();
                this.isScaleModified = true;
            }
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		  
        _setTicksCordinates: function (element, index) {
            var staticPosition, height, lineChangePosition, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {
                interval = element.type == "majorinterval" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.placement == "near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] : this.scaleStartY[this.scaleIndex];
                }
                else if (element.placement == "far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width;
                }
                else if (element.placement == "center") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
                }
                height = element.placement == "near" ? -element.height : element.height;
                for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this._getClockwiseLinePosition(value);
                        this.region = {
                            "lineChangePosition": lineChangePosition + (this.Model.orientation == "horizontal" ? element.distanceFromScale.x : (element.distanceFromScale.y)),
                            "lineStaticPosition": staticPosition + (this.Model.orientation == "horizontal" ? element.distanceFromScale.y : (element.distanceFromScale.x))
                        };
                        this.style = {
                            "lineHeight": height,
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "tickShape": element.TickShape,
                            "strokeStyle": element.color ? ((element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color)) : ((this.Model.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.tickColor)),
                            "lineWidth": element.width
                        }
                        if (this.Model.drawTicks)
                            this._onDrawTicks(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270, value);
                        this._drawTickMark(this.region, this.style);
                    }
                }

                if (this.contextEl.getImageData)
                    this.tickImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
            }
        },
		  
        _drawTickMark: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.lineWidth = style.lineWidth;
            this.contextEl.strokeStyle = style.strokeStyle;
            if (this.Model.orientation == "Vertical")
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
            else
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
            this.contextEl.lineTo(0, 0);
            if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this.contextEl.stroke();
            this.contextEl.restore();
            this.contextEl.closePath();
        },
		  
        _addDecimal: function (lblValue, lblInterval) {
            var value = lblValue.toString();
            var interval = lblInterval.toString();
            var vDecimal;
            var iDecimal;
            if (value.indexOf('.') > -1)
                vDecimal = value.length - value.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (interval.indexOf('.') > -1)
                iDecimal = interval.length - interval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var decimal = vDecimal > iDecimal ? vDecimal : iDecimal;
            var correctValue = (lblValue * Math.pow(10, decimal) + lblInterval * Math.pow(10, decimal)) / Math.pow(10, decimal);
            return correctValue;
        },
		  
        _setLabelCordinates: function (element, index) {

            var xDistanceFromScale, yDistanceFromScale, staticPosition, lineChangePosition, interval, locale = this.model.locale;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {

                if (this.Model.orientation == "Vertical") {
                    xDistanceFromScale = element.distanceFromScale.x;
                    yDistanceFromScale = element.distanceFromScale.y;
                }
                else {
                    xDistanceFromScale = element.distanceFromScale.y;
                    yDistanceFromScale = element.distanceFromScale.x;
                }
                interval = element.type == "major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.placement == "near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width - 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "right" : "center";
                }
                else if (element.placement == "far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width + 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "left" : "center";
                }
                else {
                    this.contextEl.textAlign = "center";
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + this.scaleEl[this.scaleIndex].border.width / 2;
                }
                for (var value = this.scaleEl[this.scaleIndex].minimum; value <= this.scaleEl[this.scaleIndex].maximum; value = this._addDecimal(value, interval)) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this.scaleEl[this.scaleIndex].direction == "counterclockwise" ? this._getCounterClockwiseLinePosition(value) : this._getClockwiseLinePosition(value);
                        this.labelValue = value;
                        this.region = {
                            "lineChangePosition": lineChangePosition + (yDistanceFromScale),
                            "lineStaticPosition": staticPosition + (xDistanceFromScale)
                        };
                        this.style = {
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "fillStyle": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.textColor)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                            "font": this._getFontString(this, element.font),
                            "textValue": this.labelValue
                        };
						
						this.style.textValue  = this.labelValue = (locale && this.model.enableGroupSeparator) ? this.labelValue.toLocaleString(locale) : this.labelValue;
                        if (this.Model.drawLabels)
                            this._onDrawLabels(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270);
                        this._drawLabel(this.region, this.style,false);
                    }
                }
            }

            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);

        },
		  
        _drawLabel: function (location, style, isCustomLabel) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.textBaseline = "middle";
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.font = style.font;
            if (style.opacity)
                this.contextEl.globalAlpha = style.opacity;
            if (this.Model.orientation == "Vertical") {
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
                if (this.model.tooltip.showLabelTooltip && !isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineStaticPosition + 5, "startY": location.lineChangePosition + 10, "width": 15, "height": 15, "value": style.textValue });
                if (this.model.tooltip.showCustomLabelTooltip && isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineStaticPosition - 35, "startY": location.lineChangePosition + 10, "width": 110, "height": 15, "value": style.textValue });
            }
            else {
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
                if (this.model.tooltip.showLabelTooltip && !isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineChangePosition + 10, "startY": location.lineStaticPosition + 10, "width": 15, "height": 15, "value": style.textValue });
                if (this.model.tooltip.showCustomLabelTooltip && isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineChangePosition - 35, "startY": location.lineStaticPosition + 10, "width": 110, "height": 15, "value": style.textValue });
            }
            this.contextEl.lineTo(0, 0);
            if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
			if(!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel){
			    var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPlacement;
            if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "back")
                style.textValue = style.textValue + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
            else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "front")
                style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
            }
            this.contextEl.fillText(style.textValue, 0, 0);
            this.contextEl.fill();
            this.contextEl.restore();

        },
		  
        _drawScaleBarPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var grd, lineYPosition, radius, height, width, startY, backgroundColor, startX, gradients;
            gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            radius = this.scaleEl[this.scaleIndex].type == "roundedrectangle" ? 5 : 0;
            if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise") {
                lineYPosition = this._getClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.width, this.scaleStartY[this.scaleIndex]);
                }
                else {
                    startX = this.scaleStartX[this.scaleIndex];
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + element.width / 2);
                }
            }
            else {
                lineYPosition = this._getCounterClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition, startX + element.width, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition);
                }
                else {
                    startX = this.scaleEl[this.scaleIndex].type == "thermometer" ? lineYPosition - this.scaleEl[this.scaleIndex].width / 2 - this.scaleEl[this.scaleIndex].border.width / 2 : lineYPosition - this.scaleEl[this.scaleIndex].border.width / 2;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + element.width / 2);
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            else if (this.Model.ScaleInterior)
                this._setGradientColor(this, grd, this.Model.ScaleInterior.colorInfo);
            else
                this._setGradientColor(this, grd, gradients);
            if (this.Model.orientation == "Vertical") {
                startY = this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise" ? this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].border.width / 2 : lineYPosition;
                if (this.scaleEl[this.scaleIndex].direction == "counterclockwise" && this.scaleEl[this.scaleIndex].type == "thermometer")
                    startY = startY - this.scaleEl[this.scaleIndex].width / 2;
                height = this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise" ? lineYPosition - this.scaleStartY[this.scaleIndex] : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition - this.scaleEl[this.scaleIndex].border.width / 2;
                width = element.width;
            }
            else {
                startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                height = element.width;
                width = this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise" ? lineYPosition - this.scaleStartX : this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition;
            }
            this.region = { "startX": startX + this.scaleEl[this.scaleIndex].border.width / 2, "startY": startY };
            this.style = {
                "width": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "horizontal") ? width + height / 2 - this.scaleEl[this.scaleIndex].border.width / 2 : width,
                "lineWidth": element.border.width,
                "radius": radius,
                "topRounded": false,
                "isStroke": false,
                "isFill": true,
                "height": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Vertical") ? height + width / 2 : height,
                "strokeStyle": element.border.color == null ? ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor)) : ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)),
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "calDistance": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Vertical") ? (this.scaleEl[this.scaleIndex].width - this.barPointerEl[this.barPointerIndex].width) / 2 : 0
            };
            this.value(element.value);
            if (this._notifyArrayChange)
                this._notifyArrayChange("scales[" + this.scaleIndex + "]barpointers[" + index + "]value", element.value);
            if (this.Model.drawBarPointers)
                this._onDrawBarPointers(element.value);
            this._drawFrame(this.scaleEl[this.scaleIndex].type, this.region, this.style);
            if (this.contextEl.getImageData)
                this.barPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		  
        _drawMarkerPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var startX, startY, radius, linePosition, angle, grd, backgroundColor;
            var gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            this.markerPlacement = element.placement;
            radius = Math.sqrt(element.width * element.width + element.length * element.length) / 2;
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isModify)
                this._modifyWidth();
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "near") {
                    if (element.type == "star")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale - element.width;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale + (element.border.width);
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "near") {
                    if (element.type == "star")
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale - element.length;
                    else
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale + (element.border.width);
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            linePosition = this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise" ? this._getClockwiseLinePosition(element.value) : this._getCounterClockwiseLinePosition(element.value);
            if (element.type == "star") {
                if (this.Model.orientation == "Vertical") {
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.width, this.scaleStartY[this.scaleIndex]);
                    startY = linePosition - element.length / 3;
                }
                else {
                    grd = this.contextEl.createLinearGradient(linePosition, startX, linePosition, startX + element.length);
                    startY = linePosition - element.width / 2;
                }
            }
            else {
                grd = this.contextEl.createLinearGradient(0, 0, element.width, 0);
                startY = linePosition;
            }
            if (element.type == "roundedrectangle") {
                if (this.Model.orientation == "Vertical" && this.markerPlacement == "near")
                    startY += element.length;
                else if (this.Model.orientation == "horizontal") {
                    if (this.markerPlacement == "near")
                        startY -= (element.width);
                    startX += element.width / 2;
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            else if (this.Model.PointerInterior)
                this._setGradientColor(this, grd, this.Model.PointerInterior.colorInfo);
            else
                this._setGradientColor(this, grd, gradients);

            this.region = { "startX": this.Model.orientation == "Vertical" ? startX : startY, "startY": this.Model.orientation == "Vertical" ? startY : startX };
            this.style = {
                "width": element.width,
                "radius": element.type == "rectangle" ? 0 : radius,
                "height": element.length,
                "lineWidth": element.border.width,
                "isFill": true,
                "isStroke": true,
                "angle": angle,
                "strokeStyle": ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)),
                "markerPlacement": this.markerPlacement,
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd))
            };
            this.value(element.value);
            if (this._notifyArrayChange)
                this._notifyArrayChange("scales[" + this.scaleIndex + "]markerpointers[" + index + "]value", element.value);
            if (this.Model.drawMarkerPointers)
                this._onDrawMarkerPointers(angle, element.value);
            if (element.type == "roundedrectangle")
                this.style.radius = 5;
            this._drawMarkerType(element.type, this.region, this.style);
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isModify) {
                this._restoreWidth();
                this.isScaleModified = false;
            }
            if (this.contextEl.getImageData)
                this.markerPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },

        _drawMarkerType: function (type, location, style) {
            switch (type) {
                case "rectangle":
                    this._drawRectangle(location, style, this);
                    break;
                case "triangle":
                    this._drawTriangle(location, style, this);
                    break;
                case "ellipse":
                    this._drawEllipse(location, style, this);
                    break;
                case "diamond":
                    this._drawDiamond(location, style, this);
                    break;
                case "pentagon":
                    this._drawPentagon(location, style, this);
                    break;
                case "circle":
                    this._drawCircle(location, style, this);
                    break;
                case "slider":
                    this._drawSlider(location, style, this);
                    break;
                case "star":
                    this._drawStar(location, style, this);
                    break;
                case "pointer":
                    this._drawPointer(location, style, this);
                    break;
                case "wedge":
                    this._drawWedge(location, style, this);
                    break;
                case "trapezoid":
                    this._drawTrapezoid(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawRoundedRectangle(location, style, this);
                    break;
            }
        },

		  
        _drawRange: function (element) {
            if (element.startValue < this.scaleEl[this.scaleIndex].maximum && element.endValue > this.scaleEl[this.scaleIndex].minimum && this.scaleEl[this.scaleIndex].minimum < this.scaleEl[this.scaleIndex].maximum && element.endValue <= this.scaleEl[this.scaleIndex].maximum) {
                var tempStartValue = element.startValue < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.startValue;
                var tempEndValue = element.endValue > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.endValue;
                this.rangePosition = element.placement;
                var startLinePosition, endLinePosition, startX, startY, grd, backgroundColor;
                var gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2 }];
                startLinePosition = (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise") ? this._getClockwiseLinePosition(tempStartValue) : this._getCounterClockwiseLinePosition(tempStartValue);
                endLinePosition = (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise") ? this._getClockwiseLinePosition(tempEndValue) : this._getCounterClockwiseLinePosition(tempEndValue);
                if (this.Model.orientation == "Vertical") {
                    if (element.placement == "far")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width;
                    if (element.placement == "near")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    if (element.placement == "center")
                        startX = (element.startWidth > element.endWidth) ? this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.startWidth / 2 : this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.endWidth / 2;
                    grd = this.contextEl.createLinearGradient(endLinePosition, endLinePosition, endLinePosition, startLinePosition);
                    this.region = { "startX": startX, "startY": startLinePosition, "endY": endLinePosition };
                }
                else {
                    if (element.placement == "far")
                        startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + element.distanceFromScale + this.scaleEl[this.scaleIndex].border.width;
                    if (element.placement == "near")
                        startY = this.scaleStartY[this.scaleIndex] + element.distanceFromScale;
                    if (element.placement == "center")
                        startY = (element.startWidth > element.endWidth) ? this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.startWidth / 2 : this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.endWidth / 2;
                    grd = this.contextEl.createLinearGradient(endLinePosition, startY, startLinePosition, startY);
                    this.region = { "startX": startLinePosition, "startY": startY, "endX": endLinePosition };
                }

                if (element.backgroundColor)
                    backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
                else if (element.gradients)
                    this._setGradientColor(this, grd, element.gradients.colorInfo);
                else if (this.Model.RangeInterior)
                    this._setGradientColor(this, grd, this.Model.RangeInterior.colorInfo);
                else
                    this._setGradientColor(this, grd, gradients);

                this.style = {
                    "startWidth": element.startWidth, "lineWidth": element.border.width,
                    "isStroke": true, "isFill": true, "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                    "endWidth": element.endWidth,
                    "fillStyle": element.backgroundColor ? ((backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                    "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
                };
                if (this.Model.drawRange)
                    this._onDrawRange();
                this._drawRangeBar(this.region, this.style);
                if (this.contextEl.getImageData)
                    this.rangeImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
            }
        },
		  
        _drawRangeBar: function (location, style) {
            this._contextOpenPath(style, this);
            if (this.Model.orientation == "Vertical") {
                this.contextEl.lineTo(location.startX, location.startY);
                this.contextEl.lineTo(location.startX, location.endY);
                if (this.rangePosition == "near") {
                    this.contextEl.lineTo(location.startX - style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX - style.startWidth, location.startY);
                }
                else {
                    this.contextEl.lineTo(location.startX + style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX + style.startWidth, location.startY);
                }
            }
            else {
                this.contextEl.lineTo(location.startX, location.startY);
                this.contextEl.lineTo(location.endX, location.startY);
                if (this.rangePosition == "near") {
                    this.contextEl.lineTo(location.endX, location.startY - style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY - style.startWidth);
                }
                else {
                    this.contextEl.lineTo(location.endX, location.startY + style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY + style.startWidth);
                }
            }
            this._contextClosePath(style, this);
        },
		  
        _setCustomLabelCordinates: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            var startX, startY;
            this.contextEl.textAlign = "center";
            if (this.Model.orientation == "Vertical") {
                startX = (this.bounds.width) * ((element.position.x) / 100);
                startY = (this.bounds.height) * ((element.position.y) / 100);
            }
            else {
                startX = (this.bounds.width) * ((element.position.x) / 100);
                startY = (this.bounds.height) * ((element.position.y) / 100);
            }
            this.region = { "lineStaticPosition": this.Model.orientation == "Vertical" ? startX : startY, "lineChangePosition": this.Model.orientation == "Vertical" ? startY : startX }
            this.style = { "angle": element.textAngle, "textValue": element.value, "fillStyle": element.color ? ((element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "font": this._getFontString(this, element.font) };
            if (this.Model.drawCustomLabel)
                this._onDrawCustomLabel();
            this._drawLabel(this.region, this.style,true);

            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		  
        _drawIndicator: function (index, element) {
            var self = this, xlocation, ylocation, txtLocation, isInStateRange = false;
            xlocation = (this.bounds.width - 2 * element.width) * ((element.position.x) / 100);
            ylocation = (this.bounds.height - 2 * element.height) * ((element.position.y) / 100);
            txtLocation = { "x": (this.bounds.width) * ((element.textLocation.x) / 100), "y": (this.bounds.height) * ((element.textLocation.y) / 100) };
            self.region = { "startX": element.type == "circle" ? xlocation + element.width : xlocation, "textLocation": txtLocation, "startY": element.type == "circle" ? ylocation + element.height : ylocation, "startAngle": 0, "endAngle": 2 * Math.PI };
            self.style = {
                "radius": element.type == "roundedrectangle" ? 2 : 0,
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : this._getColor(element, "#FFFFFF"),
                "angle": 0,
                "circleRadius": (element.height + element.width) / 2,
                "height": element.height,
                "width": element.width,
                "lineWidth": element.border.width,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : this._getColor(element, "#FFFFFF"),
                "isStroke": true,
                "isFill": true,
                "indicatorText": null,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.Model.drawIndicators)
                this._onDrawIndicators(self.style, self.region);
            if (element.stateRanges != null) {
                $.each(element.stateRanges, function (sRIndex, srEl) {  
                    if (self.markerPointerEl[self.markerPointerIndex].value >= srEl.startValue && self.markerPointerEl[self.markerPointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (!ej.isNullOrUndefined(srEl.text) && srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = ((srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.textColor));
                            self.style.font = self._getFontString(self, element.font);
                        }
                        if (element.type != "text") {
                            self.style.strokeStyle = ((srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.borderColor));
                            self.style.fillStyle = ((srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.backgroundColor));
                            self._drawFrame(element.type, self.region, self.style, self);
                        }
                        else if (element.type == "text")
                            self._drawText(self.region, self.style);
                    }
                });
                }
            if (!isInStateRange && element.type != "text")
                this._drawFrame(element.type, self.region, self.style, self);
            if (this.contextEl.getImageData)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },

        _drawFrame: function (type, location, style) {
            switch (type) {
                case "circle":
                    this._drawFrameCircle(location, style, this);
                    break;
                case "rectangle":
                    this._drawFrameRectangle(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawFrameRoundedRectangle(location, style, this);
                    break;
                case "thermometer":
                    this._drawFrameThermometer(location,style,this) ;
                    break;
            }
        },
		  
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
		  
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "horizontal" && element.markerPlacement == "near") {
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY);   
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.lineTo(location.startX + (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY);   
            }
            this._contextClosePath(style, element);
        },
		  
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
		  
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		  
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY - style.height / 2);
            this._setContextRotation(style, element);
            element.contextEl.lineTo(style.radius, 0);
            element.contextEl.lineTo(style.width - style.radius, 0);
            element.contextEl.quadraticCurveTo(style.width, 0, style.width, style.radius);
            element.contextEl.lineTo(style.width, style.height - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.radius, style.height);
            element.contextEl.lineTo(style.radius, style.height);
            element.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.radius);
            element.contextEl.lineTo(0, style.radius);
            element.contextEl.quadraticCurveTo(0, 0, style.radius, 0);
            this._contextClosePath(style, element);
        },
		  
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            $(image).on('load', function () {
                element.contextEl.drawImage(this, 0, 0, element.Model.width, element.Model.height);
                if (element.Model.scales != null)
                    element._drawScales();
                if (element.Model.Items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
		  
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
          
		  	
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                return this._getMarkerPointerImage();
        },
		  	
        _getBarPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
		  	
        _getMarkerPointerImage: function () {
            if (this.barPointerImage)
                return this.barPointerImage;
            else
                return this._getCustomLabelImage();
        },
		  	
        _getCustomLabelImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
		  	
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
		  	
        _getLabelImage: function () {

            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
		  	
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
		  
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
            if(value <= this.scaleEl[scaleIndex].maximum && value >= this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                    if (!ej.isNullOrUndefined(this.outerDiv) && this.model.scale[scaleIndex].showCustomLabels)
                        this.outerDiv.empty();
                        this._setCustomLabel();
                        this._setMarkerPointers();
                        this._setIndicators();
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].value;
            else
                return null;
        },
		  
        setPointerWidth: function (scaleIndex, pointerIndex, width) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && width != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].width = width;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].width;
            else
                return null;
        },
		  
        setPointerHeight: function (scaleIndex, pointerIndex, height) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && height != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].length = height;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerHeight: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].length;
            else
                return null;
        },
		  
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
		  
        setPointerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale;
            else
                return null;
        },
		  
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].placement = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].placement;
            else
                return null;
        },
		  
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].type = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].type;
            else
                return null;
        },
		  
        setBarPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
            if(value <= this.scaleEl[scaleIndex].maximum && value >= this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].barPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		  
        getBarPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].value;
            else
                return null;
        },
		  
        setBarWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].width = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		  
        getBarWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].width;
            else
                return null;
        },
		  
        setBarDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		  
        getBarDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].distanceFromScale;
            else
                return null;
        },
		  
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].value = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
		  
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].value;
            else
                return null;
        },
		  
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
		  
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle;
            else
                return null;
        },
		  
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
            else
                return null;
        },
		  
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
            else
                return null;
        },
		  
        setRangeStartWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeStartWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth;
            else
                return null;
        },
		  
        setRangeEndWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeEndWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth;
            else
                return null;
        },
		  
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
            else
                return null;
        },
		  
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].placement = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].placement;
            else
                return null;
        },
		  
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].border.width = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].border.width;
            else
                return null;
        },
		  
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
            else
                return null;
        },
		  
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].type = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].type;
            else
                return null;
        },
		  
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].placement = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].placement;
            else
                return null;
        },
		  
        setLabelXDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.x = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelXDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.x;
            else
                return null;
        },
		  
        setLabelYDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.y = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelYDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.y;
            else
                return null;
        },
		  
        setTickAngle: function (scaleIndex, tickIndex, angle) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].angle = angle;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickAngle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].angle;
            else
                return null;
        },
		  
        setTickWidth: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].width = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickWidth: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].width;
            else
                return null;
        },
		  
        setTickHeight: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].height = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickHeight: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].height;
            else
                return null;
        },
		  
        setTickStyle: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].type = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickStyle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].type;
            else
                return null;
        },
		  
        setTickPlacement: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].placement = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickPlacement: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].placement;
            else
                return null;
        },
		  
        setTickXDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.x = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickXDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.x;
            else
                return null;
        },
		  
        setTickYDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.y = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickYDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.y;
            else
                return null;
        },
		  
        setScaleLocation: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].position.x = value.x;
                this.scaleEl[scaleIndex].position.y = value.y;
                this.initialize();
            }
        },
		  
        getScaleLocation: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return { "x": this.scaleEl[scaleIndex].position.x, "y": this.scaleEl[scaleIndex].position.y };
            else
                return null;
        },
		  
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
            if(value > this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].maximum = value;
                this.initialize();
            }
        },
		  
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].maximum;
            else
                return null;
        },
		  
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
            if(value < this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].minimum = value;
                this.initialize();
            }
        },
		  
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].minimum;
            else
                return null;
        },
		  
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].width = value;
                this.initialize();
            }
        },
		  
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].width;
            else
                return null;
        },
		  
        setScaleBarLength: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].length = value;
                this.initialize();
            }
        },
		  
        setScaleStyle: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].type = value;
                this.initialize();
            }
        },
		  
        getScaleStyle: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].type;
            else
                return null;
        },
		  
        getScaleBarLength: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].length;
            else
                return null;
        },
		  
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].border.width = value;
                this.initialize();
            }
        },
		  
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].direction = value;
                this.initialize();
            }
        },
		  
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].direction;
            else
                return null;
        },
		  
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].border.width;
            else
                return null;
        },
		  
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                this.initialize();
            }
        },
		  
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
            else
                return null;
        },
		  
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                this.initialize();
            }
        },
		  
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
            else
                return null;
        },
		  
        _reDrawBarPointer: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.customLabelImage) ? this.customLabelImage() : !ej.isNullOrUndefined(this.rangeImage) ? this.rangeImage : !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setBarPointers();
                this._setMarkerPointers();
                this._setIndicators();
            }
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getBarPointerImage(), 0, 0);
                    this._setBarPointers();
                    this._setMarkerPointers();
                    this._setIndicators();
                }

            }
        },
		  
        _reDrawMarkerPointer: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.customLabelImage) ? this.customLabelImage() : !ej.isNullOrUndefined(this.rangeImage) ? this.rangeImage : !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setMarkerPointers();
            }
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                    this._setMarkerPointers();
                }

            }
        },
		  
        _reDrawCustomLabel: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.rangeImage) ? this.rangeImage : !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        _reDrawRange: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        _reDrawLabel: function () {
            if (this.Model.frame.backgroundImageUrl){
                var tmpData = !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        _reDrawTickMark: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        refresh: function () {
            this._init();
        },
		  
		exportImage: function(fileName, fileType) {
			/// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
		    /// <param name="fileType" type="String">fileType to which the image has been saved</param>
		    if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
		        return false;
		    }
		    else {
		        var image = this.canvasEl[0].toDataURL();
		        image = image.replace(/^data:[a-z]*;,/, '');
		        var image1 = image.split(',');
		        var byteString = atob(image1[1]);
		        var buffer = new ArrayBuffer(byteString.length);
		        var intArray = new Uint8Array(buffer);
		        for (var i = 0; i < byteString.length; i++) {
		            intArray[i] = byteString.charCodeAt(i);
		        }
		        var blob = new Blob([buffer], { type: "image/png" });
		        if (ej.browserInfo().name === "msie")
		            window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
		        else {
		            var pom = document.createElement('a');
		            var url = URL.createObjectURL(blob);
		            pom.href = url;
		            pom.setAttribute('download', fileName + '.' + fileType);
		            if (document.createEvent) {
		                var e = document.createEvent("MouseEvents");
		                e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
		                pom.dispatchEvent(e);
		            }
		            else if (pom.fireEvent) {
		                pom.fireEvent("onclick");
		            }
		        }
		        return true;
		    }
        },
		 
		resizeCanvas: function () {
		    if (_linearGaugeCount != 0)
		        _linearGaugeCount = _linearGaugeCount - 1;
		    else
		        _linearGaugeCount = $(".e-lineargauge").length - 1;
		    var state, chk = true;
		    if (!ej.isNullOrUndefined(this.GaugeEl.parent().attr("style")))
		        state = this.GaugeEl.parent().attr("style").split(";");
		    if (!ej.isNullOrUndefined(state)) {
		        $.each(state, function (key, val) {
		            while (val.indexOf("width") != -1) {
		                chk = val.indexOf("px") == -1 ? true : false;
		                break;
		            }
		        });
		    }
            if (chk) {
                var ratio = window.innerWidth / initialLinearDivWidth;
                this.model.width *=ratio;
				for(var i=0;this.model.scales[i]!=null;i++){
                this.model.scales[i].length *= ratio;
                for (var j = 0; this.model.scales[i].markerPointers[j] != null || this.model.scales[i].barPointers[j] != null || this.model.scales[i].indicators[j] != null || this.model.scales[i].customLabels[j] != null || this.model.scales[i].ranges[j] != null || this.model.scales[i].labels[j] != null || this.model.scales[i].ticks[j] != null; j++) {
                    if (!ej.isNullOrUndefined(this.model.scales[i].markerPointers[j])) {
                        this.model.scales[i].markerPointers[j].length *= ratio;
                        this.model.scales[i].markerPointers[j].width *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].barPointers[j])) {
                        this.model.scales[i].barPointers[j].distanceFromScale *= ratio;
                        this.model.scales[i].barPointers[j].width *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].indicators[j]) && this.model.scales[i].showIndicators) {
                        this.model.scales[i].indicators[j].height *= ratio;
                        this.model.scales[i].indicators[j].width *= ratio;
                        this.model.scales[i].indicators[j].position.x *= ratio;
                        this.model.scales[i].indicators[j].textLocation.x *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].ticks[j])) {
                        this.model.scales[i].ticks[j].length *= ratio;
                        this.model.scales[i].ticks[j].width *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].ranges[j])) {
                        this.model.scales[i].ranges[j].startWidth *= ratio;
                        this.model.scales[i].ranges[j].endWidth *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].customLabels[j])) {
                        if (this.model.scales[i].customLabels[j].positionType != "outer")
                            this.model.scales[i].customLabels[j].position.x *= ratio;
                        this.model.scales[i].customLabels[j].font.size = (parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0]) * ratio < 10) ? "10px" : ((parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0]) * ratio) > this._customLblMaxSize) ? this._customLblMaxSize.toString() + 'px' : ((parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0])) * ratio).toString() + 'px';
                    }
				}
				}
				//this._initialize();
				this._render();
				if (_linearGaugeCount == 0) {
				    initialLinearDivWidth = window.innerWidth;
				}
            }
        },
        //Client-Side Events
	 
		_onDrawTicks: function (tickAngle, tickValue) {
		    var tick = { index: this.tickIndex, element: this.tickEl[this.tickIndex], angle: parseInt(tickAngle), value: tickValue }
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawTicks", data);
        },
		 
		_onDrawLabels: function (labelAngle) {
		    var label = { index: this.labelIndex, element: this.labelEl[this.labelIndex], angle: parseInt(labelAngle), value: this.labelValue }
		    var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, label: label };
		    this._trigger("drawLabels", data);
		},
		 
        _onDrawBarPointers: function (pointerValue) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, barPointerIndex: this.barPointerIndex, barElement: this.barPointerEl[this.barPointerIndex], context: this.contextEl, style: this.style, position: this.region, pointerValue: pointerValue };
            this._trigger("drawBarPointers", data);
        },
		 
        _onDrawMarkerPointers: function (pointerAngle, pointerValue) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, markerPointerIndex: this.markerPointerIndex, markerElement: this.markerPointerEl[this.markerPointerIndex], context: this.contextEl, style: this.style, position: this.region, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle) };
            this._trigger("drawMarkerPointers", data);
        },
		 
        _onDrawRange: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeIndex], context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawRange", data);
        },
	 
        _onDrawCustomLabel: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawCustomLabel", data);
        },
		 
        _onDrawIndicators: function (location, style) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, position: this.region };
            this._trigger("drawIndicators", data);
        },
		 
        onLoad: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("load", data);
        },
	 
        _onInit: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("init", data);
        },
		 
        _onRenderComplete: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
        
        _onMouseClick: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, markerPointer: markerpointer };
            this._trigger("mouseClick", data);
        },
        
        _onMouseClickMove: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, markerPointer: markerpointer };
            this._trigger("mouseClickMove", data);
        },
        
        _onMouseClickUp: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.region, markerPointer: markerpointer };
            this._trigger("mouseClickUp", data);
        },
        //_trigger: function (type, event, data) {
        //    var fn = this.Model[type];
        //    event = $.Event(event);
        //    event.type = type;
        //    $(this.GaugeEl).trigger(event, data);
        //    return !($.isFunction(fn) &&
        //        fn(this, data) === false ||
        //        event.isDefaultPrevented());
        //},

        //trigger: function (element, eventName, e) {
        //    e = $.extend(e || {}, new $.Event(eventName));
        //    e.stopPropagation();
        //    $(element).trigger(e);
        //    return e.isDefaultPrevented();
        //},
		 
        _restoreWidth: function () {
            this.scaleEl[this.scaleIndex].length = this.scaleEl[this.scaleIndex].length + this.bottomRadius + this.scaleEl[this.scaleIndex].width;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].width / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].width / 2;
            }
        },
		 
        _modifyWidth: function () {
            this.scaleEl[this.scaleIndex].length = this.scaleEl[this.scaleIndex].length - this.bottomRadius - this.scaleEl[this.scaleIndex].width;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].direction.toLowerCase() == "clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
            }
        },
        // return y positions for vertical gauge
	 
        _getClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = (value - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100);
            return linePosition;
        },
		 
        _getCounterClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = this.scaleEl[this.scaleIndex].maximum - value + this.scaleEl[this.scaleIndex].minimum;
            tempVal = (tempVal - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100);
            return linePosition;
        },
		 
        _getValue: function (position) {
            var tempVal;
            if (this.Model.orientation == "Vertical")
                tempVal = ((position.y - this.scaleStartY[this.scaleIndex]) / this.scaleEl[this.scaleIndex].length) * 100;
            else
                tempVal = (((position.x - this.scaleStartX[this.scaleIndex])) / this.scaleEl[this.scaleIndex].length) * 100;
            var value = ((tempVal * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum)) + this.scaleEl[this.scaleIndex].minimum) / 100;
            if (this.scaleEl[this.scaleIndex].direction == "counterclockwise")
                value = this.scaleEl[this.scaleIndex].maximum - value;
            else
                value = this.scaleEl[this.scaleIndex].minimum + value;
            return value;
        },
		 
        _getPointerXPosition: function (element) {
            var startX, angle;
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "near") {
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - (Math.sqrt(element.length * element.length + element.width * element.width) / 2) + element.distanceFromScale;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "near") {
                    startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - (Math.sqrt(element.length * element.length + element.width * element.width) / 2) + element.distanceFromScale;
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            return { "startx": startX, "angle": angle };
        },
		 
        _hexFromRGB: function (r, g, b) {
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return hex.join("").toUpperCase();
        },
	 
        _setGradientColor: function (element, gradients, options) {
            if (options.Name || typeof (options) === "string") {
                gradients.addColorStop(0, options);
                gradients.addColorStop(1, options);
            }
            else
                $.each(options, function (index, colorElement) {
                    gradients.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : colorElement.color);
                });
        },
		 
        _getFontString: function (element, font) {
            return font.fontStyle + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily;
        },
	 
        _setPointerDimension: function (style, element) {
            if (element.Model.orientation) {
                if (element.Model.orientation == "horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
		 
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
        _browserInfo: function () {
            var browser = {}, clientInfo = [],
            browserClients = {
                webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
                opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
            };
            for (var client in browserClients) {
                if (browserClients.hasOwnProperty(client)) {
                    clientInfo = navigator.userAgent.match(browserClients[client]);
                    if (clientInfo) {
                        browser.name = clientInfo[1].toLowerCase();
                        browser.version = clientInfo[2];
                        if (!!navigator.userAgent.match(/Trident\/7\./)) {
                            browser.name = "msie";
                        }
                        break;
                    }
                }
            }
            browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
            browser.pointerEnabled = window.navigator.pointerEnabled;
            return browser;
        },
        wireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent),
               browserInfo = this._browserInfo(),
               isPointer = browserInfo.isMSPointerEnabled,
               isIE11Pointer = browserInfo.pointerEnabled;
            this.startEv = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
            this.endEv = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
            this.moveEv = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
            this.leaveEv = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchleave mouseleave",
            this.scrollEv = matched.browser.toLowerCase() == "mozilla" ? (isIE11Pointer ? "mousewheel" : "DOMMouseScroll") : "mousewheel";
            this.model.browserInfo = browserInfo;

            var action= this.model.readOnly? 'pan-y pan-x' : 'none';
            $(this.element).css('touch-action', action);

            this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onHoverOCustomLabel = $.proxy(this._onHoverOCustomLabel, this);
            this.onLeaveOCustomLabel = $.proxy(this._onLeaveOCustomLabel, this);
            if (!this.model.readOnly) {
                this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
                this._on($(this.element), this.startEv, this._onMouseDown);
            }

            if (this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) {
                $(this.canvasEl).bind(this.moveEv, this.onMouseMoveHandler);
                $(this.canvasEl).bind(this.scrollEv, this.onMouseMoveHandler);
                $(this.canvasEl).bind(this.startEv, this.onMouseDownHandler);
                $(this.canvasEl).bind(this.endEv, this.onLeaveOCustomLabel);
                $(this.canvasEl).bind(this.leaveEv, this.onLeaveOCustomLabel);
            }
            this.element.bind(this.startEv, this.onMouseDownHandler);
            if (this.model.tooltip.showCustomLabelTooltip) {
                $('.' + this._id + 'outercustomlbl').bind("mouseenter", this.onHoverOCustomLabel);
                $('.' + this._id + 'outercustomlbl').bind(this.leaveEv, this.onLeaveOCustomLabel);
            }
        },
		 
        unWireEvents: function () {
            this.element.unbind(this.startEv, this.onMouseDownHandler);
        },
       
        _onHoverOCustomLabel: function (e) {
            if (e.currentTarget.innerHTML != null || e.currentTarget.innerHTML != "")
                this._showTooltip(e, e.currentTarget.innerHTML);
        },
        
        _onLeaveOCustomLabel: function (evt) {
            if (this.isTouch(evt)) {
                this._performTooltip(evt);
				window.clearTimeout(this.model.timer);
				this.model.timer = setTimeout(function () {
					$('.tooltipDiv').fadeOut(500);
	            }, 1200);
            }
            else {
                this._hideTooltip();
            }
        },
		 
        isTouch: function (evt) {
            var event = evt.originalEvent ? evt.originalEvent : evt;
            if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        },

        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
		 
        _onMouseDown: function (e) {
            var endPoint, position, greaterValue, lesserValue, startX;
            this._blockDefaultActions(e);
            this._mouseDown = true;
            var padding = this.isTouch(e) ? 10 : 0;
            var touch = e.originalEvent.touches ? e.originalEvent.touches[0] : e;
            endPoint = { "x": touch.pageX - $(this.canvasEl).offset().left - (this.Model.frame.outerWidth + this.Model.frame.innerWidth), "y": touch.pageY - $(this.canvasEl).offset().top - (this.Model.frame.outerWidth + this.Model.frame.innerWidth) };
            var self = this;
            if (!this.model.readOnly) {
                $.each(this.Model.scales, function (index, element) {
                    self.scaleIndex = index;
                    if (element.markerPointers != null) {
                        self.markerPointerEl = element.markerPointers;
                        $.each(element.markerPointers, function (mpIndex, mpElement) {
                            if (self.scaleEl[self.scaleIndex].direction.toLowerCase() == "clockwise")
                                position = self._getClockwiseLinePosition(mpElement.value);
                            else
                                position = self._getCounterClockwiseLinePosition(mpElement.value);
                            greaterValue = position + mpElement.width;
                            lesserValue = position - mpElement.width;
                            startX = self._getPointerXPosition(mpElement).startx;
                            var isOffset = self._isBetween(((self.Model.orientation == "horizontal" ? endPoint.y : endPoint.x) - mpElement.width), ((self.Model.orientation == "horizontal" ? endPoint.y : endPoint.x) + mpElement.width), startX, padding);
                            if ((self.Model.orientation == "horizontal" ? self._isBetween(lesserValue, greaterValue, endPoint.x, padding) : self._isBetween(lesserValue, greaterValue, endPoint.y, padding)) && isOffset)
                                self.activeElement = mpElement;
                            if (self.Model.scales[self.scaleIndex].barPointers[mpIndex] != null)
                                self.activeBarElement = self.Model.scales[self.scaleIndex].barPointers[mpIndex];
                            if (self.model.mouseClick)
                                self._onMouseClick(mpElement.value);
                            self.onMouseMoveHandler = $.proxy(self._onMouseMove, self);
                            self.onMouseUpHandler = $.proxy(self._onMouseUp, self);
                            $(document).bind(self.moveEv, self.onMouseMoveHandler);
                            $(document).bind(self.endEv, self.onMouseUpHandler);
                        });
                    }
                });
            }
        },
		 
        _isBetween: function (first, last, number, padding) {
            return (first < last ? number >= first - padding && number <= last + padding : number >= last - padding && number <= first + padding);
        },
		 
        _onMouseUp: function () {
            this._mouseDown = false;
            this.mouseMove = false;
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            if (this.model.mouseClickUp && this.activeElement)
                this._onMouseClickUp(this.activeElement.value);
            this.activeElement = null;
        },
        _mousePosition: function (evt) {
            if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return { x: evt.pageX, y: evt.pageY };
            else if (evt.originalEvent && !ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX > 0)
                return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
            else if (evt.originalEvent && evt.originalEvent.changedTouches != undefined) {
                if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                    return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
            }
            else
                return { x: 0, y: 0 };

        },
        _calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseposition = this._mousePosition(e);
            e.pageX = mouseposition.x;
            e.pageY = mouseposition.y;
        },
        getEvent: function (event) {
            return (event.targetTouches && event.targetTouches[0]) ? event.targetTouches[0] : event
        },
        _onMouseMove: function (e) {
            if (this._mouseDown && !ej.isNullOrUndefined(this.activeElement)) {
                this._blockDefaultActions(e);
                var touch = e.originalEvent.touches ? e.originalEvent.touches[0] : e;
                var endPoint = { "x": touch.pageX - $(this.canvasEl).offset().left - (this.Model.frame.outerWidth + this.Model.frame.innerWidth), "y": touch.pageY - $(this.canvasEl).offset().top - (this.Model.frame.outerWidth + this.Model.frame.innerWidth) };
                this.activeElement.value = this._getValue(endPoint);
                this.value(this.activeElement.value);
                if (this.model.mouseClickMove)
                    this._onMouseClickMove(this.activeElement.value);
                if (this.activeBarElement)
                    this.activeBarElement.value = this._getValue(endPoint);
                if (this.contextEl.putImageData)
                    this._reDrawBarPointer();
                else
                    this._init();
            }
            else {
                if ((this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) && (!this.isTouch(e))) {
                    this._performTooltip(e);
                    
                    
                }
            }
        },

        _performTooltip: function (e) {
            var hit = false;
            var padding = 10;
            var isTouch = this.isTouch(e);
            for (var i = 0; this._savedPoints[i] != null; i++) {
                if (isTouch) {
                    var mousedownCords = this._calTouchPosition(e),
                        pointer = this.getEvent(e),
                        currX = pointer.pageX,
                        currY = pointer.pageY;
                    var current = { "X": currX - $(this.canvasEl).offset().left, "Y": currY - $(this.canvasEl).offset().top };
                    if (current.X > this._savedPoints[i].startX - padding && current.X < (this._savedPoints[i].startX + this._savedPoints[i].width + padding) && current.Y > this._savedPoints[i].startY - padding && current.Y < (this._savedPoints[i].startY + this._savedPoints[i].height + padding)) {
                        this._showTooltip(e, this._savedPoints[i].value);
                        hit = true;
                    }
                    else if (hit == false)
                        this._hideTooltip();
                } else {
                    var current = { "X": e.pageX - $(this.canvasEl).offset().left, "Y": e.pageY - $(this.canvasEl).offset().top };

                    if (current.X > this._savedPoints[i].startX && current.X < (this._savedPoints[i].startX + this._savedPoints[i].width) && current.Y > this._savedPoints[i].startY && current.Y < (this._savedPoints[i].startY + this._savedPoints[i].height)) {
                        this._showTooltip(e, this._savedPoints[i].value);
                        hit = true;
                    }
                    else if (hit == false)
                        this._hideTooltip();
                }
            }
        },
     
        _showTooltip: function (event, val) {
            var tooltipText = val + "";
            var tooltipdiv = $('.tooltipDiv');
            if (tooltipdiv.length == 0) {
                tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                $(document.body).append(tooltipdiv);
            }
            if (this.model.tooltip.templateID != "" && this.model.tooltip.templateID != null) {
                var cloneNode = $("#" + this.model.tooltip.templateID).clone();
                $('.tooltipDiv')[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                $(tooltipdiv).css({
                    'background-color': this.model.backgroundColor,
                    'border': '1px solid #bbbcbb',
                    'border-radius': '3px',
                    'color': '#565656'
                });
                tooltipdiv.html(tooltipdiv.html().replace('#label#', tooltipText));
            } else {
                $(tooltipdiv).html(tooltipText);
                $(tooltipdiv).css({
                    'background-color': 'white',
                    'border': '2px solid #bbbcbb',
                    'position': 'absolute',
                    'padding': '10px 20px',
                    'margin-top': '5px',
                    'text-align': 'left',
                    'font': '12px Segoe UI',
                    'font-stretch': 'condensed',
                    'display': 'inline-block',
                    'border-radius': '3px',
                    'color': '#565656',
                    'width': 'auto'
                });
            }
            var tooltipMargin = 10;
            var xPos = event.pageX + tooltipMargin;
            var yPos = event.pageY + tooltipMargin;
            xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
            yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());
            $(tooltipdiv).css("left", xPos);
            $(tooltipdiv).css("top", yPos);
            $('.tooltipDiv').show();
        },
        
        _hideTooltip: function () {
            $('.tooltipDiv').remove();//.fadeOut(0, "linear");
        },
       
        _setTheme: function () {
            var selectedTheme = this.model.themeProperties[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
		 
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = this.model.themeProperties;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.backgroundColor);
                this.model.borderColor = ((!this.model.borderColor || this.model.borderColor == jsonObj[result[th]].scales.border.color) ? selectedTheme.scales.border.color : this.model.borderColor);
                this.model.labelColor = ((!this.model.labelColor || this.model.labelColor == jsonObj[result[th]].scales.labels.labelColor) ? selectedTheme.scales.labels.labelColor : this.model.labelColor);
                this.model.tickColor = ((!this.model.tickColor || this.model.tickColor == jsonObj[result[th]].scales.ticks.color) ? selectedTheme.scales.ticks.color : this.model.tickColor);

                for (var i = 0; i < this.model.scales.length; i++) {
                    for (var m = 0; m < this.model.scales[i].markerPointers.length; m++) {
                        this.model.scales[i].markerPointers[m].backgroundColor = (!this.model.scales[i].markerPointers[m].backgroundColor || this.model.scales[i].markerPointers[m].backgroundColor == jsonObj[result[th]].scales.markerPointers.backgroundColor) ? selectedTheme.scales.markerPointers.backgroundColor : this.model.scales[i].markerPointers[m].backgroundColor;
                        this.model.scales[i].markerPointers[m].border.color = (!this.model.scales[i].markerPointers[m].border.color || this.model.scales[i].markerPointers[m].border.color == jsonObj[result[th]].scales.markerPointers.border.color) ? selectedTheme.scales.markerPointers.border.color : this.model.scales[i].markerPointers[m].border.color;
                    }
                    for (var b = 0; b < this.model.scales[i].barPointers.length; b++) {
                        this.model.scales[i].barPointers[b].backgroundColor = (!this.model.scales[i].barPointers[b].backgroundColor || this.model.scales[i].barPointers[b].backgroundColor == jsonObj[result[th]].scales.barPointers.backgroundColor) ? selectedTheme.scales.barPointers.backgroundColor : this.model.scales[i].barPointers[b].backgroundColor;
                        this.model.scales[i].barPointers[b].border.color = (!this.model.scales[i].barPointers[b].border.color || this.model.scales[i].barPointers[b].border.color == jsonObj[result[th]].scales.barPointers.border.color) ? selectedTheme.scales.barPointers.border.color : this.model.scales[i].barPointers[b].border.color;
                    }
                }
            }
        }
    });
    
    ej.datavisualization.LinearGauge.TickType = {
          
        MajorInterval: "majorinterval",
          
        MinorInterval: "minorinterval"
    };
      
    ej.datavisualization.LinearGauge.LabelType = {
          
        Major: "major",
          
        Minor: "minor"
    };
      
    ej.datavisualization.LinearGauge.FontStyle = {
          
        Bold: "bold",
          
        Italic: "italic",
          
        Regular: "regular",
          
        Strikeout: "strikeout",
          
        Underline: "underline"
    };


      
    ej.datavisualization.LinearGauge.PointerPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.LinearGauge.TickPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.LinearGauge.LabelPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.LinearGauge.RangePlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      

    ej.datavisualization.LinearGauge.UnitTextPlacement = {
          
        Front: "front",
          
        Back: "back"
    };
      

    ej.datavisualization.LinearGauge.Directions = {
          
        Clockwise: "clockwise",
          
        CounterClockwise: "counterclockwise"
    };
      
    ej.datavisualization.LinearGauge.ScaleType = {
          
        Rectangle: "rectangle",
          
        RoundedRectangle: "roundedrectangle",
          
        Thermometer: "thermometer"
    };
      
    ej.datavisualization.LinearGauge.IndicatorType = {
          
        Rectangle: "rectangle",
          
        Circle: "circle",
          
        RoundedRectangle: "roundedrectangle",
          
        Text: "text"
    };
      
    ej.datavisualization.LinearGauge.MarkerType = {
          
        Rectangle: "rectangle",
          
        Triangle: "triangle",
          
        Ellipse: "ellipse",
          
        Diamond: "diamond",
          
        Pentagon: "pentagon",
          
        Circle: "circle",
          
        Star: "star",
          
        Slider: "slider",
          
        Pointer: "pointer",
          
        Wedge: "wedge",
          
        Trapezoid: "trapezoid",
          
        RoundedRectangle: "roundedrectangle"
    };
      
    ej.datavisualization.LinearGauge.CustomLabelPositionType = {
          
        Inner: "inner",
          
        Outer: "outer"
    };
      
    ej.datavisualization.LinearGauge.OuterCustomLabelPosition = {
          
        Left: "left",
          
        Right: "right",
          
        Top: "top",
          
        Bottom: "bottom",
    };
	  
    ej.datavisualization.LinearGauge.Themes = {
	  
        FlatLight: 'flatlight',

		FlatDark: 'flatdark'
    };

})(jQuery, Syncfusion);
;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, undefined) {

    ej.widget({ "ejDigitalGauge": "ej.datavisualization.DigitalGauge" }, {
     
        element: null,
     
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-digitalgauge",
        
        defaults:   {
              

            segmentData: {
                "0": [0, 1, 2, 3, 4, 5, 14, 15], "1": [1, 2], "2": [0, 14, 1, 6, 8, 4, 3, 15], "3": [0, 1, 2, 3, 6, 8, 14, 15], "4": [1, 2, 5, 6, 8], "5": [0, 2, 3, 5, 6, 8, 14, 15], "6": [0, 2, 3, 4, 5, 6, 8, 14, 15],
                "7": [0, 1, 2, 14], "8": [0, 1, 2, 3, 4, 5, 6, 8, 14, 15], "9": [0, 1, 2, 3, 5, 6, 8, 14, 15], "A": [0, 1, 2, 4, 5, 6, 8, 14], "B": [0, 1, 2, 3, 7, 9, 8, 14, 15], "C": [0, 3, 4, 5, 14, 15],
                "D": [0, 1, 2, 3, 7, 9, 14, 15], "E": [0, 3, 4, 5, 6, 8, 14, 15], "F": [0, 4, 5, 6, 8, 14], "G": [0, 2, 3, 4, 5, 8, 14, 15], "H": [1, 2, 4, 5, 6, 8], "I": [0, 3, 7, 9, 14, 15],
                "J": [1, 2, 3, 4, 15], "K": [4, 5, 6, 10, 11], "L": [3, 4, 5, 15], "M": [1, 2, 4, 5, 10, 13], "N": [1, 2, 4, 5, 11, 13], "O": [0, 1, 2, 3, 4, 5, 14, 15], "P": [0, 1, 4, 5, 6, 8, 14],
                "Q": [0, 1, 2, 3, 4, 5, 11, 14, 15], "R": [0, 1, 4, 5, 6, 8, 11, 14], "S": [0, 2, 3, 5, 6, 8, 14, 15], "T": [0, 7, 9, 14], "U": [1, 2, 3, 4, 5, 15], "V": [4, 5, 10, 12], "W": [1, 2, 4, 5, 11, 12],
                "X": [10, 11, 12, 13], "Y": [1, 5, 6, 7, 8], "Z": [0, 3, 10, 12, 14, 15]
            },
              
            matrixSegmentData: {
                "1": [0, 3, 0, 4, 1, 1, 1, 2, 1, 3, 1, 4, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "2": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 2, 5, 2, 5, 1, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "3": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5],
                "4": [0, 3, 0, 4, 0, 5, 1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 4, 2, 5, 3, 0, 3, 1, 3, 4, 3, 5, 4, 0, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4, 6, 5],
                "5": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "6": [0, 3, 0, 4, 0, 5, 0, 6, 1, 2, 1, 3, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 2, 5, 3, 5, 6, 5, 7, 6, 3, 6, 4, 6, 5, 6, 6],
                "7": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "8": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "9": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 4, 6, 5],
                "0": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "a": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "b": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "c": [1, 3, 1, 4, 1, 5, 1, 6, 2, 2, 2, 3, 3, 1, 3, 2, 4, 1, 4, 2, 5, 2, 5, 3, 6, 3, 6, 4, 6, 5, 6, 6],
                "d": [0, 6, 0, 7, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 5, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "e": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "f": [0, 4, 0, 5, 0, 6, 0, 7, 1, 3, 1, 4, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "g": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "h": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "i": [0, 3, 0, 4, 2, 1, 2, 2, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "j": [1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "k": [0, 1, 0, 2, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "l": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 5, 6, 6],
                "m": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 0, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 3, 5, 4, 5, 6, 5, 7, 6, 0, 6, 1, 6, 3, 6, 4, 6, 6, 6, 7],
                "n": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 0, 2, 1, 2, 2, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 0, 5, 1, 5, 6, 5, 7, 6, 0, 6, 1, 6, 6, 6, 7],
                "o": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 5, 2, 6, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "p": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 3, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 6, 1, 6, 2],
                "q": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7],
                "r": [0, 1, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 6, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "s": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 3, 1, 3, 2, 4, 3, 4, 4, 5, 4, 5, 5, 6, 1, 6, 2, 6, 3, 6, 4],
                "t": [0, 3, 0, 4, 1, 3, 1, 4, 2, 1, 2, 2, 2, 3, 2, 4, 2, 5, 2, 6, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 4, 6, 5, 6, 6, 6, 7],
                "u": [1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "v": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 5, 4, 5, 5, 6, 4],
                "w": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 2, 5, 3, 5, 4, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "x": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 2, 5, 3, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "y": [1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "z": [1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 6, 2, 7, 3, 5, 3, 6, 4, 4, 4, 5, 5, 3, 5, 4, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "A": [0, 3, 0, 4, 1, 2, 1, 3, 1, 4, 1, 5, 2, 2, 2, 3, 2, 4, 2, 5, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "B": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "C": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 4, 0, 4, 1, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "D": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "E": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "F": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "G": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 3, 4, 3, 5, 3, 6, 4, 0, 4, 1, 4, 6, 5, 1, 5, 2, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "H": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "I": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "J": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "K": [0, 1, 0, 2, 0, 5, 0, 6, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 5, 1, 5, 2, 5, 4, 5, 5, 6, 1, 6, 2, 6, 5, 6, 6],
                "L": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "M": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "N": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "O": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "P": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "Q": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "R": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 5, 1, 5, 5, 6, 2, 6, 1, 6, 2, 6, 6],
                "S": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 2, 3, 3, 3, 4, 3, 5, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "T": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4],
                "U": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "V": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4],
                "W": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 3, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "X": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "Y": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "Z": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 2, 4, 3, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                ",": [5, 3, 5, 4, 5, 5, 6, 4, 6, 5, 7, 3, 7, 4],
                ":": [1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5],
                "%": [0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 3, 3, 3, 4, 4, 2, 4, 3, 5, 1, 5, 2, 5, 4, 5, 5, 6, 0, 6, 1, 6, 4, 6, 5],
                "!": [0, 3, 0, 4, 0, 5, 1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5, 7, 4],
                "(": [0, 2, 0, 3, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 7, 2, 7, 3],
                ")": [0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7, 7, 5, 7, 6],
                ".": [5, 3, 5, 4, 5, 5, 6, 3, 6, 4, 6, 5, 7, 3, 7, 4, 7, 5]
            },
              
            frame: {
                  
                backgroundImageUrl: null,
                  
                innerWidth: 6,
                  
                outerWidth: 10,
            },
              
            height: 150,
              
            width: 400,
              
            enableResize: false,
              
            isResponsive: false,
              
            themes: "flatlight",
              
            items: null,
              
            init: null,
              
            load: null,
              
            renderComplete: null,
              
            itemRendering: null,
              
            value: "text",
			themeProperties:{
			 flatlight: {
            items: {
                segmentSettings: {
                    color: "#232323",
                },
                shadowColor: "#232323",
                textColor: "#232323"
            }
        },
          
        flatdark: {
            items: {
                segmentSettings: {
                    color: "#b1b0b0",
                },
                shadowColor: "#b1b0b0",
                textColor: "#b1b0b0"
            }
        }
			}
        },

          
        dataTypes: {
            segmentData: "data",
            matrixSegmentData: "data",
            items: "array",
            isResponsive: "boolean",
        },

          
        _setValues: function () {
            this.gaugeEl = this.element;
            this.segmentCount = null;
            this.contextEl = null;
            this.style = null;
            this._value = null;
            this.region = null;
            this.canvasEl = null;
            this.segement16X = null;
            this.segment16Y = null;
            this.segmentHeight = null;
            this.segmentAngle = null;
            this.startX = 5;
            this.startY = 5;
            this.gradient = null;
            this.itemIndex = null;
            this.characterSpace = null;
            this.outerImage = null;
            this.radius = null;
            this.frameOuterLocation = null;
            this.frameInnerLocation = null;
            this.glassFrameLocation = null;
            this.glassFrameStyle = null;
            this.frameOuterStyle = null;
            this.character = null;
            this.frameInnerStyle = null;
            this._itemInitialize();
        },
        observables: ["value"],
        _tags: [{
            tag: "items",
            attr: ["characterSettings.count", "characterSettings.opacity", "characterSettings.spacing", "characterSettings.type", "enableCustomFont", "segmentSettings.color", "segmentSettings.gradient", "segmentSettings.length", "segmentSettings.opacity", "segmentSettings.spacing", "segmentSettings.width", "shadowBlur", "shadowOffsetX", "shadowOffsetY", "textAlign", "shadowColor", "textColor", "font.size", "font.fontFamily", "font.fontStyle", "position.x", "position.y"]
        }],
        value: ej.util.valueFunction("value"),


          
        _destroy: function () {
            this.element.empty().removeClass("e-widget");
        },
          
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this.model.height = options[option];
                        break;
                    case "width":
                        this.model.width = options[option];
                        break;
                    case "items":
                        this.model.items = options[option];
                        this._itemInitialize();
                        break;
                    case "frame":
                        $.extend(this.model.frame, options[option]); break;
                    case "themes": this.model.themes = options[option]; break;
                    case "value":
                        for (var i = 0; this.model.items[i] != null; i++)
                            this.model.items[i].value = this.value();
                        break;
                }
            }
            this.refresh();
        },
          
        _init: function () {
            _digitalGaugeCount = $(".e-digitalgauge").length;
            initialGaugeCount = _digitalGaugeCount;
            this._setValues();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            this._onWindowResize();
        },
          
        _onWindowResize: function () {
            if (this.model.enableResize || this.model.isResponsive) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this.resizeCanvas);
                else
                    this._on($(window), "orientationchange", this.resizeCanvas);
            }
        },
          
        _setTheme: function () {
            var selectedThem = this.model.themeProperties[this.model.themes];
            this._setThemeColors(selectedThem);
        },
          
        _setThemeColors: function (selectedThem) {
            var result = [], jsonObj = this.model.themeProperties;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                for (var i = 0; i < this.model.items.length; i++) {
                    this.model.items[i].segmentSettings.color = (ej.isNullOrUndefined(this.model.items[i].segmentSettings.color) || this.model.items[i].segmentSettings.color == jsonObj[result[th]].items.segmentSettings.color) ? selectedThem.items.segmentSettings.color : this.model.items[i].segmentSettings.color;
                    this.model.items[i].shadowColor = (!this.model.items[i].shadowColor || this.model.items[i].shadowColor == jsonObj[result[th]].items.shadowColor) ? selectedThem.items.shadowColor : this.model.items[i].shadowColor;
                    this.model.items[i].textColor = (!this.model.items[i].textColor || this.model.items[i].textColor == jsonObj[result[th]].items.textColor) ? selectedThem.items.textColor : this.model.items[i].textColor;
                }
            }
        },
          
        _initialize: function () {
            if (this.model.init)
                this._clientSideOnInit();
            this._initObject(this);
            if (this.model.load)
                this._clientSideOnLoad();
            if (this.model.frame.backgroundImageUrl != null)
                this._drawCustomImage(this, this.model.frame.backgroundImageUrl);
            else
                this._renderItems();
            if (this.model.renderComplete)
                this._clientSideOnRenderComplete();
        },
          
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.items != null) {
                $.each(this.model.items, function (index, element) {
                    var obj = proxy._sendDefaultItem();
                    $.extend(true, obj, element);
                    $.extend(true, element, obj);
                });
            }
            else {
                this.model.items = [this._sendDefaultItem()];
            }
        },
          
        _sendDefaultItem: function () {
            var defaultItems;
            return defaultItems = {
                  
                characterSettings: {
                      
                    count: 4,
                       
                    opacity: 1,
                      
                    spacing: 2,
                      
                    type:ej.datavisualization.DigitalGauge.CharacterType.EightCrossEightDotMatrix,
                },
                  
                enableCustomFont: false,
                  
                segmentSettings: {
                      
                    color: null,
                      
                    gradient:null,
                       
                    length: 2,
                      
                    opacity: 0,
                      
                    spacing: 1,
                      
                    width: 1,
                },
                  
                shadowBlur: 0,
                  
                shadowOffsetX: 1,
                  
                shadowOffsetY: 1,
                  
                textAlign: "left",
                  
                font:   {
                      
                    size: "11px",
                      
                    fontFamily: "Arial",
                      
                    fontStyle: "italic"
                },
                  
                position: {
                      
                    x: 0,
                      
                    y: 0
                },
                  
                shadowColor: null,
                  
                textColor: null,
                  
                value: null
            };
        },
          
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.gaugeEl = this.element;
            for (var i = 0; this.model.items[i] != null; i++) {
                if (this.model.items[i].value == null)
                    this.model.items[i].value = this.value();
            }
            if (element.canvasEl)
                element.canvasEl.parent().empty();
            element.canvasEl = $("<canvas></canvas>");
            element.gaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            if (_digitalGaugeCount == initialGaugeCount) {
                initialDigitalDivWidth = window.innerWidth;
            }
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
          
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            $(image).on('load', function () {
                element.contextEl.drawImage(this, 0, 0, element.model.width, element.model.height);
                if (element.model.Scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
          
        _setSegmentCount: function (element) {
            switch (element) {
                case "sevensegment": this._SegmentCount = 7; break;
                case "fourteensegment": this._SegmentCount = 14; break;
                case "sixteensegment": this._SegmentCount = 16; break;
                case "eightcrosseightdotmatrix": this._SegmentCount = [8, 8]; break;
                case "eightcrosseightsquarematrix": this._SegmentCount = [8, 8]; break;
                default:
                    this._SegmentCount = 7;
            }

        },
          
        _setInnerPosition: function () {
            this.contextEl.save();
            this.contextEl.translate(this.model.frame.outerWidth + this.model.frame.innerWidth, this.model.frame.outerWidth + this.model.frame.innerWidth);
            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.model.frame.outerWidth + this.model.frame.innerWidth),
                width: this.canvasEl[0].width - 2 * (this.model.frame.outerWidth + this.model.frame.innerWidth)
            };
        },
          
        _setWidth: function () {
            var characterCount = [];
            if (this.model.items != null) {
                $.each(this.model.items, function (index, element) {
                    characterCount.push(element.characterSettings.count);
                });
            }
        },

          
        _renderItems: function () {
            if (this.model.items != null) {
                this._setInnerPosition();
                var self = this;
                $.each(this.model.items, function (index, element) {
                    self._setSegmentCount(element.characterSettings.type);
                    self.itemIndex = index;
                    self.canvasEl.attr("aria-label", element.value);
                    self._setShadow(index, element);
                    if (element.enableCustomFont)
                        self._setCustomFont(index, element);
                    else if (element.characterSettings.type.indexOf("matrix") != -1)
                        self._drawMatrixSegments(index, element);
                    else
                        self._drawSegments(index, element);
                });
            }
        },

          
        _setGradientColor: function (element, gradient, options) {
            if (options.Name || typeof (options) === "string") {
                gradient.addColorStop(0, this._getColor(element, options));
                gradient.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradient.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : this._getColor(element, colorElement.color));
                });
        },

          
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.R + ", " + option.G + "," + option.B + ", " + option.A / 255 + ")");
            }
        },

          
        _drawMatrixSegments: function (index, element) {
            var segmentXCollection = [], segmentCollection = [];
            if (element.value) {
                this._value = element.value.toString().split('');
                  
                element.characterSettings.count = (this._value.length > 4)?this._value.length:4;
            }
            else
                this._value = "";
            this.radius = (element.characterSettings.type.indexOf("dot") != -1) ? (element.segmentSettings.length + element.segmentSettings.width) / 2 : element.segmentSettings.width / 2;
            var controlStartX = this.startX = (this.bounds.width - element.characterSettings.count * (this._SegmentCount[0] * (2 * this.radius) + element.characterSettings.spacing + this._SegmentCount[0] * element.segmentSettings.spacing)) * (element.position.x / 100);
            var controlStartY = this.startY = (this.bounds.height - (this._SegmentCount[1] * ((element.characterSettings.type.indexOf("dot") != -1) ? 2 * this.radius : element.segmentSettings.length) + this._SegmentCount[1] * element.segmentSettings.spacing)) * (element.position.y / 100);
            for (var character = 0; character < element.characterSettings.count; character++) {
                if (this._value) {
                    this.character = element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character];
                    segmentCollection = this.model.matrixSegmentData[this.character];
                }
                if (character != 0) {
                    controlStartX = this.startX = this.startX + element.characterSettings.spacing + element.segmentSettings.spacing + (2 * this.radius);
                    this.startY = controlStartY;
                }
                for (var dotY = 0; dotY < this._SegmentCount[1]; dotY++) {
                    if (dotY != 0) {
                        this.startY = ((element.characterSettings.type.indexOf("dot") != -1) ? (2 * this.radius) : element.segmentSettings.length) + this.startY + element.segmentSettings.spacing;
                        this.startX = controlStartX;
                    }
                    if (segmentCollection) {
                        $.each(segmentCollection, function (segIndex) {
                            if (segIndex % 2 == 0) {
                                if (segmentCollection[segIndex] > dotY)
                                    return false;
                                if (segmentCollection[segIndex] == dotY)
                                    segmentXCollection.push(parseInt(segmentCollection[segIndex + 1]));
                            }
                        });
                    }
                    for (var dotX = 0; dotX < this._SegmentCount[0]; dotX++) {
                        if (dotX != 0)
                            this.startX = this.startX + 2 * this.radius + element.segmentSettings.spacing;
                        if (element.characterSettings.type.indexOf("dot") != -1)
                            this.gradient = this.contextEl.createRadialGradient(0, 0, 0, 0, 0, this.radius);
                        else
                            this.gradient = this.contextEl.createLinearGradient(0, 0, element.segmentSettings.width, 0);
                        if (element.segmentSettings.gradient)
                            this._setGradientColor(this, this.gradient, element.segmentSettings.gradient.colorInfo);
                        this.region = { "startX": this.startX, "startY": this.startY };
                        this.style = {
                            "opacity": (segmentXCollection && ($.inArray(dotX, segmentXCollection)) != -1) ? element.characterSettings.opacity : element.segmentSettings.opacity,
                            "height": element.segmentSettings.length,
                            "width": element.segmentSettings.width,
                            "fillStyle": ((element.segmentSettings.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentSettings.color)),
                            "skewX": element.SkewAngleX,
                            "skewY": element.SkewAngleX
                        };
                        if (this.model.itemRendering)
                            this._clientSideOnItemRendering(true, dotX, dotY);
                        if (element.characterSettings.type.indexOf("dot") != -1)
                            this._drawDot(this.region, this.style);
                        else
                            this._drawSquare(this.region, this.style);
                    }
                    segmentXCollection = [];
                }
            }
        },

          
        _drawSegments: function (index, element) {
            var segmentCollection = [];
            if (element.value) {
                this._value = element.value.toUpperCase().toString().split('');
                //beyond the four character it align in center. else it align left
                element.characterSettings.count = (this._value.length > 4) ? this._value.length : 4;
            }
            this.characterSpace = element.characterSettings.type == "sevensegment" ? 2 * element.segmentSettings.width : 4 * element.segmentSettings.width;
            this._renderSegmentCalculation(element);
            this.gradient = this.contextEl.createLinearGradient(0, 0, 0, element.segmentSettings.width);
            if (element.segmentSettings.color)
                this._setGradientColor(this, this.gradient, element.segmentSettings.color);
            else if (element.segmentSettings.gradient)
                this._setGradientColor(this, this.gradient, element.segmentSettings.gradient.colorInfo);
            //else
            //    this._setGradientColor(this, this.gradient, this.model.segmentColor);
            for (var character = 0; character < element.characterSettings.count; character++) {
                if (element.value)
                    segmentCollection = this.model.segmentData[element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character]];
                for (var segment = 0; segment < this._SegmentCount; segment++) {
                    if (character != 0)
                        this.segment16X[segment] = this.segment16X[segment] + element.segmentSettings.length + this.characterSpace + element.characterSettings.spacing;
                    if (this._value)
                        this.character = element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character];
                    this.region = { "startX": this.segment16X[segment], "startY": this.segment16Y[segment] };
                    this.style = {
                        "angle": this.angle[segment],
                        "fillStyle": this.gradient,
                        "isStroke": false,
                        "isFill": true,
                        "characterHeight": element.characterSettings.type == "sevensegment" ? element.segmentSettings.length : this.segmentHeight[segment],
                        "segmentWidth": element.segmentSettings.width,
                        "opacity": (segmentCollection && ($.inArray(segment, segmentCollection) != -1)) ? element.characterSettings.opacity : element.segmentSettings.opacity
                    };
                    if (this.model.itemRendering)
                        this._clientSideOnItemRendering(false, segment);
                    this._drawSegmentLayers(this.region, this.style);
                }
                if (this._notifyArrayChange)
                    this._notifyArrayChange("items[" + index + "]value", element.value);
                this.value(element.value);
                segmentCollection = [];
            }
        },

        
        _setCustomFont: function (index, element) {
            this.startX = (this.bounds.width - this._measureText(element.value, 0, this._getFontString(this, element.font)).width) * (element.position.x / 100);
            this.startY = (this.bounds.height - this._measureText(element.value, 0, this._getFontString(this, element.font)).height) * (element.position.y / 100);
            this.region = { "startX": this.startX, "startY": this.startY };
            this.style = { "font": this._getFontString(this, element.font), "text": element.value, "textColor": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.textColor)) : ((element.segmentSettings.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentSettings.color)) };
            if (this.model.itemRendering)
                this._clientSideOnItemRendering(false, segment);
            this._drawText(this.region, this.style);
        },

          
        _getFontString: function (element, font) {
            return ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily + " " + (font.fontStyle ? font.fontStyle : "");
        },

          
        _renderSegmentCalculation: function (element) {
            var length = element.segmentSettings.length, width = element.segmentSettings.width;
            this.startX = (this.bounds.width - element.characterSettings.count * (length + this.characterSpace + element.characterSettings.spacing)) * (element.position.x / 100);
            this.startY = (this.bounds.height - 2 * length - width) * (element.position.y / 100);
            var tempLength = element.characterSettings.type == "sevensegment" ? length : length / 2;
            this.segment16X = [
                  this.startX + width / 2,
                  this.startX + length + 4 * width,
                  this.startX + length + 4 * width,
                  this.startX + width / 2,
                  this.startX,
                  this.startX,
                  this.startX + width / 2,
                  this.startX + tempLength + 2 * width,
                  this.startX + 2.5 * width + tempLength,
                  this.startX + tempLength + 2 * width,
                  this.startX + length + 2.5 * width,
                  this.startX + tempLength + 2.5 * width,
                  this.startX + tempLength + 1.5 * width,
                  this.startX + 1.5 * width,
                  this.startX + 5 * width / 2 + tempLength,
                  this.startX + 2.5 * width + tempLength
            ];
            this.segment16Y = [
                  this.startY,
                  this.startY,
                  this.startY + length + width,
                  this.startY + 2 * length + 2 * width,
                  this.startY + length + width,
                  this.startY,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY,
                  this.startY + width,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY + width,
                  this.startY,
                  this.startY + 2 * length + 2 * width
            ];
            this.segmentHeight = [
                  length / 2,
                  length,
                  length,
                  length / 2,
                  length,
                  length,
                  length / 2,
                  length,
                  length / 2,
                  length,
                  length,
                  length,
                  length,
                  length,
                  length / 2,
                  length / 2
            ];
            this.angle = [-90, 0, 0, -90, 0, 0, -90, 0, -90, 0, 27, -27, 27, -27, -90, -90];
            if (element.characterSettings.type == "sevensegment")
                this.segment16X[2] = this.segment16X[1] = this.startX + length + 2 * width;
            if (element.characterSettings.type == "fourteensegment")
                this.segmentHeight[3] = this.segmentHeight[0] = length + 2 * width;
        },

          
        _drawSegmentLayers: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * (style.angle / 180));
            this.contextEl.lineTo(0, 0);
            this.contextEl.lineTo(-style.segmentWidth, style.segmentWidth);
            this.contextEl.lineTo(-style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(0, style.characterHeight + style.segmentWidth);
            this.contextEl.lineTo(style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(style.segmentWidth, style.segmentWidth);
            this._contextClosePath(style, this);
        },

          
        _drawDot: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.arc(0, 0, this.radius, 0, 2 * Math.PI, true);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

          
        _setShadow: function (index, element) {
            this.contextEl.save();
            this.contextEl.shadowColor = ((element.shadowColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.shadowColor));
            this.contextEl.shadowOffsetY = element.shadowOffsetY;
            this.contextEl.shadowOffsetX = element.shadowOffsetX;
            this.contextEl.shadowBlur = element.shadowBlur;
        },

          
        _drawSquare: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
              
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.rect(0, 0, style.width, style.height);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

          

        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.font = style.font;
            this.contextEl.textBaseline = "hanging";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.fillText(style.text, location.startX, location.startY);
            this.contextEl.closePath();
            this.contextEl.restore();
        },

          

        setValue: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].value = value;
                this._initialize();
            }
        },

          
        getValue: function (itemIndex) {
            return this.model.items[itemIndex].value;
        },

          

        setPosition: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].position.x = value.x;
                this.model.items[itemIndex].position.y = value.y;
                this._initialize();
            }
        },


          

        getPosition: function (itemIndex) {
            if (itemIndex < this.model.items.length)
                return { "x": this.model.items[itemIndex].position.x, "y": this.model.items[itemIndex].position.y };
            else
                return null;
        },

          
        refresh: function () {
            this._setTheme();
            this._initialize();
        },

          
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
                return false;
            }
            else {
                var image = this.canvasEl[0].toDataURL();
                image = image.replace(/^data:[a-z]*;,/, '');
                var image1 = image.split(',');
                var byteString = atob(image1[1]);
                var buffer = new ArrayBuffer(byteString.length);
                var intArray = new Uint8Array(buffer);
                for (var i = 0; i < byteString.length; i++) {
                    intArray[i] = byteString.charCodeAt(i);
                }
                var blob = new Blob([buffer], { type: "image/png" });
                if (ej.browserInfo().name === "msie")
                    window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
                else {
                    var pom = document.createElement('a');
                    var url = URL.createObjectURL(blob);
                    pom.href = url;
                    pom.setAttribute('download', fileName + '.' + fileType);
                    if (document.createEvent) {
                        var e = document.createEvent("MouseEvents");
                        e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                        pom.dispatchEvent(e);
                    }
                    else if (pom.fireEvent) {
                        pom.fireEvent("onclick");
                    }
                }
                return true;
            }
        },

      
        resizeCanvas: function () {
            if (_digitalGaugeCount != 0)
                _digitalGaugeCount = _digitalGaugeCount - 1;
            else
                _digitalGaugeCount = $(".e-digitalgauge").length - 1; var state, chk = true;
            if (!ej.isNullOrUndefined(this.canvasEl.parent().attr("style")))
                state = this.GaugeEl.parent().attr("style").split(";");
            if (!ej.isNullOrUndefined(state)) {
                $.each(state, function (key, val) {
                    while (val.indexOf("width") != -1) {
                        chk = val.indexOf("px") == -1 ? true : false;
                        break;
                    }
                });
            }
            if (chk) {
                var ratio = window.innerWidth / initialDigitalDivWidth;
                this.model.width *= ratio;
                for (var i = 0; this.model.items[i] != null; i++) {
                    this.model.items[i].segmentSettings.width *= ratio;
                    this.model.items[i].segmentSettings.length *= ratio;
                    this.model.items[i].segmentSettings.spacing *= ratio;
                    this.model.items[i].characterSettings.spacing *= ratio;
                }
                this.refresh();
                if (_digitalGaugeCount == 0) {
                    initialDigitalDivWidth = window.innerWidth;
                }
            }
        },

          
          
        _clientSideOnLoad: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("load", data);
        },

          
        _clientSideOnItemRendering: function (isMatrix, row, column) {
            var data;
            if (isMatrix)
                data = { object: this, items: this.model.items, character: this.character, context: this.contextEl, position: this.region, style: this.style, row: row, column: column };
            else
                data = { object: this, model: this.model, id: this.model.ClientId, items: this.model.items, character: this.character, context: this.contextEl, position: this.region, style: this.style, segment: row };
            this._trigger("itemRendering", data);
        },

          
        _clientSideOnInit: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("init", data);
        },

          
        _clientSideOnRenderComplete: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("renderComplete", data);
        },


          
          
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },

          
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },

          
        _measureText: function (text, maxwidth, font) {
            var textObj = document.createElement('DIV');
            textObj.innerHTML = text;
            if (font != null)
                textObj.style.font = font;
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100;
            textObj.style.left = 0;
            if (maxwidth)
                textObj.style.maxwidth = maxwidth + "px";
            document.body.appendChild(textObj);
            var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            $(textObj).remove();
            return bounds;
        }

    });

      
    ej.datavisualization.DigitalGauge.CharacterType = {
          
        SevenSegment: "sevensegment",
          
        FourteenSegment: "fourteensegment",
          
        SixteenSegment: "sixteensegment",
          
        EightCrossEightDotMatrix: "eightcrosseightdotmatrix",
          
        EightCrossEightSquareMatrix: "eightcrosseightsquarematrix"
    };
      
    ej.datavisualization.DigitalGauge.TextAlign = {
          
        Left: "left",
          
        Right: "right"
    };

      
    ej.datavisualization.DigitalGauge.FontStyle = {
          
        Normal: "normal",
          
        Bold: "bold",
          
        Italic: "italic",
          
        Underline: "underline",
          
        Strikeout: "strikeout"
    };

      
    ej.datavisualization.DigitalGauge.Themes = {
         FlatLight: 'flatlight',

		FlatDark: 'flatdark'
    };		
       
  

})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.s
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
  
    ej.widget("ejBulletGraph", "ej.datavisualization.BulletGraph",    {
    
        element: null,

        model: null,
        validTags: ["div"],
    
        _tags: [{
            tag: 'qualitativeRanges',
            attr: ['rangeEnd', 'rangeStroke', 'rangeOpacity']
        },
        {
            tag: 'quantitativeScaleSettings.featureMeasures',
            attr: ['value', 'comparativeMeasureValue']
        }],
    
        defaults:   {
              
            locale: null,
			
			enableGroupSeparator: false,
			
			value: 0,
              
            comparativeMeasureValue: 0,
              
            width: null,
              
            height: null,
              
            theme: "flatlight",
              
            orientation: "horizontal",
              
            flowDirection: "forward",
              
            qualitativeRangeSize: 32,
              
            quantitativeScaleLength: 475,
              
            tooltipSettings:   {
                  
                  
                enableCaptionTooltip: false,
                  
                captionTemplate: null,
                  
                visible: true,
                  
                template: null
            },
              
            quantitativeScaleSettings:    {
                  
                location: {
                      
                    x: 10,
                      
                    y: 10
                },
                  
                minimum: 0,
                  
                maximum: 10,
                  
                interval: 1,
                  
                minorTicksPerInterval: 4,
                  
                majorTickSettings: {
                      
                    size: 13,
                      
                    stroke: null,
                      
                    width: 2
                },
                  
                minorTickSettings: {
                      
					size: 7,
                 
                    stroke: null,
                      
                    width: 2
                },
                  
                tickPosition: "far",
                  
                tickPlacement: "outside",
                  
                labelSettings: {
                      
                    labelPlacement: "outside",
                      
                    labelPrefix: "",
                      
                    labelSuffix: "",
                      
                    stroke: null,
                      
                    size: 12,
                      
                    offset: 15,
                      
                    font: {
                          
                        fontFamily: 'Segoe UI',
                          
                        fontStyle: 'Normal ',
                          
                        fontWeight: 'regular',
                          
                        opacity: 1
                    },
                      
                    position: "below"
                },
                  
                featuredMeasureSettings: {
                      

                    stroke: null,
                      
                    width: 6
                },
                  
                comparativeMeasureSettings: {
                      
                    stroke: null,
                      
                    width: 5
                },
                  
                featureMeasures: [{
                      
                    value: null,
                      
                    comparativeMeasureValue: null,
                      
                    category: null
                }]
            },
              
            fields: {
                  
                dataSource: null,
                  
                query: null,
                  
                tableName: null,
                  
                category: null,
                  
                featureMeasures: null,
                  
                comparativeMeasure: null
            },
              
            enableAnimation: true,
              
            enableResizing: true,
              
            isResponsive: true,
              
            applyRangeStrokeToTicks: false,
              
            applyRangeStrokeToLabels: false,
              
            qualitativeRanges: [{
                  
                rangeEnd: 4.3,
                  
                rangeStroke: null,
                  
                rangeOpacity: 1
            }, {
                  
                rangeEnd: 7.3,
                  
                rangeStroke: null,
                  
                rangeOpacity: 1
            }, {
                  
                rangeEnd: 10,
                  
                rangeStroke: null,
                  
                rangeOpacity: 1
            }],
              
            captionSettings: {
                  
                enableTrim: true,
                  
                textPosition: 'float',
                  
                textAlignment: 'Near',
                  
                textAnchor: 'start',
                  
                padding: 5,
                  
                textAngle: 0,
                  
                location: {
                      
                    x: 17,
                      
                    y: 30
                },
                  
                text: "",
                  
                font: {
                      
                    color: null,
                      
                    fontFamily: 'Segoe UI',
                      
                    fontStyle: 'Normal',
                      
                    size: '12px',
                      
                    fontWeight: 'regular',
                      
                    opacity: 1
                },
                  
                subTitle: {
                      
               textPosition: 'float',
                 
               textAlignment: 'Near',
                 
               textAnchor: 'start',
                 
                    padding: 5,
                      
                    textAngle: 0,
                      
                    text: "",
                      
                    location: {
                          
                        x: 10,
                          
                        y: 45
                    },
                      
                    font: {
                          
                        color: null,                          
                        
                        fontFamily: 'Segoe UI',
                          
                        
                        fontStyle: 'Normal ',                          
                        
                        size: '12px',
                          
                        fontWeight: 'regular',
                          
                        opacity: 1
                    }
                },

                  
                indicator: {
                      
                    textPosition: 'float',
                      
                    textAlignment: 'Near',
                      
                    textAnchor: 'start',
                      
                    padding:5,
                      
                    visible: false,
                      
                    textAngle: 0,
                      
                    textSpacing: 3,
                      
                    text: "",                    
                      
                    symbol: {
                          
                        border: {
                              
                            color: null,
                              
                            width: 1,
                        },
                          
                        color: null,
                          
                        shape: "",
                          
                        imageURL: "",
                          
                        size: {
                              
                            width: 10,
                              
                            height: 10
                        },
                          
                        opacity: 1
                    },
                      
                    location: {
                          
                        x: 10,
                          
                        y: 60
                    },
                      
                    font: {
                          
                        color: null,
                          

                        fontFamily: 'Segoe UI',
                          

                        fontStyle: 'Normal ',
                          

                        size: '12px',
                          
                        fontWeight: 'regular',
                          
                        opacity: 1
                    }
                }
            },

              
              
            load: '',
              
            drawTicks: null,
              
            drawLabels: null,
              
            drawCaption: null,
              
            drawIndicator: null,
              
            drawQualitativeRanges: null,
              
            drawFeatureMeasureBar: null,
              
            drawCategory: null,
              
            drawComparativeMeasureSymbol: null
        },

        observables: ["value", "comparativeMeasureValue"],
        value: ej.util.valueFunction("value"),
        comparativeMeasureValue: ej.util.valueFunction("comparativeMeasureValue"),
          
        dataTypes: {
            quantitativeScaleSettings: {
                labelSettings: "data",
                featureMeasures: "array"
            },
            fields: { dataSource: "data", query: "data" },
            qualitativeRanges: "array",
            captionSettings: "data",
            isResponsive: "boolean",
        },

        // constructor function
          
        _init: function () {
            this._renderBulletGraph();
        },
          
        _isSVG: function () {
            if (window.SVGSVGElement)
                return true;
            else
                return false;
        },
          
        _value: ej.util.valueFunction("value"),
        _comparativeMeasureValue: ej.util.valueFunction("comparativeMeasureValue"),
          
        _qualitativeRanges: function (index, property, value, old) {
            this.redraw();
            this._trigger('refresh');
        },
          
        _quantitativeScaleSettings_featureMeasures: function (index, property, value, old) {
            this.redraw();
            this._trigger('refresh');
        },
          
        _renderBulletGraph: function () {
            if (this._isSVG()) {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.svgObject = this.svgRenderer.svgObj;
                this._trigger("load");
                this._setSvgSize(this);
                this._setTheme(ej.datavisualization.BulletGraph.Themes, this.model.theme);
                this.bindEvents();
                this._renderBulletElements();               
                if (this.model.enableAnimation)
                    this._animateMeasures();
            }
        },
          
        _animateMeasures: function () {
            this._doAnimation();
            this._doLineAnimation();
        },

        _setSvgSize: function (sender) {
            var chartObj = sender;
            var containerHeight = $(chartObj.element).height();
            var height = 90; //set default height if size not specified to chart or container
            var width = (ej.isTouchDevice()) ? 250 : 595;
            var containerWidth = $(chartObj.element).width();
            if (chartObj.model.width)
                width = parseInt(chartObj.model.width);
            else if (containerWidth > 0 && containerWidth < 595)
                width = containerWidth;
            else if(containerWidth > 595)
                width = 595;

            $(chartObj.svgObject).width(width);

            if (chartObj.model.height)
                height = parseInt(chartObj.model.height);
            else if (containerHeight > 0)
                height = containerHeight;
            else
                $(chartObj.svgObject).css("display", "block");
            $(chartObj.svgObject).height(height);

            chartObj.svgObject.width = width;   
            chartObj.svgObject.height = height;
        },
        _renderBulletElements: function () {
            var width = this.model.width ? this.model.width : 595;
            var height = this.model.height ? this.model.height : 90;
            this.svgObject.setAttribute('viewBox', "0 0 " + width + " " + height);
            this.svgObject.setAttribute('preserveAspectRatio', "xMinYMin");
            this.svgWidth = $(this.svgObject).width();
            this.svgHeight = $(this.svgObject).height();
            var captionGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_captionGroup' });
            var scaleGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_scaleGroup' });
            var bulletGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_outerWrap' });

            this._initializeValues();
            var intervalValue = ((this.model.quantitativeScaleLength) / ((this._scale.maximum - this._scale.minimum) / this._scale.interval));

            //render captionSettings
            this._drawCaption();

            //render indicator
            this._drawIndicator();

            //render caption group
            this._drawCaptionGroup(captionGroup);

            //render scale
            this._scaleGroup = scaleGroup;
            this._drawScale(captionGroup, scaleGroup);

            //draw ranges
            this._drawQualitativeRanges(scaleGroup);

            //render ticks
            this._drawMajorTicks(intervalValue, scaleGroup);
            this._drawMinorTicks(intervalValue, scaleGroup);

            //render labels
            this._drawLabels(intervalValue, scaleGroup);

            // data binding
            this._bindData();
                        
            $(scaleGroup).appendTo(bulletGroup);
            $(captionGroup).appendTo(bulletGroup);

            this._changeOrientation(scaleGroup);

            $(bulletGroup).appendTo(this.svgObject);
            $(this.svgObject).appendTo(this.element);

            this._bindHighlightRemoving(); // binding highlight style removing function for feature measure bars
            
        },

        _bindHighlightRemoving:function(){
            var isPointer = this.model.browserInfo.isMSPointerEnabled,
            isIE11Pointer = this.model.browserInfo.pointerEnabled;
            touchStartOut = isPointer ? (isIE11Pointer ? "pointerout" : "MSPointerOut") : "touchout mouseout",
            this._on($("[id*=" + this.svgObject.id + "_FeatureMeasure_" + "]"), touchStartOut, function (evt) {
                if (!this.isTouch(evt))
                    $(evt.target).attr("opacity", 1);
            });
        },
      
        _drawCaptionGroup: function (captionGroup) {
            var caption = this.model.captionSettings;
            var subTitle = caption.subTitle;
            var indicator = (this._indicator ? this._indicator.settings : caption.indicator);
            var location;

            caption._location = subTitle._location = indicator._location = location;

            var smartCaptionPlacement = (caption.textPosition.toLowerCase()!='float' && caption.text!="");
            var smartSubTitlePlacement = (subTitle.textPosition.toLowerCase() != 'float' && subTitle.text != "");
            var smartIndicatorPlacement = indicator.visible && (indicator.textPosition.toLowerCase() != 'float');

            if (smartCaptionPlacement || smartSubTitlePlacement || smartIndicatorPlacement) {                

                var captionSize = ej.EjSvgRender.utils._measureText(caption.text, null, caption.font);
                var subTitleSize = ej.EjSvgRender.utils._measureText(subTitle.text, null, subTitle.font);
                var indicatorTextSize;
                var indicatorRegion;
                if (indicator.visible) {                    
                    indicatorTextSize = this._indicator.bounds;
                    var indicatorSize = {
                        width: indicator.symbol.size.width + indicatorTextSize.width + indicator.textSpacing,
                        height: Math.max(indicator.symbol.size.height, indicatorTextSize.height)
                    }
                    indicatorRegion = {
                        x: 0,
                        y: 0,
                        width: indicatorSize.width,
                        height: indicatorSize.height,
                        padding: indicator.padding,
                        anchor: indicator.textAnchor.toLowerCase(),
                        alignment: indicator.textAlignment.toLowerCase()
                    };
                }

                var captionRegion = { 
                    x: 0, 
                    y: 0, 
                    width: captionSize.width,                    
                    height: parseFloat(caption.font.size),
                    padding: caption.padding,
                    anchor: caption.textAnchor.toLowerCase(),
                    alignment: caption.textAlignment.toLowerCase()
                };
                var subTitleRegion = {
                    x: 0,
                    y: 0,
                    width: subTitleSize.width,
                    height: parseFloat(subTitle.font.size),
                    padding: subTitle.padding,
                    anchor: subTitle.textAnchor.toLowerCase(),
                    alignment: subTitle.textAlignment.toLowerCase()
                };
                
                var scaleRegion = { x: this._scale.location.x, y: this._scale.location.y, width: this.model.quantitativeScaleLength, height: this.model.qualitativeRangeSize };
                var elements= [ smartCaptionPlacement ?caption : null, smartSubTitlePlacement ? subTitle : null, smartIndicatorPlacement ? indicator : null];

                this._positionTextGroup(elements, [captionRegion, subTitleRegion, indicatorRegion], scaleRegion);
                this._locateTextGroup(elements, [captionRegion, subTitleRegion, indicatorRegion]);
            }
            var captionLocation = caption._location == undefined ? caption.location : caption._location;
            var subTitleLocation = subTitle._location == undefined ? subTitle.location : subTitle._location;
            var indicatorLocation = indicator._location == undefined ? indicator.location : indicator._location;

            caption.displayText = caption.text;
            if (caption.enableTrim == true) {
                captionLocation.x = captionLocation.x < 0 ? 0 : captionLocation.x;
                captionLocation.y = captionLocation.y < 0 ? 0 : captionLocation.y;
                caption.displayText = this._displayText(caption, captionLocation);
            }
            if (subTitle.text != "") {
                subTitle.displayText = subTitle.text;
                if (caption.enableTrim == true) {
                    subTitleLocation.x = subTitleLocation.x < 0 ? 0 : subTitleLocation.x;
                    subTitleLocation.y = subTitleLocation.y < 0 ? 0 : subTitleLocation.y;
                    subTitle.displayText = this._displayText(subTitle, subTitleLocation);
                }
            }
            if (indicator.visible) {
                indicator.displayText = indicator.text;
                if (this.model.captionSettings.enableTrim == true) {
                    indicatorLocation.x = indicatorLocation.x < 0 ? 0 : indicatorLocation.x;
                    indicatorLocation.y = indicatorLocation.y < 0 ? 0 : indicatorLocation.y;
                }
                var location = (indicator._location ? indicator._location : indicator.location);
                var symbolRect = {
                    'x': location.x + indicator.symbol.size.width / 2,
                    'y': location.y - indicator.symbol.size.height / 2,
                    'width': indicator.symbol.size.width,
                    'height': indicator.symbol.size.height,
                };
                var textOptions = this._indicatorTextOptions(indicator, symbolRect, this._indicator.bounds, this.svgObject.id + '_Indicator');
                var symbolOptions = this._indicatorSymbolOptions(captionGroup, indicator);
                this._drawBulletSymbol(this._indicator.settings.symbol.shape, symbolRect, symbolOptions, captionGroup);
                if (this.model.captionSettings.enableTrim == true)
                    indicator.displayText = this._displayText(indicator, symbolRect);
            }
            caption.displayText = caption.textPosition.toLowerCase() != "float" ? this._captionOverlap(this.model.captionSettings) : caption.displayText;
            this.svgRenderer.drawText(this._textOptions(caption, this.svgObject.id + '_Caption'), caption.displayText, captionGroup);
            if (subTitle.text != ""){
                subTitle.displayText = subTitle.textPosition.toLowerCase() != "float" ? this._subOverlap(this.model.captionSettings.subTitle) : subTitle.displayText;
                this.svgRenderer.drawText(this._textOptions(subTitle, this.svgObject.id + '_SubTitle'), subTitle.displayText, captionGroup);
            }   
            if (indicator.visible) {
                indicator.displayText = indicator.textPosition.toLowerCase() != "float" ? this._indOverlap(indicator,symbolRect) : indicator.displayText;
                this.svgRenderer.drawText(textOptions, indicator.displayText, captionGroup);
            }
        },
        // text overlap method is perform herer for caption , subtitle and indicator when position is except float
       _captionOverlap: function (settings) {
                var gap;
                if ((settings.textPosition == settings.subTitle.textPosition) && (settings.textAlignment == settings.subTitle.textAlignment) && settings.textAngle > 0 && settings.textAngle < 120 ) {
                    gap = settings.subTitle._location.y - settings._location.y;
                }
                else if ((settings.textPosition == settings.indicator.textPosition) && (settings.textAlignment == settings.indicator.textAlignment) && settings.textAngle > 0 && settings.textAngle < 120 ){
                    gap = settings.indicator._location.y - settings._location.y;
                }
                else{
                   if ((settings.textPosition == settings.subTitle.textPosition) && (settings._location.y < settings.subTitle._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = settings.subTitle._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == settings.subTitle.textPosition) && (settings._location.y > settings.subTitle._location.y) && settings.textAngle > 190 && settings.textAngle < 360     ){
                        gap = settings._location.y - settings.subTitle._location.y;
                   }
                   else if ((settings.textPosition == settings.indicator.textPosition) && (settings._location.y < settings.indicator._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = settings.indicator._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == settings.indicator.textPosition) && (settings._location.y > settings.indicator._location.y) && settings.textAngle > 190 && settings.textAngle < 360    ){
                        gap = settings._location.y - settings.indicator._location.y;
                   }
                }
                 var text = this._trim(settings.displayText,settings,gap,isRotate = true);           
            return text;             
        },
        _indOverlap: function (settings,loc) {
            var gap;
            var subTitle = this.model.captionSettings.subTitle;
            var title = this.model.captionSettings;
             if ((settings.textPosition == subTitle.textPosition) && (settings.textAlignment == subTitle.textAlignment) && settings.textAngle > 190 && settings.textAngle < 360 ) {
                    gap = loc.y - subTitle._location.y;
                }
                else if ((settings.textPosition == title.textPosition) && (settings.textAlignment ==title.textAlignment) && settings.textAngle > 190 && settings.textAngle < 360 ){
                    gap = loc.y - title._location.y;
                }
                else{
                   if ((settings.textPosition == subTitle.textPosition) && (settings._location.y < subTitle._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = subTitle._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == subTitle.textPosition) && (settings._location.y > subTitle._location.y) && settings.textAngle > 190 && settings.textAngle < 360     ){
                        gap = settings._location.y - subTitle._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y < title._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = title._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y > title._location.y) && settings.textAngle > 190 && settings.textAngle < 360    ){
                        gap = settings._location.y - title._location.y;
                   }
                }
                 var text = this._trim(settings.displayText,settings,gap,isRotate = true);           
            return text;             
        },
        _subOverlap: function (settings) {
            var gap;
            var indicator = this.model.captionSettings.indicator;
            var title = this.model.captionSettings;
             if ((settings.textPosition == indicator.textPosition) && (settings.textAlignment == indicator.textAlignment) && settings.textAngle > 0 && settings.textAngle < 180 ) {
                    gap = indicator._location.y - settings._location.y;
                }
                else if ((settings.textPosition == title.textPosition) && (settings.textAlignment ==title.textAlignment) && settings.textAngle > 190 && settings.textAngle < 360 ){
                    gap = settings._location.y - title._location.y;
                }
                else{
                   if ((settings.textPosition == indicator.textPosition) && (settings._location.y < indicator._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = indicator._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == indicator.textPosition) && (settings._location.y > indicator._location.y) && settings.textAngle > 190 && settings.textAngle < 360     ){
                        gap = settings._location.y - indicator._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y < title._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = title._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y > title._location.y) && settings.textAngle > 190 && settings.textAngle < 360    ){
                        gap = settings._location.y - title._location.y;
                   }
                }
                 var text = this._trim(settings.displayText,settings,gap,isRotate = true);           
            return text;             
        },
        // rotated label width calculated here
        rotatedLabel: function (font, sender, value, rotatedLabel) {
            var rotatedOptions = {
                'font-size': font.size,
                'transform': 'rotate(' + value + ',0,0)',
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'rotateAngle': 'rotate(' + value + 'deg)',
                'text-anchor': 'middle'
            };
            var text = sender.svgRenderer.createText(rotatedOptions, rotatedLabel);
            var width = Math.ceil((ej.EjSvgRender.utils._measureBounds(text, sender).width));
            return width;
        },
        // Trim size calculated here
        calcGap: function (angle, location, maxWidth, maxHeight, isRotate, Width, Height) {
            var trimSize, space, rotate;
            if (angle >= 0 && angle <= 90) {

                if (((maxHeight + location.y) >= Height)&& angle > 0) {
                    space = Height - location.y;
                    isRotate = true;
                }
                else if (((maxHeight + location.y) <= Height) && ((maxWidth + location.x) >= Width) && angle > 0) {
                    space = Width - location.x;
                }
                else {
                    if (((maxHeight + location.y) <= Height) && (angle > 0)) {
                        space = Height - location.y;
                        isRotate = true;
                    }
                    else
                        space = Width - location.x;
                }
            }
            else if (angle > 90 && angle < 180) {

                if (((maxHeight + location.y) >= Height)) {
                    space = Height - location.y;
                    isRotate = true;
                }
                else if (((maxHeight + location.y) <= Height) && ((maxWidth + location.x) >= Width)) {
                    space = location.x;
                }
                else {
                    if ((maxWidth + location.x) <= Width) {
                        space = location.x;
                    }
                    else {
                        space = Height - location.y;
                        isRotate = true;
                    }

                }
            }
            else if (angle >= 180 && angle < 270) {

                if ((location.x - maxWidth) >= 0) {
                    space = location.y;
                    isRotate = true;
                }
                else if (((location.y - maxHeight) <= 0) && ((location.x - maxWidth) <= 0)) {
                    space = location.x;
                }
                else {
                    if ((location.y - maxHeight) <= 0) {
                        space = Height - location.y;
                        isRotate = true;
                    }
                    else {
                        space = location.x;
                    }
                }

            }
            else {
                if ((maxWidth + location.x) <= Width) {
                    space = location.y;
                    isRotate = true;
                }
                else if (((location.y - maxHeight) >= 0) && ((maxWidth + location.x) >= Width)) {
                    space = Width - location.x;
                }
                else {
                    if ((maxWidth + location.x) >= Width) {
                        space = Width - location.x;
                    }
                    else {
                        space = location.y;
                        isRotate = true;
                    }
                }
            }

            return { trimSize: space, rotate: isRotate };
        },
        // Scale Location calculated here
        _scaleLoc: function () {
            var scale = this._scale;
            var offset = scale.labelSettings.offset;
            var tickSize = scale.majorTickSettings.size;
            var pointY, pointHeight;
            var label = this._scale.labelSettings;
            var labelBounds = ej.EjSvgRender.utils._measureText(label.labelPrefix + scale.maximum + label.labelSuffix, null, label);
            var labelSize = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? labelBounds.height : labelBounds.width;            
            pointY = scale.location.y;
            var qualitativeRangeSize = this.model.qualitativeRangeSize;
            var placement = this._tickPosition + this._tickPlacement + this._labelPosition + this._labelPlacement;
            if (this._tickPosition == "center")
                tickSize = (tickSize > qualitativeRangeSize) && ((tickSize - qualitativeRangeSize) / 2);
            switch (placement) {
                case 'faroutsidebelowoutside':
                    pointHeight = pointY + labelSize + qualitativeRangeSize + offset + ((tickSize > offset) && (tickSize - offset));
                    break;
                case 'faroutsidebelowinside':
                    pointHeight = pointY + tickSize + qualitativeRangeSize 
                    pointY = pointY - (((offset + labelSize) > qualitativeRangeSize) && ((offset + labelSize) - qualitativeRangeSize));
                    break;
                case 'faroutsideaboveoutside':
                    pointHeight = pointY + tickSize + qualitativeRangeSize;
                    pointY = pointY - (offset + labelSize)
                    break;
                case 'faroutsideaboveinside':
                    pointHeight = pointY + tickSize + qualitativeRangeSize + (((offset + labelSize) > (qualitativeRangeSize + tickSize)) ? ((offset + labelSize) - (qualitativeRangeSize + tickSize)) : 0 );
                    break;
                case 'farinsidebelowoutside':
                    pointHeight = pointY + labelSize + qualitativeRangeSize + offset;
                    pointY = pointY - ((tickSize > qualitativeRangeSize) && (tickSize - qualitativeRangeSize));
                    break;
                case 'farinsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - ((tickSize > qualitativeRangeSize) && (tickSize -qualitativeRangeSize)) - (((offset + labelSize) > (qualitativeRangeSize + tickSize)) && (offset + labelSize - tickSize)) 
                    break;
                case 'farinsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - (offset + labelSize) - ((tickSize > (offset + labelSize + qualitativeRangeSize)) && (tickSize - (offset + labelSize + qualitativeRangeSize)));
                    break;
                case 'farinsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + (((offset + labelSize) > qualitativeRangeSize) && (labelSize + offset - qualitativeRangeSize));
                    pointY = pointY - ((tickSize > qualitativeRangeSize) && (tickSize - qualitativeRangeSize));
                    break;
                case 'nearoutsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + offset + labelSize;
                    pointY = pointY - tickSize;
                    break;
                case 'nearoutsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - tickSize - (((offset + labelSize) > (qualitativeRangeSize + tickSize)) && ((offset + labelSize) - (qualitativeRangeSize + tickSize)));
                    break;
                case 'nearoutsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - tickSize - ((tickSize < (offset + labelSize)) && (offset + labelSize - tickSize));
                    break;
                case 'nearoutsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + (((offset + labelSize) > qualitativeRangeSize) && (offset + labelSize - qualitativeRangeSize));
                    pointY = pointY - tickSize;
                    break;
                case 'nearinsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (offset + labelSize + qualitativeRangeSize)) ? (tickSize - (qualitativeRangeSize)) : (offset + labelSize));
                    break;
                case 'nearinsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > qualitativeRangeSize) && (tickSize - qualitativeRangeSize));
                    pointY = pointY - (((offset + labelSize) > qualitativeRangeSize) && (offset + labelSize - qualitativeRangeSize));
                    break;
                case 'nearinsideaboveoutside':
                    pointHeight = pointY + ((tickSize > qualitativeRangeSize) ? tickSize : qualitativeRangeSize);
                    pointY = pointY - offset - labelSize;
                    break;
                case 'nearinsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (labelSize + offset)) ? (tickSize - qualitativeRangeSize) : (labelSize + offset - qualitativeRangeSize));
                    break;
                case 'centeroutsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > offset + labelSize) ? tickSize : offset + labelSize);
                    pointY = pointY - tickSize;
                    break;
                case 'centeroutsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY - tickSize - (((offset + labelSize) > (qualitativeRangeSize + tickSize)) && ((offset + labelSize) - (qualitativeRangeSize + tickSize)));
                    break;
                case 'centeroutsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY - tickSize - ((tickSize < (offset + labelSize)) && (offset + labelSize - tickSize));
                    break;
                case 'centeroutsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + (((offset + labelSize) > (qualitativeRangeSize + tickSize)) ? (offset + labelSize - qualitativeRangeSize - tickSize) : tickSize);
                    pointY = pointY - tickSize;
                    break;
                case 'centerinsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (offset + labelSize)) ? tickSize : (offset + labelSize));
                    pointY = pointY - tickSize;
                    break;
                case 'centerinsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY -  (((offset + labelSize - qualitativeRangeSize) > tickSize ) ? (offset + labelSize - qualitativeRangeSize) : tickSize);
                    break;
                case 'centerinsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY - tickSize - ((tickSize < (offset + labelSize)) && (offset + labelSize - tickSize));
                    break;
                case 'centerinsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (labelSize + offset)) ? tickSize : (labelSize + offset - qualitativeRangeSize));
                    pointY = pointY - tickSize;
                    break;
            }
            return {x: scale.location.x, y: pointY, height:pointHeight };

        },
        // display text method for trimming
        _displayText: function (settings, loc) {
             var location = $.extend(true, loc);
            var angle = settings.textAngle % 360;
            var text = settings.text;
            var font = settings.font;
            var scale = this._scale;
            var label = this._scale.labelSettings;
            var labelWidth = ej.EjSvgRender.utils._measureText(label.labelPrefix + scale.minimum + label.labelSuffix, null, label).width;
            var textWidth = ej.EjSvgRender.utils._measureText(text, null, font).width;
            var maxHeight = ej.EjSvgRender.utils.rotatedLabel(settings, this, angle, settings.text);
            var maxWidth = this.rotatedLabel(font, this, angle, settings.text);
            var gap, padding = 5;
            var height =this.svgHeight;
            var width = this.svgWidth;
            var isRotate = false;
            var scaleLocation = this._scaleLoc();
            if (settings.textSpacing) {
                textWidth = textWidth + settings.textSpacing;
                maxWidth = maxWidth + settings.textSpacing;
            }

            if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) {

               if ((location.x <= scaleLocation.x) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height)) {
                    width = (scale.location.x - labelWidth / 2);
               }
               else if ((location.x <= scaleLocation.x) && (location.y <= scaleLocation.y) && (angle < 90)) {
                    height = scaleLocation.y;
               }
               else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + this.model.quantitativeScaleLength)) && (location.y <= scaleLocation.y)) {
                    height = scaleLocation.y;
               }
               else if ((location.y <= scaleLocation.y) && (location.x >= (scaleLocation.x + this.model.quantitativeScaleLength)) && (angle >= 90)) {
                    height = scaleLocation.y;
               }

               else if ((location.x >= (scaleLocation.x + this.model.quantitativeScaleLength)) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height) && (angle < 270)) {
                    width = width - (scaleLocation.x + this.model.quantitativeScaleLength);
                    location.x = location.x - (scaleLocation.x + this.model.quantitativeScaleLength);
               }
               else if ((location.x <= scaleLocation.x) && (location.y >= scaleLocation.height) && (angle > 270)) {
                    location.y = location.y - scaleLocation.height;
               }
               else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + this.model.quantitativeScaleLength)) && (location.y >= scaleLocation.height)) {
                    location.y = location.y - scaleLocation.height;
               }
               else if ((location.x >= (scaleLocation.x + this.model.quantitativeScaleLength)) && (location.y >= scaleLocation.height) && (angle >= 180) && (angle < 270)) {
                    location.y = location.y - scaleLocation.height;
               }

               gap = this.calcGap(angle, location, maxWidth, maxHeight, isRotate, width, height);
               isRotate = gap.rotate;
               gap = gap.trimSize;
            }
            else {
                var scaleWidth = scaleLocation.height - scaleLocation.y;
                scaleLocation.y =this.svgHeight - scale.location.y - this.model.quantitativeScaleLength;
                scaleLocation.height = scaleLocation.y + this.model.quantitativeScaleLength;

                if ((location.x <= scaleLocation.x) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height)) {
                    width = (scale.location.x - labelWidth / 2);
                }
                else if ((location.x <= scaleLocation.x) && (location.y <= scaleLocation.y) && (angle < 90)) {
                    height = scaleLocation.y;
                }
                else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + scaleWidth)) && (location.y <= scaleLocation.y)) {
                    height = scaleLocation.y;
                }
                else if ((location.y <= scaleLocation.y) && (location.x >= (scaleLocation.x + scaleWidth)) && (angle >= 90)) {
                    height = scaleLocation.y;
                }

                else if ((location.x >= (scaleLocation.x + scaleWidth)) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height) && (angle <= 260)) {
                    width = width - (scaleLocation.x + scaleWidth);
                    location.x = location.x - (scaleLocation.x + scaleWidth);
                }
                else if ((location.x <= scaleLocation.x) && (location.y >= scaleLocation.height) && (angle >= 280)) {
                    location.y = location.y - scaleLocation.height;
                }
                else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + scaleWidth)) && (location.y >= scaleLocation.height)) {
                    location.y = location.y - scaleLocation.height;
                }
                else if ((location.x >= (scaleLocation.x + scaleWidth)) && (location.y >= scaleLocation.height) && (angle >= 180) && (angle <= 260)) {
                    location.y = location.y - scaleLocation.height;
                }

                gap = this.calcGap(angle, location, maxWidth, maxHeight, isRotate, width, height);
                isRotate = gap.rotate;
                gap = gap.trimSize;
            }
            // Trimming opetation called here
            var displayText = text;
            displayText = this._trim(text,settings,gap,isRotate);
            return displayText;
        },

        // trim calculation perform here
        _trim: function (text, settings, gap, isRotate) {
            var displayText = text;
            var maxWidth = this.rotatedLabel(settings.font, this, settings.textAngle, text);
            var textWidth = ej.EjSvgRender.utils._measureText(text, null, settings.font).width;
            if (settings.textSpacing) {
                textWidth = textWidth + settings.textSpacing;
                maxWidth = maxWidth + settings.textSpacing;
            }
            if (textWidth > gap && text != "") {
                var space;
                for (var t = 1; t <= text.toString().length; t++) {
                    text = displayText.toString().substring(0, t) + '... ';
                    if (isRotate == true)
                        space = ej.EjSvgRender.utils.rotatedLabel(settings, this, settings.textAngle, text);
                    else
                        space = ej.EjSvgRender.utils._measureText(text, null, settings.font).width;

                    if (settings.textSpacing) {
                        space = space + settings.textSpacing;
                    }
                    if (space >= gap) {
                        text = text.toString().substring(0, t - 1) + '... ';
                        displayText = text;
                        break;
                    }
                }
                var temp = displayText.toString(0, text.toString.lenght - 4);
                if (temp != displayText)
                    displayText = text;
            }
            return displayText;
        },
       
        _getVerticalScaleLocation: function(scaleWidth){
            return { x: this._scale.location.y, y:this.svgHeight - this._scale.location.x - scaleWidth };

        },
      
        _horizontalTextPositioning: function (element, region, scaleLocation, scaleRegion, regions) {
            var position = element.textPosition.toLowerCase();
            var alignment = region.alignment;
            var scaleLocation = this._scaleLoc();
            var label = this._scale.labelSettings;
            var maxLabelWidth = ej.EjSvgRender.utils._measureText(label.labelPrefix + this._scale.maximum + label.labelSuffix, null, label).width;            
            switch (position) {
                case 'left':
                    region.x = scaleLocation.x - region.width - region.padding - maxLabelWidth/2;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height/3: (alignment == 'far' ? scaleRegion.height : region.height));
                    regions[0].push(region);
                    break;
                case 'right':
                    region.x = scaleLocation.x + scaleRegion.width + region.padding + maxLabelWidth/2;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height / 3 : (alignment == 'far' ? scaleRegion.height : region.height));
                    regions[1].push(region);
                    break;
                case 'top':
                    region.x = (alignment == 'center') ? scaleRegion.x + scaleRegion.width / 2 - region.width / 2 : (alignment == 'near' ? scaleRegion.x : scaleRegion.x + scaleRegion.width - region.width);
                    region.y = scaleLocation.y - region.padding;
                    regions[2].push(region);
                    break;
                case 'bottom':                    
                    region.x = (alignment == 'center') ? scaleRegion.x + scaleRegion.width / 2 - region.width / 2 : (alignment == 'near' ? scaleRegion.x : scaleRegion.x + scaleRegion.width - region.width);
                    region.y = scaleLocation.height + region.height / 2 + region.padding;
                    regions[3].push(region);
                    break;
            }
        },
       
        _VerticalTextPositioning: function (element, region, scaleLocation, scaleRegion, regions) {
            var position = element.textPosition.toLowerCase();
            var alignment = region.alignment;
            var scaleBounds = this._scaleLoc();
            switch (position) {
                case 'left':
                    region.x = scaleBounds.y - region.width - region.padding;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height / 3 : (alignment == 'far' ? scaleRegion.height : region.height));
                    if (this._tickPosition == 'near' && this._tickPlacement == 'inside' && (this._labelPosition=='above' && this._labelPlacement=='inside'))
                        region.x += this._scale.majorTickSettings.size;
                    regions[0].push(region);
                    break;
                case 'right':
                    region.x = scaleBounds.height + region.padding;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height / 3 : (alignment == 'far' ? scaleRegion.height : region.height));
                    regions[1].push(region);
                    break;
                case 'top':
                    region.x = scaleRegion.x - region.width / 2 + (alignment == 'center' ? scaleRegion.width / 2 : (alignment == 'far' && scaleRegion.width));
                    region.y = scaleRegion.y - region.padding;                   
                    regions[2].push(region);
                    break;
                case 'bottom':
                    region.x = scaleRegion.x - region.width / 2 + (alignment=='center' ? scaleRegion.width / 2 : (alignment == 'far' && scaleRegion.width));
                    region.y = scaleRegion.y + scaleRegion.height + region.height + region.padding - region.height / 3 + this._scale.labelSettings.size / 2;
                    regions[3].push(region);
                    break;
            }
        },
    
        _positionTextGroup: function (elements, regions, horScaleRegion) {
            var elementRegions=[[], [], [], []];
            var isHorizontal = (this._orientation == 'horizontal');            
            var scaleLocation = isHorizontal ? this._scaleLocation : this._getVerticalScaleLocation(horScaleRegion.width);
            var scaleRegion = isHorizontal ? horScaleRegion : { x: scaleLocation.x, y: scaleLocation.y, width: horScaleRegion.height, height: horScaleRegion.width };
            for (var i = 0; i < elements.length; i++) {
                if (elements[i]) {
                    if (isHorizontal)
                        this._horizontalTextPositioning(elements[i], regions[i], scaleLocation, scaleRegion, elementRegions);
                    else
                        this._VerticalTextPositioning(elements[i], regions[i], scaleLocation, scaleRegion, elementRegions);
                }                
            }
            this._avoidElementsOverlapping(elementRegions);
        },
       
        _avoidElementsOverlapping: function (regions) {
            for (var j = 0; j < regions.length; j++) {
                var region = regions[j];
                var isOverlap = false;
                do {
                    for (var i = 0; i < region.length; i++) {
                        for (var k = i - 1; k >= 0; k--) {
                            if (this._isOverlapping(region[i], region[k])) {
                                switch (j) {
                                    //Placing overlapped text for left and right positioned text elements
                                    case 0:
                                    case 1:
                                        if (region[k].y > region[i].y)
                                            region[k].y = region[i].y + region[k].height + (region[k].padding > 1 ? region[k].padding : 1);
                                        else
                                            region[i].y = region[k].y + region[i].height + (region[i].padding > 1 ? region[i].padding : 1);
                                        break;
                                        //Placing overlapped text at top position
                                    case 2:
                                        region[k].y = region[i].y - region[i].height - (region[k].padding > 1 ? region[k].padding : 1);
                                        break;
                                        //Placing overlapped text at bottom position
                                    default:
                                        region[i].y = region[k].y + region[k].height + (region[k].padding > 1 ? region[k].padding : 1);
                                        break;
                                }
                                isOverlap = true;
                                break;
                            }
                            else
                                isOverlap = false;
                        }
                    }
                } while (isOverlap);
            }
            this._applyLeftRightAnchor([regions[0], regions[1]]);
            this._applyTopBottomAnchor([regions[2], regions[3]]);
        },
      
        _locateTextGroup: function (elements, regions) {            
            for (var i = 0; i < elements.length; i++) {
                if (elements[i]) {
                    elements[i]._location = { x: 0, y: 0 };
                    elements[i]._location.x = regions[i].x;
                    elements[i]._location.y = regions[i].y;
                }
            }
        },
     
        _applyLeftRightAnchor: function (regions) {
            var anchor;
            for (var i = 0; i < regions.length; i++) {
                var region = regions[i];
                var maxSize = 0;
                for (var j = 0; j < region.length; j++)
                    maxSize = Math.max(maxSize, region[j].width);
                for (var j = 0; j < region.length; j++) {
                    anchor = region[j].anchor;
                    if (anchor == 'start')
                        region[j].x += (i == 0) && (region[j].width - maxSize);
                    else if (anchor == 'middle')
                        region[j].x += (i == 0) ? (region[j].width - maxSize) / 2 : (maxSize - region[j].width) / 2;
                    else
                        region[j].x += (i != 0) && (maxSize - region[j].width);
                }
            }
        },
     
        _applyTopBottomAnchor: function (regions) {            
            for (var i = 0; i < regions.length; i++) {
                var nearAlignment = [], farAlignment = [], centerAlignment = [];
                var region = regions[i];
                for (var j = 0; j < region.length; j++)
                    if (region[j].alignment == 'near')
                        nearAlignment.push(region[j])
                    else if (region[j].alignment == 'far')
                        farAlignment.push(region[j]);
                    else
                        centerAlignment.push(region[j]);

                this._applyNearAnchor(nearAlignment);
                this._applyCenterAnchor(centerAlignment);
                this._applyFarAnchor(farAlignment);
            }
        },
       
        _applyNearAnchor: function (regions) {
            if (regions.length > 1) {
                var maxSize = 0;
                for (var i = 0; i < regions.length; i++)
                    maxSize = Math.max(maxSize, regions[i].width);
                for (var i = 0; i < regions.length; i++)
                    if (regions[i].anchor == 'start')
                        regions[i].x += (this._orientation == 'vertical') && (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'middle')
                        regions[i].x += (this._orientation == 'horizontal') && (maxSize - regions[i].width) / 2;
                    else
                        regions[i].x += (this._orientation == 'horizontal') ? (maxSize - regions[i].width) : (maxSize - regions[i].width) / 2;
            }
        },
      
        _applyCenterAnchor: function (regions) {
            if (regions.length > 1) {
                var maxSize = 0;
                for (var i = 0; i < regions.length; i++)
                    maxSize = Math.max(maxSize, regions[i].width);
                for (var i = 0; i < regions.length; i++)
                    if (regions[i].anchor == 'start')
                        regions[i].x += (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'end')
                        regions[i].x += (maxSize - regions[i].width) / 2;
            }
        },
     
        _applyFarAnchor: function (regions) {
            if (regions.length > 1) {
                var maxSize = 0;
                for (var i = 0; i < regions.length; i++)
                    maxSize = Math.max(maxSize, regions[i].width);
                for (var i = 0; i < regions.length; i++)
                    if (regions[i].anchor == 'start')
                        regions[i].x += (this._orientation == 'horizontal') ? regions[i].width - maxSize : (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'middle')
                        regions[i].x += (this._orientation == 'horizontal') && (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'end')
                        regions[i].x += (this._orientation == 'vertical') && (maxSize - regions[i].width) / 2;
            }
        },
       
        _isOverlapping: function (reg1, reg2) {
            return !(reg1.x + reg1.width < reg2.x || reg1.x > reg2.x + reg2.width || reg1.y - reg1.height > reg2.y || reg1.y < reg2.y - reg2.height);
        },
        
        _initializeValues: function () {
            this._scale = this.model.quantitativeScaleSettings;
            this._labelPosition = this._scale.labelSettings.position.toLowerCase();
            this._tickPosition = this._scale.tickPosition.toLowerCase();
            this._flowDirection = this.model.flowDirection.toLowerCase();
            this._orientation = this.model.orientation.toLowerCase();
            this._tickPlacement = this._scale.tickPlacement.toLowerCase();
            this._labelPlacement = this._scale.labelSettings.labelPlacement.toLowerCase();
        },
      
        _changeOrientation: function (bulletGroup) {
            if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) {               
                bulletGroup.setAttribute("transform", "translate(0," +this.svgHeight +")rotate(-90)");
            }
        },
       
        _setModel: function (options) {
            var isFlag = true;
            for (var key in options) {
                this.model.enableAnimation = false;
                switch (key) {
                    case "height":
                        this.model.height = options[key];
                        break;
                    case "width":
                        this.model.width = options[key];
                        break;
                    case "theme":
                        this.model.theme = options[key];
                        this._setTheme(ej.datavisualization.BulletGraph.Themes, this.model.theme);
                        break;
                    case "orientation":
                        this.model.orientation = options[key];
                        break;
                    case "flowDirection":
                        this.model.flowDirection = options[key];
                        break;
                    case "qualitativeRangeSize":
                        this.model.qualitativeRangeSize = options[key];
                        break;
                    case "quantitativeScaleLength":
                        this.model.quantitativeScaleLength = options[key];
                        break;
                    case "quantitativeScaleSettings":
                        $.extend(true, this.model.quantitativeScaleSettings, {}, options[key]);
                        break;
                    case "applyRangeStrokeToTicks":
                        this.model.applyRangeStrokeToTicks = options[key];
                        break;
                    case "applyRangeStrokeToLabels":
                        this.model.applyRangeStrokeToLabels = options[key];
                        break;
                    case "qualitativeRanges":
                        $.extend(true, this.model.qualitativeRanges, {}, options[key]);
                        break;
                    case "captionSettings":
                        $.extend(true, this.model.captionSettings, {}, options[key]);
                        break;
                    case "dataSource":
                        $.extend(true, this.model.fields, {}, options[key]);
                        break;
                    case "value":
                        for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                            this.model.quantitativeScaleSettings.featureMeasures[i].value = parseFloat(this.value());
                        }
                        break;
                    case "comparativeMeasureValue":
                        for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                            this.model.quantitativeScaleSettings.featureMeasures[i].comparativeMeasureValue = parseFloat(this.comparativeMeasureValue());
                        }
                        break;
                    case "enableAnimation":
                        this.model.enableAnimation = options[key];
                        if (this.model.enableAnimation) {
                            $(this.svgObject).empty();
                            this._renderBulletElements();
                            this._animateMeasures();
                        }
                        isFlag = false;
                        break;
                }
            }
            if (isFlag) {
                $(this.svgObject).empty();
                this._renderBulletElements();
            }
        },
      
        _bindData: function () {

            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                    this._initDataSource();
                }
                else {
                    this._dataCount = this.model.fields["dataSource"].length;
                    this._drawMeasures();
                }
            }
            else {      //this.model.quantitativeScaleSettings.featureMeasures != null
                this._dataCount = this._scale.featureMeasures.length;
                this._drawMeasures();
            }
        },
      
        _drawMeasures: function () {
            this._drawFeatureMeasureBar();
            this._drawComparativeMeasureSymbol();
        },
      
        _initDataSource: function () {
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.model.fields["dataSource"].executeQuery(query);
            queryPromise.done(function (e) {
                proxy.model.fields["dataSource"] = e.result;
                proxy._dataCount = e.result.length;
                proxy._drawFeatureMeasureBar();
                proxy._drawComparativeMeasureSymbol();
                proxy._bindHighlightRemoving();
            });
        },
      
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "query" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
             
        _drawCaption: function () {
            if (this.model.drawCaption) {
                var data = { font: this.model.captionSettings.font, location: this.model.captionSettings.location, subTitle: this.model.captionSettings.subTitle, text: this.model.captionSettings.text, textAngle: this.model.captionSettings.textAngle };
                this._trigger("drawCaption", data);
                this.model.captionSettings = $.extend(this.model.captionSettings, data);
            }
        },
      
        _drawBulletSymbol: function(shape, symbolRect, symbolOptions, captionGroup){
            var arrowPath = "M" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2))) + " " + (symbolRect.y) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (symbolRect.height / 4));

            switch (shape.toLowerCase()) {
                case 'circle':
                    var radius = Math.min(symbolRect.height, symbolRect.width) / 2;
                    symbolOptions.cx = symbolRect.x;
                    symbolOptions.cy = symbolRect.y,
                    symbolOptions.r = radius,
                    this.svgRenderer.drawCircle(symbolOptions, captionGroup);
                    break;

                case 'leftarrow':                        
                    symbolOptions.d = arrowPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'rightarrow':
                    symbolOptions.d = arrowPath;
                    symbolOptions.transform = 'rotate(180,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'uparrow':
                    symbolOptions.d = arrowPath;
                    symbolOptions.transform = 'rotate(90,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'downarrow':
                    symbolOptions.d = arrowPath;
                    symbolOptions.transform = 'rotate(-90,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'cross':
                    var crossPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " " +
                        "M" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = crossPath;
                    symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'horizontalline':
                    var horizPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " " + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = horizPath;
                    symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'verticalline':
                    var verPath = "M" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = verPath;
                    symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'triangle':
                    var triPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " Z";
                    symbolOptions.d = triPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'invertedtriangle':
                    var triPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " Z";
                    symbolOptions.d = triPath;
                    symbolOptions.transform = 'rotate(180,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'hexagon':
                    var hexPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 4)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 4)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = hexPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'wedge':
                    var wedgePath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + symbolRect.width / 2) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = wedgePath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'pentagon':
                    var eq = 72;
                    var radius = Math.sqrt(symbolRect.height * symbolRect.height + symbolRect.width * symbolRect.width) / 2;
                    var sb = ej.EjSvgRender.utils._getStringBuilder();
                    for (var i = 0; i <= 5; i++) {
                        var deg = i * eq;
                        var rad = (Math.PI / 180) * deg;
                        var x1 = radius * Math.cos(rad);
                        var y1 = radius * Math.sin(rad);
                        if (i == 0)
                            sb.append("M" + " " + (symbolRect.x + x1) + " " + (symbolRect.y + y1) + " ");

                        else
                            sb.append("L" + " " + (symbolRect.x + x1) + " " + (symbolRect.y + y1) + " ");

                    }                        
                    symbolOptions.d = sb.toString();
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'star':
                    var starPath = "M" + " " + (symbolRect.x + (symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 6)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 6)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = starPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'rectangle':
                    var rectPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = rectPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'trapezoid':
                    var path = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2) + (symbolRect.width)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2) + (symbolRect.width)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = path;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'diamond':
                    var path = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = path;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'ellipse':                        
                    symbolOptions.cx = symbolRect.x;
                    symbolOptions.cy = symbolRect.y;
                    symbolOptions.rx = symbolRect.width/2;
                    symbolOptions.ry = symbolRect.height/2;
                    this.svgRenderer.drawEllipse(symbolOptions, captionGroup);
                    break;
                case 'image':
                    symbolOptions.x = symbolRect.x - symbolRect.width / 2;
                    symbolOptions.y = symbolRect.y - symbolRect.height / 2;
                    symbolOptions.width = symbolRect.width;
                    symbolOptions.height = symbolRect.height;
                    symbolOptions.href = this.model.captionSettings.indicator.symbol.imageURL;
                    symbolOptions.visibility = 'visible';
                    this.svgRenderer.drawImage(symbolOptions, captionGroup);
                    break;
                default:
                    break;
            }                
        },
       
        _drawIndicator: function () {
            if (this.model.captionSettings.indicator.visible) {
                var indicatorSettings = this.model.captionSettings.indicator;
                var indicatorEventArgs = $.extend({}, { indicatorSettings: this.model.captionSettings.indicator });
                if (this.model.drawIndicator) {
                    this._trigger("drawIndicator", indicatorEventArgs);
                    if (indicatorEventArgs.cancel == false) {
                        indicatorSettings = indicatorEventArgs.indicatorSettings;
                        this.model.captionSettings.indicator = indicatorSettings;
                    }
                }
                var textBounds = ej.EjSvgRender.utils._measureText(indicatorSettings.text, null, indicatorSettings.font);
                this._indicator = { 'bounds': textBounds, 'settings': indicatorSettings };
            }
        },
      
        _indicatorTextOptions: function (settings, rect, bounds, id) {
            var location = settings._location ? settings._location : settings.location;
            return {
                'class' : id,
                'x': location.x + rect.width + settings.textSpacing,
                'y': location.y - rect.height / 2 + parseFloat(settings.font.size) / 3,
                'fill': settings.font.color,
                'font-size': settings.font.size,
                'font-family': settings.font.fontFamily,
                'font-style': settings.font.fontStyle,
                'font-weight': settings.font.fontWeight,
                'text-anchor': 'start',
                'opacity': settings.font.opacity,
                'transform': 'rotate(' + settings.textAngle + ',' + rect.x  + ',' + rect.y + ')'
            };
        },
      
        _indicatorSymbolOptions: function (group, settings) {
            return {
                'id': group.id + '_indicatorSymbol',
                'stroke': settings.symbol.border.color,
                'fill': settings.symbol.color,
                'stroke-width': settings.symbol.border.width,
                'opacity': settings.symbol.opacity
            };
        },
       
        _textOptions: function (text, id) {
            var location = (text._location ? text._location : text.location);
            var textOptions = {
                'class': id,
                'x': location.x,
                'y': location.y,
                'fill': text.font.color,
                'font-size': text.font.size,
                'font-family': text.font.fontFamily,
                'font-style': text.font.fontStyle,
                'font-weight': text.font.fontWeight,
                'text-anchor': 'start',
                'opacity': text.font.opacity,
				'cursor':'default',
                'transform': 'rotate(' + text.textAngle + ',' + (location.x) + ',' + (location.y) + ')'
            };
            return textOptions;
        },
       
        _drawScale: function (captionGroup, scaleGroup) {
            var scaleLocation = this._scale._location ? this._scale._location : this._scale.location;
            var pointY, pointX;
            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Far || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Center)
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? scaleLocation.y : scaleLocation.y + this._scale.labelSettings.offset + this._scale.labelSettings.size;
            else
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? scaleLocation.y + this._scale.majorTickSettings.size : scaleLocation.y + this._scale.labelSettings.offset + this._scale.labelSettings.size + this._scale.majorTickSettings.size;

            if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                pointX = scaleLocation.x;
            else
                pointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? scaleLocation.x - this._scale.labelSettings.offset : scaleLocation.x;

            var scaleOptions = {
                'id': this.svgObject.id + '_SvgScale',
                'x': pointX,
                'y': pointY,
                'width': this.model.quantitativeScaleLength,
                'height': this.model.qualitativeRangeSize,
                'fill': "transparent",
                'stroke-width': 0
            };
            this.svgRenderer.drawRect(scaleOptions, scaleGroup);
        },
     
        _drawMajorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, y1, tickIndex = 0;
            var scaleLocation = this._scale.location;
            var isTicksInside = (this._tickPlacement == 'inside');
            var locX = this._scale.location.x;
            var scaleLength = this.model.quantitativeScaleLength;
            var labelOffset = this._scale.labelSettings.offset;

            var rangeSize = this.model.qualitativeRangeSize;
            var majorTickWidth = this._scale.majorTickSettings.width;
            var majorTickSize = this._scale.majorTickSettings.size;

            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);
            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isBelow = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below);

            if (isForward)
                majorPointX = isHorizontal ? locX + (majorTickWidth / 2) : locX + scaleLength + (majorTickWidth / 2);
            else
                majorPointX = isHorizontal ? locX + scaleLength - (majorTickWidth / 2) : locX + (majorTickWidth / 2);
            
            if(this._tickPosition == 'far')
                y1 = scaleLocation.y + rangeSize + (isTicksInside ? -majorTickSize : 0);
            else if(this._tickPosition == 'near')
                y1 = scaleLocation.y + (isTicksInside ? 0 : -majorTickSize);
            else
                y1 = scaleLocation.y + (rangeSize / 2) - (majorTickSize / 2);
            
            var min = this._scale.minimum;
            var max = this._scale.maximum;
            var interval = this._scale.interval;
            
            for (var i = min; i <= max; i += interval) {
                tickIndex++;
                if (i >= max)
                    if (isHorizontal && !isForward)
                        majorPointX += majorTickWidth;
                    else
                        majorPointX -= majorTickWidth;
                if (!isHorizontal && isForward)
                    majorPointX = (i >= max) ? majorPointX + (majorTickWidth) : (i == min) ? majorPointX - (majorTickWidth) : majorPointX;

                if (this.model.applyRangeStrokeToTicks)
                    strokeColor = this._bindingRangeStrokes(majorPointX - (majorTickWidth / 2));

                var options = this._majorTickLines(this._scale, majorPointX, y1);

                if (!isHorizontal && isForward && i == min)
                    majorPointX = majorPointX + majorTickWidth;
                
                if (this.model.drawTicks) {
                    data = { majorTickSettings: this._scale.majorTickSettings, minorTickSettings: this._scale.minorTickSettings, minorTicksPerInterval: this._scale.minorTicksPerInterval, maximum: max, minimum: min, interval: interval };
                    this._trigger("drawTicks", data);                    
                    options = this._majorTickLines(data, majorPointX, y1);
                }
                var majorTicks = this.svgRenderer.createLine(options);
                majorPointX = ((!isForward && isHorizontal) || (isForward && !isHorizontal)) ? majorPointX - intervalValue : majorPointX + intervalValue;
                scaleGroup.appendChild(majorTicks);
            }
        },
        _majorTickLines: function (data, majorPointX, y1) {
            var options = {
                'x1': majorPointX,
                'y1': y1,
                'x2': majorPointX,
                'y2': y1 + data.majorTickSettings.size,
                'stroke-width': data.majorTickSettings.width,
                'stroke': (this.model.applyRangeStrokeToTicks && strokeColor) ? strokeColor : data.majorTickSettings.stroke
            };
            return options;
        },
       
        _forwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x)) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (var k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= (this._rangeCollection[k] + this._scale.location.x) && majorPointX <= (this._rangeCollection[k + 1] + this._scale.location.x))
                    return this.model.qualitativeRanges[k + 1].rangeStroke;
            }
        },
       
        _backwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._rangeCollection[this._rangeCollection.length - 1]) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= this._rangeCollection[k] && majorPointX < this._rangeCollection[k + 1]) {
                    var index = $.inArray(this._rangeCollection[k], this._rangeCollection)
                    return this.model.qualitativeRanges[(this._rangeCollection.length - 1) - index].rangeStroke;
                }
            }
        },
      
        _bindingRangeStrokes: function (majorPointX) {
            if (this._rangeCollection.length == 1) {
                if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x))
                    return this.model.qualitativeRanges[0].rangeStroke;
            }
            else {
                if ((this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) || (this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) {
                    return this._forwardStrokeBinding(majorPointX);
                }
                else {
                    return this._backwardStrokeBinding(majorPointX);
                }
            }
        },
       
        _drawMinorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, minorPointX, y1, y2, x, tickIndex = 0;

            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);

            var isTicksInside = (this._tickPlacement == 'inside');
            var locX = this._scale.location.x;
            var rangeSize = this.model.qualitativeRangeSize;
            var labelOffset = this._scale.labelSettings.offset;

            var majorTickSize = this._scale.majorTickSettings.size;
            var minorTickSize = this._scale.minorTickSettings.size;
            var minorTickWidth = this._scale.minorTickSettings.width;
            var scaleLength = this.model.quantitativeScaleLength;

            if (isForward)
                majorPointX = isHorizontal ? locX : locX + scaleLength;
            else
                majorPointX = isHorizontal ? locX + scaleLength - (minorTickWidth / 2) : locX;

            switch (this._tickPosition) {
                case 'far':
                    y1 = this._scale.location.y + rangeSize;
                    y2 = y1 + (isTicksInside ? -minorTickSize : minorTickSize);
                    break;
                case 'near':
                    y1 = this._scale.location.y + (isTicksInside ? minorTickSize : -minorTickSize);
                    y2 = this._scale.location.y;
                    break;                
                default:
                    y1 = this._scale.location.y + (rangeSize / 2) - (minorTickSize / 2);
                    y2 = y1 + minorTickSize;
                    break;
            }
            var maxLimit = this._scale.maximum;
            var min = this._scale.minimum;
            var interval = this._scale.interval;
            var minorTicksPerInterval = this._scale.minorTicksPerInterval;
           
            for (var i = min; i <= maxLimit; i += interval) {
                minorPointX = intervalValue / minorTicksPerInterval;

                for (var j = 1; j <= minorTicksPerInterval; j++) {
                    tickIndex++;
                    x = ((!isHorizontal && isForward) || (isHorizontal && !isForward))
                        ? majorPointX - minorPointX + (minorPointX / (minorTicksPerInterval + 1))
                        : majorPointX + minorPointX - (minorPointX / (minorTicksPerInterval + 1));

                    if (!isForward && isHorizontal) {
                        if (tickIndex >= ((maxLimit - min) * minorTicksPerInterval))
                            x += (minorTickWidth / 2);
                    }

                    if (this.model.applyRangeStrokeToTicks)
                        strokeColor = this._bindingRangeStrokes(x);
                   
                    var options = this._minorTickLines(this._scale, x, y1, y2);
                    if (this.model.drawTicks) {
                        data = { majorTickSettings: this._scale.majorTickSettings, minorTickSettings: this._scale.minorTickSettings, minorTicksPerInterval: this._scale.minorTicksPerInterval, maximum: this._scale.maximum, minimum: this._scale.minimum, interval: this._scale.interval };
                        this._trigger("drawTicks", data);
                        this._scale = $.extend(this._scale, data);
                        options = this._minorTickLines(data, x, y1, y2);
                    }
                   
                    var chkMax;
                    if (!isForward && isHorizontal) {
                        chkMax = (scaleLength + locX + 1);
                        if (x <= chkMax && x >= locX + 1) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    else if (isForward && !isHorizontal) {
                        chkMax = (locX + 1);
                        if (x >= chkMax) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    else {
                        chkMax = scaleLength + locX + 1;
                        if (x <= chkMax) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    
                    minorPointX = (intervalValue / minorTicksPerInterval) * (j + 1);
                }
                majorPointX = ((!isForward && isHorizontal) || (isForward && !isHorizontal)) ? majorPointX - intervalValue : majorPointX + intervalValue;
            }
        },
        _minorTickLines: function (data, x, y1, y2) {
            var options = {
                'x1': x,
                'y1': y1,
                'x2': x,
                'y2': y2,
                'stroke-width': data.minorTickSettings.width,
                'stroke': (this.model.applyRangeStrokeToTicks && strokeColor) ? strokeColor : data.minorTickSettings.stroke
            };
            return options;
        },
       
        _drawLabels: function (intervalValue, scaleGroup) {
            var labelX, pointY, locale = this.model.locale, localizedText = locale && this.model.enableGroupSeparator, text;
            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);

            var locX = this._scale.location.x;
            var offset = this._scale.labelSettings.offset;
            var scaleLength = this.model.quantitativeScaleLength;

            var majorTickWidth = this._scale.majorTickSettings.width;

            if (isForward)
                labelX = isHorizontal ? locX : locX + scaleLength;
            else
                labelX = isHorizontal ? locX + scaleLength : locX;
            
            var placement = this._tickPosition + this._labelPlacement;
            var isLabelsBelow = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below);

            var locY = this._scale.location.y;
            var rangeSize = this.model.qualitativeRangeSize;
            var labelSize = this._scale.labelSettings.size;
            var majorTickSize = this._scale.majorTickSettings.size;

            switch (placement) {
                case 'faroutside':
                case 'centeroutside':                    
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize + offset + labelSize : this._scale.location.y - offset;
                    break;
                case 'farinside':
                case 'centerinside':
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize - offset : this._scale.location.y + offset + (isHorizontal ? labelSize : 0);
                    break;
                case 'nearoutside':
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize + offset + labelSize : this._scale.location.y - offset;
                    break;
                case 'nearinside':
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize - offset : this._scale.location.y + offset + labelSize;
                    break;
            }
            var min = this._scale.minimum;
            var max = this._scale.maximum;
            var interval = this._scale.interval;
            var isLabelsInside = (this._labelPlacement == "inside");

            for (var i = min; i <= max; i += interval) {               
                var y = pointY;
                var x = labelX;
                var font = this._scale.labelSettings.font;
                font.size = labelSize;
                var labelWidth = {width: 0, height: 0};
                if (!isHorizontal) {
                    var label = this._scale.labelSettings.labelPrefix + i + this._scale.labelSettings.labelSuffix;
                    switch (placement) {
                        case 'faroutside':
                        case 'centeroutside':
                            if (isLabelsBelow)
                                y -= labelSize;
                            else {
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            break;
                        case 'farinside':
                        case 'centerinside':
                            if (isLabelsBelow) {                                
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            break;
                        case 'nearoutside':
                            if (!isLabelsBelow) {
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            else
                                y -= labelSize;
                            break;
                        case 'nearinside':
                            if (isLabelsBelow) {
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            else
                                y -= labelSize;
                            break;
                    }                    
                    x -= labelSize/3;
                    transformText = 'rotate(' + 90 + ',' + (x) + ',' + (y) + ')';
                }
                else
                    transformText = 'rotate(' + 0 + ',' + (x) + ',' + (y) + ')';

                if (this.model.applyRangeStrokeToLabels)
                    strokeColor = this._bindingRangeStrokes(labelX);
                var labelOptions = this._labelOptions(this.model.quantitativeScaleSettings.labelSettings, x, y);
                if (this.model.drawLabels) {
                    var data = { font: this._scale.labelSettings.font, labelPrefix: this._scale.labelSettings.labelPrefix, labelSuffix: this._scale.labelSettings.labelSuffix, offset: offset, size: labelSize, stroke: this._scale.labelSettings.stroke };
                    this._trigger("drawLabels", data);
                    labelOptions = this._labelOptions(data, x, y);
                }

                if (isForward)
                    labelX = isHorizontal ? labelX + intervalValue : labelX - intervalValue;
                else
                    labelX = isHorizontal ? labelX - intervalValue : labelX + intervalValue;
                
				 text = localizedText ? i.toLocaleString(locale) : i;
                this.svgRenderer.drawText(labelOptions, labelOptions.labelPrefix + text + labelOptions.labelSuffix, scaleGroup);
            }
        },

        _labelOptions: function (data, labelX, pointY) {
            var labelOptions = {
                'x': labelX,
                'y': pointY,
                'text-anchor': (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? 'middle' : 'start',
                'fill': (this.model.applyRangeStrokeToLabels && strokeColor) ? strokeColor : data.stroke,
                'font-size': data.size + "px",
                'font-family': data.font.fontFamily,
                'font-style': data.font.fontStyle,
                'font-weight': data.font.fontWeight,
                'opacity': data.font.opacity,
                'transform': transformText,
                'labelPrefix': data.labelPrefix,
                'labelSuffix': data.labelSuffix,
            };
            return labelOptions;
        },
      
        _drawQualitativeRanges: function (scaleGroup) {
            var pointX = this._scale.location.x, pointY = this._scale.location.y, width;
            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);
            var labelOffset = this._scale.labelSettings.offset;

            this._rangeCollection = [];

            var placement = this._tickPosition + this._labelPlacement;
            var isLabelBelow = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below);

            var scaleHeight = this.model.qualitativeRangeSize;
            
            var locX = this._scale.location.x;
            var min = this._scale.minimum;
            var max = this._scale.maximum;
            var scaleLength = this.model.quantitativeScaleLength;            

            for (var i = this.model.qualitativeRanges.length - 1; i >= 0; i--) {
                var data = { object: this, rangeIndex: i, rangeOptions: this.model.qualitativeRanges[i], rangeEndValue: this.model.qualitativeRanges[i].rangeEnd };
                if (this.model.drawQualitativeRanges)
                    this._trigger("drawQualitativeRanges", data);
                var rangeEnd = data.rangeEndValue;
                rangeEnd = (rangeEnd > max) ? max : rangeEnd;
                if (isForward) {
                    pointX = isHorizontal ? locX : locX + (scaleLength - scaleLength / ((max - min) / ((max - min) - (max - rangeEnd))));
                    width = isHorizontal ? scaleLength / ((max - min) / ((max - min) - (max - rangeEnd))) : scaleLength / ((max - min) / ((max - min) - (max - rangeEnd)));
                }
                else {
                    pointX = isHorizontal ? locX + (scaleLength - scaleLength / ((max - min) / ((max - min) - (max - rangeEnd)))) : locX;
                    width = isHorizontal ? scaleLength / ((max - min) / ((max - min) - (max - rangeEnd))) : scaleLength / ((max - min) / ((max - min) - (max - rangeEnd)));
                }
                var rangeOptions = {
                    'x': pointX,
                    'y': pointY,
                    'height': this.model.qualitativeRangeSize,
                    'width': (width > 0) ? (width < scaleLength ? width : scaleLength) : 0,
                    'fill': data.rangeOptions.rangeStroke,
                    'opacity': data.rangeOptions.rangeOpacity
                };
                ((isHorizontal && isForward) || (!isHorizontal && !isForward)) ? this._rangeCollection.push(width) : this._rangeCollection.push(pointX);                
                this.svgRenderer.drawRect(rangeOptions, scaleGroup);
            }
            this._rangeCollection.sort(this._sortRangeCollection);
        },
      
        _sortRangeCollection: function (a, b) {
            return (a - b);
        },
      
        _calculateFeatureMeasureBounds: function (value, categoryValue) {
            var min = this._scale.minimum;
            value = (value < min && min < 0) ? min : value;                
            if (value >= min) {
                var x, lx, width;
                var locX = this._scale.location.x;
                var scaleLength = this.model.quantitativeScaleLength;
                var diff = this._scale.maximum - this._scale.minimum;
                var valueDiff = this._scale.maximum - value;
                var orientation = this._flowDirection.toLowerCase() + this._orientation.toLowerCase();
                categoryValue = (categoryValue == null) ? "" : categoryValue;
                var font = this._scale.labelSettings.font;
                font.size = this._scale.labelSettings.size;
                var stringLength = ej.EjSvgRender.utils._measureText(categoryValue.toString(), null, font).width;

                switch (orientation) {
                    case 'forwardhorizontal':
                    case 'backwardvertical':                        
                            x = locX + ((min > 0) ? 0 : scaleLength / diff * Math.abs(min));
                            width = scaleLength / (diff / ( (min > 0) ? diff - valueDiff : value));                            
                            if (value < 0) {
                                width = Math.abs(width);
                                x -= width;
                            }
                            width = (x + width < locX + scaleLength) ? width : locX + scaleLength - x;
                            lx = locX - ((orientation == 'forwardhorizontal') ? (stringLength / 2 + this._scale.labelSettings.offset) : this._scale.labelSettings.offset);
                            break;
                    default:
                        x = locX + (scaleLength - scaleLength / (diff / (diff - valueDiff)));
                        width = (min > 0) ? scaleLength / (diff / (diff - valueDiff)) : scaleLength / (diff / (value));
                        if (value < 0) {
                            width = Math.abs(width);
                            x -= width;
                        }
                        if (x < locX ) {
                            width = x + width - locX;
                            x = locX;
                        }                        
                        lx = locX + scaleLength + ((orientation == 'backwardhorizontal') ? (stringLength / 2 + this._scale.labelSettings.offset) : this._scale.labelSettings.offset);
                        break;
                }
                return { pointX: x, Width: width, lPointX: lx };
            }
            return false;
        },
      
        _drawFeatureMeasureBar: function () {
            var j = 1, count, value, categoryValue;
            var pointY, lPointY, locale = this.model.locale, localizedText = locale && this.model.enableGroupSeparator;
            var isDataFields = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null);
            var isVertical = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical);
            var bulletValue = (typeof (this.model.value) == 'function') ? this._value() : this.model.value;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = isDataFields ? this.model.fields["dataSource"][i][this.model.fields.featureMeasures] : (ej.isNullOrUndefined(this._scale.featureMeasures[i].value) ? bulletValue : this._scale.featureMeasures[i].value);
                    categoryValue = isDataFields ? this.model.fields["dataSource"][i][this.model.fields.category] : this._scale.featureMeasures[i].category;

                    pointY = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2);
                    lPointY = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureSettings.width / 2) - 1;

                    var bounds = this._calculateFeatureMeasureBounds(value, categoryValue);
                    if (bounds) {

                        var featureBarOptions = this._featureBar(this._scale, bounds.pointX, pointY, bounds.Width, i);
                        if (this.model.drawFeatureMeasureBar) {
                            var data = { featuredMeasureSettings: this._scale.featuredMeasureSettings };
                            this._trigger("drawFeatureMeasureBar", data);
                            featureBarOptions = this._featureBar(data, bounds.pointX, pointY, bounds.Width, i);
                        }
                        this.svgRenderer.drawRect(featureBarOptions, this._scaleGroup);

                        if (isVertical)
                            transformText = 'rotate(' + 90 + ',' + bounds.lPointX + ',' + (lPointY - 4) + ')';
                        else
                            transformText = 'rotate(' + 0 + ',' + bounds.lPointX + ',' + lPointY + ')';


                        var categoryOptions = this._drawcategory(this.model.quantitativeScaleSettings.labelSettings, bounds.lPointX, lPointY);
                        if (this.model.drawCategory) {
                            var data = { size: this.model.quantitativeScaleSettings.labelSettings.size, stroke: this.model.quantitativeScaleSettings.labelSettings.stroke, font: this.model.quantitativeScaleSettings.labelSettings.font, categoryValue: categoryValue };
                            this._trigger("drawCategory", data);
                            categoryOptions = this._drawcategory(data, bounds.lPointX, lPointY);
                            categoryValue = data.categoryValue;
                        }
                        if (!ej.isNullOrUndefined(categoryValue))
                            this.svgRenderer.drawText(categoryOptions, localizedText ? categoryValue.toLocaleString(locale) : categoryValue, this._scaleGroup);
                        j += 2;
                        this.value(value);
                    }
                }

            }
        },
        _drawcategory: function (category, lPointX, lPointY) {
            var categoryOptions = {
                'x': lPointX,
                'y': lPointY + (this._scale.featuredMeasureSettings.width / 2),
                'text-anchor': 'middle',
                'fill': category.stroke,
                'font-size': category.size + "px",
                'font-family': category.font.fontFamily,
                'font-style': category.font.fontStyle,
                'font-weight': category.font.fontWeight,
                'opacity': category.font.opacity,
                'transform': transformText
            };
            return categoryOptions;
        },
        _featureBar: function (data, pointX, pointY, width, i) {
            var featureBarOptions = {
                'class': this.svgObject.id + "_FeatureMeasure",
                'id': this.svgObject.id + "_FeatureMeasure_" + i,
                'x': pointX,
                'y': pointY,
                'height': data.featuredMeasureSettings.width,
                'width': width,
                'fill': data.featuredMeasureSettings.stroke            
            };           
            return featureBarOptions;
        },
       
        _drawComparativeMeasureSymbol: function () {
            var j = 1, value;
            var y1, y2, x1;

            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);
            var bulletCMValue = (typeof (this.model.comparativeMeasureValue) == 'function') ? this._comparativeMeasureValue() : this.model.comparativeMeasureValue;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.comparativeMeasure] : (ej.isNullOrUndefined(this._scale.featureMeasures[i].comparativeMeasureValue) ? bulletCMValue : this._scale.featureMeasures[i].comparativeMeasureValue);
                    if (value >= this._scale.minimum && value <= this._scale.maximum) {
                        y1 = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) - this._scale.featuredMeasureSettings.width;
                        y2 = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + (2 * this._scale.featuredMeasureSettings.width);
                        
                        if (isForward)
                            x1 = isHorizontal ? this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))) : this._scale.location.x + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))));
                        else
                            x1 = isHorizontal ? this._scale.location.x + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))))) : this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))));

                        var compareMeasureOptions = this._compareMeasure(this._scale, x1, y1, y2, i, value);
                        if (this.model.drawComparativeMeasureSymbol) {
                            var data = { comparativeMeasureSettings: this._scale.comparativeMeasureSettings };
                            this._trigger("drawComparativeMeasureSymbol", data);
                            compareMeasureOptions = this._compareMeasure(data, x1, y1, y2, i, value);
                        }
                       
                        this.svgRenderer.drawLine(compareMeasureOptions, this._scaleGroup);
                        j += 2;
                        this.comparativeMeasureValue(value);
                    }
                }
            }
        },
        _compareMeasure: function(data, x1, y1, y2, i, value){
            var compareMeasureOptions = {
                'class': this.svgObject.id + "_ComparativeMeasure",
                'id': this.svgObject.id + "_ComparativeMeasure_" + i,
                'x1': (value == this._scale.maximum) ? x1 - (data.comparativeMeasureSettings.width / 2) : (value == this._scale.minimum) ? x1 + (data.comparativeMeasureSettings.width / 2) : x1,
                'y1': y1,
                'x2': (value == this._scale.maximum) ? x1 - (data.comparativeMeasureSettings.width / 2) : (value == this._scale.minimum) ? x1 + (data.comparativeMeasureSettings.width / 2) : x1,
                'y2': y2,
                'stroke-width': data.comparativeMeasureSettings.width,
                'stroke': data.comparativeMeasureSettings.stroke
            };
            return compareMeasureOptions;
        },
       
        _setTheme: function (jsonObj, themeName) {
            var result = [];
            this._scale = this.model.quantitativeScaleSettings;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var i = 0; i < result.length; i++) {
                this._scale.majorTickSettings.stroke = ((!this._scale.majorTickSettings.stroke || this._scale.majorTickSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.majorTickSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.majorTickSettings.stroke : this._scale.majorTickSettings.stroke);
                this._scale.minorTickSettings.stroke = ((!this._scale.minorTickSettings.stroke || this._scale.minorTickSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.minorTickSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.minorTickSettings.stroke : this._scale.minorTickSettings.stroke);
                this._scale.labelSettings.stroke = ((!this._scale.labelSettings.stroke || this._scale.labelSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.labelSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.labelSettings.stroke : this._scale.labelSettings.stroke);
                this._scale.featuredMeasureSettings.stroke = ((!this._scale.featuredMeasureSettings.stroke || this._scale.featuredMeasureSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.featuredMeasureSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.featuredMeasureSettings.stroke : this._scale.featuredMeasureSettings.stroke);
                this._scale.comparativeMeasureSettings.stroke = ((!this._scale.comparativeMeasureSettings.stroke || this._scale.comparativeMeasureSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.comparativeMeasureSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.comparativeMeasureSettings.stroke : this._scale.comparativeMeasureSettings.stroke);
                this.model.captionSettings.font.color = ((!this.model.captionSettings.font.color || this.model.captionSettings.font.color == jsonObj[result[i]].captionSettings.font.color) ? jsonObj[themeName].captionSettings.font.color : this.model.captionSettings.font.color);
                this.model.captionSettings.subTitle.font.color = ((!this.model.captionSettings.subTitle.font.color || this.model.captionSettings.subTitle.font.color == jsonObj[result[i]].captionSettings.subTitle.font.color) ? jsonObj[themeName].captionSettings.subTitle.font.color : this.model.captionSettings.subTitle.font.color);
                this.model.captionSettings.indicator.font.color = ((!this.model.captionSettings.indicator.font.color || this.model.captionSettings.indicator.font.color == jsonObj[result[i]].captionSettings.indicator.font.color) ? jsonObj[themeName].captionSettings.indicator.font.color : this.model.captionSettings.indicator.font.color);
                this.model.captionSettings.indicator.symbol.color = ((!this.model.captionSettings.indicator.symbol.color || this.model.captionSettings.indicator.symbol.color == jsonObj[result[i]].captionSettings.indicator.symbol.color) ? jsonObj[themeName].captionSettings.indicator.symbol.color : this.model.captionSettings.indicator.symbol.color);
                for (var j = 0; j < this.model.qualitativeRanges.length; j++)
                    this.model.qualitativeRanges[j].rangeStroke = ((!this.model.qualitativeRanges[j].rangeStroke) ? (ej.isNullOrUndefined(jsonObj[themeName].qualitativeRanges[j]) ? jsonObj[themeName].qualitativeRanges[0].rangeStroke : jsonObj[themeName].qualitativeRanges[j].rangeStroke) : this.model.qualitativeRanges[j].rangeStroke);
            }
            //            this.model = $.extend(true, this.model, jsonObj[themeName], this.model);
        },

      
       
        _onDrawQualitativeRanges: function (rangeOptions, rangeEnd, rangeIndex) {
            var data = { object: this, scaleElement: this.model.quantitativeScaleSettings, rangeIndex: rangeIndex, rangeElement: rangeOptions, rangeEndValue: rangeEnd };
            this._trigger("drawQualitativeRanges", data);
        },
        
       
        _doAnimation: function () {
            var elements = $("." + this.svgObject.id + "_FeatureMeasure");
            for (var i = elements.length - 1; i >= 0; i--) {
                var element = elements[i];
                this._animateFeatureBar(element);
            }
        },
      
        _doLineAnimation: function () {
            var compareElements = $("." + this.svgObject.id + "_ComparativeMeasure");
            var secondsPerPoint = 2000 / compareElements.length;
            for (var i = compareElements.length - 1; i >= 0; i--) {
                var element = compareElements[i];
                $(element).attr("transform", "scale(0)");
                this._doLineSymbol(element, secondsPerPoint, i);
            }
        },
       
        _animateFeatureBar: function (element) {
            var box = element.getBBox();
            var left, top;
            if (((this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) && (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) || ((this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) && (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward))) {
                left = box.x + box.width;
                top = box.y + box.height;
            }
            else {
                left = box.x;
                top = box.y;
            }
            $(element).animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    $(element).attr("transform", "translate(" + left + " " + top + ") scale(" + now + ",1) translate(" + (-left) + " " + (-top) + ")");
                }
            });
        },
      
        _doLineSymbol: function (element, sec, val) {
            var beginTime = parseInt(val * sec);
            var box = element.getBBox();
            var centerX = box.x + (box.width / 2);
            var centerY = box.y + (box.height / 2);
            $(element).delay(beginTime).animate(
                {
                    scale: 1
                },
                {
                    duration: 200,
                    step: function (now) {
                        $(element).attr("transform", "translate(" + centerX + " " + centerY + ") scale(" + now + ") translate(" + (-centerX) + " " + (-centerY) + ")");
                    }
                });
        },
      
        bindEvents: function () {
            var browserInfo = ej.EjSvgRender.utils.browserInfo(),
            isPointer = browserInfo.isMSPointerEnabled,
            isIE11Pointer = browserInfo.pointerEnabled,
            touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
            touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
            touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
            touchCancelEvent = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchleave mouseleave";
            this.model.browserInfo = browserInfo;

            this._on($(this.svgObject), touchMoveEvent, this._bulletMouseMove);
            this._on($(this.svgObject), touchCancelEvent, this._bulletMouseLeave);
            this._on($(this.svgObject), touchStartEvent, this._bulletMouseDown);
            this._on($(this.svgObject), touchStopEvent, this._bulletMouseUp);
			 
            if (this.model.enableResizing || this.model.isResponsive) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this._bulletResize);
                else
                    this._on($(window), "orientationchange", this._bulletResize);
            }
        },
       
        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
       
        _bulletResize: function () {
            var bullet = this,
				height = this.model.height,
				width = this.model.width;
            var $svgObj = $(bullet.svgObject);
            var height = 90;
            var width = (ej.isTouchDevice()) ? 250 : 595;
            var containerWidth = $(bullet.element).width();
            if (bullet.model.width)
                width = parseInt(bullet.model.width);
            else if (containerWidth > 0 && containerWidth < 595)
                width = containerWidth;
            else if(containerWidth > 595)
                width = 595;
            if (bullet.model.height)
                height = parseInt(bullet.model.height);
            var containerHeight = $(bullet.element).height() > height ? height : $(bullet.element).height();
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
				if(width < $(bullet.element).width())
					$svgObj.width(width);
				else
					$svgObj.width($(bullet.element).width());
                $svgObj.height(containerHeight);
                bullet.redraw();
            }, 500);
        },
		
        _bulletMouseLeave: function (evt) {
            if (!this.isTouch(evt)) {
                $('#tooltip').remove();
                $(".tooltipDiv" + this._id).remove();            
            }
		},
		isTouch: function (evt) {
		    var event = evt.originalEvent ? evt.originalEvent : evt;
		    if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
		        return true;
		    return false;
		},
       
        _bulletMouseMove: function (evt) {
            var currentVal, targetVal, categoryVal, measureId, targetId, targetClass;        
            targetId = $(evt.target).attr("id"),
            targetClass = $(evt.target).attr("class");           
            if (!this.isTouch(evt)) {
                $('#tooltip').remove();
                $(".tooltipDiv" + this._id).remove();
                this._elementTooltip(evt, targetClass, targetId);
                this._displayTooltip(evt, targetClass, targetId);
            }
         
        },

        _bulletMouseDown: function (evt) {
            if (this.isTouch(evt)) {
                evt.preventDefault();                 
                $('#tooltip').remove();
                $(".tooltipDiv" + this._id).remove();
                var targetId = $(evt.target).attr("id"),
                    targetClass = $(evt.target).attr("class");
                this._elementTooltip(evt, targetClass, targetId);
                this._displayTooltip(evt, targetClass, targetId);
            }
        },
        _bulletMouseUp: function (evt) {
            if (this.isTouch(evt)) {
                var bullet = this, element;
                window.clearTimeout(bullet.model.timer);
                if ($(".tooltipDiv" + this._id).length == 1)
                    element = $(".tooltipDiv" + this._id);
                if (element) {                   
                    bullet.model.trackerElement = element;
                    bullet.model.timer = setTimeout(function () {
                        if (bullet.model.trackerElement)
                            $(".tooltipDiv" + bullet._id).fadeOut(500, function () {
                                $(".tooltipDiv" + bullet._id).remove();
                                $("[id*=" + bullet.svgObject.id + "_FeatureMeasure_" + "]").attr("opacity", "1");
                            });
                    }, 1200);
                }
            }
        },

        _elementTooltip: function (evt,targetClass, targetId) {             
            var captionSettings = this.model.captionSettings,
             tooltipdiv = $("<div></div>").attr({ 'id': 'tooltip', "class": "tooltipDiv" + this._id }),
             pointer = this.mousePosition(evt);
             str = "";
            // adding css prop to the div
            $(tooltipdiv).css({
                "left": pointer.pageX + 10,
                "top": pointer.pageY + 10,
                "display": "block",
                "position": "absolute",
                "z-index": "13000",
                "cursor": "default",
                "font-family": "Segoe UI",
                "color": "#707070",
                "font-size": "12px",
                "background-color": "#FFFFFF",
                "border": "1px solid #707070"
            });
            if (targetClass == this.svgObject.id + "_Caption")
                str = evt.target.textContent == captionSettings.text ? "" : captionSettings.text;
            else if (targetClass == this.svgObject.id + "_SubTitle")
                str = evt.target.textContent == captionSettings.subTitle.text ? "" : captionSettings.subTitle.text;
            else if (targetClass == this.svgObject.id + "_Indicator")
                str = evt.target.textContent == captionSettings.indicator.text ? "" : captionSettings.indicator.text;
            if (str != "") {
                $(tooltipdiv).html("&nbsp" + str + "&nbsp");
                $(document.body).append(tooltipdiv);
            }
            else if (this.model.tooltipSettings.enableCaptionTooltip && (targetClass == this.svgObject.id + '_Caption' || targetClass == this.svgObject.id + '_SubTitle' || targetClass == this.svgObject.id + '_Indicator')) {
                var text = $('.' + targetClass).text();
                var data = { Text: text };
                var template = this.model.tooltipSettings.captionTemplate;
                if ($('.tooltipDiv' + this._id).length == 0) {
                    tooltipdiv = $("<div></div>").attr('class', 'tooltipDiv' + this._id).css({ 'position': 'absolute', 'z-index': '13000', 'display': 'block' });                   
                    $(document.body).append(tooltipdiv);
                    
                }
                if (template != "" && template != null) {
                    var cloneNode = document.getElementById(template);
                    cloneNode = (cloneNode ? $(cloneNode).clone() : $(template));
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    $(tooltipdiv).html($(cloneNode).render(data));
                } else {
                    $(tooltipdiv).html(text);
                }
                $(tooltipdiv).css('font-size', '12px');
                var xPos = pointer.clientX + $(document).scrollLeft() + 10;
                var yPos = pointer.clientY + $(document).scrollTop() + 10;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = pointer.clientX + $(document).scrollLeft() + 10;
                if (yPos === undefined || yPos === null)
                    yPos = pointer.clientY + $(document).scrollTop() + 10;
                $(tooltipdiv).css({
                    'left': xPos,
                    'top': yPos,
                    '-webkit-border-radius': "5px 5px 5px 5px",
                    '-moz-border-radius': "5px 5px 5px 5px",
                    '-o-border-radius': "5px 5px 5px 5px",
                    'border-radius': "5px 5px 5px 5px",
                    'background-color': "White",
                    'border': "1px Solid Black",
                    'padding-bottom': "5px",
                    'padding-left': "5px",
                    'padding-right': "5px",
                    'padding-top': "5px"
                });

            }
        },
        mousePosition: function (evt) {
            if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return evt;
            else if (evt.originalEvent && !ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX > 0)
                return evt.originalEvent;
            else if (evt.originalEvent && evt.originalEvent.changedTouches != undefined) {
                if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                    return evt.originalEvent.changedTouches[0];
            }
            return evt;

        },
        _displayTooltip: function (evt, targetClass, targetId) {
            if (targetClass != "undefined" && (targetClass == this.svgObject.id + '_FeatureMeasure' || targetClass == this.svgObject.id + '_ComparativeMeasure')) {
				var locale = this.model.locale, localizedText = locale && this.model.enableGroupSeparator, data;
                var bulletValue = (typeof (this.model.value) == 'function') ? this._value() : this.model.value,
                 bulletCMValue = (typeof (this.model.comparativeMeasureValue) == 'function') ? this._comparativeMeasureValue() : this.model.comparativeMeasureValue,
                 pointer = this.mousePosition(evt);
                measureId = targetId.substring(targetId.lastIndexOf("_") + 1);
                currentVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.featureMeasures] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].value) ? bulletValue : this._scale.featureMeasures[measureId].value);
                targetVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.comparativeMeasure] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].comparativeMeasureValue) ? bulletCMValue : this._scale.featureMeasures[measureId].comparativeMeasureValue);
                categoryVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? (ej.isNullOrUndefined(this.model.fields["dataSource"][measureId][this.model.fields.category])) ? null : this.model.fields["dataSource"][measureId][this.model.fields.category] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].category)) ? null : this._scale.featureMeasures[measureId].category;
				if (localizedText)
					data = {currentValue: currentVal.toLocaleString(locale), targetValue: targetVal.toLocaleString(locale), 
							category: (!ej.util.isNullOrUndefined(categoryVal) ? categoryVal.toLocaleString(locale) : categoryVal)};
				else
					data = {currentValue: currentVal, targetValue: targetVal, category: categoryVal};
                if ($('.tooltipDiv' + this._id).length == 0) {
                    tooltipdiv = $("<div></div>").attr('class', 'tooltipDiv' + this._id).css({ 'position': 'absolute', 'z-index': '13000', 'display': 'block' });                  
                    $(document.body).append(tooltipdiv);                   
                }
                if (this.model.tooltipSettings.template != "" && this.model.tooltipSettings.template != null) {
                    var cloneNode = $("#" + this.model.tooltipSettings.template).clone();
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    $(tooltipdiv).html($(cloneNode).render(data));
                } else {
                    var category = (!ej.isNullOrUndefined(data.category)) ? ("<br/> Category : " + data.category) : "";
                    $(tooltipdiv).html("Current : " + data.currentValue + "<br/> Target : " + data.targetValue + category);
                }
                $(tooltipdiv).css("font-size", "12px");
                var xPos = pointer.clientX + $(document).scrollLeft() + 10;
                var yPos = pointer.clientY + $(document).scrollTop() + 10;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = pointer.clientX + $(document).scrollLeft() + 10;
                if (yPos === undefined || yPos === null)
                    yPos = pointer.clientY + $(document).scrollTop() + 10;
                $(tooltipdiv).css({
                    'left': xPos,
                    'top': yPos,
                    '-webkit-border-radius': "5px 5px 5px 5px",
                    '-moz-border-radius': "5px 5px 5px 5px",
                    '-o-border-radius': "5px 5px 5px 5px",
                    'border-radius': "5px 5px 5px 5px",
                    'background-color': "White",
					'color':"black",
                    'border': "1px Solid Black",
                    'padding-bottom': "5px",
                    'padding-left': "5px",
                    'padding-right': "5px",
                    'padding-top': "5px"
                });
                if (targetClass == this.svgObject.id + '_FeatureMeasure')
                    $("#" + targetId).attr("opacity", "0.7");
            }          
            
        },

       
        redraw: function () {
            $(this.svgObject).empty();
            this._renderBulletElements();
        },
        
        destroy: function () {
            $(this.element).removeClass("e-bulletgraph e-js").find("#" + this.svgObject.id).remove();
        },
          
        setFeatureMeasureBarValue: function (index, measureValue) {
            this._scale = this.model.quantitativeScaleSettings;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasures.length;   
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.featureMeasures] = measureValue;
                else
                    this._scale.featureMeasures[index].value = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doAnimation();
            }
        },
          
        setComparativeMeasureSymbol: function (index, measureValue) {
            this._scale = this.model.quantitativeScaleSettings;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasures.length;   
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.comparativeMeasure] = measureValue;
                else
                    this._scale.featureMeasures[index].comparativeMeasureValue = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doLineAnimation();
            }
        }

    });
      
    ej.datavisualization.BulletGraph.Orientation = {
          
        Horizontal: "horizontal",
          
        Vertical: "vertical"
    };
      
    ej.datavisualization.BulletGraph.TickPlacement = {
          
        Inside: "inside",
          
        Outside: "outside"
    };
      
    ej.datavisualization.BulletGraph.LabelPlacement = {
          
        Inside: "inside",
          
        Outside: "outside"
    };
      
    ej.datavisualization.BulletGraph.Shape = {
          
        Circle: "circle",
          
        Cross: "cross",
          
        Diamond: "diamond",
          
        DownArrow: "downarrow",
          
        Ellipse: "ellipse",
          
        HorizontalLine: "horizontalLine",
          
        Image: "image",
          
        InvertedTriangle: "invertedtriangle",
          
        LeftArrow: "leftarrow",
          
        Pentagon: "pentagon",
          
        Rectangle: "Rectangle",
          
        RightArrow: "rightarrow",
          
        Star: "star",
          
        Trapezoid: "trapezoid",
          
        Triangle: "triangle",
          
        UpArrow: "uparrow",
          
        VerticalLine: "verticalline",
          
        Wedge: "wedge",
    };
      
    ej.datavisualization.BulletGraph.TickPosition = {
          
        Far: "far",
          
        Near: "near",
          
        Center: "center"
    };
      
    ej.datavisualization.BulletGraph.LabelPosition = {
          
        Below: "below",
          
        Above: "above"
    };
      
    ej.datavisualization.BulletGraph.FlowDirection = {
          
        Forward: "forward",
          
        Backward: "backward"
    };
      
    ej.datavisualization.BulletGraph.FontStyle = {
          
        Normal: "Normal",
          
        Italic: "Italic",
          
        Oblique: "Oblique"
    };
      
    ej.datavisualization.BulletGraph.FontWeight = {
          
        Normal: "Normal",
          
        Bold: "Bold",
          
        Bolder: "Bolder",
          
        Lighter: "Lighter"
    };
      
    ej.datavisualization.BulletGraph.Themes = {
          
        flatlight: {
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#191919" },
                minorTickSettings: { stroke: "#191919" },
                labelSettings: {
                    stroke: "#191919"
                },
                featuredMeasureSettings: { stroke: "#191919" },
                comparativeMeasureSettings: { stroke: "#191919" }
            },
            qualitativeRanges: [{
                rangeStroke: "#ebebeb"
            }, {
                rangeStroke: "#d8d8d8"
            }, {
                rangeStroke: "#7f7f7f"
            }],
            captionSettings: {
                font: { color: "#191919" },
                subTitle: {
                    font: { color: "#191919" }
                },
                indicator: {
                    font: { color: "#191919" },
                    symbol: { color: "#191919" }
                }
            }
        },
        material: {
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#333333" },
                minorTickSettings: { stroke: "#191919" },
                labelSettings: {
                    stroke: "#333333"
                },
                featuredMeasureSettings: { stroke: "#333333" },
                comparativeMeasureSettings: { stroke: "#333333" }
            },
            qualitativeRanges: [{
                rangeStroke: "#ebebeb"
            }, {
                rangeStroke: "#d8d8d8"
            }, {
                rangeStroke: "#7f7f7f"
            }],
            captionSettings: {
                font: { color: "#333333" },
                subTitle: {
                    font: { color: "#333333" }
                },
                indicator: {
                    font: { color: "#333333" },
                    symbol: { color: "#333333" }
                }
            }
        },
          
        flatdark: {
              
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#ffffff" },
                minorTickSettings: { stroke: "#ffffff" },
                labelSettings: {
                    stroke: "#ffffff"
                },
                featuredMeasureSettings: { stroke: "#ffffff" },
                comparativeMeasureSettings: { stroke: "#ffffff" }
            },
            qualitativeRanges: [{
                rangeStroke: "#b3b3b3"
            }, {
                rangeStroke: "#999999"
            }, {
                rangeStroke: "#4d4d4d"
            }],
            captionSettings: {
                font: { color: "#ffffff" },
                subTitle: {
                    font: { color: "#ffffff" }
                },
                indicator: {
                    font: { color: "#ffffff" },
                    symbol: { color: "#ffffff" }
                }
            }
        }
    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html RangeNavigator elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    
    ej.widget("ejRangeNavigator", "ej.datavisualization.RangeNavigator", {

        ejChart: "",

        validTags: ["div"],
       
        defaults: {

            theme: "",

            padding: "0",

            enableAutoResizing: false,

            isResponsive: false,

            allowSnapping: false,


            sizeSettings: {

                width: "",

                height: ""
            },
            _size: {

            },

            locale: "en-US",

            valueType: "datetime",

            valueAxisSettings: {

                rangePadding: 'none',

                range: {

                    min: null,

                    max: null,

                    interval: null
                },

                axisLine:
                    {

                        visible: false
                    },

                font: {

                    size: '0px',
                },

                majorTickLines:
                {

                    width: 0,

                    size: 0,

                    visible: true
                },

                majorGridLines: {

                    visible: false
                },

                visible: false
            },

            rangePadding: "none",

            enableRTL: false,

            enableScrollbar: false,

            dataSource: "",

            xName: "x",

            yName: "y",

            tooltipSettings: {

                visible: true,

                labelFormat: "MM/dd/yyyy",

                tooltipDisplayMode: "always",

                backgroundColor: "#303030",

                font:
                {

                    color: '#FFFFFF',

                    family: 'Segoe UI',

                    fontStyle: 'Normal',

                    size: '10px',
                    opacity: 1,

                    weight: 'regular'
                }
            },

            zoomPosition: "0",

            zoomFactor: "1",

            selectedRangeSettings: {

                start: "",

                end: ""
            },

            selectedData: "",

            rangeSettings: {

                start: "",

                end: ""
            },

            border: {

                width: 1,

                color: null,

                opacity: 1
            },

            scrollRangeSettings: {

                start: "",

                end: ""
            },

            enableDeferredUpdate: true,

            series: "",

            seriesSettings: "",

            labelSettings: {

                style: {

                    font: {

                        color: '#333333',

                        family: 'Segoe UI',

                        style: 'Normal',

                        size: '10px',

                        opacity: 1,

                        weight: 'regular'
                    },

                    horizontalAlignment: "middle"
                },


                higherLevel: {

                    intervalType: null,

                    style: {

                        font: {

                            color: "black",

                            fontFamily: "Segoe UI",

                            fontStyle: "Normal",

                            size: "12px",

                            opacity: 1,

                            fontWeight: "regular"
                        },

                        horizontalAlignment: "middle"
                    },

                    gridLineStyle: {

                        color: "#B5B5B5",

                        width: 1,

                        dashArray: "20 5 0"
                    },


                    border:
                    {

                        color: "transparent",

                        width: 0.5
                    },

                    fill: "transparent",

                    position: "top",

                    visible: true,

                    labelPlacement: "outside"
                },

                lowerLevel: {

                    intervalType: null,

                    style: {

                        font: {

                            color: "black",

                            fontFamily: "Segoe UI",

                            fontStyle: "Normal",

                            size: "12px",

                            opacity: 1,

                            fontWeight: "regular"
                        },

                        horizontalAlignment: "middle"
                    },

                    gridLineStyle:
                    {
                        color: "#B5B5B5",

                        width: 1,

                        dashArray: ""
                    },

                    border:
                    {

                        color: "transparent",

                        width: 0.5
                    },

                    fill: "transparent",

                    position: "bottom",

                    visible: true,

                    labelPlacement: "outside"
                }
            },

            navigatorStyleSettings: {

                selectedRegionColor: "#EFEFEF",

                unselectedRegionColor: "#5EABDE",

                thumbColor: "#2382C3",

                thumbRadius: 10,

                leftThumbTemplate: null,

                rightThumbTemplate: null,

                thumbStroke: "#303030",

                background: "#dddddd",

                border:
                {

                    color: "transparent",

                    width: 0.5,

                    dashArray: ""
                },

                opacity: 1,

                selectedRegionOpacity: 0,

                unselectedRegionOpacity: 0.3,

                highlightSettings: {

                    enable: false,

                    opacity: 0.5,

                    color: '#006fa0',

                    border: {

                        color: '',

                        width: 1
                    }

                },

                selectionSettings: {

                    enable: false,

                    opacity: 0.5,

                    color: '#0e4a7c',

                    border: {

                        color: '',

                        width: 1
                    }

                },

                majorGridLineStyle:
                {

                    color: "#B5B5B5",

                    visible: true
                },

                minorGridLineStyle:
                {

                    color: "#B5B5B5",

                    visible: true
                },
            },


            loaded: "",

            load: "",

            rangeChanged: "",

            scrollEnd: "",

            scrollStart: "",

            scrollChanged: "",

            selectedRangeStart: "",
            selectedRangeEnd: ""
        },

        dataTypes: {
        
            enableAutoResizing:"boolean",
            isResponsive: "boolean",
            allowSnapping:"boolean",
         
            dataSource: "data",
            tooltipSettings: {
                visible:"boolean",
                labelFormat:"string",
                tooltipDisplayMode:"string",
            },
            
            zoomPosition: "string",
            zoomFactor: "string",
           
            selectedData: "string",
            enableDeferredUpdate: "boolean",
            series: "array",
           
            labelSettings: {
                higherLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement: "string"
                },
                lowerLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement:"string"
                }
            },
            navigatorStyleSettings: {
                thumbRadius:"number",
            },
       
        },

        observables: ["selectedRangeStart", "selectedRangeEnd"],

         _tags: [{
                tag: "series", 
                attr: ["xName", "yName", "dataSource", "enableAnimation",
				[
				     {
				       tag: "points", attr:["x", "y", "text", "isEmpty", "fill", "visible"]
				     }
				 ]				
				],
				 singular:"series"
				                
            } 
         ],

          _selectedRangeStart: ej.util.valueFunction("selectedRangeStart"),
          _selectedRangeEnd: ej.util.valueFunction("selectedRangeEnd"),

		 _series: function (index, property, value, old) {		   
		  this.renderNavigator();
		  this._trigger("refresh");           
        },
		_series_points:function(index, property, value, old){		   
		  this.renderNavigator();
		  this._trigger("refresh");    
		},


        _themes: {
            flatlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1 ,weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {                        
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0},
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width:1, dashArray:"15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width:0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity:1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity:0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            azurelight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1,dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            limelight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#A9CA44 ",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            saffronlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientazure: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlime: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI',style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientsaffron: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            flatdark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {                        
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width:0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            material: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#333333', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#333333', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1 },
                    },
                    lowerLevel: {
                        style: { font: { color: '#333333', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#f5f5f5",
                    unselectedRegionColor: "#8d8afd",
                    thumbColor: "#8d8afd",
                    thumbRadius: 10,
                    thumbStroke: "#000000",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },
            office: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#333333', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#333333', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#333333', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#eaeaea",
                    unselectedRegionColor: "#0078d7",
                    thumbColor: "#0078d7",
                    thumbRadius: 10,
                    thumbStroke: "#000000",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.6,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            highcontrast01: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#008000",
                    thumbColor: "#008000",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.6,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },
            highcontrast02: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#0000ff",
                    thumbColor: "#0000ff",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.6,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            azuredark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            limedark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#A9CA44",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },
            
            saffrondark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI',style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientdark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI',style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientazuredark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientlimedark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1,weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientsaffrondark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI',style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

        },
        /**
                * To configure the properties at runtime using SetModel		
                * @private
                */
        _setModel: function (options) {
            for (var prop in options) {
              //  this.disableAnimation();
                switch (prop) {                   
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this._setTheme(this._themes, this.model.theme);
						break;
                    case "selectedRangeStart":
                        this.model.selectedRangeSettings.start = this._selectedRangeStart();
                        break;
                    case "selectedRangeEnd":
                        this.model.selectedRangeSettings.end = this._selectedRangeEnd();
                        break;
                    default:
                        $.extend(true, this.model, {}, options[prop]);
                }
            }
          //  $(this.svgDocument).empty();		  
            this.bindTo();
			this.renderNavigator(this);
			this._bindevents();
        },
     
        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },


        _createSvg: function (element) {

            this.svgSupport = (window.SVGSVGElement) ? true : false;
            this.vmlRendering = !this.svgSupport;
            var bwr = jQuery.uaMatch(navigator.userAgent);
            this.brow = bwr.browser.toLowerCase();
            if (this.svgSupport) {
                //rendering object created
                this.renderer = new ej.EjSvgRender(this.element);
                //Creating svg document
                this.svgns = this.renderer.svgLink;
                this.svgDocument = this.renderer.svgObj;
            }
            else {
                //rendering object created
                this.renderer = new ej.EjVmlRender(this.element);
                //Creating VML document
                this.svgDocument = this.renderer.svgObj;
            }

        },
        _setSVGSize: function () {

            //initializing minimum height and width for the control
            var type = this.model.valueType, height, rangeModel = this.model, size = rangeModel.sizeSettings,
                fontsize = parseFloat(rangeModel.labelSettings.higherLevel.style.font.size);
            if (type == "datetime") {
                this.minHighHeight = fontsize < 20 ?
                    20 :
                    fontsize + 2;
                fontsize = parseFloat(this.model.labelSettings.lowerLevel.style.font.size);
            }
            else
                this.minHighHeight = 0;
            this.minLowHeight = fontsize < 20 ?
                20 :
                fontsize + 2;
            this.scrollbarLoaded = (this.model.scrollObj && this.model.scrollObj[0].scrollbarLoaded) ? true : false;
            //Scrollbar height included on calculating rangenavigator height on update scrollbar with dynamically
            var scrollHeight = this.model.enableScrollbar && this.scrollbarLoaded && this.svgSupport ? 18 : 0;
            height = ($(this.element).height()) ? ($(this.element).height() > this.newHeight ? $(this.element).height() - scrollHeight : $(this.element).height()) :
                      this.newHeight == undefined ? (type == "datetime" ? 120 : 100) : this.newHeight;
            this.minCenterHeight = parseFloat(height) - parseFloat(this.minLowHeight) - parseFloat(this.minHighHeight);

            height = (height <= 0) ? 120 : height;
            this.minHeight = height;
            this.padding = parseFloat(rangeModel.padding);

            //calculating the height and width of the control
            this.newWidth = size.width ? size.width : $(this.element).width();
            this.newHeight = size.height ? size.height :
                ($(this.element).height() && this.newHeight == undefined) > this.minHeight ? $(this.element).height() - scrollHeight : this.minHeight;

            //allocate space for display scrollbar in range control
            this.scrollWidth = this.newWidth - this.padding * 2;
        },
        _setTheme: function (jsonObj, themeName) {
            var name = themeName.toLowerCase();
            if ((name.indexOf("high") >= 0) && (name.indexOf("01") >= 0))
                jsonObj[name] = $.extend(true, {}, jsonObj["highcontrast01"], jsonObj[name]);
            else if ((name.indexOf("high") >= 0) && (name.indexOf("02") >= 0))
                jsonObj[name] = $.extend(true, {}, jsonObj["highcontrast02"], jsonObj[name]);
            else if ((name.indexOf("material") >= 0))
                jsonObj[name] = $.extend(true, {}, jsonObj["material"], jsonObj[name]);
            else if ((name.indexOf("office") >= 0))
                jsonObj[name] = $.extend(true, {}, jsonObj["office"], jsonObj[name]);

            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        },

        _setPositions: function () {
            
            var higherLevel = this.model.labelSettings.higherLevel,
                lowerLevel = this.model.labelSettings.lowerLevel,
                higherposition = higherLevel.position,
                lowerposition = lowerLevel.position,
                highervisible = higherLevel.visible,
                lowervisible = lowerLevel.visible,
                higherlabelplace = higherLevel.labelPlacement,
                lowerlabelplace = lowerLevel.labelPlacement;
            //to initialize the positions of the sliders, labelbars and chart
            if (higherposition === "top" && lowerposition === "bottom") {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;                    
            }
            else if ((higherposition === "bottom" && lowerposition === "bottom")) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;

            }
            else if (higherposition === "bottom" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + 1;
                topPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;
            }
            else if (higherposition === "top" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }

            if (highervisible === false && lowervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight;
                this.bottomPostion = this.newHeight;
            }
            else if (highervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }
            else if (lowervisible === false) {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;
            }

            if (higherlabelplace === "outside" && lowerlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.centerHeight;
            }
            else if (higherlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.newHeight;
            }
            else if (lowerlabelplace === "outside") {
                this.sliderPosition = 0;
                this.sliderHeight = this.centerHeight + this.minHighHeight;
            }
            else {
                this.sliderPosition = 0;
                this.sliderHeight = this.newHeight;
            }
            
        },

        //adding redraw method

        redraw: function (evt) {

            //Set virtual range and enable scrollbar on redraw
            var virtualRange = this.model.scrollRangeSettings,rangeSettings = this.model.rangeSettings, scrollRange, start = virtualRange.start,
                end = virtualRange.end;//,actualScrollRange = this._scrollRangeSettings;
            this.loadScrollbar = false
            if (this.model.enableScrollbar) {
                this.scrollUpdate = true;
                scrollRange = this._scrollRange;
                if (this.model.valueType.toLowerCase() == "datetime") {
                    start = Date.parse(virtualRange.start);
                    end = Date.parse(virtualRange.end);
                    rangeStart = Date.parse(rangeSettings.start);
                    rangeEnd = Date.parse(rangeSettings.end);
                    //actualStart = Date.parse(actualScrollRange.start);
                    //actualEnd = Date.parse(actualScrollRange.end);
                }
                if (!scrollRange || rangeStart < scrollRange.min || scrollRange.max < rangeEnd || scrollRange.min != start || scrollRange.max != end || ej.isNullOrUndefined(this.scrollsvgObj)) {
                    //Load new scrollbar or update for the dynamic virtualRange changes
                    this.loadScrollbar = true;
                    this.scrollbarLoaded = false;
                    $(this.scrollsvgObj).remove();
                }
            }

            this.renderNavigator(this);

        },

        renderNavigator: function (evt) {
            $(this.svgDocument).empty();
            var locale = this.model.locale;
            this.culture = ej.preferredCulture(locale);
            this._localizedLabels = this._getLocalizedLabels();
            this._initializeVariables.call(this, this.model, this.model.series, this.model.seriesSettings);

            this._rootId = $(this.element).attr("id");
            this.leftPadding =parseFloat($("#"+this._rootId).css('padding-left'));

            this._setSVGSize.call(this);
            var type = this.model.valueType;
            //initializing the higherlevel and lowerlevel collections
            this._higherTextNode = [];
            this._higherTotalValues = [];
            this._higherLineLeft = [];
            this._higherTextLeft = [];
            this._lowerTextNode = [];
            this._lowerTotalValues = [];
            this._lowerLineLeft = [];
            this._lowerTextLeft = [];

            //inizialize the higher level and lower level labels regions
            this.labelRegions =[];
            
            //Checking selection and highlight enable
            var labelSettings = this.model.labelSettings,
                higherLevel = labelSettings.higherLevel,
                lowerLevel = labelSettings.lowerLevel,
                valueType = this.model.valueType;

            //calculating positions
            this.centerPosition=0;
            this.bottomPosition = 0;
            this.centerHeight = 0;
            this.sliderPosition = 0;
            this.sliderHeight = 0;
            
            //set positions
            this._setPositions();

            //set initial width and height for the svg document
            this.svgDocument.setAttribute('style', 'overflow:visible;position:relative;display:block');
            if (this.svgSupport) {
                this.svgDocument.setAttribute("height", this.newHeight);
                this.svgDocument.setAttribute("width", this.newWidth);
            }
            else {
                $(this.svgDocument).width(this.newWidth);
                $(this.svgDocument).height(this.newHeight);
            }
            this.model._size.width = this.newWidth;
            this.model._size.height = this.newHeight;
            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.data = { model: this.model };
            this._trigger("loaded", commonLoadedEventArgs);
            if (this.model.theme != "")
                this._setTheme(this._themes, this.model.theme);
            //create gradient
            var navigatorStyleSettings = this.model.navigatorStyleSettings;
            var uncolor = navigatorStyleSettings.unselectedRegionColor;
            this.unselectgrad = this.renderer.createGradientElement('unselected', uncolor, 150, 0, 150, 100, this.svgDocument);
            var navibackcolor = navigatorStyleSettings.background;
            this.naviback = this.renderer.createGradientElement('naviback', navibackcolor, 150, 0, 150, 100, this.svgDocument);
            var selcolor = navigatorStyleSettings.selectedRegionColor;
            this.selectedgrad = this.renderer.createGradientElement('selected', selcolor, 150, 0, 150, 100, this.svgDocument);
            var tbColor = navigatorStyleSettings.thumbColor;
            this.tbClr = this.renderer.createGradientElement('thumbClr', tbColor, 150, 0, 150, 100, this.svgDocument);
            
            var border = this.model.border;
            var borderColor = border.color ? border.color :(this.model.enableScrollbar? "#B4B4B4": "transparent");
            //Creating layout for navigator
            var naviLayoutOptions = {
                'id': this._rootId + '_Layout',
                'x': this.padding,
                'width': this.newWidth > 0 ? this.newWidth - this.padding * 2 : 0,
                'height': this.newHeight,
                'fill': 'transparent',
                'stroke-width': border.width,
                'stroke': borderColor,
                'opacity': border.opacity
            };

            this.renderer.drawRect(naviLayoutOptions, this.svgDocument);
            var areaHeight;
            if (type == "numeric")
                areaHeight = this.newHeight - navigatorStyleSettings.border.width - this.minLowHeight;
            else
                areaHeight = this.newHeight - navigatorStyleSettings.border.width - this.minLowHeight - this.minHighHeight;
            //Creating border for navigator
            var naviBorderOptions = {
                'id':this._rootId+'_Border',
                'x':this.padding,
                'y': this.sliderPosition,
                'width': this.newWidth >0 ? this.newWidth-this.padding * 2: 0,
                'height': this.sliderHeight,
                'fill': this.naviback,
                'stroke-width': navigatorStyleSettings.border.width,
                'stroke': navigatorStyleSettings.border.color,
                'opacity': navigatorStyleSettings.opacity,
                'stroke-dasharray': navigatorStyleSettings.border.dashArray,
                
            }

            this.renderer.drawRect(naviBorderOptions, this.svgDocument);

            //creating g tags to group higherlevel and lowerlevel elements(txt,line).
            if ($("#" + this._rootId + 'higherLevel').length == 0)
                this.higherLevel = this.renderer.createGroup({ 'id': this._rootId + 'higherLevel' });
            if ($("#" + this._rootId + 'lowerLevel').length == 0)
                this.lowerLevel = this.renderer.createGroup({ 'id': this._rootId + 'lowerLevel' });
            this.centerLevel = this.renderer.createGroup({ 'id': this._rootId + 'centerLelvel' });
            this.higherLevel.height = this.minHighHeight;
            this.lowerLevel.height = this.minLowHeight;

            //creating g tags to group tooltipSettings
            if (this.model.tooltipSettings.visible && this.svgSupport) {
                this.leftTooltip = this.renderer.createGroup({ 'id': this._rootId + 'leftTooltip' });
                this.rightTooltip = this.renderer.createGroup({ 'id': this._rootId + 'rightTooltip' });

                var leftToolOptions = {
                    'fill': this.model.tooltipSettings.backgroundColor,
                    'd': 'M 0 0 L 0 21 L 64 21 L 64 6 L 70 0 Z'
                };
                this.renderer.drawPath(leftToolOptions, this.leftTooltip);
                var tooltipoptions = this.model.tooltipSettings.font;
                

                var textOptions = {
                    'id': this._rootId + '_LeftToolText',
                    'x': 5,
                    'fill': tooltipoptions.color,
                    'font-size': tooltipoptions.size,
                    'font-family': tooltipoptions.family,
                    'font-style': tooltipoptions.style,
                    'font-weight': tooltipoptions.weight,
                    'text-anchor': 'start',
                    'opacity': tooltipoptions.opacity,
                    'dominant-baseline': 'middle',
                    
                    
                };
                this.renderer.drawText(textOptions, ej.format(new Date(1 / 1 / 2000), "MMMM, yyyy", locale), this.leftTooltip);
                this.leftTxt = this.leftTooltip.childNodes[1];
                var rightToolOptions = {
                    'fill': this.model.tooltipSettings.backgroundColor,
                    'd': 'M 0 0 L 70 0 L 70 21 L 6 21 L 6 6 Z',
                    'horizontal-alignment': 'stretch'
                };
                this.renderer.drawPath(rightToolOptions, this.rightTooltip);
                this.renderer.drawText(textOptions, ej.format(new Date(1 / 1 / 2000), "MMMM, yyyy", locale), this.rightTooltip);
                this.rightTxt = this.rightTooltip.childNodes[1];

             
            }
            //creating g tag for holding chart
            this.chartGView = this.renderer.createGroup({ 'id': this._rootId + 'charView' });
            
            this.renderer.append(this.chartGView, this.svgDocument);
            if (this.model.navigatorStyleSettings.highlightSettings.enable || this.model.navigatorStyleSettings.selectionSettings.enable) {
                this.styleRect = this.renderer.createGroup({ 'id': this._rootId + 'highlightRect' });
                this.renderer.append(this.styleRect, this.svgDocument);
            }
            if(this.model.valueType=="datetime")
            this.renderer.append(this.higherLevel, this.svgDocument);
            this.renderer.append(this.lowerLevel, this.svgDocument);
            this.renderer.append(this.centerLevel, this.svgDocument);
            
            //creating g tag for sliders
            this.leftUnSelected=this.renderer.createGroup({'id':'unselectleft'});
            this.gLeftSlider = this.renderer.createGroup({ 'id': 'leftslider' });
            this.gRightSlider = this.renderer.createGroup({ 'id': 'rightslider' });
            this.gCenterSlider = this.renderer.createGroup({ 'id': 'centerslider' });
            this.rightUnSelected = this.renderer.createGroup({ 'id': 'unselectright' });

            var unLeftSliderOptions = {
                'width': this.newWidth > 0 ? this.newWidth - this.padding * 2 : 0,
                'height': this.sliderHeight,
                'fill':this.unselectgrad,
                'opacity': navigatorStyleSettings.unselectedRegionOpacity,
                'transform': 'translate('+this.padding+',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unLeftSliderOptions, this.leftUnSelected);
            this.leftUnArea = this.leftUnSelected.firstChild;
            this.renderer.append(this.leftUnSelected, this.svgDocument);

            var centerSliderOptions = {
                'width': this.newWidth > 0 ? this.newWidth - 4 - this.padding * 2 : 0,
                'height': this.sliderHeight,
                'fill': this.selectedgrad,
                'opacity': navigatorStyleSettings.selectedRegionOpacity,
                'transform': 'translate('+this.padding+','+this.sliderPosition+')'
            };

            this.renderer.drawRect(centerSliderOptions, this.gCenterSlider);
            this.centerSlider = this.gCenterSlider.firstChild;
            $(this.centerSlider).css('cursor', 'pointer');

            this.renderer.append(this.gCenterSlider, this.svgDocument);
            var leftSliderOptions = {                 
                'width': '2',
                'height': this.sliderHeight,
                'fill': navigatorStyleSettings.thumbStroke,
                'opacity': '1',
                'y': this.sliderPosition,
                'transform': 'translate(' + this.padding + ')'
               
                
            };
            var lcircleOptions = {
                'id': 'leftCircle' + this._id,
                'cx': this.padding,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': navigatorStyleSettings.thumbRadius,
                'stroke-width': 2,
                'stroke': navigatorStyleSettings.thumbStroke,
                'fill': this.tbClr
            };

                
            this.renderer.drawRect(leftSliderOptions, this.gLeftSlider);
			// checking condition for whether using thumb
            if (this.model.navigatorStyleSettings.leftThumbTemplate == null) {
                this.renderer.drawCircle(lcircleOptions, this.gLeftSlider);
                this.leftCircle = this.gLeftSlider.lastChild;
                $(this.leftCircle).css('cursor', 'pointer');
            }

            // Adding left Thumb  

            else {
                var isID = document.getElementById(this.model.navigatorStyleSettings.leftThumbTemplate) ? true : false;
                if ($('#' + this._id + "thumbleft").length > 0)
                    $('#' + this._id + "thumbleft").remove();
                var container = this._id;
				if (($("#" + container).css('transform')) == 'none')
                    $("#" + container).css('transform', 'translate(0, 0)');
                if ($("#" + container+"template_group_").length != 0)
                    var templateContainer = $("#" + container + "template_group_");
				else
				{
                   var templateContainer = $("<div></div>").attr('id',container+"template_group_");
				    $(templateContainer).css("position", "relative");
				}
                var temp = $("<div></div>").attr("id", this._id + "thumbleft");
                var cloneNode;
                if (isID == true)
                    cloneNode = $("#" + this.model.navigatorStyleSettings.leftThumbTemplate)[0].innerHTML;
                else
                    cloneNode = this.model.navigatorStyleSettings.leftThumbTemplate;
                var $cloneNode = $(cloneNode);
                $(temp).css("position", "absolute");
                $cloneNode.appendTo($(temp));
                $(temp).appendTo(templateContainer);
                if (this.svgSupport) 
					$(templateContainer).appendTo('#' + container);
                else
                    $(templateContainer).appendTo(this.svgDocument);
            	var top = this.sliderPosition + (this.sliderHeight / 2) - (($cloneNode.height()) / 2);
				$(temp).css("top", top).css('cursor', 'pointer').css('z-index','1');
			}
            this.leftSlider = this.gLeftSlider.firstChild;
            $(this.leftSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gLeftSlider, this.svgDocument);

			var width = parseFloat(this.newWidth - this.padding*2); // To check width on initialize the range navigator.
			width = width < 0 ? 0 : width;
			
            var unRightSliderOptions = {
                'width': width,
                'height': this.sliderHeight,
                'fill': this.unselectgrad,
                'opacity': navigatorStyleSettings.unselectedRegionOpacity,
                'transform': 'translate('+this.padding+',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unRightSliderOptions, this.rightUnSelected);
            this.rightUnArea = this.rightUnSelected.firstChild;
            this.renderer.append(this.rightUnSelected, this.svgDocument);
            var rightSliderOptions = {               
                'width': '2',
                'height': this.sliderHeight,
                'fill': this.model.navigatorStyleSettings.thumbStroke,
                'opacity': '1',
                'transform': 'translate(' + parseFloat(this.newWidth - 4-this.padding) + ', ' + this.sliderPosition + ' )'
            };
            var rcircleOptions = {
                'id': "rightCircle" + this._id,
                'cx': this.newWidth - 4 - this.padding,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': this.model.navigatorStyleSettings.thumbRadius,
                'stroke-width': 2,
                'stroke': this.model.navigatorStyleSettings.thumbStroke,
                'fill': this.tbClr
            };
            this.renderer.drawRect(rightSliderOptions, this.gRightSlider);
            if (this.model.navigatorStyleSettings.rightThumbTemplate == null)
            {
                this.renderer.drawCircle(rcircleOptions, this.gRightSlider);
                this.rightCircle = this.gRightSlider.lastChild;
                $(this.rightCircle).css('cursor', 'pointer');
            }

            // append right thumb   

            else {
                var isID = document.getElementById(this.model.navigatorStyleSettings.rightThumbTemplate) ? true : false;
                if ($('#' + this._id + "thumbright").length > 0)
                    $('#' + this._id + "thumbright").remove();
                var container = this._id;
                if ($("#" + container + "template_group_").length != 0)
                    var templateContainer = $("#" + container + "template_group_");
                else
				 {
                    var templateContainer = $("<div></div>").attr('id',container+ "template_group_");
					 $(templateContainer).css("position", "absolute");
				 }
                var temp = $("<div></div>").attr("id", this._id + "thumbright");
                var cloneNode;
                if (isID == true)
                    cloneNode = $("#" + this.model.navigatorStyleSettings.rightThumbTemplate)[0].innerHTML;
                else
                    cloneNode = this.model.navigatorStyleSettings.rightThumbTemplate;
                var $cloneNode = $(cloneNode);
                $(temp).css("position", "absolute");
                $cloneNode.appendTo($(temp));
                $(temp).appendTo(templateContainer);
                if (this.svgSupport) 
					$(templateContainer).appendTo('#' + container);
                else
                    $(templateContainer).appendTo(this.svgDocument);
                var top = this.sliderPosition + (this.sliderHeight / 2) - (($cloneNode.height()) / 2);
                $(temp).css("top", top).css('cursor', 'pointer').css('z-index', '1');
             }

            this.rightSlider = this.gRightSlider.firstChild;
            $(this.rightSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gRightSlider, this.svgDocument);
            if (this.model.tooltipSettings.visible && this.svgSupport) {
                this.renderer.append(this.leftTooltip, this.svgDocument);
                this.renderer.append(this.rightTooltip, this.svgDocument);
            }
            
            this.element.append(this.svgDocument);
            if (this.svgSupport) {
                this.trueCoords = this.svgDocument.createSVGPoint();

                this.grabPoint = this.svgDocument.createSVGPoint();
            }

            this.backDrop = this.element;
            this.dragTarget = null;            
            if(this.sliderHeight > 0)
                 this._renderChart.call(this, this.model);
            if (evt != undefined && evt.target != undefined)
                this.resize = true;
            
            if ((this.startDateTime != undefined && this.endDateTime != undefined) || (this.startValue != undefined && this.endValue != undefined)) {
                this.calculateInterval.call(this, this.model.labelSettings.higherLevel, this.model.labelSettings.lowerLevel);

                this.setSliders();
            } else {
                if (!this.svgSupport)
                    this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                else
                    this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + this.padding + ',' + 0 + ')' });

                this.leftTooltip.setAttribute('opacity', 0);
                this.rightTooltip.setAttribute('opacity', 0);
            }
            if (this.model.tooltipSettings.visible && this.svgSupport && this.model.tooltipSettings.tooltipDisplayMode == "ondemand") {
                this.rightTooltip.setAttribute('opacity', 0);
                this.leftTooltip.setAttribute('opacity', 0);
            }
            this._scrollChanged = this._scrollChanged ? false : this._scrollChanged;
            //this._trigger("loaded", this.model);

            //Add or remove scrollbar dynamically
            if (!this.model.enableScrollbar) {
                $(this.scrollsvgObj).remove();
                this.scrollsvgObj = null;
                this.scrollZoomPos = null;
                this.scrollZoomFact = null;
            } else {
                if ((this.loadScrollbar && !this.scrollbarUpdate) || (this.model.scrollObj && this.model.scrollObj[0].width != this.scrollWidth)) {
                    this.model.scrollObj[0].width = this.scrollWidth;
                    this.scrollbarContainer = new ej.EjSvgScrollbarRender(this.element, this.model.scrollObj[0]);
                    this.scrollbarContainer._initializeScrollbarVariables.call(this, this.model.scrollObj[0]);
                    this.scrollbarContainer._renderScrollbar.call(this, this.model.scrollObj[0]);

                } else
                    this.renderer.append(this.scrollsvgObj, this.element);
                this._bindevents();
            }
        },

        eachInterval:"",

        //Rendering chart
        _renderChart: function (model) {
            var valueType = model.valueType.toLowerCase();
            var range, rangeSettings = model.rangeSettings;
            if (model.enableScrollbar || (rangeSettings.start != "" && rangeSettings.end != "")) {
                if (valueType == "datetime")
                    range = { min: this.startDateTime, max: this.endDateTime };
                else
                    range = { min: this.startValue, max: this.endValue };
            }
            if (model.dataSource != "" && model.series == "" && model.seriesSettings == "") {
                $(this.chartGView).ejChart(
                {
				    locale : this.model.locale,
                    border: { width: 0, color: "transparent" },
                    margin: { left: 0, right: 0, top: 0, bottom: 0 },
                    elementSpacing: 0,
                    chartArea:
                    {
                        background: "transparent",
                        border: { width: 0 }
                    },
                    primaryXAxis: {
                        axisLine: { visible: false },
                        font: { size: '0px' },
                        range:range,
                        majorTickLines:
                        {
                            lineWidth: 0,
                            size: 0,
                            visible: true
                        },
                        majorGridLines: { visible: false },
                        rangePadding: this.model.rangePadding,
                        visible: false,
                        isInversed: this.model.enableRTL

                    },
                    primaryYAxis: this.model.valueAxisSettings,
                 series: [{
                     dataSource: model.dataSource, xName: model.xName, yName: model.yName,
                     type: 'line',
                     width:1.5,
                     enableAnimation:false

                 },
                 ],
                 size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                 legend: {
                     visible: false,
                     itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                 }
             });
            }
            else if (model.dataSource != "" && model.series != "" && model.series[0].dataSource == undefined || model.seriesSettings != "" && model.seriesSettings.dataSource == undefined && model.dataSource!="") {
                model.seriesSettings.dataSource = model.dataSource;
                model.seriesSettings.xName = model.seriesSettings.xName ? model.seriesSettings.xName : model.xName;
                model.seriesSettings.yName = model.seriesSettings.yName ? model.seriesSettings.yName : model.yName;
                var series = model.series;
                if (series == "") {
                    series = [{
                        dataSource: model.dataSource, xName: model.seriesSettings.xName, yName: model.seriesSettings.yName,
                        type: 'line',
                        width: 1.5,
                        enableAnimation: false

                    }];
                }
               
                $(this.chartGView).ejChart(
            {
			    locale : this.model.locale,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px' },
                    range:range,
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.enableRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }
            else if (model.series != "" && model.series[0].dataSource != undefined || model.seriesSettings != "" && model.seriesSettings.dataSource != undefined) {
                if (model.seriesSettings == "") {
                    model.seriesSettings = {
                        type: 'line'
                    };
                }
                var series = model.series;
                var length = series.length;
                if (series == "") {
                    series = [{
                        dataSource: model.seriesSettings.dataSource, xName: model.seriesSettings.xName, yName: model.seriesSettings.yName,
                        type: 'line',
                        width: 1.5,
                        enableAnimation: false

                    }];
                }
                
                $(this.chartGView).ejChart(
            {
			    locale : this.model.locale,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    range:range,
                    font: { size: '0px' },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.enableRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }

            this.ejChart = $(this.chartGView).data("ejChart");
            this.renderer._setAttr($(this.chartGView), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')','height': this.sliderHeight });
           
        },

        _init: function () {
            this._createSvg(this);
            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs), virtualRange;
            commonLoadedEventArgs.data = { model: this.model };
            this._trigger("load", commonLoadedEventArgs);
            virtualRange = this.model.scrollRangeSettings;

            this.renderNavigator(this);
            if (this.model.enableScrollbar) {
                this.model.scrollObj = [];
                var scrollObj = this.model.scrollObj;
                scrollObj[0] = {
                   'orientation': 'horizontal',
                   'index': 0,
                   'width': this.scrollWidth,
                   'x':0,
                   'y': this.newHeight,
                   'isRTL': this.model.enableRTL,
                   'zoomPosition':this.scrollZoomPos,
                   'zoomFactor': this.scrollZoomFact,
                   'scrollRange': this._scrollRange,
                   'valueType': this.model.valueType,
                   'parent': this.element,
                   'enableResize': true
               };
               if (scrollObj[0].valueType == 'datetime') {
                    scrollObj[0].startDateTime = this.startDateTime;
                    scrollObj[0].endDateTime = this.endDateTime;
               } else {
                    scrollObj[0].startValue = this.startValue;
                    scrollObj[0].endValue = this.endValue;
               }
               this.scrollbarContainer = new ej.EjSvgScrollbarRender(this.element, this.model.scrollObj[0]);
               this.scrollbarContainer._initializeScrollbarVariables.call(this, this.model.scrollObj[0]);
               this.scrollbarContainer._renderScrollbar.call(this, this.model.scrollObj[0]);
            }
            if (this.model.enableAutoResizing || this.model.isResponsive)
                this.bindResizeEvents();
            else
               this.removeResizeEvents();
            this._bindevents();
        },
        bindResizeEvents: function () {
            if (this.model._rangeResizeEventRegistered)
                return 0;
            if (!this.model.enableScrollbar || (this.model.enableScrollbar && this.svgSupport))
                this._on($(window), "resize", this.rangeChange);
            this.model._rangeResizeEventRegistered = true;
        },

        removeResizeEvents: function () {
            if (this.model._rangeResizeEventRegistered) {
                this._off($(window), "resize", this.rangeChange);
                this.model._rangeResizeEventRegistered = false;
            }
        },
             
         
		 rangeChange:function(evt){
             var id, _this = this;
			 this.rangeResize = true;
			 clearTimeout(id);
			 
			 if (!ej.isNullOrUndefined(window.orientation))			     
			     id = setTimeout(function(){_this.renderNavigator();}, 500);
			 else 
			     this.renderNavigator();
			 if (this.model.enableScrollbar) {
			     this.model.scrollObj[0].width = this.scrollWidth;
			     this.scrollbarContainer._renderScrollbar.call(this, this.model.scrollObj[0]);
			 }
          },

        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },

		//To check whether the device is Windows
        isWindows: function () {
          if (!ej.getBooleanVal($('head'), 'data-ej-android') && !ej.getBooleanVal($('head'), 'data-ej-ios') && !ej.getBooleanVal($('head'), 'data-ej-ios7') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._windows();
        },
		
		 _windows: function () {
          return (/trident|windows phone/i.test(navigator.userAgent.toLowerCase())) || (ej.getBooleanVal($('head'), 'data-ej-windows', false) === true);
        },
		
        _bindevents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent),
                browserInfo = ej.EjSvgRender.utils.browserInfo(),
                isPointer = browserInfo.isMSPointerEnabled,
	            isIE11Pointer = browserInfo.pointerEnabled,
                touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
                touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
                touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
                touchCancelEvent = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchleave mouseleave";
            this.model.browserInfo = browserInfo;


            this._on(this.element, touchMoveEvent, this._drag);
            this._on(this.element, touchCancelEvent, this._leave);
            this._on($(document), touchStopEvent, this._drop);
            this._on($(this.element), touchStartEvent, this._grab);
            this._on($(this.element), "contextmenu", this._rightClick);


            this._on($("#" + this._id + "template_group_"), touchStartEvent, this._grab);
            this._on($("#" + this._id + "template_group_"), touchMoveEvent, this._drag);
            this._on($("#" + this._id + "template_group_"), touchStopEvent, this._drop);
            this._on($(window), touchStopEvent, this.mouseup);

            $(this.element).css('touch-action', 'none');

        },

        _initializeVariables: function (model, series, sSettings) {
            //intializing ismouseup flag
            this.ismouseup = false;
            var rangeSettings= this.model.rangeSettings,
                labelStyle = this.model.labelSettings.style,
                lowLabelStyle = this.model.labelSettings.lowerLevel.style,
                highLabelStyle = this.model.labelSettings.higherLevel.style,
                scrollRangeSettings = this.model.scrollRangeSettings;
            //checking labelstyle
            if (highLabelStyle == "")
                this.model.labelSettings.higherLevel.style = labelStyle;
            if (lowLabelStyle == "") {
                this.model.labelSettings.lowerLevel.style = labelStyle;
            }
            else if (lowLabelStyle != undefined && lowLabelStyle.horizontalAlignment == undefined) {
                this.model.labelSettings.lowerLevel.style.horizontalAlignment = "middle";
            }
            if (this.model.valueType == "numeric" && this.model.tooltipSettings.labelFormat=="MM/dd/yyyy") {
                this.model.tooltipSettings.labelFormat = "d";
            }
            //checking for datasource and finding start and end values
            if (model.dataSource != "" && model.xName != "" && model.yName != "") {
                var length = series.length;
                if (length > 0) {
                    for (var i = 0; i < length; i++) {
                        series[i].dataSource = model.dataSource;
                        series[i].type = series[i].type ? series[i].type : (sSettings.type ? model.type : "line");
                        series[i].xName = series[i].xName ? series[i].xName : model.xName;
                        series[i].yName = series[i].yName ? series[i].yName : model.yName;
                        this._processJsonData(series[i].dataSource, series[i]);
                    }
                } else
                    this._processJsonData(model.dataSource, this.model);
            }
            else if (series!="" && series != undefined && series[0].dataSource != undefined && series[0].xName != undefined && series[0].yName != undefined) {
                this._processJsonData(series[0].dataSource, series[0]);
            }
            else if (sSettings.dataSource != undefined) {
                var length = series.length;
                if (length > 0) {
                    for (var i = 0; i < length; i++) {
                        series[i].dataSource = sSettings.dataSource;
                        series[i].type = series[i].type ? series[i].type : (sSettings.type ? sSettings.type : "line");
                        series[i].xName = series[i].xName ? series[i].xName : sSettings.xName;
                        series[i].yName = series[i].yName ? series[i].yName : sSettings.yName;
                        this._processJsonData(series[i].dataSource, series[i]);
                    }
                } else
                    this._processJsonData(sSettings.dataSource, sSettings);
            }
            else if (rangeSettings.start != "" && rangeSettings.end != "") {
                var minX = startDateTime = rangeSettings.start, maxX = endDateTime = rangeSettings.end;
                if (this.model.enableScrollbar) {
                    var args = [minX, maxX, new Date(minX), new Date(maxX)];
                    this.calculateZoomingRange.apply(this, args);

                } else
                    this.setRange.call(this, minX, maxX, startDateTime, endDateTime);               
            }
            // initializing start year and end year
            if (this.startDateTime != undefined && this.endDateTime != undefined) {
                if (this.model.valueType == "datetime" && this.model.rangePadding == "round") {
                    var start = new Date(this.startValue);
                    var end = new Date(this.endValue);
                    this.startDateTime = new Date(start.getFullYear(), start.getMonth(), start.getDate(), 0, 0, 0);
                    this.endDateTime = new Date(end.getFullYear(), end.getMonth(), end.getDate(), 23, 59, 59);
                }
                this.startYear = this.startDateTime.getFullYear();
                this.startMonth = this.startDateTime.getMonth(); 
                this.endYear = this.endDateTime.getFullYear();
                this.endMonth = this.endDateTime.getMonth();
            }           
        },

        setSliders: function () {
            var wholeSize = this.newWidth - this.padding * 2,
                type = this.model.valueType,
                selectedRangeStart = this.model.selectedRangeSettings.start,
                selectedRangeEnd = this.model.selectedRangeSettings.end,
                padding = this.padding, startRange = this.startDateTime.getTime(), endRange = this.endDateTime.getTime(),
                interval, startValue = this.startValue, endValue = this.endValue;
            if (type == "datetime")
                this.eachInterval = wholeSize / parseFloat(endRange - startRange);
            else
                this.eachInterval = wholeSize / parseFloat(endValue - startValue);

            interval = this.eachInterval;
            if (!this.model.enableScrollbar) {
                if (type == "datetime") {
                    if (selectedRangeStart == "" || new Date(selectedRangeStart).getTime() < startRange)
                        selectedRangeStart = this.startDateTime;
                    if (selectedRangeEnd == "" || new Date(selectedRangeEnd).getTime() > endRange || new Date(selectedRangeEnd).getTime() < startRange)
                        selectedRangeEnd = this.endDateTime;
                }
                else {
                    if (selectedRangeStart == "" || selectedRangeStart < startValue)
                        selectedRangeStart = startValue;
                    if (selectedRangeEnd == "" || selectedRangeEnd > endValue)
                        selectedRangeEnd = endValue;
                }
            }
            
            if (!this.model.enableRTL) {
                if (type == "datetime") {
                    this.leftSliderPosition = (new Date(selectedRangeStart).getTime() - startRange) * interval + padding;
                    this.rightSliderPosition = (new Date(selectedRangeEnd).getTime() - startRange) * interval + padding;
                    selectedRangeStart = new Date(selectedRangeStart);
                    selectedRangeEnd = new Date(selectedRangeEnd)
                }
                else {
                    this.leftSliderPosition = (selectedRangeStart - startValue) * interval + padding;
                    this.rightSliderPosition = (selectedRangeEnd - startValue) * interval + padding;
                }
            } else {
                if (type == "datetime") {
                    this.leftSliderPosition = (endRange - new Date(selectedRangeEnd).getTime()) * interval + padding;
                    this.rightSliderPosition = (endRange - new Date(selectedRangeStart).getTime()) * interval + padding;
                    selectedRangeStart = new Date(selectedRangeStart);
                    selectedRangeEnd = new Date(selectedRangeEnd);
                }
                else {
                    this.leftSliderPosition = (endValue - selectedRangeEnd) * interval + padding;
                    this.rightSliderPosition = (endValue - selectedRangeStart) * interval + padding;
                }               
            }

            if (this.model.tooltipSettings.tooltipDisplayMode == "always")
                this.setopacity = 1;
            else
                this.setopacity = 0;

            if (!this.model.enableScrollbar)
                this.setSliderPositions(this.leftSliderPosition, null, null);
            else {
                if ((this.leftSliderPosition >= padding && this.leftSliderPosition < this.newWidth - padding))
                    this.setSliderPositions(this.leftSliderPosition, null, null);
                else {
                    if (this.leftSliderPosition > this.newWidth) {
                        $(this.leftTooltip).hide();
                        $(this.gLeftSlider).hide();
                    } else {
                        $(this.leftTooltip).hide();
                        $(this.gLeftSlider).hide();
                        $(this.leftUnSelected).hide();
                    }
                }
            }
            if (!this.model.enableScrollbar)
                this.setSliderPositions(null, null, this.rightSliderPosition);
            else {
                if ((this.rightSliderPosition <= (this.newWidth - padding) && this.rightSliderPosition > padding))
                    this.setSliderPositions(null, null, this.rightSliderPosition);
                else {
                    if (this.rightSliderPosition < 0) {
                        $(this.rightTooltip).hide();
                        $(this.gRightSlider).hide();
                    } else {
                        $(this.rightTooltip).hide();
                        $(this.gRightSlider).hide();
                        $(this.rightUnSelected).hide();
                    }
                }
            }
                
           this.setopacity = 1;
           if (this.model.enableDeferredUpdate) {
               this._calculateSelectedData();
               this._calculateSliderZoomFactPosition();
               // checking condition for range event fire
               if (!this._scrollChanged) {
                   if (type == "datetime") {
                       if (this.startDateTime.getTime() !== selectedRangeStart.getTime() || this.endDateTime.getTime() !== selectedRangeEnd.getTime() && !this.rangeResize) {
                           this._trigger("rangeChanged", this.model);
                       }
                   }
                   else {
                       if (this.startValue !== selectedRangeStart || this.endValue !== selectedRangeEnd && !this.rangeResize) {
                           this._trigger("rangeChanged", this.model);
                       }
                   }
               }
				this.rangeResize=false;
            }
        },

        centerSliderWidth: "",
        zoomp: 0,
        zoomf:1,

        setSliderPositions: function (leftValue, centerValue, rightValue) {
            var doubledate, locale = this.model.locale;
            var type = this.model.valueType;
            var cPosition;
            var labelPlacement = this.model.labelSettings.higherLevel.labelPlacement;
            cPosition = labelPlacement == "inside" ? 0 : this.centerPosition;
            var rtlValue, tooltipPrecision, defaultPrecision=6, highestPrecision=20,
                tooltipFormat = this.model.tooltipSettings.labelFormat;
            if (type == 'numeric' && (tooltipFormat.indexOf('e') == 0 || tooltipFormat.indexOf('E') == 0)) {
                tooltipPrecision = tooltipFormat.match(/(\d+)/g);
                tooltipPrecision = tooltipPrecision == null ? defaultPrecision : tooltipPrecision > highestPrecision ? highestPrecision : tooltipPrecision;
            }
            if (leftValue != null && leftValue >= this.padding && leftValue < this.newWidth - this.padding) {
                this.leftSliderPosition = leftValue;
                rtlValue = this.model.enableRTL ? (this.newWidth - leftValue) : leftValue;
                if(type=="datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                if (this.model.tooltipSettings.visible && this.svgSupport && this.leftTxt) {
                    this.leftTooltip.setAttribute('opacity', this.setopacity);
                    if(type=="datetime")
                        this.leftTxt.textContent = ej.format(new Date(doubledate), this.model.tooltipSettings.labelFormat, locale);
                    else {
                        if (tooltipFormat.indexOf('e') == 0 || tooltipFormat.indexOf('E') == 0)
                            this.leftTxt.textContent = doubledate.toExponential(tooltipPrecision);
                        else this.leftTxt.textContent = ej.format(doubledate, this.model.tooltipSettings.labelFormat, locale);
                    }
                    var textOffset = ej.EjSvgRender.utils._measureText(this.leftTxt.textContent, null, this.model.tooltipSettings.font);
                    var txtwidth = textOffset.width + 15;
                    var txtheight = textOffset.height;
                    var txty = this.brow!="msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.leftTxt.setAttribute('y', txty);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";

                    if (leftValue - txtwidth >= 0) {
                        this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue - txtwidth) + ',' + cPosition + ')' });
                        this.leftTxt.setAttribute('x', 5);
                    }
                    else if (leftValue + txtwidth < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue) + ',' + cPosition + ')' });
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                        this.leftTxt.setAttribute('x', 10);
                    }
                    else if (this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue) + ',' + (cPosition + 30) + ')'});
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    else {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue) + ',' + (cPosition) + ')'});
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    if (leftValue > this.newWidth - this.rightTooltip.lastChild.getBBox().width - 15 && this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue - txtwidth) + ',' + (cPosition + 30) + ')'});
                    }
                    if (this.rightTooltip.getCTM().f >= 30 && leftValue < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + this.rightTooltip.getCTM().e + ',' + (cPosition) + ')'});
                    }
                }
                if (this.ismouseup == false) {
                    // this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding});
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if(this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue-10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                        this._leftValue = {
                            'left': leftValue - this.model.navigatorStyleSettings.thumbRadius,
                            'right': leftValue + this.model.navigatorStyleSettings.thumbRadius
                        }
                    }// adding the left position for thumbleft
                    else {
                        var left = leftValue - ($("#" + this._id + "thumbleft").width() / 2);
                        $("#" + this._id + "thumbleft").css("left", left);
                        this._leftValue = {
                            'left': left,
                            'right': left + $("#" + this._id + "thumbleft").width()
                        }
                    }
                    this.renderer._setAttr($(this.centerSlider), {'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')'});                   
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
					if(width > 0) this.renderer._setAttr($(this.centerSlider), { 'width': width});
                }
                else {
                    this.ismouseup = false;
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                        this._leftValue = {
                            'left': leftValue - this.model.navigatorStyleSettings.thumbRadius,
                            'right': leftValue + this.model.navigatorStyleSettings.thumbRadius
                        }
                    }   // adding the left position for thumbleft
                    else {
                        var left = leftValue - ($("#" + this._id + "thumbleft").width() / 2);
                        $("#" + this._id + "thumbleft").css("left", left);
                        this._leftValue = {
                            'left': left,
                            'right': left + $("#" + this._id + "thumbleft").width()
                        }
                    }
                    this.renderer._setAttr($(this.centerSlider), {'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')'});
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    if(width > 0) this.renderer._setAttr($(this.centerSlider), { 'width': width});
                }
                this.model.zoomPosition = (leftValue-this.padding) / (this.newWidth-this.padding*2);
                this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
               
                if (!this.model.enableRTL) {
                    if (type == "datetime")
                        this.model.selectedRangeSettings.start = new Date(doubledate);
                    else
                        this.model.selectedRangeSettings.start = doubledate;
                } else {
                    if (type == "datetime")
                        this.model.selectedRangeSettings.end = new Date(doubledate);
                    else
                        this.model.selectedRangeSettings.end = doubledate;
                }
				
                $(this.leftTooltip).show();
                $(this.gLeftSlider).show();
                $(this.leftUnSelected).show();
            }
            else if (rightValue != null && rightValue > this.padding) {
                if (rightValue > this.newWidth - this.padding + 1)
                    rightValue = this.newWidth - this.padding;
                this.rightSliderPosition = rightValue;
                var startDateTime = new Date(this.model.rangeSettings.start);
                rtlValue = this.model.enableRTL ? (this.newWidth - rightValue) : rightValue;
                doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                if(type=="datetime")
                    doubledate = new Date(doubledate);
                    //fixed issue for end value change
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                if (!this.model.enableRTL)
                    this.model.selectedRangeSettings.end = doubledate;
                else
                    this.model.selectedRangeSettings.start = doubledate;
                if (this.model.tooltipSettings.visible && this.svgSupport && this.rightTxt) {
                    this.rightTooltip.setAttribute('opacity', this.setopacity);
                    if(type=="datetime")
                        this.rightTxt.textContent = ej.format(new Date(doubledate), this.model.tooltipSettings.labelFormat, locale);
                    else {
                        if (tooltipFormat.indexOf('e') == 0 || tooltipFormat.indexOf('E') == 0)
                            this.rightTxt.textContent = doubledate.toExponential(tooltipPrecision);
                        else this.rightTxt.textContent = ej.format(doubledate, this.model.tooltipSettings.labelFormat, locale);
                    }
                    var textOffset = ej.EjSvgRender.utils._measureText(this.rightTxt.textContent, null, this.model.tooltipSettings.font);
                    var txtwidth = textOffset.width + 15;
                    var txtheight = textOffset.height;
                    var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.rightTxt.setAttribute('y', txty);
                    this.rightTxt.setAttribute('x', 10);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                    if (rightValue + txtwidth <= this.newWidth) {
                        this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                        this.renderer._setAttr($(this.rightTooltip), {'transform': 'translate(' + (rightValue) + ',' + cPosition + ')'});
                    }
                    else if (rightValue + txtwidth >= this.newWidth && rightValue - txtwidth > this.centerSlider.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), {'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                        this.rightTxt.setAttribute('x', 5);
                    }
                    else if (this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + (cPosition + 30) + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    else {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    if (rightValue < this.leftTooltip.getCTM().e + txtwidth && this.leftTooltip.getCTM().f < 30 && this.leftSliderPosition >= this.padding) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue) + ',' + (cPosition + 30) + ')'});
                    }
                    if (this.leftTooltip.getCTM().f >= 30 && rightValue > this.leftTooltip.getCTM().e + txtwidth && this.rightTooltip.getCTM().e > this.leftTooltip.getCTM().e + txtwidth) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + this.leftTooltip.getCTM().e + ',' + (cPosition) + ')' });
                        this.rightTxt.setAttribute('x', 5);
                    }
                }
                    var rightUnAreawidth =  this.newWidth - rightValue-this.padding-2; // To check width on slider change
					rightUnAreawidth = rightUnAreawidth < 0 ? 0 : rightUnAreawidth;
					
                this.renderer._setAttr($(this.rightUnArea), {'width': rightUnAreawidth, 'transform': 'translate(' + parseFloat(rightValue + 2) + ',' + this.sliderPosition + ')' });
                this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightValue + ',' + this.sliderPosition + ')'});
                if (this.rightCircle != undefined) {
                    if (this.svgSupport)
                        this.renderer._setAttr($(this.rightCircle), { 'cx': rightValue });
                    else
                        this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightValue-10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    this._rightValue = {
                        'left': rightValue - this.model.navigatorStyleSettings.thumbRadius,
                        'right': rightValue + this.model.navigatorStyleSettings.thumbRadius
                    }
                }// adding the left position for thumbright
                else {
                    var left = rightValue - ($("#" + this._id + "thumbright").width() / 2);
                    $("#" + this._id + "thumbright").css("left", left);
                    this._rightValue = {
                        'left': left,
                        'right': left + $("#" + this._id + "thumbright").width()
                    }
                }
                this.centerSliderWidth = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                this.renderer._setAttr($(this.centerSlider), { 'width': Math.abs(this.centerSliderWidth)});
                
                this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
					
                $(this.rightTooltip).show();
                $(this.gRightSlider).show();
                $(this.rightUnSelected).show();
            }
            else if (centerValue != null) {
                var rectwidth = this.renderer._getAttrVal($(this.leftSlider),'width');
				var rangeNavigatorWidth = this.newWidth - this.padding -2;
                var rightPosX = centerValue + parseFloat(this.renderer._getAttrVal($(this.centerSlider),'width'));
				if (rightPosX > rangeNavigatorWidth){
					leftPosX = centerValue = rangeNavigatorWidth - parseFloat(this.renderer._getAttrVal($(this.centerSlider),'width'));
					rightPosX = rangeNavigatorWidth;
				} else
					leftPosX = centerValue;		
                var startDateTime = new Date(this.model.rangeSettings.start);
                rtlValue = this.model.enableRTL ? (this.newWidth - leftPosX) : leftPosX;
                if (leftPosX >= this.padding && rightPosX <= this.newWidth - this.padding - 2) {
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                if(type=="datetime")
                    doubledate = new Date(doubledate);
                if (!this.model.enableRTL) 
                    this.model.selectedRangeSettings.start = doubledate;
                else
                    this.model.selectedRangeSettings.end = doubledate;

                    if(this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + 0 + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + parseFloat(this.sliderPosition) + ')' });
                    this.leftSliderPosition = leftPosX;
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftPosX });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });                        
                        this._leftValue = {
                            'left': leftPosX - this.model.navigatorStyleSettings.thumbRadius,
                            'right': leftPosX + this.model.navigatorStyleSettings.thumbRadius
                        }
                    }
                    else {
                        var left = leftPosX - ($("#" + this._id + "thumbleft").width() / 2);
                        $("#" + this._id + "thumbleft").css("left", left);
                        this._leftValue = {
                            'left': left,
                            'right': left + $("#" + this._id + "thumbleft").width()
                        }
                    }
                    if (this.model.tooltipSettings.visible && this.svgSupport && this.leftTxt) {
                        this.leftTooltip.setAttribute('opacity', this.setopacity);
                        if(type=="datetime")
                            this.leftTxt.textContent = ej.format(new Date(doubledate), this.model.tooltipSettings.labelFormat, locale);
                        else {
                            if (tooltipFormat.indexOf('e') == 0 || tooltipFormat.indexOf('E') == 0)
                                this.leftTxt.textContent = doubledate.toExponential(tooltipPrecision);
                            else this.leftTxt.textContent = ej.format(doubledate, this.model.tooltipSettings.labelFormat, locale);
                        }
                        var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.leftTooltip.lastChild.getBBox().height;
                        var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                        this.leftTxt.setAttribute('y', txty);
                        this.rightTxt.setAttribute('y', txty);
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (leftPosX - txtwidth >= 0) {
                            this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX - txtwidth) + ',' + cPosition + ')' });
                            this.leftTxt.setAttribute('x', 5);
                        }
                        else {
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + cPosition + ')'});
                            this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                            this.leftTxt.setAttribute('x', 10);
                            if (this.leftSliderPosition + txtwidth > this.rightSlider.getCTM().e) {
                                this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + (cPosition + 30) + ')'});
                            }

                        }
                    }
                    rtlValue = this.model.enableRTL ? (this.newWidth - rightPosX) : rightPosX;
                    var doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                    if (type == "datetime")
                        doubledate = new Date(doubledate);
					// fixing issue for end value when using numeric valutype
                    else
                        doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                    if (!this.model.enableRTL)
                        this.model.selectedRangeSettings.end = doubledate;
                    else
                        this.model.selectedRangeSettings.start = doubledate;
                    this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightPosX + ',' + this.sliderPosition + ')' });
                    this.rightSliderPosition = rightPosX;
                    if (this.rightCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.rightCircle), { 'cx': rightPosX });
                        else
                            this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });                        
                        this._rightValue = {
                            'left': rightPosX - this.model.navigatorStyleSettings.thumbRadius,
                            'right': rightPosX + this.model.navigatorStyleSettings.thumbRadius
                        }
                    }
                    else {
                        var left = rightPosX - ($("#" + this._id + "thumbright").width() / 2);
                        $("#" + this._id + "thumbright").css("left", left);
                        this._rightValue = {
                            'left': left,
                            'right': left + $("#" + this._id + "thumbright").width()
                        }
                    }
                    if (this.model.tooltipSettings.visible && this.svgSupport) {
                        this.rightTooltip.setAttribute('opacity', this.setopacity);
                        if(type=="datetime")
                            this.rightTxt.textContent = ej.format(new Date(doubledate), this.model.tooltipSettings.labelFormat, locale);
                        else {
                            if (tooltipFormat.indexOf('e') == 0 || tooltipFormat.indexOf('E') == 0)
                                this.rightTxt.textContent = doubledate.toExponential(tooltipPrecision);
                            else this.rightTxt.textContent = ej.format(doubledate, this.model.tooltipSettings.labelFormat, locale);
                        }
                        var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.rightTooltip.lastChild.getBBox().height;
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (rightPosX + txtwidth <= this.newWidth) {
                            this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX) + ',' + cPosition + ')' });
                            this.rightTxt.setAttribute('x', 10);
                        }
                        else {
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + cPosition + ')'});
                            this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                            this.rightTxt.setAttribute('x', 5);
                            if (this.rightTooltip.getBoundingClientRect().left < this.leftSlider.getBoundingClientRect().left) {
                                this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + (cPosition + 30) + ')' });
                            }
                        }
                    }
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftPosX - this.padding,'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')'});
                    this.renderer._setAttr($(this.rightUnArea), { 'transform': 'translate(' + parseFloat(rightPosX + 2) + ',' + this.sliderPosition + ')','width': this.newWidth - rightPosX-this.padding-2});

                    this.model.zoomPosition = (this.leftSlider.getBoundingClientRect().left - this.padding) / (this.newWidth - this.padding * 2);

                    this.renderer._setAttr($(this.centerSlider), { 'transform': 'translate(' + centerValue + ',' + this.sliderPosition + ')' });
                    this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                    $(this.leftTooltip).show();
                    $(this.rightTooltip).show();
                }
               
            }
            if (!this._scrollChanged && !this.isSelection) {
                if ((!this.model.enableDeferredUpdate) && this.leftSlider.getBoundingClientRect().left < this.rightSlider.getBoundingClientRect().left) {
                    this._calculateSelectedData();
                    if (this.zoomp != this.model.zoomPosition || this.zoomf != this.model.zoomFactor) {
                        this._calculateSliderZoomFactPosition();
                        this._trigger("rangeChanged", this.model);
                        this.zoomp = this.model.zoomPosition;
                        this.zoomf = this.model.zoomFactor;
                    }
                }
            }
          
        },
        _calculateSliderZoomFactPosition: function () {
            var model = this.model,
                delta, zoomPos, zoomFact, start, end,
                selectedRange = model.selectedRangeSettings,
                virtualRange = model.scrollRangeSettings;
            if (model.valueType.toLowerCase() == "datetime") {
                start = virtualRange.start ? new Date(virtualRange.start) : this.startDateTime;
                start = start < this.startDateTime ? start: this.startDateTime;
                end = virtualRange.end ? new Date(virtualRange.end) : this.endDateTime;
                end = end > this.endDateTime ? end : this.endDateTime;
                delta = end - start;
                zoomPos = (new Date(selectedRange.start) - start) / delta;
                zoomFact = (new Date(selectedRange.end) - new Date(selectedRange.start)) / delta;
            } else {
                start = virtualRange.start != "" ? parseFloat(virtualRange.start) : this.startValue;
                start = start < this.startValue ? start : this.startValue;
                end = virtualRange.end != "" ? parseFloat(virtualRange.end) : this.endValue;
                end = end > this.endValue ? end : this.endValue;
                delta = end - start;
                zoomPos = (parseFloat(selectedRange.start) - start) / delta;
                zoomFact = (parseFloat(selectedRange.end) - parseFloat(selectedRange.start)) / delta;
            }
             model.zoomPosition = zoomPos;
             model.zoomFactor = zoomFact;            
        },

        _calculateSelectedData: function () {
            var j = [], count=0, isDate = this.model.valueType == "datetime", startValue = this.model.selectedRangeSettings.start, endValue = this.model.selectedRangeSettings.end;
            if (isDate) {
                startValue = Date.parse(startValue); endValue = Date.parse(endValue);
            }
            var dataSource = (this.model.dataSource != "") ? this.model.dataSource : ((this.model.series != "" && this.model.series[0].dataSource != "") ? this.model.series[0].dataSource : null);
            if (dataSource != null) {
                for (var i = 0; i < dataSource.length - 1; i++) {
                    currentValue = isDate ? Date.parse(dataSource[i][Object.keys(dataSource[i])[0]]) : dataSource[i][Object.keys(dataSource[i])[0]];
                    if ((startValue <= currentValue) && (currentValue <= endValue)) {
                        j[count] = dataSource[i];
                        count++;
                    }
                }
            }
            this.model.selectedData = j;
        },

        calculateInterval: function (higherLevel, lowerLevel) {
            lowerLevel._intervalType = lowerLevel.intervalType ? lowerLevel.intervalType : "auto";

            if (lowerLevel.visible === true && this.model.valueType == "datetime")
               this.calculateVisibleLabels(lowerLevel, "lower", 0);

            if (higherLevel.visible === true && this.model.valueType == "datetime") {
                if (higherLevel.intervalType && higherLevel.intervalType.toLowerCase() != "auto")
                    higherLevel._intervalType = higherLevel.intervalType;
                else {
                    switch (lowerLevel._intervalType) {
                        case "years":
                            higherLevel._intervalType = "years";
                            break;
                        case "quarters":
                            higherLevel._intervalType = "years";
                            break;
                        case "months":
                            higherLevel._intervalType = "quarters";
                            break;
                        case "weeks":
                            higherLevel._intervalType = "months";
                            break;
                        case "days":
                            higherLevel._intervalType = "weeks";
                            break;
                        case "hours":
                            higherLevel._intervalType = "days";
                            break;
                        case "minutes":
                            higherLevel._intervalType = "hours";
                            break;
                    }
                }

                if (higherLevel.autoInterval == false)
                    this.calculateVisibleLabels(higherLevel, "higher", 0);
                else
                    this.calculateVisibleLabels(higherLevel, "higher", 0);
                
            }

            if (lowerLevel.visible === true && this.model.valueType == "numeric")
                this.setInterval(this);
        },

        calculateVisibleLabels: function (label, labelType, level) {

            var type = label._intervalType;

            switch (type) {
                case "auto":
                    var intervalX = this.calculateDateTimeNiceInterval(this, this.startDateTime, this.endDateTime);
                    var intrvlType = intervalX.intervalType.toLowerCase();                    
                    label._intervalType = intrvlType;
                    label._interval = intervalX.interval;

                    this.calculateVisibleLabels(label, labelType, level)
                    break;
                case "years":
                    this.setYearInterval(this, labelType);
                    break;
                case "quarters":
                    this.setQuarterInterval(this, labelType, level);
                    break;
                case "months":
                    this.setMonthInterval(this, labelType, level);
                    break;
                case "weeks":
                    this.setWeekInterval(this, labelType, level);
                    break;
                case "days":
                    this.setDayInterval(this, labelType, level);
                    break;
                case "hours":
                    this.setHourInterval(this, labelType, level);
                    break;
                case "minutes":
                    this.setMinuteInterval(this, labelType, level);
                    break;
                default:
                    break;
            }
        },

        setInterval: function (naviobj) {
            var currentValue, lastValue;
            var delta = this.endValue - this.startValue;
            var rangePadding = naviobj.model.rangePadding;
            var start = this.startValue;
            var numericInterval = this.calculateNumericInterval(naviobj, delta);
            if (rangePadding == "additional") {
                this.endValue = this.endValue + numericInterval;
                this.startValue = this.startValue - numericInterval;
                delta = this.endValue - this.startValue;
              //  numericInterval = this.calculateNumericInterval(naviobj, delta);
            } else if (rangePadding == "normal") {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = this.startValue + (this.startValue / 20);

                    remaining = numericInterval + (minimum % numericInterval);

                    if ((0.365 * numericInterval) >= remaining) {
                        minimum -= numericInterval;
                    }

                    if (minimum % numericInterval < 0) {
                        minimum = (minimum - numericInterval) - (minimum % numericInterval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * this.endValue)
                        ? 0
                        : (start - (this.endValue - start) / 2);
                    if (minimum % numericInterval > 0) {
                        minimum -= (minimum % numericInterval);
                    }
                }
                var maximum = (this.endValue + (this.endValue - start) / 20);

                remaining = numericInterval - (maximum % numericInterval);

                if ((0.365 * numericInterval) >= remaining) {
                    maximum += numericInterval;
                }

                if (maximum % numericInterval > 0) {
                    maximum = (maximum + numericInterval) - (maximum % numericInterval);
                }
                numericInterval = this.calculateNumericInterval(naviobj, maximum - minimum);
                this.startValue = minimum;
                this.endValue = maximum;

            } else if (rangePadding == "round") {
                this.startValue = Math.floor((this.startValue / numericInterval) * numericInterval);
                this.endValue = Math.ceil((this.endValue / numericInterval) * numericInterval);
            }

            currentValue = naviobj.startValue;
            count = 0;
            while (currentValue <= naviobj.endValue) {
                this._insertNumericText(naviobj,currentValue);
                count++;
                currentValue = currentValue + numericInterval;
            }
            this.insertLabels(naviobj, "lower");

        },
        
        calculateDateTimeNiceInterval: function (naviObj, startDate, endDate) {

            var oneDay = 24 * 60 * 60 * 1000;            
            var totalDays = (Math.abs((startDate.getTime() - endDate.getTime()) / (oneDay)));
            var interval = this.calculateNumericInterval(naviObj, totalDays / 365);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Years" };
            }

            interval = this.calculateNumericInterval(naviObj, totalDays / 90);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Quarters" };
            }

            interval = this.calculateNumericInterval(naviObj, totalDays / 30);

            if (interval >= 1) {
                return { interval: interval, interval1: interval, intervalType: "Months" };
            }

            interval = this.calculateNumericInterval(naviObj, totalDays / 7);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Weeks", intervalType1: "Weeks" };
            }

            interval = this.calculateNumericInterval(naviObj, totalDays);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Days" };
            }

            var totalHours = totalDays * 24;
            interval = this.calculateNumericInterval(naviObj, totalHours);

            if (interval >= 1) {
                return { interval: interval, intervalType: "Hours" };
            }

            var totalMinutes = totalDays * 24 * 60;

            interval = this.calculateNumericInterval(naviObj, totalMinutes);

            return { interval: interval, intervalType: "Minutes" };
        },

        calculateNumericInterval: function (naviobj,delta) {
            var desiredIntervalsCount = this.GetDesiredIntervalsCount(naviobj.newWidth);
            var niceInterval = delta / desiredIntervalsCount;
            var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
            var intervalDivs = [10, 5, 2, 1];

            for (var i = 0; i < intervalDivs.length; i++) {
                var currentInterval = minInterval * intervalDivs[i];
                if (desiredIntervalsCount < (delta / currentInterval)) {
                    return niceInterval;
                }

                niceInterval = currentInterval;
            }

            return niceInterval;
        },

        GetDesiredIntervalsCount: function (size) {
                var desiredIntervalsCount = 0.8* 2;
                desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
                return desiredIntervalsCount;
        },

        setHourInterval: function (naviobj, position, level) {

            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addHours = function (hours) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0) {
                    dat.setMilliseconds(dat.getMilliseconds() + (1000 - dat.getMilliseconds()));
                }
                if (dat.getSeconds() !== 0) {
                    dat.setSeconds(dat.getSeconds() + (60 - dat.getSeconds()));
                }
                if (dat.getMinutes() !== 0) {                    
                    dat.setMinutes(dat.getMinutes() + (60 - dat.getMinutes()));
                }
                else
                    dat.setHours(dat.getHours() + hours);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hhtt", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addHours(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setHourInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hht", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;                        
                        currentDate = currentDate.addHours(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setHourInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hh", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addHours(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth) {
                        //Remove already inserted hours text value from DOM and calculate days interval
                        $(naviobj.lowerLevel).empty();
                        naviobj.model.labelSettings.lowerLevel._intervalType = "days";
                        naviobj.setDayInterval(naviobj, position, 0);
                    }
                    else
                        this.insertLabels(naviobj, position);
                    break;
            }
        },

        setMinuteInterval: function (naviobj, position, level) {

            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addMinutes = function (minutes) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0) {
                    dat.setMilliseconds(dat.getMilliseconds() + (1000 - dat.getMilliseconds()));
                }
                if (dat.getSeconds() !== 0) {
                    dat.setSeconds(dat.getSeconds() + (60 - dat.getSeconds()));
                }
                else
                    dat.setMinutes(dat.getMinutes() + minutes);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "mm", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMinutes(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth) {
                        //Remove already inserted minutes text value from DOM and calculate hours interval
                        $(naviobj.lowerLevel).empty();
                        naviobj.model.labelSettings.lowerLevel._intervalType = "hours";
                        naviobj.setHourInterval(naviobj, position, 0);
                    } else
                        this.insertLabels(naviobj, position);
                    break;                
            }
        },
        // Empty the labelbars
        _emptyLabelBars: function (naviobj, position) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
        },

        //to insert week text as there is no globalized format for week
        _insertWeekText: function (naviobj, position, textValue, currentDate, count, bwidth) {
            var hiLabelFont = naviobj.model.labelSettings.higherLevel.style.font;
            var loLabelFont = naviobj.model.labelSettings.lowerLevel.style.font;
            if (position === "higher") {
                if (count == 0)
                    naviobj._higherTotalValues[count] = 0;
                else
                    naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": hiLabelFont.size,
                    "font-family": hiLabelFont.family,
                    "font-style": hiLabelFont.style,
                    "font-weight": hiLabelFont.weight,
                    "fill": hiLabelFont.color
                };
                naviobj.renderer.drawText(textOptions, textValue, naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                if (count == 0)
                    naviobj._lowerTotalValues[count] = 0;
                else
                    naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": loLabelFont.size,
                    "font-family": loLabelFont.family,
                    "font-style": loLabelFont.style,
                    "font-weight": loLabelFont.weight,
                    "fill": loLabelFont.color
                };

                naviobj.renderer.drawText(textOptions, textValue, naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }

            return bwidth;
        },

        //To calculate Week Interval
        setWeekInterval: function (naviobj, position, level) {

            this._emptyLabelBars(naviobj, position);
            //get week number of a year
            Date.prototype.getWeek = function () {
                var date = new Date(this.getTime());
                date.setHours(0, 0, 0, 0);
                // Thursday in current week decides the year.
                date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
                // January 4 is always in week 1.
                var week1 = new Date(date.getFullYear(), 0, 4);
                // Adjust to Thursday in week 1 and count number of weeks from date to week1.
                return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000
                                      - 3 + (week1.getDay() + 6) % 7) / 7);
            }

            Date.prototype.dayOfWeek = function () {
                var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                return days[this.getDay()];
            }

            Date.prototype.addDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + noOfDays);
                return dat;
            }

            Date.prototype.subDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() - noOfDays);
                return dat;
            }

            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
			var culture = this.model.locale;
			var locale = ej.datavisualization.RangeNavigator.Locale[culture] ? ej.datavisualization.RangeNavigator.Locale[culture] : ej.datavisualization.RangeNavigator.Locale["en-US"];

			
			    while (currentDate.dayOfWeek() != "Monday") {
			        currentDate = currentDate.subDays(1);
			    }
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, this._localizedLabels.intervals.week.longWeeks + currentDate.getWeek() + ej.format(currentDate, " MMMM, yyyy", culture), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, this._localizedLabels.intervals.week.longWeeks + currentDate.getWeek() + ej.format(currentDate, " MMM, yy", culture), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, this._localizedLabels.intervals.week.longWeeks + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, this._localizedLabels.intervals.week.shortWeeks + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                        this.insertLabels(naviobj, position);
                    break;
            }
        },
       
        setDayInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = []; 
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addDays = function () {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + 1);
                return dat;
            }

            var hourArray = [], currentDate = naviobj.startDateTime, count = 0, txtBlockWidth = 0, bwidth = 0, bBox;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dddd, MMMM d, yyyy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;                       
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "ddd, MMM d, yy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                        
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dddd, d", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                        
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM, d", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 4);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 4:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dd", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                       
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        setMonthInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();           

            var currentDate = naviobj.startDateTime;            
			var textPadding =5;
            var count = 0;
            var date = new Date(naviobj.startDateTime);
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;

            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM, yyyy", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth + textPadding >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth + textPadding >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;                        
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth + textPadding >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth, true);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        _quarter: function (naviobj, position, quarterValue, yearString, date, txtBlockWidth, count) {
            
            for (var i = naviobj.startYear; i <= naviobj.endYear; i++) {
                var stringDate = yearString != null ? ej.format(date, yearString, naviobj.model.locale) : "";
                if (i === naviobj.startYear) {
                    for (var j = naviobj.startMonth; j <= 11 && (date <= this.endDateTime); j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight,
                                "fill":naviobj.model.labelSettings.higherLevel.style.font.color,
                              "opacity": naviobj.model.labelSettings.higherLevel.style.font.opacity
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight,
                                "fill":naviobj.model.labelSettings.lowerLevel.style.font.color,
                                "opacity": naviobj.model.labelSettings.lowerLevel.style.font.opacity
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if(position=="higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            if (j === naviobj.startMonth)
                                naviobj._lowerTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._lowerTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            if (j === naviobj.startMonth)
                                naviobj._higherTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._higherTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else if (i === naviobj.endYear) {
                    for (var j = 0; j <= naviobj.endMonth; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight,
                                "fill": naviobj.model.labelSettings.higherLevel.style.font.color,
                                "opacity": naviobj.model.labelSettings.higherLevel.style.font.opacity
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight,
                                "fill": naviobj.model.labelSettings.lowerLevel.style.font.color,
                                "opacity": naviobj.model.labelSettings.lowerLevel.style.font.opacity
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;
                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();                            
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else {
                    for (var j = 0; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }

            }
            return txtBlockWidth;
        },

        setQuarterInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
            var date = new Date(naviobj.startDateTime);
            var count = 0;
            var k = 0;
            var txtBlockWidth = 0;
			var culture = this.model.locale;
			var locale = ej.datavisualization.RangeNavigator.Locale[culture] ? ej.datavisualization.RangeNavigator.Locale[culture]:
                                                                               ej.datavisualization.RangeNavigator.Locale["en-US"];
            switch (level) {
                case 0:
                    txtBlockWidth = this._quarter(naviobj, position, this._localizedLabels.intervals.quarter.longQuarters, "yyyy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= (naviobj.newWidth - this.padding))
                        naviobj.setQuarterInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    txtBlockWidth = this._quarter(naviobj, position, this._localizedLabels.intervals.quarter.longQuarters, "yy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= (naviobj.newWidth - this.padding))
                        naviobj.setQuarterInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    txtBlockWidth = this._quarter(naviobj, position, this._localizedLabels.intervals.quarter.shortQuarters, "yyyy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= (naviobj.newWidth - this.padding))
                        naviobj.setQuarterInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    txtBlockWidth = this._quarter(naviobj, position, this._localizedLabels.intervals.quarter.shortQuarters, null, date, txtBlockWidth, count);
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;
            }
        },

        _insertText: function (naviobj, position, txtValue, currentDate, count, bwidth, setsubstring) {
            var locale = naviobj.model.locale;
            Date.prototype.addYears = function (years) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0 || dat.getSeconds() !== 0
                 || dat.getMinutes() !== 0 || dat.getHours() !== 0 || dat.getDate() !== 0
                 || dat.getMonth() !== 0) {
                    dat = new Date(dat.getFullYear() + 1, 0, 1);
                }
                else
                    dat.setYear(dat.getFullYear() + years);
                return dat;
            }

            Date.prototype.addMonths = function () {
                var dat = new Date(this.valueOf())
                dat = new Date(dat.getFullYear(), dat.getMonth() + 1, 1);
                return dat;
            }

            if (position === "higher") {
                naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                    "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                    "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                    "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight,
                    "fill":naviobj.model.labelSettings.higherLevel.style.font.color
                };
                if(setsubstring==false)
                    naviobj.renderer.drawText(textOptions, ej.format(currentDate, txtValue, locale), naviobj.higherLevel);
                else
                    naviobj.renderer.drawText(textOptions, ej.format(currentDate, txtValue, locale).toString().substring(0, 1), naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                if(this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                    "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                    "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                    "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight,
                    "fill":naviobj.model.labelSettings.lowerLevel.style.font.color
                };
                if(setsubstring==false)
                    naviobj.renderer.drawText(textOptions, ej.format(currentDate, txtValue, locale), naviobj.lowerLevel);
                else
                    naviobj.renderer.drawText(textOptions, ej.format(currentDate, txtValue, locale).substring(0, 1), naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                if(this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            return bwidth;
        },

        _insertNumericText: function (naviobj,currentValue) {
            naviobj._lowerTotalValues[count] = currentValue - naviobj.startValue;
            var font = naviobj.model.labelSettings.style.font;
            var textOptions = {
                'id': this._rootId + '_LabelBarText' + '_' + count,
                'x': 0,
                'y': 0,
                'text-anchor': 'start',
                'dominant-baseline': 'middle',
                "font-size": font.size,
                "font-family": font.family,
                "font-style": font.style,
                "font-weight": font.weight,
                "fill": font.color
            };
                naviobj.renderer.drawText(textOptions, currentValue, naviobj.lowerLevel);
            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
            var textOffset;
            if(this.svgSupport) {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
            } else {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
            } 
        },

        setYearInterval: function (naviobj, position) {
            var currentDate = naviobj.startDateTime;
            count=0;
            while (currentDate <= naviobj.endDateTime) {
                this._insertText(naviobj, position, "yyyy", currentDate,count,0,false);                   
                count++;
                currentDate = currentDate.addYears(1);
            }            
            this.insertLabels(naviobj, position);
        },

        _applyPadding:function(){
            var i;
            for (i = 0; i < this._higherTotalValues.length; i++) {
                this._higherTotalValues[i] += this.padding;
            }
            for (i = 0; i < this._lowerTotalValues.length; i++) {
                this._lowerTotalValues[i] += this.padding;
            }
        },

        insertLabels: function (naviobj, level) {
            // this._applyPadding();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var lowgridColor = this.model.labelSettings.lowerLevel.gridLineStyle.color;
            this.lgClr = this.renderer.createGradientElement('lgColor', lowgridColor, 150, 0, 150, 100, this.svgDocument);
            var type = this.model.valueType;
            var wholeSize = parseFloat(this.renderer._getAttrVal($(naviobj.svgDocument), 'width'));
            if (type == "datetime")
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endValue - this.startValue);
            var count = 0, labelcount = 0, txtwidth = 0, rectoptions, lineleft, newleft,
                textOffset, labelfont, i, highLength = naviobj._higherTextNode.length;
                lowerLevelLabels = [],
            higherLevelLabels = [];
                var offset = 0;
            if (level === "higher" && naviobj.model.labelSettings.higherLevel.visible===true) {

                if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth > 0 ? (naviobj.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.bottomPosition,
                        width: naviobj.newWidth > 0 ? (naviobj.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }

                naviobj._higherLineLeft = [];

                if (!naviobj.model.enableRTL) {
                    lineleft = (naviobj._higherTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._higherTotalValues[naviobj._higherTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var isrtl;
                if (naviobj.model.enableRTL) {
                    i = naviobj._higherTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }
                var setLeft = 0;

                for (; isrtl ? i >= 0 : i < naviobj._higherTextNode.length; isrtl?i--:i++) {
                    naviobj._higherLineLeft[i] = lineleft;
                    //  var txt =  naviobj._higherTextNode[i].getBBox();
                    labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                    txtwidth = textOffset.width;
                    newleft = naviobj._higherTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl?naviobj.newWidth-((naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding)):
                        (naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle") {
                        setleft = isrtl ? newleft - (newleft - lineleft) / 2 - txtwidth / 2: (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left") {
                        setleft = isrtl ? newleft + 10 : lineleft + 10;
                    }
                    else
                        setleft = isrtl ? lineleft - txtwidth - 2 : newleft - txtwidth - 2;
                    if (this.model.enableScrollbar) {
                        //rmove overlapped labels on higherlevel text
                        if ((!isrtl && i == 0) || (isrtl && highLength == i))
                            this.firstLabelPosition = setleft + txtwidth;
                        if (((!isrtl && i == 1) || (isrtl && highLength - 1 == i)) && this.firstLabelPosition > lineleft)
                            naviobj._higherTextNode[isrtl ? i + 1 : i - 1].textContent = "";
                    }
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'fill': naviobj.model.labelSettings.higherLevel.style.font.color });
                    naviobj._higherTextLeft[i] = setleft;
                    
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.minHighHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                        if (this.svgSupport) {
                            if(matched.browser.toLowerCase() == "msie")
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                            else
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 9);
                        }
                        else
                            naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'y': (naviobj.centerPosition / 2) - 5 });
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray':naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: this.bottomPosition,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    if ((setleft < this.padding || naviobj.newWidth - lineleft - this.padding < txtwidth) && type == "datetime") {
                        if (isrtl) {
                            if (setleft < this.padding && lineleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle")
                                    naviobj._higherTextNode[i].setAttribute("x", this.padding + (lineleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                                    naviobj._higherTextNode[i].setAttribute("x", this.padding + 10);
                                else
                                    naviobj._higherTextNode[i].setAttribute("x", lineleft - txtwidth - 2);
                            }
                            else if (naviobj.newWidth - lineleft < txtwidth && naviobj.newWidth - newleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle")
                                    naviobj._higherTextNode[i].setAttribute("x", newleft + (naviobj.newWidth - newleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                                    naviobj._higherTextNode[i].setAttribute("x", newleft + 10);
                                else
                                    naviobj._higherTextNode[i].setAttribute("x", naviobj.newWidth - txtwidth - 2 - this.padding);
                            }
                            else
                                naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                        }
                        else {
                            naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                            continue;
                        }
                    }
                    else if (setleft + txtwidth > naviobj.newWidth - this.padding && type == "datetime") {
                        if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle")
                            setleft = lineleft + (naviobj.newWidth - lineleft - this.padding) / 2 - txtwidth / 2;
                        else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                            setleft = lineleft;
                        else
                            setleft = naviobj.newWidth - txtwidth * 3/2;
                        naviobj._higherTextNode[i].setAttribute("x", setleft);
                        continue;
                    }
                    if (!this.model.enableRTL) {
                        lineleft = naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding;
                    } else {
                        lineleft= naviobj.newWidth - (naviobj._higherTotalValues[i -1] * this.eachInterval + this.padding);
                    }
                    count++;
                }
               
                //Calculate hihger level label regions
               for (var k = 0, len=naviobj._higherLineLeft.length; k < len; k++) {
                   textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[k].textContent, null, labelfont);
                   if (this.svgSupport)
                       higherLevelLabels.push({ size: textOffset, x: naviobj._higherLineLeft[k], y: naviobj._higherTextNode[k].getAttribute("y") - textOffset.height / 2 + offset, lableType: "higherLevel" });
                   else
                       higherLevelLabels.push({ size: textOffset, x: naviobj._higherLineLeft[k], y: naviobj._higherTextNode[k].offsetTop - textOffset.height / 2 + offset, lableType: "higherLevel" });

               }
               this._addLabelsRegion(naviobj, higherLevelLabels);
                 
            }
            else if (naviobj.model.labelSettings.lowerLevel.visible === true) {

                naviobj._lowerLineLeft = [];
                if (!naviobj.model.enableRTL) {
                    lineleft = (naviobj._lowerTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._lowerTotalValues[naviobj._lowerTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var hy = naviobj.model.labelSettings.higherLevel.visible ? naviobj.bottomPosition : naviobj.centerHeight;

                if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: hy,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    if (type != "numeric")
                        naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.centerHeight,
                        width: this.newWidth > 0 ? (this.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: 0,
                        width: this.newWidth > 0 ? (this.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.minHighHeight - 1,
                        width: this.newWidth > 0 ? (this.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minLowHeight,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }

                if (naviobj.model.enableRTL) {
                    i = naviobj._lowerTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }


                for (; isrtl ? i >= 0 : i < naviobj._lowerTextNode.length; isrtl?i--:i++) {
                    naviobj._lowerLineLeft[i] = lineleft;
                    labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont); // var txt = naviobj._lowerTextNode[i].getBBox();
                    txtwidth = textOffset.width;// txt.width;
                    var setleft = 0;
                    newleft = naviobj._lowerTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl ? naviobj.newWidth - ((naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding)) :
                        (naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle") {
                        setleft = isrtl ? (lineleft - newleft) / 2 + newleft - txtwidth / 2 : (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left") {
                        setleft = isrtl? newleft + 2 : lineleft + 2;
                    }
                    else
                        setleft = isrtl ? lineleft - txtwidth - 2 : newleft - txtwidth - 2;
                    if (type == "numeric")
                        setleft = lineleft - txtwidth / 2;
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'fill': naviobj.model.labelSettings.lowerLevel.style.font.color });

                    naviobj._lowerTextLeft[i] = setleft;

                    if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                        if (!this.svgSupport)
                            naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'y': naviobj.newHeight - textOffset.height });
                        else {
                            if (matched.browser.toLowerCase() == "msie")
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                            else
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 9);

                        }
                        var ht = naviobj.model.labelSettings.higherLevel.visible ? parseFloat(naviobj.minHighHeight + naviobj.centerHeight) : naviobj.minHighHeight;
                        rectoptions = {
                            x1: lineleft,
                            y1: ht,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': this.lgClr,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0 && type != "numeric")
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.bottomPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.centerHeight,
                            x2: lineleft,
                            y2: naviobj.bottomPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray':naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: 0,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.minHighHeight,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    if ((setleft < this.padding || naviobj.newWidth - lineleft - this.padding < txtwidth) && type == "datetime") {
                        if (isrtl) {
                            if (setleft < this.padding && lineleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle")
                                    naviobj._lowerTextNode[i].setAttribute("x", this.padding + (lineleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                                    naviobj._lowerTextNode[i].setAttribute("x", this.padding + 10);
                                else
                                    naviobj._lowerTextNode[i].setAttribute("x", lineleft - txtwidth - 2);
                            }
                            else if (naviobj.newWidth - lineleft < txtwidth && naviobj.newWidth -this.padding - newleft > txtwidth) {
                                if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle")
                                    naviobj._lowerTextNode[i].setAttribute("x", newleft + (naviobj.newWidth - newleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                                    naviobj._lowerTextNode[i].setAttribute("x", newleft + 10);
                                else
                                    naviobj._lowerTextNode[i].setAttribute("x", naviobj.newWidth - txtwidth - 2 - this.padding);
                            }
                            else
                                naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                            count--;
                        }
                        else {
                            naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                            count--;
                        }
                    }
                    else if (setleft + txtwidth > naviobj.newWidth - this.padding && type == "datetime") {
                        if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle")
                            setleft = lineleft + (naviobj.newWidth - lineleft - this.padding) / 2 - txtwidth / 2;
                        else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                            setleft = lineleft;
                        else
                            setleft = naviobj.newWidth - txtwidth * 3 / 2;
                        naviobj._lowerTextNode[i].setAttribute("x", setleft);                        
                    }
                    if (!this.model.enableRTL)
                        lineleft = naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding;
                    else
                        lineleft = naviobj.newWidth - (naviobj._lowerTotalValues[i - 1] * this.eachInterval + this.padding);
                    count++;
                }
                
                //Calculate lower lavel label regions
                for (var k = 0, len=naviobj._lowerLineLeft.length;  k < len; k++) {
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[k].textContent, null, labelfont);
                    if (this.svgSupport)
                        lowerLevelLabels.push({ size: textOffset, x: naviobj._lowerLineLeft[k], y: naviobj._lowerTextNode[k].getAttribute("y") - textOffset.height / 2 + offset, lableType: "lowerLevel" });
                    else
                        lowerLevelLabels.push({ size: textOffset, x: naviobj._lowerLineLeft[k], y: naviobj._lowerTextNode[k].offsetTop + offset, lableType: "lowerLevel" });
                }
                this._addLabelsRegion(naviobj, lowerLevelLabels);
                
            }
            if (!naviobj.model.enableRTL) {
                naviobj._lowerLineLeft.push(this.newWidth - this.padding);
            } else {
                naviobj._lowerLineLeft.push(this.padding);
            }            
            naviobj._setGridlines(naviobj);
            $(this.higherLevel).css('cursor', 'pointer');
            $(this.lowerLevel).css('cursor', 'pointer');
            //this.higherLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');
            //this.lowerLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');

        },

        //Add higher level and lowe level labels regions
        _addLabelsRegion: function (naviObj,labels) {
            var width, height,startX, len = labels.length;
            if (len == 1)
            naviObj.labelRegions.push({ X: labels[0].x, Y: labels[0].y, Height: labels[0].size.height, Width: naviObj.newWidth-labels[0].x, LabelType: labels[0].lableType });
            else {
                for (var k = 0; k < len; k++) {
                    if (!naviObj.model.enableRTL) {
                        width = labels[k + 1] ? labels[k + 1].x - labels[k].x : naviObj.newWidth - labels[k].x - parseInt(naviObj.model.padding);
                        startX = labels[k].x;
                    }                                                
                    else {                        
                        width = labels[k + 1] ? labels[k].x - labels[k + 1].x : labels[k].x - parseInt(naviObj.model.padding);
                            startX = labels[k + 1] ? labels[k + 1].x : parseInt(naviObj.model.padding);
                    }                       
                    height = labels[k + 1] ? labels[k].size.height : labels[k - 1].size.height;
                    naviObj.labelRegions.push({ X: startX, Y: labels[k].y, Height: height, Width: width, LabelType: labels[k].lableType });
                }
            }
        },
        _setGridlines: function (naviobj) {

            if (naviobj.model.navigatorStyleSettings.minorGridLineStyle.visible === true) {

                for (var i = 1; i < naviobj._lowerTextNode.length; i++) {
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerHeight + naviobj.minLowHeight+naviobj.minHighHeight,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.navigatorStyleSettings.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    else {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerPosition,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition+naviobj.centerHeight,
                            'stroke': naviobj.model.navigatorStyleSettings.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    
                }
            }
            else {
                for (var i = 0; i < naviobj._higherTextNode.length; i++) {
                    // var lineLeft = naviobj._lowerLineLeft[i];
                    var rectoptions = {
                        x1: naviobj._higherLineLeft[i],
                        y1: naviobj.centerHeight + naviobj.minLowHeight,
                        x2: naviobj._higherLineLeft[i],
                        y2: naviobj.centerPosition,
                        'stroke': naviobj.model.navigatorStyleSettings.majorGridLineStyle.color,
                        'stroke-width': '1'
                    };
                    naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                }
            }

        },

        bindTo: function () {
            if (!ej.util.isNullOrUndefined(this.model.dataSource)) {
                if (this.model.dataSource != null && this.model.dataSource.length > 0) {

                    this._processJsonData(this.model.dataSource, this.model);
                }
            }

        },

        _processJsonData: function (jsonObj, newmodel) {

            var xdata = [], yValues = [], index = 0;

            for (var j = 0; j < jsonObj.length; j++) {
                if (typeof jsonObj[j][newmodel.xName] == "string" && jsonObj[j][newmodel.xName].indexOf("/Date(") != -1)
                    jsonObj[j][newmodel.xName] = new Date(parseInt(jsonObj[j][newmodel.xName].substr(6)));
                if (jQuery.type(jsonObj[j][newmodel.xName]) != "string" && jQuery.type(jsonObj[j][newmodel.xName]) != "date")
                    xdata.push(parseFloat(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.xName]) ? jsonObj[j][newmodel.xName] : j));
                else {
                    xdata.push(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.xName]) ? jsonObj[j][newmodel.xName] : j);
                }
            }

            if (!ej.util.isNullOrUndefined(newmodel.yName)) {
                for (var k = 0; k < newmodel.yName.length; k++) {
                    var yNVal = [];
                    for (var yI = 0; yI < jsonObj.length; yI++) {
                        yNVal.push(jsonObj[yI][newmodel.yName[k]] === null || jsonObj[yI][newmodel.yName[k]] === undefined ? jsonObj[yI][newmodel.yName[k]] : parseFloat(jsonObj[yI][newmodel.yName[k]]));
                    }
                    yValues[k] = yNVal;
                    index = k + 1;
                }
            }

            var endDateTime = new Date(Math.max.apply(null, xdata)),
                startDateTime = new Date(Math.min.apply(null, xdata));
            if (endDateTime.getTime() == startDateTime.getTime()) {
                startDateTime.setHours(0);
                endDateTime.setHours(23);
            }
            var minX = startDateTime.getTime(), maxX = endDateTime.getTime(), scrollRangeSettings = this.model.scrollRangeSettings;
            
            if (this.model.enableScrollbar) {
                if ((!this._scrollRange && !this.scrollbarUpdate) || !this.scrollbarUpdate) {
                    var args = [minX, maxX, startDateTime, endDateTime]
                    this.calculateZoomingRange.apply(this, args);
                } else {
                    this._scrollRange = {
                        min: Math.min(this._scrollRange.min, minX),
                        max: Math.max(this._scrollRange.max, maxX),
                        delta: maxX - minX
                    };
                }
               
            } else
                this.setRange.call(this, minX, maxX, startDateTime, endDateTime);
        },        

        calculateZoomingRange: function (minX, maxX, startDateTime, endDateTime) {
            this.calculateZoomFactorPosition.call(this, minX, maxX, startDateTime, endDateTime);
            this.setRange.call(this, minX, maxX, startDateTime, endDateTime);
            return 0;
        },

        setRange: function (start, end, startDate, endDate) {
            var rangeSettings = this.model.rangeSettings;
            if (rangeSettings.start != "" && rangeSettings.end != "") {
                this.startDateTime = new Date(rangeSettings.start);
                this.endDateTime = new Date(rangeSettings.end);
                this.startValue = rangeSettings.start;
                this.endValue = rangeSettings.end;
            } else {
                this.startDateTime = new Date(startDate);
                this.startValue = start;
                this.endDateTime = new Date(endDate);
                this.endValue = end;
            }
            return 0;
        },

        calculateZoomFactorPosition: function (startX, endX, startDateTime, endDateTime) {
            var model = this.model,
                rangeSettings = model.rangeSettings;
            var delta, zoomPos, zoomFact, start = startDateTime, end = endDateTime,
                min = startX, max = endX;
            var scrollRangeSettings = model.scrollRangeSettings;
            if (model.valueType.toLowerCase() == "datetime") {
                //VirtualRange start value is not given, it takes dataSource start value
                start = Date.parse(scrollRangeSettings.start ? new Date(scrollRangeSettings.start) : startDateTime);
                //If the given virtual range start value is greater than datasource value, it takes the minimum vallue from datasource
                start = Math.min(start, Date.parse(startDateTime));
                start = rangeSettings.start != "" ? Math.min(start,Date.parse(new Date(rangeSettings.start))) : start;
                //VirtualRange end value is not given, it takes dataSource end value
                end = Date.parse(scrollRangeSettings.end ? new Date(scrollRangeSettings.end) : endDateTime);
                //If the given virtual range end value is less than datasource value, it takes the maximum vallue from datasource
                end = Math.max(end, Date.parse(endDateTime));
                end = rangeSettings.end != "" ? Math.max(end, Date.parse(new Date(rangeSettings.end))) : end;
                delta = end - start;
                if (!model.enableRTL) {
                    zoomPos = (startDateTime - start) / delta;
                    zoomFact = (endDateTime - startDateTime) / delta;
                } else {
                    zoomPos = (end - endDateTime) / delta;
                    zoomFact = (endDateTime - startDateTime) / delta;
                }
                this._scrollRange = { min: start, max: end, delta: end - start };
            } else {
                //VirtualRange Start value is not given, it takes dataSource start value
                min = scrollRangeSettings.start != "" ? parseFloat(scrollRangeSettings.start) : startX;
                //If the given virtual range start value is greater than datasource value, it takes the minimum vallue from datasource
                min = Math.min(min, startX);
                min = rangeSettings.start != "" ? Math.min(min, parseFloat(rangeSettings.start)) : min;
                //VirtualRange end value is not given, it takes dataSource end value
                max = scrollRangeSettings.end != "" ? parseFloat(scrollRangeSettings.end) : endX;
                //If the given virtual range end value is less than datasource value, it takes the maximum vallue from datasource
                max = Math.max(max, endX);
                max = rangeSettings.end != "" ? Math.max(max, parseFloat(rangeSettings.end)) : max;
                delta = max - min;
                if (!model.enableRTL) {
                    zoomPos = (startX - min) / delta;
                    zoomFact = (endX - startX) / delta;
                } else {
                    zoomPos = (max - endX) / delta;
                    zoomFact = (endX - startX) / delta;
                }
                this._scrollRange = { min: min, max: max, delta: max - min };
            }
            this.scrollZoomPos = zoomPos;
            this.scrollZoomFact = zoomFact;

            return 0;
        },
        mousePosition: function (evt) {
            if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return { x: evt.pageX, y: evt.pageY };
            else if (evt.originalEvent && !ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX > 0)
                return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
            else if (evt.originalEvent && evt.originalEvent.changedTouches != undefined) {
                if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                    return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
            }
            else
                return { x: 0, y: 0 };
        },
        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseposition = this.mousePosition(e);
            e.pageX = mouseposition.x;
            e.pageY = mouseposition.y;
            var mouseX, mouseY;
            var browser = matched.browser.toLowerCase();
            mouseX = (e.pageX) - $(this.svgDocument).offset().left;
            mouseY = (e.pageY) - $(this.svgDocument).offset().top;
			this.leftPadding = $(this.svgDocument).offset().left;


            return { X: mouseX, Y: mouseY };

        },

        calMousePosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;
            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                mouseX = (e.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = 0;//$(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else {
                mouseX = (e.originalEvent?e.originalEvent.pageX:e.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent?e.originalEvent.pageY:e.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        _performSelection: function (evt){

            var naviobj = this,
                model = naviobj.model,
                selectionOptions = model.navigatorStyleSettings.selectionSettings,
                uncolor = selectionOptions.color,
                targetParentId = evt.target.parentNode.id,
                isUnhighlight = (targetParentId == "leftslider" || targetParentId == "rightslider" || targetParentId == naviobj._id + "thumbleft" || targetParentId == naviobj._id + "thumbright"),
                valueType = naviobj.model.valueType;

            naviobj._sliderHover = isUnhighlight || naviobj.switched;
            naviobj.selectionGrad = naviobj.renderer.createGradientElement('selectionGrad', uncolor, 150, 0, 150, 100, naviobj.svgDocument);
            naviobj.isNumericType = targetParentId != naviobj._id;
            if (((naviobj.isNumericType && valueType == 'numeric') || valueType != 'numeric') && (!isUnhighlight) && !naviobj.switched) {
                $("#highlightRect" + naviobj._id).hide();
                if ($("#selectionRect" + naviobj._id).length == 0) {
                    naviobj.gselection = naviobj.renderer.createGroup({ 'id': 'selectionRect' + naviobj._id });
                    var selectionRectOptions = {
                        'height': naviobj.sliderHeight,
                        'opacity': selectionOptions.opacity,
                        'stroke': selectionOptions.border.color,
                        'stroke-width': selectionOptions.border.width
                    };
                    naviobj.renderer.drawRect(selectionRectOptions, naviobj.gselection);
                    naviobj.selectionRect = naviobj.gselection.firstChild;
                    naviobj.renderer.append(naviobj.gselection, naviobj.styleRect);
                }
                naviobj.rect = naviobj.selectionRect;
                naviobj.startX = naviobj.mouseDownX;
                naviobj.renderer._setAttr($(naviobj.selectionRect), { 'fill': naviobj.selectionGrad });
                naviobj.renderer._setAttr($(naviobj.selectionRect), { 'stroke': selectionOptions.border.color });
                naviobj._highlight.call(naviobj, naviobj, evt);
            }

        },


        _performHighlight: function (evt) {

            var naviobj = this,
               model = naviobj.model,
               targetId = evt.target.parentNode.id,
               highlightOptions = model.navigatorStyleSettings.highlightSettings,
               uncolor = highlightOptions.color,
               isUnhighlight = (targetId == "leftslider" || targetId == "rightslider" || targetId == this._id + "thumbleft" || targetId == this._id + "thumbright");

            this.highlightGrad = this.renderer.createGradientElement('highlightGrad', uncolor, 150, 0, 150, 100, this.svgDocument);

            $("#highlightRect" + this._id).hide();
            if (!isUnhighlight && !this.isSelection && !this._sliderHover && !naviobj.switched) {
                if ($("#highlightRect" + this._id).length == 0) {
                    this.gHighlight = this.renderer.createGroup({ 'id': 'highlightRect' + this._id });
                    var highlightRectOptions = {
                        'height': this.sliderHeight,
                        'fill': this.highlightGrad,
                        'opacity': highlightOptions.opacity,
                        'stroke': highlightOptions.border.color,
                        'stroke-width': highlightOptions.border.width
                    };
                    this.renderer.drawRect(highlightRectOptions, this.gHighlight);
                    this.highlightRect = this.gHighlight.firstChild;
                    this.renderer.append(this.gHighlight, this.styleRect);
                }
                this.rect = this.highlightRect;
                naviobj.startX = naviobj.mouseDownX;
                this._highlight.call(this, naviobj, evt);
            }

        },

        isTouch: function (evt) {
            var event = evt.originalEvent ? evt.originalEvent : evt;
            if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        },

        _rightClick: function (evt){
            var browserInfo = this.model.browserInfo;
            if (this.isTouch(evt) && this.model.navigatorStyleSettings.selectionSettings.enable) {
                event.preventDefault();
                event.stopPropagation();
                return false;
            }
        },
        _grab: function (evt) {
            evt.preventDefault();
            var naviobj = this,
                model = naviobj.model,
                browserInfo = model.browserInfo,
                mousedownCords,
                istouch = this.isTouch(evt),
                selectionOptions = model.navigatorStyleSettings.selectionSettings,
                mousedownCords = istouch ? naviobj.calTouchPosition(evt) : naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            if (selectionOptions.enable) this._performSelection(evt);
            // find out which element we moused down on
            
            var targetElement;
            if (naviobj.edge == true) {
                targetElement = naviobj.target;
            }
            else {
                targetElement = evt.target;
            }
            // used to find a thumb id
            var current = targetElement;
            var targetId;
            while (current.parentNode) {
				current = current.parentNode
				if (current.id == this._id + "thumbleft")
                {
                    targetId = this._id + "thumbleft";
                }
                else if (current.id == this._id + "thumbright")
                {
                    targetId = this._id + "thumbright";
                }
            }
            var isLower,
            isHigher,
            labelRegion,labelRegionX,labelRegionY,labelType,
            regionLength = naviobj.labelRegions.length,
            mouseX = naviobj.mouseDownX,
            mouseY = naviobj.mouseDownY;
            for (var k = 0; k < regionLength; k++) {
                labelRegion = naviobj.labelRegions[k];
                labelRegionX = labelRegion.X;
                labelRegionY = labelRegion.Y;
                labelType = labelRegion.LabelType;
                if ((labelRegionX <= mouseX && (labelRegionX + labelRegion.Width >= mouseX)) && (labelRegionY <= mouseY && ((labelRegionY + labelRegion.Height) >= mouseY))) {
                       isLower = labelType == "lowerLevel" ? true : false;
                       isHigher= labelType == "higherLevel" ? true : false;
                        break;
                    }                
            }

            var isDraggable = false;
            naviobj.isSelection = false;
            if (naviobj.highlightRect) {
                var highlightCord = naviobj.highlightRect.getBoundingClientRect();
                var centerCord = naviobj.centerSlider.getBoundingClientRect();
                if (highlightCord.left >= centerCord.left && highlightCord.right <= centerCord.right)
                    isDraggable = true;
            }
            
            if (selectionOptions.enable && naviobj.selectionRect && !(targetElement.parentNode.id === "leftslider" || targetElement.parentNode.id === "rightslider" || targetId == this._id + "thumbleft" || targetId == this._id + "thumbright")) {
                naviobj.isSelection = true;
                naviobj.startX = naviobj.mouseDownX;
            }
            if (istouch && !naviobj.switched) {
                var leftValue = this._leftValue,
                    rightValue = this._rightValue,
                    padding = 10,
                    leftThumb = $("#" + this._id + "thumbleft"),
                    rightThumb = $("#" + this._id + "thumbright");
                if (mouseX >= (leftValue.left - padding) && mouseX <= (leftValue.right + padding)) {
                    targetElement = this.leftCircle ? this.leftCircle : leftThumb[0].firstChild;
                    targetId = this.leftCircle ? targetId : this._id + "thumbleft";
                    this._sliderMove = true;
                }
                if (mouseX >= (rightValue.left - padding) && mouseX <= (rightValue.right + padding)) {
                    targetElement = this.rightCircle ? this.rightCircle : rightThumb[0].firstChild;
                    targetId = this.rightCircle ? targetId : this._id + "thumbright";
                    this._sliderMove = true;
                }

            }
            if ((targetElement.parentNode.id === "leftslider" || targetElement.parentNode.id === "rightslider" || targetId == this._id + "thumbleft" || targetId == this._id + "thumbright" || (targetElement.parentNode.id === "highlightRect" + this._id && isDraggable) || targetElement.parentNode.id === "centerslider") || (isLower || isHigher) || naviobj.isSelection) {
                //set the item moused down on as the element to be dragged
                if (targetElement.parentNode.id === "highlightRect" + this._id)
                    naviobj.dragTarget = $("#centerslider").children()[0];
                else
                    naviobj.dragTarget = targetElement;
                naviobj.grabbed = true;
                naviobj.edge = false;
                var dragParentId = naviobj.dragTarget.parentNode.id;
                if (this.svgSupport) {
                    if (targetId == this._id + "thumbleft" || targetId == this._id + "thumbright")
                        var transMatrix = {e:evt.clientX,f:evt.clientY};
                    else
                        var transMatrix = naviobj.dragTarget.getCTM();
                    naviobj.grabPoint.x = naviobj.mouseDownX + this.leftPadding - Number(transMatrix.e);
                    naviobj.grabPoint.y = naviobj.trueCoords.y - Number(transMatrix.f);
                }
                var txtPosition;
                var i;
                var flag = false;
                var isrtl = naviobj.model.enableRTL;
                if (isrtl) {
                    naviobj._higherLineLeft.sort(function (a, b) { return a - b });
                    naviobj._lowerLineLeft.sort(function (a, b) { return a - b });
                }
                if (dragParentId != 'leftslider' && dragParentId != 'rightslider' && naviobj.model.valueType == "datetime" && isHigher) {
                    if (naviobj.dragTarget.parentNode.id == "centerslider")
                        naviobj.grabbed = true;
                    else
                        naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i=0;i< naviobj._higherLineLeft.length;i++) {
                        if (i + 1 < naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i] && txtPosition < naviobj._higherLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[0]);
                        flag = false;
                    }
                }
                else if (dragParentId != 'leftslider' && dragParentId != 'rightslider' && naviobj.model.valueType == "datetime" && isLower) {
                    naviobj.center = naviobj.mouseDownX
                    if (naviobj.dragTarget.parentNode.id == "centerslider")
                        naviobj.grabbed = true;
                    else
                        naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[0]);
                        flag = false;
                    }
                }
                else if (naviobj.isSelection) {
                    naviobj.grabbed = true;
                }

            }
        },

        _leave: function (evt) {
            $("#highlightRect" + this._id).hide();
        },
        _drag: function (evt) {

            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);

            var naviobj = this;//evt.data.Param1;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                var mousedownCords = naviobj.calMousePosition(evt);
            }
            else if (!this.svgSupport)
                var mousedownCords = naviobj.calMousePosition(evt);
            else
                var mousedownCords = naviobj.calTouchPosition(evt);//naviobj.calMousePosition(evt);
            mouseX = naviobj.mouseDownX = mousedownCords.X;
            mouseY = naviobj.mouseDownY = mousedownCords.Y;
            var labelRegion, labelregionX, labelregionY, xLeft, newX;;
            var highlightOptions = naviobj.model.navigatorStyleSettings.highlightSettings;
            if (highlightOptions.enable && !this._sliderMove && !this.isTouch(evt))
                this._performHighlight(evt);
            if(this.svgSupport)
            naviobj.GetTrueCoords(evt);
            //Apply css cursor as pointer to lower and higher level labels
            if (naviobj.model.labelSettings.higherLevel.visible || naviobj.model.labelSettings.lowerLevel.visible) {
                var regionLength = naviobj.labelRegions.length;
                for (var k = 0; k < regionLength; k++) {
                    labelRegion = naviobj.labelRegions[k];
                    labelregionX = labelRegion.X; labelregionY = labelRegion.Y;
                    if ((labelregionX <= mouseX && (labelregionX + labelRegion.Width >= mouseX)) && (labelRegion.Y <= mouseY && ((labelRegion.Y + labelRegion.Height) >= mouseY))) {
                        $(naviobj.rightUnSelected).css('cursor', 'pointer');
                        $(naviobj.leftUnSelected).css('cursor', 'pointer');
                        break;
                    }
                    else {
                        $(naviobj.rightUnSelected).css('cursor', 'default');
                        $(naviobj.leftUnSelected).css('cursor', 'default');
                    }
                }
            }
            // if we don't currently have an element in tow, don't do anything
            if (naviobj.dragTarget) {
                // account for the offset between the element's origin and the
                //    exact place we grabbed it... this way, the drag will look more natural
                if (!naviobj.edge && naviobj.dragTarget === naviobj.leftSlider || naviobj.dragTarget === naviobj.rightSlider || naviobj.dragTarget === naviobj.highlightRect || naviobj.dragTarget === naviobj.centerSlider) {
                    if (this.svgSupport) {
					if(matched.browser.toLowerCase() == "webkit"||matched.browser.toLowerCase() == "mozilla")
					{
                        var newX = naviobj.mouseDownX - naviobj.grabPoint.x;
						 if (newX < this.padding) newX = this.padding;
                        var newY = naviobj.mouseDownY - naviobj.grabPoint.y;
						}
						else
						{
						var newX = naviobj.trueCoords.x - naviobj.grabPoint.x;
						if (newX < this.padding) newX = this.padding;
                        var newY = naviobj.trueCoords.y - naviobj.grabPoint.y;
						}
                    }
                    else {
                        var newX = naviobj.mouseDownX;
                        var newY = naviobj.mouseDownY;
                    }
                }
                else {
                    var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    //  var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    var newY = naviobj.mouseDownY;// naviobj.trueCoords.y;
                } // used to find a id of the thumb
                var current = naviobj.dragTarget;
                var targetId;
                while (current.parentNode) {
                    current = current.parentNode
                    if (current.id == this._id + "thumbleft") {
                        targetId = this._id + "thumbleft";
                    }
                    else if (current.id == this._id + "thumbright") {
                        targetId = this._id + "thumbright";
                    }
                }
                naviobj.diff = naviobj.leftSlider.getBoundingClientRect().left - naviobj.rightSlider.getBoundingClientRect().left;
                naviobj.leftdiff = naviobj.rightSlider.getBoundingClientRect().left - this.padding - $(this.svgDocument).offset().left;
                naviobj.rightdiff = naviobj.leftSlider.getBoundingClientRect().left - this.padding - $(this.svgDocument).offset().left;
                if (naviobj.dragTarget.parentNode.id === "leftslider" || targetId == this._id + "thumbleft") {
                    naviobj.grabbed = false;
                    if (naviobj.diff >= -1 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.rightdiff<naviobj.newWidth-(2*this.padding)) {
                        naviobj.target = naviobj.rightSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        if (newX <= this.rightSliderPosition) {
                            naviobj.setSliderPositions(newX, null, null);
                            this.leftSliderPosition = newX;
                            this.leftset = false;
                        }
                        else {
                            naviobj.setSliderPositions(this.rightSliderPosition, null, null);
                            this.leftset = true;
                            this.leftSliderPosition = this.rightSliderPosition;
                        }
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.left = newX;
                    if (naviobj.isSelection) {
                        naviobj.isSelection = false;
                        this.renderer._setAttr($(this.selectionRect), { 'fill': 'transparent' });
                        this.renderer._setAttr($(this.selectionRect), { 'stroke': 'transparent' });
                    }
                }
                else if (naviobj.dragTarget.parentNode.id === "rightslider" || targetId == this._id + "thumbright") {
                    naviobj.grabbed = false;
                    // if (((this.svgSupport) && naviobj.leftSlider.getCTM().e == naviobj.rightSlider.getCTM().e && newX < naviobj.newWidth-15 && naviobj.rightSlider.getCTM().e == (naviobj.newWidth - naviobj.padding)) || (naviobj.leftSlider.getBoundingClientRect().left == naviobj.rightSlider.getBoundingClientRect().left && newX < naviobj.newWidth - 15 && naviobj.rightSlider.getBoundingClientRect().left == (naviobj.newWidth - naviobj.padding))) {                                                                
                    if (naviobj.diff >= 0 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.leftdiff >0) {
                        naviobj.target = naviobj.leftSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        //if (newX >= naviobj.leftSlider.getCTM().e)
                        if (newX >= this.leftSliderPosition) { // naviobj.leftSlider.getBoundingClientRect().left - this.leftPadding)
                            naviobj.setSliderPositions(null, null, newX);
                            this.rightSliderPosition = newX;
                        } else {
                            naviobj.setSliderPositions(null, null, this.leftSliderPosition);
                            this.rightSliderPosition = this.leftSliderPosition;
                        }
                        //naviobj.setSliderPositions(null, null, naviobj.leftSlider.getCTM().e);
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.right = newX;
                    if (naviobj.isSelection) {
                        naviobj.isSelection = false;
                        this.renderer._setAttr($(this.selectionRect), { 'fill': 'transparent' });
                        this.renderer._setAttr($(this.selectionRect), { 'stroke': 'transparent' });
                    }
                }
                else if ((naviobj.dragTarget.parentNode.id === "centerslider" || naviobj.dragTarget.parentNode.id === "highlightRect" + this._id) && !naviobj.isSelection) {
                    $('#highlightRect'+this._id).hide();
                    if (newX < this.padding) newX =parseFloat(this.padding);
                        if (!naviobj.grabbed) {
                        naviobj.setSliderPositions(null, newX, null);
                        naviobj.center = newX;
                    }
                    else  
                        naviobj.grabbed = false;                     
                    if (newX > newX + this.renderer._getAttrVal($(naviobj.dragTarget), 'width'))
                        naviobj.setSliderPositions(null, newX, null);                    
                }
                else if (naviobj.isSelection) {
                    this.rect = this.selectionRect;
                    this._highlight.call(this, naviobj, evt)
                    naviobj.grabbed = false;
                }
            }
        },

        //Highlight and Selection logic perform here 
        _highlight: function (naviobj, evt) {
            var lowerRegion = [],
                higherRegion = [],
                sliderHeight = this.sliderHeight,
                startX = naviobj.startX > naviobj.mouseDownX ? naviobj.mouseDownX : naviobj.startX,
                endX = naviobj.startX == startX ? naviobj.mouseDownX : naviobj.startX,
                clientY = naviobj.mouseDownY,
                x, y, width, isHigherLevel, height,
                labelPlacement = this.model.labelSettings.higherLevel.labelPlacement,
                lowerLevelLabel = this.model.labelSettings.lowerLevel.visible;
                targetId = evt.target.parentNode.id;

            $.each(naviobj.labelRegions, function (index, region) {
                x = region.X;
                y = region.Y;
                width = region.Width;
                isHigherLevel = region.LabelType == "higherLevel";
                if (isHigherLevel)
                    height = (targetId.indexOf('higherLevel') < 0 && lowerLevelLabel ) ? region.Height : sliderHeight;
                else
                   height = region.Height;
                if (startX <= x + width && x <= endX && (y + height) >= clientY) {
                    if (isHigherLevel) {
                        if (!(higherRegion.indexOf(region) > -1))
                            higherRegion.push(region);
                    } else {
                        if (!(lowerRegion.indexOf(region) > -1))
                            lowerRegion.push(region);
                    }

                }
            });
            var minimumRegion;
            var currentRegion = function (regions) {
                if (regions.length > 0) {
                    min = regions[0].X; max = regions[0].X;
                    minimumRegion = regions[0];
                    maximumRegion = regions[0];
                    for (var i = 0; i < regions.length; i++) {
                        if (regions[i].X < min) {
                            min = regions[i].X;
                            minimumRegion = regions[i];
                        }
                        else if (regions[i].X > max) {
                            max = regions[i].X;
                            maximumRegion = regions[i];
                        }
                    }
                }                

            }

            higherRegion.length > 0 ? currentRegion(higherRegion) : currentRegion(lowerRegion);
            if (!ej.isNullOrUndefined(minimumRegion)) {
                $("#" + this.rect.parentNode.id).show();
                this.renderer._setAttr($(this.rect), {
                    'transform': 'translate(' + minimumRegion.X + ',' + this.sliderPosition + ')',
                    'width': maximumRegion.Width + maximumRegion.X - minimumRegion.X
                });
            }
        },


        _drop: function (evt) {
            // if we aren't currently dragging an element, don't do anything
            $("#highlightRect" + this._id).hide();
            this._sliderMove = false;
            if (this.dragTarget) {
                // since the element currently being dragged has its pointer-events turned off,
                //    we are afforded the opportunity to find out the element it's being dropped on
                var targetElement = evt.target;
                this._sliderHover = false;
                var naviobj = this;
                // set the global variable to null, so nothing will be dragged until we
                //    grab the next element
                if ((naviobj.model.allowSnapping && parseInt(naviobj.leftSlider.getBoundingClientRect().left + 2) < parseInt(naviobj.rightSlider.getBoundingClientRect().left)) || naviobj.isSelection) {
                    var padding = parseInt(naviobj.model.padding),
                        rect = naviobj.isSelection ? naviobj.rect.getBoundingClientRect() : 0;
                    naviobj.ismouseup = true;
                    if (naviobj.dragTarget.parentNode.id === "leftslider")
                        var txtPosition = naviobj.left;
                    else if (naviobj.dragTarget.parentNode.id === "rightslider")
                        var txtPosition = naviobj.right;
                    else if (naviobj.isSelection)
                        var txtPosition = Math.abs(parseFloat(rect.left - $("#" + this._id)[0].getBoundingClientRect().left + padding));
                    else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                        var txtPosition = naviobj.leftSliderPosition;
                        var txtPosition1 = naviobj.rightSliderPosition;
                    }
                    var pos = 0,
                        pos1 = 0,
                        lineLength = naviobj._lowerLineLeft.length,
                        currentLine,
                        nextLine;

                    for (var i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition >= naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.snapValue = naviobj._lowerLineLeft[i];
                            pos = (naviobj._lowerLineLeft[i + 1] - naviobj.snapValue);
                            if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                for (var j = i + 1; j < naviobj._lowerLineLeft.length; j++) {
                                    if (j + 1 < naviobj._lowerLineLeft.length && txtPosition1 >= naviobj._lowerLineLeft[j] && txtPosition1 < naviobj._lowerLineLeft[j + 1]) {
                                        currentLine = naviobj._lowerLineLeft[j];
                                        nextLine = naviobj._lowerLineLeft[j + 1];
                                        pos1 = (nextLine - currentLine);
                                    }
                                }
                                txtPosition1 = (txtPosition1 >= (currentLine + (pos1 / 2))) ? nextLine : currentLine;
                            }
                            if (txtPosition >= (naviobj.snapValue + (pos / 2))) {
                                naviobj.snapValue = naviobj._lowerLineLeft[i + 1];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                                else if (naviobj.isSelection) {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1] + parseFloat(rect.right - rect.left));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                }
                                else if (naviobj.dragTarget.parentNode.id === "centerslider" && !naviobj.grabbed) {
                                    naviobj.setSliderPositions(null, null, txtPosition1);
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                }

                            }
                            else {
                                naviobj.snapValue = naviobj._lowerLineLeft[i];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i]);
                                else if (naviobj.isSelection) {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i] + parseFloat(rect.right - rect.left));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                }
                                else if (naviobj.dragTarget.parentNode.id === "centerslider" && !naviobj.grabbed) {
                                    naviobj.setSliderPositions(null, null, txtPosition1);
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                }

                            }
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition >= naviobj._lowerLineLeft[i]) {
                            if (naviobj.isSelection) {
                                naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i] + parseFloat(naviobj.rect.getBoundingClientRect().width));
                                naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            } else
                                naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                        }
                    }
                }
                this.dragTarget = null;
               
                if (this.leftTooltip != undefined && this.model.tooltipSettings.tooltipDisplayMode=="always") {
                    this.leftTooltip.setAttribute('opacity', 1);
                    this.rightTooltip.setAttribute('opacity', 1);
                }
                else if (this.leftTooltip != undefined && this.model.tooltipSettings.tooltipDisplayMode == "ondemand") {
                    if (this.isTouch(evt)) {
                        var tooltip = [this.leftTooltip, this.rightTooltip]
							window.clearTimeout(this.model.timer);
							this.model.timer = setTimeout(function () {
								$(tooltip).fadeOut(500);
							}, 1200);
                    } else {
                        this.leftTooltip.setAttribute('opacity', 0);
                        this.rightTooltip.setAttribute('opacity', 0);
                    }
                }
                if (this.model.enableDeferredUpdate) {
                    this._calculateSelectedData();
                    if (!naviobj.grabbed || naviobj.isSelection) {
                        this._calculateSliderZoomFactPosition();
                        if (this.model.valueType == "datetime") {
                            if (this.startDateTime.getTime() !== new Date(this.model.selectedRangeSettings.start).getTime() || this.endDateTime.getTime() !== new Date(this.model.selectedRangeSettings.end).getTime()) 
                                this._trigger("rangeChanged", this.model);
                        }
                        else {
                            if (this.startValue !== this.model.selectedRangeSettings.start || this.endValue !== this.model.selectedRangeSettings.end)
                                this._trigger("rangeChanged", this.model);
                        }
                    }
                }
                if(naviobj.isSelection){
                    naviobj.isSelection = false;
                    this.renderer._setAttr($(this.selectionRect), { 'fill': 'transparent' });
                    this.renderer._setAttr($(this.selectionRect), { 'stroke': 'transparent' });
                }
            }
            this._setSelectedRange();
        },

        mouseup: function (evt) {
            if (this.dragTarget != null) {
                this._drop(evt);
                if (this.dragTarget != null) {
                    this.dragTarget.setAttributeNS(null, 'pointer-events', 'all');
                    this.dragTarget = null;
                }
                if (this.model.enableDeferredUpdate) {
                    this._calculateSelectedData();
                    this._calculateSliderZoomFactPosition();
                    this._trigger("rangeChanged", this.model);
                  
                }
            }
        },

        _setSelectedRange: function () {           
             
           this._selectedRangeStart(this.model.selectedRangeSettings.start);
           this._selectedRangeEnd(this.model.selectedRangeSettings.end);
          
        },

        GetTrueCoords: function (evt) {
            // find the current zoom level and pan setting, and adjust the reported
            //    mouse position accordingly
            var newScale = this.svgDocument.currentScale;
            var translation = this.svgDocument.currentTranslate;
            var matched = jQuery.uaMatch(navigator.userAgent);
            if (evt.originalEvent.toString() !== "[object TouchEvent]" && matched.browser.toLowerCase() != "msie") {
                this.trueCoords.x = (evt.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.clientY - translation.y) / newScale;
            }
            else if (matched.browser.toLowerCase() == "msie") {
                this.trueCoords.x = (evt.originalEvent.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.clientY - translation.y) / newScale;
            }
            else {
                this.trueCoords.x = (evt.originalEvent.touches[0].clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.touches[0].clientY - translation.y) / newScale;
            }
        },

        _getLocalizedLabels: function(){
           return ej.getLocalizedConstants(this.sfType, this.model.locale);
       }
    });
    ej.datavisualization.RangeNavigator.Locale = ej.datavisualization.RangeNavigator.Locale || {};

    ej.datavisualization.RangeNavigator.Locale['default'] = ej.datavisualization.RangeNavigator.Locale['en-US'] = {
        intervals: {
            quarter: { longQuarters: "Quarter", shortQuarters: "Q" },
            week: { longWeeks: "Week", shortWeeks: "W" }
        }
     };
	  ej.datavisualization.RangeNavigator.Locale['fr-FR'] = {
        intervals: {
            quarter: { longQuarters: "Trimestre", shortQuarters: "T" },
            week: { longWeeks: "Semaine", shortWeeks: "S" }
        }
	 };
     ej.datavisualization.RangeNavigator.IntervalType = {

         Auto: 'auto', 

         Years: 'years',

         Quarters: 'quarters',

         Months: 'months',

         Weeks: 'weeks',

         Days: 'days',

         Hours: 'hours',

         Minutes: 'minutes'
     };

     ej.datavisualization.RangeNavigator.Position = {

         Top: 'top',        

         Bottom: 'bottom'      

         
     };

     ej.datavisualization.RangeNavigator.FontStyle = {

         Normal: 'normal',

         Italic: 'italic',

         Bold: 'bold'
     };

     ej.datavisualization.RangeNavigator.FontWeight = {

         Regular: 'regular',

         Bold: 'bold',

         Lighter: 'lighter'
     };

     ej.datavisualization.RangeNavigator.HorizontalAlignment = {

         Left: 'left',

         Right: 'right',

         Middle: 'middle'
     };
     ej.datavisualization.RangeNavigator.LabelPlacement = {

         Inside: 'inside',

         Outside: 'outside'
      
     };    

     ej.datavisualization.RangeNavigator.ValueType = {

         Numeric: 'numeric',

         Datetime: 'datetime'

     };

     ej.datavisualization.RangeNavigator.RangePadding = {

         Additional: 'additional',

         Normal: 'normal',

         None: 'none',

         Round: 'round'        

     };
	 
	  ej.datavisualization.RangeNavigator.Theme = {

        Azure: 'azure',

        FlatLight: 'flatlight',

        Azuredark: 'azuredark',

        Lime: 'lime',

        LimeDark: 'limedark',

        Saffron: 'saffron',

        SaffronDark: 'saffrondark',

        GradientLight: 'gradientlight',

        GradientDark: 'gradientdark'
    };



})(jQuery, Syncfusion);

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};;
/**
* @fileOverview Plugin to style the Html Map elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/

(function ($, ej, undefined) {
    
    ej.widget({ "ejMap": "ej.datavisualization.Map" }, {

        validTags: ["div"],
        _initPrivateProperties: function () {
            this._rootgroup = null;
            this._bubblegroup = null;           
            this._scale = 1;
            this._prevDistance = 0;
            this._tileTranslatePoint = { x: 0, y: 0 };
            this._translatePoint = { x: 0, y: 0 };
            this._prevPoint = null;
            this._Tiles = [];
            this._prevScale = 0;
            this._tileDiv = null;
            this._containerWidth = 500;
            this._containerHeight = 400;
            this._baseTranslatePoint = { x: 0, y: 0 };
            this._isDragging = false;           
            this._prevLevel = 1;
            this._startPoint = { x: 0, y: 0 };
            this._stopPoint = { x: 0, y: 0 };
            this.mouseClickable = false;
            this._browser = null;
            this._baseScale = 0;
            this._mapBounds = null;
            this._svgns = "http://www.w3.org/2000/svg";
            this._ispanning = false;
            this._dragStartX = 0;
            this._isNavigationPressed = false;
			this._iskeyboardKeysPressed = false;
            this._isPolygonSelected = false;
            this._dragStartY = 0;
            this._width = 350;
            this._height = 350;
            this._isMapCoordinates = true;
            this._margintop = 0;
            this._marginleft = 0;
            this._svgDocument = null;
            this._tooltipElement = null;
            this._templateDiv = null;
            this._scrollLegendDiv = null;
            this._legendContainer = null;
            this._legendDiv = null;
            this._legendDivHeight = 0;
            this._legendDivWidth = 0;
            this._mapContainer = null;
            this._isSVG = true;
            this._VMLPathFractionCount = 0;
            this._sliderControl = null; 
            this._isTileMap=false;
            this._isRendered = false;
            this._urlTemplate = null;
            this._polylineCount = 0;
            this._pointscount = 0;
            this._isPinching = false;
            this._groupSize = null;			
			this._groupBorder = { x: 0, y: 0 };
        },


        defaults: {
			locale: null,
			enableGroupSeparator: false,
            background: "transparent",			
            zoomSettings:{              
                minValue: 1,
                maxValue: 100,
                factor: 1,
                level: 1,
                enableZoomOnSelection: false,
                enableZoom: true,
            },          
            centerPosition: null,           
            enableResize: true,
            isResponsive: true,
            enableAnimation: false,           
            draggingOnSelection: false,
            navigationControl:{			
				enableNavigation:false,						
				content:null,
				orientation:'vertical',			
				absolutePosition:{x:0,y:0},								
				dockPosition:'centerleft',
			},           
            enableLayerChangeAnimation: false,
            enablePan: true,
            zoomLevel: 1,
            minZoom: 1,
            maxZoom: 100,
            zoomFactor : 1,
            baseMapIndex: 0,
            shapeSelected: "",
            markerSelected: "",
            zoomedIn: "",
            onRenderComplete: "",
			refreshed:"",
            panned: "",            
            zoomedOut: "",           
            mouseover : "",
            mouseleave : "",                                  
            layers: []
        },

        observables: [            
            "baseMapIndex",
            "enablePan",
            "enableResize",
            "enableAnimation",
            "zoomLevel",
			"minZoom",
            "maxZoom",
            "zoomFactor",
        ],

        _zoomLevel: ej.util.valueFunction("zoomLevel"),
        _minZoom: ej.util.valueFunction("minZoom"),
        _maxZoom: ej.util.valueFunction("maxZoom"),
        _zoomFactor: ej.util.valueFunction("zoomFactor"),
        baseMapIndex: ej.util.valueFunction("baseMapIndex"),
        enablePan: ej.util.valueFunction("enablePan"),
        enableResize: ej.util.valueFunction("enableResize"),
        enableAnimation: ej.util.valueFunction("enableAnimation"),

		_tags: [{
            tag: "layers",
            attr: ["legendSettings.showLegend", "legendSettings.positionX", "legendSettings.positionY", "legendSettings.type", "legendSettings.labelOrientation",
                    "legendSettings.title", "legendSettings.mode", "legendSettings.position", "legendSettings.dockOnMap", "legendSettings.dockPosition",
                    "labelSettings.showLabels", "labelSettings.labelPath", "labelSettings.enableSmartLabel",
                    "legendSettings.leftLabel", "legendSettings.rightLabel", "bubbleSettings.showBubble", "bubbleSettings.valuePath", "bubbleSettings.minValue",
                    "bubbleSettings.maxValue", "bubbleSettings.color", "enableSelection", "enableMouseHover", "showTooltip", "showMapItems", "mapItemsTemplate", "shapeData",
                    "dataSource", "shapePropertyPath", "shapeDataPath", "layerType", "bingMapType", "urltemplate", "shapeSettings.highlightColor",
                    "shapeSettings.highlightBorderWidth", "shapeSettings.selectionColor", "shapeSettings.fill", "shapeSettings.strokeThickness", "shapeSettings.selectionStrokeWidth",
                    "shapeSettings.stroke", "shapeSettings.selectionStroke", "shapeSettings.highlightStroke", "shapeSettings.colorValuePath", "shapeSettings.valuePath",
                    "shapeSettings.autoFill", "shapeSettings.enableGradient", "shapeSettings.colorPalette", "shapeSettings.customPalette",
                   
			        [{
	                    tag: "markers",
	                    attr: ["label", "latitude", "longitude"],
	                    singular: "marker",
			        },
                    {
                        tag: "bubbleSettings.colorMappings.rangeColorMapping",
                        attr: ["from", "to", "color"],
                        singular: "bubblerangeColorMap",
                    },
                    {
                        tag: "bubbleSettings.colorMappings.equalColorMapping",
                        attr: ["value", "color"],
                        singular: "bubbleequalColorMap",
                    },
                    {
                        tag: "shapeSettings.colorMappings.rangeColorMapping",
                        attr: ["from", "to", "color", "gradientColors"],
                        singular: "shaperangeColorMap",
                    },
                    {
                        tag: "shapeSettings.colorMappings.equalColorMapping",
                        attr: ["value", "color"],
                        singular: "shapeequalColorMap",
                    },

			        {
	                    tag: "subLayers",
	                    attr: ["legendSettings.showLegend", "legendSettings.positionX", "legendSettings.positionY", "legendSettings.type", "legendSettings.labelOrientation",
                                "legendSettings.title", "legendSettings.mode", "legendSettings.position", "legendSettings.dockOnMap", "legendSettings.dockPosition",
                                "legendSettings.leftLabel", "legendSettings.rightLabel", "bubbleSettings.showBubble", "bubbleSettings.valuePath", "bubbleSettings.minValue",								
                                "bubbleSettings.maxValue", "bubbleSettings.color", "enableSelection", "enableMouseHover", "showTooltip", "showMapItems", "mapItemsTemplate", "shapeData",
								"dataSource", "shapePropertyPath", "shapeDataPath","layerType", "bingMapType", "key", "urltemplate", "shapeSettings.highlightColor",
								"shapeSettings.highlightBorderWidth", "shapeSettings.selectionColor","shapeSettings.fill", "shapeSettings.strokeThickness", "shapeSettings.selectionStrokeWidth",
								"shapeSettings.stroke", "shapeSettings.selectionStroke","shapeSettings.highlightStroke", "shapeSettings.colorValuePath", "shapeSettings.valuePath", 
								"shapeSettings.autoFill", "shapeSettings.enableGradient",
                                [{
                                    tag: "markers",
                                	attr: ["label", "latitude", "longitude"],
                                	    singular: "marker",
                                },
                                {
                                    tag: "bubbleSettings.colorMappings.rangeColorMapping",
                                    attr: ["from", "to", "color"],
                                    singular: "bubblerangeColorMap",
                                },
                                {
                                    tag: "bubbleSettings.colorMappings.equalColorMapping",
                                    attr: ["value", "color"],
                                    singular: "bubbleequalColorMap",
                                },
                                {
                                    tag: "shapeSettings.colorMappings.rangeColorMapping",
                                    attr: ["from", "to", "color","highlightcolor"],
                                    singular: "shaperangeColorMap",
                                },
                                {
                                    tag: "shapeSettings.colorMappings.equalColorMapping",
                                    attr: ["value", "color"],
                                    singular: "shapeequalColorMap",
                                }
                               ],
	                    ],
	                    singular: "subLayer",
			        }]
                ],
                singular: "layer",
		}],


		
       
        dataTypes: {
            layers: "array"
        },       

        _destroy: function () {            
            this._unWireEvents();
            if (_isSVG) {
                $(this.element).removeClass("e-datavisualization-map e-js").find("#svgDocument").remove();
            }
            else {
                $(this.element).removeClass("e-datavisualization-map e-js").find("#rootGroup").remove();
            }
        },
       
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "zoomSettings":
                        if (this.model.zoomSettings.enableZoom) {
                            this._zoomLevel(this.model.zoomSettings.level);
                            this._zoomFactor(this.model.zoomSettings.factor);
                            this._maxZoom(this.model.zoomSettings.maxValue);
                            this._minZoom(this.model.zoomSettings.minValue);
                            this.zoom(this._zoomLevel());                            
                        }
                        break;
                    case "zoomLevel":
                        this.model.zoomSettings.level = this._zoomLevel();
                        this.zoom(this._zoomLevel());
                        break;
                    case "zoomFactor":
                        this.model.zoomSettings.factor = this._zoomFactor();
                        this.zoom(this._zoomLevel());
                        break;
                    case "minZoom":
                        this.model.zoomSettings.minValue = this._minZoom();
                        this.zoom(this._zoomLevel());
                        break;
                    case "maxZoom":
                        this.model.zoomSettings.maxValue = this._maxZoom();
                        this.zoom(this._zoomLevel());
                        break;
                    case "baseMapIndex":
                        this.baseMapIndex(options[prop]);
                        var map = this;
						this._groupSize = null;
                        if (this.enableAnimation()) {
                            this.model.enableLayerChangeAnimation = true;
                            $(this._mapContainer).animate({ opacity: 0 }, 500, function () {
                                map.refresh(true);
                            });

                        } else {
                            this.refresh(true);
                        }

                        break;                    
                    case "background":                        
                     if ((baseLayer.layerType == "geometry")) {					
                        $(this._svgDocument).css("background", this.model.background);
						}
                }
            }
        },
		
	_layers: function(index, property, value, old){
	 if (property == "shapeSettings.fill" || property == "shapeSettings.strokeThickness" ||property =="shapeSettings.selectionColor" || property =="shapeSettings.highlightColor") {
                this.clearShapeSelection();
            }
		else if(property=="showMapItems" || property=="dataSource")
		 {
			this.refreshLayers();
		 }
		 else 
		 {
		   this.refresh(true);
		 }
    },

    _layers_markers: function(index, property, value, old){
    	   this.refreshMarkers();
    },

    _layers_subLayers: function(index, property, value, old){
    	  if(property=="showMapItems" || property=="dataSource")
		 {
			this.refreshLayers();
		 }
		 else 
		 {
		   this.refresh(true);
		 }
    },
       
        _init: function () {
            var proxy = this;
			this._navigationStyle = null;
            this.navigationControlData = null;
            this._initPrivateProperties();
            this._setZoomProperties();
			this._localizedLabels = this._getLocalizedLabels();
            $.each(this.model.layers, function (index) {
                proxy._layerInitialize(index);
            });
            this._mapContainer = this.element;
            $(this._mapContainer).css({ "overflow": "hidden" });
			  if (document.documentMode == 8) {
                $(this._mapContainer).css({ "overflow": "hidden" , "position" :"relative" });
            }
			  this._wireEvents();
            this.refresh();
            this._trigger("onRenderComplete");
            this._isRendered = true;
            if (_isSVG && this.model.enablePan) {
                $(this._svgDocument).pinchZoom(this._rootgroup, this);
            }
        },
        _setZoomProperties : function() {
		   if (typeof this.model.zoomSettings.factor != "function") {
            this._zoomFactor(this.model.zoomSettings.factor);
            this._zoomLevel(this.model.zoomSettings.level);
            this._minZoom(this.model.zoomSettings.minValue);
            this._maxZoom(this.model.zoomSettings.maxValue);
			}
        },
		
		_getLocalizedLabels: function(){
			return ej.getLocalizedConstants(this.sfType, this.model.locale);
		},
		
        _layerInitialize: function (layerindex) {
            var proxy = this;
            if (this.model.layers[layerindex] != null) {
                $.each(this.model.layers, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = new shapeLayer();
                    $.extend(obj, element);
                    $.extend(element, obj);
                    $.each(element.subLayers, function (subindex, subelement) {
                        subelement = proxy._checkArrayObject(subelement, subindex);
                        var subobj = new shapeLayer();
                        $.extend(subobj, subelement);
                        $.extend(subelement, subobj);
                    });
                });
            }
            else {

                this.layers[0] = new shapeLayer();

            }
        },
       
        _checkArrayObject: function (element, initialName) {

            var publicProperties = ["background", "zoomSettings","bubbleSettings", "minValue", "maxValue", "factor", "level", "enableZoomOnSelection", "enableZoom", "centerPosition", "enableResize", "isResponsive", "enableAnimation", "draggingOnSelection", "navigationControl", "enableNavigation", "orientation", "absolutePosition", "dockPosition", "enablePan", "baseMapIndex", "enableSelection", "selectionMode", "enableMouseHover", "shapeData", "dataSource", "showTooltip", "legendSettings", "showLegend", "showLabels", "position", "height", "width", "type", "mode", "dockOnMap", "dockPosition", "labelSettings", "labelPath", "enableSmartLabel", "smartLabelSize", "layerType", "showMapItems", "shapeSettings", "highlightColor", "highlightBorderWidth", "selectionColor", "fill", "strokeThickness", "stroke", "highlightStroke", "selectionStroke", "selectionStrokeWidth", "colorValuePath", "valuePath", "enableGradient", "autoFill", ""];

            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (publicProperties.indexOf(name) > -1) {
                    if (innerElement instanceof Array) {
                        proxy._checkArrayObject(innerElement, name);
                    }
                    else if (innerElement != null && typeof innerElement == "object") {
                        var allObjects = new shapeLayer();
                        proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                    }
                }
            });
            return element;
        },
       
        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
     
        _refreshWithAnimation: function () {
            this.model.layers[this.baseMapIndex()]._setMapItemsPositionWithAnimation(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._setMapItemsPositionWithAnimation(this);
            }
        },
       
        _resizeShape: function () {
            this.model.layers[this.baseMapIndex()]._resizeShapes(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._resizeShapes(this);
            }

        },
        
        _refrshLayers: function () {
            this.model.layers[this.baseMapIndex()]._setMapItemsPosition(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._setMapItemsPosition(this);
            }

        },
       
        _panTileMap: function (factorX, factorY, centerPosition) {
            
            var totalSize = Math.pow(2, this._zoomLevel()) * 256;
            this._tileTranslatePoint.x = (factorX / 2) - (totalSize / 2);
            this._tileTranslatePoint.y = (factorY / 2) - (totalSize / 2);
            var position = this._convertTileLatLongtoPoint(centerPosition[0],centerPosition[1]);
            this._tileTranslatePoint.x -= position.x - (factorX / 2);
            this._tileTranslatePoint.y -= position.y - (factorY / 2);

        },
       
        _generateTiles: function (zoomLevel) {
            var userLang = navigator.language || navigator.userLanguage;
            this.Tiles = [];
            var xcount, ycount;
            xcount = ycount = Math.pow(2, zoomLevel);
            var baseLayer = this.model.layers[this.baseMapIndex()];
            var endY = Math.min(ycount, ((-this._tileTranslatePoint.y + this._height) / 256) + 1);
            var endX = Math.min(xcount, ((-this._tileTranslatePoint.x + this._width) / 256) + 1);
            var startX = (-(this._tileTranslatePoint.x + 256) / 256);
            var startY = (-(this._tileTranslatePoint.y + 256) / 256);
            for (var i = Math.round(startX) ; i < Math.round(endX) ; i++) {
                for (var j = Math.round(startY) ; j < Math.round(endY) ; j++) {
                    var x = 256 * i + this._tileTranslatePoint.x;
                    var y = 256 * j + this._tileTranslatePoint.y;
                    if (x > -256 && x <= this._width && y > -256 && y < this._height) {
                        if (i >= 0 && j >= 0) {
                            var tile = new Tile(i, j);
                            tile.left = x;
                            tile.top = y;
                            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                                tile.src = this._getBingMap(tile, baseLayer.key, baseLayer.bingMapType, userLang);
                            } else {
                                tile.src = this._urlTemplate.replace("level", this._zoomLevel()).replace('tileX', tile.X).replace('tileY', tile.Y);
                            }
                            this.Tiles.push(tile);
                        }
                    }
                }
            }
            var proxTiles = $.extend(true, [], this.Tiles);
            for (var layerIndex = 0; layerIndex < this.model.layers[this.baseMapIndex()].subLayers.length;layerIndex++) {
                var layer = this.model.layers[this.baseMapIndex()].subLayers[layerIndex];
                if (layer.layerType == ej.datavisualization.Map.LayerType.OSM || layer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                    for (var tileIndex = 0; tileIndex < proxTiles.length; tileIndex++) {
                        var baseTile = proxTiles[tileIndex];
                        var subtile = $.extend(true, {}, baseTile);
                        if (layer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                            subtile.src = this._getBingMap(subtile, layer.key, layer.bingMapType, userLang);
                        } else {
                            subtile.src = layer.urlTemplate.replace("level", this._zoomLevel()).replace('tileX', baseTile.X).replace('tileY', baseTile.Y);
                        }
                        this.Tiles.push(subtile);
                    }

                }
            }
            this._arrangeTiles();
        },
       
        _getBingMap:function(tile,key,type,lang) {
            var quadKey = "";
            for (var i = this._zoomLevel() ; i > 0; i--)
            {
                var digit = 0;
                var mask = 1 << (i - 1);
                if ((tile.X & mask) != 0)
                {
                    digit++;
                }
                if ((tile.Y & mask) != 0)
                {
                    digit+=2;
                }
                quadKey = quadKey+""+digit;
            }
            var layerType;
            if (type == ej.datavisualization.Map.BingMapType.Aerial) {
                layerType = "A,G";
            }
            else if (type == ej.datavisualization.Map.BingMapType.AerialWithLabel) {
                layerType = "A,G,L";
            } else {
                layerType = "G,VE,BX,L,LA";
            }
            var imageUrl = "http://ak.dynamic.t2.tiles.virtualearth.net/comp/ch/" + quadKey + "?mkt=" + lang + "&ur=IN&it=" + layerType + "&shading=hill&og=45&n=z&Key=" + key;
            return imageUrl;
        },
               
        _arrangeTiles: function (){        
            var pathTemplate = "<div><div style='position:absolute;left:{{:left}}px;top:{{:top}}px;height:{{:height}}px;width:{{:width}}px;'><img src={{:src}}></img></div></div>";
            var tmpl2 = $.templates(pathTemplate);
            var htmlString = tmpl2.render(this.Tiles);
            var first = this._mapContainer[0].children[0];            
            this._tileDiv.html(htmlString);            
        },

        _generatePath: function () {
           
            var baseLayer = this.model.layers[this.baseMapIndex()];            
            var pathCollection = '';
            this._polylineCount = 0;
            this._pointscount = 0;
            _isSVG = (window.SVGSVGElement) ? true : false;

            
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Geometry) {
                if (baseLayer != undefined && baseLayer.shapeData != null) {
                    baseLayer._isBaseLayer = true;
                    this._isTileMap = false;
                    this._scale = this._zoomLevel();
                    pathCollection = this._readShapeData(baseLayer);
                }
            }
            else {                
                this._isTileMap = true;
                this._scale = Math.pow(2, this._zoomLevel() - this._zoomFactor());
            }
            for (var key = 0; key < baseLayer.subLayers.length; key++) {
                if (baseLayer.subLayers[key].layerType == ej.datavisualization.Map.LayerType.Geometry && baseLayer.subLayers[key].shapeData != null) {
                    baseLayer.subLayers[key]._isBaseLayer = false;
                    pathCollection += this._readShapeData(baseLayer.subLayers[key]);
                }
            }
            if (_isSVG) {
                var htmlstring = '<div style="position:relative;" id="tileDiv"/><svg xmlns="http://www.w3.org/2000/svg" style= "overflow:hidden;z-index:0;float:left;left:0,top:0"' +
                             'id="svgDocument"> <g id="rootGroup">' + pathCollection + '</g></svg>';
                this._mapContainer.html(htmlstring);               
                this._svgDocument = this.element.find("#svgDocument")[0];
                this._tileDiv = this.element.find("#tileDiv");
                
            }
            else {
                document.createStyleSheet().addRule(".vml", "behavior:url(#default#VML);display:inline-block");
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml', "#default#VML");
                var vmlGroup = '<div id="tileDiv"/><v:group   id = "rootGroup" style="position:absolute; width:' + this._width + 'px;height:' + this._height + 'px;" coordorigin = "0 0" coordsize="' + this._width + ' ' + this._height + '">' + pathCollection + '</v:group>';
                this._mapContainer.html(vmlGroup);                
                this._tileDiv = this.element.find("#tileDiv");
                this._bubblegroup = this._createGroup(false,"bubbleGroup");               
                this._bubblegroup.style.position = "absolute";
                this.element.append(this._bubblegroup);
            }
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.OSM || baseLayer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                this._urlTemplate = baseLayer.urlTemplate;
                if (this.model.centerPosition != null) {
                    this._panTileMap(this._width, this._height, this.model.centerPosition);
                }
                else {
                    this._panTileMap(this._width, this._height, [0, 0]);
                }
                this._generateTiles(this._zoomLevel());
				if(_isSVG)
				{
				  $(this._svgDocument).css("position","relative");
				}
            }
            if(_isSVG)
			{			
                $(this._mapContainer).css("position", "relative");
			}			
            this._rootgroup = this.element.find("#rootGroup")[0];

            this._on($(this.element), ej.eventType.mouseDown, { map: this }, this.dragDown);
            this._on($(this.element), ej.eventType.mouseMove, { map: this }, this.dragMove);
            this._on($(this.element), ej.eventType.mouseUp, { map: this }, this.dragUp);

        },

        _calculateBBox: function (polygonDatas, polylineDatas,pointData)
        {
            var minLatitude = maxLatitude = minLongitude = maxLongitude = 0;
            var isEntered = false;
            if (polygonDatas != null && polygonDatas.length > 0) {
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    if (currentData.geometry != null) {
                        coords = currentData.geometry.coordinates;
                    } else {
                        coords = currentData.coordinates;
                    }
                    var currentDataLength = coords.length;
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';
                        if (currentDataLength > 1) {
                            subData = coords[j][0];
                            if (subData.length <= 2) {
                                subData = coords[j];
                            }
                        } else {
                            subData = coords[j];
                        }
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                              if(subData[k][1]!=null && subData[k][0]!=null){
                                if (!isEntered) {
                                    minLatitude = maxLatitude = subData[k][1];
                                    minLongitude = maxLongitude = subData[k][0];
                                    isEntered = true;

                                } else {
                                    minLatitude = Math.min(minLatitude, subData[k][1]);
                                    maxLatitude = Math.max(maxLatitude, subData[k][1]);
                                    minLongitude = Math.min(minLongitude, subData[k][0]);
                                    maxLongitude = Math.max(maxLongitude, subData[k][0]);
                                }
                                }
                            }
                        } else {
                          if(subData[k][1]!=null && subData[k][0]!=null){
                            if (!isEntered) {
                                minLatitude = maxLatitude = subData[1];
                                minLongitude = maxLongitude = subData[0];
                                isEntered = true;
                            } else {
                                minLatitude = Math.min(minLatitude, subData[1]);
                                maxLatitude = Math.max(maxLatitude, subData[1]);
                                minLongitude = Math.min(minLongitude, subData[0]);
                                maxLongitude = Math.max(maxLongitude, subData[0]);
                            }
                           }
                        }
                    }
                }
            }
            if (pointData!=null && pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];
                    subData = currentData.geometry.coordinates;
                    if (!isEntered) {
                        minLatitude = maxLatitude = subData[1]; // (φ)
                        minLongitude = maxLongitude = subData[0];
                        isEntered = true;
                    } else {
                        minLatitude = Math.min(minLatitude, subData[1]);
                        maxLatitude = Math.max(maxLatitude, subData[1]);
                        minLongitude = Math.min(minLongitude, subData[0]);
                        maxLongitude = Math.max(maxLongitude, subData[0]);
                    }
                }
            }
            if (polylineDatas!=null && polylineDatas.length > 0) {
                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates;
                    if (polylineDatas[i].geometry == null) {
                        coordinates = polylineDatas[i].coordinates;
                    }
                    else {
                        coordinates = polylineDatas[i].geometry.coordinates;
                    }
                    for (var k = 0; k < coordinates.length; k++) {
                        if (!isEntered) {
                            minLatitude = maxLatitude = coordinates[k][1]; // (φ)
                            minLongitude = maxLongitude = coordinates[k][0];
                            isEntered = true;
                        } else {
                            minLatitude = Math.min(minLatitude, coordinates[k][1]);
                            maxLatitude = Math.max(maxLatitude, coordinates[k][1]);
                            minLongitude = Math.min(minLongitude, coordinates[k][0]);
                            maxLongitude = Math.max(maxLongitude, coordinates[k][0]);
                        }
                    }
                }
            }
            
			if (this._groupSize == null && !this._isMapCoordinates) {
            this._groupSize = { minX: minLongitude, maxX: maxLongitude, minY: minLatitude, maxY: maxLatitude };
             }
            return [[minLongitude, minLatitude], [maxLongitude, maxLatitude]];        
        },
        		
        _readShapeData: function (layer) {
            var map = this;
            if (layer.shapeData != null) {
                if (typeof layer.shapeData == 'string') {
                    var dataManger =new ej.DataManager(layer.shapeData);
                    var query = ej.Query().from(layer);
                    dataManger.executeQuery(query).done(function (e) {
                        if (e.result != null) {
                            return map._getPathCollection(e.result, layer);
                        }
                    });

                } else {
                 return this._getPathCollection(layer.shapeData, layer);
                }
                
            }
        },
       
        _getFactor:function()
        {
            var contHeight;
            var contWidth;
            var heightfactor;
            var widthfactor
            if (this._mapBounds != null) {
                var startpoint = this._convertTileLatLongtoPointForShapes(this._mapBounds[0][1], this._mapBounds[0][0], this._mapBounds);
                var endpoint = this._convertTileLatLongtoPointForShapes(this._mapBounds[1][1], this._mapBounds[1][0], this._mapBounds);
                var minDiff = 0;
                var widthfactor = heightfactor = 1;
                contHeight = endpoint.y - startpoint.y;
                contWidth = endpoint.x - startpoint.x;
            }
            else {
                contHeight = 500;
                contWidth = 500;
            }
            if (contHeight < this._height) {
                heightfactor = parseFloat(Math.abs(this._height / contHeight + "e+1").toString().split('.')[0] / 10);
            }
			else{
                heightfactor = (this._height / contHeight);
            }
            if (contWidth < this._width) {
                widthfactor = parseFloat(Math.abs(this._width / contWidth + "e+1").toString().split('.')[0] / 10);
            }
			else {
                widthfactor = (this._width / contWidth);
            }

            return Math.min(widthfactor, heightfactor);
        },

        _getPathCollection:function(data,layer)
        {
            var shapePath = "";
            var type = "";
            var dataManager="";
            if (data.features != null) {
                dataManager = new ej.DataManager(data.features);
                type = "geometry.type";
                if (layer._isBaseLayer) {
                    this._mapBounds = data.bbox;
                }
            }
            else if (data.geometries != null) {
                dataManager = new ej.DataManager(data.geometries);
                type = "type";
                if (layer._isBaseLayer) {
                    if (data.bbox != null)
                        this._mapBounds = [[data.bbox[0], data.bbox[1]], [data.bbox[2], data.bbox[3]]];
                    }                
            }
			if(data.geometries!=null || data.features != null){
            var queryPolyLine = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'LineString');
            var queryPolygon = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'Polygon');
            var queryMulitiPolygon = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'MultiPolygon');
            var polylineDatas = dataManager.executeLocal(queryPolyLine);           
            var polygonDatas = dataManager.executeLocal(queryPolygon);
            var queryPoint = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'Point');
            var pointData = dataManager.executeLocal(queryPoint);
            var multiPolygonDatas = dataManager.executeLocal(queryMulitiPolygon);
            this._polylineCount = this._polylineCount + polylineDatas.length;
            this._pointscount = this._pointscount + pointData.length;
            $.merge(polygonDatas, multiPolygonDatas);
            if (layer._isBaseLayer && layer.geometryType == ej.datavisualization.Map.GeometryType.Normal) {
                this._isMapCoordinates = false;
            }
            if (this._mapBounds == null && layer._isBaseLayer) {
                this._mapBounds = this._calculateBBox(polygonDatas, polylineDatas, pointData);
            }
			else if (this._mapBounds == null)
                   this._mapBounds = [[-180, -85], [180, 85]] ;

            var factor = this._getFactor();

			layer._newBounds = [];                 
            var pointHtml = "";
            if(this._groupSize != null)
			{
			
			minX = this._groupSize.minX;
			maxX = this._groupSize.maxX;
			minY = this._groupSize.minY;
			maxY = this._groupSize.maxY;
			}
            var minX, maxX, minY, maxY, flag = true;
            if (pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];                   
                    var check = '';
                    subData = currentData.geometry.coordinates;
                    var data = [];
                    var data_clean = [];                   
                    data.push({
                        lat: parseFloat(subData[1]),
                        lng: parseFloat(subData[0])                        
                    });                    
                    var l = 0;

                    var point = data[0];
                    latitude = point.lat; // (φ)
                    longitude = point.lng;   // (λ)                   
                    var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds,factor);
                    var boundingBox = this._mapBounds;
                    if ((longitude >= boundingBox[0][0] && longitude <= boundingBox[1][0] && latitude >= boundingBox[0][1] && latitude <= boundingBox[1][1])) {

                    if (flag && this._groupSize == null) {
                        minX = maxX = p.x;
                        minY = maxY = p.y;
                        flag = false;
                    }
                    else {
                        minX = Math.min(minX, p.x);
                        maxX = Math.max(maxX, p.x);
                        minY = Math.min(minY, p.y);
                        maxY = Math.max(maxY, p.y);
                    }

                    if (_isSVG) {
                        data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                    }
                    else {
                        data_clean.push({ x: p.x - ((10 / this._scale)/2), y: p.y-((10 / this._scale) / 2), lat: point.lat, lng: point.lng });
                    }

                    if (_isSVG) {
                        pointHtml += '<circle cx="' + data_clean[0].x + '" cy="' + data_clean[0].y + '" r="' + layer.shapeSettings.radius / this._scale + '" stroke="' + layer.shapeSettings.stroke + '" stroke-width="' + layer.shapeSettings.strokeThickness + '" fill="' + layer.shapeSettings.fill + '" />';
                    }
                    else {
                        var pointHtmlString = '<v:oval display="block" fill="' + layer.shapeSettings.fill + '" style="position:absolute;top: ' + data_clean[0].y + 'px;left:' + data_clean[0].x + 'px;width:' + 10 / this._scale + 'px;height:' + 10 / this._scale + 'px;stroke-width:' + layer.shapeSettings.strokeThickness + 'px;"></v:oval>';
                        pointHtml += pointHtmlString;
                    }
                  }  else {
                        pointData.splice(i, 1);
                        i--;
                    }
                }

            }            
            if (polygonDatas.length > 0)
            {
                
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    var midCoordinate = 0;
                    var midCoordinateLength;
                    var coords;
                    if (currentData.geometry != null) {
                        coords = currentData.geometry.coordinates;
                    }
                    else {
                        coords = currentData.coordinates;
                    }                    
                    midCoordinateLength = coords[0][0].length;
                    var currentDataLength = coords.length;
                    if (currentDataLength > 1) {
                        var lst = coords;
                        for (var index = 0; index < lst.length; index++) {
                            if (midCoordinateLength < lst[index][0].length) {
                                midCoordinateLength = lst[index][0].length;
                                midCoordinate = index;
                            }
                        }
                    }
                    var path_code = '';
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';                       
                        if (currentDataLength > 1) {
                            subData = coords[j][0];
                            if (subData.length <=2) {
                                subData = coords[j];
                            }
                        }
                        else {
                            subData = coords[j];
                        }
                        var data = [];
                        var data_clean = [];
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                                data.push({
                                    lat: parseFloat(subData[k][1]),
                                    lng: parseFloat(subData[k][0])
                                });
                            }
                        }
                        else {
                            data.push({
                                lat: parseFloat(subData[1]),
                                lng: parseFloat(subData[0])
                            });
                        }                       
                        for (var l = 0; l < data.length - 1|| (data.length==1 && l==0); l++) {
                            var point = data[l];
                            latitude = point.lat; // (φ)
                            longitude = point.lng;   // (λ                           
                            var boundingBox = this._mapBounds;
                            if (this._isTileMap||(longitude >= boundingBox[0][0] && longitude <= boundingBox[1][0] && latitude >= boundingBox[0][1] && latitude <= boundingBox[1][1])) {
                               
                                var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, boundingBox, factor);
                                if (flag && this._groupSize == null) {
                                    minX = maxX = p.x;
                                    minY = maxY = p.y;
                                    flag = false;
                                }
                                else {
                                    minX = Math.min(minX, p.x);
                                    maxX = Math.max(maxX, p.x);
                                    minY = Math.min(minY, p.y);
                                    maxY = Math.max(maxY, p.y);
                                }                                
                                if (_isSVG) {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                                else {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                            }
                            
                        }
                        if (j == midCoordinate) {
                            layer._newBounds.push(this._findMidPointofPoylgon(data_clean));
                        }
                        
                        if (data_clean.length > 0) {
                            if (_isSVG) {
                                path_code += "M" + (data_clean[0].x) + "," + (data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + (data_clean[m].x) + "," + (data_clean[m].y);
                                }
                                path_code += "Z";
                            }
                            else {
                                path_code += "m" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                path_code += " l" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + parseInt(data_clean[m].x) + "," + parseInt(data_clean[m].y);
                                }
                                path_code += " e";
                            }
                        }

                    }
                    if (path_code != "") {
                        if (_isSVG) {
                            shapePath += '<path class="mapShape" stroke=' + layer.shapeSettings.stroke +
                                        ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) +
                                         ' fill= ' + layer.shapeSettings.fill + ' d="' + path_code + '"  stroke-linejoin= round stroke-linecap= square></path>';
                        }
                        else {
                            shapePath += '<v:shape  style="width:' + this._width + 'px;height:' + this._height + 'px;"  coordsize="'
                                + this._width + ' ' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSettings.stroke +
                                           ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) + 'px"' +
                                            ' fillcolor= ' + layer.shapeSettings.fill + ' path="' + path_code + '"></v:shape>';
                        }
                    }
                    else {
                        polygonDatas.splice(i, 1);
                        i--;
                    }
                    
                }
                
            }
            
            if (polylineDatas.length > 0) {

                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates;
                    if (polylineDatas[i].geometry == null) {
                        coordinates = polylineDatas[i].coordinates;
                    }
                    else {
                        coordinates = polylineDatas[i].geometry.coordinates;
                    }
                    var linedata = '';
                    for (var k = 0; k < coordinates.length; k++) {
                        var factor = this._getFactor();
                        var points = this._convertTileLatLongtoPointForShapes(coordinates[k][1], coordinates[k][0], this._mapBounds,factor);
						
						 if (flag && this._groupSize == null) {
                        minX = maxX = points.x;
                        minY = maxY = points.y;
                        flag = false;
                    }
                    else {
                        minX = Math.min(minX, points.x);
                        maxX = Math.max(maxX, points.x);
                        minY = Math.min(minY, points.y);
                        maxY = Math.max(maxY, points.y);
                    }
                        linedata += points.x + "," + points.y;
                        if (k != coordinates.length - 1)
                            linedata += ",";
                    }

                    if (_isSVG) {
                        shapePath += '<polyline stroke=' + layer.shapeSettings.stroke +
                                    ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) + ' fill="transparent" stroke-width="1" points="' + linedata + '" ></polyline>';

                    }
                    else {
                        shapePath += '<v:polyline coordsize="'+ this._width + ',' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSettings.stroke +
                                                               ' strokeweight=' + (layer.shapeSettings.strokeThickness / this._scale) + 'px"' +
                                                                ' fillcolor=' + layer.shapeSettings.fill + '  points="' + linedata + '"/>';


                    }
                }

            }           
            if(this._isMapCoordinates)
            {
                this._groupSize = { minX: minX, maxX: maxX, minY: minY, maxY: maxY };
            }
            var lines = [].concat(polygonDatas, polylineDatas);
            layer._polygonData = $.merge(lines, pointData);
            return shapePath + pointHtml;
			}
			else{
			  layer._polygonData=[];
			    return null;
		   }
        },
             
        _createGroup: function (isRoot,name) {
            
            var vmlGroup;
            vmlGroup = document.createElement('<v:group class='+name+'>');
            vmlGroup.style.width = this._width + 'px';
            vmlGroup.style.height = this._height + 'px';            
            vmlGroup.coordorigin = "0 0";
            vmlGroup.coordsize = this._width + ' ' + this._height;
            if (isRoot) {
                this._rootgroup = vmlGroup;
                vmlGroup.style.left = '20px';
                vmlGroup.style.top = '20px';
            }
            return vmlGroup;
        },

        _generatePaletteColorsForShape: function (shape, shapelayer, palette, palettesettings) {

            if (palettesettings != null) {
                shapelayer.shapeSettings.highlightColor = palettesettings.highlightColor;
                shapelayer.shapeSettings.highlightStroke = palettesettings.highlightStroke;
                shapelayer.shapeSettings.selectionColor = palettesettings.SelectionColor;
                shapelayer.shapeSettings.selectionStroke = palettesettings.SelectionStroke;
                
            }

            if (_isSVG) {
                shape.setAttribute("fill", palette[shapelayer._prevPaletteIndex]);
            }
            else {
                shape.fillcolor = palette[shapelayer._prevPaletteIndex];
            }
            shapelayer._prevPaletteIndex = shapelayer._prevPaletteIndex + 1;
            if (shapelayer._prevPaletteIndex > palette.length - 1)
                shapelayer._prevPaletteIndex = 0;
        },

        _renderLayers: function (layer, pathCount) {

            layer._prevPaletteIndex = 0;
            layer._initializeLocalValues();   
            if (layer.selectedItems == null) {
                layer.selectedItems = [];
            }       
            var proxyrootGroup = this._rootgroup;
            var proxySVG = this._svgDocument;
            var proxyControl = this;
            var shapelayer = layer;
            if (shapelayer.legendSettings != null && shapelayer.shapeSettings.colorMappings != null && (shapelayer.legendSettings.type == undefined || shapelayer.legendSettings.type == ej.datavisualization.Map.LegendType.Layers))
                 shapelayer._generateLegends(this);
            if (shapelayer.legendSettings != null && shapelayer.bubbleSettings.colorMappings != null && (shapelayer.legendSettings.type == ej.datavisualization.Map.LegendType.Bubbles))
				 shapelayer._generateBubbleLegends(this);
            if (shapelayer.shapeSettings.colorMappings != null && shapelayer.shapeSettings.colorMappings.rangeColorMapping != null) {

                shapelayer.shapeSettings.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            if (shapelayer.bubbleSettings.colorMappings != null && shapelayer.bubbleSettings.colorMappings.rangeColorMapping != null) {
                shapelayer.bubbleSettings.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            var minValue = 0;
            var maxValue = 0;
            var sum = 0;
            if (shapelayer.dataSource != null && shapelayer.bubbleSettings != null) {
                if (shapelayer.bubbleSettings.colorValuePath  == null) {
                    shapelayer.bubbleSettings.colorValuePath  = shapelayer.bubbleSettings.valuePath;
                }
                for (var i = 0; i < shapelayer.dataSource.length; i++) {
                    var bubbledata = parseFloat(this._reflection(shapelayer.dataSource[i], shapelayer.bubbleSettings.valuePath));
                    if (i != 0) {
                        if (bubbledata > maxValue) {
                            maxValue = bubbledata;
                        } else if (bubbledata < minValue) {
                            minValue = bubbledata;
                        }
                    } else {
                        minValue = maxValue = bubbledata;
                    }
                }
            }

            this._generateTooltipForLayers(shapelayer);
            if (shapelayer.shapeData != null) {
                var shapesource = [];
                if (shapelayer.dataSource != null) {
                    for (var key = 0; key < shapelayer.dataSource.length; key++) {
                        var shapeitem = shapelayer.dataSource[key];
                        var shapeItemValue = proxyControl._reflection(shapeitem, shapelayer.shapeDataPath);
                        if (shapeItemValue != null) {
                            shapeItemValue = shapeItemValue.toLowerCase();
                        }
                        shapesource.push(shapeItemValue);
                    }
                }
                // touchevent changing
                var browserInfo = ej.browserInfo(),
                    temp = this.model,
                    isPointer = browserInfo.isMSPointerEnabled,
                    isIE11Pointer = browserInfo.pointerEnabled,
                    touchEndEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
                    touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
                    touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
                    touchCancelEvent = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchend mouseleave";
                for (var dataIndex = 0; dataIndex < shapelayer._polygonData.length; dataIndex++) {
                    var ValueObject = shapelayer._polygonData[dataIndex].properties;
                    var DBFValue = proxyControl._reflection(ValueObject, shapelayer.shapePropertyPath);
                    var dbfCondition;
                    if (DBFValue != null && (typeof DBFValue) == "string") {
                        dbfCondition = DBFValue.toLowerCase();
                    }
                    var shape = this._rootgroup.childNodes[dataIndex + pathCount];
					 if (_isSVG) {
                        shape.setAttribute("fill", shapelayer.shapeSettings.fill);
                    }
                    else {
                        shape.fillcolor = shapelayer.shapeSettings.fill;
                        shape.strokecolor = shapelayer.shapeSettings.stroke;
                    }
                    var obj = {};
                    var bounds = shapelayer._newBounds[dataIndex];

                    if (shapelayer.shapeSettings.autoFill) {
                        switch (shapelayer.shapeSettings.colorPalette) {
                            case ej.datavisualization.Map.ColorPalette.Palette1:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette1, shapelayer._colorPaletteSettings.Palette1);
                                break;
                            case ej.datavisualization.Map.ColorPalette.Palette2:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette2, shapelayer._colorPaletteSettings.Palette2);
                                break;
                            case ej.datavisualization.Map.ColorPalette.Palette3:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette3, shapelayer._colorPaletteSettings.Palette3);
                                break;
                            case ej.datavisualization.Map.ColorPalette.CustomPalette:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.shapeSettings.customPalette, null);
                                break;
                        }
                    }

                 if (shapesource.indexOf(dbfCondition) != -1) {
                        var item = shapelayer.dataSource[shapesource.indexOf(dbfCondition)];
                        var ItemValue = DBFValue;
                        if (item != null) {
                            if (shape != null) {                              
                               
                                var id = ItemValue;
                            }
                                layer._bounds.push(bounds);
                                if (shapelayer.showMapItems) {
                                    var itemTemplateDiv = $("<div class='mapItems' style='display:block;position:absolute;pointer-events: none;'></div>");
                                    itemTemplateDiv[0].className = ItemValue;
                                    proxyControl._templateDiv.append(itemTemplateDiv);
                                    $(itemTemplateDiv).css({ "left": bounds.x, "top": bounds.y });
                                    var htmlString;
									 var locale = this.model.locale;
									 var obj = $.extend({}, item);
									 if (locale && this.model.enableGroupSeparator){
										for (var key in obj)
											obj[key] = isNaN(parseFloat(obj[key])) ? obj[key] : parseFloat(obj[key]).toLocaleString(locale);
									 }
                                    if (shapelayer.mapItemsTemplate == null) {
                                        htmlString= $(' <div><label>{{:' + shapelayer.shapeSettings.valuePath + '}}</label></div>').render(obj);
                                    }
                                    else {
                                        htmlString = $("#" + shapelayer.mapItemsTemplate).render(obj);
                                    }
                                    $(itemTemplateDiv).html(htmlString);
                                    layer._mapItems.push(itemTemplateDiv);
                                }                                   
                                                             
                                if (shapelayer.shapeSettings.colorMappings != null) {
                                    if (shapelayer.shapeSettings.colorValuePath  == null) {
                                        shapelayer.shapeSettings.colorValuePath  = shapelayer.shapeSettings.valuePath;
                                    }
                                    var shapeValue = proxyControl._reflection(item, shapelayer.shapeSettings.colorValuePath );
                                    if (shapeValue != null && !shapelayer.shapeSettings.autoFill) {
                                        if (((shapelayer.legendSettings != null && shapelayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) || shapelayer.shapeSettings.enableGradient) && shapelayer.shapeSettings.colorMappings.rangeColorMapping != null) {
                                            obj["legendrect"] = shapelayer._updateLegendRange(shapeValue, shapelayer, shape);
                                            var shapeColor = shapelayer.shapeSettings.fill;
                                            if (obj["legendrect"] != undefined) {
                                                shapeColor = obj["legendrect"].color;
                                            }
                                            if (_isSVG) {
                                                shape.setAttribute("fill", shapeColor);
                                            }
                                            else {
                                                shape.fillcolor = shapeColor;
                                            }
                                        } else {
                                            shapelayer._fillColors(shapeValue, shapelayer.shapeSettings.colorMappings, shape);
                                        }
                                    }
                                }
                                else if (!shapelayer.shapeSettings.autoFill) {
                                    if (_isSVG) {
                                        shape.setAttribute("fill", shapelayer.shapeSettings.fill);
                                    }
                                    else {
                                        shape.fillcolor = shapelayer.shapeSettings.fill;
                                        shape.strokecolor = shapelayer.shapeSettings.stroke;
                                    }
                                }
                                if (shapelayer.bubbleSettings.showBubble) {
                                    if (shapelayer.bubbleSettings != null && shapelayer.bubbleSettings.valuePath != null) {
                                        if (_isSVG) {
                                            var circle = document.createElementNS(proxyControl._svgns, "circle");
                                        }
                                        else {
                                            var bubbleID = "bubble_" + id;
                                            var bubbleHtmlString = '<v:oval class="mapBubble" id=' + bubbleID + ' display="block" style="position:absolute;top: ' + 0 + 'px;left:' + 0 + 'px;width:100px;height:100px;stroke-width:' + 0 + 'px;"><v:fill opacity="0.9"/></v:oval>';
                                            this._bubblegroup.innerHTML = this._bubblegroup.innerHTML + bubbleHtmlString;
                                            var circle = document.getElementById("bubble_" + id);
                                        }

                                        var bubbleValue = proxyControl._reflection(item, shapelayer.bubbleSettings.valuePath);
                                        var bubblecolorValue = proxyControl._reflection(item, shapelayer.bubbleSettings.colorValuePath);
                                        if (shapelayer.bubbleSettings.colorMappings != null) {
                                            shapelayer._fillColors(bubblecolorValue, shapelayer.bubbleSettings.colorMappings, circle);
                                        }
                                        else {
                                            if (_isSVG) {
                                                circle.setAttribute("fill", shapelayer.bubbleSettings.color);
                                            }
                                            else {
                                                circle.strokecolor = shapelayer.bubbleSettings.color;
                                                circle.fillcolor = shapelayer.bubbleSettings.color;
                                            }
                                        }
                                        var radius = proxyControl._getRatioOfBubble(shapelayer.bubbleSettings.minValue, shapelayer.bubbleSettings.maxValue, bubbleValue, minValue, maxValue);
                                        if (_isSVG) {
                                            $(circle).attr({
                                                "cx": bounds.x, "cy": bounds.y,
                                                "fill-opacity": shapelayer.bubbleSettings.bubbleOpacity, "r": radius, "class": "mapBubble",
                                            });
                                            if (proxyControl.enableAnimation()&& !this._isTileMap) {
                                                $(circle).css("display", "none");
                                            }
                                        }
                                        else {
                                            $(circle).css({
                                                "height": +2 * radius + "px",
                                                "width": +2 * radius + "px"
                                            });
                                        }
                                        if (shapelayer.bubbleSettings.showTooltip == true) {
                                            var tooltipTemplateDiv = $("<div class='bubbleToolTip'  style='position:absolute;display:none;z-index:9999'/>");
                                            var template_ID = null;
                                            if (shapelayer.bubbleSettings.tooltipTemplate != null) {
                                                template_ID = shapelayer.bubbleSettings.tooltipTemplate;
                                            } else {
                                                $(tooltipTemplateDiv).append('<div class="bubbleToolTip"  style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">' + bubbleValue + '</label></p></div></div>');
                                            }
                                            $(circle).mouseenter({ htmlobj: tooltipTemplateDiv, templateID: template_ID, itemsrc: item }, proxyControl._bubbleEnterFunction);
                                            $(circle).mousemove({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleOverFunction);
                                            $(circle).mouseleave({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleleaveFunction);
                                        } else {
                                            $(circle).css("pointer-events", "none");
                                        }
                                        if (_isSVG) {
                                            proxySVG.appendChild(circle);
                                            layer._bubbles.push(circle);
                                        }
                                        else {
                                            layer._bubbles.push(circle);
                                        }
                                    }
                                }
                                obj["data"] = item;
                        }
                    }
                   var labelElement;
                   if (layer.labelSettings != null && layer.labelSettings.showLabels && layer.labelSettings.labelPath != null && shapelayer.geometryType != ej.datavisualization.Map.GeometryType.Normal) {
                        var labelValue = proxyControl._reflection(ValueObject, layer.labelSettings.labelPath);
                        if (labelValue != null) {
                            if (bounds == null) {
                               //bounds = (shape);
                            }
                            labelElement = $("<div class='labelStyle'></div>");                            
                            $(labelElement).css({ "pointer-events": "none", "position": "absolute" });
                            labelElement[0].innerHTML = labelValue;
                            this._templateDiv.append(labelElement);
                            if (layer.legendSettings.showLegend && layer.legendSettings.dockOnMap && layer.legendSettings.dockPosition == "top") {
                                $(this._templateDiv).css({ "margin-top": layer.legendSettings.height });
                            }
                            else if (layer.legendSettings.showLegend && layer.legendSettings.dockOnMap && layer.legendSettings.dockPosition == "left")
                            {
                                $(this._templateDiv).css({ "margin-left": layer.legendSettings.width });
                            }
                            $(labelElement).css({ "left": bounds.x, "top": bounds.y});
                            proxyControl._off($(labelElement), ej.eventType.mouseUp, proxyControl._polyClickFunction);
                            proxyControl._on($(labelElement), ej.eventType.mouseUp, { Param1: null, Param2: layer, Param3: shape }, proxyControl._polyClickFunction);
							proxyControl._off($(labelElement), ej.eventType.mouseLeave, proxyControl._polyLeaveFunction);
                            proxyControl._on($(labelElement), ej.eventType.mouseLeave, { Param1: layer, Param2: obj,map:this }, proxyControl._polyLeaveFunction);
							labelElement[0].setAttribute("nodeValue", shape.getAttribute("fill"));
                            layer._labelCollection.push(labelElement);
                            layer._labelBounds.push(bounds);
                            layer._labelData.push(labelValue);

                        }
                    }
					obj["shapeIndex"]=dataIndex;
                    obj["shape"] = shape;
                    obj["shapeData"] = layer._polygonData[dataIndex];
                    proxyControl._off($(labelElement), ej.eventType.mouseMove, proxyControl._polyMoveFunction);
                    proxyControl._on($(labelElement), ej.eventType.mouseMove, { Param1: shapelayer, Param2: obj }, proxyControl._polyMoveFunction);
                    if (shape != null) {
                        var shapefill = null; 
                        if (_isSVG) {
                            shapefill = shape.getAttribute("fill");
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSettings.fill;
                            shape.setAttribute("nodeValue", shapefill);
                        }
                        else {
                            shapefill = shape.fillcolor.value;
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSettings.fill;
                            shape.style.behavior = shapefill;
                        }
                    }
                    if (proxyControl.model.zoomSettings.enableZoomOnSelection) {
                        $(shape).hover(function() {
                            $(this).css('cursor', 'pointer');
                        }, function() {
                            $(this).css('cursor', 'auto');
                        });
					}
                    // bindtouches
					layer._mapShapes.push(obj);
					this._off($(shape), "mouseenter touchmove", proxyControl._polyEnterFunction);
					this._on($(shape), "mouseenter touchmove", { Param1: shapelayer, Param2: obj, map: proxyControl }, proxyControl._polyEnterFunction);
					this._off($(shape), touchStartEvent, proxyControl._polyMouseDown);
					this._on($(shape), touchStartEvent, { Param1: obj, Param2: shapelayer }, proxyControl._polyMouseDown);
					this._off($(shape), touchEndEvent, proxyControl._polyClickFunction);
					this._on($(shape), touchEndEvent, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyClickFunction);
					this._off($(shape), touchMoveEvent, proxyControl._polyMoveFunction);
					this._on($(shape), touchMoveEvent, { Param1: shapelayer, Param2: obj, Param3: shape }, proxyControl._polyMoveFunction);
					this._off($(shape), touchCancelEvent, proxyControl._polyLeaveFunction);
					this._on($(shape), touchCancelEvent, { Param1: shapelayer, Param2: obj, map: this }, proxyControl._polyLeaveFunction);
					this._off($(shape), touchEndEvent, proxyControl._polyUpFunction);
					this._on($(shape), touchEndEvent, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyUpFunction);
                }
            }

            if (shapelayer._isBaseLayer && shapelayer.geometryType == ej.datavisualization.Map.GeometryType.Normal && shapelayer.labelSettings.showLabels) {

                shapelayer._generateLabels(this);
                shapelayer._labelSizeCalculation(this);
                this._applyTransform(this._scale, this._translatePoint);
            }           

        },


        isTouch: function (evt) {
            var event = evt.originalEvent ? evt.originalEvent : evt;
            if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        },

        validateBubblePosition: function (polygon, midpoint, bubblerad) {
            var leftPoints = [], rightPoints = [];
			 if (polygon == null)
            {
                polygon = [];
                polygon.push(midpoint);
            }
            if (!this.isPointInPolygon(polygon, midpoint, true)) {
                var previouspoint = 0;
                for (var j = 0; j < polygon.length; j++) {
                    var point = { x: (polygon[j].x + this._translatePoint.x) * this._scale, y: (polygon[j].y + this._translatePoint.y) * this._scale };
                    if ((point.y < midpoint.y && previouspoint.y >= midpoint.y) || (previouspoint.y < midpoint.y && point.y >= midpoint.y)) {
                        if (point.x < midpoint.x)
                            leftPoints.push(point.x);
                        else
                            rightPoints.push(point.x);
                    }
                    previouspoint = point;
                }

                if (!leftPoints.length > 0 || !rightPoints.length > 0) {
                    if (leftPoints.length > 0) {
                        var j = midpoint.x + bubblerad;
                        while (!this.isPointInPolygon(polygon, { x: j - 2, y: midpoint.y }, true)) {
                            j = j - 2;
                        }
                        midpoint.x = parseFloat(j - 2 - bubblerad);
                    }
                    if (rightPoints.length > 0) {
                        var j = midpoint.x - bubblerad;
                        while (!this.isPointInPolygon(polygon, { x: j + 2, y: midpoint.y }, true)) {
                            j = j + 2;
                        }
                        midpoint.x = parseFloat(j + 2 + bubblerad);
                    }
                }
            }
            return midpoint;
        },

        isPointInPolygon: function (polygon, point, needTranslate) {
            var insidePolygon = false;
            var j = 0;
            for (var i = 0; i < polygon.length; i++) {
                var currentPoint = { x: polygon[i].x, y: polygon[i].y };
                var previousPoint = { x: polygon[j].x, y: polygon[j].y };

                if (needTranslate) {
                    currentPoint = { x: (polygon[i].x + this._translatePoint.x) * this._scale, y: (polygon[i].y + this._translatePoint.y) * this._scale };
                    previousPoint = { x: (polygon[j].x + this._translatePoint.x) * this._scale, y: (polygon[j].y + this._translatePoint.y) * this._scale };
                }

                var pointYDiff = point.y - currentPoint.y;
                var previousPointYDiff = previousPoint.y - currentPoint.y;
                var previousPointXDiff = previousPoint.x - currentPoint.x;

                if ((currentPoint.y < point.y && previousPoint.y >= point.y) || (previousPoint.y < point.y && currentPoint.y >= point.y)) {
                    if ((currentPoint.x + (pointYDiff / previousPointYDiff) * previousPointXDiff) < point.x) {
                        insidePolygon = true;
                    }
                }
                j = i;
            }
            return insidePolygon;
        },

        dragDown: function (event) {
                                  
            var map = event.data.map;
            var layer = map.model.layers[0];

            var mapArea = this._mapContainer;

            if (event.type == "mousedown")
            {
                map._startPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                map._stopPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
            }
            else if (event.type == "touchstart") {
                map._startPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                map._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
            }
            else if (event.type == "MSPointerDown") {
                map._startPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
                map._stopPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
            }
            map.mouseClickable = true;

            if (map.model.draggingOnSelection) {

                if (layer._prevSelectedShapes.length > 0) {

                    for (var i = 0; i < layer._prevSelectedShapes.length; i++) {                                               
                        layer._prevSelectedShapes[i].setAttribute("class", "mapShape");
                        if (_isSVG) {                            
                            layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                            layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                            layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                        }
                        else {
                            layer.fillcolor = layer._prevSelectedShapes[i].getAttribute("nodeValue");
                            layer.strokecolor = layer.shapeSettings.stroke
                            layer.strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                        }
                    }
                    layer._prevSelectedShapes = [];
                }
            }
        },

        dragMove: function (event) {
            
            var map = event.data.map;
            var layer = map.model.layers[0];
            var width; var height;

            var mapArea = this._mapContainer;

            if (map.model.draggingOnSelection) {

                if (map.mouseClickable) {                    
                    map.ispanning = false;


                    if (event.type == "mousemove")
                        map._stopPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                    else if (event.type == "touchmove")
                        map._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                    else if (event.type == "MSPointerMove")
                        map._stopPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
                    
                    $('.e-mapDragSelection').remove();
                    var div = $('<div class = "e-mapDragSelection"></div>');
                    
                    width = Math.abs(map._stopPoint.x - map._startPoint.x),
                    height = Math.abs(map._stopPoint.y - map._startPoint.y);

                    $(div).css({ "top": Math.min(map._startPoint.y, map._stopPoint.y), "left": Math.min(map._startPoint.x, map._stopPoint.x), width: width, height: height,  position: "absolute", "z-index": 100 });
                    $(div).appendTo("#maps");
                }
            }
        },

        dragUp: function (event) {        
            var map = event.data.map;
            var width = Math.abs(map._stopPoint.x - map._startPoint.x);
            var height = Math.abs(map._stopPoint.y - map._startPoint.y);
            var layer = map.model.layers[0];
            var mapArea = this._mapContainer;
        
            if (map.model.draggingOnSelection) {
        
                $('.e-mapDragSelection').remove();
                $(".e-mapDragSelection").css({
                    "display": "none"
                });
                map.mouseClickable = false;

                for (var i = 0; i < map.model.layers[0]._mapShapes.length; i++) {
                    var shape = map.model.layers[0]._mapShapes[i].shape;
                    var rect = shape.getBoundingClientRect();
                    if (!(rect.left - mapArea[0].offsetLeft + rect.width < Math.min(map._startPoint.x, map._stopPoint.x) || Math.min(map._startPoint.x, map._stopPoint.x) + width < rect.left - mapArea[0].offsetLeft ||
                           rect.top - mapArea[0].offsetTop + rect.height < Math.min(map._startPoint.y, map._stopPoint.y) || Math.min(map._startPoint.y, map._stopPoint.y) + height < rect.top - mapArea[0].offsetTop)) {

                        if (!map.model.layers[0]._contains(layer._prevSelectedShapes, shape))

                            layer._prevSelectedShapes.push(shape);
                    }
                }


                for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                    var shape = layer._prevSelectedShapes[i];
                    var rect = shape.getBoundingClientRect();
                    shape.setAttribute("class", "e-mapSelectedShape mapShape");
                    if (_isSVG) {
                        shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                        shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                        shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));
                    }
                }                
            }           
        },

        isIntersectBubbles: function () {
            var listofBounds = [];
            for (var k = 0; k < this.model.layers.length; k++) {
                var layer = this.model.layers[k];
                Array.prototype.push.apply(listofBounds, layer._bubbleCollection);
                for (var i = 0; i < layer.subLayers.length; i++) {
                    Array.prototype.push.apply(listofBounds, layer.subLayers[i]._bubbleCollection);
                }
            }

            for (var i = 0; i < listofBounds.length; i++) {
                var x = listofBounds[i].getAttribute('cx');
                var y = listofBounds[i].getAttribute('cy');
                var r = listofBounds[i].getAttribute('r');
                for (var j = i + 1; j < listofBounds.length; j++) {

                    var x1 = listofBounds[j].getAttribute('cx');
                    var y1 = listofBounds[j].getAttribute('cy');
                    var r1 = listofBounds[j].getAttribute('r');
                    var distBubble = Math.sqrt(((x - x1) * (x - x1)) + ((y - y1) * (y - y1)));
                    var rad = parseFloat(r) + parseFloat(r1);
                   
                    if (distBubble <= rad) {
                        return true;
                    }
                }
            }
            return false;
        },
       
        _zooming: function (delta, e) {
            var position = this._getCurrentPoint(e);
            if (delta >= 120) {
                this._zoomingIn(position.x, position.y, e);
            }
            else {
                this._zoomingOut(position.x, position.y, e);
            }
        },

        _zoomingIn: function (x, y, event, isAnimate) {
            var map = this;
            if (map.model.zoomSettings.enableZoom && map._zoomLevel() >= map._minZoom() && map._zoomLevel() <= map._maxZoom()) {
                var scal = map._scale;
                var position = { x: x, y: y };
                map._prevScale = scal;
                if (!this._isTileMap) {
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map._zoomFactor())) / (map._width / x);
                    map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map._zoomFactor())) / (map._height / y);
                    map._scale = scal + map._zoomFactor();
                    map.model.zoomSettings.level = map._zoomLevel() + map._zoomFactor();
                    map._zoomLevel(map._zoomLevel() + map._zoomFactor());
                }
                else {
                    this._tileZoom(map._zoomLevel() - map._zoomFactor(), map._zoomLevel(), position);
                    var prevLevel = map._zoomLevel();
                    map.model.zoomSettings.level = map._zoomLevel() + map._zoomFactor();
                    map._zoomLevel(map._zoomLevel() + map._zoomFactor());
                    this._generateTiles(this._zoomLevel());
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));
                }
                
                map._trigger("zoomedIn", { originalEvent: event, zoomLevel: map._zoomLevel() });
                map._applyTransform(map._scale, map._translatePoint);
                map._refrshLayers();
                map._resizeShape();               
            
                map._updateSliderValue();
            }
        },

        _zoomingOut: function (x, y, event, isAnimate) {
            var map = this;
            if (map.model.zoomSettings.enableZoom) {
                if (map._zoomLevel() > this._minZoom()) {

                    var scal = map._scale;
                    var position = { x: x, y: y };
                    map._prevScale = scal;
                    if (!map._isTileMap) {                       
                       if (map._scale > 1) {		
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale - map._zoomFactor())) / (map._width / x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale - map._zoomFactor())) / (map._height / y);
                        map._scale = map._scale - map._zoomFactor();
                        map.model.zoomSettings.level = map._zoomLevel() - map._zoomFactor();
                        map._zoomLevel(map._zoomLevel() - map._zoomFactor());
                        map._scale = map._zoomLevel();
                        }
				    }
                    else {
                        map._tileZoom(map._zoomLevel() + map._zoomFactor(), map._zoomLevel(), position);
                        var prevLevel = map._zoomLevel();
                        map.model.zoomSettings.level = map._zoomLevel() - map._zoomFactor();
                        map._zoomLevel(map._zoomLevel() - map._zoomFactor());
                       
                        map._generateTiles(map._zoomLevel());
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, map._zoomLevel() - map._zoomFactor()))) / (Math.pow(2, map._zoomLevel() - map._zoomFactor()));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, map._zoomLevel() - map._zoomFactor()))) / (Math.pow(2, map._zoomLevel() - map._zoomFactor()));
                        map._scale = (Math.pow(2, map._zoomLevel() - map._zoomFactor()));
                    }                    
                    if (map._zoomLevel() < map._minZoom()) {
                        map.model.zoomSettings.level = map._minZoom();
                        map._zoomLevel(map._minZoom());
                        map.zoom(map._zoomLevel());
                    }
                   
                    map._applyTransform(map._scale, map._translatePoint);
                    map._refrshLayers();
                    map._resizeShape();                                  

                    map._updateSliderValue();
                }
                else {                   
                    map.zoom(map._minZoom(), false);
                }
                map._trigger("zoomedOut", { originalEvent: event, zoomLevel: map._zoomLevel() });
            }
        },

        _getRatioOfBubble: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var bubbleRadius = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
			if(maxValue == minValue)
				bubbleRadius = (((parseFloat(max) - parseFloat(min)) / 100)) + parseFloat(min);
			
            return bubbleRadius;
        },

        _reflection: function (Source, path) {
            var ShapeValues = Source;
            if (path != null && Source != null) {
                var parts = path.split(".");
                var i = 0;
                parts.push(path);
                for (; i < parts.length; i++) {                    
                    if (Source != undefined && ShapeValues != null) {
                        var hasProperty = ShapeValues.hasOwnProperty(parts[i]);
                        if (hasProperty) {
                            $.each(ShapeValues, function (prop, propval) {
                                if (prop == parts[i]) {
                                    ShapeValues = propval;
                                    return false;
                                }

                            });
                        }
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        },
		
		clearShapeSelection:function(){		
		 var index = this.baseMapIndex();
            this.model.layers[index]._clearShapeWidth(this._scale);
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._clearShapeWidth(this._scale);
            }
		},
        shapeSelectionOnResize: function () {
        var index = this.baseMapIndex();
          this.model.layers[index]._shapeSelection();
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._shapeSelection();
            }
        },

       refreshMarkers: function () {
          var markerElements = document.getElementsByClassName("mapMarker");
        
         for(var index = markerElements.length-1; index >= 0; index--)
         {
          markerElements[index].parentNode.removeChild(markerElements[index]);
         }
         
            var index = this.baseMapIndex();
            this.model.layers[index]._generateMarkerForLayers(this);
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._generateMarkerForLayers(this);
            }
            this._refrshLayers();
        },

        _generateTooltipForLayers: function (layer) {
             if (layer.showTooltip) {
			   if (document.documentMode == 8){
					var tooltip = document.querySelectorAll("shapeToolTip");
				}
				else{
					var tooltip = document.getElementsByClassName("shapeToolTip");
				}
                if (tooltip != null && tooltip.length == 0) {
                    var tooltip_templateDiv = $("<div class='shapeToolTip' style='display:none;position:absolute;z-index:1000;pointer-events:none;'></div>");
                    $(document.body).append(tooltip_templateDiv);
                    layer._tooltipElement = tooltip_templateDiv;
                }
                else
                {
                    layer._tooltipElement = tooltip[0];
                }
                if (layer.tooltipTemplate == null) {
                    var path = layer.shapeSettings.valuePath;
                    if (path != null) {
                        layer.tooltipTemplate = 'defaultTooltip';                      
                        this._mapContainer.append($('<div  id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:#data["' + path + '"]}}</label></p></div></div></div>'));
                    }
                }
            }                
        },
       
        _orderByNameAscending: function (a, b) {
            if (a.Range == b.Range) {
                return 0;
            } else if (a.Range > b.Range) {
                return 1;
            }
            return -1;
        },

        _resizingContainer: function () {            
            this._scale = this._isTileMap ? Math.pow(2, this._zoomLevel() - 1) : 1;
            if (this._isTileMap) {
                this._translatePoint.x = (this._tileTranslatePoint.x - (0.5 * Math.pow(2, this._zoomLevel() - this._zoomFactor()))) / (Math.pow(2, this._zoomLevel() - this._zoomFactor()));
                this._translatePoint.y = (this._tileTranslatePoint.y - (0.5 * Math.pow(2, this._zoomLevel() - this._zoomFactor()))) / (Math.pow(2, this._zoomLevel() - this._zoomFactor()));
            }
            else {                     
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }           
            this._applyTransform(this._scale, this._translatePoint);
            if (this._zoomLevel() > 1) {
                this._scale = this._isTileMap ? Math.pow(2, this._zoomLevel() - this._zoomFactor()) : this._zoomLevel();
                this._applyTransform(this._scale, this._translatePoint);
            }
            this._baseScale = 1;
            this._baseTranslatePoint = { x: (this._translatePoint.x * 2), y: (this._translatePoint.y * 2) };
           
        },

        _calculateMarginConditions: function () {
            var trans = this._calculateMinMax();
            if (this._translatePoint.y> trans.maxY) {
                this._translatePoint.y = trans.maxY;
            }
            else if (this._translatePoint.y < trans.minY) {
                this._translatePoint.y = trans.minY;
            }
            if (this._translatePoint.x > trans.maxX) {
                this._translatePoint.x = trans.maxX;
            }
            else if (this._translatePoint.x < trans.minX) {
                this._translatePoint.x = trans.minX;
            }
        },

        _calculateMinMax: function () {           
            var bounds= this._groupBorder;            
            var maximumTransX, maximumTransY, minimumTransX, minimumTransY;
            if (this._containerHeight * this._scale <= this._height) {
                maximumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
                minimumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
            } else {
                maximumTransY = -bounds.y + 1;
                minimumTransY = (this._height - this._containerHeight * this._scale) / (this._scale) - bounds.y - 1;
            }
            if (this._containerWidth * this._scale <= this._width) {
                maximumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale)- bounds.x;
                minimumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale) - bounds.x;
            } else {
                maximumTransX = -bounds.x + 1;
                minimumTransX = (this._width - this._containerWidth * this._scale) / (this._scale) - bounds.x - 1;
            }
            return { minX: minimumTransX, maxX: maximumTransX, minY: minimumTransY, maxY: maximumTransY };
        },
      
        _findMidPointofPoylgon: function (points) {
          if(points.length>0){
            var min = 0;           
            var max = points.length;
            var initialPoint = points[0];
            var minX=initialPoint.x,maxX = initialPoint.x;
            var minY = initialPoint.y, maxY = initialPoint.y;
            var startX,startY,startX1,startY1,sum=0,xSum=0,ySum=0;
           
            for (var i = min; i <= max - 1; i++) {
                var prevPoint = points[i];
                startX = prevPoint.x;
                startY = prevPoint.y;
                if (i == max - 1) {
                    startX1 = initialPoint.x;
                    startY1 = initialPoint.y;
                }
                else {
                    var nextPoint = points[i + 1];
                    startX1 = nextPoint.x;
                    startY1 = nextPoint.y;
                }
                sum = sum + Math.abs(((startX * startY1)) - (startX1 * startY));
                xSum = xSum + Math.abs(((startX + startX1) * (((startX * startY1) - (startX1 * startY)))));
                ySum = ySum + Math.abs(((startY + startY1) * (((startX * startY1) - (startX1 * startY)))));

            }
            sum = 0.5 * sum;
            xSum = (1 / (4 * sum)) * xSum;
            ySum = (1 / (4 * sum)) * ySum;

            /* Code for finding nearest points in polygon related to midPoint*/
            var _rightMinPoint = { x: 0, y: 0 };
            var _rightMaxPoint = { x: 0, y: 0 };
            var _leftMinPoint = { x: 0, y: 0 };
            var _leftMaxPoint = { x: 0, y: 0 };

            for (var i = min; i <= max - 1; i++) {
                var point = points[i];
                if (point.x > xSum) {
                    if (point.y < ySum && ySum - point.y < ySum - _rightMinPoint.y)
                        _rightMinPoint = { x: point.x, y: point.y };
                    else if (point.y > ySum && (_rightMaxPoint.y == 0 || point.y - ySum < _rightMaxPoint.y - ySum))
                        _rightMaxPoint = { x: point.x, y: point.y };
                }
                else {
                    if (point.y < ySum && ySum - point.y < ySum - _leftMinPoint.y)
                        _leftMinPoint = { x: point.x, y: point.y };
                    else if (point.y > ySum && (_leftMaxPoint.y == 0 || point.y - ySum < _leftMaxPoint.y - ySum))
                        _leftMaxPoint = { x: point.x, y: point.y };
                }
            }

            return { x: xSum, y: ySum, rightMin: _rightMinPoint, rightMax: _rightMaxPoint, leftMin: _leftMinPoint, leftMax: _leftMaxPoint, points:points };             
}        
        },
        _convertPointToLatLong: function (pixelX, pixelY, bounds, factor){
        
            var Size;
            if (bounds == null) { bounds=[[-180,-85],[180,85]] }
            if (this._isTileMap) {
                Size = Math.pow(2, 1) * 256;
            }
            else {
                if(factor == null)
                    Size = Math.min(this._height, this._width);
                else
                    Size = Math.min(this._height, this._width)*factor;
            }

            var x=(Math.min(pixelX,0,Size-1)/Size)-0.5;
            var y=0.5-(Math.min(pixelY,0,Size-1)/Size);
            latitude = 90 - 360 * Math.atan(Math.exp(-y * 2 * Math.PI)) / Math.PI;
            longitude = 360 * x;
            return {x:latitude,y:longitude};
                        

        },
        _convertTileLatLongtoPointForShapes: function (lat, lng, bounds,factor) {
            var Size;
            if (bounds == null) { bounds=[[-180,-85],[180,85]] }
            if (this._isTileMap) {
                Size = Math.pow(2, 1) * 256;
            }
            else {
                if(factor == null)
                    Size = Math.min(this._height, this._width);
                else
                    Size = Math.min(this._height, this._width)*factor;
            }
           
            var x = (lng + 180) / 360;
            var sinLatitude = Math.sin(lat * Math.PI / 180);
            if (sinLatitude == -1)
                sinLatitude = -0.5;
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            if (this._isMapCoordinates) {
                var pixelX = (Math.min(Math.max(x * Size + 0.5, 0), Size - 1));
                var pixelY = (Math.min(Math.max(y * Size + 0.5, 0), Size - 1));
            }
            else if (factor != null) {

                var pixelX = Math.abs((lng - this._mapBounds[0][0]) * factor);
                var pixelY = Math.abs((this._mapBounds[1][1] - lat ) * factor);
            }
            else {

                var pixelX = lng;
                var pixelY = lat;
            }

            
           
            return { x: pixelX, y: pixelY };
           
            
        },
     
        _convertTileLatLongtoPoint: function (lat, lng) {
            var Size = Math.pow(2, this._zoomLevel()) * 256;
            var x = (lng + 180) / 360; 
            var sinLatitude = Math.sin(lat * Math.PI / 180);
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
			if (this._isMapCoordinates) {
            var pixelX = (x * Size + 0.5) + this._tileTranslatePoint.x;
            var pixelY = (y * Size + 0.5) + this._tileTranslatePoint.y; 
              }
            else if (factor != null){

                var pixelX = Math.abs((lng - this._mapBounds[0][0])*factor);
                var pixelY = Math.abs((this._mapBounds[1][1] - lat)*factor);
            }
            else {

                var pixelX = lng;
                var pixelY = lat;
            }			
            return { x: pixelX, y: pixelY };
        },
      
        _convertLatLongtoPointforMarker: function (lat, lng) {
            var factor = this._getFactor();
            var point = this._convertTileLatLongtoPointForShapes(lat, lng, this._mapBounds,factor);
            return { x: (point.x + this._translatePoint.x) * this._scale, y: (point.y + this._translatePoint.y) * this._scale };

        },
     
        _animate: function (animateduration) {
            this._calculateMarginConditions();
            $(this._rootgroup).stop(true, false);
            if (this._sliderControl != null) {
                $(this._sliderControl).stop(true, false);
            }
            var map = this;
            var currentTranformation = { x: this._translatePoint.x, y: this._translatePoint .y};
            var startPoint = [this._prevPoint.x, this._prevPoint.y];
            var endPoint = [this._translatePoint.x, this._translatePoint.y];

            function slope(horizontal, vertical) {
                
                if (horizontal[0] == vertical[0]) {
                    return null;
                }
                return (vertical[1] - horizontal[1]) / (vertical[0] - horizontal[0]);
            }

            function intercept(point, slopeValue) {
                if (slopeValue === null) {
                    return point[0];
                }
                return point[1] - slopeValue * point[0];
            }

            var slopeFactor = slope(startPoint, endPoint);
            var slopeIntersection = intercept(startPoint, slopeFactor);
            var horizontalDifference= endPoint[0] - startPoint[0];
            var verticalDifference=endPoint[1] - startPoint[1];
            var scaleDifference= this._scale-this._prevScale;
            var currentScale = this._scale;
            this._updateSliderValue();
            $(this._rootgroup).animate(
                {
                    count: 10
                },
                {
                    duration: animateduration,
                    step: function(now, fx) {
                        var scaleX = map._prevScale + ((now * (scaleDifference / fx.end)));
                        var transX = startPoint[0] + (now * (horizontalDifference / fx.end)) / (scaleX / currentScale);
                        var transY;
                        if (slopeFactor == null) {
                            transY = startPoint[1] + (now * (verticalDifference / fx.end));
                        } else {
                            transY = ((slopeFactor * transX) + slopeIntersection);
                        }
                        if (_isSVG) {
                            map._rootgroup.setAttribute('transform', 'scale(' + scaleX + ') translate(' + transX + ', ' + transY + ')');
                        }
                        else {
                            map._rootgroup.coordorigin = -transX + ',' + -transY;
                            map._rootgroup.coordsize = (map._width / scaleX) + ',' + (map._height / scaleX);
                        }
                        map._translatePoint.x = transX;
                        map._translatePoint.y = transY;
                        map._scale = scaleX;
                        map._refrshLayers();
                        map._resizeShape();
                    },
                    complete: function() {
                        map._translatePoint.x = currentTranformation.x;
                        map._translatePoint.y = currentTranformation.y;
                        map._scale = currentScale;
                        this.count = 0;
                    }
                }
            );

        },

        _applyTransform: function (scale, translatePoint) {
            this._scale = scale;
            this._translatePoint = translatePoint;
           if(!this._isTileMap)
            this._calculateMarginConditions();  
             if((this._translatePoint.x !=Number.POSITIVE_INFINITY) &&(this._translatePoint.y !=Number.POSITIVE_INFINITY))
            {
            if (_isSVG) {
                this._rootgroup.setAttribute('transform', 'scale(' + this._scale + ') translate(' + this._translatePoint.x + ', ' + this._translatePoint.y + ')');
            }
            else {
                this._rootgroup.coordorigin = (-this._translatePoint.x) + ','  + (-this._translatePoint.y);
                this._rootgroup.coordsize = (this._width / scale)  + ',' + (this._height / scale);
            }
            }
        },
       
        _mouseMove: function (event) {
            if (event.type == "touchmove" && event.originalEvent.touches.length > 1) {
                this._isPinching = true;
            }
            else {
                var map = this;
                if (map.ispanning && map.enablePan()) {
                    event.preventDefault();
                    var position;
                    if (event.type == "mousemove" || event.type == "pointermove")
                        position = { x: event.pageX, y: event.pageY };
                    else if (event.type == "touchmove")
                        position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                    else if (event.type == "MSPointerMove")
                        position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - (this._dragStartX - position.x);
                        var curtileY = this._tileTranslatePoint.y - (this._dragStartY - position.y);
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;                        
                        this._generateTiles(this._zoomLevel());
                    }                    
                    var curX = map._translatePoint.x - (map._dragStartX - position.x) / map._scale;
                    var curY = map._translatePoint.y - (map._dragStartY - position.y) / map._scale;
                    if (curX != map._translatePoint.x || curY != map._translatePoint.y) {
                        map._isDragging = true;
                    }
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    map._applyTransform(map._scale, map._translatePoint);
                    map._dragStartX = position.x;
                    map._dragStartY = position.y;
                    map._refrshLayers();
                }
            }
        },
       
        _svgMouseLeave: function (event) {
            var map = this;
            map.ispanning = false;
        },
        
        _mouseWheel: function (event) {
            if (this.model.zoomSettings.enableZoom) {
                if (this.enableAnimation()) {
                    $(this._rootgroup).stop(true, false);
                }
                var e = event.originalEvent;
                var original = event;
                if (!_isSVG) {
                    e = event;
                }
                if (event.target.className != "LegendDiv" && event.target.className != "e-defaultLegendLabel") {
                    e.preventDefault ? e.preventDefault() : original.preventDefault();
                    if (_isSVG) {
                        this._zooming(event.originalEvent.wheelDelta, event.originalEvent);
                    }
                    else {
                        this._zooming(event.originalEvent.wheelDelta, e);
                    }
                }
            }
        },

        _mouseButtonUp: function (event) {

            var map = this;
            if (map.ispanning) {
                map._trigger("panned", { originalEvent: event });
            }
            map.ispanning = false;
        },
        
        _mouseUp: function (event) {
            this.ispanning = false;
            this._isDragging = false;
        },

        _mouseButtonDown: function (event) {
            this._isPinching = false;
           if (event.target.className != "e-vhandle") {         
            this._isNavigationPressed = false;
            if(this.enableAnimation()){
                $(this._rootgroup).stop(true, false);
                if (this._sliderControl != null) {
                    $(this._sliderControl).stop(true, false);
                }
			}
            if (event.type == "touchstart" && event.originalEvent.touches.length > 1)
            { }
            else {
                var position;
                if (event.type == "mousedown" || event.type == "pointerdown")
                    position = { x: event.pageX, y: event.pageY };
                else if (event.type == "touchstart")
                    position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                else if (event.type == "MSPointerDown")
                    position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                var map = this;
                if (map._scale > 1 && map.model.enablePan) {
                    event.preventDefault();
                    map.ispanning = true;
                }
                map._dragStartX = position.x;
                map._dragStartY = position.y;
            }
		}
        },

        _getCurrentPoint: function (event) {
            var map = this._mapContainer;
             var xPos = event.pageX - map.offset().left;
            var yPos = event.pageY - map.offset().top; 
            return { x: xPos, y: yPos };
        },

        _legendDoubleClick: function (event) {
           
            var node= event.target;
            while (node.parentNode != null && node.parentNode.className != "ScrollLegendDiv e-scroller e-js e-widget") {
                node = node.parentNode;
            }
            if (node.parentNode != null && node.parentNode.className == "ScrollLegendDiv e-scroller e-js e-widget") {               
                this._isNavigationPressed = true;               
            }           
        },
        
        _doubleClick: function (event) {            
            var map = this;
            var prevLevel1 = map._zoomLevel();
             map._legendDoubleClick(event);
			if (!this._isNavigationPressed && event.target.className.toString().indexOf("e-icon1") == -1) {
                var position = this._getCurrentPoint(event);               
                if (map.model.zoomSettings.enableZoom && map._zoomLevel() + map._zoomFactor() >= map._minZoom() && map._zoomLevel() + map._zoomFactor() <= map._maxZoom()) {
                    var scal = map._scale;
                    map._prevScale = scal;
                    if (!this._isTileMap) {
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map._zoomFactor())) / (map._width / position.x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map._zoomFactor())) / (map._height / position.y);
                        map._scale = scal + map._zoomFactor();
                        map.model.zoomSettings.level = map._zoomLevel() + map._zoomFactor();
                        map._zoomLevel(map._zoomLevel() + map._zoomFactor());
                    }
                    else {
                        this._tileZoom(map._zoomLevel() - map._zoomFactor(), map._zoomLevel(), position);
                        var prevLevel = map._zoomLevel();
                        map.model.zoomSettings.level = map._zoomLevel() + map._zoomFactor();
                        map._zoomLevel(map._zoomLevel() + map._zoomFactor());
                        this._generateTiles(this._zoomLevel());
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._scale = (Math.pow(2, prevLevel));                        
                    }
                    if (prevLevel1 < map._zoomLevel())
                    {
					     map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map._zoomLevel() });
                    }
                    if (map.enableAnimation() && !this._isTileMap) {
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                        map._refrshLayers();
                        map._resizeShape();
                    }                 
                    map._updateSliderValue();
                }
            }
        },
       
        _tileZoom: function (prevlevel,level, position) {
            
            var map = this;
            if (level > 0 && level < 20) {
                this._tileDiv.html("");
                var prevSize = Math.pow(2, prevlevel) * 256;
                if (position == undefined) {
                    position = { x: this._width / 2, y: this._height / 2 };
                }
                var totalSize = Math.pow(2, level) * 256;
                var percentX = ((position.x - this._tileTranslatePoint.x) / prevSize) * 100;
                var percentY = ((position.y - this._tileTranslatePoint.y) / prevSize) * 100;
                var pointX = (percentX * totalSize) / 100;
                var pointY = (percentY * totalSize) / 100;
                this._tileTranslatePoint.x = position.x - pointX;
                this._tileTranslatePoint.y = position.y - pointY;
                
            }
        },          
        
        _bubbleEnterFunction: function (event) {
            if (event.data.templateID != null) {
                var tooltiphtmlString = $("#" + event.data.templateID).render(event.data.itemsrc);
                $(event.data.htmlobj).html(tooltiphtmlString);
            }
            $(document.body).append(event.data.htmlobj);
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },
        
        _bubbleleaveFunction: function (event) {
            $(event.data.htmlobj).remove();
            return $(event.data.htmlobj).css("display", "none");
        },
        
        _bubbleOverFunction: function (event) {
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },
       
        _polyEnterFunction: function (event) {
            var matched = jQuery.uaMatch(navigator.userAgent),
                layer = event.data.Param1, shape,
                isTouch = this.isTouch ? this.isTouch(event) : false;
            if (isTouch && matched.browser == 'chrome') {
                var myLocation = event.originalEvent.changedTouches[0];
                shape = document.elementFromPoint(myLocation.clientX, myLocation.clientY);
            }
            else
                shape = event.data.Param2["shape"];
            var map = event.data.map;
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            if (layer.enableMouseHover) {
                layer._clearShapeWidth(map._scale);
                if (layer._prevHoverdLegend != null && !layer._contains(layer._prevSelectedLegends, layer._prevHoverdLegend)) {
                    layer._prevHoverdLegend.css("background-color", _isSVG ? layer._prevHoveredShape.getAttribute("fill") : layer._prevHoveredShape.fillcolor.value);
                }
                if (_isSVG) {
                    shape.setAttribute("stroke-width", (layer.shapeSettings.highlightBorderWidth/map._scale));
                    shape.setAttribute('stroke', layer.shapeSettings.highlightStroke);
                } else {
                    shape.strokeweight = layer.shapeSettings.highlightBorderWidth;
                    shape.strokecolor = layer.shapeSettings.highlightStroke/map._scale;
                }

                 if (layer.shapeSettings.highlightColor != null) {
                    if (layer.shapeSettings.highlightColor != "transparent" && !layer._contains(layer._prevSelectedShapes, shape)) {
                        shape.setAttribute("class", "e-mapHighlightedShape mapShape");
                        if (_isSVG) {
                            if (layer.shapeSettings.highlightColor != 'none' && shape.highlightcolor==null) {
                                shape.setAttribute('fill', layer.shapeSettings.highlightColor);
                            }
                            else {
                                shape.setAttribute('fill', shape.highlightcolor);
                            }
							shape.setAttribute("stroke-width", (layer.shapeSettings.highlightBorderWidth/map._scale));
							shape.setAttribute('stroke', layer.shapeSettings.highlightStroke);
                        } else {
                            shape.fillcolor = layer.shapeSettings.highlightColor;
                        }
                    }
                    if (legend.length > 0 && !layer._contains(layer._prevSelectedLegends,legend[0].legend)) {
                        legend[0].legend[0].setAttribute("class", "e-mapHighlightedShape mapShape");
                        legend[0].legend.css("background-color", layer.shapeSettings.highlightColor);
                        layer._prevHoverdLegend = legend[0].legend;
                    }

                    layer._prevHoveredShape = shape;
                }

                if (layer.legendSettings!= null && layer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && layer.shapeSettings.colorMappings!=null && layer.shapeSettings.colorMappings.rangeColorMapping != null
                     && !layer.shapeSettings.autoFill && (layer.legendSettings.showLegend == undefined || layer.legendSettings.showLegend)) {
                    for (var i = 0; i < layer._mapShapes.length; i++) {
                        var mapshape = layer._mapShapes[i].shape;
                        if (mapshape == shape) {
                            var mappings = null;
                            if (layer.shapeSettings.colorMappings.rangeColorMapping != undefined) {
                                mappings = layer.shapeSettings.colorMappings.rangeColorMapping;
                            }
                            var _legendwidth = layer.legendSettings.width;
                            if (layer.legendSettings.width == undefined)
                                _legendwidth = 150;

                            var rectwidth = (_legendwidth / mappings.length) / 10;
                            if (layer._mapShapes[i].legendrect != null) {
                                var rectPosition = layer._mapShapes[i].legendrect.marginLeft;
                                $(layer._interactiveArrow).css({ "margin-left": rectPosition + Math.ceil(rectwidth) - layer._interactiveArrow.width() / 2, "display": "block" });
                                break;
                            }
                        }
                    }
                }
                map._trigger("mouseover", { originalEvent: event.data.Param2 });
            }
                                    
        },

        _updateShapeRect:function(layer) {

            for (var index = 0; index < layer._mapShapes.length; index++) {
                var obj = layer._mapShapes[index];
                if (obj.left == null) {
                    obj["left"] = layer._mapShapes[index].shape.getBoundingClientRect().left;
                    obj["right"] = layer._mapShapes[index].shape.getBoundingClientRect().right;
                    obj["top"] = layer._mapShapes[index].shape.getBoundingClientRect().top;
                    obj["bottom"] = layer._mapShapes[index].shape.getBoundingClientRect().bottom;
                    layer._mapShapes[index] = obj;
                }
            }
        },
        
        _polyUpFunction: function (event) {
            var layer = event.data.Param2,
                shape = event.data.Param3,
                isTouch = this.isTouch(event);
            if (isTouch)
                event.stopImmediatePropagation();
            if (this.model.zoomSettings.enableZoomOnSelection && !this._isDragging && !this._isTileMap && !this._isPinching) {
                    if (_isSVG) {
                        var bounds = shape.getBBox();
                        this._zoomOnSelection(bounds);
                    } else {                        
                            this._updateShapeRect(layer);

                        for (var index = 0; index < layer._mapShapes.length; index++) {
                            if (shape == layer._mapShapes[index].shape) {
                                var left = layer._mapShapes[index].left;
                                var top = layer._mapShapes[index].top;
                                var right = layer._mapShapes[index].right;
                                var bottom = layer._mapShapes[index].bottom;
                                var bound = { x: left, y: top, width: right - left, height: bottom - top };
                                this._zoomOnSelection(bound);
                                break;
                            }
                        }
                    }
                    this.ispanning = false;
            }
            var element = layer._tooltipElement;
            if (element != null) {
                $(element).delay(200).queue( function(next){ 
						$(this).css('display','none'); 
						next();
						
				});
            }
            if (isTouch && layer.showTooltip) {
                var currentXY = this.getEventXY(event),
                 map = this, cachedXY = map.model.cachedXY,
                 swipediffer = 10, differX, differY;
                differX = Math.abs(cachedXY.x - currentXY.X);
                differY = Math.abs(cachedXY.y - currentXY.Y);
                if (differX < swipediffer && differY < swipediffer) {
                    this.touchEnd = true;
                    var dataParam1 = event.data.Param1;
                    event.data.Param1 = layer;
                    event.data.Param2 = dataParam1;
                    map._polyMoveFunction(event);
                    event.data.Param1 = dataParam1;
                    event.data.Param2 = layer;
                    map.tooltipElement = layer._tooltipElement[0];
                    $(map.tooltipElement).stop(true, true);
                    map._timeOut = map._timeOut.bind(map);
                    map.touchTimeOut = window.setTimeout(map._timeOut, 1000);
                }
            }
        },

        _timeOut: function () {
            $(this.tooltipElement).fadeOut();
        },

        _markerPressed: function (event) {
            this._trigger("markerSelected", { originalEvent: event.data });
        },
			
        _polyMouseDown:function(event)
        {
           
            var layer = event.data.Param2;
            var position;
            if (event.type == "mousedown" || event.type == "pointerdown")
                position = { x: event.pageX || event.originalEvent.pageX, y: event.pageY || event.originalEvent.pageY };
            else if (event.type == "touchstart")
                position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
            else if (event.type == "MSPointerDown")
                position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };

            this.model.cachedXY = position;
            if (!this.isTouch(event)) {

            if (event.data.Param1.hasOwnProperty("data") && layer.showTooltip) {
                var element = layer._tooltipElement;
                var template = layer.tooltipTemplate;
                var parentSize = $(this._mapContainer).offset();
                if (parentSize == null)
                    parentSize = { left: 0, width: 0, height: 0 };
                var tooltipSize = layer._tooltipSize;
                var tooltipdisplayheight = this._height;
                var tooltipdisplaywidth = this._width;
				
                if (element != null && template != null) {
                    $(element).css({ "display": "block" });
                    var htmlString = $("#" + template).render(event.data.Param1["data"]);
                    $(element).html(htmlString);
                     var height= element[0]!=null ? element[0].clientHeight : element.clientHeight;
					 var width= element[0]!=null ? element[0].clientWidth : element.clientWidth;
					 tooltipSize = { height: height, width: width };

					

					var xPos = position.x - $(this._mapContainer).offset().left;
			        var yPos = position.y - $(this._mapContainer).offset().top;

                    var xnewPos = position.x;
                    var ynewPos = position.y;

			       if (tooltipSize.width + xPos >= tooltipdisplaywidth) {
                         if(xnewPos - tooltipSize.width > parentSize.left)
			                xnewPos = ((parentSize.left + tooltipdisplaywidth) - (tooltipdisplaywidth - xPos)) - tooltipSize.width;
                         else
                            xnewPos -= (tooltipSize.width + xPos - tooltipdisplaywidth);  
			            }
			            if (tooltipSize.height > tooltipdisplayheight) {
			                if (tooltipdisplayheight + tooltipSize.height > this._height + yPos) 
			                    ynewPos -= parentSize.top + 10;
			            }
                        else if (tooltipSize.height + yPos >= tooltipdisplayheight) {
			                ynewPos -= (tooltipSize.height + yPos - tooltipdisplayheight);
			            }   

                    $(element).css({ "left": xnewPos + 10, "top": ynewPos + 10 });
                }
            }
            }
        },
      
        getEventXY: function (event) {
            var X, Y;
            X = event.pageX || event.originalEvent.pageX || event.originalEvent.changedTouches[0].pageX;
            Y = event.pageY || event.originalEvent.pageY || event.originalEvent.changedTouches[0].pageY;
            return { X: X, Y: Y };
        },

       _polyClickFunction: function (event) {

           var ctrlkey = event.ctrlKey,
               layer = event.data.Param2,
               map = this, swipeDiffer = 10, cachedXY = map.model.cachedXY,// avoid tiny mouse movement changes
            differX, differY, currentXY = map.getEventXY(event);
            differX = Math.abs(cachedXY.x - currentXY.X);
            differY = Math.abs(cachedXY.y - currentXY.Y);
            if (differX < swipeDiffer && differY < swipeDiffer) {

            var shape = event.data.Param3;
            if (event.data.param4 != undefined)
            {                
                var temp = event.data.param4;
            }            
            
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            map._isPolygonSelected = true;

            if (layer.enableSelection) {

                if (ctrlkey && layer.selectionMode == ej.datavisualization.Map.SelectionMode.Multiple) {
				 if (legend != null && legend.length > 0) {
                        if ($.inArray(legend[0].legend, layer._prevSelectedLegends) == -1) {
				            legend[0].legend[0].setAttribute("class", "e-mapSelectedShape mapShape");
                            legend[0].legend.css("background-color", layer.shapeSettings.selectionColor);
                            layer._prevSelectedLegends.push(legend[0].legend);
                        }
                        else {
                            var index = layer._prevSelectedLegends.indexOf(legend[0].legend);
				            legend[0].legend[0].setAttribute("class", "mapShape");
                            legend[0].legend.css("background-color", legend[0].legend[0].getAttribute("nodeValue"));
                            layer._prevSelectedLegends.splice(index, 1);
                        }
                    }

				 if (temp != null) {                     

				     var element = document.getElementsByClassName("e-map-labelContainer")
				     if ($.inArray(temp, layer._prevSelectedTemp) == -1) {				         
				         $(temp).css("background-color", layer.shapeSettings.selectionColor);
				         $(temp).css("width", element[0].clientWidth);
				         layer._prevSelectedTemp.push(temp);
				     }
				     else {
				         $(temp).css("background-color", "");				         				         			         
				         var pos = layer._prevSelectedTemp.indexOf(temp);
				         layer._prevSelectedTemp.splice(pos, 1);
				     }

				 }

                    if ($.inArray(shape, layer._prevSelectedShapes) == -1) {
				        shape.setAttribute("class", "e-mapSelectedShape mapShape");
                        if (_isSVG) {
                            shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                            shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                            shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));                            
                        }
                        else {
                            shape.fillcolor = layer.shapeSettings.selectionColor;
                            shape.strokecolor = layer.shapeSettings.selectionStroke
                            shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / map._scale;
                        }
                        layer._prevSelectedShapes.push(shape);
                        layer.selectedItems.push(event.data.Param1);
                    }
                    else {
				        shape.setAttribute("class", "mapShape");
                        shape.setAttribute("fill", shape.getAttribute("nodeValue"));
                        shape.setAttribute("stroke", layer.shapeSettings.stroke);
                        shape.setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                        var index = layer._prevSelectedShapes.indexOf(shape);
                        layer._prevSelectedShapes.splice(index, 1);
                        layer.selectedItems.splice(index, 1);
                    }
                }

                else {
                     for (var i = 0; i < layer._prevSelectedLegends.length; i++) {
                        layer._prevSelectedLegends[i][0].setAttribute("class", "mapShape");
                        layer._prevSelectedLegends[i].css("background-color", layer._prevSelectedLegends[i][0].getAttribute("nodeValue"));
                    }
                    if ((legend != null && legend.length > 0)) {
                         legend[0].legend[0].setAttribute("class", "e-mapSelectedShape mapShape");
                         legend[0].legend.css("background-color", layer.shapeSettings.selectionColor);
                         layer._prevSelectedLegends.push(legend[0].legend);
                     }
                   
                    for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                        layer._prevSelectedShapes[i].setAttribute("class", "mapShape");                        
                            if (_isSVG) {
                                layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                                layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                                layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                            }
                            else {
                                layer.fillcolor = layer._prevSelectedShapes[i].getAttribute("nodeValue");
                                layer.strokecolor = layer.shapeSettings.stroke
                                layer.strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                            }                            
                    }

                    if (temp != null)
                    {
                        var element = document.getElementsByClassName("e-map-labelContainer")
                        if ($.inArray(temp, layer._prevSelectedTemp) == -1)
                        {                           

                            for (var i = 0; i < layer._prevSelectedTemp.length; i++)
                            {
                                $(layer._prevSelectedTemp[i]).css("background-color", "");                                
                            }
                            layer._prevSelectedTemp = [];
                            $(temp).css("background-color", layer.shapeSettings.selectionColor);
                            $(temp).css("width", element[0].clientWidth);
                            layer._prevSelectedTemp.push(temp);
                        }
                        else
                        {                            
                            for (var i = 0; i < layer._prevSelectedTemp.length; i++) {
                                $(layer._prevSelectedTemp[i]).css("background-color", "");                                
                            }
                            layer._prevSelectedTemp = [];
                        }
                        
                    }

                        if (!this._isPinching && ($.inArray(shape, layer._prevSelectedShapes) == -1)) {
                        shape.setAttribute("class", "e-mapSelectedShape mapShape");                      
                            layer._prevSelectedShapes = [];
                            layer.selectedItems = [];
                            if (_isSVG) {
                                shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                                shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                                shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));
                            }
                            else {
                                shape.fillcolor = layer.shapeSettings.selectionColor;
                                shape.strokecolor = layer.shapeSettings.selectionStroke;
                                shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / map._scale;
                            }
                            
                            layer._prevSelectedShapes.push(shape);
                            layer.selectedItems.push(event.data.Param1);
                        }
                        else {
                            layer._prevSelectedShapes=[];
                            layer.selectedItems = [];
                        }
                        
                    

                    
                }
            }
            map._trigger("shapeSelected", { originalEvent: layer.selectedItems });
          }
        },

        _updateSelection: function (layer, shape, data) {
            if (layer.enableSelection) {
                if (layer.selectionMode != ej.datavisualization.Map.SelectionMode.Multiple) {
                    layer._prevSelectedShapes.pop();
                    layer.selectedItems.pop();
                }
                layer._prevSelectedShapes.push(shape);
                layer.selectedItems.push(data);
            }
        },

	   _zoomOnSelection: function (bounds) {            
	       var layerScale;
	       if (!_isSVG) {
	           bounds.x = (bounds.x - (this._baseTranslatePoint.x/2)) - (bounds.width/2);
	           bounds.y = (bounds.y - (this._baseTranslatePoint.y/2)) - (bounds.height / 2);
	       }
	       var boundwidth = bounds.width;
	       var boundHeight =bounds.height;
	       if ((this._width - 100) / (this._height - 100) > boundwidth / boundHeight) {
	           layerScale = (this._height - 100) / boundHeight;
	       } else {
	           layerScale = (this._width - 100) / boundwidth;
	       }
	       this._prevScale = this._scale;
	       this._scale = layerScale;
	       this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
	       this.model.zoomSettings.level = this._scale - this._baseScale + 1;
	       this._zoomLevel(this._scale - this._baseScale + 1);
	       var level = this._zoomLevel();
	       if (!(level > this._minZoom() && level < this._maxZoom())) {
	           this.model.zoomSettings.level = level > (((this._maxZoom() - this._minZoom()) / 2) + this._minZoom()) ? this._maxZoom() : this._minZoom();
	           this._zoomLevel(this.model.zoomSettings.level);
	           this._scale = this._zoomLevel() + this._baseScale - 1;
	       }
	       var leftMargin = (this._width / 2) - ((boundwidth * this._scale) / 2);
	       var leftPos = leftMargin / this._scale;
	       var topMargin = (this._height / 2) - ((boundHeight * this._scale) / 2);
	       var topPos = topMargin / this._scale;
	       this._translatePoint.x = -bounds.x + leftPos;
	       this._translatePoint.y = -bounds.y + topPos;
	       if (this.enableAnimation() && !this._isTileMap) {
	           this._animate(1200);
	       }
	       else {
	           this._applyTransform(this._scale, this._translatePoint);
	           this._updateSliderValue();
	       }	                  
        },
       
          _polyMoveFunction: function (event) {
              var map = event.data.Param1,
                  isTouch = this.isTouch?this.isTouch(event):false;
           if ((!isTouch && event.type.toString().indexOf('move') > -1) || this.touchEnd) {
               this.touchEnd = false;
		   if(this.model!=null){
            var baseLayer = this.model.layers[this.baseMapIndex()];
	        var element = map._tooltipElement;
            var template = map.tooltipTemplate;
            var tooltipObject = event.data.Param2;
            var tooltipSize = map._tooltipSize;
			var parentSize=$(this._mapContainer).offset();
			 if (parentSize == null)
                parentSize = { left: 0, top:0, width: 0, height: 0 };

			if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == "right") {
			    width = this._width + baseLayer.legendSettings.width + 20 + (baseLayer.legendSettings.leftLabel.length * 10) + (baseLayer.legendSettings.rightLabel.length * 10);
			}
			else {
                width = this._width;
			}
			var tooltipdisplayheight = this._height;
			var tooltipdisplaywidth = width;

			var pageXY = this.getEventXY(event);

			var xPos = pageXY.X - parentSize.left;
			var yPos = pageXY.Y - parentSize.top;

			var xnewPos = pageXY.X;
			var ynewPos = pageXY.Y;

             if (element != null && template != null && tooltipObject.hasOwnProperty("data") && event.data.Param1.showTooltip) {
                    $(element).css({ "display": "block" });
					 var locale = this.model.locale;
					 var obj = $.extend({}, tooltipObject.data);
					 if (locale && this.model.enableGroupSeparator){
						for (var key in obj)
							obj[key] = isNaN(parseFloat(obj[key])) ? obj[key] : parseFloat(obj[key]).toLocaleString(locale);
					 }	
                    var htmlString = $("#" + template).render(obj);
                    $(element).html(htmlString);
                     var height= element[0]!=null ? element[0].clientHeight : element.clientHeight;
					 var width= element[0]!=null ? element[0].clientWidth : element.clientWidth;
					 tooltipSize = { height: height, width: width };

					 if (!ej.util.isNullOrUndefined(this.touchTimeOut))
					     window.clearTimeout(this.touchTimeOut);

            if (tooltipSize.width + xPos >= tooltipdisplaywidth) {
             if(xnewPos - tooltipSize.width > parentSize.left)
			    xnewPos = ((parentSize.left + tooltipdisplaywidth) - (tooltipdisplaywidth - xPos)) - tooltipSize.width;
             else
                xnewPos -= (tooltipSize.width + xPos - tooltipdisplaywidth);  
			}
			if (tooltipSize.height > tooltipdisplayheight) {
			    if (tooltipdisplayheight + tooltipSize.height > this._height + yPos) 
			        ynewPos -= parentSize.top + 10;
			}
            else if (tooltipSize.height + yPos >= tooltipdisplayheight) {
			    ynewPos -= (tooltipSize.height + yPos - tooltipdisplayheight);
			}  
                $(element).css({ "left": xnewPos + 10, "top": ynewPos + 10});
			}
            }
          }
          },

        _polyLeaveFunction: function (event) {
            var layer = event.data.Param1,
                isTouch = this.isTouch(event),
                map = event.data.map,
                shape = event.data.Param2.shape,
                element = layer._tooltipElement,
                selectedshape = layer._prevSelectedShapes;
            if (selectedshape.length > 0)
            {
                shape.setAttribute("class", "mapShape");
                for (i = 0; i < selectedshape.length; i++)
                {
                    selectedshape[i].setAttribute("class", "e-mapSelectedShape mapShape");                    
                }                
            }
            else
            {
                shape.setAttribute("class", "mapShape");
            }            
            if (element != null && !isTouch) {
                $(element).css("display", "none");
            }
            if (layer.legendSettings!=null && layer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && layer._interactiveArrow != null)
                $(layer._interactiveArrow).css("display", "none");
            layer._clearShapeWidth(map._scale);
            map._trigger("mouseleave", { originalEvent: event.data.Param2 });
        },

        _wireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(this._mapContainer), ej.eventType.mouseDown, this._mouseButtonDown);
			$(document).keydown({ className: "home", map: this }, this._keyboardKeysPressed);
            this._on($(this._mapContainer), ej.eventType.mouseMove, this._mouseMove);
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (isIE11)
                browser = "msie";
            if (window.navigator.msPointerEnabled) {
                this._on($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                if (this.model.zoomSettings.enableZoom)
                    $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else {
                this._on($(this._mapContainer), "mouseup", this._mouseButtonUp);
            }
            var map = this;
            this._browser = browser;
            if (browser != "mozilla") {
                this._on($(this._mapContainer), "mousewheel", this._mouseWheel);
            }
            else {
                var elem = this._svgDocument || this._mapContainer[0];
                if (elem.addEventListener) {
                    // Mouse Scrolling event for firefox
                    elem.addEventListener("DOMMouseScroll", MouseWheel, false);
                }

            }
            function MouseWheel(event) {
			if(map.enableAnimation()){
				$(map._rootgroup).stop(true, false);
			}
			  event.preventDefault(event);
                map._zooming(-40 * event.detail, event);
            }
            this._on($(this._mapContainer), "dblclick", this._doubleClick);
            if (this.model.enableResize || this.model.isResponsive) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this._mapResize);
                else
                    this._on($(window), "orientationchange", this._mapResize);
            }
        },

       clip:function(n,minValue,maxValue){
        return Math.min(Math.max(n, minValue), maxValue);
        },

        pointToLatLong:function(pointX,pointY)
        {
            var map = this;
            var factor = map._getFactor();
            var translateX = (pointX / map._scale) - (map._translatePoint.x);
            var translateY = (pointY / map._scale) - (map._translatePoint.y);
            var mapSize = Math.min(map._height, map._width) * factor;
            var transformX = (map.clip(translateX, 0, mapSize - 1) / mapSize) - 0.5;
            var transformy = 0.5 - (map.clip(translateY, 0, mapSize - 1) / mapSize);

            latitude = 90 - 360 * Math.atan(Math.exp(-transformy * 2 * Math.PI)) / Math.PI;
            longitude = 360 * transformX;
            return {latitude:latitude,longitude:longitude};
        
        },
   
        _mapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var oldSize = { width: this._width, height: this._height };
            var oldTranslatePoint = this._translatePoint;
            var map = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                if (map.model != null) {
                    var latlon = map.pointToLatLong(map._width / 2, map._height / 2);
                    map.refresh(true);
                    if (!map._isTileMap)
                        map.navigateTo(latlon.latitude, latlon.longitude, map._zoomLevel(), false);                    
                }
            }, 500);
        },


        _isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        
        _unWireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._off($(document), 'keydown', this._keyboardKeysPressed);
            this._off($(document), ej.eventType.mouseUp, this._mouseUp);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (isIE11)
                browser = "msie";
			if (browser != "mozilla") {
                this._off($(this._mapContainer), "mousewheel", this._mouseWheel);
            }
            if (window.navigator.msPointerEnabled) {
                this._off($(this._mapContainer), "MSPointerDown", this._mouseButtonDown);
                this._off($(this._mapContainer), "MSPointerMove", this._mouseMove);
                this._off($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
            }
            else if (browser == "webkit" || browser == "chrome" || browser == "mozilla") {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            else {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            this._off($(this._mapContainer), "dblclick", this._doubleClick);

        },

        navigateTo: function (latitude, longitude, level,isAnimate) {
		     if (isAnimate == undefined) {
                isAnimate = this.enableAnimation();
            }
		    level=parseFloat(level);
		    if (level == undefined) level = this._zoomLevel();
            if (level > this._minZoom() && level < this._maxZoom()) {
                this.model.zoomSettings.level = level;
                this._zoomLevel(level);
            }
            else {
                this.model.zoomSettings.level = level > (((this._maxZoom() - this._minZoom()) / 2) + this._minZoom()) ? this._maxZoom() : this._minZoom();
                this._zoomLevel(this.model.zoomSettings.level);

            }
            var factor = this._getFactor();
            var translatePoint = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds,factor);
            this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
            this._prevScale = this._scale;
            this._scale = this._baseScale + ((level - 1) * this._zoomFactor());
            var leftPosition = ((this._containerWidth + this._baseTranslatePoint.x) / 2) / this._scale;
            var topPosition = ((this._containerHeight + this._baseTranslatePoint.y) / 2) / this._scale;
            this._translatePoint.x = -translatePoint.x+ leftPosition;
            this._translatePoint.y = -translatePoint.y+ topPosition;
            if (isAnimate && !this._isTileMap ){
                this._animate(1200);
            }
            else {
                this._applyTransform(this._scale, this._translatePoint);
            }
            this._updateSliderValue();
            this._refrshLayers();
        },

        selectShape: function (obj, layer,isZoom) {
            if (obj != null) {
                if (layer == null) {
                    layer = this.model.layers[this.baseMapIndex()];
                }
                for (var i = 0; i < layer._mapShapes.length; i++) {
                    var data = layer._mapShapes[i].data;
                    var shape = layer._mapShapes[i].shape;
                    if (data != null && obj == this._reflection(data, layer.shapeSettings.valuePath)) {

                        if (layer._prevSelectedShapes.length != 0 && layer.selectionMode != ej.datavisualization.Map.SelectionMode.Multiple) {
                            for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                                if (_isSVG) {
                                    layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                                    layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                                    layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                                } else {
                                    layer._prevSelectedShapes[i].fillcolor = layer._prevSelectedShapes[i].style.behavior;
                                    layer._prevSelectedShapes[i].strokecolor = layer.shapeSettings.stroke;
                                    layer._prevSelectedShapes[i].strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                                }
                            }
                        }
                        if (layer.enableSelection) {
                            if (_isSVG) {
							    if(layer.shapeSettings.selectionColor!="none"){
                                    shape.setAttribute("fill", layer.shapeSettings.selectionColor);
							    }
                                shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                                shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / this._scale));
                            } else {
							     if(layer.shapeSettings.selectionColor!="none"){
                                      shape.fillcolor = layer.shapeSettings.selectionColor;
								}
                                shape.strokecolor = layer.shapeSettings.selectionStroke;
                                shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / this._scale;
                            }
                            this._updateSelection(layer, shape, obj);
						if (!layer._contains(layer.selectedItems, layer._mapShapes[i]))
                            layer.selectedItems.push(layer._mapShapes[i]);
                            this._trigger("shapeSelected", { originalEvent: layer.selectedItems });
                        }
                        if (isZoom && this.model.zoomSettings.enableZoomOnSelection) {
                            this._zoomOnSelection(shape.getBBox());
                        }
                        i = layer._mapShapes.length;
                    }
                }
            }

        },

        _getIntersectedElements: function (evt, shapes) {
            evt.width += 5;
            evt.height += 5;
            if (_isSVG && this._browser!='mozilla' && this._browser!='webkit') {
                var rpos = this._svgDocument.createSVGRect();
                rpos.x = evt.left;
                rpos.y = evt.top;
                rpos.width = evt.width;
                rpos.height = evt.height;
                return this._svgDocument.getIntersectionList(rpos, null);
            }
            else {
                var elements = []; 
				var parentSize=$(this._mapContainer).offset();
                var  parentLeft=parentSize.left;
                var parentTop= parentSize.top;			
                for (var i = 0; i < shapes.length; i++) {
                    var shape = shapes[i].shape;
                    var bounds = shape.getBoundingClientRect();					
					var leftPos= $(shape).offset().left- parentLeft;
					var topPos= $(shape).offset().top- parentTop;					
                    if (this._isIntersect(evt, { left: leftPos, top: topPos, height: (bounds.bottom  - bounds.top ), width: (bounds.right -bounds.left) })) {
                        elements.push(shape);
                        return elements;
                    }
                }
                return elements;
            }
                
                
        },

        _isIntersect: function (rect1, rect2) {           
            
            if (rect1.left >= (rect2.left + rect2.width) || rect1.left + rect1.width <= rect2.left ||
                rect1.top >= rect2.top + rect2.height || rect1.top + rect1.height <= rect2.top) {
                    return false;
                }
                return true;
            
        },

        pan: function (direction) {
            var map = this;
            var Xdiff = 0;
            var Ydiff = 0;
            if (this._zoomLevel() != 1) {
                switch (direction) {

                    case 'right':
                        {
                            Xdiff = this._width / 7;
                            Ydiff = 0;
                            break;
                        }
                    case 'top':
                        {
                            Xdiff = 0;
                            Ydiff = -(this._height / 7);
                            break;
                        }n
                    case 'left':
                        {
                            Xdiff = -(this._width / 7);
                            Ydiff = 0;
                            break;
                        }
                    case 'bottom':
                        {
                            Xdiff = 0;
                            Ydiff = this._height / 7;
                            break;
                        }
                }
                if (map.enablePan()) {
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - Xdiff / map._scale;
                        var curtileY = this._tileTranslatePoint.y - Ydiff / map._scale;
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;
                        this._generateTiles(this._zoomLevel());
                    }                    
                    var curX = map._translatePoint.x - Xdiff / map._scale;
                    map._prevScale = map._scale;
                    var curY = map._translatePoint.y - Ydiff / map._scale;
                    map._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    if (map.enableAnimation() && !this._isTileMap) {                       
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                    }
                    map._refrshLayers();
                }
            }
			
        },      

        zoom: function (level,isAnimate) {
           var map = this;
           var prevLevel = map._zoomLevel();
		    if (level <= this._maxZoom() && level >= this._minZoom()) {
                if (this._isTileMap) {
                    this._tileZoom(map._zoomLevel(), level, { x: this._width / 2, y: this._height / 2 });
                    var prevLevel = map._zoomLevel();
                    map.model.zoomSettings.level = level;
                    map._zoomLevel(level);
                    this._generateTiles(this._zoomLevel());
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));					
                }
                else {                    
                    var fac = (map._baseScale + ((level - 1) * map._zoomFactor()));
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._prevScale = map._scale;
                    map._translatePoint.x -= (map._width / map._scale - map._width / fac) / 2;
                    map._translatePoint.y -= (map._height / map._scale - map._height / fac) / 2;
                    map._scale = fac;
                    map.model.zoomSettings.level = level;
                    map._zoomLevel(level);

                    if (map.enableAnimation() && isAnimate || isAnimate == undefined) {
                        map._animate(600);
                    }                  
                }
				if(!map.enableAnimation() || !isAnimate)
				{
				     map._applyTransform(map._scale, map._translatePoint);
                     map._refrshLayers();
                     map._resizeShape();       
				}
				 map._updateSliderValue();                  
                
            }
		    else if (level <= this._minZoom()) {
		        this.model.zoomSettings.level = this._minZoom();
		        this._zoomLevel(this._minZoom());
		        this.zoom(this._zoomLevel());
            }
            else if (level >= this._maxZoom()) {
                this.model.zoomSettings.level = this._maxZoom();
                this._zoomLevel(this._maxZoom());
                this.zoom(this._zoomLevel());
            }
			  if (prevLevel < level) {
			      map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map._zoomLevel() });
            }
            else  if (prevLevel > level) {
                map._trigger("zoomedOut", { originalEvent: null, zoomLevel: map._zoomLevel() });
            }
			
        },

       refresh: function (isResize) {
            
           $(this._svgDocument).children().remove();
           $(this._svgDocument).empty();
           $(this._mapContainer).empty();
           
           (this.baseMapIndex() >= this.model.layers.length) && this.baseMapIndex(0);
           this.model.layers[this.baseMapIndex()]._mapItems && (this.model.layers[this.baseMapIndex()]._mapItems = null);
           if (this._svgDocument != null) {
               this._svgDocument = null;
           }
            this._scale = 1;
            this._margintop = 0;
            this._marginleft = 0;
            this._translatePoint = { x: 0, y: 0 };            
            this._height = this._mapContainer.height();
            this._width = this._mapContainer.width();
             if (this._height == 0) {
                 this._height = this._mapContainer[0].parentElement.clientHeight != 0 ? this._mapContainer[0].parentElement.clientHeight : $(document).height();
            }
            if (this._width == 0) {
                this._width = this._mapContainer[0].parentElement.clientWidth != 0 ? this._mapContainer[0].parentElement.clientWidth : $(document).width();
            }
			 var baseLayer = this.model.layers[this.baseMapIndex()];
			 if (baseLayer.legendSettings != null && baseLayer.legendSettings.showLegend && baseLayer.shapeSettings.colorMappings != null) {
                baseLayer._sizeCalculation(this);
            }
            this._generatePath();           
            if (this._groupSize != null) {
               
                if (this._isMapCoordinates) {
                    this._containerHeight = this._groupSize.maxY - this._groupSize.minY;
                    this._containerWidth = this._groupSize.maxX - this._groupSize.minX;
                    this._groupBorder = { x: this._groupSize.minX, y: this._groupSize.minY };
                }
                else
                {
                    var factor = this._getFactor();
                    this._containerHeight = Math.abs((this._groupSize.maxY - this._groupSize.minY)* factor);
                    this._containerWidth = Math.abs((this._groupSize.maxX - this._groupSize.minX)*factor);
                }
            }
			
			if ((baseLayer.layerType == "geometry")) {
                $(this._svgDocument).css("background", this.model.background);
            }
            this._resizingContainer();
            this._renderMapElements();
            
            if ((baseLayer.legendSettings != null && baseLayer.legendSettings.showLegend) &&(baseLayer.shapeSettings.colorMappings!=null)) {
                
                    this._renderLegend();
                
            } 
           if (_isSVG) {
               $(this._svgDocument).css({ height: this._height, width: this._width, "margin-top": this._margintop, "margin-left": this._marginleft });
               
            }			
            if (this.model.enableLayerChangeAnimation) {
                $(this._mapContainer).animate({ opacity: 1 }, 500);
            }
            if(this.model.centerPosition!=null && !this._isTileMap){
                this.navigateTo(this.model.centerPosition[0], this.model.centerPosition[1], this._zoomLevel(), false);
            }
             if (this._legendContainer != null) {
                this._legendContainer.css({

                    "height": this._legendSize.height


                })
            }
			 this._trigger("refreshed");
             if (!isResize) {
                 this._events = null;
                 $(this._mapContainer).removeData();
                 $(this._mapContainer).data("ejMap", this);
             }
        },

        _updateSliderValue: function (isAnimate) {
            var slider = this._sliderControl;
            if (isAnimate == undefined) isAnimate = this.enableAnimation();
            if (slider != null) {
                var obj = slider.data("ejSlider");               
                obj.option("value", this._zoomLevel());
            }
        },

        _createDivElement: function (parent, child,base,classname) {
            var parentElement = parent;
            var childElement = child;           
            childElement.appendTo(parentElement);           
            parentElement.appendTo(base);            
            this._on(childElement,ej.eventType.mouseDown,{ className: classname, map: this }, this._navigationControlPressed);
            
        },

        _navigationControlPressed: function (event) {
            
            event.stopPropagation();
            this._isNavigationPressed = true;
            var map = event.data.map;
            var isAnimate = map.enableAnimation();            
            if (event.data.className == "zoomIn") {               
                map.zoom(map._zoomLevel() + 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
				 map._refrshLayers();
            }
            else if (event.data.className == "zoomOut") {               
                map.zoom(map._zoomLevel() - 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }
            else if (event.data.className == "panLeft") {
                map.pan("left");                
            }
            else if (event.data.className == "panRight") {
                map.pan("right");
            }
            else if (event.data.className == "panTop") {
                map.pan("top");
            }
            else if (event.data.className == "panBottom") {
                map.pan("bottom");
            }
            else if (event.data.className == "home") {                
                map.zoom(1,true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }            
           
        },
	
		 _keyboardKeysPressed: function (event) {
            this._iskeyboardKeysPressed = true;
            var map = event.data.map;
            var isAnimate = map.enableAnimation();
            if (event.ctrlKey && event.keyCode==38) {    			
                map._zoomingIn(map._width / 2, map._height / 2, event, isAnimate);
            }
            else if (event.ctrlKey && event.keyCode==40) {    
                map._zoomingOut(map._width / 2, map._height / 2, event, isAnimate);
            }
            else if (event.keyCode==37) {
                map.pan("left");                
            }
            else if (event.keyCode==39) {
                map.pan("right");
            }
            else if (event.keyCode==38) {
                map.pan("top");
            }
            else if (event.keyCode==40) {
                map.pan("bottom");
            }
		},
				
        refreshNavigationControl: function (navigation) {
            
            var baseLayer = this.model.layers[this.baseMapIndex()];
            var prevNav = this._mapContainer.find("#ejNavigation");
            if (prevNav.length > 0) {
                this._mapContainer[0].removeChild(prevNav[0]);
            }
            if (this.model.navigationControl != null && this.model.navigationControl.enableNavigation) {
                if (navigation == undefined) {
                    navigation = this.model.navigationControl;
                }
                var navigationOrientation = ej.datavisualization.Map.LabelOrientation.Vertical;
                var controlSize;
                var navigationHeight = 120;
                var navigationWidth = 12;
                controlSize = { width: 90, height: 320 };
                var navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-vert'/>");
				if (this.model.navigationControl.content == null || this.model.navigationControl.content == "") {
                    var sliderDiv = $("<div style='height:120px;width:10px;margin-top:-197px;margin-left: 34px;' />");
                }
                if (navigation.orientation == 'horizontal') {
                    navigationOrientation = ej.datavisualization.Map.LabelOrientation.Horizontal;
                    navigationHeight = 12;
                    navigationWidth = 120;
                    controlSize = { width: 320, height: 90 };
                    navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-horz' />");
                    if (this.model.navigationControl.content == null || this.model.navigationControl.content == "") {
                        sliderDiv = $("<div style='height:10px;width:120px;margin-top:-18px' />");
                    }
                }
				if (this.model.navigationControl.content == null || this.model.navigationControl.content == "") {
                    var baseDiv = $("<div class='e-panContainer'/>");
                    var isHor = navigationOrientation == 'horizontal' ? 'Horz' : 'Vert';
                    if (navigation.orientation == 'horizontal') {
                        var zoominOutDiv = $("<div style='margin-left: 94px;' />");
                        var slidercontrol = $("<div style='margin-left: 137px;'/>");
                    }
                    else {
                        var zoominOutDiv = $("<div />");
                        var slidercontrol = $("<div style='margin-top: 34px;'/>");
                    }
                    baseDiv.appendTo(navigationControl);
                    zoominOutDiv.appendTo(navigationControl);
                    sliderDiv.appendTo(slidercontrol);
                    slidercontrol.appendTo(navigationControl);
                    this._sliderControl = sliderDiv;
                    var navPos = { x: 0, y: 0 };
                    if (navigation.dockPosition == null || navigation.dockPosition == ej.datavisualization.Map.Position.None) {
                        navPos.x = (this._width * navigation.absolutePosition.x) / 100;
                        navPos.y = (this._height * navigation.absolutePosition.y) / 100;
                    } else {
                        navPos = this._getPosition(navigation.dockPosition, controlSize);
                    }
                    navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
                var map = this;
				var changeZoom = this._height * 0.0025;
				if (this.model.navigationControl.content == null || this.model.navigationControl.content == "") {
                    if (this._browser != 'chrome' && this._browser != 'msie') {
                        navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 320px;position:absolute;z-index:2;zoom:' + changeZoom + ';}</style>');
                    }
                    else {
                        navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 320px;position:absolute;z-index:2;-moz-transform: scale(' + changeZoom + ');}</style>');
                    }
                }
                else {
					var contentHeight = $("#"+this.model.navigationControl.content).height() || $(this.navigationControlData[0]).height();
					var contentWidth = $("#"+this.model.navigationControl.content).width() || $(this.navigationControlData[0]).width();
					
                    if (this._browser != 'mozilla' && this._browser != 'chrome') {
                        navigationStyle = $('<style> .e-map-navigation {width: '+contentWidth+'px;height: '+contentHeight+'px;position:absolute;z-index:2;}</style>');
                    }
                    else {
                        navigationStyle = $('<style> .e-map-navigation {width: '+contentWidth+'px;height: '+contentHeight+'px;position:absolute;z-index:2;}</style>');
                    }
                }
				if (this._browser!='mozilla' && this._browser!='chrome'){
				navigationStyle.remove();
				$('html > head').append(navigationStyle);
				}
				else{
				$('html > head').append(navigationStyle);
				}
				if (this.model.navigationControl.content == null || this.model.navigationControl.content == "") {
				    $(sliderDiv).ejSlider({
				        orientation: navigationOrientation,
				        sliderType: ej.SliderType.MinRange,
				        value: 1,
				        animationSpeed: 1200,
				        minValue: this._minZoom(),
				        showTooltip: true,
				        enableAnimation: false,
				        maxValue: this._maxZoom(),
				        incrementStep: this._zoomFactor(),
				        slide: onslide,
				        change: onchange,
				        height: navigationHeight,
				        width: navigationWidth
				    });
				    navigationControl.appendTo(this._mapContainer);
				    if (navigation.orientation == 'horizontal') {
				        this._createDivElement($("<div title='" + this._localizedLabels.zoomIn + "' class='e-icon1 e-incrementButton  icon_margin1' />"), $("<div class='e-icon1 nav-inc-" + isHor + "  e-innerIncrement'/>"), zoominOutDiv, "zoomIn");
				        this._createDivElement($("<div title='" + this._localizedLabels.zoomOut + "' class='e-icon1 e-incrementButton icon_margin2'/>"), $("<div class='e-icon1 nav-dec-" + isHor + " e-innerDecrement'/>"), zoominOutDiv, "zoomOut");
				    }
				    else {
				        this._createDivElement($("<div title='" + this._localizedLabels.zoomIn + "' class='e-icon1 e-incrementButton  icon_margin1' />"), $("<div class='e-icon1 nav-inc-" + isHor + "  e-innerIncrement'/>"), zoominOutDiv, "zoomIn");
				        this._createDivElement($("<div title='" + this._localizedLabels.zoomOut + "' class='e-icon1 e-incrementButton icon_margin2'/>"), $("<div class='e-icon1 nav-dec-" + isHor + " e-innerDecrement'/>"), zoominOutDiv, "zoomOut");
				    }
				    this._createDivElement($("<div title='" + this._localizedLabels.panTop + "' class='e-icon1 e-radialTop'/>"), $("<div class='e-icon1 e-arrowUp'/>"), baseDiv, "panTop");
                    this._createDivElement($("<div title='" + this._localizedLabels.panLeft + "' class='e-icon1 e-radialLeft'/>"), $("<div class='e-icon1 e-arrowLeft'/>"), baseDiv, "panLeft");
				    this._createDivElement($("<div title='" + this._localizedLabels.panRight + "' class='e-icon1 e-radialRight'/>"), $("<div class='e-icon1 e-arrowRight'/>"), baseDiv, "panRight");
                    this._createDivElement($("<div title='" + this._localizedLabels.panBottom + "' class='e-icon1 e-radialBottom'/>"), $("<div class='e-icon1 e-arrowDown'/>"), baseDiv, "panBottom");
                    var homeDiv = $("<div title='" + this._localizedLabels.home + "' class='e-icon1 e-home-bg'><div class='e-icon1 e-map-home'></div>");
				    homeDiv.appendTo(baseDiv);
				    homeDiv.mousedown({ className: "home", map: this }, this._navigationControlPressed);
				}
				else {
                    if (this.navigationControlData == null) {
                        this.navigationControlData = $("#" + this.model.navigationControl.content);
                        this.navigationControlData.css({ 'display': 'block' });                            
                        this.navigationControlData.appendTo(navigationControl);
                    }
                    else {                            
                        this.navigationControlData.appendTo(navigationControl);
                    }                                                
                    navigationControl.appendTo(this._mapContainer);
                                       
				    controlSize = { width: this.navigationControlData[0].getBoundingClientRect().right - this.navigationControlData[0].getBoundingClientRect().left, height: this.navigationControlData[0].getBoundingClientRect().bottom - this.navigationControlData[0].getBoundingClientRect().top };                    
                    var navPos = { x: 0, y: 0 };
                    if (navigation.dockPosition == null || navigation.dockPosition == ej.datavisualization.Map.Position.None) {
                        navPos.x = (this._width * navigation.absolutePosition.x) / 100;
                        navPos.y = (this._height * navigation.absolutePosition.y) / 100;
                    } else {
                        navPos = this._getPosition(navigation.dockPosition, controlSize);
                    }

                    if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
                        navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + baseLayer.legendSettings.height  + "px" });
                    }
                    else if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
                            navigationControl.css({ "margin-left": navPos.x + baseLayer.legendSettings.width + 20 + (baseLayer.legendSettings.leftLabel.length * 10) + (baseLayer.legendSettings.rightLabel.length * 10)+ "px", "margin-top": navPos.y + "px" });
                    }
                    else if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default) {
                        navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + baseLayer.legendSettings.height + "px" });
                    }
                    else if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default) {
                        navigationControl.css({ "margin-left": navPos.x + baseLayer.legendSettings.width + "px", "margin-top": navPos.y + "px" });
                    }
                    else {
                        navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                    }
                }
            }                     

			function onchange(args) {
			    if (map != null && map._isRendered && map._zoomLevel() != args.value) {
                    map.zoom(args.value, false);
                }
            
            }

            function onslide(args) {
                if (map != null && map._isRendered && map._zoomLevel() != args.value) {
                    map.zoom(args.value, false);
                }
            }
        },

        _getPosition: function (position,size) {
            var Position = { x: 0, y: 0 };
            switch (position) {
                case ej.datavisualization.Map.Position.TopLeft:                   
                    break;
                case ej.datavisualization.Map.Position.TopCenter:
                    Position.x = (this._width / 2) - (size.width / 2);
                    break;
                case ej.datavisualization.Map.Position.TopRight:
                    Position.x = this._width - size.width;
                    break;
                case ej.datavisualization.Map.Position.CenterLeft:
                    Position.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.Position.Center:
                    Position.x = (this._width / 2) - (size.width / 2);
                    Position.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.Position.CenterRight:
                    Position.x = this._width - size.width;
                    Position.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.Position.BottomLeft:
                    Position.y = this._height - size.height;
                    break;
                case ej.datavisualization.Map.Position.BottomCenter:
                    Position.x = (this._width / 2) - (size.width / 2);
                    Position.y = this._height - size.height;
                    break;
                case ej.datavisualization.Map.Position.BottomRight:
                    Position.x = this._width - size.width;
                    Position.y = this._height - size.height;
                    break;

            }
            return Position;
        },

        _renderMapElements: function () {           
            this._templateDiv = $("<div class='TemplateDiv'/>");
            this._templateDiv.appendTo(this._mapContainer);
            this._templateDiv.css({
                'pointer-events': 'none', 'overflow': 'hidden', "position":"absolute",
                'z-index': '1', 'height': this._height, 'width': this._width
            });			

            this.refreshLayers();       
            this.refreshNavigationControl(this.model.navigationControl);
        },
		
        _renderLegend: function () {

            var baseLayer = this.model.layers[this.baseMapIndex()];
            
            this._scrollLegendDiv = $("<div class='ScrollLegendDiv'/>");
            
            this._scrollLegendDiv.appendTo(this._mapContainer);

            this._legendContainer = $("<div id='LegendcontainerDiv'/>");
            this._legendContainer.appendTo(this._scrollLegendDiv);

            this._legendDiv = $("<div class='LegendDiv'/>");

            this._legendDiv.css({
                'height': this._legendDivHeight + "px", 'width': this._legendDivWidth + "px"
            })
            this._isNavigationPressed = true;
            this._legendDiv.appendTo(this._legendContainer);
            this._legendDiv[0].getBoundingClientRect();
            this._legendContainer.css({
                "position": "relative"
            })

            this._scrollLegendDiv.ejScroller({ height: Math.round(this._legendSize.height), width: Math.round(this._legendSize.width) });

            if (baseLayer.legendSettings.labelOrientation == undefined)
                baseLayer.legendSettings.labelOrientation == ej.datavisualization.Map.LabelOrientation.Vertical;
            if (!baseLayer.legendSettings.dockOnMap) {
                if (baseLayer.legendSettings.labelOrientation == ej.datavisualization.Map.LabelOrientation.Horizontal || baseLayer.legendSettings.labelOrientation == ej.datavisualization.Map.LabelOrientation.Vertical) {

                    var position = (baseLayer.legendSettings.position == undefined) ? "topleft" : baseLayer.legendSettings.position;
                    var legendPos = this._getPosition(position, this._legendSize);
                    this._scrollLegendDiv.css({
                         "position": "absolute",
                         'z-index': '2', "margin-left": legendPos.x + "px", "margin-top": legendPos.y
                    });
                }
            }
            else {

                if (baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
                    pos = "none";
                    height = baseLayer.legendSettings.height + 50;
                    width = baseLayer.legendSettings.width + 20 + (baseLayer.legendSettings.leftLabel.length * 10) + (baseLayer.legendSettings.rightLabel.length * 10);
                }
                else {
                    pos = "auto";
                    height = baseLayer.legendSettings.height;
                    width = baseLayer.legendSettings.width;
                }
                this._scrollLegendDiv.css({
                    "position": "absolute",
                    'z-index': '2', 'height': this._legendSize.height, 'width': this._legendSize.width
                });

                if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Center) {
                    this._scrollLegendDiv.css({
                        "margin-left": (this._width / 2) - (this._legendSize.width / 2)
                    });
                }
                else if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Far) {
                    this._scrollLegendDiv.css({
                        "margin-left": this._width - this._legendSize.width
                    });
                }
                else if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Center) {
                    this._scrollLegendDiv.css({
                        "margin-top": (this._height / 2) - (this._legendSize.height / 2)
                    });
                }
                else if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Far) {
                    this._scrollLegendDiv.css({
                        "margin-top": this._height - this._legendSize.height
                    });
                }

                if (baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) {
                    this._scrollLegendDiv.css({
                        "margin-top": this._height
                    });
                }
                else if (baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) {

                    this._scrollLegendDiv.css({

                        "margin-left": this._width

                    });
                }
            }

           
            this.refreshLayers();
            this.refreshNavigationControl(this.model.navigationControl);
        },
       
		_renderMapLayers:function(){
		
		 $(this._templateDiv).empty();
            var prevPathCount = 0;
            var baseLayer = this.model.layers[this.baseMapIndex()];
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Geometry) {
                this._renderLayers(baseLayer, prevPathCount);
                if (baseLayer.shapeData != null)
                    prevPathCount = baseLayer._polygonData.length;
            }
            else {
                baseLayer._initializeLocalValues();
                var contribution = $("<div class='map-contribution'>");
                contribution[0].innerHTML = baseLayer.contribution;
                this._mapContainer.append(contribution);
            }
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                var sublayer = this.model.layers[this.baseMapIndex()].subLayers[key];
                sublayer._isBaseLayer = false;
                if (sublayer.layerType == ej.datavisualization.Map.LayerType.Geometry && sublayer.shapeData!=null) {
                    if (ej.util.isNullOrUndefined(sublayer._initializeLocalValues)) {
                        var subobj = new shapeLayer();
                        $.extend(subobj, sublayer);
                        $.extend(sublayer, subobj);
                    }
                    this._renderLayers(sublayer, prevPathCount);
                    prevPathCount += sublayer._polygonData.length;
                }
                else {
                    sublayer._initializeLocalValues();
                }
            }
            this.refreshMarkers();
            if (_isSVG && this.enableAnimation() && !this._isTileMap) {
                this._refreshWithAnimation();
            }
            else {
                this._refrshLayers();
            }

		},

        refreshLayers: function () {          
		   var baseLayer = this.model.layers[this.baseMapIndex()];
		   this._processOData(baseLayer,this);          
		   this.shapeSelectionOnResize();
        },
		
		_processOData:function(layer,map){		    
		    if(layer.dataSource!=null){
                      if (layer.dataSource instanceof ej.DataManager) {
				          var queryPromise = layer.dataSource.executeQuery(layer.query);
			             queryPromise.done(function (e) {
			             if(e.result!=null)
				         {
                            layer.dataSource=e.result;
						    map._renderMapLayers();
				         }
			           });        
				  }
				 else
				 {
				    map._renderMapLayers();
				 }
            }
			else
			{
			  map._renderMapLayers();
			}
		}
		
		

    });

 

    var Tile = function (x, y) {
        this.X = x;
        this.Y = y;
        this.left = 0;
        this.top = 0;       
        this.height = 256;
        this.width = 256;
        this.src = null;
    };
   
    var shapeLayer = function () {
                
        this.enableSelection = true;
        this.selectionMode = "default";		
		this.bingMapType = "aerial";	
        this.key = "";   
        this.selectedItems = [];        
        this.enableMouseHover = false;
        this.shapeData = null;
        this.markers = [];
        this.dataSource = null;
        this.urlTemplate = 'http://a.tile.openstreetmap.org/level/tileX/tileY.png';
        this.showTooltip = false;
        this.tooltipTemplate = null;
        this.mapItemsTemplate = null;
        this.enableAnimation = false;

        this.legendSettings = {		
			showLegend:false,		
			showLabels:false,	
            rowSpacing: 10,
            columnSpacing:10,
			position: "topleft",			
			positionX: 0,		
			positionY:0,				
			height:0, 		
			width:0,
			iconHeight: 20,
			iconWidth: 20,		
			type:"layers", 		
			mode:"default", 					
			title: null, 			
			leftLabel:null, 						
			rightLabel:null,			
			icon:"rectangle",						
			dockOnMap:false,					
			dockPosition: "top",					
            labelOrientation: "vertical",
			alignment: "bottom",
            columnCount:0
        };

        this.labelSettings = {			
			showLabels: false, 						
			labelPath: "",			
			enableSmartLabel: false,			
			smartLabelSize:"fixed",			
			labelLength:2
		};        
        this.markerTemplate = null;       
		this.showMapItems = false;      
        this.layerType = 'geometry';
        this.geometryType = "geographic";
        this._colorPaletteSettings = {
            'Palette1':
            {
                'highlightColor': '#F7CD1C',
                'highlightStroke': 'white',
                'SelectionColor': '#F96C0D',
                'SelectionStroke': 'white'
                
            },
            'Palette2':
            {
                'highlightColor': '#68A3EA',
                'highlightStroke': 'White',
                'SelectionColor': '#116EF4',
                'SelectionStroke': 'White'
               
            },
            'Palette3':
            {
                'highlightColor': '#CCCCCC',
                'highlightStroke': 'white',
                'SelectionColor': '#4D4D4D',
                'SelectionStroke': 'white'
                
            }
        };
        this.colorPalettes = {
            'Palette1': ['#4A3825', '#736F3D','#F2DABD','#BF9D7E','#7F6039','#7F715F','#70845D','#CC995C','#736F3D','#89541B'],
            'Palette2': ['#E51400', '#730202','#EF6535','#C63477','#BF004D','#F0A30B','#CE1B1B','#97993D','#D6BF38','#835A2C'],
            'Palette3': ['#A4C400', '#008B00','#1BA0E2','#0050EF','#AA00FF','#D90073','#E51400','#F96800','#E3C800','#A20026']
        };
        
        this._prevPaletteIndex = 0;        
        this._newBounds = [];      
        this.subLayers = [];

        this.shapeSettings = {           
            highlightColor: "gray",          
            highlightBorderWidth: 1,            
            selectionColor: "gray",
            fill: "#E5E5E5",          
            radius: 5,
            strokeThickness: "0.2",            
            stroke: "#C1C1C1",
            colorPalette: 'palette1',
            highlightStroke: "#C1C1C1",
            selectionStroke: "#C1C1C1",           
            selectionStrokeWidth: 1,
            colorValuePath : null,
            valuePath: null,
            enableGradient: false,        
            colorMappings: null,
            autoFill: false
        };

       
        this.bubbleSettings = {            
            showBubble: true,          
            bubbleOpacity: "0.9",         
            minValue: 10,
            maxValue: 20,
            color: "gray",
            colorValuePath : null,
            valuePath: null,
            colorMappings: null,
            showTooltip: false,
            tooltipTemplate: null
        };
    };

    shapeLayer.prototype =
    {        
        dataTypes: {
            dataSource: "data",
            markers: "array",
            subLayers: "array",
            shapeSettings: {
                colorMappings: "array"
            },
            bubbleSettings: {
                colorMappings: "array"
            }

        },
       
        _initializeLocalValues:function()
        {
            this._svgns = "http://www.w3.org/2000/svg";
            this._bounds = [];
            this._bubbleCollection = [];
            this._prevSelectedShapes = [];
            this._prevSelectedTemp = [];
            this._prevSelectedLegends = [];
            this._isBaseLayer = true;
            this._prevHoverdLegend = null;
            this._prevHoveredShape = null;
            this._labelCollection = [];
            this._scrollBar = null;
            this._mapShapes = [];
            this._bubbles = [];
            this._labelBounds = [];            
            this._bubbleCount = 0;
            this._mapItems = [];           
            this._mapMarkers = [];
            this.selectedItems;
			this._tooltipSize = { height: 0, width: 0 };
            this._smartLabels = [];
            this._labelData = [];
            this._interactiveArrow = null;
            this._legendRects = [];

        },
     
        _generateMarkerForLayers: function (map) {
            this._mapMarkers = [];
            var rootTop = map._rootgroup.getBoundingClientRect().top;
            var rootLeft = map._rootgroup.getBoundingClientRect().left;
            for (var key = 0; key < this.markers.length; key++) {
                var markerObeject = this.markers[key];               
                if (this.markerTemplate != null) {
                    var markerTemplateDiv = $('.markerTemplateDiv');
                    markerTemplateDiv = $("<div class='mapMarker' style='display:block;position:absolute;pointer-events: stroke;'></div>");                   
                    map._templateDiv.append(markerTemplateDiv);
                    $(markerTemplateDiv).css({ height: this._height, width: this._width, "margin-top": this._margintop, "margin-left": this._marginleft });
                    var htmlString = $("#" + this.markerTemplate).render(markerObeject);
					if(map.model.markerSelected==null)
					{
						
						$(markerTemplateDiv).css({  "pointer-events": "none"});
					}
                    $(markerTemplateDiv).html(htmlString);
                    map._on($(markerTemplateDiv), ej.eventType.mouseDown, { marker: $(markerTemplateDiv), data: markerObeject }, map._markerPressed);
                    this._mapMarkers.push(markerTemplateDiv);
                }
                else {                    
                    var markerItem = $(' <div class="mapMarker" style="display:block;position:absolute;pointer-events: stroke;"><label>' + markerObeject.label + '</label></div>');// document.createElementNS(this._svgns, "text");
                    if (_isSVG) {
                        map._templateDiv.append(markerItem);
                    }
                    else {
                        markerItem.appendTo(map._templateDiv);
                    }
					if(map.model.markerSelected==null)
					{
						
						$(markerItem).css({  "pointer-events": "none"});
					}
                    map._on($(markerItem), ej.eventType.mouseDown, { marker: $(markerItem), data: markerObeject }, map._markerPressed);
                    this._mapMarkers.push(markerItem);

                }
                  var baseLayer = this;
                if (baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left) {
                    this._height = this._height;
                    this._width = this._width - parseFloat(width);
                    this._marginleft = parseFloat(width);
             }            
            }            
        },

        _contains : function(array,actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length] === actualobj) {
                        return true;
                    }
                }
            }
            return false;
        },
       
          _shapeContains: function (array, actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length].shapeIndex === actualobj) {
                        return { isContains: true, index: array.length - length };
                    }
                }
            }
            return { isContains: false };

        },  

          _labelSizeCalculation: function (map) {

              var size = document.getElementsByClassName("e-map-labelContainer");
              var totalHeight = size[0].offsetHeight;
              var totalWidth = size[0].offsetWidth;
              
              map._width = map._width - totalWidth;
              map._marginleft = totalWidth;
              
          },
		
		 _sizeCalculation: function (map) {

            var colorMappings = this.shapeSettings.colorMappings;
            var isRange = false;
            if (this.shapeSettings.colorMappings != null) {

                if (colorMappings.rangeColorMapping != null)
                {
                    colorMappings = colorMappings.rangeColorMapping;
                    isRange = true;
                }
                if (colorMappings.equalColorMapping != null)
                {
                    colorMappings = colorMappings.equalColorMapping;
                }
                        
                var width = 0;
                var height = 0;
                var rowSpacing = this.legendSettings.rowSpacing;
                var columnSpacing = this.legendSettings.columnSpacing;
                var iconWidth = this.legendSettings.iconWidth + 5;
                var iconHeight = this.legendSettings.iconHeight + 5;
                var legendHeight = iconHeight + rowSpacing;
                var totalHeight = this.legendSettings.iconHeight + rowSpacing;
                var totalWidth = 0;
                var yPos = 10;
                var xPos = 10;
                var legendSettings = this.legendSettings;
                var columnCount = this.legendSettings.columnCount;
                if (legendSettings.height == 0 && legendSettings.width == 0  && columnCount == 0) {
                    for (var i = 0; i < colorMappings.length; i++) {
                        var label = colorMappings[i].legendLabel != null ? colorMappings[i].legendLabel : (isRange ? colorMappings[i].from : colorMappings[i].value);
                        var labelwidth = this._calcWidth(label);
                        var legendWidth = iconWidth + labelwidth + columnSpacing ;

                        if ((legendSettings.dockOnMap && (legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom))||!legendSettings.dockOnMap) {
                            if (map._width < (width + legendWidth)) {
                                totalHeight = ((iconHeight) + rowSpacing) + totalHeight;
                                totalWidth = Math.max(totalWidth, width);
                                width = legendWidth;
                            }
                            else {
                                width += (legendWidth + 5);
                            }
                        }
                        else {

                            if (map._height < (height + iconHeight + rowSpacing)) {
                                totalWidth += width;
                                totalHeight = Math.max(totalHeight, height);
                                width = legendWidth;
                                height = 0;
                            }
                            else {
                                height += (iconHeight + rowSpacing);
                                width = Math.max(legendWidth, width);
                            }
                        }


                    }
                    if ((legendSettings.dockOnMap && (legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom)) || !legendSettings.dockOnMap) {
                        totalWidth = Math.max(totalWidth, width);
                        totalHeight += 5;
                    }
                    else {
                        totalWidth += (width + columnSpacing);
                        totalHeight = Math.max(totalHeight, height);
                    }

                }
                else if (columnCount == 0) {
                    if ((legendSettings.height.toString().indexOf("%")) != -1) {

                        totalHeight = (map._height / 100) * parseInt(legendSettings.height.toString().replace('%', ''))
                    }
                    else {

                        totalHeight = legendSettings.height;
                    }
                    if ((legendSettings.width.toString().indexOf("%")) != -1) {

                        totalWidth = (map._width / 100) * parseInt(legendSettings.width.replace('%', ''))
                    }
                    else {

                        totalWidth = legendSettings.width;
                    }

                }

                if (legendSettings.height == 0 && legendSettings.width == 0 && columnCount != 0) {

                    for (var i = 0; i < colorMappings.length; i++) {
                        var label = colorMappings[i].legendLabel != null ? colorMappings[i].legendLabel : (isRange ? colorMappings[i].from : colorMappings[i].value);
                        var labelwidth = this._calcWidth(label);

                        var legendWidth = iconWidth + labelwidth + columnSpacing ;

                        if (i % columnCount != 0) {
                            width += legendWidth;
                            if (i == columnCount - 1) {
                                totalWidth = Math.max(totalWidth, width);
                            }

                        }
                        else {
                            if (i != 0)
                                totalHeight = (iconWidth + rowSpacing) + totalHeight;
                            totalWidth = Math.max(totalWidth, width);
                            width = legendWidth;
                        }
                    }
                }
            }
        
        
            if (legendSettings.height != 0 && legendSettings.width != 0)
			{
				 totalHeight = legendSettings.height;
				 totalWidth = legendSettings.width;
            }

			map._legendSize = { height: totalHeight, width: totalWidth };
			if (legendSettings.dockOnMap) {
			    if (legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
			        totalHeight = 55;
			    }
			    if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) {
			        map._height = map._height - parseFloat(totalHeight);
			        this.legendSettings.tempWidth = width;
			    }

			    else if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top) {
			        map._height = map._height - parseFloat(totalHeight);
			        map._margintop = parseFloat(totalHeight);
			    }
			    else if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left) {

			        map._width = map._width - totalWidth;
			        map._marginleft = totalWidth;
			    }
			    else if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) {
			        map._height = map._height;
			        map._width = map._width - totalWidth;
			    }
			}



        },

        
        _clearShapeWidth: function (scale) {
            if (scale == null) scale = 1;
            for (var index = 0; index < this._mapShapes.length; index++) {
                var mapshape = this._mapShapes[index];
                if (!this._contains(this._prevSelectedShapes,mapshape.shape)) {
                    if (_isSVG) {
                        if (this.shapeSettings.colorMappings != null || this.shapeSettings.autoFill) {
                            mapshape.shape.setAttribute("fill", mapshape.shape.getAttribute("nodeValue"));                            
                            
                        }
                        else
                        {  
						    mapshape.shape.setAttribute("nodeValue", this.shapeSettings.fill);
                            mapshape.shape.setAttribute("fill", this.shapeSettings.fill);
                        }                        

                        mapshape.shape.setAttribute("stroke-width", (this.shapeSettings.strokeThickness / scale));
                        mapshape.shape.setAttribute("stroke", this.shapeSettings.stroke);
                    }
                    else {
                        mapshape.shape.fillcolor = mapshape.shape.style.behavior;
                        mapshape.shape.strokeweight = this.shapeSettings.strokeThickness;
                        mapshape.shape.strokecolor = this.shapeSettings.stroke;
                    }
                }
            }
            


        },
			
       _shapeSelection: function () {
            for (var index = 0; index < this._mapShapes.length; index++) {
                var mapshape = this._mapShapes[index];
                var check = this._shapeContains(this.selectedItems, mapshape.shapeIndex);
                if (check.isContains) {
                     if (!this._contains(this._prevSelectedShapes, mapshape.shape))
                     this._prevSelectedShapes.push(mapshape.shape);
                     if (_isSVG) {
                        mapshape.shape.setAttribute("fill", this.shapeSettings.selectionColor);
                        mapshape.shape.setAttribute("stroke-width", this.shapeSettings.selectionStrokeWidth);
                        mapshape.shape.setAttribute("stroke", this.shapeSettings.selectionStroke);
                    }
                    else {
                        mapshape.shape.fillcolor = this.shapeSettings.selectionColor;
                        mapshape.shape.strokeweight = this.shapeSettings.selectionStrokeWidth;
                        mapshape.shape.strokecolor = this.shapeSettings.selectionStroke;
                    }
                }
            }
        },
                
        _rgbToHex: function(R, G, B) {
             return "#" + this._toHex(R) + this._toHex(G) + this._toHex(B);
        },
                
        _toHex: function (n) {
            var charstr = "0123456789ABCDEF";
            n = parseInt(n, 10);
            if (isNaN(n)) return "00";
            n = Math.max(0, Math.min(n, 255));
            return charstr.charAt((n - n % 16) / 16)
                + charstr.charAt(n % 16);
        },
                
        _hexToR: function(h) {
             return parseInt((this._cropHex(h)).substring(0, 2), 16);
        },
               
        _hexToG: function(h) {
             return parseInt((this._cropHex(h)).substring(2, 4), 16);
        },
       
        _hexToB: function(h) {
             return parseInt((this._cropHex(h)).substring(4, 6), 16);
        },
      
        _cropHex: function(h) {
             return (h.charAt(0) == "#") ? h.substring(1, 7) : h;
        },
                
        _getRangeColorValues: function(range, start, end) {
            var rangeColorValues = [];
            rangeColorValues.push(start);
            if (start > end) {
                var rangeValue = (start - end) / (range-1);
                for (var i = range; i > 2; i--) {
                    start = start - rangeValue;
                    rangeColorValues.push(start);
                }
            }
            else {
                var rangeValue = (end - start) / (range-1);
                for (var i = 2; i < range; i++) {
                    start = start + rangeValue;
                    rangeColorValues.push(start + rangeValue);
                }
            }
            rangeColorValues.push(end);
            return rangeColorValues;
        },
       
        _generateGradientCollection: function(gradientColors) {
            var gradientCollection = [];
            var startRangeColor = gradientColors[0];
            var endRangeColor = gradientColors[gradientColors.length - 1];
            var startR = this._hexToR(startRangeColor);
            var startG = this._hexToG(startRangeColor);
            var startB = this._hexToB(startRangeColor);
            var endR = this._hexToR(endRangeColor);
            var endG = this._hexToG(endRangeColor);
            var endB = this._hexToB(endRangeColor);
            var range = 10;
            var rRange = this._getRangeColorValues(range, startR, endR);
            var gRange = this._getRangeColorValues(range, startG, endG);
            var bRange = this._getRangeColorValues(range, startB, endB);
            for (var i = 0; i < range; i++) {
                gradientCollection.push(this._rgbToHex(rRange[i], gRange[i], bRange[i]));
            }
            return gradientCollection;
        },
		
        _calculateTextWidth: function (text) {
            var span = $('<span>' + text + '</span>');
            $('body').append(span);
            var width = span.width();
            span.remove();    
            return width;
        },
        
        _trimFunction: function (str, width) {
      
            var span = $("#spantag").text(str);
            var text = str;
            while (span.width() > width)
            {    
                text = text.slice(0, -2);
                span.text(text + "...");   
            }

            return text;
    
        },
               
        _createLabel:function(content, xpos, ypos,className) {
            var label = $("<div class="+className+"></div>"); // document.createElementNS(this._svgns, "text");
			label[0].innerHTML = content;
			label.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute"
			});
            return label;
        },
                
        _createInteractiveArrow: function (xpos, ypos) {
            
            var interactiveElement = $("<div class='e-icon1 e-interactiveArrow'></div>");            
            interactiveElement[0].innerHTML = "&#9650";
            
            interactiveElement.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute",                
                
            });
            return interactiveElement;
        },
         _getEllipseLegend: function (xPos, yPos) {

            var rect = $("<div class='e-mapLegend'/>");
            rect.css({
                "height": this.legendSettings.iconHeight + "px",
                "width": this.legendSettings.iconWidth + "px",
                "border-radius": this.legendSettings.iconHeight / 2 + "px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;

        },

        _getRectLegend: function (xPos, yPos) {
            var rect = $("<div />");
            rect.css({
                "height": this.legendSettings.iconHeight + "px",
                "width": this.legendSettings.iconWidth + "px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;
        },
		
		_generateLegend: function (map) {

            var isRange = true;
            var colormapping = null;
            if (this.shapeSettings.colorMappings.rangeColorMapping != undefined) {
                colormapping = this.shapeSettings.colorMappings.rangeColorMapping;
            } else if (this.shapeSettings.colorMappings.equalColorMapping != undefined) {
                colormapping = this.shapeSettings.colorMappings.equalColorMapping;
                isRange = false;
            }
                var xPos = 0;
                var yPos = 0;
                var legendSettings = this.legendSettings;
                var columnWidth = 0;
                var height1;
                var width1;                
           
            var rowSpacing = legendSettings.rowSpacing;
            var columnSpacing = legendSettings.columnSpacing;
            var iconwidth = legendSettings.iconWidth + 5;
            var totalHeight = legendSettings.iconHeight + rowSpacing;
            var totalWidth = 0;
            var columnCount = legendSettings.columnCount;
            for (var i = 0; i < colormapping.length; i++) {
                var label = colormapping[i].legendLabel = colormapping[i].legendLabel != null ? colormapping[i].legendLabel : (isRange ? colormapping[i].from : colormapping[i].value);
                var labelwidth = this._calcWidth(label);

                var legendWidth = legendSettings.iconWidth + columnSpacing + labelwidth;
                var legendHight = legendSettings.iconHeight;


                    if (columnCount != 0) {
                        if (i % columnCount != 0) {

                            this._drawLegend(colormapping[i], xPos, yPos,map);
                            xPos += (legendWidth + 5);

                        }
                    else {
                        if (i != 0)
                            yPos += (legendSettings.iconHeight + rowSpacing);
                        xPos = 0;
                        this._drawLegend(colormapping[i], xPos, yPos, map);
                        xPos += (legendWidth + 5);
                    }
                }
                else {
                    if (legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) {

                        if (map._legendSize.width < xPos + legendWidth) {

                            yPos += (legendSettings.iconHeight + rowSpacing);
                            xPos = 0;
                            this._drawLegend(colormapping[i], xPos, yPos, map);
                            xPos += legendWidth;
                        }
                        else {
                            this._drawLegend(colormapping[i], xPos, yPos, map);
                            xPos += legendWidth;
                        }
                    }
                    else {

                        if (map._legendSize.height < yPos + legendSettings.iconHeight) {

                            yPos = 0;
                            xPos += (columnWidth + (columnSpacing ));
                            this._drawLegend(colormapping[i], xPos, yPos, map);
                            columnWidth = 0;
                            yPos += (legendSettings.iconHeight + rowSpacing);
                        }
                        else {
                            this._drawLegend(colormapping[i], xPos, yPos, map);
                            columnWidth = Math.max(columnWidth, legendWidth);
                            yPos += (legendSettings.iconHeight + rowSpacing);
                        }
                    }

                }

                    if (map._legendDivHeight > yPos) {
                        height1 = map._legendDivHeight;
                    }
                    else {
                        height1 = yPos + legendHight;
                    }

                    if (map._legendDivWidth > xPos + legendWidth) {
                        width1 = map._legendDivWidth;
                    }
                    else {
                        width1 = xPos + legendWidth;
                    }

                    map._legendDivWidth = width1;
                    map._legendDivHeight = height1;                    
                    
            }
        },
        _drawLegend: function (colormapping, xPos, yPos,map) {
            var legendItem;
            if (this.legendSettings.icon == ej.datavisualization.Map.LegendIcons.Circle) {
                legendItem = this._getEllipseLegend(xPos, yPos);
            }
            else {
                legendItem = this._getRectLegend( xPos, yPos);
            }
            xPos += (this.legendSettings.iconWidth + 5);
            $(legendItem).css("background-color", colormapping.color);
            legendItem.appendTo(map._legendDiv);
            var legendText = $("<div class='e-defaultLegendLabel'/>");
            legendText.css({
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute",
                "text-overflow": "ellipsis",
                "white-space": "nowrap",
                "overflow":"hidden"
            });
            legendText[0].title = colormapping.legendLabel;
            legendText[0].innerHTML = colormapping.legendLabel;
            legendText.appendTo(map._legendDiv);
        },



      _generateLegends: function (map) {
            var colorMappings = this.shapeSettings.colorMappings;
            if ((colorMappings.rangeColorMapping != null) || (colorMappings.equalColorMapping != null)) {
                var xPos = 0;
                var yPos = 0;
                var legendSettings = this.legendSettings;
                var columnWidth = 0;
                var width = 0;
                var height = 0;
                var iconWidth = this.legendSettings.iconWidth + 5;
                var iconHeight = this.legendSettings.iconHeight + 5;
                var rowSpacing = this.legendSettings.rowSpacing;
                var columnSpacing = this.legendSettings.columnSpacing;
                var legendHeight = this.legendSettings.iconHeight + this.legendSettings.rowSpacing;
                var totalHeight = iconHeight + this.legendSettings.rowSpacing;
                var totalWidth = 0;
                if ((this.legendSettings.showLegend == undefined || this.legendSettings.showLegend) && !this.shapeSettings.autoFill) {
                    $(map._mapContainer).append($('<div  id="labelTooltip" style="display:none;background-color:grey;padding-left:5px; padding-right:5px;position:absolute;z-index:1000;pointer-events:none;"/>'));
                    var baseLayer = this;
                    var xpos = 10;
                    var ypos = 0;

                    var totalHeight = this.iconHeight + this.legendSettings.rowSpacing;
                    var _legendheight = this.legendSettings.height;
                    var _legendwidth = this.legendSettings.width;
                    var legenddiv = $("<div/>");
                    if (baseLayer.legendSettings.dockOnMap) {
                        legenddiv = $(map._legendDiv);
                    }
                    else if (!baseLayer.legendSettings.dockOnMap && (baseLayer.legendSettings.mode != ej.datavisualization.Map.LegendMode.Interactive)) {
                        legenddiv = $(map._legendDiv);
                    }
                    else {
                        if (_isSVG) {
                            legenddiv.appendTo(map._templateDiv);
                        }
                        else {
                            map._templateDiv.append(legenddiv);
                        }
                    }


                    if ((this.legendSettings.type == undefined || this.legendSettings.type == ej.datavisualization.Map.LegendType.Layers) && this.shapeSettings.colorMappings != null
                       && (this.legendSettings.mode == undefined || this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default || (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive
                       && this.shapeSettings.colorMappings.equalColorMapping != null))) {

                        this._generateLegend(map);


                    } else if ((this.legendSettings.type == undefined || this.legendSettings.type == ej.datavisualization.Map.LegendType.Layers) && this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && this.shapeSettings.colorMappings != null) {
                        var textcon = '';

                        if (this.legendSettings.height == 0)
                            _legendheight = 18;
                        if (this.legendSettings.width == 0)
                            _legendwidth = 150;

                        if (this.legendSettings.leftLabel == null)
                            this.legendSettings.leftLabel = '';
						if (this.legendSettings.rightLabel == null)
							this.legendSettings.rightLabel = '';
                        if (this.legendSettings.title != null) {
                            var newxpos = xpos;
                            if (!this.legendSettings.showLabels)
                                newxpos = xpos + this.legendSettings.leftLabel.length * 10;

                            var _legendtitlewidth = this.legendSettings.title.length * 10;
                            var text = titleText = this.legendSettings.title;;

                            if (_legendtitlewidth > _legendwidth) {
                                for (var i = 1; i < titleText.toString().length; i++) {
                                    text = titleText.toString().substring(0, i - 1) + '...';
                                }
                            }
                            var textcon = this._createLabel(text, newxpos, ypos, 'e-interactivelegend-title');
                            textcon[0].title = titleText;
                            textcon.css({
                                "width": _legendwidth + "px"
                            });
                            if (_isSVG)
                                textcon.appendTo(legenddiv);
                            else
                                legenddiv.append(textcon);
                            ypos += 25;
                        }

                        if (this.legendSettings.showLabels)
                            ypos += 25;

                        if (this.legendSettings.leftLabel != null && !this.legendSettings.showLabels) {
                            var textcon = this._createLabel(this.legendSettings.leftLabel, xpos, ypos - 3, 'e-interactivelegend-leftlabel');

                            if (_isSVG)
                                textcon.appendTo(legenddiv);
                            else
                                legenddiv.append(textcon);
                            xpos = xpos + this.legendSettings.leftLabel.length * 10;
                        }

                        var interactiveElement = this._createInteractiveArrow(xpos, ypos + _legendheight);
                        interactiveElement.appendTo(legenddiv);
                        this._interactiveArrow = interactiveElement;

                        var _legendgroup = null;
                        if (!_isSVG && this.shapeSettings.enableGradient) {
                            _legendgroup = map._createGroup(false, "legendGroup");
                            _legendgroup.style.left = 0 + 'px';
                            _legendgroup.style.top = 0 + 'px';
                            _legendgroup.style.position = "relative";
                            legenddiv.append(_legendgroup);
                        }
						
                        var mappings= colorMappings.rangeColorMapping;
                        for (var key = 0; key < mappings.length; key++) {
                            var colorMapping = mappings[key];
                            if (!colorMapping.hideLegend) {
                                var gradientCollection = [];
                                if (this.shapeSettings.enableGradient) {
                                    gradientCollection = this._generateGradientCollection(colorMapping.gradientColors);
                                }

                                var obj = {};
                                if (this.shapeSettings.enableGradient) {
                                    if (_isSVG) {
                                        var canvas = $("<canvas/>");
                                        var ctx = canvas[0].getContext("2d");
                                        var grd = ctx.createLinearGradient(0, 0, 300, 0);

                                        var temp = 0;
                                        for (var i = 0; i < 10; i++) {
                                            temp = temp + (1 / 10);
                                            if (i == 0) {
                                                grd.addColorStop(0, colorMapping.gradientColors[0]);
                                                grd.addColorStop(temp, gradientCollection[i]);
                                            } else if (i == gradientCollection.length - 1) {
                                                grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                                grd.addColorStop(temp, colorMapping.gradientColors[1]);
                                            } else {
                                                grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                                grd.addColorStop(temp, gradientCollection[i + 1]);
                                            }
                                        }
                                        ctx.fillStyle = grd;
                                        ctx.fillRect(0, 0, 300, 300);
                                        canvas.css({
                                            "height": _legendheight + "px",
                                            "width": (_legendwidth / mappings.length) + "px",
                                            "margin-left": xpos + "px",
                                            "margin-top": ypos + "px",
                                            "opacity": "0.9",
                                            "filter": "alpha(opacity=90)", /* For IE8 and earlier */
                                            "position": "absolute"
                                        });
                                        canvas.appendTo(legenddiv);
                                    } else {
                                        var legendID = "legend" + key;
                                        var legendHtmlString = '<v:rect id=' + legendID + ' display="block" style="position:absolute;top: ' + (ypos - 2) + 'px;left:' + xpos + 'px;width:' + (_legendwidth / mappings.length) + 'px;height:' + _legendheight + 'px;"><v:fill opacity="0.9px" type="gradient" method="linear sigma" angle="270"/><v:stroke opacity="0px"/></v:rect>';
                                        _legendgroup.innerHTML = _legendgroup.innerHTML + legendHtmlString;
                                        var legendelement = document.getElementById(legendID);
                                        legendelement.fillcolor = colorMapping.gradientColors[0];
                                        legendelement.fill.color2 = colorMapping.gradientColors[1];
                                    }
                                }
                                else {
                                    var rect = $("<div/>");
                                    rect.css({
                                        "height": _legendheight + "px",
                                        "width": _legendwidth / this.shapeSettings.colorMappings.rangeColorMapping.length + "px",
                                        "background-color": colorMapping.color,
                                        "margin-left": xpos + "px",
                                        "margin-top": ypos + "px",
                                        "opacity": "0.9",
                                        "filter": "alpha(opacity=90)",/* For IE8 and earlier */
                                        "position": "absolute"
                                    });

                                    if (_isSVG)
                                        rect.appendTo(legenddiv);
                                    else
                                        legenddiv.append(rect);
                                }

                                for (var i = 0; i < 10; i++) {
                                    obj = {};
                                    obj["marginLeft"] = xpos;
                                    this._legendRects.push(obj);
                                    xpos = xpos + (_legendwidth / mappings.length) / 10;
                                }

                                if (this.legendSettings.showLabels) {
                                    var labelxpos = xpos - (_legendwidth / mappings.length);
                                    var labelypos = ypos - 25;
                                    var startlabel = this._createLabel((colorMapping.from), labelxpos, labelypos, 'e-legend-rangestartlabel');
                                    labelxpos = xpos;
                                    var endlabel = this._createLabel((colorMapping.to), labelxpos, labelypos);
                                    if (colorMapping.legendLabel != undefined)
                                        endlabel = this._createLabel((colorMapping.legendLabel), labelxpos - (colorMapping.legendLabel.length * 10) / 2, labelypos, 'e-legend-rangeendlabel');
                                    if (_isSVG) {
                                        if (colorMapping == mappings[0])
                                            startlabel.appendTo(legenddiv);
                                        endlabel.appendTo(legenddiv);
                                    }
                                    else {
                                        if (colorMapping == mappings[0])
                                            legenddiv.append(startlabel);
                                        legenddiv.append(endlabel);
                                    }
                                }
                            }
                        }

                        if (this.legendSettings.rightLabel != null && !this.legendSettings.showLabels) {
                            var textcon = this._createLabel(this.legendSettings.rightLabel, xpos + 10, ypos - 3, 'e-interactivelegend-rightlabel');
                            if (_isSVG)
                                textcon.appendTo(legenddiv);
                            else
                                legenddiv.append(textcon);
                            xpos = xpos + this.legendSettings.rightLabel.length * 10;
                        }

                        totalwidth = xpos + 10;
                        totalheight = ypos + _legendheight  + 10;
						map._legendSize = { width: totalwidth, height: totalheight };
						if (baseLayer.legendSettings.dockOnMap) {
						    if (this.legendSettings.dockPosition == 'left')
						        map._marginleft = totalwidth;
						    else if (this.legendSettings.dockPosition == 'top')
						        map._margintop = totalheight;
						    
						}
                    }

                    if (!baseLayer.legendSettings.dockOnMap && this.legendSettings.position == 'none') {
                        var posx = (map._width * this.legendSettings.positionX) / 100;
                        var posy = (map._height * this.legendSettings.positionY) / 100;
                        legenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                        
                    }
                    else if (!baseLayer.legendSettings.dockOnMap && (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive)) {
                        var position = (this.legendSettings.position == undefined) ? "topleft" : this.legendSettings.position;
                        var controlSize = { width: totalwidth, height: totalheight };
                        var navPos = map._getPosition(position, controlSize);
                        legenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y });

                    }
                }
                    

                   
                
            }
        },

      _generateLabels: function (map) {          
          
          var labeldiv = $("<div class='e-map-labelContainer'></div>");

          $(labeldiv).css({ "position": "absolute", "overflow": "scroll" });
                 
          for (var dataIndex = 0; dataIndex < this._polygonData.length; dataIndex++) {

              var ValueObject = this._polygonData[dataIndex].properties;
              var labelValue = map._reflection(ValueObject, this.labelSettings.labelPath);                          
              var labelElement = $("<div class='e-map-label'></div>");
              $(labelElement).css({ "margin-top":dataIndex*20, "position": "absolute" });
              labelElement[0].innerHTML = labelValue;
              labeldiv.append(labelElement);              
              map._mapContainer.append(labeldiv);              
              labelElement.mouseenter({ Param1: this, Param2: this._mapShapes[dataIndex], map: map }, map._polyEnterFunction);

              map._off($(labelElement), ej.eventType.mouseUp, map._polyClickFunction);              
              map._on($(labelElement), ej.eventType.mouseUp, { Param1: this._mapShapes[dataIndex], Param2: this, Param3: this._mapShapes[dataIndex].shape, param4: labelElement }, map._polyClickFunction);              

          }         

      },

        _calcWidth: function (text) {
            var span = $('<span  class="e-defaultLegendLabel">' + text + '</span>');
            $('body').append(span);
            var width = span.width()+5;
            span.remove();
            return width;
        },
       

		_generateBubbleLegends: function (map) {
		    if (this.legendSettings.showLegend == undefined || this.legendSettings.showLegend) {
			    var xpos = 10;
                var ypos = 10;
                var _bubblelegendheight = this.legendSettings.iconHeight;
                var _bubblelegendwidth = this.legendSettings.iconWidth;
                var bubblelegenddiv = $("<div/>");
                if (_isSVG) {
                    bubblelegenddiv.appendTo(map._templateDiv);
                }
                else {
                    map._templateDiv.append(bubblelegenddiv);
                }
				var totalwidth = 0;
                var totalheight = 0;
                var isRange = true;
                var mappings = null;
                if (this.bubbleSettings.colorMappings.rangeColorMapping != undefined) {
                    mappings = this.bubbleSettings.colorMappings.rangeColorMapping;
                } else if (this.bubbleSettings.colorMappings.equalColorMapping != undefined) {
                    mappings = this.bubbleSettings.colorMappings.equalColorMapping;
                    isRange = false;
                }
                if ((this.legendSettings.type == ej.datavisualization.Map.LegendType.Bubbles) && this.bubbleSettings.colorMappings != null
                   && (this.legendSettings.mode == undefined || this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default || (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive
                   && this.bubbleSettings.colorMappings.equalColorMapping != null))) {                    
				    if (this.legendSettings.iconHeight == undefined)				      
				    this.legendSettings.iconHeight = 20;
				    if (this.legendSettings.iconWidth == undefined)                    
				    this.legendSettings.iconWidth = 20;
                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var rect = $("<div class='e-mapBubbleLegend'/>");

                            if (this.legendSettings.icon == ej.datavisualization.Map.LegendIcons.Circle) {
                                rect.css({
                                    "height": this.legendSettings.iconHeight + "px",
                                    "width": this.legendSettings.iconWidth + "px",
                                    "border-radius": this.legendSettings.iconHeight / 2 + "px",
                                    "background-color": colorMapping.color,
                                    "margin-left": xpos + "px",
                                    "margin-top": ypos + "px",
                                    "position": "absolute"
                                });
                            }
                            else {
                                rect.css({
                                    "height": this.legendSettings.iconHeight + "px",
                                    "width": this.legendSettings.iconWidth + "px",
                                    "background-color": colorMapping.color,
                                    "margin-left": xpos + "px",
                                    "margin-top": ypos + "px",
                                    "position": "absolute"
                                });
                            }
							var textcon = this._createLabel(colorMapping.legendLabel, xpos + _bubblelegendwidth + 5, ypos, 'e-legendlabeltext');
							if (colorMapping.legendLabel != null) {
                                textcon[0].innerText= colorMapping.legendLabel;

                            } else {
                                colorMapping.legendLabel = textcon[0].innerText = isRange ? colorMapping.from : colorMapping.value;
                            }
							if (_isSVG) {
                                rect.appendTo(bubblelegenddiv);
                                textcon.appendTo(bubblelegenddiv);
                             }
                             else {
                                 bubblelegenddiv.append(rect);
                                 bubblelegenddiv.append(textcon);
							}
							if (totalwidth < (textcon[0].innerText.length * 10) + _bubblelegendwidth)
							    totalwidth = (textcon[0].innerText.length * 10) + _bubblelegendwidth;
                            var textHeight = textcon.height();
                            var rectHeight = rect.height();
                            if (textHeight > rectHeight) {
                                ypos += (textHeight + 5);
                            } else {
                                ypos += (rectHeight + 5);
                            }
						}
                    }
                    totalheight = ypos;
                }                
				if (this.legendSettings.position == 'none') {
				    var posx = (map._width * this.legendSettings.positionX) / 100;
				    var posy = (map._height * this.legendSettings.positionY) / 100;
                    bubblelegenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                } else {
				    var position = (this.legendSettings.position == undefined) ? "topleft" : this.legendSettings.position;
                    var controlSize = { width: totalwidth, height: totalheight };
                    var navPos = map._getPosition(position, controlSize);
                    bubblelegenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
            }
				
		},
		
        _animateBubble: function (element, delayInterval) {
            var radius = { "fx": element.getAttribute("r") /2 },
                to = radius.fx * 2,
                scaleVal,
                $ele = $(element),
                layer = this;
            $(radius).delay(delayInterval).each(function () { }).animate(
                {
                    fx: to
                },
                {
                    duration: 700,

                    step: function (now) {
                        if (_isSVG) {
                            $ele.attr("style", "display:block;pointer-events:stroke;");
                            $ele.attr("r", now);
                        }
                        else {
                        }
                    },
                    complete: function () {
                        layer._bubbleCount++;
                        if (layer._bubbleCount == layer._bubbles.length) {
                            layer._setMapElements();
                        }
                    }
                }

            );
        },
        
        _setMapElements: function () {
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];                
                $(item).css({ "display": "block" });
            }
        },

        _setMapItemsPositionWithAnimation: function (map) {
            this._bubbleCollection = [];
            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var bubblerad = this._bubbles[key].getAttribute('r');
                var position = this._bounds[key];               
                var xpos = ((position.x + map._translatePoint.x) * map._scale);
                var ypos = ((position.y + map._translatePoint.y) * map._scale);

                var finalPosition = map.validateBubblePosition(position.points, { x: xpos, y: ypos }, bubblerad);

                var ubound = 20;
                var lbound = 0;
                var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                var delayInterval = parseInt(randomValue * 50);
                if (_isSVG) {
                    $(bubble).attr({ "cx": finalPosition.x, "cy": finalPosition.y });
                }
                this._animateBubble(bubble, delayInterval);
                this._bubbleCollection.push(bubble);
            }
            for (var key = 0; key < this._mapMarkers.length; key++) {
                var item = this._mapMarkers[key];
                var marker;
                if (this.markers.length > 0) {
                    marker = this.markers[key];
                }                
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(marker.latitude, marker.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforMarker(marker.latitude, marker.longitude);
                }

                if (_isSVG) {
                    var xpos = position.x;
                }
                else {
                    var xpos = ((position.x + map._transformX) * map._scale);
                }
                var ypos = position.y;
                $(item).css({ "display":"block","left": xpos, "top": ypos - 100 });
                $(item).delay(500).each(function () { }).animate({ "top": ypos }, 500);
               
            }
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();               
                var cx = this._bubbleCollection[key] ?parseFloat(this._bubbleCollection[key].getAttribute("cx")) :0;
			    var cy =this._bubbleCollection[key] ? parseFloat(this._bubbleCollection[key].getAttribute("cy")):0;
			    var xpos = this._bubbleCollection[key] ? (cx - box.width / 2) : ((position.x + map._translatePoint.x) * map._scale) - (box.width / 2);
			    var ypos = this._bubbleCollection[key] ? (cy - box.height / 4) : ((position.y + map._translatePoint.y) * map._scale) - (box.height / 4);
                if(this._bubbles.length > 0)
				    $(item).css({ "left": xpos, "top": ypos, "display": "none" });
				else
                   $(item).css({ "left": xpos, "top": ypos, "display": "block" });
               
            }
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth =_isSVG?box.width: box.right - box.left;
                var boxheight=_isSVG?box.height: box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "smartLabelStyle") {                    
                    item[0].className = "labelStyle"; 
                     $(item[0]).css({ "pointer-events": "none", "position": "absolute" });                   
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos });

            }
            if (this.labelSettings!=null && this.labelSettings.showLabels) {
                this._validateSmartLabel(map);
            }
        },

        _resizeShapes: function (map) {
            var thickness = this.shapeSettings.strokeThickness / map._scale;
            if (this._mapShapes != undefined) {
            for (var i = 0; i < this._mapShapes.length; i++) {
                var element = this._mapShapes[i].shape;
                if (_isSVG) {
                    if (element.localName == 'circle') {
                        element.setAttribute("r", this.shapeSettings.radius / map._scale);
                    } else {
                        if (this._contains(this._prevSelectedShapes,element)) {
                            element.setAttribute("stroke-width", this.shapeSettings.selectionStrokeWidth / map._scale);
                        } else {
                            element.setAttribute("stroke-width", thickness);
                        }
                    }
                } else {
                    if (element.nodeName == 'oval') {

                    } else {
                        if (this._contains(this._prevSelectedShapes,element)) {
                            element.strokeweight = this.shapeSettings.selectionStrokeWidth / map._scale;
                        } else {
                            element.strokeweight = thickness;
                        }
                    }
                }
            }
          }
        },
       
       _setMapItemsPosition: function(map) {
            this._bubbleCollection = [];
			
			if(this._bubbles!=undefined)
			{
            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var bubblerad = this._bubbles[key].getAttribute('r');
                var position = this._bounds[key];
                if (_isSVG) {
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);

                    var finalPosition = map.validateBubblePosition(position.points, { x: xpos, y: ypos }, bubblerad);

                    $(bubble).attr({ "cx": finalPosition.x, "cy": finalPosition.y });
                }
                else {
                    bubble = document.getElementById(bubble.id);
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);

                    var finalPosition = map.validateBubblePosition(position.points, { x: xpos, y: ypos }, bubblerad);

                    var bubbleTop = finalPosition.y - (bubble.getBoundingClientRect().bottom - bubble.getBoundingClientRect().top) / 2;
                    var bubbleLeft = finalPosition.x - (bubble.getBoundingClientRect().right - bubble.getBoundingClientRect().left) / 2;
                    
                    $(bubble).css({
                        "left": bubbleLeft,
                        "top": bubbleTop
                    });
                }
                this._bubbleCollection.push(bubble);
            }
			}
			if(this._mapItems!=undefined)
			{
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                $(item).css({"display":"block"});
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
			    var cx = this._bubbleCollection[key] ?parseFloat(this._bubbleCollection[key].getAttribute("cx")) :0;
			    var cy =this._bubbleCollection[key] ? parseFloat(this._bubbleCollection[key].getAttribute("cy")):0;
			    var xpos = this._bubbleCollection[key] ? (cx - box.width / 2) : ((position.x + map._translatePoint.x) * map._scale) - (box.width / 2);
			    var ypos = this._bubbleCollection[key] ? (cy - box.height / 4) : ((position.y + map._translatePoint.y) * map._scale) - (box.height / 4);
                $(item).css({ "left": xpos, "top": ypos });               
            }
			}
            if(this._mapMarkers!=undefined){
            for (var key = 0; key < this._mapMarkers.length; key++) {
                var item = this._mapMarkers[key];

                var marker;
                if (this.markers.length > 0) {
                    marker = this.markers[key];
                }               
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(marker.latitude, marker.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforMarker(marker.latitude, marker.longitude);
                }            
                var xpos = position.x;               
                var ypos = position.y;            
                $(item).css({ "left": xpos, "top": ypos });                
            }
			}
			if(this._labelCollection!=undefined){
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "smartLabelStyle") {                    
                    item[0].className = "labelStyle";
                    $(item[0]).css("background-color", "transparent");
                    $(item[0]).css({ "pointer-events": "none", "position": "absolute" });
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos});
               
            }
			}
            if (this.labelSettings != null && this.labelSettings.showLabels) {
                this._validateSmartLabel(map);
            }
        },
       
        _validateSmartLabel: function (map) {
            this._smartLabels = [];
            var filledRects = [];
            if (this._labelCollection.length > 0) {
                for (var index = 0; index < this._mapShapes.length; index++) {
                    var shapemidPoint = this._labelBounds[index];
                    var midPoint = { x: (shapemidPoint.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.y + map._translatePoint.y) * map._scale };

                    var rightMinPoint = { x: (shapemidPoint.rightMin.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.rightMin.y + map._translatePoint.y) * map._scale };
                    var rightMaxPoint = { x: (shapemidPoint.rightMax.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.rightMax.y + map._translatePoint.y) * map._scale };
                    var leftMinPoint = { x: (shapemidPoint.leftMin.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.leftMin.y + map._translatePoint.y) * map._scale };
                    var leftMaxPoint = { x: (shapemidPoint.leftMax.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.leftMax.y + map._translatePoint.y) * map._scale };

                    var labelElement = this._labelCollection[index];
                    if (midPoint.x > 0 && midPoint.x < map._width && midPoint.y > 0 && midPoint.y < map._height) {
                        var shape = this._mapShapes[index].shape;
                        var bounds = shape.getBoundingClientRect();                        
                        var flag = false;
                        var labelSize = labelElement[0].getBoundingClientRect();
                        var labelHeight = labelSize.height;
                        var labelWidth = labelSize.width;
                        if (!_isSVG) {
                            bounds = { width: (bounds.right*map._scale) - (bounds.left*map._scale), height: (bounds.bottom *map._scale)- (bounds.top*map._scale) };
                            labelHeight = labelSize.bottom - labelSize.top;
                            labelWidth = labelSize.right - labelSize.left;
                        }

                        var rightIntersect = false, leftIntersect = false;
                        if ((labelWidth / 2 > rightMinPoint.x - midPoint.x || labelWidth / 2 > rightMaxPoint.x - midPoint.x)
                            && (labelHeight / 2 > midPoint.y - rightMinPoint.y || labelHeight / 2 > rightMaxPoint.y - midPoint.y)) {
                            rightIntersect = true;
                        }
                        else if ((labelWidth / 2 > midPoint.x - leftMinPoint.x || labelWidth / 2 > midPoint.x - leftMaxPoint.x)
                            && (labelHeight / 2 > midPoint.y - leftMinPoint.y || labelHeight / 2 > leftMaxPoint.y - midPoint.y)) {
                            leftIntersect = true;
                        }
                        
                        var leftPosition = 0;
                        var topPosition = 0;
                        var factor = 20;
                        var xArray = [];
                        var yArray = [];
                        if (labelHeight > bounds.height || labelWidth > bounds.width || (rightIntersect || leftIntersect)) {
                            
                            if (this.labelSettings.enableSmartLabel) {
                                if (this.labelSettings.smartLabelSize == ej.datavisualization.Map.LabelSize.Fixed) {
                                    labelHeight = 25;
                                    labelWidth = 15 * this.labelSettings.labelLength;
                                }
                                else {                                    
                                    labelWidth *= 1.3;
                                    labelHeight = 25;
                                }
                                while (!flag) {
                                    if (factor > 400) {
                                        flag = true;
                                    }
                                    xArray = pushToArray(factor);
                                    yArray = xArray;
                                    for (var xIndex = 0; xIndex < xArray.length; xIndex++) {
                                        for (var yIndex = 0; yIndex < xArray.length; yIndex++) {
                                            leftPosition = yArray[yIndex];
                                            topPosition = xArray[xIndex];                                           
                                            if (midPoint.x + leftPosition + labelWidth > map._width && midPoint.x < map._width) {
                                                leftPosition -=(midPoint.x + leftPosition + labelWidth)- map._width;
                                            }
                                            if (midPoint.x + leftPosition <0) {
                                                leftPosition =0;
                                            }
                                            if (midPoint.y + topPosition  <0) {
                                                topPosition=0;
                                            }
                                            if (midPoint.y + topPosition + labelHeight > map._height && midPoint.y < map._height) {
                                                topPosition -= (midPoint.y + topPosition + labelHeight) - map._height;
                                            }
                                            var rect = { left: midPoint.x + leftPosition, top: midPoint.y + topPosition, height: labelHeight, width: labelWidth };
                                            if (isEmpty(rect)) {
                                                var ele = map._getIntersectedElements(rect, this._mapShapes);
                                                if (ele != null && ele.length == 0) {
                                                    flag = true;                                                   
                                                    xIndex = xArray.length;
                                                    yIndex = xArray.length;
                                                    labelElement[0].className = 'smartLabelStyle';
                                                    $(labelElement).css({ "pointer-events": "stroke", "position": "absolute" });

                                                    if (this.labelSettings.smartLabelSize == ej.datavisualization.Map.LabelSize.Fixed) {
                                                        labelElement[0].innerHTML = labelElement[0].innerHTML.substring(0, this.labelSettings.labelLength);
                                                    }
                                                    labelElement.mouseenter({ Param1: this, Param2: this._mapShapes[index],map:map }, map._polyEnterFunction);
                                                    labelElement.mousemove({ Param1: this, Param2: this._mapShapes[index] }, map._polyMoveFunction);
                                                    $(labelElement).css({ left: midPoint.x + leftPosition, top: midPoint.y + topPosition, "background-color": _isSVG ? shape.getAttribute("fill") : shape.fillcolor.value});
                                                    filledRects.push(rect);
                                                    var labelObj = {};
                                                    labelObj["shape"] = shape;
                                                    labelObj["legend"] = labelElement;
                                                    this._smartLabels.push(labelObj);
                                                }
                                            }
                                        }
                                    }
                                    factor += 10;
                                }
                            } else {
                                var canExecute = true;
                                var minY = 0;
                                var maxY = 0;
                                var flag = true;

                                // Finding Minimum and Maximum Y position point calculation
                                for (var j = 0; j < shapemidPoint.points.length; j++) {
                                    var point = { x: (shapemidPoint.points[j].x + map._translatePoint.x) * map._scale, y: (shapemidPoint.points[j].y + map._translatePoint.y) * map._scale };
                                    if (flag) {
                                        minY = point.y;
                                        maxY = point.y;
                                        flag = false;
                                    }
                                    else {
                                        minY = Math.min(minY, point.y);
                                        maxY = Math.max(maxY, point.y);
                                    }
                                }

                                var heightfactor = Math.floor((maxY - minY) / labelHeight);
                                var shapePoints = [];

                                // Grouping points based on label height
                                for (var k = 0; k < heightfactor; k++) {
                                    var groupPoints = [];
                                    shapePoints.push(groupPoints);
                                }

                                for (var j = 0; j < shapemidPoint.points.length; j++) {
                                    var point = { x: (shapemidPoint.points[j].x + map._translatePoint.x) * map._scale, y: (shapemidPoint.points[j].y + map._translatePoint.y) * map._scale };
                                    var positionfactor = Math.floor((point.y - minY) / labelHeight);
                                    if (positionfactor > 0)
                                        positionfactor -= 1;
                                    var groupPoints = shapePoints[positionfactor];
                                    if (groupPoints == undefined)
                                        groupPoints = [];
                                    groupPoints.push({ x: point.x, y: point.y });
                                    shapePoints[positionfactor] = groupPoints;
                                }

                                // Finding minimum and maximum x point calculation 
                                for (var j = 0; j < shapePoints.length; j++) {
                                    var groupPoints = shapePoints[j];
                                    flag = true;
                                    var minX =0, maxX =0, top = 0;
                                    var leftPoints = [], rightPoints = [];
                                    
                                    for (var k = 0; k < groupPoints.length; k++) {
                                        if (flag) {
                                            minX = groupPoints[k].x;
                                            maxX = groupPoints[k].x;
                                            top = groupPoints[k].y;
                                            flag = false;
                                        }
                                        else {
                                            minX = Math.min(minX, groupPoints[k].x);
                                            maxX = Math.max(maxX, groupPoints[k].x);
                                            if (groupPoints[k].x == maxX)
                                                top = groupPoints[k].y;
                                        }
                                        if (groupPoints[k].x < midPoint.x)
                                            leftPoints.push(groupPoints[k].x);
                                        if (groupPoints[k].x > midPoint.x)
                                            rightPoints.push(groupPoints[k].x);
                                    }

                                    if (bounds.left < map._mapContainer[0].getBoundingClientRect().left + 1) {
                                        minX = map._mapContainer[0].getBoundingClientRect().left + 1 - bounds.left;
                                        leftPoints.push(minX);
                                    }

                                    //  Checking intersection of points with label.
                                    var intersect = false;
                                    for (var k = 0; k < leftPoints.length; k++) {
                                        if (maxX - leftPoints[k] < labelWidth + 1) {
                                            intersect = true;
                                            break;
                                        }
                                    }
                                    if (!intersect) {
                                        for (var k = 0; k < rightPoints.length; k++) {
                                            if (rightPoints[k] - minX < labelWidth + 1) {
                                                intersect = true;
                                                break;
                                            }
                                        }
                                    }
                                    if (maxX - minX > labelWidth + 1 && !intersect) {
                                        labelElement.css("left", bounds.left * map._scale + (minX - bounds.left * map._scale) + (maxX - minX) / 2 - labelWidth / 2 + "px");
                                        labelElement.css("top", top + "px");
                                        if (maxY - top < labelHeight)
                                            labelElement.css("top", top - labelHeight + "px");
                                        canExecute = false;
                                        break;
                                    }
                                }
                                if(canExecute)
                                    labelElement.css("display", "none");
                            }
                        }
                    } 
                }
            }

            function pushToArray(value) {
                var array = [];
                array.push(0);
                for (var j = 10; j <= value; j += 20) {
                    array.push(-j);
                    array.push(j);
                }

                return array;
            }

            function isEmpty(rect) {
                for (var i = 0; i < filledRects.length; i++) {
                    if (isIntersect(rect, filledRects[i]))
                        return false;
                }
                return true;
            }
           
            function isIntersect(rect1, rect2) {
                
                if (rect1.left - 2 >= (rect2.left + rect2.width) || rect1.left - 2 + rect1.width <= rect2.left - 2 ||
                    rect1.top - 2 >= rect2.top + rect2.height || rect1.top+rect1.height <= rect2.top-2) {
                    return false;
                }
                return true;
            }


        },
              
        _fillColors: function (value, colorMapping, shape) {

            if (colorMapping.rangeColorMapping != null) {
               
                this._fillRangeColors(value, colorMapping.rangeColorMapping, shape);
            }
            else if (colorMapping.equalColorMapping != null) {
                this._fillEqualColors(value, colorMapping, shape);
            }
        },

        _fillEqualColors: function (value, colorMapping, shape) {
            $.each(colorMapping.equalColorMapping, function (index, gValue) {
                if (gValue.value == value) {
                    if (_isSVG) {
                        shape.setAttribute("fill", gValue.color);
                    }
                    else {
                        shape.fillcolor = gValue.color;
                    }
					 shape.highlightcolor = gValue.highlightcolor;
                }
            });
        },
       
        _updateLegendRange: function (value, shapelayer, shape) {
            var colormapping = shapelayer.shapeSettings.colorMappings.rangeColorMapping;
            for (var index = 0; index < colormapping.length; index++) {
                var gradientCollection = null;
                var mapping = colormapping[index];
                if (shapelayer.shapeSettings.enableGradient) {
                    gradientCollection = this._generateGradientCollection(mapping.gradientColors);
                }
                if (value >= mapping.from && value <= mapping.to) {
                    var minValue = index;
                    if (index != 0)
                        minValue = index * 10;
                    var minRange = mapping.from;
                    var maxRange = mapping.from + ((mapping.to - mapping.from) / 10);
                    for (var i = minValue; i < minValue + 10; i++) {
                        if (value >= minRange && value <= maxRange) {
                            var obj = {};
                            var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                            if (shapelayer._legendRects[i] != undefined)
                                obj = shapelayer._legendRects[i];
                            if (shapelayer.shapeSettings.enableGradient) {
                                if (index != 0) {
                                    obj["color"] = gradientCollection[i - index * 10];
                                } else {
                                    obj["color"] = gradientCollection[i];
                                }
                            } else
                                obj["color"] = mapping.color;
                                $(shape).css({ "opacity": shapeOpacity });
                            return obj;
                        }
                        minRange = maxRange;
                        maxRange = maxRange + ((mapping.to - mapping.from) / 10);
                    }
                }
            }
        },

        _fillRangeColors: function (value, colormapping, shape) {
            
            for (var index = 0; index < colormapping.length; index++) {
                var mapping = colormapping[index];
                if (value >= mapping.from && value <= mapping.to) {
                    var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                    if (_isSVG) {
                        shape.setAttribute("fill", mapping.color);
                    }
                    else {
                        shape.fillcolor = mapping.color;
                    }
					shape.highlightcolor = mapping.highlightcolor;
                    $(shape).css({ "opacity": shapeOpacity });
                }
            }
        },
       
        _getColorRatio: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var colorCode = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return colorCode;
        }

    };

    $.fn.pinchZoom = function (rootgroup, map) {

        var stateOrigin,
                    stateTf,
                    root = this[0],
                    startTouches = [],

           getTouchEventPoint = function (evt) {

               var p = root.createSVGPoint(),
                   targetTouches = evt.targetTouches,
                   offsetX,
                   offsetY;

               if (targetTouches.length) {
                   var touch = targetTouches[0];
                   offsetX = touch.pageX;
                   offsetY = touch.pageY;
               }

               p.x = offsetX;
               p.y = offsetY;

               return p;
           },

        handleTouchStart = function (evt) {
            var g = rootgroup;
            stateTf = g.getCTM().inverse();
            stateOrigin = getTouchEventPoint(evt).matrixTransform(stateTf);
            var touches = evt.touches;
            if (touches.length >= 2) {
                evt.preventDefault();
                for (var i = 0; i < touches.length; i++) {
                    var touch = touches[i]
                      , found = false;
                    for (var j = 0; j < startTouches.length; j++) {
                        var startTouch = startTouches[j];
                        if (touch.identifier === startTouch.identifier) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        var touchCopy = $.extend({}, touch);
                        startTouches.push(touchCopy);
                    }
                }
            }
        },

        getDistance = function (touch1, touch2) {
            var x = touch2.pageX - touch1.pageX,
                y = touch2.pageY - touch1.pageY;
            return Math.sqrt((x * x) + (y * y));
        },

        setCTM = function (element, matrix) {
		    var isZoomIn=null;
            var s = "matrix(" + matrix.a + "," + matrix.b + "," + matrix.c + "," + matrix.d + "," + matrix.e + "," + matrix.f + ")";
            if (matrix.a > map._baseScale && map.model.zoomSettings.enableZoom && map.enablePan()) {
			if(matrix.a > map._scale)
			{
			     isZoomIn=true;
			}
			else if(matrix.a < map._scale)
			{
			isZoomIn=false;
			}
			    map._scale = matrix.a;
			    map._translatePoint.x = matrix.e / matrix.a;
			    map._translatePoint.y = matrix.f / matrix.a;
			    map._applyTransform(map._scale, map._translatePoint);
			    map.model.zoomSettings.level = map._scale - map._baseScale + 1;
			    map._zoomLevel(map._scale - map._baseScale + 1);
	    		if (isZoomIn != null && isZoomIn) {
	    		    map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map._zoomLevel() });
				    map._updateSliderValue(false);
					map._resizeShape();
				}
				else if(isZoomIn!=null)
				{
				    map._trigger("zoomedOut", { originalEvent: null, zoomLevel: map._zoomLevel() });
				    map._updateSliderValue(false);
					map._resizeShape();
				}
	    		map._isDragging = true;
                map._refrshLayers();
            }

        },

        handleTouchEnd = function (evt) {
            var changedTouches = evt.changedTouches;
            for (var i = 0; i < changedTouches.length; i++) {
                var changedTouch = changedTouches[i];
                for (var j = 0; j < startTouches.length; j++) {
                    var startTouch = startTouches[j];
                    if (startTouch.identifier === changedTouch.identifier) {
                        var idx = startTouches.indexOf(startTouch);
                        startTouches.splice(idx, 1);
                    }
                }
            }
            evt.preventDefault();
        },

        updateTouch = function (dstTouch, srcTouch) {
            dstTouch.pageX = srcTouch.pageX;
            dstTouch.pageY = srcTouch.pageY;
        },

        handleTouchMove = function (evt) {
            var g = rootgroup,
                touches = evt.touches,
                z,
                p,
                k;

            if (touches.length >= 2) {
                evt.preventDefault();
                var touch0 = touches[0]
                  , touch1 = touches[1]
                  , startTouch0 = startTouches[0]
                  , startTouch1 = startTouches[1];
                z = getDistance(touch0, touch1) / getDistance(startTouch0, startTouch1);
                p = root.createSVGPoint();
                p.x = (touch0.pageX + touch1.pageX) / 2;
                p.y = (touch0.pageY + touch1.pageY) / 2;
                p = p.matrixTransform(g.getCTM().inverse());
                k = root.createSVGMatrix().translate(p.x, p.y).scale(z).translate(-p.x, -p.y);
                setCTM(g, g.getCTM().multiply(k));
                if (typeof stateTf === "undefined") {
                    stateTf = g.getCTM().inverse();
                }
                stateTf = stateTf.multiply(k.inverse());
                updateTouch(startTouch0, touch0);
                updateTouch(startTouch1, touch1);
            } else if (!startTouches.length) {
                p = getTouchEventPoint(evt).matrixTransform(stateTf);
                setCTM(g, stateTf.inverse().translate(p.x - stateOrigin.x, p.y - stateOrigin.y));
            }
        };
        this[0].addEventListener('touchstart', handleTouchStart, false);
        this[0].addEventListener('touchend', handleTouchEnd, false);
        this[0].addEventListener('touchmove', handleTouchMove, false);


    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };
    
	ej.datavisualization.Map.Locale = ej.datavisualization.Map.Locale || {};
    ej.datavisualization.Map.Locale['default'] = ej.datavisualization.Map.Locale['en-US'] = {
        zoomIn: "Zoom In",
        zoomOut: "Zoom Out",
        panTop: "Pan Top",
        panBottom: "Pan Bottom",
        panLeft: "Pan Left",
		panRight: "Pan Right",
		home: "Home"
    };
	
    ej.datavisualization.Map.LayerType = {
        Geometry: "geometry",
        OSM: "osm",
        Bing: "bing"
    };
   
    ej.datavisualization.Map.LegendIcons = {       
        Rectangle: "rectangle",      
        Circle: "circle"
    };
   
    ej.datavisualization.Map.LabelSize = {       
        Fixed: "fixed",        
        Default: "default"
    };

    ej.datavisualization.Map.LegendMode = {       
        Default: "default",       
        Interactive: "interactive"		
    };

    ej.datavisualization.Map.BingMapType = {    
        Aerial: "aerial",    
        AerialWithLabel: "aerialwithlabel",    
        Road: "road",
    };

    ej.datavisualization.Map.GeometryType = {
        Normal: "normal",    
        Geographic: "geographic"
        
    };

    ej.datavisualization.Map.SelectionMode = {       
        Default: "default",       
        Multiple: "multiple"
    };

    ej.datavisualization.Map.ColorPalette = {        
        Palette1: "palette1", 
        Palette2: "palette2", 
        Palette3: "palette3", 
        CustomPalette: "custompalette"
    };
   
    ej.datavisualization.Map.LegendType = {
        Layers: "layers",
        Bubbles: "bubbles"
    }
 
    ej.datavisualization.Map.Position = {
        None: "none",
        TopLeft: "topleft",
        TopCenter: "topcenter",
        TopRight: "topright",
        CenterLeft: "centerleft",
        Center: "center",
        CenterRight: "centerright",
        BottomLeft: "bottomleft",
        BottomCenter: "bottomcenter",
        BottomRight: "bottomright"
    }
    
    ej.datavisualization.Map.DockPosition = {    
        Top: "top",	
        Bottom: "bottom",    
        Right: "right",    
        Left: "left"
    };
	 ej.datavisualization.Map.Alignment = {
        
        Near: "near",       
        Center: "center",       
        Far: "far"
    };
    
    ej.datavisualization.Map.LabelOrientation = {	
        Horizontal: "horizontal",	
        Vertical: "vertical"
    };
    
})(jQuery, Syncfusion);
;
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ejSparkline;
(function (ejSparkline) {
    (function (Type) {
        Type[Type["Line"] = "line"] = "Line";
        Type[Type["Column"] = "column"] = "Column";
        Type[Type["Area"] = "area"] = "Area";
        Type[Type["WinLoss"] = "winloss"] = "WinLoss";
        Type[Type["Pie"] = 'pie'] = "Pie";
    })(ejSparkline.Type || (ejSparkline.Type = {}));
    var Type = ejSparkline.Type;
    (function (Themes) {
        Themes[Themes["flatlight"] = "flatlight"] = "flatlight";
        Themes[Themes["azurelight"] = "azurelight"] = "azurelight";
        Themes[Themes["limelight"] = "limelight"] = "limelight";
        Themes[Themes["saffronlight"] = "saffronlight"] = "saffronlight";
        Themes[Themes["gradientlight"] = "gradientlight"] = "gradientlight";
        Themes[Themes["flatdark"] = "flatdark"] = "flatdark";
        Themes[Themes["azuredark"] = "azuredark"] = "azuredark";
        Themes[Themes["limedark"] = "limedark"] = "limedark";
        Themes[Themes["saffrondark"] = "saffrondark"] = "saffrondark";
        Themes[Themes["gradientdark"] = "gradientdark"] = "gradientdark";
    })(ejSparkline.Themes || (ejSparkline.Themes = {}));
    var Themes = ejSparkline.Themes;
    var Sparkline = (function (_super) {
        __extends(Sparkline, _super);
        function Sparkline(id, options) {
            _super.call(this);
            this.defaults = {
                locale: null,
                enableGroupSeparator: false,
                enableCanvasRendering: false,
                padding: 8,
                palette: ["#8A2BE2", "#ff1a75", "#99cc00", "#4d4dff", "#660066", "#FFA500", "#FFD700", "#FF00FF", "#808000", "#990000"],
                isResponsive: true,
                dataSource: null,
                xName: "",
                yName: "",
                type: Type.Line,
                width: 1,
                stroke: null,
                opacity: 1,
                fill: "#33ccff",
                border: {
                    color: "transparent",
                    width: 1,
                },
                rangeBandSettings: {
                    startRange: null,
                    endRange: null,
                    opacity: 0.4,
                    color: "transparent"
                },
                highPointColor: null,
                lowPointColor: null,
                negativePointColor: null,
                startPointColor: null,
                endPointColor: null,
                tooltip: {
                    visible: false,
                    template: null,
                    fill: "white",
                    border: {
                        width: 1,
                        color: null,
                    },
                    font: {
                        fontFamily: "Segoe UI",
                        fontStyle: "Normal",
                        fontWeight: "Regular",
                        color: "#111111",
                        opacity: 1,
                        size: "8px"
                    }
                },
                markerSettings: {
                    visible: false,
                    fill: null,
                    width: 2,
                    opacity: 1,
                    border: {
                        color: "white",
                        width: 1,
                    }
                },
                background: "transparent",
                size: {
                    height: "",
                    width: ""
                },
                axisLineSettings: {
                    visible: false,
                    color: "#111111",
                    width: 1,
                    dashArray: "",
                },
                theme: Themes.flatlight,
                load: null,
                loaded: null,
                sparklineMouseMove: null,
                sparklineMouseLeave: null,
                seriesRendering: null,
                pointRegionMouseMove: null,
                pointRegionMouseClick: null,
                tooltipInitialize: null,
            };
            this.dataTypes = {
                dataSource: "data",
                palette: "array",
                type: "enum",
                theme: "enum",
            };
            this.model = null;
            this.svgLink = "http://www.w3.org/2000/svg";
            this._id = null;
            this.negativePointIndexes = [];
            this.validTags = ['div'];
            this._id = id;
            if (!!options)
                this.model = ejSparkline.compareExtend({}, options, this.defaults);
        }
        Sparkline.prototype.isTouch = function (evt) {
            var event = evt.originalEvent ? evt.originalEvent : evt;
            if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        };
        Sparkline.prototype.browserInfo = function () {
            var browser = {}, clientInfo = [], browserClients = {
                webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
                opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
            };
            for (var client in browserClients) {
                if (browserClients.hasOwnProperty(client)) {
                    clientInfo = navigator.userAgent.match(browserClients[client]);
                    if (clientInfo) {
                        browser.name = clientInfo[1].toLowerCase();
                        browser.version = clientInfo[2];
                        if (!!navigator.userAgent.match(/Trident\/7\./)) {
                            browser.name = "msie";
                        }
                        break;
                    }
                }
            }
            browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
            browser.pointerEnabled = window.navigator.pointerEnabled;
            return browser;
        };
        Sparkline.prototype.fadeOut = function (tooltip) {
            var op = 1;
            var timer = setInterval(function () {
                if (op <= 0.1) {
                    clearInterval(timer);
                    tooltip.parentNode ? tooltip.parentNode.removeChild(tooltip) : tooltip;
                }
                tooltip.style.opacity = op;
                tooltip.style.filter = 'alpha(opacity=' + op * 100 + ")";
                op -= op * 0.1;
            }, 50);
        };
        Sparkline.prototype._setModel = function (options) {
            for (var prop in options) {
                var $content;
                switch (prop) {
                    default:
                        ejSparkline.deepExtend(true, this.model, {}, options[prop]);
                }
            }
            this.redraw();
        };
        Sparkline.prototype.unBindEvents = function () {
            var sparklineEle = document.getElementById(this.rootId), insideEvents = "", browserInfo = this.browserInfo(), isPointer = browserInfo.isMSPointerEnabled, isIE11Pointer = browserInfo.pointerEnabled, touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup", touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove";
            insideEvents = touchStopEvent + " " + touchMoveEvent;
            insideEvents = insideEvents.split(" ");
            for (var event in insideEvents) {
                sparklineEle.removeEventListener(insideEvents[event], this.sparkMouseMove);
            }
            sparklineEle.removeEventListener("mouseout", this.sparkMouseLeave);
        };
        Sparkline.prototype.bindEvents = function (ele) {
            this.sparkMouseMove = this.sparkMouseMove.bind(this);
            this.sparkMouseLeave = this.sparkMouseLeave.bind(this);
            var insideEvents = "", browserInfo = this.browserInfo(), isPointer = browserInfo.isMSPointerEnabled, isIE11Pointer = browserInfo.pointerEnabled, touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup", touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove";
            insideEvents = touchStopEvent + " " + touchMoveEvent;
            insideEvents = insideEvents.split(" ");
            for (var event in insideEvents) {
                ele.addEventListener(insideEvents[event], this.sparkMouseMove);
            }
            ele.addEventListener("mouseout", this.sparkMouseLeave);
        };
        Sparkline.prototype.sparkMouseMove = function (evt) {
            var isTouch = this.isTouch(evt);
            if (!(isTouch && (evt.type.toString().toLowerCase().indexOf("move") > -1))) {
                var model = this.model, currentObj = this, locale = model.locale, localizedText = locale && model.enableGroupSeparator, tooltipXValue, tooltipYValue, pointsLocations = this.visiblePoints, markerId = this.container.id + "_markerExplode", pointIndex, text, labelText, markerOptions, tooltipOptions, textOptions, textPad = model.enableCanvasRendering ? 0 : 3, tooltipShape = '', tooltipHeight, tooltipWidth, eventArgs, tooltip, markerFill, containerHeight = parseInt(model.size.height), containerWidth = parseInt(model.size.width), font = model.tooltip.font, xFormat = "#point.x#", yFormat = "#point.y#", tooltipId = this.container.id + "_tooltip", tracker = document.getElementById(tooltipId), trackerPosX, tooltipBorder = model.tooltip.border.color, trackballOptions, rootele = document.getElementById(this.container.id), height = rootele.clientHeight, parent = rootele.parentNode, offset = rootele.getClientRects()[0], trackerTop = offset.top, trackerLeft = offset.left, mouseX = evt.clientX || (evt.changedTouches ? evt.changedTouches[0].clientX : evt.touches ? evt.touches[0].clientX : 0), mouseY = evt.clientY || (evt.changedTouches ? evt.changedTouches[0].clientY : evt.touches ? evt.touches[0].clientY : 0), trackerPositions = pointsLocations.map(function (obj) { return obj['location']['X'] + offset.left; }), tooltipPos = pointsLocations.map(function (obj) { return obj['location']['markerPos'] + offset.top; }), temp = Infinity, measure = this.measureText, pad, spMarker = this.model.markerSettings, mousePos, canvasTracker, canvasContext, canvasOptions, X, Y, canTracker = document.getElementById(this.container.id + "_canvasTracker");
                for (var i = 0, diff, len = trackerPositions.length; i < len; i++) {
                    diff = Math.abs(mouseX - trackerPositions[i]);
                    if (temp > diff) {
                        temp = diff;
                        mousePos = trackerPositions[i];
                        this.pointIndex = i;
                    }
                }
                if ((mouseX > trackerLeft && mouseX < (trackerLeft + offset.width)) && (mouseY > trackerTop && mouseY < (trackerTop + offset.height))) {
                    temp = pointsLocations[this.pointIndex]['location'];
                    var trackTooltipModify = function (text) {
                        pad = text.length / 2;
                        var size = measure(text, font);
                        tooltipHeight = size.height + 4;
                        tooltipWidth = size.width + pad;
                        X = temp['X'];
                        Y = temp['markerPos'];
                        if ((Y - tooltipHeight / 2) < 0)
                            Y += (tooltipHeight / 2);
                        else if ((Y + tooltipHeight / 2) > containerHeight)
                            Y -= (tooltipHeight / 2);
                        if ((X + tooltipWidth + (tooltipWidth / 10)) > (containerWidth - model.padding)) {
                            X = (temp['X'] - Number(model.markerSettings.width) - Number(model.markerSettings.border.width) - 4);
                            tooltipShape = 'M ' + (X) + " " + temp['markerPos'] + " L " + (X - tooltipWidth / 10) + " " + (Y - 4) + " L " + (X - tooltipWidth / 10) + " " + (Y - (tooltipHeight / 2)) + " L " + (X - tooltipWidth - tooltipWidth / 10) + " " + (Y - (tooltipHeight / 2)) + " L " + (X - tooltipWidth - tooltipWidth / 10) + " " + (Y + (tooltipHeight / 2)) + " L " + (X - tooltipWidth / 10) + " " + (Y + (tooltipHeight / 2)) + " L " + (X - tooltipWidth / 10) + " " + (Y + 4) + " Z";
                        }
                        else {
                            X = (temp['X'] + Number(model.markerSettings.width) + Number(model.markerSettings.border.width) + 4);
                            tooltipShape = 'M ' + (X) + " " + temp['markerPos'] + " L " + (X + tooltipWidth / 10) + " " + (Y - 4) + " L " + (X + tooltipWidth / 10) + " " + (Y - (tooltipHeight / 2)) + " L " + (X + tooltipWidth + tooltipWidth / 10) + " " + (Y - (tooltipHeight / 2)) + " L " + (X + tooltipWidth + tooltipWidth / 10) + " " + (Y + (tooltipHeight / 2)) + " L " + (X + tooltipWidth / 10) + " " + (Y + (tooltipHeight / 2)) + " L " + (X + tooltipWidth / 10) + " " + (Y + 4) + " Z";
                        }
                    };
                    if (model.tooltip.template == null || model.tooltip.template == "") {
                        tooltipXValue = localizedText && temp['Xval'] ? temp['Xval'].toLocaleString(locale) : temp['Xval'];
                        tooltipYValue = localizedText && temp['Yval'] ? temp['Yval'].toLocaleString(locale) : temp['Yval'];
                        labelText = " X : " + tooltipXValue + " Y : " + tooltipYValue + " ";
                        trackTooltipModify(labelText);
                    }
                    pointIndex = this.pointIndex;
                    if (this.prevMousePos == undefined) {
                        this.prevMousePos = mousePos;
                    }
                    var checkPointFill = function (index) {
                        var mFill, highIndex = currentObj.highPointIndex, lowIndex = currentObj.lowPointIndex, negatives = currentObj.negativePointIndexes, startIndex = currentObj.startPointIndex, endIndex = currentObj.endPointIndex;
                        if (index == highIndex)
                            mFill = model.highPointColor;
                        else if (index == lowIndex)
                            mFill = model.lowPointColor;
                        else if (index == startIndex && (model.startPointColor != null))
                            mFill = model.startPointColor;
                        else if (index == endIndex && (model.endPointColor != null))
                            mFill = model.endPointColor;
                        else if (negatives.indexOf(index) >= 0 && model.negativePointColor != null)
                            mFill = model.negativePointColor;
                        else
                            mFill = spMarker.fill ? spMarker.fill : model.fill;
                        return mFill;
                    };
                    if (tracker || canTracker) {
                        if (this.prevMousePos != mousePos) {
                            eventArgs = {
                                data: {
                                    "pointIndex": pointIndex,
                                    "currentText": labelText,
                                    "location": temp
                                }
                            };
                            if (this.model.tooltipInitialize != null)
                                this._trigger("tooltipInitialize", eventArgs);
                            trackerPosX = (temp['X'] - (model.width / 2));
                            if (labelText != eventArgs.data.currentText) {
                                labelText = eventArgs.data.currentText;
                                trackTooltipModify(labelText);
                            }
                            this.prevMousePos = mousePos;
                            markerFill = checkPointFill(pointIndex);
                            markerOptions = {
                                'id': markerId,
                                'cx': temp['X'],
                                'cy': temp['markerPos'],
                                'r': Number(spMarker.width),
                                'fill': spMarker.border.color,
                                'stroke': markerFill,
                                'stroke-width': Number(spMarker.border.width),
                            };
                            if (model.tooltip.template == null || model.tooltip.template == "") {
                                tooltipOptions = {
                                    'id': tooltipId,
                                    'fill': model.tooltip.fill,
                                    'stroke': (tooltipBorder != null) ? tooltipBorder : markerFill,
                                    'stroke-width': model.tooltip.border.width,
                                    'd': tooltipShape
                                };
                                textOptions = {
                                    'x': (temp['X'] + tooltipWidth + (tooltipWidth / 10)) > (containerWidth - model.padding) ? (X + textPad - (tooltipWidth) - (tooltipWidth / 10)) : X + textPad + (tooltipWidth / 10),
                                    'y': Y + tooltipHeight / 4,
                                    'fill': font.color,
                                    'font-size': font.size,
                                    'font-family': font.fontFamily,
                                    'font-weight': font.fontWeight,
                                    'font-style': font.fontStyle,
                                };
                            }
                            else {
                                var ele = document.getElementById(model.tooltip.template), html = ele.innerHTML, html = html.replace(xFormat, "&nbsp" + temp['Xval'] + "&nbsp"), html = html.replace(yFormat, "&nbsp" + temp['Yval'] + "&nbsp"), text1 = ele.innerText, gap = Number(model.markerSettings.width), text1 = text1.replace(xFormat, "&nbsp" + temp['Xval'] + "&nbsp"), text1 = text1.replace(yFormat, "&nbsp" + temp['Yval'] + "&nbsp"), template = measure(text1, font), toolX = temp['X'] + Number(spMarker.width) + Number(spMarker.border.width) + gap;
                                Y = (trackerTop + temp['markerPos'] - template['height'] / 2);
                                X = (trackerLeft + toolX);
                                if ((temp['markerPos'] - template['height'] / 2) < model.padding)
                                    Y += (template['height'] / 2);
                                else if ((temp['markerPos'] + template['height'] / 2) > (containerHeight - model.padding))
                                    Y -= (template['height'] / 2);
                                if ((toolX + template['width']) > (containerWidth - model.padding))
                                    X = (X - template['width']) - ((Number(spMarker.width) + Number(spMarker.border.width) + gap) * 2);
                                tooltipOptions = {
                                    'background': model.tooltip.fill,
                                    'border': model.tooltip.border.width + 'px solid ' + ((tooltipBorder != null) ? tooltipBorder : markerFill),
                                    'left': X + 'px',
                                    'top': Y + 'px',
                                };
                                tooltip = document.getElementById(tooltipId),
                                    tooltip.innerHTML = html;
                                this.setStyles(tooltipOptions, tooltip);
                            }
                            if (!model.enableCanvasRendering) {
                                var groupTooltip = document.getElementById(tooltipId + "_g");
                                this.drawCircle(markerOptions, groupTooltip);
                                if (model.tooltip.template == null) {
                                    this.drawPath(tooltipOptions, groupTooltip);
                                    text = rootele.getElementsByTagName('text')[0];
                                    text.textContent = labelText;
                                    this.setAttributes(textOptions, text);
                                    groupTooltip.appendChild(text);
                                }
                            }
                            else {
                                canvasTracker = document.getElementById(this.container.id + "_canvasTracker");
                                canvasContext = canvasTracker.getContext('2d');
                                canvasContext.clearRect(0, 0, containerWidth, containerHeight);
                                this.canvasDrawCircle(markerOptions, canvasContext);
                                if (model.tooltip.template == null || model.tooltip.template == "") {
                                    this.canvasDrawPath(tooltipOptions, canvasContext);
                                    textOptions.font = font.size + " " + font.fontFamily;
                                    this.canvasDrawText(textOptions, labelText, canvasContext);
                                }
                            }
                        }
                    }
                    else {
                        eventArgs = {
                            data: {
                                "pointIndex": pointIndex,
                                "currentText": labelText,
                                "location": temp
                            }
                        };
                        if (this.model.tooltipInitialize != null)
                            this._trigger("tooltipInitialize", eventArgs);
                        trackerPosX = (temp['X'] - (model.width / 2));
                        if (labelText != eventArgs.data.currentText) {
                            labelText = eventArgs.data.currentText;
                            trackTooltipModify(labelText);
                        }
                        var tooltipElements = document.getElementsByClassName("ej-sparkline-tooltip");
                        if (tooltipElements[0]) {
                            tooltipElements[0].parentNode.removeChild(tooltipElements[0]);
                        }
                        markerFill = checkPointFill(pointIndex);
                        markerOptions = {
                            'id': markerId,
                            'cx': temp['X'],
                            'cy': temp['markerPos'],
                            'r': Number(spMarker.width),
                            'fill': spMarker.border.color,
                            'stroke': markerFill,
                            'stroke-width': Number(spMarker.border.width),
                        };
                        if (model.tooltip.template == null || model.tooltip.template == "") {
                            tooltipOptions = {
                                'id': tooltipId,
                                'fill': model.tooltip.fill,
                                'stroke': (tooltipBorder != null) ? tooltipBorder : markerFill,
                                'stroke-width': model.tooltip.border.width,
                                'd': tooltipShape
                            };
                            textOptions = {
                                'x': (temp['X'] + tooltipWidth + (tooltipWidth / 10)) > (containerWidth - model.padding) ? (X + textPad - (tooltipWidth) - (tooltipWidth / 10)) : X + textPad + (tooltipWidth / 10),
                                'y': Y + tooltipHeight / 4,
                                'fill': font.color,
                                'opacity': font.opacity,
                                'font-size': font.size,
                                'font-family': font.fontFamily,
                                'font-weight': font.fontWeight,
                                'font-style': font.fontStyle,
                            };
                        }
                        else {
                            var ele = document.getElementById(model.tooltip.template), html = ele.innerHTML, html = html.replace(xFormat, "&nbsp" + temp['Xval'] + "&nbsp"), html = html.replace(yFormat, "&nbsp" + temp['Yval'] + "&nbsp"), text1 = ele.innerText, gap = Number(model.markerSettings.width), text1 = text1.replace(xFormat, "&nbsp" + temp['Xval'] + "&nbsp"), text1 = text1.replace(yFormat, "&nbsp" + temp['Yval'] + "&nbsp"), template = measure(text1, font);
                            var toolX = temp['X'] + Number(spMarker.width) + Number(spMarker.border.width) + gap;
                            Y = (trackerTop + temp['markerPos'] - template['height'] / 2);
                            X = (trackerLeft + toolX);
                            if ((temp['markerPos'] - template['height'] / 2) < model.padding)
                                Y += (template['height'] / 2);
                            else if ((temp['markerPos'] + template['height'] / 2) > (containerHeight - model.padding))
                                Y -= (template['height'] / 2);
                            if ((toolX + template['width']) > (containerWidth - model.padding))
                                X = (X - template['width']) - ((Number(spMarker.width) + Number(spMarker.border.width) + gap) * 2);
                            tooltipOptions = {
                                'background': model.tooltip.fill,
                                'border': model.tooltip.border.width + 'px solid ' + ((tooltipBorder != null) ? tooltipBorder : markerFill),
                                'left': X + 'px',
                                'top': Y + 'px',
                                'position': 'fixed',
                                'height': 'auto',
                                'width': 'auto',
                                'display': 'block',
                                'opacity': font.opacity,
                                'font': font.size + " " + font.fontFamily,
                            };
                            tooltip = document.createElement('div');
                            this.setAttributes({ 'id': tooltipId, 'class': 'ej-sparkline-tooltip' }, tooltip);
                            this.setStyles(tooltipOptions, tooltip);
                            tooltip.innerHTML = html;
                            parent.appendChild(tooltip);
                        }
                        if (!model.enableCanvasRendering) {
                            var tooltipGroup = document.createElementNS(this.svgLink, "g");
                            this.setAttributes({ "id": tooltipId + "_g", 'class': 'ej-sparkline-tooltip' }, tooltipGroup);
                            this.container.appendChild(tooltipGroup);
                            this.drawCircle(markerOptions, tooltipGroup);
                            if (model.tooltip.template == null) {
                                this.drawPath(tooltipOptions, tooltipGroup);
                                text = this.createText(textOptions, labelText);
                                tooltipGroup.appendChild(text);
                            }
                        }
                        else {
                            canvasTracker = document.createElement('canvas');
                            canvasOptions =
                                {
                                    'id': this.container.id + "_canvasTracker",
                                    'fill': 'transparent',
                                    'class': 'ej-sparkline-tooltip',
                                    'height': model.size.height,
                                    'width': model.size.width,
                                };
                            this.setAttributes(canvasOptions, canvasTracker);
                            this.setStyles({ 'left': (trackerLeft) + 'px', 'top': (trackerTop) + 'px', 'position': 'fixed' }, canvasTracker);
                            canvasContext = canvasTracker.getContext('2d');
                            this.canvasDrawCircle(markerOptions, canvasContext);
                            if (model.tooltip.template == null || model.tooltip.template == "") {
                                this.canvasDrawPath(tooltipOptions, canvasContext);
                                textOptions.font = font.size + " " + font.fontFamily;
                                this.canvasDrawText(textOptions, labelText, canvasContext);
                            }
                            this.container.parentNode.appendChild(canvasTracker);
                        }
                    }
                }
                if (isTouch) {
                    var eleId = (model.tooltip.template == null || model.tooltip.template == "") ? ((model.enableCanvasRendering) ? (this.container.id + "_canvasTracker") : (tooltipId + "_g")) : (tooltipId);
                    var touchtooltip = document.getElementById(eleId);
                    var sparklineObj = this;
                    setTimeout(function () {
                        sparklineObj.fadeOut(touchtooltip);
                    }, 1000);
                }
            }
        };
        Sparkline.prototype.sparkMouseLeave = function (evt) {
            var mouseX = evt.clientX || (evt.touches[0] ? evt.touches[0].clientX : evt.changedTouches[0].clientX), mouseY = evt.clientY || (evt.touches[0] ? evt.touches[0].clientY : evt.changedTouches[0].clientY), rootele = this.container, height = rootele.clientHeight, offset = rootele.getClientRects()[0], trackerTop = offset.top, trackerLeft = offset.left, model = this.model, tooltipId = this.container.id + "_tooltip";
            if ((mouseX < trackerLeft || mouseX > (trackerLeft + offset.width)) || (mouseY < trackerTop || mouseY > (trackerTop + offset.height)) || this.touchEnd) {
                if (model.tooltip.template != null && model.tooltip.template != "") {
                    var tooltemp = document.getElementById(tooltipId);
                    tooltemp ? tooltemp.parentNode.removeChild(tooltemp) : tooltemp;
                }
                if (!this.model.enableCanvasRendering) {
                    var trackerId = tooltipId + "_g", tracker = document.getElementById(trackerId);
                    tracker ? this.container.removeChild(tracker) : tracker;
                }
                else {
                    var canEle = document.getElementById(this.container.id + '_canvasTracker');
                    canEle ? canEle.parentNode.removeChild(canEle) : canEle;
                }
            }
        };
        Sparkline.prototype.bindPieEvents = function (ele) {
            this.pieTooltip = this.pieTooltip.bind(this);
            this.pieTooltipHide = this.pieTooltipHide.bind(this);
            var insideEvents = "", browserInfo = this.browserInfo(), isPointer = browserInfo.isMSPointerEnabled, isIE11Pointer = browserInfo.pointerEnabled, touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup", touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove";
            insideEvents = touchStopEvent + " " + touchMoveEvent;
            insideEvents = insideEvents.split(" ");
            for (var event in insideEvents) {
                ele.addEventListener(insideEvents[event], this.pieTooltip);
            }
            ele.addEventListener("mouseout", this.pieTooltipHide);
            if (this.model.pointRegionMouseClick != null) {
                ele.addEventListener("click", this.pieTooltip);
                ele.addEventListener("touchstart", this.pieTooltip);
            }
        };
        Sparkline.prototype.unbindPieEvents = function () {
            var insideEvents = "", browserInfo = this.browserInfo(), isPointer = browserInfo.isMSPointerEnabled, isIE11Pointer = browserInfo.pointerEnabled, touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup", touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove", sparklineEle = document.getElementById(this.rootId);
            insideEvents = touchStopEvent + " " + touchMoveEvent;
            insideEvents = insideEvents.split(" ");
            for (var event in insideEvents) {
                sparklineEle.removeEventListener(insideEvents[event], this.pieTooltip);
            }
            sparklineEle.removeEventListener("mouseout", this.pieTooltipHide);
        };
        Sparkline.prototype.pieTooltip = function (evt) {
            var isTouch = this.isTouch(evt);
            if (isTouch)
                evt.stopImmediatePropagation();
            if (!(isTouch && (evt.type.toString().toLowerCase().indexOf("move") > -1))) {
                if (this.model.sparklineMouseMove != null)
                    this._trigger("sparklineMouseMove");
                var model = this.model, mouseX = evt.clientX || (evt.changedTouches ? evt.changedTouches[0].clientX : evt.touches ? evt.touches[0].clientX : 0), containerHeight = parseInt(model.size.height), containerWidth = parseInt(model.size.width), font = model.tooltip.font, mouseY = evt.clientY || (evt.changedTouches ? evt.changedTouches[0].clientY : evt.touches ? evt.touches[0].clientY : 0), rootele = document.getElementById(this.container.id), parent = rootele.parentNode, offset = rootele.getClientRects()[0], containerTop = offset.top, containerLeft = offset.left, visiblePos = this.visiblePoints.map(function (a) { return a['coordinates']; }), data = this.visiblePoints.map(function (a) { return a['location']; }), radius = this.visiblePoints['radius'], centerPos = this.visiblePoints['centerPos'], angles = this.visiblePoints, currStDeg, currEdDeg, sliceIndex, dx = mouseX - (centerPos.X + containerLeft), dy = mouseY - (centerPos.Y + containerTop), angle = Math.atan2(dy, dx), pietipId = this.container.id + "_pieTooltip", tooltip = document.getElementById(pietipId), tooltipOptions, tooltipBorder = model.tooltip.border.color, pointRadius = Math.sqrt(dx * dx + dy * dy), eventType = (model.pointRegionMouseClick != null) ? "pointRegionMouseClick" : (model.pointRegionMouseMove != null) ? "pointRegionMouseMove" : null, colors = model.palette, locale = model.locale, localizedText = model.enableGroupSeparator;
                if (Math.abs(pointRadius) <= radius) {
                    for (var i = 0, len = angles.length; i < len; i++) {
                        currStDeg = angles[i]['stAng'];
                        currEdDeg = angles[i]['endAng'];
                        angle = angle < 0 ? (6.283 + angle) : angle;
                        if (angle <= currEdDeg && angle >= currStDeg) {
                            sliceIndex = i;
                            if ((eventType == "pointRegionMouseClick" && (evt.type == "click" || evt.type == "touchstart")) || eventType == "pointRegionMouseMove")
                                this._trigger(eventType, {
                                    data: {
                                        "pointIndex": i,
                                        "seriesType": "Pie",
                                        'locationX': mouseX,
                                        'locationY': mouseY
                                    }
                                });
                        }
                    }
                }
                else if (tooltip)
                    tooltip.parentNode.removeChild(tooltip);
                if (tooltip && sliceIndex != null) {
                    var text = data[sliceIndex]['Percent'].toFixed(2);
                    text = localizedText ? "&nbsp" + parseFloat(text).toLocaleString(locale) : text + "&nbsp%&nbsp";
                    var size = this.measureText(text, font);
                    tooltipOptions = {
                        'left': mouseX + 12 + "px",
                        'top': mouseY + "px",
                        'border': model.tooltip.border.width + "px solid " + ((tooltipBorder != null) ? tooltipBorder : colors[sliceIndex % colors.length]),
                        'background-color': model.tooltip.fill,
                        'height': size.height + "px",
                        'width': size.width + "px",
                    };
                    this.setStyles(tooltipOptions, tooltip);
                    tooltip.innerHTML = text;
                }
                else if (sliceIndex != null) {
                    tooltip = document.createElement('div');
                    var text = data[sliceIndex]['Percent'].toFixed(2);
                    text = localizedText ? "&nbsp" + parseFloat(text).toLocaleString(locale) : text + "&nbsp%&nbsp";
                    var size = this.measureText(text, font);
                    tooltipOptions = {
                        'left': mouseX + 12 + "px",
                        'top': mouseY + "px",
                        'background-color': model.tooltip.fill,
                        'color': font.color,
                        'border': model.tooltip.border.width + "px solid " + ((tooltipBorder != null) ? tooltipBorder : colors[sliceIndex % colors.length]),
                        'height': size.height + "px",
                        'width': size.width + "px",
                        'font-size': font.size,
                        'opacity': font.opacity,
                        'font-weight': font.fontWeight,
                        'font-family': font.fontFamily,
                        'font-style': font.fontStyle,
                        'z-index': "100000",
                        'position': "fixed"
                    };
                    tooltip.setAttribute('id', pietipId);
                    this.setStyles(tooltipOptions, tooltip);
                    tooltip.innerHTML = text;
                    document.body.appendChild(tooltip);
                }
                if (isTouch) {
                    this.touchEnd = true;
                    var touchtooltip = document.getElementById(this.container.id + "_pieTooltip");
                    var sparklineObj = this;
                    setTimeout(function () {
                        sparklineObj.fadeOut(touchtooltip);
                    }, 500);
                }
            }
        };
        Sparkline.prototype.pieTooltipHide = function (evt) {
            if (!this.touchEnd) {
                if (this.model.sparklineMouseLeave != null)
                    this._trigger("sparklineMouseLeave");
                var pietipId = this.container.id + "_pieTooltip", tooltip = document.getElementById(pietipId);
                tooltip ? document.body.removeChild(tooltip) : tooltip;
            }
        };
        Sparkline.prototype.bindRegionEvents = function (ele) {
            this.sparklineEvent = this.sparklineEvent.bind(this);
            var browser = this.findBrowser();
            if (browser == "IE" || browser == 'firefox') {
                ele.style['touch-action'] = 'none';
            }
            if (this.model.pointRegionMouseMove != null) {
                ele.addEventListener("mousemove", this.sparklineEvent, true);
                ele.addEventListener("touchmove", this.sparklineEvent, true);
            }
            if (this.model.pointRegionMouseClick != null) {
                ele.addEventListener("click", this.sparklineEvent, true);
                ele.addEventListener("touchstart", this.sparklineEvent, true);
            }
            if (this.model.sparklineMouseMove != null) {
                ele.addEventListener("mousemove", this.sparklineEvent, true);
                ele.addEventListener("touchmove", this.sparklineEvent, true);
            }
            if (this.model.sparklineMouseLeave != null) {
                this.sparklineLeave = this.sparklineLeave.bind(this);
                ele.addEventListener('mouseout', this.sparklineLeave, true);
                if (browser == 'firefox') {
                    ele.addEventListener('mouseup', this.sparklineLeave, true);
                }
                ele.addEventListener('touchend', this.sparklineLeave, true);
            }
        };
        Sparkline.prototype.sparklineLeave = function () {
            if (this.model.sparklineMouseLeave != null)
                this._trigger("sparklineMouseLeave");
        };
        Sparkline.prototype.sparklineEvent = function (evt) {
            if (this.model.sparklineMouseMove != null)
                this._trigger("sparklineMouseMove");
            var model = this.model, mouseX = evt.clientX || evt.touches[0].clientX, mouseY = evt.clientY || evt.touches[0].clientY, stype = model.type, pointLoc = this.visiblePoints, rX1, rY1, rX2, rY2, width, height, rootele = document.getElementById(this.container.id), parent = rootele.parentNode, offset = rootele.getClientRects()[0], containerTop = offset.top, containerLeft = offset.left, eventType;
            eventType = (model.pointRegionMouseClick != null) ? "pointRegionMouseClick" : (model.pointRegionMouseMove != null) ? "pointRegionMouseMove" : null;
            if (stype == Type.Line || stype == Type.Area) {
                width = 2 * (model.markerSettings.width + model.markerSettings.border.width);
                height = width;
                for (var i = 0, len = pointLoc.length; i < len; i++) {
                    rX1 = containerLeft + pointLoc[i]['location'].X - (width / 2);
                    rY1 = containerTop + pointLoc[i]['location'].Y - (height / 2);
                    rX2 = rX1 + width;
                    rY2 = rY1 + height;
                    if (mouseX >= rX1 && mouseX <= rX2 && mouseY >= rY1 && mouseY <= rY2) {
                        if (eventType != null)
                            this._trigger(eventType, {
                                data: {
                                    "pointIndex": i,
                                    "seriesType": stype,
                                    'locationX': pointLoc[i]['location'].X,
                                    'locationY': pointLoc[i]['location'].Y
                                }
                            });
                    }
                }
            }
            else if (stype == Type.Column || stype == Type.WinLoss) {
                width = pointLoc[0]['location'].width;
                for (var i = 0, len = pointLoc.length; i < len; i++) {
                    height = pointLoc[i]['location'].height;
                    rX1 = containerLeft + pointLoc[i]['location'].X;
                    rY1 = containerTop + pointLoc[i]['location'].Y;
                    rX2 = rX1 + width;
                    rY2 = rY1 + height;
                    if (mouseX >= rX1 && mouseX <= rX2 && mouseY >= rY1 && mouseY <= rY2) {
                        if (eventType != null)
                            this._trigger(eventType, {
                                data: {
                                    "pointIndex": i,
                                    "seriesType": stype,
                                    'locationX': pointLoc[i]['location'].X,
                                    'locationY': pointLoc[i]['location'].Y
                                }
                            });
                    }
                }
            }
        };
        Sparkline.prototype.findBrowser = function () {
            var browser;
            if (navigator.userAgent.indexOf("Firefox") != -1) {
                browser = 'firefox';
            }
            else if ((navigator.userAgent.indexOf("MSIE") != -1) || (!!document['documentMode'] == true)) {
                browser = 'IE';
            }
            else {
                browser = null;
            }
            return browser;
        };
        Sparkline.prototype.resize = function (evt) {
            var currentObj = this;
            var element = document.getElementById(this.rootId);
            if (element != null || element != undefined) {
                window.removeEventListener("resize", this.resize);
                this._destroy();
                this.createSvg();
                this.renderSparkline();
            }
            else {
                window.removeEventListener("resize", this.resize);
            }
        };
        Sparkline.prototype.supportsSvg = function () {
            return !!document.createElementNS && !!document.createElementNS('http://www.w3.org/2000/svg', "svg");
        };
        Sparkline.prototype._init = function (options) {
            if (this.supportsSvg()) {
                this.rootId = (this.rootId != null) ? this.rootId : this._id;
                this.parentElement = (this.parentElement != null) ? this.parentElement : document.getElementById(this.rootId);
                if (this.model.load != null)
                    this._trigger("load");
                this.setTheme(this.model);
                this.height = parseInt(this.model.size.height);
                this.width = parseInt(this.model.size.width);
                this.createSvg();
                this.renderSparkline();
                if (this.model.loaded != null)
                    this._trigger("loaded");
            }
        };
        Sparkline.prototype.redraw = function () {
            this.emptyContainer();
            this.renderSparkline();
        };
        Sparkline.prototype.touchCheck = function (event) {
            this.touchEnd = true;
            if (this.model.type.toString().toLocaleLowerCase() != "pie") {
                this.sparkMouseLeave(event);
            }
            else
                this.pieTooltipHide(event);
        };
        Sparkline.prototype.emptyContainer = function () {
            var model = this.model;
            if (model.enableCanvasRendering) {
                this.ctx.clearRect(0, 0, this.height, this.width);
            }
            else {
                var container = this.container;
                while (container.firstChild) {
                    container.removeChild(container.firstChild);
                }
            }
        };
        Sparkline.prototype._destroy = function () {
            var container = document.getElementById(this.container.id);
            if (container != null) {
                this.unBindEvents();
                this.unbindPieEvents();
                window.removeEventListener("resize", this.resize);
                container.parentNode.removeChild(container);
            }
            else {
                window.removeEventListener("resize", this.resize);
            }
        };
        Sparkline.prototype.renderSparkline = function () {
            this.setStyles({ 'background': this.model.background }, this.container);
            this.calculatePoints();
            this.seriesRender();
            this.resize = this.resize.bind(this);
            if (this.model.isResponsive) {
                window.addEventListener("resize", this.resize);
            }
            else {
                window.removeEventListener("resize", this.resize);
            }
            if ((this.model.type != Type.WinLoss && this.model.type != Type.Pie) && this.model.tooltip.visible)
                this.bindEvents(this.parentElement);
            else
                this.unBindEvents();
            if (this.model.type == Type.Pie && this.model.tooltip.visible)
                this.bindPieEvents(this.parentElement);
            else
                this.unbindPieEvents();
            this.bindRegionEvents(this.parentElement);
        };
        Sparkline.prototype.animateSparkline = function () {
            var model = this.model, sparkObj = this, stype = model.type, padding = model.padding, axisHeight = this.axisHeight, width = parseInt(model.size.width), height = parseInt(model.size.height), i, temp, timer, def = document.createElementNS(this.svgLink, "defs"), clipRect = document.createElementNS(this.svgLink, "clipPath"), animOption, clipOption, rect;
            if (stype == Type.Pie)
                rect = document.createElementNS(this.svgLink, "path");
            else
                rect = document.createElementNS(this.svgLink, "rect");
            if (stype == Type.Line || stype == Type.Area) {
                animOption = {
                    "id": "clipRectSparkline",
                    "x": 0,
                    "y": 0,
                    "width": 0,
                    "height": height
                };
                timer = width / 10;
                var animater = setInterval(animate, 120);
                i = timer;
            }
            else if (stype == Type.Column || stype == Type.WinLoss) {
                if (axisHeight == padding) {
                    animOption = {
                        "id": "clipRectSparkline",
                        "x": 0,
                        "y": 0,
                        "width": width,
                        "height": 0
                    };
                    timer = height / 10;
                    i = timer;
                    var ngtiveRectAnimater = setInterval(animateNegtiveRect, 120);
                }
                else if (axisHeight == (height - padding)) {
                    animOption = {
                        "id": "clipRectSparkline",
                        "x": 0,
                        "y": height,
                        "width": width,
                        "height": 0
                    };
                    timer = height / 10;
                    i = height;
                    var rectAnimater = setInterval(animateRect, 120);
                }
                else {
                    animOption = {
                        "id": "clipRectSparkline",
                        "x": 0,
                        "y": axisHeight,
                        "width": width,
                        "height": 0
                    };
                    timer = height / 10;
                    i = axisHeight;
                    var upperAnimater = setInterval(animateUpper, 120);
                }
            }
            else {
                var values = this.visiblePoints, centerPos = values['centerPos'], area = values['radius'], deg, stRad, edRad, pathArc = "", stDeg = 90, edDeg = 0, flag;
                i = 0;
                animOption = {
                    "id": "clipRectSparkline",
                    "d": pathArc,
                };
                var pieAnimater = setInterval(pieAnimate, 120);
            }
            clipOption = {
                "id": this.container.id + "_sparklineRect",
            };
            this.setAttributes(animOption, rect);
            this.setAttributes(clipOption, clipRect);
            clipRect.appendChild(rect);
            def.appendChild(clipRect);
            this.container.appendChild(def);
            function animate() {
                if (i <= width) {
                    rect.setAttribute("width", i.toString());
                }
                else {
                    clearInterval(animater);
                    sparkObj._trigger("animationComplete");
                }
                i = i + timer;
            }
            function animateRect() {
                temp = height - i;
                if (i >= 0) {
                    rect.setAttribute("y", i.toString());
                    rect.setAttribute("height", temp);
                }
                else {
                    clearInterval(rectAnimater);
                    sparkObj._trigger("animationComplete");
                }
                i = i - timer;
            }
            ;
            function animateNegtiveRect() {
                if (i <= height) {
                    rect.setAttribute("height", i.toString());
                }
                else {
                    clearInterval(ngtiveRectAnimater);
                    sparkObj._trigger("animationComplete");
                }
                i = i + timer;
            }
            ;
            function animateUpper() {
                temp = (axisHeight - i) * 2;
                if (i >= -timer) {
                    rect.setAttribute("y", i < 0 ? 0 : i.toString());
                    rect.setAttribute("height", temp);
                }
                else {
                    clearInterval(upperAnimater);
                    sparkObj._trigger("animationComplete");
                }
                i = i - timer;
            }
            ;
            function pieAnimate() {
                if (i <= (values.length - 1)) {
                    stDeg = 90;
                    deg = values[i]['location']['Degree'];
                    edDeg += deg + (i == 0 ? stDeg : 0);
                    stRad = (stDeg - 90) * Math.PI / 180.0;
                    edRad = (edDeg - 90) * Math.PI / 180.0;
                    if (i == values.length - 1)
                        edRad -= 0.0001;
                    temp = { sX: centerPos.X + (area * Math.cos(stRad)), sY: centerPos.Y + (area * Math.sin(stRad)), eX: centerPos.X + (area * Math.cos(edRad)), eY: centerPos.Y + (area * Math.sin(edRad)) };
                    pathArc = "M " + centerPos.X + " " + centerPos.Y + " L " + temp.eX + " " + temp.eY + " A " + area + " " + area + " 0 " + 1 + ",0 " + temp.sX + " " + temp.sY + " Z";
                    rect.setAttribute("d", pathArc);
                }
                else {
                    clearInterval(pieAnimater);
                    sparkObj._trigger("animationComplete");
                }
                i += 1;
            }
        };
        Sparkline.prototype.setTheme = function (model) {
            var theme = model.theme, defaults = this.defaults;
            if (theme == Themes.flatdark) {
                model.background = (model.background == defaults.background) ? "#111111" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#B5B5B5" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#F6D321" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#AAAAAA" : model.axisLineSettings.color;
            }
            else if (theme == Themes.gradientlight) {
                model.background = (model.background == defaults.background) ? "transparent" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#F34649" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#597B15" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#8E8E8E" : model.axisLineSettings.color;
            }
            else if (theme == Themes.gradientdark) {
                model.background = (model.background == defaults.background) ? "#111111" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#005378" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#6A9319" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#AAAAAA" : model.axisLineSettings.color;
            }
            else if (theme == Themes.azuredark) {
                model.background = (model.background == defaults.background) ? "#111111" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#007fff" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#f0ffff" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#336699" : model.axisLineSettings.color;
            }
            else if (theme == Themes.azurelight) {
                model.background = (model.background == defaults.background) ? "transparent" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#336699" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#007fff" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#336699" : model.axisLineSettings.color;
            }
            else if (theme == Themes.limedark) {
                model.background = (model.background == defaults.background) ? "#111111" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#238f23" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#32CD32" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#43da21" : model.axisLineSettings.color;
            }
            else if (theme == Themes.limelight) {
                model.background = (model.background == defaults.background) ? "transparent" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#238f23" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#32CD32" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#43da21" : model.axisLineSettings.color;
            }
            else if (theme == Themes.saffrondark) {
                model.background = (model.background == defaults.background) ? "#111111" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#ffaa33" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#ffdba9" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#ffc26e" : model.axisLineSettings.color;
            }
            else if (theme == Themes.saffronlight) {
                model.background = (model.background == defaults.background) ? "transparent" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#ffaa33" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#ffdba9" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#ffc26e" : model.axisLineSettings.color;
            }
            else if (theme == null || theme == Themes.flatlight) {
                model.background = (model.background == defaults.background) ? "transparent" : model.background;
                model.fill = (model.fill == defaults.fill) ? "#33ccff" : model.fill;
                model.stroke = (model.stroke == defaults.stroke) ? "#33ccff" : model.stroke;
                model.axisLineSettings.color = (model.axisLineSettings.color) == defaults.axisLineSettings.color ? "#FF0000" : model.axisLineSettings.color;
            }
        };
        Sparkline.prototype.createSvg = function () {
            var model = this.model, parentElement = this.parentElement, options, parentHeight = (parseInt(parentElement.style.height) > parentElement.clientHeight) ? parseInt(parentElement.style.height) : parentElement.clientHeight, parentWidth = (parseInt(parentElement.style.width) > parentElement.clientWidth) ? parseInt(parentElement.style.width) : parentElement.clientWidth, width = parseInt(model.size.width), height = parseInt(model.size.height);
            height = (height > 0) ? height : (parentHeight > 0) ? parentHeight : 30;
            width = (width > 0) ? width : (parentWidth > 0) ? parentWidth : 50;
            model.size.height = height.toString();
            model.size.width = width.toString();
            this.height = height;
            this.width = width;
            options = {
                "id": this.rootId + "_sparkline_svg",
                "width": width,
                "height": height,
            };
            if (!model.enableCanvasRendering) {
                this.container = document.createElementNS(this.svgLink, "svg");
            }
            else {
                this.container = document.createElement("canvas");
                this.ctx = this.container.getContext('2d');
            }
            this.setAttributes(options, this.container);
            parentElement.appendChild(this.container);
        };
        Sparkline.prototype.setStyles = function (Options, element) {
            var properties = Object.keys(Options), temp;
            var values = properties.map(function (property) { return Options[property]; });
            for (var i = 0, len = properties.length; i < len; i++) {
                temp = properties[i];
                element.style[temp] = values[i];
            }
        };
        Sparkline.prototype.setAttributes = function (Options, element) {
            var properties = Object.keys(Options);
            var values = properties.map(function (property) { return Options[property]; });
            for (var i = 0, len = properties.length; i < len; i++) {
                element.setAttribute(properties[i], values[i]);
            }
        };
        Sparkline.prototype.getDefaultPoints = function (count) {
            var data = [], i = 1;
            for (; i <= count; i++) {
                if ((Math.random() * 10) > 5)
                    data.push(-Math.round(Math.random() * 100));
                data.push(Math.round(Math.random() * 100));
            }
            return data;
        };
        Sparkline.prototype.calculatePoints = function () {
            var model = this.model, stype = model.type, data = (model.dataSource != null) ? model.dataSource : this.getDefaultPoints(12), x, y, max, min, minX, maxX, visiblePoints = [], maxPointsLength = data.length, temp, sumofValues = 0;
            if (Array.isArray(data) && typeof data[0] != 'object') {
                if (model.type == Type.Pie) {
                    for (var i = 0; i < maxPointsLength; i++) {
                        sumofValues += Math.abs(data[i]);
                    }
                }
                else {
                    max = Math.max.apply(null, data);
                    min = Math.min.apply(null, data);
                    minX = 0;
                    maxX = maxPointsLength - 1;
                }
            }
            else {
                if (model.type == Type.Pie) {
                    for (var i = 0; i < maxPointsLength; i++) {
                        sumofValues += Math.abs(data[i][model.yName]);
                    }
                }
                else {
                    if (!data[0][model.xName]) {
                        var x = data.map(function (z) { return z[model.yName]; });
                        max = Math.max.apply(null, x);
                        min = Math.min.apply(null, x);
                    }
                    else {
                        temp = data;
                        temp = temp.sort(function (a, b) { return a[model.yName] - b[model.yName]; });
                        max = temp[temp.length - 1][model.yName];
                        min = temp[0][model.yName];
                    }
                    if (data[0][model.xName]) {
                        temp = temp.sort(function (a, b) { return a[model.xName] - b[model.xName]; });
                        maxX = temp[temp.length - 1][model.xName];
                        minX = temp[0][model.xName];
                    }
                    else {
                        minX = 0;
                        maxX = maxPointsLength - 1;
                    }
                }
            }
            if (model.type != Type.Pie) {
                this.maxLength = maxPointsLength;
                var location, padding = Number(model.padding), height = parseInt(model.size.height) - (padding * 2), width = parseInt(model.size.width) - (padding * 2), unitX = maxX - minX, unitY = max - min, unitX = unitX == 0 ? 1 : unitX;
                unitY = unitY == 0 ? 1 : unitY;
                this.min = min;
                this.unitX = unitX;
                this.minX = minX;
                this.unitY = unitY;
                this.max = max;
                this.maxX = maxX;
                var X1 = 0, Y1 = height - ((height / unitY) * (-min)), Y2, Y1 = (min < 0 && max < 0) ? 0 : (min < 0 && max > 0) ? Y1 : height;
                this.axisHeight = Y1 + padding;
                if (stype != Type.WinLoss && model.axisLineSettings.visible)
                    this.drawAxis();
            }
            else
                var percent, location;
            for (var i = 0; i < maxPointsLength; i++) {
                if (!(data[i][model.xName]) && !(data[i][model.yName])) {
                    x = i;
                    y = data[i];
                }
                else if (!(data[i][model.xName])) {
                    x = i;
                    y = data[i][model.yName];
                }
                else {
                    x = data[i][model.xName];
                    y = data[i][model.yName];
                }
                if (stype == Type.Line || stype == Type.Area) {
                    Y2 = (maxPointsLength != 1) ? height - Math.round(height * ((y - min) / unitY)) : 0;
                    location = { X: (maxPointsLength != 1) ? Math.round(width * ((x - minX) / unitX)) : width / 2, Y: Y2, markerPos: Y2 };
                }
                else if (stype == Type.Column || stype == Type.WinLoss) {
                    var colWidth = width / maxPointsLength, calSpace = 0.5, space = (calSpace * 2);
                    colWidth -= (space);
                    X1 = (i * (colWidth + space)) + (space / 2);
                    if (stype == Type.WinLoss) {
                        var winLossFactor = 0.5, drawHeightFactor = 40;
                        Y2 = (y > 0) ? height / 4 : (y < 0) ? (height * winLossFactor) : (height * winLossFactor) - (height / drawHeightFactor);
                        location = { X: X1, Y: Y2, height: (y != 0) ? (height / 4) : height / 20, width: colWidth };
                    }
                    else {
                        Y2 = (maxPointsLength != 1) ? height - ((height / unitY) * (y - min)) : 0;
                        location = { X: X1, Y: (Y2 > Y1) ? Y1 : Y2, height: Math.abs(Y2 - Y1), width: colWidth, markerPos: (Y2 > Y1) ? (Y1 + Math.abs(Y2 - Y1)) : Y2 };
                    }
                }
                else if (stype == Type.Pie) {
                    percent = (Math.abs(y) / sumofValues) * 100;
                    location = {
                        Percent: percent, Degree: ((Math.abs(y) / sumofValues) * 360)
                    };
                }
                if (stype != Type.Pie) {
                    location.X += padding;
                    location.Y += padding;
                }
                if (stype != Type.WinLoss)
                    location.markerPos += padding;
                location['Xval'] = x;
                location['Yval'] = y;
                visiblePoints.push({ location: location });
            }
            this.visiblePoints = visiblePoints;
        };
        Sparkline.prototype.seriesRender = function () {
            var model = this.model, visiblePoints = this.visiblePoints, points_length = visiblePoints.length, eventArgs, seriesType = model.type;
            eventArgs = {
                data: {
                    "minX": this.minX,
                    "minY": this.min,
                    "maxX": this.maxX,
                    "maxY": this.max,
                    "xName": model.xName,
                    "yName": model.yName,
                    "pointsCount": points_length,
                    "seriesType": seriesType,
                    "visiblePoints": this.visiblePoints
                }
            };
            if (this.model.seriesRendering != null)
                this._trigger("seriesRendering", eventArgs);
            if (seriesType == Type.Line) {
                this.drawLineSeries(visiblePoints);
            }
            else if (seriesType == Type.Area) {
                this.drawAreaSeries(visiblePoints);
            }
            else if (seriesType == Type.Column) {
                this.drawColumnSeries(visiblePoints);
            }
            else if (seriesType == Type.WinLoss) {
                this.drawWinlossSeries(visiblePoints);
            }
            else if (seriesType == Type.Pie) {
                this.drawPieSeries();
            }
            if (model.markerSettings.visible && (seriesType != Type.WinLoss) && (seriesType != Type.Pie))
                this.drawMarker(visiblePoints);
            if ((model.rangeBandSettings.startRange != null) && (model.rangeBandSettings.endRange != null) && (seriesType != Type.WinLoss) && (seriesType != Type.Pie))
                this.drawRangeBand();
        };
        Sparkline.prototype.drawPieSeries = function () {
            var model = this.model, values = this.visiblePoints;
            model.padding = (model.padding == this.defaults.padding) ? 2 : model.padding;
            var len = values.length, height = parseInt(model.size.height) - (model.padding * 2), width = parseInt(model.size.width) - (model.padding * 2), area = (height <= width) ? height / 2 : width / 2, stRad, edRad, centerPos = { X: width / 2, Y: height / 2 }, temp, pathArc, pathOptions, colors = model.palette, deg = 0, stroke = model.border.color, opacity = model.opacity, strokeWidth = model.border.width, gEle;
            values['centerPos'] = centerPos;
            values['radius'] = area;
            for (var i = 0, stDeg = 90, edDeg, flag; i < values.length; i++) {
                stDeg += deg;
                deg = values[i]['location']['Degree'];
                edDeg = stDeg + deg;
                stRad = (stDeg - 90) * Math.PI / 180.0;
                edRad = (edDeg - 90) * Math.PI / 180.0;
                values[i]['stAng'] = stRad;
                values[i]['endAng'] = edRad;
                flag = (deg < 180) ? "0" : "1";
                temp = values[i]['coordinates'] = { sX: centerPos.X + (area * Math.cos(stRad)), sY: centerPos.Y + (area * Math.sin(stRad)), eX: centerPos.X + (area * Math.cos(edRad)), eY: centerPos.Y + (area * Math.sin(edRad)) };
                pathArc = "M " + centerPos.X + " " + centerPos.Y + " L " + temp.eX + " " + temp.eY + " A " + area + " " + area + " 0 " + flag + ",0 " + temp.sX + " " + temp.sY + " Z";
                pathOptions = {
                    'id': this.container.id + "_pieBase" + i,
                    'fill': colors[i % colors.length],
                    'stroke': stroke,
                    'opacity': opacity,
                    'stroke-width': strokeWidth,
                    'd': pathArc,
                    'start': edRad,
                    'end': stRad,
                    'x': centerPos.X,
                    'y': centerPos.Y,
                    'counterClockWise': flag,
                    'radius': area
                };
                if (model.enableCanvasRendering)
                    this.canvasDrawPath(pathOptions);
                else {
                    gEle = this.createGroup({ "id": "sparkpieSeries" });
                    this.setStyles({ "clip-path": "url(#" + this.container.id + "_sparklineRect)" }, gEle);
                    this.container.appendChild(gEle);
                    this.drawPath(pathOptions, gEle);
                }
            }
        };
        Sparkline.prototype.drawRangeBand = function () {
            var model = this.model, height = (parseInt(model.size.height) - model.padding * 2), width = (parseInt(model.size.width) - model.padding * 2), stValue = model.rangeBandSettings.startRange, edValue = model.rangeBandSettings.endRange, stHeight = Number(height - ((height / this.unitY) * (stValue - this.min))) + model.padding, edHeight = Number(height - ((height / this.unitY) * (edValue - this.min))) + model.padding, options;
            edHeight = edHeight > Number(height + model.padding) ? Number(height + model.padding) : edHeight < (0 + model.padding) ? (0 + model.padding) : edHeight;
            stHeight = stHeight > Number(height + model.padding) ? Number(height + model.padding) : stHeight < (0 + model.padding) ? (0 + model.padding) : stHeight;
            var path = 'M ' + model.padding + " " + stHeight + " L " + (width + Number(model.padding)) + " " + stHeight + " L " + (width + Number(model.padding)) + " " + edHeight + " L " + model.padding + " " + edHeight + " Z";
            options = {
                'id': this.container.id + "_rangeBand",
                'fill': model.rangeBandSettings.color,
                'opacity': model.rangeBandSettings.opacity,
                'stroke': "transparent",
                'stroke-width': model.border.width,
                'd': path,
            };
            if (model.enableCanvasRendering)
                this.canvasDrawPath(options);
            else
                this.drawPath(options);
        };
        Sparkline.prototype.drawAxis = function () {
            var model = this.model, height = this.axisHeight, strclr = model.axisLineSettings.color, seriesType = model.type;
            if ((seriesType != Type.WinLoss) && (seriesType != Type.Pie)) {
                if (!model.axisLineSettings.visible)
                    strclr = 'transparent';
                var xAxis = {
                    'id': this.container.id + "_Sparkline_XAxis",
                    'x1': 0 + model.padding,
                    'y1': height,
                    'x2': parseInt(model.size.width) - model.padding,
                    'y2': height,
                    'stroke': strclr,
                    'stroke-dasharray': model.axisLineSettings.dashArray,
                    'stroke-width': model.axisLineSettings.width,
                };
                if (model.enableCanvasRendering)
                    this.canvasDrawLine(xAxis);
                else
                    this.drawLine(xAxis);
            }
        };
        Sparkline.prototype.drawColumnSeries = function (points) {
            var rectOptions, temp, model = this.model, mod = [], len = points.length, locations = ejSparkline.compareExtend({}, mod, points), strwd = model.border.width, lowPos, opacity = model.opacity, highPos, fill = model.fill, stroke = model.border.color, highPointColor = model.highPointColor, lowPointColor = model.lowPointColor, startPointColor = model.startPointColor, endPointColor = model.endPointColor, negativePointColor = model.negativePointColor, gEle;
            if (highPointColor || lowPointColor) {
                var pointsYPos = locations.map(function (a) { return a['location']['markerPos']; });
                highPos = Math.min.apply(null, pointsYPos);
                lowPos = Math.max.apply(null, pointsYPos);
            }
            if (model.enableCanvasRendering == false) {
                gEle = this.createGroup({ "id": this.container.id + "sparkcolumnSeries" });
                this.setStyles({ "clip-path": "url(#" + this.container.id + "_sparklineRect)" }, gEle);
                this.container.appendChild(gEle);
            }
            for (var i = 0; i < len; i++) {
                temp = points[i].location;
                rectOptions = {
                    'id': this.container.id + "_column_series_" + i,
                    'x': temp.X,
                    'y': temp.Y,
                    'height': temp.height,
                    'width': temp.width,
                    'fill': fill,
                    'stroke': stroke,
                    'opacity': opacity,
                    "stroke-width": strwd,
                };
                if (temp.markerPos == highPos && highPointColor) {
                    rectOptions.fill = highPointColor;
                    this.highPointIndex = i;
                }
                else if (temp.markerPos == lowPos && lowPointColor) {
                    rectOptions.fill = lowPointColor;
                    this.lowPointIndex = i;
                }
                else if (i == 0 && startPointColor) {
                    rectOptions.fill = startPointColor;
                    this.startPointIndex = i;
                }
                else if ((i == (len - 1)) && endPointColor) {
                    rectOptions.fill = endPointColor;
                    this.endPointIndex = i;
                }
                else if (temp.markerPos >= this.axisHeight && negativePointColor) {
                    rectOptions.fill = negativePointColor;
                    this.negativePointIndexes.push(i);
                }
                if (model.enableCanvasRendering)
                    this.canvasDrawRectangle(rectOptions);
                else {
                    this.drawRect(rectOptions, gEle);
                }
                temp.X += temp.width / 2;
            }
        };
        Sparkline.prototype.drawWinlossSeries = function (points) {
            var rectOptions, temp, model = this.model, strwd = model.border.width, padding = model.padding, stroke = model.border.color, opacity = model.opacity, tieColor = "#EE82EE", len = points.length, height = parseInt(model.size.height) - (padding * 2), gEle;
            if (model.enableCanvasRendering == false) {
                gEle = this.createGroup({ "id": this.container.id + "sparkwinlossSeries" });
                this.setStyles({ "clip-path": "url(#" + this.container.id + "_sparklineRect)" }, gEle);
                this.container.appendChild(gEle);
            }
            for (var i = 0; i < len; i++) {
                temp = points[i].location;
                rectOptions = {
                    'id': this.container.id + "_winloss_series_" + i,
                    'x': temp.X,
                    'y': temp.Y,
                    'height': temp.height,
                    'width': temp.width,
                    'fill': (temp.Y < (height / 2 + padding)) ? (temp.Y > (height / 4 + padding)) ? tieColor : model.fill : model.negativePointColor ? model.negativePointColor : "#FF0000",
                    'stroke': stroke,
                    'opacity': opacity,
                    "stroke-width": strwd
                };
                if (model.enableCanvasRendering)
                    this.canvasDrawRectangle(rectOptions);
                else {
                    this.drawRect(rectOptions, gEle);
                }
            }
        };
        Sparkline.prototype.drawAreaSeries = function (points) {
            var linepath = "", totHeight = this.axisHeight, model = this.model, options, gEle;
            for (var i = 0, len = points.length; i < len; i++) {
                if (i == 0) {
                    linepath = "M " + points[0].location.X + " " + totHeight + " ";
                }
                linepath += "L " + points[i].location.X + " " + points[i].location.Y + " ";
                if (i == len - 1) {
                    linepath += "L " + points[i].location.X + " " + totHeight + " Z";
                }
            }
            options = {
                "id": this.container.id + "_area_series_fill",
                "fill": model.fill,
                "stroke": model.stroke ? model.stroke : model.fill,
                'fill-opacity': model.opacity,
                "stroke-width": model.width,
                "d": linepath
            };
            if (model.enableCanvasRendering)
                this.canvasDrawPath(options);
            else {
                gEle = this.createGroup({ "id": "sparkAreaSeries" });
                this.container.appendChild(gEle);
                this.drawPath(options, gEle);
                this.setStyles({ "clip-path": "url(#" + this.container.id + "_sparklineRect)" }, gEle);
            }
        };
        Sparkline.prototype.drawLineSeries = function (points) {
            var linepath = '', model = this.model, gEle;
            for (var i = 0, len = points.length; i < len; i++) {
                if (i == 0)
                    linepath = "M " + points[0].location.X + " " + points[i].location.Y + " ";
                linepath += "L " + points[i].location.X + " " + points[i].location.Y + " ";
            }
            var line_options = {
                'id': this.container.id + "_Line_series",
                'fill': 'transparent',
                'stroke': model.stroke ? model.stroke : model.fill,
                'opacity': model.opacity,
                'stroke-width': model.width,
                'd': linepath
            };
            if (model.enableCanvasRendering)
                this.canvasDrawPath(line_options);
            else {
                gEle = this.createGroup({ "id": "sparklineSeries" });
                this.container.appendChild(gEle);
                this.drawPath(line_options, gEle);
                this.setStyles({ "clip-path": "url(#" + this.container.id + "_sparklineRect)" }, gEle);
            }
        };
        Sparkline.prototype.drawMarker = function (points) {
            var length = points.length, marker_options, marker = this.model.markerSettings, mod = [], model = this.model, locations = ejSparkline.compareExtend({}, mod, points), fill = marker.fill ? marker.fill : model.fill, stroke = marker.border.color, strwid = marker.width, opacity = marker.opacity, brdwid = marker.border.width, temp, lowPos, highPos, highPointColor = model.highPointColor, lowPointColor = model.lowPointColor, startPointColor = model.startPointColor, endPointColor = model.endPointColor, negativePointColor = model.negativePointColor, gEle;
            if (model.enableCanvasRendering == false) {
                gEle = this.createGroup({ "id": this.container.id + "sparkmarkers" });
                this.setStyles({ "clip-path": "url(#" + this.container.id + "_sparklineRect)" }, gEle);
                this.container.appendChild(gEle);
            }
            if (marker.visible == false || (model.type == Type.Pie || model.type == Type.WinLoss)) {
                strwid = 0;
                brdwid = 0;
            }
            if (highPointColor || lowPointColor) {
                var pointsYPos = locations.map(function (a) { return a['location']['markerPos']; });
                highPos = Math.min.apply(null, pointsYPos);
                lowPos = Math.max.apply(null, pointsYPos);
            }
            for (var i = 0, wid; i < length; i++) {
                temp = points[i].location;
                wid = (temp.width != undefined) ? (temp.width / 2) : 0;
                marker_options = {
                    'id': this.container.id + "_marker_" + i,
                    'cx': temp.X,
                    'cy': temp.markerPos,
                    'r': strwid,
                    'fill': fill,
                    'stroke': stroke,
                    'opacity': opacity,
                    'stroke-width': brdwid
                };
                if (temp.markerPos == highPos && highPointColor) {
                    marker_options.fill = highPointColor;
                    this.highPointIndex = i;
                }
                else if (temp.markerPos == lowPos && lowPointColor) {
                    marker_options.fill = lowPointColor;
                    this.lowPointIndex = i;
                }
                else if (i == 0 && startPointColor) {
                    marker_options.fill = startPointColor;
                    this.startPointIndex = i;
                }
                else if ((i == (length - 1)) && endPointColor) {
                    marker_options.fill = endPointColor;
                    this.endPointIndex = i;
                }
                else if (temp.markerPos >= this.axisHeight && negativePointColor) {
                    marker_options.fill = negativePointColor;
                    this.negativePointIndexes.push(i);
                }
                if (this.model.enableCanvasRendering)
                    this.canvasDrawCircle(marker_options);
                else
                    this.drawCircle(marker_options, gEle);
            }
        };
        Sparkline.prototype.drawLine = function (Options) {
            var svgshape = document.getElementById(Options['id']);
            if (!svgshape)
                svgshape = document.createElementNS(this.svgLink, "line");
            this.setAttributes(Options, svgshape);
            this.container.appendChild(svgshape);
        };
        Sparkline.prototype.drawCircle = function (Options, element) {
            element = (!!element) ? element : this.container;
            var svgshape = document.getElementById(Options['id']);
            if (!svgshape)
                svgshape = document.createElementNS(this.svgLink, "circle");
            this.setAttributes(Options, svgshape);
            element.appendChild(svgshape);
        };
        Sparkline.prototype.drawPolyLine = function (Options) {
            var svgshape = document.getElementById(Options['id']);
            if (!svgshape)
                svgshape = document.createElementNS(this.svgLink, "polyline");
            this.setAttributes(Options, svgshape);
            this.container.appendChild(svgshape);
        };
        Sparkline.prototype.drawPath = function (Options, element) {
            element = (!!element) ? element : this.container;
            var svgshape = document.getElementById(Options['id']);
            if (!svgshape)
                svgshape = document.createElementNS(this.svgLink, "path");
            this.setAttributes(Options, svgshape);
            element.appendChild(svgshape);
        };
        Sparkline.prototype.drawRect = function (Options, element) {
            element = (!!element) ? element : this.container;
            var svgshape = document.getElementById(Options['id']);
            if (!svgshape)
                svgshape = document.createElementNS(this.svgLink, "rect");
            this.setAttributes(Options, svgshape);
            element.appendChild(svgshape);
        };
        Sparkline.prototype.createGroup = function (options) {
            var group = document.createElementNS(this.svgLink, "g");
            this.setAttributes(options, group);
            return group;
        };
        Sparkline.prototype.createText = function (options, label) {
            var text = document.createElementNS(this.svgLink, "text");
            this.setAttributes(options, text);
            if (label)
                text.textContent = label;
            return text;
        };
        Sparkline.prototype.canvasDrawLine = function (options, canvasContext) {
            var context2d = canvasContext ? canvasContext : this.ctx;
            context2d.save();
            context2d.beginPath();
            context2d.lineWidth = options["stroke-width"];
            context2d.strokeStyle = options["stroke"];
            context2d.moveTo(options["x1"], options["y1"]);
            context2d.lineTo(options["x2"], options["y2"]);
            context2d.stroke();
            context2d.restore();
            this.dataURL = this.container.toDataURL();
        };
        Sparkline.prototype.canvasDrawRectangle = function (options, canvasContext) {
            var context2d = canvasContext ? canvasContext : this.ctx, canvasCtx = canvasContext ? canvasContext : this.ctx;
            context2d.save();
            context2d.beginPath();
            context2d.globalAlpha = options["opacity"];
            context2d.lineWidth = options["stroke-width"];
            var dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
            if (dashArray)
                this.ctx.setLineDash(dashArray);
            context2d.strokeStyle = options["stroke"];
            context2d.rect(options["x"], options["y"], options["width"], options["height"]);
            if (options["fill"] == "none")
                options["fill"] = "transparent";
            context2d.fillStyle = options["fill"];
            context2d.fillRect(options["x"], options["y"], options["width"], options["height"]);
            context2d.stroke();
            context2d.restore();
            context2d = canvasCtx;
            this.dataURL = this.container.toDataURL();
        };
        Sparkline.prototype.canvasDrawPath = function (options, canvasContext) {
            var path = options["d"];
            var dataSplit = path.split(" ");
            var borderWidth = options["stroke-width"];
            var context2d = canvasContext ? canvasContext : this.ctx, canvasCtx = canvasContext ? canvasContext : this.ctx;
            context2d.save();
            context2d.beginPath();
            context2d.globalAlpha = options["opacity"] ? options["opacity"] : options["fill-opacity"];
            var flag = true;
            context2d.lineWidth = borderWidth;
            var dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
            if (dashArray)
                context2d.setLineDash(dashArray);
            context2d.strokeStyle = options["stroke"];
            for (var i = 0; i < dataSplit.length; i = i + 3) {
                var x1 = parseFloat(dataSplit[i + 1]);
                var y1 = parseFloat(dataSplit[i + 2]);
                switch (dataSplit[i]) {
                    case "M":
                        if (!options["innerR"] && !options["cx"])
                            context2d.moveTo(x1, y1);
                        break;
                    case "L":
                        if (!options["innerR"])
                            context2d.lineTo(x1, y1);
                        break;
                    case "C":
                        context2d.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                        i = i + 4;
                        break;
                    case "A":
                        if (!options["innerR"]) {
                            if (options["cx"]) {
                                context2d.arc(options["cx"], options["cy"], options["radius"], 0, 2 * Math.PI, options["counterClockWise"]);
                            }
                            else {
                                context2d.moveTo(options["x"], options["y"]);
                                context2d.arc(options["x"], options["y"], options["radius"], options["start"], options["end"], options["counterClockWise"]);
                                context2d.lineTo(options["x"], options["y"]);
                            }
                        }
                        else if (flag) {
                            context2d.arc(options["x"], options["y"], options["radius"], options["start"], options["end"], options["counterClockWise"]);
                            context2d.arc(options["x"], options["y"], options["innerR"], options["end"], options["start"], !options["counterClockWise"]);
                            flag = false;
                        }
                        i = i + 5;
                        break;
                    case "Z":
                        context2d.closePath();
                        break;
                }
            }
            if (options["fill"] != "none" && options["fill"] != undefined) {
                context2d.fillStyle = options["fill"];
                context2d.fill();
            }
            if (borderWidth > 0)
                context2d.stroke();
            context2d.restore();
            context2d = canvasCtx;
            this.dataURL = this.container.toDataURL();
        };
        Sparkline.prototype.canvasDrawCircle = function (options, canvasContext) {
            var context2d = canvasContext ? canvasContext : this.ctx, canvasCtx = canvasContext ? canvasContext : this.ctx, dashArray;
            context2d.save();
            context2d.beginPath();
            context2d.arc(options["cx"], options["cy"], options["r"], 0, 2 * Math.PI);
            context2d.fillStyle = options["fill"];
            context2d.globalAlpha = options["opacity"];
            context2d.fill();
            context2d.lineWidth = options["stroke-width"];
            dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
            if (dashArray)
                context2d.setLineDash(dashArray);
            context2d.strokeStyle = options["stroke"];
            context2d.stroke();
            context2d.restore();
            context2d = canvasCtx;
            this.dataURL = this.container.toDataURL();
        };
        Sparkline.prototype.canvasDrawPolyline = function (options, canvasContext) {
            var context2d = canvasContext ? canvasContext : this.ctx;
            context2d.save();
            context2d.beginPath();
            var points = options["points"].split(" ");
            for (var i = 0; i < points.length - 1; i++) {
                var point = points[i].split(",");
                var x = point[0];
                var y = point[1];
                if (i == 0)
                    context2d.moveTo(x, y);
                else
                    context2d.lineTo(x, y);
            }
            context2d.lineWidth = options["stroke-width"];
            context2d.strokeStyle = options["stroke"];
            context2d.stroke();
            context2d.restore();
            this.dataURL = this.container.toDataURL();
        };
        Sparkline.prototype.canvasDrawText = function (options, label, canvasContext) {
            var font = options['font'];
            var anchor = options["text-anchor"];
            var opacity = options["opacity"] !== undefined ? options["opacity"] : 1;
            if (anchor == "middle")
                anchor = "center";
            var context2d = canvasContext ? canvasContext : this.ctx;
            context2d.save();
            context2d.fillStyle = options["fill"];
            context2d.font = font;
            context2d.textAlign = anchor;
            context2d.globalAlpha = opacity;
            if (options["baseline"])
                context2d.textBaseline = options["baseline"];
            var txtlngth = 0;
            if (options["labelRotation"] == 90 && options["id"].indexOf("XLabel") != -1)
                txtlngth = context2d.measureText(label).width;
            context2d.translate(options["x"] + (txtlngth / 2), options["y"]);
            context2d.rotate(options["labelRotation"] * Math.PI / 180);
            context2d.fillText(label, 0, 0);
            context2d.restore();
            this.dataURL = this.container.toDataURL();
        };
        Sparkline.prototype.measureText = function (text, font) {
            var element = document.getElementById("measureTex"), textObj;
            if (!element || element.clientHeight == 0) {
                textObj = document.createElement('text');
                textObj.setAttribute('id', 'measureTex');
                document.body.appendChild(textObj);
            }
            else {
                textObj = element;
            }
            var style = null, size = null, family = null, weight = null;
            if (typeof (text) == "string" && (text.indexOf("<") > -1 || text.indexOf(">") > -1)) {
                var textArray = text.split(" ");
                for (var i = 0; i < textArray.length; i++) {
                    if (textArray[i].indexOf("<br/>") == -1)
                        textArray[i] = textArray[i].replace(/[<>]/g, '&');
                }
                text = textArray.join(' ');
            }
            textObj.innerHTML = text;
            if (font != undefined && font.size == undefined) {
                var fontarray = font;
                fontarray = fontarray.split(" ");
                style = fontarray[0];
                size = fontarray[1];
                family = fontarray[2];
                weight = fontarray[3];
            }
            if (font != null) {
                textObj.style.fontSize = (font.size > 0) ? (font.size + "px") : font.size ? font.size : size;
                if (textObj.style.fontStyle)
                    textObj.style.fontStyle = (font.fontStyle) ? font.fontStyle : style;
                textObj.style.fontFamily = font.fontFamily ? font.fontFamily : family;
                if (window.navigator.userAgent.indexOf('MSIE 8.0') == -1)
                    textObj.style.fontWeight = font.fontWeight ? font.fontWeight : weight;
            }
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100 + "px";
            textObj.style.left = 0 + 'px';
            textObj.style.visibility = 'hidden';
            textObj.style.whiteSpace = 'nowrap';
            var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            return bounds;
        };
        return Sparkline;
    }(ej.WidgetBase));
    ejSparkline.Sparkline = Sparkline;
    ej.widget("ejSparkline", "ej.Sparkline", new Sparkline());
    ejSparkline.deepExtend = function (out) {
        out = out || {};
        for (var i = 1; i < arguments.length; i++) {
            var obj = arguments[i];
            if (!obj)
                continue;
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (typeof obj[key] === 'object')
                        out[key] = ejSparkline.deepExtend(out[key], obj[key]);
                    else
                        out[key] = obj[key];
                }
            }
        }
        return out;
    };
    ejSparkline.compareExtend = function (temp, src, def) {
        if (typeof def === 'object' && def !== null) {
            var defProp = Object.keys(def), len = defProp.length, currPro;
            for (var i = 0; i < len; i++) {
                currPro = defProp[i];
                if (src.hasOwnProperty(currPro) && src[currPro] != null) {
                    if (Array.isArray(src[currPro]) || typeof src[currPro] === 'object' && src[currPro] !== null) {
                        ejSparkline.compareExtend({}, src[currPro], def[currPro]);
                    }
                }
                else {
                    src[currPro] = def[currPro];
                }
            }
        }
        return src;
    };
})(ejSparkline || (ejSparkline = {}));
;
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ejSunburstChart;
(function (ejSunburstChart) {
    (function (sunburstHorizontalAlignment) {
        sunburstHorizontalAlignment[sunburstHorizontalAlignment["left"] = "left"] = "left";
        sunburstHorizontalAlignment[sunburstHorizontalAlignment["right"] = "right"] = "right";
        sunburstHorizontalAlignment[sunburstHorizontalAlignment["center"] = "center"] = "center";
    })(ejSunburstChart.sunburstHorizontalAlignment || (ejSunburstChart.sunburstHorizontalAlignment = {}));
    var sunburstHorizontalAlignment = ejSunburstChart.sunburstHorizontalAlignment;
    (function (sunburstLegendShape) {
        sunburstLegendShape[sunburstLegendShape["circle"] = "circle"] = "circle";
        sunburstLegendShape[sunburstLegendShape["diamond"] = "diamond"] = "diamond";
        sunburstLegendShape[sunburstLegendShape["cross"] = "cross"] = "cross";
        sunburstLegendShape[sunburstLegendShape["pentagon"] = "pentagon"] = "pentagon";
        sunburstLegendShape[sunburstLegendShape["rectangle"] = "rectangle"] = "rectangle";
        sunburstLegendShape[sunburstLegendShape["triangle"] = "triangle"] = "triangle";
    })(ejSunburstChart.sunburstLegendShape || (ejSunburstChart.sunburstLegendShape = {}));
    var sunburstLegendShape = ejSunburstChart.sunburstLegendShape;
    (function (sunburstVerticalAlignment) {
        sunburstVerticalAlignment[sunburstVerticalAlignment["top"] = "top"] = "top";
        sunburstVerticalAlignment[sunburstVerticalAlignment["bottom"] = "bottom"] = "bottom";
        sunburstVerticalAlignment[sunburstVerticalAlignment["middle"] = "middle"] = "middle";
    })(ejSunburstChart.sunburstVerticalAlignment || (ejSunburstChart.sunburstVerticalAlignment = {}));
    var sunburstVerticalAlignment = ejSunburstChart.sunburstVerticalAlignment;
    (function (sunburstTheme) {
        sunburstTheme[sunburstTheme["flatlight"] = "flatlight"] = "flatlight";
        sunburstTheme[sunburstTheme["flatdark"] = "flatdark"] = "flatdark";
    })(ejSunburstChart.sunburstTheme || (ejSunburstChart.sunburstTheme = {}));
    var sunburstTheme = ejSunburstChart.sunburstTheme;
    (function (sunburstHighlightType) {
        sunburstHighlightType[sunburstHighlightType["opacity"] = "opacity"] = "opacity";
        sunburstHighlightType[sunburstHighlightType["color"] = "color"] = "color";
    })(ejSunburstChart.sunburstHighlightType || (ejSunburstChart.sunburstHighlightType = {}));
    var sunburstHighlightType = ejSunburstChart.sunburstHighlightType;
    (function (sunburstAnimationType) {
        sunburstAnimationType[sunburstAnimationType["rotation"] = "rotation"] = "rotation";
        sunburstAnimationType[sunburstAnimationType["fadeIn"] = "fadeIn"] = "fadeIn";
    })(ejSunburstChart.sunburstAnimationType || (ejSunburstChart.sunburstAnimationType = {}));
    var sunburstAnimationType = ejSunburstChart.sunburstAnimationType;
    (function (sunburstSelectionType) {
        sunburstSelectionType[sunburstSelectionType["opacity"] = "opacity"] = "opacity";
        sunburstSelectionType[sunburstSelectionType["color"] = "color"] = "color";
    })(ejSunburstChart.sunburstSelectionType || (ejSunburstChart.sunburstSelectionType = {}));
    var sunburstSelectionType = ejSunburstChart.sunburstSelectionType;
    (function (sunburstLegendPosition) {
        sunburstLegendPosition[sunburstLegendPosition["top"] = "top"] = "top";
        sunburstLegendPosition[sunburstLegendPosition["bottom"] = "bottom"] = "bottom";
        sunburstLegendPosition[sunburstLegendPosition["left"] = "left"] = "left";
        sunburstLegendPosition[sunburstLegendPosition["right"] = "right"] = "right";
        sunburstLegendPosition[sunburstLegendPosition["float"] = "float"] = "float";
    })(ejSunburstChart.sunburstLegendPosition || (ejSunburstChart.sunburstLegendPosition = {}));
    var sunburstLegendPosition = ejSunburstChart.sunburstLegendPosition;
    (function (sunburstLegendClickAction) {
        sunburstLegendClickAction[sunburstLegendClickAction["none"] = "none"] = "none";
        sunburstLegendClickAction[sunburstLegendClickAction["toggleSegmentVisibility"] = "toggleSegmentVisibility"] = "toggleSegmentVisibility";
        sunburstLegendClickAction[sunburstLegendClickAction["toggleSegmentSelection"] = "toggleSegmentSelection"] = "toggleSegmentSelection";
    })(ejSunburstChart.sunburstLegendClickAction || (ejSunburstChart.sunburstLegendClickAction = {}));
    var sunburstLegendClickAction = ejSunburstChart.sunburstLegendClickAction;
    (function (sunburstAlignment) {
        sunburstAlignment[sunburstAlignment["near"] = "near"] = "near";
        sunburstAlignment[sunburstAlignment["far"] = "far"] = "far";
        sunburstAlignment[sunburstAlignment["center"] = "center"] = "center";
    })(ejSunburstChart.sunburstAlignment || (ejSunburstChart.sunburstAlignment = {}));
    var sunburstAlignment = ejSunburstChart.sunburstAlignment;
    (function (sunburstHighlightMode) {
        sunburstHighlightMode[sunburstHighlightMode["point"] = "point"] = "point";
        sunburstHighlightMode[sunburstHighlightMode["all"] = "all"] = "all";
        sunburstHighlightMode[sunburstHighlightMode["child"] = "child"] = "child";
        sunburstHighlightMode[sunburstHighlightMode["parent"] = "parent"] = "parent";
    })(ejSunburstChart.sunburstHighlightMode || (ejSunburstChart.sunburstHighlightMode = {}));
    var sunburstHighlightMode = ejSunburstChart.sunburstHighlightMode;
    (function (sunburstSelectionMode) {
        sunburstSelectionMode[sunburstSelectionMode["point"] = "point"] = "point";
        sunburstSelectionMode[sunburstSelectionMode["all"] = "all"] = "all";
        sunburstSelectionMode[sunburstSelectionMode["child"] = "child"] = "child";
        sunburstSelectionMode[sunburstSelectionMode["parent"] = "parent"] = "parent";
    })(ejSunburstChart.sunburstSelectionMode || (ejSunburstChart.sunburstSelectionMode = {}));
    var sunburstSelectionMode = ejSunburstChart.sunburstSelectionMode;
    (function (sunburstLabelOverflowMode) {
        sunburstLabelOverflowMode[sunburstLabelOverflowMode["trim"] = "trim"] = "trim";
        sunburstLabelOverflowMode[sunburstLabelOverflowMode["hide"] = "hide"] = "hide";
        sunburstLabelOverflowMode[sunburstLabelOverflowMode["none"] = "none"] = "none";
    })(ejSunburstChart.sunburstLabelOverflowMode || (ejSunburstChart.sunburstLabelOverflowMode = {}));
    var sunburstLabelOverflowMode = ejSunburstChart.sunburstLabelOverflowMode;
    (function (sunburstLabelRotationMode) {
        sunburstLabelRotationMode[sunburstLabelRotationMode["angle"] = "angle"] = "angle";
        sunburstLabelRotationMode[sunburstLabelRotationMode["normal"] = "normal"] = "normal";
    })(ejSunburstChart.sunburstLabelRotationMode || (ejSunburstChart.sunburstLabelRotationMode = {}));
    var sunburstLabelRotationMode = ejSunburstChart.sunburstLabelRotationMode;
    (function (titleAlignment) {
        titleAlignment[titleAlignment["center"] = "center"] = "center";
        titleAlignment[titleAlignment["near"] = "near"] = "near";
        titleAlignment[titleAlignment["far"] = "far"] = "far";
    })(ejSunburstChart.titleAlignment || (ejSunburstChart.titleAlignment = {}));
    var titleAlignment = ejSunburstChart.titleAlignment;
    var SunburstChart = (function (_super) {
        __extends(SunburstChart, _super);
        function SunburstChart(id, options) {
            _super.call(this);
            this.defaults = {
                enableAnimation: false,
                load: null,
                preRender: null,
                loaded: null,
                dataLabelRendering: null,
                titleRendering: null,
                tooltipInitialize: null,
                pointRegionClick: null,
                drillDownClick: null,
                drillDownBack: null,
                drillDownReset: null,
                pointRegionMouseMove: null,
                legendItemRendering: null,
                legendItemClick: null,
                segmentRendering: null,
                animationType: sunburstAnimationType.rotation,
                palette: null,
                opacity: 1,
                valueMemberPath: null,
                margin: { top: 10, bottom: 10, left: 10, right: 10 },
                points: {
                    x: null,
                    y: null,
                    text: null,
                    fill: null
                },
                border: {
                    color: null,
                    width: 2
                },
                segmentBorder: {
                    color: null,
                    width: 2
                },
                startAngle: null,
                endAngle: null,
                dataSource: null,
                isResponsive: true,
                size: {
                    height: "",
                    width: ""
                },
                innerRadius: 0.4,
                radius: 1,
                tooltip: {
                    visible: false,
                    border: {
                        color: "#707070",
                        width: 1
                    },
                    fill: "#FFFFFF",
                    opacity: 0.95,
                    font: {
                        fontFamily: "Segoe UI",
                        fontStyle: "Normal",
                        fontWeight: "Regular",
                        color: "#707070",
                        opacity: 1,
                        size: "12px"
                    },
                    template: null,
                    format: "#point.x# : #point.y#",
                },
                dataLabelSettings: {
                    visible: false,
                    labelRotationMode: sunburstLabelRotationMode.angle,
                    font: {
                        fontFamily: "Segoe UI",
                        fontStyle: "Normal",
                        fontWeight: "Regular",
                        color: null,
                        opacity: 1,
                        size: "12px"
                    },
                    template: null,
                    fill: null,
                    labelOverflowMode: sunburstLabelOverflowMode.trim
                },
                title: {
                    text: "",
                    textAlignment: sunburstAlignment.center,
                    font: {
                        fontFamily: "Segoe UI",
                        fontWeight: "Regular",
                        fontStyle: "Normal",
                        color: null,
                        opacity: 1,
                        size: "20px"
                    },
                    subtitle: {
                        visible: true,
                        textAlignment: sunburstAlignment.far,
                        text: "",
                        font: {
                            fontFamily: "Segoe UI",
                            fontWeight: "Regular",
                            fontStyle: "Normal",
                            color: null,
                            opacity: 1,
                            size: "12px"
                        },
                    },
                    visible: true
                },
                zoomSettings: {
                    enable: false,
                    toolbarHorizontalAlignment: sunburstHorizontalAlignment.right,
                    toolbarVerticalAlignment: sunburstVerticalAlignment.top
                },
                highlightSettings: {
                    enable: false,
                    mode: sunburstHighlightMode.point,
                    opacity: 0.5,
                    color: "red",
                    type: sunburstHighlightType.opacity
                },
                selectionSettings: {
                    enable: false,
                    mode: sunburstSelectionMode.point,
                    opacity: 0.5,
                    color: "green",
                    type: sunburstSelectionType.opacity
                },
                levels: null,
                legend: {
                    clickAction: sunburstLegendClickAction.toggleSegmentVisibility,
                    visible: false,
                    position: sunburstLegendPosition.bottom,
                    rowCount: null,
                    columnCount: null,
                    shape: sunburstLegendShape.circle,
                    alignment: sunburstAlignment.center,
                    title: {
                        font: {
                            fontFamily: "Segoe UI",
                            fontWeight: "Regular",
                            fontStyle: "Normal",
                            color: null,
                            opacity: 1,
                            size: "12px"
                        },
                        text: "",
                        textAlignment: sunburstAlignment.center,
                        visible: true
                    },
                    itemStyle: {
                        height: 7.5,
                        width: 7.5,
                    },
                    itemPadding: 10,
                    size: {
                        height: null,
                        width: null
                    },
                    border: {
                        color: null,
                        width: 1
                    },
                    font: {
                        fontFamily: "Segoe UI",
                        fontWeight: "Regular",
                        fontStyle: "Normal",
                        color: null,
                        opacity: 1,
                        size: "12px"
                    },
                    location: {
                        x: null,
                        y: null
                    },
                },
                theme: sunburstTheme.flatlight,
                background: null
            };
            this.model = null;
            this.svgLink = "http://www.w3.org/2000/svg";
            this.elementSpacing = 10;
            this._id = null;
            this._legendHighlight = false;
            this.legendClicked = false;
            this._legendMaxWidth = 0;
            this._legendMaxHeight = 0;
            this._drillInnerRadius = [];
            this._drillOuterRadius = [];
            this.common = { data: null, cancel: false };
            this.legendPages = [];
            this.groupID = [];
            this.highlightgroupID = [];
            this.selectedgroupID = [];
            this.baseID = null;
            this._previousState = [];
            this._drillDownStartAngle = [];
            this._drillDownEndAngle = [];
            this._sunburstRedraw = false;
            this._isDoubleClickEvent = false;
            this.tapCount = 0;
            this.drilldownCount = 0;
            this.tapped = 0;
            this._points = [];
            this.visiblePoints = [];
            this.parentNames = [];
            this.levelNames = [];
            this.isResize = false;
            this.revAnimation = false;
            this.validTags = ['div'];
            this._id = id;
            if (!!options)
                this.model = ejSunburstChart.compareExtend({}, options, this.defaults);
        }
        SunburstChart.prototype._destroy = function () {
            var container;
            container = document.getElementById(this.container.id);
            if (!this.isNullOrUndefined(container)) {
                this.unBindEvents(this.parentElement);
                this.unbindResizeEvents();
                container.parentNode.removeChild(container);
            }
        };
        SunburstChart.prototype.unBindEvents = function (element) {
            var insideEvents = "", browserInfo = this.browserInfo(), isPointer = browserInfo.isMSPointerEnabled, isIE11Pointer = browserInfo.pointerEnabled, touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup", touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove";
            insideEvents = touchMoveEvent;
            insideEvents = insideEvents.split(" ");
            for (var event_1 in insideEvents) {
                element.removeEventListener(insideEvents[event_1], this.sunburstMousemove);
                element.removeEventListener(touchStopEvent, this.sunburstLeave);
            }
            element.removeEventListener('mouseout', this.sunburstLeave);
        };
        SunburstChart.prototype.unbindResizeEvents = function () {
            if (ej.isTouchDevice() && this._isOrientationSupported())
                window.removeEventListener("orientationchange", this.sunBurstResize);
            else
                window.removeEventListener('resize', this.sunBurstResize);
        };
        SunburstChart.prototype._setModel = function (options) {
            for (var prop in options) {
                switch (prop) {
                    default:
                        ejSunburstChart.deepExtend(true, this.model, {}, options[prop]);
                }
            }
            this.redraw();
        };
        SunburstChart.prototype.supportSVG = function () {
            return !!document.createElementNS && !!document.createElementNS('http://www.w3.org/2000/svg', "svg");
        };
        SunburstChart.prototype._init = function () {
            if (this.supportSVG()) {
                var element = void 0, svgElement = void 0, commonLoadEventArgs = void 0, commonpreRenderEventArgs = void 0, commonLoadedEventArgs = void 0;
                this._startAngle = this.model.startAngle;
                this._endAngle = this.model.endAngle;
                if (!this.isResize) {
                    this._enableAnimation = this.model.enableAnimation;
                    this._drillDownStartAngle = [];
                    this._drillDownEndAngle = [];
                    this._drillInnerRadius = [];
                    this._drillOuterRadius = [];
                }
                commonLoadEventArgs = $.extend({}, this.common);
                this._trigger("load", commonLoadEventArgs);
                this.parentElement = this.isNullOrUndefined(this.parentElement) ? document.getElementById(this._id) : this.parentElement;
                svgElement = this.createSvg({ 'id': this._id + '_svg' });
                this.container = svgElement;
                this.parentElement.appendChild(svgElement);
                this.setSvgsize(this.model.size, this._id);
                if (this._sunburstRedraw == false)
                    this.bindEvents(this.parentElement);
                if (!this.isResize) {
                    if (!this.isNullOrUndefined(this.model.dataSource) && this.model.dataSource.length > 0)
                        this.processDataSource(this.model.dataSource);
                    this.setColors(this.model.points);
                    this._previousState.push(this.model.points);
                    this.processData(this.model.points);
                }
                else
                    this.isResize = this.isResize ? false : true;
                commonpreRenderEventArgs = $.extend({}, this.common);
                commonpreRenderEventArgs['data'] = {};
                this._trigger("preRender", commonpreRenderEventArgs);
                this.drawSunburst(this.layerData);
                if (this._isDrillDown)
                    this.drawToolbar(svgElement);
                commonLoadedEventArgs = $.extend({}, this.common);
                commonLoadedEventArgs['data'] = { model: this.model };
                this._trigger("loaded", commonLoadedEventArgs);
            }
            else {
                alert("Sunburst will not be support in IE version < 8");
            }
        };
        SunburstChart.prototype.drawSunburst = function (layerData) {
            var rect, titleEle, sunburst, datalabelEle, size, title = this.model.title, legend = this.model.legend, dataLabel = this.model.dataLabelSettings, enableAnimation = this.model.enableAnimation;
            rect = this.drawContainerRect();
            document.getElementById(this._id + '_svg').appendChild(rect);
            if (legend['visible'] && !this.legendClicked) {
                this.calculateLegendBounds();
            }
            size = { width: this.width, height: this.height };
            if (title['text'] != "" && title['visible']) {
                titleEle = this.drawTitle(title, size);
                document.getElementById(this._id + '_svg').appendChild(titleEle);
            }
            this.areaBounds = this.calculateSize(size, title);
            sunburst = this.sunburstRender(layerData);
            document.getElementById(this._id + "_svg").appendChild(sunburst);
            if (this._enableAnimation) {
                var endAngle = void 0, length_1;
                this.groupingRegions(this.model.points);
                if (this.animationRegions.length > 0) {
                    if (this.model.animationType == sunburstAnimationType.rotation) {
                        if (!this._isDrillDown)
                            this.rotateAnimation(this, this.animationRegions);
                        else
                            this.rotateDrillDownAnimation(this, this.animationRegions);
                    }
                    else {
                        if (this.totallayerCount > 1) {
                            length_1 = this.animationRegions[0].length;
                            endAngle = this.animationRegions[0][length_1 - 1]['endAngle'];
                        }
                        else {
                            length_1 = this.animationRegions.length;
                            endAngle = this.animationRegions[length_1 - 1][0]['endAngle'];
                        }
                        this.animateLayer(this, sunburst, this.animationRegions[0], 0, 0, endAngle);
                    }
                }
            }
            if (dataLabel['visible']) {
                datalabelEle = this.drawDatalabel(layerData);
                if (dataLabel['template'] == null) {
                    if (this._enableAnimation)
                        datalabelEle.setAttribute('visibility', 'hidden');
                    document.getElementById(this._id + "_svg").appendChild(datalabelEle);
                }
                else {
                    if (this._enableAnimation)
                        document.getElementById(datalabelEle.id).style.visibility = 'hidden';
                }
                this.sunburstDoubleClick = this.sunburstDoubleClick.bind(this);
                document.getElementById(datalabelEle.id).addEventListener('dblclick', this.sunburstDoubleClick, true);
            }
            if (legend['visible']) {
                this.gLegendEle = this.createGroup({ id: this._id + "SunburstLegend" });
                if (this.model.legendInitialize) {
                    var commonEventArgs = $.extend({}, this.common);
                    this._trigger("legendInitialize", commonEventArgs);
                }
                this.drawSunburstLegend();
            }
        };
        SunburstChart.prototype.animateLayer = function (sunburst, sunburstEle, layer, pointInd, layerInd, totalendAngle) {
            var startAngle = layer[pointInd]['startAngle'], endAngle = layer[pointInd]['endAngle'], totalDegree, point, layerNumber, layerIndex, pointIndex, end, result, radius, path, start = startAngle, elementId, opacityRatio, opacity = 0, frameCount = 50, ct;
            totalDegree = (endAngle - startAngle) / Math.ceil(((frameCount * (endAngle - startAngle)) / totalendAngle));
            opacityRatio = (sunburst.model.opacity / 2) / Math.ceil(((frameCount * (endAngle - startAngle)) / totalendAngle));
            point = layer[pointInd];
            this.layeranim = setInterval(function () {
                layerNumber = point['layerNumber'];
                layerIndex = point['layerIndex'];
                pointIndex = point['pointIndex'];
                radius = sunburst.circularRadius[layerNumber - 1];
                elementId = point['id'];
                start = start + totalDegree;
                end = start;
                point['endAngle'] = end > endAngle ? endAngle : end;
                if (sunburst.isNullOrUndefined(sunburst.model))
                    clearInterval(sunburst.layeranim);
                else {
                    result = sunburst.calculateArcData(point, layerNumber, radius);
                    path = sunburstEle.querySelector('#' + elementId);
                    opacity += opacityRatio;
                    if (path) {
                        path.setAttribute('d', result['Direction']);
                        path.setAttribute('opacity', opacity.toString());
                    }
                    if (document.getElementById(elementId).getAttribute('visibility') == 'hidden') {
                        document.getElementById(elementId).setAttribute('visibility', 'visible');
                    }
                    if (end >= endAngle) {
                        clearInterval(sunburst.layeranim);
                        sunburst.fadeIn(path, opacity, sunburst, opacityRatio);
                        pointInd = pointInd + 1;
                        if (pointInd < layer.length) {
                            sunburst.animateLayer(sunburst, sunburstEle, layer, pointInd, layerInd, totalendAngle);
                        }
                        else {
                            pointInd = 0;
                            layerInd = layerInd + 1;
                            if (layerInd < sunburst.animationRegions.length)
                                sunburst.animateLayer(sunburst, sunburstEle, sunburst.animationRegions[layerInd], pointInd, layerInd, totalendAngle);
                            else {
                                if (sunburst.model.dataLabelSettings.visible)
                                    document.getElementById(sunburst._id + "_svg_DataLabelGroup").style.visibility = "visible";
                            }
                        }
                    }
                }
            }, 10);
        };
        SunburstChart.prototype.fadeIn = function (path, opacity, sunburst, opacityRatio) {
            var fadeIn = setInterval(function () {
                opacity = opacity + 0.1;
                path.setAttribute('opacity', opacity.toString());
                if (sunburst.model == null || opacity >= sunburst.model.opacity)
                    clearInterval(fadeIn);
            }, 50);
        };
        SunburstChart.prototype.rotateAnimation = function (sunburst, region) {
            var length = region.length, start = 0, end = 0, existingGroup, currentGroupPath, endAngle, startAngle, svg, scaleX = 0, transX, transY, angleDiff, scaleIncr, increaseAngle = 0;
            svg = document.getElementById(sunburst._id + "_svg");
            startAngle = sunburst.model.startAngle || 0;
            endAngle = sunburst.model.endAngle || 360;
            angleDiff = Math.abs(endAngle - startAngle);
            scaleIncr = 1 / (angleDiff / 10);
            end = (20 / 360) * angleDiff + startAngle;
            if (length > 0) {
                this.rotateanim = setInterval(function () {
                    if (sunburst.model) {
                        existingGroup = document.getElementById(sunburst._id + "_svg_SunBurstElementGroup");
                        if (existingGroup != null)
                            existingGroup.parentNode.removeChild(existingGroup);
                        sunburst.model.startAngle = startAngle;
                        sunburst.model.endAngle = end > endAngle ? endAngle : end;
                        sunburst._enableAnimation = false;
                        currentGroupPath = sunburst.sunburstRender(sunburst.layerData);
                        svg.appendChild(currentGroupPath);
                        scaleX = scaleX + scaleIncr;
                        scaleX = scaleX > 1 ? 1 : scaleX;
                        transX = (sunburst.width / 2) * (1 - scaleX);
                        transY = (sunburst.height / 2) * (1 - scaleX);
                        currentGroupPath.setAttribute("transform", "translate(" + transX + "," + transY + ") scale(" + scaleX + ")");
                        if (end >= endAngle) {
                            clearInterval(sunburst.rotateanim);
                            sunburst.rotateanim = undefined;
                            sunburst._enableAnimation = true;
                            sunburst.model.startAngle = startAngle;
                            sunburst.model.endAngle = endAngle;
                            if (sunburst.model.dataLabelSettings.visible) {
                                var dataLabel = document.getElementById(sunburst._id + "_svg_DataLabelGroup"), opacity = 0.1;
                                if (sunburst.model.dataLabelSettings.template == null)
                                    document.getElementById(sunburst._id + "_svg").appendChild(dataLabel);
                                else
                                    document.getElementById(sunburst._id).appendChild(dataLabel);
                                dataLabel.style.visibility = "visible";
                                dataLabel.style.opacity = opacity.toString();
                                sunburst.datalabelanim = setInterval(function () {
                                    dataLabel.style.opacity = opacity.toString();
                                    opacity = opacity + 0.05;
                                    if (opacity > 1)
                                        clearInterval(sunburst.datalabelanim);
                                }, 10);
                            }
                        }
                        end = end + (10 - increaseAngle);
                        increaseAngle = increaseAngle + 0.1;
                        scaleIncr = 1 / (angleDiff / 10 - increaseAngle);
                    }
                    else
                        clearInterval(sunburst.rotateanim);
                }, 25);
            }
        };
        SunburstChart.prototype.rotateDrillDownAnimation = function (sunburst, region) {
            var length = region.length, start = 0, end = 0, existingGroup, currentGroupPath, endAngle, startAngle, svg, increaseAngle = 0, animationStartAngle = 0, increaseStartAngle, increaseEndAngle, interval = 1, increaseInterval = 0.05, animationEndAngle = 0;
            svg = document.getElementById(sunburst._id + "_svg");
            startAngle = sunburst.model.startAngle || 0;
            endAngle = sunburst.model.endAngle || 360;
            start = animationStartAngle = sunburst._drillDownStartAngle[sunburst.drilldownCount - 1];
            end = animationEndAngle = sunburst._drillDownEndAngle[sunburst.drilldownCount - 1];
            increaseStartAngle = (start - startAngle) / 20;
            increaseEndAngle = (endAngle - end) / 20;
            if (length > 0) {
                var animate_1 = setInterval(function () {
                    existingGroup = document.getElementById(sunburst._id + "_svg_SunBurstElementGroup");
                    existingGroup.parentNode.removeChild(existingGroup);
                    sunburst.model.startAngle = animationStartAngle < startAngle ? startAngle : animationStartAngle;
                    sunburst.model.endAngle = animationEndAngle > endAngle ? endAngle : animationEndAngle;
                    sunburst._enableAnimation = false;
                    currentGroupPath = sunburst.sunburstRender(sunburst.layerData);
                    svg.appendChild(currentGroupPath);
                    if (animationStartAngle <= startAngle && animationEndAngle >= endAngle) {
                        clearInterval(animate_1);
                        sunburst._enableAnimation = true;
                        sunburst.model.startAngle = startAngle;
                        sunburst.model.endAngle = endAngle;
                        if (sunburst.model.dataLabelSettings.visible) {
                            var dataLabel = document.getElementById(sunburst._id + "_svg_DataLabelGroup"), opacity = 0.1;
                            if (sunburst.model.dataLabelSettings.template == null)
                                document.getElementById(sunburst._id + "_svg").appendChild(dataLabel);
                            else
                                document.getElementById(sunburst._id).appendChild(dataLabel);
                            dataLabel.style.visibility = "visible";
                            dataLabel.style.opacity = opacity.toString();
                            var dataLabelAnimate_1 = setInterval(function () {
                                dataLabel.style.opacity = opacity.toString();
                                opacity = opacity + 0.05;
                                if (opacity > 1)
                                    clearInterval(dataLabelAnimate_1);
                            }, 10);
                        }
                    }
                    interval = interval + (1 - increaseInterval);
                    increaseInterval = increaseInterval + 0.015;
                    increaseInterval = increaseInterval > 0.7 ? 0.7 : increaseInterval;
                    animationStartAngle = start - (increaseStartAngle * interval);
                    animationEndAngle = end + (increaseEndAngle * interval);
                }, 25);
            }
        };
        SunburstChart.prototype.groupingRegions = function (points) {
            var regions = this.sunburstRegions, elements, layer, parentRegion = this.sunburstRegions[this.sunburstRegions.length - 1]['region'], layerData, layerNumber, layers = [], groupNumber;
            this.animationRegions = [];
            layers.push(parentRegion);
            for (var i = 0; i < parentRegion.length; i++) {
                if (regions.length > 1) {
                    elements = [];
                    layer = this.layerData[parentRegion[i]['layerIndex']]['point'][parentRegion[i]['pointIndex']];
                    groupNumber = layer['groupNumber'];
                    if (!this.isNullOrUndefined(layer['point'])) {
                        this.findChildRegions(layer['point'], elements, layer['layerNumber'], layers, groupNumber);
                    }
                    if (this.model.animationType == sunburstAnimationType.rotation) {
                        elements.splice(0, 0, layer);
                        layer['count'] = 0;
                        layer['groupLength'] = elements.length;
                        this.animationRegions.push(elements);
                    }
                    else {
                        this.animationRegions = layers;
                    }
                }
                else {
                    elements = [];
                    elements.push(parentRegion[i]);
                    this.animationRegions.push(elements);
                }
            }
        };
        SunburstChart.prototype.findChildRegions = function (layer, elements, layerNumber, layers, groupNumber) {
            for (var i = 0; i < layer['length']; i++) {
                layer[i]['pointIndex'] = i;
                layer[i]['layerNumber'] = this.layerData[layer['layerData']]['layerNumber'];
                layer[i]['groupNumber'] = groupNumber;
                layer[i]['count'] = 0;
                if (this.model.animationType == sunburstAnimationType.rotation)
                    elements.push(layer[i]);
                else {
                    if (this.isNullOrUndefined(layers[layer[i]['layerNumber'] - 1]))
                        layers[layer[i]['layerNumber'] - 1] = [];
                    layers[layer[i]['layerNumber'] - 1].push(layer[i]);
                }
                if (!this.isNullOrUndefined(layer[i]['point'])) {
                    this.findChildRegions(layer[i]['point'], elements, layerNumber, layers, groupNumber);
                }
            }
        };
        SunburstChart.prototype.bindEvents = function (element) {
            var insideEvents = "", mouseMove = "", browserInfo = this.browserInfo(), isDevice = ej.isDevice(), isPointer = browserInfo.isMSPointerEnabled, isIE11Pointer = browserInfo.pointerEnabled, touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup", touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove";
            insideEvents = touchMoveEvent;
            insideEvents = insideEvents.split(" ");
            this.sunburstMousemove = this.sunburstMousemove.bind(this);
            this.sunburstClick = this.sunburstClick.bind(this);
            element.addEventListener('click', this.sunburstClick, true);
            this.sunburstLeave = this.sunburstLeave.bind(this);
            for (var event_2 in insideEvents) {
                element.addEventListener(touchStopEvent, this.sunburstLeave);
                mouseMove = insideEvents[event_2];
                if (isDevice && mouseMove == "mousemove")
                    continue;
                element.addEventListener(mouseMove, this.sunburstMousemove);
            }
            if (isDevice)
                element.addEventListener("mousedown", this.sunburstMousemove);
            element.addEventListener('mouseout', this.sunburstLeave, true);
            if (this.getIEversion() != -1) {
                this.sunburstDoubletap = this.sunburstDoubletap.bind(this);
                element.addEventListener('pointerdown', this.sunburstDoubletap, true);
            }
            if (this.model.isResponsive)
                this.bindResizeEvents();
        };
        SunburstChart.prototype.browserInfo = function () {
            var browser = {}, clientInfo = [], browserClients = {
                webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
                opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
            };
            for (var client in browserClients) {
                if (browserClients.hasOwnProperty(client)) {
                    clientInfo = navigator.userAgent.match(browserClients[client]);
                    if (clientInfo) {
                        browser.name = clientInfo[1].toLowerCase();
                        browser.version = clientInfo[2];
                        if (!!navigator.userAgent.match(/Trident\/7\./)) {
                            browser.name = "msie";
                        }
                        break;
                    }
                }
            }
            browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
            browser.pointerEnabled = window.navigator.pointerEnabled;
            return browser;
        };
        SunburstChart.prototype.sunburstDoubletap = function (event) {
            var targetid = event.target['id'], totalnum;
            if (targetid.indexOf("reset") == -1 && targetid.indexOf("back") == -1) {
                event.preventDefault();
                this.tapped = this.tapped + 1;
                totalnum = this.tapped;
            }
        };
        SunburstChart.prototype.bindResizeEvents = function () {
            this.sunBurstResize = this.sunBurstResize.bind(this);
            if (ej.isTouchDevice() && this._isOrientationSupported())
                window.addEventListener("orientationchange", this.sunBurstResize, true);
            else
                window.addEventListener('resize', this.sunBurstResize.bind(this), true);
        };
        SunburstChart.prototype._isOrientationSupported = function () {
            return ("orientation" in window && "onorientationchange" in window);
        };
        SunburstChart.prototype.sunBurstResize = function (element) {
            var sunburst = this;
            var $svgObj = $(sunburst.container);
            if (this._resizeTO)
                clearTimeout(this._resizeTO);
            this._resizeTO = setTimeout(function () {
                if (sunburst.model) {
                    sunburst.isResize = true;
                    if (!sunburst.isNullOrUndefined(document.getElementById(sunburst._id + 'reset'))) {
                        sunburst._isDrillDown = true;
                    }
                    sunburst.legendClicked = true;
                    sunburst._enableAnimation = false;
                    sunburst.redraw();
                    sunburst._enableAnimation = sunburst.model.enableAnimation;
                    sunburst._isDrillDown = false;
                    sunburst.legendClicked = false;
                }
            }, 500);
        };
        SunburstChart.prototype.getIEversion = function () {
            var rv = -1, ua, re;
            if (navigator.appName == 'Microsoft Internet Explorer') {
                ua = navigator.userAgent;
                re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
                if (re.exec(ua) != null)
                    rv = parseFloat(RegExp.$1);
            }
            else if (navigator.appName == 'Netscape') {
                ua = navigator.userAgent;
                re = new RegExp("Trident/.*rv:([0-9]{1,}[\.0-9]{0,})");
                if (re.exec(ua) != null)
                    rv = parseFloat(RegExp.$1);
            }
            return rv;
        };
        SunburstChart.prototype.sunburstDoubleClick = function (event) {
            var data;
            if (this.model.zoomSettings.enable) {
                data = this.getSunburstPoint(event);
                if (!this.isNullOrUndefined(data)) {
                    if (data['pointData'][0]['layerNumber'] == 1) {
                        if (this.drilldownCount == 0) {
                            this.drillDown(data);
                        }
                    }
                    else {
                        this.drillDown(data);
                    }
                }
            }
        };
        SunburstChart.prototype.drillDown = function (data) {
            var layer, point = [], option, drillDownData, element, commonEventArgs, svgElement;
            element = document.getElementById(this._id + '_svg');
            element.parentNode.replaceChild(element.cloneNode(false), element);
            layer = this.layerData[data.pointData[0].layerIndex]['point'][data.pointData[0].pointIndex];
            this.drilldownCount = this.drilldownCount + 1;
            this._drillDownStartAngle.push((layer['startAngle'] * 180) / Math.PI);
            this._drillDownEndAngle.push((layer['endAngle'] * 180) / Math.PI);
            this._isDrillDown = true;
            this._isDoubleClickEvent = true;
            if (this.model.enableAnimation && this.model.animationType == sunburstAnimationType.rotation) {
                var layerNumber = data.pointData[0].layerNumber;
                if (layerNumber == 1) {
                    this._drillInnerRadius.push(this.innerRadius);
                    this._drillOuterRadius.push(this.circularRadius);
                }
                else {
                    var inRadius = this.innerRadius.slice(layerNumber - 1, this.innerRadius.length);
                    var outrRadius = this.circularRadius.slice(layerNumber - 1, this.circularRadius.length);
                    this._drillInnerRadius.push(inRadius);
                    this._drillOuterRadius.push(outrRadius);
                }
            }
            drillDownData = { 'fill': data['pointData'][0].fill, 'x': data['pointData'][0].x, 'y': data['pointData'][0].y, 'point': layer['point'], 'text': data['pointData'][0].text };
            if (this.model.drillDownClick) {
                commonEventArgs = $.extend({}, this.common);
                commonEventArgs['data'] = drillDownData;
                this._trigger("drillDownClick", commonEventArgs);
            }
            point.push(drillDownData);
            this._previousState.push(point);
            this.processData(point);
            this.drawSunburst(this.layerData);
            this._isDrillDown = false;
            if (this.isNullOrUndefined(document.getElementById(this._id + 'reset'))) {
                svgElement = document.getElementById(this._id + '_svg');
                this.drawToolbar(svgElement);
            }
        };
        SunburstChart.prototype.drawToolbar = function (svgElement) {
            var backGroup, resetGroup, svgBounds;
            svgBounds = svgElement.getBoundingClientRect();
            resetGroup = this.drawResetButton(svgBounds);
            svgElement.appendChild(resetGroup);
            this.reset = this.reset.bind(this);
            document.getElementById(this._id + "reset").addEventListener('click', this.reset, true);
            backGroup = this.drawBackButton(svgBounds);
            svgElement.appendChild(backGroup);
            this.back = this.back.bind(this);
            document.getElementById(this._id + "back").addEventListener('click', this.back, true);
            if (!ej.isDevice()) {
                document.getElementById(this._id + "back").addEventListener('mousemove', this.sunburstMousemove, true);
                document.getElementById(this._id + "reset").addEventListener('mousemove', this.sunburstMousemove, true);
            }
            this.toolbarPositioning(backGroup, resetGroup);
        };
        SunburstChart.prototype.toolbarPositioning = function (backGroup, resetGroup) {
            var translate, resettransX, backtransX, transY, resetOptions, backOptions, elementSpacing = this.elementSpacing, tbhorizontalalignment = this.model.zoomSettings.toolbarHorizontalAlignment, tbverticalalignment = this.model.zoomSettings.toolbarVerticalAlignment;
            backOptions = backGroup.getBoundingClientRect();
            resetOptions = resetGroup.getBoundingClientRect();
            if (tbhorizontalalignment == sunburstHorizontalAlignment.right) {
                resettransX = this.width - elementSpacing - resetOptions['width'];
                backtransX = this.width - (2 * elementSpacing) - (2 * resetOptions['width']);
            }
            else if (tbhorizontalalignment == sunburstHorizontalAlignment.left) {
                backtransX = elementSpacing;
                resettransX = (backOptions['width']) + (2 * elementSpacing);
            }
            else if (tbhorizontalalignment == sunburstHorizontalAlignment.center) {
                backtransX = (this.width / 2) - ((backOptions['width'] + elementSpacing + resetOptions['width']) / 2);
                resettransX = backtransX + elementSpacing + backOptions['width'];
            }
            if (tbverticalalignment == sunburstVerticalAlignment.top) {
                transY = elementSpacing;
            }
            else if (tbverticalalignment == sunburstVerticalAlignment.bottom) {
                transY = this.height - elementSpacing - backOptions['height'];
            }
            else if (tbverticalalignment == sunburstVerticalAlignment.middle) {
                transY = (this.height / 2) - (backOptions['height'] / 2);
            }
            translate = 'translate(' + resettransX.toString() + ',' + transY.toString() + ')';
            resetGroup.setAttribute('transform', translate);
            translate = 'translate(' + backtransX.toString() + ',' + transY.toString() + ')';
            backGroup.setAttribute('transform', translate);
        };
        SunburstChart.prototype.drawBackButton = function (rect) {
            var options, element, backGroup;
            backGroup = this.createGroup({ 'id': this._id + 'back' });
            options = {
                'id': this._id + 'backPath',
                'd': "M24,27H3c-1.7,0-3-1.3-3-3V3c0-1.7,1.3-3,3-3h21c1.7,0,3,1.3,3,3v21C27,25.7,25.7,27,24,27z",
                'fill': '#D6D6D6',
            };
            element = this.drawPath(options);
            backGroup.appendChild(element);
            options = {
                'id': this._id + 'backPath1',
                'x': 7.7,
                'y': 12.5,
                'width': 13.8,
                'height': 2,
                'fill': "#5E5C5C"
            };
            element = this.drawRect(options);
            backGroup.appendChild(element);
            options = {
                'id': this._id + 'backPath2',
                'points': "11.4,19.8 5.5,13.5 11.4,7.2 12.9,8.6 8.2,13.5 12.9,18.4",
                'fill': "#5E5C5C",
            };
            element = this.drawPolygon(options);
            backGroup.appendChild(element);
            return backGroup;
        };
        SunburstChart.prototype.back = function (event) {
            var layerData = this.layerData, element = document.getElementById(this._id + '_svg'), backEle = document.getElementById(this._id + "back"), resetEle = document.getElementById(this._id + "reset"), model = this.model, rootEle, commonDrillDownBackEventArgs;
            if (!this.revAnimation) {
                if (model['drillDownBack']) {
                    commonDrillDownBackEventArgs = $.extend({}, this.common);
                    commonDrillDownBackEventArgs['data'] = layerData;
                    this._trigger("drillDownBack", commonDrillDownBackEventArgs);
                }
                if (this._enableAnimation && model['animationType'] == sunburstAnimationType.rotation) {
                    this._enableAnimation = false;
                    this.reverseAnimation(this, this.animationRegions);
                }
                else {
                    element.parentNode.replaceChild(element.cloneNode(false), element);
                    var len = this._previousState.length - 1;
                    this.layerData = this._previousState[len - 1];
                    this._previousState.splice(len, 1);
                    this.drilldownCount = this.drilldownCount - 1;
                    this.processData(this.layerData);
                    this.drawSunburst(this.layerData);
                    if (this.drilldownCount != 0) {
                        rootEle = document.getElementById(this._id + "_svg");
                        rootEle.appendChild(resetEle);
                        rootEle.appendChild(backEle);
                    }
                }
            }
        };
        SunburstChart.prototype.reverseAnimation = function (sunburst, region) {
            var length = region.length, start = 0, end = 0, existingGroup, currentGroupPath, endAngle, startAngle, svg, sAngle, eAngle, increaseStartAngle, increaseEndAngle, interval = 0, dataLabel, backEle = document.getElementById(sunburst._id + "back"), resetEle = document.getElementById(sunburst._id + "reset"), pieSegmentGroup, tempLabel, len, container, opacity = 0.1;
            svg = document.getElementById(sunburst._id + "_svg");
            startAngle = sunburst.model.startAngle || 0;
            endAngle = sunburst.model.endAngle || 360;
            start = sunburst._drillDownStartAngle[sunburst.drilldownCount - 1], end = sunburst._drillDownEndAngle[sunburst.drilldownCount - 1];
            increaseStartAngle = (start - startAngle) / 20;
            increaseEndAngle = (endAngle - end) / 20;
            dataLabel = document.getElementById(sunburst._id + "_svg_DataLabelGroup");
            if (length > 0) {
                var animate_2 = setInterval(function () {
                    sunburst.revAnimation = true;
                    existingGroup = document.getElementById(sunburst._id + "_svg_SunBurstElementGroup");
                    if (existingGroup)
                        existingGroup.parentNode.removeChild(existingGroup);
                    if (dataLabel)
                        dataLabel.style.visibility = "hidden";
                    sAngle = increaseStartAngle * interval + startAngle;
                    sunburst.model.startAngle = sAngle >= start ? start : sAngle;
                    eAngle = endAngle - (increaseEndAngle * interval);
                    sunburst.model.endAngle = eAngle < end ? end : eAngle;
                    currentGroupPath = sunburst.sunburstRender(sunburst.layerData);
                    svg.appendChild(currentGroupPath);
                    if (eAngle <= end && sAngle >= start) {
                        clearInterval(animate_2);
                        pieSegmentGroup = document.getElementById(sunburst._id + "_svg_SunBurstElementGroup");
                        pieSegmentGroup.id = sunburst._id + "_svg_tempSeg";
                        tempLabel = document.getElementById(sunburst._id + "_svg_DataLabelGroup");
                        svg.parentNode.replaceChild(svg.cloneNode(false), svg);
                        document.getElementById(sunburst._id + "_svg").appendChild(pieSegmentGroup);
                        if (tempLabel) {
                            tempLabel.id = sunburst._id + "_svg_tempLabel";
                            document.getElementById(sunburst._id + "_svg").appendChild(tempLabel);
                        }
                        len = sunburst._previousState.length - 1;
                        sunburst.layerData = sunburst._previousState[len - 1];
                        sunburst._previousState.splice(len, 1);
                        sunburst.model.startAngle = startAngle;
                        sunburst.model.endAngle = endAngle;
                        sunburst.drilldownCount = sunburst.drilldownCount - 1;
                        sunburst._drillDownStartAngle.length = sunburst.drilldownCount;
                        sunburst._drillDownEndAngle.length = sunburst.drilldownCount;
                        sunburst._drillInnerRadius.length = sunburst.drilldownCount;
                        sunburst._drillOuterRadius.length = sunburst.drilldownCount;
                        sunburst.processData(sunburst.layerData);
                        sunburst.drawSunburst(sunburst.layerData);
                        if (sunburst.drilldownCount != 0) {
                            svg = document.getElementById(sunburst._id + "_svg");
                            svg.appendChild(resetEle);
                            svg.appendChild(backEle);
                        }
                        sunburst._enableAnimation = true;
                        container = document.getElementById(sunburst._id + "_svg_SunBurstElementGroup");
                        container.style.opacity = opacity.toString();
                        if (dataLabel)
                            dataLabel.style.opacity = opacity.toString();
                        if (sunburst.model.dataLabelSettings.visible) {
                            dataLabel = document.getElementById(sunburst._id + "_svg_DataLabelGroup");
                            if (sunburst.model.dataLabelSettings.template == null)
                                document.getElementById(sunburst._id + "_svg").appendChild(dataLabel);
                            else
                                document.getElementById(sunburst._id).appendChild(dataLabel);
                            dataLabel.style.visibility = "visible";
                            dataLabel.style.opacity = opacity.toString();
                        }
                        var animateByOpacity_1 = setInterval(function () {
                            container.style.opacity = opacity.toString();
                            if (dataLabel)
                                dataLabel.style.opacity = opacity.toString();
                            if (opacity >= 1) {
                                clearInterval(animateByOpacity_1);
                                sunburst.revAnimation = false;
                                pieSegmentGroup.parentNode.removeChild(pieSegmentGroup);
                                if (tempLabel)
                                    tempLabel.parentNode.removeChild(tempLabel);
                            }
                            opacity = opacity + 0.1;
                        }, 50);
                    }
                    interval = interval + 1;
                }, 40);
            }
        };
        SunburstChart.prototype.drawResetButton = function (rect) {
            var options, path, resetGroup, polygon;
            resetGroup = this.createGroup({ 'id': this._id + 'reset' });
            options = {
                'id': this._id + 'resetPath',
                'd': "M24,27H3c-1.7,0-3-1.3-3-3V3c0-1.7,1.3-3,3-3h21c1.7,0,3,1.3,3,3v21C27,25.7,25.7,27,24,27z",
                'fill': '#D6D6D6',
            };
            path = this.drawPath(options);
            resetGroup.appendChild(path);
            options = {
                'id': this._id + 'resetPath1',
                'd': "M13.6,21.9c-3.6,0-6.8-2.3-8-5.7l1.8-0.6c0.9,2.6,3.4,4.4,6.2,4.4c3.4,0,6.2-2.6,6.5-6l1.9,0.2   C21.6,18.6,18,21.9,13.6,21.9z",
                'fill': "#5E5C5C"
            };
            path = this.drawPath(options);
            resetGroup.appendChild(path);
            options = {
                'id': this._id + 'resetPath2',
                'd': "M7.1,12.5l-1.9-0.3c0.6-4.1,4.2-7.1,8.3-7.1c3,0,5.8,1.6,7.3,4.2c0.1,0.2,0.2,0.4,0.3,0.6l-1.7,0.8   c-0.1-0.2-0.2-0.3-0.2-0.4c-1.2-2-3.3-3.3-5.7-3.3C10.4,7,7.6,9.3,7.1,12.5z",
                'fill': "#5E5C5C"
            };
            path = this.drawPath(options);
            resetGroup.appendChild(path);
            options = {
                'id': this._id + 'resetPath3',
                'points': "21.4,11.9 16.2,10.7 16.6,8.9 19.9,9.6 20.7,6.3 22.5,6.7",
                'fill': "#5E5C5C"
            };
            polygon = this.drawPolygon(options);
            resetGroup.appendChild(polygon);
            options = {
                'id': this._id + 'resetPath4',
                'points': "6.3,20.3 4.5,19.9 5.6,14.8 10.8,16 10.4,17.8 7.1,17.1",
                'fill': "#5E5C5C"
            };
            polygon = this.drawPolygon(options);
            resetGroup.appendChild(polygon);
            return resetGroup;
        };
        SunburstChart.prototype.reset = function (event) {
            if (this.model.drillDownReset) {
                var commonDrillDownResetEventArgs = $.extend({}, this.common);
                this._trigger("drillDownReset", commonDrillDownResetEventArgs);
            }
            this.redraw();
        };
        SunburstChart.prototype.redraw = function () {
            this.legendPages = [];
            if (!this.isResize) {
                this.drilldownCount = 0;
                this._previousState = [];
                this._drillDownStartAngle = [];
                this._drillDownEndAngle = [];
                this._drillInnerRadius = [];
                this._drillOuterRadius = [];
            }
            this._sunburstRedraw = true;
            var element = document.getElementById(this._id + "_svg");
            if (!this.isNullOrUndefined(element)) {
                element.parentNode.removeChild(element);
                if (this.rotateanim || this.model.enableAnimation) {
                    this.stopAnimation();
                }
                this.parentElement = undefined;
                this._init();
                this._sunburstRedraw = false;
            }
        };
        SunburstChart.prototype.stopAnimation = function () {
            if (this.rotateanim) {
                clearInterval(this.rotateanim);
                if (!this.isResize)
                    this._enableAnimation = this.model.enableAnimation;
                this.model.startAngle = this._startAngle;
                this.model.endAngle = this._endAngle;
                if (this.datalabelanim)
                    clearInterval(this.datalabelanim);
            }
            if (this.layeranim)
                clearInterval(this.layeranim);
        };
        SunburstChart.prototype.sunburstMousemove = function (event) {
            var elementId = event.target.id, element, data, commonPointRegionMouseMoveEventArgs, transToolTipOptions, location, layerPoint, layer;
            if (elementId.indexOf('reset') != -1 || elementId.indexOf('back') != -1) {
                this.toolbarTooltip(event, elementId);
                if (elementId.indexOf('reset') != -1) {
                    element = document.getElementById(this._id + "reset");
                    for (var i = 0; i < 5; i++) {
                        if (i == 0)
                            document.getElementById(this._id + "resetPath").setAttribute("fill", "rgb(20, 185, 255)");
                        else
                            document.getElementById(this._id + "resetPath" + i.toString()).setAttribute("fill", "#FFFFFF");
                    }
                }
                else {
                    for (var i = 0; i < 3; i++) {
                        if (i == 0)
                            document.getElementById(this._id + "backPath").setAttribute("fill", "rgb(20, 185, 255)");
                        else
                            document.getElementById(this._id + "backPath" + i.toString()).setAttribute("fill", "#FFFFFF");
                    }
                }
            }
            else {
                if (document.getElementById(this._id + "toolbarTooltip") != null && document.getElementById(this._id + "toolbarTooltip").style.visibility != "hidden") {
                    document.getElementById(this._id + "toolbarTooltip").style.visibility = "hidden";
                    if (document.getElementById(this._id + "reset") != null && document.getElementById(this._id + "back") != null) {
                        for (var i = 0; i < 5; i++) {
                            if (i == 0) {
                                document.getElementById(this._id + "resetPath").setAttribute("fill", "#D6D6D6");
                            }
                            else
                                document.getElementById(this._id + "resetPath" + i.toString()).setAttribute("fill", "#5E5C5C");
                        }
                        for (var i = 0; i < 3; i++) {
                            if (i == 0) {
                                document.getElementById(this._id + "backPath").setAttribute("fill", "#D6D6D6");
                            }
                            else
                                document.getElementById(this._id + "backPath" + i.toString()).setAttribute("fill", "#5E5C5C");
                        }
                    }
                }
            }
            data = this.getSunburstPoint(event);
            if (data != undefined && this.model.pointRegionMouseMove) {
                commonPointRegionMouseMoveEventArgs = $.extend({}, this.common);
                commonPointRegionMouseMoveEventArgs['data'] = data;
                this._trigger("pointRegionMouseMove", commonPointRegionMouseMoveEventArgs);
            }
            if (!this.isNullOrUndefined(data) && this.model.tooltip.visible) {
                transToolTipOptions = { 'id': this._id + '_ToolTip' };
                this.gTransToolEle = this.createGroup(transToolTipOptions);
                location = { X: (data['location'].x), Y: Math.abs(data['location'].y) };
                for (var i = 0; i < this.layerData.length; i++) {
                    if (this.layerData[i]['layerNumber'] == data['pointData'][0].layerNumber) {
                        if (this.isNullOrUndefined(data['pointData'][0].parentName)) {
                            layer = this.layerData[0];
                            layerPoint = this.layerData[0]['point'][data['pointData'][0].pointIndex];
                        }
                        else {
                            if (this.layerData[i]['parentName'] == data['pointData'][0].parentName) {
                                layer = this.layerData[i];
                                layerPoint = this.layerData[i]['point'][data['pointData'][0].pointIndex];
                            }
                        }
                    }
                }
                if (this.model.tooltip.template) {
                    this.createTooltiptemplate(location, layerPoint, layer, data);
                }
                else {
                    this.showTooltip(location, layerPoint, layer, data);
                }
            }
            else {
                if (!this.isNullOrUndefined(document.getElementById(this._id + "_ToolTip"))) {
                    document.getElementById(this._id + "_ToolTip").style.visibility = "hidden";
                }
            }
            if (this.model.highlightSettings.enable) {
                var legendItem = void 0, highlightSettings = this.model.highlightSettings;
                this.highlightSunburst(data);
                if (!this.isNullOrUndefined(data) && this.model.legend.visible) {
                    for (var j = 0; j < this.model.points.length; j++) {
                        legendItem = document.getElementById(this._id + "_svg_LegendItemShape" + j);
                        if (highlightSettings['type'] == sunburstHighlightType.opacity) {
                            if (this.selectedGroupNumber - 1 != j) {
                                if (data['pointData'][0].legendIndex != j) {
                                    if (!this.isNullOrUndefined(legendItem))
                                        legendItem.style.opacity = highlightSettings['opacity'].toString();
                                }
                            }
                        }
                        else {
                            if (data['pointData'][0].legendIndex == j) {
                                if (legendItem.getAttribute('class') != "SelectionStyleSunburst") {
                                    legendItem.setAttribute("class", "HighlightStyleSunburst");
                                    legendItem.style.opacity = highlightSettings['opacity'].toString();
                                }
                            }
                            else {
                                if (!(legendItem.getAttribute("class") == "HighlightStyleSunburst"))
                                    legendItem.removeAttribute("class");
                            }
                        }
                    }
                }
            }
            if (elementId.indexOf(this._id + "_svg_LegendItem") != -1 && this.model.legend.clickAction == sunburstLegendClickAction.toggleSegmentSelection) {
                this.legendSegments(elementId);
            }
        };
        SunburstChart.prototype.legendSegments = function (elementid) {
            var legendId, legendIndex, legendSegment, childNodes, highlightOpacity, pieSegment, dataLabelGroup;
            legendId = elementid.replace(this._id + "_svg", "");
            legendIndex = parseInt(legendId.replace(/[^0-9\.]/g, ''));
            legendSegment = document.getElementById(this._id + "legendItem_Group");
            childNodes = legendSegment.childNodes;
            highlightOpacity = this.model.highlightSettings.opacity.toString();
            for (var l = 0; l < childNodes.length; l++) {
                if (legendIndex != l)
                    document.getElementById(childNodes[l]['id']).style.opacity = highlightOpacity;
                else
                    document.getElementById(childNodes[l]['id']).style.opacity = "1";
            }
            pieSegment = document.getElementById(this._id + '_svg_SunBurstElementGroup');
            childNodes = pieSegment.childNodes;
            for (var j = 0; j < childNodes.length; j++) {
                if (childNodes[j]['id'].indexOf("_legendIndex" + legendIndex) == -1)
                    document.getElementById(childNodes[j]['id']).style.opacity = highlightOpacity;
            }
            if (this.model.dataLabelSettings.visible) {
                dataLabelGroup = document.getElementById(this._id + '_svg_DataLabelGroup');
                if (dataLabelGroup) {
                    childNodes = dataLabelGroup.childNodes;
                    for (var j = 0; j < childNodes.length; j++) {
                        if (childNodes[j]['id'].indexOf("_legendIndex" + legendIndex) == -1)
                            document.getElementById(childNodes[j]['id']).style.opacity = highlightOpacity;
                    }
                }
            }
            this._legendHighlight = true;
        };
        SunburstChart.prototype.toolbarTooltip = function (event, elementid) {
            var tooltipdiv, rectoptions;
            if (elementid.indexOf("reset") != -1) {
                elementid = 'Reset';
            }
            else
                elementid = 'Back';
            rectoptions = {
                'top': event['clientY'],
                'left': event['clientX'],
                'background-color': "white",
                "border-style": "solid",
                "position": "absolute",
                "border-color": "#2F4F4F",
                "border-width": "1px",
                "opacity": 0.95,
                'z-index': 1000000,
                "border-radius": "0px 0px",
                "padding-left": '5px',
                "padding-right": '5px',
                "padding-top": '2px',
                "padding-bottom": '2px'
            };
            if (this.isNullOrUndefined(document.getElementById(this._id + "toolbarTooltip"))) {
                tooltipdiv = document.createElement('div');
                tooltipdiv.setAttribute('id', this._id + "toolbarTooltip");
            }
            else {
                tooltipdiv = document.getElementById(this._id + "toolbarTooltip");
            }
            tooltipdiv['style'].pointerEvents = "none";
            this.setStyles(rectoptions, tooltipdiv);
            var textOptions = {
                'top': event['clientY'],
                'left': event['clientX'],
                "color": this.model.tooltip.font.color,
                'font-size': this.model.tooltip.font.size,
                'font-family': this.model.tooltip.font.fontFamily,
                'font-style': this.model.tooltip.font.fontStyle,
                'font-weight': this.model.tooltip.font.fontWeight,
                "align-self": "baseline"
            };
            this.setStyles(textOptions, tooltipdiv);
            if (this.isNullOrUndefined(document.getElementById(this._id + "toolbarTooltip"))) {
                document.body.appendChild(tooltipdiv);
            }
            tooltipdiv.innerHTML = elementid;
            if (tooltipdiv['style'].visibility == "hidden") {
                tooltipdiv['style'].visibility = "visible";
            }
            tooltipdiv['style'].left = (event['clientX'] + window.pageXOffset + 10).toString() + 'px';
            tooltipdiv['style'].top = (event['clientY'] + window.pageYOffset + 10).toString() + 'px';
        };
        SunburstChart.prototype.removeHighlight = function (ID) {
            for (var i = 0; i < ID.length && document.getElementById(ID[i]) != null; i++) {
                if (document.getElementById(ID[i]).getAttribute('class') == "HighlightStyleSunburst")
                    document.getElementById(ID[i]).removeAttribute('class');
            }
        };
        SunburstChart.prototype.isSelectedElement = function (elementId, selectedgrpId) {
            var selected = false;
            for (var i = 0; i < selectedgrpId.length; i++) {
                if (elementId == selectedgrpId[i])
                    selected = true;
            }
            return selected;
        };
        SunburstChart.prototype.highlightSunburst = function (pointObj) {
            var path = document.getElementsByTagName('path'), pointIndex, data = pointObj, highele, element, groupElements, baseid, highlightgroup, legendItem, legend = this.model.legend, highlightSettings = this.model.highlightSettings, points = this.model.points, selected;
            if (!this.isNullOrUndefined(data)) {
                if (highlightSettings['mode'] == sunburstHighlightMode.point) {
                    if (highlightSettings['type'] != sunburstHighlightType.opacity) {
                        highele = document.getElementsByClassName("HighlightStyleSunburst");
                        if (highlightSettings['mode'] == sunburstHighlightMode.point) {
                            if (highele.length > 0) {
                                highele[0].removeAttribute('class');
                            }
                        }
                        element = document.getElementById(data['pointData'][0]['id']);
                        selected = this.isSelectedElement(element.id, this.selectedgroupID);
                        if (document.getElementById(element.id).getAttribute("class") == null && !selected)
                            this.createHighlightStyle(element, highlightSettings);
                        this.baseID = element.id;
                    }
                    else {
                        for (var j = 0; j < path.length; j++) {
                            if (this.selectedgroupID.indexOf(path[j].id) == -1)
                                path[j].style.opacity = highlightSettings['opacity'].toString();
                        }
                        document.getElementById(data['pointData'][0]['id']).style.opacity = '1';
                    }
                }
                else if (highlightSettings['mode'] == sunburstHighlightMode.all) {
                    groupElements = this.findGroupElements(data);
                    baseid = this._id + '_legendIndex' + groupElements[0].legendIndex + '_layerNumber1_layerIndex0_pointIndex' + groupElements[0].pointIndex.toString();
                    if (highlightSettings['type'] != sunburstHighlightType.opacity) {
                        highlightgroup = document.getElementsByClassName("HighlightStyleSunburst");
                        if (highlightgroup.length > 0) {
                            for (var k_1 = 0; k_1 < highlightgroup.length; k_1++) {
                                this.highlightgroupID.push(highlightgroup[k_1].id);
                                if (this.highlightgroupID[k_1].indexOf(this._id + "_svg_LegendItemShape") < 0)
                                    this.removeHighlight(this.highlightgroupID);
                            }
                        }
                        this.groupID = undefined;
                        this.highlightgroupID = [];
                        this.selelectedElementId = undefined;
                    }
                    this.highlightGroup(groupElements, data);
                    if (highlightSettings['type'] == sunburstHighlightType.opacity) {
                        for (var k = 0; k < path.length; k++) {
                            if (this.selectedgroupID.indexOf(path[k].id) == -1)
                                path[k].style.opacity = highlightSettings['opacity'].toString();
                        }
                        for (var j = 0; j < this.highlightgroupID.length; j++) {
                            document.getElementById(this.highlightgroupID[j]).style.opacity = '1';
                        }
                    }
                }
                else if (highlightSettings['mode'] == sunburstHighlightMode.child) {
                    baseid = data['pointData'][0]['id'];
                    if (highlightSettings['type'] != sunburstHighlightType.opacity) {
                        if (!this.isNullOrUndefined(this.highlightgroupID) && this.highlightgroupID.length > 0) {
                            this.removeHighlight(this.highlightgroupID);
                        }
                        this.highlightgroupID = [];
                        this.highlightedElementId = undefined;
                    }
                    this.highlightChild(data);
                    if (highlightSettings['type'] == sunburstHighlightType.opacity) {
                        for (var l = 0; l < path.length; l++) {
                            if (this.selectedgroupID.indexOf(path[l].id) == -1)
                                path[l].style.opacity = highlightSettings['opacity'].toString();
                        }
                        for (var j = 0; j < this.highlightgroupID.length; j++) {
                            document.getElementById(this.highlightgroupID[j]).style.opacity = '1';
                        }
                        this.highlightedElementId = undefined;
                    }
                }
                else {
                    baseid = data['pointData'][0]['id'];
                    if (highlightSettings['type'] != sunburstHighlightType.opacity) {
                        if (!this.isNullOrUndefined(this.groupID) && this.groupID.length > 0)
                            this.removeHighlight(this.groupID);
                        this.groupID = undefined;
                        this.selelectedElementId = undefined;
                    }
                    this.highlightParent(data);
                    if (highlightSettings['type'] == sunburstHighlightType.opacity) {
                        for (var m = 0; m < path.length; m++) {
                            if (this.selectedgroupID.indexOf(path[m].id) == -1)
                                path[m].style.opacity = this.model.selectionSettings.opacity.toString();
                        }
                        for (var j = 0; j < this.groupID.length; j++) {
                            document.getElementById(this.groupID[j]).style.opacity = '1';
                        }
                    }
                }
            }
            else {
                if (highlightSettings['type'] != sunburstHighlightType.opacity) {
                    if (highlightSettings['mode'] == sunburstHighlightMode.point) {
                        highele = document.getElementsByClassName("HighlightStyleSunburst");
                        if (highele.length > 0) {
                            highele[0].removeAttribute('class');
                            this.baseID = undefined;
                        }
                    }
                    else if (highlightSettings['mode'] == sunburstHighlightMode.all) {
                        if (!this.isNullOrUndefined(this.highlightgroupID)) {
                            for (var i = 0; i < this.highlightgroupID.length; i++) {
                                if (document.getElementById(this.highlightgroupID[i]) != null && (document.getElementById(this.highlightgroupID[i]).getAttribute("class") == null || document.getElementById(this.highlightgroupID[i]).getAttribute("class") == "HighlightStyleSunburst"))
                                    document.getElementById(this.highlightgroupID[i]).removeAttribute('class');
                            }
                            this.groupID = undefined;
                            this.selelectedElementId = undefined;
                        }
                    }
                    else {
                        highele = document.getElementsByClassName('HighlightStyleSunburst');
                        if (highele.length > 0) {
                            for (var i_1 = 0; i_1 < highele.length; i_1++) {
                                this.highlightgroupID.push(highele[i_1].id);
                            }
                            this.removeHighlight(this.highlightgroupID);
                            this.highlightgroupID = [];
                            this.highlightedElementId = null;
                        }
                    }
                }
                else {
                    for (var j = 0; j < path.length; j++) {
                        if (this.selectedgroupID.length == 0) {
                            if (this.selectedgroupID.indexOf(path[j].id) == -1) {
                                path[j].style.opacity = '1';
                            }
                        }
                        else {
                            if (this.selectedgroupID.indexOf(path[j].id) == -1) {
                                path[j].style.opacity = this.model.selectionSettings.opacity.toString();
                            }
                        }
                    }
                }
            }
        };
        SunburstChart.prototype.highlightGroup = function (groupElement, data) {
            var element, highlightSettings = this.model.highlightSettings, selected = false;
            this.highlightgroupID = [];
            for (var i = 0; i < groupElement.length; i++) {
                if ((groupElement[i].layerNumber == 1)) {
                    element = document.getElementById(this._id + '_legendIndex' + data.pointData[0].legendIndex + '_layerNumber1_layerIndex0_pointIndex' + groupElement[i].pointIndex.toString());
                    selected = this.isSelectedElement(element.id, this.selectedgroupID);
                    if (highlightSettings['type'] != sunburstHighlightType.opacity && (document.getElementById(element.id).getAttribute("class") == null) && !selected) {
                        this.createHighlightStyle(element, highlightSettings);
                        this.selelectedElementId = element.id;
                    }
                    this.highlightgroupID.push(element.id);
                }
                else {
                    for (var j = 0; j < groupElement[i].point.length; j++) {
                        if (groupElement[i].layerNumber != 1) {
                            element = document.getElementById(this._id + '_legendIndex' + data.pointData[0].legendIndex + '_layerNumber' + groupElement[i].layerNumber.toString() + '_layerIndex' + groupElement[i].layerIndex.toString() + '_pointIndex' + j.toString());
                            selected = this.isSelectedElement(element.id, this.selectedgroupID);
                            if (highlightSettings['type'] != sunburstHighlightType.opacity && (document.getElementById(element.id).getAttribute("class") == null) && !selected)
                                this.createHighlightStyle(element, highlightSettings);
                            this.highlightgroupID.push(element.id);
                        }
                    }
                }
            }
        };
        SunburstChart.prototype.highlightParent = function (data) {
            var element, style, baseid, highlightSettings = this.model.highlightSettings, selected = false, pointData = data.pointData[0];
            this.groupID = [];
            this.parentEle = [];
            this.parentEle.push(pointData);
            baseid = pointData['id'];
            if (!this.isNullOrUndefined(pointData.parentName)) {
                this.findSelectionparent(pointData.parentName, pointData.legendIndex);
            }
            for (var i = 0; i < this.parentEle.length; i++) {
                if (!document.getElementById("selectionSunburst")) {
                    element = document.getElementById(this._id + '_legendIndex' + pointData.legendIndex + '_layerNumber' + this.parentEle[i]['layerNumber'].toString() + '_layerIndex' + this.parentEle[i]['layerIndex'].toString() + '_pointIndex' + this.parentEle[i]['pointIndex']);
                    selected = this.isSelectedElement(element.id, this.selectedgroupID);
                    if (highlightSettings['type'] != sunburstHighlightType.opacity && !selected)
                        this.createHighlightStyle(element, highlightSettings);
                    this.groupID.push(element.id);
                    this.highlightgroupID.push(element.id);
                }
                else {
                    element = document.getElementById(this._id + '_legendIndex' + pointData.legendIndex + '_layerNumber' + this.parentEle[i]['layerNumber'].toString() + '_layerIndex' + this.parentEle[i]['layerIndex'].toString() + '_pointIndex' + this.parentEle[i]['pointIndex']);
                    selected = this.isSelectedElement(element.id, this.selectedgroupID);
                    if (highlightSettings['type'] != sunburstHighlightType.opacity && !selected)
                        this.createHighlightStyle(element, highlightSettings);
                    this.groupID.push(element.id);
                    this.highlightgroupID.push(element.id);
                }
            }
        };
        SunburstChart.prototype.highlightChild = function (data) {
            var layer, baseid, highlightSettings = this.model.highlightSettings, layerNumber, childid, childele, selected = false;
            this.childEle = [];
            this.highlightgroupID = [];
            layer = this.layerData[data.pointData[0].layerIndex]['point'][data.pointData[0].pointIndex];
            baseid = data.pointData[0]['id'];
            var element = document.getElementById(baseid);
            selected = this.isSelectedElement(element.id, this.selectedgroupID);
            if (!layer['point'] && layer['layerIndex'] != 0 && !selected) {
                if (highlightSettings['type'] != sunburstHighlightType.opacity) {
                    this.createHighlightStyle(element, highlightSettings);
                    this.highlightedElementId = baseid;
                }
                this.highlightgroupID.push(element.id);
            }
            else {
                layerNumber = data.pointData[0].layerNumber;
                if (!this.isNullOrUndefined(layer['point']))
                    this.findChild(layer, layerNumber);
                if (highlightSettings['type'] != sunburstHighlightType.opacity && !selected)
                    this.createHighlightStyle(element, highlightSettings);
                this.highlightgroupID.push(element.id);
                this.highlightedElementId = baseid;
                for (var i = 0; i < this.childEle.length; i++) {
                    if (this.childEle[i].layerNumber > data.pointData[0].layerNumber) {
                        for (var j = 0; j < this.childEle[i].length; j++) {
                            childid = this._id + '_legendIndex' + data.pointData[0].legendIndex + '_layerNumber' + this.childEle[i].layerNumber.toString() + '_layerIndex' + this.childEle[i].layerIndex + '_pointIndex' + j.toString();
                            childele = document.getElementById(childid);
                            selected = this.isSelectedElement(childid, this.selectedgroupID);
                            if (highlightSettings['type'] != sunburstHighlightType.opacity && !selected)
                                this.createHighlightStyle(childele, highlightSettings);
                            this.highlightgroupID.push(childele.id);
                        }
                    }
                }
            }
        };
        SunburstChart.prototype.createHighlightStyle = function (element, settings) {
            var style, fill, opacity;
            if (element.getAttribute('class') != "SelectionStyleSunburst") {
                if (!document.getElementById("highlightSunburst")) {
                    style = document.createElement('style');
                    style.type = 'text/css';
                    fill = settings.color;
                    opacity = '1';
                    style.innerHTML = ' .HighlightStyleSunburst{ fill:' + fill + ';' + 'opacity:' + opacity + ';' + 'stroke:;stroke-width:1}';
                    style.setAttribute('id', "highlightSunburst");
                    document.body.appendChild(style);
                }
                element.setAttribute('class', 'HighlightStyleSunburst');
            }
        };
        SunburstChart.prototype.showTooltip = function (location, layerPoint, layer, data) {
            var padding = 7, tooltipText, tooltipdivRect, rectOptions, x, y, commonToolTipTextArgs, tooltip = this.model.tooltip;
            var svgEle = document.getElementById(this._id + "_svg"), scrlltop = window.pageYOffset, scrollleft = window.pageXOffset, rect = svgEle.getBoundingClientRect();
            x = location.X + (padding / 2) + scrollleft + rect['left'];
            y = location.Y - (3 * padding) + scrlltop + rect['top'];
            if (document.getElementById(this._id + "_ToolTip") == null) {
                tooltipdivRect = document.createElement("div");
                tooltipdivRect.id = this._id + "_ToolTip";
                tooltipdivRect.style.pointerEvents = 'none';
            }
            else {
                tooltipdivRect = document.getElementById(this._id + "_ToolTip");
                tooltipdivRect.style.pointerEvents = 'none';
            }
            tooltipText = this.formatTooltip(data.pointData[0], tooltip['format']);
            commonToolTipTextArgs = $.extend({}, this.common);
            commonToolTipTextArgs['data'] = { pointIndex: data.pointData[0].pointIndex, currentText: tooltipText, Location: { x: x, Y: y } };
            this._trigger("tooltipInitialize", commonToolTipTextArgs);
            tooltipdivRect.innerHTML = commonToolTipTextArgs['data'].currentText;
            rectOptions = {
                'top': commonToolTipTextArgs['data'].Location.X,
                'left': commonToolTipTextArgs['data'].Location.Y,
                'background-color': tooltip['fill'],
                "border-style": "solid",
                "position": "absolute",
                "border-color": tooltip['border'].color,
                "border-width": tooltip['border'].width + "px",
                "opacity": tooltip['opacity'],
                "z-index": 1000000,
                "border-radius": "0px 0px",
                "padding-left": '5px',
                "padding-right": '5px',
                "padding-top": '2px',
                "padding-bottom": '2px'
            };
            this.setStyles(rectOptions, tooltipdivRect);
            var textOptions = {
                'top': y,
                'left': x,
                "color": tooltip['font'].color,
                'font-size': tooltip['font'].size,
                'font-family': tooltip['font'].fontFamily,
                'font-style': tooltip['font'].fontStyle,
                'font-weight': tooltip['font'].fontWeight,
                "align-self": "baseline"
            };
            this.setStyles(textOptions, tooltipdivRect);
            if (this.isNullOrUndefined(document.getElementById(this._id + "Tooltip"))) {
                document.body.appendChild(tooltipdivRect);
            }
            if (tooltipdivRect.style.visibility == "hidden") {
                tooltipdivRect.style.visibility = "visible";
            }
            tooltipdivRect.style.left = x.toString() + 'px';
            tooltipdivRect.style.top = y.toString() + 'px';
        };
        SunburstChart.prototype.formatTooltip = function (pointData, format) {
            var text = format;
            if (text.search("#point.x#") != -1) {
                text = text.replace("#point.x#", pointData['x']);
            }
            if (text.search("#point.y#") != -1)
                text = text.replace("#point.y#", pointData['y']);
            return text;
        };
        SunburstChart.prototype.setStyles = function (Options, element) {
            var properties = Object.keys(Options), temp;
            var values = properties.map(function (property) { return Options[property]; });
            for (var i = 0, len = properties.length; i < len; i++) {
                temp = properties[i];
                element.style[temp] = values[i];
            }
        };
        SunburstChart.prototype.createTooltiptemplate = function (location, layerPoint, layer, data) {
            var tooltipdiv = document.getElementById('tooltipDiv'), x, y, svgEle = document.getElementById(this._id + "_svg"), scrlltop = window.pageYOffset, padding = 10, scrollleft = window.pageXOffset, rect = svgEle.getBoundingClientRect();
            x = location['X'] + scrollleft + rect['left'] + padding;
            y = location['Y'] + scrlltop + rect['top'];
            if (this.isNullOrUndefined(tooltipdiv)) {
                this.tooltipfirst = true;
                tooltipdiv = document.createElement("div");
                tooltipdiv.className = "tooltipDiv";
                tooltipdiv.style.position = 'absolute';
                tooltipdiv.style.zIndex = '13000';
                tooltipdiv.style.display = 'block';
                tooltipdiv.id = "tooltipDiv";
                tooltipdiv.style.backgroundColor = "white";
                document.body.appendChild(tooltipdiv);
                this.clonenode = document.getElementById(this.model.tooltip.template).cloneNode(true);
                this.clonenode['style'].display = 'block';
                document.getElementById('tooltipDiv').innerHTML = '';
                this.clonenode['innerHTML'] = this.parseTooltipTemplate(data.pointData[0], this.clonenode['innerHTML']);
                document.getElementById('tooltipDiv').appendChild(this.clonenode);
            }
            else {
                tooltipdiv.style.display = 'block';
                this.tooltipfirst = false;
                this.clonenode['innerHTML'] = this.parseTooltipTemplate(data.pointData[0], this.clonenode['innerHTML']);
            }
            document.getElementById('tooltipDiv').style.left = x.toString() + 'px';
            document.getElementById('tooltipDiv').style.top = y.toString() + 'px';
        };
        SunburstChart.prototype.parseTooltipTemplate = function (point, clonenode) {
            var clone = document.getElementById(this.model.tooltip.template).cloneNode(true);
            var text = clone.innerHTML;
            if (text.search('#point.x#') != -1) {
                text = text.replace('#point.x#', point['x'].toString());
            }
            if (text.search('#point.y#') != -1) {
                text = text.replace('#point.y#', point['y'].toString());
            }
            if (text.search('#point.text#') != -1) {
                if (!this.isNullOrUndefined(point['text'])) {
                    text = text.replace('#point.text#', point['text'].toString());
                }
                else {
                    text = text.replace('#point.text#', point['y'].toString());
                }
            }
            return text;
        };
        SunburstChart.prototype.bindClickEvent = function (event) {
            var data, commonPointClickEventArgs;
            data = this.getSunburstPoint(event);
            if (!this.isNullOrUndefined(data) && this.model.pointRegionClick) {
                commonPointClickEventArgs = $.extend({}, this.common);
                commonPointClickEventArgs['data'] = data;
                this._trigger("pointRegionClick", commonPointClickEventArgs);
            }
            return data;
        };
        SunburstChart.prototype.sunburstClick = function (event) {
            var data = this.bindClickEvent(event), sunburst, doubleTapTimer, backTooltip, selectionSettings = this.model.selectionSettings, point, legendItem;
            if (selectionSettings['enable']) {
                this.selectSunburst(data, event);
                if (!this.isNullOrUndefined(data) && this.model.legend.visible) {
                    for (var i = 0; i < this.model.points.length; i++) {
                        legendItem = document.getElementById(this._id + '_svg_LegendItemShape' + i);
                        point = this.model.points[i];
                        if (selectionSettings['type'] == sunburstSelectionType.opacity) {
                            if (data['pointData'][0].legendIndex != i) {
                                legendItem.style.opacity = selectionSettings['opacity'].toString();
                            }
                        }
                        else {
                            if (this.selectedGroupNumber - 1 == i) {
                                legendItem.setAttribute('class', 'SelectionStyleSunburst');
                                legendItem.style.opacity = selectionSettings['opacity'].toString();
                            }
                            else {
                                if (!(legendItem.getAttribute("class") == "SelectionStyleSunburst"))
                                    legendItem.removeAttribute("class");
                            }
                        }
                    }
                }
            }
            sunburst = this;
            sunburst.tapCount++;
            doubleTapTimer = setTimeout(function () {
                if (sunburst.tapCount >= 2 && sunburst._isDoubleClickEvent == false) {
                    sunburst.sunburstDoubleClick(event);
                    sunburst._isDoubleClickEvent = false;
                }
                sunburst.tapCount = 0;
            }, 200);
            backTooltip = document.getElementById(sunburst._id + "toolbarTooltip");
            if (!sunburst.isNullOrUndefined(backTooltip)) {
                backTooltip.parentNode.removeChild(backTooltip);
            }
        };
        SunburstChart.prototype.isTouch = function (evt) {
            var browserInfo, event;
            browserInfo = this.browserInfo();
            event = evt['originalEvent'] ? evt['originalEvent'] : evt;
            if ((event['pointerType'] == "touch") || (event['pointerType'] == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        };
        SunburstChart.prototype.sunburstLeave = function (event) {
            var sunburst = this, elementId = event.target.id, legendSegment, childNodes, legendIndex, sunburstSegment, legendItem, tooltipDiv, backTooltip;
            if (this._legendHighlight) {
                legendSegment = document.getElementById(this._id + "legendItem_Group");
                childNodes = legendSegment.childNodes;
                for (var l = 0; l < childNodes.length; l++)
                    document.getElementById(childNodes[l]['id']).style.opacity = "1";
                sunburstSegment = document.querySelectorAll("[id^=" + this._id + "_legendIndex]");
                for (var j = 0; j < sunburstSegment.length; j++)
                    document.getElementById(sunburstSegment[j].id).style.opacity = "1";
                this._legendHighlight = false;
            }
            if (this.model.legend.visible && this.model.highlightSettings.enable) {
                for (var l = 0; l < this.model.points.length; l++) {
                    legendItem = document.getElementById(this._id + "_svg_LegendItemShape" + l);
                    if (legendItem != null) {
                        legendItem.style.opacity = "1";
                        legendItem.removeAttribute("class");
                    }
                }
            }
            if (!this.isNullOrUndefined(document.getElementById(sunburst._id + "_ToolTip"))) {
                if (ej.isTouchDevice() || this.isTouch(event)) {
                    var hideTooltip = setTimeout(function () {
                        document.getElementById(sunburst._id + "_ToolTip").style.visibility = "hidden";
                    }, 3000);
                }
                else if (elementId == sunburst._id + "_svg_svgRect") {
                    if (!this._hideTooltip)
                        clearTimeout(this._hideTooltip);
                    this._hideTooltip = setTimeout(function () {
                        document.getElementById(sunburst._id + "_ToolTip").style.visibility = "hidden";
                    }, 1000);
                }
            }
            backTooltip = document.getElementById(sunburst._id + "toolbarTooltip");
            if (!sunburst.isNullOrUndefined(backTooltip)) {
                backTooltip.parentNode.removeChild(backTooltip);
            }
            tooltipDiv = document.getElementById('tooltipDiv');
            if (!sunburst.isNullOrUndefined(tooltipDiv)) {
                if (ej.isTouchDevice() || this.isTouch(event)) {
                    var hideTooltipTemplate = setTimeout(function () {
                        tooltipDiv.style.display = "none";
                    }, 3000);
                }
                else if (elementId == sunburst._id + "_svg_svgRect") {
                    if (!this._hideTooltipTemplate)
                        clearTimeout(this._hideTooltipTemplate);
                    this._hideTooltipTemplate = setTimeout(function () {
                        tooltipDiv.style.display = "none";
                    }, 1000);
                }
            }
            if (elementId.indexOf(this._id + 'reset') != -1) {
                for (var i = 0; i < 5; i++) {
                    if (i == 0)
                        document.getElementById(this._id + "resetPath").setAttribute("fill", "#D6D6D6");
                    else
                        document.getElementById(this._id + "resetPath" + i.toString()).setAttribute("fill", "#5E5C5C");
                }
            }
            if (elementId.indexOf(this._id + 'back') != -1) {
                for (var i = 0; i < 3; i++) {
                    if (i == 0)
                        document.getElementById(this._id + "backPath").setAttribute("fill", "#D6D6D6");
                    else
                        document.getElementById(this._id + "backPath" + i.toString()).setAttribute("fill", "#5E5C5C");
                }
            }
        };
        SunburstChart.prototype.selectSunburst = function (data, event) {
            var element, selectedele, pointIndex, style, baseID, canvas, styleOptions, fill, highlighted, selected, selectionMode = this.model.selectionSettings.mode.toString(), path = document.getElementsByTagName('path'), mouseClickedElement, selectionOpacity = this.model.selectionSettings.opacity.toString(), selecttype = this.model.selectionSettings.type, legend = this.model.legend, points = this.model.points, legendItem, groupElements, selectedEle, highlightEle;
            if (!this.isNullOrUndefined(data)) {
                if (this.selectedgroupID.length > 0)
                    this.removeSelection(this.selectedgroupID);
                if (selectionMode == "point") {
                    this.selectedgroupID = [];
                    mouseClickedElement = event.target.id.replace('_layerIndex' + data.pointData[0].layerIndex.toString(), "");
                    if (sunburstSelectionType.opacity != selecttype) {
                        if (!this.isNullOrUndefined(this.highlightgroupID) && this.highlightgroupID.length > 0) {
                            this.removeHighlight(this.highlightgroupID);
                        }
                        element = document.getElementById(data.pointData[0]['id']);
                        if (element.getAttribute('class') == 'SelectionStyleSunburst') {
                            element.removeAttribute('class');
                            if (this.highlightgroupID.indexOf(element.id) != -1) {
                                element.setAttribute('class', 'HighlightStyleSunburst');
                            }
                        }
                        else {
                            selectedEle = document.getElementsByClassName('SelectionStyleSunburst');
                            if (this.highlightgroupID.length > 0 && selectedEle.length > 0) {
                                if (this.highlightgroupID.indexOf(selectedEle[0].id) != -1) {
                                    selectedEle[0].setAttribute('class', 'HighlightStyleSunburst');
                                }
                            }
                            if (selectedEle.length > 0 && selectedEle[0].getAttribute('class') == "SelectionStyleSunburst")
                                selectedEle[0].removeAttribute('class');
                            if (!this.isNullOrUndefined(this.highlightgroupID) && this.highlightgroupID.length > 0) {
                                this.removeHighlight(this.highlightgroupID);
                            }
                            this.selectPointData(element);
                            this.selectedgroupID.push(element.id);
                        }
                    }
                    else {
                        if (this.mouseClickedElementId != mouseClickedElement) {
                            for (var i = 0; i < path.length; i++) {
                                path[i].style.opacity = selectionOpacity;
                            }
                            document.getElementById(data.pointData[0]['id']).style.opacity = '1';
                            this.mouseClickedElementId = mouseClickedElement;
                            this.selectedgroupID.push(data.pointData[0]['id']);
                        }
                        else {
                            for (var j = 0; j < path.length; j++) {
                                path[j].style.opacity = '1';
                            }
                            this.mouseClickedElementId = "";
                        }
                        this.selectedGroupNumber = data.pointData[0].groupNumber;
                    }
                    if (legend['visible'] && this.selectedgroupID.length > 0) {
                        for (var k = 0; k < points.length; k++) {
                            legendItem = document.getElementById(this._id + '_svg_LegendItemShape' + k);
                            if (data.pointData[0].legendIndex != k) {
                                if (legendItem != null) {
                                    legendItem.style.opacity = selectionOpacity;
                                    legendItem.removeAttribute("class");
                                }
                            }
                            else {
                                if (sunburstSelectionType.opacity == selecttype)
                                    legendItem.style.opacity = '1';
                                else {
                                    legendItem.setAttribute("class", "SelectionStyleSunburst");
                                    legendItem.style.opacity = selectionOpacity;
                                }
                            }
                        }
                    }
                    if (this.selectedgroupID.length == 0)
                        this.removeLegendSelection();
                }
                else if (selectionMode == "all") {
                    groupElements = this.findGroupElements(data);
                    baseID = data.pointData[0]['id'];
                    if (this.selectedgroupID.indexOf(baseID) == -1) {
                        if (this.selectedGroupNumber != groupElements[0]['groupNumber']) {
                            if (this.selectedgroupID.length > 0) {
                                this.removeSelection(this.selectedgroupID);
                            }
                            this.selectGroup(groupElements, data);
                            this.selectedGroupNumber = groupElements[0]['groupNumber'];
                        }
                        else {
                            if (this.selectedgroupID.length > 0) {
                                this.removeSelection(this.selectedgroupID);
                            }
                            this.selectedgroupID = [];
                            this.selectedGroupNumber = null;
                        }
                        if (sunburstSelectionType.opacity == selecttype) {
                            for (var i = 0; i < path.length; i++) {
                                path[i].style.opacity = selectionOpacity;
                            }
                            for (var j = 0; j < this.selectedgroupID.length; j++) {
                                document.getElementById(this.selectedgroupID[j]).style.opacity = '1';
                            }
                            this.isSelected = true;
                        }
                        if (legend['visible']) {
                            for (var k_2 = 0; k_2 < points.length; k_2++) {
                                legendItem = document.getElementById(this._id + '_svg_LegendItemShape' + k_2);
                                if (legendItem != null) {
                                    legendItem.style.opacity = selectionOpacity;
                                    legendItem.removeAttribute("class");
                                }
                            }
                            if (sunburstSelectionType.opacity != selecttype)
                                document.getElementById(this._id + '_svg_LegendItemShape' + groupElements[0]['pointIndex'].toString()).setAttribute("class", "SelectionStyleSunburst");
                            else
                                document.getElementById(this._id + '_svg_LegendItemShape' + groupElements[0]['pointIndex'].toString()).style.opacity = '1';
                        }
                    }
                    else {
                        if (this.selectedgroupID.length > 0) {
                            if (sunburstSelectionType.opacity != selecttype)
                                this.removeSelection(this.selectedgroupID);
                            else {
                                for (var i = 0; i < path.length; i++) {
                                    path[i].style.opacity = '1';
                                }
                            }
                        }
                        this.isSelected = false;
                        this.selectedGroupNumber = null;
                        this.selectedgroupID = [];
                        this.baseID = '';
                        this.removeLegendSelection();
                    }
                }
                else if (selectionMode == "child") {
                    baseID = data.pointData[0]['id'];
                    if (this.selectedgroupID.indexOf(baseID) == -1) {
                        if (!this.isNullOrUndefined(this.selectedgroupID) && this.selectedgroupID.length > 0) {
                            this.removeSelection(this.selectedgroupID);
                        }
                        if (!this.isNullOrUndefined(this.highlightgroupID) && this.highlightgroupID.length > 0) {
                            this.removeHighlight(this.highlightgroupID);
                        }
                        this.selectedgroupID = [];
                        this.selelectedElementId = undefined;
                        this.selectchild(data);
                        if (sunburstSelectionType.opacity == selecttype) {
                            for (var i = 0; i < path.length; i++) {
                                path[i].style.opacity = selectionOpacity;
                            }
                            for (var j = 0; j < this.selectedgroupID.length; j++) {
                                document.getElementById(this.selectedgroupID[j]).style.opacity = '1';
                            }
                            this.isSelected = true;
                        }
                        this.selectedGroupNumber = data.pointData[0].groupNumber;
                        if (legend['visible']) {
                            for (var k = 0; k < points.length; k++) {
                                legendItem = document.getElementById(this._id + '_svg_LegendItemShape' + k);
                                if (data.pointData[0].legendIndex != k) {
                                    if (legendItem != null) {
                                        legendItem.style.opacity = selectionOpacity;
                                        legendItem.removeAttribute("class");
                                    }
                                }
                                else {
                                    if (sunburstSelectionType.opacity == selecttype)
                                        legendItem.style.opacity = '1';
                                    else {
                                        legendItem.setAttribute("class", "SelectionStyleSunburst");
                                        legendItem.style.opacity = selectionOpacity;
                                    }
                                }
                            }
                        }
                    }
                    else {
                        this.selectedgroupID = [];
                        this.selelectedElementId = this.baseID = '';
                        this.removeSegmentSelection(path, selecttype);
                        this.removeLegendSelection();
                        this.isSelected = false;
                        this.childEle = [];
                    }
                }
                else {
                    baseID = data.pointData[0]['id'];
                    if (this.selectedgroupID.indexOf(baseID) == -1) {
                        if (!this.isNullOrUndefined(this.groupNumber) && this.groupNumber != data.pointData[0].groupNumber)
                            this.removeSegmentSelection(path, this.model.selectionSettings.type);
                        else if (this.selectedgroupID.length > 0) {
                            this.removeSelection(this.selectedgroupID);
                            this.selectedgroupID = [];
                        }
                        if (!this.isNullOrUndefined(this.highlightgroupID) && this.highlightgroupID.length > 0) {
                            this.removeHighlight(this.highlightgroupID);
                        }
                        this.selectParent(data);
                        this.groupNumber = data.pointData[0].groupNumber;
                        if (sunburstSelectionType.opacity == selecttype) {
                            for (var i = 0; i < path.length; i++) {
                                path[i].style.opacity = selectionOpacity;
                            }
                            for (var j = 0; j < this.selectedgroupID.length; j++) {
                                document.getElementById(this.selectedgroupID[j]).style.opacity = '1';
                            }
                        }
                        this.selectedGroupNumber = data.pointData[0].groupNumber;
                        pointIndex = data.pointData[0].legendIndex;
                        if (legend['visible']) {
                            for (var k_3 = 0; k_3 < points.length; k_3++) {
                                legendItem = document.getElementById(this._id + '_svg_LegendItemShape' + k_3);
                                if (legendItem != null) {
                                    legendItem.style.opacity = selectionOpacity;
                                    legendItem.removeAttribute("class");
                                }
                            }
                            if (sunburstSelectionType.opacity != selecttype)
                                document.getElementById(this._id + '_svg_LegendItemShape' + pointIndex.toString()).setAttribute("class", "SelectionStyleSunburst");
                            else
                                document.getElementById(this._id + '_svg_LegendItemShape' + pointIndex.toString()).style.opacity = '1';
                        }
                    }
                    else {
                        this.selectedgroupID = [];
                        this.selelectedElementId = this.baseID = '';
                        this.removeSegmentSelection(path, this.model.selectionSettings.type);
                        this.removeLegendSelection();
                        this.parentEle = [];
                    }
                }
            }
            else {
                if (sunburstSelectionType.opacity != selecttype) {
                    if (selectionMode == "point") {
                        selectedEle = document.getElementsByClassName('SelectionStyleSunburst');
                        if (selectedEle.length > 0)
                            selectedEle[0].removeAttribute('class');
                    }
                    else {
                        if (this.selectedgroupID.length > 0) {
                            this.removeSelection(this.selectedgroupID);
                            this.selectedgroupID = [];
                            this.selectedGroupNumber = null;
                            if (selectionMode == "child")
                                this.selelectedElementId = null;
                            if (selectionMode == "parent") {
                                this.groupNumber = null;
                            }
                        }
                    }
                }
                else {
                    this.selectedgroupID = [];
                    for (var j = 0; j < path.length; j++) {
                        path[j].style.opacity = '1';
                    }
                    this.mouseClickedElementId = "";
                    this.selectedGroupNumber = null;
                }
                this.removeLegendSelection();
            }
        };
        SunburstChart.prototype.removeSegmentSelection = function (path, selectType) {
            for (var i = 0; i < path.length; i++) {
                if (selectType == sunburstSelectionType.opacity)
                    path[i].style.opacity = "1";
                else
                    path[i].removeAttribute('class');
            }
        };
        SunburstChart.prototype.removeLegendSelection = function () {
            var legendItem;
            if (this.model.legend.visible) {
                for (var k = 0; k < this.model.points.length; k++) {
                    legendItem = document.getElementById(this._id + '_svg_LegendItemShape' + k);
                    if (legendItem != null) {
                        legendItem.style.opacity = "1";
                        legendItem.removeAttribute("class");
                    }
                }
            }
        };
        SunburstChart.prototype.selectParent = function (data) {
            var element, style, lgndIndx, selectionSettings = this.model.selectionSettings, selectedEle, highlightEle;
            this.parentEle = [];
            lgndIndx = data.pointData[0].legendIndex;
            if (sunburstSelectionType.opacity == selectionSettings['type'])
                this.selectedgroupID = [];
            this.parentEle.push(data.pointData[0]);
            var baseid = data.pointData[0]['id'];
            if (sunburstSelectionType.opacity != selectionSettings['type']) {
                this.selectedgroupID = [];
                this.removeSelection(this.selectedgroupID);
                this.selelectedElementId = baseid;
            }
            if (!this.isNullOrUndefined(data.pointData[0].parentName)) {
                this.findSelectionparent(data.pointData[0].parentName, lgndIndx);
            }
            for (var i = 0; i < this.parentEle.length; i++) {
                if (!document.getElementById("selectionSunburst")) {
                    element = document.getElementById(this._id + '_legendIndex' + lgndIndx + '_layerNumber' + this.parentEle[i]['layerNumber'].toString() + '_layerIndex' + this.parentEle[i]['layerIndex'].toString() + '_pointIndex' + this.parentEle[i]['pointIndex']);
                    if (sunburstSelectionType.opacity != this.model.selectionSettings.type) {
                        style = this.createselectionStyle(element, this.model.selectionSettings);
                        document.body.appendChild(style);
                        element.setAttribute('class', 'SelectionStyleSunburst');
                    }
                    this.selectedgroupID.push(element.id);
                }
                else {
                    element = document.getElementById(this._id + '_legendIndex' + lgndIndx + '_layerNumber' + this.parentEle[i]['layerNumber'].toString() + '_layerIndex' + this.parentEle[i]['layerIndex'].toString() + '_pointIndex' + this.parentEle[i]['pointIndex']);
                    if (sunburstSelectionType.opacity != this.model.selectionSettings.type)
                        element.setAttribute('class', 'SelectionStyleSunburst');
                    this.selectedgroupID.push(element.id);
                }
            }
        };
        SunburstChart.prototype.findSelectionparent = function (data, legendIndex) {
            var regions = this.sunburstRegions, length = regions.length, region;
            loop1: for (var i = 0; i < length; i++) {
                region = regions[i];
                for (var j = 0; j < region['region'].length; j++) {
                    if (data == region['region'][j]['x'] && legendIndex == region['region'][j].legendIndex) {
                        this.parentEle.push(region['region'][j]);
                        if (!this.isNullOrUndefined(region['region'][j]['parentName'])) {
                            this.findSelectionparent(region['region'][j]['parentName'], region['region'][j].legendIndex);
                            break loop1;
                        }
                        else {
                            break loop1;
                        }
                    }
                }
            }
        };
        SunburstChart.prototype.selectchild = function (data) {
            var layer, baseid, element, selectionSettings = this.model.selectionSettings, style, lgndIndx, childid, childele;
            this.childEle = [];
            layer = this.layerData[data.pointData[0].layerIndex]['point'][data.pointData[0].pointIndex];
            baseid = data.pointData[0]['id'];
            element = document.getElementById(baseid);
            if (!layer['point'] && layer['layerIndex'] != 0) {
                if (selectionSettings['type'] != sunburstSelectionType.opacity) {
                    if (!document.getElementById("selectionSunburst")) {
                        style = this.createselectionStyle(element, selectionSettings);
                        document.body.appendChild(style);
                    }
                    element.setAttribute('class', 'SelectionStyleSunburst');
                    this.selelectedElementId = baseid;
                }
                this.selectedgroupID.push(element.id);
            }
            else {
                var layerNumber = data.pointData[0].layerNumber;
                if (!this.isNullOrUndefined(layer['point']))
                    this.findChild(layer, layerNumber);
                if (selectionSettings['type'] != sunburstSelectionType.opacity) {
                    if (!document.getElementById("SelectionStyleSunburst")) {
                        style = this.createselectionStyle(element, selectionSettings);
                        document.body.appendChild(style);
                    }
                    element.setAttribute('class', 'SelectionStyleSunburst');
                }
                this.selectedgroupID.push(element.id);
                this.selelectedElementId = baseid;
                lgndIndx = data.pointData[0].legendIndex;
                for (var i = 0; i < this.childEle.length; i++) {
                    if (this.childEle[i].layerNumber > data.pointData[0].layerNumber) {
                        for (var j = 0; j < this.childEle[i].length; j++) {
                            childid = this._id + '_legendIndex' + lgndIndx + '_layerNumber' + this.childEle[i].layerNumber.toString() + '_layerIndex' + this.childEle[i].layerIndex + '_pointIndex' + j.toString();
                            childele = document.getElementById(childid);
                            if (selectionSettings['type'] != sunburstSelectionType.opacity)
                                childele.setAttribute('class', 'SelectionStyleSunburst');
                            this.selectedgroupID.push(childele.id);
                        }
                    }
                }
            }
        };
        SunburstChart.prototype.findChild = function (layer, layerNumber) {
            var points = [];
            for (var i = 0; i < layer['point'].length; i++) {
                if (layer['point'][i].point) {
                    points.push(layer['point'][i]);
                    this.findChild(layer['point'][i], layerNumber + 1);
                }
                else {
                    points.push(layer['point'][i]);
                }
            }
            points.layerNumber = layerNumber + 1;
            points.layerIndex = layer['point'].layerData;
            this.childEle.push(points);
        };
        SunburstChart.prototype.selectGroup = function (groupElement, data) {
            var legendIndex, selectionSettings = this.model.selectionSettings, element, style;
            this.selectedgroupID = [];
            legendIndex = data.pointData[0].legendIndex;
            for (var i = 0; i < groupElement.length; i++) {
                if (groupElement[i].layerNumber == 1) {
                    element = document.getElementById(this._id + '_legendIndex' + legendIndex + '_layerNumber1_layerIndex0_pointIndex' + groupElement[i].pointIndex.toString());
                    if (selectionSettings['type'] != sunburstSelectionType.opacity) {
                        style = this.createselectionStyle(element, selectionSettings);
                        document.body.appendChild(style);
                        element.setAttribute('class', 'SelectionStyleSunburst');
                    }
                    this.selectedgroupID.push(element.id);
                }
                else {
                    for (var j = 0; j < groupElement[i].point.length; j++) {
                        element = document.getElementById(this._id + '_legendIndex' + legendIndex + '_layerNumber' + groupElement[i].layerNumber.toString() + '_layerIndex' + groupElement[i].layerIndex.toString() + '_pointIndex' + j.toString());
                        if (selectionSettings['type'] != sunburstSelectionType.opacity) {
                            var selele = document.getElementById('selectionSunburst');
                            if (selele)
                                element.setAttribute('class', 'SelectionStyleSunburst');
                            else {
                                style = this.createselectionStyle(element, selectionSettings);
                                document.body.appendChild(style);
                                element.setAttribute('class', 'SelectionStyleSunburst');
                            }
                        }
                        this.selectedgroupID.push(element.id);
                    }
                }
            }
        };
        SunburstChart.prototype.removeSelection = function (ID) {
            for (var i = 0; i < ID.length && document.getElementById(ID[i]) != null; i++) {
                if (document.getElementById(ID[i]).getAttribute('class') == "SelectionStyleSunburst")
                    document.getElementById(ID[i]).removeAttribute('class');
            }
        };
        SunburstChart.prototype.findGroupElements = function (data) {
            var groupNumber = data.pointData[0].groupNumber ? data.pointData[0].groupNumber : data.pointData[0].pointIndex + 1, groupPoints = [];
            for (var i = 0; i < this.layerData.length; i++) {
                if (i == 0) {
                    for (var j = 0; j < this.layerData[i]['point'].length; j++) {
                        if (this.layerData[i]['point'][j].groupNumber == groupNumber) {
                            groupPoints.push(this.layerData[i]['point'][j]);
                        }
                    }
                }
                else {
                    if (this.layerData[i]['groupNumber'] == groupNumber) {
                        this.layerData[i]['layerIndex'] = i;
                        groupPoints.push(this.layerData[i]);
                    }
                }
            }
            return groupPoints;
        };
        SunburstChart.prototype.selectPointData = function (element) {
            var style;
            if (!document.getElementById("selectionSunburst")) {
                style = this.createselectionStyle(element, this.model.selectionSettings);
                document.body.appendChild(style);
                element.setAttribute('class', 'SelectionStyleSunburst');
                this.selelectedElementId = element.id;
            }
            else {
                element.setAttribute('class', 'SelectionStyleSunburst');
                this.selelectedElementId = element.id;
            }
        };
        SunburstChart.prototype.createselectionStyle = function (element, settings) {
            var style, fill, opacity;
            style = document.createElement('style');
            style.type = 'text/css';
            fill = settings.color;
            opacity = '1';
            style.innerHTML = ' .SelectionStyleSunburst{ fill:' + fill + ';' + 'opacity:' + opacity + ';' + 'stroke:;stroke-width:1}';
            style.setAttribute('id', "selectionSunburst");
            return style;
        };
        SunburstChart.prototype.getSunburstPoint = function (event) {
            var mouseX, mouseY, currentX, currentY, region, width, height, distanceFromcenter, radiusCollection, mouseposition, startAngle, endAngle, sunStartAngle;
            mouseposition = this.calMousePosition(event);
            radiusCollection = this.circularRadius;
            currentX = mouseposition['X'];
            currentY = mouseposition['Y'];
            startAngle = this.model.startAngle;
            endAngle = this.model.endAngle;
            width = this.areaBounds['actualWidth'];
            height = this.areaBounds['actualHeight'];
            sunStartAngle = this.model.startAngle * (Math.PI / 180);
            this.sunburstRegions.forEach(function (regionItem, regionIndex) {
                var sunburstData = regionItem.sunburstData, regionData = regionItem.region, fromCenterX, fromCenterY, innerRadius, segment, arcAngle = 0, clickAngle, pointStartAngle, pointEndAngle, pointData = [], totalDegree, chartStartAngle;
                innerRadius = sunburstData['innerRadius'] ? sunburstData['innerRadius'] : 0;
                fromCenterX = (currentX) - (sunburstData['centerX']);
                fromCenterY = (currentY) - (sunburstData['centerY']);
                segment = regionItem.region;
                startAngle = startAngle < 0 ? startAngle + 360 : startAngle;
                endAngle = endAngle < 0 ? endAngle + 360 : endAngle;
                totalDegree = endAngle - startAngle;
                chartStartAngle = (-.5 * Math.PI) + sunStartAngle;
                if (totalDegree < 0) {
                    endAngle = endAngle / 360;
                    arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
                    clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                    if (clickAngle < 0)
                        clickAngle = 2 * Math.PI + clickAngle;
                    for (var i = 0; i < regionData.length; i++) {
                        pointStartAngle = parseFloat(regionData[i]['startAngle'].toFixed(14));
                        pointEndAngle = parseFloat(regionData[i]['endAngle'].toFixed(14));
                        pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                        pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                        pointStartAngle -= arcAngle;
                        pointEndAngle -= arcAngle;
                        pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                        pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                        if (clickAngle <= pointStartAngle && clickAngle >= pointEndAngle) {
                            pointData.push(regionData[i]);
                            break;
                        }
                    }
                }
                else {
                    clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle) % (2 * Math.PI);
                    if (clickAngle < 0)
                        clickAngle = 2 * Math.PI + clickAngle;
                    for (var i = 0; i < regionData.length; i++) {
                        pointStartAngle = parseFloat(regionData[i]['startAngle'].toFixed(14));
                        pointEndAngle = parseFloat(regionData[i]['endAngle'].toFixed(14));
                        pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                        pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                        pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                        pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                        if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                            pointData.push(regionData[i]);
                            break;
                        }
                    }
                }
                if (pointData.length > 0) {
                    var distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                    if (distanceFromCenter <= sunburstData['Radius'] && distanceFromCenter > innerRadius) {
                        region = { pointData: pointData, location: { x: currentX, y: currentY } };
                    }
                }
            });
            return region;
        };
        SunburstChart.prototype.calMousePosition = function (evt) {
            var mouseX, mouseY, mouseposition, element, rect, scrlltop, scrollleft;
            mouseposition = this.mousePosition(evt);
            evt.pageX = mouseposition['x'];
            evt.pageY = mouseposition['y'];
            element = document.getElementById(this._id + "_svg");
            scrlltop = window.pageYOffset;
            scrollleft = window.pageXOffset;
            rect = element.getBoundingClientRect();
            mouseX = evt.pageX - scrollleft - rect['left'];
            mouseY = evt.pageY - scrlltop - rect['top'];
            return { X: mouseX, Y: mouseY };
        };
        SunburstChart.prototype.mousePosition = function (evt) {
            if (!this.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return { x: evt.pageX, y: evt.pageY };
            else
                return { x: 0, y: 0 };
        };
        SunburstChart.prototype.drawSunburstLegend = function () {
            var legend = this.model.legend, legendTitle = this.model.legend.title, maximumLegendSpace, translate, width, x, legendBounds, y, scrollButtons, divele, isScroll = false, legendGrp, legElegrp, elementSpacing = this.elementSpacing, legendItems, titleTransform, legendgrp, legendItemgrp;
            this.gLegendEle = this.createGroup({ 'id': this._id + "_legend_group" });
            if (legend['position'] == sunburstLegendPosition.top || legend['postion'] == sunburstLegendPosition.bottom) {
                maximumLegendSpace = this.height * (20 / 100);
            }
            document.getElementById(this._id + '_svg').appendChild(this.gLegendEle);
            if (legendTitle['text'] != "" && legendTitle['visible'])
                this.gLegendEle.appendChild(this.drawLegendTitle(legendTitle, legend));
            legendGrp = this.drawlegendItem(legend, this.legendCollection, isScroll);
            if (this.drilldownCount == 0) {
                this.legendClick = this.legendClick.bind(this);
                this.legendItemGroup.addEventListener('click', this.legendClick, true);
            }
            this.gLegendEle.appendChild(legendGrp);
            if (this.legendPages.length > 0) {
                this.drawScrollbuttons(legend);
            }
            this.legendPositioning(legend, isScroll);
        };
        SunburstChart.prototype.legendPositioning = function (legend, isScroll) {
            var legendgrp, legendItemgrp, legendItems, legendBounds, width, x, elementSpacing = this.elementSpacing, y, translate, divele, scrollButtons, titleTransform;
            legendgrp = document.getElementById(this._id + "_legend_group");
            legendItemgrp = document.getElementById(this._id + "legendItem_Group");
            if (legend['position'] == sunburstLegendPosition.bottom || legend['position'] == sunburstLegendPosition.top) {
                legendItems = document.getElementById(this._id + "legendItem_Group");
                legendBounds = legendItems.getBoundingClientRect();
                width = legendBounds['width'];
                x = (this.width / 2) - (width / 2);
                if (legend['position'] == sunburstLegendPosition.bottom) {
                    if (this.isNullOrUndefined(legend['size']['height']))
                        y = this.LegendBounds['Height'] - elementSpacing;
                    else
                        y = legendBounds['height'] + elementSpacing;
                }
                else {
                    if (this.isNullOrUndefined(legend['size']['height']))
                        y = 0;
                    else {
                        y = this.LegendBounds['Height'] - legendBounds['top'] + elementSpacing;
                    }
                }
                translate = 'translate(' + x.toString() + ',' + '0)';
                legendItems.setAttribute('transform', translate);
                if (legend['title']['text'] != "" && legend['title']['visible'])
                    document.getElementById(this._id + "_LegendTitleText").setAttribute('transform', translate);
                if (this.legendPages.length > 0) {
                    divele = document.getElementById(this._id).getBoundingClientRect();
                    x = (this.width / 2) - (document.getElementById("scrollButtons").getBoundingClientRect().width / 2);
                    if (legend['position'] == sunburstLegendPosition.bottom) {
                        y = legendBounds['bottom'] - divele['top'];
                    }
                    else if (legend['position'] == sunburstLegendPosition.top) {
                        y = legendBounds['top'] + legendBounds['height'];
                    }
                    translate = 'translate(' + x.toString() + ',' + y.toString() + ')';
                    document.getElementById("scrollButtons").setAttribute('transform', translate);
                }
            }
            else if ((legend['position'] == sunburstLegendPosition.left || legend['position'] == sunburstLegendPosition.right)) {
                if (!isScroll) {
                    var grpXTransform = void 0;
                    var grpYTransform = (document.getElementById(this._id + "_svg").getBoundingClientRect().height / 2) - (legendgrp.getBoundingClientRect().height / 2);
                    if (legend['position'] == sunburstLegendPosition.left)
                        translate = 'translate(' + '0' + ',' + grpYTransform.toString() + ')';
                    else {
                        if (this.isNullOrUndefined(legend['size']['width']))
                            grpXTransform = (this.width - legendgrp.getBoundingClientRect().width - (2 * this.elementSpacing));
                        else {
                            grpXTransform = (this.width - legendgrp.getBoundingClientRect().width - legendgrp.getBoundingClientRect().left - (2 * this.elementSpacing));
                            grpXTransform = grpXTransform - legend['size']['width'];
                        }
                        translate = 'translate(' + grpXTransform.toString() + ',' + grpYTransform.toString() + ')';
                    }
                    legendgrp.setAttribute('transform', translate);
                }
                if (this.legendPages.length > 0) {
                    legendBounds = legendgrp.getBoundingClientRect();
                    scrollButtons = document.getElementById("scrollButtons").getBoundingClientRect();
                    x = legendBounds['width'] / 2 - scrollButtons['width'] / 2;
                    y = legendBounds['height'];
                    translate = 'translate(' + x.toString() + ',' + y.toString() + ')';
                    document.getElementById("scrollButtons").setAttribute('transform', translate);
                }
            }
            else {
                if (legend['title']['visible'] && legend['title']['text'] != "") {
                    titleTransform = (legendItemgrp.getBoundingClientRect().width - document.getElementById(this._id + "_LegendTitleText").getBoundingClientRect().width) / 2;
                    translate = 'translate(' + titleTransform.toString() + ',0)';
                    document.getElementById(this._id + "_LegendTitleText").setAttribute('transform', translate);
                }
            }
            if ((legend['position'] == sunburstLegendPosition.bottom || legend['position'] == sunburstLegendPosition.top) && legend['alignment'] != sunburstAlignment.center) {
                if (legend['alignment'] == sunburstAlignment.near) {
                    x = -Math.abs(legendItemgrp.getBoundingClientRect().left) + this.elementSpacing;
                    translate = 'translate(' + x.toString() + '0)';
                    legendgrp.setAttribute('transform', translate);
                }
                else if (legend['alignment'] == sunburstAlignment.far) {
                    x = Math.abs(legendItemgrp.getBoundingClientRect().right - legendItemgrp.getBoundingClientRect().width);
                    x = x - (this.elementSpacing + this.model.margin.right);
                    translate = 'translate(' + x.toString() + ',0)';
                    legendgrp.setAttribute('transform', translate);
                }
            }
            else if ((legend['position'] == sunburstLegendPosition.left || legend['position'] == sunburstLegendPosition.right) && legend['alignment'] != sunburstAlignment.center) {
                var trans = legendgrp.getAttribute("transform").split(" ");
                if (legend['alignment'] == sunburstAlignment.near) {
                    x = legendgrp.getBoundingClientRect().left + this.elementSpacing;
                    y = this.elementSpacing + this.model.margin.top;
                    translate = trans[0] + ',' + y.toString() + ')';
                    legendgrp.setAttribute('transform', translate);
                }
                else {
                    x = legendgrp.getBoundingClientRect().left + this.elementSpacing;
                    y = this.areaBounds['actualHeight'] - legendgrp.getBoundingClientRect().height + this.elementSpacing + this.model.margin.bottom;
                    translate = trans[0] + ',' + y.toString() + ')';
                    legendgrp.setAttribute('transform', translate);
                }
            }
        };
        SunburstChart.prototype.drawScrollbuttons = function (legend) {
            var legendItemgroup, legendItemsize, position = legend['position'], options, scrollgroup, scrollPath, pageText, pageNumber;
            if (legend['position'] != sunburstLegendPosition.float) {
                scrollgroup = this.createGroup({ 'id': 'scrollButtons' });
                options = {
                    'id': 'scrollUp',
                    'd': 'M 6 0 L 12 12 0 12 Z',
                    'fill': '#3E576F'
                };
                scrollPath = this.drawPath(options);
                scrollgroup.appendChild(scrollPath);
                options = {
                    'id': 'scrollDown',
                    'd': 'M 40.359375 0 L 52.359375 0 46.359375 12 Z',
                    'fill': '#3E576F'
                };
                scrollPath = this.drawPath(options);
                scrollgroup.appendChild(scrollPath);
                options = {
                    'id': 'pageNumber',
                    'x': 15,
                    'y': 10,
                    'fill': legend['font']['color'],
                    'font-size': legend['font']['size'],
                    'font-style': legend['font']['fontStyle'],
                    'font-family': legend['font']['fontFamily'],
                    'font-weight': legend['font']['fontWeight'],
                    'text-anchor': 'start',
                };
                pageNumber = this.currentpageNumber.toString() + '/' + this.totalpageNumbers;
                pageText = this.createText(options, pageNumber);
                scrollgroup.appendChild(pageText);
                this.scrollClick = this.scrollClick.bind(this);
                scrollgroup.addEventListener('click', this.scrollClick, true);
            }
            this.gLegendEle.appendChild(scrollgroup);
        };
        SunburstChart.prototype.scrollClick = function (event) {
            var legend = this.model.legend, targetId = event.target['id'], legendGroupEle, transform, currentPageNumber = this.currentpageNumber, totalPageNumber = this.totalpageNumbers, pageNumber, legendGrp;
            legendGroupEle = document.getElementById(this._id + "legendItem_Group");
            pageNumber = document.getElementById("pageNumber");
            if (targetId == 'scrollDown' && currentPageNumber != totalPageNumber) {
                this.currentpageNumber = currentPageNumber + 1;
                legendGroupEle.parentNode.removeChild(legendGroupEle);
                legendGrp = this.drawlegendItem(legend, this.legendPages[this.currentpageNumber - 1], true);
                this.gLegendEle.appendChild(legendGrp);
                document.getElementById(this._id + "_svg").appendChild(this.gLegendEle);
                pageNumber.textContent = this.currentpageNumber.toString() + '/' + this.totalpageNumbers.toString();
                this.legendClick = this.legendClick.bind(this);
                legendGrp.addEventListener('click', this.legendClick, true);
            }
            else if (targetId == 'scrollUp' && currentPageNumber != 1) {
                this.currentpageNumber = currentPageNumber - 1;
                legendGroupEle.parentNode.removeChild(legendGroupEle);
                legendGrp = this.drawlegendItem(legend, this.legendPages[this.currentpageNumber - 1], true);
                this.gLegendEle.appendChild(legendGrp);
                document.getElementById(this._id + "_svg").appendChild(this.gLegendEle);
                pageNumber.textContent = this.currentpageNumber.toString() + '/' + this.totalpageNumbers.toString();
                this.legendClick = this.legendClick.bind(this);
                legendGrp.addEventListener('click', this.legendClick, true);
            }
            if (legend['position'] == sunburstLegendPosition.top || legend['position'] == sunburstLegendPosition.bottom)
                this.legendPositioning(legend, false);
        };
        SunburstChart.prototype.drawLegendTitle = function (legendTitle, legend) {
            var text = legendTitle['text'], textLocation, textSize = this.measureText(legendTitle['text'], legendTitle['font']), options, elementSpacing = this.elementSpacing, radius = this.circularRadius[this.totallayerCount - 1], x, y, textAnchor, titleText, startX = 0, startY = 0, fill = legendTitle['font'].color;
            fill = this.isNullOrUndefined(fill) ? (this.model.theme.toString().indexOf("dark") != -1 ? "white" : "black") : fill;
            if (legend['position'] == sunburstLegendPosition.bottom) {
                startY = this.areaBounds['actualHeight'] + (textSize['height'] / 2) + elementSpacing + this.yOffset;
            }
            else if (legend['position'] == sunburstLegendPosition.top) {
                startY = elementSpacing + this.yOffset + (textSize['height'] / 2);
                if (!this.isNullOrUndefined(legend['size']['height']))
                    startY = legend['size']['height'] + this.yOffset;
            }
            else if (legend['position'] == sunburstLegendPosition.left || legend['position'] == sunburstLegendPosition.right) {
                startY = elementSpacing + this.model.margin.top;
                startX -= elementSpacing;
            }
            else {
                startY = legend['location'].y;
                startX = legend['location'].x;
            }
            switch (legendTitle['textAlignment']) {
                case 'far':
                    startX += this.LegendBounds['Width'] - textSize['width'] / 2;
                    break;
                case 'near':
                    startX += (textSize['width']) / 2;
                    break;
                case 'center':
                    startX += this.LegendBounds['Width'] / 2;
            }
            this.legendTitleLocation = { 'x': startX, 'y': startY, width: textSize['width'], height: textSize['height'] };
            options = {
                'id': this._id + '_LegendTitleText',
                'x': startX,
                'y': startY,
                'fill': fill,
                'font-size': legendTitle['font']['size'],
                'font-style': legendTitle['font']['fontStyle'],
                'font-family': legendTitle['font']['fontFamily'],
                'font-weight': legendTitle['font']['fontWeight'],
                'text-anchor': 'middle',
            };
            titleText = this.createText(options, legendTitle['text']);
            return titleText;
        };
        SunburstChart.prototype.drawlegendItem = function (legend, legendCollection, isScroll) {
            var xlocation = 0, ylocation = 0, textSize, radius, symbol = this.model.legend.itemStyle, fill, shape, textwidth = 0, legendGroup, textoptions, text, itemPadding;
            this.legendItemGroup = this.createGroup({ 'id': this._id + 'legendItem_Group' });
            if (this.model.legend.position == sunburstLegendPosition.bottom || this.model.legend.position == sunburstLegendPosition.top) {
                return this.drawBottomTopLegend(legend, legendCollection);
            }
            else if (this.model.legend.position == sunburstLegendPosition.left || this.model.legend.position == sunburstLegendPosition.right) {
                return this.drawLeftRightLegend(legend, legendCollection, isScroll);
            }
            else {
                itemPadding = 0;
                if (legend['title']['visible'] && legend['title']['text'] != " ") {
                    textSize = this.measureText(legend['title']['text'], legend['title']['font']);
                    ylocation = legend['location']['y'] + (textSize['height'] / 2) + this.elementSpacing;
                }
                else
                    ylocation = legend['location']['y'];
                xlocation = legend['location']['x'];
                for (var i = 0; i < legendCollection.length; i++) {
                    xlocation += textwidth + itemPadding;
                    radius = Math.sqrt(symbol['width'] * symbol['width'] + symbol['height'] * symbol['height']) / 2;
                    fill = legendCollection[i]['visibility'] ? legendCollection[i]['fill'] : 'grey';
                    shape = this.drawLegendShape(legendCollection[i], xlocation, ylocation, i);
                    if (this.model.legendItemRendering) {
                        var commonEventArgs = $.extend({}, this.common);
                        var data = {
                            location: { x: xlocation, y: ylocation },
                            text: legendCollection[i]['Text'],
                            shape: legendCollection[i]['sunburstLegendShape']
                        };
                        commonEventArgs.data = data;
                        this._trigger("legendItemRendering", commonEventArgs);
                    }
                    textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'] + (radius) + this.model.legend.itemPadding;
                    itemPadding = legend['itemPadding'];
                    legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                    legendGroup['style']['cursor'] = 'pointer';
                    textoptions = {
                        'id': this._id + '_svg' + '_LegendItemText' + i.toString(),
                        'x': xlocation + (radius),
                        'y': ylocation + (((radius * 3) / 4)),
                        'fill': legendCollection[i]['font']['color'],
                        'font-size': legendCollection[i]['font']['size'],
                        'font-style': legendCollection[i]['font']['fontStyle'],
                        'font-family': legendCollection[i]['font']['fontFamily'],
                        'font-weight': legendCollection[i]['font']['fontWeight'],
                        'text-anchor': 'start',
                    };
                    text = this.createText(textoptions, legendCollection[i]['Text']);
                    legendGroup.appendChild(shape);
                    legendGroup.appendChild(text);
                    this.legendItemGroup.appendChild(legendGroup);
                }
                return this.legendItemGroup;
            }
        };
        SunburstChart.prototype.legendClick = function (event) {
            var targetId, childElement, legendText, points = [], legendpoint = {}, circleElement, fill, parentElement, element;
            if (this.rotateanim || this.model.enableAnimation) {
                this.stopAnimation();
            }
            if (this.model.legend.clickAction == sunburstLegendClickAction.toggleSegmentVisibility) {
                targetId = event['target']['id'];
                parentElement = document.getElementById(targetId).parentNode;
                childElement = parentElement.childNodes[1];
                circleElement = parentElement.childNodes[0];
                legendText = childElement.textContent;
                element = document.getElementById(this._id + '_svg');
                element.parentNode.replaceChild(element.cloneNode(false), element);
                for (var i = 0; i < this.legendCollection.length; i++) {
                    if (legendText == this.legendCollection[i]['Text']) {
                        if (this.legendCollection[i]['visibility']) {
                            this.legendCollection[i]['visibility'] = false;
                            legendpoint = { 'x': this.legendCollection[i]['x'], 'y': this.legendCollection[i]['y'], 'fill': this.legendCollection[i]['fill'], 'text': this.legendCollection[i]['Text'], 'point': this.legendCollection[i]['points'] };
                        }
                        else {
                            this.legendCollection[i]['visibility'] = true;
                            points.push({ 'x': this.legendCollection[i]['x'], 'y': this.legendCollection[i]['y'], 'fill': this.legendCollection[i]['fill'], 'text': this.legendCollection[i]['Text'], 'point': this.legendCollection[i]['points'] });
                        }
                    }
                    else {
                        if (this.legendCollection[i]['visibility'])
                            points.push({ 'x': this.legendCollection[i]['x'], 'y': this.legendCollection[i]['y'], 'fill': this.legendCollection[i]['fill'], 'text': this.legendCollection[i]['Text'], 'point': this.legendCollection[i]['points'] });
                    }
                }
                if (this.model.legendItemClick) {
                    var commonEventArgs = $.extend({}, this.common);
                    var data = {
                        location: { x: legendpoint['x'], y: legendpoint['y'] },
                        text: legendText,
                        point: legendpoint['point']
                    };
                    commonEventArgs.data = data;
                    this._trigger("legendItemClick", commonEventArgs);
                }
                this.processData(points);
                this.legendClicked = true;
                this.drawSunburst(this.layerData);
                this.legendClicked = false;
            }
        };
        SunburstChart.prototype.drawLegendShape = function (legendCollection, locX, locY, index) {
            var symbol = this.model.legend.itemStyle, width = symbol['width'], height = symbol['height'], fill = legendCollection['visibility'] ? legendCollection['fill'] : 'grey', shape = legendCollection['sunburstLegendShape'], funcName = 'Path', stroke = legendCollection['visibility'] ? this.model.legend.border.color : 'grey', path, x = locX + (-width / 2), y = locY + (-height / 2), radius, renderOption, eq, xValue, yValue;
            renderOption = {
                'id': this._id + '_svg' + '_LegendItemShape' + index.toString(),
                'fill': fill,
                'stroke': stroke,
                'stroke-width': this.model.legend.border.width,
                'x': x,
                'y': y,
                'width': width,
                'height': height
            };
            switch (shape) {
                case 'circle':
                    funcName = 'Circle';
                    radius = Math.sqrt(height * height + width * width) / 2;
                    renderOption['r'] = radius;
                    renderOption['cx'] = locX;
                    renderOption['cy'] = locY;
                    break;
                case 'diamond':
                    path = 'M' + ' ' + x + ' ' + locY + ' ' +
                        'L' + ' ' + locX + ' ' + (locY + (-height / 2)) + ' ' +
                        'L' + ' ' + (locX + (width / 2)) + ' ' + locY + ' ' +
                        'L' + ' ' + locX + ' ' + (locY + (height / 2)) + ' ' +
                        'L' + ' ' + x + ' ' + locY + ' z';
                    renderOption['d'] = path;
                    break;
                case 'rectangle':
                    path = 'M' + ' ' + x + ' ' + (locY + (-height / 2)) + ' ' +
                        'L' + ' ' + (locX + (width / 2)) + ' ' + (locY + (-height / 2)) + ' ' +
                        'L' + ' ' + (locX + (width / 2)) + ' ' + (locY + (height / 2)) + ' ' +
                        'L' + ' ' + x + ' ' + (locY + (height / 2)) + ' ' +
                        'L' + ' ' + x + ' ' + (locY + (-height / 2)) + ' z';
                    renderOption['d'] = path;
                    break;
                case 'triangle':
                    path = 'M' + ' ' + x + ' ' + (locY + (height / 2)) + ' ' +
                        'L' + ' ' + locX + ' ' + (locY + (-height / 2)) + ' ' +
                        'L' + ' ' + (locX + (width / 2)) + ' ' + (locY + (height / 2)) + ' ' +
                        'L' + ' ' + x + ' ' + (locY + (height / 2)) + ' z';
                    renderOption['d'] = path;
                    break;
                case 'pentagon':
                    eq = 72;
                    radius = Math.sqrt(height * height + width * width) / 2;
                    for (var i = 0; i <= 5; i++) {
                        xValue = radius * Math.cos((Math.PI / 180) * (i * eq));
                        yValue = radius * Math.sin((Math.PI / 180) * (i * eq));
                        if (i === 0) {
                            path = 'M' + ' ' + (locX + xValue) + ' ' + (locY + yValue) + ' ';
                        }
                        else {
                            path = path.concat('L' + ' ' + (locX + xValue) + ' ' + (locY + yValue) + ' ');
                        }
                    }
                    path = path.concat('Z');
                    renderOption['d'] = path;
                    break;
                case 'cross':
                    path = "M" + " " + (locX - width) + " " + locY + " " + "L" + " " + (locX + width) + " " + locY + " " +
                        "M" + " " + locX + " " + (locY + height) + " " + "L" + " " + locX + " " + (locY - height);
                    renderOption['d'] = path;
                    renderOption['stroke'] = fill;
                    break;
            }
            return this['draw' + funcName](renderOption);
        };
        SunburstChart.prototype.drawBottomTopLegend = function (legend, legendCollection) {
            var legendTitleLocation = this.legendTitleLocation, margin = this.model.margin, yOffset = this.yOffset, xlocation = 0, itemPadding = 0, elementSpacing = this.elementSpacing, legendTextcolor, ylocation, textSize, radius, symbol = this.model.legend.itemStyle, fill = this.model.legend.font.color, circleoptions, shape, textwidth = 0, legendGroup, textoptions, text;
            legendTextcolor = this.isNullOrUndefined(fill) ? (this.model.theme.toString().indexOf("dark") != -1 ? "white" : "black") : fill;
            if (this.isNullOrUndefined(legend['rowCount'] || legend['columnCount'])) {
                this.legendItemGroup = this.createGroup({ 'id': this._id + 'legendItem_Group' });
                if (legend['title'].visible && legend['title'].text != " ") {
                    textSize = this.measureText(legend['title']['text'], legend['title']['font']);
                    if (legend['position'] == sunburstLegendPosition.bottom)
                        ylocation = this.areaBounds['actualHeight'] + (textSize['height']) + (2 * elementSpacing) + yOffset;
                    else
                        ylocation = yOffset + textSize['height'] + (2 * elementSpacing);
                }
                else {
                    if (legend['position'] = sunburstLegendPosition.bottom)
                        ylocation = elementSpacing + margin['top'] + (2 * this.circularRadius[this.totallayerCount - 1]) + yOffset + elementSpacing;
                    else
                        ylocation = elementSpacing + margin['top'] + yOffset + elementSpacing;
                }
                for (var i = 0; i < legendCollection.length; i++) {
                    xlocation += elementSpacing + textwidth + itemPadding;
                    textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'];
                    if (xlocation + elementSpacing + textwidth + itemPadding > this.width) {
                        xlocation = elementSpacing;
                        ylocation += this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['height'];
                    }
                    radius = Math.sqrt(symbol['width'] * symbol['width'] + symbol['height'] * symbol['height']) / 2;
                    fill = legendCollection[i]['visibility'] ? legendCollection[i]['fill'] : 'grey';
                    if (xlocation + textwidth > 900) {
                        ylocation = ylocation + this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['height'] + elementSpacing;
                        xlocation = elementSpacing;
                    }
                    this.findLegendPosition(radius, xlocation, ylocation, legend, legendCollection[i]);
                    if (this.legendInRegion) {
                        if (this.model.legendItemRendering) {
                            var commonEventArgs = $.extend({}, this.common);
                            var data = {
                                location: { x: xlocation, y: ylocation },
                                text: legendCollection[i]['Text'],
                                shape: legendCollection[i]['sunburstLegendShape']
                            };
                            commonEventArgs.data = data;
                            this._trigger("legendItemRendering", commonEventArgs);
                        }
                        legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                        legendGroup['style']['cursor'] = 'pointer';
                        shape = this.drawLegendShape(legendCollection[i], xlocation, ylocation, i);
                        legendGroup.appendChild(shape);
                        textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'] + (radius * 2);
                        textoptions = {
                            'id': this._id + '_svg' + '_LegendItemText' + i.toString(),
                            'x': xlocation + elementSpacing,
                            'y': ylocation + ((radius * 3) / 4),
                            'fill': legendTextcolor,
                            'font-size': legendCollection[i]['font']['size'],
                            'font-style': legendCollection[i]['font']['fontStyle'],
                            'font-family': legendCollection[i]['font']['fontFamily'],
                            'font-weight': legendCollection[i]['font']['fontWeight'],
                            'text-anchor': 'start',
                        };
                        itemPadding = legend['itemPadding'];
                        text = this.createText(textoptions, legendCollection[i]['Text']);
                        legendGroup.appendChild(text);
                        this.legendItemGroup.appendChild(legendGroup);
                    }
                    else {
                        this.legendPaging(legend, i, legendCollection);
                        break;
                    }
                }
            }
            else {
                var rowCount = legend['rowCount'], columnCount = legend['columnCount'], legendItemCount = legendCollection.length, Num = 0, renderasper = void 0, count = void 0, currentX = 10, currentY = 26, startX = 10;
                if (legend['title'].visible && legend['title'].text != " ") {
                    textSize = this.measureText(legend['title']['text'], legend['title']['font']);
                    if (legend['position'] == sunburstLegendPosition.bottom)
                        ylocation = (2 * elementSpacing) + margin['top'] + (2 * this.circularRadius[this.totallayerCount - 1]) + yOffset + (textSize['height']);
                    else
                        ylocation = (2 * elementSpacing) + margin['top'] + yOffset + textSize['height'];
                }
                else {
                    if (legend['position'] == sunburstLegendPosition.bottom)
                        ylocation = (2 * elementSpacing) + margin['top'] + (2 * this.circularRadius[this.totallayerCount - 1]) + yOffset;
                    else
                        ylocation = (2 * elementSpacing) + margin['top'] + yOffset;
                }
                for (var i = 0; i < legendItemCount; i++) {
                    fill = legendCollection[i]['visibility'] ? legendCollection[i]['fill'] : 'grey';
                    legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                    radius = Math.sqrt(symbol['width'] * symbol['width'] + symbol['height'] * symbol['height']) / 2;
                    if (currentX + legendCollection[i]['Bounds']['Width'] > this.LegendBounds['Width'] + startX) {
                        currentX = textwidth = 10;
                        ylocation += elementSpacing + legendCollection[i]['Bounds']['Height'];
                    }
                    this.findLegendPosition(radius, currentX, ylocation, legend, legendCollection[i]);
                    if (this.legendInRegion) {
                        if (this.model.legendItemRendering) {
                            var commonEventArgs = $.extend({}, this.common);
                            var data = {
                                location: { x: currentX, y: ylocation },
                                text: legendCollection[i]['Text'],
                                shape: legendCollection[i]['sunburstLegendShape']
                            };
                            commonEventArgs.data = data;
                            this._trigger("legendItemRendering", commonEventArgs);
                        }
                        textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'] + (radius * 2);
                        legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                        legendGroup['style']['cursor'] = 'pointer';
                        shape = this.drawLegendShape(legendCollection[i], currentX, ylocation, i);
                        legendGroup.appendChild(shape);
                        textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'] + (radius * 2);
                        textoptions = {
                            'id': this._id + '_svg' + '_LegendItemText' + i.toString(),
                            'x': currentX + elementSpacing,
                            'y': ylocation + (((radius * 3) / 4)),
                            'fill': legendTextcolor,
                            'font-size': legendCollection[i]['font']['size'],
                            'font-style': legendCollection[i]['font']['fontStyle'],
                            'font-family': legendCollection[i]['font']['fontFamily'],
                            'font-weight': legendCollection[i]['font']['fontWeight'],
                            'text-anchor': 'start',
                        };
                        currentX += textwidth + elementSpacing;
                        text = this.createText(textoptions, legendCollection[i]['Text']);
                        legendGroup.appendChild(text);
                        this.legendItemGroup.appendChild(legendGroup);
                    }
                    else {
                        this.legendPaging(legend, i, legendCollection);
                        break;
                    }
                }
            }
            return this.legendItemGroup;
        };
        SunburstChart.prototype.drawLeftRightLegend = function (legend, legendCollection, isScroll) {
            var xlocation = 0, itemPadding = 0, legendTextcolor, ylocation = 0, textSize, radius, symbol = this.model.legend.itemStyle, fill = this.model.legend.font.color, shape, textwidth = 0, legendGroup, textoptions, text, textheight = 0, elementSpacing = this.elementSpacing, margin = this.model.margin;
            legendTextcolor = this.isNullOrUndefined(fill) ? (this.model.theme.toString().indexOf("dark") != -1 ? "white" : "black") : fill;
            if (this.isNullOrUndefined(legend['rowCount'] || legend['columnCount'])) {
                if (legend['title'].visible && legend['title'].text != " ") {
                    textSize = this.measureText(legend['title']['text'], legend['title']['font']);
                    textheight = (textSize['height']) + elementSpacing;
                }
                ylocation = textheight + elementSpacing + itemPadding;
                xlocation = elementSpacing + margin['left'];
                for (var i = 0; i < legendCollection.length; i++) {
                    textSize = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font']);
                    radius = Math.sqrt(symbol['width'] * symbol['width'] + symbol['height'] * symbol['height']) / 2;
                    fill = legendCollection[i]['visibility'] ? legendCollection[i]['fill'] : 'grey';
                    this.findLegendPosition(radius, xlocation, ylocation, legend, legendCollection[i]);
                    if (this.legendInRegion) {
                        if (this.model.legendItemRendering) {
                            var commonEventArgs = $.extend({}, this.common);
                            var data = {
                                location: { x: xlocation, y: ylocation },
                                text: legendCollection[i]['Text'],
                                shape: legendCollection[i]['sunburstLegendShape']
                            };
                            commonEventArgs.data = data;
                            this._trigger("legendItemRendering", commonEventArgs);
                        }
                        shape = this.drawLegendShape(legendCollection[i], xlocation, ylocation, i);
                        textwidth += textSize['width'] + (radius);
                        legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                        textoptions = {
                            'id': this._id + '_svg' + '_LegendItemText' + i.toString(),
                            'x': xlocation + (this.elementSpacing),
                            'y': ylocation + (((radius * 3) / 4)),
                            'fill': legendTextcolor,
                            'font-size': legendCollection[i]['font']['size'],
                            'font-style': legendCollection[i]['font']['fontStyle'],
                            'font-family': legendCollection[i]['font']['fontFamily'],
                            'font-weight': legendCollection[i]['font']['fontWeight'],
                            'text-anchor': 'start',
                        };
                        itemPadding = legend['itemPadding'];
                        ylocation += textSize['height'] / 2 + this.elementSpacing;
                        text = this.createText(textoptions, legendCollection[i]['Text']);
                        legendGroup.appendChild(shape);
                        legendGroup.appendChild(text);
                        this.legendItemGroup.appendChild(legendGroup);
                    }
                    else {
                        this.legendPaging(legend, i, legendCollection);
                        break;
                    }
                }
                return this.legendItemGroup;
            }
            else {
                var rowCount = legend['rowCount'], columnCount = legend['columnCount'], legendItemCount = legendCollection.length, Num = 0, renderasper = void 0, count = void 0, currentX = 10, currentY = 26, startX = 10, maxWidth = 0;
                if (!isScroll)
                    this.LegendBounds['Height'] += this.measureText(legend['title']['text'], legend['title']['font'])['height'];
                if (this.model.legend.title.visible && this.model.legend.title.text != " ") {
                    textSize = this.measureText(legend['title']['text'], legend['title']['font']);
                    textheight = (textSize['height']);
                }
                ylocation = textheight + this.elementSpacing;
                xlocation = this.elementSpacing + this.model.margin.left;
                for (var i = 0; i < legendItemCount; i++) {
                    fill = legendCollection[i]['visibility'] ? legendCollection[i]['fill'] : 'grey';
                    legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                    radius = Math.sqrt(symbol['width'] * symbol['width'] + symbol['height'] * symbol['height']) / 2;
                    if (((currentY + legendCollection[i]['Bounds']['Height'] + legend['itemPadding']) + textheight + 2 > this.LegendBounds['Height'] + 26)) {
                        currentY = 26;
                        currentX += maxWidth + (legend['itemPadding']);
                    }
                    currentY += (legendCollection[i]['Bounds']['Height'] / 2) + legend['itemPadding'];
                    maxWidth = Math.max(maxWidth, legendCollection[i]['Bounds']['Width']);
                    this.findLegendPosition(radius, currentX, currentY, legend, legendCollection[i]);
                    if (this.legendInRegion) {
                        if (this.model.legendItemRendering) {
                            var commonEventArgs = $.extend({}, this.common);
                            var data = {
                                location: { x: currentX, y: currentY },
                                text: legendCollection[i]['Text'],
                                shape: legendCollection[i]['sunburstLegendShape']
                            };
                            commonEventArgs.data = data;
                            this._trigger("legendItemRendering", commonEventArgs);
                        }
                        textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'] + (radius * 2);
                        legendGroup = this.createGroup({ id: this._id + '_svg' + '_Legend' + i.toString() });
                        legendGroup['style']['cursor'] = 'pointer';
                        shape = this.drawLegendShape(legendCollection[i], currentX, currentY, i);
                        legendGroup.appendChild(shape);
                        textwidth = this.measureText(legendCollection[i]['Text'], legendCollection[i]['font'])['width'] + (radius * 2);
                        textoptions = {
                            'id': this._id + '_svg' + '_LegendItemText' + i.toString(),
                            'x': currentX + radius,
                            'y': currentY + (((radius * 3) / 4)),
                            'fill': legendTextcolor,
                            'font-size': legendCollection[i]['font']['size'],
                            'font-style': legendCollection[i]['font']['fontStyle'],
                            'font-family': legendCollection[i]['font']['fontFamily'],
                            'font-weight': legendCollection[i]['font']['fontWeight'],
                            'text-anchor': 'start',
                        };
                        text = this.createText(textoptions, legendCollection[i]['Text']);
                        legendGroup.appendChild(text);
                        this.legendItemGroup.appendChild(legendGroup);
                    }
                    else {
                        this.legendPaging(legend, i, legendCollection);
                        break;
                    }
                }
                return this.legendItemGroup;
            }
        };
        SunburstChart.prototype.legendPaging = function (legend, drawnLegendCount, legendCollection) {
            var totalLegendCount = this.legendCollection.length, visibleLegendCount = drawnLegendCount, pages;
            this.legendPages = [];
            this.currentpageNumber = 1;
            this.totalpageNumbers = drawnLegendCount == 0 ? totalLegendCount : (totalLegendCount % drawnLegendCount) == 0 ? Math.floor(totalLegendCount / drawnLegendCount) : Math.floor(totalLegendCount / drawnLegendCount) + 1;
            for (var i = 0; i < totalLegendCount; i = i + visibleLegendCount) {
                pages = [];
                for (var j = 0; j < visibleLegendCount; j++) {
                    if ((i + j) < totalLegendCount)
                        pages.push(legendCollection[i + j]);
                }
                if (visibleLegendCount == 0) {
                    pages.push(legendCollection[i]);
                    this.legendPages.push(pages);
                    break;
                }
                this.legendPages.push(pages);
            }
        };
        SunburstChart.prototype.findLegendPosition = function (radius, xlocation, ylocation, legend, legendItem) {
            var elementSpacing = this.elementSpacing, textsize, maximum = 20 / 100;
            if (legend['position'] == sunburstLegendPosition.bottom) {
                if (radius + ylocation + elementSpacing > this.height - elementSpacing) {
                    this.legendInRegion = false;
                }
                else
                    this.legendInRegion = true;
            }
            else if (legend['position'] == sunburstLegendPosition.top) {
                if ((this.yOffset + (maximum * this.height) - this.elementSpacing - radius) > ylocation)
                    this.legendInRegion = true;
                else
                    this.legendInRegion = false;
            }
            else if ((legend['position'] == sunburstLegendPosition.left) || (legend['position'] == sunburstLegendPosition.right)) {
                if (!this.isNullOrUndefined(legend['rowCount']) || !this.isNullOrUndefined(legend['columnCount'])) {
                    textsize = this.measureText(legendItem['Text'], legendItem['font']);
                    if ((maximum * this.width + elementSpacing) > (xlocation + radius + textsize['width']))
                        this.legendInRegion = true;
                    else
                        this.legendInRegion = false;
                }
                else {
                    if (radius + ylocation + elementSpacing > this.height - elementSpacing)
                        this.legendInRegion = false;
                    else
                        this.legendInRegion = true;
                }
            }
        };
        SunburstChart.prototype.drawDatalabel = function (layerData) {
            this.dataLabelGroupEle = this.createGroup({ 'id': this._id + '_svg_DataLabelGroup' });
            for (var i = 0; i < layerData.length; i++) {
                var layerPoints = layerData[i];
                var point = void 0, text = void 0, textsize = void 0, sunburstStartAngle = void 0, midAngle = void 0, midPoint = void 0, childMidPoint = void 0, bounds = void 0, radius = void 0, size = void 0, position = void 0, positionX = void 0, positionY = void 0, textAnchor = void 0, textOptions = void 0, textele = void 0, startPoint = void 0, textOffset = void 0, startX = void 0, startY = void 0, midX = void 0, midY = void 0, childMidX = void 0, childMidY = void 0, layerNumber = void 0, dMidX = void 0, dMidY = void 0, model = this.model, sliceRadius = this.circularRadius[0] - this.innerRadius[0], sunburstMidPoint = void 0;
                for (var j = 0; j < layerPoints.point.length; j++) {
                    point = layerPoints.point[j];
                    if (point['y'] != null) {
                        layerNumber = layerPoints['layerNumber'];
                        text = point['text'] ? point['text'] : (model['dataSource']) ? point['x'] : point['y'].toString();
                        textsize = this.measureText(text, model['dataLabelSettings'].font);
                        sunburstStartAngle = -0.5 * Math.PI;
                        radius = this.circularRadius[layerPoints.layerNumber - 1];
                        textOffset = textsize['height'] / 2;
                        startX = this.circleCenterX;
                        startY = this.circleCenterY;
                        midAngle = point['midAngle'] + sunburstStartAngle;
                        midX = this.getXcordinate(startX, radius, midAngle);
                        midY = this.getYcordinate(startY, radius, midAngle);
                        dMidX = this.getXcordinate(startX, this.innerRadius[layerNumber - 1], midAngle);
                        dMidY = this.getYcordinate(startY, this.innerRadius[layerNumber - 1], midAngle);
                        sunburstMidPoint = { 'dMidX': dMidX, 'dMidY': dMidY };
                        if (layerPoints.layerNumber > 1) {
                            childMidX = this.getXcordinate(startX, this.circularRadius[layerPoints.layerNumber - 2], midAngle);
                            childMidY = this.getYcordinate(startY, this.circularRadius[layerPoints.layerNumber - 2], midAngle);
                        }
                        midPoint = { midX: midX, midY: midY };
                        childMidPoint = { childMidX: childMidX, childMidY: childMidY };
                        bounds = { midPoint: midPoint, childMidPoint: childMidPoint, 'sunburstMidPoint': sunburstMidPoint };
                        startPoint = { startX: startX, startY: startY };
                        size = {
                            width: this.width, height: this.height
                        };
                        position = this.calculateLabelPosition(startPoint, bounds);
                        positionX = position['positionX'];
                        positionY = position['positionY'];
                        textAnchor = "middle";
                        textOptions = this.textOptions(point, positionX, positionY, textsize, textAnchor, layerPoints.layerNumber, j);
                        if (this.isNullOrUndefined(model['dataLabelSettings'].template)) {
                            var commonDataLabelEventArgs = $.extend({}, this.common);
                            commonDataLabelEventArgs.data = { Text: text, location: { x: positionX, y: positionY }, pointIndex: point['pointIndex'] };
                            this._trigger("dataLabelRendering", commonDataLabelEventArgs);
                            if (model['dataLabelSettings'].labelRotationMode == sunburstLabelRotationMode.normal && model['dataLabelSettings'].labelOverflowMode != sunburstLabelOverflowMode.none && this.isNullOrUndefined(model['dataLabelSettings'].template)) {
                                text = this.horizontalTrim(textOptions, textsize['width'], commonDataLabelEventArgs.data.Text, startX, startY, point, this.model.dataLabelSettings.font, layerNumber - 1);
                            }
                            else if (model['dataLabelSettings'].labelRotationMode == sunburstLabelRotationMode.angle && model['dataLabelSettings'].labelOverflowMode != sunburstLabelOverflowMode.none && this.isNullOrUndefined(model['dataLabelSettings'].template)) {
                                text = this.rotatedTrim(textOptions, textsize['width'], commonDataLabelEventArgs.data.Text, sliceRadius, point);
                            }
                            if (text != '') {
                                textele = this.createText(textOptions, text);
                                this.dataLabelGroupEle.appendChild(textele);
                            }
                        }
                        else {
                            this.labelTemplate(positionX, positionY, textOptions, textsize, point);
                        }
                    }
                }
            }
            return this.dataLabelGroupEle;
        };
        SunburstChart.prototype.labelTemplate = function (positionX, positionY, textOptions, textsize, point) {
            var areaBoundsX, areaBoundsY, areaBoundsWidth, areaBoundsHeight, xPosition = 0, yPosition = 0, sunburstContainer = this._id, style = { 'interior': this.model.dataLabelSettings.fill, 'opacity': 1, 'borderColor': 'white', 'borderWidth': 0.1 }, color, width, height, templateContainer, templategroup, left, top, right, backgroundColor;
            if (!this.isNullOrUndefined(style) && style['interior']) {
                color = style['interior'];
            }
            else {
                color = "transparent";
            }
            templategroup = document.getElementById(this._id + "_svg_DataLabelGroup");
            if (this.isNullOrUndefined(templategroup)) {
                templategroup = document.createElement('div');
                templategroup.id = this._id + "_svg_DataLabelGroup";
                templategroup.style.zIndex = '5000';
                document.getElementById(this._id).appendChild(templategroup);
            }
            templateContainer = document.createElement('div');
            templateContainer.id = 'template_ele_' + this._id;
            this.clonenode = document.getElementById(this.model.dataLabelSettings.template).cloneNode(true);
            this.clonenode['style'].display = 'block';
            this.clonenode['innerHTML'] = this.parseTemplate(this.clonenode, point);
            templateContainer.style.backgroundColor = color;
            templateContainer.style.display = 'block';
            templateContainer.style.cursor = 'default';
            templateContainer.style.position = 'absolute';
            templateContainer.appendChild(this.clonenode);
            document.getElementById(this._id + "_svg_DataLabelGroup").appendChild(templateContainer);
            left = templateContainer.clientWidth;
            top = templateContainer.clientHeight;
            templateContainer.style.left = (textOptions['x'] - left / 2).toString() + 'px';
            templateContainer.style.top = (textOptions['y'] - top / 2).toString() + 'px';
        };
        SunburstChart.prototype.parseTemplate = function (clonenode, point) {
            var str;
            str = clonenode.innerHTML;
            if (str.search('#point.x#') != -1) {
                str = str.replace('#point.x#', point['x'].toString());
            }
            if (str.search('#point.y#') != -1) {
                str = str.replace('#point.y#', point['y'].toString());
            }
            if (str.search('#point.text#') != -1) {
                if (!this.isNullOrUndefined(point['text'])) {
                    str = str.replace('#point.text#', point['text'].toString());
                }
                else {
                    str = str.replace('#point.text#', point['y'].toString());
                }
            }
            return str;
        };
        SunburstChart.prototype.rotatedTrim = function (textOptions, textSize, text, radius, point) {
            var ellipsis = '...', model = this.model;
            while (radius - model['segmentBorder'].width < textSize) {
                if (model['dataLabelSettings'].labelOverflowMode == sunburstLabelOverflowMode.trim) {
                    text = this.trimText(text, text.length, ellipsis);
                    if (text == '...') {
                        text = '';
                        break;
                    }
                    else
                        textSize = this.measureText(text, model['dataLabelSettings'].font)['width'];
                }
                else {
                    text = '';
                    break;
                }
            }
            return text;
        };
        SunburstChart.prototype.trimText = function (text, maxLength, ellipsis) {
            var length, trimmedText;
            maxLength--;
            length = maxLength - ellipsis.length;
            trimmedText = text.substr(0, length);
            return trimmedText + ellipsis;
        };
        SunburstChart.prototype.horizontalTrim = function (textOptions, datalabelLength, datalabelText, startX, startY, point, datalabelFont, layerNumber) {
            var sunburstStartAngle = -0.5 * Math.PI, startAngle, endAngle, pointStartAngle, pointEndAngle, ellipsis = '...', totalDegree, distanceFromCenter, textinRegion, model = this.model, position;
            position = this.calculatePosition(textOptions, datalabelLength, startX, startY);
            startAngle = model['startAngle'];
            endAngle = model['endAngle'];
            totalDegree = endAngle - startAngle;
            startAngle = (Math.atan2(position['rightEndY'], position['rightEndX']) - sunburstStartAngle) % (2 * Math.PI);
            if (startAngle < 0)
                startAngle = 2 * Math.PI + startAngle;
            endAngle = (Math.atan2(position['leftEndY'], position['leftEndX']) - sunburstStartAngle) % (2 * Math.PI);
            if (endAngle < 0)
                endAngle = 2 * Math.PI + endAngle;
            pointStartAngle = parseFloat(point['start'].toFixed(14));
            pointEndAngle = parseFloat(point['end'].toFixed(14));
            if (totalDegree > 0) {
                pointStartAngle = pointStartAngle;
                pointEndAngle = pointEndAngle;
            }
            else {
                pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
            }
            if (startAngle < 0 && (endAngle > 0 || endAngle == null)) {
                pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                pointEndAngle = pointEndAngle <= 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
            }
            if (this.circleCenterX < textOptions['x'])
                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position['rightEndX']), 2) + Math.pow(Math.abs(position['rightEndY']), 2));
            else
                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position['leftEndX']), 2) + Math.pow(Math.abs(position['leftEndY']), 2));
            if (totalDegree < 0)
                pointEndAngle = [pointStartAngle, pointStartAngle = pointEndAngle][0];
            if ((endAngle >= pointStartAngle && endAngle <= pointEndAngle) && (startAngle >= pointStartAngle && startAngle <= pointEndAngle) &&
                (distanceFromCenter <= this.circularRadius[layerNumber] - model['segmentBorder'].width && distanceFromCenter > 0)) {
                textinRegion = true;
            }
            else {
                if (model['dataLabelSettings'].labelOverflowMode == sunburstLabelOverflowMode.hide) {
                    return '';
                }
                else {
                    textinRegion = false;
                    while (!textinRegion) {
                        datalabelText = this.trimText(datalabelText, datalabelText.length, ellipsis);
                        if (datalabelText == ellipsis) {
                            datalabelText = '';
                            break;
                        }
                        datalabelLength = this.measureText(datalabelText, model['dataLabelSettings'].font)['width'];
                        position = this.calculatePosition(textOptions, datalabelLength, startX, startY);
                        startAngle = (Math.atan2(position['rightEndY'], position['rightEndX']) - sunburstStartAngle) % (2 * Math.PI);
                        if (startAngle < 0)
                            startAngle = 2 * Math.PI + startAngle;
                        endAngle = (Math.atan2(position['leftEndY'], position['leftEndX']) - sunburstStartAngle) % (2 * Math.PI);
                        if (endAngle < 0)
                            endAngle = 2 * Math.PI + endAngle;
                        if (this.circleCenterX < textOptions['x'])
                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position['rightEndX']), 2) + Math.pow(Math.abs(position['rightEndY']), 2));
                        else
                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position['leftEndX']), 2) + Math.pow(Math.abs(position['leftEndY']), 2));
                        if ((endAngle >= pointStartAngle && endAngle <= pointEndAngle) && (startAngle >= pointStartAngle && startAngle <= pointEndAngle) && (distanceFromCenter <= this.circularRadius[layerNumber] && distanceFromCenter > 0)) {
                            textinRegion = true;
                        }
                    }
                }
            }
            return datalabelText;
        };
        SunburstChart.prototype.calculatePosition = function (textOptions, datalabelLength, startX, startY) {
            var rightEndX, rightEndY, leftEndX, leftEndY;
            rightEndX = (textOptions['x'] + (datalabelLength / 2)) - startX;
            rightEndY = textOptions['y'] - startY;
            leftEndX = (textOptions['x'] - (datalabelLength / 2)) - startX;
            leftEndY = textOptions['y'] - startY;
            return { 'rightEndX': rightEndX, 'rightEndY': rightEndY, 'leftEndX': leftEndX, 'leftEndY': leftEndY };
        };
        SunburstChart.prototype.textOptions = function (point, positionX, positionY, textsize, textAnchor, layerNumber, pointIndex) {
            var degree, correctAngle, angle = 0, textOptions, hexaValue, pointColor, dataLabelFont = this.model.dataLabelSettings.font, textColor = dataLabelFont['color'];
            if (this.model.dataLabelSettings.labelRotationMode == sunburstLabelRotationMode.angle) {
                if (point.midAngle > (2 * Math.PI) || point.midAngle < 0) {
                    correctAngle = this.findAngle(point.midAngle);
                }
                if (point.midAngle <= Math.PI || !this.isNullOrUndefined(correctAngle)) {
                    if (this.isNullOrUndefined(correctAngle))
                        angle = (((point.startAngle - (Math.PI / 2)) + (point.endAngle - (Math.PI / 2))) / 2) * (180 / Math.PI);
                    else {
                        if (correctAngle <= Math.PI)
                            angle = (((point.startAngle - (Math.PI / 2)) + (point.endAngle - (Math.PI / 2))) / 2) * (180 / Math.PI);
                        else
                            angle = ((((point.startAngle + point.endAngle) / 2) * 180) / Math.PI) - 270;
                    }
                }
                else {
                    angle = ((((point.startAngle + point.endAngle) / 2) * 180) / Math.PI) - 270;
                }
            }
            pointColor = this._colorNameToHex(point.fill);
            if (this.isNullOrUndefined(textColor) || textColor == "") {
                var rgbValue = this._hexToRgb(pointColor);
                var contrast = Math.round(((parseInt(rgbValue.r) * 299) + (parseInt(rgbValue.g) * 587) + (parseInt(rgbValue.b) * 114)) / 1000);
                textColor = (contrast >= 128) ? "black" : "white";
            }
            textOptions = {
                'id': this._id + '_legendIndex' + point.legendIndex + '_layerNumber' + layerNumber.toString() + '_pointIndex' + pointIndex.toString(),
                'x': positionX,
                'y': positionY + (textsize.height / 4),
                'fill': textColor,
                'font-size': dataLabelFont['size'],
                'font-family': dataLabelFont['fontFamily'],
                'font-style': dataLabelFont['fontStyle'],
                'font-weight': dataLabelFont['fontWeight'],
                'opacity': dataLabelFont['opacity'],
                'text-anchor': textAnchor,
                'cursor': 'default',
                'transform': 'rotate(' + angle + ',' + (positionX) + ',' + (positionY) + ')',
            };
            return textOptions;
        };
        SunburstChart.prototype._colorNameToHex = function (colour) {
            var color = colour;
            var colours = {
                "aliceblue": "#f0f8ff", "antiquewhite": "#faebd7", "aqua": "#00ffff", "aquamarine": "#7fffd4", "azure": "#f0ffff",
                "beige": "#f5f5dc", "bisque": "#ffe4c4", "black": "#000000", "blanchedalmond": "#ffebcd", "blue": "#0000ff", "blueviolet": "#8a2be2", "brown": "#a52a2a", "burlywood": "#deb887",
                "cadetblue": "#5f9ea0", "chartreuse": "#7fff00", "chocolate": "#d2691e", "coral": "#ff7f50", "cornflowerblue": "#6495ed", "cornsilk": "#fff8dc", "crimson": "#dc143c", "cyan": "#00ffff",
                "darkblue": "#00008b", "darkcyan": "#008b8b", "darkgoldenrod": "#b8860b", "darkgray": "#a9a9a9", "darkgreen": "#006400", "darkkhaki": "#bdb76b", "darkmagenta": "#8b008b", "darkolivegreen": "#556b2f",
                "darkorange": "#ff8c00", "darkorchid": "#9932cc", "darkred": "#8b0000", "darksalmon": "#e9967a", "darkseagreen": "#8fbc8f", "darkslateblue": "#483d8b", "darkslategray": "#2f4f4f", "darkturquoise": "#00ced1",
                "darkviolet": "#9400d3", "deeppink": "#ff1493", "deepskyblue": "#00bfff", "dimgray": "#696969", "dodgerblue": "#1e90ff",
                "firebrick": "#b22222", "floralwhite": "#fffaf0", "forestgreen": "#228b22", "fuchsia": "#ff00ff",
                "gainsboro": "#dcdcdc", "ghostwhite": "#f8f8ff", "gold": "#ffd700", "goldenrod": "#daa520", "gray": "#808080", "green": "#008000", "greenyellow": "#adff2f",
                "honeydew": "#f0fff0", "hotpink": "#ff69b4",
                "indianred ": "#cd5c5c", "indigo ": "#4b0082", "ivory": "#fffff0", "khaki": "#f0e68c",
                "lavender": "#e6e6fa", "lavenderblush": "#fff0f5", "lawngreen": "#7cfc00", "lemonchiffon": "#fffacd", "lightblue": "#add8e6", "lightcoral": "#f08080", "lightcyan": "#e0ffff", "lightgoldenrodyellow": "#fafad2",
                "lightgrey": "#d3d3d3", "lightgreen": "#90ee90", "lightpink": "#ffb6c1", "lightsalmon": "#ffa07a", "lightseagreen": "#20b2aa", "lightskyblue": "#87cefa", "lightslategray": "#778899", "lightsteelblue": "#b0c4de",
                "lightyellow": "#ffffe0", "lime": "#00ff00", "limegreen": "#32cd32", "linen": "#faf0e6",
                "magenta": "#ff00ff", "maroon": "#800000", "mediumaquamarine": "#66cdaa", "mediumblue": "#0000cd", "mediumorchid": "#ba55d3", "mediumpurple": "#9370d8", "mediumseagreen": "#3cb371", "mediumslateblue": "#7b68ee",
                "mediumspringgreen": "#00fa9a", "mediumturquoise": "#48d1cc", "mediumvioletred": "#c71585", "midnightblue": "#191970", "mintcream": "#f5fffa", "mistyrose": "#ffe4e1", "moccasin": "#ffe4b5",
                "navajowhite": "#ffdead", "navy": "#000080",
                "oldlace": "#fdf5e6", "olive": "#808000", "olivedrab": "#6b8e23", "orange": "#ffa500", "orangered": "#ff4500", "orchid": "#da70d6",
                "palegoldenrod": "#eee8aa", "palegreen": "#98fb98", "paleturquoise": "#afeeee", "palevioletred": "#d87093", "papayawhip": "#ffefd5", "peachpuff": "#ffdab9", "peru": "#cd853f", "pink": "#ffc0cb", "plum": "#dda0dd", "powderblue": "#b0e0e6", "purple": "#800080",
                "red": "#ff0000", "rosybrown": "#bc8f8f", "royalblue": "#4169e1",
                "saddlebrown": "#8b4513", "salmon": "#fa8072", "sandybrown": "#f4a460", "seagreen": "#2e8b57", "seashell": "#fff5ee", "sienna": "#a0522d", "silver": "#c0c0c0", "skyblue": "#87ceeb", "slateblue": "#6a5acd", "slategray": "#708090", "snow": "#fffafa", "springgreen": "#00ff7f", "steelblue": "#4682b4",
                "tan": "#d2b48c", "teal": "#008080", "thistle": "#d8bfd8", "tomato": "#ff6347", "turquoise": "#40e0d0",
                "violet": "#ee82ee",
                "wheat": "#f5deb3", "white": "#ffffff", "whitesmoke": "#f5f5f5",
                "yellow": "#ffff00", "yellowgreen": "#9acd32"
            };
            if (Object.prototype.toString.call(color) == '[object Array]')
                return color;
            if (typeof colours[color.toLowerCase()] != 'undefined')
                return colours[color.toLowerCase()];
            return color;
        };
        SunburstChart.prototype._hexToRgb = function (hex) {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return result ? {
                r: parseInt(result[1], 16),
                g: parseInt(result[2], 16),
                b: parseInt(result[3], 16)
            } : null;
        };
        SunburstChart.prototype.findAngle = function (midAngle) {
            var totalangle;
            if (midAngle > 0) {
                totalangle = midAngle % (2 * Math.PI);
                if (totalangle > (2 * Math.PI)) {
                    this.findAngle(totalangle);
                }
                else
                    return totalangle;
            }
            else {
                return ((2 * Math.PI) + midAngle);
            }
        };
        SunburstChart.prototype.calculateLabelPosition = function (startPoints, bounds) {
            var positionX, positionY, midX, midY, childMidX, childMidY, startX, startY, dMidX, dMidY;
            midX = bounds.midPoint.midX;
            midY = bounds.midPoint.midY;
            childMidX = bounds.childMidPoint.childMidX;
            childMidY = bounds.childMidPoint.childMidY;
            startX = startPoints.startX;
            startY = startPoints.startY;
            dMidX = bounds.sunburstMidPoint.dMidX;
            dMidY = bounds.sunburstMidPoint.dMidY;
            if (this.isNullOrUndefined(childMidX)) {
                positionX = (midX + dMidX) / 2;
                positionY = (midY + dMidY) / 2;
            }
            else {
                positionX = (midX + childMidX) / 2;
                positionY = (midY + childMidY) / 2;
            }
            return { positionX: positionX, positionY: positionY };
        };
        SunburstChart.prototype.drawPath = function (Options) {
            var path = document.getElementById(Options._id);
            if (path === null) {
                path = this.createElement('path');
            }
            this.setAttributes(Options, path);
            return path;
        };
        SunburstChart.prototype.sunburstRender = function (layerData) {
            this.circleCenterX = [];
            this.circleCenterY = [];
            this.circularRadius = [];
            this.sunburstRegions = [];
            this.startX = [];
            this.startY = [];
            this._visiblePoints = layerData;
            var layer;
            this.calculateSliceAngle(layerData[0].point);
            for (layer = 2; layer <= this.totallayerCount; layer++) {
                for (var j = 1; j < layerData.length; j++) {
                    if (layer == layerData[j].layerNumber) {
                        this.calculateChildAngle(layerData[j]);
                    }
                }
            }
            this.circularRadius = this.calculateRadius((this.areaBounds['actualWidth']), (this.areaBounds['actualHeight']), this.totallayerCount);
            if (this._drillInnerRadius.length == 0)
                this.circularRadius = this.calculateRadius((this.areaBounds['actualWidth']), (this.areaBounds['actualHeight']), this.totallayerCount);
            else {
                this.circularRadius = this._drillOuterRadius[this.drilldownCount - 1];
                this.innerRadius = this._drillInnerRadius[this.drilldownCount - 1];
            }
            this.gSeriesGroupEle = this.createGroup({ 'id': this._id + '_svg_SunBurstElementGroup' });
            for (var i = this.totallayerCount; i > 0; i--) {
                for (var j = 0; j < layerData.length; j++) {
                    if (i == layerData[j].layerNumber) {
                        this.drawRegion(layerData[j], j);
                    }
                }
            }
            this.sunburstDoubleClick = this.sunburstDoubleClick.bind(this);
            this.gSeriesGroupEle.addEventListener('dblclick', this.sunburstDoubleClick, true);
            return this.gSeriesGroupEle;
        };
        SunburstChart.prototype.calculateArcData = function (point, layerNumber, radius) {
            var visiblePoints, chartStartAngle, startAngle, endAngle, totalDegree, longArc, midAngle, direction, startX, startY, clockWise, x1, x2, y1, y2, centerx, centery;
            visiblePoints = this.model.points;
            chartStartAngle = -.5 * Math.PI;
            startAngle = point.startAngle + chartStartAngle;
            endAngle = point.endAngle + chartStartAngle - 0.000001;
            totalDegree = this.model.endAngle - this.model.startAngle;
            longArc = endAngle - startAngle < Math.PI ? 0 : 1;
            midAngle = (startAngle + endAngle) / 2;
            if (point.currentMidAngle == undefined) {
                point.currentMidAngle = (startAngle + endAngle) / 2;
            }
            clockWise = totalDegree > 0 ? 1 : 0;
            longArc = clockWise ? endAngle - startAngle < Math.PI ? 0 : 1 : endAngle - startAngle > -1 * Math.PI ? 0 : 1;
            startX = this.circleCenterX;
            startY = this.circleCenterY;
            x1 = startX + radius * Math.cos(startAngle);
            y1 = startY + radius * Math.sin(startAngle);
            x2 = startX + radius * Math.cos(endAngle);
            y2 = startY + radius * Math.sin(endAngle);
            var dStartX = startX + this.innerRadius[layerNumber - 1] * Math.cos(startAngle);
            var dStartY = startY + this.innerRadius[layerNumber - 1] * Math.sin(startAngle);
            var dEndX = startX + this.innerRadius[layerNumber - 1] * Math.cos(endAngle);
            var dEndY = startY + this.innerRadius[layerNumber - 1] * Math.sin(endAngle);
            var dClockWise = clockWise ? 0 : 1;
            if ((startAngle < 0) && Math.round(point.endAngle - point.startAngle) == 6) {
                dEndX = dEndX - 0.01;
                x2 = x2 - 0.01;
            }
            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + this.innerRadius[layerNumber - 1] + " " + this.innerRadius[layerNumber - 1] + " " + "1" + " " + longArc + " " + dClockWise + " " + dStartX + " " + dStartY + " " + "z";
            return { Direction: direction, centerX: centerx, centerY: centery };
        };
        SunburstChart.prototype.findParent = function (visiblePoints) {
            var parentLayer = visiblePoints.layerNumber - 1;
            for (var i = 0; i < this._visiblePoints.length; i++) {
                if (parentLayer == this._visiblePoints[i]['layerNumber']) {
                    for (var j = 0; j < this._visiblePoints[i]['point'].length; j++) {
                        if (this._visiblePoints[i]['point'][j].x == visiblePoints.parentName
                            && this._visiblePoints[i]['point'][j].legendIndex == visiblePoints.legendIndex)
                            return this._visiblePoints[i]['point'][j];
                    }
                }
            }
        };
        SunburstChart.prototype.drawRegion = function (visiblePoints, layerIndex) {
            var result, options, sliceXY, point, id, region, sunburstRegion = [], sunburstData, radius, model = this.model, startAngle, opacity, fill = model["segmentBorder"].color, commonsegmentRendering, selected, selection, regionColor, pathElement;
            startAngle = this.model.startAngle * (Math.PI / 180);
            selection = model['selectionSettings'];
            fill = this.isNullOrUndefined(fill) ? (this.model.theme.toString().search("dark") != -1 ? "black" : "white") : fill;
            for (var j = 0; j < visiblePoints.point.length; j++) {
                selected = false;
                opacity = model['opacity'];
                if (isNaN(visiblePoints.point[j].startAngle))
                    continue;
                point = visiblePoints.point[j];
                regionColor = point['fill'];
                result = this.calculateArcData(point, visiblePoints.layerNumber, this.circularRadius[visiblePoints.layerNumber - 1]);
                sliceXY = result['Direction'].split(" ");
                radius = this.circularRadius[visiblePoints.layerNumber - 1];
                point['layerIndex'] = layerIndex;
                id = this._id + '_legendIndex' + point['legendIndex'] + '_layerNumber' + visiblePoints.layerNumber.toString() + '_layerIndex' + layerIndex.toString() + '_pointIndex' + j.toString();
                for (var i = 0; i < this.selectedgroupID.length; i++) {
                    if (id == this.selectedgroupID[i]) {
                        selected = true;
                    }
                }
                if (selection['type'] == sunburstSelectionType.opacity && !selected && this.selectedgroupID.length > 0)
                    opacity = selection['opacity'];
                point['id'] = id;
                options = {
                    'id': id,
                    'fill': point['fill'],
                    'stroke-width': model['segmentBorder'].width,
                    'stroke-dasharray': '',
                    'd': result['Direction'],
                    'stroke': fill,
                    'stroke-linecap': "butt",
                    'stroke-linejoin': "round",
                    'opacity': opacity,
                    'radius': radius,
                    'start': point['startAngle'] - 1.57,
                    'end': point['endAngle'] - 1.57,
                    'innerR': this.innerRadius[visiblePoints['layerNumber'] - 1],
                    'counterClockWise': false,
                    'x': this.circleCenterX,
                    'y': this.circleCenterY,
                    'visibility': this._enableAnimation ? 'hidden' : 'visible'
                };
                commonsegmentRendering = $.extend({}, this.common);
                commonsegmentRendering['data'] = { 'point': point, pathOptions: options };
                this._trigger("segmentRendering", commonsegmentRendering);
                pathElement = this.drawPath(options);
                if (selected && selection['type'] == sunburstSelectionType.color)
                    this.selectPointData(pathElement);
                this.gSeriesGroupEle.appendChild(pathElement);
                region = {
                    'x': point['x'], 'y': point['y'], 'fill': point['fill'], 'text': point['text'],
                    layerNumber: visiblePoints.layerNumber, layerIndex: layerIndex, parentName: visiblePoints.parentName,
                    pointIndex: j, startAngle: point['startAngle'] - startAngle, endAngle: point['endAngle'] - startAngle, startX: this.startX,
                    startY: this.startY, groupNumber: visiblePoints.groupNumber, id: id,
                    legendIndex: point['legendIndex']
                };
                sunburstRegion.push(region);
            }
            sunburstData = { Radius: this.circularRadius[visiblePoints.layerNumber - 1], centerX: this.circleCenterX, centerY: this.circleCenterY, innerRadius: this.innerRadius[visiblePoints['layerNumber'] - 1] };
            var finalRegion = { sunburstData: sunburstData, region: sunburstRegion };
            this.sunburstRegions.push(finalRegion);
        };
        SunburstChart.prototype.calculateRadius = function (width, height, layercount) {
            var radius = 0, radiusCollection = [], sunburstCoefficient, coefficient = this.model.radius, minimum = Math.min(width, height), availableRadius, holeCoefficient = this.model.innerRadius;
            this.innerRadius = [];
            sunburstCoefficient = coefficient > 1 ? 1 : coefficient < 0 ? 0 : coefficient;
            availableRadius = (minimum / 2) * sunburstCoefficient;
            if (holeCoefficient != 0)
                this.innerRadius.push(holeCoefficient * availableRadius);
            else
                this.innerRadius.push(0);
            for (var i = 0; i < layercount; i++) {
                if (i == 0) {
                    radius = ((availableRadius - this.innerRadius[0]) / layercount) + this.innerRadius[0];
                    radiusCollection[i] = radius;
                }
                else {
                    this.innerRadius.push(radius);
                    radius = (radius - this.innerRadius[i - 1]) + radius;
                    radiusCollection[i] = radius;
                }
            }
            return radiusCollection;
        };
        SunburstChart.prototype.calculateSliceAngle = function (visiblePoints) {
            var visiblePointLength = visiblePoints.length, sumofYValues = 0, totalDegree = 360, point, margin = this.model.margin, actualHeight, actualWidth, endAngle, itemCurrentXPos = 0, seriesIndex = 0, chartStarttingAngle, sunburstFactor, pointIndex, midAngle, legXSpace = 0, legYSpace = 0, borderWidth = this.model.border.width, legend = this.model.legend;
            for (var i = 0; i < visiblePointLength; i++) {
                visiblePoints[i].y = !this.isNullOrUndefined(visiblePoints[i].y) ? visiblePoints[i].y : (!this.isNullOrUndefined(visiblePoints[i].point) ? this.calculateSumofY(visiblePoints[i], sumofYValues) : 0);
                sumofYValues += visiblePoints[i].y;
            }
            if (this.model.startAngle > this.model.endAngle) {
                if (this.model.endAngle == 0)
                    this.model.endAngle = 360;
                else if (this.model.endAngle < 0) {
                    this.model.endAngle = 360 + this.model.endAngle;
                }
            }
            if (this.model.endAngle != null && this.model.startAngle < this.model.endAngle)
                totalDegree = this.model.endAngle - this.model.startAngle;
            else if (this.model.startAngle > this.model.endAngle && !this.isNullOrUndefined(this.model.endAngle)) {
                totalDegree = (360 - this.model.startAngle) + this.model.endAngle;
            }
            else
                totalDegree = 360;
            totalDegree = (totalDegree != 360 && totalDegree != -360) ? totalDegree % 360 : totalDegree;
            if (this.model.startAngle) {
                this.model.startAngle = this.model.startAngle % 360;
                this.model.endAngle = this.model.startAngle + totalDegree;
                itemCurrentXPos = this.model.startAngle ? (sumofYValues / 360) * this.model.startAngle : 0;
                itemCurrentXPos = itemCurrentXPos / sumofYValues;
            }
            else
                this.model.endAngle = totalDegree;
            if (legend['visible']) {
                if (legend['position'] == sunburstLegendPosition.right || legend['position'] == sunburstLegendPosition.left) {
                    legXSpace += this.elementSpacing + this.LegendBounds['Width'];
                    if (!this.isNullOrUndefined(legend['size']['width']))
                        legXSpace = parseFloat(legend['size']['width']);
                }
                else if (legend['position'] == sunburstLegendPosition.bottom || legend['position'] == sunburstLegendPosition.top) {
                    legYSpace += this.elementSpacing + this.LegendBounds['Height'];
                    if (!this.isNullOrUndefined(legend['size']['height']))
                        legYSpace = parseFloat(legend['size']['height']);
                }
            }
            actualHeight = this.height - (legYSpace + margin['top'] + margin['bottom'] + (borderWidth * 2) + this.yOffset);
            actualWidth = this.width - (legXSpace + margin['left'] + margin['right'] + (borderWidth * 2));
            this.circleCenterX = (actualWidth * 0.5) + margin['left'] + ((legend['visible'] && legend['position'] == sunburstLegendPosition.left) ? legXSpace : 0);
            this.circleCenterY = (actualHeight * 0.5) + margin['top'] + this.yOffset + ((legend['visible'] && legend['position'] == sunburstLegendPosition.top) ? legYSpace : 0);
            totalDegree = this.model.endAngle - this.model.startAngle;
            sunburstFactor = totalDegree / 180;
            for (pointIndex = 0; pointIndex < visiblePointLength; pointIndex++) {
                point = visiblePoints[pointIndex];
                if (pointIndex == 0)
                    point.startAngle = 2 * Math.PI * itemCurrentXPos;
                else
                    point.startAngle = endAngle;
                endAngle = point.endAngle = sunburstFactor * Math.PI * (point.y / sumofYValues) + point.startAngle;
                endAngle = point.endAngle = (isNaN(endAngle)) ? point.startAngle : endAngle;
                point.start = point.startAngle;
                point.end = point.endAngle;
                point.midAngle = (point.endAngle + point.startAngle) / 2;
                point.pointIndex = pointIndex;
                chartStarttingAngle = -.5 * Math.PI;
                point.radian = (point.midAngle / 2) % (2 * Math.PI);
                itemCurrentXPos += point.y / sumofYValues;
                midAngle = point.midAngle + chartStarttingAngle;
                this.startX[pointIndex] = this.circleCenterX;
                this.startY[pointIndex] = this.circleCenterY;
            }
        };
        SunburstChart.prototype.calculateTotaldegree = function (startAngle, endAngle) {
            var totalDegree = 0;
            if (startAngle < endAngle) {
                totalDegree = endAngle - startAngle;
            }
            else if (startAngle > endAngle) {
                totalDegree = 360 - startAngle + endAngle;
            }
            return totalDegree;
        };
        SunburstChart.prototype.calculateSumofY = function (visiblePoints, sumofYvalues) {
            var point = visiblePoints.point;
            for (var i = 0; i < point.length; i++) {
                if (!this.isNullOrUndefined(point[i]['y']))
                    sumofYvalues += point[i]['y'];
                else {
                    if (!this.isNullOrUndefined(point[i]['point'])) {
                        point[i]['y'] = this.calculateSumofY(point[i], sumofYvalues);
                        sumofYvalues += point[i]['y'];
                    }
                }
            }
            return sumofYvalues;
        };
        SunburstChart.prototype.calculateChildAngle = function (visiblePoints) {
            var parentNode, endAngle, startAngle, totalAngle, sumofY = 0, parentY, ratio;
            parentNode = this.findParent(visiblePoints);
            for (var j = 0; j < visiblePoints.point.length; j++) {
                sumofY = sumofY + visiblePoints.point[j].y;
            }
            parentY = parentNode.y;
            startAngle = endAngle = parentNode.startAngle;
            totalAngle = parentNode.endAngle - parentNode.startAngle;
            for (var i = 0; i < visiblePoints.point.length; i++) {
                if (i == 0) {
                    visiblePoints.point[i].startAngle = startAngle;
                    visiblePoints.point[i].start = startAngle;
                    endAngle = ((visiblePoints.point[i].y / sumofY) * totalAngle) + startAngle;
                    visiblePoints.point[i].end = visiblePoints.point[i].endAngle = endAngle;
                    visiblePoints.point[i].midAngle = (visiblePoints.point[i].startAngle + visiblePoints.point[i].endAngle) / 2;
                }
                else {
                    visiblePoints.point[i].startAngle = endAngle;
                    visiblePoints.point[i].start = endAngle;
                    endAngle = ((visiblePoints.point[i].y / sumofY) * totalAngle) + endAngle;
                    visiblePoints.point[i].end = visiblePoints.point[i].endAngle = endAngle;
                    visiblePoints.point[i].midAngle = (visiblePoints.point[i].startAngle + visiblePoints.point[i].endAngle) / 2;
                }
            }
        };
        SunburstChart.prototype.calculateSize = function (size, title) {
            var actualWidth, actualHeight, titleText, yOffset, realwidth, borderWidth = this.model.border.width, spaceValue, x, rightSpace, width, y, margin = this.model.margin, elementSpacing = this.elementSpacing;
            spaceValue = this._getLegendSpace(this.model.legend);
            x = spaceValue['leftLegendWidth'] + borderWidth + margin['left'];
            rightSpace = spaceValue['rightLegendWidth'] + margin['right'] + margin['left'] + (2 * borderWidth);
            width = size['width'];
            actualWidth = width - (x + rightSpace);
            y = margin['top'] + elementSpacing + spaceValue['modelTitleHeight'] + (spaceValue['modelsubTitleHeight']) + spaceValue['topLegendHeight'] + borderWidth;
            this.yOffset = (title['text'] && title['visible']) ? (this.titleLocation['size'].height + elementSpacing) + (title['subtitle'].text == '' ? 0 : (title['subtitle'].text && title['subtitle'].visible) ? (this.subTitleLocatation['size'].height) : 0) : 0;
            actualHeight = size['height'] - (spaceValue['modelTitleHeight'] + elementSpacing + margin['top'] + spaceValue['modelsubTitleHeight'] + spaceValue['topLegendHeight'] + spaceValue['bottomLegendHeight']);
            return { x: x, y: y, actualWidth: actualWidth, actualHeight: actualHeight };
        };
        SunburstChart.prototype._getLegendSpace = function (legend) {
            var bounds = this.LegendActualBounds, itemPadding = 10, elementSpacing = this.elementSpacing, legendBorder = this.model.legend.border.width, leftLegendWidth = 0, rightLegendWidth = 0, topLegendHeight = 0, bottomLegendHeight = 0, space, position = this.model.legend.position, ltheight = 0, titleHeight = 0, subtitleHeight = 0, modelTitleHeight, modelsubTitleHeight, maxHeight, maxWidth, title = this.model.title;
            elementSpacing = this.legendClicked ? 0 : elementSpacing;
            if (legend['visible']) {
                space = (parseFloat(bounds['Width']) + (itemPadding / 2) + elementSpacing + (2 * legendBorder));
                maxWidth = this.width * (20 / 100);
                leftLegendWidth = position == sunburstLegendPosition.left ? space : 0;
                if (leftLegendWidth > maxWidth && this.isNullOrUndefined(legend['size']['width'])) {
                    leftLegendWidth = maxWidth;
                }
                rightLegendWidth = position == sunburstLegendPosition.right ? space : 0;
                if (rightLegendWidth > maxWidth && this.isNullOrUndefined(legend['size']['width'])) {
                    rightLegendWidth = maxWidth;
                }
                maxHeight = this.height * (20 / 100);
                ltheight = legend['title'].visible ? this.measureText(legend['title'].text, legend['title'].font)['height'] : 0;
                topLegendHeight = position == sunburstLegendPosition.top ? elementSpacing + parseFloat(bounds['Height']) + ltheight : 0;
                bottomLegendHeight = position == sunburstLegendPosition.bottom ? elementSpacing + parseFloat(bounds['Height']) + ltheight : 0;
                if (topLegendHeight > maxHeight && this.isNullOrUndefined(legend['size']['height'])) {
                    topLegendHeight = maxHeight;
                }
                else if (bottomLegendHeight > maxHeight && this.isNullOrUndefined(legend['size']['height'])) {
                    bottomLegendHeight = maxHeight;
                }
                this.LegendActualBounds['Height'] = (topLegendHeight == 0 && bottomLegendHeight != 0) ? bottomLegendHeight : (topLegendHeight != 0) ? topLegendHeight : this.LegendActualBounds['Height'];
                this.LegendActualBounds['Width'] = (rightLegendWidth == 0 && leftLegendWidth != 0) ? leftLegendWidth : (rightLegendWidth != 0) ? rightLegendWidth : this.LegendActualBounds['Width'];
            }
            titleHeight = this.measureText(title['text'], title['font'])['height'];
            subtitleHeight = this.measureText(title['subtitle'].text, title['subtitle'].font)['height'];
            modelTitleHeight = (title['text'] == "" || !title['visible']) ? 0 : (titleHeight + elementSpacing);
            modelsubTitleHeight = (title['subtitle'].text == "" || !title['subtitle'].visible) ? 0 : (subtitleHeight);
            return {
                leftLegendWidth: leftLegendWidth, rightLegendWidth: rightLegendWidth, topLegendHeight: topLegendHeight, bottomLegendHeight: bottomLegendHeight, modelTitleHeight: modelTitleHeight, modelsubTitleHeight: modelsubTitleHeight
            };
        };
        SunburstChart.prototype.calculateLegendBounds = function () {
            this.legendCollection = [];
            this.legendRegion = [];
            this.legendTextRegion = [];
            this._legendMaxHeight = 0;
            this._legendMaxWidth = 0;
            var legend = this.model.legend, padding = 10, legendSizeHeight = legend['size']['height'], legendSizeWidth = legend['size']['width'], itemPadding = legend['itemPadding'] > 0 ? legend['itemPadding'] : 0, position = legend['position'].toLowerCase(), width = 0, height = 0, svgHeight = this.height, svgWidth = this.width, legendItemWidth = 0, legendItemHeight = 0, legendWidth = 0, legendHeight = 0, legnedHeightIncr = 1, legendviewerHeight = 0, legendviewerWidth = 0, tempSeries = {}, legendSeries = [], rowCount = this.model.legend.rowCount, columnCount = this.model.legend.columnCount, elementSpacing = this.elementSpacing, titleSize = this.measureText(legend['title']['text'], legend['title']['font']), legendBorder = legend['border']['width'], borderSize = this.model.border.width, svgObjectHeight = svgHeight - ((elementSpacing * 4) + (borderSize * 2) + (legendBorder * 2)), svgObjectWidth = svgWidth - ((elementSpacing * 4) + (borderSize * 2) + (legendBorder * 2)), visiblePoints = [], length, legendColor, legendName, legendSeriesLength, currentLegend, shapeWidth, legendsize, colorGradName, point, bounds, legendHeightTemp, legendBounds;
            if (legend['visible']) {
                visiblePoints = this.layerData[0]['point'];
                length = visiblePoints.length;
                for (var i = 0; i < length; i++) {
                    point = visiblePoints[i];
                    legendColor = visiblePoints[i]['fill'];
                    legendName = visiblePoints[i]['x'];
                    tempSeries = {
                        'sunburstLegendShape': legend['shape'],
                        'Text': legendName,
                        'displayText': legendName,
                        'font': legend['font'],
                        'groupNumber': visiblePoints[i]['groupNumber'],
                        'layerIndex': visiblePoints[i]['layerIndex'],
                        'layerNumber': visiblePoints[i]['layerNumber'],
                        'Shape': 'None',
                        'visibility': true,
                        'fill': visiblePoints[i]['fill'],
                        'points': visiblePoints[i]['point'],
                        'x': visiblePoints[i]['x'],
                        'y': visiblePoints[i]['y']
                    };
                    legendSeries.push(tempSeries);
                }
                legendSeriesLength = legendSeries.length;
                for (var j = 0; j < legendSeriesLength; j++) {
                    currentLegend = legendSeries[j];
                    shapeWidth = legend['itemStyle']['width'];
                    legendsize = this._getLegendSize(currentLegend);
                    legendItemWidth = Math.max(this._legendMaxWidth > 0 ? (this._legendMaxWidth + itemPadding + shapeWidth) : legendsize['Width'], legendItemWidth);
                    legendItemHeight = Math.max(legendsize['Height'], legendItemHeight);
                }
                legendHeight = legendItemHeight + elementSpacing * 2;
                legendWidth = legendItemWidth;
                if (rowCount || columnCount) {
                    legendBounds = this.legendItemBounds(legendSeriesLength, legendItemWidth + itemPadding, legendItemHeight + itemPadding);
                    legendWidth = legendBounds['LegendWidth'];
                    legendHeight = legendBounds['LegendHeight'];
                    if (position === 'top' || position === 'bottom' || position === 'custom')
                        legendHeight = legendHeight - itemPadding + elementSpacing;
                    else
                        legendWidth = legendWidth - itemPadding;
                }
                for (var k_4 = 0; k_4 < legendSeriesLength; k_4++) {
                    currentLegend = legendSeries[k_4];
                    legendsize = this._getLegendSize(currentLegend);
                    bounds = (rowCount || columnCount) ? { 'Width': legendItemWidth, 'Height': legendItemHeight } : legendsize;
                    bounds['_Width'] = legendsize['Width'];
                    legendSeries[k_4]['Bounds'] = bounds;
                    this.legendCollection.push(currentLegend);
                }
                for (var k = 0; k < legendSeriesLength; k++) {
                    currentLegend = legendSeries[k];
                    legendsize = this._getLegendSize(currentLegend);
                    if (!(rowCount) && !(columnCount)) {
                        if ((position == 'top' || position == 'bottom' || position == 'custom')) {
                            width += legendsize['Width'] + itemPadding;
                            if (width > svgObjectWidth && k != 0) {
                                width -= legendsize['Width'] + itemPadding;
                                legendWidth = Math.max(legendWidth, width);
                                width = legendsize['Width'] + itemPadding;
                                legnedHeightIncr++;
                                legendHeight += legendItemHeight + itemPadding;
                            }
                            else
                                legendWidth = Math.max(legendWidth, width);
                            height = Math.max(height, legendItemHeight);
                        }
                        else {
                            height += legendsize['Height'] + itemPadding;
                            if (height > svgObjectHeight) {
                                height -= legendsize['Height'] + itemPadding;
                                legendHeight = Math.max(legendHeight, height);
                                height = legendsize['Height'] + itemPadding;
                                legendWidth += legendItemWidth + itemPadding;
                            }
                            else
                                legendHeight = Math.max(legendHeight, height);
                            width = Math.max(width, legendItemWidth);
                        }
                    }
                    bounds = (rowCount || columnCount) ? { Width: legendItemWidth, Height: legendItemHeight } : legendsize;
                    bounds['_Width'] = legendsize['Width'];
                    legendSeries[k]['Bounds'] = bounds;
                }
                if (position === 'top' || position === 'bottom' || position === 'custom') {
                    legendWidth = titleSize['width'] > legendWidth - itemPadding ? (titleSize['width'] + padding * 2 + itemPadding) : legendWidth + padding * 2;
                    width += padding;
                    height += padding * 2;
                    this.LegendBounds = { 'Width': Math.max(legendWidth, width) - itemPadding, 'Height': Math.max(legendHeight, height), 'Rows': legnedHeightIncr };
                }
                else {
                    legendWidth = titleSize['width'] > legendWidth - itemPadding ? (titleSize['width'] + padding * 2 + itemPadding) : legendWidth + padding * 2;
                    width += padding;
                    height += padding * 2;
                    if (legend['rowCount'] == null && legend['columnCount'] == null)
                        this.LegendBounds = { 'Width': width, 'Height': height, 'Columns': legnedHeightIncr };
                    else
                        this.LegendBounds = { 'Width': Math.max(legendWidth, width), 'Height': Math.max(legendHeight, height) + padding - itemPadding, 'Columns': legnedHeightIncr };
                }
            }
            else {
                this.LegendBounds = { Width: 0, Height: 0 };
            }
            this.LegendActualBounds = this.LegendBounds;
            if (legendSizeWidth != null)
                this.LegendActualBounds['Width'] = legendSizeWidth;
            if (legendSizeHeight != null)
                this.LegendActualBounds['Height'] = legendSizeHeight;
        };
        SunburstChart.prototype._getLegendSize = function (series) {
            var legend = this.model.legend, symbolSize = legend['itemStyle'], textsize = this.measureText(series['Text'], series['font']), padding = 10, width = symbolSize['width'] + padding + textsize['width'], height = Math.max(symbolSize['height'], textsize['height']);
            return { 'Width': width, 'Height': height };
        };
        SunburstChart.prototype.legendItemBounds = function (itemCount, legendItemWidth, legendItemHeight) {
            var legend = this.model.legend, legVal, position = legend['position'], itemPadding = legend['itemPadding'], legnedHeightIncr, column, legendWidth = 0, legendHeight = 0;
            if (this.isNullOrUndefined(legend['columnCount']) && legend['rowCount']) {
                legnedHeightIncr = legend['rowCount'];
                column = Math.ceil(itemCount / legnedHeightIncr);
                legendWidth = legendItemWidth * column;
                legendHeight = legendItemHeight * legnedHeightIncr;
            }
            else if (this.isNullOrUndefined(legend['rowCount']) && legend['columnCount']) {
                legnedHeightIncr = Math.ceil(itemCount / legend['columnCount']);
                legVal = legend['columnCount'];
                legendWidth = legendItemWidth * legVal;
                legendHeight = legendItemHeight * legnedHeightIncr;
            }
            else if ((legend['rowCount']) && legend['columnCount']) {
                if (legend['columnCount'] < legend['rowCount']) {
                    legnedHeightIncr = legend['rowCount'];
                    column = Math.ceil(itemCount / legnedHeightIncr);
                    legendWidth = legendItemWidth * column;
                    legendHeight = legendItemHeight * legnedHeightIncr;
                }
                else if (legend['columnCount'] > legend['rowCount']) {
                    if (position === 'top' || position === 'bottom' || position === 'custom') {
                        legnedHeightIncr = Math.ceil(itemCount / legend['columnCount']);
                        legVal = legend['columnCount'];
                        legendWidth = legendItemWidth * legVal;
                        legendHeight = legendItemHeight * legnedHeightIncr;
                    }
                    else {
                        legnedHeightIncr = Math.ceil(itemCount / legend['columnCount']);
                        column = Math.ceil(itemCount / legnedHeightIncr);
                        legendWidth = legendItemWidth * column;
                        legendHeight = legendItemHeight * legnedHeightIncr;
                    }
                }
                else {
                    if (position === 'top' || position === 'bottom') {
                        legnedHeightIncr = Math.ceil(itemCount / legend['columnCount']);
                        legVal = Math.ceil(itemCount / legend['rowCount']);
                        legendWidth = legendItemWidth * legend['columnCount'];
                        legendHeight = legendItemHeight * legVal;
                    }
                    else {
                        legnedHeightIncr = legend['rowCount'];
                        column = Math.ceil(itemCount / legnedHeightIncr);
                        legendWidth = legendItemWidth * column;
                        legendHeight = legendItemHeight * legnedHeightIncr;
                    }
                }
            }
            legendHeight += this.elementSpacing;
            return { LegendWidth: legendWidth, LegendHeight: legendHeight };
        };
        SunburstChart.prototype.drawTitle = function (titleOptions, size) {
            var textSize, x, y, options, titleEle, commonTitleEventArgs, elementSpacing = this.elementSpacing, fill = titleOptions['font'].color;
            fill = this.isNullOrUndefined(fill) ? (this.model.theme.toString().indexOf("dark") != -1 ? "white" : "black") : fill;
            textSize = this.measureText(titleOptions['text'], titleOptions['font']);
            x = titleOptions['textAlignment'] == sunburstAlignment.center ? (size['width'] / 2 - textSize['width'] / 2) : (titleOptions['textAlignment'] == sunburstAlignment.near ? (2 * elementSpacing) : (size['width'] - textSize['width'] - (2 * elementSpacing)));
            y = textSize['height'] / 2 + elementSpacing;
            commonTitleEventArgs = $.extend({}, this.common);
            commonTitleEventArgs.data = { TitleText: titleOptions['text'], location: { X: x, Y: y }, size: textSize };
            this._trigger("titleRendering", commonTitleEventArgs);
            options = {
                'id': this._id + "SunburstTitle",
                'x': commonTitleEventArgs.data.location.X,
                'y': commonTitleEventArgs.data.location.Y,
                'fill': fill,
                'font-size': titleOptions['font'].size,
                'font-family': titleOptions['font'].fontFamily,
                'font-style': titleOptions['font'].fontStyle,
                'font-weight': titleOptions['font'].fontWeight,
                'opacity': titleOptions['font'].opacity,
                'text-anchor': 'start'
            };
            this.gTitleGroupElement = this.createGroup({ id: this._id + "SunburstTitle" });
            titleEle = this.createText(options, commonTitleEventArgs.data.TitleText);
            this.gTitleGroupElement.appendChild(titleEle);
            this.titleLocation = { size: textSize, x: commonTitleEventArgs.data.location.X, y: commonTitleEventArgs.data.location.Y };
            if (titleOptions['subtitle'].text != "" && titleOptions['subtitle'].visible) {
                this.drawSubtitle(titleOptions, size, textSize, this.titleLocation);
            }
            return this.gTitleGroupElement;
        };
        SunburstChart.prototype.drawSubtitle = function (titleOptions, size, titleSize, titleLocation) {
            var subtitleSize, x, y, options, subtitleEle, textAnchor, fill = titleOptions['subtitle'].font.color;
            fill = this.isNullOrUndefined(fill) ? (this.model.theme.toString().indexOf("dark") != -1 ? "white" : "black") : fill;
            subtitleSize = this.measureText(titleOptions['subtitle'].text, titleOptions['subtitle'].font);
            x = titleOptions['subtitle'].textAlignment == sunburstAlignment.far ? titleLocation['x'] + titleLocation['size'].width : (titleOptions['subtitle'].textAlignment == sunburstAlignment.near) ? titleLocation['x'] : (titleLocation['x'] + (titleSize['width'] / 2));
            y = titleLocation['size'].height + (2 * this.elementSpacing);
            textAnchor = titleOptions['subtitle'].textAlignment == sunburstAlignment.far ? "end" : (titleOptions['subtitle'].textAlignment == sunburstAlignment.near) ? "start" : "middle";
            options = {
                'id': this._id + "SunburstSunbtitle",
                'x': x,
                'y': y,
                'fill': fill,
                'font-size': titleOptions['subtitle'].font.size,
                'font-family': titleOptions['subtitle'].font.fontFamily,
                'font-style': titleOptions['subtitle'].font.fontStyle,
                'font-weight': titleOptions['subtitle'].font.fontWeight,
                'opacity': titleOptions['subtitle'].font.opacity,
                'text-anchor': textAnchor,
            };
            this.subTitleLocatation = { size: subtitleSize, x: x, y: y };
            subtitleEle = this.createText(options, titleOptions['subtitle'].text);
            this.gTitleGroupElement.appendChild(subtitleEle);
        };
        SunburstChart.prototype.measureText = function (text1, fontOptions) {
            var element = document.querySelectorAll("#measureTex"), textObj;
            if (element.length == 0) {
                textObj = document.createElement('text');
                textObj.setAttribute('id', 'measureTex');
                document.body.appendChild(textObj);
            }
            else {
                textObj = element[0];
                textObj.style.display = 'block';
            }
            textObj.innerHTML = text1;
            textObj.style.fontSize = fontOptions['size'];
            textObj.style.fontFamily = fontOptions['fontFamily'];
            textObj.style.fontWeight = fontOptions['fontWeight'];
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.visibility = 'hidden';
            textObj.style.whiteSpace = 'nowrap';
            return { width: textObj.offsetWidth, height: textObj.offsetHeight };
        };
        SunburstChart.prototype.drawContainerRect = function () {
            var rect, rectOptions;
            rectOptions = {
                'id': this._id + "_svg" + '_svgRect',
                'x': 0,
                'y': 0,
                'width': this.width,
                'height': this.height,
                'fill': this.model.background || "transparent",
                'stroke': this.model.border.color,
                'stroke-width': this.model.border.width,
                'opacity': '0.5'
            };
            rect = this.drawRect(rectOptions);
            return rect;
        };
        SunburstChart.prototype.processData = function (visiblePoints) {
            var visiblePointsLength = visiblePoints.length, point;
            this.findBaseData(visiblePoints);
            for (var i = 0; i < visiblePoints.length; i++) {
                point = visiblePoints[i];
                point['legendIndex'] = i;
                if (visiblePoints[i].point) {
                    this.findLayerData(visiblePoints[i].point, visiblePoints[i].x, 1, visiblePoints[i].groupNumber, i);
                }
            }
        };
        SunburstChart.prototype.findBaseData = function (visiblePoins) {
            var layerpoints = [];
            this.layerData = [];
            for (var i = 0; i < visiblePoins.length; i++) {
                layerpoints.push(visiblePoins[i]);
                layerpoints[i]['groupNumber'] = i + 1;
                layerpoints[i]['layerIndex'] = 0;
                layerpoints[i]['layerNumber'] = 1;
                layerpoints[i]['legendIndex'] = i;
                visiblePoins[i].groupNumber = i + 1;
            }
            this.layerData.push({ point: layerpoints, layerNumber: 1, legendIndex: 0 });
            this.totallayerCount = 1;
        };
        SunburstChart.prototype.findLayerData = function (point, parentName, layerNumber, groupNumber, legendIndex) {
            var points = [];
            for (var j = 0; j < point.length; j++) {
                if (point[j].point) {
                    point[j].legendIndex = legendIndex;
                    this.findLayerData(point[j].point, point[j].x, layerNumber + 1, groupNumber, legendIndex);
                    point[j].layerIndex = this.layerData.length;
                    points.push(point[j]);
                }
                else {
                    point[j].legendIndex = legendIndex;
                    point[j].layerIndex = this.layerData.length;
                    points.push(point[j]);
                }
            }
            point.layerData = this.layerData.length;
            this.layerData.push({ point: points, parentName: parentName, layerNumber: layerNumber + 1, groupNumber: groupNumber, layerIndex: this.layerData.length, legendIndex: legendIndex });
            if ((layerNumber + 1) > this.totallayerCount) {
                this.totallayerCount = layerNumber + 1;
            }
        };
        SunburstChart.prototype.setColors = function (points) {
            var defaultColors = ["#3082bd", "#e55725", "#9cbb59", "#F6851F", "#8165a3", "#4fa045", "#d456a0", "#0dbdef", "#e2a325", "#03a09c"], index, palette = this.model.palette, fill;
            for (var i = 0; i < points.length; i++) {
                if (this.isNullOrUndefined(points[i]['fill']) || this._sunburstRedraw) {
                    if (!this.isNullOrUndefined(palette) && palette.length == 0)
                        points[i]['fill'] = "black";
                    else {
                        points[i]['fill'] = (this.isNullOrUndefined(palette)) ? defaultColors[i % defaultColors.length] : palette[i % palette.length];
                        if (points[i]['fill'].trim().length == 0)
                            points[i]['fill'] = "black";
                    }
                    if (!this.isNullOrUndefined(points[i]['point'])) {
                        this.setchildColors(points[i]['point'], points[i]['fill']);
                    }
                }
                else {
                    if (!this.isNullOrUndefined(points[i]['point'])) {
                        if (points[i]['fill'].trim().length == 0)
                            points[i]['fill'] = "black";
                        this.setchildColors(points[i]['point'], points[i]['fill']);
                    }
                }
            }
        };
        SunburstChart.prototype.setchildColors = function (point, fill) {
            fill = fill.trim().length == 0 ? "black" : fill;
            for (var i = 0; i < point.length; i++) {
                if (this.isNullOrUndefined(point[i]['fill']) || this._sunburstRedraw) {
                    point[i]['fill'] = fill;
                    if (point[i]['fill'].trim().length == 0)
                        point[i]['fill'] = "black";
                    if (!this.isNullOrUndefined(point[i]['point'])) {
                        this.setchildColors(point[i]['point'], fill);
                    }
                }
                else {
                    if (!this.isNullOrUndefined(point[i]['point'])) {
                        this.setchildColors(point[i]['point'], fill);
                    }
                }
            }
        };
        SunburstChart.prototype.processDataSource = function (dataSource) {
            var levelscount = this.model.levels.length, checkPoints = [], i = 0, data, memberPath, xValue, check;
            this.model.points = [];
            this.parentNames = [];
            this.levelNames = [];
            this.childProcess = undefined;
            for (var j = 0; j < dataSource.length; j++) {
                data = dataSource[j];
                memberPath = this.model.levels[i]['groupMemberPath'];
                xValue = data[memberPath];
                check = $.inArray(xValue, checkPoints);
                if (check < 0) {
                    this.model.points.push({ x: xValue, y: dataSource[j][this.model.valueMemberPath], layer: i + 1, parentName: xValue, parentChildName: xValue });
                    checkPoints.push(xValue);
                }
                else {
                    for (var k = 0; k < this.model.points.length; k++)
                        if (this.model.points[k].x == xValue)
                            this.model.points[k].y += dataSource[j][this.model.valueMemberPath];
                }
            }
            if (this.model.levels.length > 1)
                this.createOtherLayerPoints(dataSource, 1);
        };
        SunburstChart.prototype.createOtherLayerPoints = function (dataSource, startLevel) {
            var data = dataSource, currentMemberPath = this.model.levels[startLevel]["groupMemberPath"], yValue, levels = this.model.levels, points = this.model.points;
            for (var j = 0; j < data.length; j++) {
                var parent_level_Name = "", parentName = "", count = 0, parent_1 = void 0, member = void 0, check = void 0, member1 = void 0;
                for (var start = 0; start <= startLevel; start++) {
                    member = levels[start]["groupMemberPath"];
                    parentName = data[j][member];
                    if (!this.isNullOrUndefined(parentName))
                        parent_level_Name += parentName + "_";
                    if ((count) == startLevel)
                        parent_level_Name = parent_level_Name.substring(0, parent_level_Name.length - 1);
                    count++;
                }
                check = $.inArray(parent_level_Name, this.levelNames);
                member1 = levels[startLevel - 1]["groupMemberPath"];
                parent_1 = data[j][member1];
                if (check < 0 && !this.isNullOrUndefined((data[j][currentMemberPath]))) {
                    this.levelNames.push(parent_level_Name);
                    yValue = data[j][this.model.valueMemberPath];
                    points.push({ x: data[j][currentMemberPath], 'y': yValue, layer: startLevel + 1, parentChildName: parent_level_Name, parentName: parent_1 });
                }
                else if (!this.isNullOrUndefined((data[j][currentMemberPath]))) {
                    for (var i = 0; i < points.length; i++) {
                        if (points[i]['parentName'] == parent_1 && parent_level_Name == points[i]['parentChildName'])
                            points[i]['y'] += yValue;
                    }
                }
            }
            if (startLevel < (levels.length - 1))
                this.createOtherLayerPoints(data, startLevel + 1);
            else {
                this._points = points;
                this.model.points = [];
                this.alignDataSourcePoints(data, 1);
            }
        };
        SunburstChart.prototype.alignDataSourcePoints = function (dataSource, startLevel) {
            var levels = this.model.levels;
            for (var i = startLevel; i < levels.length; i++) {
                for (var j = 0; j < this._points.length; j++) {
                    if (i == this._points[j]["layer"]) {
                        this.parentNames.push(this._points[j]["parentName"] + "_layer_" + this._points[j]["layer"]);
                    }
                    else {
                        i = levels.length;
                        break;
                    }
                }
            }
            this.findParentElements(this.model.points);
        };
        SunburstChart.prototype.findParentElements = function (points) {
            for (var i = 0; i < this.parentNames.length; i++) {
                var name = this.parentNames[i];
                for (var j = 0; j < this._points.length; j++) {
                    if (name == this._points[j]["x"] + "_layer_" + this._points[j]["layer"]) {
                        points.push(this._points[j]);
                    }
                }
            }
            if (this.isNullOrUndefined(this.childProcess) || this.childProcess)
                this.findChildElements(points, 0);
        };
        SunburstChart.prototype.findChildElements = function (points, pointCount) {
            var point, currentPoints;
            this.visiblePoints = [];
            if (pointCount <= points.length - 1) {
                this.childProcess = true;
                this.visiblePoints.push(points[pointCount]);
                this.findOtherChildElements(this.visiblePoints, this.parentNames[pointCount], 2, 0, point, currentPoints, pointCount);
            }
            else
                this.childProcess = false;
        };
        SunburstChart.prototype.findOtherChildElements = function (points, parentName, startLayer, count, previousPoints, currentPoints, pointCount) {
            var parenPoints = points;
            var childPoint;
            var parentActualName = parentName.substring(0, parentName.indexOf("_layer_"));
            for (var i = 0; i < parenPoints.length; i++) {
                parenPoints[i]["point"] = [];
                for (var j = 0; j < this._points.length; j++) {
                    if (startLayer == this._points[j]["layer"] && parenPoints[i]["x"] == this._points[j]["parentName"] && this._points[j]["parentChildName"].indexOf(parentActualName) >= 0) {
                        parenPoints[i]["point"].push(this._points[j]);
                    }
                }
                if (this.isNullOrUndefined(previousPoints)) {
                    previousPoints = parenPoints;
                    currentPoints = [];
                    currentPoints.push({ layer: startLayer });
                    currentPoints[0]["point"] = [];
                }
                else if (count >= previousPoints[i]["point"].length) {
                    if (count >= previousPoints[i]["point"].length && startLayer >= this.model.levels.length) {
                        this.findChildElements(this.model.points, (pointCount + 1));
                    }
                    count = 0;
                    for (var m = 0; m < parenPoints[i]["point"].length; m++) {
                        currentPoints[0].point.push(parenPoints[i]["point"][m]);
                    }
                    previousPoints = currentPoints;
                }
                else {
                    for (var m = 0; m < parenPoints[i]["point"].length; m++) {
                        currentPoints[0]["point"].push(parenPoints[i]["point"][m]);
                    }
                }
                for (var p = count; p < previousPoints[i]["point"].length; p++) {
                    if (count < previousPoints[i]["point"].length) {
                        startLayer = (count == 0) ? (startLayer + 1) : startLayer;
                        count++;
                        var totalPoint = previousPoints[i]["point"][p];
                        childPoint = [];
                        childPoint.push(totalPoint);
                    }
                    if (this.childProcess)
                        this.findOtherChildElements(childPoint, parentName, startLayer, count, previousPoints, currentPoints, pointCount);
                }
            }
        };
        SunburstChart.prototype.setSvgsize = function (size, id) {
            var containerWidth, containerHeight, height, width, chartWidth, chartHeight;
            containerWidth = document.getElementById(id).clientWidth;
            containerHeight = document.getElementById(id).clientHeight;
            height = 450;
            width = (ej.isTouchDevice()) ? 250 : 600;
            if (size['width'] != "" && (!this.isResize || !this.model.isResponsive)) {
                chartWidth = size['width'];
                if (typeof (chartWidth) == "string" && chartWidth.indexOf("%") != -1)
                    width = (containerWidth / 100) * parseInt(chartWidth);
                else
                    width = parseFloat(size['width']);
            }
            else if (containerWidth > 0)
                width = containerWidth;
            this.width = width;
            this.container.setAttribute('width', width.toString());
            if (size['height'] != "") {
                chartHeight = size['height'];
                if (typeof (chartHeight) == "string" && chartHeight.indexOf("%") != -1)
                    height = (containerHeight / 100) * parseInt(chartHeight);
                else
                    height = parseFloat(size['height']);
            }
            else if (containerHeight > 0)
                height = containerHeight;
            this.height = height;
            this.container.setAttribute('height', height.toString());
        };
        SunburstChart.prototype.isNullOrUndefined = function (value) {
            return value === undefined || value === null;
        };
        SunburstChart.prototype.createSvg = function (option) {
            var svg = this.createElement("svg");
            this.setAttributes(option, svg);
            return svg;
        };
        SunburstChart.prototype.drawPolygon = function (Options) {
            var polygon = document.getElementById(Options['id']);
            if (this.isNullOrUndefined(polygon))
                polygon = this.createElement('polygon');
            this.setAttributes(Options, polygon);
            return polygon;
        };
        SunburstChart.prototype.createText = function (options, label) {
            var text = this.createElement("text");
            this.setAttributes(options, text);
            if (label)
                text.textContent = label;
            return text;
        };
        SunburstChart.prototype.createGroup = function (options) {
            var group = this.createElement("g");
            this.setAttributes(options, group);
            return group;
        };
        SunburstChart.prototype.drawRect = function (Options) {
            var rectangle = document.getElementById(Options._id);
            if (rectangle === null) {
                rectangle = this.createElement('rect');
            }
            this.setAttributes(Options, rectangle);
            return rectangle;
        };
        SunburstChart.prototype.drawCircle = function (Options) {
            var circle = document.getElementById(Options['id']);
            if (this.isNullOrUndefined(circle))
                circle = this.createElement('circle');
            this.setAttributes(Options, circle);
            return circle;
        };
        SunburstChart.prototype.createElement = function (elementName) {
            return document.createElementNS("http://www.w3.org/2000/svg", elementName);
        };
        SunburstChart.prototype.getXcordinate = function (x, radius, angle) {
            return (x + radius * (Math.cos(angle)));
        };
        SunburstChart.prototype.getYcordinate = function (y, radius, angle) {
            return (y + radius * (Math.sin(angle)));
        };
        SunburstChart.prototype.setAttributes = function (Options, element) {
            var properties, values;
            properties = Object.keys(Options);
            values = properties.map(function (property) { return Options[property]; });
            for (var i = 0, len = properties.length; i < len; i++) {
                element.setAttribute(properties[i], values[i]);
            }
        };
        return SunburstChart;
    }(ej.WidgetBase));
    ejSunburstChart.SunburstChart = SunburstChart;
    ej.widget("ejSunburstChart", "ej.SunburstChart", new SunburstChart());
    ejSunburstChart.compareExtend = function (temp, src, def) {
        if (typeof def === 'object' && def !== null) {
            var defProp = Object.keys(def), len = defProp.length, currPro;
            for (var i = 0; i < len; i++) {
                currPro = defProp[i];
                if (src.hasOwnProperty(currPro) && src[currPro] != null) {
                    if (Array.isArray(src[currPro]) || typeof src[currPro] === 'object' && src[currPro] !== null) {
                        ejSunburstChart.compareExtend({}, src[currPro], def[currPro]);
                    }
                }
                else {
                    src[currPro] = def[currPro];
                }
            }
        }
        return src;
    };
    ejSunburstChart.deepExtend = function (out) {
        out = out || {};
        for (var i = 1; i < arguments.length; i++) {
            var obj = arguments[i];
            if (!obj)
                continue;
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (typeof obj[key] === 'object')
                        out[key] = ejSunburstChart.deepExtend(out[key], obj[key]);
                    else
                        out[key] = obj[key];
                }
            }
        }
        return out;
    };
})(ejSunburstChart || (ejSunburstChart = {}));
;
/**
* @fileOverview Plugin to style the Html TreeMap elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/
(function ($, ej, undefined) {
   
    ej.widget({ "ejTreeMap": "ej.datavisualization.TreeMap" }, {

        validTags: ["div"],

        defaults: {           
			locale : null,    
			enableGroupSeparator: false,
            leafItemSettings: {               
                borderThickness: 1,               
                borderBrush: "white",
                showLabels: false,
                labelPath: null,
                gap:0,
                itemTemplate: null,
                labelPosition: "topleft",
                labelVisibilityMode: "visible"
            },
            header: null,
            isHierarchicalDatasource: false,
            dataSource: null,
            groupColorMapping:[],			
            enableDrillDown: false,
            drillDownHeaderColor: null,
            drillDownSelectionColor: '#000000',          			
            colorValuePath: null,
            weightValuePath: null,			
            treeMapItems: [],
            showLegend: false,
            borderBrush: "white",
            borderThickness: 1,
            enableResize: true,
			enableGradient: false,
            isResponsive: true,
            itemsLayoutMode: "squarified",						
            levels: [],
                       
           legendSettings: {
                template: "",
                iconHeight: 15,
                iconWidth: 15,
                title:'',
                height: 0,
                width: 0,
                mode: "default",
                leftLabel: "",
                rightLabel: "",
                dockPosition: "top",
                alignment: "near",
                columnCount: 0
            },         			         
            rangeColorMapping: [],

            uniColorMapping: { 			
                color: null 
            },
			            
            desaturationColorMapping: { 				            
                from:0, 				               
                to:0,				              
                color: "",								
                rangeMinimum:0,				
                rangeMaximum:0
            },

            paletteColorMapping: {                
                colors:[], 
            },            
            highlightOnSelection: false,			           
			selectionMode: "default",		
            highlightGroupOnSelection: false,
			groupSelectionMode: "default",
			draggingOnSelection: false,
			draggingGroupOnSelection: false,
            showTooltip: false,
            tooltipTemplate: null,
            highlightBorderThickness: 5,
            highlightGroupBorderThickness: 4,
            highlightBorderBrush: "gray",
            highlightGroupBorderBrush: "gray"

        },
        
        dataTypes: {
            dataSource: "data",
            treeMapItems: "array",
            levels: "array",
            rangeColorMapping: "array",
            paletteColorMapping: "object",
            groupColorMapping: "array"
        },
		
        observables: ["dataSource",
              "colorValuePath",
              "weightValuePath",
              "showLegend",
              "enableResize",
              "highlightOnSelection",
			  "selectionMode",
			  "groupSelectionMode",
             "highlightGroupOnSelection",
             "enableDrillDown",
              "drillDownHeaderColor",
             "drillDownSelectionColor",
             "showTooltip",
             "highlightBorderThickness",
             "highlightBorderBrush",
               "itemsLayoutMode",
              "leafItemSettings.borderThickness",
              "leafItemSettings.borderBrush",
              "leafItemSettings.showLabels",
              "leafItemSettings.labelPath",
              "legendSettings.iconWidth",
              "legendSettings.iconHeight",
              "legendSettings.dockPosition",
              "legendSettings.height",
              "legendSettings.width",
              "uniColorMapping.color",
              "desaturationColorMapping.from",
              "desaturationColorMapping.to",
              "desaturationColorMapping.color",
              "desaturationColorMapping.rangeMinimum",
              "desaturationColorMapping.rangeMaximum"              
        ],
        _tags: [
            {
                tag: "levels",
                attr: ["groupPath", "groupGap", "headerHeight", "showHeader", "groupPadding", "groupBackground", "groupBorderColor", "groupBorderThickness"],
                singular: "level"
            },
            {
                tag: "rangeColorMapping",
                attr: ["color", "legendLabel", "from", "to"],
                singular: "rangeColor"
            },           
            {
                tag: "groupColorMapping",
                attr: ["groupID",
            {
                tag: "groupColorMapping.rangeColorMapping",
                attr: ["color", "legendLabel", "from", "to"],
                singular: "groupRangeColor"
            },            

              "uniColorMapping.color",
              "desaturationColorMapping.from",
              "desaturationColorMapping.to",
              "desaturationColorMapping.color",
              "desaturationColorMapping.rangeMinimum",
              "desaturationColorMapping.rangeMaximum"
                ],
                singular: "groupColor"
            }

        ],

        dataSource: ej.util.valueFunction("dataSource"),
        colorValuePath: ej.util.valueFunction("colorValuePath"),
        weightValuePath: ej.util.valueFunction("weightValuePath"),
        showLegend: ej.util.valueFunction("showLegend"),
        enableResize: ej.util.valueFunction("enableResize"),
        highlightOnSelection: ej.util.valueFunction("highlightOnSelection"),
		selectionMode:ej.util.valueFunction("selectionMode"),
        highlightGroupOnSelection: ej.util.valueFunction("highlightGroupOnSelection"),
		groupSelectionMode:ej.util.valueFunction("groupSelectionMode"),
        enableDrillDown: ej.util.valueFunction("enableDrillDown"),
        drillDownHeaderColor: ej.util.valueFunction("drillDownHeaderColor"),
        drillDownSelectionColor: ej.util.valueFunction("drillDownSelectionColor"),
        showTooltip: ej.util.valueFunction("showTooltip"),
        highlightBorderThickness: ej.util.valueFunction("highlightBorderThickness"),
        itemsLayoutMode: ej.util.valueFunction("itemsLayoutMode"),
        highlightBorderBrush: ej.util.valueFunction("highlightBorderBrush"),
        borderThickness: ej.util.valueFunction("leafItemSettings.borderThickness"),
        borderBrush: ej.util.valueFunction("leafItemSettings.borderBrush"),
        showLabels: ej.util.valueFunction("leafItemSettings.showLabels"),
        labelPath: ej.util.valueFunction("leafItemSettings.labelPath"),
        iconWidth: ej.util.valueFunction("legendSettings.iconWidth"),
        iconHeight: ej.util.valueFunction("legendSettings.iconHeight"),
        dockPosition: ej.util.valueFunction("legendSettings.dockPosition"),
        legendheight: ej.util.valueFunction("legendSettings.height"),
        legendwidth: ej.util.valueFunction("legendSettings.width"),
        itemTemplate: ej.util.valueFunction("leafItemSettings.itemTemplate"),
        uniColor: ej.util.valueFunction("uniColorMapping.color"),
        _color: ej.util.valueFunction("desaturationColorMapping.color"),
        _from: ej.util.valueFunction("desaturationColorMapping.from"),
        _to: ej.util.valueFunction("desaturationColorMapping.to"),
        _rangeMinimum: ej.util.valueFunction("desaturationColorMapping.rangeMinimum"),
        _rangeMaximum: ej.util.valueFunction("desaturationColorMapping.rangeMaximum"),

        _initPrivateProperties: function () {
            this._svgDocument = null;
            this._templateDiv = null;
            this._legenddiv = null;
            this._drillHeaderDiv = null;
            this._drillHoverDiv = null;
            this._legendHeight = 0;
            this._backgroundTile = null;
            this._tooltipSize = { height: 0, width: 0 };
            this._height = 500;
            this._margintop = null;
            this._marginleft = null;
            this._startPoint = { x: 0, y: 0 };
            this._stopPoint = { x: 0, y: 0 };
            this.mouseClickable = false;
            this.dragDiv = null;
            this._initDiv = null;
            this._width = 500;
            this._svgns = "http://www.w3.org/2000/svg";
            this._prevSelectedItems=[];
            this._prevSelectedHeaders = [];
            this._isLevelColors;
			 if (this.selectedItems == null)
                this.selectedItems = [];
            this.treeMapItemSelected = null;
            this.treeMapGroupSelected = null;
            this._browser = null;
            this._toolTipElement = null;
            this._rootTreeMapItems = [];
            this.treemapgroups = [];
            this._drillHeader = null;
            this._drilldownItem = null;
            this._drilldownItems = [];
            this._treeMapHeaders = [];
            this._drilldownHeaders = [];
            this.drillStarted = null;
            this.drillDownItemSelected = null;
            this._treeMapLabels = [];
            this._labelTemplateElements = [];
            this._itemGroups = [];
            this._prevSelectedGroupDatas = [];
            this._interactiveArrow = null;
            this._legendRects = [];
        },

        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "itemsLayoutMode":
                        this.itemsLayoutMode(options[prop]);
                        this.refresh();
                        break;
                }
            }
        },
		
		
		
        _levels: function(index, property, value, old){
            this.refresh();
        },
        _rangeColorMapping: function(index, property, value, old){
            this._setColorMappings(this._rootTreeMapItems, this.model);
        },
        _groupColorMapping: function(index, property, value, old){
            this._setColorMappings(this._rootTreeMapItems, this.model);
        },
        _groupColorMapping_rangeColorMapping: function(index, property, value, old){
            this._setColorMappings(this._rootTreeMapItems, this.model);
        },
      
        _init: function () {

            this._initPrivateProperties();
            var style = $('<style>.drillGroupRect:hover{ background-color:' + this.drillDownSelectionColor() + ';}</style>');
            $('html > head').append(style);
            this._levelInitialize();
            if (!this._isSVG())
            {
                document.createStyleSheet().addRule(".vml", "behavior:url(#default#VML);display:inline-block");
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml', "#default#VML");
            }
            this.refresh();           

        },

        _destroy: function() {
            this._unWireEvents();
            $(this.element).removeClass("e-treemap e-js").find().remove(".tooltipelement");
            $(this.element).empty();
        },
       
        _levelInitialize: function () {
            var proxy = this;
            if (this.model.levels != null) {
                $.each(this.model.levels, function (index, element) {                    
                    element = proxy._checkArrayObject(element, index);
                    var obj = new treeMapLevel();                    
                    $.extend(obj, element);
                    $.extend(element, obj);                    
                });
            }
            else {
                this.levels.treeMapLevel = new treeMapLevel();

            }			
          
        },

        _checkArrayObject: function (element, initialName) {

            var publicProperties = ["borderBrush", "dataSource", "groupColorMapping", "enableDrillDown", "drillDownHeaderColor", "drillDownSelectionColor", "colorValuePath", "weightValuePath", "treeMapItems", "showLegend", "borderBrush", "borderThickness", "enableResize", "enableGradient", "isResponsive", "itemsLayoutMode", "levels", "groupBackground", "groupBorderColor", "groupBorderThickness", "groupPadding", "groupPath", "groupGap", "headerHeight", "showHeader", "showLabels", "headerTemplate", "labelTemplate", "labelPosition", "headerVisibilityMode", "labelVisibilityMode", "color", "leafItemSettings", "showLabels", "labelPath", "gap", "itemTemplate", "legendSettings", "template", "iconHeight", "iconWidth", "height", "width", "mode", "leftLabel", "rightLabel", "dockPosition", "alignment", "highlightOnSelection", "selectionMode", "highlightGroupOnSelection", "groupSelectionMode", "draggingOnSelection", "draggingGroupOnSelection", "showTooltip", "tooltipTemplate", "highlightBorderThickness", "highlightGroupBorderThickness", "highlightBorderBrush", "highlightGroupBorderBrush"];

            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (publicProperties.indexOf(name) > -1) {
                    if (innerElement instanceof Array) {
                        proxy._checkArrayObject(innerElement, name);
                    }
                    else if (innerElement != null && typeof innerElement == "object") {
                        var allObjects = new treeMapLevel();
                        proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                    }
                }
            });
            return element;
        },

        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        refresh: function (isClear) {
            if(document.getElementById(this._id) != null || document.getElementById(this._id) != undefined){
                this._unWireEvents();
            if (!isClear) {
                this._drilldownItems = [];
            }
            if (this.model.enableResize || this.model.isResponsive) {
                if (this._drilldownItems.length == 0) {
                    this._initPrivateProperties();
                }
            }
            else {
                this._initPrivateProperties();
            }

            $(this.element).empty();
            if (this._svgDocument != null) {
                $(this._svgDocument).empty();
            }
            this._height = this.element.height();
            this._width = this.element.width();
            if (this._height == 0) {
                this._height = this.element[0].parentElement.clientHeight != 0 ? this.element[0].parentElement.clientHeight : $(document).height();
            }
            if (this._width == 0) {
                this._width = this.element[0].parentElement.clientWidth != 0 ? this.element[0].parentElement.clientWidth : $(document).width();
            }
            if (this.showLegend()) {
                this._sizeCalculation();
            }            
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            this._browser = browser;
            $(this.element).css({ 'position': 'relative' });			
           
            if (this._height != 0 && this._width != 0) {                
                this._templateDiv = $("<div class='_templateDiv'></div>");
                this._templateDiv.css({
                    'pointer-events': 'none', 'overflow': 'hidden', 'float': 'left',
                    'z-index': '2', 'height': this._height, 'width': this._width, "margin-top": this._margintop,  "margin-left": this._marginleft, "position": "absolute", "left": "0", "top": "0"
                });
                if (this.enableDrillDown() && this._drilldownItems.length == 0) {
                    this._drillHeaderDiv = $("<div class='_drillHeaderDiv'></div>");
                    this._drillHeaderDiv.css({
                        'overflow': 'hidden', 'float': 'left',
                        'z-index': '3', "position": "absolute", "left": "0", "top": "0"
                    });
                    this._drillHoverDiv = $("<div class='_drillHoverDiv'></div>");
                    this._drillHoverDiv.css({
                        'overflow': 'hidden', 'float': 'left',
                        'z-index': '3', 'height': this._height - 30, 'width': this._width, "position": "absolute", "left": "0", "top": "30px"
                    });
                }
                this._backgroundTile = $('<div id="backgroundTile" style="overflow:hidden;z-index:0;"></div>');
                if(this.showLegend()) {
                    if (this.model.legendSettings != null) {
                        this._renderLegend();                       
                    }
                }
                if(this.dataSource()!=null){
                    if (this._drilldownItems.length > 0) {
                        if (this.enableDrillDown() && this.model.levels.length >= 1 && ((this._drilldownItem != null) || this._drilldownItem == null)) {
                            for (var i = 0; i < this._drillHeaderDiv[0].children[0].children.length; i++) {
                                if (this._drillHeaderDiv[0].children[0].children[i].className == "e-drilldownlabel")
                                    header = this._drillHeaderDiv[0].children[0].children[i].innerHTML;
                            }
                            this._backgroundTile[0].innerHTML = "";
                            this._drillHeaderDiv[0].innerHTML = "";
                            this._templateDiv[0].innerHTML = "";
                            this._drillHoverDiv[0].innerHTML = "";
                            this._drillDownHeader(header, true);
                            this._groupdrillDownLevels(this._drilldownItems[this._drilldownItems.length - 1], this._drilldownItems.length);
                        }
                        if (this.showLegend()) {
                            this._generateLegend();
                        }
                        this._generateToolTip();
                    }
                    else {
                        if (this.model.dataSource instanceof ej.DataManager) {
                            this._processOData(this);
                        }
                        else {
                            this._renderTreeMap(this.dataSource());
                        }
                    }
                }
                if (this.enableResize() || this.model.isResponsive) {
                    this._on($(window), "resize", this._treeMapResize);
                }
            }
			this._selectItemResize();
            this._trigger("refreshed", "");
            }
            
        },
		
       _renderTreeMap: function (data) {
            this.model.browserInfo = this.browserInfo();
            if (this.showLegend()) {
                this._generateLegend();
            }
            this._groupLevels(data);

            this._generateToolTip();
            this._renderLabels();
        },
		
   _renderLegend: function () {
            this._legenddiv = $("<div class='LegendDiv'/>");
            this._legenddiv.appendTo(this.element);

            this._legenddiv.css({
                'pointer-events': 'none', 'overflow': 'hidden', "position": "absolute",
                'z-index': '2', 'height': this._legendSize.height, 'width': this._legendSize.width
            });
        },

        _processOData: function (treemap) {  
            var treeMap= this;		 
            var queryPromise = treemap.model.dataSource.executeQuery(treemap.model.query);
            queryPromise.done(function (e) {
                if(e.result!=null)
                {
                    treeMap._renderTreeMap(e.result);
                }
            });        
        },

        _treeMapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var treemap = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                treemap.refresh(true);
            }, 500);
        },
              
        _unWireEvents: function(){
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();                      
            

        },

        _groupLevels: function (data) {
           
            var subItems = null, isHierarchical = this.model.isHierarchicalDatasource;
            if (this.enableDrillDown()) {                
                this._drilldownHeaders.push(this.model.header || this.model.levels[0].groupPath);
                this._drillDownHeader(false);
            }
            var rootItems = this._getTopGroupitem(this.labelPath(), data, 0);
            subItems = rootItems;
            if (this.model.levels.length == 0) {
                var levelItem = this.model.leafItemSettings;
                this._rootTreeMapItems = rootItems;
                this._getTopLevels(rootItems[0].innerItems, levelItem);
                this._generateTreeMapItems(rootItems[0].innerItems, "gray", this.colorValuePath());
                if(this.showLabels())
                    this._generateLabels(rootItems[0].innerItems, levelItem);
            }
            if(this.model.levels.length>0){
                for (var i = 0; i <= this.model.levels.length; i++) {
                    var levelItem = this.model.levels[i];
                  if (levelItem != null) {
                        if (!$.isNumeric(levelItem.groupGap)) levelItem.groupGap = 0;
                        if (!$.isNumeric(levelItem.groupPadding)) levelItem.groupPadding = 0;
                        if (!$.isNumeric(levelItem.groupBorderThickness)) levelItem.groupBorderThickness = 0;
                    }
                    if (i != 0) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) levelItem = this.model.leafItemSettings;
                        this._generateSubItems(cloneItems, levelItem, subItems, PrevItem);
                        
                        this._rootTreeMapItems = subItems;                        
                        if (this.enableDrillDown())
                        {
                            if (i == 1) {
                                this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);                                
                                return false;                          
                            }
                        }
                        else if (i == this.model.levels.length) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);
                        }                                                  
                        if (this.showLabels() && i == this.model.levels.length) {
                            this._generateLabels(subItems, levelItem);
                        }
                      
                    }
                    else {
                        var rootItems1 = this._getGroupitem(isHierarchical ? this.labelPath() : levelItem.groupPath, rootItems[0].Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;                        
                    }
                }
                
            }
        },
        
        _groupdrillDownLevels: function (data, actualindex) {       
          
            var subItems = null, isHierarchical = this.model.isHierarchicalDatasource;
            if (actualindex == this.model.levels.length-1)
            {
                this._drillHoverDiv.css("pointer-events", "none");
            }
            else
            {
                this._drillHoverDiv.css("pointer-events", "auto");
            }
            if (this.model.levels.length > 0) {
                var canExecute = false;
                for (var i = isHierarchical ? actualindex - 1 : 0; i < this.model.levels.length; i++) {                   
                    var levelItem = this.model.levels[i];
                  if (levelItem != null) {
                        if (!$.isNumeric(levelItem.groupGap)) levelItem.groupGap = 0;
                        if (!$.isNumeric(levelItem.groupPadding)) levelItem.groupPadding = 0;
                        if (!$.isNumeric(levelItem.groupBorderThickness)) levelItem.groupBorderThickness = 0;
                    }
                    if (i != (isHierarchical ? actualindex - 1 : 0)) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) {
                            levelItem = this.model.leafItemSettings;
                        }
                        if (i == actualindex + 1) {
                            this._generateSubItems(cloneItems, levelItem, subItems, PrevItem, null, (actualindex + 1 != i));

                        }
                        else {
                            this._generateSubItems(cloneItems, levelItem, subItems, PrevItem, true, (actualindex + 1 != i));

                        }
                        this._rootTreeMapItems = subItems;
                        if ((i == actualindex + 1 || i == this.model.levels.length - 1) && !canExecute) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);                                                       
                            canExecute = true;
                        }                        
                        
                        if (actualindex == this.model.levels.length - 1 && i == actualindex && levelItem.showLabels) {
                            this._generateLabels(subItems, levelItem);
                        }
                           
                    }
                    else {

                        var rootItems1 = this._getGroupitem(this.model.isHierarchicalDatasource ? null : levelItem.groupPath, data.Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;
                    }
                }
            }
            
            if (this.showLegend()) {
                this._generateLegend();
            }
            this._renderLabels();
            this._trigger("drillDownItemSelected", { level: actualindex, header: this.getDrillDownHeader(this._drilldownHeaders), prevLevel: actualindex - 1 });
        },

        _getTopLevels: function (rootItems, level) {
            var gap = (level.groupGap != null) ? level.groupGap : (level.gap!=null?level.gap:0);
            var layout = this.itemsLayoutMode();
            var legendHeight = 0;
            var defaultspacing = 0;
            if (this.enableDrillDown()) {
                defaultspacing = 30;
            }
            if (this.showLegend() && this.model.legendSettings != null) {
                legendHeight = legendHeight;                    
            }
                
            this._legendHeight = legendHeight;
            if (this.enableDrillDown()) {
                this._drillHeaderDiv.css({
                    "top": legendHeight
                });

                this._drillHoverDiv.css({
                    "top": 30 + legendHeight
                })
            }               
                
            if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceHorizontal) {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, true, legendHeight);
            }
            else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceVertical) {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, false, legendHeight);
            }
            else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceAuto) {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, null, legendHeight);
            }
            else {
                this._calculateSquarifiedItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, legendHeight);
            }
            
        },
      
        _generateSubItems: function (cloneItems, levelItem, subItems, PrevItem, disablelabels, disablebackground) {
            var grouppading = PrevItem.groupPadding;
            if (grouppading == "") { grouppading = 0; }
            var layout = this.itemsLayoutMode();
            var cloneItemsCount = cloneItems.length;
            var path = this.model.isHierarchicalDatasource ? null : levelItem.groupPath;
            if (path == null) path = this.labelPath();
            if (path == null) path = this.weightValuePath();
            var gap = (levelItem.groupGap != null) ? levelItem.groupGap : (levelItem.gap != null) ? levelItem.gap:0;
            var headerHeight = ((PrevItem.showHeader || PrevItem.showHeader == null) && !this.enableDrillDown()) ? PrevItem.headerHeight : 0;
            var totalItemsCount = 0;
            for (var j = 0; j < cloneItemsCount; j++) {
                var treeItems = this._getGroupitem(path, cloneItems[j].Data, levelItem.headerHeight, this.model.isHierarchicalDatasource ? PrevItem.groupPath:null);
                if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceHorizontal) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, true, 0);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceVertical) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, false, 0);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceAuto) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, null, 0);
                }
                else {
                    this._calculateSquarifiedItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, 0);
                }                
                var headerItem = this._rootTreeMapItems[j];                
                headerItem.headerHeight = PrevItem.headerHeight;
                headerItem.showHeader = PrevItem.showHeader;
                headerItem.headerWidth = cloneItems[j].ItemWidth;
                headerItem.headerTemplate = PrevItem.headerTemplate;
                headerItem.headerLeftPosition = cloneItems[j].LeftPosition;
                headerItem.headerTopPosition = cloneItems[j].TopPosition;

                if (this.enableDrillDown())
                    this._createBackGround(headerItem, PrevItem, cloneItems[j], disablebackground);                

                if (PrevItem.showHeader || PrevItem.showHeader == null) {
                   
                    if(!this.enableDrillDown()){
                        this._createBackGround(headerItem, PrevItem, cloneItems[j]);
                        this._generateHeaders(headerItem, PrevItem);
                    }
                    
                }
               
                cloneItems[j].ChildtreeMapItems = treeItems;
                cloneItems[j].GroupingLevel = this.model.levels.indexOf(PrevItem) + 1;
				
                if (this.model.levels.indexOf(PrevItem) == 0){
                    if (this.model.groupColorMapping != null && this.model.groupColorMapping.length > 0){
                        this._isLevelColors= true;
                        for (var a = 0; a < this.model.groupColorMapping.length; a++){
                            var colormapping = this.model.groupColorMapping[a];
                            if (colormapping.groupID == headerItem.header){
                                this._setColorMappings(treeItems, colormapping,true);
                            }
                        }
                    }
                }
                else if( this.model.levels.indexOf(PrevItem) > 0 )
                {				
                    if (this.model.groupColorMapping != null  && this.model.groupColorMapping.length > 0){
                        for (var a = 0; a < this.model.groupColorMapping.length; a++){				
                            var colormapping = this.model.groupColorMapping[a];
                            if (colormapping.groupID == cloneItems[j].ParentHeader){
                                this._setColorMappings(treeItems, colormapping,true);
                            }
                        }
                    }
                }
				
                for (var k = 0; k < treeItems.length; k++) {
                    subItems[totalItemsCount] = treeItems[k];
                    if (treeItems[k].backgroundColor == null)
                        treeItems[k].backgroundColor = cloneItems[j].backgroundColor;
                    if (treeItems[k].backgroundOpacity == null)
                        treeItems[k].backgroundOpacity = cloneItems[j].backgroundOpacity;                   
                    if ((this.enableDrillDown()) || (subItems[totalItemsCount].ItemHeight < headerItem.ItemHeight - headerItem.headerHeight)) {
                        subItems[totalItemsCount].ParentHeader = headerItem.header;
                        subItems[totalItemsCount].ItemHeight -= grouppading;
                        subItems[totalItemsCount].ItemWidth -= grouppading;
                        subItems[totalItemsCount].LeftPosition += PrevItem.groupBorderThickness;
                        subItems[totalItemsCount].TopPosition += PrevItem.groupBorderThickness;
                       
                    }
                    totalItemsCount++;
                }              
            }
            if (disablelabels == undefined &&  PrevItem.showLabels) {
                this._generateLabels(cloneItems, PrevItem);
            }
        },
       
        _createBackGround: function (header, level,parentitem, disablebackground) {            
            var rect = $("<div />");
            var height = (header.showHeader || header.showHeader == null || this.enableDrillDown()) ? header.ItemHeight - (2 * level.groupBorderThickness) : 0;
            var top = (header.showHeader || header.showHeader == null || this.enableDrillDown()) ? header.TopPosition : 0;
            rect.css({
                "height": height + "px",
                "width": header.ItemWidth - (2 * level.groupBorderThickness) + "px",
                "left": header.LeftPosition + "px",
                "top": top + "px",
                "border-style": "solid",
                "border-width": level.groupBorderThickness+"px",
                "position": "absolute"               
            });

            if(level.groupBorderColor!=null)
            {
                rect.css("border-color",level.groupBorderColor);
            }
            if(level.groupBackground!=null)
            {
                rect.css("background-color",level.groupBackground);
            }
            this.treemapgroups.push({header:header.header,element:rect});
            rect.appendTo(this._backgroundTile);
            var height = parseFloat(rect[0].style.top) - 30 - this._legendHeight + "px";            
            if (this.enableDrillDown() && !disablebackground) {
                var hoverrect = $("<div class='drillGroupRect'/>");
                hoverrect.css({
                    "height": rect[0].style.height,
                    "width": rect[0].style.width,
                    "left": rect[0].style.left,
                    "top": height,
                    "position": "absolute",
                    "opacity": 0.2,
                    "display": "block"
                });
                this._drillHoverDiv.append(hoverrect);
                $(hoverrect).mousedown({ treemap: this, level: level, param1: parentitem }, this._headerClickFunction);
                $(hoverrect).mouseleave({ treemap: this, param1: header.header }, this._mouseLeaveFunction);
                var hoverData = { data: header.Data, label:header.header, header: header.header };
                $(hoverrect).mousemove({ treemap: this, hoverItem: hoverData }, this._mouseRectHoverFunction);
            }
            $(rect).mousedown({ treemap: this, level: level, param1: parentitem }, this._headerClickFunction);
            $(rect).mousemove({ treemap: this, hoverItem: header.Data }, this._mouseRectHoverFunction);
        },

        _mouseRectHoverFunction: function (event) {
          
            var treeMap = event.data.treemap;
            var item = event.data.hoverItem;
            if (treeMap.showTooltip()) {
                var element = treeMap._toolTipElement;
                var template = treeMap.model.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                    var htmlString = $("#" + template).render(item);
                    $(element).html(htmlString);
                    var height = element[0] != null ? element[0].clientHeight : element.clientHeight;
                    var width = element[0] != null ? element[0].clientWidth : element.clientWidth;
                    treeMap._tooltipSize = { height: height, width: width };
                }

                var tooltipSize = treeMap._tooltipSize;
                if (tooltipSize.width + event.pageX >= treeMap._width) {
                    event.pageX -= tooltipSize.width;
                    if (event.pageX < 0) {
                        event.pageX = 10;
                    }
                }
                if (tooltipSize.height + event.pageY >= treeMap._height) {
                    event.pageY -= tooltipSize.height;
                    if (event.pageY < 0) {
                        event.pageY = 10;
                    }
                }               
                if (element != null) {

                    if (treeMap.enableDrillDown()) {
                        $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });

                    }
                    else {
                        $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "none" });

                    }

                }

            }
        },		        
               
        _mouseLeaveFunction: function (event) {
            var treeMap = event.data.treemap;
            if (treeMap.showTooltip())
            {
                if (treeMap._toolTipElement != null) {
                    $(treeMap._toolTipElement).css("display", "none");
                }
            }
        },
		       
        _headerClickFunction: function (event) {
            var level = event.data.level;
            var data = event.data.param1;
            var ctrlkey = event.ctrlKey;
            var treeMap = event.data.treemap;
            if (treeMap.highlightGroupOnSelection()) {
                if (treeMap._browser != "msie") {
                    var lastItem = treeMap._backgroundTile[0].children[treeMap._backgroundTile[0].children.length - 0];
                    treeMap._backgroundTile[0].insertBefore(this, lastItem);
                }
                if (treeMap.groupSelectionMode() == "multiple" &&  ctrlkey) {

                    if ($.inArray(this,treeMap._prevSelectedHeaders)==-1) {
                        $(this).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                        treeMap._prevSelectedHeaders.push(this);
                        treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                    }
                    else {
                        
                        $(this).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                        var index = treeMap._prevSelectedHeaders.indexOf(this);
                        treeMap._prevSelectedHeaders.splice(index, 1);
                        treeMap._prevSelectedGroupDatas.splice(index, 1);
                    }
                }
                else {
                    for (var k = 0; k < treeMap._prevSelectedHeaders.length; k++) {
		               
                        $(treeMap._prevSelectedHeaders[k]).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                    }

                    if ($.inArray(this, treeMap._prevSelectedHeaders) == -1) {
                        $(this).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                        treeMap._prevSelectedHeaders = [];
                        treeMap._prevSelectedHeaders.push(this);
                    }
                    else {
                        treeMap._prevSelectedHeaders = [];
                    }
		            
                    treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                }
            }
            if (treeMap.enableDrillDown())
            {
                treeMap._trigger("drillStarted", event);
            }
			if (treeMap.enableDrillDown() && treeMap.model.levels.length > 1 && ((treeMap._drilldownItem != null && treeMap._drilldownItem.header != data.header) || treeMap._drilldownItem == null)) {
                    var header = "";
                    for(var i=0;i< treeMap._drillHeaderDiv[0].children[0].children.length;i++)
                    {
                        if (treeMap._drillHeaderDiv[0].children[0].children[i].className == "e-drilldownlabel")
                            header = treeMap._drillHeaderDiv[0].children[0].children[i].innerHTML;
                    }
                    treeMap._backgroundTile[0].innerHTML = "";
                    treeMap._labelTemplateElements = [];
                    treeMap._drillHeaderDiv[0].innerHTML = "";
                    treeMap._templateDiv[0].innerHTML = "";
                    treeMap._drillHoverDiv[0].innerHTML = "";
                    treeMap._drilldownHeaders.push(data.header);
                    treeMap._drillDownHeader(true);
                     treeMap._drilldownItem = data;
                    treeMap._drilldownItems.push(data);
                    treeMap._groupdrillDownLevels(data, treeMap._drilldownItems.length);
                    return false;
		       		        
                }
            },
            
       _rectMouseDown: function (event) {
            if (event.data.treeMap.isTouch(event))
                event.preventDefault();

        },
        _docClickFunction: function (event) {
            var treeMap = event.data.treemap;
            var element = this._toolTipElement, treemap = this;
            clearTimeout(treemap.model.timer);
            if (element) {
                treemap.model.trackerElement = element;
                treemap.model.timer = setTimeout(function () {
                    if (treemap.model.trackerElement)
                        $(treemap.model.trackerElement).fadeOut(500, function () {
                            treemap.model.trackerElement = null;
                        });

                }, 1200);

            }
        },
        mousePosition: function (evt) {
            if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX > 0)
                return evt;
            else if (evt.originalEvent && !ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX > 0)
                return evt.originalEvent;
            else if (evt.originalEvent && evt.originalEvent.changedTouches != undefined) {
                if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                    return evt.originalEvent.changedTouches[0];
            }
            return evt;

        },
        _highlightTreemap: function (event) {
            var treeMap = event.data.treemap,
                itemCollection = event.data.Param1,
                level = treeMap.model.levels[0], tooltipObject, isContain, tooltipContain, ctrlkey = event.ctrlKey,
                selectedGroup, item, rect,
                pointer = treeMap.mousePosition(event);
            if (treeMap.highlightGroupOnSelection()) {
                for (var i = 0; i < treeMap.treemapgroups.length; i++) {
                    selectedGroup = treeMap.treemapgroups[i];
                    item = treeMap.treemapgroups[i].element;
                    rect = item[0].getBoundingClientRect();

                    if (rect.left < pointer.clientX && rect.left + rect.width > pointer.clientX
                   && rect.top < pointer.clientY && rect.top + rect.height > pointer.clientY) {
                        $(item).css({
                            "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush(),
                            "box-sizing": "border-box",
                            "-moz-box-sizing": "border-box",
                            "-webkit-box-sizing": "border-box",
                        });
                        isContain = true;
                        break;
                    }
                }
                if (isContain) {
                    if (treeMap.groupSelectionMode() == "multiple" && ctrlkey) {

                        if ($.inArray(item, treeMap._prevSelectedHeaders) == -1) {
                            $(item).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                            treeMap._prevSelectedHeaders.push(selectedGroup);
                            treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                        }
                        else {

                            $(item).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor, });
                            var index = treeMap._prevSelectedHeaders.indexOf(selectedGroup);
                            treeMap._prevSelectedHeaders.splice(index, 1);
                            treeMap._prevSelectedGroupDatas.splice(index, 1);
                        }
                    }
                    else {
                        for (var k = 0; k < treeMap._prevSelectedHeaders.length; k++) {

                            $(treeMap._prevSelectedHeaders[k].element).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                        }

                        if ($.inArray(item, treeMap._prevSelectedHeaders) == -1) {
                            $(item).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                            treeMap._prevSelectedHeaders = [];
                            treeMap._prevSelectedHeaders.push(selectedGroup);
                        }
                        else {
                            treeMap._prevSelectedHeaders = [];
                        }

                        treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                    }
                }
                treeMap._trigger("treeMapGroupSelected", { selectedGroups: treeMap._prevSelectedHeaders, orginalEvent: event });
            }

            if (treeMap.highlightOnSelection()) {

                for (var i = 0; i < treeMap._rootTreeMapItems.length; i++) {
                    var item = treeMap._rootTreeMapItems[i];
                    var rect = item.Rectangle.getBoundingClientRect();
                    if (rect.left < pointer.clientX && rect.left + rect.width > pointer.clientX
                        && rect.top < pointer.clientY && rect.top + rect.height > pointer.clientY) {
                        $(item.Rectangle).css({
                            "border-width": treeMap.highlightBorderThickness(), "border-color": treeMap.highlightBorderBrush(),
                            "box-sizing": "border-box",
                            "-moz-box-sizing": "border-box",
                            "-webkit-box-sizing": "border-box",
                        });
                        isContain = true;
                        break;
                    }
                }

                if (isContain) {
                    if (treeMap.selectionMode() == "multiple" && ctrlkey) {
                        if ($.inArray(item.Rectangle, treeMap._prevSelectedItems) == -1) {
                            $(item.Rectangle).css({
                                "border-width": treeMap.highlightBorderThickness(), "border-color": treeMap.highlightBorderBrush(),
                                "box-sizing": "border-box",
                                "-moz-box-sizing": "border-box",
                                "-webkit-box-sizing": "border-box",
                            });
                            treeMap._prevSelectedItems.push(item.Rectangle);
                            treeMap.selectedItems.push(item.Data);
                        }
                        else {
                            $(item.Rectangle).css({ "border-width": treeMap.borderThickness(), "border-color": treeMap.borderBrush() });
                            var index = treeMap._prevSelectedItems.indexOf(item.Rectangle);
                            treeMap._prevSelectedItems.splice(index, 1);
                            treeMap.selectedItems.splice(index, 1);
                        }
                    }
                    else {
                        for (var k = 0; k < treeMap._prevSelectedItems.length; k++) {
                            $(treeMap._prevSelectedItems[k]).css({ "border-width": treeMap.borderThickness(), "border-color": treeMap.borderBrush() });
                        }

                        if ($.inArray(item.Rectangle, treeMap._prevSelectedItems) == -1) {
                            $(item.Rectangle).css({ "border-width": treeMap.highlightBorderThickness(), "border-color": treeMap.highlightBorderBrush() });
                            treeMap._prevSelectedItems = [];
                            treeMap.selectedItems = [];
                            treeMap._prevSelectedItems.push(item.Rectangle);
                            treeMap.selectedItems.push(item.Data);
                        }
                        else {
                            treeMap._prevSelectedItems = [];
                            treeMap.selectedItems = [];
                        }
                    }
                    treeMap._trigger("treeMapItemSelected", { selectedItems: treeMap.selectedItems, originalEvent: event });
                }

            }
          
        },
        doubleTap: function (event) {

            var pointer = event.targetTouches ? event.targetTouches[0] : event,
               currX = pointer.pageX,
               currY = pointer.pageY,
               model = this.model,
               padding = 200,
               element,

               treeMap = event.data.treemap;

            treeMap.model.cachedX = treeMap.model.cachedX || currX;

            treeMap.model.cachedY = treeMap.model.cachedY || currY;

            timestamp = new Date().getTime();


            if (Math.abs(currX - treeMap.model.cachedX) < padding && Math.abs(currY - treeMap.model.cachedY) < padding)
                treeMap.model.tapNum++;

        },

        rectClick: function (event) {

            var treeMap = event.data.treemap;
            treeMap.doubleTap(event);
            if (!treeMap.isTouch(event)) {
                treeMap._highlightTreemap(event);
            }
        },        
            dragDown: function (event) {

                var treeMap = event.data.treemap;
                var level = treeMap.model.levels[0];

                var treeMapArea = treeMap._svgDocument;
                if (event.type == "mousedown") {
                    treeMap._startPoint = { x: event.pageX - treeMapArea[0].offsetLeft, y: event.pageY - treeMapArea[0].offsetTop };
                    treeMap._stopPoint = { x: event.pageX - treeMapArea[0].offsetLeft, y: event.pageY - treeMapArea[0].offsetTop };
                }
                else if (event.type == "touchstart") {
                    treeMap._startPoint = { x: event.originalEvent.changedTouches[0].pageX - treeMapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - treeMapArea[0].offsetTop };
                    treeMap._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - treeMapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - treeMapArea[0].offsetTop };
                }
                else if (event.type == "MSPointerDown") {
                    treeMap._startPoint = { x: event.originalEvent.pageX - treeMapArea[0].offsetLeft, y: event.originalEvent.pageY - treeMapArea[0].offsetTop };
                    treeMap._stopPoint = { x: event.originalEvent.pageX - treeMapArea[0].offsetLeft, y: event.originalEvent.pageY - treeMapArea[0].offsetTop };
                }

                treeMap.mouseClickable = true;

                if (treeMap.model.draggingGroupOnSelection) {

                    if (treeMap._prevSelectedGroupDatas.length > 0) {
                        for (var i = 0; i < treeMap._prevSelectedGroupDatas.length; i++) {
                            var item = treeMap._prevSelectedGroupDatas[i][0];
                            var rect = item.getBoundingClientRect();
                            if ($.inArray(item, treeMap._prevSelectedGroupDatas[i][0]) == -1) {
                                $(treeMap._prevSelectedGroupDatas[i]).css({ "border-width": level.groupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush });
                            }
                        }
                        treeMap._prevSelectedGroupDatas = [];
                    }
                }

                if (treeMap.model.draggingOnSelection) {

                    if (treeMap._prevSelectedItems.length > 0) {
                        for (var i = 0; i < treeMap._prevSelectedItems.length; i++) {
                            var item = treeMap._prevSelectedItems[i].rectangle;
                            var rect = item.getBoundingClientRect();
                            if ($.inArray(item, treeMap._prevSelectedItems[i].rectangle) == -1) {
                                $(treeMap._prevSelectedItems[i].rectangle).css({ "border-width": level.groupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush });
                            }
                        }
                        treeMap._prevSelectedItems = [];
                    }
                }
            },

            dragMove: function (event) {

                var width; var height;
                var treeMap = event.data.treemap;
                var treeMapArea = treeMap._svgDocument;

                if (treeMap.model.draggingGroupOnSelection || treeMap.model.draggingOnSelection) {

                    if (treeMap.mouseClickable) {

                        if (event.type == "mousemove")
                            treeMap._stopPoint = { x: event.pageX - treeMapArea[0].offsetLeft, y: event.pageY - treeMapArea[0].offsetTop };
                        else if (event.type == "touchmove")
                            treeMap._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - treeMapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - treeMapArea[0].offsetTop };
                        else if (event.type == "MSPointerMove")
                            treeMap._stopPoint = { x: event.originalEvent.pageX - treeMapArea[0].offsetLeft, y: event.originalEvent.pageY - treeMapArea[0].offsetTop };


                        $('#dragDiv').remove();
                        var div = $('<div id = "dragDiv"></div>');

                        width = Math.abs(treeMap._stopPoint.x - treeMap._startPoint.x),
                        height = Math.abs(treeMap._stopPoint.y - treeMap._startPoint.y);
                        $(div).css({ "top": Math.min(treeMap._startPoint.y, treeMap._stopPoint.y), "left": Math.min(treeMap._startPoint.x, treeMap._stopPoint.x), width: width, height: height, border: "1px solid green", position: "absolute", "z-index": 100 });
                        $(div).appendTo("#svgDocument");
                    }
                }
            },

            dragUp: function (event) {

                var treeMap = event.data.treemap;
                var width = Math.abs(treeMap._stopPoint.x - treeMap._startPoint.x);
                var height = Math.abs(treeMap._stopPoint.y - treeMap._startPoint.y);
                var treeMapArea = treeMap._svgDocument;

                if (treeMap.model.draggingGroupOnSelection || treeMap.model.draggingOnSelection) {

                    $('#dragDiv').remove();
                    $("#dragDiv").css({
                        "display": "none"
                    });
                    treeMap.mouseClickable = false;
                }

                if (treeMap.model.draggingGroupOnSelection) {

                    for (var i = 0; i < treeMap.treemapgroups.length; i++) {
                        var item = treeMap.treemapgroups[i].element;
                        var rect = item[0].getBoundingClientRect();

                        if (!(rect.left - treeMapArea[0].offsetLeft + rect.width < Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) || Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) + width < rect.left - treeMapArea[0].offsetLeft ||
                            rect.top - treeMapArea[0].offsetTop + rect.height < Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) || Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) + height < rect.top - treeMapArea[0].offsetTop)) {
                            if (!treeMap._contains(treeMap._prevSelectedGroupDatas, item))
                                treeMap._prevSelectedGroupDatas.push(item);
                        }
                    }

                    for (var i = 0; i < treeMap._prevSelectedGroupDatas.length; i++) {
                        var item = treeMap._prevSelectedGroupDatas[i][0];
                        var rect = item.getBoundingClientRect();

                        $(item).css({
                            "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush,
                            "box-sizing": "border-box",
                            "-moz-box-sizing": "border-box",
                            "-webkit-box-sizing": "border-box",
                        });
                    }
                    treeMap.selectedItems.push(treeMap._prevSelectedGroupDatas);
                }

                if (treeMap.model.draggingOnSelection) {

                    for (var i = 0; i < treeMap._rootTreeMapItems.length; i++) {
                        var item = treeMap._rootTreeMapItems[i];
                        var rect = item.rectangle.getBoundingClientRect();

                        if (!(rect.left - treeMapArea[0].offsetLeft + rect.width < Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) || Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) + width < rect.left - treeMapArea[0].offsetLeft ||
                             rect.top - treeMapArea[0].offsetTop + rect.height < Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) || Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) + height < rect.top - treeMapArea[0].offsetTop)) {
                            if (!treeMap._contains(treeMap._prevSelectedItems, item))
                                treeMap._prevSelectedItems.push(item);
                        }
                    }

                    for (var i = 0; i < treeMap._prevSelectedItems.length; i++) {
                        var item = treeMap._prevSelectedItems[i].rectangle;
                        var rect = item.getBoundingClientRect();
                        $(item).css({
                            "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush,
                            "box-sizing": "border-box",
                            "-moz-box-sizing": "border-box",
                            "-webkit-box-sizing": "border-box",
                        });
                    }
                    treeMap.selectedItems.push(treeMap._prevSelectedItems);
                }
            },

        _generateLabels: function (Items,level) {            
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                if (level.labelTemplate == null) {                    
                    var leftpaddingsize = 8;
                    var toppaddingsize = 5;
                    var textcon = $('<label class="e-treemap-label" >' + item.label + '</label>');
                    if (this.enableDrillDown())
                    {
                        $(textcon).css({
                            "position": "absolute","color":"black",
                            "overflow": "hidden",  "left": item.LeftPosition + leftpaddingsize + "px",
                            "top": item.TopPosition + toppaddingsize + "px", "font-weight":"normal","pointer-events":"none"
                        });
                    }
                    else
                    {
                        $(textcon).css({
                            "position": "absolute", "color": "white",
                            "overflow": "hidden", "left": item.LeftPosition + leftpaddingsize + "px",
                            "top": item.TopPosition + toppaddingsize + "px", "font-weight": "normal", "pointer-events": "none"
                        });
                    }
                    this._treeMapLabels.push(textcon);
                    this._templateDiv.append(textcon);
                    var padding = 0, borderthickness = 0;
                    if (level.groupPadding != undefined)
                        padding = level.groupPadding;
                    if (level.groupBorderThickness != undefined)
                        borderthickness = level.groupBorderThickness;
                    var labelWidth = (textcon[0].getBoundingClientRect().width > 0) ? textcon[0].getBoundingClientRect().width + padding + borderthickness + 2 : item.label.length * 8;
                    var labelHeight = textcon[0].getBoundingClientRect().height;
                    if (labelHeight == 0 && item.ItemHeight > 18)
                        labelHeight = 18;
                    else if (labelHeight > 0 && labelHeight > item.ItemHeight)
                        labelHeight = item.ItemHeight - toppaddingsize;
                    $(textcon).css({ "width": labelWidth + "px", "height": labelHeight + "px" });
                    item.labelPosition = level.labelPosition;
                    item.labelVisibilityMode = borderthickness;
                    item.groupPadding = level.groupPadding;
                    item.groupBorderThickness = level.groupBorderThickness;
                    textcon[0].data = item;
                    this._labelTemplateElements.push(textcon);
					if ( level.labelVisibilityMode== ej.datavisualization.TreeMap.VisibilityMode.HideOnExceededLength) {
						if (labelWidth > item.ItemWidth) {
                        $(textcon).css({
                            "display": "none",


                        });
                        if(item.Rectangle!=null)
                        item.Rectangle.title =item.label;

                        }
						var textMode = level.labelVisibilityMode;
                  }				  				    

				  else {
					    if (labelWidth < item.ItemWidth) {
					        $(textcon).css({
					            "width": labelWidth
					        });
					    }
					    else {
					        $(textcon).css({
					            "width": item.ItemWidth
					        });
					    }                           
				  }				  				  			
                }
                if(level.labelTemplate != null) {
                    var OriginalNode = $("#" + level.labelTemplate);
                    var item_templateDiv = $("<div style='overflow:hidden;display:block;position:absolute;pointer-events: none;'></div>");
                    item_templateDiv[0].data = item;
                    item.labelPosition = level.labelPosition;
                    item.labelVisibilityMode = level.labelVisibilityMode;
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = "";
                    if (item.Data instanceof Array) {
                        htmlString = tmpl2.render({ data: item.Data });
                    }
                    else
                        htmlString = tmpl2.render(item.Data);
                    $(item_templateDiv).html(htmlString);
                    this._templateDiv.append(item_templateDiv);
                    this._labelTemplateElements.push(item_templateDiv);
                }
				}
				           
        },

        _generateToolTip: function () {

            if (this.showTooltip()) {
                if (document.documentMode == 8){
                    var tooltip = document.querySelectorAll("tooltipelement");
                }
                else{
                    var tooltip = document.getElementsByClassName("tooltipelement" + this._id);
                }
                if (tooltip != null && tooltip.length == 0) {
                    var tooltip_templateDiv = $("<div></div>").attr('class', 'tooltipelement' + this._id).css({ 'position': 'absolute', 'z-index': '1000', 'display': 'none', 'pointer-events': 'none' });

                    $(document.body).append(tooltip_templateDiv);
                    this._toolTipElement = tooltip_templateDiv;
                }
                else
                {
                    this._toolTipElement = tooltip[0];
                }
                if (this.model.tooltipTemplate == null) {

                    var path;
                    if (this.labelPath()!=null)
                        path = this.labelPath();
                    else 
                        path = this.weightValuePath();
                    if (path != null) {
                        this.model.tooltipTemplate = 'defaultTooltip';

                        this.element.append($('<div id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:#data["' + path + '"]}}</label></p></div></div></div>'));
                    }
                }
            }           
        },

        _generateLegend: function () {
            if (this.model.rangeColorMapping != null) {
                var colormapping = this.model.rangeColorMapping;
                var legendSettings = this.model.legendSettings;
                if (legendSettings.mode != ej.datavisualization.TreeMap.LegendMode.Interactive) {

                var xPos = 0;
                var yPos = 0;

                var columnWidth = 0;
                var width = 0;
                var height = 0;
                var iconwidth = this.iconWidth() + 5;
                var totalHeight = this.iconHeight() + 15;
                var totalWidth = 0;
                var columnCount = this.model.legendSettings.columnCount;
                for (var i = 0; i < colormapping.length; i++) {

                    var labelwidth = this._calcWidth(colormapping[i].legendLabel);
                    var legendWidth = this.iconWidth() + 10 + labelwidth;


                    if (columnCount != 0) {
                        if (i % columnCount != 0) {
                           
                            this._drawLegend(colormapping[i], xPos, yPos);
                            xPos += (legendWidth+5);

                        }
                        else {
                            if (i != 0)
                            yPos += (this.iconHeight() + 18);
                            xPos = 0;
                            this._drawLegend(colormapping[i], xPos, yPos);
                            xPos += (legendWidth + 5);
                        }
                        }
                    else {
                        if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {

                            if (this._legendSize.width < xPos + legendWidth) {

                                yPos += (this.iconHeight() + 18);
                                xPos = 0;
                                this._drawLegend(colormapping[i], xPos, yPos);
                                xPos += legendWidth;
                            }
                            else {
                                this._drawLegend(colormapping[i], xPos, yPos);
                                xPos += legendWidth;
                            }
                            }
                        else {

                            if (this._legendSize.height < yPos + this.iconHeight()) {

                                yPos = 0;
                                xPos += (columnWidth + 10);
                                this._drawLegend(colormapping[i], xPos, yPos);
                                columnWidth = 0;
                                yPos += (this.iconHeight() + 18);
                            }
                            else {
                                this._drawLegend(colormapping[i], xPos, yPos);
                                columnWidth = Math.max(columnWidth, legendWidth);
                                yPos += (this.iconHeight() + 18);
                                }
                            }

                        }
                    }

                }
                else {
                    this._drawInteractiveLegend();
                }
                if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Center) {
                    this._legenddiv.css({
                        "margin-left": (this._width / 2) - (this._legendSize.width / 2)
                    });
                }
                else if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Far) {
                    this._legenddiv.css({
                        "margin-left": this._width - this._legendSize.width
                    });
                }
                else if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Left || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Center) {
                    this._legenddiv.css({
                        "margin-top": (this._height / 2) - (this._legendSize.height / 2)
                    });
                }
                else if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Left || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Far) {
                    this._legenddiv.css({
                        "margin-top": this._height - this._legendSize.height
                    });
                }

                if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                    this._legenddiv.css({
                        "margin-top": this._height + 5
                    });
                }
                else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) {

                    this._legenddiv.css({

                        "margin-left": this._width + 5

                    });
                }
            }
        },
		 _drawLegend: function (colormapping, xPos, yPos) {
            var legendItem;
            if (this.model.legendSettings.template == "Ellipse") {
                legendItem = this._getEllipseLegend(this.model.legendSettings, xPos, yPos);
            }
            else {
                legendItem = this._getRectLegend(this.model.legendSettings, xPos, yPos);
            }
            xPos += (this.iconWidth() + 5);
            $(legendItem).css("background-color", colormapping.color);
            legendItem.appendTo(this._legenddiv);
            var legendText = $("<div class='e-treemap-legendLabel'/>");
            legendText.css({
                "left": xPos + "px",
                "top": yPos + "px",
                        "position": "absolute"
                    });
                   
            legendText[0].innerHTML = colormapping.legendLabel;
            legendText.appendTo(this._legenddiv);
        },
       
        _calcWidth: function (text) {
            var span = $('<span class ="e-treemap-legendLabel" >' + text + '</span>');
            $('body').append(span);
            var width = span.width();
            span.remove();
            return width;
        },
        
        _getEllipseLegend: function (legengItem, xPos, yPos) {

            var rect = $("<div class='e-mapLegend'/>");
            rect.css({
                "height": this.iconHeight()+ "px",
                "width": this.iconWidth() + "px",
                "border-radius":this.iconHeight()/2+"px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;

        },
       
        _getRectLegend: function (legengItem, xPos, yPos) {
            var rect = $("<div />");
            rect.css({
                "height": this.iconHeight()+ "px",
                "width": this.iconWidth() + "px",                
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;
        },
        
        _getPositionFromParent: function (element, parentelement) {
            var child = element.getBoundingClientRect();
            var parent = parentelement.getBoundingClientRect();
            var width = window.SVGSVGElement ? child.width: child.right-child.left;
            var height = window.SVGSVGElement ? child.height : child.bottom - child.top;            
            return { left: child.left - parent.left, top: child.top - parent.top, height: height, width: width };
        },
        
        _renderLabels: function () {
           
            for (var j = 0; j < this._labelTemplateElements.length; j++) {
                var label = this._labelTemplateElements[j];
                var bounds = this._getPositionFromParent(label[0], this._templateDiv[0]);
                var item = label[0].data;
                var pos = this._getDockPositionPoint({left:item.LeftPosition,top:item.TopPosition, width: item.ItemWidth, height: item.ItemHeight }, bounds, item);
                
                $(label).css({
                    "left": pos.x,
                    "top": pos.y,
                    "pointer-events": "none",
                    "overflow":"hidden"
                });
                if (item.labelVisibilityMode == ej.datavisualization.TreeMap.VisibilityMode.HideOnExceededLength) {
                    if (bounds.height > item.ItemHeight || bounds.width > item.ItemWidth) {
                        $(label).css("display", "none");
                    }
                }
            }
        },
       
        _generateHeaders: function (headerValue,level) {           
            var textMode = level.headerVisibilityMode;
            var generateHeader = false;
            var leftpadding = 2;
            var toppadding = 2;
            if (textMode == ej.datavisualization.TreeMap.VisibilityMode.HideOnExceededLength) {
                var textelement = $('<span>'+headerValue.header+'</span>');
                $(document.body).append(textelement);
                var width = textelement.outerWidth();
                textelement.remove();
                if (headerValue.headerHeight < (headerValue.ItemHeight- (2 * leftpadding)) && width < (headerValue.ItemWidth-(2 * toppadding))){
                    generateHeader = true;
                }					
            }
            else {
                if (headerValue.headerHeight < headerValue.ItemHeight){
                    generateHeader = true;
                }
            }
            if(generateHeader) {
                if (headerValue.headerTemplate == null) {
                    var headerItem = $('<div style="display:block;position:absolute;pointer-events: stroke;overflow: hidden;"><label class="e-treemap-header">' + headerValue.header + '</label></div>');
                    $(headerItem).css({
                        "left": headerValue.headerLeftPosition+leftpadding,
                        "top": headerValue.headerTopPosition+toppadding,
                        "width": headerValue.headerWidth - level.groupPadding,
                        "height": headerValue.headerHeight,
                        "margin-left": level.groupPadding                    
                    });
                    this._templateDiv.append(headerItem);
                }
                else {
                    var item_templateDiv = $("<div style='display:block;position:absolute;pointer-events: none;overflow: hidden;'></div>");
                    this._templateDiv.append(item_templateDiv);
                    $(item_templateDiv).css({
                        "left": headerValue.headerLeftPosition,
                        "width": headerValue.headerWidth, "top": headerValue.headerTopPosition,
                        "height": headerValue.headerHeight
                    });
                    var arguments = { header: headerValue.header, data: headerValue.Data };
                    var htmlString = $("#" + headerValue.headerTemplate).render(arguments);
                    $(item_templateDiv).html(htmlString);
                }
            }
        },

        getDrillDownHeader:function(header)
        {
            var headerString = "";
            if(header.length==1)
            {
                return header[0];
            }
            else{
                for (var index = 0; index < header.length; index++)
                {
                    headerString += header[index];
                    if(index!=header.length-1)
                    {
                        headerString += ".";
                    }
                }
            }
            return headerString;
            
        },
	
        _drillDownHeader: function (enableNavigation) {

            var labelValue = [];
            var label = [];            
            for (i = 0; i < this._drilldownHeaders.length;i++)
            {                
                labelValue += "<label class='e-drilldownlabel' style='margin-left:-8px'>" + this._drilldownHeaders[i] + "</label>";
                if (this._drilldownHeaders.length > 1 && i < this._drilldownHeaders.length-1)
                {                    
                    labelValue = labelValue + "."                    
                }                  
            }                       

            var item_templateDiv = $("<div class='e-drilldownHeader' style='display:block;overflow: hidden;'><label class='e-drilldownlabel'  style='position:absolute;height:30px;position:absolute;margin-top: 5px;top: 0px;'>" + labelValue + "</label></div>");
            if (enableNavigation)
                item_templateDiv = $("<div class='e-drilldownHeader' style='display:block;overflow: hidden;'><svg class='e-drilldownarrow' style='width:15px;height:15px;margin-top: 10px;'><polyline points='8,0 8,10 0,5 8,0'/></svg><label class='e-drilldownlabel'  style='left:17px;position:absolute;height:30px;position:absolute;margin-top: 5px;top: 0px;'>" + labelValue + "</label></div>");
            
            $(item_templateDiv).css({
                "left": "0px",
                "width": this._width+"px","top": 0,
                "height": "30px","cursor":"pointer"
            });    
            if(this.drillDownHeaderColor()!=null)
            {
                $(item_templateDiv).css("background-color",this.drillDownHeaderColor());
            }			
            this._drillHeaderDiv.append(item_templateDiv);
            $(item_templateDiv).mousedown({ treemap: this }, this._drilldownfunction);
            $(".e-drilldownlabel").mousedown({ treemap: this }, this._drilldownLabel);
            
        },


        _drilldownLabel: function (event) {
            var label = this.innerHTML;
            var treeMap = event.data.treemap;
            var pos;
            if (treeMap._drilldownHeaders[0] == label)
            {                
                treeMap._drilldownItems = [];
                treeMap.refresh();
            }
            else
            {                
                var header = "";
                
                treeMap._drillHeaderDiv[0].innerHTML = "";
                treeMap._backgroundTile[0].innerHTML = "";
                var drillHeader = "";               

                for (i = 0; i < treeMap._drilldownHeaders.length; i++) {
                    if (label == treeMap._drilldownHeaders[i]) {                        
                        pos = i + 1;                        
                        treeMap._drilldownHeaders.splice(i+1);
                    }
                }
                
                treeMap._labelTemplateElements = [];
                treeMap._drillDownHeader(true);
                               
                treeMap._drilldownItem = treeMap._drilldownItems[pos - 2];
                treeMap._drilldownItems.splice(pos - 1);
                
                treeMap._templateDiv[0].innerHTML = "";
                treeMap._groupdrillDownLevels(treeMap._drilldownItem, treeMap._drilldownItems.length);                                
                return false;
            }
        },


		
        _drilldownfunction: function (event) {
            var treeMap = event.data.treemap;                       
            treeMap._trigger("drillStarted", event);
            if (treeMap._drilldownItems.length > 0) {                
                if (treeMap._drilldownItems.length == 1) {
                    treeMap._drilldownItems = [];
                    treeMap.refresh();
                }
                else {
                    var header = "";
                    for (var i = 0; i < treeMap._drillHeaderDiv[0].children[0].children.length; i++) {
                        if (treeMap._drillHeaderDiv[0].children[0].children[i].className == "e-drilldownlabel")
                            header = treeMap._drillHeaderDiv[0].children[0].children[i].innerHTML;
                    }
                    var headers = header.split(".");
                    treeMap._drillHeaderDiv[0].innerHTML = "";
                    treeMap._backgroundTile[0].innerHTML = "";
                    var drillHeader = "";
                    if (treeMap._drilldownHeaders.length > 1)
                    {
                        treeMap._drilldownHeaders.pop(treeMap._drilldownHeaders.length - 1);
                    }                    
                    treeMap._labelTemplateElements = [];
                    treeMap._drillDownHeader(true);
                    treeMap._drilldownItem = treeMap._drilldownItems[treeMap._drilldownItems.length - 2];
                    treeMap._drilldownItems.pop(treeMap._drilldownItems[treeMap._drilldownItems.length - 1]);
                    treeMap._templateDiv[0].innerHTML = "";
                    treeMap._groupdrillDownLevels(treeMap._drilldownItem, treeMap._drilldownItems.length);
                }
                treeMap._trigger("drillDownItemSelected", { level: treeMap._drilldownItems.length, header: treeMap.getDrillDownHeader(treeMap._drilldownHeaders), prevLevel: treeMap._drilldownItems.length - 1 });
            }            
        },
       
      _sizeCalculation: function () {

            if (this.model.rangeColorMapping != null) {
                var colormapping = this.model.rangeColorMapping;
                var width = 0;
                var height = 0;
                var iconwidth = this.iconWidth() + 5;
                var totalHeight = this.iconHeight() + 15;
                var totalWidth = 0;
                var columnCount = this.model.legendSettings.columnCount;


                var legendSettings = this.model.legendSettings;
                if (this.model.legendSettings.mode != ej.datavisualization.TreeMap.LegendMode.Interactive) {
                if ((this.legendheight() == 0 && this.legendwidth() == 0) && (columnCount == 0)) {
                    for (var i = 0; i < colormapping.length; i++) {
                        if((colormapping[i].legendLabel) == undefined){
                            colormapping[i].legendLabel =  (colormapping[i].from) + "-" + (colormapping[i].to) ;
                        }
                        var labelwidth = this._calcWidth(colormapping[i].legendLabel);
                        var legendWidth = iconwidth + labelwidth + 10;
                        if (this.legendheight() == 0 && this.legendwidth() == 0) {
                            if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                                if (this._width < (width + legendWidth)) {
                                    totalHeight = (this.iconHeight() + 15) + totalHeight;
                                    totalWidth = Math.max(totalWidth, width);
                                    width = legendWidth;
                                }
                                else {
                                    width += legendWidth;
                                }
                            }
                            else {

                                    if (this._height < (height + this.iconHeight() + 15)) {
                                    totalWidth += width;
                                    totalHeight = Math.max(totalHeight, height);
                                    width = legendWidth;
                                    height = 0;
                                }
                                else {
                                    height += (this.iconHeight() + 18);
                                    width = Math.max(legendWidth, width);
                                }
                            }
                        }
                        else {

                            xPos += _legendlabelwidth + 18;
                            if (xPos + 150 >= this.legendwidth()) {
                                xPos = 10;
                                yPos += 40;
                            }
                        }
                    }

                    if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                        totalWidth = Math.max(totalWidth, width);
                        totalHeight += 5;
                    }
                    else {
                        totalWidth += (width + 15);
                        totalHeight = Math.max(totalHeight, height);
                    }
                }
               else if (columnCount == 0) {

                        for (var i = 0; i < colormapping.length; i++) {
                            if ((colormapping[i].legendLabel) == undefined) {
                                colormapping[i].legendLabel = (colormapping[i].from) + "-" + (colormapping[i].to);
                            }
                        }
                    if ((this.legendheight().toString().indexOf("%")) != -1) {

                        totalHeight = (this._height / 100) * parseInt(this.legendheight().replace('%', ''))
                    }
                    else {

                        totalHeight = this.legendheight();
                    }
                    if ((this.legendwidth().toString().indexOf("%")) != -1) {

                        totalWidth = (this._width / 100) * parseInt(this.legendwidth().replace('%', ''))
                    }
                    else {

                        totalWidth = this.legendwidth();
                    }

                }

                if (columnCount != 0) {

                    for (var i = 0; i < colormapping.length; i++) {
                            if ((colormapping[i].legendLabel) == undefined) {
                                colormapping[i].legendLabel = (colormapping[i].from) + "-" + (colormapping[i].to) ;
                            }
                             var labelwidth = this._calcWidth(colormapping[i].legendLabel);
                            
                            legendWidth = iconwidth + labelwidth + 10;

                        if (i % columnCount != 0) {
                            width += legendWidth;
                            if (i == columnCount - 1) {
                                totalWidth = Math.max(totalWidth, width);
                            }

                        }
                        else {
                            if (i != 0)
                                totalHeight = (this.iconHeight() + 15) + totalHeight;
                            totalWidth = Math.max(totalWidth, width);
                            width = legendWidth;
                        }
                    }
                }
                }
                else {
                    totalHeight = (legendSettings.height != 0 ? legendSettings.height : 30) + 18;
                    if (legendSettings.title != null && legendSettings.title != "") {
                        totalHeight += 25;
                    }
                    totalWidth = (legendSettings.width != 0 ? legendSettings.width : 100) + 20 + (legendSettings.leftLabel.length * 10) + (legendSettings.rightLabel.length * 10);

                }
                this._legendSize = { height: totalHeight, width: totalWidth };
                if (this.model.legendSettings != null) {

                    if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                        this._height = this._height - parseFloat(totalHeight);
                        this.model.legendSettings.tempWidth = width;
                    }

                    else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top) {
                        this._height = this._height - parseFloat(totalHeight);

                        this._margintop = parseFloat(totalHeight);
                    }
                    else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Left) {
                        this._width = this._width - totalWidth;
                        this._marginleft = totalWidth;

                    }
                    else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) {
                        this._height = this._height;
                        this._width = this._width - totalWidth;
                    }
                }
            }
        },

        _getDockPositionPoint:function(size,bounds,item)
        {
            var position = item.labelPosition;
            var x = 2, y = 2;
            if (item.groupPadding != undefined && item.groupBorderThickness != undefined) {
                x = item.groupPadding + item.groupBorderThickness + x;
                y = item.groupPadding + item.groupBorderThickness + y;
            }
            if (item.showHeader && !this.enableDrillDown())
                y = y + item.headerHeight;
            if (position == ej.datavisualization.TreeMap.Position.TopCenter) {
                x = (size.width/2)-(bounds.width/2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.TopRight) {
                x = size.width - bounds.width - x;
            }
            else if (position == ej.datavisualization.TreeMap.Position.CenterLeft) {
                y = (size.height / 2) - (bounds.height / 2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.Center || position==null) {
                x = (size.width / 2) - (bounds.width / 2);
                y = (size.height / 2) - (bounds.height / 2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.CenterRight) {
                x = size.width - bounds.width -x;
                y = (size.height / 2) - (bounds.height / 2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.BottomLeft) {
                y = size.height - bounds.height - y;
            }
            else if (position == ej.datavisualization.TreeMap.Position.BottomCenter) {
                x = (size.width / 2) - (bounds.width / 2);
                y = size.height - bounds.height-y;
            }
            else if (position == ej.datavisualization.TreeMap.Position.BottomRight) {
                x = size.width - bounds.width-x;
                y = size.height - bounds.height-y;
            }
            return {x:size.left+x,y:size.top+y};
        },
       
        _generateTreeMapItems: function (Items, fill, path) {
            this._rootTreeMapItems= Items;
            this._getTreeMapItemTemplate(Items);
            if (Items != null) {
                if (!this._isLevelColors){
                    this._setColorMappings(Items, this.model);
                }
                for (var i = 0; i < Items.length; i++) {
                    var item = Items[i];                    
                                      
                    var rect = item.rectangle;
                    if (item.Data != null && item.Data.length>0) {
                        item.Data = item.Data[0];
                    }
                    item.Rectangle = rect;
					 $(rect).css({
                        "box-sizing": "border-box",
                        "-moz-box-sizing": "border-box",
                        "-webkit-box-sizing": "border-box",
                    });
                    if(item.backgroundColor!=null)
                        $(rect).css("background-color",item.backgroundColor);
                    else
                        $(rect).css("background-color","#E5E5E5");
                    if(item.backgroundOpacity !=null)
                    {
                        $(rect).css("opacity",item.backgroundOpacity);
                    }
                    this._wireEventForTreeMapItem(item, rect);
                 
                }
                if (this.model.rangeColorMapping.length == 0 && this.uniColor() == null && this._color() != "") {
                    this._setDesaturationColor(Items, this.model.desaturationColorMapping);
                }
                else if (this.model.PaletteColorMapping != null) {
                    this._setPaletteColor(Items, this.model.PaletteColorMapping);
                }
            }
        },

        browserInfo: function () {
            var browser = {}, clientInfo = [],
            browserClients = {
                webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
                opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
            };
            for (var client in browserClients) {
                if (browserClients.hasOwnProperty(client)) {
                    clientInfo = navigator.userAgent.match(browserClients[client]);
                    if (clientInfo) {
                        browser.name = clientInfo[1].toLowerCase();
                        browser.version = clientInfo[2];
                        if (!!navigator.userAgent.match(/Trident\/7\./)) {
                            browser.name = "msie";
                        }
                        break;
                    }
                }
            }
            browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
            browser.pointerEnabled = window.navigator.pointerEnabled;
            return browser;
        },

        isTouch: function (evt) {
            
                var event = evt.originalEvent ? evt.originalEvent : evt;
            if ((event.pointerType == "touch") || (event.pointerType == 2) || (event.type.indexOf("touch") > -1))
                return true;
            return false;
        },

        _wireEventForTreeMapItem: function (item, rect) {
            browserInfo = this.model.browserInfo,
            isPointer = browserInfo.isMSPointerEnabled,
            isIE11Pointer = browserInfo.pointerEnabled,
            touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
            touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup",
            touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : "touchmove mousemove",
            touchEnterEvent = isPointer ? (isIE11Pointer ? "pointerenter" : "MSPointerEnter") : "touchenter mouseenter",
            touchCancelEvent = isPointer ? (isIE11Pointer ? "pointerleave" : "MSPointerOut") : "touchleave mouseleave";        

            this._on($(rect), touchMoveEvent, { Param1: item, Param2: this }, this._rectMouseMove);
            this._on($(rect), touchCancelEvent, { treeMap: this }, this._rectMouseLeave);

            this._on($(rect), touchStopEvent, { Param1: item, Param2: this }, this._rectMouseUp);
            this._on($(rect), touchStartEvent, { treeMap: this }, this._rectMouseDown);
            this._on($(rect), touchEnterEvent, { Param1: item, Param2: this }, this._rectMouseEnter);

           
        },

        _getTreeMapItemTemplate: function (Items) {            
            
            var rects ="";
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                if (this.model.leafItemSettings.itemTemplate == null)
                    rects += '<div style="border-style:solid;position:absolute;left:' + item.LeftPosition + 'px;top:' + item.TopPosition + 'px;height:' + item.ItemHeight + 'px;width:' + item.ItemWidth + 'px;border-width:' + this.borderThickness() + 'px;border-color:' + this.borderBrush() + '"></div>';
                else {
                    var OriginalNode = $("#" + this.model.leafItemSettings.itemTemplate);
                    if (item.Data != null && item.Data.length == 1) {
                        item.Data = item.Data[0];
                    }
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = "<div style='overflow:hidden;position:absolute;left:" + item.LeftPosition + "px;top:" + item.TopPosition + "px;height:" + item.ItemHeight + "px;width:" + item.ItemWidth + "px'>" + tmpl2.render(item) + "</div>";
                    rects += htmlString;
                }
            }
            if (this._svgDocument != null && this.enableDrillDown()) {
                this._svgDocument[0].innerHTML = "";
            }
            this._svgDocument = $('<div style= "overflow:hidden;' +
                              'z-index:1;"' +
                             'id="svgDocument">' + rects + '</div>');
            this._backgroundTile.appendTo(this.element);
            this._svgDocument.appendTo(this.element);           
            this._svgDocument.css({
                "height": this._height + "px", "width": this._width + "px", "margin-top": this._margintop + "px", "margin-left": this._marginleft, "overflow": "hidden",
                "z-index": 1, "position": "absolute", "left": "0", "top": "0"
            });
            $(this._backgroundTile).css({
                "height": this._height + "px", "width": this._width + "px", "margin-top": this._margintop + "px", "margin-left": this._marginleft, "overflow": "hidden",
                "z-index": 0, "position": "absolute", "left": "0", "top": "0"
            });
            this._templateDiv.appendTo(this.element);
            /* Below event hook added for IE10 tooltip issue*/
            $(this._templateDiv).mousemove({ Param2: this }, this._rectMouseMove);
            var browserInfo = this.model.browserInfo,
            isPointer = browserInfo.isMSPointerEnabled,
            isIE11Pointer = browserInfo.pointerEnabled,
            touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : "touchstart mousedown",
             touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : "touchend mouseup";
            this._on($(this.element), touchStartEvent, { treemap: this, Param1: Items }, this.rectClick);
            this._on($(this.element), touchStopEvent, { treemap: this, Param1: Items }, this._docClickFunction);            
            this._on($(this.element), ej.eventType.mouseDown, { treemap: this }, this.dragDown);
            this._on($(this.element), ej.eventType.mouseMove, { treemap: this }, this.dragMove);
            this._on($(this.element), ej.eventType.mouseUp, { treemap: this }, this.dragUp);

            if (this.enableDrillDown()) {
                this._drillHeaderDiv.appendTo(this.element);
                this._drillHoverDiv.appendTo(this.element);
            }
            for (var i = 0; i<Items.length; i++) {
                Items[i].rectangle = this._svgDocument[0].childNodes[i];
            }
        },	       

        selectItem: function (obj) {
            if (this._rootTreeMapItems != null) {
                for (var i = 0; i < this._rootTreeMapItems.length; i++) {
                    var item = this._rootTreeMapItems[i];
                    var rect = item.rectangle;
                    if (item.Data != null && item.Data.length > 0) {
                        item.Data = item.Data[0];
                    }
                   
                    if (obj == item.Data) {
                        if (this.highlightOnSelection()) {

                            $(rect).css({ "border-width": this.highlightBorderThickness(), "border-color": this.highlightBorderBrush() });
                            if (this._browser != "msie") {
                                var lastItem = this._svgDocument[0].children[this._svgDocument[0].children.length - 1];
                                this._svgDocument[0].insertBefore(rect, lastItem);
                            }
							this._prevSelectedItems.push(rect);
							this._trigger("treeMapItemSelected", { selectedItems: obj, originalEvent: event });                              
                        }
					}
                    
                }
            }

        },
      
        _selectTreemapItem: function (obj) {

            if (this._rootTreeMapItems != null) {
                for (var i = 0; i < this._rootTreeMapItems.length; i++) {
                    var item = this._rootTreeMapItems[i];
                    var rect = item.rectangle;
                    if (item.Data != null && item.Data.length > 0) {
                        item.Data = item.Data[0];
                    }
                    if (obj == item.Data && !this._contains(this.selectedItems, obj)) {

                        if (this.highlightOnSelection()) {
                            $(rect).css({ "border-width": this.highlightBorderThickness(), "border-color": this.highlightBorderBrush() });
                            if (this._browser != "msie") {
                                var lastItem = this._svgDocument[0].children[this._svgDocument[0].children.length - 1];
                                this._svgDocument[0].insertBefore(rect, lastItem);
                            }
                            this._prevSelectedItems = [];
                            this.selectedItems = [];
                            this._prevSelectedItems.push(rect);
                            this.selectedItems.push(obj);
                        }
                    }

                }
            }
        },
        
        _selectItemResize: function () {

            for (var i = 0; i < this._rootTreeMapItems.length; i++) {
                var item = this._rootTreeMapItems[i];
                var rect = item.rectangle;
                if (item.Data != null && item.Data.length > 0) {
                    item.Data = item.Data[0];
                }
                if (this._contains(this.selectedItems, item.Data)) {
                    this.selectItem(item.Data);
                }
                
            }
        },
		    _updateLegendRange: function (value, treemapitem) {
            var colormapping = this.model.rangeColorMapping;
            for (var index = 0; index < colormapping.length; index++) {
                var gradientCollection = null;
                var mapping = colormapping[index];
                if (this.model.enableGradient) {
                    gradientCollection = this._generateGradientCollection(mapping.gradientColors);
                }
                if (value >= mapping.from && value <= mapping.to) {
                    var minValue = index;
                    if (index != 0)
                        minValue = index * 10;
                    var minRange = mapping.from;
                    var maxRange = mapping.from + ((mapping.to - mapping.from) / 10);
                    for (var i = minValue; i < minValue + 10; i++) {
                        if (value >= minRange && value <= maxRange) {
                            var obj = {};
                            var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                            if (this._legendRects[i] != undefined)
                                obj = this._legendRects[i];
                            if (this.model.enableGradient) {
                                if (index != 0) {
                                    treemapitem.backgroundColor = gradientCollection[i - index * 10];
                                } else {
                                    treemapitem.backgroundColor = gradientCollection[i];
                                }
                            } else
                                treemapitem.backgroundColor = mapping.color;
                            treemapitem.backgroundOpacity = shapeOpacity;
                            obj["color"] = treemapitem.backgroundColor;
                            return obj;
                        }
                        minRange = maxRange;
                        maxRange = maxRange + ((mapping.to - mapping.from) / 10);
                    }
                }
            }
        },
		
        selectItemByPath: function (path, value, seperator) {
            var propertyPath = path.split(seperator);
            var propertyValue = value.split(seperator);
            var path, value;
            var treemapItems = this._rootTreeMapItems;
            for (var i = 0; i < propertyPath.length; i++)
            {
                var path = propertyPath[i];
                var value = propertyValue[i];
                var itemsCollection = [];
                for (var k = 0; k < treemapItems.length; k++) {
                    var item = treemapItems[k];
                    var dataObj = item.Data;
                    var objValue;
                    for (var j = 0; j < propertyPath.length; j++) {
                        objValue = this._reflection(dataObj, path);
                    }

                    if (value == objValue) {
                        itemsCollection.push(item);
                    }
                }
                treemapItems = itemsCollection;
            }
            for (var i = 0; i < treemapItems.length; i++)
            {
                var item = treemapItems[i];
                this.selectItem(item.Data);
            }
        },
          _getColorRatio: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var colorCode = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return colorCode;
        },
        _contains: function (array, actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length] === actualobj) {
                        return true;
                    }
                }
            }
            return false;
        },

        _rectMouseEnter: function(event) {
		    var treemap = event.data.Param2;
            var legendSettings = treemap.model.legendSettings;
            var tooltipObject = event.data.Param1.Data;
            if (tooltipObject != null) {
                var element = event.data.Param2._toolTipElement;
                var template = event.data.Param2.model.tooltipTemplate;
                if (element != null && template != null) {
                    if(!treemap.isTouch(event))
                       $(element).css({ "left": event.pageX+10, "top": event.pageY+10, "display": "block" });
					var locale = this.model.locale;
					var obj = $.extend({}, tooltipObject);
					if (locale && this.model.enableGroupSeparator){
						for (var key in obj)
							obj[key] = isNaN(parseFloat(obj[key])) ? obj[key] : parseFloat(obj[key]).toLocaleString(locale);
					}	
                    var htmlString = $("#" + template).render(obj);
                    $(element).html(htmlString);
                    var height= element[0]!=null ? element[0].clientHeight : element.clientHeight;
                    var width= element[0]!=null ? element[0].clientWidth : element.clientWidth;
                    event.data.Param2._tooltipSize = { height: height, width: width };
                    
                }
            }
			
			 if (legendSettings != null && legendSettings.mode == ej.datavisualization.TreeMap.LegendMode.Interactive && treemap.model.rangeColorMapping != null
                   && (treemap.showLegend() == undefined || treemap.showLegend())) {
                for (var i = 0; i < treemap._rootTreeMapItems.length; i++) {
                    var item = treemap._rootTreeMapItems[i].Rectangle;

                    if (item == event.target) {
                        var mappings = null;
                        if (treemap.model.rangeColorMapping != undefined) {
                            mappings = treemap.model.rangeColorMapping;
                        }
                        var _legendwidth = legendSettings.width;
                        if (legendSettings.width == undefined)
                            _legendwidth = 150;

                        var rectwidth = (_legendwidth / mappings.length) / 10;
                        if (treemap._rootTreeMapItems[i].legendrect != null) {
                            var rectPosition = treemap._rootTreeMapItems[i].legendrect.marginLeft;
                            var position = rectPosition + Math.ceil(rectwidth) - treemap._interactiveArrow.width() / 2;
                            $(treemap._interactiveArrow).css({ "margin-left": position, "display": "block" });
                            break;
                        }
                    }
                }
            }
        },

        displayTooltip: function (event) {
            var treeMap = event.data.Param2,
                element = treeMap._toolTipElement,
                pointer = treeMap.mousePosition(event),
                canExecute = true;           
            if (treeMap.model.trackerElement) {
                clearTimeout(treeMap.model.timer);
                $(".tooltipelement" + this._id).finish();              
            }
            if (event.data.Param1 == undefined) {
                if (element != null) {
                    var tooltipObject;
                    for (var i = 0; i < treeMap._rootTreeMapItems.length; i++) {
                        var item = treeMap._rootTreeMapItems[i];
                        if (item.rectangle.offsetLeft < pointer.offsetX && item.rectangle.offsetLeft + item.rectangle.offsetWidth > pointer.offsetX
                            && item.rectangle.offsetTop < pointer.offsetY && item.rectangle.offsetTop + item.rectangle.offsetHeight > pointer.offsetY) {
                            tooltipObject = item.Data;
                            break;
                        }
                    }
                    if (tooltipObject != null) {
                        var template = event.data.Param2.model.tooltipTemplate;
                        if (element != null && template != null) {
                            $(element).css({ "left": pointer.pageX + 10, "top": pointer.pageY + 10, "display": "block" });
                            var htmlString = $("#" + template).render(tooltipObject);
                            $(element).html(htmlString);
                            var height = element[0] != null ? element[0].clientHeight : element.clientHeight;
                            var width = element[0] != null ? element[0].clientWidth : element.clientWidth;
                            treeMap._tooltipSize = { height: height, width: width };

                        }
                    }
                    else {
                        if (element != null) {
                            canExecute = false;
                            $(element).css("display", "none");
                        }
                    }
                }
            }

            if (canExecute) {
                var tooltipSize = treeMap._tooltipSize;
                if (tooltipSize.width + pointer.pageX >= treeMap._width) {
                    pointer.pageX -= tooltipSize.width;
                    if (pointer.pageX < 0) {
                        pointer.pageX = 10;
                    }
                }
                if (tooltipSize.height + pointer.pageY >= treeMap._height) {
                    pointer.pageY -= tooltipSize.height;
                    if (pointer.pageY < 0) {
                        pointer.pageY = 10;
                    }
                }
                if (treeMap.enableDrillDown() && treeMap._browser != 'msie') {
                    for (var i = 0; i < treeMap._drillHoverDiv[0].children.length; i++) {
                        var child = treeMap._drillHoverDiv[0].children[i];
                        if (child.className == event.data.Param1.ParentHeader) {
                            $(child).css({ "display": "block" });
                        }
                    }
                }
                if (element != null) {
                    $(element).css({ "left": pointer.pageX + 10, "top": pointer.pageY + 10, "display": "block" });
                }
            }
        },

       

        _rectMouseUp: function (event) {
            
            if (this.isTouch(event))
                this.displayTooltip(event);          
        },
       
        _rectMouseMove: function (event) {
          
            /* Below If condition code was added for IE10 Tooltip issue*/
            if (!event.data.Param2.isTouch(event))
                event.data.Param2.displayTooltip(event);
        },

        _rectMouseLeave: function (event) {            
            var element = event.data.treeMap._toolTipElement;
            if (element != null && !event.data.treeMap.isTouch(event)) {
                $(element).css("display", "none");
            }
        },
		
     
       
        _setColorMappings: function(Items, colorMappings,groupmapping){
		    var obj = {};
            var proxyControl = this;
            var legendSettings = this.model.legendSettings;
            for (var i = 0; i < Items.length; i++) {
                if (!groupmapping) {
                    if (Items[i].Data instanceof Array) {
                       
                            var shapeValue = proxyControl._reflection(Items[i].Data[0], this.colorValuePath());
                    }
                    else {
                        var shapeValue = proxyControl._reflection(Items[i].Data, this.colorValuePath());
                    }
                }
                if ((colorMappings.rangeColorMapping != null && colorMappings.rangeColorMapping.length > 0) || (colorMappings.uniColorMapping != null && (colorMappings.uniColorMapping.color != null || this.uniColor() != null))) {

                    if ((legendSettings.mode == ej.datavisualization.TreeMap.LegendMode.Interactive) || this.model.enableGradient) {
                        Items[i].legendrect = this._updateLegendRange(shapeValue, Items[i]);
                    }
                    else {
                        var ColorValue = Items[i]._generateColorMappings(colorMappings, this);
                        Items[i].backgroundColor = ColorValue;
                    }
                }
                else if (colorMappings.desaturationColorMapping != null && (colorMappings.desaturationColorMapping.color != "" || this._color() != "")) {
                    this._setDesaturationColor(Items, colorMappings.desaturationColorMapping);
                }
                else if (colorMappings.paletteColorMapping != null) {
                    this._setPaletteColor(Items, colorMappings.paletteColorMapping);
                }
            }
        },
     
        _setDesaturationColor: function (Items, desaturationColorMapping) {
            Items = Items.sort(this._orderBycolorWeight);

            var from = (typeof desaturationColorMapping.from == "number") ? desaturationColorMapping.from : this._from();
            var to = (typeof desaturationColorMapping.to == "number") ? desaturationColorMapping.to : this._to();
            var rangeMin = (typeof desaturationColorMapping.rangeMinimum == "number") ? desaturationColorMapping.rangeMinimum : this._rangeMinimum();
            var rangeMax = (typeof desaturationColorMapping.rangeMaximum == "number") ? desaturationColorMapping.rangeMaximum : this._rangeMaximum();
          
            var values = [];
            for (var j = 0; j < Items.length; j++)
            {
                var item = Items[j].Data[0];
                var colorWeightValue = this._reflection(item, this.colorValuePath());
                if (colorWeightValue!=null && (typeof colorWeightValue != "number")) {

                    colorWeightValue = Number(colorWeightValue.replace(/[^0-9\.]+/g, ""));
                }               
                values.push(colorWeightValue);                
            }

            if (this._rangeMinimum() == 0)
            {
                rangeMin = Math.min.apply(Math, values);
            }            
            
            if (this._rangeMaximum() == 0)
            {
                rangeMax = Math.max.apply(Math, values);
            }            

            if (from < 0 || from > 1)
                from = 1;
            if (to > 1 || to < 0)
                to = 0;

            var interval = (from - to) / Items.length;
            for (var i = 0; i < Items.length; i++) {
                Items[i].backgroundColor = (typeof desaturationColorMapping.color == "string") ? desaturationColorMapping.color : this._color();              
                if (Items[i].colorWeight >= rangeMin && Items[i].colorWeight <= rangeMax) {
                    Items[i].backgroundOpacity= from;
                    from = from - interval;
                }
            }
        },
        
		
		  _generateGradientCollection: function (gradientColors) {
            var gradientCollection = [];
            var startRangeColor = gradientColors[0];
            var endRangeColor = gradientColors[gradientColors.length - 1];
            var startR = this._hexToR(startRangeColor);
            var startG = this._hexToG(startRangeColor);
            var startB = this._hexToB(startRangeColor);
            var endR = this._hexToR(endRangeColor);
            var endG = this._hexToG(endRangeColor);
            var endB = this._hexToB(endRangeColor);
            var range = 10;
            var rRange = this._getRangeColorValues(range, startR, endR);
            var gRange = this._getRangeColorValues(range, startG, endG);
            var bRange = this._getRangeColorValues(range, startB, endB);
            for (var i = 0; i < range; i++) {
                gradientCollection.push(this._rgbToHex(rRange[i], gRange[i], bRange[i]));
            }
            return gradientCollection;
        },


        _hexToR: function (h) {
            return parseInt((this._cropHex(h)).substring(0, 2), 16);
        },

        _hexToG: function (h) {
            return parseInt((this._cropHex(h)).substring(2, 4), 16);
        },

        _hexToB: function (h) {
            return parseInt((this._cropHex(h)).substring(4, 6), 16);
        },
        _cropHex: function (h) {
            return (h.charAt(0) == "#") ? h.substring(1, 7) : h;
        },

        _rgbToHex: function (R, G, B) {
            return "#" + this._toHex(R) + this._toHex(G) + this._toHex(B);
        },

        _toHex: function (n) {
            var charstr = "0123456789ABCDEF";
            n = parseInt(n, 10);
            if (isNaN(n)) return "00";
            n = Math.max(0, Math.min(n, 255));
            return charstr.charAt((n - n % 16) / 16)
                + charstr.charAt(n % 16);
        },
        _getRangeColorValues: function (range, start, end) {
            var rangeColorValues = [];
            rangeColorValues.push(start);
            if (start > end) {
                var rangeValue = (start - end) / (range - 1);
                for (var i = range; i > 2; i--) {
                    start = start - rangeValue;
                    rangeColorValues.push(start);
                }
            }
            else {
                var rangeValue = (end - start) / (range - 1);
                for (var i = 2; i < range; i++) {
                    start = start + rangeValue;
                    rangeColorValues.push(start + rangeValue);
                }
            }
            rangeColorValues.push(end);
            return rangeColorValues;
        },


        _isSVG: function () {
            if (window.SVGSVGElement)
                return true;
            else
                return false;
        },

        _drawInteractiveLegend: function () {
            var legendSettings = this.model.legendSettings;
            var xPos = 0;
            var yPos = 0;

            //var _isSVG = (window.SVGSVGElement) ? true : false;

            var legenddiv = this._legenddiv;
            var _legendheight = this.model.legendSettings.height != 0 ? this.model.legendSettings.height : 30;
            var _legendwidth = this.model.legendSettings.width != 0 ? this.model.legendSettings.width : 100;
            if (legendSettings.mode == ej.datavisualization.TreeMap.LegendMode.Interactive || this.model.rangeColorMapping != null) {
                var textcon = '';

                if (legendSettings.height == 0)
                    _legendheight = 30;
                if (legendSettings.width == 0)
                    _legendwidth = 100;

                if (legendSettings.leftLabel == null)
                    legendSettings.leftLabel = '';
                if (legendSettings.title != null) {
                    var newxpos = xPos;
                    if (!legendSettings.showLabels)
                        newxpos = xPos + legendSettings.leftLabel.length * 10;

                    var _legendtitlewidth = legendSettings.title.length * 10;
                    var text = titleText = legendSettings.title;;

                    if (_legendtitlewidth > _legendwidth) { 
                      
                      text=  this._trimFunction(text, _legendwidth);
                    }

                    var textcon = this._createLabel(text, newxpos, yPos, 'e-interactivelegend-title');
                    textcon[0].title = titleText;
                    textcon.css({
                        "width": _legendwidth + "px"
                    });
                    if (this._isSVG())
                        textcon.appendTo(legenddiv);
                    else
                        legenddiv.append(textcon);
                    yPos += 25;
                }

                if (legendSettings.showLabels)
                    yPos += 25;

                if (legendSettings.leftLabel != null && !legendSettings.showLabels) {
                    var textcon = this._createLabel(legendSettings.leftLabel, xPos, yPos - 3, 'e-interactivelegend-leftlabel');

                    if (this._isSVG())
                        textcon.appendTo(legenddiv);
                    else

                        legenddiv.append(textcon);
                    xPos = xPos + legendSettings.leftLabel.length * 10;
                }

                var interactiveElement = this._createInteractiveArrow(xPos, yPos + _legendheight);
                interactiveElement.appendTo(legenddiv);
                this._interactiveArrow = interactiveElement;

                var _legendgroup = null;
                if (!this._isSVG() && this.model.enableGradient) {
                    _legendgroup = this._createGroup(false, "legendGroup");
                    _legendgroup.style.left = 0 + 'px';
                    _legendgroup.style.top = 0 + 'px';
                    _legendgroup.style.position = "relative";
                    legenddiv.append(_legendgroup);
                }

                var mappings = this.model.rangeColorMapping;
                for (var key = 0; key < mappings.length; key++) {
                    var colorMapping = mappings[key];
                    if (!colorMapping.hideLegend) {
                        var gradientCollection = [];
                        if (this.model.enableGradient) {
                            gradientCollection = this._generateGradientCollection(colorMapping.gradientColors);
                        }

                        var obj = {};
                        if (this.model.enableGradient) {
                            if (this._isSVG()) {
                                var canvas = $("<canvas/>");
                                var ctx = canvas[0].getContext("2d");
                                var grd = ctx.createLinearGradient(0, 0, 300, 0);

                                var temp = 0;
                                for (var i = 0; i < 10; i++) {
                                    temp = temp + (1 / 10);
                                    if (i == 0) {
                                        grd.addColorStop(0, colorMapping.gradientColors[0]);
                                        grd.addColorStop(temp, gradientCollection[i]);
                                    } else if (i == gradientCollection.length - 1) {
                                        grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                        grd.addColorStop(temp, colorMapping.gradientColors[1]);
                                    } else {
                                        grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                        grd.addColorStop(temp, gradientCollection[i + 1]);
                                    }
                                }
                                ctx.fillStyle = grd;
                                ctx.fillRect(0, 0, 300, 300);

                                canvas.css({
                                    "height": _legendheight + "px",
                                    "width": (_legendwidth / mappings.length) + "px",
                                    "margin-left": xPos + "px",
                                    "margin-top": yPos + "px",
                                    "opacity": "0.9",
                                    "filter": "alpha(opacity=90)", /* For IE8 and earlier */
                                    "position": "absolute"
                                });
                                canvas.appendTo(legenddiv);
                            } else {
                                var legendID = "legend" + key;
                                var legendHtmlString = '<v:rect id=' + legendID + ' display="block" style="position:absolute;top: ' + (yPos - 2) + 'px;left:' + xPos + 'px;width:' + (_legendwidth / mappings.length) + 'px;height:' + _legendheight + 'px;"><v:fill opacity="0.9px" type="gradient" method="linear sigma" angle="270"/><v:stroke opacity="0px"/></v:rect>';
                                _legendgroup.innerHTML = _legendgroup.innerHTML + legendHtmlString;
                                var legendelement = document.getElementById(legendID);
                                legendelement.fillcolor = colorMapping.gradientColors[0];
                                legendelement.fill.color2 = colorMapping.gradientColors[1];
                            }
                        }
                        else {
                            var rect = $("<div/>");
                             rect.css({
                                "height": _legendheight + "px",
                                "width": _legendwidth / this.model.rangeColorMapping.length + "px",
                                "background-color": colorMapping.color,
                                "margin-left": xPos + "px",
                                "margin-top": yPos + "px",
                                "opacity": "0.9",
                                "filter": "alpha(opacity=90)",/* For IE8 and earlier */
                                "position": "absolute"
                            });

                            if (this._isSVG())
                                rect.appendTo(legenddiv);
                            else
                                legenddiv.append(rect);
                        }

                        for (var i = 0; i < 10; i++) {
                            obj = {};
                            obj["marginLeft"] = xPos;
                            this._legendRects.push(obj);
                            xPos = xPos + (_legendwidth / mappings.length) / 10;
                        }

                        if (this.model.legendSettings.showLabels) {
                            var labelxpos = xPos - (_legendwidth / mappings.length);
                            var labelypos = yPos - 25;
                            var startlabel = this._createLabel((colorMapping.from), labelxpos, labelypos, 'e-legend-rangestartlabel');
                            labelxpos = xpos;
                            var endlabel = this._createLabel((colorMapping.to), labelxpos, labelypos);
                            if (colorMapping.legendLabel != undefined)
                                endlabel = this._createLabel((colorMapping.legendLabel), labelxpos - (colorMapping.legendLabel.length * 10) / 2, labelypos, 'e-legend-rangeendlabel');
                            if (this._isSVG()) {
                                if (colorMapping == mappings[0])
                                    startlabel.appendTo(legenddiv);
                                endlabel.appendTo(legenddiv);
                            }
                            else {
                                if (colorMapping == mappings[0])
                                    legenddiv.append(startlabel);
                                legenddiv.append(endlabel);
                            }
                        }
                    }
                }

                if (legendSettings.rightLabel != null && !legendSettings.showLabels) {
                    var textcon = this._createLabel(legendSettings.rightLabel, xPos + 10, yPos - 3, 'e-interactivelegend-rightlabel');
                    if (this._isSVG())
                        textcon.appendTo(legenddiv);
                    else
                        legenddiv.append(textcon);
                    xPos = xPos + legendSettings.rightLabel.length * 10;
                }

                totalwidth = xPos + 10;
                totalheight = yPos + _legendheight + this._interactiveArrow.height();
                this._legendSize = { width: totalwidth, height: totalheight };

                if (legendSettings.dockPosition == 'left')
                    this._marginleft = totalwidth;
                else if (legendSettings.dockPosition == 'top')
                    this._margintop = totalheight;


            }






        },
		   _trimFunction: function (str, width) {
          
       
            var span = $('<div style="width:auto;position:absolute;" class= "e-interactivelegend-title">' + str + '</div>');
             $(document.body).append(span);
             var text = str;
            while (span.width() > width) {

                text = text.slice(0, -2);
                span.text(text + "...");
                
            }
            document.body.removeChild(span[0]);
            return span.text();
                  
        


        },
        _createLabel: function (content, xpos, ypos, className) {
            var label = $("<div class=" + className + "></div>");
            label[0].innerHTML = content;
            label.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute"
            });
            return label;
        },
        _createInteractiveArrow: function (xpos, ypos) {

            var interactiveElement = $("<div class='e-icon1 e-interactiveArrow'></div>");
            interactiveElement[0].innerHTML = "&#9650";

            interactiveElement.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute",

            });
            return interactiveElement;
        },

        _createGroup: function (isRoot, name) {

            var vmlGroup;
            vmlGroup = document.createElement('<v:group class=' + name + '>');
            vmlGroup.style.width = this._width + 'px';
            vmlGroup.style.height = this._height + 'px';
            vmlGroup.coordorigin = "0 0";
            vmlGroup.coordsize = this._width + ' ' + this._height;
            if (isRoot) {
                this._rootgroup = vmlGroup;
                vmlGroup.style.left = '20px';
                vmlGroup.style.top = '20px';
            }
            return vmlGroup;
        },
		
        _setPaletteColor: function (Items, PaletteColors) {
            Items = Items.sort(this._orderBycolorWeight);
            for (var i = 0; i < Items.length; i++) {
                if (PaletteColors.colors != null && PaletteColors.colors.length > 0) {
                    Items[i].backgroundColor= PaletteColors.colors[i % (PaletteColors.colors.length)];
                } 
            }
        },

        _orderBycolorWeight: function (a, b) {
            if (a.colorWeight == b.colorWeight) {
                return 0;
            } else if (a.colorWeight < b.colorWeight) {
                return 1;
            }
            return -1;
        },
       
        _orderByAreaWight: function (a, b) {
            if (a.AreaByWeight == b.AreaByWeight) {
                return 0;
            } else if (a.AreaByWeight < b.AreaByWeight) {
                return 1;
            }
            return -1;
        },

        _calculateSliceAndDiceItemSize: function (Items, xPos, yPos, width, height, gap, headerHeight, isHorizontal,legendHeight) {

            if (gap == "") { gap = 0;}
            var Gap = gap;
            var totalWeight = this._getTotalWeight(Items);
            if (isHorizontal == undefined) {
                isHorizontal = width > height ? true : false;
            }
            var AvailableSize = { "Width": width, "Height": height-legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var parentArea = AvailableSize.Height * AvailableSize.Width;
            var itemsCount = Items.length;
            if (isHorizontal) {
                var parentHeight = AvailableSize.Height;
                var allottedWidth = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childWidth = childarea / parentHeight;
                    gap = (childWidth > Gap) ? Gap : 0;

                    if (allottedWidth <= AvailableSize.Width) {
                        Items[i].ItemWidth = (i != itemsCount - 1) ? childWidth - parseFloat(gap) : childWidth;
                        Items[i].ItemHeight = parentHeight;
                        Items[i].LeftPosition = allottedWidth + xPos;
                        Items[i].TopPosition = yPos + headerHeight;
                        allottedWidth += childWidth;
                    }
                }
            }
            else {
                var parentWidth = AvailableSize.Width;
                var allottedHeight = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childHeight = childarea / parentWidth;
                    gap = (childHeight > Gap) ? Gap : 0;
                    if (allottedHeight <= AvailableSize.Height) {
                        Items[i].ItemWidth = parentWidth;
                        Items[i].ItemHeight = (i != itemsCount - 1) ? childHeight - parseFloat(gap) : childHeight;
                        Items[i].LeftPosition = xPos;
                        Items[i].TopPosition = allottedHeight + yPos + headerSize;
                        allottedHeight += childHeight;
                    }
                }
            }
        },

        _calculateSquarifiedItemSize: function (Items, xPos, yPos, width, height, Gap, headerHeight, legendHeight) {

            if (Gap == "") { Gap = 0; }            
            var totalweight = this._getTotalWeight(Items);
            var AvailableSize = { "Width": width, "Height": height-legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            for (var i = Items.length - 1; i >= 0; i--) {
                var item = Items[i];
                item["AreaByWeight"] = (AvailableSize.Height * AvailableSize.Width) * item.weight / totalweight;
                item.headerTopPosition = yPos;
            }
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var OrderedItems = Items.sort(this._orderByAreaWight);
            var curX = 0, curY = 0;
            var j = 0;
            for (var i = 0; i < Items.length; i = j) {
                var firstTreemapItem = OrderedItems[i];
                weightObject = this._getGroupTotalWeight(firstTreemapItem, OrderedItems, AvailableArea, i);
                var GroupTotalWeight = weightObject.totalWeight;
                j = weightObject.index;
                var currentRect = {};
                var isHorizontal = (AvailableArea.Width > AvailableArea.Height) ? true : false;
                for (var k = i; k < j; k++) {
                    var item = OrderedItems[k];
                    var areaSum = GroupTotalWeight;
                    if (k == i) {
                        currentRect.X = AvailableArea.X;
                        currentRect.Y = AvailableArea.Y;
                        if (isHorizontal) {
                            currentRect.Width = areaSum / AvailableArea.Height;
                            currentRect.Height = AvailableArea.Height;
                            AvailableArea.X += currentRect.Width;
                            AvailableArea.Width = Math.max(0, AvailableArea.Width - currentRect.Width);
                        }
                        else {
                            currentRect.Width = AvailableArea.Width;
                            currentRect.Height = areaSum / AvailableArea.Width;
                            AvailableArea.Y += currentRect.Height;
                            AvailableArea.Height = Math.max(0, AvailableArea.Height - currentRect.Height);
                        }
                        curX = currentRect.X;
                        curY = currentRect.Y;
                    }
                    var rect = {};
                    if (OrderedItems.indexOf(item) != OrderedItems.length - 1) {
                        rect.X = 0;
                        rect.Y = 0;
                        rect.Width = (isHorizontal) ? currentRect.Width - parseFloat(Gap) : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height - parseFloat(Gap);
                        if (j - k != 1) {
                            if (isHorizontal) {
                                rect.Height -= parseFloat(Gap);
                            }
                            else {
                                rect.Width -= parseFloat(Gap);
                            }
                        }
                    }
                    else {
                        rect.Width = (isHorizontal) ? currentRect.Width : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height;
                    }
                    item.ItemWidth = rect.Width;
                    item.ItemHeight = rect.Height;
                    item.LeftPosition = curX;
                    item.TopPosition = curY;
                    if (isHorizontal) {
                        curY += (j - k != 1) ? rect.Height + parseFloat(Gap) : rect.Height;
                    }
                    else {
                        curX += (j - k != 1) ? rect.Width + parseFloat(Gap) : rect.Width;
                    }
                }
            }
        },
        
        _getGroupTotalWeight: function (firstTreemapItem, OrderedItems, AvailableArea, j) {
            var GroupTotalWeight = 0;
            var GroupMaxAspectRatio = 0;
            for (; j < OrderedItems.length; j++) {
                var ShorterSideLength = this._getShortersideLength(AvailableArea.Width, AvailableArea.Height);
                var lastTreemapItem = OrderedItems[j];
                GroupTotalWeight += lastTreemapItem.AreaByWeight;
                var GroupWidth = GroupTotalWeight / ShorterSideLength;
                var firstitemheight = firstTreemapItem.AreaByWeight / GroupWidth;
                var lastitemheight = lastTreemapItem.AreaByWeight / GroupWidth;
                if (j == 0)
                    GroupMaxAspectRatio = this._aspectRatio(GroupWidth, ShorterSideLength);
                var TempAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                if (GroupTotalWeight == lastTreemapItem.AreaByWeight || TempAspectRatio < GroupMaxAspectRatio) {
                    GroupMaxAspectRatio = TempAspectRatio;
                }
                else {
                    GroupTotalWeight -= lastTreemapItem.AreaByWeight;
                    GroupWidth = GroupTotalWeight / ShorterSideLength;
                    GroupMaxAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                    break;
                }
            }
            return { totalWeight: GroupTotalWeight, index: j };
        },
       
        _aspectRatio: function (x, y) {
            return (x > y) ? (x / y) : (y / x);
        },

        _getShortersideLength: function (width, height) {
            return width > height ? height : width;
        },

        _getTotalWeight: function (Items) {
            var total = 0;
            for (var i = 0; i < Items.length; i++) {
                total += parseFloat(Items[i].weight);
            }
            return total;

        },

        _getGroupitem: function (path, dataSource, headerHeight, labelPath) {
            var obj = [];            
			if(labelPath)
					dataSource = dataSource[0][labelPath];
			if(path == null)
				path = this.labelPath() || this.weightValuePath() ;
            if (dataSource != null && this.weightValuePath() != null) {                
                for (var i in dataSource) {
                    var item = dataSource[i];
                    if (item != null && item.hasOwnProperty(path) ? true : this._containsProperty(item,path)) {
                        var itemValue = this._reflection(item, path);
                        if ($.inArray(itemValue, obj) == -1) {
                            obj.push(itemValue);
                        }
                    }
                }                
                var objCollection = [];
                for (var i = 0; i < obj.length; i++) {
                    var Coll = {};
                    var groupObj = [];
                    var groupId = obj[i];
                    var colorWeight = 0;
                    var weight = 0;
                    var count = 0;
                    for (var j = 0; j < dataSource.length;j++) {
                        var item = dataSource[j];
                        var itemValue = this._reflection(item, path);
                        if (itemValue == groupId) {
                            var weightValue = this._reflection(item, this.weightValuePath());
                       
                            if (weightValue!=null && (typeof weightValue != "number")) {
                                weightValue = Number(weightValue.replace(/[^0-9\.]+/g, ""));
                            }
                            if (weightValue < 0) {
                                weightValue = -1 * weightValue;
                            }
                            var colorWeightValue = this._reflection(item, this.colorValuePath());
                            if (colorWeightValue!=null && (typeof colorWeightValue != "number")) {

                                colorWeightValue = Number(colorWeightValue.replace(/[^0-9\.]+/g, ""));
                            }
                            groupObj.push(item);
                            count++;
                            if (weightValue != null) {
                                weight += parseFloat(weightValue);
                            }
                            if (colorWeightValue != null) {
                                colorWeight += parseFloat(colorWeightValue);
                            }
                        }
                    }
                    
                    Coll["header"] = groupId;
                    Coll["data"] = groupObj;
                    Coll["weight"] = weight;
                    Coll["colorWeight"] = colorWeight;
                    Coll["headerHeight"] = headerHeight;
                    if (weight > 0) {
                        var newTreemapItem = new treeMapItem(Coll);
                        objCollection.push(newTreemapItem);
                    }
                }
                return objCollection;
            }
            else { return null; }
        },
      
        _getTopGroupitem: function (path, dataSource) {            
            if (path == null) path = this.labelPath();
            if (path == null) path = this.weightValuePath();
            var obj = [];
            var Coll = {};
            var weight = 0, colorWeight = 0;
            var groupId="";
            var innerobjCollection = [];
            var itemColl = [];
            for (var j = 0; j < dataSource.length; j++) {
                var item = dataSource[j];
                var itemValue = this._reflection(item, path);
                var innerColl={};
                var weightValue = this._reflection(item, this.weightValuePath());
                if (weightValue != null && (typeof weightValue != "number")) {
                    weightValue = Number(weightValue.replace(/[^0-9\.]+/g, ""));
                }
                if (weightValue < 0) {
                    weightValue = -1 * weightValue;
                }
                var colorWeightValue = this._reflection(item, this.colorValuePath());
                if (colorWeightValue!=null && (typeof colorWeightValue != "number")) {

                    colorWeightValue = Number(colorWeightValue.replace(/[^0-9\.]+/g, ""));
                }  
                if (weightValue != null) {
                    weight += parseFloat(weightValue);
                }
                if (colorWeightValue != null) {
                    colorWeight += parseFloat(colorWeightValue);
                }
                itemColl.push(item);
                innerColl["weight"] = weightValue;
                innerColl["data"] = item;                
                innerColl["header"] = itemValue;
                innerColl["colorWeight"] = colorWeightValue;                                       
                var newinnerTreemapItem = new treeMapItem(innerColl);
                innerobjCollection.push(newinnerTreemapItem);
            }        
            Coll["header"] = groupId;
            Coll["data"] = itemColl;
            Coll["weight"] = weight;
            Coll["innerItem"] = innerobjCollection;
            Coll["colorWeight"] = colorWeight;                                   
            var treemapColl = [];
            var treeitem = new treeMapItem(Coll);
            treemapColl.push(treeitem);
            return  treemapColl;
        },         
       
        _containsProperty: function (object, propertyName) {
            for (var key in object)
            {
                if(key == propertyName)
                {
                    return true;
                }
            }
            return false;
        },

        _reflection: function (Source, Path) {
            var ShapeValues = Source;
            if (Path != null && Source != null) {
                var parts = Path.split(".");
                parts.push(Path);
                var i = 0;
                for (; i < parts.length; i++) {
                    if (ShapeValues != null) {
                        var hasProperty = ShapeValues.hasOwnProperty(parts[i]) ? true : this._containsProperty(ShapeValues, parts[i]);
                        if (hasProperty) {
                            $.each(ShapeValues, function (prop, propval) {
                                if (prop == parts[i]) {
                                    ShapeValues = propval;
                                    return false;
                                }
                            });
                        }
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        }

    });

    var treeMapLevel = function () {        
        this.groupBackground = null;       
        this.groupBorderColor = null;
        this.groupBorderThickness = 1;       
        this.groupPadding = 4;
        this.groupPath = null;
        this.groupGap = 1;       
        this.headerHeight = 20;
        this.showHeader = true;       
        this.showLabels = false;
        this.headerTemplate = null;       
        this.labelTemplate = null;
        this.labelPosition = "center";
        labelVisibilityMode = "visible";      
        headerVisibilityMode = "visible";

        this.treeMapItems = [];
    };

    treeMapLevel.prototype = {

    };
	
    var TreeMapGroupColorMapping = function () {	
        this.groupID = null;		       
        this.rangeColorMapping = [];

        this.desaturationColorMapping = { 				            
            from:0, 				            
            to:0,			
            color: "",								            
            rangeMinimum:0,		
            rangeMaximum:0
        };  
       
        this.uniColorMapping = {             
            color: null 
        };
     
        this.paletteColorMapping = {           
            colors: [],
        };
    };		
	
    var TreeMapRangeColorMapping = function () {
        
        this.from = -1;		
        this.to = -1;		    
        this.legendlabel = null;		      
        this.color = null;
    };
	
    var treeMapItem = function (params) {
        this.weight = params.weight;
        this.colorWeight = params.colorWeight;
        this.Data = params.data;
        this.headerHeight = params.headerHeight;
        this.header = params.header;
        this.label = params.header;
        this.headerLeftPosition = 0;
        this.headerTopPosition = 0;
        this.innerItems = params.innerItem;
        this.headerWidth = 0;
        this.headerTemplate = null;
    };

    treeMapItem.prototype = {

        _generateColorMappings: function (colorMappings,treeMap) {
            if (colorMappings.rangeColorMapping != null && colorMappings.rangeColorMapping.length > 0) {
                return this._getRangeBrushColor(colorMappings.rangeColorMapping);
            }
            if (treeMap.uniColor() != null && colorMappings.uniColorMapping!=null) {
                return treeMap.uniColor();
            }
            else if (colorMappings.uniColorMapping.color != null && colorMappings.uniColorMapping != null) {
                return this._getUniColor(colorMappings.uniColorMapping);
            }
            
        },

        _getUniColor: function (uniColorMapping) {
            return uniColorMapping.color;
        },
      
        _getRangeBrushColor: function (rangeColorMapping) {
            for (var j = 0; j < rangeColorMapping.length; j++) {
                var rangeBrush = rangeColorMapping[j];
                if (this.colorWeight >= rangeBrush.from && this.colorWeight <= rangeBrush.to) {
                    return rangeBrush.color;
                }
            }
        }
    };
	
	ej.datavisualization.TreeMap.selectionMode = {
		Default: "default",
		Multiple: "multiple"
	};
	
	ej.datavisualization.TreeMap.groupSelectionMode = {
		Default: "default",
		Multiple: "multiple"
	};
	
    ej.datavisualization.TreeMap.ItemsLayoutMode = {        
        Squarified: "squarified",        
        SliceAndDiceHorizontal: "sliceanddicehorizontal",        
        SliceAndDiceVertical: "sliceanddicevertical",        
        SliceAndDiceAuto: "sliceanddiceauto"
    };  

    ej.datavisualization.TreeMap.DockPosition = {        
        Top: "top",        
        Bottom: "bottom",        
        Right: "right",        
        Left: "left"
    };
	ej.datavisualization.TreeMap.LegendMode = {
      Default: "default",
      Interactive: "interactive"
    };
    ej.datavisualization.TreeMap.Position = {        
        TopLeft: "topleft",        
        TopCenter: "topcenter",        
        TopRight: "topright",        
        CenterLeft: "centerleft",        
        Center: "center",        
        CenterRight: "centerright",        
        BottomLeft: "bottomleft",        
        BottomCenter: "bottomcenter",        
        BottomRight:"bottomright"
    };
    ej.datavisualization.TreeMap.Alignment = {       
        Near: "near",        
        Center: "center",        
        Far: "far"
    }
    ej.datavisualization.TreeMap.VisibilityMode = {        
        Visible:"visible",        
        HideOnExceededLength:"hideonexceededlength"
    };
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Barcode elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
* Use of this code is subject to the terms of our license.
* A copy of the current license can be obtained at any time by e-mailing
* licensing@syncfusion.com. Any infringement will be prosecuted under
* applicable laws.
* @version 12.1
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /** ejBarcode is the plugin name.*/
    /** "ej.datavisualization.Barcode" is "namespace.className" will hold functions and properties.*/
   
    ej.widget("ejBarcode", "ej.datavisualization.Barcode", {
        /** Widget element will be automatically set in this*/
        element: {
            target: null
        },

        /** User defined model will be automatically set in this*/
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-barcode",

        defaults: {
            displayText: true,
            text: "",
            height: "",
            width: "",
            symbologyType: "qrbarcode",
            textColor: 'black',
            lightBarColor: 'white',
            darkBarColor: 'black',
            quietZone: {
                left: 1,
                top: 1,
                right: 1,
                bottom: 1,
                all: 1
            },
            narrowBarWidth: 1,
            wideBarWidth: 3,
            barHeight: 150,
            barcodeToTextGapHeight: 10,
            xDimension: 4,
            encodeStartStopSymbol: true,
            load: null,
            enabled: true,
        },

        /**
         * Specify the data types for default properties 
         * @private
         */
        dataTypes: {
            displayText: "boolean",
            text: "string",
            enabled: "boolean",
            symbologyType: "enum",
            narrowBarWidth: "number",
            wideBarWidth: "number",
            barHeight: "number",
            encodeStartStopSymbol: "boolean"
        },

        /**
         * @private
         */
        _defaultScaleValues: function () {
            return {
                enabled: true,
                displayText: true,
                text: "",
                symbologyType: "qrbarcode",
                textColor: 'black',
                lightBarColor: 'white',
                darkBarColor: 'black',
                quietZone: { left: 1, top: 1, right: 1, bottom: 1 },
                narrowBarWidth: 1,
                wideBarWidth: 3,
                barHeight: 150,
                barcodeToTextGapHeight: 10,
                xDimension: 4,
                encodeStartStopSymbol: true
            };
        },

        /**
         * @constructor
         * @private
         */
        _init: function () {
            this._initialize();
            this._renderBarcode();
        },

        /**
         * @private
         */
        _destroy: function () {
            this.element.empty();
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "lightBarColor": this.model.lightBarColor = options[option]; break;
                    case "darkBarColor": this.model.darkBarColor = options[option]; break;
                    case "textColor": this.model.textColor = options[option]; break;
                    case "displayText": this.model.displayText = options[option]; break;
                    case "quietZone": this.model.quietZone = options[option]; break;
                    case "narrowBarWidth": this.model.narrowBarWidth = options[option]; break;
                    case "wideBarWidth": this.model.wideBarWidth = options[option]; break;
                    case "barHeight": this.model.barHeight = options[option]; break;
                    case "xDimension": this.model.xDimension = options[option]; break;
                    case "encodeStartStopSymbol": this.model.encodeStartStopSymbol = options[option]; break;
                    case "symbologyType":
                        this.model.symbologyType = options[option];
                        this._itemInitialize();
                        break;
                    case "text":
                        this.model.text = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._renderBarcode();
        },

        /**
         * Initializes control.
         * @private
         */
        _initialize: function () {
            this.BarcodeEl = this.element;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },

        /**
         * Renders barcode.
         * @private
         */
        _renderBarcode: function () {
            this.initialize();
        },

        /**
         * @private
         */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },

        /**
         * @private
         */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
         * @private
         */
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, Ele) {
                if (Ele instanceof Array) {
                    Ele = proxy._checkArrayObject(Ele, name);
                }
                else if (Ele != null && typeof Ele == "object") {
                    Ele = proxy._LoadIndividualDefaultValues(Ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        /**
         * Initializes the control.
         * @private
         */
        initialize: function () {
            if (!this.model.enabled)
                return;

            this._initObject(this);

            if (this.Model.text != null)
                this._findBarcodeType();
        },

        /**
        * To enable the barcode 
        */
        enable: function () {
            if (!this.model.enabled) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },
        
        /**
        * To change the height of Barcode
        */
        height: function (value) {
            this.Model.height = value;
        },
        
        /**
        * To change the width of Barcode
        */
        width: function (value) {
            this.Model.width = value;
        },
        
        /**
        * To resize the barcode 
        */
        resize: function () {
            if (this.Model.text != null)
                this._findBarcodeType();
        },

        /**
        * To disable the barcode  
        */
        disable: function () {
            if (this.model.enabled) {
                this.element.addClass("e-disable");
                this.model.enabled = false;
            }
        },

        /**
         * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        /**
         * Initializes rendering canvas.
         * @private
         */
        _initObject: function (element) {
            element.BarcodeEl = $("#" + element.target.id);
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.BarcodeEl.append(element.canvasEl);
            element.BarcodeEl.css("width", element.model.width);
            element.BarcodeEl.css("height", element.model.height);
            element.canvasEl[0].setAttribute("width", element.BarcodeEl.width());
            element.canvasEl[0].setAttribute("height", element.BarcodeEl.height());
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");

            if (element.contextEl == undefined) //IE 8
                return;
        },

        /**
         * Finds the barcode type and encodes text.
         * @private
         */
        _findBarcodeType: function () {
            if (this.contextEl == undefined) //IE 8
                return;

            if (this.Model.text == "")
                return;

            this.textFont = { size: "12px", fontFamily: "Segoe UI", fontStyle: "Regular" };

            if (this.model.quietZone.all > 1)
                this.model.quietZone.left = this.model.quietZone.right = this.model.quietZone.top = this.model.quietZone.bottom = this.model.quietZone.all;

            intercharacterGap = 1;
            extendedText = "";
            continuous = false;
            validatorExp = "";
            encodedText = this.Model.text;
            var startSymbol, stopSymbol, symbolTable, extendedCodes;

            var type = this.Model.symbologyType;
            var oneD = true;

            /** Apply symbology setting.*/
            if (ej.datavisualization.Barcode.SymbologyType.dataMatrix == type || ej.datavisualization.Barcode.SymbologyType.qrBarcode == type) {
                oneD = false;
            }
            else {
                startSymbol = ej.datavisualization.Barcode.symbologySettings[type].startSymbol;
                stopSymbol = ej.datavisualization.Barcode.symbologySettings[type].stopSymbol;
                validatorExp = ej.datavisualization.Barcode.symbologySettings[type].validatorExp;
                symbolTable = ej.datavisualization.Barcode.symbologySettings[type].symbolTable;
                extendedCodes = ej.datavisualization.Barcode.symbologySettings[type].extendedCodes;

                encodedText = this._calculateCheckDigit(encodedText, symbolTable, type);

                if (ej.datavisualization.Barcode.SymbologyType.code39Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
                else if (ej.datavisualization.Barcode.SymbologyType.code93Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
            }

            /** Process one dimensional barcode.*/
            if (oneD) {
                if (!this._validateText(this.Model.text, validatorExp)) {
                    console.log("Barcode Text contains characters that are not accepted by this barcode specification.");
                    return;
                }

                if (this.model.symbologyType == "code32" && this.model.text.length != 8) {
                    console.log("Barcode Text Length that are not accepted by this barcode specification.");
                    return;
                }

                if (extendedText.length > 0)
                    encodedText = extendedText;

                if (this.Model.encodeStartStopSymbol)
                    encodedText = startSymbol.toString() + encodedText + stopSymbol.toString();

                var w = this._getCharWidth(encodedText, symbolTable);

                if (continuous == false)
                    w -= (intercharacterGap * encodedText.length);
                else {
                    w -= (extendedText.length > 0) ?
                        intercharacterGap * (extendedText.length - this.Model.text.length) :
                        intercharacterGap;
                }

                var textHeight = parseInt(this.Model.displayText ? this.textFont.size.replace('px', '') : 0);
                var h = this.Model.quietZone.top + this.Model.quietZone.bottom + this.Model.barHeight + intercharacterGap + (this.Model.displayText ? (textHeight + this.Model.barcodeToTextGapHeight) : 0);
                w += this.Model.quietZone.left + this.Model.quietZone.right;

                

                /** Draw one dimensional barcode.*/
                this._draw1DBarcode(encodedText, symbolTable, w);
            }
                /** Process two dimensional barcode.*/
            else {
                if (ej.datavisualization.Barcode.SymbologyType.dataMatrix == type)
                    this._buildDataMatrix();
                else if (ej.datavisualization.Barcode.SymbologyType.qrBarcode == type)
                    this._buildQRBarcode();
            }

            /** Raise load event*/
            this._raiseEvent("load");
        },

        /**
         * Calculate the checkdigit value.
         * @private
         */
        _calculateCheckDigit: function (encodedText, symbolTable, type) {
            if (type == "code128b") {
                var checkValue = 0;
                var char = [encodedText.split("")];
                for (var m = 0; m < encodedText.split("").length; m++) {
                    var item = char[0][m];
                    var checkdigit = this._findCheckDigit(item, symbolTable)[1];
                    checkValue += (checkdigit * (m + 1));

                }
                //The start character value for the code128B is 104.
                checkValue += 104;
                //To claculate the checksum value.
                checkValue = checkValue % 0x67;
                var ch = [1];
                for (var e = 0; e < symbolTable.length; e++) {
                    if (e == checkValue) {
                        ch[0] = symbolTable[e][0];
                    }
                }
                encodedText += ch.toString();
            }
            else {
                if(type=="upcbarcode")
                {
                    var char = [encodedText.split("")];
                    var iSum = 0;
                    var iDigit = 0;
                    for (var m = 0; m < encodedText.split("").length; m++) {
                        if (m < 11) {
                            iDigit = encodedText[m];
                            if (m % 2 == 0) {    // odd
                                iSum += iDigit * 3;
                            }
                            else {    // even
                                iSum += iDigit * 1;
                            }
                        }

                    }
                    var iCheckSum = (10 - (iSum % 10)) % 10;
                    var data = encodedText.split("");
                    // Add the leading digit pattern for the text.
                    var str =6;
                    for (var k = 0; k < data.length; k++)
                    {
                        if (k == 6)
                        {
                            // Add the separator digit to middle of the text.
                            str += "B" + data[k];

                        }
                        else
                        {
                            if (k != 11 )
                                str += data[k];
                        }

                    }
                    encodedText = str;

                    //Add the checkdigit value. 
                    encodedText += iCheckSum;

                    //Add the ending digit pattern for the text.
                    encodedText += 6;
                   
                }
            }
            return encodedText;

        },

        /**
       * Finds the checkdigit value from symboltable.
       * @private
       */
        _findCheckDigit: function (encodedText, symbolTable) {
            for (var i = 0; i < symbolTable.length; i++) {
                if (symbolTable[i][0] == encodedText) {
                    return symbolTable[i];
                }
            }
        },

        /**
         * Load event.
         * @private
         * @event
         */
        _raiseEvent: function (event) {
            if (this.model[event])
                this._trigger(event);
        },

        /**
         * Generates QR barcode.
         * @private
         */
        _buildQRBarcode: function () {
            var txt = this.Model.text;
            this.m_EciAssignmentNumber = 3;
            this.mode = "numeric";
            this.isEci = false;
            this.m_Version = 1;
            var dataCapacityOfVersions = new Array(14, 26, 42, 62, 84, 106, 122, 152, 180, 213, 251, 287, 331, 362, 412, 450, 504, 560, 624, 666, 711, 779, 857, 911, 997, 1059, 1125, 1190, 1264, 1370, 1452, 1538, 1628, 1722, 1809, 1911, 1989, 2099, 2213, 2331);
            for (var i = 0; i < txt.length; i++) {
                var cCode = txt.charCodeAt(i);
                if (cCode < 58 && cCode > 47) {
                    /** numeric only*/
                }
                else if ((cCode < 91 && cCode > 64) || txt[i] == '$' || txt[i] == '%' || txt[i] == '*' || txt[i] == '+' || txt[i] == '-' || txt[i] == '.' || txt[i] == '/' || txt[i] == ':' || txt[i] == ' ') {
                    this.mode = "alphanumeric";
                }
                else if ((cCode >= 65377 && cCode <= 65439) || (cCode >= 97 && cCode <= 122)) {
                    this.mode = "binary";
                    break;
                }
                else {
                    this.mode = "binary";
                    this.isEci = true;
                    break;
                }
            }
            if (this.mode == "numeric")
                dataCapacityOfVersions = new Array(34, 63, 101, 149, 202, 255, 293, 365, 432, 513, 604, 691, 796, 871, 991, 1082, 1212, 1346, 1500, 1600, 1708, 1872, 2059, 2188, 2395, 2544, 2701, 2857, 3035, 3289, 3486, 3693, 3909, 4134, 4343, 4588, 4775, 5039, 5313, 5596);
            else if (this.mode == "alphanumeric")
                dataCapacityOfVersions = new Array(20, 38, 61, 90, 122, 154, 178, 221, 262, 311, 366, 419, 483, 528, 600, 656, 734, 816, 909, 970, 1035, 1134, 1248, 1326, 1451, 1542, 1637, 1732, 1839, 1994, 2113, 2238, 2369, 2506, 2632, 2780, 2894, 3054, 3220, 3391);

            if (this.isEci == true) {
                this.m_EciAssignmentNumber = this._findECINumber(txt);
            }
            for (var i = 0; i < dataCapacityOfVersions.length; i++) {
                if (dataCapacityOfVersions[i] > txt.length) {
                    this.m_Version = i + 1;
                    break;
                }
            }

            this.qrbarcodeValues = { Version: this.m_Version, ErrorCorrectionLevel: 1, NumberOfDataCodeWord: this._getNumberOfDataCodeWord(), NumberOfErrorCorrectingCodeWords: this._getNumberOfErrorCorrectingCodeWords(), NumberOfErrorCorrectionBlocks: this._getNumberOfErrorCorrectionBlocks(), End: this._getEnd(), DataCapacity: this._getDataCapacity(), FormatInformation: this._getFormatInformation(), VersionInformation: this._getVersionInformation() };
            this.noOfModules = (this.m_Version - 1) * 4 + 21;

            this.moduleValue = this._create2DArray(this.noOfModules, this.noOfModules);

            for (var i = 0; i < this.noOfModules; i++) {
                for (var j = 0; j < this.noOfModules; j++) {
                    this.moduleValue[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            this._drawPDP(0, 0);
            this._drawPDP(this.noOfModules - 7, 0);
            this._drawPDP(0, this.noOfModules - 7);
            this._drawTimingPattern();

            if (this.m_Version != 1) {
                var allignCoOrdinates = this._getAlignmentPatternCoOrdinates();

                for (var i = 0; i < allignCoOrdinates.length; i++) {
                    for (var j = 0; j < allignCoOrdinates.length; j++) {
                        var x = allignCoOrdinates[i];
                        var y = allignCoOrdinates[j];
                        if (this.moduleValue[x][y].IsPDP != true)
                            this._drawAlignmentPattern(x, y);
                    }
                }
            }

            this._allocateFormatVersionInfo();
            this._encodeData();
            this._drawFormatInformation();
            this._addQuietZone();

            var w = this.moduleValue.length * this.Model.xDimension;
            var h = this.moduleValue[0].length * this.Model.xDimension;

            

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw QR barcode in canvas.*/
            this._drawQRBarcode(h, w);
        },

        /** 
         * Allocate format and version information.
         * @private
         */
        _allocateFormatVersionInfo: function () {
            for (var i = 0; i < 9; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            for (var i = this.noOfModules - 8; i < this.noOfModules; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            if (this.m_Version > 6) {
                var versionInformation = this.qrbarcodeValues.VersionInformation;
                var count = 0;
                for (var i = 0; i < 6; i++) {
                    for (var j = 2; j >= 0; j--) {
                        this.moduleValue[i][this.noOfModules - 9 - j].IsBlack = versionInformation[count] == 1 ? true : false;
                        this.moduleValue[i][this.noOfModules - 9 - j].IsFilled = true;

                        this.moduleValue[this.noOfModules - 9 - j][i].IsBlack = versionInformation[count++] == 1 ? true : false;
                        this.moduleValue[this.noOfModules - 9 - j][i].IsFilled = true;
                    }
                }
            }
        },

        /**
         * @private
         */
        _drawTimingPattern: function () {
            for (var i = 8; i < this.noOfModules - 8; i += 2) {
                this.moduleValue[i][6].IsBlack = true;
                this.moduleValue[i][6].IsFilled = true;

                this.moduleValue[i + 1][6].IsBlack = false;
                this.moduleValue[i + 1][6].IsFilled = true;

                this.moduleValue[6][i].IsBlack = true;
                this.moduleValue[6][i].IsFilled = true;

                this.moduleValue[6][i + 1].IsBlack = false;
                this.moduleValue[6][i + 1].IsFilled = true;
            }
            this.moduleValue[this.noOfModules - 8][8].IsBlack = true;
            this.moduleValue[this.noOfModules - 8][8].IsFilled = true;
        },

        /** 
         * Draws QR barcode.
         * @private
         */
        _drawQRBarcode: function (ht, w) {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;

            var whiteBrush = 'white';
            var blackBrush = 'black';
            
            var dim = parseInt(this.Model.xDimension);
            if(this.Model.height != "" && this.Model.width != "")
			{
            this.Model.height = this._getProperValue(this.Model.height);
            this.Model.width = this._getProperValue(this.Model.width);
            }
			else
			{ 
			this.Model.height = ht;
			this.Model.width = w;
			this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);
			this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
			}			
            var dimension = Math.min(this.Model.width, this.Model.height);
            
            if(dimension != 0)
                dim = dimension / (this.noOfModules + (2 * quietZone));

            var y = 0;

            var w = this.noOfModules + 2 * quietZone, h = this.noOfModules + 2 * quietZone;

            this.contextEl.beginPath();

            for (var i = 0; i < w; i++) {
                var x = 0;

                for (var j = 0; j < h; j++) {
                    var colorBrush = null;
                    if (this.moduleValue[i][j].IsBlack)
                       colorBrush = this.model.darkBarColor;
						
					else
				
                      colorBrush = this.model.lightBarColor;

                    if (this.dataAllocationValues[j][i].IsFilled)
                        if (this.dataAllocationValues[j][i].IsBlack)
                            colorBrush = 'black';

                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)

                    x += dim;
                }

                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },


         _getProperValue: function (value) {
             if(value == "")
                return "";
             if(typeof(value) == "string")
             {
                value = value.replace("px","");
                return parseInt(value);
             }
             else if(typeof(value) == "number")
                return value;
             return 0;
         },
         
        /** 
         * Adds quiet zone to the barcode.
         * @private
         */
        _addQuietZone: function () {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;
            var w = this.noOfModules + 2 * quietZone;
            var h = this.noOfModules + 2 * quietZone;
            var tempValue1 = this._create2DArray(w, h);
            var tempValue2 = this._create2DArray(w, h);

            /** Initialize array.*/
            for (var i = 0; i < w; i++) {
                for (var j = 0; j < h; j++) {
                    tempValue1[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                    tempValue2[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    tempValue1[i][j] = this.moduleValue[i - quietZone][j - quietZone];
                    tempValue2[i][j] = this.dataAllocationValues[i - quietZone][j - quietZone];
                }
            }

            this.moduleValue = tempValue1;
            this.dataAllocationValues = tempValue2;
        },

        /** 
         * Adds format information to the barcode.
         * @private
         */
        _drawFormatInformation: function () {
            var formatInformation = this.qrbarcodeValues.FormatInformation;

            var count = 0;
            for (var i = 0; i < 7; i++) {
                /** Draw from 14 to 8.*/
                if (i == 6)
                    this.moduleValue[i + 1][8].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[i][8].IsBlack = formatInformation[count] == 1 ? true : false;

                this.moduleValue[8][this.noOfModules - i - 1].IsBlack = formatInformation[count++] == 1 ? true : false;
            }
            count = 14;
            for (var i = 0; i < 7; i++) {
                /** Draw from 0 to 6.*/
                if (i == 6)
                    this.moduleValue[8][i + 1].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[8][i].IsBlack = formatInformation[count] == 1 ? true : false;
                this.moduleValue[this.noOfModules - i - 1][8].IsBlack = formatInformation[count--] == 1 ? true : false;
            }

            /** Draw 7.*/
            this.moduleValue[8][8].IsBlack = formatInformation[7] == 1 ? true : false;
            this.moduleValue[8][this.noOfModules - 8].IsBlack = formatInformation[7] == 1 ? true : false;
        },

        /** 
         * @private
         */
        _dataAllocationAndMasking: function (Data) {
            var num = this.noOfModules;
            this.dataAllocationValues = this._create2DArray(num, num);
            var point = 0;

            for (var i = 0; i < num; i++) 
                for (var j = 0; j < num; j++) 
                    this.dataAllocationValues[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };

            for (var i = num - 1; i >= 0; i -= 2) {
                for (var j = num - 1; j >= 0; j--)
                    point = this._allocateValues(Data, i, j, point);

                i -= 2;
                if (i == 6)
                    i--;

                for (var j = 0; j < num; j++)
                    point = this._allocateValues(Data, i, j, point);
            }
            for (var i = 0; i < num; i++)
                for (var j = 0; j < num; j++)
                    if (!this.moduleValue[i][j].IsFilled) {
                        var flag = this.dataAllocationValues[i][j].IsBlack;
                        if (flag)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }


        },

        /** 
         * @private
         */
        _allocateValues: function (Data, i, j, point) {
            if (!(this.moduleValue[i][j].IsFilled && this.moduleValue[i - 1][j].IsFilled)) {
                if (!this.moduleValue[i][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i][j].IsBlack = Data[point++];
                    if ((i + j) % 3 == 0) {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }

                    this.dataAllocationValues[i][j].IsFilled = true;
                }

                if (!this.moduleValue[i - 1][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i - 1][j].IsBlack = Data[point++];
                    if ((i - 1 + j) % 3 == 0) {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                    }
                    this.dataAllocationValues[i - 1][j].IsFilled = true;
                }
            }

            return point;
        },

        /** 
         * Encodes data.
         * @private
         */
        _encodeData: function () {
            var eData = new Array();
            var c = -1;

            switch (this.mode) {
                case "numeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    break;

                case "alphanumeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    break;

                case "binary":
                    if (this.isEci) {
                        /** Add ECI Mode Indicator.*/
                        eData[++c] = false;
                        eData[++c] = true;
                        eData[++c] = true;
                        eData[++c] = true;

                        /** Add ECI assignment number.*/
                        var numberInBool = this._stringToBoolArray(this.m_EciAssignmentNumber.toString(), 8);
                        for (var e = 0; e < numberInBool.length; e++)
                            eData[++c] = numberInBool[e];
                    }
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    break;
            }

            var numberOfBitsInCharacterCountIndicator = 0;
            if (this.m_Version < 10) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 10;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 9;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 8;
                        break;
                }
            }
            else if (this.m_Version < 27) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 12;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 11;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }
            else {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 14;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 13;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }

            var dataStringArray = this.Model.text;
            var numberOfBitsInCharacterCountIndicatorInBool = this._intToBoolArray(dataStringArray.length, numberOfBitsInCharacterCountIndicator);

            for (var i = 0; i < numberOfBitsInCharacterCountIndicator; i++)
                eData[++c] = numberOfBitsInCharacterCountIndicatorInBool[i];

            if (this.mode == "numeric") {
                var number = "";
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    number += dataStringArray[i];

                    if (i % 3 == 2 && i != 0 || i == dataStringArray.length - 1) {
                        if (number.length == 3)
                            numberInBool = this._stringToBoolArray(number, 10);
                        else if (number.length == 2)
                            numberInBool = this._stringToBoolArray(number, 7);
                        else
                            numberInBool = this._stringToBoolArray(number, 4);
                        number = "";
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }
            else if (this.mode == "alphanumeric") {
                var numberInString = "";
                var number = 0;
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    numberInString += dataStringArray[i];

                    if (i % 2 == 0 && i + 1 != dataStringArray.length)
                        number = 45 * this._getAlphanumericvalues(dataStringArray[i]);

                    if (i % 2 == 1 && i != 0) {
                        number += this._getAlphanumericvalues(dataStringArray[i]);
                        numberInBool = this._intToBoolArray(number, 11);
                        number = 0;
                        for (var n = 0; n < numberInBool.length; n++)
                            eData[++c] = numberInBool[n];
                        numberInString = "";
                    }
                    if (i != 1 && numberInString != null)
                        if (i + 1 == dataStringArray.length && numberInString.length == 1) {
                            number = this._getAlphanumericvalues(dataStringArray[i]);
                            numberInBool = this._intToBoolArray(number, 6);
                            number = 0;
                            for (var j = 0; j < numberInBool.length; j++)
                                eData[++c] = numberInBool[j];
                        }
                }
            }
            else if (this.mode = "binary") {
                {
                    for (var i = 0; i < dataStringArray.length; i++) {
                        var number = 0;
                        if ((dataStringArray.charCodeAt(i) >= 32 && dataStringArray.charCodeAt(i) <= 126) || (dataStringArray.charCodeAt(i) >= 161 && dataStringArray.charCodeAt(i) <= 255))
                            number = dataStringArray.charCodeAt(i);
                        else if (dataStringArray.charCodeAt(i) >= 65377 && dataStringArray.charCodeAt(i) <= 65439)
                            number = dataStringArray.charCodeAt(i) - 65216;
                        else {
                            console.log("Input text contains non-convertable characters");
                            return;
                        }
                        var numberInBool = this._intToBoolArray(number, 8);
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }

            /** Terminator.*/
            for (var i = 0; i < 4; i++)
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else
                    eData[++c] = false;

            /** Encode to code words.*/
            for (; ;)              /** Add Padding Bits.*/
            {
                if (eData.length % 8 == 0)
                    break;
                else
                    eData[++c] = false;
            }

            for (; ;) {
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = true;       //11101100
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                }
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = false;       //00010001
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                }
            }

            var dataBits = this.qrbarcodeValues.NumberOfDataCodeWord;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            var totalBlockSize = blocks[0];
            if (blocks.length == 6)
                totalBlockSize = blocks[0] + blocks[3];

            var ds1 = new Array(totalBlockSize);

            var testEncodeData = eData;
            var n = -1;
            if (blocks.length == 6)     /** If seperated into two seperate blocks.*/
            {
                var dataCodeWordLength = blocks[0] * blocks[2] * 8;
                testEncodeData = new Array(dataCodeWordLength);
                for (var i = 0; i < dataCodeWordLength; i++)
                    testEncodeData[++n] = eData[i];
            }
            var dsOne = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[0]);
            dsOne = this._createBlocks(testEncodeData, blocks[0]);

            for (var i = 0; i < blocks[0]; i++) {
                ds1[i] = this._splitCodeWord(dsOne, i, testEncodeData.length / 8 / blocks[0]);
            }

            n = -1;
            if (blocks.length == 6) {
                testEncodeData = new Array();
                for (var i = parseInt(blocks[0] * blocks[2] * 8) ; i < eData.length; i++)
                    testEncodeData[++n] = eData[i];

                var dsTwo = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[3]);
                dsTwo = this._createBlocks(testEncodeData, blocks[3]);

                for (var i = blocks[0], count = 0; i < totalBlockSize; i++)
                    ds1[i] = this._splitCodeWord(dsTwo, count++, testEncodeData.length / 8 / blocks[3]);
            }
            eData = new Array();
            c = -1;
            for (var i = 0; i < 125; i++) {
                for (var k = 0; k < totalBlockSize; k++)
                    for (var j = 0; j < 8; j++)
                        if (i < ds1[k].length) {
                            var temp = ds1[k][i][j];
                            eData[++c] = (temp == '1' ? true : false);
                        }
            }

            this.corCodeWords = { DataBits: this.qrbarcodeValues.NumberOfDataCodeWord, ECCW: this.qrbarcodeValues.NumberOfErrorCorrectingCodeWords, DC: 0 };

            var dataBits = this.corCodeWords.DataBits;
            var eccw = this.corCodeWords.ECCW;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            if (blocks.length == 6)
                this.corCodeWords.DataBits = (dataBits - blocks[3] * blocks[5]) / blocks[0];
            else
                this.corCodeWords.DataBits = dataBits / blocks[0];

            this.corCodeWords.ECCW = eccw / totalBlockSize;
            var polynomial = new Array(totalBlockSize);
            var count = 0;

            for (var i = 0; i < blocks[0]; i++) {
                this.corCodeWords.DC = ds1[count];
                polynomial[count++] = this._getERCW();
            }
            if (blocks.length == 6) {
                this.corCodeWords.DataBits = (dataBits - blocks[0] * blocks[2]) / blocks[3];

                for (var i = 0; i < blocks[3]; i++) {
                    this.corCodeWords.DC = ds1[count];
                    polynomial[count++] = this._getERCW();
                }
            }
            if (blocks.length != 6) {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < blocks[0]; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }
            else {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < totalBlockSize; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }


            this._dataAllocationAndMasking(eData);
        },

        /** 
         * @private
         */
        _getERCW: function () {
            var decimalRepresentation;
            var ecw;
            var decimalValue = new Array(this.corCodeWords.DataBits);
            var gx = new Array();

            switch (this.corCodeWords.ECCW) {
                case 7:
                    gx = new Array(0, 87, 229, 146, 149, 238, 102, 21);
                    gx = this._getElement(gx);
                    break;
                case 10:
                    gx = new Array(0, 251, 67, 46, 61, 118, 70, 64, 94, 32, 45);
                    gx = this._getElement(gx);
                    break;
                case 13:
                    gx = new Array(0, 74, 152, 176, 100, 86, 100, 106, 104, 130, 218, 206, 140, 78);
                    gx = this._getElement(gx);
                    break;
                case 15:
                    gx = new Array(0, 8, 183, 61, 91, 202, 37, 51, 58, 58, 237, 140, 124, 5, 99, 105);
                    gx = this._getElement(gx);
                    break;
                case 16:
                    gx = new Array(0, 120, 104, 107, 109, 102, 161, 76, 3, 91, 191, 147, 169, 182, 194, 225, 120);
                    gx = this._getElement(gx);
                    break;
                case 17:
                    gx = new Array(0, 43, 139, 206, 78, 43, 239, 123, 206, 214, 147, 24, 99, 150, 39, 243, 163, 136);
                    gx = this._getElement(gx);
                    break;
                case 18:
                    gx = new Array(0, 215, 234, 158, 94, 184, 97, 118, 170, 79, 187, 152, 148, 252, 179, 5, 98, 96, 153);
                    gx = this._getElement(gx);
                    break;
                case 20:
                    gx = new Array(0, 17, 60, 79, 50, 61, 163, 26, 187, 202, 180, 221, 225, 83, 239, 156, 164, 212, 212, 188, 190);
                    gx = this._getElement(gx);
                    break;
                case 22:
                    gx = new Array(0, 210, 171, 247, 242, 93, 230, 14, 109, 221, 53, 200, 74, 8, 172, 98, 80, 219, 134, 160, 105, 165, 231);
                    gx = this._getElement(gx);
                    break;
                case 24:
                    gx = new Array(0, 229, 121, 135, 48, 211, 117, 251, 126, 159, 180, 169, 152, 192, 226, 228, 218, 111, 0, 117, 232, 87, 96, 227, 21);
                    gx = this._getElement(gx);
                    break;
                case 26:
                    gx = new Array(0, 173, 125, 158, 2, 103, 182, 118, 17, 145, 201, 111, 28, 165, 53, 161, 21, 245, 142, 13, 102, 48, 227, 153, 145, 218, 70);
                    gx = this._getElement(gx);
                    break;
                case 28:
                    gx = new Array(0, 168, 223, 200, 104, 224, 234, 108, 180, 110, 190, 195, 147, 205, 27, 232, 201, 21, 43, 245, 87, 42, 195, 212, 119, 242, 37, 9, 123);
                    gx = this._getElement(gx);
                    break;
                case 30:
                    gx = new Array(0, 41, 173, 145, 152, 216, 31, 179, 182, 50, 48, 110, 86, 239, 96, 222, 125, 42, 173, 226, 193, 224, 130, 156, 37, 251, 216, 238, 40, 192, 180);
                    gx = this._getElement(gx);
                    break;
                case 32:
                    gx = new Array(0, 10, 6, 106, 190, 249, 167, 4, 67, 209, 138, 138, 32, 242, 123, 89, 27, 120, 185, 80, 156, 38, 69, 171, 60, 28, 222, 80, 52, 254, 185, 220, 241);
                    gx = this._getElement(gx);
                    break;
                case 34:
                    gx = new Array(0, 111, 77, 146, 94, 26, 21, 108, 19, 105, 94, 113, 193, 86, 140, 163, 125, 58, 158, 229, 239, 218, 103, 56, 70, 114, 61, 183, 129, 167, 13, 98, 62, 129, 51);
                    gx = this._getElement(gx);
                    break;
                case 36:
                    gx = new Array(0, 200, 183, 98, 16, 172, 31, 246, 234, 60, 152, 115, 0, 167, 152, 113, 248, 238, 107, 18, 63, 218, 37, 87, 210, 105, 177, 120, 74, 121, 196, 117, 251, 113, 233, 30, 120);
                    gx = this._getElement(gx);
                    break;
                case 40:
                    gx = new Array(0, 59, 116, 79, 161, 252, 98, 128, 205, 128, 161, 247, 57, 163, 56, 235, 106, 53, 26, 187, 174, 226, 104, 170, 7, 175, 35, 181, 114, 88, 41, 47, 163, 125, 134, 72, 20, 232, 53, 35, 15);
                    gx = this._getElement(gx);
                    break;
                case 42:
                    gx = new Array(0, 250, 103, 221, 230, 25, 18, 137, 231, 0, 3, 58, 242, 221, 191, 110, 84, 230, 8, 188, 106, 96, 147, 15, 131, 139, 34, 101, 223, 39, 101, 213, 199, 237, 254, 201, 123, 171, 162, 194, 117, 50, 96);
                    gx = this._getElement(gx);
                    break;
                case 44:
                    gx = new Array(0, 190, 7, 61, 121, 71, 246, 69, 55, 168, 188, 89, 243, 191, 25, 72, 123, 9, 145, 14, 247, 1, 238, 44, 78, 143, 62, 224, 126, 118, 114, 68, 163, 52, 194, 217, 147, 204, 169, 37, 130, 113, 102, 73, 181);
                    gx = this._getElement(gx);
                    break;
                case 46:
                    gx = new Array(0, 112, 94, 88, 112, 253, 224, 202, 115, 187, 99, 89, 5, 54, 113, 129, 44, 58, 16, 135, 216, 169, 211, 36, 1, 4, 96, 60, 241, 73, 104, 234, 8, 249, 245, 119, 174, 52, 25, 157, 224, 43, 202, 223, 19, 82, 15);
                    gx = this._getElement(gx);
                    break;
                case 48:
                    gx = new Array(0, 228, 25, 196, 130, 211, 146, 60, 24, 251, 90, 39, 102, 240, 61, 178, 63, 46, 123, 115, 18, 221, 111, 135, 160, 182, 205, 107, 206, 95, 150, 120, 184, 91, 21, 247, 156, 140, 238, 191, 11, 94, 227, 84, 50, 163, 39, 34, 108);
                    gx = this._getElement(gx);
                    break;
                case 50:
                    gx = new Array(0, 232, 125, 157, 161, 164, 9, 118, 46, 209, 99, 203, 193, 35, 3, 209, 111, 195, 242, 203, 225, 46, 13, 32, 160, 126, 209, 130, 160, 242, 215, 242, 75, 77, 42, 189, 32, 113, 65, 124, 69, 228, 114, 235, 175, 124, 170, 215, 232, 133, 205);
                    gx = this._getElement(gx);
                    break;
                case 52:
                    gx = new Array(0, 116, 50, 86, 186, 50, 220, 251, 89, 192, 46, 86, 127, 124, 19, 184, 233, 151, 215, 22, 14, 59, 145, 37, 242, 203, 134, 254, 89, 190, 94, 59, 65, 124, 113, 100, 233, 235, 121, 22, 76, 86, 97, 39, 242, 200, 220, 101, 33, 239, 254, 116, 51);
                    gx = this._getElement(gx);
                    break;
                case 54:
                    gx = new Array(0, 183, 26, 201, 87, 210, 221, 113, 21, 46, 65, 45, 50, 238, 184, 249, 225, 102, 58, 209, 218, 109, 165, 26, 95, 184, 192, 52, 245, 35, 254, 238, 175, 172, 79, 123, 25, 122, 43, 120, 108, 215, 80, 128, 201, 235, 8, 153, 59, 101, 31, 198, 76, 31, 156);
                    gx = this._getElement(gx);
                    break;
                case 56:
                    gx = new Array(0, 106, 120, 107, 157, 164, 216, 112, 116, 2, 91, 248, 163, 36, 201, 202, 229, 6, 144, 254, 155, 135, 208, 170, 209, 12, 139, 127, 142, 182, 249, 177, 174, 190, 28, 10, 85, 239, 184, 101, 124, 152, 206, 96, 23, 163, 61, 27, 196, 247, 151, 154, 202, 207, 20, 61, 10);
                    gx = this._getElement(gx);
                    break;
                case 58:
                    gx = new Array(0, 82, 116, 26, 247, 66, 27, 62, 107, 252, 182, 200, 185, 235, 55, 251, 242, 210, 144, 154, 237, 176, 141, 192, 248, 152, 249, 206, 85, 253, 142, 65, 165, 125, 23, 24, 30, 122, 240, 214, 6, 129, 218, 29, 145, 127, 134, 206, 245, 117, 29, 41, 63, 159, 142, 233, 125, 148, 123);
                    gx = this._getElement(gx);
                    break;
                case 60:
                    gx = new Array(0, 107, 140, 26, 12, 9, 141, 243, 197, 226, 197, 219, 45, 211, 101, 219, 120, 28, 181, 127, 6, 100, 247, 2, 205, 198, 57, 115, 219, 101, 109, 160, 82, 37, 38, 238, 49, 160, 209, 121, 86, 11, 124, 30, 181, 84, 25, 194, 87, 65, 102, 190, 220, 70, 27, 209, 16, 89, 7, 33, 240);
                    gx = this._getElement(gx);
                    break;
                case 62:
                    gx = new Array(0, 65, 202, 113, 98, 71, 223, 248, 118, 214, 94, 0, 122, 37, 23, 2, 228, 58, 121, 7, 105, 135, 78, 243, 118, 70, 76, 223, 89, 72, 50, 70, 111, 194, 17, 212, 126, 181, 35, 221, 117, 235, 11, 229, 149, 147, 123, 213, 40, 115, 6, 200, 100, 26, 246, 182, 218, 127, 215, 36, 186, 110, 106);
                    gx = this._getElement(gx);
                    break;
                case 64:
                    gx = new Array(0, 45, 51, 175, 9, 7, 158, 159, 49, 68, 119, 92, 123, 177, 204, 187, 254, 200, 78, 141, 149, 119, 26, 127, 53, 160, 93, 199, 212, 29, 24, 145, 156, 208, 150, 218, 209, 4, 216, 91, 47, 184, 146, 47, 140, 195, 195, 125, 242, 238, 63, 99, 108, 140, 230, 242, 31, 204, 11, 178, 243, 217, 156, 213, 231);
                    gx = this._getElement(gx);
                    break;
                case 66:
                    gx = new Array(0, 5, 118, 222, 180, 136, 136, 162, 51, 46, 117, 13, 215, 81, 17, 139, 247, 197, 171, 95, 173, 65, 137, 178, 68, 111, 95, 101, 41, 72, 214, 169, 197, 95, 7, 44, 154, 77, 111, 236, 40, 121, 143, 63, 87, 80, 253, 240, 126, 217, 77, 34, 232, 106, 50, 168, 82, 76, 146, 67, 106, 171, 25, 132, 93, 45, 105);
                    gx = this._getElement(gx);
                    break;
                case 68:
                    gx = new Array(0, 247, 159, 223, 33, 224, 93, 77, 70, 90, 160, 32, 254, 43, 150, 84, 101, 190, 205, 133, 52, 60, 202, 165, 220, 203, 151, 93, 84, 15, 84, 253, 173, 160, 89, 227, 52, 199, 97, 95, 231, 52, 177, 41, 125, 137, 241, 166, 225, 118, 2, 54, 32, 82, 215, 175, 198, 43, 238, 235, 27, 101, 184, 127, 3, 5, 8, 163, 238);
                    gx = this._getElement(gx);
                    break;
            }

            decimalValue = this._toDecimal(this.corCodeWords.DC, decimalValue);
            decimalRepresentation = this._divide(gx, decimalValue);
            ecw = this._toBinary(decimalRepresentation);
            return ecw;
        },

        /** 
         * Convert to binary.
         * @private
         */
        _toBinary: function (decimalRepresentation) {
            var toBinary = new Array(this.corCodeWords.ECCW);
            for (var i = 0; i < decimalRepresentation.length; i++) {
                var temp = decimalRepresentation[i].toString(2);
                var str = new String();
                for (var j = 0; j < (8 - temp.length) ; j++)
                    str += "0";
                toBinary[i] = str + temp;
            }
            return toBinary;
        },

        /** 
         * @private
         */
        _divide: function (gx, decimalValue) {
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            var eccWords = this.corCodeWords.ECCW;
            var messagePolynom = [];
            for (var i = 0; i < decimalValue.length; i++) {
                messagePolynom.push({
                Exponent: decimalValue.length - 1 - i,
                Coefficient: decimalValue[i]
            })
            }

            var generatorPolynom = [];
            for (var i = 0; i < gx.length; i++)
            {
                generatorPolynom.push({
                Exponent: gx.length - 1 - i,
                Coefficient: this._findElement(gx[i], alpha)
            })
            }

            for (var i = 0; i < messagePolynom.length; i++)
            {
                messagePolynom[i].Exponent = messagePolynom[i].Exponent + eccWords;
            }

            var genLeadtermFactor = messagePolynom[0].Exponent - generatorPolynom[0].Exponent;
            for (var i = 0; i < generatorPolynom.length; i++) {
                generatorPolynom[i].Coefficient = generatorPolynom[i].Coefficient;
                generatorPolynom[i].Exponent = generatorPolynom[i].Exponent + genLeadtermFactor;
            }

            var leadTermSource = messagePolynom;
            for (var i = 0; i < messagePolynom.length; i++)
            {
                if (leadTermSource[0].Coefficient == 0 && messagePolynom.length % 2 == 0) {
                    leadTermSource.splice(0, 1);
                }
                else {
                    var alphaNotation = this._convertToAlphaNotation(leadTermSource, alpha);
                    var resPoly = this._multiplyGeneratorPolynomByLeadterm(generatorPolynom, alphaNotation[0], i);
                    resPoly = this._convertToDecNotation(resPoly,alpha);
                    resPoly = this._XORPolynoms(leadTermSource, resPoly);
                    leadTermSource = resPoly;
                }

            }
            var returnArray = [];
            for (var i = 0; i < leadTermSource.length; i++) {
                returnArray.push(leadTermSource[i].Coefficient);
            }
            return returnArray;
        },

        /** 
         * @private
         */
         _convertToAlphaNotation: function(poly, alpha) {
             var newPoly = [];
             for (var i = 0; i < poly.length; i++) {
                 if (poly[i].Coefficient != 0) {
                     newPoly.push({
                         Exponent: poly[i].Exponent,
                         Coefficient: this._findElement(poly[i].Coefficient, alpha)
                     })
                 }
             }
             return newPoly;
         },

         /** 
         * @private
         */
         _multiplyGeneratorPolynomByLeadterm: function(genPolynom, leadTerm, lowerExponentBy) {
             var tempPolynom = [];
             for (var i = 0; i < genPolynom.length; i++) {
                 tempPolynom.push({
                     Exponent: genPolynom[i].Exponent - lowerExponentBy,
                     Coefficient: (genPolynom[i].Coefficient + leadTerm.Coefficient) % 255
             })
             }
             return tempPolynom;
         },

         /** 
         * @private
         */
         _convertToDecNotation: function(poly, alpha) {
             var tempPolynom = [];
             for (var i = 0; i < poly.length; i++) {
             tempPolynom.push({
                     Exponent: poly[i].Exponent,
                     Coefficient:  this._getIntValFromAlphaExp(poly[i].Coefficient, alpha)
             })
             }
             return tempPolynom;
         },

         /** 
         * @private
         */
         _getIntValFromAlphaExp: function(element, alpha) {
             if (element > 255)
            {
                element = element - 255;
            }
            return alpha[element];
         },
         
         
         /** 
         * @private
         */
         _XORPolynoms: function(messagePolynom, resPolynom) {
              var resultPolynom = [];
              var longPoly = [];
              var shortPoly = [];
            if (messagePolynom.length >= resPolynom.length)
            {
                longPoly = messagePolynom;
                shortPoly = resPolynom;
            }
            else
            {
                longPoly = resPolynom;
                shortPoly = messagePolynom;
            }
            
            for (var i = 0; i < longPoly.length; i++)
            {
            resultPolynom.push({
                     Exponent: messagePolynom[0].Exponent - i,
                     Coefficient:  longPoly[i].Coefficient ^ (shortPoly.length > i ? shortPoly[i].Coefficient : 0)
             })
            }
            resultPolynom.splice(0, 1);
            return resultPolynom;
         },

        /** 
         * @private
         */
        _findElement: function (element, alpha) {
            var j = 0;
            for (; j < alpha.length; j++) {
                if (element == alpha[j])
                    break;
            }
            return j;
        },

        /** 
         * @private
         */
        _getElement: function (element) {
            var gx = new Array(element.length);
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            for (var i = 0; i < element.length; i++) {
                if (element[i] > 255)
                    element[i] = element[i] - 255;
                gx[i] = alpha[element[i]];
            }
            return gx;
        },

        /** 
         * @private
         */
        _toDecimal: function (inString) {
            var decimalValue = new Array(inString.length);
            for (var i = 0; i < inString.length; i++)
                decimalValue[i] = parseInt(inString[i], 2);

            return decimalValue;
        },

        /** 
         * @private
         */
        _splitCodeWord: function (ds, blk, count) {
            count = parseInt(count);
            var ds1 = new Array(count);
            for (var i = 0; i < count; i++)
                ds1[i] = ds[blk][i];
            return ds1;
        },

        /** 
         * Create blocks for datamatrix barcode.
         * @private
         */
        _createBlocks: function (data, noOfBlocks) {
            var ret = this._create2DArray(noOfBlocks, data.length / 8 / noOfBlocks);
            var stringValue = "";
            var j = 0, i = 0, blockNumber = 0;

            for (; j < data.length; j++) {
                if (j % 8 == 0 && j != 0) {
                    ret[blockNumber][i] = stringValue;
                    stringValue = "";
                    i++;

                    if (i == (data.length / noOfBlocks / 8)) {
                        blockNumber++;
                        i = 0;
                    }
                }

                stringValue += data[j] ? 1 : 0;
            }
            ret[blockNumber][i] = stringValue;
            return ret;
        },

        /** 
         * Returns alphanumeric values for the character.
         * @private
         */
        _getAlphanumericvalues: function (charValue) {
            var valueInInt = -1;
            var alphaNumArray = new Array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', ' ', '$', '%', '*', '+', '-', '.', '/', ':');
            valueInInt = alphaNumArray.indexOf(charValue);
            return valueInInt;
        },

        /** 
         * @private
         */
        _stringToBoolArray: function (numberInString, noOfBits) {
            var numberInBool = new Array(noOfBits);

            var number = 0;
            for (var i = 0; i < numberInString.length; i++)
                number = number * 10 + numberInString.charCodeAt(i) - 48;

            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;

            return numberInBool;
        },

        /** 
         * @private
         */
        _intToBoolArray: function (number, noOfBits) {
            var numberInBool = new Array(noOfBits);
            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;
            return numberInBool;
        },

        /** 
         * @private
         */
        _drawPDP: function (x, y) {
            var i, j;
            for (i = x, j = y; i < x + 7; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 6].IsBlack = true;
                this.moduleValue[i][y + 6].IsFilled = true;
                this.moduleValue[i][y + 6].IsPDP = true;

                if (y + 7 < this.noOfModules) {
                    this.moduleValue[i][y + 7].IsBlack = false;
                    this.moduleValue[i][y + 7].IsFilled = true;
                    this.moduleValue[i][y + 7].IsPDP = true;
                }
                else if (y - 1 >= 0) {
                    this.moduleValue[i][y - 1].IsBlack = false;
                    this.moduleValue[i][y - 1].IsFilled = true;
                    this.moduleValue[i][y - 1].IsPDP = true;
                }

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 6][j].IsBlack = true;
                this.moduleValue[x + 6][j].IsFilled = true;
                this.moduleValue[x + 6][j].IsPDP = true;

                if (x + 7 < this.noOfModules) {
                    this.moduleValue[x + 7][j].IsBlack = false;
                    this.moduleValue[x + 7][j].IsFilled = true;
                    this.moduleValue[x + 7][j].IsPDP = true;
                }
                else if (x - 1 >= 0) {
                    this.moduleValue[x - 1][j].IsBlack = false;
                    this.moduleValue[x - 1][j].IsFilled = true;
                    this.moduleValue[x - 1][j].IsPDP = true;
                }
            }

            if (x + 7 < this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x + 7][y + 7].IsBlack = false;
                this.moduleValue[x + 7][y + 7].IsFilled = true;
                this.moduleValue[x + 7][y + 7].IsPDP = true;
            }
            else if (x + 7 < this.noOfModules && y + 7 >= this.noOfModules) {
                this.moduleValue[x + 7][y - 1].IsBlack = false;
                this.moduleValue[x + 7][y - 1].IsFilled = true;
                this.moduleValue[x + 7][y - 1].IsPDP = true;
            }
            else if (x + 7 >= this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x - 1][y + 7].IsBlack = false;
                this.moduleValue[x - 1][y + 7].IsFilled = true;
                this.moduleValue[x - 1][y + 7].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 5; i++, j++) {
                this.moduleValue[i][y].IsBlack = false;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 4].IsBlack = false;
                this.moduleValue[i][y + 4].IsFilled = true;
                this.moduleValue[i][y + 4].IsPDP = true;

                this.moduleValue[x][j].IsBlack = false;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 4][j].IsBlack = false;
                this.moduleValue[x + 4][j].IsFilled = true;
                this.moduleValue[x + 4][j].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 3; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;
                this.moduleValue[i][y + 2].IsPDP = true;

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
                this.moduleValue[x + 2][j].IsPDP = true;
            }
            this.moduleValue[x + 1][y + 1].IsBlack = true;
            this.moduleValue[x + 1][y + 1].IsFilled = true;
            this.moduleValue[x + 1][y + 1].IsPDP = true;
        },

        /** 
         * @private
         */
        _drawAlignmentPattern: function (x, y) {
            var i, j;
            for (i = x - 2, j = y - 2; i < x + 3; i++, j++) {
                this.moduleValue[i][y - 2].IsBlack = true;
                this.moduleValue[i][y - 2].IsFilled = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;

                this.moduleValue[x - 2][j].IsBlack = true;
                this.moduleValue[x - 2][j].IsFilled = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
            }

            for (i = x - 1, j = y - 1; i < x + 2; i++, j++) {
                this.moduleValue[i][y - 1].IsBlack = false;
                this.moduleValue[i][y - 1].IsFilled = true;

                this.moduleValue[i][y + 1].IsBlack = false;
                this.moduleValue[i][y + 1].IsFilled = true;

                this.moduleValue[x - 1][j].IsBlack = false;
                this.moduleValue[x - 1][j].IsFilled = true;

                this.moduleValue[x + 1][j].IsBlack = false;
                this.moduleValue[x + 1][j].IsFilled = true;
            }
            this.moduleValue[x][y].IsBlack = true;
            this.moduleValue[x][y].IsFilled = true;
        },

        /** 
         * @private
         */
        _getAlignmentPatternCoOrdinates: function () {
            var allign = new Array();
            switch (this.m_Version) {
                case 02:
                    allign = new Array(6, 18);
                    break;
                case 03:
                    allign = new Array(6, 22);
                    break;
                case 04:
                    allign = new Array(6, 26);
                    break;
                case 05:
                    allign = new Array(6, 30);
                    break;
                case 06:
                    allign = new Array(6, 34);
                    break;
                case 07:
                    allign = new Array(6, 22, 38);
                    break;
                case 08:
                    allign = new Array(6, 24, 42);
                    break;
                case 09:
                    allign = new Array(6, 26, 46);
                    break;
                case 10:
                    allign = new Array(6, 28, 50);
                    break;
                case 11:
                    allign = new Array(6, 30, 54);
                    break;
                case 12:
                    allign = new Array(6, 32, 58);
                    break;
                case 13:
                    allign = new Array(6, 34, 62);
                    break;
                case 14:
                    allign = new Array(6, 26, 46, 66);
                    break;
                case 15:
                    allign = new Array(6, 26, 48, 70);
                    break;
                case 16:
                    allign = new Array(6, 26, 50, 74);
                    break;
                case 17:
                    allign = new Array(6, 30, 54, 78);
                    break;
                case 18:
                    allign = new Array(6, 30, 56, 82);
                    break;
                case 19:
                    allign = new Array(6, 30, 58, 86);
                    break;
                case 20:
                    allign = new Array(6, 34, 62, 90);
                    break;
                case 21:
                    allign = new Array(6, 28, 50, 72, 94);
                    break;
                case 22:
                    allign = new Array(6, 26, 50, 74, 98);
                    break;
                case 23:
                    allign = new Array(6, 30, 54, 78, 102);
                    break;
                case 24:
                    allign = new Array(6, 28, 54, 80, 106);
                    break;
                case 25:
                    allign = new Array(6, 32, 58, 84, 110);
                    break;
                case 26:
                    allign = new Array(6, 30, 58, 86, 114);
                    break;
                case 27:
                    allign = new Array(6, 34, 62, 90, 118);
                    break;
                case 28:
                    allign = new Array(6, 26, 50, 74, 98, 122);
                    break;
                case 29:
                    allign = new Array(6, 30, 54, 78, 102, 126);
                    break;
                case 30:
                    allign = new Array(6, 26, 52, 78, 104, 130);
                    break;
                case 31:
                    allign = new Array(6, 30, 56, 82, 108, 134);
                    break;
                case 32:
                    allign = new Array(6, 34, 60, 86, 112, 138);
                    break;
                case 33:
                    allign = new Array(6, 30, 58, 86, 114, 142);
                    break;
                case 34:
                    allign = new Array(6, 34, 62, 90, 118, 146);
                    break;
                case 35:
                    allign = new Array(6, 30, 54, 78, 102, 126, 150);
                    break;
                case 36:
                    allign = new Array(6, 24, 50, 76, 102, 128, 154);
                    break;
                case 37:
                    allign = new Array(6, 28, 54, 80, 106, 132, 158);
                    break;
                case 38:
                    allign = new Array(6, 32, 58, 84, 110, 136, 162);
                    break;
                case 39:
                    allign = new Array(6, 26, 54, 82, 110, 138, 166);
                    break;
                case 40:
                    allign = new Array(6, 30, 58, 86, 114, 142, 170);
                    break;
            }
            return allign;
        },

        /** 
         * @private
         */
        _findECINumber: function (text) {
            var num = 3;
            for (var i = 0; i < text.length; i++) {
                var cCode = text.charCodeAt(i);
                if (cCode >= 32 && cCode <= 255) {
                    continue;
                }

                var check = cCode.toString(16);
                var CP437CharSet = new Array("2591", "2592", "2593", "2502", "2524", "2561", "2562", "2556", "2555", "2563", "2551", "2557", "255D", "255C", "255B", "2510", "2514", "2534", "252C", "251C", "2500", "253C", "255E", "255F", "255A", "2554", "2569", "2566", "2560", "2550", "256C", "2567", "2568", "2564", "2565", "2559", "2558", "2552", "2553", "256B", "256A", "2518", "250C", "2588", "2584", "258C", "2590", "2580", "25A0");
                var latin2CharSet = new Array("104", "2D8", "141", "13D", "15A", "160", "15E", "164", "179", "17D", "17B", "105", "2DB", "142", "13E", "15B", "2C7", "161", "15F", "165", "17A", "2DD", "17E", "17C", "154", "102", "139", "106", "10C", "118", "11A", "10E", "110", "143", "147", "150", "158", "16E", "170", "162", "155", "103", "13A", "107", "10D", "119", "11B", "10F", "111", "144", "148", "151", "159", "16F", "171", "163", "2D9");
                var latin3CharSet = new Array("126", "124", "130", "15E", "11E", "134", "17B", "127", "125", "131", "15F", "11F", "135", "17C", "10A", "108", "120", "11C", "16C", "15C", "10B", "109", "121", "11D", "16D", "15D");
                var latin4CharSet = new Array("104", "138", "156", "128", "13B", "160", "112", "122", "166", "17D", "105", "2DB", "157", "129", "13C", "2C7", "161", "113", "123", "167", "14A", "17E", "14B", "100", "12E", "10C", "118", "116", "12A", "110", "145", "14C", "136", "172", "168", "16A", "101", "12F", "10D", "119", "117", "12B", "111", "146", "14D", "137", "173", "169", "16B");
                var windows1250CharSet = new Array("141", "104", "15E", "17B", "142", "105", "15F", "13D", "13E", "17C");
                var windows1251CharSet = new Array("402", "403", "453", "409", "40A", "40C", "40B", "40F", "452", "459", "45A", "45C", "45B", "45F", "40E", "45E", "408", "490", "401", "404", "407", "406", "456", "491", "451", "454", "458", "405", "455", "457");
                var windows1252CharSet = new Array("20AC", "201A", "192", "201E", "2026", "2020", "2021", "2C6", "2030", "160", "2039", "152", "17D", "2018", "2019", "201C", "201D", "2022", "2013", "2014", "2DC", "2122", "161", "203A", "153", "17E", "178");
                var windows1256CharSet = new Array("67E", "679", "152", "686", "698", "688", "6AF", "6A9", "691", "153", "6BA", "6BE", "6C1");

                /** Check for CP437*/
                if (CP437CharSet.indexOf(check) != -1) {
                    num = 2;
                    break;
                }
                    /** Check for ISO/IEC 8859-2*/
                else if (latin2CharSet.indexOf(check) != -1) {
                    num = 4;
                    break;
                }
                    /** Check for ISO/IEC 8859-3*/
                else if (latin3CharSet.indexOf(check) != -1) {
                    num = 5;
                    break;
                }
                    /** Check for ISO/IEC 8859-4*/
                else if (latin4CharSet.indexOf(check) != -1) {
                    num = 6;
                    break;
                }
                    /** Check for ISO/IEC 8859-5*/
                else if (check >= 1025 && check <= 1119 && check != 1037 && check != 1104 && check != 1117) {
                    num = 7;
                    break;
                }
                    /** Check for ISO/IEC 8859-6*/
                else if ((check >= 1569 && check <= 1594) || (check >= 1600 && check <= 1618) || check == 1567 || check == 1563 || check == 1548) {
                    num = 8;
                    break;
                }
                    /** Check for ISO/IEC 8859-7*/
                else if ((check >= 900 && check <= 974) || check == 890) {
                    num = 9;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 1488 && check <= 1514) {
                    num = 10;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 3585 && check <= 3675) {
                    num = 13;
                    break;
                }
                    /** Check for Windows1250*/
                else if ((windows1250CharSet.indexOf(check) != 1) || (check >= 1569 && check <= 1610)) {
                    num = 21;
                    break;
                }
                    /** Check for Windows1251*/
                else if ((windows1251CharSet.indexOf(check) != 1) || (check >= 1040 && check <= 1103)) {
                    num = 22;
                    break;
                }
                    /** Check for Windows1252*/
                else if (windows1252CharSet.indexOf(check) != -1) {
                    num = 23;
                    break;
                }
                    /** Check for Windows1256*/
                else if (windows1256CharSet.indexOf(check) != -1) {
                    num = 24;
                    break;
                }
            }
            return num;
        },

        /** 
         * @private
         */
        _getNumberOfDataCodeWord: function () {
            var countOfDataCodeWord = 0;

            switch (this.m_Version) {
                case 01:
                    countOfDataCodeWord = 16;
                    break;
                case 02:
                    countOfDataCodeWord = 28;
                    break;
                case 03:
                    countOfDataCodeWord = 44;
                    break;
                case 04:
                    countOfDataCodeWord = 64;
                    break;
                case 05:
                    countOfDataCodeWord = 86;
                    break;
                case 06:
                    countOfDataCodeWord = 108;
                    break;
                case 07:
                    countOfDataCodeWord = 124;
                    break;
                case 08:
                    countOfDataCodeWord = 154;
                    break;
                case 09:
                    countOfDataCodeWord = 182;
                    break;
                case 10:
                    countOfDataCodeWord = 216;
                    break;
                case 11:
                    countOfDataCodeWord = 254;
                    break;
                case 12:
                    countOfDataCodeWord = 290;
                    break;
                case 13:
                    countOfDataCodeWord = 334;
                    break;
                case 14:
                    countOfDataCodeWord = 365;
                    break;
                case 15:
                    countOfDataCodeWord = 415;
                    break;
                case 16:
                    countOfDataCodeWord = 453;
                    break;
                case 17:
                    countOfDataCodeWord = 507;
                    break;
                case 18:
                    countOfDataCodeWord = 563;
                    break;
                case 19:
                    countOfDataCodeWord = 627;
                    break;
                case 20:
                    countOfDataCodeWord = 669;
                    break;
                case 21:
                    countOfDataCodeWord = 714;
                    break;
                case 22:
                    countOfDataCodeWord = 782;
                    break;
                case 23:
                    countOfDataCodeWord = 860;
                    break;
                case 24:
                    countOfDataCodeWord = 914;
                    break;
                case 25:
                    countOfDataCodeWord = 1000;
                    break;
                case 26:
                    countOfDataCodeWord = 1062;
                    break;
                case 27:
                    countOfDataCodeWord = 1128;
                    break;
                case 28:
                    countOfDataCodeWord = 1193;
                    break;
                case 29:
                    countOfDataCodeWord = 1267;
                    break;
                case 30:
                    countOfDataCodeWord = 1373;
                    break;
                case 31:
                    countOfDataCodeWord = 1455;
                    break;
                case 32:
                    countOfDataCodeWord = 1541;
                    break;
                case 33:
                    countOfDataCodeWord = 1631;
                    break;
                case 34:
                    countOfDataCodeWord = 1725;
                    break;
                case 35:
                    countOfDataCodeWord = 1812;
                    break;
                case 36:
                    countOfDataCodeWord = 1914;
                    break;
                case 37:
                    countOfDataCodeWord = 1992;
                    break;
                case 38:
                    countOfDataCodeWord = 2102;
                    break;
                case 39:
                    countOfDataCodeWord = 2216;
                    break;
                case 40:
                    countOfDataCodeWord = 2334;
                    break;
            }

            return countOfDataCodeWord;
        },

        /** 
         * Returns the number of error correcting codewords based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectingCodeWords: function () {
            var index = (this.m_Version - 1) * 4;
            index += 1;
            var numberOfErrorCorrectingCodeWords = new Array(7, 10, 13, 17, 10, 16, 22, 28, 15, 26, 36, 44, 20, 36, 52, 64, 26, 48, 72, 88, 36, 64, 96, 112, 40, 72, 108, 130, 48, 88, 132, 156, 60, 110, 160, 192, 72, 130, 192, 224, 80, 150, 224, 264, 96, 176, 260, 308, 104, 198, 288, 352, 120, 216, 320, 384, 132, 240, 360, 432, 144, 280, 408, 480, 168, 308, 448, 532, 180, 338, 504, 588, 196, 364, 546, 650, 224, 416, 600, 700, 224, 442, 644, 750, 252, 476, 690, 816, 270, 504, 750, 900, 300, 560, 810, 960, 312, 588, 870, 1050, 336, 644, 952, 1110, 360, 700, 1020, 1200, 390, 728, 1050, 1260, 420, 784, 1140, 1350, 450, 812, 1200, 1440, 480, 868, 1290, 1530, 510, 924, 1350, 1620, 540, 980, 1440, 1710, 570, 1036, 1530, 1800, 570, 1064, 1590, 1890, 600, 1120, 1680, 1980, 630, 1204, 1770, 2100, 660, 1260, 1860, 2220, 720, 1316, 1950, 2310, 750, 1372, 2040, 2430);
            return numberOfErrorCorrectingCodeWords[index];
        },

        /** 
         * Returns the number of error correction blocks based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectionBlocks: function () {
            var numberOfErrorCorrectionBlocks = new Array();
            switch (this.m_Version) {
                case 01:
                case 02:
                case 03:
                    numberOfErrorCorrectionBlocks[0] = 1;
                    break;
                case 04:
                case 05:
                    numberOfErrorCorrectionBlocks[0] = 2;
                    break;
                case 06:
                case 07:
                    numberOfErrorCorrectionBlocks[0] = 4;
                    break;
                case 08:
                    numberOfErrorCorrectionBlocks = new Array(2, 60, 38, 2, 61, 39);
                    break;
                case 09:
                    numberOfErrorCorrectionBlocks = new Array(3, 58, 36, 2, 59, 37);
                    break;
                case 10:
                    numberOfErrorCorrectionBlocks = new Array(4, 69, 43, 1, 70, 44);
                    break;
                case 11:
                    numberOfErrorCorrectionBlocks = new Array(1, 80, 50, 4, 81, 51);
                    break;
                case 12:
                    numberOfErrorCorrectionBlocks = new Array(6, 58, 36, 2, 59, 37);
                    break;
                case 13:
                    numberOfErrorCorrectionBlocks = new Array(8, 59, 37, 1, 60, 38);
                    break;
                case 14:
                    numberOfErrorCorrectionBlocks = new Array(4, 64, 40, 5, 65, 41);
                    break;
                case 15:
                    numberOfErrorCorrectionBlocks = new Array(5, 65, 41, 5, 66, 42);
                    break;
                case 16:
                    numberOfErrorCorrectionBlocks = new Array(7, 73, 45, 3, 74, 46);
                    break;
                case 17:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 1, 75, 47);
                    break;
                case 18:
                    numberOfErrorCorrectionBlocks = new Array(9, 69, 43, 4, 70, 44);
                    break;
                case 19:
                    numberOfErrorCorrectionBlocks = new Array(3, 70, 44, 11, 71, 45);
                    break;
                case 20:
                    numberOfErrorCorrectionBlocks = new Array(3, 67, 41, 13, 68, 42);
                    break;
                case 21:
                case 22:
                    numberOfErrorCorrectionBlocks[0] = 17;
                    break;
                case 23:
                    numberOfErrorCorrectionBlocks = new Array(4, 75, 47, 14, 76, 48);
                    break;
                case 24:
                    numberOfErrorCorrectionBlocks = new Array(6, 73, 45, 14, 74, 46);
                    break;
                case 25:
                    numberOfErrorCorrectionBlocks = new Array(8, 75, 47, 13, 76, 48);
                    break;
                case 26:
                    numberOfErrorCorrectionBlocks = new Array(19, 74, 46, 4, 75, 47);
                    break;
                case 27:
                    numberOfErrorCorrectionBlocks = new Array(22, 73, 45, 3, 74, 46);
                    break;
                case 28:
                    numberOfErrorCorrectionBlocks = new Array(3, 73, 45, 23, 74, 46);
                    break;
                case 29:
                    numberOfErrorCorrectionBlocks = new Array(21, 73, 45, 7, 74, 46);
                    break;
                case 30:
                    numberOfErrorCorrectionBlocks = new Array(19, 75, 47, 10, 76, 48);
                    break;
                case 31:
                    numberOfErrorCorrectionBlocks = new Array(2, 74, 46, 29, 75, 47);
                    break;
                case 32:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 23, 75, 47);
                    break;
                case 33:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 21, 75, 47);
                    break;
                case 34:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 23, 75, 47);
                    break;
                case 35:
                    numberOfErrorCorrectionBlocks = new Array(12, 75, 47, 26, 76, 48);
                    break;
                case 36:
                    numberOfErrorCorrectionBlocks = new Array(6, 75, 47, 34, 76, 48);
                    break;
                case 37:
                    numberOfErrorCorrectionBlocks = new Array(29, 74, 46, 14, 75, 47);
                    break;
                case 38:
                    numberOfErrorCorrectionBlocks = new Array(13, 74, 46, 32, 75, 47);
                    break;
                case 39:
                    numberOfErrorCorrectionBlocks = new Array(40, 75, 47, 7, 76, 48);
                    break;
                case 40:
                    numberOfErrorCorrectionBlocks = new Array(18, 75, 47, 31, 76, 48);
                    break;
            }

            return numberOfErrorCorrectionBlocks;
        },

        /** 
         * Returns the end values based on qr version
         * @private
         */
        _getEnd: function () {
            var endValues = new Array(208, 359, 567, 807, 1079, 1383, 1568, 1936, 2336, 2768, 3232, 3728, 4256, 4651, 5243, 5867, 6523, 7211, 7931, 8683, 9252, 10068, 10916, 11796, 12708, 13652, 14628, 15371, 16411, 17483, 18587, 19723, 20891, 22091, 23008, 24272, 25568, 26896, 28256, 29648);
            return endValues[this.m_Version - 1];
        },

        /** 
         * Returns the data capacity based on QR version
         * @private
         */
        _getDataCapacity: function () {
            var dataCapacityValues = new Array(26, 44, 70, 100, 134, 172, 196, 242, 292, 346, 404, 466, 532, 581, 655, 733, 815, 901, 991, 1085, 1156, 1258, 1364, 1474, 1588, 1706, 1828, 1921, 2051, 2185, 2323, 2465, 2611, 2761, 2876, 3034, 3196, 3362, 3532, 3706);
            return dataCapacityValues[this.m_Version - 1];
        },

        /** 
         * @private
         */
        _getFormatInformation: function () {
            var formatInformation = new Array(1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1);
            return formatInformation;
        },

        /** 
         * Returns the QR version
         * @private
         */
        _getVersionInformation: function () {
            var versionInformation = new Array();
            switch (this.m_Version) {
                case 07:
                    versionInformation = new Array(0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0);
                    break;
                case 08:
                    versionInformation = new Array(0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0);
                    break;
                case 09:
                    versionInformation = new Array(1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0);
                    break;
                case 10:
                    versionInformation = new Array(1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0);
                    break;
                case 11:
                    versionInformation = new Array(0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0);
                    break;
                case 12:
                    versionInformation = new Array(0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0);
                    break;
                case 13:
                    versionInformation = new Array(1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0);
                    break;
                case 14:
                    versionInformation = new Array(1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0);
                    break;
                case 15:
                    versionInformation = new Array(0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0);
                    break;
                case 16:
                    versionInformation = new Array(0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0);
                    break;
                case 17:
                    versionInformation = new Array(1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0);
                    break;
                case 18:
                    versionInformation = new Array(1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0);
                    break;
                case 19:
                    versionInformation = new Array(0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0);
                    break;
                case 20:
                    versionInformation = new Array(0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0);
                    break;
                case 21:
                    versionInformation = new Array(1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0);
                    break;
                case 22:
                    versionInformation = new Array(1, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0);
                    break;
                case 23:
                    versionInformation = new Array(0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0);
                    break;
                case 24:
                    versionInformation = new Array(0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0);
                    break;
                case 25:
                    versionInformation = new Array(1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0);
                    break;
                case 26:
                    versionInformation = new Array(1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0);
                    break;
                case 27:
                    versionInformation = new Array(0, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0);
                    break;
                case 28:
                    versionInformation = new Array(0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0);
                    break;
                case 29:
                    versionInformation = new Array(1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0);
                    break;
                case 30:
                    versionInformation = new Array(1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0);
                    break;
                case 31:
                    versionInformation = new Array(0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0);
                    break;
                case 32:
                    versionInformation = new Array(1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1);
                    break;
                case 33:
                    versionInformation = new Array(0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1);
                    break;
                case 34:
                    versionInformation = new Array(0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1);
                    break;
                case 35:
                    versionInformation = new Array(1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1);
                    break;
                case 36:
                    versionInformation = new Array(1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1);
                    break;
                case 37:
                    versionInformation = new Array(0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1);
                    break;
                case 38:
                    versionInformation = new Array(0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1);
                    break;
                case 39:
                    versionInformation = new Array(1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1);
                    break;
                case 40:
                    versionInformation = new Array(1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1);
                    break;
            }

            return versionInformation;
        },

        /** 
         * Returns the code points of the text
         * @private
         */
        _getStringCodePoints: (function () {
            function surrogatePairToCodePoint(charCode1, charCode2) {
                return ((charCode1 & 0x3FF) << 10) + (charCode2 & 0x3FF) + 0x10000;
            }

            /** Read string in character by character and create an array of code points.*/
            return function (str) {
                var codePoints = [], i = 0, charCode;
                while (i < str.length) {
                    charCode = str.charCodeAt(i);
                    if ((charCode & 0xF800) == 0xD800)
                        codePoints.push(surrogatePairToCodePoint(charCode, str.charCodeAt(++i)));
                    else
                        codePoints.push(charCode);
                    ++i;
                }
                return codePoints;
            }
        })(),

        /** 
         * Creates 2 dimensional array.
         * @private
         */
        _create2DArray: function (rows, columns) {
            rows = parseInt(rows);
            columns = parseInt(columns);
            var twoDArray = new Array(rows);

            for (var i = 0; i < rows; i++)
                twoDArray[i] = new Array(columns);

            return twoDArray;
        },

        /** 
         * Builds datamatrix barcode.
         * @private
         */
        _buildDataMatrix: function () {
            var symbolAttributes = [[10, 10, 1, 1, 3, 5, 1, 3], [12, 12, 1, 1, 5, 7, 1, 5], [14, 14, 1, 1, 8, 10, 1, 8], [16, 16, 1, 1, 12, 12, 1, 12], [18, 18, 1, 1, 18, 14, 1, 18], [20, 20, 1, 1, 22, 18, 1, 22], [22, 22, 1, 1, 30, 20, 1, 30], [24, 24, 1, 1, 36, 24, 1, 36], [26, 26, 1, 1, 44, 28, 1, 44], [32, 32, 2, 2, 62, 36, 1, 62], [36, 36, 2, 2, 86, 42, 1, 86], [40, 40, 2, 2, 114, 48, 1, 114], [44, 44, 2, 2, 144, 56, 1, 144], [48, 48, 2, 2, 174, 68, 1, 174], [52, 52, 2, 2, 204, 84, 2, 102], [64, 64, 4, 4, 280, 112, 2, 140], [72, 72, 4, 4, 368, 144, 4, 92], [80, 80, 4, 4, 456, 192, 4, 114], [88, 88, 4, 4, 576, 224, 4, 144], [96, 96, 4, 4, 696, 272, 4, 174], [104, 104, 4, 4, 816, 336, 6, 136], [120, 120, 6, 6, 1050, 408, 6, 175], [132, 132, 6, 6, 1304, 496, 8, 163], [144, 144, 6, 6, 1558, 620, 10, 156], [8, 18, 1, 1, 5, 7, 1, 5], [8, 32, 2, 1, 10, 11, 1, 10], [12, 26, 1, 1, 16, 14, 1, 16], [12, 36, 2, 1, 22, 18, 1, 22], [16, 36, 2, 1, 32, 24, 1, 32], [16, 48, 2, 1, 49, 28, 1, 49]];
            var codeWords = this._getStringCodePoints(this.Model.text);
            var correctionCodewords;
            var symAttr;

            /** ASCII Encoder.*/
            var encodedPoints = new Array(); var n = 0;
            for (var e = 0; e < codeWords.length; e++) {
                if (codeWords[e] < 127)
                    encodedPoints[n] = codeWords[e] + 1;
                else {
                    encodedPoints[n++] = 235;
                    encodedPoints[n] = codeWords[e] - 127;
                }
                n++;
            }

            /** Find symbol attribute.*/
            for (var e = 0; e < symbolAttributes.length; e++) {
                if (symbolAttributes[e][4] >= codeWords.length) {
                    symAttr = symbolAttributes[e];
                    break;
                }
            }

            if (symAttr[4] > encodedPoints.length) { //pad codewords
                var l = encodedPoints.length;
                var temp = new Array(symAttr[4]);
                var t = 0;

                for (var i = 0; i < l; i++)
                    temp[i] = encodedPoints[i];

                if (l < (symAttr[4] + 1))
                    temp[l] = 129; // denotes padding.

                l++;
                while (l < symAttr[4]) {	// more padding
                    var v = 129 + (((l + 1) * 149) % 253) + 1;
                    if (v > 254)
                        v -= 254;
                    temp[l++] = v;
                }

                encodedPoints = temp;
            }

            correctionCodewords = symAttr[5];
            errCorrArray = this._errorCorrection(correctionCodewords, encodedPoints);

            var finalCodeWord = encodedPoints.concat(errCorrArray);
            var dataMatrixArray = this._createMatrix(finalCodeWord, symAttr);

            var w = dataMatrixArray.length * this.Model.xDimension;
            var h = dataMatrixArray[0].length * this.Model.xDimension;         

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw DataMatrix.*/
            this._drawDataMatrix(dataMatrixArray, w, h);
        },

        /** 
         * Draws datamatrix barcode.
         * @private
         */
        _drawDataMatrix: function (dmArray, w, h) {
            
            var dim = parseInt(this.Model.xDimension);
            
            if(dmArray.length == dmArray[0].length && this.Model.height != "" && this.Model.width != "")
            {
                this.Model.height = this._getProperValue(this.Model.height);
                this.Model.width = this._getProperValue(this.Model.width);
            
                var dimension = Math.min(this.Model.width, this.Model.height);
            
                dim = dimension / dmArray.length;
            }
			else
			{
			this.Model.height = h;
			this.Model.width = w;
		    this.canvasEl[0].setAttribute("width", this.Model.width);
           this.BarcodeEl.css("width", this.Model.width);
			this.canvasEl[0].setAttribute("height", this.Model.height);
           this.BarcodeEl.css("height", this.Model.height);
			}
			
            var y = 0;

            this.contextEl.beginPath();

            for (var i = 0; i < dmArray.length; i++) {
                var x = 0;
                for (var j = 0; j < dmArray[i].length; j++) {
                    var colorBrush = null;
                    if (dmArray[i][j] == 1)
                        colorBrush = this.model.darkBarColor;
						else
						colorBrush = this.model.lightBarColor;
                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)
                    x += dim;
                }
                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Creates matrix.
         * @private
         */
        _createMatrix: function (codeWord, symAttr) {
            var x, y, NC, NR;
            var places = new Array();

            var W = symAttr[1], H = symAttr[0];
            var FW = W / symAttr[2], FH = H / symAttr[3];

            NC = W - 2 * (W / FW);
            NR = H - 2 * (H / FH);

            var places = new Array(NC * NR);
            this._ecc200placement(places, NR, NC);
            var matrix = new Array(W * H);

            for (var i = 0; i < matrix.length; i++)
                matrix[i] = 0;

            for (y = 0; y < H; y += FH) {
                for (x = 0; x < W; x++)
                    matrix[parseInt(y * W) + x] = 1;
                for (x = 0; x < W; x += 2)
                    matrix[parseInt((y + FH - 1) * W) + x] = 1;
            }
            for (x = 0; x < W; x += FW) {
                for (y = 0; y < H; y++)
                    matrix[parseInt(y * W) + x] = 1;
                for (y = 0; y < H; y += 2)
                    matrix[parseInt(y * W) + x + FW - 1] = 1;
            }
            for (y = 0; y < NR; y++) {
                for (x = 0; x < NC; x++) {
                    var v = places[((NR - y - 1) * NC) + x];
                    if (v == 1 || v > 7 && (codeWord[(v >> 3) - 1] & (1 << (v & 7))) != 0) {
                        var inde = parseInt((parseInt(1 + y + (2 * parseInt(y / (FH - 2)))) * W) + 1 + x + +(2 * parseInt(x / (FW - 2))));
                        matrix[inde] = 1;
                    }
                }
            }

            var _w = symAttr[1], _h = symAttr[0];

            var tempArray = this._create2DArray(_w, _h);

            for (var x1 = 0; x1 < _w; x1++) {
                for (var y1 = 0; y1 < _h; y1++) {
                    tempArray[x1][y1] = matrix[(_w * y1) + x1];
                }
            }

            /** Rotate the array left.*/
            var tempArray2 = this._create2DArray(_h, _w);

            for (var i = 0; i < _h; i++) {
                for (var j = 0; j < _w; j++) {
                    tempArray2[_h - 1 - i][j] = tempArray[j][i];
                }
            }

            var w = symAttr[0] + (this.Model.quietZone.top + this.Model.quietZone.bottom);
            var h = symAttr[1] + (this.Model.quietZone.left + this.Model.quietZone.right);
            var quietZone = this.Model.quietZone.all;

            var dataMatrixArray = this._create2DArray(w, h);

            for (var i = 0; i < w; i++)
                for (var j = 0; j < h; j++)
                    dataMatrixArray[i][j] = 0;

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    dataMatrixArray[i][j] = tempArray2[i - quietZone][j - quietZone];
                }
            }

            return dataMatrixArray;
        },

        /** 
         * @private
         */
        _ecc200placement: function (array, NR, NC) {
            var r, c, p;
            // invalidate
            for (r = 0; r < NR; r++)
                for (c = 0; c < NC; c++)
                    array[r * NC + c] = 0;
            // start
            p = 1;
            r = 4;
            c = 0;
            do {
                // check corner
                if (r == NR && !(c != 0))
                    this._ecc200placementcornerA(array, NR, NC, p++);
                if ((r == NR - 2) && !(c != 0) && ((NC % 4) != 0))
                    this._ecc200placementcornerB(array, NR, NC, p++);
                if (r == NR - 2 && !(c != 0) && (NC % 8) == 4)
                    this._ecc200placementcornerC(array, NR, NC, p++);
                if (r == NR + 4 && c == 2 && !((NC % 8) != 0))
                    this._ecc200placementcornerD(array, NR, NC, p++);
                // up/right
                do {
                    if (r < NR && c >= 0 && !(array[r * NC + c] != 0)) {
                        this._ecc200placementblock(array, NR, NC, r, c, p++);
                    }
                    r -= 2;
                    c += 2;
                }
                while (r >= 0 && c < NC);
                r++;
                c += 3;
                // down/left
                do {
                    if (r > -1) {
                        if (c < NC && !(array[r * NC + c] != 0)) {
                            this._ecc200placementblock(array, NR, NC, r, c, p++);
                        }
                    }
                    r += 2;
                    c -= 2;
                }
                while (r < NR && c >= 0);
                r += 3;
                c++;
            }
            while (r < NR || c < NC);
            // unfilled corner
            if (!(array[NR * NC - 1] != 0))
                array[NR * NC - 1] = array[NR * NC - NC - 2] = 1;
        },

        /** 
         * @private
         */
        _ecc200placementcornerA: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, 1, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 2, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerB: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 4, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 3);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerC: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerD: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, NC - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 3, p, 2);
            this._ecc200placementbit(array, NR, NC, 1, NC - 2, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementblock: function (array, NR, NC, r, c, p) {
            this._ecc200placementbit(array, NR, NC, r - 2, c - 2, p, 7);
            this._ecc200placementbit(array, NR, NC, r - 2, c - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 2, p, 5);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 1, p, 4);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 0, p, 3);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 0, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementbit: function (array, NR, NC, r, c, p, b) {
            if (r < 0) {
                r += NR;
                c += 4 - ((NR + 4) % 8);
            }
            if (c < 0) {
                c += NC;
                r += 4 - ((NC + 4) % 8);
            }
            array[r * NC + c] = (p << 3) + b;
        },

        /** 
         * Adds error correction codeword.
         * @private
         */
        _errorCorrection: function (num, codeWords) {
            var i, k, index = 1;
            var rsPolynomial;
            var temp = new Array(num + 1);

            /** Create log array.*/
            {
                var m_log = new Array(256);
                var m_aLog = new Array(256);

                m_log[0] = -255;
                m_aLog[0] = 1;

                for (var i = 1; i <= 255; i++) {
                    m_aLog[i] = m_aLog[i - 1] * 2;

                    if (m_aLog[i] >= 256)
                        m_aLog[i] = m_aLog[i] ^ 301;

                    m_log[m_aLog[i]] = i;
                }
            }

            /** CreateRSPolynomial.*/
            temp[0] = 1;
            for (i = 1; i <= num; i++) {
                temp[i] = 1;
                for (k = i - 1; k > 0; k--) {
                    if (temp[k] != 0)
                        temp[k] = m_aLog[(m_log[temp[k]] + index) % 255];
                    temp[k] ^= temp[k - 1];
                }
                temp[0] = m_aLog[(m_log[temp[0]] + index) % 255];
                index++;
            }

            rsPolynomial = new Array(num);
            for (var e = 0; e < rsPolynomial.length; e++)
                rsPolynomial[e] = temp[e];

            var t = 0;

            /** Create error correction array.*/
            var errCorrArray = new Array(num);

            for (var i = 0; i < num; i++)
                errCorrArray[i] = 0;

            for (var i = 0; i < codeWords.length; i++) {
                t = errCorrArray[num - 1] ^ codeWords[i];
                for (var j = num - 1; j > 0; j--) {
                    if (t != 0 && rsPolynomial[j] != 0)
                        errCorrArray[j] = (errCorrArray[j - 1] ^ m_aLog[(m_log[t] + m_log[rsPolynomial[j]]) % 255]);
                    else
                        errCorrArray[j] = errCorrArray[j - 1];
                }
                if (t != 0 && rsPolynomial[0] != 0)
                    errCorrArray[0] = (m_aLog[(m_log[t] + m_log[rsPolynomial[0]]) % 255]);
                else
                    errCorrArray[0] = 0;
            }

            errCorrArray = errCorrArray.reverse();
            return errCorrArray;
        },

        /** 
         * Validates the text against the barcode type.
         * @private
         */
        _validateText: function (str, exp) {
            var patt = new RegExp(exp);
            return patt.test(str);
        },

        /** 
         * @private
         */
        _getCharWidth: function (encodedText, symbolTable) {
            var charW = 0;
            var charArray = encodedText.split("");
            for (var e = 0; e < charArray.length; e++)
                charW = charW + intercharacterGap + this._getWidth(symbolTable, charArray[e]);

            return charW;
        },

        /** 
         * @private
         */
        _getWidth: function (symbolTable, c) {
            var check = false;
            var wid = 0;
            for (var e = 0; e < symbolTable.length; e++) {
                if (symbolTable[e][0] == c) {
                    var bars = symbolTable[e][2];
                    for (var i = 0; i < bars.length; i++) {
                        if (check == false) {
                            if (bars.length % 2 != 0)
                                continuous = true;
                        }

                        wid = wid + (bars[i] * this.Model.narrowBarWidth);
                        check = true;
                    }
                    return wid;
                }
            }
            return wid;
        },

        /** 
         * Draws 1 dimensional barcode.
         * @private
         */
        _draw1DBarcode: function (encodedText, symbolTable, w) {
            if (symbolTable.length > 0) {
                var charArray = encodedText.split("");
                var left = this.Model.quietZone.left;
                var top = this.Model.quietZone.top;

                var textSize = parseInt(this.textFont.size.replace('px', ''));
				if(this.Model.height == "" && this.Model.width == "")
				{
				this.Model.height = top + this.Model.barHeight + this.Model.quietZone.bottom;
				this.Model.width = w;
                if (this.Model.displayText) {
                    this.Model.height += (textSize + this.Model.barcodeToTextGapHeight);
                }
				 this.canvasEl[0].setAttribute("width", this.Model.width);
                this.BarcodeEl.css("width", this.Model.width);
                this.canvasEl[0].setAttribute("height", this.Model.height);
                this.BarcodeEl.css("height", this.Model.height);
				}
				
                var wid = this.Model.width;
                var hei = this.Model.height;

                //find number of bars here
                var totalBars = 0;
                for (var e = 0; e < charArray.length; e++) {
                    for (var h = 0; h < symbolTable.length; h++) {
                        if (symbolTable[h][0] == charArray[e]) {
                            var bars = symbolTable[h][2];
                            for (var i = 0; i < bars.length; i++) {
                                var barWidth = bars[i] * this.Model.narrowBarWidth;
                                totalBars += bars[i];
                                
                            }
                            if (bars.length % 2 != 0)
                              totalBars++;
                        }
                    }
                }
                
                var narrowBarWid = (this._getProperValue(this.Model.width) - this.Model.quietZone.right - this.Model.quietZone.left) / (totalBars);
                if (narrowBarWid < 1) {
                    narrowBarWid = 1;
                }
                this.Model.narrowBarWidth = Math.floor(narrowBarWid);
                // return the decimal value of narrobarwidth.
                var decimalValue = (narrowBarWid % 1).toFixed(3);
                var totalBarsWidth = totalBars * decimalValue;
                left = parseInt(this.model.quietZone.left) + Math.round(totalBarsWidth / 2);
          
                this.model.barHeight = this._getProperValue(this.Model.height) - this.Model.quietZone.top - this.Model.quietZone.bottom -(textSize + this.Model.barcodeToTextGapHeight);
                if (this.model.barHeight < 0) this.model.barHeight = 0;
                intercharacterGap = this.Model.narrowBarWidth;
                this.contextEl.fillStyle = 'white';
                this.contextEl.fillRect(0, 0, wid, hei);

                for (var e = 0; e < charArray.length; e++) {
                    for (var h = 0; h < symbolTable.length; h++) {
                        if (symbolTable[h][0] == charArray[e]) {
                            var bars = symbolTable[h][2];
                            if (this.model.symbologyType == "upcbarcode") {
                                for (var i = 0; i < bars.length; i++) {
                                    var barWidth = bars[i] * this.Model.narrowBarWidth;
                                    if (e > 1 && e <= 7) {
                                       
                                        if (i % 2 != 0) {

                                            this.contextEl.beginPath();
                                            this.contextEl.fillStyle = this.Model.darkBarColor;
                                            this.contextEl.fillRect(left, top, barWidth, this.Model.barHeight)
                                            this.contextEl.closePath();
                                        }                                      
                                    }
                                    else 
                                    {
                                        if(i % 2 == 0)
                                        {
                                            this.contextEl.beginPath();
                                            this.contextEl.fillStyle = this.Model.darkBarColor;
                                            this.contextEl.fillRect(left, top, barWidth, this.Model.barHeight)
                                            this.contextEl.closePath();
                                        }
                                        else {
                                           
                                                this.contextEl.beginPath();
                                                this.contextEl.fillStyle = this.model.lightBarColor;
                                                this.contextEl.fillRect(left, top, barWidth, this.model.barHeight);
                                          
                                        }
                                    }                                  
                                    left += barWidth;
                                    if(e == 1&& i==bars.length-1)
                                    {
                                        left -= barWidth;
                                    }
                                   
                                }
                            }
                            else {
                                for (var i = 0; i < bars.length; i++) {
                                    var barWidth = bars[i] * this.Model.narrowBarWidth;
                                    if (i % 2 == 0) {
                                        this.contextEl.beginPath();
                                        this.contextEl.fillStyle = this.Model.darkBarColor;
                                        this.contextEl.fillRect(left, top, barWidth, this.Model.barHeight)
                                        this.contextEl.closePath();
                                    }
                                    else {
                                        this.contextEl.beginPath();
                                        this.contextEl.fillStyle = this.model.lightBarColor;
                                        this.contextEl.fillRect(left, top, barWidth, this.model.barHeight);
                                        this.contextEl.closePath();
                                    }
                                    left += barWidth;                  
                                }
                            }
                            if (bars.length % 2 != 0 || this.model.symbologyType=="upcbarcode" && e==7)
                                left += intercharacterGap;
                        }
                    }
                }

                

                //this.Model.width = left += (this.Model.quietZone.right);

                /** Draws text for 1D barcodes.*/
                if (this.Model.displayText) {
                    var txtF = (this.textFont.fontStyle.toLowerCase() == "regular") ? "" : +this.textFont.fontStyle.toLowerCase() + " ";
                    txtF += this.textFont.size + " " + this.textFont.fontFamily.toLowerCase() + ",sans-serif";
                    this.contextEl.font = txtF;

                    var textWidth = 0;
                    textWidth = this.contextEl.measureText(this.Model.text).width;
                    if (this.Model.width < textWidth) {
                        //this.Model.width = textWidth;
                        this.canvasEl[0].setAttribute("width", this.Model.width);
                        this.BarcodeEl.css("width", this.Model.width);
                    }

                    this.contextEl.beginPath();
                    this.contextEl.fillStyle = this.Model.textColor;
                    this.contextEl.textAlign = "center";
                    hei = hei - textSize + textSize / 2;
                    if (this.model.symbologyType == "upcbarcode")
                    {
                        var data = encodedText.split("");
                        var txt = '';
                        for (var k = 0; k < data.length; k++)
                        {
                            if (k > 1 && k <= 7 || (k > 8 && k <= 14))
                                txt += data[k];
                        }
                        this.contextEl.fillText(txt, (this.Model.width / 2), hei);
                    }
                    else
                    this.contextEl.fillText(this.Model.text, (this.Model.width / 2), hei);
                    this.contextEl.closePath();
                    this.contextEl.stroke();
                }
            }
        }
    });

    /**
     * Enum for Symbol type.
	 * @enum {string}
     * @global
     */
    ej.datavisualization.Barcode.SymbologyType = {
        /** Specifies Code 39 Barcode. */
        code39: "code39",
        /** Specifies Code 39 Extended Barcode. */
        code39Extended: "code39extended",
        /** Specifies Code 11 Barcode. */
        code11: "code11",
        /** Specifies Codabar Barcode. */
        codabar: "codabar",
        /** Specifies Code 32 Barcode. */
        code32: "code32",
        /** Specifies Code 93 Barcode. */
        code93: "code93",
        /** Specifies Code 93 Extended Barcode. */
        code93Extended: "code93extended",
        /** Specifies Code 128 A Barcode. */
        code128A: "code128a",
        /** Specifies Code 128 B Barcode. */
        code128B: "code128b",
        /** Specifies Code 128 C Barcode. */
        code128C: "code128c",
        /** Specifies DataMatrix Barcode. */
        dataMatrix: "datamatrix",
        /** Specifies QR Barcode. */
        qrBarcode: "qrbarcode",
        /** Specifies UPC-A Barcode. */
        upca:"upcbarcode"
    };

    /**
     * Enum for Symbol type.
     * @readonly
     * @private
     */
    ej.datavisualization.Barcode.symbologySettings = {
        /** Specifies symbology settinngs for the Code 39 Barcode. */
        code39: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 39 Extended Barcode. */
        code39extended: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x00-\x7F]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]],
            extendedCodes: [['\0', ['%', 'U']], [String.fromCharCode(parseInt('0001', 16)), ['$', 'A']], [String.fromCharCode(parseInt('0002', 16)), ['$', 'B']], [String.fromCharCode(parseInt('0003', 16)), ['$', 'C']], [String.fromCharCode(parseInt('0004', 16)), ['$', 'D']], [String.fromCharCode(parseInt('0005', 16)), ['$', 'E']], [String.fromCharCode(parseInt('0006', 16)), ['$', 'F']], ['\a', ['$', 'G']], ['\b', ['$', 'H']], ['\t', ['$', 'I']], ['\n', ['$', 'J']], ['\v', ['$', 'K']], ['\f', ['$', 'L']], ['\r', ['$', 'M']], [String.fromCharCode(parseInt('000e', 16)), ['$', 'N']], [String.fromCharCode(parseInt('000f', 16)), ['$', 'O']], [String.fromCharCode(parseInt('0010', 16)), ['$', 'P']], [String.fromCharCode(parseInt('0011', 16)), ['$', 'Q']], [String.fromCharCode(parseInt('0012', 16)), ['$', 'R']], [String.fromCharCode(parseInt('0013', 16)), ['$', 'S']], [String.fromCharCode(parseInt('0014', 16)), ['$', 'T']], [String.fromCharCode(parseInt('0015', 16)), ['$', 'U']], [String.fromCharCode(parseInt('0016', 16)), ['$', 'V']], [String.fromCharCode(parseInt('0017', 16)), ['$', 'W']], [String.fromCharCode(parseInt('0018', 16)), ['$', 'X']], [String.fromCharCode(parseInt('0019', 16)), ['$', 'Y']], [String.fromCharCode(parseInt('001a', 16)), ['$', 'Z']], [String.fromCharCode(parseInt('001b', 16)), ['%', 'A']], [String.fromCharCode(parseInt('001c', 16)), ['%', 'B']], [String.fromCharCode(parseInt('001d', 16)), ['%', 'C']], [String.fromCharCode(parseInt('001e', 16)), ['%', 'D']], [String.fromCharCode(parseInt('001f', 16)), ['%', 'E']], [' ', [' ']], ['!', ['/', 'A']], ['"', ['/', 'B']], ['#', ['/', 'C']], ['$', ['/', 'D']], ['%', ['/', 'E']], ['&', ['/', 'F']], ['\'', ['/', 'G']], ['(', ['/', 'H']], [')', ['/', 'I']], ['*', ['/', 'J']], ['+', ['/', 'K']], [',', ['/', 'L']], ['-', ['/', 'M']], ['.', ['/', 'N']], ['/', ['/', 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', ['/', 'Z']], [';', ['%', 'F']], ['<', ['%', 'G']], ['=', ['%', 'H']], ['>', ['%', 'I']], ['?', ['%', 'J']], ['@', ['%', 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', ['%', 'K']], ['\\', ['%', 'L']], [']', ['%', 'M']], ['^', ['%', 'N']], ['_', ['%', 'O']], ['`', ['%', 'W']], ['a', ['+', 'A']], ['b', ['+', 'B']], ['c', ['+', 'C']], ['d', ['+', 'D']], ['e', ['+', 'E']], ['f', ['+', 'F']], ['g', ['+', 'G']], ['h', ['+', 'H']], ['i', ['+', 'I']], ['j', ['+', 'J']], ['k', ['+', 'K']], ['l', ['+', 'L']], ['m', ['+', 'M']], ['n', ['+', 'N']], ['o', ['+', 'O']], ['p', ['+', 'P']], ['q', ['+', 'Q']], ['r', ['+', 'R']], ['s', ['+', 'S']], ['t', ['+', 'T']], ['u', ['+', 'U']], ['v', ['+', 'V']], ['w', ['+', 'W']], ['x', ['+', 'X']], ['y', ['+', 'Y']], ['z', ['+', 'Z']], ['{', ['%', 'P']], ['|', ['%', 'Q']], ['}', ['%', 'R']], ['~', ['%', 'S']], [String.fromCharCode(parseInt('007f', 16)), ['%', 'T']]]
        },
        /** Specifies symbology settings for the Code 11 Barcode. */
        code11: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[0-9\-]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 2]], ["1", 1, [2, 1, 1, 1, 2]], ["2", 2, [1, 2, 1, 1, 2]], ["3", 3, [2, 2, 1, 1, 1]], ["4", 4, [1, 1, 2, 1, 2]], ["5", 5, [2, 1, 2, 1, 1]], ["6", 6, [1, 2, 2, 1, 1]], ["7", 7, [1, 1, 1, 2, 2]], ["8", 8, [2, 1, 1, 2, 1]], ["9", 9, [2, 1, 1, 1, 1]], ["-", 10, [1, 1, 2, 1, 1]], ["*", 0, [1, 1, 2, 2, 1]]]
        },
        /** Specifies symbology settings for the Codabar Barcode. */
        codabar: {
            startSymbol: 'A',
            stopSymbol: 'B',
            validatorExp: "^[0-9\-\.\$\/\:\+]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 1, 2, 2]], ["1", 0, [1, 1, 1, 1, 2, 2, 1]], ["2", 0, [1, 1, 1, 2, 1, 1, 2]], ["3", 0, [2, 2, 1, 1, 1, 1, 1]], ["4", 0, [1, 1, 2, 1, 1, 2, 1]], ["5", 0, [2, 1, 1, 1, 1, 2, 1]], ["6", 0, [1, 2, 1, 1, 1, 1, 2]], ["7", 0, [1, 2, 1, 1, 2, 1, 1]], ["8", 0, [1, 2, 2, 1, 1, 1, 1]], ["9", 0, [2, 1, 1, 2, 1, 1, 1]], ["-", 0, [1, 1, 1, 2, 2, 1, 1]], ["$", 0, [1, 1, 2, 2, 1, 1, 1]], [":", 0, [2, 1, 1, 1, 2, 1, 2]], ["/", 0, [2, 1, 2, 1, 1, 1, 2]], [".", 0, [2, 1, 2, 1, 2, 1, 1]], ["+", 0, [1, 1, 2, 1, 2, 1, 2]], ["A", 0, [1, 1, 2, 2, 1, 2, 1]], ["B", 0, [1, 1, 1, 2, 1, 2, 2]]]
        },
        /** Specifies symbology settings for the Code 32 Barcode. */
        code32: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\*\.\$\/\+\%]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["B", 10, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 11, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 12, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["F", 13, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 14, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 15, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["J", 0x10, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 0x11, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 0x12, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 0x13, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 20, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["P", 0x15, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 0x16, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 0x17, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 0x18, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 0x19, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 0x1a, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 0x1b, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 0x1c, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 0x1d, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 30, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 0x1f, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Barcode. */
        code93: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x41-\x5A\x30-\x39\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Extended Barcode. */
        code93extended: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F\x00fb\x00fd\x00fe\'þ'\'ü'\'ý']+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]],
            extendedCodes: [['\0', [String.fromCharCode(parseInt('00fc', 16)), 'U']], [String.fromCharCode(parseInt('0001', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'A']], [String.fromCharCode(parseInt('0002', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'B']], [String.fromCharCode(parseInt('0003', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'C']], [String.fromCharCode(parseInt('0004', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'D']], [String.fromCharCode(parseInt('0005', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'E']], [String.fromCharCode(parseInt('0006', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'F']], ['\a', [String.fromCharCode(parseInt('00fb', 16)), 'G']], ['\b', [String.fromCharCode(parseInt('00fb', 16)), 'H']], ['\t', [String.fromCharCode(parseInt('00fb', 16)), 'I']], ['\n', [String.fromCharCode(parseInt('00fb', 16)), 'J']], ['\v', [String.fromCharCode(parseInt('00fb', 16)), 'K']], ['\f', [String.fromCharCode(parseInt('00fb', 16)), 'L']], ['\r', [String.fromCharCode(parseInt('00fb', 16)), 'M']], [String.fromCharCode(parseInt('000e', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'N']], [String.fromCharCode(parseInt('000f', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'O']], [String.fromCharCode(parseInt('0010', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'P']], [String.fromCharCode(parseInt('0011', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Q']], [String.fromCharCode(parseInt('0012', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'R']], [String.fromCharCode(parseInt('0013', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'S']], [String.fromCharCode(parseInt('0014', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'T']], [String.fromCharCode(parseInt('0015', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'U']], [String.fromCharCode(parseInt('0016', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'V']], [String.fromCharCode(parseInt('0017', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'W']], [String.fromCharCode(parseInt('0018', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'X']], [String.fromCharCode(parseInt('0019', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Y']], [String.fromCharCode(parseInt('001a', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Z']], [String.fromCharCode(parseInt('001b', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'A']], [String.fromCharCode(parseInt('001c', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'B']], [String.fromCharCode(parseInt('001d', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'C']], [String.fromCharCode(parseInt('001e', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'D']], [String.fromCharCode(parseInt('001f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'E']], [' ', [' ']], ['!', [String.fromCharCode(parseInt('00fd', 16)), 'A']], ['"', [String.fromCharCode(parseInt('00fd', 16)), 'B']], ['#', [String.fromCharCode(parseInt('00fd', 16)), 'C']], ['$', [String.fromCharCode(parseInt('00fd', 16)), 'D']], ['%', [String.fromCharCode(parseInt('00fd', 16)), 'E']], ['&', [String.fromCharCode(parseInt('00fd', 16)), 'F']], ['\'', [String.fromCharCode(parseInt('00fd', 16)), 'G']], ['(', [String.fromCharCode(parseInt('00fd', 16)), 'H']], [')', [String.fromCharCode(parseInt('00fd', 16)), 'I']], ['*', [String.fromCharCode(parseInt('00fd', 16)), 'J']], ['+', [String.fromCharCode(parseInt('00fd', 16)), 'K']], [',', [String.fromCharCode(parseInt('00fd', 16)), 'L']], ['-', [String.fromCharCode(parseInt('00fd', 16)), 'M']], ['.', [String.fromCharCode(parseInt('00fd', 16)), 'N']], ['/', [String.fromCharCode(parseInt('00fd', 16)), 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', [String.fromCharCode(parseInt('00fd', 16)), 'Z']], [';', [String.fromCharCode(parseInt('00fc', 16)), 'F']], ['<', [String.fromCharCode(parseInt('00fc', 16)), 'G']], ['=', [String.fromCharCode(parseInt('00fc', 16)), 'H']], ['>', [String.fromCharCode(parseInt('00fc', 16)), 'I']], ['?', [String.fromCharCode(parseInt('00fc', 16)), 'J']], ['@', [String.fromCharCode(parseInt('00fc', 16)), 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', [String.fromCharCode(parseInt('00fc', 16)), 'K']], ['\\', [String.fromCharCode(parseInt('00fc', 16)), 'L']], [']', [String.fromCharCode(parseInt('00fc', 16)), 'M']], ['^', [String.fromCharCode(parseInt('00fc', 16)), 'N']], ['_', [String.fromCharCode(parseInt('00fc', 16)), 'O']], ['`', [String.fromCharCode(parseInt('00fc', 16)), 'W']], ['a', [String.fromCharCode(parseInt('00fe', 16)), 'A']], ['b', [String.fromCharCode(parseInt('00fe', 16)), 'B']], ['c', [String.fromCharCode(parseInt('00fe', 16)), 'C']], ['d', [String.fromCharCode(parseInt('00fe', 16)), 'D']], ['e', [String.fromCharCode(parseInt('00fe', 16)), 'E']], ['f', [String.fromCharCode(parseInt('00fe', 16)), 'F']], ['g', [String.fromCharCode(parseInt('00fe', 16)), 'G']], ['h', [String.fromCharCode(parseInt('00fe', 16)), 'H']], ['i', [String.fromCharCode(parseInt('00fe', 16)), 'I']], ['j', [String.fromCharCode(parseInt('00fe', 16)), 'J']], ['k', [String.fromCharCode(parseInt('00fe', 16)), 'K']], ['l', [String.fromCharCode(parseInt('00fe', 16)), 'L']], ['m', [String.fromCharCode(parseInt('00fe', 16)), 'M']], ['n', [String.fromCharCode(parseInt('00fe', 16)), 'N']], ['o', [String.fromCharCode(parseInt('00fe', 16)), 'O']], ['p', [String.fromCharCode(parseInt('00fe', 16)), 'P']], ['q', [String.fromCharCode(parseInt('00fe', 16)), 'Q']], ['r', [String.fromCharCode(parseInt('00fe', 16)), 'R']], ['s', [String.fromCharCode(parseInt('00fe', 16)), 'S']], ['t', [String.fromCharCode(parseInt('00fe', 16)), 'T']], ['u', [String.fromCharCode(parseInt('00fe', 16)), 'U']], ['v', [String.fromCharCode(parseInt('00fe', 16)), 'V']], ['w', [String.fromCharCode(parseInt('00fe', 16)), 'W']], ['x', [String.fromCharCode(parseInt('00fe', 16)), 'X']], ['y', [String.fromCharCode(parseInt('00fe', 16)), 'Y']], ['z', [String.fromCharCode(parseInt('00fe', 16)), 'Z']], ['{', [String.fromCharCode(parseInt('00fc', 16)), 'P']], ['|', [String.fromCharCode(parseInt('00fc', 16)), 'Q']], ['}', [String.fromCharCode(parseInt('00fc', 16)), 'R']], ['~', [String.fromCharCode(parseInt('00fc', 16)), 'S']], [String.fromCharCode(parseInt('007f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'T']]],
        },
        /** Specifies symbology settings for the Code 128 A Barcode. */
        code128a: {
            startSymbol: String.fromCharCode(parseInt('00f9', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[\0\x0001\x0002\x0003\x0004\x0005\x0006\a\b\t\n\v\f\r\x000e\x000f\x0010\x0011\x0012\x0013\x0014\x0015\x0016\x0017\x0018\x0019\x001a\x001b\x001c\x001d\x001e\x001f !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_\x00f0\x00f1\x00f2\x00f3\x00f4]",
            symbolTable: [["\0", 0x40, [1, 1, 1, 4, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 0x41, [1, 2, 1, 1, 2, 4]], [String.fromCharCode(parseInt('0002', 16)), 0x42, [1, 2, 1, 4, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 0x43, [1, 4, 1, 1, 2, 2]], [String.fromCharCode(parseInt('0004', 16)), 0x44, [1, 4, 1, 2, 2, 1]], [String.fromCharCode(parseInt('0005', 16)), 0x45, [1, 1, 2, 2, 1, 4]], [String.fromCharCode(parseInt('0006', 16)), 70, [1, 1, 2, 4, 1, 2]], ["\a", 0x47, [1, 2, 2, 1, 1, 4]], ["\b", 0x48, [1, 2, 2, 4, 1, 1]], ["\t", 0x49, [1, 4, 2, 1, 1, 2]], ["\n", 0x4a, [1, 4, 2, 2, 1, 1]], ["\v", 0x4b, [2, 4, 1, 2, 1, 1]], ["\f", 0x4c, [2, 2, 1, 1, 1, 4]], ["\r", 0x4d, [4, 1, 3, 1, 1, 1]], [String.fromCharCode(parseInt('000e', 16)), 0x4e, [2, 4, 1, 1, 1, 2]], [String.fromCharCode(parseInt('000f', 16)), 0x4f, [1, 3, 4, 1, 1, 1]], [String.fromCharCode(parseInt('0010', 16)), 80, [1, 1, 1, 2, 4, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x51, [1, 2, 1, 1, 4, 2]], [String.fromCharCode(parseInt('0012', 16)), 0x52, [1, 2, 1, 2, 4, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x53, [1, 1, 4, 2, 1, 2]], [String.fromCharCode(parseInt('0014', 16)), 0x54, [1, 2, 4, 1, 1, 2]], [String.fromCharCode(parseInt('0015', 16)), 0x55, [1, 2, 4, 2, 1, 1]], [String.fromCharCode(parseInt('0016', 16)), 0x56, [4, 1, 1, 2, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x57, [4, 2, 1, 1, 1, 2]], [String.fromCharCode(parseInt('0018', 16)), 0x58, [4, 2, 1, 2, 1, 1]], [String.fromCharCode(parseInt('0019', 16)), 0x59, [2, 1, 2, 1, 4, 1]], [String.fromCharCode(parseInt('001a', 16)), 90, [2, 1, 4, 1, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x5b, [4, 1, 2, 1, 2, 1]], [String.fromCharCode(parseInt('001c', 16)), 0x5c, [1, 1, 1, 1, 4, 3]], [String.fromCharCode(parseInt('001d', 16)), 0x5d, [1, 1, 1, 3, 4, 1]], [String.fromCharCode(parseInt('001e', 16)), 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('001f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f9', 16)), 0x67, [2, 1, 1, 4, 1, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 B Barcode. */
        code128b: {
            startSymbol: String.fromCharCode(parseInt('00fd', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F]",
            symbolTable: [[" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], ["`", 0x40, [1, 1, 1, 4, 2, 2]], ["a", 0x41, [1, 2, 1, 1, 2, 4]], ["b", 0x42, [1, 2, 1, 4, 2, 1]], ["c", 0x43, [1, 4, 1, 1, 2, 2]], ["d", 0x44, [1, 4, 1, 2, 2, 1]], ["e", 0x45, [1, 1, 2, 2, 1, 4]], ["f", 70, [1, 1, 2, 4, 1, 2]], ["g", 0x47, [1, 2, 2, 1, 1, 4]], ["h", 0x48, [1, 2, 2, 4, 1, 1]], ["i", 0x49, [1, 4, 2, 1, 1, 2]], ["j", 0x4a, [1, 4, 2, 2, 1, 1]], ["k", 0x4b, [2, 4, 1, 2, 1, 1]], ["l", 0x4c, [2, 2, 1, 1, 1, 4]], ["m", 0x4d, [4, 1, 3, 1, 1, 1]], ["n", 0x4e, [2, 4, 1, 1, 1, 2]], ["o", 0x4f, [1, 3, 4, 1, 1, 1]], ["p", 80, [1, 1, 1, 2, 4, 2]], ["q", 0x51, [1, 2, 1, 1, 4, 2]], ["r", 0x52, [1, 2, 1, 2, 4, 1]], ["s", 0x53, [1, 1, 4, 2, 1, 2]], ["t", 0x54, [1, 2, 4, 1, 1, 2]], ["u", 0x55, [1, 2, 4, 2, 1, 1]], ["v", 0x56, [4, 1, 1, 2, 1, 2]], ["w", 0x57, [4, 2, 1, 1, 1, 2]], ["x", 0x58, [4, 2, 1, 2, 1, 1]], ["y", 0x59, [2, 1, 2, 1, 4, 1]], ["z", 90, [2, 1, 4, 1, 2, 1]], ["{", 0x5b, [4, 1, 2, 1, 2, 1]], ["|", 0x5c, [1, 1, 1, 1, 4, 3]], ["}", 0x5d, [1, 1, 1, 3, 4, 1]], ["~", 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('007f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x68, [2, 1, 1, 2, 1, 4]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 C Barcode. */
        code128c: {
            startSymbol: String.fromCharCode(parseInt('00fe', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[0-9]",
            symbolTable: [["\0", 0, [2, 1, 2, 2, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 1, [2, 2, 2, 1, 2, 2]], [String.fromCharCode(parseInt('0002', 16)), 2, [2, 2, 2, 2, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 3, [1, 2, 1, 2, 2, 3]], [String.fromCharCode(parseInt('0004', 16)), 4, [1, 2, 1, 3, 2, 2]], [String.fromCharCode(parseInt('0005', 16)), 5, [1, 3, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0006', 16)), 6, [1, 2, 2, 2, 1, 3]], ["\a", 7, [1, 2, 2, 3, 1, 2]], ["\b", 8, [1, 3, 2, 2, 1, 2]], ["\t", 9, [2, 2, 1, 2, 1, 3]], ["\n", 10, [2, 2, 1, 3, 1, 2]], ["\v", 11, [2, 3, 1, 2, 1, 2]], ["\f", 12, [1, 1, 2, 2, 3, 2]], ["\r", 13, [1, 2, 2, 1, 3, 2]], [String.fromCharCode(parseInt('000e', 16)), 14, [1, 2, 2, 2, 3, 1]], [String.fromCharCode(parseInt('000f', 16)), 15, [1, 1, 3, 2, 2, 2]], [String.fromCharCode(parseInt('0010', 16)), 0x10, [1, 2, 3, 1, 2, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x11, [1, 2, 3, 2, 2, 1]], [String.fromCharCode(parseInt('0012', 16)), 0x12, [2, 2, 3, 2, 1, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x13, [2, 2, 1, 1, 3, 2]], [String.fromCharCode(parseInt('0014', 16)), 20, [2, 2, 1, 2, 3, 1]], [String.fromCharCode(parseInt('0015', 16)), 0x15, [2, 1, 3, 2, 1, 2]], [String.fromCharCode(parseInt('0016', 16)), 0x16, [2, 2, 3, 1, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x17, [3, 1, 2, 1, 3, 1]], [String.fromCharCode(parseInt('0018', 16)), 0x18, [3, 1, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0019', 16)), 0x19, [3, 2, 1, 1, 2, 2]], [String.fromCharCode(parseInt('001a', 16)), 0x1a, [3, 2, 1, 2, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x1b, [3, 1, 2, 2, 1, 2]], [String.fromCharCode(parseInt('001c', 16)), 0x1c, [3, 2, 2, 1, 1, 2]], [String.fromCharCode(parseInt('001d', 16)), 0x1d, [3, 2, 2, 2, 1, 1]], [String.fromCharCode(parseInt('001e', 16)), 30, [2, 1, 2, 1, 2, 3]], [String.fromCharCode(parseInt('001f', 16)), 0x1f, [2, 1, 2, 3, 2, 1]], [" ", 0x20, [2, 3, 2, 1, 2, 1]], ["!", 0x21, [1, 1, 1, 3, 2, 3]], ["\"", 0x22, [1, 3, 1, 1, 2, 3]], ["#", 0x23, [1, 3, 1, 3, 2, 1]], ["$", 0x24, [1, 1, 2, 3, 1, 3]], ["%", 0x25, [1, 3, 2, 1, 1, 3]], ["&", 0x26, [1, 3, 2, 3, 1, 1]], ["\'", 0x27, [2, 1, 1, 3, 1, 3]], ["[", 40, [2, 3, 1, 1, 1, 3]], [")", 0x29, [2, 3, 1, 3, 1, 1]], ["*", 0x2a, [1, 1, 2, 1, 3, 3]], ["+", 0x2b, [1, 1, 2, 3, 3, 1]], [",", 0x2c, [1, 3, 2, 1, 3, 1]], ["-", 0x2d, [1, 1, 3, 1, 2, 3]], [".", 0x2e, [1, 1, 3, 3, 2, 1]], ["/", 0x2f, [1, 3, 3, 1, 2, 1]], ["0", 0x30, [3, 1, 3, 1, 2, 1]], ["1", 0x31, [2, 1, 1, 3, 3, 1]], ["2", 50, [2, 3, 1, 1, 3, 1]], ["3", 0x33, [2, 1, 3, 1, 1, 3]], ["4", 0x34, [2, 1, 3, 3, 1, 1]], ["5", 0x35, [2, 1, 3, 1, 3, 1]], ["6", 0x36, [3, 1, 1, 1, 2, 3]], ["7", 0x37, [3, 1, 1, 3, 2, 1]], ["8", 0x38, [3, 3, 1, 1, 2, 1]], ["9", 0x39, [3, 1, 2, 1, 1, 3]], [":", 0x3a, [3, 1, 2, 3, 1, 1]], [";", 0x3b, [3, 3, 2, 1, 1, 1]], ["<", 60, [3, 1, 4, 1, 1, 1]], ["=", 0x3d, [2, 2, 1, 4, 1, 1]], [">", 0x3e, [4, 3, 1, 1, 1, 1]], ["?", 0x3f, [1, 1, 1, 2, 2, 4]], ["@", 0x40, [1, 1, 1, 4, 2, 2]], ["A", 0x41, [1, 2, 1, 1, 2, 4]], ["B", 0x42, [1, 2, 1, 4, 2, 1]], ["C", 0x43, [1, 4, 1, 1, 2, 2]], ["D", 0x44, [1, 4, 1, 2, 2, 1]], ["E", 0x45, [1, 1, 2, 2, 1, 4]], ["F", 70, [1, 1, 2, 4, 1, 2]], ["G", 0x47, [1, 2, 2, 1, 1, 4]], ["H", 0x48, [1, 2, 2, 4, 1, 1]], ["I", 0x49, [1, 4, 2, 1, 1, 2]], ["J", 0x4a, [1, 4, 2, 2, 1, 1]], ["K", 0x4b, [2, 4, 1, 2, 1, 1]], ["L", 0x4c, [2, 2, 1, 1, 1, 4]], ["M", 0x4d, [4, 1, 3, 1, 1, 1]], ["N", 0x4e, [2, 4, 1, 1, 1, 2]], ["O", 0x4f, [1, 3, 4, 1, 1, 1]], ["P", 80, [1, 1, 1, 2, 4, 2]], ["Q", 0x51, [1, 2, 1, 1, 4, 2]], ["R", 0x52, [1, 2, 1, 2, 4, 1]], ["S", 0x53, [1, 1, 4, 2, 1, 2]], ["T", 0x54, [1, 2, 4, 1, 1, 2]], ["U", 0x55, [1, 2, 4, 2, 1, 1]], ["V", 0x56, [4, 1, 1, 2, 1, 2]], ["W", 0x57, [4, 2, 1, 1, 1, 2]], ["X", 0x58, [4, 2, 1, 2, 1, 1]], ["Y", 0x59, [2, 1, 2, 1, 4, 1]], ["Z", 90, [2, 1, 4, 1, 2, 1]], ["[", 0x5b, [4, 1, 2, 1, 2, 1]], ["\\", 0x5c, [1, 1, 1, 1, 4, 3]], ["]", 0x5d, [1, 1, 1, 3, 4, 1]], ["^", 0x5e, [1, 3, 1, 1, 4, 1]], ["_", 0x5f, [1, 1, 4, 1, 1, 3]], ["`", 0x60, [1, 1, 4, 3, 1, 1]], ["a", 0x61, [4, 1, 1, 1, 1, 3]], ["b", 0x62, [4, 1, 1, 3, 1, 1]], ["c", 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x69, [2, 1, 1, 2, 3, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the UPC-A Barcode. */
        upcbarcode: {
            startSymbol: String.fromCharCode(parseInt('00fd', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[0-9]",
            symbolTable: [["0", 0, [3, 2, 1, 1]], ["1", 1, [2, 2, 2, 1]], ["2", 2, [2, 1, 2, 2]], ["3", 3, [1, 4, 1, 1]], ["4", 4, [1, 1, 3, 2]], ["5", 5, [1, 2, 3, 1]], ["6", 6, [1, 1, 1, 4]], ["7", 7, [1, 3, 1, 2]], ["8", 8, [1, 2, 1, 3]], ["9", 9, [3, 1, 1, 2]], ["B", 0 , [1, 1, 1, 1]]],
        },
    };

})(jQuery, Syncfusion);;
/// <reference path="ej.core.js" />

(function ($, ej, wd, undefined) {
    'use strict';

    var options = wd.unobtrusive = {
        dataRole: false,
        ejRole: false,
        directive: false
    };
    var propMaps = {};

    // IE8
    String.prototype.trim = String.prototype.trim || function () {
        return this.replace(/^\s+|\s+$/g, '');
    }

    var div = $(document.createElement("div"));

    var processAttrib = function (obj, name, value) {
        if (name.indexOf("-") === -1) {
            obj[name] = value;
            return;
        }

        ej.createObject(name.replace(/-/g, '.'), value, obj);
    }

    var processData = function (value) {
        return value === "true" ? true :
            value === "false" ? false :
            +value + "" === value ? +value : value;
    }

    var readAttributes = function (element, op, plugin) {
        var attributes = element[0].attributes, att, data = {},
            control = plugin.slice(2).toLowerCase(),
            dataName = "data-ej-",
            dataLength = dataName.length,
            ejName = "ej-",
            ejLength = ejName.length,
            value;

        for (var i = 0; i < attributes.length; i++) {
            att = attributes[i];
            if (op.dataRole && att.name.startsWith(dataName)) {
                processAttrib(data, att.name.slice(dataLength), processData(att.value));
            }
            if ((op.ejRole && att.name.startsWith(ejName)) ||
                (op.dataRole === false && op.ejRole === false && op.directive === true && att.name.startsWith(ejName))) {
                processAttrib(data, att.name.slice(ejLength), processData(att.value));
            }
        }

        return data;
    }

    var iterateAndSetModel = function (model, data, map, type) {
        var field = "", current;
        for (var prop in data) {
            field = map && prop in map ? map[prop] : prop;
            current = data[prop];
            if (type && type[field] === "string")
                current = current.toString();
            if (ej.isPlainObject(current) && (map || type)) {
                model[field] = {};
                iterateAndSetModel(model[field], current, map[prop + ".value"] ? map[prop + ".value"] : map, ej.isPlainObject(type[field]) ? type[field] : type);
                continue;
            }
            if (type && (type[field] === "array" || type[field] === "parent")) {
                try {
                    current = JSON.parse(current);
                }
                catch (e) {
                    current = eval(current);
                }
            }
            else if (type && type[field] === "data") {
                if (/^\[{0,1}\{.+\}\]{0,1}$/.test(current))
                    current = JSON.parse(current);
                else if (current.indexOf('/') === -1) // json instance
                    current = ej.getObject(current.indexOf("window.") === 0 ? current.slice(7) : current, window) || current;
            }

            model[field] = current;
        }
    }

    var changeTag = function (el, tag) {
        var $el = $(el);
        if (ej.support.outerHTML) {
            div.insertBefore($el);
            var currentTag = el.tagName, res = [];
            res[0] = "<";
            res[1] = el.outerHTML.trim().replace(RegExp("^<" + currentTag + "|" + currentTag + ">$", "ig"), tag);
            if (!res[1].endsWith(">"))
                res[2] = ">";
            el.outerHTML = res.join("");
            el = div.next()[0];
            div.remove();
            return el;
        }
        var clone = $el.clone(), currentTag = el.tagName, html = "";
        $el.html(html);
        html += "<";
        html += div.append(clone).html().replace(RegExp("^<" + currentTag + "|" + currentTag + ">$", "ig"), tag);
        html += ">";
        div.html(html);
        clone = $el.wrap(div.children().first()).parent();
        $el.remove();
        div.empty();
        return clone[0];
    }

    var findAndChangeTag = function (controls, tag, query, el) {
        var requiresFind = false, len = controls.length;
        for (var i = 0; i < len; i++) {
            if (controls[i].offsetParent) {
                controls[i] = changeTag(controls[i], tag);
            }
            else {
                controls.splice(i--, 1);
                requiresFind = true;
                len--;
            }
        }
        if (requiresFind) {
            controls.push.apply(controls, findAndChangeTag(el.find(query), tag, query, el));
        }
        return controls;
    }

    var findElements = function (name, op, el, proto) {
        var query = [], controlName = name.replace("ej", '').toLowerCase(), res = {};

        if (op.dataRole === true) {
            query.push("[data-role='" + name.toLowerCase() + "']");
        }
        if (op.ejRole === true) {
            query.push("[ej-" + controlName + "]");
        }

        res["role"] = el.find(query.join(','));

        if (op.directive === true) {
            res["directive"] = findAndChangeTag(el.find(controlName), (proto.validTags && proto.validTags[0]) || "div", controlName, el);
        }
        return res;
    };

    var generatePropMap = function (obj) {
        var map = {}, field;
        for (var prop in obj) {
            field = prop.toLowerCase();
            if (ej.isPlainObject(obj[prop])) {
                map[field + ".value"] = generatePropMap(obj[prop]);
            }
            map[field] = prop;
        }
        return map;
    };

    var initControls = function (controls, op, name, proto, map, dir) {
        var types = proto.dataTypes, len = controls.length;
        for (var i = 0; i < len; i++) {
            var control = controls.eq(i), model = {};
            iterateAndSetModel(model, readAttributes(control, op, name), map, types);
            control[name](model);
        }
    }

    var checkUnobtrusive = function (name, proto, op, el) {
        var controls = findElements(name, op, el, proto), map = propMaps[name];

        if (((controls.role && controls.role.length) || (controls.directive && controls.directive.length)) && !map) {
            map = generatePropMap(proto.defaults);
            map["serverevents"] = "serverEvents";
            map["clientid"] = "clientId";
            map["uniqueid"] = "uniqueId";
            propMaps[name] = map;

            if (proto._unobtrusive)
                $.extend(true, map, proto._unobtrusive);
        }

        initControls(controls.role, op, name, proto, map);

        if (controls.directive)
            initControls(controls.directive, op, name, proto, map, true);
    };

    var readBoolAttr = function (obj, attr) {
        if (obj.hasOwnProperty(attr))
            return obj[attr] !== false;

        return false;
    }

    wd.init = function (element) {
        if (!element) element = $(document);
        else element = element.jquery ? element : $(element);

        var widgets = wd.registeredWidgets;
        for (var name in widgets) {
            checkUnobtrusive(widgets[name].name, widgets[name].proto, options, element);
        }
    }

    $(function () {
        var ds = $(document.body).data();
        options.ejRole = ds.hasOwnProperty("ejrole") ? readBoolAttr(ds, "ejrole") : options.ejRole;
        options.directive = ds.hasOwnProperty("directive") ? readBoolAttr(ds, "directive") : options.directive;
        if (options.ejRole !== true && options.directive !== true) options.dataRole = true;
        options.dataRole = ds.hasOwnProperty("datarole") ? readBoolAttr(ds, "datarole") : options.dataRole;

        if (options.ejRole !== true && options.dataRole !== true && options.directive !== true)
            return;

        wd.autoInit = ds.hasOwnProperty("autoinit") ? readBoolAttr(ds, "autoinit") : true;

        if (wd.autoInit !== true)
            return;

        var query = [], el;
        if (options.dataRole === true)
            query.push("[data-ej-init]");
        if (options.ejRole === true)
            query.push("[ej-init]");
        el = $(query.join(','));

        if (!el.length) el = $(document);
        wd.init(el);
    });

})(window.jQuery, window.Syncfusion, window.Syncfusion.widget);;